// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_et[] = {
    {"kusimargiga_huuumark", "2049"},
    {"kaubamark", "2122"},
    {"teabeallikas", "2139"},
    {"nool_paremale_vasakule", "2194"},
    {"nool_ules_alla", "2195"},
    {"nool_ules_vasakule", "2196"},
    {"nool_ules_paremale", "2197"},
    {"nool_alla_paremale", "2198"},
    {"nool_alla_vasakule", "2199"},
    {"klaviatuur", "2328"},
    {"paike", "2600"},
    {"pilv", "2601"},
    {"vihmavari", "2602"},
    {"lumememm", "2603"},
    {"komeet", "2604"},
    {"margitud_ruut", "2611"},
    {"piiskadega_vihmavari", "2614"},
    {"kuum_jook", "2615"},
    {"ristik", "2618"},
    {"kolp_ja_saareluud", "2620"},
    {"radioaktiivne", "2622"},
    {"bioloogiline_oht", "2623"},
    {"oigeusu_rist", "2626"},
    {"dharma_ratas", "2638"},
    {"kulmu_kortsutav_nagu", "2639"},
    {"naisesumbol", "2640"},
    {"mehesumbol", "2642"},
    {"jaar", "2648"},
    {"sonn", "2649"},
    {"ambur", "2650"},
    {"kaljukits", "2651"},
    {"veevalaja", "2652"},
    {"kalad", "2653"},
    {"poti", "2660"},
    {"risti", "2663"},
    {"artu", "2665"},
    {"ruutu", "2666"},
    {"kuumaveeallikad", "2668"},
    {"haamer_ja_kirka", "2692"},
    {"ankur", "2693"},
    {"ristatud_moogad", "2694"},
    {"meditsiinisumbol", "2695"},
    {"kaalukausid", "2696"},
    {"destilleerimisnou", "2697"},
    {"hammasratas", "2699"},
    {"kaarid", "2702"},
    {"markenupp", "2705"},
    {"lennuk", "2708"},
    {"umbrik", "2709"},
    {"must_sulepea", "2712"},
    {"marge", "2714"},
    {"korrutamine", "2716"},
    {"taaveti_taht", "2721"},
    {"sademed", "2728"},
    {"kaheksa_tipuga_tarn", "2733"},
    {"kaheksa_tipuga_taht", "2734"},
    {"lumehelves", "2744"},
    {"sadelus", "2747"},
    {"punane_kusimark", "2753"},
    {"valge_kusimark", "2754"},
    {"valge_huuumark", "2755"},
    {"punane_huuumark", "2757"},
    {"sudamega_huuumark", "2763"},
    {"punane_suda", "2764"},
    {"liitmine", "2795"},
    {"lahutamine", "2796"},
    {"jagamine", "2797"},
    {"kaardus_nool_ules", "2934"},
    {"kaardus_nool_alla", "2935"},
    {"laineline_kriips", "3030"},
    {"jaapani_nupp_onnitlused", "3297"},
    {"jaapani_nupp_saladus", "3299"},
    {"autorioigus", "00A9"},
    {"registreeritud", "00AE"},
    {"topelthuuumark", "203C"},
    {"kaardus_nool_vasakule", "21A9"},
    {"kaardus_nool_paremale", "21AA"},
    {"kaekell", "231A"},
    {"liivakell", "231B"},
    {"valjutamisnupp", "23CF"},
    {"edasikerimisnupp", "23E9"},
    {"tagasikerimise_nupp", "23EA"},
    {"kiirkerimine_ules", "23EB"},
    {"kiirkerimine_alla", "23EC"},
    {"jargmise_loo_nupp", "23ED"},
    {"eelmise_loo_nupp", "23EE"},
    {"esitus_ja_pausinupp", "23EF"},
    {"aratuskell", "23F0"},
    {"stopper", "23F1"},
    {"taimer", "23F2"},
    {"voolava_liivaga_kell", "23F3"},
    {"pausinupp", "23F8"},
    {"peatamisnupp", "23F9"},
    {"salvestusnupp", "23FA"},
    {"m_ringis", "24C2"},
    {"vaike_must_ruut", "25AA"},
    {"vaike_valge_ruut", "25AB"},
    {"esitusnupp", "25B6"},
    {"tagurpidi_esitamise_nupp", "25C0"},
    {"keskmine_valge_ruut", "25FB"},
    {"keskmine_must_ruut", "25FC"},
    {"vaiksem_valge_ruut", "25FD"},
    {"vaiksem_must_ruut", "25FE"},
    {"telefon", "260E"},
    {"ules_osutav_sorm", "261D"},
    {"ules_osutav_sorm_varvi1", "261D-1F3FB"},
    {"ules_osutav_sorm_varvi2", "261D-1F3FC"},
    {"ules_osutav_sorm_varvi3", "261D-1F3FD"},
    {"ules_osutav_sorm_varvi4", "261D-1F3FE"},
    {"ules_osutav_sorm_varvi5", "261D-1F3FF"},
    {"taht_ja_kuusirp", "262A"},
    {"rahu_sumbol", "262E"},
    {"yin_yang", "262F"},
    {"naerunagu", "263A"},
    {"kaksikud", "264A"},
    {"vahk", "264B"},
    {"lovi", "264C"},
    {"neitsi", "264D"},
    {"kaalud", "264E"},
    {"skorpion", "264F"},
    {"ettur", "265F"},
    {"taaskasutuse_sumbol", "267B"},
    {"lopmatus", "267E"},
    {"ratastool", "267F"},
    {"aatomi_sumbol", "269B"},
    {"prantsuse_liilia", "269C"},
    {"hoiatus", "26A0"},
    {"korgepinge", "26A1"},
    {"transsoolisuse_sumbol", "26A7"},
    {"valge_ring", "26AA"},
    {"must_ring", "26AB"},
    {"kirst", "26B0"},
    {"tuhaurn", "26B1"},
    {"jalgpall", "26BD"},
    {"pesapall", "26BE"},
    {"lumeta_lumememm", "26C4"},
    {"paike_pilve_taga", "26C5"},
    {"aikese_ja_vihmapilv", "26C8"},
    {"maokandja", "26CE"},
    {"kirka", "26CF"},
    {"valge_ristiga_kiiver", "26D1"},
    {"ahelad", "26D3"},
    {"sisenemine_keelatud", "26D4"},
    {"shinto_tempel", "26E9"},
    {"kirik", "26EA"},
    {"magi", "26F0"},
    {"paikesevari_maas", "26F1"},
    {"purskkaev", "26F2"},
    {"golfilipp", "26F3"},
    {"praam", "26F4"},
    {"purjepaat", "26F5"},
    {"suusataja", "26F7"},
    {"uisk", "26F8"},
    {"inimene_palliga", "26F9"},
    {"inimene_palliga_varvi1", "26F9-1F3FB"},
    {"inimene_palliga_varvi2", "26F9-1F3FC"},
    {"inimene_palliga_varvi3", "26F9-1F3FD"},
    {"inimene_palliga_varvi4", "26F9-1F3FE"},
    {"inimene_palliga_varvi5", "26F9-1F3FF"},
    {"telk", "26FA"},
    {"tankur", "26FD"},
    {"ulestostetud_rusikas", "270A"},
    {"ulestostetud_rusikas_varvi1", "270A-1F3FB"},
    {"ulestostetud_rusikas_varvi2", "270A-1F3FC"},
    {"ulestostetud_rusikas_varvi3", "270A-1F3FD"},
    {"ulestostetud_rusikas_varvi4", "270A-1F3FE"},
    {"ulestostetud_rusikas_varvi5", "270A-1F3FF"},
    {"ulestostetud_kasi", "270B"},
    {"ulestostetud_kasi_varvi1", "270B-1F3FB"},
    {"ulestostetud_kasi_varvi2", "270B-1F3FC"},
    {"ulestostetud_kasi_varvi3", "270B-1F3FD"},
    {"ulestostetud_kasi_varvi4", "270B-1F3FE"},
    {"ulestostetud_kasi_varvi5", "270B-1F3FF"},
    {"voit", "270C"},
    {"voit_varvi1", "270C-1F3FB"},
    {"voit_varvi2", "270C-1F3FC"},
    {"voit_varvi3", "270C-1F3FD"},
    {"voit_varvi4", "270C-1F3FE"},
    {"voit_varvi5", "270C-1F3FF"},
    {"kirjutav_kasi", "270D"},
    {"kirjutav_kasi_varvi1", "270D-1F3FB"},
    {"kirjutav_kasi_varvi2", "270D-1F3FC"},
    {"kirjutav_kasi_varvi3", "270D-1F3FD"},
    {"kirjutav_kasi_varvi4", "270D-1F3FE"},
    {"kirjutav_kasi_varvi5", "270D-1F3FF"},
    {"pliiats", "270F"},
    {"ladina_rist", "271D"},
    {"rist", "274C"},
    {"korrutusmargiga_nupp", "274E"},
    {"nool_paremale", "27A1"},
    {"silmus", "27B0"},
    {"topeltsilmus", "27BF"},
    {"nool_vasakule", "2B05"},
    {"nool_ules", "2B06"},
    {"nool_alla", "2B07"},
    {"suur_must_ruut", "2B1B"},
    {"suur_valge_ruut", "2B1C"},
    {"taht", "2B50"},
    {"seest_tuhi_punane_ringjoon", "2B55"},
    {"osa_vaheldumise_mark", "303D"},
    {"mahjongi_punane_draakon", "1F004"},
    {"jokker", "1F0CF"},
    {"nupp_a_veregrupp", "1F170"},
    {"nupp_b_veregrupp", "1F171"},
    {"nupp_0_veregrupp", "1F17E"},
    {"nupp_p", "1F17F"},
    {"nupp_ab_veregrupp", "1F18E"},
    {"nupp_cl", "1F191"},
    {"nupp_cool", "1F192"},
    {"nupp_free", "1F193"},
    {"nupp_id", "1F194"},
    {"nupp_new", "1F195"},
    {"nupp_ng", "1F196"},
    {"nupp_ok", "1F197"},
    {"nupp_sos", "1F198"},
    {"nupp_up", "1F199"},
    {"nupp_vs", "1F19A"},
    {"piirkondlik_naitaja_a", "1F1E6"},
    {"piirkondlik_naitaja_b", "1F1E7"},
    {"piirkondlik_naitaja_c", "1F1E8"},
    {"piirkondlik_naitaja_d", "1F1E9"},
    {"piirkondlik_naitaja_e", "1F1EA"},
    {"piirkondlik_naitaja_f", "1F1EB"},
    {"piirkondlik_naitaja_g", "1F1EC"},
    {"piirkondlik_naitaja_h", "1F1ED"},
    {"piirkondlik_naitaja_i", "1F1EE"},
    {"piirkondlik_naitaja_j", "1F1EF"},
    {"piirkondlik_naitaja_k", "1F1F0"},
    {"piirkondlik_naitaja_l", "1F1F1"},
    {"piirkondlik_naitaja_m", "1F1F2"},
    {"piirkondlik_naitaja_n", "1F1F3"},
    {"piirkondlik_naitaja_o", "1F1F4"},
    {"piirkondlik_naitaja_p", "1F1F5"},
    {"piirkondlik_naitaja_q", "1F1F6"},
    {"piirkondlik_naitaja_r", "1F1F7"},
    {"piirkondlik_naitaja_s", "1F1F8"},
    {"piirkondlik_naitaja_t", "1F1F9"},
    {"piirkondlik_naitaja_u", "1F1FA"},
    {"piirkondlik_naitaja_v", "1F1FB"},
    {"piirkondlik_naitaja_w", "1F1FC"},
    {"piirkondlik_naitaja_x", "1F1FD"},
    {"piirkondlik_naitaja_y", "1F1FE"},
    {"piirkondlik_naitaja_z", "1F1FF"},
    {"jaapani_nupp_siin", "1F201"},
    {"jaapani_nupp_teenustasu", "1F202"},
    {"jaapani_nupp_tasuta", "1F21A"},
    {"jaapani_nupp_reserveeritud", "1F22F"},
    {"jaapani_nupp_keelatud", "1F232"},
    {"jaapani_nupp_vaba", "1F233"},
    {"jaapani_nupp_sooritatud", "1F234"},
    {"jaapani_nupp_pole_vaba", "1F235"},
    {"jaapani_nupp_pole_tasuta", "1F236"},
    {"jaapani_nupp_kuutasu", "1F237"},
    {"jaapani_nupp_rakendus", "1F238"},
    {"jaapani_nupp_allahindlus", "1F239"},
    {"jaapani_nupp_avatud", "1F23A"},
    {"jaapani_nupp_soodne", "1F250"},
    {"jaapani_nupp_vastuvoetav", "1F251"},
    {"tsuklon", "1F300"},
    {"udune", "1F301"},
    {"suletud_vihmavari", "1F302"},
    {"oine_tahistaevas", "1F303"},
    {"paikesetous_magedes", "1F304"},
    {"paikesetous", "1F305"},
    {"linna_siluett_videvikus", "1F306"},
    {"paikeseloojang", "1F307"},
    {"vikerkaar", "1F308"},
    {"sild_oosel", "1F309"},
    {"veelaine", "1F30A"},
    {"vulkaan", "1F30B"},
    {"linnutee", "1F30C"},
    {"euroopa_ja_aafrika_gloobusel", "1F30D"},
    {"ameerika_mandrid_gloobusel", "1F30E"},
    {"aasia_ja_austraalia_gloobusel", "1F30F"},
    {"meridiaanidega_gloobus", "1F310"},
    {"noorkuu", "1F311"},
    {"kasvav_kuusirp", "1F312"},
    {"kasvav_poolkuu", "1F313"},
    {"kasvav_kuu", "1F314"},
    {"taiskuu", "1F315"},
    {"kahanev_kuu", "1F316"},
    {"kahanev_poolkuu", "1F317"},
    {"kahanev_kuusirp", "1F318"},
    {"kuusirp", "1F319"},
    {"naoga_noorkuu", "1F31A"},
    {"naoga_kasvav_kuu", "1F31B"},
    {"naoga_kahanev_kuu", "1F31C"},
    {"naoga_taiskuu", "1F31D"},
    {"naoga_paike", "1F31E"},
    {"helendav_taht", "1F31F"},
    {"langev_taht", "1F320"},
    {"kraadiklaas", "1F321"},
    {"paike_vaikese_pilve_taga", "1F324"},
    {"paike_suure_pilve_taga", "1F325"},
    {"paike_vihmapilve_taga", "1F326"},
    {"vihmapilv", "1F327"},
    {"lumepilv", "1F328"},
    {"aikesepilv", "1F329"},
    {"keeristorm", "1F32A"},
    {"udu", "1F32B"},
    {"naoga_tuul", "1F32C"},
    {"hot_dog", "1F32D"},
    {"tako", "1F32E"},
    {"burrito", "1F32F"},
    {"kastan", "1F330"},
    {"vorse", "1F331"},
    {"igihaljas_puu", "1F332"},
    {"lehtpuu", "1F333"},
    {"palmipuu", "1F334"},
    {"kaktus", "1F335"},
    {"tuline_pipar", "1F336"},
    {"tulp", "1F337"},
    {"kirsiois", "1F338"},
    {"roos", "1F339"},
    {"hibisk", "1F33A"},
    {"paevalill", "1F33B"},
    {"ois", "1F33C"},
    {"maisitolvik", "1F33D"},
    {"riisitaim", "1F33E"},
    {"maitsetaim", "1F33F"},
    {"nelja_lehega_ristik", "1F340"},
    {"vahtraleht", "1F341"},
    {"langenud_leht", "1F342"},
    {"tuules_lendlev_leht", "1F343"},
    {"seen", "1F344"},
    {"tomat", "1F345"},
    {"baklazaan", "1F346"},
    {"viinamarjad", "1F347"},
    {"melon", "1F348"},
    {"arbuus", "1F349"},
    {"mandariin", "1F34A"},
    {"sidrun", "1F34B"},
    {"banaan", "1F34C"},
    {"ananass", "1F34D"},
    {"punane_oun", "1F34E"},
    {"roheline_oun", "1F34F"},
    {"pirn", "1F350"},
    {"virsik", "1F351"},
    {"kirsid", "1F352"},
    {"maasikas", "1F353"},
    {"hamburger", "1F354"},
    {"pitsa", "1F355"},
    {"kondiga_liha", "1F356"},
    {"kanakoib", "1F357"},
    {"riisikupsis", "1F358"},
    {"riisipall", "1F359"},
    {"keedetud_riis", "1F35A"},
    {"karriga_riis", "1F35B"},
    {"aurav_kauss", "1F35C"},
    {"spagetid", "1F35D"},
    {"leib", "1F35E"},
    {"friikartulid", "1F35F"},
    {"rostitud_bataat", "1F360"},
    {"dango", "1F361"},
    {"oden", "1F362"},
    {"susi", "1F363"},
    {"praetud_krevett", "1F364"},
    {"keerdudega_kalakotlet", "1F365"},
    {"pehme_jaatis", "1F366"},
    {"mahlajaa", "1F367"},
    {"jaatis", "1F368"},
    {"soorik", "1F369"},
    {"kupsis", "1F36A"},
    {"sokolaaditahvel", "1F36B"},
    {"komm", "1F36C"},
    {"pulgakomm", "1F36D"},
    {"vanillipuding", "1F36E"},
    {"meepott", "1F36F"},
    {"liivataignakook", "1F370"},
    {"bento_einekarp", "1F371"},
    {"potitais_toitu", "1F372"},
    {"toiduvalmistamine", "1F373"},
    {"kahvel_ja_nuga", "1F374"},
    {"korvata_teetass", "1F375"},
    {"riisiviin", "1F376"},
    {"veiniklaas", "1F377"},
    {"kokteiliklaas", "1F378"},
    {"troopiline_jook", "1F379"},
    {"ollekann", "1F37A"},
    {"kolisevad_ollekannud", "1F37B"},
    {"lutipudel", "1F37C"},
    {"kahvel_nuga_ja_taldrik", "1F37D"},
    {"pudeli_avamine", "1F37E"},
    {"popkorn", "1F37F"},
    {"lint", "1F380"},
    {"kingipakk", "1F381"},
    {"sunnipaevatort", "1F382"},
    {"korvitsalatern", "1F383"},
    {"joulupuu", "1F384"},
    {"jouluvana", "1F385"},
    {"jouluvana_varvi1", "1F385-1F3FB"},
    {"jouluvana_varvi2", "1F385-1F3FC"},
    {"jouluvana_varvi3", "1F385-1F3FD"},
    {"jouluvana_varvi4", "1F385-1F3FE"},
    {"jouluvana_varvi5", "1F385-1F3FF"},
    {"ilutulestik", "1F386"},
    {"sarakuunal", "1F387"},
    {"ohupall", "1F388"},
    {"pidupaev", "1F389"},
    {"konfetid", "1F38A"},
    {"tanabata_soovipuu", "1F38B"},
    {"ristatud_lipud", "1F38C"},
    {"varavamand", "1F38D"},
    {"jaapani_nukud", "1F38E"},
    {"kalakujuline_vimpel", "1F38F"},
    {"tuulekell", "1F390"},
    {"kuutseremoonia", "1F391"},
    {"ranits", "1F392"},
    {"koolilopumuts", "1F393"},
    {"sojamedal", "1F396"},
    {"kollane_lint", "1F397"},
    {"stuudiomikrofon", "1F399"},
    {"liugur", "1F39A"},
    {"juhtnupud", "1F39B"},
    {"filmikaadrid", "1F39E"},
    {"piletid", "1F39F"},
    {"karussellihobune", "1F3A0"},
    {"vaateratas", "1F3A1"},
    {"ameerika_maed", "1F3A2"},
    {"ong", "1F3A3"},
    {"mikrofon", "1F3A4"},
    {"filmikaamera", "1F3A5"},
    {"kino", "1F3A6"},
    {"korvaklapid", "1F3A7"},
    {"kunstniku_palett", "1F3A8"},
    {"torukubar", "1F3A9"},
    {"tsirkusetelk", "1F3AA"},
    {"pilet", "1F3AB"},
    {"klapptahvel", "1F3AC"},
    {"lavakunst", "1F3AD"},
    {"videomang", "1F3AE"},
    {"taistabamus", "1F3AF"},
    {"manguautomaat", "1F3B0"},
    {"piljardikuul", "1F3B1"},
    {"taring", "1F3B2"},
    {"keegel", "1F3B3"},
    {"mangukaardid", "1F3B4"},
    {"noot", "1F3B5"},
    {"noodid", "1F3B6"},
    {"saksofon", "1F3B7"},
    {"kitarr", "1F3B8"},
    {"klahvpill", "1F3B9"},
    {"trompet", "1F3BA"},
    {"viiul", "1F3BB"},
    {"noodijoonestik", "1F3BC"},
    {"jooksusark", "1F3BD"},
    {"tennis", "1F3BE"},
    {"suusad", "1F3BF"},
    {"korvpall", "1F3C0"},
    {"ruudulipp", "1F3C1"},
    {"lumelaudur", "1F3C2"},
    {"lumelaudur_varvi1", "1F3C2-1F3FB"},
    {"lumelaudur_varvi2", "1F3C2-1F3FC"},
    {"lumelaudur_varvi3", "1F3C2-1F3FD"},
    {"lumelaudur_varvi4", "1F3C2-1F3FE"},
    {"lumelaudur_varvi5", "1F3C2-1F3FF"},
    {"jooksja", "1F3C3"},
    {"jooksja_varvi1", "1F3C3-1F3FB"},
    {"jooksja_varvi2", "1F3C3-1F3FC"},
    {"jooksja_varvi3", "1F3C3-1F3FD"},
    {"jooksja_varvi4", "1F3C3-1F3FE"},
    {"jooksja_varvi5", "1F3C3-1F3FF"},
    {"surfar", "1F3C4"},
    {"surfar_varvi1", "1F3C4-1F3FB"},
    {"surfar_varvi2", "1F3C4-1F3FC"},
    {"surfar_varvi3", "1F3C4-1F3FD"},
    {"surfar_varvi4", "1F3C4-1F3FE"},
    {"surfar_varvi5", "1F3C4-1F3FF"},
    {"medal", "1F3C5"},
    {"karikas", "1F3C6"},
    {"hobuste_voiduajamine", "1F3C7"},
    {"hobuste_voiduajamine_varvi1", "1F3C7-1F3FB"},
    {"hobuste_voiduajamine_varvi2", "1F3C7-1F3FC"},
    {"hobuste_voiduajamine_varvi3", "1F3C7-1F3FD"},
    {"hobuste_voiduajamine_varvi4", "1F3C7-1F3FE"},
    {"hobuste_voiduajamine_varvi5", "1F3C7-1F3FF"},
    {"ameerika_jalgpall", "1F3C8"},
    {"ragbipall", "1F3C9"},
    {"ujuja", "1F3CA"},
    {"ujuja_varvi1", "1F3CA-1F3FB"},
    {"ujuja_varvi2", "1F3CA-1F3FC"},
    {"ujuja_varvi3", "1F3CA-1F3FD"},
    {"ujuja_varvi4", "1F3CA-1F3FE"},
    {"ujuja_varvi5", "1F3CA-1F3FF"},
    {"joutostja", "1F3CB"},
    {"joutostja_varvi1", "1F3CB-1F3FB"},
    {"joutostja_varvi2", "1F3CB-1F3FC"},
    {"joutostja_varvi3", "1F3CB-1F3FD"},
    {"joutostja_varvi4", "1F3CB-1F3FE"},
    {"joutostja_varvi5", "1F3CB-1F3FF"},
    {"golfimangija", "1F3CC"},
    {"golfimangija_varvi1", "1F3CC-1F3FB"},
    {"golfimangija_varvi2", "1F3CC-1F3FC"},
    {"golfimangija_varvi3", "1F3CC-1F3FD"},
    {"golfimangija_varvi4", "1F3CC-1F3FE"},
    {"golfimangija_varvi5", "1F3CC-1F3FF"},
    {"mootorratas", "1F3CD"},
    {"voidusoiduauto", "1F3CE"},
    {"kriket", "1F3CF"},
    {"vorkpall", "1F3D0"},
    {"maahoki", "1F3D1"},
    {"jaahoki", "1F3D2"},
    {"pingpong", "1F3D3"},
    {"lumine_magi", "1F3D4"},
    {"telkimine", "1F3D5"},
    {"paikesevari_rannas", "1F3D6"},
    {"ehitus", "1F3D7"},
    {"majad", "1F3D8"},
    {"linna_siluett", "1F3D9"},
    {"mahajaetud_maja", "1F3DA"},
    {"antiikhoone", "1F3DB"},
    {"korb", "1F3DC"},
    {"asustamata_saar", "1F3DD"},
    {"rahvuspark", "1F3DE"},
    {"staadion", "1F3DF"},
    {"maja", "1F3E0"},
    {"aiaga_maja", "1F3E1"},
    {"kontorihoone", "1F3E2"},
    {"jaapani_postkontor", "1F3E3"},
    {"postkontor", "1F3E4"},
    {"haigla", "1F3E5"},
    {"pank", "1F3E6"},
    {"pangaautomaat", "1F3E7"},
    {"hotell", "1F3E8"},
    {"armuhotell", "1F3E9"},
    {"kauplus", "1F3EA"},
    {"kool", "1F3EB"},
    {"kaubamaja", "1F3EC"},
    {"tehas", "1F3ED"},
    {"punane_paberlatern", "1F3EE"},
    {"jaapani_loss", "1F3EF"},
    {"loss", "1F3F0"},
    {"valge_lipp", "1F3F3"},
    {"must_lipp", "1F3F4"},
    {"rosett", "1F3F5"},
    {"silt", "1F3F7"},
    {"sulgpall", "1F3F8"},
    {"vibu_ja_nool", "1F3F9"},
    {"amfora", "1F3FA"},
    {"hele_nahatoon", "1F3FB"},
    {"keskmiselt_hele_nahatoon", "1F3FC"},
    {"keskmine_nahatoon", "1F3FD"},
    {"keskmiselt_tume_nahatoon", "1F3FE"},
    {"tume_nahatoon", "1F3FF"},
    {"rott", "1F400"},
    {"hiir", "1F401"},
    {"harg", "1F402"},
    {"vesipuhvel", "1F403"},
    {"lehm", "1F404"},
    {"tiiger", "1F405"},
    {"leopard", "1F406"},
    {"janes", "1F407"},
    {"kass", "1F408"},
    {"draakon", "1F409"},
    {"krokodill", "1F40A"},
    {"vaal", "1F40B"},
    {"tigu", "1F40C"},
    {"madu", "1F40D"},
    {"hobune", "1F40E"},
    {"oinas", "1F40F"},
    {"kits", "1F410"},
    {"lammas", "1F411"},
    {"ahv", "1F412"},
    {"kukk", "1F413"},
    {"kana", "1F414"},
    {"koer", "1F415"},
    {"siga", "1F416"},
    {"metssiga", "1F417"},
    {"elevant", "1F418"},
    {"kaheksajalg", "1F419"},
    {"spiraalne_teokarp", "1F41A"},
    {"roovik", "1F41B"},
    {"sipelgas", "1F41C"},
    {"mesilane", "1F41D"},
    {"lepatriinu", "1F41E"},
    {"kala", "1F41F"},
    {"troopiline_kala", "1F420"},
    {"siilkala", "1F421"},
    {"kilpkonn", "1F422"},
    {"kooruv_tibu", "1F423"},
    {"vaike_tibu", "1F424"},
    {"vaike_tibu_eestvaates", "1F425"},
    {"lind", "1F426"},
    {"pingviin", "1F427"},
    {"koaala", "1F428"},
    {"puudel", "1F429"},
    {"kaamel", "1F42A"},
    {"kahe_kuuruga_kaamel", "1F42B"},
    {"delfiin", "1F42C"},
    {"hiire_nagu", "1F42D"},
    {"lehma_nagu", "1F42E"},
    {"tiigri_nagu", "1F42F"},
    {"janese_nagu", "1F430"},
    {"kassi_nagu", "1F431"},
    {"draakoni_nagu", "1F432"},
    {"vett_pritsiv_vaal", "1F433"},
    {"hobuse_nagu", "1F434"},
    {"ahvi_nagu", "1F435"},
    {"koera_nagu", "1F436"},
    {"sea_nagu", "1F437"},
    {"konn", "1F438"},
    {"hamster", "1F439"},
    {"hunt", "1F43A"},
    {"karu", "1F43B"},
    {"panda", "1F43C"},
    {"sea_karss", "1F43D"},
    {"kapajaljed", "1F43E"},
    {"vootorav", "1F43F"},
    {"silmad", "1F440"},
    {"silm", "1F441"},
    {"korv", "1F9FA"},
    {"korv_varvi1", "1F442-1F3FB"},
    {"korv_varvi2", "1F442-1F3FC"},
    {"korv_varvi3", "1F442-1F3FD"},
    {"korv_varvi4", "1F442-1F3FE"},
    {"korv_varvi5", "1F442-1F3FF"},
    {"nina", "1F443"},
    {"nina_varvi1", "1F443-1F3FB"},
    {"nina_varvi2", "1F443-1F3FC"},
    {"nina_varvi3", "1F443-1F3FD"},
    {"nina_varvi4", "1F443-1F3FE"},
    {"nina_varvi5", "1F443-1F3FF"},
    {"suu", "1F444"},
    {"keel", "1F445"},
    {"ules_osutav_sorm_tagantvaates", "1F446"},
    {"ules_osutav_sorm_tagantvaates_varvi1", "1F446-1F3FB"},
    {"ules_osutav_sorm_tagantvaates_varvi2", "1F446-1F3FC"},
    {"ules_osutav_sorm_tagantvaates_varvi3", "1F446-1F3FD"},
    {"ules_osutav_sorm_tagantvaates_varvi4", "1F446-1F3FE"},
    {"ules_osutav_sorm_tagantvaates_varvi5", "1F446-1F3FF"},
    {"alla_osutav_sorm", "1F447"},
    {"alla_osutav_sorm_varvi1", "1F447-1F3FB"},
    {"alla_osutav_sorm_varvi2", "1F447-1F3FC"},
    {"alla_osutav_sorm_varvi3", "1F447-1F3FD"},
    {"alla_osutav_sorm_varvi4", "1F447-1F3FE"},
    {"alla_osutav_sorm_varvi5", "1F447-1F3FF"},
    {"vasakule_osutav_sorm", "1F448"},
    {"vasakule_osutav_sorm_varvi1", "1F448-1F3FB"},
    {"vasakule_osutav_sorm_varvi2", "1F448-1F3FC"},
    {"vasakule_osutav_sorm_varvi3", "1F448-1F3FD"},
    {"vasakule_osutav_sorm_varvi4", "1F448-1F3FE"},
    {"vasakule_osutav_sorm_varvi5", "1F448-1F3FF"},
    {"paremale_osutav_sorm", "1F449"},
    {"paremale_osutav_sorm_varvi1", "1F449-1F3FB"},
    {"paremale_osutav_sorm_varvi2", "1F449-1F3FC"},
    {"paremale_osutav_sorm_varvi3", "1F449-1F3FD"},
    {"paremale_osutav_sorm_varvi4", "1F449-1F3FE"},
    {"paremale_osutav_sorm_varvi5", "1F449-1F3FF"},
    {"rusikalook", "1F44A"},
    {"rusikalook_varvi1", "1F44A-1F3FB"},
    {"rusikalook_varvi2", "1F44A-1F3FC"},
    {"rusikalook_varvi3", "1F44A-1F3FD"},
    {"rusikalook_varvi4", "1F44A-1F3FE"},
    {"rusikalook_varvi5", "1F44A-1F3FF"},
    {"lehvitav_kasi", "1F44B"},
    {"lehvitav_kasi_varvi1", "1F44B-1F3FB"},
    {"lehvitav_kasi_varvi2", "1F44B-1F3FC"},
    {"lehvitav_kasi_varvi3", "1F44B-1F3FD"},
    {"lehvitav_kasi_varvi4", "1F44B-1F3FE"},
    {"lehvitav_kasi_varvi5", "1F44B-1F3FF"},
    {"ok_kaemark", "1F44C"},
    {"ok_kaemark_varvi1", "1F44C-1F3FB"},
    {"ok_kaemark_varvi2", "1F44C-1F3FC"},
    {"ok_kaemark_varvi3", "1F44C-1F3FD"},
    {"ok_kaemark_varvi4", "1F44C-1F3FE"},
    {"ok_kaemark_varvi5", "1F44C-1F3FF"},
    {"meeldib", "1F44D"},
    {"meeldib_varvi1", "1F44D-1F3FB"},
    {"meeldib_varvi2", "1F44D-1F3FC"},
    {"meeldib_varvi3", "1F44D-1F3FD"},
    {"meeldib_varvi4", "1F44D-1F3FE"},
    {"meeldib_varvi5", "1F44D-1F3FF"},
    {"ei_meeldi", "1F44E"},
    {"ei_meeldi_varvi1", "1F44E-1F3FB"},
    {"ei_meeldi_varvi2", "1F44E-1F3FC"},
    {"ei_meeldi_varvi3", "1F44E-1F3FD"},
    {"ei_meeldi_varvi4", "1F44E-1F3FE"},
    {"ei_meeldi_varvi5", "1F44E-1F3FF"},
    {"plaksutavad_kaed", "1F44F"},
    {"plaksutavad_kaed_varvi1", "1F44F-1F3FB"},
    {"plaksutavad_kaed_varvi2", "1F44F-1F3FC"},
    {"plaksutavad_kaed_varvi3", "1F44F-1F3FD"},
    {"plaksutavad_kaed_varvi4", "1F44F-1F3FE"},
    {"plaksutavad_kaed_varvi5", "1F44F-1F3FF"},
    {"avatud_kaed", "1F450"},
    {"avatud_kaed_varvi1", "1F450-1F3FB"},
    {"avatud_kaed_varvi2", "1F450-1F3FC"},
    {"avatud_kaed_varvi3", "1F450-1F3FD"},
    {"avatud_kaed_varvi4", "1F450-1F3FE"},
    {"avatud_kaed_varvi5", "1F450-1F3FF"},
    {"kroon", "1F451"},
    {"naistekubar", "1F452"},
    {"prillid", "1F453"},
    {"lips", "1F454"},
    {"t_sark", "1F455"},
    {"teksad", "1F456"},
    {"kleit", "1F457"},
    {"kimono", "1F458"},
    {"bikiinid", "1F459"},
    {"naisteriided", "1F45A"},
    {"ridikul", "1F45B"},
    {"kaekott", "1F45C"},
    {"kott", "1F45D"},
    {"meesteking", "1F45E"},
    {"jooksuking", "1F45F"},
    {"korge_kontsaga_king", "1F460"},
    {"naistesandaal", "1F461"},
    {"naistesaabas", "1F462"},
    {"jalajaljed", "1F463"},
    {"busti_siluett", "1F464"},
    {"bustide_siluetid", "1F465"},
    {"poiss", "1F466"},
    {"poiss_varvi1", "1F466-1F3FB"},
    {"poiss_varvi2", "1F466-1F3FC"},
    {"poiss_varvi3", "1F466-1F3FD"},
    {"poiss_varvi4", "1F466-1F3FE"},
    {"poiss_varvi5", "1F466-1F3FF"},
    {"tudruk", "1F467"},
    {"tudruk_varvi1", "1F467-1F3FB"},
    {"tudruk_varvi2", "1F467-1F3FC"},
    {"tudruk_varvi3", "1F467-1F3FD"},
    {"tudruk_varvi4", "1F467-1F3FE"},
    {"tudruk_varvi5", "1F467-1F3FF"},
    {"mees", "1F468"},
    {"mees_varvi1", "1F468-1F3FB"},
    {"mees_varvi2", "1F468-1F3FC"},
    {"mees_varvi3", "1F468-1F3FD"},
    {"mees_varvi4", "1F468-1F3FE"},
    {"mees_varvi5", "1F468-1F3FF"},
    {"naine", "1F469"},
    {"naine_varvi1", "1F469-1F3FB"},
    {"naine_varvi2", "1F469-1F3FC"},
    {"naine_varvi3", "1F469-1F3FD"},
    {"naine_varvi4", "1F469-1F3FE"},
    {"naine_varvi5", "1F469-1F3FF"},
    {"perekond", "1F46A"},
    {"mees_ja_naine_hoiavad_kaest_kinni", "1F46B"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi1", "1F46B-1F3FB"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi2", "1F46B-1F3FC"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi3", "1F46B-1F3FD"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi4", "1F46B-1F3FE"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi5", "1F46B-1F3FF"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mees_ja_naine_hoiavad_kaest_kinni_varvi5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"kaest_kinni_hoidvad_mehed", "1F46C"},
    {"kaest_kinni_hoidvad_mehed_varvi1", "1F46C-1F3FB"},
    {"kaest_kinni_hoidvad_mehed_varvi2", "1F46C-1F3FC"},
    {"kaest_kinni_hoidvad_mehed_varvi3", "1F46C-1F3FD"},
    {"kaest_kinni_hoidvad_mehed_varvi4", "1F46C-1F3FE"},
    {"kaest_kinni_hoidvad_mehed_varvi5", "1F46C-1F3FF"},
    {"kaest_kinni_hoidvad_mehed_varvi1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"kaest_kinni_hoidvad_mehed_varvi1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"kaest_kinni_hoidvad_mehed_varvi1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"kaest_kinni_hoidvad_mehed_varvi1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"kaest_kinni_hoidvad_mehed_varvi2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"kaest_kinni_hoidvad_mehed_varvi2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"kaest_kinni_hoidvad_mehed_varvi2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"kaest_kinni_hoidvad_mehed_varvi2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"kaest_kinni_hoidvad_mehed_varvi3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"kaest_kinni_hoidvad_mehed_varvi3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"kaest_kinni_hoidvad_mehed_varvi3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"kaest_kinni_hoidvad_mehed_varvi3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"kaest_kinni_hoidvad_mehed_varvi4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"kaest_kinni_hoidvad_mehed_varvi4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"kaest_kinni_hoidvad_mehed_varvi4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"kaest_kinni_hoidvad_mehed_varvi4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"kaest_kinni_hoidvad_mehed_varvi5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"kaest_kinni_hoidvad_mehed_varvi5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"kaest_kinni_hoidvad_mehed_varvi5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"kaest_kinni_hoidvad_mehed_varvi5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"kaest_kinni_hoidvad_naised", "1F46D"},
    {"kaest_kinni_hoidvad_naised_varvi1", "1F46D-1F3FB"},
    {"kaest_kinni_hoidvad_naised_varvi2", "1F46D-1F3FC"},
    {"kaest_kinni_hoidvad_naised_varvi3", "1F46D-1F3FD"},
    {"kaest_kinni_hoidvad_naised_varvi4", "1F46D-1F3FE"},
    {"kaest_kinni_hoidvad_naised_varvi5", "1F46D-1F3FF"},
    {"kaest_kinni_hoidvad_naised_varvi1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"kaest_kinni_hoidvad_naised_varvi1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"kaest_kinni_hoidvad_naised_varvi1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"kaest_kinni_hoidvad_naised_varvi1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"kaest_kinni_hoidvad_naised_varvi2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"kaest_kinni_hoidvad_naised_varvi2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"kaest_kinni_hoidvad_naised_varvi2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"kaest_kinni_hoidvad_naised_varvi2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"kaest_kinni_hoidvad_naised_varvi3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"kaest_kinni_hoidvad_naised_varvi3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"kaest_kinni_hoidvad_naised_varvi3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"kaest_kinni_hoidvad_naised_varvi3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"kaest_kinni_hoidvad_naised_varvi4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"kaest_kinni_hoidvad_naised_varvi4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"kaest_kinni_hoidvad_naised_varvi4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"kaest_kinni_hoidvad_naised_varvi4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"kaest_kinni_hoidvad_naised_varvi5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"kaest_kinni_hoidvad_naised_varvi5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"kaest_kinni_hoidvad_naised_varvi5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"kaest_kinni_hoidvad_naised_varvi5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"politseinik", "1F46E"},
    {"politseinik_varvi1", "1F46E-1F3FB"},
    {"politseinik_varvi2", "1F46E-1F3FC"},
    {"politseinik_varvi3", "1F46E-1F3FD"},
    {"politseinik_varvi4", "1F46E-1F3FE"},
    {"politseinik_varvi5", "1F46E-1F3FF"},
    {"pidutsevad_janesekorvadega_inimesed", "1F46F"},
    {"pidutsevad_janesekorvadega_inimesed_varvi1", "1F46F-1F3FB"},
    {"pidutsevad_janesekorvadega_inimesed_varvi2", "1F46F-1F3FC"},
    {"pidutsevad_janesekorvadega_inimesed_varvi3", "1F46F-1F3FD"},
    {"pidutsevad_janesekorvadega_inimesed_varvi4", "1F46F-1F3FE"},
    {"pidutsevad_janesekorvadega_inimesed_varvi5", "1F46F-1F3FF"},
    {"pidutsevad_janesekorvadega_inimesed_varvi1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"pidutsevad_janesekorvadega_inimesed_varvi1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"pidutsevad_janesekorvadega_inimesed_varvi1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"pidutsevad_janesekorvadega_inimesed_varvi1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"pidutsevad_janesekorvadega_inimesed_varvi2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"pidutsevad_janesekorvadega_inimesed_varvi2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"pidutsevad_janesekorvadega_inimesed_varvi2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"pidutsevad_janesekorvadega_inimesed_varvi2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"pidutsevad_janesekorvadega_inimesed_varvi3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"pidutsevad_janesekorvadega_inimesed_varvi3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"pidutsevad_janesekorvadega_inimesed_varvi3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"pidutsevad_janesekorvadega_inimesed_varvi3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"pidutsevad_janesekorvadega_inimesed_varvi4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"pidutsevad_janesekorvadega_inimesed_varvi4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"pidutsevad_janesekorvadega_inimesed_varvi4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"pidutsevad_janesekorvadega_inimesed_varvi4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"pidutsevad_janesekorvadega_inimesed_varvi5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"pidutsevad_janesekorvadega_inimesed_varvi5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"pidutsevad_janesekorvadega_inimesed_varvi5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"pidutsevad_janesekorvadega_inimesed_varvi5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"loori_kandev_inimene", "1F470"},
    {"loori_kandev_inimene_varvi1", "1F470-1F3FB"},
    {"loori_kandev_inimene_varvi2", "1F470-1F3FC"},
    {"loori_kandev_inimene_varvi3", "1F470-1F3FD"},
    {"loori_kandev_inimene_varvi4", "1F470-1F3FE"},
    {"loori_kandev_inimene_varvi5", "1F470-1F3FF"},
    {"inimene_blondide_juustega", "1F471"},
    {"inimene_blondide_juustega_varvi1", "1F471-1F3FB"},
    {"inimene_blondide_juustega_varvi2", "1F471-1F3FC"},
    {"inimene_blondide_juustega_varvi3", "1F471-1F3FD"},
    {"inimene_blondide_juustega_varvi4", "1F471-1F3FE"},
    {"inimene_blondide_juustega_varvi5", "1F471-1F3FF"},
    {"hiina_mutsiga_mees", "1F472"},
    {"hiina_mutsiga_mees_varvi1", "1F472-1F3FB"},
    {"hiina_mutsiga_mees_varvi2", "1F472-1F3FC"},
    {"hiina_mutsiga_mees_varvi3", "1F472-1F3FD"},
    {"hiina_mutsiga_mees_varvi4", "1F472-1F3FE"},
    {"hiina_mutsiga_mees_varvi5", "1F472-1F3FF"},
    {"turbaniga_inimene", "1F473"},
    {"turbaniga_inimene_varvi1", "1F473-1F3FB"},
    {"turbaniga_inimene_varvi2", "1F473-1F3FC"},
    {"turbaniga_inimene_varvi3", "1F473-1F3FD"},
    {"turbaniga_inimene_varvi4", "1F473-1F3FE"},
    {"turbaniga_inimene_varvi5", "1F473-1F3FF"},
    {"eakas_mees", "1F474"},
    {"eakas_mees_varvi1", "1F474-1F3FB"},
    {"eakas_mees_varvi2", "1F474-1F3FC"},
    {"eakas_mees_varvi3", "1F474-1F3FD"},
    {"eakas_mees_varvi4", "1F474-1F3FE"},
    {"eakas_mees_varvi5", "1F474-1F3FF"},
    {"eakas_naine", "1F475"},
    {"eakas_naine_varvi1", "1F475-1F3FB"},
    {"eakas_naine_varvi2", "1F475-1F3FC"},
    {"eakas_naine_varvi3", "1F475-1F3FD"},
    {"eakas_naine_varvi4", "1F475-1F3FE"},
    {"eakas_naine_varvi5", "1F475-1F3FF"},
    {"beebi", "1F476"},
    {"beebi_varvi1", "1F476-1F3FB"},
    {"beebi_varvi2", "1F476-1F3FC"},
    {"beebi_varvi3", "1F476-1F3FD"},
    {"beebi_varvi4", "1F476-1F3FE"},
    {"beebi_varvi5", "1F476-1F3FF"},
    {"ehitaja", "1F477"},
    {"ehitaja_varvi1", "1F477-1F3FB"},
    {"ehitaja_varvi2", "1F477-1F3FC"},
    {"ehitaja_varvi3", "1F477-1F3FD"},
    {"ehitaja_varvi4", "1F477-1F3FE"},
    {"ehitaja_varvi5", "1F477-1F3FF"},
    {"printsess", "1F478"},
    {"printsess_varvi1", "1F478-1F3FB"},
    {"printsess_varvi2", "1F478-1F3FC"},
    {"printsess_varvi3", "1F478-1F3FD"},
    {"printsess_varvi4", "1F478-1F3FE"},
    {"printsess_varvi5", "1F478-1F3FF"},
    {"sookoll", "1F479"},
    {"maekoll", "1F47A"},
    {"kummitus", "1F47B"},
    {"inglibeebi", "1F47C"},
    {"inglibeebi_varvi1", "1F47C-1F3FB"},
    {"inglibeebi_varvi2", "1F47C-1F3FC"},
    {"inglibeebi_varvi3", "1F47C-1F3FD"},
    {"inglibeebi_varvi4", "1F47C-1F3FE"},
    {"inglibeebi_varvi5", "1F47C-1F3FF"},
    {"tulnukas", "1F47D"},
    {"tulnukkoletis", "1F47E"},
    {"deemon", "1F47F"},
    {"kolp", "1F480"},
    {"infoleti_tootaja", "1F481"},
    {"infoleti_tootaja_varvi1", "1F481-1F3FB"},
    {"infoleti_tootaja_varvi2", "1F481-1F3FC"},
    {"infoleti_tootaja_varvi3", "1F481-1F3FD"},
    {"infoleti_tootaja_varvi4", "1F481-1F3FE"},
    {"infoleti_tootaja_varvi5", "1F481-1F3FF"},
    {"valvur", "1F482"},
    {"valvur_varvi1", "1F482-1F3FB"},
    {"valvur_varvi2", "1F482-1F3FC"},
    {"valvur_varvi3", "1F482-1F3FD"},
    {"valvur_varvi4", "1F482-1F3FE"},
    {"valvur_varvi5", "1F482-1F3FF"},
    {"tantsiv_naine", "1F483"},
    {"tantsiv_naine_varvi1", "1F483-1F3FB"},
    {"tantsiv_naine_varvi2", "1F483-1F3FC"},
    {"tantsiv_naine_varvi3", "1F483-1F3FD"},
    {"tantsiv_naine_varvi4", "1F483-1F3FE"},
    {"tantsiv_naine_varvi5", "1F483-1F3FF"},
    {"huulepulk", "1F484"},
    {"kuunelakk", "1F485"},
    {"kuunelakk_varvi1", "1F485-1F3FB"},
    {"kuunelakk_varvi2", "1F485-1F3FC"},
    {"kuunelakk_varvi3", "1F485-1F3FD"},
    {"kuunelakk_varvi4", "1F485-1F3FE"},
    {"kuunelakk_varvi5", "1F485-1F3FF"},
    {"naomassaaz", "1F486"},
    {"naomassaaz_varvi1", "1F486-1F3FB"},
    {"naomassaaz_varvi2", "1F486-1F3FC"},
    {"naomassaaz_varvi3", "1F486-1F3FD"},
    {"naomassaaz_varvi4", "1F486-1F3FE"},
    {"naomassaaz_varvi5", "1F486-1F3FF"},
    {"juukseloikus", "1F487"},
    {"juukseloikus_varvi1", "1F487-1F3FB"},
    {"juukseloikus_varvi2", "1F487-1F3FC"},
    {"juukseloikus_varvi3", "1F487-1F3FD"},
    {"juukseloikus_varvi4", "1F487-1F3FE"},
    {"juukseloikus_varvi5", "1F487-1F3FF"},
    {"habemeajaja_tahis", "1F488"},
    {"sustal", "1F489"},
    {"tablett", "1F48A"},
    {"suudluse_jalg", "1F48B"},
    {"armastuskiri", "1F48C"},
    {"sormus", "1F48D"},
    {"kalliskivi", "1F48E"},
    {"musi", "1F48F"},
    {"musi_varvi1", "1F48F-1F3FB"},
    {"musi_varvi2", "1F48F-1F3FC"},
    {"musi_varvi3", "1F48F-1F3FD"},
    {"musi_varvi4", "1F48F-1F3FE"},
    {"musi_varvi5", "1F48F-1F3FF"},
    {"musi_varvi1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"musi_varvi1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"musi_varvi1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"musi_varvi1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"musi_varvi2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"musi_varvi2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"musi_varvi2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"musi_varvi2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"musi_varvi3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"musi_varvi3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"musi_varvi3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"musi_varvi3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"musi_varvi4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"musi_varvi4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"musi_varvi4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"musi_varvi4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"musi_varvi5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"musi_varvi5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"musi_varvi5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"musi_varvi5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"kimp", "1F490"},
    {"paarike_sudamega", "1F491"},
    {"paarike_sudamega_varvi1", "1F491-1F3FB"},
    {"paarike_sudamega_varvi2", "1F491-1F3FC"},
    {"paarike_sudamega_varvi3", "1F491-1F3FD"},
    {"paarike_sudamega_varvi4", "1F491-1F3FE"},
    {"paarike_sudamega_varvi5", "1F491-1F3FF"},
    {"paarike_sudamega_varvi1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"paarike_sudamega_varvi1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"paarike_sudamega_varvi1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"paarike_sudamega_varvi1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"paarike_sudamega_varvi2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"paarike_sudamega_varvi2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"paarike_sudamega_varvi2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"paarike_sudamega_varvi2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"paarike_sudamega_varvi3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"paarike_sudamega_varvi3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"paarike_sudamega_varvi3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"paarike_sudamega_varvi3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"paarike_sudamega_varvi4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"paarike_sudamega_varvi4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"paarike_sudamega_varvi4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"paarike_sudamega_varvi4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"paarike_sudamega_varvi5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"paarike_sudamega_varvi5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"paarike_sudamega_varvi5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"paarike_sudamega_varvi5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pulmad", "1F492"},
    {"tukslev_suda", "1F493"},
    {"murtud_suda", "1F494"},
    {"kaks_sudant", "1F495"},
    {"sadelev_suda", "1F496"},
    {"kasvav_suda", "1F497"},
    {"noolega_suda", "1F498"},
    {"sinine_suda", "1F499"},
    {"roheline_suda", "1F49A"},
    {"kollane_suda", "1F49B"},
    {"lilla_suda", "1F49C"},
    {"lindiga_suda", "1F49D"},
    {"poorlevad_sudamed", "1F49E"},
    {"sudamekujuline_kaunistus", "1F49F"},
    {"punktiga_romb", "1F4A0"},
    {"lamp", "1F4A1"},
    {"viha_sumbol", "1F4A2"},
    {"pomm", "1F4A3"},
    {"zzz", "1F4A4"},
    {"piraki", "1F4A5"},
    {"higipiisad", "1F4A6"},
    {"piisk", "1F4A7"},
    {"kiirustamine", "1F4A8"},
    {"kakahunnik", "1F4A9"},
    {"pingutatud_biitseps", "1F4AA"},
    {"pingutatud_biitseps_varvi1", "1F4AA-1F3FB"},
    {"pingutatud_biitseps_varvi2", "1F4AA-1F3FC"},
    {"pingutatud_biitseps_varvi3", "1F4AA-1F3FD"},
    {"pingutatud_biitseps_varvi4", "1F4AA-1F3FE"},
    {"pingutatud_biitseps_varvi5", "1F4AA-1F3FF"},
    {"peadpooritav", "1F4AB"},
    {"jutumull", "1F4AC"},
    {"mottemull", "1F4AD"},
    {"valge_lill", "1F4AE"},
    {"sada_punkti", "1F4AF"},
    {"rahapaun", "1F4B0"},
    {"valuutavahetus", "1F4B1"},
    {"suur_dollarimark", "1F4B2"},
    {"krediitkaart", "1F4B3"},
    {"jeeni_rahataht", "1F4B4"},
    {"dollari_rahataht", "1F4B5"},
    {"euro_rahataht", "1F4B6"},
    {"naela_rahataht", "1F4B7"},
    {"tiibadega_raha", "1F4B8"},
    {"tousev_graafik_jeenisumboliga", "1F4B9"},
    {"iste", "1F4BA"},
    {"sulearvuti", "1F4BB"},
    {"portfell", "1F4BC"},
    {"arvutiketas", "1F4BD"},
    {"diskett", "1F4BE"},
    {"optiline_ketas", "1F4BF"},
    {"dvd", "1F4C0"},
    {"kaust", "1F4C1"},
    {"avatud_kaust", "1F4C2"},
    {"kaardunud_paberileht", "1F4C3"},
    {"paberilehe_eestvaade", "1F4C4"},
    {"kalender", "1F4C5"},
    {"rebitav_kalender", "1F4C6"},
    {"kataloog", "1F4C7"},
    {"tousev_graafik", "1F4C8"},
    {"langev_graafik", "1F4C9"},
    {"tulpdiagramm", "1F4CA"},
    {"kirjutamisalus", "1F4CB"},
    {"noopnoel", "1F4CC"},
    {"umara_peaga_noopnoel", "1F4CD"},
    {"kirjaklamber", "1F4CE"},
    {"sirge_joonlaud", "1F4CF"},
    {"kolmnurkne_joonlaud", "1F4D0"},
    {"jarjehoidjad", "1F4D1"},
    {"arveraamat", "1F4D2"},
    {"paevik", "1F4D3"},
    {"dekoratiivse_kaanega_markmik", "1F4D4"},
    {"suletud_raamat", "1F4D5"},
    {"avatud_raamat", "1F4D6"},
    {"roheline_raamat", "1F4D7"},
    {"sinine_raamat", "1F4D8"},
    {"oranz_raamat", "1F4D9"},
    {"raamatud", "1F4DA"},
    {"nimesilt", "1F4DB"},
    {"kasikiri", "1F4DC"},
    {"memo", "1F4DD"},
    {"telefonitoru", "1F4DE"},
    {"piipar", "1F4DF"},
    {"faksimasin", "1F4E0"},
    {"satelliitantenn", "1F4E1"},
    {"valjuhaaldi", "1F4E2"},
    {"megafon", "1F4E3"},
    {"valjaminev_post", "1F4E4"},
    {"sissetulev_post", "1F4E5"},
    {"pakk", "1F4E6"},
    {"meilisonum", "1F4E7"},
    {"sissetulev_umbrik", "1F4E8"},
    {"noolega_umbrik", "1F4E9"},
    {"suletud_postkast_langetatud_lipuga", "1F4EA"},
    {"suletud_postkast_tostetud_lipuga", "1F4EB"},
    {"avatud_postkast_tostetud_lipuga", "1F4EC"},
    {"avatud_postkast_langetatud_lipuga", "1F4ED"},
    {"postkast", "1F4EE"},
    {"postisarv", "1F4EF"},
    {"ajaleht", "1F4F0"},
    {"mobiiltelefon", "1F4F1"},
    {"noolega_mobiiltelefon", "1F4F2"},
    {"varinareziim", "1F4F3"},
    {"valjalulitatud_mobiiltelefon", "1F4F4"},
    {"telefonid_on_keelatud", "1F4F5"},
    {"levi", "1F4F6"},
    {"kaamera", "1F4F7"},
    {"valguga_kaamera", "1F4F8"},
    {"videokaamera", "1F4F9"},
    {"teler", "1F4FA"},
    {"raadio", "1F4FB"},
    {"videokassett", "1F4FC"},
    {"filmiprojektor", "1F4FD"},
    {"palvehelmed", "1F4FF"},
    {"juhusliku_jarjekorra_nupp", "1F500"},
    {"kordamisnupp", "1F501"},
    {"uhe_korduse_nupp", "1F502"},
    {"vertikaalsed_nooled_paripaeva", "1F503"},
    {"nooled_vastupaeva", "1F504"},
    {"tuhmistamisnupp", "1F505"},
    {"ereduse_nupp", "1F506"},
    {"vaigistatud_kolar", "1F507"},
    {"vaikse_heliga_kolar", "1F508"},
    {"poolvalju_heliga_kolar", "1F509"},
    {"valju_heliga_kolar", "1F50A"},
    {"aku", "1F50B"},
    {"elektripistik", "1F50C"},
    {"vasakule_kallutatud_luup", "1F50D"},
    {"paremale_kallutatud_luup", "1F50E"},
    {"lukk_ja_pastakas", "1F50F"},
    {"suletud_lukk_votmega", "1F510"},
    {"voti", "1F511"},
    {"lukk", "1F512"},
    {"avatud_lukk", "1F513"},
    {"kelluke", "1F514"},
    {"vaigistatud_kelluke", "1F515"},
    {"jarjehoidja", "1F516"},
    {"luli", "1F517"},
    {"raadionupp", "1F518"},
    {"nool_back", "1F519"},
    {"nool_end", "1F51A"},
    {"nool_on", "1F51B"},
    {"nool_soon", "1F51C"},
    {"nool_top", "1F51D"},
    {"alla_18_aastastele_keelatud", "1F51E"},
    {"klahv_10", "1F51F"},
    {"ladina_suurtahed", "1F520"},
    {"ladina_vaiketahed", "1F521"},
    {"numbrid", "1F522"},
    {"sumbolid", "1F523"},
    {"ladina_tahed", "1F524"},
    {"tuli", "1F525"},
    {"taskulamp", "1F526"},
    {"mutrivoti", "1F527"},
    {"haamer", "1F528"},
    {"polt_ja_mutter", "1F529"},
    {"kooginuga", "1F52A"},
    {"veepustol", "1F52B"},
    {"mikroskoop", "1F52C"},
    {"teleskoop", "1F52D"},
    {"kristallkuul", "1F52E"},
    {"punktiga_kuusnurkne_taht", "1F52F"},
    {"algaja_sumbol", "1F530"},
    {"kolmhargi_sumbol", "1F531"},
    {"must_ruudukujuline_nupp", "1F532"},
    {"valge_ruudukujuline_nupp", "1F533"},
    {"punane_ring", "1F534"},
    {"sinine_ring", "1F535"},
    {"suur_oranz_romb", "1F536"},
    {"suur_sinine_romb", "1F537"},
    {"vaike_oranz_romb", "1F538"},
    {"vaike_sinine_romb", "1F539"},
    {"punane_kolmnurk", "1F53A"},
    {"punane_tagurpidi_kolmnurk", "1F53B"},
    {"ulesnupp", "1F53C"},
    {"allanupp", "1F53D"},
    {"hinduism", "1F549"},
    {"tuvi", "1F54A"},
    {"kaaba", "1F54B"},
    {"mosee", "1F54C"},
    {"sunagoog", "1F54D"},
    {"menoraa", "1F54E"},
    {"kell_uks", "1F550"},
    {"kell_kaks", "1F551"},
    {"kell_kolm", "1F552"},
    {"kell_neli", "1F553"},
    {"kell_viis", "1F554"},
    {"kell_kuus", "1F555"},
    {"kell_seitse", "1F556"},
    {"kell_kaheksa", "1F557"},
    {"kell_uheksa", "1F558"},
    {"kell_kumme", "1F559"},
    {"kell_uksteist", "1F55A"},
    {"kell_kaksteist", "1F55B"},
    {"kell_pool_kaks", "1F55C"},
    {"kell_pool_kolm", "1F55D"},
    {"kell_pool_neli", "1F55E"},
    {"kell_pool_viis", "1F55F"},
    {"kell_pool_kuus", "1F560"},
    {"kell_pool_seitse", "1F561"},
    {"kell_pool_kaheksa", "1F562"},
    {"kell_pool_uheksa", "1F563"},
    {"kell_pool_kumme", "1F564"},
    {"kell_pool_uksteist", "1F565"},
    {"kell_pool_kaksteist", "1F566"},
    {"kell_pool_uks", "1F567"},
    {"kuunal", "1F56F"},
    {"kaminakell", "1F570"},
    {"auk", "1F573"},
    {"holjuv_ulikonnaga_mees", "1F574"},
    {"holjuv_ulikonnaga_mees_varvi1", "1F574-1F3FB"},
    {"holjuv_ulikonnaga_mees_varvi2", "1F574-1F3FC"},
    {"holjuv_ulikonnaga_mees_varvi3", "1F574-1F3FD"},
    {"holjuv_ulikonnaga_mees_varvi4", "1F574-1F3FE"},
    {"holjuv_ulikonnaga_mees_varvi5", "1F574-1F3FF"},
    {"detektiiv", "1F575"},
    {"detektiiv_varvi1", "1F575-1F3FB"},
    {"detektiiv_varvi2", "1F575-1F3FC"},
    {"detektiiv_varvi3", "1F575-1F3FD"},
    {"detektiiv_varvi4", "1F575-1F3FE"},
    {"detektiiv_varvi5", "1F575-1F3FF"},
    {"paikeseprillid", "1F576"},
    {"amblik", "1F577"},
    {"amblikuvork", "1F578"},
    {"juhtpult", "1F579"},
    {"tantsiv_mees", "1F57A"},
    {"tantsiv_mees_varvi1", "1F57A-1F3FB"},
    {"tantsiv_mees_varvi2", "1F57A-1F3FC"},
    {"tantsiv_mees_varvi3", "1F57A-1F3FD"},
    {"tantsiv_mees_varvi4", "1F57A-1F3FE"},
    {"tantsiv_mees_varvi5", "1F57A-1F3FF"},
    {"uhendatud_kirjaklambrid", "1F587"},
    {"pastapliiats", "1F58A"},
    {"sulepea", "1F58B"},
    {"pintsel", "1F58C"},
    {"kriit", "1F58D"},
    {"harali_sormedega_kasi", "1F590"},
    {"harali_sormedega_kasi_varvi1", "1F590-1F3FB"},
    {"harali_sormedega_kasi_varvi2", "1F590-1F3FC"},
    {"harali_sormedega_kasi_varvi3", "1F590-1F3FD"},
    {"harali_sormedega_kasi_varvi4", "1F590-1F3FE"},
    {"harali_sormedega_kasi_varvi5", "1F590-1F3FF"},
    {"keskmine_sorm", "1F595"},
    {"keskmine_sorm_varvi1", "1F595-1F3FB"},
    {"keskmine_sorm_varvi2", "1F595-1F3FC"},
    {"keskmine_sorm_varvi3", "1F595-1F3FD"},
    {"keskmine_sorm_varvi4", "1F595-1F3FE"},
    {"keskmine_sorm_varvi5", "1F595-1F3FF"},
    {"vulcanite_tervitus", "1F596"},
    {"vulcanite_tervitus_varvi1", "1F596-1F3FB"},
    {"vulcanite_tervitus_varvi2", "1F596-1F3FC"},
    {"vulcanite_tervitus_varvi3", "1F596-1F3FD"},
    {"vulcanite_tervitus_varvi4", "1F596-1F3FE"},
    {"vulcanite_tervitus_varvi5", "1F596-1F3FF"},
    {"must_suda", "1F5A4"},
    {"lauaarvuti", "1F5A5"},
    {"printer", "1F5A8"},
    {"arvutihiir", "1F5B1"},
    {"juhtkuul", "1F5B2"},
    {"raamitud_pilt", "1F5BC"},
    {"register", "1F5C2"},
    {"dokumendikast", "1F5C3"},
    {"dokumendikapp", "1F5C4"},
    {"prugikorv", "1F5D1"},
    {"spiraalmarkmik", "1F5D2"},
    {"spiraalkalender", "1F5D3"},
    {"kruustangid", "1F5DC"},
    {"vana_voti", "1F5DD"},
    {"rullikeeratud_ajaleht", "1F5DE"},
    {"pistoda", "1F5E1"},
    {"konelev_pea", "1F5E3"},
    {"jutumull_suunaga_vasakule", "1F5E8"},
    {"vihamull_suunaga_paremale", "1F5EF"},
    {"haaletuskast", "1F5F3"},
    {"maailmakaart", "1F5FA"},
    {"fuji_magi", "1F5FB"},
    {"tokyo_torn", "1F5FC"},
    {"vabadussammas", "1F5FD"},
    {"jaapani_kaart", "1F5FE"},
    {"moai", "1F5FF"},
    {"laia_naeratusega_nagu", "1F600"},
    {"naerul_silmade_ja_laia_naeratusega_nagu", "1F601"},
    {"roomupisaratega_nagu", "1F602"},
    {"suurte_silmadega_naerunagu", "1F603"},
    {"avatud_suu_ja_naeratavate_silmadega_naerunagu", "1F604"},
    {"higipiisa_ja_avatud_suuga_naerunagu", "1F605"},
    {"avatud_suu_ja_suletud_silmadega_naerunagu", "1F606"},
    {"aupaistega_naerunagu", "1F607"},
    {"sarviline_naerunagu", "1F608"},
    {"silma_pilgutav_nagu", "1F609"},
    {"naeratavate_silmadega_naerunagu", "1F60A"},
    {"maitsvat_toitu_nautiv_nagu", "1F60B"},
    {"kergendust_tundev_nagu", "1F60C"},
    {"sudamekujuliste_silmadega_naerunagu", "1F60D"},
    {"paikeseprillidega_naerunagu", "1F60E"},
    {"muigav_nagu", "1F60F"},
    {"erapooletu_nagu", "1F610"},
    {"ilmetu_nagu", "1F611"},
    {"polglik_nagu", "1F612"},
    {"kulma_higiga_kaetud_nagu", "1F613"},
    {"motlik_nagu", "1F614"},
    {"segaduses_nagu", "1F615"},
    {"kimbatuses_nagu", "1F616"},
    {"suudlev_nagu", "1F617"},
    {"suudlust_saatev_nagu", "1F618"},
    {"naeratavate_silmadega_suudlev_nagu", "1F619"},
    {"suletud_silmadega_suudlev_nagu", "1F61A"},
    {"suust_valjas_keelega_nagu", "1F61B"},
    {"suust_valjas_keelega_ja_silma_pilgutav_nagu", "1F61C"},
    {"keelt_naitav_suletud_silmadega_nagu", "1F61D"},
    {"pettunud_nagu", "1F61E"},
    {"murelik_nagu", "1F61F"},
    {"tige_nagu", "1F620"},
    {"mossitav_nagu", "1F621"},
    {"nuttev_nagu", "1F622"},
    {"sihikindel_nagu", "1F623"},
    {"ninast_auru_ajav_nagu", "1F624"},
    {"pettunud_kuid_kergendust_tundev_nagu", "1F625"},
    {"avatud_suuga_mossitav_nagu", "1F626"},
    {"ahastuses_nagu", "1F627"},
    {"hirmunud_nagu", "1F628"},
    {"rammestunud_nagu", "1F629"},
    {"unine_nagu", "1F62A"},
    {"vasinud_nagu", "1F62B"},
    {"grimassitav_nagu", "1F62C"},
    {"valjusti_nuttev_nagu", "1F62D"},
    {"avatud_suuga_nagu", "1F62E"},
    {"vaigistatud_nagu", "1F62F"},
    {"kulma_higiga_kaetud_ja_avatud_suuga_nagu", "1F630"},
    {"hirmunult_karjuv_nagu", "1F631"},
    {"hammastunud_nagu", "1F632"},
    {"punastav_nagu", "1F633"},
    {"magav_nagu", "1F634"},
    {"oimetu_nagu", "1F635"},
    {"ilma_suuta_nagu", "1F636"},
    {"kirurgimaskiga_kaetud_nagu", "1F637"},
    {"lobusate_silmadega_irvitav_kass", "1F638"},
    {"roomupisarais_kass", "1F639"},
    {"avatud_suuga_naeratav_kass", "1F63A"},
    {"sudamekujuliste_silmadega_naeratav_kass", "1F63B"},
    {"virilalt_naeratav_kass", "1F63C"},
    {"suletud_silmadega_suudlev_kass", "1F63D"},
    {"mossitav_kass", "1F63E"},
    {"nuttev_kass", "1F63F"},
    {"ullatunud_kass", "1F640"},
    {"veidi_kulmu_kortsutav_nagu", "1F641"},
    {"naeratusega_nagu", "1F642"},
    {"tagurpidi_nagu", "1F643"},
    {"pooritavate_silmadega_nagu", "1F644"},
    {"eitav_zest", "1F645"},
    {"eitav_zest_varvi1", "1F645-1F3FB"},
    {"eitav_zest_varvi2", "1F645-1F3FC"},
    {"eitav_zest_varvi3", "1F645-1F3FD"},
    {"eitav_zest_varvi4", "1F645-1F3FE"},
    {"eitav_zest_varvi5", "1F645-1F3FF"},
    {"noustuv_zest", "1F646"},
    {"noustuv_zest_varvi1", "1F646-1F3FB"},
    {"noustuv_zest_varvi2", "1F646-1F3FC"},
    {"noustuv_zest_varvi3", "1F646-1F3FD"},
    {"noustuv_zest_varvi4", "1F646-1F3FE"},
    {"noustuv_zest_varvi5", "1F646-1F3FF"},
    {"kummardav_inimene", "1F647"},
    {"kummardav_inimene_varvi1", "1F647-1F3FB"},
    {"kummardav_inimene_varvi2", "1F647-1F3FC"},
    {"kummardav_inimene_varvi3", "1F647-1F3FD"},
    {"kummardav_inimene_varvi4", "1F647-1F3FE"},
    {"kummardav_inimene_varvi5", "1F647-1F3FF"},
    {"kaetud_silmadega_ahv", "1F648"},
    {"kaetud_korvadega_ahv", "1F649"},
    {"kaetud_suuga_ahv", "1F64A"},
    {"katt_tostev_roomus_inimene", "1F64B"},
    {"katt_tostev_roomus_inimene_varvi1", "1F64B-1F3FB"},
    {"katt_tostev_roomus_inimene_varvi2", "1F64B-1F3FC"},
    {"katt_tostev_roomus_inimene_varvi3", "1F64B-1F3FD"},
    {"katt_tostev_roomus_inimene_varvi4", "1F64B-1F3FE"},
    {"katt_tostev_roomus_inimene_varvi5", "1F64B-1F3FF"},
    {"ulestostetud_kaed", "1F64C"},
    {"ulestostetud_kaed_varvi1", "1F64C-1F3FB"},
    {"ulestostetud_kaed_varvi2", "1F64C-1F3FC"},
    {"ulestostetud_kaed_varvi3", "1F64C-1F3FD"},
    {"ulestostetud_kaed_varvi4", "1F64C-1F3FE"},
    {"ulestostetud_kaed_varvi5", "1F64C-1F3FF"},
    {"kulmu_kortsutav_inimene", "1F64D"},
    {"kulmu_kortsutav_inimene_varvi1", "1F64D-1F3FB"},
    {"kulmu_kortsutav_inimene_varvi2", "1F64D-1F3FC"},
    {"kulmu_kortsutav_inimene_varvi3", "1F64D-1F3FD"},
    {"kulmu_kortsutav_inimene_varvi4", "1F64D-1F3FE"},
    {"kulmu_kortsutav_inimene_varvi5", "1F64D-1F3FF"},
    {"mossitav_inimene", "1F64E"},
    {"mossitav_inimene_varvi1", "1F64E-1F3FB"},
    {"mossitav_inimene_varvi2", "1F64E-1F3FC"},
    {"mossitav_inimene_varvi3", "1F64E-1F3FD"},
    {"mossitav_inimene_varvi4", "1F64E-1F3FE"},
    {"mossitav_inimene_varvi5", "1F64E-1F3FF"},
    {"palveasendis_kaed", "1F64F"},
    {"palveasendis_kaed_varvi1", "1F64F-1F3FB"},
    {"palveasendis_kaed_varvi2", "1F64F-1F3FC"},
    {"palveasendis_kaed_varvi3", "1F64F-1F3FD"},
    {"palveasendis_kaed_varvi4", "1F64F-1F3FE"},
    {"palveasendis_kaed_varvi5", "1F64F-1F3FF"},
    {"rakett", "1F680"},
    {"helikopter", "1F681"},
    {"vedur", "1F682"},
    {"rongivagun", "1F683"},
    {"kiirrong", "1F684"},
    {"terava_ninaga_kiirrong", "1F685"},
    {"rong", "1F686"},
    {"metroo", "1F687"},
    {"kergraudtee", "1F688"},
    {"jaam", "1F689"},
    {"tramm", "1F68A"},
    {"trammivagun", "1F68B"},
    {"buss", "1F68C"},
    {"buss_eestvaates", "1F68D"},
    {"trollibuss", "1F68E"},
    {"bussipeatus", "1F68F"},
    {"vaikebuss", "1F690"},
    {"kiirabiauto", "1F691"},
    {"tuletorjeauto", "1F692"},
    {"politseiauto", "1F693"},
    {"politseiauto_eestvaates", "1F694"},
    {"takso", "1F695"},
    {"takso_eestvaates", "1F696"},
    {"auto", "1F697"},
    {"auto_eestvaates", "1F698"},
    {"mahtuniversaal", "1F699"},
    {"kulleriauto", "1F69A"},
    {"veok", "1F69B"},
    {"traktor", "1F69C"},
    {"uheroopmeline_raudtee", "1F69D"},
    {"magiraudtee", "1F69E"},
    {"rippraudtee", "1F69F"},
    {"koistee_magedes", "1F6A0"},
    {"koisraudtee", "1F6A1"},
    {"laev", "1F6A2"},
    {"aerupaadiga_inimene", "1F6A3"},
    {"aerupaadiga_inimene_varvi1", "1F6A3-1F3FB"},
    {"aerupaadiga_inimene_varvi2", "1F6A3-1F3FC"},
    {"aerupaadiga_inimene_varvi3", "1F6A3-1F3FD"},
    {"aerupaadiga_inimene_varvi4", "1F6A3-1F3FE"},
    {"aerupaadiga_inimene_varvi5", "1F6A3-1F3FF"},
    {"kiirkaater", "1F6A4"},
    {"horisontaalne_foor", "1F6A5"},
    {"vertikaalne_foor", "1F6A6"},
    {"ehitustood", "1F6A7"},
    {"politseiauto_vilkur", "1F6A8"},
    {"kolmnurkne_lipp", "1F6A9"},
    {"uks", "1F6AA"},
    {"keelatud", "1F6AB"},
    {"sigaret", "1F6AC"},
    {"suitsetamine_keelatud", "1F6AD"},
    {"prugikasti_silt", "1F6AE"},
    {"risustamine_keelatud", "1F6AF"},
    {"joogivesi", "1F6B0"},
    {"joomiseks_kolbmatu_vesi", "1F6B1"},
    {"jalgratas", "1F6B2"},
    {"jalgrattad_keelatud", "1F6B3"},
    {"jalgrattur", "1F6B4"},
    {"jalgrattur_varvi1", "1F6B4-1F3FB"},
    {"jalgrattur_varvi2", "1F6B4-1F3FC"},
    {"jalgrattur_varvi3", "1F6B4-1F3FD"},
    {"jalgrattur_varvi4", "1F6B4-1F3FE"},
    {"jalgrattur_varvi5", "1F6B4-1F3FF"},
    {"magirattur", "1F6B5"},
    {"magirattur_varvi1", "1F6B5-1F3FB"},
    {"magirattur_varvi2", "1F6B5-1F3FC"},
    {"magirattur_varvi3", "1F6B5-1F3FD"},
    {"magirattur_varvi4", "1F6B5-1F3FE"},
    {"magirattur_varvi5", "1F6B5-1F3FF"},
    {"jalakaija", "1F6B6"},
    {"jalakaija_varvi1", "1F6B6-1F3FB"},
    {"jalakaija_varvi2", "1F6B6-1F3FC"},
    {"jalakaija_varvi3", "1F6B6-1F3FD"},
    {"jalakaija_varvi4", "1F6B6-1F3FE"},
    {"jalakaija_varvi5", "1F6B6-1F3FF"},
    {"jalakaijad_keelatud", "1F6B7"},
    {"lapsed_teel", "1F6B8"},
    {"meeste_wc", "1F6B9"},
    {"naiste_wc", "1F6BA"},
    {"tualett", "1F6BB"},
    {"beebi_sumbol", "1F6BC"},
    {"wc_pott", "1F6BD"},
    {"wc", "1F6BE"},
    {"duss", "1F6BF"},
    {"inimene_vannis", "1F6C0"},
    {"inimene_vannis_varvi1", "1F6C0-1F3FB"},
    {"inimene_vannis_varvi2", "1F6C0-1F3FC"},
    {"inimene_vannis_varvi3", "1F6C0-1F3FD"},
    {"inimene_vannis_varvi4", "1F6C0-1F3FE"},
    {"inimene_vannis_varvi5", "1F6C0-1F3FF"},
    {"vann", "1F6C1"},
    {"passikontroll", "1F6C2"},
    {"toll", "1F6C3"},
    {"pagasi_kattesaamine", "1F6C4"},
    {"hoiukapid", "1F6C5"},
    {"diivan_ja_lamp", "1F6CB"},
    {"inimene_voodis", "1F6CC"},
    {"inimene_voodis_varvi1", "1F6CC-1F3FB"},
    {"inimene_voodis_varvi2", "1F6CC-1F3FC"},
    {"inimene_voodis_varvi3", "1F6CC-1F3FD"},
    {"inimene_voodis_varvi4", "1F6CC-1F3FE"},
    {"inimene_voodis_varvi5", "1F6CC-1F3FF"},
    {"ostukotid", "1F6CD"},
    {"margukell", "1F6CE"},
    {"voodi", "1F6CF"},
    {"puhakoda", "1F6D0"},
    {"stoppmark", "1F6D1"},
    {"ostukaru", "1F6D2"},
    {"hinduistlik_tempel", "1F6D5"},
    {"hutt", "1F6D6"},
    {"lift", "1F6D7"},
    {"maalihe", "1F6D8"},
    {"juhtmevaba", "1F6DC"},
    {"liumagi", "1F6DD"},
    {"ratas", "1F6DE"},
    {"paasterongas", "1F6DF"},
    {"haamer_ja_mutrivoti", "1F6E0"},
    {"kilp", "1F6E1"},
    {"olitunn", "1F6E2"},
    {"kiirtee", "1F6E3"},
    {"raudteeroopad", "1F6E4"},
    {"mootorpaat", "1F6E5"},
    {"vaikelennuk", "1F6E9"},
    {"ohkutousev_lennuk", "1F6EB"},
    {"maanduv_lennuk", "1F6EC"},
    {"satelliit", "1F6F0"},
    {"reisilaev", "1F6F3"},
    {"toukeratas", "1F6F4"},
    {"motoroller", "1F6F5"},
    {"kanuu", "1F6F6"},
    {"kelk", "1F6F7"},
    {"lendav_taldrik", "1F6F8"},
    {"rula", "1F6F9"},
    {"autoriksa", "1F6FA"},
    {"pikap", "1F6FB"},
    {"rulluisk", "1F6FC"},
    {"oranz_ring", "1F7E0"},
    {"kollane_ring", "1F7E1"},
    {"roheline_ring", "1F7E2"},
    {"lilla_ring", "1F7E3"},
    {"pruun_ring", "1F7E4"},
    {"punane_ruut", "1F7E5"},
    {"sinine_ruut", "1F7E6"},
    {"oranz_ruut", "1F7E7"},
    {"kollane_ruut", "1F7E8"},
    {"roheline_ruut", "1F7E9"},
    {"lilla_ruut", "1F7EA"},
    {"pruun_ruut", "1F7EB"},
    {"tugev_vordusmark", "1F7F0"},
    {"kokkusurutud_sormed", "1F90C"},
    {"kokkusurutud_sormed_varvi1", "1F90C-1F3FB"},
    {"kokkusurutud_sormed_varvi2", "1F90C-1F3FC"},
    {"kokkusurutud_sormed_varvi3", "1F90C-1F3FD"},
    {"kokkusurutud_sormed_varvi4", "1F90C-1F3FE"},
    {"kokkusurutud_sormed_varvi5", "1F90C-1F3FF"},
    {"valge_suda", "1F90D"},
    {"pruun_suda", "1F90E"},
    {"napistav_kasi", "1F90F"},
    {"napistav_kasi_varvi1", "1F90F-1F3FB"},
    {"napistav_kasi_varvi2", "1F90F-1F3FC"},
    {"napistav_kasi_varvi3", "1F90F-1F3FD"},
    {"napistav_kasi_varvi4", "1F90F-1F3FE"},
    {"napistav_kasi_varvi5", "1F90F-1F3FF"},
    {"tomblukusuuga_nagu", "1F910"},
    {"nagu_millel_on_raha_suus", "1F911"},
    {"kraadiklaasiga_nagu", "1F912"},
    {"nohiku_nagu", "1F913"},
    {"motlev_nagu", "1F914"},
    {"kinniseotud_peaga_nagu", "1F915"},
    {"robot", "1F916"},
    {"kallistav_nagu", "1F917"},
    {"sarvede_mark", "1F918"},
    {"sarvede_mark_varvi1", "1F918-1F3FB"},
    {"sarvede_mark_varvi2", "1F918-1F3FC"},
    {"sarvede_mark_varvi3", "1F918-1F3FD"},
    {"sarvede_mark_varvi4", "1F918-1F3FE"},
    {"sarvede_mark_varvi5", "1F918-1F3FF"},
    {"helista_mulle", "1F919"},
    {"helista_mulle_varvi1", "1F919-1F3FB"},
    {"helista_mulle_varvi2", "1F919-1F3FC"},
    {"helista_mulle_varvi3", "1F919-1F3FD"},
    {"helista_mulle_varvi4", "1F919-1F3FE"},
    {"helista_mulle_varvi5", "1F919-1F3FF"},
    {"tostetud_kasi", "1F91A"},
    {"tostetud_kasi_varvi1", "1F91A-1F3FB"},
    {"tostetud_kasi_varvi2", "1F91A-1F3FC"},
    {"tostetud_kasi_varvi3", "1F91A-1F3FD"},
    {"tostetud_kasi_varvi4", "1F91A-1F3FE"},
    {"tostetud_kasi_varvi5", "1F91A-1F3FF"},
    {"vasakule_suunatud_rusikas", "1F91B"},
    {"vasakule_suunatud_rusikas_varvi1", "1F91B-1F3FB"},
    {"vasakule_suunatud_rusikas_varvi2", "1F91B-1F3FC"},
    {"vasakule_suunatud_rusikas_varvi3", "1F91B-1F3FD"},
    {"vasakule_suunatud_rusikas_varvi4", "1F91B-1F3FE"},
    {"vasakule_suunatud_rusikas_varvi5", "1F91B-1F3FF"},
    {"paremale_suunatud_rusikas", "1F91C"},
    {"paremale_suunatud_rusikas_varvi1", "1F91C-1F3FB"},
    {"paremale_suunatud_rusikas_varvi2", "1F91C-1F3FC"},
    {"paremale_suunatud_rusikas_varvi3", "1F91C-1F3FD"},
    {"paremale_suunatud_rusikas_varvi4", "1F91C-1F3FE"},
    {"paremale_suunatud_rusikas_varvi5", "1F91C-1F3FF"},
    {"kaepigistus", "1F91D"},
    {"kaepigistus_varvi1", "1F91D-1F3FB"},
    {"kaepigistus_varvi2", "1F91D-1F3FC"},
    {"kaepigistus_varvi3", "1F91D-1F3FD"},
    {"kaepigistus_varvi4", "1F91D-1F3FE"},
    {"kaepigistus_varvi5", "1F91D-1F3FF"},
    {"kaepigistus_varvi1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"kaepigistus_varvi1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"kaepigistus_varvi1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"kaepigistus_varvi1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"kaepigistus_varvi2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"kaepigistus_varvi2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"kaepigistus_varvi2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"kaepigistus_varvi2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"kaepigistus_varvi3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"kaepigistus_varvi3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"kaepigistus_varvi3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"kaepigistus_varvi3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"kaepigistus_varvi4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"kaepigistus_varvi4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"kaepigistus_varvi4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"kaepigistus_varvi4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"kaepigistus_varvi5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"kaepigistus_varvi5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"kaepigistus_varvi5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"kaepigistus_varvi5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"sormed_risti", "1F91E"},
    {"sormed_risti_varvi1", "1F91E-1F3FB"},
    {"sormed_risti_varvi2", "1F91E-1F3FC"},
    {"sormed_risti_varvi3", "1F91E-1F3FD"},
    {"sormed_risti_varvi4", "1F91E-1F3FE"},
    {"sormed_risti_varvi5", "1F91E-1F3FF"},
    {"armastust_valjendav_zest", "1F91F"},
    {"armastust_valjendav_zest_varvi1", "1F91F-1F3FB"},
    {"armastust_valjendav_zest_varvi2", "1F91F-1F3FC"},
    {"armastust_valjendav_zest_varvi3", "1F91F-1F3FD"},
    {"armastust_valjendav_zest_varvi4", "1F91F-1F3FE"},
    {"armastust_valjendav_zest_varvi5", "1F91F-1F3FF"},
    {"kauboimutsiga_nagu", "1F920"},
    {"klouninagu", "1F921"},
    {"iiveldust_tundev_nagu", "1F922"},
    {"naerust_koveras", "1F923"},
    {"ilastav_nagu", "1F924"},
    {"valetav_nagu", "1F925"},
    {"meeleheide", "1F926"},
    {"meeleheide_varvi1", "1F926-1F3FB"},
    {"meeleheide_varvi2", "1F926-1F3FC"},
    {"meeleheide_varvi3", "1F926-1F3FD"},
    {"meeleheide_varvi4", "1F926-1F3FE"},
    {"meeleheide_varvi5", "1F926-1F3FF"},
    {"aevastav_nagu", "1F927"},
    {"kergitatud_kulmudega_nagu", "1F928"},
    {"staariga_silmitsi", "1F929"},
    {"poorase_ilmega_nagu", "1F92A"},
    {"sorme_suul_hoidev_nagu", "1F92B"},
    {"suul_olevate_sumbolitega_nagu", "1F92C"},
    {"katt_suul_hoidev_nagu", "1F92D"},
    {"oksendav_nagu", "1F92E"},
    {"plahvatav_pea", "1F92F"},
    {"rase_naine", "1F930"},
    {"rase_naine_varvi1", "1F930-1F3FB"},
    {"rase_naine_varvi2", "1F930-1F3FC"},
    {"rase_naine_varvi3", "1F930-1F3FD"},
    {"rase_naine_varvi4", "1F930-1F3FE"},
    {"rase_naine_varvi5", "1F930-1F3FF"},
    {"rinnaga_toitmine", "1F931"},
    {"rinnaga_toitmine_varvi1", "1F931-1F3FB"},
    {"rinnaga_toitmine_varvi2", "1F931-1F3FC"},
    {"rinnaga_toitmine_varvi3", "1F931-1F3FD"},
    {"rinnaga_toitmine_varvi4", "1F931-1F3FE"},
    {"rinnaga_toitmine_varvi5", "1F931-1F3FF"},
    {"peopesad_uleval_koos", "1F932"},
    {"peopesad_uleval_koos_varvi1", "1F932-1F3FB"},
    {"peopesad_uleval_koos_varvi2", "1F932-1F3FC"},
    {"peopesad_uleval_koos_varvi3", "1F932-1F3FD"},
    {"peopesad_uleval_koos_varvi4", "1F932-1F3FE"},
    {"peopesad_uleval_koos_varvi5", "1F932-1F3FF"},
    {"selfi", "1F933"},
    {"selfi_varvi1", "1F933-1F3FB"},
    {"selfi_varvi2", "1F933-1F3FC"},
    {"selfi_varvi3", "1F933-1F3FD"},
    {"selfi_varvi4", "1F933-1F3FE"},
    {"selfi_varvi5", "1F933-1F3FF"},
    {"prints", "1F934"},
    {"prints_varvi1", "1F934-1F3FB"},
    {"prints_varvi2", "1F934-1F3FC"},
    {"prints_varvi3", "1F934-1F3FD"},
    {"prints_varvi4", "1F934-1F3FE"},
    {"prints_varvi5", "1F934-1F3FF"},
    {"inimene_smokingus", "1F935"},
    {"inimene_smokingus_varvi1", "1F935-1F3FB"},
    {"inimene_smokingus_varvi2", "1F935-1F3FC"},
    {"inimene_smokingus_varvi3", "1F935-1F3FD"},
    {"inimene_smokingus_varvi4", "1F935-1F3FE"},
    {"inimene_smokingus_varvi5", "1F935-1F3FF"},
    {"jouluvana_abikaasa", "1F936"},
    {"jouluvana_abikaasa_varvi1", "1F936-1F3FB"},
    {"jouluvana_abikaasa_varvi2", "1F936-1F3FC"},
    {"jouluvana_abikaasa_varvi3", "1F936-1F3FD"},
    {"jouluvana_abikaasa_varvi4", "1F936-1F3FE"},
    {"jouluvana_abikaasa_varvi5", "1F936-1F3FF"},
    {"olakehitus", "1F937"},
    {"olakehitus_varvi1", "1F937-1F3FB"},
    {"olakehitus_varvi2", "1F937-1F3FC"},
    {"olakehitus_varvi3", "1F937-1F3FD"},
    {"olakehitus_varvi4", "1F937-1F3FE"},
    {"olakehitus_varvi5", "1F937-1F3FF"},
    {"hundiratast_viskav_inimene", "1F938"},
    {"hundiratast_viskav_inimene_varvi1", "1F938-1F3FB"},
    {"hundiratast_viskav_inimene_varvi2", "1F938-1F3FC"},
    {"hundiratast_viskav_inimene_varvi3", "1F938-1F3FD"},
    {"hundiratast_viskav_inimene_varvi4", "1F938-1F3FE"},
    {"hundiratast_viskav_inimene_varvi5", "1F938-1F3FF"},
    {"zongloor", "1F939"},
    {"zongloor_varvi1", "1F939-1F3FB"},
    {"zongloor_varvi2", "1F939-1F3FC"},
    {"zongloor_varvi3", "1F939-1F3FD"},
    {"zongloor_varvi4", "1F939-1F3FE"},
    {"zongloor_varvi5", "1F939-1F3FF"},
    {"vehkleja", "1F93A"},
    {"maadlejad", "1F93C"},
    {"maadlejad_varvi1", "1F93C-1F3FB"},
    {"maadlejad_varvi2", "1F93C-1F3FC"},
    {"maadlejad_varvi3", "1F93C-1F3FD"},
    {"maadlejad_varvi4", "1F93C-1F3FE"},
    {"maadlejad_varvi5", "1F93C-1F3FF"},
    {"maadlejad_varvi1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"maadlejad_varvi1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"maadlejad_varvi1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"maadlejad_varvi1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"maadlejad_varvi2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"maadlejad_varvi2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"maadlejad_varvi2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"maadlejad_varvi2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"maadlejad_varvi3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"maadlejad_varvi3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"maadlejad_varvi3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"maadlejad_varvi3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"maadlejad_varvi4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"maadlejad_varvi4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"maadlejad_varvi4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"maadlejad_varvi4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"maadlejad_varvi5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"maadlejad_varvi5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"maadlejad_varvi5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"maadlejad_varvi5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"veepalli_mangiv_inimene", "1F93D"},
    {"veepalli_mangiv_inimene_varvi1", "1F93D-1F3FB"},
    {"veepalli_mangiv_inimene_varvi2", "1F93D-1F3FC"},
    {"veepalli_mangiv_inimene_varvi3", "1F93D-1F3FD"},
    {"veepalli_mangiv_inimene_varvi4", "1F93D-1F3FE"},
    {"veepalli_mangiv_inimene_varvi5", "1F93D-1F3FF"},
    {"kasipalli_mangiv_inimene", "1F93E"},
    {"kasipalli_mangiv_inimene_varvi1", "1F93E-1F3FB"},
    {"kasipalli_mangiv_inimene_varvi2", "1F93E-1F3FC"},
    {"kasipalli_mangiv_inimene_varvi3", "1F93E-1F3FD"},
    {"kasipalli_mangiv_inimene_varvi4", "1F93E-1F3FE"},
    {"kasipalli_mangiv_inimene_varvi5", "1F93E-1F3FF"},
    {"sukeldumismask", "1F93F"},
    {"narbunud_lill", "1F940"},
    {"trumm", "1F941"},
    {"kolisevad_klaasid", "1F942"},
    {"joogiklaas", "1F943"},
    {"lusikas", "1F944"},
    {"varavavork", "1F945"},
    {"kuldmedal", "1F947"},
    {"hobemedal", "1F948"},
    {"pronksmedal", "1F949"},
    {"poksikinnas", "1F94A"},
    {"voitluskunstide_riietus", "1F94B"},
    {"jaakeegli_kivi", "1F94C"},
    {"kahvpall", "1F94D"},
    {"pehmepall", "1F94E"},
    {"visketaldrik", "1F94F"},
    {"sarvesai", "1F950"},
    {"avokaado", "1F951"},
    {"kurk", "1F952"},
    {"peekon", "1F953"},
    {"kartul", "1F954"},
    {"porgand", "1F955"},
    {"baguette", "1F956"},
    {"roheline_salat", "1F957"},
    {"panniroog", "1F958"},
    {"taidetud_pita", "1F959"},
    {"muna", "1F95A"},
    {"klaas_piima", "1F95B"},
    {"maapahklid", "1F95C"},
    {"kiivi", "1F95D"},
    {"pannkoogid", "1F95E"},
    {"pelmeen", "1F95F"},
    {"onnekupsis", "1F960"},
    {"kaasamuugikarp", "1F961"},
    {"soogipulgad", "1F962"},
    {"lusikaga_kauss", "1F963"},
    {"korrega_tops", "1F964"},
    {"kookospahkel", "1F965"},
    {"brokoli", "1F966"},
    {"pirukas", "1F967"},
    {"soolakringel", "1F968"},
    {"lihaloik", "1F969"},
    {"voileib", "1F96A"},
    {"toidukonserv", "1F96B"},
    {"lehtkoogivili", "1F96C"},
    {"mango", "1F96D"},
    {"kuukook", "1F96E"},
    {"rongassai", "1F96F"},
    {"naeratav_sudametega_nagu", "1F970"},
    {"haigutav_nagu", "1F971"},
    {"naeratav_pisaraga_nagu", "1F972"},
    {"pidutsev_nagu", "1F973"},
    {"uimane_nagu", "1F974"},
    {"higistav_nagu", "1F975"},
    {"kulmunud_nagu", "1F976"},
    {"ninja", "1F977"},
    {"ninja_varvi1", "1F977-1F3FB"},
    {"ninja_varvi2", "1F977-1F3FC"},
    {"ninja_varvi3", "1F977-1F3FD"},
    {"ninja_varvi4", "1F977-1F3FE"},
    {"ninja_varvi5", "1F977-1F3FF"},
    {"maskeeritud_nagu", "1F978"},
    {"pisaraid_tagasi_hoidev_nagu", "1F979"},
    {"anuv_nagu", "1F97A"},
    {"sari", "1F97B"},
    {"laborikittel", "1F97C"},
    {"kaitseprillid", "1F97D"},
    {"matkasaabas", "1F97E"},
    {"madal_king", "1F97F"},
    {"krabi", "1F980"},
    {"lovi_nagu", "1F981"},
    {"suur_skorpion", "1F982"},
    {"kalkun", "1F983"},
    {"ukssarvik", "1F984"},
    {"kotkas", "1F985"},
    {"part", "1F986"},
    {"nahkhiir", "1F987"},
    {"hai", "1F988"},
    {"ookull", "1F989"},
    {"rebane", "1F98A"},
    {"liblikas", "1F98B"},
    {"hirv", "1F98C"},
    {"gorilla", "1F98D"},
    {"sisalik", "1F98E"},
    {"ninasarvik", "1F98F"},
    {"krevett", "1F990"},
    {"kalmaar", "1F991"},
    {"kaelkirjak", "1F992"},
    {"sebra", "1F993"},
    {"siil", "1F994"},
    {"sauropood", "1F995"},
    {"turannosaurus", "1F996"},
    {"kilk", "1F997"},
    {"kanguru", "1F998"},
    {"laama", "1F999"},
    {"paabulind", "1F99A"},
    {"joehobu", "1F99B"},
    {"papagoi", "1F99C"},
    {"pesukaru", "1F99D"},
    {"homaar", "1F99E"},
    {"saask", "1F99F"},
    {"mikroob", "1F9A0"},
    {"mager", "1F9A1"},
    {"luik", "1F9A2"},
    {"mammut", "1F9A3"},
    {"dodo", "1F9A4"},
    {"laiskloom", "1F9A5"},
    {"saarmas", "1F9A6"},
    {"orangutan", "1F9A7"},
    {"skunk", "1F9A8"},
    {"flamingo", "1F9A9"},
    {"auster", "1F9AA"},
    {"kobras", "1F9AB"},
    {"piison", "1F9AC"},
    {"huljes", "1F9AD"},
    {"juhtkoer", "1F9AE"},
    {"valge_kepp", "1F9AF"},
    {"punased_juuksed", "1F9B0"},
    {"lokkis_juuksed", "1F9B1"},
    {"kiilaspea", "1F9B2"},
    {"heledad_juuksed", "1F9B3"},
    {"kont", "1F9B4"},
    {"jalg", "1F9B5"},
    {"jalg_varvi1", "1F9B5-1F3FB"},
    {"jalg_varvi2", "1F9B5-1F3FC"},
    {"jalg_varvi3", "1F9B5-1F3FD"},
    {"jalg_varvi4", "1F9B5-1F3FE"},
    {"jalg_varvi5", "1F9B5-1F3FF"},
    {"jalalaba", "1F9B6"},
    {"jalalaba_varvi1", "1F9B6-1F3FB"},
    {"jalalaba_varvi2", "1F9B6-1F3FC"},
    {"jalalaba_varvi3", "1F9B6-1F3FD"},
    {"jalalaba_varvi4", "1F9B6-1F3FE"},
    {"jalalaba_varvi5", "1F9B6-1F3FF"},
    {"hammas", "1F9B7"},
    {"superkangelane", "1F9B8"},
    {"superkangelane_varvi1", "1F9B8-1F3FB"},
    {"superkangelane_varvi2", "1F9B8-1F3FC"},
    {"superkangelane_varvi3", "1F9B8-1F3FD"},
    {"superkangelane_varvi4", "1F9B8-1F3FE"},
    {"superkangelane_varvi5", "1F9B8-1F3FF"},
    {"superkurikael", "1F9B9"},
    {"superkurikael_varvi1", "1F9B9-1F3FB"},
    {"superkurikael_varvi2", "1F9B9-1F3FC"},
    {"superkurikael_varvi3", "1F9B9-1F3FD"},
    {"superkurikael_varvi4", "1F9B9-1F3FE"},
    {"superkurikael_varvi5", "1F9B9-1F3FF"},
    {"ohutusvest", "1F9BA"},
    {"kuuldeaparaadiga_korv", "1F9BB"},
    {"kuuldeaparaadiga_korv_varvi1", "1F9BB-1F3FB"},
    {"kuuldeaparaadiga_korv_varvi2", "1F9BB-1F3FC"},
    {"kuuldeaparaadiga_korv_varvi3", "1F9BB-1F3FD"},
    {"kuuldeaparaadiga_korv_varvi4", "1F9BB-1F3FE"},
    {"kuuldeaparaadiga_korv_varvi5", "1F9BB-1F3FF"},
    {"elektriratastool", "1F9BC"},
    {"manuaalratastool", "1F9BD"},
    {"mehaaniline_kasi", "1F9BE"},
    {"mehaaniline_jalg", "1F9BF"},
    {"juustutukk", "1F9C0"},
    {"muffin", "1F9C1"},
    {"sool", "1F9C2"},
    {"joogipakk", "1F9C3"},
    {"kuuslauk", "1F9C4"},
    {"sibul", "1F9C5"},
    {"falafel", "1F9C6"},
    {"vahvel", "1F9C7"},
    {"voi", "1F9C8"},
    {"mate", "1F9C9"},
    {"jaakuubik", "1F9CA"},
    {"parlitee", "1F9CB"},
    {"troll", "1F9CC"},
    {"seisev_inimene", "1F9CD"},
    {"seisev_inimene_varvi1", "1F9CD-1F3FB"},
    {"seisev_inimene_varvi2", "1F9CD-1F3FC"},
    {"seisev_inimene_varvi3", "1F9CD-1F3FD"},
    {"seisev_inimene_varvi4", "1F9CD-1F3FE"},
    {"seisev_inimene_varvi5", "1F9CD-1F3FF"},
    {"polvitav_inimene", "1F9CE"},
    {"polvitav_inimene_varvi1", "1F9CE-1F3FB"},
    {"polvitav_inimene_varvi2", "1F9CE-1F3FC"},
    {"polvitav_inimene_varvi3", "1F9CE-1F3FD"},
    {"polvitav_inimene_varvi4", "1F9CE-1F3FE"},
    {"polvitav_inimene_varvi5", "1F9CE-1F3FF"},
    {"kurt_inimene", "1F9CF"},
    {"kurt_inimene_varvi1", "1F9CF-1F3FB"},
    {"kurt_inimene_varvi2", "1F9CF-1F3FC"},
    {"kurt_inimene_varvi3", "1F9CF-1F3FD"},
    {"kurt_inimene_varvi4", "1F9CF-1F3FE"},
    {"kurt_inimene_varvi5", "1F9CF-1F3FF"},
    {"monokliga_nagu", "1F9D0"},
    {"taiskasvanu", "1F9D1"},
    {"taiskasvanu_varvi1", "1F9D1-1F3FB"},
    {"taiskasvanu_varvi2", "1F9D1-1F3FC"},
    {"taiskasvanu_varvi3", "1F9D1-1F3FD"},
    {"taiskasvanu_varvi4", "1F9D1-1F3FE"},
    {"taiskasvanu_varvi5", "1F9D1-1F3FF"},
    {"laps", "1F9D2"},
    {"laps_varvi1", "1F9D2-1F3FB"},
    {"laps_varvi2", "1F9D2-1F3FC"},
    {"laps_varvi3", "1F9D2-1F3FD"},
    {"laps_varvi4", "1F9D2-1F3FE"},
    {"laps_varvi5", "1F9D2-1F3FF"},
    {"eakas", "1F9D3"},
    {"eakas_varvi1", "1F9D3-1F3FB"},
    {"eakas_varvi2", "1F9D3-1F3FC"},
    {"eakas_varvi3", "1F9D3-1F3FD"},
    {"eakas_varvi4", "1F9D3-1F3FE"},
    {"eakas_varvi5", "1F9D3-1F3FF"},
    {"mees_habemega", "1F9D4"},
    {"mees_habemega_varvi1", "1F9D4-1F3FB"},
    {"mees_habemega_varvi2", "1F9D4-1F3FC"},
    {"mees_habemega_varvi3", "1F9D4-1F3FD"},
    {"mees_habemega_varvi4", "1F9D4-1F3FE"},
    {"mees_habemega_varvi5", "1F9D4-1F3FF"},
    {"pearatiga_naine", "1F9D5"},
    {"pearatiga_naine_varvi1", "1F9D5-1F3FB"},
    {"pearatiga_naine_varvi2", "1F9D5-1F3FC"},
    {"pearatiga_naine_varvi3", "1F9D5-1F3FD"},
    {"pearatiga_naine_varvi4", "1F9D5-1F3FE"},
    {"pearatiga_naine_varvi5", "1F9D5-1F3FF"},
    {"inimene_leiliruumis", "1F9D6"},
    {"inimene_leiliruumis_varvi1", "1F9D6-1F3FB"},
    {"inimene_leiliruumis_varvi2", "1F9D6-1F3FC"},
    {"inimene_leiliruumis_varvi3", "1F9D6-1F3FD"},
    {"inimene_leiliruumis_varvi4", "1F9D6-1F3FE"},
    {"inimene_leiliruumis_varvi5", "1F9D6-1F3FF"},
    {"makke_roniv_inimene", "1F9D7"},
    {"makke_roniv_inimene_varvi1", "1F9D7-1F3FB"},
    {"makke_roniv_inimene_varvi2", "1F9D7-1F3FC"},
    {"makke_roniv_inimene_varvi3", "1F9D7-1F3FD"},
    {"makke_roniv_inimene_varvi4", "1F9D7-1F3FE"},
    {"makke_roniv_inimene_varvi5", "1F9D7-1F3FF"},
    {"lootosasendis_inimene", "1F9D8"},
    {"lootosasendis_inimene_varvi1", "1F9D8-1F3FB"},
    {"lootosasendis_inimene_varvi2", "1F9D8-1F3FC"},
    {"lootosasendis_inimene_varvi3", "1F9D8-1F3FD"},
    {"lootosasendis_inimene_varvi4", "1F9D8-1F3FE"},
    {"lootosasendis_inimene_varvi5", "1F9D8-1F3FF"},
    {"maag", "1F9D9"},
    {"maag_varvi1", "1F9D9-1F3FB"},
    {"maag_varvi2", "1F9D9-1F3FC"},
    {"maag_varvi3", "1F9D9-1F3FD"},
    {"maag_varvi4", "1F9D9-1F3FE"},
    {"maag_varvi5", "1F9D9-1F3FF"},
    {"haldjas", "1F9DA"},
    {"haldjas_varvi1", "1F9DA-1F3FB"},
    {"haldjas_varvi2", "1F9DA-1F3FC"},
    {"haldjas_varvi3", "1F9DA-1F3FD"},
    {"haldjas_varvi4", "1F9DA-1F3FE"},
    {"haldjas_varvi5", "1F9DA-1F3FF"},
    {"vampiir", "1F9DB"},
    {"vampiir_varvi1", "1F9DB-1F3FB"},
    {"vampiir_varvi2", "1F9DB-1F3FC"},
    {"vampiir_varvi3", "1F9DB-1F3FD"},
    {"vampiir_varvi4", "1F9DB-1F3FE"},
    {"vampiir_varvi5", "1F9DB-1F3FF"},
    {"nakk", "1F9DC"},
    {"nakk_varvi1", "1F9DC-1F3FB"},
    {"nakk_varvi2", "1F9DC-1F3FC"},
    {"nakk_varvi3", "1F9DC-1F3FD"},
    {"nakk_varvi4", "1F9DC-1F3FE"},
    {"nakk_varvi5", "1F9DC-1F3FF"},
    {"pakapikk", "1F9DD"},
    {"pakapikk_varvi1", "1F9DD-1F3FB"},
    {"pakapikk_varvi2", "1F9DD-1F3FC"},
    {"pakapikk_varvi3", "1F9DD-1F3FD"},
    {"pakapikk_varvi4", "1F9DD-1F3FE"},
    {"pakapikk_varvi5", "1F9DD-1F3FF"},
    {"dzinn", "1F9DE"},
    {"zombi", "1F9DF"},
    {"aju", "1F9E0"},
    {"oranz_suda", "1F9E1"},
    {"nokkmuts", "1F9E2"},
    {"sall", "1F9E3"},
    {"kindad", "1F9E4"},
    {"mantel", "1F9E5"},
    {"sokid", "1F9E6"},
    {"punane_umbrik", "1F9E7"},
    {"ilutulestikurakett", "1F9E8"},
    {"pusletukk", "1F9E9"},
    {"katseklaas", "1F9EA"},
    {"petri_tass", "1F9EB"},
    {"dna", "1F9EC"},
    {"kompass", "1F9ED"},
    {"arvelaud", "1F9EE"},
    {"tulekustuti", "1F9EF"},
    {"tooriistakast", "1F9F0"},
    {"telliskivi", "1F9F1"},
    {"magnet", "1F9F2"},
    {"kohver", "1F9F3"},
    {"emulsioonipudel", "1F9F4"},
    {"niit", "1F9F5"},
    {"long", "1F9F6"},
    {"haaknoel", "1F9F7"},
    {"mangukaru", "1F9F8"},
    {"luud", "1F9F9"},
    {"paberirull", "1F9FB"},
    {"seep", "1F9FC"},
    {"pesukasn", "1F9FD"},
    {"kviitung", "1F9FE"},
    {"nazari_amulett", "1F9FF"},
    {"balletikingad", "1FA70"},
    {"ujumistrikoo", "1FA71"},
    {"ujumispuksid", "1FA72"},
    {"sortsid", "1FA73"},
    {"varbavahesandaalid", "1FA74"},
    {"helesinine_suda", "1FA75"},
    {"hall_suda", "1FA76"},
    {"roosa_suda", "1FA77"},
    {"veretilk", "1FA78"},
    {"plaaster", "1FA79"},
    {"stetoskoop", "1FA7A"},
    {"rontgen", "1FA7B"},
    {"kark", "1FA7C"},
    {"jojo", "1FA80"},
    {"tuulelohe", "1FA81"},
    {"langevarjur", "1FA82"},
    {"bumerang", "1FA83"},
    {"volukepp", "1FA84"},
    {"pinata", "1FA85"},
    {"matrjoska", "1FA86"},
    {"marakas", "1FA87"},
    {"floot", "1FA88"},
    {"harf", "1FA89"},
    {"tromboon", "1FA8A"},
    {"aardelaegas", "1FA8E"},
    {"labidas", "1FA8F"},
    {"rongastega_planeet", "1FA90"},
    {"tool", "1FA91"},
    {"habemenuga", "1FA92"},
    {"kirves", "1FA93"},
    {"diya_lamp", "1FA94"},
    {"bandzo", "1FA95"},
    {"sodurikiiver", "1FA96"},
    {"akordion", "1FA97"},
    {"aafrika_trumm", "1FA98"},
    {"munt", "1FA99"},
    {"kasisaag", "1FA9A"},
    {"kruvikeeraja", "1FA9B"},
    {"redel", "1FA9C"},
    {"konks", "1FA9D"},
    {"peegel", "1FA9E"},
    {"aken", "1FA9F"},
    {"valamupump", "1FAA0"},
    {"omblusnoel", "1FAA1"},
    {"solm", "1FAA2"},
    {"amber", "1FAA3"},
    {"hiireloks", "1FAA4"},
    {"hambahari", "1FAA5"},
    {"hauakivi", "1FAA6"},
    {"plakat", "1FAA7"},
    {"kivi", "1FAA8"},
    {"diskokera", "1FAA9"},
    {"id_kaart", "1FAAA"},
    {"aku_tuhjeneb", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"volditav_lehvik", "1FAAD"},
    {"juuksehark", "1FAAE"},
    {"khanda", "1FAAF"},
    {"karbes", "1FAB0"},
    {"uss", "1FAB1"},
    {"mardikas", "1FAB2"},
    {"prussakas", "1FAB3"},
    {"potilill", "1FAB4"},
    {"puit", "1FAB5"},
    {"sulg", "1FAB6"},
    {"lootos", "1FAB7"},
    {"korall", "1FAB8"},
    {"tuhi_pesa", "1FAB9"},
    {"munadega_pesa", "1FABA"},
    {"huatsint", "1FABB"},
    {"meduus", "1FABC"},
    {"tiib", "1FABD"},
    {"raagus_puu", "1FABE"},
    {"hani", "1FABF"},
    {"suda", "1FAC0"},
    {"kopsud", "1FAC1"},
    {"kallistavad_inimesed", "1FAC2"},
    {"rase_mees", "1FAC3"},
    {"rase_mees_varvi1", "1FAC3-1F3FB"},
    {"rase_mees_varvi2", "1FAC3-1F3FC"},
    {"rase_mees_varvi3", "1FAC3-1F3FD"},
    {"rase_mees_varvi4", "1FAC3-1F3FE"},
    {"rase_mees_varvi5", "1FAC3-1F3FF"},
    {"rase_inimene", "1FAC4"},
    {"rase_inimene_varvi1", "1FAC4-1F3FB"},
    {"rase_inimene_varvi2", "1FAC4-1F3FC"},
    {"rase_inimene_varvi3", "1FAC4-1F3FD"},
    {"rase_inimene_varvi4", "1FAC4-1F3FE"},
    {"rase_inimene_varvi5", "1FAC4-1F3FF"},
    {"krooniga_inimene", "1FAC5"},
    {"krooniga_inimene_varvi1", "1FAC5-1F3FB"},
    {"krooniga_inimene_varvi2", "1FAC5-1F3FC"},
    {"krooniga_inimene_varvi3", "1FAC5-1F3FD"},
    {"krooniga_inimene_varvi4", "1FAC5-1F3FE"},
    {"krooniga_inimene_varvi5", "1FAC5-1F3FF"},
    {"sormejalg", "1FAC6"},
    {"karvane_olend", "1FAC8"},
    {"mookvaal", "1FACD"},
    {"poder", "1FACE"},
    {"eesel", "1FACF"},
    {"mustikad", "1FAD0"},
    {"paprika", "1FAD1"},
    {"oliiv", "1FAD2"},
    {"lameleib", "1FAD3"},
    {"tamale", "1FAD4"},
    {"fonduu", "1FAD5"},
    {"teekann", "1FAD6"},
    {"vedeliku_kallamine", "1FAD7"},
    {"oad", "1FAD8"},
    {"purk", "1FAD9"},
    {"ingver", "1FADA"},
    {"hernekaun", "1FADB"},
    {"juurvili", "1FADC"},
    {"plarakas", "1FADF"},
    {"sulav_nagu", "1FAE0"},
    {"au_andev_nagu", "1FAE1"},
    {"katt_suul_hoidev_parani_silmadega_nagu", "1FAE2"},
    {"kate_vahelt_piiluva_silmaga_nagu", "1FAE3"},
    {"diagonaalse_suuga_nagu", "1FAE4"},
    {"punktiirjoonega_nagu", "1FAE5"},
    {"huulde_hammustav", "1FAE6"},
    {"mullid", "1FAE7"},
    {"varisev_nagu", "1FAE8"},
    {"nagu_kottidega_silmade_all", "1FAE9"},
    {"moondunud_nagu", "1FAEA"},
    {"kakluspilv", "1FAEF"},
    {"poialt_ja_nimetissorme_risti_hoidev_kasi", "1FAF0"},
    {"poialt_ja_nimetissorme_risti_hoidev_kasi_varvi1", "1FAF0-1F3FB"},
    {"poialt_ja_nimetissorme_risti_hoidev_kasi_varvi2", "1FAF0-1F3FC"},
    {"poialt_ja_nimetissorme_risti_hoidev_kasi_varvi3", "1FAF0-1F3FD"},
    {"poialt_ja_nimetissorme_risti_hoidev_kasi_varvi4", "1FAF0-1F3FE"},
    {"poialt_ja_nimetissorme_risti_hoidev_kasi_varvi5", "1FAF0-1F3FF"},
    {"kasi_paremale", "1FAF1"},
    {"kasi_paremale_varvi1", "1FAF1-1F3FB"},
    {"kasi_paremale_varvi2", "1FAF1-1F3FC"},
    {"kasi_paremale_varvi3", "1FAF1-1F3FD"},
    {"kasi_paremale_varvi4", "1FAF1-1F3FE"},
    {"kasi_paremale_varvi5", "1FAF1-1F3FF"},
    {"kasi_vasakule", "1FAF2"},
    {"kasi_vasakule_varvi1", "1FAF2-1F3FB"},
    {"kasi_vasakule_varvi2", "1FAF2-1F3FC"},
    {"kasi_vasakule_varvi3", "1FAF2-1F3FD"},
    {"kasi_vasakule_varvi4", "1FAF2-1F3FE"},
    {"kasi_vasakule_varvi5", "1FAF2-1F3FF"},
    {"kasi_peopesaga_allapoole", "1FAF3"},
    {"kasi_peopesaga_allapoole_varvi1", "1FAF3-1F3FB"},
    {"kasi_peopesaga_allapoole_varvi2", "1FAF3-1F3FC"},
    {"kasi_peopesaga_allapoole_varvi3", "1FAF3-1F3FD"},
    {"kasi_peopesaga_allapoole_varvi4", "1FAF3-1F3FE"},
    {"kasi_peopesaga_allapoole_varvi5", "1FAF3-1F3FF"},
    {"kasi_peopesaga_ulespoole", "1FAF4"},
    {"kasi_peopesaga_ulespoole_varvi1", "1FAF4-1F3FB"},
    {"kasi_peopesaga_ulespoole_varvi2", "1FAF4-1F3FC"},
    {"kasi_peopesaga_ulespoole_varvi3", "1FAF4-1F3FD"},
    {"kasi_peopesaga_ulespoole_varvi4", "1FAF4-1F3FE"},
    {"kasi_peopesaga_ulespoole_varvi5", "1FAF4-1F3FF"},
    {"vaataja_poole_osutav_nimetissorm", "1FAF5"},
    {"vaataja_poole_osutav_nimetissorm_varvi1", "1FAF5-1F3FB"},
    {"vaataja_poole_osutav_nimetissorm_varvi2", "1FAF5-1F3FC"},
    {"vaataja_poole_osutav_nimetissorm_varvi3", "1FAF5-1F3FD"},
    {"vaataja_poole_osutav_nimetissorm_varvi4", "1FAF5-1F3FE"},
    {"vaataja_poole_osutav_nimetissorm_varvi5", "1FAF5-1F3FF"},
    {"sudant_moodustavad_kaed", "1FAF6"},
    {"sudant_moodustavad_kaed_varvi1", "1FAF6-1F3FB"},
    {"sudant_moodustavad_kaed_varvi2", "1FAF6-1F3FC"},
    {"sudant_moodustavad_kaed_varvi3", "1FAF6-1F3FD"},
    {"sudant_moodustavad_kaed_varvi4", "1FAF6-1F3FE"},
    {"sudant_moodustavad_kaed_varvi5", "1FAF6-1F3FF"},
    {"vasakule_lukkav_kasi", "1FAF7"},
    {"vasakule_lukkav_kasi_varvi1", "1FAF7-1F3FB"},
    {"vasakule_lukkav_kasi_varvi2", "1FAF7-1F3FC"},
    {"vasakule_lukkav_kasi_varvi3", "1FAF7-1F3FD"},
    {"vasakule_lukkav_kasi_varvi4", "1FAF7-1F3FE"},
    {"vasakule_lukkav_kasi_varvi5", "1FAF7-1F3FF"},
    {"paremale_lukkav_kasi", "1FAF8"},
    {"paremale_lukkav_kasi_varvi1", "1FAF8-1F3FB"},
    {"paremale_lukkav_kasi_varvi2", "1FAF8-1F3FC"},
    {"paremale_lukkav_kasi_varvi3", "1FAF8-1F3FD"},
    {"paremale_lukkav_kasi_varvi4", "1FAF8-1F3FE"},
    {"paremale_lukkav_kasi_varvi5", "1FAF8-1F3FF"},
    {"klahv_numbrimark", "0023-FE0F-20E3"},
    {"klahv_tarn", "002A-FE0F-20E3"},
    {"klahv_0", "0030-FE0F-20E3"},
    {"klahv_1", "0031-FE0F-20E3"},
    {"klahv_2", "0032-FE0F-20E3"},
    {"klahv_3", "0033-FE0F-20E3"},
    {"klahv_4", "0034-FE0F-20E3"},
    {"klahv_5", "0035-FE0F-20E3"},
    {"klahv_6", "0036-FE0F-20E3"},
    {"klahv_7", "0037-FE0F-20E3"},
    {"klahv_8", "0038-FE0F-20E3"},
    {"klahv_9", "0039-FE0F-20E3"},
    {"lipp_ascensioni_saar", "1F1E6-1F1E8"},
    {"lipp_andorra", "1F1E6-1F1E9"},
    {"lipp_araabia_uhendemiraadid", "1F1E6-1F1EA"},
    {"lipp_afganistan", "1F1E6-1F1EB"},
    {"lipp_antigua_ja_barbuda", "1F1E6-1F1EC"},
    {"lipp_anguilla", "1F1E6-1F1EE"},
    {"lipp_albaania", "1F1E6-1F1F1"},
    {"lipp_armeenia", "1F1E6-1F1F2"},
    {"lipp_angola", "1F1E6-1F1F4"},
    {"lipp_antarktika", "1F1E6-1F1F6"},
    {"lipp_argentina", "1F1E6-1F1F7"},
    {"lipp_ameerika_samoa", "1F1E6-1F1F8"},
    {"lipp_austria", "1F1E6-1F1F9"},
    {"lipp_austraalia", "1F1E6-1F1FA"},
    {"lipp_aruba", "1F1E6-1F1FC"},
    {"lipp_ahvenamaa", "1F1E6-1F1FD"},
    {"lipp_aserbaidzaan", "1F1E6-1F1FF"},
    {"lipp_bosnia_ja_hertsegoviina", "1F1E7-1F1E6"},
    {"lipp_barbados", "1F1E7-1F1E7"},
    {"lipp_bangladesh", "1F1E7-1F1E9"},
    {"lipp_belgia", "1F1E7-1F1EA"},
    {"lipp_burkina_faso", "1F1E7-1F1EB"},
    {"lipp_bulgaaria", "1F1E7-1F1EC"},
    {"lipp_bahrein", "1F1E7-1F1ED"},
    {"lipp_burundi", "1F1E7-1F1EE"},
    {"lipp_benin", "1F1E7-1F1EF"},
    {"lipp_saint_barthelemy", "1F1E7-1F1F1"},
    {"lipp_bermuda", "1F1E7-1F1F2"},
    {"lipp_brunei", "1F1E7-1F1F3"},
    {"lipp_boliivia", "1F1E7-1F1F4"},
    {"lipp_kariibi_madalmaad", "1F1E7-1F1F6"},
    {"lipp_brasiilia", "1F1E7-1F1F7"},
    {"lipp_bahama", "1F1E7-1F1F8"},
    {"lipp_bhutan", "1F1E7-1F1F9"},
    {"lipp_bouvet_saar", "1F1E7-1F1FB"},
    {"lipp_botswana", "1F1E7-1F1FC"},
    {"lipp_valgevene", "1F1E7-1F1FE"},
    {"lipp_belize", "1F1E7-1F1FF"},
    {"lipp_kanada", "1F1E8-1F1E6"},
    {"lipp_kookossaared", "1F1E8-1F1E8"},
    {"lipp_kongo_dv", "1F1E8-1F1E9"},
    {"lipp_kesk_aafrika_vabariik", "1F1E8-1F1EB"},
    {"lipp_kongo_vabariik", "1F1E8-1F1EC"},
    {"lipp_sveits", "1F1E8-1F1ED"},
    {"lipp_elevandiluurannik", "1F1E8-1F1EE"},
    {"lipp_cooki_saared", "1F1E8-1F1F0"},
    {"lipp_tsiili", "1F1E8-1F1F1"},
    {"lipp_kamerun", "1F1E8-1F1F2"},
    {"lipp_hiina", "1F1E8-1F1F3"},
    {"lipp_colombia", "1F1E8-1F1F4"},
    {"lipp_clippertoni_saar", "1F1E8-1F1F5"},
    {"lipp_sark", "1F1E8-1F1F6"},
    {"lipp_costa_rica", "1F1E8-1F1F7"},
    {"lipp_kuuba", "1F1E8-1F1FA"},
    {"lipp_roheneemesaared", "1F1E8-1F1FB"},
    {"lipp_curacao", "1F1E8-1F1FC"},
    {"lipp_joulusaar", "1F1E8-1F1FD"},
    {"lipp_kupros", "1F1E8-1F1FE"},
    {"lipp_tsehhi", "1F1E8-1F1FF"},
    {"lipp_saksamaa", "1F1E9-1F1EA"},
    {"lipp_diego_garcia", "1F1E9-1F1EC"},
    {"lipp_djibouti", "1F1E9-1F1EF"},
    {"lipp_taani", "1F1E9-1F1F0"},
    {"lipp_dominica", "1F1E9-1F1F2"},
    {"lipp_dominikaani_vabariik", "1F1E9-1F1F4"},
    {"lipp_alzeeria", "1F1E9-1F1FF"},
    {"lipp_ceuta_ja_melilla", "1F1EA-1F1E6"},
    {"lipp_ecuador", "1F1EA-1F1E8"},
    {"lipp_eesti", "1F1EA-1F1EA"},
    {"lipp_egiptus", "1F1EA-1F1EC"},
    {"lipp_laane_sahara", "1F1EA-1F1ED"},
    {"lipp_eritrea", "1F1EA-1F1F7"},
    {"lipp_hispaania", "1F1EA-1F1F8"},
    {"lipp_etioopia", "1F1EA-1F1F9"},
    {"lipp_euroopa_liit", "1F1EA-1F1FA"},
    {"lipp_soome", "1F1EB-1F1EE"},
    {"lipp_fidzi", "1F1EB-1F1EF"},
    {"lipp_falklandi_saared", "1F1EB-1F1F0"},
    {"lipp_mikroneesia", "1F1EB-1F1F2"},
    {"lipp_faari_saared", "1F1EB-1F1F4"},
    {"lipp_prantsusmaa", "1F1EB-1F1F7"},
    {"lipp_gabon", "1F1EC-1F1E6"},
    {"lipp_uhendkuningriik", "1F1EC-1F1E7"},
    {"lipp_grenada", "1F1EC-1F1E9"},
    {"lipp_gruusia", "1F1EC-1F1EA"},
    {"lipp_prantsuse_guajaana", "1F1EC-1F1EB"},
    {"lipp_guernsey", "1F1EC-1F1EC"},
    {"lipp_ghana", "1F1EC-1F1ED"},
    {"lipp_gibraltar", "1F1EC-1F1EE"},
    {"lipp_groonimaa", "1F1EC-1F1F1"},
    {"lipp_gambia", "1F1EC-1F1F2"},
    {"lipp_guinea", "1F1EC-1F1F3"},
    {"lipp_guadeloupe", "1F1EC-1F1F5"},
    {"lipp_ekvatoriaal_guinea", "1F1EC-1F1F6"},
    {"lipp_kreeka", "1F1EC-1F1F7"},
    {"lipp_louna_georgia_ja_louna_sandwichi_saared", "1F1EC-1F1F8"},
    {"lipp_guatemala", "1F1EC-1F1F9"},
    {"lipp_guam", "1F1EC-1F1FA"},
    {"lipp_guinea_bissau", "1F1EC-1F1FC"},
    {"lipp_guyana", "1F1EC-1F1FE"},
    {"lipp_hongkongi_erihalduspiirkond", "1F1ED-1F1F0"},
    {"lipp_heardi_ja_mcdonaldi_saared", "1F1ED-1F1F2"},
    {"lipp_honduras", "1F1ED-1F1F3"},
    {"lipp_horvaatia", "1F1ED-1F1F7"},
    {"lipp_haiti", "1F1ED-1F1F9"},
    {"lipp_ungari", "1F1ED-1F1FA"},
    {"lipp_kanaari_saared", "1F1EE-1F1E8"},
    {"lipp_indoneesia", "1F1EE-1F1E9"},
    {"lipp_iirimaa", "1F1EE-1F1EA"},
    {"lipp_iisrael", "1F1EE-1F1F1"},
    {"lipp_mani_saar", "1F1EE-1F1F2"},
    {"lipp_india", "1F1EE-1F1F3"},
    {"lipp_briti_india_ookeani_ala", "1F1EE-1F1F4"},
    {"lipp_iraak", "1F1EE-1F1F6"},
    {"lipp_iraan", "1F1EE-1F1F7"},
    {"lipp_island", "1F1EE-1F1F8"},
    {"lipp_itaalia", "1F1EE-1F1F9"},
    {"lipp_jersey", "1F1EF-1F1EA"},
    {"lipp_jamaica", "1F1EF-1F1F2"},
    {"lipp_jordaania", "1F1EF-1F1F4"},
    {"lipp_jaapan", "1F1EF-1F1F5"},
    {"lipp_keenia", "1F1F0-1F1EA"},
    {"lipp_korgozstan", "1F1F0-1F1EC"},
    {"lipp_kambodza", "1F1F0-1F1ED"},
    {"lipp_kiribati", "1F1F0-1F1EE"},
    {"lipp_komoorid", "1F1F0-1F1F2"},
    {"lipp_saint_kitts_ja_nevis", "1F1F0-1F1F3"},
    {"lipp_pohja_korea", "1F1F0-1F1F5"},
    {"lipp_louna_korea", "1F1F0-1F1F7"},
    {"lipp_kuveit", "1F1F0-1F1FC"},
    {"lipp_kaimanisaared", "1F1F0-1F1FE"},
    {"lipp_kasahstan", "1F1F0-1F1FF"},
    {"lipp_laos", "1F1F1-1F1E6"},
    {"lipp_liibanon", "1F1F1-1F1E7"},
    {"lipp_saint_lucia", "1F1F1-1F1E8"},
    {"lipp_liechtenstein", "1F1F1-1F1EE"},
    {"lipp_sri_lanka", "1F1F1-1F1F0"},
    {"lipp_libeeria", "1F1F1-1F1F7"},
    {"lipp_lesotho", "1F1F1-1F1F8"},
    {"lipp_leedu", "1F1F1-1F1F9"},
    {"lipp_luksemburg", "1F1F1-1F1FA"},
    {"lipp_lati", "1F1F1-1F1FB"},
    {"lipp_liibua", "1F1F1-1F1FE"},
    {"lipp_maroko", "1F1F2-1F1E6"},
    {"lipp_monaco", "1F1F2-1F1E8"},
    {"lipp_moldova", "1F1F2-1F1E9"},
    {"lipp_montenegro", "1F1F2-1F1EA"},
    {"lipp_saint_martin", "1F1F2-1F1EB"},
    {"lipp_madagaskar", "1F1F2-1F1EC"},
    {"lipp_marshalli_saared", "1F1F2-1F1ED"},
    {"lipp_pohja_makedoonia", "1F1F2-1F1F0"},
    {"lipp_mali", "1F1F2-1F1F1"},
    {"lipp_myanmar_birma", "1F1F2-1F1F2"},
    {"lipp_mongoolia", "1F1F2-1F1F3"},
    {"lipp_macau_erihalduspiirkond", "1F1F2-1F1F4"},
    {"lipp_pohja_mariaanid", "1F1F2-1F1F5"},
    {"lipp_martinique", "1F1F2-1F1F6"},
    {"lipp_mauritaania", "1F1F2-1F1F7"},
    {"lipp_montserrat", "1F1F2-1F1F8"},
    {"lipp_malta", "1F1F2-1F1F9"},
    {"lipp_mauritius", "1F1F2-1F1FA"},
    {"lipp_maldiivid", "1F1F2-1F1FB"},
    {"lipp_malawi", "1F1F2-1F1FC"},
    {"lipp_mehhiko", "1F1F2-1F1FD"},
    {"lipp_malaisia", "1F1F2-1F1FE"},
    {"lipp_mosambiik", "1F1F2-1F1FF"},
    {"lipp_namiibia", "1F1F3-1F1E6"},
    {"lipp_uus_kaledoonia", "1F1F3-1F1E8"},
    {"lipp_niger", "1F1F3-1F1EA"},
    {"lipp_norfolk", "1F1F3-1F1EB"},
    {"lipp_nigeeria", "1F1F3-1F1EC"},
    {"lipp_nicaragua", "1F1F3-1F1EE"},
    {"lipp_holland", "1F1F3-1F1F1"},
    {"lipp_norra", "1F1F3-1F1F4"},
    {"lipp_nepal", "1F1F3-1F1F5"},
    {"lipp_nauru", "1F1F3-1F1F7"},
    {"lipp_niue", "1F1F3-1F1FA"},
    {"lipp_uus_meremaa", "1F1F3-1F1FF"},
    {"lipp_omaan", "1F1F4-1F1F2"},
    {"lipp_panama", "1F1F5-1F1E6"},
    {"lipp_peruu", "1F1F5-1F1EA"},
    {"lipp_prantsuse_poluneesia", "1F1F5-1F1EB"},
    {"lipp_paapua_uus_guinea", "1F1F5-1F1EC"},
    {"lipp_filipiinid", "1F1F5-1F1ED"},
    {"lipp_pakistan", "1F1F5-1F1F0"},
    {"lipp_poola", "1F1F5-1F1F1"},
    {"lipp_saint_pierre_ja_miquelon", "1F1F5-1F1F2"},
    {"lipp_pitcairni_saared", "1F1F5-1F1F3"},
    {"lipp_puerto_rico", "1F1F5-1F1F7"},
    {"lipp_palestiina_alad", "1F1F5-1F1F8"},
    {"lipp_portugal", "1F1F5-1F1F9"},
    {"lipp_belau", "1F1F5-1F1FC"},
    {"lipp_paraguay", "1F1F5-1F1FE"},
    {"lipp_katar", "1F1F6-1F1E6"},
    {"lipp_reunion", "1F1F7-1F1EA"},
    {"lipp_rumeenia", "1F1F7-1F1F4"},
    {"lipp_serbia", "1F1F7-1F1F8"},
    {"lipp_venemaa", "1F1F7-1F1FA"},
    {"lipp_rwanda", "1F1F7-1F1FC"},
    {"lipp_saudi_araabia", "1F1F8-1F1E6"},
    {"lipp_saalomoni_saared", "1F1F8-1F1E7"},
    {"lipp_seisellid", "1F1F8-1F1E8"},
    {"lipp_sudaan", "1F1F8-1F1E9"},
    {"lipp_rootsi", "1F1F8-1F1EA"},
    {"lipp_singapur", "1F1F8-1F1EC"},
    {"lipp_saint_helena", "1F1F8-1F1ED"},
    {"lipp_sloveenia", "1F1F8-1F1EE"},
    {"lipp_svalbard_ja_jan_mayen", "1F1F8-1F1EF"},
    {"lipp_slovakkia", "1F1F8-1F1F0"},
    {"lipp_sierra_leone", "1F1F8-1F1F1"},
    {"lipp_san_marino", "1F1F8-1F1F2"},
    {"lipp_senegal", "1F1F8-1F1F3"},
    {"lipp_somaalia", "1F1F8-1F1F4"},
    {"lipp_suriname", "1F1F8-1F1F7"},
    {"lipp_louna_sudaan", "1F1F8-1F1F8"},
    {"lipp_sao_tome_ja_principe", "1F1F8-1F1F9"},
    {"lipp_el_salvador", "1F1F8-1F1FB"},
    {"lipp_sint_maarten", "1F1F8-1F1FD"},
    {"lipp_suuria", "1F1F8-1F1FE"},
    {"lipp_svaasimaa", "1F1F8-1F1FF"},
    {"lipp_tristan_da_cunha", "1F1F9-1F1E6"},
    {"lipp_turks_ja_caicos", "1F1F9-1F1E8"},
    {"lipp_tsaad", "1F1F9-1F1E9"},
    {"lipp_prantsuse_lounaalad", "1F1F9-1F1EB"},
    {"lipp_togo", "1F1F9-1F1EC"},
    {"lipp_tai", "1F1F9-1F1ED"},
    {"lipp_tadzikistan", "1F1F9-1F1EF"},
    {"lipp_tokelau", "1F1F9-1F1F0"},
    {"lipp_ida_timor", "1F1F9-1F1F1"},
    {"lipp_turkmenistan", "1F1F9-1F1F2"},
    {"lipp_tuneesia", "1F1F9-1F1F3"},
    {"lipp_tonga", "1F1F9-1F1F4"},
    {"lipp_turgi", "1F1F9-1F1F7"},
    {"lipp_trinidad_ja_tobago", "1F1F9-1F1F9"},
    {"lipp_tuvalu", "1F1F9-1F1FB"},
    {"lipp_taiwan", "1F1F9-1F1FC"},
    {"lipp_tansaania", "1F1F9-1F1FF"},
    {"lipp_ukraina", "1F1FA-1F1E6"},
    {"lipp_uganda", "1F1FA-1F1EC"},
    {"lipp_uhendriikide_hajasaared", "1F1FA-1F1F2"},
    {"lipp_uhinenud_rahvaste_organisatsioon", "1F1FA-1F1F3"},
    {"lipp_ameerika_uhendriigid", "1F1FA-1F1F8"},
    {"lipp_uruguay", "1F1FA-1F1FE"},
    {"lipp_usbekistan", "1F1FA-1F1FF"},
    {"lipp_vatikan", "1F1FB-1F1E6"},
    {"lipp_saint_vincent_ja_grenadiinid", "1F1FB-1F1E8"},
    {"lipp_venezuela", "1F1FB-1F1EA"},
    {"lipp_briti_neitsisaared", "1F1FB-1F1EC"},
    {"lipp_usa_neitsisaared", "1F1FB-1F1EE"},
    {"lipp_vietnam", "1F1FB-1F1F3"},
    {"lipp_vanuatu", "1F1FB-1F1FA"},
    {"lipp_wallis_ja_futuna", "1F1FC-1F1EB"},
    {"lipp_samoa", "1F1FC-1F1F8"},
    {"lipp_kosovo", "1F1FD-1F1F0"},
    {"lipp_jeemen", "1F1FE-1F1EA"},
    {"lipp_mayotte", "1F1FE-1F1F9"},
    {"lipp_louna_aafrika_vabariik", "1F1FF-1F1E6"},
    {"lipp_sambia", "1F1FF-1F1F2"},
    {"lipp_zimbabwe", "1F1FF-1F1FC"},
    {"lipp_inglismaa", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"lipp_sotimaa", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"lipp_wales", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"paarike_sudamega_mees_mees", "1F468-200D-2764-FE0F-200D-1F468"},
    {"paarike_sudamega_mees_mees_varvi1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_sudamega_mees_mees_varvi1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_sudamega_mees_mees_varvi1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_sudamega_mees_mees_varvi1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_sudamega_mees_mees_varvi1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_sudamega_mees_mees_varvi2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_sudamega_mees_mees_varvi2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_sudamega_mees_mees_varvi2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_sudamega_mees_mees_varvi2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_sudamega_mees_mees_varvi2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_sudamega_mees_mees_varvi3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_sudamega_mees_mees_varvi3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_sudamega_mees_mees_varvi3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_sudamega_mees_mees_varvi3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_sudamega_mees_mees_varvi3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_sudamega_mees_mees_varvi4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_sudamega_mees_mees_varvi4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_sudamega_mees_mees_varvi4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_sudamega_mees_mees_varvi4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_sudamega_mees_mees_varvi4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_sudamega_mees_mees_varvi5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_sudamega_mees_mees_varvi5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_sudamega_mees_mees_varvi5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_sudamega_mees_mees_varvi5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_sudamega_mees_mees_varvi5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"musi_mees_mees", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"musi_mees_mees_varvi1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"musi_mees_mees_varvi1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"musi_mees_mees_varvi1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"musi_mees_mees_varvi1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"musi_mees_mees_varvi1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"musi_mees_mees_varvi2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"musi_mees_mees_varvi2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"musi_mees_mees_varvi2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"musi_mees_mees_varvi2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"musi_mees_mees_varvi2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"musi_mees_mees_varvi3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"musi_mees_mees_varvi3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"musi_mees_mees_varvi3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"musi_mees_mees_varvi3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"musi_mees_mees_varvi3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"musi_mees_mees_varvi4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"musi_mees_mees_varvi4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"musi_mees_mees_varvi4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"musi_mees_mees_varvi4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"musi_mees_mees_varvi4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"musi_mees_mees_varvi5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"musi_mees_mees_varvi5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"musi_mees_mees_varvi5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"musi_mees_mees_varvi5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"musi_mees_mees_varvi5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"perekond_mees_poiss", "1F468-200D-1F466"},
    {"perekond_mees_poiss_poiss", "1F468-200D-1F466-200D-1F466"},
    {"perekond_mees_tudruk", "1F468-200D-1F467"},
    {"perekond_mees_tudruk_poiss", "1F468-200D-1F467-200D-1F466"},
    {"perekond_mees_tudruk_tudruk", "1F468-200D-1F467-200D-1F467"},
    {"perekond_mees_mees_poiss", "1F468-200D-1F468-200D-1F466"},
    {"perekond_mees_mees_poiss_poiss", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"perekond_mees_mees_tudruk", "1F468-200D-1F468-200D-1F467"},
    {"perekond_mees_mees_tudruk_poiss", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"perekond_mees_mees_tudruk_tudruk", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"perekond_mees_naine_poiss", "1F468-200D-1F469-200D-1F466"},
    {"perekond_mees_naine_poiss_poiss", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"perekond_mees_naine_tudruk", "1F468-200D-1F469-200D-1F467"},
    {"perekond_mees_naine_tudruk_poiss", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"perekond_mees_naine_tudruk_tudruk", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"paarike_sudamega_naine_mees", "1F469-200D-2764-FE0F-200D-1F468"},
    {"paarike_sudamega_naine_mees_varvi1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_sudamega_naine_mees_varvi1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_sudamega_naine_mees_varvi1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_sudamega_naine_mees_varvi1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_sudamega_naine_mees_varvi1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_sudamega_naine_mees_varvi2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_sudamega_naine_mees_varvi2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_sudamega_naine_mees_varvi2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_sudamega_naine_mees_varvi2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_sudamega_naine_mees_varvi2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_sudamega_naine_mees_varvi3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_sudamega_naine_mees_varvi3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_sudamega_naine_mees_varvi3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_sudamega_naine_mees_varvi3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_sudamega_naine_mees_varvi3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_sudamega_naine_mees_varvi4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_sudamega_naine_mees_varvi4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_sudamega_naine_mees_varvi4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_sudamega_naine_mees_varvi4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_sudamega_naine_mees_varvi4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_sudamega_naine_mees_varvi5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_sudamega_naine_mees_varvi5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_sudamega_naine_mees_varvi5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_sudamega_naine_mees_varvi5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_sudamega_naine_mees_varvi5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_sudamega_naine_naine", "1F469-200D-2764-FE0F-200D-1F469"},
    {"paarike_sudamega_naine_naine_varvi1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"paarike_sudamega_naine_naine_varvi1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"paarike_sudamega_naine_naine_varvi1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"paarike_sudamega_naine_naine_varvi1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"paarike_sudamega_naine_naine_varvi1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"paarike_sudamega_naine_naine_varvi2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"paarike_sudamega_naine_naine_varvi2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"paarike_sudamega_naine_naine_varvi2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"paarike_sudamega_naine_naine_varvi2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"paarike_sudamega_naine_naine_varvi2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"paarike_sudamega_naine_naine_varvi3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"paarike_sudamega_naine_naine_varvi3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"paarike_sudamega_naine_naine_varvi3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"paarike_sudamega_naine_naine_varvi3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"paarike_sudamega_naine_naine_varvi3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"paarike_sudamega_naine_naine_varvi4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"paarike_sudamega_naine_naine_varvi4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"paarike_sudamega_naine_naine_varvi4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"paarike_sudamega_naine_naine_varvi4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"paarike_sudamega_naine_naine_varvi4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"paarike_sudamega_naine_naine_varvi5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"paarike_sudamega_naine_naine_varvi5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"paarike_sudamega_naine_naine_varvi5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"paarike_sudamega_naine_naine_varvi5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"paarike_sudamega_naine_naine_varvi5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"musi_naine_mees", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"musi_naine_mees_varvi1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"musi_naine_mees_varvi1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"musi_naine_mees_varvi1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"musi_naine_mees_varvi1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"musi_naine_mees_varvi1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"musi_naine_mees_varvi2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"musi_naine_mees_varvi2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"musi_naine_mees_varvi2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"musi_naine_mees_varvi2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"musi_naine_mees_varvi2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"musi_naine_mees_varvi3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"musi_naine_mees_varvi3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"musi_naine_mees_varvi3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"musi_naine_mees_varvi3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"musi_naine_mees_varvi3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"musi_naine_mees_varvi4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"musi_naine_mees_varvi4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"musi_naine_mees_varvi4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"musi_naine_mees_varvi4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"musi_naine_mees_varvi4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"musi_naine_mees_varvi5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"musi_naine_mees_varvi5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"musi_naine_mees_varvi5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"musi_naine_mees_varvi5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"musi_naine_mees_varvi5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"musi_naine_naine", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"musi_naine_naine_varvi1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"musi_naine_naine_varvi1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"musi_naine_naine_varvi1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"musi_naine_naine_varvi1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"musi_naine_naine_varvi1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"musi_naine_naine_varvi2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"musi_naine_naine_varvi2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"musi_naine_naine_varvi2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"musi_naine_naine_varvi2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"musi_naine_naine_varvi2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"musi_naine_naine_varvi3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"musi_naine_naine_varvi3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"musi_naine_naine_varvi3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"musi_naine_naine_varvi3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"musi_naine_naine_varvi3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"musi_naine_naine_varvi4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"musi_naine_naine_varvi4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"musi_naine_naine_varvi4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"musi_naine_naine_varvi4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"musi_naine_naine_varvi4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"musi_naine_naine_varvi5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"musi_naine_naine_varvi5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"musi_naine_naine_varvi5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"musi_naine_naine_varvi5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"musi_naine_naine_varvi5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"perekond_naine_poiss", "1F469-200D-1F466"},
    {"perekond_naine_poiss_poiss", "1F469-200D-1F466-200D-1F466"},
    {"perekond_naine_tudruk", "1F469-200D-1F467"},
    {"perekond_naine_tudruk_poiss", "1F469-200D-1F467-200D-1F466"},
    {"perekond_naine_tudruk_tudruk", "1F469-200D-1F467-200D-1F467"},
    {"perekond_naine_naine_poiss", "1F469-200D-1F469-200D-1F466"},
    {"perekond_naine_naine_poiss_poiss", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"perekond_naine_naine_tudruk", "1F469-200D-1F469-200D-1F467"},
    {"perekond_naine_naine_tudruk_poiss", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"perekond_naine_naine_tudruk_tudruk", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"inimesed_hoiavad_kaest_kinni", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"inimesed_hoiavad_kaest_kinni_varvi1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"inimesed_hoiavad_kaest_kinni_varvi1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"inimesed_hoiavad_kaest_kinni_varvi1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"inimesed_hoiavad_kaest_kinni_varvi1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"inimesed_hoiavad_kaest_kinni_varvi1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"inimesed_hoiavad_kaest_kinni_varvi2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"inimesed_hoiavad_kaest_kinni_varvi2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"inimesed_hoiavad_kaest_kinni_varvi2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"inimesed_hoiavad_kaest_kinni_varvi2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"inimesed_hoiavad_kaest_kinni_varvi2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"inimesed_hoiavad_kaest_kinni_varvi3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"inimesed_hoiavad_kaest_kinni_varvi3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"inimesed_hoiavad_kaest_kinni_varvi3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"inimesed_hoiavad_kaest_kinni_varvi3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"inimesed_hoiavad_kaest_kinni_varvi3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"inimesed_hoiavad_kaest_kinni_varvi4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"inimesed_hoiavad_kaest_kinni_varvi4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"inimesed_hoiavad_kaest_kinni_varvi4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"inimesed_hoiavad_kaest_kinni_varvi4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"inimesed_hoiavad_kaest_kinni_varvi4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"inimesed_hoiavad_kaest_kinni_varvi5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"inimesed_hoiavad_kaest_kinni_varvi5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"inimesed_hoiavad_kaest_kinni_varvi5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"inimesed_hoiavad_kaest_kinni_varvi5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"inimesed_hoiavad_kaest_kinni_varvi5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"perekond_taiskasvanu_taiskasvanu_laps", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"perekond_taiskasvanu_taiskasvanu_laps_laps", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"perekond_taiskasvanu_laps", "1F9D1-200D-1F9D2"},
    {"perekond_taiskasvanu_laps_laps", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"jooksja_suunaga_paremale", "1F3C3-200D-27A1-FE0F"},
    {"jooksja_suunaga_paremale_varvi1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"jooksja_suunaga_paremale_varvi2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"jooksja_suunaga_paremale_varvi3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"jooksja_suunaga_paremale_varvi4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"jooksja_suunaga_paremale_varvi5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"meesarst", "1F468-200D-2695-FE0F"},
    {"meesarst_varvi1", "1F468-1F3FB-200D-2695-FE0F"},
    {"meesarst_varvi2", "1F468-1F3FC-200D-2695-FE0F"},
    {"meesarst_varvi3", "1F468-1F3FD-200D-2695-FE0F"},
    {"meesarst_varvi4", "1F468-1F3FE-200D-2695-FE0F"},
    {"meesarst_varvi5", "1F468-1F3FF-200D-2695-FE0F"},
    {"meeskohtunik", "1F468-200D-2696-FE0F"},
    {"meeskohtunik_varvi1", "1F468-1F3FB-200D-2696-FE0F"},
    {"meeskohtunik_varvi2", "1F468-1F3FC-200D-2696-FE0F"},
    {"meeskohtunik_varvi3", "1F468-1F3FD-200D-2696-FE0F"},
    {"meeskohtunik_varvi4", "1F468-1F3FE-200D-2696-FE0F"},
    {"meeskohtunik_varvi5", "1F468-1F3FF-200D-2696-FE0F"},
    {"meespiloot", "1F468-200D-2708-FE0F"},
    {"meespiloot_varvi1", "1F468-1F3FB-200D-2708-FE0F"},
    {"meespiloot_varvi2", "1F468-1F3FC-200D-2708-FE0F"},
    {"meespiloot_varvi3", "1F468-1F3FD-200D-2708-FE0F"},
    {"meespiloot_varvi4", "1F468-1F3FE-200D-2708-FE0F"},
    {"meespiloot_varvi5", "1F468-1F3FF-200D-2708-FE0F"},
    {"meesfarmer", "1F468-200D-1F33E"},
    {"meesfarmer_varvi1", "1F468-1F3FB-200D-1F33E"},
    {"meesfarmer_varvi2", "1F468-1F3FC-200D-1F33E"},
    {"meesfarmer_varvi3", "1F468-1F3FD-200D-1F33E"},
    {"meesfarmer_varvi4", "1F468-1F3FE-200D-1F33E"},
    {"meesfarmer_varvi5", "1F468-1F3FF-200D-1F33E"},
    {"meeskokk", "1F468-200D-1F373"},
    {"meeskokk_varvi1", "1F468-1F3FB-200D-1F373"},
    {"meeskokk_varvi2", "1F468-1F3FC-200D-1F373"},
    {"meeskokk_varvi3", "1F468-1F3FD-200D-1F373"},
    {"meeskokk_varvi4", "1F468-1F3FE-200D-1F373"},
    {"meeskokk_varvi5", "1F468-1F3FF-200D-1F373"},
    {"beebit_toitev_mees", "1F468-200D-1F37C"},
    {"beebit_toitev_mees_varvi1", "1F468-1F3FB-200D-1F37C"},
    {"beebit_toitev_mees_varvi2", "1F468-1F3FC-200D-1F37C"},
    {"beebit_toitev_mees_varvi3", "1F468-1F3FD-200D-1F37C"},
    {"beebit_toitev_mees_varvi4", "1F468-1F3FE-200D-1F37C"},
    {"beebit_toitev_mees_varvi5", "1F468-1F3FF-200D-1F37C"},
    {"meesopilane", "1F468-200D-1F393"},
    {"meesopilane_varvi1", "1F468-1F3FB-200D-1F393"},
    {"meesopilane_varvi2", "1F468-1F3FC-200D-1F393"},
    {"meesopilane_varvi3", "1F468-1F3FD-200D-1F393"},
    {"meesopilane_varvi4", "1F468-1F3FE-200D-1F393"},
    {"meesopilane_varvi5", "1F468-1F3FF-200D-1F393"},
    {"meeslaulja", "1F468-200D-1F3A4"},
    {"meeslaulja_varvi1", "1F468-1F3FB-200D-1F3A4"},
    {"meeslaulja_varvi2", "1F468-1F3FC-200D-1F3A4"},
    {"meeslaulja_varvi3", "1F468-1F3FD-200D-1F3A4"},
    {"meeslaulja_varvi4", "1F468-1F3FE-200D-1F3A4"},
    {"meeslaulja_varvi5", "1F468-1F3FF-200D-1F3A4"},
    {"meeskunstnik", "1F468-200D-1F3A8"},
    {"meeskunstnik_varvi1", "1F468-1F3FB-200D-1F3A8"},
    {"meeskunstnik_varvi2", "1F468-1F3FC-200D-1F3A8"},
    {"meeskunstnik_varvi3", "1F468-1F3FD-200D-1F3A8"},
    {"meeskunstnik_varvi4", "1F468-1F3FE-200D-1F3A8"},
    {"meeskunstnik_varvi5", "1F468-1F3FF-200D-1F3A8"},
    {"meesopetaja", "1F468-200D-1F3EB"},
    {"meesopetaja_varvi1", "1F468-1F3FB-200D-1F3EB"},
    {"meesopetaja_varvi2", "1F468-1F3FC-200D-1F3EB"},
    {"meesopetaja_varvi3", "1F468-1F3FD-200D-1F3EB"},
    {"meesopetaja_varvi4", "1F468-1F3FE-200D-1F3EB"},
    {"meesopetaja_varvi5", "1F468-1F3FF-200D-1F3EB"},
    {"meessoost_tehasetooline", "1F468-200D-1F3ED"},
    {"meessoost_tehasetooline_varvi1", "1F468-1F3FB-200D-1F3ED"},
    {"meessoost_tehasetooline_varvi2", "1F468-1F3FC-200D-1F3ED"},
    {"meessoost_tehasetooline_varvi3", "1F468-1F3FD-200D-1F3ED"},
    {"meessoost_tehasetooline_varvi4", "1F468-1F3FE-200D-1F3ED"},
    {"meessoost_tehasetooline_varvi5", "1F468-1F3FF-200D-1F3ED"},
    {"meesprogrammeerija", "1F468-200D-1F4BB"},
    {"meesprogrammeerija_varvi1", "1F468-1F3FB-200D-1F4BB"},
    {"meesprogrammeerija_varvi2", "1F468-1F3FC-200D-1F4BB"},
    {"meesprogrammeerija_varvi3", "1F468-1F3FD-200D-1F4BB"},
    {"meesprogrammeerija_varvi4", "1F468-1F3FE-200D-1F4BB"},
    {"meesprogrammeerija_varvi5", "1F468-1F3FF-200D-1F4BB"},
    {"meeskontoritootaja", "1F468-200D-1F4BC"},
    {"meeskontoritootaja_varvi1", "1F468-1F3FB-200D-1F4BC"},
    {"meeskontoritootaja_varvi2", "1F468-1F3FC-200D-1F4BC"},
    {"meeskontoritootaja_varvi3", "1F468-1F3FD-200D-1F4BC"},
    {"meeskontoritootaja_varvi4", "1F468-1F3FE-200D-1F4BC"},
    {"meeskontoritootaja_varvi5", "1F468-1F3FF-200D-1F4BC"},
    {"meesmehaanik", "1F468-200D-1F527"},
    {"meesmehaanik_varvi1", "1F468-1F3FB-200D-1F527"},
    {"meesmehaanik_varvi2", "1F468-1F3FC-200D-1F527"},
    {"meesmehaanik_varvi3", "1F468-1F3FD-200D-1F527"},
    {"meesmehaanik_varvi4", "1F468-1F3FE-200D-1F527"},
    {"meesmehaanik_varvi5", "1F468-1F3FF-200D-1F527"},
    {"meesteadlane", "1F468-200D-1F52C"},
    {"meesteadlane_varvi1", "1F468-1F3FB-200D-1F52C"},
    {"meesteadlane_varvi2", "1F468-1F3FC-200D-1F52C"},
    {"meesteadlane_varvi3", "1F468-1F3FD-200D-1F52C"},
    {"meesteadlane_varvi4", "1F468-1F3FE-200D-1F52C"},
    {"meesteadlane_varvi5", "1F468-1F3FF-200D-1F52C"},
    {"meesastronaut", "1F468-200D-1F680"},
    {"meesastronaut_varvi1", "1F468-1F3FB-200D-1F680"},
    {"meesastronaut_varvi2", "1F468-1F3FC-200D-1F680"},
    {"meesastronaut_varvi3", "1F468-1F3FD-200D-1F680"},
    {"meesastronaut_varvi4", "1F468-1F3FE-200D-1F680"},
    {"meesastronaut_varvi5", "1F468-1F3FF-200D-1F680"},
    {"meestuletorjuja", "1F468-200D-1F692"},
    {"meestuletorjuja_varvi1", "1F468-1F3FB-200D-1F692"},
    {"meestuletorjuja_varvi2", "1F468-1F3FC-200D-1F692"},
    {"meestuletorjuja_varvi3", "1F468-1F3FD-200D-1F692"},
    {"meestuletorjuja_varvi4", "1F468-1F3FE-200D-1F692"},
    {"meestuletorjuja_varvi5", "1F468-1F3FF-200D-1F692"},
    {"valge_kepiga_mees", "1F468-200D-1F9AF"},
    {"valge_kepiga_mees_varvi1", "1F468-1F3FB-200D-1F9AF"},
    {"valge_kepiga_mees_varvi2", "1F468-1F3FC-200D-1F9AF"},
    {"valge_kepiga_mees_varvi3", "1F468-1F3FD-200D-1F9AF"},
    {"valge_kepiga_mees_varvi4", "1F468-1F3FE-200D-1F9AF"},
    {"valge_kepiga_mees_varvi5", "1F468-1F3FF-200D-1F9AF"},
    {"valge_kepiga_mees_suunaga_paremale", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_mees_suunaga_paremale_varvi1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_mees_suunaga_paremale_varvi2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_mees_suunaga_paremale_varvi3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_mees_suunaga_paremale_varvi4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_mees_suunaga_paremale_varvi5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"elektriratastoolis_olev_mees", "1F468-200D-1F9BC"},
    {"elektriratastoolis_olev_mees_varvi1", "1F468-1F3FB-200D-1F9BC"},
    {"elektriratastoolis_olev_mees_varvi2", "1F468-1F3FC-200D-1F9BC"},
    {"elektriratastoolis_olev_mees_varvi3", "1F468-1F3FD-200D-1F9BC"},
    {"elektriratastoolis_olev_mees_varvi4", "1F468-1F3FE-200D-1F9BC"},
    {"elektriratastoolis_olev_mees_varvi5", "1F468-1F3FF-200D-1F9BC"},
    {"elektriratastoolis_olev_mees_suunaga_paremale", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"elektriratastoolis_olev_mees_suunaga_paremale_varvi1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"elektriratastoolis_olev_mees_suunaga_paremale_varvi2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"elektriratastoolis_olev_mees_suunaga_paremale_varvi3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"elektriratastoolis_olev_mees_suunaga_paremale_varvi4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"elektriratastoolis_olev_mees_suunaga_paremale_varvi5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"manuaalratastoolis_olev_mees", "1F468-200D-1F9BD"},
    {"manuaalratastoolis_olev_mees_varvi1", "1F468-1F3FB-200D-1F9BD"},
    {"manuaalratastoolis_olev_mees_varvi2", "1F468-1F3FC-200D-1F9BD"},
    {"manuaalratastoolis_olev_mees_varvi3", "1F468-1F3FD-200D-1F9BD"},
    {"manuaalratastoolis_olev_mees_varvi4", "1F468-1F3FE-200D-1F9BD"},
    {"manuaalratastoolis_olev_mees_varvi5", "1F468-1F3FF-200D-1F9BD"},
    {"manuaalratastoolis_olev_mees_suunaga_paremale", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalratastoolis_olev_mees_suunaga_paremale_varvi1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalratastoolis_olev_mees_suunaga_paremale_varvi2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalratastoolis_olev_mees_suunaga_paremale_varvi3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalratastoolis_olev_mees_suunaga_paremale_varvi4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalratastoolis_olev_mees_suunaga_paremale_varvi5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"naisarst", "1F469-200D-2695-FE0F"},
    {"naisarst_varvi1", "1F469-1F3FB-200D-2695-FE0F"},
    {"naisarst_varvi2", "1F469-1F3FC-200D-2695-FE0F"},
    {"naisarst_varvi3", "1F469-1F3FD-200D-2695-FE0F"},
    {"naisarst_varvi4", "1F469-1F3FE-200D-2695-FE0F"},
    {"naisarst_varvi5", "1F469-1F3FF-200D-2695-FE0F"},
    {"naiskohtunik", "1F469-200D-2696-FE0F"},
    {"naiskohtunik_varvi1", "1F469-1F3FB-200D-2696-FE0F"},
    {"naiskohtunik_varvi2", "1F469-1F3FC-200D-2696-FE0F"},
    {"naiskohtunik_varvi3", "1F469-1F3FD-200D-2696-FE0F"},
    {"naiskohtunik_varvi4", "1F469-1F3FE-200D-2696-FE0F"},
    {"naiskohtunik_varvi5", "1F469-1F3FF-200D-2696-FE0F"},
    {"naispiloot", "1F469-200D-2708-FE0F"},
    {"naispiloot_varvi1", "1F469-1F3FB-200D-2708-FE0F"},
    {"naispiloot_varvi2", "1F469-1F3FC-200D-2708-FE0F"},
    {"naispiloot_varvi3", "1F469-1F3FD-200D-2708-FE0F"},
    {"naispiloot_varvi4", "1F469-1F3FE-200D-2708-FE0F"},
    {"naispiloot_varvi5", "1F469-1F3FF-200D-2708-FE0F"},
    {"naisfarmer", "1F469-200D-1F33E"},
    {"naisfarmer_varvi1", "1F469-1F3FB-200D-1F33E"},
    {"naisfarmer_varvi2", "1F469-1F3FC-200D-1F33E"},
    {"naisfarmer_varvi3", "1F469-1F3FD-200D-1F33E"},
    {"naisfarmer_varvi4", "1F469-1F3FE-200D-1F33E"},
    {"naisfarmer_varvi5", "1F469-1F3FF-200D-1F33E"},
    {"naiskokk", "1F469-200D-1F373"},
    {"naiskokk_varvi1", "1F469-1F3FB-200D-1F373"},
    {"naiskokk_varvi2", "1F469-1F3FC-200D-1F373"},
    {"naiskokk_varvi3", "1F469-1F3FD-200D-1F373"},
    {"naiskokk_varvi4", "1F469-1F3FE-200D-1F373"},
    {"naiskokk_varvi5", "1F469-1F3FF-200D-1F373"},
    {"beebit_toitev_naine", "1F469-200D-1F37C"},
    {"beebit_toitev_naine_varvi1", "1F469-1F3FB-200D-1F37C"},
    {"beebit_toitev_naine_varvi2", "1F469-1F3FC-200D-1F37C"},
    {"beebit_toitev_naine_varvi3", "1F469-1F3FD-200D-1F37C"},
    {"beebit_toitev_naine_varvi4", "1F469-1F3FE-200D-1F37C"},
    {"beebit_toitev_naine_varvi5", "1F469-1F3FF-200D-1F37C"},
    {"naisopilane", "1F469-200D-1F393"},
    {"naisopilane_varvi1", "1F469-1F3FB-200D-1F393"},
    {"naisopilane_varvi2", "1F469-1F3FC-200D-1F393"},
    {"naisopilane_varvi3", "1F469-1F3FD-200D-1F393"},
    {"naisopilane_varvi4", "1F469-1F3FE-200D-1F393"},
    {"naisopilane_varvi5", "1F469-1F3FF-200D-1F393"},
    {"naislaulja", "1F469-200D-1F3A4"},
    {"naislaulja_varvi1", "1F469-1F3FB-200D-1F3A4"},
    {"naislaulja_varvi2", "1F469-1F3FC-200D-1F3A4"},
    {"naislaulja_varvi3", "1F469-1F3FD-200D-1F3A4"},
    {"naislaulja_varvi4", "1F469-1F3FE-200D-1F3A4"},
    {"naislaulja_varvi5", "1F469-1F3FF-200D-1F3A4"},
    {"naiskunstnik", "1F469-200D-1F3A8"},
    {"naiskunstnik_varvi1", "1F469-1F3FB-200D-1F3A8"},
    {"naiskunstnik_varvi2", "1F469-1F3FC-200D-1F3A8"},
    {"naiskunstnik_varvi3", "1F469-1F3FD-200D-1F3A8"},
    {"naiskunstnik_varvi4", "1F469-1F3FE-200D-1F3A8"},
    {"naiskunstnik_varvi5", "1F469-1F3FF-200D-1F3A8"},
    {"naisopetaja", "1F469-200D-1F3EB"},
    {"naisopetaja_varvi1", "1F469-1F3FB-200D-1F3EB"},
    {"naisopetaja_varvi2", "1F469-1F3FC-200D-1F3EB"},
    {"naisopetaja_varvi3", "1F469-1F3FD-200D-1F3EB"},
    {"naisopetaja_varvi4", "1F469-1F3FE-200D-1F3EB"},
    {"naisopetaja_varvi5", "1F469-1F3FF-200D-1F3EB"},
    {"naissoost_tehasetooline", "1F469-200D-1F3ED"},
    {"naissoost_tehasetooline_varvi1", "1F469-1F3FB-200D-1F3ED"},
    {"naissoost_tehasetooline_varvi2", "1F469-1F3FC-200D-1F3ED"},
    {"naissoost_tehasetooline_varvi3", "1F469-1F3FD-200D-1F3ED"},
    {"naissoost_tehasetooline_varvi4", "1F469-1F3FE-200D-1F3ED"},
    {"naissoost_tehasetooline_varvi5", "1F469-1F3FF-200D-1F3ED"},
    {"naisprogrammeerija", "1F469-200D-1F4BB"},
    {"naisprogrammeerija_varvi1", "1F469-1F3FB-200D-1F4BB"},
    {"naisprogrammeerija_varvi2", "1F469-1F3FC-200D-1F4BB"},
    {"naisprogrammeerija_varvi3", "1F469-1F3FD-200D-1F4BB"},
    {"naisprogrammeerija_varvi4", "1F469-1F3FE-200D-1F4BB"},
    {"naisprogrammeerija_varvi5", "1F469-1F3FF-200D-1F4BB"},
    {"naiskontoritootaja", "1F469-200D-1F4BC"},
    {"naiskontoritootaja_varvi1", "1F469-1F3FB-200D-1F4BC"},
    {"naiskontoritootaja_varvi2", "1F469-1F3FC-200D-1F4BC"},
    {"naiskontoritootaja_varvi3", "1F469-1F3FD-200D-1F4BC"},
    {"naiskontoritootaja_varvi4", "1F469-1F3FE-200D-1F4BC"},
    {"naiskontoritootaja_varvi5", "1F469-1F3FF-200D-1F4BC"},
    {"naismehaanik", "1F469-200D-1F527"},
    {"naismehaanik_varvi1", "1F469-1F3FB-200D-1F527"},
    {"naismehaanik_varvi2", "1F469-1F3FC-200D-1F527"},
    {"naismehaanik_varvi3", "1F469-1F3FD-200D-1F527"},
    {"naismehaanik_varvi4", "1F469-1F3FE-200D-1F527"},
    {"naismehaanik_varvi5", "1F469-1F3FF-200D-1F527"},
    {"naisteadlane", "1F469-200D-1F52C"},
    {"naisteadlane_varvi1", "1F469-1F3FB-200D-1F52C"},
    {"naisteadlane_varvi2", "1F469-1F3FC-200D-1F52C"},
    {"naisteadlane_varvi3", "1F469-1F3FD-200D-1F52C"},
    {"naisteadlane_varvi4", "1F469-1F3FE-200D-1F52C"},
    {"naisteadlane_varvi5", "1F469-1F3FF-200D-1F52C"},
    {"naisastronaut", "1F469-200D-1F680"},
    {"naisastronaut_varvi1", "1F469-1F3FB-200D-1F680"},
    {"naisastronaut_varvi2", "1F469-1F3FC-200D-1F680"},
    {"naisastronaut_varvi3", "1F469-1F3FD-200D-1F680"},
    {"naisastronaut_varvi4", "1F469-1F3FE-200D-1F680"},
    {"naisastronaut_varvi5", "1F469-1F3FF-200D-1F680"},
    {"naistuletorjuja", "1F469-200D-1F692"},
    {"naistuletorjuja_varvi1", "1F469-1F3FB-200D-1F692"},
    {"naistuletorjuja_varvi2", "1F469-1F3FC-200D-1F692"},
    {"naistuletorjuja_varvi3", "1F469-1F3FD-200D-1F692"},
    {"naistuletorjuja_varvi4", "1F469-1F3FE-200D-1F692"},
    {"naistuletorjuja_varvi5", "1F469-1F3FF-200D-1F692"},
    {"valge_kepiga_naine", "1F469-200D-1F9AF"},
    {"valge_kepiga_naine_varvi1", "1F469-1F3FB-200D-1F9AF"},
    {"valge_kepiga_naine_varvi2", "1F469-1F3FC-200D-1F9AF"},
    {"valge_kepiga_naine_varvi3", "1F469-1F3FD-200D-1F9AF"},
    {"valge_kepiga_naine_varvi4", "1F469-1F3FE-200D-1F9AF"},
    {"valge_kepiga_naine_varvi5", "1F469-1F3FF-200D-1F9AF"},
    {"valge_kepiga_naine_suunaga_paremale", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_naine_suunaga_paremale_varvi1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_naine_suunaga_paremale_varvi2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_naine_suunaga_paremale_varvi3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_naine_suunaga_paremale_varvi4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_naine_suunaga_paremale_varvi5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"elektriratastoolis_olev_naine", "1F469-200D-1F9BC"},
    {"elektriratastoolis_olev_naine_varvi1", "1F469-1F3FB-200D-1F9BC"},
    {"elektriratastoolis_olev_naine_varvi2", "1F469-1F3FC-200D-1F9BC"},
    {"elektriratastoolis_olev_naine_varvi3", "1F469-1F3FD-200D-1F9BC"},
    {"elektriratastoolis_olev_naine_varvi4", "1F469-1F3FE-200D-1F9BC"},
    {"elektriratastoolis_olev_naine_varvi5", "1F469-1F3FF-200D-1F9BC"},
    {"elektriratastoolis_olev_naine_suunaga_paremale", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"elektriratastoolis_olev_naine_suunaga_paremale_varvi1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"elektriratastoolis_olev_naine_suunaga_paremale_varvi2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"elektriratastoolis_olev_naine_suunaga_paremale_varvi3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"elektriratastoolis_olev_naine_suunaga_paremale_varvi4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"elektriratastoolis_olev_naine_suunaga_paremale_varvi5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"manuaalratastoolis_olev_naine", "1F469-200D-1F9BD"},
    {"manuaalratastoolis_olev_naine_varvi1", "1F469-1F3FB-200D-1F9BD"},
    {"manuaalratastoolis_olev_naine_varvi2", "1F469-1F3FC-200D-1F9BD"},
    {"manuaalratastoolis_olev_naine_varvi3", "1F469-1F3FD-200D-1F9BD"},
    {"manuaalratastoolis_olev_naine_varvi4", "1F469-1F3FE-200D-1F9BD"},
    {"manuaalratastoolis_olev_naine_varvi5", "1F469-1F3FF-200D-1F9BD"},
    {"manuaalratastoolis_olev_naine_suunaga_paremale", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalratastoolis_olev_naine_suunaga_paremale_varvi1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalratastoolis_olev_naine_suunaga_paremale_varvi2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalratastoolis_olev_naine_suunaga_paremale_varvi3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalratastoolis_olev_naine_suunaga_paremale_varvi4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalratastoolis_olev_naine_suunaga_paremale_varvi5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"jalakaija_suunaga_paremale", "1F6B6-200D-27A1-FE0F"},
    {"jalakaija_suunaga_paremale_varvi1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"jalakaija_suunaga_paremale_varvi2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"jalakaija_suunaga_paremale_varvi3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"jalakaija_suunaga_paremale_varvi4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"jalakaija_suunaga_paremale_varvi5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"polvitav_inimene_suunaga_paremale", "1F9CE-200D-27A1-FE0F"},
    {"polvitav_inimene_suunaga_paremale_varvi1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"polvitav_inimene_suunaga_paremale_varvi2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"polvitav_inimene_suunaga_paremale_varvi3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"polvitav_inimene_suunaga_paremale_varvi4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"polvitav_inimene_suunaga_paremale_varvi5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"tervishoiutootaja", "1F9D1-200D-2695-FE0F"},
    {"tervishoiutootaja_varvi1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"tervishoiutootaja_varvi2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"tervishoiutootaja_varvi3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"tervishoiutootaja_varvi4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"tervishoiutootaja_varvi5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"kohtunik", "1F9D1-200D-2696-FE0F"},
    {"kohtunik_varvi1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"kohtunik_varvi2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"kohtunik_varvi3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"kohtunik_varvi4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"kohtunik_varvi5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"piloot", "1F9D1-200D-2708-FE0F"},
    {"piloot_varvi1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"piloot_varvi2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"piloot_varvi3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"piloot_varvi4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"piloot_varvi5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"talunik", "1F9D1-200D-1F33E"},
    {"talunik_varvi1", "1F9D1-1F3FB-200D-1F33E"},
    {"talunik_varvi2", "1F9D1-1F3FC-200D-1F33E"},
    {"talunik_varvi3", "1F9D1-1F3FD-200D-1F33E"},
    {"talunik_varvi4", "1F9D1-1F3FE-200D-1F33E"},
    {"talunik_varvi5", "1F9D1-1F3FF-200D-1F33E"},
    {"kokk", "1F9D1-200D-1F373"},
    {"kokk_varvi1", "1F9D1-1F3FB-200D-1F373"},
    {"kokk_varvi2", "1F9D1-1F3FC-200D-1F373"},
    {"kokk_varvi3", "1F9D1-1F3FD-200D-1F373"},
    {"kokk_varvi4", "1F9D1-1F3FE-200D-1F373"},
    {"kokk_varvi5", "1F9D1-1F3FF-200D-1F373"},
    {"beebit_toitev_inimene", "1F9D1-200D-1F37C"},
    {"beebit_toitev_inimene_varvi1", "1F9D1-1F3FB-200D-1F37C"},
    {"beebit_toitev_inimene_varvi2", "1F9D1-1F3FC-200D-1F37C"},
    {"beebit_toitev_inimene_varvi3", "1F9D1-1F3FD-200D-1F37C"},
    {"beebit_toitev_inimene_varvi4", "1F9D1-1F3FE-200D-1F37C"},
    {"beebit_toitev_inimene_varvi5", "1F9D1-1F3FF-200D-1F37C"},
    {"pakats", "1F9D1-200D-1F384"},
    {"pakats_varvi1", "1F9D1-1F3FB-200D-1F384"},
    {"pakats_varvi2", "1F9D1-1F3FC-200D-1F384"},
    {"pakats_varvi3", "1F9D1-1F3FD-200D-1F384"},
    {"pakats_varvi4", "1F9D1-1F3FE-200D-1F384"},
    {"pakats_varvi5", "1F9D1-1F3FF-200D-1F384"},
    {"opilane", "1F9D1-200D-1F393"},
    {"opilane_varvi1", "1F9D1-1F3FB-200D-1F393"},
    {"opilane_varvi2", "1F9D1-1F3FC-200D-1F393"},
    {"opilane_varvi3", "1F9D1-1F3FD-200D-1F393"},
    {"opilane_varvi4", "1F9D1-1F3FE-200D-1F393"},
    {"opilane_varvi5", "1F9D1-1F3FF-200D-1F393"},
    {"laulja", "1F9D1-200D-1F3A4"},
    {"laulja_varvi1", "1F9D1-1F3FB-200D-1F3A4"},
    {"laulja_varvi2", "1F9D1-1F3FC-200D-1F3A4"},
    {"laulja_varvi3", "1F9D1-1F3FD-200D-1F3A4"},
    {"laulja_varvi4", "1F9D1-1F3FE-200D-1F3A4"},
    {"laulja_varvi5", "1F9D1-1F3FF-200D-1F3A4"},
    {"kunstnik", "1F9D1-200D-1F3A8"},
    {"kunstnik_varvi1", "1F9D1-1F3FB-200D-1F3A8"},
    {"kunstnik_varvi2", "1F9D1-1F3FC-200D-1F3A8"},
    {"kunstnik_varvi3", "1F9D1-1F3FD-200D-1F3A8"},
    {"kunstnik_varvi4", "1F9D1-1F3FE-200D-1F3A8"},
    {"kunstnik_varvi5", "1F9D1-1F3FF-200D-1F3A8"},
    {"opetaja", "1F9D1-200D-1F3EB"},
    {"opetaja_varvi1", "1F9D1-1F3FB-200D-1F3EB"},
    {"opetaja_varvi2", "1F9D1-1F3FC-200D-1F3EB"},
    {"opetaja_varvi3", "1F9D1-1F3FD-200D-1F3EB"},
    {"opetaja_varvi4", "1F9D1-1F3FE-200D-1F3EB"},
    {"opetaja_varvi5", "1F9D1-1F3FF-200D-1F3EB"},
    {"tehasetooline", "1F9D1-200D-1F3ED"},
    {"tehasetooline_varvi1", "1F9D1-1F3FB-200D-1F3ED"},
    {"tehasetooline_varvi2", "1F9D1-1F3FC-200D-1F3ED"},
    {"tehasetooline_varvi3", "1F9D1-1F3FD-200D-1F3ED"},
    {"tehasetooline_varvi4", "1F9D1-1F3FE-200D-1F3ED"},
    {"tehasetooline_varvi5", "1F9D1-1F3FF-200D-1F3ED"},
    {"programmeerija", "1F9D1-200D-1F4BB"},
    {"programmeerija_varvi1", "1F9D1-1F3FB-200D-1F4BB"},
    {"programmeerija_varvi2", "1F9D1-1F3FC-200D-1F4BB"},
    {"programmeerija_varvi3", "1F9D1-1F3FD-200D-1F4BB"},
    {"programmeerija_varvi4", "1F9D1-1F3FE-200D-1F4BB"},
    {"programmeerija_varvi5", "1F9D1-1F3FF-200D-1F4BB"},
    {"kontoritootaja", "1F9D1-200D-1F4BC"},
    {"kontoritootaja_varvi1", "1F9D1-1F3FB-200D-1F4BC"},
    {"kontoritootaja_varvi2", "1F9D1-1F3FC-200D-1F4BC"},
    {"kontoritootaja_varvi3", "1F9D1-1F3FD-200D-1F4BC"},
    {"kontoritootaja_varvi4", "1F9D1-1F3FE-200D-1F4BC"},
    {"kontoritootaja_varvi5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mehaanik", "1F9D1-200D-1F527"},
    {"mehaanik_varvi1", "1F9D1-1F3FB-200D-1F527"},
    {"mehaanik_varvi2", "1F9D1-1F3FC-200D-1F527"},
    {"mehaanik_varvi3", "1F9D1-1F3FD-200D-1F527"},
    {"mehaanik_varvi4", "1F9D1-1F3FE-200D-1F527"},
    {"mehaanik_varvi5", "1F9D1-1F3FF-200D-1F527"},
    {"teadlane", "1F9D1-200D-1F52C"},
    {"teadlane_varvi1", "1F9D1-1F3FB-200D-1F52C"},
    {"teadlane_varvi2", "1F9D1-1F3FC-200D-1F52C"},
    {"teadlane_varvi3", "1F9D1-1F3FD-200D-1F52C"},
    {"teadlane_varvi4", "1F9D1-1F3FE-200D-1F52C"},
    {"teadlane_varvi5", "1F9D1-1F3FF-200D-1F52C"},
    {"astronaut", "1F9D1-200D-1F680"},
    {"astronaut_varvi1", "1F9D1-1F3FB-200D-1F680"},
    {"astronaut_varvi2", "1F9D1-1F3FC-200D-1F680"},
    {"astronaut_varvi3", "1F9D1-1F3FD-200D-1F680"},
    {"astronaut_varvi4", "1F9D1-1F3FE-200D-1F680"},
    {"astronaut_varvi5", "1F9D1-1F3FF-200D-1F680"},
    {"tuletorjuja", "1F9D1-200D-1F692"},
    {"tuletorjuja_varvi1", "1F9D1-1F3FB-200D-1F692"},
    {"tuletorjuja_varvi2", "1F9D1-1F3FC-200D-1F692"},
    {"tuletorjuja_varvi3", "1F9D1-1F3FD-200D-1F692"},
    {"tuletorjuja_varvi4", "1F9D1-1F3FE-200D-1F692"},
    {"tuletorjuja_varvi5", "1F9D1-1F3FF-200D-1F692"},
    {"valge_kepiga_inimene", "1F9D1-200D-1F9AF"},
    {"valge_kepiga_inimene_varvi1", "1F9D1-1F3FB-200D-1F9AF"},
    {"valge_kepiga_inimene_varvi2", "1F9D1-1F3FC-200D-1F9AF"},
    {"valge_kepiga_inimene_varvi3", "1F9D1-1F3FD-200D-1F9AF"},
    {"valge_kepiga_inimene_varvi4", "1F9D1-1F3FE-200D-1F9AF"},
    {"valge_kepiga_inimene_varvi5", "1F9D1-1F3FF-200D-1F9AF"},
    {"valge_kepiga_inimene_suunaga_paremale", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_inimene_suunaga_paremale_varvi1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_inimene_suunaga_paremale_varvi2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_inimene_suunaga_paremale_varvi3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_inimene_suunaga_paremale_varvi4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"valge_kepiga_inimene_suunaga_paremale_varvi5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"elektrilises_ratastoolis_inimene", "1F9D1-200D-1F9BC"},
    {"elektrilises_ratastoolis_inimene_varvi1", "1F9D1-1F3FB-200D-1F9BC"},
    {"elektrilises_ratastoolis_inimene_varvi2", "1F9D1-1F3FC-200D-1F9BC"},
    {"elektrilises_ratastoolis_inimene_varvi3", "1F9D1-1F3FD-200D-1F9BC"},
    {"elektrilises_ratastoolis_inimene_varvi4", "1F9D1-1F3FE-200D-1F9BC"},
    {"elektrilises_ratastoolis_inimene_varvi5", "1F9D1-1F3FF-200D-1F9BC"},
    {"elektrilises_ratastoolis_inimene_suunaga_paremale", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"elektrilises_ratastoolis_inimene_suunaga_paremale_varvi1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"elektrilises_ratastoolis_inimene_suunaga_paremale_varvi2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"elektrilises_ratastoolis_inimene_suunaga_paremale_varvi3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"elektrilises_ratastoolis_inimene_suunaga_paremale_varvi4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"elektrilises_ratastoolis_inimene_suunaga_paremale_varvi5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"manuaalses_ratastoolis_inimene", "1F9D1-200D-1F9BD"},
    {"manuaalses_ratastoolis_inimene_varvi1", "1F9D1-1F3FB-200D-1F9BD"},
    {"manuaalses_ratastoolis_inimene_varvi2", "1F9D1-1F3FC-200D-1F9BD"},
    {"manuaalses_ratastoolis_inimene_varvi3", "1F9D1-1F3FD-200D-1F9BD"},
    {"manuaalses_ratastoolis_inimene_varvi4", "1F9D1-1F3FE-200D-1F9BD"},
    {"manuaalses_ratastoolis_inimene_varvi5", "1F9D1-1F3FF-200D-1F9BD"},
    {"manuaalses_ratastoolis_inimene_suunaga_paremale", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalses_ratastoolis_inimene_suunaga_paremale_varvi1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalses_ratastoolis_inimene_suunaga_paremale_varvi2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalses_ratastoolis_inimene_suunaga_paremale_varvi3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalses_ratastoolis_inimene_suunaga_paremale_varvi4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"manuaalses_ratastoolis_inimene_suunaga_paremale_varvi5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"naine_palliga", "26F9-FE0F-200D-2640-FE0F"},
    {"naine_palliga_varvi1", "26F9-1F3FB-200D-2640-FE0F"},
    {"naine_palliga_varvi2", "26F9-1F3FC-200D-2640-FE0F"},
    {"naine_palliga_varvi3", "26F9-1F3FD-200D-2640-FE0F"},
    {"naine_palliga_varvi4", "26F9-1F3FE-200D-2640-FE0F"},
    {"naine_palliga_varvi5", "26F9-1F3FF-200D-2640-FE0F"},
    {"mees_palliga", "26F9-FE0F-200D-2642-FE0F"},
    {"mees_palliga_varvi1", "26F9-1F3FB-200D-2642-FE0F"},
    {"mees_palliga_varvi2", "26F9-1F3FC-200D-2642-FE0F"},
    {"mees_palliga_varvi3", "26F9-1F3FD-200D-2642-FE0F"},
    {"mees_palliga_varvi4", "26F9-1F3FE-200D-2642-FE0F"},
    {"mees_palliga_varvi5", "26F9-1F3FF-200D-2642-FE0F"},
    {"jooksev_naine", "1F3C3-200D-2640-FE0F"},
    {"jooksev_naine_varvi1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"jooksev_naine_varvi2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"jooksev_naine_varvi3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"jooksev_naine_varvi4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"jooksev_naine_varvi5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"jooksev_naine_suunaga_paremale", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"jooksev_naine_suunaga_paremale_varvi1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"jooksev_naine_suunaga_paremale_varvi2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"jooksev_naine_suunaga_paremale_varvi3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"jooksev_naine_suunaga_paremale_varvi4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"jooksev_naine_suunaga_paremale_varvi5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"jooksev_mees", "1F3C3-200D-2642-FE0F"},
    {"jooksev_mees_varvi1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"jooksev_mees_varvi2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"jooksev_mees_varvi3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"jooksev_mees_varvi4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"jooksev_mees_varvi5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"jooksev_mees_suunaga_paremale", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"jooksev_mees_suunaga_paremale_varvi1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"jooksev_mees_suunaga_paremale_varvi2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"jooksev_mees_suunaga_paremale_varvi3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"jooksev_mees_suunaga_paremale_varvi4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"jooksev_mees_suunaga_paremale_varvi5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"naissurfar", "1F3C4-200D-2640-FE0F"},
    {"naissurfar_varvi1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"naissurfar_varvi2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"naissurfar_varvi3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"naissurfar_varvi4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"naissurfar_varvi5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"meessurfar", "1F3C4-200D-2642-FE0F"},
    {"meessurfar_varvi1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"meessurfar_varvi2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"meessurfar_varvi3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"meessurfar_varvi4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"meessurfar_varvi5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"naisujuja", "1F3CA-200D-2640-FE0F"},
    {"naisujuja_varvi1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"naisujuja_varvi2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"naisujuja_varvi3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"naisujuja_varvi4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"naisujuja_varvi5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"meesujuja", "1F3CA-200D-2642-FE0F"},
    {"meesujuja_varvi1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"meesujuja_varvi2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"meesujuja_varvi3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"meesujuja_varvi4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"meesujuja_varvi5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"naisjoutostja", "1F3CB-FE0F-200D-2640-FE0F"},
    {"naisjoutostja_varvi1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"naisjoutostja_varvi2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"naisjoutostja_varvi3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"naisjoutostja_varvi4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"naisjoutostja_varvi5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"meesjoutostja", "1F3CB-FE0F-200D-2642-FE0F"},
    {"meesjoutostja_varvi1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"meesjoutostja_varvi2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"meesjoutostja_varvi3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"meesjoutostja_varvi4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"meesjoutostja_varvi5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"golfi_mangiv_naine", "1F3CC-FE0F-200D-2640-FE0F"},
    {"golfi_mangiv_naine_varvi1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"golfi_mangiv_naine_varvi2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"golfi_mangiv_naine_varvi3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"golfi_mangiv_naine_varvi4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"golfi_mangiv_naine_varvi5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"golfi_mangiv_mees", "1F3CC-FE0F-200D-2642-FE0F"},
    {"golfi_mangiv_mees_varvi1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"golfi_mangiv_mees_varvi2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"golfi_mangiv_mees_varvi3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"golfi_mangiv_mees_varvi4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"golfi_mangiv_mees_varvi5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"naispolitseinik", "1F46E-200D-2640-FE0F"},
    {"naispolitseinik_varvi1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"naispolitseinik_varvi2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"naispolitseinik_varvi3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"naispolitseinik_varvi4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"naispolitseinik_varvi5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"meespolitseinik", "1F46E-200D-2642-FE0F"},
    {"meespolitseinik_varvi1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"meespolitseinik_varvi2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"meespolitseinik_varvi3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"meespolitseinik_varvi4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"meespolitseinik_varvi5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"pidutsevad_janesekorvadega_naised", "1F46F-200D-2640-FE0F"},
    {"pidutsevad_janesekorvadega_naised_varvi1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"pidutsevad_janesekorvadega_naised_varvi1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"pidutsevad_janesekorvadega_naised_varvi1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"pidutsevad_janesekorvadega_naised_varvi1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"pidutsevad_janesekorvadega_naised_varvi2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"pidutsevad_janesekorvadega_naised_varvi2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"pidutsevad_janesekorvadega_naised_varvi2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"pidutsevad_janesekorvadega_naised_varvi2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"pidutsevad_janesekorvadega_naised_varvi3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"pidutsevad_janesekorvadega_naised_varvi3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"pidutsevad_janesekorvadega_naised_varvi3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"pidutsevad_janesekorvadega_naised_varvi3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"pidutsevad_janesekorvadega_naised_varvi4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"pidutsevad_janesekorvadega_naised_varvi4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"pidutsevad_janesekorvadega_naised_varvi4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"pidutsevad_janesekorvadega_naised_varvi4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"pidutsevad_janesekorvadega_naised_varvi5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"pidutsevad_janesekorvadega_naised_varvi5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"pidutsevad_janesekorvadega_naised_varvi5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"pidutsevad_janesekorvadega_naised_varvi5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"pidutsevad_janesekorvadega_naised_varvi1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"pidutsevad_janesekorvadega_naised_varvi2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"pidutsevad_janesekorvadega_naised_varvi3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"pidutsevad_janesekorvadega_naised_varvi4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"pidutsevad_janesekorvadega_naised_varvi5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"pidutsevad_janesekorvadega_mehed", "1F46F-200D-2642-FE0F"},
    {"pidutsevad_janesekorvadega_mehed_varvi1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"pidutsevad_janesekorvadega_mehed_varvi1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"pidutsevad_janesekorvadega_mehed_varvi1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"pidutsevad_janesekorvadega_mehed_varvi1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"pidutsevad_janesekorvadega_mehed_varvi2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"pidutsevad_janesekorvadega_mehed_varvi2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"pidutsevad_janesekorvadega_mehed_varvi2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"pidutsevad_janesekorvadega_mehed_varvi2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"pidutsevad_janesekorvadega_mehed_varvi3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"pidutsevad_janesekorvadega_mehed_varvi3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"pidutsevad_janesekorvadega_mehed_varvi3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"pidutsevad_janesekorvadega_mehed_varvi3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"pidutsevad_janesekorvadega_mehed_varvi4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"pidutsevad_janesekorvadega_mehed_varvi4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"pidutsevad_janesekorvadega_mehed_varvi4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"pidutsevad_janesekorvadega_mehed_varvi4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"pidutsevad_janesekorvadega_mehed_varvi5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"pidutsevad_janesekorvadega_mehed_varvi5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"pidutsevad_janesekorvadega_mehed_varvi5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"pidutsevad_janesekorvadega_mehed_varvi5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"pidutsevad_janesekorvadega_mehed_varvi1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"pidutsevad_janesekorvadega_mehed_varvi2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"pidutsevad_janesekorvadega_mehed_varvi3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"pidutsevad_janesekorvadega_mehed_varvi4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"pidutsevad_janesekorvadega_mehed_varvi5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"naine_looriga", "1F470-200D-2640-FE0F"},
    {"naine_looriga_varvi1", "1F470-1F3FB-200D-2640-FE0F"},
    {"naine_looriga_varvi2", "1F470-1F3FC-200D-2640-FE0F"},
    {"naine_looriga_varvi3", "1F470-1F3FD-200D-2640-FE0F"},
    {"naine_looriga_varvi4", "1F470-1F3FE-200D-2640-FE0F"},
    {"naine_looriga_varvi5", "1F470-1F3FF-200D-2640-FE0F"},
    {"mees_looriga", "1F470-200D-2642-FE0F"},
    {"mees_looriga_varvi1", "1F470-1F3FB-200D-2642-FE0F"},
    {"mees_looriga_varvi2", "1F470-1F3FC-200D-2642-FE0F"},
    {"mees_looriga_varvi3", "1F470-1F3FD-200D-2642-FE0F"},
    {"mees_looriga_varvi4", "1F470-1F3FE-200D-2642-FE0F"},
    {"mees_looriga_varvi5", "1F470-1F3FF-200D-2642-FE0F"},
    {"naine_blond", "1F471-200D-2640-FE0F"},
    {"naine_blond_varvi1", "1F471-1F3FB-200D-2640-FE0F"},
    {"naine_blond_varvi2", "1F471-1F3FC-200D-2640-FE0F"},
    {"naine_blond_varvi3", "1F471-1F3FD-200D-2640-FE0F"},
    {"naine_blond_varvi4", "1F471-1F3FE-200D-2640-FE0F"},
    {"naine_blond_varvi5", "1F471-1F3FF-200D-2640-FE0F"},
    {"mees_blond", "1F471-200D-2642-FE0F"},
    {"mees_blond_varvi1", "1F471-1F3FB-200D-2642-FE0F"},
    {"mees_blond_varvi2", "1F471-1F3FC-200D-2642-FE0F"},
    {"mees_blond_varvi3", "1F471-1F3FD-200D-2642-FE0F"},
    {"mees_blond_varvi4", "1F471-1F3FE-200D-2642-FE0F"},
    {"mees_blond_varvi5", "1F471-1F3FF-200D-2642-FE0F"},
    {"naine_turbaninga", "1F473-200D-2640-FE0F"},
    {"naine_turbaninga_varvi1", "1F473-1F3FB-200D-2640-FE0F"},
    {"naine_turbaninga_varvi2", "1F473-1F3FC-200D-2640-FE0F"},
    {"naine_turbaninga_varvi3", "1F473-1F3FD-200D-2640-FE0F"},
    {"naine_turbaninga_varvi4", "1F473-1F3FE-200D-2640-FE0F"},
    {"naine_turbaninga_varvi5", "1F473-1F3FF-200D-2640-FE0F"},
    {"mees_turbaniga", "1F473-200D-2642-FE0F"},
    {"mees_turbaniga_varvi1", "1F473-1F3FB-200D-2642-FE0F"},
    {"mees_turbaniga_varvi2", "1F473-1F3FC-200D-2642-FE0F"},
    {"mees_turbaniga_varvi3", "1F473-1F3FD-200D-2642-FE0F"},
    {"mees_turbaniga_varvi4", "1F473-1F3FE-200D-2642-FE0F"},
    {"mees_turbaniga_varvi5", "1F473-1F3FF-200D-2642-FE0F"},
    {"naisehitaja", "1F477-200D-2640-FE0F"},
    {"naisehitaja_varvi1", "1F477-1F3FB-200D-2640-FE0F"},
    {"naisehitaja_varvi2", "1F477-1F3FC-200D-2640-FE0F"},
    {"naisehitaja_varvi3", "1F477-1F3FD-200D-2640-FE0F"},
    {"naisehitaja_varvi4", "1F477-1F3FE-200D-2640-FE0F"},
    {"naisehitaja_varvi5", "1F477-1F3FF-200D-2640-FE0F"},
    {"meesehitaja", "1F477-200D-2642-FE0F"},
    {"meesehitaja_varvi1", "1F477-1F3FB-200D-2642-FE0F"},
    {"meesehitaja_varvi2", "1F477-1F3FC-200D-2642-FE0F"},
    {"meesehitaja_varvi3", "1F477-1F3FD-200D-2642-FE0F"},
    {"meesehitaja_varvi4", "1F477-1F3FE-200D-2642-FE0F"},
    {"meesehitaja_varvi5", "1F477-1F3FF-200D-2642-FE0F"},
    {"infoleti_naistootaja", "1F481-200D-2640-FE0F"},
    {"infoleti_naistootaja_varvi1", "1F481-1F3FB-200D-2640-FE0F"},
    {"infoleti_naistootaja_varvi2", "1F481-1F3FC-200D-2640-FE0F"},
    {"infoleti_naistootaja_varvi3", "1F481-1F3FD-200D-2640-FE0F"},
    {"infoleti_naistootaja_varvi4", "1F481-1F3FE-200D-2640-FE0F"},
    {"infoleti_naistootaja_varvi5", "1F481-1F3FF-200D-2640-FE0F"},
    {"infoleti_meestootaja", "1F481-200D-2642-FE0F"},
    {"infoleti_meestootaja_varvi1", "1F481-1F3FB-200D-2642-FE0F"},
    {"infoleti_meestootaja_varvi2", "1F481-1F3FC-200D-2642-FE0F"},
    {"infoleti_meestootaja_varvi3", "1F481-1F3FD-200D-2642-FE0F"},
    {"infoleti_meestootaja_varvi4", "1F481-1F3FE-200D-2642-FE0F"},
    {"infoleti_meestootaja_varvi5", "1F481-1F3FF-200D-2642-FE0F"},
    {"naisvalvur", "1F482-200D-2640-FE0F"},
    {"naisvalvur_varvi1", "1F482-1F3FB-200D-2640-FE0F"},
    {"naisvalvur_varvi2", "1F482-1F3FC-200D-2640-FE0F"},
    {"naisvalvur_varvi3", "1F482-1F3FD-200D-2640-FE0F"},
    {"naisvalvur_varvi4", "1F482-1F3FE-200D-2640-FE0F"},
    {"naisvalvur_varvi5", "1F482-1F3FF-200D-2640-FE0F"},
    {"meesvalvur", "1F482-200D-2642-FE0F"},
    {"meesvalvur_varvi1", "1F482-1F3FB-200D-2642-FE0F"},
    {"meesvalvur_varvi2", "1F482-1F3FC-200D-2642-FE0F"},
    {"meesvalvur_varvi3", "1F482-1F3FD-200D-2642-FE0F"},
    {"meesvalvur_varvi4", "1F482-1F3FE-200D-2642-FE0F"},
    {"meesvalvur_varvi5", "1F482-1F3FF-200D-2642-FE0F"},
    {"naomassaazi_saav_naine", "1F486-200D-2640-FE0F"},
    {"naomassaazi_saav_naine_varvi1", "1F486-1F3FB-200D-2640-FE0F"},
    {"naomassaazi_saav_naine_varvi2", "1F486-1F3FC-200D-2640-FE0F"},
    {"naomassaazi_saav_naine_varvi3", "1F486-1F3FD-200D-2640-FE0F"},
    {"naomassaazi_saav_naine_varvi4", "1F486-1F3FE-200D-2640-FE0F"},
    {"naomassaazi_saav_naine_varvi5", "1F486-1F3FF-200D-2640-FE0F"},
    {"naomassaazi_saav_mees", "1F486-200D-2642-FE0F"},
    {"naomassaazi_saav_mees_varvi1", "1F486-1F3FB-200D-2642-FE0F"},
    {"naomassaazi_saav_mees_varvi2", "1F486-1F3FC-200D-2642-FE0F"},
    {"naomassaazi_saav_mees_varvi3", "1F486-1F3FD-200D-2642-FE0F"},
    {"naomassaazi_saav_mees_varvi4", "1F486-1F3FE-200D-2642-FE0F"},
    {"naomassaazi_saav_mees_varvi5", "1F486-1F3FF-200D-2642-FE0F"},
    {"naine_juuksuris", "1F487-200D-2640-FE0F"},
    {"naine_juuksuris_varvi1", "1F487-1F3FB-200D-2640-FE0F"},
    {"naine_juuksuris_varvi2", "1F487-1F3FC-200D-2640-FE0F"},
    {"naine_juuksuris_varvi3", "1F487-1F3FD-200D-2640-FE0F"},
    {"naine_juuksuris_varvi4", "1F487-1F3FE-200D-2640-FE0F"},
    {"naine_juuksuris_varvi5", "1F487-1F3FF-200D-2640-FE0F"},
    {"mees_juuksuris", "1F487-200D-2642-FE0F"},
    {"mees_juuksuris_varvi1", "1F487-1F3FB-200D-2642-FE0F"},
    {"mees_juuksuris_varvi2", "1F487-1F3FC-200D-2642-FE0F"},
    {"mees_juuksuris_varvi3", "1F487-1F3FD-200D-2642-FE0F"},
    {"mees_juuksuris_varvi4", "1F487-1F3FE-200D-2642-FE0F"},
    {"mees_juuksuris_varvi5", "1F487-1F3FF-200D-2642-FE0F"},
    {"naisdetektiiv", "1F575-FE0F-200D-2640-FE0F"},
    {"naisdetektiiv_varvi1", "1F575-1F3FB-200D-2640-FE0F"},
    {"naisdetektiiv_varvi2", "1F575-1F3FC-200D-2640-FE0F"},
    {"naisdetektiiv_varvi3", "1F575-1F3FD-200D-2640-FE0F"},
    {"naisdetektiiv_varvi4", "1F575-1F3FE-200D-2640-FE0F"},
    {"naisdetektiiv_varvi5", "1F575-1F3FF-200D-2640-FE0F"},
    {"meesdetektiiv", "1F575-FE0F-200D-2642-FE0F"},
    {"meesdetektiiv_varvi1", "1F575-1F3FB-200D-2642-FE0F"},
    {"meesdetektiiv_varvi2", "1F575-1F3FC-200D-2642-FE0F"},
    {"meesdetektiiv_varvi3", "1F575-1F3FD-200D-2642-FE0F"},
    {"meesdetektiiv_varvi4", "1F575-1F3FE-200D-2642-FE0F"},
    {"meesdetektiiv_varvi5", "1F575-1F3FF-200D-2642-FE0F"},
    {"eitavat_zesti_naitav_naine", "1F645-200D-2640-FE0F"},
    {"eitavat_zesti_naitav_naine_varvi1", "1F645-1F3FB-200D-2640-FE0F"},
    {"eitavat_zesti_naitav_naine_varvi2", "1F645-1F3FC-200D-2640-FE0F"},
    {"eitavat_zesti_naitav_naine_varvi3", "1F645-1F3FD-200D-2640-FE0F"},
    {"eitavat_zesti_naitav_naine_varvi4", "1F645-1F3FE-200D-2640-FE0F"},
    {"eitavat_zesti_naitav_naine_varvi5", "1F645-1F3FF-200D-2640-FE0F"},
    {"eitavat_zesti_naitav_mees", "1F645-200D-2642-FE0F"},
    {"eitavat_zesti_naitav_mees_varvi1", "1F645-1F3FB-200D-2642-FE0F"},
    {"eitavat_zesti_naitav_mees_varvi2", "1F645-1F3FC-200D-2642-FE0F"},
    {"eitavat_zesti_naitav_mees_varvi3", "1F645-1F3FD-200D-2642-FE0F"},
    {"eitavat_zesti_naitav_mees_varvi4", "1F645-1F3FE-200D-2642-FE0F"},
    {"eitavat_zesti_naitav_mees_varvi5", "1F645-1F3FF-200D-2642-FE0F"},
    {"noustuvat_zesti_naitav_naine", "1F646-200D-2640-FE0F"},
    {"noustuvat_zesti_naitav_naine_varvi1", "1F646-1F3FB-200D-2640-FE0F"},
    {"noustuvat_zesti_naitav_naine_varvi2", "1F646-1F3FC-200D-2640-FE0F"},
    {"noustuvat_zesti_naitav_naine_varvi3", "1F646-1F3FD-200D-2640-FE0F"},
    {"noustuvat_zesti_naitav_naine_varvi4", "1F646-1F3FE-200D-2640-FE0F"},
    {"noustuvat_zesti_naitav_naine_varvi5", "1F646-1F3FF-200D-2640-FE0F"},
    {"noustuvat_zesti_naitav_mees", "1F646-200D-2642-FE0F"},
    {"noustuvat_zesti_naitav_mees_varvi1", "1F646-1F3FB-200D-2642-FE0F"},
    {"noustuvat_zesti_naitav_mees_varvi2", "1F646-1F3FC-200D-2642-FE0F"},
    {"noustuvat_zesti_naitav_mees_varvi3", "1F646-1F3FD-200D-2642-FE0F"},
    {"noustuvat_zesti_naitav_mees_varvi4", "1F646-1F3FE-200D-2642-FE0F"},
    {"noustuvat_zesti_naitav_mees_varvi5", "1F646-1F3FF-200D-2642-FE0F"},
    {"sugavalt_kummarduv_naine", "1F647-200D-2640-FE0F"},
    {"sugavalt_kummarduv_naine_varvi1", "1F647-1F3FB-200D-2640-FE0F"},
    {"sugavalt_kummarduv_naine_varvi2", "1F647-1F3FC-200D-2640-FE0F"},
    {"sugavalt_kummarduv_naine_varvi3", "1F647-1F3FD-200D-2640-FE0F"},
    {"sugavalt_kummarduv_naine_varvi4", "1F647-1F3FE-200D-2640-FE0F"},
    {"sugavalt_kummarduv_naine_varvi5", "1F647-1F3FF-200D-2640-FE0F"},
    {"sugavalt_kummarduv_mees", "1F647-200D-2642-FE0F"},
    {"sugavalt_kummarduv_mees_varvi1", "1F647-1F3FB-200D-2642-FE0F"},
    {"sugavalt_kummarduv_mees_varvi2", "1F647-1F3FC-200D-2642-FE0F"},
    {"sugavalt_kummarduv_mees_varvi3", "1F647-1F3FD-200D-2642-FE0F"},
    {"sugavalt_kummarduv_mees_varvi4", "1F647-1F3FE-200D-2642-FE0F"},
    {"sugavalt_kummarduv_mees_varvi5", "1F647-1F3FF-200D-2642-FE0F"},
    {"katt_tostev_naine", "1F64B-200D-2640-FE0F"},
    {"katt_tostev_naine_varvi1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"katt_tostev_naine_varvi2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"katt_tostev_naine_varvi3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"katt_tostev_naine_varvi4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"katt_tostev_naine_varvi5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"katt_tostev_mees", "1F64B-200D-2642-FE0F"},
    {"katt_tostev_mees_varvi1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"katt_tostev_mees_varvi2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"katt_tostev_mees_varvi3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"katt_tostev_mees_varvi4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"katt_tostev_mees_varvi5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"kulmu_kortsutav_naine", "1F64D-200D-2640-FE0F"},
    {"kulmu_kortsutav_naine_varvi1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"kulmu_kortsutav_naine_varvi2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"kulmu_kortsutav_naine_varvi3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"kulmu_kortsutav_naine_varvi4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"kulmu_kortsutav_naine_varvi5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"kulmu_kortsutav_mees", "1F64D-200D-2642-FE0F"},
    {"kulmu_kortsutav_mees_varvi1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"kulmu_kortsutav_mees_varvi2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"kulmu_kortsutav_mees_varvi3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"kulmu_kortsutav_mees_varvi4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"kulmu_kortsutav_mees_varvi5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"mossitav_naine", "1F64E-200D-2640-FE0F"},
    {"mossitav_naine_varvi1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"mossitav_naine_varvi2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"mossitav_naine_varvi3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"mossitav_naine_varvi4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"mossitav_naine_varvi5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"mossitav_mees", "1F64E-200D-2642-FE0F"},
    {"mossitav_mees_varvi1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"mossitav_mees_varvi2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"mossitav_mees_varvi3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"mossitav_mees_varvi4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"mossitav_mees_varvi5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"aerupaadiga_naine", "1F6A3-200D-2640-FE0F"},
    {"aerupaadiga_naine_varvi1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"aerupaadiga_naine_varvi2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"aerupaadiga_naine_varvi3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"aerupaadiga_naine_varvi4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"aerupaadiga_naine_varvi5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"aerupaadiga_mees", "1F6A3-200D-2642-FE0F"},
    {"aerupaadiga_mees_varvi1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"aerupaadiga_mees_varvi2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"aerupaadiga_mees_varvi3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"aerupaadiga_mees_varvi4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"aerupaadiga_mees_varvi5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"naisrattur", "1F6B4-200D-2640-FE0F"},
    {"naisrattur_varvi1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"naisrattur_varvi2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"naisrattur_varvi3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"naisrattur_varvi4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"naisrattur_varvi5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"meesrattur", "1F6B4-200D-2642-FE0F"},
    {"meesrattur_varvi1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"meesrattur_varvi2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"meesrattur_varvi3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"meesrattur_varvi4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"meesrattur_varvi5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"naismagirattur", "1F6B5-200D-2640-FE0F"},
    {"naismagirattur_varvi1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"naismagirattur_varvi2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"naismagirattur_varvi3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"naismagirattur_varvi4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"naismagirattur_varvi5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"meesmagirattur", "1F6B5-200D-2642-FE0F"},
    {"meesmagirattur_varvi1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"meesmagirattur_varvi2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"meesmagirattur_varvi3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"meesmagirattur_varvi4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"meesmagirattur_varvi5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"kondiv_naine", "1F6B6-200D-2640-FE0F"},
    {"kondiv_naine_varvi1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"kondiv_naine_varvi2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"kondiv_naine_varvi3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"kondiv_naine_varvi4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"kondiv_naine_varvi5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"kondiv_naine_suunaga_paremale", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kondiv_naine_suunaga_paremale_varvi1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kondiv_naine_suunaga_paremale_varvi2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kondiv_naine_suunaga_paremale_varvi3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kondiv_naine_suunaga_paremale_varvi4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kondiv_naine_suunaga_paremale_varvi5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kondiv_mees", "1F6B6-200D-2642-FE0F"},
    {"kondiv_mees_varvi1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"kondiv_mees_varvi2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"kondiv_mees_varvi3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"kondiv_mees_varvi4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"kondiv_mees_varvi5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"kondiv_mees_suunaga_paremale", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kondiv_mees_suunaga_paremale_varvi1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kondiv_mees_suunaga_paremale_varvi2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kondiv_mees_suunaga_paremale_varvi3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kondiv_mees_suunaga_paremale_varvi4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kondiv_mees_suunaga_paremale_varvi5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kaega_vastu_pead_loov_naine", "1F926-200D-2640-FE0F"},
    {"kaega_vastu_pead_loov_naine_varvi1", "1F926-1F3FB-200D-2640-FE0F"},
    {"kaega_vastu_pead_loov_naine_varvi2", "1F926-1F3FC-200D-2640-FE0F"},
    {"kaega_vastu_pead_loov_naine_varvi3", "1F926-1F3FD-200D-2640-FE0F"},
    {"kaega_vastu_pead_loov_naine_varvi4", "1F926-1F3FE-200D-2640-FE0F"},
    {"kaega_vastu_pead_loov_naine_varvi5", "1F926-1F3FF-200D-2640-FE0F"},
    {"kaega_vastu_pead_loov_mees", "1F926-200D-2642-FE0F"},
    {"kaega_vastu_pead_loov_mees_varvi1", "1F926-1F3FB-200D-2642-FE0F"},
    {"kaega_vastu_pead_loov_mees_varvi2", "1F926-1F3FC-200D-2642-FE0F"},
    {"kaega_vastu_pead_loov_mees_varvi3", "1F926-1F3FD-200D-2642-FE0F"},
    {"kaega_vastu_pead_loov_mees_varvi4", "1F926-1F3FE-200D-2642-FE0F"},
    {"kaega_vastu_pead_loov_mees_varvi5", "1F926-1F3FF-200D-2642-FE0F"},
    {"naine_smokingus", "1F935-200D-2640-FE0F"},
    {"naine_smokingus_varvi1", "1F935-1F3FB-200D-2640-FE0F"},
    {"naine_smokingus_varvi2", "1F935-1F3FC-200D-2640-FE0F"},
    {"naine_smokingus_varvi3", "1F935-1F3FD-200D-2640-FE0F"},
    {"naine_smokingus_varvi4", "1F935-1F3FE-200D-2640-FE0F"},
    {"naine_smokingus_varvi5", "1F935-1F3FF-200D-2640-FE0F"},
    {"mees_smokingis", "1F935-200D-2642-FE0F"},
    {"mees_smokingis_varvi1", "1F935-1F3FB-200D-2642-FE0F"},
    {"mees_smokingis_varvi2", "1F935-1F3FC-200D-2642-FE0F"},
    {"mees_smokingis_varvi3", "1F935-1F3FD-200D-2642-FE0F"},
    {"mees_smokingis_varvi4", "1F935-1F3FE-200D-2642-FE0F"},
    {"mees_smokingis_varvi5", "1F935-1F3FF-200D-2642-FE0F"},
    {"olgu_kehitav_naine", "1F937-200D-2640-FE0F"},
    {"olgu_kehitav_naine_varvi1", "1F937-1F3FB-200D-2640-FE0F"},
    {"olgu_kehitav_naine_varvi2", "1F937-1F3FC-200D-2640-FE0F"},
    {"olgu_kehitav_naine_varvi3", "1F937-1F3FD-200D-2640-FE0F"},
    {"olgu_kehitav_naine_varvi4", "1F937-1F3FE-200D-2640-FE0F"},
    {"olgu_kehitav_naine_varvi5", "1F937-1F3FF-200D-2640-FE0F"},
    {"olgu_kehitav_mees", "1F937-200D-2642-FE0F"},
    {"olgu_kehitav_mees_varvi1", "1F937-1F3FB-200D-2642-FE0F"},
    {"olgu_kehitav_mees_varvi2", "1F937-1F3FC-200D-2642-FE0F"},
    {"olgu_kehitav_mees_varvi3", "1F937-1F3FD-200D-2642-FE0F"},
    {"olgu_kehitav_mees_varvi4", "1F937-1F3FE-200D-2642-FE0F"},
    {"olgu_kehitav_mees_varvi5", "1F937-1F3FF-200D-2642-FE0F"},
    {"hundiratast_viskav_naine", "1F938-200D-2640-FE0F"},
    {"hundiratast_viskav_naine_varvi1", "1F938-1F3FB-200D-2640-FE0F"},
    {"hundiratast_viskav_naine_varvi2", "1F938-1F3FC-200D-2640-FE0F"},
    {"hundiratast_viskav_naine_varvi3", "1F938-1F3FD-200D-2640-FE0F"},
    {"hundiratast_viskav_naine_varvi4", "1F938-1F3FE-200D-2640-FE0F"},
    {"hundiratast_viskav_naine_varvi5", "1F938-1F3FF-200D-2640-FE0F"},
    {"hundiratast_viskav_mees", "1F938-200D-2642-FE0F"},
    {"hundiratast_viskav_mees_varvi1", "1F938-1F3FB-200D-2642-FE0F"},
    {"hundiratast_viskav_mees_varvi2", "1F938-1F3FC-200D-2642-FE0F"},
    {"hundiratast_viskav_mees_varvi3", "1F938-1F3FD-200D-2642-FE0F"},
    {"hundiratast_viskav_mees_varvi4", "1F938-1F3FE-200D-2642-FE0F"},
    {"hundiratast_viskav_mees_varvi5", "1F938-1F3FF-200D-2642-FE0F"},
    {"naiszongloor", "1F939-200D-2640-FE0F"},
    {"naiszongloor_varvi1", "1F939-1F3FB-200D-2640-FE0F"},
    {"naiszongloor_varvi2", "1F939-1F3FC-200D-2640-FE0F"},
    {"naiszongloor_varvi3", "1F939-1F3FD-200D-2640-FE0F"},
    {"naiszongloor_varvi4", "1F939-1F3FE-200D-2640-FE0F"},
    {"naiszongloor_varvi5", "1F939-1F3FF-200D-2640-FE0F"},
    {"meeszongloor", "1F939-200D-2642-FE0F"},
    {"meeszongloor_varvi1", "1F939-1F3FB-200D-2642-FE0F"},
    {"meeszongloor_varvi2", "1F939-1F3FC-200D-2642-FE0F"},
    {"meeszongloor_varvi3", "1F939-1F3FD-200D-2642-FE0F"},
    {"meeszongloor_varvi4", "1F939-1F3FE-200D-2642-FE0F"},
    {"meeszongloor_varvi5", "1F939-1F3FF-200D-2642-FE0F"},
    {"maadlevad_naised", "1F93C-200D-2640-FE0F"},
    {"maadlevad_naised_varvi1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"maadlevad_naised_varvi1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"maadlevad_naised_varvi1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"maadlevad_naised_varvi1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"maadlevad_naised_varvi2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"maadlevad_naised_varvi2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"maadlevad_naised_varvi2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"maadlevad_naised_varvi2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"maadlevad_naised_varvi3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"maadlevad_naised_varvi3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"maadlevad_naised_varvi3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"maadlevad_naised_varvi3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"maadlevad_naised_varvi4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"maadlevad_naised_varvi4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"maadlevad_naised_varvi4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"maadlevad_naised_varvi4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"maadlevad_naised_varvi5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"maadlevad_naised_varvi5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"maadlevad_naised_varvi5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"maadlevad_naised_varvi5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"maadlevad_naised_varvi1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"maadlevad_naised_varvi2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"maadlevad_naised_varvi3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"maadlevad_naised_varvi4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"maadlevad_naised_varvi5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"maadlevad_mehed", "1F93C-200D-2642-FE0F"},
    {"maadlevad_mehed_varvi1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"maadlevad_mehed_varvi1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"maadlevad_mehed_varvi1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"maadlevad_mehed_varvi1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"maadlevad_mehed_varvi2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"maadlevad_mehed_varvi2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"maadlevad_mehed_varvi2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"maadlevad_mehed_varvi2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"maadlevad_mehed_varvi3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"maadlevad_mehed_varvi3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"maadlevad_mehed_varvi3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"maadlevad_mehed_varvi3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"maadlevad_mehed_varvi4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"maadlevad_mehed_varvi4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"maadlevad_mehed_varvi4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"maadlevad_mehed_varvi4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"maadlevad_mehed_varvi5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"maadlevad_mehed_varvi5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"maadlevad_mehed_varvi5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"maadlevad_mehed_varvi5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"maadlevad_mehed_varvi1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"maadlevad_mehed_varvi2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"maadlevad_mehed_varvi3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"maadlevad_mehed_varvi4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"maadlevad_mehed_varvi5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"veepalli_mangiv_naine", "1F93D-200D-2640-FE0F"},
    {"veepalli_mangiv_naine_varvi1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"veepalli_mangiv_naine_varvi2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"veepalli_mangiv_naine_varvi3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"veepalli_mangiv_naine_varvi4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"veepalli_mangiv_naine_varvi5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"veepalli_mangiv_mees", "1F93D-200D-2642-FE0F"},
    {"veepalli_mangiv_mees_varvi1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"veepalli_mangiv_mees_varvi2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"veepalli_mangiv_mees_varvi3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"veepalli_mangiv_mees_varvi4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"veepalli_mangiv_mees_varvi5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"kasipalli_mangiv_naine", "1F93E-200D-2640-FE0F"},
    {"kasipalli_mangiv_naine_varvi1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"kasipalli_mangiv_naine_varvi2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"kasipalli_mangiv_naine_varvi3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"kasipalli_mangiv_naine_varvi4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"kasipalli_mangiv_naine_varvi5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"kasipalli_mangiv_mees", "1F93E-200D-2642-FE0F"},
    {"kasipalli_mangiv_mees_varvi1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"kasipalli_mangiv_mees_varvi2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"kasipalli_mangiv_mees_varvi3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"kasipalli_mangiv_mees_varvi4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"kasipalli_mangiv_mees_varvi5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"naissuperkangelane", "1F9B8-200D-2640-FE0F"},
    {"naissuperkangelane_varvi1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"naissuperkangelane_varvi2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"naissuperkangelane_varvi3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"naissuperkangelane_varvi4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"naissuperkangelane_varvi5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"meeskangelane", "1F9B8-200D-2642-FE0F"},
    {"meeskangelane_varvi1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"meeskangelane_varvi2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"meeskangelane_varvi3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"meeskangelane_varvi4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"meeskangelane_varvi5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"naissoost_superkurikael", "1F9B9-200D-2640-FE0F"},
    {"naissoost_superkurikael_varvi1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"naissoost_superkurikael_varvi2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"naissoost_superkurikael_varvi3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"naissoost_superkurikael_varvi4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"naissoost_superkurikael_varvi5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"meessoost_superkurikael", "1F9B9-200D-2642-FE0F"},
    {"meessoost_superkurikael_varvi1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"meessoost_superkurikael_varvi2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"meessoost_superkurikael_varvi3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"meessoost_superkurikael_varvi4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"meessoost_superkurikael_varvi5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"seisev_naine", "1F9CD-200D-2640-FE0F"},
    {"seisev_naine_varvi1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"seisev_naine_varvi2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"seisev_naine_varvi3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"seisev_naine_varvi4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"seisev_naine_varvi5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"seisev_mees", "1F9CD-200D-2642-FE0F"},
    {"seisev_mees_varvi1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"seisev_mees_varvi2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"seisev_mees_varvi3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"seisev_mees_varvi4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"seisev_mees_varvi5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"polvitav_naine", "1F9CE-200D-2640-FE0F"},
    {"polvitav_naine_varvi1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"polvitav_naine_varvi2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"polvitav_naine_varvi3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"polvitav_naine_varvi4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"polvitav_naine_varvi5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"polvitav_naine_suunaga_paremale", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"polvitav_naine_suunaga_paremale_varvi1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"polvitav_naine_suunaga_paremale_varvi2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"polvitav_naine_suunaga_paremale_varvi3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"polvitav_naine_suunaga_paremale_varvi4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"polvitav_naine_suunaga_paremale_varvi5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"polvitav_mees", "1F9CE-200D-2642-FE0F"},
    {"polvitav_mees_varvi1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"polvitav_mees_varvi2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"polvitav_mees_varvi3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"polvitav_mees_varvi4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"polvitav_mees_varvi5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"polvitav_mees_suunaga_paremale", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"polvitav_mees_suunaga_paremale_varvi1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"polvitav_mees_suunaga_paremale_varvi2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"polvitav_mees_suunaga_paremale_varvi3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"polvitav_mees_suunaga_paremale_varvi4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"polvitav_mees_suunaga_paremale_varvi5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kurt_naine", "1F9CF-200D-2640-FE0F"},
    {"kurt_naine_varvi1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"kurt_naine_varvi2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"kurt_naine_varvi3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"kurt_naine_varvi4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"kurt_naine_varvi5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"kurt_mees", "1F9CF-200D-2642-FE0F"},
    {"kurt_mees_varvi1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"kurt_mees_varvi2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"kurt_mees_varvi3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"kurt_mees_varvi4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"kurt_mees_varvi5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"habemega_naine", "1F9D4-200D-2640-FE0F"},
    {"habemega_naine_varvi1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"habemega_naine_varvi2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"habemega_naine_varvi3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"habemega_naine_varvi4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"habemega_naine_varvi5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"habemega_mees", "1F9D4-200D-2642-FE0F"},
    {"habemega_mees_varvi1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"habemega_mees_varvi2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"habemega_mees_varvi3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"habemega_mees_varvi4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"habemega_mees_varvi5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"naine_leiliruumis", "1F9D6-200D-2640-FE0F"},
    {"naine_leiliruumis_varvi1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"naine_leiliruumis_varvi2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"naine_leiliruumis_varvi3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"naine_leiliruumis_varvi4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"naine_leiliruumis_varvi5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"leiliruumis_olev_mees", "1F9D6-200D-2642-FE0F"},
    {"leiliruumis_olev_mees_varvi1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"leiliruumis_olev_mees_varvi2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"leiliruumis_olev_mees_varvi3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"leiliruumis_olev_mees_varvi4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"leiliruumis_olev_mees_varvi5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"makke_roniv_naine", "1F9D7-200D-2640-FE0F"},
    {"makke_roniv_naine_varvi1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"makke_roniv_naine_varvi2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"makke_roniv_naine_varvi3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"makke_roniv_naine_varvi4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"makke_roniv_naine_varvi5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"makke_roniv_mees", "1F9D7-200D-2642-FE0F"},
    {"makke_roniv_mees_varvi1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"makke_roniv_mees_varvi2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"makke_roniv_mees_varvi3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"makke_roniv_mees_varvi4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"makke_roniv_mees_varvi5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"lootosasendis_naine", "1F9D8-200D-2640-FE0F"},
    {"lootosasendis_naine_varvi1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"lootosasendis_naine_varvi2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"lootosasendis_naine_varvi3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"lootosasendis_naine_varvi4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"lootosasendis_naine_varvi5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"lootosasendis_mees", "1F9D8-200D-2642-FE0F"},
    {"lootosasendis_mees_varvi1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"lootosasendis_mees_varvi2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"lootosasendis_mees_varvi3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"lootosasendis_mees_varvi4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"lootosasendis_mees_varvi5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"naismaag", "1F9D9-200D-2640-FE0F"},
    {"naismaag_varvi1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"naismaag_varvi2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"naismaag_varvi3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"naismaag_varvi4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"naismaag_varvi5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"meesmaag", "1F9D9-200D-2642-FE0F"},
    {"meesmaag_varvi1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"meesmaag_varvi2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"meesmaag_varvi3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"meesmaag_varvi4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"meesmaag_varvi5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"naishaldjas", "1F9DA-200D-2640-FE0F"},
    {"naishaldjas_varvi1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"naishaldjas_varvi2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"naishaldjas_varvi3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"naishaldjas_varvi4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"naishaldjas_varvi5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"meeshaldjas", "1F9DA-200D-2642-FE0F"},
    {"meeshaldjas_varvi1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"meeshaldjas_varvi2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"meeshaldjas_varvi3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"meeshaldjas_varvi4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"meeshaldjas_varvi5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"naisvampiir", "1F9DB-200D-2640-FE0F"},
    {"naisvampiir_varvi1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"naisvampiir_varvi2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"naisvampiir_varvi3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"naisvampiir_varvi4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"naisvampiir_varvi5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"meesvampiir", "1F9DB-200D-2642-FE0F"},
    {"meesvampiir_varvi1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"meesvampiir_varvi2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"meesvampiir_varvi3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"meesvampiir_varvi4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"meesvampiir_varvi5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"merineitsi", "1F9DC-200D-2640-FE0F"},
    {"merineitsi_varvi1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"merineitsi_varvi2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"merineitsi_varvi3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"merineitsi_varvi4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"merineitsi_varvi5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"meesnakk", "1F9DC-200D-2642-FE0F"},
    {"meesnakk_varvi1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"meesnakk_varvi2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"meesnakk_varvi3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"meesnakk_varvi4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"meesnakk_varvi5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"naispakapikk", "1F9DD-200D-2640-FE0F"},
    {"naispakapikk_varvi1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"naispakapikk_varvi2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"naispakapikk_varvi3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"naispakapikk_varvi4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"naispakapikk_varvi5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"meespakapikk", "1F9DD-200D-2642-FE0F"},
    {"meespakapikk_varvi1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"meespakapikk_varvi2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"meespakapikk_varvi3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"meespakapikk_varvi4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"meespakapikk_varvi5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"naisdzinn", "1F9DE-200D-2640-FE0F"},
    {"meesdzinn", "1F9DE-200D-2642-FE0F"},
    {"naiszombi", "1F9DF-200D-2640-FE0F"},
    {"meeszombi", "1F9DF-200D-2642-FE0F"},
    {"mees_punased_juuksed", "1F468-200D-1F9B0"},
    {"mees_punased_juuksed_varvi1", "1F468-1F3FB-200D-1F9B0"},
    {"mees_punased_juuksed_varvi2", "1F468-1F3FC-200D-1F9B0"},
    {"mees_punased_juuksed_varvi3", "1F468-1F3FD-200D-1F9B0"},
    {"mees_punased_juuksed_varvi4", "1F468-1F3FE-200D-1F9B0"},
    {"mees_punased_juuksed_varvi5", "1F468-1F3FF-200D-1F9B0"},
    {"mees_lokkis_juuksed", "1F468-200D-1F9B1"},
    {"mees_lokkis_juuksed_varvi1", "1F468-1F3FB-200D-1F9B1"},
    {"mees_lokkis_juuksed_varvi2", "1F468-1F3FC-200D-1F9B1"},
    {"mees_lokkis_juuksed_varvi3", "1F468-1F3FD-200D-1F9B1"},
    {"mees_lokkis_juuksed_varvi4", "1F468-1F3FE-200D-1F9B1"},
    {"mees_lokkis_juuksed_varvi5", "1F468-1F3FF-200D-1F9B1"},
    {"mees_kiilaspea", "1F468-200D-1F9B2"},
    {"mees_kiilaspea_varvi1", "1F468-1F3FB-200D-1F9B2"},
    {"mees_kiilaspea_varvi2", "1F468-1F3FC-200D-1F9B2"},
    {"mees_kiilaspea_varvi3", "1F468-1F3FD-200D-1F9B2"},
    {"mees_kiilaspea_varvi4", "1F468-1F3FE-200D-1F9B2"},
    {"mees_kiilaspea_varvi5", "1F468-1F3FF-200D-1F9B2"},
    {"mees_heledad_juuksed", "1F468-200D-1F9B3"},
    {"mees_heledad_juuksed_varvi1", "1F468-1F3FB-200D-1F9B3"},
    {"mees_heledad_juuksed_varvi2", "1F468-1F3FC-200D-1F9B3"},
    {"mees_heledad_juuksed_varvi3", "1F468-1F3FD-200D-1F9B3"},
    {"mees_heledad_juuksed_varvi4", "1F468-1F3FE-200D-1F9B3"},
    {"mees_heledad_juuksed_varvi5", "1F468-1F3FF-200D-1F9B3"},
    {"naine_punased_juuksed", "1F469-200D-1F9B0"},
    {"naine_punased_juuksed_varvi1", "1F469-1F3FB-200D-1F9B0"},
    {"naine_punased_juuksed_varvi2", "1F469-1F3FC-200D-1F9B0"},
    {"naine_punased_juuksed_varvi3", "1F469-1F3FD-200D-1F9B0"},
    {"naine_punased_juuksed_varvi4", "1F469-1F3FE-200D-1F9B0"},
    {"naine_punased_juuksed_varvi5", "1F469-1F3FF-200D-1F9B0"},
    {"naine_lokkis_juuksed", "1F469-200D-1F9B1"},
    {"naine_lokkis_juuksed_varvi1", "1F469-1F3FB-200D-1F9B1"},
    {"naine_lokkis_juuksed_varvi2", "1F469-1F3FC-200D-1F9B1"},
    {"naine_lokkis_juuksed_varvi3", "1F469-1F3FD-200D-1F9B1"},
    {"naine_lokkis_juuksed_varvi4", "1F469-1F3FE-200D-1F9B1"},
    {"naine_lokkis_juuksed_varvi5", "1F469-1F3FF-200D-1F9B1"},
    {"naine_kiilaspea", "1F469-200D-1F9B2"},
    {"naine_kiilaspea_varvi1", "1F469-1F3FB-200D-1F9B2"},
    {"naine_kiilaspea_varvi2", "1F469-1F3FC-200D-1F9B2"},
    {"naine_kiilaspea_varvi3", "1F469-1F3FD-200D-1F9B2"},
    {"naine_kiilaspea_varvi4", "1F469-1F3FE-200D-1F9B2"},
    {"naine_kiilaspea_varvi5", "1F469-1F3FF-200D-1F9B2"},
    {"naine_heledad_juuksed", "1F469-200D-1F9B3"},
    {"naine_heledad_juuksed_varvi1", "1F469-1F3FB-200D-1F9B3"},
    {"naine_heledad_juuksed_varvi2", "1F469-1F3FC-200D-1F9B3"},
    {"naine_heledad_juuksed_varvi3", "1F469-1F3FD-200D-1F9B3"},
    {"naine_heledad_juuksed_varvi4", "1F469-1F3FE-200D-1F9B3"},
    {"naine_heledad_juuksed_varvi5", "1F469-1F3FF-200D-1F9B3"},
    {"taiskasvanu_punased_juuksed", "1F9D1-200D-1F9B0"},
    {"taiskasvanu_punased_juuksed_varvi1", "1F9D1-1F3FB-200D-1F9B0"},
    {"taiskasvanu_punased_juuksed_varvi2", "1F9D1-1F3FC-200D-1F9B0"},
    {"taiskasvanu_punased_juuksed_varvi3", "1F9D1-1F3FD-200D-1F9B0"},
    {"taiskasvanu_punased_juuksed_varvi4", "1F9D1-1F3FE-200D-1F9B0"},
    {"taiskasvanu_punased_juuksed_varvi5", "1F9D1-1F3FF-200D-1F9B0"},
    {"taiskasvanu_lokkis_juuksed", "1F9D1-200D-1F9B1"},
    {"taiskasvanu_lokkis_juuksed_varvi1", "1F9D1-1F3FB-200D-1F9B1"},
    {"taiskasvanu_lokkis_juuksed_varvi2", "1F9D1-1F3FC-200D-1F9B1"},
    {"taiskasvanu_lokkis_juuksed_varvi3", "1F9D1-1F3FD-200D-1F9B1"},
    {"taiskasvanu_lokkis_juuksed_varvi4", "1F9D1-1F3FE-200D-1F9B1"},
    {"taiskasvanu_lokkis_juuksed_varvi5", "1F9D1-1F3FF-200D-1F9B1"},
    {"taiskasvanu_kiilaspea", "1F9D1-200D-1F9B2"},
    {"taiskasvanu_kiilaspea_varvi1", "1F9D1-1F3FB-200D-1F9B2"},
    {"taiskasvanu_kiilaspea_varvi2", "1F9D1-1F3FC-200D-1F9B2"},
    {"taiskasvanu_kiilaspea_varvi3", "1F9D1-1F3FD-200D-1F9B2"},
    {"taiskasvanu_kiilaspea_varvi4", "1F9D1-1F3FE-200D-1F9B2"},
    {"taiskasvanu_kiilaspea_varvi5", "1F9D1-1F3FF-200D-1F9B2"},
    {"taiskasvanu_heledad_juuksed", "1F9D1-200D-1F9B3"},
    {"taiskasvanu_heledad_juuksed_varvi1", "1F9D1-1F3FB-200D-1F9B3"},
    {"taiskasvanu_heledad_juuksed_varvi2", "1F9D1-1F3FC-200D-1F9B3"},
    {"taiskasvanu_heledad_juuksed_varvi3", "1F9D1-1F3FD-200D-1F9B3"},
    {"taiskasvanu_heledad_juuksed_varvi4", "1F9D1-1F3FE-200D-1F9B3"},
    {"taiskasvanu_heledad_juuksed_varvi5", "1F9D1-1F3FF-200D-1F9B3"},
    {"katkine_kett", "26D3-FE0F-200D-1F4A5"},
    {"leegitsev_suda", "2764-FE0F-200D-1F525"},
    {"tervenev_suda", "2764-FE0F-200D-1FA79"},
    {"sampinjon", "1F344-200D-1F7EB"},
    {"laim", "1F34B-200D-1F7E9"},
    {"transsoolisuse_lipp", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"vikerkaarelipp", "1F3F3-FE0F-200D-1F308"},
    {"piraadilipp", "1F3F4-200D-2620-FE0F"},
    {"must_kass", "1F408-200D-2B1B"},
    {"teenistuskoer", "1F415-200D-1F9BA"},
    {"must_lind", "1F426-200D-2B1B"},
    {"fooniks", "1F426-200D-1F525"},
    {"jaakaru", "1F43B-200D-2744-FE0F"},
    {"silm_jutumullis", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"valjahingav_nagu", "1F62E-200D-1F4A8"},
    {"spiraalsilmadega_nagu", "1F635-200D-1F4AB"},
    {"nagu_pilvedes", "1F636-200D-1F32B-FE0F"},
    {"pearaputus", "1F642-200D-2194-FE0F"},
    {"peanoogutus", "1F642-200D-2195-FE0F"},
    {"balletitantsija", "1F9D1-200D-1FA70"},
    {"balletitantsija_varvi1", "1F9D1-1F3FB-200D-1FA70"},
    {"balletitantsija_varvi2", "1F9D1-1F3FC-200D-1FA70"},
    {"balletitantsija_varvi3", "1F9D1-1F3FD-200D-1FA70"},
    {"balletitantsija_varvi4", "1F9D1-1F3FE-200D-1FA70"},
    {"balletitantsija_varvi5", "1F9D1-1F3FF-200D-1FA70"},
    {"küsimärgiga_hüüumärk", "2049"},
    {"kaubamärk", "2122"},
    {"nool_üles_alla", "2195"},
    {"nool_üles_vasakule", "2196"},
    {"nool_üles_paremale", "2197"},
    {"päike", "2600"},
    {"märgitud_ruut", "2611"},
    {"kolp_ja_sääreluud", "2620"},
    {"õigeusu_rist", "2626"},
    {"kulmu_kortsutav_nägu", "2639"},
    {"naisesümbol", "2640"},
    {"mehesümbol", "2642"},
    {"jäär", "2648"},
    {"sõnn", "2649"},
    {"ärtu", "2665"},
    {"ristatud_mõõgad", "2694"},
    {"meditsiinisümbol", "2695"},
    {"destilleerimisnõu", "2697"},
    {"käärid", "2702"},
    {"märkenupp", "2705"},
    {"ümbrik", "2709"},
    {"märge", "2714"},
    {"taaveti_täht", "2721"},
    {"sädemed", "2728"},
    {"kaheksa_tipuga_tärn", "2733"},
    {"kaheksa_tipuga_täht", "2734"},
    {"sädelus", "2747"},
    {"punane_küsimärk", "2753"},
    {"valge_küsimärk", "2754"},
    {"valge_hüüumärk", "2755"},
    {"punane_hüüumärk", "2757"},
    {"südamega_hüüumärk", "2763"},
    {"punane_süda", "2764"},
    {"kaardus_nool_üles", "2934"},
    {"jaapani_nupp_õnnitlused", "3297"},
    {"autoriõigus", "00A9"},
    {"topelthüüumärk", "203C"},
    {"käekell", "231A"},
    {"väljutamisnupp", "23CF"},
    {"kiirkerimine_üles", "23EB"},
    {"järgmise_loo_nupp", "23ED"},
    {"äratuskell", "23F0"},
    {"väike_must_ruut", "25AA"},
    {"väike_valge_ruut", "25AB"},
    {"väiksem_valge_ruut", "25FD"},
    {"väiksem_must_ruut", "25FE"},
    {"üles_osutav_sõrm", "261D"},
    {"üles_osutav_sõrm_värvi1", "261D-1F3FB"},
    {"üles_osutav_sõrm_värvi2", "261D-1F3FC"},
    {"üles_osutav_sõrm_värvi3", "261D-1F3FD"},
    {"üles_osutav_sõrm_värvi4", "261D-1F3FE"},
    {"üles_osutav_sõrm_värvi5", "261D-1F3FF"},
    {"täht_ja_kuusirp", "262A"},
    {"rahu_sümbol", "262E"},
    {"naerunägu", "263A"},
    {"vähk", "264B"},
    {"lõvi", "264C"},
    {"taaskasutuse_sümbol", "267B"},
    {"lõpmatus", "267E"},
    {"aatomi_sümbol", "269B"},
    {"kõrgepinge", "26A1"},
    {"transsoolisuse_sümbol", "26A7"},
    {"päike_pilve_taga", "26C5"},
    {"äikese_ja_vihmapilv", "26C8"},
    {"mägi", "26F0"},
    {"päikesevari_maas", "26F1"},
    {"ülestõstetud_rusikas", "270A"},
    {"ülestõstetud_rusikas_värvi1", "270A-1F3FB"},
    {"ülestõstetud_rusikas_värvi2", "270A-1F3FC"},
    {"ülestõstetud_rusikas_värvi3", "270A-1F3FD"},
    {"ülestõstetud_rusikas_värvi4", "270A-1F3FE"},
    {"ülestõstetud_rusikas_värvi5", "270A-1F3FF"},
    {"ülestõstetud_käsi", "270B"},
    {"ülestõstetud_käsi_värvi1", "270B-1F3FB"},
    {"ülestõstetud_käsi_värvi2", "270B-1F3FC"},
    {"ülestõstetud_käsi_värvi3", "270B-1F3FD"},
    {"ülestõstetud_käsi_värvi4", "270B-1F3FE"},
    {"ülestõstetud_käsi_värvi5", "270B-1F3FF"},
    {"võit", "270C"},
    {"võit_värvi1", "270C-1F3FB"},
    {"võit_värvi2", "270C-1F3FC"},
    {"võit_värvi3", "270C-1F3FD"},
    {"võit_värvi4", "270C-1F3FE"},
    {"võit_värvi5", "270C-1F3FF"},
    {"kirjutav_käsi", "270D"},
    {"kirjutav_käsi_värvi1", "270D-1F3FB"},
    {"kirjutav_käsi_värvi2", "270D-1F3FC"},
    {"kirjutav_käsi_värvi3", "270D-1F3FD"},
    {"kirjutav_käsi_värvi4", "270D-1F3FE"},
    {"kirjutav_käsi_värvi5", "270D-1F3FF"},
    {"korrutusmärgiga_nupp", "274E"},
    {"nool_üles", "2B06"},
    {"täht", "2B50"},
    {"seest_tühi_punane_ringjoon", "2B55"},
    {"osa_vaheldumise_märk", "303D"},
    {"piirkondlik_näitaja_a", "1F1E6"},
    {"piirkondlik_näitaja_b", "1F1E7"},
    {"piirkondlik_näitaja_c", "1F1E8"},
    {"piirkondlik_näitaja_d", "1F1E9"},
    {"piirkondlik_näitaja_e", "1F1EA"},
    {"piirkondlik_näitaja_f", "1F1EB"},
    {"piirkondlik_näitaja_g", "1F1EC"},
    {"piirkondlik_näitaja_h", "1F1ED"},
    {"piirkondlik_näitaja_i", "1F1EE"},
    {"piirkondlik_näitaja_j", "1F1EF"},
    {"piirkondlik_näitaja_k", "1F1F0"},
    {"piirkondlik_näitaja_l", "1F1F1"},
    {"piirkondlik_näitaja_m", "1F1F2"},
    {"piirkondlik_näitaja_n", "1F1F3"},
    {"piirkondlik_näitaja_o", "1F1F4"},
    {"piirkondlik_näitaja_p", "1F1F5"},
    {"piirkondlik_näitaja_q", "1F1F6"},
    {"piirkondlik_näitaja_r", "1F1F7"},
    {"piirkondlik_näitaja_s", "1F1F8"},
    {"piirkondlik_näitaja_t", "1F1F9"},
    {"piirkondlik_näitaja_u", "1F1FA"},
    {"piirkondlik_näitaja_v", "1F1FB"},
    {"piirkondlik_näitaja_w", "1F1FC"},
    {"piirkondlik_näitaja_x", "1F1FD"},
    {"piirkondlik_näitaja_y", "1F1FE"},
    {"piirkondlik_näitaja_z", "1F1FF"},
    {"jaapani_nupp_vastuvõetav", "1F251"},
    {"tsüklon", "1F300"},
    {"öine_tähistaevas", "1F303"},
    {"päikesetõus_mägedes", "1F304"},
    {"päikesetõus", "1F305"},
    {"päikeseloojang", "1F307"},
    {"sild_öösel", "1F309"},
    {"täiskuu", "1F315"},
    {"näoga_noorkuu", "1F31A"},
    {"näoga_kasvav_kuu", "1F31B"},
    {"näoga_kahanev_kuu", "1F31C"},
    {"näoga_täiskuu", "1F31D"},
    {"näoga_päike", "1F31E"},
    {"helendav_täht", "1F31F"},
    {"langev_täht", "1F320"},
    {"päike_väikese_pilve_taga", "1F324"},
    {"päike_suure_pilve_taga", "1F325"},
    {"päike_vihmapilve_taga", "1F326"},
    {"äikesepilv", "1F329"},
    {"näoga_tuul", "1F32C"},
    {"võrse", "1F331"},
    {"kirsiõis", "1F338"},
    {"päevalill", "1F33B"},
    {"õis", "1F33C"},
    {"maisitõlvik", "1F33D"},
    {"baklažaan", "1F346"},
    {"punane_õun", "1F34E"},
    {"roheline_õun", "1F34F"},
    {"riisiküpsis", "1F358"},
    {"röstitud_bataat", "1F360"},
    {"suši", "1F363"},
    {"pehme_jäätis", "1F366"},
    {"mahlajää", "1F367"},
    {"jäätis", "1F368"},
    {"sõõrik", "1F369"},
    {"küpsis", "1F36A"},
    {"šokolaaditahvel", "1F36B"},
    {"potitäis_toitu", "1F372"},
    {"kõrvata_teetass", "1F375"},
    {"õllekann", "1F37A"},
    {"kõlisevad_õllekannud", "1F37B"},
    {"sünnipäevatort", "1F382"},
    {"kõrvitsalatern", "1F383"},
    {"jõulupuu", "1F384"},
    {"jõuluvana", "1F385"},
    {"jõuluvana_värvi1", "1F385-1F3FB"},
    {"jõuluvana_värvi2", "1F385-1F3FC"},
    {"jõuluvana_värvi3", "1F385-1F3FD"},
    {"jõuluvana_värvi4", "1F385-1F3FE"},
    {"jõuluvana_värvi5", "1F385-1F3FF"},
    {"säraküünal", "1F387"},
    {"õhupall", "1F388"},
    {"pidupäev", "1F389"},
    {"väravamänd", "1F38D"},
    {"koolilõpumüts", "1F393"},
    {"sõjamedal", "1F396"},
    {"ameerika_mäed", "1F3A2"},
    {"õng", "1F3A3"},
    {"kõrvaklapid", "1F3A7"},
    {"torukübar", "1F3A9"},
    {"videomäng", "1F3AE"},
    {"täistabamus", "1F3AF"},
    {"mänguautomaat", "1F3B0"},
    {"täring", "1F3B2"},
    {"mängukaardid", "1F3B4"},
    {"jooksusärk", "1F3BD"},
    {"hobuste_võiduajamine", "1F3C7"},
    {"hobuste_võiduajamine_värvi1", "1F3C7-1F3FB"},
    {"hobuste_võiduajamine_värvi2", "1F3C7-1F3FC"},
    {"hobuste_võiduajamine_värvi3", "1F3C7-1F3FD"},
    {"hobuste_võiduajamine_värvi4", "1F3C7-1F3FE"},
    {"hobuste_võiduajamine_värvi5", "1F3C7-1F3FF"},
    {"jõutõstja", "1F3CB"},
    {"jõutõstja_värvi1", "1F3CB-1F3FB"},
    {"jõutõstja_värvi2", "1F3CB-1F3FC"},
    {"jõutõstja_värvi3", "1F3CB-1F3FD"},
    {"jõutõstja_värvi4", "1F3CB-1F3FE"},
    {"jõutõstja_värvi5", "1F3CB-1F3FF"},
    {"golfimängija", "1F3CC"},
    {"golfimängija_värvi1", "1F3CC-1F3FB"},
    {"golfimängija_värvi2", "1F3CC-1F3FC"},
    {"golfimängija_värvi3", "1F3CC-1F3FD"},
    {"golfimängija_värvi4", "1F3CC-1F3FE"},
    {"golfimängija_värvi5", "1F3CC-1F3FF"},
    {"võidusõiduauto", "1F3CE"},
    {"võrkpall", "1F3D0"},
    {"jäähoki", "1F3D2"},
    {"lumine_mägi", "1F3D4"},
    {"päikesevari_rannas", "1F3D6"},
    {"mahajäetud_maja", "1F3DA"},
    {"kõrb", "1F3DC"},
    {"härg", "1F402"},
    {"vesipühvel", "1F403"},
    {"jänes", "1F407"},
    {"röövik", "1F41B"},
    {"väike_tibu", "1F424"},
    {"väike_tibu_eestvaates", "1F425"},
    {"kahe_küüruga_kaamel", "1F42B"},
    {"hiire_nägu", "1F42D"},
    {"lehma_nägu", "1F42E"},
    {"tiigri_nägu", "1F42F"},
    {"jänese_nägu", "1F430"},
    {"kassi_nägu", "1F431"},
    {"draakoni_nägu", "1F432"},
    {"hobuse_nägu", "1F434"},
    {"ahvi_nägu", "1F435"},
    {"koera_nägu", "1F436"},
    {"sea_nägu", "1F437"},
    {"sea_kärss", "1F43D"},
    {"käpajäljed", "1F43E"},
    {"vöötorav", "1F43F"},
    {"kõrv", "1F442"},
    {"kõrv_värvi1", "1F442-1F3FB"},
    {"kõrv_värvi2", "1F442-1F3FC"},
    {"kõrv_värvi3", "1F442-1F3FD"},
    {"kõrv_värvi4", "1F442-1F3FE"},
    {"kõrv_värvi5", "1F442-1F3FF"},
    {"üles_osutav_sõrm_tagantvaates", "1F446"},
    {"üles_osutav_sõrm_tagantvaates_värvi1", "1F446-1F3FB"},
    {"üles_osutav_sõrm_tagantvaates_värvi2", "1F446-1F3FC"},
    {"üles_osutav_sõrm_tagantvaates_värvi3", "1F446-1F3FD"},
    {"üles_osutav_sõrm_tagantvaates_värvi4", "1F446-1F3FE"},
    {"üles_osutav_sõrm_tagantvaates_värvi5", "1F446-1F3FF"},
    {"alla_osutav_sõrm", "1F447"},
    {"alla_osutav_sõrm_värvi1", "1F447-1F3FB"},
    {"alla_osutav_sõrm_värvi2", "1F447-1F3FC"},
    {"alla_osutav_sõrm_värvi3", "1F447-1F3FD"},
    {"alla_osutav_sõrm_värvi4", "1F447-1F3FE"},
    {"alla_osutav_sõrm_värvi5", "1F447-1F3FF"},
    {"vasakule_osutav_sõrm", "1F448"},
    {"vasakule_osutav_sõrm_värvi1", "1F448-1F3FB"},
    {"vasakule_osutav_sõrm_värvi2", "1F448-1F3FC"},
    {"vasakule_osutav_sõrm_värvi3", "1F448-1F3FD"},
    {"vasakule_osutav_sõrm_värvi4", "1F448-1F3FE"},
    {"vasakule_osutav_sõrm_värvi5", "1F448-1F3FF"},
    {"paremale_osutav_sõrm", "1F449"},
    {"paremale_osutav_sõrm_värvi1", "1F449-1F3FB"},
    {"paremale_osutav_sõrm_värvi2", "1F449-1F3FC"},
    {"paremale_osutav_sõrm_värvi3", "1F449-1F3FD"},
    {"paremale_osutav_sõrm_värvi4", "1F449-1F3FE"},
    {"paremale_osutav_sõrm_värvi5", "1F449-1F3FF"},
    {"rusikalöök", "1F44A"},
    {"rusikalöök_värvi1", "1F44A-1F3FB"},
    {"rusikalöök_värvi2", "1F44A-1F3FC"},
    {"rusikalöök_värvi3", "1F44A-1F3FD"},
    {"rusikalöök_värvi4", "1F44A-1F3FE"},
    {"rusikalöök_värvi5", "1F44A-1F3FF"},
    {"lehvitav_käsi", "1F44B"},
    {"lehvitav_käsi_värvi1", "1F44B-1F3FB"},
    {"lehvitav_käsi_värvi2", "1F44B-1F3FC"},
    {"lehvitav_käsi_värvi3", "1F44B-1F3FD"},
    {"lehvitav_käsi_värvi4", "1F44B-1F3FE"},
    {"lehvitav_käsi_värvi5", "1F44B-1F3FF"},
    {"ok_käemärk", "1F44C"},
    {"ok_käemärk_värvi1", "1F44C-1F3FB"},
    {"ok_käemärk_värvi2", "1F44C-1F3FC"},
    {"ok_käemärk_värvi3", "1F44C-1F3FD"},
    {"ok_käemärk_värvi4", "1F44C-1F3FE"},
    {"ok_käemärk_värvi5", "1F44C-1F3FF"},
    {"plaksutavad_käed", "1F44F"},
    {"plaksutavad_käed_värvi1", "1F44F-1F3FB"},
    {"plaksutavad_käed_värvi2", "1F44F-1F3FC"},
    {"plaksutavad_käed_värvi3", "1F44F-1F3FD"},
    {"plaksutavad_käed_värvi4", "1F44F-1F3FE"},
    {"plaksutavad_käed_värvi5", "1F44F-1F3FF"},
    {"avatud_käed", "1F450"},
    {"avatud_käed_värvi1", "1F450-1F3FB"},
    {"avatud_käed_värvi2", "1F450-1F3FC"},
    {"avatud_käed_värvi3", "1F450-1F3FD"},
    {"avatud_käed_värvi4", "1F450-1F3FE"},
    {"avatud_käed_värvi5", "1F450-1F3FF"},
    {"naistekübar", "1F452"},
    {"t_särk", "1F455"},
    {"ridikül", "1F45B"},
    {"käekott", "1F45C"},
    {"kõrge_kontsaga_king", "1F460"},
    {"jalajäljed", "1F463"},
    {"büsti_siluett", "1F464"},
    {"büstide_siluetid", "1F465"},
    {"tüdruk", "1F467"},
    {"tüdruk_värvi1", "1F467-1F3FB"},
    {"tüdruk_värvi2", "1F467-1F3FC"},
    {"tüdruk_värvi3", "1F467-1F3FD"},
    {"tüdruk_värvi4", "1F467-1F3FE"},
    {"tüdruk_värvi5", "1F467-1F3FF"},
    {"mees_ja_naine_hoiavad_käest_kinni", "1F46B"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi1", "1F46B-1F3FB"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi2", "1F46B-1F3FC"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi3", "1F46B-1F3FD"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi4", "1F46B-1F3FE"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi5", "1F46B-1F3FF"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mees_ja_naine_hoiavad_käest_kinni_värvi5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"käest_kinni_hoidvad_mehed", "1F46C"},
    {"käest_kinni_hoidvad_mehed_värvi1", "1F46C-1F3FB"},
    {"käest_kinni_hoidvad_mehed_värvi2", "1F46C-1F3FC"},
    {"käest_kinni_hoidvad_mehed_värvi3", "1F46C-1F3FD"},
    {"käest_kinni_hoidvad_mehed_värvi4", "1F46C-1F3FE"},
    {"käest_kinni_hoidvad_mehed_värvi5", "1F46C-1F3FF"},
    {"käest_kinni_hoidvad_mehed_värvi1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"käest_kinni_hoidvad_mehed_värvi1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"käest_kinni_hoidvad_mehed_värvi1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"käest_kinni_hoidvad_mehed_värvi1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"käest_kinni_hoidvad_mehed_värvi2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"käest_kinni_hoidvad_mehed_värvi2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"käest_kinni_hoidvad_mehed_värvi2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"käest_kinni_hoidvad_mehed_värvi2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"käest_kinni_hoidvad_mehed_värvi3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"käest_kinni_hoidvad_mehed_värvi3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"käest_kinni_hoidvad_mehed_värvi3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"käest_kinni_hoidvad_mehed_värvi3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"käest_kinni_hoidvad_mehed_värvi4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"käest_kinni_hoidvad_mehed_värvi4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"käest_kinni_hoidvad_mehed_värvi4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"käest_kinni_hoidvad_mehed_värvi4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"käest_kinni_hoidvad_mehed_värvi5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"käest_kinni_hoidvad_mehed_värvi5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"käest_kinni_hoidvad_mehed_värvi5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"käest_kinni_hoidvad_mehed_värvi5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"käest_kinni_hoidvad_naised", "1F46D"},
    {"käest_kinni_hoidvad_naised_värvi1", "1F46D-1F3FB"},
    {"käest_kinni_hoidvad_naised_värvi2", "1F46D-1F3FC"},
    {"käest_kinni_hoidvad_naised_värvi3", "1F46D-1F3FD"},
    {"käest_kinni_hoidvad_naised_värvi4", "1F46D-1F3FE"},
    {"käest_kinni_hoidvad_naised_värvi5", "1F46D-1F3FF"},
    {"käest_kinni_hoidvad_naised_värvi1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"käest_kinni_hoidvad_naised_värvi1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"käest_kinni_hoidvad_naised_värvi1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"käest_kinni_hoidvad_naised_värvi1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"käest_kinni_hoidvad_naised_värvi2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"käest_kinni_hoidvad_naised_värvi2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"käest_kinni_hoidvad_naised_värvi2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"käest_kinni_hoidvad_naised_värvi2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"käest_kinni_hoidvad_naised_värvi3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"käest_kinni_hoidvad_naised_värvi3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"käest_kinni_hoidvad_naised_värvi3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"käest_kinni_hoidvad_naised_värvi3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"käest_kinni_hoidvad_naised_värvi4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"käest_kinni_hoidvad_naised_värvi4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"käest_kinni_hoidvad_naised_värvi4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"käest_kinni_hoidvad_naised_värvi4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"käest_kinni_hoidvad_naised_värvi5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"käest_kinni_hoidvad_naised_värvi5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"käest_kinni_hoidvad_naised_värvi5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"käest_kinni_hoidvad_naised_värvi5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"pidutsevad_jänesekõrvadega_inimesed", "1F46F"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi1", "1F46F-1F3FB"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi2", "1F46F-1F3FC"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi3", "1F46F-1F3FD"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi4", "1F46F-1F3FE"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi5", "1F46F-1F3FF"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"pidutsevad_jänesekõrvadega_inimesed_värvi5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"hiina_mütsiga_mees", "1F472"},
    {"hiina_mütsiga_mees_värvi1", "1F472-1F3FB"},
    {"hiina_mütsiga_mees_värvi2", "1F472-1F3FC"},
    {"hiina_mütsiga_mees_värvi3", "1F472-1F3FD"},
    {"hiina_mütsiga_mees_värvi4", "1F472-1F3FE"},
    {"hiina_mütsiga_mees_värvi5", "1F472-1F3FF"},
    {"mäekoll", "1F47A"},
    {"infoleti_töötaja", "1F481"},
    {"infoleti_töötaja_värvi1", "1F481-1F3FB"},
    {"infoleti_töötaja_värvi2", "1F481-1F3FC"},
    {"infoleti_töötaja_värvi3", "1F481-1F3FD"},
    {"infoleti_töötaja_värvi4", "1F481-1F3FE"},
    {"infoleti_töötaja_värvi5", "1F481-1F3FF"},
    {"küünelakk", "1F485"},
    {"küünelakk_värvi1", "1F485-1F3FB"},
    {"küünelakk_värvi2", "1F485-1F3FC"},
    {"küünelakk_värvi3", "1F485-1F3FD"},
    {"küünelakk_värvi4", "1F485-1F3FE"},
    {"küünelakk_värvi5", "1F485-1F3FF"},
    {"näomassaaž", "1F486"},
    {"näomassaaž_värvi1", "1F486-1F3FB"},
    {"näomassaaž_värvi2", "1F486-1F3FC"},
    {"näomassaaž_värvi3", "1F486-1F3FD"},
    {"näomassaaž_värvi4", "1F486-1F3FE"},
    {"näomassaaž_värvi5", "1F486-1F3FF"},
    {"juukselõikus", "1F487"},
    {"juukselõikus_värvi1", "1F487-1F3FB"},
    {"juukselõikus_värvi2", "1F487-1F3FC"},
    {"juukselõikus_värvi3", "1F487-1F3FD"},
    {"juukselõikus_värvi4", "1F487-1F3FE"},
    {"juukselõikus_värvi5", "1F487-1F3FF"},
    {"habemeajaja_tähis", "1F488"},
    {"süstal", "1F489"},
    {"suudluse_jälg", "1F48B"},
    {"sõrmus", "1F48D"},
    {"paarike_südamega", "1F491"},
    {"paarike_südamega_värvi1", "1F491-1F3FB"},
    {"paarike_südamega_värvi2", "1F491-1F3FC"},
    {"paarike_südamega_värvi3", "1F491-1F3FD"},
    {"paarike_südamega_värvi4", "1F491-1F3FE"},
    {"paarike_südamega_värvi5", "1F491-1F3FF"},
    {"paarike_südamega_värvi1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"paarike_südamega_värvi1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"paarike_südamega_värvi1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"paarike_südamega_värvi1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"paarike_südamega_värvi2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"paarike_südamega_värvi2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"paarike_südamega_värvi2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"paarike_südamega_värvi2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"paarike_südamega_värvi3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"paarike_südamega_värvi3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"paarike_südamega_värvi3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"paarike_südamega_värvi3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"paarike_südamega_värvi4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"paarike_südamega_värvi4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"paarike_südamega_värvi4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"paarike_südamega_värvi4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"paarike_südamega_värvi5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"paarike_südamega_värvi5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"paarike_südamega_värvi5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"paarike_südamega_värvi5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"tukslev_süda", "1F493"},
    {"murtud_süda", "1F494"},
    {"kaks_südant", "1F495"},
    {"sädelev_süda", "1F496"},
    {"kasvav_süda", "1F497"},
    {"noolega_süda", "1F498"},
    {"sinine_süda", "1F499"},
    {"roheline_süda", "1F49A"},
    {"kollane_süda", "1F49B"},
    {"lilla_süda", "1F49C"},
    {"lindiga_süda", "1F49D"},
    {"pöörlevad_südamed", "1F49E"},
    {"südamekujuline_kaunistus", "1F49F"},
    {"viha_sümbol", "1F4A2"},
    {"peadpööritav", "1F4AB"},
    {"mõttemull", "1F4AD"},
    {"suur_dollarimärk", "1F4B2"},
    {"jeeni_rahatäht", "1F4B4"},
    {"dollari_rahatäht", "1F4B5"},
    {"euro_rahatäht", "1F4B6"},
    {"naela_rahatäht", "1F4B7"},
    {"tõusev_graafik_jeenisümboliga", "1F4B9"},
    {"sülearvuti", "1F4BB"},
    {"tõusev_graafik", "1F4C8"},
    {"nööpnõel", "1F4CC"},
    {"ümara_peaga_nööpnõel", "1F4CD"},
    {"järjehoidjad", "1F4D1"},
    {"päevik", "1F4D3"},
    {"dekoratiivse_kaanega_märkmik", "1F4D4"},
    {"oranž_raamat", "1F4D9"},
    {"käsikiri", "1F4DC"},
    {"valjuhääldi", "1F4E2"},
    {"väljaminev_post", "1F4E4"},
    {"meilisõnum", "1F4E7"},
    {"sissetulev_ümbrik", "1F4E8"},
    {"noolega_ümbrik", "1F4E9"},
    {"suletud_postkast_tõstetud_lipuga", "1F4EB"},
    {"avatud_postkast_tõstetud_lipuga", "1F4EC"},
    {"värinarežiim", "1F4F3"},
    {"väljalülitatud_mobiiltelefon", "1F4F4"},
    {"välguga_kaamera", "1F4F8"},
    {"juhusliku_järjekorra_nupp", "1F500"},
    {"ühe_korduse_nupp", "1F502"},
    {"vertikaalsed_nooled_päripäeva", "1F503"},
    {"nooled_vastupäeva", "1F504"},
    {"vaigistatud_kõlar", "1F507"},
    {"vaikse_heliga_kõlar", "1F508"},
    {"poolvalju_heliga_kõlar", "1F509"},
    {"valju_heliga_kõlar", "1F50A"},
    {"suletud_lukk_võtmega", "1F510"},
    {"võti", "1F511"},
    {"järjehoidja", "1F516"},
    {"lüli", "1F517"},
    {"ladina_suurtähed", "1F520"},
    {"ladina_väiketähed", "1F521"},
    {"sümbolid", "1F523"},
    {"ladina_tähed", "1F524"},
    {"mutrivõti", "1F527"},
    {"kööginuga", "1F52A"},
    {"veepüstol", "1F52B"},
    {"punktiga_kuusnurkne_täht", "1F52F"},
    {"algaja_sümbol", "1F530"},
    {"kolmhargi_sümbol", "1F531"},
    {"suur_oranž_romb", "1F536"},
    {"väike_oranž_romb", "1F538"},
    {"väike_sinine_romb", "1F539"},
    {"ülesnupp", "1F53C"},
    {"mošee", "1F54C"},
    {"sünagoog", "1F54D"},
    {"kell_üks", "1F550"},
    {"kell_üheksa", "1F558"},
    {"kell_kümme", "1F559"},
    {"kell_üksteist", "1F55A"},
    {"kell_pool_üheksa", "1F563"},
    {"kell_pool_kümme", "1F564"},
    {"kell_pool_üksteist", "1F565"},
    {"kell_pool_üks", "1F567"},
    {"küünal", "1F56F"},
    {"hõljuv_ülikonnaga_mees", "1F574"},
    {"hõljuv_ülikonnaga_mees_värvi1", "1F574-1F3FB"},
    {"hõljuv_ülikonnaga_mees_värvi2", "1F574-1F3FC"},
    {"hõljuv_ülikonnaga_mees_värvi3", "1F574-1F3FD"},
    {"hõljuv_ülikonnaga_mees_värvi4", "1F574-1F3FE"},
    {"hõljuv_ülikonnaga_mees_värvi5", "1F574-1F3FF"},
    {"päikeseprillid", "1F576"},
    {"ämblik", "1F577"},
    {"ämblikuvõrk", "1F578"},
    {"ühendatud_kirjaklambrid", "1F587"},
    {"harali_sõrmedega_käsi", "1F590"},
    {"harali_sõrmedega_käsi_värvi1", "1F590-1F3FB"},
    {"harali_sõrmedega_käsi_värvi2", "1F590-1F3FC"},
    {"harali_sõrmedega_käsi_värvi3", "1F590-1F3FD"},
    {"harali_sõrmedega_käsi_värvi4", "1F590-1F3FE"},
    {"harali_sõrmedega_käsi_värvi5", "1F590-1F3FF"},
    {"keskmine_sõrm", "1F595"},
    {"keskmine_sõrm_värvi1", "1F595-1F3FB"},
    {"keskmine_sõrm_värvi2", "1F595-1F3FC"},
    {"keskmine_sõrm_värvi3", "1F595-1F3FD"},
    {"keskmine_sõrm_värvi4", "1F595-1F3FE"},
    {"keskmine_sõrm_värvi5", "1F595-1F3FF"},
    {"must_süda", "1F5A4"},
    {"prügikorv", "1F5D1"},
    {"spiraalmärkmik", "1F5D2"},
    {"vana_võti", "1F5DD"},
    {"kõnelev_pea", "1F5E3"},
    {"hääletuskast", "1F5F3"},
    {"fuji_mägi", "1F5FB"},
    {"laia_naeratusega_nägu", "1F600"},
    {"naerul_silmade_ja_laia_naeratusega_nägu", "1F601"},
    {"rõõmupisaratega_nägu", "1F602"},
    {"suurte_silmadega_naerunägu", "1F603"},
    {"avatud_suu_ja_naeratavate_silmadega_naerunägu", "1F604"},
    {"higipiisa_ja_avatud_suuga_naerunägu", "1F605"},
    {"avatud_suu_ja_suletud_silmadega_naerunägu", "1F606"},
    {"aupaistega_naerunägu", "1F607"},
    {"sarviline_naerunägu", "1F608"},
    {"silma_pilgutav_nägu", "1F609"},
    {"naeratavate_silmadega_naerunägu", "1F60A"},
    {"maitsvat_toitu_nautiv_nägu", "1F60B"},
    {"kergendust_tundev_nägu", "1F60C"},
    {"südamekujuliste_silmadega_naerunägu", "1F60D"},
    {"päikeseprillidega_naerunägu", "1F60E"},
    {"muigav_nägu", "1F60F"},
    {"erapooletu_nägu", "1F610"},
    {"ilmetu_nägu", "1F611"},
    {"põlglik_nägu", "1F612"},
    {"külma_higiga_kaetud_nägu", "1F613"},
    {"mõtlik_nägu", "1F614"},
    {"segaduses_nägu", "1F615"},
    {"kimbatuses_nägu", "1F616"},
    {"suudlev_nägu", "1F617"},
    {"suudlust_saatev_nägu", "1F618"},
    {"naeratavate_silmadega_suudlev_nägu", "1F619"},
    {"suletud_silmadega_suudlev_nägu", "1F61A"},
    {"suust_väljas_keelega_nägu", "1F61B"},
    {"suust_väljas_keelega_ja_silma_pilgutav_nägu", "1F61C"},
    {"keelt_näitav_suletud_silmadega_nägu", "1F61D"},
    {"pettunud_nägu", "1F61E"},
    {"murelik_nägu", "1F61F"},
    {"tige_nägu", "1F620"},
    {"mossitav_nägu", "1F621"},
    {"nuttev_nägu", "1F622"},
    {"sihikindel_nägu", "1F623"},
    {"ninast_auru_ajav_nägu", "1F624"},
    {"pettunud_kuid_kergendust_tundev_nägu", "1F625"},
    {"avatud_suuga_mossitav_nägu", "1F626"},
    {"ahastuses_nägu", "1F627"},
    {"hirmunud_nägu", "1F628"},
    {"rammestunud_nägu", "1F629"},
    {"unine_nägu", "1F62A"},
    {"väsinud_nägu", "1F62B"},
    {"grimassitav_nägu", "1F62C"},
    {"valjusti_nuttev_nägu", "1F62D"},
    {"avatud_suuga_nägu", "1F62E"},
    {"vaigistatud_nägu", "1F62F"},
    {"külma_higiga_kaetud_ja_avatud_suuga_nägu", "1F630"},
    {"hirmunult_karjuv_nägu", "1F631"},
    {"hämmastunud_nägu", "1F632"},
    {"punastav_nägu", "1F633"},
    {"magav_nägu", "1F634"},
    {"oimetu_nägu", "1F635"},
    {"ilma_suuta_nägu", "1F636"},
    {"kirurgimaskiga_kaetud_nägu", "1F637"},
    {"lõbusate_silmadega_irvitav_kass", "1F638"},
    {"rõõmupisarais_kass", "1F639"},
    {"südamekujuliste_silmadega_naeratav_kass", "1F63B"},
    {"üllatunud_kass", "1F640"},
    {"veidi_kulmu_kortsutav_nägu", "1F641"},
    {"naeratusega_nägu", "1F642"},
    {"tagurpidi_nägu", "1F643"},
    {"pööritavate_silmadega_nägu", "1F644"},
    {"eitav_žest", "1F645"},
    {"eitav_žest_värvi1", "1F645-1F3FB"},
    {"eitav_žest_värvi2", "1F645-1F3FC"},
    {"eitav_žest_värvi3", "1F645-1F3FD"},
    {"eitav_žest_värvi4", "1F645-1F3FE"},
    {"eitav_žest_värvi5", "1F645-1F3FF"},
    {"nõustuv_žest", "1F646"},
    {"nõustuv_žest_värvi1", "1F646-1F3FB"},
    {"nõustuv_žest_värvi2", "1F646-1F3FC"},
    {"nõustuv_žest_värvi3", "1F646-1F3FD"},
    {"nõustuv_žest_värvi4", "1F646-1F3FE"},
    {"nõustuv_žest_värvi5", "1F646-1F3FF"},
    {"kaetud_kõrvadega_ahv", "1F649"},
    {"kätt_tõstev_rõõmus_inimene", "1F64B"},
    {"kätt_tõstev_rõõmus_inimene_värvi1", "1F64B-1F3FB"},
    {"kätt_tõstev_rõõmus_inimene_värvi2", "1F64B-1F3FC"},
    {"kätt_tõstev_rõõmus_inimene_värvi3", "1F64B-1F3FD"},
    {"kätt_tõstev_rõõmus_inimene_värvi4", "1F64B-1F3FE"},
    {"kätt_tõstev_rõõmus_inimene_värvi5", "1F64B-1F3FF"},
    {"ülestõstetud_käed", "1F64C"},
    {"ülestõstetud_käed_värvi1", "1F64C-1F3FB"},
    {"ülestõstetud_käed_värvi2", "1F64C-1F3FC"},
    {"ülestõstetud_käed_värvi3", "1F64C-1F3FD"},
    {"ülestõstetud_käed_värvi4", "1F64C-1F3FE"},
    {"ülestõstetud_käed_värvi5", "1F64C-1F3FF"},
    {"palveasendis_käed", "1F64F"},
    {"palveasendis_käed_värvi1", "1F64F-1F3FB"},
    {"palveasendis_käed_värvi2", "1F64F-1F3FC"},
    {"palveasendis_käed_värvi3", "1F64F-1F3FD"},
    {"palveasendis_käed_värvi4", "1F64F-1F3FE"},
    {"palveasendis_käed_värvi5", "1F64F-1F3FF"},
    {"väikebuss", "1F690"},
    {"tuletõrjeauto", "1F692"},
    {"üherööpmeline_raudtee", "1F69D"},
    {"mägiraudtee", "1F69E"},
    {"köistee_mägedes", "1F6A0"},
    {"köisraudtee", "1F6A1"},
    {"ehitustööd", "1F6A7"},
    {"prügikasti_silt", "1F6AE"},
    {"joomiseks_kõlbmatu_vesi", "1F6B1"},
    {"mägirattur", "1F6B5"},
    {"mägirattur_värvi1", "1F6B5-1F3FB"},
    {"mägirattur_värvi2", "1F6B5-1F3FC"},
    {"mägirattur_värvi3", "1F6B5-1F3FD"},
    {"mägirattur_värvi4", "1F6B5-1F3FE"},
    {"mägirattur_värvi5", "1F6B5-1F3FF"},
    {"jalakäija", "1F6B6"},
    {"jalakäija_värvi1", "1F6B6-1F3FB"},
    {"jalakäija_värvi2", "1F6B6-1F3FC"},
    {"jalakäija_värvi3", "1F6B6-1F3FD"},
    {"jalakäija_värvi4", "1F6B6-1F3FE"},
    {"jalakäija_värvi5", "1F6B6-1F3FF"},
    {"jalakäijad_keelatud", "1F6B7"},
    {"beebi_sümbol", "1F6BC"},
    {"dušš", "1F6BF"},
    {"pagasi_kättesaamine", "1F6C4"},
    {"märgukell", "1F6CE"},
    {"pühakoda", "1F6D0"},
    {"stoppmärk", "1F6D1"},
    {"ostukäru", "1F6D2"},
    {"hütt", "1F6D6"},
    {"liumägi", "1F6DD"},
    {"päästerõngas", "1F6DF"},
    {"haamer_ja_mutrivõti", "1F6E0"},
    {"õlitünn", "1F6E2"},
    {"raudteerööpad", "1F6E4"},
    {"väikelennuk", "1F6E9"},
    {"õhkutõusev_lennuk", "1F6EB"},
    {"tõukeratas", "1F6F4"},
    {"autorikša", "1F6FA"},
    {"oranž_ring", "1F7E0"},
    {"oranž_ruut", "1F7E7"},
    {"tugev_võrdusmärk", "1F7F0"},
    {"kokkusurutud_sõrmed", "1F90C"},
    {"kokkusurutud_sõrmed_värvi1", "1F90C-1F3FB"},
    {"kokkusurutud_sõrmed_värvi2", "1F90C-1F3FC"},
    {"kokkusurutud_sõrmed_värvi3", "1F90C-1F3FD"},
    {"kokkusurutud_sõrmed_värvi4", "1F90C-1F3FE"},
    {"kokkusurutud_sõrmed_värvi5", "1F90C-1F3FF"},
    {"valge_süda", "1F90D"},
    {"pruun_süda", "1F90E"},
    {"näpistav_käsi", "1F90F"},
    {"näpistav_käsi_värvi1", "1F90F-1F3FB"},
    {"näpistav_käsi_värvi2", "1F90F-1F3FC"},
    {"näpistav_käsi_värvi3", "1F90F-1F3FD"},
    {"näpistav_käsi_värvi4", "1F90F-1F3FE"},
    {"näpistav_käsi_värvi5", "1F90F-1F3FF"},
    {"tõmblukusuuga_nägu", "1F910"},
    {"nägu_millel_on_raha_suus", "1F911"},
    {"kraadiklaasiga_nägu", "1F912"},
    {"nohiku_nägu", "1F913"},
    {"mõtlev_nägu", "1F914"},
    {"kinniseotud_peaga_nägu", "1F915"},
    {"kallistav_nägu", "1F917"},
    {"sarvede_märk", "1F918"},
    {"sarvede_märk_värvi1", "1F918-1F3FB"},
    {"sarvede_märk_värvi2", "1F918-1F3FC"},
    {"sarvede_märk_värvi3", "1F918-1F3FD"},
    {"sarvede_märk_värvi4", "1F918-1F3FE"},
    {"sarvede_märk_värvi5", "1F918-1F3FF"},
    {"tõstetud_käsi", "1F91A"},
    {"tõstetud_käsi_värvi1", "1F91A-1F3FB"},
    {"tõstetud_käsi_värvi2", "1F91A-1F3FC"},
    {"tõstetud_käsi_värvi3", "1F91A-1F3FD"},
    {"tõstetud_käsi_värvi4", "1F91A-1F3FE"},
    {"tõstetud_käsi_värvi5", "1F91A-1F3FF"},
    {"käepigistus", "1F91D"},
    {"käepigistus_värvi1", "1F91D-1F3FB"},
    {"käepigistus_värvi2", "1F91D-1F3FC"},
    {"käepigistus_värvi3", "1F91D-1F3FD"},
    {"käepigistus_värvi4", "1F91D-1F3FE"},
    {"käepigistus_värvi5", "1F91D-1F3FF"},
    {"käepigistus_värvi1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"käepigistus_värvi1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"käepigistus_värvi1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"käepigistus_värvi1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"käepigistus_värvi2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"käepigistus_värvi2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"käepigistus_värvi2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"käepigistus_värvi2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"käepigistus_värvi3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"käepigistus_värvi3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"käepigistus_värvi3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"käepigistus_värvi3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"käepigistus_värvi4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"käepigistus_värvi4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"käepigistus_värvi4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"käepigistus_värvi4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"käepigistus_värvi5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"käepigistus_värvi5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"käepigistus_värvi5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"käepigistus_värvi5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"sõrmed_risti", "1F91E"},
    {"sõrmed_risti_värvi1", "1F91E-1F3FB"},
    {"sõrmed_risti_värvi2", "1F91E-1F3FC"},
    {"sõrmed_risti_värvi3", "1F91E-1F3FD"},
    {"sõrmed_risti_värvi4", "1F91E-1F3FE"},
    {"sõrmed_risti_värvi5", "1F91E-1F3FF"},
    {"armastust_väljendav_žest", "1F91F"},
    {"armastust_väljendav_žest_värvi1", "1F91F-1F3FB"},
    {"armastust_väljendav_žest_värvi2", "1F91F-1F3FC"},
    {"armastust_väljendav_žest_värvi3", "1F91F-1F3FD"},
    {"armastust_väljendav_žest_värvi4", "1F91F-1F3FE"},
    {"armastust_väljendav_žest_värvi5", "1F91F-1F3FF"},
    {"kauboimütsiga_nägu", "1F920"},
    {"klouninägu", "1F921"},
    {"iiveldust_tundev_nägu", "1F922"},
    {"naerust_kõveras", "1F923"},
    {"ilastav_nägu", "1F924"},
    {"valetav_nägu", "1F925"},
    {"aevastav_nägu", "1F927"},
    {"kergitatud_kulmudega_nägu", "1F928"},
    {"pöörase_ilmega_nägu", "1F92A"},
    {"sõrme_suul_hoidev_nägu", "1F92B"},
    {"suul_olevate_sümbolitega_nägu", "1F92C"},
    {"kätt_suul_hoidev_nägu", "1F92D"},
    {"oksendav_nägu", "1F92E"},
    {"peopesad_üleval_koos", "1F932"},
    {"peopesad_üleval_koos_värvi1", "1F932-1F3FB"},
    {"peopesad_üleval_koos_värvi2", "1F932-1F3FC"},
    {"peopesad_üleval_koos_värvi3", "1F932-1F3FD"},
    {"peopesad_üleval_koos_värvi4", "1F932-1F3FE"},
    {"peopesad_üleval_koos_värvi5", "1F932-1F3FF"},
    {"jõuluvana_abikaasa", "1F936"},
    {"jõuluvana_abikaasa_värvi1", "1F936-1F3FB"},
    {"jõuluvana_abikaasa_värvi2", "1F936-1F3FC"},
    {"jõuluvana_abikaasa_värvi3", "1F936-1F3FD"},
    {"jõuluvana_abikaasa_värvi4", "1F936-1F3FE"},
    {"jõuluvana_abikaasa_värvi5", "1F936-1F3FF"},
    {"õlakehitus", "1F937"},
    {"õlakehitus_värvi1", "1F937-1F3FB"},
    {"õlakehitus_värvi2", "1F937-1F3FC"},
    {"õlakehitus_värvi3", "1F937-1F3FD"},
    {"õlakehitus_värvi4", "1F937-1F3FE"},
    {"õlakehitus_värvi5", "1F937-1F3FF"},
    {"žonglöör", "1F939"},
    {"žonglöör_värvi1", "1F939-1F3FB"},
    {"žonglöör_värvi2", "1F939-1F3FC"},
    {"žonglöör_värvi3", "1F939-1F3FD"},
    {"žonglöör_värvi4", "1F939-1F3FE"},
    {"žonglöör_värvi5", "1F939-1F3FF"},
    {"veepalli_mängiv_inimene", "1F93D"},
    {"veepalli_mängiv_inimene_värvi1", "1F93D-1F3FB"},
    {"veepalli_mängiv_inimene_värvi2", "1F93D-1F3FC"},
    {"veepalli_mängiv_inimene_värvi3", "1F93D-1F3FD"},
    {"veepalli_mängiv_inimene_värvi4", "1F93D-1F3FE"},
    {"veepalli_mängiv_inimene_värvi5", "1F93D-1F3FF"},
    {"käsipalli_mängiv_inimene", "1F93E"},
    {"käsipalli_mängiv_inimene_värvi1", "1F93E-1F3FB"},
    {"käsipalli_mängiv_inimene_värvi2", "1F93E-1F3FC"},
    {"käsipalli_mängiv_inimene_värvi3", "1F93E-1F3FD"},
    {"käsipalli_mängiv_inimene_värvi4", "1F93E-1F3FE"},
    {"käsipalli_mängiv_inimene_värvi5", "1F93E-1F3FF"},
    {"närbunud_lill", "1F940"},
    {"kõlisevad_klaasid", "1F942"},
    {"väravavõrk", "1F945"},
    {"hõbemedal", "1F948"},
    {"võitluskunstide_riietus", "1F94B"},
    {"jääkeegli_kivi", "1F94C"},
    {"täidetud_pita", "1F959"},
    {"maapähklid", "1F95C"},
    {"õnneküpsis", "1F960"},
    {"kaasamüügikarp", "1F961"},
    {"söögipulgad", "1F962"},
    {"kõrrega_tops", "1F964"},
    {"kookospähkel", "1F965"},
    {"lihalõik", "1F969"},
    {"võileib", "1F96A"},
    {"lehtköögivili", "1F96C"},
    {"rõngassai", "1F96F"},
    {"naeratav_südametega_nägu", "1F970"},
    {"haigutav_nägu", "1F971"},
    {"naeratav_pisaraga_nägu", "1F972"},
    {"pidutsev_nägu", "1F973"},
    {"uimane_nägu", "1F974"},
    {"higistav_nägu", "1F975"},
    {"külmunud_nägu", "1F976"},
    {"maskeeritud_nägu", "1F978"},
    {"pisaraid_tagasi_hoidev_nägu", "1F979"},
    {"anuv_nägu", "1F97A"},
    {"lõvi_nägu", "1F981"},
    {"ükssarvik", "1F984"},
    {"öökull", "1F989"},
    {"türannosaurus", "1F996"},
    {"känguru", "1F998"},
    {"jõehobu", "1F99B"},
    {"sääsk", "1F99F"},
    {"mäger", "1F9A1"},
    {"hüljes", "1F9AD"},
    {"kuuldeaparaadiga_kõrv", "1F9BB"},
    {"kuuldeaparaadiga_kõrv_värvi1", "1F9BB-1F3FB"},
    {"kuuldeaparaadiga_kõrv_värvi2", "1F9BB-1F3FC"},
    {"kuuldeaparaadiga_kõrv_värvi3", "1F9BB-1F3FD"},
    {"kuuldeaparaadiga_kõrv_värvi4", "1F9BB-1F3FE"},
    {"kuuldeaparaadiga_kõrv_värvi5", "1F9BB-1F3FF"},
    {"mehaaniline_käsi", "1F9BE"},
    {"juustutükk", "1F9C0"},
    {"küüslauk", "1F9C4"},
    {"või", "1F9C8"},
    {"jääkuubik", "1F9CA"},
    {"pärlitee", "1F9CB"},
    {"põlvitav_inimene", "1F9CE"},
    {"põlvitav_inimene_värvi1", "1F9CE-1F3FB"},
    {"põlvitav_inimene_värvi2", "1F9CE-1F3FC"},
    {"põlvitav_inimene_värvi3", "1F9CE-1F3FD"},
    {"põlvitav_inimene_värvi4", "1F9CE-1F3FE"},
    {"põlvitav_inimene_värvi5", "1F9CE-1F3FF"},
    {"monokliga_nägu", "1F9D0"},
    {"täiskasvanu", "1F9D1"},
    {"täiskasvanu_värvi1", "1F9D1-1F3FB"},
    {"täiskasvanu_värvi2", "1F9D1-1F3FC"},
    {"täiskasvanu_värvi3", "1F9D1-1F3FD"},
    {"täiskasvanu_värvi4", "1F9D1-1F3FE"},
    {"täiskasvanu_värvi5", "1F9D1-1F3FF"},
    {"pearätiga_naine", "1F9D5"},
    {"pearätiga_naine_värvi1", "1F9D5-1F3FB"},
    {"pearätiga_naine_värvi2", "1F9D5-1F3FC"},
    {"pearätiga_naine_värvi3", "1F9D5-1F3FD"},
    {"pearätiga_naine_värvi4", "1F9D5-1F3FE"},
    {"pearätiga_naine_värvi5", "1F9D5-1F3FF"},
    {"mäkke_roniv_inimene", "1F9D7"},
    {"mäkke_roniv_inimene_värvi1", "1F9D7-1F3FB"},
    {"mäkke_roniv_inimene_värvi2", "1F9D7-1F3FC"},
    {"mäkke_roniv_inimene_värvi3", "1F9D7-1F3FD"},
    {"mäkke_roniv_inimene_värvi4", "1F9D7-1F3FE"},
    {"mäkke_roniv_inimene_värvi5", "1F9D7-1F3FF"},
    {"näkk", "1F9DC"},
    {"näkk_värvi1", "1F9DC-1F3FB"},
    {"näkk_värvi2", "1F9DC-1F3FC"},
    {"näkk_värvi3", "1F9DC-1F3FD"},
    {"näkk_värvi4", "1F9DC-1F3FE"},
    {"näkk_värvi5", "1F9DC-1F3FF"},
    {"päkapikk", "1F9DD"},
    {"päkapikk_värvi1", "1F9DD-1F3FB"},
    {"päkapikk_värvi2", "1F9DD-1F3FC"},
    {"päkapikk_värvi3", "1F9DD-1F3FD"},
    {"päkapikk_värvi4", "1F9DD-1F3FE"},
    {"päkapikk_värvi5", "1F9DD-1F3FF"},
    {"džinn", "1F9DE"},
    {"oranž_süda", "1F9E1"},
    {"nokkmüts", "1F9E2"},
    {"punane_ümbrik", "1F9E7"},
    {"pusletükk", "1F9E9"},
    {"tööriistakast", "1F9F0"},
    {"lõng", "1F9F6"},
    {"haaknõel", "1F9F7"},
    {"mängukaru", "1F9F8"},
    {"pesukäsn", "1F9FD"},
    {"ujumispüksid", "1FA72"},
    {"šortsid", "1FA73"},
    {"helesinine_süda", "1FA75"},
    {"hall_süda", "1FA76"},
    {"roosa_süda", "1FA77"},
    {"röntgen", "1FA7B"},
    {"võlukepp", "1FA84"},
    {"piñata", "1FA85"},
    {"matrjoška", "1FA86"},
    {"flööt", "1FA88"},
    {"rõngastega_planeet", "1FA90"},
    {"bandžo", "1FA95"},
    {"sõdurikiiver", "1FA96"},
    {"münt", "1FA99"},
    {"käsisaag", "1FA9A"},
    {"õmblusnõel", "1FAA1"},
    {"sõlm", "1FAA2"},
    {"ämber", "1FAA3"},
    {"hiirelõks", "1FAA4"},
    {"aku_tühjeneb", "1FAAB"},
    {"kärbes", "1FAB0"},
    {"tühi_pesa", "1FAB9"},
    {"hüatsint", "1FABB"},
    {"süda", "1FAC0"},
    {"sõrmejälg", "1FAC6"},
    {"mõõkvaal", "1FACD"},
    {"põder", "1FACE"},
    {"fondüü", "1FAD5"},
    {"plärakas", "1FADF"},
    {"sulav_nägu", "1FAE0"},
    {"au_andev_nägu", "1FAE1"},
    {"kätt_suul_hoidev_pärani_silmadega_nägu", "1FAE2"},
    {"käte_vahelt_piiluva_silmaga_nägu", "1FAE3"},
    {"diagonaalse_suuga_nägu", "1FAE4"},
    {"punktiirjoonega_nägu", "1FAE5"},
    {"värisev_nägu", "1FAE8"},
    {"nägu_kottidega_silmade_all", "1FAE9"},
    {"moondunud_nägu", "1FAEA"},
    {"pöialt_ja_nimetissõrme_risti_hoidev_käsi", "1FAF0"},
    {"pöialt_ja_nimetissõrme_risti_hoidev_käsi_värvi1", "1FAF0-1F3FB"},
    {"pöialt_ja_nimetissõrme_risti_hoidev_käsi_värvi2", "1FAF0-1F3FC"},
    {"pöialt_ja_nimetissõrme_risti_hoidev_käsi_värvi3", "1FAF0-1F3FD"},
    {"pöialt_ja_nimetissõrme_risti_hoidev_käsi_värvi4", "1FAF0-1F3FE"},
    {"pöialt_ja_nimetissõrme_risti_hoidev_käsi_värvi5", "1FAF0-1F3FF"},
    {"käsi_paremale", "1FAF1"},
    {"käsi_paremale_värvi1", "1FAF1-1F3FB"},
    {"käsi_paremale_värvi2", "1FAF1-1F3FC"},
    {"käsi_paremale_värvi3", "1FAF1-1F3FD"},
    {"käsi_paremale_värvi4", "1FAF1-1F3FE"},
    {"käsi_paremale_värvi5", "1FAF1-1F3FF"},
    {"käsi_vasakule", "1FAF2"},
    {"käsi_vasakule_värvi1", "1FAF2-1F3FB"},
    {"käsi_vasakule_värvi2", "1FAF2-1F3FC"},
    {"käsi_vasakule_värvi3", "1FAF2-1F3FD"},
    {"käsi_vasakule_värvi4", "1FAF2-1F3FE"},
    {"käsi_vasakule_värvi5", "1FAF2-1F3FF"},
    {"käsi_peopesaga_allapoole", "1FAF3"},
    {"käsi_peopesaga_allapoole_värvi1", "1FAF3-1F3FB"},
    {"käsi_peopesaga_allapoole_värvi2", "1FAF3-1F3FC"},
    {"käsi_peopesaga_allapoole_värvi3", "1FAF3-1F3FD"},
    {"käsi_peopesaga_allapoole_värvi4", "1FAF3-1F3FE"},
    {"käsi_peopesaga_allapoole_värvi5", "1FAF3-1F3FF"},
    {"käsi_peopesaga_ülespoole", "1FAF4"},
    {"käsi_peopesaga_ülespoole_värvi1", "1FAF4-1F3FB"},
    {"käsi_peopesaga_ülespoole_värvi2", "1FAF4-1F3FC"},
    {"käsi_peopesaga_ülespoole_värvi3", "1FAF4-1F3FD"},
    {"käsi_peopesaga_ülespoole_värvi4", "1FAF4-1F3FE"},
    {"käsi_peopesaga_ülespoole_värvi5", "1FAF4-1F3FF"},
    {"vaataja_poole_osutav_nimetissõrm", "1FAF5"},
    {"vaataja_poole_osutav_nimetissõrm_värvi1", "1FAF5-1F3FB"},
    {"vaataja_poole_osutav_nimetissõrm_värvi2", "1FAF5-1F3FC"},
    {"vaataja_poole_osutav_nimetissõrm_värvi3", "1FAF5-1F3FD"},
    {"vaataja_poole_osutav_nimetissõrm_värvi4", "1FAF5-1F3FE"},
    {"vaataja_poole_osutav_nimetissõrm_värvi5", "1FAF5-1F3FF"},
    {"südant_moodustavad_käed", "1FAF6"},
    {"südant_moodustavad_käed_värvi1", "1FAF6-1F3FB"},
    {"südant_moodustavad_käed_värvi2", "1FAF6-1F3FC"},
    {"südant_moodustavad_käed_värvi3", "1FAF6-1F3FD"},
    {"südant_moodustavad_käed_värvi4", "1FAF6-1F3FE"},
    {"südant_moodustavad_käed_värvi5", "1FAF6-1F3FF"},
    {"vasakule_lükkav_käsi", "1FAF7"},
    {"vasakule_lükkav_käsi_värvi1", "1FAF7-1F3FB"},
    {"vasakule_lükkav_käsi_värvi2", "1FAF7-1F3FC"},
    {"vasakule_lükkav_käsi_värvi3", "1FAF7-1F3FD"},
    {"vasakule_lükkav_käsi_värvi4", "1FAF7-1F3FE"},
    {"vasakule_lükkav_käsi_värvi5", "1FAF7-1F3FF"},
    {"paremale_lükkav_käsi", "1FAF8"},
    {"paremale_lükkav_käsi_värvi1", "1FAF8-1F3FB"},
    {"paremale_lükkav_käsi_värvi2", "1FAF8-1F3FC"},
    {"paremale_lükkav_käsi_värvi3", "1FAF8-1F3FD"},
    {"paremale_lükkav_käsi_värvi4", "1FAF8-1F3FE"},
    {"paremale_lükkav_käsi_värvi5", "1FAF8-1F3FF"},
    {"klahv_numbrimärk", "0023-FE0F-20E3"},
    {"klahv_tärn", "002A-FE0F-20E3"},
    {"lipp_araabia_ühendemiraadid", "1F1E6-1F1EA"},
    {"lipp_aserbaidžaan", "1F1E6-1F1FF"},
    {"lipp_saint_barthélemy", "1F1E7-1F1F1"},
    {"lipp_šveits", "1F1E8-1F1ED"},
    {"lipp_tšiili", "1F1E8-1F1F1"},
    {"lipp_curaçao", "1F1E8-1F1FC"},
    {"lipp_jõulusaar", "1F1E8-1F1FD"},
    {"lipp_küpros", "1F1E8-1F1FE"},
    {"lipp_tšehhi", "1F1E8-1F1FF"},
    {"lipp_alžeeria", "1F1E9-1F1FF"},
    {"lipp_lääne_sahara", "1F1EA-1F1ED"},
    {"lipp_fidži", "1F1EB-1F1EF"},
    {"lipp_fääri_saared", "1F1EB-1F1F4"},
    {"lipp_ühendkuningriik", "1F1EC-1F1E7"},
    {"lipp_gröönimaa", "1F1EC-1F1F1"},
    {"lipp_lõuna_georgia_ja_lõuna_sandwichi_saared", "1F1EC-1F1F8"},
    {"lipp_kõrgõzstan", "1F1F0-1F1EC"},
    {"lipp_kambodža", "1F1F0-1F1ED"},
    {"lipp_põhja_korea", "1F1F0-1F1F5"},
    {"lipp_lõuna_korea", "1F1F0-1F1F7"},
    {"lipp_läti", "1F1F1-1F1FB"},
    {"lipp_liibüa", "1F1F1-1F1FE"},
    {"lipp_põhja_makedoonia", "1F1F2-1F1F0"},
    {"lipp_põhja_mariaanid", "1F1F2-1F1F5"},
    {"lipp_prantsuse_polüneesia", "1F1F5-1F1EB"},
    {"lipp_réunion", "1F1F7-1F1EA"},
    {"lipp_seišellid", "1F1F8-1F1E8"},
    {"lipp_lõuna_sudaan", "1F1F8-1F1F8"},
    {"lipp_são_tomé_ja_príncipe", "1F1F8-1F1F9"},
    {"lipp_süüria", "1F1F8-1F1FE"},
    {"lipp_tšaad", "1F1F9-1F1E9"},
    {"lipp_prantsuse_lõunaalad", "1F1F9-1F1EB"},
    {"lipp_tadžikistan", "1F1F9-1F1EF"},
    {"lipp_türkmenistan", "1F1F9-1F1F2"},
    {"lipp_türgi", "1F1F9-1F1F7"},
    {"lipp_ühendriikide_hajasaared", "1F1FA-1F1F2"},
    {"lipp_ühinenud_rahvaste_organisatsioon", "1F1FA-1F1F3"},
    {"lipp_ameerika_ühendriigid", "1F1FA-1F1F8"},
    {"lipp_lõuna_aafrika_vabariik", "1F1FF-1F1E6"},
    {"lipp_šotimaa", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"paarike_südamega_mees_mees", "1F468-200D-2764-FE0F-200D-1F468"},
    {"paarike_südamega_mees_mees_värvi1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_südamega_mees_mees_värvi1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_südamega_mees_mees_värvi1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_südamega_mees_mees_värvi1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_südamega_mees_mees_värvi1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_südamega_mees_mees_värvi2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_südamega_mees_mees_värvi2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_südamega_mees_mees_värvi2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_südamega_mees_mees_värvi2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_südamega_mees_mees_värvi2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_südamega_mees_mees_värvi3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_südamega_mees_mees_värvi3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_südamega_mees_mees_värvi3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_südamega_mees_mees_värvi3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_südamega_mees_mees_värvi3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_südamega_mees_mees_värvi4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_südamega_mees_mees_värvi4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_südamega_mees_mees_värvi4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_südamega_mees_mees_värvi4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_südamega_mees_mees_värvi4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_südamega_mees_mees_värvi5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_südamega_mees_mees_värvi5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_südamega_mees_mees_värvi5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_südamega_mees_mees_värvi5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_südamega_mees_mees_värvi5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"perekond_mees_tüdruk", "1F468-200D-1F467"},
    {"perekond_mees_tüdruk_poiss", "1F468-200D-1F467-200D-1F466"},
    {"perekond_mees_tüdruk_tüdruk", "1F468-200D-1F467-200D-1F467"},
    {"perekond_mees_mees_tüdruk", "1F468-200D-1F468-200D-1F467"},
    {"perekond_mees_mees_tüdruk_poiss", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"perekond_mees_mees_tüdruk_tüdruk", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"perekond_mees_naine_tüdruk", "1F468-200D-1F469-200D-1F467"},
    {"perekond_mees_naine_tüdruk_poiss", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"perekond_mees_naine_tüdruk_tüdruk", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"paarike_südamega_naine_mees", "1F469-200D-2764-FE0F-200D-1F468"},
    {"paarike_südamega_naine_mees_värvi1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_südamega_naine_mees_värvi1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_südamega_naine_mees_värvi1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_südamega_naine_mees_värvi1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_südamega_naine_mees_värvi1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_südamega_naine_mees_värvi2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_südamega_naine_mees_värvi2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_südamega_naine_mees_värvi2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_südamega_naine_mees_värvi2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_südamega_naine_mees_värvi2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_südamega_naine_mees_värvi3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_südamega_naine_mees_värvi3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_südamega_naine_mees_värvi3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_südamega_naine_mees_värvi3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_südamega_naine_mees_värvi3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_südamega_naine_mees_värvi4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_südamega_naine_mees_värvi4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_südamega_naine_mees_värvi4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_südamega_naine_mees_värvi4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_südamega_naine_mees_värvi4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_südamega_naine_mees_värvi5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"paarike_südamega_naine_mees_värvi5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"paarike_südamega_naine_mees_värvi5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"paarike_südamega_naine_mees_värvi5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"paarike_südamega_naine_mees_värvi5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"paarike_südamega_naine_naine", "1F469-200D-2764-FE0F-200D-1F469"},
    {"paarike_südamega_naine_naine_värvi1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"paarike_südamega_naine_naine_värvi1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"paarike_südamega_naine_naine_värvi1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"paarike_südamega_naine_naine_värvi1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"paarike_südamega_naine_naine_värvi1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"paarike_südamega_naine_naine_värvi2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"paarike_südamega_naine_naine_värvi2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"paarike_südamega_naine_naine_värvi2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"paarike_südamega_naine_naine_värvi2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"paarike_südamega_naine_naine_värvi2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"paarike_südamega_naine_naine_värvi3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"paarike_südamega_naine_naine_värvi3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"paarike_südamega_naine_naine_värvi3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"paarike_südamega_naine_naine_värvi3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"paarike_südamega_naine_naine_värvi3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"paarike_südamega_naine_naine_värvi4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"paarike_südamega_naine_naine_värvi4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"paarike_südamega_naine_naine_värvi4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"paarike_südamega_naine_naine_värvi4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"paarike_südamega_naine_naine_värvi4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"paarike_südamega_naine_naine_värvi5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"paarike_südamega_naine_naine_värvi5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"paarike_südamega_naine_naine_värvi5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"paarike_südamega_naine_naine_värvi5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"paarike_südamega_naine_naine_värvi5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"perekond_naine_tüdruk", "1F469-200D-1F467"},
    {"perekond_naine_tüdruk_poiss", "1F469-200D-1F467-200D-1F466"},
    {"perekond_naine_tüdruk_tüdruk", "1F469-200D-1F467-200D-1F467"},
    {"perekond_naine_naine_tüdruk", "1F469-200D-1F469-200D-1F467"},
    {"perekond_naine_naine_tüdruk_poiss", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"perekond_naine_naine_tüdruk_tüdruk", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"inimesed_hoiavad_käest_kinni", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"inimesed_hoiavad_käest_kinni_värvi1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"inimesed_hoiavad_käest_kinni_värvi1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"inimesed_hoiavad_käest_kinni_värvi1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"inimesed_hoiavad_käest_kinni_värvi1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"inimesed_hoiavad_käest_kinni_värvi1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"inimesed_hoiavad_käest_kinni_värvi2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"inimesed_hoiavad_käest_kinni_värvi2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"inimesed_hoiavad_käest_kinni_värvi2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"inimesed_hoiavad_käest_kinni_värvi2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"inimesed_hoiavad_käest_kinni_värvi2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"inimesed_hoiavad_käest_kinni_värvi3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"inimesed_hoiavad_käest_kinni_värvi3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"inimesed_hoiavad_käest_kinni_värvi3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"inimesed_hoiavad_käest_kinni_värvi3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"inimesed_hoiavad_käest_kinni_värvi3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"inimesed_hoiavad_käest_kinni_värvi4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"inimesed_hoiavad_käest_kinni_värvi4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"inimesed_hoiavad_käest_kinni_värvi4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"inimesed_hoiavad_käest_kinni_värvi4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"inimesed_hoiavad_käest_kinni_värvi4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"inimesed_hoiavad_käest_kinni_värvi5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"inimesed_hoiavad_käest_kinni_värvi5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"inimesed_hoiavad_käest_kinni_värvi5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"inimesed_hoiavad_käest_kinni_värvi5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"inimesed_hoiavad_käest_kinni_värvi5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"perekond_täiskasvanu_täiskasvanu_laps", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"perekond_täiskasvanu_täiskasvanu_laps_laps", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"perekond_täiskasvanu_laps", "1F9D1-200D-1F9D2"},
    {"perekond_täiskasvanu_laps_laps", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"meesõpilane", "1F468-200D-1F393"},
    {"meesõpilane_värvi1", "1F468-1F3FB-200D-1F393"},
    {"meesõpilane_värvi2", "1F468-1F3FC-200D-1F393"},
    {"meesõpilane_värvi3", "1F468-1F3FD-200D-1F393"},
    {"meesõpilane_värvi4", "1F468-1F3FE-200D-1F393"},
    {"meesõpilane_värvi5", "1F468-1F3FF-200D-1F393"},
    {"meesõpetaja", "1F468-200D-1F3EB"},
    {"meesõpetaja_värvi1", "1F468-1F3FB-200D-1F3EB"},
    {"meesõpetaja_värvi2", "1F468-1F3FC-200D-1F3EB"},
    {"meesõpetaja_värvi3", "1F468-1F3FD-200D-1F3EB"},
    {"meesõpetaja_värvi4", "1F468-1F3FE-200D-1F3EB"},
    {"meesõpetaja_värvi5", "1F468-1F3FF-200D-1F3EB"},
    {"meessoost_tehasetööline", "1F468-200D-1F3ED"},
    {"meessoost_tehasetööline_värvi1", "1F468-1F3FB-200D-1F3ED"},
    {"meessoost_tehasetööline_värvi2", "1F468-1F3FC-200D-1F3ED"},
    {"meessoost_tehasetööline_värvi3", "1F468-1F3FD-200D-1F3ED"},
    {"meessoost_tehasetööline_värvi4", "1F468-1F3FE-200D-1F3ED"},
    {"meessoost_tehasetööline_värvi5", "1F468-1F3FF-200D-1F3ED"},
    {"meeskontoritöötaja", "1F468-200D-1F4BC"},
    {"meeskontoritöötaja_värvi1", "1F468-1F3FB-200D-1F4BC"},
    {"meeskontoritöötaja_värvi2", "1F468-1F3FC-200D-1F4BC"},
    {"meeskontoritöötaja_värvi3", "1F468-1F3FD-200D-1F4BC"},
    {"meeskontoritöötaja_värvi4", "1F468-1F3FE-200D-1F4BC"},
    {"meeskontoritöötaja_värvi5", "1F468-1F3FF-200D-1F4BC"},
    {"meestuletõrjuja", "1F468-200D-1F692"},
    {"meestuletõrjuja_värvi1", "1F468-1F3FB-200D-1F692"},
    {"meestuletõrjuja_värvi2", "1F468-1F3FC-200D-1F692"},
    {"meestuletõrjuja_värvi3", "1F468-1F3FD-200D-1F692"},
    {"meestuletõrjuja_värvi4", "1F468-1F3FE-200D-1F692"},
    {"meestuletõrjuja_värvi5", "1F468-1F3FF-200D-1F692"},
    {"naisõpilane", "1F469-200D-1F393"},
    {"naisõpilane_värvi1", "1F469-1F3FB-200D-1F393"},
    {"naisõpilane_värvi2", "1F469-1F3FC-200D-1F393"},
    {"naisõpilane_värvi3", "1F469-1F3FD-200D-1F393"},
    {"naisõpilane_värvi4", "1F469-1F3FE-200D-1F393"},
    {"naisõpilane_värvi5", "1F469-1F3FF-200D-1F393"},
    {"naisõpetaja", "1F469-200D-1F3EB"},
    {"naisõpetaja_värvi1", "1F469-1F3FB-200D-1F3EB"},
    {"naisõpetaja_värvi2", "1F469-1F3FC-200D-1F3EB"},
    {"naisõpetaja_värvi3", "1F469-1F3FD-200D-1F3EB"},
    {"naisõpetaja_värvi4", "1F469-1F3FE-200D-1F3EB"},
    {"naisõpetaja_värvi5", "1F469-1F3FF-200D-1F3EB"},
    {"naissoost_tehasetööline", "1F469-200D-1F3ED"},
    {"naissoost_tehasetööline_värvi1", "1F469-1F3FB-200D-1F3ED"},
    {"naissoost_tehasetööline_värvi2", "1F469-1F3FC-200D-1F3ED"},
    {"naissoost_tehasetööline_värvi3", "1F469-1F3FD-200D-1F3ED"},
    {"naissoost_tehasetööline_värvi4", "1F469-1F3FE-200D-1F3ED"},
    {"naissoost_tehasetööline_värvi5", "1F469-1F3FF-200D-1F3ED"},
    {"naiskontoritöötaja", "1F469-200D-1F4BC"},
    {"naiskontoritöötaja_värvi1", "1F469-1F3FB-200D-1F4BC"},
    {"naiskontoritöötaja_värvi2", "1F469-1F3FC-200D-1F4BC"},
    {"naiskontoritöötaja_värvi3", "1F469-1F3FD-200D-1F4BC"},
    {"naiskontoritöötaja_värvi4", "1F469-1F3FE-200D-1F4BC"},
    {"naiskontoritöötaja_värvi5", "1F469-1F3FF-200D-1F4BC"},
    {"naistuletõrjuja", "1F469-200D-1F692"},
    {"naistuletõrjuja_värvi1", "1F469-1F3FB-200D-1F692"},
    {"naistuletõrjuja_värvi2", "1F469-1F3FC-200D-1F692"},
    {"naistuletõrjuja_värvi3", "1F469-1F3FD-200D-1F692"},
    {"naistuletõrjuja_värvi4", "1F469-1F3FE-200D-1F692"},
    {"naistuletõrjuja_värvi5", "1F469-1F3FF-200D-1F692"},
    {"jalakäija_suunaga_paremale", "1F6B6-200D-27A1-FE0F"},
    {"jalakäija_suunaga_paremale_värvi1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"jalakäija_suunaga_paremale_värvi2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"jalakäija_suunaga_paremale_värvi3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"jalakäija_suunaga_paremale_värvi4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"jalakäija_suunaga_paremale_värvi5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"põlvitav_inimene_suunaga_paremale", "1F9CE-200D-27A1-FE0F"},
    {"põlvitav_inimene_suunaga_paremale_värvi1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"põlvitav_inimene_suunaga_paremale_värvi2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"põlvitav_inimene_suunaga_paremale_värvi3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"põlvitav_inimene_suunaga_paremale_värvi4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"põlvitav_inimene_suunaga_paremale_värvi5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"tervishoiutöötaja", "1F9D1-200D-2695-FE0F"},
    {"tervishoiutöötaja_värvi1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"tervishoiutöötaja_värvi2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"tervishoiutöötaja_värvi3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"tervishoiutöötaja_värvi4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"tervishoiutöötaja_värvi5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"päkats", "1F9D1-200D-1F384"},
    {"päkats_värvi1", "1F9D1-1F3FB-200D-1F384"},
    {"päkats_värvi2", "1F9D1-1F3FC-200D-1F384"},
    {"päkats_värvi3", "1F9D1-1F3FD-200D-1F384"},
    {"päkats_värvi4", "1F9D1-1F3FE-200D-1F384"},
    {"päkats_värvi5", "1F9D1-1F3FF-200D-1F384"},
    {"õpilane", "1F9D1-200D-1F393"},
    {"õpilane_värvi1", "1F9D1-1F3FB-200D-1F393"},
    {"õpilane_värvi2", "1F9D1-1F3FC-200D-1F393"},
    {"õpilane_värvi3", "1F9D1-1F3FD-200D-1F393"},
    {"õpilane_värvi4", "1F9D1-1F3FE-200D-1F393"},
    {"õpilane_värvi5", "1F9D1-1F3FF-200D-1F393"},
    {"õpetaja", "1F9D1-200D-1F3EB"},
    {"õpetaja_värvi1", "1F9D1-1F3FB-200D-1F3EB"},
    {"õpetaja_värvi2", "1F9D1-1F3FC-200D-1F3EB"},
    {"õpetaja_värvi3", "1F9D1-1F3FD-200D-1F3EB"},
    {"õpetaja_värvi4", "1F9D1-1F3FE-200D-1F3EB"},
    {"õpetaja_värvi5", "1F9D1-1F3FF-200D-1F3EB"},
    {"tehasetööline", "1F9D1-200D-1F3ED"},
    {"tehasetööline_värvi1", "1F9D1-1F3FB-200D-1F3ED"},
    {"tehasetööline_värvi2", "1F9D1-1F3FC-200D-1F3ED"},
    {"tehasetööline_värvi3", "1F9D1-1F3FD-200D-1F3ED"},
    {"tehasetööline_värvi4", "1F9D1-1F3FE-200D-1F3ED"},
    {"tehasetööline_värvi5", "1F9D1-1F3FF-200D-1F3ED"},
    {"kontoritöötaja", "1F9D1-200D-1F4BC"},
    {"kontoritöötaja_värvi1", "1F9D1-1F3FB-200D-1F4BC"},
    {"kontoritöötaja_värvi2", "1F9D1-1F3FC-200D-1F4BC"},
    {"kontoritöötaja_värvi3", "1F9D1-1F3FD-200D-1F4BC"},
    {"kontoritöötaja_värvi4", "1F9D1-1F3FE-200D-1F4BC"},
    {"kontoritöötaja_värvi5", "1F9D1-1F3FF-200D-1F4BC"},
    {"tuletõrjuja", "1F9D1-200D-1F692"},
    {"tuletõrjuja_värvi1", "1F9D1-1F3FB-200D-1F692"},
    {"tuletõrjuja_värvi2", "1F9D1-1F3FC-200D-1F692"},
    {"tuletõrjuja_värvi3", "1F9D1-1F3FD-200D-1F692"},
    {"tuletõrjuja_värvi4", "1F9D1-1F3FE-200D-1F692"},
    {"tuletõrjuja_värvi5", "1F9D1-1F3FF-200D-1F692"},
    {"naisjõutõstja", "1F3CB-FE0F-200D-2640-FE0F"},
    {"naisjõutõstja_värvi1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"naisjõutõstja_värvi2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"naisjõutõstja_värvi3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"naisjõutõstja_värvi4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"naisjõutõstja_värvi5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"meesjõutõstja", "1F3CB-FE0F-200D-2642-FE0F"},
    {"meesjõutõstja_värvi1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"meesjõutõstja_värvi2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"meesjõutõstja_värvi3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"meesjõutõstja_värvi4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"meesjõutõstja_värvi5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"golfi_mängiv_naine", "1F3CC-FE0F-200D-2640-FE0F"},
    {"golfi_mängiv_naine_värvi1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"golfi_mängiv_naine_värvi2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"golfi_mängiv_naine_värvi3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"golfi_mängiv_naine_värvi4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"golfi_mängiv_naine_värvi5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"golfi_mängiv_mees", "1F3CC-FE0F-200D-2642-FE0F"},
    {"golfi_mängiv_mees_värvi1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"golfi_mängiv_mees_värvi2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"golfi_mängiv_mees_värvi3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"golfi_mängiv_mees_värvi4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"golfi_mängiv_mees_värvi5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"pidutsevad_jänesekõrvadega_naised", "1F46F-200D-2640-FE0F"},
    {"pidutsevad_jänesekõrvadega_naised_värvi1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"pidutsevad_jänesekõrvadega_naised_värvi1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"pidutsevad_jänesekõrvadega_naised_värvi1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"pidutsevad_jänesekõrvadega_naised_värvi1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"pidutsevad_jänesekõrvadega_naised_värvi2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"pidutsevad_jänesekõrvadega_naised_värvi2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"pidutsevad_jänesekõrvadega_naised_värvi2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"pidutsevad_jänesekõrvadega_naised_värvi2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"pidutsevad_jänesekõrvadega_naised_värvi3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"pidutsevad_jänesekõrvadega_naised_värvi3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"pidutsevad_jänesekõrvadega_naised_värvi3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"pidutsevad_jänesekõrvadega_naised_värvi3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"pidutsevad_jänesekõrvadega_naised_värvi4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"pidutsevad_jänesekõrvadega_naised_värvi4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"pidutsevad_jänesekõrvadega_naised_värvi4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"pidutsevad_jänesekõrvadega_naised_värvi4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"pidutsevad_jänesekõrvadega_naised_värvi5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"pidutsevad_jänesekõrvadega_naised_värvi5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"pidutsevad_jänesekõrvadega_naised_värvi5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"pidutsevad_jänesekõrvadega_naised_värvi5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"pidutsevad_jänesekõrvadega_naised_värvi1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"pidutsevad_jänesekõrvadega_naised_värvi2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"pidutsevad_jänesekõrvadega_naised_värvi3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"pidutsevad_jänesekõrvadega_naised_värvi4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"pidutsevad_jänesekõrvadega_naised_värvi5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"pidutsevad_jänesekõrvadega_mehed", "1F46F-200D-2642-FE0F"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"pidutsevad_jänesekõrvadega_mehed_värvi5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"infoleti_naistöötaja", "1F481-200D-2640-FE0F"},
    {"infoleti_naistöötaja_värvi1", "1F481-1F3FB-200D-2640-FE0F"},
    {"infoleti_naistöötaja_värvi2", "1F481-1F3FC-200D-2640-FE0F"},
    {"infoleti_naistöötaja_värvi3", "1F481-1F3FD-200D-2640-FE0F"},
    {"infoleti_naistöötaja_värvi4", "1F481-1F3FE-200D-2640-FE0F"},
    {"infoleti_naistöötaja_värvi5", "1F481-1F3FF-200D-2640-FE0F"},
    {"infoleti_meestöötaja", "1F481-200D-2642-FE0F"},
    {"infoleti_meestöötaja_värvi1", "1F481-1F3FB-200D-2642-FE0F"},
    {"infoleti_meestöötaja_värvi2", "1F481-1F3FC-200D-2642-FE0F"},
    {"infoleti_meestöötaja_värvi3", "1F481-1F3FD-200D-2642-FE0F"},
    {"infoleti_meestöötaja_värvi4", "1F481-1F3FE-200D-2642-FE0F"},
    {"infoleti_meestöötaja_värvi5", "1F481-1F3FF-200D-2642-FE0F"},
    {"näomassaaži_saav_naine", "1F486-200D-2640-FE0F"},
    {"näomassaaži_saav_naine_värvi1", "1F486-1F3FB-200D-2640-FE0F"},
    {"näomassaaži_saav_naine_värvi2", "1F486-1F3FC-200D-2640-FE0F"},
    {"näomassaaži_saav_naine_värvi3", "1F486-1F3FD-200D-2640-FE0F"},
    {"näomassaaži_saav_naine_värvi4", "1F486-1F3FE-200D-2640-FE0F"},
    {"näomassaaži_saav_naine_värvi5", "1F486-1F3FF-200D-2640-FE0F"},
    {"näomassaaži_saav_mees", "1F486-200D-2642-FE0F"},
    {"näomassaaži_saav_mees_värvi1", "1F486-1F3FB-200D-2642-FE0F"},
    {"näomassaaži_saav_mees_värvi2", "1F486-1F3FC-200D-2642-FE0F"},
    {"näomassaaži_saav_mees_värvi3", "1F486-1F3FD-200D-2642-FE0F"},
    {"näomassaaži_saav_mees_värvi4", "1F486-1F3FE-200D-2642-FE0F"},
    {"näomassaaži_saav_mees_värvi5", "1F486-1F3FF-200D-2642-FE0F"},
    {"eitavat_žesti_näitav_naine", "1F645-200D-2640-FE0F"},
    {"eitavat_žesti_näitav_naine_värvi1", "1F645-1F3FB-200D-2640-FE0F"},
    {"eitavat_žesti_näitav_naine_värvi2", "1F645-1F3FC-200D-2640-FE0F"},
    {"eitavat_žesti_näitav_naine_värvi3", "1F645-1F3FD-200D-2640-FE0F"},
    {"eitavat_žesti_näitav_naine_värvi4", "1F645-1F3FE-200D-2640-FE0F"},
    {"eitavat_žesti_näitav_naine_värvi5", "1F645-1F3FF-200D-2640-FE0F"},
    {"eitavat_žesti_näitav_mees", "1F645-200D-2642-FE0F"},
    {"eitavat_žesti_näitav_mees_värvi1", "1F645-1F3FB-200D-2642-FE0F"},
    {"eitavat_žesti_näitav_mees_värvi2", "1F645-1F3FC-200D-2642-FE0F"},
    {"eitavat_žesti_näitav_mees_värvi3", "1F645-1F3FD-200D-2642-FE0F"},
    {"eitavat_žesti_näitav_mees_värvi4", "1F645-1F3FE-200D-2642-FE0F"},
    {"eitavat_žesti_näitav_mees_värvi5", "1F645-1F3FF-200D-2642-FE0F"},
    {"nõustuvat_žesti_näitav_naine", "1F646-200D-2640-FE0F"},
    {"nõustuvat_žesti_näitav_naine_värvi1", "1F646-1F3FB-200D-2640-FE0F"},
    {"nõustuvat_žesti_näitav_naine_värvi2", "1F646-1F3FC-200D-2640-FE0F"},
    {"nõustuvat_žesti_näitav_naine_värvi3", "1F646-1F3FD-200D-2640-FE0F"},
    {"nõustuvat_žesti_näitav_naine_värvi4", "1F646-1F3FE-200D-2640-FE0F"},
    {"nõustuvat_žesti_näitav_naine_värvi5", "1F646-1F3FF-200D-2640-FE0F"},
    {"nõustuvat_žesti_näitav_mees", "1F646-200D-2642-FE0F"},
    {"nõustuvat_žesti_näitav_mees_värvi1", "1F646-1F3FB-200D-2642-FE0F"},
    {"nõustuvat_žesti_näitav_mees_värvi2", "1F646-1F3FC-200D-2642-FE0F"},
    {"nõustuvat_žesti_näitav_mees_värvi3", "1F646-1F3FD-200D-2642-FE0F"},
    {"nõustuvat_žesti_näitav_mees_värvi4", "1F646-1F3FE-200D-2642-FE0F"},
    {"nõustuvat_žesti_näitav_mees_värvi5", "1F646-1F3FF-200D-2642-FE0F"},
    {"sügavalt_kummarduv_naine", "1F647-200D-2640-FE0F"},
    {"sügavalt_kummarduv_naine_värvi1", "1F647-1F3FB-200D-2640-FE0F"},
    {"sügavalt_kummarduv_naine_värvi2", "1F647-1F3FC-200D-2640-FE0F"},
    {"sügavalt_kummarduv_naine_värvi3", "1F647-1F3FD-200D-2640-FE0F"},
    {"sügavalt_kummarduv_naine_värvi4", "1F647-1F3FE-200D-2640-FE0F"},
    {"sügavalt_kummarduv_naine_värvi5", "1F647-1F3FF-200D-2640-FE0F"},
    {"sügavalt_kummarduv_mees", "1F647-200D-2642-FE0F"},
    {"sügavalt_kummarduv_mees_värvi1", "1F647-1F3FB-200D-2642-FE0F"},
    {"sügavalt_kummarduv_mees_värvi2", "1F647-1F3FC-200D-2642-FE0F"},
    {"sügavalt_kummarduv_mees_värvi3", "1F647-1F3FD-200D-2642-FE0F"},
    {"sügavalt_kummarduv_mees_värvi4", "1F647-1F3FE-200D-2642-FE0F"},
    {"sügavalt_kummarduv_mees_värvi5", "1F647-1F3FF-200D-2642-FE0F"},
    {"kätt_tõstev_naine", "1F64B-200D-2640-FE0F"},
    {"kätt_tõstev_naine_värvi1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"kätt_tõstev_naine_värvi2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"kätt_tõstev_naine_värvi3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"kätt_tõstev_naine_värvi4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"kätt_tõstev_naine_värvi5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"kätt_tõstev_mees", "1F64B-200D-2642-FE0F"},
    {"kätt_tõstev_mees_värvi1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"kätt_tõstev_mees_värvi2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"kätt_tõstev_mees_värvi3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"kätt_tõstev_mees_värvi4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"kätt_tõstev_mees_värvi5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"naismägirattur", "1F6B5-200D-2640-FE0F"},
    {"naismägirattur_värvi1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"naismägirattur_värvi2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"naismägirattur_värvi3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"naismägirattur_värvi4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"naismägirattur_värvi5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"meesmägirattur", "1F6B5-200D-2642-FE0F"},
    {"meesmägirattur_värvi1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"meesmägirattur_värvi2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"meesmägirattur_värvi3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"meesmägirattur_värvi4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"meesmägirattur_värvi5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"kõndiv_naine", "1F6B6-200D-2640-FE0F"},
    {"kõndiv_naine_värvi1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"kõndiv_naine_värvi2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"kõndiv_naine_värvi3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"kõndiv_naine_värvi4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"kõndiv_naine_värvi5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"kõndiv_naine_suunaga_paremale", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kõndiv_naine_suunaga_paremale_värvi1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kõndiv_naine_suunaga_paremale_värvi2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kõndiv_naine_suunaga_paremale_värvi3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kõndiv_naine_suunaga_paremale_värvi4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kõndiv_naine_suunaga_paremale_värvi5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kõndiv_mees", "1F6B6-200D-2642-FE0F"},
    {"kõndiv_mees_värvi1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"kõndiv_mees_värvi2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"kõndiv_mees_värvi3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"kõndiv_mees_värvi4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"kõndiv_mees_värvi5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"kõndiv_mees_suunaga_paremale", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kõndiv_mees_suunaga_paremale_värvi1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kõndiv_mees_suunaga_paremale_värvi2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kõndiv_mees_suunaga_paremale_värvi3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kõndiv_mees_suunaga_paremale_värvi4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kõndiv_mees_suunaga_paremale_värvi5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"käega_vastu_pead_lööv_naine", "1F926-200D-2640-FE0F"},
    {"käega_vastu_pead_lööv_naine_värvi1", "1F926-1F3FB-200D-2640-FE0F"},
    {"käega_vastu_pead_lööv_naine_värvi2", "1F926-1F3FC-200D-2640-FE0F"},
    {"käega_vastu_pead_lööv_naine_värvi3", "1F926-1F3FD-200D-2640-FE0F"},
    {"käega_vastu_pead_lööv_naine_värvi4", "1F926-1F3FE-200D-2640-FE0F"},
    {"käega_vastu_pead_lööv_naine_värvi5", "1F926-1F3FF-200D-2640-FE0F"},
    {"käega_vastu_pead_lööv_mees", "1F926-200D-2642-FE0F"},
    {"käega_vastu_pead_lööv_mees_värvi1", "1F926-1F3FB-200D-2642-FE0F"},
    {"käega_vastu_pead_lööv_mees_värvi2", "1F926-1F3FC-200D-2642-FE0F"},
    {"käega_vastu_pead_lööv_mees_värvi3", "1F926-1F3FD-200D-2642-FE0F"},
    {"käega_vastu_pead_lööv_mees_värvi4", "1F926-1F3FE-200D-2642-FE0F"},
    {"käega_vastu_pead_lööv_mees_värvi5", "1F926-1F3FF-200D-2642-FE0F"},
    {"õlgu_kehitav_naine", "1F937-200D-2640-FE0F"},
    {"õlgu_kehitav_naine_värvi1", "1F937-1F3FB-200D-2640-FE0F"},
    {"õlgu_kehitav_naine_värvi2", "1F937-1F3FC-200D-2640-FE0F"},
    {"õlgu_kehitav_naine_värvi3", "1F937-1F3FD-200D-2640-FE0F"},
    {"õlgu_kehitav_naine_värvi4", "1F937-1F3FE-200D-2640-FE0F"},
    {"õlgu_kehitav_naine_värvi5", "1F937-1F3FF-200D-2640-FE0F"},
    {"õlgu_kehitav_mees", "1F937-200D-2642-FE0F"},
    {"õlgu_kehitav_mees_värvi1", "1F937-1F3FB-200D-2642-FE0F"},
    {"õlgu_kehitav_mees_värvi2", "1F937-1F3FC-200D-2642-FE0F"},
    {"õlgu_kehitav_mees_värvi3", "1F937-1F3FD-200D-2642-FE0F"},
    {"õlgu_kehitav_mees_värvi4", "1F937-1F3FE-200D-2642-FE0F"},
    {"õlgu_kehitav_mees_värvi5", "1F937-1F3FF-200D-2642-FE0F"},
    {"naisžonglöör", "1F939-200D-2640-FE0F"},
    {"naisžonglöör_värvi1", "1F939-1F3FB-200D-2640-FE0F"},
    {"naisžonglöör_värvi2", "1F939-1F3FC-200D-2640-FE0F"},
    {"naisžonglöör_värvi3", "1F939-1F3FD-200D-2640-FE0F"},
    {"naisžonglöör_värvi4", "1F939-1F3FE-200D-2640-FE0F"},
    {"naisžonglöör_värvi5", "1F939-1F3FF-200D-2640-FE0F"},
    {"meesžonglöör", "1F939-200D-2642-FE0F"},
    {"meesžonglöör_värvi1", "1F939-1F3FB-200D-2642-FE0F"},
    {"meesžonglöör_värvi2", "1F939-1F3FC-200D-2642-FE0F"},
    {"meesžonglöör_värvi3", "1F939-1F3FD-200D-2642-FE0F"},
    {"meesžonglöör_värvi4", "1F939-1F3FE-200D-2642-FE0F"},
    {"meesžonglöör_värvi5", "1F939-1F3FF-200D-2642-FE0F"},
    {"veepalli_mängiv_naine", "1F93D-200D-2640-FE0F"},
    {"veepalli_mängiv_naine_värvi1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"veepalli_mängiv_naine_värvi2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"veepalli_mängiv_naine_värvi3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"veepalli_mängiv_naine_värvi4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"veepalli_mängiv_naine_värvi5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"veepalli_mängiv_mees", "1F93D-200D-2642-FE0F"},
    {"veepalli_mängiv_mees_värvi1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"veepalli_mängiv_mees_värvi2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"veepalli_mängiv_mees_värvi3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"veepalli_mängiv_mees_värvi4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"veepalli_mängiv_mees_värvi5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"käsipalli_mängiv_naine", "1F93E-200D-2640-FE0F"},
    {"käsipalli_mängiv_naine_värvi1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"käsipalli_mängiv_naine_värvi2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"käsipalli_mängiv_naine_värvi3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"käsipalli_mängiv_naine_värvi4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"käsipalli_mängiv_naine_värvi5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"käsipalli_mängiv_mees", "1F93E-200D-2642-FE0F"},
    {"käsipalli_mängiv_mees_värvi1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"käsipalli_mängiv_mees_värvi2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"käsipalli_mängiv_mees_värvi3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"käsipalli_mängiv_mees_värvi4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"käsipalli_mängiv_mees_värvi5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"põlvitav_naine", "1F9CE-200D-2640-FE0F"},
    {"põlvitav_naine_värvi1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"põlvitav_naine_värvi2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"põlvitav_naine_värvi3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"põlvitav_naine_värvi4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"põlvitav_naine_värvi5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"põlvitav_naine_suunaga_paremale", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"põlvitav_naine_suunaga_paremale_värvi1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"põlvitav_naine_suunaga_paremale_värvi2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"põlvitav_naine_suunaga_paremale_värvi3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"põlvitav_naine_suunaga_paremale_värvi4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"põlvitav_naine_suunaga_paremale_värvi5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"põlvitav_mees", "1F9CE-200D-2642-FE0F"},
    {"põlvitav_mees_värvi1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"põlvitav_mees_värvi2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"põlvitav_mees_värvi3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"põlvitav_mees_värvi4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"põlvitav_mees_värvi5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"põlvitav_mees_suunaga_paremale", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"põlvitav_mees_suunaga_paremale_värvi1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"põlvitav_mees_suunaga_paremale_värvi2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"põlvitav_mees_suunaga_paremale_värvi3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"põlvitav_mees_suunaga_paremale_värvi4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"põlvitav_mees_suunaga_paremale_värvi5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mäkke_roniv_naine", "1F9D7-200D-2640-FE0F"},
    {"mäkke_roniv_naine_värvi1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"mäkke_roniv_naine_värvi2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"mäkke_roniv_naine_värvi3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"mäkke_roniv_naine_värvi4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"mäkke_roniv_naine_värvi5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"mäkke_roniv_mees", "1F9D7-200D-2642-FE0F"},
    {"mäkke_roniv_mees_värvi1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"mäkke_roniv_mees_värvi2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"mäkke_roniv_mees_värvi3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"mäkke_roniv_mees_värvi4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"mäkke_roniv_mees_värvi5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"meesnäkk", "1F9DC-200D-2642-FE0F"},
    {"meesnäkk_värvi1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"meesnäkk_värvi2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"meesnäkk_värvi3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"meesnäkk_värvi4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"meesnäkk_värvi5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"naispäkapikk", "1F9DD-200D-2640-FE0F"},
    {"naispäkapikk_värvi1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"naispäkapikk_värvi2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"naispäkapikk_värvi3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"naispäkapikk_värvi4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"naispäkapikk_värvi5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"meespäkapikk", "1F9DD-200D-2642-FE0F"},
    {"meespäkapikk_värvi1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"meespäkapikk_värvi2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"meespäkapikk_värvi3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"meespäkapikk_värvi4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"meespäkapikk_värvi5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"naisdžinn", "1F9DE-200D-2640-FE0F"},
    {"meesdžinn", "1F9DE-200D-2642-FE0F"},
    {"täiskasvanu_punased_juuksed", "1F9D1-200D-1F9B0"},
    {"täiskasvanu_punased_juuksed_värvi1", "1F9D1-1F3FB-200D-1F9B0"},
    {"täiskasvanu_punased_juuksed_värvi2", "1F9D1-1F3FC-200D-1F9B0"},
    {"täiskasvanu_punased_juuksed_värvi3", "1F9D1-1F3FD-200D-1F9B0"},
    {"täiskasvanu_punased_juuksed_värvi4", "1F9D1-1F3FE-200D-1F9B0"},
    {"täiskasvanu_punased_juuksed_värvi5", "1F9D1-1F3FF-200D-1F9B0"},
    {"täiskasvanu_lokkis_juuksed", "1F9D1-200D-1F9B1"},
    {"täiskasvanu_lokkis_juuksed_värvi1", "1F9D1-1F3FB-200D-1F9B1"},
    {"täiskasvanu_lokkis_juuksed_värvi2", "1F9D1-1F3FC-200D-1F9B1"},
    {"täiskasvanu_lokkis_juuksed_värvi3", "1F9D1-1F3FD-200D-1F9B1"},
    {"täiskasvanu_lokkis_juuksed_värvi4", "1F9D1-1F3FE-200D-1F9B1"},
    {"täiskasvanu_lokkis_juuksed_värvi5", "1F9D1-1F3FF-200D-1F9B1"},
    {"täiskasvanu_kiilaspea", "1F9D1-200D-1F9B2"},
    {"täiskasvanu_kiilaspea_värvi1", "1F9D1-1F3FB-200D-1F9B2"},
    {"täiskasvanu_kiilaspea_värvi2", "1F9D1-1F3FC-200D-1F9B2"},
    {"täiskasvanu_kiilaspea_värvi3", "1F9D1-1F3FD-200D-1F9B2"},
    {"täiskasvanu_kiilaspea_värvi4", "1F9D1-1F3FE-200D-1F9B2"},
    {"täiskasvanu_kiilaspea_värvi5", "1F9D1-1F3FF-200D-1F9B2"},
    {"täiskasvanu_heledad_juuksed", "1F9D1-200D-1F9B3"},
    {"täiskasvanu_heledad_juuksed_värvi1", "1F9D1-1F3FB-200D-1F9B3"},
    {"täiskasvanu_heledad_juuksed_värvi2", "1F9D1-1F3FC-200D-1F9B3"},
    {"täiskasvanu_heledad_juuksed_värvi3", "1F9D1-1F3FD-200D-1F9B3"},
    {"täiskasvanu_heledad_juuksed_värvi4", "1F9D1-1F3FE-200D-1F9B3"},
    {"täiskasvanu_heledad_juuksed_värvi5", "1F9D1-1F3FF-200D-1F9B3"},
    {"leegitsev_süda", "2764-FE0F-200D-1F525"},
    {"tervenev_süda", "2764-FE0F-200D-1FA79"},
    {"šampinjon", "1F344-200D-1F7EB"},
    {"fööniks", "1F426-200D-1F525"},
    {"jääkaru", "1F43B-200D-2744-FE0F"},
    {"väljahingav_nägu", "1F62E-200D-1F4A8"},
    {"spiraalsilmadega_nägu", "1F635-200D-1F4AB"},
    {"nägu_pilvedes", "1F636-200D-1F32B-FE0F"},
};

