// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_pt[] = {
    {"exclamacao_com_interrogacao", "2049"},
    {"simbolo_de_marca_registrada", "2122"},
    {"informacoes", "2139"},
    {"seta_para_esquerda_e_direita", "2194"},
    {"seta_para_cima_e_para_baixo", "2195"},
    {"seta_para_cima_e_para_a_esquerda", "2196"},
    {"seta_para_cima_e_para_a_direita", "2197"},
    {"seta_para_baixo_e_para_a_direita", "2198"},
    {"seta_para_baixo_e_para_a_esquerda", "2199"},
    {"teclado", "2328"},
    {"sol", "2600"},
    {"nuvem", "2601"},
    {"guarda_chuva", "2602"},
    {"boneco_de_neve", "2603"},
    {"cometa", "2604"},
    {"caixa_de_selecao_marcada_com_tique", "2611"},
    {"sombrinha_na_chuva", "2614"},
    {"cafe", "2615"},
    {"trevo", "2618"},
    {"caveira_e_ossos_cruzados", "2620"},
    {"radioativo", "2622"},
    {"risco_biologico", "2623"},
    {"cruz_ortodoxa", "2626"},
    {"roda_do_dharma", "2638"},
    {"rosto_descontente", "2639"},
    {"simbolo_de_feminino", "2640"},
    {"simbolo_de_masculino", "2642"},
    {"signo_de_aries", "2648"},
    {"signo_de_touro", "2649"},
    {"signo_de_sagitario", "2650"},
    {"signo_de_capricornio", "2651"},
    {"signo_de_aquario", "2652"},
    {"signo_de_peixes", "2653"},
    {"naipe_de_espadas", "2660"},
    {"naipe_de_paus", "2663"},
    {"naipe_de_copas", "2665"},
    {"naipe_de_ouros", "2666"},
    {"chamas", "2668"},
    {"martelo_e_picareta", "2692"},
    {"ancora", "2693"},
    {"espadas_cruzadas", "2694"},
    {"simbolo_da_medicina", "2695"},
    {"balanca", "2696"},
    {"alambique", "2697"},
    {"engrenagem", "2699"},
    {"tesoura", "2702"},
    {"marca_de_selecao_branca", "2705"},
    {"aviao", "2708"},
    {"envelope", "2709"},
    {"ponta_de_caneta_tinteiro", "2712"},
    {"marca_de_selecao", "2714"},
    {"sinal_de_multiplicacao", "2716"},
    {"estrela_de_davi", "2721"},
    {"brilhos", "2728"},
    {"asterisco_de_oito_pontas", "2733"},
    {"estrela_de_oito_pontas", "2734"},
    {"floco_de_neve", "2744"},
    {"faisca", "2747"},
    {"ponto_de_interrogacao_vermelho", "2753"},
    {"ponto_de_interrogacao_branco", "2754"},
    {"ponto_de_exclamacao_branco", "2755"},
    {"ponto_de_exclamacao_vermelho", "2757"},
    {"exclamacao_de_coracao", "2763"},
    {"coracao_vermelho", "2764"},
    {"simbolo_de_adicao", "2795"},
    {"simbolo_de_subtracao", "2796"},
    {"simbolo_de_divisao", "2797"},
    {"seta_para_a_direita_curvada_para_cima", "2934"},
    {"seta_para_a_direita_curvada_para_baixo", "2935"},
    {"travessao_ondulado", "3030"},
    {"botao_japones_de_parabens", "3297"},
    {"botao_japones_de_segredo", "3299"},
    {"simbolo_de_copyright", "00A9"},
    {"simbolo_de_registrado", "00AE"},
    {"dupla_exclamacao", "203C"},
    {"seta_curva_da_direita_para_a_esquerda", "21A9"},
    {"seta_curva_da_esquerda_para_a_direita", "21AA"},
    {"relogio_de_pulso", "231A"},
    {"ampulheta", "231B"},
    {"botao_ejetar", "23CF"},
    {"botao_avancar", "23E9"},
    {"botao_de_retroceder", "23EA"},
    {"botao_de_avanco_para_cima", "23EB"},
    {"botao_de_avanco_para_baixo", "23EC"},
    {"botao_de_proxima_faixa", "23ED"},
    {"botao_de_faixa_anterior", "23EE"},
    {"botao_de_reproduzir_ou_pausar", "23EF"},
    {"despertador", "23F0"},
    {"cronometro", "23F1"},
    {"relogio_temporizador", "23F2"},
    {"ampulheta_contando_o_tempo", "23F3"},
    {"botao_pausar", "23F8"},
    {"botao_parar", "23F9"},
    {"botao_gravar", "23FA"},
    {"circulo_com_a_letra_m", "24C2"},
    {"quadrado_preto_pequeno", "25AA"},
    {"quadrado_branco_pequeno", "25AB"},
    {"botao_reproduzir", "25B6"},
    {"botao_de_voltar", "25C0"},
    {"quadrado_branco_medio", "25FB"},
    {"quadrado_preto_medio", "25FC"},
    {"quadrado_branco_medio_menor", "25FD"},
    {"quadrado_preto_medio_menor", "25FE"},
    {"telefone_no_gancho", "260E"},
    {"indicador_apontando_para_cima", "261D"},
    {"indicador_apontando_para_cima_cor1", "261D-1F3FB"},
    {"indicador_apontando_para_cima_cor2", "261D-1F3FC"},
    {"indicador_apontando_para_cima_cor3", "261D-1F3FD"},
    {"indicador_apontando_para_cima_cor4", "261D-1F3FE"},
    {"indicador_apontando_para_cima_cor5", "261D-1F3FF"},
    {"estrela_e_lua_crescente", "262A"},
    {"simbolo_da_paz", "262E"},
    {"yin_yang", "262F"},
    {"rosto_sorridente", "263A"},
    {"signo_de_gemeos", "264A"},
    {"signo_de_cancer", "264B"},
    {"signo_de_leao", "264C"},
    {"signo_de_virgem", "264D"},
    {"signo_de_libra", "264E"},
    {"signo_de_escorpiao", "264F"},
    {"peao_de_xadrez", "265F"},
    {"simbolo_de_reciclagem", "267B"},
    {"infinito", "267E"},
    {"simbolo_de_cadeira_de_rodas", "267F"},
    {"simbolo_de_atomo", "269B"},
    {"flor_de_lis", "269C"},
    {"aviso", "26A0"},
    {"alta_tensao", "26A1"},
    {"simbolo_transgenero", "26A7"},
    {"circulo_branco", "26AA"},
    {"circulo_preto", "26AB"},
    {"caixao", "26B0"},
    {"urna_funeraria", "26B1"},
    {"bola_de_futebol", "26BD"},
    {"bola_de_beisebol", "26BE"},
    {"boneco_de_neve_sem_neve", "26C4"},
    {"sol_por_tras_das_nuvens", "26C5"},
    {"chuva_com_trovao", "26C8"},
    {"signo_de_ofiuco", "26CE"},
    {"picareta", "26CF"},
    {"capacacete_de_socorrista", "26D1"},
    {"correntes", "26D3"},
    {"entrada_proibida", "26D4"},
    {"santuario_japones", "26E9"},
    {"igreja", "26EA"},
    {"montanha", "26F0"},
    {"guarda_sol", "26F1"},
    {"fonte", "26F2"},
    {"bandeira_no_buraco", "26F3"},
    {"balsa", "26F4"},
    {"barco_a_vela", "26F5"},
    {"esquiador", "26F7"},
    {"patins_de_gelo", "26F8"},
    {"pessoa_jogando_basquete", "26F9"},
    {"pessoa_jogando_basquete_cor1", "26F9-1F3FB"},
    {"pessoa_jogando_basquete_cor2", "26F9-1F3FC"},
    {"pessoa_jogando_basquete_cor3", "26F9-1F3FD"},
    {"pessoa_jogando_basquete_cor4", "26F9-1F3FE"},
    {"pessoa_jogando_basquete_cor5", "26F9-1F3FF"},
    {"barraca", "26FA"},
    {"posto_de_gasolina", "26FD"},
    {"punho_levantado", "270A"},
    {"punho_levantado_cor1", "270A-1F3FB"},
    {"punho_levantado_cor2", "270A-1F3FC"},
    {"punho_levantado_cor3", "270A-1F3FD"},
    {"punho_levantado_cor4", "270A-1F3FE"},
    {"punho_levantado_cor5", "270A-1F3FF"},
    {"mao_levantada", "270B"},
    {"mao_levantada_cor1", "270B-1F3FB"},
    {"mao_levantada_cor2", "270B-1F3FC"},
    {"mao_levantada_cor3", "270B-1F3FD"},
    {"mao_levantada_cor4", "270B-1F3FE"},
    {"mao_levantada_cor5", "270B-1F3FF"},
    {"mao_em_v_de_vitoria", "270C"},
    {"mao_em_v_de_vitoria_cor1", "270C-1F3FB"},
    {"mao_em_v_de_vitoria_cor2", "270C-1F3FC"},
    {"mao_em_v_de_vitoria_cor3", "270C-1F3FD"},
    {"mao_em_v_de_vitoria_cor4", "270C-1F3FE"},
    {"mao_em_v_de_vitoria_cor5", "270C-1F3FF"},
    {"escrevendo_a_mao", "270D"},
    {"escrevendo_a_mao_cor1", "270D-1F3FB"},
    {"escrevendo_a_mao_cor2", "270D-1F3FC"},
    {"escrevendo_a_mao_cor3", "270D-1F3FD"},
    {"escrevendo_a_mao_cor4", "270D-1F3FE"},
    {"escrevendo_a_mao_cor5", "270D-1F3FF"},
    {"lapis", "270F"},
    {"cruz_latina", "271D"},
    {"xis", "274C"},
    {"botao_de_xis", "274E"},
    {"seta_para_a_direita", "27A1"},
    {"laco_encaracolado", "27B0"},
    {"loop_encaracolado_duas_vezes", "27BF"},
    {"seta_para_a_esquerda", "2B05"},
    {"seta_para_cima", "2B06"},
    {"seta_para_baixo", "2B07"},
    {"quadrado_preto_grande", "2B1B"},
    {"quadrado_branco_grande", "2B1C"},
    {"estrela_branca_media", "2B50"},
    {"circulo_grande_oco", "2B55"},
    {"sinal_japones_indicando_inicio_de_musica", "303D"},
    {"dragao_vermelho_de_mahjong", "1F004"},
    {"curinga", "1F0CF"},
    {"botao_a_tipo_sanguineo", "1F170"},
    {"botao_b_tipo_sanguineo", "1F171"},
    {"botao_o_tipo_sanguineo", "1F17E"},
    {"botao_p", "1F17F"},
    {"botao_ab_tipo_sanguineo", "1F18E"},
    {"botao_cl", "1F191"},
    {"botao_cool", "1F192"},
    {"botao_free", "1F193"},
    {"botao_id", "1F194"},
    {"botao_new", "1F195"},
    {"botao_ng", "1F196"},
    {"botao_ok", "1F197"},
    {"botao_sos", "1F198"},
    {"botao_up", "1F199"},
    {"botao_vs", "1F19A"},
    {"indicador_regional_a", "1F1E6"},
    {"indicador_regional_b", "1F1E7"},
    {"indicador_regional_c", "1F1E8"},
    {"indicador_regional_d", "1F1E9"},
    {"indicador_regional_e", "1F1EA"},
    {"indicador_regional_f", "1F1EB"},
    {"indicador_regional_g", "1F1EC"},
    {"indicador_regional_h", "1F1ED"},
    {"indicador_regional_i", "1F1EE"},
    {"indicador_regional_j", "1F1EF"},
    {"indicador_regional_k", "1F1F0"},
    {"indicador_regional_l", "1F1F1"},
    {"indicador_regional_m", "1F1F2"},
    {"indicador_regional_n", "1F1F3"},
    {"indicador_regional_o", "1F1F4"},
    {"indicador_regional_p", "1F1F5"},
    {"indicador_regional_q", "1F1F6"},
    {"indicador_regional_r", "1F1F7"},
    {"indicador_regional_s", "1F1F8"},
    {"indicador_regional_t", "1F1F9"},
    {"indicador_regional_u", "1F1FA"},
    {"indicador_regional_v", "1F1FB"},
    {"indicador_regional_w", "1F1FC"},
    {"indicador_regional_x", "1F1FD"},
    {"indicador_regional_y", "1F1FE"},
    {"indicador_regional_z", "1F1FF"},
    {"botao_japones_de_aqui", "1F201"},
    {"botao_japones_de_taxa_de_servico", "1F202"},
    {"botao_japones_de_gratuito", "1F21A"},
    {"botao_japones_de_reservado", "1F22F"},
    {"botao_japones_de_proibido", "1F232"},
    {"botao_japones_de_vago", "1F233"},
    {"botao_japones_de_nota_minima", "1F234"},
    {"botao_japones_de_sem_vagas", "1F235"},
    {"botao_japones_de_nao_gratuito", "1F236"},
    {"botao_japones_de_quantidade_mensal", "1F237"},
    {"botao_japones_de_aplicacao", "1F238"},
    {"botao_japones_de_desconto", "1F239"},
    {"botao_japones_de_aberto_para_negocios", "1F23A"},
    {"botao_japones_de_barganha", "1F250"},
    {"botao_japones_de_aceitavel", "1F251"},
    {"ciclone", "1F300"},
    {"enevoado", "1F301"},
    {"guarda_chuva_fechado", "1F302"},
    {"noite_estrelada", "1F303"},
    {"aurora_sobre_montanhas", "1F304"},
    {"aurora_sobre_agua", "1F305"},
    {"cidade_ao_anoitecer", "1F306"},
    {"por_do_sol", "1F307"},
    {"arco_iris", "1F308"},
    {"ponte_a_noite", "1F309"},
    {"onda", "1F30A"},
    {"vulcao", "1F30B"},
    {"via_lactea", "1F30C"},
    {"globo_mostrando_europa_e_africa", "1F30D"},
    {"globo_mostrando_as_americas", "1F30E"},
    {"globo_mostrando_asia_e_oceania", "1F30F"},
    {"globo_com_meridianos", "1F310"},
    {"lua_nova", "1F311"},
    {"lua_crescente_concava", "1F312"},
    {"quarto_crescente", "1F313"},
    {"lua_crescente_convexa", "1F314"},
    {"lua_cheia", "1F315"},
    {"lua_minguante_convexa", "1F316"},
    {"quarto_minguante", "1F317"},
    {"lua_minguante_concava", "1F318"},
    {"lua_crescente", "1F319"},
    {"rosto_da_lua_nova", "1F31A"},
    {"rosto_da_lua_de_quarto_crescente", "1F31B"},
    {"rosto_da_lua_de_quarto_minguante", "1F31C"},
    {"rosto_da_lua_cheia", "1F31D"},
    {"rosto_do_sol", "1F31E"},
    {"estrela_brilhante", "1F31F"},
    {"estrela_cadente", "1F320"},
    {"termometro", "1F321"},
    {"sol_com_nuvens", "1F324"},
    {"nublado", "1F325"},
    {"sol_com_chuva", "1F326"},
    {"nuvem_com_chuva", "1F327"},
    {"nuvem_com_neve", "1F328"},
    {"nuvem_com_trovao", "1F329"},
    {"tornado", "1F32A"},
    {"nevoeiro", "1F32B"},
    {"rosto_de_vento", "1F32C"},
    {"cachorro_quente", "1F32D"},
    {"taco", "1F32E"},
    {"burrito", "1F32F"},
    {"castanha", "1F330"},
    {"muda_de_planta", "1F331"},
    {"conifera", "1F332"},
    {"arvore_caidica", "1F333"},
    {"palmeira", "1F334"},
    {"cacto", "1F335"},
    {"pimenta", "1F336"},
    {"tulipa", "1F337"},
    {"flor_de_cerejeira", "1F338"},
    {"rosa", "1F339"},
    {"hibisco", "1F33A"},
    {"girassol", "1F33B"},
    {"flor", "1F33C"},
    {"milho", "1F33D"},
    {"planta_de_arroz", "1F33E"},
    {"erva", "1F33F"},
    {"trevo_de_quatro_folhas", "1F340"},
    {"folha_de_bordo", "1F341"},
    {"folhas_caidas", "1F342"},
    {"folha_ao_vento", "1F343"},
    {"cogumelo", "1F344"},
    {"tomate", "1F345"},
    {"berinjela", "1F346"},
    {"uvas", "1F347"},
    {"melao", "1F348"},
    {"melancia", "1F349"},
    {"tangerina", "1F34A"},
    {"limao", "1F34B"},
    {"banana", "1F34C"},
    {"abacaxi", "1F34D"},
    {"maca_vermelha", "1F34E"},
    {"maca_verde", "1F34F"},
    {"pera", "1F350"},
    {"pessego", "1F351"},
    {"cereja", "1F352"},
    {"morango", "1F353"},
    {"hamburguer", "1F354"},
    {"pizza", "1F355"},
    {"carne", "1F356"},
    {"coxa_de_frango", "1F357"},
    {"biscoito_de_arroz", "1F358"},
    {"arroz_japones", "1F359"},
    {"arroz_cozido", "1F35A"},
    {"arroz_com_curry", "1F35B"},
    {"lamen", "1F35C"},
    {"espaguete", "1F35D"},
    {"pao", "1F35E"},
    {"batata_frita", "1F35F"},
    {"batata_assada", "1F360"},
    {"dango", "1F361"},
    {"oden", "1F362"},
    {"sushi", "1F363"},
    {"camarao_frito", "1F364"},
    {"bolinho_de_peixe", "1F365"},
    {"sorvete_italiano", "1F366"},
    {"raspadinha_de_gelo", "1F367"},
    {"sorvete", "1F368"},
    {"donut", "1F369"},
    {"biscoito", "1F36A"},
    {"chocolate", "1F36B"},
    {"bala", "1F36C"},
    {"pirulito", "1F36D"},
    {"pudim", "1F36E"},
    {"pote_de_mel", "1F36F"},
    {"pao_de_lo_de_morango", "1F370"},
    {"bento", "1F371"},
    {"panela", "1F372"},
    {"ovo_frito", "1F373"},
    {"garfo_e_faca", "1F374"},
    {"xicara_de_cha_sem_alca", "1F375"},
    {"saque", "1F376"},
    {"vinho", "1F377"},
    {"coquetel", "1F378"},
    {"bebida_tropical", "1F379"},
    {"cerveja", "1F37A"},
    {"canecas_de_cerveja", "1F37B"},
    {"mamadeira", "1F37C"},
    {"prato_com_talheres", "1F37D"},
    {"garrafa_de_champanhe", "1F37E"},
    {"pipoca", "1F37F"},
    {"laco_de_fita", "1F380"},
    {"presente", "1F381"},
    {"bolo_de_aniversario", "1F382"},
    {"abobora_de_halloween", "1F383"},
    {"arvore_de_natal", "1F384"},
    {"papai_noel", "1F385"},
    {"papai_noel_cor1", "1F385-1F3FB"},
    {"papai_noel_cor2", "1F385-1F3FC"},
    {"papai_noel_cor3", "1F385-1F3FD"},
    {"papai_noel_cor4", "1F385-1F3FE"},
    {"papai_noel_cor5", "1F385-1F3FF"},
    {"fogos_de_artificio", "1F386"},
    {"vela_estrela", "1F387"},
    {"balao", "1F388"},
    {"cone_de_festa", "1F389"},
    {"confete", "1F38A"},
    {"arvore_de_tanabata", "1F38B"},
    {"bandeiras_cruzadas", "1F38C"},
    {"decoracao_de_pinhos", "1F38D"},
    {"bonecas_japonesas", "1F38E"},
    {"bandeira_de_carpas", "1F38F"},
    {"carrilhao_de_vento", "1F390"},
    {"contemplacao_da_lua", "1F391"},
    {"mochila", "1F392"},
    {"chapeu_de_formatura", "1F393"},
    {"medalha_militar", "1F396"},
    {"fita_de_lembrete", "1F397"},
    {"microfone_de_estudio", "1F399"},
    {"controle_de_volume", "1F39A"},
    {"botoes_giratorios", "1F39B"},
    {"rolo_de_filmes", "1F39E"},
    {"ingresso_de_cinema", "1F39F"},
    {"carrossel", "1F3A0"},
    {"roda_gigante", "1F3A1"},
    {"montanha_russa", "1F3A2"},
    {"pesca", "1F3A3"},
    {"microfone", "1F3A4"},
    {"camera_de_cinema", "1F3A5"},
    {"cinema", "1F3A6"},
    {"fones_de_ouvido", "1F3A7"},
    {"paleta_de_tintas", "1F3A8"},
    {"cartola", "1F3A9"},
    {"circo", "1F3AA"},
    {"ingresso", "1F3AB"},
    {"claquete", "1F3AC"},
    {"mascara", "1F3AD"},
    {"videogame", "1F3AE"},
    {"no_alvo", "1F3AF"},
    {"caca_niquel", "1F3B0"},
    {"bilhar", "1F3B1"},
    {"jogo_de_dado", "1F3B2"},
    {"boliche", "1F3B3"},
    {"carta_de_flores", "1F3B4"},
    {"nota_musical", "1F3B5"},
    {"notas_musicais", "1F3B6"},
    {"saxofone", "1F3B7"},
    {"guitarra", "1F3B8"},
    {"teclado_musical", "1F3B9"},
    {"trompete", "1F3BA"},
    {"violino", "1F3BB"},
    {"partitura_musical", "1F3BC"},
    {"camiseta_de_corrida", "1F3BD"},
    {"tenis", "1F3BE"},
    {"esqui", "1F3BF"},
    {"bola_de_basquete", "1F3C0"},
    {"bandeira_quadriculada", "1F3C1"},
    {"praticante_de_snowboard", "1F3C2"},
    {"praticante_de_snowboard_cor1", "1F3C2-1F3FB"},
    {"praticante_de_snowboard_cor2", "1F3C2-1F3FC"},
    {"praticante_de_snowboard_cor3", "1F3C2-1F3FD"},
    {"praticante_de_snowboard_cor4", "1F3C2-1F3FE"},
    {"praticante_de_snowboard_cor5", "1F3C2-1F3FF"},
    {"pessoa_correndo", "1F3C3"},
    {"pessoa_correndo_cor1", "1F3C3-1F3FB"},
    {"pessoa_correndo_cor2", "1F3C3-1F3FC"},
    {"pessoa_correndo_cor3", "1F3C3-1F3FD"},
    {"pessoa_correndo_cor4", "1F3C3-1F3FE"},
    {"pessoa_correndo_cor5", "1F3C3-1F3FF"},
    {"surfista", "1F3C4"},
    {"surfista_cor1", "1F3C4-1F3FB"},
    {"surfista_cor2", "1F3C4-1F3FC"},
    {"surfista_cor3", "1F3C4-1F3FD"},
    {"surfista_cor4", "1F3C4-1F3FE"},
    {"surfista_cor5", "1F3C4-1F3FF"},
    {"medalha_esportiva", "1F3C5"},
    {"trofeu", "1F3C6"},
    {"corrida_de_cavalos", "1F3C7"},
    {"corrida_de_cavalos_cor1", "1F3C7-1F3FB"},
    {"corrida_de_cavalos_cor2", "1F3C7-1F3FC"},
    {"corrida_de_cavalos_cor3", "1F3C7-1F3FD"},
    {"corrida_de_cavalos_cor4", "1F3C7-1F3FE"},
    {"corrida_de_cavalos_cor5", "1F3C7-1F3FF"},
    {"bola_de_futebol_americano", "1F3C8"},
    {"bola_de_rugby", "1F3C9"},
    {"pessoa_nadando", "1F3CA"},
    {"pessoa_nadando_cor1", "1F3CA-1F3FB"},
    {"pessoa_nadando_cor2", "1F3CA-1F3FC"},
    {"pessoa_nadando_cor3", "1F3CA-1F3FD"},
    {"pessoa_nadando_cor4", "1F3CA-1F3FE"},
    {"pessoa_nadando_cor5", "1F3CA-1F3FF"},
    {"pessoa_levantando_peso", "1F3CB"},
    {"pessoa_levantando_peso_cor1", "1F3CB-1F3FB"},
    {"pessoa_levantando_peso_cor2", "1F3CB-1F3FC"},
    {"pessoa_levantando_peso_cor3", "1F3CB-1F3FD"},
    {"pessoa_levantando_peso_cor4", "1F3CB-1F3FE"},
    {"pessoa_levantando_peso_cor5", "1F3CB-1F3FF"},
    {"golfista", "1F3CC"},
    {"golfista_cor1", "1F3CC-1F3FB"},
    {"golfista_cor2", "1F3CC-1F3FC"},
    {"golfista_cor3", "1F3CC-1F3FD"},
    {"golfista_cor4", "1F3CC-1F3FE"},
    {"golfista_cor5", "1F3CC-1F3FF"},
    {"motocicleta", "1F3CD"},
    {"carro_de_corrida", "1F3CE"},
    {"criquete", "1F3CF"},
    {"bola_de_volei", "1F3D0"},
    {"hoquei_de_campo", "1F3D1"},
    {"hoquei_no_gelo", "1F3D2"},
    {"pingue_pongue", "1F3D3"},
    {"montanha_com_neve", "1F3D4"},
    {"acampamento", "1F3D5"},
    {"praia_e_guarda_sol", "1F3D6"},
    {"construcao", "1F3D7"},
    {"casas", "1F3D8"},
    {"cidade", "1F3D9"},
    {"casa_abandonada", "1F3DA"},
    {"predio_grego", "1F3DB"},
    {"deserto", "1F3DC"},
    {"ilha_deserta", "1F3DD"},
    {"parque_nacional", "1F3DE"},
    {"estadio", "1F3DF"},
    {"casa", "1F3E0"},
    {"casa_com_jardim", "1F3E1"},
    {"edificio_comercial", "1F3E2"},
    {"correio_japones", "1F3E3"},
    {"correio", "1F3E4"},
    {"hospital", "1F3E5"},
    {"banco", "1F3E6"},
    {"simbolo_de_caixa_automatico", "1F3E7"},
    {"hotel", "1F3E8"},
    {"motel", "1F3E9"},
    {"loja_de_conveniencia", "1F3EA"},
    {"escola", "1F3EB"},
    {"loja_de_departamentos", "1F3EC"},
    {"fabrica", "1F3ED"},
    {"lanterna_vermelha_de_papel", "1F3EE"},
    {"castelo_japones", "1F3EF"},
    {"castelo", "1F3F0"},
    {"bandeira_branca", "1F3F3"},
    {"bandeira_preta", "1F3F4"},
    {"roseta", "1F3F5"},
    {"etiqueta", "1F3F7"},
    {"badminton", "1F3F8"},
    {"arco_e_flecha", "1F3F9"},
    {"anfora", "1F3FA"},
    {"pele_clara", "1F3FB"},
    {"pele_morena_clara", "1F3FC"},
    {"pele_morena", "1F3FD"},
    {"pele_morena_escura", "1F3FE"},
    {"pele_escura", "1F3FF"},
    {"rato", "1F400"},
    {"camundongo", "1F401"},
    {"boi", "1F402"},
    {"bufalo_asiatico", "1F403"},
    {"vaca", "1F404"},
    {"tigre", "1F405"},
    {"leopardo", "1F406"},
    {"coelho", "1F407"},
    {"gato", "1F408"},
    {"dragao", "1F409"},
    {"crocodilo", "1F40A"},
    {"baleia", "1F40B"},
    {"caracol", "1F40C"},
    {"cobra", "1F40D"},
    {"cavalo", "1F40E"},
    {"carneiro", "1F40F"},
    {"cabra", "1F410"},
    {"ovelha", "1F411"},
    {"macaco", "1F412"},
    {"galo", "1F413"},
    {"galinha", "1F414"},
    {"cachorro", "1F415"},
    {"porco", "1F416"},
    {"javali", "1F417"},
    {"elefante", "1F418"},
    {"polvo", "1F419"},
    {"caramujo", "1F41A"},
    {"inseto", "1F41B"},
    {"formiga", "1F41C"},
    {"abelha", "1F41D"},
    {"joaninha", "1F41E"},
    {"peixe", "1F41F"},
    {"peixe_tropical", "1F420"},
    {"baiacu", "1F421"},
    {"tartaruga", "1F422"},
    {"pintinho_chocando", "1F423"},
    {"pintinho_de_perfil", "1F424"},
    {"pintinho_de_frente", "1F425"},
    {"passaro", "1F426"},
    {"pinguim", "1F427"},
    {"coala", "1F428"},
    {"poodle", "1F429"},
    {"camelo", "1F42A"},
    {"camelo_com_duas_corcovas", "1F42B"},
    {"golfinho", "1F42C"},
    {"rosto_de_camundongo", "1F42D"},
    {"rosto_de_vaca", "1F42E"},
    {"rosto_de_tigre", "1F42F"},
    {"rosto_de_coelho", "1F430"},
    {"rosto_de_gato", "1F431"},
    {"rosto_de_dragao", "1F432"},
    {"baleia_esguichando_agua", "1F433"},
    {"rosto_de_cavalo", "1F434"},
    {"rosto_de_macaco", "1F435"},
    {"rosto_de_cachorro", "1F436"},
    {"rosto_de_porco", "1F437"},
    {"sapo", "1F438"},
    {"rosto_de_hamster", "1F439"},
    {"rosto_de_lobo", "1F43A"},
    {"rosto_de_urso", "1F43B"},
    {"rosto_de_panda", "1F43C"},
    {"nariz_de_porco", "1F43D"},
    {"patas", "1F43E"},
    {"esquilo", "1F43F"},
    {"olhos", "1F440"},
    {"olho", "1F441"},
    {"orelha", "1F442"},
    {"orelha_cor1", "1F442-1F3FB"},
    {"orelha_cor2", "1F442-1F3FC"},
    {"orelha_cor3", "1F442-1F3FD"},
    {"orelha_cor4", "1F442-1F3FE"},
    {"orelha_cor5", "1F442-1F3FF"},
    {"nariz", "1F443"},
    {"nariz_cor1", "1F443-1F3FB"},
    {"nariz_cor2", "1F443-1F3FC"},
    {"nariz_cor3", "1F443-1F3FD"},
    {"nariz_cor4", "1F443-1F3FE"},
    {"nariz_cor5", "1F443-1F3FF"},
    {"boca", "1F444"},
    {"lingua", "1F445"},
    {"dorso_da_mao_com_indicador_apontando_para_cima", "1F446"},
    {"dorso_da_mao_com_indicador_apontando_para_cima_cor1", "1F446-1F3FB"},
    {"dorso_da_mao_com_indicador_apontando_para_cima_cor2", "1F446-1F3FC"},
    {"dorso_da_mao_com_indicador_apontando_para_cima_cor3", "1F446-1F3FD"},
    {"dorso_da_mao_com_indicador_apontando_para_cima_cor4", "1F446-1F3FE"},
    {"dorso_da_mao_com_indicador_apontando_para_cima_cor5", "1F446-1F3FF"},
    {"dorso_da_mao_com_indicador_apontando_para_baixo", "1F447"},
    {"dorso_da_mao_com_indicador_apontando_para_baixo_cor1", "1F447-1F3FB"},
    {"dorso_da_mao_com_indicador_apontando_para_baixo_cor2", "1F447-1F3FC"},
    {"dorso_da_mao_com_indicador_apontando_para_baixo_cor3", "1F447-1F3FD"},
    {"dorso_da_mao_com_indicador_apontando_para_baixo_cor4", "1F447-1F3FE"},
    {"dorso_da_mao_com_indicador_apontando_para_baixo_cor5", "1F447-1F3FF"},
    {"dorso_da_mao_com_indicador_apontando_para_a_esquerda", "1F448"},
    {"dorso_da_mao_com_indicador_apontando_para_a_esquerda_cor1", "1F448-1F3FB"},
    {"dorso_da_mao_com_indicador_apontando_para_a_esquerda_cor2", "1F448-1F3FC"},
    {"dorso_da_mao_com_indicador_apontando_para_a_esquerda_cor3", "1F448-1F3FD"},
    {"dorso_da_mao_com_indicador_apontando_para_a_esquerda_cor4", "1F448-1F3FE"},
    {"dorso_da_mao_com_indicador_apontando_para_a_esquerda_cor5", "1F448-1F3FF"},
    {"dorso_da_mao_com_indicador_apontando_para_a_direita", "1F449"},
    {"dorso_da_mao_com_indicador_apontando_para_a_direita_cor1", "1F449-1F3FB"},
    {"dorso_da_mao_com_indicador_apontando_para_a_direita_cor2", "1F449-1F3FC"},
    {"dorso_da_mao_com_indicador_apontando_para_a_direita_cor3", "1F449-1F3FD"},
    {"dorso_da_mao_com_indicador_apontando_para_a_direita_cor4", "1F449-1F3FE"},
    {"dorso_da_mao_com_indicador_apontando_para_a_direita_cor5", "1F449-1F3FF"},
    {"soco", "1F44A"},
    {"soco_cor1", "1F44A-1F3FB"},
    {"soco_cor2", "1F44A-1F3FC"},
    {"soco_cor3", "1F44A-1F3FD"},
    {"soco_cor4", "1F44A-1F3FE"},
    {"soco_cor5", "1F44A-1F3FF"},
    {"mao_acenando", "1F44B"},
    {"mao_acenando_cor1", "1F44B-1F3FB"},
    {"mao_acenando_cor2", "1F44B-1F3FC"},
    {"mao_acenando_cor3", "1F44B-1F3FD"},
    {"mao_acenando_cor4", "1F44B-1F3FE"},
    {"mao_acenando_cor5", "1F44B-1F3FF"},
    {"sinal_de_ok", "1F44C"},
    {"sinal_de_ok_cor1", "1F44C-1F3FB"},
    {"sinal_de_ok_cor2", "1F44C-1F3FC"},
    {"sinal_de_ok_cor3", "1F44C-1F3FD"},
    {"sinal_de_ok_cor4", "1F44C-1F3FE"},
    {"sinal_de_ok_cor5", "1F44C-1F3FF"},
    {"polegar_para_cima", "1F44D"},
    {"polegar_para_cima_cor1", "1F44D-1F3FB"},
    {"polegar_para_cima_cor2", "1F44D-1F3FC"},
    {"polegar_para_cima_cor3", "1F44D-1F3FD"},
    {"polegar_para_cima_cor4", "1F44D-1F3FE"},
    {"polegar_para_cima_cor5", "1F44D-1F3FF"},
    {"polegar_para_baixo", "1F44E"},
    {"polegar_para_baixo_cor1", "1F44E-1F3FB"},
    {"polegar_para_baixo_cor2", "1F44E-1F3FC"},
    {"polegar_para_baixo_cor3", "1F44E-1F3FD"},
    {"polegar_para_baixo_cor4", "1F44E-1F3FE"},
    {"polegar_para_baixo_cor5", "1F44E-1F3FF"},
    {"maos_aplaudindo", "1F44F"},
    {"maos_aplaudindo_cor1", "1F44F-1F3FB"},
    {"maos_aplaudindo_cor2", "1F44F-1F3FC"},
    {"maos_aplaudindo_cor3", "1F44F-1F3FD"},
    {"maos_aplaudindo_cor4", "1F44F-1F3FE"},
    {"maos_aplaudindo_cor5", "1F44F-1F3FF"},
    {"maos_abertas", "1F450"},
    {"maos_abertas_cor1", "1F450-1F3FB"},
    {"maos_abertas_cor2", "1F450-1F3FC"},
    {"maos_abertas_cor3", "1F450-1F3FD"},
    {"maos_abertas_cor4", "1F450-1F3FE"},
    {"maos_abertas_cor5", "1F450-1F3FF"},
    {"coroa", "1F451"},
    {"chapeu_feminino", "1F452"},
    {"oculos", "1F453"},
    {"gravata", "1F454"},
    {"camiseta", "1F455"},
    {"jeans", "1F456"},
    {"vestido", "1F457"},
    {"quimono", "1F458"},
    {"biquini", "1F459"},
    {"roupas_femininas", "1F45A"},
    {"bolsinha", "1F45B"},
    {"bolsa_de_mao", "1F45C"},
    {"bolsa_pequena", "1F45D"},
    {"sapato_masculino", "1F45E"},
    {"tenis_de_corrida", "1F45F"},
    {"sapato_de_salto_alto", "1F460"},
    {"sandalia_feminina", "1F461"},
    {"bota_feminina", "1F462"},
    {"pegadas", "1F463"},
    {"silhueta_de_busto", "1F464"},
    {"silhueta_de_bustos", "1F465"},
    {"menino", "1F466"},
    {"menino_cor1", "1F466-1F3FB"},
    {"menino_cor2", "1F466-1F3FC"},
    {"menino_cor3", "1F466-1F3FD"},
    {"menino_cor4", "1F466-1F3FE"},
    {"menino_cor5", "1F466-1F3FF"},
    {"menina", "1F467"},
    {"menina_cor1", "1F467-1F3FB"},
    {"menina_cor2", "1F467-1F3FC"},
    {"menina_cor3", "1F467-1F3FD"},
    {"menina_cor4", "1F467-1F3FE"},
    {"menina_cor5", "1F467-1F3FF"},
    {"homem", "1F468"},
    {"homem_cor1", "1F468-1F3FB"},
    {"homem_cor2", "1F468-1F3FC"},
    {"homem_cor3", "1F468-1F3FD"},
    {"homem_cor4", "1F468-1F3FE"},
    {"homem_cor5", "1F468-1F3FF"},
    {"mulher", "1F469"},
    {"mulher_cor1", "1F469-1F3FB"},
    {"mulher_cor2", "1F469-1F3FC"},
    {"mulher_cor3", "1F469-1F3FD"},
    {"mulher_cor4", "1F469-1F3FE"},
    {"mulher_cor5", "1F469-1F3FF"},
    {"familia", "1F46A"},
    {"homem_e_mulher_de_maos_dadas", "1F46B"},
    {"homem_e_mulher_de_maos_dadas_cor1", "1F46B-1F3FB"},
    {"homem_e_mulher_de_maos_dadas_cor2", "1F46B-1F3FC"},
    {"homem_e_mulher_de_maos_dadas_cor3", "1F46B-1F3FD"},
    {"homem_e_mulher_de_maos_dadas_cor4", "1F46B-1F3FE"},
    {"homem_e_mulher_de_maos_dadas_cor5", "1F46B-1F3FF"},
    {"homem_e_mulher_de_maos_dadas_cor1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"homem_e_mulher_de_maos_dadas_cor1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"homem_e_mulher_de_maos_dadas_cor1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"homem_e_mulher_de_maos_dadas_cor1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"homem_e_mulher_de_maos_dadas_cor2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"homem_e_mulher_de_maos_dadas_cor2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"homem_e_mulher_de_maos_dadas_cor2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"homem_e_mulher_de_maos_dadas_cor2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"homem_e_mulher_de_maos_dadas_cor3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"homem_e_mulher_de_maos_dadas_cor3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"homem_e_mulher_de_maos_dadas_cor3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"homem_e_mulher_de_maos_dadas_cor3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"homem_e_mulher_de_maos_dadas_cor4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"homem_e_mulher_de_maos_dadas_cor4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"homem_e_mulher_de_maos_dadas_cor4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"homem_e_mulher_de_maos_dadas_cor4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"homem_e_mulher_de_maos_dadas_cor5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"homem_e_mulher_de_maos_dadas_cor5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"homem_e_mulher_de_maos_dadas_cor5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"homem_e_mulher_de_maos_dadas_cor5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"dois_homens_de_maos_dadas", "1F46C"},
    {"dois_homens_de_maos_dadas_cor1", "1F46C-1F3FB"},
    {"dois_homens_de_maos_dadas_cor2", "1F46C-1F3FC"},
    {"dois_homens_de_maos_dadas_cor3", "1F46C-1F3FD"},
    {"dois_homens_de_maos_dadas_cor4", "1F46C-1F3FE"},
    {"dois_homens_de_maos_dadas_cor5", "1F46C-1F3FF"},
    {"dois_homens_de_maos_dadas_cor1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"dois_homens_de_maos_dadas_cor1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"dois_homens_de_maos_dadas_cor1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"dois_homens_de_maos_dadas_cor1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"dois_homens_de_maos_dadas_cor2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"dois_homens_de_maos_dadas_cor2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"dois_homens_de_maos_dadas_cor2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"dois_homens_de_maos_dadas_cor2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"dois_homens_de_maos_dadas_cor3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"dois_homens_de_maos_dadas_cor3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"dois_homens_de_maos_dadas_cor3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"dois_homens_de_maos_dadas_cor3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"dois_homens_de_maos_dadas_cor4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"dois_homens_de_maos_dadas_cor4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"dois_homens_de_maos_dadas_cor4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"dois_homens_de_maos_dadas_cor4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"dois_homens_de_maos_dadas_cor5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"dois_homens_de_maos_dadas_cor5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"dois_homens_de_maos_dadas_cor5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"dois_homens_de_maos_dadas_cor5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"duas_mulheres_de_maos_dadas", "1F46D"},
    {"duas_mulheres_de_maos_dadas_cor1", "1F46D-1F3FB"},
    {"duas_mulheres_de_maos_dadas_cor2", "1F46D-1F3FC"},
    {"duas_mulheres_de_maos_dadas_cor3", "1F46D-1F3FD"},
    {"duas_mulheres_de_maos_dadas_cor4", "1F46D-1F3FE"},
    {"duas_mulheres_de_maos_dadas_cor5", "1F46D-1F3FF"},
    {"duas_mulheres_de_maos_dadas_cor1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"duas_mulheres_de_maos_dadas_cor1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"duas_mulheres_de_maos_dadas_cor1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"duas_mulheres_de_maos_dadas_cor1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"duas_mulheres_de_maos_dadas_cor2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"duas_mulheres_de_maos_dadas_cor2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"duas_mulheres_de_maos_dadas_cor2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"duas_mulheres_de_maos_dadas_cor2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"duas_mulheres_de_maos_dadas_cor3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"duas_mulheres_de_maos_dadas_cor3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"duas_mulheres_de_maos_dadas_cor3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"duas_mulheres_de_maos_dadas_cor3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"duas_mulheres_de_maos_dadas_cor4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"duas_mulheres_de_maos_dadas_cor4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"duas_mulheres_de_maos_dadas_cor4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"duas_mulheres_de_maos_dadas_cor4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"duas_mulheres_de_maos_dadas_cor5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"duas_mulheres_de_maos_dadas_cor5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"duas_mulheres_de_maos_dadas_cor5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"duas_mulheres_de_maos_dadas_cor5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"policial", "1F46E"},
    {"policial_cor1", "1F46E-1F3FB"},
    {"policial_cor2", "1F46E-1F3FC"},
    {"policial_cor3", "1F46E-1F3FD"},
    {"policial_cor4", "1F46E-1F3FE"},
    {"policial_cor5", "1F46E-1F3FF"},
    {"pessoas_com_orelhas_de_coelho", "1F46F"},
    {"pessoas_com_orelhas_de_coelho_cor1", "1F46F-1F3FB"},
    {"pessoas_com_orelhas_de_coelho_cor2", "1F46F-1F3FC"},
    {"pessoas_com_orelhas_de_coelho_cor3", "1F46F-1F3FD"},
    {"pessoas_com_orelhas_de_coelho_cor4", "1F46F-1F3FE"},
    {"pessoas_com_orelhas_de_coelho_cor5", "1F46F-1F3FF"},
    {"pessoas_com_orelhas_de_coelho_cor1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"pessoas_com_orelhas_de_coelho_cor1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"pessoas_com_orelhas_de_coelho_cor1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"pessoas_com_orelhas_de_coelho_cor1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"pessoas_com_orelhas_de_coelho_cor2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"pessoas_com_orelhas_de_coelho_cor2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"pessoas_com_orelhas_de_coelho_cor2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"pessoas_com_orelhas_de_coelho_cor2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"pessoas_com_orelhas_de_coelho_cor3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"pessoas_com_orelhas_de_coelho_cor3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"pessoas_com_orelhas_de_coelho_cor3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"pessoas_com_orelhas_de_coelho_cor3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"pessoas_com_orelhas_de_coelho_cor4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"pessoas_com_orelhas_de_coelho_cor4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"pessoas_com_orelhas_de_coelho_cor4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"pessoas_com_orelhas_de_coelho_cor4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"pessoas_com_orelhas_de_coelho_cor5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"pessoas_com_orelhas_de_coelho_cor5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"pessoas_com_orelhas_de_coelho_cor5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"pessoas_com_orelhas_de_coelho_cor5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"pessoa_com_veu", "1F470"},
    {"pessoa_com_veu_cor1", "1F470-1F3FB"},
    {"pessoa_com_veu_cor2", "1F470-1F3FC"},
    {"pessoa_com_veu_cor3", "1F470-1F3FD"},
    {"pessoa_com_veu_cor4", "1F470-1F3FE"},
    {"pessoa_com_veu_cor5", "1F470-1F3FF"},
    {"pessoa_cabelo_louro", "1F471"},
    {"pessoa_cabelo_louro_cor1", "1F471-1F3FB"},
    {"pessoa_cabelo_louro_cor2", "1F471-1F3FC"},
    {"pessoa_cabelo_louro_cor3", "1F471-1F3FD"},
    {"pessoa_cabelo_louro_cor4", "1F471-1F3FE"},
    {"pessoa_cabelo_louro_cor5", "1F471-1F3FF"},
    {"homem_de_bone", "1F472"},
    {"homem_de_bone_cor1", "1F472-1F3FB"},
    {"homem_de_bone_cor2", "1F472-1F3FC"},
    {"homem_de_bone_cor3", "1F472-1F3FD"},
    {"homem_de_bone_cor4", "1F472-1F3FE"},
    {"homem_de_bone_cor5", "1F472-1F3FF"},
    {"pessoa_com_turbante", "1F473"},
    {"pessoa_com_turbante_cor1", "1F473-1F3FB"},
    {"pessoa_com_turbante_cor2", "1F473-1F3FC"},
    {"pessoa_com_turbante_cor3", "1F473-1F3FD"},
    {"pessoa_com_turbante_cor4", "1F473-1F3FE"},
    {"pessoa_com_turbante_cor5", "1F473-1F3FF"},
    {"homem_idoso", "1F474"},
    {"homem_idoso_cor1", "1F474-1F3FB"},
    {"homem_idoso_cor2", "1F474-1F3FC"},
    {"homem_idoso_cor3", "1F474-1F3FD"},
    {"homem_idoso_cor4", "1F474-1F3FE"},
    {"homem_idoso_cor5", "1F474-1F3FF"},
    {"idosa", "1F475"},
    {"idosa_cor1", "1F475-1F3FB"},
    {"idosa_cor2", "1F475-1F3FC"},
    {"idosa_cor3", "1F475-1F3FD"},
    {"idosa_cor4", "1F475-1F3FE"},
    {"idosa_cor5", "1F475-1F3FF"},
    {"bebe", "1F476"},
    {"bebe_cor1", "1F476-1F3FB"},
    {"bebe_cor2", "1F476-1F3FC"},
    {"bebe_cor3", "1F476-1F3FD"},
    {"bebe_cor4", "1F476-1F3FE"},
    {"bebe_cor5", "1F476-1F3FF"},
    {"trabalhador_de_construcao_civil", "1F477"},
    {"trabalhador_de_construcao_civil_cor1", "1F477-1F3FB"},
    {"trabalhador_de_construcao_civil_cor2", "1F477-1F3FC"},
    {"trabalhador_de_construcao_civil_cor3", "1F477-1F3FD"},
    {"trabalhador_de_construcao_civil_cor4", "1F477-1F3FE"},
    {"trabalhador_de_construcao_civil_cor5", "1F477-1F3FF"},
    {"princesa", "1F478"},
    {"princesa_cor1", "1F478-1F3FB"},
    {"princesa_cor2", "1F478-1F3FC"},
    {"princesa_cor3", "1F478-1F3FD"},
    {"princesa_cor4", "1F478-1F3FE"},
    {"princesa_cor5", "1F478-1F3FF"},
    {"ogro", "1F479"},
    {"duende_japones", "1F47A"},
    {"fantasma", "1F47B"},
    {"bebe_anjo", "1F47C"},
    {"bebe_anjo_cor1", "1F47C-1F3FB"},
    {"bebe_anjo_cor2", "1F47C-1F3FC"},
    {"bebe_anjo_cor3", "1F47C-1F3FD"},
    {"bebe_anjo_cor4", "1F47C-1F3FE"},
    {"bebe_anjo_cor5", "1F47C-1F3FF"},
    {"alienigena", "1F47D"},
    {"monstro_alienigena", "1F47E"},
    {"rosto_zangado_com_chifres", "1F47F"},
    {"caveira", "1F480"},
    {"pessoa_com_a_palma_virada_para_cima", "1F481"},
    {"pessoa_com_a_palma_virada_para_cima_cor1", "1F481-1F3FB"},
    {"pessoa_com_a_palma_virada_para_cima_cor2", "1F481-1F3FC"},
    {"pessoa_com_a_palma_virada_para_cima_cor3", "1F481-1F3FD"},
    {"pessoa_com_a_palma_virada_para_cima_cor4", "1F481-1F3FE"},
    {"pessoa_com_a_palma_virada_para_cima_cor5", "1F481-1F3FF"},
    {"guarda", "1F482"},
    {"guarda_cor1", "1F482-1F3FB"},
    {"guarda_cor2", "1F482-1F3FC"},
    {"guarda_cor3", "1F482-1F3FD"},
    {"guarda_cor4", "1F482-1F3FE"},
    {"guarda_cor5", "1F482-1F3FF"},
    {"mulher_dancando", "1F483"},
    {"mulher_dancando_cor1", "1F483-1F3FB"},
    {"mulher_dancando_cor2", "1F483-1F3FC"},
    {"mulher_dancando_cor3", "1F483-1F3FD"},
    {"mulher_dancando_cor4", "1F483-1F3FE"},
    {"mulher_dancando_cor5", "1F483-1F3FF"},
    {"batom", "1F484"},
    {"esmalte_de_unha", "1F485"},
    {"esmalte_de_unha_cor1", "1F485-1F3FB"},
    {"esmalte_de_unha_cor2", "1F485-1F3FC"},
    {"esmalte_de_unha_cor3", "1F485-1F3FD"},
    {"esmalte_de_unha_cor4", "1F485-1F3FE"},
    {"esmalte_de_unha_cor5", "1F485-1F3FF"},
    {"pessoa_recebendo_massagem_facial", "1F486"},
    {"pessoa_recebendo_massagem_facial_cor1", "1F486-1F3FB"},
    {"pessoa_recebendo_massagem_facial_cor2", "1F486-1F3FC"},
    {"pessoa_recebendo_massagem_facial_cor3", "1F486-1F3FD"},
    {"pessoa_recebendo_massagem_facial_cor4", "1F486-1F3FE"},
    {"pessoa_recebendo_massagem_facial_cor5", "1F486-1F3FF"},
    {"pessoa_cortando_o_cabelo", "1F487"},
    {"pessoa_cortando_o_cabelo_cor1", "1F487-1F3FB"},
    {"pessoa_cortando_o_cabelo_cor2", "1F487-1F3FC"},
    {"pessoa_cortando_o_cabelo_cor3", "1F487-1F3FD"},
    {"pessoa_cortando_o_cabelo_cor4", "1F487-1F3FE"},
    {"pessoa_cortando_o_cabelo_cor5", "1F487-1F3FF"},
    {"barbearia", "1F488"},
    {"seringa", "1F489"},
    {"comprimido", "1F48A"},
    {"marca_de_beijo", "1F48B"},
    {"carta_de_amor", "1F48C"},
    {"anel", "1F48D"},
    {"pedra_preciosa", "1F48E"},
    {"beijo", "1F48F"},
    {"beijo_cor1", "1F48F-1F3FB"},
    {"beijo_cor2", "1F48F-1F3FC"},
    {"beijo_cor3", "1F48F-1F3FD"},
    {"beijo_cor4", "1F48F-1F3FE"},
    {"beijo_cor5", "1F48F-1F3FF"},
    {"beijo_cor1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"beijo_cor1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"beijo_cor1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"beijo_cor1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"beijo_cor2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"beijo_cor2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"beijo_cor2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"beijo_cor2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"beijo_cor3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"beijo_cor3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"beijo_cor3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"beijo_cor3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"beijo_cor4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"beijo_cor4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"beijo_cor4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"beijo_cor4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"beijo_cor5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"beijo_cor5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"beijo_cor5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"beijo_cor5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"buque", "1F490"},
    {"casal_apaixonado", "1F491"},
    {"casal_apaixonado_cor1", "1F491-1F3FB"},
    {"casal_apaixonado_cor2", "1F491-1F3FC"},
    {"casal_apaixonado_cor3", "1F491-1F3FD"},
    {"casal_apaixonado_cor4", "1F491-1F3FE"},
    {"casal_apaixonado_cor5", "1F491-1F3FF"},
    {"casal_apaixonado_cor1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"casal_apaixonado_cor1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"casal_apaixonado_cor1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"casal_apaixonado_cor1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"casal_apaixonado_cor2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"casal_apaixonado_cor2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"casal_apaixonado_cor2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"casal_apaixonado_cor2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"casal_apaixonado_cor3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"casal_apaixonado_cor3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"casal_apaixonado_cor3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"casal_apaixonado_cor3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"casal_apaixonado_cor4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"casal_apaixonado_cor4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"casal_apaixonado_cor4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"casal_apaixonado_cor4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"casal_apaixonado_cor5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"casal_apaixonado_cor5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"casal_apaixonado_cor5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"casal_apaixonado_cor5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"capela_de_casamento", "1F492"},
    {"coracao_pulsante", "1F493"},
    {"coracao_partido", "1F494"},
    {"dois_coracoes", "1F495"},
    {"coracao_brilhante", "1F496"},
    {"coracao_crescendo", "1F497"},
    {"coracao_com_flecha", "1F498"},
    {"coracao_azul", "1F499"},
    {"coracao_verde", "1F49A"},
    {"coracao_amarelo", "1F49B"},
    {"coracao_roxo", "1F49C"},
    {"coracao_com_fita", "1F49D"},
    {"coracoes_girando", "1F49E"},
    {"coracao_decorativo", "1F49F"},
    {"diamante_com_um_ponto", "1F4A0"},
    {"lampada", "1F4A1"},
    {"simbolo_de_raiva", "1F4A2"},
    {"bomba", "1F4A3"},
    {"zzz", "1F4A4"},
    {"colisao", "1F4A5"},
    {"pingos_de_suor", "1F4A6"},
    {"gota", "1F4A7"},
    {"rapidez", "1F4A8"},
    {"coco", "1F965"},
    {"biceps", "1F4AA"},
    {"biceps_cor1", "1F4AA-1F3FB"},
    {"biceps_cor2", "1F4AA-1F3FC"},
    {"biceps_cor3", "1F4AA-1F3FD"},
    {"biceps_cor4", "1F4AA-1F3FE"},
    {"biceps_cor5", "1F4AA-1F3FF"},
    {"zonzo", "1F4AB"},
    {"balao_de_dialogo", "1F4AC"},
    {"balao_de_pensamento", "1F4AD"},
    {"flor_branca", "1F4AE"},
    {"cem_pontos", "1F4AF"},
    {"saco_de_dinheiro", "1F4B0"},
    {"cambio_de_moeda", "1F4B1"},
    {"cifrao", "1F4B2"},
    {"cartao_de_credito", "1F4B3"},
    {"nota_de_iene", "1F4B4"},
    {"nota_de_dolar", "1F4B5"},
    {"nota_de_euro", "1F4B6"},
    {"nota_de_libra", "1F4B7"},
    {"dinheiro_voando", "1F4B8"},
    {"grafico_subindo_com_iene", "1F4B9"},
    {"assento", "1F4BA"},
    {"laptop", "1F4BB"},
    {"maleta", "1F4BC"},
    {"minidisc", "1F4BD"},
    {"disquete", "1F4BE"},
    {"cd", "1F4BF"},
    {"dvd", "1F4C0"},
    {"pasta_de_arquivos", "1F4C1"},
    {"pasta_de_arquivos_aberta", "1F4C2"},
    {"pagina_dobrada", "1F4C3"},
    {"pagina_voltada_para_cima", "1F4C4"},
    {"calendario", "1F4C5"},
    {"calendario_com_folhas_destacaveis", "1F4C6"},
    {"indice_de_cartoes", "1F4C7"},
    {"grafico_subindo", "1F4C8"},
    {"grafico_caindo", "1F4C9"},
    {"grafico_de_barras", "1F4CA"},
    {"prancheta", "1F4CB"},
    {"tacha", "1F4CC"},
    {"tacha_redonda", "1F4CD"},
    {"clipe_de_papel", "1F4CE"},
    {"regua_reta", "1F4CF"},
    {"regua_triangular", "1F4D0"},
    {"marcadores_de_pagina", "1F4D1"},
    {"livro_contabil", "1F4D2"},
    {"caderno", "1F4D3"},
    {"caderno_decorado", "1F4D4"},
    {"livro_fechado", "1F4D5"},
    {"livro_aberto", "1F4D6"},
    {"livro_verde", "1F4D7"},
    {"livro_azul", "1F4D8"},
    {"livro_laranja", "1F4D9"},
    {"livros", "1F4DA"},
    {"cracha", "1F4DB"},
    {"pergaminho", "1F4DC"},
    {"memorando", "1F4DD"},
    {"telefone", "1F4DE"},
    {"pager", "1F4DF"},
    {"fax", "1F4E0"},
    {"antena_parabolica", "1F4E1"},
    {"buzina", "1F4E2"},
    {"megafone", "1F4E3"},
    {"bandeja_de_saida", "1F4E4"},
    {"bandeja_de_entrada", "1F4E5"},
    {"pacote", "1F4E6"},
    {"email", "1F4E7"},
    {"envelope_chegando", "1F4E8"},
    {"envelope_com_seta", "1F4E9"},
    {"caixa_de_correio_fechada_com_bandeira_abaixada", "1F4EA"},
    {"caixa_de_correio_fechada_com_bandeira_levantada", "1F4EB"},
    {"caixa_de_correio_aberta_com_bandeira_levantada", "1F4EC"},
    {"caixa_de_correio_aberta_com_bandeira_abaixada", "1F4ED"},
    {"caixa_de_correio", "1F4EE"},
    {"corneta_postal", "1F4EF"},
    {"jornal", "1F4F0"},
    {"telefone_celular", "1F4F1"},
    {"telefone_celular_com_seta", "1F4F2"},
    {"modo_vibratorio", "1F4F3"},
    {"telefone_celular_desligado", "1F4F4"},
    {"proibido_o_uso_de_telefone_celular", "1F4F5"},
    {"barras_de_sinal", "1F4F6"},
    {"camera", "1F4F7"},
    {"camera_com_flash", "1F4F8"},
    {"camera_de_video", "1F4F9"},
    {"televisao", "1F4FA"},
    {"radio", "1F4FB"},
    {"videocassete", "1F4FC"},
    {"projetor_de_filmes", "1F4FD"},
    {"rosario_de_oracao", "1F4FF"},
    {"botao_de_musicas_aleatorias", "1F500"},
    {"botao_de_repetir", "1F501"},
    {"botao_de_repetir_uma_unica_faixa", "1F502"},
    {"setas_verticais_no_sentido_horario", "1F503"},
    {"botao_de_setas_em_sentido_anti_horario", "1F504"},
    {"botao_de_diminuir_brilho", "1F505"},
    {"botao_de_aumentar_brilho", "1F506"},
    {"alto_falante_silenciado", "1F507"},
    {"alto_falante_com_volume_baixo", "1F508"},
    {"alto_falante_com_volume_medio", "1F509"},
    {"alto_falante_com_volume_alto", "1F50A"},
    {"pilha", "1F50B"},
    {"tomada_eletrica", "1F50C"},
    {"lupa_para_a_esquerda", "1F50D"},
    {"lupa_para_a_direita", "1F50E"},
    {"cadeado_com_caneta", "1F50F"},
    {"cadeado_fechado_com_chave", "1F510"},
    {"chave", "1F511"},
    {"cadeado", "1F512"},
    {"cadeado_aberto", "1F513"},
    {"sino", "1F514"},
    {"sino_silenciado", "1F515"},
    {"marcador_de_pagina", "1F516"},
    {"link", "1F517"},
    {"botao_de_opcao", "1F518"},
    {"seta_back", "1F519"},
    {"seta_end", "1F51A"},
    {"seta_on", "1F51B"},
    {"seta_soon", "1F51C"},
    {"seta_top", "1F51D"},
    {"proibido_para_menores_de_18_anos", "1F51E"},
    {"tecla_10", "1F51F"},
    {"letras_latinas_maiusculas", "1F520"},
    {"letras_latinas_minusculas", "1F521"},
    {"numeros", "1F522"},
    {"simbolos", "1F523"},
    {"letras_latinas", "1F524"},
    {"fogo", "1F525"},
    {"lanterna", "1F526"},
    {"chave_inglesa", "1F527"},
    {"martelo", "1F528"},
    {"porca_e_parafuso", "1F529"},
    {"faca_de_cozinha", "1F52A"},
    {"pistola_dagua", "1F52B"},
    {"microscopio", "1F52C"},
    {"telescopio", "1F52D"},
    {"bola_de_cristal", "1F52E"},
    {"estrela_de_seis_pontas", "1F52F"},
    {"simbolo_japones_de_principiante", "1F530"},
    {"emblema_de_tridente", "1F531"},
    {"botao_quadrado_preto", "1F532"},
    {"botao_quadrado_branco", "1F533"},
    {"circulo_vermelho", "1F534"},
    {"circulo_azul", "1F535"},
    {"losango_laranja_grande", "1F536"},
    {"losango_azul_grande", "1F537"},
    {"losango_laranja_pequeno", "1F538"},
    {"losango_azul_pequeno", "1F539"},
    {"triangulo_vermelho_para_cima", "1F53A"},
    {"triangulo_vermelho_para_baixo", "1F53B"},
    {"botao_apontando_para_cima", "1F53C"},
    {"botao_apontando_para_baixo", "1F53D"},
    {"om", "1F549"},
    {"pomba_branca", "1F54A"},
    {"caaba_isla", "1F54B"},
    {"mesquita", "1F54C"},
    {"sinagoga", "1F54D"},
    {"menora", "1F54E"},
    {"1_hora", "1F550"},
    {"2_horas", "1F551"},
    {"3_horas", "1F552"},
    {"4_horas", "1F553"},
    {"5_horas", "1F554"},
    {"6_horas", "1F555"},
    {"7_horas", "1F556"},
    {"8_horas", "1F557"},
    {"9_horas", "1F558"},
    {"10_horas", "1F559"},
    {"11_horas", "1F55A"},
    {"12_horas", "1F55B"},
    {"uma_e_meia", "1F55C"},
    {"duas_e_meia", "1F55D"},
    {"tres_e_meia", "1F55E"},
    {"quatro_e_meia", "1F55F"},
    {"cinco_e_meia", "1F560"},
    {"seis_e_meia", "1F561"},
    {"sete_e_meia", "1F562"},
    {"oito_e_meia", "1F563"},
    {"nove_e_meia", "1F564"},
    {"dez_e_meia", "1F565"},
    {"onze_e_meia", "1F566"},
    {"doze_e_meia", "1F567"},
    {"vela", "1F56F"},
    {"relogio_de_mesa", "1F570"},
    {"buraco", "1F573"},
    {"homem_de_terno_levitando", "1F574"},
    {"homem_de_terno_levitando_cor1", "1F574-1F3FB"},
    {"homem_de_terno_levitando_cor2", "1F574-1F3FC"},
    {"homem_de_terno_levitando_cor3", "1F574-1F3FD"},
    {"homem_de_terno_levitando_cor4", "1F574-1F3FE"},
    {"homem_de_terno_levitando_cor5", "1F574-1F3FF"},
    {"detetive", "1F575"},
    {"detetive_cor1", "1F575-1F3FB"},
    {"detetive_cor2", "1F575-1F3FC"},
    {"detetive_cor3", "1F575-1F3FD"},
    {"detetive_cor4", "1F575-1F3FE"},
    {"detetive_cor5", "1F575-1F3FF"},
    {"oculos_escuros", "1F576"},
    {"aranha", "1F577"},
    {"teia_de_aranha", "1F578"},
    {"joystick", "1F579"},
    {"homem_dancando", "1F57A"},
    {"homem_dancando_cor1", "1F57A-1F3FB"},
    {"homem_dancando_cor2", "1F57A-1F3FC"},
    {"homem_dancando_cor3", "1F57A-1F3FD"},
    {"homem_dancando_cor4", "1F57A-1F3FE"},
    {"homem_dancando_cor5", "1F57A-1F3FF"},
    {"clipes_de_papel_conectados", "1F587"},
    {"caneta", "1F58A"},
    {"caneta_tinteiro", "1F58B"},
    {"pincel", "1F58C"},
    {"giz_de_cera", "1F58D"},
    {"mao_aberta_com_os_dedos_separados", "1F590"},
    {"mao_aberta_com_os_dedos_separados_cor1", "1F590-1F3FB"},
    {"mao_aberta_com_os_dedos_separados_cor2", "1F590-1F3FC"},
    {"mao_aberta_com_os_dedos_separados_cor3", "1F590-1F3FD"},
    {"mao_aberta_com_os_dedos_separados_cor4", "1F590-1F3FE"},
    {"mao_aberta_com_os_dedos_separados_cor5", "1F590-1F3FF"},
    {"dedo_do_meio", "1F595"},
    {"dedo_do_meio_cor1", "1F595-1F3FB"},
    {"dedo_do_meio_cor2", "1F595-1F3FC"},
    {"dedo_do_meio_cor3", "1F595-1F3FD"},
    {"dedo_do_meio_cor4", "1F595-1F3FE"},
    {"dedo_do_meio_cor5", "1F595-1F3FF"},
    {"saudacao_vulcana", "1F596"},
    {"saudacao_vulcana_cor1", "1F596-1F3FB"},
    {"saudacao_vulcana_cor2", "1F596-1F3FC"},
    {"saudacao_vulcana_cor3", "1F596-1F3FD"},
    {"saudacao_vulcana_cor4", "1F596-1F3FE"},
    {"saudacao_vulcana_cor5", "1F596-1F3FF"},
    {"coracao_preto", "1F5A4"},
    {"computador_de_mesa", "1F5A5"},
    {"impressora", "1F5A8"},
    {"mouse", "1F5B1"},
    {"trackball", "1F5B2"},
    {"quadro_emoldurado", "1F5BC"},
    {"divisores_de_pastas", "1F5C2"},
    {"caixa_de_arquivos", "1F5C3"},
    {"gavetas_de_escritorio", "1F5C4"},
    {"lixeira", "1F5D1"},
    {"bloco_espiral", "1F5D2"},
    {"calendario_espiral", "1F5D3"},
    {"bracadeira", "1F5DC"},
    {"chave_antiga", "1F5DD"},
    {"jornal_enrolado", "1F5DE"},
    {"adaga", "1F5E1"},
    {"silhueta_falando", "1F5E3"},
    {"balao_de_dialogo_a_esquerda", "1F5E8"},
    {"balao_de_raiva_a_direita", "1F5EF"},
    {"urna_eleitoral_com_cedula", "1F5F3"},
    {"mapa_mundi", "1F5FA"},
    {"monte_fuji", "1F5FB"},
    {"torre_de_toquio", "1F5FC"},
    {"estatua_da_liberdade", "1F5FD"},
    {"mapa_do_japao", "1F5FE"},
    {"moai", "1F5FF"},
    {"rosto_risonho", "1F600"},
    {"rosto_contente_com_olhos_sorridentes", "1F601"},
    {"rosto_chorando_de_rir", "1F602"},
    {"rosto_risonho_com_olhos_bem_abertos", "1F603"},
    {"rosto_risonho_com_olhos_sorridentes", "1F604"},
    {"rosto_risonho_com_gota_de_suor", "1F605"},
    {"rosto_risonho_com_olhos_semicerrados", "1F606"},
    {"rosto_sorridente_com_aureola", "1F607"},
    {"rosto_sorridente_com_chifres", "1F608"},
    {"rosto_com_olho_piscando", "1F609"},
    {"rosto_sorridente_com_olhos_sorridentes", "1F60A"},
    {"rosto_saboreando_comida", "1F60B"},
    {"rosto_aliviado", "1F60C"},
    {"rosto_sorridente_com_olhos_de_coracao", "1F60D"},
    {"rosto_sorridente_com_oculos_escuros", "1F60E"},
    {"rosto_com_sorriso_maroto", "1F60F"},
    {"rosto_neutro", "1F610"},
    {"rosto_inexpressivo", "1F611"},
    {"rosto_aborrecido", "1F612"},
    {"rosto_cabisbaixo_com_gota_de_suor", "1F613"},
    {"rosto_deprimido", "1F614"},
    {"rosto_confuso", "1F615"},
    {"rosto_perplexo", "1F616"},
    {"rosto_beijando", "1F617"},
    {"rosto_mandando_um_beijo", "1F618"},
    {"rosto_beijando_com_olhos_sorridentes", "1F619"},
    {"rosto_beijando_com_olhos_fechados", "1F61A"},
    {"rosto_mostrando_a_lingua", "1F61B"},
    {"rosto_piscando_e_com_lingua_para_fora", "1F61C"},
    {"rosto_com_olhos_semicerrados_e_lingua_para_fora", "1F61D"},
    {"rosto_desapontado", "1F61E"},
    {"rosto_preocupado", "1F61F"},
    {"rosto_zangado", "1F620"},
    {"rosto_furioso", "1F621"},
    {"rosto_chorando", "1F622"},
    {"rosto_perseverante", "1F623"},
    {"rosto_soltando_vapor_pelo_nariz", "1F624"},
    {"rosto_triste_mas_aliviado", "1F625"},
    {"rosto_franzido_com_boca_aberta", "1F626"},
    {"rosto_angustiado", "1F627"},
    {"rosto_amedrontado", "1F628"},
    {"rosto_desolado", "1F629"},
    {"rosto_sonolento", "1F62A"},
    {"rosto_cansado", "1F62B"},
    {"rosto_expressando_desagrado", "1F62C"},
    {"rosto_chorando_aos_berros", "1F62D"},
    {"rosto_com_boca_aberta", "1F62E"},
    {"rosto_surpreso", "1F62F"},
    {"rosto_ansioso_com_gota_de_suor", "1F630"},
    {"rosto_gritando_de_medo", "1F631"},
    {"rosto_espantado", "1F632"},
    {"rosto_ruborizado", "1F633"},
    {"rosto_dormindo", "1F634"},
    {"rosto_atordoado", "1F635"},
    {"rosto_sem_boca", "1F636"},
    {"rosto_com_mascara_medica", "1F637"},
    {"rosto_de_gato_sorrindo_com_olhos_sorridentes", "1F638"},
    {"rosto_de_gato_com_lagrimas_de_alegria", "1F639"},
    {"rosto_de_gato_sorrindo", "1F63A"},
    {"rosto_de_gato_sorridente_com_olhos_de_coracao", "1F63B"},
    {"rosto_de_gato_com_sorriso_ironico", "1F63C"},
    {"rosto_de_gato_mandando_um_beijo", "1F63D"},
    {"rosto_de_gato_mal_humorado", "1F63E"},
    {"rosto_de_gato_chorando", "1F63F"},
    {"rosto_de_gato_desolado", "1F640"},
    {"rosto_meio_triste", "1F641"},
    {"rosto_levemente_sorridente", "1F642"},
    {"rosto_de_cabeca_para_baixo", "1F643"},
    {"rosto_com_olhos_revirados", "1F644"},
    {"pessoa_fazendo_gesto_de_nao", "1F645"},
    {"pessoa_fazendo_gesto_de_nao_cor1", "1F645-1F3FB"},
    {"pessoa_fazendo_gesto_de_nao_cor2", "1F645-1F3FC"},
    {"pessoa_fazendo_gesto_de_nao_cor3", "1F645-1F3FD"},
    {"pessoa_fazendo_gesto_de_nao_cor4", "1F645-1F3FE"},
    {"pessoa_fazendo_gesto_de_nao_cor5", "1F645-1F3FF"},
    {"pessoa_fazendo_gesto_de_ok", "1F646"},
    {"pessoa_fazendo_gesto_de_ok_cor1", "1F646-1F3FB"},
    {"pessoa_fazendo_gesto_de_ok_cor2", "1F646-1F3FC"},
    {"pessoa_fazendo_gesto_de_ok_cor3", "1F646-1F3FD"},
    {"pessoa_fazendo_gesto_de_ok_cor4", "1F646-1F3FE"},
    {"pessoa_fazendo_gesto_de_ok_cor5", "1F646-1F3FF"},
    {"pessoa_fazendo_reverencia", "1F647"},
    {"pessoa_fazendo_reverencia_cor1", "1F647-1F3FB"},
    {"pessoa_fazendo_reverencia_cor2", "1F647-1F3FC"},
    {"pessoa_fazendo_reverencia_cor3", "1F647-1F3FD"},
    {"pessoa_fazendo_reverencia_cor4", "1F647-1F3FE"},
    {"pessoa_fazendo_reverencia_cor5", "1F647-1F3FF"},
    {"macaco_que_nao_ve_nada", "1F648"},
    {"macaco_que_nao_ouve_nada", "1F649"},
    {"macaco_que_nao_fala_nada", "1F64A"},
    {"pessoa_levantando_a_mao", "1F64B"},
    {"pessoa_levantando_a_mao_cor1", "1F64B-1F3FB"},
    {"pessoa_levantando_a_mao_cor2", "1F64B-1F3FC"},
    {"pessoa_levantando_a_mao_cor3", "1F64B-1F3FD"},
    {"pessoa_levantando_a_mao_cor4", "1F64B-1F3FE"},
    {"pessoa_levantando_a_mao_cor5", "1F64B-1F3FF"},
    {"maos_para_cima", "1F64C"},
    {"maos_para_cima_cor1", "1F64C-1F3FB"},
    {"maos_para_cima_cor2", "1F64C-1F3FC"},
    {"maos_para_cima_cor3", "1F64C-1F3FD"},
    {"maos_para_cima_cor4", "1F64C-1F3FE"},
    {"maos_para_cima_cor5", "1F64C-1F3FF"},
    {"franzindo_a_sobrancelha", "1F64D"},
    {"franzindo_a_sobrancelha_cor1", "1F64D-1F3FB"},
    {"franzindo_a_sobrancelha_cor2", "1F64D-1F3FC"},
    {"franzindo_a_sobrancelha_cor3", "1F64D-1F3FD"},
    {"franzindo_a_sobrancelha_cor4", "1F64D-1F3FE"},
    {"franzindo_a_sobrancelha_cor5", "1F64D-1F3FF"},
    {"pessoa_fazendo_bico", "1F64E"},
    {"pessoa_fazendo_bico_cor1", "1F64E-1F3FB"},
    {"pessoa_fazendo_bico_cor2", "1F64E-1F3FC"},
    {"pessoa_fazendo_bico_cor3", "1F64E-1F3FD"},
    {"pessoa_fazendo_bico_cor4", "1F64E-1F3FE"},
    {"pessoa_fazendo_bico_cor5", "1F64E-1F3FF"},
    {"maos_juntas", "1F64F"},
    {"maos_juntas_cor1", "1F64F-1F3FB"},
    {"maos_juntas_cor2", "1F64F-1F3FC"},
    {"maos_juntas_cor3", "1F64F-1F3FD"},
    {"maos_juntas_cor4", "1F64F-1F3FE"},
    {"maos_juntas_cor5", "1F64F-1F3FF"},
    {"foguete", "1F680"},
    {"helicoptero", "1F681"},
    {"locomotiva", "1F682"},
    {"vagao_de_trem", "1F683"},
    {"trem_de_alta_velocidade", "1F684"},
    {"trem_de_alta_velocidade_japones", "1F685"},
    {"trem", "1F686"},
    {"metro", "1F687"},
    {"trem_urbano", "1F688"},
    {"estacao", "1F689"},
    {"bonde_eletrico", "1F68A"},
    {"bonde", "1F68B"},
    {"onibus", "1F68C"},
    {"onibus_se_aproximando", "1F68D"},
    {"trolebus", "1F68E"},
    {"ponto_de_onibus", "1F68F"},
    {"van", "1F690"},
    {"ambulancia", "1F691"},
    {"carro_do_corpo_de_bombeiros", "1F692"},
    {"viatura_policial", "1F693"},
    {"viatura_policial_se_aproximando", "1F694"},
    {"taxi", "1F695"},
    {"taxi_se_aproximando", "1F696"},
    {"carro", "1F697"},
    {"carro_se_aproximando", "1F698"},
    {"suv", "1F699"},
    {"caminhao_de_entrega", "1F69A"},
    {"caminhao", "1F69B"},
    {"trator", "1F69C"},
    {"monotrilho", "1F69D"},
    {"estrada_de_ferro_na_montanha", "1F69E"},
    {"estrada_de_ferro_suspensa", "1F69F"},
    {"teleferico_de_montanha", "1F6A0"},
    {"teleferico", "1F6A1"},
    {"navio", "1F6A2"},
    {"pessoa_remando", "1F6A3"},
    {"pessoa_remando_cor1", "1F6A3-1F3FB"},
    {"pessoa_remando_cor2", "1F6A3-1F3FC"},
    {"pessoa_remando_cor3", "1F6A3-1F3FD"},
    {"pessoa_remando_cor4", "1F6A3-1F3FE"},
    {"pessoa_remando_cor5", "1F6A3-1F3FF"},
    {"lancha", "1F6A4"},
    {"semaforo_horizontal", "1F6A5"},
    {"semaforo_vertical", "1F6A6"},
    {"em_construcao", "1F6A7"},
    {"sirene", "1F6A8"},
    {"bandeira_triangular", "1F6A9"},
    {"porta", "1F6AA"},
    {"proibido", "1F6AB"},
    {"cigarro", "1F6AC"},
    {"proibido_fumar", "1F6AD"},
    {"simbolo_de_lixeira", "1F6AE"},
    {"proibido_jogar_lixo_no_chao", "1F6AF"},
    {"agua_potavel", "1F6B0"},
    {"agua_nao_potavel", "1F6B1"},
    {"bicicleta", "1F6B2"},
    {"proibido_andar_de_bicicleta", "1F6B3"},
    {"ciclista", "1F6B4"},
    {"ciclista_cor1", "1F6B4-1F3FB"},
    {"ciclista_cor2", "1F6B4-1F3FC"},
    {"ciclista_cor3", "1F6B4-1F3FD"},
    {"ciclista_cor4", "1F6B4-1F3FE"},
    {"ciclista_cor5", "1F6B4-1F3FF"},
    {"pessoa_fazendo_mountain_bike", "1F6B5"},
    {"pessoa_fazendo_mountain_bike_cor1", "1F6B5-1F3FB"},
    {"pessoa_fazendo_mountain_bike_cor2", "1F6B5-1F3FC"},
    {"pessoa_fazendo_mountain_bike_cor3", "1F6B5-1F3FD"},
    {"pessoa_fazendo_mountain_bike_cor4", "1F6B5-1F3FE"},
    {"pessoa_fazendo_mountain_bike_cor5", "1F6B5-1F3FF"},
    {"pessoa_andando", "1F6B6"},
    {"pessoa_andando_cor1", "1F6B6-1F3FB"},
    {"pessoa_andando_cor2", "1F6B6-1F3FC"},
    {"pessoa_andando_cor3", "1F6B6-1F3FD"},
    {"pessoa_andando_cor4", "1F6B6-1F3FE"},
    {"pessoa_andando_cor5", "1F6B6-1F3FF"},
    {"proibida_a_passagem_de_pedestres", "1F6B7"},
    {"criancas_atravessando", "1F6B8"},
    {"banheiro_masculino", "1F6B9"},
    {"banheiro_feminino", "1F6BA"},
    {"banheiro", "1F6BB"},
    {"simbolo_de_bebe", "1F6BC"},
    {"vaso_sanitario", "1F6BD"},
    {"wc", "1F6BE"},
    {"chuveiro", "1F6BF"},
    {"pessoa_tomando_banho", "1F6C0"},
    {"pessoa_tomando_banho_cor1", "1F6C0-1F3FB"},
    {"pessoa_tomando_banho_cor2", "1F6C0-1F3FC"},
    {"pessoa_tomando_banho_cor3", "1F6C0-1F3FD"},
    {"pessoa_tomando_banho_cor4", "1F6C0-1F3FE"},
    {"pessoa_tomando_banho_cor5", "1F6C0-1F3FF"},
    {"banheira", "1F6C1"},
    {"controle_de_passaportes", "1F6C2"},
    {"alfandega", "1F6C3"},
    {"restituicao_de_bagagem", "1F6C4"},
    {"deposito_de_bagagem", "1F6C5"},
    {"sofa_e_luminaria", "1F6CB"},
    {"pessoa_deitada_na_cama", "1F6CC"},
    {"pessoa_deitada_na_cama_cor1", "1F6CC-1F3FB"},
    {"pessoa_deitada_na_cama_cor2", "1F6CC-1F3FC"},
    {"pessoa_deitada_na_cama_cor3", "1F6CC-1F3FD"},
    {"pessoa_deitada_na_cama_cor4", "1F6CC-1F3FE"},
    {"pessoa_deitada_na_cama_cor5", "1F6CC-1F3FF"},
    {"sacolas_de_compras", "1F6CD"},
    {"sineta", "1F6CE"},
    {"cama", "1F6CF"},
    {"local_de_culto", "1F6D0"},
    {"sinal_de_pare", "1F6D1"},
    {"carrinho_de_compras", "1F6D2"},
    {"templo_hindu", "1F6D5"},
    {"cabana", "1F6D6"},
    {"elevador", "1F6D7"},
    {"deslizamento", "1F6D8"},
    {"sem_fio", "1F6DC"},
    {"escorregador", "1F6DD"},
    {"roda", "1F6DE"},
    {"boia", "1F6DF"},
    {"martelo_e_chave_inglesa", "1F6E0"},
    {"escudo", "1F6E1"},
    {"barril_de_oleo", "1F6E2"},
    {"estrada", "1F6E3"},
    {"trilhos", "1F6E4"},
    {"barco", "1F6E5"},
    {"aviao_pequeno", "1F6E9"},
    {"aviao_decolando", "1F6EB"},
    {"aviao_aterrissando", "1F6EC"},
    {"satelite", "1F6F0"},
    {"cruzeiro", "1F6F3"},
    {"patinete", "1F6F4"},
    {"scooter", "1F6F5"},
    {"canoa", "1F6F6"},
    {"treno", "1F6F7"},
    {"disco_voador", "1F6F8"},
    {"skate", "1F6F9"},
    {"automovel_riquixa", "1F6FA"},
    {"caminhonete", "1F6FB"},
    {"patins_de_rodas", "1F6FC"},
    {"circulo_laranja", "1F7E0"},
    {"circulo_amarelo", "1F7E1"},
    {"circulo_verde", "1F7E2"},
    {"circulo_roxo", "1F7E3"},
    {"circulo_marrom", "1F7E4"},
    {"quadrado_vermelho", "1F7E5"},
    {"quadrado_azul", "1F7E6"},
    {"quadrado_laranja", "1F7E7"},
    {"quadrado_amarelo", "1F7E8"},
    {"quadrado_verde", "1F7E9"},
    {"quadrado_roxo", "1F7EA"},
    {"quadrado_marrom", "1F7EB"},
    {"sinal_de_igual", "1F7F0"},
    {"dedos_comprimidos", "1F90C"},
    {"dedos_comprimidos_cor1", "1F90C-1F3FB"},
    {"dedos_comprimidos_cor2", "1F90C-1F3FC"},
    {"dedos_comprimidos_cor3", "1F90C-1F3FD"},
    {"dedos_comprimidos_cor4", "1F90C-1F3FE"},
    {"dedos_comprimidos_cor5", "1F90C-1F3FF"},
    {"coracao_branco", "1F90D"},
    {"coracao_marrom", "1F90E"},
    {"mao_beliscando", "1F90F"},
    {"mao_beliscando_cor1", "1F90F-1F3FB"},
    {"mao_beliscando_cor2", "1F90F-1F3FC"},
    {"mao_beliscando_cor3", "1F90F-1F3FD"},
    {"mao_beliscando_cor4", "1F90F-1F3FE"},
    {"mao_beliscando_cor5", "1F90F-1F3FF"},
    {"rosto_com_boca_de_ziper", "1F910"},
    {"rosto_com_cifroes", "1F911"},
    {"rosto_com_termometro", "1F912"},
    {"rosto_de_nerd", "1F913"},
    {"rosto_pensativo", "1F914"},
    {"rosto_com_atadura_na_cabeca", "1F915"},
    {"rosto_de_robo", "1F916"},
    {"rosto_abracando", "1F917"},
    {"saudacao_do_rock", "1F918"},
    {"saudacao_do_rock_cor1", "1F918-1F3FB"},
    {"saudacao_do_rock_cor2", "1F918-1F3FC"},
    {"saudacao_do_rock_cor3", "1F918-1F3FD"},
    {"saudacao_do_rock_cor4", "1F918-1F3FE"},
    {"saudacao_do_rock_cor5", "1F918-1F3FF"},
    {"sinal_me_liga", "1F919"},
    {"sinal_me_liga_cor1", "1F919-1F3FB"},
    {"sinal_me_liga_cor2", "1F919-1F3FC"},
    {"sinal_me_liga_cor3", "1F919-1F3FD"},
    {"sinal_me_liga_cor4", "1F919-1F3FE"},
    {"sinal_me_liga_cor5", "1F919-1F3FF"},
    {"dorso_da_mao_levantado", "1F91A"},
    {"dorso_da_mao_levantado_cor1", "1F91A-1F3FB"},
    {"dorso_da_mao_levantado_cor2", "1F91A-1F3FC"},
    {"dorso_da_mao_levantado_cor3", "1F91A-1F3FD"},
    {"dorso_da_mao_levantado_cor4", "1F91A-1F3FE"},
    {"dorso_da_mao_levantado_cor5", "1F91A-1F3FF"},
    {"punho_esquerdo", "1F91B"},
    {"punho_esquerdo_cor1", "1F91B-1F3FB"},
    {"punho_esquerdo_cor2", "1F91B-1F3FC"},
    {"punho_esquerdo_cor3", "1F91B-1F3FD"},
    {"punho_esquerdo_cor4", "1F91B-1F3FE"},
    {"punho_esquerdo_cor5", "1F91B-1F3FF"},
    {"punho_direito", "1F91C"},
    {"punho_direito_cor1", "1F91C-1F3FB"},
    {"punho_direito_cor2", "1F91C-1F3FC"},
    {"punho_direito_cor3", "1F91C-1F3FD"},
    {"punho_direito_cor4", "1F91C-1F3FE"},
    {"punho_direito_cor5", "1F91C-1F3FF"},
    {"aperto_de_maos", "1F91D"},
    {"aperto_de_maos_cor1", "1F91D-1F3FB"},
    {"aperto_de_maos_cor2", "1F91D-1F3FC"},
    {"aperto_de_maos_cor3", "1F91D-1F3FD"},
    {"aperto_de_maos_cor4", "1F91D-1F3FE"},
    {"aperto_de_maos_cor5", "1F91D-1F3FF"},
    {"aperto_de_maos_cor1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"aperto_de_maos_cor1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"aperto_de_maos_cor1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"aperto_de_maos_cor1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"aperto_de_maos_cor2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"aperto_de_maos_cor2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"aperto_de_maos_cor2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"aperto_de_maos_cor2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"aperto_de_maos_cor3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"aperto_de_maos_cor3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"aperto_de_maos_cor3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"aperto_de_maos_cor3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"aperto_de_maos_cor4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"aperto_de_maos_cor4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"aperto_de_maos_cor4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"aperto_de_maos_cor4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"aperto_de_maos_cor5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"aperto_de_maos_cor5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"aperto_de_maos_cor5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"aperto_de_maos_cor5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"dedos_cruzados", "1F91E"},
    {"dedos_cruzados_cor1", "1F91E-1F3FB"},
    {"dedos_cruzados_cor2", "1F91E-1F3FC"},
    {"dedos_cruzados_cor3", "1F91E-1F3FD"},
    {"dedos_cruzados_cor4", "1F91E-1F3FE"},
    {"dedos_cruzados_cor5", "1F91E-1F3FF"},
    {"gesto_de_te_amo", "1F91F"},
    {"gesto_de_te_amo_cor1", "1F91F-1F3FB"},
    {"gesto_de_te_amo_cor2", "1F91F-1F3FC"},
    {"gesto_de_te_amo_cor3", "1F91F-1F3FD"},
    {"gesto_de_te_amo_cor4", "1F91F-1F3FE"},
    {"gesto_de_te_amo_cor5", "1F91F-1F3FF"},
    {"rosto_com_chapeu_de_cauboi", "1F920"},
    {"rosto_de_palhaco", "1F921"},
    {"rosto_nauseado", "1F922"},
    {"rolando_no_chao_de_rir", "1F923"},
    {"rosto_babando", "1F924"},
    {"rosto_de_mentiroso", "1F925"},
    {"pessoa_decepcionada", "1F926"},
    {"pessoa_decepcionada_cor1", "1F926-1F3FB"},
    {"pessoa_decepcionada_cor2", "1F926-1F3FC"},
    {"pessoa_decepcionada_cor3", "1F926-1F3FD"},
    {"pessoa_decepcionada_cor4", "1F926-1F3FE"},
    {"pessoa_decepcionada_cor5", "1F926-1F3FF"},
    {"rosto_espirrando", "1F927"},
    {"rosto_com_sobrancelha_levantada", "1F928"},
    {"rosto_com_olhar_maravilhado", "1F929"},
    {"rosto_bizarro", "1F92A"},
    {"rosto_fazendo_sinal_de_silencio", "1F92B"},
    {"rosto_com_simbolos_na_boca", "1F92C"},
    {"rosto_com_a_mao_sobre_a_boca", "1F92D"},
    {"rosto_vomitando", "1F92E"},
    {"cabeca_explodindo", "1F92F"},
    {"gravida", "1F930"},
    {"gravida_cor1", "1F930-1F3FB"},
    {"gravida_cor2", "1F930-1F3FC"},
    {"gravida_cor3", "1F930-1F3FD"},
    {"gravida_cor4", "1F930-1F3FE"},
    {"gravida_cor5", "1F930-1F3FF"},
    {"amamentando", "1F931"},
    {"amamentando_cor1", "1F931-1F3FB"},
    {"amamentando_cor2", "1F931-1F3FC"},
    {"amamentando_cor3", "1F931-1F3FD"},
    {"amamentando_cor4", "1F931-1F3FE"},
    {"amamentando_cor5", "1F931-1F3FF"},
    {"palmas_unidas_para_cima", "1F932"},
    {"palmas_unidas_para_cima_cor1", "1F932-1F3FB"},
    {"palmas_unidas_para_cima_cor2", "1F932-1F3FC"},
    {"palmas_unidas_para_cima_cor3", "1F932-1F3FD"},
    {"palmas_unidas_para_cima_cor4", "1F932-1F3FE"},
    {"palmas_unidas_para_cima_cor5", "1F932-1F3FF"},
    {"selfie", "1F933"},
    {"selfie_cor1", "1F933-1F3FB"},
    {"selfie_cor2", "1F933-1F3FC"},
    {"selfie_cor3", "1F933-1F3FD"},
    {"selfie_cor4", "1F933-1F3FE"},
    {"selfie_cor5", "1F933-1F3FF"},
    {"principe", "1F934"},
    {"principe_cor1", "1F934-1F3FB"},
    {"principe_cor2", "1F934-1F3FC"},
    {"principe_cor3", "1F934-1F3FD"},
    {"principe_cor4", "1F934-1F3FE"},
    {"principe_cor5", "1F934-1F3FF"},
    {"pessoa_de_smoking", "1F935"},
    {"pessoa_de_smoking_cor1", "1F935-1F3FB"},
    {"pessoa_de_smoking_cor2", "1F935-1F3FC"},
    {"pessoa_de_smoking_cor3", "1F935-1F3FD"},
    {"pessoa_de_smoking_cor4", "1F935-1F3FE"},
    {"pessoa_de_smoking_cor5", "1F935-1F3FF"},
    {"mamae_noel", "1F936"},
    {"mamae_noel_cor1", "1F936-1F3FB"},
    {"mamae_noel_cor2", "1F936-1F3FC"},
    {"mamae_noel_cor3", "1F936-1F3FD"},
    {"mamae_noel_cor4", "1F936-1F3FE"},
    {"mamae_noel_cor5", "1F936-1F3FF"},
    {"pessoa_dando_de_ombros", "1F937"},
    {"pessoa_dando_de_ombros_cor1", "1F937-1F3FB"},
    {"pessoa_dando_de_ombros_cor2", "1F937-1F3FC"},
    {"pessoa_dando_de_ombros_cor3", "1F937-1F3FD"},
    {"pessoa_dando_de_ombros_cor4", "1F937-1F3FE"},
    {"pessoa_dando_de_ombros_cor5", "1F937-1F3FF"},
    {"pessoa_fazendo_estrela", "1F938"},
    {"pessoa_fazendo_estrela_cor1", "1F938-1F3FB"},
    {"pessoa_fazendo_estrela_cor2", "1F938-1F3FC"},
    {"pessoa_fazendo_estrela_cor3", "1F938-1F3FD"},
    {"pessoa_fazendo_estrela_cor4", "1F938-1F3FE"},
    {"pessoa_fazendo_estrela_cor5", "1F938-1F3FF"},
    {"malabarista", "1F939"},
    {"malabarista_cor1", "1F939-1F3FB"},
    {"malabarista_cor2", "1F939-1F3FC"},
    {"malabarista_cor3", "1F939-1F3FD"},
    {"malabarista_cor4", "1F939-1F3FE"},
    {"malabarista_cor5", "1F939-1F3FF"},
    {"esgrimista", "1F93A"},
    {"pessoas_lutando", "1F93C"},
    {"pessoas_lutando_cor1", "1F93C-1F3FB"},
    {"pessoas_lutando_cor2", "1F93C-1F3FC"},
    {"pessoas_lutando_cor3", "1F93C-1F3FD"},
    {"pessoas_lutando_cor4", "1F93C-1F3FE"},
    {"pessoas_lutando_cor5", "1F93C-1F3FF"},
    {"pessoas_lutando_cor1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"pessoas_lutando_cor1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"pessoas_lutando_cor1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"pessoas_lutando_cor1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"pessoas_lutando_cor2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"pessoas_lutando_cor2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"pessoas_lutando_cor2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"pessoas_lutando_cor2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"pessoas_lutando_cor3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"pessoas_lutando_cor3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"pessoas_lutando_cor3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"pessoas_lutando_cor3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"pessoas_lutando_cor4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"pessoas_lutando_cor4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"pessoas_lutando_cor4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"pessoas_lutando_cor4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"pessoas_lutando_cor5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"pessoas_lutando_cor5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"pessoas_lutando_cor5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"pessoas_lutando_cor5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"pessoa_jogando_polo_aquatico", "1F93D"},
    {"pessoa_jogando_polo_aquatico_cor1", "1F93D-1F3FB"},
    {"pessoa_jogando_polo_aquatico_cor2", "1F93D-1F3FC"},
    {"pessoa_jogando_polo_aquatico_cor3", "1F93D-1F3FD"},
    {"pessoa_jogando_polo_aquatico_cor4", "1F93D-1F3FE"},
    {"pessoa_jogando_polo_aquatico_cor5", "1F93D-1F3FF"},
    {"handebol", "1F93E"},
    {"handebol_cor1", "1F93E-1F3FB"},
    {"handebol_cor2", "1F93E-1F3FC"},
    {"handebol_cor3", "1F93E-1F3FD"},
    {"handebol_cor4", "1F93E-1F3FE"},
    {"handebol_cor5", "1F93E-1F3FF"},
    {"mascara_de_mergulho", "1F93F"},
    {"flor_murcha", "1F940"},
    {"tambor", "1F941"},
    {"tacas_brindando", "1F942"},
    {"copo", "1F943"},
    {"colher", "1F944"},
    {"gol", "1F945"},
    {"medalha_de_ouro", "1F947"},
    {"medalha_de_prata", "1F948"},
    {"medalha_de_bronze", "1F949"},
    {"luva_de_boxe", "1F94A"},
    {"quimono_de_artes_marciais", "1F94B"},
    {"pedra_de_curling", "1F94C"},
    {"lacrosse", "1F94D"},
    {"softbol", "1F94E"},
    {"frisbee", "1F94F"},
    {"croissant", "1F950"},
    {"abacate", "1F951"},
    {"pepino", "1F952"},
    {"bacon", "1F953"},
    {"batata", "1F954"},
    {"cenoura", "1F955"},
    {"baguete", "1F956"},
    {"salada_verde", "1F957"},
    {"cacarola_com_comida", "1F958"},
    {"pao_recheado", "1F959"},
    {"ovo", "1F95A"},
    {"copo_de_leite", "1F95B"},
    {"amendoim", "1F95C"},
    {"kiwi", "1F95D"},
    {"panquecas", "1F95E"},
    {"bolinho_asiatico", "1F95F"},
    {"biscoito_da_sorte", "1F960"},
    {"caixa_para_viagem", "1F961"},
    {"hashi", "1F962"},
    {"tigela_com_colher", "1F963"},
    {"copo_com_canudo", "1F964"},
    {"brocolis", "1F966"},
    {"torta", "1F967"},
    {"pretzel", "1F968"},
    {"corte_de_carne", "1F969"},
    {"sanduiche", "1F96A"},
    {"comida_enlatada", "1F96B"},
    {"verdura", "1F96C"},
    {"manga", "1F96D"},
    {"bolo_lunar", "1F96E"},
    {"bagel", "1F96F"},
    {"rosto_sorridente_com_3_coracoes", "1F970"},
    {"rosto_bocejando", "1F971"},
    {"rosto_sorridente_com_lagrima", "1F972"},
    {"rosto_festivo", "1F973"},
    {"rosto_embriagado", "1F974"},
    {"rosto_fervendo_de_calor", "1F975"},
    {"rosto_gelado", "1F976"},
    {"ninja", "1F977"},
    {"ninja_cor1", "1F977-1F3FB"},
    {"ninja_cor2", "1F977-1F3FC"},
    {"ninja_cor3", "1F977-1F3FD"},
    {"ninja_cor4", "1F977-1F3FE"},
    {"ninja_cor5", "1F977-1F3FF"},
    {"rosto_disfarcado", "1F978"},
    {"rosto_segurando_as_lagrimas", "1F979"},
    {"rosto_implorando", "1F97A"},
    {"sari", "1F97B"},
    {"jaleco", "1F97C"},
    {"oculos_de_protecao", "1F97D"},
    {"bota_de_trekking", "1F97E"},
    {"sapatilha", "1F97F"},
    {"caranguejo", "1F980"},
    {"rosto_de_leao", "1F981"},
    {"escorpiao", "1F982"},
    {"peru", "1F983"},
    {"rosto_de_unicornio", "1F984"},
    {"aguia", "1F985"},
    {"pato", "1F986"},
    {"morcego", "1F987"},
    {"tubarao", "1F988"},
    {"coruja", "1F989"},
    {"rosto_de_raposa", "1F98A"},
    {"borboleta", "1F98B"},
    {"cervo", "1F98C"},
    {"gorila", "1F98D"},
    {"lagartixa", "1F98E"},
    {"rinoceronte", "1F98F"},
    {"camarao", "1F990"},
    {"lula", "1F991"},
    {"girafa", "1F992"},
    {"zebra", "1F993"},
    {"porco_espinho", "1F994"},
    {"sauropode", "1F995"},
    {"tiranossauro_rex", "1F996"},
    {"grilo", "1F997"},
    {"canguru", "1F998"},
    {"lhama", "1F999"},
    {"pavao", "1F99A"},
    {"hipopotamo", "1F99B"},
    {"papagaio", "1F99C"},
    {"guaxinim", "1F99D"},
    {"lagosta", "1F99E"},
    {"mosquito", "1F99F"},
    {"microbio", "1F9A0"},
    {"texugo", "1F9A1"},
    {"cisne", "1F9A2"},
    {"mamute", "1F9A3"},
    {"dodo", "1F9A4"},
    {"preguica", "1F9A5"},
    {"lontra", "1F9A6"},
    {"orangotango", "1F9A7"},
    {"gamba", "1F9A8"},
    {"flamingo", "1F9A9"},
    {"ostra", "1F9AA"},
    {"castor", "1F9AB"},
    {"bisao", "1F9AC"},
    {"foca", "1F9AD"},
    {"cao_guia", "1F9AE"},
    {"bengala_para_cegos", "1F9AF"},
    {"cabelo_vermelho", "1F9B0"},
    {"cabelo_cacheado", "1F9B1"},
    {"careca", "1F9B2"},
    {"cabelo_branco", "1F9B3"},
    {"osso", "1F9B4"},
    {"perna", "1F9B5"},
    {"perna_cor1", "1F9B5-1F3FB"},
    {"perna_cor2", "1F9B5-1F3FC"},
    {"perna_cor3", "1F9B5-1F3FD"},
    {"perna_cor4", "1F9B5-1F3FE"},
    {"perna_cor5", "1F9B5-1F3FF"},
    {"pe", "1F9B6"},
    {"pe_cor1", "1F9B6-1F3FB"},
    {"pe_cor2", "1F9B6-1F3FC"},
    {"pe_cor3", "1F9B6-1F3FD"},
    {"pe_cor4", "1F9B6-1F3FE"},
    {"pe_cor5", "1F9B6-1F3FF"},
    {"dente", "1F9B7"},
    {"super_heroi", "1F9B8"},
    {"super_heroi_cor1", "1F9B8-1F3FB"},
    {"super_heroi_cor2", "1F9B8-1F3FC"},
    {"super_heroi_cor3", "1F9B8-1F3FD"},
    {"super_heroi_cor4", "1F9B8-1F3FE"},
    {"super_heroi_cor5", "1F9B8-1F3FF"},
    {"supervilao", "1F9B9"},
    {"supervilao_cor1", "1F9B9-1F3FB"},
    {"supervilao_cor2", "1F9B9-1F3FC"},
    {"supervilao_cor3", "1F9B9-1F3FD"},
    {"supervilao_cor4", "1F9B9-1F3FE"},
    {"supervilao_cor5", "1F9B9-1F3FF"},
    {"colete_salva_vidas", "1F9BA"},
    {"ouvido_com_aparelho_auditivo", "1F9BB"},
    {"ouvido_com_aparelho_auditivo_cor1", "1F9BB-1F3FB"},
    {"ouvido_com_aparelho_auditivo_cor2", "1F9BB-1F3FC"},
    {"ouvido_com_aparelho_auditivo_cor3", "1F9BB-1F3FD"},
    {"ouvido_com_aparelho_auditivo_cor4", "1F9BB-1F3FE"},
    {"ouvido_com_aparelho_auditivo_cor5", "1F9BB-1F3FF"},
    {"cadeira_de_rodas_motorizada", "1F9BC"},
    {"cadeira_de_rodas_manual", "1F9BD"},
    {"braco_mecanico", "1F9BE"},
    {"perna_mecanica", "1F9BF"},
    {"queijo", "1F9C0"},
    {"cupcake", "1F9C1"},
    {"sal", "1F9C2"},
    {"suco_de_caixa", "1F9C3"},
    {"alho", "1F9C4"},
    {"cebola", "1F9C5"},
    {"falafel", "1F9C6"},
    {"waffle", "1F9C7"},
    {"manteiga", "1F9C8"},
    {"mate", "1F9C9"},
    {"cubo_de_gelo", "1F9CA"},
    {"cha_perolado", "1F9CB"},
    {"troll", "1F9CC"},
    {"pessoa_em_pe", "1F9CD"},
    {"pessoa_em_pe_cor1", "1F9CD-1F3FB"},
    {"pessoa_em_pe_cor2", "1F9CD-1F3FC"},
    {"pessoa_em_pe_cor3", "1F9CD-1F3FD"},
    {"pessoa_em_pe_cor4", "1F9CD-1F3FE"},
    {"pessoa_em_pe_cor5", "1F9CD-1F3FF"},
    {"pessoa_ajoelhando", "1F9CE"},
    {"pessoa_ajoelhando_cor1", "1F9CE-1F3FB"},
    {"pessoa_ajoelhando_cor2", "1F9CE-1F3FC"},
    {"pessoa_ajoelhando_cor3", "1F9CE-1F3FD"},
    {"pessoa_ajoelhando_cor4", "1F9CE-1F3FE"},
    {"pessoa_ajoelhando_cor5", "1F9CE-1F3FF"},
    {"pessoa_surda", "1F9CF"},
    {"pessoa_surda_cor1", "1F9CF-1F3FB"},
    {"pessoa_surda_cor2", "1F9CF-1F3FC"},
    {"pessoa_surda_cor3", "1F9CF-1F3FD"},
    {"pessoa_surda_cor4", "1F9CF-1F3FE"},
    {"pessoa_surda_cor5", "1F9CF-1F3FF"},
    {"rosto_com_monoculo", "1F9D0"},
    {"pessoa", "1F9D1"},
    {"pessoa_cor1", "1F9D1-1F3FB"},
    {"pessoa_cor2", "1F9D1-1F3FC"},
    {"pessoa_cor3", "1F9D1-1F3FD"},
    {"pessoa_cor4", "1F9D1-1F3FE"},
    {"pessoa_cor5", "1F9D1-1F3FF"},
    {"crianca", "1F9D2"},
    {"crianca_cor1", "1F9D2-1F3FB"},
    {"crianca_cor2", "1F9D2-1F3FC"},
    {"crianca_cor3", "1F9D2-1F3FD"},
    {"crianca_cor4", "1F9D2-1F3FE"},
    {"crianca_cor5", "1F9D2-1F3FF"},
    {"idoso", "1F9D3"},
    {"idoso_cor1", "1F9D3-1F3FB"},
    {"idoso_cor2", "1F9D3-1F3FC"},
    {"idoso_cor3", "1F9D3-1F3FD"},
    {"idoso_cor4", "1F9D3-1F3FE"},
    {"idoso_cor5", "1F9D3-1F3FF"},
    {"homem_barba", "1F9D4"},
    {"homem_barba_cor1", "1F9D4-1F3FB"},
    {"homem_barba_cor2", "1F9D4-1F3FC"},
    {"homem_barba_cor3", "1F9D4-1F3FD"},
    {"homem_barba_cor4", "1F9D4-1F3FE"},
    {"homem_barba_cor5", "1F9D4-1F3FF"},
    {"mulher_com_veu", "1F9D5"},
    {"mulher_com_veu_cor1", "1F9D5-1F3FB"},
    {"mulher_com_veu_cor2", "1F9D5-1F3FC"},
    {"mulher_com_veu_cor3", "1F9D5-1F3FD"},
    {"mulher_com_veu_cor4", "1F9D5-1F3FE"},
    {"mulher_com_veu_cor5", "1F9D5-1F3FF"},
    {"pessoa_na_sauna", "1F9D6"},
    {"pessoa_na_sauna_cor1", "1F9D6-1F3FB"},
    {"pessoa_na_sauna_cor2", "1F9D6-1F3FC"},
    {"pessoa_na_sauna_cor3", "1F9D6-1F3FD"},
    {"pessoa_na_sauna_cor4", "1F9D6-1F3FE"},
    {"pessoa_na_sauna_cor5", "1F9D6-1F3FF"},
    {"pessoa_escalando", "1F9D7"},
    {"pessoa_escalando_cor1", "1F9D7-1F3FB"},
    {"pessoa_escalando_cor2", "1F9D7-1F3FC"},
    {"pessoa_escalando_cor3", "1F9D7-1F3FD"},
    {"pessoa_escalando_cor4", "1F9D7-1F3FE"},
    {"pessoa_escalando_cor5", "1F9D7-1F3FF"},
    {"pessoa_na_posicao_de_lotus", "1F9D8"},
    {"pessoa_na_posicao_de_lotus_cor1", "1F9D8-1F3FB"},
    {"pessoa_na_posicao_de_lotus_cor2", "1F9D8-1F3FC"},
    {"pessoa_na_posicao_de_lotus_cor3", "1F9D8-1F3FD"},
    {"pessoa_na_posicao_de_lotus_cor4", "1F9D8-1F3FE"},
    {"pessoa_na_posicao_de_lotus_cor5", "1F9D8-1F3FF"},
    {"mago", "1F9D9"},
    {"mago_cor1", "1F9D9-1F3FB"},
    {"mago_cor2", "1F9D9-1F3FC"},
    {"mago_cor3", "1F9D9-1F3FD"},
    {"mago_cor4", "1F9D9-1F3FE"},
    {"mago_cor5", "1F9D9-1F3FF"},
    {"fada", "1F9DA"},
    {"fada_cor1", "1F9DA-1F3FB"},
    {"fada_cor2", "1F9DA-1F3FC"},
    {"fada_cor3", "1F9DA-1F3FD"},
    {"fada_cor4", "1F9DA-1F3FE"},
    {"fada_cor5", "1F9DA-1F3FF"},
    {"vampiro", "1F9DB"},
    {"vampiro_cor1", "1F9DB-1F3FB"},
    {"vampiro_cor2", "1F9DB-1F3FC"},
    {"vampiro_cor3", "1F9DB-1F3FD"},
    {"vampiro_cor4", "1F9DB-1F3FE"},
    {"vampiro_cor5", "1F9DB-1F3FF"},
    {"pessoa_sereia", "1F9DC"},
    {"pessoa_sereia_cor1", "1F9DC-1F3FB"},
    {"pessoa_sereia_cor2", "1F9DC-1F3FC"},
    {"pessoa_sereia_cor3", "1F9DC-1F3FD"},
    {"pessoa_sereia_cor4", "1F9DC-1F3FE"},
    {"pessoa_sereia_cor5", "1F9DC-1F3FF"},
    {"elfo", "1F9DD"},
    {"elfo_cor1", "1F9DD-1F3FB"},
    {"elfo_cor2", "1F9DD-1F3FC"},
    {"elfo_cor3", "1F9DD-1F3FD"},
    {"elfo_cor4", "1F9DD-1F3FE"},
    {"elfo_cor5", "1F9DD-1F3FF"},
    {"genio", "1F9DE"},
    {"zumbi", "1F9DF"},
    {"cerebro", "1F9E0"},
    {"coracao_laranja", "1F9E1"},
    {"bone", "1F9E2"},
    {"cachecol", "1F9E3"},
    {"luvas", "1F9E4"},
    {"casaco", "1F9E5"},
    {"meias", "1F9E6"},
    {"envelope_vermelho", "1F9E7"},
    {"bombinha", "1F9E8"},
    {"quebra_cabeca", "1F9E9"},
    {"tubo_de_ensaio", "1F9EA"},
    {"placa_de_petri", "1F9EB"},
    {"dna", "1F9EC"},
    {"bussola", "1F9ED"},
    {"abaco", "1F9EE"},
    {"extintor_de_incendio", "1F9EF"},
    {"caixa_de_ferramentas", "1F9F0"},
    {"tijolo", "1F9F1"},
    {"ima", "1F9F2"},
    {"mala", "1F9F3"},
    {"frasco_de_locao", "1F9F4"},
    {"carretel", "1F9F5"},
    {"novelo", "1F9F6"},
    {"alfinete_de_seguranca", "1F9F7"},
    {"ursinho_de_pelucia", "1F9F8"},
    {"vassoura", "1F9F9"},
    {"cesta", "1F9FA"},
    {"rolo_de_papel", "1F9FB"},
    {"sabonete", "1F9FC"},
    {"esponja", "1F9FD"},
    {"recibo", "1F9FE"},
    {"olho_grego", "1F9FF"},
    {"sapatilha_de_bale", "1FA70"},
    {"maio", "1FA71"},
    {"cueca", "1FA72"},
    {"shorts", "1FA73"},
    {"chinelo", "1FA74"},
    {"coracao_azul_claro", "1FA75"},
    {"coracao_cinza", "1FA76"},
    {"coracao_rosa", "1FA77"},
    {"gota_de_sangue", "1FA78"},
    {"atadura_adesiva", "1FA79"},
    {"estetoscopio", "1FA7A"},
    {"raio_x", "1FA7B"},
    {"muleta", "1FA7C"},
    {"ioio", "1FA80"},
    {"pipa", "1FA81"},
    {"paraquedas", "1FA82"},
    {"bumerangue", "1FA83"},
    {"varinha_magica", "1FA84"},
    {"pinhata", "1FA85"},
    {"boneca_russa", "1FA86"},
    {"maracas", "1FA87"},
    {"flauta", "1FA88"},
    {"harpa", "1FA89"},
    {"trombone", "1FA8A"},
    {"bau_de_tesouro", "1FA8E"},
    {"pa", "1FA8F"},
    {"planeta_com_aneis", "1FA90"},
    {"cadeira", "1FA91"},
    {"lamina", "1FA92"},
    {"machado", "1FA93"},
    {"lampada_de_oleo", "1FA94"},
    {"banjo", "1FA95"},
    {"capacete_militar", "1FA96"},
    {"acordeao", "1FA97"},
    {"tambor_comprido", "1FA98"},
    {"moeda", "1FA99"},
    {"serrote", "1FA9A"},
    {"chave_de_fenda", "1FA9B"},
    {"escada", "1FA9C"},
    {"gancho", "1FA9D"},
    {"espelho", "1FA9E"},
    {"janela", "1FA9F"},
    {"desentupidor", "1FAA0"},
    {"agulha_de_costura", "1FAA1"},
    {"no", "1FAA2"},
    {"balde", "1FAA3"},
    {"ratoeira", "1FAA4"},
    {"escova_de_dentes", "1FAA5"},
    {"lapide", "1FAA6"},
    {"placa", "1FAA7"},
    {"pedra", "1FAA8"},
    {"globo_de_espelhos", "1FAA9"},
    {"cartao_de_identificacao", "1FAAA"},
    {"pouca_bateria", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"leque_dobravel", "1FAAD"},
    {"pente_de_cabelo", "1FAAE"},
    {"khanda", "1FAAF"},
    {"mosca", "1FAB0"},
    {"minhoca", "1FAB1"},
    {"besouro", "1FAB2"},
    {"barata", "1FAB3"},
    {"vaso_com_planta", "1FAB4"},
    {"madeira", "1FAB5"},
    {"pena", "1FAB6"},
    {"lotus", "1FAB7"},
    {"coral", "1FAB8"},
    {"ninho_vazio", "1FAB9"},
    {"ninho_com_ovos", "1FABA"},
    {"jacinto", "1FABB"},
    {"agua_viva", "1FABC"},
    {"asa", "1FABD"},
    {"arvore_seca", "1FABE"},
    {"ganso", "1FABF"},
    {"coracao_humano", "1FAC0"},
    {"pulmoes", "1FAC1"},
    {"pessoas_se_abracando", "1FAC2"},
    {"homem_gravido", "1FAC3"},
    {"homem_gravido_cor1", "1FAC3-1F3FB"},
    {"homem_gravido_cor2", "1FAC3-1F3FC"},
    {"homem_gravido_cor3", "1FAC3-1F3FD"},
    {"homem_gravido_cor4", "1FAC3-1F3FE"},
    {"homem_gravido_cor5", "1FAC3-1F3FF"},
    {"pessoa_gravida", "1FAC4"},
    {"pessoa_gravida_cor1", "1FAC4-1F3FB"},
    {"pessoa_gravida_cor2", "1FAC4-1F3FC"},
    {"pessoa_gravida_cor3", "1FAC4-1F3FD"},
    {"pessoa_gravida_cor4", "1FAC4-1F3FE"},
    {"pessoa_gravida_cor5", "1FAC4-1F3FF"},
    {"pessoa_com_coroa", "1FAC5"},
    {"pessoa_com_coroa_cor1", "1FAC5-1F3FB"},
    {"pessoa_com_coroa_cor2", "1FAC5-1F3FC"},
    {"pessoa_com_coroa_cor3", "1FAC5-1F3FD"},
    {"pessoa_com_coroa_cor4", "1FAC5-1F3FE"},
    {"pessoa_com_coroa_cor5", "1FAC5-1F3FF"},
    {"impressao_digital", "1FAC6"},
    {"criatura_peluda", "1FAC8"},
    {"orca", "1FACD"},
    {"alce", "1FACE"},
    {"burro", "1FACF"},
    {"mirtilos", "1FAD0"},
    {"pimentao", "1FAD1"},
    {"azeitona", "1FAD2"},
    {"pao_sirio", "1FAD3"},
    {"tamale", "1FAD4"},
    {"fondue", "1FAD5"},
    {"bule", "1FAD6"},
    {"derramando_liquido", "1FAD7"},
    {"feijoes", "1FAD8"},
    {"jarro", "1FAD9"},
    {"gengibre", "1FADA"},
    {"vagem", "1FADB"},
    {"tuberculo", "1FADC"},
    {"respingo", "1FADF"},
    {"rosto_derretendo", "1FAE0"},
    {"rosto_saudando", "1FAE1"},
    {"rosto_com_olhos_abertos_e_mao_sobre_a_boca", "1FAE2"},
    {"rosto_com_olho_espiando", "1FAE3"},
    {"rosto_com_boca_diagonal", "1FAE4"},
    {"rosto_com_linha_pontilhada", "1FAE5"},
    {"mordendo_o_labio", "1FAE6"},
    {"bolhas", "1FAE7"},
    {"rosto_tremendo", "1FAE8"},
    {"rosto_com_olheiras", "1FAE9"},
    {"rosto_distorcido", "1FAEA"},
    {"nuvem_de_briga", "1FAEF"},
    {"mao_com_dedo_indicador_e_polegar_cruzados", "1FAF0"},
    {"mao_com_dedo_indicador_e_polegar_cruzados_cor1", "1FAF0-1F3FB"},
    {"mao_com_dedo_indicador_e_polegar_cruzados_cor2", "1FAF0-1F3FC"},
    {"mao_com_dedo_indicador_e_polegar_cruzados_cor3", "1FAF0-1F3FD"},
    {"mao_com_dedo_indicador_e_polegar_cruzados_cor4", "1FAF0-1F3FE"},
    {"mao_com_dedo_indicador_e_polegar_cruzados_cor5", "1FAF0-1F3FF"},
    {"mao_para_a_direita", "1FAF1"},
    {"mao_para_a_direita_cor1", "1FAF1-1F3FB"},
    {"mao_para_a_direita_cor2", "1FAF1-1F3FC"},
    {"mao_para_a_direita_cor3", "1FAF1-1F3FD"},
    {"mao_para_a_direita_cor4", "1FAF1-1F3FE"},
    {"mao_para_a_direita_cor5", "1FAF1-1F3FF"},
    {"mao_para_a_esquerda", "1FAF2"},
    {"mao_para_a_esquerda_cor1", "1FAF2-1F3FB"},
    {"mao_para_a_esquerda_cor2", "1FAF2-1F3FC"},
    {"mao_para_a_esquerda_cor3", "1FAF2-1F3FD"},
    {"mao_para_a_esquerda_cor4", "1FAF2-1F3FE"},
    {"mao_para_a_esquerda_cor5", "1FAF2-1F3FF"},
    {"mao_com_a_palma_para_baixo", "1FAF3"},
    {"mao_com_a_palma_para_baixo_cor1", "1FAF3-1F3FB"},
    {"mao_com_a_palma_para_baixo_cor2", "1FAF3-1F3FC"},
    {"mao_com_a_palma_para_baixo_cor3", "1FAF3-1F3FD"},
    {"mao_com_a_palma_para_baixo_cor4", "1FAF3-1F3FE"},
    {"mao_com_a_palma_para_baixo_cor5", "1FAF3-1F3FF"},
    {"mao_com_a_palma_para_cima", "1FAF4"},
    {"mao_com_a_palma_para_cima_cor1", "1FAF4-1F3FB"},
    {"mao_com_a_palma_para_cima_cor2", "1FAF4-1F3FC"},
    {"mao_com_a_palma_para_cima_cor3", "1FAF4-1F3FD"},
    {"mao_com_a_palma_para_cima_cor4", "1FAF4-1F3FE"},
    {"mao_com_a_palma_para_cima_cor5", "1FAF4-1F3FF"},
    {"indicador_apontando_para_o_visualizador", "1FAF5"},
    {"indicador_apontando_para_o_visualizador_cor1", "1FAF5-1F3FB"},
    {"indicador_apontando_para_o_visualizador_cor2", "1FAF5-1F3FC"},
    {"indicador_apontando_para_o_visualizador_cor3", "1FAF5-1F3FD"},
    {"indicador_apontando_para_o_visualizador_cor4", "1FAF5-1F3FE"},
    {"indicador_apontando_para_o_visualizador_cor5", "1FAF5-1F3FF"},
    {"maos_de_coracao", "1FAF6"},
    {"maos_de_coracao_cor1", "1FAF6-1F3FB"},
    {"maos_de_coracao_cor2", "1FAF6-1F3FC"},
    {"maos_de_coracao_cor3", "1FAF6-1F3FD"},
    {"maos_de_coracao_cor4", "1FAF6-1F3FE"},
    {"maos_de_coracao_cor5", "1FAF6-1F3FF"},
    {"mao_empurrando_para_a_esquerda", "1FAF7"},
    {"mao_empurrando_para_a_esquerda_cor1", "1FAF7-1F3FB"},
    {"mao_empurrando_para_a_esquerda_cor2", "1FAF7-1F3FC"},
    {"mao_empurrando_para_a_esquerda_cor3", "1FAF7-1F3FD"},
    {"mao_empurrando_para_a_esquerda_cor4", "1FAF7-1F3FE"},
    {"mao_empurrando_para_a_esquerda_cor5", "1FAF7-1F3FF"},
    {"mao_empurrando_para_a_direita", "1FAF8"},
    {"mao_empurrando_para_a_direita_cor1", "1FAF8-1F3FB"},
    {"mao_empurrando_para_a_direita_cor2", "1FAF8-1F3FC"},
    {"mao_empurrando_para_a_direita_cor3", "1FAF8-1F3FD"},
    {"mao_empurrando_para_a_direita_cor4", "1FAF8-1F3FE"},
    {"mao_empurrando_para_a_direita_cor5", "1FAF8-1F3FF"},
    {"tecla_sinal_de_numero", "0023-FE0F-20E3"},
    {"tecla_asterisco", "002A-FE0F-20E3"},
    {"tecla_0", "0030-FE0F-20E3"},
    {"tecla_1", "0031-FE0F-20E3"},
    {"tecla_2", "0032-FE0F-20E3"},
    {"tecla_3", "0033-FE0F-20E3"},
    {"tecla_4", "0034-FE0F-20E3"},
    {"tecla_5", "0035-FE0F-20E3"},
    {"tecla_6", "0036-FE0F-20E3"},
    {"tecla_7", "0037-FE0F-20E3"},
    {"tecla_8", "0038-FE0F-20E3"},
    {"tecla_9", "0039-FE0F-20E3"},
    {"bandeira_ilha_de_ascensao", "1F1E6-1F1E8"},
    {"bandeira_andorra", "1F1E6-1F1E9"},
    {"bandeira_emirados_arabes_unidos", "1F1E6-1F1EA"},
    {"bandeira_afeganistao", "1F1E6-1F1EB"},
    {"bandeira_antigua_e_barbuda", "1F1E6-1F1EC"},
    {"bandeira_anguila", "1F1E6-1F1EE"},
    {"bandeira_albania", "1F1E6-1F1F1"},
    {"bandeira_armenia", "1F1E6-1F1F2"},
    {"bandeira_angola", "1F1E6-1F1F4"},
    {"bandeira_antartida", "1F1E6-1F1F6"},
    {"bandeira_argentina", "1F1E6-1F1F7"},
    {"bandeira_samoa_americana", "1F1E6-1F1F8"},
    {"bandeira_austria", "1F1E6-1F1F9"},
    {"bandeira_australia", "1F1E6-1F1FA"},
    {"bandeira_aruba", "1F1E6-1F1FC"},
    {"bandeira_ilhas_aland", "1F1E6-1F1FD"},
    {"bandeira_azerbaijao", "1F1E6-1F1FF"},
    {"bandeira_bosnia_e_herzegovina", "1F1E7-1F1E6"},
    {"bandeira_barbados", "1F1E7-1F1E7"},
    {"bandeira_bangladesh", "1F1E7-1F1E9"},
    {"bandeira_belgica", "1F1E7-1F1EA"},
    {"bandeira_burquina_faso", "1F1E7-1F1EB"},
    {"bandeira_bulgaria", "1F1E7-1F1EC"},
    {"bandeira_barein", "1F1E7-1F1ED"},
    {"bandeira_burundi", "1F1E7-1F1EE"},
    {"bandeira_benin", "1F1E7-1F1EF"},
    {"bandeira_sao_bartolomeu", "1F1E7-1F1F1"},
    {"bandeira_bermudas", "1F1E7-1F1F2"},
    {"bandeira_brunei", "1F1E7-1F1F3"},
    {"bandeira_bolivia", "1F1E7-1F1F4"},
    {"bandeira_paises_baixos_caribenhos", "1F1E7-1F1F6"},
    {"bandeira_brasil", "1F1E7-1F1F7"},
    {"bandeira_bahamas", "1F1E7-1F1F8"},
    {"bandeira_butao", "1F1E7-1F1F9"},
    {"bandeira_ilha_bouvet", "1F1E7-1F1FB"},
    {"bandeira_botsuana", "1F1E7-1F1FC"},
    {"bandeira_bielorrussia", "1F1E7-1F1FE"},
    {"bandeira_belize", "1F1E7-1F1FF"},
    {"bandeira_canada", "1F1E8-1F1E6"},
    {"bandeira_ilhas_cocos_keeling", "1F1E8-1F1E8"},
    {"bandeira_congo_kinshasa", "1F1E8-1F1E9"},
    {"bandeira_republica_centro_africana", "1F1E8-1F1EB"},
    {"bandeira_republica_do_congo", "1F1E8-1F1EC"},
    {"bandeira_suica", "1F1E8-1F1ED"},
    {"bandeira_costa_do_marfim", "1F1E8-1F1EE"},
    {"bandeira_ilhas_cook", "1F1E8-1F1F0"},
    {"bandeira_chile", "1F1E8-1F1F1"},
    {"bandeira_camaroes", "1F1E8-1F1F2"},
    {"bandeira_china", "1F1E8-1F1F3"},
    {"bandeira_colombia", "1F1E8-1F1F4"},
    {"bandeira_ilha_de_clipperton", "1F1E8-1F1F5"},
    {"bandeira_sark", "1F1E8-1F1F6"},
    {"bandeira_costa_rica", "1F1E8-1F1F7"},
    {"bandeira_cuba", "1F1E8-1F1FA"},
    {"bandeira_cabo_verde", "1F1E8-1F1FB"},
    {"bandeira_curacao", "1F1E8-1F1FC"},
    {"bandeira_ilha_christmas", "1F1E8-1F1FD"},
    {"bandeira_chipre", "1F1E8-1F1FE"},
    {"bandeira_tchequia", "1F1E8-1F1FF"},
    {"bandeira_alemanha", "1F1E9-1F1EA"},
    {"bandeira_diego_garcia", "1F1E9-1F1EC"},
    {"bandeira_djibuti", "1F1E9-1F1EF"},
    {"bandeira_dinamarca", "1F1E9-1F1F0"},
    {"bandeira_dominica", "1F1E9-1F1F2"},
    {"bandeira_republica_dominicana", "1F1E9-1F1F4"},
    {"bandeira_argelia", "1F1E9-1F1FF"},
    {"bandeira_ceuta_e_melilla", "1F1EA-1F1E6"},
    {"bandeira_equador", "1F1EA-1F1E8"},
    {"bandeira_estonia", "1F1EA-1F1EA"},
    {"bandeira_egito", "1F1EA-1F1EC"},
    {"bandeira_saara_ocidental", "1F1EA-1F1ED"},
    {"bandeira_eritreia", "1F1EA-1F1F7"},
    {"bandeira_espanha", "1F1EA-1F1F8"},
    {"bandeira_etiopia", "1F1EA-1F1F9"},
    {"bandeira_uniao_europeia", "1F1EA-1F1FA"},
    {"bandeira_finlandia", "1F1EB-1F1EE"},
    {"bandeira_fiji", "1F1EB-1F1EF"},
    {"bandeira_ilhas_malvinas", "1F1EB-1F1F0"},
    {"bandeira_micronesia", "1F1EB-1F1F2"},
    {"bandeira_ilhas_faroe", "1F1EB-1F1F4"},
    {"bandeira_franca", "1F1EB-1F1F7"},
    {"bandeira_gabao", "1F1EC-1F1E6"},
    {"bandeira_reino_unido", "1F1EC-1F1E7"},
    {"bandeira_granada", "1F1EC-1F1E9"},
    {"bandeira_georgia", "1F1EC-1F1EA"},
    {"bandeira_guiana_francesa", "1F1EC-1F1EB"},
    {"bandeira_guernsey", "1F1EC-1F1EC"},
    {"bandeira_gana", "1F1EC-1F1ED"},
    {"bandeira_gibraltar", "1F1EC-1F1EE"},
    {"bandeira_groenlandia", "1F1EC-1F1F1"},
    {"bandeira_gambia", "1F1EC-1F1F2"},
    {"bandeira_guine", "1F1EC-1F1F3"},
    {"bandeira_guadalupe", "1F1EC-1F1F5"},
    {"bandeira_guine_equatorial", "1F1EC-1F1F6"},
    {"bandeira_grecia", "1F1EC-1F1F7"},
    {"bandeira_ilhas_georgia_do_sul_e_sandwich_do_sul", "1F1EC-1F1F8"},
    {"bandeira_guatemala", "1F1EC-1F1F9"},
    {"bandeira_guam", "1F1EC-1F1FA"},
    {"bandeira_guine_bissau", "1F1EC-1F1FC"},
    {"bandeira_guiana", "1F1EC-1F1FE"},
    {"bandeira_hong_kong_rae_da_china", "1F1ED-1F1F0"},
    {"bandeira_ilhas_heard_e_mcdonald", "1F1ED-1F1F2"},
    {"bandeira_honduras", "1F1ED-1F1F3"},
    {"bandeira_croacia", "1F1ED-1F1F7"},
    {"bandeira_haiti", "1F1ED-1F1F9"},
    {"bandeira_hungria", "1F1ED-1F1FA"},
    {"bandeira_ilhas_canarias", "1F1EE-1F1E8"},
    {"bandeira_indonesia", "1F1EE-1F1E9"},
    {"bandeira_irlanda", "1F1EE-1F1EA"},
    {"bandeira_israel", "1F1EE-1F1F1"},
    {"bandeira_ilha_de_man", "1F1EE-1F1F2"},
    {"bandeira_india", "1F1EE-1F1F3"},
    {"bandeira_territorio_britanico_do_oceano_indico", "1F1EE-1F1F4"},
    {"bandeira_iraque", "1F1EE-1F1F6"},
    {"bandeira_ira", "1F1EE-1F1F7"},
    {"bandeira_islandia", "1F1EE-1F1F8"},
    {"bandeira_italia", "1F1EE-1F1F9"},
    {"bandeira_jersey", "1F1EF-1F1EA"},
    {"bandeira_jamaica", "1F1EF-1F1F2"},
    {"bandeira_jordania", "1F1EF-1F1F4"},
    {"bandeira_japao", "1F1EF-1F1F5"},
    {"bandeira_quenia", "1F1F0-1F1EA"},
    {"bandeira_quirguistao", "1F1F0-1F1EC"},
    {"bandeira_camboja", "1F1F0-1F1ED"},
    {"bandeira_quiribati", "1F1F0-1F1EE"},
    {"bandeira_comores", "1F1F0-1F1F2"},
    {"bandeira_sao_cristovao_e_nevis", "1F1F0-1F1F3"},
    {"bandeira_coreia_do_norte", "1F1F0-1F1F5"},
    {"bandeira_coreia_do_sul", "1F1F0-1F1F7"},
    {"bandeira_kuwait", "1F1F0-1F1FC"},
    {"bandeira_ilhas_cayman", "1F1F0-1F1FE"},
    {"bandeira_cazaquistao", "1F1F0-1F1FF"},
    {"bandeira_laos", "1F1F1-1F1E6"},
    {"bandeira_libano", "1F1F1-1F1E7"},
    {"bandeira_santa_lucia", "1F1F1-1F1E8"},
    {"bandeira_liechtenstein", "1F1F1-1F1EE"},
    {"bandeira_sri_lanka", "1F1F1-1F1F0"},
    {"bandeira_liberia", "1F1F1-1F1F7"},
    {"bandeira_lesoto", "1F1F1-1F1F8"},
    {"bandeira_lituania", "1F1F1-1F1F9"},
    {"bandeira_luxemburgo", "1F1F1-1F1FA"},
    {"bandeira_letonia", "1F1F1-1F1FB"},
    {"bandeira_libia", "1F1F1-1F1FE"},
    {"bandeira_marrocos", "1F1F2-1F1E6"},
    {"bandeira_monaco", "1F1F2-1F1E8"},
    {"bandeira_moldavia", "1F1F2-1F1E9"},
    {"bandeira_montenegro", "1F1F2-1F1EA"},
    {"bandeira_sao_martinho", "1F1F2-1F1EB"},
    {"bandeira_madagascar", "1F1F2-1F1EC"},
    {"bandeira_ilhas_marshall", "1F1F2-1F1ED"},
    {"bandeira_macedonia_do_norte", "1F1F2-1F1F0"},
    {"bandeira_mali", "1F1F2-1F1F1"},
    {"bandeira_mianmar_birmania", "1F1F2-1F1F2"},
    {"bandeira_mongolia", "1F1F2-1F1F3"},
    {"bandeira_macau_rae_da_china", "1F1F2-1F1F4"},
    {"bandeira_ilhas_marianas_do_norte", "1F1F2-1F1F5"},
    {"bandeira_martinica", "1F1F2-1F1F6"},
    {"bandeira_mauritania", "1F1F2-1F1F7"},
    {"bandeira_montserrat", "1F1F2-1F1F8"},
    {"bandeira_malta", "1F1F2-1F1F9"},
    {"bandeira_mauricio", "1F1F2-1F1FA"},
    {"bandeira_maldivas", "1F1F2-1F1FB"},
    {"bandeira_malaui", "1F1F2-1F1FC"},
    {"bandeira_mexico", "1F1F2-1F1FD"},
    {"bandeira_malasia", "1F1F2-1F1FE"},
    {"bandeira_mocambique", "1F1F2-1F1FF"},
    {"bandeira_namibia", "1F1F3-1F1E6"},
    {"bandeira_nova_caledonia", "1F1F3-1F1E8"},
    {"bandeira_niger", "1F1F3-1F1EA"},
    {"bandeira_ilha_norfolk", "1F1F3-1F1EB"},
    {"bandeira_nigeria", "1F1F3-1F1EC"},
    {"bandeira_nicaragua", "1F1F3-1F1EE"},
    {"bandeira_paises_baixos", "1F1F3-1F1F1"},
    {"bandeira_noruega", "1F1F3-1F1F4"},
    {"bandeira_nepal", "1F1F3-1F1F5"},
    {"bandeira_nauru", "1F1F3-1F1F7"},
    {"bandeira_niue", "1F1F3-1F1FA"},
    {"bandeira_nova_zelandia", "1F1F3-1F1FF"},
    {"bandeira_oma", "1F1F4-1F1F2"},
    {"bandeira_panama", "1F1F5-1F1E6"},
    {"bandeira_peru", "1F1F5-1F1EA"},
    {"bandeira_polinesia_francesa", "1F1F5-1F1EB"},
    {"bandeira_papua_nova_guine", "1F1F5-1F1EC"},
    {"bandeira_filipinas", "1F1F5-1F1ED"},
    {"bandeira_paquistao", "1F1F5-1F1F0"},
    {"bandeira_polonia", "1F1F5-1F1F1"},
    {"bandeira_sao_pedro_e_miquelao", "1F1F5-1F1F2"},
    {"bandeira_ilhas_pitcairn", "1F1F5-1F1F3"},
    {"bandeira_porto_rico", "1F1F5-1F1F7"},
    {"bandeira_territorios_palestinos", "1F1F5-1F1F8"},
    {"bandeira_portugal", "1F1F5-1F1F9"},
    {"bandeira_palau", "1F1F5-1F1FC"},
    {"bandeira_paraguai", "1F1F5-1F1FE"},
    {"bandeira_catar", "1F1F6-1F1E6"},
    {"bandeira_reuniao", "1F1F7-1F1EA"},
    {"bandeira_romenia", "1F1F7-1F1F4"},
    {"bandeira_servia", "1F1F7-1F1F8"},
    {"bandeira_russia", "1F1F7-1F1FA"},
    {"bandeira_ruanda", "1F1F7-1F1FC"},
    {"bandeira_arabia_saudita", "1F1F8-1F1E6"},
    {"bandeira_ilhas_salomao", "1F1F8-1F1E7"},
    {"bandeira_seicheles", "1F1F8-1F1E8"},
    {"bandeira_sudao", "1F1F8-1F1E9"},
    {"bandeira_suecia", "1F1F8-1F1EA"},
    {"bandeira_singapura", "1F1F8-1F1EC"},
    {"bandeira_santa_helena", "1F1F8-1F1ED"},
    {"bandeira_eslovenia", "1F1F8-1F1EE"},
    {"bandeira_svalbard_e_jan_mayen", "1F1F8-1F1EF"},
    {"bandeira_eslovaquia", "1F1F8-1F1F0"},
    {"bandeira_serra_leoa", "1F1F8-1F1F1"},
    {"bandeira_san_marino", "1F1F8-1F1F2"},
    {"bandeira_senegal", "1F1F8-1F1F3"},
    {"bandeira_somalia", "1F1F8-1F1F4"},
    {"bandeira_suriname", "1F1F8-1F1F7"},
    {"bandeira_sudao_do_sul", "1F1F8-1F1F8"},
    {"bandeira_sao_tome_e_principe", "1F1F8-1F1F9"},
    {"bandeira_el_salvador", "1F1F8-1F1FB"},
    {"bandeira_sint_maarten", "1F1F8-1F1FD"},
    {"bandeira_siria", "1F1F8-1F1FE"},
    {"bandeira_essuatini", "1F1F8-1F1FF"},
    {"bandeira_tristao_da_cunha", "1F1F9-1F1E6"},
    {"bandeira_ilhas_turcas_e_caicos", "1F1F9-1F1E8"},
    {"bandeira_chade", "1F1F9-1F1E9"},
    {"bandeira_territorios_franceses_do_sul", "1F1F9-1F1EB"},
    {"bandeira_togo", "1F1F9-1F1EC"},
    {"bandeira_tailandia", "1F1F9-1F1ED"},
    {"bandeira_tadjiquistao", "1F1F9-1F1EF"},
    {"bandeira_tokelau", "1F1F9-1F1F0"},
    {"bandeira_timor_leste", "1F1F9-1F1F1"},
    {"bandeira_turcomenistao", "1F1F9-1F1F2"},
    {"bandeira_tunisia", "1F1F9-1F1F3"},
    {"bandeira_tonga", "1F1F9-1F1F4"},
    {"bandeira_turquia", "1F1F9-1F1F7"},
    {"bandeira_trinidad_e_tobago", "1F1F9-1F1F9"},
    {"bandeira_tuvalu", "1F1F9-1F1FB"},
    {"bandeira_taiwan", "1F1F9-1F1FC"},
    {"bandeira_tanzania", "1F1F9-1F1FF"},
    {"bandeira_ucrania", "1F1FA-1F1E6"},
    {"bandeira_uganda", "1F1FA-1F1EC"},
    {"bandeira_ilhas_menores_distantes_dos_eua", "1F1FA-1F1F2"},
    {"bandeira_nacoes_unidas", "1F1FA-1F1F3"},
    {"bandeira_estados_unidos", "1F1FA-1F1F8"},
    {"bandeira_uruguai", "1F1FA-1F1FE"},
    {"bandeira_uzbequistao", "1F1FA-1F1FF"},
    {"bandeira_cidade_do_vaticano", "1F1FB-1F1E6"},
    {"bandeira_sao_vicente_e_granadinas", "1F1FB-1F1E8"},
    {"bandeira_venezuela", "1F1FB-1F1EA"},
    {"bandeira_ilhas_virgens_britanicas", "1F1FB-1F1EC"},
    {"bandeira_ilhas_virgens_americanas", "1F1FB-1F1EE"},
    {"bandeira_vietna", "1F1FB-1F1F3"},
    {"bandeira_vanuatu", "1F1FB-1F1FA"},
    {"bandeira_wallis_e_futuna", "1F1FC-1F1EB"},
    {"bandeira_samoa", "1F1FC-1F1F8"},
    {"bandeira_kosovo", "1F1FD-1F1F0"},
    {"bandeira_iemen", "1F1FE-1F1EA"},
    {"bandeira_mayotte", "1F1FE-1F1F9"},
    {"bandeira_africa_do_sul", "1F1FF-1F1E6"},
    {"bandeira_zambia", "1F1FF-1F1F2"},
    {"bandeira_zimbabue", "1F1FF-1F1FC"},
    {"bandeira_inglaterra", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"bandeira_escocia", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"bandeira_pais_de_gales", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"casal_apaixonado_homem_e_homem", "1F468-200D-2764-FE0F-200D-1F468"},
    {"casal_apaixonado_homem_e_homem_cor1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"casal_apaixonado_homem_e_homem_cor1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"casal_apaixonado_homem_e_homem_cor1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"casal_apaixonado_homem_e_homem_cor1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"casal_apaixonado_homem_e_homem_cor1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"casal_apaixonado_homem_e_homem_cor2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"casal_apaixonado_homem_e_homem_cor2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"casal_apaixonado_homem_e_homem_cor2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"casal_apaixonado_homem_e_homem_cor2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"casal_apaixonado_homem_e_homem_cor2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"casal_apaixonado_homem_e_homem_cor3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"casal_apaixonado_homem_e_homem_cor3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"casal_apaixonado_homem_e_homem_cor3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"casal_apaixonado_homem_e_homem_cor3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"casal_apaixonado_homem_e_homem_cor3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"casal_apaixonado_homem_e_homem_cor4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"casal_apaixonado_homem_e_homem_cor4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"casal_apaixonado_homem_e_homem_cor4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"casal_apaixonado_homem_e_homem_cor4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"casal_apaixonado_homem_e_homem_cor4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"casal_apaixonado_homem_e_homem_cor5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"casal_apaixonado_homem_e_homem_cor5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"casal_apaixonado_homem_e_homem_cor5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"casal_apaixonado_homem_e_homem_cor5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"casal_apaixonado_homem_e_homem_cor5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"beijo_homem_e_homem", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"beijo_homem_e_homem_cor1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beijo_homem_e_homem_cor1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beijo_homem_e_homem_cor1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beijo_homem_e_homem_cor1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beijo_homem_e_homem_cor1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beijo_homem_e_homem_cor2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beijo_homem_e_homem_cor2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beijo_homem_e_homem_cor2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beijo_homem_e_homem_cor2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beijo_homem_e_homem_cor2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beijo_homem_e_homem_cor3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beijo_homem_e_homem_cor3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beijo_homem_e_homem_cor3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beijo_homem_e_homem_cor3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beijo_homem_e_homem_cor3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beijo_homem_e_homem_cor4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beijo_homem_e_homem_cor4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beijo_homem_e_homem_cor4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beijo_homem_e_homem_cor4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beijo_homem_e_homem_cor4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beijo_homem_e_homem_cor5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beijo_homem_e_homem_cor5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beijo_homem_e_homem_cor5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beijo_homem_e_homem_cor5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beijo_homem_e_homem_cor5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"familia_homem_e_menino", "1F468-200D-1F466"},
    {"familia_homem_menino_e_menino", "1F468-200D-1F466-200D-1F466"},
    {"familia_homem_e_menina", "1F468-200D-1F467"},
    {"familia_homem_menina_e_menino", "1F468-200D-1F467-200D-1F466"},
    {"familia_homem_menina_e_menina", "1F468-200D-1F467-200D-1F467"},
    {"familia_homem_homem_e_menino", "1F468-200D-1F468-200D-1F466"},
    {"familia_homem_homem_menino_e_menino", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"familia_homem_homem_e_menina", "1F468-200D-1F468-200D-1F467"},
    {"familia_homem_homem_menina_e_menino", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"familia_homem_homem_menina_e_menina", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"familia_homem_mulher_e_menino", "1F468-200D-1F469-200D-1F466"},
    {"familia_homem_mulher_menino_e_menino", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"familia_homem_mulher_e_menina", "1F468-200D-1F469-200D-1F467"},
    {"familia_homem_mulher_menina_e_menino", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"familia_homem_mulher_menina_e_menina", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"casal_apaixonado_mulher_e_homem", "1F469-200D-2764-FE0F-200D-1F468"},
    {"casal_apaixonado_mulher_e_homem_cor1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"casal_apaixonado_mulher_e_homem_cor1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"casal_apaixonado_mulher_e_homem_cor1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"casal_apaixonado_mulher_e_homem_cor1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"casal_apaixonado_mulher_e_homem_cor1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"casal_apaixonado_mulher_e_homem_cor2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"casal_apaixonado_mulher_e_homem_cor2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"casal_apaixonado_mulher_e_homem_cor2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"casal_apaixonado_mulher_e_homem_cor2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"casal_apaixonado_mulher_e_homem_cor2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"casal_apaixonado_mulher_e_homem_cor3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"casal_apaixonado_mulher_e_homem_cor3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"casal_apaixonado_mulher_e_homem_cor3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"casal_apaixonado_mulher_e_homem_cor3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"casal_apaixonado_mulher_e_homem_cor3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"casal_apaixonado_mulher_e_homem_cor4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"casal_apaixonado_mulher_e_homem_cor4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"casal_apaixonado_mulher_e_homem_cor4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"casal_apaixonado_mulher_e_homem_cor4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"casal_apaixonado_mulher_e_homem_cor4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"casal_apaixonado_mulher_e_homem_cor5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"casal_apaixonado_mulher_e_homem_cor5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"casal_apaixonado_mulher_e_homem_cor5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"casal_apaixonado_mulher_e_homem_cor5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"casal_apaixonado_mulher_e_homem_cor5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"casal_apaixonado_mulher_e_mulher", "1F469-200D-2764-FE0F-200D-1F469"},
    {"casal_apaixonado_mulher_e_mulher_cor1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"casal_apaixonado_mulher_e_mulher_cor1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"casal_apaixonado_mulher_e_mulher_cor1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"casal_apaixonado_mulher_e_mulher_cor1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"casal_apaixonado_mulher_e_mulher_cor1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"casal_apaixonado_mulher_e_mulher_cor2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"casal_apaixonado_mulher_e_mulher_cor2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"casal_apaixonado_mulher_e_mulher_cor2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"casal_apaixonado_mulher_e_mulher_cor2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"casal_apaixonado_mulher_e_mulher_cor2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"casal_apaixonado_mulher_e_mulher_cor3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"casal_apaixonado_mulher_e_mulher_cor3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"casal_apaixonado_mulher_e_mulher_cor3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"casal_apaixonado_mulher_e_mulher_cor3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"casal_apaixonado_mulher_e_mulher_cor3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"casal_apaixonado_mulher_e_mulher_cor4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"casal_apaixonado_mulher_e_mulher_cor4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"casal_apaixonado_mulher_e_mulher_cor4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"casal_apaixonado_mulher_e_mulher_cor4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"casal_apaixonado_mulher_e_mulher_cor4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"casal_apaixonado_mulher_e_mulher_cor5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"casal_apaixonado_mulher_e_mulher_cor5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"casal_apaixonado_mulher_e_mulher_cor5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"casal_apaixonado_mulher_e_mulher_cor5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"casal_apaixonado_mulher_e_mulher_cor5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"beijo_mulher_e_homem", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"beijo_mulher_e_homem_cor1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beijo_mulher_e_homem_cor1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beijo_mulher_e_homem_cor1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beijo_mulher_e_homem_cor1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beijo_mulher_e_homem_cor1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beijo_mulher_e_homem_cor2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beijo_mulher_e_homem_cor2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beijo_mulher_e_homem_cor2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beijo_mulher_e_homem_cor2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beijo_mulher_e_homem_cor2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beijo_mulher_e_homem_cor3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beijo_mulher_e_homem_cor3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beijo_mulher_e_homem_cor3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beijo_mulher_e_homem_cor3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beijo_mulher_e_homem_cor3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beijo_mulher_e_homem_cor4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beijo_mulher_e_homem_cor4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beijo_mulher_e_homem_cor4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beijo_mulher_e_homem_cor4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beijo_mulher_e_homem_cor4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beijo_mulher_e_homem_cor5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beijo_mulher_e_homem_cor5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beijo_mulher_e_homem_cor5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beijo_mulher_e_homem_cor5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beijo_mulher_e_homem_cor5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beijo_mulher_e_mulher", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"beijo_mulher_e_mulher_cor1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"beijo_mulher_e_mulher_cor1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"beijo_mulher_e_mulher_cor1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"beijo_mulher_e_mulher_cor1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"beijo_mulher_e_mulher_cor1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"beijo_mulher_e_mulher_cor2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"beijo_mulher_e_mulher_cor2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"beijo_mulher_e_mulher_cor2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"beijo_mulher_e_mulher_cor2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"beijo_mulher_e_mulher_cor2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"beijo_mulher_e_mulher_cor3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"beijo_mulher_e_mulher_cor3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"beijo_mulher_e_mulher_cor3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"beijo_mulher_e_mulher_cor3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"beijo_mulher_e_mulher_cor3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"beijo_mulher_e_mulher_cor4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"beijo_mulher_e_mulher_cor4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"beijo_mulher_e_mulher_cor4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"beijo_mulher_e_mulher_cor4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"beijo_mulher_e_mulher_cor4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"beijo_mulher_e_mulher_cor5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"beijo_mulher_e_mulher_cor5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"beijo_mulher_e_mulher_cor5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"beijo_mulher_e_mulher_cor5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"beijo_mulher_e_mulher_cor5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"familia_mulher_e_menino", "1F469-200D-1F466"},
    {"familia_mulher_menino_e_menino", "1F469-200D-1F466-200D-1F466"},
    {"familia_mulher_e_menina", "1F469-200D-1F467"},
    {"familia_mulher_menina_e_menino", "1F469-200D-1F467-200D-1F466"},
    {"familia_mulher_menina_e_menina", "1F469-200D-1F467-200D-1F467"},
    {"familia_mulher_mulher_e_menino", "1F469-200D-1F469-200D-1F466"},
    {"familia_mulher_mulher_menino_e_menino", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"familia_mulher_mulher_e_menina", "1F469-200D-1F469-200D-1F467"},
    {"familia_mulher_mulher_menina_e_menino", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"familia_mulher_mulher_menina_e_menina", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"pessoas_de_maos_dadas", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"pessoas_de_maos_dadas_cor1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"pessoas_de_maos_dadas_cor1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"pessoas_de_maos_dadas_cor1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"pessoas_de_maos_dadas_cor1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"pessoas_de_maos_dadas_cor1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"pessoas_de_maos_dadas_cor2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"pessoas_de_maos_dadas_cor2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"pessoas_de_maos_dadas_cor2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"pessoas_de_maos_dadas_cor2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"pessoas_de_maos_dadas_cor2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"pessoas_de_maos_dadas_cor3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"pessoas_de_maos_dadas_cor3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"pessoas_de_maos_dadas_cor3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"pessoas_de_maos_dadas_cor3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"pessoas_de_maos_dadas_cor3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"pessoas_de_maos_dadas_cor4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"pessoas_de_maos_dadas_cor4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"pessoas_de_maos_dadas_cor4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"pessoas_de_maos_dadas_cor4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"pessoas_de_maos_dadas_cor4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"pessoas_de_maos_dadas_cor5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"pessoas_de_maos_dadas_cor5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"pessoas_de_maos_dadas_cor5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"pessoas_de_maos_dadas_cor5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"pessoas_de_maos_dadas_cor5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"familia_adulto_adulto_crianca", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"familia_adulto_adulto_crianca_crianca", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"familia_adulto_crianca", "1F9D1-200D-1F9D2"},
    {"familia_adulto_crianca_crianca", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"pessoa_correndo_de_frente_para_a_direita", "1F3C3-200D-27A1-FE0F"},
    {"pessoa_correndo_de_frente_para_a_direita_cor1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"pessoa_correndo_de_frente_para_a_direita_cor2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"pessoa_correndo_de_frente_para_a_direita_cor3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"pessoa_correndo_de_frente_para_a_direita_cor4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"pessoa_correndo_de_frente_para_a_direita_cor5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"homem_profissional_da_saude", "1F468-200D-2695-FE0F"},
    {"homem_profissional_da_saude_cor1", "1F468-1F3FB-200D-2695-FE0F"},
    {"homem_profissional_da_saude_cor2", "1F468-1F3FC-200D-2695-FE0F"},
    {"homem_profissional_da_saude_cor3", "1F468-1F3FD-200D-2695-FE0F"},
    {"homem_profissional_da_saude_cor4", "1F468-1F3FE-200D-2695-FE0F"},
    {"homem_profissional_da_saude_cor5", "1F468-1F3FF-200D-2695-FE0F"},
    {"juiz", "1F468-200D-2696-FE0F"},
    {"juiz_cor1", "1F468-1F3FB-200D-2696-FE0F"},
    {"juiz_cor2", "1F468-1F3FC-200D-2696-FE0F"},
    {"juiz_cor3", "1F468-1F3FD-200D-2696-FE0F"},
    {"juiz_cor4", "1F468-1F3FE-200D-2696-FE0F"},
    {"juiz_cor5", "1F468-1F3FF-200D-2696-FE0F"},
    {"piloto_de_aviao_homem", "1F468-200D-2708-FE0F"},
    {"piloto_de_aviao_homem_cor1", "1F468-1F3FB-200D-2708-FE0F"},
    {"piloto_de_aviao_homem_cor2", "1F468-1F3FC-200D-2708-FE0F"},
    {"piloto_de_aviao_homem_cor3", "1F468-1F3FD-200D-2708-FE0F"},
    {"piloto_de_aviao_homem_cor4", "1F468-1F3FE-200D-2708-FE0F"},
    {"piloto_de_aviao_homem_cor5", "1F468-1F3FF-200D-2708-FE0F"},
    {"fazendeiro", "1F468-200D-1F33E"},
    {"fazendeiro_cor1", "1F468-1F3FB-200D-1F33E"},
    {"fazendeiro_cor2", "1F468-1F3FC-200D-1F33E"},
    {"fazendeiro_cor3", "1F468-1F3FD-200D-1F33E"},
    {"fazendeiro_cor4", "1F468-1F3FE-200D-1F33E"},
    {"fazendeiro_cor5", "1F468-1F3FF-200D-1F33E"},
    {"cozinheiro", "1F468-200D-1F373"},
    {"cozinheiro_cor1", "1F468-1F3FB-200D-1F373"},
    {"cozinheiro_cor2", "1F468-1F3FC-200D-1F373"},
    {"cozinheiro_cor3", "1F468-1F3FD-200D-1F373"},
    {"cozinheiro_cor4", "1F468-1F3FE-200D-1F373"},
    {"cozinheiro_cor5", "1F468-1F3FF-200D-1F373"},
    {"homem_alimentando_bebe", "1F468-200D-1F37C"},
    {"homem_alimentando_bebe_cor1", "1F468-1F3FB-200D-1F37C"},
    {"homem_alimentando_bebe_cor2", "1F468-1F3FC-200D-1F37C"},
    {"homem_alimentando_bebe_cor3", "1F468-1F3FD-200D-1F37C"},
    {"homem_alimentando_bebe_cor4", "1F468-1F3FE-200D-1F37C"},
    {"homem_alimentando_bebe_cor5", "1F468-1F3FF-200D-1F37C"},
    {"estudante", "1F468-200D-1F393"},
    {"estudante_cor1", "1F468-1F3FB-200D-1F393"},
    {"estudante_cor2", "1F468-1F3FC-200D-1F393"},
    {"estudante_cor3", "1F468-1F3FD-200D-1F393"},
    {"estudante_cor4", "1F468-1F3FE-200D-1F393"},
    {"estudante_cor5", "1F468-1F3FF-200D-1F393"},
    {"cantor_homem", "1F468-200D-1F3A4"},
    {"cantor_homem_cor1", "1F468-1F3FB-200D-1F3A4"},
    {"cantor_homem_cor2", "1F468-1F3FC-200D-1F3A4"},
    {"cantor_homem_cor3", "1F468-1F3FD-200D-1F3A4"},
    {"cantor_homem_cor4", "1F468-1F3FE-200D-1F3A4"},
    {"cantor_homem_cor5", "1F468-1F3FF-200D-1F3A4"},
    {"artista_plastico", "1F468-200D-1F3A8"},
    {"artista_plastico_cor1", "1F468-1F3FB-200D-1F3A8"},
    {"artista_plastico_cor2", "1F468-1F3FC-200D-1F3A8"},
    {"artista_plastico_cor3", "1F468-1F3FD-200D-1F3A8"},
    {"artista_plastico_cor4", "1F468-1F3FE-200D-1F3A8"},
    {"artista_plastico_cor5", "1F468-1F3FF-200D-1F3A8"},
    {"professor", "1F468-200D-1F3EB"},
    {"professor_cor1", "1F468-1F3FB-200D-1F3EB"},
    {"professor_cor2", "1F468-1F3FC-200D-1F3EB"},
    {"professor_cor3", "1F468-1F3FD-200D-1F3EB"},
    {"professor_cor4", "1F468-1F3FE-200D-1F3EB"},
    {"professor_cor5", "1F468-1F3FF-200D-1F3EB"},
    {"operario", "1F468-200D-1F3ED"},
    {"operario_cor1", "1F468-1F3FB-200D-1F3ED"},
    {"operario_cor2", "1F468-1F3FC-200D-1F3ED"},
    {"operario_cor3", "1F468-1F3FD-200D-1F3ED"},
    {"operario_cor4", "1F468-1F3FE-200D-1F3ED"},
    {"operario_cor5", "1F468-1F3FF-200D-1F3ED"},
    {"tecnologo", "1F468-200D-1F4BB"},
    {"tecnologo_cor1", "1F468-1F3FB-200D-1F4BB"},
    {"tecnologo_cor2", "1F468-1F3FC-200D-1F4BB"},
    {"tecnologo_cor3", "1F468-1F3FD-200D-1F4BB"},
    {"tecnologo_cor4", "1F468-1F3FE-200D-1F4BB"},
    {"tecnologo_cor5", "1F468-1F3FF-200D-1F4BB"},
    {"funcionario_de_escritorio", "1F468-200D-1F4BC"},
    {"funcionario_de_escritorio_cor1", "1F468-1F3FB-200D-1F4BC"},
    {"funcionario_de_escritorio_cor2", "1F468-1F3FC-200D-1F4BC"},
    {"funcionario_de_escritorio_cor3", "1F468-1F3FD-200D-1F4BC"},
    {"funcionario_de_escritorio_cor4", "1F468-1F3FE-200D-1F4BC"},
    {"funcionario_de_escritorio_cor5", "1F468-1F3FF-200D-1F4BC"},
    {"mecanico", "1F468-200D-1F527"},
    {"mecanico_cor1", "1F468-1F3FB-200D-1F527"},
    {"mecanico_cor2", "1F468-1F3FC-200D-1F527"},
    {"mecanico_cor3", "1F468-1F3FD-200D-1F527"},
    {"mecanico_cor4", "1F468-1F3FE-200D-1F527"},
    {"mecanico_cor5", "1F468-1F3FF-200D-1F527"},
    {"cientista_homem", "1F468-200D-1F52C"},
    {"cientista_homem_cor1", "1F468-1F3FB-200D-1F52C"},
    {"cientista_homem_cor2", "1F468-1F3FC-200D-1F52C"},
    {"cientista_homem_cor3", "1F468-1F3FD-200D-1F52C"},
    {"cientista_homem_cor4", "1F468-1F3FE-200D-1F52C"},
    {"cientista_homem_cor5", "1F468-1F3FF-200D-1F52C"},
    {"astronauta_homem", "1F468-200D-1F680"},
    {"astronauta_homem_cor1", "1F468-1F3FB-200D-1F680"},
    {"astronauta_homem_cor2", "1F468-1F3FC-200D-1F680"},
    {"astronauta_homem_cor3", "1F468-1F3FD-200D-1F680"},
    {"astronauta_homem_cor4", "1F468-1F3FE-200D-1F680"},
    {"astronauta_homem_cor5", "1F468-1F3FF-200D-1F680"},
    {"bombeiro_homem", "1F468-200D-1F692"},
    {"bombeiro_homem_cor1", "1F468-1F3FB-200D-1F692"},
    {"bombeiro_homem_cor2", "1F468-1F3FC-200D-1F692"},
    {"bombeiro_homem_cor3", "1F468-1F3FD-200D-1F692"},
    {"bombeiro_homem_cor4", "1F468-1F3FE-200D-1F692"},
    {"bombeiro_homem_cor5", "1F468-1F3FF-200D-1F692"},
    {"homem_com_bengala_para_cego", "1F468-200D-1F9AF"},
    {"homem_com_bengala_para_cego_cor1", "1F468-1F3FB-200D-1F9AF"},
    {"homem_com_bengala_para_cego_cor2", "1F468-1F3FC-200D-1F9AF"},
    {"homem_com_bengala_para_cego_cor3", "1F468-1F3FD-200D-1F9AF"},
    {"homem_com_bengala_para_cego_cor4", "1F468-1F3FE-200D-1F9AF"},
    {"homem_com_bengala_para_cego_cor5", "1F468-1F3FF-200D-1F9AF"},
    {"homem_com_bengala_para_cego_de_frente_para_a_direita", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"homem_com_bengala_para_cego_de_frente_para_a_direita_cor1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"homem_com_bengala_para_cego_de_frente_para_a_direita_cor2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"homem_com_bengala_para_cego_de_frente_para_a_direita_cor3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"homem_com_bengala_para_cego_de_frente_para_a_direita_cor4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"homem_com_bengala_para_cego_de_frente_para_a_direita_cor5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"homem_em_cadeira_de_rodas_motorizada", "1F468-200D-1F9BC"},
    {"homem_em_cadeira_de_rodas_motorizada_cor1", "1F468-1F3FB-200D-1F9BC"},
    {"homem_em_cadeira_de_rodas_motorizada_cor2", "1F468-1F3FC-200D-1F9BC"},
    {"homem_em_cadeira_de_rodas_motorizada_cor3", "1F468-1F3FD-200D-1F9BC"},
    {"homem_em_cadeira_de_rodas_motorizada_cor4", "1F468-1F3FE-200D-1F9BC"},
    {"homem_em_cadeira_de_rodas_motorizada_cor5", "1F468-1F3FF-200D-1F9BC"},
    {"homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"homem_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"homem_em_cadeira_de_rodas_manual", "1F468-200D-1F9BD"},
    {"homem_em_cadeira_de_rodas_manual_cor1", "1F468-1F3FB-200D-1F9BD"},
    {"homem_em_cadeira_de_rodas_manual_cor2", "1F468-1F3FC-200D-1F9BD"},
    {"homem_em_cadeira_de_rodas_manual_cor3", "1F468-1F3FD-200D-1F9BD"},
    {"homem_em_cadeira_de_rodas_manual_cor4", "1F468-1F3FE-200D-1F9BD"},
    {"homem_em_cadeira_de_rodas_manual_cor5", "1F468-1F3FF-200D-1F9BD"},
    {"homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"homem_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"mulher_profissional_da_saude", "1F469-200D-2695-FE0F"},
    {"mulher_profissional_da_saude_cor1", "1F469-1F3FB-200D-2695-FE0F"},
    {"mulher_profissional_da_saude_cor2", "1F469-1F3FC-200D-2695-FE0F"},
    {"mulher_profissional_da_saude_cor3", "1F469-1F3FD-200D-2695-FE0F"},
    {"mulher_profissional_da_saude_cor4", "1F469-1F3FE-200D-2695-FE0F"},
    {"mulher_profissional_da_saude_cor5", "1F469-1F3FF-200D-2695-FE0F"},
    {"juiza", "1F469-200D-2696-FE0F"},
    {"juiza_cor1", "1F469-1F3FB-200D-2696-FE0F"},
    {"juiza_cor2", "1F469-1F3FC-200D-2696-FE0F"},
    {"juiza_cor3", "1F469-1F3FD-200D-2696-FE0F"},
    {"juiza_cor4", "1F469-1F3FE-200D-2696-FE0F"},
    {"juiza_cor5", "1F469-1F3FF-200D-2696-FE0F"},
    {"piloto_de_aviao_mulher", "1F469-200D-2708-FE0F"},
    {"piloto_de_aviao_mulher_cor1", "1F469-1F3FB-200D-2708-FE0F"},
    {"piloto_de_aviao_mulher_cor2", "1F469-1F3FC-200D-2708-FE0F"},
    {"piloto_de_aviao_mulher_cor3", "1F469-1F3FD-200D-2708-FE0F"},
    {"piloto_de_aviao_mulher_cor4", "1F469-1F3FE-200D-2708-FE0F"},
    {"piloto_de_aviao_mulher_cor5", "1F469-1F3FF-200D-2708-FE0F"},
    {"fazendeira", "1F469-200D-1F33E"},
    {"fazendeira_cor1", "1F469-1F3FB-200D-1F33E"},
    {"fazendeira_cor2", "1F469-1F3FC-200D-1F33E"},
    {"fazendeira_cor3", "1F469-1F3FD-200D-1F33E"},
    {"fazendeira_cor4", "1F469-1F3FE-200D-1F33E"},
    {"fazendeira_cor5", "1F469-1F3FF-200D-1F33E"},
    {"cozinheira", "1F469-200D-1F373"},
    {"cozinheira_cor1", "1F469-1F3FB-200D-1F373"},
    {"cozinheira_cor2", "1F469-1F3FC-200D-1F373"},
    {"cozinheira_cor3", "1F469-1F3FD-200D-1F373"},
    {"cozinheira_cor4", "1F469-1F3FE-200D-1F373"},
    {"cozinheira_cor5", "1F469-1F3FF-200D-1F373"},
    {"mulher_alimentando_bebe", "1F469-200D-1F37C"},
    {"mulher_alimentando_bebe_cor1", "1F469-1F3FB-200D-1F37C"},
    {"mulher_alimentando_bebe_cor2", "1F469-1F3FC-200D-1F37C"},
    {"mulher_alimentando_bebe_cor3", "1F469-1F3FD-200D-1F37C"},
    {"mulher_alimentando_bebe_cor4", "1F469-1F3FE-200D-1F37C"},
    {"mulher_alimentando_bebe_cor5", "1F469-1F3FF-200D-1F37C"},
    {"aluna", "1F469-200D-1F393"},
    {"aluna_cor1", "1F469-1F3FB-200D-1F393"},
    {"aluna_cor2", "1F469-1F3FC-200D-1F393"},
    {"aluna_cor3", "1F469-1F3FD-200D-1F393"},
    {"aluna_cor4", "1F469-1F3FE-200D-1F393"},
    {"aluna_cor5", "1F469-1F3FF-200D-1F393"},
    {"cantora", "1F469-200D-1F3A4"},
    {"cantora_cor1", "1F469-1F3FB-200D-1F3A4"},
    {"cantora_cor2", "1F469-1F3FC-200D-1F3A4"},
    {"cantora_cor3", "1F469-1F3FD-200D-1F3A4"},
    {"cantora_cor4", "1F469-1F3FE-200D-1F3A4"},
    {"cantora_cor5", "1F469-1F3FF-200D-1F3A4"},
    {"artista_plastica", "1F469-200D-1F3A8"},
    {"artista_plastica_cor1", "1F469-1F3FB-200D-1F3A8"},
    {"artista_plastica_cor2", "1F469-1F3FC-200D-1F3A8"},
    {"artista_plastica_cor3", "1F469-1F3FD-200D-1F3A8"},
    {"artista_plastica_cor4", "1F469-1F3FE-200D-1F3A8"},
    {"artista_plastica_cor5", "1F469-1F3FF-200D-1F3A8"},
    {"professora", "1F469-200D-1F3EB"},
    {"professora_cor1", "1F469-1F3FB-200D-1F3EB"},
    {"professora_cor2", "1F469-1F3FC-200D-1F3EB"},
    {"professora_cor3", "1F469-1F3FD-200D-1F3EB"},
    {"professora_cor4", "1F469-1F3FE-200D-1F3EB"},
    {"professora_cor5", "1F469-1F3FF-200D-1F3EB"},
    {"operaria", "1F469-200D-1F3ED"},
    {"operaria_cor1", "1F469-1F3FB-200D-1F3ED"},
    {"operaria_cor2", "1F469-1F3FC-200D-1F3ED"},
    {"operaria_cor3", "1F469-1F3FD-200D-1F3ED"},
    {"operaria_cor4", "1F469-1F3FE-200D-1F3ED"},
    {"operaria_cor5", "1F469-1F3FF-200D-1F3ED"},
    {"tecnologa", "1F469-200D-1F4BB"},
    {"tecnologa_cor1", "1F469-1F3FB-200D-1F4BB"},
    {"tecnologa_cor2", "1F469-1F3FC-200D-1F4BB"},
    {"tecnologa_cor3", "1F469-1F3FD-200D-1F4BB"},
    {"tecnologa_cor4", "1F469-1F3FE-200D-1F4BB"},
    {"tecnologa_cor5", "1F469-1F3FF-200D-1F4BB"},
    {"funcionaria_de_escritorio", "1F469-200D-1F4BC"},
    {"funcionaria_de_escritorio_cor1", "1F469-1F3FB-200D-1F4BC"},
    {"funcionaria_de_escritorio_cor2", "1F469-1F3FC-200D-1F4BC"},
    {"funcionaria_de_escritorio_cor3", "1F469-1F3FD-200D-1F4BC"},
    {"funcionaria_de_escritorio_cor4", "1F469-1F3FE-200D-1F4BC"},
    {"funcionaria_de_escritorio_cor5", "1F469-1F3FF-200D-1F4BC"},
    {"mecanica", "1F469-200D-1F527"},
    {"mecanica_cor1", "1F469-1F3FB-200D-1F527"},
    {"mecanica_cor2", "1F469-1F3FC-200D-1F527"},
    {"mecanica_cor3", "1F469-1F3FD-200D-1F527"},
    {"mecanica_cor4", "1F469-1F3FE-200D-1F527"},
    {"mecanica_cor5", "1F469-1F3FF-200D-1F527"},
    {"cientista_mulher", "1F469-200D-1F52C"},
    {"cientista_mulher_cor1", "1F469-1F3FB-200D-1F52C"},
    {"cientista_mulher_cor2", "1F469-1F3FC-200D-1F52C"},
    {"cientista_mulher_cor3", "1F469-1F3FD-200D-1F52C"},
    {"cientista_mulher_cor4", "1F469-1F3FE-200D-1F52C"},
    {"cientista_mulher_cor5", "1F469-1F3FF-200D-1F52C"},
    {"astronauta_mulher", "1F469-200D-1F680"},
    {"astronauta_mulher_cor1", "1F469-1F3FB-200D-1F680"},
    {"astronauta_mulher_cor2", "1F469-1F3FC-200D-1F680"},
    {"astronauta_mulher_cor3", "1F469-1F3FD-200D-1F680"},
    {"astronauta_mulher_cor4", "1F469-1F3FE-200D-1F680"},
    {"astronauta_mulher_cor5", "1F469-1F3FF-200D-1F680"},
    {"bombeira", "1F469-200D-1F692"},
    {"bombeira_cor1", "1F469-1F3FB-200D-1F692"},
    {"bombeira_cor2", "1F469-1F3FC-200D-1F692"},
    {"bombeira_cor3", "1F469-1F3FD-200D-1F692"},
    {"bombeira_cor4", "1F469-1F3FE-200D-1F692"},
    {"bombeira_cor5", "1F469-1F3FF-200D-1F692"},
    {"mulher_com_bengala_para_cego", "1F469-200D-1F9AF"},
    {"mulher_com_bengala_para_cego_cor1", "1F469-1F3FB-200D-1F9AF"},
    {"mulher_com_bengala_para_cego_cor2", "1F469-1F3FC-200D-1F9AF"},
    {"mulher_com_bengala_para_cego_cor3", "1F469-1F3FD-200D-1F9AF"},
    {"mulher_com_bengala_para_cego_cor4", "1F469-1F3FE-200D-1F9AF"},
    {"mulher_com_bengala_para_cego_cor5", "1F469-1F3FF-200D-1F9AF"},
    {"mulher_com_bengala_para_cego_de_frente_para_a_direita", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"mulher_com_bengala_para_cego_de_frente_para_a_direita_cor1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"mulher_com_bengala_para_cego_de_frente_para_a_direita_cor2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"mulher_com_bengala_para_cego_de_frente_para_a_direita_cor3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"mulher_com_bengala_para_cego_de_frente_para_a_direita_cor4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"mulher_com_bengala_para_cego_de_frente_para_a_direita_cor5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"mulher_em_cadeira_de_rodas_motorizada", "1F469-200D-1F9BC"},
    {"mulher_em_cadeira_de_rodas_motorizada_cor1", "1F469-1F3FB-200D-1F9BC"},
    {"mulher_em_cadeira_de_rodas_motorizada_cor2", "1F469-1F3FC-200D-1F9BC"},
    {"mulher_em_cadeira_de_rodas_motorizada_cor3", "1F469-1F3FD-200D-1F9BC"},
    {"mulher_em_cadeira_de_rodas_motorizada_cor4", "1F469-1F3FE-200D-1F9BC"},
    {"mulher_em_cadeira_de_rodas_motorizada_cor5", "1F469-1F3FF-200D-1F9BC"},
    {"mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"mulher_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mulher_em_cadeira_de_rodas_manual", "1F469-200D-1F9BD"},
    {"mulher_em_cadeira_de_rodas_manual_cor1", "1F469-1F3FB-200D-1F9BD"},
    {"mulher_em_cadeira_de_rodas_manual_cor2", "1F469-1F3FC-200D-1F9BD"},
    {"mulher_em_cadeira_de_rodas_manual_cor3", "1F469-1F3FD-200D-1F9BD"},
    {"mulher_em_cadeira_de_rodas_manual_cor4", "1F469-1F3FE-200D-1F9BD"},
    {"mulher_em_cadeira_de_rodas_manual_cor5", "1F469-1F3FF-200D-1F9BD"},
    {"mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"mulher_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"pessoa_andando_de_frente_para_a_direita", "1F6B6-200D-27A1-FE0F"},
    {"pessoa_andando_de_frente_para_a_direita_cor1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"pessoa_andando_de_frente_para_a_direita_cor2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"pessoa_andando_de_frente_para_a_direita_cor3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"pessoa_andando_de_frente_para_a_direita_cor4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"pessoa_andando_de_frente_para_a_direita_cor5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"pessoa_ajoelhando_de_frente_para_a_direita", "1F9CE-200D-27A1-FE0F"},
    {"pessoa_ajoelhando_de_frente_para_a_direita_cor1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"pessoa_ajoelhando_de_frente_para_a_direita_cor2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"pessoa_ajoelhando_de_frente_para_a_direita_cor3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"pessoa_ajoelhando_de_frente_para_a_direita_cor4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"pessoa_ajoelhando_de_frente_para_a_direita_cor5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"profissional_de_saude", "1F9D1-200D-2695-FE0F"},
    {"profissional_de_saude_cor1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"profissional_de_saude_cor2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"profissional_de_saude_cor3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"profissional_de_saude_cor4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"profissional_de_saude_cor5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"juiz_no_tribunal", "1F9D1-200D-2696-FE0F"},
    {"juiz_no_tribunal_cor1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"juiz_no_tribunal_cor2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"juiz_no_tribunal_cor3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"juiz_no_tribunal_cor4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"juiz_no_tribunal_cor5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"piloto", "1F9D1-200D-2708-FE0F"},
    {"piloto_cor1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"piloto_cor2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"piloto_cor3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"piloto_cor4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"piloto_cor5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"agricultor", "1F9D1-200D-1F33E"},
    {"agricultor_cor1", "1F9D1-1F3FB-200D-1F33E"},
    {"agricultor_cor2", "1F9D1-1F3FC-200D-1F33E"},
    {"agricultor_cor3", "1F9D1-1F3FD-200D-1F33E"},
    {"agricultor_cor4", "1F9D1-1F3FE-200D-1F33E"},
    {"agricultor_cor5", "1F9D1-1F3FF-200D-1F33E"},
    {"chef_de_cozinha", "1F9D1-200D-1F373"},
    {"chef_de_cozinha_cor1", "1F9D1-1F3FB-200D-1F373"},
    {"chef_de_cozinha_cor2", "1F9D1-1F3FC-200D-1F373"},
    {"chef_de_cozinha_cor3", "1F9D1-1F3FD-200D-1F373"},
    {"chef_de_cozinha_cor4", "1F9D1-1F3FE-200D-1F373"},
    {"chef_de_cozinha_cor5", "1F9D1-1F3FF-200D-1F373"},
    {"pessoa_alimentando_bebe", "1F9D1-200D-1F37C"},
    {"pessoa_alimentando_bebe_cor1", "1F9D1-1F3FB-200D-1F37C"},
    {"pessoa_alimentando_bebe_cor2", "1F9D1-1F3FC-200D-1F37C"},
    {"pessoa_alimentando_bebe_cor3", "1F9D1-1F3FD-200D-1F37C"},
    {"pessoa_alimentando_bebe_cor4", "1F9D1-1F3FE-200D-1F37C"},
    {"pessoa_alimentando_bebe_cor5", "1F9D1-1F3FF-200D-1F37C"},
    {"noel", "1F9D1-200D-1F384"},
    {"noel_cor1", "1F9D1-1F3FB-200D-1F384"},
    {"noel_cor2", "1F9D1-1F3FC-200D-1F384"},
    {"noel_cor3", "1F9D1-1F3FD-200D-1F384"},
    {"noel_cor4", "1F9D1-1F3FE-200D-1F384"},
    {"noel_cor5", "1F9D1-1F3FF-200D-1F384"},
    {"aluno", "1F9D1-200D-1F393"},
    {"aluno_cor1", "1F9D1-1F3FB-200D-1F393"},
    {"aluno_cor2", "1F9D1-1F3FC-200D-1F393"},
    {"aluno_cor3", "1F9D1-1F3FD-200D-1F393"},
    {"aluno_cor4", "1F9D1-1F3FE-200D-1F393"},
    {"aluno_cor5", "1F9D1-1F3FF-200D-1F393"},
    {"cantor", "1F9D1-200D-1F3A4"},
    {"cantor_cor1", "1F9D1-1F3FB-200D-1F3A4"},
    {"cantor_cor2", "1F9D1-1F3FC-200D-1F3A4"},
    {"cantor_cor3", "1F9D1-1F3FD-200D-1F3A4"},
    {"cantor_cor4", "1F9D1-1F3FE-200D-1F3A4"},
    {"cantor_cor5", "1F9D1-1F3FF-200D-1F3A4"},
    {"artista", "1F9D1-200D-1F3A8"},
    {"artista_cor1", "1F9D1-1F3FB-200D-1F3A8"},
    {"artista_cor2", "1F9D1-1F3FC-200D-1F3A8"},
    {"artista_cor3", "1F9D1-1F3FD-200D-1F3A8"},
    {"artista_cor4", "1F9D1-1F3FE-200D-1F3A8"},
    {"artista_cor5", "1F9D1-1F3FF-200D-1F3A8"},
    {"professora_na_escola", "1F9D1-200D-1F3EB"},
    {"professora_na_escola_cor1", "1F9D1-1F3FB-200D-1F3EB"},
    {"professora_na_escola_cor2", "1F9D1-1F3FC-200D-1F3EB"},
    {"professora_na_escola_cor3", "1F9D1-1F3FD-200D-1F3EB"},
    {"professora_na_escola_cor4", "1F9D1-1F3FE-200D-1F3EB"},
    {"professora_na_escola_cor5", "1F9D1-1F3FF-200D-1F3EB"},
    {"funcionario_de_fabrica", "1F9D1-200D-1F3ED"},
    {"funcionario_de_fabrica_cor1", "1F9D1-1F3FB-200D-1F3ED"},
    {"funcionario_de_fabrica_cor2", "1F9D1-1F3FC-200D-1F3ED"},
    {"funcionario_de_fabrica_cor3", "1F9D1-1F3FD-200D-1F3ED"},
    {"funcionario_de_fabrica_cor4", "1F9D1-1F3FE-200D-1F3ED"},
    {"funcionario_de_fabrica_cor5", "1F9D1-1F3FF-200D-1F3ED"},
    {"programador", "1F9D1-200D-1F4BB"},
    {"programador_cor1", "1F9D1-1F3FB-200D-1F4BB"},
    {"programador_cor2", "1F9D1-1F3FC-200D-1F4BB"},
    {"programador_cor3", "1F9D1-1F3FD-200D-1F4BB"},
    {"programador_cor4", "1F9D1-1F3FE-200D-1F4BB"},
    {"programador_cor5", "1F9D1-1F3FF-200D-1F4BB"},
    {"trabalhador_de_escritorio", "1F9D1-200D-1F4BC"},
    {"trabalhador_de_escritorio_cor1", "1F9D1-1F3FB-200D-1F4BC"},
    {"trabalhador_de_escritorio_cor2", "1F9D1-1F3FC-200D-1F4BC"},
    {"trabalhador_de_escritorio_cor3", "1F9D1-1F3FD-200D-1F4BC"},
    {"trabalhador_de_escritorio_cor4", "1F9D1-1F3FE-200D-1F4BC"},
    {"trabalhador_de_escritorio_cor5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mecanico_trabalhando", "1F9D1-200D-1F527"},
    {"mecanico_trabalhando_cor1", "1F9D1-1F3FB-200D-1F527"},
    {"mecanico_trabalhando_cor2", "1F9D1-1F3FC-200D-1F527"},
    {"mecanico_trabalhando_cor3", "1F9D1-1F3FD-200D-1F527"},
    {"mecanico_trabalhando_cor4", "1F9D1-1F3FE-200D-1F527"},
    {"mecanico_trabalhando_cor5", "1F9D1-1F3FF-200D-1F527"},
    {"cientista", "1F9D1-200D-1F52C"},
    {"cientista_cor1", "1F9D1-1F3FB-200D-1F52C"},
    {"cientista_cor2", "1F9D1-1F3FC-200D-1F52C"},
    {"cientista_cor3", "1F9D1-1F3FD-200D-1F52C"},
    {"cientista_cor4", "1F9D1-1F3FE-200D-1F52C"},
    {"cientista_cor5", "1F9D1-1F3FF-200D-1F52C"},
    {"astronauta", "1F9D1-200D-1F680"},
    {"astronauta_cor1", "1F9D1-1F3FB-200D-1F680"},
    {"astronauta_cor2", "1F9D1-1F3FC-200D-1F680"},
    {"astronauta_cor3", "1F9D1-1F3FD-200D-1F680"},
    {"astronauta_cor4", "1F9D1-1F3FE-200D-1F680"},
    {"astronauta_cor5", "1F9D1-1F3FF-200D-1F680"},
    {"bombeiro", "1F9D1-200D-1F692"},
    {"bombeiro_cor1", "1F9D1-1F3FB-200D-1F692"},
    {"bombeiro_cor2", "1F9D1-1F3FC-200D-1F692"},
    {"bombeiro_cor3", "1F9D1-1F3FD-200D-1F692"},
    {"bombeiro_cor4", "1F9D1-1F3FE-200D-1F692"},
    {"bombeiro_cor5", "1F9D1-1F3FF-200D-1F692"},
    {"pessoa_com_bengala_para_cego", "1F9D1-200D-1F9AF"},
    {"pessoa_com_bengala_para_cego_cor1", "1F9D1-1F3FB-200D-1F9AF"},
    {"pessoa_com_bengala_para_cego_cor2", "1F9D1-1F3FC-200D-1F9AF"},
    {"pessoa_com_bengala_para_cego_cor3", "1F9D1-1F3FD-200D-1F9AF"},
    {"pessoa_com_bengala_para_cego_cor4", "1F9D1-1F3FE-200D-1F9AF"},
    {"pessoa_com_bengala_para_cego_cor5", "1F9D1-1F3FF-200D-1F9AF"},
    {"pessoa_com_bengala_para_cego_de_frente_para_a_direita", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"pessoa_com_bengala_para_cego_de_frente_para_a_direita_cor1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"pessoa_com_bengala_para_cego_de_frente_para_a_direita_cor2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"pessoa_com_bengala_para_cego_de_frente_para_a_direita_cor3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"pessoa_com_bengala_para_cego_de_frente_para_a_direita_cor4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"pessoa_com_bengala_para_cego_de_frente_para_a_direita_cor5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"pessoa_em_cadeira_de_rodas_motorizada", "1F9D1-200D-1F9BC"},
    {"pessoa_em_cadeira_de_rodas_motorizada_cor1", "1F9D1-1F3FB-200D-1F9BC"},
    {"pessoa_em_cadeira_de_rodas_motorizada_cor2", "1F9D1-1F3FC-200D-1F9BC"},
    {"pessoa_em_cadeira_de_rodas_motorizada_cor3", "1F9D1-1F3FD-200D-1F9BC"},
    {"pessoa_em_cadeira_de_rodas_motorizada_cor4", "1F9D1-1F3FE-200D-1F9BC"},
    {"pessoa_em_cadeira_de_rodas_motorizada_cor5", "1F9D1-1F3FF-200D-1F9BC"},
    {"pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"pessoa_em_cadeira_de_rodas_motorizada_de_frente_para_a_direita_cor5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"pessoa_em_cadeira_de_rodas_manual", "1F9D1-200D-1F9BD"},
    {"pessoa_em_cadeira_de_rodas_manual_cor1", "1F9D1-1F3FB-200D-1F9BD"},
    {"pessoa_em_cadeira_de_rodas_manual_cor2", "1F9D1-1F3FC-200D-1F9BD"},
    {"pessoa_em_cadeira_de_rodas_manual_cor3", "1F9D1-1F3FD-200D-1F9BD"},
    {"pessoa_em_cadeira_de_rodas_manual_cor4", "1F9D1-1F3FE-200D-1F9BD"},
    {"pessoa_em_cadeira_de_rodas_manual_cor5", "1F9D1-1F3FF-200D-1F9BD"},
    {"pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"pessoa_em_cadeira_de_rodas_manual_de_frente_para_a_direita_cor5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"mulher_jogando_basquete", "26F9-FE0F-200D-2640-FE0F"},
    {"mulher_jogando_basquete_cor1", "26F9-1F3FB-200D-2640-FE0F"},
    {"mulher_jogando_basquete_cor2", "26F9-1F3FC-200D-2640-FE0F"},
    {"mulher_jogando_basquete_cor3", "26F9-1F3FD-200D-2640-FE0F"},
    {"mulher_jogando_basquete_cor4", "26F9-1F3FE-200D-2640-FE0F"},
    {"mulher_jogando_basquete_cor5", "26F9-1F3FF-200D-2640-FE0F"},
    {"homem_jogando_basquete", "26F9-FE0F-200D-2642-FE0F"},
    {"homem_jogando_basquete_cor1", "26F9-1F3FB-200D-2642-FE0F"},
    {"homem_jogando_basquete_cor2", "26F9-1F3FC-200D-2642-FE0F"},
    {"homem_jogando_basquete_cor3", "26F9-1F3FD-200D-2642-FE0F"},
    {"homem_jogando_basquete_cor4", "26F9-1F3FE-200D-2642-FE0F"},
    {"homem_jogando_basquete_cor5", "26F9-1F3FF-200D-2642-FE0F"},
    {"mulher_correndo", "1F3C3-200D-2640-FE0F"},
    {"mulher_correndo_cor1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"mulher_correndo_cor2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"mulher_correndo_cor3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"mulher_correndo_cor4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"mulher_correndo_cor5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"mulher_correndo_de_frente_para_a_direita", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_correndo_de_frente_para_a_direita_cor1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_correndo_de_frente_para_a_direita_cor2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_correndo_de_frente_para_a_direita_cor3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_correndo_de_frente_para_a_direita_cor4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_correndo_de_frente_para_a_direita_cor5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"homem_correndo", "1F3C3-200D-2642-FE0F"},
    {"homem_correndo_cor1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"homem_correndo_cor2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"homem_correndo_cor3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"homem_correndo_cor4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"homem_correndo_cor5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"homem_correndo_de_frente_para_a_direita", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_correndo_de_frente_para_a_direita_cor1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_correndo_de_frente_para_a_direita_cor2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_correndo_de_frente_para_a_direita_cor3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_correndo_de_frente_para_a_direita_cor4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_correndo_de_frente_para_a_direita_cor5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mulher_surfista", "1F3C4-200D-2640-FE0F"},
    {"mulher_surfista_cor1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"mulher_surfista_cor2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"mulher_surfista_cor3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"mulher_surfista_cor4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"mulher_surfista_cor5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"homem_surfista", "1F3C4-200D-2642-FE0F"},
    {"homem_surfista_cor1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"homem_surfista_cor2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"homem_surfista_cor3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"homem_surfista_cor4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"homem_surfista_cor5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"mulher_nadando", "1F3CA-200D-2640-FE0F"},
    {"mulher_nadando_cor1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"mulher_nadando_cor2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"mulher_nadando_cor3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"mulher_nadando_cor4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"mulher_nadando_cor5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"homem_nadando", "1F3CA-200D-2642-FE0F"},
    {"homem_nadando_cor1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"homem_nadando_cor2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"homem_nadando_cor3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"homem_nadando_cor4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"homem_nadando_cor5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"mulher_levantando_peso", "1F3CB-FE0F-200D-2640-FE0F"},
    {"mulher_levantando_peso_cor1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"mulher_levantando_peso_cor2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"mulher_levantando_peso_cor3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"mulher_levantando_peso_cor4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"mulher_levantando_peso_cor5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"homem_levantando_peso", "1F3CB-FE0F-200D-2642-FE0F"},
    {"homem_levantando_peso_cor1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"homem_levantando_peso_cor2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"homem_levantando_peso_cor3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"homem_levantando_peso_cor4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"homem_levantando_peso_cor5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"mulher_golfista", "1F3CC-FE0F-200D-2640-FE0F"},
    {"mulher_golfista_cor1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"mulher_golfista_cor2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"mulher_golfista_cor3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"mulher_golfista_cor4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"mulher_golfista_cor5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"homem_golfista", "1F3CC-FE0F-200D-2642-FE0F"},
    {"homem_golfista_cor1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"homem_golfista_cor2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"homem_golfista_cor3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"homem_golfista_cor4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"homem_golfista_cor5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"policial_mulher", "1F46E-200D-2640-FE0F"},
    {"policial_mulher_cor1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"policial_mulher_cor2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"policial_mulher_cor3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"policial_mulher_cor4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"policial_mulher_cor5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"policial_homem", "1F46E-200D-2642-FE0F"},
    {"policial_homem_cor1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"policial_homem_cor2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"policial_homem_cor3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"policial_homem_cor4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"policial_homem_cor5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"mulheres_com_orelhas_de_coelho", "1F46F-200D-2640-FE0F"},
    {"mulheres_com_orelhas_de_coelho_cor1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"mulheres_com_orelhas_de_coelho_cor1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"mulheres_com_orelhas_de_coelho_cor1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"mulheres_com_orelhas_de_coelho_cor1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"mulheres_com_orelhas_de_coelho_cor2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"mulheres_com_orelhas_de_coelho_cor2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"mulheres_com_orelhas_de_coelho_cor2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"mulheres_com_orelhas_de_coelho_cor2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"mulheres_com_orelhas_de_coelho_cor3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"mulheres_com_orelhas_de_coelho_cor3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"mulheres_com_orelhas_de_coelho_cor3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"mulheres_com_orelhas_de_coelho_cor3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"mulheres_com_orelhas_de_coelho_cor4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"mulheres_com_orelhas_de_coelho_cor4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"mulheres_com_orelhas_de_coelho_cor4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"mulheres_com_orelhas_de_coelho_cor4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"mulheres_com_orelhas_de_coelho_cor5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"mulheres_com_orelhas_de_coelho_cor5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"mulheres_com_orelhas_de_coelho_cor5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"mulheres_com_orelhas_de_coelho_cor5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"mulheres_com_orelhas_de_coelho_cor1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"mulheres_com_orelhas_de_coelho_cor2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"mulheres_com_orelhas_de_coelho_cor3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"mulheres_com_orelhas_de_coelho_cor4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"mulheres_com_orelhas_de_coelho_cor5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"homens_com_orelhas_de_coelho", "1F46F-200D-2642-FE0F"},
    {"homens_com_orelhas_de_coelho_cor1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"homens_com_orelhas_de_coelho_cor1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"homens_com_orelhas_de_coelho_cor1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"homens_com_orelhas_de_coelho_cor1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"homens_com_orelhas_de_coelho_cor2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"homens_com_orelhas_de_coelho_cor2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"homens_com_orelhas_de_coelho_cor2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"homens_com_orelhas_de_coelho_cor2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"homens_com_orelhas_de_coelho_cor3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"homens_com_orelhas_de_coelho_cor3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"homens_com_orelhas_de_coelho_cor3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"homens_com_orelhas_de_coelho_cor3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"homens_com_orelhas_de_coelho_cor4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"homens_com_orelhas_de_coelho_cor4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"homens_com_orelhas_de_coelho_cor4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"homens_com_orelhas_de_coelho_cor4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"homens_com_orelhas_de_coelho_cor5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"homens_com_orelhas_de_coelho_cor5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"homens_com_orelhas_de_coelho_cor5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"homens_com_orelhas_de_coelho_cor5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"homens_com_orelhas_de_coelho_cor1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"homens_com_orelhas_de_coelho_cor2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"homens_com_orelhas_de_coelho_cor3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"homens_com_orelhas_de_coelho_cor4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"homens_com_orelhas_de_coelho_cor5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"noiva_com_veu", "1F470-200D-2640-FE0F"},
    {"noiva_com_veu_cor1", "1F470-1F3FB-200D-2640-FE0F"},
    {"noiva_com_veu_cor2", "1F470-1F3FC-200D-2640-FE0F"},
    {"noiva_com_veu_cor3", "1F470-1F3FD-200D-2640-FE0F"},
    {"noiva_com_veu_cor4", "1F470-1F3FE-200D-2640-FE0F"},
    {"noiva_com_veu_cor5", "1F470-1F3FF-200D-2640-FE0F"},
    {"noivo_com_veu", "1F470-200D-2642-FE0F"},
    {"noivo_com_veu_cor1", "1F470-1F3FB-200D-2642-FE0F"},
    {"noivo_com_veu_cor2", "1F470-1F3FC-200D-2642-FE0F"},
    {"noivo_com_veu_cor3", "1F470-1F3FD-200D-2642-FE0F"},
    {"noivo_com_veu_cor4", "1F470-1F3FE-200D-2642-FE0F"},
    {"noivo_com_veu_cor5", "1F470-1F3FF-200D-2642-FE0F"},
    {"mulher_cabelo_loiro", "1F471-200D-2640-FE0F"},
    {"mulher_cabelo_loiro_cor1", "1F471-1F3FB-200D-2640-FE0F"},
    {"mulher_cabelo_loiro_cor2", "1F471-1F3FC-200D-2640-FE0F"},
    {"mulher_cabelo_loiro_cor3", "1F471-1F3FD-200D-2640-FE0F"},
    {"mulher_cabelo_loiro_cor4", "1F471-1F3FE-200D-2640-FE0F"},
    {"mulher_cabelo_loiro_cor5", "1F471-1F3FF-200D-2640-FE0F"},
    {"homem_cabelo_loiro", "1F471-200D-2642-FE0F"},
    {"homem_cabelo_loiro_cor1", "1F471-1F3FB-200D-2642-FE0F"},
    {"homem_cabelo_loiro_cor2", "1F471-1F3FC-200D-2642-FE0F"},
    {"homem_cabelo_loiro_cor3", "1F471-1F3FD-200D-2642-FE0F"},
    {"homem_cabelo_loiro_cor4", "1F471-1F3FE-200D-2642-FE0F"},
    {"homem_cabelo_loiro_cor5", "1F471-1F3FF-200D-2642-FE0F"},
    {"mulher_com_turbante", "1F473-200D-2640-FE0F"},
    {"mulher_com_turbante_cor1", "1F473-1F3FB-200D-2640-FE0F"},
    {"mulher_com_turbante_cor2", "1F473-1F3FC-200D-2640-FE0F"},
    {"mulher_com_turbante_cor3", "1F473-1F3FD-200D-2640-FE0F"},
    {"mulher_com_turbante_cor4", "1F473-1F3FE-200D-2640-FE0F"},
    {"mulher_com_turbante_cor5", "1F473-1F3FF-200D-2640-FE0F"},
    {"homem_com_turbante", "1F473-200D-2642-FE0F"},
    {"homem_com_turbante_cor1", "1F473-1F3FB-200D-2642-FE0F"},
    {"homem_com_turbante_cor2", "1F473-1F3FC-200D-2642-FE0F"},
    {"homem_com_turbante_cor3", "1F473-1F3FD-200D-2642-FE0F"},
    {"homem_com_turbante_cor4", "1F473-1F3FE-200D-2642-FE0F"},
    {"homem_com_turbante_cor5", "1F473-1F3FF-200D-2642-FE0F"},
    {"pedreira", "1F477-200D-2640-FE0F"},
    {"pedreira_cor1", "1F477-1F3FB-200D-2640-FE0F"},
    {"pedreira_cor2", "1F477-1F3FC-200D-2640-FE0F"},
    {"pedreira_cor3", "1F477-1F3FD-200D-2640-FE0F"},
    {"pedreira_cor4", "1F477-1F3FE-200D-2640-FE0F"},
    {"pedreira_cor5", "1F477-1F3FF-200D-2640-FE0F"},
    {"pedreiro", "1F477-200D-2642-FE0F"},
    {"pedreiro_cor1", "1F477-1F3FB-200D-2642-FE0F"},
    {"pedreiro_cor2", "1F477-1F3FC-200D-2642-FE0F"},
    {"pedreiro_cor3", "1F477-1F3FD-200D-2642-FE0F"},
    {"pedreiro_cor4", "1F477-1F3FE-200D-2642-FE0F"},
    {"pedreiro_cor5", "1F477-1F3FF-200D-2642-FE0F"},
    {"mulher_com_a_palma_virada_para_cima", "1F481-200D-2640-FE0F"},
    {"mulher_com_a_palma_virada_para_cima_cor1", "1F481-1F3FB-200D-2640-FE0F"},
    {"mulher_com_a_palma_virada_para_cima_cor2", "1F481-1F3FC-200D-2640-FE0F"},
    {"mulher_com_a_palma_virada_para_cima_cor3", "1F481-1F3FD-200D-2640-FE0F"},
    {"mulher_com_a_palma_virada_para_cima_cor4", "1F481-1F3FE-200D-2640-FE0F"},
    {"mulher_com_a_palma_virada_para_cima_cor5", "1F481-1F3FF-200D-2640-FE0F"},
    {"homem_com_a_palma_virada_para_cima", "1F481-200D-2642-FE0F"},
    {"homem_com_a_palma_virada_para_cima_cor1", "1F481-1F3FB-200D-2642-FE0F"},
    {"homem_com_a_palma_virada_para_cima_cor2", "1F481-1F3FC-200D-2642-FE0F"},
    {"homem_com_a_palma_virada_para_cima_cor3", "1F481-1F3FD-200D-2642-FE0F"},
    {"homem_com_a_palma_virada_para_cima_cor4", "1F481-1F3FE-200D-2642-FE0F"},
    {"homem_com_a_palma_virada_para_cima_cor5", "1F481-1F3FF-200D-2642-FE0F"},
    {"guarda_mulher", "1F482-200D-2640-FE0F"},
    {"guarda_mulher_cor1", "1F482-1F3FB-200D-2640-FE0F"},
    {"guarda_mulher_cor2", "1F482-1F3FC-200D-2640-FE0F"},
    {"guarda_mulher_cor3", "1F482-1F3FD-200D-2640-FE0F"},
    {"guarda_mulher_cor4", "1F482-1F3FE-200D-2640-FE0F"},
    {"guarda_mulher_cor5", "1F482-1F3FF-200D-2640-FE0F"},
    {"guarda_homem", "1F482-200D-2642-FE0F"},
    {"guarda_homem_cor1", "1F482-1F3FB-200D-2642-FE0F"},
    {"guarda_homem_cor2", "1F482-1F3FC-200D-2642-FE0F"},
    {"guarda_homem_cor3", "1F482-1F3FD-200D-2642-FE0F"},
    {"guarda_homem_cor4", "1F482-1F3FE-200D-2642-FE0F"},
    {"guarda_homem_cor5", "1F482-1F3FF-200D-2642-FE0F"},
    {"mulher_recebendo_massagem_facial", "1F486-200D-2640-FE0F"},
    {"mulher_recebendo_massagem_facial_cor1", "1F486-1F3FB-200D-2640-FE0F"},
    {"mulher_recebendo_massagem_facial_cor2", "1F486-1F3FC-200D-2640-FE0F"},
    {"mulher_recebendo_massagem_facial_cor3", "1F486-1F3FD-200D-2640-FE0F"},
    {"mulher_recebendo_massagem_facial_cor4", "1F486-1F3FE-200D-2640-FE0F"},
    {"mulher_recebendo_massagem_facial_cor5", "1F486-1F3FF-200D-2640-FE0F"},
    {"homem_recebendo_massagem_facial", "1F486-200D-2642-FE0F"},
    {"homem_recebendo_massagem_facial_cor1", "1F486-1F3FB-200D-2642-FE0F"},
    {"homem_recebendo_massagem_facial_cor2", "1F486-1F3FC-200D-2642-FE0F"},
    {"homem_recebendo_massagem_facial_cor3", "1F486-1F3FD-200D-2642-FE0F"},
    {"homem_recebendo_massagem_facial_cor4", "1F486-1F3FE-200D-2642-FE0F"},
    {"homem_recebendo_massagem_facial_cor5", "1F486-1F3FF-200D-2642-FE0F"},
    {"mulher_cortando_o_cabelo", "1F487-200D-2640-FE0F"},
    {"mulher_cortando_o_cabelo_cor1", "1F487-1F3FB-200D-2640-FE0F"},
    {"mulher_cortando_o_cabelo_cor2", "1F487-1F3FC-200D-2640-FE0F"},
    {"mulher_cortando_o_cabelo_cor3", "1F487-1F3FD-200D-2640-FE0F"},
    {"mulher_cortando_o_cabelo_cor4", "1F487-1F3FE-200D-2640-FE0F"},
    {"mulher_cortando_o_cabelo_cor5", "1F487-1F3FF-200D-2640-FE0F"},
    {"homem_cortando_o_cabelo", "1F487-200D-2642-FE0F"},
    {"homem_cortando_o_cabelo_cor1", "1F487-1F3FB-200D-2642-FE0F"},
    {"homem_cortando_o_cabelo_cor2", "1F487-1F3FC-200D-2642-FE0F"},
    {"homem_cortando_o_cabelo_cor3", "1F487-1F3FD-200D-2642-FE0F"},
    {"homem_cortando_o_cabelo_cor4", "1F487-1F3FE-200D-2642-FE0F"},
    {"homem_cortando_o_cabelo_cor5", "1F487-1F3FF-200D-2642-FE0F"},
    {"detetive_mulher", "1F575-FE0F-200D-2640-FE0F"},
    {"detetive_mulher_cor1", "1F575-1F3FB-200D-2640-FE0F"},
    {"detetive_mulher_cor2", "1F575-1F3FC-200D-2640-FE0F"},
    {"detetive_mulher_cor3", "1F575-1F3FD-200D-2640-FE0F"},
    {"detetive_mulher_cor4", "1F575-1F3FE-200D-2640-FE0F"},
    {"detetive_mulher_cor5", "1F575-1F3FF-200D-2640-FE0F"},
    {"detetive_homem", "1F575-FE0F-200D-2642-FE0F"},
    {"detetive_homem_cor1", "1F575-1F3FB-200D-2642-FE0F"},
    {"detetive_homem_cor2", "1F575-1F3FC-200D-2642-FE0F"},
    {"detetive_homem_cor3", "1F575-1F3FD-200D-2642-FE0F"},
    {"detetive_homem_cor4", "1F575-1F3FE-200D-2642-FE0F"},
    {"detetive_homem_cor5", "1F575-1F3FF-200D-2642-FE0F"},
    {"mulher_fazendo_gesto_de_nao", "1F645-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_nao_cor1", "1F645-1F3FB-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_nao_cor2", "1F645-1F3FC-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_nao_cor3", "1F645-1F3FD-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_nao_cor4", "1F645-1F3FE-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_nao_cor5", "1F645-1F3FF-200D-2640-FE0F"},
    {"homem_fazendo_gesto_de_nao", "1F645-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_nao_cor1", "1F645-1F3FB-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_nao_cor2", "1F645-1F3FC-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_nao_cor3", "1F645-1F3FD-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_nao_cor4", "1F645-1F3FE-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_nao_cor5", "1F645-1F3FF-200D-2642-FE0F"},
    {"mulher_fazendo_gesto_de_ok", "1F646-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_ok_cor1", "1F646-1F3FB-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_ok_cor2", "1F646-1F3FC-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_ok_cor3", "1F646-1F3FD-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_ok_cor4", "1F646-1F3FE-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_ok_cor5", "1F646-1F3FF-200D-2640-FE0F"},
    {"homem_fazendo_gesto_de_ok", "1F646-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_ok_cor1", "1F646-1F3FB-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_ok_cor2", "1F646-1F3FC-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_ok_cor3", "1F646-1F3FD-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_ok_cor4", "1F646-1F3FE-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_ok_cor5", "1F646-1F3FF-200D-2642-FE0F"},
    {"mulher_fazendo_reverencia", "1F647-200D-2640-FE0F"},
    {"mulher_fazendo_reverencia_cor1", "1F647-1F3FB-200D-2640-FE0F"},
    {"mulher_fazendo_reverencia_cor2", "1F647-1F3FC-200D-2640-FE0F"},
    {"mulher_fazendo_reverencia_cor3", "1F647-1F3FD-200D-2640-FE0F"},
    {"mulher_fazendo_reverencia_cor4", "1F647-1F3FE-200D-2640-FE0F"},
    {"mulher_fazendo_reverencia_cor5", "1F647-1F3FF-200D-2640-FE0F"},
    {"homem_fazendo_reverencia", "1F647-200D-2642-FE0F"},
    {"homem_fazendo_reverencia_cor1", "1F647-1F3FB-200D-2642-FE0F"},
    {"homem_fazendo_reverencia_cor2", "1F647-1F3FC-200D-2642-FE0F"},
    {"homem_fazendo_reverencia_cor3", "1F647-1F3FD-200D-2642-FE0F"},
    {"homem_fazendo_reverencia_cor4", "1F647-1F3FE-200D-2642-FE0F"},
    {"homem_fazendo_reverencia_cor5", "1F647-1F3FF-200D-2642-FE0F"},
    {"mulher_levantando_a_mao", "1F64B-200D-2640-FE0F"},
    {"mulher_levantando_a_mao_cor1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"mulher_levantando_a_mao_cor2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"mulher_levantando_a_mao_cor3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"mulher_levantando_a_mao_cor4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"mulher_levantando_a_mao_cor5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"homem_levantando_a_mao", "1F64B-200D-2642-FE0F"},
    {"homem_levantando_a_mao_cor1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"homem_levantando_a_mao_cor2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"homem_levantando_a_mao_cor3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"homem_levantando_a_mao_cor4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"homem_levantando_a_mao_cor5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"mulher_franzindo_a_sobrancelha", "1F64D-200D-2640-FE0F"},
    {"mulher_franzindo_a_sobrancelha_cor1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"mulher_franzindo_a_sobrancelha_cor2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"mulher_franzindo_a_sobrancelha_cor3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"mulher_franzindo_a_sobrancelha_cor4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"mulher_franzindo_a_sobrancelha_cor5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"homem_franzindo_a_sobrancelha", "1F64D-200D-2642-FE0F"},
    {"homem_franzindo_a_sobrancelha_cor1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"homem_franzindo_a_sobrancelha_cor2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"homem_franzindo_a_sobrancelha_cor3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"homem_franzindo_a_sobrancelha_cor4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"homem_franzindo_a_sobrancelha_cor5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"mulher_fazendo_bico", "1F64E-200D-2640-FE0F"},
    {"mulher_fazendo_bico_cor1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"mulher_fazendo_bico_cor2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"mulher_fazendo_bico_cor3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"mulher_fazendo_bico_cor4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"mulher_fazendo_bico_cor5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"homem_fazendo_bico", "1F64E-200D-2642-FE0F"},
    {"homem_fazendo_bico_cor1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"homem_fazendo_bico_cor2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"homem_fazendo_bico_cor3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"homem_fazendo_bico_cor4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"homem_fazendo_bico_cor5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"mulher_remando", "1F6A3-200D-2640-FE0F"},
    {"mulher_remando_cor1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"mulher_remando_cor2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"mulher_remando_cor3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"mulher_remando_cor4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"mulher_remando_cor5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"homem_remando", "1F6A3-200D-2642-FE0F"},
    {"homem_remando_cor1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"homem_remando_cor2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"homem_remando_cor3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"homem_remando_cor4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"homem_remando_cor5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"mulher_ciclista", "1F6B4-200D-2640-FE0F"},
    {"mulher_ciclista_cor1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"mulher_ciclista_cor2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"mulher_ciclista_cor3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"mulher_ciclista_cor4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"mulher_ciclista_cor5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"homem_ciclista", "1F6B4-200D-2642-FE0F"},
    {"homem_ciclista_cor1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"homem_ciclista_cor2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"homem_ciclista_cor3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"homem_ciclista_cor4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"homem_ciclista_cor5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"mulher_fazendo_mountain_bike", "1F6B5-200D-2640-FE0F"},
    {"mulher_fazendo_mountain_bike_cor1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"mulher_fazendo_mountain_bike_cor2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"mulher_fazendo_mountain_bike_cor3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"mulher_fazendo_mountain_bike_cor4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"mulher_fazendo_mountain_bike_cor5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"homem_fazendo_mountain_bike", "1F6B5-200D-2642-FE0F"},
    {"homem_fazendo_mountain_bike_cor1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"homem_fazendo_mountain_bike_cor2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"homem_fazendo_mountain_bike_cor3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"homem_fazendo_mountain_bike_cor4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"homem_fazendo_mountain_bike_cor5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"mulher_andando", "1F6B6-200D-2640-FE0F"},
    {"mulher_andando_cor1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"mulher_andando_cor2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"mulher_andando_cor3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"mulher_andando_cor4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"mulher_andando_cor5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"mulher_andando_de_frente_para_a_direita", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_andando_de_frente_para_a_direita_cor1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_andando_de_frente_para_a_direita_cor2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_andando_de_frente_para_a_direita_cor3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_andando_de_frente_para_a_direita_cor4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_andando_de_frente_para_a_direita_cor5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"homem_andando", "1F6B6-200D-2642-FE0F"},
    {"homem_andando_cor1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"homem_andando_cor2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"homem_andando_cor3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"homem_andando_cor4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"homem_andando_cor5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"homem_andando_de_frente_para_a_direita", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_andando_de_frente_para_a_direita_cor1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_andando_de_frente_para_a_direita_cor2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_andando_de_frente_para_a_direita_cor3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_andando_de_frente_para_a_direita_cor4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_andando_de_frente_para_a_direita_cor5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mulher_decepcionada", "1F926-200D-2640-FE0F"},
    {"mulher_decepcionada_cor1", "1F926-1F3FB-200D-2640-FE0F"},
    {"mulher_decepcionada_cor2", "1F926-1F3FC-200D-2640-FE0F"},
    {"mulher_decepcionada_cor3", "1F926-1F3FD-200D-2640-FE0F"},
    {"mulher_decepcionada_cor4", "1F926-1F3FE-200D-2640-FE0F"},
    {"mulher_decepcionada_cor5", "1F926-1F3FF-200D-2640-FE0F"},
    {"homem_decepcionado", "1F926-200D-2642-FE0F"},
    {"homem_decepcionado_cor1", "1F926-1F3FB-200D-2642-FE0F"},
    {"homem_decepcionado_cor2", "1F926-1F3FC-200D-2642-FE0F"},
    {"homem_decepcionado_cor3", "1F926-1F3FD-200D-2642-FE0F"},
    {"homem_decepcionado_cor4", "1F926-1F3FE-200D-2642-FE0F"},
    {"homem_decepcionado_cor5", "1F926-1F3FF-200D-2642-FE0F"},
    {"mulher_de_smoking", "1F935-200D-2640-FE0F"},
    {"mulher_de_smoking_cor1", "1F935-1F3FB-200D-2640-FE0F"},
    {"mulher_de_smoking_cor2", "1F935-1F3FC-200D-2640-FE0F"},
    {"mulher_de_smoking_cor3", "1F935-1F3FD-200D-2640-FE0F"},
    {"mulher_de_smoking_cor4", "1F935-1F3FE-200D-2640-FE0F"},
    {"mulher_de_smoking_cor5", "1F935-1F3FF-200D-2640-FE0F"},
    {"homem_de_smoking", "1F935-200D-2642-FE0F"},
    {"homem_de_smoking_cor1", "1F935-1F3FB-200D-2642-FE0F"},
    {"homem_de_smoking_cor2", "1F935-1F3FC-200D-2642-FE0F"},
    {"homem_de_smoking_cor3", "1F935-1F3FD-200D-2642-FE0F"},
    {"homem_de_smoking_cor4", "1F935-1F3FE-200D-2642-FE0F"},
    {"homem_de_smoking_cor5", "1F935-1F3FF-200D-2642-FE0F"},
    {"mulher_dando_de_ombros", "1F937-200D-2640-FE0F"},
    {"mulher_dando_de_ombros_cor1", "1F937-1F3FB-200D-2640-FE0F"},
    {"mulher_dando_de_ombros_cor2", "1F937-1F3FC-200D-2640-FE0F"},
    {"mulher_dando_de_ombros_cor3", "1F937-1F3FD-200D-2640-FE0F"},
    {"mulher_dando_de_ombros_cor4", "1F937-1F3FE-200D-2640-FE0F"},
    {"mulher_dando_de_ombros_cor5", "1F937-1F3FF-200D-2640-FE0F"},
    {"homem_dando_de_ombros", "1F937-200D-2642-FE0F"},
    {"homem_dando_de_ombros_cor1", "1F937-1F3FB-200D-2642-FE0F"},
    {"homem_dando_de_ombros_cor2", "1F937-1F3FC-200D-2642-FE0F"},
    {"homem_dando_de_ombros_cor3", "1F937-1F3FD-200D-2642-FE0F"},
    {"homem_dando_de_ombros_cor4", "1F937-1F3FE-200D-2642-FE0F"},
    {"homem_dando_de_ombros_cor5", "1F937-1F3FF-200D-2642-FE0F"},
    {"mulher_fazendo_estrela", "1F938-200D-2640-FE0F"},
    {"mulher_fazendo_estrela_cor1", "1F938-1F3FB-200D-2640-FE0F"},
    {"mulher_fazendo_estrela_cor2", "1F938-1F3FC-200D-2640-FE0F"},
    {"mulher_fazendo_estrela_cor3", "1F938-1F3FD-200D-2640-FE0F"},
    {"mulher_fazendo_estrela_cor4", "1F938-1F3FE-200D-2640-FE0F"},
    {"mulher_fazendo_estrela_cor5", "1F938-1F3FF-200D-2640-FE0F"},
    {"homem_fazendo_estrela", "1F938-200D-2642-FE0F"},
    {"homem_fazendo_estrela_cor1", "1F938-1F3FB-200D-2642-FE0F"},
    {"homem_fazendo_estrela_cor2", "1F938-1F3FC-200D-2642-FE0F"},
    {"homem_fazendo_estrela_cor3", "1F938-1F3FD-200D-2642-FE0F"},
    {"homem_fazendo_estrela_cor4", "1F938-1F3FE-200D-2642-FE0F"},
    {"homem_fazendo_estrela_cor5", "1F938-1F3FF-200D-2642-FE0F"},
    {"mulher_malabarista", "1F939-200D-2640-FE0F"},
    {"mulher_malabarista_cor1", "1F939-1F3FB-200D-2640-FE0F"},
    {"mulher_malabarista_cor2", "1F939-1F3FC-200D-2640-FE0F"},
    {"mulher_malabarista_cor3", "1F939-1F3FD-200D-2640-FE0F"},
    {"mulher_malabarista_cor4", "1F939-1F3FE-200D-2640-FE0F"},
    {"mulher_malabarista_cor5", "1F939-1F3FF-200D-2640-FE0F"},
    {"homem_malabarista", "1F939-200D-2642-FE0F"},
    {"homem_malabarista_cor1", "1F939-1F3FB-200D-2642-FE0F"},
    {"homem_malabarista_cor2", "1F939-1F3FC-200D-2642-FE0F"},
    {"homem_malabarista_cor3", "1F939-1F3FD-200D-2642-FE0F"},
    {"homem_malabarista_cor4", "1F939-1F3FE-200D-2642-FE0F"},
    {"homem_malabarista_cor5", "1F939-1F3FF-200D-2642-FE0F"},
    {"mulheres_lutando", "1F93C-200D-2640-FE0F"},
    {"mulheres_lutando_cor1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"mulheres_lutando_cor1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"mulheres_lutando_cor1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"mulheres_lutando_cor1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"mulheres_lutando_cor2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"mulheres_lutando_cor2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"mulheres_lutando_cor2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"mulheres_lutando_cor2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"mulheres_lutando_cor3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"mulheres_lutando_cor3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"mulheres_lutando_cor3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"mulheres_lutando_cor3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"mulheres_lutando_cor4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"mulheres_lutando_cor4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"mulheres_lutando_cor4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"mulheres_lutando_cor4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"mulheres_lutando_cor5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"mulheres_lutando_cor5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"mulheres_lutando_cor5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"mulheres_lutando_cor5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"mulheres_lutando_cor1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"mulheres_lutando_cor2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"mulheres_lutando_cor3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"mulheres_lutando_cor4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"mulheres_lutando_cor5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"homens_lutando", "1F93C-200D-2642-FE0F"},
    {"homens_lutando_cor1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"homens_lutando_cor1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"homens_lutando_cor1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"homens_lutando_cor1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"homens_lutando_cor2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"homens_lutando_cor2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"homens_lutando_cor2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"homens_lutando_cor2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"homens_lutando_cor3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"homens_lutando_cor3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"homens_lutando_cor3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"homens_lutando_cor3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"homens_lutando_cor4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"homens_lutando_cor4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"homens_lutando_cor4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"homens_lutando_cor4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"homens_lutando_cor5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"homens_lutando_cor5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"homens_lutando_cor5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"homens_lutando_cor5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"homens_lutando_cor1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"homens_lutando_cor2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"homens_lutando_cor3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"homens_lutando_cor4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"homens_lutando_cor5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"mulher_jogando_polo_aquatico", "1F93D-200D-2640-FE0F"},
    {"mulher_jogando_polo_aquatico_cor1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"mulher_jogando_polo_aquatico_cor2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"mulher_jogando_polo_aquatico_cor3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"mulher_jogando_polo_aquatico_cor4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"mulher_jogando_polo_aquatico_cor5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"homem_jogando_polo_aquatico", "1F93D-200D-2642-FE0F"},
    {"homem_jogando_polo_aquatico_cor1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"homem_jogando_polo_aquatico_cor2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"homem_jogando_polo_aquatico_cor3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"homem_jogando_polo_aquatico_cor4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"homem_jogando_polo_aquatico_cor5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"jogadora_de_handebol", "1F93E-200D-2640-FE0F"},
    {"jogadora_de_handebol_cor1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"jogadora_de_handebol_cor2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"jogadora_de_handebol_cor3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"jogadora_de_handebol_cor4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"jogadora_de_handebol_cor5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"jogador_de_handebol", "1F93E-200D-2642-FE0F"},
    {"jogador_de_handebol_cor1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"jogador_de_handebol_cor2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"jogador_de_handebol_cor3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"jogador_de_handebol_cor4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"jogador_de_handebol_cor5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"super_heroina", "1F9B8-200D-2640-FE0F"},
    {"super_heroina_cor1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"super_heroina_cor2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"super_heroina_cor3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"super_heroina_cor4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"super_heroina_cor5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"homem_super_heroi", "1F9B8-200D-2642-FE0F"},
    {"homem_super_heroi_cor1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"homem_super_heroi_cor2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"homem_super_heroi_cor3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"homem_super_heroi_cor4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"homem_super_heroi_cor5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"supervila", "1F9B9-200D-2640-FE0F"},
    {"supervila_cor1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"supervila_cor2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"supervila_cor3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"supervila_cor4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"supervila_cor5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"homem_supervilao", "1F9B9-200D-2642-FE0F"},
    {"homem_supervilao_cor1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"homem_supervilao_cor2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"homem_supervilao_cor3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"homem_supervilao_cor4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"homem_supervilao_cor5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"mulher_em_pe", "1F9CD-200D-2640-FE0F"},
    {"mulher_em_pe_cor1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"mulher_em_pe_cor2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"mulher_em_pe_cor3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"mulher_em_pe_cor4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"mulher_em_pe_cor5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"homem_em_pe", "1F9CD-200D-2642-FE0F"},
    {"homem_em_pe_cor1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"homem_em_pe_cor2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"homem_em_pe_cor3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"homem_em_pe_cor4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"homem_em_pe_cor5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"mulher_ajoelhando", "1F9CE-200D-2640-FE0F"},
    {"mulher_ajoelhando_cor1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"mulher_ajoelhando_cor2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"mulher_ajoelhando_cor3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"mulher_ajoelhando_cor4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"mulher_ajoelhando_cor5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"mulher_ajoelhando_de_frente_para_a_direita", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_ajoelhando_de_frente_para_a_direita_cor1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_ajoelhando_de_frente_para_a_direita_cor2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_ajoelhando_de_frente_para_a_direita_cor3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_ajoelhando_de_frente_para_a_direita_cor4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mulher_ajoelhando_de_frente_para_a_direita_cor5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"homem_ajoelhando", "1F9CE-200D-2642-FE0F"},
    {"homem_ajoelhando_cor1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"homem_ajoelhando_cor2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"homem_ajoelhando_cor3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"homem_ajoelhando_cor4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"homem_ajoelhando_cor5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"homem_ajoelhando_de_frente_para_a_direita", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_ajoelhando_de_frente_para_a_direita_cor1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_ajoelhando_de_frente_para_a_direita_cor2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_ajoelhando_de_frente_para_a_direita_cor3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_ajoelhando_de_frente_para_a_direita_cor4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homem_ajoelhando_de_frente_para_a_direita_cor5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mulher_surda", "1F9CF-200D-2640-FE0F"},
    {"mulher_surda_cor1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"mulher_surda_cor2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"mulher_surda_cor3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"mulher_surda_cor4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"mulher_surda_cor5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"homem_surdo", "1F9CF-200D-2642-FE0F"},
    {"homem_surdo_cor1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"homem_surdo_cor2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"homem_surdo_cor3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"homem_surdo_cor4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"homem_surdo_cor5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"mulher_barba", "1F9D4-200D-2640-FE0F"},
    {"mulher_barba_cor1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"mulher_barba_cor2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"mulher_barba_cor3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"mulher_barba_cor4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"mulher_barba_cor5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"homem_barbudo", "1F9D4-200D-2642-FE0F"},
    {"homem_barbudo_cor1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"homem_barbudo_cor2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"homem_barbudo_cor3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"homem_barbudo_cor4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"homem_barbudo_cor5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"mulher_na_sauna", "1F9D6-200D-2640-FE0F"},
    {"mulher_na_sauna_cor1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"mulher_na_sauna_cor2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"mulher_na_sauna_cor3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"mulher_na_sauna_cor4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"mulher_na_sauna_cor5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"homem_na_sauna", "1F9D6-200D-2642-FE0F"},
    {"homem_na_sauna_cor1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"homem_na_sauna_cor2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"homem_na_sauna_cor3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"homem_na_sauna_cor4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"homem_na_sauna_cor5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"mulher_escalando", "1F9D7-200D-2640-FE0F"},
    {"mulher_escalando_cor1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"mulher_escalando_cor2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"mulher_escalando_cor3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"mulher_escalando_cor4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"mulher_escalando_cor5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"homem_escalando", "1F9D7-200D-2642-FE0F"},
    {"homem_escalando_cor1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"homem_escalando_cor2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"homem_escalando_cor3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"homem_escalando_cor4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"homem_escalando_cor5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"mulher_na_posicao_de_lotus", "1F9D8-200D-2640-FE0F"},
    {"mulher_na_posicao_de_lotus_cor1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"mulher_na_posicao_de_lotus_cor2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"mulher_na_posicao_de_lotus_cor3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"mulher_na_posicao_de_lotus_cor4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"mulher_na_posicao_de_lotus_cor5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"homem_na_posicao_de_lotus", "1F9D8-200D-2642-FE0F"},
    {"homem_na_posicao_de_lotus_cor1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"homem_na_posicao_de_lotus_cor2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"homem_na_posicao_de_lotus_cor3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"homem_na_posicao_de_lotus_cor4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"homem_na_posicao_de_lotus_cor5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"maga", "1F9D9-200D-2640-FE0F"},
    {"maga_cor1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"maga_cor2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"maga_cor3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"maga_cor4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"maga_cor5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"homem_mago", "1F9D9-200D-2642-FE0F"},
    {"homem_mago_cor1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"homem_mago_cor2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"homem_mago_cor3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"homem_mago_cor4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"homem_mago_cor5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"mulher_fada", "1F9DA-200D-2640-FE0F"},
    {"mulher_fada_cor1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"mulher_fada_cor2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"mulher_fada_cor3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"mulher_fada_cor4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"mulher_fada_cor5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"homem_fada", "1F9DA-200D-2642-FE0F"},
    {"homem_fada_cor1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"homem_fada_cor2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"homem_fada_cor3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"homem_fada_cor4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"homem_fada_cor5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"mulher_vampira", "1F9DB-200D-2640-FE0F"},
    {"mulher_vampira_cor1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"mulher_vampira_cor2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"mulher_vampira_cor3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"mulher_vampira_cor4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"mulher_vampira_cor5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"homem_vampiro", "1F9DB-200D-2642-FE0F"},
    {"homem_vampiro_cor1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"homem_vampiro_cor2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"homem_vampiro_cor3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"homem_vampiro_cor4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"homem_vampiro_cor5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"sereia", "1F9DC-200D-2640-FE0F"},
    {"sereia_cor1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"sereia_cor2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"sereia_cor3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"sereia_cor4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"sereia_cor5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"sereio", "1F9DC-200D-2642-FE0F"},
    {"sereio_cor1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"sereio_cor2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"sereio_cor3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"sereio_cor4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"sereio_cor5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"elfa", "1F9DD-200D-2640-FE0F"},
    {"elfa_cor1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"elfa_cor2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"elfa_cor3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"elfa_cor4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"elfa_cor5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"elfo_homem", "1F9DD-200D-2642-FE0F"},
    {"elfo_homem_cor1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"elfo_homem_cor2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"elfo_homem_cor3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"elfo_homem_cor4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"elfo_homem_cor5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"mulher_genio", "1F9DE-200D-2640-FE0F"},
    {"homem_genio", "1F9DE-200D-2642-FE0F"},
    {"mulher_zumbi", "1F9DF-200D-2640-FE0F"},
    {"homem_zumbi", "1F9DF-200D-2642-FE0F"},
    {"homem_cabelo_vermelho", "1F468-200D-1F9B0"},
    {"homem_cabelo_vermelho_cor1", "1F468-1F3FB-200D-1F9B0"},
    {"homem_cabelo_vermelho_cor2", "1F468-1F3FC-200D-1F9B0"},
    {"homem_cabelo_vermelho_cor3", "1F468-1F3FD-200D-1F9B0"},
    {"homem_cabelo_vermelho_cor4", "1F468-1F3FE-200D-1F9B0"},
    {"homem_cabelo_vermelho_cor5", "1F468-1F3FF-200D-1F9B0"},
    {"homem_cabelo_cacheado", "1F468-200D-1F9B1"},
    {"homem_cabelo_cacheado_cor1", "1F468-1F3FB-200D-1F9B1"},
    {"homem_cabelo_cacheado_cor2", "1F468-1F3FC-200D-1F9B1"},
    {"homem_cabelo_cacheado_cor3", "1F468-1F3FD-200D-1F9B1"},
    {"homem_cabelo_cacheado_cor4", "1F468-1F3FE-200D-1F9B1"},
    {"homem_cabelo_cacheado_cor5", "1F468-1F3FF-200D-1F9B1"},
    {"homem_careca", "1F468-200D-1F9B2"},
    {"homem_careca_cor1", "1F468-1F3FB-200D-1F9B2"},
    {"homem_careca_cor2", "1F468-1F3FC-200D-1F9B2"},
    {"homem_careca_cor3", "1F468-1F3FD-200D-1F9B2"},
    {"homem_careca_cor4", "1F468-1F3FE-200D-1F9B2"},
    {"homem_careca_cor5", "1F468-1F3FF-200D-1F9B2"},
    {"homem_cabelo_branco", "1F468-200D-1F9B3"},
    {"homem_cabelo_branco_cor1", "1F468-1F3FB-200D-1F9B3"},
    {"homem_cabelo_branco_cor2", "1F468-1F3FC-200D-1F9B3"},
    {"homem_cabelo_branco_cor3", "1F468-1F3FD-200D-1F9B3"},
    {"homem_cabelo_branco_cor4", "1F468-1F3FE-200D-1F9B3"},
    {"homem_cabelo_branco_cor5", "1F468-1F3FF-200D-1F9B3"},
    {"mulher_cabelo_vermelho", "1F469-200D-1F9B0"},
    {"mulher_cabelo_vermelho_cor1", "1F469-1F3FB-200D-1F9B0"},
    {"mulher_cabelo_vermelho_cor2", "1F469-1F3FC-200D-1F9B0"},
    {"mulher_cabelo_vermelho_cor3", "1F469-1F3FD-200D-1F9B0"},
    {"mulher_cabelo_vermelho_cor4", "1F469-1F3FE-200D-1F9B0"},
    {"mulher_cabelo_vermelho_cor5", "1F469-1F3FF-200D-1F9B0"},
    {"mulher_cabelo_cacheado", "1F469-200D-1F9B1"},
    {"mulher_cabelo_cacheado_cor1", "1F469-1F3FB-200D-1F9B1"},
    {"mulher_cabelo_cacheado_cor2", "1F469-1F3FC-200D-1F9B1"},
    {"mulher_cabelo_cacheado_cor3", "1F469-1F3FD-200D-1F9B1"},
    {"mulher_cabelo_cacheado_cor4", "1F469-1F3FE-200D-1F9B1"},
    {"mulher_cabelo_cacheado_cor5", "1F469-1F3FF-200D-1F9B1"},
    {"mulher_careca", "1F469-200D-1F9B2"},
    {"mulher_careca_cor1", "1F469-1F3FB-200D-1F9B2"},
    {"mulher_careca_cor2", "1F469-1F3FC-200D-1F9B2"},
    {"mulher_careca_cor3", "1F469-1F3FD-200D-1F9B2"},
    {"mulher_careca_cor4", "1F469-1F3FE-200D-1F9B2"},
    {"mulher_careca_cor5", "1F469-1F3FF-200D-1F9B2"},
    {"mulher_cabelo_branco", "1F469-200D-1F9B3"},
    {"mulher_cabelo_branco_cor1", "1F469-1F3FB-200D-1F9B3"},
    {"mulher_cabelo_branco_cor2", "1F469-1F3FC-200D-1F9B3"},
    {"mulher_cabelo_branco_cor3", "1F469-1F3FD-200D-1F9B3"},
    {"mulher_cabelo_branco_cor4", "1F469-1F3FE-200D-1F9B3"},
    {"mulher_cabelo_branco_cor5", "1F469-1F3FF-200D-1F9B3"},
    {"pessoa_cabelo_vermelho", "1F9D1-200D-1F9B0"},
    {"pessoa_cabelo_vermelho_cor1", "1F9D1-1F3FB-200D-1F9B0"},
    {"pessoa_cabelo_vermelho_cor2", "1F9D1-1F3FC-200D-1F9B0"},
    {"pessoa_cabelo_vermelho_cor3", "1F9D1-1F3FD-200D-1F9B0"},
    {"pessoa_cabelo_vermelho_cor4", "1F9D1-1F3FE-200D-1F9B0"},
    {"pessoa_cabelo_vermelho_cor5", "1F9D1-1F3FF-200D-1F9B0"},
    {"pessoa_cabelo_cacheado", "1F9D1-200D-1F9B1"},
    {"pessoa_cabelo_cacheado_cor1", "1F9D1-1F3FB-200D-1F9B1"},
    {"pessoa_cabelo_cacheado_cor2", "1F9D1-1F3FC-200D-1F9B1"},
    {"pessoa_cabelo_cacheado_cor3", "1F9D1-1F3FD-200D-1F9B1"},
    {"pessoa_cabelo_cacheado_cor4", "1F9D1-1F3FE-200D-1F9B1"},
    {"pessoa_cabelo_cacheado_cor5", "1F9D1-1F3FF-200D-1F9B1"},
    {"pessoa_careca", "1F9D1-200D-1F9B2"},
    {"pessoa_careca_cor1", "1F9D1-1F3FB-200D-1F9B2"},
    {"pessoa_careca_cor2", "1F9D1-1F3FC-200D-1F9B2"},
    {"pessoa_careca_cor3", "1F9D1-1F3FD-200D-1F9B2"},
    {"pessoa_careca_cor4", "1F9D1-1F3FE-200D-1F9B2"},
    {"pessoa_careca_cor5", "1F9D1-1F3FF-200D-1F9B2"},
    {"pessoa_cabelo_branco", "1F9D1-200D-1F9B3"},
    {"pessoa_cabelo_branco_cor1", "1F9D1-1F3FB-200D-1F9B3"},
    {"pessoa_cabelo_branco_cor2", "1F9D1-1F3FC-200D-1F9B3"},
    {"pessoa_cabelo_branco_cor3", "1F9D1-1F3FD-200D-1F9B3"},
    {"pessoa_cabelo_branco_cor4", "1F9D1-1F3FE-200D-1F9B3"},
    {"pessoa_cabelo_branco_cor5", "1F9D1-1F3FF-200D-1F9B3"},
    {"corrente_quebrada", "26D3-FE0F-200D-1F4A5"},
    {"coracao_em_chamas", "2764-FE0F-200D-1F525"},
    {"coracao_remendado", "2764-FE0F-200D-1FA79"},
    {"cogumelo_marrom", "1F344-200D-1F7EB"},
    {"lima", "1F34B-200D-1F7E9"},
    {"bandeira_transgenero", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"bandeira_do_arco_iris", "1F3F3-FE0F-200D-1F308"},
    {"bandeira_de_pirata", "1F3F4-200D-2620-FE0F"},
    {"gato_preto", "1F408-200D-2B1B"},
    {"cao_de_servico", "1F415-200D-1F9BA"},
    {"passaro_preto", "1F426-200D-2B1B"},
    {"fenix", "1F426-200D-1F525"},
    {"urso_polar", "1F43B-200D-2744-FE0F"},
    {"olho_no_balao_de_dialogo", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"rosto_exalando", "1F62E-200D-1F4A8"},
    {"rosto_com_olhos_em_espiral", "1F635-200D-1F4AB"},
    {"rosto_nas_nuvens", "1F636-200D-1F32B-FE0F"},
    {"cabeca_virando_de_um_lado_para_o_outro", "1F642-200D-2194-FE0F"},
    {"cabeca_balancando_na_vertical", "1F642-200D-2195-FE0F"},
    {"bailarina", "1F9D1-200D-1FA70"},
    {"bailarina_cor1", "1F9D1-1F3FB-200D-1FA70"},
    {"bailarina_cor2", "1F9D1-1F3FC-200D-1FA70"},
    {"bailarina_cor3", "1F9D1-1F3FD-200D-1FA70"},
    {"bailarina_cor4", "1F9D1-1F3FE-200D-1FA70"},
    {"bailarina_cor5", "1F9D1-1F3FF-200D-1FA70"},
    {"exclamação_com_interrogação", "2049"},
    {"símbolo_de_marca_registrada", "2122"},
    {"informações", "2139"},
    {"caixa_de_seleção_marcada_com_tique", "2611"},
    {"café", "2615"},
    {"risco_biológico", "2623"},
    {"símbolo_de_feminino", "2640"},
    {"símbolo_de_masculino", "2642"},
    {"signo_de_áries", "2648"},
    {"signo_de_sagitário", "2650"},
    {"signo_de_capricórnio", "2651"},
    {"signo_de_aquário", "2652"},
    {"âncora", "2693"},
    {"símbolo_da_medicina", "2695"},
    {"balança", "2696"},
    {"marca_de_seleção_branca", "2705"},
    {"avião", "2708"},
    {"marca_de_seleção", "2714"},
    {"sinal_de_multiplicação", "2716"},
    {"faísca", "2747"},
    {"ponto_de_interrogação_vermelho", "2753"},
    {"ponto_de_interrogação_branco", "2754"},
    {"ponto_de_exclamação_branco", "2755"},
    {"ponto_de_exclamação_vermelho", "2757"},
    {"exclamação_de_coração", "2763"},
    {"coração_vermelho", "2764"},
    {"símbolo_de_adição", "2795"},
    {"símbolo_de_subtração", "2796"},
    {"símbolo_de_divisão", "2797"},
    {"travessão_ondulado", "3030"},
    {"botão_japonês_de_parabéns", "3297"},
    {"botão_japonês_de_segredo", "3299"},
    {"símbolo_de_copyright", "00A9"},
    {"símbolo_de_registrado", "00AE"},
    {"dupla_exclamação", "203C"},
    {"relógio_de_pulso", "231A"},
    {"botão_ejetar", "23CF"},
    {"botão_avançar", "23E9"},
    {"botão_de_retroceder", "23EA"},
    {"botão_de_avanço_para_cima", "23EB"},
    {"botão_de_avanço_para_baixo", "23EC"},
    {"botão_de_próxima_faixa", "23ED"},
    {"botão_de_faixa_anterior", "23EE"},
    {"botão_de_reproduzir_ou_pausar", "23EF"},
    {"cronômetro", "23F1"},
    {"relógio_temporizador", "23F2"},
    {"botão_pausar", "23F8"},
    {"botão_parar", "23F9"},
    {"botão_gravar", "23FA"},
    {"círculo_com_a_letra_m", "24C2"},
    {"botão_reproduzir", "25B6"},
    {"botão_de_voltar", "25C0"},
    {"quadrado_branco_médio", "25FB"},
    {"quadrado_preto_médio", "25FC"},
    {"quadrado_branco_médio_menor", "25FD"},
    {"quadrado_preto_médio_menor", "25FE"},
    {"símbolo_da_paz", "262E"},
    {"signo_de_gêmeos", "264A"},
    {"signo_de_câncer", "264B"},
    {"signo_de_leão", "264C"},
    {"signo_de_escorpião", "264F"},
    {"peão_de_xadrez", "265F"},
    {"símbolo_de_reciclagem", "267B"},
    {"símbolo_de_cadeira_de_rodas", "267F"},
    {"símbolo_de_átomo", "269B"},
    {"alta_tensão", "26A1"},
    {"símbolo_transgênero", "26A7"},
    {"círculo_branco", "26AA"},
    {"círculo_preto", "26AB"},
    {"caixão", "26B0"},
    {"urna_funerária", "26B1"},
    {"sol_por_trás_das_nuvens", "26C5"},
    {"chuva_com_trovão", "26C8"},
    {"signo_de_ofiúco", "26CE"},
    {"santuário_japonês", "26E9"},
    {"mão_levantada", "270B"},
    {"mão_levantada_cor1", "270B-1F3FB"},
    {"mão_levantada_cor2", "270B-1F3FC"},
    {"mão_levantada_cor3", "270B-1F3FD"},
    {"mão_levantada_cor4", "270B-1F3FE"},
    {"mão_levantada_cor5", "270B-1F3FF"},
    {"mão_em_v_de_vitória", "270C"},
    {"mão_em_v_de_vitória_cor1", "270C-1F3FB"},
    {"mão_em_v_de_vitória_cor2", "270C-1F3FC"},
    {"mão_em_v_de_vitória_cor3", "270C-1F3FD"},
    {"mão_em_v_de_vitória_cor4", "270C-1F3FE"},
    {"mão_em_v_de_vitória_cor5", "270C-1F3FF"},
    {"escrevendo_à_mão", "270D"},
    {"escrevendo_à_mão_cor1", "270D-1F3FB"},
    {"escrevendo_à_mão_cor2", "270D-1F3FC"},
    {"escrevendo_à_mão_cor3", "270D-1F3FD"},
    {"escrevendo_à_mão_cor4", "270D-1F3FE"},
    {"escrevendo_à_mão_cor5", "270D-1F3FF"},
    {"lápis", "270F"},
    {"botão_de_xis", "274E"},
    {"laço_encaracolado", "27B0"},
    {"estrela_branca_média", "2B50"},
    {"círculo_grande_oco", "2B55"},
    {"sinal_japonês_indicando_início_de_música", "303D"},
    {"dragão_vermelho_de_mahjong", "1F004"},
    {"botão_a_tipo_sanguíneo", "1F170"},
    {"botão_b_tipo_sanguíneo", "1F171"},
    {"botão_o_tipo_sanguíneo", "1F17E"},
    {"botão_p", "1F17F"},
    {"botão_ab_tipo_sanguíneo", "1F18E"},
    {"botão_cl", "1F191"},
    {"botão_cool", "1F192"},
    {"botão_free", "1F193"},
    {"botão_id", "1F194"},
    {"botão_new", "1F195"},
    {"botão_ng", "1F196"},
    {"botão_ok", "1F197"},
    {"botão_sos", "1F198"},
    {"botão_up", "1F199"},
    {"botão_vs", "1F19A"},
    {"botão_japonês_de_aqui", "1F201"},
    {"botão_japonês_de_taxa_de_serviço", "1F202"},
    {"botão_japonês_de_gratuito", "1F21A"},
    {"botão_japonês_de_reservado", "1F22F"},
    {"botão_japonês_de_proibido", "1F232"},
    {"botão_japonês_de_vago", "1F233"},
    {"botão_japonês_de_nota_mínima", "1F234"},
    {"botão_japonês_de_sem_vagas", "1F235"},
    {"botão_japonês_de_não_gratuito", "1F236"},
    {"botão_japonês_de_quantidade_mensal", "1F237"},
    {"botão_japonês_de_aplicação", "1F238"},
    {"botão_japonês_de_desconto", "1F239"},
    {"botão_japonês_de_aberto_para_negócios", "1F23A"},
    {"botão_japonês_de_barganha", "1F250"},
    {"botão_japonês_de_aceitável", "1F251"},
    {"aurora_sobre_água", "1F305"},
    {"pôr_do_sol", "1F307"},
    {"arco_íris", "1F308"},
    {"ponte_à_noite", "1F309"},
    {"vulcão", "1F30B"},
    {"via_láctea", "1F30C"},
    {"globo_mostrando_europa_e_áfrica", "1F30D"},
    {"globo_mostrando_as_américas", "1F30E"},
    {"globo_mostrando_ásia_e_oceania", "1F30F"},
    {"lua_crescente_côncava", "1F312"},
    {"lua_minguante_côncava", "1F318"},
    {"termômetro", "1F321"},
    {"nuvem_com_trovão", "1F329"},
    {"conífera", "1F332"},
    {"árvore_caidiça", "1F333"},
    {"folhas_caídas", "1F342"},
    {"melão", "1F348"},
    {"limão", "1F34B"},
    {"maçã_vermelha", "1F34E"},
    {"maçã_verde", "1F34F"},
    {"pêssego", "1F351"},
    {"hambúrguer", "1F354"},
    {"arroz_japonês", "1F359"},
    {"lámen", "1F35C"},
    {"pão", "1F35E"},
    {"camarão_frito", "1F364"},
    {"pão_de_ló_de_morango", "1F370"},
    {"bentô", "1F371"},
    {"xícara_de_chá_sem_alça", "1F375"},
    {"saquê", "1F376"},
    {"laço_de_fita", "1F380"},
    {"bolo_de_aniversário", "1F382"},
    {"abóbora_de_halloween", "1F383"},
    {"árvore_de_natal", "1F384"},
    {"fogos_de_artifício", "1F386"},
    {"balão", "1F388"},
    {"árvore_de_tanabata", "1F38B"},
    {"decoração_de_pinhos", "1F38D"},
    {"carrilhão_de_vento", "1F390"},
    {"contemplação_da_lua", "1F391"},
    {"chapéu_de_formatura", "1F393"},
    {"microfone_de_estúdio", "1F399"},
    {"botões_giratórios", "1F39B"},
    {"câmera_de_cinema", "1F3A5"},
    {"máscara", "1F3AD"},
    {"caça_níquel", "1F3B0"},
    {"tênis", "1F3BE"},
    {"troféu", "1F3C6"},
    {"críquete", "1F3CF"},
    {"bola_de_vôlei", "1F3D0"},
    {"hóquei_de_campo", "1F3D1"},
    {"hóquei_no_gelo", "1F3D2"},
    {"construção", "1F3D7"},
    {"prédio_grego", "1F3DB"},
    {"estádio", "1F3DF"},
    {"edifício_comercial", "1F3E2"},
    {"correio_japonês", "1F3E3"},
    {"símbolo_de_caixa_automático", "1F3E7"},
    {"loja_de_conveniência", "1F3EA"},
    {"fábrica", "1F3ED"},
    {"castelo_japonês", "1F3EF"},
    {"ânfora", "1F3FA"},
    {"búfalo_asiático", "1F403"},
    {"dragão", "1F409"},
    {"pássaro", "1F426"},
    {"rosto_de_dragão", "1F432"},
    {"baleia_esguichando_água", "1F433"},
    {"língua", "1F445"},
    {"dorso_da_mão_com_indicador_apontando_para_cima", "1F446"},
    {"dorso_da_mão_com_indicador_apontando_para_cima_cor1", "1F446-1F3FB"},
    {"dorso_da_mão_com_indicador_apontando_para_cima_cor2", "1F446-1F3FC"},
    {"dorso_da_mão_com_indicador_apontando_para_cima_cor3", "1F446-1F3FD"},
    {"dorso_da_mão_com_indicador_apontando_para_cima_cor4", "1F446-1F3FE"},
    {"dorso_da_mão_com_indicador_apontando_para_cima_cor5", "1F446-1F3FF"},
    {"dorso_da_mão_com_indicador_apontando_para_baixo", "1F447"},
    {"dorso_da_mão_com_indicador_apontando_para_baixo_cor1", "1F447-1F3FB"},
    {"dorso_da_mão_com_indicador_apontando_para_baixo_cor2", "1F447-1F3FC"},
    {"dorso_da_mão_com_indicador_apontando_para_baixo_cor3", "1F447-1F3FD"},
    {"dorso_da_mão_com_indicador_apontando_para_baixo_cor4", "1F447-1F3FE"},
    {"dorso_da_mão_com_indicador_apontando_para_baixo_cor5", "1F447-1F3FF"},
    {"dorso_da_mão_com_indicador_apontando_para_a_esquerda", "1F448"},
    {"dorso_da_mão_com_indicador_apontando_para_a_esquerda_cor1", "1F448-1F3FB"},
    {"dorso_da_mão_com_indicador_apontando_para_a_esquerda_cor2", "1F448-1F3FC"},
    {"dorso_da_mão_com_indicador_apontando_para_a_esquerda_cor3", "1F448-1F3FD"},
    {"dorso_da_mão_com_indicador_apontando_para_a_esquerda_cor4", "1F448-1F3FE"},
    {"dorso_da_mão_com_indicador_apontando_para_a_esquerda_cor5", "1F448-1F3FF"},
    {"dorso_da_mão_com_indicador_apontando_para_a_direita", "1F449"},
    {"dorso_da_mão_com_indicador_apontando_para_a_direita_cor1", "1F449-1F3FB"},
    {"dorso_da_mão_com_indicador_apontando_para_a_direita_cor2", "1F449-1F3FC"},
    {"dorso_da_mão_com_indicador_apontando_para_a_direita_cor3", "1F449-1F3FD"},
    {"dorso_da_mão_com_indicador_apontando_para_a_direita_cor4", "1F449-1F3FE"},
    {"dorso_da_mão_com_indicador_apontando_para_a_direita_cor5", "1F449-1F3FF"},
    {"mão_acenando", "1F44B"},
    {"mão_acenando_cor1", "1F44B-1F3FB"},
    {"mão_acenando_cor2", "1F44B-1F3FC"},
    {"mão_acenando_cor3", "1F44B-1F3FD"},
    {"mão_acenando_cor4", "1F44B-1F3FE"},
    {"mão_acenando_cor5", "1F44B-1F3FF"},
    {"mãos_aplaudindo", "1F44F"},
    {"mãos_aplaudindo_cor1", "1F44F-1F3FB"},
    {"mãos_aplaudindo_cor2", "1F44F-1F3FC"},
    {"mãos_aplaudindo_cor3", "1F44F-1F3FD"},
    {"mãos_aplaudindo_cor4", "1F44F-1F3FE"},
    {"mãos_aplaudindo_cor5", "1F44F-1F3FF"},
    {"mãos_abertas", "1F450"},
    {"mãos_abertas_cor1", "1F450-1F3FB"},
    {"mãos_abertas_cor2", "1F450-1F3FC"},
    {"mãos_abertas_cor3", "1F450-1F3FD"},
    {"mãos_abertas_cor4", "1F450-1F3FE"},
    {"mãos_abertas_cor5", "1F450-1F3FF"},
    {"chapéu_feminino", "1F452"},
    {"óculos", "1F453"},
    {"biquíni", "1F459"},
    {"bolsa_de_mão", "1F45C"},
    {"tênis_de_corrida", "1F45F"},
    {"sandália_feminina", "1F461"},
    {"família", "1F46A"},
    {"homem_e_mulher_de_mãos_dadas", "1F46B"},
    {"homem_e_mulher_de_mãos_dadas_cor1", "1F46B-1F3FB"},
    {"homem_e_mulher_de_mãos_dadas_cor2", "1F46B-1F3FC"},
    {"homem_e_mulher_de_mãos_dadas_cor3", "1F46B-1F3FD"},
    {"homem_e_mulher_de_mãos_dadas_cor4", "1F46B-1F3FE"},
    {"homem_e_mulher_de_mãos_dadas_cor5", "1F46B-1F3FF"},
    {"homem_e_mulher_de_mãos_dadas_cor1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"homem_e_mulher_de_mãos_dadas_cor1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"homem_e_mulher_de_mãos_dadas_cor1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"homem_e_mulher_de_mãos_dadas_cor1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"homem_e_mulher_de_mãos_dadas_cor2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"homem_e_mulher_de_mãos_dadas_cor2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"homem_e_mulher_de_mãos_dadas_cor2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"homem_e_mulher_de_mãos_dadas_cor2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"homem_e_mulher_de_mãos_dadas_cor3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"homem_e_mulher_de_mãos_dadas_cor3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"homem_e_mulher_de_mãos_dadas_cor3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"homem_e_mulher_de_mãos_dadas_cor3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"homem_e_mulher_de_mãos_dadas_cor4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"homem_e_mulher_de_mãos_dadas_cor4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"homem_e_mulher_de_mãos_dadas_cor4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"homem_e_mulher_de_mãos_dadas_cor4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"homem_e_mulher_de_mãos_dadas_cor5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"homem_e_mulher_de_mãos_dadas_cor5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"homem_e_mulher_de_mãos_dadas_cor5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"homem_e_mulher_de_mãos_dadas_cor5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"dois_homens_de_mãos_dadas", "1F46C"},
    {"dois_homens_de_mãos_dadas_cor1", "1F46C-1F3FB"},
    {"dois_homens_de_mãos_dadas_cor2", "1F46C-1F3FC"},
    {"dois_homens_de_mãos_dadas_cor3", "1F46C-1F3FD"},
    {"dois_homens_de_mãos_dadas_cor4", "1F46C-1F3FE"},
    {"dois_homens_de_mãos_dadas_cor5", "1F46C-1F3FF"},
    {"dois_homens_de_mãos_dadas_cor1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"dois_homens_de_mãos_dadas_cor1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"dois_homens_de_mãos_dadas_cor1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"dois_homens_de_mãos_dadas_cor1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"dois_homens_de_mãos_dadas_cor2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"dois_homens_de_mãos_dadas_cor2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"dois_homens_de_mãos_dadas_cor2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"dois_homens_de_mãos_dadas_cor2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"dois_homens_de_mãos_dadas_cor3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"dois_homens_de_mãos_dadas_cor3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"dois_homens_de_mãos_dadas_cor3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"dois_homens_de_mãos_dadas_cor3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"dois_homens_de_mãos_dadas_cor4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"dois_homens_de_mãos_dadas_cor4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"dois_homens_de_mãos_dadas_cor4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"dois_homens_de_mãos_dadas_cor4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"dois_homens_de_mãos_dadas_cor5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"dois_homens_de_mãos_dadas_cor5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"dois_homens_de_mãos_dadas_cor5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"dois_homens_de_mãos_dadas_cor5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"duas_mulheres_de_mãos_dadas", "1F46D"},
    {"duas_mulheres_de_mãos_dadas_cor1", "1F46D-1F3FB"},
    {"duas_mulheres_de_mãos_dadas_cor2", "1F46D-1F3FC"},
    {"duas_mulheres_de_mãos_dadas_cor3", "1F46D-1F3FD"},
    {"duas_mulheres_de_mãos_dadas_cor4", "1F46D-1F3FE"},
    {"duas_mulheres_de_mãos_dadas_cor5", "1F46D-1F3FF"},
    {"duas_mulheres_de_mãos_dadas_cor1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"duas_mulheres_de_mãos_dadas_cor1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"duas_mulheres_de_mãos_dadas_cor1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"duas_mulheres_de_mãos_dadas_cor1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"duas_mulheres_de_mãos_dadas_cor2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"duas_mulheres_de_mãos_dadas_cor2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"duas_mulheres_de_mãos_dadas_cor2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"duas_mulheres_de_mãos_dadas_cor2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"duas_mulheres_de_mãos_dadas_cor3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"duas_mulheres_de_mãos_dadas_cor3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"duas_mulheres_de_mãos_dadas_cor3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"duas_mulheres_de_mãos_dadas_cor3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"duas_mulheres_de_mãos_dadas_cor4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"duas_mulheres_de_mãos_dadas_cor4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"duas_mulheres_de_mãos_dadas_cor4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"duas_mulheres_de_mãos_dadas_cor4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"duas_mulheres_de_mãos_dadas_cor5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"duas_mulheres_de_mãos_dadas_cor5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"duas_mulheres_de_mãos_dadas_cor5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"duas_mulheres_de_mãos_dadas_cor5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"pessoa_com_véu", "1F470"},
    {"pessoa_com_véu_cor1", "1F470-1F3FB"},
    {"pessoa_com_véu_cor2", "1F470-1F3FC"},
    {"pessoa_com_véu_cor3", "1F470-1F3FD"},
    {"pessoa_com_véu_cor4", "1F470-1F3FE"},
    {"pessoa_com_véu_cor5", "1F470-1F3FF"},
    {"homem_de_boné", "1F472"},
    {"homem_de_boné_cor1", "1F472-1F3FB"},
    {"homem_de_boné_cor2", "1F472-1F3FC"},
    {"homem_de_boné_cor3", "1F472-1F3FD"},
    {"homem_de_boné_cor4", "1F472-1F3FE"},
    {"homem_de_boné_cor5", "1F472-1F3FF"},
    {"bebê", "1F476"},
    {"bebê_cor1", "1F476-1F3FB"},
    {"bebê_cor2", "1F476-1F3FC"},
    {"bebê_cor3", "1F476-1F3FD"},
    {"bebê_cor4", "1F476-1F3FE"},
    {"bebê_cor5", "1F476-1F3FF"},
    {"trabalhador_de_construção_civil", "1F477"},
    {"trabalhador_de_construção_civil_cor1", "1F477-1F3FB"},
    {"trabalhador_de_construção_civil_cor2", "1F477-1F3FC"},
    {"trabalhador_de_construção_civil_cor3", "1F477-1F3FD"},
    {"trabalhador_de_construção_civil_cor4", "1F477-1F3FE"},
    {"trabalhador_de_construção_civil_cor5", "1F477-1F3FF"},
    {"duende_japonês", "1F47A"},
    {"bebê_anjo", "1F47C"},
    {"bebê_anjo_cor1", "1F47C-1F3FB"},
    {"bebê_anjo_cor2", "1F47C-1F3FC"},
    {"bebê_anjo_cor3", "1F47C-1F3FD"},
    {"bebê_anjo_cor4", "1F47C-1F3FE"},
    {"bebê_anjo_cor5", "1F47C-1F3FF"},
    {"alienígena", "1F47D"},
    {"monstro_alienígena", "1F47E"},
    {"mulher_dançando", "1F483"},
    {"mulher_dançando_cor1", "1F483-1F3FB"},
    {"mulher_dançando_cor2", "1F483-1F3FC"},
    {"mulher_dançando_cor3", "1F483-1F3FD"},
    {"mulher_dançando_cor4", "1F483-1F3FE"},
    {"mulher_dançando_cor5", "1F483-1F3FF"},
    {"buquê", "1F490"},
    {"coração_pulsante", "1F493"},
    {"coração_partido", "1F494"},
    {"dois_corações", "1F495"},
    {"coração_brilhante", "1F496"},
    {"coração_crescendo", "1F497"},
    {"coração_com_flecha", "1F498"},
    {"coração_azul", "1F499"},
    {"coração_verde", "1F49A"},
    {"coração_amarelo", "1F49B"},
    {"coração_roxo", "1F49C"},
    {"coração_com_fita", "1F49D"},
    {"corações_girando", "1F49E"},
    {"coração_decorativo", "1F49F"},
    {"lâmpada", "1F4A1"},
    {"símbolo_de_raiva", "1F4A2"},
    {"colisão", "1F4A5"},
    {"cocô", "1F4A9"},
    {"bíceps", "1F4AA"},
    {"bíceps_cor1", "1F4AA-1F3FB"},
    {"bíceps_cor2", "1F4AA-1F3FC"},
    {"bíceps_cor3", "1F4AA-1F3FD"},
    {"bíceps_cor4", "1F4AA-1F3FE"},
    {"bíceps_cor5", "1F4AA-1F3FF"},
    {"balão_de_diálogo", "1F4AC"},
    {"balão_de_pensamento", "1F4AD"},
    {"câmbio_de_moeda", "1F4B1"},
    {"cifrão", "1F4B2"},
    {"cartão_de_crédito", "1F4B3"},
    {"nota_de_dólar", "1F4B5"},
    {"gráfico_subindo_com_iene", "1F4B9"},
    {"página_dobrada", "1F4C3"},
    {"página_voltada_para_cima", "1F4C4"},
    {"calendário", "1F4C5"},
    {"calendário_com_folhas_destacáveis", "1F4C6"},
    {"índice_de_cartões", "1F4C7"},
    {"gráfico_subindo", "1F4C8"},
    {"gráfico_caindo", "1F4C9"},
    {"gráfico_de_barras", "1F4CA"},
    {"régua_reta", "1F4CF"},
    {"régua_triangular", "1F4D0"},
    {"marcadores_de_página", "1F4D1"},
    {"livro_contábil", "1F4D2"},
    {"crachá", "1F4DB"},
    {"antena_parabólica", "1F4E1"},
    {"bandeja_de_saída", "1F4E4"},
    {"modo_vibratório", "1F4F3"},
    {"câmera", "1F4F7"},
    {"câmera_com_flash", "1F4F8"},
    {"câmera_de_vídeo", "1F4F9"},
    {"televisão", "1F4FA"},
    {"rádio", "1F4FB"},
    {"rosário_de_oração", "1F4FF"},
    {"botão_de_músicas_aleatórias", "1F500"},
    {"botão_de_repetir", "1F501"},
    {"botão_de_repetir_uma_única_faixa", "1F502"},
    {"setas_verticais_no_sentido_horário", "1F503"},
    {"botão_de_setas_em_sentido_anti_horário", "1F504"},
    {"botão_de_diminuir_brilho", "1F505"},
    {"botão_de_aumentar_brilho", "1F506"},
    {"alto_falante_com_volume_médio", "1F509"},
    {"tomada_elétrica", "1F50C"},
    {"marcador_de_página", "1F516"},
    {"botão_de_opção", "1F518"},
    {"letras_latinas_maiúsculas", "1F520"},
    {"letras_latinas_minúsculas", "1F521"},
    {"números", "1F522"},
    {"símbolos", "1F523"},
    {"pistola_dágua", "1F52B"},
    {"microscópio", "1F52C"},
    {"telescópio", "1F52D"},
    {"símbolo_japonês_de_principiante", "1F530"},
    {"botão_quadrado_preto", "1F532"},
    {"botão_quadrado_branco", "1F533"},
    {"círculo_vermelho", "1F534"},
    {"círculo_azul", "1F535"},
    {"triângulo_vermelho_para_cima", "1F53A"},
    {"triângulo_vermelho_para_baixo", "1F53B"},
    {"botão_apontando_para_cima", "1F53C"},
    {"botão_apontando_para_baixo", "1F53D"},
    {"caaba_islã", "1F54B"},
    {"menorá", "1F54E"},
    {"três_e_meia", "1F55E"},
    {"relógio_de_mesa", "1F570"},
    {"óculos_escuros", "1F576"},
    {"homem_dançando", "1F57A"},
    {"homem_dançando_cor1", "1F57A-1F3FB"},
    {"homem_dançando_cor2", "1F57A-1F3FC"},
    {"homem_dançando_cor3", "1F57A-1F3FD"},
    {"homem_dançando_cor4", "1F57A-1F3FE"},
    {"homem_dançando_cor5", "1F57A-1F3FF"},
    {"mão_aberta_com_os_dedos_separados", "1F590"},
    {"mão_aberta_com_os_dedos_separados_cor1", "1F590-1F3FB"},
    {"mão_aberta_com_os_dedos_separados_cor2", "1F590-1F3FC"},
    {"mão_aberta_com_os_dedos_separados_cor3", "1F590-1F3FD"},
    {"mão_aberta_com_os_dedos_separados_cor4", "1F590-1F3FE"},
    {"mão_aberta_com_os_dedos_separados_cor5", "1F590-1F3FF"},
    {"saudação_vulcana", "1F596"},
    {"saudação_vulcana_cor1", "1F596-1F3FB"},
    {"saudação_vulcana_cor2", "1F596-1F3FC"},
    {"saudação_vulcana_cor3", "1F596-1F3FD"},
    {"saudação_vulcana_cor4", "1F596-1F3FE"},
    {"saudação_vulcana_cor5", "1F596-1F3FF"},
    {"coração_preto", "1F5A4"},
    {"gavetas_de_escritório", "1F5C4"},
    {"calendário_espiral", "1F5D3"},
    {"braçadeira", "1F5DC"},
    {"balão_de_diálogo_à_esquerda", "1F5E8"},
    {"balão_de_raiva_à_direita", "1F5EF"},
    {"urna_eleitoral_com_cédula", "1F5F3"},
    {"mapa_múndi", "1F5FA"},
    {"torre_de_tóquio", "1F5FC"},
    {"estátua_da_liberdade", "1F5FD"},
    {"mapa_do_japão", "1F5FE"},
    {"rosto_sorridente_com_auréola", "1F607"},
    {"rosto_sorridente_com_olhos_de_coração", "1F60D"},
    {"rosto_sorridente_com_óculos_escuros", "1F60E"},
    {"rosto_mostrando_a_língua", "1F61B"},
    {"rosto_piscando_e_com_língua_para_fora", "1F61C"},
    {"rosto_com_olhos_semicerrados_e_língua_para_fora", "1F61D"},
    {"rosto_com_máscara_médica", "1F637"},
    {"rosto_de_gato_com_lágrimas_de_alegria", "1F639"},
    {"rosto_de_gato_sorridente_com_olhos_de_coração", "1F63B"},
    {"rosto_de_gato_com_sorriso_irônico", "1F63C"},
    {"rosto_de_cabeça_para_baixo", "1F643"},
    {"pessoa_fazendo_gesto_de_não", "1F645"},
    {"pessoa_fazendo_gesto_de_não_cor1", "1F645-1F3FB"},
    {"pessoa_fazendo_gesto_de_não_cor2", "1F645-1F3FC"},
    {"pessoa_fazendo_gesto_de_não_cor3", "1F645-1F3FD"},
    {"pessoa_fazendo_gesto_de_não_cor4", "1F645-1F3FE"},
    {"pessoa_fazendo_gesto_de_não_cor5", "1F645-1F3FF"},
    {"pessoa_fazendo_reverência", "1F647"},
    {"pessoa_fazendo_reverência_cor1", "1F647-1F3FB"},
    {"pessoa_fazendo_reverência_cor2", "1F647-1F3FC"},
    {"pessoa_fazendo_reverência_cor3", "1F647-1F3FD"},
    {"pessoa_fazendo_reverência_cor4", "1F647-1F3FE"},
    {"pessoa_fazendo_reverência_cor5", "1F647-1F3FF"},
    {"macaco_que_não_vê_nada", "1F648"},
    {"macaco_que_não_ouve_nada", "1F649"},
    {"macaco_que_não_fala_nada", "1F64A"},
    {"pessoa_levantando_a_mão", "1F64B"},
    {"pessoa_levantando_a_mão_cor1", "1F64B-1F3FB"},
    {"pessoa_levantando_a_mão_cor2", "1F64B-1F3FC"},
    {"pessoa_levantando_a_mão_cor3", "1F64B-1F3FD"},
    {"pessoa_levantando_a_mão_cor4", "1F64B-1F3FE"},
    {"pessoa_levantando_a_mão_cor5", "1F64B-1F3FF"},
    {"mãos_para_cima", "1F64C"},
    {"mãos_para_cima_cor1", "1F64C-1F3FB"},
    {"mãos_para_cima_cor2", "1F64C-1F3FC"},
    {"mãos_para_cima_cor3", "1F64C-1F3FD"},
    {"mãos_para_cima_cor4", "1F64C-1F3FE"},
    {"mãos_para_cima_cor5", "1F64C-1F3FF"},
    {"mãos_juntas", "1F64F"},
    {"mãos_juntas_cor1", "1F64F-1F3FB"},
    {"mãos_juntas_cor2", "1F64F-1F3FC"},
    {"mãos_juntas_cor3", "1F64F-1F3FD"},
    {"mãos_juntas_cor4", "1F64F-1F3FE"},
    {"mãos_juntas_cor5", "1F64F-1F3FF"},
    {"helicóptero", "1F681"},
    {"vagão_de_trem", "1F683"},
    {"trem_de_alta_velocidade_japonês", "1F685"},
    {"metrô", "1F687"},
    {"estação", "1F689"},
    {"bonde_elétrico", "1F68A"},
    {"ônibus", "1F68C"},
    {"ônibus_se_aproximando", "1F68D"},
    {"trólebus", "1F68E"},
    {"ponto_de_ônibus", "1F68F"},
    {"ambulância", "1F691"},
    {"táxi", "1F695"},
    {"táxi_se_aproximando", "1F696"},
    {"caminhão_de_entrega", "1F69A"},
    {"caminhão", "1F69B"},
    {"teleférico_de_montanha", "1F6A0"},
    {"teleférico", "1F6A1"},
    {"semáforo_horizontal", "1F6A5"},
    {"semáforo_vertical", "1F6A6"},
    {"em_construção", "1F6A7"},
    {"símbolo_de_lixeira", "1F6AE"},
    {"proibido_jogar_lixo_no_chão", "1F6AF"},
    {"água_potável", "1F6B0"},
    {"água_não_potável", "1F6B1"},
    {"crianças_atravessando", "1F6B8"},
    {"símbolo_de_bebê", "1F6BC"},
    {"vaso_sanitário", "1F6BD"},
    {"alfândega", "1F6C3"},
    {"restituição_de_bagagem", "1F6C4"},
    {"depósito_de_bagagem", "1F6C5"},
    {"sofá_e_luminária", "1F6CB"},
    {"barril_de_óleo", "1F6E2"},
    {"avião_pequeno", "1F6E9"},
    {"avião_decolando", "1F6EB"},
    {"avião_aterrissando", "1F6EC"},
    {"satélite", "1F6F0"},
    {"trenó", "1F6F7"},
    {"automóvel_riquixá", "1F6FA"},
    {"círculo_laranja", "1F7E0"},
    {"círculo_amarelo", "1F7E1"},
    {"círculo_verde", "1F7E2"},
    {"círculo_roxo", "1F7E3"},
    {"círculo_marrom", "1F7E4"},
    {"coração_branco", "1F90D"},
    {"coração_marrom", "1F90E"},
    {"mão_beliscando", "1F90F"},
    {"mão_beliscando_cor1", "1F90F-1F3FB"},
    {"mão_beliscando_cor2", "1F90F-1F3FC"},
    {"mão_beliscando_cor3", "1F90F-1F3FD"},
    {"mão_beliscando_cor4", "1F90F-1F3FE"},
    {"mão_beliscando_cor5", "1F90F-1F3FF"},
    {"rosto_com_boca_de_zíper", "1F910"},
    {"rosto_com_cifrões", "1F911"},
    {"rosto_com_termômetro", "1F912"},
    {"rosto_com_atadura_na_cabeça", "1F915"},
    {"rosto_de_robô", "1F916"},
    {"rosto_abraçando", "1F917"},
    {"saudação_do_rock", "1F918"},
    {"saudação_do_rock_cor1", "1F918-1F3FB"},
    {"saudação_do_rock_cor2", "1F918-1F3FC"},
    {"saudação_do_rock_cor3", "1F918-1F3FD"},
    {"saudação_do_rock_cor4", "1F918-1F3FE"},
    {"saudação_do_rock_cor5", "1F918-1F3FF"},
    {"dorso_da_mão_levantado", "1F91A"},
    {"dorso_da_mão_levantado_cor1", "1F91A-1F3FB"},
    {"dorso_da_mão_levantado_cor2", "1F91A-1F3FC"},
    {"dorso_da_mão_levantado_cor3", "1F91A-1F3FD"},
    {"dorso_da_mão_levantado_cor4", "1F91A-1F3FE"},
    {"dorso_da_mão_levantado_cor5", "1F91A-1F3FF"},
    {"aperto_de_mãos", "1F91D"},
    {"aperto_de_mãos_cor1", "1F91D-1F3FB"},
    {"aperto_de_mãos_cor2", "1F91D-1F3FC"},
    {"aperto_de_mãos_cor3", "1F91D-1F3FD"},
    {"aperto_de_mãos_cor4", "1F91D-1F3FE"},
    {"aperto_de_mãos_cor5", "1F91D-1F3FF"},
    {"aperto_de_mãos_cor1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"aperto_de_mãos_cor1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"aperto_de_mãos_cor1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"aperto_de_mãos_cor1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"aperto_de_mãos_cor2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"aperto_de_mãos_cor2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"aperto_de_mãos_cor2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"aperto_de_mãos_cor2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"aperto_de_mãos_cor3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"aperto_de_mãos_cor3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"aperto_de_mãos_cor3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"aperto_de_mãos_cor3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"aperto_de_mãos_cor4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"aperto_de_mãos_cor4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"aperto_de_mãos_cor4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"aperto_de_mãos_cor4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"aperto_de_mãos_cor5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"aperto_de_mãos_cor5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"aperto_de_mãos_cor5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"aperto_de_mãos_cor5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"rosto_com_chapéu_de_caubói", "1F920"},
    {"rosto_de_palhaço", "1F921"},
    {"rolando_no_chão_de_rir", "1F923"},
    {"rosto_fazendo_sinal_de_silêncio", "1F92B"},
    {"rosto_com_símbolos_na_boca", "1F92C"},
    {"rosto_com_a_mão_sobre_a_boca", "1F92D"},
    {"cabeça_explodindo", "1F92F"},
    {"grávida", "1F930"},
    {"grávida_cor1", "1F930-1F3FB"},
    {"grávida_cor2", "1F930-1F3FC"},
    {"grávida_cor3", "1F930-1F3FD"},
    {"grávida_cor4", "1F930-1F3FE"},
    {"grávida_cor5", "1F930-1F3FF"},
    {"príncipe", "1F934"},
    {"príncipe_cor1", "1F934-1F3FB"},
    {"príncipe_cor2", "1F934-1F3FC"},
    {"príncipe_cor3", "1F934-1F3FD"},
    {"príncipe_cor4", "1F934-1F3FE"},
    {"príncipe_cor5", "1F934-1F3FF"},
    {"mamãe_noel", "1F936"},
    {"mamãe_noel_cor1", "1F936-1F3FB"},
    {"mamãe_noel_cor2", "1F936-1F3FC"},
    {"mamãe_noel_cor3", "1F936-1F3FD"},
    {"mamãe_noel_cor4", "1F936-1F3FE"},
    {"mamãe_noel_cor5", "1F936-1F3FF"},
    {"pessoa_jogando_polo_aquático", "1F93D"},
    {"pessoa_jogando_polo_aquático_cor1", "1F93D-1F3FB"},
    {"pessoa_jogando_polo_aquático_cor2", "1F93D-1F3FC"},
    {"pessoa_jogando_polo_aquático_cor3", "1F93D-1F3FD"},
    {"pessoa_jogando_polo_aquático_cor4", "1F93D-1F3FE"},
    {"pessoa_jogando_polo_aquático_cor5", "1F93D-1F3FF"},
    {"máscara_de_mergulho", "1F93F"},
    {"taças_brindando", "1F942"},
    {"caçarola_com_comida", "1F958"},
    {"pão_recheado", "1F959"},
    {"bolinho_asiático", "1F95F"},
    {"brócolis", "1F966"},
    {"sanduíche", "1F96A"},
    {"rosto_sorridente_com_3_corações", "1F970"},
    {"rosto_sorridente_com_lágrima", "1F972"},
    {"rosto_disfarçado", "1F978"},
    {"rosto_segurando_as_lágrimas", "1F979"},
    {"óculos_de_proteção", "1F97D"},
    {"rosto_de_leão", "1F981"},
    {"escorpião", "1F982"},
    {"rosto_de_unicórnio", "1F984"},
    {"águia", "1F985"},
    {"tubarão", "1F988"},
    {"camarão", "1F990"},
    {"saurópode", "1F995"},
    {"pavão", "1F99A"},
    {"hipopótamo", "1F99B"},
    {"micróbio", "1F9A0"},
    {"dodô", "1F9A4"},
    {"preguiça", "1F9A5"},
    {"gambá", "1F9A8"},
    {"bisão", "1F9AC"},
    {"cão_guia", "1F9AE"},
    {"pé", "1F9B6"},
    {"pé_cor1", "1F9B6-1F3FB"},
    {"pé_cor2", "1F9B6-1F3FC"},
    {"pé_cor3", "1F9B6-1F3FD"},
    {"pé_cor4", "1F9B6-1F3FE"},
    {"pé_cor5", "1F9B6-1F3FF"},
    {"super_herói", "1F9B8"},
    {"super_herói_cor1", "1F9B8-1F3FB"},
    {"super_herói_cor2", "1F9B8-1F3FC"},
    {"super_herói_cor3", "1F9B8-1F3FD"},
    {"super_herói_cor4", "1F9B8-1F3FE"},
    {"super_herói_cor5", "1F9B8-1F3FF"},
    {"supervilão", "1F9B9"},
    {"supervilão_cor1", "1F9B9-1F3FB"},
    {"supervilão_cor2", "1F9B9-1F3FC"},
    {"supervilão_cor3", "1F9B9-1F3FD"},
    {"supervilão_cor4", "1F9B9-1F3FE"},
    {"supervilão_cor5", "1F9B9-1F3FF"},
    {"braço_mecânico", "1F9BE"},
    {"perna_mecânica", "1F9BF"},
    {"chá_perolado", "1F9CB"},
    {"pessoa_em_pé", "1F9CD"},
    {"pessoa_em_pé_cor1", "1F9CD-1F3FB"},
    {"pessoa_em_pé_cor2", "1F9CD-1F3FC"},
    {"pessoa_em_pé_cor3", "1F9CD-1F3FD"},
    {"pessoa_em_pé_cor4", "1F9CD-1F3FE"},
    {"pessoa_em_pé_cor5", "1F9CD-1F3FF"},
    {"rosto_com_monóculo", "1F9D0"},
    {"criança", "1F9D2"},
    {"criança_cor1", "1F9D2-1F3FB"},
    {"criança_cor2", "1F9D2-1F3FC"},
    {"criança_cor3", "1F9D2-1F3FD"},
    {"criança_cor4", "1F9D2-1F3FE"},
    {"criança_cor5", "1F9D2-1F3FF"},
    {"mulher_com_véu", "1F9D5"},
    {"mulher_com_véu_cor1", "1F9D5-1F3FB"},
    {"mulher_com_véu_cor2", "1F9D5-1F3FC"},
    {"mulher_com_véu_cor3", "1F9D5-1F3FD"},
    {"mulher_com_véu_cor4", "1F9D5-1F3FE"},
    {"mulher_com_véu_cor5", "1F9D5-1F3FF"},
    {"pessoa_na_posição_de_lótus", "1F9D8"},
    {"pessoa_na_posição_de_lótus_cor1", "1F9D8-1F3FB"},
    {"pessoa_na_posição_de_lótus_cor2", "1F9D8-1F3FC"},
    {"pessoa_na_posição_de_lótus_cor3", "1F9D8-1F3FD"},
    {"pessoa_na_posição_de_lótus_cor4", "1F9D8-1F3FE"},
    {"pessoa_na_posição_de_lótus_cor5", "1F9D8-1F3FF"},
    {"gênio", "1F9DE"},
    {"cérebro", "1F9E0"},
    {"coração_laranja", "1F9E1"},
    {"boné", "1F9E2"},
    {"quebra_cabeça", "1F9E9"},
    {"bússola", "1F9ED"},
    {"ábaco", "1F9EE"},
    {"extintor_de_incêndio", "1F9EF"},
    {"ímã", "1F9F2"},
    {"frasco_de_loção", "1F9F4"},
    {"alfinete_de_segurança", "1F9F7"},
    {"ursinho_de_pelúcia", "1F9F8"},
    {"sapatilha_de_balé", "1FA70"},
    {"maiô", "1FA71"},
    {"coração_azul_claro", "1FA75"},
    {"coração_cinza", "1FA76"},
    {"coração_rosa", "1FA77"},
    {"estetoscópio", "1FA7A"},
    {"ioiô", "1FA80"},
    {"varinha_mágica", "1FA84"},
    {"maracás", "1FA87"},
    {"baú_de_tesouro", "1FA8E"},
    {"pá", "1FA8F"},
    {"planeta_com_anéis", "1FA90"},
    {"lâmina", "1FA92"},
    {"lâmpada_de_óleo", "1FA94"},
    {"acordeão", "1FA97"},
    {"nó", "1FAA2"},
    {"lápide", "1FAA6"},
    {"cartão_de_identificação", "1FAAA"},
    {"hamsá", "1FAAC"},
    {"leque_dobrável", "1FAAD"},
    {"lótus", "1FAB7"},
    {"água_viva", "1FABC"},
    {"árvore_seca", "1FABE"},
    {"coração_humano", "1FAC0"},
    {"pulmões", "1FAC1"},
    {"pessoas_se_abraçando", "1FAC2"},
    {"homem_grávido", "1FAC3"},
    {"homem_grávido_cor1", "1FAC3-1F3FB"},
    {"homem_grávido_cor2", "1FAC3-1F3FC"},
    {"homem_grávido_cor3", "1FAC3-1F3FD"},
    {"homem_grávido_cor4", "1FAC3-1F3FE"},
    {"homem_grávido_cor5", "1FAC3-1F3FF"},
    {"pessoa_grávida", "1FAC4"},
    {"pessoa_grávida_cor1", "1FAC4-1F3FB"},
    {"pessoa_grávida_cor2", "1FAC4-1F3FC"},
    {"pessoa_grávida_cor3", "1FAC4-1F3FD"},
    {"pessoa_grávida_cor4", "1FAC4-1F3FE"},
    {"pessoa_grávida_cor5", "1FAC4-1F3FF"},
    {"impressão_digital", "1FAC6"},
    {"pimentão", "1FAD1"},
    {"pão_sírio", "1FAD3"},
    {"derramando_líquido", "1FAD7"},
    {"feijões", "1FAD8"},
    {"tubérculo", "1FADC"},
    {"rosto_com_olhos_abertos_e_mão_sobre_a_boca", "1FAE2"},
    {"mordendo_o_lábio", "1FAE6"},
    {"mão_com_dedo_indicador_e_polegar_cruzados", "1FAF0"},
    {"mão_com_dedo_indicador_e_polegar_cruzados_cor1", "1FAF0-1F3FB"},
    {"mão_com_dedo_indicador_e_polegar_cruzados_cor2", "1FAF0-1F3FC"},
    {"mão_com_dedo_indicador_e_polegar_cruzados_cor3", "1FAF0-1F3FD"},
    {"mão_com_dedo_indicador_e_polegar_cruzados_cor4", "1FAF0-1F3FE"},
    {"mão_com_dedo_indicador_e_polegar_cruzados_cor5", "1FAF0-1F3FF"},
    {"mão_para_a_direita", "1FAF1"},
    {"mão_para_a_direita_cor1", "1FAF1-1F3FB"},
    {"mão_para_a_direita_cor2", "1FAF1-1F3FC"},
    {"mão_para_a_direita_cor3", "1FAF1-1F3FD"},
    {"mão_para_a_direita_cor4", "1FAF1-1F3FE"},
    {"mão_para_a_direita_cor5", "1FAF1-1F3FF"},
    {"mão_para_a_esquerda", "1FAF2"},
    {"mão_para_a_esquerda_cor1", "1FAF2-1F3FB"},
    {"mão_para_a_esquerda_cor2", "1FAF2-1F3FC"},
    {"mão_para_a_esquerda_cor3", "1FAF2-1F3FD"},
    {"mão_para_a_esquerda_cor4", "1FAF2-1F3FE"},
    {"mão_para_a_esquerda_cor5", "1FAF2-1F3FF"},
    {"mão_com_a_palma_para_baixo", "1FAF3"},
    {"mão_com_a_palma_para_baixo_cor1", "1FAF3-1F3FB"},
    {"mão_com_a_palma_para_baixo_cor2", "1FAF3-1F3FC"},
    {"mão_com_a_palma_para_baixo_cor3", "1FAF3-1F3FD"},
    {"mão_com_a_palma_para_baixo_cor4", "1FAF3-1F3FE"},
    {"mão_com_a_palma_para_baixo_cor5", "1FAF3-1F3FF"},
    {"mão_com_a_palma_para_cima", "1FAF4"},
    {"mão_com_a_palma_para_cima_cor1", "1FAF4-1F3FB"},
    {"mão_com_a_palma_para_cima_cor2", "1FAF4-1F3FC"},
    {"mão_com_a_palma_para_cima_cor3", "1FAF4-1F3FD"},
    {"mão_com_a_palma_para_cima_cor4", "1FAF4-1F3FE"},
    {"mão_com_a_palma_para_cima_cor5", "1FAF4-1F3FF"},
    {"mãos_de_coração", "1FAF6"},
    {"mãos_de_coração_cor1", "1FAF6-1F3FB"},
    {"mãos_de_coração_cor2", "1FAF6-1F3FC"},
    {"mãos_de_coração_cor3", "1FAF6-1F3FD"},
    {"mãos_de_coração_cor4", "1FAF6-1F3FE"},
    {"mãos_de_coração_cor5", "1FAF6-1F3FF"},
    {"mão_empurrando_para_a_esquerda", "1FAF7"},
    {"mão_empurrando_para_a_esquerda_cor1", "1FAF7-1F3FB"},
    {"mão_empurrando_para_a_esquerda_cor2", "1FAF7-1F3FC"},
    {"mão_empurrando_para_a_esquerda_cor3", "1FAF7-1F3FD"},
    {"mão_empurrando_para_a_esquerda_cor4", "1FAF7-1F3FE"},
    {"mão_empurrando_para_a_esquerda_cor5", "1FAF7-1F3FF"},
    {"mão_empurrando_para_a_direita", "1FAF8"},
    {"mão_empurrando_para_a_direita_cor1", "1FAF8-1F3FB"},
    {"mão_empurrando_para_a_direita_cor2", "1FAF8-1F3FC"},
    {"mão_empurrando_para_a_direita_cor3", "1FAF8-1F3FD"},
    {"mão_empurrando_para_a_direita_cor4", "1FAF8-1F3FE"},
    {"mão_empurrando_para_a_direita_cor5", "1FAF8-1F3FF"},
    {"tecla_sinal_de_número", "0023-FE0F-20E3"},
    {"bandeira_ilha_de_ascensão", "1F1E6-1F1E8"},
    {"bandeira_emirados_árabes_unidos", "1F1E6-1F1EA"},
    {"bandeira_afeganistão", "1F1E6-1F1EB"},
    {"bandeira_antígua_e_barbuda", "1F1E6-1F1EC"},
    {"bandeira_albânia", "1F1E6-1F1F1"},
    {"bandeira_armênia", "1F1E6-1F1F2"},
    {"bandeira_antártida", "1F1E6-1F1F6"},
    {"bandeira_áustria", "1F1E6-1F1F9"},
    {"bandeira_austrália", "1F1E6-1F1FA"},
    {"bandeira_azerbaijão", "1F1E6-1F1FF"},
    {"bandeira_bósnia_e_herzegovina", "1F1E7-1F1E6"},
    {"bandeira_bélgica", "1F1E7-1F1EA"},
    {"bandeira_bulgária", "1F1E7-1F1EC"},
    {"bandeira_são_bartolomeu", "1F1E7-1F1F1"},
    {"bandeira_bolívia", "1F1E7-1F1F4"},
    {"bandeira_países_baixos_caribenhos", "1F1E7-1F1F6"},
    {"bandeira_butão", "1F1E7-1F1F9"},
    {"bandeira_bielorrússia", "1F1E7-1F1FE"},
    {"bandeira_canadá", "1F1E8-1F1E6"},
    {"bandeira_república_centro_africana", "1F1E8-1F1EB"},
    {"bandeira_república_do_congo", "1F1E8-1F1EC"},
    {"bandeira_suíça", "1F1E8-1F1ED"},
    {"bandeira_camarões", "1F1E8-1F1F2"},
    {"bandeira_colômbia", "1F1E8-1F1F4"},
    {"bandeira_curaçao", "1F1E8-1F1FC"},
    {"bandeira_tchéquia", "1F1E8-1F1FF"},
    {"bandeira_república_dominicana", "1F1E9-1F1F4"},
    {"bandeira_argélia", "1F1E9-1F1FF"},
    {"bandeira_estônia", "1F1EA-1F1EA"},
    {"bandeira_etiópia", "1F1EA-1F1F9"},
    {"bandeira_união_europeia", "1F1EA-1F1FA"},
    {"bandeira_finlândia", "1F1EB-1F1EE"},
    {"bandeira_micronésia", "1F1EB-1F1F2"},
    {"bandeira_ilhas_faroé", "1F1EB-1F1F4"},
    {"bandeira_frança", "1F1EB-1F1F7"},
    {"bandeira_gabão", "1F1EC-1F1E6"},
    {"bandeira_geórgia", "1F1EC-1F1EA"},
    {"bandeira_groenlândia", "1F1EC-1F1F1"},
    {"bandeira_gâmbia", "1F1EC-1F1F2"},
    {"bandeira_guiné", "1F1EC-1F1F3"},
    {"bandeira_guiné_equatorial", "1F1EC-1F1F6"},
    {"bandeira_grécia", "1F1EC-1F1F7"},
    {"bandeira_ilhas_geórgia_do_sul_e_sandwich_do_sul", "1F1EC-1F1F8"},
    {"bandeira_guiné_bissau", "1F1EC-1F1FC"},
    {"bandeira_croácia", "1F1ED-1F1F7"},
    {"bandeira_ilhas_canárias", "1F1EE-1F1E8"},
    {"bandeira_indonésia", "1F1EE-1F1E9"},
    {"bandeira_índia", "1F1EE-1F1F3"},
    {"bandeira_território_britânico_do_oceano_índico", "1F1EE-1F1F4"},
    {"bandeira_irã", "1F1EE-1F1F7"},
    {"bandeira_islândia", "1F1EE-1F1F8"},
    {"bandeira_itália", "1F1EE-1F1F9"},
    {"bandeira_jordânia", "1F1EF-1F1F4"},
    {"bandeira_japão", "1F1EF-1F1F5"},
    {"bandeira_quênia", "1F1F0-1F1EA"},
    {"bandeira_quirguistão", "1F1F0-1F1EC"},
    {"bandeira_são_cristóvão_e_névis", "1F1F0-1F1F3"},
    {"bandeira_cazaquistão", "1F1F0-1F1FF"},
    {"bandeira_líbano", "1F1F1-1F1E7"},
    {"bandeira_santa_lúcia", "1F1F1-1F1E8"},
    {"bandeira_libéria", "1F1F1-1F1F7"},
    {"bandeira_lituânia", "1F1F1-1F1F9"},
    {"bandeira_letônia", "1F1F1-1F1FB"},
    {"bandeira_líbia", "1F1F1-1F1FE"},
    {"bandeira_mônaco", "1F1F2-1F1E8"},
    {"bandeira_moldávia", "1F1F2-1F1E9"},
    {"bandeira_são_martinho", "1F1F2-1F1EB"},
    {"bandeira_macedônia_do_norte", "1F1F2-1F1F0"},
    {"bandeira_mianmar_birmânia", "1F1F2-1F1F2"},
    {"bandeira_mongólia", "1F1F2-1F1F3"},
    {"bandeira_mauritânia", "1F1F2-1F1F7"},
    {"bandeira_maurício", "1F1F2-1F1FA"},
    {"bandeira_méxico", "1F1F2-1F1FD"},
    {"bandeira_malásia", "1F1F2-1F1FE"},
    {"bandeira_moçambique", "1F1F2-1F1FF"},
    {"bandeira_namíbia", "1F1F3-1F1E6"},
    {"bandeira_nova_caledônia", "1F1F3-1F1E8"},
    {"bandeira_níger", "1F1F3-1F1EA"},
    {"bandeira_nigéria", "1F1F3-1F1EC"},
    {"bandeira_nicarágua", "1F1F3-1F1EE"},
    {"bandeira_países_baixos", "1F1F3-1F1F1"},
    {"bandeira_nova_zelândia", "1F1F3-1F1FF"},
    {"bandeira_omã", "1F1F4-1F1F2"},
    {"bandeira_panamá", "1F1F5-1F1E6"},
    {"bandeira_polinésia_francesa", "1F1F5-1F1EB"},
    {"bandeira_papua_nova_guiné", "1F1F5-1F1EC"},
    {"bandeira_paquistão", "1F1F5-1F1F0"},
    {"bandeira_polônia", "1F1F5-1F1F1"},
    {"bandeira_são_pedro_e_miquelão", "1F1F5-1F1F2"},
    {"bandeira_territórios_palestinos", "1F1F5-1F1F8"},
    {"bandeira_reunião", "1F1F7-1F1EA"},
    {"bandeira_romênia", "1F1F7-1F1F4"},
    {"bandeira_sérvia", "1F1F7-1F1F8"},
    {"bandeira_rússia", "1F1F7-1F1FA"},
    {"bandeira_arábia_saudita", "1F1F8-1F1E6"},
    {"bandeira_ilhas_salomão", "1F1F8-1F1E7"},
    {"bandeira_sudão", "1F1F8-1F1E9"},
    {"bandeira_suécia", "1F1F8-1F1EA"},
    {"bandeira_eslovênia", "1F1F8-1F1EE"},
    {"bandeira_eslováquia", "1F1F8-1F1F0"},
    {"bandeira_somália", "1F1F8-1F1F4"},
    {"bandeira_sudão_do_sul", "1F1F8-1F1F8"},
    {"bandeira_são_tomé_e_príncipe", "1F1F8-1F1F9"},
    {"bandeira_síria", "1F1F8-1F1FE"},
    {"bandeira_essuatíni", "1F1F8-1F1FF"},
    {"bandeira_tristão_da_cunha", "1F1F9-1F1E6"},
    {"bandeira_territórios_franceses_do_sul", "1F1F9-1F1EB"},
    {"bandeira_tailândia", "1F1F9-1F1ED"},
    {"bandeira_tadjiquistão", "1F1F9-1F1EF"},
    {"bandeira_turcomenistão", "1F1F9-1F1F2"},
    {"bandeira_tunísia", "1F1F9-1F1F3"},
    {"bandeira_tanzânia", "1F1F9-1F1FF"},
    {"bandeira_ucrânia", "1F1FA-1F1E6"},
    {"bandeira_nações_unidas", "1F1FA-1F1F3"},
    {"bandeira_uzbequistão", "1F1FA-1F1FF"},
    {"bandeira_são_vicente_e_granadinas", "1F1FB-1F1E8"},
    {"bandeira_ilhas_virgens_britânicas", "1F1FB-1F1EC"},
    {"bandeira_vietnã", "1F1FB-1F1F3"},
    {"bandeira_iêmen", "1F1FE-1F1EA"},
    {"bandeira_áfrica_do_sul", "1F1FF-1F1E6"},
    {"bandeira_zâmbia", "1F1FF-1F1F2"},
    {"bandeira_zimbábue", "1F1FF-1F1FC"},
    {"bandeira_escócia", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"bandeira_país_de_gales", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"família_homem_e_menino", "1F468-200D-1F466"},
    {"família_homem_menino_e_menino", "1F468-200D-1F466-200D-1F466"},
    {"família_homem_e_menina", "1F468-200D-1F467"},
    {"família_homem_menina_e_menino", "1F468-200D-1F467-200D-1F466"},
    {"família_homem_menina_e_menina", "1F468-200D-1F467-200D-1F467"},
    {"família_homem_homem_e_menino", "1F468-200D-1F468-200D-1F466"},
    {"família_homem_homem_menino_e_menino", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"família_homem_homem_e_menina", "1F468-200D-1F468-200D-1F467"},
    {"família_homem_homem_menina_e_menino", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"família_homem_homem_menina_e_menina", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"família_homem_mulher_e_menino", "1F468-200D-1F469-200D-1F466"},
    {"família_homem_mulher_menino_e_menino", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"família_homem_mulher_e_menina", "1F468-200D-1F469-200D-1F467"},
    {"família_homem_mulher_menina_e_menino", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"família_homem_mulher_menina_e_menina", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"família_mulher_e_menino", "1F469-200D-1F466"},
    {"família_mulher_menino_e_menino", "1F469-200D-1F466-200D-1F466"},
    {"família_mulher_e_menina", "1F469-200D-1F467"},
    {"família_mulher_menina_e_menino", "1F469-200D-1F467-200D-1F466"},
    {"família_mulher_menina_e_menina", "1F469-200D-1F467-200D-1F467"},
    {"família_mulher_mulher_e_menino", "1F469-200D-1F469-200D-1F466"},
    {"família_mulher_mulher_menino_e_menino", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"família_mulher_mulher_e_menina", "1F469-200D-1F469-200D-1F467"},
    {"família_mulher_mulher_menina_e_menino", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"família_mulher_mulher_menina_e_menina", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"pessoas_de_mãos_dadas", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"pessoas_de_mãos_dadas_cor1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"pessoas_de_mãos_dadas_cor1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"pessoas_de_mãos_dadas_cor1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"pessoas_de_mãos_dadas_cor1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"pessoas_de_mãos_dadas_cor1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"pessoas_de_mãos_dadas_cor2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"pessoas_de_mãos_dadas_cor2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"pessoas_de_mãos_dadas_cor2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"pessoas_de_mãos_dadas_cor2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"pessoas_de_mãos_dadas_cor2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"pessoas_de_mãos_dadas_cor3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"pessoas_de_mãos_dadas_cor3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"pessoas_de_mãos_dadas_cor3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"pessoas_de_mãos_dadas_cor3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"pessoas_de_mãos_dadas_cor3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"pessoas_de_mãos_dadas_cor4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"pessoas_de_mãos_dadas_cor4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"pessoas_de_mãos_dadas_cor4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"pessoas_de_mãos_dadas_cor4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"pessoas_de_mãos_dadas_cor4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"pessoas_de_mãos_dadas_cor5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"pessoas_de_mãos_dadas_cor5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"pessoas_de_mãos_dadas_cor5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"pessoas_de_mãos_dadas_cor5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"pessoas_de_mãos_dadas_cor5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"família_adulto_adulto_criança", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"família_adulto_adulto_criança_criança", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"família_adulto_criança", "1F9D1-200D-1F9D2"},
    {"família_adulto_criança_criança", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"homem_profissional_da_saúde", "1F468-200D-2695-FE0F"},
    {"homem_profissional_da_saúde_cor1", "1F468-1F3FB-200D-2695-FE0F"},
    {"homem_profissional_da_saúde_cor2", "1F468-1F3FC-200D-2695-FE0F"},
    {"homem_profissional_da_saúde_cor3", "1F468-1F3FD-200D-2695-FE0F"},
    {"homem_profissional_da_saúde_cor4", "1F468-1F3FE-200D-2695-FE0F"},
    {"homem_profissional_da_saúde_cor5", "1F468-1F3FF-200D-2695-FE0F"},
    {"piloto_de_avião_homem", "1F468-200D-2708-FE0F"},
    {"piloto_de_avião_homem_cor1", "1F468-1F3FB-200D-2708-FE0F"},
    {"piloto_de_avião_homem_cor2", "1F468-1F3FC-200D-2708-FE0F"},
    {"piloto_de_avião_homem_cor3", "1F468-1F3FD-200D-2708-FE0F"},
    {"piloto_de_avião_homem_cor4", "1F468-1F3FE-200D-2708-FE0F"},
    {"piloto_de_avião_homem_cor5", "1F468-1F3FF-200D-2708-FE0F"},
    {"homem_alimentando_bebê", "1F468-200D-1F37C"},
    {"homem_alimentando_bebê_cor1", "1F468-1F3FB-200D-1F37C"},
    {"homem_alimentando_bebê_cor2", "1F468-1F3FC-200D-1F37C"},
    {"homem_alimentando_bebê_cor3", "1F468-1F3FD-200D-1F37C"},
    {"homem_alimentando_bebê_cor4", "1F468-1F3FE-200D-1F37C"},
    {"homem_alimentando_bebê_cor5", "1F468-1F3FF-200D-1F37C"},
    {"artista_plástico", "1F468-200D-1F3A8"},
    {"artista_plástico_cor1", "1F468-1F3FB-200D-1F3A8"},
    {"artista_plástico_cor2", "1F468-1F3FC-200D-1F3A8"},
    {"artista_plástico_cor3", "1F468-1F3FD-200D-1F3A8"},
    {"artista_plástico_cor4", "1F468-1F3FE-200D-1F3A8"},
    {"artista_plástico_cor5", "1F468-1F3FF-200D-1F3A8"},
    {"operário", "1F468-200D-1F3ED"},
    {"operário_cor1", "1F468-1F3FB-200D-1F3ED"},
    {"operário_cor2", "1F468-1F3FC-200D-1F3ED"},
    {"operário_cor3", "1F468-1F3FD-200D-1F3ED"},
    {"operário_cor4", "1F468-1F3FE-200D-1F3ED"},
    {"operário_cor5", "1F468-1F3FF-200D-1F3ED"},
    {"tecnólogo", "1F468-200D-1F4BB"},
    {"tecnólogo_cor1", "1F468-1F3FB-200D-1F4BB"},
    {"tecnólogo_cor2", "1F468-1F3FC-200D-1F4BB"},
    {"tecnólogo_cor3", "1F468-1F3FD-200D-1F4BB"},
    {"tecnólogo_cor4", "1F468-1F3FE-200D-1F4BB"},
    {"tecnólogo_cor5", "1F468-1F3FF-200D-1F4BB"},
    {"funcionário_de_escritório", "1F468-200D-1F4BC"},
    {"funcionário_de_escritório_cor1", "1F468-1F3FB-200D-1F4BC"},
    {"funcionário_de_escritório_cor2", "1F468-1F3FC-200D-1F4BC"},
    {"funcionário_de_escritório_cor3", "1F468-1F3FD-200D-1F4BC"},
    {"funcionário_de_escritório_cor4", "1F468-1F3FE-200D-1F4BC"},
    {"funcionário_de_escritório_cor5", "1F468-1F3FF-200D-1F4BC"},
    {"mecânico", "1F468-200D-1F527"},
    {"mecânico_cor1", "1F468-1F3FB-200D-1F527"},
    {"mecânico_cor2", "1F468-1F3FC-200D-1F527"},
    {"mecânico_cor3", "1F468-1F3FD-200D-1F527"},
    {"mecânico_cor4", "1F468-1F3FE-200D-1F527"},
    {"mecânico_cor5", "1F468-1F3FF-200D-1F527"},
    {"mulher_profissional_da_saúde", "1F469-200D-2695-FE0F"},
    {"mulher_profissional_da_saúde_cor1", "1F469-1F3FB-200D-2695-FE0F"},
    {"mulher_profissional_da_saúde_cor2", "1F469-1F3FC-200D-2695-FE0F"},
    {"mulher_profissional_da_saúde_cor3", "1F469-1F3FD-200D-2695-FE0F"},
    {"mulher_profissional_da_saúde_cor4", "1F469-1F3FE-200D-2695-FE0F"},
    {"mulher_profissional_da_saúde_cor5", "1F469-1F3FF-200D-2695-FE0F"},
    {"juíza", "1F469-200D-2696-FE0F"},
    {"juíza_cor1", "1F469-1F3FB-200D-2696-FE0F"},
    {"juíza_cor2", "1F469-1F3FC-200D-2696-FE0F"},
    {"juíza_cor3", "1F469-1F3FD-200D-2696-FE0F"},
    {"juíza_cor4", "1F469-1F3FE-200D-2696-FE0F"},
    {"juíza_cor5", "1F469-1F3FF-200D-2696-FE0F"},
    {"piloto_de_avião_mulher", "1F469-200D-2708-FE0F"},
    {"piloto_de_avião_mulher_cor1", "1F469-1F3FB-200D-2708-FE0F"},
    {"piloto_de_avião_mulher_cor2", "1F469-1F3FC-200D-2708-FE0F"},
    {"piloto_de_avião_mulher_cor3", "1F469-1F3FD-200D-2708-FE0F"},
    {"piloto_de_avião_mulher_cor4", "1F469-1F3FE-200D-2708-FE0F"},
    {"piloto_de_avião_mulher_cor5", "1F469-1F3FF-200D-2708-FE0F"},
    {"mulher_alimentando_bebê", "1F469-200D-1F37C"},
    {"mulher_alimentando_bebê_cor1", "1F469-1F3FB-200D-1F37C"},
    {"mulher_alimentando_bebê_cor2", "1F469-1F3FC-200D-1F37C"},
    {"mulher_alimentando_bebê_cor3", "1F469-1F3FD-200D-1F37C"},
    {"mulher_alimentando_bebê_cor4", "1F469-1F3FE-200D-1F37C"},
    {"mulher_alimentando_bebê_cor5", "1F469-1F3FF-200D-1F37C"},
    {"artista_plástica", "1F469-200D-1F3A8"},
    {"artista_plástica_cor1", "1F469-1F3FB-200D-1F3A8"},
    {"artista_plástica_cor2", "1F469-1F3FC-200D-1F3A8"},
    {"artista_plástica_cor3", "1F469-1F3FD-200D-1F3A8"},
    {"artista_plástica_cor4", "1F469-1F3FE-200D-1F3A8"},
    {"artista_plástica_cor5", "1F469-1F3FF-200D-1F3A8"},
    {"operária", "1F469-200D-1F3ED"},
    {"operária_cor1", "1F469-1F3FB-200D-1F3ED"},
    {"operária_cor2", "1F469-1F3FC-200D-1F3ED"},
    {"operária_cor3", "1F469-1F3FD-200D-1F3ED"},
    {"operária_cor4", "1F469-1F3FE-200D-1F3ED"},
    {"operária_cor5", "1F469-1F3FF-200D-1F3ED"},
    {"tecnóloga", "1F469-200D-1F4BB"},
    {"tecnóloga_cor1", "1F469-1F3FB-200D-1F4BB"},
    {"tecnóloga_cor2", "1F469-1F3FC-200D-1F4BB"},
    {"tecnóloga_cor3", "1F469-1F3FD-200D-1F4BB"},
    {"tecnóloga_cor4", "1F469-1F3FE-200D-1F4BB"},
    {"tecnóloga_cor5", "1F469-1F3FF-200D-1F4BB"},
    {"funcionária_de_escritório", "1F469-200D-1F4BC"},
    {"funcionária_de_escritório_cor1", "1F469-1F3FB-200D-1F4BC"},
    {"funcionária_de_escritório_cor2", "1F469-1F3FC-200D-1F4BC"},
    {"funcionária_de_escritório_cor3", "1F469-1F3FD-200D-1F4BC"},
    {"funcionária_de_escritório_cor4", "1F469-1F3FE-200D-1F4BC"},
    {"funcionária_de_escritório_cor5", "1F469-1F3FF-200D-1F4BC"},
    {"mecânica", "1F469-200D-1F527"},
    {"mecânica_cor1", "1F469-1F3FB-200D-1F527"},
    {"mecânica_cor2", "1F469-1F3FC-200D-1F527"},
    {"mecânica_cor3", "1F469-1F3FD-200D-1F527"},
    {"mecânica_cor4", "1F469-1F3FE-200D-1F527"},
    {"mecânica_cor5", "1F469-1F3FF-200D-1F527"},
    {"profissional_de_saúde", "1F9D1-200D-2695-FE0F"},
    {"profissional_de_saúde_cor1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"profissional_de_saúde_cor2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"profissional_de_saúde_cor3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"profissional_de_saúde_cor4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"profissional_de_saúde_cor5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"pessoa_alimentando_bebê", "1F9D1-200D-1F37C"},
    {"pessoa_alimentando_bebê_cor1", "1F9D1-1F3FB-200D-1F37C"},
    {"pessoa_alimentando_bebê_cor2", "1F9D1-1F3FC-200D-1F37C"},
    {"pessoa_alimentando_bebê_cor3", "1F9D1-1F3FD-200D-1F37C"},
    {"pessoa_alimentando_bebê_cor4", "1F9D1-1F3FE-200D-1F37C"},
    {"pessoa_alimentando_bebê_cor5", "1F9D1-1F3FF-200D-1F37C"},
    {"funcionário_de_fábrica", "1F9D1-200D-1F3ED"},
    {"funcionário_de_fábrica_cor1", "1F9D1-1F3FB-200D-1F3ED"},
    {"funcionário_de_fábrica_cor2", "1F9D1-1F3FC-200D-1F3ED"},
    {"funcionário_de_fábrica_cor3", "1F9D1-1F3FD-200D-1F3ED"},
    {"funcionário_de_fábrica_cor4", "1F9D1-1F3FE-200D-1F3ED"},
    {"funcionário_de_fábrica_cor5", "1F9D1-1F3FF-200D-1F3ED"},
    {"trabalhador_de_escritório", "1F9D1-200D-1F4BC"},
    {"trabalhador_de_escritório_cor1", "1F9D1-1F3FB-200D-1F4BC"},
    {"trabalhador_de_escritório_cor2", "1F9D1-1F3FC-200D-1F4BC"},
    {"trabalhador_de_escritório_cor3", "1F9D1-1F3FD-200D-1F4BC"},
    {"trabalhador_de_escritório_cor4", "1F9D1-1F3FE-200D-1F4BC"},
    {"trabalhador_de_escritório_cor5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mecânico_trabalhando", "1F9D1-200D-1F527"},
    {"mecânico_trabalhando_cor1", "1F9D1-1F3FB-200D-1F527"},
    {"mecânico_trabalhando_cor2", "1F9D1-1F3FC-200D-1F527"},
    {"mecânico_trabalhando_cor3", "1F9D1-1F3FD-200D-1F527"},
    {"mecânico_trabalhando_cor4", "1F9D1-1F3FE-200D-1F527"},
    {"mecânico_trabalhando_cor5", "1F9D1-1F3FF-200D-1F527"},
    {"noiva_com_véu", "1F470-200D-2640-FE0F"},
    {"noiva_com_véu_cor1", "1F470-1F3FB-200D-2640-FE0F"},
    {"noiva_com_véu_cor2", "1F470-1F3FC-200D-2640-FE0F"},
    {"noiva_com_véu_cor3", "1F470-1F3FD-200D-2640-FE0F"},
    {"noiva_com_véu_cor4", "1F470-1F3FE-200D-2640-FE0F"},
    {"noiva_com_véu_cor5", "1F470-1F3FF-200D-2640-FE0F"},
    {"noivo_com_véu", "1F470-200D-2642-FE0F"},
    {"noivo_com_véu_cor1", "1F470-1F3FB-200D-2642-FE0F"},
    {"noivo_com_véu_cor2", "1F470-1F3FC-200D-2642-FE0F"},
    {"noivo_com_véu_cor3", "1F470-1F3FD-200D-2642-FE0F"},
    {"noivo_com_véu_cor4", "1F470-1F3FE-200D-2642-FE0F"},
    {"noivo_com_véu_cor5", "1F470-1F3FF-200D-2642-FE0F"},
    {"mulher_fazendo_gesto_de_não", "1F645-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_não_cor1", "1F645-1F3FB-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_não_cor2", "1F645-1F3FC-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_não_cor3", "1F645-1F3FD-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_não_cor4", "1F645-1F3FE-200D-2640-FE0F"},
    {"mulher_fazendo_gesto_de_não_cor5", "1F645-1F3FF-200D-2640-FE0F"},
    {"homem_fazendo_gesto_de_não", "1F645-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_não_cor1", "1F645-1F3FB-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_não_cor2", "1F645-1F3FC-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_não_cor3", "1F645-1F3FD-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_não_cor4", "1F645-1F3FE-200D-2642-FE0F"},
    {"homem_fazendo_gesto_de_não_cor5", "1F645-1F3FF-200D-2642-FE0F"},
    {"mulher_fazendo_reverência", "1F647-200D-2640-FE0F"},
    {"mulher_fazendo_reverência_cor1", "1F647-1F3FB-200D-2640-FE0F"},
    {"mulher_fazendo_reverência_cor2", "1F647-1F3FC-200D-2640-FE0F"},
    {"mulher_fazendo_reverência_cor3", "1F647-1F3FD-200D-2640-FE0F"},
    {"mulher_fazendo_reverência_cor4", "1F647-1F3FE-200D-2640-FE0F"},
    {"mulher_fazendo_reverência_cor5", "1F647-1F3FF-200D-2640-FE0F"},
    {"homem_fazendo_reverência", "1F647-200D-2642-FE0F"},
    {"homem_fazendo_reverência_cor1", "1F647-1F3FB-200D-2642-FE0F"},
    {"homem_fazendo_reverência_cor2", "1F647-1F3FC-200D-2642-FE0F"},
    {"homem_fazendo_reverência_cor3", "1F647-1F3FD-200D-2642-FE0F"},
    {"homem_fazendo_reverência_cor4", "1F647-1F3FE-200D-2642-FE0F"},
    {"homem_fazendo_reverência_cor5", "1F647-1F3FF-200D-2642-FE0F"},
    {"mulher_levantando_a_mão", "1F64B-200D-2640-FE0F"},
    {"mulher_levantando_a_mão_cor1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"mulher_levantando_a_mão_cor2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"mulher_levantando_a_mão_cor3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"mulher_levantando_a_mão_cor4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"mulher_levantando_a_mão_cor5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"homem_levantando_a_mão", "1F64B-200D-2642-FE0F"},
    {"homem_levantando_a_mão_cor1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"homem_levantando_a_mão_cor2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"homem_levantando_a_mão_cor3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"homem_levantando_a_mão_cor4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"homem_levantando_a_mão_cor5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"mulher_jogando_polo_aquático", "1F93D-200D-2640-FE0F"},
    {"mulher_jogando_polo_aquático_cor1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"mulher_jogando_polo_aquático_cor2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"mulher_jogando_polo_aquático_cor3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"mulher_jogando_polo_aquático_cor4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"mulher_jogando_polo_aquático_cor5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"homem_jogando_polo_aquático", "1F93D-200D-2642-FE0F"},
    {"homem_jogando_polo_aquático_cor1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"homem_jogando_polo_aquático_cor2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"homem_jogando_polo_aquático_cor3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"homem_jogando_polo_aquático_cor4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"homem_jogando_polo_aquático_cor5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"super_heroína", "1F9B8-200D-2640-FE0F"},
    {"super_heroína_cor1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"super_heroína_cor2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"super_heroína_cor3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"super_heroína_cor4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"super_heroína_cor5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"homem_super_herói", "1F9B8-200D-2642-FE0F"},
    {"homem_super_herói_cor1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"homem_super_herói_cor2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"homem_super_herói_cor3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"homem_super_herói_cor4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"homem_super_herói_cor5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"supervilã", "1F9B9-200D-2640-FE0F"},
    {"supervilã_cor1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"supervilã_cor2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"supervilã_cor3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"supervilã_cor4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"supervilã_cor5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"homem_supervilão", "1F9B9-200D-2642-FE0F"},
    {"homem_supervilão_cor1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"homem_supervilão_cor2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"homem_supervilão_cor3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"homem_supervilão_cor4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"homem_supervilão_cor5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"mulher_em_pé", "1F9CD-200D-2640-FE0F"},
    {"mulher_em_pé_cor1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"mulher_em_pé_cor2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"mulher_em_pé_cor3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"mulher_em_pé_cor4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"mulher_em_pé_cor5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"homem_em_pé", "1F9CD-200D-2642-FE0F"},
    {"homem_em_pé_cor1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"homem_em_pé_cor2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"homem_em_pé_cor3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"homem_em_pé_cor4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"homem_em_pé_cor5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"mulher_na_posição_de_lótus", "1F9D8-200D-2640-FE0F"},
    {"mulher_na_posição_de_lótus_cor1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"mulher_na_posição_de_lótus_cor2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"mulher_na_posição_de_lótus_cor3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"mulher_na_posição_de_lótus_cor4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"mulher_na_posição_de_lótus_cor5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"homem_na_posição_de_lótus", "1F9D8-200D-2642-FE0F"},
    {"homem_na_posição_de_lótus_cor1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"homem_na_posição_de_lótus_cor2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"homem_na_posição_de_lótus_cor3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"homem_na_posição_de_lótus_cor4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"homem_na_posição_de_lótus_cor5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"mulher_gênio", "1F9DE-200D-2640-FE0F"},
    {"homem_gênio", "1F9DE-200D-2642-FE0F"},
    {"coração_em_chamas", "2764-FE0F-200D-1F525"},
    {"coração_remendado", "2764-FE0F-200D-1FA79"},
    {"bandeira_transgênero", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"bandeira_do_arco_íris", "1F3F3-FE0F-200D-1F308"},
    {"cão_de_serviço", "1F415-200D-1F9BA"},
    {"pássaro_preto", "1F426-200D-2B1B"},
    {"fênix", "1F426-200D-1F525"},
    {"olho_no_balão_de_diálogo", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"cabeça_virando_de_um_lado_para_o_outro", "1F642-200D-2194-FE0F"},
    {"cabeça_balançando_na_vertical", "1F642-200D-2195-FE0F"},
};

