// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_uk[] = {
    {"znak_okliku_i_pitannya", "2049"},
    {"torgovelna_marka", "2122"},
    {"dzherelo_informaciyi", "2139"},
    {"strilka_vlivo_vpravo", "2194"},
    {"strilka_vgoru_vniz", "2195"},
    {"strilka_vgoru_vlivo", "2196"},
    {"strilka_vgoru_vpravo", "2197"},
    {"strilka_vniz_upravo", "2198"},
    {"strilka_vniz_ulivo", "2199"},
    {"klaviatura", "2328"},
    {"sonce", "2600"},
    {"hmara", "2601"},
    {"parasolka", "2602"},
    {"snigovik", "2603"},
    {"kometa", "2604"},
    {"klitinka_z_galochkoyu", "2611"},
    {"parasolka_z_kraplyami_doshchu", "2614"},
    {"garyachiy_napiy", "2615"},
    {"trilisnik", "2618"},
    {"cherep_i_kistki", "2620"},
    {"radiaciyna_nebezpeka", "2622"},
    {"biologichna_zagroza", "2623"},
    {"pravoslavniy_hrest", "2626"},
    {"koleso_dharmi", "2638"},
    {"pohmure_oblichchya", "2639"},
    {"simvol_zhinki", "2640"},
    {"simvol_cholovika", "2642"},
    {"oven", "2648"},
    {"telec", "2649"},
    {"strilec", "2650"},
    {"kozorig", "2651"},
    {"vodoliy", "2652"},
    {"ribi", "2653"},
    {"pika", "2660"},
    {"trefa", "2663"},
    {"chirva", "2665"},
    {"bubna", "2666"},
    {"garyachi_dzherela", "2668"},
    {"molotok_i_kirka", "2692"},
    {"yakir", "2693"},
    {"shreshcheni_mechi", "2694"},
    {"simvol_medicini", "2695"},
    {"vagi", "2696"},
    {"pereginniy_kub", "2697"},
    {"shestirnya", "2699"},
    {"nozhici", "2702"},
    {"knopka_z_galochkoyu", "2705"},
    {"litak", "2708"},
    {"konvert", "2709"},
    {"kinchik_chornilnogo_pera", "2712"},
    {"galochka", "2714"},
    {"mnozhennya", "2716"},
    {"zirka_davida", "2721"},
    {"bliskitki", "2728"},
    {"vosmikutniy_asterisk", "2733"},
    {"vosmikinceva_zirka", "2734"},
    {"snizhinka", "2744"},
    {"bliskitka", "2747"},
    {"chervoniy_znak_pitannya", "2753"},
    {"biliy_znak_pitannya", "2754"},
    {"biliy_znak_okliku", "2755"},
    {"chervoniy_znak_okliku", "2757"},
    {"znak_okliku_u_formi_sercya", "2763"},
    {"chervone_serce", "2764"},
    {"plyus", "2795"},
    {"minus", "2796"},
    {"dilennya", "2797"},
    {"strilka_vpravo_z_viginom_ugoru", "2934"},
    {"strilka_vpravo_z_viginom_uniz", "2935"},
    {"hvilyaste_tire", "3030"},
    {"knopka_vitannya_yaponskoyu", "3297"},
    {"knopka_sekret_yaponskoyu", "3299"},
    {"avtorske_pravo", "00A9"},
    {"zareiestrovana_torgovelna_marka", "00AE"},
    {"podviyniy_znak_okliku", "203C"},
    {"strilka_vpravo_z_viginom_ulivo", "21A9"},
    {"strilka_vlivo_z_viginom_upravo", "21AA"},
    {"naruchniy_godinnik", "231A"},
    {"piskoviy_godinnik", "231B"},
    {"knopka_vityagannya", "23CF"},
    {"knopka_peremotuvannya_vpered", "23E9"},
    {"knopka_peremotuvannya_nazad", "23EA"},
    {"knopka_motannya_vgoru", "23EB"},
    {"knopka_motannya_vniz", "23EC"},
    {"knopka_nastupnoyi_kompoziciyi", "23ED"},
    {"knopka_povernennya_do_poperednoyi_kompoziciyi", "23EE"},
    {"knopka_vidtvoriti_abo_prizupiniti", "23EF"},
    {"budilnik", "23F0"},
    {"sekundomir", "23F1"},
    {"taymer", "23F2"},
    {"piskoviy_godinnik_iz_piskom_yakiy_sipletsya", "23F3"},
    {"knopka_pauzi", "23F8"},
    {"knopka_stop", "23F9"},
    {"knopka_zapisu", "23FA"},
    {"kolo_z_literoyu_m", "24C2"},
    {"maliy_chorniy_kvadrat", "25AA"},
    {"maliy_biliy_kvadrat", "25AB"},
    {"knopka_vidtvorennya", "25B6"},
    {"knopka_reversu", "25C0"},
    {"seredniy_biliy_kvadrat", "25FB"},
    {"seredniy_chorniy_kvadrat", "25FC"},
    {"menshiy_za_seredniy_biliy_kvadrat", "25FD"},
    {"menshiy_za_seredniy_chorniy_kvadrat", "25FE"},
    {"telefon", "260E"},
    {"vkazivniy_palec_shcho_vkazuie_vgoru", "261D"},
    {"vkazivniy_palec_shcho_vkazuie_vgoru_kolir1", "261D-1F3FB"},
    {"vkazivniy_palec_shcho_vkazuie_vgoru_kolir2", "261D-1F3FC"},
    {"vkazivniy_palec_shcho_vkazuie_vgoru_kolir3", "261D-1F3FD"},
    {"vkazivniy_palec_shcho_vkazuie_vgoru_kolir4", "261D-1F3FE"},
    {"vkazivniy_palec_shcho_vkazuie_vgoru_kolir5", "261D-1F3FF"},
    {"zirka_i_pivmisyac", "262A"},
    {"simvol_miru", "262E"},
    {"in_yan", "262F"},
    {"usmihnene_oblichchya", "263A"},
    {"bliznyuki", "264A"},
    {"rak", "264B"},
    {"lev_znak_zodiaku", "264C"},
    {"diva", "264D"},
    {"terezi", "264E"},
    {"skorpion_znak_zodiaku", "264F"},
    {"pishak", "265F"},
    {"simvol_utilizaciyi", "267B"},
    {"neskinchennist", "267E"},
    {"simvol_krisla_vizka", "267F"},
    {"simvol_atoma", "269B"},
    {"geraldichna_liliya", "269C"},
    {"poperedzhennya", "26A0"},
    {"visoka_napruga", "26A1"},
    {"transgenderniy_simvol", "26A7"},
    {"biliy_krug", "26AA"},
    {"chorniy_krug", "26AB"},
    {"truna", "26B0"},
    {"urna_dlya_tlinu", "26B1"},
    {"futbolniy_myach", "26BD"},
    {"beysbolniy_myach", "26BE"},
    {"snigovik_bez_snigu", "26C4"},
    {"sonce_za_hmaroyu", "26C5"},
    {"hmara_z_bliskavkoyu_ta_doshchem", "26C8"},
    {"zmiienosec", "26CE"},
    {"kirka", "26CF"},
    {"kaska_ryatuvalnika", "26D1"},
    {"lancyugi", "26D3"},
    {"vyizd_zaboroneno", "26D4"},
    {"sintoyistskiy_hram", "26E9"},
    {"cerkva", "26EA"},
    {"gora", "26F0"},
    {"plyazhna_parasolka", "26F1"},
    {"fontan", "26F2"},
    {"lunka_dlya_golfu_z_praporcem", "26F3"},
    {"porom", "26F4"},
    {"vitrilnik", "26F5"},
    {"lizhnik", "26F7"},
    {"kovzan", "26F8"},
    {"lyudina_shcho_vede_myach", "26F9"},
    {"lyudina_shcho_vede_myach_kolir1", "26F9-1F3FB"},
    {"lyudina_shcho_vede_myach_kolir2", "26F9-1F3FC"},
    {"lyudina_shcho_vede_myach_kolir3", "26F9-1F3FD"},
    {"lyudina_shcho_vede_myach_kolir4", "26F9-1F3FE"},
    {"lyudina_shcho_vede_myach_kolir5", "26F9-1F3FF"},
    {"namet", "26FA"},
    {"benzokolonka", "26FD"},
    {"pidnyatiy_kulak", "270A"},
    {"pidnyatiy_kulak_kolir1", "270A-1F3FB"},
    {"pidnyatiy_kulak_kolir2", "270A-1F3FC"},
    {"pidnyatiy_kulak_kolir3", "270A-1F3FD"},
    {"pidnyatiy_kulak_kolir4", "270A-1F3FE"},
    {"pidnyatiy_kulak_kolir5", "270A-1F3FF"},
    {"pidnyata_ruka", "270B"},
    {"pidnyata_ruka_kolir1", "270B-1F3FB"},
    {"pidnyata_ruka_kolir2", "270B-1F3FC"},
    {"pidnyata_ruka_kolir3", "270B-1F3FD"},
    {"pidnyata_ruka_kolir4", "270B-1F3FE"},
    {"pidnyata_ruka_kolir5", "270B-1F3FF"},
    {"zhest_peremogi", "270C"},
    {"zhest_peremogi_kolir1", "270C-1F3FB"},
    {"zhest_peremogi_kolir2", "270C-1F3FC"},
    {"zhest_peremogi_kolir3", "270C-1F3FD"},
    {"zhest_peremogi_kolir4", "270C-1F3FE"},
    {"zhest_peremogi_kolir5", "270C-1F3FF"},
    {"ruka_shcho_pishe", "270D"},
    {"ruka_shcho_pishe_kolir1", "270D-1F3FB"},
    {"ruka_shcho_pishe_kolir2", "270D-1F3FC"},
    {"ruka_shcho_pishe_kolir3", "270D-1F3FD"},
    {"ruka_shcho_pishe_kolir4", "270D-1F3FE"},
    {"ruka_shcho_pishe_kolir5", "270D-1F3FF"},
    {"olivec", "270F"},
    {"latinskiy_hrest", "271D"},
    {"hrestik", "274C"},
    {"knopka_z_hrestikom", "274E"},
    {"strilka_vpravo", "27A1"},
    {"zakruchena_petlya", "27B0"},
    {"podviyna_zakruchena_petlya", "27BF"},
    {"strilka_vlivo", "2B05"},
    {"strilka_vgoru", "2B06"},
    {"strilka_vniz", "2B07"},
    {"velikiy_chorniy_kvadrat", "2B1B"},
    {"velikiy_biliy_kvadrat", "2B1C"},
    {"zirka", "2B50"},
    {"porozhniste_chervone_kolo", "2B55"},
    {"yaponskiy_znak_pochatku_chastini_pisni", "303D"},
    {"fishka_madzhongu_chervoniy_drakon", "1F004"},
    {"dzhoker", "1F0CF"},
    {"knopka_a_ii_grupa_krovi", "1F170"},
    {"knopka_b_iii_grupa_krovi", "1F171"},
    {"knopka_o_i_grupa_krovi", "1F17E"},
    {"knopka_parkuvannya", "1F17F"},
    {"knopka_ab_iv_grupa_krovi", "1F18E"},
    {"knopka_ochistiti", "1F191"},
    {"knopka_cool", "1F192"},
    {"knopka_free", "1F193"},
    {"knopka_id", "1F194"},
    {"knopka_new", "1F195"},
    {"knopka_ng", "1F196"},
    {"knopka_ok", "1F197"},
    {"knopka_sos", "1F198"},
    {"knopka_up", "1F199"},
    {"knopka_vs", "1F19A"},
    {"regionalniy_pokaznik_a", "1F1E6"},
    {"regionalniy_pokaznik_b", "1F1E7"},
    {"regionalniy_pokaznik_c", "1F1E8"},
    {"regionalniy_pokaznik_d", "1F1E9"},
    {"regionalniy_pokaznik_e", "1F1EA"},
    {"regionalniy_pokaznik_f", "1F1EB"},
    {"regionalniy_pokaznik_g", "1F1EC"},
    {"regionalniy_pokaznik_h", "1F1ED"},
    {"regionalniy_pokaznik_i", "1F1EE"},
    {"regionalniy_pokaznik_j", "1F1EF"},
    {"regionalniy_pokaznik_k", "1F1F0"},
    {"regionalniy_pokaznik_l", "1F1F1"},
    {"regionalniy_pokaznik_m", "1F1F2"},
    {"regionalniy_pokaznik_n", "1F1F3"},
    {"regionalniy_pokaznik_o", "1F1F4"},
    {"regionalniy_pokaznik_p", "1F1F5"},
    {"regionalniy_pokaznik_q", "1F1F6"},
    {"regionalniy_pokaznik_r", "1F1F7"},
    {"regionalniy_pokaznik_s", "1F1F8"},
    {"regionalniy_pokaznik_t", "1F1F9"},
    {"regionalniy_pokaznik_u", "1F1FA"},
    {"regionalniy_pokaznik_v", "1F1FB"},
    {"regionalniy_pokaznik_w", "1F1FC"},
    {"regionalniy_pokaznik_x", "1F1FD"},
    {"regionalniy_pokaznik_y", "1F1FE"},
    {"regionalniy_pokaznik_z", "1F1FF"},
    {"knopka_tut_yaponskoyu", "1F201"},
    {"knopka_servisniy_zbir_yaponskoyu", "1F202"},
    {"knopka_bezkoshtovno_yaponskoyu", "1F21A"},
    {"knopka_zarezervovano_yaponskoyu", "1F22F"},
    {"knopka_zaboroneno_yaponskoyu", "1F232"},
    {"knopka_vilno_yaponskoyu", "1F233"},
    {"knopka_dostatnya_ocinka_yaponskoyu", "1F234"},
    {"knopka_misc_nemaie_yaponskoyu", "1F235"},
    {"knopka_ne_bezkoshtovno_yaponskoyu", "1F236"},
    {"knopka_shchomisyacya_yaponskoyu", "1F237"},
    {"knopka_zayavka_yaponskoyu", "1F238"},
    {"knopka_znizhka_yaponskoyu", "1F239"},
    {"knopka_robota_yaponskoyu", "1F23A"},
    {"knopka_ugoda_yaponskoyu", "1F250"},
    {"knopka_priynyatno_yaponskoyu", "1F251"},
    {"ciklon", "1F300"},
    {"tumanno", "1F301"},
    {"skladena_parasolka", "1F302"},
    {"zoryana_nich", "1F303"},
    {"shid_soncya_nad_gorami", "1F304"},
    {"shid_soncya", "1F305"},
    {"misto_v_sutinkah", "1F306"},
    {"zahid_soncya", "1F307"},
    {"veselka", "1F308"},
    {"mist_unochi", "1F309"},
    {"hvilya", "1F30A"},
    {"vulkan", "1F30B"},
    {"chumackiy_shlyah", "1F30C"},
    {"globus_shcho_pokazuie_ievropu_y_afriku", "1F30D"},
    {"globus_shcho_pokazuie_ameriki", "1F30E"},
    {"globus_shcho_pokazuie_aziyu_i_avstraliyu", "1F30F"},
    {"zemna_kulya_z_meridianami", "1F310"},
    {"molodiy_misyac", "1F311"},
    {"misyac_u_pershiy_chverti", "1F312"},
    {"persha_chvert_misyacya", "1F313"},
    {"misyac_u_drugiy_chverti", "1F314"},
    {"povniy_misyac", "1F315"},
    {"misyac_u_tretiy_chverti", "1F316"},
    {"ostannya_chvert_misyacya", "1F317"},
    {"misyac_u_chetvertiy_chverti", "1F318"},
    {"serp_misyacya", "1F319"},
    {"molodiy_misyac_z_oblichchyam", "1F31A"},
    {"persha_chvert_misyacya_z_oblichchyam", "1F31B"},
    {"ostannya_chvert_misyacya_z_oblichchyam", "1F31C"},
    {"povniy_misyac_z_oblichchyam", "1F31D"},
    {"sonce_z_oblichchyam", "1F31E"},
    {"syayna_zirka", "1F31F"},
    {"meteor", "1F320"},
    {"termometr", "1F321"},
    {"sonce_za_hmarkoyu", "1F324"},
    {"sonce_za_hmarinoyu", "1F325"},
    {"sonce_za_doshchovoyu_hmaroyu", "1F326"},
    {"hmara_z_doshchem", "1F327"},
    {"hmara_zi_snigom", "1F328"},
    {"hmara_z_bliskavkoyu", "1F329"},
    {"tornado", "1F32A"},
    {"tuman", "1F32B"},
    {"oblichchya_shcho_dme", "1F32C"},
    {"hot_dog", "1F32D"},
    {"tako", "1F32E"},
    {"burito", "1F32F"},
    {"kashtan", "1F330"},
    {"siyanec", "1F331"},
    {"vichnozelene_derevo", "1F332"},
    {"listyane_derevo", "1F333"},
    {"palma", "1F334"},
    {"kaktus", "1F335"},
    {"gostriy_perec", "1F336"},
    {"tyulpan", "1F337"},
    {"vishneviy_cvit", "1F338"},
    {"troyanda", "1F339"},
    {"gibiskus", "1F33A"},
    {"sonyashnik", "1F33B"},
    {"zhovta_kvitka", "1F33C"},
    {"kachan_kukurudzi", "1F33D"},
    {"snipok_risu", "1F33E"},
    {"likarska_roslina", "1F33F"},
    {"chotirilistkova_konyushina", "1F340"},
    {"klenoviy_listok", "1F341"},
    {"opale_listya", "1F342"},
    {"listya_shcho_kruzhlyaie_za_vitrom", "1F343"},
    {"grib", "1F344"},
    {"tomat", "1F345"},
    {"baklazhan", "1F346"},
    {"vinograd", "1F347"},
    {"dinya", "1F348"},
    {"kavun", "1F349"},
    {"tanzherin", "1F34A"},
    {"limon", "1F34B"},
    {"banan", "1F34C"},
    {"ananas", "1F34D"},
    {"chervone_yabluko", "1F34E"},
    {"zelene_yabluko", "1F34F"},
    {"grusha", "1F350"},
    {"persik", "1F351"},
    {"vishni", "1F352"},
    {"polunicya", "1F353"},
    {"gamburger", "1F354"},
    {"pica", "1F355"},
    {"myaso_na_kistci", "1F356"},
    {"nizhka_sviyskoyi_ptici", "1F357"},
    {"risoviy_kreker", "1F358"},
    {"risova_kulka", "1F359"},
    {"prigotovaniy_ris", "1F35A"},
    {"ris_iz_kari", "1F35B"},
    {"miska_z_yakoyi_pidnimaietsya_para", "1F35C"},
    {"spageti", "1F35D"},
    {"hlib", "1F35E"},
    {"kartoplya_fri", "1F35F"},
    {"pecheniy_batat", "1F360"},
    {"risovi_kulki_na_shpazhci", "1F361"},
    {"oden", "1F362"},
    {"sushi", "1F363"},
    {"smazhena_krevetka", "1F364"},
    {"ribniy_rulet_zi_spiralnim_vizerunkom", "1F365"},
    {"morozivo_v_rizhku", "1F366"},
    {"lodova_struzhka", "1F367"},
    {"morozivo", "1F368"},
    {"ponchik", "1F369"},
    {"pechivo", "1F36A"},
    {"plitka_shokoladu", "1F36B"},
    {"cukerka", "1F36C"},
    {"lodyanik", "1F36D"},
    {"zavarniy_krem", "1F36E"},
    {"gorshchik_iz_medom", "1F36F"},
    {"tistechko", "1F370"},
    {"obid_u_korobci", "1F371"},
    {"gorshchik_z_yizheyu", "1F372"},
    {"gotuvannya_yizhi", "1F373"},
    {"videlka_ta_nizh", "1F374"},
    {"chayna_chashka_bez_vushka", "1F375"},
    {"sake", "1F376"},
    {"kelih_dlya_vina", "1F377"},
    {"bokal_dlya_kokteylyu", "1F378"},
    {"tropichniy_napiy", "1F379"},
    {"kuhol_iz_pivom", "1F37A"},
    {"kuhli_z_pivom_shcho_cokayutsya", "1F37B"},
    {"plyashechka_dlya_nemovlyati", "1F37C"},
    {"videlka_nizh_i_tarilka", "1F37D"},
    {"plyashka_z_korkom_shcho_vilitaie", "1F37E"},
    {"popkorn", "1F37F"},
    {"bant_zi_strichki", "1F380"},
    {"podarunok", "1F381"},
    {"tort_na_den_narodzhennya", "1F382"},
    {"lihtar_garbuz", "1F383"},
    {"rizdvyana_yalinka", "1F384"},
    {"santa_klaus", "1F385"},
    {"santa_klaus_kolir1", "1F385-1F3FB"},
    {"santa_klaus_kolir2", "1F385-1F3FC"},
    {"santa_klaus_kolir3", "1F385-1F3FD"},
    {"santa_klaus_kolir4", "1F385-1F3FE"},
    {"santa_klaus_kolir5", "1F385-1F3FF"},
    {"feierverki", "1F386"},
    {"bengalskiy_vogon", "1F387"},
    {"povitryana_kulka", "1F388"},
    {"hlopavka", "1F389"},
    {"kulka_z_konfeti", "1F38A"},
    {"tanabata_gilka_tandzaku", "1F38B"},
    {"shreshcheni_prapori", "1F38C"},
    {"prikrasi_na_yalinku", "1F38D"},
    {"yaponski_lyalki", "1F38E"},
    {"vitrovkaz_u_formi_koropiv", "1F38F"},
    {"kitayski_dzvinochki", "1F390"},
    {"ceremoniya_vshanuvannya_misyacya", "1F391"},
    {"shkilniy_ryukzak", "1F392"},
    {"kvadratna_akademichna_shapochka", "1F393"},
    {"viyskova_medal", "1F396"},
    {"strichka_na_zgadku", "1F397"},
    {"studiyniy_mikrofon", "1F399"},
    {"povzunok_rivnya", "1F39A"},
    {"obertovi_regulyatori", "1F39B"},
    {"kadri_kinoplivki", "1F39E"},
    {"vhidni_kvitki", "1F39F"},
    {"konik_na_karuseli", "1F3A0"},
    {"chortove_koleso", "1F3A1"},
    {"amerikanski_girki", "1F3A2"},
    {"ribacka_vudka", "1F3A3"},
    {"mikrofon", "1F3A4"},
    {"kinokamera", "1F3A5"},
    {"kinematograf", "1F3A6"},
    {"navushniki", "1F3A7"},
    {"palitra_hudozhnika", "1F3A8"},
    {"cilindr_kapelyuh", "1F3A9"},
    {"cirkove_shatro", "1F3AA"},
    {"kvitok", "1F3AB"},
    {"kinohlopavka", "1F3AC"},
    {"teatralne_mistectvo", "1F3AD"},
    {"igroviy_kontroler", "1F3AE"},
    {"mishen_iz_pryamim_vluchennyam", "1F3AF"},
    {"gralniy_avtomat", "1F3B0"},
    {"bilyardna_kulya_8", "1F3B1"},
    {"gralna_kist", "1F3B2"},
    {"kulya_i_kegli_dlya_boulingu", "1F3B3"},
    {"gralni_karti_z_kvitami_hanafuda", "1F3B4"},
    {"muzichna_nota", "1F3B5"},
    {"muzichni_noti", "1F3B6"},
    {"saksofon", "1F3B7"},
    {"gitara", "1F3B8"},
    {"muzichna_klaviatura", "1F3B9"},
    {"truba", "1F3BA"},
    {"skripka", "1F4CE"},
    {"partitura", "1F3BC"},
    {"bigova_teniska", "1F3BD"},
    {"tenisniy_myach", "1F3BE"},
    {"lizhi", "1F3BF"},
    {"basketbolniy_myach", "1F3C0"},
    {"kartatiy_prapor", "1F3C1"},
    {"snoubordist", "1F3C2"},
    {"snoubordist_kolir1", "1F3C2-1F3FB"},
    {"snoubordist_kolir2", "1F3C2-1F3FC"},
    {"snoubordist_kolir3", "1F3C2-1F3FD"},
    {"snoubordist_kolir4", "1F3C2-1F3FE"},
    {"snoubordist_kolir5", "1F3C2-1F3FF"},
    {"lyudina_shcho_bizhit", "1F3C3"},
    {"lyudina_shcho_bizhit_kolir1", "1F3C3-1F3FB"},
    {"lyudina_shcho_bizhit_kolir2", "1F3C3-1F3FC"},
    {"lyudina_shcho_bizhit_kolir3", "1F3C3-1F3FD"},
    {"lyudina_shcho_bizhit_kolir4", "1F3C3-1F3FE"},
    {"lyudina_shcho_bizhit_kolir5", "1F3C3-1F3FF"},
    {"lyudina_shcho_zaymaietsya_serfingom", "1F3C4"},
    {"lyudina_shcho_zaymaietsya_serfingom_kolir1", "1F3C4-1F3FB"},
    {"lyudina_shcho_zaymaietsya_serfingom_kolir2", "1F3C4-1F3FC"},
    {"lyudina_shcho_zaymaietsya_serfingom_kolir3", "1F3C4-1F3FD"},
    {"lyudina_shcho_zaymaietsya_serfingom_kolir4", "1F3C4-1F3FE"},
    {"lyudina_shcho_zaymaietsya_serfingom_kolir5", "1F3C4-1F3FF"},
    {"sportivna_medal", "1F3C5"},
    {"priz", "1F3C6"},
    {"kinni_peregoni", "1F3C7"},
    {"kinni_peregoni_kolir1", "1F3C7-1F3FB"},
    {"kinni_peregoni_kolir2", "1F3C7-1F3FC"},
    {"kinni_peregoni_kolir3", "1F3C7-1F3FD"},
    {"kinni_peregoni_kolir4", "1F3C7-1F3FE"},
    {"kinni_peregoni_kolir5", "1F3C7-1F3FF"},
    {"myach_dlya_amerikanskogo_futbolu", "1F3C8"},
    {"myach_dlya_regbi", "1F3C9"},
    {"lyudina_shcho_plive", "1F3CA"},
    {"lyudina_shcho_plive_kolir1", "1F3CA-1F3FB"},
    {"lyudina_shcho_plive_kolir2", "1F3CA-1F3FC"},
    {"lyudina_shcho_plive_kolir3", "1F3CA-1F3FD"},
    {"lyudina_shcho_plive_kolir4", "1F3CA-1F3FE"},
    {"lyudina_shcho_plive_kolir5", "1F3CA-1F3FF"},
    {"vazhkoatlet", "1F3CB"},
    {"vazhkoatlet_kolir1", "1F3CB-1F3FB"},
    {"vazhkoatlet_kolir2", "1F3CB-1F3FC"},
    {"vazhkoatlet_kolir3", "1F3CB-1F3FD"},
    {"vazhkoatlet_kolir4", "1F3CB-1F3FE"},
    {"vazhkoatlet_kolir5", "1F3CB-1F3FF"},
    {"lyudina_shcho_graie_v_golf", "1F3CC"},
    {"lyudina_shcho_graie_v_golf_kolir1", "1F3CC-1F3FB"},
    {"lyudina_shcho_graie_v_golf_kolir2", "1F3CC-1F3FC"},
    {"lyudina_shcho_graie_v_golf_kolir3", "1F3CC-1F3FD"},
    {"lyudina_shcho_graie_v_golf_kolir4", "1F3CC-1F3FE"},
    {"lyudina_shcho_graie_v_golf_kolir5", "1F3CC-1F3FF"},
    {"motocikl", "1F3CD"},
    {"cportivniy_avtomobil", "1F3CE"},
    {"bita_i_myach_dlya_kriketu", "1F3CF"},
    {"voleybolniy_myach", "1F3D0"},
    {"klyuchka_i_myach_dlya_hokeyu_na_travi", "1F3D1"},
    {"klyuchka_i_shayba_dlya_hokeyu", "1F3D2"},
    {"raketka_i_kulka_dlya_ping_pongu", "1F3D3"},
    {"gora_iz_zasnizhenoyu_verhivkoyu", "1F3D4"},
    {"kemping", "1F3D5"},
    {"plyazh_i_parasolka", "1F3D6"},
    {"sporudzhennya_budivli", "1F3D7"},
    {"grupa_budinkiv", "1F3D8"},
    {"miskiy_kraievid", "1F3D9"},
    {"zanedbaniy_budinok", "1F3DA"},
    {"antichna_budivlya", "1F3DB"},
    {"pustelya", "1F3DC"},
    {"bezlyudniy_ostriv", "1F3DD"},
    {"nacionalniy_park", "1F3DE"},
    {"stadion", "1F3DF"},
    {"budinok", "1F3E0"},
    {"budinok_iz_sadom", "1F3E1"},
    {"ofisna_budivlya", "1F3E2"},
    {"yaponske_poshtove_viddilennya", "1F3E3"},
    {"poshtove_viddilennya", "1F3E4"},
    {"likarnya", "1F3E5"},
    {"bank", "1F3E6"},
    {"simvol_bankomata", "1F3E7"},
    {"gotel", "1F3E8"},
    {"gotel_kohannya", "1F3E9"},
    {"cilodoboviy_magazin", "1F3EA"},
    {"shkola", "1F3EB"},
    {"univermag", "1F3EC"},
    {"fabrika", "1F3ED"},
    {"chervoniy_paperoviy_lihtar", "1F3EE"},
    {"yaponskiy_zamok", "1F3EF"},
    {"zamok", "1F3F0"},
    {"biliy_prapor", "1F3F3"},
    {"chorniy_prapor", "1F3F4"},
    {"rozeta", "1F3F5"},
    {"birka", "1F3F7"},
    {"raketka_i_volan_dlya_badmintonu", "1F3F8"},
    {"luk_i_strila", "1F3F9"},
    {"amfora", "1F3FA"},
    {"svitliy_ton_shkiri", "1F3FB"},
    {"pomirno_svitliy_ton_shkiri", "1F3FC"},
    {"pomirniy_ton_shkiri", "1F3FD"},
    {"pomirno_temniy_ton_shkiri", "1F3FE"},
    {"temniy_ton_shkiri", "1F3FF"},
    {"shchur", "1F400"},
    {"misha", "1F401"},
    {"bik", "1F402"},
    {"buyvil", "1F403"},
    {"korova", "1F404"},
    {"tigr", "1F405"},
    {"leopard", "1F406"},
    {"krolik", "1F407"},
    {"kit", "1F40B"},
    {"drakon", "1F409"},
    {"krokodil", "1F40A"},
    {"ravlik", "1F40C"},
    {"zmiya", "1F40D"},
    {"kin", "1F40E"},
    {"baran", "1F40F"},
    {"cap", "1F410"},
    {"vivcya", "1F411"},
    {"mavpa", "1F412"},
    {"piven", "1F413"},
    {"kurka", "1F414"},
    {"sobaka", "1F415"},
    {"svinya", "1F416"},
    {"vepr", "1F417"},
    {"slon", "1F418"},
    {"vosminig", "1F419"},
    {"morska_mushlya", "1F41A"},
    {"komaha", "1F41B"},
    {"muraha", "1F41C"},
    {"bdzhola", "1F41D"},
    {"sonechko", "1F41E"},
    {"riba", "1F41F"},
    {"tropichna_riba", "1F420"},
    {"riba_kulya", "1F421"},
    {"cherepaha", "1F422"},
    {"kurcha_shcho_viluplyuietsya", "1F423"},
    {"male_kurcha", "1F424"},
    {"kurcha_shcho_stoyit", "1F425"},
    {"ptah", "1F426"},
    {"pingvin", "1F427"},
    {"koala", "1F428"},
    {"pudel", "1F429"},
    {"odnogorbiy_verblyud", "1F42A"},
    {"dvogorbiy_verblyud", "1F42B"},
    {"delfin", "1F42C"},
    {"morda_mishi", "1F42D"},
    {"morda_korovi", "1F42E"},
    {"morda_tigra", "1F42F"},
    {"morda_krolya", "1F430"},
    {"morda_kota", "1F431"},
    {"golova_drakona", "1F432"},
    {"kit_shcho_puskaie_fontan", "1F433"},
    {"golova_konya", "1F434"},
    {"morda_mavpi", "1F435"},
    {"morda_sobaki", "1F436"},
    {"rilo_svini", "1F437"},
    {"zhaba", "1F438"},
    {"homyak", "1F439"},
    {"vovk", "1F43A"},
    {"vedmid", "1F43B"},
    {"panda", "1F43C"},
    {"svinyachiy_pyatak", "1F43D"},
    {"slidi_lap", "1F43E"},
    {"burunduk", "1F43F"},
    {"ochi", "1F440"},
    {"oko", "1F441"},
    {"vuho", "1F442"},
    {"vuho_kolir1", "1F442-1F3FB"},
    {"vuho_kolir2", "1F442-1F3FC"},
    {"vuho_kolir3", "1F442-1F3FD"},
    {"vuho_kolir4", "1F442-1F3FE"},
    {"vuho_kolir5", "1F442-1F3FF"},
    {"nis", "1F443"},
    {"nis_kolir1", "1F443-1F3FB"},
    {"nis_kolir2", "1F443-1F3FC"},
    {"nis_kolir3", "1F443-1F3FD"},
    {"nis_kolir4", "1F443-1F3FE"},
    {"nis_kolir5", "1F443-1F3FF"},
    {"rot", "1F444"},
    {"yazik", "1F445"},
    {"palec_tilnim_bokom_shcho_vkazuie_vgoru", "1F446"},
    {"palec_tilnim_bokom_shcho_vkazuie_vgoru_kolir1", "1F446-1F3FB"},
    {"palec_tilnim_bokom_shcho_vkazuie_vgoru_kolir2", "1F446-1F3FC"},
    {"palec_tilnim_bokom_shcho_vkazuie_vgoru_kolir3", "1F446-1F3FD"},
    {"palec_tilnim_bokom_shcho_vkazuie_vgoru_kolir4", "1F446-1F3FE"},
    {"palec_tilnim_bokom_shcho_vkazuie_vgoru_kolir5", "1F446-1F3FF"},
    {"palec_tilnim_bokom_shcho_vkazuie_vniz", "1F447"},
    {"palec_tilnim_bokom_shcho_vkazuie_vniz_kolir1", "1F447-1F3FB"},
    {"palec_tilnim_bokom_shcho_vkazuie_vniz_kolir2", "1F447-1F3FC"},
    {"palec_tilnim_bokom_shcho_vkazuie_vniz_kolir3", "1F447-1F3FD"},
    {"palec_tilnim_bokom_shcho_vkazuie_vniz_kolir4", "1F447-1F3FE"},
    {"palec_tilnim_bokom_shcho_vkazuie_vniz_kolir5", "1F447-1F3FF"},
    {"palec_tilnim_bokom_shcho_vkazuie_vlivo", "1F448"},
    {"palec_tilnim_bokom_shcho_vkazuie_vlivo_kolir1", "1F448-1F3FB"},
    {"palec_tilnim_bokom_shcho_vkazuie_vlivo_kolir2", "1F448-1F3FC"},
    {"palec_tilnim_bokom_shcho_vkazuie_vlivo_kolir3", "1F448-1F3FD"},
    {"palec_tilnim_bokom_shcho_vkazuie_vlivo_kolir4", "1F448-1F3FE"},
    {"palec_tilnim_bokom_shcho_vkazuie_vlivo_kolir5", "1F448-1F3FF"},
    {"palec_tilnim_bokom_shcho_vkazuie_vpravo", "1F449"},
    {"palec_tilnim_bokom_shcho_vkazuie_vpravo_kolir1", "1F449-1F3FB"},
    {"palec_tilnim_bokom_shcho_vkazuie_vpravo_kolir2", "1F449-1F3FC"},
    {"palec_tilnim_bokom_shcho_vkazuie_vpravo_kolir3", "1F449-1F3FD"},
    {"palec_tilnim_bokom_shcho_vkazuie_vpravo_kolir4", "1F449-1F3FE"},
    {"palec_tilnim_bokom_shcho_vkazuie_vpravo_kolir5", "1F449-1F3FF"},
    {"kulak_shcho_nasuvaietsya", "1F44A"},
    {"kulak_shcho_nasuvaietsya_kolir1", "1F44A-1F3FB"},
    {"kulak_shcho_nasuvaietsya_kolir2", "1F44A-1F3FC"},
    {"kulak_shcho_nasuvaietsya_kolir3", "1F44A-1F3FD"},
    {"kulak_shcho_nasuvaietsya_kolir4", "1F44A-1F3FE"},
    {"kulak_shcho_nasuvaietsya_kolir5", "1F44A-1F3FF"},
    {"ruka_mahaie", "1F44B"},
    {"ruka_mahaie_kolir1", "1F44B-1F3FB"},
    {"ruka_mahaie_kolir2", "1F44B-1F3FC"},
    {"ruka_mahaie_kolir3", "1F44B-1F3FD"},
    {"ruka_mahaie_kolir4", "1F44B-1F3FE"},
    {"ruka_mahaie_kolir5", "1F44B-1F3FF"},
    {"zhest_okey", "1F44C"},
    {"zhest_okey_kolir1", "1F44C-1F3FB"},
    {"zhest_okey_kolir2", "1F44C-1F3FC"},
    {"zhest_okey_kolir3", "1F44C-1F3FD"},
    {"zhest_okey_kolir4", "1F44C-1F3FE"},
    {"zhest_okey_kolir5", "1F44C-1F3FF"},
    {"veliki_palci_vgoru", "1F44D"},
    {"veliki_palci_vgoru_kolir1", "1F44D-1F3FB"},
    {"veliki_palci_vgoru_kolir2", "1F44D-1F3FC"},
    {"veliki_palci_vgoru_kolir3", "1F44D-1F3FD"},
    {"veliki_palci_vgoru_kolir4", "1F44D-1F3FE"},
    {"veliki_palci_vgoru_kolir5", "1F44D-1F3FF"},
    {"veliki_palci_vniz", "1F44E"},
    {"veliki_palci_vniz_kolir1", "1F44E-1F3FB"},
    {"veliki_palci_vniz_kolir2", "1F44E-1F3FC"},
    {"veliki_palci_vniz_kolir3", "1F44E-1F3FD"},
    {"veliki_palci_vniz_kolir4", "1F44E-1F3FE"},
    {"veliki_palci_vniz_kolir5", "1F44E-1F3FF"},
    {"ruki_shcho_aploduyut", "1F44F"},
    {"ruki_shcho_aploduyut_kolir1", "1F44F-1F3FB"},
    {"ruki_shcho_aploduyut_kolir2", "1F44F-1F3FC"},
    {"ruki_shcho_aploduyut_kolir3", "1F44F-1F3FD"},
    {"ruki_shcho_aploduyut_kolir4", "1F44F-1F3FE"},
    {"ruki_shcho_aploduyut_kolir5", "1F44F-1F3FF"},
    {"vidkriti_doloni", "1F450"},
    {"vidkriti_doloni_kolir1", "1F450-1F3FB"},
    {"vidkriti_doloni_kolir2", "1F450-1F3FC"},
    {"vidkriti_doloni_kolir3", "1F450-1F3FD"},
    {"vidkriti_doloni_kolir4", "1F450-1F3FE"},
    {"vidkriti_doloni_kolir5", "1F450-1F3FF"},
    {"korona", "1F451"},
    {"zhinochiy_kapelyushok", "1F452"},
    {"okulyari", "1F453"},
    {"kravatka", "1F454"},
    {"teniska_polo", "1F455"},
    {"dzhinsi", "1F456"},
    {"suknya", "1F457"},
    {"kimono", "1F458"},
    {"bikini", "1F459"},
    {"zhinochiy_odyag", "1F45A"},
    {"gamanec", "1F45B"},
    {"sumka", "1F45C"},
    {"klatch", "1F45D"},
    {"cholovicha_tuflya", "1F45E"},
    {"krosivka", "1F45F"},
    {"tuflya_na_visokomu_pidbori", "1F460"},
    {"zhinocha_sandaliya_z_pidborom", "1F461"},
    {"zhinochiy_chobit", "1F462"},
    {"slidi_nig", "1F463"},
    {"siluet_byustu_lyudini", "1F464"},
    {"silueti_dvoh_lyudskih_byustiv", "1F465"},
    {"hlopchik", "1F466"},
    {"hlopchik_kolir1", "1F466-1F3FB"},
    {"hlopchik_kolir2", "1F466-1F3FC"},
    {"hlopchik_kolir3", "1F466-1F3FD"},
    {"hlopchik_kolir4", "1F466-1F3FE"},
    {"hlopchik_kolir5", "1F466-1F3FF"},
    {"divchinka", "1F467"},
    {"divchinka_kolir1", "1F467-1F3FB"},
    {"divchinka_kolir2", "1F467-1F3FC"},
    {"divchinka_kolir3", "1F467-1F3FD"},
    {"divchinka_kolir4", "1F467-1F3FE"},
    {"divchinka_kolir5", "1F467-1F3FF"},
    {"cholovik", "1F468"},
    {"cholovik_kolir1", "1F468-1F3FB"},
    {"cholovik_kolir2", "1F468-1F3FC"},
    {"cholovik_kolir3", "1F468-1F3FD"},
    {"cholovik_kolir4", "1F468-1F3FE"},
    {"cholovik_kolir5", "1F468-1F3FF"},
    {"zhinka", "1F469"},
    {"zhinka_kolir1", "1F469-1F3FB"},
    {"zhinka_kolir2", "1F469-1F3FC"},
    {"zhinka_kolir3", "1F469-1F3FD"},
    {"zhinka_kolir4", "1F469-1F3FE"},
    {"zhinka_kolir5", "1F469-1F3FF"},
    {"rodina", "1F46A"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki", "1F46B"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir1", "1F46B-1F3FB"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir2", "1F46B-1F3FC"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir3", "1F46B-1F3FD"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir4", "1F46B-1F3FE"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir5", "1F46B-1F3FF"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"zhinka_y_cholovik_shcho_trimayutsya_za_ruki_kolir5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"choloviki_shcho_trimayutsya_za_ruki", "1F46C"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir1", "1F46C-1F3FB"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir2", "1F46C-1F3FC"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir3", "1F46C-1F3FD"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir4", "1F46C-1F3FE"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir5", "1F46C-1F3FF"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"choloviki_shcho_trimayutsya_za_ruki_kolir5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"zhinki_shcho_trimayutsya_za_ruki", "1F46D"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir1", "1F46D-1F3FB"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir2", "1F46D-1F3FC"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir3", "1F46D-1F3FD"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir4", "1F46D-1F3FE"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir5", "1F46D-1F3FF"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"zhinki_shcho_trimayutsya_za_ruki_kolir5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"pracivnik_policiyi", "1F46E"},
    {"pracivnik_policiyi_kolir1", "1F46E-1F3FB"},
    {"pracivnik_policiyi_kolir2", "1F46E-1F3FC"},
    {"pracivnik_policiyi_kolir3", "1F46E-1F3FD"},
    {"pracivnik_policiyi_kolir4", "1F46E-1F3FE"},
    {"pracivnik_policiyi_kolir5", "1F46E-1F3FF"},
    {"lyudi_z_krolyachimi_vuhami", "1F46F"},
    {"lyudi_z_krolyachimi_vuhami_kolir1", "1F46F-1F3FB"},
    {"lyudi_z_krolyachimi_vuhami_kolir2", "1F46F-1F3FC"},
    {"lyudi_z_krolyachimi_vuhami_kolir3", "1F46F-1F3FD"},
    {"lyudi_z_krolyachimi_vuhami_kolir4", "1F46F-1F3FE"},
    {"lyudi_z_krolyachimi_vuhami_kolir5", "1F46F-1F3FF"},
    {"lyudi_z_krolyachimi_vuhami_kolir1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"lyudi_z_krolyachimi_vuhami_kolir1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"lyudi_z_krolyachimi_vuhami_kolir1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"lyudi_z_krolyachimi_vuhami_kolir1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"lyudi_z_krolyachimi_vuhami_kolir2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"lyudi_z_krolyachimi_vuhami_kolir2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"lyudi_z_krolyachimi_vuhami_kolir2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"lyudi_z_krolyachimi_vuhami_kolir2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"lyudi_z_krolyachimi_vuhami_kolir3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"lyudi_z_krolyachimi_vuhami_kolir3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"lyudi_z_krolyachimi_vuhami_kolir3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"lyudi_z_krolyachimi_vuhami_kolir3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"lyudi_z_krolyachimi_vuhami_kolir4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"lyudi_z_krolyachimi_vuhami_kolir4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"lyudi_z_krolyachimi_vuhami_kolir4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"lyudi_z_krolyachimi_vuhami_kolir4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"lyudi_z_krolyachimi_vuhami_kolir5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"lyudi_z_krolyachimi_vuhami_kolir5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"lyudi_z_krolyachimi_vuhami_kolir5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"lyudi_z_krolyachimi_vuhami_kolir5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"lyudina_u_fati", "1F470"},
    {"lyudina_u_fati_kolir1", "1F470-1F3FB"},
    {"lyudina_u_fati_kolir2", "1F470-1F3FC"},
    {"lyudina_u_fati_kolir3", "1F470-1F3FD"},
    {"lyudina_u_fati_kolir4", "1F470-1F3FE"},
    {"lyudina_u_fati_kolir5", "1F470-1F3FF"},
    {"lyudina_z_bilyavim_volossyam", "1F471"},
    {"lyudina_z_bilyavim_volossyam_kolir1", "1F471-1F3FB"},
    {"lyudina_z_bilyavim_volossyam_kolir2", "1F471-1F3FC"},
    {"lyudina_z_bilyavim_volossyam_kolir3", "1F471-1F3FD"},
    {"lyudina_z_bilyavim_volossyam_kolir4", "1F471-1F3FE"},
    {"lyudina_z_bilyavim_volossyam_kolir5", "1F471-1F3FF"},
    {"cholovik_u_kitayskiy_shapci", "1F472"},
    {"cholovik_u_kitayskiy_shapci_kolir1", "1F472-1F3FB"},
    {"cholovik_u_kitayskiy_shapci_kolir2", "1F472-1F3FC"},
    {"cholovik_u_kitayskiy_shapci_kolir3", "1F472-1F3FD"},
    {"cholovik_u_kitayskiy_shapci_kolir4", "1F472-1F3FE"},
    {"cholovik_u_kitayskiy_shapci_kolir5", "1F472-1F3FF"},
    {"lyudina_v_tyurbani", "1F473"},
    {"lyudina_v_tyurbani_kolir1", "1F473-1F3FB"},
    {"lyudina_v_tyurbani_kolir2", "1F473-1F3FC"},
    {"lyudina_v_tyurbani_kolir3", "1F473-1F3FD"},
    {"lyudina_v_tyurbani_kolir4", "1F473-1F3FE"},
    {"lyudina_v_tyurbani_kolir5", "1F473-1F3FF"},
    {"litniy_cholovik", "1F474"},
    {"litniy_cholovik_kolir1", "1F474-1F3FB"},
    {"litniy_cholovik_kolir2", "1F474-1F3FC"},
    {"litniy_cholovik_kolir3", "1F474-1F3FD"},
    {"litniy_cholovik_kolir4", "1F474-1F3FE"},
    {"litniy_cholovik_kolir5", "1F474-1F3FF"},
    {"litnya_zhinka", "1F475"},
    {"litnya_zhinka_kolir1", "1F475-1F3FB"},
    {"litnya_zhinka_kolir2", "1F475-1F3FC"},
    {"litnya_zhinka_kolir3", "1F475-1F3FD"},
    {"litnya_zhinka_kolir4", "1F475-1F3FE"},
    {"litnya_zhinka_kolir5", "1F475-1F3FF"},
    {"nemovlya", "1F476"},
    {"nemovlya_kolir1", "1F476-1F3FB"},
    {"nemovlya_kolir2", "1F476-1F3FC"},
    {"nemovlya_kolir3", "1F476-1F3FD"},
    {"nemovlya_kolir4", "1F476-1F3FE"},
    {"nemovlya_kolir5", "1F476-1F3FF"},
    {"budivelnik", "1F477"},
    {"budivelnik_kolir1", "1F477-1F3FB"},
    {"budivelnik_kolir2", "1F477-1F3FC"},
    {"budivelnik_kolir3", "1F477-1F3FD"},
    {"budivelnik_kolir4", "1F477-1F3FE"},
    {"budivelnik_kolir5", "1F477-1F3FF"},
    {"princesa", "1F478"},
    {"princesa_kolir1", "1F478-1F3FB"},
    {"princesa_kolir2", "1F478-1F3FC"},
    {"princesa_kolir3", "1F478-1F3FD"},
    {"princesa_kolir4", "1F478-1F3FE"},
    {"princesa_kolir5", "1F478-1F3FF"},
    {"veleten_lyudozher", "1F479"},
    {"goblin", "1F47A"},
    {"privid", "1F47B"},
    {"yangolyatko", "1F47C"},
    {"yangolyatko_kolir1", "1F47C-1F3FB"},
    {"yangolyatko_kolir2", "1F47C-1F3FC"},
    {"yangolyatko_kolir3", "1F47C-1F3FD"},
    {"yangolyatko_kolir4", "1F47C-1F3FE"},
    {"yangolyatko_kolir5", "1F47C-1F3FF"},
    {"pribulec", "1F47D"},
    {"inshoplanetniy_monstr", "1F47E"},
    {"serdite_oblichchya_z_rogami", "1F47F"},
    {"cherep", "1F480"},
    {"lyudina_shcho_prostyagaie_ruku_dopomogi", "1F481"},
    {"lyudina_shcho_prostyagaie_ruku_dopomogi_kolir1", "1F481-1F3FB"},
    {"lyudina_shcho_prostyagaie_ruku_dopomogi_kolir2", "1F481-1F3FC"},
    {"lyudina_shcho_prostyagaie_ruku_dopomogi_kolir3", "1F481-1F3FD"},
    {"lyudina_shcho_prostyagaie_ruku_dopomogi_kolir4", "1F481-1F3FE"},
    {"lyudina_shcho_prostyagaie_ruku_dopomogi_kolir5", "1F481-1F3FF"},
    {"varta", "1F482"},
    {"varta_kolir1", "1F482-1F3FB"},
    {"varta_kolir2", "1F482-1F3FC"},
    {"varta_kolir3", "1F482-1F3FD"},
    {"varta_kolir4", "1F482-1F3FE"},
    {"varta_kolir5", "1F482-1F3FF"},
    {"zhinka_shcho_tancyuie", "1F483"},
    {"zhinka_shcho_tancyuie_kolir1", "1F483-1F3FB"},
    {"zhinka_shcho_tancyuie_kolir2", "1F483-1F3FC"},
    {"zhinka_shcho_tancyuie_kolir3", "1F483-1F3FD"},
    {"zhinka_shcho_tancyuie_kolir4", "1F483-1F3FE"},
    {"zhinka_shcho_tancyuie_kolir5", "1F483-1F3FF"},
    {"pomada", "1F484"},
    {"lak_dlya_nigtiv", "1F485"},
    {"lak_dlya_nigtiv_kolir1", "1F485-1F3FB"},
    {"lak_dlya_nigtiv_kolir2", "1F485-1F3FC"},
    {"lak_dlya_nigtiv_kolir3", "1F485-1F3FD"},
    {"lak_dlya_nigtiv_kolir4", "1F485-1F3FE"},
    {"lak_dlya_nigtiv_kolir5", "1F485-1F3FF"},
    {"lyudina_yakiy_roblyat_masazh_golovi", "1F486"},
    {"lyudina_yakiy_roblyat_masazh_golovi_kolir1", "1F486-1F3FB"},
    {"lyudina_yakiy_roblyat_masazh_golovi_kolir2", "1F486-1F3FC"},
    {"lyudina_yakiy_roblyat_masazh_golovi_kolir3", "1F486-1F3FD"},
    {"lyudina_yakiy_roblyat_masazh_golovi_kolir4", "1F486-1F3FE"},
    {"lyudina_yakiy_roblyat_masazh_golovi_kolir5", "1F486-1F3FF"},
    {"lyudina_yakiy_roblyat_strizhku", "1F487"},
    {"lyudina_yakiy_roblyat_strizhku_kolir1", "1F487-1F3FB"},
    {"lyudina_yakiy_roblyat_strizhku_kolir2", "1F487-1F3FC"},
    {"lyudina_yakiy_roblyat_strizhku_kolir3", "1F487-1F3FD"},
    {"lyudina_yakiy_roblyat_strizhku_kolir4", "1F487-1F3FE"},
    {"lyudina_yakiy_roblyat_strizhku_kolir5", "1F487-1F3FF"},
    {"viviska_perukarya_smugastiy_stovp", "1F488"},
    {"shpric", "1F489"},
    {"pigulka", "1F48A"},
    {"slid_cilunku", "1F48B"},
    {"lyubovniy_list", "1F48C"},
    {"kabluchka", "1F48D"},
    {"koshtovniy_kamin", "1F48E"},
    {"cholovik_i_zhinka_shcho_ciluyutsya", "1F48F"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir1", "1F48F-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir2", "1F48F-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir3", "1F48F-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir4", "1F48F-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir5", "1F48F-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_kolir5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"buket", "1F490"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi", "1F491"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir1", "1F491-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir2", "1F491-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir3", "1F491-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir4", "1F491-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir5", "1F491-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_kolir5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"vesillya", "1F492"},
    {"serce_shcho_bietsya", "1F493"},
    {"rozbite_serce", "1F494"},
    {"dva_sercya", "1F495"},
    {"serce_shcho_vibliskuie", "1F496"},
    {"serce_shcho_zrostaie", "1F497"},
    {"serce_probite_striloyu", "1F498"},
    {"sinie_serce", "1F499"},
    {"zelene_serce", "1F49A"},
    {"zhovte_serce", "1F49B"},
    {"purpurove_serce", "1F49C"},
    {"serce_zi_strichkoyu", "1F49D"},
    {"sercya_shcho_obertayutsya", "1F49E"},
    {"prikrasa_iz_sercem", "1F49F"},
    {"romb_iz_krapkoyu", "1F4A0"},
    {"lampochka", "1F4A1"},
    {"simvol_gnivu", "1F4A2"},
    {"bomba", "1F4A3"},
    {"umovne_poznachennya_snu", "1F4A4"},
    {"zitknennya", "1F4A5"},
    {"krapli_potu", "1F4A6"},
    {"kraplya", "1F4A7"},
    {"poriv_vitru", "1F4A8"},
    {"usmihnena_kupa_layna", "1F4A9"},
    {"napruzheniy_biceps", "1F4AA"},
    {"napruzheniy_biceps_kolir1", "1F4AA-1F3FB"},
    {"napruzheniy_biceps_kolir2", "1F4AA-1F3FC"},
    {"napruzheniy_biceps_kolir3", "1F4AA-1F3FD"},
    {"napruzheniy_biceps_kolir4", "1F4AA-1F3FE"},
    {"napruzheniy_biceps_kolir5", "1F4AA-1F3FF"},
    {"simvol_zapamorochennya", "1F4AB"},
    {"tekstova_bulka", "1F4AC"},
    {"bulka_rozdumiv", "1F4AD"},
    {"bila_kvitka", "1F4AE"},
    {"cto_baliv", "1F4AF"},
    {"mishok_groshey", "1F4B0"},
    {"obmin_valyut", "1F4B1"},
    {"znak_dolara", "1F4B2"},
    {"kreditna_kartka", "1F4B3"},
    {"ienovi_banknoti", "1F4B4"},
    {"dolarovi_banknoti", "1F4B5"},
    {"ievrovi_banknoti", "1F4B6"},
    {"funtovi_banknoti", "1F4B7"},
    {"groshi_z_krilami", "1F4B8"},
    {"grafik_zrostannya_i_znak_ieni", "1F4B9"},
    {"sidinnya", "1F4BA"},
    {"noutbuk", "1F4BB"},
    {"keys", "1F4BC"},
    {"kompyuterniy_disk", "1F4BD"},
    {"disketa", "1F4BE"},
    {"kompakt_disk", "1F4BF"},
    {"dvd_disk", "1F4C0"},
    {"papka_dlya_fayliv", "1F4C1"},
    {"vidkrita_papka_dlya_fayliv", "1F4C2"},
    {"storinka_iz_zavitkom", "1F4C3"},
    {"storinka_licevoyu_storonoyu", "1F4C4"},
    {"kalendar", "1F4C5"},
    {"vidrivniy_kalendar", "1F4C6"},
    {"kartoteka", "1F4C7"},
    {"vishidniy_grafik", "1F4C8"},
    {"nizhidniy_grafik", "1F4C9"},
    {"gistograma", "1F4CA"},
    {"planshet_dlya_arkushiv", "1F4CB"},
    {"kancelyarska_knopka", "1F4CC"},
    {"shpilka_z_krugloyu_golivkoyu", "1F4CD"},
    {"pryama_liniyka", "1F4CF"},
    {"trikutna_liniyka", "1F4D0"},
    {"mitki_zakladok", "1F4D1"},
    {"grosbuh", "1F4D2"},
    {"zoshit_dlya_tvoriv", "1F4D3"},
    {"zapisnik_u_dekorativniy_obkladinci", "1F4D4"},
    {"zakrita_kniga", "1F4D5"},
    {"vidkrita_kniga", "1F4D6"},
    {"zeleniy_pidruchnik", "1F4D7"},
    {"siniy_pidruchnik", "1F4D8"},
    {"oranzheviy_pidruchnik", "1F4D9"},
    {"knigi", "1F4DA"},
    {"beydzh_z_imenem", "1F4DB"},
    {"suviy", "1F4DC"},
    {"pamyatka", "1F4DD"},
    {"telefonna_sluhavka", "1F4DE"},
    {"peydzher", "1F4DF"},
    {"faks", "1F4E0"},
    {"suputnikova_antena", "1F4E1"},
    {"guchnomovec", "1F4E2"},
    {"megafon", "1F4E3"},
    {"lotok_vihidnoyi_skrinki", "1F4E4"},
    {"lotok_vhidnoyi_skrinki", "1F4E5"},
    {"pakunok", "1F4E6"},
    {"elektronniy_list", "1F4E7"},
    {"konvert_vhidnogo_lista", "1F4E8"},
    {"konvert_zi_strilkoyu", "1F4E9"},
    {"zakrita_poshtova_skrinka_z_opushchenim_praporcem", "1F4EA"},
    {"zakrita_poshtova_skrinka_z_pidnyatim_praporcem", "1F4EB"},
    {"vidkrita_poshtova_skrinka_z_pidnyatim_praporcem", "1F4EC"},
    {"vidkrita_poshtova_skrinka_z_opushchenim_praporcem", "1F4ED"},
    {"poshtova_skrinka", "1F4EE"},
    {"poshtoviy_rizhok", "1F4EF"},
    {"gazeta", "1F4F0"},
    {"mobilniy_telefon", "1F4F1"},
    {"mobilniy_telefon_zi_strilkoyu", "1F4F2"},
    {"vibrorezhim", "1F4F3"},
    {"mobilniy_telefon_vimkneno", "1F4F4"},
    {"mobilni_telefoni_zaboroneno", "1F4F5"},
    {"riven_signalu", "1F4F6"},
    {"fotoaparat", "1F4F7"},
    {"fotoaparat_zi_spalahom", "1F4F8"},
    {"videokamera", "1F4F9"},
    {"televizor", "1F4FA"},
    {"radiopriymach", "1F4FB"},
    {"videokaseta", "1F4FC"},
    {"kinoproektor", "1F4FD"},
    {"molitovni_chotki", "1F4FF"},
    {"knopka_tasuvannya_kompoziciy", "1F500"},
    {"knopka_povtoru", "1F501"},
    {"knopka_odnorazovogo_povtoru", "1F502"},
    {"vertikalni_strilki_za_hodom_godinnika", "1F503"},
    {"knopka_strilok_proti_hodu_godinnika", "1F504"},
    {"knopka_zmenshennya_yaskravosti", "1F505"},
    {"knopka_zbilshennya_yaskravosti", "1F506"},
    {"zaglusheniy_dinamik", "1F507"},
    {"nizka_guchnist_dinamika", "1F508"},
    {"serednya_guchnist_dinamika", "1F509"},
    {"visokiy_riven_guchnosti_dinamika", "1F50A"},
    {"batareya", "1F50B"},
    {"shtepsel", "1F50C"},
    {"zbilshuvalne_sklo_z_nahilom_ulivo", "1F50D"},
    {"zbilshuvalne_sklo_z_nahilom_upravo", "1F50E"},
    {"zamok_i_chornilna_ruchka", "1F50F"},
    {"zamkneniy_zamok_iz_klyuchem", "1F510"},
    {"klyuch", "1F511"},
    {"zamkneniy_zamok", "1F512"},
    {"vidimkneniy_zamok", "1F513"},
    {"dzvinok", "1F514"},
    {"zaglusheniy_dzvinok", "1F515"},
    {"zakladka", "1F516"},
    {"simvol_ziednannya_lanok", "1F517"},
    {"peremikach", "1F518"},
    {"strilka_nazad_back", "1F519"},
    {"strilka_end", "1F51A"},
    {"strilka_on", "1F51B"},
    {"strilka_soon", "1F51C"},
    {"strilka_top", "1F51D"},
    {"vhid_osobam_do_18_rokiv_zaboroneniy", "1F51E"},
    {"klavisha_10", "1F51F"},
    {"vvedennya_velikimi_latinskimi_literami", "1F520"},
    {"vvedennya_malimi_latinskimi_literami", "1F521"},
    {"vvedennya_ciframi", "1F522"},
    {"vvedennya_simvoliv", "1F523"},
    {"vvedennya_latinskimi_literami", "1F524"},
    {"vogon", "1F525"},
    {"lihtarik", "1F526"},
    {"gaykoviy_klyuch", "1F527"},
    {"molotok", "1F528"},
    {"gayka_ta_bolt", "1F529"},
    {"kuhonniy_nizh", "1F52A"},
    {"vodyaniy_pistolet", "1F52B"},
    {"mikroskop", "1F52C"},
    {"teleskop", "1F52D"},
    {"krishtaleva_kulya", "1F52E"},
    {"shestikutna_zirka_z_krapkoyu", "1F52F"},
    {"yaponskiy_simvol_pochatkivec", "1F530"},
    {"trizub", "1F531"},
    {"kvadratna_knopka_z_chornoyu_ramkoyu", "1F532"},
    {"kvadratna_knopka_z_biloyu_ramkoyu", "1F533"},
    {"chervoniy_krug", "1F534"},
    {"ciniy_krug", "1F535"},
    {"velikiy_oranzheviy_romb", "1F536"},
    {"velikiy_siniy_romb", "1F537"},
    {"maliy_oranzheviy_romb", "1F538"},
    {"maliy_siniy_romb", "1F539"},
    {"chervoniy_trikutnik_napravleniy_ugoru", "1F53A"},
    {"chervoniy_trikutnik_napravleniy_uniz", "1F53B"},
    {"knopka_dogori", "1F53C"},
    {"knopka_donizu", "1F53D"},
    {"zvuk_om", "1F549"},
    {"golub", "1F54A"},
    {"kaaba", "1F54B"},
    {"mechet", "1F54C"},
    {"sinagoga", "1F54D"},
    {"menora", "1F54E"},
    {"persha_godina", "1F550"},
    {"druga_godina", "1F551"},
    {"tretya_godina", "1F552"},
    {"chetverta_godina", "1F553"},
    {"pyata_godina", "1F554"},
    {"shosta_godina", "1F555"},
    {"soma_godina", "1F556"},
    {"vosma_godina", "1F557"},
    {"devyata_godina", "1F558"},
    {"desyata_godina", "1F559"},
    {"odinadcyata_godina", "1F55A"},
    {"dvanadcyata_godina", "1F55B"},
    {"piv_na_drugu", "1F55C"},
    {"piv_na_tretyu", "1F55D"},
    {"piv_na_chetvertu", "1F55E"},
    {"piv_na_pyatu", "1F55F"},
    {"piv_na_shostu", "1F560"},
    {"piv_na_somu", "1F561"},
    {"piv_na_vosmu", "1F562"},
    {"piv_na_devyatu", "1F563"},
    {"piv_na_desyatu", "1F564"},
    {"piv_na_odinadcyatu", "1F565"},
    {"piv_na_dvanadcyatu", "1F566"},
    {"piv_na_pershu", "1F567"},
    {"svichka", "1F56F"},
    {"kaminniy_godinnik", "1F570"},
    {"lunka", "1F573"},
    {"cholovik_u_dilovomu_kostyumi_shcho_levituie", "1F574"},
    {"cholovik_u_dilovomu_kostyumi_shcho_levituie_kolir1", "1F574-1F3FB"},
    {"cholovik_u_dilovomu_kostyumi_shcho_levituie_kolir2", "1F574-1F3FC"},
    {"cholovik_u_dilovomu_kostyumi_shcho_levituie_kolir3", "1F574-1F3FD"},
    {"cholovik_u_dilovomu_kostyumi_shcho_levituie_kolir4", "1F574-1F3FE"},
    {"cholovik_u_dilovomu_kostyumi_shcho_levituie_kolir5", "1F574-1F3FF"},
    {"detektiv", "1F575"},
    {"detektiv_kolir1", "1F575-1F3FB"},
    {"detektiv_kolir2", "1F575-1F3FC"},
    {"detektiv_kolir3", "1F575-1F3FD"},
    {"detektiv_kolir4", "1F575-1F3FE"},
    {"detektiv_kolir5", "1F575-1F3FF"},
    {"soncezahisni_okulyari", "1F576"},
    {"pavuk", "1F577"},
    {"pavutinnya", "1F578"},
    {"dzhoystik", "1F579"},
    {"cholovik_shcho_tancyuie", "1F57A"},
    {"cholovik_shcho_tancyuie_kolir1", "1F57A-1F3FB"},
    {"cholovik_shcho_tancyuie_kolir2", "1F57A-1F3FC"},
    {"cholovik_shcho_tancyuie_kolir3", "1F57A-1F3FD"},
    {"cholovik_shcho_tancyuie_kolir4", "1F57A-1F3FE"},
    {"cholovik_shcho_tancyuie_kolir5", "1F57A-1F3FF"},
    {"zchepleni_skripki", "1F587"},
    {"kulkova_ruchka", "1F58A"},
    {"chornilna_ruchka", "1F58B"},
    {"penzel", "1F58C"},
    {"pastel", "1F58D"},
    {"pidnyata_ruka_z_rozvedenimi_palcyami", "1F590"},
    {"pidnyata_ruka_z_rozvedenimi_palcyami_kolir1", "1F590-1F3FB"},
    {"pidnyata_ruka_z_rozvedenimi_palcyami_kolir2", "1F590-1F3FC"},
    {"pidnyata_ruka_z_rozvedenimi_palcyami_kolir3", "1F590-1F3FD"},
    {"pidnyata_ruka_z_rozvedenimi_palcyami_kolir4", "1F590-1F3FE"},
    {"pidnyata_ruka_z_rozvedenimi_palcyami_kolir5", "1F590-1F3FF"},
    {"seredniy_palec", "1F595"},
    {"seredniy_palec_kolir1", "1F595-1F3FB"},
    {"seredniy_palec_kolir2", "1F595-1F3FC"},
    {"seredniy_palec_kolir3", "1F595-1F3FD"},
    {"seredniy_palec_kolir4", "1F595-1F3FE"},
    {"seredniy_palec_kolir5", "1F595-1F3FF"},
    {"vulkanske_vitannya", "1F596"},
    {"vulkanske_vitannya_kolir1", "1F596-1F3FB"},
    {"vulkanske_vitannya_kolir2", "1F596-1F3FC"},
    {"vulkanske_vitannya_kolir3", "1F596-1F3FD"},
    {"vulkanske_vitannya_kolir4", "1F596-1F3FE"},
    {"vulkanske_vitannya_kolir5", "1F596-1F3FF"},
    {"chorne_serce", "1F5A4"},
    {"kompyuter", "1F5A5"},
    {"printer", "1F5A8"},
    {"kompyuterna_misha", "1F5B1"},
    {"trekbol", "1F5B2"},
    {"kartina_v_ramci", "1F5BC"},
    {"rozdilniki_v_kartoteci", "1F5C2"},
    {"skrinka_dlya_kartok", "1F5C3"},
    {"shafa_dlya_dokumentiv", "1F5C4"},
    {"smittieviy_koshik", "1F5D1"},
    {"bloknot_na_spirali", "1F5D2"},
    {"kalendar_na_spirali", "1F5D3"},
    {"zatiskach", "1F5DC"},
    {"starovinniy_klyuch", "1F5DD"},
    {"skruchena_gazeta", "1F5DE"},
    {"kindzhal", "1F5E1"},
    {"golova_shcho_govorit", "1F5E3"},
    {"hmarinka_z_tekstom_zliva", "1F5E8"},
    {"bulka_iz_serditoyu_replikoyu", "1F5EF"},
    {"urna_dlya_golosuvannya_i_byuleten", "1F5F3"},
    {"karta_svitu", "1F5FA"},
    {"gora_fudzi", "1F5FB"},
    {"tokiyska_vezha", "1F5FC"},
    {"statuya_svobodi", "1F5FD"},
    {"karta_yaponiyi", "1F5FE"},
    {"statuya_z_ostrova_pashi", "1F5FF"},
    {"shiroko_vsmihnene_oblichchya", "1F600"},
    {"osyayno_vsmihnene_oblichchya_y_ochi", "1F601"},
    {"oblichchya_v_slozah_radosti", "1F602"},
    {"shiroko_vsmihnene_oblichchya_z_ochishchami", "1F603"},
    {"shiroko_vsmihnene_oblichchya_z_primruzhenimi_ochima", "1F604"},
    {"shiroko_vsmihnene_oblichchya_z_potom", "1F605"},
    {"oblichchya_iz_shirokoyu_posmishkoyu_y_zamruzhenimi_ochima", "1F606"},
    {"usmihnene_oblichchya_z_nimbom", "1F607"},
    {"usmihnene_oblichchya_z_rogami", "1F608"},
    {"oblichchya_shcho_pidmorguie", "1F609"},
    {"usmihnene_oblichchya_y_ochi", "1F60A"},
    {"usmihnene_oblichchya_shcho_oblizuie_gubi", "1F60B"},
    {"zaspokoiene_oblichchya", "1F60C"},
    {"usmihnene_oblichchya_z_ochima_sercyami", "1F60D"},
    {"usmihnene_oblichchya_v_temnih_okulyarah", "1F60E"},
    {"samovdovolene_oblichchya", "1F60F"},
    {"oblichchya_z_neytralnim_virazom", "1F610"},
    {"oblichchya_bez_emociy", "1F611"},
    {"nevdovolene_oblichchya", "1F612"},
    {"ponurene_oblichchya_v_potu", "1F613"},
    {"zamislene_oblichchya", "1F614"},
    {"spantelichene_oblichchya", "1F615"},
    {"zbentezhene_oblichchya", "1F616"},
    {"oblichchya_shcho_ciluie", "1F617"},
    {"oblichchya_shcho_nadsilaie_cilunok", "1F618"},
    {"oblichchya_z_primruzhenimi_ochima_shcho_ciluie", "1F619"},
    {"oblichchya_iz_zaplyushchenimi_ochima_shcho_ciluie", "1F61A"},
    {"oblichchya_z_visunutim_yazikom", "1F61B"},
    {"oblichchya_shcho_pidmorguie_i_pokazuie_yazika", "1F61C"},
    {"oblichchya_z_visunutim_yazikom_i_zaplyushchenimi_ochima", "1F61D"},
    {"rozcharovane_oblichchya", "1F61E"},
    {"sturbovane_oblichchya", "1F61F"},
    {"serdite_oblichchya", "1F620"},
    {"oblichchya_shcho_duietsya", "1F621"},
    {"oblichchya_shcho_plache", "1F622"},
    {"nevidstupne_oblichchya", "1F623"},
    {"oblichchya_shcho_serdito_vidihaie", "1F624"},
    {"cumne_ale_vtishene_oblichchya", "1F625"},
    {"pohmure_oblichchya_z_vidkritim_rotom", "1F626"},
    {"strazhdenne_oblichchya", "1F627"},
    {"nalyakane_oblichchya", "1F628"},
    {"zamuchene_oblichchya", "1F629"},
    {"zaspane_oblichchya", "1F62A"},
    {"stomlene_oblichchya", "1F62B"},
    {"oblichchya_z_grimasoyu", "1F62C"},
    {"oblichchya_shcho_guchno_ridaie", "1F62D"},
    {"oblichchya_z_vidkritim_rotom", "1F62E"},
    {"zdivovane_i_zbentezhene_oblichchya_z_brovami", "1F62F"},
    {"zanepokoiene_oblichchya_z_potom", "1F630"},
    {"oblichchya_shcho_krichit_iz_perelyaku", "1F631"},
    {"vrazhene_oblichchya", "1F632"},
    {"zasharile_oblichchya", "1F633"},
    {"oblichchya_shcho_spit", "1F634"},
    {"zapamorochene_oblichchya", "1F635"},
    {"oblichchya_bez_rota", "1F636"},
    {"oblichchya_v_medichniy_masci", "1F637"},
    {"shiroko_vsmihneniy_kit_z_usmihom_v_ochah", "1F638"},
    {"kit_zi_slozami_radosti", "1F639"},
    {"shiroko_vsmihneniy_kit", "1F63A"},
    {"usmihneniy_kit_z_ochima_yak_sercya", "1F63B"},
    {"kit_iz_krivoyu_posmishkoyu", "1F63C"},
    {"kit_shcho_ciluie", "1F63D"},
    {"serditiy_kit", "1F63E"},
    {"kit_shcho_plache", "1F63F"},
    {"shokovaniy_kit", "1F640"},
    {"trohi_pohmure_oblichchya", "1F641"},
    {"trohi_vsmihnene_oblichchya", "1F642"},
    {"oblichchya_dogori_drigom", "1F643"},
    {"oblichchya_iz_zakochenimi_ochima", "1F644"},
    {"lyudina_shcho_zhestom_pokazuie_ni", "1F645"},
    {"lyudina_shcho_zhestom_pokazuie_ni_kolir1", "1F645-1F3FB"},
    {"lyudina_shcho_zhestom_pokazuie_ni_kolir2", "1F645-1F3FC"},
    {"lyudina_shcho_zhestom_pokazuie_ni_kolir3", "1F645-1F3FD"},
    {"lyudina_shcho_zhestom_pokazuie_ni_kolir4", "1F645-1F3FE"},
    {"lyudina_shcho_zhestom_pokazuie_ni_kolir5", "1F645-1F3FF"},
    {"lyudina_shcho_zhestom_pokazuie_okey", "1F646"},
    {"lyudina_shcho_zhestom_pokazuie_okey_kolir1", "1F646-1F3FB"},
    {"lyudina_shcho_zhestom_pokazuie_okey_kolir2", "1F646-1F3FC"},
    {"lyudina_shcho_zhestom_pokazuie_okey_kolir3", "1F646-1F3FD"},
    {"lyudina_shcho_zhestom_pokazuie_okey_kolir4", "1F646-1F3FE"},
    {"lyudina_shcho_zhestom_pokazuie_okey_kolir5", "1F646-1F3FF"},
    {"lyudina_shcho_vklonyaietsya", "1F647"},
    {"lyudina_shcho_vklonyaietsya_kolir1", "1F647-1F3FB"},
    {"lyudina_shcho_vklonyaietsya_kolir2", "1F647-1F3FC"},
    {"lyudina_shcho_vklonyaietsya_kolir3", "1F647-1F3FD"},
    {"lyudina_shcho_vklonyaietsya_kolir4", "1F647-1F3FE"},
    {"lyudina_shcho_vklonyaietsya_kolir5", "1F647-1F3FF"},
    {"mavpa_shcho_ne_bachit", "1F648"},
    {"mavpa_shcho_ne_chuie", "1F649"},
    {"mavpa_shcho_ne_govorit", "1F64A"},
    {"lyudina_z_pidnyatoyu_rukoyu", "1F64B"},
    {"lyudina_z_pidnyatoyu_rukoyu_kolir1", "1F64B-1F3FB"},
    {"lyudina_z_pidnyatoyu_rukoyu_kolir2", "1F64B-1F3FC"},
    {"lyudina_z_pidnyatoyu_rukoyu_kolir3", "1F64B-1F3FD"},
    {"lyudina_z_pidnyatoyu_rukoyu_kolir4", "1F64B-1F3FE"},
    {"lyudina_z_pidnyatoyu_rukoyu_kolir5", "1F64B-1F3FF"},
    {"ruki_pidnyati_dlya_svyatkuvannya", "1F64C"},
    {"ruki_pidnyati_dlya_svyatkuvannya_kolir1", "1F64C-1F3FB"},
    {"ruki_pidnyati_dlya_svyatkuvannya_kolir2", "1F64C-1F3FC"},
    {"ruki_pidnyati_dlya_svyatkuvannya_kolir3", "1F64C-1F3FD"},
    {"ruki_pidnyati_dlya_svyatkuvannya_kolir4", "1F64C-1F3FE"},
    {"ruki_pidnyati_dlya_svyatkuvannya_kolir5", "1F64C-1F3FF"},
    {"pohmura_lyudina", "1F64D"},
    {"pohmura_lyudina_kolir1", "1F64D-1F3FB"},
    {"pohmura_lyudina_kolir2", "1F64D-1F3FC"},
    {"pohmura_lyudina_kolir3", "1F64D-1F3FD"},
    {"pohmura_lyudina_kolir4", "1F64D-1F3FE"},
    {"pohmura_lyudina_kolir5", "1F64D-1F3FF"},
    {"lyudina_shcho_duietsya", "1F64E"},
    {"lyudina_shcho_duietsya_kolir1", "1F64E-1F3FB"},
    {"lyudina_shcho_duietsya_kolir2", "1F64E-1F3FC"},
    {"lyudina_shcho_duietsya_kolir3", "1F64E-1F3FD"},
    {"lyudina_shcho_duietsya_kolir4", "1F64E-1F3FE"},
    {"lyudina_shcho_duietsya_kolir5", "1F64E-1F3FF"},
    {"ruki_stisnuti_razom", "1F64F"},
    {"ruki_stisnuti_razom_kolir1", "1F64F-1F3FB"},
    {"ruki_stisnuti_razom_kolir2", "1F64F-1F3FC"},
    {"ruki_stisnuti_razom_kolir3", "1F64F-1F3FD"},
    {"ruki_stisnuti_razom_kolir4", "1F64F-1F3FE"},
    {"ruki_stisnuti_razom_kolir5", "1F64F-1F3FF"},
    {"raketa", "1F680"},
    {"gelikopter", "1F681"},
    {"paroviy_lokomotiv", "1F682"},
    {"vagon", "1F683"},
    {"shvidkisniy_potyag", "1F684"},
    {"shvidkisniy_potyag_z_okruglim_nosom", "1F685"},
    {"potyag", "1F686"},
    {"metro", "1F687"},
    {"shvidkisniy_tramvay", "1F688"},
    {"stanciya", "1F689"},
    {"tramvay", "1F68A"},
    {"vagon_tramvaya", "1F68B"},
    {"avtobus", "1F68C"},
    {"avtobus_shcho_nablizhaietsya", "1F68D"},
    {"troleybus", "1F68E"},
    {"avtobusna_zupinka", "1F68F"},
    {"mikroavtobus", "1F690"},
    {"avtomobil_shvidkoyi_dopomogi", "1F691"},
    {"pozhezhna_mashina", "1F692"},
    {"policeyska_mashina", "1F693"},
    {"policeyska_mashina_shcho_nablizhaietsya", "1F694"},
    {"taksi", "1F695"},
    {"taksi_shcho_nablizhaietsya", "1F696"},
    {"avtomobil", "1F697"},
    {"avtomobil_shcho_nablizhaietsya", "1F698"},
    {"pozashlyahovik", "1F699"},
    {"vantazhivka_dlya_dostavki", "1F69A"},
    {"vantazhivka_z_prichepom", "1F69B"},
    {"traktor", "1F69C"},
    {"monoreyka", "1F69D"},
    {"zaliznicya_v_gorah", "1F69E"},
    {"pidvisna_monoreyka", "1F69F"},
    {"kanatna_doroga_v_gorah", "1F6A0"},
    {"kanatna_doroga", "1F6A1"},
    {"korabel", "1F6A2"},
    {"lyudina_shcho_vesluie_na_chovni", "1F6A3"},
    {"lyudina_shcho_vesluie_na_chovni_kolir1", "1F6A3-1F3FB"},
    {"lyudina_shcho_vesluie_na_chovni_kolir2", "1F6A3-1F3FC"},
    {"lyudina_shcho_vesluie_na_chovni_kolir3", "1F6A3-1F3FD"},
    {"lyudina_shcho_vesluie_na_chovni_kolir4", "1F6A3-1F3FE"},
    {"lyudina_shcho_vesluie_na_chovni_kolir5", "1F6A3-1F3FF"},
    {"shvidkohidniy_kater", "1F6A4"},
    {"gorizontalniy_svitlofor", "1F6A5"},
    {"vertikalniy_svitlofor", "1F6A6"},
    {"znak_remontnih_robit", "1F6A7"},
    {"probliskoviy_mayachok", "1F6A8"},
    {"trikutniy_prapor", "1F6A9"},
    {"dveri", "1F6AA"},
    {"zaboroneno", "1F6AB"},
    {"sigareta", "1F6AC"},
    {"paliti_zaboroneno", "1F6AD"},
    {"znak_kidati_smittya_v_urnu", "1F6AE"},
    {"smititi_zaboroneno", "1F6AF"},
    {"pitna_voda", "1F6B0"},
    {"nepridatna_dlya_pittya_voda", "1F6B1"},
    {"velosiped", "1F6B2"},
    {"velosipedi_zaboroneno", "1F6B3"},
    {"lyudina_shcho_yide_na_velosipedi", "1F6B4"},
    {"lyudina_shcho_yide_na_velosipedi_kolir1", "1F6B4-1F3FB"},
    {"lyudina_shcho_yide_na_velosipedi_kolir2", "1F6B4-1F3FC"},
    {"lyudina_shcho_yide_na_velosipedi_kolir3", "1F6B4-1F3FD"},
    {"lyudina_shcho_yide_na_velosipedi_kolir4", "1F6B4-1F3FE"},
    {"lyudina_shcho_yide_na_velosipedi_kolir5", "1F6B4-1F3FF"},
    {"lyudina_shcho_yide_na_girskomu_velosipedi", "1F6B5"},
    {"lyudina_shcho_yide_na_girskomu_velosipedi_kolir1", "1F6B5-1F3FB"},
    {"lyudina_shcho_yide_na_girskomu_velosipedi_kolir2", "1F6B5-1F3FC"},
    {"lyudina_shcho_yide_na_girskomu_velosipedi_kolir3", "1F6B5-1F3FD"},
    {"lyudina_shcho_yide_na_girskomu_velosipedi_kolir4", "1F6B5-1F3FE"},
    {"lyudina_shcho_yide_na_girskomu_velosipedi_kolir5", "1F6B5-1F3FF"},
    {"lyudina_shcho_yde_pishki", "1F6B6"},
    {"lyudina_shcho_yde_pishki_kolir1", "1F6B6-1F3FB"},
    {"lyudina_shcho_yde_pishki_kolir2", "1F6B6-1F3FC"},
    {"lyudina_shcho_yde_pishki_kolir3", "1F6B6-1F3FD"},
    {"lyudina_shcho_yde_pishki_kolir4", "1F6B6-1F3FE"},
    {"lyudina_shcho_yde_pishki_kolir5", "1F6B6-1F3FF"},
    {"pishohidniy_ruh_zaboroneno", "1F6B7"},
    {"poperedzhuvalniy_znak_diti", "1F6B8"},
    {"cholovicha_vbiralnya", "1F6B9"},
    {"zhinocha_vbiralnya", "1F6BA"},
    {"gromadskiy_tualet", "1F6BB"},
    {"simvol_nemovlyati", "1F6BC"},
    {"unitaz", "1F6BD"},
    {"simvol_ubiralni_wc", "1F6BE"},
    {"dush", "1F6BF"},
    {"lyudina_shcho_priymaie_vannu", "1F6C0"},
    {"lyudina_shcho_priymaie_vannu_kolir1", "1F6C0-1F3FB"},
    {"lyudina_shcho_priymaie_vannu_kolir2", "1F6C0-1F3FC"},
    {"lyudina_shcho_priymaie_vannu_kolir3", "1F6C0-1F3FD"},
    {"lyudina_shcho_priymaie_vannu_kolir4", "1F6C0-1F3FE"},
    {"lyudina_shcho_priymaie_vannu_kolir5", "1F6C0-1F3FF"},
    {"vanna", "1F6C1"},
    {"pasportniy_kontrol", "1F6C2"},
    {"mitnicya", "1F6C3"},
    {"vidacha_bagazhu", "1F6C4"},
    {"kamera_shovu", "1F6C5"},
    {"kanapa_y_torsher", "1F6CB"},
    {"lyudina_v_lizhku", "1F6CC"},
    {"lyudina_v_lizhku_kolir1", "1F6CC-1F3FB"},
    {"lyudina_v_lizhku_kolir2", "1F6CC-1F3FC"},
    {"lyudina_v_lizhku_kolir3", "1F6CC-1F3FD"},
    {"lyudina_v_lizhku_kolir4", "1F6CC-1F3FE"},
    {"lyudina_v_lizhku_kolir5", "1F6CC-1F3FF"},
    {"sumki_dlya_pokupok", "1F6CD"},
    {"gotelniy_dzvonik", "1F6CE"},
    {"lizhko", "1F6CF"},
    {"kultova_sporuda", "1F6D0"},
    {"vosmikutniy_chervoniy_znak", "1F6D1"},
    {"magazinniy_vizok", "1F6D2"},
    {"induyistskiy_hram", "1F6D5"},
    {"hizha", "1F6D6"},
    {"lift", "1F6D7"},
    {"zsuv", "1F6D8"},
    {"bezdrotoviy", "1F6DC"},
    {"dityacha_girka", "1F6DD"},
    {"koleso", "1F6DE"},
    {"ryativniy_krug", "1F6DF"},
    {"molotok_i_gaykoviy_klyuch", "1F6E0"},
    {"shchit", "1F6E1"},
    {"bochka_dlya_nafti", "1F6E2"},
    {"avtomagistral", "1F6E3"},
    {"zaliznichna_koliya", "1F6E4"},
    {"motorniy_choven", "1F6E5"},
    {"maliy_litak", "1F6E9"},
    {"litak_zlitaie", "1F6EB"},
    {"litak_sidaie", "1F6EC"},
    {"suputnik", "1F6F0"},
    {"pasazhirskiy_korabel", "1F6F3"},
    {"samokat", "1F6F4"},
    {"moped", "1F6F5"},
    {"kanoe", "1F6F6"},
    {"sani", "1F6F7"},
    {"letyucha_tarilka", "1F6F8"},
    {"skeytbord", "1F6F9"},
    {"avtoriksha", "1F6FA"},
    {"avtomobil_pikap", "1F6FB"},
    {"rolikoviy_kovzan", "1F6FC"},
    {"oranzheviy_krug", "1F7E0"},
    {"zhovtiy_krug", "1F7E1"},
    {"zeleniy_krug", "1F7E2"},
    {"fioletoviy_krug", "1F7E3"},
    {"korichneviy_krug", "1F7E4"},
    {"chervoniy_kvadrat", "1F7E5"},
    {"siniy_kvadrat", "1F7E6"},
    {"oranzheviy_kvadrat", "1F7E7"},
    {"zhovtiy_kvadrat", "1F7E8"},
    {"zeleniy_kvadrat", "1F7E9"},
    {"fioletoviy_kvadrat", "1F7EA"},
    {"korichneviy_kvadrat", "1F7EB"},
    {"znak_tochnoyi_rivnosti", "1F7F0"},
    {"zvedeni_palci", "1F90C"},
    {"zvedeni_palci_kolir1", "1F90C-1F3FB"},
    {"zvedeni_palci_kolir2", "1F90C-1F3FC"},
    {"zvedeni_palci_kolir3", "1F90C-1F3FD"},
    {"zvedeni_palci_kolir4", "1F90C-1F3FE"},
    {"zvedeni_palci_kolir5", "1F90C-1F3FF"},
    {"bile_serce", "1F90D"},
    {"korichneve_serce", "1F90E"},
    {"zhest_malo", "1F90F"},
    {"zhest_malo_kolir1", "1F90F-1F3FB"},
    {"zhest_malo_kolir2", "1F90F-1F3FC"},
    {"zhest_malo_kolir3", "1F90F-1F3FD"},
    {"zhest_malo_kolir4", "1F90F-1F3FE"},
    {"zhest_malo_kolir5", "1F90F-1F3FF"},
    {"oblichchya_iz_zastibkoyu_zamist_rota", "1F910"},
    {"oblichchya_z_grishmi_zamist_ochey_ta_yazika", "1F911"},
    {"oblichchya_z_termometrom", "1F912"},
    {"oblichchya_vseznayka", "1F913"},
    {"oblichchya_v_rozdumah", "1F914"},
    {"oblichchya_z_golovnoyu_povyazkoyu", "1F915"},
    {"robot", "1F916"},
    {"shchaslive_oblichchya_z_rukami_shcho_obiymayut", "1F917"},
    {"zhest_rogi", "1F918"},
    {"zhest_rogi_kolir1", "1F918-1F3FB"},
    {"zhest_rogi_kolir2", "1F918-1F3FC"},
    {"zhest_rogi_kolir3", "1F918-1F3FD"},
    {"zhest_rogi_kolir4", "1F918-1F3FE"},
    {"zhest_rogi_kolir5", "1F918-1F3FF"},
    {"zhest_podzvoni_meni", "1F919"},
    {"zhest_podzvoni_meni_kolir1", "1F919-1F3FB"},
    {"zhest_podzvoni_meni_kolir2", "1F919-1F3FC"},
    {"zhest_podzvoni_meni_kolir3", "1F919-1F3FD"},
    {"zhest_podzvoni_meni_kolir4", "1F919-1F3FE"},
    {"zhest_podzvoni_meni_kolir5", "1F919-1F3FF"},
    {"pidnyata_ruka_tilnoyu_storonoyu", "1F91A"},
    {"pidnyata_ruka_tilnoyu_storonoyu_kolir1", "1F91A-1F3FB"},
    {"pidnyata_ruka_tilnoyu_storonoyu_kolir2", "1F91A-1F3FC"},
    {"pidnyata_ruka_tilnoyu_storonoyu_kolir3", "1F91A-1F3FD"},
    {"pidnyata_ruka_tilnoyu_storonoyu_kolir4", "1F91A-1F3FE"},
    {"pidnyata_ruka_tilnoyu_storonoyu_kolir5", "1F91A-1F3FF"},
    {"kulak_ulivo", "1F91B"},
    {"kulak_ulivo_kolir1", "1F91B-1F3FB"},
    {"kulak_ulivo_kolir2", "1F91B-1F3FC"},
    {"kulak_ulivo_kolir3", "1F91B-1F3FD"},
    {"kulak_ulivo_kolir4", "1F91B-1F3FE"},
    {"kulak_ulivo_kolir5", "1F91B-1F3FF"},
    {"kulak_upravo", "1F91C"},
    {"kulak_upravo_kolir1", "1F91C-1F3FB"},
    {"kulak_upravo_kolir2", "1F91C-1F3FC"},
    {"kulak_upravo_kolir3", "1F91C-1F3FD"},
    {"kulak_upravo_kolir4", "1F91C-1F3FE"},
    {"kulak_upravo_kolir5", "1F91C-1F3FF"},
    {"rukostiskannya", "1F91D"},
    {"rukostiskannya_kolir1", "1F91D-1F3FB"},
    {"rukostiskannya_kolir2", "1F91D-1F3FC"},
    {"rukostiskannya_kolir3", "1F91D-1F3FD"},
    {"rukostiskannya_kolir4", "1F91D-1F3FE"},
    {"rukostiskannya_kolir5", "1F91D-1F3FF"},
    {"rukostiskannya_kolir1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"rukostiskannya_kolir1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"rukostiskannya_kolir1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"rukostiskannya_kolir1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"rukostiskannya_kolir2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"rukostiskannya_kolir2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"rukostiskannya_kolir2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"rukostiskannya_kolir2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"rukostiskannya_kolir3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"rukostiskannya_kolir3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"rukostiskannya_kolir3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"rukostiskannya_kolir3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"rukostiskannya_kolir4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"rukostiskannya_kolir4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"rukostiskannya_kolir4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"rukostiskannya_kolir4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"rukostiskannya_kolir5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"rukostiskannya_kolir5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"rukostiskannya_kolir5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"rukostiskannya_kolir5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"shreshcheni_palci", "1F91E"},
    {"shreshcheni_palci_kolir1", "1F91E-1F3FB"},
    {"shreshcheni_palci_kolir2", "1F91E-1F3FC"},
    {"shreshcheni_palci_kolir3", "1F91E-1F3FD"},
    {"shreshcheni_palci_kolir4", "1F91E-1F3FE"},
    {"shreshcheni_palci_kolir5", "1F91E-1F3FF"},
    {"zhest_kohayu_tebe", "1F91F"},
    {"zhest_kohayu_tebe_kolir1", "1F91F-1F3FB"},
    {"zhest_kohayu_tebe_kolir2", "1F91F-1F3FC"},
    {"zhest_kohayu_tebe_kolir3", "1F91F-1F3FD"},
    {"zhest_kohayu_tebe_kolir4", "1F91F-1F3FE"},
    {"zhest_kohayu_tebe_kolir5", "1F91F-1F3FF"},
    {"oblichchya_u_kovboyskomu_kapelyusi", "1F920"},
    {"oblichchya_klouna", "1F921"},
    {"oblichchya_yake_nudit", "1F922"},
    {"kachaietsya_vid_smihu", "1F923"},
    {"oblichchya_shcho_puskaie_slinu", "1F924"},
    {"oblichchya_shcho_breshe", "1F925"},
    {"lyudina_z_doloneyu_do_oblichchya", "1F926"},
    {"lyudina_z_doloneyu_do_oblichchya_kolir1", "1F926-1F3FB"},
    {"lyudina_z_doloneyu_do_oblichchya_kolir2", "1F926-1F3FC"},
    {"lyudina_z_doloneyu_do_oblichchya_kolir3", "1F926-1F3FD"},
    {"lyudina_z_doloneyu_do_oblichchya_kolir4", "1F926-1F3FE"},
    {"lyudina_z_doloneyu_do_oblichchya_kolir5", "1F926-1F3FF"},
    {"oblichchya_shcho_chhaie", "1F927"},
    {"oblichchya_z_pidnyatimi_brovami", "1F928"},
    {"ochi_zirochki", "1F929"},
    {"bozhevilne_oblichchya", "1F92A"},
    {"oblichchya_shcho_cikaie", "1F92B"},
    {"oblichchya_iz_simvolami_v_roti", "1F92C"},
    {"oblichchya_shcho_prikrivaie_rota_rukoyu", "1F92D"},
    {"oblichchya_shcho_blyuie", "1F92E"},
    {"golova_shcho_vibuhaie", "1F92F"},
    {"vagitna_zhinka", "1F930"},
    {"vagitna_zhinka_kolir1", "1F930-1F3FB"},
    {"vagitna_zhinka_kolir2", "1F930-1F3FC"},
    {"vagitna_zhinka_kolir3", "1F930-1F3FD"},
    {"vagitna_zhinka_kolir4", "1F930-1F3FE"},
    {"vagitna_zhinka_kolir5", "1F930-1F3FF"},
    {"grudne_vigodovuvannya", "1F931"},
    {"grudne_vigodovuvannya_kolir1", "1F931-1F3FB"},
    {"grudne_vigodovuvannya_kolir2", "1F931-1F3FC"},
    {"grudne_vigodovuvannya_kolir3", "1F931-1F3FD"},
    {"grudne_vigodovuvannya_kolir4", "1F931-1F3FE"},
    {"grudne_vigodovuvannya_kolir5", "1F931-1F3FF"},
    {"doloni_pidnyati_razom", "1F932"},
    {"doloni_pidnyati_razom_kolir1", "1F932-1F3FB"},
    {"doloni_pidnyati_razom_kolir2", "1F932-1F3FC"},
    {"doloni_pidnyati_razom_kolir3", "1F932-1F3FD"},
    {"doloni_pidnyati_razom_kolir4", "1F932-1F3FE"},
    {"doloni_pidnyati_razom_kolir5", "1F932-1F3FF"},
    {"selfi", "1F933"},
    {"selfi_kolir1", "1F933-1F3FB"},
    {"selfi_kolir2", "1F933-1F3FC"},
    {"selfi_kolir3", "1F933-1F3FD"},
    {"selfi_kolir4", "1F933-1F3FE"},
    {"selfi_kolir5", "1F933-1F3FF"},
    {"princ", "1F934"},
    {"princ_kolir1", "1F934-1F3FB"},
    {"princ_kolir2", "1F934-1F3FC"},
    {"princ_kolir3", "1F934-1F3FD"},
    {"princ_kolir4", "1F934-1F3FE"},
    {"princ_kolir5", "1F934-1F3FF"},
    {"lyudina_v_smokingu", "1F935"},
    {"lyudina_v_smokingu_kolir1", "1F935-1F3FB"},
    {"lyudina_v_smokingu_kolir2", "1F935-1F3FC"},
    {"lyudina_v_smokingu_kolir3", "1F935-1F3FD"},
    {"lyudina_v_smokingu_kolir4", "1F935-1F3FE"},
    {"lyudina_v_smokingu_kolir5", "1F935-1F3FF"},
    {"pani_klaus", "1F936"},
    {"pani_klaus_kolir1", "1F936-1F3FB"},
    {"pani_klaus_kolir2", "1F936-1F3FC"},
    {"pani_klaus_kolir3", "1F936-1F3FD"},
    {"pani_klaus_kolir4", "1F936-1F3FE"},
    {"pani_klaus_kolir5", "1F936-1F3FF"},
    {"lyudina_shcho_znizuie_plechima", "1F937"},
    {"lyudina_shcho_znizuie_plechima_kolir1", "1F937-1F3FB"},
    {"lyudina_shcho_znizuie_plechima_kolir2", "1F937-1F3FC"},
    {"lyudina_shcho_znizuie_plechima_kolir3", "1F937-1F3FD"},
    {"lyudina_shcho_znizuie_plechima_kolir4", "1F937-1F3FE"},
    {"lyudina_shcho_znizuie_plechima_kolir5", "1F937-1F3FF"},
    {"lyudina_shcho_perevertaietsya_kolesom", "1F938"},
    {"lyudina_shcho_perevertaietsya_kolesom_kolir1", "1F938-1F3FB"},
    {"lyudina_shcho_perevertaietsya_kolesom_kolir2", "1F938-1F3FC"},
    {"lyudina_shcho_perevertaietsya_kolesom_kolir3", "1F938-1F3FD"},
    {"lyudina_shcho_perevertaietsya_kolesom_kolir4", "1F938-1F3FE"},
    {"lyudina_shcho_perevertaietsya_kolesom_kolir5", "1F938-1F3FF"},
    {"lyudina_shcho_zhonglyuie", "1F939"},
    {"lyudina_shcho_zhonglyuie_kolir1", "1F939-1F3FB"},
    {"lyudina_shcho_zhonglyuie_kolir2", "1F939-1F3FC"},
    {"lyudina_shcho_zhonglyuie_kolir3", "1F939-1F3FD"},
    {"lyudina_shcho_zhonglyuie_kolir4", "1F939-1F3FE"},
    {"lyudina_shcho_zhonglyuie_kolir5", "1F939-1F3FF"},
    {"fehtuvalnik", "1F93A"},
    {"borci", "1F93C"},
    {"borci_kolir1", "1F93C-1F3FB"},
    {"borci_kolir2", "1F93C-1F3FC"},
    {"borci_kolir3", "1F93C-1F3FD"},
    {"borci_kolir4", "1F93C-1F3FE"},
    {"borci_kolir5", "1F93C-1F3FF"},
    {"borci_kolir1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"borci_kolir1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"borci_kolir1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"borci_kolir1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"borci_kolir2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"borci_kolir2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"borci_kolir2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"borci_kolir2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"borci_kolir3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"borci_kolir3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"borci_kolir3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"borci_kolir3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"borci_kolir4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"borci_kolir4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"borci_kolir4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"borci_kolir4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"borci_kolir5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"borci_kolir5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"borci_kolir5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"borci_kolir5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"lyudina_shcho_graie_u_vaterpolo", "1F93D"},
    {"lyudina_shcho_graie_u_vaterpolo_kolir1", "1F93D-1F3FB"},
    {"lyudina_shcho_graie_u_vaterpolo_kolir2", "1F93D-1F3FC"},
    {"lyudina_shcho_graie_u_vaterpolo_kolir3", "1F93D-1F3FD"},
    {"lyudina_shcho_graie_u_vaterpolo_kolir4", "1F93D-1F3FE"},
    {"lyudina_shcho_graie_u_vaterpolo_kolir5", "1F93D-1F3FF"},
    {"lyudina_shcho_graie_u_gandbol", "1F93E"},
    {"lyudina_shcho_graie_u_gandbol_kolir1", "1F93E-1F3FB"},
    {"lyudina_shcho_graie_u_gandbol_kolir2", "1F93E-1F3FC"},
    {"lyudina_shcho_graie_u_gandbol_kolir3", "1F93E-1F3FD"},
    {"lyudina_shcho_graie_u_gandbol_kolir4", "1F93E-1F3FE"},
    {"lyudina_shcho_graie_u_gandbol_kolir5", "1F93E-1F3FF"},
    {"maska_z_trubkoyu", "1F93F"},
    {"privyala_troyanda", "1F940"},
    {"baraban", "1F941"},
    {"dzvin_bokaliv", "1F942"},
    {"bokal_viski", "1F943"},
    {"lozhka", "1F944"},
    {"sitka_vorit", "1F945"},
    {"zolota_medal", "1F947"},
    {"sribna_medal", "1F948"},
    {"bronzova_medal", "1F949"},
    {"bokserska_rukavichka", "1F94A"},
    {"odyag_dlya_iedinoborstv", "1F94B"},
    {"kamyana_shayba_dlya_kerlingu", "1F94C"},
    {"stik_dlya_lakrosu", "1F94D"},
    {"softbolniy_myach", "1F94E"},
    {"litayuchiy_disk", "1F94F"},
    {"kruasan", "1F950"},
    {"avokado", "1F951"},
    {"ogirok", "1F952"},
    {"bekon", "1F953"},
    {"kartoplya", "1F954"},
    {"morkva", "1F955"},
    {"baget", "1F956"},
    {"zeleniy_salat", "1F957"},
    {"kastrulya_z_yizheyu", "1F958"},
    {"pita", "1F959"},
    {"yayce", "1F95A"},
    {"sklyanka_moloka", "1F95B"},
    {"arahis", "1F95C"},
    {"kivi", "1F95D"},
    {"mlinci", "1F95E"},
    {"varenik", "1F95F"},
    {"pechivo_z_peredbachennyam", "1F960"},
    {"korobka_z_yizheyu_na_vinos", "1F961"},
    {"palichki_dlya_yizhi", "1F962"},
    {"miska_z_lozhkoyu", "1F963"},
    {"stakan_iz_solominkoyu", "1F964"},
    {"kokos", "1F965"},
    {"brokoli", "1F966"},
    {"pirig", "1F967"},
    {"krendel", "1F968"},
    {"myasniy_filey", "1F969"},
    {"sandvich", "1F96A"},
    {"konservi", "1F96B"},
    {"listya_salatu", "1F96C"},
    {"mango", "1F96D"},
    {"munkeyk", "1F96E"},
    {"beygl", "1F96F"},
    {"usmihnene_oblichchya_iz_serdechkami", "1F970"},
    {"oblichchya_shcho_pozihaie", "1F971"},
    {"usmishka_v_slozah", "1F972"},
    {"svyatkove_oblichchya", "1F973"},
    {"ochmanile_oblichchya", "1F974"},
    {"rozpashile_oblichchya", "1F975"},
    {"zamerzle_oblichchya", "1F976"},
    {"nindzya", "1F977"},
    {"nindzya_kolir1", "1F977-1F3FB"},
    {"nindzya_kolir2", "1F977-1F3FC"},
    {"nindzya_kolir3", "1F977-1F3FD"},
    {"nindzya_kolir4", "1F977-1F3FE"},
    {"nindzya_kolir5", "1F977-1F3FF"},
    {"zamaskovane_oblichchya", "1F978"},
    {"oblichchya_shcho_ledve_strimuie_slozi", "1F979"},
    {"blagalne_oblichchya", "1F97A"},
    {"sari", "1F97B"},
    {"laboratorniy_halat", "1F97C"},
    {"zahisni_okulyari", "1F97D"},
    {"pohidni_chereviki", "1F97E"},
    {"tufli_bez_pidboriv", "1F97F"},
    {"krab", "1F980"},
    {"lev", "1F981"},
    {"skorpion", "1F982"},
    {"indik", "1F983"},
    {"iedinorig", "1F984"},
    {"orel", "1F985"},
    {"kachka", "1F986"},
    {"kazhan", "1F987"},
    {"akula", "1F988"},
    {"sova", "1F989"},
    {"lis", "1F98A"},
    {"metelik", "1F98B"},
    {"olen", "1F98C"},
    {"gorila", "1F98D"},
    {"yashchirka", "1F98E"},
    {"nosorig", "1F98F"},
    {"krevetka", "1F990"},
    {"kalmar", "1F991"},
    {"zhirafa", "1F992"},
    {"zebra", "1F993"},
    {"yizhak", "1F994"},
    {"zavropod", "1F995"},
    {"tiranozavr", "1F996"},
    {"cvirkun", "1F997"},
    {"kenguru", "1F998"},
    {"lama", "1F999"},
    {"pavich", "1F99A"},
    {"begemot", "1F99B"},
    {"papuga", "1F99C"},
    {"ienot", "1F99D"},
    {"omar", "1F99E"},
    {"komar", "1F99F"},
    {"mikrob", "1F9A0"},
    {"borsuk", "1F9A1"},
    {"lebid", "1F9A2"},
    {"mamont", "1F9A3"},
    {"dront", "1F9A4"},
    {"linivec", "1F9A5"},
    {"vidra", "1F9A6"},
    {"orangutang", "1F9A7"},
    {"skuns", "1F9A8"},
    {"flamingo", "1F9A9"},
    {"ustricya", "1F9AA"},
    {"bober", "1F9AB"},
    {"bizon", "1F9AC"},
    {"tyulen", "1F9AD"},
    {"sobaka_povodir", "1F9AE"},
    {"taktilna_trostina", "1F9AF"},
    {"rude_volossya", "1F9B0"},
    {"kucheryave_volossya", "1F9B1"},
    {"lisina", "1F9B2"},
    {"sive_volossya", "1F9B3"},
    {"kistka", "1F9B4"},
    {"noga", "1F9B5"},
    {"noga_kolir1", "1F9B5-1F3FB"},
    {"noga_kolir2", "1F9B5-1F3FC"},
    {"noga_kolir3", "1F9B5-1F3FD"},
    {"noga_kolir4", "1F9B5-1F3FE"},
    {"noga_kolir5", "1F9B5-1F3FF"},
    {"stupnya", "1F9B6"},
    {"stupnya_kolir1", "1F9B6-1F3FB"},
    {"stupnya_kolir2", "1F9B6-1F3FC"},
    {"stupnya_kolir3", "1F9B6-1F3FD"},
    {"stupnya_kolir4", "1F9B6-1F3FE"},
    {"stupnya_kolir5", "1F9B6-1F3FF"},
    {"zub", "1F9B7"},
    {"supergeroy", "1F9B8"},
    {"supergeroy_kolir1", "1F9B8-1F3FB"},
    {"supergeroy_kolir2", "1F9B8-1F3FC"},
    {"supergeroy_kolir3", "1F9B8-1F3FD"},
    {"supergeroy_kolir4", "1F9B8-1F3FE"},
    {"supergeroy_kolir5", "1F9B8-1F3FF"},
    {"superzlodiy", "1F9B9"},
    {"superzlodiy_kolir1", "1F9B9-1F3FB"},
    {"superzlodiy_kolir2", "1F9B9-1F3FC"},
    {"superzlodiy_kolir3", "1F9B9-1F3FD"},
    {"superzlodiy_kolir4", "1F9B9-1F3FE"},
    {"superzlodiy_kolir5", "1F9B9-1F3FF"},
    {"signalniy_zhilet", "1F9BA"},
    {"vuho_zi_sluhovim_aparatom", "1F9BB"},
    {"vuho_zi_sluhovim_aparatom_kolir1", "1F9BB-1F3FB"},
    {"vuho_zi_sluhovim_aparatom_kolir2", "1F9BB-1F3FC"},
    {"vuho_zi_sluhovim_aparatom_kolir3", "1F9BB-1F3FD"},
    {"vuho_zi_sluhovim_aparatom_kolir4", "1F9BB-1F3FE"},
    {"vuho_zi_sluhovim_aparatom_kolir5", "1F9BB-1F3FF"},
    {"kolisniy_vizok_z_elektroprivodom", "1F9BC"},
    {"kolisniy_vizok_iz_ruchnim_privodom", "1F9BD"},
    {"mehanichna_ruka", "1F9BE"},
    {"mehanichna_noga", "1F9BF"},
    {"kusen_siru", "1F9C0"},
    {"kapkeyk", "1F9C1"},
    {"sil", "1F9C2"},
    {"paketik_iz_napoiem", "1F9C3"},
    {"chasnik", "1F9C4"},
    {"cibulya", "1F9C5"},
    {"falafel", "1F9C6"},
    {"vaflya", "1F9C7"},
    {"maslo", "1F9C8"},
    {"mate", "1F9C9"},
    {"kubik_lodu", "1F9CA"},
    {"bulbashkoviy_chay", "1F9CB"},
    {"trol", "1F9CC"},
    {"lyudina_shcho_stoyit", "1F9CD"},
    {"lyudina_shcho_stoyit_kolir1", "1F9CD-1F3FB"},
    {"lyudina_shcho_stoyit_kolir2", "1F9CD-1F3FC"},
    {"lyudina_shcho_stoyit_kolir3", "1F9CD-1F3FD"},
    {"lyudina_shcho_stoyit_kolir4", "1F9CD-1F3FE"},
    {"lyudina_shcho_stoyit_kolir5", "1F9CD-1F3FF"},
    {"lyudina_navkolishkah", "1F9CE"},
    {"lyudina_navkolishkah_kolir1", "1F9CE-1F3FB"},
    {"lyudina_navkolishkah_kolir2", "1F9CE-1F3FC"},
    {"lyudina_navkolishkah_kolir3", "1F9CE-1F3FD"},
    {"lyudina_navkolishkah_kolir4", "1F9CE-1F3FE"},
    {"lyudina_navkolishkah_kolir5", "1F9CE-1F3FF"},
    {"lyudina_z_porushennyam_sluhu", "1F9CF"},
    {"lyudina_z_porushennyam_sluhu_kolir1", "1F9CF-1F3FB"},
    {"lyudina_z_porushennyam_sluhu_kolir2", "1F9CF-1F3FC"},
    {"lyudina_z_porushennyam_sluhu_kolir3", "1F9CF-1F3FD"},
    {"lyudina_z_porushennyam_sluhu_kolir4", "1F9CF-1F3FE"},
    {"lyudina_z_porushennyam_sluhu_kolir5", "1F9CF-1F3FF"},
    {"oblichchya_v_monokli", "1F9D0"},
    {"lyudina", "1F9D1"},
    {"lyudina_kolir1", "1F9D1-1F3FB"},
    {"lyudina_kolir2", "1F9D1-1F3FC"},
    {"lyudina_kolir3", "1F9D1-1F3FD"},
    {"lyudina_kolir4", "1F9D1-1F3FE"},
    {"lyudina_kolir5", "1F9D1-1F3FF"},
    {"ditina", "1F9D2"},
    {"ditina_kolir1", "1F9D2-1F3FB"},
    {"ditina_kolir2", "1F9D2-1F3FC"},
    {"ditina_kolir3", "1F9D2-1F3FD"},
    {"ditina_kolir4", "1F9D2-1F3FE"},
    {"ditina_kolir5", "1F9D2-1F3FF"},
    {"litnya_lyudina", "1F9D3"},
    {"litnya_lyudina_kolir1", "1F9D3-1F3FB"},
    {"litnya_lyudina_kolir2", "1F9D3-1F3FC"},
    {"litnya_lyudina_kolir3", "1F9D3-1F3FD"},
    {"litnya_lyudina_kolir4", "1F9D3-1F3FE"},
    {"litnya_lyudina_kolir5", "1F9D3-1F3FF"},
    {"lyudina_z_borodoyu", "1F9D4"},
    {"lyudina_z_borodoyu_kolir1", "1F9D4-1F3FB"},
    {"lyudina_z_borodoyu_kolir2", "1F9D4-1F3FC"},
    {"lyudina_z_borodoyu_kolir3", "1F9D4-1F3FD"},
    {"lyudina_z_borodoyu_kolir4", "1F9D4-1F3FE"},
    {"lyudina_z_borodoyu_kolir5", "1F9D4-1F3FF"},
    {"zhinka_v_hustini", "1F9D5"},
    {"zhinka_v_hustini_kolir1", "1F9D5-1F3FB"},
    {"zhinka_v_hustini_kolir2", "1F9D5-1F3FC"},
    {"zhinka_v_hustini_kolir3", "1F9D5-1F3FD"},
    {"zhinka_v_hustini_kolir4", "1F9D5-1F3FE"},
    {"zhinka_v_hustini_kolir5", "1F9D5-1F3FF"},
    {"lyudina_v_parkomu_primishchenni", "1F9D6"},
    {"lyudina_v_parkomu_primishchenni_kolir1", "1F9D6-1F3FB"},
    {"lyudina_v_parkomu_primishchenni_kolir2", "1F9D6-1F3FC"},
    {"lyudina_v_parkomu_primishchenni_kolir3", "1F9D6-1F3FD"},
    {"lyudina_v_parkomu_primishchenni_kolir4", "1F9D6-1F3FE"},
    {"lyudina_v_parkomu_primishchenni_kolir5", "1F9D6-1F3FF"},
    {"lyudina_shcho_lize_skeleyu", "1F9D7"},
    {"lyudina_shcho_lize_skeleyu_kolir1", "1F9D7-1F3FB"},
    {"lyudina_shcho_lize_skeleyu_kolir2", "1F9D7-1F3FC"},
    {"lyudina_shcho_lize_skeleyu_kolir3", "1F9D7-1F3FD"},
    {"lyudina_shcho_lize_skeleyu_kolir4", "1F9D7-1F3FE"},
    {"lyudina_shcho_lize_skeleyu_kolir5", "1F9D7-1F3FF"},
    {"lyudina_v_pozi_lotosa", "1F9D8"},
    {"lyudina_v_pozi_lotosa_kolir1", "1F9D8-1F3FB"},
    {"lyudina_v_pozi_lotosa_kolir2", "1F9D8-1F3FC"},
    {"lyudina_v_pozi_lotosa_kolir3", "1F9D8-1F3FD"},
    {"lyudina_v_pozi_lotosa_kolir4", "1F9D8-1F3FE"},
    {"lyudina_v_pozi_lotosa_kolir5", "1F9D8-1F3FF"},
    {"mag", "1F9D9"},
    {"mag_kolir1", "1F9D9-1F3FB"},
    {"mag_kolir2", "1F9D9-1F3FC"},
    {"mag_kolir3", "1F9D9-1F3FD"},
    {"mag_kolir4", "1F9D9-1F3FE"},
    {"mag_kolir5", "1F9D9-1F3FF"},
    {"feya", "1F9DA"},
    {"feya_kolir1", "1F9DA-1F3FB"},
    {"feya_kolir2", "1F9DA-1F3FC"},
    {"feya_kolir3", "1F9DA-1F3FD"},
    {"feya_kolir4", "1F9DA-1F3FE"},
    {"feya_kolir5", "1F9DA-1F3FF"},
    {"vampir", "1F9DB"},
    {"vampir_kolir1", "1F9DB-1F3FB"},
    {"vampir_kolir2", "1F9DB-1F3FC"},
    {"vampir_kolir3", "1F9DB-1F3FD"},
    {"vampir_kolir4", "1F9DB-1F3FE"},
    {"vampir_kolir5", "1F9DB-1F3FF"},
    {"kazkova_vodyana_istota", "1F9DC"},
    {"kazkova_vodyana_istota_kolir1", "1F9DC-1F3FB"},
    {"kazkova_vodyana_istota_kolir2", "1F9DC-1F3FC"},
    {"kazkova_vodyana_istota_kolir3", "1F9DC-1F3FD"},
    {"kazkova_vodyana_istota_kolir4", "1F9DC-1F3FE"},
    {"kazkova_vodyana_istota_kolir5", "1F9DC-1F3FF"},
    {"elf", "1F9DD"},
    {"elf_kolir1", "1F9DD-1F3FB"},
    {"elf_kolir2", "1F9DD-1F3FC"},
    {"elf_kolir3", "1F9DD-1F3FD"},
    {"elf_kolir4", "1F9DD-1F3FE"},
    {"elf_kolir5", "1F9DD-1F3FF"},
    {"dzhin", "1F9DE"},
    {"zombi", "1F9DF"},
    {"mozok", "1F9E0"},
    {"oranzheve_serce", "1F9E1"},
    {"beysbolka", "1F9E2"},
    {"sharf", "1F9E3"},
    {"rukavichki", "1F9E4"},
    {"palto", "1F9E5"},
    {"shkarpetki", "1F9E6"},
    {"chervoniy_konvert", "1F9E7"},
    {"petarda", "1F9E8"},
    {"element_pazla", "1F9E9"},
    {"probirka", "1F9EA"},
    {"chashka_petri", "1F9EB"},
    {"dnk", "1F9EC"},
    {"kompas", "1F9ED"},
    {"rahivnicya", "1F9EE"},
    {"vognegasnik", "1F9EF"},
    {"skrinya_dlya_instrumentiv", "1F9F0"},
    {"cegla", "1F9F1"},
    {"magnit", "1F9F2"},
    {"bagazh", "1F9F3"},
    {"losyon", "1F9F4"},
    {"nitka", "1F9F5"},
    {"pryazha", "1F9F6"},
    {"angliyska_shpilka", "1F9F7"},
    {"vedmedik", "1F9F8"},
    {"mitla", "1F9F9"},
    {"koshik", "1F9FA"},
    {"rulon_paperu", "1F9FB"},
    {"milo", "1F9FC"},
    {"gubka", "1F9FD"},
    {"kvitanciya", "1F9FE"},
    {"oko_fatimi", "1F9FF"},
    {"baletni_tufli", "1FA70"},
    {"sucilniy_kupalnik", "1FA71"},
    {"plavki", "1FA72"},
    {"shorti", "1FA73"},
    {"vietnamki", "1FA74"},
    {"blakitne_serce", "1FA75"},
    {"sire_serce", "1FA76"},
    {"rozheve_serce", "1FA77"},
    {"kraplya_krovi", "1FA78"},
    {"plastir", "1FA79"},
    {"stetoskop", "1FA7A"},
    {"rentgen", "1FA7B"},
    {"milicya", "1FA7C"},
    {"yo_yo", "1FA80"},
    {"povitryaniy_zmiy", "1FA81"},
    {"parashut", "1FA82"},
    {"bumerang", "1FA83"},
    {"charivna_palichka", "1FA84"},
    {"pinyata", "1FA85"},
    {"lyalki_matroshki", "1FA86"},
    {"marakas", "1FA87"},
    {"fleyta", "1FA88"},
    {"arfa", "1FA89"},
    {"trombon", "1FA8A"},
    {"skrinya_skarbiv", "1FA8E"},
    {"lopata", "1FA8F"},
    {"planeta_z_kilcem", "1FA90"},
    {"stilec", "1FA91"},
    {"britva", "1FA92"},
    {"sokira", "1FA93"},
    {"kaganec", "1FA94"},
    {"bandzho", "1FA95"},
    {"viyskoviy_sholom", "1FA96"},
    {"akordeon", "1FA97"},
    {"cilindrichniy_baraban", "1FA98"},
    {"moneta", "1FA99"},
    {"teslyarska_pila", "1FA9A"},
    {"vikrutka", "1FA9B"},
    {"drabina", "1FA9C"},
    {"gachok", "1FA9D"},
    {"dzerkalo", "1FA9E"},
    {"vikno", "1FA9F"},
    {"vantuz", "1FAA0"},
    {"shvacka_golka", "1FAA1"},
    {"vuzol", "1FAA2"},
    {"vidro", "1FAA3"},
    {"misholovka", "1FAA4"},
    {"zubna_shchitka", "1FAA5"},
    {"nadgrobok", "1FAA6"},
    {"plakat", "1FAA7"},
    {"kamin", "1FAA8"},
    {"dzerkalna_kulya", "1FAA9"},
    {"posvidchennya", "1FAAA"},
    {"nizkiy_zaryad_batareyi", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"skladane_viyalo", "1FAAD"},
    {"grebin", "1FAAE"},
    {"khanda", "1FAAF"},
    {"muha", "1FAB0"},
    {"hrobak", "1FAB1"},
    {"zhuk", "1FAB2"},
    {"targan", "1FAB3"},
    {"roslina_v_gorshchiku", "1FAB4"},
    {"derevina", "1FAB5"},
    {"piryina", "1FAB6"},
    {"lotos", "1FAB7"},
    {"koral", "1FAB8"},
    {"porozhnie_gnizdo", "1FAB9"},
    {"gnizdo_z_ptashinimi_yaycyami", "1FABA"},
    {"giacint", "1FABB"},
    {"meduza", "1FABC"},
    {"krilo", "1FABD"},
    {"suhe_derevo", "1FABE"},
    {"gusak", "1FABF"},
    {"serce", "1FAC0"},
    {"legeni", "1FAC1"},
    {"lyudi_obiymayutsya", "1FAC2"},
    {"cholovik_iz_vagitnim_zhivotom", "1FAC3"},
    {"cholovik_iz_vagitnim_zhivotom_kolir1", "1FAC3-1F3FB"},
    {"cholovik_iz_vagitnim_zhivotom_kolir2", "1FAC3-1F3FC"},
    {"cholovik_iz_vagitnim_zhivotom_kolir3", "1FAC3-1F3FD"},
    {"cholovik_iz_vagitnim_zhivotom_kolir4", "1FAC3-1F3FE"},
    {"cholovik_iz_vagitnim_zhivotom_kolir5", "1FAC3-1F3FF"},
    {"vagitna_lyudina", "1FAC4"},
    {"vagitna_lyudina_kolir1", "1FAC4-1F3FB"},
    {"vagitna_lyudina_kolir2", "1FAC4-1F3FC"},
    {"vagitna_lyudina_kolir3", "1FAC4-1F3FD"},
    {"vagitna_lyudina_kolir4", "1FAC4-1F3FE"},
    {"vagitna_lyudina_kolir5", "1FAC4-1F3FF"},
    {"lyudina_z_koronoyu_na_golovi", "1FAC5"},
    {"lyudina_z_koronoyu_na_golovi_kolir1", "1FAC5-1F3FB"},
    {"lyudina_z_koronoyu_na_golovi_kolir2", "1FAC5-1F3FC"},
    {"lyudina_z_koronoyu_na_golovi_kolir3", "1FAC5-1F3FD"},
    {"lyudina_z_koronoyu_na_golovi_kolir4", "1FAC5-1F3FE"},
    {"lyudina_z_koronoyu_na_golovi_kolir5", "1FAC5-1F3FF"},
    {"vidbitok_palcya", "1FAC6"},
    {"volohata_istota", "1FAC8"},
    {"kosatka", "1FACD"},
    {"los", "1FACE"},
    {"vislyuk", "1FACF"},
    {"lohina", "1FAD0"},
    {"solodkiy_perec", "1FAD1"},
    {"olivka", "1FAD2"},
    {"korzh", "1FAD3"},
    {"tamale", "1FAD4"},
    {"fondyu", "1FAD5"},
    {"chaynik", "1FAD6"},
    {"sklyanka_z_yakoyi_vilivaietsya_ridina", "1FAD7"},
    {"kvasolya", "1FAD8"},
    {"banka", "1FAD9"},
    {"imbir", "1FADA"},
    {"struchok_gorohu", "1FADB"},
    {"koreneplid", "1FADC"},
    {"lyapka", "1FADF"},
    {"rozplavlene_oblichchya", "1FAE0"},
    {"oblichchya_shcho_koziryaie", "1FAE1"},
    {"oblichchya_z_rozplyushchenimi_ochima_i_rukoyu_na_roti", "1FAE2"},
    {"oblichchya_shcho_pidglyadaie_odnim_okom", "1FAE3"},
    {"oblichchya_z_rotom_navskis", "1FAE4"},
    {"oblichchya_punktirom", "1FAE5"},
    {"zakushena_nizhnya_guba", "1FAE6"},
    {"bulbashki", "1FAE7"},
    {"tremtlive_oblichchya", "1FAE8"},
    {"nevispane_oblichchya", "1FAE9"},
    {"spotvorene_oblichchya", "1FAEA"},
    {"biyka_v_hmari", "1FAEF"},
    {"klacannya_palcyami", "1FAF0"},
    {"klacannya_palcyami_kolir1", "1FAF0-1F3FB"},
    {"klacannya_palcyami_kolir2", "1FAF0-1F3FC"},
    {"klacannya_palcyami_kolir3", "1FAF0-1F3FD"},
    {"klacannya_palcyami_kolir4", "1FAF0-1F3FE"},
    {"klacannya_palcyami_kolir5", "1FAF0-1F3FF"},
    {"dolonya_vpravo", "1FAF1"},
    {"dolonya_vpravo_kolir1", "1FAF1-1F3FB"},
    {"dolonya_vpravo_kolir2", "1FAF1-1F3FC"},
    {"dolonya_vpravo_kolir3", "1FAF1-1F3FD"},
    {"dolonya_vpravo_kolir4", "1FAF1-1F3FE"},
    {"dolonya_vpravo_kolir5", "1FAF1-1F3FF"},
    {"dolonya_vlivo", "1FAF2"},
    {"dolonya_vlivo_kolir1", "1FAF2-1F3FB"},
    {"dolonya_vlivo_kolir2", "1FAF2-1F3FC"},
    {"dolonya_vlivo_kolir3", "1FAF2-1F3FD"},
    {"dolonya_vlivo_kolir4", "1FAF2-1F3FE"},
    {"dolonya_vlivo_kolir5", "1FAF2-1F3FF"},
    {"ruka_doloneyu_vniz", "1FAF3"},
    {"ruka_doloneyu_vniz_kolir1", "1FAF3-1F3FB"},
    {"ruka_doloneyu_vniz_kolir2", "1FAF3-1F3FC"},
    {"ruka_doloneyu_vniz_kolir3", "1FAF3-1F3FD"},
    {"ruka_doloneyu_vniz_kolir4", "1FAF3-1F3FE"},
    {"ruka_doloneyu_vniz_kolir5", "1FAF3-1F3FF"},
    {"ruka_doloneyu_vgoru", "1FAF4"},
    {"ruka_doloneyu_vgoru_kolir1", "1FAF4-1F3FB"},
    {"ruka_doloneyu_vgoru_kolir2", "1FAF4-1F3FC"},
    {"ruka_doloneyu_vgoru_kolir3", "1FAF4-1F3FD"},
    {"ruka_doloneyu_vgoru_kolir4", "1FAF4-1F3FE"},
    {"ruka_doloneyu_vgoru_kolir5", "1FAF4-1F3FF"},
    {"vkazivniy_palec_shcho_vkazuie_na_glyadacha", "1FAF5"},
    {"vkazivniy_palec_shcho_vkazuie_na_glyadacha_kolir1", "1FAF5-1F3FB"},
    {"vkazivniy_palec_shcho_vkazuie_na_glyadacha_kolir2", "1FAF5-1F3FC"},
    {"vkazivniy_palec_shcho_vkazuie_na_glyadacha_kolir3", "1FAF5-1F3FD"},
    {"vkazivniy_palec_shcho_vkazuie_na_glyadacha_kolir4", "1FAF5-1F3FE"},
    {"vkazivniy_palec_shcho_vkazuie_na_glyadacha_kolir5", "1FAF5-1F3FF"},
    {"zhest_serce", "1FAF6"},
    {"zhest_serce_kolir1", "1FAF6-1F3FB"},
    {"zhest_serce_kolir2", "1FAF6-1F3FC"},
    {"zhest_serce_kolir3", "1FAF6-1F3FD"},
    {"zhest_serce_kolir4", "1FAF6-1F3FE"},
    {"zhest_serce_kolir5", "1FAF6-1F3FF"},
    {"dolonya_shtovhaie_vlivo", "1FAF7"},
    {"dolonya_shtovhaie_vlivo_kolir1", "1FAF7-1F3FB"},
    {"dolonya_shtovhaie_vlivo_kolir2", "1FAF7-1F3FC"},
    {"dolonya_shtovhaie_vlivo_kolir3", "1FAF7-1F3FD"},
    {"dolonya_shtovhaie_vlivo_kolir4", "1FAF7-1F3FE"},
    {"dolonya_shtovhaie_vlivo_kolir5", "1FAF7-1F3FF"},
    {"dolonya_shtovhaie_vpravo", "1FAF8"},
    {"dolonya_shtovhaie_vpravo_kolir1", "1FAF8-1F3FB"},
    {"dolonya_shtovhaie_vpravo_kolir2", "1FAF8-1F3FC"},
    {"dolonya_shtovhaie_vpravo_kolir3", "1FAF8-1F3FD"},
    {"dolonya_shtovhaie_vpravo_kolir4", "1FAF8-1F3FE"},
    {"dolonya_shtovhaie_vpravo_kolir5", "1FAF8-1F3FF"},
    {"klavisha_znak_chisla", "0023-FE0F-20E3"},
    {"klavisha_zirochka", "002A-FE0F-20E3"},
    {"klavisha_0", "0030-FE0F-20E3"},
    {"klavisha_1", "0031-FE0F-20E3"},
    {"klavisha_2", "0032-FE0F-20E3"},
    {"klavisha_3", "0033-FE0F-20E3"},
    {"klavisha_4", "0034-FE0F-20E3"},
    {"klavisha_5", "0035-FE0F-20E3"},
    {"klavisha_6", "0036-FE0F-20E3"},
    {"klavisha_7", "0037-FE0F-20E3"},
    {"klavisha_8", "0038-FE0F-20E3"},
    {"klavisha_9", "0039-FE0F-20E3"},
    {"prapor_ostriv_voznesinnya", "1F1E6-1F1E8"},
    {"prapor_andorra", "1F1E6-1F1E9"},
    {"prapor_obiednani_arabski_emirati", "1F1E6-1F1EA"},
    {"prapor_afganistan", "1F1E6-1F1EB"},
    {"prapor_antigua_i_barbuda", "1F1E6-1F1EC"},
    {"prapor_angilya", "1F1E6-1F1EE"},
    {"prapor_albaniya", "1F1E6-1F1F1"},
    {"prapor_virmeniya", "1F1E6-1F1F2"},
    {"prapor_angola", "1F1E6-1F1F4"},
    {"prapor_antarktika", "1F1E6-1F1F6"},
    {"prapor_argentina", "1F1E6-1F1F7"},
    {"prapor_amerikanske_samoa", "1F1E6-1F1F8"},
    {"prapor_avstriya", "1F1E6-1F1F9"},
    {"prapor_avstraliya", "1F1E6-1F1FA"},
    {"prapor_aruba", "1F1E6-1F1FC"},
    {"prapor_alandski_ostrovi", "1F1E6-1F1FD"},
    {"prapor_azerbaydzhan", "1F1E6-1F1FF"},
    {"prapor_bosniya_i_gercegovina", "1F1E7-1F1E6"},
    {"prapor_barbados", "1F1E7-1F1E7"},
    {"prapor_bangladesh", "1F1E7-1F1E9"},
    {"prapor_belgiya", "1F1E7-1F1EA"},
    {"prapor_burkina_faso", "1F1E7-1F1EB"},
    {"prapor_bolgariya", "1F1E7-1F1EC"},
    {"prapor_bahreyn", "1F1E7-1F1ED"},
    {"prapor_burundi", "1F1E7-1F1EE"},
    {"prapor_benin", "1F1E7-1F1EF"},
    {"prapor_sen_bartelemi", "1F1E7-1F1F1"},
    {"prapor_bermudski_ostrovi", "1F1E7-1F1F2"},
    {"prapor_bruney", "1F1E7-1F1F3"},
    {"prapor_boliviya", "1F1E7-1F1F4"},
    {"prapor_karibski_niderlandi", "1F1E7-1F1F6"},
    {"prapor_braziliya", "1F1E7-1F1F7"},
    {"prapor_bagamski_ostrovi", "1F1E7-1F1F8"},
    {"prapor_butan", "1F1E7-1F1F9"},
    {"prapor_ostriv_buve", "1F1E7-1F1FB"},
    {"prapor_botsvana", "1F1E7-1F1FC"},
    {"prapor_bilorus", "1F1E7-1F1FE"},
    {"prapor_beliz", "1F1E7-1F1FF"},
    {"prapor_kanada", "1F1E8-1F1E6"},
    {"prapor_kokosovi_kiling_ostrovi", "1F1E8-1F1E8"},
    {"prapor_kongo_kinshasa", "1F1E8-1F1E9"},
    {"prapor_centralnoafrikanska_respublika", "1F1E8-1F1EB"},
    {"prapor_kongo_brazzavil", "1F1E8-1F1EC"},
    {"prapor_shveycariya", "1F1E8-1F1ED"},
    {"prapor_kot_divuar", "1F1E8-1F1EE"},
    {"prapor_ostrovi_kuka", "1F1E8-1F1F0"},
    {"prapor_chili", "1F1E8-1F1F1"},
    {"prapor_kamerun", "1F1E8-1F1F2"},
    {"prapor_kitay", "1F1E8-1F1F3"},
    {"prapor_kolumbiya", "1F1E8-1F1F4"},
    {"prapor_ostriv_klipperton", "1F1E8-1F1F5"},
    {"prapor_sark", "1F1E8-1F1F6"},
    {"prapor_kosta_rika", "1F1E8-1F1F7"},
    {"prapor_kuba", "1F1E8-1F1FA"},
    {"prapor_kabo_verde", "1F1E8-1F1FB"},
    {"prapor_kyurasao", "1F1E8-1F1FC"},
    {"prapor_ostriv_rizdva", "1F1E8-1F1FD"},
    {"prapor_kipr", "1F1E8-1F1FE"},
    {"prapor_chehiya", "1F1E8-1F1FF"},
    {"prapor_nimechchina", "1F1E9-1F1EA"},
    {"prapor_diiego_garsiya", "1F1E9-1F1EC"},
    {"prapor_dzhibuti", "1F1E9-1F1EF"},
    {"prapor_daniya", "1F1E9-1F1F0"},
    {"prapor_dominika", "1F1E9-1F1F2"},
    {"prapor_dominikanska_respublika", "1F1E9-1F1F4"},
    {"prapor_alzhir", "1F1E9-1F1FF"},
    {"prapor_seuta_i_melilya", "1F1EA-1F1E6"},
    {"prapor_ekvador", "1F1EA-1F1E8"},
    {"prapor_estoniya", "1F1EA-1F1EA"},
    {"prapor_iegipet", "1F1EA-1F1EC"},
    {"prapor_zahidna_sahara", "1F1EA-1F1ED"},
    {"prapor_eritreya", "1F1EA-1F1F7"},
    {"prapor_ispaniya", "1F1EA-1F1F8"},
    {"prapor_efiopiya", "1F1EA-1F1F9"},
    {"prapor_ievropeyskiy_soyuz", "1F1EA-1F1FA"},
    {"prapor_finlyandiya", "1F1EB-1F1EE"},
    {"prapor_fidzhi", "1F1EB-1F1EF"},
    {"prapor_folklendski_ostrovi", "1F1EB-1F1F0"},
    {"prapor_mikroneziya", "1F1EB-1F1F2"},
    {"prapor_farerski_ostrovi", "1F1EB-1F1F4"},
    {"prapor_franciya", "1F1EB-1F1F7"},
    {"prapor_gabon", "1F1EC-1F1E6"},
    {"prapor_velika_britaniya", "1F1EC-1F1E7"},
    {"prapor_grenada", "1F1EC-1F1E9"},
    {"prapor_gruziya", "1F1EC-1F1EA"},
    {"prapor_francuzka_gviana", "1F1EC-1F1EB"},
    {"prapor_gernsi", "1F1EC-1F1EC"},
    {"prapor_gana", "1F1EC-1F1ED"},
    {"prapor_gibraltar", "1F1EC-1F1EE"},
    {"prapor_grenlandiya", "1F1EC-1F1F1"},
    {"prapor_gambiya", "1F1EC-1F1F2"},
    {"prapor_gvineya", "1F1EC-1F1F3"},
    {"prapor_gvadelupa", "1F1EC-1F1F5"},
    {"prapor_ekvatorialna_gvineya", "1F1EC-1F1F6"},
    {"prapor_greciya", "1F1EC-1F1F7"},
    {"prapor_pivdenna_dzhordzhiya_ta_pivdenni_sandvichevi_ostrovi", "1F1EC-1F1F8"},
    {"prapor_gvatemala", "1F1EC-1F1F9"},
    {"prapor_guam", "1F1EC-1F1FA"},
    {"prapor_gvineya_bisau", "1F1EC-1F1FC"},
    {"prapor_gayana", "1F1EC-1F1FE"},
    {"prapor_gonkong_oar_kitayu", "1F1ED-1F1F0"},
    {"prapor_ostrovi_gerd_i_makdonald", "1F1ED-1F1F2"},
    {"prapor_gonduras", "1F1ED-1F1F3"},
    {"prapor_horvatiya", "1F1ED-1F1F7"},
    {"prapor_gayiti", "1F1ED-1F1F9"},
    {"prapor_ugorshchina", "1F1ED-1F1FA"},
    {"prapor_kanarski_ostrovi", "1F1EE-1F1E8"},
    {"prapor_indoneziya", "1F1EE-1F1E9"},
    {"prapor_irlandiya", "1F1EE-1F1EA"},
    {"prapor_izrayil", "1F1EE-1F1F1"},
    {"prapor_ostriv_men", "1F1EE-1F1F2"},
    {"prapor_indiya", "1F1EE-1F1F3"},
    {"prapor_britanska_teritoriya_v_indiyskomu_okeani", "1F1EE-1F1F4"},
    {"prapor_irak", "1F1EE-1F1F6"},
    {"prapor_iran", "1F1EE-1F1F7"},
    {"prapor_islandiya", "1F1EE-1F1F8"},
    {"prapor_italiya", "1F1EE-1F1F9"},
    {"prapor_dzhersi", "1F1EF-1F1EA"},
    {"prapor_yamayka", "1F1EF-1F1F2"},
    {"prapor_yordaniya", "1F1EF-1F1F4"},
    {"prapor_yaponiya", "1F1EF-1F1F5"},
    {"prapor_keniya", "1F1F0-1F1EA"},
    {"prapor_kirgizstan", "1F1F0-1F1EC"},
    {"prapor_kambodzha", "1F1F0-1F1ED"},
    {"prapor_kiribati", "1F1F0-1F1EE"},
    {"prapor_komori", "1F1F0-1F1F2"},
    {"prapor_sent_kits_i_nevis", "1F1F0-1F1F3"},
    {"prapor_pivnichna_koreya", "1F1F0-1F1F5"},
    {"prapor_pivdenna_koreya", "1F1F0-1F1F7"},
    {"prapor_kuveyt", "1F1F0-1F1FC"},
    {"prapor_kaymanovi_ostrovi", "1F1F0-1F1FE"},
    {"prapor_kazahstan", "1F1F0-1F1FF"},
    {"prapor_laos", "1F1F1-1F1E6"},
    {"prapor_livan", "1F1F1-1F1E7"},
    {"prapor_sent_lyusiya", "1F1F1-1F1E8"},
    {"prapor_lihtenshteyn", "1F1F1-1F1EE"},
    {"prapor_shri_lanka", "1F1F1-1F1F0"},
    {"prapor_liberiya", "1F1F1-1F1F7"},
    {"prapor_lesoto", "1F1F1-1F1F8"},
    {"prapor_litva", "1F1F1-1F1F9"},
    {"prapor_lyuksemburg", "1F1F1-1F1FA"},
    {"prapor_latviya", "1F1F1-1F1FB"},
    {"prapor_liviya", "1F1F1-1F1FE"},
    {"prapor_marokko", "1F1F2-1F1E6"},
    {"prapor_monako", "1F1F2-1F1E8"},
    {"prapor_moldova", "1F1F2-1F1E9"},
    {"prapor_chornogoriya", "1F1F2-1F1EA"},
    {"prapor_sen_marten", "1F1F2-1F1EB"},
    {"prapor_madagaskar", "1F1F2-1F1EC"},
    {"prapor_marshallovi_ostrovi", "1F1F2-1F1ED"},
    {"prapor_pivnichna_makedoniya", "1F1F2-1F1F0"},
    {"prapor_mali", "1F1F2-1F1F1"},
    {"prapor_myanma_birma", "1F1F2-1F1F2"},
    {"prapor_mongoliya", "1F1F2-1F1F3"},
    {"prapor_makao_oar_kitayu", "1F1F2-1F1F4"},
    {"prapor_pivnichni_marianski_ostrovi", "1F1F2-1F1F5"},
    {"prapor_martinika", "1F1F2-1F1F6"},
    {"prapor_mavritaniya", "1F1F2-1F1F7"},
    {"prapor_montserrat", "1F1F2-1F1F8"},
    {"prapor_malta", "1F1F2-1F1F9"},
    {"prapor_mavrikiy", "1F1F2-1F1FA"},
    {"prapor_maldivi", "1F1F2-1F1FB"},
    {"prapor_malavi", "1F1F2-1F1FC"},
    {"prapor_meksika", "1F1F2-1F1FD"},
    {"prapor_malayziya", "1F1F2-1F1FE"},
    {"prapor_mozambik", "1F1F2-1F1FF"},
    {"prapor_namibiya", "1F1F3-1F1E6"},
    {"prapor_nova_kaledoniya", "1F1F3-1F1E8"},
    {"prapor_niger", "1F1F3-1F1EA"},
    {"prapor_ostriv_norfolk", "1F1F3-1F1EB"},
    {"prapor_nigeriya", "1F1F3-1F1EC"},
    {"prapor_nikaragua", "1F1F3-1F1EE"},
    {"prapor_niderlandi", "1F1F3-1F1F1"},
    {"prapor_norvegiya", "1F1F3-1F1F4"},
    {"prapor_nepal", "1F1F3-1F1F5"},
    {"prapor_nauru", "1F1F3-1F1F7"},
    {"prapor_niue", "1F1F3-1F1FA"},
    {"prapor_nova_zelandiya", "1F1F3-1F1FF"},
    {"prapor_oman", "1F1F4-1F1F2"},
    {"prapor_panama", "1F1F5-1F1E6"},
    {"prapor_peru", "1F1F5-1F1EA"},
    {"prapor_francuzka_polineziya", "1F1F5-1F1EB"},
    {"prapor_papua_nova_gvineya", "1F1F5-1F1EC"},
    {"prapor_filippini", "1F1F5-1F1ED"},
    {"prapor_pakistan", "1F1F5-1F1F0"},
    {"prapor_polshcha", "1F1F5-1F1F1"},
    {"prapor_sen_pier_i_mikelon", "1F1F5-1F1F2"},
    {"prapor_ostrovi_pitkern", "1F1F5-1F1F3"},
    {"prapor_puerto_riko", "1F1F5-1F1F7"},
    {"prapor_palestinski_teritoriyi", "1F1F5-1F1F8"},
    {"prapor_portugaliya", "1F1F5-1F1F9"},
    {"prapor_palau", "1F1F5-1F1FC"},
    {"prapor_paragvay", "1F1F5-1F1FE"},
    {"prapor_katar", "1F1F6-1F1E6"},
    {"prapor_reyunyon", "1F1F7-1F1EA"},
    {"prapor_rumuniya", "1F1F7-1F1F4"},
    {"prapor_serbiya", "1F1F7-1F1F8"},
    {"prapor_rosiya", "1F1F7-1F1FA"},
    {"prapor_ruanda", "1F1F7-1F1FC"},
    {"prapor_saudivska_araviya", "1F1F8-1F1E6"},
    {"prapor_solomonovi_ostrovi", "1F1F8-1F1E7"},
    {"prapor_seyshelski_ostrovi", "1F1F8-1F1E8"},
    {"prapor_sudan", "1F1F8-1F1E9"},
    {"prapor_shveciya", "1F1F8-1F1EA"},
    {"prapor_singapur", "1F1F8-1F1EC"},
    {"prapor_ostriv_svyatoyi_ieleni", "1F1F8-1F1ED"},
    {"prapor_sloveniya", "1F1F8-1F1EE"},
    {"prapor_shpicbergen_ta_yan_maien", "1F1F8-1F1EF"},
    {"prapor_slovachchina", "1F1F8-1F1F0"},
    {"prapor_sierra_leone", "1F1F8-1F1F1"},
    {"prapor_san_marino", "1F1F8-1F1F2"},
    {"prapor_senegal", "1F1F8-1F1F3"},
    {"prapor_somali", "1F1F8-1F1F4"},
    {"prapor_surinam", "1F1F8-1F1F7"},
    {"prapor_pivdenniy_sudan", "1F1F8-1F1F8"},
    {"prapor_san_tome_i_prinsipi", "1F1F8-1F1F9"},
    {"prapor_salvador", "1F1F8-1F1FB"},
    {"prapor_sint_marten", "1F1F8-1F1FD"},
    {"prapor_siriya", "1F1F8-1F1FE"},
    {"prapor_esvatini", "1F1F8-1F1FF"},
    {"prapor_tristan_da_kunya", "1F1F9-1F1E6"},
    {"prapor_ostrovi_terks_i_kaykos", "1F1F9-1F1E8"},
    {"prapor_chad", "1F1F9-1F1E9"},
    {"prapor_francuzki_pivdenni_teritoriyi", "1F1F9-1F1EB"},
    {"prapor_togo", "1F1F9-1F1EC"},
    {"prapor_tayiland", "1F1F9-1F1ED"},
    {"prapor_tadzhikistan", "1F1F9-1F1EF"},
    {"prapor_tokelau", "1F1F9-1F1F0"},
    {"prapor_timor_leshti", "1F1F9-1F1F1"},
    {"prapor_turkmenistan", "1F1F9-1F1F2"},
    {"prapor_tunis", "1F1F9-1F1F3"},
    {"prapor_tonga", "1F1F9-1F1F4"},
    {"prapor_turechchina", "1F1F9-1F1F7"},
    {"prapor_trinidad_i_tobago", "1F1F9-1F1F9"},
    {"prapor_tuvalu", "1F1F9-1F1FB"},
    {"prapor_tayvan", "1F1F9-1F1FC"},
    {"prapor_tanzaniya", "1F1F9-1F1FF"},
    {"prapor_ukrayina", "1F1FA-1F1E6"},
    {"prapor_uganda", "1F1FA-1F1EC"},
    {"prapor_viddaleni_ostrovi_ssha", "1F1FA-1F1F2"},
    {"prapor_organizaciya_obiednanih_naciy", "1F1FA-1F1F3"},
    {"prapor_spolucheni_shtati", "1F1FA-1F1F8"},
    {"prapor_urugvay", "1F1FA-1F1FE"},
    {"prapor_uzbekistan", "1F1FA-1F1FF"},
    {"prapor_vatikan", "1F1FB-1F1E6"},
    {"prapor_sent_vinsent_i_grenadini", "1F1FB-1F1E8"},
    {"prapor_venesuela", "1F1FB-1F1EA"},
    {"prapor_britanski_virginski_ostrovi", "1F1FB-1F1EC"},
    {"prapor_virginski_ostrovi_ssha", "1F1FB-1F1EE"},
    {"prapor_vietnam", "1F1FB-1F1F3"},
    {"prapor_vanuatu", "1F1FB-1F1FA"},
    {"prapor_uollis_i_futuna", "1F1FC-1F1EB"},
    {"prapor_samoa", "1F1FC-1F1F8"},
    {"prapor_kosovo", "1F1FD-1F1F0"},
    {"prapor_iemen", "1F1FE-1F1EA"},
    {"prapor_mayotta", "1F1FE-1F1F9"},
    {"prapor_pivdenno_afrikanska_respublika", "1F1FF-1F1E6"},
    {"prapor_zambiya", "1F1FF-1F1F2"},
    {"prapor_zimbabve", "1F1FF-1F1FC"},
    {"prapor_angliya", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"prapor_shotlandiya", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"prapor_uels", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik", "1F468-200D-2764-FE0F-200D-1F468"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_cholovik_i_cholovik_kolir5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_cholovik_i_cholovik_kolir5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"rodina_cholovik_i_hlopchik", "1F468-200D-1F466"},
    {"rodina_cholovik_hlopchik_i_hlopchik", "1F468-200D-1F466-200D-1F466"},
    {"rodina_cholovik_i_divchinka", "1F468-200D-1F467"},
    {"rodina_cholovik_divchinka_i_hlopchik", "1F468-200D-1F467-200D-1F466"},
    {"rodina_cholovik_divchinka_i_divchinka", "1F468-200D-1F467-200D-1F467"},
    {"rodina_cholovik_cholovik_i_hlopchik", "1F468-200D-1F468-200D-1F466"},
    {"rodina_cholovik_cholovik_hlopchik_i_hlopchik", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"rodina_cholovik_cholovik_i_divchinka", "1F468-200D-1F468-200D-1F467"},
    {"rodina_cholovik_cholovik_divchinka_i_hlopchik", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"rodina_cholovik_cholovik_divchinka_i_divchinka", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"rodina_cholovik_zhinka_i_hlopchik", "1F468-200D-1F469-200D-1F466"},
    {"rodina_cholovik_zhinka_hlopchik_i_hlopchik", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"rodina_cholovik_zhinka_i_divchinka", "1F468-200D-1F469-200D-1F467"},
    {"rodina_cholovik_zhinka_divchinka_i_hlopchik", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"rodina_cholovik_zhinka_divchinka_i_divchinka", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik", "1F469-200D-2764-FE0F-200D-1F468"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_cholovik_kolir5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka", "1F469-200D-2764-FE0F-200D-1F469"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"cholovik_i_zhinka_iz_sercem_mizh_nimi_zhinka_i_zhinka_kolir5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_cholovik_kolir5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"cholovik_i_zhinka_shcho_ciluyutsya_zhinka_i_zhinka_kolir5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"rodina_zhinka_i_hlopchik", "1F469-200D-1F466"},
    {"rodina_zhinka_hlopchik_i_hlopchik", "1F469-200D-1F466-200D-1F466"},
    {"rodina_zhinka_i_divchinka", "1F469-200D-1F467"},
    {"rodina_zhinka_divchinka_i_hlopchik", "1F469-200D-1F467-200D-1F466"},
    {"rodina_zhinka_divchinka_i_divchinka", "1F469-200D-1F467-200D-1F467"},
    {"rodina_zhinka_zhinka_i_hlopchik", "1F469-200D-1F469-200D-1F466"},
    {"rodina_zhinka_zhinka_hlopchik_i_hlopchik", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"rodina_zhinka_zhinka_i_divchinka", "1F469-200D-1F469-200D-1F467"},
    {"rodina_zhinka_zhinka_divchinka_i_hlopchik", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"rodina_zhinka_zhinka_divchinka_i_divchinka", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"lyudi_shcho_trimayutsya_za_ruki", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"lyudi_shcho_trimayutsya_za_ruki_kolir5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"rodina_dorosliy_dorosliy_ditina", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"rodina_dvoie_doroslih_i_dvoie_ditey", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"rodina_dorosliy_i_ditina", "1F9D1-200D-1F9D2"},
    {"rodina_dorosliy_i_diti", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"lyudina_shcho_bizhit_praviy_profil", "1F3C3-200D-27A1-FE0F"},
    {"lyudina_shcho_bizhit_praviy_profil_kolir1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"lyudina_shcho_bizhit_praviy_profil_kolir2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"lyudina_shcho_bizhit_praviy_profil_kolir3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"lyudina_shcho_bizhit_praviy_profil_kolir4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"lyudina_shcho_bizhit_praviy_profil_kolir5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"medichniy_pracivnik", "1F468-200D-2695-FE0F"},
    {"medichniy_pracivnik_kolir1", "1F468-1F3FB-200D-2695-FE0F"},
    {"medichniy_pracivnik_kolir2", "1F468-1F3FC-200D-2695-FE0F"},
    {"medichniy_pracivnik_kolir3", "1F468-1F3FD-200D-2695-FE0F"},
    {"medichniy_pracivnik_kolir4", "1F468-1F3FE-200D-2695-FE0F"},
    {"medichniy_pracivnik_kolir5", "1F468-1F3FF-200D-2695-FE0F"},
    {"suddya_cholovik", "1F468-200D-2696-FE0F"},
    {"suddya_cholovik_kolir1", "1F468-1F3FB-200D-2696-FE0F"},
    {"suddya_cholovik_kolir2", "1F468-1F3FC-200D-2696-FE0F"},
    {"suddya_cholovik_kolir3", "1F468-1F3FD-200D-2696-FE0F"},
    {"suddya_cholovik_kolir4", "1F468-1F3FE-200D-2696-FE0F"},
    {"suddya_cholovik_kolir5", "1F468-1F3FF-200D-2696-FE0F"},
    {"pilot", "1F468-200D-2708-FE0F"},
    {"pilot_kolir1", "1F468-1F3FB-200D-2708-FE0F"},
    {"pilot_kolir2", "1F468-1F3FC-200D-2708-FE0F"},
    {"pilot_kolir3", "1F468-1F3FD-200D-2708-FE0F"},
    {"pilot_kolir4", "1F468-1F3FE-200D-2708-FE0F"},
    {"pilot_kolir5", "1F468-1F3FF-200D-2708-FE0F"},
    {"fermer", "1F468-200D-1F33E"},
    {"fermer_kolir1", "1F468-1F3FB-200D-1F33E"},
    {"fermer_kolir2", "1F468-1F3FC-200D-1F33E"},
    {"fermer_kolir3", "1F468-1F3FD-200D-1F33E"},
    {"fermer_kolir4", "1F468-1F3FE-200D-1F33E"},
    {"fermer_kolir5", "1F468-1F3FF-200D-1F33E"},
    {"kuhar", "1F468-200D-1F373"},
    {"kuhar_kolir1", "1F468-1F3FB-200D-1F373"},
    {"kuhar_kolir2", "1F468-1F3FC-200D-1F373"},
    {"kuhar_kolir3", "1F468-1F3FD-200D-1F373"},
    {"kuhar_kolir4", "1F468-1F3FE-200D-1F373"},
    {"kuhar_kolir5", "1F468-1F3FF-200D-1F373"},
    {"cholovik_goduie_ditinu", "1F468-200D-1F37C"},
    {"cholovik_goduie_ditinu_kolir1", "1F468-1F3FB-200D-1F37C"},
    {"cholovik_goduie_ditinu_kolir2", "1F468-1F3FC-200D-1F37C"},
    {"cholovik_goduie_ditinu_kolir3", "1F468-1F3FD-200D-1F37C"},
    {"cholovik_goduie_ditinu_kolir4", "1F468-1F3FE-200D-1F37C"},
    {"cholovik_goduie_ditinu_kolir5", "1F468-1F3FF-200D-1F37C"},
    {"student", "1F468-200D-1F393"},
    {"student_kolir1", "1F468-1F3FB-200D-1F393"},
    {"student_kolir2", "1F468-1F3FC-200D-1F393"},
    {"student_kolir3", "1F468-1F3FD-200D-1F393"},
    {"student_kolir4", "1F468-1F3FE-200D-1F393"},
    {"student_kolir5", "1F468-1F3FF-200D-1F393"},
    {"spivak", "1F468-200D-1F3A4"},
    {"spivak_kolir1", "1F468-1F3FB-200D-1F3A4"},
    {"spivak_kolir2", "1F468-1F3FC-200D-1F3A4"},
    {"spivak_kolir3", "1F468-1F3FD-200D-1F3A4"},
    {"spivak_kolir4", "1F468-1F3FE-200D-1F3A4"},
    {"spivak_kolir5", "1F468-1F3FF-200D-1F3A4"},
    {"cholovik_hudozhnik", "1F468-200D-1F3A8"},
    {"cholovik_hudozhnik_kolir1", "1F468-1F3FB-200D-1F3A8"},
    {"cholovik_hudozhnik_kolir2", "1F468-1F3FC-200D-1F3A8"},
    {"cholovik_hudozhnik_kolir3", "1F468-1F3FD-200D-1F3A8"},
    {"cholovik_hudozhnik_kolir4", "1F468-1F3FE-200D-1F3A8"},
    {"cholovik_hudozhnik_kolir5", "1F468-1F3FF-200D-1F3A8"},
    {"uchitel", "1F468-200D-1F3EB"},
    {"uchitel_kolir1", "1F468-1F3FB-200D-1F3EB"},
    {"uchitel_kolir2", "1F468-1F3FC-200D-1F3EB"},
    {"uchitel_kolir3", "1F468-1F3FD-200D-1F3EB"},
    {"uchitel_kolir4", "1F468-1F3FE-200D-1F3EB"},
    {"uchitel_kolir5", "1F468-1F3FF-200D-1F3EB"},
    {"robitnik", "1F468-200D-1F3ED"},
    {"robitnik_kolir1", "1F468-1F3FB-200D-1F3ED"},
    {"robitnik_kolir2", "1F468-1F3FC-200D-1F3ED"},
    {"robitnik_kolir3", "1F468-1F3FD-200D-1F3ED"},
    {"robitnik_kolir4", "1F468-1F3FE-200D-1F3ED"},
    {"robitnik_kolir5", "1F468-1F3FF-200D-1F3ED"},
    {"tehnolog", "1F468-200D-1F4BB"},
    {"tehnolog_kolir1", "1F468-1F3FB-200D-1F4BB"},
    {"tehnolog_kolir2", "1F468-1F3FC-200D-1F4BB"},
    {"tehnolog_kolir3", "1F468-1F3FD-200D-1F4BB"},
    {"tehnolog_kolir4", "1F468-1F3FE-200D-1F4BB"},
    {"tehnolog_kolir5", "1F468-1F3FF-200D-1F4BB"},
    {"sluzhbovec", "1F468-200D-1F4BC"},
    {"sluzhbovec_kolir1", "1F468-1F3FB-200D-1F4BC"},
    {"sluzhbovec_kolir2", "1F468-1F3FC-200D-1F4BC"},
    {"sluzhbovec_kolir3", "1F468-1F3FD-200D-1F4BC"},
    {"sluzhbovec_kolir4", "1F468-1F3FE-200D-1F4BC"},
    {"sluzhbovec_kolir5", "1F468-1F3FF-200D-1F4BC"},
    {"mehanik", "1F468-200D-1F527"},
    {"mehanik_kolir1", "1F468-1F3FB-200D-1F527"},
    {"mehanik_kolir2", "1F468-1F3FC-200D-1F527"},
    {"mehanik_kolir3", "1F468-1F3FD-200D-1F527"},
    {"mehanik_kolir4", "1F468-1F3FE-200D-1F527"},
    {"mehanik_kolir5", "1F468-1F3FF-200D-1F527"},
    {"naukovec", "1F468-200D-1F52C"},
    {"naukovec_kolir1", "1F468-1F3FB-200D-1F52C"},
    {"naukovec_kolir2", "1F468-1F3FC-200D-1F52C"},
    {"naukovec_kolir3", "1F468-1F3FD-200D-1F52C"},
    {"naukovec_kolir4", "1F468-1F3FE-200D-1F52C"},
    {"naukovec_kolir5", "1F468-1F3FF-200D-1F52C"},
    {"astronavt", "1F468-200D-1F680"},
    {"astronavt_kolir1", "1F468-1F3FB-200D-1F680"},
    {"astronavt_kolir2", "1F468-1F3FC-200D-1F680"},
    {"astronavt_kolir3", "1F468-1F3FD-200D-1F680"},
    {"astronavt_kolir4", "1F468-1F3FE-200D-1F680"},
    {"astronavt_kolir5", "1F468-1F3FF-200D-1F680"},
    {"pozhezhnik", "1F468-200D-1F692"},
    {"pozhezhnik_kolir1", "1F468-1F3FB-200D-1F692"},
    {"pozhezhnik_kolir2", "1F468-1F3FC-200D-1F692"},
    {"pozhezhnik_kolir3", "1F468-1F3FD-200D-1F692"},
    {"pozhezhnik_kolir4", "1F468-1F3FE-200D-1F692"},
    {"pozhezhnik_kolir5", "1F468-1F3FF-200D-1F692"},
    {"cholovik_iz_taktilnoyu_trostinoyu", "1F468-200D-1F9AF"},
    {"cholovik_iz_taktilnoyu_trostinoyu_kolir1", "1F468-1F3FB-200D-1F9AF"},
    {"cholovik_iz_taktilnoyu_trostinoyu_kolir2", "1F468-1F3FC-200D-1F9AF"},
    {"cholovik_iz_taktilnoyu_trostinoyu_kolir3", "1F468-1F3FD-200D-1F9AF"},
    {"cholovik_iz_taktilnoyu_trostinoyu_kolir4", "1F468-1F3FE-200D-1F9AF"},
    {"cholovik_iz_taktilnoyu_trostinoyu_kolir5", "1F468-1F3FF-200D-1F9AF"},
    {"cholovik_iz_taktilnoyu_trostinoyu_praviy_profil", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"cholovik_iz_taktilnoyu_trostinoyu_praviy_profil_kolir1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"cholovik_iz_taktilnoyu_trostinoyu_praviy_profil_kolir2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"cholovik_iz_taktilnoyu_trostinoyu_praviy_profil_kolir3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"cholovik_iz_taktilnoyu_trostinoyu_praviy_profil_kolir4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"cholovik_iz_taktilnoyu_trostinoyu_praviy_profil_kolir5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"cholovik_u_vizku_z_elektroprivodom", "1F468-200D-1F9BC"},
    {"cholovik_u_vizku_z_elektroprivodom_kolir1", "1F468-1F3FB-200D-1F9BC"},
    {"cholovik_u_vizku_z_elektroprivodom_kolir2", "1F468-1F3FC-200D-1F9BC"},
    {"cholovik_u_vizku_z_elektroprivodom_kolir3", "1F468-1F3FD-200D-1F9BC"},
    {"cholovik_u_vizku_z_elektroprivodom_kolir4", "1F468-1F3FE-200D-1F9BC"},
    {"cholovik_u_vizku_z_elektroprivodom_kolir5", "1F468-1F3FF-200D-1F9BC"},
    {"cholovik_u_vizku_z_elektroprivodom_praviy_profil", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"cholovik_u_vizku_z_elektroprivodom_praviy_profil_kolir1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"cholovik_u_vizku_z_elektroprivodom_praviy_profil_kolir2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"cholovik_u_vizku_z_elektroprivodom_praviy_profil_kolir3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"cholovik_u_vizku_z_elektroprivodom_praviy_profil_kolir4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"cholovik_u_vizku_z_elektroprivodom_praviy_profil_kolir5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"cholovik_u_vizku_z_ruchnim_privodom", "1F468-200D-1F9BD"},
    {"cholovik_u_vizku_z_ruchnim_privodom_kolir1", "1F468-1F3FB-200D-1F9BD"},
    {"cholovik_u_vizku_z_ruchnim_privodom_kolir2", "1F468-1F3FC-200D-1F9BD"},
    {"cholovik_u_vizku_z_ruchnim_privodom_kolir3", "1F468-1F3FD-200D-1F9BD"},
    {"cholovik_u_vizku_z_ruchnim_privodom_kolir4", "1F468-1F3FE-200D-1F9BD"},
    {"cholovik_u_vizku_z_ruchnim_privodom_kolir5", "1F468-1F3FF-200D-1F9BD"},
    {"cholovik_u_vizku_z_ruchnim_privodom_praviy_profil", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"cholovik_u_vizku_z_ruchnim_privodom_praviy_profil_kolir1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"cholovik_u_vizku_z_ruchnim_privodom_praviy_profil_kolir2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"cholovik_u_vizku_z_ruchnim_privodom_praviy_profil_kolir3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"cholovik_u_vizku_z_ruchnim_privodom_praviy_profil_kolir4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"cholovik_u_vizku_z_ruchnim_privodom_praviy_profil_kolir5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"medichna_pracivnicya", "1F469-200D-2695-FE0F"},
    {"medichna_pracivnicya_kolir1", "1F469-1F3FB-200D-2695-FE0F"},
    {"medichna_pracivnicya_kolir2", "1F469-1F3FC-200D-2695-FE0F"},
    {"medichna_pracivnicya_kolir3", "1F469-1F3FD-200D-2695-FE0F"},
    {"medichna_pracivnicya_kolir4", "1F469-1F3FE-200D-2695-FE0F"},
    {"medichna_pracivnicya_kolir5", "1F469-1F3FF-200D-2695-FE0F"},
    {"zhinka_suddya", "1F469-200D-2696-FE0F"},
    {"zhinka_suddya_kolir1", "1F469-1F3FB-200D-2696-FE0F"},
    {"zhinka_suddya_kolir2", "1F469-1F3FC-200D-2696-FE0F"},
    {"zhinka_suddya_kolir3", "1F469-1F3FD-200D-2696-FE0F"},
    {"zhinka_suddya_kolir4", "1F469-1F3FE-200D-2696-FE0F"},
    {"zhinka_suddya_kolir5", "1F469-1F3FF-200D-2696-FE0F"},
    {"zhinka_pilot", "1F469-200D-2708-FE0F"},
    {"zhinka_pilot_kolir1", "1F469-1F3FB-200D-2708-FE0F"},
    {"zhinka_pilot_kolir2", "1F469-1F3FC-200D-2708-FE0F"},
    {"zhinka_pilot_kolir3", "1F469-1F3FD-200D-2708-FE0F"},
    {"zhinka_pilot_kolir4", "1F469-1F3FE-200D-2708-FE0F"},
    {"zhinka_pilot_kolir5", "1F469-1F3FF-200D-2708-FE0F"},
    {"fermerka", "1F469-200D-1F33E"},
    {"fermerka_kolir1", "1F469-1F3FB-200D-1F33E"},
    {"fermerka_kolir2", "1F469-1F3FC-200D-1F33E"},
    {"fermerka_kolir3", "1F469-1F3FD-200D-1F33E"},
    {"fermerka_kolir4", "1F469-1F3FE-200D-1F33E"},
    {"fermerka_kolir5", "1F469-1F3FF-200D-1F33E"},
    {"kuharka", "1F469-200D-1F373"},
    {"kuharka_kolir1", "1F469-1F3FB-200D-1F373"},
    {"kuharka_kolir2", "1F469-1F3FC-200D-1F373"},
    {"kuharka_kolir3", "1F469-1F3FD-200D-1F373"},
    {"kuharka_kolir4", "1F469-1F3FE-200D-1F373"},
    {"kuharka_kolir5", "1F469-1F3FF-200D-1F373"},
    {"zhinka_goduie_ditinu", "1F469-200D-1F37C"},
    {"zhinka_goduie_ditinu_kolir1", "1F469-1F3FB-200D-1F37C"},
    {"zhinka_goduie_ditinu_kolir2", "1F469-1F3FC-200D-1F37C"},
    {"zhinka_goduie_ditinu_kolir3", "1F469-1F3FD-200D-1F37C"},
    {"zhinka_goduie_ditinu_kolir4", "1F469-1F3FE-200D-1F37C"},
    {"zhinka_goduie_ditinu_kolir5", "1F469-1F3FF-200D-1F37C"},
    {"studentka", "1F469-200D-1F393"},
    {"studentka_kolir1", "1F469-1F3FB-200D-1F393"},
    {"studentka_kolir2", "1F469-1F3FC-200D-1F393"},
    {"studentka_kolir3", "1F469-1F3FD-200D-1F393"},
    {"studentka_kolir4", "1F469-1F3FE-200D-1F393"},
    {"studentka_kolir5", "1F469-1F3FF-200D-1F393"},
    {"spivachka", "1F469-200D-1F3A4"},
    {"spivachka_kolir1", "1F469-1F3FB-200D-1F3A4"},
    {"spivachka_kolir2", "1F469-1F3FC-200D-1F3A4"},
    {"spivachka_kolir3", "1F469-1F3FD-200D-1F3A4"},
    {"spivachka_kolir4", "1F469-1F3FE-200D-1F3A4"},
    {"spivachka_kolir5", "1F469-1F3FF-200D-1F3A4"},
    {"hudozhnicya", "1F469-200D-1F3A8"},
    {"hudozhnicya_kolir1", "1F469-1F3FB-200D-1F3A8"},
    {"hudozhnicya_kolir2", "1F469-1F3FC-200D-1F3A8"},
    {"hudozhnicya_kolir3", "1F469-1F3FD-200D-1F3A8"},
    {"hudozhnicya_kolir4", "1F469-1F3FE-200D-1F3A8"},
    {"hudozhnicya_kolir5", "1F469-1F3FF-200D-1F3A8"},
    {"uchitelka", "1F469-200D-1F3EB"},
    {"uchitelka_kolir1", "1F469-1F3FB-200D-1F3EB"},
    {"uchitelka_kolir2", "1F469-1F3FC-200D-1F3EB"},
    {"uchitelka_kolir3", "1F469-1F3FD-200D-1F3EB"},
    {"uchitelka_kolir4", "1F469-1F3FE-200D-1F3EB"},
    {"uchitelka_kolir5", "1F469-1F3FF-200D-1F3EB"},
    {"robitnicya", "1F469-200D-1F3ED"},
    {"robitnicya_kolir1", "1F469-1F3FB-200D-1F3ED"},
    {"robitnicya_kolir2", "1F469-1F3FC-200D-1F3ED"},
    {"robitnicya_kolir3", "1F469-1F3FD-200D-1F3ED"},
    {"robitnicya_kolir4", "1F469-1F3FE-200D-1F3ED"},
    {"robitnicya_kolir5", "1F469-1F3FF-200D-1F3ED"},
    {"zhinka_tehnolog", "1F469-200D-1F4BB"},
    {"zhinka_tehnolog_kolir1", "1F469-1F3FB-200D-1F4BB"},
    {"zhinka_tehnolog_kolir2", "1F469-1F3FC-200D-1F4BB"},
    {"zhinka_tehnolog_kolir3", "1F469-1F3FD-200D-1F4BB"},
    {"zhinka_tehnolog_kolir4", "1F469-1F3FE-200D-1F4BB"},
    {"zhinka_tehnolog_kolir5", "1F469-1F3FF-200D-1F4BB"},
    {"sluzhbovka", "1F469-200D-1F4BC"},
    {"sluzhbovka_kolir1", "1F469-1F3FB-200D-1F4BC"},
    {"sluzhbovka_kolir2", "1F469-1F3FC-200D-1F4BC"},
    {"sluzhbovka_kolir3", "1F469-1F3FD-200D-1F4BC"},
    {"sluzhbovka_kolir4", "1F469-1F3FE-200D-1F4BC"},
    {"sluzhbovka_kolir5", "1F469-1F3FF-200D-1F4BC"},
    {"zhinka_mehanik", "1F469-200D-1F527"},
    {"zhinka_mehanik_kolir1", "1F469-1F3FB-200D-1F527"},
    {"zhinka_mehanik_kolir2", "1F469-1F3FC-200D-1F527"},
    {"zhinka_mehanik_kolir3", "1F469-1F3FD-200D-1F527"},
    {"zhinka_mehanik_kolir4", "1F469-1F3FE-200D-1F527"},
    {"zhinka_mehanik_kolir5", "1F469-1F3FF-200D-1F527"},
    {"zhinka_naukovec", "1F469-200D-1F52C"},
    {"zhinka_naukovec_kolir1", "1F469-1F3FB-200D-1F52C"},
    {"zhinka_naukovec_kolir2", "1F469-1F3FC-200D-1F52C"},
    {"zhinka_naukovec_kolir3", "1F469-1F3FD-200D-1F52C"},
    {"zhinka_naukovec_kolir4", "1F469-1F3FE-200D-1F52C"},
    {"zhinka_naukovec_kolir5", "1F469-1F3FF-200D-1F52C"},
    {"astronavtka", "1F469-200D-1F680"},
    {"astronavtka_kolir1", "1F469-1F3FB-200D-1F680"},
    {"astronavtka_kolir2", "1F469-1F3FC-200D-1F680"},
    {"astronavtka_kolir3", "1F469-1F3FD-200D-1F680"},
    {"astronavtka_kolir4", "1F469-1F3FE-200D-1F680"},
    {"astronavtka_kolir5", "1F469-1F3FF-200D-1F680"},
    {"zhinka_pozhezhnik", "1F469-200D-1F692"},
    {"zhinka_pozhezhnik_kolir1", "1F469-1F3FB-200D-1F692"},
    {"zhinka_pozhezhnik_kolir2", "1F469-1F3FC-200D-1F692"},
    {"zhinka_pozhezhnik_kolir3", "1F469-1F3FD-200D-1F692"},
    {"zhinka_pozhezhnik_kolir4", "1F469-1F3FE-200D-1F692"},
    {"zhinka_pozhezhnik_kolir5", "1F469-1F3FF-200D-1F692"},
    {"zhinka_z_taktilnoyu_trostinoyu", "1F469-200D-1F9AF"},
    {"zhinka_z_taktilnoyu_trostinoyu_kolir1", "1F469-1F3FB-200D-1F9AF"},
    {"zhinka_z_taktilnoyu_trostinoyu_kolir2", "1F469-1F3FC-200D-1F9AF"},
    {"zhinka_z_taktilnoyu_trostinoyu_kolir3", "1F469-1F3FD-200D-1F9AF"},
    {"zhinka_z_taktilnoyu_trostinoyu_kolir4", "1F469-1F3FE-200D-1F9AF"},
    {"zhinka_z_taktilnoyu_trostinoyu_kolir5", "1F469-1F3FF-200D-1F9AF"},
    {"zhinka_z_taktilnoyu_trostinoyu_praviy_profil", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"zhinka_z_taktilnoyu_trostinoyu_praviy_profil_kolir1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"zhinka_z_taktilnoyu_trostinoyu_praviy_profil_kolir2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"zhinka_z_taktilnoyu_trostinoyu_praviy_profil_kolir3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"zhinka_z_taktilnoyu_trostinoyu_praviy_profil_kolir4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"zhinka_z_taktilnoyu_trostinoyu_praviy_profil_kolir5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"zhinka_u_vizku_z_elektroprivodom", "1F469-200D-1F9BC"},
    {"zhinka_u_vizku_z_elektroprivodom_kolir1", "1F469-1F3FB-200D-1F9BC"},
    {"zhinka_u_vizku_z_elektroprivodom_kolir2", "1F469-1F3FC-200D-1F9BC"},
    {"zhinka_u_vizku_z_elektroprivodom_kolir3", "1F469-1F3FD-200D-1F9BC"},
    {"zhinka_u_vizku_z_elektroprivodom_kolir4", "1F469-1F3FE-200D-1F9BC"},
    {"zhinka_u_vizku_z_elektroprivodom_kolir5", "1F469-1F3FF-200D-1F9BC"},
    {"zhinka_u_vizku_z_elektroprivodom_praviy_profil", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"zhinka_u_vizku_z_elektroprivodom_praviy_profil_kolir1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"zhinka_u_vizku_z_elektroprivodom_praviy_profil_kolir2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"zhinka_u_vizku_z_elektroprivodom_praviy_profil_kolir3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"zhinka_u_vizku_z_elektroprivodom_praviy_profil_kolir4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"zhinka_u_vizku_z_elektroprivodom_praviy_profil_kolir5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"zhinka_u_vizku_z_ruchnim_privodom", "1F469-200D-1F9BD"},
    {"zhinka_u_vizku_z_ruchnim_privodom_kolir1", "1F469-1F3FB-200D-1F9BD"},
    {"zhinka_u_vizku_z_ruchnim_privodom_kolir2", "1F469-1F3FC-200D-1F9BD"},
    {"zhinka_u_vizku_z_ruchnim_privodom_kolir3", "1F469-1F3FD-200D-1F9BD"},
    {"zhinka_u_vizku_z_ruchnim_privodom_kolir4", "1F469-1F3FE-200D-1F9BD"},
    {"zhinka_u_vizku_z_ruchnim_privodom_kolir5", "1F469-1F3FF-200D-1F9BD"},
    {"zhinka_u_vizku_z_ruchnim_privodom_praviy_profil", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"zhinka_u_vizku_z_ruchnim_privodom_praviy_profil_kolir1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"zhinka_u_vizku_z_ruchnim_privodom_praviy_profil_kolir2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"zhinka_u_vizku_z_ruchnim_privodom_praviy_profil_kolir3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"zhinka_u_vizku_z_ruchnim_privodom_praviy_profil_kolir4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"zhinka_u_vizku_z_ruchnim_privodom_praviy_profil_kolir5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"lyudina_shcho_yde_pishki_praviy_profil", "1F6B6-200D-27A1-FE0F"},
    {"lyudina_shcho_yde_pishki_praviy_profil_kolir1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"lyudina_shcho_yde_pishki_praviy_profil_kolir2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"lyudina_shcho_yde_pishki_praviy_profil_kolir3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"lyudina_shcho_yde_pishki_praviy_profil_kolir4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"lyudina_shcho_yde_pishki_praviy_profil_kolir5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"lyudina_navkolishkah_praviy_profil", "1F9CE-200D-27A1-FE0F"},
    {"lyudina_navkolishkah_praviy_profil_kolir1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"lyudina_navkolishkah_praviy_profil_kolir2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"lyudina_navkolishkah_praviy_profil_kolir3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"lyudina_navkolishkah_praviy_profil_kolir4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"lyudina_navkolishkah_praviy_profil_kolir5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"medik", "1F9D1-200D-2695-FE0F"},
    {"medik_kolir1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"medik_kolir2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"medik_kolir3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"medik_kolir4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"medik_kolir5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"suddya", "1F9D1-200D-2696-FE0F"},
    {"suddya_kolir1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"suddya_kolir2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"suddya_kolir3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"suddya_kolir4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"suddya_kolir5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"aviapilot", "1F9D1-200D-2708-FE0F"},
    {"aviapilot_kolir1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"aviapilot_kolir2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"aviapilot_kolir3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"aviapilot_kolir4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"aviapilot_kolir5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"selyanin", "1F9D1-200D-1F33E"},
    {"selyanin_kolir1", "1F9D1-1F3FB-200D-1F33E"},
    {"selyanin_kolir2", "1F9D1-1F3FC-200D-1F33E"},
    {"selyanin_kolir3", "1F9D1-1F3FD-200D-1F33E"},
    {"selyanin_kolir4", "1F9D1-1F3FE-200D-1F33E"},
    {"selyanin_kolir5", "1F9D1-1F3FF-200D-1F33E"},
    {"kuhovar", "1F9D1-200D-1F373"},
    {"kuhovar_kolir1", "1F9D1-1F3FB-200D-1F373"},
    {"kuhovar_kolir2", "1F9D1-1F3FC-200D-1F373"},
    {"kuhovar_kolir3", "1F9D1-1F3FD-200D-1F373"},
    {"kuhovar_kolir4", "1F9D1-1F3FE-200D-1F373"},
    {"kuhovar_kolir5", "1F9D1-1F3FF-200D-1F373"},
    {"lyudina_goduie_ditinu", "1F9D1-200D-1F37C"},
    {"lyudina_goduie_ditinu_kolir1", "1F9D1-1F3FB-200D-1F37C"},
    {"lyudina_goduie_ditinu_kolir2", "1F9D1-1F3FC-200D-1F37C"},
    {"lyudina_goduie_ditinu_kolir3", "1F9D1-1F3FD-200D-1F37C"},
    {"lyudina_goduie_ditinu_kolir4", "1F9D1-1F3FE-200D-1F37C"},
    {"lyudina_goduie_ditinu_kolir5", "1F9D1-1F3FF-200D-1F37C"},
    {"miks_klaus", "1F9D1-200D-1F384"},
    {"miks_klaus_kolir1", "1F9D1-1F3FB-200D-1F384"},
    {"miks_klaus_kolir2", "1F9D1-1F3FC-200D-1F384"},
    {"miks_klaus_kolir3", "1F9D1-1F3FD-200D-1F384"},
    {"miks_klaus_kolir4", "1F9D1-1F3FE-200D-1F384"},
    {"miks_klaus_kolir5", "1F9D1-1F3FF-200D-1F384"},
    {"sluhach", "1F9D1-200D-1F393"},
    {"sluhach_kolir1", "1F9D1-1F3FB-200D-1F393"},
    {"sluhach_kolir2", "1F9D1-1F3FC-200D-1F393"},
    {"sluhach_kolir3", "1F9D1-1F3FD-200D-1F393"},
    {"sluhach_kolir4", "1F9D1-1F3FE-200D-1F393"},
    {"sluhach_kolir5", "1F9D1-1F3FF-200D-1F393"},
    {"vokalist", "1F9D1-200D-1F3A4"},
    {"vokalist_kolir1", "1F9D1-1F3FB-200D-1F3A4"},
    {"vokalist_kolir2", "1F9D1-1F3FC-200D-1F3A4"},
    {"vokalist_kolir3", "1F9D1-1F3FD-200D-1F3A4"},
    {"vokalist_kolir4", "1F9D1-1F3FE-200D-1F3A4"},
    {"vokalist_kolir5", "1F9D1-1F3FF-200D-1F3A4"},
    {"hudozhnik", "1F9D1-200D-1F3A8"},
    {"hudozhnik_kolir1", "1F9D1-1F3FB-200D-1F3A8"},
    {"hudozhnik_kolir2", "1F9D1-1F3FC-200D-1F3A8"},
    {"hudozhnik_kolir3", "1F9D1-1F3FD-200D-1F3A8"},
    {"hudozhnik_kolir4", "1F9D1-1F3FE-200D-1F3A8"},
    {"hudozhnik_kolir5", "1F9D1-1F3FF-200D-1F3A8"},
    {"osvityanin", "1F9D1-200D-1F3EB"},
    {"osvityanin_kolir1", "1F9D1-1F3FB-200D-1F3EB"},
    {"osvityanin_kolir2", "1F9D1-1F3FC-200D-1F3EB"},
    {"osvityanin_kolir3", "1F9D1-1F3FD-200D-1F3EB"},
    {"osvityanin_kolir4", "1F9D1-1F3FE-200D-1F3EB"},
    {"osvityanin_kolir5", "1F9D1-1F3FF-200D-1F3EB"},
    {"fabrichniy_pracivnik", "1F9D1-200D-1F3ED"},
    {"fabrichniy_pracivnik_kolir1", "1F9D1-1F3FB-200D-1F3ED"},
    {"fabrichniy_pracivnik_kolir2", "1F9D1-1F3FC-200D-1F3ED"},
    {"fabrichniy_pracivnik_kolir3", "1F9D1-1F3FD-200D-1F3ED"},
    {"fabrichniy_pracivnik_kolir4", "1F9D1-1F3FE-200D-1F3ED"},
    {"fabrichniy_pracivnik_kolir5", "1F9D1-1F3FF-200D-1F3ED"},
    {"tehnichniy_pracivnik", "1F9D1-200D-1F4BB"},
    {"tehnichniy_pracivnik_kolir1", "1F9D1-1F3FB-200D-1F4BB"},
    {"tehnichniy_pracivnik_kolir2", "1F9D1-1F3FC-200D-1F4BB"},
    {"tehnichniy_pracivnik_kolir3", "1F9D1-1F3FD-200D-1F4BB"},
    {"tehnichniy_pracivnik_kolir4", "1F9D1-1F3FE-200D-1F4BB"},
    {"tehnichniy_pracivnik_kolir5", "1F9D1-1F3FF-200D-1F4BB"},
    {"ofisniy_pracivnik", "1F9D1-200D-1F4BC"},
    {"ofisniy_pracivnik_kolir1", "1F9D1-1F3FB-200D-1F4BC"},
    {"ofisniy_pracivnik_kolir2", "1F9D1-1F3FC-200D-1F4BC"},
    {"ofisniy_pracivnik_kolir3", "1F9D1-1F3FD-200D-1F4BC"},
    {"ofisniy_pracivnik_kolir4", "1F9D1-1F3FE-200D-1F4BC"},
    {"ofisniy_pracivnik_kolir5", "1F9D1-1F3FF-200D-1F4BC"},
    {"fahivec_iz_mehaniki", "1F9D1-200D-1F527"},
    {"fahivec_iz_mehaniki_kolir1", "1F9D1-1F3FB-200D-1F527"},
    {"fahivec_iz_mehaniki_kolir2", "1F9D1-1F3FC-200D-1F527"},
    {"fahivec_iz_mehaniki_kolir3", "1F9D1-1F3FD-200D-1F527"},
    {"fahivec_iz_mehaniki_kolir4", "1F9D1-1F3FE-200D-1F527"},
    {"fahivec_iz_mehaniki_kolir5", "1F9D1-1F3FF-200D-1F527"},
    {"naukoviy_pracivnik", "1F9D1-200D-1F52C"},
    {"naukoviy_pracivnik_kolir1", "1F9D1-1F3FB-200D-1F52C"},
    {"naukoviy_pracivnik_kolir2", "1F9D1-1F3FC-200D-1F52C"},
    {"naukoviy_pracivnik_kolir3", "1F9D1-1F3FD-200D-1F52C"},
    {"naukoviy_pracivnik_kolir4", "1F9D1-1F3FE-200D-1F52C"},
    {"naukoviy_pracivnik_kolir5", "1F9D1-1F3FF-200D-1F52C"},
    {"kosmonavt", "1F9D1-200D-1F680"},
    {"kosmonavt_kolir1", "1F9D1-1F3FB-200D-1F680"},
    {"kosmonavt_kolir2", "1F9D1-1F3FC-200D-1F680"},
    {"kosmonavt_kolir3", "1F9D1-1F3FD-200D-1F680"},
    {"kosmonavt_kolir4", "1F9D1-1F3FE-200D-1F680"},
    {"kosmonavt_kolir5", "1F9D1-1F3FF-200D-1F680"},
    {"vogneborec", "1F9D1-200D-1F692"},
    {"vogneborec_kolir1", "1F9D1-1F3FB-200D-1F692"},
    {"vogneborec_kolir2", "1F9D1-1F3FC-200D-1F692"},
    {"vogneborec_kolir3", "1F9D1-1F3FD-200D-1F692"},
    {"vogneborec_kolir4", "1F9D1-1F3FE-200D-1F692"},
    {"vogneborec_kolir5", "1F9D1-1F3FF-200D-1F692"},
    {"lyudina_z_taktilnim_cipkom", "1F9D1-200D-1F9AF"},
    {"lyudina_z_taktilnim_cipkom_kolir1", "1F9D1-1F3FB-200D-1F9AF"},
    {"lyudina_z_taktilnim_cipkom_kolir2", "1F9D1-1F3FC-200D-1F9AF"},
    {"lyudina_z_taktilnim_cipkom_kolir3", "1F9D1-1F3FD-200D-1F9AF"},
    {"lyudina_z_taktilnim_cipkom_kolir4", "1F9D1-1F3FE-200D-1F9AF"},
    {"lyudina_z_taktilnim_cipkom_kolir5", "1F9D1-1F3FF-200D-1F9AF"},
    {"lyudina_z_taktilnim_cipkom_praviy_profil", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"lyudina_z_taktilnim_cipkom_praviy_profil_kolir1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"lyudina_z_taktilnim_cipkom_praviy_profil_kolir2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"lyudina_z_taktilnim_cipkom_praviy_profil_kolir3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"lyudina_z_taktilnim_cipkom_praviy_profil_kolir4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"lyudina_z_taktilnim_cipkom_praviy_profil_kolir5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"lyudina_na_motorizovanomu_vizku", "1F9D1-200D-1F9BC"},
    {"lyudina_na_motorizovanomu_vizku_kolir1", "1F9D1-1F3FB-200D-1F9BC"},
    {"lyudina_na_motorizovanomu_vizku_kolir2", "1F9D1-1F3FC-200D-1F9BC"},
    {"lyudina_na_motorizovanomu_vizku_kolir3", "1F9D1-1F3FD-200D-1F9BC"},
    {"lyudina_na_motorizovanomu_vizku_kolir4", "1F9D1-1F3FE-200D-1F9BC"},
    {"lyudina_na_motorizovanomu_vizku_kolir5", "1F9D1-1F3FF-200D-1F9BC"},
    {"lyudina_na_motorizovanomu_vizku_praviy_profil", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"lyudina_na_motorizovanomu_vizku_praviy_profil_kolir1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"lyudina_na_motorizovanomu_vizku_praviy_profil_kolir2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"lyudina_na_motorizovanomu_vizku_praviy_profil_kolir3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"lyudina_na_motorizovanomu_vizku_praviy_profil_kolir4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"lyudina_na_motorizovanomu_vizku_praviy_profil_kolir5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"lyudina_u_vizku_z_ruchnim_keruvannyam", "1F9D1-200D-1F9BD"},
    {"lyudina_u_vizku_z_ruchnim_keruvannyam_kolir1", "1F9D1-1F3FB-200D-1F9BD"},
    {"lyudina_u_vizku_z_ruchnim_keruvannyam_kolir2", "1F9D1-1F3FC-200D-1F9BD"},
    {"lyudina_u_vizku_z_ruchnim_keruvannyam_kolir3", "1F9D1-1F3FD-200D-1F9BD"},
    {"lyudina_u_vizku_z_ruchnim_keruvannyam_kolir4", "1F9D1-1F3FE-200D-1F9BD"},
    {"lyudina_u_vizku_z_ruchnim_keruvannyam_kolir5", "1F9D1-1F3FF-200D-1F9BD"},
    {"lyudina_u_vizku_z_ruchnim_keruvannyam_praviy_profil", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"lyudina_u_vizku_z_ruchnim_keruvannyam_praviy_profil_kolir1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"lyudina_u_vizku_z_ruchnim_keruvannyam_praviy_profil_kolir2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"lyudina_u_vizku_z_ruchnim_keruvannyam_praviy_profil_kolir3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"lyudina_u_vizku_z_ruchnim_keruvannyam_praviy_profil_kolir4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"lyudina_u_vizku_z_ruchnim_keruvannyam_praviy_profil_kolir5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"zhinka_shcho_vede_myach", "26F9-FE0F-200D-2640-FE0F"},
    {"zhinka_shcho_vede_myach_kolir1", "26F9-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_vede_myach_kolir2", "26F9-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_vede_myach_kolir3", "26F9-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_vede_myach_kolir4", "26F9-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_vede_myach_kolir5", "26F9-1F3FF-200D-2640-FE0F"},
    {"cholovik_shcho_vede_myach", "26F9-FE0F-200D-2642-FE0F"},
    {"cholovik_shcho_vede_myach_kolir1", "26F9-1F3FB-200D-2642-FE0F"},
    {"cholovik_shcho_vede_myach_kolir2", "26F9-1F3FC-200D-2642-FE0F"},
    {"cholovik_shcho_vede_myach_kolir3", "26F9-1F3FD-200D-2642-FE0F"},
    {"cholovik_shcho_vede_myach_kolir4", "26F9-1F3FE-200D-2642-FE0F"},
    {"cholovik_shcho_vede_myach_kolir5", "26F9-1F3FF-200D-2642-FE0F"},
    {"zhinka_shcho_bizhit", "1F3C3-200D-2640-FE0F"},
    {"zhinka_shcho_bizhit_kolir1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_bizhit_kolir2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_bizhit_kolir3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_bizhit_kolir4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_bizhit_kolir5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"zhinka_shcho_bizhit_praviy_profil", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_shcho_bizhit_praviy_profil_kolir1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_shcho_bizhit_praviy_profil_kolir2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_shcho_bizhit_praviy_profil_kolir3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_shcho_bizhit_praviy_profil_kolir4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_shcho_bizhit_praviy_profil_kolir5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"cholovik_shcho_bizhit", "1F3C3-200D-2642-FE0F"},
    {"cholovik_shcho_bizhit_kolir1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"cholovik_shcho_bizhit_kolir2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"cholovik_shcho_bizhit_kolir3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"cholovik_shcho_bizhit_kolir4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"cholovik_shcho_bizhit_kolir5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"cholovik_shcho_bizhit_praviy_profil", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_shcho_bizhit_praviy_profil_kolir1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_shcho_bizhit_praviy_profil_kolir2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_shcho_bizhit_praviy_profil_kolir3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_shcho_bizhit_praviy_profil_kolir4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_shcho_bizhit_praviy_profil_kolir5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"serfingistka", "1F3C4-200D-2640-FE0F"},
    {"serfingistka_kolir1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"serfingistka_kolir2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"serfingistka_kolir3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"serfingistka_kolir4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"serfingistka_kolir5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"serfingist", "1F3C4-200D-2642-FE0F"},
    {"serfingist_kolir1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"serfingist_kolir2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"serfingist_kolir3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"serfingist_kolir4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"serfingist_kolir5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"plavchinya", "1F3CA-200D-2640-FE0F"},
    {"plavchinya_kolir1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"plavchinya_kolir2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"plavchinya_kolir3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"plavchinya_kolir4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"plavchinya_kolir5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"plavec", "1F3CA-200D-2642-FE0F"},
    {"plavec_kolir1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"plavec_kolir2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"plavec_kolir3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"plavec_kolir4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"plavec_kolir5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"shtangistka", "1F3CB-FE0F-200D-2640-FE0F"},
    {"shtangistka_kolir1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"shtangistka_kolir2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"shtangistka_kolir3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"shtangistka_kolir4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"shtangistka_kolir5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"shtangist", "1F3CB-FE0F-200D-2642-FE0F"},
    {"shtangist_kolir1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"shtangist_kolir2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"shtangist_kolir3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"shtangist_kolir4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"shtangist_kolir5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"golfistka", "1F3CC-FE0F-200D-2640-FE0F"},
    {"golfistka_kolir1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"golfistka_kolir2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"golfistka_kolir3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"golfistka_kolir4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"golfistka_kolir5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"golfist", "1F3CC-FE0F-200D-2642-FE0F"},
    {"golfist_kolir1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"golfist_kolir2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"golfist_kolir3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"golfist_kolir4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"golfist_kolir5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"zhinka_policeyskiy", "1F46E-200D-2640-FE0F"},
    {"zhinka_policeyskiy_kolir1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"zhinka_policeyskiy_kolir2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"zhinka_policeyskiy_kolir3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"zhinka_policeyskiy_kolir4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"zhinka_policeyskiy_kolir5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"policeyskiy", "1F46E-200D-2642-FE0F"},
    {"policeyskiy_kolir1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"policeyskiy_kolir2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"policeyskiy_kolir3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"policeyskiy_kolir4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"policeyskiy_kolir5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"zhinki_z_krolyachimi_vuhami", "1F46F-200D-2640-FE0F"},
    {"zhinki_z_krolyachimi_vuhami_kolir1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"zhinki_z_krolyachimi_vuhami_kolir1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"zhinki_z_krolyachimi_vuhami_kolir1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"zhinki_z_krolyachimi_vuhami_kolir1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"zhinki_z_krolyachimi_vuhami_kolir2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"zhinki_z_krolyachimi_vuhami_kolir2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"zhinki_z_krolyachimi_vuhami_kolir2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"zhinki_z_krolyachimi_vuhami_kolir2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"zhinki_z_krolyachimi_vuhami_kolir3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"zhinki_z_krolyachimi_vuhami_kolir3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"zhinki_z_krolyachimi_vuhami_kolir3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"zhinki_z_krolyachimi_vuhami_kolir3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"zhinki_z_krolyachimi_vuhami_kolir4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"zhinki_z_krolyachimi_vuhami_kolir4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"zhinki_z_krolyachimi_vuhami_kolir4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"zhinki_z_krolyachimi_vuhami_kolir4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"zhinki_z_krolyachimi_vuhami_kolir5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"zhinki_z_krolyachimi_vuhami_kolir5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"zhinki_z_krolyachimi_vuhami_kolir5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"zhinki_z_krolyachimi_vuhami_kolir5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"zhinki_z_krolyachimi_vuhami_kolir1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"zhinki_z_krolyachimi_vuhami_kolir2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"zhinki_z_krolyachimi_vuhami_kolir3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"zhinki_z_krolyachimi_vuhami_kolir4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"zhinki_z_krolyachimi_vuhami_kolir5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"choloviki_z_krolyachimi_vuhami", "1F46F-200D-2642-FE0F"},
    {"choloviki_z_krolyachimi_vuhami_kolir1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"choloviki_z_krolyachimi_vuhami_kolir1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"choloviki_z_krolyachimi_vuhami_kolir1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"choloviki_z_krolyachimi_vuhami_kolir1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"choloviki_z_krolyachimi_vuhami_kolir2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"choloviki_z_krolyachimi_vuhami_kolir2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"choloviki_z_krolyachimi_vuhami_kolir2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"choloviki_z_krolyachimi_vuhami_kolir2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"choloviki_z_krolyachimi_vuhami_kolir3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"choloviki_z_krolyachimi_vuhami_kolir3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"choloviki_z_krolyachimi_vuhami_kolir3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"choloviki_z_krolyachimi_vuhami_kolir3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"choloviki_z_krolyachimi_vuhami_kolir4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"choloviki_z_krolyachimi_vuhami_kolir4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"choloviki_z_krolyachimi_vuhami_kolir4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"choloviki_z_krolyachimi_vuhami_kolir4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"choloviki_z_krolyachimi_vuhami_kolir5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"choloviki_z_krolyachimi_vuhami_kolir5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"choloviki_z_krolyachimi_vuhami_kolir5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"choloviki_z_krolyachimi_vuhami_kolir5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"choloviki_z_krolyachimi_vuhami_kolir1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"choloviki_z_krolyachimi_vuhami_kolir2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"choloviki_z_krolyachimi_vuhami_kolir3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"choloviki_z_krolyachimi_vuhami_kolir4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"choloviki_z_krolyachimi_vuhami_kolir5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"zhinka_u_fati", "1F470-200D-2640-FE0F"},
    {"zhinka_u_fati_kolir1", "1F470-1F3FB-200D-2640-FE0F"},
    {"zhinka_u_fati_kolir2", "1F470-1F3FC-200D-2640-FE0F"},
    {"zhinka_u_fati_kolir3", "1F470-1F3FD-200D-2640-FE0F"},
    {"zhinka_u_fati_kolir4", "1F470-1F3FE-200D-2640-FE0F"},
    {"zhinka_u_fati_kolir5", "1F470-1F3FF-200D-2640-FE0F"},
    {"cholovik_u_fati", "1F470-200D-2642-FE0F"},
    {"cholovik_u_fati_kolir1", "1F470-1F3FB-200D-2642-FE0F"},
    {"cholovik_u_fati_kolir2", "1F470-1F3FC-200D-2642-FE0F"},
    {"cholovik_u_fati_kolir3", "1F470-1F3FD-200D-2642-FE0F"},
    {"cholovik_u_fati_kolir4", "1F470-1F3FE-200D-2642-FE0F"},
    {"cholovik_u_fati_kolir5", "1F470-1F3FF-200D-2642-FE0F"},
    {"zhinka_z_bilyavim_volossyam", "1F471-200D-2640-FE0F"},
    {"zhinka_z_bilyavim_volossyam_kolir1", "1F471-1F3FB-200D-2640-FE0F"},
    {"zhinka_z_bilyavim_volossyam_kolir2", "1F471-1F3FC-200D-2640-FE0F"},
    {"zhinka_z_bilyavim_volossyam_kolir3", "1F471-1F3FD-200D-2640-FE0F"},
    {"zhinka_z_bilyavim_volossyam_kolir4", "1F471-1F3FE-200D-2640-FE0F"},
    {"zhinka_z_bilyavim_volossyam_kolir5", "1F471-1F3FF-200D-2640-FE0F"},
    {"cholovik_iz_bilyavim_volossyam", "1F471-200D-2642-FE0F"},
    {"cholovik_iz_bilyavim_volossyam_kolir1", "1F471-1F3FB-200D-2642-FE0F"},
    {"cholovik_iz_bilyavim_volossyam_kolir2", "1F471-1F3FC-200D-2642-FE0F"},
    {"cholovik_iz_bilyavim_volossyam_kolir3", "1F471-1F3FD-200D-2642-FE0F"},
    {"cholovik_iz_bilyavim_volossyam_kolir4", "1F471-1F3FE-200D-2642-FE0F"},
    {"cholovik_iz_bilyavim_volossyam_kolir5", "1F471-1F3FF-200D-2642-FE0F"},
    {"zhinka_v_tyurbani", "1F473-200D-2640-FE0F"},
    {"zhinka_v_tyurbani_kolir1", "1F473-1F3FB-200D-2640-FE0F"},
    {"zhinka_v_tyurbani_kolir2", "1F473-1F3FC-200D-2640-FE0F"},
    {"zhinka_v_tyurbani_kolir3", "1F473-1F3FD-200D-2640-FE0F"},
    {"zhinka_v_tyurbani_kolir4", "1F473-1F3FE-200D-2640-FE0F"},
    {"zhinka_v_tyurbani_kolir5", "1F473-1F3FF-200D-2640-FE0F"},
    {"cholovik_u_tyurbani", "1F473-200D-2642-FE0F"},
    {"cholovik_u_tyurbani_kolir1", "1F473-1F3FB-200D-2642-FE0F"},
    {"cholovik_u_tyurbani_kolir2", "1F473-1F3FC-200D-2642-FE0F"},
    {"cholovik_u_tyurbani_kolir3", "1F473-1F3FD-200D-2642-FE0F"},
    {"cholovik_u_tyurbani_kolir4", "1F473-1F3FE-200D-2642-FE0F"},
    {"cholovik_u_tyurbani_kolir5", "1F473-1F3FF-200D-2642-FE0F"},
    {"zhinka_budivelnik", "1F477-200D-2640-FE0F"},
    {"zhinka_budivelnik_kolir1", "1F477-1F3FB-200D-2640-FE0F"},
    {"zhinka_budivelnik_kolir2", "1F477-1F3FC-200D-2640-FE0F"},
    {"zhinka_budivelnik_kolir3", "1F477-1F3FD-200D-2640-FE0F"},
    {"zhinka_budivelnik_kolir4", "1F477-1F3FE-200D-2640-FE0F"},
    {"zhinka_budivelnik_kolir5", "1F477-1F3FF-200D-2640-FE0F"},
    {"cholovik_budivelnik", "1F477-200D-2642-FE0F"},
    {"cholovik_budivelnik_kolir1", "1F477-1F3FB-200D-2642-FE0F"},
    {"cholovik_budivelnik_kolir2", "1F477-1F3FC-200D-2642-FE0F"},
    {"cholovik_budivelnik_kolir3", "1F477-1F3FD-200D-2642-FE0F"},
    {"cholovik_budivelnik_kolir4", "1F477-1F3FE-200D-2642-FE0F"},
    {"cholovik_budivelnik_kolir5", "1F477-1F3FF-200D-2642-FE0F"},
    {"zhinka_shcho_prostyagaie_ruku_dopomogi", "1F481-200D-2640-FE0F"},
    {"zhinka_shcho_prostyagaie_ruku_dopomogi_kolir1", "1F481-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_prostyagaie_ruku_dopomogi_kolir2", "1F481-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_prostyagaie_ruku_dopomogi_kolir3", "1F481-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_prostyagaie_ruku_dopomogi_kolir4", "1F481-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_prostyagaie_ruku_dopomogi_kolir5", "1F481-1F3FF-200D-2640-FE0F"},
    {"cholovik_shcho_prostyagaie_ruku_dopomogi", "1F481-200D-2642-FE0F"},
    {"cholovik_shcho_prostyagaie_ruku_dopomogi_kolir1", "1F481-1F3FB-200D-2642-FE0F"},
    {"cholovik_shcho_prostyagaie_ruku_dopomogi_kolir2", "1F481-1F3FC-200D-2642-FE0F"},
    {"cholovik_shcho_prostyagaie_ruku_dopomogi_kolir3", "1F481-1F3FD-200D-2642-FE0F"},
    {"cholovik_shcho_prostyagaie_ruku_dopomogi_kolir4", "1F481-1F3FE-200D-2642-FE0F"},
    {"cholovik_shcho_prostyagaie_ruku_dopomogi_kolir5", "1F481-1F3FF-200D-2642-FE0F"},
    {"vartova", "1F482-200D-2640-FE0F"},
    {"vartova_kolir1", "1F482-1F3FB-200D-2640-FE0F"},
    {"vartova_kolir2", "1F482-1F3FC-200D-2640-FE0F"},
    {"vartova_kolir3", "1F482-1F3FD-200D-2640-FE0F"},
    {"vartova_kolir4", "1F482-1F3FE-200D-2640-FE0F"},
    {"vartova_kolir5", "1F482-1F3FF-200D-2640-FE0F"},
    {"vartoviy", "1F482-200D-2642-FE0F"},
    {"vartoviy_kolir1", "1F482-1F3FB-200D-2642-FE0F"},
    {"vartoviy_kolir2", "1F482-1F3FC-200D-2642-FE0F"},
    {"vartoviy_kolir3", "1F482-1F3FD-200D-2642-FE0F"},
    {"vartoviy_kolir4", "1F482-1F3FE-200D-2642-FE0F"},
    {"vartoviy_kolir5", "1F482-1F3FF-200D-2642-FE0F"},
    {"zhinka_yakiy_roblyat_masazh_golovi", "1F486-200D-2640-FE0F"},
    {"zhinka_yakiy_roblyat_masazh_golovi_kolir1", "1F486-1F3FB-200D-2640-FE0F"},
    {"zhinka_yakiy_roblyat_masazh_golovi_kolir2", "1F486-1F3FC-200D-2640-FE0F"},
    {"zhinka_yakiy_roblyat_masazh_golovi_kolir3", "1F486-1F3FD-200D-2640-FE0F"},
    {"zhinka_yakiy_roblyat_masazh_golovi_kolir4", "1F486-1F3FE-200D-2640-FE0F"},
    {"zhinka_yakiy_roblyat_masazh_golovi_kolir5", "1F486-1F3FF-200D-2640-FE0F"},
    {"cholovik_yakomu_roblyat_masazh_golovi", "1F486-200D-2642-FE0F"},
    {"cholovik_yakomu_roblyat_masazh_golovi_kolir1", "1F486-1F3FB-200D-2642-FE0F"},
    {"cholovik_yakomu_roblyat_masazh_golovi_kolir2", "1F486-1F3FC-200D-2642-FE0F"},
    {"cholovik_yakomu_roblyat_masazh_golovi_kolir3", "1F486-1F3FD-200D-2642-FE0F"},
    {"cholovik_yakomu_roblyat_masazh_golovi_kolir4", "1F486-1F3FE-200D-2642-FE0F"},
    {"cholovik_yakomu_roblyat_masazh_golovi_kolir5", "1F486-1F3FF-200D-2642-FE0F"},
    {"zhinka_yakiy_roblyat_strizhku", "1F487-200D-2640-FE0F"},
    {"zhinka_yakiy_roblyat_strizhku_kolir1", "1F487-1F3FB-200D-2640-FE0F"},
    {"zhinka_yakiy_roblyat_strizhku_kolir2", "1F487-1F3FC-200D-2640-FE0F"},
    {"zhinka_yakiy_roblyat_strizhku_kolir3", "1F487-1F3FD-200D-2640-FE0F"},
    {"zhinka_yakiy_roblyat_strizhku_kolir4", "1F487-1F3FE-200D-2640-FE0F"},
    {"zhinka_yakiy_roblyat_strizhku_kolir5", "1F487-1F3FF-200D-2640-FE0F"},
    {"cholovik_yakomu_roblyat_strizhku", "1F487-200D-2642-FE0F"},
    {"cholovik_yakomu_roblyat_strizhku_kolir1", "1F487-1F3FB-200D-2642-FE0F"},
    {"cholovik_yakomu_roblyat_strizhku_kolir2", "1F487-1F3FC-200D-2642-FE0F"},
    {"cholovik_yakomu_roblyat_strizhku_kolir3", "1F487-1F3FD-200D-2642-FE0F"},
    {"cholovik_yakomu_roblyat_strizhku_kolir4", "1F487-1F3FE-200D-2642-FE0F"},
    {"cholovik_yakomu_roblyat_strizhku_kolir5", "1F487-1F3FF-200D-2642-FE0F"},
    {"zhinka_detektiv", "1F575-FE0F-200D-2640-FE0F"},
    {"zhinka_detektiv_kolir1", "1F575-1F3FB-200D-2640-FE0F"},
    {"zhinka_detektiv_kolir2", "1F575-1F3FC-200D-2640-FE0F"},
    {"zhinka_detektiv_kolir3", "1F575-1F3FD-200D-2640-FE0F"},
    {"zhinka_detektiv_kolir4", "1F575-1F3FE-200D-2640-FE0F"},
    {"zhinka_detektiv_kolir5", "1F575-1F3FF-200D-2640-FE0F"},
    {"cholovik_detektiv", "1F575-FE0F-200D-2642-FE0F"},
    {"cholovik_detektiv_kolir1", "1F575-1F3FB-200D-2642-FE0F"},
    {"cholovik_detektiv_kolir2", "1F575-1F3FC-200D-2642-FE0F"},
    {"cholovik_detektiv_kolir3", "1F575-1F3FD-200D-2642-FE0F"},
    {"cholovik_detektiv_kolir4", "1F575-1F3FE-200D-2642-FE0F"},
    {"cholovik_detektiv_kolir5", "1F575-1F3FF-200D-2642-FE0F"},
    {"zhinka_shcho_zhestom_pokazuie_ni", "1F645-200D-2640-FE0F"},
    {"zhinka_shcho_zhestom_pokazuie_ni_kolir1", "1F645-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_zhestom_pokazuie_ni_kolir2", "1F645-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_zhestom_pokazuie_ni_kolir3", "1F645-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_zhestom_pokazuie_ni_kolir4", "1F645-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_zhestom_pokazuie_ni_kolir5", "1F645-1F3FF-200D-2640-FE0F"},
    {"cholovik_shcho_zhestom_pokazuie_ni", "1F645-200D-2642-FE0F"},
    {"cholovik_shcho_zhestom_pokazuie_ni_kolir1", "1F645-1F3FB-200D-2642-FE0F"},
    {"cholovik_shcho_zhestom_pokazuie_ni_kolir2", "1F645-1F3FC-200D-2642-FE0F"},
    {"cholovik_shcho_zhestom_pokazuie_ni_kolir3", "1F645-1F3FD-200D-2642-FE0F"},
    {"cholovik_shcho_zhestom_pokazuie_ni_kolir4", "1F645-1F3FE-200D-2642-FE0F"},
    {"cholovik_shcho_zhestom_pokazuie_ni_kolir5", "1F645-1F3FF-200D-2642-FE0F"},
    {"zhinka_shcho_zhestom_pokazuie_okey", "1F646-200D-2640-FE0F"},
    {"zhinka_shcho_zhestom_pokazuie_okey_kolir1", "1F646-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_zhestom_pokazuie_okey_kolir2", "1F646-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_zhestom_pokazuie_okey_kolir3", "1F646-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_zhestom_pokazuie_okey_kolir4", "1F646-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_zhestom_pokazuie_okey_kolir5", "1F646-1F3FF-200D-2640-FE0F"},
    {"cholovik_shcho_zhestom_pokazuie_okey", "1F646-200D-2642-FE0F"},
    {"cholovik_shcho_zhestom_pokazuie_okey_kolir1", "1F646-1F3FB-200D-2642-FE0F"},
    {"cholovik_shcho_zhestom_pokazuie_okey_kolir2", "1F646-1F3FC-200D-2642-FE0F"},
    {"cholovik_shcho_zhestom_pokazuie_okey_kolir3", "1F646-1F3FD-200D-2642-FE0F"},
    {"cholovik_shcho_zhestom_pokazuie_okey_kolir4", "1F646-1F3FE-200D-2642-FE0F"},
    {"cholovik_shcho_zhestom_pokazuie_okey_kolir5", "1F646-1F3FF-200D-2642-FE0F"},
    {"zhinka_shcho_vklonyaietsya", "1F647-200D-2640-FE0F"},
    {"zhinka_shcho_vklonyaietsya_kolir1", "1F647-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_vklonyaietsya_kolir2", "1F647-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_vklonyaietsya_kolir3", "1F647-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_vklonyaietsya_kolir4", "1F647-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_vklonyaietsya_kolir5", "1F647-1F3FF-200D-2640-FE0F"},
    {"cholovik_shcho_vklonyaietsya", "1F647-200D-2642-FE0F"},
    {"cholovik_shcho_vklonyaietsya_kolir1", "1F647-1F3FB-200D-2642-FE0F"},
    {"cholovik_shcho_vklonyaietsya_kolir2", "1F647-1F3FC-200D-2642-FE0F"},
    {"cholovik_shcho_vklonyaietsya_kolir3", "1F647-1F3FD-200D-2642-FE0F"},
    {"cholovik_shcho_vklonyaietsya_kolir4", "1F647-1F3FE-200D-2642-FE0F"},
    {"cholovik_shcho_vklonyaietsya_kolir5", "1F647-1F3FF-200D-2642-FE0F"},
    {"zhinka_z_pidnyatoyu_rukoyu", "1F64B-200D-2640-FE0F"},
    {"zhinka_z_pidnyatoyu_rukoyu_kolir1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"zhinka_z_pidnyatoyu_rukoyu_kolir2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"zhinka_z_pidnyatoyu_rukoyu_kolir3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"zhinka_z_pidnyatoyu_rukoyu_kolir4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"zhinka_z_pidnyatoyu_rukoyu_kolir5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"cholovik_iz_pidnyatoyu_rukoyu", "1F64B-200D-2642-FE0F"},
    {"cholovik_iz_pidnyatoyu_rukoyu_kolir1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"cholovik_iz_pidnyatoyu_rukoyu_kolir2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"cholovik_iz_pidnyatoyu_rukoyu_kolir3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"cholovik_iz_pidnyatoyu_rukoyu_kolir4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"cholovik_iz_pidnyatoyu_rukoyu_kolir5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"pohmura_zhinka", "1F64D-200D-2640-FE0F"},
    {"pohmura_zhinka_kolir1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"pohmura_zhinka_kolir2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"pohmura_zhinka_kolir3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"pohmura_zhinka_kolir4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"pohmura_zhinka_kolir5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"pohmuriy_cholovik", "1F64D-200D-2642-FE0F"},
    {"pohmuriy_cholovik_kolir1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"pohmuriy_cholovik_kolir2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"pohmuriy_cholovik_kolir3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"pohmuriy_cholovik_kolir4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"pohmuriy_cholovik_kolir5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"zhinka_shcho_duietsya", "1F64E-200D-2640-FE0F"},
    {"zhinka_shcho_duietsya_kolir1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_duietsya_kolir2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_duietsya_kolir3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_duietsya_kolir4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_duietsya_kolir5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"cholovik_shcho_duietsya", "1F64E-200D-2642-FE0F"},
    {"cholovik_shcho_duietsya_kolir1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"cholovik_shcho_duietsya_kolir2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"cholovik_shcho_duietsya_kolir3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"cholovik_shcho_duietsya_kolir4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"cholovik_shcho_duietsya_kolir5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"zhinka_shcho_vesluie_na_chovni", "1F6A3-200D-2640-FE0F"},
    {"zhinka_shcho_vesluie_na_chovni_kolir1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_vesluie_na_chovni_kolir2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_vesluie_na_chovni_kolir3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_vesluie_na_chovni_kolir4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_vesluie_na_chovni_kolir5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"cholovik_shcho_vesluie_na_chovni", "1F6A3-200D-2642-FE0F"},
    {"cholovik_shcho_vesluie_na_chovni_kolir1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"cholovik_shcho_vesluie_na_chovni_kolir2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"cholovik_shcho_vesluie_na_chovni_kolir3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"cholovik_shcho_vesluie_na_chovni_kolir4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"cholovik_shcho_vesluie_na_chovni_kolir5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"velosipedistka", "1F6B4-200D-2640-FE0F"},
    {"velosipedistka_kolir1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"velosipedistka_kolir2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"velosipedistka_kolir3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"velosipedistka_kolir4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"velosipedistka_kolir5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"velosipedist", "1F6B4-200D-2642-FE0F"},
    {"velosipedist_kolir1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"velosipedist_kolir2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"velosipedist_kolir3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"velosipedist_kolir4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"velosipedist_kolir5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"girska_velosipedistka", "1F6B5-200D-2640-FE0F"},
    {"girska_velosipedistka_kolir1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"girska_velosipedistka_kolir2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"girska_velosipedistka_kolir3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"girska_velosipedistka_kolir4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"girska_velosipedistka_kolir5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"girskiy_velosipedist", "1F6B5-200D-2642-FE0F"},
    {"girskiy_velosipedist_kolir1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"girskiy_velosipedist_kolir2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"girskiy_velosipedist_kolir3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"girskiy_velosipedist_kolir4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"girskiy_velosipedist_kolir5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"zhinka_shcho_yde_pishki", "1F6B6-200D-2640-FE0F"},
    {"zhinka_shcho_yde_pishki_kolir1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_yde_pishki_kolir2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_yde_pishki_kolir3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_yde_pishki_kolir4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_yde_pishki_kolir5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"zhinka_shcho_yde_pishki_praviy_profil", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_shcho_yde_pishki_praviy_profil_kolir1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_shcho_yde_pishki_praviy_profil_kolir2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_shcho_yde_pishki_praviy_profil_kolir3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_shcho_yde_pishki_praviy_profil_kolir4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_shcho_yde_pishki_praviy_profil_kolir5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"cholovik_shcho_yde_pishki", "1F6B6-200D-2642-FE0F"},
    {"cholovik_shcho_yde_pishki_kolir1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"cholovik_shcho_yde_pishki_kolir2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"cholovik_shcho_yde_pishki_kolir3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"cholovik_shcho_yde_pishki_kolir4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"cholovik_shcho_yde_pishki_kolir5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"cholovik_shcho_yde_pishki_praviy_profil", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_shcho_yde_pishki_praviy_profil_kolir1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_shcho_yde_pishki_praviy_profil_kolir2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_shcho_yde_pishki_praviy_profil_kolir3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_shcho_yde_pishki_praviy_profil_kolir4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_shcho_yde_pishki_praviy_profil_kolir5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"zhinka_shcho_pidnosit_dolonyu_do_oblichchya", "1F926-200D-2640-FE0F"},
    {"zhinka_shcho_pidnosit_dolonyu_do_oblichchya_kolir1", "1F926-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_pidnosit_dolonyu_do_oblichchya_kolir2", "1F926-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_pidnosit_dolonyu_do_oblichchya_kolir3", "1F926-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_pidnosit_dolonyu_do_oblichchya_kolir4", "1F926-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_pidnosit_dolonyu_do_oblichchya_kolir5", "1F926-1F3FF-200D-2640-FE0F"},
    {"cholovik_iz_doloneyu_do_oblichchya", "1F926-200D-2642-FE0F"},
    {"cholovik_iz_doloneyu_do_oblichchya_kolir1", "1F926-1F3FB-200D-2642-FE0F"},
    {"cholovik_iz_doloneyu_do_oblichchya_kolir2", "1F926-1F3FC-200D-2642-FE0F"},
    {"cholovik_iz_doloneyu_do_oblichchya_kolir3", "1F926-1F3FD-200D-2642-FE0F"},
    {"cholovik_iz_doloneyu_do_oblichchya_kolir4", "1F926-1F3FE-200D-2642-FE0F"},
    {"cholovik_iz_doloneyu_do_oblichchya_kolir5", "1F926-1F3FF-200D-2642-FE0F"},
    {"zhinka_v_smokingu", "1F935-200D-2640-FE0F"},
    {"zhinka_v_smokingu_kolir1", "1F935-1F3FB-200D-2640-FE0F"},
    {"zhinka_v_smokingu_kolir2", "1F935-1F3FC-200D-2640-FE0F"},
    {"zhinka_v_smokingu_kolir3", "1F935-1F3FD-200D-2640-FE0F"},
    {"zhinka_v_smokingu_kolir4", "1F935-1F3FE-200D-2640-FE0F"},
    {"zhinka_v_smokingu_kolir5", "1F935-1F3FF-200D-2640-FE0F"},
    {"cholovik_u_smokingu", "1F935-200D-2642-FE0F"},
    {"cholovik_u_smokingu_kolir1", "1F935-1F3FB-200D-2642-FE0F"},
    {"cholovik_u_smokingu_kolir2", "1F935-1F3FC-200D-2642-FE0F"},
    {"cholovik_u_smokingu_kolir3", "1F935-1F3FD-200D-2642-FE0F"},
    {"cholovik_u_smokingu_kolir4", "1F935-1F3FE-200D-2642-FE0F"},
    {"cholovik_u_smokingu_kolir5", "1F935-1F3FF-200D-2642-FE0F"},
    {"zhinka_shcho_znizuie_plechima", "1F937-200D-2640-FE0F"},
    {"zhinka_shcho_znizuie_plechima_kolir1", "1F937-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_znizuie_plechima_kolir2", "1F937-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_znizuie_plechima_kolir3", "1F937-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_znizuie_plechima_kolir4", "1F937-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_znizuie_plechima_kolir5", "1F937-1F3FF-200D-2640-FE0F"},
    {"cholovik_shcho_znizuie_plechima", "1F937-200D-2642-FE0F"},
    {"cholovik_shcho_znizuie_plechima_kolir1", "1F937-1F3FB-200D-2642-FE0F"},
    {"cholovik_shcho_znizuie_plechima_kolir2", "1F937-1F3FC-200D-2642-FE0F"},
    {"cholovik_shcho_znizuie_plechima_kolir3", "1F937-1F3FD-200D-2642-FE0F"},
    {"cholovik_shcho_znizuie_plechima_kolir4", "1F937-1F3FE-200D-2642-FE0F"},
    {"cholovik_shcho_znizuie_plechima_kolir5", "1F937-1F3FF-200D-2642-FE0F"},
    {"zhinka_shcho_perevertaietsya_kolesom", "1F938-200D-2640-FE0F"},
    {"zhinka_shcho_perevertaietsya_kolesom_kolir1", "1F938-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_perevertaietsya_kolesom_kolir2", "1F938-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_perevertaietsya_kolesom_kolir3", "1F938-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_perevertaietsya_kolesom_kolir4", "1F938-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_perevertaietsya_kolesom_kolir5", "1F938-1F3FF-200D-2640-FE0F"},
    {"cholovik_shcho_perevertaietsya_kolesom", "1F938-200D-2642-FE0F"},
    {"cholovik_shcho_perevertaietsya_kolesom_kolir1", "1F938-1F3FB-200D-2642-FE0F"},
    {"cholovik_shcho_perevertaietsya_kolesom_kolir2", "1F938-1F3FC-200D-2642-FE0F"},
    {"cholovik_shcho_perevertaietsya_kolesom_kolir3", "1F938-1F3FD-200D-2642-FE0F"},
    {"cholovik_shcho_perevertaietsya_kolesom_kolir4", "1F938-1F3FE-200D-2642-FE0F"},
    {"cholovik_shcho_perevertaietsya_kolesom_kolir5", "1F938-1F3FF-200D-2642-FE0F"},
    {"zhonglerka", "1F939-200D-2640-FE0F"},
    {"zhonglerka_kolir1", "1F939-1F3FB-200D-2640-FE0F"},
    {"zhonglerka_kolir2", "1F939-1F3FC-200D-2640-FE0F"},
    {"zhonglerka_kolir3", "1F939-1F3FD-200D-2640-FE0F"},
    {"zhonglerka_kolir4", "1F939-1F3FE-200D-2640-FE0F"},
    {"zhonglerka_kolir5", "1F939-1F3FF-200D-2640-FE0F"},
    {"zhongler", "1F939-200D-2642-FE0F"},
    {"zhongler_kolir1", "1F939-1F3FB-200D-2642-FE0F"},
    {"zhongler_kolir2", "1F939-1F3FC-200D-2642-FE0F"},
    {"zhongler_kolir3", "1F939-1F3FD-200D-2642-FE0F"},
    {"zhongler_kolir4", "1F939-1F3FE-200D-2642-FE0F"},
    {"zhongler_kolir5", "1F939-1F3FF-200D-2642-FE0F"},
    {"borchinya", "1F93C-200D-2640-FE0F"},
    {"borchinya_kolir1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"borchinya_kolir1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"borchinya_kolir1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"borchinya_kolir1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"borchinya_kolir2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"borchinya_kolir2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"borchinya_kolir2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"borchinya_kolir2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"borchinya_kolir3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"borchinya_kolir3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"borchinya_kolir3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"borchinya_kolir3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"borchinya_kolir4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"borchinya_kolir4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"borchinya_kolir4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"borchinya_kolir4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"borchinya_kolir5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"borchinya_kolir5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"borchinya_kolir5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"borchinya_kolir5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"borchinya_kolir1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"borchinya_kolir2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"borchinya_kolir3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"borchinya_kolir4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"borchinya_kolir5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"borec", "1F93C-200D-2642-FE0F"},
    {"borec_kolir1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"borec_kolir1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"borec_kolir1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"borec_kolir1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"borec_kolir2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"borec_kolir2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"borec_kolir2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"borec_kolir2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"borec_kolir3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"borec_kolir3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"borec_kolir3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"borec_kolir3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"borec_kolir4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"borec_kolir4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"borec_kolir4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"borec_kolir4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"borec_kolir5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"borec_kolir5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"borec_kolir5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"borec_kolir5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"borec_kolir1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"borec_kolir2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"borec_kolir3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"borec_kolir4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"borec_kolir5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"vaterpolistka", "1F93D-200D-2640-FE0F"},
    {"vaterpolistka_kolir1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"vaterpolistka_kolir2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"vaterpolistka_kolir3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"vaterpolistka_kolir4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"vaterpolistka_kolir5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"vaterpolist", "1F93D-200D-2642-FE0F"},
    {"vaterpolist_kolir1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"vaterpolist_kolir2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"vaterpolist_kolir3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"vaterpolist_kolir4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"vaterpolist_kolir5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"gandbolistka", "1F93E-200D-2640-FE0F"},
    {"gandbolistka_kolir1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"gandbolistka_kolir2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"gandbolistka_kolir3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"gandbolistka_kolir4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"gandbolistka_kolir5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"gandbolist", "1F93E-200D-2642-FE0F"},
    {"gandbolist_kolir1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"gandbolist_kolir2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"gandbolist_kolir3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"gandbolist_kolir4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"gandbolist_kolir5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"zhinka_supergeroy", "1F9B8-200D-2640-FE0F"},
    {"zhinka_supergeroy_kolir1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"zhinka_supergeroy_kolir2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"zhinka_supergeroy_kolir3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"zhinka_supergeroy_kolir4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"zhinka_supergeroy_kolir5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"cholovik_supergeroy", "1F9B8-200D-2642-FE0F"},
    {"cholovik_supergeroy_kolir1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"cholovik_supergeroy_kolir2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"cholovik_supergeroy_kolir3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"cholovik_supergeroy_kolir4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"cholovik_supergeroy_kolir5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"superlihodiyka", "1F9B9-200D-2640-FE0F"},
    {"superlihodiyka_kolir1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"superlihodiyka_kolir2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"superlihodiyka_kolir3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"superlihodiyka_kolir4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"superlihodiyka_kolir5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"superlihodiy", "1F9B9-200D-2642-FE0F"},
    {"superlihodiy_kolir1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"superlihodiy_kolir2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"superlihodiy_kolir3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"superlihodiy_kolir4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"superlihodiy_kolir5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"zhinka_shcho_stoyit", "1F9CD-200D-2640-FE0F"},
    {"zhinka_shcho_stoyit_kolir1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"zhinka_shcho_stoyit_kolir2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"zhinka_shcho_stoyit_kolir3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"zhinka_shcho_stoyit_kolir4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"zhinka_shcho_stoyit_kolir5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"cholovik_shcho_stoyit", "1F9CD-200D-2642-FE0F"},
    {"cholovik_shcho_stoyit_kolir1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"cholovik_shcho_stoyit_kolir2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"cholovik_shcho_stoyit_kolir3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"cholovik_shcho_stoyit_kolir4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"cholovik_shcho_stoyit_kolir5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"zhinka_navkolishkah", "1F9CE-200D-2640-FE0F"},
    {"zhinka_navkolishkah_kolir1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"zhinka_navkolishkah_kolir2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"zhinka_navkolishkah_kolir3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"zhinka_navkolishkah_kolir4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"zhinka_navkolishkah_kolir5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"zhinka_navkolishkah_praviy_profil", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_navkolishkah_praviy_profil_kolir1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_navkolishkah_praviy_profil_kolir2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_navkolishkah_praviy_profil_kolir3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_navkolishkah_praviy_profil_kolir4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhinka_navkolishkah_praviy_profil_kolir5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"cholovik_navkolishkah", "1F9CE-200D-2642-FE0F"},
    {"cholovik_navkolishkah_kolir1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"cholovik_navkolishkah_kolir2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"cholovik_navkolishkah_kolir3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"cholovik_navkolishkah_kolir4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"cholovik_navkolishkah_kolir5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"cholovik_navkolishkah_praviy_profil", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_navkolishkah_praviy_profil_kolir1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_navkolishkah_praviy_profil_kolir2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_navkolishkah_praviy_profil_kolir3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_navkolishkah_praviy_profil_kolir4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"cholovik_navkolishkah_praviy_profil_kolir5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"zhinka_z_porushennyam_sluhu", "1F9CF-200D-2640-FE0F"},
    {"zhinka_z_porushennyam_sluhu_kolir1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"zhinka_z_porushennyam_sluhu_kolir2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"zhinka_z_porushennyam_sluhu_kolir3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"zhinka_z_porushennyam_sluhu_kolir4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"zhinka_z_porushennyam_sluhu_kolir5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"cholovik_iz_porushennyam_sluhu", "1F9CF-200D-2642-FE0F"},
    {"cholovik_iz_porushennyam_sluhu_kolir1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"cholovik_iz_porushennyam_sluhu_kolir2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"cholovik_iz_porushennyam_sluhu_kolir3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"cholovik_iz_porushennyam_sluhu_kolir4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"cholovik_iz_porushennyam_sluhu_kolir5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"zhinka_z_borodoyu", "1F9D4-200D-2640-FE0F"},
    {"zhinka_z_borodoyu_kolir1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"zhinka_z_borodoyu_kolir2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"zhinka_z_borodoyu_kolir3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"zhinka_z_borodoyu_kolir4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"zhinka_z_borodoyu_kolir5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"cholovik_iz_borodoyu", "1F9D4-200D-2642-FE0F"},
    {"cholovik_iz_borodoyu_kolir1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"cholovik_iz_borodoyu_kolir2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"cholovik_iz_borodoyu_kolir3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"cholovik_iz_borodoyu_kolir4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"cholovik_iz_borodoyu_kolir5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"zhinka_v_parkomu_primishchenni", "1F9D6-200D-2640-FE0F"},
    {"zhinka_v_parkomu_primishchenni_kolir1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"zhinka_v_parkomu_primishchenni_kolir2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"zhinka_v_parkomu_primishchenni_kolir3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"zhinka_v_parkomu_primishchenni_kolir4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"zhinka_v_parkomu_primishchenni_kolir5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"cholovik_u_parkomu_primishchenni", "1F9D6-200D-2642-FE0F"},
    {"cholovik_u_parkomu_primishchenni_kolir1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"cholovik_u_parkomu_primishchenni_kolir2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"cholovik_u_parkomu_primishchenni_kolir3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"cholovik_u_parkomu_primishchenni_kolir4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"cholovik_u_parkomu_primishchenni_kolir5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"zhinka_skelelaz", "1F9D7-200D-2640-FE0F"},
    {"zhinka_skelelaz_kolir1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"zhinka_skelelaz_kolir2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"zhinka_skelelaz_kolir3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"zhinka_skelelaz_kolir4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"zhinka_skelelaz_kolir5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"cholovik_skelelaz", "1F9D7-200D-2642-FE0F"},
    {"cholovik_skelelaz_kolir1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"cholovik_skelelaz_kolir2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"cholovik_skelelaz_kolir3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"cholovik_skelelaz_kolir4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"cholovik_skelelaz_kolir5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"zhinka_v_pozi_lotosa", "1F9D8-200D-2640-FE0F"},
    {"zhinka_v_pozi_lotosa_kolir1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"zhinka_v_pozi_lotosa_kolir2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"zhinka_v_pozi_lotosa_kolir3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"zhinka_v_pozi_lotosa_kolir4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"zhinka_v_pozi_lotosa_kolir5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"cholovik_u_pozi_lotosa", "1F9D8-200D-2642-FE0F"},
    {"cholovik_u_pozi_lotosa_kolir1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"cholovik_u_pozi_lotosa_kolir2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"cholovik_u_pozi_lotosa_kolir3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"cholovik_u_pozi_lotosa_kolir4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"cholovik_u_pozi_lotosa_kolir5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"chaklunka", "1F9D9-200D-2640-FE0F"},
    {"chaklunka_kolir1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"chaklunka_kolir2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"chaklunka_kolir3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"chaklunka_kolir4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"chaklunka_kolir5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"chaklun", "1F9D9-200D-2642-FE0F"},
    {"chaklun_kolir1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"chaklun_kolir2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"chaklun_kolir3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"chaklun_kolir4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"chaklun_kolir5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"zhinka_feya", "1F9DA-200D-2640-FE0F"},
    {"zhinka_feya_kolir1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"zhinka_feya_kolir2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"zhinka_feya_kolir3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"zhinka_feya_kolir4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"zhinka_feya_kolir5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"cholovik_feya", "1F9DA-200D-2642-FE0F"},
    {"cholovik_feya_kolir1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"cholovik_feya_kolir2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"cholovik_feya_kolir3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"cholovik_feya_kolir4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"cholovik_feya_kolir5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"zhinka_vampir", "1F9DB-200D-2640-FE0F"},
    {"zhinka_vampir_kolir1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"zhinka_vampir_kolir2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"zhinka_vampir_kolir3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"zhinka_vampir_kolir4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"zhinka_vampir_kolir5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"cholovik_vampir", "1F9DB-200D-2642-FE0F"},
    {"cholovik_vampir_kolir1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"cholovik_vampir_kolir2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"cholovik_vampir_kolir3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"cholovik_vampir_kolir4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"cholovik_vampir_kolir5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"rusalka", "1F9DC-200D-2640-FE0F"},
    {"rusalka_kolir1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"rusalka_kolir2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"rusalka_kolir3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"rusalka_kolir4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"rusalka_kolir5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"cholovik_rusalka", "1F9DC-200D-2642-FE0F"},
    {"cholovik_rusalka_kolir1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"cholovik_rusalka_kolir2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"cholovik_rusalka_kolir3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"cholovik_rusalka_kolir4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"cholovik_rusalka_kolir5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"zhinka_elf", "1F9DD-200D-2640-FE0F"},
    {"zhinka_elf_kolir1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"zhinka_elf_kolir2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"zhinka_elf_kolir3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"zhinka_elf_kolir4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"zhinka_elf_kolir5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"cholovik_elf", "1F9DD-200D-2642-FE0F"},
    {"cholovik_elf_kolir1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"cholovik_elf_kolir2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"cholovik_elf_kolir3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"cholovik_elf_kolir4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"cholovik_elf_kolir5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"zhinka_dzhin", "1F9DE-200D-2640-FE0F"},
    {"cholovik_dzhin", "1F9DE-200D-2642-FE0F"},
    {"zhinka_zombi", "1F9DF-200D-2640-FE0F"},
    {"cholovik_zombi", "1F9DF-200D-2642-FE0F"},
    {"cholovik_rude_volossya", "1F468-200D-1F9B0"},
    {"cholovik_rude_volossya_kolir1", "1F468-1F3FB-200D-1F9B0"},
    {"cholovik_rude_volossya_kolir2", "1F468-1F3FC-200D-1F9B0"},
    {"cholovik_rude_volossya_kolir3", "1F468-1F3FD-200D-1F9B0"},
    {"cholovik_rude_volossya_kolir4", "1F468-1F3FE-200D-1F9B0"},
    {"cholovik_rude_volossya_kolir5", "1F468-1F3FF-200D-1F9B0"},
    {"cholovik_kucheryave_volossya", "1F468-200D-1F9B1"},
    {"cholovik_kucheryave_volossya_kolir1", "1F468-1F3FB-200D-1F9B1"},
    {"cholovik_kucheryave_volossya_kolir2", "1F468-1F3FC-200D-1F9B1"},
    {"cholovik_kucheryave_volossya_kolir3", "1F468-1F3FD-200D-1F9B1"},
    {"cholovik_kucheryave_volossya_kolir4", "1F468-1F3FE-200D-1F9B1"},
    {"cholovik_kucheryave_volossya_kolir5", "1F468-1F3FF-200D-1F9B1"},
    {"cholovik_lisina", "1F468-200D-1F9B2"},
    {"cholovik_lisina_kolir1", "1F468-1F3FB-200D-1F9B2"},
    {"cholovik_lisina_kolir2", "1F468-1F3FC-200D-1F9B2"},
    {"cholovik_lisina_kolir3", "1F468-1F3FD-200D-1F9B2"},
    {"cholovik_lisina_kolir4", "1F468-1F3FE-200D-1F9B2"},
    {"cholovik_lisina_kolir5", "1F468-1F3FF-200D-1F9B2"},
    {"cholovik_sive_volossya", "1F468-200D-1F9B3"},
    {"cholovik_sive_volossya_kolir1", "1F468-1F3FB-200D-1F9B3"},
    {"cholovik_sive_volossya_kolir2", "1F468-1F3FC-200D-1F9B3"},
    {"cholovik_sive_volossya_kolir3", "1F468-1F3FD-200D-1F9B3"},
    {"cholovik_sive_volossya_kolir4", "1F468-1F3FE-200D-1F9B3"},
    {"cholovik_sive_volossya_kolir5", "1F468-1F3FF-200D-1F9B3"},
    {"zhinka_rude_volossya", "1F469-200D-1F9B0"},
    {"zhinka_rude_volossya_kolir1", "1F469-1F3FB-200D-1F9B0"},
    {"zhinka_rude_volossya_kolir2", "1F469-1F3FC-200D-1F9B0"},
    {"zhinka_rude_volossya_kolir3", "1F469-1F3FD-200D-1F9B0"},
    {"zhinka_rude_volossya_kolir4", "1F469-1F3FE-200D-1F9B0"},
    {"zhinka_rude_volossya_kolir5", "1F469-1F3FF-200D-1F9B0"},
    {"zhinka_kucheryave_volossya", "1F469-200D-1F9B1"},
    {"zhinka_kucheryave_volossya_kolir1", "1F469-1F3FB-200D-1F9B1"},
    {"zhinka_kucheryave_volossya_kolir2", "1F469-1F3FC-200D-1F9B1"},
    {"zhinka_kucheryave_volossya_kolir3", "1F469-1F3FD-200D-1F9B1"},
    {"zhinka_kucheryave_volossya_kolir4", "1F469-1F3FE-200D-1F9B1"},
    {"zhinka_kucheryave_volossya_kolir5", "1F469-1F3FF-200D-1F9B1"},
    {"zhinka_lisina", "1F469-200D-1F9B2"},
    {"zhinka_lisina_kolir1", "1F469-1F3FB-200D-1F9B2"},
    {"zhinka_lisina_kolir2", "1F469-1F3FC-200D-1F9B2"},
    {"zhinka_lisina_kolir3", "1F469-1F3FD-200D-1F9B2"},
    {"zhinka_lisina_kolir4", "1F469-1F3FE-200D-1F9B2"},
    {"zhinka_lisina_kolir5", "1F469-1F3FF-200D-1F9B2"},
    {"zhinka_sive_volossya", "1F469-200D-1F9B3"},
    {"zhinka_sive_volossya_kolir1", "1F469-1F3FB-200D-1F9B3"},
    {"zhinka_sive_volossya_kolir2", "1F469-1F3FC-200D-1F9B3"},
    {"zhinka_sive_volossya_kolir3", "1F469-1F3FD-200D-1F9B3"},
    {"zhinka_sive_volossya_kolir4", "1F469-1F3FE-200D-1F9B3"},
    {"zhinka_sive_volossya_kolir5", "1F469-1F3FF-200D-1F9B3"},
    {"lyudina_rude_volossya", "1F9D1-200D-1F9B0"},
    {"lyudina_rude_volossya_kolir1", "1F9D1-1F3FB-200D-1F9B0"},
    {"lyudina_rude_volossya_kolir2", "1F9D1-1F3FC-200D-1F9B0"},
    {"lyudina_rude_volossya_kolir3", "1F9D1-1F3FD-200D-1F9B0"},
    {"lyudina_rude_volossya_kolir4", "1F9D1-1F3FE-200D-1F9B0"},
    {"lyudina_rude_volossya_kolir5", "1F9D1-1F3FF-200D-1F9B0"},
    {"lyudina_kucheryave_volossya", "1F9D1-200D-1F9B1"},
    {"lyudina_kucheryave_volossya_kolir1", "1F9D1-1F3FB-200D-1F9B1"},
    {"lyudina_kucheryave_volossya_kolir2", "1F9D1-1F3FC-200D-1F9B1"},
    {"lyudina_kucheryave_volossya_kolir3", "1F9D1-1F3FD-200D-1F9B1"},
    {"lyudina_kucheryave_volossya_kolir4", "1F9D1-1F3FE-200D-1F9B1"},
    {"lyudina_kucheryave_volossya_kolir5", "1F9D1-1F3FF-200D-1F9B1"},
    {"lyudina_lisina", "1F9D1-200D-1F9B2"},
    {"lyudina_lisina_kolir1", "1F9D1-1F3FB-200D-1F9B2"},
    {"lyudina_lisina_kolir2", "1F9D1-1F3FC-200D-1F9B2"},
    {"lyudina_lisina_kolir3", "1F9D1-1F3FD-200D-1F9B2"},
    {"lyudina_lisina_kolir4", "1F9D1-1F3FE-200D-1F9B2"},
    {"lyudina_lisina_kolir5", "1F9D1-1F3FF-200D-1F9B2"},
    {"lyudina_sive_volossya", "1F9D1-200D-1F9B3"},
    {"lyudina_sive_volossya_kolir1", "1F9D1-1F3FB-200D-1F9B3"},
    {"lyudina_sive_volossya_kolir2", "1F9D1-1F3FC-200D-1F9B3"},
    {"lyudina_sive_volossya_kolir3", "1F9D1-1F3FD-200D-1F9B3"},
    {"lyudina_sive_volossya_kolir4", "1F9D1-1F3FE-200D-1F9B3"},
    {"lyudina_sive_volossya_kolir5", "1F9D1-1F3FF-200D-1F9B3"},
    {"rozirvaniy_lancyug", "26D3-FE0F-200D-1F4A5"},
    {"palke_serce", "2764-FE0F-200D-1F525"},
    {"serce_shcho_zazhivaie", "2764-FE0F-200D-1FA79"},
    {"korichneviy_grib", "1F344-200D-1F7EB"},
    {"laym", "1F34B-200D-1F7E9"},
    {"prapor_transgenderiv", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"veselkoviy_prapor", "1F3F3-FE0F-200D-1F308"},
    {"piratskiy_prapor", "1F3F4-200D-2620-FE0F"},
    {"chorniy_kit", "1F408-200D-2B1B"},
    {"sluzhboviy_sobaka", "1F415-200D-1F9BA"},
    {"chorniy_ptah", "1F426-200D-2B1B"},
    {"feniks", "1F426-200D-1F525"},
    {"polyarniy_vedmid", "1F43B-200D-2744-FE0F"},
    {"oko_v_tekstoviy_bulci", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"oblichchya_shcho_vidihaie", "1F62E-200D-1F4A8"},
    {"ochi_spirali", "1F635-200D-1F4AB"},
    {"oblichchya_v_hmarah", "1F636-200D-1F32B-FE0F"},
    {"zaperechlive_hitannya_golovoyu", "1F642-200D-2194-FE0F"},
    {"kivannya_golovoyu", "1F642-200D-2195-FE0F"},
    {"tancivnik_baletu", "1F9D1-200D-1FA70"},
    {"tancivnik_baletu_kolir1", "1F9D1-1F3FB-200D-1FA70"},
    {"tancivnik_baletu_kolir2", "1F9D1-1F3FC-200D-1FA70"},
    {"tancivnik_baletu_kolir3", "1F9D1-1F3FD-200D-1FA70"},
    {"tancivnik_baletu_kolir4", "1F9D1-1F3FE-200D-1FA70"},
    {"tancivnik_baletu_kolir5", "1F9D1-1F3FF-200D-1FA70"},
    {"знак_оклику_і_питання", "2049"},
    {"торговельна_марка", "2122"},
    {"джерело_інформації", "2139"},
    {"стрілка_вліво_вправо", "2194"},
    {"стрілка_вгору_вниз", "2195"},
    {"стрілка_вгору_вліво", "2196"},
    {"стрілка_вгору_вправо", "2197"},
    {"стрілка_вниз_управо", "2198"},
    {"стрілка_вниз_уліво", "2199"},
    {"клавіатура", "2328"},
    {"сонце", "2600"},
    {"хмара", "2601"},
    {"парасолька", "2602"},
    {"сніговик", "2603"},
    {"комета", "2604"},
    {"клітинка_з_галочкою", "2611"},
    {"парасолька_з_краплями_дощу", "2614"},
    {"гарячий_напій", "2615"},
    {"трилисник", "2618"},
    {"череп_і_кістки", "2620"},
    {"радіаційна_небезпека", "2622"},
    {"біологічна_загроза", "2623"},
    {"православний_хрест", "2626"},
    {"колесо_дхарми", "2638"},
    {"похмуре_обличчя", "2639"},
    {"символ_жінки", "2640"},
    {"символ_чоловіка", "2642"},
    {"овен", "2648"},
    {"телець", "2649"},
    {"стрілець", "2650"},
    {"козоріг", "2651"},
    {"водолій", "2652"},
    {"риби", "2653"},
    {"піка", "2660"},
    {"трефа", "2663"},
    {"чирва", "2665"},
    {"бубна", "2666"},
    {"гарячі_джерела", "2668"},
    {"молоток_і_кирка", "2692"},
    {"якір", "2693"},
    {"схрещені_мечі", "2694"},
    {"символ_медицини", "2695"},
    {"ваги", "2696"},
    {"перегінний_куб", "2697"},
    {"шестірня", "2699"},
    {"ножиці", "2702"},
    {"кнопка_з_галочкою", "2705"},
    {"літак", "2708"},
    {"конверт", "2709"},
    {"кінчик_чорнильного_пера", "2712"},
    {"галочка", "2714"},
    {"множення", "2716"},
    {"зірка_давида", "2721"},
    {"блискітки", "2728"},
    {"восьмикутний_астериск", "2733"},
    {"восьмикінцева_зірка", "2734"},
    {"сніжинка", "2744"},
    {"блискітка", "2747"},
    {"червоний_знак_питання", "2753"},
    {"білий_знак_питання", "2754"},
    {"білий_знак_оклику", "2755"},
    {"червоний_знак_оклику", "2757"},
    {"знак_оклику_у_формі_серця", "2763"},
    {"червоне_серце", "2764"},
    {"плюс", "2795"},
    {"мінус", "2796"},
    {"ділення", "2797"},
    {"стрілка_вправо_з_вигином_угору", "2934"},
    {"стрілка_вправо_з_вигином_униз", "2935"},
    {"хвилясте_тире", "3030"},
    {"кнопка_вітання_японською", "3297"},
    {"кнопка_секрет_японською", "3299"},
    {"авторське_право", "00A9"},
    {"зареєстрована_торговельна_марка", "00AE"},
    {"подвійний_знак_оклику", "203C"},
    {"стрілка_вправо_з_вигином_уліво", "21A9"},
    {"стрілка_вліво_з_вигином_управо", "21AA"},
    {"наручний_годинник", "231A"},
    {"пісковий_годинник", "231B"},
    {"кнопка_витягання", "23CF"},
    {"кнопка_перемотування_вперед", "23E9"},
    {"кнопка_перемотування_назад", "23EA"},
    {"кнопка_мотання_вгору", "23EB"},
    {"кнопка_мотання_вниз", "23EC"},
    {"кнопка_наступної_композиції", "23ED"},
    {"кнопка_повернення_до_попередньої_композиції", "23EE"},
    {"кнопка_відтворити_або_призупинити", "23EF"},
    {"будильник", "23F0"},
    {"секундомір", "23F1"},
    {"таймер", "23F2"},
    {"пісковий_годинник_із_піском_який_сиплеться", "23F3"},
    {"кнопка_паузи", "23F8"},
    {"кнопка_стоп", "23F9"},
    {"кнопка_запису", "23FA"},
    {"коло_з_літерою_м", "24C2"},
    {"малий_чорний_квадрат", "25AA"},
    {"малий_білий_квадрат", "25AB"},
    {"кнопка_відтворення", "25B6"},
    {"кнопка_реверсу", "25C0"},
    {"середній_білий_квадрат", "25FB"},
    {"середній_чорний_квадрат", "25FC"},
    {"менший_за_середній_білий_квадрат", "25FD"},
    {"менший_за_середній_чорний_квадрат", "25FE"},
    {"телефон", "260E"},
    {"вказівний_палець_що_вказує_вгору", "261D"},
    {"вказівний_палець_що_вказує_вгору_колір1", "261D-1F3FB"},
    {"вказівний_палець_що_вказує_вгору_колір2", "261D-1F3FC"},
    {"вказівний_палець_що_вказує_вгору_колір3", "261D-1F3FD"},
    {"вказівний_палець_що_вказує_вгору_колір4", "261D-1F3FE"},
    {"вказівний_палець_що_вказує_вгору_колір5", "261D-1F3FF"},
    {"зірка_і_півмісяць", "262A"},
    {"символ_миру", "262E"},
    {"інь_ян", "262F"},
    {"усміхнене_обличчя", "263A"},
    {"близнюки", "264A"},
    {"рак", "264B"},
    {"лев_знак_зодіаку", "264C"},
    {"діва", "264D"},
    {"терези", "264E"},
    {"скорпіон_знак_зодіаку", "264F"},
    {"пішак", "265F"},
    {"символ_утилізації", "267B"},
    {"нескінченність", "267E"},
    {"символ_крісла_візка", "267F"},
    {"символ_атома", "269B"},
    {"геральдична_лілія", "269C"},
    {"попередження", "26A0"},
    {"висока_напруга", "26A1"},
    {"трансгендерний_символ", "26A7"},
    {"білий_круг", "26AA"},
    {"чорний_круг", "26AB"},
    {"труна", "26B0"},
    {"урна_для_тліну", "26B1"},
    {"футбольний_мяч", "26BD"},
    {"бейсбольний_мяч", "26BE"},
    {"сніговик_без_снігу", "26C4"},
    {"сонце_за_хмарою", "26C5"},
    {"хмара_з_блискавкою_та_дощем", "26C8"},
    {"змієносець", "26CE"},
    {"кирка", "26CF"},
    {"каска_рятувальника", "26D1"},
    {"ланцюги", "26D3"},
    {"вїзд_заборонено", "26D4"},
    {"синтоїстський_храм", "26E9"},
    {"церква", "26EA"},
    {"гора", "26F0"},
    {"пляжна_парасолька", "26F1"},
    {"фонтан", "26F2"},
    {"лунка_для_гольфу_з_прапорцем", "26F3"},
    {"пором", "26F4"},
    {"вітрильник", "26F5"},
    {"лижник", "26F7"},
    {"ковзан", "26F8"},
    {"людина_що_веде_мяч", "26F9"},
    {"людина_що_веде_мяч_колір1", "26F9-1F3FB"},
    {"людина_що_веде_мяч_колір2", "26F9-1F3FC"},
    {"людина_що_веде_мяч_колір3", "26F9-1F3FD"},
    {"людина_що_веде_мяч_колір4", "26F9-1F3FE"},
    {"людина_що_веде_мяч_колір5", "26F9-1F3FF"},
    {"намет", "26FA"},
    {"бензоколонка", "26FD"},
    {"піднятий_кулак", "270A"},
    {"піднятий_кулак_колір1", "270A-1F3FB"},
    {"піднятий_кулак_колір2", "270A-1F3FC"},
    {"піднятий_кулак_колір3", "270A-1F3FD"},
    {"піднятий_кулак_колір4", "270A-1F3FE"},
    {"піднятий_кулак_колір5", "270A-1F3FF"},
    {"піднята_рука", "270B"},
    {"піднята_рука_колір1", "270B-1F3FB"},
    {"піднята_рука_колір2", "270B-1F3FC"},
    {"піднята_рука_колір3", "270B-1F3FD"},
    {"піднята_рука_колір4", "270B-1F3FE"},
    {"піднята_рука_колір5", "270B-1F3FF"},
    {"жест_перемоги", "270C"},
    {"жест_перемоги_колір1", "270C-1F3FB"},
    {"жест_перемоги_колір2", "270C-1F3FC"},
    {"жест_перемоги_колір3", "270C-1F3FD"},
    {"жест_перемоги_колір4", "270C-1F3FE"},
    {"жест_перемоги_колір5", "270C-1F3FF"},
    {"рука_що_пише", "270D"},
    {"рука_що_пише_колір1", "270D-1F3FB"},
    {"рука_що_пише_колір2", "270D-1F3FC"},
    {"рука_що_пише_колір3", "270D-1F3FD"},
    {"рука_що_пише_колір4", "270D-1F3FE"},
    {"рука_що_пише_колір5", "270D-1F3FF"},
    {"олівець", "270F"},
    {"латинський_хрест", "271D"},
    {"хрестик", "274C"},
    {"кнопка_з_хрестиком", "274E"},
    {"стрілка_вправо", "27A1"},
    {"закручена_петля", "27B0"},
    {"подвійна_закручена_петля", "27BF"},
    {"стрілка_вліво", "2B05"},
    {"стрілка_вгору", "2B06"},
    {"стрілка_вниз", "2B07"},
    {"великий_чорний_квадрат", "2B1B"},
    {"великий_білий_квадрат", "2B1C"},
    {"зірка", "2B50"},
    {"порожнисте_червоне_коло", "2B55"},
    {"японський_знак_початку_частини_пісні", "303D"},
    {"фішка_маджонгу_червоний_дракон", "1F004"},
    {"джокер", "1F0CF"},
    {"кнопка_a_iі_група_крові", "1F170"},
    {"кнопка_b_iii_група_крові", "1F171"},
    {"кнопка_o_і_група_крові", "1F17E"},
    {"кнопка_паркування", "1F17F"},
    {"кнопка_ab_iv_група_крові", "1F18E"},
    {"кнопка_очистити", "1F191"},
    {"кнопка_cool", "1F192"},
    {"кнопка_free", "1F193"},
    {"кнопка_id", "1F194"},
    {"кнопка_new", "1F195"},
    {"кнопка_ng", "1F196"},
    {"кнопка_ок", "1F197"},
    {"кнопка_sos", "1F198"},
    {"кнопка_up", "1F199"},
    {"кнопка_vs", "1F19A"},
    {"регіональний_показник_a", "1F1E6"},
    {"регіональний_показник_b", "1F1E7"},
    {"регіональний_показник_c", "1F1E8"},
    {"регіональний_показник_d", "1F1E9"},
    {"регіональний_показник_e", "1F1EA"},
    {"регіональний_показник_f", "1F1EB"},
    {"регіональний_показник_g", "1F1EC"},
    {"регіональний_показник_h", "1F1ED"},
    {"регіональний_показник_i", "1F1EE"},
    {"регіональний_показник_j", "1F1EF"},
    {"регіональний_показник_k", "1F1F0"},
    {"регіональний_показник_l", "1F1F1"},
    {"регіональний_показник_m", "1F1F2"},
    {"регіональний_показник_n", "1F1F3"},
    {"регіональний_показник_o", "1F1F4"},
    {"регіональний_показник_p", "1F1F5"},
    {"регіональний_показник_q", "1F1F6"},
    {"регіональний_показник_r", "1F1F7"},
    {"регіональний_показник_s", "1F1F8"},
    {"регіональний_показник_t", "1F1F9"},
    {"регіональний_показник_u", "1F1FA"},
    {"регіональний_показник_v", "1F1FB"},
    {"регіональний_показник_w", "1F1FC"},
    {"регіональний_показник_x", "1F1FD"},
    {"регіональний_показник_y", "1F1FE"},
    {"регіональний_показник_z", "1F1FF"},
    {"кнопка_тут_японською", "1F201"},
    {"кнопка_сервісний_збір_японською", "1F202"},
    {"кнопка_безкоштовно_японською", "1F21A"},
    {"кнопка_зарезервовано_японською", "1F22F"},
    {"кнопка_заборонено_японською", "1F232"},
    {"кнопка_вільно_японською", "1F233"},
    {"кнопка_достатня_оцінка_японською", "1F234"},
    {"кнопка_місць_немає_японською", "1F235"},
    {"кнопка_не_безкоштовно_японською", "1F236"},
    {"кнопка_щомісяця_японською", "1F237"},
    {"кнопка_заявка_японською", "1F238"},
    {"кнопка_знижка_японською", "1F239"},
    {"кнопка_робота_японською", "1F23A"},
    {"кнопка_угода_японською", "1F250"},
    {"кнопка_прийнятно_японською", "1F251"},
    {"циклон", "1F300"},
    {"туманно", "1F301"},
    {"складена_парасолька", "1F302"},
    {"зоряна_ніч", "1F303"},
    {"схід_сонця_над_горами", "1F304"},
    {"схід_сонця", "1F305"},
    {"місто_в_сутінках", "1F306"},
    {"захід_сонця", "1F307"},
    {"веселка", "1F308"},
    {"міст_уночі", "1F309"},
    {"хвиля", "1F30A"},
    {"вулкан", "1F30B"},
    {"чумацький_шлях", "1F30C"},
    {"глобус_що_показує_європу_й_африку", "1F30D"},
    {"глобус_що_показує_америки", "1F30E"},
    {"глобус_що_показує_азію_і_австралію", "1F30F"},
    {"земна_куля_з_меридіанами", "1F310"},
    {"молодий_місяць", "1F311"},
    {"місяць_у_першій_чверті", "1F312"},
    {"перша_чверть_місяця", "1F313"},
    {"місяць_у_другій_чверті", "1F314"},
    {"повний_місяць", "1F315"},
    {"місяць_у_третій_чверті", "1F316"},
    {"остання_чверть_місяця", "1F317"},
    {"місяць_у_четвертій_чверті", "1F318"},
    {"серп_місяця", "1F319"},
    {"молодий_місяць_з_обличчям", "1F31A"},
    {"перша_чверть_місяця_з_обличчям", "1F31B"},
    {"остання_чверть_місяця_з_обличчям", "1F31C"},
    {"повний_місяць_з_обличчям", "1F31D"},
    {"сонце_з_обличчям", "1F31E"},
    {"сяйна_зірка", "1F31F"},
    {"метеор", "1F320"},
    {"термометр", "1F321"},
    {"сонце_за_хмаркою", "1F324"},
    {"сонце_за_хмариною", "1F325"},
    {"сонце_за_дощовою_хмарою", "1F326"},
    {"хмара_з_дощем", "1F327"},
    {"хмара_зі_снігом", "1F328"},
    {"хмара_з_блискавкою", "1F329"},
    {"торнадо", "1F32A"},
    {"туман", "1F32B"},
    {"обличчя_що_дме", "1F32C"},
    {"хот_дог", "1F32D"},
    {"тако", "1F32E"},
    {"буріто", "1F32F"},
    {"каштан", "1F330"},
    {"сіянець", "1F331"},
    {"вічнозелене_дерево", "1F332"},
    {"листяне_дерево", "1F333"},
    {"пальма", "1F334"},
    {"кактус", "1F335"},
    {"гострий_перець", "1F336"},
    {"тюльпан", "1F337"},
    {"вишневий_цвіт", "1F338"},
    {"троянда", "1F339"},
    {"гібіскус", "1F33A"},
    {"соняшник", "1F33B"},
    {"жовта_квітка", "1F33C"},
    {"качан_кукурудзи", "1F33D"},
    {"сніпок_рису", "1F33E"},
    {"лікарська_рослина", "1F33F"},
    {"чотирилисткова_конюшина", "1F340"},
    {"кленовий_листок", "1F341"},
    {"опале_листя", "1F342"},
    {"листя_що_кружляє_за_вітром", "1F343"},
    {"гриб", "1F344"},
    {"томат", "1F345"},
    {"баклажан", "1F346"},
    {"виноград", "1F347"},
    {"диня", "1F348"},
    {"кавун", "1F349"},
    {"танжерин", "1F34A"},
    {"лимон", "1F34B"},
    {"банан", "1F34C"},
    {"ананас", "1F34D"},
    {"червоне_яблуко", "1F34E"},
    {"зелене_яблуко", "1F34F"},
    {"груша", "1F350"},
    {"персик", "1F351"},
    {"вишні", "1F352"},
    {"полуниця", "1F353"},
    {"гамбургер", "1F354"},
    {"піца", "1F355"},
    {"мясо_на_кістці", "1F356"},
    {"ніжка_свійської_птиці", "1F357"},
    {"рисовий_крекер", "1F358"},
    {"рисова_кулька", "1F359"},
    {"приготований_рис", "1F35A"},
    {"рис_із_карі", "1F35B"},
    {"миска_з_якої_піднімається_пара", "1F35C"},
    {"спагеті", "1F35D"},
    {"хліб", "1F35E"},
    {"картопля_фрі", "1F35F"},
    {"печений_батат", "1F360"},
    {"рисові_кульки_на_шпажці", "1F361"},
    {"оден", "1F362"},
    {"суші", "1F363"},
    {"смажена_креветка", "1F364"},
    {"рибний_рулет_зі_спіральним_візерунком", "1F365"},
    {"морозиво_в_ріжку", "1F366"},
    {"льодова_стружка", "1F367"},
    {"морозиво", "1F368"},
    {"пончик", "1F369"},
    {"печиво", "1F36A"},
    {"плитка_шоколаду", "1F36B"},
    {"цукерка", "1F36C"},
    {"льодяник", "1F36D"},
    {"заварний_крем", "1F36E"},
    {"горщик_із_медом", "1F36F"},
    {"тістечко", "1F370"},
    {"обід_у_коробці", "1F371"},
    {"горщик_з_їжею", "1F372"},
    {"готування_їжі", "1F373"},
    {"виделка_та_ніж", "1F374"},
    {"чайна_чашка_без_вушка", "1F375"},
    {"саке", "1F376"},
    {"келих_для_вина", "1F377"},
    {"бокал_для_коктейлю", "1F378"},
    {"тропічний_напій", "1F379"},
    {"кухоль_із_пивом", "1F37A"},
    {"кухлі_з_пивом_що_цокаються", "1F37B"},
    {"пляшечка_для_немовляти", "1F37C"},
    {"виделка_ніж_і_тарілка", "1F37D"},
    {"пляшка_з_корком_що_вилітає", "1F37E"},
    {"попкорн", "1F37F"},
    {"бант_зі_стрічки", "1F380"},
    {"подарунок", "1F381"},
    {"торт_на_день_народження", "1F382"},
    {"ліхтар_гарбуз", "1F383"},
    {"різдвяна_ялинка", "1F384"},
    {"санта_клаус", "1F385"},
    {"санта_клаус_колір1", "1F385-1F3FB"},
    {"санта_клаус_колір2", "1F385-1F3FC"},
    {"санта_клаус_колір3", "1F385-1F3FD"},
    {"санта_клаус_колір4", "1F385-1F3FE"},
    {"санта_клаус_колір5", "1F385-1F3FF"},
    {"феєрверки", "1F386"},
    {"бенгальський_вогонь", "1F387"},
    {"повітряна_кулька", "1F388"},
    {"хлопавка", "1F389"},
    {"кулька_з_конфеті", "1F38A"},
    {"танабата_гілка_тандзаку", "1F38B"},
    {"схрещені_прапори", "1F38C"},
    {"прикраси_на_ялинку", "1F38D"},
    {"японські_ляльки", "1F38E"},
    {"вітровказ_у_формі_коропів", "1F38F"},
    {"китайські_дзвіночки", "1F390"},
    {"церемонія_вшанування_місяця", "1F391"},
    {"шкільний_рюкзак", "1F392"},
    {"квадратна_академічна_шапочка", "1F393"},
    {"військова_медаль", "1F396"},
    {"стрічка_на_згадку", "1F397"},
    {"студійний_мікрофон", "1F399"},
    {"повзунок_рівня", "1F39A"},
    {"обертові_регулятори", "1F39B"},
    {"кадри_кіноплівки", "1F39E"},
    {"вхідні_квитки", "1F39F"},
    {"коник_на_каруселі", "1F3A0"},
    {"чортове_колесо", "1F3A1"},
    {"американські_гірки", "1F3A2"},
    {"рибацька_вудка", "1F3A3"},
    {"мікрофон", "1F3A4"},
    {"кінокамера", "1F3A5"},
    {"кінематограф", "1F3A6"},
    {"навушники", "1F3A7"},
    {"палітра_художника", "1F3A8"},
    {"циліндр_капелюх", "1F3A9"},
    {"циркове_шатро", "1F3AA"},
    {"квиток", "1F3AB"},
    {"кінохлопавка", "1F3AC"},
    {"театральне_мистецтво", "1F3AD"},
    {"ігровий_контролер", "1F3AE"},
    {"мішень_із_прямим_влученням", "1F3AF"},
    {"гральний_автомат", "1F3B0"},
    {"більярдна_куля_8", "1F3B1"},
    {"гральна_кість", "1F3B2"},
    {"куля_і_кеглі_для_боулінгу", "1F3B3"},
    {"гральні_карти_з_квітами_ханафуда", "1F3B4"},
    {"музична_нота", "1F3B5"},
    {"музичні_ноти", "1F3B6"},
    {"саксофон", "1F3B7"},
    {"гітара", "1F3B8"},
    {"музична_клавіатура", "1F3B9"},
    {"труба", "1F3BA"},
    {"скрипка", "1F3BB"},
    {"партитура", "1F3BC"},
    {"бігова_теніска", "1F3BD"},
    {"тенісний_мяч", "1F3BE"},
    {"лижі", "1F3BF"},
    {"баскетбольний_мяч", "1F3C0"},
    {"картатий_прапор", "1F3C1"},
    {"сноубордист", "1F3C2"},
    {"сноубордист_колір1", "1F3C2-1F3FB"},
    {"сноубордист_колір2", "1F3C2-1F3FC"},
    {"сноубордист_колір3", "1F3C2-1F3FD"},
    {"сноубордист_колір4", "1F3C2-1F3FE"},
    {"сноубордист_колір5", "1F3C2-1F3FF"},
    {"людина_що_біжить", "1F3C3"},
    {"людина_що_біжить_колір1", "1F3C3-1F3FB"},
    {"людина_що_біжить_колір2", "1F3C3-1F3FC"},
    {"людина_що_біжить_колір3", "1F3C3-1F3FD"},
    {"людина_що_біжить_колір4", "1F3C3-1F3FE"},
    {"людина_що_біжить_колір5", "1F3C3-1F3FF"},
    {"людина_що_займається_серфінгом", "1F3C4"},
    {"людина_що_займається_серфінгом_колір1", "1F3C4-1F3FB"},
    {"людина_що_займається_серфінгом_колір2", "1F3C4-1F3FC"},
    {"людина_що_займається_серфінгом_колір3", "1F3C4-1F3FD"},
    {"людина_що_займається_серфінгом_колір4", "1F3C4-1F3FE"},
    {"людина_що_займається_серфінгом_колір5", "1F3C4-1F3FF"},
    {"спортивна_медаль", "1F3C5"},
    {"приз", "1F3C6"},
    {"кінні_перегони", "1F3C7"},
    {"кінні_перегони_колір1", "1F3C7-1F3FB"},
    {"кінні_перегони_колір2", "1F3C7-1F3FC"},
    {"кінні_перегони_колір3", "1F3C7-1F3FD"},
    {"кінні_перегони_колір4", "1F3C7-1F3FE"},
    {"кінні_перегони_колір5", "1F3C7-1F3FF"},
    {"мяч_для_американського_футболу", "1F3C8"},
    {"мяч_для_регбі", "1F3C9"},
    {"людина_що_пливе", "1F3CA"},
    {"людина_що_пливе_колір1", "1F3CA-1F3FB"},
    {"людина_що_пливе_колір2", "1F3CA-1F3FC"},
    {"людина_що_пливе_колір3", "1F3CA-1F3FD"},
    {"людина_що_пливе_колір4", "1F3CA-1F3FE"},
    {"людина_що_пливе_колір5", "1F3CA-1F3FF"},
    {"важкоатлет", "1F3CB"},
    {"важкоатлет_колір1", "1F3CB-1F3FB"},
    {"важкоатлет_колір2", "1F3CB-1F3FC"},
    {"важкоатлет_колір3", "1F3CB-1F3FD"},
    {"важкоатлет_колір4", "1F3CB-1F3FE"},
    {"важкоатлет_колір5", "1F3CB-1F3FF"},
    {"людина_що_грає_в_гольф", "1F3CC"},
    {"людина_що_грає_в_гольф_колір1", "1F3CC-1F3FB"},
    {"людина_що_грає_в_гольф_колір2", "1F3CC-1F3FC"},
    {"людина_що_грає_в_гольф_колір3", "1F3CC-1F3FD"},
    {"людина_що_грає_в_гольф_колір4", "1F3CC-1F3FE"},
    {"людина_що_грає_в_гольф_колір5", "1F3CC-1F3FF"},
    {"мотоцикл", "1F3CD"},
    {"cпортивний_автомобіль", "1F3CE"},
    {"бита_і_мяч_для_крикету", "1F3CF"},
    {"волейбольний_мяч", "1F3D0"},
    {"ключка_і_мяч_для_хокею_на_траві", "1F3D1"},
    {"ключка_і_шайба_для_хокею", "1F3D2"},
    {"ракетка_і_кулька_для_пінг_понгу", "1F3D3"},
    {"гора_із_засніженою_верхівкою", "1F3D4"},
    {"кемпінг", "1F3D5"},
    {"пляж_і_парасолька", "1F3D6"},
    {"спорудження_будівлі", "1F3D7"},
    {"група_будинків", "1F3D8"},
    {"міський_краєвид", "1F3D9"},
    {"занедбаний_будинок", "1F3DA"},
    {"антична_будівля", "1F3DB"},
    {"пустеля", "1F3DC"},
    {"безлюдний_острів", "1F3DD"},
    {"національний_парк", "1F3DE"},
    {"стадіон", "1F3DF"},
    {"будинок", "1F3E0"},
    {"будинок_із_садом", "1F3E1"},
    {"офісна_будівля", "1F3E2"},
    {"японське_поштове_відділення", "1F3E3"},
    {"поштове_відділення", "1F3E4"},
    {"лікарня", "1F3E5"},
    {"банк", "1F3E6"},
    {"символ_банкомата", "1F3E7"},
    {"готель", "1F3E8"},
    {"готель_кохання", "1F3E9"},
    {"цілодобовий_магазин", "1F3EA"},
    {"школа", "1F3EB"},
    {"універмаг", "1F3EC"},
    {"фабрика", "1F3ED"},
    {"червоний_паперовий_ліхтар", "1F3EE"},
    {"японський_замок", "1F3EF"},
    {"замок", "1F3F0"},
    {"білий_прапор", "1F3F3"},
    {"чорний_прапор", "1F3F4"},
    {"розета", "1F3F5"},
    {"бирка", "1F3F7"},
    {"ракетка_і_волан_для_бадмінтону", "1F3F8"},
    {"лук_і_стріла", "1F3F9"},
    {"амфора", "1F3FA"},
    {"світлий_тон_шкіри", "1F3FB"},
    {"помірно_світлий_тон_шкіри", "1F3FC"},
    {"помірний_тон_шкіри", "1F3FD"},
    {"помірно_темний_тон_шкіри", "1F3FE"},
    {"темний_тон_шкіри", "1F3FF"},
    {"щур", "1F400"},
    {"миша", "1F401"},
    {"бик", "1F402"},
    {"буйвіл", "1F403"},
    {"корова", "1F404"},
    {"тигр", "1F405"},
    {"леопард", "1F406"},
    {"кролик", "1F407"},
    {"кіт", "1F408"},
    {"дракон", "1F409"},
    {"крокодил", "1F40A"},
    {"кит", "1F40B"},
    {"равлик", "1F40C"},
    {"змія", "1F40D"},
    {"кінь", "1F40E"},
    {"баран", "1F40F"},
    {"цап", "1F410"},
    {"вівця", "1F411"},
    {"мавпа", "1F412"},
    {"півень", "1F413"},
    {"курка", "1F414"},
    {"собака", "1F415"},
    {"свиня", "1F416"},
    {"вепр", "1F417"},
    {"слон", "1F418"},
    {"восьминіг", "1F419"},
    {"морська_мушля", "1F41A"},
    {"комаха", "1F41B"},
    {"мураха", "1F41C"},
    {"бджола", "1F41D"},
    {"сонечко", "1F41E"},
    {"риба", "1F41F"},
    {"тропічна_риба", "1F420"},
    {"риба_куля", "1F421"},
    {"черепаха", "1F422"},
    {"курча_що_вилуплюється", "1F423"},
    {"мале_курча", "1F424"},
    {"курча_що_стоїть", "1F425"},
    {"птах", "1F426"},
    {"пінгвін", "1F427"},
    {"коала", "1F428"},
    {"пудель", "1F429"},
    {"одногорбий_верблюд", "1F42A"},
    {"двогорбий_верблюд", "1F42B"},
    {"дельфін", "1F42C"},
    {"морда_миші", "1F42D"},
    {"морда_корови", "1F42E"},
    {"морда_тигра", "1F42F"},
    {"морда_кроля", "1F430"},
    {"морда_кота", "1F431"},
    {"голова_дракона", "1F432"},
    {"кит_що_пускає_фонтан", "1F433"},
    {"голова_коня", "1F434"},
    {"морда_мавпи", "1F435"},
    {"морда_собаки", "1F436"},
    {"рило_свині", "1F437"},
    {"жаба", "1F438"},
    {"хомяк", "1F439"},
    {"вовк", "1F43A"},
    {"ведмідь", "1F43B"},
    {"панда", "1F43C"},
    {"свинячий_пятак", "1F43D"},
    {"сліди_лап", "1F43E"},
    {"бурундук", "1F43F"},
    {"очі", "1F440"},
    {"око", "1F441"},
    {"вухо", "1F442"},
    {"вухо_колір1", "1F442-1F3FB"},
    {"вухо_колір2", "1F442-1F3FC"},
    {"вухо_колір3", "1F442-1F3FD"},
    {"вухо_колір4", "1F442-1F3FE"},
    {"вухо_колір5", "1F442-1F3FF"},
    {"ніс", "1F443"},
    {"ніс_колір1", "1F443-1F3FB"},
    {"ніс_колір2", "1F443-1F3FC"},
    {"ніс_колір3", "1F443-1F3FD"},
    {"ніс_колір4", "1F443-1F3FE"},
    {"ніс_колір5", "1F443-1F3FF"},
    {"рот", "1F444"},
    {"язик", "1F445"},
    {"палець_тильним_боком_що_вказує_вгору", "1F446"},
    {"палець_тильним_боком_що_вказує_вгору_колір1", "1F446-1F3FB"},
    {"палець_тильним_боком_що_вказує_вгору_колір2", "1F446-1F3FC"},
    {"палець_тильним_боком_що_вказує_вгору_колір3", "1F446-1F3FD"},
    {"палець_тильним_боком_що_вказує_вгору_колір4", "1F446-1F3FE"},
    {"палець_тильним_боком_що_вказує_вгору_колір5", "1F446-1F3FF"},
    {"палець_тильним_боком_що_вказує_вниз", "1F447"},
    {"палець_тильним_боком_що_вказує_вниз_колір1", "1F447-1F3FB"},
    {"палець_тильним_боком_що_вказує_вниз_колір2", "1F447-1F3FC"},
    {"палець_тильним_боком_що_вказує_вниз_колір3", "1F447-1F3FD"},
    {"палець_тильним_боком_що_вказує_вниз_колір4", "1F447-1F3FE"},
    {"палець_тильним_боком_що_вказує_вниз_колір5", "1F447-1F3FF"},
    {"палець_тильним_боком_що_вказує_вліво", "1F448"},
    {"палець_тильним_боком_що_вказує_вліво_колір1", "1F448-1F3FB"},
    {"палець_тильним_боком_що_вказує_вліво_колір2", "1F448-1F3FC"},
    {"палець_тильним_боком_що_вказує_вліво_колір3", "1F448-1F3FD"},
    {"палець_тильним_боком_що_вказує_вліво_колір4", "1F448-1F3FE"},
    {"палець_тильним_боком_що_вказує_вліво_колір5", "1F448-1F3FF"},
    {"палець_тильним_боком_що_вказує_вправо", "1F449"},
    {"палець_тильним_боком_що_вказує_вправо_колір1", "1F449-1F3FB"},
    {"палець_тильним_боком_що_вказує_вправо_колір2", "1F449-1F3FC"},
    {"палець_тильним_боком_що_вказує_вправо_колір3", "1F449-1F3FD"},
    {"палець_тильним_боком_що_вказує_вправо_колір4", "1F449-1F3FE"},
    {"палець_тильним_боком_що_вказує_вправо_колір5", "1F449-1F3FF"},
    {"кулак_що_насувається", "1F44A"},
    {"кулак_що_насувається_колір1", "1F44A-1F3FB"},
    {"кулак_що_насувається_колір2", "1F44A-1F3FC"},
    {"кулак_що_насувається_колір3", "1F44A-1F3FD"},
    {"кулак_що_насувається_колір4", "1F44A-1F3FE"},
    {"кулак_що_насувається_колір5", "1F44A-1F3FF"},
    {"рука_махає", "1F44B"},
    {"рука_махає_колір1", "1F44B-1F3FB"},
    {"рука_махає_колір2", "1F44B-1F3FC"},
    {"рука_махає_колір3", "1F44B-1F3FD"},
    {"рука_махає_колір4", "1F44B-1F3FE"},
    {"рука_махає_колір5", "1F44B-1F3FF"},
    {"жест_окей", "1F44C"},
    {"жест_окей_колір1", "1F44C-1F3FB"},
    {"жест_окей_колір2", "1F44C-1F3FC"},
    {"жест_окей_колір3", "1F44C-1F3FD"},
    {"жест_окей_колір4", "1F44C-1F3FE"},
    {"жест_окей_колір5", "1F44C-1F3FF"},
    {"великі_пальці_вгору", "1F44D"},
    {"великі_пальці_вгору_колір1", "1F44D-1F3FB"},
    {"великі_пальці_вгору_колір2", "1F44D-1F3FC"},
    {"великі_пальці_вгору_колір3", "1F44D-1F3FD"},
    {"великі_пальці_вгору_колір4", "1F44D-1F3FE"},
    {"великі_пальці_вгору_колір5", "1F44D-1F3FF"},
    {"великі_пальці_вниз", "1F44E"},
    {"великі_пальці_вниз_колір1", "1F44E-1F3FB"},
    {"великі_пальці_вниз_колір2", "1F44E-1F3FC"},
    {"великі_пальці_вниз_колір3", "1F44E-1F3FD"},
    {"великі_пальці_вниз_колір4", "1F44E-1F3FE"},
    {"великі_пальці_вниз_колір5", "1F44E-1F3FF"},
    {"руки_що_аплодують", "1F44F"},
    {"руки_що_аплодують_колір1", "1F44F-1F3FB"},
    {"руки_що_аплодують_колір2", "1F44F-1F3FC"},
    {"руки_що_аплодують_колір3", "1F44F-1F3FD"},
    {"руки_що_аплодують_колір4", "1F44F-1F3FE"},
    {"руки_що_аплодують_колір5", "1F44F-1F3FF"},
    {"відкриті_долоні", "1F450"},
    {"відкриті_долоні_колір1", "1F450-1F3FB"},
    {"відкриті_долоні_колір2", "1F450-1F3FC"},
    {"відкриті_долоні_колір3", "1F450-1F3FD"},
    {"відкриті_долоні_колір4", "1F450-1F3FE"},
    {"відкриті_долоні_колір5", "1F450-1F3FF"},
    {"корона", "1F451"},
    {"жіночий_капелюшок", "1F452"},
    {"окуляри", "1F453"},
    {"краватка", "1F454"},
    {"теніска_поло", "1F455"},
    {"джинси", "1F456"},
    {"сукня", "1F457"},
    {"кімоно", "1F458"},
    {"бікіні", "1F459"},
    {"жіночий_одяг", "1F45A"},
    {"гаманець", "1F45B"},
    {"сумка", "1F45C"},
    {"клатч", "1F45D"},
    {"чоловіча_туфля", "1F45E"},
    {"кросівка", "1F45F"},
    {"туфля_на_високому_підборі", "1F460"},
    {"жіноча_сандалія_з_підбором", "1F461"},
    {"жіночий_чобіт", "1F462"},
    {"сліди_ніг", "1F463"},
    {"силует_бюсту_людини", "1F464"},
    {"силуети_двох_людських_бюстів", "1F465"},
    {"хлопчик", "1F466"},
    {"хлопчик_колір1", "1F466-1F3FB"},
    {"хлопчик_колір2", "1F466-1F3FC"},
    {"хлопчик_колір3", "1F466-1F3FD"},
    {"хлопчик_колір4", "1F466-1F3FE"},
    {"хлопчик_колір5", "1F466-1F3FF"},
    {"дівчинка", "1F467"},
    {"дівчинка_колір1", "1F467-1F3FB"},
    {"дівчинка_колір2", "1F467-1F3FC"},
    {"дівчинка_колір3", "1F467-1F3FD"},
    {"дівчинка_колір4", "1F467-1F3FE"},
    {"дівчинка_колір5", "1F467-1F3FF"},
    {"чоловік", "1F468"},
    {"чоловік_колір1", "1F468-1F3FB"},
    {"чоловік_колір2", "1F468-1F3FC"},
    {"чоловік_колір3", "1F468-1F3FD"},
    {"чоловік_колір4", "1F468-1F3FE"},
    {"чоловік_колір5", "1F468-1F3FF"},
    {"жінка", "1F469"},
    {"жінка_колір1", "1F469-1F3FB"},
    {"жінка_колір2", "1F469-1F3FC"},
    {"жінка_колір3", "1F469-1F3FD"},
    {"жінка_колір4", "1F469-1F3FE"},
    {"жінка_колір5", "1F469-1F3FF"},
    {"родина", "1F46A"},
    {"жінка_й_чоловік_що_тримаються_за_руки", "1F46B"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір1", "1F46B-1F3FB"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір2", "1F46B-1F3FC"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір3", "1F46B-1F3FD"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір4", "1F46B-1F3FE"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір5", "1F46B-1F3FF"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"жінка_й_чоловік_що_тримаються_за_руки_колір5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"чоловіки_що_тримаються_за_руки", "1F46C"},
    {"чоловіки_що_тримаються_за_руки_колір1", "1F46C-1F3FB"},
    {"чоловіки_що_тримаються_за_руки_колір2", "1F46C-1F3FC"},
    {"чоловіки_що_тримаються_за_руки_колір3", "1F46C-1F3FD"},
    {"чоловіки_що_тримаються_за_руки_колір4", "1F46C-1F3FE"},
    {"чоловіки_що_тримаються_за_руки_колір5", "1F46C-1F3FF"},
    {"чоловіки_що_тримаються_за_руки_колір1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"чоловіки_що_тримаються_за_руки_колір1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"чоловіки_що_тримаються_за_руки_колір1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"чоловіки_що_тримаються_за_руки_колір1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"чоловіки_що_тримаються_за_руки_колір2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"чоловіки_що_тримаються_за_руки_колір2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"чоловіки_що_тримаються_за_руки_колір2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"чоловіки_що_тримаються_за_руки_колір2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"чоловіки_що_тримаються_за_руки_колір3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"чоловіки_що_тримаються_за_руки_колір3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"чоловіки_що_тримаються_за_руки_колір3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"чоловіки_що_тримаються_за_руки_колір3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"чоловіки_що_тримаються_за_руки_колір4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"чоловіки_що_тримаються_за_руки_колір4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"чоловіки_що_тримаються_за_руки_колір4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"чоловіки_що_тримаються_за_руки_колір4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"чоловіки_що_тримаються_за_руки_колір5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"чоловіки_що_тримаються_за_руки_колір5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"чоловіки_що_тримаються_за_руки_колір5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"чоловіки_що_тримаються_за_руки_колір5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"жінки_що_тримаються_за_руки", "1F46D"},
    {"жінки_що_тримаються_за_руки_колір1", "1F46D-1F3FB"},
    {"жінки_що_тримаються_за_руки_колір2", "1F46D-1F3FC"},
    {"жінки_що_тримаються_за_руки_колір3", "1F46D-1F3FD"},
    {"жінки_що_тримаються_за_руки_колір4", "1F46D-1F3FE"},
    {"жінки_що_тримаються_за_руки_колір5", "1F46D-1F3FF"},
    {"жінки_що_тримаються_за_руки_колір1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"жінки_що_тримаються_за_руки_колір1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"жінки_що_тримаються_за_руки_колір1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"жінки_що_тримаються_за_руки_колір1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"жінки_що_тримаються_за_руки_колір2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"жінки_що_тримаються_за_руки_колір2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"жінки_що_тримаються_за_руки_колір2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"жінки_що_тримаються_за_руки_колір2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"жінки_що_тримаються_за_руки_колір3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"жінки_що_тримаються_за_руки_колір3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"жінки_що_тримаються_за_руки_колір3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"жінки_що_тримаються_за_руки_колір3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"жінки_що_тримаються_за_руки_колір4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"жінки_що_тримаються_за_руки_колір4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"жінки_що_тримаються_за_руки_колір4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"жінки_що_тримаються_за_руки_колір4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"жінки_що_тримаються_за_руки_колір5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"жінки_що_тримаються_за_руки_колір5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"жінки_що_тримаються_за_руки_колір5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"жінки_що_тримаються_за_руки_колір5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"працівник_поліції", "1F46E"},
    {"працівник_поліції_колір1", "1F46E-1F3FB"},
    {"працівник_поліції_колір2", "1F46E-1F3FC"},
    {"працівник_поліції_колір3", "1F46E-1F3FD"},
    {"працівник_поліції_колір4", "1F46E-1F3FE"},
    {"працівник_поліції_колір5", "1F46E-1F3FF"},
    {"люди_з_кролячими_вухами", "1F46F"},
    {"люди_з_кролячими_вухами_колір1", "1F46F-1F3FB"},
    {"люди_з_кролячими_вухами_колір2", "1F46F-1F3FC"},
    {"люди_з_кролячими_вухами_колір3", "1F46F-1F3FD"},
    {"люди_з_кролячими_вухами_колір4", "1F46F-1F3FE"},
    {"люди_з_кролячими_вухами_колір5", "1F46F-1F3FF"},
    {"люди_з_кролячими_вухами_колір1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"люди_з_кролячими_вухами_колір1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"люди_з_кролячими_вухами_колір1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"люди_з_кролячими_вухами_колір1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"люди_з_кролячими_вухами_колір2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"люди_з_кролячими_вухами_колір2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"люди_з_кролячими_вухами_колір2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"люди_з_кролячими_вухами_колір2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"люди_з_кролячими_вухами_колір3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"люди_з_кролячими_вухами_колір3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"люди_з_кролячими_вухами_колір3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"люди_з_кролячими_вухами_колір3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"люди_з_кролячими_вухами_колір4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"люди_з_кролячими_вухами_колір4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"люди_з_кролячими_вухами_колір4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"люди_з_кролячими_вухами_колір4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"люди_з_кролячими_вухами_колір5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"люди_з_кролячими_вухами_колір5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"люди_з_кролячими_вухами_колір5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"люди_з_кролячими_вухами_колір5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"людина_у_фаті", "1F470"},
    {"людина_у_фаті_колір1", "1F470-1F3FB"},
    {"людина_у_фаті_колір2", "1F470-1F3FC"},
    {"людина_у_фаті_колір3", "1F470-1F3FD"},
    {"людина_у_фаті_колір4", "1F470-1F3FE"},
    {"людина_у_фаті_колір5", "1F470-1F3FF"},
    {"людина_з_білявим_волоссям", "1F471"},
    {"людина_з_білявим_волоссям_колір1", "1F471-1F3FB"},
    {"людина_з_білявим_волоссям_колір2", "1F471-1F3FC"},
    {"людина_з_білявим_волоссям_колір3", "1F471-1F3FD"},
    {"людина_з_білявим_волоссям_колір4", "1F471-1F3FE"},
    {"людина_з_білявим_волоссям_колір5", "1F471-1F3FF"},
    {"чоловік_у_китайській_шапці", "1F472"},
    {"чоловік_у_китайській_шапці_колір1", "1F472-1F3FB"},
    {"чоловік_у_китайській_шапці_колір2", "1F472-1F3FC"},
    {"чоловік_у_китайській_шапці_колір3", "1F472-1F3FD"},
    {"чоловік_у_китайській_шапці_колір4", "1F472-1F3FE"},
    {"чоловік_у_китайській_шапці_колір5", "1F472-1F3FF"},
    {"людина_в_тюрбані", "1F473"},
    {"людина_в_тюрбані_колір1", "1F473-1F3FB"},
    {"людина_в_тюрбані_колір2", "1F473-1F3FC"},
    {"людина_в_тюрбані_колір3", "1F473-1F3FD"},
    {"людина_в_тюрбані_колір4", "1F473-1F3FE"},
    {"людина_в_тюрбані_колір5", "1F473-1F3FF"},
    {"літній_чоловік", "1F474"},
    {"літній_чоловік_колір1", "1F474-1F3FB"},
    {"літній_чоловік_колір2", "1F474-1F3FC"},
    {"літній_чоловік_колір3", "1F474-1F3FD"},
    {"літній_чоловік_колір4", "1F474-1F3FE"},
    {"літній_чоловік_колір5", "1F474-1F3FF"},
    {"літня_жінка", "1F475"},
    {"літня_жінка_колір1", "1F475-1F3FB"},
    {"літня_жінка_колір2", "1F475-1F3FC"},
    {"літня_жінка_колір3", "1F475-1F3FD"},
    {"літня_жінка_колір4", "1F475-1F3FE"},
    {"літня_жінка_колір5", "1F475-1F3FF"},
    {"немовля", "1F476"},
    {"немовля_колір1", "1F476-1F3FB"},
    {"немовля_колір2", "1F476-1F3FC"},
    {"немовля_колір3", "1F476-1F3FD"},
    {"немовля_колір4", "1F476-1F3FE"},
    {"немовля_колір5", "1F476-1F3FF"},
    {"будівельник", "1F477"},
    {"будівельник_колір1", "1F477-1F3FB"},
    {"будівельник_колір2", "1F477-1F3FC"},
    {"будівельник_колір3", "1F477-1F3FD"},
    {"будівельник_колір4", "1F477-1F3FE"},
    {"будівельник_колір5", "1F477-1F3FF"},
    {"принцеса", "1F478"},
    {"принцеса_колір1", "1F478-1F3FB"},
    {"принцеса_колір2", "1F478-1F3FC"},
    {"принцеса_колір3", "1F478-1F3FD"},
    {"принцеса_колір4", "1F478-1F3FE"},
    {"принцеса_колір5", "1F478-1F3FF"},
    {"велетень_людожер", "1F479"},
    {"гоблін", "1F47A"},
    {"привид", "1F47B"},
    {"янголятко", "1F47C"},
    {"янголятко_колір1", "1F47C-1F3FB"},
    {"янголятко_колір2", "1F47C-1F3FC"},
    {"янголятко_колір3", "1F47C-1F3FD"},
    {"янголятко_колір4", "1F47C-1F3FE"},
    {"янголятко_колір5", "1F47C-1F3FF"},
    {"прибулець", "1F47D"},
    {"іншопланетний_монстр", "1F47E"},
    {"сердите_обличчя_з_рогами", "1F47F"},
    {"череп", "1F480"},
    {"людина_що_простягає_руку_допомоги", "1F481"},
    {"людина_що_простягає_руку_допомоги_колір1", "1F481-1F3FB"},
    {"людина_що_простягає_руку_допомоги_колір2", "1F481-1F3FC"},
    {"людина_що_простягає_руку_допомоги_колір3", "1F481-1F3FD"},
    {"людина_що_простягає_руку_допомоги_колір4", "1F481-1F3FE"},
    {"людина_що_простягає_руку_допомоги_колір5", "1F481-1F3FF"},
    {"варта", "1F482"},
    {"варта_колір1", "1F482-1F3FB"},
    {"варта_колір2", "1F482-1F3FC"},
    {"варта_колір3", "1F482-1F3FD"},
    {"варта_колір4", "1F482-1F3FE"},
    {"варта_колір5", "1F482-1F3FF"},
    {"жінка_що_танцює", "1F483"},
    {"жінка_що_танцює_колір1", "1F483-1F3FB"},
    {"жінка_що_танцює_колір2", "1F483-1F3FC"},
    {"жінка_що_танцює_колір3", "1F483-1F3FD"},
    {"жінка_що_танцює_колір4", "1F483-1F3FE"},
    {"жінка_що_танцює_колір5", "1F483-1F3FF"},
    {"помада", "1F484"},
    {"лак_для_нігтів", "1F485"},
    {"лак_для_нігтів_колір1", "1F485-1F3FB"},
    {"лак_для_нігтів_колір2", "1F485-1F3FC"},
    {"лак_для_нігтів_колір3", "1F485-1F3FD"},
    {"лак_для_нігтів_колір4", "1F485-1F3FE"},
    {"лак_для_нігтів_колір5", "1F485-1F3FF"},
    {"людина_якій_роблять_масаж_голови", "1F486"},
    {"людина_якій_роблять_масаж_голови_колір1", "1F486-1F3FB"},
    {"людина_якій_роблять_масаж_голови_колір2", "1F486-1F3FC"},
    {"людина_якій_роблять_масаж_голови_колір3", "1F486-1F3FD"},
    {"людина_якій_роблять_масаж_голови_колір4", "1F486-1F3FE"},
    {"людина_якій_роблять_масаж_голови_колір5", "1F486-1F3FF"},
    {"людина_якій_роблять_стрижку", "1F487"},
    {"людина_якій_роблять_стрижку_колір1", "1F487-1F3FB"},
    {"людина_якій_роблять_стрижку_колір2", "1F487-1F3FC"},
    {"людина_якій_роблять_стрижку_колір3", "1F487-1F3FD"},
    {"людина_якій_роблять_стрижку_колір4", "1F487-1F3FE"},
    {"людина_якій_роблять_стрижку_колір5", "1F487-1F3FF"},
    {"вивіска_перукаря_смугастий_стовп", "1F488"},
    {"шприц", "1F489"},
    {"пігулка", "1F48A"},
    {"слід_цілунку", "1F48B"},
    {"любовний_лист", "1F48C"},
    {"каблучка", "1F48D"},
    {"коштовний_камінь", "1F48E"},
    {"чоловік_і_жінка_що_цілуються", "1F48F"},
    {"чоловік_і_жінка_що_цілуються_колір1", "1F48F-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_колір2", "1F48F-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_колір3", "1F48F-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_колір4", "1F48F-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_колір5", "1F48F-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_колір1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_колір1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_колір1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_колір1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_колір2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_колір2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_колір2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_колір2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_колір3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_колір3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_колір3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_колір3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_колір4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_колір4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_колір4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_колір4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_колір5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_колір5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_колір5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_колір5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"букет", "1F490"},
    {"чоловік_і_жінка_із_серцем_між_ними", "1F491"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір1", "1F491-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір2", "1F491-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір3", "1F491-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір4", "1F491-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір5", "1F491-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_колір5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"весілля", "1F492"},
    {"серце_що_бється", "1F493"},
    {"розбите_серце", "1F494"},
    {"два_серця", "1F495"},
    {"серце_що_виблискує", "1F496"},
    {"серце_що_зростає", "1F497"},
    {"серце_пробите_стрілою", "1F498"},
    {"синє_серце", "1F499"},
    {"зелене_серце", "1F49A"},
    {"жовте_серце", "1F49B"},
    {"пурпурове_серце", "1F49C"},
    {"серце_зі_стрічкою", "1F49D"},
    {"серця_що_обертаються", "1F49E"},
    {"прикраса_із_серцем", "1F49F"},
    {"ромб_із_крапкою", "1F4A0"},
    {"лампочка", "1F4A1"},
    {"символ_гніву", "1F4A2"},
    {"бомба", "1F4A3"},
    {"умовне_позначення_сну", "1F4A4"},
    {"зіткнення", "1F4A5"},
    {"краплі_поту", "1F4A6"},
    {"крапля", "1F4A7"},
    {"порив_вітру", "1F4A8"},
    {"усміхнена_купа_лайна", "1F4A9"},
    {"напружений_біцепс", "1F4AA"},
    {"напружений_біцепс_колір1", "1F4AA-1F3FB"},
    {"напружений_біцепс_колір2", "1F4AA-1F3FC"},
    {"напружений_біцепс_колір3", "1F4AA-1F3FD"},
    {"напружений_біцепс_колір4", "1F4AA-1F3FE"},
    {"напружений_біцепс_колір5", "1F4AA-1F3FF"},
    {"символ_запаморочення", "1F4AB"},
    {"текстова_булька", "1F4AC"},
    {"булька_роздумів", "1F4AD"},
    {"біла_квітка", "1F4AE"},
    {"cто_балів", "1F4AF"},
    {"мішок_грошей", "1F4B0"},
    {"обмін_валют", "1F4B1"},
    {"знак_долара", "1F4B2"},
    {"кредитна_картка", "1F4B3"},
    {"єнові_банкноти", "1F4B4"},
    {"доларові_банкноти", "1F4B5"},
    {"єврові_банкноти", "1F4B6"},
    {"фунтові_банкноти", "1F4B7"},
    {"гроші_з_крилами", "1F4B8"},
    {"графік_зростання_і_знак_єни", "1F4B9"},
    {"сидіння", "1F4BA"},
    {"ноутбук", "1F4BB"},
    {"кейс", "1F4BC"},
    {"компютерний_диск", "1F4BD"},
    {"дискета", "1F4BE"},
    {"компакт_диск", "1F4BF"},
    {"dvd_диск", "1F4C0"},
    {"папка_для_файлів", "1F4C1"},
    {"відкрита_папка_для_файлів", "1F4C2"},
    {"сторінка_із_завитком", "1F4C3"},
    {"сторінка_лицевою_стороною", "1F4C4"},
    {"календар", "1F4C5"},
    {"відривний_календар", "1F4C6"},
    {"картотека", "1F4C7"},
    {"висхідний_графік", "1F4C8"},
    {"низхідний_графік", "1F4C9"},
    {"гістограма", "1F4CA"},
    {"планшет_для_аркушів", "1F4CB"},
    {"канцелярська_кнопка", "1F4CC"},
    {"шпилька_з_круглою_голівкою", "1F4CD"},
    {"скріпка", "1F4CE"},
    {"пряма_лінійка", "1F4CF"},
    {"трикутна_лінійка", "1F4D0"},
    {"мітки_закладок", "1F4D1"},
    {"гросбух", "1F4D2"},
    {"зошит_для_творів", "1F4D3"},
    {"записник_у_декоративній_обкладинці", "1F4D4"},
    {"закрита_книга", "1F4D5"},
    {"відкрита_книга", "1F4D6"},
    {"зелений_підручник", "1F4D7"},
    {"синій_підручник", "1F4D8"},
    {"оранжевий_підручник", "1F4D9"},
    {"книги", "1F4DA"},
    {"бейдж_з_іменем", "1F4DB"},
    {"сувій", "1F4DC"},
    {"памятка", "1F4DD"},
    {"телефонна_слухавка", "1F4DE"},
    {"пейджер", "1F4DF"},
    {"факс", "1F4E0"},
    {"супутникова_антена", "1F4E1"},
    {"гучномовець", "1F4E2"},
    {"мегафон", "1F4E3"},
    {"лоток_вихідної_скриньки", "1F4E4"},
    {"лоток_вхідної_скриньки", "1F4E5"},
    {"пакунок", "1F4E6"},
    {"електронний_лист", "1F4E7"},
    {"конверт_вхідного_листа", "1F4E8"},
    {"конверт_зі_стрілкою", "1F4E9"},
    {"закрита_поштова_скринька_з_опущеним_прапорцем", "1F4EA"},
    {"закрита_поштова_скринька_з_піднятим_прапорцем", "1F4EB"},
    {"відкрита_поштова_скринька_з_піднятим_прапорцем", "1F4EC"},
    {"відкрита_поштова_скринька_з_опущеним_прапорцем", "1F4ED"},
    {"поштова_скринька", "1F4EE"},
    {"поштовий_ріжок", "1F4EF"},
    {"газета", "1F4F0"},
    {"мобільний_телефон", "1F4F1"},
    {"мобільний_телефон_зі_стрілкою", "1F4F2"},
    {"віброрежим", "1F4F3"},
    {"мобільний_телефон_вимкнено", "1F4F4"},
    {"мобільні_телефони_заборонено", "1F4F5"},
    {"рівень_сигналу", "1F4F6"},
    {"фотоапарат", "1F4F7"},
    {"фотоапарат_зі_спалахом", "1F4F8"},
    {"відеокамера", "1F4F9"},
    {"телевізор", "1F4FA"},
    {"радіоприймач", "1F4FB"},
    {"відеокасета", "1F4FC"},
    {"кінопроектор", "1F4FD"},
    {"молитовні_чотки", "1F4FF"},
    {"кнопка_тасування_композицій", "1F500"},
    {"кнопка_повтору", "1F501"},
    {"кнопка_одноразового_повтору", "1F502"},
    {"вертикальні_стрілки_за_ходом_годинника", "1F503"},
    {"кнопка_стрілок_проти_ходу_годинника", "1F504"},
    {"кнопка_зменшення_яскравості", "1F505"},
    {"кнопка_збільшення_яскравості", "1F506"},
    {"заглушений_динамік", "1F507"},
    {"низька_гучність_динаміка", "1F508"},
    {"середня_гучність_динаміка", "1F509"},
    {"високий_рівень_гучності_динаміка", "1F50A"},
    {"батарея", "1F50B"},
    {"штепсель", "1F50C"},
    {"збільшувальне_скло_з_нахилом_уліво", "1F50D"},
    {"збільшувальне_скло_з_нахилом_управо", "1F50E"},
    {"замок_і_чорнильна_ручка", "1F50F"},
    {"замкнений_замок_із_ключем", "1F510"},
    {"ключ", "1F511"},
    {"замкнений_замок", "1F512"},
    {"відімкнений_замок", "1F513"},
    {"дзвінок", "1F514"},
    {"заглушений_дзвінок", "1F515"},
    {"закладка", "1F516"},
    {"символ_зєднання_ланок", "1F517"},
    {"перемикач", "1F518"},
    {"стрілка_назад_back", "1F519"},
    {"стрілка_end", "1F51A"},
    {"стрілка_on", "1F51B"},
    {"стрілка_soon", "1F51C"},
    {"стрілка_top", "1F51D"},
    {"вхід_особам_до_18_років_заборонений", "1F51E"},
    {"клавіша_10", "1F51F"},
    {"введення_великими_латинськими_літерами", "1F520"},
    {"введення_малими_латинськими_літерами", "1F521"},
    {"введення_цифрами", "1F522"},
    {"введення_символів", "1F523"},
    {"введення_латинськими_літерами", "1F524"},
    {"вогонь", "1F525"},
    {"ліхтарик", "1F526"},
    {"гайковий_ключ", "1F527"},
    {"молоток", "1F528"},
    {"гайка_та_болт", "1F529"},
    {"кухонний_ніж", "1F52A"},
    {"водяний_пістолет", "1F52B"},
    {"мікроскоп", "1F52C"},
    {"телескоп", "1F52D"},
    {"кришталева_куля", "1F52E"},
    {"шестикутна_зірка_з_крапкою", "1F52F"},
    {"японський_символ_початківець", "1F530"},
    {"тризуб", "1F531"},
    {"квадратна_кнопка_з_чорною_рамкою", "1F532"},
    {"квадратна_кнопка_з_білою_рамкою", "1F533"},
    {"червоний_круг", "1F534"},
    {"cиній_круг", "1F535"},
    {"великий_оранжевий_ромб", "1F536"},
    {"великий_синій_ромб", "1F537"},
    {"малий_оранжевий_ромб", "1F538"},
    {"малий_синій_ромб", "1F539"},
    {"червоний_трикутник_направлений_угору", "1F53A"},
    {"червоний_трикутник_направлений_униз", "1F53B"},
    {"кнопка_догори", "1F53C"},
    {"кнопка_донизу", "1F53D"},
    {"звук_ом", "1F549"},
    {"голуб", "1F54A"},
    {"кааба", "1F54B"},
    {"мечеть", "1F54C"},
    {"синагога", "1F54D"},
    {"менора", "1F54E"},
    {"перша_година", "1F550"},
    {"друга_година", "1F551"},
    {"третя_година", "1F552"},
    {"четверта_година", "1F553"},
    {"пята_година", "1F554"},
    {"шоста_година", "1F555"},
    {"сьома_година", "1F556"},
    {"восьма_година", "1F557"},
    {"девята_година", "1F558"},
    {"десята_година", "1F559"},
    {"одинадцята_година", "1F55A"},
    {"дванадцята_година", "1F55B"},
    {"пів_на_другу", "1F55C"},
    {"пів_на_третю", "1F55D"},
    {"пів_на_четверту", "1F55E"},
    {"пів_на_пяту", "1F55F"},
    {"пів_на_шосту", "1F560"},
    {"пів_на_сьому", "1F561"},
    {"пів_на_восьму", "1F562"},
    {"пів_на_девяту", "1F563"},
    {"пів_на_десяту", "1F564"},
    {"пів_на_одинадцяту", "1F565"},
    {"пів_на_дванадцяту", "1F566"},
    {"пів_на_першу", "1F567"},
    {"свічка", "1F56F"},
    {"камінний_годинник", "1F570"},
    {"лунка", "1F573"},
    {"чоловік_у_діловому_костюмі_що_левітує", "1F574"},
    {"чоловік_у_діловому_костюмі_що_левітує_колір1", "1F574-1F3FB"},
    {"чоловік_у_діловому_костюмі_що_левітує_колір2", "1F574-1F3FC"},
    {"чоловік_у_діловому_костюмі_що_левітує_колір3", "1F574-1F3FD"},
    {"чоловік_у_діловому_костюмі_що_левітує_колір4", "1F574-1F3FE"},
    {"чоловік_у_діловому_костюмі_що_левітує_колір5", "1F574-1F3FF"},
    {"детектив", "1F575"},
    {"детектив_колір1", "1F575-1F3FB"},
    {"детектив_колір2", "1F575-1F3FC"},
    {"детектив_колір3", "1F575-1F3FD"},
    {"детектив_колір4", "1F575-1F3FE"},
    {"детектив_колір5", "1F575-1F3FF"},
    {"сонцезахисні_окуляри", "1F576"},
    {"павук", "1F577"},
    {"павутиння", "1F578"},
    {"джойстик", "1F579"},
    {"чоловік_що_танцює", "1F57A"},
    {"чоловік_що_танцює_колір1", "1F57A-1F3FB"},
    {"чоловік_що_танцює_колір2", "1F57A-1F3FC"},
    {"чоловік_що_танцює_колір3", "1F57A-1F3FD"},
    {"чоловік_що_танцює_колір4", "1F57A-1F3FE"},
    {"чоловік_що_танцює_колір5", "1F57A-1F3FF"},
    {"зчеплені_скріпки", "1F587"},
    {"кулькова_ручка", "1F58A"},
    {"чорнильна_ручка", "1F58B"},
    {"пензель", "1F58C"},
    {"пастель", "1F58D"},
    {"піднята_рука_з_розведеними_пальцями", "1F590"},
    {"піднята_рука_з_розведеними_пальцями_колір1", "1F590-1F3FB"},
    {"піднята_рука_з_розведеними_пальцями_колір2", "1F590-1F3FC"},
    {"піднята_рука_з_розведеними_пальцями_колір3", "1F590-1F3FD"},
    {"піднята_рука_з_розведеними_пальцями_колір4", "1F590-1F3FE"},
    {"піднята_рука_з_розведеними_пальцями_колір5", "1F590-1F3FF"},
    {"середній_палець", "1F595"},
    {"середній_палець_колір1", "1F595-1F3FB"},
    {"середній_палець_колір2", "1F595-1F3FC"},
    {"середній_палець_колір3", "1F595-1F3FD"},
    {"середній_палець_колір4", "1F595-1F3FE"},
    {"середній_палець_колір5", "1F595-1F3FF"},
    {"вулканське_вітання", "1F596"},
    {"вулканське_вітання_колір1", "1F596-1F3FB"},
    {"вулканське_вітання_колір2", "1F596-1F3FC"},
    {"вулканське_вітання_колір3", "1F596-1F3FD"},
    {"вулканське_вітання_колір4", "1F596-1F3FE"},
    {"вулканське_вітання_колір5", "1F596-1F3FF"},
    {"чорне_серце", "1F5A4"},
    {"компютер", "1F5A5"},
    {"принтер", "1F5A8"},
    {"компютерна_миша", "1F5B1"},
    {"трекбол", "1F5B2"},
    {"картина_в_рамці", "1F5BC"},
    {"роздільники_в_картотеці", "1F5C2"},
    {"скринька_для_карток", "1F5C3"},
    {"шафа_для_документів", "1F5C4"},
    {"сміттєвий_кошик", "1F5D1"},
    {"блокнот_на_спіралі", "1F5D2"},
    {"календар_на_спіралі", "1F5D3"},
    {"затискач", "1F5DC"},
    {"старовинний_ключ", "1F5DD"},
    {"скручена_газета", "1F5DE"},
    {"кинджал", "1F5E1"},
    {"голова_що_говорить", "1F5E3"},
    {"хмаринка_з_текстом_зліва", "1F5E8"},
    {"булька_із_сердитою_реплікою", "1F5EF"},
    {"урна_для_голосування_і_бюлетень", "1F5F3"},
    {"карта_світу", "1F5FA"},
    {"гора_фудзі", "1F5FB"},
    {"токійська_вежа", "1F5FC"},
    {"статуя_свободи", "1F5FD"},
    {"карта_японії", "1F5FE"},
    {"статуя_з_острова_пасхи", "1F5FF"},
    {"широко_всміхнене_обличчя", "1F600"},
    {"осяйно_всміхнене_обличчя_й_очі", "1F601"},
    {"обличчя_в_сльозах_радості", "1F602"},
    {"широко_всміхнене_обличчя_з_очищами", "1F603"},
    {"широко_всміхнене_обличчя_з_примруженими_очима", "1F604"},
    {"широко_всміхнене_обличчя_з_потом", "1F605"},
    {"обличчя_із_широкою_посмішкою_й_замруженими_очима", "1F606"},
    {"усміхнене_обличчя_з_німбом", "1F607"},
    {"усміхнене_обличчя_з_рогами", "1F608"},
    {"обличчя_що_підморгує", "1F609"},
    {"усміхнене_обличчя_й_очі", "1F60A"},
    {"усміхнене_обличчя_що_облизує_губи", "1F60B"},
    {"заспокоєне_обличчя", "1F60C"},
    {"усміхнене_обличчя_з_очима_серцями", "1F60D"},
    {"усміхнене_обличчя_в_темних_окулярах", "1F60E"},
    {"самовдоволене_обличчя", "1F60F"},
    {"обличчя_з_нейтральним_виразом", "1F610"},
    {"обличчя_без_емоцій", "1F611"},
    {"невдоволене_обличчя", "1F612"},
    {"понурене_обличчя_в_поту", "1F613"},
    {"замислене_обличчя", "1F614"},
    {"спантеличене_обличчя", "1F615"},
    {"збентежене_обличчя", "1F616"},
    {"обличчя_що_цілує", "1F617"},
    {"обличчя_що_надсилає_цілунок", "1F618"},
    {"обличчя_з_примруженими_очима_що_цілує", "1F619"},
    {"обличчя_із_заплющеними_очима_що_цілує", "1F61A"},
    {"обличчя_з_висунутим_язиком", "1F61B"},
    {"обличчя_що_підморгує_і_показує_язика", "1F61C"},
    {"обличчя_з_висунутим_язиком_і_заплющеними_очима", "1F61D"},
    {"розчароване_обличчя", "1F61E"},
    {"стурбоване_обличчя", "1F61F"},
    {"сердите_обличчя", "1F620"},
    {"обличчя_що_дується", "1F621"},
    {"обличчя_що_плаче", "1F622"},
    {"невідступне_обличчя", "1F623"},
    {"обличчя_що_сердито_видихає", "1F624"},
    {"cумне_але_втішене_обличчя", "1F625"},
    {"похмуре_обличчя_з_відкритим_ротом", "1F626"},
    {"стражденне_обличчя", "1F627"},
    {"налякане_обличчя", "1F628"},
    {"замучене_обличчя", "1F629"},
    {"заспане_обличчя", "1F62A"},
    {"стомлене_обличчя", "1F62B"},
    {"обличчя_з_гримасою", "1F62C"},
    {"обличчя_що_гучно_ридає", "1F62D"},
    {"обличчя_з_відкритим_ротом", "1F62E"},
    {"здивоване_і_збентежене_обличчя_з_бровами", "1F62F"},
    {"занепокоєне_обличчя_з_потом", "1F630"},
    {"обличчя_що_кричить_із_переляку", "1F631"},
    {"вражене_обличчя", "1F632"},
    {"зашаріле_обличчя", "1F633"},
    {"обличчя_що_спить", "1F634"},
    {"запаморочене_обличчя", "1F635"},
    {"обличчя_без_рота", "1F636"},
    {"обличчя_в_медичній_масці", "1F637"},
    {"широко_всміхнений_кіт_з_усміхом_в_очах", "1F638"},
    {"кіт_зі_сльозами_радості", "1F639"},
    {"широко_всміхнений_кіт", "1F63A"},
    {"усміхнений_кіт_з_очима_як_серця", "1F63B"},
    {"кіт_із_кривою_посмішкою", "1F63C"},
    {"кіт_що_цілує", "1F63D"},
    {"сердитий_кіт", "1F63E"},
    {"кіт_що_плаче", "1F63F"},
    {"шокований_кіт", "1F640"},
    {"трохи_похмуре_обличчя", "1F641"},
    {"трохи_всміхнене_обличчя", "1F642"},
    {"обличчя_догори_дриґом", "1F643"},
    {"обличчя_із_закоченими_очима", "1F644"},
    {"людина_що_жестом_показує_ні", "1F645"},
    {"людина_що_жестом_показує_ні_колір1", "1F645-1F3FB"},
    {"людина_що_жестом_показує_ні_колір2", "1F645-1F3FC"},
    {"людина_що_жестом_показує_ні_колір3", "1F645-1F3FD"},
    {"людина_що_жестом_показує_ні_колір4", "1F645-1F3FE"},
    {"людина_що_жестом_показує_ні_колір5", "1F645-1F3FF"},
    {"людина_що_жестом_показує_окей", "1F646"},
    {"людина_що_жестом_показує_окей_колір1", "1F646-1F3FB"},
    {"людина_що_жестом_показує_окей_колір2", "1F646-1F3FC"},
    {"людина_що_жестом_показує_окей_колір3", "1F646-1F3FD"},
    {"людина_що_жестом_показує_окей_колір4", "1F646-1F3FE"},
    {"людина_що_жестом_показує_окей_колір5", "1F646-1F3FF"},
    {"людина_що_вклоняється", "1F647"},
    {"людина_що_вклоняється_колір1", "1F647-1F3FB"},
    {"людина_що_вклоняється_колір2", "1F647-1F3FC"},
    {"людина_що_вклоняється_колір3", "1F647-1F3FD"},
    {"людина_що_вклоняється_колір4", "1F647-1F3FE"},
    {"людина_що_вклоняється_колір5", "1F647-1F3FF"},
    {"мавпа_що_не_бачить", "1F648"},
    {"мавпа_що_не_чує", "1F649"},
    {"мавпа_що_не_говорить", "1F64A"},
    {"людина_з_піднятою_рукою", "1F64B"},
    {"людина_з_піднятою_рукою_колір1", "1F64B-1F3FB"},
    {"людина_з_піднятою_рукою_колір2", "1F64B-1F3FC"},
    {"людина_з_піднятою_рукою_колір3", "1F64B-1F3FD"},
    {"людина_з_піднятою_рукою_колір4", "1F64B-1F3FE"},
    {"людина_з_піднятою_рукою_колір5", "1F64B-1F3FF"},
    {"руки_підняті_для_святкування", "1F64C"},
    {"руки_підняті_для_святкування_колір1", "1F64C-1F3FB"},
    {"руки_підняті_для_святкування_колір2", "1F64C-1F3FC"},
    {"руки_підняті_для_святкування_колір3", "1F64C-1F3FD"},
    {"руки_підняті_для_святкування_колір4", "1F64C-1F3FE"},
    {"руки_підняті_для_святкування_колір5", "1F64C-1F3FF"},
    {"похмура_людина", "1F64D"},
    {"похмура_людина_колір1", "1F64D-1F3FB"},
    {"похмура_людина_колір2", "1F64D-1F3FC"},
    {"похмура_людина_колір3", "1F64D-1F3FD"},
    {"похмура_людина_колір4", "1F64D-1F3FE"},
    {"похмура_людина_колір5", "1F64D-1F3FF"},
    {"людина_що_дується", "1F64E"},
    {"людина_що_дується_колір1", "1F64E-1F3FB"},
    {"людина_що_дується_колір2", "1F64E-1F3FC"},
    {"людина_що_дується_колір3", "1F64E-1F3FD"},
    {"людина_що_дується_колір4", "1F64E-1F3FE"},
    {"людина_що_дується_колір5", "1F64E-1F3FF"},
    {"руки_стиснуті_разом", "1F64F"},
    {"руки_стиснуті_разом_колір1", "1F64F-1F3FB"},
    {"руки_стиснуті_разом_колір2", "1F64F-1F3FC"},
    {"руки_стиснуті_разом_колір3", "1F64F-1F3FD"},
    {"руки_стиснуті_разом_колір4", "1F64F-1F3FE"},
    {"руки_стиснуті_разом_колір5", "1F64F-1F3FF"},
    {"ракета", "1F680"},
    {"гелікоптер", "1F681"},
    {"паровий_локомотив", "1F682"},
    {"вагон", "1F683"},
    {"швидкісний_потяг", "1F684"},
    {"швидкісний_потяг_з_округлим_носом", "1F685"},
    {"потяг", "1F686"},
    {"метро", "1F687"},
    {"швидкісний_трамвай", "1F688"},
    {"станція", "1F689"},
    {"трамвай", "1F68A"},
    {"вагон_трамвая", "1F68B"},
    {"автобус", "1F68C"},
    {"автобус_що_наближається", "1F68D"},
    {"тролейбус", "1F68E"},
    {"автобусна_зупинка", "1F68F"},
    {"мікроавтобус", "1F690"},
    {"автомобіль_швидкої_допомоги", "1F691"},
    {"пожежна_машина", "1F692"},
    {"поліцейська_машина", "1F693"},
    {"поліцейська_машина_що_наближається", "1F694"},
    {"таксі", "1F695"},
    {"таксі_що_наближається", "1F696"},
    {"автомобіль", "1F697"},
    {"автомобіль_що_наближається", "1F698"},
    {"позашляховик", "1F699"},
    {"вантажівка_для_доставки", "1F69A"},
    {"вантажівка_з_причепом", "1F69B"},
    {"трактор", "1F69C"},
    {"монорейка", "1F69D"},
    {"залізниця_в_горах", "1F69E"},
    {"підвісна_монорейка", "1F69F"},
    {"канатна_дорога_в_горах", "1F6A0"},
    {"канатна_дорога", "1F6A1"},
    {"корабель", "1F6A2"},
    {"людина_що_веслує_на_човні", "1F6A3"},
    {"людина_що_веслує_на_човні_колір1", "1F6A3-1F3FB"},
    {"людина_що_веслує_на_човні_колір2", "1F6A3-1F3FC"},
    {"людина_що_веслує_на_човні_колір3", "1F6A3-1F3FD"},
    {"людина_що_веслує_на_човні_колір4", "1F6A3-1F3FE"},
    {"людина_що_веслує_на_човні_колір5", "1F6A3-1F3FF"},
    {"швидкохідний_катер", "1F6A4"},
    {"горизонтальний_світлофор", "1F6A5"},
    {"вертикальний_світлофор", "1F6A6"},
    {"знак_ремонтних_робіт", "1F6A7"},
    {"проблисковий_маячок", "1F6A8"},
    {"трикутний_прапор", "1F6A9"},
    {"двері", "1F6AA"},
    {"заборонено", "1F6AB"},
    {"сигарета", "1F6AC"},
    {"палити_заборонено", "1F6AD"},
    {"знак_кидати_сміття_в_урну", "1F6AE"},
    {"смітити_заборонено", "1F6AF"},
    {"питна_вода", "1F6B0"},
    {"непридатна_для_пиття_вода", "1F6B1"},
    {"велосипед", "1F6B2"},
    {"велосипеди_заборонено", "1F6B3"},
    {"людина_що_їде_на_велосипеді", "1F6B4"},
    {"людина_що_їде_на_велосипеді_колір1", "1F6B4-1F3FB"},
    {"людина_що_їде_на_велосипеді_колір2", "1F6B4-1F3FC"},
    {"людина_що_їде_на_велосипеді_колір3", "1F6B4-1F3FD"},
    {"людина_що_їде_на_велосипеді_колір4", "1F6B4-1F3FE"},
    {"людина_що_їде_на_велосипеді_колір5", "1F6B4-1F3FF"},
    {"людина_що_їде_на_гірському_велосипеді", "1F6B5"},
    {"людина_що_їде_на_гірському_велосипеді_колір1", "1F6B5-1F3FB"},
    {"людина_що_їде_на_гірському_велосипеді_колір2", "1F6B5-1F3FC"},
    {"людина_що_їде_на_гірському_велосипеді_колір3", "1F6B5-1F3FD"},
    {"людина_що_їде_на_гірському_велосипеді_колір4", "1F6B5-1F3FE"},
    {"людина_що_їде_на_гірському_велосипеді_колір5", "1F6B5-1F3FF"},
    {"людина_що_йде_пішки", "1F6B6"},
    {"людина_що_йде_пішки_колір1", "1F6B6-1F3FB"},
    {"людина_що_йде_пішки_колір2", "1F6B6-1F3FC"},
    {"людина_що_йде_пішки_колір3", "1F6B6-1F3FD"},
    {"людина_що_йде_пішки_колір4", "1F6B6-1F3FE"},
    {"людина_що_йде_пішки_колір5", "1F6B6-1F3FF"},
    {"пішохідний_рух_заборонено", "1F6B7"},
    {"попереджувальний_знак_діти", "1F6B8"},
    {"чоловіча_вбиральня", "1F6B9"},
    {"жіноча_вбиральня", "1F6BA"},
    {"громадський_туалет", "1F6BB"},
    {"символ_немовляти", "1F6BC"},
    {"унітаз", "1F6BD"},
    {"символ_убиральні_wc", "1F6BE"},
    {"душ", "1F6BF"},
    {"людина_що_приймає_ванну", "1F6C0"},
    {"людина_що_приймає_ванну_колір1", "1F6C0-1F3FB"},
    {"людина_що_приймає_ванну_колір2", "1F6C0-1F3FC"},
    {"людина_що_приймає_ванну_колір3", "1F6C0-1F3FD"},
    {"людина_що_приймає_ванну_колір4", "1F6C0-1F3FE"},
    {"людина_що_приймає_ванну_колір5", "1F6C0-1F3FF"},
    {"ванна", "1F6C1"},
    {"паспортний_контроль", "1F6C2"},
    {"митниця", "1F6C3"},
    {"видача_багажу", "1F6C4"},
    {"камера_схову", "1F6C5"},
    {"канапа_й_торшер", "1F6CB"},
    {"людина_в_ліжку", "1F6CC"},
    {"людина_в_ліжку_колір1", "1F6CC-1F3FB"},
    {"людина_в_ліжку_колір2", "1F6CC-1F3FC"},
    {"людина_в_ліжку_колір3", "1F6CC-1F3FD"},
    {"людина_в_ліжку_колір4", "1F6CC-1F3FE"},
    {"людина_в_ліжку_колір5", "1F6CC-1F3FF"},
    {"сумки_для_покупок", "1F6CD"},
    {"готельний_дзвоник", "1F6CE"},
    {"ліжко", "1F6CF"},
    {"культова_споруда", "1F6D0"},
    {"восьмикутний_червоний_знак", "1F6D1"},
    {"магазинний_візок", "1F6D2"},
    {"індуїстський_храм", "1F6D5"},
    {"хижа", "1F6D6"},
    {"ліфт", "1F6D7"},
    {"зсув", "1F6D8"},
    {"бездротовий", "1F6DC"},
    {"дитяча_гірка", "1F6DD"},
    {"колесо", "1F6DE"},
    {"рятівний_круг", "1F6DF"},
    {"молоток_і_гайковий_ключ", "1F6E0"},
    {"щит", "1F6E1"},
    {"бочка_для_нафти", "1F6E2"},
    {"автомагістраль", "1F6E3"},
    {"залізнична_колія", "1F6E4"},
    {"моторний_човен", "1F6E5"},
    {"малий_літак", "1F6E9"},
    {"літак_злітає", "1F6EB"},
    {"літак_сідає", "1F6EC"},
    {"супутник", "1F6F0"},
    {"пасажирський_корабель", "1F6F3"},
    {"самокат", "1F6F4"},
    {"мопед", "1F6F5"},
    {"каное", "1F6F6"},
    {"сани", "1F6F7"},
    {"летюча_тарілка", "1F6F8"},
    {"скейтборд", "1F6F9"},
    {"авторикша", "1F6FA"},
    {"автомобіль_пікап", "1F6FB"},
    {"роликовий_ковзан", "1F6FC"},
    {"оранжевий_круг", "1F7E0"},
    {"жовтий_круг", "1F7E1"},
    {"зелений_круг", "1F7E2"},
    {"фіолетовий_круг", "1F7E3"},
    {"коричневий_круг", "1F7E4"},
    {"червоний_квадрат", "1F7E5"},
    {"синій_квадрат", "1F7E6"},
    {"оранжевий_квадрат", "1F7E7"},
    {"жовтий_квадрат", "1F7E8"},
    {"зелений_квадрат", "1F7E9"},
    {"фіолетовий_квадрат", "1F7EA"},
    {"коричневий_квадрат", "1F7EB"},
    {"знак_точної_рівності", "1F7F0"},
    {"зведені_пальці", "1F90C"},
    {"зведені_пальці_колір1", "1F90C-1F3FB"},
    {"зведені_пальці_колір2", "1F90C-1F3FC"},
    {"зведені_пальці_колір3", "1F90C-1F3FD"},
    {"зведені_пальці_колір4", "1F90C-1F3FE"},
    {"зведені_пальці_колір5", "1F90C-1F3FF"},
    {"біле_серце", "1F90D"},
    {"коричневе_серце", "1F90E"},
    {"жест_мало", "1F90F"},
    {"жест_мало_колір1", "1F90F-1F3FB"},
    {"жест_мало_колір2", "1F90F-1F3FC"},
    {"жест_мало_колір3", "1F90F-1F3FD"},
    {"жест_мало_колір4", "1F90F-1F3FE"},
    {"жест_мало_колір5", "1F90F-1F3FF"},
    {"обличчя_із_застібкою_замість_рота", "1F910"},
    {"обличчя_з_грішми_замість_очей_та_язика", "1F911"},
    {"обличчя_з_термометром", "1F912"},
    {"обличчя_всезнайка", "1F913"},
    {"обличчя_в_роздумах", "1F914"},
    {"обличчя_з_головною_повязкою", "1F915"},
    {"робот", "1F916"},
    {"щасливе_обличчя_з_руками_що_обіймають", "1F917"},
    {"жест_роги", "1F918"},
    {"жест_роги_колір1", "1F918-1F3FB"},
    {"жест_роги_колір2", "1F918-1F3FC"},
    {"жест_роги_колір3", "1F918-1F3FD"},
    {"жест_роги_колір4", "1F918-1F3FE"},
    {"жест_роги_колір5", "1F918-1F3FF"},
    {"жест_подзвони_мені", "1F919"},
    {"жест_подзвони_мені_колір1", "1F919-1F3FB"},
    {"жест_подзвони_мені_колір2", "1F919-1F3FC"},
    {"жест_подзвони_мені_колір3", "1F919-1F3FD"},
    {"жест_подзвони_мені_колір4", "1F919-1F3FE"},
    {"жест_подзвони_мені_колір5", "1F919-1F3FF"},
    {"піднята_рука_тильною_стороною", "1F91A"},
    {"піднята_рука_тильною_стороною_колір1", "1F91A-1F3FB"},
    {"піднята_рука_тильною_стороною_колір2", "1F91A-1F3FC"},
    {"піднята_рука_тильною_стороною_колір3", "1F91A-1F3FD"},
    {"піднята_рука_тильною_стороною_колір4", "1F91A-1F3FE"},
    {"піднята_рука_тильною_стороною_колір5", "1F91A-1F3FF"},
    {"кулак_уліво", "1F91B"},
    {"кулак_уліво_колір1", "1F91B-1F3FB"},
    {"кулак_уліво_колір2", "1F91B-1F3FC"},
    {"кулак_уліво_колір3", "1F91B-1F3FD"},
    {"кулак_уліво_колір4", "1F91B-1F3FE"},
    {"кулак_уліво_колір5", "1F91B-1F3FF"},
    {"кулак_управо", "1F91C"},
    {"кулак_управо_колір1", "1F91C-1F3FB"},
    {"кулак_управо_колір2", "1F91C-1F3FC"},
    {"кулак_управо_колір3", "1F91C-1F3FD"},
    {"кулак_управо_колір4", "1F91C-1F3FE"},
    {"кулак_управо_колір5", "1F91C-1F3FF"},
    {"рукостискання", "1F91D"},
    {"рукостискання_колір1", "1F91D-1F3FB"},
    {"рукостискання_колір2", "1F91D-1F3FC"},
    {"рукостискання_колір3", "1F91D-1F3FD"},
    {"рукостискання_колір4", "1F91D-1F3FE"},
    {"рукостискання_колір5", "1F91D-1F3FF"},
    {"рукостискання_колір1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"рукостискання_колір1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"рукостискання_колір1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"рукостискання_колір1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"рукостискання_колір2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"рукостискання_колір2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"рукостискання_колір2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"рукостискання_колір2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"рукостискання_колір3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"рукостискання_колір3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"рукостискання_колір3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"рукостискання_колір3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"рукостискання_колір4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"рукостискання_колір4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"рукостискання_колір4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"рукостискання_колір4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"рукостискання_колір5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"рукостискання_колір5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"рукостискання_колір5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"рукостискання_колір5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"схрещені_пальці", "1F91E"},
    {"схрещені_пальці_колір1", "1F91E-1F3FB"},
    {"схрещені_пальці_колір2", "1F91E-1F3FC"},
    {"схрещені_пальці_колір3", "1F91E-1F3FD"},
    {"схрещені_пальці_колір4", "1F91E-1F3FE"},
    {"схрещені_пальці_колір5", "1F91E-1F3FF"},
    {"жест_кохаю_тебе", "1F91F"},
    {"жест_кохаю_тебе_колір1", "1F91F-1F3FB"},
    {"жест_кохаю_тебе_колір2", "1F91F-1F3FC"},
    {"жест_кохаю_тебе_колір3", "1F91F-1F3FD"},
    {"жест_кохаю_тебе_колір4", "1F91F-1F3FE"},
    {"жест_кохаю_тебе_колір5", "1F91F-1F3FF"},
    {"обличчя_у_ковбойському_капелюсі", "1F920"},
    {"обличчя_клоуна", "1F921"},
    {"обличчя_яке_нудить", "1F922"},
    {"качається_від_сміху", "1F923"},
    {"обличчя_що_пускає_слину", "1F924"},
    {"обличчя_що_бреше", "1F925"},
    {"людина_з_долонею_до_обличчя", "1F926"},
    {"людина_з_долонею_до_обличчя_колір1", "1F926-1F3FB"},
    {"людина_з_долонею_до_обличчя_колір2", "1F926-1F3FC"},
    {"людина_з_долонею_до_обличчя_колір3", "1F926-1F3FD"},
    {"людина_з_долонею_до_обличчя_колір4", "1F926-1F3FE"},
    {"людина_з_долонею_до_обличчя_колір5", "1F926-1F3FF"},
    {"обличчя_що_чхає", "1F927"},
    {"обличчя_з_піднятими_бровами", "1F928"},
    {"очі_зірочки", "1F929"},
    {"божевільне_обличчя", "1F92A"},
    {"обличчя_що_цикає", "1F92B"},
    {"обличчя_із_символами_в_роті", "1F92C"},
    {"обличчя_що_прикриває_рота_рукою", "1F92D"},
    {"обличчя_що_блює", "1F92E"},
    {"голова_що_вибухає", "1F92F"},
    {"вагітна_жінка", "1F930"},
    {"вагітна_жінка_колір1", "1F930-1F3FB"},
    {"вагітна_жінка_колір2", "1F930-1F3FC"},
    {"вагітна_жінка_колір3", "1F930-1F3FD"},
    {"вагітна_жінка_колір4", "1F930-1F3FE"},
    {"вагітна_жінка_колір5", "1F930-1F3FF"},
    {"грудне_вигодовування", "1F931"},
    {"грудне_вигодовування_колір1", "1F931-1F3FB"},
    {"грудне_вигодовування_колір2", "1F931-1F3FC"},
    {"грудне_вигодовування_колір3", "1F931-1F3FD"},
    {"грудне_вигодовування_колір4", "1F931-1F3FE"},
    {"грудне_вигодовування_колір5", "1F931-1F3FF"},
    {"долоні_підняті_разом", "1F932"},
    {"долоні_підняті_разом_колір1", "1F932-1F3FB"},
    {"долоні_підняті_разом_колір2", "1F932-1F3FC"},
    {"долоні_підняті_разом_колір3", "1F932-1F3FD"},
    {"долоні_підняті_разом_колір4", "1F932-1F3FE"},
    {"долоні_підняті_разом_колір5", "1F932-1F3FF"},
    {"селфі", "1F933"},
    {"селфі_колір1", "1F933-1F3FB"},
    {"селфі_колір2", "1F933-1F3FC"},
    {"селфі_колір3", "1F933-1F3FD"},
    {"селфі_колір4", "1F933-1F3FE"},
    {"селфі_колір5", "1F933-1F3FF"},
    {"принц", "1F934"},
    {"принц_колір1", "1F934-1F3FB"},
    {"принц_колір2", "1F934-1F3FC"},
    {"принц_колір3", "1F934-1F3FD"},
    {"принц_колір4", "1F934-1F3FE"},
    {"принц_колір5", "1F934-1F3FF"},
    {"людина_в_смокінгу", "1F935"},
    {"людина_в_смокінгу_колір1", "1F935-1F3FB"},
    {"людина_в_смокінгу_колір2", "1F935-1F3FC"},
    {"людина_в_смокінгу_колір3", "1F935-1F3FD"},
    {"людина_в_смокінгу_колір4", "1F935-1F3FE"},
    {"людина_в_смокінгу_колір5", "1F935-1F3FF"},
    {"пані_клаус", "1F936"},
    {"пані_клаус_колір1", "1F936-1F3FB"},
    {"пані_клаус_колір2", "1F936-1F3FC"},
    {"пані_клаус_колір3", "1F936-1F3FD"},
    {"пані_клаус_колір4", "1F936-1F3FE"},
    {"пані_клаус_колір5", "1F936-1F3FF"},
    {"людина_що_знизує_плечима", "1F937"},
    {"людина_що_знизує_плечима_колір1", "1F937-1F3FB"},
    {"людина_що_знизує_плечима_колір2", "1F937-1F3FC"},
    {"людина_що_знизує_плечима_колір3", "1F937-1F3FD"},
    {"людина_що_знизує_плечима_колір4", "1F937-1F3FE"},
    {"людина_що_знизує_плечима_колір5", "1F937-1F3FF"},
    {"людина_що_перевертається_колесом", "1F938"},
    {"людина_що_перевертається_колесом_колір1", "1F938-1F3FB"},
    {"людина_що_перевертається_колесом_колір2", "1F938-1F3FC"},
    {"людина_що_перевертається_колесом_колір3", "1F938-1F3FD"},
    {"людина_що_перевертається_колесом_колір4", "1F938-1F3FE"},
    {"людина_що_перевертається_колесом_колір5", "1F938-1F3FF"},
    {"людина_що_жонглює", "1F939"},
    {"людина_що_жонглює_колір1", "1F939-1F3FB"},
    {"людина_що_жонглює_колір2", "1F939-1F3FC"},
    {"людина_що_жонглює_колір3", "1F939-1F3FD"},
    {"людина_що_жонглює_колір4", "1F939-1F3FE"},
    {"людина_що_жонглює_колір5", "1F939-1F3FF"},
    {"фехтувальник", "1F93A"},
    {"борці", "1F93C"},
    {"борці_колір1", "1F93C-1F3FB"},
    {"борці_колір2", "1F93C-1F3FC"},
    {"борці_колір3", "1F93C-1F3FD"},
    {"борці_колір4", "1F93C-1F3FE"},
    {"борці_колір5", "1F93C-1F3FF"},
    {"борці_колір1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"борці_колір1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"борці_колір1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"борці_колір1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"борці_колір2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"борці_колір2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"борці_колір2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"борці_колір2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"борці_колір3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"борці_колір3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"борці_колір3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"борці_колір3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"борці_колір4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"борці_колір4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"борці_колір4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"борці_колір4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"борці_колір5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"борці_колір5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"борці_колір5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"борці_колір5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"людина_що_грає_у_ватерполо", "1F93D"},
    {"людина_що_грає_у_ватерполо_колір1", "1F93D-1F3FB"},
    {"людина_що_грає_у_ватерполо_колір2", "1F93D-1F3FC"},
    {"людина_що_грає_у_ватерполо_колір3", "1F93D-1F3FD"},
    {"людина_що_грає_у_ватерполо_колір4", "1F93D-1F3FE"},
    {"людина_що_грає_у_ватерполо_колір5", "1F93D-1F3FF"},
    {"людина_що_грає_у_гандбол", "1F93E"},
    {"людина_що_грає_у_гандбол_колір1", "1F93E-1F3FB"},
    {"людина_що_грає_у_гандбол_колір2", "1F93E-1F3FC"},
    {"людина_що_грає_у_гандбол_колір3", "1F93E-1F3FD"},
    {"людина_що_грає_у_гандбол_колір4", "1F93E-1F3FE"},
    {"людина_що_грає_у_гандбол_колір5", "1F93E-1F3FF"},
    {"маска_з_трубкою", "1F93F"},
    {"привяла_троянда", "1F940"},
    {"барабан", "1F941"},
    {"дзвін_бокалів", "1F942"},
    {"бокал_віскі", "1F943"},
    {"ложка", "1F944"},
    {"сітка_воріт", "1F945"},
    {"золота_медаль", "1F947"},
    {"срібна_медаль", "1F948"},
    {"бронзова_медаль", "1F949"},
    {"боксерська_рукавичка", "1F94A"},
    {"одяг_для_єдиноборств", "1F94B"},
    {"камяна_шайба_для_керлінгу", "1F94C"},
    {"стік_для_лакросу", "1F94D"},
    {"софтбольний_мяч", "1F94E"},
    {"літаючий_диск", "1F94F"},
    {"круасан", "1F950"},
    {"авокадо", "1F951"},
    {"огірок", "1F952"},
    {"бекон", "1F953"},
    {"картопля", "1F954"},
    {"морква", "1F955"},
    {"багет", "1F956"},
    {"зелений_салат", "1F957"},
    {"каструля_з_їжею", "1F958"},
    {"піта", "1F959"},
    {"яйце", "1F95A"},
    {"склянка_молока", "1F95B"},
    {"арахіс", "1F95C"},
    {"ківі", "1F95D"},
    {"млинці", "1F95E"},
    {"вареник", "1F95F"},
    {"печиво_з_передбаченням", "1F960"},
    {"коробка_з_їжею_на_винос", "1F961"},
    {"палички_для_їжі", "1F962"},
    {"миска_з_ложкою", "1F963"},
    {"стакан_із_соломинкою", "1F964"},
    {"кокос", "1F965"},
    {"броколі", "1F966"},
    {"пиріг", "1F967"},
    {"крендель", "1F968"},
    {"мясний_філей", "1F969"},
    {"сандвіч", "1F96A"},
    {"консерви", "1F96B"},
    {"листя_салату", "1F96C"},
    {"манго", "1F96D"},
    {"мункейк", "1F96E"},
    {"бейгл", "1F96F"},
    {"усміхнене_обличчя_із_сердечками", "1F970"},
    {"обличчя_що_позіхає", "1F971"},
    {"усмішка_в_сльозах", "1F972"},
    {"святкове_обличчя", "1F973"},
    {"очманіле_обличчя", "1F974"},
    {"розпашіле_обличчя", "1F975"},
    {"замерзле_обличчя", "1F976"},
    {"ніндзя", "1F977"},
    {"ніндзя_колір1", "1F977-1F3FB"},
    {"ніндзя_колір2", "1F977-1F3FC"},
    {"ніндзя_колір3", "1F977-1F3FD"},
    {"ніндзя_колір4", "1F977-1F3FE"},
    {"ніндзя_колір5", "1F977-1F3FF"},
    {"замасковане_обличчя", "1F978"},
    {"обличчя_що_ледве_стримує_сльози", "1F979"},
    {"благальне_обличчя", "1F97A"},
    {"сарі", "1F97B"},
    {"лабораторний_халат", "1F97C"},
    {"захисні_окуляри", "1F97D"},
    {"похідні_черевики", "1F97E"},
    {"туфлі_без_підборів", "1F97F"},
    {"краб", "1F980"},
    {"лев", "1F981"},
    {"скорпіон", "1F982"},
    {"індик", "1F983"},
    {"єдиноріг", "1F984"},
    {"орел", "1F985"},
    {"качка", "1F986"},
    {"кажан", "1F987"},
    {"акула", "1F988"},
    {"сова", "1F989"},
    {"лис", "1F98A"},
    {"метелик", "1F98B"},
    {"олень", "1F98C"},
    {"горила", "1F98D"},
    {"ящірка", "1F98E"},
    {"носоріг", "1F98F"},
    {"креветка", "1F990"},
    {"кальмар", "1F991"},
    {"жирафа", "1F992"},
    {"зебра", "1F993"},
    {"їжак", "1F994"},
    {"завропод", "1F995"},
    {"тиранозавр", "1F996"},
    {"цвіркун", "1F997"},
    {"кенгуру", "1F998"},
    {"лама", "1F999"},
    {"павич", "1F99A"},
    {"бегемот", "1F99B"},
    {"папуга", "1F99C"},
    {"єнот", "1F99D"},
    {"омар", "1F99E"},
    {"комар", "1F99F"},
    {"мікроб", "1F9A0"},
    {"борсук", "1F9A1"},
    {"лебідь", "1F9A2"},
    {"мамонт", "1F9A3"},
    {"дронт", "1F9A4"},
    {"лінивець", "1F9A5"},
    {"видра", "1F9A6"},
    {"орангутанг", "1F9A7"},
    {"скунс", "1F9A8"},
    {"фламінго", "1F9A9"},
    {"устриця", "1F9AA"},
    {"бобер", "1F9AB"},
    {"бізон", "1F9AC"},
    {"тюлень", "1F9AD"},
    {"собака_поводир", "1F9AE"},
    {"тактильна_тростина", "1F9AF"},
    {"руде_волосся", "1F9B0"},
    {"кучеряве_волосся", "1F9B1"},
    {"лисина", "1F9B2"},
    {"сиве_волосся", "1F9B3"},
    {"кістка", "1F9B4"},
    {"нога", "1F9B5"},
    {"нога_колір1", "1F9B5-1F3FB"},
    {"нога_колір2", "1F9B5-1F3FC"},
    {"нога_колір3", "1F9B5-1F3FD"},
    {"нога_колір4", "1F9B5-1F3FE"},
    {"нога_колір5", "1F9B5-1F3FF"},
    {"ступня", "1F9B6"},
    {"ступня_колір1", "1F9B6-1F3FB"},
    {"ступня_колір2", "1F9B6-1F3FC"},
    {"ступня_колір3", "1F9B6-1F3FD"},
    {"ступня_колір4", "1F9B6-1F3FE"},
    {"ступня_колір5", "1F9B6-1F3FF"},
    {"зуб", "1F9B7"},
    {"супергерой", "1F9B8"},
    {"супергерой_колір1", "1F9B8-1F3FB"},
    {"супергерой_колір2", "1F9B8-1F3FC"},
    {"супергерой_колір3", "1F9B8-1F3FD"},
    {"супергерой_колір4", "1F9B8-1F3FE"},
    {"супергерой_колір5", "1F9B8-1F3FF"},
    {"суперзлодій", "1F9B9"},
    {"суперзлодій_колір1", "1F9B9-1F3FB"},
    {"суперзлодій_колір2", "1F9B9-1F3FC"},
    {"суперзлодій_колір3", "1F9B9-1F3FD"},
    {"суперзлодій_колір4", "1F9B9-1F3FE"},
    {"суперзлодій_колір5", "1F9B9-1F3FF"},
    {"сигнальний_жилет", "1F9BA"},
    {"вухо_зі_слуховим_апаратом", "1F9BB"},
    {"вухо_зі_слуховим_апаратом_колір1", "1F9BB-1F3FB"},
    {"вухо_зі_слуховим_апаратом_колір2", "1F9BB-1F3FC"},
    {"вухо_зі_слуховим_апаратом_колір3", "1F9BB-1F3FD"},
    {"вухо_зі_слуховим_апаратом_колір4", "1F9BB-1F3FE"},
    {"вухо_зі_слуховим_апаратом_колір5", "1F9BB-1F3FF"},
    {"колісний_візок_з_електроприводом", "1F9BC"},
    {"колісний_візок_із_ручним_приводом", "1F9BD"},
    {"механічна_рука", "1F9BE"},
    {"механічна_нога", "1F9BF"},
    {"кусень_сиру", "1F9C0"},
    {"капкейк", "1F9C1"},
    {"сіль", "1F9C2"},
    {"пакетик_із_напоєм", "1F9C3"},
    {"часник", "1F9C4"},
    {"цибуля", "1F9C5"},
    {"фалафель", "1F9C6"},
    {"вафля", "1F9C7"},
    {"масло", "1F9C8"},
    {"мате", "1F9C9"},
    {"кубик_льоду", "1F9CA"},
    {"бульбашковий_чай", "1F9CB"},
    {"троль", "1F9CC"},
    {"людина_що_стоїть", "1F9CD"},
    {"людина_що_стоїть_колір1", "1F9CD-1F3FB"},
    {"людина_що_стоїть_колір2", "1F9CD-1F3FC"},
    {"людина_що_стоїть_колір3", "1F9CD-1F3FD"},
    {"людина_що_стоїть_колір4", "1F9CD-1F3FE"},
    {"людина_що_стоїть_колір5", "1F9CD-1F3FF"},
    {"людина_навколішках", "1F9CE"},
    {"людина_навколішках_колір1", "1F9CE-1F3FB"},
    {"людина_навколішках_колір2", "1F9CE-1F3FC"},
    {"людина_навколішках_колір3", "1F9CE-1F3FD"},
    {"людина_навколішках_колір4", "1F9CE-1F3FE"},
    {"людина_навколішках_колір5", "1F9CE-1F3FF"},
    {"людина_з_порушенням_слуху", "1F9CF"},
    {"людина_з_порушенням_слуху_колір1", "1F9CF-1F3FB"},
    {"людина_з_порушенням_слуху_колір2", "1F9CF-1F3FC"},
    {"людина_з_порушенням_слуху_колір3", "1F9CF-1F3FD"},
    {"людина_з_порушенням_слуху_колір4", "1F9CF-1F3FE"},
    {"людина_з_порушенням_слуху_колір5", "1F9CF-1F3FF"},
    {"обличчя_в_моноклі", "1F9D0"},
    {"людина", "1F9D1"},
    {"людина_колір1", "1F9D1-1F3FB"},
    {"людина_колір2", "1F9D1-1F3FC"},
    {"людина_колір3", "1F9D1-1F3FD"},
    {"людина_колір4", "1F9D1-1F3FE"},
    {"людина_колір5", "1F9D1-1F3FF"},
    {"дитина", "1F9D2"},
    {"дитина_колір1", "1F9D2-1F3FB"},
    {"дитина_колір2", "1F9D2-1F3FC"},
    {"дитина_колір3", "1F9D2-1F3FD"},
    {"дитина_колір4", "1F9D2-1F3FE"},
    {"дитина_колір5", "1F9D2-1F3FF"},
    {"літня_людина", "1F9D3"},
    {"літня_людина_колір1", "1F9D3-1F3FB"},
    {"літня_людина_колір2", "1F9D3-1F3FC"},
    {"літня_людина_колір3", "1F9D3-1F3FD"},
    {"літня_людина_колір4", "1F9D3-1F3FE"},
    {"літня_людина_колір5", "1F9D3-1F3FF"},
    {"людина_з_бородою", "1F9D4"},
    {"людина_з_бородою_колір1", "1F9D4-1F3FB"},
    {"людина_з_бородою_колір2", "1F9D4-1F3FC"},
    {"людина_з_бородою_колір3", "1F9D4-1F3FD"},
    {"людина_з_бородою_колір4", "1F9D4-1F3FE"},
    {"людина_з_бородою_колір5", "1F9D4-1F3FF"},
    {"жінка_в_хустині", "1F9D5"},
    {"жінка_в_хустині_колір1", "1F9D5-1F3FB"},
    {"жінка_в_хустині_колір2", "1F9D5-1F3FC"},
    {"жінка_в_хустині_колір3", "1F9D5-1F3FD"},
    {"жінка_в_хустині_колір4", "1F9D5-1F3FE"},
    {"жінка_в_хустині_колір5", "1F9D5-1F3FF"},
    {"людина_в_паркому_приміщенні", "1F9D6"},
    {"людина_в_паркому_приміщенні_колір1", "1F9D6-1F3FB"},
    {"людина_в_паркому_приміщенні_колір2", "1F9D6-1F3FC"},
    {"людина_в_паркому_приміщенні_колір3", "1F9D6-1F3FD"},
    {"людина_в_паркому_приміщенні_колір4", "1F9D6-1F3FE"},
    {"людина_в_паркому_приміщенні_колір5", "1F9D6-1F3FF"},
    {"людина_що_лізе_скелею", "1F9D7"},
    {"людина_що_лізе_скелею_колір1", "1F9D7-1F3FB"},
    {"людина_що_лізе_скелею_колір2", "1F9D7-1F3FC"},
    {"людина_що_лізе_скелею_колір3", "1F9D7-1F3FD"},
    {"людина_що_лізе_скелею_колір4", "1F9D7-1F3FE"},
    {"людина_що_лізе_скелею_колір5", "1F9D7-1F3FF"},
    {"людина_в_позі_лотоса", "1F9D8"},
    {"людина_в_позі_лотоса_колір1", "1F9D8-1F3FB"},
    {"людина_в_позі_лотоса_колір2", "1F9D8-1F3FC"},
    {"людина_в_позі_лотоса_колір3", "1F9D8-1F3FD"},
    {"людина_в_позі_лотоса_колір4", "1F9D8-1F3FE"},
    {"людина_в_позі_лотоса_колір5", "1F9D8-1F3FF"},
    {"маг", "1F9D9"},
    {"маг_колір1", "1F9D9-1F3FB"},
    {"маг_колір2", "1F9D9-1F3FC"},
    {"маг_колір3", "1F9D9-1F3FD"},
    {"маг_колір4", "1F9D9-1F3FE"},
    {"маг_колір5", "1F9D9-1F3FF"},
    {"фея", "1F9DA"},
    {"фея_колір1", "1F9DA-1F3FB"},
    {"фея_колір2", "1F9DA-1F3FC"},
    {"фея_колір3", "1F9DA-1F3FD"},
    {"фея_колір4", "1F9DA-1F3FE"},
    {"фея_колір5", "1F9DA-1F3FF"},
    {"вампір", "1F9DB"},
    {"вампір_колір1", "1F9DB-1F3FB"},
    {"вампір_колір2", "1F9DB-1F3FC"},
    {"вампір_колір3", "1F9DB-1F3FD"},
    {"вампір_колір4", "1F9DB-1F3FE"},
    {"вампір_колір5", "1F9DB-1F3FF"},
    {"казкова_водяна_істота", "1F9DC"},
    {"казкова_водяна_істота_колір1", "1F9DC-1F3FB"},
    {"казкова_водяна_істота_колір2", "1F9DC-1F3FC"},
    {"казкова_водяна_істота_колір3", "1F9DC-1F3FD"},
    {"казкова_водяна_істота_колір4", "1F9DC-1F3FE"},
    {"казкова_водяна_істота_колір5", "1F9DC-1F3FF"},
    {"ельф", "1F9DD"},
    {"ельф_колір1", "1F9DD-1F3FB"},
    {"ельф_колір2", "1F9DD-1F3FC"},
    {"ельф_колір3", "1F9DD-1F3FD"},
    {"ельф_колір4", "1F9DD-1F3FE"},
    {"ельф_колір5", "1F9DD-1F3FF"},
    {"джин", "1F9DE"},
    {"зомбі", "1F9DF"},
    {"мозок", "1F9E0"},
    {"оранжеве_серце", "1F9E1"},
    {"бейсболка", "1F9E2"},
    {"шарф", "1F9E3"},
    {"рукавички", "1F9E4"},
    {"пальто", "1F9E5"},
    {"шкарпетки", "1F9E6"},
    {"червоний_конверт", "1F9E7"},
    {"петарда", "1F9E8"},
    {"елемент_пазла", "1F9E9"},
    {"пробірка", "1F9EA"},
    {"чашка_петрі", "1F9EB"},
    {"днк", "1F9EC"},
    {"компас", "1F9ED"},
    {"рахівниця", "1F9EE"},
    {"вогнегасник", "1F9EF"},
    {"скриня_для_інструментів", "1F9F0"},
    {"цегла", "1F9F1"},
    {"магніт", "1F9F2"},
    {"багаж", "1F9F3"},
    {"лосьйон", "1F9F4"},
    {"нитка", "1F9F5"},
    {"пряжа", "1F9F6"},
    {"англійська_шпилька", "1F9F7"},
    {"ведмедик", "1F9F8"},
    {"мітла", "1F9F9"},
    {"кошик", "1F9FA"},
    {"рулон_паперу", "1F9FB"},
    {"мило", "1F9FC"},
    {"губка", "1F9FD"},
    {"квитанція", "1F9FE"},
    {"око_фатіми", "1F9FF"},
    {"балетні_туфлі", "1FA70"},
    {"суцільний_купальник", "1FA71"},
    {"плавки", "1FA72"},
    {"шорти", "1FA73"},
    {"вєтнамки", "1FA74"},
    {"блакитне_серце", "1FA75"},
    {"сіре_серце", "1FA76"},
    {"рожеве_серце", "1FA77"},
    {"крапля_крові", "1FA78"},
    {"пластир", "1FA79"},
    {"стетоскоп", "1FA7A"},
    {"рентген", "1FA7B"},
    {"милиця", "1FA7C"},
    {"йо_йо", "1FA80"},
    {"повітряний_змій", "1FA81"},
    {"парашут", "1FA82"},
    {"бумеранг", "1FA83"},
    {"чарівна_паличка", "1FA84"},
    {"піньята", "1FA85"},
    {"ляльки_матрьошки", "1FA86"},
    {"маракас", "1FA87"},
    {"флейта", "1FA88"},
    {"арфа", "1FA89"},
    {"тромбон", "1FA8A"},
    {"скриня_скарбів", "1FA8E"},
    {"лопата", "1FA8F"},
    {"планета_з_кільцем", "1FA90"},
    {"стілець", "1FA91"},
    {"бритва", "1FA92"},
    {"сокира", "1FA93"},
    {"каганець", "1FA94"},
    {"банджо", "1FA95"},
    {"військовий_шолом", "1FA96"},
    {"акордеон", "1FA97"},
    {"циліндричний_барабан", "1FA98"},
    {"монета", "1FA99"},
    {"теслярська_пила", "1FA9A"},
    {"викрутка", "1FA9B"},
    {"драбина", "1FA9C"},
    {"гачок", "1FA9D"},
    {"дзеркало", "1FA9E"},
    {"вікно", "1FA9F"},
    {"вантуз", "1FAA0"},
    {"швацька_голка", "1FAA1"},
    {"вузол", "1FAA2"},
    {"відро", "1FAA3"},
    {"мишоловка", "1FAA4"},
    {"зубна_щітка", "1FAA5"},
    {"надгробок", "1FAA6"},
    {"плакат", "1FAA7"},
    {"камінь", "1FAA8"},
    {"дзеркальна_куля", "1FAA9"},
    {"посвідчення", "1FAAA"},
    {"низький_заряд_батареї", "1FAAB"},
    {"хамса", "1FAAC"},
    {"складане_віяло", "1FAAD"},
    {"гребінь", "1FAAE"},
    {"кханда", "1FAAF"},
    {"муха", "1FAB0"},
    {"хробак", "1FAB1"},
    {"жук", "1FAB2"},
    {"тарган", "1FAB3"},
    {"рослина_в_горщику", "1FAB4"},
    {"деревина", "1FAB5"},
    {"пірїна", "1FAB6"},
    {"лотос", "1FAB7"},
    {"корал", "1FAB8"},
    {"порожнє_гніздо", "1FAB9"},
    {"гніздо_з_пташиними_яйцями", "1FABA"},
    {"гіацинт", "1FABB"},
    {"медуза", "1FABC"},
    {"крило", "1FABD"},
    {"сухе_дерево", "1FABE"},
    {"гусак", "1FABF"},
    {"серце", "1FAC0"},
    {"легені", "1FAC1"},
    {"люди_обіймаються", "1FAC2"},
    {"чоловік_із_вагітним_животом", "1FAC3"},
    {"чоловік_із_вагітним_животом_колір1", "1FAC3-1F3FB"},
    {"чоловік_із_вагітним_животом_колір2", "1FAC3-1F3FC"},
    {"чоловік_із_вагітним_животом_колір3", "1FAC3-1F3FD"},
    {"чоловік_із_вагітним_животом_колір4", "1FAC3-1F3FE"},
    {"чоловік_із_вагітним_животом_колір5", "1FAC3-1F3FF"},
    {"вагітна_людина", "1FAC4"},
    {"вагітна_людина_колір1", "1FAC4-1F3FB"},
    {"вагітна_людина_колір2", "1FAC4-1F3FC"},
    {"вагітна_людина_колір3", "1FAC4-1F3FD"},
    {"вагітна_людина_колір4", "1FAC4-1F3FE"},
    {"вагітна_людина_колір5", "1FAC4-1F3FF"},
    {"людина_з_короною_на_голові", "1FAC5"},
    {"людина_з_короною_на_голові_колір1", "1FAC5-1F3FB"},
    {"людина_з_короною_на_голові_колір2", "1FAC5-1F3FC"},
    {"людина_з_короною_на_голові_колір3", "1FAC5-1F3FD"},
    {"людина_з_короною_на_голові_колір4", "1FAC5-1F3FE"},
    {"людина_з_короною_на_голові_колір5", "1FAC5-1F3FF"},
    {"відбиток_пальця", "1FAC6"},
    {"волохата_істота", "1FAC8"},
    {"косатка", "1FACD"},
    {"лось", "1FACE"},
    {"віслюк", "1FACF"},
    {"лохина", "1FAD0"},
    {"солодкий_перець", "1FAD1"},
    {"оливка", "1FAD2"},
    {"корж", "1FAD3"},
    {"тамале", "1FAD4"},
    {"фондю", "1FAD5"},
    {"чайник", "1FAD6"},
    {"склянка_з_якої_виливається_рідина", "1FAD7"},
    {"квасоля", "1FAD8"},
    {"банка", "1FAD9"},
    {"імбир", "1FADA"},
    {"стручок_гороху", "1FADB"},
    {"коренеплід", "1FADC"},
    {"ляпка", "1FADF"},
    {"розплавлене_обличчя", "1FAE0"},
    {"обличчя_що_козиряє", "1FAE1"},
    {"обличчя_з_розплющеними_очима_і_рукою_на_роті", "1FAE2"},
    {"обличчя_що_підглядає_одним_оком", "1FAE3"},
    {"обличчя_з_ротом_навскіс", "1FAE4"},
    {"обличчя_пунктиром", "1FAE5"},
    {"закушена_нижня_губа", "1FAE6"},
    {"бульбашки", "1FAE7"},
    {"тремтливе_обличчя", "1FAE8"},
    {"невиспане_обличчя", "1FAE9"},
    {"спотворене_обличчя", "1FAEA"},
    {"бійка_в_хмарі", "1FAEF"},
    {"клацання_пальцями", "1FAF0"},
    {"клацання_пальцями_колір1", "1FAF0-1F3FB"},
    {"клацання_пальцями_колір2", "1FAF0-1F3FC"},
    {"клацання_пальцями_колір3", "1FAF0-1F3FD"},
    {"клацання_пальцями_колір4", "1FAF0-1F3FE"},
    {"клацання_пальцями_колір5", "1FAF0-1F3FF"},
    {"долоня_вправо", "1FAF1"},
    {"долоня_вправо_колір1", "1FAF1-1F3FB"},
    {"долоня_вправо_колір2", "1FAF1-1F3FC"},
    {"долоня_вправо_колір3", "1FAF1-1F3FD"},
    {"долоня_вправо_колір4", "1FAF1-1F3FE"},
    {"долоня_вправо_колір5", "1FAF1-1F3FF"},
    {"долоня_вліво", "1FAF2"},
    {"долоня_вліво_колір1", "1FAF2-1F3FB"},
    {"долоня_вліво_колір2", "1FAF2-1F3FC"},
    {"долоня_вліво_колір3", "1FAF2-1F3FD"},
    {"долоня_вліво_колір4", "1FAF2-1F3FE"},
    {"долоня_вліво_колір5", "1FAF2-1F3FF"},
    {"рука_долонею_вниз", "1FAF3"},
    {"рука_долонею_вниз_колір1", "1FAF3-1F3FB"},
    {"рука_долонею_вниз_колір2", "1FAF3-1F3FC"},
    {"рука_долонею_вниз_колір3", "1FAF3-1F3FD"},
    {"рука_долонею_вниз_колір4", "1FAF3-1F3FE"},
    {"рука_долонею_вниз_колір5", "1FAF3-1F3FF"},
    {"рука_долонею_вгору", "1FAF4"},
    {"рука_долонею_вгору_колір1", "1FAF4-1F3FB"},
    {"рука_долонею_вгору_колір2", "1FAF4-1F3FC"},
    {"рука_долонею_вгору_колір3", "1FAF4-1F3FD"},
    {"рука_долонею_вгору_колір4", "1FAF4-1F3FE"},
    {"рука_долонею_вгору_колір5", "1FAF4-1F3FF"},
    {"вказівний_палець_що_вказує_на_глядача", "1FAF5"},
    {"вказівний_палець_що_вказує_на_глядача_колір1", "1FAF5-1F3FB"},
    {"вказівний_палець_що_вказує_на_глядача_колір2", "1FAF5-1F3FC"},
    {"вказівний_палець_що_вказує_на_глядача_колір3", "1FAF5-1F3FD"},
    {"вказівний_палець_що_вказує_на_глядача_колір4", "1FAF5-1F3FE"},
    {"вказівний_палець_що_вказує_на_глядача_колір5", "1FAF5-1F3FF"},
    {"жест_серце", "1FAF6"},
    {"жест_серце_колір1", "1FAF6-1F3FB"},
    {"жест_серце_колір2", "1FAF6-1F3FC"},
    {"жест_серце_колір3", "1FAF6-1F3FD"},
    {"жест_серце_колір4", "1FAF6-1F3FE"},
    {"жест_серце_колір5", "1FAF6-1F3FF"},
    {"долоня_штовхає_вліво", "1FAF7"},
    {"долоня_штовхає_вліво_колір1", "1FAF7-1F3FB"},
    {"долоня_штовхає_вліво_колір2", "1FAF7-1F3FC"},
    {"долоня_штовхає_вліво_колір3", "1FAF7-1F3FD"},
    {"долоня_штовхає_вліво_колір4", "1FAF7-1F3FE"},
    {"долоня_штовхає_вліво_колір5", "1FAF7-1F3FF"},
    {"долоня_штовхає_вправо", "1FAF8"},
    {"долоня_штовхає_вправо_колір1", "1FAF8-1F3FB"},
    {"долоня_штовхає_вправо_колір2", "1FAF8-1F3FC"},
    {"долоня_штовхає_вправо_колір3", "1FAF8-1F3FD"},
    {"долоня_штовхає_вправо_колір4", "1FAF8-1F3FE"},
    {"долоня_штовхає_вправо_колір5", "1FAF8-1F3FF"},
    {"клавіша_знак_числа", "0023-FE0F-20E3"},
    {"клавіша_зірочка", "002A-FE0F-20E3"},
    {"клавіша_0", "0030-FE0F-20E3"},
    {"клавіша_1", "0031-FE0F-20E3"},
    {"клавіша_2", "0032-FE0F-20E3"},
    {"клавіша_3", "0033-FE0F-20E3"},
    {"клавіша_4", "0034-FE0F-20E3"},
    {"клавіша_5", "0035-FE0F-20E3"},
    {"клавіша_6", "0036-FE0F-20E3"},
    {"клавіша_7", "0037-FE0F-20E3"},
    {"клавіша_8", "0038-FE0F-20E3"},
    {"клавіша_9", "0039-FE0F-20E3"},
    {"прапор_острів_вознесіння", "1F1E6-1F1E8"},
    {"прапор_андорра", "1F1E6-1F1E9"},
    {"прапор_обєднані_арабські_емірати", "1F1E6-1F1EA"},
    {"прапор_афганістан", "1F1E6-1F1EB"},
    {"прапор_антигуа_і_барбуда", "1F1E6-1F1EC"},
    {"прапор_ангілья", "1F1E6-1F1EE"},
    {"прапор_албанія", "1F1E6-1F1F1"},
    {"прапор_вірменія", "1F1E6-1F1F2"},
    {"прапор_ангола", "1F1E6-1F1F4"},
    {"прапор_антарктика", "1F1E6-1F1F6"},
    {"прапор_аргентина", "1F1E6-1F1F7"},
    {"прапор_американське_самоа", "1F1E6-1F1F8"},
    {"прапор_австрія", "1F1E6-1F1F9"},
    {"прапор_австралія", "1F1E6-1F1FA"},
    {"прапор_аруба", "1F1E6-1F1FC"},
    {"прапор_аландські_острови", "1F1E6-1F1FD"},
    {"прапор_азербайджан", "1F1E6-1F1FF"},
    {"прапор_боснія_і_герцеговина", "1F1E7-1F1E6"},
    {"прапор_барбадос", "1F1E7-1F1E7"},
    {"прапор_бангладеш", "1F1E7-1F1E9"},
    {"прапор_бельгія", "1F1E7-1F1EA"},
    {"прапор_буркіна_фасо", "1F1E7-1F1EB"},
    {"прапор_болгарія", "1F1E7-1F1EC"},
    {"прапор_бахрейн", "1F1E7-1F1ED"},
    {"прапор_бурунді", "1F1E7-1F1EE"},
    {"прапор_бенін", "1F1E7-1F1EF"},
    {"прапор_сен_бартелемі", "1F1E7-1F1F1"},
    {"прапор_бермудські_острови", "1F1E7-1F1F2"},
    {"прапор_бруней", "1F1E7-1F1F3"},
    {"прапор_болівія", "1F1E7-1F1F4"},
    {"прапор_карибські_нідерланди", "1F1E7-1F1F6"},
    {"прапор_бразилія", "1F1E7-1F1F7"},
    {"прапор_багамські_острови", "1F1E7-1F1F8"},
    {"прапор_бутан", "1F1E7-1F1F9"},
    {"прапор_острів_буве", "1F1E7-1F1FB"},
    {"прапор_ботсвана", "1F1E7-1F1FC"},
    {"прапор_білорусь", "1F1E7-1F1FE"},
    {"прапор_беліз", "1F1E7-1F1FF"},
    {"прапор_канада", "1F1E8-1F1E6"},
    {"прапор_кокосові_кілінг_острови", "1F1E8-1F1E8"},
    {"прапор_конго_кіншаса", "1F1E8-1F1E9"},
    {"прапор_центральноафриканська_республіка", "1F1E8-1F1EB"},
    {"прапор_конго_браззавіль", "1F1E8-1F1EC"},
    {"прапор_швейцарія", "1F1E8-1F1ED"},
    {"прапор_кот_дівуар", "1F1E8-1F1EE"},
    {"прапор_острови_кука", "1F1E8-1F1F0"},
    {"прапор_чилі", "1F1E8-1F1F1"},
    {"прапор_камерун", "1F1E8-1F1F2"},
    {"прапор_китай", "1F1E8-1F1F3"},
    {"прапор_колумбія", "1F1E8-1F1F4"},
    {"прапор_острів_кліппертон", "1F1E8-1F1F5"},
    {"прапор_сарк", "1F1E8-1F1F6"},
    {"прапор_коста_рика", "1F1E8-1F1F7"},
    {"прапор_куба", "1F1E8-1F1FA"},
    {"прапор_кабо_верде", "1F1E8-1F1FB"},
    {"прапор_кюрасао", "1F1E8-1F1FC"},
    {"прапор_острів_різдва", "1F1E8-1F1FD"},
    {"прапор_кіпр", "1F1E8-1F1FE"},
    {"прапор_чехія", "1F1E8-1F1FF"},
    {"прапор_німеччина", "1F1E9-1F1EA"},
    {"прапор_дієго_гарсія", "1F1E9-1F1EC"},
    {"прапор_джибуті", "1F1E9-1F1EF"},
    {"прапор_данія", "1F1E9-1F1F0"},
    {"прапор_домініка", "1F1E9-1F1F2"},
    {"прапор_домініканська_республіка", "1F1E9-1F1F4"},
    {"прапор_алжир", "1F1E9-1F1FF"},
    {"прапор_сеута_і_мелілья", "1F1EA-1F1E6"},
    {"прапор_еквадор", "1F1EA-1F1E8"},
    {"прапор_естонія", "1F1EA-1F1EA"},
    {"прапор_єгипет", "1F1EA-1F1EC"},
    {"прапор_західна_сахара", "1F1EA-1F1ED"},
    {"прапор_еритрея", "1F1EA-1F1F7"},
    {"прапор_іспанія", "1F1EA-1F1F8"},
    {"прапор_ефіопія", "1F1EA-1F1F9"},
    {"прапор_європейський_союз", "1F1EA-1F1FA"},
    {"прапор_фінляндія", "1F1EB-1F1EE"},
    {"прапор_фіджі", "1F1EB-1F1EF"},
    {"прапор_фолклендські_острови", "1F1EB-1F1F0"},
    {"прапор_мікронезія", "1F1EB-1F1F2"},
    {"прапор_фарерські_острови", "1F1EB-1F1F4"},
    {"прапор_франція", "1F1EB-1F1F7"},
    {"прапор_габон", "1F1EC-1F1E6"},
    {"прапор_велика_британія", "1F1EC-1F1E7"},
    {"прапор_гренада", "1F1EC-1F1E9"},
    {"прапор_грузія", "1F1EC-1F1EA"},
    {"прапор_французька_гвіана", "1F1EC-1F1EB"},
    {"прапор_гернсі", "1F1EC-1F1EC"},
    {"прапор_гана", "1F1EC-1F1ED"},
    {"прапор_гібралтар", "1F1EC-1F1EE"},
    {"прапор_гренландія", "1F1EC-1F1F1"},
    {"прапор_гамбія", "1F1EC-1F1F2"},
    {"прапор_гвінея", "1F1EC-1F1F3"},
    {"прапор_гваделупа", "1F1EC-1F1F5"},
    {"прапор_екваторіальна_гвінея", "1F1EC-1F1F6"},
    {"прапор_греція", "1F1EC-1F1F7"},
    {"прапор_південна_джорджія_та_південні_сандвічеві_острови", "1F1EC-1F1F8"},
    {"прапор_гватемала", "1F1EC-1F1F9"},
    {"прапор_гуам", "1F1EC-1F1FA"},
    {"прапор_гвінея_бісау", "1F1EC-1F1FC"},
    {"прапор_гаяна", "1F1EC-1F1FE"},
    {"прапор_гонконг_оар_китаю", "1F1ED-1F1F0"},
    {"прапор_острови_герд_і_макдоналд", "1F1ED-1F1F2"},
    {"прапор_гондурас", "1F1ED-1F1F3"},
    {"прапор_хорватія", "1F1ED-1F1F7"},
    {"прапор_гаїті", "1F1ED-1F1F9"},
    {"прапор_угорщина", "1F1ED-1F1FA"},
    {"прапор_канарські_острови", "1F1EE-1F1E8"},
    {"прапор_індонезія", "1F1EE-1F1E9"},
    {"прапор_ірландія", "1F1EE-1F1EA"},
    {"прапор_ізраїль", "1F1EE-1F1F1"},
    {"прапор_острів_мен", "1F1EE-1F1F2"},
    {"прапор_індія", "1F1EE-1F1F3"},
    {"прапор_британська_територія_в_індійському_океані", "1F1EE-1F1F4"},
    {"прапор_ірак", "1F1EE-1F1F6"},
    {"прапор_іран", "1F1EE-1F1F7"},
    {"прапор_ісландія", "1F1EE-1F1F8"},
    {"прапор_італія", "1F1EE-1F1F9"},
    {"прапор_джерсі", "1F1EF-1F1EA"},
    {"прапор_ямайка", "1F1EF-1F1F2"},
    {"прапор_йорданія", "1F1EF-1F1F4"},
    {"прапор_японія", "1F1EF-1F1F5"},
    {"прапор_кенія", "1F1F0-1F1EA"},
    {"прапор_киргизстан", "1F1F0-1F1EC"},
    {"прапор_камбоджа", "1F1F0-1F1ED"},
    {"прапор_кірибаті", "1F1F0-1F1EE"},
    {"прапор_комори", "1F1F0-1F1F2"},
    {"прапор_сент_кітс_і_невіс", "1F1F0-1F1F3"},
    {"прапор_північна_корея", "1F1F0-1F1F5"},
    {"прапор_південна_корея", "1F1F0-1F1F7"},
    {"прапор_кувейт", "1F1F0-1F1FC"},
    {"прапор_кайманові_острови", "1F1F0-1F1FE"},
    {"прапор_казахстан", "1F1F0-1F1FF"},
    {"прапор_лаос", "1F1F1-1F1E6"},
    {"прапор_ліван", "1F1F1-1F1E7"},
    {"прапор_сент_люсія", "1F1F1-1F1E8"},
    {"прапор_ліхтенштейн", "1F1F1-1F1EE"},
    {"прапор_шрі_ланка", "1F1F1-1F1F0"},
    {"прапор_ліберія", "1F1F1-1F1F7"},
    {"прапор_лесото", "1F1F1-1F1F8"},
    {"прапор_литва", "1F1F1-1F1F9"},
    {"прапор_люксембург", "1F1F1-1F1FA"},
    {"прапор_латвія", "1F1F1-1F1FB"},
    {"прапор_лівія", "1F1F1-1F1FE"},
    {"прапор_марокко", "1F1F2-1F1E6"},
    {"прапор_монако", "1F1F2-1F1E8"},
    {"прапор_молдова", "1F1F2-1F1E9"},
    {"прапор_чорногорія", "1F1F2-1F1EA"},
    {"прапор_сен_мартен", "1F1F2-1F1EB"},
    {"прапор_мадагаскар", "1F1F2-1F1EC"},
    {"прапор_маршаллові_острови", "1F1F2-1F1ED"},
    {"прапор_північна_македонія", "1F1F2-1F1F0"},
    {"прапор_малі", "1F1F2-1F1F1"},
    {"прапор_мянма_бірма", "1F1F2-1F1F2"},
    {"прапор_монголія", "1F1F2-1F1F3"},
    {"прапор_макао_оар_китаю", "1F1F2-1F1F4"},
    {"прапор_північні_маріанські_острови", "1F1F2-1F1F5"},
    {"прапор_мартиніка", "1F1F2-1F1F6"},
    {"прапор_мавританія", "1F1F2-1F1F7"},
    {"прапор_монтсеррат", "1F1F2-1F1F8"},
    {"прапор_мальта", "1F1F2-1F1F9"},
    {"прапор_маврикій", "1F1F2-1F1FA"},
    {"прапор_мальдіви", "1F1F2-1F1FB"},
    {"прапор_малаві", "1F1F2-1F1FC"},
    {"прапор_мексика", "1F1F2-1F1FD"},
    {"прапор_малайзія", "1F1F2-1F1FE"},
    {"прапор_мозамбік", "1F1F2-1F1FF"},
    {"прапор_намібія", "1F1F3-1F1E6"},
    {"прапор_нова_каледонія", "1F1F3-1F1E8"},
    {"прапор_нігер", "1F1F3-1F1EA"},
    {"прапор_острів_норфолк", "1F1F3-1F1EB"},
    {"прапор_нігерія", "1F1F3-1F1EC"},
    {"прапор_нікарагуа", "1F1F3-1F1EE"},
    {"прапор_нідерланди", "1F1F3-1F1F1"},
    {"прапор_норвегія", "1F1F3-1F1F4"},
    {"прапор_непал", "1F1F3-1F1F5"},
    {"прапор_науру", "1F1F3-1F1F7"},
    {"прапор_ніуе", "1F1F3-1F1FA"},
    {"прапор_нова_зеландія", "1F1F3-1F1FF"},
    {"прапор_оман", "1F1F4-1F1F2"},
    {"прапор_панама", "1F1F5-1F1E6"},
    {"прапор_перу", "1F1F5-1F1EA"},
    {"прапор_французька_полінезія", "1F1F5-1F1EB"},
    {"прапор_папуа_нова_гвінея", "1F1F5-1F1EC"},
    {"прапор_філіппіни", "1F1F5-1F1ED"},
    {"прапор_пакистан", "1F1F5-1F1F0"},
    {"прапор_польща", "1F1F5-1F1F1"},
    {"прапор_сен_пєр_і_мікелон", "1F1F5-1F1F2"},
    {"прапор_острови_піткерн", "1F1F5-1F1F3"},
    {"прапор_пуерто_рико", "1F1F5-1F1F7"},
    {"прапор_палестинські_території", "1F1F5-1F1F8"},
    {"прапор_португалія", "1F1F5-1F1F9"},
    {"прапор_палау", "1F1F5-1F1FC"},
    {"прапор_парагвай", "1F1F5-1F1FE"},
    {"прапор_катар", "1F1F6-1F1E6"},
    {"прапор_реюньйон", "1F1F7-1F1EA"},
    {"прапор_румунія", "1F1F7-1F1F4"},
    {"прапор_сербія", "1F1F7-1F1F8"},
    {"прапор_росія", "1F1F7-1F1FA"},
    {"прапор_руанда", "1F1F7-1F1FC"},
    {"прапор_саудівська_аравія", "1F1F8-1F1E6"},
    {"прапор_соломонові_острови", "1F1F8-1F1E7"},
    {"прапор_сейшельські_острови", "1F1F8-1F1E8"},
    {"прапор_судан", "1F1F8-1F1E9"},
    {"прапор_швеція", "1F1F8-1F1EA"},
    {"прапор_сінгапур", "1F1F8-1F1EC"},
    {"прапор_острів_святої_єлени", "1F1F8-1F1ED"},
    {"прапор_словенія", "1F1F8-1F1EE"},
    {"прапор_шпіцберген_та_ян_маєн", "1F1F8-1F1EF"},
    {"прапор_словаччина", "1F1F8-1F1F0"},
    {"прапор_сьєрра_леоне", "1F1F8-1F1F1"},
    {"прапор_сан_марино", "1F1F8-1F1F2"},
    {"прапор_сенегал", "1F1F8-1F1F3"},
    {"прапор_сомалі", "1F1F8-1F1F4"},
    {"прапор_суринам", "1F1F8-1F1F7"},
    {"прапор_південний_судан", "1F1F8-1F1F8"},
    {"прапор_сан_томе_і_принсіпі", "1F1F8-1F1F9"},
    {"прапор_сальвадор", "1F1F8-1F1FB"},
    {"прапор_сінт_мартен", "1F1F8-1F1FD"},
    {"прапор_сирія", "1F1F8-1F1FE"},
    {"прапор_есватіні", "1F1F8-1F1FF"},
    {"прапор_трістан_да_кунья", "1F1F9-1F1E6"},
    {"прапор_острови_теркс_і_кайкос", "1F1F9-1F1E8"},
    {"прапор_чад", "1F1F9-1F1E9"},
    {"прапор_французькі_південні_території", "1F1F9-1F1EB"},
    {"прапор_того", "1F1F9-1F1EC"},
    {"прапор_таїланд", "1F1F9-1F1ED"},
    {"прапор_таджикистан", "1F1F9-1F1EF"},
    {"прапор_токелау", "1F1F9-1F1F0"},
    {"прапор_тимор_лешті", "1F1F9-1F1F1"},
    {"прапор_туркменістан", "1F1F9-1F1F2"},
    {"прапор_туніс", "1F1F9-1F1F3"},
    {"прапор_тонга", "1F1F9-1F1F4"},
    {"прапор_туреччина", "1F1F9-1F1F7"},
    {"прапор_тринідад_і_тобаго", "1F1F9-1F1F9"},
    {"прапор_тувалу", "1F1F9-1F1FB"},
    {"прапор_тайвань", "1F1F9-1F1FC"},
    {"прапор_танзанія", "1F1F9-1F1FF"},
    {"прапор_україна", "1F1FA-1F1E6"},
    {"прапор_уганда", "1F1FA-1F1EC"},
    {"прапор_віддалені_острови_сша", "1F1FA-1F1F2"},
    {"прапор_організація_обєднаних_націй", "1F1FA-1F1F3"},
    {"прапор_сполучені_штати", "1F1FA-1F1F8"},
    {"прапор_уругвай", "1F1FA-1F1FE"},
    {"прапор_узбекистан", "1F1FA-1F1FF"},
    {"прапор_ватикан", "1F1FB-1F1E6"},
    {"прапор_сент_вінсент_і_гренадіни", "1F1FB-1F1E8"},
    {"прапор_венесуела", "1F1FB-1F1EA"},
    {"прапор_британські_віргінські_острови", "1F1FB-1F1EC"},
    {"прапор_віргінські_острови_сша", "1F1FB-1F1EE"},
    {"прапор_вєтнам", "1F1FB-1F1F3"},
    {"прапор_вануату", "1F1FB-1F1FA"},
    {"прапор_уолліс_і_футуна", "1F1FC-1F1EB"},
    {"прапор_самоа", "1F1FC-1F1F8"},
    {"прапор_косово", "1F1FD-1F1F0"},
    {"прапор_ємен", "1F1FE-1F1EA"},
    {"прапор_майотта", "1F1FE-1F1F9"},
    {"прапор_південно_африканська_республіка", "1F1FF-1F1E6"},
    {"прапор_замбія", "1F1FF-1F1F2"},
    {"прапор_зімбабве", "1F1FF-1F1FC"},
    {"прапор_англія", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"прапор_шотландія", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"прапор_уельс", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік", "1F468-200D-2764-FE0F-200D-1F468"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_чоловік_і_чоловік_колір5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_чоловік_і_чоловік_колір5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"родина_чоловік_і_хлопчик", "1F468-200D-1F466"},
    {"родина_чоловік_хлопчик_і_хлопчик", "1F468-200D-1F466-200D-1F466"},
    {"родина_чоловік_і_дівчинка", "1F468-200D-1F467"},
    {"родина_чоловік_дівчинка_і_хлопчик", "1F468-200D-1F467-200D-1F466"},
    {"родина_чоловік_дівчинка_і_дівчинка", "1F468-200D-1F467-200D-1F467"},
    {"родина_чоловік_чоловік_і_хлопчик", "1F468-200D-1F468-200D-1F466"},
    {"родина_чоловік_чоловік_хлопчик_і_хлопчик", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"родина_чоловік_чоловік_і_дівчинка", "1F468-200D-1F468-200D-1F467"},
    {"родина_чоловік_чоловік_дівчинка_і_хлопчик", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"родина_чоловік_чоловік_дівчинка_і_дівчинка", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"родина_чоловік_жінка_і_хлопчик", "1F468-200D-1F469-200D-1F466"},
    {"родина_чоловік_жінка_хлопчик_і_хлопчик", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"родина_чоловік_жінка_і_дівчинка", "1F468-200D-1F469-200D-1F467"},
    {"родина_чоловік_жінка_дівчинка_і_хлопчик", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"родина_чоловік_жінка_дівчинка_і_дівчинка", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік", "1F469-200D-2764-FE0F-200D-1F468"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_чоловік_колір5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка", "1F469-200D-2764-FE0F-200D-1F469"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"чоловік_і_жінка_із_серцем_між_ними_жінка_і_жінка_колір5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_чоловік_колір5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"чоловік_і_жінка_що_цілуються_жінка_і_жінка_колір5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"родина_жінка_і_хлопчик", "1F469-200D-1F466"},
    {"родина_жінка_хлопчик_і_хлопчик", "1F469-200D-1F466-200D-1F466"},
    {"родина_жінка_і_дівчинка", "1F469-200D-1F467"},
    {"родина_жінка_дівчинка_і_хлопчик", "1F469-200D-1F467-200D-1F466"},
    {"родина_жінка_дівчинка_і_дівчинка", "1F469-200D-1F467-200D-1F467"},
    {"родина_жінка_жінка_і_хлопчик", "1F469-200D-1F469-200D-1F466"},
    {"родина_жінка_жінка_хлопчик_і_хлопчик", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"родина_жінка_жінка_і_дівчинка", "1F469-200D-1F469-200D-1F467"},
    {"родина_жінка_жінка_дівчинка_і_хлопчик", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"родина_жінка_жінка_дівчинка_і_дівчинка", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"люди_що_тримаються_за_руки", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"люди_що_тримаються_за_руки_колір1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"люди_що_тримаються_за_руки_колір1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"люди_що_тримаються_за_руки_колір1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"люди_що_тримаються_за_руки_колір1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"люди_що_тримаються_за_руки_колір1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"люди_що_тримаються_за_руки_колір2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"люди_що_тримаються_за_руки_колір2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"люди_що_тримаються_за_руки_колір2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"люди_що_тримаються_за_руки_колір2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"люди_що_тримаються_за_руки_колір2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"люди_що_тримаються_за_руки_колір3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"люди_що_тримаються_за_руки_колір3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"люди_що_тримаються_за_руки_колір3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"люди_що_тримаються_за_руки_колір3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"люди_що_тримаються_за_руки_колір3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"люди_що_тримаються_за_руки_колір4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"люди_що_тримаються_за_руки_колір4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"люди_що_тримаються_за_руки_колір4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"люди_що_тримаються_за_руки_колір4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"люди_що_тримаються_за_руки_колір4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"люди_що_тримаються_за_руки_колір5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"люди_що_тримаються_за_руки_колір5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"люди_що_тримаються_за_руки_колір5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"люди_що_тримаються_за_руки_колір5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"люди_що_тримаються_за_руки_колір5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"родина_дорослий_дорослий_дитина", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"родина_двоє_дорослих_і_двоє_дітей", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"родина_дорослий_і_дитина", "1F9D1-200D-1F9D2"},
    {"родина_дорослий_і_діти", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"людина_що_біжить_правий_профіль", "1F3C3-200D-27A1-FE0F"},
    {"людина_що_біжить_правий_профіль_колір1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"людина_що_біжить_правий_профіль_колір2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"людина_що_біжить_правий_профіль_колір3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"людина_що_біжить_правий_профіль_колір4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"людина_що_біжить_правий_профіль_колір5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"медичний_працівник", "1F468-200D-2695-FE0F"},
    {"медичний_працівник_колір1", "1F468-1F3FB-200D-2695-FE0F"},
    {"медичний_працівник_колір2", "1F468-1F3FC-200D-2695-FE0F"},
    {"медичний_працівник_колір3", "1F468-1F3FD-200D-2695-FE0F"},
    {"медичний_працівник_колір4", "1F468-1F3FE-200D-2695-FE0F"},
    {"медичний_працівник_колір5", "1F468-1F3FF-200D-2695-FE0F"},
    {"суддя_чоловік", "1F468-200D-2696-FE0F"},
    {"суддя_чоловік_колір1", "1F468-1F3FB-200D-2696-FE0F"},
    {"суддя_чоловік_колір2", "1F468-1F3FC-200D-2696-FE0F"},
    {"суддя_чоловік_колір3", "1F468-1F3FD-200D-2696-FE0F"},
    {"суддя_чоловік_колір4", "1F468-1F3FE-200D-2696-FE0F"},
    {"суддя_чоловік_колір5", "1F468-1F3FF-200D-2696-FE0F"},
    {"пілот", "1F468-200D-2708-FE0F"},
    {"пілот_колір1", "1F468-1F3FB-200D-2708-FE0F"},
    {"пілот_колір2", "1F468-1F3FC-200D-2708-FE0F"},
    {"пілот_колір3", "1F468-1F3FD-200D-2708-FE0F"},
    {"пілот_колір4", "1F468-1F3FE-200D-2708-FE0F"},
    {"пілот_колір5", "1F468-1F3FF-200D-2708-FE0F"},
    {"фермер", "1F468-200D-1F33E"},
    {"фермер_колір1", "1F468-1F3FB-200D-1F33E"},
    {"фермер_колір2", "1F468-1F3FC-200D-1F33E"},
    {"фермер_колір3", "1F468-1F3FD-200D-1F33E"},
    {"фермер_колір4", "1F468-1F3FE-200D-1F33E"},
    {"фермер_колір5", "1F468-1F3FF-200D-1F33E"},
    {"кухар", "1F468-200D-1F373"},
    {"кухар_колір1", "1F468-1F3FB-200D-1F373"},
    {"кухар_колір2", "1F468-1F3FC-200D-1F373"},
    {"кухар_колір3", "1F468-1F3FD-200D-1F373"},
    {"кухар_колір4", "1F468-1F3FE-200D-1F373"},
    {"кухар_колір5", "1F468-1F3FF-200D-1F373"},
    {"чоловік_годує_дитину", "1F468-200D-1F37C"},
    {"чоловік_годує_дитину_колір1", "1F468-1F3FB-200D-1F37C"},
    {"чоловік_годує_дитину_колір2", "1F468-1F3FC-200D-1F37C"},
    {"чоловік_годує_дитину_колір3", "1F468-1F3FD-200D-1F37C"},
    {"чоловік_годує_дитину_колір4", "1F468-1F3FE-200D-1F37C"},
    {"чоловік_годує_дитину_колір5", "1F468-1F3FF-200D-1F37C"},
    {"студент", "1F468-200D-1F393"},
    {"студент_колір1", "1F468-1F3FB-200D-1F393"},
    {"студент_колір2", "1F468-1F3FC-200D-1F393"},
    {"студент_колір3", "1F468-1F3FD-200D-1F393"},
    {"студент_колір4", "1F468-1F3FE-200D-1F393"},
    {"студент_колір5", "1F468-1F3FF-200D-1F393"},
    {"співак", "1F468-200D-1F3A4"},
    {"співак_колір1", "1F468-1F3FB-200D-1F3A4"},
    {"співак_колір2", "1F468-1F3FC-200D-1F3A4"},
    {"співак_колір3", "1F468-1F3FD-200D-1F3A4"},
    {"співак_колір4", "1F468-1F3FE-200D-1F3A4"},
    {"співак_колір5", "1F468-1F3FF-200D-1F3A4"},
    {"чоловік_художник", "1F468-200D-1F3A8"},
    {"чоловік_художник_колір1", "1F468-1F3FB-200D-1F3A8"},
    {"чоловік_художник_колір2", "1F468-1F3FC-200D-1F3A8"},
    {"чоловік_художник_колір3", "1F468-1F3FD-200D-1F3A8"},
    {"чоловік_художник_колір4", "1F468-1F3FE-200D-1F3A8"},
    {"чоловік_художник_колір5", "1F468-1F3FF-200D-1F3A8"},
    {"учитель", "1F468-200D-1F3EB"},
    {"учитель_колір1", "1F468-1F3FB-200D-1F3EB"},
    {"учитель_колір2", "1F468-1F3FC-200D-1F3EB"},
    {"учитель_колір3", "1F468-1F3FD-200D-1F3EB"},
    {"учитель_колір4", "1F468-1F3FE-200D-1F3EB"},
    {"учитель_колір5", "1F468-1F3FF-200D-1F3EB"},
    {"робітник", "1F468-200D-1F3ED"},
    {"робітник_колір1", "1F468-1F3FB-200D-1F3ED"},
    {"робітник_колір2", "1F468-1F3FC-200D-1F3ED"},
    {"робітник_колір3", "1F468-1F3FD-200D-1F3ED"},
    {"робітник_колір4", "1F468-1F3FE-200D-1F3ED"},
    {"робітник_колір5", "1F468-1F3FF-200D-1F3ED"},
    {"технолог", "1F468-200D-1F4BB"},
    {"технолог_колір1", "1F468-1F3FB-200D-1F4BB"},
    {"технолог_колір2", "1F468-1F3FC-200D-1F4BB"},
    {"технолог_колір3", "1F468-1F3FD-200D-1F4BB"},
    {"технолог_колір4", "1F468-1F3FE-200D-1F4BB"},
    {"технолог_колір5", "1F468-1F3FF-200D-1F4BB"},
    {"службовець", "1F468-200D-1F4BC"},
    {"службовець_колір1", "1F468-1F3FB-200D-1F4BC"},
    {"службовець_колір2", "1F468-1F3FC-200D-1F4BC"},
    {"службовець_колір3", "1F468-1F3FD-200D-1F4BC"},
    {"службовець_колір4", "1F468-1F3FE-200D-1F4BC"},
    {"службовець_колір5", "1F468-1F3FF-200D-1F4BC"},
    {"механік", "1F468-200D-1F527"},
    {"механік_колір1", "1F468-1F3FB-200D-1F527"},
    {"механік_колір2", "1F468-1F3FC-200D-1F527"},
    {"механік_колір3", "1F468-1F3FD-200D-1F527"},
    {"механік_колір4", "1F468-1F3FE-200D-1F527"},
    {"механік_колір5", "1F468-1F3FF-200D-1F527"},
    {"науковець", "1F468-200D-1F52C"},
    {"науковець_колір1", "1F468-1F3FB-200D-1F52C"},
    {"науковець_колір2", "1F468-1F3FC-200D-1F52C"},
    {"науковець_колір3", "1F468-1F3FD-200D-1F52C"},
    {"науковець_колір4", "1F468-1F3FE-200D-1F52C"},
    {"науковець_колір5", "1F468-1F3FF-200D-1F52C"},
    {"астронавт", "1F468-200D-1F680"},
    {"астронавт_колір1", "1F468-1F3FB-200D-1F680"},
    {"астронавт_колір2", "1F468-1F3FC-200D-1F680"},
    {"астронавт_колір3", "1F468-1F3FD-200D-1F680"},
    {"астронавт_колір4", "1F468-1F3FE-200D-1F680"},
    {"астронавт_колір5", "1F468-1F3FF-200D-1F680"},
    {"пожежник", "1F468-200D-1F692"},
    {"пожежник_колір1", "1F468-1F3FB-200D-1F692"},
    {"пожежник_колір2", "1F468-1F3FC-200D-1F692"},
    {"пожежник_колір3", "1F468-1F3FD-200D-1F692"},
    {"пожежник_колір4", "1F468-1F3FE-200D-1F692"},
    {"пожежник_колір5", "1F468-1F3FF-200D-1F692"},
    {"чоловік_із_тактильною_тростиною", "1F468-200D-1F9AF"},
    {"чоловік_із_тактильною_тростиною_колір1", "1F468-1F3FB-200D-1F9AF"},
    {"чоловік_із_тактильною_тростиною_колір2", "1F468-1F3FC-200D-1F9AF"},
    {"чоловік_із_тактильною_тростиною_колір3", "1F468-1F3FD-200D-1F9AF"},
    {"чоловік_із_тактильною_тростиною_колір4", "1F468-1F3FE-200D-1F9AF"},
    {"чоловік_із_тактильною_тростиною_колір5", "1F468-1F3FF-200D-1F9AF"},
    {"чоловік_із_тактильною_тростиною_правий_профіль", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"чоловік_із_тактильною_тростиною_правий_профіль_колір1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"чоловік_із_тактильною_тростиною_правий_профіль_колір2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"чоловік_із_тактильною_тростиною_правий_профіль_колір3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"чоловік_із_тактильною_тростиною_правий_профіль_колір4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"чоловік_із_тактильною_тростиною_правий_профіль_колір5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"чоловік_у_візку_з_електроприводом", "1F468-200D-1F9BC"},
    {"чоловік_у_візку_з_електроприводом_колір1", "1F468-1F3FB-200D-1F9BC"},
    {"чоловік_у_візку_з_електроприводом_колір2", "1F468-1F3FC-200D-1F9BC"},
    {"чоловік_у_візку_з_електроприводом_колір3", "1F468-1F3FD-200D-1F9BC"},
    {"чоловік_у_візку_з_електроприводом_колір4", "1F468-1F3FE-200D-1F9BC"},
    {"чоловік_у_візку_з_електроприводом_колір5", "1F468-1F3FF-200D-1F9BC"},
    {"чоловік_у_візку_з_електроприводом_правий_профіль", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"чоловік_у_візку_з_електроприводом_правий_профіль_колір1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"чоловік_у_візку_з_електроприводом_правий_профіль_колір2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"чоловік_у_візку_з_електроприводом_правий_профіль_колір3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"чоловік_у_візку_з_електроприводом_правий_профіль_колір4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"чоловік_у_візку_з_електроприводом_правий_профіль_колір5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"чоловік_у_візку_з_ручним_приводом", "1F468-200D-1F9BD"},
    {"чоловік_у_візку_з_ручним_приводом_колір1", "1F468-1F3FB-200D-1F9BD"},
    {"чоловік_у_візку_з_ручним_приводом_колір2", "1F468-1F3FC-200D-1F9BD"},
    {"чоловік_у_візку_з_ручним_приводом_колір3", "1F468-1F3FD-200D-1F9BD"},
    {"чоловік_у_візку_з_ручним_приводом_колір4", "1F468-1F3FE-200D-1F9BD"},
    {"чоловік_у_візку_з_ручним_приводом_колір5", "1F468-1F3FF-200D-1F9BD"},
    {"чоловік_у_візку_з_ручним_приводом_правий_профіль", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"чоловік_у_візку_з_ручним_приводом_правий_профіль_колір1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"чоловік_у_візку_з_ручним_приводом_правий_профіль_колір2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"чоловік_у_візку_з_ручним_приводом_правий_профіль_колір3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"чоловік_у_візку_з_ручним_приводом_правий_профіль_колір4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"чоловік_у_візку_з_ручним_приводом_правий_профіль_колір5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"медична_працівниця", "1F469-200D-2695-FE0F"},
    {"медична_працівниця_колір1", "1F469-1F3FB-200D-2695-FE0F"},
    {"медична_працівниця_колір2", "1F469-1F3FC-200D-2695-FE0F"},
    {"медична_працівниця_колір3", "1F469-1F3FD-200D-2695-FE0F"},
    {"медична_працівниця_колір4", "1F469-1F3FE-200D-2695-FE0F"},
    {"медична_працівниця_колір5", "1F469-1F3FF-200D-2695-FE0F"},
    {"жінка_суддя", "1F469-200D-2696-FE0F"},
    {"жінка_суддя_колір1", "1F469-1F3FB-200D-2696-FE0F"},
    {"жінка_суддя_колір2", "1F469-1F3FC-200D-2696-FE0F"},
    {"жінка_суддя_колір3", "1F469-1F3FD-200D-2696-FE0F"},
    {"жінка_суддя_колір4", "1F469-1F3FE-200D-2696-FE0F"},
    {"жінка_суддя_колір5", "1F469-1F3FF-200D-2696-FE0F"},
    {"жінка_пілот", "1F469-200D-2708-FE0F"},
    {"жінка_пілот_колір1", "1F469-1F3FB-200D-2708-FE0F"},
    {"жінка_пілот_колір2", "1F469-1F3FC-200D-2708-FE0F"},
    {"жінка_пілот_колір3", "1F469-1F3FD-200D-2708-FE0F"},
    {"жінка_пілот_колір4", "1F469-1F3FE-200D-2708-FE0F"},
    {"жінка_пілот_колір5", "1F469-1F3FF-200D-2708-FE0F"},
    {"фермерка", "1F469-200D-1F33E"},
    {"фермерка_колір1", "1F469-1F3FB-200D-1F33E"},
    {"фермерка_колір2", "1F469-1F3FC-200D-1F33E"},
    {"фермерка_колір3", "1F469-1F3FD-200D-1F33E"},
    {"фермерка_колір4", "1F469-1F3FE-200D-1F33E"},
    {"фермерка_колір5", "1F469-1F3FF-200D-1F33E"},
    {"кухарка", "1F469-200D-1F373"},
    {"кухарка_колір1", "1F469-1F3FB-200D-1F373"},
    {"кухарка_колір2", "1F469-1F3FC-200D-1F373"},
    {"кухарка_колір3", "1F469-1F3FD-200D-1F373"},
    {"кухарка_колір4", "1F469-1F3FE-200D-1F373"},
    {"кухарка_колір5", "1F469-1F3FF-200D-1F373"},
    {"жінка_годує_дитину", "1F469-200D-1F37C"},
    {"жінка_годує_дитину_колір1", "1F469-1F3FB-200D-1F37C"},
    {"жінка_годує_дитину_колір2", "1F469-1F3FC-200D-1F37C"},
    {"жінка_годує_дитину_колір3", "1F469-1F3FD-200D-1F37C"},
    {"жінка_годує_дитину_колір4", "1F469-1F3FE-200D-1F37C"},
    {"жінка_годує_дитину_колір5", "1F469-1F3FF-200D-1F37C"},
    {"студентка", "1F469-200D-1F393"},
    {"студентка_колір1", "1F469-1F3FB-200D-1F393"},
    {"студентка_колір2", "1F469-1F3FC-200D-1F393"},
    {"студентка_колір3", "1F469-1F3FD-200D-1F393"},
    {"студентка_колір4", "1F469-1F3FE-200D-1F393"},
    {"студентка_колір5", "1F469-1F3FF-200D-1F393"},
    {"співачка", "1F469-200D-1F3A4"},
    {"співачка_колір1", "1F469-1F3FB-200D-1F3A4"},
    {"співачка_колір2", "1F469-1F3FC-200D-1F3A4"},
    {"співачка_колір3", "1F469-1F3FD-200D-1F3A4"},
    {"співачка_колір4", "1F469-1F3FE-200D-1F3A4"},
    {"співачка_колір5", "1F469-1F3FF-200D-1F3A4"},
    {"художниця", "1F469-200D-1F3A8"},
    {"художниця_колір1", "1F469-1F3FB-200D-1F3A8"},
    {"художниця_колір2", "1F469-1F3FC-200D-1F3A8"},
    {"художниця_колір3", "1F469-1F3FD-200D-1F3A8"},
    {"художниця_колір4", "1F469-1F3FE-200D-1F3A8"},
    {"художниця_колір5", "1F469-1F3FF-200D-1F3A8"},
    {"учителька", "1F469-200D-1F3EB"},
    {"учителька_колір1", "1F469-1F3FB-200D-1F3EB"},
    {"учителька_колір2", "1F469-1F3FC-200D-1F3EB"},
    {"учителька_колір3", "1F469-1F3FD-200D-1F3EB"},
    {"учителька_колір4", "1F469-1F3FE-200D-1F3EB"},
    {"учителька_колір5", "1F469-1F3FF-200D-1F3EB"},
    {"робітниця", "1F469-200D-1F3ED"},
    {"робітниця_колір1", "1F469-1F3FB-200D-1F3ED"},
    {"робітниця_колір2", "1F469-1F3FC-200D-1F3ED"},
    {"робітниця_колір3", "1F469-1F3FD-200D-1F3ED"},
    {"робітниця_колір4", "1F469-1F3FE-200D-1F3ED"},
    {"робітниця_колір5", "1F469-1F3FF-200D-1F3ED"},
    {"жінка_технолог", "1F469-200D-1F4BB"},
    {"жінка_технолог_колір1", "1F469-1F3FB-200D-1F4BB"},
    {"жінка_технолог_колір2", "1F469-1F3FC-200D-1F4BB"},
    {"жінка_технолог_колір3", "1F469-1F3FD-200D-1F4BB"},
    {"жінка_технолог_колір4", "1F469-1F3FE-200D-1F4BB"},
    {"жінка_технолог_колір5", "1F469-1F3FF-200D-1F4BB"},
    {"службовка", "1F469-200D-1F4BC"},
    {"службовка_колір1", "1F469-1F3FB-200D-1F4BC"},
    {"службовка_колір2", "1F469-1F3FC-200D-1F4BC"},
    {"службовка_колір3", "1F469-1F3FD-200D-1F4BC"},
    {"службовка_колір4", "1F469-1F3FE-200D-1F4BC"},
    {"службовка_колір5", "1F469-1F3FF-200D-1F4BC"},
    {"жінка_механік", "1F469-200D-1F527"},
    {"жінка_механік_колір1", "1F469-1F3FB-200D-1F527"},
    {"жінка_механік_колір2", "1F469-1F3FC-200D-1F527"},
    {"жінка_механік_колір3", "1F469-1F3FD-200D-1F527"},
    {"жінка_механік_колір4", "1F469-1F3FE-200D-1F527"},
    {"жінка_механік_колір5", "1F469-1F3FF-200D-1F527"},
    {"жінка_науковець", "1F469-200D-1F52C"},
    {"жінка_науковець_колір1", "1F469-1F3FB-200D-1F52C"},
    {"жінка_науковець_колір2", "1F469-1F3FC-200D-1F52C"},
    {"жінка_науковець_колір3", "1F469-1F3FD-200D-1F52C"},
    {"жінка_науковець_колір4", "1F469-1F3FE-200D-1F52C"},
    {"жінка_науковець_колір5", "1F469-1F3FF-200D-1F52C"},
    {"астронавтка", "1F469-200D-1F680"},
    {"астронавтка_колір1", "1F469-1F3FB-200D-1F680"},
    {"астронавтка_колір2", "1F469-1F3FC-200D-1F680"},
    {"астронавтка_колір3", "1F469-1F3FD-200D-1F680"},
    {"астронавтка_колір4", "1F469-1F3FE-200D-1F680"},
    {"астронавтка_колір5", "1F469-1F3FF-200D-1F680"},
    {"жінка_пожежник", "1F469-200D-1F692"},
    {"жінка_пожежник_колір1", "1F469-1F3FB-200D-1F692"},
    {"жінка_пожежник_колір2", "1F469-1F3FC-200D-1F692"},
    {"жінка_пожежник_колір3", "1F469-1F3FD-200D-1F692"},
    {"жінка_пожежник_колір4", "1F469-1F3FE-200D-1F692"},
    {"жінка_пожежник_колір5", "1F469-1F3FF-200D-1F692"},
    {"жінка_з_тактильною_тростиною", "1F469-200D-1F9AF"},
    {"жінка_з_тактильною_тростиною_колір1", "1F469-1F3FB-200D-1F9AF"},
    {"жінка_з_тактильною_тростиною_колір2", "1F469-1F3FC-200D-1F9AF"},
    {"жінка_з_тактильною_тростиною_колір3", "1F469-1F3FD-200D-1F9AF"},
    {"жінка_з_тактильною_тростиною_колір4", "1F469-1F3FE-200D-1F9AF"},
    {"жінка_з_тактильною_тростиною_колір5", "1F469-1F3FF-200D-1F9AF"},
    {"жінка_з_тактильною_тростиною_правий_профіль", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"жінка_з_тактильною_тростиною_правий_профіль_колір1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"жінка_з_тактильною_тростиною_правий_профіль_колір2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"жінка_з_тактильною_тростиною_правий_профіль_колір3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"жінка_з_тактильною_тростиною_правий_профіль_колір4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"жінка_з_тактильною_тростиною_правий_профіль_колір5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"жінка_у_візку_з_електроприводом", "1F469-200D-1F9BC"},
    {"жінка_у_візку_з_електроприводом_колір1", "1F469-1F3FB-200D-1F9BC"},
    {"жінка_у_візку_з_електроприводом_колір2", "1F469-1F3FC-200D-1F9BC"},
    {"жінка_у_візку_з_електроприводом_колір3", "1F469-1F3FD-200D-1F9BC"},
    {"жінка_у_візку_з_електроприводом_колір4", "1F469-1F3FE-200D-1F9BC"},
    {"жінка_у_візку_з_електроприводом_колір5", "1F469-1F3FF-200D-1F9BC"},
    {"жінка_у_візку_з_електроприводом_правий_профіль", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"жінка_у_візку_з_електроприводом_правий_профіль_колір1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"жінка_у_візку_з_електроприводом_правий_профіль_колір2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"жінка_у_візку_з_електроприводом_правий_профіль_колір3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"жінка_у_візку_з_електроприводом_правий_профіль_колір4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"жінка_у_візку_з_електроприводом_правий_профіль_колір5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"жінка_у_візку_з_ручним_приводом", "1F469-200D-1F9BD"},
    {"жінка_у_візку_з_ручним_приводом_колір1", "1F469-1F3FB-200D-1F9BD"},
    {"жінка_у_візку_з_ручним_приводом_колір2", "1F469-1F3FC-200D-1F9BD"},
    {"жінка_у_візку_з_ручним_приводом_колір3", "1F469-1F3FD-200D-1F9BD"},
    {"жінка_у_візку_з_ручним_приводом_колір4", "1F469-1F3FE-200D-1F9BD"},
    {"жінка_у_візку_з_ручним_приводом_колір5", "1F469-1F3FF-200D-1F9BD"},
    {"жінка_у_візку_з_ручним_приводом_правий_профіль", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"жінка_у_візку_з_ручним_приводом_правий_профіль_колір1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"жінка_у_візку_з_ручним_приводом_правий_профіль_колір2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"жінка_у_візку_з_ручним_приводом_правий_профіль_колір3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"жінка_у_візку_з_ручним_приводом_правий_профіль_колір4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"жінка_у_візку_з_ручним_приводом_правий_профіль_колір5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"людина_що_йде_пішки_правий_профіль", "1F6B6-200D-27A1-FE0F"},
    {"людина_що_йде_пішки_правий_профіль_колір1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"людина_що_йде_пішки_правий_профіль_колір2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"людина_що_йде_пішки_правий_профіль_колір3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"людина_що_йде_пішки_правий_профіль_колір4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"людина_що_йде_пішки_правий_профіль_колір5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"людина_навколішках_правий_профіль", "1F9CE-200D-27A1-FE0F"},
    {"людина_навколішках_правий_профіль_колір1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"людина_навколішках_правий_профіль_колір2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"людина_навколішках_правий_профіль_колір3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"людина_навколішках_правий_профіль_колір4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"людина_навколішках_правий_профіль_колір5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"медик", "1F9D1-200D-2695-FE0F"},
    {"медик_колір1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"медик_колір2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"медик_колір3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"медик_колір4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"медик_колір5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"суддя", "1F9D1-200D-2696-FE0F"},
    {"суддя_колір1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"суддя_колір2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"суддя_колір3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"суддя_колір4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"суддя_колір5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"авіапілот", "1F9D1-200D-2708-FE0F"},
    {"авіапілот_колір1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"авіапілот_колір2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"авіапілот_колір3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"авіапілот_колір4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"авіапілот_колір5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"селянин", "1F9D1-200D-1F33E"},
    {"селянин_колір1", "1F9D1-1F3FB-200D-1F33E"},
    {"селянин_колір2", "1F9D1-1F3FC-200D-1F33E"},
    {"селянин_колір3", "1F9D1-1F3FD-200D-1F33E"},
    {"селянин_колір4", "1F9D1-1F3FE-200D-1F33E"},
    {"селянин_колір5", "1F9D1-1F3FF-200D-1F33E"},
    {"куховар", "1F9D1-200D-1F373"},
    {"куховар_колір1", "1F9D1-1F3FB-200D-1F373"},
    {"куховар_колір2", "1F9D1-1F3FC-200D-1F373"},
    {"куховар_колір3", "1F9D1-1F3FD-200D-1F373"},
    {"куховар_колір4", "1F9D1-1F3FE-200D-1F373"},
    {"куховар_колір5", "1F9D1-1F3FF-200D-1F373"},
    {"людина_годує_дитину", "1F9D1-200D-1F37C"},
    {"людина_годує_дитину_колір1", "1F9D1-1F3FB-200D-1F37C"},
    {"людина_годує_дитину_колір2", "1F9D1-1F3FC-200D-1F37C"},
    {"людина_годує_дитину_колір3", "1F9D1-1F3FD-200D-1F37C"},
    {"людина_годує_дитину_колір4", "1F9D1-1F3FE-200D-1F37C"},
    {"людина_годує_дитину_колір5", "1F9D1-1F3FF-200D-1F37C"},
    {"мікс_клаус", "1F9D1-200D-1F384"},
    {"мікс_клаус_колір1", "1F9D1-1F3FB-200D-1F384"},
    {"мікс_клаус_колір2", "1F9D1-1F3FC-200D-1F384"},
    {"мікс_клаус_колір3", "1F9D1-1F3FD-200D-1F384"},
    {"мікс_клаус_колір4", "1F9D1-1F3FE-200D-1F384"},
    {"мікс_клаус_колір5", "1F9D1-1F3FF-200D-1F384"},
    {"слухач", "1F9D1-200D-1F393"},
    {"слухач_колір1", "1F9D1-1F3FB-200D-1F393"},
    {"слухач_колір2", "1F9D1-1F3FC-200D-1F393"},
    {"слухач_колір3", "1F9D1-1F3FD-200D-1F393"},
    {"слухач_колір4", "1F9D1-1F3FE-200D-1F393"},
    {"слухач_колір5", "1F9D1-1F3FF-200D-1F393"},
    {"вокаліст", "1F9D1-200D-1F3A4"},
    {"вокаліст_колір1", "1F9D1-1F3FB-200D-1F3A4"},
    {"вокаліст_колір2", "1F9D1-1F3FC-200D-1F3A4"},
    {"вокаліст_колір3", "1F9D1-1F3FD-200D-1F3A4"},
    {"вокаліст_колір4", "1F9D1-1F3FE-200D-1F3A4"},
    {"вокаліст_колір5", "1F9D1-1F3FF-200D-1F3A4"},
    {"художник", "1F9D1-200D-1F3A8"},
    {"художник_колір1", "1F9D1-1F3FB-200D-1F3A8"},
    {"художник_колір2", "1F9D1-1F3FC-200D-1F3A8"},
    {"художник_колір3", "1F9D1-1F3FD-200D-1F3A8"},
    {"художник_колір4", "1F9D1-1F3FE-200D-1F3A8"},
    {"художник_колір5", "1F9D1-1F3FF-200D-1F3A8"},
    {"освітянин", "1F9D1-200D-1F3EB"},
    {"освітянин_колір1", "1F9D1-1F3FB-200D-1F3EB"},
    {"освітянин_колір2", "1F9D1-1F3FC-200D-1F3EB"},
    {"освітянин_колір3", "1F9D1-1F3FD-200D-1F3EB"},
    {"освітянин_колір4", "1F9D1-1F3FE-200D-1F3EB"},
    {"освітянин_колір5", "1F9D1-1F3FF-200D-1F3EB"},
    {"фабричний_працівник", "1F9D1-200D-1F3ED"},
    {"фабричний_працівник_колір1", "1F9D1-1F3FB-200D-1F3ED"},
    {"фабричний_працівник_колір2", "1F9D1-1F3FC-200D-1F3ED"},
    {"фабричний_працівник_колір3", "1F9D1-1F3FD-200D-1F3ED"},
    {"фабричний_працівник_колір4", "1F9D1-1F3FE-200D-1F3ED"},
    {"фабричний_працівник_колір5", "1F9D1-1F3FF-200D-1F3ED"},
    {"технічний_працівник", "1F9D1-200D-1F4BB"},
    {"технічний_працівник_колір1", "1F9D1-1F3FB-200D-1F4BB"},
    {"технічний_працівник_колір2", "1F9D1-1F3FC-200D-1F4BB"},
    {"технічний_працівник_колір3", "1F9D1-1F3FD-200D-1F4BB"},
    {"технічний_працівник_колір4", "1F9D1-1F3FE-200D-1F4BB"},
    {"технічний_працівник_колір5", "1F9D1-1F3FF-200D-1F4BB"},
    {"офісний_працівник", "1F9D1-200D-1F4BC"},
    {"офісний_працівник_колір1", "1F9D1-1F3FB-200D-1F4BC"},
    {"офісний_працівник_колір2", "1F9D1-1F3FC-200D-1F4BC"},
    {"офісний_працівник_колір3", "1F9D1-1F3FD-200D-1F4BC"},
    {"офісний_працівник_колір4", "1F9D1-1F3FE-200D-1F4BC"},
    {"офісний_працівник_колір5", "1F9D1-1F3FF-200D-1F4BC"},
    {"фахівець_із_механіки", "1F9D1-200D-1F527"},
    {"фахівець_із_механіки_колір1", "1F9D1-1F3FB-200D-1F527"},
    {"фахівець_із_механіки_колір2", "1F9D1-1F3FC-200D-1F527"},
    {"фахівець_із_механіки_колір3", "1F9D1-1F3FD-200D-1F527"},
    {"фахівець_із_механіки_колір4", "1F9D1-1F3FE-200D-1F527"},
    {"фахівець_із_механіки_колір5", "1F9D1-1F3FF-200D-1F527"},
    {"науковий_працівник", "1F9D1-200D-1F52C"},
    {"науковий_працівник_колір1", "1F9D1-1F3FB-200D-1F52C"},
    {"науковий_працівник_колір2", "1F9D1-1F3FC-200D-1F52C"},
    {"науковий_працівник_колір3", "1F9D1-1F3FD-200D-1F52C"},
    {"науковий_працівник_колір4", "1F9D1-1F3FE-200D-1F52C"},
    {"науковий_працівник_колір5", "1F9D1-1F3FF-200D-1F52C"},
    {"космонавт", "1F9D1-200D-1F680"},
    {"космонавт_колір1", "1F9D1-1F3FB-200D-1F680"},
    {"космонавт_колір2", "1F9D1-1F3FC-200D-1F680"},
    {"космонавт_колір3", "1F9D1-1F3FD-200D-1F680"},
    {"космонавт_колір4", "1F9D1-1F3FE-200D-1F680"},
    {"космонавт_колір5", "1F9D1-1F3FF-200D-1F680"},
    {"вогнеборець", "1F9D1-200D-1F692"},
    {"вогнеборець_колір1", "1F9D1-1F3FB-200D-1F692"},
    {"вогнеборець_колір2", "1F9D1-1F3FC-200D-1F692"},
    {"вогнеборець_колір3", "1F9D1-1F3FD-200D-1F692"},
    {"вогнеборець_колір4", "1F9D1-1F3FE-200D-1F692"},
    {"вогнеборець_колір5", "1F9D1-1F3FF-200D-1F692"},
    {"людина_з_тактильним_ціпком", "1F9D1-200D-1F9AF"},
    {"людина_з_тактильним_ціпком_колір1", "1F9D1-1F3FB-200D-1F9AF"},
    {"людина_з_тактильним_ціпком_колір2", "1F9D1-1F3FC-200D-1F9AF"},
    {"людина_з_тактильним_ціпком_колір3", "1F9D1-1F3FD-200D-1F9AF"},
    {"людина_з_тактильним_ціпком_колір4", "1F9D1-1F3FE-200D-1F9AF"},
    {"людина_з_тактильним_ціпком_колір5", "1F9D1-1F3FF-200D-1F9AF"},
    {"людина_з_тактильним_ціпком_правий_профіль", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"людина_з_тактильним_ціпком_правий_профіль_колір1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"людина_з_тактильним_ціпком_правий_профіль_колір2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"людина_з_тактильним_ціпком_правий_профіль_колір3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"людина_з_тактильним_ціпком_правий_профіль_колір4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"людина_з_тактильним_ціпком_правий_профіль_колір5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"людина_на_моторизованому_візку", "1F9D1-200D-1F9BC"},
    {"людина_на_моторизованому_візку_колір1", "1F9D1-1F3FB-200D-1F9BC"},
    {"людина_на_моторизованому_візку_колір2", "1F9D1-1F3FC-200D-1F9BC"},
    {"людина_на_моторизованому_візку_колір3", "1F9D1-1F3FD-200D-1F9BC"},
    {"людина_на_моторизованому_візку_колір4", "1F9D1-1F3FE-200D-1F9BC"},
    {"людина_на_моторизованому_візку_колір5", "1F9D1-1F3FF-200D-1F9BC"},
    {"людина_на_моторизованому_візку_правий_профіль", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"людина_на_моторизованому_візку_правий_профіль_колір1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"людина_на_моторизованому_візку_правий_профіль_колір2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"людина_на_моторизованому_візку_правий_профіль_колір3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"людина_на_моторизованому_візку_правий_профіль_колір4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"людина_на_моторизованому_візку_правий_профіль_колір5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"людина_у_візку_з_ручним_керуванням", "1F9D1-200D-1F9BD"},
    {"людина_у_візку_з_ручним_керуванням_колір1", "1F9D1-1F3FB-200D-1F9BD"},
    {"людина_у_візку_з_ручним_керуванням_колір2", "1F9D1-1F3FC-200D-1F9BD"},
    {"людина_у_візку_з_ручним_керуванням_колір3", "1F9D1-1F3FD-200D-1F9BD"},
    {"людина_у_візку_з_ручним_керуванням_колір4", "1F9D1-1F3FE-200D-1F9BD"},
    {"людина_у_візку_з_ручним_керуванням_колір5", "1F9D1-1F3FF-200D-1F9BD"},
    {"людина_у_візку_з_ручним_керуванням_правий_профіль", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"людина_у_візку_з_ручним_керуванням_правий_профіль_колір1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"людина_у_візку_з_ручним_керуванням_правий_профіль_колір2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"людина_у_візку_з_ручним_керуванням_правий_профіль_колір3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"людина_у_візку_з_ручним_керуванням_правий_профіль_колір4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"людина_у_візку_з_ручним_керуванням_правий_профіль_колір5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"жінка_що_веде_мяч", "26F9-FE0F-200D-2640-FE0F"},
    {"жінка_що_веде_мяч_колір1", "26F9-1F3FB-200D-2640-FE0F"},
    {"жінка_що_веде_мяч_колір2", "26F9-1F3FC-200D-2640-FE0F"},
    {"жінка_що_веде_мяч_колір3", "26F9-1F3FD-200D-2640-FE0F"},
    {"жінка_що_веде_мяч_колір4", "26F9-1F3FE-200D-2640-FE0F"},
    {"жінка_що_веде_мяч_колір5", "26F9-1F3FF-200D-2640-FE0F"},
    {"чоловік_що_веде_мяч", "26F9-FE0F-200D-2642-FE0F"},
    {"чоловік_що_веде_мяч_колір1", "26F9-1F3FB-200D-2642-FE0F"},
    {"чоловік_що_веде_мяч_колір2", "26F9-1F3FC-200D-2642-FE0F"},
    {"чоловік_що_веде_мяч_колір3", "26F9-1F3FD-200D-2642-FE0F"},
    {"чоловік_що_веде_мяч_колір4", "26F9-1F3FE-200D-2642-FE0F"},
    {"чоловік_що_веде_мяч_колір5", "26F9-1F3FF-200D-2642-FE0F"},
    {"жінка_що_біжить", "1F3C3-200D-2640-FE0F"},
    {"жінка_що_біжить_колір1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"жінка_що_біжить_колір2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"жінка_що_біжить_колір3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"жінка_що_біжить_колір4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"жінка_що_біжить_колір5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"жінка_що_біжить_правий_профіль", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_що_біжить_правий_профіль_колір1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_що_біжить_правий_профіль_колір2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_що_біжить_правий_профіль_колір3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_що_біжить_правий_профіль_колір4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_що_біжить_правий_профіль_колір5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"чоловік_що_біжить", "1F3C3-200D-2642-FE0F"},
    {"чоловік_що_біжить_колір1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"чоловік_що_біжить_колір2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"чоловік_що_біжить_колір3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"чоловік_що_біжить_колір4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"чоловік_що_біжить_колір5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"чоловік_що_біжить_правий_профіль", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_що_біжить_правий_профіль_колір1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_що_біжить_правий_профіль_колір2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_що_біжить_правий_профіль_колір3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_що_біжить_правий_профіль_колір4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_що_біжить_правий_профіль_колір5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"серфінгістка", "1F3C4-200D-2640-FE0F"},
    {"серфінгістка_колір1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"серфінгістка_колір2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"серфінгістка_колір3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"серфінгістка_колір4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"серфінгістка_колір5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"серфінгіст", "1F3C4-200D-2642-FE0F"},
    {"серфінгіст_колір1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"серфінгіст_колір2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"серфінгіст_колір3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"серфінгіст_колір4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"серфінгіст_колір5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"плавчиня", "1F3CA-200D-2640-FE0F"},
    {"плавчиня_колір1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"плавчиня_колір2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"плавчиня_колір3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"плавчиня_колір4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"плавчиня_колір5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"плавець", "1F3CA-200D-2642-FE0F"},
    {"плавець_колір1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"плавець_колір2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"плавець_колір3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"плавець_колір4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"плавець_колір5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"штангістка", "1F3CB-FE0F-200D-2640-FE0F"},
    {"штангістка_колір1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"штангістка_колір2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"штангістка_колір3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"штангістка_колір4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"штангістка_колір5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"штангіст", "1F3CB-FE0F-200D-2642-FE0F"},
    {"штангіст_колір1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"штангіст_колір2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"штангіст_колір3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"штангіст_колір4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"штангіст_колір5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"гольфістка", "1F3CC-FE0F-200D-2640-FE0F"},
    {"гольфістка_колір1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"гольфістка_колір2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"гольфістка_колір3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"гольфістка_колір4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"гольфістка_колір5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"гольфіст", "1F3CC-FE0F-200D-2642-FE0F"},
    {"гольфіст_колір1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"гольфіст_колір2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"гольфіст_колір3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"гольфіст_колір4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"гольфіст_колір5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"жінка_поліцейський", "1F46E-200D-2640-FE0F"},
    {"жінка_поліцейський_колір1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"жінка_поліцейський_колір2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"жінка_поліцейський_колір3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"жінка_поліцейський_колір4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"жінка_поліцейський_колір5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"поліцейський", "1F46E-200D-2642-FE0F"},
    {"поліцейський_колір1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"поліцейський_колір2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"поліцейський_колір3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"поліцейський_колір4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"поліцейський_колір5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"жінки_з_кролячими_вухами", "1F46F-200D-2640-FE0F"},
    {"жінки_з_кролячими_вухами_колір1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"жінки_з_кролячими_вухами_колір1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"жінки_з_кролячими_вухами_колір1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"жінки_з_кролячими_вухами_колір1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"жінки_з_кролячими_вухами_колір2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"жінки_з_кролячими_вухами_колір2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"жінки_з_кролячими_вухами_колір2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"жінки_з_кролячими_вухами_колір2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"жінки_з_кролячими_вухами_колір3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"жінки_з_кролячими_вухами_колір3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"жінки_з_кролячими_вухами_колір3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"жінки_з_кролячими_вухами_колір3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"жінки_з_кролячими_вухами_колір4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"жінки_з_кролячими_вухами_колір4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"жінки_з_кролячими_вухами_колір4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"жінки_з_кролячими_вухами_колір4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"жінки_з_кролячими_вухами_колір5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"жінки_з_кролячими_вухами_колір5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"жінки_з_кролячими_вухами_колір5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"жінки_з_кролячими_вухами_колір5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"жінки_з_кролячими_вухами_колір1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"жінки_з_кролячими_вухами_колір2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"жінки_з_кролячими_вухами_колір3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"жінки_з_кролячими_вухами_колір4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"жінки_з_кролячими_вухами_колір5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"чоловіки_з_кролячими_вухами", "1F46F-200D-2642-FE0F"},
    {"чоловіки_з_кролячими_вухами_колір1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"чоловіки_з_кролячими_вухами_колір1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"чоловіки_з_кролячими_вухами_колір1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"чоловіки_з_кролячими_вухами_колір1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"чоловіки_з_кролячими_вухами_колір2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"чоловіки_з_кролячими_вухами_колір2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"чоловіки_з_кролячими_вухами_колір2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"чоловіки_з_кролячими_вухами_колір2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"чоловіки_з_кролячими_вухами_колір3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"чоловіки_з_кролячими_вухами_колір3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"чоловіки_з_кролячими_вухами_колір3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"чоловіки_з_кролячими_вухами_колір3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"чоловіки_з_кролячими_вухами_колір4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"чоловіки_з_кролячими_вухами_колір4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"чоловіки_з_кролячими_вухами_колір4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"чоловіки_з_кролячими_вухами_колір4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"чоловіки_з_кролячими_вухами_колір5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"чоловіки_з_кролячими_вухами_колір5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"чоловіки_з_кролячими_вухами_колір5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"чоловіки_з_кролячими_вухами_колір5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"чоловіки_з_кролячими_вухами_колір1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"чоловіки_з_кролячими_вухами_колір2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"чоловіки_з_кролячими_вухами_колір3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"чоловіки_з_кролячими_вухами_колір4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"чоловіки_з_кролячими_вухами_колір5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"жінка_у_фаті", "1F470-200D-2640-FE0F"},
    {"жінка_у_фаті_колір1", "1F470-1F3FB-200D-2640-FE0F"},
    {"жінка_у_фаті_колір2", "1F470-1F3FC-200D-2640-FE0F"},
    {"жінка_у_фаті_колір3", "1F470-1F3FD-200D-2640-FE0F"},
    {"жінка_у_фаті_колір4", "1F470-1F3FE-200D-2640-FE0F"},
    {"жінка_у_фаті_колір5", "1F470-1F3FF-200D-2640-FE0F"},
    {"чоловік_у_фаті", "1F470-200D-2642-FE0F"},
    {"чоловік_у_фаті_колір1", "1F470-1F3FB-200D-2642-FE0F"},
    {"чоловік_у_фаті_колір2", "1F470-1F3FC-200D-2642-FE0F"},
    {"чоловік_у_фаті_колір3", "1F470-1F3FD-200D-2642-FE0F"},
    {"чоловік_у_фаті_колір4", "1F470-1F3FE-200D-2642-FE0F"},
    {"чоловік_у_фаті_колір5", "1F470-1F3FF-200D-2642-FE0F"},
    {"жінка_з_білявим_волоссям", "1F471-200D-2640-FE0F"},
    {"жінка_з_білявим_волоссям_колір1", "1F471-1F3FB-200D-2640-FE0F"},
    {"жінка_з_білявим_волоссям_колір2", "1F471-1F3FC-200D-2640-FE0F"},
    {"жінка_з_білявим_волоссям_колір3", "1F471-1F3FD-200D-2640-FE0F"},
    {"жінка_з_білявим_волоссям_колір4", "1F471-1F3FE-200D-2640-FE0F"},
    {"жінка_з_білявим_волоссям_колір5", "1F471-1F3FF-200D-2640-FE0F"},
    {"чоловік_із_білявим_волоссям", "1F471-200D-2642-FE0F"},
    {"чоловік_із_білявим_волоссям_колір1", "1F471-1F3FB-200D-2642-FE0F"},
    {"чоловік_із_білявим_волоссям_колір2", "1F471-1F3FC-200D-2642-FE0F"},
    {"чоловік_із_білявим_волоссям_колір3", "1F471-1F3FD-200D-2642-FE0F"},
    {"чоловік_із_білявим_волоссям_колір4", "1F471-1F3FE-200D-2642-FE0F"},
    {"чоловік_із_білявим_волоссям_колір5", "1F471-1F3FF-200D-2642-FE0F"},
    {"жінка_в_тюрбані", "1F473-200D-2640-FE0F"},
    {"жінка_в_тюрбані_колір1", "1F473-1F3FB-200D-2640-FE0F"},
    {"жінка_в_тюрбані_колір2", "1F473-1F3FC-200D-2640-FE0F"},
    {"жінка_в_тюрбані_колір3", "1F473-1F3FD-200D-2640-FE0F"},
    {"жінка_в_тюрбані_колір4", "1F473-1F3FE-200D-2640-FE0F"},
    {"жінка_в_тюрбані_колір5", "1F473-1F3FF-200D-2640-FE0F"},
    {"чоловік_у_тюрбані", "1F473-200D-2642-FE0F"},
    {"чоловік_у_тюрбані_колір1", "1F473-1F3FB-200D-2642-FE0F"},
    {"чоловік_у_тюрбані_колір2", "1F473-1F3FC-200D-2642-FE0F"},
    {"чоловік_у_тюрбані_колір3", "1F473-1F3FD-200D-2642-FE0F"},
    {"чоловік_у_тюрбані_колір4", "1F473-1F3FE-200D-2642-FE0F"},
    {"чоловік_у_тюрбані_колір5", "1F473-1F3FF-200D-2642-FE0F"},
    {"жінка_будівельник", "1F477-200D-2640-FE0F"},
    {"жінка_будівельник_колір1", "1F477-1F3FB-200D-2640-FE0F"},
    {"жінка_будівельник_колір2", "1F477-1F3FC-200D-2640-FE0F"},
    {"жінка_будівельник_колір3", "1F477-1F3FD-200D-2640-FE0F"},
    {"жінка_будівельник_колір4", "1F477-1F3FE-200D-2640-FE0F"},
    {"жінка_будівельник_колір5", "1F477-1F3FF-200D-2640-FE0F"},
    {"чоловік_будівельник", "1F477-200D-2642-FE0F"},
    {"чоловік_будівельник_колір1", "1F477-1F3FB-200D-2642-FE0F"},
    {"чоловік_будівельник_колір2", "1F477-1F3FC-200D-2642-FE0F"},
    {"чоловік_будівельник_колір3", "1F477-1F3FD-200D-2642-FE0F"},
    {"чоловік_будівельник_колір4", "1F477-1F3FE-200D-2642-FE0F"},
    {"чоловік_будівельник_колір5", "1F477-1F3FF-200D-2642-FE0F"},
    {"жінка_що_простягає_руку_допомоги", "1F481-200D-2640-FE0F"},
    {"жінка_що_простягає_руку_допомоги_колір1", "1F481-1F3FB-200D-2640-FE0F"},
    {"жінка_що_простягає_руку_допомоги_колір2", "1F481-1F3FC-200D-2640-FE0F"},
    {"жінка_що_простягає_руку_допомоги_колір3", "1F481-1F3FD-200D-2640-FE0F"},
    {"жінка_що_простягає_руку_допомоги_колір4", "1F481-1F3FE-200D-2640-FE0F"},
    {"жінка_що_простягає_руку_допомоги_колір5", "1F481-1F3FF-200D-2640-FE0F"},
    {"чоловік_що_простягає_руку_допомоги", "1F481-200D-2642-FE0F"},
    {"чоловік_що_простягає_руку_допомоги_колір1", "1F481-1F3FB-200D-2642-FE0F"},
    {"чоловік_що_простягає_руку_допомоги_колір2", "1F481-1F3FC-200D-2642-FE0F"},
    {"чоловік_що_простягає_руку_допомоги_колір3", "1F481-1F3FD-200D-2642-FE0F"},
    {"чоловік_що_простягає_руку_допомоги_колір4", "1F481-1F3FE-200D-2642-FE0F"},
    {"чоловік_що_простягає_руку_допомоги_колір5", "1F481-1F3FF-200D-2642-FE0F"},
    {"вартова", "1F482-200D-2640-FE0F"},
    {"вартова_колір1", "1F482-1F3FB-200D-2640-FE0F"},
    {"вартова_колір2", "1F482-1F3FC-200D-2640-FE0F"},
    {"вартова_колір3", "1F482-1F3FD-200D-2640-FE0F"},
    {"вартова_колір4", "1F482-1F3FE-200D-2640-FE0F"},
    {"вартова_колір5", "1F482-1F3FF-200D-2640-FE0F"},
    {"вартовий", "1F482-200D-2642-FE0F"},
    {"вартовий_колір1", "1F482-1F3FB-200D-2642-FE0F"},
    {"вартовий_колір2", "1F482-1F3FC-200D-2642-FE0F"},
    {"вартовий_колір3", "1F482-1F3FD-200D-2642-FE0F"},
    {"вартовий_колір4", "1F482-1F3FE-200D-2642-FE0F"},
    {"вартовий_колір5", "1F482-1F3FF-200D-2642-FE0F"},
    {"жінка_якій_роблять_масаж_голови", "1F486-200D-2640-FE0F"},
    {"жінка_якій_роблять_масаж_голови_колір1", "1F486-1F3FB-200D-2640-FE0F"},
    {"жінка_якій_роблять_масаж_голови_колір2", "1F486-1F3FC-200D-2640-FE0F"},
    {"жінка_якій_роблять_масаж_голови_колір3", "1F486-1F3FD-200D-2640-FE0F"},
    {"жінка_якій_роблять_масаж_голови_колір4", "1F486-1F3FE-200D-2640-FE0F"},
    {"жінка_якій_роблять_масаж_голови_колір5", "1F486-1F3FF-200D-2640-FE0F"},
    {"чоловік_якому_роблять_масаж_голови", "1F486-200D-2642-FE0F"},
    {"чоловік_якому_роблять_масаж_голови_колір1", "1F486-1F3FB-200D-2642-FE0F"},
    {"чоловік_якому_роблять_масаж_голови_колір2", "1F486-1F3FC-200D-2642-FE0F"},
    {"чоловік_якому_роблять_масаж_голови_колір3", "1F486-1F3FD-200D-2642-FE0F"},
    {"чоловік_якому_роблять_масаж_голови_колір4", "1F486-1F3FE-200D-2642-FE0F"},
    {"чоловік_якому_роблять_масаж_голови_колір5", "1F486-1F3FF-200D-2642-FE0F"},
    {"жінка_якій_роблять_стрижку", "1F487-200D-2640-FE0F"},
    {"жінка_якій_роблять_стрижку_колір1", "1F487-1F3FB-200D-2640-FE0F"},
    {"жінка_якій_роблять_стрижку_колір2", "1F487-1F3FC-200D-2640-FE0F"},
    {"жінка_якій_роблять_стрижку_колір3", "1F487-1F3FD-200D-2640-FE0F"},
    {"жінка_якій_роблять_стрижку_колір4", "1F487-1F3FE-200D-2640-FE0F"},
    {"жінка_якій_роблять_стрижку_колір5", "1F487-1F3FF-200D-2640-FE0F"},
    {"чоловік_якому_роблять_стрижку", "1F487-200D-2642-FE0F"},
    {"чоловік_якому_роблять_стрижку_колір1", "1F487-1F3FB-200D-2642-FE0F"},
    {"чоловік_якому_роблять_стрижку_колір2", "1F487-1F3FC-200D-2642-FE0F"},
    {"чоловік_якому_роблять_стрижку_колір3", "1F487-1F3FD-200D-2642-FE0F"},
    {"чоловік_якому_роблять_стрижку_колір4", "1F487-1F3FE-200D-2642-FE0F"},
    {"чоловік_якому_роблять_стрижку_колір5", "1F487-1F3FF-200D-2642-FE0F"},
    {"жінка_детектив", "1F575-FE0F-200D-2640-FE0F"},
    {"жінка_детектив_колір1", "1F575-1F3FB-200D-2640-FE0F"},
    {"жінка_детектив_колір2", "1F575-1F3FC-200D-2640-FE0F"},
    {"жінка_детектив_колір3", "1F575-1F3FD-200D-2640-FE0F"},
    {"жінка_детектив_колір4", "1F575-1F3FE-200D-2640-FE0F"},
    {"жінка_детектив_колір5", "1F575-1F3FF-200D-2640-FE0F"},
    {"чоловік_детектив", "1F575-FE0F-200D-2642-FE0F"},
    {"чоловік_детектив_колір1", "1F575-1F3FB-200D-2642-FE0F"},
    {"чоловік_детектив_колір2", "1F575-1F3FC-200D-2642-FE0F"},
    {"чоловік_детектив_колір3", "1F575-1F3FD-200D-2642-FE0F"},
    {"чоловік_детектив_колір4", "1F575-1F3FE-200D-2642-FE0F"},
    {"чоловік_детектив_колір5", "1F575-1F3FF-200D-2642-FE0F"},
    {"жінка_що_жестом_показує_ні", "1F645-200D-2640-FE0F"},
    {"жінка_що_жестом_показує_ні_колір1", "1F645-1F3FB-200D-2640-FE0F"},
    {"жінка_що_жестом_показує_ні_колір2", "1F645-1F3FC-200D-2640-FE0F"},
    {"жінка_що_жестом_показує_ні_колір3", "1F645-1F3FD-200D-2640-FE0F"},
    {"жінка_що_жестом_показує_ні_колір4", "1F645-1F3FE-200D-2640-FE0F"},
    {"жінка_що_жестом_показує_ні_колір5", "1F645-1F3FF-200D-2640-FE0F"},
    {"чоловік_що_жестом_показує_ні", "1F645-200D-2642-FE0F"},
    {"чоловік_що_жестом_показує_ні_колір1", "1F645-1F3FB-200D-2642-FE0F"},
    {"чоловік_що_жестом_показує_ні_колір2", "1F645-1F3FC-200D-2642-FE0F"},
    {"чоловік_що_жестом_показує_ні_колір3", "1F645-1F3FD-200D-2642-FE0F"},
    {"чоловік_що_жестом_показує_ні_колір4", "1F645-1F3FE-200D-2642-FE0F"},
    {"чоловік_що_жестом_показує_ні_колір5", "1F645-1F3FF-200D-2642-FE0F"},
    {"жінка_що_жестом_показує_окей", "1F646-200D-2640-FE0F"},
    {"жінка_що_жестом_показує_окей_колір1", "1F646-1F3FB-200D-2640-FE0F"},
    {"жінка_що_жестом_показує_окей_колір2", "1F646-1F3FC-200D-2640-FE0F"},
    {"жінка_що_жестом_показує_окей_колір3", "1F646-1F3FD-200D-2640-FE0F"},
    {"жінка_що_жестом_показує_окей_колір4", "1F646-1F3FE-200D-2640-FE0F"},
    {"жінка_що_жестом_показує_окей_колір5", "1F646-1F3FF-200D-2640-FE0F"},
    {"чоловік_що_жестом_показує_окей", "1F646-200D-2642-FE0F"},
    {"чоловік_що_жестом_показує_окей_колір1", "1F646-1F3FB-200D-2642-FE0F"},
    {"чоловік_що_жестом_показує_окей_колір2", "1F646-1F3FC-200D-2642-FE0F"},
    {"чоловік_що_жестом_показує_окей_колір3", "1F646-1F3FD-200D-2642-FE0F"},
    {"чоловік_що_жестом_показує_окей_колір4", "1F646-1F3FE-200D-2642-FE0F"},
    {"чоловік_що_жестом_показує_окей_колір5", "1F646-1F3FF-200D-2642-FE0F"},
    {"жінка_що_вклоняється", "1F647-200D-2640-FE0F"},
    {"жінка_що_вклоняється_колір1", "1F647-1F3FB-200D-2640-FE0F"},
    {"жінка_що_вклоняється_колір2", "1F647-1F3FC-200D-2640-FE0F"},
    {"жінка_що_вклоняється_колір3", "1F647-1F3FD-200D-2640-FE0F"},
    {"жінка_що_вклоняється_колір4", "1F647-1F3FE-200D-2640-FE0F"},
    {"жінка_що_вклоняється_колір5", "1F647-1F3FF-200D-2640-FE0F"},
    {"чоловік_що_вклоняється", "1F647-200D-2642-FE0F"},
    {"чоловік_що_вклоняється_колір1", "1F647-1F3FB-200D-2642-FE0F"},
    {"чоловік_що_вклоняється_колір2", "1F647-1F3FC-200D-2642-FE0F"},
    {"чоловік_що_вклоняється_колір3", "1F647-1F3FD-200D-2642-FE0F"},
    {"чоловік_що_вклоняється_колір4", "1F647-1F3FE-200D-2642-FE0F"},
    {"чоловік_що_вклоняється_колір5", "1F647-1F3FF-200D-2642-FE0F"},
    {"жінка_з_піднятою_рукою", "1F64B-200D-2640-FE0F"},
    {"жінка_з_піднятою_рукою_колір1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"жінка_з_піднятою_рукою_колір2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"жінка_з_піднятою_рукою_колір3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"жінка_з_піднятою_рукою_колір4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"жінка_з_піднятою_рукою_колір5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"чоловік_із_піднятою_рукою", "1F64B-200D-2642-FE0F"},
    {"чоловік_із_піднятою_рукою_колір1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"чоловік_із_піднятою_рукою_колір2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"чоловік_із_піднятою_рукою_колір3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"чоловік_із_піднятою_рукою_колір4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"чоловік_із_піднятою_рукою_колір5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"похмура_жінка", "1F64D-200D-2640-FE0F"},
    {"похмура_жінка_колір1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"похмура_жінка_колір2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"похмура_жінка_колір3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"похмура_жінка_колір4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"похмура_жінка_колір5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"похмурий_чоловік", "1F64D-200D-2642-FE0F"},
    {"похмурий_чоловік_колір1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"похмурий_чоловік_колір2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"похмурий_чоловік_колір3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"похмурий_чоловік_колір4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"похмурий_чоловік_колір5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"жінка_що_дується", "1F64E-200D-2640-FE0F"},
    {"жінка_що_дується_колір1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"жінка_що_дується_колір2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"жінка_що_дується_колір3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"жінка_що_дується_колір4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"жінка_що_дується_колір5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"чоловік_що_дується", "1F64E-200D-2642-FE0F"},
    {"чоловік_що_дується_колір1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"чоловік_що_дується_колір2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"чоловік_що_дується_колір3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"чоловік_що_дується_колір4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"чоловік_що_дується_колір5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"жінка_що_веслує_на_човні", "1F6A3-200D-2640-FE0F"},
    {"жінка_що_веслує_на_човні_колір1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"жінка_що_веслує_на_човні_колір2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"жінка_що_веслує_на_човні_колір3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"жінка_що_веслує_на_човні_колір4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"жінка_що_веслує_на_човні_колір5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"чоловік_що_веслує_на_човні", "1F6A3-200D-2642-FE0F"},
    {"чоловік_що_веслує_на_човні_колір1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"чоловік_що_веслує_на_човні_колір2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"чоловік_що_веслує_на_човні_колір3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"чоловік_що_веслує_на_човні_колір4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"чоловік_що_веслує_на_човні_колір5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"велосипедистка", "1F6B4-200D-2640-FE0F"},
    {"велосипедистка_колір1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"велосипедистка_колір2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"велосипедистка_колір3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"велосипедистка_колір4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"велосипедистка_колір5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"велосипедист", "1F6B4-200D-2642-FE0F"},
    {"велосипедист_колір1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"велосипедист_колір2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"велосипедист_колір3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"велосипедист_колір4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"велосипедист_колір5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"гірська_велосипедистка", "1F6B5-200D-2640-FE0F"},
    {"гірська_велосипедистка_колір1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"гірська_велосипедистка_колір2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"гірська_велосипедистка_колір3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"гірська_велосипедистка_колір4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"гірська_велосипедистка_колір5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"гірський_велосипедист", "1F6B5-200D-2642-FE0F"},
    {"гірський_велосипедист_колір1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"гірський_велосипедист_колір2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"гірський_велосипедист_колір3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"гірський_велосипедист_колір4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"гірський_велосипедист_колір5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"жінка_що_йде_пішки", "1F6B6-200D-2640-FE0F"},
    {"жінка_що_йде_пішки_колір1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"жінка_що_йде_пішки_колір2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"жінка_що_йде_пішки_колір3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"жінка_що_йде_пішки_колір4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"жінка_що_йде_пішки_колір5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"жінка_що_йде_пішки_правий_профіль", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_що_йде_пішки_правий_профіль_колір1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_що_йде_пішки_правий_профіль_колір2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_що_йде_пішки_правий_профіль_колір3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_що_йде_пішки_правий_профіль_колір4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_що_йде_пішки_правий_профіль_колір5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"чоловік_що_йде_пішки", "1F6B6-200D-2642-FE0F"},
    {"чоловік_що_йде_пішки_колір1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"чоловік_що_йде_пішки_колір2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"чоловік_що_йде_пішки_колір3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"чоловік_що_йде_пішки_колір4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"чоловік_що_йде_пішки_колір5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"чоловік_що_йде_пішки_правий_профіль", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_що_йде_пішки_правий_профіль_колір1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_що_йде_пішки_правий_профіль_колір2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_що_йде_пішки_правий_профіль_колір3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_що_йде_пішки_правий_профіль_колір4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_що_йде_пішки_правий_профіль_колір5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"жінка_що_підносить_долоню_до_обличчя", "1F926-200D-2640-FE0F"},
    {"жінка_що_підносить_долоню_до_обличчя_колір1", "1F926-1F3FB-200D-2640-FE0F"},
    {"жінка_що_підносить_долоню_до_обличчя_колір2", "1F926-1F3FC-200D-2640-FE0F"},
    {"жінка_що_підносить_долоню_до_обличчя_колір3", "1F926-1F3FD-200D-2640-FE0F"},
    {"жінка_що_підносить_долоню_до_обличчя_колір4", "1F926-1F3FE-200D-2640-FE0F"},
    {"жінка_що_підносить_долоню_до_обличчя_колір5", "1F926-1F3FF-200D-2640-FE0F"},
    {"чоловік_із_долонею_до_обличчя", "1F926-200D-2642-FE0F"},
    {"чоловік_із_долонею_до_обличчя_колір1", "1F926-1F3FB-200D-2642-FE0F"},
    {"чоловік_із_долонею_до_обличчя_колір2", "1F926-1F3FC-200D-2642-FE0F"},
    {"чоловік_із_долонею_до_обличчя_колір3", "1F926-1F3FD-200D-2642-FE0F"},
    {"чоловік_із_долонею_до_обличчя_колір4", "1F926-1F3FE-200D-2642-FE0F"},
    {"чоловік_із_долонею_до_обличчя_колір5", "1F926-1F3FF-200D-2642-FE0F"},
    {"жінка_в_смокінгу", "1F935-200D-2640-FE0F"},
    {"жінка_в_смокінгу_колір1", "1F935-1F3FB-200D-2640-FE0F"},
    {"жінка_в_смокінгу_колір2", "1F935-1F3FC-200D-2640-FE0F"},
    {"жінка_в_смокінгу_колір3", "1F935-1F3FD-200D-2640-FE0F"},
    {"жінка_в_смокінгу_колір4", "1F935-1F3FE-200D-2640-FE0F"},
    {"жінка_в_смокінгу_колір5", "1F935-1F3FF-200D-2640-FE0F"},
    {"чоловік_у_смокінгу", "1F935-200D-2642-FE0F"},
    {"чоловік_у_смокінгу_колір1", "1F935-1F3FB-200D-2642-FE0F"},
    {"чоловік_у_смокінгу_колір2", "1F935-1F3FC-200D-2642-FE0F"},
    {"чоловік_у_смокінгу_колір3", "1F935-1F3FD-200D-2642-FE0F"},
    {"чоловік_у_смокінгу_колір4", "1F935-1F3FE-200D-2642-FE0F"},
    {"чоловік_у_смокінгу_колір5", "1F935-1F3FF-200D-2642-FE0F"},
    {"жінка_що_знизує_плечима", "1F937-200D-2640-FE0F"},
    {"жінка_що_знизує_плечима_колір1", "1F937-1F3FB-200D-2640-FE0F"},
    {"жінка_що_знизує_плечима_колір2", "1F937-1F3FC-200D-2640-FE0F"},
    {"жінка_що_знизує_плечима_колір3", "1F937-1F3FD-200D-2640-FE0F"},
    {"жінка_що_знизує_плечима_колір4", "1F937-1F3FE-200D-2640-FE0F"},
    {"жінка_що_знизує_плечима_колір5", "1F937-1F3FF-200D-2640-FE0F"},
    {"чоловік_що_знизує_плечима", "1F937-200D-2642-FE0F"},
    {"чоловік_що_знизує_плечима_колір1", "1F937-1F3FB-200D-2642-FE0F"},
    {"чоловік_що_знизує_плечима_колір2", "1F937-1F3FC-200D-2642-FE0F"},
    {"чоловік_що_знизує_плечима_колір3", "1F937-1F3FD-200D-2642-FE0F"},
    {"чоловік_що_знизує_плечима_колір4", "1F937-1F3FE-200D-2642-FE0F"},
    {"чоловік_що_знизує_плечима_колір5", "1F937-1F3FF-200D-2642-FE0F"},
    {"жінка_що_перевертається_колесом", "1F938-200D-2640-FE0F"},
    {"жінка_що_перевертається_колесом_колір1", "1F938-1F3FB-200D-2640-FE0F"},
    {"жінка_що_перевертається_колесом_колір2", "1F938-1F3FC-200D-2640-FE0F"},
    {"жінка_що_перевертається_колесом_колір3", "1F938-1F3FD-200D-2640-FE0F"},
    {"жінка_що_перевертається_колесом_колір4", "1F938-1F3FE-200D-2640-FE0F"},
    {"жінка_що_перевертається_колесом_колір5", "1F938-1F3FF-200D-2640-FE0F"},
    {"чоловік_що_перевертається_колесом", "1F938-200D-2642-FE0F"},
    {"чоловік_що_перевертається_колесом_колір1", "1F938-1F3FB-200D-2642-FE0F"},
    {"чоловік_що_перевертається_колесом_колір2", "1F938-1F3FC-200D-2642-FE0F"},
    {"чоловік_що_перевертається_колесом_колір3", "1F938-1F3FD-200D-2642-FE0F"},
    {"чоловік_що_перевертається_колесом_колір4", "1F938-1F3FE-200D-2642-FE0F"},
    {"чоловік_що_перевертається_колесом_колір5", "1F938-1F3FF-200D-2642-FE0F"},
    {"жонглерка", "1F939-200D-2640-FE0F"},
    {"жонглерка_колір1", "1F939-1F3FB-200D-2640-FE0F"},
    {"жонглерка_колір2", "1F939-1F3FC-200D-2640-FE0F"},
    {"жонглерка_колір3", "1F939-1F3FD-200D-2640-FE0F"},
    {"жонглерка_колір4", "1F939-1F3FE-200D-2640-FE0F"},
    {"жонглерка_колір5", "1F939-1F3FF-200D-2640-FE0F"},
    {"жонглер", "1F939-200D-2642-FE0F"},
    {"жонглер_колір1", "1F939-1F3FB-200D-2642-FE0F"},
    {"жонглер_колір2", "1F939-1F3FC-200D-2642-FE0F"},
    {"жонглер_колір3", "1F939-1F3FD-200D-2642-FE0F"},
    {"жонглер_колір4", "1F939-1F3FE-200D-2642-FE0F"},
    {"жонглер_колір5", "1F939-1F3FF-200D-2642-FE0F"},
    {"борчиня", "1F93C-200D-2640-FE0F"},
    {"борчиня_колір1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"борчиня_колір1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"борчиня_колір1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"борчиня_колір1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"борчиня_колір2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"борчиня_колір2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"борчиня_колір2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"борчиня_колір2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"борчиня_колір3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"борчиня_колір3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"борчиня_колір3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"борчиня_колір3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"борчиня_колір4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"борчиня_колір4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"борчиня_колір4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"борчиня_колір4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"борчиня_колір5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"борчиня_колір5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"борчиня_колір5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"борчиня_колір5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"борчиня_колір1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"борчиня_колір2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"борчиня_колір3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"борчиня_колір4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"борчиня_колір5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"борець", "1F93C-200D-2642-FE0F"},
    {"борець_колір1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"борець_колір1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"борець_колір1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"борець_колір1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"борець_колір2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"борець_колір2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"борець_колір2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"борець_колір2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"борець_колір3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"борець_колір3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"борець_колір3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"борець_колір3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"борець_колір4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"борець_колір4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"борець_колір4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"борець_колір4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"борець_колір5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"борець_колір5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"борець_колір5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"борець_колір5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"борець_колір1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"борець_колір2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"борець_колір3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"борець_колір4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"борець_колір5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"ватерполістка", "1F93D-200D-2640-FE0F"},
    {"ватерполістка_колір1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"ватерполістка_колір2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"ватерполістка_колір3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"ватерполістка_колір4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"ватерполістка_колір5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"ватерполіст", "1F93D-200D-2642-FE0F"},
    {"ватерполіст_колір1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"ватерполіст_колір2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"ватерполіст_колір3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"ватерполіст_колір4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"ватерполіст_колір5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"гандболістка", "1F93E-200D-2640-FE0F"},
    {"гандболістка_колір1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"гандболістка_колір2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"гандболістка_колір3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"гандболістка_колір4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"гандболістка_колір5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"гандболіст", "1F93E-200D-2642-FE0F"},
    {"гандболіст_колір1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"гандболіст_колір2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"гандболіст_колір3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"гандболіст_колір4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"гандболіст_колір5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"жінка_супергерой", "1F9B8-200D-2640-FE0F"},
    {"жінка_супергерой_колір1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"жінка_супергерой_колір2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"жінка_супергерой_колір3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"жінка_супергерой_колір4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"жінка_супергерой_колір5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"чоловік_супергерой", "1F9B8-200D-2642-FE0F"},
    {"чоловік_супергерой_колір1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"чоловік_супергерой_колір2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"чоловік_супергерой_колір3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"чоловік_супергерой_колір4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"чоловік_супергерой_колір5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"суперлиходійка", "1F9B9-200D-2640-FE0F"},
    {"суперлиходійка_колір1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"суперлиходійка_колір2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"суперлиходійка_колір3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"суперлиходійка_колір4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"суперлиходійка_колір5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"суперлиходій", "1F9B9-200D-2642-FE0F"},
    {"суперлиходій_колір1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"суперлиходій_колір2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"суперлиходій_колір3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"суперлиходій_колір4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"суперлиходій_колір5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"жінка_що_стоїть", "1F9CD-200D-2640-FE0F"},
    {"жінка_що_стоїть_колір1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"жінка_що_стоїть_колір2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"жінка_що_стоїть_колір3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"жінка_що_стоїть_колір4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"жінка_що_стоїть_колір5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"чоловік_що_стоїть", "1F9CD-200D-2642-FE0F"},
    {"чоловік_що_стоїть_колір1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"чоловік_що_стоїть_колір2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"чоловік_що_стоїть_колір3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"чоловік_що_стоїть_колір4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"чоловік_що_стоїть_колір5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"жінка_навколішках", "1F9CE-200D-2640-FE0F"},
    {"жінка_навколішках_колір1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"жінка_навколішках_колір2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"жінка_навколішках_колір3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"жінка_навколішках_колір4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"жінка_навколішках_колір5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"жінка_навколішках_правий_профіль", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_навколішках_правий_профіль_колір1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_навколішках_правий_профіль_колір2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_навколішках_правий_профіль_колір3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_навколішках_правий_профіль_колір4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"жінка_навколішках_правий_профіль_колір5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"чоловік_навколішках", "1F9CE-200D-2642-FE0F"},
    {"чоловік_навколішках_колір1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"чоловік_навколішках_колір2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"чоловік_навколішках_колір3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"чоловік_навколішках_колір4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"чоловік_навколішках_колір5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"чоловік_навколішках_правий_профіль", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_навколішках_правий_профіль_колір1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_навколішках_правий_профіль_колір2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_навколішках_правий_профіль_колір3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_навколішках_правий_профіль_колір4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"чоловік_навколішках_правий_профіль_колір5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"жінка_з_порушенням_слуху", "1F9CF-200D-2640-FE0F"},
    {"жінка_з_порушенням_слуху_колір1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"жінка_з_порушенням_слуху_колір2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"жінка_з_порушенням_слуху_колір3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"жінка_з_порушенням_слуху_колір4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"жінка_з_порушенням_слуху_колір5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"чоловік_із_порушенням_слуху", "1F9CF-200D-2642-FE0F"},
    {"чоловік_із_порушенням_слуху_колір1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"чоловік_із_порушенням_слуху_колір2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"чоловік_із_порушенням_слуху_колір3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"чоловік_із_порушенням_слуху_колір4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"чоловік_із_порушенням_слуху_колір5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"жінка_з_бородою", "1F9D4-200D-2640-FE0F"},
    {"жінка_з_бородою_колір1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"жінка_з_бородою_колір2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"жінка_з_бородою_колір3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"жінка_з_бородою_колір4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"жінка_з_бородою_колір5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"чоловік_із_бородою", "1F9D4-200D-2642-FE0F"},
    {"чоловік_із_бородою_колір1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"чоловік_із_бородою_колір2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"чоловік_із_бородою_колір3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"чоловік_із_бородою_колір4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"чоловік_із_бородою_колір5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"жінка_в_паркому_приміщенні", "1F9D6-200D-2640-FE0F"},
    {"жінка_в_паркому_приміщенні_колір1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"жінка_в_паркому_приміщенні_колір2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"жінка_в_паркому_приміщенні_колір3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"жінка_в_паркому_приміщенні_колір4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"жінка_в_паркому_приміщенні_колір5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"чоловік_у_паркому_приміщенні", "1F9D6-200D-2642-FE0F"},
    {"чоловік_у_паркому_приміщенні_колір1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"чоловік_у_паркому_приміщенні_колір2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"чоловік_у_паркому_приміщенні_колір3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"чоловік_у_паркому_приміщенні_колір4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"чоловік_у_паркому_приміщенні_колір5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"жінка_скелелаз", "1F9D7-200D-2640-FE0F"},
    {"жінка_скелелаз_колір1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"жінка_скелелаз_колір2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"жінка_скелелаз_колір3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"жінка_скелелаз_колір4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"жінка_скелелаз_колір5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"чоловік_скелелаз", "1F9D7-200D-2642-FE0F"},
    {"чоловік_скелелаз_колір1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"чоловік_скелелаз_колір2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"чоловік_скелелаз_колір3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"чоловік_скелелаз_колір4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"чоловік_скелелаз_колір5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"жінка_в_позі_лотоса", "1F9D8-200D-2640-FE0F"},
    {"жінка_в_позі_лотоса_колір1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"жінка_в_позі_лотоса_колір2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"жінка_в_позі_лотоса_колір3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"жінка_в_позі_лотоса_колір4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"жінка_в_позі_лотоса_колір5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"чоловік_у_позі_лотоса", "1F9D8-200D-2642-FE0F"},
    {"чоловік_у_позі_лотоса_колір1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"чоловік_у_позі_лотоса_колір2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"чоловік_у_позі_лотоса_колір3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"чоловік_у_позі_лотоса_колір4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"чоловік_у_позі_лотоса_колір5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"чаклунка", "1F9D9-200D-2640-FE0F"},
    {"чаклунка_колір1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"чаклунка_колір2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"чаклунка_колір3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"чаклунка_колір4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"чаклунка_колір5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"чаклун", "1F9D9-200D-2642-FE0F"},
    {"чаклун_колір1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"чаклун_колір2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"чаклун_колір3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"чаклун_колір4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"чаклун_колір5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"жінка_фея", "1F9DA-200D-2640-FE0F"},
    {"жінка_фея_колір1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"жінка_фея_колір2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"жінка_фея_колір3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"жінка_фея_колір4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"жінка_фея_колір5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"чоловік_фея", "1F9DA-200D-2642-FE0F"},
    {"чоловік_фея_колір1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"чоловік_фея_колір2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"чоловік_фея_колір3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"чоловік_фея_колір4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"чоловік_фея_колір5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"жінка_вампір", "1F9DB-200D-2640-FE0F"},
    {"жінка_вампір_колір1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"жінка_вампір_колір2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"жінка_вампір_колір3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"жінка_вампір_колір4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"жінка_вампір_колір5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"чоловік_вампір", "1F9DB-200D-2642-FE0F"},
    {"чоловік_вампір_колір1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"чоловік_вампір_колір2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"чоловік_вампір_колір3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"чоловік_вампір_колір4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"чоловік_вампір_колір5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"русалка", "1F9DC-200D-2640-FE0F"},
    {"русалка_колір1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"русалка_колір2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"русалка_колір3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"русалка_колір4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"русалка_колір5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"чоловік_русалка", "1F9DC-200D-2642-FE0F"},
    {"чоловік_русалка_колір1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"чоловік_русалка_колір2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"чоловік_русалка_колір3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"чоловік_русалка_колір4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"чоловік_русалка_колір5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"жінка_ельф", "1F9DD-200D-2640-FE0F"},
    {"жінка_ельф_колір1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"жінка_ельф_колір2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"жінка_ельф_колір3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"жінка_ельф_колір4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"жінка_ельф_колір5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"чоловік_ельф", "1F9DD-200D-2642-FE0F"},
    {"чоловік_ельф_колір1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"чоловік_ельф_колір2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"чоловік_ельф_колір3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"чоловік_ельф_колір4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"чоловік_ельф_колір5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"жінка_джин", "1F9DE-200D-2640-FE0F"},
    {"чоловік_джин", "1F9DE-200D-2642-FE0F"},
    {"жінка_зомбі", "1F9DF-200D-2640-FE0F"},
    {"чоловік_зомбі", "1F9DF-200D-2642-FE0F"},
    {"чоловік_руде_волосся", "1F468-200D-1F9B0"},
    {"чоловік_руде_волосся_колір1", "1F468-1F3FB-200D-1F9B0"},
    {"чоловік_руде_волосся_колір2", "1F468-1F3FC-200D-1F9B0"},
    {"чоловік_руде_волосся_колір3", "1F468-1F3FD-200D-1F9B0"},
    {"чоловік_руде_волосся_колір4", "1F468-1F3FE-200D-1F9B0"},
    {"чоловік_руде_волосся_колір5", "1F468-1F3FF-200D-1F9B0"},
    {"чоловік_кучеряве_волосся", "1F468-200D-1F9B1"},
    {"чоловік_кучеряве_волосся_колір1", "1F468-1F3FB-200D-1F9B1"},
    {"чоловік_кучеряве_волосся_колір2", "1F468-1F3FC-200D-1F9B1"},
    {"чоловік_кучеряве_волосся_колір3", "1F468-1F3FD-200D-1F9B1"},
    {"чоловік_кучеряве_волосся_колір4", "1F468-1F3FE-200D-1F9B1"},
    {"чоловік_кучеряве_волосся_колір5", "1F468-1F3FF-200D-1F9B1"},
    {"чоловік_лисина", "1F468-200D-1F9B2"},
    {"чоловік_лисина_колір1", "1F468-1F3FB-200D-1F9B2"},
    {"чоловік_лисина_колір2", "1F468-1F3FC-200D-1F9B2"},
    {"чоловік_лисина_колір3", "1F468-1F3FD-200D-1F9B2"},
    {"чоловік_лисина_колір4", "1F468-1F3FE-200D-1F9B2"},
    {"чоловік_лисина_колір5", "1F468-1F3FF-200D-1F9B2"},
    {"чоловік_сиве_волосся", "1F468-200D-1F9B3"},
    {"чоловік_сиве_волосся_колір1", "1F468-1F3FB-200D-1F9B3"},
    {"чоловік_сиве_волосся_колір2", "1F468-1F3FC-200D-1F9B3"},
    {"чоловік_сиве_волосся_колір3", "1F468-1F3FD-200D-1F9B3"},
    {"чоловік_сиве_волосся_колір4", "1F468-1F3FE-200D-1F9B3"},
    {"чоловік_сиве_волосся_колір5", "1F468-1F3FF-200D-1F9B3"},
    {"жінка_руде_волосся", "1F469-200D-1F9B0"},
    {"жінка_руде_волосся_колір1", "1F469-1F3FB-200D-1F9B0"},
    {"жінка_руде_волосся_колір2", "1F469-1F3FC-200D-1F9B0"},
    {"жінка_руде_волосся_колір3", "1F469-1F3FD-200D-1F9B0"},
    {"жінка_руде_волосся_колір4", "1F469-1F3FE-200D-1F9B0"},
    {"жінка_руде_волосся_колір5", "1F469-1F3FF-200D-1F9B0"},
    {"жінка_кучеряве_волосся", "1F469-200D-1F9B1"},
    {"жінка_кучеряве_волосся_колір1", "1F469-1F3FB-200D-1F9B1"},
    {"жінка_кучеряве_волосся_колір2", "1F469-1F3FC-200D-1F9B1"},
    {"жінка_кучеряве_волосся_колір3", "1F469-1F3FD-200D-1F9B1"},
    {"жінка_кучеряве_волосся_колір4", "1F469-1F3FE-200D-1F9B1"},
    {"жінка_кучеряве_волосся_колір5", "1F469-1F3FF-200D-1F9B1"},
    {"жінка_лисина", "1F469-200D-1F9B2"},
    {"жінка_лисина_колір1", "1F469-1F3FB-200D-1F9B2"},
    {"жінка_лисина_колір2", "1F469-1F3FC-200D-1F9B2"},
    {"жінка_лисина_колір3", "1F469-1F3FD-200D-1F9B2"},
    {"жінка_лисина_колір4", "1F469-1F3FE-200D-1F9B2"},
    {"жінка_лисина_колір5", "1F469-1F3FF-200D-1F9B2"},
    {"жінка_сиве_волосся", "1F469-200D-1F9B3"},
    {"жінка_сиве_волосся_колір1", "1F469-1F3FB-200D-1F9B3"},
    {"жінка_сиве_волосся_колір2", "1F469-1F3FC-200D-1F9B3"},
    {"жінка_сиве_волосся_колір3", "1F469-1F3FD-200D-1F9B3"},
    {"жінка_сиве_волосся_колір4", "1F469-1F3FE-200D-1F9B3"},
    {"жінка_сиве_волосся_колір5", "1F469-1F3FF-200D-1F9B3"},
    {"людина_руде_волосся", "1F9D1-200D-1F9B0"},
    {"людина_руде_волосся_колір1", "1F9D1-1F3FB-200D-1F9B0"},
    {"людина_руде_волосся_колір2", "1F9D1-1F3FC-200D-1F9B0"},
    {"людина_руде_волосся_колір3", "1F9D1-1F3FD-200D-1F9B0"},
    {"людина_руде_волосся_колір4", "1F9D1-1F3FE-200D-1F9B0"},
    {"людина_руде_волосся_колір5", "1F9D1-1F3FF-200D-1F9B0"},
    {"людина_кучеряве_волосся", "1F9D1-200D-1F9B1"},
    {"людина_кучеряве_волосся_колір1", "1F9D1-1F3FB-200D-1F9B1"},
    {"людина_кучеряве_волосся_колір2", "1F9D1-1F3FC-200D-1F9B1"},
    {"людина_кучеряве_волосся_колір3", "1F9D1-1F3FD-200D-1F9B1"},
    {"людина_кучеряве_волосся_колір4", "1F9D1-1F3FE-200D-1F9B1"},
    {"людина_кучеряве_волосся_колір5", "1F9D1-1F3FF-200D-1F9B1"},
    {"людина_лисина", "1F9D1-200D-1F9B2"},
    {"людина_лисина_колір1", "1F9D1-1F3FB-200D-1F9B2"},
    {"людина_лисина_колір2", "1F9D1-1F3FC-200D-1F9B2"},
    {"людина_лисина_колір3", "1F9D1-1F3FD-200D-1F9B2"},
    {"людина_лисина_колір4", "1F9D1-1F3FE-200D-1F9B2"},
    {"людина_лисина_колір5", "1F9D1-1F3FF-200D-1F9B2"},
    {"людина_сиве_волосся", "1F9D1-200D-1F9B3"},
    {"людина_сиве_волосся_колір1", "1F9D1-1F3FB-200D-1F9B3"},
    {"людина_сиве_волосся_колір2", "1F9D1-1F3FC-200D-1F9B3"},
    {"людина_сиве_волосся_колір3", "1F9D1-1F3FD-200D-1F9B3"},
    {"людина_сиве_волосся_колір4", "1F9D1-1F3FE-200D-1F9B3"},
    {"людина_сиве_волосся_колір5", "1F9D1-1F3FF-200D-1F9B3"},
    {"розірваний_ланцюг", "26D3-FE0F-200D-1F4A5"},
    {"палке_серце", "2764-FE0F-200D-1F525"},
    {"серце_що_заживає", "2764-FE0F-200D-1FA79"},
    {"коричневий_гриб", "1F344-200D-1F7EB"},
    {"лайм", "1F34B-200D-1F7E9"},
    {"прапор_трансгендерів", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"веселковий_прапор", "1F3F3-FE0F-200D-1F308"},
    {"піратський_прапор", "1F3F4-200D-2620-FE0F"},
    {"чорний_кіт", "1F408-200D-2B1B"},
    {"службовий_собака", "1F415-200D-1F9BA"},
    {"чорний_птах", "1F426-200D-2B1B"},
    {"фенікс", "1F426-200D-1F525"},
    {"полярний_ведмідь", "1F43B-200D-2744-FE0F"},
    {"око_в_текстовій_бульці", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"обличчя_що_видихає", "1F62E-200D-1F4A8"},
    {"очі_спіралі", "1F635-200D-1F4AB"},
    {"обличчя_в_хмарах", "1F636-200D-1F32B-FE0F"},
    {"заперечливе_хитання_головою", "1F642-200D-2194-FE0F"},
    {"кивання_головою", "1F642-200D-2195-FE0F"},
    {"танцівник_балету", "1F9D1-200D-1FA70"},
    {"танцівник_балету_колір1", "1F9D1-1F3FB-200D-1FA70"},
    {"танцівник_балету_колір2", "1F9D1-1F3FC-200D-1FA70"},
    {"танцівник_балету_колір3", "1F9D1-1F3FD-200D-1FA70"},
    {"танцівник_балету_колір4", "1F9D1-1F3FE-200D-1FA70"},
    {"танцівник_балету_колір5", "1F9D1-1F3FF-200D-1FA70"},
};

