// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_nb[] = {
    {"utropstegn_og_sporsmalstegn", "2049"},
    {"varemerke", "2122"},
    {"informasjon", "2139"},
    {"pil_venstre_hoyre", "2194"},
    {"pil_opp_og_ned", "2195"},
    {"pil_opp_venstre", "2196"},
    {"pil_opp_hoyre", "2197"},
    {"pil_ned_hoyre", "2198"},
    {"pil_ned_venstre", "2199"},
    {"tastatur", "2328"},
    {"sol", "2600"},
    {"sky", "2601"},
    {"paraply", "2602"},
    {"snomann", "2603"},
    {"komet", "2604"},
    {"boks_med_hakemerke", "2611"},
    {"paraply_med_regndraper", "2614"},
    {"varm_drikke", "2615"},
    {"treklover", "2618"},
    {"dodningehode", "2620"},
    {"radioaktivt", "2622"},
    {"farlig_biologisk_materiale", "2623"},
    {"ortodokst_kors", "2626"},
    {"dharmas_hjul", "2638"},
    {"surt_fjes", "2639"},
    {"kvinnesymbol", "2640"},
    {"mannssymbol", "2642"},
    {"vaeren", "2648"},
    {"tyren", "2649"},
    {"skytten", "2650"},
    {"steinbukken", "2651"},
    {"vannmannen", "2652"},
    {"fiskene", "2653"},
    {"spar", "2660"},
    {"klover", "2663"},
    {"hjerter", "2665"},
    {"ruter", "2666"},
    {"varme_kilder", "2668"},
    {"hammer_og_hakke", "2692"},
    {"anker", "2693"},
    {"sverd_i_kryss", "2694"},
    {"legesymbol", "2695"},
    {"likearmet_skalvekt", "2696"},
    {"destillerkolbe", "2697"},
    {"tannhjul", "2699"},
    {"saks", "2702"},
    {"hakemerke_i_firkant", "2705"},
    {"fly", "2708"},
    {"konvolutt", "2709"},
    {"svart_pennesplitt", "2712"},
    {"hakemerke", "2714"},
    {"gange", "2716"},
    {"davidstjerne", "2721"},
    {"stjerner", "2728"},
    {"asterisk_med_atte_spisser", "2733"},
    {"attetagget_stjerne", "2734"},
    {"snofnugg", "2744"},
    {"gnist", "2747"},
    {"rodt_sporsmalstegn", "2753"},
    {"hvitt_sporsmalstegn", "2754"},
    {"hvitt_utropstegn", "2755"},
    {"rodt_utropstegn", "2757"},
    {"utropstegn_med_hjerte", "2763"},
    {"rodt_hjerte", "2764"},
    {"pluss", "2795"},
    {"minus", "2796"},
    {"dele", "2797"},
    {"pil_som_boyer_opp", "2934"},
    {"pil_som_boyer_ned", "2935"},
    {"bolgestrek", "3030"},
    {"gratulerer_pa_japansk", "3297"},
    {"hemmelig_pa_japansk", "3299"},
    {"opphavsrett", "00A9"},
    {"registrert_varemerke", "00AE"},
    {"dobbelt_utropstegn", "203C"},
    {"pil_som_boyer_mot_venstre", "21A9"},
    {"pil_som_boyer_mot_hoyre", "21AA"},
    {"armbandsur", "231A"},
    {"ferdig_timeglass", "231B"},
    {"mat_ut_knapp", "23CF"},
    {"spol_framover_knapp", "23E9"},
    {"spol_tilbake_knapp", "23EA"},
    {"raskt_opp_knapp", "23EB"},
    {"raskt_ned_knapp", "23EC"},
    {"neste_spor_knapp", "23ED"},
    {"forrige_spor_knapp", "23EE"},
    {"spill_av_pause_knapp", "23EF"},
    {"vekkerklokke", "23F0"},
    {"stoppeklokke", "23F1"},
    {"tidtaker", "23F2"},
    {"startet_timeglass", "23F3"},
    {"pause_knapp", "23F8"},
    {"stopp_knapp", "23F9"},
    {"spill_inn_knapp", "23FA"},
    {"m_med_ring_rundt", "24C2"},
    {"lite_svart_kvadrat", "25AA"},
    {"lite_hvitt_kvadrat", "25AB"},
    {"avspillingsknapp", "25B6"},
    {"bakover_knapp", "25C0"},
    {"mellomstort_hvitt_kvadrat", "25FB"},
    {"mellomstort_svart_kvadrat", "25FC"},
    {"lite_mellomstort_hvitt_kvadrat", "25FD"},
    {"lite_mellomstort_svart_kvadrat", "25FE"},
    {"telefon", "260E"},
    {"peker_opp", "261D"},
    {"peker_opp_farge1", "261D-1F3FB"},
    {"peker_opp_farge2", "261D-1F3FC"},
    {"peker_opp_farge3", "261D-1F3FD"},
    {"peker_opp_farge4", "261D-1F3FE"},
    {"peker_opp_farge5", "261D-1F3FF"},
    {"stjerne_og_halvmane", "262A"},
    {"fredssymbol", "262E"},
    {"yin_yang", "262F"},
    {"smilefjes", "263A"},
    {"tvillingene", "264A"},
    {"krepsen", "264B"},
    {"loven", "264C"},
    {"jomfruen", "264D"},
    {"vekten", "264E"},
    {"skorpionen", "264F"},
    {"bonde", "265F"},
    {"resirkuleringssymbol", "267B"},
    {"uendelig", "267E"},
    {"rullestolsymbol", "267F"},
    {"atomsymbol", "269B"},
    {"fransk_lilje", "269C"},
    {"advarsel", "26A0"},
    {"hoyspenning", "26A1"},
    {"transperson_symbol", "26A7"},
    {"hvit_sirkel", "26AA"},
    {"svart_sirkel", "26AB"},
    {"kiste", "26B0"},
    {"urne", "26B1"},
    {"fotball", "26BD"},
    {"baseball", "26BE"},
    {"snomann_uten_sno", "26C4"},
    {"sol_bak_sky", "26C5"},
    {"sky_med_lyn_og_regn", "26C8"},
    {"slangebaereren", "26CE"},
    {"hakke", "26CF"},
    {"hjelm_med_hvitt_kors", "26D1"},
    {"kjetting", "26D3"},
    {"ingen_adgang", "26D4"},
    {"shinto_alter", "26E9"},
    {"kirke", "26EA"},
    {"fjell", "26F0"},
    {"parasoll_pa_bakken", "26F1"},
    {"fontene", "26F2"},
    {"golfflagg", "26F3"},
    {"ferge", "26F4"},
    {"seilbat", "26F5"},
    {"skiloper", "26F7"},
    {"skoyter", "26F8"},
    {"person_med_ball", "26F9"},
    {"person_med_ball_farge1", "26F9-1F3FB"},
    {"person_med_ball_farge2", "26F9-1F3FC"},
    {"person_med_ball_farge3", "26F9-1F3FD"},
    {"person_med_ball_farge4", "26F9-1F3FE"},
    {"person_med_ball_farge5", "26F9-1F3FF"},
    {"telt", "26FA"},
    {"bensinpumpe", "26FD"},
    {"hevet_knyttneve", "270A"},
    {"hevet_knyttneve_farge1", "270A-1F3FB"},
    {"hevet_knyttneve_farge2", "270A-1F3FC"},
    {"hevet_knyttneve_farge3", "270A-1F3FD"},
    {"hevet_knyttneve_farge4", "270A-1F3FE"},
    {"hevet_knyttneve_farge5", "270A-1F3FF"},
    {"hevet_hand", "270B"},
    {"hevet_hand_farge1", "270B-1F3FB"},
    {"hevet_hand_farge2", "270B-1F3FC"},
    {"hevet_hand_farge3", "270B-1F3FD"},
    {"hevet_hand_farge4", "270B-1F3FE"},
    {"hevet_hand_farge5", "270B-1F3FF"},
    {"seierstegn", "270C"},
    {"seierstegn_farge1", "270C-1F3FB"},
    {"seierstegn_farge2", "270C-1F3FC"},
    {"seierstegn_farge3", "270C-1F3FD"},
    {"seierstegn_farge4", "270C-1F3FE"},
    {"seierstegn_farge5", "270C-1F3FF"},
    {"skrivende_hand", "270D"},
    {"skrivende_hand_farge1", "270D-1F3FB"},
    {"skrivende_hand_farge2", "270D-1F3FC"},
    {"skrivende_hand_farge3", "270D-1F3FD"},
    {"skrivende_hand_farge4", "270D-1F3FE"},
    {"skrivende_hand_farge5", "270D-1F3FF"},
    {"blyant", "270F"},
    {"latinsk_kors", "271D"},
    {"kryss", "274C"},
    {"kryss_i_firkant", "274E"},
    {"pil_hoyre", "27A1"},
    {"lokke", "27B0"},
    {"dobbel_lokke", "27BF"},
    {"pil_venstre", "2B05"},
    {"pil_opp", "2B06"},
    {"pil_ned", "2B07"},
    {"stort_svart_kvadrat", "2B1B"},
    {"stort_hvitt_kvadrat", "2B1C"},
    {"stjerne", "2B50"},
    {"tykk_rod_sirkel", "2B55"},
    {"merke_for_nytt_parti", "303D"},
    {"mahjongbrikke_med_rod_drage", "1F004"},
    {"joker", "1F0CF"},
    {"a_knapp_blodtype", "1F170"},
    {"b_knapp_blodtype", "1F171"},
    {"o_knapp_blodtype", "1F17E"},
    {"p_knapp", "1F17F"},
    {"ab_knapp_blodtype", "1F18E"},
    {"cl_knapp", "1F191"},
    {"cool_knapp", "1F192"},
    {"free_knapp", "1F193"},
    {"id_knapp", "1F194"},
    {"new_knapp", "1F195"},
    {"ng_knapp", "1F196"},
    {"ok_knapp", "1F197"},
    {"sos_knapp", "1F198"},
    {"up_knapp", "1F199"},
    {"vs_knapp", "1F19A"},
    {"regional_indikator_a", "1F1E6"},
    {"regional_indikator_b", "1F1E7"},
    {"regional_indikator_c", "1F1E8"},
    {"regional_indikator_d", "1F1E9"},
    {"regional_indikator_e", "1F1EA"},
    {"regional_indikator_f", "1F1EB"},
    {"regional_indikator_g", "1F1EC"},
    {"regional_indikator_h", "1F1ED"},
    {"regional_indikator_i", "1F1EE"},
    {"regional_indikator_j", "1F1EF"},
    {"regional_indikator_k", "1F1F0"},
    {"regional_indikator_l", "1F1F1"},
    {"regional_indikator_m", "1F1F2"},
    {"regional_indikator_n", "1F1F3"},
    {"regional_indikator_o", "1F1F4"},
    {"regional_indikator_p", "1F1F5"},
    {"regional_indikator_q", "1F1F6"},
    {"regional_indikator_r", "1F1F7"},
    {"regional_indikator_s", "1F1F8"},
    {"regional_indikator_t", "1F1F9"},
    {"regional_indikator_u", "1F1FA"},
    {"regional_indikator_v", "1F1FB"},
    {"regional_indikator_w", "1F1FC"},
    {"regional_indikator_x", "1F1FD"},
    {"regional_indikator_y", "1F1FE"},
    {"regional_indikator_z", "1F1FF"},
    {"her_pa_japansk", "1F201"},
    {"gebyr_pa_japansk", "1F202"},
    {"gratis_pa_japansk", "1F21A"},
    {"reservert_pa_japansk", "1F22F"},
    {"forbudt_pa_japansk", "1F232"},
    {"ledig_pa_japansk", "1F233"},
    {"bestatt_pa_japansk", "1F234"},
    {"fullt_pa_japansk", "1F235"},
    {"ikke_gratis_pa_japansk", "1F236"},
    {"manedlig_belop_pa_japansk", "1F237"},
    {"soknad_pa_japansk", "1F238"},
    {"rabatt_pa_japansk", "1F239"},
    {"apen_for_handel_pa_japansk", "1F23A"},
    {"spesialtilbud_pa_japansk", "1F250"},
    {"akseptabelt_pa_japansk", "1F251"},
    {"syklon", "1F300"},
    {"takete", "1F301"},
    {"lukket_paraply", "1F302"},
    {"stjernenatt", "1F303"},
    {"soloppgang_over_fjell", "1F304"},
    {"soloppgang", "1F305"},
    {"bylandskap_i_skumring", "1F306"},
    {"solnedgang", "1F307"},
    {"regnbue", "1F308"},
    {"bro_om_natten", "1F309"},
    {"bolge", "1F30A"},
    {"vulkan", "1F30B"},
    {"melkeveien", "1F30C"},
    {"jordklode_med_europa_og_afrika", "1F30D"},
    {"jordklode_med_amerika", "1F30E"},
    {"jordklode_med_asia_og_australia", "1F30F"},
    {"jordklode_med_meridianer", "1F310"},
    {"nymane", "1F311"},
    {"voksende_nymane", "1F312"},
    {"voksende_halvmane", "1F313"},
    {"voksende_fullmane", "1F314"},
    {"fullmane", "1F315"},
    {"avtakende_fullmane", "1F316"},
    {"avtakende_halvmane", "1F317"},
    {"avtakende_nemane", "1F318"},
    {"manesigd", "1F319"},
    {"nymaneansikt", "1F31A"},
    {"nymane_med_ansikt", "1F31B"},
    {"nemane_med_ansikt", "1F31C"},
    {"fullmaneansikt", "1F31D"},
    {"solansikt", "1F31E"},
    {"lysende_stjerne", "1F31F"},
    {"stjerneskudd", "1F320"},
    {"termometer", "1F321"},
    {"sol_bak_liten_sky", "1F324"},
    {"sol_bak_stor_sky", "1F325"},
    {"sol_bak_sky_med_regn", "1F326"},
    {"sky_med_regn", "1F327"},
    {"sky_med_sno", "1F328"},
    {"sky_med_lyn", "1F329"},
    {"tornado", "1F32A"},
    {"take", "1F32B"},
    {"vindansikt", "1F32C"},
    {"polse_med_brod", "1F32D"},
    {"taco", "1F32E"},
    {"burrito", "1F32F"},
    {"kastanje", "1F330"},
    {"spire", "1F331"},
    {"bartre", "1F332"},
    {"lovtre", "1F333"},
    {"palme", "1F334"},
    {"kaktus", "1F335"},
    {"rod_chili", "1F336"},
    {"tulipan", "1F337"},
    {"kirsebaerblomst", "1F338"},
    {"rose", "1F339"},
    {"hibiskus", "1F33A"},
    {"solsikke", "1F33B"},
    {"blomst", "1F33C"},
    {"maiskolbe", "1F33D"},
    {"risplante", "1F33E"},
    {"urteblader", "1F33F"},
    {"firklover", "1F340"},
    {"lonneblad", "1F341"},
    {"hostblad", "1F342"},
    {"blader_i_vind", "1F343"},
    {"sopp", "1F344"},
    {"tomat", "1F345"},
    {"aubergine", "1F346"},
    {"druer", "1F347"},
    {"melon", "1F348"},
    {"vannmelon", "1F349"},
    {"mandarin", "1F34A"},
    {"sitron", "1F34B"},
    {"banan", "1F34C"},
    {"ananas", "1F34D"},
    {"rodt_eple", "1F34E"},
    {"gront_eple", "1F34F"},
    {"paere", "1F350"},
    {"fersken", "1F351"},
    {"kirsebaer", "1F352"},
    {"jordbaer", "1F353"},
    {"hamburger", "1F354"},
    {"pizzastykke", "1F355"},
    {"kjott_med_bein", "1F356"},
    {"kyllinglar", "1F357"},
    {"riskjeks", "1F358"},
    {"risball", "1F359"},
    {"kokt_ris", "1F35A"},
    {"karriris", "1F35B"},
    {"matbolle_som_damper", "1F35C"},
    {"spagetti", "1F35D"},
    {"brod", "1F35E"},
    {"pommes_frites", "1F35F"},
    {"bakt_sotpotet", "1F360"},
    {"dango", "1F361"},
    {"oden", "1F362"},
    {"sushi", "1F363"},
    {"fritert_reke", "1F364"},
    {"fiskekake_med_monster", "1F365"},
    {"soft_is", "1F366"},
    {"slush_is", "1F367"},
    {"iskrem", "1F368"},
    {"donut", "1F369"},
    {"kjeks", "1F36A"},
    {"sjokoladeplate", "1F36B"},
    {"godteri", "1F36C"},
    {"kjaerlighet_pa_pinne", "1F36D"},
    {"pudding", "1F36E"},
    {"honningkrukke", "1F36F"},
    {"kakestykke", "1F370"},
    {"bento_boks", "1F371"},
    {"gryterett", "1F372"},
    {"matlaging", "1F373"},
    {"kniv_og_gaffel", "1F374"},
    {"tekopp", "1F375"},
    {"sake", "1F376"},
    {"vinglass", "1F377"},
    {"cocktailglass", "1F378"},
    {"tropisk_drink", "1F379"},
    {"olkrus", "1F37A"},
    {"olkrus_som_skaler", "1F37B"},
    {"tateflaske", "1F37C"},
    {"tallerken_med_kniv_og_gaffel", "1F37D"},
    {"flaske_med_sprettkork", "1F37E"},
    {"popkorn", "1F37F"},
    {"sloyfe", "1F380"},
    {"gave", "1F381"},
    {"bursdagskake", "1F382"},
    {"gresskarlykt", "1F383"},
    {"juletre", "1F384"},
    {"julenissen", "1F385"},
    {"julenissen_farge1", "1F385-1F3FB"},
    {"julenissen_farge2", "1F385-1F3FC"},
    {"julenissen_farge3", "1F385-1F3FD"},
    {"julenissen_farge4", "1F385-1F3FE"},
    {"julenissen_farge5", "1F385-1F3FF"},
    {"fyrverkeri", "1F386"},
    {"stjerneskudd_pa_pinne", "1F387"},
    {"ballong", "1F388"},
    {"smell_bon_bon", "1F389"},
    {"konfettiball", "1F38A"},
    {"tanabatatre", "1F38B"},
    {"kryssede_flagg", "1F38C"},
    {"furudekorasjon", "1F38D"},
    {"japanske_dukker", "1F38E"},
    {"koinobori", "1F38F"},
    {"vindspill", "1F390"},
    {"manebeskuelsesseremoni", "1F391"},
    {"skolesekk", "1F392"},
    {"studentlue", "1F393"},
    {"militaermedalje", "1F396"},
    {"minnesloyfe", "1F397"},
    {"studiomikrofon", "1F399"},
    {"glidebryter", "1F39A"},
    {"kontrollbrytere", "1F39B"},
    {"filmruter", "1F39E"},
    {"inngangsbilletter", "1F39F"},
    {"karusellhest", "1F3A0"},
    {"pariserhjul", "1F3A1"},
    {"berg_og_dal_bane", "1F3A2"},
    {"fiskestang", "1F3A3"},
    {"mikrofon", "1F3A4"},
    {"filmkamera", "1F3A5"},
    {"kino", "1F3A6"},
    {"hodetelefoner", "1F3A7"},
    {"malerpalett", "1F3A8"},
    {"flosshatt", "1F3A9"},
    {"sirkustelt", "1F3AA"},
    {"billett", "1F3AB"},
    {"filmklapper", "1F3AC"},
    {"teatermasker", "1F3AD"},
    {"videospill", "1F3AE"},
    {"blink", "1F3AF"},
    {"spilleautomat", "1F3B0"},
    {"biljard", "1F3B1"},
    {"terning", "1F3B2"},
    {"bowling", "1F3B3"},
    {"spillkort", "1F3B4"},
    {"note", "1F3B5"},
    {"noter", "1F3B6"},
    {"saksofon", "1F3B7"},
    {"gitar", "1F3B8"},
    {"piano", "1F3B9"},
    {"trompet", "1F3BA"},
    {"fiolin", "1F3BB"},
    {"musikk", "1F3BC"},
    {"treningstroye", "1F3BD"},
    {"tennis", "1F3BE"},
    {"ski", "1F3BF"},
    {"basketball", "1F3C0"},
    {"rutete_flagg", "1F3C1"},
    {"snowboarder", "1F3C2"},
    {"snowboarder_farge1", "1F3C2-1F3FB"},
    {"snowboarder_farge2", "1F3C2-1F3FC"},
    {"snowboarder_farge3", "1F3C2-1F3FD"},
    {"snowboarder_farge4", "1F3C2-1F3FE"},
    {"snowboarder_farge5", "1F3C2-1F3FF"},
    {"loper", "1F3C3"},
    {"loper_farge1", "1F3C3-1F3FB"},
    {"loper_farge2", "1F3C3-1F3FC"},
    {"loper_farge3", "1F3C3-1F3FD"},
    {"loper_farge4", "1F3C3-1F3FE"},
    {"loper_farge5", "1F3C3-1F3FF"},
    {"surfer", "1F3C4"},
    {"surfer_farge1", "1F3C4-1F3FB"},
    {"surfer_farge2", "1F3C4-1F3FC"},
    {"surfer_farge3", "1F3C4-1F3FD"},
    {"surfer_farge4", "1F3C4-1F3FE"},
    {"surfer_farge5", "1F3C4-1F3FF"},
    {"idrettsmedalje", "1F3C5"},
    {"trofe", "1F3C6"},
    {"hesteveddelop", "1F3C7"},
    {"hesteveddelop_farge1", "1F3C7-1F3FB"},
    {"hesteveddelop_farge2", "1F3C7-1F3FC"},
    {"hesteveddelop_farge3", "1F3C7-1F3FD"},
    {"hesteveddelop_farge4", "1F3C7-1F3FE"},
    {"hesteveddelop_farge5", "1F3C7-1F3FF"},
    {"amerikansk_fotball", "1F3C8"},
    {"rugby", "1F3C9"},
    {"svommer", "1F3CA"},
    {"svommer_farge1", "1F3CA-1F3FB"},
    {"svommer_farge2", "1F3CA-1F3FC"},
    {"svommer_farge3", "1F3CA-1F3FD"},
    {"svommer_farge4", "1F3CA-1F3FE"},
    {"svommer_farge5", "1F3CA-1F3FF"},
    {"vektlofter", "1F3CB"},
    {"vektlofter_farge1", "1F3CB-1F3FB"},
    {"vektlofter_farge2", "1F3CB-1F3FC"},
    {"vektlofter_farge3", "1F3CB-1F3FD"},
    {"vektlofter_farge4", "1F3CB-1F3FE"},
    {"vektlofter_farge5", "1F3CB-1F3FF"},
    {"golfspiller", "1F3CC"},
    {"golfspiller_farge1", "1F3CC-1F3FB"},
    {"golfspiller_farge2", "1F3CC-1F3FC"},
    {"golfspiller_farge3", "1F3CC-1F3FD"},
    {"golfspiller_farge4", "1F3CC-1F3FE"},
    {"golfspiller_farge5", "1F3CC-1F3FF"},
    {"motorsykkel", "1F3CD"},
    {"racerbil", "1F3CE"},
    {"cricket", "1F3CF"},
    {"volleyball", "1F3D0"},
    {"landhockey", "1F3D1"},
    {"ishockey", "1F3D2"},
    {"bordtennis", "1F3D3"},
    {"snodekt_fjell", "1F3D4"},
    {"telttur", "1F3D5"},
    {"strand_med_parasoll", "1F3D6"},
    {"bygning_under_konstruksjon", "1F3D7"},
    {"boliger", "1F3D8"},
    {"bylandskap", "1F3D9"},
    {"falleferdig_hus", "1F3DA"},
    {"klassisk_bygning", "1F3DB"},
    {"orken", "1F3DC"},
    {"ode_oy", "1F3DD"},
    {"nasjonalpark", "1F3DE"},
    {"stadion", "1F3DF"},
    {"hus", "1F3E0"},
    {"hus_med_hage", "1F3E1"},
    {"kontorbygg", "1F3E2"},
    {"japansk_postkontor", "1F3E3"},
    {"postkontor", "1F3E4"},
    {"sykehus", "1F3E5"},
    {"bank", "1F3E6"},
    {"minibankskilt", "1F3E7"},
    {"hotell", "1F3E8"},
    {"kjaerlighetshotell", "1F3E9"},
    {"dagligvarebutikk", "1F3EA"},
    {"skole", "1F3EB"},
    {"kjopesenter", "1F3EC"},
    {"fabrikk", "1F3ED"},
    {"rod_papirlykt", "1F3EE"},
    {"japansk_slott", "1F3EF"},
    {"slott", "1F3F0"},
    {"hvitt_flagg", "1F3F3"},
    {"svart_flagg", "1F3F4"},
    {"rosett", "1F3F5"},
    {"etikett", "1F3F7"},
    {"badminton", "1F3F8"},
    {"pil_og_bue", "1F3F9"},
    {"amfora", "1F3FA"},
    {"hudtype_1_2", "1F3FB"},
    {"hudtype_3", "1F3FC"},
    {"hudtype_4", "1F3FD"},
    {"hudtype_5", "1F3FE"},
    {"hudtype_6", "1F3FF"},
    {"rotte", "1F400"},
    {"mus", "1F401"},
    {"okse", "1F402"},
    {"vannboffel", "1F403"},
    {"ku", "1F404"},
    {"tiger", "1F405"},
    {"leopard", "1F406"},
    {"kanin", "1F407"},
    {"katt", "1F408"},
    {"drage", "1F409"},
    {"krokodille", "1F40A"},
    {"hval", "1F40B"},
    {"snegle", "1F40C"},
    {"slange", "1F40D"},
    {"hest", "1F40E"},
    {"bukk", "1F40F"},
    {"geit", "1F410"},
    {"soye", "1F411"},
    {"ape", "1F412"},
    {"hane", "1F413"},
    {"hone", "1F414"},
    {"hund", "1F415"},
    {"gris", "1F416"},
    {"villsvin", "1F417"},
    {"elefant", "1F418"},
    {"blekksprut", "1F419"},
    {"spiralskjell", "1F41A"},
    {"insekt", "1F41B"},
    {"maur", "1F41C"},
    {"honningbie", "1F41D"},
    {"marihone", "1F41E"},
    {"fisk", "1F41F"},
    {"tropisk_fisk", "1F420"},
    {"kulefisk", "1F421"},
    {"skilpadde", "1F422"},
    {"nyklekket_kylling", "1F423"},
    {"kylling", "1F424"},
    {"kylling_forfra", "1F425"},
    {"fugl", "1F426"},
    {"pingvin", "1F427"},
    {"koala", "1F428"},
    {"puddel", "1F429"},
    {"dromedar", "1F42A"},
    {"kamel", "1F42B"},
    {"delfin", "1F42C"},
    {"musefjes", "1F42D"},
    {"kufjes", "1F42E"},
    {"tigerfjes", "1F42F"},
    {"kaninfjes", "1F430"},
    {"kattefjes", "1F431"},
    {"dragefjes", "1F432"},
    {"sprutende_hval", "1F433"},
    {"hestefjes", "1F434"},
    {"apefjes", "1F435"},
    {"hundefjes", "1F436"},
    {"grisefjes", "1F437"},
    {"frosk", "1F438"},
    {"hamster", "1F439"},
    {"ulv", "1F43A"},
    {"bjorn", "1F43B"},
    {"panda", "1F43C"},
    {"grisetryne", "1F43D"},
    {"poteavtrykk", "1F43E"},
    {"ekorn", "1F43F"},
    {"oyne", "1F440"},
    {"oye", "1F441"},
    {"ore", "1F442"},
    {"ore_farge1", "1F442-1F3FB"},
    {"ore_farge2", "1F442-1F3FC"},
    {"ore_farge3", "1F442-1F3FD"},
    {"ore_farge4", "1F442-1F3FE"},
    {"ore_farge5", "1F442-1F3FF"},
    {"nese", "1F443"},
    {"nese_farge1", "1F443-1F3FB"},
    {"nese_farge2", "1F443-1F3FC"},
    {"nese_farge3", "1F443-1F3FD"},
    {"nese_farge4", "1F443-1F3FE"},
    {"nese_farge5", "1F443-1F3FF"},
    {"munn", "1F444"},
    {"tunge", "1F445"},
    {"peker_opp_bakhand", "1F446"},
    {"peker_opp_bakhand_farge1", "1F446-1F3FB"},
    {"peker_opp_bakhand_farge2", "1F446-1F3FC"},
    {"peker_opp_bakhand_farge3", "1F446-1F3FD"},
    {"peker_opp_bakhand_farge4", "1F446-1F3FE"},
    {"peker_opp_bakhand_farge5", "1F446-1F3FF"},
    {"peker_ned_bakhand", "1F447"},
    {"peker_ned_bakhand_farge1", "1F447-1F3FB"},
    {"peker_ned_bakhand_farge2", "1F447-1F3FC"},
    {"peker_ned_bakhand_farge3", "1F447-1F3FD"},
    {"peker_ned_bakhand_farge4", "1F447-1F3FE"},
    {"peker_ned_bakhand_farge5", "1F447-1F3FF"},
    {"peker_mot_venstre_bakhand", "1F448"},
    {"peker_mot_venstre_bakhand_farge1", "1F448-1F3FB"},
    {"peker_mot_venstre_bakhand_farge2", "1F448-1F3FC"},
    {"peker_mot_venstre_bakhand_farge3", "1F448-1F3FD"},
    {"peker_mot_venstre_bakhand_farge4", "1F448-1F3FE"},
    {"peker_mot_venstre_bakhand_farge5", "1F448-1F3FF"},
    {"peker_mot_hoyre_bakhand", "1F449"},
    {"peker_mot_hoyre_bakhand_farge1", "1F449-1F3FB"},
    {"peker_mot_hoyre_bakhand_farge2", "1F449-1F3FC"},
    {"peker_mot_hoyre_bakhand_farge3", "1F449-1F3FD"},
    {"peker_mot_hoyre_bakhand_farge4", "1F449-1F3FE"},
    {"peker_mot_hoyre_bakhand_farge5", "1F449-1F3FF"},
    {"knyttneve", "1F44A"},
    {"knyttneve_farge1", "1F44A-1F3FB"},
    {"knyttneve_farge2", "1F44A-1F3FC"},
    {"knyttneve_farge3", "1F44A-1F3FD"},
    {"knyttneve_farge4", "1F44A-1F3FE"},
    {"knyttneve_farge5", "1F44A-1F3FF"},
    {"vinkende_hand", "1F44B"},
    {"vinkende_hand_farge1", "1F44B-1F3FB"},
    {"vinkende_hand_farge2", "1F44B-1F3FC"},
    {"vinkende_hand_farge3", "1F44B-1F3FD"},
    {"vinkende_hand_farge4", "1F44B-1F3FE"},
    {"vinkende_hand_farge5", "1F44B-1F3FF"},
    {"ok_hand", "1F44C"},
    {"ok_hand_farge1", "1F44C-1F3FB"},
    {"ok_hand_farge2", "1F44C-1F3FC"},
    {"ok_hand_farge3", "1F44C-1F3FD"},
    {"ok_hand_farge4", "1F44C-1F3FE"},
    {"ok_hand_farge5", "1F44C-1F3FF"},
    {"tommel_opp", "1F44D"},
    {"tommel_opp_farge1", "1F44D-1F3FB"},
    {"tommel_opp_farge2", "1F44D-1F3FC"},
    {"tommel_opp_farge3", "1F44D-1F3FD"},
    {"tommel_opp_farge4", "1F44D-1F3FE"},
    {"tommel_opp_farge5", "1F44D-1F3FF"},
    {"tommel_ned", "1F44E"},
    {"tommel_ned_farge1", "1F44E-1F3FB"},
    {"tommel_ned_farge2", "1F44E-1F3FC"},
    {"tommel_ned_farge3", "1F44E-1F3FD"},
    {"tommel_ned_farge4", "1F44E-1F3FE"},
    {"tommel_ned_farge5", "1F44E-1F3FF"},
    {"klappende_hender", "1F44F"},
    {"klappende_hender_farge1", "1F44F-1F3FB"},
    {"klappende_hender_farge2", "1F44F-1F3FC"},
    {"klappende_hender_farge3", "1F44F-1F3FD"},
    {"klappende_hender_farge4", "1F44F-1F3FE"},
    {"klappende_hender_farge5", "1F44F-1F3FF"},
    {"apne_hender", "1F450"},
    {"apne_hender_farge1", "1F450-1F3FB"},
    {"apne_hender_farge2", "1F450-1F3FC"},
    {"apne_hender_farge3", "1F450-1F3FD"},
    {"apne_hender_farge4", "1F450-1F3FE"},
    {"apne_hender_farge5", "1F450-1F3FF"},
    {"krone", "1F451"},
    {"damehatt", "1F452"},
    {"briller", "1F453"},
    {"slips", "1F454"},
    {"t_skjorte", "1F455"},
    {"bukser", "1F456"},
    {"kjole", "1F457"},
    {"kimono", "1F458"},
    {"bikini", "1F459"},
    {"dameklaer", "1F45A"},
    {"portemone", "1F45B"},
    {"handveske", "1F45C"},
    {"clutch_veske", "1F45D"},
    {"herresko", "1F45E"},
    {"joggesko", "1F45F"},
    {"hoyhaelt_sko", "1F460"},
    {"damesandal", "1F461"},
    {"damestovlett", "1F462"},
    {"fotavtrykk", "1F463"},
    {"silhuett_av_byste", "1F464"},
    {"silhuett_av_byster", "1F465"},
    {"gutt", "1F466"},
    {"gutt_farge1", "1F466-1F3FB"},
    {"gutt_farge2", "1F466-1F3FC"},
    {"gutt_farge3", "1F466-1F3FD"},
    {"gutt_farge4", "1F466-1F3FE"},
    {"gutt_farge5", "1F466-1F3FF"},
    {"jente", "1F467"},
    {"jente_farge1", "1F467-1F3FB"},
    {"jente_farge2", "1F467-1F3FC"},
    {"jente_farge3", "1F467-1F3FD"},
    {"jente_farge4", "1F467-1F3FE"},
    {"jente_farge5", "1F467-1F3FF"},
    {"mann", "1F468"},
    {"mann_farge1", "1F468-1F3FB"},
    {"mann_farge2", "1F468-1F3FC"},
    {"mann_farge3", "1F468-1F3FD"},
    {"mann_farge4", "1F468-1F3FE"},
    {"mann_farge5", "1F468-1F3FF"},
    {"kvinne", "1F469"},
    {"kvinne_farge1", "1F469-1F3FB"},
    {"kvinne_farge2", "1F469-1F3FC"},
    {"kvinne_farge3", "1F469-1F3FD"},
    {"kvinne_farge4", "1F469-1F3FE"},
    {"kvinne_farge5", "1F469-1F3FF"},
    {"familie", "1F46A"},
    {"mann_og_kvinne_som_leier", "1F46B"},
    {"mann_og_kvinne_som_leier_farge1", "1F46B-1F3FB"},
    {"mann_og_kvinne_som_leier_farge2", "1F46B-1F3FC"},
    {"mann_og_kvinne_som_leier_farge3", "1F46B-1F3FD"},
    {"mann_og_kvinne_som_leier_farge4", "1F46B-1F3FE"},
    {"mann_og_kvinne_som_leier_farge5", "1F46B-1F3FF"},
    {"mann_og_kvinne_som_leier_farge1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mann_og_kvinne_som_leier_farge1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mann_og_kvinne_som_leier_farge1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mann_og_kvinne_som_leier_farge1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mann_og_kvinne_som_leier_farge2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mann_og_kvinne_som_leier_farge2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mann_og_kvinne_som_leier_farge2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mann_og_kvinne_som_leier_farge2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mann_og_kvinne_som_leier_farge3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mann_og_kvinne_som_leier_farge3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mann_og_kvinne_som_leier_farge3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mann_og_kvinne_som_leier_farge3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mann_og_kvinne_som_leier_farge4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mann_og_kvinne_som_leier_farge4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mann_og_kvinne_som_leier_farge4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mann_og_kvinne_som_leier_farge4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mann_og_kvinne_som_leier_farge5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mann_og_kvinne_som_leier_farge5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mann_og_kvinne_som_leier_farge5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mann_og_kvinne_som_leier_farge5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"menn_som_leier", "1F46C"},
    {"menn_som_leier_farge1", "1F46C-1F3FB"},
    {"menn_som_leier_farge2", "1F46C-1F3FC"},
    {"menn_som_leier_farge3", "1F46C-1F3FD"},
    {"menn_som_leier_farge4", "1F46C-1F3FE"},
    {"menn_som_leier_farge5", "1F46C-1F3FF"},
    {"menn_som_leier_farge1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"menn_som_leier_farge1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"menn_som_leier_farge1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"menn_som_leier_farge1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"menn_som_leier_farge2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"menn_som_leier_farge2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"menn_som_leier_farge2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"menn_som_leier_farge2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"menn_som_leier_farge3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"menn_som_leier_farge3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"menn_som_leier_farge3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"menn_som_leier_farge3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"menn_som_leier_farge4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"menn_som_leier_farge4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"menn_som_leier_farge4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"menn_som_leier_farge4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"menn_som_leier_farge5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"menn_som_leier_farge5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"menn_som_leier_farge5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"menn_som_leier_farge5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"kvinner_som_leier", "1F46D"},
    {"kvinner_som_leier_farge1", "1F46D-1F3FB"},
    {"kvinner_som_leier_farge2", "1F46D-1F3FC"},
    {"kvinner_som_leier_farge3", "1F46D-1F3FD"},
    {"kvinner_som_leier_farge4", "1F46D-1F3FE"},
    {"kvinner_som_leier_farge5", "1F46D-1F3FF"},
    {"kvinner_som_leier_farge1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"kvinner_som_leier_farge1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"kvinner_som_leier_farge1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"kvinner_som_leier_farge1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"kvinner_som_leier_farge2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"kvinner_som_leier_farge2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"kvinner_som_leier_farge2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"kvinner_som_leier_farge2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"kvinner_som_leier_farge3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"kvinner_som_leier_farge3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"kvinner_som_leier_farge3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"kvinner_som_leier_farge3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"kvinner_som_leier_farge4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"kvinner_som_leier_farge4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"kvinner_som_leier_farge4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"kvinner_som_leier_farge4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"kvinner_som_leier_farge5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"kvinner_som_leier_farge5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"kvinner_som_leier_farge5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"kvinner_som_leier_farge5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"politibetjent", "1F46E"},
    {"politibetjent_farge1", "1F46E-1F3FB"},
    {"politibetjent_farge2", "1F46E-1F3FC"},
    {"politibetjent_farge3", "1F46E-1F3FD"},
    {"politibetjent_farge4", "1F46E-1F3FE"},
    {"politibetjent_farge5", "1F46E-1F3FF"},
    {"personer_med_kaninorer", "1F46F"},
    {"personer_med_kaninorer_farge1", "1F46F-1F3FB"},
    {"personer_med_kaninorer_farge2", "1F46F-1F3FC"},
    {"personer_med_kaninorer_farge3", "1F46F-1F3FD"},
    {"personer_med_kaninorer_farge4", "1F46F-1F3FE"},
    {"personer_med_kaninorer_farge5", "1F46F-1F3FF"},
    {"personer_med_kaninorer_farge1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninorer_farge1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninorer_farge1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninorer_farge1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninorer_farge2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninorer_farge2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninorer_farge2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninorer_farge2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninorer_farge3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninorer_farge3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninorer_farge3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninorer_farge3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninorer_farge4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninorer_farge4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninorer_farge4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninorer_farge4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninorer_farge5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninorer_farge5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninorer_farge5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninorer_farge5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"person_med_slor", "1F470"},
    {"person_med_slor_farge1", "1F470-1F3FB"},
    {"person_med_slor_farge2", "1F470-1F3FC"},
    {"person_med_slor_farge3", "1F470-1F3FD"},
    {"person_med_slor_farge4", "1F470-1F3FE"},
    {"person_med_slor_farge5", "1F470-1F3FF"},
    {"person_blond", "1F471"},
    {"person_blond_farge1", "1F471-1F3FB"},
    {"person_blond_farge2", "1F471-1F3FC"},
    {"person_blond_farge3", "1F471-1F3FD"},
    {"person_blond_farge4", "1F471-1F3FE"},
    {"person_blond_farge5", "1F471-1F3FF"},
    {"mann_med_kinesisk_lue", "1F472"},
    {"mann_med_kinesisk_lue_farge1", "1F472-1F3FB"},
    {"mann_med_kinesisk_lue_farge2", "1F472-1F3FC"},
    {"mann_med_kinesisk_lue_farge3", "1F472-1F3FD"},
    {"mann_med_kinesisk_lue_farge4", "1F472-1F3FE"},
    {"mann_med_kinesisk_lue_farge5", "1F472-1F3FF"},
    {"turbankledd_person", "1F473"},
    {"turbankledd_person_farge1", "1F473-1F3FB"},
    {"turbankledd_person_farge2", "1F473-1F3FC"},
    {"turbankledd_person_farge3", "1F473-1F3FD"},
    {"turbankledd_person_farge4", "1F473-1F3FE"},
    {"turbankledd_person_farge5", "1F473-1F3FF"},
    {"eldre_mann", "1F474"},
    {"eldre_mann_farge1", "1F474-1F3FB"},
    {"eldre_mann_farge2", "1F474-1F3FC"},
    {"eldre_mann_farge3", "1F474-1F3FD"},
    {"eldre_mann_farge4", "1F474-1F3FE"},
    {"eldre_mann_farge5", "1F474-1F3FF"},
    {"eldre_kvinne", "1F475"},
    {"eldre_kvinne_farge1", "1F475-1F3FB"},
    {"eldre_kvinne_farge2", "1F475-1F3FC"},
    {"eldre_kvinne_farge3", "1F475-1F3FD"},
    {"eldre_kvinne_farge4", "1F475-1F3FE"},
    {"eldre_kvinne_farge5", "1F475-1F3FF"},
    {"baby", "1F476"},
    {"baby_farge1", "1F476-1F3FB"},
    {"baby_farge2", "1F476-1F3FC"},
    {"baby_farge3", "1F476-1F3FD"},
    {"baby_farge4", "1F476-1F3FE"},
    {"baby_farge5", "1F476-1F3FF"},
    {"anleggsarbeider", "1F477"},
    {"anleggsarbeider_farge1", "1F477-1F3FB"},
    {"anleggsarbeider_farge2", "1F477-1F3FC"},
    {"anleggsarbeider_farge3", "1F477-1F3FD"},
    {"anleggsarbeider_farge4", "1F477-1F3FE"},
    {"anleggsarbeider_farge5", "1F477-1F3FF"},
    {"prinsesse", "1F478"},
    {"prinsesse_farge1", "1F478-1F3FB"},
    {"prinsesse_farge2", "1F478-1F3FC"},
    {"prinsesse_farge3", "1F478-1F3FD"},
    {"prinsesse_farge4", "1F478-1F3FE"},
    {"prinsesse_farge5", "1F478-1F3FF"},
    {"japansk_troll", "1F479"},
    {"tengu", "1F47A"},
    {"spokelse", "1F47B"},
    {"babyengel", "1F47C"},
    {"babyengel_farge1", "1F47C-1F3FB"},
    {"babyengel_farge2", "1F47C-1F3FC"},
    {"babyengel_farge3", "1F47C-1F3FD"},
    {"babyengel_farge4", "1F47C-1F3FE"},
    {"babyengel_farge5", "1F47C-1F3FF"},
    {"romvesen", "1F47D"},
    {"rommonster", "1F47E"},
    {"sint_djevel", "1F47F"},
    {"hodeskalle", "1F480"},
    {"hjelpende_person", "1F481"},
    {"hjelpende_person_farge1", "1F481-1F3FB"},
    {"hjelpende_person_farge2", "1F481-1F3FC"},
    {"hjelpende_person_farge3", "1F481-1F3FD"},
    {"hjelpende_person_farge4", "1F481-1F3FE"},
    {"hjelpende_person_farge5", "1F481-1F3FF"},
    {"gardist", "1F482"},
    {"gardist_farge1", "1F482-1F3FB"},
    {"gardist_farge2", "1F482-1F3FC"},
    {"gardist_farge3", "1F482-1F3FD"},
    {"gardist_farge4", "1F482-1F3FE"},
    {"gardist_farge5", "1F482-1F3FF"},
    {"dansende_kvinne", "1F483"},
    {"dansende_kvinne_farge1", "1F483-1F3FB"},
    {"dansende_kvinne_farge2", "1F483-1F3FC"},
    {"dansende_kvinne_farge3", "1F483-1F3FD"},
    {"dansende_kvinne_farge4", "1F483-1F3FE"},
    {"dansende_kvinne_farge5", "1F483-1F3FF"},
    {"leppestift", "1F484"},
    {"neglelakk", "1F485"},
    {"neglelakk_farge1", "1F485-1F3FB"},
    {"neglelakk_farge2", "1F485-1F3FC"},
    {"neglelakk_farge3", "1F485-1F3FD"},
    {"neglelakk_farge4", "1F485-1F3FE"},
    {"neglelakk_farge5", "1F485-1F3FF"},
    {"ansiktsmassasje", "1F486"},
    {"ansiktsmassasje_farge1", "1F486-1F3FB"},
    {"ansiktsmassasje_farge2", "1F486-1F3FC"},
    {"ansiktsmassasje_farge3", "1F486-1F3FD"},
    {"ansiktsmassasje_farge4", "1F486-1F3FE"},
    {"ansiktsmassasje_farge5", "1F486-1F3FF"},
    {"harklipp", "1F487"},
    {"harklipp_farge1", "1F487-1F3FB"},
    {"harklipp_farge2", "1F487-1F3FC"},
    {"harklipp_farge3", "1F487-1F3FD"},
    {"harklipp_farge4", "1F487-1F3FE"},
    {"harklipp_farge5", "1F487-1F3FF"},
    {"barberskilt", "1F488"},
    {"sproyte", "1F489"},
    {"pille", "1F48A"},
    {"kyssemerke", "1F48B"},
    {"kjaerlighesbrev", "1F48C"},
    {"ring", "1F48D"},
    {"diamant", "1F48E"},
    {"kyss", "1F48F"},
    {"kyss_farge1", "1F48F-1F3FB"},
    {"kyss_farge2", "1F48F-1F3FC"},
    {"kyss_farge3", "1F48F-1F3FD"},
    {"kyss_farge4", "1F48F-1F3FE"},
    {"kyss_farge5", "1F48F-1F3FF"},
    {"kyss_farge1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"kyss_farge1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"kyss_farge1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"kyss_farge1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"kyss_farge2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"kyss_farge2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"kyss_farge2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"kyss_farge2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"kyss_farge3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"kyss_farge3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"kyss_farge3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"kyss_farge3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"kyss_farge4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"kyss_farge4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"kyss_farge4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"kyss_farge4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"kyss_farge5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"kyss_farge5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"kyss_farge5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"kyss_farge5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bukett", "1F490"},
    {"par_med_hjerte", "1F491"},
    {"par_med_hjerte_farge1", "1F491-1F3FB"},
    {"par_med_hjerte_farge2", "1F491-1F3FC"},
    {"par_med_hjerte_farge3", "1F491-1F3FD"},
    {"par_med_hjerte_farge4", "1F491-1F3FE"},
    {"par_med_hjerte_farge5", "1F491-1F3FF"},
    {"par_med_hjerte_farge1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjerte_farge1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjerte_farge1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"par_med_hjerte_farge1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjerte_farge2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjerte_farge2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjerte_farge2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"par_med_hjerte_farge2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjerte_farge3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjerte_farge3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjerte_farge3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"par_med_hjerte_farge3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjerte_farge4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjerte_farge4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjerte_farge4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjerte_farge4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjerte_farge5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjerte_farge5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjerte_farge5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjerte_farge5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"bryllup", "1F492"},
    {"bankende_hjerte", "1F493"},
    {"knust_hjerte", "1F494"},
    {"to_hjerter", "1F495"},
    {"skinnende_hjerte", "1F496"},
    {"voksende_hjerte", "1F497"},
    {"hjerte_med_pil", "1F498"},
    {"blatt_hjerte", "1F499"},
    {"gront_hjerte", "1F49A"},
    {"gult_hjerte", "1F49B"},
    {"lilla_hjerte", "1F49C"},
    {"hjerte_med_band", "1F49D"},
    {"snurrende_hjerter", "1F49E"},
    {"hjertedekorasjon", "1F49F"},
    {"diamant_med_prikk", "1F4A0"},
    {"lyspaere", "1F4A1"},
    {"sinne", "1F4A2"},
    {"bombe", "1F4A3"},
    {"zzz", "1F4A4"},
    {"krasj", "1F4A5"},
    {"svettedraper", "1F4A6"},
    {"drape", "1F4A7"},
    {"stikker_av", "1F4A8"},
    {"baesj", "1F4A9"},
    {"biceps", "1F4AA"},
    {"biceps_farge1", "1F4AA-1F3FB"},
    {"biceps_farge2", "1F4AA-1F3FC"},
    {"biceps_farge3", "1F4AA-1F3FD"},
    {"biceps_farge4", "1F4AA-1F3FE"},
    {"biceps_farge5", "1F4AA-1F3FF"},
    {"svimmel", "1F4AB"},
    {"snakkeboble", "1F4AC"},
    {"tankeboble", "1F4AD"},
    {"hvit_blomst", "1F4AE"},
    {"hundre_poeng", "1F4AF"},
    {"pengesekk", "1F4B0"},
    {"valutaveksling", "1F4B1"},
    {"dollartegn", "1F4B2"},
    {"kredittkort", "1F4B3"},
    {"yenseddel", "1F4B4"},
    {"dollarseddel", "1F4B5"},
    {"euroseddel", "1F4B6"},
    {"pundseddel", "1F4B7"},
    {"penger_med_vinger", "1F4B8"},
    {"stigende_diagram_med_yen", "1F4B9"},
    {"sete", "1F4BA"},
    {"baerbar_datamaskin", "1F4BB"},
    {"dokumentmappe", "1F4BC"},
    {"datadisk", "1F4BD"},
    {"diskett", "1F4BE"},
    {"optisk_plate", "1F4BF"},
    {"dvd", "1F4C0"},
    {"arkivmappe", "1F4C1"},
    {"apen_arkivmappe", "1F4C2"},
    {"side_med_boy", "1F4C3"},
    {"side_som_vender_opp", "1F4C4"},
    {"kalender", "1F4C5"},
    {"veggkalender", "1F4C6"},
    {"kartotek", "1F4C7"},
    {"diagram_som_peker_oppover", "1F4C8"},
    {"diagram_som_peker_nedover", "1F4C9"},
    {"stolpediagram", "1F4CA"},
    {"skriveplate", "1F4CB"},
    {"stift", "1F4CC"},
    {"rund_stift", "1F4CD"},
    {"binders", "1F4CE"},
    {"linjal", "1F4CF"},
    {"trekantlinjal", "1F4D0"},
    {"bokmerkefane", "1F4D1"},
    {"spiralblokk", "1F4D2"},
    {"notatbok", "1F4D3"},
    {"dekorert_notatbok", "1F4D4"},
    {"lukket_bok", "1F4D5"},
    {"apen_bok", "1F4D6"},
    {"gronn_bok", "1F4D7"},
    {"bla_bok", "1F4D8"},
    {"oransje_bok", "1F4D9"},
    {"boker", "1F4DA"},
    {"navneskilt", "1F4DB"},
    {"pergamentrull", "1F4DC"},
    {"notat", "1F4DD"},
    {"telefonror", "1F4DE"},
    {"personsoker", "1F4DF"},
    {"faksmaskin", "1F4E0"},
    {"satellittantenne", "1F4E1"},
    {"hoyttaler", "1F4E2"},
    {"megafon", "1F4E3"},
    {"utboks", "1F4E4"},
    {"innboks", "1F4E5"},
    {"pakke", "1F4E6"},
    {"e_post", "1F4E7"},
    {"innkommende_konvolutt", "1F4E8"},
    {"konvolutt_med_pil", "1F4E9"},
    {"lukket_postkasse_med_flagget_nede", "1F4EA"},
    {"lukket_postkasse_med_flagget_oppe", "1F4EB"},
    {"apen_postkasse_med_flagget_oppe", "1F4EC"},
    {"apen_postkasse_med_flagget_nede", "1F4ED"},
    {"postkasse", "1F4EE"},
    {"posthorn", "1F4EF"},
    {"avis", "1F4F0"},
    {"mobiltelefon", "1F4F1"},
    {"mobiltelefon_med_pil", "1F4F2"},
    {"vibreringsmodus", "1F4F3"},
    {"mobiltelefon_av", "1F4F4"},
    {"mobiltelefon_ikke_tillatt", "1F4F5"},
    {"mobilsignaler", "1F4F6"},
    {"kamera", "1F4F7"},
    {"kamera_med_blits", "1F4F8"},
    {"videokamera", "1F4F9"},
    {"tv", "1F4FA"},
    {"radio", "1F4FB"},
    {"videokassett", "1F4FC"},
    {"filmfremviser", "1F4FD"},
    {"bonnekrans", "1F4FF"},
    {"bland_spor_symbol", "1F500"},
    {"gjenta_knapp", "1F501"},
    {"gjenta_ett_spor_knapp", "1F502"},
    {"vertikale_piler_med_klokken", "1F503"},
    {"piler_mot_klokken", "1F504"},
    {"lite_lys_knapp", "1F505"},
    {"mye_lys_knapp", "1F506"},
    {"dempet_hoyttaler", "1F507"},
    {"hoyttaler_med_lavt_volum", "1F508"},
    {"hoyttaler_med_middels_volum", "1F509"},
    {"hoyttaler_med_hoyt_volum", "1F50A"},
    {"batteri", "1F50B"},
    {"stopsel", "1F50C"},
    {"forstorrelsesglass_mot_venstre", "1F50D"},
    {"forstorrelsesglass_mot_hoyre", "1F50E"},
    {"last_med_penn", "1F50F"},
    {"last_med_nokkel", "1F510"},
    {"nokkel", "1F511"},
    {"last", "1F512"},
    {"ulast", "1F513"},
    {"bjelle", "1F514"},
    {"overstreket_bjelle", "1F515"},
    {"bokmerke", "1F516"},
    {"lenke", "1F517"},
    {"radioknapp", "1F518"},
    {"back_pil", "1F519"},
    {"end_pil", "1F51A"},
    {"on_pil", "1F51B"},
    {"soon_pil", "1F51C"},
    {"top_pil", "1F51D"},
    {"aldersgrense_18_ar", "1F51E"},
    {"taster_10", "1F51F"},
    {"store_bokstaver", "1F520"},
    {"sma_bokstaver", "1F521"},
    {"tall", "1F522"},
    {"symboler", "1F523"},
    {"latinsk_alfabet", "1F524"},
    {"ild", "1F525"},
    {"lommelykt", "1F526"},
    {"fastnokkel", "1F527"},
    {"hammer", "1F528"},
    {"mutter_og_skrue", "1F529"},
    {"kokkekniv", "1F52A"},
    {"vannpistol", "1F52B"},
    {"mikroskop", "1F52C"},
    {"teleskop", "1F52D"},
    {"krystallkule", "1F52E"},
    {"sekstagget_stjerne_med_prikk", "1F52F"},
    {"japansk_tegn_for_nybegynner", "1F530"},
    {"treforkemblem", "1F531"},
    {"svart_firkantet_knapp", "1F532"},
    {"hvit_firkantet_knapp", "1F533"},
    {"rod_sirkel", "1F534"},
    {"bla_sirkel", "1F535"},
    {"stort_oransje_rutersymbol", "1F536"},
    {"stort_blatt_rutersymbol", "1F537"},
    {"lite_oransje_rutersymbol", "1F538"},
    {"lite_blatt_rutersymbol", "1F539"},
    {"rod_trekant_som_peker_opp", "1F53A"},
    {"rod_trekant_som_peker_ned", "1F53B"},
    {"oppover_knapp", "1F53C"},
    {"nedover_knapp", "1F53D"},
    {"om", "1F549"},
    {"fredsdue", "1F54A"},
    {"kaba", "1F54B"},
    {"moske", "1F54C"},
    {"synagoge", "1F54D"},
    {"menora", "1F54E"},
    {"klokken_ett", "1F550"},
    {"klokken_to", "1F551"},
    {"klokken_tre", "1F552"},
    {"klokken_fire", "1F553"},
    {"klokken_fem", "1F554"},
    {"klokken_seks", "1F555"},
    {"klokken_sju", "1F556"},
    {"klokken_atte", "1F557"},
    {"klokken_ni", "1F558"},
    {"klokken_ti", "1F559"},
    {"klokken_elleve", "1F55A"},
    {"klokken_tolv", "1F55B"},
    {"klokken_halv_to", "1F55C"},
    {"klokken_halv_tre", "1F55D"},
    {"klokken_halv_fire", "1F55E"},
    {"klokken_halv_fem", "1F55F"},
    {"klokken_halv_seks", "1F560"},
    {"klokken_halv_sju", "1F561"},
    {"klokken_halv_atte", "1F562"},
    {"klokken_halv_ni", "1F563"},
    {"klokken_halv_ti", "1F564"},
    {"klokken_halv_elleve", "1F565"},
    {"klokken_halv_tolv", "1F566"},
    {"klokken_halv_ett", "1F567"},
    {"stearinlys", "1F56F"},
    {"peisur", "1F570"},
    {"hull_i_bakken", "1F573"},
    {"svevende_mann_i_dress", "1F574"},
    {"svevende_mann_i_dress_farge1", "1F574-1F3FB"},
    {"svevende_mann_i_dress_farge2", "1F574-1F3FC"},
    {"svevende_mann_i_dress_farge3", "1F574-1F3FD"},
    {"svevende_mann_i_dress_farge4", "1F574-1F3FE"},
    {"svevende_mann_i_dress_farge5", "1F574-1F3FF"},
    {"detektiv", "1F575"},
    {"detektiv_farge1", "1F575-1F3FB"},
    {"detektiv_farge2", "1F575-1F3FC"},
    {"detektiv_farge3", "1F575-1F3FD"},
    {"detektiv_farge4", "1F575-1F3FE"},
    {"detektiv_farge5", "1F575-1F3FF"},
    {"solbriller", "1F576"},
    {"edderkopp", "1F577"},
    {"spindelvev", "1F578"},
    {"joystick", "1F579"},
    {"dansende_mann", "1F57A"},
    {"dansende_mann_farge1", "1F57A-1F3FB"},
    {"dansende_mann_farge2", "1F57A-1F3FC"},
    {"dansende_mann_farge3", "1F57A-1F3FD"},
    {"dansende_mann_farge4", "1F57A-1F3FE"},
    {"dansende_mann_farge5", "1F57A-1F3FF"},
    {"sammenlenkede_binders", "1F587"},
    {"penn", "1F58A"},
    {"fyllepenn", "1F58B"},
    {"pensel", "1F58C"},
    {"fargestift", "1F58D"},
    {"flat_hand_med_spredte_fingre", "1F590"},
    {"flat_hand_med_spredte_fingre_farge1", "1F590-1F3FB"},
    {"flat_hand_med_spredte_fingre_farge2", "1F590-1F3FC"},
    {"flat_hand_med_spredte_fingre_farge3", "1F590-1F3FD"},
    {"flat_hand_med_spredte_fingre_farge4", "1F590-1F3FE"},
    {"flat_hand_med_spredte_fingre_farge5", "1F590-1F3FF"},
    {"langfinger", "1F595"},
    {"langfinger_farge1", "1F595-1F3FB"},
    {"langfinger_farge2", "1F595-1F3FC"},
    {"langfinger_farge3", "1F595-1F3FD"},
    {"langfinger_farge4", "1F595-1F3FE"},
    {"langfinger_farge5", "1F595-1F3FF"},
    {"vulcan_hilsen", "1F596"},
    {"vulcan_hilsen_farge1", "1F596-1F3FB"},
    {"vulcan_hilsen_farge2", "1F596-1F3FC"},
    {"vulcan_hilsen_farge3", "1F596-1F3FD"},
    {"vulcan_hilsen_farge4", "1F596-1F3FE"},
    {"vulcan_hilsen_farge5", "1F596-1F3FF"},
    {"svart_hjerte", "1F5A4"},
    {"stasjonaer_datamaskin", "1F5A5"},
    {"skriver", "1F5A8"},
    {"datamus", "1F5B1"},
    {"styrekule", "1F5B2"},
    {"innrammet_bilde", "1F5BC"},
    {"skillekort", "1F5C2"},
    {"kartotekboks", "1F5C3"},
    {"arkivskap", "1F5C4"},
    {"papirkurv", "1F5D1"},
    {"spiralhefte", "1F5D2"},
    {"spiralkalender", "1F5D3"},
    {"skrutvinge", "1F5DC"},
    {"gammeldags_nokkel", "1F5DD"},
    {"sammenrullet_avis", "1F5DE"},
    {"dolk", "1F5E1"},
    {"snakkende_hode_i_profil", "1F5E3"},
    {"snakkeboble_mot_venstre", "1F5E8"},
    {"sint_snakkeboble_mot_hoyre", "1F5EF"},
    {"urne_med_stemmeseddel", "1F5F3"},
    {"verdenskart", "1F5FA"},
    {"fuji_fjellet", "1F5FB"},
    {"tokyo_tarnet", "1F5FC"},
    {"frihetsgudinnen", "1F5FD"},
    {"kart_over_japan", "1F5FE"},
    {"moai_statue", "1F5FF"},
    {"glisefjes", "1F600"},
    {"glisefjes_med_smilende_oyne", "1F601"},
    {"gledestarer", "1F602"},
    {"glisende_fjes_med_vidapne_oyne", "1F603"},
    {"glisende_fjes_med_smilende_oyne", "1F604"},
    {"glisende_fjes_som_svetter", "1F605"},
    {"smil_med_sammenknepne_oyne", "1F606"},
    {"smil_med_glorie", "1F607"},
    {"smilende_djevel", "1F608"},
    {"blunkende_ansikt", "1F609"},
    {"smil_med_smilende_oyne", "1F60A"},
    {"nam_nam", "1F60B"},
    {"lettet", "1F60C"},
    {"smilefjes_med_hjerteoyne", "1F60D"},
    {"smilefjes_med_solbriller", "1F60E"},
    {"lurt_smil", "1F60F"},
    {"noytral", "1F610"},
    {"uttrykkslos", "1F611"},
    {"misfornoyd", "1F612"},
    {"kaldsvetter", "1F613"},
    {"tankefull", "1F614"},
    {"forvirret", "1F615"},
    {"fortvilet", "1F616"},
    {"kyssefjes", "1F617"},
    {"slengkyss", "1F618"},
    {"kyss_med_smilende_oyne", "1F619"},
    {"kyss_med_lukkede_oyne", "1F61A"},
    {"rekker_tunge", "1F61B"},
    {"blunker_og_rekker_tunge", "1F61C"},
    {"aesj", "1F61D"},
    {"skuffet", "1F61E"},
    {"bekymret", "1F61F"},
    {"sint", "1F620"},
    {"rasende", "1F621"},
    {"grater", "1F622"},
    {"bommert", "1F623"},
    {"triumferende", "1F624"},
    {"puh", "1F625"},
    {"oppgitt", "1F626"},
    {"pine", "1F627"},
    {"redd", "1F628"},
    {"sliten", "1F629"},
    {"sovnig", "1F62A"},
    {"trott_fjes", "1F62B"},
    {"grimase", "1F62C"},
    {"strigrater", "1F62D"},
    {"forbauset", "1F62E"},
    {"overrasket", "1F62F"},
    {"engstelig_og_svetter", "1F630"},
    {"livredd", "1F631"},
    {"sjokkert", "1F632"},
    {"rodmer", "1F633"},
    {"sover", "1F634"},
    {"or", "1F635"},
    {"uten_munn", "1F636"},
    {"fjes_med_munnbind", "1F637"},
    {"flirende_katt_med_smilende_oyne", "1F638"},
    {"katt_med_gledestarer", "1F639"},
    {"smilende_katt_med_apen_munn", "1F63A"},
    {"smilende_katt_med_hjerteoyne", "1F63B"},
    {"katt_med_ironisk_smil", "1F63C"},
    {"kyssende_katt", "1F63D"},
    {"surmulende_katt", "1F63E"},
    {"gratende_katt", "1F63F"},
    {"sliten_katt", "1F640"},
    {"litt_surt_fjes", "1F641"},
    {"smiler_litt", "1F642"},
    {"fjes_som_er_opp_ned", "1F643"},
    {"himler_med_oynene", "1F644"},
    {"gestikulerer_nei", "1F645"},
    {"gestikulerer_nei_farge1", "1F645-1F3FB"},
    {"gestikulerer_nei_farge2", "1F645-1F3FC"},
    {"gestikulerer_nei_farge3", "1F645-1F3FD"},
    {"gestikulerer_nei_farge4", "1F645-1F3FE"},
    {"gestikulerer_nei_farge5", "1F645-1F3FF"},
    {"gestikulerer_ok", "1F646"},
    {"gestikulerer_ok_farge1", "1F646-1F3FB"},
    {"gestikulerer_ok_farge2", "1F646-1F3FC"},
    {"gestikulerer_ok_farge3", "1F646-1F3FD"},
    {"gestikulerer_ok_farge4", "1F646-1F3FE"},
    {"gestikulerer_ok_farge5", "1F646-1F3FF"},
    {"bukker", "1F647"},
    {"bukker_farge1", "1F647-1F3FB"},
    {"bukker_farge2", "1F647-1F3FC"},
    {"bukker_farge3", "1F647-1F3FD"},
    {"bukker_farge4", "1F647-1F3FE"},
    {"bukker_farge5", "1F647-1F3FF"},
    {"ser_ingenting_ondt", "1F648"},
    {"horer_ingenting_ondt", "1F649"},
    {"sier_ingenting_ondt", "1F64A"},
    {"rekker_opp_handen", "1F64B"},
    {"rekker_opp_handen_farge1", "1F64B-1F3FB"},
    {"rekker_opp_handen_farge2", "1F64B-1F3FC"},
    {"rekker_opp_handen_farge3", "1F64B-1F3FD"},
    {"rekker_opp_handen_farge4", "1F64B-1F3FE"},
    {"rekker_opp_handen_farge5", "1F64B-1F3FF"},
    {"hevede_hender", "1F64C"},
    {"hevede_hender_farge1", "1F64C-1F3FB"},
    {"hevede_hender_farge2", "1F64C-1F3FC"},
    {"hevede_hender_farge3", "1F64C-1F3FD"},
    {"hevede_hender_farge4", "1F64C-1F3FE"},
    {"hevede_hender_farge5", "1F64C-1F3FF"},
    {"rynker_brynene", "1F64D"},
    {"rynker_brynene_farge1", "1F64D-1F3FB"},
    {"rynker_brynene_farge2", "1F64D-1F3FC"},
    {"rynker_brynene_farge3", "1F64D-1F3FD"},
    {"rynker_brynene_farge4", "1F64D-1F3FE"},
    {"rynker_brynene_farge5", "1F64D-1F3FF"},
    {"furter", "1F64E"},
    {"furter_farge1", "1F64E-1F3FB"},
    {"furter_farge2", "1F64E-1F3FC"},
    {"furter_farge3", "1F64E-1F3FD"},
    {"furter_farge4", "1F64E-1F3FE"},
    {"furter_farge5", "1F64E-1F3FF"},
    {"foldede_hender", "1F64F"},
    {"foldede_hender_farge1", "1F64F-1F3FB"},
    {"foldede_hender_farge2", "1F64F-1F3FC"},
    {"foldede_hender_farge3", "1F64F-1F3FD"},
    {"foldede_hender_farge4", "1F64F-1F3FE"},
    {"foldede_hender_farge5", "1F64F-1F3FF"},
    {"rakett", "1F680"},
    {"helikopter", "1F681"},
    {"lokomotiv", "1F682"},
    {"togvogn", "1F683"},
    {"hoyhastighetstog", "1F684"},
    {"hoyhastighetstog_med_kulenese", "1F685"},
    {"tog", "1F686"},
    {"tunnelbane", "1F687"},
    {"lokaltog", "1F688"},
    {"togstasjon", "1F689"},
    {"trikk", "1F68A"},
    {"trikkevogn", "1F68B"},
    {"buss", "1F68C"},
    {"motgaende_buss", "1F68D"},
    {"trolleybuss", "1F68E"},
    {"busstopp", "1F68F"},
    {"minibuss", "1F690"},
    {"ambulanse", "1F691"},
    {"brannbil", "1F692"},
    {"politibil", "1F693"},
    {"motgaende_politibil", "1F694"},
    {"taxi", "1F695"},
    {"motgaende_taxi", "1F696"},
    {"bil", "1F697"},
    {"motgaende_bil", "1F698"},
    {"suv", "1F699"},
    {"varebil", "1F69A"},
    {"lastebil", "1F69B"},
    {"traktor", "1F69C"},
    {"monorail", "1F69D"},
    {"fjellbane", "1F69E"},
    {"svevebane", "1F69F"},
    {"kabelbane", "1F6A0"},
    {"taubane", "1F6A1"},
    {"skip", "1F6A2"},
    {"person_i_robat", "1F6A3"},
    {"person_i_robat_farge1", "1F6A3-1F3FB"},
    {"person_i_robat_farge2", "1F6A3-1F3FC"},
    {"person_i_robat_farge3", "1F6A3-1F3FD"},
    {"person_i_robat_farge4", "1F6A3-1F3FE"},
    {"person_i_robat_farge5", "1F6A3-1F3FF"},
    {"speedbat", "1F6A4"},
    {"horisontale_trafikklys", "1F6A5"},
    {"vertikale_trafiklys", "1F6A6"},
    {"anleggsarbeid", "1F6A7"},
    {"politilys", "1F6A8"},
    {"vimpel", "1F6A9"},
    {"dor", "1F6AA"},
    {"forbudt", "1F6AB"},
    {"sigarett", "1F6AC"},
    {"royking_forbudt", "1F6AD"},
    {"avfallsskilt", "1F6AE"},
    {"forsopling_ikke_tillatt", "1F6AF"},
    {"drikkevann", "1F6B0"},
    {"ikke_drikkevann", "1F6B1"},
    {"sykkel", "1F6B2"},
    {"sykler_forbudt", "1F6B3"},
    {"syklist", "1F6B4"},
    {"syklist_farge1", "1F6B4-1F3FB"},
    {"syklist_farge2", "1F6B4-1F3FC"},
    {"syklist_farge3", "1F6B4-1F3FD"},
    {"syklist_farge4", "1F6B4-1F3FE"},
    {"syklist_farge5", "1F6B4-1F3FF"},
    {"terrengsyklist", "1F6B5"},
    {"terrengsyklist_farge1", "1F6B5-1F3FB"},
    {"terrengsyklist_farge2", "1F6B5-1F3FC"},
    {"terrengsyklist_farge3", "1F6B5-1F3FD"},
    {"terrengsyklist_farge4", "1F6B5-1F3FE"},
    {"terrengsyklist_farge5", "1F6B5-1F3FF"},
    {"fotgjenger", "1F6B6"},
    {"fotgjenger_farge1", "1F6B6-1F3FB"},
    {"fotgjenger_farge2", "1F6B6-1F3FC"},
    {"fotgjenger_farge3", "1F6B6-1F3FD"},
    {"fotgjenger_farge4", "1F6B6-1F3FE"},
    {"fotgjenger_farge5", "1F6B6-1F3FF"},
    {"fotgjengere_forbudt", "1F6B7"},
    {"barn_krysser_veien", "1F6B8"},
    {"herretoalett", "1F6B9"},
    {"dametoalett", "1F6BA"},
    {"toalettskilt", "1F6BB"},
    {"babysymbol", "1F6BC"},
    {"toalett", "1F6BD"},
    {"do", "1F6BE"},
    {"dusj", "1F6BF"},
    {"person_i_badekar", "1F6C0"},
    {"person_i_badekar_farge1", "1F6C0-1F3FB"},
    {"person_i_badekar_farge2", "1F6C0-1F3FC"},
    {"person_i_badekar_farge3", "1F6C0-1F3FD"},
    {"person_i_badekar_farge4", "1F6C0-1F3FE"},
    {"person_i_badekar_farge5", "1F6C0-1F3FF"},
    {"badekar", "1F6C1"},
    {"passkontroll", "1F6C2"},
    {"toll", "1F6C3"},
    {"bagasjeband", "1F6C4"},
    {"bagasjeoppbevaring", "1F6C5"},
    {"sofa_og_lampe", "1F6CB"},
    {"person_i_seng", "1F6CC"},
    {"person_i_seng_farge1", "1F6CC-1F3FB"},
    {"person_i_seng_farge2", "1F6CC-1F3FC"},
    {"person_i_seng_farge3", "1F6CC-1F3FD"},
    {"person_i_seng_farge4", "1F6CC-1F3FE"},
    {"person_i_seng_farge5", "1F6CC-1F3FF"},
    {"handleposer", "1F6CD"},
    {"resepsjonsklokke", "1F6CE"},
    {"seng", "1F6CF"},
    {"religiost_samlingssted", "1F6D0"},
    {"stoppskilt", "1F6D1"},
    {"handlevogn", "1F6D2"},
    {"hindutempel", "1F6D5"},
    {"hytte", "1F6D6"},
    {"heis", "1F6D7"},
    {"jordskred", "1F6D8"},
    {"tradlost", "1F6DC"},
    {"sklie", "1F6DD"},
    {"hjul", "1F6DE"},
    {"livboye", "1F6DF"},
    {"hammer_og_fastnokkel", "1F6E0"},
    {"skjold", "1F6E1"},
    {"oljefat", "1F6E2"},
    {"motorvei", "1F6E3"},
    {"jernbanespor", "1F6E4"},
    {"motorbat", "1F6E5"},
    {"lite_fly", "1F6E9"},
    {"fly_som_tar_av", "1F6EB"},
    {"fly_som_lander", "1F6EC"},
    {"satellitt", "1F6F0"},
    {"passasjerskip", "1F6F3"},
    {"sparkesykkel", "1F6F4"},
    {"scooter", "1F6F5"},
    {"kano", "1F6F6"},
    {"kjelke", "1F6F7"},
    {"flygende_tallerken", "1F6F8"},
    {"skateboard", "1F6F9"},
    {"autorickshaw", "1F6FA"},
    {"pickup", "1F6FB"},
    {"rulleskoyter", "1F6FC"},
    {"oransje_sirkel", "1F7E0"},
    {"gul_sirkel", "1F7E1"},
    {"gronn_sirkel", "1F7E2"},
    {"lilla_sirkel", "1F7E3"},
    {"brun_sirkel", "1F7E4"},
    {"rodt_kvadrat", "1F7E5"},
    {"blatt_kvadrat", "1F7E6"},
    {"oransje_kvadrat", "1F7E7"},
    {"gult_kvadrat", "1F7E8"},
    {"gront_kvadrat", "1F7E9"},
    {"lilla_kvadrat", "1F7EA"},
    {"brunt_kvadrat", "1F7EB"},
    {"likhetstegn_i_fet_skrift", "1F7F0"},
    {"hand_med_fingertuppene_sammen", "1F90C"},
    {"hand_med_fingertuppene_sammen_farge1", "1F90C-1F3FB"},
    {"hand_med_fingertuppene_sammen_farge2", "1F90C-1F3FC"},
    {"hand_med_fingertuppene_sammen_farge3", "1F90C-1F3FD"},
    {"hand_med_fingertuppene_sammen_farge4", "1F90C-1F3FE"},
    {"hand_med_fingertuppene_sammen_farge5", "1F90C-1F3FF"},
    {"hvitt_hjerte", "1F90D"},
    {"brunt_hjerte", "1F90E"},
    {"hand_som_klyper", "1F90F"},
    {"hand_som_klyper_farge1", "1F90F-1F3FB"},
    {"hand_som_klyper_farge2", "1F90F-1F3FC"},
    {"hand_som_klyper_farge3", "1F90F-1F3FD"},
    {"hand_som_klyper_farge4", "1F90F-1F3FE"},
    {"hand_som_klyper_farge5", "1F90F-1F3FF"},
    {"fjes_med_glidelasmunn", "1F910"},
    {"pengegal", "1F911"},
    {"fjes_med_termometer", "1F912"},
    {"nerdefjes", "1F913"},
    {"tenkefjes", "1F914"},
    {"bandasjert_hode", "1F915"},
    {"robot", "1F916"},
    {"fjes_som_gir_klem", "1F917"},
    {"rocknroll", "1F918"},
    {"rocknroll_farge1", "1F918-1F3FB"},
    {"rocknroll_farge2", "1F918-1F3FC"},
    {"rocknroll_farge3", "1F918-1F3FD"},
    {"rocknroll_farge4", "1F918-1F3FE"},
    {"rocknroll_farge5", "1F918-1F3FF"},
    {"ring_meg_hand", "1F919"},
    {"ring_meg_hand_farge1", "1F919-1F3FB"},
    {"ring_meg_hand_farge2", "1F919-1F3FC"},
    {"ring_meg_hand_farge3", "1F919-1F3FD"},
    {"ring_meg_hand_farge4", "1F919-1F3FE"},
    {"ring_meg_hand_farge5", "1F919-1F3FF"},
    {"loftet_handbak", "1F91A"},
    {"loftet_handbak_farge1", "1F91A-1F3FB"},
    {"loftet_handbak_farge2", "1F91A-1F3FC"},
    {"loftet_handbak_farge3", "1F91A-1F3FD"},
    {"loftet_handbak_farge4", "1F91A-1F3FE"},
    {"loftet_handbak_farge5", "1F91A-1F3FF"},
    {"knyttneve_mot_venstre", "1F91B"},
    {"knyttneve_mot_venstre_farge1", "1F91B-1F3FB"},
    {"knyttneve_mot_venstre_farge2", "1F91B-1F3FC"},
    {"knyttneve_mot_venstre_farge3", "1F91B-1F3FD"},
    {"knyttneve_mot_venstre_farge4", "1F91B-1F3FE"},
    {"knyttneve_mot_venstre_farge5", "1F91B-1F3FF"},
    {"knyttneve_mot_hoyre", "1F91C"},
    {"knyttneve_mot_hoyre_farge1", "1F91C-1F3FB"},
    {"knyttneve_mot_hoyre_farge2", "1F91C-1F3FC"},
    {"knyttneve_mot_hoyre_farge3", "1F91C-1F3FD"},
    {"knyttneve_mot_hoyre_farge4", "1F91C-1F3FE"},
    {"knyttneve_mot_hoyre_farge5", "1F91C-1F3FF"},
    {"handhilse", "1F91D"},
    {"handhilse_farge1", "1F91D-1F3FB"},
    {"handhilse_farge2", "1F91D-1F3FC"},
    {"handhilse_farge3", "1F91D-1F3FD"},
    {"handhilse_farge4", "1F91D-1F3FE"},
    {"handhilse_farge5", "1F91D-1F3FF"},
    {"handhilse_farge1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"handhilse_farge1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"handhilse_farge1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"handhilse_farge1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"handhilse_farge2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"handhilse_farge2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"handhilse_farge2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"handhilse_farge2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"handhilse_farge3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"handhilse_farge3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"handhilse_farge3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"handhilse_farge3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"handhilse_farge4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"handhilse_farge4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"handhilse_farge4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"handhilse_farge4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"handhilse_farge5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"handhilse_farge5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"handhilse_farge5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"handhilse_farge5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"kryssede_fingre", "1F91E"},
    {"kryssede_fingre_farge1", "1F91E-1F3FB"},
    {"kryssede_fingre_farge2", "1F91E-1F3FC"},
    {"kryssede_fingre_farge3", "1F91E-1F3FD"},
    {"kryssede_fingre_farge4", "1F91E-1F3FE"},
    {"kryssede_fingre_farge5", "1F91E-1F3FF"},
    {"glad_i_deg_tegn", "1F91F"},
    {"glad_i_deg_tegn_farge1", "1F91F-1F3FB"},
    {"glad_i_deg_tegn_farge2", "1F91F-1F3FC"},
    {"glad_i_deg_tegn_farge3", "1F91F-1F3FD"},
    {"glad_i_deg_tegn_farge4", "1F91F-1F3FE"},
    {"glad_i_deg_tegn_farge5", "1F91F-1F3FF"},
    {"fjes_med_cowboyhatt", "1F920"},
    {"klovnefjes", "1F921"},
    {"kvalm", "1F922"},
    {"ruller_rundt_pa_gulvet_i_latter", "1F923"},
    {"sikler", "1F924"},
    {"lyver", "1F925"},
    {"slar_handen_mot_pannen", "1F926"},
    {"slar_handen_mot_pannen_farge1", "1F926-1F3FB"},
    {"slar_handen_mot_pannen_farge2", "1F926-1F3FC"},
    {"slar_handen_mot_pannen_farge3", "1F926-1F3FD"},
    {"slar_handen_mot_pannen_farge4", "1F926-1F3FE"},
    {"slar_handen_mot_pannen_farge5", "1F926-1F3FF"},
    {"nyser", "1F927"},
    {"hevet_oyenbryn", "1F928"},
    {"stjerner_i_oynene", "1F929"},
    {"gal", "1F92A"},
    {"hysj", "1F92B"},
    {"fjes_med_symboler_over_munnen", "1F92C"},
    {"handen_over_munnen", "1F92D"},
    {"kaster_opp", "1F92E"},
    {"eksploderende_hode", "1F92F"},
    {"gravid_kvinne", "1F930"},
    {"gravid_kvinne_farge1", "1F930-1F3FB"},
    {"gravid_kvinne_farge2", "1F930-1F3FC"},
    {"gravid_kvinne_farge3", "1F930-1F3FD"},
    {"gravid_kvinne_farge4", "1F930-1F3FE"},
    {"gravid_kvinne_farge5", "1F930-1F3FF"},
    {"ammer", "1F931"},
    {"ammer_farge1", "1F931-1F3FB"},
    {"ammer_farge2", "1F931-1F3FC"},
    {"ammer_farge3", "1F931-1F3FD"},
    {"ammer_farge4", "1F931-1F3FE"},
    {"ammer_farge5", "1F931-1F3FF"},
    {"handflatene_sammen", "1F932"},
    {"handflatene_sammen_farge1", "1F932-1F3FB"},
    {"handflatene_sammen_farge2", "1F932-1F3FC"},
    {"handflatene_sammen_farge3", "1F932-1F3FD"},
    {"handflatene_sammen_farge4", "1F932-1F3FE"},
    {"handflatene_sammen_farge5", "1F932-1F3FF"},
    {"selfie", "1F933"},
    {"selfie_farge1", "1F933-1F3FB"},
    {"selfie_farge2", "1F933-1F3FC"},
    {"selfie_farge3", "1F933-1F3FD"},
    {"selfie_farge4", "1F933-1F3FE"},
    {"selfie_farge5", "1F933-1F3FF"},
    {"prins", "1F934"},
    {"prins_farge1", "1F934-1F3FB"},
    {"prins_farge2", "1F934-1F3FC"},
    {"prins_farge3", "1F934-1F3FD"},
    {"prins_farge4", "1F934-1F3FE"},
    {"prins_farge5", "1F934-1F3FF"},
    {"person_i_smoking", "1F935"},
    {"person_i_smoking_farge1", "1F935-1F3FB"},
    {"person_i_smoking_farge2", "1F935-1F3FC"},
    {"person_i_smoking_farge3", "1F935-1F3FD"},
    {"person_i_smoking_farge4", "1F935-1F3FE"},
    {"person_i_smoking_farge5", "1F935-1F3FF"},
    {"julenissemor", "1F936"},
    {"julenissemor_farge1", "1F936-1F3FB"},
    {"julenissemor_farge2", "1F936-1F3FC"},
    {"julenissemor_farge3", "1F936-1F3FD"},
    {"julenissemor_farge4", "1F936-1F3FE"},
    {"julenissemor_farge5", "1F936-1F3FF"},
    {"trekker_pa_skuldrene", "1F937"},
    {"trekker_pa_skuldrene_farge1", "1F937-1F3FB"},
    {"trekker_pa_skuldrene_farge2", "1F937-1F3FC"},
    {"trekker_pa_skuldrene_farge3", "1F937-1F3FD"},
    {"trekker_pa_skuldrene_farge4", "1F937-1F3FE"},
    {"trekker_pa_skuldrene_farge5", "1F937-1F3FF"},
    {"sla_hjul", "1F938"},
    {"sla_hjul_farge1", "1F938-1F3FB"},
    {"sla_hjul_farge2", "1F938-1F3FC"},
    {"sla_hjul_farge3", "1F938-1F3FD"},
    {"sla_hjul_farge4", "1F938-1F3FE"},
    {"sla_hjul_farge5", "1F938-1F3FF"},
    {"sjonglering", "1F939"},
    {"sjonglering_farge1", "1F939-1F3FB"},
    {"sjonglering_farge2", "1F939-1F3FC"},
    {"sjonglering_farge3", "1F939-1F3FD"},
    {"sjonglering_farge4", "1F939-1F3FE"},
    {"sjonglering_farge5", "1F939-1F3FF"},
    {"fekter", "1F93A"},
    {"brytere", "1F93C"},
    {"brytere_farge1", "1F93C-1F3FB"},
    {"brytere_farge2", "1F93C-1F3FC"},
    {"brytere_farge3", "1F93C-1F3FD"},
    {"brytere_farge4", "1F93C-1F3FE"},
    {"brytere_farge5", "1F93C-1F3FF"},
    {"brytere_farge1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"brytere_farge1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"brytere_farge1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"brytere_farge1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"brytere_farge2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"brytere_farge2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"brytere_farge2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"brytere_farge2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"brytere_farge3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"brytere_farge3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"brytere_farge3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"brytere_farge3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"brytere_farge4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"brytere_farge4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"brytere_farge4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"brytere_farge4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"brytere_farge5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"brytere_farge5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"brytere_farge5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"brytere_farge5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"vannpolo", "1F93D"},
    {"vannpolo_farge1", "1F93D-1F3FB"},
    {"vannpolo_farge2", "1F93D-1F3FC"},
    {"vannpolo_farge3", "1F93D-1F3FD"},
    {"vannpolo_farge4", "1F93D-1F3FE"},
    {"vannpolo_farge5", "1F93D-1F3FF"},
    {"handball", "1F93E"},
    {"handball_farge1", "1F93E-1F3FB"},
    {"handball_farge2", "1F93E-1F3FC"},
    {"handball_farge3", "1F93E-1F3FD"},
    {"handball_farge4", "1F93E-1F3FE"},
    {"handball_farge5", "1F93E-1F3FF"},
    {"dykkermaske", "1F93F"},
    {"vissen_blomst", "1F940"},
    {"tromme", "1F941"},
    {"glass_som_skaler", "1F942"},
    {"whiskyglass", "1F943"},
    {"skje", "1F944"},
    {"malnett", "1F945"},
    {"gullmedalje", "1F947"},
    {"solvmedalje", "1F948"},
    {"bronsemedalje", "1F949"},
    {"boksehanske", "1F94A"},
    {"kampsportdrakt", "1F94B"},
    {"curlingstein", "1F94C"},
    {"lacrosse", "1F94D"},
    {"softball", "1F94E"},
    {"frisbee", "1F94F"},
    {"croissant", "1F950"},
    {"avokado", "1F951"},
    {"agurk", "1F952"},
    {"bacon", "1F953"},
    {"potet", "1F954"},
    {"gulrot", "1F955"},
    {"bagett", "1F956"},
    {"gronn_salat", "1F957"},
    {"lav_gryte_med_mat", "1F958"},
    {"fylt_pitabrod", "1F959"},
    {"egg", "1F95A"},
    {"melkeglass", "1F95B"},
    {"peanotter", "1F95C"},
    {"kiwi", "1F95D"},
    {"pannekaker", "1F95E"},
    {"dumpling", "1F95F"},
    {"lykkekake", "1F960"},
    {"takeaway_eske", "1F961"},
    {"spisepinner", "1F962"},
    {"bolle_med_skje", "1F963"},
    {"kopp_med_sugeror", "1F964"},
    {"kokosnott", "1F965"},
    {"brokkoli", "1F966"},
    {"pai", "1F967"},
    {"saltkringle", "1F968"},
    {"kjottstykke", "1F969"},
    {"sandwich", "1F96A"},
    {"hermetikk", "1F96B"},
    {"bladgronnsaker", "1F96C"},
    {"mango", "1F96D"},
    {"manekake", "1F96E"},
    {"bagel", "1F96F"},
    {"smilefjes_med_hjerter", "1F970"},
    {"gjespende_fjes", "1F971"},
    {"smilefjes_med_tare", "1F972"},
    {"partyfjes", "1F973"},
    {"svimmelt_fjes", "1F974"},
    {"varmt_fjes", "1F975"},
    {"kaldt_fjes", "1F976"},
    {"ninja", "1F977"},
    {"ninja_farge1", "1F977-1F3FB"},
    {"ninja_farge2", "1F977-1F3FC"},
    {"ninja_farge3", "1F977-1F3FD"},
    {"ninja_farge4", "1F977-1F3FE"},
    {"ninja_farge5", "1F977-1F3FF"},
    {"fjes_med_forkledning", "1F978"},
    {"fjes_som_holder_tilbake_tarene", "1F979"},
    {"bedende_fjes", "1F97A"},
    {"sari", "1F97B"},
    {"labfrakk", "1F97C"},
    {"beskyttelsesbriller", "1F97D"},
    {"turstovel", "1F97E"},
    {"pumps", "1F97F"},
    {"krabbe", "1F980"},
    {"love", "1F981"},
    {"skorpion", "1F982"},
    {"kalkun", "1F983"},
    {"enhjorning", "1F984"},
    {"orn", "1F985"},
    {"and", "1F9DE"},
    {"flaggermus", "1F987"},
    {"hai", "1F988"},
    {"ugle", "1F989"},
    {"rev", "1F98A"},
    {"sommerfugl", "1F98B"},
    {"hjort", "1F98C"},
    {"gorilla", "1F98D"},
    {"firfisle", "1F98E"},
    {"neshorn", "1F98F"},
    {"reke", "1F990"},
    {"akkar", "1F991"},
    {"sjiraff", "1F992"},
    {"sebra", "1F993"},
    {"pinnsvin", "1F994"},
    {"sauropod", "1F995"},
    {"trex", "1F996"},
    {"gresshoppe", "1F997"},
    {"kenguru", "1F998"},
    {"lama", "1F999"},
    {"pafugl", "1F99A"},
    {"flodhest", "1F99B"},
    {"papegoye", "1F99C"},
    {"vaskebjorn", "1F99D"},
    {"hummer", "1F99E"},
    {"mygg", "1F99F"},
    {"mikrobe", "1F9A0"},
    {"grevling", "1F9A1"},
    {"svane", "1F9A2"},
    {"mammut", "1F9A3"},
    {"dronte", "1F9A4"},
    {"dovendyr", "1F9A5"},
    {"oter", "1F9A6"},
    {"orangutang", "1F9A7"},
    {"stinkdyr", "1F9A8"},
    {"flamingo", "1F9A9"},
    {"osters", "1F9AA"},
    {"bever", "1F9AB"},
    {"bison", "1F9AC"},
    {"sel", "1F9AD"},
    {"forerhund", "1F9AE"},
    {"blindestokk", "1F9AF"},
    {"rodt_har", "1F9B0"},
    {"kruset_har", "1F9B1"},
    {"skallet", "1F9B2"},
    {"gratt_har", "1F9B3"},
    {"knokkel", "1F9B4"},
    {"bein", "1F9B5"},
    {"bein_farge1", "1F9B5-1F3FB"},
    {"bein_farge2", "1F9B5-1F3FC"},
    {"bein_farge3", "1F9B5-1F3FD"},
    {"bein_farge4", "1F9B5-1F3FE"},
    {"bein_farge5", "1F9B5-1F3FF"},
    {"fot", "1F9B6"},
    {"fot_farge1", "1F9B6-1F3FB"},
    {"fot_farge2", "1F9B6-1F3FC"},
    {"fot_farge3", "1F9B6-1F3FD"},
    {"fot_farge4", "1F9B6-1F3FE"},
    {"fot_farge5", "1F9B6-1F3FF"},
    {"tann", "1F9B7"},
    {"superhelt", "1F9B8"},
    {"superhelt_farge1", "1F9B8-1F3FB"},
    {"superhelt_farge2", "1F9B8-1F3FC"},
    {"superhelt_farge3", "1F9B8-1F3FD"},
    {"superhelt_farge4", "1F9B8-1F3FE"},
    {"superhelt_farge5", "1F9B8-1F3FF"},
    {"superskurk", "1F9B9"},
    {"superskurk_farge1", "1F9B9-1F3FB"},
    {"superskurk_farge2", "1F9B9-1F3FC"},
    {"superskurk_farge3", "1F9B9-1F3FD"},
    {"superskurk_farge4", "1F9B9-1F3FE"},
    {"superskurk_farge5", "1F9B9-1F3FF"},
    {"refleksvest", "1F9BA"},
    {"ore_med_horeapparat", "1F9BB"},
    {"ore_med_horeapparat_farge1", "1F9BB-1F3FB"},
    {"ore_med_horeapparat_farge2", "1F9BB-1F3FC"},
    {"ore_med_horeapparat_farge3", "1F9BB-1F3FD"},
    {"ore_med_horeapparat_farge4", "1F9BB-1F3FE"},
    {"ore_med_horeapparat_farge5", "1F9BB-1F3FF"},
    {"motorisert_rullestol", "1F9BC"},
    {"manuell_rullestol", "1F9BD"},
    {"mekanisk_arm", "1F9BE"},
    {"mekanisk_bein", "1F9BF"},
    {"ostestykke", "1F9C0"},
    {"cupcake", "1F9C1"},
    {"salt", "1F9C2"},
    {"drikkekartong", "1F9C3"},
    {"hvitlok", "1F9C4"},
    {"lok", "1F9C5"},
    {"falafel", "1F9C6"},
    {"belgisk_vaffel", "1F9C7"},
    {"smor", "1F9C8"},
    {"mate", "1F9C9"},
    {"isterning", "1F9CA"},
    {"boble_te", "1F9CB"},
    {"troll", "1F9CC"},
    {"person_som_star", "1F9CD"},
    {"person_som_star_farge1", "1F9CD-1F3FB"},
    {"person_som_star_farge2", "1F9CD-1F3FC"},
    {"person_som_star_farge3", "1F9CD-1F3FD"},
    {"person_som_star_farge4", "1F9CD-1F3FE"},
    {"person_som_star_farge5", "1F9CD-1F3FF"},
    {"person_som_kneler", "1F9CE"},
    {"person_som_kneler_farge1", "1F9CE-1F3FB"},
    {"person_som_kneler_farge2", "1F9CE-1F3FC"},
    {"person_som_kneler_farge3", "1F9CE-1F3FD"},
    {"person_som_kneler_farge4", "1F9CE-1F3FE"},
    {"person_som_kneler_farge5", "1F9CE-1F3FF"},
    {"dov_person", "1F9CF"},
    {"dov_person_farge1", "1F9CF-1F3FB"},
    {"dov_person_farge2", "1F9CF-1F3FC"},
    {"dov_person_farge3", "1F9CF-1F3FD"},
    {"dov_person_farge4", "1F9CF-1F3FE"},
    {"dov_person_farge5", "1F9CF-1F3FF"},
    {"fjes_med_monokkel", "1F9D0"},
    {"voksen", "1F9D1"},
    {"voksen_farge1", "1F9D1-1F3FB"},
    {"voksen_farge2", "1F9D1-1F3FC"},
    {"voksen_farge3", "1F9D1-1F3FD"},
    {"voksen_farge4", "1F9D1-1F3FE"},
    {"voksen_farge5", "1F9D1-1F3FF"},
    {"barn", "1F9D2"},
    {"barn_farge1", "1F9D2-1F3FB"},
    {"barn_farge2", "1F9D2-1F3FC"},
    {"barn_farge3", "1F9D2-1F3FD"},
    {"barn_farge4", "1F9D2-1F3FE"},
    {"barn_farge5", "1F9D2-1F3FF"},
    {"eldre_voksen", "1F9D3"},
    {"eldre_voksen_farge1", "1F9D3-1F3FB"},
    {"eldre_voksen_farge2", "1F9D3-1F3FC"},
    {"eldre_voksen_farge3", "1F9D3-1F3FD"},
    {"eldre_voksen_farge4", "1F9D3-1F3FE"},
    {"eldre_voksen_farge5", "1F9D3-1F3FF"},
    {"person_skjegg", "1F9D4"},
    {"person_skjegg_farge1", "1F9D4-1F3FB"},
    {"person_skjegg_farge2", "1F9D4-1F3FC"},
    {"person_skjegg_farge3", "1F9D4-1F3FD"},
    {"person_skjegg_farge4", "1F9D4-1F3FE"},
    {"person_skjegg_farge5", "1F9D4-1F3FF"},
    {"kvinne_med_hodeslor", "1F9D5"},
    {"kvinne_med_hodeslor_farge1", "1F9D5-1F3FB"},
    {"kvinne_med_hodeslor_farge2", "1F9D5-1F3FC"},
    {"kvinne_med_hodeslor_farge3", "1F9D5-1F3FD"},
    {"kvinne_med_hodeslor_farge4", "1F9D5-1F3FE"},
    {"kvinne_med_hodeslor_farge5", "1F9D5-1F3FF"},
    {"person_i_badstu", "1F9D6"},
    {"person_i_badstu_farge1", "1F9D6-1F3FB"},
    {"person_i_badstu_farge2", "1F9D6-1F3FC"},
    {"person_i_badstu_farge3", "1F9D6-1F3FD"},
    {"person_i_badstu_farge4", "1F9D6-1F3FE"},
    {"person_i_badstu_farge5", "1F9D6-1F3FF"},
    {"klatrer", "1F9D7"},
    {"klatrer_farge1", "1F9D7-1F3FB"},
    {"klatrer_farge2", "1F9D7-1F3FC"},
    {"klatrer_farge3", "1F9D7-1F3FD"},
    {"klatrer_farge4", "1F9D7-1F3FE"},
    {"klatrer_farge5", "1F9D7-1F3FF"},
    {"lotusstilling", "1F9D8"},
    {"lotusstilling_farge1", "1F9D8-1F3FB"},
    {"lotusstilling_farge2", "1F9D8-1F3FC"},
    {"lotusstilling_farge3", "1F9D8-1F3FD"},
    {"lotusstilling_farge4", "1F9D8-1F3FE"},
    {"lotusstilling_farge5", "1F9D8-1F3FF"},
    {"trollperson", "1F9D9"},
    {"trollperson_farge1", "1F9D9-1F3FB"},
    {"trollperson_farge2", "1F9D9-1F3FC"},
    {"trollperson_farge3", "1F9D9-1F3FD"},
    {"trollperson_farge4", "1F9D9-1F3FE"},
    {"trollperson_farge5", "1F9D9-1F3FF"},
    {"fe", "1F9DA"},
    {"fe_farge1", "1F9DA-1F3FB"},
    {"fe_farge2", "1F9DA-1F3FC"},
    {"fe_farge3", "1F9DA-1F3FD"},
    {"fe_farge4", "1F9DA-1F3FE"},
    {"fe_farge5", "1F9DA-1F3FF"},
    {"vampyr", "1F9DB"},
    {"vampyr_farge1", "1F9DB-1F3FB"},
    {"vampyr_farge2", "1F9DB-1F3FC"},
    {"vampyr_farge3", "1F9DB-1F3FD"},
    {"vampyr_farge4", "1F9DB-1F3FE"},
    {"vampyr_farge5", "1F9DB-1F3FF"},
    {"havperson", "1F9DC"},
    {"havperson_farge1", "1F9DC-1F3FB"},
    {"havperson_farge2", "1F9DC-1F3FC"},
    {"havperson_farge3", "1F9DC-1F3FD"},
    {"havperson_farge4", "1F9DC-1F3FE"},
    {"havperson_farge5", "1F9DC-1F3FF"},
    {"alv", "1F9DD"},
    {"alv_farge1", "1F9DD-1F3FB"},
    {"alv_farge2", "1F9DD-1F3FC"},
    {"alv_farge3", "1F9DD-1F3FD"},
    {"alv_farge4", "1F9DD-1F3FE"},
    {"alv_farge5", "1F9DD-1F3FF"},
    {"zombie", "1F9DF"},
    {"hjerne", "1F9E0"},
    {"oransje_hjerte", "1F9E1"},
    {"caps", "1F9E2"},
    {"skjerf", "1F9E3"},
    {"hansker", "1F9E4"},
    {"jakke", "1F9E5"},
    {"sokker", "1F9E6"},
    {"rod_konvolutt", "1F9E7"},
    {"kinaputt", "1F9E8"},
    {"puslespillbrikke", "1F9E9"},
    {"proveror", "1F9EA"},
    {"petriskal", "1F9EB"},
    {"dna", "1F9EC"},
    {"kompass", "1F9ED"},
    {"kuleramme", "1F9EE"},
    {"brannslukningsapparat", "1F9EF"},
    {"verktoykasse", "1F9F0"},
    {"murstein", "1F9F1"},
    {"magnet", "1F9F2"},
    {"bagasje", "1F9F3"},
    {"lotionflaske", "1F9F4"},
    {"trad", "1F9F5"},
    {"garn", "1F9F6"},
    {"sikkerhetsnal", "1F9F7"},
    {"teddybjorn", "1F9F8"},
    {"feiekost", "1F9F9"},
    {"kurv", "1F9FA"},
    {"rull_med_papir", "1F9FB"},
    {"sape", "1F9FC"},
    {"svamp", "1F9FD"},
    {"kvittering", "1F9FE"},
    {"nazar", "1F9FF"},
    {"ballettsko", "1FA70"},
    {"badedrakt", "1FA71"},
    {"truse", "1FA72"},
    {"shorts", "1FA73"},
    {"flippflopp", "1FA74"},
    {"lyseblatt_hjerte", "1FA75"},
    {"gratt_hjerte", "1FA76"},
    {"rosa_hjerte", "1FA77"},
    {"bloddrape", "1FA78"},
    {"plaster", "1FA79"},
    {"stetoskop", "1FA7A"},
    {"rontgen", "1FA7B"},
    {"krykke", "1FA7C"},
    {"jojo", "1FA80"},
    {"papirdrage", "1FA81"},
    {"fallskjerm", "1FA82"},
    {"bumerang", "1FA83"},
    {"tryllestav", "1FA84"},
    {"pinata", "1FA85"},
    {"matrjosjka", "1FA86"},
    {"marakas", "1FA87"},
    {"floyte", "1FA88"},
    {"harpe", "1FA89"},
    {"trombone", "1FA8A"},
    {"skattkiste", "1FA8E"},
    {"spade", "1FA8F"},
    {"planet_med_ringer", "1FA90"},
    {"stol", "1FA91"},
    {"barberkniv", "1FA92"},
    {"oks", "1FA93"},
    {"apen_oljelampe", "1FA94"},
    {"banjo", "1FA95"},
    {"militaerhjelm", "1FA96"},
    {"trekkspill", "1FA97"},
    {"langtromme", "1FA98"},
    {"mynt", "1FA99"},
    {"handsag", "1FA9A"},
    {"skrutrekker", "1FA9B"},
    {"stige", "1FA9C"},
    {"krok", "1FA9D"},
    {"speil", "1FA9E"},
    {"vindu", "1FA9F"},
    {"avlopsapner", "1FAA0"},
    {"synal", "1FAA1"},
    {"knute", "1FAA2"},
    {"botte", "1FAA3"},
    {"musefelle", "1FAA4"},
    {"tannborste", "1FAA5"},
    {"gravstein", "1FAA6"},
    {"plakat", "1FAA7"},
    {"stein", "1FAA8"},
    {"speilkule", "1FAA9"},
    {"id_kort", "1FAAA"},
    {"lite_batteri", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"handvifte", "1FAAD"},
    {"afrokam", "1FAAE"},
    {"khanda", "1FAAF"},
    {"flue", "1FAB0"},
    {"mark", "1FAB1"},
    {"bille", "1FAB2"},
    {"kakerlakk", "1FAB3"},
    {"potteplante", "1FAB4"},
    {"tre", "1FAB5"},
    {"fjaer", "1FAB6"},
    {"lotusblomst", "1FAB7"},
    {"korall", "1FAB8"},
    {"tomt_rede", "1FAB9"},
    {"rede_med_egg", "1FABA"},
    {"hyasint", "1FABB"},
    {"manet", "1FABC"},
    {"vinge", "1FABD"},
    {"bladlost_tre", "1FABE"},
    {"gas", "1FABF"},
    {"hjertemuskel", "1FAC0"},
    {"lunger", "1FAC1"},
    {"klem", "1FAC2"},
    {"gravid_mann", "1FAC3"},
    {"gravid_mann_farge1", "1FAC3-1F3FB"},
    {"gravid_mann_farge2", "1FAC3-1F3FC"},
    {"gravid_mann_farge3", "1FAC3-1F3FD"},
    {"gravid_mann_farge4", "1FAC3-1F3FE"},
    {"gravid_mann_farge5", "1FAC3-1F3FF"},
    {"gravid_person", "1FAC4"},
    {"gravid_person_farge1", "1FAC4-1F3FB"},
    {"gravid_person_farge2", "1FAC4-1F3FC"},
    {"gravid_person_farge3", "1FAC4-1F3FD"},
    {"gravid_person_farge4", "1FAC4-1F3FE"},
    {"gravid_person_farge5", "1FAC4-1F3FF"},
    {"person_med_krone", "1FAC5"},
    {"person_med_krone_farge1", "1FAC5-1F3FB"},
    {"person_med_krone_farge2", "1FAC5-1F3FC"},
    {"person_med_krone_farge3", "1FAC5-1F3FD"},
    {"person_med_krone_farge4", "1FAC5-1F3FE"},
    {"person_med_krone_farge5", "1FAC5-1F3FF"},
    {"fingeravtrykk", "1FAC6"},
    {"harete_vesen", "1FAC8"},
    {"spekkhogger", "1FACD"},
    {"elg", "1FACE"},
    {"esel", "1FACF"},
    {"blabaer", "1FAD0"},
    {"paprika", "1FAD1"},
    {"oliven", "1FAD2"},
    {"pitabrod", "1FAD3"},
    {"tamal", "1FAD4"},
    {"fondy", "1FAD5"},
    {"tekanne", "1FAD6"},
    {"helle_vaeske", "1FAD7"},
    {"bonner", "1FAD8"},
    {"krukke", "1FAD9"},
    {"ingefaerrot", "1FADA"},
    {"ertebelg", "1FADB"},
    {"rotgronnsak", "1FADC"},
    {"sprut", "1FADF"},
    {"fjes_som_smelter", "1FAE0"},
    {"fjes_som_viser_honnor", "1FAE1"},
    {"fjes_med_apne_oyne_og_hand_over_munnen", "1FAE2"},
    {"fjes_som_titter_frem_bak_hendene", "1FAE3"},
    {"fjes_med_diagonal_munn", "1FAE4"},
    {"fjes_med_stiplet_omriss", "1FAE5"},
    {"bite_seg_i_leppen", "1FAE6"},
    {"bobler", "1FAE7"},
    {"skjelvende_fjes", "1FAE8"},
    {"fjes_med_poser_under_oynene", "1FAE9"},
    {"fordreid_fjes", "1FAEA"},
    {"slasskampsky", "1FAEF"},
    {"hand_med_pekefinger_og_tommel_krysset", "1FAF0"},
    {"hand_med_pekefinger_og_tommel_krysset_farge1", "1FAF0-1F3FB"},
    {"hand_med_pekefinger_og_tommel_krysset_farge2", "1FAF0-1F3FC"},
    {"hand_med_pekefinger_og_tommel_krysset_farge3", "1FAF0-1F3FD"},
    {"hand_med_pekefinger_og_tommel_krysset_farge4", "1FAF0-1F3FE"},
    {"hand_med_pekefinger_og_tommel_krysset_farge5", "1FAF0-1F3FF"},
    {"hand_mot_hoyre", "1FAF1"},
    {"hand_mot_hoyre_farge1", "1FAF1-1F3FB"},
    {"hand_mot_hoyre_farge2", "1FAF1-1F3FC"},
    {"hand_mot_hoyre_farge3", "1FAF1-1F3FD"},
    {"hand_mot_hoyre_farge4", "1FAF1-1F3FE"},
    {"hand_mot_hoyre_farge5", "1FAF1-1F3FF"},
    {"hand_mot_venstre", "1FAF2"},
    {"hand_mot_venstre_farge1", "1FAF2-1F3FB"},
    {"hand_mot_venstre_farge2", "1FAF2-1F3FC"},
    {"hand_mot_venstre_farge3", "1FAF2-1F3FD"},
    {"hand_mot_venstre_farge4", "1FAF2-1F3FE"},
    {"hand_mot_venstre_farge5", "1FAF2-1F3FF"},
    {"handflaten_ned", "1FAF3"},
    {"handflaten_ned_farge1", "1FAF3-1F3FB"},
    {"handflaten_ned_farge2", "1FAF3-1F3FC"},
    {"handflaten_ned_farge3", "1FAF3-1F3FD"},
    {"handflaten_ned_farge4", "1FAF3-1F3FE"},
    {"handflaten_ned_farge5", "1FAF3-1F3FF"},
    {"handflaten_opp", "1FAF4"},
    {"handflaten_opp_farge1", "1FAF4-1F3FB"},
    {"handflaten_opp_farge2", "1FAF4-1F3FC"},
    {"handflaten_opp_farge3", "1FAF4-1F3FD"},
    {"handflaten_opp_farge4", "1FAF4-1F3FE"},
    {"handflaten_opp_farge5", "1FAF4-1F3FF"},
    {"pekefinger_peker_pa_leseren", "1FAF5"},
    {"pekefinger_peker_pa_leseren_farge1", "1FAF5-1F3FB"},
    {"pekefinger_peker_pa_leseren_farge2", "1FAF5-1F3FC"},
    {"pekefinger_peker_pa_leseren_farge3", "1FAF5-1F3FD"},
    {"pekefinger_peker_pa_leseren_farge4", "1FAF5-1F3FE"},
    {"pekefinger_peker_pa_leseren_farge5", "1FAF5-1F3FF"},
    {"hjertehender", "1FAF6"},
    {"hjertehender_farge1", "1FAF6-1F3FB"},
    {"hjertehender_farge2", "1FAF6-1F3FC"},
    {"hjertehender_farge3", "1FAF6-1F3FD"},
    {"hjertehender_farge4", "1FAF6-1F3FE"},
    {"hjertehender_farge5", "1FAF6-1F3FF"},
    {"handflate_hevet_mot_venstre", "1FAF7"},
    {"handflate_hevet_mot_venstre_farge1", "1FAF7-1F3FB"},
    {"handflate_hevet_mot_venstre_farge2", "1FAF7-1F3FC"},
    {"handflate_hevet_mot_venstre_farge3", "1FAF7-1F3FD"},
    {"handflate_hevet_mot_venstre_farge4", "1FAF7-1F3FE"},
    {"handflate_hevet_mot_venstre_farge5", "1FAF7-1F3FF"},
    {"handflate_hevet_mot_hoyre", "1FAF8"},
    {"handflate_hevet_mot_hoyre_farge1", "1FAF8-1F3FB"},
    {"handflate_hevet_mot_hoyre_farge2", "1FAF8-1F3FC"},
    {"handflate_hevet_mot_hoyre_farge3", "1FAF8-1F3FD"},
    {"handflate_hevet_mot_hoyre_farge4", "1FAF8-1F3FE"},
    {"handflate_hevet_mot_hoyre_farge5", "1FAF8-1F3FF"},
    {"taster_nummertegn", "0023-FE0F-20E3"},
    {"taster_stjernen", "002A-FE0F-20E3"},
    {"taster_0", "0030-FE0F-20E3"},
    {"taster_1", "0031-FE0F-20E3"},
    {"taster_2", "0032-FE0F-20E3"},
    {"taster_3", "0033-FE0F-20E3"},
    {"taster_4", "0034-FE0F-20E3"},
    {"taster_5", "0035-FE0F-20E3"},
    {"taster_6", "0036-FE0F-20E3"},
    {"taster_7", "0037-FE0F-20E3"},
    {"taster_8", "0038-FE0F-20E3"},
    {"taster_9", "0039-FE0F-20E3"},
    {"flagg_ascension", "1F1E6-1F1E8"},
    {"flagg_andorra", "1F1E6-1F1E9"},
    {"flagg_de_forente_arabiske_emirater", "1F1E6-1F1EA"},
    {"flagg_afghanistan", "1F1E6-1F1EB"},
    {"flagg_antigua_og_barbuda", "1F1E6-1F1EC"},
    {"flagg_anguilla", "1F1E6-1F1EE"},
    {"flagg_albania", "1F1E6-1F1F1"},
    {"flagg_armenia", "1F1E6-1F1F2"},
    {"flagg_angola", "1F1E6-1F1F4"},
    {"flagg_antarktis", "1F1E6-1F1F6"},
    {"flagg_argentina", "1F1E6-1F1F7"},
    {"flagg_amerikansk_samoa", "1F1E6-1F1F8"},
    {"flagg_osterrike", "1F1E6-1F1F9"},
    {"flagg_australia", "1F1E6-1F1FA"},
    {"flagg_aruba", "1F1E6-1F1FC"},
    {"flagg_aland", "1F1E6-1F1FD"},
    {"flagg_aserbajdsjan", "1F1E6-1F1FF"},
    {"flagg_bosnia_hercegovina", "1F1E7-1F1E6"},
    {"flagg_barbados", "1F1E7-1F1E7"},
    {"flagg_bangladesh", "1F1E7-1F1E9"},
    {"flagg_belgia", "1F1E7-1F1EA"},
    {"flagg_burkina_faso", "1F1E7-1F1EB"},
    {"flagg_bulgaria", "1F1E7-1F1EC"},
    {"flagg_bahrain", "1F1E7-1F1ED"},
    {"flagg_burundi", "1F1E7-1F1EE"},
    {"flagg_benin", "1F1E7-1F1EF"},
    {"flagg_saint_barthelemy", "1F1E7-1F1F1"},
    {"flagg_bermuda", "1F1E7-1F1F2"},
    {"flagg_brunei", "1F1E7-1F1F3"},
    {"flagg_bolivia", "1F1E7-1F1F4"},
    {"flagg_karibisk_nederland", "1F1E7-1F1F6"},
    {"flagg_brasil", "1F1E7-1F1F7"},
    {"flagg_bahamas", "1F1E7-1F1F8"},
    {"flagg_bhutan", "1F1E7-1F1F9"},
    {"flagg_bouvetoya", "1F1E7-1F1FB"},
    {"flagg_botswana", "1F1E7-1F1FC"},
    {"flagg_belarus", "1F1E7-1F1FE"},
    {"flagg_belize", "1F1E7-1F1FF"},
    {"flagg_canada", "1F1E8-1F1E6"},
    {"flagg_kokosoyene", "1F1E8-1F1E8"},
    {"flagg_kongo", "1F1E8-1F1E9"},
    {"flagg_den_sentralafrikanske_republikk", "1F1E8-1F1EB"},
    {"flagg_kongo_brazzaville", "1F1E8-1F1EC"},
    {"flagg_sveits", "1F1E8-1F1ED"},
    {"flagg_elfenbenskysten", "1F1E8-1F1EE"},
    {"flagg_cookoyene", "1F1E8-1F1F0"},
    {"flagg_chile", "1F1E8-1F1F1"},
    {"flagg_kamerun", "1F1E8-1F1F2"},
    {"flagg_kina", "1F1E8-1F1F3"},
    {"flagg_colombia", "1F1E8-1F1F4"},
    {"flagg_clippertonoya", "1F1E8-1F1F5"},
    {"flagg_sark", "1F1E8-1F1F6"},
    {"flagg_costa_rica", "1F1E8-1F1F7"},
    {"flagg_cuba", "1F1E8-1F1FA"},
    {"flagg_kapp_verde", "1F1E8-1F1FB"},
    {"flagg_curacao", "1F1E8-1F1FC"},
    {"flagg_christmasoya", "1F1E8-1F1FD"},
    {"flagg_kypros", "1F1E8-1F1FE"},
    {"flagg_tsjekkia", "1F1E8-1F1FF"},
    {"flagg_tyskland", "1F1E9-1F1EA"},
    {"flagg_diego_garcia", "1F1E9-1F1EC"},
    {"flagg_djibouti", "1F1E9-1F1EF"},
    {"flagg_danmark", "1F1E9-1F1F0"},
    {"flagg_dominica", "1F1E9-1F1F2"},
    {"flagg_den_dominikanske_republikk", "1F1E9-1F1F4"},
    {"flagg_algerie", "1F1E9-1F1FF"},
    {"flagg_ceuta_og_melilla", "1F1EA-1F1E6"},
    {"flagg_ecuador", "1F1EA-1F1E8"},
    {"flagg_estland", "1F1EA-1F1EA"},
    {"flagg_egypt", "1F1EA-1F1EC"},
    {"flagg_vest_sahara", "1F1EA-1F1ED"},
    {"flagg_eritrea", "1F1EA-1F1F7"},
    {"flagg_spania", "1F1EA-1F1F8"},
    {"flagg_etiopia", "1F1EA-1F1F9"},
    {"flagg_den_europeiske_union", "1F1EA-1F1FA"},
    {"flagg_finland", "1F1EB-1F1EE"},
    {"flagg_fiji", "1F1EB-1F1EF"},
    {"flagg_falklandsoyene", "1F1EB-1F1F0"},
    {"flagg_mikronesiafoderasjonen", "1F1EB-1F1F2"},
    {"flagg_faeroyene", "1F1EB-1F1F4"},
    {"flagg_frankrike", "1F1EB-1F1F7"},
    {"flagg_gabon", "1F1EC-1F1E6"},
    {"flagg_storbritannia", "1F1EC-1F1E7"},
    {"flagg_grenada", "1F1EC-1F1E9"},
    {"flagg_georgia", "1F1EC-1F1EA"},
    {"flagg_fransk_guyana", "1F1EC-1F1EB"},
    {"flagg_guernsey", "1F1EC-1F1EC"},
    {"flagg_ghana", "1F1EC-1F1ED"},
    {"flagg_gibraltar", "1F1EC-1F1EE"},
    {"flagg_gronland", "1F1EC-1F1F1"},
    {"flagg_gambia", "1F1EC-1F1F2"},
    {"flagg_guinea", "1F1EC-1F1F3"},
    {"flagg_guadeloupe", "1F1EC-1F1F5"},
    {"flagg_ekvatorial_guinea", "1F1EC-1F1F6"},
    {"flagg_hellas", "1F1EC-1F1F7"},
    {"flagg_sor_georgia_og_sor_sandwichoyene", "1F1EC-1F1F8"},
    {"flagg_guatemala", "1F1EC-1F1F9"},
    {"flagg_guam", "1F1EC-1F1FA"},
    {"flagg_guinea_bissau", "1F1EC-1F1FC"},
    {"flagg_guyana", "1F1EC-1F1FE"},
    {"flagg_hongkong_sar_kina", "1F1ED-1F1F0"},
    {"flagg_heard_og_mcdonaldoyene", "1F1ED-1F1F2"},
    {"flagg_honduras", "1F1ED-1F1F3"},
    {"flagg_kroatia", "1F1ED-1F1F7"},
    {"flagg_haiti", "1F1ED-1F1F9"},
    {"flagg_ungarn", "1F1ED-1F1FA"},
    {"flagg_kanarioyene", "1F1EE-1F1E8"},
    {"flagg_indonesia", "1F1EE-1F1E9"},
    {"flagg_irland", "1F1EE-1F1EA"},
    {"flagg_israel", "1F1EE-1F1F1"},
    {"flagg_man", "1F1EE-1F1F2"},
    {"flagg_india", "1F1EE-1F1F3"},
    {"flagg_det_britiske_territoriet_i_indiahavet", "1F1EE-1F1F4"},
    {"flagg_irak", "1F1EE-1F1F6"},
    {"flagg_iran", "1F1EE-1F1F7"},
    {"flagg_island", "1F1EE-1F1F8"},
    {"flagg_italia", "1F1EE-1F1F9"},
    {"flagg_jersey", "1F1EF-1F1EA"},
    {"flagg_jamaica", "1F1EF-1F1F2"},
    {"flagg_jordan", "1F1EF-1F1F4"},
    {"flagg_japan", "1F1EF-1F1F5"},
    {"flagg_kenya", "1F1F0-1F1EA"},
    {"flagg_kirgisistan", "1F1F0-1F1EC"},
    {"flagg_kambodsja", "1F1F0-1F1ED"},
    {"flagg_kiribati", "1F1F0-1F1EE"},
    {"flagg_komorene", "1F1F0-1F1F2"},
    {"flagg_saint_kitts_og_nevis", "1F1F0-1F1F3"},
    {"flagg_nord_korea", "1F1F0-1F1F5"},
    {"flagg_sor_korea", "1F1F0-1F1F7"},
    {"flagg_kuwait", "1F1F0-1F1FC"},
    {"flagg_caymanoyene", "1F1F0-1F1FE"},
    {"flagg_kasakhstan", "1F1F0-1F1FF"},
    {"flagg_laos", "1F1F1-1F1E6"},
    {"flagg_libanon", "1F1F1-1F1E7"},
    {"flagg_st_lucia", "1F1F1-1F1E8"},
    {"flagg_liechtenstein", "1F1F1-1F1EE"},
    {"flagg_sri_lanka", "1F1F1-1F1F0"},
    {"flagg_liberia", "1F1F1-1F1F7"},
    {"flagg_lesotho", "1F1F1-1F1F8"},
    {"flagg_litauen", "1F1F1-1F1F9"},
    {"flagg_luxemburg", "1F1F1-1F1FA"},
    {"flagg_latvia", "1F1F1-1F1FB"},
    {"flagg_libya", "1F1F1-1F1FE"},
    {"flagg_marokko", "1F1F2-1F1E6"},
    {"flagg_monaco", "1F1F2-1F1E8"},
    {"flagg_moldova", "1F1F2-1F1E9"},
    {"flagg_montenegro", "1F1F2-1F1EA"},
    {"flagg_saint_martin", "1F1F2-1F1EB"},
    {"flagg_madagaskar", "1F1F2-1F1EC"},
    {"flagg_marshalloyene", "1F1F2-1F1ED"},
    {"flagg_nord_makedonia", "1F1F2-1F1F0"},
    {"flagg_mali", "1F1F2-1F1F1"},
    {"flagg_myanmar_burma", "1F1F2-1F1F2"},
    {"flagg_mongolia", "1F1F2-1F1F3"},
    {"flagg_macao_sar_kina", "1F1F2-1F1F4"},
    {"flagg_nord_marianene", "1F1F2-1F1F5"},
    {"flagg_martinique", "1F1F2-1F1F6"},
    {"flagg_mauritania", "1F1F2-1F1F7"},
    {"flagg_montserrat", "1F1F2-1F1F8"},
    {"flagg_malta", "1F1F2-1F1F9"},
    {"flagg_mauritius", "1F1F2-1F1FA"},
    {"flagg_maldivene", "1F1F2-1F1FB"},
    {"flagg_malawi", "1F1F2-1F1FC"},
    {"flagg_mexico", "1F1F2-1F1FD"},
    {"flagg_malaysia", "1F1F2-1F1FE"},
    {"flagg_mosambik", "1F1F2-1F1FF"},
    {"flagg_namibia", "1F1F3-1F1E6"},
    {"flagg_ny_caledonia", "1F1F3-1F1E8"},
    {"flagg_niger", "1F1F3-1F1EA"},
    {"flagg_norfolkoya", "1F1F3-1F1EB"},
    {"flagg_nigeria", "1F1F3-1F1EC"},
    {"flagg_nicaragua", "1F1F3-1F1EE"},
    {"flagg_nederland", "1F1F3-1F1F1"},
    {"flagg_norge", "1F1F3-1F1F4"},
    {"flagg_nepal", "1F1F3-1F1F5"},
    {"flagg_nauru", "1F1F3-1F1F7"},
    {"flagg_niue", "1F1F3-1F1FA"},
    {"flagg_new_zealand", "1F1F3-1F1FF"},
    {"flagg_oman", "1F1F4-1F1F2"},
    {"flagg_panama", "1F1F5-1F1E6"},
    {"flagg_peru", "1F1F5-1F1EA"},
    {"flagg_fransk_polynesia", "1F1F5-1F1EB"},
    {"flagg_papua_ny_guinea", "1F1F5-1F1EC"},
    {"flagg_filippinene", "1F1F5-1F1ED"},
    {"flagg_pakistan", "1F1F5-1F1F0"},
    {"flagg_polen", "1F1F5-1F1F1"},
    {"flagg_saint_pierre_et_miquelon", "1F1F5-1F1F2"},
    {"flagg_pitcairnoyene", "1F1F5-1F1F3"},
    {"flagg_puerto_rico", "1F1F5-1F1F7"},
    {"flagg_det_palestinske_omradet", "1F1F5-1F1F8"},
    {"flagg_portugal", "1F1F5-1F1F9"},
    {"flagg_palau", "1F1F5-1F1FC"},
    {"flagg_paraguay", "1F1F5-1F1FE"},
    {"flagg_qatar", "1F1F6-1F1E6"},
    {"flagg_reunion", "1F1F7-1F1EA"},
    {"flagg_romania", "1F1F7-1F1F4"},
    {"flagg_serbia", "1F1F7-1F1F8"},
    {"flagg_russland", "1F1F7-1F1FA"},
    {"flagg_rwanda", "1F1F7-1F1FC"},
    {"flagg_saudi_arabia", "1F1F8-1F1E6"},
    {"flagg_salomonoyene", "1F1F8-1F1E7"},
    {"flagg_seychellene", "1F1F8-1F1E8"},
    {"flagg_sudan", "1F1F8-1F1E9"},
    {"flagg_sverige", "1F1F8-1F1EA"},
    {"flagg_singapore", "1F1F8-1F1EC"},
    {"flagg_st_helena", "1F1F8-1F1ED"},
    {"flagg_slovenia", "1F1F8-1F1EE"},
    {"flagg_svalbard_og_jan_mayen", "1F1F8-1F1EF"},
    {"flagg_slovakia", "1F1F8-1F1F0"},
    {"flagg_sierra_leone", "1F1F8-1F1F1"},
    {"flagg_san_marino", "1F1F8-1F1F2"},
    {"flagg_senegal", "1F1F8-1F1F3"},
    {"flagg_somalia", "1F1F8-1F1F4"},
    {"flagg_surinam", "1F1F8-1F1F7"},
    {"flagg_sor_sudan", "1F1F8-1F1F8"},
    {"flagg_sao_tome_og_principe", "1F1F8-1F1F9"},
    {"flagg_el_salvador", "1F1F8-1F1FB"},
    {"flagg_sint_maarten", "1F1F8-1F1FD"},
    {"flagg_syria", "1F1F8-1F1FE"},
    {"flagg_eswatini", "1F1F8-1F1FF"},
    {"flagg_tristan_da_cunha", "1F1F9-1F1E6"},
    {"flagg_turks_og_caicosoyene", "1F1F9-1F1E8"},
    {"flagg_tsjad", "1F1F9-1F1E9"},
    {"flagg_de_franske_sorterritorier", "1F1F9-1F1EB"},
    {"flagg_togo", "1F1F9-1F1EC"},
    {"flagg_thailand", "1F1F9-1F1ED"},
    {"flagg_tadsjikistan", "1F1F9-1F1EF"},
    {"flagg_tokelau", "1F1F9-1F1F0"},
    {"flagg_ost_timor", "1F1F9-1F1F1"},
    {"flagg_turkmenistan", "1F1F9-1F1F2"},
    {"flagg_tunisia", "1F1F9-1F1F3"},
    {"flagg_tonga", "1F1F9-1F1F4"},
    {"flagg_tyrkia", "1F1F9-1F1F7"},
    {"flagg_trinidad_og_tobago", "1F1F9-1F1F9"},
    {"flagg_tuvalu", "1F1F9-1F1FB"},
    {"flagg_taiwan", "1F1F9-1F1FC"},
    {"flagg_tanzania", "1F1F9-1F1FF"},
    {"flagg_ukraina", "1F1FA-1F1E6"},
    {"flagg_uganda", "1F1FA-1F1EC"},
    {"flagg_usas_ytre_oyer", "1F1FA-1F1F2"},
    {"flagg_fn", "1F1FA-1F1F3"},
    {"flagg_usa", "1F1FA-1F1F8"},
    {"flagg_uruguay", "1F1FA-1F1FE"},
    {"flagg_usbekistan", "1F1FA-1F1FF"},
    {"flagg_vatikanstaten", "1F1FB-1F1E6"},
    {"flagg_st_vincent_og_grenadinene", "1F1FB-1F1E8"},
    {"flagg_venezuela", "1F1FB-1F1EA"},
    {"flagg_de_britiske_jomfruoyene", "1F1FB-1F1EC"},
    {"flagg_de_amerikanske_jomfruoyene", "1F1FB-1F1EE"},
    {"flagg_vietnam", "1F1FB-1F1F3"},
    {"flagg_vanuatu", "1F1FB-1F1FA"},
    {"flagg_wallis_og_futuna", "1F1FC-1F1EB"},
    {"flagg_samoa", "1F1FC-1F1F8"},
    {"flagg_kosovo", "1F1FD-1F1F0"},
    {"flagg_jemen", "1F1FE-1F1EA"},
    {"flagg_mayotte", "1F1FE-1F1F9"},
    {"flagg_sor_afrika", "1F1FF-1F1E6"},
    {"flagg_zambia", "1F1FF-1F1F2"},
    {"flagg_zimbabwe", "1F1FF-1F1FC"},
    {"flagg_england", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"flagg_skottland", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"flagg_wales", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"par_med_hjerte_mann_mann", "1F468-200D-2764-FE0F-200D-1F468"},
    {"par_med_hjerte_mann_mann_farge1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_mann_mann_farge1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_mann_mann_farge1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_mann_mann_farge1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_mann_mann_farge1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_mann_mann_farge2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_mann_mann_farge2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_mann_mann_farge2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_mann_mann_farge2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_mann_mann_farge2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_mann_mann_farge3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_mann_mann_farge3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_mann_mann_farge3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_mann_mann_farge3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_mann_mann_farge3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_mann_mann_farge4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_mann_mann_farge4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_mann_mann_farge4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_mann_mann_farge4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_mann_mann_farge4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_mann_mann_farge5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_mann_mann_farge5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_mann_mann_farge5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_mann_mann_farge5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_mann_mann_farge5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"kyss_mann_mann", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"kyss_mann_mann_farge1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kyss_mann_mann_farge1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kyss_mann_mann_farge1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kyss_mann_mann_farge1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kyss_mann_mann_farge1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kyss_mann_mann_farge2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kyss_mann_mann_farge2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kyss_mann_mann_farge2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kyss_mann_mann_farge2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kyss_mann_mann_farge2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kyss_mann_mann_farge3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kyss_mann_mann_farge3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kyss_mann_mann_farge3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kyss_mann_mann_farge3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kyss_mann_mann_farge3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kyss_mann_mann_farge4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kyss_mann_mann_farge4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kyss_mann_mann_farge4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kyss_mann_mann_farge4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kyss_mann_mann_farge4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kyss_mann_mann_farge5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kyss_mann_mann_farge5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kyss_mann_mann_farge5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kyss_mann_mann_farge5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kyss_mann_mann_farge5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"familie_mann_gutt", "1F468-200D-1F466"},
    {"familie_mann_gutt_gutt", "1F468-200D-1F466-200D-1F466"},
    {"familie_mann_jente", "1F468-200D-1F467"},
    {"familie_mann_jente_gutt", "1F468-200D-1F467-200D-1F466"},
    {"familie_mann_jente_jente", "1F468-200D-1F467-200D-1F467"},
    {"familie_mann_mann_gutt", "1F468-200D-1F468-200D-1F466"},
    {"familie_mann_mann_gutt_gutt", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"familie_mann_mann_jente", "1F468-200D-1F468-200D-1F467"},
    {"familie_mann_mann_jente_gutt", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"familie_mann_mann_jente_jente", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"familie_mann_kvinne_gutt", "1F468-200D-1F469-200D-1F466"},
    {"familie_mann_kvinne_gutt_gutt", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"familie_mann_kvinne_jente", "1F468-200D-1F469-200D-1F467"},
    {"familie_mann_kvinne_jente_gutt", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"familie_mann_kvinne_jente_jente", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"par_med_hjerte_kvinne_mann", "1F469-200D-2764-FE0F-200D-1F468"},
    {"par_med_hjerte_kvinne_mann_farge1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_kvinne_mann_farge1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_kvinne_mann_farge1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_kvinne_mann_farge1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_kvinne_mann_farge1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_kvinne_mann_farge2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_kvinne_mann_farge2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_kvinne_mann_farge2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_kvinne_mann_farge2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_kvinne_mann_farge2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_kvinne_mann_farge3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_kvinne_mann_farge3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_kvinne_mann_farge3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_kvinne_mann_farge3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_kvinne_mann_farge3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_kvinne_mann_farge4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_kvinne_mann_farge4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_kvinne_mann_farge4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_kvinne_mann_farge4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_kvinne_mann_farge4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_kvinne_mann_farge5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_kvinne_mann_farge5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_kvinne_mann_farge5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_kvinne_mann_farge5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_kvinne_mann_farge5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_kvinne_kvinne", "1F469-200D-2764-FE0F-200D-1F469"},
    {"par_med_hjerte_kvinne_kvinne_farge1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjerte_kvinne_kvinne_farge1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjerte_kvinne_kvinne_farge1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjerte_kvinne_kvinne_farge1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjerte_kvinne_kvinne_farge1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjerte_kvinne_kvinne_farge2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjerte_kvinne_kvinne_farge2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjerte_kvinne_kvinne_farge2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjerte_kvinne_kvinne_farge2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjerte_kvinne_kvinne_farge2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjerte_kvinne_kvinne_farge3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjerte_kvinne_kvinne_farge3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjerte_kvinne_kvinne_farge3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjerte_kvinne_kvinne_farge3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjerte_kvinne_kvinne_farge3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjerte_kvinne_kvinne_farge4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjerte_kvinne_kvinne_farge4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjerte_kvinne_kvinne_farge4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjerte_kvinne_kvinne_farge4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjerte_kvinne_kvinne_farge4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjerte_kvinne_kvinne_farge5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjerte_kvinne_kvinne_farge5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjerte_kvinne_kvinne_farge5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjerte_kvinne_kvinne_farge5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjerte_kvinne_kvinne_farge5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"kyss_kvinne_mann", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"kyss_kvinne_mann_farge1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kyss_kvinne_mann_farge1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kyss_kvinne_mann_farge1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kyss_kvinne_mann_farge1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kyss_kvinne_mann_farge1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kyss_kvinne_mann_farge2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kyss_kvinne_mann_farge2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kyss_kvinne_mann_farge2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kyss_kvinne_mann_farge2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kyss_kvinne_mann_farge2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kyss_kvinne_mann_farge3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kyss_kvinne_mann_farge3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kyss_kvinne_mann_farge3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kyss_kvinne_mann_farge3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kyss_kvinne_mann_farge3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kyss_kvinne_mann_farge4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kyss_kvinne_mann_farge4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kyss_kvinne_mann_farge4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kyss_kvinne_mann_farge4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kyss_kvinne_mann_farge4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kyss_kvinne_mann_farge5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kyss_kvinne_mann_farge5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kyss_kvinne_mann_farge5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kyss_kvinne_mann_farge5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kyss_kvinne_mann_farge5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kyss_kvinne_kvinne", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"kyss_kvinne_kvinne_farge1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kyss_kvinne_kvinne_farge1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kyss_kvinne_kvinne_farge1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kyss_kvinne_kvinne_farge1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kyss_kvinne_kvinne_farge1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"kyss_kvinne_kvinne_farge2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kyss_kvinne_kvinne_farge2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kyss_kvinne_kvinne_farge2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kyss_kvinne_kvinne_farge2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kyss_kvinne_kvinne_farge2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"kyss_kvinne_kvinne_farge3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kyss_kvinne_kvinne_farge3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kyss_kvinne_kvinne_farge3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kyss_kvinne_kvinne_farge3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kyss_kvinne_kvinne_farge3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"kyss_kvinne_kvinne_farge4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kyss_kvinne_kvinne_farge4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kyss_kvinne_kvinne_farge4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kyss_kvinne_kvinne_farge4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kyss_kvinne_kvinne_farge4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"kyss_kvinne_kvinne_farge5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kyss_kvinne_kvinne_farge5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kyss_kvinne_kvinne_farge5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kyss_kvinne_kvinne_farge5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kyss_kvinne_kvinne_farge5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"familie_kvinne_gutt", "1F469-200D-1F466"},
    {"familie_kvinne_gutt_gutt", "1F469-200D-1F466-200D-1F466"},
    {"familie_kvinne_jente", "1F469-200D-1F467"},
    {"familie_kvinne_jente_gutt", "1F469-200D-1F467-200D-1F466"},
    {"familie_kvinne_jente_jente", "1F469-200D-1F467-200D-1F467"},
    {"familie_kvinne_kvinne_gutt", "1F469-200D-1F469-200D-1F466"},
    {"familie_kvinne_kvinne_gutt_gutt", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"familie_kvinne_kvinne_jente", "1F469-200D-1F469-200D-1F467"},
    {"familie_kvinne_kvinne_jente_gutt", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"familie_kvinne_kvinne_jente_jente", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"personer_som_leier", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"personer_som_leier_farge1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_leier_farge1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_leier_farge1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_leier_farge1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_leier_farge1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_som_leier_farge2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_leier_farge2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_leier_farge2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_leier_farge2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_leier_farge2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_som_leier_farge3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_leier_farge3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_leier_farge3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_leier_farge3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_leier_farge3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_som_leier_farge4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_leier_farge4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_leier_farge4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_leier_farge4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_leier_farge4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_som_leier_farge5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_leier_farge5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_leier_farge5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_leier_farge5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_leier_farge5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"familie_voksen_voksen_barn", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"familie_voksen_voksen_barn_barn", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"familie_voksen_barn", "1F9D1-200D-1F9D2"},
    {"familie_voksen_barn_barn", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"loper_vendt_mot_hoyre", "1F3C3-200D-27A1-FE0F"},
    {"loper_vendt_mot_hoyre_farge1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"loper_vendt_mot_hoyre_farge2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"loper_vendt_mot_hoyre_farge3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"loper_vendt_mot_hoyre_farge4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"loper_vendt_mot_hoyre_farge5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"mannlig_helsearbeider", "1F468-200D-2695-FE0F"},
    {"mannlig_helsearbeider_farge1", "1F468-1F3FB-200D-2695-FE0F"},
    {"mannlig_helsearbeider_farge2", "1F468-1F3FC-200D-2695-FE0F"},
    {"mannlig_helsearbeider_farge3", "1F468-1F3FD-200D-2695-FE0F"},
    {"mannlig_helsearbeider_farge4", "1F468-1F3FE-200D-2695-FE0F"},
    {"mannlig_helsearbeider_farge5", "1F468-1F3FF-200D-2695-FE0F"},
    {"mannlig_dommer", "1F468-200D-2696-FE0F"},
    {"mannlig_dommer_farge1", "1F468-1F3FB-200D-2696-FE0F"},
    {"mannlig_dommer_farge2", "1F468-1F3FC-200D-2696-FE0F"},
    {"mannlig_dommer_farge3", "1F468-1F3FD-200D-2696-FE0F"},
    {"mannlig_dommer_farge4", "1F468-1F3FE-200D-2696-FE0F"},
    {"mannlig_dommer_farge5", "1F468-1F3FF-200D-2696-FE0F"},
    {"mannlig_pilot", "1F468-200D-2708-FE0F"},
    {"mannlig_pilot_farge1", "1F468-1F3FB-200D-2708-FE0F"},
    {"mannlig_pilot_farge2", "1F468-1F3FC-200D-2708-FE0F"},
    {"mannlig_pilot_farge3", "1F468-1F3FD-200D-2708-FE0F"},
    {"mannlig_pilot_farge4", "1F468-1F3FE-200D-2708-FE0F"},
    {"mannlig_pilot_farge5", "1F468-1F3FF-200D-2708-FE0F"},
    {"mannlig_gardbruker", "1F468-200D-1F33E"},
    {"mannlig_gardbruker_farge1", "1F468-1F3FB-200D-1F33E"},
    {"mannlig_gardbruker_farge2", "1F468-1F3FC-200D-1F33E"},
    {"mannlig_gardbruker_farge3", "1F468-1F3FD-200D-1F33E"},
    {"mannlig_gardbruker_farge4", "1F468-1F3FE-200D-1F33E"},
    {"mannlig_gardbruker_farge5", "1F468-1F3FF-200D-1F33E"},
    {"mannlig_kokk", "1F468-200D-1F373"},
    {"mannlig_kokk_farge1", "1F468-1F3FB-200D-1F373"},
    {"mannlig_kokk_farge2", "1F468-1F3FC-200D-1F373"},
    {"mannlig_kokk_farge3", "1F468-1F3FD-200D-1F373"},
    {"mannlig_kokk_farge4", "1F468-1F3FE-200D-1F373"},
    {"mannlig_kokk_farge5", "1F468-1F3FF-200D-1F373"},
    {"mann_som_mater_baby", "1F468-200D-1F37C"},
    {"mann_som_mater_baby_farge1", "1F468-1F3FB-200D-1F37C"},
    {"mann_som_mater_baby_farge2", "1F468-1F3FC-200D-1F37C"},
    {"mann_som_mater_baby_farge3", "1F468-1F3FD-200D-1F37C"},
    {"mann_som_mater_baby_farge4", "1F468-1F3FE-200D-1F37C"},
    {"mann_som_mater_baby_farge5", "1F468-1F3FF-200D-1F37C"},
    {"mannlig_student", "1F468-200D-1F393"},
    {"mannlig_student_farge1", "1F468-1F3FB-200D-1F393"},
    {"mannlig_student_farge2", "1F468-1F3FC-200D-1F393"},
    {"mannlig_student_farge3", "1F468-1F3FD-200D-1F393"},
    {"mannlig_student_farge4", "1F468-1F3FE-200D-1F393"},
    {"mannlig_student_farge5", "1F468-1F3FF-200D-1F393"},
    {"mannlig_artist", "1F468-200D-1F3A4"},
    {"mannlig_artist_farge1", "1F468-1F3FB-200D-1F3A4"},
    {"mannlig_artist_farge2", "1F468-1F3FC-200D-1F3A4"},
    {"mannlig_artist_farge3", "1F468-1F3FD-200D-1F3A4"},
    {"mannlig_artist_farge4", "1F468-1F3FE-200D-1F3A4"},
    {"mannlig_artist_farge5", "1F468-1F3FF-200D-1F3A4"},
    {"mannlig_kunstner", "1F468-200D-1F3A8"},
    {"mannlig_kunstner_farge1", "1F468-1F3FB-200D-1F3A8"},
    {"mannlig_kunstner_farge2", "1F468-1F3FC-200D-1F3A8"},
    {"mannlig_kunstner_farge3", "1F468-1F3FD-200D-1F3A8"},
    {"mannlig_kunstner_farge4", "1F468-1F3FE-200D-1F3A8"},
    {"mannlig_kunstner_farge5", "1F468-1F3FF-200D-1F3A8"},
    {"mannlig_laerer", "1F468-200D-1F3EB"},
    {"mannlig_laerer_farge1", "1F468-1F3FB-200D-1F3EB"},
    {"mannlig_laerer_farge2", "1F468-1F3FC-200D-1F3EB"},
    {"mannlig_laerer_farge3", "1F468-1F3FD-200D-1F3EB"},
    {"mannlig_laerer_farge4", "1F468-1F3FE-200D-1F3EB"},
    {"mannlig_laerer_farge5", "1F468-1F3FF-200D-1F3EB"},
    {"mannlig_industriarbeider", "1F468-200D-1F3ED"},
    {"mannlig_industriarbeider_farge1", "1F468-1F3FB-200D-1F3ED"},
    {"mannlig_industriarbeider_farge2", "1F468-1F3FC-200D-1F3ED"},
    {"mannlig_industriarbeider_farge3", "1F468-1F3FD-200D-1F3ED"},
    {"mannlig_industriarbeider_farge4", "1F468-1F3FE-200D-1F3ED"},
    {"mannlig_industriarbeider_farge5", "1F468-1F3FF-200D-1F3ED"},
    {"mannlig_it_ekspert", "1F468-200D-1F4BB"},
    {"mannlig_it_ekspert_farge1", "1F468-1F3FB-200D-1F4BB"},
    {"mannlig_it_ekspert_farge2", "1F468-1F3FC-200D-1F4BB"},
    {"mannlig_it_ekspert_farge3", "1F468-1F3FD-200D-1F4BB"},
    {"mannlig_it_ekspert_farge4", "1F468-1F3FE-200D-1F4BB"},
    {"mannlig_it_ekspert_farge5", "1F468-1F3FF-200D-1F4BB"},
    {"mannlig_kontorarbeider", "1F468-200D-1F4BC"},
    {"mannlig_kontorarbeider_farge1", "1F468-1F3FB-200D-1F4BC"},
    {"mannlig_kontorarbeider_farge2", "1F468-1F3FC-200D-1F4BC"},
    {"mannlig_kontorarbeider_farge3", "1F468-1F3FD-200D-1F4BC"},
    {"mannlig_kontorarbeider_farge4", "1F468-1F3FE-200D-1F4BC"},
    {"mannlig_kontorarbeider_farge5", "1F468-1F3FF-200D-1F4BC"},
    {"mannlig_mekaniker", "1F468-200D-1F527"},
    {"mannlig_mekaniker_farge1", "1F468-1F3FB-200D-1F527"},
    {"mannlig_mekaniker_farge2", "1F468-1F3FC-200D-1F527"},
    {"mannlig_mekaniker_farge3", "1F468-1F3FD-200D-1F527"},
    {"mannlig_mekaniker_farge4", "1F468-1F3FE-200D-1F527"},
    {"mannlig_mekaniker_farge5", "1F468-1F3FF-200D-1F527"},
    {"mannlig_forsker", "1F468-200D-1F52C"},
    {"mannlig_forsker_farge1", "1F468-1F3FB-200D-1F52C"},
    {"mannlig_forsker_farge2", "1F468-1F3FC-200D-1F52C"},
    {"mannlig_forsker_farge3", "1F468-1F3FD-200D-1F52C"},
    {"mannlig_forsker_farge4", "1F468-1F3FE-200D-1F52C"},
    {"mannlig_forsker_farge5", "1F468-1F3FF-200D-1F52C"},
    {"mannlig_astronaut", "1F468-200D-1F680"},
    {"mannlig_astronaut_farge1", "1F468-1F3FB-200D-1F680"},
    {"mannlig_astronaut_farge2", "1F468-1F3FC-200D-1F680"},
    {"mannlig_astronaut_farge3", "1F468-1F3FD-200D-1F680"},
    {"mannlig_astronaut_farge4", "1F468-1F3FE-200D-1F680"},
    {"mannlig_astronaut_farge5", "1F468-1F3FF-200D-1F680"},
    {"mannlig_brannkonstabel", "1F468-200D-1F692"},
    {"mannlig_brannkonstabel_farge1", "1F468-1F3FB-200D-1F692"},
    {"mannlig_brannkonstabel_farge2", "1F468-1F3FC-200D-1F692"},
    {"mannlig_brannkonstabel_farge3", "1F468-1F3FD-200D-1F692"},
    {"mannlig_brannkonstabel_farge4", "1F468-1F3FE-200D-1F692"},
    {"mannlig_brannkonstabel_farge5", "1F468-1F3FF-200D-1F692"},
    {"mann_med_blindestokk", "1F468-200D-1F9AF"},
    {"mann_med_blindestokk_farge1", "1F468-1F3FB-200D-1F9AF"},
    {"mann_med_blindestokk_farge2", "1F468-1F3FC-200D-1F9AF"},
    {"mann_med_blindestokk_farge3", "1F468-1F3FD-200D-1F9AF"},
    {"mann_med_blindestokk_farge4", "1F468-1F3FE-200D-1F9AF"},
    {"mann_med_blindestokk_farge5", "1F468-1F3FF-200D-1F9AF"},
    {"mann_med_blindestokk_vendt_mot_hoyre", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_med_blindestokk_vendt_mot_hoyre_farge1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_med_blindestokk_vendt_mot_hoyre_farge2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_med_blindestokk_vendt_mot_hoyre_farge3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_med_blindestokk_vendt_mot_hoyre_farge4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_med_blindestokk_vendt_mot_hoyre_farge5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_i_motorisert_rullestol", "1F468-200D-1F9BC"},
    {"mann_i_motorisert_rullestol_farge1", "1F468-1F3FB-200D-1F9BC"},
    {"mann_i_motorisert_rullestol_farge2", "1F468-1F3FC-200D-1F9BC"},
    {"mann_i_motorisert_rullestol_farge3", "1F468-1F3FD-200D-1F9BC"},
    {"mann_i_motorisert_rullestol_farge4", "1F468-1F3FE-200D-1F9BC"},
    {"mann_i_motorisert_rullestol_farge5", "1F468-1F3FF-200D-1F9BC"},
    {"mann_i_motorisert_rullestol_vendt_mot_hoyre", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_i_motorisert_rullestol_vendt_mot_hoyre_farge1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_i_motorisert_rullestol_vendt_mot_hoyre_farge2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_i_motorisert_rullestol_vendt_mot_hoyre_farge3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_i_motorisert_rullestol_vendt_mot_hoyre_farge4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_i_motorisert_rullestol_vendt_mot_hoyre_farge5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_i_manuell_rullestol", "1F468-200D-1F9BD"},
    {"mann_i_manuell_rullestol_farge1", "1F468-1F3FB-200D-1F9BD"},
    {"mann_i_manuell_rullestol_farge2", "1F468-1F3FC-200D-1F9BD"},
    {"mann_i_manuell_rullestol_farge3", "1F468-1F3FD-200D-1F9BD"},
    {"mann_i_manuell_rullestol_farge4", "1F468-1F3FE-200D-1F9BD"},
    {"mann_i_manuell_rullestol_farge5", "1F468-1F3FF-200D-1F9BD"},
    {"mann_i_manuell_rullestol_vendt_mot_hoyre", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_i_manuell_rullestol_vendt_mot_hoyre_farge1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_i_manuell_rullestol_vendt_mot_hoyre_farge2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_i_manuell_rullestol_vendt_mot_hoyre_farge3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_i_manuell_rullestol_vendt_mot_hoyre_farge4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_i_manuell_rullestol_vendt_mot_hoyre_farge5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinnelig_helsearbeider", "1F469-200D-2695-FE0F"},
    {"kvinnelig_helsearbeider_farge1", "1F469-1F3FB-200D-2695-FE0F"},
    {"kvinnelig_helsearbeider_farge2", "1F469-1F3FC-200D-2695-FE0F"},
    {"kvinnelig_helsearbeider_farge3", "1F469-1F3FD-200D-2695-FE0F"},
    {"kvinnelig_helsearbeider_farge4", "1F469-1F3FE-200D-2695-FE0F"},
    {"kvinnelig_helsearbeider_farge5", "1F469-1F3FF-200D-2695-FE0F"},
    {"kvinnelig_dommer", "1F469-200D-2696-FE0F"},
    {"kvinnelig_dommer_farge1", "1F469-1F3FB-200D-2696-FE0F"},
    {"kvinnelig_dommer_farge2", "1F469-1F3FC-200D-2696-FE0F"},
    {"kvinnelig_dommer_farge3", "1F469-1F3FD-200D-2696-FE0F"},
    {"kvinnelig_dommer_farge4", "1F469-1F3FE-200D-2696-FE0F"},
    {"kvinnelig_dommer_farge5", "1F469-1F3FF-200D-2696-FE0F"},
    {"kvinnelig_pilot", "1F469-200D-2708-FE0F"},
    {"kvinnelig_pilot_farge1", "1F469-1F3FB-200D-2708-FE0F"},
    {"kvinnelig_pilot_farge2", "1F469-1F3FC-200D-2708-FE0F"},
    {"kvinnelig_pilot_farge3", "1F469-1F3FD-200D-2708-FE0F"},
    {"kvinnelig_pilot_farge4", "1F469-1F3FE-200D-2708-FE0F"},
    {"kvinnelig_pilot_farge5", "1F469-1F3FF-200D-2708-FE0F"},
    {"kvinnelig_gardbruker", "1F469-200D-1F33E"},
    {"kvinnelig_gardbruker_farge1", "1F469-1F3FB-200D-1F33E"},
    {"kvinnelig_gardbruker_farge2", "1F469-1F3FC-200D-1F33E"},
    {"kvinnelig_gardbruker_farge3", "1F469-1F3FD-200D-1F33E"},
    {"kvinnelig_gardbruker_farge4", "1F469-1F3FE-200D-1F33E"},
    {"kvinnelig_gardbruker_farge5", "1F469-1F3FF-200D-1F33E"},
    {"kvinnelig_kokk", "1F469-200D-1F373"},
    {"kvinnelig_kokk_farge1", "1F469-1F3FB-200D-1F373"},
    {"kvinnelig_kokk_farge2", "1F469-1F3FC-200D-1F373"},
    {"kvinnelig_kokk_farge3", "1F469-1F3FD-200D-1F373"},
    {"kvinnelig_kokk_farge4", "1F469-1F3FE-200D-1F373"},
    {"kvinnelig_kokk_farge5", "1F469-1F3FF-200D-1F373"},
    {"kvinne_som_mater_baby", "1F469-200D-1F37C"},
    {"kvinne_som_mater_baby_farge1", "1F469-1F3FB-200D-1F37C"},
    {"kvinne_som_mater_baby_farge2", "1F469-1F3FC-200D-1F37C"},
    {"kvinne_som_mater_baby_farge3", "1F469-1F3FD-200D-1F37C"},
    {"kvinne_som_mater_baby_farge4", "1F469-1F3FE-200D-1F37C"},
    {"kvinne_som_mater_baby_farge5", "1F469-1F3FF-200D-1F37C"},
    {"kvinnelig_student", "1F469-200D-1F393"},
    {"kvinnelig_student_farge1", "1F469-1F3FB-200D-1F393"},
    {"kvinnelig_student_farge2", "1F469-1F3FC-200D-1F393"},
    {"kvinnelig_student_farge3", "1F469-1F3FD-200D-1F393"},
    {"kvinnelig_student_farge4", "1F469-1F3FE-200D-1F393"},
    {"kvinnelig_student_farge5", "1F469-1F3FF-200D-1F393"},
    {"kvinnelig_artist", "1F469-200D-1F3A4"},
    {"kvinnelig_artist_farge1", "1F469-1F3FB-200D-1F3A4"},
    {"kvinnelig_artist_farge2", "1F469-1F3FC-200D-1F3A4"},
    {"kvinnelig_artist_farge3", "1F469-1F3FD-200D-1F3A4"},
    {"kvinnelig_artist_farge4", "1F469-1F3FE-200D-1F3A4"},
    {"kvinnelig_artist_farge5", "1F469-1F3FF-200D-1F3A4"},
    {"kvinnelig_kunstner", "1F469-200D-1F3A8"},
    {"kvinnelig_kunstner_farge1", "1F469-1F3FB-200D-1F3A8"},
    {"kvinnelig_kunstner_farge2", "1F469-1F3FC-200D-1F3A8"},
    {"kvinnelig_kunstner_farge3", "1F469-1F3FD-200D-1F3A8"},
    {"kvinnelig_kunstner_farge4", "1F469-1F3FE-200D-1F3A8"},
    {"kvinnelig_kunstner_farge5", "1F469-1F3FF-200D-1F3A8"},
    {"kvinnelig_laerer", "1F469-200D-1F3EB"},
    {"kvinnelig_laerer_farge1", "1F469-1F3FB-200D-1F3EB"},
    {"kvinnelig_laerer_farge2", "1F469-1F3FC-200D-1F3EB"},
    {"kvinnelig_laerer_farge3", "1F469-1F3FD-200D-1F3EB"},
    {"kvinnelig_laerer_farge4", "1F469-1F3FE-200D-1F3EB"},
    {"kvinnelig_laerer_farge5", "1F469-1F3FF-200D-1F3EB"},
    {"kvinnelig_industriarbeider", "1F469-200D-1F3ED"},
    {"kvinnelig_industriarbeider_farge1", "1F469-1F3FB-200D-1F3ED"},
    {"kvinnelig_industriarbeider_farge2", "1F469-1F3FC-200D-1F3ED"},
    {"kvinnelig_industriarbeider_farge3", "1F469-1F3FD-200D-1F3ED"},
    {"kvinnelig_industriarbeider_farge4", "1F469-1F3FE-200D-1F3ED"},
    {"kvinnelig_industriarbeider_farge5", "1F469-1F3FF-200D-1F3ED"},
    {"kvinnelig_it_ekspert", "1F469-200D-1F4BB"},
    {"kvinnelig_it_ekspert_farge1", "1F469-1F3FB-200D-1F4BB"},
    {"kvinnelig_it_ekspert_farge2", "1F469-1F3FC-200D-1F4BB"},
    {"kvinnelig_it_ekspert_farge3", "1F469-1F3FD-200D-1F4BB"},
    {"kvinnelig_it_ekspert_farge4", "1F469-1F3FE-200D-1F4BB"},
    {"kvinnelig_it_ekspert_farge5", "1F469-1F3FF-200D-1F4BB"},
    {"kvinnelig_kontorarbeider", "1F469-200D-1F4BC"},
    {"kvinnelig_kontorarbeider_farge1", "1F469-1F3FB-200D-1F4BC"},
    {"kvinnelig_kontorarbeider_farge2", "1F469-1F3FC-200D-1F4BC"},
    {"kvinnelig_kontorarbeider_farge3", "1F469-1F3FD-200D-1F4BC"},
    {"kvinnelig_kontorarbeider_farge4", "1F469-1F3FE-200D-1F4BC"},
    {"kvinnelig_kontorarbeider_farge5", "1F469-1F3FF-200D-1F4BC"},
    {"kvinnelig_mekaniker", "1F469-200D-1F527"},
    {"kvinnelig_mekaniker_farge1", "1F469-1F3FB-200D-1F527"},
    {"kvinnelig_mekaniker_farge2", "1F469-1F3FC-200D-1F527"},
    {"kvinnelig_mekaniker_farge3", "1F469-1F3FD-200D-1F527"},
    {"kvinnelig_mekaniker_farge4", "1F469-1F3FE-200D-1F527"},
    {"kvinnelig_mekaniker_farge5", "1F469-1F3FF-200D-1F527"},
    {"kvinnelig_forsker", "1F469-200D-1F52C"},
    {"kvinnelig_forsker_farge1", "1F469-1F3FB-200D-1F52C"},
    {"kvinnelig_forsker_farge2", "1F469-1F3FC-200D-1F52C"},
    {"kvinnelig_forsker_farge3", "1F469-1F3FD-200D-1F52C"},
    {"kvinnelig_forsker_farge4", "1F469-1F3FE-200D-1F52C"},
    {"kvinnelig_forsker_farge5", "1F469-1F3FF-200D-1F52C"},
    {"kvinnelig_astronaut", "1F469-200D-1F680"},
    {"kvinnelig_astronaut_farge1", "1F469-1F3FB-200D-1F680"},
    {"kvinnelig_astronaut_farge2", "1F469-1F3FC-200D-1F680"},
    {"kvinnelig_astronaut_farge3", "1F469-1F3FD-200D-1F680"},
    {"kvinnelig_astronaut_farge4", "1F469-1F3FE-200D-1F680"},
    {"kvinnelig_astronaut_farge5", "1F469-1F3FF-200D-1F680"},
    {"kvinnelig_brannkonstabel", "1F469-200D-1F692"},
    {"kvinnelig_brannkonstabel_farge1", "1F469-1F3FB-200D-1F692"},
    {"kvinnelig_brannkonstabel_farge2", "1F469-1F3FC-200D-1F692"},
    {"kvinnelig_brannkonstabel_farge3", "1F469-1F3FD-200D-1F692"},
    {"kvinnelig_brannkonstabel_farge4", "1F469-1F3FE-200D-1F692"},
    {"kvinnelig_brannkonstabel_farge5", "1F469-1F3FF-200D-1F692"},
    {"kvinne_med_blindestokk", "1F469-200D-1F9AF"},
    {"kvinne_med_blindestokk_farge1", "1F469-1F3FB-200D-1F9AF"},
    {"kvinne_med_blindestokk_farge2", "1F469-1F3FC-200D-1F9AF"},
    {"kvinne_med_blindestokk_farge3", "1F469-1F3FD-200D-1F9AF"},
    {"kvinne_med_blindestokk_farge4", "1F469-1F3FE-200D-1F9AF"},
    {"kvinne_med_blindestokk_farge5", "1F469-1F3FF-200D-1F9AF"},
    {"kvinne_med_blindestokk_vendt_mot_hoyre", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinne_med_blindestokk_vendt_mot_hoyre_farge1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinne_med_blindestokk_vendt_mot_hoyre_farge2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinne_med_blindestokk_vendt_mot_hoyre_farge3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinne_med_blindestokk_vendt_mot_hoyre_farge4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinne_med_blindestokk_vendt_mot_hoyre_farge5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinne_i_motorisert_rullestol", "1F469-200D-1F9BC"},
    {"kvinne_i_motorisert_rullestol_farge1", "1F469-1F3FB-200D-1F9BC"},
    {"kvinne_i_motorisert_rullestol_farge2", "1F469-1F3FC-200D-1F9BC"},
    {"kvinne_i_motorisert_rullestol_farge3", "1F469-1F3FD-200D-1F9BC"},
    {"kvinne_i_motorisert_rullestol_farge4", "1F469-1F3FE-200D-1F9BC"},
    {"kvinne_i_motorisert_rullestol_farge5", "1F469-1F3FF-200D-1F9BC"},
    {"kvinne_i_motorisert_rullestol_vendt_mot_hoyre", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinne_i_motorisert_rullestol_vendt_mot_hoyre_farge1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinne_i_motorisert_rullestol_vendt_mot_hoyre_farge2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinne_i_motorisert_rullestol_vendt_mot_hoyre_farge3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinne_i_motorisert_rullestol_vendt_mot_hoyre_farge4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinne_i_motorisert_rullestol_vendt_mot_hoyre_farge5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinne_i_manuell_rullestol", "1F469-200D-1F9BD"},
    {"kvinne_i_manuell_rullestol_farge1", "1F469-1F3FB-200D-1F9BD"},
    {"kvinne_i_manuell_rullestol_farge2", "1F469-1F3FC-200D-1F9BD"},
    {"kvinne_i_manuell_rullestol_farge3", "1F469-1F3FD-200D-1F9BD"},
    {"kvinne_i_manuell_rullestol_farge4", "1F469-1F3FE-200D-1F9BD"},
    {"kvinne_i_manuell_rullestol_farge5", "1F469-1F3FF-200D-1F9BD"},
    {"kvinne_i_manuell_rullestol_vendt_mot_hoyre", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinne_i_manuell_rullestol_vendt_mot_hoyre_farge1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinne_i_manuell_rullestol_vendt_mot_hoyre_farge2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinne_i_manuell_rullestol_vendt_mot_hoyre_farge3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinne_i_manuell_rullestol_vendt_mot_hoyre_farge4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinne_i_manuell_rullestol_vendt_mot_hoyre_farge5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"fotgjenger_vendt_mot_hoyre", "1F6B6-200D-27A1-FE0F"},
    {"fotgjenger_vendt_mot_hoyre_farge1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"fotgjenger_vendt_mot_hoyre_farge2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"fotgjenger_vendt_mot_hoyre_farge3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"fotgjenger_vendt_mot_hoyre_farge4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"fotgjenger_vendt_mot_hoyre_farge5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"person_som_kneler_vendt_mot_hoyre", "1F9CE-200D-27A1-FE0F"},
    {"person_som_kneler_vendt_mot_hoyre_farge1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"person_som_kneler_vendt_mot_hoyre_farge2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"person_som_kneler_vendt_mot_hoyre_farge3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"person_som_kneler_vendt_mot_hoyre_farge4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"person_som_kneler_vendt_mot_hoyre_farge5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"helsearbeider", "1F9D1-200D-2695-FE0F"},
    {"helsearbeider_farge1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"helsearbeider_farge2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"helsearbeider_farge3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"helsearbeider_farge4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"helsearbeider_farge5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"dommer", "1F9D1-200D-2696-FE0F"},
    {"dommer_farge1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"dommer_farge2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"dommer_farge3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"dommer_farge4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"dommer_farge5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"pilot", "1F9D1-200D-2708-FE0F"},
    {"pilot_farge1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"pilot_farge2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"pilot_farge3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"pilot_farge4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"pilot_farge5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"gardbruker", "1F9D1-200D-1F33E"},
    {"gardbruker_farge1", "1F9D1-1F3FB-200D-1F33E"},
    {"gardbruker_farge2", "1F9D1-1F3FC-200D-1F33E"},
    {"gardbruker_farge3", "1F9D1-1F3FD-200D-1F33E"},
    {"gardbruker_farge4", "1F9D1-1F3FE-200D-1F33E"},
    {"gardbruker_farge5", "1F9D1-1F3FF-200D-1F33E"},
    {"kokk", "1F9D1-200D-1F373"},
    {"kokk_farge1", "1F9D1-1F3FB-200D-1F373"},
    {"kokk_farge2", "1F9D1-1F3FC-200D-1F373"},
    {"kokk_farge3", "1F9D1-1F3FD-200D-1F373"},
    {"kokk_farge4", "1F9D1-1F3FE-200D-1F373"},
    {"kokk_farge5", "1F9D1-1F3FF-200D-1F373"},
    {"person_som_mater_baby", "1F9D1-200D-1F37C"},
    {"person_som_mater_baby_farge1", "1F9D1-1F3FB-200D-1F37C"},
    {"person_som_mater_baby_farge2", "1F9D1-1F3FC-200D-1F37C"},
    {"person_som_mater_baby_farge3", "1F9D1-1F3FD-200D-1F37C"},
    {"person_som_mater_baby_farge4", "1F9D1-1F3FE-200D-1F37C"},
    {"person_som_mater_baby_farge5", "1F9D1-1F3FF-200D-1F37C"},
    {"julenisse", "1F9D1-200D-1F384"},
    {"julenisse_farge1", "1F9D1-1F3FB-200D-1F384"},
    {"julenisse_farge2", "1F9D1-1F3FC-200D-1F384"},
    {"julenisse_farge3", "1F9D1-1F3FD-200D-1F384"},
    {"julenisse_farge4", "1F9D1-1F3FE-200D-1F384"},
    {"julenisse_farge5", "1F9D1-1F3FF-200D-1F384"},
    {"student", "1F9D1-200D-1F393"},
    {"student_farge1", "1F9D1-1F3FB-200D-1F393"},
    {"student_farge2", "1F9D1-1F3FC-200D-1F393"},
    {"student_farge3", "1F9D1-1F3FD-200D-1F393"},
    {"student_farge4", "1F9D1-1F3FE-200D-1F393"},
    {"student_farge5", "1F9D1-1F3FF-200D-1F393"},
    {"artist", "1F9D1-200D-1F3A4"},
    {"artist_farge1", "1F9D1-1F3FB-200D-1F3A4"},
    {"artist_farge2", "1F9D1-1F3FC-200D-1F3A4"},
    {"artist_farge3", "1F9D1-1F3FD-200D-1F3A4"},
    {"artist_farge4", "1F9D1-1F3FE-200D-1F3A4"},
    {"artist_farge5", "1F9D1-1F3FF-200D-1F3A4"},
    {"kunstner", "1F9D1-200D-1F3A8"},
    {"kunstner_farge1", "1F9D1-1F3FB-200D-1F3A8"},
    {"kunstner_farge2", "1F9D1-1F3FC-200D-1F3A8"},
    {"kunstner_farge3", "1F9D1-1F3FD-200D-1F3A8"},
    {"kunstner_farge4", "1F9D1-1F3FE-200D-1F3A8"},
    {"kunstner_farge5", "1F9D1-1F3FF-200D-1F3A8"},
    {"laerer", "1F9D1-200D-1F3EB"},
    {"laerer_farge1", "1F9D1-1F3FB-200D-1F3EB"},
    {"laerer_farge2", "1F9D1-1F3FC-200D-1F3EB"},
    {"laerer_farge3", "1F9D1-1F3FD-200D-1F3EB"},
    {"laerer_farge4", "1F9D1-1F3FE-200D-1F3EB"},
    {"laerer_farge5", "1F9D1-1F3FF-200D-1F3EB"},
    {"industriarbeider", "1F9D1-200D-1F3ED"},
    {"industriarbeider_farge1", "1F9D1-1F3FB-200D-1F3ED"},
    {"industriarbeider_farge2", "1F9D1-1F3FC-200D-1F3ED"},
    {"industriarbeider_farge3", "1F9D1-1F3FD-200D-1F3ED"},
    {"industriarbeider_farge4", "1F9D1-1F3FE-200D-1F3ED"},
    {"industriarbeider_farge5", "1F9D1-1F3FF-200D-1F3ED"},
    {"it_ekspert", "1F9D1-200D-1F4BB"},
    {"it_ekspert_farge1", "1F9D1-1F3FB-200D-1F4BB"},
    {"it_ekspert_farge2", "1F9D1-1F3FC-200D-1F4BB"},
    {"it_ekspert_farge3", "1F9D1-1F3FD-200D-1F4BB"},
    {"it_ekspert_farge4", "1F9D1-1F3FE-200D-1F4BB"},
    {"it_ekspert_farge5", "1F9D1-1F3FF-200D-1F4BB"},
    {"kontorarbeider", "1F9D1-200D-1F4BC"},
    {"kontorarbeider_farge1", "1F9D1-1F3FB-200D-1F4BC"},
    {"kontorarbeider_farge2", "1F9D1-1F3FC-200D-1F4BC"},
    {"kontorarbeider_farge3", "1F9D1-1F3FD-200D-1F4BC"},
    {"kontorarbeider_farge4", "1F9D1-1F3FE-200D-1F4BC"},
    {"kontorarbeider_farge5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mekaniker", "1F9D1-200D-1F527"},
    {"mekaniker_farge1", "1F9D1-1F3FB-200D-1F527"},
    {"mekaniker_farge2", "1F9D1-1F3FC-200D-1F527"},
    {"mekaniker_farge3", "1F9D1-1F3FD-200D-1F527"},
    {"mekaniker_farge4", "1F9D1-1F3FE-200D-1F527"},
    {"mekaniker_farge5", "1F9D1-1F3FF-200D-1F527"},
    {"forsker", "1F9D1-200D-1F52C"},
    {"forsker_farge1", "1F9D1-1F3FB-200D-1F52C"},
    {"forsker_farge2", "1F9D1-1F3FC-200D-1F52C"},
    {"forsker_farge3", "1F9D1-1F3FD-200D-1F52C"},
    {"forsker_farge4", "1F9D1-1F3FE-200D-1F52C"},
    {"forsker_farge5", "1F9D1-1F3FF-200D-1F52C"},
    {"astronaut", "1F9D1-200D-1F680"},
    {"astronaut_farge1", "1F9D1-1F3FB-200D-1F680"},
    {"astronaut_farge2", "1F9D1-1F3FC-200D-1F680"},
    {"astronaut_farge3", "1F9D1-1F3FD-200D-1F680"},
    {"astronaut_farge4", "1F9D1-1F3FE-200D-1F680"},
    {"astronaut_farge5", "1F9D1-1F3FF-200D-1F680"},
    {"brannkonstabel", "1F9D1-200D-1F692"},
    {"brannkonstabel_farge1", "1F9D1-1F3FB-200D-1F692"},
    {"brannkonstabel_farge2", "1F9D1-1F3FC-200D-1F692"},
    {"brannkonstabel_farge3", "1F9D1-1F3FD-200D-1F692"},
    {"brannkonstabel_farge4", "1F9D1-1F3FE-200D-1F692"},
    {"brannkonstabel_farge5", "1F9D1-1F3FF-200D-1F692"},
    {"person_med_blindestokk", "1F9D1-200D-1F9AF"},
    {"person_med_blindestokk_farge1", "1F9D1-1F3FB-200D-1F9AF"},
    {"person_med_blindestokk_farge2", "1F9D1-1F3FC-200D-1F9AF"},
    {"person_med_blindestokk_farge3", "1F9D1-1F3FD-200D-1F9AF"},
    {"person_med_blindestokk_farge4", "1F9D1-1F3FE-200D-1F9AF"},
    {"person_med_blindestokk_farge5", "1F9D1-1F3FF-200D-1F9AF"},
    {"person_med_blindestokk_vendt_mot_hoyre", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestokk_vendt_mot_hoyre_farge1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestokk_vendt_mot_hoyre_farge2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestokk_vendt_mot_hoyre_farge3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestokk_vendt_mot_hoyre_farge4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestokk_vendt_mot_hoyre_farge5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"person_i_motorisert_rullestol", "1F9D1-200D-1F9BC"},
    {"person_i_motorisert_rullestol_farge1", "1F9D1-1F3FB-200D-1F9BC"},
    {"person_i_motorisert_rullestol_farge2", "1F9D1-1F3FC-200D-1F9BC"},
    {"person_i_motorisert_rullestol_farge3", "1F9D1-1F3FD-200D-1F9BC"},
    {"person_i_motorisert_rullestol_farge4", "1F9D1-1F3FE-200D-1F9BC"},
    {"person_i_motorisert_rullestol_farge5", "1F9D1-1F3FF-200D-1F9BC"},
    {"person_i_motorisert_rullestol_vendt_mot_hoyre", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_motorisert_rullestol_vendt_mot_hoyre_farge1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_motorisert_rullestol_vendt_mot_hoyre_farge2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_motorisert_rullestol_vendt_mot_hoyre_farge3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_motorisert_rullestol_vendt_mot_hoyre_farge4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_motorisert_rullestol_vendt_mot_hoyre_farge5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_manuell_rullestol", "1F9D1-200D-1F9BD"},
    {"person_i_manuell_rullestol_farge1", "1F9D1-1F3FB-200D-1F9BD"},
    {"person_i_manuell_rullestol_farge2", "1F9D1-1F3FC-200D-1F9BD"},
    {"person_i_manuell_rullestol_farge3", "1F9D1-1F3FD-200D-1F9BD"},
    {"person_i_manuell_rullestol_farge4", "1F9D1-1F3FE-200D-1F9BD"},
    {"person_i_manuell_rullestol_farge5", "1F9D1-1F3FF-200D-1F9BD"},
    {"person_i_manuell_rullestol_vendt_mot_hoyre", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullestol_vendt_mot_hoyre_farge1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullestol_vendt_mot_hoyre_farge2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullestol_vendt_mot_hoyre_farge3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullestol_vendt_mot_hoyre_farge4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullestol_vendt_mot_hoyre_farge5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinne_med_ball", "26F9-FE0F-200D-2640-FE0F"},
    {"kvinne_med_ball_farge1", "26F9-1F3FB-200D-2640-FE0F"},
    {"kvinne_med_ball_farge2", "26F9-1F3FC-200D-2640-FE0F"},
    {"kvinne_med_ball_farge3", "26F9-1F3FD-200D-2640-FE0F"},
    {"kvinne_med_ball_farge4", "26F9-1F3FE-200D-2640-FE0F"},
    {"kvinne_med_ball_farge5", "26F9-1F3FF-200D-2640-FE0F"},
    {"mann_med_ball", "26F9-FE0F-200D-2642-FE0F"},
    {"mann_med_ball_farge1", "26F9-1F3FB-200D-2642-FE0F"},
    {"mann_med_ball_farge2", "26F9-1F3FC-200D-2642-FE0F"},
    {"mann_med_ball_farge3", "26F9-1F3FD-200D-2642-FE0F"},
    {"mann_med_ball_farge4", "26F9-1F3FE-200D-2642-FE0F"},
    {"mann_med_ball_farge5", "26F9-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_loper", "1F3C3-200D-2640-FE0F"},
    {"kvinnelig_loper_farge1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_loper_farge2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_loper_farge3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_loper_farge4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_loper_farge5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"kvinnelig_loper_vendt_mot_hoyre", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_loper_vendt_mot_hoyre_farge1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_loper_vendt_mot_hoyre_farge2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_loper_vendt_mot_hoyre_farge3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_loper_vendt_mot_hoyre_farge4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_loper_vendt_mot_hoyre_farge5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mannlig_loper", "1F3C3-200D-2642-FE0F"},
    {"mannlig_loper_farge1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"mannlig_loper_farge2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"mannlig_loper_farge3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"mannlig_loper_farge4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"mannlig_loper_farge5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"mannlig_loper_vendt_mot_hoyre", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_loper_vendt_mot_hoyre_farge1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_loper_vendt_mot_hoyre_farge2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_loper_vendt_mot_hoyre_farge3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_loper_vendt_mot_hoyre_farge4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_loper_vendt_mot_hoyre_farge5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_surfer", "1F3C4-200D-2640-FE0F"},
    {"kvinnelig_surfer_farge1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_surfer_farge2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_surfer_farge3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_surfer_farge4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_surfer_farge5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"mannlig_surfer", "1F3C4-200D-2642-FE0F"},
    {"mannlig_surfer_farge1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"mannlig_surfer_farge2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"mannlig_surfer_farge3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"mannlig_surfer_farge4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"mannlig_surfer_farge5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_svommer", "1F3CA-200D-2640-FE0F"},
    {"kvinnelig_svommer_farge1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_svommer_farge2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_svommer_farge3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_svommer_farge4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_svommer_farge5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"mannlig_svommer", "1F3CA-200D-2642-FE0F"},
    {"mannlig_svommer_farge1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"mannlig_svommer_farge2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"mannlig_svommer_farge3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"mannlig_svommer_farge4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"mannlig_svommer_farge5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_vektlofter", "1F3CB-FE0F-200D-2640-FE0F"},
    {"kvinnelig_vektlofter_farge1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_vektlofter_farge2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_vektlofter_farge3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_vektlofter_farge4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_vektlofter_farge5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"mannlig_vektlofter", "1F3CB-FE0F-200D-2642-FE0F"},
    {"mannlig_vektlofter_farge1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"mannlig_vektlofter_farge2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"mannlig_vektlofter_farge3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"mannlig_vektlofter_farge4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"mannlig_vektlofter_farge5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_golfspiller", "1F3CC-FE0F-200D-2640-FE0F"},
    {"kvinnelig_golfspiller_farge1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_golfspiller_farge2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_golfspiller_farge3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_golfspiller_farge4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_golfspiller_farge5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"mannlig_golfspiller", "1F3CC-FE0F-200D-2642-FE0F"},
    {"mannlig_golfspiller_farge1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"mannlig_golfspiller_farge2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"mannlig_golfspiller_farge3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"mannlig_golfspiller_farge4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"mannlig_golfspiller_farge5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_politibetjent", "1F46E-200D-2640-FE0F"},
    {"kvinnelig_politibetjent_farge1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_politibetjent_farge2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_politibetjent_farge3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_politibetjent_farge4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_politibetjent_farge5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"mannlig_politibetjent", "1F46E-200D-2642-FE0F"},
    {"mannlig_politibetjent_farge1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"mannlig_politibetjent_farge2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"mannlig_politibetjent_farge3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"mannlig_politibetjent_farge4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"mannlig_politibetjent_farge5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"kvinner_med_kaninorer", "1F46F-200D-2640-FE0F"},
    {"kvinner_med_kaninorer_farge1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"kvinner_med_kaninorer_farge1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"kvinner_med_kaninorer_farge1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"kvinner_med_kaninorer_farge1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"kvinner_med_kaninorer_farge2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"kvinner_med_kaninorer_farge2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"kvinner_med_kaninorer_farge2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"kvinner_med_kaninorer_farge2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"kvinner_med_kaninorer_farge3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"kvinner_med_kaninorer_farge3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"kvinner_med_kaninorer_farge3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"kvinner_med_kaninorer_farge3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"kvinner_med_kaninorer_farge4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"kvinner_med_kaninorer_farge4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"kvinner_med_kaninorer_farge4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"kvinner_med_kaninorer_farge4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"kvinner_med_kaninorer_farge5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"kvinner_med_kaninorer_farge5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"kvinner_med_kaninorer_farge5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"kvinner_med_kaninorer_farge5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"kvinner_med_kaninorer_farge1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"kvinner_med_kaninorer_farge2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"kvinner_med_kaninorer_farge3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"kvinner_med_kaninorer_farge4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"kvinner_med_kaninorer_farge5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"menn_med_kaninorer", "1F46F-200D-2642-FE0F"},
    {"menn_med_kaninorer_farge1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"menn_med_kaninorer_farge1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"menn_med_kaninorer_farge1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"menn_med_kaninorer_farge1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"menn_med_kaninorer_farge2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"menn_med_kaninorer_farge2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"menn_med_kaninorer_farge2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"menn_med_kaninorer_farge2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"menn_med_kaninorer_farge3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"menn_med_kaninorer_farge3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"menn_med_kaninorer_farge3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"menn_med_kaninorer_farge3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"menn_med_kaninorer_farge4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"menn_med_kaninorer_farge4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"menn_med_kaninorer_farge4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"menn_med_kaninorer_farge4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"menn_med_kaninorer_farge5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"menn_med_kaninorer_farge5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"menn_med_kaninorer_farge5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"menn_med_kaninorer_farge5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"menn_med_kaninorer_farge1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"menn_med_kaninorer_farge2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"menn_med_kaninorer_farge3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"menn_med_kaninorer_farge4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"menn_med_kaninorer_farge5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"kvinne_med_slor", "1F470-200D-2640-FE0F"},
    {"kvinne_med_slor_farge1", "1F470-1F3FB-200D-2640-FE0F"},
    {"kvinne_med_slor_farge2", "1F470-1F3FC-200D-2640-FE0F"},
    {"kvinne_med_slor_farge3", "1F470-1F3FD-200D-2640-FE0F"},
    {"kvinne_med_slor_farge4", "1F470-1F3FE-200D-2640-FE0F"},
    {"kvinne_med_slor_farge5", "1F470-1F3FF-200D-2640-FE0F"},
    {"mann_med_slor", "1F470-200D-2642-FE0F"},
    {"mann_med_slor_farge1", "1F470-1F3FB-200D-2642-FE0F"},
    {"mann_med_slor_farge2", "1F470-1F3FC-200D-2642-FE0F"},
    {"mann_med_slor_farge3", "1F470-1F3FD-200D-2642-FE0F"},
    {"mann_med_slor_farge4", "1F470-1F3FE-200D-2642-FE0F"},
    {"mann_med_slor_farge5", "1F470-1F3FF-200D-2642-FE0F"},
    {"kvinne_blond", "1F471-200D-2640-FE0F"},
    {"kvinne_blond_farge1", "1F471-1F3FB-200D-2640-FE0F"},
    {"kvinne_blond_farge2", "1F471-1F3FC-200D-2640-FE0F"},
    {"kvinne_blond_farge3", "1F471-1F3FD-200D-2640-FE0F"},
    {"kvinne_blond_farge4", "1F471-1F3FE-200D-2640-FE0F"},
    {"kvinne_blond_farge5", "1F471-1F3FF-200D-2640-FE0F"},
    {"mann_blond", "1F471-200D-2642-FE0F"},
    {"mann_blond_farge1", "1F471-1F3FB-200D-2642-FE0F"},
    {"mann_blond_farge2", "1F471-1F3FC-200D-2642-FE0F"},
    {"mann_blond_farge3", "1F471-1F3FD-200D-2642-FE0F"},
    {"mann_blond_farge4", "1F471-1F3FE-200D-2642-FE0F"},
    {"mann_blond_farge5", "1F471-1F3FF-200D-2642-FE0F"},
    {"turbankledd_kvinne", "1F473-200D-2640-FE0F"},
    {"turbankledd_kvinne_farge1", "1F473-1F3FB-200D-2640-FE0F"},
    {"turbankledd_kvinne_farge2", "1F473-1F3FC-200D-2640-FE0F"},
    {"turbankledd_kvinne_farge3", "1F473-1F3FD-200D-2640-FE0F"},
    {"turbankledd_kvinne_farge4", "1F473-1F3FE-200D-2640-FE0F"},
    {"turbankledd_kvinne_farge5", "1F473-1F3FF-200D-2640-FE0F"},
    {"turbankledd_mann", "1F473-200D-2642-FE0F"},
    {"turbankledd_mann_farge1", "1F473-1F3FB-200D-2642-FE0F"},
    {"turbankledd_mann_farge2", "1F473-1F3FC-200D-2642-FE0F"},
    {"turbankledd_mann_farge3", "1F473-1F3FD-200D-2642-FE0F"},
    {"turbankledd_mann_farge4", "1F473-1F3FE-200D-2642-FE0F"},
    {"turbankledd_mann_farge5", "1F473-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_anleggsarbeider", "1F477-200D-2640-FE0F"},
    {"kvinnelig_anleggsarbeider_farge1", "1F477-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_anleggsarbeider_farge2", "1F477-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_anleggsarbeider_farge3", "1F477-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_anleggsarbeider_farge4", "1F477-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_anleggsarbeider_farge5", "1F477-1F3FF-200D-2640-FE0F"},
    {"mannlig_anleggsarbeider", "1F477-200D-2642-FE0F"},
    {"mannlig_anleggsarbeider_farge1", "1F477-1F3FB-200D-2642-FE0F"},
    {"mannlig_anleggsarbeider_farge2", "1F477-1F3FC-200D-2642-FE0F"},
    {"mannlig_anleggsarbeider_farge3", "1F477-1F3FD-200D-2642-FE0F"},
    {"mannlig_anleggsarbeider_farge4", "1F477-1F3FE-200D-2642-FE0F"},
    {"mannlig_anleggsarbeider_farge5", "1F477-1F3FF-200D-2642-FE0F"},
    {"hjelpende_kvinne", "1F481-200D-2640-FE0F"},
    {"hjelpende_kvinne_farge1", "1F481-1F3FB-200D-2640-FE0F"},
    {"hjelpende_kvinne_farge2", "1F481-1F3FC-200D-2640-FE0F"},
    {"hjelpende_kvinne_farge3", "1F481-1F3FD-200D-2640-FE0F"},
    {"hjelpende_kvinne_farge4", "1F481-1F3FE-200D-2640-FE0F"},
    {"hjelpende_kvinne_farge5", "1F481-1F3FF-200D-2640-FE0F"},
    {"hjelpende_mann", "1F481-200D-2642-FE0F"},
    {"hjelpende_mann_farge1", "1F481-1F3FB-200D-2642-FE0F"},
    {"hjelpende_mann_farge2", "1F481-1F3FC-200D-2642-FE0F"},
    {"hjelpende_mann_farge3", "1F481-1F3FD-200D-2642-FE0F"},
    {"hjelpende_mann_farge4", "1F481-1F3FE-200D-2642-FE0F"},
    {"hjelpende_mann_farge5", "1F481-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_gardist", "1F482-200D-2640-FE0F"},
    {"kvinnelig_gardist_farge1", "1F482-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_gardist_farge2", "1F482-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_gardist_farge3", "1F482-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_gardist_farge4", "1F482-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_gardist_farge5", "1F482-1F3FF-200D-2640-FE0F"},
    {"mannlig_gardist", "1F482-200D-2642-FE0F"},
    {"mannlig_gardist_farge1", "1F482-1F3FB-200D-2642-FE0F"},
    {"mannlig_gardist_farge2", "1F482-1F3FC-200D-2642-FE0F"},
    {"mannlig_gardist_farge3", "1F482-1F3FD-200D-2642-FE0F"},
    {"mannlig_gardist_farge4", "1F482-1F3FE-200D-2642-FE0F"},
    {"mannlig_gardist_farge5", "1F482-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_far_ansiktsmassasje", "1F486-200D-2640-FE0F"},
    {"kvinne_som_far_ansiktsmassasje_farge1", "1F486-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_far_ansiktsmassasje_farge2", "1F486-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_far_ansiktsmassasje_farge3", "1F486-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_far_ansiktsmassasje_farge4", "1F486-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_far_ansiktsmassasje_farge5", "1F486-1F3FF-200D-2640-FE0F"},
    {"mann_som_far_ansiktsmassasje", "1F486-200D-2642-FE0F"},
    {"mann_som_far_ansiktsmassasje_farge1", "1F486-1F3FB-200D-2642-FE0F"},
    {"mann_som_far_ansiktsmassasje_farge2", "1F486-1F3FC-200D-2642-FE0F"},
    {"mann_som_far_ansiktsmassasje_farge3", "1F486-1F3FD-200D-2642-FE0F"},
    {"mann_som_far_ansiktsmassasje_farge4", "1F486-1F3FE-200D-2642-FE0F"},
    {"mann_som_far_ansiktsmassasje_farge5", "1F486-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_far_harklipp", "1F487-200D-2640-FE0F"},
    {"kvinne_som_far_harklipp_farge1", "1F487-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_far_harklipp_farge2", "1F487-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_far_harklipp_farge3", "1F487-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_far_harklipp_farge4", "1F487-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_far_harklipp_farge5", "1F487-1F3FF-200D-2640-FE0F"},
    {"mann_som_far_harklipp", "1F487-200D-2642-FE0F"},
    {"mann_som_far_harklipp_farge1", "1F487-1F3FB-200D-2642-FE0F"},
    {"mann_som_far_harklipp_farge2", "1F487-1F3FC-200D-2642-FE0F"},
    {"mann_som_far_harklipp_farge3", "1F487-1F3FD-200D-2642-FE0F"},
    {"mann_som_far_harklipp_farge4", "1F487-1F3FE-200D-2642-FE0F"},
    {"mann_som_far_harklipp_farge5", "1F487-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_detektiv", "1F575-FE0F-200D-2640-FE0F"},
    {"kvinnelig_detektiv_farge1", "1F575-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_detektiv_farge2", "1F575-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_detektiv_farge3", "1F575-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_detektiv_farge4", "1F575-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_detektiv_farge5", "1F575-1F3FF-200D-2640-FE0F"},
    {"mannlig_detektiv", "1F575-FE0F-200D-2642-FE0F"},
    {"mannlig_detektiv_farge1", "1F575-1F3FB-200D-2642-FE0F"},
    {"mannlig_detektiv_farge2", "1F575-1F3FC-200D-2642-FE0F"},
    {"mannlig_detektiv_farge3", "1F575-1F3FD-200D-2642-FE0F"},
    {"mannlig_detektiv_farge4", "1F575-1F3FE-200D-2642-FE0F"},
    {"mannlig_detektiv_farge5", "1F575-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_gestikulerer_nei", "1F645-200D-2640-FE0F"},
    {"kvinne_som_gestikulerer_nei_farge1", "1F645-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_gestikulerer_nei_farge2", "1F645-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_gestikulerer_nei_farge3", "1F645-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_gestikulerer_nei_farge4", "1F645-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_gestikulerer_nei_farge5", "1F645-1F3FF-200D-2640-FE0F"},
    {"mann_som_gestikulerer_nei", "1F645-200D-2642-FE0F"},
    {"mann_som_gestikulerer_nei_farge1", "1F645-1F3FB-200D-2642-FE0F"},
    {"mann_som_gestikulerer_nei_farge2", "1F645-1F3FC-200D-2642-FE0F"},
    {"mann_som_gestikulerer_nei_farge3", "1F645-1F3FD-200D-2642-FE0F"},
    {"mann_som_gestikulerer_nei_farge4", "1F645-1F3FE-200D-2642-FE0F"},
    {"mann_som_gestikulerer_nei_farge5", "1F645-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_gestikulerer_ok", "1F646-200D-2640-FE0F"},
    {"kvinne_som_gestikulerer_ok_farge1", "1F646-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_gestikulerer_ok_farge2", "1F646-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_gestikulerer_ok_farge3", "1F646-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_gestikulerer_ok_farge4", "1F646-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_gestikulerer_ok_farge5", "1F646-1F3FF-200D-2640-FE0F"},
    {"mann_som_gestikulerer_ok", "1F646-200D-2642-FE0F"},
    {"mann_som_gestikulerer_ok_farge1", "1F646-1F3FB-200D-2642-FE0F"},
    {"mann_som_gestikulerer_ok_farge2", "1F646-1F3FC-200D-2642-FE0F"},
    {"mann_som_gestikulerer_ok_farge3", "1F646-1F3FD-200D-2642-FE0F"},
    {"mann_som_gestikulerer_ok_farge4", "1F646-1F3FE-200D-2642-FE0F"},
    {"mann_som_gestikulerer_ok_farge5", "1F646-1F3FF-200D-2642-FE0F"},
    {"bukkende_kvinne", "1F647-200D-2640-FE0F"},
    {"bukkende_kvinne_farge1", "1F647-1F3FB-200D-2640-FE0F"},
    {"bukkende_kvinne_farge2", "1F647-1F3FC-200D-2640-FE0F"},
    {"bukkende_kvinne_farge3", "1F647-1F3FD-200D-2640-FE0F"},
    {"bukkende_kvinne_farge4", "1F647-1F3FE-200D-2640-FE0F"},
    {"bukkende_kvinne_farge5", "1F647-1F3FF-200D-2640-FE0F"},
    {"bukkende_mann", "1F647-200D-2642-FE0F"},
    {"bukkende_mann_farge1", "1F647-1F3FB-200D-2642-FE0F"},
    {"bukkende_mann_farge2", "1F647-1F3FC-200D-2642-FE0F"},
    {"bukkende_mann_farge3", "1F647-1F3FD-200D-2642-FE0F"},
    {"bukkende_mann_farge4", "1F647-1F3FE-200D-2642-FE0F"},
    {"bukkende_mann_farge5", "1F647-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_rekker_opp_handen", "1F64B-200D-2640-FE0F"},
    {"kvinne_som_rekker_opp_handen_farge1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_rekker_opp_handen_farge2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_rekker_opp_handen_farge3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_rekker_opp_handen_farge4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_rekker_opp_handen_farge5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"mann_som_rekker_opp_handen", "1F64B-200D-2642-FE0F"},
    {"mann_som_rekker_opp_handen_farge1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"mann_som_rekker_opp_handen_farge2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"mann_som_rekker_opp_handen_farge3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"mann_som_rekker_opp_handen_farge4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"mann_som_rekker_opp_handen_farge5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_rynker_brynene", "1F64D-200D-2640-FE0F"},
    {"kvinne_som_rynker_brynene_farge1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_rynker_brynene_farge2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_rynker_brynene_farge3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_rynker_brynene_farge4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_rynker_brynene_farge5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"mann_som_rynker_brynene", "1F64D-200D-2642-FE0F"},
    {"mann_som_rynker_brynene_farge1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"mann_som_rynker_brynene_farge2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"mann_som_rynker_brynene_farge3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"mann_som_rynker_brynene_farge4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"mann_som_rynker_brynene_farge5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_furter", "1F64E-200D-2640-FE0F"},
    {"kvinne_som_furter_farge1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_furter_farge2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_furter_farge3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_furter_farge4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_furter_farge5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"mann_som_furter", "1F64E-200D-2642-FE0F"},
    {"mann_som_furter_farge1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"mann_som_furter_farge2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"mann_som_furter_farge3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"mann_som_furter_farge4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"mann_som_furter_farge5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"kvinne_i_robat", "1F6A3-200D-2640-FE0F"},
    {"kvinne_i_robat_farge1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"kvinne_i_robat_farge2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"kvinne_i_robat_farge3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"kvinne_i_robat_farge4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"kvinne_i_robat_farge5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"mann_i_robat", "1F6A3-200D-2642-FE0F"},
    {"mann_i_robat_farge1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"mann_i_robat_farge2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"mann_i_robat_farge3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"mann_i_robat_farge4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"mann_i_robat_farge5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_syklist", "1F6B4-200D-2640-FE0F"},
    {"kvinnelig_syklist_farge1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_syklist_farge2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_syklist_farge3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_syklist_farge4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_syklist_farge5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"mannlig_syklist", "1F6B4-200D-2642-FE0F"},
    {"mannlig_syklist_farge1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"mannlig_syklist_farge2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"mannlig_syklist_farge3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"mannlig_syklist_farge4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"mannlig_syklist_farge5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"kvinne_pa_terrengsykkel", "1F6B5-200D-2640-FE0F"},
    {"kvinne_pa_terrengsykkel_farge1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"kvinne_pa_terrengsykkel_farge2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"kvinne_pa_terrengsykkel_farge3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"kvinne_pa_terrengsykkel_farge4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"kvinne_pa_terrengsykkel_farge5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"mann_pa_terrengsykkel", "1F6B5-200D-2642-FE0F"},
    {"mann_pa_terrengsykkel_farge1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"mann_pa_terrengsykkel_farge2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"mann_pa_terrengsykkel_farge3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"mann_pa_terrengsykkel_farge4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"mann_pa_terrengsykkel_farge5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_fotgjenger", "1F6B6-200D-2640-FE0F"},
    {"kvinnelig_fotgjenger_farge1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_fotgjenger_farge2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_fotgjenger_farge3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_fotgjenger_farge4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_fotgjenger_farge5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"kvinnelig_fotgjenger_vendt_mot_hoyre", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_fotgjenger_vendt_mot_hoyre_farge1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_fotgjenger_vendt_mot_hoyre_farge2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_fotgjenger_vendt_mot_hoyre_farge3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_fotgjenger_vendt_mot_hoyre_farge4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_fotgjenger_vendt_mot_hoyre_farge5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mannlig_fotgjenger", "1F6B6-200D-2642-FE0F"},
    {"mannlig_fotgjenger_farge1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"mannlig_fotgjenger_farge2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"mannlig_fotgjenger_farge3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"mannlig_fotgjenger_farge4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"mannlig_fotgjenger_farge5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"mannlig_fotgjenger_vendt_mot_hoyre", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_fotgjenger_vendt_mot_hoyre_farge1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_fotgjenger_vendt_mot_hoyre_farge2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_fotgjenger_vendt_mot_hoyre_farge3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_fotgjenger_vendt_mot_hoyre_farge4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_fotgjenger_vendt_mot_hoyre_farge5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kvinne_som_slar_handen_mot_pannen", "1F926-200D-2640-FE0F"},
    {"kvinne_som_slar_handen_mot_pannen_farge1", "1F926-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_slar_handen_mot_pannen_farge2", "1F926-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_slar_handen_mot_pannen_farge3", "1F926-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_slar_handen_mot_pannen_farge4", "1F926-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_slar_handen_mot_pannen_farge5", "1F926-1F3FF-200D-2640-FE0F"},
    {"mann_som_slar_handen_mot_pannen", "1F926-200D-2642-FE0F"},
    {"mann_som_slar_handen_mot_pannen_farge1", "1F926-1F3FB-200D-2642-FE0F"},
    {"mann_som_slar_handen_mot_pannen_farge2", "1F926-1F3FC-200D-2642-FE0F"},
    {"mann_som_slar_handen_mot_pannen_farge3", "1F926-1F3FD-200D-2642-FE0F"},
    {"mann_som_slar_handen_mot_pannen_farge4", "1F926-1F3FE-200D-2642-FE0F"},
    {"mann_som_slar_handen_mot_pannen_farge5", "1F926-1F3FF-200D-2642-FE0F"},
    {"kvinne_i_smoking", "1F935-200D-2640-FE0F"},
    {"kvinne_i_smoking_farge1", "1F935-1F3FB-200D-2640-FE0F"},
    {"kvinne_i_smoking_farge2", "1F935-1F3FC-200D-2640-FE0F"},
    {"kvinne_i_smoking_farge3", "1F935-1F3FD-200D-2640-FE0F"},
    {"kvinne_i_smoking_farge4", "1F935-1F3FE-200D-2640-FE0F"},
    {"kvinne_i_smoking_farge5", "1F935-1F3FF-200D-2640-FE0F"},
    {"mann_i_smoking", "1F935-200D-2642-FE0F"},
    {"mann_i_smoking_farge1", "1F935-1F3FB-200D-2642-FE0F"},
    {"mann_i_smoking_farge2", "1F935-1F3FC-200D-2642-FE0F"},
    {"mann_i_smoking_farge3", "1F935-1F3FD-200D-2642-FE0F"},
    {"mann_i_smoking_farge4", "1F935-1F3FE-200D-2642-FE0F"},
    {"mann_i_smoking_farge5", "1F935-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_trekker_pa_skuldrene", "1F937-200D-2640-FE0F"},
    {"kvinne_som_trekker_pa_skuldrene_farge1", "1F937-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_trekker_pa_skuldrene_farge2", "1F937-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_trekker_pa_skuldrene_farge3", "1F937-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_trekker_pa_skuldrene_farge4", "1F937-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_trekker_pa_skuldrene_farge5", "1F937-1F3FF-200D-2640-FE0F"},
    {"mann_som_trekker_pa_skuldrene", "1F937-200D-2642-FE0F"},
    {"mann_som_trekker_pa_skuldrene_farge1", "1F937-1F3FB-200D-2642-FE0F"},
    {"mann_som_trekker_pa_skuldrene_farge2", "1F937-1F3FC-200D-2642-FE0F"},
    {"mann_som_trekker_pa_skuldrene_farge3", "1F937-1F3FD-200D-2642-FE0F"},
    {"mann_som_trekker_pa_skuldrene_farge4", "1F937-1F3FE-200D-2642-FE0F"},
    {"mann_som_trekker_pa_skuldrene_farge5", "1F937-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_slar_hjul", "1F938-200D-2640-FE0F"},
    {"kvinne_som_slar_hjul_farge1", "1F938-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_slar_hjul_farge2", "1F938-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_slar_hjul_farge3", "1F938-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_slar_hjul_farge4", "1F938-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_slar_hjul_farge5", "1F938-1F3FF-200D-2640-FE0F"},
    {"mann_som_slar_hjul", "1F938-200D-2642-FE0F"},
    {"mann_som_slar_hjul_farge1", "1F938-1F3FB-200D-2642-FE0F"},
    {"mann_som_slar_hjul_farge2", "1F938-1F3FC-200D-2642-FE0F"},
    {"mann_som_slar_hjul_farge3", "1F938-1F3FD-200D-2642-FE0F"},
    {"mann_som_slar_hjul_farge4", "1F938-1F3FE-200D-2642-FE0F"},
    {"mann_som_slar_hjul_farge5", "1F938-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_sjonglor", "1F939-200D-2640-FE0F"},
    {"kvinnelig_sjonglor_farge1", "1F939-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_sjonglor_farge2", "1F939-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_sjonglor_farge3", "1F939-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_sjonglor_farge4", "1F939-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_sjonglor_farge5", "1F939-1F3FF-200D-2640-FE0F"},
    {"mannlig_sjonglor", "1F939-200D-2642-FE0F"},
    {"mannlig_sjonglor_farge1", "1F939-1F3FB-200D-2642-FE0F"},
    {"mannlig_sjonglor_farge2", "1F939-1F3FC-200D-2642-FE0F"},
    {"mannlig_sjonglor_farge3", "1F939-1F3FD-200D-2642-FE0F"},
    {"mannlig_sjonglor_farge4", "1F939-1F3FE-200D-2642-FE0F"},
    {"mannlig_sjonglor_farge5", "1F939-1F3FF-200D-2642-FE0F"},
    {"kvinnelige_brytere", "1F93C-200D-2640-FE0F"},
    {"kvinnelige_brytere_farge1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"kvinnelige_brytere_farge1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"kvinnelige_brytere_farge1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"kvinnelige_brytere_farge1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"kvinnelige_brytere_farge2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"kvinnelige_brytere_farge2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"kvinnelige_brytere_farge2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"kvinnelige_brytere_farge2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"kvinnelige_brytere_farge3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"kvinnelige_brytere_farge3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"kvinnelige_brytere_farge3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"kvinnelige_brytere_farge3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"kvinnelige_brytere_farge4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"kvinnelige_brytere_farge4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"kvinnelige_brytere_farge4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"kvinnelige_brytere_farge4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"kvinnelige_brytere_farge5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"kvinnelige_brytere_farge5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"kvinnelige_brytere_farge5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"kvinnelige_brytere_farge5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"kvinnelige_brytere_farge1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"kvinnelige_brytere_farge2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"kvinnelige_brytere_farge3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"kvinnelige_brytere_farge4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"kvinnelige_brytere_farge5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"mannlige_brytere", "1F93C-200D-2642-FE0F"},
    {"mannlige_brytere_farge1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"mannlige_brytere_farge1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"mannlige_brytere_farge1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"mannlige_brytere_farge1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"mannlige_brytere_farge2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"mannlige_brytere_farge2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"mannlige_brytere_farge2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"mannlige_brytere_farge2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"mannlige_brytere_farge3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"mannlige_brytere_farge3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"mannlige_brytere_farge3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"mannlige_brytere_farge3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"mannlige_brytere_farge4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"mannlige_brytere_farge4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"mannlige_brytere_farge4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"mannlige_brytere_farge4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"mannlige_brytere_farge5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"mannlige_brytere_farge5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"mannlige_brytere_farge5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"mannlige_brytere_farge5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"mannlige_brytere_farge1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"mannlige_brytere_farge2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"mannlige_brytere_farge3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"mannlige_brytere_farge4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"mannlige_brytere_farge5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_vannpolospiller", "1F93D-200D-2640-FE0F"},
    {"kvinnelig_vannpolospiller_farge1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_vannpolospiller_farge2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_vannpolospiller_farge3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_vannpolospiller_farge4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_vannpolospiller_farge5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"mannlig_vannpolospiller", "1F93D-200D-2642-FE0F"},
    {"mannlig_vannpolospiller_farge1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"mannlig_vannpolospiller_farge2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"mannlig_vannpolospiller_farge3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"mannlig_vannpolospiller_farge4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"mannlig_vannpolospiller_farge5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_handballspiller", "1F93E-200D-2640-FE0F"},
    {"kvinnelig_handballspiller_farge1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_handballspiller_farge2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_handballspiller_farge3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_handballspiller_farge4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_handballspiller_farge5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"mannlig_handballspiller", "1F93E-200D-2642-FE0F"},
    {"mannlig_handballspiller_farge1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"mannlig_handballspiller_farge2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"mannlig_handballspiller_farge3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"mannlig_handballspiller_farge4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"mannlig_handballspiller_farge5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_superhelt", "1F9B8-200D-2640-FE0F"},
    {"kvinnelig_superhelt_farge1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_superhelt_farge2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_superhelt_farge3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_superhelt_farge4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_superhelt_farge5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"mannlig_superhelt", "1F9B8-200D-2642-FE0F"},
    {"mannlig_superhelt_farge1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"mannlig_superhelt_farge2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"mannlig_superhelt_farge3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"mannlig_superhelt_farge4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"mannlig_superhelt_farge5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_superskurk", "1F9B9-200D-2640-FE0F"},
    {"kvinnelig_superskurk_farge1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_superskurk_farge2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_superskurk_farge3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_superskurk_farge4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_superskurk_farge5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"mannlig_superskurk", "1F9B9-200D-2642-FE0F"},
    {"mannlig_superskurk_farge1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"mannlig_superskurk_farge2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"mannlig_superskurk_farge3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"mannlig_superskurk_farge4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"mannlig_superskurk_farge5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_star", "1F9CD-200D-2640-FE0F"},
    {"kvinne_som_star_farge1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_star_farge2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_star_farge3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_star_farge4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_star_farge5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"mann_som_star", "1F9CD-200D-2642-FE0F"},
    {"mann_som_star_farge1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"mann_som_star_farge2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"mann_som_star_farge3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"mann_som_star_farge4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"mann_som_star_farge5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_kneler", "1F9CE-200D-2640-FE0F"},
    {"kvinne_som_kneler_farge1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_kneler_farge2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_kneler_farge3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_kneler_farge4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_kneler_farge5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"kvinne_som_kneler_vendt_mot_hoyre", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinne_som_kneler_vendt_mot_hoyre_farge1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinne_som_kneler_vendt_mot_hoyre_farge2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinne_som_kneler_vendt_mot_hoyre_farge3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinne_som_kneler_vendt_mot_hoyre_farge4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinne_som_kneler_vendt_mot_hoyre_farge5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mann_som_kneler", "1F9CE-200D-2642-FE0F"},
    {"mann_som_kneler_farge1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"mann_som_kneler_farge2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"mann_som_kneler_farge3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"mann_som_kneler_farge4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"mann_som_kneler_farge5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"mann_som_kneler_vendt_mot_hoyre", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mann_som_kneler_vendt_mot_hoyre_farge1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mann_som_kneler_vendt_mot_hoyre_farge2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mann_som_kneler_vendt_mot_hoyre_farge3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mann_som_kneler_vendt_mot_hoyre_farge4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mann_som_kneler_vendt_mot_hoyre_farge5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"dov_kvinne", "1F9CF-200D-2640-FE0F"},
    {"dov_kvinne_farge1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"dov_kvinne_farge2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"dov_kvinne_farge3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"dov_kvinne_farge4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"dov_kvinne_farge5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"dov_mann", "1F9CF-200D-2642-FE0F"},
    {"dov_mann_farge1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"dov_mann_farge2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"dov_mann_farge3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"dov_mann_farge4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"dov_mann_farge5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"kvinne_skjegg", "1F9D4-200D-2640-FE0F"},
    {"kvinne_skjegg_farge1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"kvinne_skjegg_farge2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"kvinne_skjegg_farge3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"kvinne_skjegg_farge4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"kvinne_skjegg_farge5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"mann_skjegg", "1F9D4-200D-2642-FE0F"},
    {"mann_skjegg_farge1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"mann_skjegg_farge2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"mann_skjegg_farge3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"mann_skjegg_farge4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"mann_skjegg_farge5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"kvinne_i_badstu", "1F9D6-200D-2640-FE0F"},
    {"kvinne_i_badstu_farge1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"kvinne_i_badstu_farge2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"kvinne_i_badstu_farge3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"kvinne_i_badstu_farge4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"kvinne_i_badstu_farge5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"mann_i_badstu", "1F9D6-200D-2642-FE0F"},
    {"mann_i_badstu_farge1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"mann_i_badstu_farge2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"mann_i_badstu_farge3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"mann_i_badstu_farge4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"mann_i_badstu_farge5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_klatrer", "1F9D7-200D-2640-FE0F"},
    {"kvinnelig_klatrer_farge1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_klatrer_farge2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_klatrer_farge3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_klatrer_farge4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_klatrer_farge5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"mannlig_klatrer", "1F9D7-200D-2642-FE0F"},
    {"mannlig_klatrer_farge1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"mannlig_klatrer_farge2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"mannlig_klatrer_farge3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"mannlig_klatrer_farge4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"mannlig_klatrer_farge5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"kvinne_i_lotusstilling", "1F9D8-200D-2640-FE0F"},
    {"kvinne_i_lotusstilling_farge1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"kvinne_i_lotusstilling_farge2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"kvinne_i_lotusstilling_farge3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"kvinne_i_lotusstilling_farge4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"kvinne_i_lotusstilling_farge5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"mann_i_lotusstilling", "1F9D8-200D-2642-FE0F"},
    {"mann_i_lotusstilling_farge1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"mann_i_lotusstilling_farge2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"mann_i_lotusstilling_farge3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"mann_i_lotusstilling_farge4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"mann_i_lotusstilling_farge5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"trollkvinne", "1F9D9-200D-2640-FE0F"},
    {"trollkvinne_farge1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"trollkvinne_farge2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"trollkvinne_farge3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"trollkvinne_farge4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"trollkvinne_farge5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"trollmann", "1F9D9-200D-2642-FE0F"},
    {"trollmann_farge1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"trollmann_farge2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"trollmann_farge3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"trollmann_farge4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"trollmann_farge5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_fe", "1F9DA-200D-2640-FE0F"},
    {"kvinnelig_fe_farge1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_fe_farge2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_fe_farge3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_fe_farge4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_fe_farge5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"mannlig_fe", "1F9DA-200D-2642-FE0F"},
    {"mannlig_fe_farge1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"mannlig_fe_farge2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"mannlig_fe_farge3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"mannlig_fe_farge4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"mannlig_fe_farge5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_vampyr", "1F9DB-200D-2640-FE0F"},
    {"kvinnelig_vampyr_farge1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_vampyr_farge2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_vampyr_farge3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_vampyr_farge4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_vampyr_farge5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"mannlig_vampyr", "1F9DB-200D-2642-FE0F"},
    {"mannlig_vampyr_farge1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"mannlig_vampyr_farge2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"mannlig_vampyr_farge3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"mannlig_vampyr_farge4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"mannlig_vampyr_farge5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"havfrue", "1F9DC-200D-2640-FE0F"},
    {"havfrue_farge1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"havfrue_farge2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"havfrue_farge3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"havfrue_farge4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"havfrue_farge5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"havmann", "1F9DC-200D-2642-FE0F"},
    {"havmann_farge1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"havmann_farge2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"havmann_farge3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"havmann_farge4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"havmann_farge5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_alv", "1F9DD-200D-2640-FE0F"},
    {"kvinnelig_alv_farge1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_alv_farge2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_alv_farge3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_alv_farge4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_alv_farge5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"mannlig_alv", "1F9DD-200D-2642-FE0F"},
    {"mannlig_alv_farge1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"mannlig_alv_farge2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"mannlig_alv_farge3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"mannlig_alv_farge4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"mannlig_alv_farge5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_and", "1F9DE-200D-2640-FE0F"},
    {"mannlig_and", "1F9DE-200D-2642-FE0F"},
    {"kvinnelig_zombie", "1F9DF-200D-2640-FE0F"},
    {"mannlig_zombie", "1F9DF-200D-2642-FE0F"},
    {"mann_rodt_har", "1F468-200D-1F9B0"},
    {"mann_rodt_har_farge1", "1F468-1F3FB-200D-1F9B0"},
    {"mann_rodt_har_farge2", "1F468-1F3FC-200D-1F9B0"},
    {"mann_rodt_har_farge3", "1F468-1F3FD-200D-1F9B0"},
    {"mann_rodt_har_farge4", "1F468-1F3FE-200D-1F9B0"},
    {"mann_rodt_har_farge5", "1F468-1F3FF-200D-1F9B0"},
    {"mann_kruset_har", "1F468-200D-1F9B1"},
    {"mann_kruset_har_farge1", "1F468-1F3FB-200D-1F9B1"},
    {"mann_kruset_har_farge2", "1F468-1F3FC-200D-1F9B1"},
    {"mann_kruset_har_farge3", "1F468-1F3FD-200D-1F9B1"},
    {"mann_kruset_har_farge4", "1F468-1F3FE-200D-1F9B1"},
    {"mann_kruset_har_farge5", "1F468-1F3FF-200D-1F9B1"},
    {"mann_skallet", "1F468-200D-1F9B2"},
    {"mann_skallet_farge1", "1F468-1F3FB-200D-1F9B2"},
    {"mann_skallet_farge2", "1F468-1F3FC-200D-1F9B2"},
    {"mann_skallet_farge3", "1F468-1F3FD-200D-1F9B2"},
    {"mann_skallet_farge4", "1F468-1F3FE-200D-1F9B2"},
    {"mann_skallet_farge5", "1F468-1F3FF-200D-1F9B2"},
    {"mann_gratt_har", "1F468-200D-1F9B3"},
    {"mann_gratt_har_farge1", "1F468-1F3FB-200D-1F9B3"},
    {"mann_gratt_har_farge2", "1F468-1F3FC-200D-1F9B3"},
    {"mann_gratt_har_farge3", "1F468-1F3FD-200D-1F9B3"},
    {"mann_gratt_har_farge4", "1F468-1F3FE-200D-1F9B3"},
    {"mann_gratt_har_farge5", "1F468-1F3FF-200D-1F9B3"},
    {"kvinne_rodt_har", "1F469-200D-1F9B0"},
    {"kvinne_rodt_har_farge1", "1F469-1F3FB-200D-1F9B0"},
    {"kvinne_rodt_har_farge2", "1F469-1F3FC-200D-1F9B0"},
    {"kvinne_rodt_har_farge3", "1F469-1F3FD-200D-1F9B0"},
    {"kvinne_rodt_har_farge4", "1F469-1F3FE-200D-1F9B0"},
    {"kvinne_rodt_har_farge5", "1F469-1F3FF-200D-1F9B0"},
    {"kvinne_kruset_har", "1F469-200D-1F9B1"},
    {"kvinne_kruset_har_farge1", "1F469-1F3FB-200D-1F9B1"},
    {"kvinne_kruset_har_farge2", "1F469-1F3FC-200D-1F9B1"},
    {"kvinne_kruset_har_farge3", "1F469-1F3FD-200D-1F9B1"},
    {"kvinne_kruset_har_farge4", "1F469-1F3FE-200D-1F9B1"},
    {"kvinne_kruset_har_farge5", "1F469-1F3FF-200D-1F9B1"},
    {"kvinne_skallet", "1F469-200D-1F9B2"},
    {"kvinne_skallet_farge1", "1F469-1F3FB-200D-1F9B2"},
    {"kvinne_skallet_farge2", "1F469-1F3FC-200D-1F9B2"},
    {"kvinne_skallet_farge3", "1F469-1F3FD-200D-1F9B2"},
    {"kvinne_skallet_farge4", "1F469-1F3FE-200D-1F9B2"},
    {"kvinne_skallet_farge5", "1F469-1F3FF-200D-1F9B2"},
    {"kvinne_gratt_har", "1F469-200D-1F9B3"},
    {"kvinne_gratt_har_farge1", "1F469-1F3FB-200D-1F9B3"},
    {"kvinne_gratt_har_farge2", "1F469-1F3FC-200D-1F9B3"},
    {"kvinne_gratt_har_farge3", "1F469-1F3FD-200D-1F9B3"},
    {"kvinne_gratt_har_farge4", "1F469-1F3FE-200D-1F9B3"},
    {"kvinne_gratt_har_farge5", "1F469-1F3FF-200D-1F9B3"},
    {"voksen_rodt_har", "1F9D1-200D-1F9B0"},
    {"voksen_rodt_har_farge1", "1F9D1-1F3FB-200D-1F9B0"},
    {"voksen_rodt_har_farge2", "1F9D1-1F3FC-200D-1F9B0"},
    {"voksen_rodt_har_farge3", "1F9D1-1F3FD-200D-1F9B0"},
    {"voksen_rodt_har_farge4", "1F9D1-1F3FE-200D-1F9B0"},
    {"voksen_rodt_har_farge5", "1F9D1-1F3FF-200D-1F9B0"},
    {"voksen_kruset_har", "1F9D1-200D-1F9B1"},
    {"voksen_kruset_har_farge1", "1F9D1-1F3FB-200D-1F9B1"},
    {"voksen_kruset_har_farge2", "1F9D1-1F3FC-200D-1F9B1"},
    {"voksen_kruset_har_farge3", "1F9D1-1F3FD-200D-1F9B1"},
    {"voksen_kruset_har_farge4", "1F9D1-1F3FE-200D-1F9B1"},
    {"voksen_kruset_har_farge5", "1F9D1-1F3FF-200D-1F9B1"},
    {"voksen_skallet", "1F9D1-200D-1F9B2"},
    {"voksen_skallet_farge1", "1F9D1-1F3FB-200D-1F9B2"},
    {"voksen_skallet_farge2", "1F9D1-1F3FC-200D-1F9B2"},
    {"voksen_skallet_farge3", "1F9D1-1F3FD-200D-1F9B2"},
    {"voksen_skallet_farge4", "1F9D1-1F3FE-200D-1F9B2"},
    {"voksen_skallet_farge5", "1F9D1-1F3FF-200D-1F9B2"},
    {"voksen_gratt_har", "1F9D1-200D-1F9B3"},
    {"voksen_gratt_har_farge1", "1F9D1-1F3FB-200D-1F9B3"},
    {"voksen_gratt_har_farge2", "1F9D1-1F3FC-200D-1F9B3"},
    {"voksen_gratt_har_farge3", "1F9D1-1F3FD-200D-1F9B3"},
    {"voksen_gratt_har_farge4", "1F9D1-1F3FE-200D-1F9B3"},
    {"voksen_gratt_har_farge5", "1F9D1-1F3FF-200D-1F9B3"},
    {"brutt_lenke", "26D3-FE0F-200D-1F4A5"},
    {"hjerte_i_brann", "2764-FE0F-200D-1F525"},
    {"lindret_hjerte", "2764-FE0F-200D-1FA79"},
    {"brun_sopp", "1F344-200D-1F7EB"},
    {"lime", "1F34B-200D-1F7E9"},
    {"transpersonflagg", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"regnbueflagg", "1F3F3-FE0F-200D-1F308"},
    {"sjoroverflagg", "1F3F4-200D-2620-FE0F"},
    {"svart_katt", "1F408-200D-2B1B"},
    {"servicehund", "1F415-200D-1F9BA"},
    {"svart_fugl", "1F426-200D-2B1B"},
    {"foniks", "1F426-200D-1F525"},
    {"isbjorn", "1F43B-200D-2744-FE0F"},
    {"oye_i_snakkeboble", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"puster_ut", "1F62E-200D-1F4A8"},
    {"fjes_med_spiraloyne", "1F635-200D-1F4AB"},
    {"fjes_i_skyer", "1F636-200D-1F32B-FE0F"},
    {"ristende_hode", "1F642-200D-2194-FE0F"},
    {"nikkende_hode", "1F642-200D-2195-FE0F"},
    {"ballettdanser", "1F9D1-200D-1FA70"},
    {"ballettdanser_farge1", "1F9D1-1F3FB-200D-1FA70"},
    {"ballettdanser_farge2", "1F9D1-1F3FC-200D-1FA70"},
    {"ballettdanser_farge3", "1F9D1-1F3FD-200D-1FA70"},
    {"ballettdanser_farge4", "1F9D1-1F3FE-200D-1FA70"},
    {"ballettdanser_farge5", "1F9D1-1F3FF-200D-1FA70"},
    {"utropstegn_og_spørsmålstegn", "2049"},
    {"pil_venstre_høyre", "2194"},
    {"pil_opp_høyre", "2197"},
    {"pil_ned_høyre", "2198"},
    {"snømann", "2603"},
    {"paraply_med_regndråper", "2614"},
    {"trekløver", "2618"},
    {"dødningehode", "2620"},
    {"væren", "2648"},
    {"kløver", "2663"},
    {"likearmet_skålvekt", "2696"},
    {"asterisk_med_åtte_spisser", "2733"},
    {"åttetagget_stjerne", "2734"},
    {"snøfnugg", "2744"},
    {"rødt_spørsmålstegn", "2753"},
    {"hvitt_spørsmålstegn", "2754"},
    {"rødt_utropstegn", "2757"},
    {"rødt_hjerte", "2764"},
    {"pil_som_bøyer_opp", "2934"},
    {"pil_som_bøyer_ned", "2935"},
    {"bølgestrek", "3030"},
    {"gratulerer_på_japansk", "3297"},
    {"hemmelig_på_japansk", "3299"},
    {"pil_som_bøyer_mot_venstre", "21A9"},
    {"pil_som_bøyer_mot_høyre", "21AA"},
    {"armbåndsur", "231A"},
    {"stjerne_og_halvmåne", "262A"},
    {"løven", "264C"},
    {"høyspenning", "26A1"},
    {"snømann_uten_snø", "26C4"},
    {"slangebæreren", "26CE"},
    {"parasoll_på_bakken", "26F1"},
    {"seilbåt", "26F5"},
    {"skiløper", "26F7"},
    {"skøyter", "26F8"},
    {"hevet_hånd", "270B"},
    {"hevet_hånd_farge1", "270B-1F3FB"},
    {"hevet_hånd_farge2", "270B-1F3FC"},
    {"hevet_hånd_farge3", "270B-1F3FD"},
    {"hevet_hånd_farge4", "270B-1F3FE"},
    {"hevet_hånd_farge5", "270B-1F3FF"},
    {"skrivende_hånd", "270D"},
    {"skrivende_hånd_farge1", "270D-1F3FB"},
    {"skrivende_hånd_farge2", "270D-1F3FC"},
    {"skrivende_hånd_farge3", "270D-1F3FD"},
    {"skrivende_hånd_farge4", "270D-1F3FE"},
    {"skrivende_hånd_farge5", "270D-1F3FF"},
    {"pil_høyre", "27A1"},
    {"løkke", "27B0"},
    {"dobbel_løkke", "27BF"},
    {"tykk_rød_sirkel", "2B55"},
    {"mahjongbrikke_med_rød_drage", "1F004"},
    {"her_på_japansk", "1F201"},
    {"gebyr_på_japansk", "1F202"},
    {"gratis_på_japansk", "1F21A"},
    {"reservert_på_japansk", "1F22F"},
    {"forbudt_på_japansk", "1F232"},
    {"ledig_på_japansk", "1F233"},
    {"bestått_på_japansk", "1F234"},
    {"fullt_på_japansk", "1F235"},
    {"ikke_gratis_på_japansk", "1F236"},
    {"månedlig_beløp_på_japansk", "1F237"},
    {"søknad_på_japansk", "1F238"},
    {"rabatt_på_japansk", "1F239"},
    {"åpen_for_handel_på_japansk", "1F23A"},
    {"spesialtilbud_på_japansk", "1F250"},
    {"akseptabelt_på_japansk", "1F251"},
    {"tåkete", "1F301"},
    {"bølge", "1F30A"},
    {"nymåne", "1F311"},
    {"voksende_nymåne", "1F312"},
    {"voksende_halvmåne", "1F313"},
    {"voksende_fullmåne", "1F314"},
    {"fullmåne", "1F315"},
    {"avtakende_fullmåne", "1F316"},
    {"avtakende_halvmåne", "1F317"},
    {"avtakende_nemåne", "1F318"},
    {"månesigd", "1F319"},
    {"nymåneansikt", "1F31A"},
    {"nymåne_med_ansikt", "1F31B"},
    {"nemåne_med_ansikt", "1F31C"},
    {"fullmåneansikt", "1F31D"},
    {"sky_med_snø", "1F328"},
    {"tåke", "1F32B"},
    {"pølse_med_brød", "1F32D"},
    {"løvtre", "1F333"},
    {"rød_chili", "1F336"},
    {"kirsebærblomst", "1F338"},
    {"firkløver", "1F340"},
    {"lønneblad", "1F341"},
    {"høstblad", "1F342"},
    {"rødt_eple", "1F34E"},
    {"grønt_eple", "1F34F"},
    {"pære", "1F350"},
    {"kirsebær", "1F352"},
    {"jordbær", "1F353"},
    {"kjøtt_med_bein", "1F356"},
    {"kyllinglår", "1F357"},
    {"brød", "1F35E"},
    {"bakt_søtpotet", "1F360"},
    {"fiskekake_med_mønster", "1F365"},
    {"kjærlighet_på_pinne", "1F36D"},
    {"ølkrus", "1F37A"},
    {"ølkrus_som_skåler", "1F37B"},
    {"tåteflaske", "1F37C"},
    {"sløyfe", "1F380"},
    {"stjerneskudd_på_pinne", "1F387"},
    {"månebeskuelsesseremoni", "1F391"},
    {"militærmedalje", "1F396"},
    {"minnesløyfe", "1F397"},
    {"treningstrøye", "1F3BD"},
    {"løper", "1F3C3"},
    {"løper_farge1", "1F3C3-1F3FB"},
    {"løper_farge2", "1F3C3-1F3FC"},
    {"løper_farge3", "1F3C3-1F3FD"},
    {"løper_farge4", "1F3C3-1F3FE"},
    {"løper_farge5", "1F3C3-1F3FF"},
    {"trofé", "1F3C6"},
    {"hesteveddeløp", "1F3C7"},
    {"hesteveddeløp_farge1", "1F3C7-1F3FB"},
    {"hesteveddeløp_farge2", "1F3C7-1F3FC"},
    {"hesteveddeløp_farge3", "1F3C7-1F3FD"},
    {"hesteveddeløp_farge4", "1F3C7-1F3FE"},
    {"hesteveddeløp_farge5", "1F3C7-1F3FF"},
    {"svømmer", "1F3CA"},
    {"svømmer_farge1", "1F3CA-1F3FB"},
    {"svømmer_farge2", "1F3CA-1F3FC"},
    {"svømmer_farge3", "1F3CA-1F3FD"},
    {"svømmer_farge4", "1F3CA-1F3FE"},
    {"svømmer_farge5", "1F3CA-1F3FF"},
    {"vektløfter", "1F3CB"},
    {"vektløfter_farge1", "1F3CB-1F3FB"},
    {"vektløfter_farge2", "1F3CB-1F3FC"},
    {"vektløfter_farge3", "1F3CB-1F3FD"},
    {"vektløfter_farge4", "1F3CB-1F3FE"},
    {"vektløfter_farge5", "1F3CB-1F3FF"},
    {"snødekt_fjell", "1F3D4"},
    {"ørken", "1F3DC"},
    {"øde_øy", "1F3DD"},
    {"kjærlighetshotell", "1F3E9"},
    {"kjøpesenter", "1F3EC"},
    {"rød_papirlykt", "1F3EE"},
    {"hudtype_12", "1F3FB"},
    {"vannbøffel", "1F403"},
    {"søye", "1F411"},
    {"høne", "1F414"},
    {"marihøne", "1F41E"},
    {"bjørn", "1F43B"},
    {"øyne", "1F440"},
    {"øye", "1F441"},
    {"øre", "1F442"},
    {"øre_farge1", "1F442-1F3FB"},
    {"øre_farge2", "1F442-1F3FC"},
    {"øre_farge3", "1F442-1F3FD"},
    {"øre_farge4", "1F442-1F3FE"},
    {"øre_farge5", "1F442-1F3FF"},
    {"peker_opp_bakhånd", "1F446"},
    {"peker_opp_bakhånd_farge1", "1F446-1F3FB"},
    {"peker_opp_bakhånd_farge2", "1F446-1F3FC"},
    {"peker_opp_bakhånd_farge3", "1F446-1F3FD"},
    {"peker_opp_bakhånd_farge4", "1F446-1F3FE"},
    {"peker_opp_bakhånd_farge5", "1F446-1F3FF"},
    {"peker_ned_bakhånd", "1F447"},
    {"peker_ned_bakhånd_farge1", "1F447-1F3FB"},
    {"peker_ned_bakhånd_farge2", "1F447-1F3FC"},
    {"peker_ned_bakhånd_farge3", "1F447-1F3FD"},
    {"peker_ned_bakhånd_farge4", "1F447-1F3FE"},
    {"peker_ned_bakhånd_farge5", "1F447-1F3FF"},
    {"peker_mot_venstre_bakhånd", "1F448"},
    {"peker_mot_venstre_bakhånd_farge1", "1F448-1F3FB"},
    {"peker_mot_venstre_bakhånd_farge2", "1F448-1F3FC"},
    {"peker_mot_venstre_bakhånd_farge3", "1F448-1F3FD"},
    {"peker_mot_venstre_bakhånd_farge4", "1F448-1F3FE"},
    {"peker_mot_venstre_bakhånd_farge5", "1F448-1F3FF"},
    {"peker_mot_høyre_bakhånd", "1F449"},
    {"peker_mot_høyre_bakhånd_farge1", "1F449-1F3FB"},
    {"peker_mot_høyre_bakhånd_farge2", "1F449-1F3FC"},
    {"peker_mot_høyre_bakhånd_farge3", "1F449-1F3FD"},
    {"peker_mot_høyre_bakhånd_farge4", "1F449-1F3FE"},
    {"peker_mot_høyre_bakhånd_farge5", "1F449-1F3FF"},
    {"vinkende_hånd", "1F44B"},
    {"vinkende_hånd_farge1", "1F44B-1F3FB"},
    {"vinkende_hånd_farge2", "1F44B-1F3FC"},
    {"vinkende_hånd_farge3", "1F44B-1F3FD"},
    {"vinkende_hånd_farge4", "1F44B-1F3FE"},
    {"vinkende_hånd_farge5", "1F44B-1F3FF"},
    {"ok_hånd", "1F44C"},
    {"ok_hånd_farge1", "1F44C-1F3FB"},
    {"ok_hånd_farge2", "1F44C-1F3FC"},
    {"ok_hånd_farge3", "1F44C-1F3FD"},
    {"ok_hånd_farge4", "1F44C-1F3FE"},
    {"ok_hånd_farge5", "1F44C-1F3FF"},
    {"åpne_hender", "1F450"},
    {"åpne_hender_farge1", "1F450-1F3FB"},
    {"åpne_hender_farge2", "1F450-1F3FC"},
    {"åpne_hender_farge3", "1F450-1F3FD"},
    {"åpne_hender_farge4", "1F450-1F3FE"},
    {"åpne_hender_farge5", "1F450-1F3FF"},
    {"dameklær", "1F45A"},
    {"portemoné", "1F45B"},
    {"håndveske", "1F45C"},
    {"høyhælt_sko", "1F460"},
    {"damestøvlett", "1F462"},
    {"personer_med_kaninører", "1F46F"},
    {"personer_med_kaninører_farge1", "1F46F-1F3FB"},
    {"personer_med_kaninører_farge2", "1F46F-1F3FC"},
    {"personer_med_kaninører_farge3", "1F46F-1F3FD"},
    {"personer_med_kaninører_farge4", "1F46F-1F3FE"},
    {"personer_med_kaninører_farge5", "1F46F-1F3FF"},
    {"personer_med_kaninører_farge1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninører_farge1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninører_farge1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninører_farge1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninører_farge2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninører_farge2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninører_farge2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninører_farge2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninører_farge3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninører_farge3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninører_farge3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninører_farge3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninører_farge4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninører_farge4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninører_farge4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninører_farge4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninører_farge5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninører_farge5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninører_farge5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninører_farge5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"person_med_slør", "1F470"},
    {"person_med_slør_farge1", "1F470-1F3FB"},
    {"person_med_slør_farge2", "1F470-1F3FC"},
    {"person_med_slør_farge3", "1F470-1F3FD"},
    {"person_med_slør_farge4", "1F470-1F3FE"},
    {"person_med_slør_farge5", "1F470-1F3FF"},
    {"spøkelse", "1F47B"},
    {"hårklipp", "1F487"},
    {"hårklipp_farge1", "1F487-1F3FB"},
    {"hårklipp_farge2", "1F487-1F3FC"},
    {"hårklipp_farge3", "1F487-1F3FD"},
    {"hårklipp_farge4", "1F487-1F3FE"},
    {"hårklipp_farge5", "1F487-1F3FF"},
    {"sprøyte", "1F489"},
    {"kjærlighesbrev", "1F48C"},
    {"blått_hjerte", "1F499"},
    {"grønt_hjerte", "1F49A"},
    {"hjerte_med_bånd", "1F49D"},
    {"lyspære", "1F4A1"},
    {"svettedråper", "1F4A6"},
    {"dråpe", "1F4A7"},
    {"bæsj", "1F4A9"},
    {"bærbar_datamaskin", "1F4BB"},
    {"åpen_arkivmappe", "1F4C2"},
    {"side_med_bøy", "1F4C3"},
    {"åpen_bok", "1F4D6"},
    {"grønn_bok", "1F4D7"},
    {"blå_bok", "1F4D8"},
    {"bøker", "1F4DA"},
    {"telefonrør", "1F4DE"},
    {"personsøker", "1F4DF"},
    {"høyttaler", "1F4E2"},
    {"åpen_postkasse_med_flagget_oppe", "1F4EC"},
    {"åpen_postkasse_med_flagget_nede", "1F4ED"},
    {"bønnekrans", "1F4FF"},
    {"dempet_høyttaler", "1F507"},
    {"høyttaler_med_lavt_volum", "1F508"},
    {"høyttaler_med_middels_volum", "1F509"},
    {"høyttaler_med_høyt_volum", "1F50A"},
    {"støpsel", "1F50C"},
    {"forstørrelsesglass_mot_venstre", "1F50D"},
    {"forstørrelsesglass_mot_høyre", "1F50E"},
    {"låst_med_penn", "1F50F"},
    {"låst_med_nøkkel", "1F510"},
    {"nøkkel", "1F511"},
    {"låst", "1F512"},
    {"ulåst", "1F513"},
    {"aldersgrense_18_år", "1F51E"},
    {"små_bokstaver", "1F521"},
    {"fastnøkkel", "1F527"},
    {"rød_sirkel", "1F534"},
    {"blå_sirkel", "1F535"},
    {"stort_blått_rutersymbol", "1F537"},
    {"lite_blått_rutersymbol", "1F539"},
    {"rød_trekant_som_peker_opp", "1F53A"},
    {"rød_trekant_som_peker_ned", "1F53B"},
    {"moské", "1F54C"},
    {"klokken_åtte", "1F557"},
    {"klokken_halv_åtte", "1F562"},
    {"flat_hånd_med_spredte_fingre", "1F590"},
    {"flat_hånd_med_spredte_fingre_farge1", "1F590-1F3FB"},
    {"flat_hånd_med_spredte_fingre_farge2", "1F590-1F3FC"},
    {"flat_hånd_med_spredte_fingre_farge3", "1F590-1F3FD"},
    {"flat_hånd_med_spredte_fingre_farge4", "1F590-1F3FE"},
    {"flat_hånd_med_spredte_fingre_farge5", "1F590-1F3FF"},
    {"stasjonær_datamaskin", "1F5A5"},
    {"gammeldags_nøkkel", "1F5DD"},
    {"sint_snakkeboble_mot_høyre", "1F5EF"},
    {"tokyo_tårnet", "1F5FC"},
    {"glisefjes_med_smilende_øyne", "1F601"},
    {"gledestårer", "1F602"},
    {"glisende_fjes_med_vidåpne_øyne", "1F603"},
    {"glisende_fjes_med_smilende_øyne", "1F604"},
    {"smil_med_sammenknepne_øyne", "1F606"},
    {"smil_med_smilende_øyne", "1F60A"},
    {"smilefjes_med_hjerteøyne", "1F60D"},
    {"nøytral", "1F610"},
    {"uttrykksløs", "1F611"},
    {"misfornøyd", "1F612"},
    {"kyss_med_smilende_øyne", "1F619"},
    {"kyss_med_lukkede_øyne", "1F61A"},
    {"æsj", "1F61D"},
    {"gråter", "1F622"},
    {"søvnig", "1F62A"},
    {"trøtt_fjes", "1F62B"},
    {"strigråter", "1F62D"},
    {"rødmer", "1F633"},
    {"ør", "1F635"},
    {"flirende_katt_med_smilende_øyne", "1F638"},
    {"katt_med_gledestårer", "1F639"},
    {"smilende_katt_med_åpen_munn", "1F63A"},
    {"smilende_katt_med_hjerteøyne", "1F63B"},
    {"gråtende_katt", "1F63F"},
    {"himler_med_øynene", "1F644"},
    {"hører_ingenting_ondt", "1F649"},
    {"rekker_opp_hånden", "1F64B"},
    {"rekker_opp_hånden_farge1", "1F64B-1F3FB"},
    {"rekker_opp_hånden_farge2", "1F64B-1F3FC"},
    {"rekker_opp_hånden_farge3", "1F64B-1F3FD"},
    {"rekker_opp_hånden_farge4", "1F64B-1F3FE"},
    {"rekker_opp_hånden_farge5", "1F64B-1F3FF"},
    {"høyhastighetstog", "1F684"},
    {"høyhastighetstog_med_kulenese", "1F685"},
    {"motgående_buss", "1F68D"},
    {"motgående_politibil", "1F694"},
    {"motgående_taxi", "1F696"},
    {"motgående_bil", "1F698"},
    {"person_i_robåt", "1F6A3"},
    {"person_i_robåt_farge1", "1F6A3-1F3FB"},
    {"person_i_robåt_farge2", "1F6A3-1F3FC"},
    {"person_i_robåt_farge3", "1F6A3-1F3FD"},
    {"person_i_robåt_farge4", "1F6A3-1F3FE"},
    {"person_i_robåt_farge5", "1F6A3-1F3FF"},
    {"speedbåt", "1F6A4"},
    {"dør", "1F6AA"},
    {"røyking_forbudt", "1F6AD"},
    {"forsøpling_ikke_tillatt", "1F6AF"},
    {"bagasjebånd", "1F6C4"},
    {"religiøst_samlingssted", "1F6D0"},
    {"trådløst", "1F6DC"},
    {"livbøye", "1F6DF"},
    {"hammer_og_fastnøkkel", "1F6E0"},
    {"motorbåt", "1F6E5"},
    {"rulleskøyter", "1F6FC"},
    {"grønn_sirkel", "1F7E2"},
    {"rødt_kvadrat", "1F7E5"},
    {"blått_kvadrat", "1F7E6"},
    {"grønt_kvadrat", "1F7E9"},
    {"hånd_med_fingertuppene_sammen", "1F90C"},
    {"hånd_med_fingertuppene_sammen_farge1", "1F90C-1F3FB"},
    {"hånd_med_fingertuppene_sammen_farge2", "1F90C-1F3FC"},
    {"hånd_med_fingertuppene_sammen_farge3", "1F90C-1F3FD"},
    {"hånd_med_fingertuppene_sammen_farge4", "1F90C-1F3FE"},
    {"hånd_med_fingertuppene_sammen_farge5", "1F90C-1F3FF"},
    {"hånd_som_klyper", "1F90F"},
    {"hånd_som_klyper_farge1", "1F90F-1F3FB"},
    {"hånd_som_klyper_farge2", "1F90F-1F3FC"},
    {"hånd_som_klyper_farge3", "1F90F-1F3FD"},
    {"hånd_som_klyper_farge4", "1F90F-1F3FE"},
    {"hånd_som_klyper_farge5", "1F90F-1F3FF"},
    {"fjes_med_glidelåsmunn", "1F910"},
    {"ring_meg_hånd", "1F919"},
    {"ring_meg_hånd_farge1", "1F919-1F3FB"},
    {"ring_meg_hånd_farge2", "1F919-1F3FC"},
    {"ring_meg_hånd_farge3", "1F919-1F3FD"},
    {"ring_meg_hånd_farge4", "1F919-1F3FE"},
    {"ring_meg_hånd_farge5", "1F919-1F3FF"},
    {"løftet_håndbak", "1F91A"},
    {"løftet_håndbak_farge1", "1F91A-1F3FB"},
    {"løftet_håndbak_farge2", "1F91A-1F3FC"},
    {"løftet_håndbak_farge3", "1F91A-1F3FD"},
    {"løftet_håndbak_farge4", "1F91A-1F3FE"},
    {"løftet_håndbak_farge5", "1F91A-1F3FF"},
    {"knyttneve_mot_høyre", "1F91C"},
    {"knyttneve_mot_høyre_farge1", "1F91C-1F3FB"},
    {"knyttneve_mot_høyre_farge2", "1F91C-1F3FC"},
    {"knyttneve_mot_høyre_farge3", "1F91C-1F3FD"},
    {"knyttneve_mot_høyre_farge4", "1F91C-1F3FE"},
    {"knyttneve_mot_høyre_farge5", "1F91C-1F3FF"},
    {"håndhilse", "1F91D"},
    {"håndhilse_farge1", "1F91D-1F3FB"},
    {"håndhilse_farge2", "1F91D-1F3FC"},
    {"håndhilse_farge3", "1F91D-1F3FD"},
    {"håndhilse_farge4", "1F91D-1F3FE"},
    {"håndhilse_farge5", "1F91D-1F3FF"},
    {"håndhilse_farge1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"håndhilse_farge1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"håndhilse_farge1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"håndhilse_farge1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"håndhilse_farge2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"håndhilse_farge2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"håndhilse_farge2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"håndhilse_farge2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"håndhilse_farge3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"håndhilse_farge3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"håndhilse_farge3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"håndhilse_farge3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"håndhilse_farge4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"håndhilse_farge4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"håndhilse_farge4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"håndhilse_farge4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"håndhilse_farge5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"håndhilse_farge5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"håndhilse_farge5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"håndhilse_farge5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"ruller_rundt_på_gulvet_i_latter", "1F923"},
    {"slår_hånden_mot_pannen", "1F926"},
    {"slår_hånden_mot_pannen_farge1", "1F926-1F3FB"},
    {"slår_hånden_mot_pannen_farge2", "1F926-1F3FC"},
    {"slår_hånden_mot_pannen_farge3", "1F926-1F3FD"},
    {"slår_hånden_mot_pannen_farge4", "1F926-1F3FE"},
    {"slår_hånden_mot_pannen_farge5", "1F926-1F3FF"},
    {"hevet_øyenbryn", "1F928"},
    {"stjerner_i_øynene", "1F929"},
    {"hånden_over_munnen", "1F92D"},
    {"håndflatene_sammen", "1F932"},
    {"håndflatene_sammen_farge1", "1F932-1F3FB"},
    {"håndflatene_sammen_farge2", "1F932-1F3FC"},
    {"håndflatene_sammen_farge3", "1F932-1F3FD"},
    {"håndflatene_sammen_farge4", "1F932-1F3FE"},
    {"håndflatene_sammen_farge5", "1F932-1F3FF"},
    {"trekker_på_skuldrene", "1F937"},
    {"trekker_på_skuldrene_farge1", "1F937-1F3FB"},
    {"trekker_på_skuldrene_farge2", "1F937-1F3FC"},
    {"trekker_på_skuldrene_farge3", "1F937-1F3FD"},
    {"trekker_på_skuldrene_farge4", "1F937-1F3FE"},
    {"trekker_på_skuldrene_farge5", "1F937-1F3FF"},
    {"slå_hjul", "1F938"},
    {"slå_hjul_farge1", "1F938-1F3FB"},
    {"slå_hjul_farge2", "1F938-1F3FC"},
    {"slå_hjul_farge3", "1F938-1F3FD"},
    {"slå_hjul_farge4", "1F938-1F3FE"},
    {"slå_hjul_farge5", "1F938-1F3FF"},
    {"håndball", "1F93E"},
    {"håndball_farge1", "1F93E-1F3FB"},
    {"håndball_farge2", "1F93E-1F3FC"},
    {"håndball_farge3", "1F93E-1F3FD"},
    {"håndball_farge4", "1F93E-1F3FE"},
    {"håndball_farge5", "1F93E-1F3FF"},
    {"glass_som_skåler", "1F942"},
    {"målnett", "1F945"},
    {"sølvmedalje", "1F948"},
    {"grønn_salat", "1F957"},
    {"fylt_pitabrød", "1F959"},
    {"peanøtter", "1F95C"},
    {"kopp_med_sugerør", "1F964"},
    {"kokosnøtt", "1F965"},
    {"kjøttstykke", "1F969"},
    {"bladgrønnsaker", "1F96C"},
    {"månekake", "1F96E"},
    {"smilefjes_med_tåre", "1F972"},
    {"fjes_som_holder_tilbake_tårene", "1F979"},
    {"turstøvel", "1F97E"},
    {"løve", "1F981"},
    {"enhjørning", "1F984"},
    {"ørn", "1F985"},
    {"påfugl", "1F99A"},
    {"papegøye", "1F99C"},
    {"vaskebjørn", "1F99D"},
    {"østers", "1F9AA"},
    {"førerhund", "1F9AE"},
    {"rødt_hår", "1F9B0"},
    {"kruset_hår", "1F9B1"},
    {"grått_hår", "1F9B3"},
    {"øre_med_høreapparat", "1F9BB"},
    {"øre_med_høreapparat_farge1", "1F9BB-1F3FB"},
    {"øre_med_høreapparat_farge2", "1F9BB-1F3FC"},
    {"øre_med_høreapparat_farge3", "1F9BB-1F3FD"},
    {"øre_med_høreapparat_farge4", "1F9BB-1F3FE"},
    {"øre_med_høreapparat_farge5", "1F9BB-1F3FF"},
    {"hvitløk", "1F9C4"},
    {"løk", "1F9C5"},
    {"smør", "1F9C8"},
    {"person_som_står", "1F9CD"},
    {"person_som_står_farge1", "1F9CD-1F3FB"},
    {"person_som_står_farge2", "1F9CD-1F3FC"},
    {"person_som_står_farge3", "1F9CD-1F3FD"},
    {"person_som_står_farge4", "1F9CD-1F3FE"},
    {"person_som_står_farge5", "1F9CD-1F3FF"},
    {"døv_person", "1F9CF"},
    {"døv_person_farge1", "1F9CF-1F3FB"},
    {"døv_person_farge2", "1F9CF-1F3FC"},
    {"døv_person_farge3", "1F9CF-1F3FD"},
    {"døv_person_farge4", "1F9CF-1F3FE"},
    {"døv_person_farge5", "1F9CF-1F3FF"},
    {"kvinne_med_hodeslør", "1F9D5"},
    {"kvinne_med_hodeslør_farge1", "1F9D5-1F3FB"},
    {"kvinne_med_hodeslør_farge2", "1F9D5-1F3FC"},
    {"kvinne_med_hodeslør_farge3", "1F9D5-1F3FD"},
    {"kvinne_med_hodeslør_farge4", "1F9D5-1F3FE"},
    {"kvinne_med_hodeslør_farge5", "1F9D5-1F3FF"},
    {"ånd", "1F9DE"},
    {"rød_konvolutt", "1F9E7"},
    {"prøverør", "1F9EA"},
    {"petriskål", "1F9EB"},
    {"verktøykasse", "1F9F0"},
    {"tråd", "1F9F5"},
    {"sikkerhetsnål", "1F9F7"},
    {"teddybjørn", "1F9F8"},
    {"såpe", "1F9FC"},
    {"lyseblått_hjerte", "1FA75"},
    {"grått_hjerte", "1FA76"},
    {"bloddråpe", "1FA78"},
    {"røntgen", "1FA7B"},
    {"piñata", "1FA85"},
    {"fløyte", "1FA88"},
    {"øks", "1FA93"},
    {"åpen_oljelampe", "1FA94"},
    {"militærhjelm", "1FA96"},
    {"håndsag", "1FA9A"},
    {"avløpsåpner", "1FAA0"},
    {"synål", "1FAA1"},
    {"bøtte", "1FAA3"},
    {"tannbørste", "1FAA5"},
    {"håndvifte", "1FAAD"},
    {"fjær", "1FAB6"},
    {"bladløst_tre", "1FABE"},
    {"gås", "1FABF"},
    {"hårete_vesen", "1FAC8"},
    {"blåbær", "1FAD0"},
    {"pitabrød", "1FAD3"},
    {"helle_væske", "1FAD7"},
    {"bønner", "1FAD8"},
    {"ingefærrot", "1FADA"},
    {"rotgrønnsak", "1FADC"},
    {"fjes_som_viser_honnør", "1FAE1"},
    {"fjes_med_åpne_øyne_og_hånd_over_munnen", "1FAE2"},
    {"fjes_med_poser_under_øynene", "1FAE9"},
    {"slåsskampsky", "1FAEF"},
    {"hånd_med_pekefinger_og_tommel_krysset", "1FAF0"},
    {"hånd_med_pekefinger_og_tommel_krysset_farge1", "1FAF0-1F3FB"},
    {"hånd_med_pekefinger_og_tommel_krysset_farge2", "1FAF0-1F3FC"},
    {"hånd_med_pekefinger_og_tommel_krysset_farge3", "1FAF0-1F3FD"},
    {"hånd_med_pekefinger_og_tommel_krysset_farge4", "1FAF0-1F3FE"},
    {"hånd_med_pekefinger_og_tommel_krysset_farge5", "1FAF0-1F3FF"},
    {"hånd_mot_høyre", "1FAF1"},
    {"hånd_mot_høyre_farge1", "1FAF1-1F3FB"},
    {"hånd_mot_høyre_farge2", "1FAF1-1F3FC"},
    {"hånd_mot_høyre_farge3", "1FAF1-1F3FD"},
    {"hånd_mot_høyre_farge4", "1FAF1-1F3FE"},
    {"hånd_mot_høyre_farge5", "1FAF1-1F3FF"},
    {"hånd_mot_venstre", "1FAF2"},
    {"hånd_mot_venstre_farge1", "1FAF2-1F3FB"},
    {"hånd_mot_venstre_farge2", "1FAF2-1F3FC"},
    {"hånd_mot_venstre_farge3", "1FAF2-1F3FD"},
    {"hånd_mot_venstre_farge4", "1FAF2-1F3FE"},
    {"hånd_mot_venstre_farge5", "1FAF2-1F3FF"},
    {"håndflaten_ned", "1FAF3"},
    {"håndflaten_ned_farge1", "1FAF3-1F3FB"},
    {"håndflaten_ned_farge2", "1FAF3-1F3FC"},
    {"håndflaten_ned_farge3", "1FAF3-1F3FD"},
    {"håndflaten_ned_farge4", "1FAF3-1F3FE"},
    {"håndflaten_ned_farge5", "1FAF3-1F3FF"},
    {"håndflaten_opp", "1FAF4"},
    {"håndflaten_opp_farge1", "1FAF4-1F3FB"},
    {"håndflaten_opp_farge2", "1FAF4-1F3FC"},
    {"håndflaten_opp_farge3", "1FAF4-1F3FD"},
    {"håndflaten_opp_farge4", "1FAF4-1F3FE"},
    {"håndflaten_opp_farge5", "1FAF4-1F3FF"},
    {"pekefinger_peker_på_leseren", "1FAF5"},
    {"pekefinger_peker_på_leseren_farge1", "1FAF5-1F3FB"},
    {"pekefinger_peker_på_leseren_farge2", "1FAF5-1F3FC"},
    {"pekefinger_peker_på_leseren_farge3", "1FAF5-1F3FD"},
    {"pekefinger_peker_på_leseren_farge4", "1FAF5-1F3FE"},
    {"pekefinger_peker_på_leseren_farge5", "1FAF5-1F3FF"},
    {"håndflate_hevet_mot_venstre", "1FAF7"},
    {"håndflate_hevet_mot_venstre_farge1", "1FAF7-1F3FB"},
    {"håndflate_hevet_mot_venstre_farge2", "1FAF7-1F3FC"},
    {"håndflate_hevet_mot_venstre_farge3", "1FAF7-1F3FD"},
    {"håndflate_hevet_mot_venstre_farge4", "1FAF7-1F3FE"},
    {"håndflate_hevet_mot_venstre_farge5", "1FAF7-1F3FF"},
    {"håndflate_hevet_mot_høyre", "1FAF8"},
    {"håndflate_hevet_mot_høyre_farge1", "1FAF8-1F3FB"},
    {"håndflate_hevet_mot_høyre_farge2", "1FAF8-1F3FC"},
    {"håndflate_hevet_mot_høyre_farge3", "1FAF8-1F3FD"},
    {"håndflate_hevet_mot_høyre_farge4", "1FAF8-1F3FE"},
    {"håndflate_hevet_mot_høyre_farge5", "1FAF8-1F3FF"},
    {"flagg_østerrike", "1F1E6-1F1F9"},
    {"flagg_åland", "1F1E6-1F1FD"},
    {"flagg_saint_barthélemy", "1F1E7-1F1F1"},
    {"flagg_bouvetøya", "1F1E7-1F1FB"},
    {"flagg_kokosøyene", "1F1E8-1F1E8"},
    {"flagg_cookøyene", "1F1E8-1F1F0"},
    {"flagg_clippertonøya", "1F1E8-1F1F5"},
    {"flagg_curaçao", "1F1E8-1F1FC"},
    {"flagg_christmasøya", "1F1E8-1F1FD"},
    {"flagg_falklandsøyene", "1F1EB-1F1F0"},
    {"flagg_mikronesiaføderasjonen", "1F1EB-1F1F2"},
    {"flagg_færøyene", "1F1EB-1F1F4"},
    {"flagg_grønland", "1F1EC-1F1F1"},
    {"flagg_sør_georgia_og_sør_sandwichøyene", "1F1EC-1F1F8"},
    {"flagg_heard_og_mcdonaldøyene", "1F1ED-1F1F2"},
    {"flagg_kanariøyene", "1F1EE-1F1E8"},
    {"flagg_sør_korea", "1F1F0-1F1F7"},
    {"flagg_caymanøyene", "1F1F0-1F1FE"},
    {"flagg_marshalløyene", "1F1F2-1F1ED"},
    {"flagg_norfolkøya", "1F1F3-1F1EB"},
    {"flagg_pitcairnøyene", "1F1F5-1F1F3"},
    {"flagg_det_palestinske_området", "1F1F5-1F1F8"},
    {"flagg_réunion", "1F1F7-1F1EA"},
    {"flagg_salomonøyene", "1F1F8-1F1E7"},
    {"flagg_sør_sudan", "1F1F8-1F1F8"},
    {"flagg_são_tomé_og_príncipe", "1F1F8-1F1F9"},
    {"flagg_turks_og_caicosøyene", "1F1F9-1F1E8"},
    {"flagg_de_franske_sørterritorier", "1F1F9-1F1EB"},
    {"flagg_øst_timor", "1F1F9-1F1F1"},
    {"flagg_usas_ytre_øyer", "1F1FA-1F1F2"},
    {"flagg_de_britiske_jomfruøyene", "1F1FB-1F1EC"},
    {"flagg_de_amerikanske_jomfruøyene", "1F1FB-1F1EE"},
    {"flagg_sør_afrika", "1F1FF-1F1E6"},
    {"løper_vendt_mot_høyre", "1F3C3-200D-27A1-FE0F"},
    {"løper_vendt_mot_høyre_farge1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"løper_vendt_mot_høyre_farge2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"løper_vendt_mot_høyre_farge3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"løper_vendt_mot_høyre_farge4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"løper_vendt_mot_høyre_farge5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"mannlig_gårdbruker", "1F468-200D-1F33E"},
    {"mannlig_gårdbruker_farge1", "1F468-1F3FB-200D-1F33E"},
    {"mannlig_gårdbruker_farge2", "1F468-1F3FC-200D-1F33E"},
    {"mannlig_gårdbruker_farge3", "1F468-1F3FD-200D-1F33E"},
    {"mannlig_gårdbruker_farge4", "1F468-1F3FE-200D-1F33E"},
    {"mannlig_gårdbruker_farge5", "1F468-1F3FF-200D-1F33E"},
    {"mannlig_lærer", "1F468-200D-1F3EB"},
    {"mannlig_lærer_farge1", "1F468-1F3FB-200D-1F3EB"},
    {"mannlig_lærer_farge2", "1F468-1F3FC-200D-1F3EB"},
    {"mannlig_lærer_farge3", "1F468-1F3FD-200D-1F3EB"},
    {"mannlig_lærer_farge4", "1F468-1F3FE-200D-1F3EB"},
    {"mannlig_lærer_farge5", "1F468-1F3FF-200D-1F3EB"},
    {"mann_med_blindestokk_vendt_mot_høyre", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_med_blindestokk_vendt_mot_høyre_farge1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_med_blindestokk_vendt_mot_høyre_farge2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_med_blindestokk_vendt_mot_høyre_farge3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_med_blindestokk_vendt_mot_høyre_farge4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_med_blindestokk_vendt_mot_høyre_farge5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_i_motorisert_rullestol_vendt_mot_høyre", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_i_motorisert_rullestol_vendt_mot_høyre_farge1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_i_motorisert_rullestol_vendt_mot_høyre_farge2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_i_motorisert_rullestol_vendt_mot_høyre_farge3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_i_motorisert_rullestol_vendt_mot_høyre_farge4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_i_motorisert_rullestol_vendt_mot_høyre_farge5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_i_manuell_rullestol_vendt_mot_høyre", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_i_manuell_rullestol_vendt_mot_høyre_farge1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_i_manuell_rullestol_vendt_mot_høyre_farge2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_i_manuell_rullestol_vendt_mot_høyre_farge3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_i_manuell_rullestol_vendt_mot_høyre_farge4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_i_manuell_rullestol_vendt_mot_høyre_farge5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinnelig_gårdbruker", "1F469-200D-1F33E"},
    {"kvinnelig_gårdbruker_farge1", "1F469-1F3FB-200D-1F33E"},
    {"kvinnelig_gårdbruker_farge2", "1F469-1F3FC-200D-1F33E"},
    {"kvinnelig_gårdbruker_farge3", "1F469-1F3FD-200D-1F33E"},
    {"kvinnelig_gårdbruker_farge4", "1F469-1F3FE-200D-1F33E"},
    {"kvinnelig_gårdbruker_farge5", "1F469-1F3FF-200D-1F33E"},
    {"kvinnelig_lærer", "1F469-200D-1F3EB"},
    {"kvinnelig_lærer_farge1", "1F469-1F3FB-200D-1F3EB"},
    {"kvinnelig_lærer_farge2", "1F469-1F3FC-200D-1F3EB"},
    {"kvinnelig_lærer_farge3", "1F469-1F3FD-200D-1F3EB"},
    {"kvinnelig_lærer_farge4", "1F469-1F3FE-200D-1F3EB"},
    {"kvinnelig_lærer_farge5", "1F469-1F3FF-200D-1F3EB"},
    {"kvinne_med_blindestokk_vendt_mot_høyre", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinne_med_blindestokk_vendt_mot_høyre_farge1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinne_med_blindestokk_vendt_mot_høyre_farge2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinne_med_blindestokk_vendt_mot_høyre_farge3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinne_med_blindestokk_vendt_mot_høyre_farge4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinne_med_blindestokk_vendt_mot_høyre_farge5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinne_i_motorisert_rullestol_vendt_mot_høyre", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinne_i_motorisert_rullestol_vendt_mot_høyre_farge1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinne_i_motorisert_rullestol_vendt_mot_høyre_farge2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinne_i_motorisert_rullestol_vendt_mot_høyre_farge3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinne_i_motorisert_rullestol_vendt_mot_høyre_farge4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinne_i_motorisert_rullestol_vendt_mot_høyre_farge5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinne_i_manuell_rullestol_vendt_mot_høyre", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinne_i_manuell_rullestol_vendt_mot_høyre_farge1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinne_i_manuell_rullestol_vendt_mot_høyre_farge2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinne_i_manuell_rullestol_vendt_mot_høyre_farge3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinne_i_manuell_rullestol_vendt_mot_høyre_farge4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinne_i_manuell_rullestol_vendt_mot_høyre_farge5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"fotgjenger_vendt_mot_høyre", "1F6B6-200D-27A1-FE0F"},
    {"fotgjenger_vendt_mot_høyre_farge1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"fotgjenger_vendt_mot_høyre_farge2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"fotgjenger_vendt_mot_høyre_farge3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"fotgjenger_vendt_mot_høyre_farge4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"fotgjenger_vendt_mot_høyre_farge5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"person_som_kneler_vendt_mot_høyre", "1F9CE-200D-27A1-FE0F"},
    {"person_som_kneler_vendt_mot_høyre_farge1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"person_som_kneler_vendt_mot_høyre_farge2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"person_som_kneler_vendt_mot_høyre_farge3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"person_som_kneler_vendt_mot_høyre_farge4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"person_som_kneler_vendt_mot_høyre_farge5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"gårdbruker", "1F9D1-200D-1F33E"},
    {"gårdbruker_farge1", "1F9D1-1F3FB-200D-1F33E"},
    {"gårdbruker_farge2", "1F9D1-1F3FC-200D-1F33E"},
    {"gårdbruker_farge3", "1F9D1-1F3FD-200D-1F33E"},
    {"gårdbruker_farge4", "1F9D1-1F3FE-200D-1F33E"},
    {"gårdbruker_farge5", "1F9D1-1F3FF-200D-1F33E"},
    {"lærer", "1F9D1-200D-1F3EB"},
    {"lærer_farge1", "1F9D1-1F3FB-200D-1F3EB"},
    {"lærer_farge2", "1F9D1-1F3FC-200D-1F3EB"},
    {"lærer_farge3", "1F9D1-1F3FD-200D-1F3EB"},
    {"lærer_farge4", "1F9D1-1F3FE-200D-1F3EB"},
    {"lærer_farge5", "1F9D1-1F3FF-200D-1F3EB"},
    {"person_med_blindestokk_vendt_mot_høyre", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestokk_vendt_mot_høyre_farge1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestokk_vendt_mot_høyre_farge2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestokk_vendt_mot_høyre_farge3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestokk_vendt_mot_høyre_farge4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestokk_vendt_mot_høyre_farge5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"person_i_motorisert_rullestol_vendt_mot_høyre", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_motorisert_rullestol_vendt_mot_høyre_farge1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_motorisert_rullestol_vendt_mot_høyre_farge2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_motorisert_rullestol_vendt_mot_høyre_farge3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_motorisert_rullestol_vendt_mot_høyre_farge4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_motorisert_rullestol_vendt_mot_høyre_farge5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_manuell_rullestol_vendt_mot_høyre", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullestol_vendt_mot_høyre_farge1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullestol_vendt_mot_høyre_farge2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullestol_vendt_mot_høyre_farge3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullestol_vendt_mot_høyre_farge4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullestol_vendt_mot_høyre_farge5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinnelig_løper", "1F3C3-200D-2640-FE0F"},
    {"kvinnelig_løper_farge1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_løper_farge2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_løper_farge3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_løper_farge4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_løper_farge5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"kvinnelig_løper_vendt_mot_høyre", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_løper_vendt_mot_høyre_farge1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_løper_vendt_mot_høyre_farge2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_løper_vendt_mot_høyre_farge3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_løper_vendt_mot_høyre_farge4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_løper_vendt_mot_høyre_farge5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mannlig_løper", "1F3C3-200D-2642-FE0F"},
    {"mannlig_løper_farge1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"mannlig_løper_farge2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"mannlig_løper_farge3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"mannlig_løper_farge4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"mannlig_løper_farge5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"mannlig_løper_vendt_mot_høyre", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_løper_vendt_mot_høyre_farge1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_løper_vendt_mot_høyre_farge2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_løper_vendt_mot_høyre_farge3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_løper_vendt_mot_høyre_farge4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_løper_vendt_mot_høyre_farge5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_svømmer", "1F3CA-200D-2640-FE0F"},
    {"kvinnelig_svømmer_farge1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_svømmer_farge2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_svømmer_farge3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_svømmer_farge4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_svømmer_farge5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"mannlig_svømmer", "1F3CA-200D-2642-FE0F"},
    {"mannlig_svømmer_farge1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"mannlig_svømmer_farge2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"mannlig_svømmer_farge3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"mannlig_svømmer_farge4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"mannlig_svømmer_farge5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_vektløfter", "1F3CB-FE0F-200D-2640-FE0F"},
    {"kvinnelig_vektløfter_farge1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_vektløfter_farge2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_vektløfter_farge3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_vektløfter_farge4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_vektløfter_farge5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"mannlig_vektløfter", "1F3CB-FE0F-200D-2642-FE0F"},
    {"mannlig_vektløfter_farge1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"mannlig_vektløfter_farge2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"mannlig_vektløfter_farge3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"mannlig_vektløfter_farge4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"mannlig_vektløfter_farge5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"kvinner_med_kaninører", "1F46F-200D-2640-FE0F"},
    {"kvinner_med_kaninører_farge1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"kvinner_med_kaninører_farge1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"kvinner_med_kaninører_farge1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"kvinner_med_kaninører_farge1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"kvinner_med_kaninører_farge2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"kvinner_med_kaninører_farge2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"kvinner_med_kaninører_farge2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"kvinner_med_kaninører_farge2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"kvinner_med_kaninører_farge3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"kvinner_med_kaninører_farge3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"kvinner_med_kaninører_farge3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"kvinner_med_kaninører_farge3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"kvinner_med_kaninører_farge4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"kvinner_med_kaninører_farge4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"kvinner_med_kaninører_farge4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"kvinner_med_kaninører_farge4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"kvinner_med_kaninører_farge5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"kvinner_med_kaninører_farge5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"kvinner_med_kaninører_farge5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"kvinner_med_kaninører_farge5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"kvinner_med_kaninører_farge1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"kvinner_med_kaninører_farge2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"kvinner_med_kaninører_farge3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"kvinner_med_kaninører_farge4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"kvinner_med_kaninører_farge5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"menn_med_kaninører", "1F46F-200D-2642-FE0F"},
    {"menn_med_kaninører_farge1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"menn_med_kaninører_farge1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"menn_med_kaninører_farge1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"menn_med_kaninører_farge1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"menn_med_kaninører_farge2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"menn_med_kaninører_farge2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"menn_med_kaninører_farge2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"menn_med_kaninører_farge2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"menn_med_kaninører_farge3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"menn_med_kaninører_farge3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"menn_med_kaninører_farge3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"menn_med_kaninører_farge3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"menn_med_kaninører_farge4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"menn_med_kaninører_farge4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"menn_med_kaninører_farge4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"menn_med_kaninører_farge4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"menn_med_kaninører_farge5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"menn_med_kaninører_farge5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"menn_med_kaninører_farge5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"menn_med_kaninører_farge5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"menn_med_kaninører_farge1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"menn_med_kaninører_farge2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"menn_med_kaninører_farge3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"menn_med_kaninører_farge4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"menn_med_kaninører_farge5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"kvinne_med_slør", "1F470-200D-2640-FE0F"},
    {"kvinne_med_slør_farge1", "1F470-1F3FB-200D-2640-FE0F"},
    {"kvinne_med_slør_farge2", "1F470-1F3FC-200D-2640-FE0F"},
    {"kvinne_med_slør_farge3", "1F470-1F3FD-200D-2640-FE0F"},
    {"kvinne_med_slør_farge4", "1F470-1F3FE-200D-2640-FE0F"},
    {"kvinne_med_slør_farge5", "1F470-1F3FF-200D-2640-FE0F"},
    {"mann_med_slør", "1F470-200D-2642-FE0F"},
    {"mann_med_slør_farge1", "1F470-1F3FB-200D-2642-FE0F"},
    {"mann_med_slør_farge2", "1F470-1F3FC-200D-2642-FE0F"},
    {"mann_med_slør_farge3", "1F470-1F3FD-200D-2642-FE0F"},
    {"mann_med_slør_farge4", "1F470-1F3FE-200D-2642-FE0F"},
    {"mann_med_slør_farge5", "1F470-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_får_ansiktsmassasje", "1F486-200D-2640-FE0F"},
    {"kvinne_som_får_ansiktsmassasje_farge1", "1F486-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_får_ansiktsmassasje_farge2", "1F486-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_får_ansiktsmassasje_farge3", "1F486-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_får_ansiktsmassasje_farge4", "1F486-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_får_ansiktsmassasje_farge5", "1F486-1F3FF-200D-2640-FE0F"},
    {"mann_som_får_ansiktsmassasje", "1F486-200D-2642-FE0F"},
    {"mann_som_får_ansiktsmassasje_farge1", "1F486-1F3FB-200D-2642-FE0F"},
    {"mann_som_får_ansiktsmassasje_farge2", "1F486-1F3FC-200D-2642-FE0F"},
    {"mann_som_får_ansiktsmassasje_farge3", "1F486-1F3FD-200D-2642-FE0F"},
    {"mann_som_får_ansiktsmassasje_farge4", "1F486-1F3FE-200D-2642-FE0F"},
    {"mann_som_får_ansiktsmassasje_farge5", "1F486-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_får_hårklipp", "1F487-200D-2640-FE0F"},
    {"kvinne_som_får_hårklipp_farge1", "1F487-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_får_hårklipp_farge2", "1F487-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_får_hårklipp_farge3", "1F487-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_får_hårklipp_farge4", "1F487-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_får_hårklipp_farge5", "1F487-1F3FF-200D-2640-FE0F"},
    {"mann_som_får_hårklipp", "1F487-200D-2642-FE0F"},
    {"mann_som_får_hårklipp_farge1", "1F487-1F3FB-200D-2642-FE0F"},
    {"mann_som_får_hårklipp_farge2", "1F487-1F3FC-200D-2642-FE0F"},
    {"mann_som_får_hårklipp_farge3", "1F487-1F3FD-200D-2642-FE0F"},
    {"mann_som_får_hårklipp_farge4", "1F487-1F3FE-200D-2642-FE0F"},
    {"mann_som_får_hårklipp_farge5", "1F487-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_rekker_opp_hånden", "1F64B-200D-2640-FE0F"},
    {"kvinne_som_rekker_opp_hånden_farge1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_rekker_opp_hånden_farge2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_rekker_opp_hånden_farge3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_rekker_opp_hånden_farge4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_rekker_opp_hånden_farge5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"mann_som_rekker_opp_hånden", "1F64B-200D-2642-FE0F"},
    {"mann_som_rekker_opp_hånden_farge1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"mann_som_rekker_opp_hånden_farge2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"mann_som_rekker_opp_hånden_farge3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"mann_som_rekker_opp_hånden_farge4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"mann_som_rekker_opp_hånden_farge5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"kvinne_i_robåt", "1F6A3-200D-2640-FE0F"},
    {"kvinne_i_robåt_farge1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"kvinne_i_robåt_farge2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"kvinne_i_robåt_farge3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"kvinne_i_robåt_farge4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"kvinne_i_robåt_farge5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"mann_i_robåt", "1F6A3-200D-2642-FE0F"},
    {"mann_i_robåt_farge1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"mann_i_robåt_farge2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"mann_i_robåt_farge3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"mann_i_robåt_farge4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"mann_i_robåt_farge5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"kvinne_på_terrengsykkel", "1F6B5-200D-2640-FE0F"},
    {"kvinne_på_terrengsykkel_farge1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"kvinne_på_terrengsykkel_farge2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"kvinne_på_terrengsykkel_farge3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"kvinne_på_terrengsykkel_farge4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"kvinne_på_terrengsykkel_farge5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"mann_på_terrengsykkel", "1F6B5-200D-2642-FE0F"},
    {"mann_på_terrengsykkel_farge1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"mann_på_terrengsykkel_farge2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"mann_på_terrengsykkel_farge3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"mann_på_terrengsykkel_farge4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"mann_på_terrengsykkel_farge5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_fotgjenger_vendt_mot_høyre", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_fotgjenger_vendt_mot_høyre_farge1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_fotgjenger_vendt_mot_høyre_farge2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_fotgjenger_vendt_mot_høyre_farge3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_fotgjenger_vendt_mot_høyre_farge4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinnelig_fotgjenger_vendt_mot_høyre_farge5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mannlig_fotgjenger_vendt_mot_høyre", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_fotgjenger_vendt_mot_høyre_farge1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_fotgjenger_vendt_mot_høyre_farge2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_fotgjenger_vendt_mot_høyre_farge3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_fotgjenger_vendt_mot_høyre_farge4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mannlig_fotgjenger_vendt_mot_høyre_farge5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kvinne_som_slår_hånden_mot_pannen", "1F926-200D-2640-FE0F"},
    {"kvinne_som_slår_hånden_mot_pannen_farge1", "1F926-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_slår_hånden_mot_pannen_farge2", "1F926-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_slår_hånden_mot_pannen_farge3", "1F926-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_slår_hånden_mot_pannen_farge4", "1F926-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_slår_hånden_mot_pannen_farge5", "1F926-1F3FF-200D-2640-FE0F"},
    {"mann_som_slår_hånden_mot_pannen", "1F926-200D-2642-FE0F"},
    {"mann_som_slår_hånden_mot_pannen_farge1", "1F926-1F3FB-200D-2642-FE0F"},
    {"mann_som_slår_hånden_mot_pannen_farge2", "1F926-1F3FC-200D-2642-FE0F"},
    {"mann_som_slår_hånden_mot_pannen_farge3", "1F926-1F3FD-200D-2642-FE0F"},
    {"mann_som_slår_hånden_mot_pannen_farge4", "1F926-1F3FE-200D-2642-FE0F"},
    {"mann_som_slår_hånden_mot_pannen_farge5", "1F926-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_trekker_på_skuldrene", "1F937-200D-2640-FE0F"},
    {"kvinne_som_trekker_på_skuldrene_farge1", "1F937-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_trekker_på_skuldrene_farge2", "1F937-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_trekker_på_skuldrene_farge3", "1F937-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_trekker_på_skuldrene_farge4", "1F937-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_trekker_på_skuldrene_farge5", "1F937-1F3FF-200D-2640-FE0F"},
    {"mann_som_trekker_på_skuldrene", "1F937-200D-2642-FE0F"},
    {"mann_som_trekker_på_skuldrene_farge1", "1F937-1F3FB-200D-2642-FE0F"},
    {"mann_som_trekker_på_skuldrene_farge2", "1F937-1F3FC-200D-2642-FE0F"},
    {"mann_som_trekker_på_skuldrene_farge3", "1F937-1F3FD-200D-2642-FE0F"},
    {"mann_som_trekker_på_skuldrene_farge4", "1F937-1F3FE-200D-2642-FE0F"},
    {"mann_som_trekker_på_skuldrene_farge5", "1F937-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_slår_hjul", "1F938-200D-2640-FE0F"},
    {"kvinne_som_slår_hjul_farge1", "1F938-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_slår_hjul_farge2", "1F938-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_slår_hjul_farge3", "1F938-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_slår_hjul_farge4", "1F938-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_slår_hjul_farge5", "1F938-1F3FF-200D-2640-FE0F"},
    {"mann_som_slår_hjul", "1F938-200D-2642-FE0F"},
    {"mann_som_slår_hjul_farge1", "1F938-1F3FB-200D-2642-FE0F"},
    {"mann_som_slår_hjul_farge2", "1F938-1F3FC-200D-2642-FE0F"},
    {"mann_som_slår_hjul_farge3", "1F938-1F3FD-200D-2642-FE0F"},
    {"mann_som_slår_hjul_farge4", "1F938-1F3FE-200D-2642-FE0F"},
    {"mann_som_slår_hjul_farge5", "1F938-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_sjonglør", "1F939-200D-2640-FE0F"},
    {"kvinnelig_sjonglør_farge1", "1F939-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_sjonglør_farge2", "1F939-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_sjonglør_farge3", "1F939-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_sjonglør_farge4", "1F939-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_sjonglør_farge5", "1F939-1F3FF-200D-2640-FE0F"},
    {"mannlig_sjonglør", "1F939-200D-2642-FE0F"},
    {"mannlig_sjonglør_farge1", "1F939-1F3FB-200D-2642-FE0F"},
    {"mannlig_sjonglør_farge2", "1F939-1F3FC-200D-2642-FE0F"},
    {"mannlig_sjonglør_farge3", "1F939-1F3FD-200D-2642-FE0F"},
    {"mannlig_sjonglør_farge4", "1F939-1F3FE-200D-2642-FE0F"},
    {"mannlig_sjonglør_farge5", "1F939-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_håndballspiller", "1F93E-200D-2640-FE0F"},
    {"kvinnelig_håndballspiller_farge1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"kvinnelig_håndballspiller_farge2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"kvinnelig_håndballspiller_farge3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"kvinnelig_håndballspiller_farge4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"kvinnelig_håndballspiller_farge5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"mannlig_håndballspiller", "1F93E-200D-2642-FE0F"},
    {"mannlig_håndballspiller_farge1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"mannlig_håndballspiller_farge2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"mannlig_håndballspiller_farge3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"mannlig_håndballspiller_farge4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"mannlig_håndballspiller_farge5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_står", "1F9CD-200D-2640-FE0F"},
    {"kvinne_som_står_farge1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"kvinne_som_står_farge2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"kvinne_som_står_farge3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"kvinne_som_står_farge4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"kvinne_som_står_farge5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"mann_som_står", "1F9CD-200D-2642-FE0F"},
    {"mann_som_står_farge1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"mann_som_står_farge2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"mann_som_står_farge3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"mann_som_står_farge4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"mann_som_står_farge5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"kvinne_som_kneler_vendt_mot_høyre", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinne_som_kneler_vendt_mot_høyre_farge1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinne_som_kneler_vendt_mot_høyre_farge2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinne_som_kneler_vendt_mot_høyre_farge3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinne_som_kneler_vendt_mot_høyre_farge4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinne_som_kneler_vendt_mot_høyre_farge5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mann_som_kneler_vendt_mot_høyre", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mann_som_kneler_vendt_mot_høyre_farge1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mann_som_kneler_vendt_mot_høyre_farge2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mann_som_kneler_vendt_mot_høyre_farge3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mann_som_kneler_vendt_mot_høyre_farge4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mann_som_kneler_vendt_mot_høyre_farge5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"døv_kvinne", "1F9CF-200D-2640-FE0F"},
    {"døv_kvinne_farge1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"døv_kvinne_farge2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"døv_kvinne_farge3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"døv_kvinne_farge4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"døv_kvinne_farge5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"døv_mann", "1F9CF-200D-2642-FE0F"},
    {"døv_mann_farge1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"døv_mann_farge2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"døv_mann_farge3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"døv_mann_farge4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"døv_mann_farge5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"kvinnelig_ånd", "1F9DE-200D-2640-FE0F"},
    {"mannlig_ånd", "1F9DE-200D-2642-FE0F"},
    {"mann_rødt_hår", "1F468-200D-1F9B0"},
    {"mann_rødt_hår_farge1", "1F468-1F3FB-200D-1F9B0"},
    {"mann_rødt_hår_farge2", "1F468-1F3FC-200D-1F9B0"},
    {"mann_rødt_hår_farge3", "1F468-1F3FD-200D-1F9B0"},
    {"mann_rødt_hår_farge4", "1F468-1F3FE-200D-1F9B0"},
    {"mann_rødt_hår_farge5", "1F468-1F3FF-200D-1F9B0"},
    {"mann_kruset_hår", "1F468-200D-1F9B1"},
    {"mann_kruset_hår_farge1", "1F468-1F3FB-200D-1F9B1"},
    {"mann_kruset_hår_farge2", "1F468-1F3FC-200D-1F9B1"},
    {"mann_kruset_hår_farge3", "1F468-1F3FD-200D-1F9B1"},
    {"mann_kruset_hår_farge4", "1F468-1F3FE-200D-1F9B1"},
    {"mann_kruset_hår_farge5", "1F468-1F3FF-200D-1F9B1"},
    {"mann_grått_hår", "1F468-200D-1F9B3"},
    {"mann_grått_hår_farge1", "1F468-1F3FB-200D-1F9B3"},
    {"mann_grått_hår_farge2", "1F468-1F3FC-200D-1F9B3"},
    {"mann_grått_hår_farge3", "1F468-1F3FD-200D-1F9B3"},
    {"mann_grått_hår_farge4", "1F468-1F3FE-200D-1F9B3"},
    {"mann_grått_hår_farge5", "1F468-1F3FF-200D-1F9B3"},
    {"kvinne_rødt_hår", "1F469-200D-1F9B0"},
    {"kvinne_rødt_hår_farge1", "1F469-1F3FB-200D-1F9B0"},
    {"kvinne_rødt_hår_farge2", "1F469-1F3FC-200D-1F9B0"},
    {"kvinne_rødt_hår_farge3", "1F469-1F3FD-200D-1F9B0"},
    {"kvinne_rødt_hår_farge4", "1F469-1F3FE-200D-1F9B0"},
    {"kvinne_rødt_hår_farge5", "1F469-1F3FF-200D-1F9B0"},
    {"kvinne_kruset_hår", "1F469-200D-1F9B1"},
    {"kvinne_kruset_hår_farge1", "1F469-1F3FB-200D-1F9B1"},
    {"kvinne_kruset_hår_farge2", "1F469-1F3FC-200D-1F9B1"},
    {"kvinne_kruset_hår_farge3", "1F469-1F3FD-200D-1F9B1"},
    {"kvinne_kruset_hår_farge4", "1F469-1F3FE-200D-1F9B1"},
    {"kvinne_kruset_hår_farge5", "1F469-1F3FF-200D-1F9B1"},
    {"kvinne_grått_hår", "1F469-200D-1F9B3"},
    {"kvinne_grått_hår_farge1", "1F469-1F3FB-200D-1F9B3"},
    {"kvinne_grått_hår_farge2", "1F469-1F3FC-200D-1F9B3"},
    {"kvinne_grått_hår_farge3", "1F469-1F3FD-200D-1F9B3"},
    {"kvinne_grått_hår_farge4", "1F469-1F3FE-200D-1F9B3"},
    {"kvinne_grått_hår_farge5", "1F469-1F3FF-200D-1F9B3"},
    {"voksen_rødt_hår", "1F9D1-200D-1F9B0"},
    {"voksen_rødt_hår_farge1", "1F9D1-1F3FB-200D-1F9B0"},
    {"voksen_rødt_hår_farge2", "1F9D1-1F3FC-200D-1F9B0"},
    {"voksen_rødt_hår_farge3", "1F9D1-1F3FD-200D-1F9B0"},
    {"voksen_rødt_hår_farge4", "1F9D1-1F3FE-200D-1F9B0"},
    {"voksen_rødt_hår_farge5", "1F9D1-1F3FF-200D-1F9B0"},
    {"voksen_kruset_hår", "1F9D1-200D-1F9B1"},
    {"voksen_kruset_hår_farge1", "1F9D1-1F3FB-200D-1F9B1"},
    {"voksen_kruset_hår_farge2", "1F9D1-1F3FC-200D-1F9B1"},
    {"voksen_kruset_hår_farge3", "1F9D1-1F3FD-200D-1F9B1"},
    {"voksen_kruset_hår_farge4", "1F9D1-1F3FE-200D-1F9B1"},
    {"voksen_kruset_hår_farge5", "1F9D1-1F3FF-200D-1F9B1"},
    {"voksen_grått_hår", "1F9D1-200D-1F9B3"},
    {"voksen_grått_hår_farge1", "1F9D1-1F3FB-200D-1F9B3"},
    {"voksen_grått_hår_farge2", "1F9D1-1F3FC-200D-1F9B3"},
    {"voksen_grått_hår_farge3", "1F9D1-1F3FD-200D-1F9B3"},
    {"voksen_grått_hår_farge4", "1F9D1-1F3FE-200D-1F9B3"},
    {"voksen_grått_hår_farge5", "1F9D1-1F3FF-200D-1F9B3"},
    {"sjørøverflagg", "1F3F4-200D-2620-FE0F"},
    {"føniks", "1F426-200D-1F525"},
    {"isbjørn", "1F43B-200D-2744-FE0F"},
    {"øye_i_snakkeboble", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"fjes_med_spiraløyne", "1F635-200D-1F4AB"},
};

