// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_fr[] = {
    {"points_dexclamation_et_dinterrogation", "2049"},
    {"marque_commerciale_deposee", "2122"},
    {"source_dinformations", "2139"},
    {"fleche_gauche_droite", "2194"},
    {"fleche_haut_bas", "2195"},
    {"fleche_haut_gauche", "2196"},
    {"fleche_haut_droite", "2197"},
    {"fleche_bas_droite", "2198"},
    {"fleche_bas_gauche", "2199"},
    {"clavier", "2328"},
    {"soleil", "2600"},
    {"nuage", "2601"},
    {"parapluie_ouvert", "2602"},
    {"bonhomme_de_neige", "2603"},
    {"comete", "2604"},
    {"case_cochee", "2611"},
    {"parapluie_avec_gouttes_de_pluie", "2614"},
    {"boisson_chaude", "2615"},
    {"trefle", "2618"},
    {"tete_de_mort", "2620"},
    {"radioactif", "2622"},
    {"danger_biologique", "2623"},
    {"croix_orthodoxe", "2626"},
    {"roue_du_dharma", "2638"},
    {"visage_mecontent", "2639"},
    {"symbole_de_la_femme", "2640"},
    {"symbole_de_lhomme", "2642"},
    {"belier_zodiaque", "2648"},
    {"taureau", "2649"},
    {"sagittaire", "2650"},
    {"capricorne", "2651"},
    {"verseau", "2652"},
    {"poissons", "2653"},
    {"pique", "2660"},
    {"trefle_cartes", "2663"},
    {"coeur_cartes", "2665"},
    {"carreau", "2666"},
    {"sources_chaudes", "2668"},
    {"marteau_et_pioche", "2692"},
    {"ancre", "2693"},
    {"epees_croisees", "2694"},
    {"caducee", "2695"},
    {"balance_a_poids", "2696"},
    {"alambic", "2697"},
    {"roue_dentee", "2699"},
    {"ciseaux", "2702"},
    {"bouton_coche", "2705"},
    {"avion", "2708"},
    {"enveloppe", "2709"},
    {"stylo_plume_noir", "2712"},
    {"coche", "2714"},
    {"signe_multiplier", "2716"},
    {"etoile_de_david", "2721"},
    {"etincelles", "2728"},
    {"asterisque_huit_branches", "2733"},
    {"etoile_huit_branches", "2734"},
    {"flocon", "2744"},
    {"eclat", "2747"},
    {"point_dinterrogation_rouge", "2753"},
    {"point_dinterrogation_blanc", "2754"},
    {"point_dexclamation_blanc", "2755"},
    {"point_dexclamation_rouge", "2757"},
    {"coeur_point_dexclamation", "2763"},
    {"coeur_rouge", "2764"},
    {"plus", "2795"},
    {"moins", "2796"},
    {"signe_diviser", "2797"},
    {"fleche_courbe_haut", "2934"},
    {"fleche_courbe_bas", "2935"},
    {"ligne_ondulee", "3030"},
    {"bouton_felicitations_en_japonais", "3297"},
    {"bouton_secret_en_japonais", "3299"},
    {"symbole_copyright", "00A9"},
    {"marque_deposee", "00AE"},
    {"double_point_dexclamation", "203C"},
    {"fleche_courbe_gauche", "21A9"},
    {"fleche_courbe_droite", "21AA"},
    {"montre", "231A"},
    {"sablier", "231B"},
    {"bouton_ejecter", "23CF"},
    {"bouton_avance_rapide", "23E9"},
    {"bouton_retour_rapide", "23EA"},
    {"double_fleche_vers_le_haut", "23EB"},
    {"double_fleche_vers_le_bas", "23EC"},
    {"bouton_piste_suivante", "23ED"},
    {"bouton_piste_precedente", "23EE"},
    {"bouton_lecture_pause", "23EF"},
    {"reveil", "23F0"},
    {"chronometre", "23F1"},
    {"horloge", "23F2"},
    {"sablier_avec_sable_qui_coule", "23F3"},
    {"bouton_pause", "23F8"},
    {"bouton_stop", "23F9"},
    {"bouton_enregistrer", "23FA"},
    {"m_encercle", "24C2"},
    {"petit_carre_noir", "25AA"},
    {"petit_carre_blanc", "25AB"},
    {"bouton_lecture", "25B6"},
    {"bouton_retour", "25C0"},
    {"carre_moyen_blanc", "25FB"},
    {"carre_moyen_noir", "25FC"},
    {"carre_petit_moyen_blanc", "25FD"},
    {"carre_petit_moyen_noir", "25FE"},
    {"telephone", "260E"},
    {"index_pointant_vers_le_haut", "261D"},
    {"index_pointant_vers_le_haut_couleur1", "261D-1F3FB"},
    {"index_pointant_vers_le_haut_couleur2", "261D-1F3FC"},
    {"index_pointant_vers_le_haut_couleur3", "261D-1F3FD"},
    {"index_pointant_vers_le_haut_couleur4", "261D-1F3FE"},
    {"index_pointant_vers_le_haut_couleur5", "261D-1F3FF"},
    {"lune_et_etoile", "262A"},
    {"symbole_de_paix", "262E"},
    {"yin_yang", "262F"},
    {"visage_souriant", "263A"},
    {"gemeaux", "264A"},
    {"cancer", "264B"},
    {"lion", "264C"},
    {"vierge", "264D"},
    {"balance", "264E"},
    {"scorpion_zodiaque", "264F"},
    {"pion_dechec", "265F"},
    {"symbole_recyclage", "267B"},
    {"infini", "267E"},
    {"symbole_acces_handicapes", "267F"},
    {"symbole_de_latome", "269B"},
    {"fleur_de_lys", "269C"},
    {"symbole_davertissement", "26A0"},
    {"haute_tension", "26A1"},
    {"symbole_de_la_communaute_transgenre", "26A7"},
    {"disque_blanc", "26AA"},
    {"disque_noir", "26AB"},
    {"cercueil", "26B0"},
    {"urne_funeraire", "26B1"},
    {"ballon_de_football", "26BD"},
    {"baseball", "26BE"},
    {"bonhomme_de_neige_sans_neige", "26C4"},
    {"soleil_derriere_les_nuages", "26C5"},
    {"nuage_avec_eclair_et_pluie", "26C8"},
    {"serpentaire", "26CE"},
    {"pioche", "26CF"},
    {"casque_de_secouriste", "26D1"},
    {"chaines", "26D3"},
    {"sens_interdit", "26D4"},
    {"sanctuaire_shinto", "26E9"},
    {"eglise", "26EA"},
    {"montagne", "26F0"},
    {"parasol_sur_le_sol", "26F1"},
    {"fontaine", "26F2"},
    {"drapeau_de_golf", "26F3"},
    {"ferry", "26F4"},
    {"voilier", "26F5"},
    {"skieur", "26F7"},
    {"patin_a_glace", "26F8"},
    {"personne_avec_ballon", "26F9"},
    {"personne_avec_ballon_couleur1", "26F9-1F3FB"},
    {"personne_avec_ballon_couleur2", "26F9-1F3FC"},
    {"personne_avec_ballon_couleur3", "26F9-1F3FD"},
    {"personne_avec_ballon_couleur4", "26F9-1F3FE"},
    {"personne_avec_ballon_couleur5", "26F9-1F3FF"},
    {"tente", "26FA"},
    {"pompe_a_essence", "26FD"},
    {"poing_leve", "270A"},
    {"poing_leve_couleur1", "270A-1F3FB"},
    {"poing_leve_couleur2", "270A-1F3FC"},
    {"poing_leve_couleur3", "270A-1F3FD"},
    {"poing_leve_couleur4", "270A-1F3FE"},
    {"poing_leve_couleur5", "270A-1F3FF"},
    {"main_levee", "270B"},
    {"main_levee_couleur1", "270B-1F3FB"},
    {"main_levee_couleur2", "270B-1F3FC"},
    {"main_levee_couleur3", "270B-1F3FD"},
    {"main_levee_couleur4", "270B-1F3FE"},
    {"main_levee_couleur5", "270B-1F3FF"},
    {"v_de_la_victoire", "270C"},
    {"v_de_la_victoire_couleur1", "270C-1F3FB"},
    {"v_de_la_victoire_couleur2", "270C-1F3FC"},
    {"v_de_la_victoire_couleur3", "270C-1F3FD"},
    {"v_de_la_victoire_couleur4", "270C-1F3FE"},
    {"v_de_la_victoire_couleur5", "270C-1F3FF"},
    {"main_qui_ecrit", "270D"},
    {"main_qui_ecrit_couleur1", "270D-1F3FB"},
    {"main_qui_ecrit_couleur2", "270D-1F3FC"},
    {"main_qui_ecrit_couleur3", "270D-1F3FD"},
    {"main_qui_ecrit_couleur4", "270D-1F3FE"},
    {"main_qui_ecrit_couleur5", "270D-1F3FF"},
    {"crayon", "270F"},
    {"croix_latine", "271D"},
    {"croix", "274C"},
    {"bouton_croix", "274E"},
    {"fleche_droite", "27A1"},
    {"boucle", "27B0"},
    {"double_boucle", "27BF"},
    {"fleche_gauche", "2B05"},
    {"fleche_haut", "2B06"},
    {"fleche_bas", "2B07"},
    {"grand_carre_noir", "2B1B"},
    {"grand_carre_blanc", "2B1C"},
    {"etoile", "2B50"},
    {"cercle_rouge", "2B55"},
    {"alternance", "303D"},
    {"dragon_rouge_mahjong", "1F004"},
    {"carte_joker", "1F0CF"},
    {"groupe_sanguin_a", "1F170"},
    {"groupe_sanguin_b", "1F171"},
    {"groupe_sanguin_o", "1F17E"},
    {"bouton_p", "1F17F"},
    {"groupe_sanguin_ab", "1F18E"},
    {"bouton_effacer", "1F191"},
    {"bouton_cool", "1F192"},
    {"bouton_gratuit", "1F193"},
    {"bouton_identifiant", "1F194"},
    {"bouton_nouveau", "1F195"},
    {"bouton_pas_bien", "1F196"},
    {"bouton_ok", "1F197"},
    {"bouton_sos", "1F198"},
    {"bouton_vers_le_haut", "1F199"},
    {"bouton_vs", "1F19A"},
    {"indicateur_regional_a", "1F1E6"},
    {"indicateur_regional_b", "1F1E7"},
    {"indicateur_regional_c", "1F1E8"},
    {"indicateur_regional_d", "1F1E9"},
    {"indicateur_regional_e", "1F1EA"},
    {"indicateur_regional_f", "1F1EB"},
    {"indicateur_regional_g", "1F1EC"},
    {"indicateur_regional_h", "1F1ED"},
    {"indicateur_regional_i", "1F1EE"},
    {"indicateur_regional_j", "1F1EF"},
    {"indicateur_regional_k", "1F1F0"},
    {"indicateur_regional_l", "1F1F1"},
    {"indicateur_regional_m", "1F1F2"},
    {"indicateur_regional_n", "1F1F3"},
    {"indicateur_regional_o", "1F1F4"},
    {"indicateur_regional_p", "1F1F5"},
    {"indicateur_regional_q", "1F1F6"},
    {"indicateur_regional_r", "1F1F7"},
    {"indicateur_regional_s", "1F1F8"},
    {"indicateur_regional_t", "1F1F9"},
    {"indicateur_regional_u", "1F1FA"},
    {"indicateur_regional_v", "1F1FB"},
    {"indicateur_regional_w", "1F1FC"},
    {"indicateur_regional_x", "1F1FD"},
    {"indicateur_regional_y", "1F1FE"},
    {"indicateur_regional_z", "1F1FF"},
    {"bouton_ici_en_japonais", "1F201"},
    {"bouton_frais_de_service_en_japonais", "1F202"},
    {"bouton_gratuit_en_japonais", "1F21A"},
    {"bouton_reserve_en_japonais", "1F22F"},
    {"bouton_interdit_en_japonais", "1F232"},
    {"bouton_chambres_disponibles_en_japonais", "1F233"},
    {"bouton_note_pour_reussir_en_japonais", "1F234"},
    {"bouton_complet_en_japonais", "1F235"},
    {"bouton_pas_gratuit_en_japonais", "1F236"},
    {"bouton_montant_mensuel_en_japonais", "1F237"},
    {"bouton_application_en_japonais", "1F238"},
    {"bouton_reduction_en_japonais", "1F239"},
    {"bouton_ouvert_pour_affaires_en_japonais", "1F23A"},
    {"bouton_bonne_affaire_en_japonais", "1F250"},
    {"bouton_accepter_en_japonais", "1F251"},
    {"cyclone", "1F300"},
    {"brume", "1F301"},
    {"parapluie_ferme", "1F302"},
    {"nuit_etoilee", "1F303"},
    {"soleil_levant_derriere_les_montagnes", "1F304"},
    {"soleil_levant", "1F305"},
    {"ville_au_crepuscule", "1F306"},
    {"coucher_de_soleil", "1F307"},
    {"arc_en_ciel", "1F308"},
    {"pont_de_nuit", "1F309"},
    {"vague", "1F30A"},
    {"volcan", "1F30B"},
    {"voie_lactee", "1F30C"},
    {"globe_tourne_sur_lafrique_et_leurope", "1F30D"},
    {"globe_tourne_sur_les_ameriques", "1F30E"},
    {"globe_tourne_sur_lasie_et_laustralie", "1F30F"},
    {"globe_avec_meridiens", "1F310"},
    {"nouvelle_lune", "1F311"},
    {"lune_croissante", "1F312"},
    {"premier_quartier_de_lune", "1F313"},
    {"lune_gibbeuse_croissante", "1F314"},
    {"pleine_lune", "1F315"},
    {"lune_gibbeuse_decroissante", "1F316"},
    {"dernier_quartier_de_lune", "1F317"},
    {"lune_decroissante", "1F318"},
    {"croissant_de_lune", "1F319"},
    {"nouvelle_lune_avec_visage", "1F31A"},
    {"premier_quartier_de_lune_avec_visage", "1F31B"},
    {"dernier_quartier_de_lune_avec_visage", "1F31C"},
    {"pleine_lune_avec_visage", "1F31D"},
    {"soleil_avec_visage", "1F31E"},
    {"etoile_brillante", "1F31F"},
    {"etoile_filante", "1F320"},
    {"thermometre", "1F321"},
    {"soleil_derriere_un_petit_nuage", "1F324"},
    {"soleil_derriere_un_gros_nuage", "1F325"},
    {"soleil_derriere_un_nuage_de_pluie", "1F326"},
    {"nuage_avec_pluie", "1F327"},
    {"nuage_avec_neige", "1F328"},
    {"nuage_avec_eclair", "1F329"},
    {"tornade", "1F32A"},
    {"brouillard", "1F32B"},
    {"vent_avec_visage", "1F32C"},
    {"hot_dog", "1F32D"},
    {"taco", "1F32E"},
    {"burrito", "1F32F"},
    {"chataigne", "1F330"},
    {"jeune_pousse", "1F331"},
    {"conifere", "1F332"},
    {"arbre_a_feuilles_caduques", "1F333"},
    {"palmier", "1F334"},
    {"cactus", "1F335"},
    {"piment_rouge", "1F336"},
    {"tulipe", "1F337"},
    {"fleur_de_cerisier", "1F338"},
    {"rose", "1F339"},
    {"hibiscus", "1F33A"},
    {"tournesol", "1F33B"},
    {"bourgeon", "1F33C"},
    {"epi_de_mais", "1F33D"},
    {"plant_de_riz", "1F33E"},
    {"feuille", "1F33F"},
    {"trefle_a_quatre_feuilles", "1F340"},
    {"feuille_derable", "1F341"},
    {"feuille_morte", "1F342"},
    {"feuille_virevoltante", "1F343"},
    {"champignon", "1F344"},
    {"tomate", "1F345"},
    {"aubergine", "1F346"},
    {"raisin", "1F347"},
    {"melon", "1F348"},
    {"pasteque", "1F349"},
    {"mandarine", "1F34A"},
    {"citron", "1F34B"},
    {"banane", "1F34C"},
    {"ananas", "1F34D"},
    {"pomme_rouge", "1F34E"},
    {"pomme_verte", "1F34F"},
    {"poire", "1F350"},
    {"peche", "1F351"},
    {"cerises", "1F352"},
    {"fraise", "1F353"},
    {"hamburger", "1F354"},
    {"pizza", "1F355"},
    {"viande_sur_un_os", "1F356"},
    {"cuisse_de_poulet", "1F357"},
    {"galette_de_riz", "1F358"},
    {"boulette_de_riz", "1F359"},
    {"bol_de_riz", "1F35A"},
    {"riz_au_curry", "1F35B"},
    {"bol_fumant", "1F35C"},
    {"spaghetti", "1F35D"},
    {"pain", "1F35E"},
    {"frites", "1F35F"},
    {"patate_douce", "1F360"},
    {"brochette_de_bonbons", "1F361"},
    {"brochette_de_poisson", "1F362"},
    {"sushi", "1F363"},
    {"beignet_de_crevette", "1F364"},
    {"croquette_de_poisson", "1F365"},
    {"glace_italienne", "1F366"},
    {"granite", "1F367"},
    {"glace", "1F368"},
    {"doughnut", "1F369"},
    {"cookie", "1F36A"},
    {"barre_chocolatee", "1F36B"},
    {"bonbon", "1F36C"},
    {"sucette", "1F36D"},
    {"creme_renversee", "1F36E"},
    {"pot_de_miel", "1F36F"},
    {"gateau_sable", "1F370"},
    {"boite_dejeuner", "1F371"},
    {"marmite", "1F372"},
    {"oeuf_au_plat", "1F373"},
    {"fourchette_et_couteau", "1F374"},
    {"tasse", "1F375"},
    {"sake", "1F376"},
    {"verre_de_vin", "1F377"},
    {"cocktail", "1F378"},
    {"cocktail_tropical", "1F379"},
    {"chope", "1F37A"},
    {"chopes_de_biere", "1F37B"},
    {"biberon", "1F37C"},
    {"fourchette_et_couteau_avec_assiette", "1F37D"},
    {"bouteille_de_champagne", "1F37E"},
    {"pop_corn", "1F37F"},
    {"ruban", "1F380"},
    {"cadeau", "1F381"},
    {"gateau_danniversaire", "1F382"},
    {"citrouille", "1F383"},
    {"sapin_de_noel", "1F384"},
    {"pere_noel", "1F385"},
    {"pere_noel_couleur1", "1F385-1F3FB"},
    {"pere_noel_couleur2", "1F385-1F3FC"},
    {"pere_noel_couleur3", "1F385-1F3FD"},
    {"pere_noel_couleur4", "1F385-1F3FE"},
    {"pere_noel_couleur5", "1F385-1F3FF"},
    {"feu_dartifice", "1F386"},
    {"cierge_magique", "1F387"},
    {"ballon_gonflable", "1F388"},
    {"cotillons", "1F389"},
    {"confettis", "1F38A"},
    {"arbre_a_voeux", "1F38B"},
    {"drapeaux_croises", "1F38C"},
    {"bambou_decoratif", "1F38D"},
    {"poupees_japonaises", "1F38E"},
    {"koinobori", "1F38F"},
    {"carillon_eolien", "1F390"},
    {"ceremonie_de_la_lune", "1F391"},
    {"cartable", "1F392"},
    {"toque_universitaire", "1F393"},
    {"medaille_militaire", "1F396"},
    {"ruban_de_memoire", "1F397"},
    {"micro_de_studio", "1F399"},
    {"curseur_de_niveau", "1F39A"},
    {"boutons_de_reglage", "1F39B"},
    {"pellicule", "1F39E"},
    {"billet_dentree", "1F39F"},
    {"cheval_de_manege", "1F3A0"},
    {"grande_roue", "1F3A1"},
    {"montagnes_russes", "1F3A2"},
    {"peche_a_la_ligne", "1F3A3"},
    {"micro", "1F3A4"},
    {"camera", "1F3A5"},
    {"cinema", "1F3A6"},
    {"casque", "1F3A7"},
    {"palette_de_peinture", "1F3A8"},
    {"haut_de_forme", "1F3A9"},
    {"chapiteau", "1F3AA"},
    {"billet", "1F3AB"},
    {"clap", "1F3AC"},
    {"spectacle_vivant", "1F3AD"},
    {"jeu_video", "1F3AE"},
    {"dans_le_mille", "1F3AF"},
    {"machine_a_sous", "1F3B0"},
    {"boule_de_billard", "1F3B1"},
    {"des", "1F3B2"},
    {"bowling", "1F3B3"},
    {"jeu_des_fleurs", "1F3B4"},
    {"note_de_musique", "1F3B5"},
    {"notes_de_musique", "1F3B6"},
    {"saxophone", "1F3B7"},
    {"guitare", "1F3B8"},
    {"piano", "1F3B9"},
    {"trompette", "1F3BA"},
    {"violon", "1F3BB"},
    {"partition", "1F3BC"},
    {"maillot_de_course", "1F3BD"},
    {"tennis", "1F3BE"},
    {"ski", "1F3BF"},
    {"basket", "1F3C0"},
    {"drapeau_a_damier", "1F3C1"},
    {"snowboardeur", "1F3C2"},
    {"snowboardeur_couleur1", "1F3C2-1F3FB"},
    {"snowboardeur_couleur2", "1F3C2-1F3FC"},
    {"snowboardeur_couleur3", "1F3C2-1F3FD"},
    {"snowboardeur_couleur4", "1F3C2-1F3FE"},
    {"snowboardeur_couleur5", "1F3C2-1F3FF"},
    {"personne_qui_court", "1F3C3"},
    {"personne_qui_court_couleur1", "1F3C3-1F3FB"},
    {"personne_qui_court_couleur2", "1F3C3-1F3FC"},
    {"personne_qui_court_couleur3", "1F3C3-1F3FD"},
    {"personne_qui_court_couleur4", "1F3C3-1F3FE"},
    {"personne_qui_court_couleur5", "1F3C3-1F3FF"},
    {"personne_faisant_du_surf", "1F3C4"},
    {"personne_faisant_du_surf_couleur1", "1F3C4-1F3FB"},
    {"personne_faisant_du_surf_couleur2", "1F3C4-1F3FC"},
    {"personne_faisant_du_surf_couleur3", "1F3C4-1F3FD"},
    {"personne_faisant_du_surf_couleur4", "1F3C4-1F3FE"},
    {"personne_faisant_du_surf_couleur5", "1F3C4-1F3FF"},
    {"medaille_sportive", "1F3C5"},
    {"trophee", "1F3C6"},
    {"course_hippique", "1F3C7"},
    {"course_hippique_couleur1", "1F3C7-1F3FB"},
    {"course_hippique_couleur2", "1F3C7-1F3FC"},
    {"course_hippique_couleur3", "1F3C7-1F3FD"},
    {"course_hippique_couleur4", "1F3C7-1F3FE"},
    {"course_hippique_couleur5", "1F3C7-1F3FF"},
    {"football_americain", "1F3C8"},
    {"rugby", "1F3C9"},
    {"personne_nageant", "1F3CA"},
    {"personne_nageant_couleur1", "1F3CA-1F3FB"},
    {"personne_nageant_couleur2", "1F3CA-1F3FC"},
    {"personne_nageant_couleur3", "1F3CA-1F3FD"},
    {"personne_nageant_couleur4", "1F3CA-1F3FE"},
    {"personne_nageant_couleur5", "1F3CA-1F3FF"},
    {"halterophile", "1F3CB"},
    {"halterophile_couleur1", "1F3CB-1F3FB"},
    {"halterophile_couleur2", "1F3CB-1F3FC"},
    {"halterophile_couleur3", "1F3CB-1F3FD"},
    {"halterophile_couleur4", "1F3CB-1F3FE"},
    {"halterophile_couleur5", "1F3CB-1F3FF"},
    {"joueur_de_golf", "1F3CC"},
    {"joueur_de_golf_couleur1", "1F3CC-1F3FB"},
    {"joueur_de_golf_couleur2", "1F3CC-1F3FC"},
    {"joueur_de_golf_couleur3", "1F3CC-1F3FD"},
    {"joueur_de_golf_couleur4", "1F3CC-1F3FE"},
    {"joueur_de_golf_couleur5", "1F3CC-1F3FF"},
    {"moto", "1F3CD"},
    {"voiture_de_course", "1F3CE"},
    {"cricket", "1F3CF"},
    {"volley_ball", "1F3D0"},
    {"hockey_sur_gazon", "1F3D1"},
    {"hockey_sur_glace", "1F3D2"},
    {"ping_pong", "1F3D3"},
    {"montagne_enneigee", "1F3D4"},
    {"camping", "1F3D5"},
    {"plage_avec_parasol", "1F3D6"},
    {"construction_dun_batiment", "1F3D7"},
    {"maisons", "1F3D8"},
    {"ville", "1F3D9"},
    {"maison_abandonnee", "1F3DA"},
    {"monument_classique", "1F3DB"},
    {"desert", "1F3DC"},
    {"ile_deserte", "1F3DD"},
    {"parc_national", "1F3DE"},
    {"stade", "1F3DF"},
    {"maison", "1F3E0"},
    {"maison_avec_jardin", "1F3E1"},
    {"immeuble_de_bureaux", "1F3E2"},
    {"bureau_de_poste_japonais", "1F3E3"},
    {"bureau_de_poste", "1F3E4"},
    {"hopital", "1F3E5"},
    {"banque", "1F3E6"},
    {"distributeur_de_billets", "1F3E7"},
    {"hotel", "1F3E8"},
    {"love_hotel", "1F3E9"},
    {"superette", "1F3EA"},
    {"ecole", "1F3EB"},
    {"grand_magasin", "1F3EC"},
    {"usine", "1F3ED"},
    {"lampion_rouge", "1F3EE"},
    {"chateau_japonais", "1F3EF"},
    {"chateau", "1F3F0"},
    {"drapeau_blanc", "1F3F3"},
    {"drapeau_noir", "1F3F4"},
    {"rosette", "1F3F5"},
    {"etiquette", "1F3F7"},
    {"badminton", "1F3F8"},
    {"arc_et_fleche", "1F3F9"},
    {"amphore", "1F3FA"},
    {"peau_claire", "1F3FB"},
    {"peau_moyennement_claire", "1F3FC"},
    {"peau_legerement_mate", "1F3FD"},
    {"peau_mate", "1F3FE"},
    {"peau_foncee", "1F3FF"},
    {"rat", "1F400"},
    {"souris", "1F401"},
    {"boeuf", "1F402"},
    {"buffle", "1F403"},
    {"vache", "1F404"},
    {"tigre", "1F405"},
    {"leopard", "1F406"},
    {"lapin", "1F407"},
    {"chat", "1F408"},
    {"dragon", "1F409"},
    {"crocodile", "1F40A"},
    {"baleine", "1F40B"},
    {"escargot", "1F40C"},
    {"serpent", "1F40D"},
    {"cheval", "1F40E"},
    {"belier", "1F40F"},
    {"chevre", "1F410"},
    {"mouton", "1F411"},
    {"singe", "1F412"},
    {"coq", "1F413"},
    {"poule", "1F414"},
    {"chien", "1F415"},
    {"cochon", "1F416"},
    {"sanglier", "1F417"},
    {"elephant", "1F418"},
    {"pieuvre", "1F419"},
    {"coquille_en_spirale", "1F41A"},
    {"chenille", "1F41B"},
    {"fourmi", "1F41C"},
    {"abeille", "1F41D"},
    {"coccinelle", "1F41E"},
    {"poisson", "1F41F"},
    {"poisson_tropical", "1F420"},
    {"poisson_lune", "1F421"},
    {"tortue", "1F422"},
    {"poussin_qui_eclot", "1F423"},
    {"poussin", "1F424"},
    {"poussin_de_face", "1F425"},
    {"oiseau", "1F426"},
    {"pingouin", "1F427"},
    {"koala", "1F428"},
    {"caniche", "1F429"},
    {"dromadaire", "1F42A"},
    {"chameau", "1F42B"},
    {"dauphin", "1F42C"},
    {"tete_de_souris", "1F42D"},
    {"tete_de_vache", "1F42E"},
    {"tete_de_tigre", "1F42F"},
    {"tete_de_lapin", "1F430"},
    {"tete_de_chat", "1F431"},
    {"tete_de_dragon", "1F432"},
    {"baleine_soufflant_par_son_event", "1F433"},
    {"tete_de_cheval", "1F434"},
    {"tete_de_singe", "1F435"},
    {"tete_de_chien", "1F436"},
    {"tete_de_cochon", "1F437"},
    {"grenouille", "1F438"},
    {"hamster", "1F439"},
    {"loup", "1F43A"},
    {"ours", "1F43B"},
    {"panda", "1F43C"},
    {"groin", "1F43D"},
    {"empreintes_danimaux", "1F43E"},
    {"ecureuil", "1F43F"},
    {"yeux", "1F440"},
    {"oeil", "1F441"},
    {"oreille", "1F442"},
    {"oreille_couleur1", "1F442-1F3FB"},
    {"oreille_couleur2", "1F442-1F3FC"},
    {"oreille_couleur3", "1F442-1F3FD"},
    {"oreille_couleur4", "1F442-1F3FE"},
    {"oreille_couleur5", "1F442-1F3FF"},
    {"nez", "1F443"},
    {"nez_couleur1", "1F443-1F3FB"},
    {"nez_couleur2", "1F443-1F3FC"},
    {"nez_couleur3", "1F443-1F3FD"},
    {"nez_couleur4", "1F443-1F3FE"},
    {"nez_couleur5", "1F443-1F3FF"},
    {"bouche", "1F444"},
    {"langue", "1F445"},
    {"main_avec_index_pointant_vers_le_haut", "1F446"},
    {"main_avec_index_pointant_vers_le_haut_couleur1", "1F446-1F3FB"},
    {"main_avec_index_pointant_vers_le_haut_couleur2", "1F446-1F3FC"},
    {"main_avec_index_pointant_vers_le_haut_couleur3", "1F446-1F3FD"},
    {"main_avec_index_pointant_vers_le_haut_couleur4", "1F446-1F3FE"},
    {"main_avec_index_pointant_vers_le_haut_couleur5", "1F446-1F3FF"},
    {"main_avec_index_pointant_vers_le_bas", "1F447"},
    {"main_avec_index_pointant_vers_le_bas_couleur1", "1F447-1F3FB"},
    {"main_avec_index_pointant_vers_le_bas_couleur2", "1F447-1F3FC"},
    {"main_avec_index_pointant_vers_le_bas_couleur3", "1F447-1F3FD"},
    {"main_avec_index_pointant_vers_le_bas_couleur4", "1F447-1F3FE"},
    {"main_avec_index_pointant_vers_le_bas_couleur5", "1F447-1F3FF"},
    {"main_avec_index_pointant_a_gauche", "1F448"},
    {"main_avec_index_pointant_a_gauche_couleur1", "1F448-1F3FB"},
    {"main_avec_index_pointant_a_gauche_couleur2", "1F448-1F3FC"},
    {"main_avec_index_pointant_a_gauche_couleur3", "1F448-1F3FD"},
    {"main_avec_index_pointant_a_gauche_couleur4", "1F448-1F3FE"},
    {"main_avec_index_pointant_a_gauche_couleur5", "1F448-1F3FF"},
    {"main_avec_index_pointant_a_droite", "1F449"},
    {"main_avec_index_pointant_a_droite_couleur1", "1F449-1F3FB"},
    {"main_avec_index_pointant_a_droite_couleur2", "1F449-1F3FC"},
    {"main_avec_index_pointant_a_droite_couleur3", "1F449-1F3FD"},
    {"main_avec_index_pointant_a_droite_couleur4", "1F449-1F3FE"},
    {"main_avec_index_pointant_a_droite_couleur5", "1F449-1F3FF"},
    {"poing_de_face", "1F44A"},
    {"poing_de_face_couleur1", "1F44A-1F3FB"},
    {"poing_de_face_couleur2", "1F44A-1F3FC"},
    {"poing_de_face_couleur3", "1F44A-1F3FD"},
    {"poing_de_face_couleur4", "1F44A-1F3FE"},
    {"poing_de_face_couleur5", "1F44A-1F3FF"},
    {"signe_de_la_main", "1F44B"},
    {"signe_de_la_main_couleur1", "1F44B-1F3FB"},
    {"signe_de_la_main_couleur2", "1F44B-1F3FC"},
    {"signe_de_la_main_couleur3", "1F44B-1F3FD"},
    {"signe_de_la_main_couleur4", "1F44B-1F3FE"},
    {"signe_de_la_main_couleur5", "1F44B-1F3FF"},
    {"ok", "1F44C"},
    {"ok_couleur1", "1F44C-1F3FB"},
    {"ok_couleur2", "1F44C-1F3FC"},
    {"ok_couleur3", "1F44C-1F3FD"},
    {"ok_couleur4", "1F44C-1F3FE"},
    {"ok_couleur5", "1F44C-1F3FF"},
    {"pouce_vers_le_haut", "1F44D"},
    {"pouce_vers_le_haut_couleur1", "1F44D-1F3FB"},
    {"pouce_vers_le_haut_couleur2", "1F44D-1F3FC"},
    {"pouce_vers_le_haut_couleur3", "1F44D-1F3FD"},
    {"pouce_vers_le_haut_couleur4", "1F44D-1F3FE"},
    {"pouce_vers_le_haut_couleur5", "1F44D-1F3FF"},
    {"pouce_vers_le_bas", "1F44E"},
    {"pouce_vers_le_bas_couleur1", "1F44E-1F3FB"},
    {"pouce_vers_le_bas_couleur2", "1F44E-1F3FC"},
    {"pouce_vers_le_bas_couleur3", "1F44E-1F3FD"},
    {"pouce_vers_le_bas_couleur4", "1F44E-1F3FE"},
    {"pouce_vers_le_bas_couleur5", "1F44E-1F3FF"},
    {"applaudissements", "1F44F"},
    {"applaudissements_couleur1", "1F44F-1F3FB"},
    {"applaudissements_couleur2", "1F44F-1F3FC"},
    {"applaudissements_couleur3", "1F44F-1F3FD"},
    {"applaudissements_couleur4", "1F44F-1F3FE"},
    {"applaudissements_couleur5", "1F44F-1F3FF"},
    {"mains_ouvertes", "1F450"},
    {"mains_ouvertes_couleur1", "1F450-1F3FB"},
    {"mains_ouvertes_couleur2", "1F450-1F3FC"},
    {"mains_ouvertes_couleur3", "1F450-1F3FD"},
    {"mains_ouvertes_couleur4", "1F450-1F3FE"},
    {"mains_ouvertes_couleur5", "1F450-1F3FF"},
    {"couronne", "1F451"},
    {"chapeau_de_femme", "1F452"},
    {"lunettes_de_vue", "1F453"},
    {"cravate", "1F454"},
    {"t_shirt", "1F455"},
    {"jean", "1F456"},
    {"robe", "1F457"},
    {"kimono", "1F458"},
    {"bikini", "1F459"},
    {"vetements_de_femme", "1F45A"},
    {"porte_monnaie", "1F45B"},
    {"sac_a_main", "1F45C"},
    {"pochette", "1F45D"},
    {"chaussure_dhomme", "1F45E"},
    {"chaussure_de_sport", "1F45F"},
    {"chaussure_a_talon_haut", "1F460"},
    {"sandale_de_femme", "1F461"},
    {"botte_de_femme", "1F462"},
    {"traces_de_pas", "1F463"},
    {"silhouette_de_buste", "1F464"},
    {"silhouettes_de_bustes", "1F465"},
    {"garcon", "1F466"},
    {"garcon_couleur1", "1F466-1F3FB"},
    {"garcon_couleur2", "1F466-1F3FC"},
    {"garcon_couleur3", "1F466-1F3FD"},
    {"garcon_couleur4", "1F466-1F3FE"},
    {"garcon_couleur5", "1F466-1F3FF"},
    {"fille", "1F467"},
    {"fille_couleur1", "1F467-1F3FB"},
    {"fille_couleur2", "1F467-1F3FC"},
    {"fille_couleur3", "1F467-1F3FD"},
    {"fille_couleur4", "1F467-1F3FE"},
    {"fille_couleur5", "1F467-1F3FF"},
    {"homme", "1F468"},
    {"homme_couleur1", "1F468-1F3FB"},
    {"homme_couleur2", "1F468-1F3FC"},
    {"homme_couleur3", "1F468-1F3FD"},
    {"homme_couleur4", "1F468-1F3FE"},
    {"homme_couleur5", "1F468-1F3FF"},
    {"femme", "1F469"},
    {"femme_couleur1", "1F469-1F3FB"},
    {"femme_couleur2", "1F469-1F3FC"},
    {"femme_couleur3", "1F469-1F3FD"},
    {"femme_couleur4", "1F469-1F3FE"},
    {"femme_couleur5", "1F469-1F3FF"},
    {"famille", "1F46A"},
    {"femme_et_homme_se_tenant_la_main", "1F46B"},
    {"femme_et_homme_se_tenant_la_main_couleur1", "1F46B-1F3FB"},
    {"femme_et_homme_se_tenant_la_main_couleur2", "1F46B-1F3FC"},
    {"femme_et_homme_se_tenant_la_main_couleur3", "1F46B-1F3FD"},
    {"femme_et_homme_se_tenant_la_main_couleur4", "1F46B-1F3FE"},
    {"femme_et_homme_se_tenant_la_main_couleur5", "1F46B-1F3FF"},
    {"femme_et_homme_se_tenant_la_main_couleur1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"femme_et_homme_se_tenant_la_main_couleur1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"femme_et_homme_se_tenant_la_main_couleur1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"femme_et_homme_se_tenant_la_main_couleur1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"femme_et_homme_se_tenant_la_main_couleur2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"femme_et_homme_se_tenant_la_main_couleur2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"femme_et_homme_se_tenant_la_main_couleur2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"femme_et_homme_se_tenant_la_main_couleur2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"femme_et_homme_se_tenant_la_main_couleur3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"femme_et_homme_se_tenant_la_main_couleur3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"femme_et_homme_se_tenant_la_main_couleur3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"femme_et_homme_se_tenant_la_main_couleur3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"femme_et_homme_se_tenant_la_main_couleur4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"femme_et_homme_se_tenant_la_main_couleur4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"femme_et_homme_se_tenant_la_main_couleur4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"femme_et_homme_se_tenant_la_main_couleur4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"femme_et_homme_se_tenant_la_main_couleur5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"femme_et_homme_se_tenant_la_main_couleur5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"femme_et_homme_se_tenant_la_main_couleur5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"femme_et_homme_se_tenant_la_main_couleur5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"deux_hommes_se_tenant_la_main", "1F46C"},
    {"deux_hommes_se_tenant_la_main_couleur1", "1F46C-1F3FB"},
    {"deux_hommes_se_tenant_la_main_couleur2", "1F46C-1F3FC"},
    {"deux_hommes_se_tenant_la_main_couleur3", "1F46C-1F3FD"},
    {"deux_hommes_se_tenant_la_main_couleur4", "1F46C-1F3FE"},
    {"deux_hommes_se_tenant_la_main_couleur5", "1F46C-1F3FF"},
    {"deux_hommes_se_tenant_la_main_couleur1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"deux_hommes_se_tenant_la_main_couleur1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"deux_hommes_se_tenant_la_main_couleur1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"deux_hommes_se_tenant_la_main_couleur1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"deux_hommes_se_tenant_la_main_couleur2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"deux_hommes_se_tenant_la_main_couleur2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"deux_hommes_se_tenant_la_main_couleur2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"deux_hommes_se_tenant_la_main_couleur2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"deux_hommes_se_tenant_la_main_couleur3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"deux_hommes_se_tenant_la_main_couleur3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"deux_hommes_se_tenant_la_main_couleur3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"deux_hommes_se_tenant_la_main_couleur3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"deux_hommes_se_tenant_la_main_couleur4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"deux_hommes_se_tenant_la_main_couleur4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"deux_hommes_se_tenant_la_main_couleur4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"deux_hommes_se_tenant_la_main_couleur4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"deux_hommes_se_tenant_la_main_couleur5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"deux_hommes_se_tenant_la_main_couleur5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"deux_hommes_se_tenant_la_main_couleur5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"deux_hommes_se_tenant_la_main_couleur5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"deux_femmes_se_tenant_la_main", "1F46D"},
    {"deux_femmes_se_tenant_la_main_couleur1", "1F46D-1F3FB"},
    {"deux_femmes_se_tenant_la_main_couleur2", "1F46D-1F3FC"},
    {"deux_femmes_se_tenant_la_main_couleur3", "1F46D-1F3FD"},
    {"deux_femmes_se_tenant_la_main_couleur4", "1F46D-1F3FE"},
    {"deux_femmes_se_tenant_la_main_couleur5", "1F46D-1F3FF"},
    {"deux_femmes_se_tenant_la_main_couleur1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"deux_femmes_se_tenant_la_main_couleur1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"deux_femmes_se_tenant_la_main_couleur1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"deux_femmes_se_tenant_la_main_couleur1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"deux_femmes_se_tenant_la_main_couleur2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"deux_femmes_se_tenant_la_main_couleur2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"deux_femmes_se_tenant_la_main_couleur2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"deux_femmes_se_tenant_la_main_couleur2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"deux_femmes_se_tenant_la_main_couleur3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"deux_femmes_se_tenant_la_main_couleur3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"deux_femmes_se_tenant_la_main_couleur3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"deux_femmes_se_tenant_la_main_couleur3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"deux_femmes_se_tenant_la_main_couleur4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"deux_femmes_se_tenant_la_main_couleur4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"deux_femmes_se_tenant_la_main_couleur4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"deux_femmes_se_tenant_la_main_couleur4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"deux_femmes_se_tenant_la_main_couleur5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"deux_femmes_se_tenant_la_main_couleur5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"deux_femmes_se_tenant_la_main_couleur5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"deux_femmes_se_tenant_la_main_couleur5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"officier_de_police", "1F46E"},
    {"officier_de_police_couleur1", "1F46E-1F3FB"},
    {"officier_de_police_couleur2", "1F46E-1F3FC"},
    {"officier_de_police_couleur3", "1F46E-1F3FD"},
    {"officier_de_police_couleur4", "1F46E-1F3FE"},
    {"officier_de_police_couleur5", "1F46E-1F3FF"},
    {"personnes_avec_des_oreilles_de_lapin", "1F46F"},
    {"personnes_avec_des_oreilles_de_lapin_couleur1", "1F46F-1F3FB"},
    {"personnes_avec_des_oreilles_de_lapin_couleur2", "1F46F-1F3FC"},
    {"personnes_avec_des_oreilles_de_lapin_couleur3", "1F46F-1F3FD"},
    {"personnes_avec_des_oreilles_de_lapin_couleur4", "1F46F-1F3FE"},
    {"personnes_avec_des_oreilles_de_lapin_couleur5", "1F46F-1F3FF"},
    {"personnes_avec_des_oreilles_de_lapin_couleur1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"personnes_avec_des_oreilles_de_lapin_couleur1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"personnes_avec_des_oreilles_de_lapin_couleur1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"personnes_avec_des_oreilles_de_lapin_couleur1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"personnes_avec_des_oreilles_de_lapin_couleur2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"personnes_avec_des_oreilles_de_lapin_couleur2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"personnes_avec_des_oreilles_de_lapin_couleur2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"personnes_avec_des_oreilles_de_lapin_couleur2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"personnes_avec_des_oreilles_de_lapin_couleur3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"personnes_avec_des_oreilles_de_lapin_couleur3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"personnes_avec_des_oreilles_de_lapin_couleur3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"personnes_avec_des_oreilles_de_lapin_couleur3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"personnes_avec_des_oreilles_de_lapin_couleur4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"personnes_avec_des_oreilles_de_lapin_couleur4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"personnes_avec_des_oreilles_de_lapin_couleur4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"personnes_avec_des_oreilles_de_lapin_couleur4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"personnes_avec_des_oreilles_de_lapin_couleur5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"personnes_avec_des_oreilles_de_lapin_couleur5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"personnes_avec_des_oreilles_de_lapin_couleur5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"personnes_avec_des_oreilles_de_lapin_couleur5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"personne_mariee_avec_voile", "1F470"},
    {"personne_mariee_avec_voile_couleur1", "1F470-1F3FB"},
    {"personne_mariee_avec_voile_couleur2", "1F470-1F3FC"},
    {"personne_mariee_avec_voile_couleur3", "1F470-1F3FD"},
    {"personne_mariee_avec_voile_couleur4", "1F470-1F3FE"},
    {"personne_mariee_avec_voile_couleur5", "1F470-1F3FF"},
    {"personne_blonde", "1F471"},
    {"personne_blonde_couleur1", "1F471-1F3FB"},
    {"personne_blonde_couleur2", "1F471-1F3FC"},
    {"personne_blonde_couleur3", "1F471-1F3FD"},
    {"personne_blonde_couleur4", "1F471-1F3FE"},
    {"personne_blonde_couleur5", "1F471-1F3FF"},
    {"homme_avec_casquette_chinoise", "1F472"},
    {"homme_avec_casquette_chinoise_couleur1", "1F472-1F3FB"},
    {"homme_avec_casquette_chinoise_couleur2", "1F472-1F3FC"},
    {"homme_avec_casquette_chinoise_couleur3", "1F472-1F3FD"},
    {"homme_avec_casquette_chinoise_couleur4", "1F472-1F3FE"},
    {"homme_avec_casquette_chinoise_couleur5", "1F472-1F3FF"},
    {"personne_en_turban", "1F473"},
    {"personne_en_turban_couleur1", "1F473-1F3FB"},
    {"personne_en_turban_couleur2", "1F473-1F3FC"},
    {"personne_en_turban_couleur3", "1F473-1F3FD"},
    {"personne_en_turban_couleur4", "1F473-1F3FE"},
    {"personne_en_turban_couleur5", "1F473-1F3FF"},
    {"homme_age", "1F474"},
    {"homme_age_couleur1", "1F474-1F3FB"},
    {"homme_age_couleur2", "1F474-1F3FC"},
    {"homme_age_couleur3", "1F474-1F3FD"},
    {"homme_age_couleur4", "1F474-1F3FE"},
    {"homme_age_couleur5", "1F474-1F3FF"},
    {"femme_agee", "1F475"},
    {"femme_agee_couleur1", "1F475-1F3FB"},
    {"femme_agee_couleur2", "1F475-1F3FC"},
    {"femme_agee_couleur3", "1F475-1F3FD"},
    {"femme_agee_couleur4", "1F475-1F3FE"},
    {"femme_agee_couleur5", "1F475-1F3FF"},
    {"bebe", "1F476"},
    {"bebe_couleur1", "1F476-1F3FB"},
    {"bebe_couleur2", "1F476-1F3FC"},
    {"bebe_couleur3", "1F476-1F3FD"},
    {"bebe_couleur4", "1F476-1F3FE"},
    {"bebe_couleur5", "1F476-1F3FF"},
    {"personnel_du_batiment", "1F477"},
    {"personnel_du_batiment_couleur1", "1F477-1F3FB"},
    {"personnel_du_batiment_couleur2", "1F477-1F3FC"},
    {"personnel_du_batiment_couleur3", "1F477-1F3FD"},
    {"personnel_du_batiment_couleur4", "1F477-1F3FE"},
    {"personnel_du_batiment_couleur5", "1F477-1F3FF"},
    {"princesse", "1F478"},
    {"princesse_couleur1", "1F478-1F3FB"},
    {"princesse_couleur2", "1F478-1F3FC"},
    {"princesse_couleur3", "1F478-1F3FD"},
    {"princesse_couleur4", "1F478-1F3FE"},
    {"princesse_couleur5", "1F478-1F3FF"},
    {"ogre", "1F479"},
    {"monstre_japonais", "1F47A"},
    {"fantome", "1F47B"},
    {"bebe_ange", "1F47C"},
    {"bebe_ange_couleur1", "1F47C-1F3FB"},
    {"bebe_ange_couleur2", "1F47C-1F3FC"},
    {"bebe_ange_couleur3", "1F47C-1F3FD"},
    {"bebe_ange_couleur4", "1F47C-1F3FE"},
    {"bebe_ange_couleur5", "1F47C-1F3FF"},
    {"alien", "1F47D"},
    {"monstre_de_lespace", "1F47E"},
    {"visage_en_colere_avec_des_cornes", "1F47F"},
    {"crane", "1F480"},
    {"personne_paume_vers_le_haut", "1F481"},
    {"personne_paume_vers_le_haut_couleur1", "1F481-1F3FB"},
    {"personne_paume_vers_le_haut_couleur2", "1F481-1F3FC"},
    {"personne_paume_vers_le_haut_couleur3", "1F481-1F3FD"},
    {"personne_paume_vers_le_haut_couleur4", "1F481-1F3FE"},
    {"personne_paume_vers_le_haut_couleur5", "1F481-1F3FF"},
    {"garde", "1F482"},
    {"garde_couleur1", "1F482-1F3FB"},
    {"garde_couleur2", "1F482-1F3FC"},
    {"garde_couleur3", "1F482-1F3FD"},
    {"garde_couleur4", "1F482-1F3FE"},
    {"garde_couleur5", "1F482-1F3FF"},
    {"danseuse", "1F483"},
    {"danseuse_couleur1", "1F483-1F3FB"},
    {"danseuse_couleur2", "1F483-1F3FC"},
    {"danseuse_couleur3", "1F483-1F3FD"},
    {"danseuse_couleur4", "1F483-1F3FE"},
    {"danseuse_couleur5", "1F483-1F3FF"},
    {"rouge_a_levres", "1F484"},
    {"vernis_a_ongles", "1F485"},
    {"vernis_a_ongles_couleur1", "1F485-1F3FB"},
    {"vernis_a_ongles_couleur2", "1F485-1F3FC"},
    {"vernis_a_ongles_couleur3", "1F485-1F3FD"},
    {"vernis_a_ongles_couleur4", "1F485-1F3FE"},
    {"vernis_a_ongles_couleur5", "1F485-1F3FF"},
    {"personne_qui_se_fait_masser", "1F486"},
    {"personne_qui_se_fait_masser_couleur1", "1F486-1F3FB"},
    {"personne_qui_se_fait_masser_couleur2", "1F486-1F3FC"},
    {"personne_qui_se_fait_masser_couleur3", "1F486-1F3FD"},
    {"personne_qui_se_fait_masser_couleur4", "1F486-1F3FE"},
    {"personne_qui_se_fait_masser_couleur5", "1F486-1F3FF"},
    {"personne_qui_se_fait_couper_les_cheveux", "1F487"},
    {"personne_qui_se_fait_couper_les_cheveux_couleur1", "1F487-1F3FB"},
    {"personne_qui_se_fait_couper_les_cheveux_couleur2", "1F487-1F3FC"},
    {"personne_qui_se_fait_couper_les_cheveux_couleur3", "1F487-1F3FD"},
    {"personne_qui_se_fait_couper_les_cheveux_couleur4", "1F487-1F3FE"},
    {"personne_qui_se_fait_couper_les_cheveux_couleur5", "1F487-1F3FF"},
    {"enseigne_de_barbier", "1F488"},
    {"seringue", "1F489"},
    {"pilule", "1F48A"},
    {"trace_de_rouge_a_levres", "1F48B"},
    {"lettre_damour", "1F48C"},
    {"bague", "1F48D"},
    {"pierre_precieuse", "1F48E"},
    {"bisou", "1F48F"},
    {"bisou_couleur1", "1F48F-1F3FB"},
    {"bisou_couleur2", "1F48F-1F3FC"},
    {"bisou_couleur3", "1F48F-1F3FD"},
    {"bisou_couleur4", "1F48F-1F3FE"},
    {"bisou_couleur5", "1F48F-1F3FF"},
    {"bisou_couleur1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bisou_couleur1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bisou_couleur1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bisou_couleur1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bisou_couleur2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bisou_couleur2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bisou_couleur2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bisou_couleur2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bisou_couleur3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bisou_couleur3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bisou_couleur3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bisou_couleur3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bisou_couleur4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bisou_couleur4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bisou_couleur4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bisou_couleur4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bisou_couleur5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bisou_couleur5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bisou_couleur5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bisou_couleur5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bouquet", "1F490"},
    {"couple_avec_coeur", "1F491"},
    {"couple_avec_coeur_couleur1", "1F491-1F3FB"},
    {"couple_avec_coeur_couleur2", "1F491-1F3FC"},
    {"couple_avec_coeur_couleur3", "1F491-1F3FD"},
    {"couple_avec_coeur_couleur4", "1F491-1F3FE"},
    {"couple_avec_coeur_couleur5", "1F491-1F3FF"},
    {"couple_avec_coeur_couleur1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"couple_avec_coeur_couleur1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"couple_avec_coeur_couleur1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"couple_avec_coeur_couleur1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"couple_avec_coeur_couleur2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"couple_avec_coeur_couleur2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"couple_avec_coeur_couleur2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"couple_avec_coeur_couleur2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"couple_avec_coeur_couleur3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"couple_avec_coeur_couleur3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"couple_avec_coeur_couleur3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"couple_avec_coeur_couleur3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"couple_avec_coeur_couleur4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"couple_avec_coeur_couleur4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"couple_avec_coeur_couleur4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"couple_avec_coeur_couleur4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"couple_avec_coeur_couleur5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"couple_avec_coeur_couleur5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"couple_avec_coeur_couleur5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"couple_avec_coeur_couleur5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"mariage", "1F492"},
    {"coeur_battant", "1F493"},
    {"coeur_brise", "1F494"},
    {"deux_coeurs", "1F495"},
    {"coeur_etincelant", "1F496"},
    {"coeur_grandissant", "1F497"},
    {"coeur_et_fleche", "1F498"},
    {"coeur_bleu", "1F499"},
    {"coeur_vert", "1F49A"},
    {"coeur_jaune", "1F49B"},
    {"coeur_violet", "1F49C"},
    {"coeur_avec_ruban", "1F49D"},
    {"coeurs_qui_tournent", "1F49E"},
    {"decoration_avec_coeur", "1F49F"},
    {"diamant_avec_un_point", "1F4A0"},
    {"ampoule", "1F4A1"},
    {"symbole_de_colere", "1F4A2"},
    {"bombe", "1F4A3"},
    {"endormi", "1F4A4"},
    {"explosion", "1F4A5"},
    {"gouttes_de_sueur", "1F4A6"},
    {"goutte_deau", "1F4A7"},
    {"decamper", "1F4A8"},
    {"tas_de_crotte", "1F4A9"},
    {"biceps_contracte", "1F4AA"},
    {"biceps_contracte_couleur1", "1F4AA-1F3FB"},
    {"biceps_contracte_couleur2", "1F4AA-1F3FC"},
    {"biceps_contracte_couleur3", "1F4AA-1F3FD"},
    {"biceps_contracte_couleur4", "1F4AA-1F3FE"},
    {"biceps_contracte_couleur5", "1F4AA-1F3FF"},
    {"etourdissement", "1F4AB"},
    {"bulle_de_parole", "1F4AC"},
    {"bulle_de_pensee", "1F4AD"},
    {"fleur_blanche", "1F4AE"},
    {"cent_points", "1F4AF"},
    {"sac_plein_dargent", "1F4B0"},
    {"conversion_de_devise", "1F4B1"},
    {"symbole_dollar", "1F4B2"},
    {"carte_bancaire", "1F4B3"},
    {"billet_en_yens", "1F4B4"},
    {"billet_en_dollars", "1F4B5"},
    {"billet_en_euros", "1F4B6"},
    {"billet_en_livres", "1F4B7"},
    {"billet_avec_des_ailes", "1F4B8"},
    {"courbe_avec_yen_en_hausse", "1F4B9"},
    {"siege", "1F4BA"},
    {"ordinateur_portable", "1F4BB"},
    {"porte_documents", "1F4BC"},
    {"disque_dordinateur", "1F4BD"},
    {"disquette", "1F4BE"},
    {"cd", "1F4BF"},
    {"dvd", "1F4C0"},
    {"dossier", "1F4C1"},
    {"dossier_ouvert", "1F4C2"},
    {"page_enroulee", "1F4C3"},
    {"page", "1F4C4"},
    {"calendrier", "1F4C5"},
    {"ephemeride", "1F4C6"},
    {"carnet_dadresses", "1F4C7"},
    {"graphique_en_hausse", "1F4C8"},
    {"graphique_en_baisse", "1F4C9"},
    {"graphiques_a_barres", "1F4CA"},
    {"porte_bloc", "1F4CB"},
    {"punaise", "1F4CC"},
    {"epingle", "1F4CD"},
    {"trombone", "1FA8A"},
    {"regle", "1F4CF"},
    {"equerre", "1F4D0"},
    {"signets", "1F4D1"},
    {"carnet_de_compte", "1F4D2"},
    {"carnet", "1F4D3"},
    {"carnet_avec_couverture", "1F4D4"},
    {"livre_ferme", "1F4D5"},
    {"livre_ouvert", "1F4D6"},
    {"livre_vert", "1F4D7"},
    {"livre_bleu", "1F4D8"},
    {"livre_orange", "1F4D9"},
    {"livres", "1F4DA"},
    {"badge_nominatif", "1F4DB"},
    {"parchemin", "1F4DC"},
    {"memo", "1F4DD"},
    {"combine_telephonique", "1F4DE"},
    {"bipeur", "1F4DF"},
    {"fax", "1F4E0"},
    {"antenne_satellite", "1F4E1"},
    {"haut_parleur", "1F4E2"},
    {"porte_voix", "1F4E3"},
    {"boite_denvoi", "1F4E4"},
    {"boite_de_reception", "1F4E5"},
    {"colis", "1F4E6"},
    {"email", "1F4E7"},
    {"message_recu", "1F4E8"},
    {"enveloppe_avec_fleche", "1F4E9"},
    {"boite_aux_lettres_fermee_drapeau_baisse", "1F4EA"},
    {"boite_aux_lettres_fermee_drapeau_leve", "1F4EB"},
    {"boite_aux_lettres_ouverte_drapeau_leve", "1F4EC"},
    {"boite_aux_lettres_ouverte_drapeau_baisse", "1F4ED"},
    {"boite_aux_lettres", "1F4EE"},
    {"cor_postal", "1F4EF"},
    {"journal", "1F4F0"},
    {"telephone_portable", "1F4F1"},
    {"appel_entrant", "1F4F2"},
    {"mode_vibreur", "1F4F3"},
    {"telephone_eteint", "1F4F4"},
    {"telephones_portables_interdits", "1F4F5"},
    {"barres_de_reseau", "1F4F6"},
    {"appareil_photo", "1F4F7"},
    {"appareil_photo_avec_flash", "1F4F8"},
    {"camescope", "1F4F9"},
    {"televiseur", "1F4FA"},
    {"radio", "1F4FB"},
    {"cassette_video", "1F4FC"},
    {"projecteur_cinematographique", "1F4FD"},
    {"chapelet", "1F4FF"},
    {"bouton_lecture_aleatoire", "1F500"},
    {"bouton_repetition", "1F501"},
    {"bouton_repetition_de_la_piste", "1F502"},
    {"fleches_dans_le_sens_horaire", "1F503"},
    {"fleches_dans_le_sens_antihoraire", "1F504"},
    {"luminosite_faible", "1F505"},
    {"luminosite_elevee", "1F506"},
    {"muet", "1F507"},
    {"volume_des_enceintes_faible", "1F508"},
    {"volume_des_enceintes_moyen", "1F509"},
    {"volume_des_enceintes_eleve", "1F50A"},
    {"pile", "1F50B"},
    {"cable_avec_fiche_electrique", "1F50C"},
    {"loupe_orientee_a_gauche", "1F50D"},
    {"loupe_orientee_a_droite", "1F50E"},
    {"cadenas_ferme_avec_stylo", "1F50F"},
    {"cadenas_ferme_avec_cle", "1F510"},
    {"cle", "1F511"},
    {"cadenas_ferme", "1F512"},
    {"cadenas_ouvert", "1F513"},
    {"cloche", "1F514"},
    {"alarme_desactivee", "1F515"},
    {"marque_page", "1F516"},
    {"chainons", "1F517"},
    {"bouton_radio", "1F518"},
    {"fleche_retour", "1F519"},
    {"fleche_fin", "1F51A"},
    {"fleche_active", "1F51B"},
    {"fleche_bientot", "1F51C"},
    {"fleche_en_haut", "1F51D"},
    {"18_ans_et_plus", "1F51E"},
    {"touches_10", "1F51F"},
    {"majuscules", "1F520"},
    {"minuscules", "1F521"},
    {"saisie_de_chiffres", "1F522"},
    {"saisie_de_symboles", "1F523"},
    {"alphabet_latin", "1F524"},
    {"feu", "1F525"},
    {"torche", "1F526"},
    {"cle_a_molette", "1F527"},
    {"marteau", "1F528"},
    {"vis_et_ecrou", "1F529"},
    {"couteau_de_cuisine", "1F52A"},
    {"pistolet_a_eau", "1F52B"},
    {"microscope", "1F52C"},
    {"telescope", "1F52D"},
    {"boule_de_cristal", "1F52E"},
    {"etoile_a_6_branches", "1F52F"},
    {"symbole_japonais_de_debutant", "1F530"},
    {"trident", "1F531"},
    {"carre_noir", "1F532"},
    {"carre_blanc", "1F533"},
    {"disque_rouge", "1F534"},
    {"disque_bleu", "1F535"},
    {"grand_losange_orange", "1F536"},
    {"grand_losange_bleu", "1F537"},
    {"petit_losange_orange", "1F538"},
    {"petit_losange_bleu", "1F539"},
    {"triangle_rouge_pointant_vers_le_haut", "1F53A"},
    {"triangle_rouge_pointant_vers_le_bas", "1F53B"},
    {"petit_triangle_haut", "1F53C"},
    {"petit_triangle_bas", "1F53D"},
    {"om", "1F549"},
    {"colombe", "1F54A"},
    {"kaaba", "1F54B"},
    {"mosquee", "1F54C"},
    {"synagogue", "1F54D"},
    {"chandelier_a_sept_branches", "1F54E"},
    {"une_heure", "1F550"},
    {"deux_heures", "1F551"},
    {"trois_heures", "1F552"},
    {"quatre_heures", "1F553"},
    {"cinq_heures", "1F554"},
    {"six_heures", "1F555"},
    {"sept_heures", "1F556"},
    {"huit_heures", "1F557"},
    {"neuf_heures", "1F558"},
    {"dix_heures", "1F559"},
    {"onze_heures", "1F55A"},
    {"midi_minuit", "1F55B"},
    {"une_heure_et_demie", "1F55C"},
    {"deux_heures_et_demie", "1F55D"},
    {"trois_heures_et_demie", "1F55E"},
    {"quatre_heures_et_demie", "1F55F"},
    {"cinq_heures_et_demie", "1F560"},
    {"six_heures_et_demie", "1F561"},
    {"sept_heures_et_demie", "1F562"},
    {"huit_heures_et_demie", "1F563"},
    {"neuf_heures_et_demie", "1F564"},
    {"dix_heures_et_demie", "1F565"},
    {"onze_heures_et_demie", "1F566"},
    {"midi_minuit_et_demie", "1F567"},
    {"bougie", "1F56F"},
    {"pendule", "1F570"},
    {"trou", "1F573"},
    {"homme_daffaires_en_levitation", "1F574"},
    {"homme_daffaires_en_levitation_couleur1", "1F574-1F3FB"},
    {"homme_daffaires_en_levitation_couleur2", "1F574-1F3FC"},
    {"homme_daffaires_en_levitation_couleur3", "1F574-1F3FD"},
    {"homme_daffaires_en_levitation_couleur4", "1F574-1F3FE"},
    {"homme_daffaires_en_levitation_couleur5", "1F574-1F3FF"},
    {"detective", "1F575"},
    {"detective_couleur1", "1F575-1F3FB"},
    {"detective_couleur2", "1F575-1F3FC"},
    {"detective_couleur3", "1F575-1F3FD"},
    {"detective_couleur4", "1F575-1F3FE"},
    {"detective_couleur5", "1F575-1F3FF"},
    {"lunettes_de_soleil", "1F576"},
    {"araignee", "1F577"},
    {"toile_daraignee", "1F578"},
    {"manette_de_jeu", "1F579"},
    {"danseur", "1F57A"},
    {"danseur_couleur1", "1F57A-1F3FB"},
    {"danseur_couleur2", "1F57A-1F3FC"},
    {"danseur_couleur3", "1F57A-1F3FD"},
    {"danseur_couleur4", "1F57A-1F3FE"},
    {"danseur_couleur5", "1F57A-1F3FF"},
    {"trombones", "1F587"},
    {"stylo", "1F58A"},
    {"stylo_plume", "1F58B"},
    {"pinceau", "1F58C"},
    {"crayon_pastel", "1F58D"},
    {"main_levee_doigts_ecartes", "1F590"},
    {"main_levee_doigts_ecartes_couleur1", "1F590-1F3FB"},
    {"main_levee_doigts_ecartes_couleur2", "1F590-1F3FC"},
    {"main_levee_doigts_ecartes_couleur3", "1F590-1F3FD"},
    {"main_levee_doigts_ecartes_couleur4", "1F590-1F3FE"},
    {"main_levee_doigts_ecartes_couleur5", "1F590-1F3FF"},
    {"doigt_dhonneur", "1F595"},
    {"doigt_dhonneur_couleur1", "1F595-1F3FB"},
    {"doigt_dhonneur_couleur2", "1F595-1F3FC"},
    {"doigt_dhonneur_couleur3", "1F595-1F3FD"},
    {"doigt_dhonneur_couleur4", "1F595-1F3FE"},
    {"doigt_dhonneur_couleur5", "1F595-1F3FF"},
    {"salut_vulcain", "1F596"},
    {"salut_vulcain_couleur1", "1F596-1F3FB"},
    {"salut_vulcain_couleur2", "1F596-1F3FC"},
    {"salut_vulcain_couleur3", "1F596-1F3FD"},
    {"salut_vulcain_couleur4", "1F596-1F3FE"},
    {"salut_vulcain_couleur5", "1F596-1F3FF"},
    {"coeur_noir", "1F5A4"},
    {"ordinateur_de_bureau", "1F5A5"},
    {"imprimante", "1F5A8"},
    {"souris_dordinateur", "1F5B1"},
    {"boule_de_commande", "1F5B2"},
    {"cadre_avec_image", "1F5BC"},
    {"intercalaires", "1F5C2"},
    {"boite_a_dossiers", "1F5C3"},
    {"meuble_a_dossiers", "1F5C4"},
    {"corbeille_a_papiers", "1F5D1"},
    {"bloc_notes_a_spirale", "1F5D2"},
    {"calendrier_a_spirale", "1F5D3"},
    {"serre_joint", "1F5DC"},
    {"cle_ancienne", "1F5DD"},
    {"journal_roule", "1F5DE"},
    {"dague", "1F5E1"},
    {"tete_qui_parle", "1F5E3"},
    {"bulle_de_parole_gauche", "1F5E8"},
    {"bulle_colere_droite", "1F5EF"},
    {"urne_electorale", "1F5F3"},
    {"carte_du_monde", "1F5FA"},
    {"mont_fuji", "1F5FB"},
    {"tour_de_tokyo", "1F5FC"},
    {"statue_de_la_liberte", "1F5FD"},
    {"carte_du_japon", "1F5FE"},
    {"moai", "1F5FF"},
    {"visage_rieur", "1F600"},
    {"visage_souriant_aux_yeux_rieurs", "1F601"},
    {"visage_riant_aux_larmes", "1F602"},
    {"visage_souriant_avec_de_grands_yeux", "1F603"},
    {"visage_tres_souriant_aux_yeux_rieurs", "1F604"},
    {"visage_souriant_avec_une_goutte_de_sueur", "1F605"},
    {"visage_souriant_avec_yeux_plisses", "1F606"},
    {"visage_souriant_avec_aureole", "1F607"},
    {"visage_souriant_avec_des_cornes", "1F608"},
    {"visage_faisant_un_clin_doeil", "1F609"},
    {"visage_souriant_avec_yeux_rieurs", "1F60A"},
    {"miam", "1F60B"},
    {"visage_soulage", "1F60C"},
    {"visage_souriant_avec_yeux_en_forme_de_coeur", "1F60D"},
    {"visage_avec_lunettes_de_soleil", "1F60E"},
    {"visage_avec_un_sourire_malin", "1F60F"},
    {"visage_neutre", "1F610"},
    {"visage_sans_expression", "1F611"},
    {"visage_blase", "1F612"},
    {"visage_demoralise_avec_goutte_de_sueur", "1F613"},
    {"visage_pensif", "1F614"},
    {"visage_confus", "1F615"},
    {"visage_deconcerte", "1F616"},
    {"visage_faisant_un_bisou", "1F617"},
    {"visage_envoyant_un_bisou", "1F618"},
    {"visage_aux_yeux_rieurs_faisant_un_bisou", "1F619"},
    {"visage_faisant_un_bisou_avec_les_yeux_fermes", "1F61A"},
    {"visage_qui_tire_la_langue", "1F61B"},
    {"visage_qui_tire_la_langue_et_fait_un_clin_doeil", "1F61C"},
    {"visage_qui_tire_la_langue_les_yeux_plisses", "1F61D"},
    {"visage_decu", "1F61E"},
    {"visage_inquiet", "1F61F"},
    {"visage_en_colere", "1F620"},
    {"visage_boudeur", "1F621"},
    {"visage_qui_pleure", "1F622"},
    {"visage_perseverant", "1F623"},
    {"visage_avec_fumee_sortant_des_narines", "1F624"},
    {"visage_triste_mais_soulage", "1F625"},
    {"visage_mecontent_avec_bouche_ouverte", "1F626"},
    {"visage_angoisse", "1F627"},
    {"visage_effraye", "1F628"},
    {"visage_epuise", "1F629"},
    {"visage_endormi", "1F62A"},
    {"visage_fatigue", "1F62B"},
    {"visage_grimacant", "1F62C"},
    {"visage_qui_pleure_a_chaudes_larmes", "1F62D"},
    {"visage_avec_bouche_ouverte", "1F62E"},
    {"visage_ebahi", "1F62F"},
    {"visage_anxieux_avec_goutte_de_sueur", "1F630"},
    {"visage_qui_hurle_de_peur", "1F631"},
    {"visage_stupefait", "1F632"},
    {"visage_qui_rougit", "1F633"},
    {"visage_somnolent", "1F634"},
    {"visage_etourdi", "1F635"},
    {"visage_sans_bouche", "1F636"},
    {"visage_avec_masque", "1F637"},
    {"chat_qui_sourit_avec_des_yeux_rieurs", "1F638"},
    {"chat_qui_pleure_de_joie", "1F639"},
    {"chat_qui_sourit", "1F63A"},
    {"chat_souriant_aux_yeux_en_coeurs", "1F63B"},
    {"chat_avec_sourire_en_coin", "1F63C"},
    {"chat_qui_fait_un_bisou", "1F63D"},
    {"chat_qui_boude", "1F63E"},
    {"chat_qui_pleure", "1F63F"},
    {"chat_fatigue", "1F640"},
    {"visage_legerement_mecontent", "1F641"},
    {"visage_avec_un_leger_sourire", "1F642"},
    {"tete_a_lenvers", "1F643"},
    {"visage_roulant_des_yeux", "1F644"},
    {"personne_faisant_un_geste_dinterdiction", "1F645"},
    {"personne_faisant_un_geste_dinterdiction_couleur1", "1F645-1F3FB"},
    {"personne_faisant_un_geste_dinterdiction_couleur2", "1F645-1F3FC"},
    {"personne_faisant_un_geste_dinterdiction_couleur3", "1F645-1F3FD"},
    {"personne_faisant_un_geste_dinterdiction_couleur4", "1F645-1F3FE"},
    {"personne_faisant_un_geste_dinterdiction_couleur5", "1F645-1F3FF"},
    {"personne_faisant_un_geste_dacceptation", "1F646"},
    {"personne_faisant_un_geste_dacceptation_couleur1", "1F646-1F3FB"},
    {"personne_faisant_un_geste_dacceptation_couleur2", "1F646-1F3FC"},
    {"personne_faisant_un_geste_dacceptation_couleur3", "1F646-1F3FD"},
    {"personne_faisant_un_geste_dacceptation_couleur4", "1F646-1F3FE"},
    {"personne_faisant_un_geste_dacceptation_couleur5", "1F646-1F3FF"},
    {"personne_qui_sincline", "1F647"},
    {"personne_qui_sincline_couleur1", "1F647-1F3FB"},
    {"personne_qui_sincline_couleur2", "1F647-1F3FC"},
    {"personne_qui_sincline_couleur3", "1F647-1F3FD"},
    {"personne_qui_sincline_couleur4", "1F647-1F3FE"},
    {"personne_qui_sincline_couleur5", "1F647-1F3FF"},
    {"singe_ne_rien_voir", "1F648"},
    {"singe_ne_rien_entendre", "1F649"},
    {"singe_ne_rien_dire", "1F64A"},
    {"personne_qui_leve_la_main", "1F64B"},
    {"personne_qui_leve_la_main_couleur1", "1F64B-1F3FB"},
    {"personne_qui_leve_la_main_couleur2", "1F64B-1F3FC"},
    {"personne_qui_leve_la_main_couleur3", "1F64B-1F3FD"},
    {"personne_qui_leve_la_main_couleur4", "1F64B-1F3FE"},
    {"personne_qui_leve_la_main_couleur5", "1F64B-1F3FF"},
    {"mains_levees", "1F64C"},
    {"mains_levees_couleur1", "1F64C-1F3FB"},
    {"mains_levees_couleur2", "1F64C-1F3FC"},
    {"mains_levees_couleur3", "1F64C-1F3FD"},
    {"mains_levees_couleur4", "1F64C-1F3FE"},
    {"mains_levees_couleur5", "1F64C-1F3FF"},
    {"personne_froncant_les_sourcils", "1F64D"},
    {"personne_froncant_les_sourcils_couleur1", "1F64D-1F3FB"},
    {"personne_froncant_les_sourcils_couleur2", "1F64D-1F3FC"},
    {"personne_froncant_les_sourcils_couleur3", "1F64D-1F3FD"},
    {"personne_froncant_les_sourcils_couleur4", "1F64D-1F3FE"},
    {"personne_froncant_les_sourcils_couleur5", "1F64D-1F3FF"},
    {"personne_qui_boude", "1F64E"},
    {"personne_qui_boude_couleur1", "1F64E-1F3FB"},
    {"personne_qui_boude_couleur2", "1F64E-1F3FC"},
    {"personne_qui_boude_couleur3", "1F64E-1F3FD"},
    {"personne_qui_boude_couleur4", "1F64E-1F3FE"},
    {"personne_qui_boude_couleur5", "1F64E-1F3FF"},
    {"mains_en_priere", "1F64F"},
    {"mains_en_priere_couleur1", "1F64F-1F3FB"},
    {"mains_en_priere_couleur2", "1F64F-1F3FC"},
    {"mains_en_priere_couleur3", "1F64F-1F3FD"},
    {"mains_en_priere_couleur4", "1F64F-1F3FE"},
    {"mains_en_priere_couleur5", "1F64F-1F3FF"},
    {"fusee", "1F680"},
    {"helicoptere", "1F681"},
    {"locomotive", "1F682"},
    {"wagon", "1F683"},
    {"tgv", "1F684"},
    {"train_a_grande_vitesse", "1F685"},
    {"train", "1F686"},
    {"metro", "1F687"},
    {"metro_leger", "1F688"},
    {"gare", "1F689"},
    {"tramway", "1F68A"},
    {"wagon_de_tramway", "1F68B"},
    {"bus", "1F68C"},
    {"bus_de_face", "1F68D"},
    {"trolleybus", "1F68E"},
    {"arret_de_bus", "1F68F"},
    {"minibus", "1F690"},
    {"ambulance", "1F691"},
    {"camion_de_pompier", "1F692"},
    {"voiture_de_police", "1F693"},
    {"voiture_de_police_de_face", "1F694"},
    {"taxi", "1F695"},
    {"taxi_de_face", "1F696"},
    {"voiture", "1F697"},
    {"voiture_de_face", "1F698"},
    {"vehicule_utilitaire_sport", "1F699"},
    {"camion_de_livraison", "1F69A"},
    {"semi_remorque", "1F69B"},
    {"tracteur", "1F69C"},
    {"monorail", "1F69D"},
    {"train_de_montagne", "1F69E"},
    {"train_suspendu", "1F69F"},
    {"telepherique", "1F6A0"},
    {"tramway_aerien", "1F6A1"},
    {"navire", "1F6A2"},
    {"personne_ramant_dans_une_barque", "1F6A3"},
    {"personne_ramant_dans_une_barque_couleur1", "1F6A3-1F3FB"},
    {"personne_ramant_dans_une_barque_couleur2", "1F6A3-1F3FC"},
    {"personne_ramant_dans_une_barque_couleur3", "1F6A3-1F3FD"},
    {"personne_ramant_dans_une_barque_couleur4", "1F6A3-1F3FE"},
    {"personne_ramant_dans_une_barque_couleur5", "1F6A3-1F3FF"},
    {"hors_bord", "1F6A4"},
    {"feu_tricolore_horizontal", "1F6A5"},
    {"feu_tricolore_vertical", "1F6A6"},
    {"travaux", "1F6A7"},
    {"gyrophare", "1F6A8"},
    {"drapeau_triangulaire", "1F6A9"},
    {"porte", "1F6AA"},
    {"symbole_dinterdiction", "1F6AB"},
    {"cigarette", "1F6AC"},
    {"interdiction_de_fumer", "1F6AD"},
    {"icone_poubelle", "1F6AE"},
    {"depot_dordures_interdit", "1F6AF"},
    {"eau_potable", "1F6B0"},
    {"eau_non_potable", "1F6B1"},
    {"velo", "1F6B2"},
    {"velos_interdits", "1F6B3"},
    {"cycliste", "1F6B4"},
    {"cycliste_couleur1", "1F6B4-1F3FB"},
    {"cycliste_couleur2", "1F6B4-1F3FC"},
    {"cycliste_couleur3", "1F6B4-1F3FD"},
    {"cycliste_couleur4", "1F6B4-1F3FE"},
    {"cycliste_couleur5", "1F6B4-1F3FF"},
    {"personne_en_vtt", "1F6B5"},
    {"personne_en_vtt_couleur1", "1F6B5-1F3FB"},
    {"personne_en_vtt_couleur2", "1F6B5-1F3FC"},
    {"personne_en_vtt_couleur3", "1F6B5-1F3FD"},
    {"personne_en_vtt_couleur4", "1F6B5-1F3FE"},
    {"personne_en_vtt_couleur5", "1F6B5-1F3FF"},
    {"personne_qui_marche", "1F6B6"},
    {"personne_qui_marche_couleur1", "1F6B6-1F3FB"},
    {"personne_qui_marche_couleur2", "1F6B6-1F3FC"},
    {"personne_qui_marche_couleur3", "1F6B6-1F3FD"},
    {"personne_qui_marche_couleur4", "1F6B6-1F3FE"},
    {"personne_qui_marche_couleur5", "1F6B6-1F3FF"},
    {"interdit_aux_pietons", "1F6B7"},
    {"traversee_denfants", "1F6B8"},
    {"symbole_toilettes_hommes", "1F6B9"},
    {"symbole_toilettes_femmes", "1F6BA"},
    {"panneau_toilettes", "1F6BB"},
    {"symbole_bebe", "1F6BC"},
    {"toilettes", "1F6BD"},
    {"wc", "1F6BE"},
    {"douche", "1F6BF"},
    {"personne_prenant_un_bain", "1F6C0"},
    {"personne_prenant_un_bain_couleur1", "1F6C0-1F3FB"},
    {"personne_prenant_un_bain_couleur2", "1F6C0-1F3FC"},
    {"personne_prenant_un_bain_couleur3", "1F6C0-1F3FD"},
    {"personne_prenant_un_bain_couleur4", "1F6C0-1F3FE"},
    {"personne_prenant_un_bain_couleur5", "1F6C0-1F3FF"},
    {"baignoire", "1F6C1"},
    {"controle_des_passeports", "1F6C2"},
    {"douane", "1F6C3"},
    {"retrait_des_bagages", "1F6C4"},
    {"consigne", "1F6C5"},
    {"canape_et_lampe", "1F6CB"},
    {"personne_au_lit", "1F6CC"},
    {"personne_au_lit_couleur1", "1F6CC-1F3FB"},
    {"personne_au_lit_couleur2", "1F6CC-1F3FC"},
    {"personne_au_lit_couleur3", "1F6CC-1F3FD"},
    {"personne_au_lit_couleur4", "1F6CC-1F3FE"},
    {"personne_au_lit_couleur5", "1F6CC-1F3FF"},
    {"sacs_de_shopping", "1F6CD"},
    {"cloche_de_comptoir", "1F6CE"},
    {"lit", "1F6CF"},
    {"lieu_de_culte", "1F6D0"},
    {"stop", "1F6D1"},
    {"chariot", "1F6D2"},
    {"temple_hindou", "1F6D5"},
    {"hutte", "1F6D6"},
    {"ascenseur", "1F6D7"},
    {"eboulement", "1F6D8"},
    {"sans_fil", "1F6DC"},
    {"toboggan", "1F6DD"},
    {"roue", "1F6DE"},
    {"bouee_de_sauvetage", "1F6DF"},
    {"marteau_et_cle_a_molette", "1F6E0"},
    {"bouclier", "1F6E1"},
    {"baril_de_petrole", "1F6E2"},
    {"autoroute", "1F6E3"},
    {"voie_ferree", "1F6E4"},
    {"bateau_a_moteur", "1F6E5"},
    {"petit_avion", "1F6E9"},
    {"avion_au_decollage", "1F6EB"},
    {"avion_a_latterrissage", "1F6EC"},
    {"satellite", "1F6F0"},
    {"paquebot", "1F6F3"},
    {"trottinette", "1F6F4"},
    {"scooter", "1F6F5"},
    {"canoe", "1F6F6"},
    {"luge", "1F6F7"},
    {"soucoupe_volante", "1F6F8"},
    {"planche_a_roulettes", "1F6F9"},
    {"tuk_tuk", "1F6FA"},
    {"pick_up", "1F6FB"},
    {"patin_a_roulettes", "1F6FC"},
    {"disque_orange", "1F7E0"},
    {"disque_jaune", "1F7E1"},
    {"disque_vert", "1F7E2"},
    {"disque_violet", "1F7E3"},
    {"disque_marron", "1F7E4"},
    {"carre_rouge", "1F7E5"},
    {"carre_bleu", "1F7E6"},
    {"carre_orange", "1F7E7"},
    {"carre_jaune", "1F7E8"},
    {"carre_vert", "1F7E9"},
    {"carre_violet", "1F7EA"},
    {"carre_marron", "1F7EB"},
    {"signe_egal_gras", "1F7F0"},
    {"bout_des_doigts_joints", "1F90C"},
    {"bout_des_doigts_joints_couleur1", "1F90C-1F3FB"},
    {"bout_des_doigts_joints_couleur2", "1F90C-1F3FC"},
    {"bout_des_doigts_joints_couleur3", "1F90C-1F3FD"},
    {"bout_des_doigts_joints_couleur4", "1F90C-1F3FE"},
    {"bout_des_doigts_joints_couleur5", "1F90C-1F3FF"},
    {"coeur_blanc", "1F90D"},
    {"coeur_marron", "1F90E"},
    {"pouce_et_index_rapproches", "1F90F"},
    {"pouce_et_index_rapproches_couleur1", "1F90F-1F3FB"},
    {"pouce_et_index_rapproches_couleur2", "1F90F-1F3FC"},
    {"pouce_et_index_rapproches_couleur3", "1F90F-1F3FD"},
    {"pouce_et_index_rapproches_couleur4", "1F90F-1F3FE"},
    {"pouce_et_index_rapproches_couleur5", "1F90F-1F3FF"},
    {"visage_avec_bouche_fermeture_eclair", "1F910"},
    {"argent_dans_les_yeux_et_la_bouche", "1F911"},
    {"visage_avec_thermometre", "1F912"},
    {"visage_de_geek", "1F913"},
    {"visage_en_pleine_reflexion", "1F914"},
    {"visage_avec_bandage_autour_de_la_tete", "1F915"},
    {"robot", "1F916"},
    {"visage_qui_fait_un_calin", "1F917"},
    {"cornes_avec_les_doigts", "1F918"},
    {"cornes_avec_les_doigts_couleur1", "1F918-1F3FB"},
    {"cornes_avec_les_doigts_couleur2", "1F918-1F3FC"},
    {"cornes_avec_les_doigts_couleur3", "1F918-1F3FD"},
    {"cornes_avec_les_doigts_couleur4", "1F918-1F3FE"},
    {"cornes_avec_les_doigts_couleur5", "1F918-1F3FF"},
    {"signe_appel_telephonique_avec_les_doigts", "1F919"},
    {"signe_appel_telephonique_avec_les_doigts_couleur1", "1F919-1F3FB"},
    {"signe_appel_telephonique_avec_les_doigts_couleur2", "1F919-1F3FC"},
    {"signe_appel_telephonique_avec_les_doigts_couleur3", "1F919-1F3FD"},
    {"signe_appel_telephonique_avec_les_doigts_couleur4", "1F919-1F3FE"},
    {"signe_appel_telephonique_avec_les_doigts_couleur5", "1F919-1F3FF"},
    {"dos_de_main_levee", "1F91A"},
    {"dos_de_main_levee_couleur1", "1F91A-1F3FB"},
    {"dos_de_main_levee_couleur2", "1F91A-1F3FC"},
    {"dos_de_main_levee_couleur3", "1F91A-1F3FD"},
    {"dos_de_main_levee_couleur4", "1F91A-1F3FE"},
    {"dos_de_main_levee_couleur5", "1F91A-1F3FF"},
    {"poing_a_gauche", "1F91B"},
    {"poing_a_gauche_couleur1", "1F91B-1F3FB"},
    {"poing_a_gauche_couleur2", "1F91B-1F3FC"},
    {"poing_a_gauche_couleur3", "1F91B-1F3FD"},
    {"poing_a_gauche_couleur4", "1F91B-1F3FE"},
    {"poing_a_gauche_couleur5", "1F91B-1F3FF"},
    {"poing_a_droite", "1F91C"},
    {"poing_a_droite_couleur1", "1F91C-1F3FB"},
    {"poing_a_droite_couleur2", "1F91C-1F3FC"},
    {"poing_a_droite_couleur3", "1F91C-1F3FD"},
    {"poing_a_droite_couleur4", "1F91C-1F3FE"},
    {"poing_a_droite_couleur5", "1F91C-1F3FF"},
    {"poignee_de_main", "1F91D"},
    {"poignee_de_main_couleur1", "1F91D-1F3FB"},
    {"poignee_de_main_couleur2", "1F91D-1F3FC"},
    {"poignee_de_main_couleur3", "1F91D-1F3FD"},
    {"poignee_de_main_couleur4", "1F91D-1F3FE"},
    {"poignee_de_main_couleur5", "1F91D-1F3FF"},
    {"poignee_de_main_couleur1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"poignee_de_main_couleur1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"poignee_de_main_couleur1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"poignee_de_main_couleur1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"poignee_de_main_couleur2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"poignee_de_main_couleur2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"poignee_de_main_couleur2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"poignee_de_main_couleur2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"poignee_de_main_couleur3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"poignee_de_main_couleur3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"poignee_de_main_couleur3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"poignee_de_main_couleur3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"poignee_de_main_couleur4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"poignee_de_main_couleur4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"poignee_de_main_couleur4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"poignee_de_main_couleur4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"poignee_de_main_couleur5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"poignee_de_main_couleur5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"poignee_de_main_couleur5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"poignee_de_main_couleur5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"doigts_croises", "1F91E"},
    {"doigts_croises_couleur1", "1F91E-1F3FB"},
    {"doigts_croises_couleur2", "1F91E-1F3FC"},
    {"doigts_croises_couleur3", "1F91E-1F3FD"},
    {"doigts_croises_couleur4", "1F91E-1F3FE"},
    {"doigts_croises_couleur5", "1F91E-1F3FF"},
    {"signe_je_taime", "1F91F"},
    {"signe_je_taime_couleur1", "1F91F-1F3FB"},
    {"signe_je_taime_couleur2", "1F91F-1F3FC"},
    {"signe_je_taime_couleur3", "1F91F-1F3FD"},
    {"signe_je_taime_couleur4", "1F91F-1F3FE"},
    {"signe_je_taime_couleur5", "1F91F-1F3FF"},
    {"visage_avec_chapeau_de_cowboy", "1F920"},
    {"visage_de_clown", "1F921"},
    {"visage_nauseeux", "1F922"},
    {"se_rouler_par_terre_de_rire", "1F923"},
    {"visage_qui_bave", "1F924"},
    {"visage_de_menteur", "1F925"},
    {"personne_avec_la_paume_sur_le_visage", "1F926"},
    {"personne_avec_la_paume_sur_le_visage_couleur1", "1F926-1F3FB"},
    {"personne_avec_la_paume_sur_le_visage_couleur2", "1F926-1F3FC"},
    {"personne_avec_la_paume_sur_le_visage_couleur3", "1F926-1F3FD"},
    {"personne_avec_la_paume_sur_le_visage_couleur4", "1F926-1F3FE"},
    {"personne_avec_la_paume_sur_le_visage_couleur5", "1F926-1F3FF"},
    {"visage_qui_eternue", "1F927"},
    {"visage_avec_les_sourcils_releves", "1F928"},
    {"visage_avec_des_etoiles_a_la_place_des_yeux", "1F929"},
    {"tete_de_fou", "1F92A"},
    {"visage_avec_un_doigt_sur_la_bouche", "1F92B"},
    {"visage_avec_des_symboles_dans_la_bouche", "1F92C"},
    {"visage_avec_une_main_sur_la_bouche", "1F92D"},
    {"visage_qui_vomit", "1F92E"},
    {"tete_qui_explose", "1F92F"},
    {"femme_enceinte", "1F930"},
    {"femme_enceinte_couleur1", "1F930-1F3FB"},
    {"femme_enceinte_couleur2", "1F930-1F3FC"},
    {"femme_enceinte_couleur3", "1F930-1F3FD"},
    {"femme_enceinte_couleur4", "1F930-1F3FE"},
    {"femme_enceinte_couleur5", "1F930-1F3FF"},
    {"allaitement", "1F931"},
    {"allaitement_couleur1", "1F931-1F3FB"},
    {"allaitement_couleur2", "1F931-1F3FC"},
    {"allaitement_couleur3", "1F931-1F3FD"},
    {"allaitement_couleur4", "1F931-1F3FE"},
    {"allaitement_couleur5", "1F931-1F3FF"},
    {"paume_contre_paume_doigts_vers_le_haut", "1F932"},
    {"paume_contre_paume_doigts_vers_le_haut_couleur1", "1F932-1F3FB"},
    {"paume_contre_paume_doigts_vers_le_haut_couleur2", "1F932-1F3FC"},
    {"paume_contre_paume_doigts_vers_le_haut_couleur3", "1F932-1F3FD"},
    {"paume_contre_paume_doigts_vers_le_haut_couleur4", "1F932-1F3FE"},
    {"paume_contre_paume_doigts_vers_le_haut_couleur5", "1F932-1F3FF"},
    {"selfie", "1F933"},
    {"selfie_couleur1", "1F933-1F3FB"},
    {"selfie_couleur2", "1F933-1F3FC"},
    {"selfie_couleur3", "1F933-1F3FD"},
    {"selfie_couleur4", "1F933-1F3FE"},
    {"selfie_couleur5", "1F933-1F3FF"},
    {"prince", "1F934"},
    {"prince_couleur1", "1F934-1F3FB"},
    {"prince_couleur2", "1F934-1F3FC"},
    {"prince_couleur3", "1F934-1F3FD"},
    {"prince_couleur4", "1F934-1F3FE"},
    {"prince_couleur5", "1F934-1F3FF"},
    {"personne_en_smoking", "1F935"},
    {"personne_en_smoking_couleur1", "1F935-1F3FB"},
    {"personne_en_smoking_couleur2", "1F935-1F3FC"},
    {"personne_en_smoking_couleur3", "1F935-1F3FD"},
    {"personne_en_smoking_couleur4", "1F935-1F3FE"},
    {"personne_en_smoking_couleur5", "1F935-1F3FF"},
    {"mere_noel", "1F936"},
    {"mere_noel_couleur1", "1F936-1F3FB"},
    {"mere_noel_couleur2", "1F936-1F3FC"},
    {"mere_noel_couleur3", "1F936-1F3FD"},
    {"mere_noel_couleur4", "1F936-1F3FE"},
    {"mere_noel_couleur5", "1F936-1F3FF"},
    {"personne_qui_hausse_les_epaules", "1F937"},
    {"personne_qui_hausse_les_epaules_couleur1", "1F937-1F3FB"},
    {"personne_qui_hausse_les_epaules_couleur2", "1F937-1F3FC"},
    {"personne_qui_hausse_les_epaules_couleur3", "1F937-1F3FD"},
    {"personne_qui_hausse_les_epaules_couleur4", "1F937-1F3FE"},
    {"personne_qui_hausse_les_epaules_couleur5", "1F937-1F3FF"},
    {"personne_faisant_la_roue", "1F938"},
    {"personne_faisant_la_roue_couleur1", "1F938-1F3FB"},
    {"personne_faisant_la_roue_couleur2", "1F938-1F3FC"},
    {"personne_faisant_la_roue_couleur3", "1F938-1F3FD"},
    {"personne_faisant_la_roue_couleur4", "1F938-1F3FE"},
    {"personne_faisant_la_roue_couleur5", "1F938-1F3FF"},
    {"personne_qui_jongle", "1F939"},
    {"personne_qui_jongle_couleur1", "1F939-1F3FB"},
    {"personne_qui_jongle_couleur2", "1F939-1F3FC"},
    {"personne_qui_jongle_couleur3", "1F939-1F3FD"},
    {"personne_qui_jongle_couleur4", "1F939-1F3FE"},
    {"personne_qui_jongle_couleur5", "1F939-1F3FF"},
    {"escrimeur", "1F93A"},
    {"personnes_faisant_de_la_lutte", "1F93C"},
    {"personnes_faisant_de_la_lutte_couleur1", "1F93C-1F3FB"},
    {"personnes_faisant_de_la_lutte_couleur2", "1F93C-1F3FC"},
    {"personnes_faisant_de_la_lutte_couleur3", "1F93C-1F3FD"},
    {"personnes_faisant_de_la_lutte_couleur4", "1F93C-1F3FE"},
    {"personnes_faisant_de_la_lutte_couleur5", "1F93C-1F3FF"},
    {"personnes_faisant_de_la_lutte_couleur1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"personnes_faisant_de_la_lutte_couleur1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"personnes_faisant_de_la_lutte_couleur1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"personnes_faisant_de_la_lutte_couleur1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"personnes_faisant_de_la_lutte_couleur2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"personnes_faisant_de_la_lutte_couleur2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"personnes_faisant_de_la_lutte_couleur2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"personnes_faisant_de_la_lutte_couleur2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"personnes_faisant_de_la_lutte_couleur3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"personnes_faisant_de_la_lutte_couleur3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"personnes_faisant_de_la_lutte_couleur3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"personnes_faisant_de_la_lutte_couleur3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"personnes_faisant_de_la_lutte_couleur4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"personnes_faisant_de_la_lutte_couleur4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"personnes_faisant_de_la_lutte_couleur4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"personnes_faisant_de_la_lutte_couleur4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"personnes_faisant_de_la_lutte_couleur5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"personnes_faisant_de_la_lutte_couleur5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"personnes_faisant_de_la_lutte_couleur5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"personnes_faisant_de_la_lutte_couleur5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"personne_jouant_au_water_polo", "1F93D"},
    {"personne_jouant_au_water_polo_couleur1", "1F93D-1F3FB"},
    {"personne_jouant_au_water_polo_couleur2", "1F93D-1F3FC"},
    {"personne_jouant_au_water_polo_couleur3", "1F93D-1F3FD"},
    {"personne_jouant_au_water_polo_couleur4", "1F93D-1F3FE"},
    {"personne_jouant_au_water_polo_couleur5", "1F93D-1F3FF"},
    {"personne_jouant_au_handball", "1F93E"},
    {"personne_jouant_au_handball_couleur1", "1F93E-1F3FB"},
    {"personne_jouant_au_handball_couleur2", "1F93E-1F3FC"},
    {"personne_jouant_au_handball_couleur3", "1F93E-1F3FD"},
    {"personne_jouant_au_handball_couleur4", "1F93E-1F3FE"},
    {"personne_jouant_au_handball_couleur5", "1F93E-1F3FF"},
    {"masque_de_plongee", "1F93F"},
    {"fleur_fanee", "1F940"},
    {"batterie", "1F941"},
    {"trinquer", "1F942"},
    {"verre_tumbler", "1F943"},
    {"cuillere", "1F944"},
    {"cage", "1F945"},
    {"medaille_dor", "1F947"},
    {"medaille_dargent", "1F948"},
    {"medaille_de_bronze", "1F949"},
    {"gant_de_boxe", "1F94A"},
    {"tenue_darts_martiaux", "1F94B"},
    {"pierre_de_curling", "1F94C"},
    {"crosse", "1F94D"},
    {"softball", "1F94E"},
    {"disque_volant", "1F94F"},
    {"croissant", "1F950"},
    {"avocat", "1F951"},
    {"concombre", "1F952"},
    {"lard", "1F953"},
    {"pomme_de_terre", "1F954"},
    {"carotte", "1F955"},
    {"baguette", "1F956"},
    {"salade_verte", "1F957"},
    {"plat_mitonne", "1F958"},
    {"kebab", "1F959"},
    {"oeuf", "1F95A"},
    {"verre_de_lait", "1F95B"},
    {"cacahuetes", "1F95C"},
    {"kiwi", "1F95D"},
    {"pancakes", "1F95E"},
    {"boulette_de_pate", "1F95F"},
    {"biscuit_chinois", "1F960"},
    {"boite_a_emporter", "1F961"},
    {"baguettes", "1F962"},
    {"bol_avec_cuillere", "1F963"},
    {"gobelet_avec_paille", "1F964"},
    {"noix_de_coco", "1F965"},
    {"brocoli", "1F966"},
    {"tarte", "1F967"},
    {"bretzel", "1F968"},
    {"morceau_de_viande", "1F969"},
    {"sandwich", "1F96A"},
    {"aliments_en_conserve", "1F96B"},
    {"legume_a_feuilles_vertes", "1F96C"},
    {"mangue", "1F96D"},
    {"gateau_de_lune", "1F96E"},
    {"bagel", "1F96F"},
    {"visage_souriant_avec_coeurs", "1F970"},
    {"visage_baillant", "1F971"},
    {"visage_souriant_avec_une_larme", "1F972"},
    {"visage_festif", "1F973"},
    {"visage_emeche", "1F974"},
    {"visage_rouge_et_chaud", "1F975"},
    {"visage_bleu_et_froid", "1F976"},
    {"ninja", "1F977"},
    {"ninja_couleur1", "1F977-1F3FB"},
    {"ninja_couleur2", "1F977-1F3FC"},
    {"ninja_couleur3", "1F977-1F3FD"},
    {"ninja_couleur4", "1F977-1F3FE"},
    {"ninja_couleur5", "1F977-1F3FF"},
    {"visage_deguise", "1F978"},
    {"visage_retenant_ses_larmes", "1F979"},
    {"visage_implorant", "1F97A"},
    {"sari", "1F97B"},
    {"blouse_blanche", "1F97C"},
    {"lunettes", "1F97D"},
    {"chaussure_de_randonnee", "1F97E"},
    {"chaussure_plate", "1F97F"},
    {"crabe", "1F980"},
    {"tete_de_lion", "1F981"},
    {"scorpion", "1F982"},
    {"dindon", "1F983"},
    {"licorne", "1F984"},
    {"aigle", "1F985"},
    {"canard", "1F986"},
    {"chauve_souris", "1F987"},
    {"requin", "1F988"},
    {"chouette", "1F989"},
    {"renard", "1F98A"},
    {"papillon", "1F98B"},
    {"cerf", "1F98C"},
    {"gorille", "1F98D"},
    {"lezard", "1F98E"},
    {"rhinoceros", "1F98F"},
    {"crevette", "1F990"},
    {"calamar", "1F991"},
    {"girafe", "1F992"},
    {"zebre", "1F993"},
    {"herisson", "1F994"},
    {"sauropode", "1F995"},
    {"trex", "1F996"},
    {"criquet", "1F997"},
    {"kangourou", "1F998"},
    {"lama", "1F999"},
    {"paon", "1F99A"},
    {"hippopotame", "1F99B"},
    {"perroquet", "1F99C"},
    {"raton_laveur", "1F99D"},
    {"homard", "1F99E"},
    {"moustique", "1F99F"},
    {"microbe", "1F9A0"},
    {"blaireau", "1F9A1"},
    {"cygne", "1F9A2"},
    {"mammouth", "1F9A3"},
    {"dodo", "1F9A4"},
    {"paresseux", "1F9A5"},
    {"loutre", "1F9A6"},
    {"orang_outan", "1F9A7"},
    {"mouffette", "1F9A8"},
    {"flamant", "1F9A9"},
    {"huitre", "1F9AA"},
    {"castor", "1F9AB"},
    {"bison", "1F9AC"},
    {"phoque", "1F9AD"},
    {"chien_guide", "1F9AE"},
    {"canne_blanche", "1F9AF"},
    {"cheveux_roux", "1F9B0"},
    {"cheveux_boucles", "1F9B1"},
    {"chauve", "1F9B2"},
    {"cheveux_blancs", "1F9B3"},
    {"os", "1F9B4"},
    {"jambe", "1F9B5"},
    {"jambe_couleur1", "1F9B5-1F3FB"},
    {"jambe_couleur2", "1F9B5-1F3FC"},
    {"jambe_couleur3", "1F9B5-1F3FD"},
    {"jambe_couleur4", "1F9B5-1F3FE"},
    {"jambe_couleur5", "1F9B5-1F3FF"},
    {"pied", "1F9B6"},
    {"pied_couleur1", "1F9B6-1F3FB"},
    {"pied_couleur2", "1F9B6-1F3FC"},
    {"pied_couleur3", "1F9B6-1F3FD"},
    {"pied_couleur4", "1F9B6-1F3FE"},
    {"pied_couleur5", "1F9B6-1F3FF"},
    {"dent", "1F9B7"},
    {"super_heros", "1F9B8"},
    {"super_heros_couleur1", "1F9B8-1F3FB"},
    {"super_heros_couleur2", "1F9B8-1F3FC"},
    {"super_heros_couleur3", "1F9B8-1F3FD"},
    {"super_heros_couleur4", "1F9B8-1F3FE"},
    {"super_heros_couleur5", "1F9B8-1F3FF"},
    {"super_vilain", "1F9B9"},
    {"super_vilain_couleur1", "1F9B9-1F3FB"},
    {"super_vilain_couleur2", "1F9B9-1F3FC"},
    {"super_vilain_couleur3", "1F9B9-1F3FD"},
    {"super_vilain_couleur4", "1F9B9-1F3FE"},
    {"super_vilain_couleur5", "1F9B9-1F3FF"},
    {"gilet_de_securite", "1F9BA"},
    {"oreille_appareillee", "1F9BB"},
    {"oreille_appareillee_couleur1", "1F9BB-1F3FB"},
    {"oreille_appareillee_couleur2", "1F9BB-1F3FC"},
    {"oreille_appareillee_couleur3", "1F9BB-1F3FD"},
    {"oreille_appareillee_couleur4", "1F9BB-1F3FE"},
    {"oreille_appareillee_couleur5", "1F9BB-1F3FF"},
    {"fauteuil_motorise", "1F9BC"},
    {"fauteuil_roulant_manuel", "1F9BD"},
    {"bras_mecanique", "1F9BE"},
    {"jambe_mecanique", "1F9BF"},
    {"part_de_fromage", "1F9C0"},
    {"cupcake", "1F9C1"},
    {"sel", "1F9C2"},
    {"briquette_de_jus", "1F9C3"},
    {"ail", "1F9C4"},
    {"oignon", "1F9C5"},
    {"falafels", "1F9C6"},
    {"gaufre", "1F9C7"},
    {"beurre", "1F9C8"},
    {"mate", "1F9C9"},
    {"glacon", "1F9CA"},
    {"the_aux_perles", "1F9CB"},
    {"troll", "1F9CC"},
    {"personne_debout", "1F9CD"},
    {"personne_debout_couleur1", "1F9CD-1F3FB"},
    {"personne_debout_couleur2", "1F9CD-1F3FC"},
    {"personne_debout_couleur3", "1F9CD-1F3FD"},
    {"personne_debout_couleur4", "1F9CD-1F3FE"},
    {"personne_debout_couleur5", "1F9CD-1F3FF"},
    {"personne_a_genoux", "1F9CE"},
    {"personne_a_genoux_couleur1", "1F9CE-1F3FB"},
    {"personne_a_genoux_couleur2", "1F9CE-1F3FC"},
    {"personne_a_genoux_couleur3", "1F9CE-1F3FD"},
    {"personne_a_genoux_couleur4", "1F9CE-1F3FE"},
    {"personne_a_genoux_couleur5", "1F9CE-1F3FF"},
    {"personne_sourde", "1F9CF"},
    {"personne_sourde_couleur1", "1F9CF-1F3FB"},
    {"personne_sourde_couleur2", "1F9CF-1F3FC"},
    {"personne_sourde_couleur3", "1F9CF-1F3FD"},
    {"personne_sourde_couleur4", "1F9CF-1F3FE"},
    {"personne_sourde_couleur5", "1F9CF-1F3FF"},
    {"visage_avec_un_monocle", "1F9D0"},
    {"adulte", "1F9D1"},
    {"adulte_couleur1", "1F9D1-1F3FB"},
    {"adulte_couleur2", "1F9D1-1F3FC"},
    {"adulte_couleur3", "1F9D1-1F3FD"},
    {"adulte_couleur4", "1F9D1-1F3FE"},
    {"adulte_couleur5", "1F9D1-1F3FF"},
    {"enfant", "1F9D2"},
    {"enfant_couleur1", "1F9D2-1F3FB"},
    {"enfant_couleur2", "1F9D2-1F3FC"},
    {"enfant_couleur3", "1F9D2-1F3FD"},
    {"enfant_couleur4", "1F9D2-1F3FE"},
    {"enfant_couleur5", "1F9D2-1F3FF"},
    {"personne_agee", "1F9D3"},
    {"personne_agee_couleur1", "1F9D3-1F3FB"},
    {"personne_agee_couleur2", "1F9D3-1F3FC"},
    {"personne_agee_couleur3", "1F9D3-1F3FD"},
    {"personne_agee_couleur4", "1F9D3-1F3FE"},
    {"personne_agee_couleur5", "1F9D3-1F3FF"},
    {"personne_barbue", "1F9D4"},
    {"personne_barbue_couleur1", "1F9D4-1F3FB"},
    {"personne_barbue_couleur2", "1F9D4-1F3FC"},
    {"personne_barbue_couleur3", "1F9D4-1F3FD"},
    {"personne_barbue_couleur4", "1F9D4-1F3FE"},
    {"personne_barbue_couleur5", "1F9D4-1F3FF"},
    {"femme_avec_foulard", "1F9D5"},
    {"femme_avec_foulard_couleur1", "1F9D5-1F3FB"},
    {"femme_avec_foulard_couleur2", "1F9D5-1F3FC"},
    {"femme_avec_foulard_couleur3", "1F9D5-1F3FD"},
    {"femme_avec_foulard_couleur4", "1F9D5-1F3FE"},
    {"femme_avec_foulard_couleur5", "1F9D5-1F3FF"},
    {"personne_au_hammam", "1F9D6"},
    {"personne_au_hammam_couleur1", "1F9D6-1F3FB"},
    {"personne_au_hammam_couleur2", "1F9D6-1F3FC"},
    {"personne_au_hammam_couleur3", "1F9D6-1F3FD"},
    {"personne_au_hammam_couleur4", "1F9D6-1F3FE"},
    {"personne_au_hammam_couleur5", "1F9D6-1F3FF"},
    {"personne_qui_grimpe", "1F9D7"},
    {"personne_qui_grimpe_couleur1", "1F9D7-1F3FB"},
    {"personne_qui_grimpe_couleur2", "1F9D7-1F3FC"},
    {"personne_qui_grimpe_couleur3", "1F9D7-1F3FD"},
    {"personne_qui_grimpe_couleur4", "1F9D7-1F3FE"},
    {"personne_qui_grimpe_couleur5", "1F9D7-1F3FF"},
    {"personne_dans_la_posture_du_lotus", "1F9D8"},
    {"personne_dans_la_posture_du_lotus_couleur1", "1F9D8-1F3FB"},
    {"personne_dans_la_posture_du_lotus_couleur2", "1F9D8-1F3FC"},
    {"personne_dans_la_posture_du_lotus_couleur3", "1F9D8-1F3FD"},
    {"personne_dans_la_posture_du_lotus_couleur4", "1F9D8-1F3FE"},
    {"personne_dans_la_posture_du_lotus_couleur5", "1F9D8-1F3FF"},
    {"mage", "1F9D9"},
    {"mage_couleur1", "1F9D9-1F3FB"},
    {"mage_couleur2", "1F9D9-1F3FC"},
    {"mage_couleur3", "1F9D9-1F3FD"},
    {"mage_couleur4", "1F9D9-1F3FE"},
    {"mage_couleur5", "1F9D9-1F3FF"},
    {"personnage_feerique", "1F9DA"},
    {"personnage_feerique_couleur1", "1F9DA-1F3FB"},
    {"personnage_feerique_couleur2", "1F9DA-1F3FC"},
    {"personnage_feerique_couleur3", "1F9DA-1F3FD"},
    {"personnage_feerique_couleur4", "1F9DA-1F3FE"},
    {"personnage_feerique_couleur5", "1F9DA-1F3FF"},
    {"vampire", "1F9DB"},
    {"vampire_couleur1", "1F9DB-1F3FB"},
    {"vampire_couleur2", "1F9DB-1F3FC"},
    {"vampire_couleur3", "1F9DB-1F3FD"},
    {"vampire_couleur4", "1F9DB-1F3FE"},
    {"vampire_couleur5", "1F9DB-1F3FF"},
    {"creature_aquatique", "1F9DC"},
    {"creature_aquatique_couleur1", "1F9DC-1F3FB"},
    {"creature_aquatique_couleur2", "1F9DC-1F3FC"},
    {"creature_aquatique_couleur3", "1F9DC-1F3FD"},
    {"creature_aquatique_couleur4", "1F9DC-1F3FE"},
    {"creature_aquatique_couleur5", "1F9DC-1F3FF"},
    {"elfe", "1F9DD"},
    {"elfe_couleur1", "1F9DD-1F3FB"},
    {"elfe_couleur2", "1F9DD-1F3FC"},
    {"elfe_couleur3", "1F9DD-1F3FD"},
    {"elfe_couleur4", "1F9DD-1F3FE"},
    {"elfe_couleur5", "1F9DD-1F3FF"},
    {"genie", "1F9DE"},
    {"zombie", "1F9DF"},
    {"cerveau", "1F9E0"},
    {"coeur_orange", "1F9E1"},
    {"casquette_americaine", "1F9E2"},
    {"foulard", "1F9E3"},
    {"gants", "1F9E4"},
    {"manteau", "1F9E5"},
    {"chaussettes", "1F9E6"},
    {"enveloppe_rouge", "1F9E7"},
    {"petard", "1F9E8"},
    {"piece_de_puzzle", "1F9E9"},
    {"tube_a_essai", "1F9EA"},
    {"boite_de_petri", "1F9EB"},
    {"adn", "1F9EC"},
    {"boussole", "1F9ED"},
    {"abaque", "1F9EE"},
    {"extincteur", "1F9EF"},
    {"boite_a_outils", "1F9F0"},
    {"brique", "1F9F1"},
    {"aimant", "1F9F2"},
    {"bagage", "1F9F3"},
    {"bouteille_de_lotion", "1F9F4"},
    {"bobine_de_fil", "1F9F5"},
    {"fil", "1F9F6"},
    {"epingle_de_surete", "1F9F7"},
    {"ours_en_peluche", "1F9F8"},
    {"balai", "1F9F9"},
    {"panier", "1F9FA"},
    {"rouleau_de_papier", "1F9FB"},
    {"savon", "1F9FC"},
    {"eponge", "1F9FD"},
    {"recu", "1F9FE"},
    {"mauvais_oeil", "1F9FF"},
    {"chaussons_de_danse", "1FA70"},
    {"maillot_de_bain_une_piece", "1FA71"},
    {"slip", "1FA72"},
    {"short", "1FA73"},
    {"tong", "1FA74"},
    {"coeur_bleu_clair", "1FA75"},
    {"coeur_gris", "1FA76"},
    {"coeur_rose", "1FA77"},
    {"goutte_de_sang", "1FA78"},
    {"sparadrap", "1FA79"},
    {"stethoscope", "1FA7A"},
    {"radiographie", "1FA7B"},
    {"bequille", "1FA7C"},
    {"yoyo", "1FA80"},
    {"cerf_volant", "1FA81"},
    {"parachute", "1FA82"},
    {"boomerang", "1FA83"},
    {"baguette_magique", "1FA84"},
    {"pinata", "1FA85"},
    {"poupees_russes", "1FA86"},
    {"maracas", "1FA87"},
    {"flute", "1FA88"},
    {"harpe", "1FA89"},
    {"coffre_au_tresor", "1FA8E"},
    {"pelle", "1FA8F"},
    {"planete_a_anneaux", "1FA90"},
    {"chaise", "1FA91"},
    {"rasoir", "1FA92"},
    {"hache", "1FA93"},
    {"diya", "1FA94"},
    {"banjo", "1FA95"},
    {"casque_militaire", "1FA96"},
    {"accordeon", "1FA97"},
    {"djembe", "1FA98"},
    {"piece", "1FA99"},
    {"scie", "1FA9A"},
    {"tournevis", "1FA9B"},
    {"echelle", "1FA9C"},
    {"crochet", "1FA9D"},
    {"miroir", "1FA9E"},
    {"fenetre", "1FA9F"},
    {"ventouse", "1FAA0"},
    {"aiguille_a_coudre", "1FAA1"},
    {"noeud", "1FAA2"},
    {"seau", "1FAA3"},
    {"tapette_a_souris", "1FAA4"},
    {"brosse_a_dents", "1FAA5"},
    {"pierre_tombale", "1FAA6"},
    {"pancarte", "1FAA7"},
    {"rocher", "1FAA8"},
    {"boule_a_facettes", "1FAA9"},
    {"carte_didentite", "1FAAA"},
    {"pile_faible", "1FAAB"},
    {"main_de_fatma", "1FAAC"},
    {"eventail", "1FAAD"},
    {"peigne_afro", "1FAAE"},
    {"khanda", "1FAAF"},
    {"mouche", "1FAB0"},
    {"lombric", "1FAB1"},
    {"scarabee", "1FAB2"},
    {"cafard", "1FAB3"},
    {"plante_en_pot", "1FAB4"},
    {"bois", "1FAB5"},
    {"plume", "1FAB6"},
    {"lotus", "1FAB7"},
    {"corail", "1FAB8"},
    {"nid_vide", "1FAB9"},
    {"nid_avec_oeufs", "1FABA"},
    {"jacinthe", "1FABB"},
    {"meduse", "1FABC"},
    {"aile", "1FABD"},
    {"arbre_sans_feuille", "1FABE"},
    {"oie", "1FABF"},
    {"coeur", "1FAC0"},
    {"poumons", "1FAC1"},
    {"etreinte", "1FAC2"},
    {"homme_enceint", "1FAC3"},
    {"homme_enceint_couleur1", "1FAC3-1F3FB"},
    {"homme_enceint_couleur2", "1FAC3-1F3FC"},
    {"homme_enceint_couleur3", "1FAC3-1F3FD"},
    {"homme_enceint_couleur4", "1FAC3-1F3FE"},
    {"homme_enceint_couleur5", "1FAC3-1F3FF"},
    {"personne_enceinte", "1FAC4"},
    {"personne_enceinte_couleur1", "1FAC4-1F3FB"},
    {"personne_enceinte_couleur2", "1FAC4-1F3FC"},
    {"personne_enceinte_couleur3", "1FAC4-1F3FD"},
    {"personne_enceinte_couleur4", "1FAC4-1F3FE"},
    {"personne_enceinte_couleur5", "1FAC4-1F3FF"},
    {"personne_avec_une_couronne", "1FAC5"},
    {"personne_avec_une_couronne_couleur1", "1FAC5-1F3FB"},
    {"personne_avec_une_couronne_couleur2", "1FAC5-1F3FC"},
    {"personne_avec_une_couronne_couleur3", "1FAC5-1F3FD"},
    {"personne_avec_une_couronne_couleur4", "1FAC5-1F3FE"},
    {"personne_avec_une_couronne_couleur5", "1FAC5-1F3FF"},
    {"empreinte_digitale", "1FAC6"},
    {"creature_velue", "1FAC8"},
    {"orque", "1FACD"},
    {"elan", "1FACE"},
    {"ane", "1FACF"},
    {"myrtilles", "1FAD0"},
    {"poivron", "1FAD1"},
    {"olive", "1FAD2"},
    {"galette", "1FAD3"},
    {"tamal", "1FAD4"},
    {"fondue", "1FAD5"},
    {"theiere", "1FAD6"},
    {"verser_un_liquide", "1FAD7"},
    {"haricots", "1FAD8"},
    {"bocal", "1FAD9"},
    {"racine_de_gingembre", "1FADA"},
    {"cosse_de_petits_pois", "1FADB"},
    {"legume_racine", "1FADC"},
    {"eclaboussure", "1FADF"},
    {"visage_qui_fond", "1FAE0"},
    {"visage_qui_fait_un_salut_militaire", "1FAE1"},
    {"visage_avec_yeux_ouverts_et_main_sur_la_bouche", "1FAE2"},
    {"visage_qui_regarde_entre_ses_doigts", "1FAE3"},
    {"visage_avec_bouche_en_diagonale", "1FAE4"},
    {"visage_en_pointilles", "1FAE5"},
    {"levres_qui_se_mordent", "1FAE6"},
    {"bulles", "1FAE7"},
    {"visage_secoue", "1FAE8"},
    {"visage_avec_des_valises_sous_les_yeux", "1FAE9"},
    {"visage_deforme", "1FAEA"},
    {"nuage_de_colere", "1FAEF"},
    {"main_avec_index_et_pouce_croises", "1FAF0"},
    {"main_avec_index_et_pouce_croises_couleur1", "1FAF0-1F3FB"},
    {"main_avec_index_et_pouce_croises_couleur2", "1FAF0-1F3FC"},
    {"main_avec_index_et_pouce_croises_couleur3", "1FAF0-1F3FD"},
    {"main_avec_index_et_pouce_croises_couleur4", "1FAF0-1F3FE"},
    {"main_avec_index_et_pouce_croises_couleur5", "1FAF0-1F3FF"},
    {"main_vers_la_droite", "1FAF1"},
    {"main_vers_la_droite_couleur1", "1FAF1-1F3FB"},
    {"main_vers_la_droite_couleur2", "1FAF1-1F3FC"},
    {"main_vers_la_droite_couleur3", "1FAF1-1F3FD"},
    {"main_vers_la_droite_couleur4", "1FAF1-1F3FE"},
    {"main_vers_la_droite_couleur5", "1FAF1-1F3FF"},
    {"main_vers_la_gauche", "1FAF2"},
    {"main_vers_la_gauche_couleur1", "1FAF2-1F3FB"},
    {"main_vers_la_gauche_couleur2", "1FAF2-1F3FC"},
    {"main_vers_la_gauche_couleur3", "1FAF2-1F3FD"},
    {"main_vers_la_gauche_couleur4", "1FAF2-1F3FE"},
    {"main_vers_la_gauche_couleur5", "1FAF2-1F3FF"},
    {"main_paume_vers_le_bas", "1FAF3"},
    {"main_paume_vers_le_bas_couleur1", "1FAF3-1F3FB"},
    {"main_paume_vers_le_bas_couleur2", "1FAF3-1F3FC"},
    {"main_paume_vers_le_bas_couleur3", "1FAF3-1F3FD"},
    {"main_paume_vers_le_bas_couleur4", "1FAF3-1F3FE"},
    {"main_paume_vers_le_bas_couleur5", "1FAF3-1F3FF"},
    {"main_paume_vers_le_haut", "1FAF4"},
    {"main_paume_vers_le_haut_couleur1", "1FAF4-1F3FB"},
    {"main_paume_vers_le_haut_couleur2", "1FAF4-1F3FC"},
    {"main_paume_vers_le_haut_couleur3", "1FAF4-1F3FD"},
    {"main_paume_vers_le_haut_couleur4", "1FAF4-1F3FE"},
    {"main_paume_vers_le_haut_couleur5", "1FAF4-1F3FF"},
    {"index_pointant_vers_lutilisateur", "1FAF5"},
    {"index_pointant_vers_lutilisateur_couleur1", "1FAF5-1F3FB"},
    {"index_pointant_vers_lutilisateur_couleur2", "1FAF5-1F3FC"},
    {"index_pointant_vers_lutilisateur_couleur3", "1FAF5-1F3FD"},
    {"index_pointant_vers_lutilisateur_couleur4", "1FAF5-1F3FE"},
    {"index_pointant_vers_lutilisateur_couleur5", "1FAF5-1F3FF"},
    {"mains_qui_forment_un_coeur", "1FAF6"},
    {"mains_qui_forment_un_coeur_couleur1", "1FAF6-1F3FB"},
    {"mains_qui_forment_un_coeur_couleur2", "1FAF6-1F3FC"},
    {"mains_qui_forment_un_coeur_couleur3", "1FAF6-1F3FD"},
    {"mains_qui_forment_un_coeur_couleur4", "1FAF6-1F3FE"},
    {"mains_qui_forment_un_coeur_couleur5", "1FAF6-1F3FF"},
    {"main_qui_pousse_vers_la_gauche", "1FAF7"},
    {"main_qui_pousse_vers_la_gauche_couleur1", "1FAF7-1F3FB"},
    {"main_qui_pousse_vers_la_gauche_couleur2", "1FAF7-1F3FC"},
    {"main_qui_pousse_vers_la_gauche_couleur3", "1FAF7-1F3FD"},
    {"main_qui_pousse_vers_la_gauche_couleur4", "1FAF7-1F3FE"},
    {"main_qui_pousse_vers_la_gauche_couleur5", "1FAF7-1F3FF"},
    {"main_qui_pousse_vers_la_droite", "1FAF8"},
    {"main_qui_pousse_vers_la_droite_couleur1", "1FAF8-1F3FB"},
    {"main_qui_pousse_vers_la_droite_couleur2", "1FAF8-1F3FC"},
    {"main_qui_pousse_vers_la_droite_couleur3", "1FAF8-1F3FD"},
    {"main_qui_pousse_vers_la_droite_couleur4", "1FAF8-1F3FE"},
    {"main_qui_pousse_vers_la_droite_couleur5", "1FAF8-1F3FF"},
    {"touches_signe_numerique", "0023-FE0F-20E3"},
    {"touches_asterisque", "002A-FE0F-20E3"},
    {"touches_0", "0030-FE0F-20E3"},
    {"touches_1", "0031-FE0F-20E3"},
    {"touches_2", "0032-FE0F-20E3"},
    {"touches_3", "0033-FE0F-20E3"},
    {"touches_4", "0034-FE0F-20E3"},
    {"touches_5", "0035-FE0F-20E3"},
    {"touches_6", "0036-FE0F-20E3"},
    {"touches_7", "0037-FE0F-20E3"},
    {"touches_8", "0038-FE0F-20E3"},
    {"touches_9", "0039-FE0F-20E3"},
    {"drapeau_ile_de_lascension", "1F1E6-1F1E8"},
    {"drapeau_andorre", "1F1E6-1F1E9"},
    {"drapeau_emirats_arabes_unis", "1F1E6-1F1EA"},
    {"drapeau_afghanistan", "1F1E6-1F1EB"},
    {"drapeau_antigua_et_barbuda", "1F1E6-1F1EC"},
    {"drapeau_anguilla", "1F1E6-1F1EE"},
    {"drapeau_albanie", "1F1E6-1F1F1"},
    {"drapeau_armenie", "1F1E6-1F1F2"},
    {"drapeau_angola", "1F1E6-1F1F4"},
    {"drapeau_antarctique", "1F1E6-1F1F6"},
    {"drapeau_argentine", "1F1E6-1F1F7"},
    {"drapeau_samoa_americaines", "1F1E6-1F1F8"},
    {"drapeau_autriche", "1F1E6-1F1F9"},
    {"drapeau_australie", "1F1E6-1F1FA"},
    {"drapeau_aruba", "1F1E6-1F1FC"},
    {"drapeau_iles_aland", "1F1E6-1F1FD"},
    {"drapeau_azerbaidjan", "1F1E6-1F1FF"},
    {"drapeau_bosnie_herzegovine", "1F1E7-1F1E6"},
    {"drapeau_barbade", "1F1E7-1F1E7"},
    {"drapeau_bangladesh", "1F1E7-1F1E9"},
    {"drapeau_belgique", "1F1E7-1F1EA"},
    {"drapeau_burkina_faso", "1F1E7-1F1EB"},
    {"drapeau_bulgarie", "1F1E7-1F1EC"},
    {"drapeau_bahrein", "1F1E7-1F1ED"},
    {"drapeau_burundi", "1F1E7-1F1EE"},
    {"drapeau_benin", "1F1E7-1F1EF"},
    {"drapeau_saint_barthelemy", "1F1E7-1F1F1"},
    {"drapeau_bermudes", "1F1E7-1F1F2"},
    {"drapeau_brunei", "1F1E7-1F1F3"},
    {"drapeau_bolivie", "1F1E7-1F1F4"},
    {"drapeau_pays_bas_caribeens", "1F1E7-1F1F6"},
    {"drapeau_bresil", "1F1E7-1F1F7"},
    {"drapeau_bahamas", "1F1E7-1F1F8"},
    {"drapeau_bhoutan", "1F1E7-1F1F9"},
    {"drapeau_ile_bouvet", "1F1E7-1F1FB"},
    {"drapeau_botswana", "1F1E7-1F1FC"},
    {"drapeau_bielorussie", "1F1E7-1F1FE"},
    {"drapeau_belize", "1F1E7-1F1FF"},
    {"drapeau_canada", "1F1E8-1F1E6"},
    {"drapeau_iles_cocos", "1F1E8-1F1E8"},
    {"drapeau_congo_kinshasa", "1F1E8-1F1E9"},
    {"drapeau_republique_centrafricaine", "1F1E8-1F1EB"},
    {"drapeau_congo_brazzaville", "1F1E8-1F1EC"},
    {"drapeau_suisse", "1F1E8-1F1ED"},
    {"drapeau_cote_divoire", "1F1E8-1F1EE"},
    {"drapeau_iles_cook", "1F1E8-1F1F0"},
    {"drapeau_chili", "1F1E8-1F1F1"},
    {"drapeau_cameroun", "1F1E8-1F1F2"},
    {"drapeau_chine", "1F1E8-1F1F3"},
    {"drapeau_colombie", "1F1E8-1F1F4"},
    {"drapeau_ile_clipperton", "1F1E8-1F1F5"},
    {"drapeau_sercq", "1F1E8-1F1F6"},
    {"drapeau_costa_rica", "1F1E8-1F1F7"},
    {"drapeau_cuba", "1F1E8-1F1FA"},
    {"drapeau_cap_vert", "1F1E8-1F1FB"},
    {"drapeau_curacao", "1F1E8-1F1FC"},
    {"drapeau_ile_christmas", "1F1E8-1F1FD"},
    {"drapeau_chypre", "1F1E8-1F1FE"},
    {"drapeau_tchequie", "1F1E8-1F1FF"},
    {"drapeau_allemagne", "1F1E9-1F1EA"},
    {"drapeau_diego_garcia", "1F1E9-1F1EC"},
    {"drapeau_djibouti", "1F1E9-1F1EF"},
    {"drapeau_danemark", "1F1E9-1F1F0"},
    {"drapeau_dominique", "1F1E9-1F1F2"},
    {"drapeau_republique_dominicaine", "1F1E9-1F1F4"},
    {"drapeau_algerie", "1F1E9-1F1FF"},
    {"drapeau_ceuta_et_melilla", "1F1EA-1F1E6"},
    {"drapeau_equateur", "1F1EA-1F1E8"},
    {"drapeau_estonie", "1F1EA-1F1EA"},
    {"drapeau_egypte", "1F1EA-1F1EC"},
    {"drapeau_sahara_occidental", "1F1EA-1F1ED"},
    {"drapeau_erythree", "1F1EA-1F1F7"},
    {"drapeau_espagne", "1F1EA-1F1F8"},
    {"drapeau_ethiopie", "1F1EA-1F1F9"},
    {"drapeau_union_europeenne", "1F1EA-1F1FA"},
    {"drapeau_finlande", "1F1EB-1F1EE"},
    {"drapeau_fidji", "1F1EB-1F1EF"},
    {"drapeau_iles_malouines", "1F1EB-1F1F0"},
    {"drapeau_micronesie", "1F1EB-1F1F2"},
    {"drapeau_iles_feroe", "1F1EB-1F1F4"},
    {"drapeau_france", "1F1EB-1F1F7"},
    {"drapeau_gabon", "1F1EC-1F1E6"},
    {"drapeau_royaume_uni", "1F1EC-1F1E7"},
    {"drapeau_grenade", "1F1EC-1F1E9"},
    {"drapeau_georgie", "1F1EC-1F1EA"},
    {"drapeau_guyane_francaise", "1F1EC-1F1EB"},
    {"drapeau_guernesey", "1F1EC-1F1EC"},
    {"drapeau_ghana", "1F1EC-1F1ED"},
    {"drapeau_gibraltar", "1F1EC-1F1EE"},
    {"drapeau_groenland", "1F1EC-1F1F1"},
    {"drapeau_gambie", "1F1EC-1F1F2"},
    {"drapeau_guinee", "1F1EC-1F1F3"},
    {"drapeau_guadeloupe", "1F1EC-1F1F5"},
    {"drapeau_guinee_equatoriale", "1F1EC-1F1F6"},
    {"drapeau_grece", "1F1EC-1F1F7"},
    {"drapeau_georgie_du_sud_et_les_iles_sandwich_du_sud", "1F1EC-1F1F8"},
    {"drapeau_guatemala", "1F1EC-1F1F9"},
    {"drapeau_guam", "1F1EC-1F1FA"},
    {"drapeau_guinee_bissau", "1F1EC-1F1FC"},
    {"drapeau_guyana", "1F1EC-1F1FE"},
    {"drapeau_ras_chinoise_de_hong_kong", "1F1ED-1F1F0"},
    {"drapeau_iles_heard_et_macdonald", "1F1ED-1F1F2"},
    {"drapeau_honduras", "1F1ED-1F1F3"},
    {"drapeau_croatie", "1F1ED-1F1F7"},
    {"drapeau_haiti", "1F1ED-1F1F9"},
    {"drapeau_hongrie", "1F1ED-1F1FA"},
    {"drapeau_iles_canaries", "1F1EE-1F1E8"},
    {"drapeau_indonesie", "1F1EE-1F1E9"},
    {"drapeau_irlande", "1F1EE-1F1EA"},
    {"drapeau_israel", "1F1EE-1F1F1"},
    {"drapeau_ile_de_man", "1F1EE-1F1F2"},
    {"drapeau_inde", "1F1EE-1F1F3"},
    {"drapeau_territoire_britannique_de_locean_indien", "1F1EE-1F1F4"},
    {"drapeau_irak", "1F1EE-1F1F6"},
    {"drapeau_iran", "1F1EE-1F1F7"},
    {"drapeau_islande", "1F1EE-1F1F8"},
    {"drapeau_italie", "1F1EE-1F1F9"},
    {"drapeau_jersey", "1F1EF-1F1EA"},
    {"drapeau_jamaique", "1F1EF-1F1F2"},
    {"drapeau_jordanie", "1F1EF-1F1F4"},
    {"drapeau_japon", "1F1EF-1F1F5"},
    {"drapeau_kenya", "1F1F0-1F1EA"},
    {"drapeau_kirghizstan", "1F1F0-1F1EC"},
    {"drapeau_cambodge", "1F1F0-1F1ED"},
    {"drapeau_kiribati", "1F1F0-1F1EE"},
    {"drapeau_comores", "1F1F0-1F1F2"},
    {"drapeau_saint_christophe_et_nieves", "1F1F0-1F1F3"},
    {"drapeau_coree_du_nord", "1F1F0-1F1F5"},
    {"drapeau_coree_du_sud", "1F1F0-1F1F7"},
    {"drapeau_koweit", "1F1F0-1F1FC"},
    {"drapeau_iles_caimans", "1F1F0-1F1FE"},
    {"drapeau_kazakhstan", "1F1F0-1F1FF"},
    {"drapeau_laos", "1F1F1-1F1E6"},
    {"drapeau_liban", "1F1F1-1F1E7"},
    {"drapeau_sainte_lucie", "1F1F1-1F1E8"},
    {"drapeau_liechtenstein", "1F1F1-1F1EE"},
    {"drapeau_sri_lanka", "1F1F1-1F1F0"},
    {"drapeau_liberia", "1F1F1-1F1F7"},
    {"drapeau_lesotho", "1F1F1-1F1F8"},
    {"drapeau_lituanie", "1F1F1-1F1F9"},
    {"drapeau_luxembourg", "1F1F1-1F1FA"},
    {"drapeau_lettonie", "1F1F1-1F1FB"},
    {"drapeau_libye", "1F1F1-1F1FE"},
    {"drapeau_maroc", "1F1F2-1F1E6"},
    {"drapeau_monaco", "1F1F2-1F1E8"},
    {"drapeau_moldavie", "1F1F2-1F1E9"},
    {"drapeau_montenegro", "1F1F2-1F1EA"},
    {"drapeau_saint_martin", "1F1F2-1F1EB"},
    {"drapeau_madagascar", "1F1F2-1F1EC"},
    {"drapeau_iles_marshall", "1F1F2-1F1ED"},
    {"drapeau_macedoine_du_nord", "1F1F2-1F1F0"},
    {"drapeau_mali", "1F1F2-1F1F1"},
    {"drapeau_myanmar_birmanie", "1F1F2-1F1F2"},
    {"drapeau_mongolie", "1F1F2-1F1F3"},
    {"drapeau_ras_chinoise_de_macao", "1F1F2-1F1F4"},
    {"drapeau_iles_mariannes_du_nord", "1F1F2-1F1F5"},
    {"drapeau_martinique", "1F1F2-1F1F6"},
    {"drapeau_mauritanie", "1F1F2-1F1F7"},
    {"drapeau_montserrat", "1F1F2-1F1F8"},
    {"drapeau_malte", "1F1F2-1F1F9"},
    {"drapeau_maurice", "1F1F2-1F1FA"},
    {"drapeau_maldives", "1F1F2-1F1FB"},
    {"drapeau_malawi", "1F1F2-1F1FC"},
    {"drapeau_mexique", "1F1F2-1F1FD"},
    {"drapeau_malaisie", "1F1F2-1F1FE"},
    {"drapeau_mozambique", "1F1F2-1F1FF"},
    {"drapeau_namibie", "1F1F3-1F1E6"},
    {"drapeau_nouvelle_caledonie", "1F1F3-1F1E8"},
    {"drapeau_niger", "1F1F3-1F1EA"},
    {"drapeau_ile_norfolk", "1F1F3-1F1EB"},
    {"drapeau_nigeria", "1F1F3-1F1EC"},
    {"drapeau_nicaragua", "1F1F3-1F1EE"},
    {"drapeau_pays_bas", "1F1F3-1F1F1"},
    {"drapeau_norvege", "1F1F3-1F1F4"},
    {"drapeau_nepal", "1F1F3-1F1F5"},
    {"drapeau_nauru", "1F1F3-1F1F7"},
    {"drapeau_niue", "1F1F3-1F1FA"},
    {"drapeau_nouvelle_zelande", "1F1F3-1F1FF"},
    {"drapeau_oman", "1F1F4-1F1F2"},
    {"drapeau_panama", "1F1F5-1F1E6"},
    {"drapeau_perou", "1F1F5-1F1EA"},
    {"drapeau_polynesie_francaise", "1F1F5-1F1EB"},
    {"drapeau_papouasie_nouvelle_guinee", "1F1F5-1F1EC"},
    {"drapeau_philippines", "1F1F5-1F1ED"},
    {"drapeau_pakistan", "1F1F5-1F1F0"},
    {"drapeau_pologne", "1F1F5-1F1F1"},
    {"drapeau_saint_pierre_et_miquelon", "1F1F5-1F1F2"},
    {"drapeau_iles_pitcairn", "1F1F5-1F1F3"},
    {"drapeau_porto_rico", "1F1F5-1F1F7"},
    {"drapeau_territoires_palestiniens", "1F1F5-1F1F8"},
    {"drapeau_portugal", "1F1F5-1F1F9"},
    {"drapeau_palaos", "1F1F5-1F1FC"},
    {"drapeau_paraguay", "1F1F5-1F1FE"},
    {"drapeau_qatar", "1F1F6-1F1E6"},
    {"drapeau_la_reunion", "1F1F7-1F1EA"},
    {"drapeau_roumanie", "1F1F7-1F1F4"},
    {"drapeau_serbie", "1F1F7-1F1F8"},
    {"drapeau_russie", "1F1F7-1F1FA"},
    {"drapeau_rwanda", "1F1F7-1F1FC"},
    {"drapeau_arabie_saoudite", "1F1F8-1F1E6"},
    {"drapeau_iles_salomon", "1F1F8-1F1E7"},
    {"drapeau_seychelles", "1F1F8-1F1E8"},
    {"drapeau_soudan", "1F1F8-1F1E9"},
    {"drapeau_suede", "1F1F8-1F1EA"},
    {"drapeau_singapour", "1F1F8-1F1EC"},
    {"drapeau_sainte_helene", "1F1F8-1F1ED"},
    {"drapeau_slovenie", "1F1F8-1F1EE"},
    {"drapeau_svalbard_et_jan_mayen", "1F1F8-1F1EF"},
    {"drapeau_slovaquie", "1F1F8-1F1F0"},
    {"drapeau_sierra_leone", "1F1F8-1F1F1"},
    {"drapeau_saint_marin", "1F1F8-1F1F2"},
    {"drapeau_senegal", "1F1F8-1F1F3"},
    {"drapeau_somalie", "1F1F8-1F1F4"},
    {"drapeau_suriname", "1F1F8-1F1F7"},
    {"drapeau_soudan_du_sud", "1F1F8-1F1F8"},
    {"drapeau_sao_tome_et_principe", "1F1F8-1F1F9"},
    {"drapeau_salvador", "1F1F8-1F1FB"},
    {"drapeau_saint_martin_partie_neerlandaise", "1F1F8-1F1FD"},
    {"drapeau_syrie", "1F1F8-1F1FE"},
    {"drapeau_eswatini", "1F1F8-1F1FF"},
    {"drapeau_tristan_da_cunha", "1F1F9-1F1E6"},
    {"drapeau_iles_turques_et_caiques", "1F1F9-1F1E8"},
    {"drapeau_tchad", "1F1F9-1F1E9"},
    {"drapeau_terres_australes_francaises", "1F1F9-1F1EB"},
    {"drapeau_togo", "1F1F9-1F1EC"},
    {"drapeau_thailande", "1F1F9-1F1ED"},
    {"drapeau_tadjikistan", "1F1F9-1F1EF"},
    {"drapeau_tokelau", "1F1F9-1F1F0"},
    {"drapeau_timor_oriental", "1F1F9-1F1F1"},
    {"drapeau_turkmenistan", "1F1F9-1F1F2"},
    {"drapeau_tunisie", "1F1F9-1F1F3"},
    {"drapeau_tonga", "1F1F9-1F1F4"},
    {"drapeau_turquie", "1F1F9-1F1F7"},
    {"drapeau_trinite_et_tobago", "1F1F9-1F1F9"},
    {"drapeau_tuvalu", "1F1F9-1F1FB"},
    {"drapeau_taiwan", "1F1F9-1F1FC"},
    {"drapeau_tanzanie", "1F1F9-1F1FF"},
    {"drapeau_ukraine", "1F1FA-1F1E6"},
    {"drapeau_ouganda", "1F1FA-1F1EC"},
    {"drapeau_iles_mineures_eloignees_des_etats_unis", "1F1FA-1F1F2"},
    {"drapeau_nations_unies", "1F1FA-1F1F3"},
    {"drapeau_etats_unis", "1F1FA-1F1F8"},
    {"drapeau_uruguay", "1F1FA-1F1FE"},
    {"drapeau_ouzbekistan", "1F1FA-1F1FF"},
    {"drapeau_etat_de_la_cite_du_vatican", "1F1FB-1F1E6"},
    {"drapeau_saint_vincent_et_les_grenadines", "1F1FB-1F1E8"},
    {"drapeau_venezuela", "1F1FB-1F1EA"},
    {"drapeau_iles_vierges_britanniques", "1F1FB-1F1EC"},
    {"drapeau_iles_vierges_des_etats_unis", "1F1FB-1F1EE"},
    {"drapeau_viet_nam", "1F1FB-1F1F3"},
    {"drapeau_vanuatu", "1F1FB-1F1FA"},
    {"drapeau_wallis_et_futuna", "1F1FC-1F1EB"},
    {"drapeau_samoa", "1F1FC-1F1F8"},
    {"drapeau_kosovo", "1F1FD-1F1F0"},
    {"drapeau_yemen", "1F1FE-1F1EA"},
    {"drapeau_mayotte", "1F1FE-1F1F9"},
    {"drapeau_afrique_du_sud", "1F1FF-1F1E6"},
    {"drapeau_zambie", "1F1FF-1F1F2"},
    {"drapeau_zimbabwe", "1F1FF-1F1FC"},
    {"drapeau_angleterre", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"drapeau_ecosse", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"drapeau_pays_de_galles", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"couple_avec_coeur_homme_et_homme", "1F468-200D-2764-FE0F-200D-1F468"},
    {"couple_avec_coeur_homme_et_homme_couleur1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_coeur_homme_et_homme_couleur1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_coeur_homme_et_homme_couleur1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_coeur_homme_et_homme_couleur1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_coeur_homme_et_homme_couleur1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_coeur_homme_et_homme_couleur2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_coeur_homme_et_homme_couleur2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_coeur_homme_et_homme_couleur2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_coeur_homme_et_homme_couleur2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_coeur_homme_et_homme_couleur2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_coeur_homme_et_homme_couleur3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_coeur_homme_et_homme_couleur3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_coeur_homme_et_homme_couleur3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_coeur_homme_et_homme_couleur3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_coeur_homme_et_homme_couleur3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_coeur_homme_et_homme_couleur4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_coeur_homme_et_homme_couleur4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_coeur_homme_et_homme_couleur4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_coeur_homme_et_homme_couleur4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_coeur_homme_et_homme_couleur4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_coeur_homme_et_homme_couleur5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_coeur_homme_et_homme_couleur5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_coeur_homme_et_homme_couleur5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_coeur_homme_et_homme_couleur5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_coeur_homme_et_homme_couleur5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"bisou_homme_et_homme", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"bisou_homme_et_homme_couleur1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bisou_homme_et_homme_couleur1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bisou_homme_et_homme_couleur1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bisou_homme_et_homme_couleur1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bisou_homme_et_homme_couleur1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bisou_homme_et_homme_couleur2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bisou_homme_et_homme_couleur2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bisou_homme_et_homme_couleur2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bisou_homme_et_homme_couleur2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bisou_homme_et_homme_couleur2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bisou_homme_et_homme_couleur3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bisou_homme_et_homme_couleur3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bisou_homme_et_homme_couleur3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bisou_homme_et_homme_couleur3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bisou_homme_et_homme_couleur3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bisou_homme_et_homme_couleur4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bisou_homme_et_homme_couleur4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bisou_homme_et_homme_couleur4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bisou_homme_et_homme_couleur4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bisou_homme_et_homme_couleur4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bisou_homme_et_homme_couleur5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bisou_homme_et_homme_couleur5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bisou_homme_et_homme_couleur5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bisou_homme_et_homme_couleur5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bisou_homme_et_homme_couleur5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"famille_homme_et_garcon", "1F468-200D-1F466"},
    {"famille_homme_garcon_et_garcon", "1F468-200D-1F466-200D-1F466"},
    {"famille_homme_et_fille", "1F468-200D-1F467"},
    {"famille_homme_fille_et_garcon", "1F468-200D-1F467-200D-1F466"},
    {"famille_homme_fille_et_fille", "1F468-200D-1F467-200D-1F467"},
    {"famille_homme_homme_et_garcon", "1F468-200D-1F468-200D-1F466"},
    {"famille_homme_homme_garcon_et_garcon", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"famille_homme_homme_et_fille", "1F468-200D-1F468-200D-1F467"},
    {"famille_homme_homme_fille_et_garcon", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"famille_homme_homme_fille_et_fille", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"famille_homme_femme_et_garcon", "1F468-200D-1F469-200D-1F466"},
    {"famille_homme_femme_garcon_et_garcon", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"famille_homme_femme_et_fille", "1F468-200D-1F469-200D-1F467"},
    {"famille_homme_femme_fille_et_garcon", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"famille_homme_femme_fille_et_fille", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"couple_avec_coeur_femme_et_homme", "1F469-200D-2764-FE0F-200D-1F468"},
    {"couple_avec_coeur_femme_et_homme_couleur1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_coeur_femme_et_homme_couleur1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_coeur_femme_et_homme_couleur1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_coeur_femme_et_homme_couleur1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_coeur_femme_et_homme_couleur1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_coeur_femme_et_homme_couleur2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_coeur_femme_et_homme_couleur2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_coeur_femme_et_homme_couleur2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_coeur_femme_et_homme_couleur2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_coeur_femme_et_homme_couleur2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_coeur_femme_et_homme_couleur3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_coeur_femme_et_homme_couleur3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_coeur_femme_et_homme_couleur3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_coeur_femme_et_homme_couleur3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_coeur_femme_et_homme_couleur3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_coeur_femme_et_homme_couleur4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_coeur_femme_et_homme_couleur4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_coeur_femme_et_homme_couleur4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_coeur_femme_et_homme_couleur4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_coeur_femme_et_homme_couleur4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_coeur_femme_et_homme_couleur5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_coeur_femme_et_homme_couleur5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_coeur_femme_et_homme_couleur5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_coeur_femme_et_homme_couleur5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_coeur_femme_et_homme_couleur5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_coeur_femme_et_femme", "1F469-200D-2764-FE0F-200D-1F469"},
    {"couple_avec_coeur_femme_et_femme_couleur1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"couple_avec_coeur_femme_et_femme_couleur1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"couple_avec_coeur_femme_et_femme_couleur1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"couple_avec_coeur_femme_et_femme_couleur1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"couple_avec_coeur_femme_et_femme_couleur1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"couple_avec_coeur_femme_et_femme_couleur2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"couple_avec_coeur_femme_et_femme_couleur2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"couple_avec_coeur_femme_et_femme_couleur2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"couple_avec_coeur_femme_et_femme_couleur2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"couple_avec_coeur_femme_et_femme_couleur2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"couple_avec_coeur_femme_et_femme_couleur3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"couple_avec_coeur_femme_et_femme_couleur3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"couple_avec_coeur_femme_et_femme_couleur3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"couple_avec_coeur_femme_et_femme_couleur3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"couple_avec_coeur_femme_et_femme_couleur3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"couple_avec_coeur_femme_et_femme_couleur4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"couple_avec_coeur_femme_et_femme_couleur4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"couple_avec_coeur_femme_et_femme_couleur4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"couple_avec_coeur_femme_et_femme_couleur4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"couple_avec_coeur_femme_et_femme_couleur4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"couple_avec_coeur_femme_et_femme_couleur5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"couple_avec_coeur_femme_et_femme_couleur5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"couple_avec_coeur_femme_et_femme_couleur5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"couple_avec_coeur_femme_et_femme_couleur5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"couple_avec_coeur_femme_et_femme_couleur5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"bisou_femme_et_homme", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"bisou_femme_et_homme_couleur1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bisou_femme_et_homme_couleur1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bisou_femme_et_homme_couleur1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bisou_femme_et_homme_couleur1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bisou_femme_et_homme_couleur1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bisou_femme_et_homme_couleur2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bisou_femme_et_homme_couleur2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bisou_femme_et_homme_couleur2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bisou_femme_et_homme_couleur2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bisou_femme_et_homme_couleur2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bisou_femme_et_homme_couleur3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bisou_femme_et_homme_couleur3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bisou_femme_et_homme_couleur3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bisou_femme_et_homme_couleur3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bisou_femme_et_homme_couleur3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bisou_femme_et_homme_couleur4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bisou_femme_et_homme_couleur4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bisou_femme_et_homme_couleur4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bisou_femme_et_homme_couleur4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bisou_femme_et_homme_couleur4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bisou_femme_et_homme_couleur5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bisou_femme_et_homme_couleur5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bisou_femme_et_homme_couleur5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bisou_femme_et_homme_couleur5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bisou_femme_et_homme_couleur5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bisou_femme_et_femme", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"bisou_femme_et_femme_couleur1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bisou_femme_et_femme_couleur1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bisou_femme_et_femme_couleur1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bisou_femme_et_femme_couleur1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bisou_femme_et_femme_couleur1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bisou_femme_et_femme_couleur2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bisou_femme_et_femme_couleur2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bisou_femme_et_femme_couleur2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bisou_femme_et_femme_couleur2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bisou_femme_et_femme_couleur2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bisou_femme_et_femme_couleur3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bisou_femme_et_femme_couleur3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bisou_femme_et_femme_couleur3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bisou_femme_et_femme_couleur3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bisou_femme_et_femme_couleur3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bisou_femme_et_femme_couleur4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bisou_femme_et_femme_couleur4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bisou_femme_et_femme_couleur4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bisou_femme_et_femme_couleur4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bisou_femme_et_femme_couleur4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bisou_femme_et_femme_couleur5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bisou_femme_et_femme_couleur5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bisou_femme_et_femme_couleur5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bisou_femme_et_femme_couleur5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bisou_femme_et_femme_couleur5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"famille_femme_et_garcon", "1F469-200D-1F466"},
    {"famille_femme_garcon_et_garcon", "1F469-200D-1F466-200D-1F466"},
    {"famille_femme_et_fille", "1F469-200D-1F467"},
    {"famille_femme_fille_et_garcon", "1F469-200D-1F467-200D-1F466"},
    {"famille_femme_fille_et_fille", "1F469-200D-1F467-200D-1F467"},
    {"famille_femme_femme_et_garcon", "1F469-200D-1F469-200D-1F466"},
    {"famille_femme_femme_garcon_et_garcon", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"famille_femme_femme_et_fille", "1F469-200D-1F469-200D-1F467"},
    {"famille_femme_femme_fille_et_garcon", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"famille_femme_femme_fille_et_fille", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"deux_personnes_se_tenant_la_main", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"deux_personnes_se_tenant_la_main_couleur1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"deux_personnes_se_tenant_la_main_couleur1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"deux_personnes_se_tenant_la_main_couleur1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"deux_personnes_se_tenant_la_main_couleur1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"deux_personnes_se_tenant_la_main_couleur1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"deux_personnes_se_tenant_la_main_couleur2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"deux_personnes_se_tenant_la_main_couleur2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"deux_personnes_se_tenant_la_main_couleur2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"deux_personnes_se_tenant_la_main_couleur2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"deux_personnes_se_tenant_la_main_couleur2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"deux_personnes_se_tenant_la_main_couleur3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"deux_personnes_se_tenant_la_main_couleur3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"deux_personnes_se_tenant_la_main_couleur3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"deux_personnes_se_tenant_la_main_couleur3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"deux_personnes_se_tenant_la_main_couleur3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"deux_personnes_se_tenant_la_main_couleur4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"deux_personnes_se_tenant_la_main_couleur4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"deux_personnes_se_tenant_la_main_couleur4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"deux_personnes_se_tenant_la_main_couleur4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"deux_personnes_se_tenant_la_main_couleur4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"deux_personnes_se_tenant_la_main_couleur5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"deux_personnes_se_tenant_la_main_couleur5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"deux_personnes_se_tenant_la_main_couleur5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"deux_personnes_se_tenant_la_main_couleur5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"deux_personnes_se_tenant_la_main_couleur5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"famille_adulte_adulte_enfant", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"famille_adulte_adulte_enfant_enfant", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"famille_adulte_enfant", "1F9D1-200D-1F9D2"},
    {"famille_adulte_enfant_enfant", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"personne_qui_court_vers_la_droite", "1F3C3-200D-27A1-FE0F"},
    {"personne_qui_court_vers_la_droite_couleur1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"personne_qui_court_vers_la_droite_couleur2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"personne_qui_court_vers_la_droite_couleur3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"personne_qui_court_vers_la_droite_couleur4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"personne_qui_court_vers_la_droite_couleur5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"professionnel_de_la_sante", "1F468-200D-2695-FE0F"},
    {"professionnel_de_la_sante_couleur1", "1F468-1F3FB-200D-2695-FE0F"},
    {"professionnel_de_la_sante_couleur2", "1F468-1F3FC-200D-2695-FE0F"},
    {"professionnel_de_la_sante_couleur3", "1F468-1F3FD-200D-2695-FE0F"},
    {"professionnel_de_la_sante_couleur4", "1F468-1F3FE-200D-2695-FE0F"},
    {"professionnel_de_la_sante_couleur5", "1F468-1F3FF-200D-2695-FE0F"},
    {"juge_homme", "1F468-200D-2696-FE0F"},
    {"juge_homme_couleur1", "1F468-1F3FB-200D-2696-FE0F"},
    {"juge_homme_couleur2", "1F468-1F3FC-200D-2696-FE0F"},
    {"juge_homme_couleur3", "1F468-1F3FD-200D-2696-FE0F"},
    {"juge_homme_couleur4", "1F468-1F3FE-200D-2696-FE0F"},
    {"juge_homme_couleur5", "1F468-1F3FF-200D-2696-FE0F"},
    {"pilote_homme", "1F468-200D-2708-FE0F"},
    {"pilote_homme_couleur1", "1F468-1F3FB-200D-2708-FE0F"},
    {"pilote_homme_couleur2", "1F468-1F3FC-200D-2708-FE0F"},
    {"pilote_homme_couleur3", "1F468-1F3FD-200D-2708-FE0F"},
    {"pilote_homme_couleur4", "1F468-1F3FE-200D-2708-FE0F"},
    {"pilote_homme_couleur5", "1F468-1F3FF-200D-2708-FE0F"},
    {"fermier", "1F468-200D-1F33E"},
    {"fermier_couleur1", "1F468-1F3FB-200D-1F33E"},
    {"fermier_couleur2", "1F468-1F3FC-200D-1F33E"},
    {"fermier_couleur3", "1F468-1F3FD-200D-1F33E"},
    {"fermier_couleur4", "1F468-1F3FE-200D-1F33E"},
    {"fermier_couleur5", "1F468-1F3FF-200D-1F33E"},
    {"cuisinier", "1F468-200D-1F373"},
    {"cuisinier_couleur1", "1F468-1F3FB-200D-1F373"},
    {"cuisinier_couleur2", "1F468-1F3FC-200D-1F373"},
    {"cuisinier_couleur3", "1F468-1F3FD-200D-1F373"},
    {"cuisinier_couleur4", "1F468-1F3FE-200D-1F373"},
    {"cuisinier_couleur5", "1F468-1F3FF-200D-1F373"},
    {"homme_allaitant_un_bebe", "1F468-200D-1F37C"},
    {"homme_allaitant_un_bebe_couleur1", "1F468-1F3FB-200D-1F37C"},
    {"homme_allaitant_un_bebe_couleur2", "1F468-1F3FC-200D-1F37C"},
    {"homme_allaitant_un_bebe_couleur3", "1F468-1F3FD-200D-1F37C"},
    {"homme_allaitant_un_bebe_couleur4", "1F468-1F3FE-200D-1F37C"},
    {"homme_allaitant_un_bebe_couleur5", "1F468-1F3FF-200D-1F37C"},
    {"etudiant", "1F468-200D-1F393"},
    {"etudiant_couleur1", "1F468-1F3FB-200D-1F393"},
    {"etudiant_couleur2", "1F468-1F3FC-200D-1F393"},
    {"etudiant_couleur3", "1F468-1F3FD-200D-1F393"},
    {"etudiant_couleur4", "1F468-1F3FE-200D-1F393"},
    {"etudiant_couleur5", "1F468-1F3FF-200D-1F393"},
    {"chanteur", "1F468-200D-1F3A4"},
    {"chanteur_couleur1", "1F468-1F3FB-200D-1F3A4"},
    {"chanteur_couleur2", "1F468-1F3FC-200D-1F3A4"},
    {"chanteur_couleur3", "1F468-1F3FD-200D-1F3A4"},
    {"chanteur_couleur4", "1F468-1F3FE-200D-1F3A4"},
    {"chanteur_couleur5", "1F468-1F3FF-200D-1F3A4"},
    {"artiste_homme", "1F468-200D-1F3A8"},
    {"artiste_homme_couleur1", "1F468-1F3FB-200D-1F3A8"},
    {"artiste_homme_couleur2", "1F468-1F3FC-200D-1F3A8"},
    {"artiste_homme_couleur3", "1F468-1F3FD-200D-1F3A8"},
    {"artiste_homme_couleur4", "1F468-1F3FE-200D-1F3A8"},
    {"artiste_homme_couleur5", "1F468-1F3FF-200D-1F3A8"},
    {"enseignant", "1F468-200D-1F3EB"},
    {"enseignant_couleur1", "1F468-1F3FB-200D-1F3EB"},
    {"enseignant_couleur2", "1F468-1F3FC-200D-1F3EB"},
    {"enseignant_couleur3", "1F468-1F3FD-200D-1F3EB"},
    {"enseignant_couleur4", "1F468-1F3FE-200D-1F3EB"},
    {"enseignant_couleur5", "1F468-1F3FF-200D-1F3EB"},
    {"ouvrier", "1F468-200D-1F3ED"},
    {"ouvrier_couleur1", "1F468-1F3FB-200D-1F3ED"},
    {"ouvrier_couleur2", "1F468-1F3FC-200D-1F3ED"},
    {"ouvrier_couleur3", "1F468-1F3FD-200D-1F3ED"},
    {"ouvrier_couleur4", "1F468-1F3FE-200D-1F3ED"},
    {"ouvrier_couleur5", "1F468-1F3FF-200D-1F3ED"},
    {"informaticien", "1F468-200D-1F4BB"},
    {"informaticien_couleur1", "1F468-1F3FB-200D-1F4BB"},
    {"informaticien_couleur2", "1F468-1F3FC-200D-1F4BB"},
    {"informaticien_couleur3", "1F468-1F3FD-200D-1F4BB"},
    {"informaticien_couleur4", "1F468-1F3FE-200D-1F4BB"},
    {"informaticien_couleur5", "1F468-1F3FF-200D-1F4BB"},
    {"employe_de_bureau", "1F468-200D-1F4BC"},
    {"employe_de_bureau_couleur1", "1F468-1F3FB-200D-1F4BC"},
    {"employe_de_bureau_couleur2", "1F468-1F3FC-200D-1F4BC"},
    {"employe_de_bureau_couleur3", "1F468-1F3FD-200D-1F4BC"},
    {"employe_de_bureau_couleur4", "1F468-1F3FE-200D-1F4BC"},
    {"employe_de_bureau_couleur5", "1F468-1F3FF-200D-1F4BC"},
    {"mecanicien", "1F468-200D-1F527"},
    {"mecanicien_couleur1", "1F468-1F3FB-200D-1F527"},
    {"mecanicien_couleur2", "1F468-1F3FC-200D-1F527"},
    {"mecanicien_couleur3", "1F468-1F3FD-200D-1F527"},
    {"mecanicien_couleur4", "1F468-1F3FE-200D-1F527"},
    {"mecanicien_couleur5", "1F468-1F3FF-200D-1F527"},
    {"scientifique_homme", "1F468-200D-1F52C"},
    {"scientifique_homme_couleur1", "1F468-1F3FB-200D-1F52C"},
    {"scientifique_homme_couleur2", "1F468-1F3FC-200D-1F52C"},
    {"scientifique_homme_couleur3", "1F468-1F3FD-200D-1F52C"},
    {"scientifique_homme_couleur4", "1F468-1F3FE-200D-1F52C"},
    {"scientifique_homme_couleur5", "1F468-1F3FF-200D-1F52C"},
    {"astronaute_homme", "1F468-200D-1F680"},
    {"astronaute_homme_couleur1", "1F468-1F3FB-200D-1F680"},
    {"astronaute_homme_couleur2", "1F468-1F3FC-200D-1F680"},
    {"astronaute_homme_couleur3", "1F468-1F3FD-200D-1F680"},
    {"astronaute_homme_couleur4", "1F468-1F3FE-200D-1F680"},
    {"astronaute_homme_couleur5", "1F468-1F3FF-200D-1F680"},
    {"pompier_homme", "1F468-200D-1F692"},
    {"pompier_homme_couleur1", "1F468-1F3FB-200D-1F692"},
    {"pompier_homme_couleur2", "1F468-1F3FC-200D-1F692"},
    {"pompier_homme_couleur3", "1F468-1F3FD-200D-1F692"},
    {"pompier_homme_couleur4", "1F468-1F3FE-200D-1F692"},
    {"pompier_homme_couleur5", "1F468-1F3FF-200D-1F692"},
    {"homme_avec_canne_blanche", "1F468-200D-1F9AF"},
    {"homme_avec_canne_blanche_couleur1", "1F468-1F3FB-200D-1F9AF"},
    {"homme_avec_canne_blanche_couleur2", "1F468-1F3FC-200D-1F9AF"},
    {"homme_avec_canne_blanche_couleur3", "1F468-1F3FD-200D-1F9AF"},
    {"homme_avec_canne_blanche_couleur4", "1F468-1F3FE-200D-1F9AF"},
    {"homme_avec_canne_blanche_couleur5", "1F468-1F3FF-200D-1F9AF"},
    {"homme_avec_canne_blanche_vers_la_droite", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"homme_avec_canne_blanche_vers_la_droite_couleur1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"homme_avec_canne_blanche_vers_la_droite_couleur2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"homme_avec_canne_blanche_vers_la_droite_couleur3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"homme_avec_canne_blanche_vers_la_droite_couleur4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"homme_avec_canne_blanche_vers_la_droite_couleur5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"homme_en_fauteuil_motorise", "1F468-200D-1F9BC"},
    {"homme_en_fauteuil_motorise_couleur1", "1F468-1F3FB-200D-1F9BC"},
    {"homme_en_fauteuil_motorise_couleur2", "1F468-1F3FC-200D-1F9BC"},
    {"homme_en_fauteuil_motorise_couleur3", "1F468-1F3FD-200D-1F9BC"},
    {"homme_en_fauteuil_motorise_couleur4", "1F468-1F3FE-200D-1F9BC"},
    {"homme_en_fauteuil_motorise_couleur5", "1F468-1F3FF-200D-1F9BC"},
    {"homme_en_fauteuil_motorise_vers_la_droite", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"homme_en_fauteuil_motorise_vers_la_droite_couleur1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"homme_en_fauteuil_motorise_vers_la_droite_couleur2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"homme_en_fauteuil_motorise_vers_la_droite_couleur3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"homme_en_fauteuil_motorise_vers_la_droite_couleur4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"homme_en_fauteuil_motorise_vers_la_droite_couleur5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"homme_en_fauteuil_roulant_manuel", "1F468-200D-1F9BD"},
    {"homme_en_fauteuil_roulant_manuel_couleur1", "1F468-1F3FB-200D-1F9BD"},
    {"homme_en_fauteuil_roulant_manuel_couleur2", "1F468-1F3FC-200D-1F9BD"},
    {"homme_en_fauteuil_roulant_manuel_couleur3", "1F468-1F3FD-200D-1F9BD"},
    {"homme_en_fauteuil_roulant_manuel_couleur4", "1F468-1F3FE-200D-1F9BD"},
    {"homme_en_fauteuil_roulant_manuel_couleur5", "1F468-1F3FF-200D-1F9BD"},
    {"homme_en_fauteuil_roulant_manuel_vers_la_droite", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"homme_en_fauteuil_roulant_manuel_vers_la_droite_couleur1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"homme_en_fauteuil_roulant_manuel_vers_la_droite_couleur2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"homme_en_fauteuil_roulant_manuel_vers_la_droite_couleur3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"homme_en_fauteuil_roulant_manuel_vers_la_droite_couleur4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"homme_en_fauteuil_roulant_manuel_vers_la_droite_couleur5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"professionnelle_de_la_sante", "1F469-200D-2695-FE0F"},
    {"professionnelle_de_la_sante_couleur1", "1F469-1F3FB-200D-2695-FE0F"},
    {"professionnelle_de_la_sante_couleur2", "1F469-1F3FC-200D-2695-FE0F"},
    {"professionnelle_de_la_sante_couleur3", "1F469-1F3FD-200D-2695-FE0F"},
    {"professionnelle_de_la_sante_couleur4", "1F469-1F3FE-200D-2695-FE0F"},
    {"professionnelle_de_la_sante_couleur5", "1F469-1F3FF-200D-2695-FE0F"},
    {"juge_femme", "1F469-200D-2696-FE0F"},
    {"juge_femme_couleur1", "1F469-1F3FB-200D-2696-FE0F"},
    {"juge_femme_couleur2", "1F469-1F3FC-200D-2696-FE0F"},
    {"juge_femme_couleur3", "1F469-1F3FD-200D-2696-FE0F"},
    {"juge_femme_couleur4", "1F469-1F3FE-200D-2696-FE0F"},
    {"juge_femme_couleur5", "1F469-1F3FF-200D-2696-FE0F"},
    {"pilote_femme", "1F469-200D-2708-FE0F"},
    {"pilote_femme_couleur1", "1F469-1F3FB-200D-2708-FE0F"},
    {"pilote_femme_couleur2", "1F469-1F3FC-200D-2708-FE0F"},
    {"pilote_femme_couleur3", "1F469-1F3FD-200D-2708-FE0F"},
    {"pilote_femme_couleur4", "1F469-1F3FE-200D-2708-FE0F"},
    {"pilote_femme_couleur5", "1F469-1F3FF-200D-2708-FE0F"},
    {"fermiere", "1F469-200D-1F33E"},
    {"fermiere_couleur1", "1F469-1F3FB-200D-1F33E"},
    {"fermiere_couleur2", "1F469-1F3FC-200D-1F33E"},
    {"fermiere_couleur3", "1F469-1F3FD-200D-1F33E"},
    {"fermiere_couleur4", "1F469-1F3FE-200D-1F33E"},
    {"fermiere_couleur5", "1F469-1F3FF-200D-1F33E"},
    {"cuisiniere", "1F469-200D-1F373"},
    {"cuisiniere_couleur1", "1F469-1F3FB-200D-1F373"},
    {"cuisiniere_couleur2", "1F469-1F3FC-200D-1F373"},
    {"cuisiniere_couleur3", "1F469-1F3FD-200D-1F373"},
    {"cuisiniere_couleur4", "1F469-1F3FE-200D-1F373"},
    {"cuisiniere_couleur5", "1F469-1F3FF-200D-1F373"},
    {"femme_allaitant_un_bebe", "1F469-200D-1F37C"},
    {"femme_allaitant_un_bebe_couleur1", "1F469-1F3FB-200D-1F37C"},
    {"femme_allaitant_un_bebe_couleur2", "1F469-1F3FC-200D-1F37C"},
    {"femme_allaitant_un_bebe_couleur3", "1F469-1F3FD-200D-1F37C"},
    {"femme_allaitant_un_bebe_couleur4", "1F469-1F3FE-200D-1F37C"},
    {"femme_allaitant_un_bebe_couleur5", "1F469-1F3FF-200D-1F37C"},
    {"etudiante", "1F469-200D-1F393"},
    {"etudiante_couleur1", "1F469-1F3FB-200D-1F393"},
    {"etudiante_couleur2", "1F469-1F3FC-200D-1F393"},
    {"etudiante_couleur3", "1F469-1F3FD-200D-1F393"},
    {"etudiante_couleur4", "1F469-1F3FE-200D-1F393"},
    {"etudiante_couleur5", "1F469-1F3FF-200D-1F393"},
    {"chanteuse", "1F469-200D-1F3A4"},
    {"chanteuse_couleur1", "1F469-1F3FB-200D-1F3A4"},
    {"chanteuse_couleur2", "1F469-1F3FC-200D-1F3A4"},
    {"chanteuse_couleur3", "1F469-1F3FD-200D-1F3A4"},
    {"chanteuse_couleur4", "1F469-1F3FE-200D-1F3A4"},
    {"chanteuse_couleur5", "1F469-1F3FF-200D-1F3A4"},
    {"artiste_femme", "1F469-200D-1F3A8"},
    {"artiste_femme_couleur1", "1F469-1F3FB-200D-1F3A8"},
    {"artiste_femme_couleur2", "1F469-1F3FC-200D-1F3A8"},
    {"artiste_femme_couleur3", "1F469-1F3FD-200D-1F3A8"},
    {"artiste_femme_couleur4", "1F469-1F3FE-200D-1F3A8"},
    {"artiste_femme_couleur5", "1F469-1F3FF-200D-1F3A8"},
    {"enseignante", "1F469-200D-1F3EB"},
    {"enseignante_couleur1", "1F469-1F3FB-200D-1F3EB"},
    {"enseignante_couleur2", "1F469-1F3FC-200D-1F3EB"},
    {"enseignante_couleur3", "1F469-1F3FD-200D-1F3EB"},
    {"enseignante_couleur4", "1F469-1F3FE-200D-1F3EB"},
    {"enseignante_couleur5", "1F469-1F3FF-200D-1F3EB"},
    {"ouvriere", "1F469-200D-1F3ED"},
    {"ouvriere_couleur1", "1F469-1F3FB-200D-1F3ED"},
    {"ouvriere_couleur2", "1F469-1F3FC-200D-1F3ED"},
    {"ouvriere_couleur3", "1F469-1F3FD-200D-1F3ED"},
    {"ouvriere_couleur4", "1F469-1F3FE-200D-1F3ED"},
    {"ouvriere_couleur5", "1F469-1F3FF-200D-1F3ED"},
    {"informaticienne", "1F469-200D-1F4BB"},
    {"informaticienne_couleur1", "1F469-1F3FB-200D-1F4BB"},
    {"informaticienne_couleur2", "1F469-1F3FC-200D-1F4BB"},
    {"informaticienne_couleur3", "1F469-1F3FD-200D-1F4BB"},
    {"informaticienne_couleur4", "1F469-1F3FE-200D-1F4BB"},
    {"informaticienne_couleur5", "1F469-1F3FF-200D-1F4BB"},
    {"employee_de_bureau", "1F469-200D-1F4BC"},
    {"employee_de_bureau_couleur1", "1F469-1F3FB-200D-1F4BC"},
    {"employee_de_bureau_couleur2", "1F469-1F3FC-200D-1F4BC"},
    {"employee_de_bureau_couleur3", "1F469-1F3FD-200D-1F4BC"},
    {"employee_de_bureau_couleur4", "1F469-1F3FE-200D-1F4BC"},
    {"employee_de_bureau_couleur5", "1F469-1F3FF-200D-1F4BC"},
    {"mecanicienne", "1F469-200D-1F527"},
    {"mecanicienne_couleur1", "1F469-1F3FB-200D-1F527"},
    {"mecanicienne_couleur2", "1F469-1F3FC-200D-1F527"},
    {"mecanicienne_couleur3", "1F469-1F3FD-200D-1F527"},
    {"mecanicienne_couleur4", "1F469-1F3FE-200D-1F527"},
    {"mecanicienne_couleur5", "1F469-1F3FF-200D-1F527"},
    {"scientifique_femme", "1F469-200D-1F52C"},
    {"scientifique_femme_couleur1", "1F469-1F3FB-200D-1F52C"},
    {"scientifique_femme_couleur2", "1F469-1F3FC-200D-1F52C"},
    {"scientifique_femme_couleur3", "1F469-1F3FD-200D-1F52C"},
    {"scientifique_femme_couleur4", "1F469-1F3FE-200D-1F52C"},
    {"scientifique_femme_couleur5", "1F469-1F3FF-200D-1F52C"},
    {"astronaute_femme", "1F469-200D-1F680"},
    {"astronaute_femme_couleur1", "1F469-1F3FB-200D-1F680"},
    {"astronaute_femme_couleur2", "1F469-1F3FC-200D-1F680"},
    {"astronaute_femme_couleur3", "1F469-1F3FD-200D-1F680"},
    {"astronaute_femme_couleur4", "1F469-1F3FE-200D-1F680"},
    {"astronaute_femme_couleur5", "1F469-1F3FF-200D-1F680"},
    {"pompier_femme", "1F469-200D-1F692"},
    {"pompier_femme_couleur1", "1F469-1F3FB-200D-1F692"},
    {"pompier_femme_couleur2", "1F469-1F3FC-200D-1F692"},
    {"pompier_femme_couleur3", "1F469-1F3FD-200D-1F692"},
    {"pompier_femme_couleur4", "1F469-1F3FE-200D-1F692"},
    {"pompier_femme_couleur5", "1F469-1F3FF-200D-1F692"},
    {"femme_avec_canne_blanche", "1F469-200D-1F9AF"},
    {"femme_avec_canne_blanche_couleur1", "1F469-1F3FB-200D-1F9AF"},
    {"femme_avec_canne_blanche_couleur2", "1F469-1F3FC-200D-1F9AF"},
    {"femme_avec_canne_blanche_couleur3", "1F469-1F3FD-200D-1F9AF"},
    {"femme_avec_canne_blanche_couleur4", "1F469-1F3FE-200D-1F9AF"},
    {"femme_avec_canne_blanche_couleur5", "1F469-1F3FF-200D-1F9AF"},
    {"femme_avec_canne_blanche_vers_la_droite", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"femme_avec_canne_blanche_vers_la_droite_couleur1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"femme_avec_canne_blanche_vers_la_droite_couleur2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"femme_avec_canne_blanche_vers_la_droite_couleur3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"femme_avec_canne_blanche_vers_la_droite_couleur4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"femme_avec_canne_blanche_vers_la_droite_couleur5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"femme_en_fauteuil_motorise", "1F469-200D-1F9BC"},
    {"femme_en_fauteuil_motorise_couleur1", "1F469-1F3FB-200D-1F9BC"},
    {"femme_en_fauteuil_motorise_couleur2", "1F469-1F3FC-200D-1F9BC"},
    {"femme_en_fauteuil_motorise_couleur3", "1F469-1F3FD-200D-1F9BC"},
    {"femme_en_fauteuil_motorise_couleur4", "1F469-1F3FE-200D-1F9BC"},
    {"femme_en_fauteuil_motorise_couleur5", "1F469-1F3FF-200D-1F9BC"},
    {"femme_en_fauteuil_motorise_vers_la_droite", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"femme_en_fauteuil_motorise_vers_la_droite_couleur1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"femme_en_fauteuil_motorise_vers_la_droite_couleur2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"femme_en_fauteuil_motorise_vers_la_droite_couleur3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"femme_en_fauteuil_motorise_vers_la_droite_couleur4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"femme_en_fauteuil_motorise_vers_la_droite_couleur5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"femme_en_fauteuil_roulant_manuel", "1F469-200D-1F9BD"},
    {"femme_en_fauteuil_roulant_manuel_couleur1", "1F469-1F3FB-200D-1F9BD"},
    {"femme_en_fauteuil_roulant_manuel_couleur2", "1F469-1F3FC-200D-1F9BD"},
    {"femme_en_fauteuil_roulant_manuel_couleur3", "1F469-1F3FD-200D-1F9BD"},
    {"femme_en_fauteuil_roulant_manuel_couleur4", "1F469-1F3FE-200D-1F9BD"},
    {"femme_en_fauteuil_roulant_manuel_couleur5", "1F469-1F3FF-200D-1F9BD"},
    {"femme_en_fauteuil_roulant_manuel_vers_la_droite", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"femme_en_fauteuil_roulant_manuel_vers_la_droite_couleur1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"femme_en_fauteuil_roulant_manuel_vers_la_droite_couleur2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"femme_en_fauteuil_roulant_manuel_vers_la_droite_couleur3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"femme_en_fauteuil_roulant_manuel_vers_la_droite_couleur4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"femme_en_fauteuil_roulant_manuel_vers_la_droite_couleur5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"personne_qui_marche_vers_la_droite", "1F6B6-200D-27A1-FE0F"},
    {"personne_qui_marche_vers_la_droite_couleur1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"personne_qui_marche_vers_la_droite_couleur2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"personne_qui_marche_vers_la_droite_couleur3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"personne_qui_marche_vers_la_droite_couleur4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"personne_qui_marche_vers_la_droite_couleur5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"personne_a_genoux_vers_la_droite", "1F9CE-200D-27A1-FE0F"},
    {"personne_a_genoux_vers_la_droite_couleur1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"personne_a_genoux_vers_la_droite_couleur2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"personne_a_genoux_vers_la_droite_couleur3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"personne_a_genoux_vers_la_droite_couleur4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"personne_a_genoux_vers_la_droite_couleur5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"professionnel_de_la_sante_tous_genres", "1F9D1-200D-2695-FE0F"},
    {"professionnel_de_la_sante_tous_genres_couleur1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"professionnel_de_la_sante_tous_genres_couleur2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"professionnel_de_la_sante_tous_genres_couleur3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"professionnel_de_la_sante_tous_genres_couleur4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"professionnel_de_la_sante_tous_genres_couleur5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"juge", "1F9D1-200D-2696-FE0F"},
    {"juge_couleur1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"juge_couleur2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"juge_couleur3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"juge_couleur4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"juge_couleur5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"pilote", "1F9D1-200D-2708-FE0F"},
    {"pilote_couleur1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"pilote_couleur2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"pilote_couleur3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"pilote_couleur4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"pilote_couleur5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"fermier_tous_genres", "1F9D1-200D-1F33E"},
    {"fermier_tous_genres_couleur1", "1F9D1-1F3FB-200D-1F33E"},
    {"fermier_tous_genres_couleur2", "1F9D1-1F3FC-200D-1F33E"},
    {"fermier_tous_genres_couleur3", "1F9D1-1F3FD-200D-1F33E"},
    {"fermier_tous_genres_couleur4", "1F9D1-1F3FE-200D-1F33E"},
    {"fermier_tous_genres_couleur5", "1F9D1-1F3FF-200D-1F33E"},
    {"cuisinier_tous_genres", "1F9D1-200D-1F373"},
    {"cuisinier_tous_genres_couleur1", "1F9D1-1F3FB-200D-1F373"},
    {"cuisinier_tous_genres_couleur2", "1F9D1-1F3FC-200D-1F373"},
    {"cuisinier_tous_genres_couleur3", "1F9D1-1F3FD-200D-1F373"},
    {"cuisinier_tous_genres_couleur4", "1F9D1-1F3FE-200D-1F373"},
    {"cuisinier_tous_genres_couleur5", "1F9D1-1F3FF-200D-1F373"},
    {"personne_allaitant_un_bebe", "1F9D1-200D-1F37C"},
    {"personne_allaitant_un_bebe_couleur1", "1F9D1-1F3FB-200D-1F37C"},
    {"personne_allaitant_un_bebe_couleur2", "1F9D1-1F3FC-200D-1F37C"},
    {"personne_allaitant_un_bebe_couleur3", "1F9D1-1F3FD-200D-1F37C"},
    {"personne_allaitant_un_bebe_couleur4", "1F9D1-1F3FE-200D-1F37C"},
    {"personne_allaitant_un_bebe_couleur5", "1F9D1-1F3FF-200D-1F37C"},
    {"santa", "1F9D1-200D-1F384"},
    {"santa_couleur1", "1F9D1-1F3FB-200D-1F384"},
    {"santa_couleur2", "1F9D1-1F3FC-200D-1F384"},
    {"santa_couleur3", "1F9D1-1F3FD-200D-1F384"},
    {"santa_couleur4", "1F9D1-1F3FE-200D-1F384"},
    {"santa_couleur5", "1F9D1-1F3FF-200D-1F384"},
    {"etudiant_tous_genres", "1F9D1-200D-1F393"},
    {"etudiant_tous_genres_couleur1", "1F9D1-1F3FB-200D-1F393"},
    {"etudiant_tous_genres_couleur2", "1F9D1-1F3FC-200D-1F393"},
    {"etudiant_tous_genres_couleur3", "1F9D1-1F3FD-200D-1F393"},
    {"etudiant_tous_genres_couleur4", "1F9D1-1F3FE-200D-1F393"},
    {"etudiant_tous_genres_couleur5", "1F9D1-1F3FF-200D-1F393"},
    {"chanteur_tous_genres", "1F9D1-200D-1F3A4"},
    {"chanteur_tous_genres_couleur1", "1F9D1-1F3FB-200D-1F3A4"},
    {"chanteur_tous_genres_couleur2", "1F9D1-1F3FC-200D-1F3A4"},
    {"chanteur_tous_genres_couleur3", "1F9D1-1F3FD-200D-1F3A4"},
    {"chanteur_tous_genres_couleur4", "1F9D1-1F3FE-200D-1F3A4"},
    {"chanteur_tous_genres_couleur5", "1F9D1-1F3FF-200D-1F3A4"},
    {"artiste", "1F9D1-200D-1F3A8"},
    {"artiste_couleur1", "1F9D1-1F3FB-200D-1F3A8"},
    {"artiste_couleur2", "1F9D1-1F3FC-200D-1F3A8"},
    {"artiste_couleur3", "1F9D1-1F3FD-200D-1F3A8"},
    {"artiste_couleur4", "1F9D1-1F3FE-200D-1F3A8"},
    {"artiste_couleur5", "1F9D1-1F3FF-200D-1F3A8"},
    {"personnel_enseignant", "1F9D1-200D-1F3EB"},
    {"personnel_enseignant_couleur1", "1F9D1-1F3FB-200D-1F3EB"},
    {"personnel_enseignant_couleur2", "1F9D1-1F3FC-200D-1F3EB"},
    {"personnel_enseignant_couleur3", "1F9D1-1F3FD-200D-1F3EB"},
    {"personnel_enseignant_couleur4", "1F9D1-1F3FE-200D-1F3EB"},
    {"personnel_enseignant_couleur5", "1F9D1-1F3FF-200D-1F3EB"},
    {"ouvrier_tous_genres", "1F9D1-200D-1F3ED"},
    {"ouvrier_tous_genres_couleur1", "1F9D1-1F3FB-200D-1F3ED"},
    {"ouvrier_tous_genres_couleur2", "1F9D1-1F3FC-200D-1F3ED"},
    {"ouvrier_tous_genres_couleur3", "1F9D1-1F3FD-200D-1F3ED"},
    {"ouvrier_tous_genres_couleur4", "1F9D1-1F3FE-200D-1F3ED"},
    {"ouvrier_tous_genres_couleur5", "1F9D1-1F3FF-200D-1F3ED"},
    {"informaticien_tous_genres", "1F9D1-200D-1F4BB"},
    {"informaticien_tous_genres_couleur1", "1F9D1-1F3FB-200D-1F4BB"},
    {"informaticien_tous_genres_couleur2", "1F9D1-1F3FC-200D-1F4BB"},
    {"informaticien_tous_genres_couleur3", "1F9D1-1F3FD-200D-1F4BB"},
    {"informaticien_tous_genres_couleur4", "1F9D1-1F3FE-200D-1F4BB"},
    {"informaticien_tous_genres_couleur5", "1F9D1-1F3FF-200D-1F4BB"},
    {"employe_de_bureau_tous_genres", "1F9D1-200D-1F4BC"},
    {"employe_de_bureau_tous_genres_couleur1", "1F9D1-1F3FB-200D-1F4BC"},
    {"employe_de_bureau_tous_genres_couleur2", "1F9D1-1F3FC-200D-1F4BC"},
    {"employe_de_bureau_tous_genres_couleur3", "1F9D1-1F3FD-200D-1F4BC"},
    {"employe_de_bureau_tous_genres_couleur4", "1F9D1-1F3FE-200D-1F4BC"},
    {"employe_de_bureau_tous_genres_couleur5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mecanicien_tous_genres", "1F9D1-200D-1F527"},
    {"mecanicien_tous_genres_couleur1", "1F9D1-1F3FB-200D-1F527"},
    {"mecanicien_tous_genres_couleur2", "1F9D1-1F3FC-200D-1F527"},
    {"mecanicien_tous_genres_couleur3", "1F9D1-1F3FD-200D-1F527"},
    {"mecanicien_tous_genres_couleur4", "1F9D1-1F3FE-200D-1F527"},
    {"mecanicien_tous_genres_couleur5", "1F9D1-1F3FF-200D-1F527"},
    {"scientifique", "1F9D1-200D-1F52C"},
    {"scientifique_couleur1", "1F9D1-1F3FB-200D-1F52C"},
    {"scientifique_couleur2", "1F9D1-1F3FC-200D-1F52C"},
    {"scientifique_couleur3", "1F9D1-1F3FD-200D-1F52C"},
    {"scientifique_couleur4", "1F9D1-1F3FE-200D-1F52C"},
    {"scientifique_couleur5", "1F9D1-1F3FF-200D-1F52C"},
    {"astronaute", "1F9D1-200D-1F680"},
    {"astronaute_couleur1", "1F9D1-1F3FB-200D-1F680"},
    {"astronaute_couleur2", "1F9D1-1F3FC-200D-1F680"},
    {"astronaute_couleur3", "1F9D1-1F3FD-200D-1F680"},
    {"astronaute_couleur4", "1F9D1-1F3FE-200D-1F680"},
    {"astronaute_couleur5", "1F9D1-1F3FF-200D-1F680"},
    {"pompier", "1F9D1-200D-1F692"},
    {"pompier_couleur1", "1F9D1-1F3FB-200D-1F692"},
    {"pompier_couleur2", "1F9D1-1F3FC-200D-1F692"},
    {"pompier_couleur3", "1F9D1-1F3FD-200D-1F692"},
    {"pompier_couleur4", "1F9D1-1F3FE-200D-1F692"},
    {"pompier_couleur5", "1F9D1-1F3FF-200D-1F692"},
    {"personne_avec_une_canne_blanche", "1F9D1-200D-1F9AF"},
    {"personne_avec_une_canne_blanche_couleur1", "1F9D1-1F3FB-200D-1F9AF"},
    {"personne_avec_une_canne_blanche_couleur2", "1F9D1-1F3FC-200D-1F9AF"},
    {"personne_avec_une_canne_blanche_couleur3", "1F9D1-1F3FD-200D-1F9AF"},
    {"personne_avec_une_canne_blanche_couleur4", "1F9D1-1F3FE-200D-1F9AF"},
    {"personne_avec_une_canne_blanche_couleur5", "1F9D1-1F3FF-200D-1F9AF"},
    {"personne_avec_une_canne_blanche_vers_la_droite", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"personne_avec_une_canne_blanche_vers_la_droite_couleur1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"personne_avec_une_canne_blanche_vers_la_droite_couleur2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"personne_avec_une_canne_blanche_vers_la_droite_couleur3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"personne_avec_une_canne_blanche_vers_la_droite_couleur4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"personne_avec_une_canne_blanche_vers_la_droite_couleur5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"personne_en_fauteuil_motorise", "1F9D1-200D-1F9BC"},
    {"personne_en_fauteuil_motorise_couleur1", "1F9D1-1F3FB-200D-1F9BC"},
    {"personne_en_fauteuil_motorise_couleur2", "1F9D1-1F3FC-200D-1F9BC"},
    {"personne_en_fauteuil_motorise_couleur3", "1F9D1-1F3FD-200D-1F9BC"},
    {"personne_en_fauteuil_motorise_couleur4", "1F9D1-1F3FE-200D-1F9BC"},
    {"personne_en_fauteuil_motorise_couleur5", "1F9D1-1F3FF-200D-1F9BC"},
    {"personne_en_fauteuil_motorise_vers_la_droite", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"personne_en_fauteuil_motorise_vers_la_droite_couleur1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"personne_en_fauteuil_motorise_vers_la_droite_couleur2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"personne_en_fauteuil_motorise_vers_la_droite_couleur3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"personne_en_fauteuil_motorise_vers_la_droite_couleur4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"personne_en_fauteuil_motorise_vers_la_droite_couleur5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"personne_en_fauteuil_roulant_manuel", "1F9D1-200D-1F9BD"},
    {"personne_en_fauteuil_roulant_manuel_couleur1", "1F9D1-1F3FB-200D-1F9BD"},
    {"personne_en_fauteuil_roulant_manuel_couleur2", "1F9D1-1F3FC-200D-1F9BD"},
    {"personne_en_fauteuil_roulant_manuel_couleur3", "1F9D1-1F3FD-200D-1F9BD"},
    {"personne_en_fauteuil_roulant_manuel_couleur4", "1F9D1-1F3FE-200D-1F9BD"},
    {"personne_en_fauteuil_roulant_manuel_couleur5", "1F9D1-1F3FF-200D-1F9BD"},
    {"personne_en_fauteuil_roulant_manuel_vers_la_droite", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"personne_en_fauteuil_roulant_manuel_vers_la_droite_couleur1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"personne_en_fauteuil_roulant_manuel_vers_la_droite_couleur2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"personne_en_fauteuil_roulant_manuel_vers_la_droite_couleur3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"personne_en_fauteuil_roulant_manuel_vers_la_droite_couleur4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"personne_en_fauteuil_roulant_manuel_vers_la_droite_couleur5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"femme_avec_ballon", "26F9-FE0F-200D-2640-FE0F"},
    {"femme_avec_ballon_couleur1", "26F9-1F3FB-200D-2640-FE0F"},
    {"femme_avec_ballon_couleur2", "26F9-1F3FC-200D-2640-FE0F"},
    {"femme_avec_ballon_couleur3", "26F9-1F3FD-200D-2640-FE0F"},
    {"femme_avec_ballon_couleur4", "26F9-1F3FE-200D-2640-FE0F"},
    {"femme_avec_ballon_couleur5", "26F9-1F3FF-200D-2640-FE0F"},
    {"homme_avec_ballon", "26F9-FE0F-200D-2642-FE0F"},
    {"homme_avec_ballon_couleur1", "26F9-1F3FB-200D-2642-FE0F"},
    {"homme_avec_ballon_couleur2", "26F9-1F3FC-200D-2642-FE0F"},
    {"homme_avec_ballon_couleur3", "26F9-1F3FD-200D-2642-FE0F"},
    {"homme_avec_ballon_couleur4", "26F9-1F3FE-200D-2642-FE0F"},
    {"homme_avec_ballon_couleur5", "26F9-1F3FF-200D-2642-FE0F"},
    {"femme_qui_court", "1F3C3-200D-2640-FE0F"},
    {"femme_qui_court_couleur1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"femme_qui_court_couleur2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"femme_qui_court_couleur3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"femme_qui_court_couleur4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"femme_qui_court_couleur5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"femme_qui_court_vers_la_droite", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_qui_court_vers_la_droite_couleur1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_qui_court_vers_la_droite_couleur2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_qui_court_vers_la_droite_couleur3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_qui_court_vers_la_droite_couleur4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_qui_court_vers_la_droite_couleur5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"homme_qui_court", "1F3C3-200D-2642-FE0F"},
    {"homme_qui_court_couleur1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"homme_qui_court_couleur2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"homme_qui_court_couleur3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"homme_qui_court_couleur4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"homme_qui_court_couleur5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"homme_qui_court_vers_la_droite", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_qui_court_vers_la_droite_couleur1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_qui_court_vers_la_droite_couleur2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_qui_court_vers_la_droite_couleur3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_qui_court_vers_la_droite_couleur4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_qui_court_vers_la_droite_couleur5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"surfeuse", "1F3C4-200D-2640-FE0F"},
    {"surfeuse_couleur1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"surfeuse_couleur2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"surfeuse_couleur3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"surfeuse_couleur4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"surfeuse_couleur5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"surfeur", "1F3C4-200D-2642-FE0F"},
    {"surfeur_couleur1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"surfeur_couleur2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"surfeur_couleur3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"surfeur_couleur4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"surfeur_couleur5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"nageuse", "1F3CA-200D-2640-FE0F"},
    {"nageuse_couleur1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"nageuse_couleur2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"nageuse_couleur3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"nageuse_couleur4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"nageuse_couleur5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"nageur", "1F3CA-200D-2642-FE0F"},
    {"nageur_couleur1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"nageur_couleur2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"nageur_couleur3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"nageur_couleur4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"nageur_couleur5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"femme_halterophile", "1F3CB-FE0F-200D-2640-FE0F"},
    {"femme_halterophile_couleur1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"femme_halterophile_couleur2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"femme_halterophile_couleur3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"femme_halterophile_couleur4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"femme_halterophile_couleur5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"homme_halterophile", "1F3CB-FE0F-200D-2642-FE0F"},
    {"homme_halterophile_couleur1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"homme_halterophile_couleur2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"homme_halterophile_couleur3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"homme_halterophile_couleur4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"homme_halterophile_couleur5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"golfeuse", "1F3CC-FE0F-200D-2640-FE0F"},
    {"golfeuse_couleur1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"golfeuse_couleur2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"golfeuse_couleur3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"golfeuse_couleur4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"golfeuse_couleur5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"golfeur", "1F3CC-FE0F-200D-2642-FE0F"},
    {"golfeur_couleur1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"golfeur_couleur2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"golfeur_couleur3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"golfeur_couleur4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"golfeur_couleur5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"policiere", "1F46E-200D-2640-FE0F"},
    {"policiere_couleur1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"policiere_couleur2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"policiere_couleur3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"policiere_couleur4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"policiere_couleur5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"policier", "1F46E-200D-2642-FE0F"},
    {"policier_couleur1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"policier_couleur2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"policier_couleur3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"policier_couleur4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"policier_couleur5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"femmes_avec_des_oreilles_de_lapin", "1F46F-200D-2640-FE0F"},
    {"femmes_avec_des_oreilles_de_lapin_couleur1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"femmes_avec_des_oreilles_de_lapin_couleur1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"femmes_avec_des_oreilles_de_lapin_couleur1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"femmes_avec_des_oreilles_de_lapin_couleur1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"femmes_avec_des_oreilles_de_lapin_couleur2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"femmes_avec_des_oreilles_de_lapin_couleur2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"femmes_avec_des_oreilles_de_lapin_couleur2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"femmes_avec_des_oreilles_de_lapin_couleur2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"femmes_avec_des_oreilles_de_lapin_couleur3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"femmes_avec_des_oreilles_de_lapin_couleur3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"femmes_avec_des_oreilles_de_lapin_couleur3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"femmes_avec_des_oreilles_de_lapin_couleur3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"femmes_avec_des_oreilles_de_lapin_couleur4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"femmes_avec_des_oreilles_de_lapin_couleur4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"femmes_avec_des_oreilles_de_lapin_couleur4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"femmes_avec_des_oreilles_de_lapin_couleur4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"femmes_avec_des_oreilles_de_lapin_couleur5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"femmes_avec_des_oreilles_de_lapin_couleur5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"femmes_avec_des_oreilles_de_lapin_couleur5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"femmes_avec_des_oreilles_de_lapin_couleur5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"femmes_avec_des_oreilles_de_lapin_couleur1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"femmes_avec_des_oreilles_de_lapin_couleur2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"femmes_avec_des_oreilles_de_lapin_couleur3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"femmes_avec_des_oreilles_de_lapin_couleur4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"femmes_avec_des_oreilles_de_lapin_couleur5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"hommes_avec_des_oreilles_de_lapin", "1F46F-200D-2642-FE0F"},
    {"hommes_avec_des_oreilles_de_lapin_couleur1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"hommes_avec_des_oreilles_de_lapin_couleur1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"hommes_avec_des_oreilles_de_lapin_couleur1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"hommes_avec_des_oreilles_de_lapin_couleur1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"hommes_avec_des_oreilles_de_lapin_couleur2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"hommes_avec_des_oreilles_de_lapin_couleur2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"hommes_avec_des_oreilles_de_lapin_couleur2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"hommes_avec_des_oreilles_de_lapin_couleur2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"hommes_avec_des_oreilles_de_lapin_couleur3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"hommes_avec_des_oreilles_de_lapin_couleur3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"hommes_avec_des_oreilles_de_lapin_couleur3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"hommes_avec_des_oreilles_de_lapin_couleur3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"hommes_avec_des_oreilles_de_lapin_couleur4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"hommes_avec_des_oreilles_de_lapin_couleur4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"hommes_avec_des_oreilles_de_lapin_couleur4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"hommes_avec_des_oreilles_de_lapin_couleur4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"hommes_avec_des_oreilles_de_lapin_couleur5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"hommes_avec_des_oreilles_de_lapin_couleur5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"hommes_avec_des_oreilles_de_lapin_couleur5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"hommes_avec_des_oreilles_de_lapin_couleur5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"hommes_avec_des_oreilles_de_lapin_couleur1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"hommes_avec_des_oreilles_de_lapin_couleur2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"hommes_avec_des_oreilles_de_lapin_couleur3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"hommes_avec_des_oreilles_de_lapin_couleur4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"hommes_avec_des_oreilles_de_lapin_couleur5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"femme_avec_voile", "1F470-200D-2640-FE0F"},
    {"femme_avec_voile_couleur1", "1F470-1F3FB-200D-2640-FE0F"},
    {"femme_avec_voile_couleur2", "1F470-1F3FC-200D-2640-FE0F"},
    {"femme_avec_voile_couleur3", "1F470-1F3FD-200D-2640-FE0F"},
    {"femme_avec_voile_couleur4", "1F470-1F3FE-200D-2640-FE0F"},
    {"femme_avec_voile_couleur5", "1F470-1F3FF-200D-2640-FE0F"},
    {"homme_avec_voile", "1F470-200D-2642-FE0F"},
    {"homme_avec_voile_couleur1", "1F470-1F3FB-200D-2642-FE0F"},
    {"homme_avec_voile_couleur2", "1F470-1F3FC-200D-2642-FE0F"},
    {"homme_avec_voile_couleur3", "1F470-1F3FD-200D-2642-FE0F"},
    {"homme_avec_voile_couleur4", "1F470-1F3FE-200D-2642-FE0F"},
    {"homme_avec_voile_couleur5", "1F470-1F3FF-200D-2642-FE0F"},
    {"femme_blonde", "1F471-200D-2640-FE0F"},
    {"femme_blonde_couleur1", "1F471-1F3FB-200D-2640-FE0F"},
    {"femme_blonde_couleur2", "1F471-1F3FC-200D-2640-FE0F"},
    {"femme_blonde_couleur3", "1F471-1F3FD-200D-2640-FE0F"},
    {"femme_blonde_couleur4", "1F471-1F3FE-200D-2640-FE0F"},
    {"femme_blonde_couleur5", "1F471-1F3FF-200D-2640-FE0F"},
    {"homme_blond", "1F471-200D-2642-FE0F"},
    {"homme_blond_couleur1", "1F471-1F3FB-200D-2642-FE0F"},
    {"homme_blond_couleur2", "1F471-1F3FC-200D-2642-FE0F"},
    {"homme_blond_couleur3", "1F471-1F3FD-200D-2642-FE0F"},
    {"homme_blond_couleur4", "1F471-1F3FE-200D-2642-FE0F"},
    {"homme_blond_couleur5", "1F471-1F3FF-200D-2642-FE0F"},
    {"femme_en_turban", "1F473-200D-2640-FE0F"},
    {"femme_en_turban_couleur1", "1F473-1F3FB-200D-2640-FE0F"},
    {"femme_en_turban_couleur2", "1F473-1F3FC-200D-2640-FE0F"},
    {"femme_en_turban_couleur3", "1F473-1F3FD-200D-2640-FE0F"},
    {"femme_en_turban_couleur4", "1F473-1F3FE-200D-2640-FE0F"},
    {"femme_en_turban_couleur5", "1F473-1F3FF-200D-2640-FE0F"},
    {"homme_en_turban", "1F473-200D-2642-FE0F"},
    {"homme_en_turban_couleur1", "1F473-1F3FB-200D-2642-FE0F"},
    {"homme_en_turban_couleur2", "1F473-1F3FC-200D-2642-FE0F"},
    {"homme_en_turban_couleur3", "1F473-1F3FD-200D-2642-FE0F"},
    {"homme_en_turban_couleur4", "1F473-1F3FE-200D-2642-FE0F"},
    {"homme_en_turban_couleur5", "1F473-1F3FF-200D-2642-FE0F"},
    {"ouvriere_du_batiment", "1F477-200D-2640-FE0F"},
    {"ouvriere_du_batiment_couleur1", "1F477-1F3FB-200D-2640-FE0F"},
    {"ouvriere_du_batiment_couleur2", "1F477-1F3FC-200D-2640-FE0F"},
    {"ouvriere_du_batiment_couleur3", "1F477-1F3FD-200D-2640-FE0F"},
    {"ouvriere_du_batiment_couleur4", "1F477-1F3FE-200D-2640-FE0F"},
    {"ouvriere_du_batiment_couleur5", "1F477-1F3FF-200D-2640-FE0F"},
    {"ouvrier_du_batiment", "1F477-200D-2642-FE0F"},
    {"ouvrier_du_batiment_couleur1", "1F477-1F3FB-200D-2642-FE0F"},
    {"ouvrier_du_batiment_couleur2", "1F477-1F3FC-200D-2642-FE0F"},
    {"ouvrier_du_batiment_couleur3", "1F477-1F3FD-200D-2642-FE0F"},
    {"ouvrier_du_batiment_couleur4", "1F477-1F3FE-200D-2642-FE0F"},
    {"ouvrier_du_batiment_couleur5", "1F477-1F3FF-200D-2642-FE0F"},
    {"femme_paume_vers_le_haut", "1F481-200D-2640-FE0F"},
    {"femme_paume_vers_le_haut_couleur1", "1F481-1F3FB-200D-2640-FE0F"},
    {"femme_paume_vers_le_haut_couleur2", "1F481-1F3FC-200D-2640-FE0F"},
    {"femme_paume_vers_le_haut_couleur3", "1F481-1F3FD-200D-2640-FE0F"},
    {"femme_paume_vers_le_haut_couleur4", "1F481-1F3FE-200D-2640-FE0F"},
    {"femme_paume_vers_le_haut_couleur5", "1F481-1F3FF-200D-2640-FE0F"},
    {"homme_paume_vers_le_haut", "1F481-200D-2642-FE0F"},
    {"homme_paume_vers_le_haut_couleur1", "1F481-1F3FB-200D-2642-FE0F"},
    {"homme_paume_vers_le_haut_couleur2", "1F481-1F3FC-200D-2642-FE0F"},
    {"homme_paume_vers_le_haut_couleur3", "1F481-1F3FD-200D-2642-FE0F"},
    {"homme_paume_vers_le_haut_couleur4", "1F481-1F3FE-200D-2642-FE0F"},
    {"homme_paume_vers_le_haut_couleur5", "1F481-1F3FF-200D-2642-FE0F"},
    {"garde_femme", "1F482-200D-2640-FE0F"},
    {"garde_femme_couleur1", "1F482-1F3FB-200D-2640-FE0F"},
    {"garde_femme_couleur2", "1F482-1F3FC-200D-2640-FE0F"},
    {"garde_femme_couleur3", "1F482-1F3FD-200D-2640-FE0F"},
    {"garde_femme_couleur4", "1F482-1F3FE-200D-2640-FE0F"},
    {"garde_femme_couleur5", "1F482-1F3FF-200D-2640-FE0F"},
    {"garde_homme", "1F482-200D-2642-FE0F"},
    {"garde_homme_couleur1", "1F482-1F3FB-200D-2642-FE0F"},
    {"garde_homme_couleur2", "1F482-1F3FC-200D-2642-FE0F"},
    {"garde_homme_couleur3", "1F482-1F3FD-200D-2642-FE0F"},
    {"garde_homme_couleur4", "1F482-1F3FE-200D-2642-FE0F"},
    {"garde_homme_couleur5", "1F482-1F3FF-200D-2642-FE0F"},
    {"femme_qui_se_fait_masser", "1F486-200D-2640-FE0F"},
    {"femme_qui_se_fait_masser_couleur1", "1F486-1F3FB-200D-2640-FE0F"},
    {"femme_qui_se_fait_masser_couleur2", "1F486-1F3FC-200D-2640-FE0F"},
    {"femme_qui_se_fait_masser_couleur3", "1F486-1F3FD-200D-2640-FE0F"},
    {"femme_qui_se_fait_masser_couleur4", "1F486-1F3FE-200D-2640-FE0F"},
    {"femme_qui_se_fait_masser_couleur5", "1F486-1F3FF-200D-2640-FE0F"},
    {"homme_qui_se_fait_masser", "1F486-200D-2642-FE0F"},
    {"homme_qui_se_fait_masser_couleur1", "1F486-1F3FB-200D-2642-FE0F"},
    {"homme_qui_se_fait_masser_couleur2", "1F486-1F3FC-200D-2642-FE0F"},
    {"homme_qui_se_fait_masser_couleur3", "1F486-1F3FD-200D-2642-FE0F"},
    {"homme_qui_se_fait_masser_couleur4", "1F486-1F3FE-200D-2642-FE0F"},
    {"homme_qui_se_fait_masser_couleur5", "1F486-1F3FF-200D-2642-FE0F"},
    {"femme_qui_se_fait_couper_les_cheveux", "1F487-200D-2640-FE0F"},
    {"femme_qui_se_fait_couper_les_cheveux_couleur1", "1F487-1F3FB-200D-2640-FE0F"},
    {"femme_qui_se_fait_couper_les_cheveux_couleur2", "1F487-1F3FC-200D-2640-FE0F"},
    {"femme_qui_se_fait_couper_les_cheveux_couleur3", "1F487-1F3FD-200D-2640-FE0F"},
    {"femme_qui_se_fait_couper_les_cheveux_couleur4", "1F487-1F3FE-200D-2640-FE0F"},
    {"femme_qui_se_fait_couper_les_cheveux_couleur5", "1F487-1F3FF-200D-2640-FE0F"},
    {"homme_qui_se_fait_couper_les_cheveux", "1F487-200D-2642-FE0F"},
    {"homme_qui_se_fait_couper_les_cheveux_couleur1", "1F487-1F3FB-200D-2642-FE0F"},
    {"homme_qui_se_fait_couper_les_cheveux_couleur2", "1F487-1F3FC-200D-2642-FE0F"},
    {"homme_qui_se_fait_couper_les_cheveux_couleur3", "1F487-1F3FD-200D-2642-FE0F"},
    {"homme_qui_se_fait_couper_les_cheveux_couleur4", "1F487-1F3FE-200D-2642-FE0F"},
    {"homme_qui_se_fait_couper_les_cheveux_couleur5", "1F487-1F3FF-200D-2642-FE0F"},
    {"detective_femme", "1F575-FE0F-200D-2640-FE0F"},
    {"detective_femme_couleur1", "1F575-1F3FB-200D-2640-FE0F"},
    {"detective_femme_couleur2", "1F575-1F3FC-200D-2640-FE0F"},
    {"detective_femme_couleur3", "1F575-1F3FD-200D-2640-FE0F"},
    {"detective_femme_couleur4", "1F575-1F3FE-200D-2640-FE0F"},
    {"detective_femme_couleur5", "1F575-1F3FF-200D-2640-FE0F"},
    {"detective_homme", "1F575-FE0F-200D-2642-FE0F"},
    {"detective_homme_couleur1", "1F575-1F3FB-200D-2642-FE0F"},
    {"detective_homme_couleur2", "1F575-1F3FC-200D-2642-FE0F"},
    {"detective_homme_couleur3", "1F575-1F3FD-200D-2642-FE0F"},
    {"detective_homme_couleur4", "1F575-1F3FE-200D-2642-FE0F"},
    {"detective_homme_couleur5", "1F575-1F3FF-200D-2642-FE0F"},
    {"femme_faisant_un_geste_dinterdiction", "1F645-200D-2640-FE0F"},
    {"femme_faisant_un_geste_dinterdiction_couleur1", "1F645-1F3FB-200D-2640-FE0F"},
    {"femme_faisant_un_geste_dinterdiction_couleur2", "1F645-1F3FC-200D-2640-FE0F"},
    {"femme_faisant_un_geste_dinterdiction_couleur3", "1F645-1F3FD-200D-2640-FE0F"},
    {"femme_faisant_un_geste_dinterdiction_couleur4", "1F645-1F3FE-200D-2640-FE0F"},
    {"femme_faisant_un_geste_dinterdiction_couleur5", "1F645-1F3FF-200D-2640-FE0F"},
    {"homme_faisant_un_geste_dinterdiction", "1F645-200D-2642-FE0F"},
    {"homme_faisant_un_geste_dinterdiction_couleur1", "1F645-1F3FB-200D-2642-FE0F"},
    {"homme_faisant_un_geste_dinterdiction_couleur2", "1F645-1F3FC-200D-2642-FE0F"},
    {"homme_faisant_un_geste_dinterdiction_couleur3", "1F645-1F3FD-200D-2642-FE0F"},
    {"homme_faisant_un_geste_dinterdiction_couleur4", "1F645-1F3FE-200D-2642-FE0F"},
    {"homme_faisant_un_geste_dinterdiction_couleur5", "1F645-1F3FF-200D-2642-FE0F"},
    {"femme_faisant_un_geste_dacceptation", "1F646-200D-2640-FE0F"},
    {"femme_faisant_un_geste_dacceptation_couleur1", "1F646-1F3FB-200D-2640-FE0F"},
    {"femme_faisant_un_geste_dacceptation_couleur2", "1F646-1F3FC-200D-2640-FE0F"},
    {"femme_faisant_un_geste_dacceptation_couleur3", "1F646-1F3FD-200D-2640-FE0F"},
    {"femme_faisant_un_geste_dacceptation_couleur4", "1F646-1F3FE-200D-2640-FE0F"},
    {"femme_faisant_un_geste_dacceptation_couleur5", "1F646-1F3FF-200D-2640-FE0F"},
    {"homme_faisant_un_geste_dacceptation", "1F646-200D-2642-FE0F"},
    {"homme_faisant_un_geste_dacceptation_couleur1", "1F646-1F3FB-200D-2642-FE0F"},
    {"homme_faisant_un_geste_dacceptation_couleur2", "1F646-1F3FC-200D-2642-FE0F"},
    {"homme_faisant_un_geste_dacceptation_couleur3", "1F646-1F3FD-200D-2642-FE0F"},
    {"homme_faisant_un_geste_dacceptation_couleur4", "1F646-1F3FE-200D-2642-FE0F"},
    {"homme_faisant_un_geste_dacceptation_couleur5", "1F646-1F3FF-200D-2642-FE0F"},
    {"femme_qui_sincline", "1F647-200D-2640-FE0F"},
    {"femme_qui_sincline_couleur1", "1F647-1F3FB-200D-2640-FE0F"},
    {"femme_qui_sincline_couleur2", "1F647-1F3FC-200D-2640-FE0F"},
    {"femme_qui_sincline_couleur3", "1F647-1F3FD-200D-2640-FE0F"},
    {"femme_qui_sincline_couleur4", "1F647-1F3FE-200D-2640-FE0F"},
    {"femme_qui_sincline_couleur5", "1F647-1F3FF-200D-2640-FE0F"},
    {"homme_qui_sincline", "1F647-200D-2642-FE0F"},
    {"homme_qui_sincline_couleur1", "1F647-1F3FB-200D-2642-FE0F"},
    {"homme_qui_sincline_couleur2", "1F647-1F3FC-200D-2642-FE0F"},
    {"homme_qui_sincline_couleur3", "1F647-1F3FD-200D-2642-FE0F"},
    {"homme_qui_sincline_couleur4", "1F647-1F3FE-200D-2642-FE0F"},
    {"homme_qui_sincline_couleur5", "1F647-1F3FF-200D-2642-FE0F"},
    {"femme_qui_leve_la_main", "1F64B-200D-2640-FE0F"},
    {"femme_qui_leve_la_main_couleur1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"femme_qui_leve_la_main_couleur2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"femme_qui_leve_la_main_couleur3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"femme_qui_leve_la_main_couleur4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"femme_qui_leve_la_main_couleur5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"homme_qui_leve_la_main", "1F64B-200D-2642-FE0F"},
    {"homme_qui_leve_la_main_couleur1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"homme_qui_leve_la_main_couleur2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"homme_qui_leve_la_main_couleur3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"homme_qui_leve_la_main_couleur4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"homme_qui_leve_la_main_couleur5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"femme_froncant_les_sourcils", "1F64D-200D-2640-FE0F"},
    {"femme_froncant_les_sourcils_couleur1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"femme_froncant_les_sourcils_couleur2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"femme_froncant_les_sourcils_couleur3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"femme_froncant_les_sourcils_couleur4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"femme_froncant_les_sourcils_couleur5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"homme_froncant_les_sourcils", "1F64D-200D-2642-FE0F"},
    {"homme_froncant_les_sourcils_couleur1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"homme_froncant_les_sourcils_couleur2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"homme_froncant_les_sourcils_couleur3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"homme_froncant_les_sourcils_couleur4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"homme_froncant_les_sourcils_couleur5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"femme_qui_boude", "1F64E-200D-2640-FE0F"},
    {"femme_qui_boude_couleur1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"femme_qui_boude_couleur2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"femme_qui_boude_couleur3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"femme_qui_boude_couleur4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"femme_qui_boude_couleur5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"homme_qui_boude", "1F64E-200D-2642-FE0F"},
    {"homme_qui_boude_couleur1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"homme_qui_boude_couleur2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"homme_qui_boude_couleur3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"homme_qui_boude_couleur4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"homme_qui_boude_couleur5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"rameuse_dans_une_barque", "1F6A3-200D-2640-FE0F"},
    {"rameuse_dans_une_barque_couleur1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"rameuse_dans_une_barque_couleur2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"rameuse_dans_une_barque_couleur3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"rameuse_dans_une_barque_couleur4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"rameuse_dans_une_barque_couleur5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"rameur_dans_une_barque", "1F6A3-200D-2642-FE0F"},
    {"rameur_dans_une_barque_couleur1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"rameur_dans_une_barque_couleur2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"rameur_dans_une_barque_couleur3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"rameur_dans_une_barque_couleur4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"rameur_dans_une_barque_couleur5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"cycliste_femme", "1F6B4-200D-2640-FE0F"},
    {"cycliste_femme_couleur1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"cycliste_femme_couleur2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"cycliste_femme_couleur3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"cycliste_femme_couleur4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"cycliste_femme_couleur5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"cycliste_homme", "1F6B4-200D-2642-FE0F"},
    {"cycliste_homme_couleur1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"cycliste_homme_couleur2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"cycliste_homme_couleur3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"cycliste_homme_couleur4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"cycliste_homme_couleur5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"femme_en_vtt", "1F6B5-200D-2640-FE0F"},
    {"femme_en_vtt_couleur1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"femme_en_vtt_couleur2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"femme_en_vtt_couleur3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"femme_en_vtt_couleur4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"femme_en_vtt_couleur5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"homme_en_vtt", "1F6B5-200D-2642-FE0F"},
    {"homme_en_vtt_couleur1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"homme_en_vtt_couleur2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"homme_en_vtt_couleur3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"homme_en_vtt_couleur4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"homme_en_vtt_couleur5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"femme_qui_marche", "1F6B6-200D-2640-FE0F"},
    {"femme_qui_marche_couleur1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"femme_qui_marche_couleur2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"femme_qui_marche_couleur3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"femme_qui_marche_couleur4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"femme_qui_marche_couleur5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"femme_qui_marche_vers_la_droite", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_qui_marche_vers_la_droite_couleur1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_qui_marche_vers_la_droite_couleur2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_qui_marche_vers_la_droite_couleur3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_qui_marche_vers_la_droite_couleur4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_qui_marche_vers_la_droite_couleur5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"homme_qui_marche", "1F6B6-200D-2642-FE0F"},
    {"homme_qui_marche_couleur1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"homme_qui_marche_couleur2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"homme_qui_marche_couleur3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"homme_qui_marche_couleur4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"homme_qui_marche_couleur5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"homme_qui_marche_vers_la_droite", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_qui_marche_vers_la_droite_couleur1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_qui_marche_vers_la_droite_couleur2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_qui_marche_vers_la_droite_couleur3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_qui_marche_vers_la_droite_couleur4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_qui_marche_vers_la_droite_couleur5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"femme_avec_la_paume_sur_le_visage", "1F926-200D-2640-FE0F"},
    {"femme_avec_la_paume_sur_le_visage_couleur1", "1F926-1F3FB-200D-2640-FE0F"},
    {"femme_avec_la_paume_sur_le_visage_couleur2", "1F926-1F3FC-200D-2640-FE0F"},
    {"femme_avec_la_paume_sur_le_visage_couleur3", "1F926-1F3FD-200D-2640-FE0F"},
    {"femme_avec_la_paume_sur_le_visage_couleur4", "1F926-1F3FE-200D-2640-FE0F"},
    {"femme_avec_la_paume_sur_le_visage_couleur5", "1F926-1F3FF-200D-2640-FE0F"},
    {"homme_avec_la_paume_sur_le_visage", "1F926-200D-2642-FE0F"},
    {"homme_avec_la_paume_sur_le_visage_couleur1", "1F926-1F3FB-200D-2642-FE0F"},
    {"homme_avec_la_paume_sur_le_visage_couleur2", "1F926-1F3FC-200D-2642-FE0F"},
    {"homme_avec_la_paume_sur_le_visage_couleur3", "1F926-1F3FD-200D-2642-FE0F"},
    {"homme_avec_la_paume_sur_le_visage_couleur4", "1F926-1F3FE-200D-2642-FE0F"},
    {"homme_avec_la_paume_sur_le_visage_couleur5", "1F926-1F3FF-200D-2642-FE0F"},
    {"femme_en_smoking", "1F935-200D-2640-FE0F"},
    {"femme_en_smoking_couleur1", "1F935-1F3FB-200D-2640-FE0F"},
    {"femme_en_smoking_couleur2", "1F935-1F3FC-200D-2640-FE0F"},
    {"femme_en_smoking_couleur3", "1F935-1F3FD-200D-2640-FE0F"},
    {"femme_en_smoking_couleur4", "1F935-1F3FE-200D-2640-FE0F"},
    {"femme_en_smoking_couleur5", "1F935-1F3FF-200D-2640-FE0F"},
    {"homme_en_smoking", "1F935-200D-2642-FE0F"},
    {"homme_en_smoking_couleur1", "1F935-1F3FB-200D-2642-FE0F"},
    {"homme_en_smoking_couleur2", "1F935-1F3FC-200D-2642-FE0F"},
    {"homme_en_smoking_couleur3", "1F935-1F3FD-200D-2642-FE0F"},
    {"homme_en_smoking_couleur4", "1F935-1F3FE-200D-2642-FE0F"},
    {"homme_en_smoking_couleur5", "1F935-1F3FF-200D-2642-FE0F"},
    {"femme_qui_hausse_les_epaules", "1F937-200D-2640-FE0F"},
    {"femme_qui_hausse_les_epaules_couleur1", "1F937-1F3FB-200D-2640-FE0F"},
    {"femme_qui_hausse_les_epaules_couleur2", "1F937-1F3FC-200D-2640-FE0F"},
    {"femme_qui_hausse_les_epaules_couleur3", "1F937-1F3FD-200D-2640-FE0F"},
    {"femme_qui_hausse_les_epaules_couleur4", "1F937-1F3FE-200D-2640-FE0F"},
    {"femme_qui_hausse_les_epaules_couleur5", "1F937-1F3FF-200D-2640-FE0F"},
    {"homme_qui_hausse_les_epaules", "1F937-200D-2642-FE0F"},
    {"homme_qui_hausse_les_epaules_couleur1", "1F937-1F3FB-200D-2642-FE0F"},
    {"homme_qui_hausse_les_epaules_couleur2", "1F937-1F3FC-200D-2642-FE0F"},
    {"homme_qui_hausse_les_epaules_couleur3", "1F937-1F3FD-200D-2642-FE0F"},
    {"homme_qui_hausse_les_epaules_couleur4", "1F937-1F3FE-200D-2642-FE0F"},
    {"homme_qui_hausse_les_epaules_couleur5", "1F937-1F3FF-200D-2642-FE0F"},
    {"femme_faisant_la_roue", "1F938-200D-2640-FE0F"},
    {"femme_faisant_la_roue_couleur1", "1F938-1F3FB-200D-2640-FE0F"},
    {"femme_faisant_la_roue_couleur2", "1F938-1F3FC-200D-2640-FE0F"},
    {"femme_faisant_la_roue_couleur3", "1F938-1F3FD-200D-2640-FE0F"},
    {"femme_faisant_la_roue_couleur4", "1F938-1F3FE-200D-2640-FE0F"},
    {"femme_faisant_la_roue_couleur5", "1F938-1F3FF-200D-2640-FE0F"},
    {"homme_faisant_la_roue", "1F938-200D-2642-FE0F"},
    {"homme_faisant_la_roue_couleur1", "1F938-1F3FB-200D-2642-FE0F"},
    {"homme_faisant_la_roue_couleur2", "1F938-1F3FC-200D-2642-FE0F"},
    {"homme_faisant_la_roue_couleur3", "1F938-1F3FD-200D-2642-FE0F"},
    {"homme_faisant_la_roue_couleur4", "1F938-1F3FE-200D-2642-FE0F"},
    {"homme_faisant_la_roue_couleur5", "1F938-1F3FF-200D-2642-FE0F"},
    {"jongleuse", "1F939-200D-2640-FE0F"},
    {"jongleuse_couleur1", "1F939-1F3FB-200D-2640-FE0F"},
    {"jongleuse_couleur2", "1F939-1F3FC-200D-2640-FE0F"},
    {"jongleuse_couleur3", "1F939-1F3FD-200D-2640-FE0F"},
    {"jongleuse_couleur4", "1F939-1F3FE-200D-2640-FE0F"},
    {"jongleuse_couleur5", "1F939-1F3FF-200D-2640-FE0F"},
    {"jongleur", "1F939-200D-2642-FE0F"},
    {"jongleur_couleur1", "1F939-1F3FB-200D-2642-FE0F"},
    {"jongleur_couleur2", "1F939-1F3FC-200D-2642-FE0F"},
    {"jongleur_couleur3", "1F939-1F3FD-200D-2642-FE0F"},
    {"jongleur_couleur4", "1F939-1F3FE-200D-2642-FE0F"},
    {"jongleur_couleur5", "1F939-1F3FF-200D-2642-FE0F"},
    {"lutteuses", "1F93C-200D-2640-FE0F"},
    {"lutteuses_couleur1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"lutteuses_couleur1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"lutteuses_couleur1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"lutteuses_couleur1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"lutteuses_couleur2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"lutteuses_couleur2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"lutteuses_couleur2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"lutteuses_couleur2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"lutteuses_couleur3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"lutteuses_couleur3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"lutteuses_couleur3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"lutteuses_couleur3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"lutteuses_couleur4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"lutteuses_couleur4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"lutteuses_couleur4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"lutteuses_couleur4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"lutteuses_couleur5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"lutteuses_couleur5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"lutteuses_couleur5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"lutteuses_couleur5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"lutteuses_couleur1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"lutteuses_couleur2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"lutteuses_couleur3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"lutteuses_couleur4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"lutteuses_couleur5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"lutteurs", "1F93C-200D-2642-FE0F"},
    {"lutteurs_couleur1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"lutteurs_couleur1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"lutteurs_couleur1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"lutteurs_couleur1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"lutteurs_couleur2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"lutteurs_couleur2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"lutteurs_couleur2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"lutteurs_couleur2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"lutteurs_couleur3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"lutteurs_couleur3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"lutteurs_couleur3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"lutteurs_couleur3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"lutteurs_couleur4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"lutteurs_couleur4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"lutteurs_couleur4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"lutteurs_couleur4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"lutteurs_couleur5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"lutteurs_couleur5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"lutteurs_couleur5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"lutteurs_couleur5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"lutteurs_couleur1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"lutteurs_couleur2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"lutteurs_couleur3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"lutteurs_couleur4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"lutteurs_couleur5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"joueuse_de_water_polo", "1F93D-200D-2640-FE0F"},
    {"joueuse_de_water_polo_couleur1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"joueuse_de_water_polo_couleur2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"joueuse_de_water_polo_couleur3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"joueuse_de_water_polo_couleur4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"joueuse_de_water_polo_couleur5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"joueur_de_water_polo", "1F93D-200D-2642-FE0F"},
    {"joueur_de_water_polo_couleur1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"joueur_de_water_polo_couleur2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"joueur_de_water_polo_couleur3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"joueur_de_water_polo_couleur4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"joueur_de_water_polo_couleur5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"handballeuse", "1F93E-200D-2640-FE0F"},
    {"handballeuse_couleur1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"handballeuse_couleur2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"handballeuse_couleur3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"handballeuse_couleur4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"handballeuse_couleur5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"handballeur", "1F93E-200D-2642-FE0F"},
    {"handballeur_couleur1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"handballeur_couleur2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"handballeur_couleur3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"handballeur_couleur4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"handballeur_couleur5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"super_heroine", "1F9B8-200D-2640-FE0F"},
    {"super_heroine_couleur1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"super_heroine_couleur2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"super_heroine_couleur3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"super_heroine_couleur4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"super_heroine_couleur5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"super_heros_homme", "1F9B8-200D-2642-FE0F"},
    {"super_heros_homme_couleur1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"super_heros_homme_couleur2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"super_heros_homme_couleur3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"super_heros_homme_couleur4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"super_heros_homme_couleur5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"super_vilain_femme", "1F9B9-200D-2640-FE0F"},
    {"super_vilain_femme_couleur1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"super_vilain_femme_couleur2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"super_vilain_femme_couleur3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"super_vilain_femme_couleur4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"super_vilain_femme_couleur5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"super_vilain_homme", "1F9B9-200D-2642-FE0F"},
    {"super_vilain_homme_couleur1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"super_vilain_homme_couleur2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"super_vilain_homme_couleur3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"super_vilain_homme_couleur4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"super_vilain_homme_couleur5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"femme_debout", "1F9CD-200D-2640-FE0F"},
    {"femme_debout_couleur1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"femme_debout_couleur2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"femme_debout_couleur3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"femme_debout_couleur4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"femme_debout_couleur5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"homme_debout", "1F9CD-200D-2642-FE0F"},
    {"homme_debout_couleur1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"homme_debout_couleur2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"homme_debout_couleur3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"homme_debout_couleur4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"homme_debout_couleur5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"femme_a_genoux", "1F9CE-200D-2640-FE0F"},
    {"femme_a_genoux_couleur1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"femme_a_genoux_couleur2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"femme_a_genoux_couleur3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"femme_a_genoux_couleur4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"femme_a_genoux_couleur5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"femme_a_genoux_vers_la_droite", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_a_genoux_vers_la_droite_couleur1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_a_genoux_vers_la_droite_couleur2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_a_genoux_vers_la_droite_couleur3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_a_genoux_vers_la_droite_couleur4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_a_genoux_vers_la_droite_couleur5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"homme_a_genoux", "1F9CE-200D-2642-FE0F"},
    {"homme_a_genoux_couleur1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"homme_a_genoux_couleur2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"homme_a_genoux_couleur3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"homme_a_genoux_couleur4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"homme_a_genoux_couleur5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"homme_a_genoux_vers_la_droite", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_a_genoux_vers_la_droite_couleur1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_a_genoux_vers_la_droite_couleur2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_a_genoux_vers_la_droite_couleur3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_a_genoux_vers_la_droite_couleur4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_a_genoux_vers_la_droite_couleur5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"femme_sourde", "1F9CF-200D-2640-FE0F"},
    {"femme_sourde_couleur1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"femme_sourde_couleur2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"femme_sourde_couleur3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"femme_sourde_couleur4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"femme_sourde_couleur5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"homme_sourd", "1F9CF-200D-2642-FE0F"},
    {"homme_sourd_couleur1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"homme_sourd_couleur2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"homme_sourd_couleur3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"homme_sourd_couleur4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"homme_sourd_couleur5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"femme_barbue", "1F9D4-200D-2640-FE0F"},
    {"femme_barbue_couleur1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"femme_barbue_couleur2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"femme_barbue_couleur3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"femme_barbue_couleur4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"femme_barbue_couleur5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"homme_barbu", "1F9D4-200D-2642-FE0F"},
    {"homme_barbu_couleur1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"homme_barbu_couleur2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"homme_barbu_couleur3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"homme_barbu_couleur4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"homme_barbu_couleur5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"femme_au_hammam", "1F9D6-200D-2640-FE0F"},
    {"femme_au_hammam_couleur1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"femme_au_hammam_couleur2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"femme_au_hammam_couleur3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"femme_au_hammam_couleur4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"femme_au_hammam_couleur5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"homme_au_hammam", "1F9D6-200D-2642-FE0F"},
    {"homme_au_hammam_couleur1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"homme_au_hammam_couleur2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"homme_au_hammam_couleur3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"homme_au_hammam_couleur4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"homme_au_hammam_couleur5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"femme_qui_grimpe", "1F9D7-200D-2640-FE0F"},
    {"femme_qui_grimpe_couleur1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"femme_qui_grimpe_couleur2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"femme_qui_grimpe_couleur3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"femme_qui_grimpe_couleur4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"femme_qui_grimpe_couleur5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"homme_qui_grimpe", "1F9D7-200D-2642-FE0F"},
    {"homme_qui_grimpe_couleur1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"homme_qui_grimpe_couleur2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"homme_qui_grimpe_couleur3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"homme_qui_grimpe_couleur4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"homme_qui_grimpe_couleur5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"femme_dans_la_posture_du_lotus", "1F9D8-200D-2640-FE0F"},
    {"femme_dans_la_posture_du_lotus_couleur1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"femme_dans_la_posture_du_lotus_couleur2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"femme_dans_la_posture_du_lotus_couleur3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"femme_dans_la_posture_du_lotus_couleur4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"femme_dans_la_posture_du_lotus_couleur5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"homme_dans_la_posture_du_lotus", "1F9D8-200D-2642-FE0F"},
    {"homme_dans_la_posture_du_lotus_couleur1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"homme_dans_la_posture_du_lotus_couleur2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"homme_dans_la_posture_du_lotus_couleur3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"homme_dans_la_posture_du_lotus_couleur4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"homme_dans_la_posture_du_lotus_couleur5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"mage_femme", "1F9D9-200D-2640-FE0F"},
    {"mage_femme_couleur1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"mage_femme_couleur2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"mage_femme_couleur3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"mage_femme_couleur4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"mage_femme_couleur5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"mage_homme", "1F9D9-200D-2642-FE0F"},
    {"mage_homme_couleur1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"mage_homme_couleur2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"mage_homme_couleur3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"mage_homme_couleur4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"mage_homme_couleur5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"fee", "1F9DA-200D-2640-FE0F"},
    {"fee_couleur1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"fee_couleur2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"fee_couleur3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"fee_couleur4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"fee_couleur5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"feetaud", "1F9DA-200D-2642-FE0F"},
    {"feetaud_couleur1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"feetaud_couleur2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"feetaud_couleur3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"feetaud_couleur4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"feetaud_couleur5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"vampire_femme", "1F9DB-200D-2640-FE0F"},
    {"vampire_femme_couleur1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"vampire_femme_couleur2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"vampire_femme_couleur3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"vampire_femme_couleur4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"vampire_femme_couleur5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"vampire_homme", "1F9DB-200D-2642-FE0F"},
    {"vampire_homme_couleur1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"vampire_homme_couleur2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"vampire_homme_couleur3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"vampire_homme_couleur4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"vampire_homme_couleur5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"sirene", "1F9DC-200D-2640-FE0F"},
    {"sirene_couleur1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"sirene_couleur2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"sirene_couleur3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"sirene_couleur4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"sirene_couleur5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"triton", "1F9DC-200D-2642-FE0F"},
    {"triton_couleur1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"triton_couleur2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"triton_couleur3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"triton_couleur4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"triton_couleur5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"elfe_femme", "1F9DD-200D-2640-FE0F"},
    {"elfe_femme_couleur1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"elfe_femme_couleur2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"elfe_femme_couleur3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"elfe_femme_couleur4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"elfe_femme_couleur5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"elfe_homme", "1F9DD-200D-2642-FE0F"},
    {"elfe_homme_couleur1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"elfe_homme_couleur2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"elfe_homme_couleur3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"elfe_homme_couleur4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"elfe_homme_couleur5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"genie_femme", "1F9DE-200D-2640-FE0F"},
    {"genie_homme", "1F9DE-200D-2642-FE0F"},
    {"zombie_femme", "1F9DF-200D-2640-FE0F"},
    {"zombie_homme", "1F9DF-200D-2642-FE0F"},
    {"homme_cheveux_roux", "1F468-200D-1F9B0"},
    {"homme_cheveux_roux_couleur1", "1F468-1F3FB-200D-1F9B0"},
    {"homme_cheveux_roux_couleur2", "1F468-1F3FC-200D-1F9B0"},
    {"homme_cheveux_roux_couleur3", "1F468-1F3FD-200D-1F9B0"},
    {"homme_cheveux_roux_couleur4", "1F468-1F3FE-200D-1F9B0"},
    {"homme_cheveux_roux_couleur5", "1F468-1F3FF-200D-1F9B0"},
    {"homme_cheveux_boucles", "1F468-200D-1F9B1"},
    {"homme_cheveux_boucles_couleur1", "1F468-1F3FB-200D-1F9B1"},
    {"homme_cheveux_boucles_couleur2", "1F468-1F3FC-200D-1F9B1"},
    {"homme_cheveux_boucles_couleur3", "1F468-1F3FD-200D-1F9B1"},
    {"homme_cheveux_boucles_couleur4", "1F468-1F3FE-200D-1F9B1"},
    {"homme_cheveux_boucles_couleur5", "1F468-1F3FF-200D-1F9B1"},
    {"homme_chauve", "1F468-200D-1F9B2"},
    {"homme_chauve_couleur1", "1F468-1F3FB-200D-1F9B2"},
    {"homme_chauve_couleur2", "1F468-1F3FC-200D-1F9B2"},
    {"homme_chauve_couleur3", "1F468-1F3FD-200D-1F9B2"},
    {"homme_chauve_couleur4", "1F468-1F3FE-200D-1F9B2"},
    {"homme_chauve_couleur5", "1F468-1F3FF-200D-1F9B2"},
    {"homme_cheveux_blancs", "1F468-200D-1F9B3"},
    {"homme_cheveux_blancs_couleur1", "1F468-1F3FB-200D-1F9B3"},
    {"homme_cheveux_blancs_couleur2", "1F468-1F3FC-200D-1F9B3"},
    {"homme_cheveux_blancs_couleur3", "1F468-1F3FD-200D-1F9B3"},
    {"homme_cheveux_blancs_couleur4", "1F468-1F3FE-200D-1F9B3"},
    {"homme_cheveux_blancs_couleur5", "1F468-1F3FF-200D-1F9B3"},
    {"femme_cheveux_roux", "1F469-200D-1F9B0"},
    {"femme_cheveux_roux_couleur1", "1F469-1F3FB-200D-1F9B0"},
    {"femme_cheveux_roux_couleur2", "1F469-1F3FC-200D-1F9B0"},
    {"femme_cheveux_roux_couleur3", "1F469-1F3FD-200D-1F9B0"},
    {"femme_cheveux_roux_couleur4", "1F469-1F3FE-200D-1F9B0"},
    {"femme_cheveux_roux_couleur5", "1F469-1F3FF-200D-1F9B0"},
    {"femme_cheveux_boucles", "1F469-200D-1F9B1"},
    {"femme_cheveux_boucles_couleur1", "1F469-1F3FB-200D-1F9B1"},
    {"femme_cheveux_boucles_couleur2", "1F469-1F3FC-200D-1F9B1"},
    {"femme_cheveux_boucles_couleur3", "1F469-1F3FD-200D-1F9B1"},
    {"femme_cheveux_boucles_couleur4", "1F469-1F3FE-200D-1F9B1"},
    {"femme_cheveux_boucles_couleur5", "1F469-1F3FF-200D-1F9B1"},
    {"femme_chauve", "1F469-200D-1F9B2"},
    {"femme_chauve_couleur1", "1F469-1F3FB-200D-1F9B2"},
    {"femme_chauve_couleur2", "1F469-1F3FC-200D-1F9B2"},
    {"femme_chauve_couleur3", "1F469-1F3FD-200D-1F9B2"},
    {"femme_chauve_couleur4", "1F469-1F3FE-200D-1F9B2"},
    {"femme_chauve_couleur5", "1F469-1F3FF-200D-1F9B2"},
    {"femme_cheveux_blancs", "1F469-200D-1F9B3"},
    {"femme_cheveux_blancs_couleur1", "1F469-1F3FB-200D-1F9B3"},
    {"femme_cheveux_blancs_couleur2", "1F469-1F3FC-200D-1F9B3"},
    {"femme_cheveux_blancs_couleur3", "1F469-1F3FD-200D-1F9B3"},
    {"femme_cheveux_blancs_couleur4", "1F469-1F3FE-200D-1F9B3"},
    {"femme_cheveux_blancs_couleur5", "1F469-1F3FF-200D-1F9B3"},
    {"adulte_cheveux_roux", "1F9D1-200D-1F9B0"},
    {"adulte_cheveux_roux_couleur1", "1F9D1-1F3FB-200D-1F9B0"},
    {"adulte_cheveux_roux_couleur2", "1F9D1-1F3FC-200D-1F9B0"},
    {"adulte_cheveux_roux_couleur3", "1F9D1-1F3FD-200D-1F9B0"},
    {"adulte_cheveux_roux_couleur4", "1F9D1-1F3FE-200D-1F9B0"},
    {"adulte_cheveux_roux_couleur5", "1F9D1-1F3FF-200D-1F9B0"},
    {"adulte_cheveux_boucles", "1F9D1-200D-1F9B1"},
    {"adulte_cheveux_boucles_couleur1", "1F9D1-1F3FB-200D-1F9B1"},
    {"adulte_cheveux_boucles_couleur2", "1F9D1-1F3FC-200D-1F9B1"},
    {"adulte_cheveux_boucles_couleur3", "1F9D1-1F3FD-200D-1F9B1"},
    {"adulte_cheveux_boucles_couleur4", "1F9D1-1F3FE-200D-1F9B1"},
    {"adulte_cheveux_boucles_couleur5", "1F9D1-1F3FF-200D-1F9B1"},
    {"adulte_chauve", "1F9D1-200D-1F9B2"},
    {"adulte_chauve_couleur1", "1F9D1-1F3FB-200D-1F9B2"},
    {"adulte_chauve_couleur2", "1F9D1-1F3FC-200D-1F9B2"},
    {"adulte_chauve_couleur3", "1F9D1-1F3FD-200D-1F9B2"},
    {"adulte_chauve_couleur4", "1F9D1-1F3FE-200D-1F9B2"},
    {"adulte_chauve_couleur5", "1F9D1-1F3FF-200D-1F9B2"},
    {"adulte_cheveux_blancs", "1F9D1-200D-1F9B3"},
    {"adulte_cheveux_blancs_couleur1", "1F9D1-1F3FB-200D-1F9B3"},
    {"adulte_cheveux_blancs_couleur2", "1F9D1-1F3FC-200D-1F9B3"},
    {"adulte_cheveux_blancs_couleur3", "1F9D1-1F3FD-200D-1F9B3"},
    {"adulte_cheveux_blancs_couleur4", "1F9D1-1F3FE-200D-1F9B3"},
    {"adulte_cheveux_blancs_couleur5", "1F9D1-1F3FF-200D-1F9B3"},
    {"chaine_brisee", "26D3-FE0F-200D-1F4A5"},
    {"coeur_enflamme", "2764-FE0F-200D-1F525"},
    {"coeur_avec_bandages", "2764-FE0F-200D-1FA79"},
    {"champignon_marron", "1F344-200D-1F7EB"},
    {"citron_vert", "1F34B-200D-1F7E9"},
    {"drapeau_transgenre", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"drapeau_arc_en_ciel", "1F3F3-FE0F-200D-1F308"},
    {"drapeau_de_pirate", "1F3F4-200D-2620-FE0F"},
    {"chat_noir", "1F408-200D-2B1B"},
    {"chien_dassistance", "1F415-200D-1F9BA"},
    {"oiseau_noir", "1F426-200D-2B1B"},
    {"phenix", "1F426-200D-1F525"},
    {"ours_polaire", "1F43B-200D-2744-FE0F"},
    {"oeil_dans_une_bulle_de_bd", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"visage_expirant", "1F62E-200D-1F4A8"},
    {"visage_aux_yeux_en_spirales", "1F635-200D-1F4AB"},
    {"visage_dans_les_nuages", "1F636-200D-1F32B-FE0F"},
    {"tete_secouee_horizontalement", "1F642-200D-2194-FE0F"},
    {"tete_secouee_verticalement", "1F642-200D-2195-FE0F"},
    {"danseuse_de_ballet", "1F9D1-200D-1FA70"},
    {"danseuse_de_ballet_couleur1", "1F9D1-1F3FB-200D-1FA70"},
    {"danseuse_de_ballet_couleur2", "1F9D1-1F3FC-200D-1FA70"},
    {"danseuse_de_ballet_couleur3", "1F9D1-1F3FD-200D-1FA70"},
    {"danseuse_de_ballet_couleur4", "1F9D1-1F3FE-200D-1FA70"},
    {"danseuse_de_ballet_couleur5", "1F9D1-1F3FF-200D-1FA70"},
    {"marque_commerciale_déposée", "2122"},
    {"flèche_gauche_droite", "2194"},
    {"flèche_haut_bas", "2195"},
    {"flèche_haut_gauche", "2196"},
    {"flèche_haut_droite", "2197"},
    {"flèche_bas_droite", "2198"},
    {"flèche_bas_gauche", "2199"},
    {"comète", "2604"},
    {"case_cochée", "2611"},
    {"trèfle", "2618"},
    {"tête_de_mort", "2620"},
    {"visage_mécontent", "2639"},
    {"bélier_zodiaque", "2648"},
    {"trèfle_cartes", "2663"},
    {"cœur_cartes", "2665"},
    {"épées_croisées", "2694"},
    {"caducée", "2695"},
    {"balance_à_poids", "2696"},
    {"roue_dentée", "2699"},
    {"bouton_coché", "2705"},
    {"étoile_de_david", "2721"},
    {"étincelles", "2728"},
    {"astérisque_huit_branches", "2733"},
    {"étoile_huit_branches", "2734"},
    {"éclat", "2747"},
    {"cœur_point_dexclamation", "2763"},
    {"cœur_rouge", "2764"},
    {"flèche_courbe_haut", "2934"},
    {"flèche_courbe_bas", "2935"},
    {"ligne_ondulée", "3030"},
    {"bouton_félicitations_en_japonais", "3297"},
    {"marque_déposée", "00AE"},
    {"flèche_courbe_gauche", "21A9"},
    {"flèche_courbe_droite", "21AA"},
    {"bouton_éjecter", "23CF"},
    {"double_flèche_vers_le_haut", "23EB"},
    {"double_flèche_vers_le_bas", "23EC"},
    {"bouton_piste_précédente", "23EE"},
    {"réveil", "23F0"},
    {"chronomètre", "23F1"},
    {"m_encerclé", "24C2"},
    {"petit_carré_noir", "25AA"},
    {"petit_carré_blanc", "25AB"},
    {"carré_moyen_blanc", "25FB"},
    {"carré_moyen_noir", "25FC"},
    {"carré_petit_moyen_blanc", "25FD"},
    {"carré_petit_moyen_noir", "25FE"},
    {"téléphone", "260E"},
    {"lune_et_étoile", "262A"},
    {"gémeaux", "264A"},
    {"pion_déchec", "265F"},
    {"symbole_accès_handicapés", "267F"},
    {"symbole_de_la_communauté_transgenre", "26A7"},
    {"urne_funéraire", "26B1"},
    {"soleil_derrière_les_nuages", "26C5"},
    {"nuage_avec_éclair_et_pluie", "26C8"},
    {"chaînes", "26D3"},
    {"église", "26EA"},
    {"patin_à_glace", "26F8"},
    {"pompe_à_essence", "26FD"},
    {"poing_levé", "270A"},
    {"poing_levé_couleur1", "270A-1F3FB"},
    {"poing_levé_couleur2", "270A-1F3FC"},
    {"poing_levé_couleur3", "270A-1F3FD"},
    {"poing_levé_couleur4", "270A-1F3FE"},
    {"poing_levé_couleur5", "270A-1F3FF"},
    {"main_levée", "270B"},
    {"main_levée_couleur1", "270B-1F3FB"},
    {"main_levée_couleur2", "270B-1F3FC"},
    {"main_levée_couleur3", "270B-1F3FD"},
    {"main_levée_couleur4", "270B-1F3FE"},
    {"main_levée_couleur5", "270B-1F3FF"},
    {"main_qui_écrit", "270D"},
    {"main_qui_écrit_couleur1", "270D-1F3FB"},
    {"main_qui_écrit_couleur2", "270D-1F3FC"},
    {"main_qui_écrit_couleur3", "270D-1F3FD"},
    {"main_qui_écrit_couleur4", "270D-1F3FE"},
    {"main_qui_écrit_couleur5", "270D-1F3FF"},
    {"flèche_droite", "27A1"},
    {"flèche_gauche", "2B05"},
    {"flèche_haut", "2B06"},
    {"flèche_bas", "2B07"},
    {"grand_carré_noir", "2B1B"},
    {"grand_carré_blanc", "2B1C"},
    {"étoile", "2B50"},
    {"indicateur_régional_a", "1F1E6"},
    {"indicateur_régional_b", "1F1E7"},
    {"indicateur_régional_c", "1F1E8"},
    {"indicateur_régional_d", "1F1E9"},
    {"indicateur_régional_e", "1F1EA"},
    {"indicateur_régional_f", "1F1EB"},
    {"indicateur_régional_g", "1F1EC"},
    {"indicateur_régional_h", "1F1ED"},
    {"indicateur_régional_i", "1F1EE"},
    {"indicateur_régional_j", "1F1EF"},
    {"indicateur_régional_k", "1F1F0"},
    {"indicateur_régional_l", "1F1F1"},
    {"indicateur_régional_m", "1F1F2"},
    {"indicateur_régional_n", "1F1F3"},
    {"indicateur_régional_o", "1F1F4"},
    {"indicateur_régional_p", "1F1F5"},
    {"indicateur_régional_q", "1F1F6"},
    {"indicateur_régional_r", "1F1F7"},
    {"indicateur_régional_s", "1F1F8"},
    {"indicateur_régional_t", "1F1F9"},
    {"indicateur_régional_u", "1F1FA"},
    {"indicateur_régional_v", "1F1FB"},
    {"indicateur_régional_w", "1F1FC"},
    {"indicateur_régional_x", "1F1FD"},
    {"indicateur_régional_y", "1F1FE"},
    {"indicateur_régional_z", "1F1FF"},
    {"bouton_réservé_en_japonais", "1F22F"},
    {"bouton_note_pour_réussir_en_japonais", "1F234"},
    {"bouton_réduction_en_japonais", "1F239"},
    {"parapluie_fermé", "1F302"},
    {"nuit_étoilée", "1F303"},
    {"soleil_levant_derrière_les_montagnes", "1F304"},
    {"ville_au_crépuscule", "1F306"},
    {"voie_lactée", "1F30C"},
    {"globe_tourné_sur_lafrique_et_leurope", "1F30D"},
    {"globe_tourné_sur_les_amériques", "1F30E"},
    {"globe_tourné_sur_lasie_et_laustralie", "1F30F"},
    {"globe_avec_méridiens", "1F310"},
    {"lune_gibbeuse_décroissante", "1F316"},
    {"lune_décroissante", "1F318"},
    {"étoile_brillante", "1F31F"},
    {"étoile_filante", "1F320"},
    {"thermomètre", "1F321"},
    {"soleil_derrière_un_petit_nuage", "1F324"},
    {"soleil_derrière_un_gros_nuage", "1F325"},
    {"soleil_derrière_un_nuage_de_pluie", "1F326"},
    {"nuage_avec_éclair", "1F329"},
    {"châtaigne", "1F330"},
    {"conifère", "1F332"},
    {"arbre_à_feuilles_caduques", "1F333"},
    {"épi_de_maïs", "1F33D"},
    {"trèfle_à_quatre_feuilles", "1F340"},
    {"feuille_dérable", "1F341"},
    {"pastèque", "1F349"},
    {"pêche", "1F351"},
    {"granité", "1F367"},
    {"barre_chocolatée", "1F36B"},
    {"crème_renversée", "1F36E"},
    {"gâteau_sablé", "1F370"},
    {"boîte_déjeuner", "1F371"},
    {"œuf_au_plat", "1F373"},
    {"saké", "1F376"},
    {"chopes_de_bière", "1F37B"},
    {"gâteau_danniversaire", "1F382"},
    {"sapin_de_noël", "1F384"},
    {"père_noël", "1F385"},
    {"père_noël_couleur1", "1F385-1F3FB"},
    {"père_noël_couleur2", "1F385-1F3FC"},
    {"père_noël_couleur3", "1F385-1F3FD"},
    {"père_noël_couleur4", "1F385-1F3FE"},
    {"père_noël_couleur5", "1F385-1F3FF"},
    {"arbre_à_vœux", "1F38B"},
    {"drapeaux_croisés", "1F38C"},
    {"bambou_décoratif", "1F38D"},
    {"poupées_japonaises", "1F38E"},
    {"carillon_éolien", "1F390"},
    {"cérémonie_de_la_lune", "1F391"},
    {"médaille_militaire", "1F396"},
    {"ruban_de_mémoire", "1F397"},
    {"boutons_de_réglage", "1F39B"},
    {"billet_dentrée", "1F39F"},
    {"cheval_de_manège", "1F3A0"},
    {"pêche_à_la_ligne", "1F3A3"},
    {"caméra", "1F3A5"},
    {"cinéma", "1F3A6"},
    {"jeu_vidéo", "1F3AE"},
    {"machine_à_sous", "1F3B0"},
    {"dés", "1F3B2"},
    {"drapeau_à_damier", "1F3C1"},
    {"médaille_sportive", "1F3C5"},
    {"trophée", "1F3C6"},
    {"football_américain", "1F3C8"},
    {"haltérophile", "1F3CB"},
    {"haltérophile_couleur1", "1F3CB-1F3FB"},
    {"haltérophile_couleur2", "1F3CB-1F3FC"},
    {"haltérophile_couleur3", "1F3CB-1F3FD"},
    {"haltérophile_couleur4", "1F3CB-1F3FE"},
    {"haltérophile_couleur5", "1F3CB-1F3FF"},
    {"montagne_enneigée", "1F3D4"},
    {"construction_dun_bâtiment", "1F3D7"},
    {"maison_abandonnée", "1F3DA"},
    {"désert", "1F3DC"},
    {"île_déserte", "1F3DD"},
    {"hôpital", "1F3E5"},
    {"hôtel", "1F3E8"},
    {"supérette", "1F3EA"},
    {"école", "1F3EB"},
    {"château_japonais", "1F3EF"},
    {"château", "1F3F0"},
    {"étiquette", "1F3F7"},
    {"arc_et_flèche", "1F3F9"},
    {"peau_légèrement_mate", "1F3FD"},
    {"peau_foncée", "1F3FF"},
    {"bœuf", "1F402"},
    {"léopard", "1F406"},
    {"bélier", "1F40F"},
    {"chèvre", "1F410"},
    {"éléphant", "1F418"},
    {"poussin_qui_éclôt", "1F423"},
    {"tête_de_souris", "1F42D"},
    {"tête_de_vache", "1F42E"},
    {"tête_de_tigre", "1F42F"},
    {"tête_de_lapin", "1F430"},
    {"tête_de_chat", "1F431"},
    {"tête_de_dragon", "1F432"},
    {"baleine_soufflant_par_son_évent", "1F433"},
    {"tête_de_cheval", "1F434"},
    {"tête_de_singe", "1F435"},
    {"tête_de_chien", "1F436"},
    {"tête_de_cochon", "1F437"},
    {"écureuil", "1F43F"},
    {"œil", "1F441"},
    {"main_avec_index_pointant_à_gauche", "1F448"},
    {"main_avec_index_pointant_à_gauche_couleur1", "1F448-1F3FB"},
    {"main_avec_index_pointant_à_gauche_couleur2", "1F448-1F3FC"},
    {"main_avec_index_pointant_à_gauche_couleur3", "1F448-1F3FD"},
    {"main_avec_index_pointant_à_gauche_couleur4", "1F448-1F3FE"},
    {"main_avec_index_pointant_à_gauche_couleur5", "1F448-1F3FF"},
    {"main_avec_index_pointant_à_droite", "1F449"},
    {"main_avec_index_pointant_à_droite_couleur1", "1F449-1F3FB"},
    {"main_avec_index_pointant_à_droite_couleur2", "1F449-1F3FC"},
    {"main_avec_index_pointant_à_droite_couleur3", "1F449-1F3FD"},
    {"main_avec_index_pointant_à_droite_couleur4", "1F449-1F3FE"},
    {"main_avec_index_pointant_à_droite_couleur5", "1F449-1F3FF"},
    {"vêtements_de_femme", "1F45A"},
    {"sac_à_main", "1F45C"},
    {"chaussure_à_talon_haut", "1F460"},
    {"garçon", "1F466"},
    {"garçon_couleur1", "1F466-1F3FB"},
    {"garçon_couleur2", "1F466-1F3FC"},
    {"garçon_couleur3", "1F466-1F3FD"},
    {"garçon_couleur4", "1F466-1F3FE"},
    {"garçon_couleur5", "1F466-1F3FF"},
    {"personne_mariée_avec_voile", "1F470"},
    {"personne_mariée_avec_voile_couleur1", "1F470-1F3FB"},
    {"personne_mariée_avec_voile_couleur2", "1F470-1F3FC"},
    {"personne_mariée_avec_voile_couleur3", "1F470-1F3FD"},
    {"personne_mariée_avec_voile_couleur4", "1F470-1F3FE"},
    {"personne_mariée_avec_voile_couleur5", "1F470-1F3FF"},
    {"homme_âgé", "1F474"},
    {"homme_âgé_couleur1", "1F474-1F3FB"},
    {"homme_âgé_couleur2", "1F474-1F3FC"},
    {"homme_âgé_couleur3", "1F474-1F3FD"},
    {"homme_âgé_couleur4", "1F474-1F3FE"},
    {"homme_âgé_couleur5", "1F474-1F3FF"},
    {"femme_âgée", "1F475"},
    {"femme_âgée_couleur1", "1F475-1F3FB"},
    {"femme_âgée_couleur2", "1F475-1F3FC"},
    {"femme_âgée_couleur3", "1F475-1F3FD"},
    {"femme_âgée_couleur4", "1F475-1F3FE"},
    {"femme_âgée_couleur5", "1F475-1F3FF"},
    {"bébé", "1F476"},
    {"bébé_couleur1", "1F476-1F3FB"},
    {"bébé_couleur2", "1F476-1F3FC"},
    {"bébé_couleur3", "1F476-1F3FD"},
    {"bébé_couleur4", "1F476-1F3FE"},
    {"bébé_couleur5", "1F476-1F3FF"},
    {"personnel_du_bâtiment", "1F477"},
    {"personnel_du_bâtiment_couleur1", "1F477-1F3FB"},
    {"personnel_du_bâtiment_couleur2", "1F477-1F3FC"},
    {"personnel_du_bâtiment_couleur3", "1F477-1F3FD"},
    {"personnel_du_bâtiment_couleur4", "1F477-1F3FE"},
    {"personnel_du_bâtiment_couleur5", "1F477-1F3FF"},
    {"fantôme", "1F47B"},
    {"bébé_ange", "1F47C"},
    {"bébé_ange_couleur1", "1F47C-1F3FB"},
    {"bébé_ange_couleur2", "1F47C-1F3FC"},
    {"bébé_ange_couleur3", "1F47C-1F3FD"},
    {"bébé_ange_couleur4", "1F47C-1F3FE"},
    {"bébé_ange_couleur5", "1F47C-1F3FF"},
    {"visage_en_colère_avec_des_cornes", "1F47F"},
    {"crâne", "1F480"},
    {"rouge_à_lèvres", "1F484"},
    {"vernis_à_ongles", "1F485"},
    {"vernis_à_ongles_couleur1", "1F485-1F3FB"},
    {"vernis_à_ongles_couleur2", "1F485-1F3FC"},
    {"vernis_à_ongles_couleur3", "1F485-1F3FD"},
    {"vernis_à_ongles_couleur4", "1F485-1F3FE"},
    {"vernis_à_ongles_couleur5", "1F485-1F3FF"},
    {"trace_de_rouge_à_lèvres", "1F48B"},
    {"pierre_précieuse", "1F48E"},
    {"couple_avec_cœur", "1F491"},
    {"couple_avec_cœur_couleur1", "1F491-1F3FB"},
    {"couple_avec_cœur_couleur2", "1F491-1F3FC"},
    {"couple_avec_cœur_couleur3", "1F491-1F3FD"},
    {"couple_avec_cœur_couleur4", "1F491-1F3FE"},
    {"couple_avec_cœur_couleur5", "1F491-1F3FF"},
    {"couple_avec_cœur_couleur1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"couple_avec_cœur_couleur1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"couple_avec_cœur_couleur1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"couple_avec_cœur_couleur1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"couple_avec_cœur_couleur2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"couple_avec_cœur_couleur2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"couple_avec_cœur_couleur2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"couple_avec_cœur_couleur2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"couple_avec_cœur_couleur3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"couple_avec_cœur_couleur3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"couple_avec_cœur_couleur3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"couple_avec_cœur_couleur3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"couple_avec_cœur_couleur4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"couple_avec_cœur_couleur4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"couple_avec_cœur_couleur4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"couple_avec_cœur_couleur4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"couple_avec_cœur_couleur5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"couple_avec_cœur_couleur5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"couple_avec_cœur_couleur5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"couple_avec_cœur_couleur5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"cœur_battant", "1F493"},
    {"cœur_brisé", "1F494"},
    {"deux_cœurs", "1F495"},
    {"cœur_étincelant", "1F496"},
    {"cœur_grandissant", "1F497"},
    {"cœur_et_flèche", "1F498"},
    {"cœur_bleu", "1F499"},
    {"cœur_vert", "1F49A"},
    {"cœur_jaune", "1F49B"},
    {"cœur_violet", "1F49C"},
    {"cœur_avec_ruban", "1F49D"},
    {"cœurs_qui_tournent", "1F49E"},
    {"décoration_avec_cœur", "1F49F"},
    {"symbole_de_colère", "1F4A2"},
    {"décamper", "1F4A8"},
    {"biceps_contracté", "1F4AA"},
    {"biceps_contracté_couleur1", "1F4AA-1F3FB"},
    {"biceps_contracté_couleur2", "1F4AA-1F3FC"},
    {"biceps_contracté_couleur3", "1F4AA-1F3FD"},
    {"biceps_contracté_couleur4", "1F4AA-1F3FE"},
    {"biceps_contracté_couleur5", "1F4AA-1F3FF"},
    {"étourdissement", "1F4AB"},
    {"bulle_de_pensée", "1F4AD"},
    {"siège", "1F4BA"},
    {"page_enroulée", "1F4C3"},
    {"éphéméride", "1F4C6"},
    {"graphiques_à_barres", "1F4CA"},
    {"épingle", "1F4CD"},
    {"règle", "1F4CF"},
    {"équerre", "1F4D0"},
    {"livre_fermé", "1F4D5"},
    {"mémo", "1F4DD"},
    {"combiné_téléphonique", "1F4DE"},
    {"boîte_denvoi", "1F4E4"},
    {"boîte_de_réception", "1F4E5"},
    {"message_reçu", "1F4E8"},
    {"enveloppe_avec_flèche", "1F4E9"},
    {"boîte_aux_lettres_fermée_drapeau_baissé", "1F4EA"},
    {"boîte_aux_lettres_fermée_drapeau_levé", "1F4EB"},
    {"boîte_aux_lettres_ouverte_drapeau_levé", "1F4EC"},
    {"boîte_aux_lettres_ouverte_drapeau_baissé", "1F4ED"},
    {"boîte_aux_lettres", "1F4EE"},
    {"téléphone_portable", "1F4F1"},
    {"téléphone_éteint", "1F4F4"},
    {"téléphones_portables_interdits", "1F4F5"},
    {"barres_de_réseau", "1F4F6"},
    {"caméscope", "1F4F9"},
    {"téléviseur", "1F4FA"},
    {"cassette_vidéo", "1F4FC"},
    {"projecteur_cinématographique", "1F4FD"},
    {"bouton_lecture_aléatoire", "1F500"},
    {"bouton_répétition", "1F501"},
    {"bouton_répétition_de_la_piste", "1F502"},
    {"flèches_dans_le_sens_horaire", "1F503"},
    {"flèches_dans_le_sens_antihoraire", "1F504"},
    {"luminosité_faible", "1F505"},
    {"luminosité_élevée", "1F506"},
    {"volume_des_enceintes_élevé", "1F50A"},
    {"câble_avec_fiche_électrique", "1F50C"},
    {"loupe_orientée_à_gauche", "1F50D"},
    {"loupe_orientée_à_droite", "1F50E"},
    {"cadenas_fermé_avec_stylo", "1F50F"},
    {"cadenas_fermé_avec_clé", "1F510"},
    {"clé", "1F511"},
    {"cadenas_fermé", "1F512"},
    {"alarme_désactivée", "1F515"},
    {"chaînons", "1F517"},
    {"flèche_retour", "1F519"},
    {"flèche_fin", "1F51A"},
    {"flèche_activé", "1F51B"},
    {"flèche_bientôt", "1F51C"},
    {"flèche_en_haut", "1F51D"},
    {"clé_à_molette", "1F527"},
    {"vis_et_écrou", "1F529"},
    {"pistolet_à_eau", "1F52B"},
    {"télescope", "1F52D"},
    {"étoile_à_6_branches", "1F52F"},
    {"symbole_japonais_de_débutant", "1F530"},
    {"carré_noir", "1F532"},
    {"carré_blanc", "1F533"},
    {"mosquée", "1F54C"},
    {"chandelier_à_sept_branches", "1F54E"},
    {"homme_daffaires_en_lévitation", "1F574"},
    {"homme_daffaires_en_lévitation_couleur1", "1F574-1F3FB"},
    {"homme_daffaires_en_lévitation_couleur2", "1F574-1F3FC"},
    {"homme_daffaires_en_lévitation_couleur3", "1F574-1F3FD"},
    {"homme_daffaires_en_lévitation_couleur4", "1F574-1F3FE"},
    {"homme_daffaires_en_lévitation_couleur5", "1F574-1F3FF"},
    {"détective", "1F575"},
    {"détective_couleur1", "1F575-1F3FB"},
    {"détective_couleur2", "1F575-1F3FC"},
    {"détective_couleur3", "1F575-1F3FD"},
    {"détective_couleur4", "1F575-1F3FE"},
    {"détective_couleur5", "1F575-1F3FF"},
    {"araignée", "1F577"},
    {"toile_daraignée", "1F578"},
    {"main_levée_doigts_écartés", "1F590"},
    {"main_levée_doigts_écartés_couleur1", "1F590-1F3FB"},
    {"main_levée_doigts_écartés_couleur2", "1F590-1F3FC"},
    {"main_levée_doigts_écartés_couleur3", "1F590-1F3FD"},
    {"main_levée_doigts_écartés_couleur4", "1F590-1F3FE"},
    {"main_levée_doigts_écartés_couleur5", "1F590-1F3FF"},
    {"cœur_noir", "1F5A4"},
    {"boîte_à_dossiers", "1F5C3"},
    {"meuble_à_dossiers", "1F5C4"},
    {"corbeille_à_papiers", "1F5D1"},
    {"bloc_notes_à_spirale", "1F5D2"},
    {"calendrier_à_spirale", "1F5D3"},
    {"clé_ancienne", "1F5DD"},
    {"journal_roulé", "1F5DE"},
    {"tête_qui_parle", "1F5E3"},
    {"bulle_colère_droite", "1F5EF"},
    {"urne_électorale", "1F5F3"},
    {"statue_de_la_liberté", "1F5FD"},
    {"visage_très_souriant_aux_yeux_rieurs", "1F604"},
    {"visage_souriant_avec_yeux_plissés", "1F606"},
    {"visage_souriant_avec_auréole", "1F607"},
    {"visage_faisant_un_clin_dœil", "1F609"},
    {"visage_soulagé", "1F60C"},
    {"visage_souriant_avec_yeux_en_forme_de_cœur", "1F60D"},
    {"visage_blasé", "1F612"},
    {"visage_démoralisé_avec_goutte_de_sueur", "1F613"},
    {"visage_déconcerté", "1F616"},
    {"visage_faisant_un_bisou_avec_les_yeux_fermés", "1F61A"},
    {"visage_qui_tire_la_langue_et_fait_un_clin_dœil", "1F61C"},
    {"visage_qui_tire_la_langue_les_yeux_plissés", "1F61D"},
    {"visage_déçu", "1F61E"},
    {"visage_en_colère", "1F620"},
    {"visage_persévérant", "1F623"},
    {"visage_avec_fumée_sortant_des_narines", "1F624"},
    {"visage_triste_mais_soulagé", "1F625"},
    {"visage_mécontent_avec_bouche_ouverte", "1F626"},
    {"visage_angoissé", "1F627"},
    {"visage_effrayé", "1F628"},
    {"visage_épuisé", "1F629"},
    {"visage_fatigué", "1F62B"},
    {"visage_grimaçant", "1F62C"},
    {"visage_qui_pleure_à_chaudes_larmes", "1F62D"},
    {"visage_ébahi", "1F62F"},
    {"visage_stupéfait", "1F632"},
    {"visage_étourdi", "1F635"},
    {"chat_souriant_aux_yeux_en_cœurs", "1F63B"},
    {"chat_fatigué", "1F640"},
    {"visage_légèrement_mécontent", "1F641"},
    {"visage_avec_un_léger_sourire", "1F642"},
    {"tête_à_lenvers", "1F643"},
    {"personne_qui_lève_la_main", "1F64B"},
    {"personne_qui_lève_la_main_couleur1", "1F64B-1F3FB"},
    {"personne_qui_lève_la_main_couleur2", "1F64B-1F3FC"},
    {"personne_qui_lève_la_main_couleur3", "1F64B-1F3FD"},
    {"personne_qui_lève_la_main_couleur4", "1F64B-1F3FE"},
    {"personne_qui_lève_la_main_couleur5", "1F64B-1F3FF"},
    {"mains_levées", "1F64C"},
    {"mains_levées_couleur1", "1F64C-1F3FB"},
    {"mains_levées_couleur2", "1F64C-1F3FC"},
    {"mains_levées_couleur3", "1F64C-1F3FD"},
    {"mains_levées_couleur4", "1F64C-1F3FE"},
    {"mains_levées_couleur5", "1F64C-1F3FF"},
    {"personne_fronçant_les_sourcils", "1F64D"},
    {"personne_fronçant_les_sourcils_couleur1", "1F64D-1F3FB"},
    {"personne_fronçant_les_sourcils_couleur2", "1F64D-1F3FC"},
    {"personne_fronçant_les_sourcils_couleur3", "1F64D-1F3FD"},
    {"personne_fronçant_les_sourcils_couleur4", "1F64D-1F3FE"},
    {"personne_fronçant_les_sourcils_couleur5", "1F64D-1F3FF"},
    {"mains_en_prière", "1F64F"},
    {"mains_en_prière_couleur1", "1F64F-1F3FB"},
    {"mains_en_prière_couleur2", "1F64F-1F3FC"},
    {"mains_en_prière_couleur3", "1F64F-1F3FD"},
    {"mains_en_prière_couleur4", "1F64F-1F3FE"},
    {"mains_en_prière_couleur5", "1F64F-1F3FF"},
    {"fusée", "1F680"},
    {"hélicoptère", "1F681"},
    {"train_à_grande_vitesse", "1F685"},
    {"métro", "1F687"},
    {"métro_léger", "1F688"},
    {"arrêt_de_bus", "1F68F"},
    {"véhicule_utilitaire_sport", "1F699"},
    {"téléphérique", "1F6A0"},
    {"tramway_aérien", "1F6A1"},
    {"icône_poubelle", "1F6AE"},
    {"dépôt_dordures_interdit", "1F6AF"},
    {"vélo", "1F6B2"},
    {"vélos_interdits", "1F6B3"},
    {"interdit_aux_piétons", "1F6B7"},
    {"traversée_denfants", "1F6B8"},
    {"symbole_bébé", "1F6BC"},
    {"contrôle_des_passeports", "1F6C2"},
    {"canapé_et_lampe", "1F6CB"},
    {"éboulement", "1F6D8"},
    {"bouée_de_sauvetage", "1F6DF"},
    {"marteau_et_clé_à_molette", "1F6E0"},
    {"baril_de_pétrole", "1F6E2"},
    {"voie_ferrée", "1F6E4"},
    {"bateau_à_moteur", "1F6E5"},
    {"avion_au_décollage", "1F6EB"},
    {"avion_à_latterrissage", "1F6EC"},
    {"canoë", "1F6F6"},
    {"planche_à_roulettes", "1F6F9"},
    {"patin_à_roulettes", "1F6FC"},
    {"carré_rouge", "1F7E5"},
    {"carré_bleu", "1F7E6"},
    {"carré_orange", "1F7E7"},
    {"carré_jaune", "1F7E8"},
    {"carré_vert", "1F7E9"},
    {"carré_violet", "1F7EA"},
    {"carré_marron", "1F7EB"},
    {"signe_égal_gras", "1F7F0"},
    {"cœur_blanc", "1F90D"},
    {"cœur_marron", "1F90E"},
    {"pouce_et_index_rapprochés", "1F90F"},
    {"pouce_et_index_rapprochés_couleur1", "1F90F-1F3FB"},
    {"pouce_et_index_rapprochés_couleur2", "1F90F-1F3FC"},
    {"pouce_et_index_rapprochés_couleur3", "1F90F-1F3FD"},
    {"pouce_et_index_rapprochés_couleur4", "1F90F-1F3FE"},
    {"pouce_et_index_rapprochés_couleur5", "1F90F-1F3FF"},
    {"visage_avec_bouche_fermeture_éclair", "1F910"},
    {"visage_avec_thermomètre", "1F912"},
    {"visage_en_pleine_réflexion", "1F914"},
    {"visage_avec_bandage_autour_de_la_tête", "1F915"},
    {"visage_qui_fait_un_câlin", "1F917"},
    {"signe_appel_téléphonique_avec_les_doigts", "1F919"},
    {"signe_appel_téléphonique_avec_les_doigts_couleur1", "1F919-1F3FB"},
    {"signe_appel_téléphonique_avec_les_doigts_couleur2", "1F919-1F3FC"},
    {"signe_appel_téléphonique_avec_les_doigts_couleur3", "1F919-1F3FD"},
    {"signe_appel_téléphonique_avec_les_doigts_couleur4", "1F919-1F3FE"},
    {"signe_appel_téléphonique_avec_les_doigts_couleur5", "1F919-1F3FF"},
    {"dos_de_main_levée", "1F91A"},
    {"dos_de_main_levée_couleur1", "1F91A-1F3FB"},
    {"dos_de_main_levée_couleur2", "1F91A-1F3FC"},
    {"dos_de_main_levée_couleur3", "1F91A-1F3FD"},
    {"dos_de_main_levée_couleur4", "1F91A-1F3FE"},
    {"dos_de_main_levée_couleur5", "1F91A-1F3FF"},
    {"poing_à_gauche", "1F91B"},
    {"poing_à_gauche_couleur1", "1F91B-1F3FB"},
    {"poing_à_gauche_couleur2", "1F91B-1F3FC"},
    {"poing_à_gauche_couleur3", "1F91B-1F3FD"},
    {"poing_à_gauche_couleur4", "1F91B-1F3FE"},
    {"poing_à_gauche_couleur5", "1F91B-1F3FF"},
    {"poing_à_droite", "1F91C"},
    {"poing_à_droite_couleur1", "1F91C-1F3FB"},
    {"poing_à_droite_couleur2", "1F91C-1F3FC"},
    {"poing_à_droite_couleur3", "1F91C-1F3FD"},
    {"poing_à_droite_couleur4", "1F91C-1F3FE"},
    {"poing_à_droite_couleur5", "1F91C-1F3FF"},
    {"poignée_de_main", "1F91D"},
    {"poignée_de_main_couleur1", "1F91D-1F3FB"},
    {"poignée_de_main_couleur2", "1F91D-1F3FC"},
    {"poignée_de_main_couleur3", "1F91D-1F3FD"},
    {"poignée_de_main_couleur4", "1F91D-1F3FE"},
    {"poignée_de_main_couleur5", "1F91D-1F3FF"},
    {"poignée_de_main_couleur1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"poignée_de_main_couleur1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"poignée_de_main_couleur1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"poignée_de_main_couleur1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"poignée_de_main_couleur2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"poignée_de_main_couleur2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"poignée_de_main_couleur2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"poignée_de_main_couleur2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"poignée_de_main_couleur3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"poignée_de_main_couleur3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"poignée_de_main_couleur3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"poignée_de_main_couleur3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"poignée_de_main_couleur4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"poignée_de_main_couleur4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"poignée_de_main_couleur4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"poignée_de_main_couleur4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"poignée_de_main_couleur5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"poignée_de_main_couleur5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"poignée_de_main_couleur5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"poignée_de_main_couleur5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"doigts_croisés", "1F91E"},
    {"doigts_croisés_couleur1", "1F91E-1F3FB"},
    {"doigts_croisés_couleur2", "1F91E-1F3FC"},
    {"doigts_croisés_couleur3", "1F91E-1F3FD"},
    {"doigts_croisés_couleur4", "1F91E-1F3FE"},
    {"doigts_croisés_couleur5", "1F91E-1F3FF"},
    {"visage_nauséeux", "1F922"},
    {"visage_qui_éternue", "1F927"},
    {"visage_avec_les_sourcils_relevés", "1F928"},
    {"visage_avec_des_étoiles_à_la_place_des_yeux", "1F929"},
    {"tête_de_fou", "1F92A"},
    {"tête_qui_explose", "1F92F"},
    {"mère_noël", "1F936"},
    {"mère_noël_couleur1", "1F936-1F3FB"},
    {"mère_noël_couleur2", "1F936-1F3FC"},
    {"mère_noël_couleur3", "1F936-1F3FD"},
    {"mère_noël_couleur4", "1F936-1F3FE"},
    {"mère_noël_couleur5", "1F936-1F3FF"},
    {"personne_qui_hausse_les_épaules", "1F937"},
    {"personne_qui_hausse_les_épaules_couleur1", "1F937-1F3FB"},
    {"personne_qui_hausse_les_épaules_couleur2", "1F937-1F3FC"},
    {"personne_qui_hausse_les_épaules_couleur3", "1F937-1F3FD"},
    {"personne_qui_hausse_les_épaules_couleur4", "1F937-1F3FE"},
    {"personne_qui_hausse_les_épaules_couleur5", "1F937-1F3FF"},
    {"masque_de_plongée", "1F93F"},
    {"fleur_fanée", "1F940"},
    {"cuillère", "1F944"},
    {"médaille_dor", "1F947"},
    {"médaille_dargent", "1F948"},
    {"médaille_de_bronze", "1F949"},
    {"plat_mitonné", "1F958"},
    {"œuf", "1F95A"},
    {"cacahuètes", "1F95C"},
    {"boulette_de_pâte", "1F95F"},
    {"boîte_à_emporter", "1F961"},
    {"bol_avec_cuillère", "1F963"},
    {"légume_à_feuilles_vertes", "1F96C"},
    {"gâteau_de_lune", "1F96E"},
    {"visage_souriant_avec_cœurs", "1F970"},
    {"visage_bâillant", "1F971"},
    {"visage_éméché", "1F974"},
    {"visage_déguisé", "1F978"},
    {"chaussure_de_randonnée", "1F97E"},
    {"tête_de_lion", "1F981"},
    {"lézard", "1F98E"},
    {"rhinocéros", "1F98F"},
    {"zèbre", "1F993"},
    {"hérisson", "1F994"},
    {"huître", "1F9AA"},
    {"cheveux_bouclés", "1F9B1"},
    {"super_héros", "1F9B8"},
    {"super_héros_couleur1", "1F9B8-1F3FB"},
    {"super_héros_couleur2", "1F9B8-1F3FC"},
    {"super_héros_couleur3", "1F9B8-1F3FD"},
    {"super_héros_couleur4", "1F9B8-1F3FE"},
    {"super_héros_couleur5", "1F9B8-1F3FF"},
    {"gilet_de_sécurité", "1F9BA"},
    {"oreille_appareillée", "1F9BB"},
    {"oreille_appareillée_couleur1", "1F9BB-1F3FB"},
    {"oreille_appareillée_couleur2", "1F9BB-1F3FC"},
    {"oreille_appareillée_couleur3", "1F9BB-1F3FD"},
    {"oreille_appareillée_couleur4", "1F9BB-1F3FE"},
    {"oreille_appareillée_couleur5", "1F9BB-1F3FF"},
    {"fauteuil_motorisé", "1F9BC"},
    {"bras_mécanique", "1F9BE"},
    {"jambe_mécanique", "1F9BF"},
    {"maté", "1F9C9"},
    {"glaçon", "1F9CA"},
    {"thé_aux_perles", "1F9CB"},
    {"personne_à_genoux", "1F9CE"},
    {"personne_à_genoux_couleur1", "1F9CE-1F3FB"},
    {"personne_à_genoux_couleur2", "1F9CE-1F3FC"},
    {"personne_à_genoux_couleur3", "1F9CE-1F3FD"},
    {"personne_à_genoux_couleur4", "1F9CE-1F3FE"},
    {"personne_à_genoux_couleur5", "1F9CE-1F3FF"},
    {"personne_âgée", "1F9D3"},
    {"personne_âgée_couleur1", "1F9D3-1F3FB"},
    {"personne_âgée_couleur2", "1F9D3-1F3FC"},
    {"personne_âgée_couleur3", "1F9D3-1F3FD"},
    {"personne_âgée_couleur4", "1F9D3-1F3FE"},
    {"personne_âgée_couleur5", "1F9D3-1F3FF"},
    {"personnage_féérique", "1F9DA"},
    {"personnage_féérique_couleur1", "1F9DA-1F3FB"},
    {"personnage_féérique_couleur2", "1F9DA-1F3FC"},
    {"personnage_féérique_couleur3", "1F9DA-1F3FD"},
    {"personnage_féérique_couleur4", "1F9DA-1F3FE"},
    {"personnage_féérique_couleur5", "1F9DA-1F3FF"},
    {"créature_aquatique", "1F9DC"},
    {"créature_aquatique_couleur1", "1F9DC-1F3FB"},
    {"créature_aquatique_couleur2", "1F9DC-1F3FC"},
    {"créature_aquatique_couleur3", "1F9DC-1F3FD"},
    {"créature_aquatique_couleur4", "1F9DC-1F3FE"},
    {"créature_aquatique_couleur5", "1F9DC-1F3FF"},
    {"génie", "1F9DE"},
    {"cœur_orange", "1F9E1"},
    {"casquette_américaine", "1F9E2"},
    {"pétard", "1F9E8"},
    {"pièce_de_puzzle", "1F9E9"},
    {"tube_à_essai", "1F9EA"},
    {"boîte_de_pétri", "1F9EB"},
    {"boîte_à_outils", "1F9F0"},
    {"épingle_de_sûreté", "1F9F7"},
    {"éponge", "1F9FD"},
    {"reçu", "1F9FE"},
    {"mauvais_œil", "1F9FF"},
    {"maillot_de_bain_une_pièce", "1FA71"},
    {"cœur_bleu_clair", "1FA75"},
    {"cœur_gris", "1FA76"},
    {"cœur_rose", "1FA77"},
    {"stéthoscope", "1FA7A"},
    {"béquille", "1FA7C"},
    {"piñata", "1FA85"},
    {"poupées_russes", "1FA86"},
    {"flûte", "1FA88"},
    {"coffre_au_trésor", "1FA8E"},
    {"planète_à_anneaux", "1FA90"},
    {"accordéon", "1FA97"},
    {"djembé", "1FA98"},
    {"pièce", "1FA99"},
    {"échelle", "1FA9C"},
    {"fenêtre", "1FA9F"},
    {"aiguille_à_coudre", "1FAA1"},
    {"nœud", "1FAA2"},
    {"tapette_à_souris", "1FAA4"},
    {"brosse_à_dents", "1FAA5"},
    {"boule_à_facettes", "1FAA9"},
    {"carte_didentité", "1FAAA"},
    {"éventail", "1FAAD"},
    {"scarabée", "1FAB2"},
    {"nid_avec_œufs", "1FABA"},
    {"méduse", "1FABC"},
    {"cœur", "1FAC0"},
    {"étreinte", "1FAC2"},
    {"créature_velue", "1FAC8"},
    {"élan", "1FACE"},
    {"âne", "1FACF"},
    {"théière", "1FAD6"},
    {"légume_racine", "1FADC"},
    {"éclaboussure", "1FADF"},
    {"visage_en_pointillés", "1FAE5"},
    {"lèvres_qui_se_mordent", "1FAE6"},
    {"visage_secoué", "1FAE8"},
    {"visage_déformé", "1FAEA"},
    {"nuage_de_colère", "1FAEF"},
    {"main_avec_index_et_pouce_croisés", "1FAF0"},
    {"main_avec_index_et_pouce_croisés_couleur1", "1FAF0-1F3FB"},
    {"main_avec_index_et_pouce_croisés_couleur2", "1FAF0-1F3FC"},
    {"main_avec_index_et_pouce_croisés_couleur3", "1FAF0-1F3FD"},
    {"main_avec_index_et_pouce_croisés_couleur4", "1FAF0-1F3FE"},
    {"main_avec_index_et_pouce_croisés_couleur5", "1FAF0-1F3FF"},
    {"mains_qui_forment_un_cœur", "1FAF6"},
    {"mains_qui_forment_un_cœur_couleur1", "1FAF6-1F3FB"},
    {"mains_qui_forment_un_cœur_couleur2", "1FAF6-1F3FC"},
    {"mains_qui_forment_un_cœur_couleur3", "1FAF6-1F3FD"},
    {"mains_qui_forment_un_cœur_couleur4", "1FAF6-1F3FE"},
    {"mains_qui_forment_un_cœur_couleur5", "1FAF6-1F3FF"},
    {"touches_signe_numérique", "0023-FE0F-20E3"},
    {"touches_astérisque", "002A-FE0F-20E3"},
    {"drapeau_île_de_lascension", "1F1E6-1F1E8"},
    {"drapeau_émirats_arabes_unis", "1F1E6-1F1EA"},
    {"drapeau_arménie", "1F1E6-1F1F2"},
    {"drapeau_samoa_américaines", "1F1E6-1F1F8"},
    {"drapeau_îles_åland", "1F1E6-1F1FD"},
    {"drapeau_azerbaïdjan", "1F1E6-1F1FF"},
    {"drapeau_bosnie_herzégovine", "1F1E7-1F1E6"},
    {"drapeau_bahreïn", "1F1E7-1F1ED"},
    {"drapeau_bénin", "1F1E7-1F1EF"},
    {"drapeau_saint_barthélemy", "1F1E7-1F1F1"},
    {"drapeau_pays_bas_caribéens", "1F1E7-1F1F6"},
    {"drapeau_brésil", "1F1E7-1F1F7"},
    {"drapeau_île_bouvet", "1F1E7-1F1FB"},
    {"drapeau_biélorussie", "1F1E7-1F1FE"},
    {"drapeau_îles_cocos", "1F1E8-1F1E8"},
    {"drapeau_république_centrafricaine", "1F1E8-1F1EB"},
    {"drapeau_côte_divoire", "1F1E8-1F1EE"},
    {"drapeau_îles_cook", "1F1E8-1F1F0"},
    {"drapeau_île_clipperton", "1F1E8-1F1F5"},
    {"drapeau_curaçao", "1F1E8-1F1FC"},
    {"drapeau_île_christmas", "1F1E8-1F1FD"},
    {"drapeau_tchéquie", "1F1E8-1F1FF"},
    {"drapeau_république_dominicaine", "1F1E9-1F1F4"},
    {"drapeau_algérie", "1F1E9-1F1FF"},
    {"drapeau_équateur", "1F1EA-1F1E8"},
    {"drapeau_égypte", "1F1EA-1F1EC"},
    {"drapeau_érythrée", "1F1EA-1F1F7"},
    {"drapeau_éthiopie", "1F1EA-1F1F9"},
    {"drapeau_union_européenne", "1F1EA-1F1FA"},
    {"drapeau_îles_malouines", "1F1EB-1F1F0"},
    {"drapeau_micronésie", "1F1EB-1F1F2"},
    {"drapeau_îles_féroé", "1F1EB-1F1F4"},
    {"drapeau_géorgie", "1F1EC-1F1EA"},
    {"drapeau_guyane_française", "1F1EC-1F1EB"},
    {"drapeau_guinée", "1F1EC-1F1F3"},
    {"drapeau_guinée_équatoriale", "1F1EC-1F1F6"},
    {"drapeau_grèce", "1F1EC-1F1F7"},
    {"drapeau_géorgie_du_sud_et_les_îles_sandwich_du_sud", "1F1EC-1F1F8"},
    {"drapeau_guinée_bissau", "1F1EC-1F1FC"},
    {"drapeau_îles_heard_et_macdonald", "1F1ED-1F1F2"},
    {"drapeau_haïti", "1F1ED-1F1F9"},
    {"drapeau_îles_canaries", "1F1EE-1F1E8"},
    {"drapeau_indonésie", "1F1EE-1F1E9"},
    {"drapeau_israël", "1F1EE-1F1F1"},
    {"drapeau_île_de_man", "1F1EE-1F1F2"},
    {"drapeau_territoire_britannique_de_locéan_indien", "1F1EE-1F1F4"},
    {"drapeau_jamaïque", "1F1EF-1F1F2"},
    {"drapeau_saint_christophe_et_niévès", "1F1F0-1F1F3"},
    {"drapeau_corée_du_nord", "1F1F0-1F1F5"},
    {"drapeau_corée_du_sud", "1F1F0-1F1F7"},
    {"drapeau_koweït", "1F1F0-1F1FC"},
    {"drapeau_îles_caïmans", "1F1F0-1F1FE"},
    {"drapeau_monténégro", "1F1F2-1F1EA"},
    {"drapeau_îles_marshall", "1F1F2-1F1ED"},
    {"drapeau_macédoine_du_nord", "1F1F2-1F1F0"},
    {"drapeau_îles_mariannes_du_nord", "1F1F2-1F1F5"},
    {"drapeau_nouvelle_calédonie", "1F1F3-1F1E8"},
    {"drapeau_île_norfolk", "1F1F3-1F1EB"},
    {"drapeau_norvège", "1F1F3-1F1F4"},
    {"drapeau_népal", "1F1F3-1F1F5"},
    {"drapeau_nouvelle_zélande", "1F1F3-1F1FF"},
    {"drapeau_pérou", "1F1F5-1F1EA"},
    {"drapeau_polynésie_française", "1F1F5-1F1EB"},
    {"drapeau_papouasie_nouvelle_guinée", "1F1F5-1F1EC"},
    {"drapeau_îles_pitcairn", "1F1F5-1F1F3"},
    {"drapeau_la_réunion", "1F1F7-1F1EA"},
    {"drapeau_îles_salomon", "1F1F8-1F1E7"},
    {"drapeau_suède", "1F1F8-1F1EA"},
    {"drapeau_sainte_hélène", "1F1F8-1F1ED"},
    {"drapeau_slovénie", "1F1F8-1F1EE"},
    {"drapeau_sénégal", "1F1F8-1F1F3"},
    {"drapeau_sao_tomé_et_principe", "1F1F8-1F1F9"},
    {"drapeau_saint_martin_partie_néerlandaise", "1F1F8-1F1FD"},
    {"drapeau_îles_turques_et_caïques", "1F1F9-1F1E8"},
    {"drapeau_terres_australes_françaises", "1F1F9-1F1EB"},
    {"drapeau_thaïlande", "1F1F9-1F1ED"},
    {"drapeau_turkménistan", "1F1F9-1F1F2"},
    {"drapeau_trinité_et_tobago", "1F1F9-1F1F9"},
    {"drapeau_taïwan", "1F1F9-1F1FC"},
    {"drapeau_îles_mineures_éloignées_des_états_unis", "1F1FA-1F1F2"},
    {"drapeau_états_unis", "1F1FA-1F1F8"},
    {"drapeau_ouzbékistan", "1F1FA-1F1FF"},
    {"drapeau_état_de_la_cité_du_vatican", "1F1FB-1F1E6"},
    {"drapeau_îles_vierges_britanniques", "1F1FB-1F1EC"},
    {"drapeau_îles_vierges_des_états_unis", "1F1FB-1F1EE"},
    {"drapeau_viêt_nam", "1F1FB-1F1F3"},
    {"drapeau_yémen", "1F1FE-1F1EA"},
    {"drapeau_écosse", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"couple_avec_cœur_homme_et_homme", "1F468-200D-2764-FE0F-200D-1F468"},
    {"couple_avec_cœur_homme_et_homme_couleur1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_cœur_homme_et_homme_couleur1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_cœur_homme_et_homme_couleur1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_cœur_homme_et_homme_couleur1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_cœur_homme_et_homme_couleur1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_cœur_homme_et_homme_couleur2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_cœur_homme_et_homme_couleur2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_cœur_homme_et_homme_couleur2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_cœur_homme_et_homme_couleur2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_cœur_homme_et_homme_couleur2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_cœur_homme_et_homme_couleur3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_cœur_homme_et_homme_couleur3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_cœur_homme_et_homme_couleur3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_cœur_homme_et_homme_couleur3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_cœur_homme_et_homme_couleur3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_cœur_homme_et_homme_couleur4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_cœur_homme_et_homme_couleur4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_cœur_homme_et_homme_couleur4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_cœur_homme_et_homme_couleur4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_cœur_homme_et_homme_couleur4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_cœur_homme_et_homme_couleur5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_cœur_homme_et_homme_couleur5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_cœur_homme_et_homme_couleur5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_cœur_homme_et_homme_couleur5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_cœur_homme_et_homme_couleur5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"famille_homme_et_garçon", "1F468-200D-1F466"},
    {"famille_homme_garçon_et_garçon", "1F468-200D-1F466-200D-1F466"},
    {"famille_homme_fille_et_garçon", "1F468-200D-1F467-200D-1F466"},
    {"famille_homme_homme_et_garçon", "1F468-200D-1F468-200D-1F466"},
    {"famille_homme_homme_garçon_et_garçon", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"famille_homme_homme_fille_et_garçon", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"famille_homme_femme_et_garçon", "1F468-200D-1F469-200D-1F466"},
    {"famille_homme_femme_garçon_et_garçon", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"famille_homme_femme_fille_et_garçon", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"couple_avec_cœur_femme_et_homme", "1F469-200D-2764-FE0F-200D-1F468"},
    {"couple_avec_cœur_femme_et_homme_couleur1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_cœur_femme_et_homme_couleur1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_cœur_femme_et_homme_couleur1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_cœur_femme_et_homme_couleur1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_cœur_femme_et_homme_couleur1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_cœur_femme_et_homme_couleur2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_cœur_femme_et_homme_couleur2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_cœur_femme_et_homme_couleur2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_cœur_femme_et_homme_couleur2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_cœur_femme_et_homme_couleur2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_cœur_femme_et_homme_couleur3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_cœur_femme_et_homme_couleur3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_cœur_femme_et_homme_couleur3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_cœur_femme_et_homme_couleur3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_cœur_femme_et_homme_couleur3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_cœur_femme_et_homme_couleur4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_cœur_femme_et_homme_couleur4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_cœur_femme_et_homme_couleur4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_cœur_femme_et_homme_couleur4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_cœur_femme_et_homme_couleur4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_cœur_femme_et_homme_couleur5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"couple_avec_cœur_femme_et_homme_couleur5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"couple_avec_cœur_femme_et_homme_couleur5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"couple_avec_cœur_femme_et_homme_couleur5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"couple_avec_cœur_femme_et_homme_couleur5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"couple_avec_cœur_femme_et_femme", "1F469-200D-2764-FE0F-200D-1F469"},
    {"couple_avec_cœur_femme_et_femme_couleur1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"couple_avec_cœur_femme_et_femme_couleur1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"couple_avec_cœur_femme_et_femme_couleur1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"couple_avec_cœur_femme_et_femme_couleur1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"couple_avec_cœur_femme_et_femme_couleur1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"couple_avec_cœur_femme_et_femme_couleur2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"couple_avec_cœur_femme_et_femme_couleur2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"couple_avec_cœur_femme_et_femme_couleur2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"couple_avec_cœur_femme_et_femme_couleur2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"couple_avec_cœur_femme_et_femme_couleur2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"couple_avec_cœur_femme_et_femme_couleur3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"couple_avec_cœur_femme_et_femme_couleur3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"couple_avec_cœur_femme_et_femme_couleur3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"couple_avec_cœur_femme_et_femme_couleur3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"couple_avec_cœur_femme_et_femme_couleur3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"couple_avec_cœur_femme_et_femme_couleur4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"couple_avec_cœur_femme_et_femme_couleur4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"couple_avec_cœur_femme_et_femme_couleur4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"couple_avec_cœur_femme_et_femme_couleur4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"couple_avec_cœur_femme_et_femme_couleur4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"couple_avec_cœur_femme_et_femme_couleur5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"couple_avec_cœur_femme_et_femme_couleur5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"couple_avec_cœur_femme_et_femme_couleur5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"couple_avec_cœur_femme_et_femme_couleur5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"couple_avec_cœur_femme_et_femme_couleur5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"famille_femme_et_garçon", "1F469-200D-1F466"},
    {"famille_femme_garçon_et_garçon", "1F469-200D-1F466-200D-1F466"},
    {"famille_femme_fille_et_garçon", "1F469-200D-1F467-200D-1F466"},
    {"famille_femme_femme_et_garçon", "1F469-200D-1F469-200D-1F466"},
    {"famille_femme_femme_garçon_et_garçon", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"famille_femme_femme_fille_et_garçon", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"professionnel_de_la_santé", "1F468-200D-2695-FE0F"},
    {"professionnel_de_la_santé_couleur1", "1F468-1F3FB-200D-2695-FE0F"},
    {"professionnel_de_la_santé_couleur2", "1F468-1F3FC-200D-2695-FE0F"},
    {"professionnel_de_la_santé_couleur3", "1F468-1F3FD-200D-2695-FE0F"},
    {"professionnel_de_la_santé_couleur4", "1F468-1F3FE-200D-2695-FE0F"},
    {"professionnel_de_la_santé_couleur5", "1F468-1F3FF-200D-2695-FE0F"},
    {"homme_allaitant_un_bébé", "1F468-200D-1F37C"},
    {"homme_allaitant_un_bébé_couleur1", "1F468-1F3FB-200D-1F37C"},
    {"homme_allaitant_un_bébé_couleur2", "1F468-1F3FC-200D-1F37C"},
    {"homme_allaitant_un_bébé_couleur3", "1F468-1F3FD-200D-1F37C"},
    {"homme_allaitant_un_bébé_couleur4", "1F468-1F3FE-200D-1F37C"},
    {"homme_allaitant_un_bébé_couleur5", "1F468-1F3FF-200D-1F37C"},
    {"étudiant", "1F468-200D-1F393"},
    {"étudiant_couleur1", "1F468-1F3FB-200D-1F393"},
    {"étudiant_couleur2", "1F468-1F3FC-200D-1F393"},
    {"étudiant_couleur3", "1F468-1F3FD-200D-1F393"},
    {"étudiant_couleur4", "1F468-1F3FE-200D-1F393"},
    {"étudiant_couleur5", "1F468-1F3FF-200D-1F393"},
    {"employé_de_bureau", "1F468-200D-1F4BC"},
    {"employé_de_bureau_couleur1", "1F468-1F3FB-200D-1F4BC"},
    {"employé_de_bureau_couleur2", "1F468-1F3FC-200D-1F4BC"},
    {"employé_de_bureau_couleur3", "1F468-1F3FD-200D-1F4BC"},
    {"employé_de_bureau_couleur4", "1F468-1F3FE-200D-1F4BC"},
    {"employé_de_bureau_couleur5", "1F468-1F3FF-200D-1F4BC"},
    {"mécanicien", "1F468-200D-1F527"},
    {"mécanicien_couleur1", "1F468-1F3FB-200D-1F527"},
    {"mécanicien_couleur2", "1F468-1F3FC-200D-1F527"},
    {"mécanicien_couleur3", "1F468-1F3FD-200D-1F527"},
    {"mécanicien_couleur4", "1F468-1F3FE-200D-1F527"},
    {"mécanicien_couleur5", "1F468-1F3FF-200D-1F527"},
    {"homme_en_fauteuil_motorisé", "1F468-200D-1F9BC"},
    {"homme_en_fauteuil_motorisé_couleur1", "1F468-1F3FB-200D-1F9BC"},
    {"homme_en_fauteuil_motorisé_couleur2", "1F468-1F3FC-200D-1F9BC"},
    {"homme_en_fauteuil_motorisé_couleur3", "1F468-1F3FD-200D-1F9BC"},
    {"homme_en_fauteuil_motorisé_couleur4", "1F468-1F3FE-200D-1F9BC"},
    {"homme_en_fauteuil_motorisé_couleur5", "1F468-1F3FF-200D-1F9BC"},
    {"homme_en_fauteuil_motorisé_vers_la_droite", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"homme_en_fauteuil_motorisé_vers_la_droite_couleur1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"homme_en_fauteuil_motorisé_vers_la_droite_couleur2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"homme_en_fauteuil_motorisé_vers_la_droite_couleur3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"homme_en_fauteuil_motorisé_vers_la_droite_couleur4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"homme_en_fauteuil_motorisé_vers_la_droite_couleur5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"professionnelle_de_la_santé", "1F469-200D-2695-FE0F"},
    {"professionnelle_de_la_santé_couleur1", "1F469-1F3FB-200D-2695-FE0F"},
    {"professionnelle_de_la_santé_couleur2", "1F469-1F3FC-200D-2695-FE0F"},
    {"professionnelle_de_la_santé_couleur3", "1F469-1F3FD-200D-2695-FE0F"},
    {"professionnelle_de_la_santé_couleur4", "1F469-1F3FE-200D-2695-FE0F"},
    {"professionnelle_de_la_santé_couleur5", "1F469-1F3FF-200D-2695-FE0F"},
    {"fermière", "1F469-200D-1F33E"},
    {"fermière_couleur1", "1F469-1F3FB-200D-1F33E"},
    {"fermière_couleur2", "1F469-1F3FC-200D-1F33E"},
    {"fermière_couleur3", "1F469-1F3FD-200D-1F33E"},
    {"fermière_couleur4", "1F469-1F3FE-200D-1F33E"},
    {"fermière_couleur5", "1F469-1F3FF-200D-1F33E"},
    {"cuisinière", "1F469-200D-1F373"},
    {"cuisinière_couleur1", "1F469-1F3FB-200D-1F373"},
    {"cuisinière_couleur2", "1F469-1F3FC-200D-1F373"},
    {"cuisinière_couleur3", "1F469-1F3FD-200D-1F373"},
    {"cuisinière_couleur4", "1F469-1F3FE-200D-1F373"},
    {"cuisinière_couleur5", "1F469-1F3FF-200D-1F373"},
    {"femme_allaitant_un_bébé", "1F469-200D-1F37C"},
    {"femme_allaitant_un_bébé_couleur1", "1F469-1F3FB-200D-1F37C"},
    {"femme_allaitant_un_bébé_couleur2", "1F469-1F3FC-200D-1F37C"},
    {"femme_allaitant_un_bébé_couleur3", "1F469-1F3FD-200D-1F37C"},
    {"femme_allaitant_un_bébé_couleur4", "1F469-1F3FE-200D-1F37C"},
    {"femme_allaitant_un_bébé_couleur5", "1F469-1F3FF-200D-1F37C"},
    {"étudiante", "1F469-200D-1F393"},
    {"étudiante_couleur1", "1F469-1F3FB-200D-1F393"},
    {"étudiante_couleur2", "1F469-1F3FC-200D-1F393"},
    {"étudiante_couleur3", "1F469-1F3FD-200D-1F393"},
    {"étudiante_couleur4", "1F469-1F3FE-200D-1F393"},
    {"étudiante_couleur5", "1F469-1F3FF-200D-1F393"},
    {"ouvrière", "1F469-200D-1F3ED"},
    {"ouvrière_couleur1", "1F469-1F3FB-200D-1F3ED"},
    {"ouvrière_couleur2", "1F469-1F3FC-200D-1F3ED"},
    {"ouvrière_couleur3", "1F469-1F3FD-200D-1F3ED"},
    {"ouvrière_couleur4", "1F469-1F3FE-200D-1F3ED"},
    {"ouvrière_couleur5", "1F469-1F3FF-200D-1F3ED"},
    {"employée_de_bureau", "1F469-200D-1F4BC"},
    {"employée_de_bureau_couleur1", "1F469-1F3FB-200D-1F4BC"},
    {"employée_de_bureau_couleur2", "1F469-1F3FC-200D-1F4BC"},
    {"employée_de_bureau_couleur3", "1F469-1F3FD-200D-1F4BC"},
    {"employée_de_bureau_couleur4", "1F469-1F3FE-200D-1F4BC"},
    {"employée_de_bureau_couleur5", "1F469-1F3FF-200D-1F4BC"},
    {"mécanicienne", "1F469-200D-1F527"},
    {"mécanicienne_couleur1", "1F469-1F3FB-200D-1F527"},
    {"mécanicienne_couleur2", "1F469-1F3FC-200D-1F527"},
    {"mécanicienne_couleur3", "1F469-1F3FD-200D-1F527"},
    {"mécanicienne_couleur4", "1F469-1F3FE-200D-1F527"},
    {"mécanicienne_couleur5", "1F469-1F3FF-200D-1F527"},
    {"femme_en_fauteuil_motorisé", "1F469-200D-1F9BC"},
    {"femme_en_fauteuil_motorisé_couleur1", "1F469-1F3FB-200D-1F9BC"},
    {"femme_en_fauteuil_motorisé_couleur2", "1F469-1F3FC-200D-1F9BC"},
    {"femme_en_fauteuil_motorisé_couleur3", "1F469-1F3FD-200D-1F9BC"},
    {"femme_en_fauteuil_motorisé_couleur4", "1F469-1F3FE-200D-1F9BC"},
    {"femme_en_fauteuil_motorisé_couleur5", "1F469-1F3FF-200D-1F9BC"},
    {"femme_en_fauteuil_motorisé_vers_la_droite", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"femme_en_fauteuil_motorisé_vers_la_droite_couleur1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"femme_en_fauteuil_motorisé_vers_la_droite_couleur2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"femme_en_fauteuil_motorisé_vers_la_droite_couleur3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"femme_en_fauteuil_motorisé_vers_la_droite_couleur4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"femme_en_fauteuil_motorisé_vers_la_droite_couleur5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"personne_à_genoux_vers_la_droite", "1F9CE-200D-27A1-FE0F"},
    {"personne_à_genoux_vers_la_droite_couleur1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"personne_à_genoux_vers_la_droite_couleur2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"personne_à_genoux_vers_la_droite_couleur3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"personne_à_genoux_vers_la_droite_couleur4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"personne_à_genoux_vers_la_droite_couleur5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"professionnel_de_la_santé_tous_genres", "1F9D1-200D-2695-FE0F"},
    {"professionnel_de_la_santé_tous_genres_couleur1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"professionnel_de_la_santé_tous_genres_couleur2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"professionnel_de_la_santé_tous_genres_couleur3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"professionnel_de_la_santé_tous_genres_couleur4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"professionnel_de_la_santé_tous_genres_couleur5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"personne_allaitant_un_bébé", "1F9D1-200D-1F37C"},
    {"personne_allaitant_un_bébé_couleur1", "1F9D1-1F3FB-200D-1F37C"},
    {"personne_allaitant_un_bébé_couleur2", "1F9D1-1F3FC-200D-1F37C"},
    {"personne_allaitant_un_bébé_couleur3", "1F9D1-1F3FD-200D-1F37C"},
    {"personne_allaitant_un_bébé_couleur4", "1F9D1-1F3FE-200D-1F37C"},
    {"personne_allaitant_un_bébé_couleur5", "1F9D1-1F3FF-200D-1F37C"},
    {"étudiant_tous_genres", "1F9D1-200D-1F393"},
    {"étudiant_tous_genres_couleur1", "1F9D1-1F3FB-200D-1F393"},
    {"étudiant_tous_genres_couleur2", "1F9D1-1F3FC-200D-1F393"},
    {"étudiant_tous_genres_couleur3", "1F9D1-1F3FD-200D-1F393"},
    {"étudiant_tous_genres_couleur4", "1F9D1-1F3FE-200D-1F393"},
    {"étudiant_tous_genres_couleur5", "1F9D1-1F3FF-200D-1F393"},
    {"employé_de_bureau_tous_genres", "1F9D1-200D-1F4BC"},
    {"employé_de_bureau_tous_genres_couleur1", "1F9D1-1F3FB-200D-1F4BC"},
    {"employé_de_bureau_tous_genres_couleur2", "1F9D1-1F3FC-200D-1F4BC"},
    {"employé_de_bureau_tous_genres_couleur3", "1F9D1-1F3FD-200D-1F4BC"},
    {"employé_de_bureau_tous_genres_couleur4", "1F9D1-1F3FE-200D-1F4BC"},
    {"employé_de_bureau_tous_genres_couleur5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mécanicien_tous_genres", "1F9D1-200D-1F527"},
    {"mécanicien_tous_genres_couleur1", "1F9D1-1F3FB-200D-1F527"},
    {"mécanicien_tous_genres_couleur2", "1F9D1-1F3FC-200D-1F527"},
    {"mécanicien_tous_genres_couleur3", "1F9D1-1F3FD-200D-1F527"},
    {"mécanicien_tous_genres_couleur4", "1F9D1-1F3FE-200D-1F527"},
    {"mécanicien_tous_genres_couleur5", "1F9D1-1F3FF-200D-1F527"},
    {"personne_en_fauteuil_motorisé", "1F9D1-200D-1F9BC"},
    {"personne_en_fauteuil_motorisé_couleur1", "1F9D1-1F3FB-200D-1F9BC"},
    {"personne_en_fauteuil_motorisé_couleur2", "1F9D1-1F3FC-200D-1F9BC"},
    {"personne_en_fauteuil_motorisé_couleur3", "1F9D1-1F3FD-200D-1F9BC"},
    {"personne_en_fauteuil_motorisé_couleur4", "1F9D1-1F3FE-200D-1F9BC"},
    {"personne_en_fauteuil_motorisé_couleur5", "1F9D1-1F3FF-200D-1F9BC"},
    {"personne_en_fauteuil_motorisé_vers_la_droite", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"personne_en_fauteuil_motorisé_vers_la_droite_couleur1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"personne_en_fauteuil_motorisé_vers_la_droite_couleur2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"personne_en_fauteuil_motorisé_vers_la_droite_couleur3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"personne_en_fauteuil_motorisé_vers_la_droite_couleur4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"personne_en_fauteuil_motorisé_vers_la_droite_couleur5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"femme_haltérophile", "1F3CB-FE0F-200D-2640-FE0F"},
    {"femme_haltérophile_couleur1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"femme_haltérophile_couleur2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"femme_haltérophile_couleur3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"femme_haltérophile_couleur4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"femme_haltérophile_couleur5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"homme_haltérophile", "1F3CB-FE0F-200D-2642-FE0F"},
    {"homme_haltérophile_couleur1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"homme_haltérophile_couleur2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"homme_haltérophile_couleur3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"homme_haltérophile_couleur4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"homme_haltérophile_couleur5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"policière", "1F46E-200D-2640-FE0F"},
    {"policière_couleur1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"policière_couleur2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"policière_couleur3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"policière_couleur4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"policière_couleur5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"ouvrière_du_bâtiment", "1F477-200D-2640-FE0F"},
    {"ouvrière_du_bâtiment_couleur1", "1F477-1F3FB-200D-2640-FE0F"},
    {"ouvrière_du_bâtiment_couleur2", "1F477-1F3FC-200D-2640-FE0F"},
    {"ouvrière_du_bâtiment_couleur3", "1F477-1F3FD-200D-2640-FE0F"},
    {"ouvrière_du_bâtiment_couleur4", "1F477-1F3FE-200D-2640-FE0F"},
    {"ouvrière_du_bâtiment_couleur5", "1F477-1F3FF-200D-2640-FE0F"},
    {"ouvrier_du_bâtiment", "1F477-200D-2642-FE0F"},
    {"ouvrier_du_bâtiment_couleur1", "1F477-1F3FB-200D-2642-FE0F"},
    {"ouvrier_du_bâtiment_couleur2", "1F477-1F3FC-200D-2642-FE0F"},
    {"ouvrier_du_bâtiment_couleur3", "1F477-1F3FD-200D-2642-FE0F"},
    {"ouvrier_du_bâtiment_couleur4", "1F477-1F3FE-200D-2642-FE0F"},
    {"ouvrier_du_bâtiment_couleur5", "1F477-1F3FF-200D-2642-FE0F"},
    {"détective_femme", "1F575-FE0F-200D-2640-FE0F"},
    {"détective_femme_couleur1", "1F575-1F3FB-200D-2640-FE0F"},
    {"détective_femme_couleur2", "1F575-1F3FC-200D-2640-FE0F"},
    {"détective_femme_couleur3", "1F575-1F3FD-200D-2640-FE0F"},
    {"détective_femme_couleur4", "1F575-1F3FE-200D-2640-FE0F"},
    {"détective_femme_couleur5", "1F575-1F3FF-200D-2640-FE0F"},
    {"détective_homme", "1F575-FE0F-200D-2642-FE0F"},
    {"détective_homme_couleur1", "1F575-1F3FB-200D-2642-FE0F"},
    {"détective_homme_couleur2", "1F575-1F3FC-200D-2642-FE0F"},
    {"détective_homme_couleur3", "1F575-1F3FD-200D-2642-FE0F"},
    {"détective_homme_couleur4", "1F575-1F3FE-200D-2642-FE0F"},
    {"détective_homme_couleur5", "1F575-1F3FF-200D-2642-FE0F"},
    {"femme_qui_lève_la_main", "1F64B-200D-2640-FE0F"},
    {"femme_qui_lève_la_main_couleur1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"femme_qui_lève_la_main_couleur2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"femme_qui_lève_la_main_couleur3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"femme_qui_lève_la_main_couleur4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"femme_qui_lève_la_main_couleur5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"homme_qui_lève_la_main", "1F64B-200D-2642-FE0F"},
    {"homme_qui_lève_la_main_couleur1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"homme_qui_lève_la_main_couleur2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"homme_qui_lève_la_main_couleur3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"homme_qui_lève_la_main_couleur4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"homme_qui_lève_la_main_couleur5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"femme_fronçant_les_sourcils", "1F64D-200D-2640-FE0F"},
    {"femme_fronçant_les_sourcils_couleur1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"femme_fronçant_les_sourcils_couleur2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"femme_fronçant_les_sourcils_couleur3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"femme_fronçant_les_sourcils_couleur4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"femme_fronçant_les_sourcils_couleur5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"homme_fronçant_les_sourcils", "1F64D-200D-2642-FE0F"},
    {"homme_fronçant_les_sourcils_couleur1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"homme_fronçant_les_sourcils_couleur2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"homme_fronçant_les_sourcils_couleur3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"homme_fronçant_les_sourcils_couleur4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"homme_fronçant_les_sourcils_couleur5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"femme_qui_hausse_les_épaules", "1F937-200D-2640-FE0F"},
    {"femme_qui_hausse_les_épaules_couleur1", "1F937-1F3FB-200D-2640-FE0F"},
    {"femme_qui_hausse_les_épaules_couleur2", "1F937-1F3FC-200D-2640-FE0F"},
    {"femme_qui_hausse_les_épaules_couleur3", "1F937-1F3FD-200D-2640-FE0F"},
    {"femme_qui_hausse_les_épaules_couleur4", "1F937-1F3FE-200D-2640-FE0F"},
    {"femme_qui_hausse_les_épaules_couleur5", "1F937-1F3FF-200D-2640-FE0F"},
    {"homme_qui_hausse_les_épaules", "1F937-200D-2642-FE0F"},
    {"homme_qui_hausse_les_épaules_couleur1", "1F937-1F3FB-200D-2642-FE0F"},
    {"homme_qui_hausse_les_épaules_couleur2", "1F937-1F3FC-200D-2642-FE0F"},
    {"homme_qui_hausse_les_épaules_couleur3", "1F937-1F3FD-200D-2642-FE0F"},
    {"homme_qui_hausse_les_épaules_couleur4", "1F937-1F3FE-200D-2642-FE0F"},
    {"homme_qui_hausse_les_épaules_couleur5", "1F937-1F3FF-200D-2642-FE0F"},
    {"super_héroïne", "1F9B8-200D-2640-FE0F"},
    {"super_héroïne_couleur1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"super_héroïne_couleur2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"super_héroïne_couleur3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"super_héroïne_couleur4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"super_héroïne_couleur5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"super_héros_homme", "1F9B8-200D-2642-FE0F"},
    {"super_héros_homme_couleur1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"super_héros_homme_couleur2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"super_héros_homme_couleur3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"super_héros_homme_couleur4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"super_héros_homme_couleur5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"femme_à_genoux", "1F9CE-200D-2640-FE0F"},
    {"femme_à_genoux_couleur1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"femme_à_genoux_couleur2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"femme_à_genoux_couleur3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"femme_à_genoux_couleur4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"femme_à_genoux_couleur5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"femme_à_genoux_vers_la_droite", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_à_genoux_vers_la_droite_couleur1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_à_genoux_vers_la_droite_couleur2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_à_genoux_vers_la_droite_couleur3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_à_genoux_vers_la_droite_couleur4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"femme_à_genoux_vers_la_droite_couleur5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"homme_à_genoux", "1F9CE-200D-2642-FE0F"},
    {"homme_à_genoux_couleur1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"homme_à_genoux_couleur2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"homme_à_genoux_couleur3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"homme_à_genoux_couleur4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"homme_à_genoux_couleur5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"homme_à_genoux_vers_la_droite", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_à_genoux_vers_la_droite_couleur1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_à_genoux_vers_la_droite_couleur2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_à_genoux_vers_la_droite_couleur3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_à_genoux_vers_la_droite_couleur4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"homme_à_genoux_vers_la_droite_couleur5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"fée", "1F9DA-200D-2640-FE0F"},
    {"fée_couleur1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"fée_couleur2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"fée_couleur3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"fée_couleur4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"fée_couleur5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"féetaud", "1F9DA-200D-2642-FE0F"},
    {"féetaud_couleur1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"féetaud_couleur2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"féetaud_couleur3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"féetaud_couleur4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"féetaud_couleur5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"sirène", "1F9DC-200D-2640-FE0F"},
    {"sirène_couleur1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"sirène_couleur2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"sirène_couleur3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"sirène_couleur4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"sirène_couleur5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"génie_femme", "1F9DE-200D-2640-FE0F"},
    {"génie_homme", "1F9DE-200D-2642-FE0F"},
    {"homme_cheveux_bouclés", "1F468-200D-1F9B1"},
    {"homme_cheveux_bouclés_couleur1", "1F468-1F3FB-200D-1F9B1"},
    {"homme_cheveux_bouclés_couleur2", "1F468-1F3FC-200D-1F9B1"},
    {"homme_cheveux_bouclés_couleur3", "1F468-1F3FD-200D-1F9B1"},
    {"homme_cheveux_bouclés_couleur4", "1F468-1F3FE-200D-1F9B1"},
    {"homme_cheveux_bouclés_couleur5", "1F468-1F3FF-200D-1F9B1"},
    {"femme_cheveux_bouclés", "1F469-200D-1F9B1"},
    {"femme_cheveux_bouclés_couleur1", "1F469-1F3FB-200D-1F9B1"},
    {"femme_cheveux_bouclés_couleur2", "1F469-1F3FC-200D-1F9B1"},
    {"femme_cheveux_bouclés_couleur3", "1F469-1F3FD-200D-1F9B1"},
    {"femme_cheveux_bouclés_couleur4", "1F469-1F3FE-200D-1F9B1"},
    {"femme_cheveux_bouclés_couleur5", "1F469-1F3FF-200D-1F9B1"},
    {"adulte_cheveux_bouclés", "1F9D1-200D-1F9B1"},
    {"adulte_cheveux_bouclés_couleur1", "1F9D1-1F3FB-200D-1F9B1"},
    {"adulte_cheveux_bouclés_couleur2", "1F9D1-1F3FC-200D-1F9B1"},
    {"adulte_cheveux_bouclés_couleur3", "1F9D1-1F3FD-200D-1F9B1"},
    {"adulte_cheveux_bouclés_couleur4", "1F9D1-1F3FE-200D-1F9B1"},
    {"adulte_cheveux_bouclés_couleur5", "1F9D1-1F3FF-200D-1F9B1"},
    {"chaîne_brisée", "26D3-FE0F-200D-1F4A5"},
    {"cœur_enflammé", "2764-FE0F-200D-1F525"},
    {"cœur_avec_bandages", "2764-FE0F-200D-1FA79"},
    {"phénix", "1F426-200D-1F525"},
    {"œil_dans_une_bulle_de_bd", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"tête_secouée_horizontalement", "1F642-200D-2194-FE0F"},
    {"tête_secouée_verticalement", "1F642-200D-2195-FE0F"},
};

