// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_ms[] = {
    {"tanda_soal_seru", "2049"},
    {"tanda_dagangan", "2122"},
    {"sumber_maklumat", "2139"},
    {"anak_panah_ke_kiri_ke_kanan", "2194"},
    {"anak_panah_ke_atas_ke_bawah", "2195"},
    {"anak_panah_ke_penjuru_kiri_atas", "2196"},
    {"anak_panah_penjuru_atas_kanan", "2197"},
    {"anak_panah_penjuru_bawah_kanan", "2198"},
    {"anak_panah_ke_penjuru_kiri_bawah", "2199"},
    {"papan_kekunci", "2328"},
    {"matahari", "2600"},
    {"awan", "2601"},
    {"payung", "2602"},
    {"orang_orang_salji", "2603"},
    {"komet", "2604"},
    {"peti_undi_dengan_tanda_semak", "2611"},
    {"payung_dengan_titisan_hujan", "2614"},
    {"minuman_panas", "2615"},
    {"syamrok", "2618"},
    {"tengkorak_dan_tulang_bersilang", "2620"},
    {"radioaktif", "2622"},
    {"biobahaya", "2623"},
    {"salib_ortodoks", "2626"},
    {"roda_dharma", "2638"},
    {"muka_berkerut", "2639"},
    {"simbol_perempuan", "2640"},
    {"simbol_lelaki", "2642"},
    {"aries", "2648"},
    {"taurus", "2649"},
    {"sagittarius", "2650"},
    {"capricorn", "2651"},
    {"aquarius", "2652"},
    {"pisces", "2653"},
    {"daun_sped", "2660"},
    {"daun_kelawar", "2663"},
    {"daun_lekuk", "2665"},
    {"daun_daiman", "2666"},
    {"mata_air_panas", "2668"},
    {"tukul_dan_beliung", "2692"},
    {"sauh", "2693"},
    {"pedang_bersilang", "2694"},
    {"simbol_perubatan", "2695"},
    {"neraca_timbang", "2696"},
    {"alembic", "2697"},
    {"gear", "2699"},
    {"gunting", "2702"},
    {"butang_tanda_rait", "2705"},
    {"kapal_terbang", "2708"},
    {"sampul_surat", "2709"},
    {"mata_pen_hitam", "2712"},
    {"tanda_rait", "2714"},
    {"darab", "2716"},
    {"bintang_daud", "2721"},
    {"berkilauan", "2728"},
    {"asterisk_berjejari_lapan", "2733"},
    {"bintang_berpenjuru_lapan", "2734"},
    {"emping_salji", "2744"},
    {"kilauan", "2747"},
    {"tanda_soal", "2753"},
    {"tanda_soal_putih", "2754"},
    {"tanda_seru_putih", "2755"},
    {"tanda_seru_merah", "2757"},
    {"tanda_seru_hati", "2763"},
    {"hati_merah", "2764"},
    {"tambah", "2795"},
    {"tolak", "2796"},
    {"bahagi", "2797"},
    {"anak_panah_ke_kanan_melengkung_ke_atas", "2934"},
    {"anak_panah_ke_kanan_melengkung_ke_bawah", "2935"},
    {"sengkang_berombak_ombak", "3030"},
    {"butang_tahniah_jepun", "3297"},
    {"butang_rahsia_jepun", "3299"},
    {"hak_cipta", "00A9"},
    {"berdaftar", "00AE"},
    {"tanda_seru_berganda", "203C"},
    {"anak_panah_ke_kanan_melengkung_ke_kiri", "21A9"},
    {"anak_panah_ke_kiri_melengkung_ke_kanan", "21AA"},
    {"jam_tangan", "231A"},
    {"jam_pasir", "231B"},
    {"butang_keluarkan", "23CF"},
    {"butang_mara_laju", "23E9"},
    {"butang_undur_laju", "23EA"},
    {"butang_laju_ke_atas", "23EB"},
    {"butang_laju_ke_bawah", "23EC"},
    {"butang_trek_berikutnya", "23ED"},
    {"butang_runut_terakhir", "23EE"},
    {"butang_main_atau_jeda", "23EF"},
    {"jam_loceng", "23F0"},
    {"jam_randik", "23F1"},
    {"jam_pemasa", "23F2"},
    {"jam_pasir_dengan_pasir_mengalir", "23F3"},
    {"butang_jeda", "23F8"},
    {"butang_henti", "23F9"},
    {"butang_rakam", "23FA"},
    {"huruf_m_dibulatkan", "24C2"},
    {"persegi_kecil_hitam", "25AA"},
    {"persegi_kecil_putih", "25AB"},
    {"butang_main", "25B6"},
    {"butang_undur", "25C0"},
    {"persegi_sederhana_putih", "25FB"},
    {"persegi_sederhana_hitam", "25FC"},
    {"persegi_sederhana_kecil_putih", "25FD"},
    {"persegi_sederhana_kecil_hitam", "25FE"},
    {"telefon", "260E"},
    {"jari_telunjuk_menunjuk_ke_atas", "261D"},
    {"jari_telunjuk_menunjuk_ke_atas_warna1", "261D-1F3FB"},
    {"jari_telunjuk_menunjuk_ke_atas_warna2", "261D-1F3FC"},
    {"jari_telunjuk_menunjuk_ke_atas_warna3", "261D-1F3FD"},
    {"jari_telunjuk_menunjuk_ke_atas_warna4", "261D-1F3FE"},
    {"jari_telunjuk_menunjuk_ke_atas_warna5", "261D-1F3FF"},
    {"bintang_dan_bulan_sabit", "262A"},
    {"simbol_keamanan", "262E"},
    {"yin_yang", "262F"},
    {"muka_tersenyum", "263A"},
    {"gemini", "264A"},
    {"cancer", "264B"},
    {"leo", "264C"},
    {"virgo", "264D"},
    {"libra", "264E"},
    {"scorpio", "264F"},
    {"bidak_catur", "265F"},
    {"simbol_kitar_semula", "267B"},
    {"infiniti", "267E"},
    {"kerusi_roda", "267F"},
    {"simbol_atom", "269B"},
    {"fleur_de_lis", "269C"},
    {"amaran", "26A0"},
    {"voltan_tinggi", "26A1"},
    {"simbol_transgender", "26A7"},
    {"bulatan_putih", "26AA"},
    {"bulatan_hitam", "26AB"},
    {"keranda", "26B0"},
    {"takar_abu_mayat", "26B1"},
    {"bola_sepak", "26BD"},
    {"besbol", "26BE"},
    {"orang_orang_salji_tanpa_salji", "26C4"},
    {"matahari_di_sebalik_awan", "26C5"},
    {"awan_dengan_kilat_dan_hujan", "26C8"},
    {"ophiuchus", "26CE"},
    {"beliung", "26CF"},
    {"topi_keselamatan_dengan_palang_putih", "26D1"},
    {"rantai", "26D3"},
    {"dilarang_masuk", "26D4"},
    {"kuil_shinto", "26E9"},
    {"gereja", "26EA"},
    {"gunung", "26F0"},
    {"payung_di_atas_tanah", "26F1"},
    {"air_pancut", "26F2"},
    {"bendera_dalam_lubang", "26F3"},
    {"feri", "26F4"},
    {"perahu_layar", "26F5"},
    {"pemain_ski", "26F7"},
    {"sepatu_luncur_ais", "26F8"},
    {"orang_melantun_bola", "26F9"},
    {"orang_melantun_bola_warna1", "26F9-1F3FB"},
    {"orang_melantun_bola_warna2", "26F9-1F3FC"},
    {"orang_melantun_bola_warna3", "26F9-1F3FD"},
    {"orang_melantun_bola_warna4", "26F9-1F3FE"},
    {"orang_melantun_bola_warna5", "26F9-1F3FF"},
    {"khemah", "26FA"},
    {"pam_minyak", "26FD"},
    {"angkat_penumbuk", "270A"},
    {"angkat_penumbuk_warna1", "270A-1F3FB"},
    {"angkat_penumbuk_warna2", "270A-1F3FC"},
    {"angkat_penumbuk_warna3", "270A-1F3FD"},
    {"angkat_penumbuk_warna4", "270A-1F3FE"},
    {"angkat_penumbuk_warna5", "270A-1F3FF"},
    {"angkat_tangan", "270B"},
    {"angkat_tangan_warna1", "270B-1F3FB"},
    {"angkat_tangan_warna2", "270B-1F3FC"},
    {"angkat_tangan_warna3", "270B-1F3FD"},
    {"angkat_tangan_warna4", "270B-1F3FE"},
    {"angkat_tangan_warna5", "270B-1F3FF"},
    {"simbol_kemenangan", "270C"},
    {"simbol_kemenangan_warna1", "270C-1F3FB"},
    {"simbol_kemenangan_warna2", "270C-1F3FC"},
    {"simbol_kemenangan_warna3", "270C-1F3FD"},
    {"simbol_kemenangan_warna4", "270C-1F3FE"},
    {"simbol_kemenangan_warna5", "270C-1F3FF"},
    {"tangan_menulis", "270D"},
    {"tangan_menulis_warna1", "270D-1F3FB"},
    {"tangan_menulis_warna2", "270D-1F3FC"},
    {"tangan_menulis_warna3", "270D-1F3FD"},
    {"tangan_menulis_warna4", "270D-1F3FE"},
    {"tangan_menulis_warna5", "270D-1F3FF"},
    {"pensel", "270F"},
    {"salib_latin", "271D"},
    {"tanda_pangkah", "274C"},
    {"butang_tanda_pangkah", "274E"},
    {"anak_panah_ke_kanan", "27A1"},
    {"gelung_ikal", "27B0"},
    {"gelung_ikal_berganda", "27BF"},
    {"anak_panah_ke_kiri", "2B05"},
    {"anak_panah_ke_atas", "2B06"},
    {"anak_panah_ke_bawah", "2B07"},
    {"persegi_besar_hitam", "2B1B"},
    {"persegi_besar_putih", "2B1C"},
    {"bintang_sederhana_putih", "2B50"},
    {"lohong_bulatan_besar", "2B55"},
    {"tanda_penyelingan_bahagian", "303D"},
    {"naga_merah_mahjung", "1F004"},
    {"joker", "1F0CF"},
    {"butang_a_jenis_darah", "1F170"},
    {"butang_b_jenis_darah", "1F171"},
    {"butang_o_jenis_darah", "1F17E"},
    {"butang_p", "1F17F"},
    {"butang_ab_jenis_darah", "1F18E"},
    {"butang_cl", "1F191"},
    {"butang_cool", "1F192"},
    {"butang_free", "1F193"},
    {"butang_id", "1F194"},
    {"butang_new", "1F195"},
    {"butang_ng", "1F196"},
    {"butang_ok", "1F197"},
    {"butang_sos", "1F198"},
    {"butang_up", "1F199"},
    {"butang_vs", "1F19A"},
    {"penunjuk_wilayah_a", "1F1E6"},
    {"penunjuk_wilayah_b", "1F1E7"},
    {"penunjuk_wilayah_c", "1F1E8"},
    {"penunjuk_wilayah_d", "1F1E9"},
    {"penunjuk_wilayah_e", "1F1EA"},
    {"penunjuk_wilayah_f", "1F1EB"},
    {"penunjuk_wilayah_g", "1F1EC"},
    {"penunjuk_wilayah_h", "1F1ED"},
    {"penunjuk_wilayah_i", "1F1EE"},
    {"penunjuk_wilayah_j", "1F1EF"},
    {"penunjuk_wilayah_k", "1F1F0"},
    {"penunjuk_wilayah_l", "1F1F1"},
    {"penunjuk_wilayah_m", "1F1F2"},
    {"penunjuk_wilayah_n", "1F1F3"},
    {"penunjuk_wilayah_o", "1F1F4"},
    {"penunjuk_wilayah_p", "1F1F5"},
    {"penunjuk_wilayah_q", "1F1F6"},
    {"penunjuk_wilayah_r", "1F1F7"},
    {"penunjuk_wilayah_s", "1F1F8"},
    {"penunjuk_wilayah_t", "1F1F9"},
    {"penunjuk_wilayah_u", "1F1FA"},
    {"penunjuk_wilayah_v", "1F1FB"},
    {"penunjuk_wilayah_w", "1F1FC"},
    {"penunjuk_wilayah_x", "1F1FD"},
    {"penunjuk_wilayah_y", "1F1FE"},
    {"penunjuk_wilayah_z", "1F1FF"},
    {"butang_di_sini_jepun", "1F201"},
    {"butang_caj_perkhidmatan_jepun", "1F202"},
    {"butang_percuma_jepun", "1F21A"},
    {"butang_ditempah_jepun", "1F22F"},
    {"butang_dilarang_jepun", "1F232"},
    {"butang_kerja_kosong_jepun", "1F233"},
    {"butang_gred_lulus_jepun", "1F234"},
    {"butang_tiada_kekosongan_jepun", "1F235"},
    {"butang_bukan_percuma_jepun", "1F236"},
    {"butang_amaun_bulanan_jepun", "1F237"},
    {"butang_aplikasi_jepun", "1F238"},
    {"butang_diskaun_jepun", "1F239"},
    {"butang_dibuka_untuk_perniagaan_jepun", "1F23A"},
    {"butang_murah_jepun", "1F250"},
    {"butang_boleh_diterima_jepun", "1F251"},
    {"taufan", "1F300"},
    {"berkabus", "1F301"},
    {"payung_tertutup", "1F302"},
    {"malam_berbintang", "1F303"},
    {"matahari_terbit_di_banjaran_pergunungan", "1F304"},
    {"matahari_terbit", "1F305"},
    {"panorama_kota_waktu_senja", "1F306"},
    {"matahari_terbenam", "1F307"},
    {"pelangi", "1F308"},
    {"jambatan_pada_waktu_malam", "1F309"},
    {"ombak_laut", "1F30A"},
    {"gunung_berapi", "1F30B"},
    {"bima_sakti", "1F30C"},
    {"peta_glob_eropah_afrika", "1F30D"},
    {"peta_glob_amerika", "1F30E"},
    {"peta_glob_asia_australia", "1F30F"},
    {"glob_dengan_garisan_meridian", "1F310"},
    {"bulan_baharu", "1F311"},
    {"bulan_sabit_membesar", "1F312"},
    {"bulan_suku_pertama", "1F313"},
    {"bulan_hampir_purnama_mengambang", "1F314"},
    {"bulan_purnama", "1F315"},
    {"bulan_hampir_purnama_memalap", "1F316"},
    {"bulan_suku_terakhir", "1F317"},
    {"bulan_sabit_memalap", "1F318"},
    {"bulan_sabit", "1F319"},
    {"muka_bulan_baharu", "1F31A"},
    {"muka_bulan_suku_pertama", "1F31B"},
    {"muka_bulan_suku_terakhir", "1F31C"},
    {"muka_bulan_purnama", "1F31D"},
    {"matahari_dengan_muka", "1F31E"},
    {"bintang_bersinar", "1F31F"},
    {"tahi_bintang", "1F320"},
    {"termometer", "1F321"},
    {"matahari_di_sebalik_awan_kecil", "1F324"},
    {"matahari_di_sebalik_awan_besar", "1F325"},
    {"matahari_di_sebalik_awan_hujan", "1F326"},
    {"awan_dengan_hujan", "1F327"},
    {"awan_dengan_salji", "1F328"},
    {"awan_dengan_kilat", "1F329"},
    {"puting_beliung", "1F32A"},
    {"kabus", "1F32B"},
    {"muka_angin", "1F32C"},
    {"hot_dog", "1F32D"},
    {"tako", "1F32E"},
    {"burito", "1F32F"},
    {"buah_berangan", "1F330"},
    {"anak_benih", "1F331"},
    {"malar_hijau", "1F332"},
    {"pokok_daun_luruh", "1F333"},
    {"pokok_palma", "1F334"},
    {"kaktus", "1F335"},
    {"cili_api", "1F336"},
    {"bunga_tulip", "1F337"},
    {"bunga_sakura", "1F338"},
    {"bunga_mawar", "1F339"},
    {"bunga_raya", "1F33A"},
    {"bunga_matahari", "1F33B"},
    {"bunga_mekar", "1F33C"},
    {"tongkol_jagung", "1F33D"},
    {"seberkas_padi", "1F33E"},
    {"herba", "1F33F"},
    {"daun_bunga_cengkih", "1F340"},
    {"daun_mapel", "1F341"},
    {"daun_luruh", "1F342"},
    {"daun_melayang_ditiup_angin", "1F343"},
    {"cendawan", "1F344"},
    {"tomato", "1F345"},
    {"terung", "1F346"},
    {"anggur", "1F347"},
    {"tembikai_susu", "1F348"},
    {"tembikai", "1F349"},
    {"limau_tangerin", "1F34A"},
    {"lemon", "1F34B"},
    {"pisang", "1F34C"},
    {"nanas", "1F34D"},
    {"epal_merah", "1F34E"},
    {"epal_hijau", "1F34F"},
    {"buah_pear", "1F350"},
    {"buah_pic", "1F351"},
    {"buah_ceri", "1F352"},
    {"buah_strawberi", "1F353"},
    {"hamburger", "1F354"},
    {"piza", "1F355"},
    {"daging_salut_tulang", "1F356"},
    {"paha_ayam", "1F357"},
    {"keropok_beras", "1F358"},
    {"bebola_nasi", "1F359"},
    {"nasi_masak", "1F35A"},
    {"nasi_kari", "1F35B"},
    {"mangkuk_kukus", "1F35C"},
    {"spageti", "1F35D"},
    {"roti", "1F35E"},
    {"kentang_goreng", "1F35F"},
    {"ubi_keledek_bakar", "1F360"},
    {"dango", "1F361"},
    {"oden", "1F362"},
    {"susyi", "1F363"},
    {"udang_goreng", "1F364"},
    {"kek_ikan_dengan_corak_pusar", "1F365"},
    {"ais_krim_lembut", "1F366"},
    {"air_batu_campur", "1F367"},
    {"ais_krim", "1F368"},
    {"donat", "1F369"},
    {"biskut", "1F36A"},
    {"sekeping_coklat", "1F36B"},
    {"gula_gula", "1F36C"},
    {"lolipop", "1F36D"},
    {"kastard", "1F36E"},
    {"periuk_madu", "1F36F"},
    {"sepotong_kek", "1F370"},
    {"kotak_bento", "1F371"},
    {"seperiuk_makanan", "1F372"},
    {"memasak", "1F373"},
    {"garpu_dan_pisau", "1F374"},
    {"cawan_teh_tanpa_tangkai", "1F375"},
    {"sake", "1F376"},
    {"gelas_wain", "1F377"},
    {"gelas_koktel", "1F378"},
    {"minuman_tropika", "1F379"},
    {"kole_bir", "1F37A"},
    {"kole_bir_berdenting", "1F37B"},
    {"botol_susu", "1F37C"},
    {"garpu_dan_pisau_dengan_pinggan", "1F37D"},
    {"botol_dengan_penutup_gabus_terbuka", "1F37E"},
    {"bertih_jagung", "1F37F"},
    {"reben", "1F380"},
    {"bungkusan_hadiah", "1F381"},
    {"kek_hari_jadi", "1F382"},
    {"tanglung_labu", "1F383"},
    {"pokok_krismas", "1F384"},
    {"santa_klaus", "1F385"},
    {"santa_klaus_warna1", "1F385-1F3FB"},
    {"santa_klaus_warna2", "1F385-1F3FC"},
    {"santa_klaus_warna3", "1F385-1F3FD"},
    {"santa_klaus_warna4", "1F385-1F3FE"},
    {"santa_klaus_warna5", "1F385-1F3FF"},
    {"bunga_api", "1F386"},
    {"mercun", "1F387"},
    {"belon", "1F388"},
    {"pencetus_bunyi_parti", "1F389"},
    {"bebola_konfeti", "1F38A"},
    {"pokok_tanabata", "1F38B"},
    {"bendera_bersilang", "1F38C"},
    {"hiasan_pain", "1F38D"},
    {"anak_patung_jepun", "1F38E"},
    {"panji_panji_kap", "1F38F"},
    {"peranggu_loceng_angin", "1F390"},
    {"upacara_bulan", "1F391"},
    {"beg_galas_sekolah", "1F392"},
    {"topi_konvokesyen", "1F393"},
    {"pingat_tentera", "1F396"},
    {"reben_peringatan", "1F397"},
    {"mikrofon_studio", "1F399"},
    {"pelungsur_aras", "1F39A"},
    {"tombol_kawalan", "1F39B"},
    {"bingkai_filem", "1F39E"},
    {"tiket_masuk", "1F39F"},
    {"kuda_karusel", "1F3A0"},
    {"roda_ferris", "1F3A1"},
    {"roller_coaster", "1F3A2"},
    {"joran_mengail", "1F3A3"},
    {"mikrofon", "1F3A4"},
    {"kamera_filem", "1F3A5"},
    {"panggung", "1F3A6"},
    {"fon_kepala", "1F3A7"},
    {"palet_artis", "1F3A8"},
    {"topi_tinggi", "1F3A9"},
    {"khemah_sarkas", "1F3AA"},
    {"tiket", "1F3AB"},
    {"papan_pengetap", "1F3AC"},
    {"seni_persembahan", "1F3AD"},
    {"permainan_video", "1F3AE"},
    {"balingan_tepat", "1F3AF"},
    {"mesin_judi", "1F3B0"},
    {"biliard", "1F3B1"},
    {"dadu_permainan", "1F3B2"},
    {"boling", "1F3B3"},
    {"terup_bunga", "1F3B4"},
    {"nota_muzik", "1F3B5"},
    {"nota_nota_muzik", "1F3B6"},
    {"saksofon", "1F3B7"},
    {"gitar", "1F3B8"},
    {"keyboard_muzik", "1F3B9"},
    {"trompet", "1F3BA"},
    {"biola", "1F3BB"},
    {"skor_muzik", "1F3BC"},
    {"baju_larian", "1F3BD"},
    {"tenis", "1F3BE"},
    {"ski", "1F3BF"},
    {"bola_keranjang", "1F3C0"},
    {"bendera_pelepas", "1F3C1"},
    {"peluncur_papan_salji", "1F3C2"},
    {"peluncur_papan_salji_warna1", "1F3C2-1F3FB"},
    {"peluncur_papan_salji_warna2", "1F3C2-1F3FC"},
    {"peluncur_papan_salji_warna3", "1F3C2-1F3FD"},
    {"peluncur_papan_salji_warna4", "1F3C2-1F3FE"},
    {"peluncur_papan_salji_warna5", "1F3C2-1F3FF"},
    {"pelari", "1F3C3"},
    {"pelari_warna1", "1F3C3-1F3FB"},
    {"pelari_warna2", "1F3C3-1F3FC"},
    {"pelari_warna3", "1F3C3-1F3FD"},
    {"pelari_warna4", "1F3C3-1F3FE"},
    {"pelari_warna5", "1F3C3-1F3FF"},
    {"peluncur_air", "1F3C4"},
    {"peluncur_air_warna1", "1F3C4-1F3FB"},
    {"peluncur_air_warna2", "1F3C4-1F3FC"},
    {"peluncur_air_warna3", "1F3C4-1F3FD"},
    {"peluncur_air_warna4", "1F3C4-1F3FE"},
    {"peluncur_air_warna5", "1F3C4-1F3FF"},
    {"pingat_sukan", "1F3C5"},
    {"piala", "1F3C6"},
    {"lumba_kuda", "1F3C7"},
    {"lumba_kuda_warna1", "1F3C7-1F3FB"},
    {"lumba_kuda_warna2", "1F3C7-1F3FC"},
    {"lumba_kuda_warna3", "1F3C7-1F3FD"},
    {"lumba_kuda_warna4", "1F3C7-1F3FE"},
    {"lumba_kuda_warna5", "1F3C7-1F3FF"},
    {"bola_sepak_amerika", "1F3C8"},
    {"bola_sepak_ragbi", "1F3C9"},
    {"perenang", "1F3CA"},
    {"perenang_warna1", "1F3CA-1F3FB"},
    {"perenang_warna2", "1F3CA-1F3FC"},
    {"perenang_warna3", "1F3CA-1F3FD"},
    {"perenang_warna4", "1F3CA-1F3FE"},
    {"perenang_warna5", "1F3CA-1F3FF"},
    {"ahli_angkat_berat", "1F3CB"},
    {"ahli_angkat_berat_warna1", "1F3CB-1F3FB"},
    {"ahli_angkat_berat_warna2", "1F3CB-1F3FC"},
    {"ahli_angkat_berat_warna3", "1F3CB-1F3FD"},
    {"ahli_angkat_berat_warna4", "1F3CB-1F3FE"},
    {"ahli_angkat_berat_warna5", "1F3CB-1F3FF"},
    {"pemain_golf", "1F3CC"},
    {"pemain_golf_warna1", "1F3CC-1F3FB"},
    {"pemain_golf_warna2", "1F3CC-1F3FC"},
    {"pemain_golf_warna3", "1F3CC-1F3FD"},
    {"pemain_golf_warna4", "1F3CC-1F3FE"},
    {"pemain_golf_warna5", "1F3CC-1F3FF"},
    {"motosikal", "1F3CD"},
    {"kereta_lumba", "1F3CE"},
    {"kriket", "1F3CF"},
    {"bola_tampar", "1F3D0"},
    {"hoki_padang", "1F3D1"},
    {"hoki_ais", "1F3D2"},
    {"ping_pong", "1F3D3"},
    {"gunung_dengan_puncak_bersalji", "1F3D4"},
    {"berkhemah", "1F3D5"},
    {"pantai_dengan_payung", "1F3D6"},
    {"pembinaan_bangunan", "1F3D7"},
    {"bangunan_perumahan", "1F3D8"},
    {"panorama_kota", "1F3D9"},
    {"rumah_terbiar", "1F3DA"},
    {"bangunan_klasik", "1F3DB"},
    {"padang_pasir", "1F3DC"},
    {"pulau_padang_pasir", "1F3DD"},
    {"taman_negara", "1F3DE"},
    {"stadium", "1F3DF"},
    {"rumah", "1F3E0"},
    {"rumah_dengan_taman", "1F3E1"},
    {"bangunan_pejabat", "1F3E2"},
    {"pejabat_pos_jepun", "1F3E3"},
    {"pejabat_pos", "1F3E4"},
    {"hospital", "1F3E5"},
    {"bank", "1F3E6"},
    {"tanda_atm", "1F3E7"},
    {"hotel", "1F3E8"},
    {"hotel_cinta", "1F3E9"},
    {"kedai_serbaneka", "1F3EA"},
    {"sekolah", "1F3EB"},
    {"gedung_serbaneka", "1F3EC"},
    {"kilang", "1F3ED"},
    {"tanglung_kertas_merah", "1F3EE"},
    {"istana_jepun", "1F3EF"},
    {"istana", "1F3F0"},
    {"bendera_putih", "1F3F3"},
    {"bendera_hitam", "1F3F4"},
    {"roset", "1F3F5"},
    {"label", "1F3F7"},
    {"badminton", "1F3F8"},
    {"panah_dan_busur", "1F3F9"},
    {"amfora", "1F3FA"},
    {"ton_kulit_cerah", "1F3FB"},
    {"ton_kulit_sederhana_cerah", "1F3FC"},
    {"ton_kulit_sederhana", "1F3FD"},
    {"ton_kulit_sederhana_gelap", "1F3FE"},
    {"ton_kulit_gelap", "1F3FF"},
    {"tikus", "1F400"},
    {"mencit", "1F401"},
    {"lembu_jantan", "1F402"},
    {"kerbau", "1F403"},
    {"lembu", "1F404"},
    {"harimau", "1F405"},
    {"harimau_bintang", "1F406"},
    {"arnab", "1F407"},
    {"kucing", "1F408"},
    {"naga", "1F409"},
    {"buaya", "1F40A"},
    {"ikan_paus", "1F40B"},
    {"siput", "1F40C"},
    {"ular", "1F40D"},
    {"kuda", "1F40E"},
    {"biri_biri_jantan", "1F40F"},
    {"kambing", "1F410"},
    {"biri_biri_betina", "1F411"},
    {"monyet", "1F412"},
    {"ayam_jantan", "1F413"},
    {"ayam", "1F414"},
    {"anjing", "1F415"},
    {"khinzir", "1F416"},
    {"babi_hutan", "1F417"},
    {"gajah", "1F418"},
    {"sotong_kurita", "1F419"},
    {"kulit_kerang_berpilin", "1F41A"},
    {"ulat", "1F41B"},
    {"semut", "1F41C"},
    {"lebah_madu", "1F41D"},
    {"kumbang_kura_kura", "1F41E"},
    {"ikan", "1F41F"},
    {"ikan_tropika", "1F420"},
    {"ikan_buntal", "1F421"},
    {"penyu", "1F422"},
    {"anak_ayam_menetas", "1F423"},
    {"anak_ayam", "1F424"},
    {"anak_ayam_menghadap_depan", "1F425"},
    {"burung", "1F426"},
    {"penguin", "1F427"},
    {"koala", "1F428"},
    {"anjing_poodle", "1F429"},
    {"unta", "1F42A"},
    {"unta_bonggol_dua", "1F42B"},
    {"ikan_lumba_lumba", "1F42C"},
    {"muka_tikus", "1F42D"},
    {"muka_lembu", "1F42E"},
    {"muka_harimau", "1F42F"},
    {"muka_arnab", "1F430"},
    {"muka_kucing", "1F431"},
    {"muka_naga", "1F432"},
    {"ikan_paus_memancutkan_air", "1F433"},
    {"muka_kuda", "1F434"},
    {"muka_monyet", "1F435"},
    {"muka_anjing", "1F436"},
    {"muka_khinzir", "1F437"},
    {"katak", "1F438"},
    {"hamster", "1F439"},
    {"serigala", "1F43A"},
    {"beruang", "1F43B"},
    {"panda", "1F43C"},
    {"hidung_khinzir", "1F43D"},
    {"tapak_kaki_haiwan", "1F43E"},
    {"tupai_tanah", "1F43F"},
    {"sepasang_mata", "1F440"},
    {"mata", "1F441"},
    {"telinga", "1F442"},
    {"telinga_warna1", "1F442-1F3FB"},
    {"telinga_warna2", "1F442-1F3FC"},
    {"telinga_warna3", "1F442-1F3FD"},
    {"telinga_warna4", "1F442-1F3FE"},
    {"telinga_warna5", "1F442-1F3FF"},
    {"hidung", "1F443"},
    {"hidung_warna1", "1F443-1F3FB"},
    {"hidung_warna2", "1F443-1F3FC"},
    {"hidung_warna3", "1F443-1F3FD"},
    {"hidung_warna4", "1F443-1F3FE"},
    {"hidung_warna5", "1F443-1F3FF"},
    {"mulut", "1F444"},
    {"lidah", "1F445"},
    {"jari_telunjuk_tunjuk_hala_ke_atas", "1F446"},
    {"jari_telunjuk_tunjuk_hala_ke_atas_warna1", "1F446-1F3FB"},
    {"jari_telunjuk_tunjuk_hala_ke_atas_warna2", "1F446-1F3FC"},
    {"jari_telunjuk_tunjuk_hala_ke_atas_warna3", "1F446-1F3FD"},
    {"jari_telunjuk_tunjuk_hala_ke_atas_warna4", "1F446-1F3FE"},
    {"jari_telunjuk_tunjuk_hala_ke_atas_warna5", "1F446-1F3FF"},
    {"jari_telunjuk_belakang_tangan_tuding_ke_bawah", "1F447"},
    {"jari_telunjuk_belakang_tangan_tuding_ke_bawah_warna1", "1F447-1F3FB"},
    {"jari_telunjuk_belakang_tangan_tuding_ke_bawah_warna2", "1F447-1F3FC"},
    {"jari_telunjuk_belakang_tangan_tuding_ke_bawah_warna3", "1F447-1F3FD"},
    {"jari_telunjuk_belakang_tangan_tuding_ke_bawah_warna4", "1F447-1F3FE"},
    {"jari_telunjuk_belakang_tangan_tuding_ke_bawah_warna5", "1F447-1F3FF"},
    {"jari_telunjuk_tunjuk_kiri", "1F448"},
    {"jari_telunjuk_tunjuk_kiri_warna1", "1F448-1F3FB"},
    {"jari_telunjuk_tunjuk_kiri_warna2", "1F448-1F3FC"},
    {"jari_telunjuk_tunjuk_kiri_warna3", "1F448-1F3FD"},
    {"jari_telunjuk_tunjuk_kiri_warna4", "1F448-1F3FE"},
    {"jari_telunjuk_tunjuk_kiri_warna5", "1F448-1F3FF"},
    {"jari_telunjuk_tunjuk_kanan", "1F449"},
    {"jari_telunjuk_tunjuk_kanan_warna1", "1F449-1F3FB"},
    {"jari_telunjuk_tunjuk_kanan_warna2", "1F449-1F3FC"},
    {"jari_telunjuk_tunjuk_kanan_warna3", "1F449-1F3FD"},
    {"jari_telunjuk_tunjuk_kanan_warna4", "1F449-1F3FE"},
    {"jari_telunjuk_tunjuk_kanan_warna5", "1F449-1F3FF"},
    {"penumbuk_dari_depan", "1F44A"},
    {"penumbuk_dari_depan_warna1", "1F44A-1F3FB"},
    {"penumbuk_dari_depan_warna2", "1F44A-1F3FC"},
    {"penumbuk_dari_depan_warna3", "1F44A-1F3FD"},
    {"penumbuk_dari_depan_warna4", "1F44A-1F3FE"},
    {"penumbuk_dari_depan_warna5", "1F44A-1F3FF"},
    {"lambai_tangan", "1F44B"},
    {"lambai_tangan_warna1", "1F44B-1F3FB"},
    {"lambai_tangan_warna2", "1F44B-1F3FC"},
    {"lambai_tangan_warna3", "1F44B-1F3FD"},
    {"lambai_tangan_warna4", "1F44B-1F3FE"},
    {"lambai_tangan_warna5", "1F44B-1F3FF"},
    {"simbol_ok", "1F44C"},
    {"simbol_ok_warna1", "1F44C-1F3FB"},
    {"simbol_ok_warna2", "1F44C-1F3FC"},
    {"simbol_ok_warna3", "1F44C-1F3FD"},
    {"simbol_ok_warna4", "1F44C-1F3FE"},
    {"simbol_ok_warna5", "1F44C-1F3FF"},
    {"simbol_bagus", "1F44D"},
    {"simbol_bagus_warna1", "1F44D-1F3FB"},
    {"simbol_bagus_warna2", "1F44D-1F3FC"},
    {"simbol_bagus_warna3", "1F44D-1F3FD"},
    {"simbol_bagus_warna4", "1F44D-1F3FE"},
    {"simbol_bagus_warna5", "1F44D-1F3FF"},
    {"simbol_tidak_bagus", "1F44E"},
    {"simbol_tidak_bagus_warna1", "1F44E-1F3FB"},
    {"simbol_tidak_bagus_warna2", "1F44E-1F3FC"},
    {"simbol_tidak_bagus_warna3", "1F44E-1F3FD"},
    {"simbol_tidak_bagus_warna4", "1F44E-1F3FE"},
    {"simbol_tidak_bagus_warna5", "1F44E-1F3FF"},
    {"bertepuk_tangan", "1F44F"},
    {"bertepuk_tangan_warna1", "1F44F-1F3FB"},
    {"bertepuk_tangan_warna2", "1F44F-1F3FC"},
    {"bertepuk_tangan_warna3", "1F44F-1F3FD"},
    {"bertepuk_tangan_warna4", "1F44F-1F3FE"},
    {"bertepuk_tangan_warna5", "1F44F-1F3FF"},
    {"tangan_terbuka", "1F450"},
    {"tangan_terbuka_warna1", "1F450-1F3FB"},
    {"tangan_terbuka_warna2", "1F450-1F3FC"},
    {"tangan_terbuka_warna3", "1F450-1F3FD"},
    {"tangan_terbuka_warna4", "1F450-1F3FE"},
    {"tangan_terbuka_warna5", "1F450-1F3FF"},
    {"mahkota", "1F451"},
    {"topi_wanita", "1F452"},
    {"cermin_mata", "1F453"},
    {"tali_leher", "1F454"},
    {"kemeja_t", "1F455"},
    {"seluar_jean", "1F456"},
    {"gaun", "1F457"},
    {"kimono", "1F458"},
    {"bikini", "1F459"},
    {"pakaian_wanita", "1F45A"},
    {"dompet", "1F45B"},
    {"beg_tangan", "1F45C"},
    {"pau", "1F45D"},
    {"kasut_lelaki", "1F45E"},
    {"kasut_lari", "1F45F"},
    {"kasut_tumit_tinggi", "1F460"},
    {"sandal_wanita", "1F461"},
    {"but_wanita", "1F462"},
    {"jejak_kaki", "1F463"},
    {"bayang_dada", "1F464"},
    {"bayang_beberapa_dada", "1F465"},
    {"kanak_kanak_lelaki", "1F466"},
    {"kanak_kanak_lelaki_warna1", "1F466-1F3FB"},
    {"kanak_kanak_lelaki_warna2", "1F466-1F3FC"},
    {"kanak_kanak_lelaki_warna3", "1F466-1F3FD"},
    {"kanak_kanak_lelaki_warna4", "1F466-1F3FE"},
    {"kanak_kanak_lelaki_warna5", "1F466-1F3FF"},
    {"budak_perempuan", "1F467"},
    {"budak_perempuan_warna1", "1F467-1F3FB"},
    {"budak_perempuan_warna2", "1F467-1F3FC"},
    {"budak_perempuan_warna3", "1F467-1F3FD"},
    {"budak_perempuan_warna4", "1F467-1F3FE"},
    {"budak_perempuan_warna5", "1F467-1F3FF"},
    {"lelaki", "1F468"},
    {"lelaki_warna1", "1F468-1F3FB"},
    {"lelaki_warna2", "1F468-1F3FC"},
    {"lelaki_warna3", "1F468-1F3FD"},
    {"lelaki_warna4", "1F468-1F3FE"},
    {"lelaki_warna5", "1F468-1F3FF"},
    {"perempuan", "1F469"},
    {"perempuan_warna1", "1F469-1F3FB"},
    {"perempuan_warna2", "1F469-1F3FC"},
    {"perempuan_warna3", "1F469-1F3FD"},
    {"perempuan_warna4", "1F469-1F3FE"},
    {"perempuan_warna5", "1F469-1F3FF"},
    {"keluarga", "1F46A"},
    {"lelaki_dan_wanita_berpegangan_tangan", "1F46B"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna1", "1F46B-1F3FB"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna2", "1F46B-1F3FC"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna3", "1F46B-1F3FD"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna4", "1F46B-1F3FE"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna5", "1F46B-1F3FF"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"lelaki_dan_wanita_berpegangan_tangan_warna5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"dua_lelaki_berpegangan_tangan", "1F46C"},
    {"dua_lelaki_berpegangan_tangan_warna1", "1F46C-1F3FB"},
    {"dua_lelaki_berpegangan_tangan_warna2", "1F46C-1F3FC"},
    {"dua_lelaki_berpegangan_tangan_warna3", "1F46C-1F3FD"},
    {"dua_lelaki_berpegangan_tangan_warna4", "1F46C-1F3FE"},
    {"dua_lelaki_berpegangan_tangan_warna5", "1F46C-1F3FF"},
    {"dua_lelaki_berpegangan_tangan_warna1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"dua_lelaki_berpegangan_tangan_warna1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"dua_lelaki_berpegangan_tangan_warna1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"dua_lelaki_berpegangan_tangan_warna1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"dua_lelaki_berpegangan_tangan_warna2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"dua_lelaki_berpegangan_tangan_warna2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"dua_lelaki_berpegangan_tangan_warna2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"dua_lelaki_berpegangan_tangan_warna2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"dua_lelaki_berpegangan_tangan_warna3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"dua_lelaki_berpegangan_tangan_warna3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"dua_lelaki_berpegangan_tangan_warna3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"dua_lelaki_berpegangan_tangan_warna3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"dua_lelaki_berpegangan_tangan_warna4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"dua_lelaki_berpegangan_tangan_warna4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"dua_lelaki_berpegangan_tangan_warna4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"dua_lelaki_berpegangan_tangan_warna4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"dua_lelaki_berpegangan_tangan_warna5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"dua_lelaki_berpegangan_tangan_warna5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"dua_lelaki_berpegangan_tangan_warna5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"dua_lelaki_berpegangan_tangan_warna5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"dua_wanita_berpegangan_tangan", "1F46D"},
    {"dua_wanita_berpegangan_tangan_warna1", "1F46D-1F3FB"},
    {"dua_wanita_berpegangan_tangan_warna2", "1F46D-1F3FC"},
    {"dua_wanita_berpegangan_tangan_warna3", "1F46D-1F3FD"},
    {"dua_wanita_berpegangan_tangan_warna4", "1F46D-1F3FE"},
    {"dua_wanita_berpegangan_tangan_warna5", "1F46D-1F3FF"},
    {"dua_wanita_berpegangan_tangan_warna1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"dua_wanita_berpegangan_tangan_warna1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"dua_wanita_berpegangan_tangan_warna1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"dua_wanita_berpegangan_tangan_warna1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"dua_wanita_berpegangan_tangan_warna2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"dua_wanita_berpegangan_tangan_warna2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"dua_wanita_berpegangan_tangan_warna2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"dua_wanita_berpegangan_tangan_warna2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"dua_wanita_berpegangan_tangan_warna3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"dua_wanita_berpegangan_tangan_warna3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"dua_wanita_berpegangan_tangan_warna3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"dua_wanita_berpegangan_tangan_warna3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"dua_wanita_berpegangan_tangan_warna4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"dua_wanita_berpegangan_tangan_warna4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"dua_wanita_berpegangan_tangan_warna4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"dua_wanita_berpegangan_tangan_warna4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"dua_wanita_berpegangan_tangan_warna5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"dua_wanita_berpegangan_tangan_warna5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"dua_wanita_berpegangan_tangan_warna5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"dua_wanita_berpegangan_tangan_warna5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"pegawai_polis", "1F46E"},
    {"pegawai_polis_warna1", "1F46E-1F3FB"},
    {"pegawai_polis_warna2", "1F46E-1F3FC"},
    {"pegawai_polis_warna3", "1F46E-1F3FD"},
    {"pegawai_polis_warna4", "1F46E-1F3FE"},
    {"pegawai_polis_warna5", "1F46E-1F3FF"},
    {"orang_dengan_telinga_arnab", "1F46F"},
    {"orang_dengan_telinga_arnab_warna1", "1F46F-1F3FB"},
    {"orang_dengan_telinga_arnab_warna2", "1F46F-1F3FC"},
    {"orang_dengan_telinga_arnab_warna3", "1F46F-1F3FD"},
    {"orang_dengan_telinga_arnab_warna4", "1F46F-1F3FE"},
    {"orang_dengan_telinga_arnab_warna5", "1F46F-1F3FF"},
    {"orang_dengan_telinga_arnab_warna1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"orang_dengan_telinga_arnab_warna1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"orang_dengan_telinga_arnab_warna1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"orang_dengan_telinga_arnab_warna1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"orang_dengan_telinga_arnab_warna2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"orang_dengan_telinga_arnab_warna2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"orang_dengan_telinga_arnab_warna2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"orang_dengan_telinga_arnab_warna2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"orang_dengan_telinga_arnab_warna3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"orang_dengan_telinga_arnab_warna3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"orang_dengan_telinga_arnab_warna3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"orang_dengan_telinga_arnab_warna3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"orang_dengan_telinga_arnab_warna4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"orang_dengan_telinga_arnab_warna4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"orang_dengan_telinga_arnab_warna4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"orang_dengan_telinga_arnab_warna4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"orang_dengan_telinga_arnab_warna5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"orang_dengan_telinga_arnab_warna5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"orang_dengan_telinga_arnab_warna5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"orang_dengan_telinga_arnab_warna5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"orang_memakai_vel", "1F470"},
    {"orang_memakai_vel_warna1", "1F470-1F3FB"},
    {"orang_memakai_vel_warna2", "1F470-1F3FC"},
    {"orang_memakai_vel_warna3", "1F470-1F3FD"},
    {"orang_memakai_vel_warna4", "1F470-1F3FE"},
    {"orang_memakai_vel_warna5", "1F470-1F3FF"},
    {"orang_rambut_perang", "1F471"},
    {"orang_rambut_perang_warna1", "1F471-1F3FB"},
    {"orang_rambut_perang_warna2", "1F471-1F3FC"},
    {"orang_rambut_perang_warna3", "1F471-1F3FD"},
    {"orang_rambut_perang_warna4", "1F471-1F3FE"},
    {"orang_rambut_perang_warna5", "1F471-1F3FF"},
    {"lelaki_bertopi_cina", "1F472"},
    {"lelaki_bertopi_cina_warna1", "1F472-1F3FB"},
    {"lelaki_bertopi_cina_warna2", "1F472-1F3FC"},
    {"lelaki_bertopi_cina_warna3", "1F472-1F3FD"},
    {"lelaki_bertopi_cina_warna4", "1F472-1F3FE"},
    {"lelaki_bertopi_cina_warna5", "1F472-1F3FF"},
    {"orang_berserban", "1F473"},
    {"orang_berserban_warna1", "1F473-1F3FB"},
    {"orang_berserban_warna2", "1F473-1F3FC"},
    {"orang_berserban_warna3", "1F473-1F3FD"},
    {"orang_berserban_warna4", "1F473-1F3FE"},
    {"orang_berserban_warna5", "1F473-1F3FF"},
    {"lelaki_tua", "1F474"},
    {"lelaki_tua_warna1", "1F474-1F3FB"},
    {"lelaki_tua_warna2", "1F474-1F3FC"},
    {"lelaki_tua_warna3", "1F474-1F3FD"},
    {"lelaki_tua_warna4", "1F474-1F3FE"},
    {"lelaki_tua_warna5", "1F474-1F3FF"},
    {"wanita_tua", "1F475"},
    {"wanita_tua_warna1", "1F475-1F3FB"},
    {"wanita_tua_warna2", "1F475-1F3FC"},
    {"wanita_tua_warna3", "1F475-1F3FD"},
    {"wanita_tua_warna4", "1F475-1F3FE"},
    {"wanita_tua_warna5", "1F475-1F3FF"},
    {"bayi", "1F476"},
    {"bayi_warna1", "1F476-1F3FB"},
    {"bayi_warna2", "1F476-1F3FC"},
    {"bayi_warna3", "1F476-1F3FD"},
    {"bayi_warna4", "1F476-1F3FE"},
    {"bayi_warna5", "1F476-1F3FF"},
    {"pekerja_pembinaan", "1F477"},
    {"pekerja_pembinaan_warna1", "1F477-1F3FB"},
    {"pekerja_pembinaan_warna2", "1F477-1F3FC"},
    {"pekerja_pembinaan_warna3", "1F477-1F3FD"},
    {"pekerja_pembinaan_warna4", "1F477-1F3FE"},
    {"pekerja_pembinaan_warna5", "1F477-1F3FF"},
    {"puteri", "1F478"},
    {"puteri_warna1", "1F478-1F3FB"},
    {"puteri_warna2", "1F478-1F3FC"},
    {"puteri_warna3", "1F478-1F3FD"},
    {"puteri_warna4", "1F478-1F3FE"},
    {"puteri_warna5", "1F478-1F3FF"},
    {"gergasi", "1F479"},
    {"goblin", "1F47A"},
    {"hantu", "1F47B"},
    {"pari_pari_bayi", "1F47C"},
    {"pari_pari_bayi_warna1", "1F47C-1F3FB"},
    {"pari_pari_bayi_warna2", "1F47C-1F3FC"},
    {"pari_pari_bayi_warna3", "1F47C-1F3FD"},
    {"pari_pari_bayi_warna4", "1F47C-1F3FE"},
    {"pari_pari_bayi_warna5", "1F47C-1F3FF"},
    {"makhluk_asing", "1F47D"},
    {"raksasa_angkasa_lepas", "1F47E"},
    {"muka_marah_bertanduk", "1F47F"},
    {"tengkorak", "1F480"},
    {"pegawai_kaunter_pertanyaan", "1F481"},
    {"pegawai_kaunter_pertanyaan_warna1", "1F481-1F3FB"},
    {"pegawai_kaunter_pertanyaan_warna2", "1F481-1F3FC"},
    {"pegawai_kaunter_pertanyaan_warna3", "1F481-1F3FD"},
    {"pegawai_kaunter_pertanyaan_warna4", "1F481-1F3FE"},
    {"pegawai_kaunter_pertanyaan_warna5", "1F481-1F3FF"},
    {"pengawal", "1F482"},
    {"pengawal_warna1", "1F482-1F3FB"},
    {"pengawal_warna2", "1F482-1F3FC"},
    {"pengawal_warna3", "1F482-1F3FD"},
    {"pengawal_warna4", "1F482-1F3FE"},
    {"pengawal_warna5", "1F482-1F3FF"},
    {"perempuan_menari", "1F483"},
    {"perempuan_menari_warna1", "1F483-1F3FB"},
    {"perempuan_menari_warna2", "1F483-1F3FC"},
    {"perempuan_menari_warna3", "1F483-1F3FD"},
    {"perempuan_menari_warna4", "1F483-1F3FE"},
    {"perempuan_menari_warna5", "1F483-1F3FF"},
    {"gincu", "1F484"},
    {"pengilat_kuku", "1F485"},
    {"pengilat_kuku_warna1", "1F485-1F3FB"},
    {"pengilat_kuku_warna2", "1F485-1F3FC"},
    {"pengilat_kuku_warna3", "1F485-1F3FD"},
    {"pengilat_kuku_warna4", "1F485-1F3FE"},
    {"pengilat_kuku_warna5", "1F485-1F3FF"},
    {"urut_muka", "1F486"},
    {"urut_muka_warna1", "1F486-1F3FB"},
    {"urut_muka_warna2", "1F486-1F3FC"},
    {"urut_muka_warna3", "1F486-1F3FD"},
    {"urut_muka_warna4", "1F486-1F3FE"},
    {"urut_muka_warna5", "1F486-1F3FF"},
    {"gunting_rambut", "1F487"},
    {"gunting_rambut_warna1", "1F487-1F3FB"},
    {"gunting_rambut_warna2", "1F487-1F3FC"},
    {"gunting_rambut_warna3", "1F487-1F3FD"},
    {"gunting_rambut_warna4", "1F487-1F3FE"},
    {"gunting_rambut_warna5", "1F487-1F3FF"},
    {"jalur_kedai_gunting_rambut", "1F488"},
    {"picagari", "1F489"},
    {"pil", "1F48A"},
    {"tanda_ciuman", "1F48B"},
    {"surat_cinta", "1F48C"},
    {"cincin", "1F48D"},
    {"batu_permata", "1F48E"},
    {"cium", "1F48F"},
    {"cium_warna1", "1F48F-1F3FB"},
    {"cium_warna2", "1F48F-1F3FC"},
    {"cium_warna3", "1F48F-1F3FD"},
    {"cium_warna4", "1F48F-1F3FE"},
    {"cium_warna5", "1F48F-1F3FF"},
    {"cium_warna1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"cium_warna1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"cium_warna1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"cium_warna1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"cium_warna2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"cium_warna2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"cium_warna2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"cium_warna2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"cium_warna3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"cium_warna3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"cium_warna3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"cium_warna3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"cium_warna4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"cium_warna4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"cium_warna4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"cium_warna4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"cium_warna5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"cium_warna5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"cium_warna5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"cium_warna5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"sejambak_bunga", "1F490"},
    {"pasangan_dengan_simbol_hati", "1F491"},
    {"pasangan_dengan_simbol_hati_warna1", "1F491-1F3FB"},
    {"pasangan_dengan_simbol_hati_warna2", "1F491-1F3FC"},
    {"pasangan_dengan_simbol_hati_warna3", "1F491-1F3FD"},
    {"pasangan_dengan_simbol_hati_warna4", "1F491-1F3FE"},
    {"pasangan_dengan_simbol_hati_warna5", "1F491-1F3FF"},
    {"pasangan_dengan_simbol_hati_warna1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pasangan_dengan_simbol_hati_warna1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pasangan_dengan_simbol_hati_warna1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pasangan_dengan_simbol_hati_warna1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pasangan_dengan_simbol_hati_warna2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pasangan_dengan_simbol_hati_warna2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pasangan_dengan_simbol_hati_warna2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pasangan_dengan_simbol_hati_warna2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pasangan_dengan_simbol_hati_warna3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pasangan_dengan_simbol_hati_warna3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pasangan_dengan_simbol_hati_warna3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pasangan_dengan_simbol_hati_warna3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pasangan_dengan_simbol_hati_warna4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pasangan_dengan_simbol_hati_warna4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pasangan_dengan_simbol_hati_warna4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pasangan_dengan_simbol_hati_warna4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pasangan_dengan_simbol_hati_warna5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pasangan_dengan_simbol_hati_warna5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pasangan_dengan_simbol_hati_warna5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pasangan_dengan_simbol_hati_warna5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"perkahwinan", "1F492"},
    {"jantung_berdegup", "1F493"},
    {"patah_hati", "1F494"},
    {"dua_hati", "1F495"},
    {"hati_berkilauan", "1F496"},
    {"hati_membesar", "1F497"},
    {"hati_ditembusi_anak_panah", "1F498"},
    {"hati_biru", "1F499"},
    {"hati_hijau", "1F49A"},
    {"hati_kuning", "1F49B"},
    {"hati_ungu", "1F49C"},
    {"hati_dengan_reben", "1F49D"},
    {"hati_berputar", "1F49E"},
    {"hiasan_hati", "1F49F"},
    {"berlian_berbintik", "1F4A0"},
    {"mentol_lampu", "1F4A1"},
    {"simbol_marah", "1F4A2"},
    {"bom", "1F4A3"},
    {"zzz", "1F4A4"},
    {"pelanggaran", "1F4A5"},
    {"titis_peluh", "1F4A6"},
    {"titisan_air", "1F4A7"},
    {"meluru", "1F4A8"},
    {"selonggok_najis", "1F4A9"},
    {"biseps_dilenturtegang", "1F4AA"},
    {"biseps_dilenturtegang_warna1", "1F4AA-1F3FB"},
    {"biseps_dilenturtegang_warna2", "1F4AA-1F3FC"},
    {"biseps_dilenturtegang_warna3", "1F4AA-1F3FD"},
    {"biseps_dilenturtegang_warna4", "1F4AA-1F3FE"},
    {"biseps_dilenturtegang_warna5", "1F4AA-1F3FF"},
    {"pening", "1F4AB"},
    {"gelembung_pertuturan", "1F4AC"},
    {"gelembung_fikir", "1F4AD"},
    {"bunga_putih", "1F4AE"},
    {"seratus_mata", "1F4AF"},
    {"karung_duit", "1F4B0"},
    {"pertukaran_mata_wang", "1F4B1"},
    {"tanda_dolar_tebal", "1F4B2"},
    {"kad_kredit", "1F4B3"},
    {"wang_kertas_yen", "1F4B4"},
    {"wang_kertas_dolar", "1F4B5"},
    {"wang_kertas_euro", "1F4B6"},
    {"wang_kertas_paun", "1F4B7"},
    {"wang_kertas_bersayap", "1F4B8"},
    {"carta_yen_meningkat", "1F4B9"},
    {"tempat_duduk", "1F4BA"},
    {"komputer_riba", "1F4BB"},
    {"beg_bimbit", "1F4BC"},
    {"cakera_komputer", "1F4BD"},
    {"cakera_liut", "1F4BE"},
    {"cakera_optik", "1F4BF"},
    {"dvd", "1F4C0"},
    {"folder_fail", "1F4C1"},
    {"folder_buka_fail", "1F4C2"},
    {"halaman_ikal", "1F4C3"},
    {"halaman_menghadap_atas", "1F4C4"},
    {"kalendar", "1F4C5"},
    {"kalendar_koyak", "1F4C6"},
    {"indeks_kad", "1F4C7"},
    {"carta_menaik", "1F4C8"},
    {"carta_menurun", "1F4C9"},
    {"carta_bar", "1F4CA"},
    {"papan_klip", "1F4CB"},
    {"paku_tekan", "1F4CC"},
    {"paku_tekan_bulat", "1F4CD"},
    {"klip_kertas", "1F4CE"},
    {"pembaris_lurus", "1F4CF"},
    {"pembaris_segi_tiga", "1F4D0"},
    {"tab_penanda_halaman", "1F4D1"},
    {"lejar", "1F4D2"},
    {"buku_nota", "1F4D3"},
    {"buku_nota_dengan_kulit_berhias", "1F4D4"},
    {"buku_tertutup", "1F4D5"},
    {"buka_terbuka", "1F4D6"},
    {"buku_hijau", "1F4D7"},
    {"buku_biru", "1F4D8"},
    {"buku_oren", "1F4D9"},
    {"buku", "1F4DA"},
    {"lencana_nama", "1F4DB"},
    {"skrol", "1F4DC"},
    {"memo", "1F4DD"},
    {"gagang_telefon", "1F4DE"},
    {"alat_kelui", "1F4DF"},
    {"mesin_faksimili", "1F4E0"},
    {"antena_satelit", "1F4E1"},
    {"corong_pembesar_suara", "1F4E2"},
    {"megafon", "1F4E3"},
    {"dulang_keluar", "1F4E4"},
    {"dulang_masuk", "1F4E5"},
    {"pakej", "1F4E6"},
    {"e_mel", "1F4E7"},
    {"sampul_surat_masuk", "1F4E8"},
    {"sampul_surat_dengan_anak_panah", "1F4E9"},
    {"peti_surat_tertutup_dengan_bendera_diturunkan", "1F4EA"},
    {"peti_surat_tertutup_dengan_bendera_dinaikkan", "1F4EB"},
    {"peti_surat_terbuka_dengan_bendera_dinaikkan", "1F4EC"},
    {"peti_surat_terbuka_dengan_bendera_diturunkan", "1F4ED"},
    {"peti_pos", "1F4EE"},
    {"hon_pos", "1F4EF"},
    {"akhbar", "1F4F0"},
    {"telefon_bimbit", "1F4F1"},
    {"telefon_bimbit_dengan_anak_panah", "1F4F2"},
    {"mod_getaran", "1F4F3"},
    {"telefon_mudah_alih_dimatikan", "1F4F4"},
    {"dilarang_guna_telefon_bimbit", "1F4F5"},
    {"palang_antena", "1F4F6"},
    {"kamera", "1F4F7"},
    {"kamera_dengan_denyar", "1F4F8"},
    {"kamera_video", "1F4F9"},
    {"televisyen", "1F4FA"},
    {"radio", "1F4FB"},
    {"pita_video", "1F4FC"},
    {"projektor_filem", "1F4FD"},
    {"tasbih", "1F4FF"},
    {"butang_rombak_trek", "1F500"},
    {"butang_ulang", "1F501"},
    {"butang_ulang_sekali", "1F502"},
    {"anak_panah_menegak_ikut_jam", "1F503"},
    {"butang_anak_panah_lawan_jam", "1F504"},
    {"butang_malap", "1F505"},
    {"butang_cerah", "1F506"},
    {"pembesar_suara_dimatikan", "1F507"},
    {"kelantangan_rendah_pembesar_suara", "1F508"},
    {"kelantangan_sederhana_pembesar_suara", "1F509"},
    {"kelantangan_tinggi_pembesar_suara", "1F50A"},
    {"bateri", "1F50B"},
    {"palam_elektrik", "1F50C"},
    {"kanta_pembesar_condong_ke_kiri", "1F50D"},
    {"kanta_pembesar_condong_ke_kanan", "1F50E"},
    {"mangga_dengan_pen", "1F50F"},
    {"mangga_terkunci_dengan_anak_kunci", "1F510"},
    {"kunci", "1F511"},
    {"bermangga", "1F512"},
    {"mangga_terbuka", "1F513"},
    {"loceng", "1F514"},
    {"loceng_berpalang", "1F515"},
    {"penanda_buku", "1F516"},
    {"pautan", "1F517"},
    {"butang_radio", "1F518"},
    {"anak_panah_back", "1F519"},
    {"anak_panah_end", "1F51A"},
    {"anak_panah_on", "1F51B"},
    {"anak_panah_soon", "1F51C"},
    {"anak_panah_top", "1F51D"},
    {"bawah_lapan_belas_tidak_dibenarkan", "1F51E"},
    {"butang_kekunci_10", "1F51F"},
    {"huruf_besar_latin_input", "1F520"},
    {"huruf_kecil_latin_input", "1F521"},
    {"nombor_input", "1F522"},
    {"simbol_input", "1F523"},
    {"huruf_latin_input", "1F524"},
    {"api", "1F525"},
    {"lampu_suluh", "1F526"},
    {"perengkuh", "1F527"},
    {"tukul", "1F528"},
    {"nat_dan_bolt", "1F529"},
    {"pisau_dapur", "1F52A"},
    {"pistol_air", "1F52B"},
    {"mikroskop", "1F52C"},
    {"teleskop", "1F52D"},
    {"bola_kristal", "1F52E"},
    {"bintang_berpenjuru_enam_berbintik", "1F52F"},
    {"simbol_jepun_untuk_pemula", "1F530"},
    {"lambang_trisula", "1F531"},
    {"butang_persegi_hitam", "1F532"},
    {"butang_persegi_putih", "1F533"},
    {"bulatan_merah", "1F534"},
    {"bulatan_biru", "1F535"},
    {"berlian_jingga_besar", "1F536"},
    {"berlian_biru_besar", "1F537"},
    {"berlian_jingga_kecil", "1F538"},
    {"berlian_biru_kecil", "1F539"},
    {"segi_tiga_merah_menunjuk_ke_atas", "1F53A"},
    {"segi_tiga_merah_menunjuk_ke_bawah", "1F53B"},
    {"butang_ke_atas", "1F53C"},
    {"butang_ke_bawah", "1F53D"},
    {"om", "1F549"},
    {"burung_merpati", "1F54A"},
    {"kaabah", "1F54B"},
    {"masjid", "1F54C"},
    {"saumaah", "1F54D"},
    {"menorah", "1F54E"},
    {"pukul_satu", "1F550"},
    {"pukul_dua", "1F551"},
    {"pukul_3", "1F552"},
    {"pukul_empat", "1F553"},
    {"pukul_lima", "1F554"},
    {"pukul_enam", "1F555"},
    {"pukul_tujuh", "1F556"},
    {"pukul_lapan", "1F557"},
    {"pukul_sembilan", "1F558"},
    {"pukul_sepuluh", "1F559"},
    {"pukul_sebelas", "1F55A"},
    {"pukul_dua_belas", "1F55B"},
    {"satu_setengah", "1F55C"},
    {"dua_setengah", "1F55D"},
    {"tiga_setengah", "1F55E"},
    {"empat_setengah", "1F55F"},
    {"lima_setengah", "1F560"},
    {"enam_setengah", "1F561"},
    {"tujuh_setengah", "1F562"},
    {"lapan_setengah", "1F563"},
    {"sembilan_setengah", "1F564"},
    {"sepuluh_setengah", "1F565"},
    {"sebelas_setengah", "1F566"},
    {"dua_belas_setengah", "1F567"},
    {"lilin", "1F56F"},
    {"jam_para_pendiang", "1F570"},
    {"lubang", "1F573"},
    {"lelaki_memakai_sut_urusan_terapung_apung", "1F574"},
    {"lelaki_memakai_sut_urusan_terapung_apung_warna1", "1F574-1F3FB"},
    {"lelaki_memakai_sut_urusan_terapung_apung_warna2", "1F574-1F3FC"},
    {"lelaki_memakai_sut_urusan_terapung_apung_warna3", "1F574-1F3FD"},
    {"lelaki_memakai_sut_urusan_terapung_apung_warna4", "1F574-1F3FE"},
    {"lelaki_memakai_sut_urusan_terapung_apung_warna5", "1F574-1F3FF"},
    {"detektif", "1F575"},
    {"detektif_warna1", "1F575-1F3FB"},
    {"detektif_warna2", "1F575-1F3FC"},
    {"detektif_warna3", "1F575-1F3FD"},
    {"detektif_warna4", "1F575-1F3FE"},
    {"detektif_warna5", "1F575-1F3FF"},
    {"kaca_mata_hitam", "1F576"},
    {"labah_labah", "1F577"},
    {"sarang_labah_labah", "1F578"},
    {"kayu_bedik", "1F579"},
    {"lelaki_menari", "1F57A"},
    {"lelaki_menari_warna1", "1F57A-1F3FB"},
    {"lelaki_menari_warna2", "1F57A-1F3FC"},
    {"lelaki_menari_warna3", "1F57A-1F3FD"},
    {"lelaki_menari_warna4", "1F57A-1F3FE"},
    {"lelaki_menari_warna5", "1F57A-1F3FF"},
    {"klip_kertas_berpaut", "1F587"},
    {"pen", "1F58A"},
    {"pena", "1F58B"},
    {"berus_cat", "1F58C"},
    {"krayon", "1F58D"},
    {"tangan_dengan_jari_terkepak", "1F590"},
    {"tangan_dengan_jari_terkepak_warna1", "1F590-1F3FB"},
    {"tangan_dengan_jari_terkepak_warna2", "1F590-1F3FC"},
    {"tangan_dengan_jari_terkepak_warna3", "1F590-1F3FD"},
    {"tangan_dengan_jari_terkepak_warna4", "1F590-1F3FE"},
    {"tangan_dengan_jari_terkepak_warna5", "1F590-1F3FF"},
    {"jari_hantu", "1F595"},
    {"jari_hantu_warna1", "1F595-1F3FB"},
    {"jari_hantu_warna2", "1F595-1F3FC"},
    {"jari_hantu_warna3", "1F595-1F3FD"},
    {"jari_hantu_warna4", "1F595-1F3FE"},
    {"jari_hantu_warna5", "1F595-1F3FF"},
    {"tabik_vulcan", "1F596"},
    {"tabik_vulcan_warna1", "1F596-1F3FB"},
    {"tabik_vulcan_warna2", "1F596-1F3FC"},
    {"tabik_vulcan_warna3", "1F596-1F3FD"},
    {"tabik_vulcan_warna4", "1F596-1F3FE"},
    {"tabik_vulcan_warna5", "1F596-1F3FF"},
    {"hati_hitam", "1F5A4"},
    {"komputer_meja", "1F5A5"},
    {"pencetak", "1F5A8"},
    {"tetikus_komputer", "1F5B1"},
    {"bebola_jejak", "1F5B2"},
    {"bingkai_dengan_gambar", "1F5BC"},
    {"pembahagi_indeks_kad", "1F5C2"},
    {"kotak_fail_kad", "1F5C3"},
    {"kabinet_fail", "1F5C4"},
    {"tong_sampah", "1F5D1"},
    {"pad_nota_pilin", "1F5D2"},
    {"kalendar_pilin", "1F5D3"},
    {"ragum", "1F5DC"},
    {"kunci_lama", "1F5DD"},
    {"akhbar_bergulung", "1F5DE"},
    {"pisau_belati", "1F5E1"},
    {"kepala_bercakap", "1F5E3"},
    {"gelembung_pertuturan_kiri", "1F5E8"},
    {"gelembung_marah_kanan", "1F5EF"},
    {"peti_undi_dengan_undian", "1F5F3"},
    {"peta_dunia", "1F5FA"},
    {"gunung_fuji", "1F5FB"},
    {"menara_tokyo", "1F5FC"},
    {"patung_liberty", "1F5FD"},
    {"peta_jepun", "1F5FE"},
    {"moyai", "1F5FF"},
    {"muka_tersengih", "1F600"},
    {"muka_berseri_seri_dengan_mata_tersenyum", "1F601"},
    {"muka_dengan_air_mata_kegembiraan", "1F602"},
    {"muka_tersengih_dengan_mata_besar", "1F603"},
    {"muka_tersengih_dan_mata_tersenyum", "1F604"},
    {"muka_tersengih_dan_berpeluh", "1F605"},
    {"muka_tersengih_mata_tertutup_rapat", "1F606"},
    {"muka_tersenyum_dengan_halo", "1F607"},
    {"muka_senyum_bertanduk", "1F608"},
    {"muka_kenyit_mata", "1F609"},
    {"muka_tersenyum_dengan_mata_tersenyum", "1F60A"},
    {"muka_menikmati_makanan_lazat", "1F60B"},
    {"muka_lega", "1F60C"},
    {"muka_tersenyum_dengan_mata_berbentuk_hati", "1F60D"},
    {"muka_tersenyum_berkaca_mata_hitam", "1F60E"},
    {"muka_tersenyum_sinis", "1F60F"},
    {"muka_neutral", "1F610"},
    {"muka_selamba", "1F611"},
    {"muka_tidak_puas_hati", "1F612"},
    {"muka_berpeluh_sejuk", "1F613"},
    {"muka_bertafakur", "1F614"},
    {"muka_keliru", "1F615"},
    {"muka_bingung", "1F616"},
    {"muka_memberi_ciuman", "1F617"},
    {"muka_melayangkan_ciuman", "1F618"},
    {"muka_memberi_ciuman_dengan_mata_tersenyum", "1F619"},
    {"muka_memberi_ciuman_dengan_mata_tertutup", "1F61A"},
    {"muka_menjelir_lidah", "1F61B"},
    {"muka_menjelir_lidah_dan_mengenyit_mata", "1F61C"},
    {"muka_menjelir_lidah_dan_mata_tertutup_rapat", "1F61D"},
    {"muka_kecewa", "1F61E"},
    {"muka_risau", "1F61F"},
    {"muka_marah", "1F620"},
    {"muka_mencebik", "1F621"},
    {"muka_menangis", "1F622"},
    {"muka_tekun", "1F623"},
    {"muka_bangga_kemenangan", "1F624"},
    {"muka_kecewa_namun_lega", "1F625"},
    {"muka_berkerut_dengan_mulut_ternganga", "1F626"},
    {"muka_merana", "1F627"},
    {"muka_ketakutan", "1F628"},
    {"muka_tak_bermaya", "1F629"},
    {"muka_mengantuk", "1F62A"},
    {"muka_letih", "1F62B"},
    {"muka_menyeringai", "1F62C"},
    {"muka_meraung", "1F62D"},
    {"muka_bermulut_ternganga", "1F62E"},
    {"muka_terkejut", "1F62F"},
    {"muka_gementar_dan_berpeluh_sejuk", "1F630"},
    {"muka_menjerit_ketakutan", "1F631"},
    {"muka_terperanjat", "1F632"},
    {"muka_malu", "1F633"},
    {"muka_tidur", "1F634"},
    {"muka_pening", "1F635"},
    {"muka_tanpa_mulut", "1F636"},
    {"muka_memakai_topeng_perubatan", "1F637"},
    {"kucing_tersengih_dengan_mata_tersenyum", "1F638"},
    {"kucing_dengan_air_mata_kegembiraan", "1F639"},
    {"kucing_tersengih", "1F63A"},
    {"kucing_tersenyum_dengan_mata_berbentuk_hati", "1F63B"},
    {"muka_kucing_tersenyum_sinis", "1F63C"},
    {"kucing_melemparkan_ciuman", "1F63D"},
    {"muka_kucing_mulut_muncung", "1F63E"},
    {"muka_kucing_menangis", "1F63F"},
    {"muka_kucing_keletihan", "1F640"},
    {"muka_berkerut_sedikit", "1F641"},
    {"muka_tersenyum_sedikit", "1F642"},
    {"muka_terbalik", "1F643"},
    {"muka_dengan_mata_juling_ke_atas", "1F644"},
    {"gerak_isyarat_tidak", "1F645"},
    {"gerak_isyarat_tidak_warna1", "1F645-1F3FB"},
    {"gerak_isyarat_tidak_warna2", "1F645-1F3FC"},
    {"gerak_isyarat_tidak_warna3", "1F645-1F3FD"},
    {"gerak_isyarat_tidak_warna4", "1F645-1F3FE"},
    {"gerak_isyarat_tidak_warna5", "1F645-1F3FF"},
    {"gerak_isyarat_okey", "1F646"},
    {"gerak_isyarat_okey_warna1", "1F646-1F3FB"},
    {"gerak_isyarat_okey_warna2", "1F646-1F3FC"},
    {"gerak_isyarat_okey_warna3", "1F646-1F3FD"},
    {"gerak_isyarat_okey_warna4", "1F646-1F3FE"},
    {"gerak_isyarat_okey_warna5", "1F646-1F3FF"},
    {"orang_menunduk", "1F647"},
    {"orang_menunduk_warna1", "1F647-1F3FB"},
    {"orang_menunduk_warna2", "1F647-1F3FC"},
    {"orang_menunduk_warna3", "1F647-1F3FD"},
    {"orang_menunduk_warna4", "1F647-1F3FE"},
    {"orang_menunduk_warna5", "1F647-1F3FF"},
    {"monyet_tutup_mata", "1F648"},
    {"monyet_tutup_telinga", "1F649"},
    {"monyet_tutup_mulut", "1F64A"},
    {"orang_gembira_mengangkat_tangan", "1F64B"},
    {"orang_gembira_mengangkat_tangan_warna1", "1F64B-1F3FB"},
    {"orang_gembira_mengangkat_tangan_warna2", "1F64B-1F3FC"},
    {"orang_gembira_mengangkat_tangan_warna3", "1F64B-1F3FD"},
    {"orang_gembira_mengangkat_tangan_warna4", "1F64B-1F3FE"},
    {"orang_gembira_mengangkat_tangan_warna5", "1F64B-1F3FF"},
    {"orang_mengangkat_tangan", "1F64C"},
    {"orang_mengangkat_tangan_warna1", "1F64C-1F3FB"},
    {"orang_mengangkat_tangan_warna2", "1F64C-1F3FC"},
    {"orang_mengangkat_tangan_warna3", "1F64C-1F3FD"},
    {"orang_mengangkat_tangan_warna4", "1F64C-1F3FE"},
    {"orang_mengangkat_tangan_warna5", "1F64C-1F3FF"},
    {"orang_berkerut_dahi", "1F64D"},
    {"orang_berkerut_dahi_warna1", "1F64D-1F3FB"},
    {"orang_berkerut_dahi_warna2", "1F64D-1F3FC"},
    {"orang_berkerut_dahi_warna3", "1F64D-1F3FD"},
    {"orang_berkerut_dahi_warna4", "1F64D-1F3FE"},
    {"orang_berkerut_dahi_warna5", "1F64D-1F3FF"},
    {"orang_mencebik", "1F64E"},
    {"orang_mencebik_warna1", "1F64E-1F3FB"},
    {"orang_mencebik_warna2", "1F64E-1F3FC"},
    {"orang_mencebik_warna3", "1F64E-1F3FD"},
    {"orang_mencebik_warna4", "1F64E-1F3FE"},
    {"orang_mencebik_warna5", "1F64E-1F3FF"},
    {"menyusun_jari", "1F64F"},
    {"menyusun_jari_warna1", "1F64F-1F3FB"},
    {"menyusun_jari_warna2", "1F64F-1F3FC"},
    {"menyusun_jari_warna3", "1F64F-1F3FD"},
    {"menyusun_jari_warna4", "1F64F-1F3FE"},
    {"menyusun_jari_warna5", "1F64F-1F3FF"},
    {"roket", "1F680"},
    {"helikopter", "1F681"},
    {"lokomotif", "1F682"},
    {"gerabak_kereta_api", "1F683"},
    {"kereta_api_laju", "1F684"},
    {"kereta_api_laju_dengan_muncung_peluru", "1F685"},
    {"kereta_api", "1F686"},
    {"kereta_api_bawah_tanah", "1F687"},
    {"rel_ringan", "1F688"},
    {"stesen", "1F689"},
    {"trem", "1F68A"},
    {"gerabak_trem", "1F68B"},
    {"bas", "1F68C"},
    {"bas_dari_arah_depan", "1F68D"},
    {"bas_troli", "1F68E"},
    {"perhentian_bas", "1F68F"},
    {"bas_mini", "1F690"},
    {"ambulans", "1F691"},
    {"kereta_bomba", "1F692"},
    {"kereta_polis", "1F693"},
    {"kereta_polis_dari_arah_depan", "1F694"},
    {"teksi", "1F695"},
    {"teksi_dari_arah_depan", "1F696"},
    {"kereta", "1F697"},
    {"kereta_dari_arah_depan", "1F698"},
    {"kenderaan_rekreasi", "1F699"},
    {"trak_penghantaran", "1F69A"},
    {"lori_bersendi", "1F69B"},
    {"traktor", "1F69C"},
    {"monorel", "1F69D"},
    {"kereta_api_gunung", "1F69E"},
    {"kereta_api_landasan_tergantung", "1F69F"},
    {"jalan_kabel_gunung", "1F6A0"},
    {"jalan_trem_udara", "1F6A1"},
    {"kapal", "1F6A2"},
    {"orang_mendayung_sampan", "1F6A3"},
    {"orang_mendayung_sampan_warna1", "1F6A3-1F3FB"},
    {"orang_mendayung_sampan_warna2", "1F6A3-1F3FC"},
    {"orang_mendayung_sampan_warna3", "1F6A3-1F3FD"},
    {"orang_mendayung_sampan_warna4", "1F6A3-1F3FE"},
    {"orang_mendayung_sampan_warna5", "1F6A3-1F3FF"},
    {"bot_laju", "1F6A4"},
    {"lampu_isyarat_melintang", "1F6A5"},
    {"lampu_isyarat_menegak", "1F6A6"},
    {"pembinaan", "1F6A7"},
    {"lampu_kereta_polis", "1F6A8"},
    {"bendera_tiga_segi", "1F6A9"},
    {"pintu", "1F6AA"},
    {"dilarang", "1F6AB"},
    {"rokok", "1F6AC"},
    {"dilarang_merokok", "1F6AD"},
    {"tanda_buang_ke_dalam_tong_sampah", "1F6AE"},
    {"dilarang_membuang_sampah", "1F6AF"},
    {"air_minum", "1F6B0"},
    {"air_tak_boleh_minum", "1F6B1"},
    {"basikal", "1F6B2"},
    {"basikal_dilarang", "1F6B3"},
    {"menunggang_basikal", "1F6B4"},
    {"menunggang_basikal_warna1", "1F6B4-1F3FB"},
    {"menunggang_basikal_warna2", "1F6B4-1F3FC"},
    {"menunggang_basikal_warna3", "1F6B4-1F3FD"},
    {"menunggang_basikal_warna4", "1F6B4-1F3FE"},
    {"menunggang_basikal_warna5", "1F6B4-1F3FF"},
    {"penunggang_basikal_bukit", "1F6B5"},
    {"penunggang_basikal_bukit_warna1", "1F6B5-1F3FB"},
    {"penunggang_basikal_bukit_warna2", "1F6B5-1F3FC"},
    {"penunggang_basikal_bukit_warna3", "1F6B5-1F3FD"},
    {"penunggang_basikal_bukit_warna4", "1F6B5-1F3FE"},
    {"penunggang_basikal_bukit_warna5", "1F6B5-1F3FF"},
    {"pejalan_kaki", "1F6B6"},
    {"pejalan_kaki_warna1", "1F6B6-1F3FB"},
    {"pejalan_kaki_warna2", "1F6B6-1F3FC"},
    {"pejalan_kaki_warna3", "1F6B6-1F3FD"},
    {"pejalan_kaki_warna4", "1F6B6-1F3FE"},
    {"pejalan_kaki_warna5", "1F6B6-1F3FF"},
    {"pejalan_kaki_dilarang", "1F6B7"},
    {"kanak_kanak_melintas", "1F6B8"},
    {"tandas_lelaki", "1F6B9"},
    {"tandas_wanita", "1F6BA"},
    {"bilik_air", "1F6BB"},
    {"simbol_bayi", "1F6BC"},
    {"mangkuk_tandas", "1F6BD"},
    {"tandas_pam", "1F6BE"},
    {"pancuran", "1F6BF"},
    {"orang_sedang_mandi", "1F6C0"},
    {"orang_sedang_mandi_warna1", "1F6C0-1F3FB"},
    {"orang_sedang_mandi_warna2", "1F6C0-1F3FC"},
    {"orang_sedang_mandi_warna3", "1F6C0-1F3FD"},
    {"orang_sedang_mandi_warna4", "1F6C0-1F3FE"},
    {"orang_sedang_mandi_warna5", "1F6C0-1F3FF"},
    {"tab_mandi", "1F6C1"},
    {"kawalan_pasport", "1F6C2"},
    {"kastam", "1F6C3"},
    {"tuntutan_bagasi", "1F6C4"},
    {"bagasi_tertinggal", "1F6C5"},
    {"sofa_dan_lampu", "1F6CB"},
    {"orang_tidur_atas_katil", "1F6CC"},
    {"orang_tidur_atas_katil_warna1", "1F6CC-1F3FB"},
    {"orang_tidur_atas_katil_warna2", "1F6CC-1F3FC"},
    {"orang_tidur_atas_katil_warna3", "1F6CC-1F3FD"},
    {"orang_tidur_atas_katil_warna4", "1F6CC-1F3FE"},
    {"orang_tidur_atas_katil_warna5", "1F6CC-1F3FF"},
    {"beg_beli_belah", "1F6CD"},
    {"loceng_porter", "1F6CE"},
    {"katil", "1F6CF"},
    {"tempat_ibadat", "1F6D0"},
    {"tanda_berhenti", "1F6D1"},
    {"troli_beli_belah", "1F6D2"},
    {"kuil_hindu", "1F6D5"},
    {"pondok", "1F6D6"},
    {"elevator", "1F6D7"},
    {"tanah_runtuh", "1F6D8"},
    {"wayarles", "1F6DC"},
    {"papan_gelongsor_taman_permainan", "1F6DD"},
    {"roda", "1F6DE"},
    {"boya", "1F6DF"},
    {"tukul_dan_perengkuh", "1F6E0"},
    {"perisai", "1F6E1"},
    {"tong_minyak", "1F6E2"},
    {"jalan_raya", "1F6E3"},
    {"jalan_kereta_api", "1F6E4"},
    {"motobot", "1F6E5"},
    {"kapal_terbang_kecil", "1F6E9"},
    {"kapal_terbang_berlepas", "1F6EB"},
    {"kapal_terbang_mendarat", "1F6EC"},
    {"satelit", "1F6F0"},
    {"kapal_penumpang", "1F6F3"},
    {"skuter_tolak", "1F6F4"},
    {"skuter_bermotor", "1F6F5"},
    {"kanu", "1F6F6"},
    {"geluncur", "1F6F7"},
    {"piring_terbang", "1F6F8"},
    {"papan_luncur", "1F6F9"},
    {"lanca_auto", "1F6FA"},
    {"trak_pikap", "1F6FB"},
    {"kasut_roda", "1F6FC"},
    {"bulatan_jingga", "1F7E0"},
    {"bulatan_kuning", "1F7E1"},
    {"bulatan_hijau", "1F7E2"},
    {"bulatan_ungu", "1F7E3"},
    {"bulatan_perang", "1F7E4"},
    {"persegi_merah", "1F7E5"},
    {"persegi_biru", "1F7E6"},
    {"persegi_jingga", "1F7E7"},
    {"persegi_kuning", "1F7E8"},
    {"persegi_hijau", "1F7E9"},
    {"persegi_ungu", "1F7EA"},
    {"persegi_perang", "1F7EB"},
    {"tanda_sama_dengan_tebal", "1F7F0"},
    {"jari_mencubit", "1F90C"},
    {"jari_mencubit_warna1", "1F90C-1F3FB"},
    {"jari_mencubit_warna2", "1F90C-1F3FC"},
    {"jari_mencubit_warna3", "1F90C-1F3FD"},
    {"jari_mencubit_warna4", "1F90C-1F3FE"},
    {"jari_mencubit_warna5", "1F90C-1F3FF"},
    {"hati_putih", "1F90D"},
    {"hati_perang", "1F90E"},
    {"tangan_mencubit", "1F90F"},
    {"tangan_mencubit_warna1", "1F90F-1F3FB"},
    {"tangan_mencubit_warna2", "1F90F-1F3FC"},
    {"tangan_mencubit_warna3", "1F90F-1F3FD"},
    {"tangan_mencubit_warna4", "1F90F-1F3FE"},
    {"tangan_mencubit_warna5", "1F90F-1F3FF"},
    {"muka_mulut_berzip", "1F910"},
    {"muka_dengan_wang_dalam_mulut", "1F911"},
    {"muka_dengan_termometer", "1F912"},
    {"muka_nerda", "1F913"},
    {"muka_sedang_berfikir", "1F914"},
    {"muka_dengan_kepala_berbalut", "1F915"},
    {"robot", "1F916"},
    {"muka_memberi_pelukan", "1F917"},
    {"simbol_tanduk", "1F918"},
    {"simbol_tanduk_warna1", "1F918-1F3FB"},
    {"simbol_tanduk_warna2", "1F918-1F3FC"},
    {"simbol_tanduk_warna3", "1F918-1F3FD"},
    {"simbol_tanduk_warna4", "1F918-1F3FE"},
    {"simbol_tanduk_warna5", "1F918-1F3FF"},
    {"telefon_saya", "1F919"},
    {"telefon_saya_warna1", "1F919-1F3FB"},
    {"telefon_saya_warna2", "1F919-1F3FC"},
    {"telefon_saya_warna3", "1F919-1F3FD"},
    {"telefon_saya_warna4", "1F919-1F3FE"},
    {"telefon_saya_warna5", "1F919-1F3FF"},
    {"angkat_belakang_tapak_tangan", "1F91A"},
    {"angkat_belakang_tapak_tangan_warna1", "1F91A-1F3FB"},
    {"angkat_belakang_tapak_tangan_warna2", "1F91A-1F3FC"},
    {"angkat_belakang_tapak_tangan_warna3", "1F91A-1F3FD"},
    {"angkat_belakang_tapak_tangan_warna4", "1F91A-1F3FE"},
    {"angkat_belakang_tapak_tangan_warna5", "1F91A-1F3FF"},
    {"penumbuk_ke_kiri", "1F91B"},
    {"penumbuk_ke_kiri_warna1", "1F91B-1F3FB"},
    {"penumbuk_ke_kiri_warna2", "1F91B-1F3FC"},
    {"penumbuk_ke_kiri_warna3", "1F91B-1F3FD"},
    {"penumbuk_ke_kiri_warna4", "1F91B-1F3FE"},
    {"penumbuk_ke_kiri_warna5", "1F91B-1F3FF"},
    {"penumbuk_ke_kanan", "1F91C"},
    {"penumbuk_ke_kanan_warna1", "1F91C-1F3FB"},
    {"penumbuk_ke_kanan_warna2", "1F91C-1F3FC"},
    {"penumbuk_ke_kanan_warna3", "1F91C-1F3FD"},
    {"penumbuk_ke_kanan_warna4", "1F91C-1F3FE"},
    {"penumbuk_ke_kanan_warna5", "1F91C-1F3FF"},
    {"berjabat_tangan", "1F91D"},
    {"berjabat_tangan_warna1", "1F91D-1F3FB"},
    {"berjabat_tangan_warna2", "1F91D-1F3FC"},
    {"berjabat_tangan_warna3", "1F91D-1F3FD"},
    {"berjabat_tangan_warna4", "1F91D-1F3FE"},
    {"berjabat_tangan_warna5", "1F91D-1F3FF"},
    {"berjabat_tangan_warna1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"berjabat_tangan_warna1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"berjabat_tangan_warna1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"berjabat_tangan_warna1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"berjabat_tangan_warna2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"berjabat_tangan_warna2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"berjabat_tangan_warna2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"berjabat_tangan_warna2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"berjabat_tangan_warna3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"berjabat_tangan_warna3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"berjabat_tangan_warna3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"berjabat_tangan_warna3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"berjabat_tangan_warna4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"berjabat_tangan_warna4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"berjabat_tangan_warna4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"berjabat_tangan_warna4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"berjabat_tangan_warna5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"berjabat_tangan_warna5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"berjabat_tangan_warna5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"berjabat_tangan_warna5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"silang_jari", "1F91E"},
    {"silang_jari_warna1", "1F91E-1F3FB"},
    {"silang_jari_warna2", "1F91E-1F3FC"},
    {"silang_jari_warna3", "1F91E-1F3FD"},
    {"silang_jari_warna4", "1F91E-1F3FE"},
    {"silang_jari_warna5", "1F91E-1F3FF"},
    {"gerak_isyarat_cinta_kamu", "1F91F"},
    {"gerak_isyarat_cinta_kamu_warna1", "1F91F-1F3FB"},
    {"gerak_isyarat_cinta_kamu_warna2", "1F91F-1F3FC"},
    {"gerak_isyarat_cinta_kamu_warna3", "1F91F-1F3FD"},
    {"gerak_isyarat_cinta_kamu_warna4", "1F91F-1F3FE"},
    {"gerak_isyarat_cinta_kamu_warna5", "1F91F-1F3FF"},
    {"muka_koboi", "1F920"},
    {"muka_badut", "1F921"},
    {"muka_mual", "1F922"},
    {"ketawa_tergolek_golek", "1F923"},
    {"muka_meliur", "1F924"},
    {"muka_bohong", "1F925"},
    {"tepuk_dahi", "1F926"},
    {"tepuk_dahi_warna1", "1F926-1F3FB"},
    {"tepuk_dahi_warna2", "1F926-1F3FC"},
    {"tepuk_dahi_warna3", "1F926-1F3FD"},
    {"tepuk_dahi_warna4", "1F926-1F3FE"},
    {"tepuk_dahi_warna5", "1F926-1F3FF"},
    {"muka_bersin", "1F927"},
    {"muka_dengan_kening_terangkat", "1F928"},
    {"terpesona", "1F929"},
    {"muka_gila_gila", "1F92A"},
    {"muka_suruh_senyap", "1F92B"},
    {"muka_dengan_simbol_pada_mulut", "1F92C"},
    {"muka_dengan_tangan_atas_mulut", "1F92D"},
    {"muka_muntah", "1F92E"},
    {"kepala_meletup", "1F92F"},
    {"wanita_hamil", "1F930"},
    {"wanita_hamil_warna1", "1F930-1F3FB"},
    {"wanita_hamil_warna2", "1F930-1F3FC"},
    {"wanita_hamil_warna3", "1F930-1F3FD"},
    {"wanita_hamil_warna4", "1F930-1F3FE"},
    {"wanita_hamil_warna5", "1F930-1F3FF"},
    {"menyusu_badan", "1F931"},
    {"menyusu_badan_warna1", "1F931-1F3FB"},
    {"menyusu_badan_warna2", "1F931-1F3FC"},
    {"menyusu_badan_warna3", "1F931-1F3FD"},
    {"menyusu_badan_warna4", "1F931-1F3FE"},
    {"menyusu_badan_warna5", "1F931-1F3FF"},
    {"tapak_tangan_rapat", "1F932"},
    {"tapak_tangan_rapat_warna1", "1F932-1F3FB"},
    {"tapak_tangan_rapat_warna2", "1F932-1F3FC"},
    {"tapak_tangan_rapat_warna3", "1F932-1F3FD"},
    {"tapak_tangan_rapat_warna4", "1F932-1F3FE"},
    {"tapak_tangan_rapat_warna5", "1F932-1F3FF"},
    {"swafoto", "1F933"},
    {"swafoto_warna1", "1F933-1F3FB"},
    {"swafoto_warna2", "1F933-1F3FC"},
    {"swafoto_warna3", "1F933-1F3FD"},
    {"swafoto_warna4", "1F933-1F3FE"},
    {"swafoto_warna5", "1F933-1F3FF"},
    {"putera", "1F934"},
    {"putera_warna1", "1F934-1F3FB"},
    {"putera_warna2", "1F934-1F3FC"},
    {"putera_warna3", "1F934-1F3FD"},
    {"putera_warna4", "1F934-1F3FE"},
    {"putera_warna5", "1F934-1F3FF"},
    {"orang_bertuksedo", "1F935"},
    {"orang_bertuksedo_warna1", "1F935-1F3FB"},
    {"orang_bertuksedo_warna2", "1F935-1F3FC"},
    {"orang_bertuksedo_warna3", "1F935-1F3FD"},
    {"orang_bertuksedo_warna4", "1F935-1F3FE"},
    {"orang_bertuksedo_warna5", "1F935-1F3FF"},
    {"pn_claus", "1F936"},
    {"pn_claus_warna1", "1F936-1F3FB"},
    {"pn_claus_warna2", "1F936-1F3FC"},
    {"pn_claus_warna3", "1F936-1F3FD"},
    {"pn_claus_warna4", "1F936-1F3FE"},
    {"pn_claus_warna5", "1F936-1F3FF"},
    {"angkat_bahu", "1F937"},
    {"angkat_bahu_warna1", "1F937-1F3FB"},
    {"angkat_bahu_warna2", "1F937-1F3FC"},
    {"angkat_bahu_warna3", "1F937-1F3FD"},
    {"angkat_bahu_warna4", "1F937-1F3FE"},
    {"angkat_bahu_warna5", "1F937-1F3FF"},
    {"putar_roda", "1F938"},
    {"putar_roda_warna1", "1F938-1F3FB"},
    {"putar_roda_warna2", "1F938-1F3FC"},
    {"putar_roda_warna3", "1F938-1F3FD"},
    {"putar_roda_warna4", "1F938-1F3FE"},
    {"putar_roda_warna5", "1F938-1F3FF"},
    {"menjugel", "1F939"},
    {"menjugel_warna1", "1F939-1F3FB"},
    {"menjugel_warna2", "1F939-1F3FC"},
    {"menjugel_warna3", "1F939-1F3FD"},
    {"menjugel_warna4", "1F939-1F3FE"},
    {"menjugel_warna5", "1F939-1F3FF"},
    {"pemain_pedang", "1F93A"},
    {"bergusti", "1F93C"},
    {"bergusti_warna1", "1F93C-1F3FB"},
    {"bergusti_warna2", "1F93C-1F3FC"},
    {"bergusti_warna3", "1F93C-1F3FD"},
    {"bergusti_warna4", "1F93C-1F3FE"},
    {"bergusti_warna5", "1F93C-1F3FF"},
    {"bergusti_warna1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"bergusti_warna1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"bergusti_warna1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"bergusti_warna1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"bergusti_warna2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"bergusti_warna2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"bergusti_warna2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"bergusti_warna2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"bergusti_warna3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"bergusti_warna3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"bergusti_warna3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"bergusti_warna3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"bergusti_warna4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"bergusti_warna4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"bergusti_warna4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"bergusti_warna4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"bergusti_warna5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"bergusti_warna5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"bergusti_warna5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"bergusti_warna5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"polo_air", "1F93D"},
    {"polo_air_warna1", "1F93D-1F3FB"},
    {"polo_air_warna2", "1F93D-1F3FC"},
    {"polo_air_warna3", "1F93D-1F3FD"},
    {"polo_air_warna4", "1F93D-1F3FE"},
    {"polo_air_warna5", "1F93D-1F3FF"},
    {"main_bola_baling", "1F93E"},
    {"main_bola_baling_warna1", "1F93E-1F3FB"},
    {"main_bola_baling_warna2", "1F93E-1F3FC"},
    {"main_bola_baling_warna3", "1F93E-1F3FD"},
    {"main_bola_baling_warna4", "1F93E-1F3FE"},
    {"main_bola_baling_warna5", "1F93E-1F3FF"},
    {"topeng_menyelam", "1F93F"},
    {"bunga_layu", "1F940"},
    {"dram", "1F941"},
    {"melaga_gelas", "1F942"},
    {"gelas_pendek", "1F943"},
    {"sudu", "1F944"},
    {"jaring_gol", "1F945"},
    {"pingat_tempat_pertama", "1F947"},
    {"pingat_tempat_kedua", "1F948"},
    {"pingat_tempat_ketiga", "1F949"},
    {"sarung_tangan_tinju", "1F94A"},
    {"pakaian_seni_bela_diri", "1F94B"},
    {"batu_curling", "1F94C"},
    {"lacrosse", "1F94D"},
    {"bola_lisut", "1F94E"},
    {"cakera_terbang", "1F94F"},
    {"kroisan", "1F950"},
    {"avokado", "1F951"},
    {"timun", "1F952"},
    {"bakon", "1F953"},
    {"ubi_kentang", "1F954"},
    {"lobak_merah", "1F955"},
    {"roti_baguette", "1F956"},
    {"salad_hijau", "1F957"},
    {"makanan_dalam_belanga", "1F958"},
    {"roti_kebab", "1F959"},
    {"telur", "1F95A"},
    {"segelas_susu", "1F95B"},
    {"kacang_tanah", "1F95C"},
    {"buah_kiwi", "1F95D"},
    {"penkek", "1F95E"},
    {"ladu", "1F95F"},
    {"biskut_nasib", "1F960"},
    {"kotak_bungkus", "1F961"},
    {"penyepit", "1F962"},
    {"mangkuk_dan_sudu", "1F963"},
    {"cawan_dan_straw", "1F964"},
    {"kelapa", "1F965"},
    {"brokoli", "1F966"},
    {"pai", "1F967"},
    {"pretzel", "1F968"},
    {"sepotong_daging", "1F969"},
    {"sandwic", "1F96A"},
    {"makanan_tin", "1F96B"},
    {"sayuran_hijau", "1F96C"},
    {"mangga", "1F96D"},
    {"kuih_bulan", "1F96E"},
    {"bagel", "1F96F"},
    {"muka_tersenyum_dengan_hati", "1F970"},
    {"muka_menguap", "1F971"},
    {"muka_tersenyum_dengan_airmata", "1F972"},
    {"muka_berparti", "1F973"},
    {"muka_kepeningan", "1F974"},
    {"muka_sebak", "1F975"},
    {"muka_dingin", "1F976"},
    {"ninja", "1F977"},
    {"ninja_warna1", "1F977-1F3FB"},
    {"ninja_warna2", "1F977-1F3FC"},
    {"ninja_warna3", "1F977-1F3FD"},
    {"ninja_warna4", "1F977-1F3FE"},
    {"ninja_warna5", "1F977-1F3FF"},
    {"muka_samaran", "1F978"},
    {"muka_menahan_air_mata", "1F979"},
    {"muka_merayu", "1F97A"},
    {"sari", "1F97B"},
    {"kot_makmal", "1F97C"},
    {"gogal", "1F97D"},
    {"but_kembara", "1F97E"},
    {"kasut_rata", "1F97F"},
    {"ketam", "1F980"},
    {"singa", "1F981"},
    {"kala_jengking", "1F982"},
    {"ayam_belanda", "1F983"},
    {"unikorn", "1F984"},
    {"helang", "1F985"},
    {"itik", "1F986"},
    {"kelawar", "1F987"},
    {"jerung", "1F988"},
    {"burung_hantu", "1F989"},
    {"rubah", "1F98A"},
    {"rama_rama", "1F98B"},
    {"rusa", "1F98C"},
    {"gorila", "1F98D"},
    {"cicak", "1F98E"},
    {"badak_sumbu", "1F98F"},
    {"udang", "1F990"},
    {"sotong", "1F991"},
    {"zirafah", "1F992"},
    {"zebra", "1F993"},
    {"landak_kecil", "1F994"},
    {"sauropod", "1F995"},
    {"trex", "1F996"},
    {"cengkerik", "1F997"},
    {"kanggaru", "1F998"},
    {"llama", "1F999"},
    {"merak", "1F99A"},
    {"badak_air", "1F99B"},
    {"kakak_tua", "1F99C"},
    {"rakun", "1F99D"},
    {"udang_karang", "1F99E"},
    {"nyamuk", "1F99F"},
    {"mikrob", "1F9A0"},
    {"bejar", "1F9A1"},
    {"swan", "1F9A2"},
    {"mamot", "1F9A3"},
    {"dodo", "1F9A4"},
    {"sloth", "1F9A5"},
    {"memerang", "1F9A6"},
    {"orang_utan", "1F9A7"},
    {"skunk", "1F9A8"},
    {"flamingo", "1F9A9"},
    {"tiram", "1F9AA"},
    {"beaver", "1F9AB"},
    {"bison", "1F9AC"},
    {"anjing_laut", "1F9AD"},
    {"anjing_pandu", "1F9AE"},
    {"tongkat_penduga", "1F9AF"},
    {"rambut_merah", "1F9B0"},
    {"rambut_kerinting", "1F9B1"},
    {"botak", "1F9B2"},
    {"rambut_putih", "1F9B3"},
    {"tulang", "1F9B4"},
    {"kaki", "1F9B5"},
    {"kaki_warna1", "1F9B5-1F3FB"},
    {"kaki_warna2", "1F9B5-1F3FC"},
    {"kaki_warna3", "1F9B5-1F3FD"},
    {"kaki_warna4", "1F9B5-1F3FE"},
    {"kaki_warna5", "1F9B5-1F3FF"},
    {"hujung_kaki", "1F9B6"},
    {"hujung_kaki_warna1", "1F9B6-1F3FB"},
    {"hujung_kaki_warna2", "1F9B6-1F3FC"},
    {"hujung_kaki_warna3", "1F9B6-1F3FD"},
    {"hujung_kaki_warna4", "1F9B6-1F3FE"},
    {"hujung_kaki_warna5", "1F9B6-1F3FF"},
    {"gigi", "1F9B7"},
    {"adiwira", "1F9B8"},
    {"adiwira_warna1", "1F9B8-1F3FB"},
    {"adiwira_warna2", "1F9B8-1F3FC"},
    {"adiwira_warna3", "1F9B8-1F3FD"},
    {"adiwira_warna4", "1F9B8-1F3FE"},
    {"adiwira_warna5", "1F9B8-1F3FF"},
    {"ketua_penjahat", "1F9B9"},
    {"ketua_penjahat_warna1", "1F9B9-1F3FB"},
    {"ketua_penjahat_warna2", "1F9B9-1F3FC"},
    {"ketua_penjahat_warna3", "1F9B9-1F3FD"},
    {"ketua_penjahat_warna4", "1F9B9-1F3FE"},
    {"ketua_penjahat_warna5", "1F9B9-1F3FF"},
    {"rompi_keselamatan", "1F9BA"},
    {"telinga_dengan_alat_bantu_dengar", "1F9BB"},
    {"telinga_dengan_alat_bantu_dengar_warna1", "1F9BB-1F3FB"},
    {"telinga_dengan_alat_bantu_dengar_warna2", "1F9BB-1F3FC"},
    {"telinga_dengan_alat_bantu_dengar_warna3", "1F9BB-1F3FD"},
    {"telinga_dengan_alat_bantu_dengar_warna4", "1F9BB-1F3FE"},
    {"telinga_dengan_alat_bantu_dengar_warna5", "1F9BB-1F3FF"},
    {"kerosi_roda_bermotor", "1F9BC"},
    {"kerusi_roda_manual", "1F9BD"},
    {"lengan_mekanikal", "1F9BE"},
    {"kaki_mekanikal", "1F9BF"},
    {"sepotong_keju", "1F9C0"},
    {"kek_cawan", "1F9C1"},
    {"garam", "1F9C2"},
    {"minuman_air_kotak", "1F9C3"},
    {"bawang_putih", "1F9C4"},
    {"bawang", "1F9C5"},
    {"falafel", "1F9C6"},
    {"wafel", "1F9C7"},
    {"mentega", "1F9C8"},
    {"mate", "1F9C9"},
    {"kiub_ais", "1F9CA"},
    {"teh_buih", "1F9CB"},
    {"raksasa", "1F9CC"},
    {"orang_berdiri", "1F9CD"},
    {"orang_berdiri_warna1", "1F9CD-1F3FB"},
    {"orang_berdiri_warna2", "1F9CD-1F3FC"},
    {"orang_berdiri_warna3", "1F9CD-1F3FD"},
    {"orang_berdiri_warna4", "1F9CD-1F3FE"},
    {"orang_berdiri_warna5", "1F9CD-1F3FF"},
    {"orang_bersimpuh", "1F9CE"},
    {"orang_bersimpuh_warna1", "1F9CE-1F3FB"},
    {"orang_bersimpuh_warna2", "1F9CE-1F3FC"},
    {"orang_bersimpuh_warna3", "1F9CE-1F3FD"},
    {"orang_bersimpuh_warna4", "1F9CE-1F3FE"},
    {"orang_bersimpuh_warna5", "1F9CE-1F3FF"},
    {"orang_pekak", "1F9CF"},
    {"orang_pekak_warna1", "1F9CF-1F3FB"},
    {"orang_pekak_warna2", "1F9CF-1F3FC"},
    {"orang_pekak_warna3", "1F9CF-1F3FD"},
    {"orang_pekak_warna4", "1F9CF-1F3FE"},
    {"orang_pekak_warna5", "1F9CF-1F3FF"},
    {"muka_dengan_cermin_mata_satu_kanta", "1F9D0"},
    {"dewasa", "1F9D1"},
    {"dewasa_warna1", "1F9D1-1F3FB"},
    {"dewasa_warna2", "1F9D1-1F3FC"},
    {"dewasa_warna3", "1F9D1-1F3FD"},
    {"dewasa_warna4", "1F9D1-1F3FE"},
    {"dewasa_warna5", "1F9D1-1F3FF"},
    {"kanak_kanak", "1F9D2"},
    {"kanak_kanak_warna1", "1F9D2-1F3FB"},
    {"kanak_kanak_warna2", "1F9D2-1F3FC"},
    {"kanak_kanak_warna3", "1F9D2-1F3FD"},
    {"kanak_kanak_warna4", "1F9D2-1F3FE"},
    {"kanak_kanak_warna5", "1F9D2-1F3FF"},
    {"dewasa_lebih_tua", "1F9D3"},
    {"dewasa_lebih_tua_warna1", "1F9D3-1F3FB"},
    {"dewasa_lebih_tua_warna2", "1F9D3-1F3FC"},
    {"dewasa_lebih_tua_warna3", "1F9D3-1F3FD"},
    {"dewasa_lebih_tua_warna4", "1F9D3-1F3FE"},
    {"dewasa_lebih_tua_warna5", "1F9D3-1F3FF"},
    {"orang_janggut", "1F9D4"},
    {"orang_janggut_warna1", "1F9D4-1F3FB"},
    {"orang_janggut_warna2", "1F9D4-1F3FC"},
    {"orang_janggut_warna3", "1F9D4-1F3FD"},
    {"orang_janggut_warna4", "1F9D4-1F3FE"},
    {"orang_janggut_warna5", "1F9D4-1F3FF"},
    {"wanita_bertudung", "1F9D5"},
    {"wanita_bertudung_warna1", "1F9D5-1F3FB"},
    {"wanita_bertudung_warna2", "1F9D5-1F3FC"},
    {"wanita_bertudung_warna3", "1F9D5-1F3FD"},
    {"wanita_bertudung_warna4", "1F9D5-1F3FE"},
    {"wanita_bertudung_warna5", "1F9D5-1F3FF"},
    {"orang_dalam_bilik_berwap", "1F9D6"},
    {"orang_dalam_bilik_berwap_warna1", "1F9D6-1F3FB"},
    {"orang_dalam_bilik_berwap_warna2", "1F9D6-1F3FC"},
    {"orang_dalam_bilik_berwap_warna3", "1F9D6-1F3FD"},
    {"orang_dalam_bilik_berwap_warna4", "1F9D6-1F3FE"},
    {"orang_dalam_bilik_berwap_warna5", "1F9D6-1F3FF"},
    {"orang_mendaki", "1F9D7"},
    {"orang_mendaki_warna1", "1F9D7-1F3FB"},
    {"orang_mendaki_warna2", "1F9D7-1F3FC"},
    {"orang_mendaki_warna3", "1F9D7-1F3FD"},
    {"orang_mendaki_warna4", "1F9D7-1F3FE"},
    {"orang_mendaki_warna5", "1F9D7-1F3FF"},
    {"orang_dalam_kedudukan_teratai", "1F9D8"},
    {"orang_dalam_kedudukan_teratai_warna1", "1F9D8-1F3FB"},
    {"orang_dalam_kedudukan_teratai_warna2", "1F9D8-1F3FC"},
    {"orang_dalam_kedudukan_teratai_warna3", "1F9D8-1F3FD"},
    {"orang_dalam_kedudukan_teratai_warna4", "1F9D8-1F3FE"},
    {"orang_dalam_kedudukan_teratai_warna5", "1F9D8-1F3FF"},
    {"ahli_silap_mata", "1F9D9"},
    {"ahli_silap_mata_warna1", "1F9D9-1F3FB"},
    {"ahli_silap_mata_warna2", "1F9D9-1F3FC"},
    {"ahli_silap_mata_warna3", "1F9D9-1F3FD"},
    {"ahli_silap_mata_warna4", "1F9D9-1F3FE"},
    {"ahli_silap_mata_warna5", "1F9D9-1F3FF"},
    {"pari_pari", "1F9DA"},
    {"pari_pari_warna1", "1F9DA-1F3FB"},
    {"pari_pari_warna2", "1F9DA-1F3FC"},
    {"pari_pari_warna3", "1F9DA-1F3FD"},
    {"pari_pari_warna4", "1F9DA-1F3FE"},
    {"pari_pari_warna5", "1F9DA-1F3FF"},
    {"puntianak", "1F9DB"},
    {"puntianak_warna1", "1F9DB-1F3FB"},
    {"puntianak_warna2", "1F9DB-1F3FC"},
    {"puntianak_warna3", "1F9DB-1F3FD"},
    {"puntianak_warna4", "1F9DB-1F3FE"},
    {"puntianak_warna5", "1F9DB-1F3FF"},
    {"manusia_duyung", "1F9DC"},
    {"manusia_duyung_warna1", "1F9DC-1F3FB"},
    {"manusia_duyung_warna2", "1F9DC-1F3FC"},
    {"manusia_duyung_warna3", "1F9DC-1F3FD"},
    {"manusia_duyung_warna4", "1F9DC-1F3FE"},
    {"manusia_duyung_warna5", "1F9DC-1F3FF"},
    {"orang_halus", "1F9DD"},
    {"orang_halus_warna1", "1F9DD-1F3FB"},
    {"orang_halus_warna2", "1F9DD-1F3FC"},
    {"orang_halus_warna3", "1F9DD-1F3FD"},
    {"orang_halus_warna4", "1F9DD-1F3FE"},
    {"orang_halus_warna5", "1F9DD-1F3FF"},
    {"jin", "1F9DE"},
    {"zombi", "1F9DF"},
    {"otak", "1F9E0"},
    {"hati_jingga", "1F9E1"},
    {"kep_berbil", "1F9E2"},
    {"skaf", "1F9E3"},
    {"sarung_tangan", "1F9E4"},
    {"kot", "1F9E5"},
    {"stoking", "1F9E6"},
    {"angpau", "1F9E7"},
    {"petasan", "1F9E8"},
    {"kepingan_suai_gambar", "1F9E9"},
    {"tabung_uji", "1F9EA"},
    {"piring_petri", "1F9EB"},
    {"dna", "1F9EC"},
    {"kompas", "1F9ED"},
    {"sempua", "1F9EE"},
    {"alat_pemadam_api", "1F9EF"},
    {"kotak_alat", "1F9F0"},
    {"bata", "1F9F1"},
    {"magnet", "1F9F2"},
    {"bagasi", "1F9F3"},
    {"botol_losyen", "1F9F4"},
    {"benang", "1F9F5"},
    {"yarn", "1F9F6"},
    {"pin_dawai", "1F9F7"},
    {"beruang_teddy", "1F9F8"},
    {"penyapu", "1F9F9"},
    {"bakul", "1F9FA"},
    {"gulungan_kertas", "1F9FB"},
    {"sabun", "1F9FC"},
    {"span", "1F9FD"},
    {"resit", "1F9FE"},
    {"azimat_nazar", "1F9FF"},
    {"kasut_balet", "1FA70"},
    {"sut_renang_terus", "1FA71"},
    {"seluar_dalam", "1FA72"},
    {"seluar_pendek", "1FA73"},
    {"sandal_tali_kulit", "1FA74"},
    {"hati_biru_muda", "1FA75"},
    {"hati_kelabu", "1FA76"},
    {"hati_merah_jambu", "1FA77"},
    {"titisan_darah", "1FA78"},
    {"pembalut_adhesif", "1FA79"},
    {"stetoskop", "1FA7A"},
    {"sinar_x", "1FA7B"},
    {"topang", "1FA7C"},
    {"yo_yo", "1FA80"},
    {"layang_layang", "1FA81"},
    {"paracut", "1FA82"},
    {"bumerang", "1FA83"},
    {"tongkat_sakti", "1FA84"},
    {"pinata", "1FA85"},
    {"anak_potong_sekelompok", "1FA86"},
    {"marakas", "1FA87"},
    {"seruling", "1FA88"},
    {"hap", "1FA89"},
    {"trombon", "1FA8A"},
    {"peti_harta_karun", "1FA8E"},
    {"penyodok", "1FA8F"},
    {"planet_bergelang", "1FA90"},
    {"kerusi", "1FA91"},
    {"pisau_cukur", "1FA92"},
    {"kapak", "1FA93"},
    {"lampu_diya", "1FA94"},
    {"banjo", "1FA95"},
    {"helmet_tentera", "1FA96"},
    {"akordion", "1FA97"},
    {"karatung", "1FA98"},
    {"syiling", "1FA99"},
    {"gergaji_kayu", "1FA9A"},
    {"pemutar_skru", "1FA9B"},
    {"tangga", "1FA9C"},
    {"cangkuk", "1FA9D"},
    {"cermin", "1FA9E"},
    {"tingkap", "1FA9F"},
    {"pelocok", "1FAA0"},
    {"jarum_menjahit", "1FAA1"},
    {"simpul", "1FAA2"},
    {"timba", "1FAA3"},
    {"perangkap_tikus", "1FAA4"},
    {"berus_gigi", "1FAA5"},
    {"nisan", "1FAA6"},
    {"plakad", "1FAA7"},
    {"batuan", "1FAA8"},
    {"bola_cermin", "1FAA9"},
    {"kad_pengenalan", "1FAAA"},
    {"bateri_lemah", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"kipas_tangan_boleh_lipat", "1FAAD"},
    {"pin_rambut", "1FAAE"},
    {"khanda", "1FAAF"},
    {"lalat", "1FAB0"},
    {"cacing", "1FAB1"},
    {"kumbang", "1FAB2"},
    {"lipas", "1FAB3"},
    {"pokok_dalam_pasu", "1FAB4"},
    {"kayu", "1FAB5"},
    {"bulu_pelepah", "1FAB6"},
    {"teratai", "1FAB7"},
    {"karang", "1FAB8"},
    {"sarang_kosong", "1FAB9"},
    {"sarang_dengan_telur", "1FABA"},
    {"keladi_bunting", "1FABB"},
    {"ubur_ubur", "1FABC"},
    {"sayap", "1FABD"},
    {"pokok_tidak_berdaun", "1FABE"},
    {"angsa", "1FABF"},
    {"organ_jantung", "1FAC0"},
    {"paru_paru", "1FAC1"},
    {"orang_berpelukan", "1FAC2"},
    {"lelaki_bunting", "1FAC3"},
    {"lelaki_bunting_warna1", "1FAC3-1F3FB"},
    {"lelaki_bunting_warna2", "1FAC3-1F3FC"},
    {"lelaki_bunting_warna3", "1FAC3-1F3FD"},
    {"lelaki_bunting_warna4", "1FAC3-1F3FE"},
    {"lelaki_bunting_warna5", "1FAC3-1F3FF"},
    {"orang_hamil", "1FAC4"},
    {"orang_hamil_warna1", "1FAC4-1F3FB"},
    {"orang_hamil_warna2", "1FAC4-1F3FC"},
    {"orang_hamil_warna3", "1FAC4-1F3FD"},
    {"orang_hamil_warna4", "1FAC4-1F3FE"},
    {"orang_hamil_warna5", "1FAC4-1F3FF"},
    {"orang_memakai_mahkota", "1FAC5"},
    {"orang_memakai_mahkota_warna1", "1FAC5-1F3FB"},
    {"orang_memakai_mahkota_warna2", "1FAC5-1F3FC"},
    {"orang_memakai_mahkota_warna3", "1FAC5-1F3FD"},
    {"orang_memakai_mahkota_warna4", "1FAC5-1F3FE"},
    {"orang_memakai_mahkota_warna5", "1FAC5-1F3FF"},
    {"cap_jari", "1FAC6"},
    {"makhluk_berbulu", "1FAC8"},
    {"orca", "1FACD"},
    {"moose", "1FACE"},
    {"keldai", "1FACF"},
    {"beri_biru", "1FAD0"},
    {"cili_kembung", "1FAD1"},
    {"buah_zaitun", "1FAD2"},
    {"roti_leper", "1FAD3"},
    {"tamale", "1FAD4"},
    {"fondue", "1FAD5"},
    {"teko_teh", "1FAD6"},
    {"menuang_cecair", "1FAD7"},
    {"kacang", "1FAD8"},
    {"balang", "1FAD9"},
    {"akar_halia", "1FADA"},
    {"lenggai", "1FADB"},
    {"sayur_akar", "1FADC"},
    {"percik", "1FADF"},
    {"muka_mencair", "1FAE0"},
    {"muka_sedang_menabik", "1FAE1"},
    {"muka_dengan_mata_terbuka_dan_tangan_atas_mulut", "1FAE2"},
    {"muka_dengan_mata_mengintai", "1FAE3"},
    {"muka_dengan_mulut_serong", "1FAE4"},
    {"muka_garis_putus_putus", "1FAE5"},
    {"menggigit_bibir", "1FAE6"},
    {"buih", "1FAE7"},
    {"muka_bergoyang", "1FAE8"},
    {"muka_dengan_lingkaran_gelap_bawah_mata", "1FAE9"},
    {"muka_herot", "1FAEA"},
    {"awan_pergelutan", "1FAEF"},
    {"tangan_dengan_jari_telunjuk_dan_ibu_jari_bersilang", "1FAF0"},
    {"tangan_dengan_jari_telunjuk_dan_ibu_jari_bersilang_warna1", "1FAF0-1F3FB"},
    {"tangan_dengan_jari_telunjuk_dan_ibu_jari_bersilang_warna2", "1FAF0-1F3FC"},
    {"tangan_dengan_jari_telunjuk_dan_ibu_jari_bersilang_warna3", "1FAF0-1F3FD"},
    {"tangan_dengan_jari_telunjuk_dan_ibu_jari_bersilang_warna4", "1FAF0-1F3FE"},
    {"tangan_dengan_jari_telunjuk_dan_ibu_jari_bersilang_warna5", "1FAF0-1F3FF"},
    {"tangan_kanan", "1FAF1"},
    {"tangan_kanan_warna1", "1FAF1-1F3FB"},
    {"tangan_kanan_warna2", "1FAF1-1F3FC"},
    {"tangan_kanan_warna3", "1FAF1-1F3FD"},
    {"tangan_kanan_warna4", "1FAF1-1F3FE"},
    {"tangan_kanan_warna5", "1FAF1-1F3FF"},
    {"tangan_kiri", "1FAF2"},
    {"tangan_kiri_warna1", "1FAF2-1F3FB"},
    {"tangan_kiri_warna2", "1FAF2-1F3FC"},
    {"tangan_kiri_warna3", "1FAF2-1F3FD"},
    {"tangan_kiri_warna4", "1FAF2-1F3FE"},
    {"tangan_kiri_warna5", "1FAF2-1F3FF"},
    {"tapak_tangan_ke_bawah", "1FAF3"},
    {"tapak_tangan_ke_bawah_warna1", "1FAF3-1F3FB"},
    {"tapak_tangan_ke_bawah_warna2", "1FAF3-1F3FC"},
    {"tapak_tangan_ke_bawah_warna3", "1FAF3-1F3FD"},
    {"tapak_tangan_ke_bawah_warna4", "1FAF3-1F3FE"},
    {"tapak_tangan_ke_bawah_warna5", "1FAF3-1F3FF"},
    {"tapak_tangan_ke_atas", "1FAF4"},
    {"tapak_tangan_ke_atas_warna1", "1FAF4-1F3FB"},
    {"tapak_tangan_ke_atas_warna2", "1FAF4-1F3FC"},
    {"tapak_tangan_ke_atas_warna3", "1FAF4-1F3FD"},
    {"tapak_tangan_ke_atas_warna4", "1FAF4-1F3FE"},
    {"tapak_tangan_ke_atas_warna5", "1FAF4-1F3FF"},
    {"jari_telunjuk_menunjuk_ke_arah_pemandang", "1FAF5"},
    {"jari_telunjuk_menunjuk_ke_arah_pemandang_warna1", "1FAF5-1F3FB"},
    {"jari_telunjuk_menunjuk_ke_arah_pemandang_warna2", "1FAF5-1F3FC"},
    {"jari_telunjuk_menunjuk_ke_arah_pemandang_warna3", "1FAF5-1F3FD"},
    {"jari_telunjuk_menunjuk_ke_arah_pemandang_warna4", "1FAF5-1F3FE"},
    {"jari_telunjuk_menunjuk_ke_arah_pemandang_warna5", "1FAF5-1F3FF"},
    {"tangan_berbentuk_hati", "1FAF6"},
    {"tangan_berbentuk_hati_warna1", "1FAF6-1F3FB"},
    {"tangan_berbentuk_hati_warna2", "1FAF6-1F3FC"},
    {"tangan_berbentuk_hati_warna3", "1FAF6-1F3FD"},
    {"tangan_berbentuk_hati_warna4", "1FAF6-1F3FE"},
    {"tangan_berbentuk_hati_warna5", "1FAF6-1F3FF"},
    {"tangan_menolak_ke_arah_kiri", "1FAF7"},
    {"tangan_menolak_ke_arah_kiri_warna1", "1FAF7-1F3FB"},
    {"tangan_menolak_ke_arah_kiri_warna2", "1FAF7-1F3FC"},
    {"tangan_menolak_ke_arah_kiri_warna3", "1FAF7-1F3FD"},
    {"tangan_menolak_ke_arah_kiri_warna4", "1FAF7-1F3FE"},
    {"tangan_menolak_ke_arah_kiri_warna5", "1FAF7-1F3FF"},
    {"tangan_menolak_ke_arah_kanan", "1FAF8"},
    {"tangan_menolak_ke_arah_kanan_warna1", "1FAF8-1F3FB"},
    {"tangan_menolak_ke_arah_kanan_warna2", "1FAF8-1F3FC"},
    {"tangan_menolak_ke_arah_kanan_warna3", "1FAF8-1F3FD"},
    {"tangan_menolak_ke_arah_kanan_warna4", "1FAF8-1F3FE"},
    {"tangan_menolak_ke_arah_kanan_warna5", "1FAF8-1F3FF"},
    {"butang_kekunci_tanda_nombor", "0023-FE0F-20E3"},
    {"butang_kekunci_tanda_bintang", "002A-FE0F-20E3"},
    {"butang_kekunci_0", "0030-FE0F-20E3"},
    {"butang_kekunci_1", "0031-FE0F-20E3"},
    {"butang_kekunci_2", "0032-FE0F-20E3"},
    {"butang_kekunci_3", "0033-FE0F-20E3"},
    {"butang_kekunci_4", "0034-FE0F-20E3"},
    {"butang_kekunci_5", "0035-FE0F-20E3"},
    {"butang_kekunci_6", "0036-FE0F-20E3"},
    {"butang_kekunci_7", "0037-FE0F-20E3"},
    {"butang_kekunci_8", "0038-FE0F-20E3"},
    {"butang_kekunci_9", "0039-FE0F-20E3"},
    {"bendera_pulau_ascension", "1F1E6-1F1E8"},
    {"bendera_andorra", "1F1E6-1F1E9"},
    {"bendera_emiriah_arab_bersatu", "1F1E6-1F1EA"},
    {"bendera_afghanistan", "1F1E6-1F1EB"},
    {"bendera_antigua_dan_barbuda", "1F1E6-1F1EC"},
    {"bendera_anguilla", "1F1E6-1F1EE"},
    {"bendera_albania", "1F1E6-1F1F1"},
    {"bendera_armenia", "1F1E6-1F1F2"},
    {"bendera_angola", "1F1E6-1F1F4"},
    {"bendera_antartika", "1F1E6-1F1F6"},
    {"bendera_argentina", "1F1E6-1F1F7"},
    {"bendera_samoa_amerika", "1F1E6-1F1F8"},
    {"bendera_austria", "1F1E6-1F1F9"},
    {"bendera_australia", "1F1E6-1F1FA"},
    {"bendera_aruba", "1F1E6-1F1FC"},
    {"bendera_kepulauan_aland", "1F1E6-1F1FD"},
    {"bendera_azerbaijan", "1F1E6-1F1FF"},
    {"bendera_bosnia_dan_herzegovina", "1F1E7-1F1E6"},
    {"bendera_barbados", "1F1E7-1F1E7"},
    {"bendera_bangladesh", "1F1E7-1F1E9"},
    {"bendera_belgium", "1F1E7-1F1EA"},
    {"bendera_burkina_faso", "1F1E7-1F1EB"},
    {"bendera_bulgaria", "1F1E7-1F1EC"},
    {"bendera_bahrain", "1F1E7-1F1ED"},
    {"bendera_burundi", "1F1E7-1F1EE"},
    {"bendera_benin", "1F1E7-1F1EF"},
    {"bendera_st_barthelemy", "1F1E7-1F1F1"},
    {"bendera_bermuda", "1F1E7-1F1F2"},
    {"bendera_brunei", "1F1E7-1F1F3"},
    {"bendera_bolivia", "1F1E7-1F1F4"},
    {"bendera_belanda_caribbean", "1F1E7-1F1F6"},
    {"bendera_brazil", "1F1E7-1F1F7"},
    {"bendera_bahamas", "1F1E7-1F1F8"},
    {"bendera_bhutan", "1F1E7-1F1F9"},
    {"bendera_pulau_bouvet", "1F1E7-1F1FB"},
    {"bendera_botswana", "1F1E7-1F1FC"},
    {"bendera_belarus", "1F1E7-1F1FE"},
    {"bendera_belize", "1F1E7-1F1FF"},
    {"bendera_kanada", "1F1E8-1F1E6"},
    {"bendera_kepulauan_cocos_keeling", "1F1E8-1F1E8"},
    {"bendera_congo_kinshasa", "1F1E8-1F1E9"},
    {"bendera_republik_afrika_tengah", "1F1E8-1F1EB"},
    {"bendera_congo_brazzaville", "1F1E8-1F1EC"},
    {"bendera_switzerland", "1F1E8-1F1ED"},
    {"bendera_cote_divoire", "1F1E8-1F1EE"},
    {"bendera_kepulauan_cook", "1F1E8-1F1F0"},
    {"bendera_chile", "1F1E8-1F1F1"},
    {"bendera_cameroon", "1F1E8-1F1F2"},
    {"bendera_china", "1F1E8-1F1F3"},
    {"bendera_colombia", "1F1E8-1F1F4"},
    {"bendera_pulau_clipperton", "1F1E8-1F1F5"},
    {"bendera_sark", "1F1E8-1F1F6"},
    {"bendera_costa_rica", "1F1E8-1F1F7"},
    {"bendera_cuba", "1F1E8-1F1FA"},
    {"bendera_cape_verde", "1F1E8-1F1FB"},
    {"bendera_curacao", "1F1E8-1F1FC"},
    {"bendera_pulau_krismas", "1F1E8-1F1FD"},
    {"bendera_cyprus", "1F1E8-1F1FE"},
    {"bendera_czechia", "1F1E8-1F1FF"},
    {"bendera_jerman", "1F1E9-1F1EA"},
    {"bendera_diego_garcia", "1F1E9-1F1EC"},
    {"bendera_djibouti", "1F1E9-1F1EF"},
    {"bendera_denmark", "1F1E9-1F1F0"},
    {"bendera_dominica", "1F1E9-1F1F2"},
    {"bendera_republik_dominica", "1F1E9-1F1F4"},
    {"bendera_algeria", "1F1E9-1F1FF"},
    {"bendera_ceuta_dan_melilla", "1F1EA-1F1E6"},
    {"bendera_ecuador", "1F1EA-1F1E8"},
    {"bendera_estonia", "1F1EA-1F1EA"},
    {"bendera_mesir", "1F1EA-1F1EC"},
    {"bendera_sahara_barat", "1F1EA-1F1ED"},
    {"bendera_eritrea", "1F1EA-1F1F7"},
    {"bendera_sepanyol", "1F1EA-1F1F8"},
    {"bendera_ethiopia", "1F1EA-1F1F9"},
    {"bendera_kesatuan_eropah", "1F1EA-1F1FA"},
    {"bendera_finland", "1F1EB-1F1EE"},
    {"bendera_fiji", "1F1EB-1F1EF"},
    {"bendera_kepulauan_falkland", "1F1EB-1F1F0"},
    {"bendera_micronesia", "1F1EB-1F1F2"},
    {"bendera_kepulauan_faroe", "1F1EB-1F1F4"},
    {"bendera_perancis", "1F1EB-1F1F7"},
    {"bendera_gabon", "1F1EC-1F1E6"},
    {"bendera_united_kingdom", "1F1EC-1F1E7"},
    {"bendera_grenada", "1F1EC-1F1E9"},
    {"bendera_georgia", "1F1EC-1F1EA"},
    {"bendera_guiana_perancis", "1F1EC-1F1EB"},
    {"bendera_guernsey", "1F1EC-1F1EC"},
    {"bendera_ghana", "1F1EC-1F1ED"},
    {"bendera_gibraltar", "1F1EC-1F1EE"},
    {"bendera_greenland", "1F1EC-1F1F1"},
    {"bendera_gambia", "1F1EC-1F1F2"},
    {"bendera_guinea", "1F1EC-1F1F3"},
    {"bendera_guadeloupe", "1F1EC-1F1F5"},
    {"bendera_guinea_khatulistiwa", "1F1EC-1F1F6"},
    {"bendera_greece", "1F1EC-1F1F7"},
    {"bendera_kepulauan_georgia_selatan_sandwich_selatan", "1F1EC-1F1F8"},
    {"bendera_guatemala", "1F1EC-1F1F9"},
    {"bendera_guam", "1F1EC-1F1FA"},
    {"bendera_guinea_bissau", "1F1EC-1F1FC"},
    {"bendera_guyana", "1F1EC-1F1FE"},
    {"bendera_hong_kong_sar_china", "1F1ED-1F1F0"},
    {"bendera_kepulauan_heard_mcdonald", "1F1ED-1F1F2"},
    {"bendera_honduras", "1F1ED-1F1F3"},
    {"bendera_croatia", "1F1ED-1F1F7"},
    {"bendera_haiti", "1F1ED-1F1F9"},
    {"bendera_hungary", "1F1ED-1F1FA"},
    {"bendera_kepulauan_canary", "1F1EE-1F1E8"},
    {"bendera_indonesia", "1F1EE-1F1E9"},
    {"bendera_ireland", "1F1EE-1F1EA"},
    {"bendera_israel", "1F1EE-1F1F1"},
    {"bendera_isle_of_man", "1F1EE-1F1F2"},
    {"bendera_india", "1F1EE-1F1F3"},
    {"bendera_wilayah_lautan_hindi_british", "1F1EE-1F1F4"},
    {"bendera_iraq", "1F1EE-1F1F6"},
    {"bendera_iran", "1F1EE-1F1F7"},
    {"bendera_iceland", "1F1EE-1F1F8"},
    {"bendera_itali", "1F1EE-1F1F9"},
    {"bendera_jersey", "1F1EF-1F1EA"},
    {"bendera_jamaica", "1F1EF-1F1F2"},
    {"bendera_jordan", "1F1EF-1F1F4"},
    {"bendera_jepun", "1F1EF-1F1F5"},
    {"bendera_kenya", "1F1F0-1F1EA"},
    {"bendera_kyrgyzstan", "1F1F0-1F1EC"},
    {"bendera_kemboja", "1F1F0-1F1ED"},
    {"bendera_kiribati", "1F1F0-1F1EE"},
    {"bendera_comoros", "1F1F0-1F1F2"},
    {"bendera_saint_kitts_dan_nevis", "1F1F0-1F1F3"},
    {"bendera_korea_utara", "1F1F0-1F1F5"},
    {"bendera_korea_selatan", "1F1F0-1F1F7"},
    {"bendera_kuwait", "1F1F0-1F1FC"},
    {"bendera_kepulauan_cayman", "1F1F0-1F1FE"},
    {"bendera_kazakhstan", "1F1F0-1F1FF"},
    {"bendera_laos", "1F1F1-1F1E6"},
    {"bendera_lubnan", "1F1F1-1F1E7"},
    {"bendera_saint_lucia", "1F1F1-1F1E8"},
    {"bendera_liechtenstein", "1F1F1-1F1EE"},
    {"bendera_sri_lanka", "1F1F1-1F1F0"},
    {"bendera_liberia", "1F1F1-1F1F7"},
    {"bendera_lesotho", "1F1F1-1F1F8"},
    {"bendera_lithuania", "1F1F1-1F1F9"},
    {"bendera_luxembourg", "1F1F1-1F1FA"},
    {"bendera_latvia", "1F1F1-1F1FB"},
    {"bendera_libya", "1F1F1-1F1FE"},
    {"bendera_maghribi", "1F1F2-1F1E6"},
    {"bendera_monaco", "1F1F2-1F1E8"},
    {"bendera_moldova", "1F1F2-1F1E9"},
    {"bendera_montenegro", "1F1F2-1F1EA"},
    {"bendera_saint_martin", "1F1F2-1F1EB"},
    {"bendera_madagaskar", "1F1F2-1F1EC"},
    {"bendera_kepulauan_marshall", "1F1F2-1F1ED"},
    {"bendera_macedonia_utara", "1F1F2-1F1F0"},
    {"bendera_mali", "1F1F2-1F1F1"},
    {"bendera_myanmar_burma", "1F1F2-1F1F2"},
    {"bendera_mongolia", "1F1F2-1F1F3"},
    {"bendera_macau_sar_china", "1F1F2-1F1F4"},
    {"bendera_kepulauan_mariana_utara", "1F1F2-1F1F5"},
    {"bendera_martinique", "1F1F2-1F1F6"},
    {"bendera_mauritania", "1F1F2-1F1F7"},
    {"bendera_montserrat", "1F1F2-1F1F8"},
    {"bendera_malta", "1F1F2-1F1F9"},
    {"bendera_mauritius", "1F1F2-1F1FA"},
    {"bendera_maldives", "1F1F2-1F1FB"},
    {"bendera_malawi", "1F1F2-1F1FC"},
    {"bendera_mexico", "1F1F2-1F1FD"},
    {"bendera_malaysia", "1F1F2-1F1FE"},
    {"bendera_mozambique", "1F1F2-1F1FF"},
    {"bendera_namibia", "1F1F3-1F1E6"},
    {"bendera_new_caledonia", "1F1F3-1F1E8"},
    {"bendera_niger", "1F1F3-1F1EA"},
    {"bendera_pulau_norfolk", "1F1F3-1F1EB"},
    {"bendera_nigeria", "1F1F3-1F1EC"},
    {"bendera_nicaragua", "1F1F3-1F1EE"},
    {"bendera_belanda", "1F1F3-1F1F1"},
    {"bendera_norway", "1F1F3-1F1F4"},
    {"bendera_nepal", "1F1F3-1F1F5"},
    {"bendera_nauru", "1F1F3-1F1F7"},
    {"bendera_niue", "1F1F3-1F1FA"},
    {"bendera_new_zealand", "1F1F3-1F1FF"},
    {"bendera_oman", "1F1F4-1F1F2"},
    {"bendera_panama", "1F1F5-1F1E6"},
    {"bendera_peru", "1F1F5-1F1EA"},
    {"bendera_polinesia_perancis", "1F1F5-1F1EB"},
    {"bendera_papua_new_guinea", "1F1F5-1F1EC"},
    {"bendera_filipina", "1F1F5-1F1ED"},
    {"bendera_pakistan", "1F1F5-1F1F0"},
    {"bendera_poland", "1F1F5-1F1F1"},
    {"bendera_saint_pierre_dan_miquelon", "1F1F5-1F1F2"},
    {"bendera_kepulauan_pitcairn", "1F1F5-1F1F3"},
    {"bendera_puerto_rico", "1F1F5-1F1F7"},
    {"bendera_wilayah_palestin", "1F1F5-1F1F8"},
    {"bendera_portugal", "1F1F5-1F1F9"},
    {"bendera_palau", "1F1F5-1F1FC"},
    {"bendera_paraguay", "1F1F5-1F1FE"},
    {"bendera_qatar", "1F1F6-1F1E6"},
    {"bendera_reunion", "1F1F7-1F1EA"},
    {"bendera_romania", "1F1F7-1F1F4"},
    {"bendera_serbia", "1F1F7-1F1F8"},
    {"bendera_rusia", "1F1F7-1F1FA"},
    {"bendera_rwanda", "1F1F7-1F1FC"},
    {"bendera_arab_saudi", "1F1F8-1F1E6"},
    {"bendera_kepulauan_solomon", "1F1F8-1F1E7"},
    {"bendera_seychelles", "1F1F8-1F1E8"},
    {"bendera_sudan", "1F1F8-1F1E9"},
    {"bendera_sweden", "1F1F8-1F1EA"},
    {"bendera_singapura", "1F1F8-1F1EC"},
    {"bendera_saint_helena", "1F1F8-1F1ED"},
    {"bendera_slovenia", "1F1F8-1F1EE"},
    {"bendera_svalbard_dan_jan_mayen", "1F1F8-1F1EF"},
    {"bendera_slovakia", "1F1F8-1F1F0"},
    {"bendera_sierra_leone", "1F1F8-1F1F1"},
    {"bendera_san_marino", "1F1F8-1F1F2"},
    {"bendera_senegal", "1F1F8-1F1F3"},
    {"bendera_somalia", "1F1F8-1F1F4"},
    {"bendera_surinam", "1F1F8-1F1F7"},
    {"bendera_sudan_selatan", "1F1F8-1F1F8"},
    {"bendera_sao_tome_dan_principe", "1F1F8-1F1F9"},
    {"bendera_el_salvador", "1F1F8-1F1FB"},
    {"bendera_sint_maarten", "1F1F8-1F1FD"},
    {"bendera_syria", "1F1F8-1F1FE"},
    {"bendera_eswatini", "1F1F8-1F1FF"},
    {"bendera_tristan_da_cunha", "1F1F9-1F1E6"},
    {"bendera_kepulauan_turks_dan_caicos", "1F1F9-1F1E8"},
    {"bendera_chad", "1F1F9-1F1E9"},
    {"bendera_wilayah_selatan_perancis", "1F1F9-1F1EB"},
    {"bendera_togo", "1F1F9-1F1EC"},
    {"bendera_thailand", "1F1F9-1F1ED"},
    {"bendera_tajikistan", "1F1F9-1F1EF"},
    {"bendera_tokelau", "1F1F9-1F1F0"},
    {"bendera_timor_leste", "1F1F9-1F1F1"},
    {"bendera_turkmenistan", "1F1F9-1F1F2"},
    {"bendera_tunisia", "1F1F9-1F1F3"},
    {"bendera_tonga", "1F1F9-1F1F4"},
    {"bendera_turkiye", "1F1F9-1F1F7"},
    {"bendera_trinidad_dan_tobago", "1F1F9-1F1F9"},
    {"bendera_tuvalu", "1F1F9-1F1FB"},
    {"bendera_taiwan", "1F1F9-1F1FC"},
    {"bendera_tanzania", "1F1F9-1F1FF"},
    {"bendera_ukraine", "1F1FA-1F1E6"},
    {"bendera_uganda", "1F1FA-1F1EC"},
    {"bendera_kepulauan_terpencil_as", "1F1FA-1F1F2"},
    {"bendera_bangsa_bangsa_bersatu", "1F1FA-1F1F3"},
    {"bendera_amerika_syarikat", "1F1FA-1F1F8"},
    {"bendera_uruguay", "1F1FA-1F1FE"},
    {"bendera_uzbekistan", "1F1FA-1F1FF"},
    {"bendera_kota_vatican", "1F1FB-1F1E6"},
    {"bendera_saint_vincent_dan_grenadines", "1F1FB-1F1E8"},
    {"bendera_venezuela", "1F1FB-1F1EA"},
    {"bendera_kepulauan_virgin_british", "1F1FB-1F1EC"},
    {"bendera_kepulauan_virgin_as", "1F1FB-1F1EE"},
    {"bendera_vietnam", "1F1FB-1F1F3"},
    {"bendera_vanuatu", "1F1FB-1F1FA"},
    {"bendera_wallis_dan_futuna", "1F1FC-1F1EB"},
    {"bendera_samoa", "1F1FC-1F1F8"},
    {"bendera_kosovo", "1F1FD-1F1F0"},
    {"bendera_yaman", "1F1FE-1F1EA"},
    {"bendera_mayotte", "1F1FE-1F1F9"},
    {"bendera_afrika_selatan", "1F1FF-1F1E6"},
    {"bendera_zambia", "1F1FF-1F1F2"},
    {"bendera_zimbabwe", "1F1FF-1F1FC"},
    {"bendera_england", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"bendera_scotland", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"bendera_wales", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki", "1F468-200D-2764-FE0F-200D-1F468"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pasangan_dengan_simbol_hati_lelaki_dan_lelaki_warna5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"cium_lelaki_dan_lelaki", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"cium_lelaki_dan_lelaki_warna1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cium_lelaki_dan_lelaki_warna1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cium_lelaki_dan_lelaki_warna1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cium_lelaki_dan_lelaki_warna1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cium_lelaki_dan_lelaki_warna1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cium_lelaki_dan_lelaki_warna2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cium_lelaki_dan_lelaki_warna2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cium_lelaki_dan_lelaki_warna2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cium_lelaki_dan_lelaki_warna2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cium_lelaki_dan_lelaki_warna2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cium_lelaki_dan_lelaki_warna3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cium_lelaki_dan_lelaki_warna3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cium_lelaki_dan_lelaki_warna3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cium_lelaki_dan_lelaki_warna3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cium_lelaki_dan_lelaki_warna3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cium_lelaki_dan_lelaki_warna4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cium_lelaki_dan_lelaki_warna4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cium_lelaki_dan_lelaki_warna4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cium_lelaki_dan_lelaki_warna4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cium_lelaki_dan_lelaki_warna4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cium_lelaki_dan_lelaki_warna5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cium_lelaki_dan_lelaki_warna5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cium_lelaki_dan_lelaki_warna5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cium_lelaki_dan_lelaki_warna5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cium_lelaki_dan_lelaki_warna5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"keluarga_lelaki_dan_kanak_kanak_lelaki", "1F468-200D-1F466"},
    {"keluarga_lelaki_kanak_kanak_lelaki_dan_kanak_kanak_lelaki", "1F468-200D-1F466-200D-1F466"},
    {"keluarga_lelaki_dan_budak_perempuan", "1F468-200D-1F467"},
    {"keluarga_lelaki_budak_perempuan_dan_kanak_kanak_lelaki", "1F468-200D-1F467-200D-1F466"},
    {"keluarga_lelaki_budak_perempuan_dan_budak_perempuan", "1F468-200D-1F467-200D-1F467"},
    {"keluarga_lelaki_lelaki_dan_kanak_kanak_lelaki", "1F468-200D-1F468-200D-1F466"},
    {"keluarga_lelaki_lelaki_kanak_kanak_lelaki_dan_kanak_kanak_lelaki", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"keluarga_lelaki_lelaki_dan_budak_perempuan", "1F468-200D-1F468-200D-1F467"},
    {"keluarga_lelaki_lelaki_budak_perempuan_dan_kanak_kanak_lelaki", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"keluarga_lelaki_lelaki_budak_perempuan_dan_budak_perempuan", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"keluarga_lelaki_perempuan_dan_kanak_kanak_lelaki", "1F468-200D-1F469-200D-1F466"},
    {"keluarga_lelaki_perempuan_kanak_kanak_lelaki_dan_kanak_kanak_lelaki", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"keluarga_lelaki_perempuan_dan_budak_perempuan", "1F468-200D-1F469-200D-1F467"},
    {"keluarga_lelaki_perempuan_budak_perempuan_dan_kanak_kanak_lelaki", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"keluarga_lelaki_perempuan_budak_perempuan_dan_budak_perempuan", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki", "1F469-200D-2764-FE0F-200D-1F468"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_lelaki_warna5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan", "1F469-200D-2764-FE0F-200D-1F469"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pasangan_dengan_simbol_hati_perempuan_dan_perempuan_warna5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"cium_perempuan_dan_lelaki", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"cium_perempuan_dan_lelaki_warna1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cium_perempuan_dan_lelaki_warna1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cium_perempuan_dan_lelaki_warna1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cium_perempuan_dan_lelaki_warna1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cium_perempuan_dan_lelaki_warna1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cium_perempuan_dan_lelaki_warna2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cium_perempuan_dan_lelaki_warna2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cium_perempuan_dan_lelaki_warna2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cium_perempuan_dan_lelaki_warna2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cium_perempuan_dan_lelaki_warna2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cium_perempuan_dan_lelaki_warna3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cium_perempuan_dan_lelaki_warna3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cium_perempuan_dan_lelaki_warna3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cium_perempuan_dan_lelaki_warna3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cium_perempuan_dan_lelaki_warna3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cium_perempuan_dan_lelaki_warna4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cium_perempuan_dan_lelaki_warna4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cium_perempuan_dan_lelaki_warna4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cium_perempuan_dan_lelaki_warna4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cium_perempuan_dan_lelaki_warna4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cium_perempuan_dan_lelaki_warna5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"cium_perempuan_dan_lelaki_warna5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"cium_perempuan_dan_lelaki_warna5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"cium_perempuan_dan_lelaki_warna5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"cium_perempuan_dan_lelaki_warna5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"cium_perempuan_dan_perempuan", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"cium_perempuan_dan_perempuan_warna1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"cium_perempuan_dan_perempuan_warna1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"cium_perempuan_dan_perempuan_warna1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"cium_perempuan_dan_perempuan_warna1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"cium_perempuan_dan_perempuan_warna1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"cium_perempuan_dan_perempuan_warna2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"cium_perempuan_dan_perempuan_warna2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"cium_perempuan_dan_perempuan_warna2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"cium_perempuan_dan_perempuan_warna2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"cium_perempuan_dan_perempuan_warna2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"cium_perempuan_dan_perempuan_warna3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"cium_perempuan_dan_perempuan_warna3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"cium_perempuan_dan_perempuan_warna3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"cium_perempuan_dan_perempuan_warna3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"cium_perempuan_dan_perempuan_warna3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"cium_perempuan_dan_perempuan_warna4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"cium_perempuan_dan_perempuan_warna4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"cium_perempuan_dan_perempuan_warna4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"cium_perempuan_dan_perempuan_warna4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"cium_perempuan_dan_perempuan_warna4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"cium_perempuan_dan_perempuan_warna5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"cium_perempuan_dan_perempuan_warna5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"cium_perempuan_dan_perempuan_warna5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"cium_perempuan_dan_perempuan_warna5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"cium_perempuan_dan_perempuan_warna5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"keluarga_perempuan_dan_kanak_kanak_lelaki", "1F469-200D-1F466"},
    {"keluarga_perempuan_kanak_kanak_lelaki_dan_kanak_kanak_lelaki", "1F469-200D-1F466-200D-1F466"},
    {"keluarga_perempuan_dan_budak_perempuan", "1F469-200D-1F467"},
    {"keluarga_perempuan_budak_perempuan_dan_kanak_kanak_lelaki", "1F469-200D-1F467-200D-1F466"},
    {"keluarga_perempuan_budak_perempuan_dan_budak_perempuan", "1F469-200D-1F467-200D-1F467"},
    {"keluarga_perempuan_perempuan_dan_kanak_kanak_lelaki", "1F469-200D-1F469-200D-1F466"},
    {"keluarga_perempuan_perempuan_kanak_kanak_lelaki_dan_kanak_kanak_lelaki", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"keluarga_perempuan_perempuan_dan_budak_perempuan", "1F469-200D-1F469-200D-1F467"},
    {"keluarga_perempuan_perempuan_budak_perempuan_dan_kanak_kanak_lelaki", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"keluarga_perempuan_perempuan_budak_perempuan_dan_budak_perempuan", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"orang_berpegangan_tangan", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"orang_berpegangan_tangan_warna1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"orang_berpegangan_tangan_warna1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"orang_berpegangan_tangan_warna1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"orang_berpegangan_tangan_warna1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"orang_berpegangan_tangan_warna1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"orang_berpegangan_tangan_warna2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"orang_berpegangan_tangan_warna2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"orang_berpegangan_tangan_warna2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"orang_berpegangan_tangan_warna2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"orang_berpegangan_tangan_warna2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"orang_berpegangan_tangan_warna3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"orang_berpegangan_tangan_warna3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"orang_berpegangan_tangan_warna3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"orang_berpegangan_tangan_warna3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"orang_berpegangan_tangan_warna3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"orang_berpegangan_tangan_warna4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"orang_berpegangan_tangan_warna4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"orang_berpegangan_tangan_warna4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"orang_berpegangan_tangan_warna4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"orang_berpegangan_tangan_warna4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"orang_berpegangan_tangan_warna5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"orang_berpegangan_tangan_warna5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"orang_berpegangan_tangan_warna5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"orang_berpegangan_tangan_warna5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"orang_berpegangan_tangan_warna5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"keluarga_dewasa_dewasa_kanak_kanak", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"keluarga_dewasa_dewasa_kanak_kanak_kanak_kanak", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"keluarga_dewasa_kanak_kanak", "1F9D1-200D-1F9D2"},
    {"keluarga_dewasa_kanak_kanak_kanak_kanak", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"pelari_menghadap_kanan", "1F3C3-200D-27A1-FE0F"},
    {"pelari_menghadap_kanan_warna1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"pelari_menghadap_kanan_warna2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"pelari_menghadap_kanan_warna3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"pelari_menghadap_kanan_warna4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"pelari_menghadap_kanan_warna5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"pekerja_kesihatan_lelaki", "1F468-200D-2695-FE0F"},
    {"pekerja_kesihatan_lelaki_warna1", "1F468-1F3FB-200D-2695-FE0F"},
    {"pekerja_kesihatan_lelaki_warna2", "1F468-1F3FC-200D-2695-FE0F"},
    {"pekerja_kesihatan_lelaki_warna3", "1F468-1F3FD-200D-2695-FE0F"},
    {"pekerja_kesihatan_lelaki_warna4", "1F468-1F3FE-200D-2695-FE0F"},
    {"pekerja_kesihatan_lelaki_warna5", "1F468-1F3FF-200D-2695-FE0F"},
    {"hakim_lelaki", "1F468-200D-2696-FE0F"},
    {"hakim_lelaki_warna1", "1F468-1F3FB-200D-2696-FE0F"},
    {"hakim_lelaki_warna2", "1F468-1F3FC-200D-2696-FE0F"},
    {"hakim_lelaki_warna3", "1F468-1F3FD-200D-2696-FE0F"},
    {"hakim_lelaki_warna4", "1F468-1F3FE-200D-2696-FE0F"},
    {"hakim_lelaki_warna5", "1F468-1F3FF-200D-2696-FE0F"},
    {"juruterbang_lelaki", "1F468-200D-2708-FE0F"},
    {"juruterbang_lelaki_warna1", "1F468-1F3FB-200D-2708-FE0F"},
    {"juruterbang_lelaki_warna2", "1F468-1F3FC-200D-2708-FE0F"},
    {"juruterbang_lelaki_warna3", "1F468-1F3FD-200D-2708-FE0F"},
    {"juruterbang_lelaki_warna4", "1F468-1F3FE-200D-2708-FE0F"},
    {"juruterbang_lelaki_warna5", "1F468-1F3FF-200D-2708-FE0F"},
    {"petani_lelaki", "1F468-200D-1F33E"},
    {"petani_lelaki_warna1", "1F468-1F3FB-200D-1F33E"},
    {"petani_lelaki_warna2", "1F468-1F3FC-200D-1F33E"},
    {"petani_lelaki_warna3", "1F468-1F3FD-200D-1F33E"},
    {"petani_lelaki_warna4", "1F468-1F3FE-200D-1F33E"},
    {"petani_lelaki_warna5", "1F468-1F3FF-200D-1F33E"},
    {"tukang_masak_lelaki", "1F468-200D-1F373"},
    {"tukang_masak_lelaki_warna1", "1F468-1F3FB-200D-1F373"},
    {"tukang_masak_lelaki_warna2", "1F468-1F3FC-200D-1F373"},
    {"tukang_masak_lelaki_warna3", "1F468-1F3FD-200D-1F373"},
    {"tukang_masak_lelaki_warna4", "1F468-1F3FE-200D-1F373"},
    {"tukang_masak_lelaki_warna5", "1F468-1F3FF-200D-1F373"},
    {"lelaki_menyusui_bayi", "1F468-200D-1F37C"},
    {"lelaki_menyusui_bayi_warna1", "1F468-1F3FB-200D-1F37C"},
    {"lelaki_menyusui_bayi_warna2", "1F468-1F3FC-200D-1F37C"},
    {"lelaki_menyusui_bayi_warna3", "1F468-1F3FD-200D-1F37C"},
    {"lelaki_menyusui_bayi_warna4", "1F468-1F3FE-200D-1F37C"},
    {"lelaki_menyusui_bayi_warna5", "1F468-1F3FF-200D-1F37C"},
    {"pelajar_lelaki", "1F468-200D-1F393"},
    {"pelajar_lelaki_warna1", "1F468-1F3FB-200D-1F393"},
    {"pelajar_lelaki_warna2", "1F468-1F3FC-200D-1F393"},
    {"pelajar_lelaki_warna3", "1F468-1F3FD-200D-1F393"},
    {"pelajar_lelaki_warna4", "1F468-1F3FE-200D-1F393"},
    {"pelajar_lelaki_warna5", "1F468-1F3FF-200D-1F393"},
    {"penyanyi_lelaki", "1F468-200D-1F3A4"},
    {"penyanyi_lelaki_warna1", "1F468-1F3FB-200D-1F3A4"},
    {"penyanyi_lelaki_warna2", "1F468-1F3FC-200D-1F3A4"},
    {"penyanyi_lelaki_warna3", "1F468-1F3FD-200D-1F3A4"},
    {"penyanyi_lelaki_warna4", "1F468-1F3FE-200D-1F3A4"},
    {"penyanyi_lelaki_warna5", "1F468-1F3FF-200D-1F3A4"},
    {"pelukis_lelaki", "1F468-200D-1F3A8"},
    {"pelukis_lelaki_warna1", "1F468-1F3FB-200D-1F3A8"},
    {"pelukis_lelaki_warna2", "1F468-1F3FC-200D-1F3A8"},
    {"pelukis_lelaki_warna3", "1F468-1F3FD-200D-1F3A8"},
    {"pelukis_lelaki_warna4", "1F468-1F3FE-200D-1F3A8"},
    {"pelukis_lelaki_warna5", "1F468-1F3FF-200D-1F3A8"},
    {"guru_lelaki", "1F468-200D-1F3EB"},
    {"guru_lelaki_warna1", "1F468-1F3FB-200D-1F3EB"},
    {"guru_lelaki_warna2", "1F468-1F3FC-200D-1F3EB"},
    {"guru_lelaki_warna3", "1F468-1F3FD-200D-1F3EB"},
    {"guru_lelaki_warna4", "1F468-1F3FE-200D-1F3EB"},
    {"guru_lelaki_warna5", "1F468-1F3FF-200D-1F3EB"},
    {"pekerja_kilang_lelaki", "1F468-200D-1F3ED"},
    {"pekerja_kilang_lelaki_warna1", "1F468-1F3FB-200D-1F3ED"},
    {"pekerja_kilang_lelaki_warna2", "1F468-1F3FC-200D-1F3ED"},
    {"pekerja_kilang_lelaki_warna3", "1F468-1F3FD-200D-1F3ED"},
    {"pekerja_kilang_lelaki_warna4", "1F468-1F3FE-200D-1F3ED"},
    {"pekerja_kilang_lelaki_warna5", "1F468-1F3FF-200D-1F3ED"},
    {"ahli_teknologi_lelaki", "1F468-200D-1F4BB"},
    {"ahli_teknologi_lelaki_warna1", "1F468-1F3FB-200D-1F4BB"},
    {"ahli_teknologi_lelaki_warna2", "1F468-1F3FC-200D-1F4BB"},
    {"ahli_teknologi_lelaki_warna3", "1F468-1F3FD-200D-1F4BB"},
    {"ahli_teknologi_lelaki_warna4", "1F468-1F3FE-200D-1F4BB"},
    {"ahli_teknologi_lelaki_warna5", "1F468-1F3FF-200D-1F4BB"},
    {"pekerja_pejabat_lelaki", "1F468-200D-1F4BC"},
    {"pekerja_pejabat_lelaki_warna1", "1F468-1F3FB-200D-1F4BC"},
    {"pekerja_pejabat_lelaki_warna2", "1F468-1F3FC-200D-1F4BC"},
    {"pekerja_pejabat_lelaki_warna3", "1F468-1F3FD-200D-1F4BC"},
    {"pekerja_pejabat_lelaki_warna4", "1F468-1F3FE-200D-1F4BC"},
    {"pekerja_pejabat_lelaki_warna5", "1F468-1F3FF-200D-1F4BC"},
    {"mekanik_lelaki", "1F468-200D-1F527"},
    {"mekanik_lelaki_warna1", "1F468-1F3FB-200D-1F527"},
    {"mekanik_lelaki_warna2", "1F468-1F3FC-200D-1F527"},
    {"mekanik_lelaki_warna3", "1F468-1F3FD-200D-1F527"},
    {"mekanik_lelaki_warna4", "1F468-1F3FE-200D-1F527"},
    {"mekanik_lelaki_warna5", "1F468-1F3FF-200D-1F527"},
    {"saintis_lelaki", "1F468-200D-1F52C"},
    {"saintis_lelaki_warna1", "1F468-1F3FB-200D-1F52C"},
    {"saintis_lelaki_warna2", "1F468-1F3FC-200D-1F52C"},
    {"saintis_lelaki_warna3", "1F468-1F3FD-200D-1F52C"},
    {"saintis_lelaki_warna4", "1F468-1F3FE-200D-1F52C"},
    {"saintis_lelaki_warna5", "1F468-1F3FF-200D-1F52C"},
    {"angkasawan_lelaki", "1F468-200D-1F680"},
    {"angkasawan_lelaki_warna1", "1F468-1F3FB-200D-1F680"},
    {"angkasawan_lelaki_warna2", "1F468-1F3FC-200D-1F680"},
    {"angkasawan_lelaki_warna3", "1F468-1F3FD-200D-1F680"},
    {"angkasawan_lelaki_warna4", "1F468-1F3FE-200D-1F680"},
    {"angkasawan_lelaki_warna5", "1F468-1F3FF-200D-1F680"},
    {"ahli_bomba_lelaki", "1F468-200D-1F692"},
    {"ahli_bomba_lelaki_warna1", "1F468-1F3FB-200D-1F692"},
    {"ahli_bomba_lelaki_warna2", "1F468-1F3FC-200D-1F692"},
    {"ahli_bomba_lelaki_warna3", "1F468-1F3FD-200D-1F692"},
    {"ahli_bomba_lelaki_warna4", "1F468-1F3FE-200D-1F692"},
    {"ahli_bomba_lelaki_warna5", "1F468-1F3FF-200D-1F692"},
    {"lelaki_dengan_tongkat_putih", "1F468-200D-1F9AF"},
    {"lelaki_dengan_tongkat_putih_warna1", "1F468-1F3FB-200D-1F9AF"},
    {"lelaki_dengan_tongkat_putih_warna2", "1F468-1F3FC-200D-1F9AF"},
    {"lelaki_dengan_tongkat_putih_warna3", "1F468-1F3FD-200D-1F9AF"},
    {"lelaki_dengan_tongkat_putih_warna4", "1F468-1F3FE-200D-1F9AF"},
    {"lelaki_dengan_tongkat_putih_warna5", "1F468-1F3FF-200D-1F9AF"},
    {"lelaki_dengan_tongkat_putih_menghadap_kanan", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"lelaki_dengan_tongkat_putih_menghadap_kanan_warna1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"lelaki_dengan_tongkat_putih_menghadap_kanan_warna2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"lelaki_dengan_tongkat_putih_menghadap_kanan_warna3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"lelaki_dengan_tongkat_putih_menghadap_kanan_warna4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"lelaki_dengan_tongkat_putih_menghadap_kanan_warna5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"lelaki_dalam_kerusi_roda_bermotor", "1F468-200D-1F9BC"},
    {"lelaki_dalam_kerusi_roda_bermotor_warna1", "1F468-1F3FB-200D-1F9BC"},
    {"lelaki_dalam_kerusi_roda_bermotor_warna2", "1F468-1F3FC-200D-1F9BC"},
    {"lelaki_dalam_kerusi_roda_bermotor_warna3", "1F468-1F3FD-200D-1F9BC"},
    {"lelaki_dalam_kerusi_roda_bermotor_warna4", "1F468-1F3FE-200D-1F9BC"},
    {"lelaki_dalam_kerusi_roda_bermotor_warna5", "1F468-1F3FF-200D-1F9BC"},
    {"lelaki_dalam_kerusi_roda_bermotor_menghadap_kanan", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"lelaki_dalam_kerusi_roda_bermotor_menghadap_kanan_warna1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"lelaki_dalam_kerusi_roda_bermotor_menghadap_kanan_warna2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"lelaki_dalam_kerusi_roda_bermotor_menghadap_kanan_warna3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"lelaki_dalam_kerusi_roda_bermotor_menghadap_kanan_warna4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"lelaki_dalam_kerusi_roda_bermotor_menghadap_kanan_warna5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"lelaki_dalam_kerusi_roda_manual", "1F468-200D-1F9BD"},
    {"lelaki_dalam_kerusi_roda_manual_warna1", "1F468-1F3FB-200D-1F9BD"},
    {"lelaki_dalam_kerusi_roda_manual_warna2", "1F468-1F3FC-200D-1F9BD"},
    {"lelaki_dalam_kerusi_roda_manual_warna3", "1F468-1F3FD-200D-1F9BD"},
    {"lelaki_dalam_kerusi_roda_manual_warna4", "1F468-1F3FE-200D-1F9BD"},
    {"lelaki_dalam_kerusi_roda_manual_warna5", "1F468-1F3FF-200D-1F9BD"},
    {"lelaki_dalam_kerusi_roda_manual_menghadap_kanan", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"lelaki_dalam_kerusi_roda_manual_menghadap_kanan_warna1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"lelaki_dalam_kerusi_roda_manual_menghadap_kanan_warna2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"lelaki_dalam_kerusi_roda_manual_menghadap_kanan_warna3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"lelaki_dalam_kerusi_roda_manual_menghadap_kanan_warna4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"lelaki_dalam_kerusi_roda_manual_menghadap_kanan_warna5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"pekerja_kesihatan_wanita", "1F469-200D-2695-FE0F"},
    {"pekerja_kesihatan_wanita_warna1", "1F469-1F3FB-200D-2695-FE0F"},
    {"pekerja_kesihatan_wanita_warna2", "1F469-1F3FC-200D-2695-FE0F"},
    {"pekerja_kesihatan_wanita_warna3", "1F469-1F3FD-200D-2695-FE0F"},
    {"pekerja_kesihatan_wanita_warna4", "1F469-1F3FE-200D-2695-FE0F"},
    {"pekerja_kesihatan_wanita_warna5", "1F469-1F3FF-200D-2695-FE0F"},
    {"hakim_wanita", "1F469-200D-2696-FE0F"},
    {"hakim_wanita_warna1", "1F469-1F3FB-200D-2696-FE0F"},
    {"hakim_wanita_warna2", "1F469-1F3FC-200D-2696-FE0F"},
    {"hakim_wanita_warna3", "1F469-1F3FD-200D-2696-FE0F"},
    {"hakim_wanita_warna4", "1F469-1F3FE-200D-2696-FE0F"},
    {"hakim_wanita_warna5", "1F469-1F3FF-200D-2696-FE0F"},
    {"juruterbang_wanita", "1F469-200D-2708-FE0F"},
    {"juruterbang_wanita_warna1", "1F469-1F3FB-200D-2708-FE0F"},
    {"juruterbang_wanita_warna2", "1F469-1F3FC-200D-2708-FE0F"},
    {"juruterbang_wanita_warna3", "1F469-1F3FD-200D-2708-FE0F"},
    {"juruterbang_wanita_warna4", "1F469-1F3FE-200D-2708-FE0F"},
    {"juruterbang_wanita_warna5", "1F469-1F3FF-200D-2708-FE0F"},
    {"petani_wanita", "1F469-200D-1F33E"},
    {"petani_wanita_warna1", "1F469-1F3FB-200D-1F33E"},
    {"petani_wanita_warna2", "1F469-1F3FC-200D-1F33E"},
    {"petani_wanita_warna3", "1F469-1F3FD-200D-1F33E"},
    {"petani_wanita_warna4", "1F469-1F3FE-200D-1F33E"},
    {"petani_wanita_warna5", "1F469-1F3FF-200D-1F33E"},
    {"tukang_masak_wanita", "1F469-200D-1F373"},
    {"tukang_masak_wanita_warna1", "1F469-1F3FB-200D-1F373"},
    {"tukang_masak_wanita_warna2", "1F469-1F3FC-200D-1F373"},
    {"tukang_masak_wanita_warna3", "1F469-1F3FD-200D-1F373"},
    {"tukang_masak_wanita_warna4", "1F469-1F3FE-200D-1F373"},
    {"tukang_masak_wanita_warna5", "1F469-1F3FF-200D-1F373"},
    {"wanita_menyusui_bayi", "1F469-200D-1F37C"},
    {"wanita_menyusui_bayi_warna1", "1F469-1F3FB-200D-1F37C"},
    {"wanita_menyusui_bayi_warna2", "1F469-1F3FC-200D-1F37C"},
    {"wanita_menyusui_bayi_warna3", "1F469-1F3FD-200D-1F37C"},
    {"wanita_menyusui_bayi_warna4", "1F469-1F3FE-200D-1F37C"},
    {"wanita_menyusui_bayi_warna5", "1F469-1F3FF-200D-1F37C"},
    {"pelajar_wanita", "1F469-200D-1F393"},
    {"pelajar_wanita_warna1", "1F469-1F3FB-200D-1F393"},
    {"pelajar_wanita_warna2", "1F469-1F3FC-200D-1F393"},
    {"pelajar_wanita_warna3", "1F469-1F3FD-200D-1F393"},
    {"pelajar_wanita_warna4", "1F469-1F3FE-200D-1F393"},
    {"pelajar_wanita_warna5", "1F469-1F3FF-200D-1F393"},
    {"penyanyi_wanita", "1F469-200D-1F3A4"},
    {"penyanyi_wanita_warna1", "1F469-1F3FB-200D-1F3A4"},
    {"penyanyi_wanita_warna2", "1F469-1F3FC-200D-1F3A4"},
    {"penyanyi_wanita_warna3", "1F469-1F3FD-200D-1F3A4"},
    {"penyanyi_wanita_warna4", "1F469-1F3FE-200D-1F3A4"},
    {"penyanyi_wanita_warna5", "1F469-1F3FF-200D-1F3A4"},
    {"pelukis_wanita", "1F469-200D-1F3A8"},
    {"pelukis_wanita_warna1", "1F469-1F3FB-200D-1F3A8"},
    {"pelukis_wanita_warna2", "1F469-1F3FC-200D-1F3A8"},
    {"pelukis_wanita_warna3", "1F469-1F3FD-200D-1F3A8"},
    {"pelukis_wanita_warna4", "1F469-1F3FE-200D-1F3A8"},
    {"pelukis_wanita_warna5", "1F469-1F3FF-200D-1F3A8"},
    {"guru_wanita", "1F469-200D-1F3EB"},
    {"guru_wanita_warna1", "1F469-1F3FB-200D-1F3EB"},
    {"guru_wanita_warna2", "1F469-1F3FC-200D-1F3EB"},
    {"guru_wanita_warna3", "1F469-1F3FD-200D-1F3EB"},
    {"guru_wanita_warna4", "1F469-1F3FE-200D-1F3EB"},
    {"guru_wanita_warna5", "1F469-1F3FF-200D-1F3EB"},
    {"pekerja_kilang_wanita", "1F469-200D-1F3ED"},
    {"pekerja_kilang_wanita_warna1", "1F469-1F3FB-200D-1F3ED"},
    {"pekerja_kilang_wanita_warna2", "1F469-1F3FC-200D-1F3ED"},
    {"pekerja_kilang_wanita_warna3", "1F469-1F3FD-200D-1F3ED"},
    {"pekerja_kilang_wanita_warna4", "1F469-1F3FE-200D-1F3ED"},
    {"pekerja_kilang_wanita_warna5", "1F469-1F3FF-200D-1F3ED"},
    {"ahli_teknologi_wanita", "1F469-200D-1F4BB"},
    {"ahli_teknologi_wanita_warna1", "1F469-1F3FB-200D-1F4BB"},
    {"ahli_teknologi_wanita_warna2", "1F469-1F3FC-200D-1F4BB"},
    {"ahli_teknologi_wanita_warna3", "1F469-1F3FD-200D-1F4BB"},
    {"ahli_teknologi_wanita_warna4", "1F469-1F3FE-200D-1F4BB"},
    {"ahli_teknologi_wanita_warna5", "1F469-1F3FF-200D-1F4BB"},
    {"pekerja_pejabat_wanita", "1F469-200D-1F4BC"},
    {"pekerja_pejabat_wanita_warna1", "1F469-1F3FB-200D-1F4BC"},
    {"pekerja_pejabat_wanita_warna2", "1F469-1F3FC-200D-1F4BC"},
    {"pekerja_pejabat_wanita_warna3", "1F469-1F3FD-200D-1F4BC"},
    {"pekerja_pejabat_wanita_warna4", "1F469-1F3FE-200D-1F4BC"},
    {"pekerja_pejabat_wanita_warna5", "1F469-1F3FF-200D-1F4BC"},
    {"mekanik_wanita", "1F469-200D-1F527"},
    {"mekanik_wanita_warna1", "1F469-1F3FB-200D-1F527"},
    {"mekanik_wanita_warna2", "1F469-1F3FC-200D-1F527"},
    {"mekanik_wanita_warna3", "1F469-1F3FD-200D-1F527"},
    {"mekanik_wanita_warna4", "1F469-1F3FE-200D-1F527"},
    {"mekanik_wanita_warna5", "1F469-1F3FF-200D-1F527"},
    {"saintis_wanita", "1F469-200D-1F52C"},
    {"saintis_wanita_warna1", "1F469-1F3FB-200D-1F52C"},
    {"saintis_wanita_warna2", "1F469-1F3FC-200D-1F52C"},
    {"saintis_wanita_warna3", "1F469-1F3FD-200D-1F52C"},
    {"saintis_wanita_warna4", "1F469-1F3FE-200D-1F52C"},
    {"saintis_wanita_warna5", "1F469-1F3FF-200D-1F52C"},
    {"angkasawati", "1F469-200D-1F680"},
    {"angkasawati_warna1", "1F469-1F3FB-200D-1F680"},
    {"angkasawati_warna2", "1F469-1F3FC-200D-1F680"},
    {"angkasawati_warna3", "1F469-1F3FD-200D-1F680"},
    {"angkasawati_warna4", "1F469-1F3FE-200D-1F680"},
    {"angkasawati_warna5", "1F469-1F3FF-200D-1F680"},
    {"ahli_bomba_wanita", "1F469-200D-1F692"},
    {"ahli_bomba_wanita_warna1", "1F469-1F3FB-200D-1F692"},
    {"ahli_bomba_wanita_warna2", "1F469-1F3FC-200D-1F692"},
    {"ahli_bomba_wanita_warna3", "1F469-1F3FD-200D-1F692"},
    {"ahli_bomba_wanita_warna4", "1F469-1F3FE-200D-1F692"},
    {"ahli_bomba_wanita_warna5", "1F469-1F3FF-200D-1F692"},
    {"perempuan_dengan_tongkat_putih", "1F469-200D-1F9AF"},
    {"perempuan_dengan_tongkat_putih_warna1", "1F469-1F3FB-200D-1F9AF"},
    {"perempuan_dengan_tongkat_putih_warna2", "1F469-1F3FC-200D-1F9AF"},
    {"perempuan_dengan_tongkat_putih_warna3", "1F469-1F3FD-200D-1F9AF"},
    {"perempuan_dengan_tongkat_putih_warna4", "1F469-1F3FE-200D-1F9AF"},
    {"perempuan_dengan_tongkat_putih_warna5", "1F469-1F3FF-200D-1F9AF"},
    {"perempuan_dengan_tongkat_putih_menghadap_kanan", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"perempuan_dengan_tongkat_putih_menghadap_kanan_warna1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"perempuan_dengan_tongkat_putih_menghadap_kanan_warna2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"perempuan_dengan_tongkat_putih_menghadap_kanan_warna3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"perempuan_dengan_tongkat_putih_menghadap_kanan_warna4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"perempuan_dengan_tongkat_putih_menghadap_kanan_warna5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"perempuan_dalam_kerusi_roda_bermotor", "1F469-200D-1F9BC"},
    {"perempuan_dalam_kerusi_roda_bermotor_warna1", "1F469-1F3FB-200D-1F9BC"},
    {"perempuan_dalam_kerusi_roda_bermotor_warna2", "1F469-1F3FC-200D-1F9BC"},
    {"perempuan_dalam_kerusi_roda_bermotor_warna3", "1F469-1F3FD-200D-1F9BC"},
    {"perempuan_dalam_kerusi_roda_bermotor_warna4", "1F469-1F3FE-200D-1F9BC"},
    {"perempuan_dalam_kerusi_roda_bermotor_warna5", "1F469-1F3FF-200D-1F9BC"},
    {"perempuan_dalam_kerusi_roda_bermotor_menghadap_kanan", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"perempuan_dalam_kerusi_roda_bermotor_menghadap_kanan_warna1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"perempuan_dalam_kerusi_roda_bermotor_menghadap_kanan_warna2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"perempuan_dalam_kerusi_roda_bermotor_menghadap_kanan_warna3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"perempuan_dalam_kerusi_roda_bermotor_menghadap_kanan_warna4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"perempuan_dalam_kerusi_roda_bermotor_menghadap_kanan_warna5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"perempuan_dalam_kereta_roda_manual", "1F469-200D-1F9BD"},
    {"perempuan_dalam_kereta_roda_manual_warna1", "1F469-1F3FB-200D-1F9BD"},
    {"perempuan_dalam_kereta_roda_manual_warna2", "1F469-1F3FC-200D-1F9BD"},
    {"perempuan_dalam_kereta_roda_manual_warna3", "1F469-1F3FD-200D-1F9BD"},
    {"perempuan_dalam_kereta_roda_manual_warna4", "1F469-1F3FE-200D-1F9BD"},
    {"perempuan_dalam_kereta_roda_manual_warna5", "1F469-1F3FF-200D-1F9BD"},
    {"perempuan_dalam_kereta_roda_manual_menghadap_kanan", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"perempuan_dalam_kereta_roda_manual_menghadap_kanan_warna1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"perempuan_dalam_kereta_roda_manual_menghadap_kanan_warna2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"perempuan_dalam_kereta_roda_manual_menghadap_kanan_warna3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"perempuan_dalam_kereta_roda_manual_menghadap_kanan_warna4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"perempuan_dalam_kereta_roda_manual_menghadap_kanan_warna5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"pejalan_kaki_menghadap_kanan", "1F6B6-200D-27A1-FE0F"},
    {"pejalan_kaki_menghadap_kanan_warna1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"pejalan_kaki_menghadap_kanan_warna2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"pejalan_kaki_menghadap_kanan_warna3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"pejalan_kaki_menghadap_kanan_warna4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"pejalan_kaki_menghadap_kanan_warna5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"orang_bersimpuh_menghadap_kanan", "1F9CE-200D-27A1-FE0F"},
    {"orang_bersimpuh_menghadap_kanan_warna1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"orang_bersimpuh_menghadap_kanan_warna2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"orang_bersimpuh_menghadap_kanan_warna3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"orang_bersimpuh_menghadap_kanan_warna4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"orang_bersimpuh_menghadap_kanan_warna5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"pekerja_kesihatan", "1F9D1-200D-2695-FE0F"},
    {"pekerja_kesihatan_warna1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"pekerja_kesihatan_warna2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"pekerja_kesihatan_warna3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"pekerja_kesihatan_warna4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"pekerja_kesihatan_warna5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"hakim", "1F9D1-200D-2696-FE0F"},
    {"hakim_warna1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"hakim_warna2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"hakim_warna3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"hakim_warna4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"hakim_warna5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"juruterbang", "1F9D1-200D-2708-FE0F"},
    {"juruterbang_warna1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"juruterbang_warna2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"juruterbang_warna3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"juruterbang_warna4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"juruterbang_warna5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"petani", "1F9D1-200D-1F33E"},
    {"petani_warna1", "1F9D1-1F3FB-200D-1F33E"},
    {"petani_warna2", "1F9D1-1F3FC-200D-1F33E"},
    {"petani_warna3", "1F9D1-1F3FD-200D-1F33E"},
    {"petani_warna4", "1F9D1-1F3FE-200D-1F33E"},
    {"petani_warna5", "1F9D1-1F3FF-200D-1F33E"},
    {"tukang_masak", "1F9D1-200D-1F373"},
    {"tukang_masak_warna1", "1F9D1-1F3FB-200D-1F373"},
    {"tukang_masak_warna2", "1F9D1-1F3FC-200D-1F373"},
    {"tukang_masak_warna3", "1F9D1-1F3FD-200D-1F373"},
    {"tukang_masak_warna4", "1F9D1-1F3FE-200D-1F373"},
    {"tukang_masak_warna5", "1F9D1-1F3FF-200D-1F373"},
    {"orang_menyusui_bayi", "1F9D1-200D-1F37C"},
    {"orang_menyusui_bayi_warna1", "1F9D1-1F3FB-200D-1F37C"},
    {"orang_menyusui_bayi_warna2", "1F9D1-1F3FC-200D-1F37C"},
    {"orang_menyusui_bayi_warna3", "1F9D1-1F3FD-200D-1F37C"},
    {"orang_menyusui_bayi_warna4", "1F9D1-1F3FE-200D-1F37C"},
    {"orang_menyusui_bayi_warna5", "1F9D1-1F3FF-200D-1F37C"},
    {"orang_claus", "1F9D1-200D-1F384"},
    {"orang_claus_warna1", "1F9D1-1F3FB-200D-1F384"},
    {"orang_claus_warna2", "1F9D1-1F3FC-200D-1F384"},
    {"orang_claus_warna3", "1F9D1-1F3FD-200D-1F384"},
    {"orang_claus_warna4", "1F9D1-1F3FE-200D-1F384"},
    {"orang_claus_warna5", "1F9D1-1F3FF-200D-1F384"},
    {"pelajar", "1F9D1-200D-1F393"},
    {"pelajar_warna1", "1F9D1-1F3FB-200D-1F393"},
    {"pelajar_warna2", "1F9D1-1F3FC-200D-1F393"},
    {"pelajar_warna3", "1F9D1-1F3FD-200D-1F393"},
    {"pelajar_warna4", "1F9D1-1F3FE-200D-1F393"},
    {"pelajar_warna5", "1F9D1-1F3FF-200D-1F393"},
    {"penyanyi", "1F9D1-200D-1F3A4"},
    {"penyanyi_warna1", "1F9D1-1F3FB-200D-1F3A4"},
    {"penyanyi_warna2", "1F9D1-1F3FC-200D-1F3A4"},
    {"penyanyi_warna3", "1F9D1-1F3FD-200D-1F3A4"},
    {"penyanyi_warna4", "1F9D1-1F3FE-200D-1F3A4"},
    {"penyanyi_warna5", "1F9D1-1F3FF-200D-1F3A4"},
    {"artis", "1F9D1-200D-1F3A8"},
    {"artis_warna1", "1F9D1-1F3FB-200D-1F3A8"},
    {"artis_warna2", "1F9D1-1F3FC-200D-1F3A8"},
    {"artis_warna3", "1F9D1-1F3FD-200D-1F3A8"},
    {"artis_warna4", "1F9D1-1F3FE-200D-1F3A8"},
    {"artis_warna5", "1F9D1-1F3FF-200D-1F3A8"},
    {"guru", "1F9D1-200D-1F3EB"},
    {"guru_warna1", "1F9D1-1F3FB-200D-1F3EB"},
    {"guru_warna2", "1F9D1-1F3FC-200D-1F3EB"},
    {"guru_warna3", "1F9D1-1F3FD-200D-1F3EB"},
    {"guru_warna4", "1F9D1-1F3FE-200D-1F3EB"},
    {"guru_warna5", "1F9D1-1F3FF-200D-1F3EB"},
    {"pekerja_kilang", "1F9D1-200D-1F3ED"},
    {"pekerja_kilang_warna1", "1F9D1-1F3FB-200D-1F3ED"},
    {"pekerja_kilang_warna2", "1F9D1-1F3FC-200D-1F3ED"},
    {"pekerja_kilang_warna3", "1F9D1-1F3FD-200D-1F3ED"},
    {"pekerja_kilang_warna4", "1F9D1-1F3FE-200D-1F3ED"},
    {"pekerja_kilang_warna5", "1F9D1-1F3FF-200D-1F3ED"},
    {"ahli_teknologi", "1F9D1-200D-1F4BB"},
    {"ahli_teknologi_warna1", "1F9D1-1F3FB-200D-1F4BB"},
    {"ahli_teknologi_warna2", "1F9D1-1F3FC-200D-1F4BB"},
    {"ahli_teknologi_warna3", "1F9D1-1F3FD-200D-1F4BB"},
    {"ahli_teknologi_warna4", "1F9D1-1F3FE-200D-1F4BB"},
    {"ahli_teknologi_warna5", "1F9D1-1F3FF-200D-1F4BB"},
    {"pekerja_pejabat", "1F9D1-200D-1F4BC"},
    {"pekerja_pejabat_warna1", "1F9D1-1F3FB-200D-1F4BC"},
    {"pekerja_pejabat_warna2", "1F9D1-1F3FC-200D-1F4BC"},
    {"pekerja_pejabat_warna3", "1F9D1-1F3FD-200D-1F4BC"},
    {"pekerja_pejabat_warna4", "1F9D1-1F3FE-200D-1F4BC"},
    {"pekerja_pejabat_warna5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mekanik", "1F9D1-200D-1F527"},
    {"mekanik_warna1", "1F9D1-1F3FB-200D-1F527"},
    {"mekanik_warna2", "1F9D1-1F3FC-200D-1F527"},
    {"mekanik_warna3", "1F9D1-1F3FD-200D-1F527"},
    {"mekanik_warna4", "1F9D1-1F3FE-200D-1F527"},
    {"mekanik_warna5", "1F9D1-1F3FF-200D-1F527"},
    {"saintis", "1F9D1-200D-1F52C"},
    {"saintis_warna1", "1F9D1-1F3FB-200D-1F52C"},
    {"saintis_warna2", "1F9D1-1F3FC-200D-1F52C"},
    {"saintis_warna3", "1F9D1-1F3FD-200D-1F52C"},
    {"saintis_warna4", "1F9D1-1F3FE-200D-1F52C"},
    {"saintis_warna5", "1F9D1-1F3FF-200D-1F52C"},
    {"angkasawan", "1F9D1-200D-1F680"},
    {"angkasawan_warna1", "1F9D1-1F3FB-200D-1F680"},
    {"angkasawan_warna2", "1F9D1-1F3FC-200D-1F680"},
    {"angkasawan_warna3", "1F9D1-1F3FD-200D-1F680"},
    {"angkasawan_warna4", "1F9D1-1F3FE-200D-1F680"},
    {"angkasawan_warna5", "1F9D1-1F3FF-200D-1F680"},
    {"ahli_bomba", "1F9D1-200D-1F692"},
    {"ahli_bomba_warna1", "1F9D1-1F3FB-200D-1F692"},
    {"ahli_bomba_warna2", "1F9D1-1F3FC-200D-1F692"},
    {"ahli_bomba_warna3", "1F9D1-1F3FD-200D-1F692"},
    {"ahli_bomba_warna4", "1F9D1-1F3FE-200D-1F692"},
    {"ahli_bomba_warna5", "1F9D1-1F3FF-200D-1F692"},
    {"orang_dengan_tongkat_putih", "1F9D1-200D-1F9AF"},
    {"orang_dengan_tongkat_putih_warna1", "1F9D1-1F3FB-200D-1F9AF"},
    {"orang_dengan_tongkat_putih_warna2", "1F9D1-1F3FC-200D-1F9AF"},
    {"orang_dengan_tongkat_putih_warna3", "1F9D1-1F3FD-200D-1F9AF"},
    {"orang_dengan_tongkat_putih_warna4", "1F9D1-1F3FE-200D-1F9AF"},
    {"orang_dengan_tongkat_putih_warna5", "1F9D1-1F3FF-200D-1F9AF"},
    {"orang_dengan_tongkat_putih_menghadap_kanan", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"orang_dengan_tongkat_putih_menghadap_kanan_warna1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"orang_dengan_tongkat_putih_menghadap_kanan_warna2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"orang_dengan_tongkat_putih_menghadap_kanan_warna3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"orang_dengan_tongkat_putih_menghadap_kanan_warna4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"orang_dengan_tongkat_putih_menghadap_kanan_warna5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"orang_dalam_kerusi_roda_bermotor", "1F9D1-200D-1F9BC"},
    {"orang_dalam_kerusi_roda_bermotor_warna1", "1F9D1-1F3FB-200D-1F9BC"},
    {"orang_dalam_kerusi_roda_bermotor_warna2", "1F9D1-1F3FC-200D-1F9BC"},
    {"orang_dalam_kerusi_roda_bermotor_warna3", "1F9D1-1F3FD-200D-1F9BC"},
    {"orang_dalam_kerusi_roda_bermotor_warna4", "1F9D1-1F3FE-200D-1F9BC"},
    {"orang_dalam_kerusi_roda_bermotor_warna5", "1F9D1-1F3FF-200D-1F9BC"},
    {"orang_dalam_kerusi_roda_bermotor_menghadap_kanan", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"orang_dalam_kerusi_roda_bermotor_menghadap_kanan_warna1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"orang_dalam_kerusi_roda_bermotor_menghadap_kanan_warna2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"orang_dalam_kerusi_roda_bermotor_menghadap_kanan_warna3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"orang_dalam_kerusi_roda_bermotor_menghadap_kanan_warna4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"orang_dalam_kerusi_roda_bermotor_menghadap_kanan_warna5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"orang_dalam_kerusi_roda_manual", "1F9D1-200D-1F9BD"},
    {"orang_dalam_kerusi_roda_manual_warna1", "1F9D1-1F3FB-200D-1F9BD"},
    {"orang_dalam_kerusi_roda_manual_warna2", "1F9D1-1F3FC-200D-1F9BD"},
    {"orang_dalam_kerusi_roda_manual_warna3", "1F9D1-1F3FD-200D-1F9BD"},
    {"orang_dalam_kerusi_roda_manual_warna4", "1F9D1-1F3FE-200D-1F9BD"},
    {"orang_dalam_kerusi_roda_manual_warna5", "1F9D1-1F3FF-200D-1F9BD"},
    {"orang_dalam_kerusi_roda_manual_menghadap_kanan", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"orang_dalam_kerusi_roda_manual_menghadap_kanan_warna1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"orang_dalam_kerusi_roda_manual_menghadap_kanan_warna2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"orang_dalam_kerusi_roda_manual_menghadap_kanan_warna3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"orang_dalam_kerusi_roda_manual_menghadap_kanan_warna4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"orang_dalam_kerusi_roda_manual_menghadap_kanan_warna5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"wanita_melantun_bola", "26F9-FE0F-200D-2640-FE0F"},
    {"wanita_melantun_bola_warna1", "26F9-1F3FB-200D-2640-FE0F"},
    {"wanita_melantun_bola_warna2", "26F9-1F3FC-200D-2640-FE0F"},
    {"wanita_melantun_bola_warna3", "26F9-1F3FD-200D-2640-FE0F"},
    {"wanita_melantun_bola_warna4", "26F9-1F3FE-200D-2640-FE0F"},
    {"wanita_melantun_bola_warna5", "26F9-1F3FF-200D-2640-FE0F"},
    {"lelaki_melantun_bola", "26F9-FE0F-200D-2642-FE0F"},
    {"lelaki_melantun_bola_warna1", "26F9-1F3FB-200D-2642-FE0F"},
    {"lelaki_melantun_bola_warna2", "26F9-1F3FC-200D-2642-FE0F"},
    {"lelaki_melantun_bola_warna3", "26F9-1F3FD-200D-2642-FE0F"},
    {"lelaki_melantun_bola_warna4", "26F9-1F3FE-200D-2642-FE0F"},
    {"lelaki_melantun_bola_warna5", "26F9-1F3FF-200D-2642-FE0F"},
    {"perempuan_berlari", "1F3C3-200D-2640-FE0F"},
    {"perempuan_berlari_warna1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"perempuan_berlari_warna2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"perempuan_berlari_warna3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"perempuan_berlari_warna4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"perempuan_berlari_warna5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"perempuan_berlari_menghadap_kanan", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_berlari_menghadap_kanan_warna1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_berlari_menghadap_kanan_warna2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_berlari_menghadap_kanan_warna3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_berlari_menghadap_kanan_warna4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_berlari_menghadap_kanan_warna5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"lelaki_berlari", "1F3C3-200D-2642-FE0F"},
    {"lelaki_berlari_warna1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"lelaki_berlari_warna2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"lelaki_berlari_warna3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"lelaki_berlari_warna4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"lelaki_berlari_warna5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"lelaki_berlari_menghadap_kanan", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_berlari_menghadap_kanan_warna1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_berlari_menghadap_kanan_warna2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_berlari_menghadap_kanan_warna3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_berlari_menghadap_kanan_warna4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_berlari_menghadap_kanan_warna5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"wanita_meluncur_air", "1F3C4-200D-2640-FE0F"},
    {"wanita_meluncur_air_warna1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"wanita_meluncur_air_warna2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"wanita_meluncur_air_warna3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"wanita_meluncur_air_warna4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"wanita_meluncur_air_warna5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"lelaki_meluncur_air", "1F3C4-200D-2642-FE0F"},
    {"lelaki_meluncur_air_warna1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"lelaki_meluncur_air_warna2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"lelaki_meluncur_air_warna3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"lelaki_meluncur_air_warna4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"lelaki_meluncur_air_warna5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"wanita_berenang", "1F3CA-200D-2640-FE0F"},
    {"wanita_berenang_warna1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"wanita_berenang_warna2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"wanita_berenang_warna3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"wanita_berenang_warna4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"wanita_berenang_warna5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"lelaki_berenang", "1F3CA-200D-2642-FE0F"},
    {"lelaki_berenang_warna1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"lelaki_berenang_warna2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"lelaki_berenang_warna3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"lelaki_berenang_warna4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"lelaki_berenang_warna5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"wanita_mengangkat_berat", "1F3CB-FE0F-200D-2640-FE0F"},
    {"wanita_mengangkat_berat_warna1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"wanita_mengangkat_berat_warna2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"wanita_mengangkat_berat_warna3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"wanita_mengangkat_berat_warna4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"wanita_mengangkat_berat_warna5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"lelaki_mengangkat_berat", "1F3CB-FE0F-200D-2642-FE0F"},
    {"lelaki_mengangkat_berat_warna1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"lelaki_mengangkat_berat_warna2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"lelaki_mengangkat_berat_warna3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"lelaki_mengangkat_berat_warna4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"lelaki_mengangkat_berat_warna5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"wanita_bermain_golf", "1F3CC-FE0F-200D-2640-FE0F"},
    {"wanita_bermain_golf_warna1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"wanita_bermain_golf_warna2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"wanita_bermain_golf_warna3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"wanita_bermain_golf_warna4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"wanita_bermain_golf_warna5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"lelaki_bermain_golf", "1F3CC-FE0F-200D-2642-FE0F"},
    {"lelaki_bermain_golf_warna1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"lelaki_bermain_golf_warna2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"lelaki_bermain_golf_warna3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"lelaki_bermain_golf_warna4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"lelaki_bermain_golf_warna5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"pegawai_polis_wanita", "1F46E-200D-2640-FE0F"},
    {"pegawai_polis_wanita_warna1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"pegawai_polis_wanita_warna2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"pegawai_polis_wanita_warna3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"pegawai_polis_wanita_warna4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"pegawai_polis_wanita_warna5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"pegawai_polis_lelaki", "1F46E-200D-2642-FE0F"},
    {"pegawai_polis_lelaki_warna1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"pegawai_polis_lelaki_warna2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"pegawai_polis_lelaki_warna3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"pegawai_polis_lelaki_warna4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"pegawai_polis_lelaki_warna5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"perempuan_berpesta", "1F46F-200D-2640-FE0F"},
    {"perempuan_berpesta_warna1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"perempuan_berpesta_warna1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"perempuan_berpesta_warna1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"perempuan_berpesta_warna1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"perempuan_berpesta_warna2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"perempuan_berpesta_warna2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"perempuan_berpesta_warna2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"perempuan_berpesta_warna2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"perempuan_berpesta_warna3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"perempuan_berpesta_warna3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"perempuan_berpesta_warna3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"perempuan_berpesta_warna3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"perempuan_berpesta_warna4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"perempuan_berpesta_warna4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"perempuan_berpesta_warna4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"perempuan_berpesta_warna4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"perempuan_berpesta_warna5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"perempuan_berpesta_warna5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"perempuan_berpesta_warna5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"perempuan_berpesta_warna5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"perempuan_berpesta_warna1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"perempuan_berpesta_warna2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"perempuan_berpesta_warna3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"perempuan_berpesta_warna4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"perempuan_berpesta_warna5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"lelaki_berpesta", "1F46F-200D-2642-FE0F"},
    {"lelaki_berpesta_warna1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"lelaki_berpesta_warna1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"lelaki_berpesta_warna1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"lelaki_berpesta_warna1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"lelaki_berpesta_warna2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"lelaki_berpesta_warna2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"lelaki_berpesta_warna2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"lelaki_berpesta_warna2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"lelaki_berpesta_warna3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"lelaki_berpesta_warna3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"lelaki_berpesta_warna3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"lelaki_berpesta_warna3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"lelaki_berpesta_warna4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"lelaki_berpesta_warna4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"lelaki_berpesta_warna4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"lelaki_berpesta_warna4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"lelaki_berpesta_warna5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"lelaki_berpesta_warna5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"lelaki_berpesta_warna5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"lelaki_berpesta_warna5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"lelaki_berpesta_warna1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"lelaki_berpesta_warna2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"lelaki_berpesta_warna3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"lelaki_berpesta_warna4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"lelaki_berpesta_warna5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"wanita_memakai_vel", "1F470-200D-2640-FE0F"},
    {"wanita_memakai_vel_warna1", "1F470-1F3FB-200D-2640-FE0F"},
    {"wanita_memakai_vel_warna2", "1F470-1F3FC-200D-2640-FE0F"},
    {"wanita_memakai_vel_warna3", "1F470-1F3FD-200D-2640-FE0F"},
    {"wanita_memakai_vel_warna4", "1F470-1F3FE-200D-2640-FE0F"},
    {"wanita_memakai_vel_warna5", "1F470-1F3FF-200D-2640-FE0F"},
    {"lelaki_memakai_vel", "1F470-200D-2642-FE0F"},
    {"lelaki_memakai_vel_warna1", "1F470-1F3FB-200D-2642-FE0F"},
    {"lelaki_memakai_vel_warna2", "1F470-1F3FC-200D-2642-FE0F"},
    {"lelaki_memakai_vel_warna3", "1F470-1F3FD-200D-2642-FE0F"},
    {"lelaki_memakai_vel_warna4", "1F470-1F3FE-200D-2642-FE0F"},
    {"lelaki_memakai_vel_warna5", "1F470-1F3FF-200D-2642-FE0F"},
    {"perempuan_rambut_perang", "1F471-200D-2640-FE0F"},
    {"perempuan_rambut_perang_warna1", "1F471-1F3FB-200D-2640-FE0F"},
    {"perempuan_rambut_perang_warna2", "1F471-1F3FC-200D-2640-FE0F"},
    {"perempuan_rambut_perang_warna3", "1F471-1F3FD-200D-2640-FE0F"},
    {"perempuan_rambut_perang_warna4", "1F471-1F3FE-200D-2640-FE0F"},
    {"perempuan_rambut_perang_warna5", "1F471-1F3FF-200D-2640-FE0F"},
    {"lelaki_rambut_perang", "1F471-200D-2642-FE0F"},
    {"lelaki_rambut_perang_warna1", "1F471-1F3FB-200D-2642-FE0F"},
    {"lelaki_rambut_perang_warna2", "1F471-1F3FC-200D-2642-FE0F"},
    {"lelaki_rambut_perang_warna3", "1F471-1F3FD-200D-2642-FE0F"},
    {"lelaki_rambut_perang_warna4", "1F471-1F3FE-200D-2642-FE0F"},
    {"lelaki_rambut_perang_warna5", "1F471-1F3FF-200D-2642-FE0F"},
    {"wanita_berserban", "1F473-200D-2640-FE0F"},
    {"wanita_berserban_warna1", "1F473-1F3FB-200D-2640-FE0F"},
    {"wanita_berserban_warna2", "1F473-1F3FC-200D-2640-FE0F"},
    {"wanita_berserban_warna3", "1F473-1F3FD-200D-2640-FE0F"},
    {"wanita_berserban_warna4", "1F473-1F3FE-200D-2640-FE0F"},
    {"wanita_berserban_warna5", "1F473-1F3FF-200D-2640-FE0F"},
    {"lelaki_berserban", "1F473-200D-2642-FE0F"},
    {"lelaki_berserban_warna1", "1F473-1F3FB-200D-2642-FE0F"},
    {"lelaki_berserban_warna2", "1F473-1F3FC-200D-2642-FE0F"},
    {"lelaki_berserban_warna3", "1F473-1F3FD-200D-2642-FE0F"},
    {"lelaki_berserban_warna4", "1F473-1F3FE-200D-2642-FE0F"},
    {"lelaki_berserban_warna5", "1F473-1F3FF-200D-2642-FE0F"},
    {"pekerja_pembinaan_wanita", "1F477-200D-2640-FE0F"},
    {"pekerja_pembinaan_wanita_warna1", "1F477-1F3FB-200D-2640-FE0F"},
    {"pekerja_pembinaan_wanita_warna2", "1F477-1F3FC-200D-2640-FE0F"},
    {"pekerja_pembinaan_wanita_warna3", "1F477-1F3FD-200D-2640-FE0F"},
    {"pekerja_pembinaan_wanita_warna4", "1F477-1F3FE-200D-2640-FE0F"},
    {"pekerja_pembinaan_wanita_warna5", "1F477-1F3FF-200D-2640-FE0F"},
    {"pekerja_pembinaan_lelaki", "1F477-200D-2642-FE0F"},
    {"pekerja_pembinaan_lelaki_warna1", "1F477-1F3FB-200D-2642-FE0F"},
    {"pekerja_pembinaan_lelaki_warna2", "1F477-1F3FC-200D-2642-FE0F"},
    {"pekerja_pembinaan_lelaki_warna3", "1F477-1F3FD-200D-2642-FE0F"},
    {"pekerja_pembinaan_lelaki_warna4", "1F477-1F3FE-200D-2642-FE0F"},
    {"pekerja_pembinaan_lelaki_warna5", "1F477-1F3FF-200D-2642-FE0F"},
    {"perempuan_menadah_tangan", "1F481-200D-2640-FE0F"},
    {"perempuan_menadah_tangan_warna1", "1F481-1F3FB-200D-2640-FE0F"},
    {"perempuan_menadah_tangan_warna2", "1F481-1F3FC-200D-2640-FE0F"},
    {"perempuan_menadah_tangan_warna3", "1F481-1F3FD-200D-2640-FE0F"},
    {"perempuan_menadah_tangan_warna4", "1F481-1F3FE-200D-2640-FE0F"},
    {"perempuan_menadah_tangan_warna5", "1F481-1F3FF-200D-2640-FE0F"},
    {"lelaki_menadah_tangan", "1F481-200D-2642-FE0F"},
    {"lelaki_menadah_tangan_warna1", "1F481-1F3FB-200D-2642-FE0F"},
    {"lelaki_menadah_tangan_warna2", "1F481-1F3FC-200D-2642-FE0F"},
    {"lelaki_menadah_tangan_warna3", "1F481-1F3FD-200D-2642-FE0F"},
    {"lelaki_menadah_tangan_warna4", "1F481-1F3FE-200D-2642-FE0F"},
    {"lelaki_menadah_tangan_warna5", "1F481-1F3FF-200D-2642-FE0F"},
    {"pengawal_wanita", "1F482-200D-2640-FE0F"},
    {"pengawal_wanita_warna1", "1F482-1F3FB-200D-2640-FE0F"},
    {"pengawal_wanita_warna2", "1F482-1F3FC-200D-2640-FE0F"},
    {"pengawal_wanita_warna3", "1F482-1F3FD-200D-2640-FE0F"},
    {"pengawal_wanita_warna4", "1F482-1F3FE-200D-2640-FE0F"},
    {"pengawal_wanita_warna5", "1F482-1F3FF-200D-2640-FE0F"},
    {"pengawal_lelaki", "1F482-200D-2642-FE0F"},
    {"pengawal_lelaki_warna1", "1F482-1F3FB-200D-2642-FE0F"},
    {"pengawal_lelaki_warna2", "1F482-1F3FC-200D-2642-FE0F"},
    {"pengawal_lelaki_warna3", "1F482-1F3FD-200D-2642-FE0F"},
    {"pengawal_lelaki_warna4", "1F482-1F3FE-200D-2642-FE0F"},
    {"pengawal_lelaki_warna5", "1F482-1F3FF-200D-2642-FE0F"},
    {"perempuan_diurut_muka", "1F486-200D-2640-FE0F"},
    {"perempuan_diurut_muka_warna1", "1F486-1F3FB-200D-2640-FE0F"},
    {"perempuan_diurut_muka_warna2", "1F486-1F3FC-200D-2640-FE0F"},
    {"perempuan_diurut_muka_warna3", "1F486-1F3FD-200D-2640-FE0F"},
    {"perempuan_diurut_muka_warna4", "1F486-1F3FE-200D-2640-FE0F"},
    {"perempuan_diurut_muka_warna5", "1F486-1F3FF-200D-2640-FE0F"},
    {"lelaki_diurut_muka", "1F486-200D-2642-FE0F"},
    {"lelaki_diurut_muka_warna1", "1F486-1F3FB-200D-2642-FE0F"},
    {"lelaki_diurut_muka_warna2", "1F486-1F3FC-200D-2642-FE0F"},
    {"lelaki_diurut_muka_warna3", "1F486-1F3FD-200D-2642-FE0F"},
    {"lelaki_diurut_muka_warna4", "1F486-1F3FE-200D-2642-FE0F"},
    {"lelaki_diurut_muka_warna5", "1F486-1F3FF-200D-2642-FE0F"},
    {"perempuan_digunting_rambut", "1F487-200D-2640-FE0F"},
    {"perempuan_digunting_rambut_warna1", "1F487-1F3FB-200D-2640-FE0F"},
    {"perempuan_digunting_rambut_warna2", "1F487-1F3FC-200D-2640-FE0F"},
    {"perempuan_digunting_rambut_warna3", "1F487-1F3FD-200D-2640-FE0F"},
    {"perempuan_digunting_rambut_warna4", "1F487-1F3FE-200D-2640-FE0F"},
    {"perempuan_digunting_rambut_warna5", "1F487-1F3FF-200D-2640-FE0F"},
    {"lelaki_digunting_rambut", "1F487-200D-2642-FE0F"},
    {"lelaki_digunting_rambut_warna1", "1F487-1F3FB-200D-2642-FE0F"},
    {"lelaki_digunting_rambut_warna2", "1F487-1F3FC-200D-2642-FE0F"},
    {"lelaki_digunting_rambut_warna3", "1F487-1F3FD-200D-2642-FE0F"},
    {"lelaki_digunting_rambut_warna4", "1F487-1F3FE-200D-2642-FE0F"},
    {"lelaki_digunting_rambut_warna5", "1F487-1F3FF-200D-2642-FE0F"},
    {"mata_mata_gelap_wanita", "1F575-FE0F-200D-2640-FE0F"},
    {"mata_mata_gelap_wanita_warna1", "1F575-1F3FB-200D-2640-FE0F"},
    {"mata_mata_gelap_wanita_warna2", "1F575-1F3FC-200D-2640-FE0F"},
    {"mata_mata_gelap_wanita_warna3", "1F575-1F3FD-200D-2640-FE0F"},
    {"mata_mata_gelap_wanita_warna4", "1F575-1F3FE-200D-2640-FE0F"},
    {"mata_mata_gelap_wanita_warna5", "1F575-1F3FF-200D-2640-FE0F"},
    {"mata_mata_gelap_lelaki", "1F575-FE0F-200D-2642-FE0F"},
    {"mata_mata_gelap_lelaki_warna1", "1F575-1F3FB-200D-2642-FE0F"},
    {"mata_mata_gelap_lelaki_warna2", "1F575-1F3FC-200D-2642-FE0F"},
    {"mata_mata_gelap_lelaki_warna3", "1F575-1F3FD-200D-2642-FE0F"},
    {"mata_mata_gelap_lelaki_warna4", "1F575-1F3FE-200D-2642-FE0F"},
    {"mata_mata_gelap_lelaki_warna5", "1F575-1F3FF-200D-2642-FE0F"},
    {"perempuan_memberi_isyarat_tidak_ok", "1F645-200D-2640-FE0F"},
    {"perempuan_memberi_isyarat_tidak_ok_warna1", "1F645-1F3FB-200D-2640-FE0F"},
    {"perempuan_memberi_isyarat_tidak_ok_warna2", "1F645-1F3FC-200D-2640-FE0F"},
    {"perempuan_memberi_isyarat_tidak_ok_warna3", "1F645-1F3FD-200D-2640-FE0F"},
    {"perempuan_memberi_isyarat_tidak_ok_warna4", "1F645-1F3FE-200D-2640-FE0F"},
    {"perempuan_memberi_isyarat_tidak_ok_warna5", "1F645-1F3FF-200D-2640-FE0F"},
    {"lelaki_memberi_isyarat_tidak_ok", "1F645-200D-2642-FE0F"},
    {"lelaki_memberi_isyarat_tidak_ok_warna1", "1F645-1F3FB-200D-2642-FE0F"},
    {"lelaki_memberi_isyarat_tidak_ok_warna2", "1F645-1F3FC-200D-2642-FE0F"},
    {"lelaki_memberi_isyarat_tidak_ok_warna3", "1F645-1F3FD-200D-2642-FE0F"},
    {"lelaki_memberi_isyarat_tidak_ok_warna4", "1F645-1F3FE-200D-2642-FE0F"},
    {"lelaki_memberi_isyarat_tidak_ok_warna5", "1F645-1F3FF-200D-2642-FE0F"},
    {"perempuan_memberi_isyarat_ok", "1F646-200D-2640-FE0F"},
    {"perempuan_memberi_isyarat_ok_warna1", "1F646-1F3FB-200D-2640-FE0F"},
    {"perempuan_memberi_isyarat_ok_warna2", "1F646-1F3FC-200D-2640-FE0F"},
    {"perempuan_memberi_isyarat_ok_warna3", "1F646-1F3FD-200D-2640-FE0F"},
    {"perempuan_memberi_isyarat_ok_warna4", "1F646-1F3FE-200D-2640-FE0F"},
    {"perempuan_memberi_isyarat_ok_warna5", "1F646-1F3FF-200D-2640-FE0F"},
    {"lelaki_memberi_isyarat_ok", "1F646-200D-2642-FE0F"},
    {"lelaki_memberi_isyarat_ok_warna1", "1F646-1F3FB-200D-2642-FE0F"},
    {"lelaki_memberi_isyarat_ok_warna2", "1F646-1F3FC-200D-2642-FE0F"},
    {"lelaki_memberi_isyarat_ok_warna3", "1F646-1F3FD-200D-2642-FE0F"},
    {"lelaki_memberi_isyarat_ok_warna4", "1F646-1F3FE-200D-2642-FE0F"},
    {"lelaki_memberi_isyarat_ok_warna5", "1F646-1F3FF-200D-2642-FE0F"},
    {"perempuan_menunduk", "1F647-200D-2640-FE0F"},
    {"perempuan_menunduk_warna1", "1F647-1F3FB-200D-2640-FE0F"},
    {"perempuan_menunduk_warna2", "1F647-1F3FC-200D-2640-FE0F"},
    {"perempuan_menunduk_warna3", "1F647-1F3FD-200D-2640-FE0F"},
    {"perempuan_menunduk_warna4", "1F647-1F3FE-200D-2640-FE0F"},
    {"perempuan_menunduk_warna5", "1F647-1F3FF-200D-2640-FE0F"},
    {"lelaki_menunduk", "1F647-200D-2642-FE0F"},
    {"lelaki_menunduk_warna1", "1F647-1F3FB-200D-2642-FE0F"},
    {"lelaki_menunduk_warna2", "1F647-1F3FC-200D-2642-FE0F"},
    {"lelaki_menunduk_warna3", "1F647-1F3FD-200D-2642-FE0F"},
    {"lelaki_menunduk_warna4", "1F647-1F3FE-200D-2642-FE0F"},
    {"lelaki_menunduk_warna5", "1F647-1F3FF-200D-2642-FE0F"},
    {"perempuan_mengangkat_tangan", "1F64B-200D-2640-FE0F"},
    {"perempuan_mengangkat_tangan_warna1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"perempuan_mengangkat_tangan_warna2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"perempuan_mengangkat_tangan_warna3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"perempuan_mengangkat_tangan_warna4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"perempuan_mengangkat_tangan_warna5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"lelaki_mengangkat_tangan", "1F64B-200D-2642-FE0F"},
    {"lelaki_mengangkat_tangan_warna1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"lelaki_mengangkat_tangan_warna2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"lelaki_mengangkat_tangan_warna3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"lelaki_mengangkat_tangan_warna4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"lelaki_mengangkat_tangan_warna5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"perempuan_berkerut_dahi", "1F64D-200D-2640-FE0F"},
    {"perempuan_berkerut_dahi_warna1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"perempuan_berkerut_dahi_warna2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"perempuan_berkerut_dahi_warna3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"perempuan_berkerut_dahi_warna4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"perempuan_berkerut_dahi_warna5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"lelaki_berkerut_dahi", "1F64D-200D-2642-FE0F"},
    {"lelaki_berkerut_dahi_warna1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"lelaki_berkerut_dahi_warna2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"lelaki_berkerut_dahi_warna3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"lelaki_berkerut_dahi_warna4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"lelaki_berkerut_dahi_warna5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"perempuan_mencebik", "1F64E-200D-2640-FE0F"},
    {"perempuan_mencebik_warna1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"perempuan_mencebik_warna2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"perempuan_mencebik_warna3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"perempuan_mencebik_warna4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"perempuan_mencebik_warna5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"lelaki_mencebik", "1F64E-200D-2642-FE0F"},
    {"lelaki_mencebik_warna1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"lelaki_mencebik_warna2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"lelaki_mencebik_warna3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"lelaki_mencebik_warna4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"lelaki_mencebik_warna5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"wanita_mendayung_sampan", "1F6A3-200D-2640-FE0F"},
    {"wanita_mendayung_sampan_warna1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"wanita_mendayung_sampan_warna2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"wanita_mendayung_sampan_warna3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"wanita_mendayung_sampan_warna4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"wanita_mendayung_sampan_warna5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"lelaki_mendayung_sampan", "1F6A3-200D-2642-FE0F"},
    {"lelaki_mendayung_sampan_warna1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"lelaki_mendayung_sampan_warna2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"lelaki_mendayung_sampan_warna3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"lelaki_mendayung_sampan_warna4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"lelaki_mendayung_sampan_warna5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"wanita_berbasikal", "1F6B4-200D-2640-FE0F"},
    {"wanita_berbasikal_warna1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"wanita_berbasikal_warna2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"wanita_berbasikal_warna3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"wanita_berbasikal_warna4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"wanita_berbasikal_warna5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"lelaki_berbasikal", "1F6B4-200D-2642-FE0F"},
    {"lelaki_berbasikal_warna1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"lelaki_berbasikal_warna2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"lelaki_berbasikal_warna3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"lelaki_berbasikal_warna4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"lelaki_berbasikal_warna5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"wanita_berbasikal_gunung", "1F6B5-200D-2640-FE0F"},
    {"wanita_berbasikal_gunung_warna1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"wanita_berbasikal_gunung_warna2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"wanita_berbasikal_gunung_warna3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"wanita_berbasikal_gunung_warna4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"wanita_berbasikal_gunung_warna5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"lelaki_berbasikal_gunung", "1F6B5-200D-2642-FE0F"},
    {"lelaki_berbasikal_gunung_warna1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"lelaki_berbasikal_gunung_warna2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"lelaki_berbasikal_gunung_warna3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"lelaki_berbasikal_gunung_warna4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"lelaki_berbasikal_gunung_warna5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"perempuan_berjalan", "1F6B6-200D-2640-FE0F"},
    {"perempuan_berjalan_warna1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"perempuan_berjalan_warna2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"perempuan_berjalan_warna3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"perempuan_berjalan_warna4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"perempuan_berjalan_warna5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"perempuan_berjalan_menghadap_kanan", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_berjalan_menghadap_kanan_warna1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_berjalan_menghadap_kanan_warna2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_berjalan_menghadap_kanan_warna3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_berjalan_menghadap_kanan_warna4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_berjalan_menghadap_kanan_warna5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"lelaki_berjalan", "1F6B6-200D-2642-FE0F"},
    {"lelaki_berjalan_warna1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"lelaki_berjalan_warna2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"lelaki_berjalan_warna3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"lelaki_berjalan_warna4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"lelaki_berjalan_warna5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"lelaki_berjalan_menghadap_kanan", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_berjalan_menghadap_kanan_warna1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_berjalan_menghadap_kanan_warna2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_berjalan_menghadap_kanan_warna3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_berjalan_menghadap_kanan_warna4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_berjalan_menghadap_kanan_warna5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"perempuan_menepuk_muka", "1F926-200D-2640-FE0F"},
    {"perempuan_menepuk_muka_warna1", "1F926-1F3FB-200D-2640-FE0F"},
    {"perempuan_menepuk_muka_warna2", "1F926-1F3FC-200D-2640-FE0F"},
    {"perempuan_menepuk_muka_warna3", "1F926-1F3FD-200D-2640-FE0F"},
    {"perempuan_menepuk_muka_warna4", "1F926-1F3FE-200D-2640-FE0F"},
    {"perempuan_menepuk_muka_warna5", "1F926-1F3FF-200D-2640-FE0F"},
    {"lelaki_menepuk_muka", "1F926-200D-2642-FE0F"},
    {"lelaki_menepuk_muka_warna1", "1F926-1F3FB-200D-2642-FE0F"},
    {"lelaki_menepuk_muka_warna2", "1F926-1F3FC-200D-2642-FE0F"},
    {"lelaki_menepuk_muka_warna3", "1F926-1F3FD-200D-2642-FE0F"},
    {"lelaki_menepuk_muka_warna4", "1F926-1F3FE-200D-2642-FE0F"},
    {"lelaki_menepuk_muka_warna5", "1F926-1F3FF-200D-2642-FE0F"},
    {"wanita_berpakaian_tuksedo", "1F935-200D-2640-FE0F"},
    {"wanita_berpakaian_tuksedo_warna1", "1F935-1F3FB-200D-2640-FE0F"},
    {"wanita_berpakaian_tuksedo_warna2", "1F935-1F3FC-200D-2640-FE0F"},
    {"wanita_berpakaian_tuksedo_warna3", "1F935-1F3FD-200D-2640-FE0F"},
    {"wanita_berpakaian_tuksedo_warna4", "1F935-1F3FE-200D-2640-FE0F"},
    {"wanita_berpakaian_tuksedo_warna5", "1F935-1F3FF-200D-2640-FE0F"},
    {"lelaki_berpakaian_tuksedo", "1F935-200D-2642-FE0F"},
    {"lelaki_berpakaian_tuksedo_warna1", "1F935-1F3FB-200D-2642-FE0F"},
    {"lelaki_berpakaian_tuksedo_warna2", "1F935-1F3FC-200D-2642-FE0F"},
    {"lelaki_berpakaian_tuksedo_warna3", "1F935-1F3FD-200D-2642-FE0F"},
    {"lelaki_berpakaian_tuksedo_warna4", "1F935-1F3FE-200D-2642-FE0F"},
    {"lelaki_berpakaian_tuksedo_warna5", "1F935-1F3FF-200D-2642-FE0F"},
    {"perempuan_mengangkat_bahu", "1F937-200D-2640-FE0F"},
    {"perempuan_mengangkat_bahu_warna1", "1F937-1F3FB-200D-2640-FE0F"},
    {"perempuan_mengangkat_bahu_warna2", "1F937-1F3FC-200D-2640-FE0F"},
    {"perempuan_mengangkat_bahu_warna3", "1F937-1F3FD-200D-2640-FE0F"},
    {"perempuan_mengangkat_bahu_warna4", "1F937-1F3FE-200D-2640-FE0F"},
    {"perempuan_mengangkat_bahu_warna5", "1F937-1F3FF-200D-2640-FE0F"},
    {"lelaki_mengangkat_bahu", "1F937-200D-2642-FE0F"},
    {"lelaki_mengangkat_bahu_warna1", "1F937-1F3FB-200D-2642-FE0F"},
    {"lelaki_mengangkat_bahu_warna2", "1F937-1F3FC-200D-2642-FE0F"},
    {"lelaki_mengangkat_bahu_warna3", "1F937-1F3FD-200D-2642-FE0F"},
    {"lelaki_mengangkat_bahu_warna4", "1F937-1F3FE-200D-2642-FE0F"},
    {"lelaki_mengangkat_bahu_warna5", "1F937-1F3FF-200D-2642-FE0F"},
    {"wanita_membuat_putar_roda", "1F938-200D-2640-FE0F"},
    {"wanita_membuat_putar_roda_warna1", "1F938-1F3FB-200D-2640-FE0F"},
    {"wanita_membuat_putar_roda_warna2", "1F938-1F3FC-200D-2640-FE0F"},
    {"wanita_membuat_putar_roda_warna3", "1F938-1F3FD-200D-2640-FE0F"},
    {"wanita_membuat_putar_roda_warna4", "1F938-1F3FE-200D-2640-FE0F"},
    {"wanita_membuat_putar_roda_warna5", "1F938-1F3FF-200D-2640-FE0F"},
    {"lelaki_membuat_putar_roda", "1F938-200D-2642-FE0F"},
    {"lelaki_membuat_putar_roda_warna1", "1F938-1F3FB-200D-2642-FE0F"},
    {"lelaki_membuat_putar_roda_warna2", "1F938-1F3FC-200D-2642-FE0F"},
    {"lelaki_membuat_putar_roda_warna3", "1F938-1F3FD-200D-2642-FE0F"},
    {"lelaki_membuat_putar_roda_warna4", "1F938-1F3FE-200D-2642-FE0F"},
    {"lelaki_membuat_putar_roda_warna5", "1F938-1F3FF-200D-2642-FE0F"},
    {"wanita_menjugel", "1F939-200D-2640-FE0F"},
    {"wanita_menjugel_warna1", "1F939-1F3FB-200D-2640-FE0F"},
    {"wanita_menjugel_warna2", "1F939-1F3FC-200D-2640-FE0F"},
    {"wanita_menjugel_warna3", "1F939-1F3FD-200D-2640-FE0F"},
    {"wanita_menjugel_warna4", "1F939-1F3FE-200D-2640-FE0F"},
    {"wanita_menjugel_warna5", "1F939-1F3FF-200D-2640-FE0F"},
    {"lelaki_menjugel", "1F939-200D-2642-FE0F"},
    {"lelaki_menjugel_warna1", "1F939-1F3FB-200D-2642-FE0F"},
    {"lelaki_menjugel_warna2", "1F939-1F3FC-200D-2642-FE0F"},
    {"lelaki_menjugel_warna3", "1F939-1F3FD-200D-2642-FE0F"},
    {"lelaki_menjugel_warna4", "1F939-1F3FE-200D-2642-FE0F"},
    {"lelaki_menjugel_warna5", "1F939-1F3FF-200D-2642-FE0F"},
    {"wanita_bergusti", "1F93C-200D-2640-FE0F"},
    {"wanita_bergusti_warna1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"wanita_bergusti_warna1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"wanita_bergusti_warna1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"wanita_bergusti_warna1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"wanita_bergusti_warna2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"wanita_bergusti_warna2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"wanita_bergusti_warna2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"wanita_bergusti_warna2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"wanita_bergusti_warna3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"wanita_bergusti_warna3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"wanita_bergusti_warna3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"wanita_bergusti_warna3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"wanita_bergusti_warna4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"wanita_bergusti_warna4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"wanita_bergusti_warna4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"wanita_bergusti_warna4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"wanita_bergusti_warna5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"wanita_bergusti_warna5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"wanita_bergusti_warna5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"wanita_bergusti_warna5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"wanita_bergusti_warna1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"wanita_bergusti_warna2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"wanita_bergusti_warna3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"wanita_bergusti_warna4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"wanita_bergusti_warna5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"lelaki_bergusti", "1F93C-200D-2642-FE0F"},
    {"lelaki_bergusti_warna1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"lelaki_bergusti_warna1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"lelaki_bergusti_warna1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"lelaki_bergusti_warna1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"lelaki_bergusti_warna2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"lelaki_bergusti_warna2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"lelaki_bergusti_warna2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"lelaki_bergusti_warna2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"lelaki_bergusti_warna3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"lelaki_bergusti_warna3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"lelaki_bergusti_warna3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"lelaki_bergusti_warna3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"lelaki_bergusti_warna4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"lelaki_bergusti_warna4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"lelaki_bergusti_warna4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"lelaki_bergusti_warna4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"lelaki_bergusti_warna5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"lelaki_bergusti_warna5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"lelaki_bergusti_warna5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"lelaki_bergusti_warna5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"lelaki_bergusti_warna1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"lelaki_bergusti_warna2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"lelaki_bergusti_warna3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"lelaki_bergusti_warna4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"lelaki_bergusti_warna5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"wanita_bermain_polo_air", "1F93D-200D-2640-FE0F"},
    {"wanita_bermain_polo_air_warna1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"wanita_bermain_polo_air_warna2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"wanita_bermain_polo_air_warna3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"wanita_bermain_polo_air_warna4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"wanita_bermain_polo_air_warna5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"lelaki_bermain_polo_air", "1F93D-200D-2642-FE0F"},
    {"lelaki_bermain_polo_air_warna1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"lelaki_bermain_polo_air_warna2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"lelaki_bermain_polo_air_warna3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"lelaki_bermain_polo_air_warna4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"lelaki_bermain_polo_air_warna5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"wanita_bermain_bola_baling", "1F93E-200D-2640-FE0F"},
    {"wanita_bermain_bola_baling_warna1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"wanita_bermain_bola_baling_warna2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"wanita_bermain_bola_baling_warna3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"wanita_bermain_bola_baling_warna4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"wanita_bermain_bola_baling_warna5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"lelaki_bermain_bola_baling", "1F93E-200D-2642-FE0F"},
    {"lelaki_bermain_bola_baling_warna1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"lelaki_bermain_bola_baling_warna2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"lelaki_bermain_bola_baling_warna3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"lelaki_bermain_bola_baling_warna4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"lelaki_bermain_bola_baling_warna5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"adi_wira_wanita", "1F9B8-200D-2640-FE0F"},
    {"adi_wira_wanita_warna1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"adi_wira_wanita_warna2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"adi_wira_wanita_warna3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"adi_wira_wanita_warna4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"adi_wira_wanita_warna5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"adi_wira_lelaki", "1F9B8-200D-2642-FE0F"},
    {"adi_wira_lelaki_warna1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"adi_wira_lelaki_warna2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"adi_wira_lelaki_warna3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"adi_wira_lelaki_warna4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"adi_wira_lelaki_warna5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"ketua_penjahat_wanita", "1F9B9-200D-2640-FE0F"},
    {"ketua_penjahat_wanita_warna1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"ketua_penjahat_wanita_warna2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"ketua_penjahat_wanita_warna3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"ketua_penjahat_wanita_warna4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"ketua_penjahat_wanita_warna5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"ketua_penjahat_lelaki", "1F9B9-200D-2642-FE0F"},
    {"ketua_penjahat_lelaki_warna1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"ketua_penjahat_lelaki_warna2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"ketua_penjahat_lelaki_warna3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"ketua_penjahat_lelaki_warna4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"ketua_penjahat_lelaki_warna5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"perempuan_berdiri", "1F9CD-200D-2640-FE0F"},
    {"perempuan_berdiri_warna1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"perempuan_berdiri_warna2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"perempuan_berdiri_warna3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"perempuan_berdiri_warna4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"perempuan_berdiri_warna5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"lelaki_berdiri", "1F9CD-200D-2642-FE0F"},
    {"lelaki_berdiri_warna1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"lelaki_berdiri_warna2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"lelaki_berdiri_warna3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"lelaki_berdiri_warna4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"lelaki_berdiri_warna5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"perempuan_bersimpuh", "1F9CE-200D-2640-FE0F"},
    {"perempuan_bersimpuh_warna1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"perempuan_bersimpuh_warna2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"perempuan_bersimpuh_warna3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"perempuan_bersimpuh_warna4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"perempuan_bersimpuh_warna5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"perempuan_bersimpuh_menghadap_kanan", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_bersimpuh_menghadap_kanan_warna1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_bersimpuh_menghadap_kanan_warna2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_bersimpuh_menghadap_kanan_warna3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_bersimpuh_menghadap_kanan_warna4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"perempuan_bersimpuh_menghadap_kanan_warna5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"lelaki_bersimpuh", "1F9CE-200D-2642-FE0F"},
    {"lelaki_bersimpuh_warna1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"lelaki_bersimpuh_warna2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"lelaki_bersimpuh_warna3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"lelaki_bersimpuh_warna4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"lelaki_bersimpuh_warna5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"lelaki_bersimpuh_menghadap_kanan", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_bersimpuh_menghadap_kanan_warna1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_bersimpuh_menghadap_kanan_warna2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_bersimpuh_menghadap_kanan_warna3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_bersimpuh_menghadap_kanan_warna4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"lelaki_bersimpuh_menghadap_kanan_warna5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"perempuan_pekak", "1F9CF-200D-2640-FE0F"},
    {"perempuan_pekak_warna1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"perempuan_pekak_warna2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"perempuan_pekak_warna3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"perempuan_pekak_warna4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"perempuan_pekak_warna5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"lelaki_pekak", "1F9CF-200D-2642-FE0F"},
    {"lelaki_pekak_warna1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"lelaki_pekak_warna2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"lelaki_pekak_warna3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"lelaki_pekak_warna4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"lelaki_pekak_warna5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"wanita_janggut", "1F9D4-200D-2640-FE0F"},
    {"wanita_janggut_warna1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"wanita_janggut_warna2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"wanita_janggut_warna3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"wanita_janggut_warna4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"wanita_janggut_warna5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"lelaki_janggut", "1F9D4-200D-2642-FE0F"},
    {"lelaki_janggut_warna1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"lelaki_janggut_warna2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"lelaki_janggut_warna3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"lelaki_janggut_warna4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"lelaki_janggut_warna5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"wanita_dalam_bilik_berwap", "1F9D6-200D-2640-FE0F"},
    {"wanita_dalam_bilik_berwap_warna1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"wanita_dalam_bilik_berwap_warna2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"wanita_dalam_bilik_berwap_warna3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"wanita_dalam_bilik_berwap_warna4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"wanita_dalam_bilik_berwap_warna5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"lelaki_dalam_bilik_berwap", "1F9D6-200D-2642-FE0F"},
    {"lelaki_dalam_bilik_berwap_warna1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"lelaki_dalam_bilik_berwap_warna2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"lelaki_dalam_bilik_berwap_warna3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"lelaki_dalam_bilik_berwap_warna4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"lelaki_dalam_bilik_berwap_warna5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"wanita_mendaki", "1F9D7-200D-2640-FE0F"},
    {"wanita_mendaki_warna1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"wanita_mendaki_warna2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"wanita_mendaki_warna3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"wanita_mendaki_warna4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"wanita_mendaki_warna5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"lelaki_mendaki", "1F9D7-200D-2642-FE0F"},
    {"lelaki_mendaki_warna1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"lelaki_mendaki_warna2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"lelaki_mendaki_warna3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"lelaki_mendaki_warna4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"lelaki_mendaki_warna5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"wanita_duduk_teratai", "1F9D8-200D-2640-FE0F"},
    {"wanita_duduk_teratai_warna1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"wanita_duduk_teratai_warna2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"wanita_duduk_teratai_warna3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"wanita_duduk_teratai_warna4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"wanita_duduk_teratai_warna5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"lelaki_duduk_teratai", "1F9D8-200D-2642-FE0F"},
    {"lelaki_duduk_teratai_warna1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"lelaki_duduk_teratai_warna2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"lelaki_duduk_teratai_warna3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"lelaki_duduk_teratai_warna4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"lelaki_duduk_teratai_warna5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"ahli_sihir_wanita", "1F9D9-200D-2640-FE0F"},
    {"ahli_sihir_wanita_warna1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"ahli_sihir_wanita_warna2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"ahli_sihir_wanita_warna3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"ahli_sihir_wanita_warna4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"ahli_sihir_wanita_warna5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"ahli_sihir_lelaki", "1F9D9-200D-2642-FE0F"},
    {"ahli_sihir_lelaki_warna1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"ahli_sihir_lelaki_warna2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"ahli_sihir_lelaki_warna3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"ahli_sihir_lelaki_warna4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"ahli_sihir_lelaki_warna5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"pari_pari_wanita", "1F9DA-200D-2640-FE0F"},
    {"pari_pari_wanita_warna1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"pari_pari_wanita_warna2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"pari_pari_wanita_warna3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"pari_pari_wanita_warna4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"pari_pari_wanita_warna5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"pari_pari_lelaki", "1F9DA-200D-2642-FE0F"},
    {"pari_pari_lelaki_warna1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"pari_pari_lelaki_warna2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"pari_pari_lelaki_warna3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"pari_pari_lelaki_warna4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"pari_pari_lelaki_warna5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"puntianak_wanita", "1F9DB-200D-2640-FE0F"},
    {"puntianak_wanita_warna1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"puntianak_wanita_warna2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"puntianak_wanita_warna3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"puntianak_wanita_warna4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"puntianak_wanita_warna5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"puntianak_lelaki", "1F9DB-200D-2642-FE0F"},
    {"puntianak_lelaki_warna1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"puntianak_lelaki_warna2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"puntianak_lelaki_warna3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"puntianak_lelaki_warna4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"puntianak_lelaki_warna5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"ikan_duyung", "1F9DC-200D-2640-FE0F"},
    {"ikan_duyung_warna1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"ikan_duyung_warna2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"ikan_duyung_warna3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"ikan_duyung_warna4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"ikan_duyung_warna5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"duyung_jantan", "1F9DC-200D-2642-FE0F"},
    {"duyung_jantan_warna1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"duyung_jantan_warna2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"duyung_jantan_warna3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"duyung_jantan_warna4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"duyung_jantan_warna5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"orang_halus_wanita", "1F9DD-200D-2640-FE0F"},
    {"orang_halus_wanita_warna1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"orang_halus_wanita_warna2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"orang_halus_wanita_warna3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"orang_halus_wanita_warna4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"orang_halus_wanita_warna5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"orang_halus_lelaki", "1F9DD-200D-2642-FE0F"},
    {"orang_halus_lelaki_warna1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"orang_halus_lelaki_warna2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"orang_halus_lelaki_warna3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"orang_halus_lelaki_warna4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"orang_halus_lelaki_warna5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"jin_betina", "1F9DE-200D-2640-FE0F"},
    {"jin_jantan", "1F9DE-200D-2642-FE0F"},
    {"zombi_wanita", "1F9DF-200D-2640-FE0F"},
    {"zombi_lelaki", "1F9DF-200D-2642-FE0F"},
    {"lelaki_rambut_merah", "1F468-200D-1F9B0"},
    {"lelaki_rambut_merah_warna1", "1F468-1F3FB-200D-1F9B0"},
    {"lelaki_rambut_merah_warna2", "1F468-1F3FC-200D-1F9B0"},
    {"lelaki_rambut_merah_warna3", "1F468-1F3FD-200D-1F9B0"},
    {"lelaki_rambut_merah_warna4", "1F468-1F3FE-200D-1F9B0"},
    {"lelaki_rambut_merah_warna5", "1F468-1F3FF-200D-1F9B0"},
    {"lelaki_rambut_kerinting", "1F468-200D-1F9B1"},
    {"lelaki_rambut_kerinting_warna1", "1F468-1F3FB-200D-1F9B1"},
    {"lelaki_rambut_kerinting_warna2", "1F468-1F3FC-200D-1F9B1"},
    {"lelaki_rambut_kerinting_warna3", "1F468-1F3FD-200D-1F9B1"},
    {"lelaki_rambut_kerinting_warna4", "1F468-1F3FE-200D-1F9B1"},
    {"lelaki_rambut_kerinting_warna5", "1F468-1F3FF-200D-1F9B1"},
    {"lelaki_botak", "1F468-200D-1F9B2"},
    {"lelaki_botak_warna1", "1F468-1F3FB-200D-1F9B2"},
    {"lelaki_botak_warna2", "1F468-1F3FC-200D-1F9B2"},
    {"lelaki_botak_warna3", "1F468-1F3FD-200D-1F9B2"},
    {"lelaki_botak_warna4", "1F468-1F3FE-200D-1F9B2"},
    {"lelaki_botak_warna5", "1F468-1F3FF-200D-1F9B2"},
    {"lelaki_rambut_putih", "1F468-200D-1F9B3"},
    {"lelaki_rambut_putih_warna1", "1F468-1F3FB-200D-1F9B3"},
    {"lelaki_rambut_putih_warna2", "1F468-1F3FC-200D-1F9B3"},
    {"lelaki_rambut_putih_warna3", "1F468-1F3FD-200D-1F9B3"},
    {"lelaki_rambut_putih_warna4", "1F468-1F3FE-200D-1F9B3"},
    {"lelaki_rambut_putih_warna5", "1F468-1F3FF-200D-1F9B3"},
    {"perempuan_rambut_merah", "1F469-200D-1F9B0"},
    {"perempuan_rambut_merah_warna1", "1F469-1F3FB-200D-1F9B0"},
    {"perempuan_rambut_merah_warna2", "1F469-1F3FC-200D-1F9B0"},
    {"perempuan_rambut_merah_warna3", "1F469-1F3FD-200D-1F9B0"},
    {"perempuan_rambut_merah_warna4", "1F469-1F3FE-200D-1F9B0"},
    {"perempuan_rambut_merah_warna5", "1F469-1F3FF-200D-1F9B0"},
    {"perempuan_rambut_kerinting", "1F469-200D-1F9B1"},
    {"perempuan_rambut_kerinting_warna1", "1F469-1F3FB-200D-1F9B1"},
    {"perempuan_rambut_kerinting_warna2", "1F469-1F3FC-200D-1F9B1"},
    {"perempuan_rambut_kerinting_warna3", "1F469-1F3FD-200D-1F9B1"},
    {"perempuan_rambut_kerinting_warna4", "1F469-1F3FE-200D-1F9B1"},
    {"perempuan_rambut_kerinting_warna5", "1F469-1F3FF-200D-1F9B1"},
    {"perempuan_botak", "1F469-200D-1F9B2"},
    {"perempuan_botak_warna1", "1F469-1F3FB-200D-1F9B2"},
    {"perempuan_botak_warna2", "1F469-1F3FC-200D-1F9B2"},
    {"perempuan_botak_warna3", "1F469-1F3FD-200D-1F9B2"},
    {"perempuan_botak_warna4", "1F469-1F3FE-200D-1F9B2"},
    {"perempuan_botak_warna5", "1F469-1F3FF-200D-1F9B2"},
    {"perempuan_rambut_putih", "1F469-200D-1F9B3"},
    {"perempuan_rambut_putih_warna1", "1F469-1F3FB-200D-1F9B3"},
    {"perempuan_rambut_putih_warna2", "1F469-1F3FC-200D-1F9B3"},
    {"perempuan_rambut_putih_warna3", "1F469-1F3FD-200D-1F9B3"},
    {"perempuan_rambut_putih_warna4", "1F469-1F3FE-200D-1F9B3"},
    {"perempuan_rambut_putih_warna5", "1F469-1F3FF-200D-1F9B3"},
    {"dewasa_rambut_merah", "1F9D1-200D-1F9B0"},
    {"dewasa_rambut_merah_warna1", "1F9D1-1F3FB-200D-1F9B0"},
    {"dewasa_rambut_merah_warna2", "1F9D1-1F3FC-200D-1F9B0"},
    {"dewasa_rambut_merah_warna3", "1F9D1-1F3FD-200D-1F9B0"},
    {"dewasa_rambut_merah_warna4", "1F9D1-1F3FE-200D-1F9B0"},
    {"dewasa_rambut_merah_warna5", "1F9D1-1F3FF-200D-1F9B0"},
    {"dewasa_rambut_kerinting", "1F9D1-200D-1F9B1"},
    {"dewasa_rambut_kerinting_warna1", "1F9D1-1F3FB-200D-1F9B1"},
    {"dewasa_rambut_kerinting_warna2", "1F9D1-1F3FC-200D-1F9B1"},
    {"dewasa_rambut_kerinting_warna3", "1F9D1-1F3FD-200D-1F9B1"},
    {"dewasa_rambut_kerinting_warna4", "1F9D1-1F3FE-200D-1F9B1"},
    {"dewasa_rambut_kerinting_warna5", "1F9D1-1F3FF-200D-1F9B1"},
    {"dewasa_botak", "1F9D1-200D-1F9B2"},
    {"dewasa_botak_warna1", "1F9D1-1F3FB-200D-1F9B2"},
    {"dewasa_botak_warna2", "1F9D1-1F3FC-200D-1F9B2"},
    {"dewasa_botak_warna3", "1F9D1-1F3FD-200D-1F9B2"},
    {"dewasa_botak_warna4", "1F9D1-1F3FE-200D-1F9B2"},
    {"dewasa_botak_warna5", "1F9D1-1F3FF-200D-1F9B2"},
    {"dewasa_rambut_putih", "1F9D1-200D-1F9B3"},
    {"dewasa_rambut_putih_warna1", "1F9D1-1F3FB-200D-1F9B3"},
    {"dewasa_rambut_putih_warna2", "1F9D1-1F3FC-200D-1F9B3"},
    {"dewasa_rambut_putih_warna3", "1F9D1-1F3FD-200D-1F9B3"},
    {"dewasa_rambut_putih_warna4", "1F9D1-1F3FE-200D-1F9B3"},
    {"dewasa_rambut_putih_warna5", "1F9D1-1F3FF-200D-1F9B3"},
    {"rantai_terputus", "26D3-FE0F-200D-1F4A5"},
    {"hati_membara", "2764-FE0F-200D-1F525"},
    {"hati_sembuh", "2764-FE0F-200D-1FA79"},
    {"cendawan_perang", "1F344-200D-1F7EB"},
    {"limau_nipis", "1F34B-200D-1F7E9"},
    {"bendera_birumerah_jambu_dan_putih", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"bendera_pelangi", "1F3F3-FE0F-200D-1F308"},
    {"bendera_lanun", "1F3F4-200D-2620-FE0F"},
    {"kucing_hitam", "1F408-200D-2B1B"},
    {"anjing_khidmat", "1F415-200D-1F9BA"},
    {"burung_hitam", "1F426-200D-2B1B"},
    {"burung_phoenix", "1F426-200D-1F525"},
    {"beruang_kutub", "1F43B-200D-2744-FE0F"},
    {"mata_dalam_gelembung_pertuturan", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"muka_menghembuskan_nafas", "1F62E-200D-1F4A8"},
    {"muka_dengan_mata_berpusar", "1F635-200D-1F4AB"},
    {"muka_dikepung_awan", "1F636-200D-1F32B-FE0F"},
    {"menggeleng_kepala", "1F642-200D-2194-FE0F"},
    {"mengangguk_kepala", "1F642-200D-2195-FE0F"},
    {"penari_balet", "1F9D1-200D-1FA70"},
    {"penari_balet_warna1", "1F9D1-1F3FB-200D-1FA70"},
    {"penari_balet_warna2", "1F9D1-1F3FC-200D-1FA70"},
    {"penari_balet_warna3", "1F9D1-1F3FD-200D-1FA70"},
    {"penari_balet_warna4", "1F9D1-1F3FE-200D-1FA70"},
    {"penari_balet_warna5", "1F9D1-1F3FF-200D-1FA70"},
};

