// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_es[] = {
    {"exclamacion_e_interrogacion", "2049"},
    {"simbolo_de_marca_comercial", "2122"},
    {"informacion", "2139"},
    {"flecha_izquierda_y_derecha", "2194"},
    {"flecha_arriba_y_abajo", "2195"},
    {"flecha_hacia_la_esquina_superior_izquierda", "2196"},
    {"flecha_hacia_la_esquina_superior_derecha", "2197"},
    {"flecha_hacia_la_esquina_inferior_derecha", "2198"},
    {"flecha_hacia_la_esquina_inferior_izquierda", "2199"},
    {"teclado", "2328"},
    {"sol", "2600"},
    {"nube", "2601"},
    {"paraguas", "2602"},
    {"muneco_de_nieve_con_nieve", "2603"},
    {"meteorito", "2604"},
    {"casilla_con_marca_de_verificacion", "2611"},
    {"paraguas_con_gotas_de_lluvia", "2614"},
    {"bebida_caliente", "2615"},
    {"trebol", "2618"},
    {"calavera_y_huesos_cruzados", "2620"},
    {"radiactivo", "2622"},
    {"riesgo_biologico", "2623"},
    {"cruz_ortodoxa", "2626"},
    {"rueda_del_dharma", "2638"},
    {"cara_con_el_ceno_fruncido", "2639"},
    {"signo_femenino", "2640"},
    {"signo_masculino", "2642"},
    {"aries", "2648"},
    {"tauro", "2649"},
    {"sagitario", "2650"},
    {"capricornio", "2651"},
    {"acuario", "2652"},
    {"piscis", "2653"},
    {"palo_de_picas", "2660"},
    {"palo_de_treboles", "2663"},
    {"palo_de_corazones", "2665"},
    {"palo_de_diamantes", "2666"},
    {"aguas_termales", "2668"},
    {"martillo_y_pico", "2692"},
    {"ancla", "2693"},
    {"espadas_cruzadas", "2694"},
    {"simbolo_de_medicina", "2695"},
    {"balanza", "2696"},
    {"alambique", "2697"},
    {"engranaje", "2699"},
    {"tijeras", "2702"},
    {"boton_de_marca_de_verificacion", "2705"},
    {"avion", "2708"},
    {"sobre", "2709"},
    {"pluma_negra", "2712"},
    {"marca_de_verificacion", "2714"},
    {"multiplicacion", "2716"},
    {"estrella_de_david", "2721"},
    {"chispas", "2728"},
    {"asterisco_de_ocho_puntas", "2733"},
    {"estrella_de_ocho_puntas", "2734"},
    {"copo_de_nieve", "2744"},
    {"chispa", "2747"},
    {"interrogacion_roja", "2753"},
    {"interrogacion_blanca", "2754"},
    {"exclamacion_blanca", "2755"},
    {"exclamacion_roja", "2757"},
    {"exclamacion_de_corazon", "2763"},
    {"corazon_rojo", "2764"},
    {"mas", "2795"},
    {"menos", "2796"},
    {"division", "2797"},
    {"flecha_derecha_curvandose_hacia_arriba", "2934"},
    {"flecha_derecha_curvandose_hacia_abajo", "2935"},
    {"guion_ondulado", "3030"},
    {"ideograma_japones_para_enhorabuena", "3297"},
    {"ideograma_japones_para_secreto", "3299"},
    {"copyright", "00A9"},
    {"marca_registrada", "00AE"},
    {"exclamacion_doble", "203C"},
    {"flecha_derecha_curvandose_a_la_izquierda", "21A9"},
    {"flecha_izquierda_curvandose_a_la_derecha", "21AA"},
    {"reloj", "231A"},
    {"reloj_de_arena_sin_tiempo", "231B"},
    {"expulsar", "23CF"},
    {"avance_rapido", "23E9"},
    {"retroceso_rapido", "23EA"},
    {"triangulo_doble_hacia_arriba", "23EB"},
    {"triangulo_doble_hacia_abajo", "23EC"},
    {"pista_siguiente", "23ED"},
    {"pista_anterior", "23EE"},
    {"reproducir_o_pausa", "23EF"},
    {"reloj_despertador", "23F0"},
    {"cronometro", "23F1"},
    {"temporizador", "23F2"},
    {"reloj_de_arena_con_tiempo", "23F3"},
    {"pausa", "23F8"},
    {"detener", "23F9"},
    {"grabar", "23FA"},
    {"m_en_circulo", "24C2"},
    {"cuadrado_negro_pequeno", "25AA"},
    {"cuadrado_blanco_pequeno", "25AB"},
    {"reproducir", "25B6"},
    {"retroceso", "25C0"},
    {"cuadrado_blanco_mediano", "25FB"},
    {"cuadrado_negro_mediano", "25FC"},
    {"cuadrado_blanco_mediano_pequeno", "25FD"},
    {"cuadrado_negro_mediano_pequeno", "25FE"},
    {"telefono", "260E"},
    {"dedo_indice_hacia_arriba", "261D"},
    {"dedo_indice_hacia_arriba_color1", "261D-1F3FB"},
    {"dedo_indice_hacia_arriba_color2", "261D-1F3FC"},
    {"dedo_indice_hacia_arriba_color3", "261D-1F3FD"},
    {"dedo_indice_hacia_arriba_color4", "261D-1F3FE"},
    {"dedo_indice_hacia_arriba_color5", "261D-1F3FF"},
    {"media_luna_y_estrella", "262A"},
    {"simbolo_de_la_paz", "262E"},
    {"yin_yang", "262F"},
    {"cara_sonriente", "263A"},
    {"geminis", "264A"},
    {"cancer", "264B"},
    {"leo", "264C"},
    {"virgo", "264D"},
    {"libra", "264E"},
    {"escorpio", "264F"},
    {"peon_de_ajedrez", "265F"},
    {"simbolo_de_reciclaje", "267B"},
    {"infinito", "267E"},
    {"simbolo_de_silla_de_ruedas", "267F"},
    {"simbolo_de_atomo", "269B"},
    {"flor_de_lis", "269C"},
    {"advertencia", "26A0"},
    {"alto_voltaje", "26A1"},
    {"simbolo_de_transgenero", "26A7"},
    {"circulo_blanco", "26AA"},
    {"circulo_negro", "26AB"},
    {"ataud", "26B0"},
    {"urna_funeraria", "26B1"},
    {"balon_de_futbol", "26BD"},
    {"beisbol", "26BE"},
    {"muneco_de_nieve", "26C4"},
    {"sol_detras_de_una_nube", "26C5"},
    {"nube_con_rayo_y_lluvia", "26C8"},
    {"ofiuco", "26CE"},
    {"pico", "26CF"},
    {"casco_con_una_cruz_blanca", "26D1"},
    {"cadenas", "26D3"},
    {"direccion_prohibida", "26D4"},
    {"santuario_sintoista", "26E9"},
    {"iglesia", "26EA"},
    {"montana", "26F0"},
    {"sombrilla_en_la_arena", "26F1"},
    {"fuente", "26F2"},
    {"banderin_en_hoyo", "26F3"},
    {"ferri", "26F4"},
    {"velero", "26F5"},
    {"persona_esquiando", "26F7"},
    {"patin_de_hielo", "26F8"},
    {"persona_botando_un_balon", "26F9"},
    {"persona_botando_un_balon_color1", "26F9-1F3FB"},
    {"persona_botando_un_balon_color2", "26F9-1F3FC"},
    {"persona_botando_un_balon_color3", "26F9-1F3FD"},
    {"persona_botando_un_balon_color4", "26F9-1F3FE"},
    {"persona_botando_un_balon_color5", "26F9-1F3FF"},
    {"tienda_de_campana", "26FA"},
    {"surtidor_de_gasolina", "26FD"},
    {"puno_en_alto", "270A"},
    {"puno_en_alto_color1", "270A-1F3FB"},
    {"puno_en_alto_color2", "270A-1F3FC"},
    {"puno_en_alto_color3", "270A-1F3FD"},
    {"puno_en_alto_color4", "270A-1F3FE"},
    {"puno_en_alto_color5", "270A-1F3FF"},
    {"mano_levantada", "270B"},
    {"mano_levantada_color1", "270B-1F3FB"},
    {"mano_levantada_color2", "270B-1F3FC"},
    {"mano_levantada_color3", "270B-1F3FD"},
    {"mano_levantada_color4", "270B-1F3FE"},
    {"mano_levantada_color5", "270B-1F3FF"},
    {"mano_con_senal_de_victoria", "270C"},
    {"mano_con_senal_de_victoria_color1", "270C-1F3FB"},
    {"mano_con_senal_de_victoria_color2", "270C-1F3FC"},
    {"mano_con_senal_de_victoria_color3", "270C-1F3FD"},
    {"mano_con_senal_de_victoria_color4", "270C-1F3FE"},
    {"mano_con_senal_de_victoria_color5", "270C-1F3FF"},
    {"mano_escribiendo", "270D"},
    {"mano_escribiendo_color1", "270D-1F3FB"},
    {"mano_escribiendo_color2", "270D-1F3FC"},
    {"mano_escribiendo_color3", "270D-1F3FD"},
    {"mano_escribiendo_color4", "270D-1F3FE"},
    {"mano_escribiendo_color5", "270D-1F3FF"},
    {"lapiz", "270F"},
    {"cruz_latina", "271D"},
    {"marca_de_cruz", "274C"},
    {"boton_con_marca_de_cruz", "274E"},
    {"flecha_hacia_la_derecha", "27A1"},
    {"bucle", "27B0"},
    {"bucle_doble", "27BF"},
    {"flecha_hacia_la_izquierda", "2B05"},
    {"flecha_hacia_arriba", "2B06"},
    {"flecha_hacia_abajo", "2B07"},
    {"cuadrado_negro_grande", "2B1B"},
    {"cuadrado_blanco_grande", "2B1C"},
    {"estrella", "2B50"},
    {"circulo_rojo_hueco", "2B55"},
    {"marca_de_alternancia", "303D"},
    {"dragon_rojo_de_mahjong", "1F004"},
    {"comodin", "1F0CF"},
    {"grupo_sanguineo_a", "1F170"},
    {"grupo_sanguineo_b", "1F171"},
    {"grupo_sanguineo_tipo_o", "1F17E"},
    {"aparcamiento", "1F17F"},
    {"grupo_sanguineo_ab", "1F18E"},
    {"borrar", "1F191"},
    {"boton_cool", "1F192"},
    {"boton_free", "1F193"},
    {"simbolo_de_identificacion", "1F194"},
    {"boton_new", "1F195"},
    {"boton_ng", "1F196"},
    {"boton_ok", "1F197"},
    {"simbolo_de_socorro", "1F198"},
    {"boton_up", "1F199"},
    {"boton_vs", "1F19A"},
    {"indicador_regional_a", "1F1E6"},
    {"indicador_regional_b", "1F1E7"},
    {"indicador_regional_c", "1F1E8"},
    {"indicador_regional_d", "1F1E9"},
    {"indicador_regional_e", "1F1EA"},
    {"indicador_regional_f", "1F1EB"},
    {"indicador_regional_g", "1F1EC"},
    {"indicador_regional_h", "1F1ED"},
    {"indicador_regional_i", "1F1EE"},
    {"indicador_regional_j", "1F1EF"},
    {"indicador_regional_k", "1F1F0"},
    {"indicador_regional_l", "1F1F1"},
    {"indicador_regional_m", "1F1F2"},
    {"indicador_regional_n", "1F1F3"},
    {"indicador_regional_o", "1F1F4"},
    {"indicador_regional_p", "1F1F5"},
    {"indicador_regional_q", "1F1F6"},
    {"indicador_regional_r", "1F1F7"},
    {"indicador_regional_s", "1F1F8"},
    {"indicador_regional_t", "1F1F9"},
    {"indicador_regional_u", "1F1FA"},
    {"indicador_regional_v", "1F1FB"},
    {"indicador_regional_w", "1F1FC"},
    {"indicador_regional_x", "1F1FD"},
    {"indicador_regional_y", "1F1FE"},
    {"indicador_regional_z", "1F1FF"},
    {"ideograma_japones_para_aqui", "1F201"},
    {"ideograma_japones_para_de_cortesia", "1F202"},
    {"ideograma_japones_para_gratis", "1F21A"},
    {"ideograma_japones_para_reservado", "1F22F"},
    {"ideograma_japones_para_prohibido", "1F232"},
    {"ideograma_japones_para_vacante", "1F233"},
    {"ideograma_japones_para_aprobado", "1F234"},
    {"ideograma_japones_para_completo", "1F235"},
    {"ideograma_japones_para_de_pago", "1F236"},
    {"ideograma_japones_para_cantidad_mensual", "1F237"},
    {"ideograma_japones_para_aplicacion", "1F238"},
    {"ideograma_japones_para_descuento", "1F239"},
    {"ideograma_japones_para_abierto", "1F23A"},
    {"ideograma_japones_para_ganga", "1F250"},
    {"ideograma_japones_para_aceptable", "1F251"},
    {"ciclon", "1F300"},
    {"bruma", "1F301"},
    {"paraguas_cerrado", "1F302"},
    {"noche_estrellada", "1F303"},
    {"amanecer_sobre_montanas", "1F304"},
    {"amanecer", "1F305"},
    {"ciudad_al_atardecer", "1F306"},
    {"puesta_del_sol", "1F307"},
    {"arcoiris", "1F308"},
    {"puente_de_noche", "1F309"},
    {"ola_de_mar", "1F30A"},
    {"volcan", "1F30B"},
    {"via_lactea", "1F30C"},
    {"globo_terraqueo_mostrando_europa_y_africa", "1F30D"},
    {"globo_terraqueo_mostrando_america", "1F30E"},
    {"globo_terraqueo_mostrando_asia_y_australia", "1F30F"},
    {"globo_terraqueo_con_meridianos", "1F310"},
    {"luna_nueva", "1F311"},
    {"luna_creciente", "1F312"},
    {"luna_en_cuarto_creciente", "1F313"},
    {"luna_gibosa_creciente", "1F314"},
    {"luna_llena", "1F315"},
    {"luna_gibosa_menguante", "1F316"},
    {"luna_en_cuarto_menguante", "1F317"},
    {"luna_menguante", "1F318"},
    {"luna", "1F319"},
    {"luna_nueva_con_cara", "1F31A"},
    {"luna_de_cuarto_creciente_con_cara", "1F31B"},
    {"luna_de_cuarto_menguante_con_cara", "1F31C"},
    {"luna_llena_con_cara", "1F31D"},
    {"sol_con_cara", "1F31E"},
    {"estrella_brillante", "1F31F"},
    {"estrella_fugaz", "1F320"},
    {"termometro", "1F321"},
    {"sol_detras_de_una_nube_pequena", "1F324"},
    {"sol_detras_de_una_nube_grande", "1F325"},
    {"sol_detras_de_una_nube_con_lluvia", "1F326"},
    {"nube_con_lluvia", "1F327"},
    {"nube_con_nieve", "1F328"},
    {"nube_con_rayo", "1F329"},
    {"tornado", "1F32A"},
    {"niebla", "1F32B"},
    {"cara_de_viento", "1F32C"},
    {"perrito_caliente", "1F32D"},
    {"taco", "1F32E"},
    {"burrito", "1F32F"},
    {"castana", "1F330"},
    {"planta_joven", "1F331"},
    {"arbol_de_hoja_perenne", "1F332"},
    {"arbol_de_hoja_caduca", "1F333"},
    {"palmera", "1F334"},
    {"cactus", "1F335"},
    {"chile_picante", "1F336"},
    {"tulipan", "1F337"},
    {"flor_de_cerezo", "1F338"},
    {"rosa", "1F339"},
    {"flor_de_hibisco", "1F33A"},
    {"girasol", "1F33B"},
    {"flor", "1F33C"},
    {"espiga_de_maiz", "1F33D"},
    {"espiga_de_arroz", "1F33E"},
    {"hierba", "1F33F"},
    {"trebol_de_cuatro_hojas", "1F340"},
    {"hoja_de_arce", "1F341"},
    {"hojas_caidas", "1F342"},
    {"hojas_revoloteando_al_viento", "1F343"},
    {"champinon", "1F344"},
    {"tomate", "1F345"},
    {"berenjena", "1F346"},
    {"uvas", "1F347"},
    {"melon", "1F348"},
    {"sandia", "1F349"},
    {"mandarina", "1F34A"},
    {"limon", "1F34B"},
    {"platano", "1F34C"},
    {"pina", "1F34D"},
    {"manzana_roja", "1F34E"},
    {"manzana_verde", "1F34F"},
    {"pera", "1F350"},
    {"melocoton", "1F351"},
    {"cerezas", "1F352"},
    {"fresa", "1F353"},
    {"hamburguesa", "1F354"},
    {"pizza", "1F355"},
    {"carne_con_hueso", "1F356"},
    {"muslo_de_pollo", "1F357"},
    {"galleta_de_arroz", "1F358"},
    {"bola_de_arroz", "1F359"},
    {"arroz_cocido", "1F35A"},
    {"arroz_con_curry", "1F35B"},
    {"tazon_de_fideos", "1F35C"},
    {"espagueti", "1F35D"},
    {"pan_de_molde", "1F35E"},
    {"patatas_fritas", "1F35F"},
    {"patata_asada", "1F360"},
    {"dango", "1F361"},
    {"brocheta", "1F362"},
    {"sushi", "1F363"},
    {"gamba_frita", "1F364"},
    {"pastel_de_pescado_japones", "1F365"},
    {"cucurucho_de_helado", "1F366"},
    {"granizado_hawaiano", "1F367"},
    {"helado", "1F368"},
    {"donut", "1F369"},
    {"galleta", "1F36A"},
    {"tableta_de_chocolate", "1F36B"},
    {"caramelo", "1F36C"},
    {"piruleta", "1F36D"},
    {"flan", "1F36E"},
    {"tarro_de_miel", "1F36F"},
    {"trozo_de_tarta", "1F370"},
    {"caja_de_bento", "1F371"},
    {"olla_de_comida", "1F372"},
    {"cocinar", "1F373"},
    {"tenedor_y_cuchillo", "1F374"},
    {"tazon_de_te", "1F375"},
    {"sake", "1F376"},
    {"copa_de_vino", "1F377"},
    {"copa_de_coctel", "1F378"},
    {"bebida_tropical", "1F379"},
    {"jarra_de_cerveza", "1F37A"},
    {"jarras_de_cerveza_brindando", "1F37B"},
    {"biberon", "1F37C"},
    {"cuchillo_y_tenedor_con_un_plato", "1F37D"},
    {"botella_descorchada", "1F37E"},
    {"palomitas", "1F37F"},
    {"lazo", "1F380"},
    {"regalo", "1F381"},
    {"tarta_de_cumpleanos", "1F382"},
    {"calabaza_de_halloween", "1F383"},
    {"arbol_de_navidad", "1F384"},
    {"papa_noel", "1F385"},
    {"papa_noel_color1", "1F385-1F3FB"},
    {"papa_noel_color2", "1F385-1F3FC"},
    {"papa_noel_color3", "1F385-1F3FD"},
    {"papa_noel_color4", "1F385-1F3FE"},
    {"papa_noel_color5", "1F385-1F3FF"},
    {"fuegos_artificiales", "1F386"},
    {"bengala", "1F387"},
    {"globo", "1F388"},
    {"canon_de_confeti", "1F389"},
    {"bola_de_confeti", "1F38A"},
    {"arbol_de_tanabata", "1F38B"},
    {"banderas_cruzadas", "1F38C"},
    {"decoracion_de_pino", "1F38D"},
    {"munecas_japonesas", "1F38E"},
    {"banderin_de_carpas", "1F38F"},
    {"campanilla_de_viento", "1F390"},
    {"ceremonia_de_contemplacion_de_la_luna", "1F391"},
    {"mochila_escolar", "1F392"},
    {"birrete", "1F393"},
    {"medalla_militar", "1F396"},
    {"lazo_conmemorativo", "1F397"},
    {"microfono_de_estudio", "1F399"},
    {"control_de_volumen", "1F39A"},
    {"ruedas_de_control", "1F39B"},
    {"fotograma_de_pelicula", "1F39E"},
    {"entradas", "1F39F"},
    {"caballo_de_tiovivo", "1F3A0"},
    {"noria_de_feria", "1F3A1"},
    {"montana_rusa", "1F3A2"},
    {"cana_de_pescar", "1F3A3"},
    {"microfono", "1F3A4"},
    {"camara_de_cine", "1F3A5"},
    {"cine", "1F3A6"},
    {"auricular", "1F3A7"},
    {"paleta_de_pintor", "1F3A8"},
    {"sombrero_de_copa", "1F3A9"},
    {"carpa_de_circo", "1F3AA"},
    {"tique", "1F3AB"},
    {"claqueta", "1F3AC"},
    {"mascaras_de_teatro", "1F3AD"},
    {"mando_de_videoconsola", "1F3AE"},
    {"diana", "1F3AF"},
    {"maquina_tragaperras", "1F3B0"},
    {"bola_negra_de_billar", "1F3B1"},
    {"dado", "1F3B2"},
    {"bolos", "1F3B3"},
    {"cartas_de_flores", "1F3B4"},
    {"nota_musical", "1F3B5"},
    {"notas_musicales", "1F3B6"},
    {"saxofon", "1F3B7"},
    {"guitarra", "1F3B8"},
    {"teclado_musical", "1F3B9"},
    {"trompeta", "1F3BA"},
    {"violin", "1F3BB"},
    {"pentagrama", "1F3BC"},
    {"camiseta_sin_mangas", "1F3BD"},
    {"pelota_de_tenis", "1F3BE"},
    {"esquis", "1F3BF"},
    {"balon_de_baloncesto", "1F3C0"},
    {"bandera_de_cuadros", "1F3C1"},
    {"practicante_de_snowboard", "1F3C2"},
    {"practicante_de_snowboard_color1", "1F3C2-1F3FB"},
    {"practicante_de_snowboard_color2", "1F3C2-1F3FC"},
    {"practicante_de_snowboard_color3", "1F3C2-1F3FD"},
    {"practicante_de_snowboard_color4", "1F3C2-1F3FE"},
    {"practicante_de_snowboard_color5", "1F3C2-1F3FF"},
    {"persona_corriendo", "1F3C3"},
    {"persona_corriendo_color1", "1F3C3-1F3FB"},
    {"persona_corriendo_color2", "1F3C3-1F3FC"},
    {"persona_corriendo_color3", "1F3C3-1F3FD"},
    {"persona_corriendo_color4", "1F3C3-1F3FE"},
    {"persona_corriendo_color5", "1F3C3-1F3FF"},
    {"persona_haciendo_surf", "1F3C4"},
    {"persona_haciendo_surf_color1", "1F3C4-1F3FB"},
    {"persona_haciendo_surf_color2", "1F3C4-1F3FC"},
    {"persona_haciendo_surf_color3", "1F3C4-1F3FD"},
    {"persona_haciendo_surf_color4", "1F3C4-1F3FE"},
    {"persona_haciendo_surf_color5", "1F3C4-1F3FF"},
    {"medalla_deportiva", "1F3C5"},
    {"trofeo", "1F3C6"},
    {"carrera_de_caballos", "1F3C7"},
    {"carrera_de_caballos_color1", "1F3C7-1F3FB"},
    {"carrera_de_caballos_color2", "1F3C7-1F3FC"},
    {"carrera_de_caballos_color3", "1F3C7-1F3FD"},
    {"carrera_de_caballos_color4", "1F3C7-1F3FE"},
    {"carrera_de_caballos_color5", "1F3C7-1F3FF"},
    {"balon_de_futbol_americano", "1F3C8"},
    {"balon_de_rugby", "1F3C9"},
    {"persona_nadando", "1F3CA"},
    {"persona_nadando_color1", "1F3CA-1F3FB"},
    {"persona_nadando_color2", "1F3CA-1F3FC"},
    {"persona_nadando_color3", "1F3CA-1F3FD"},
    {"persona_nadando_color4", "1F3CA-1F3FE"},
    {"persona_nadando_color5", "1F3CA-1F3FF"},
    {"persona_levantando_pesas", "1F3CB"},
    {"persona_levantando_pesas_color1", "1F3CB-1F3FB"},
    {"persona_levantando_pesas_color2", "1F3CB-1F3FC"},
    {"persona_levantando_pesas_color3", "1F3CB-1F3FD"},
    {"persona_levantando_pesas_color4", "1F3CB-1F3FE"},
    {"persona_levantando_pesas_color5", "1F3CB-1F3FF"},
    {"persona_jugando_al_golf", "1F3CC"},
    {"persona_jugando_al_golf_color1", "1F3CC-1F3FB"},
    {"persona_jugando_al_golf_color2", "1F3CC-1F3FC"},
    {"persona_jugando_al_golf_color3", "1F3CC-1F3FD"},
    {"persona_jugando_al_golf_color4", "1F3CC-1F3FE"},
    {"persona_jugando_al_golf_color5", "1F3CC-1F3FF"},
    {"moto", "1F3CD"},
    {"coche_de_carreras", "1F3CE"},
    {"criquet", "1F3CF"},
    {"pelota_de_voleibol", "1F3D0"},
    {"hockey_sobre_hierba", "1F3D1"},
    {"hockey_sobre_hielo", "1F3D2"},
    {"tenis_de_mesa", "1F3D3"},
    {"montana_con_nieve", "1F3D4"},
    {"camping", "1F3D5"},
    {"playa_y_sombrilla", "1F3D6"},
    {"construccion", "1F3D7"},
    {"casas", "1F3D8"},
    {"paisaje_urbano", "1F3D9"},
    {"casa_abandonada", "1F3DA"},
    {"edificio_clasico", "1F3DB"},
    {"desierto", "1F3DC"},
    {"isla_desierta", "1F3DD"},
    {"parque_nacional", "1F3DE"},
    {"estadio", "1F3DF"},
    {"casa", "1F3E0"},
    {"casa_con_jardin", "1F3E1"},
    {"edificio_de_oficinas", "1F3E2"},
    {"oficina_de_correos_japonesa", "1F3E3"},
    {"oficina_de_correos_europea", "1F3E4"},
    {"hospital", "1F3E5"},
    {"banco", "1F3E6"},
    {"senal_de_cajero_automatico", "1F3E7"},
    {"hotel", "1F3E8"},
    {"hotel_del_amor", "1F3E9"},
    {"tienda_24_horas", "1F3EA"},
    {"colegio", "1F3EB"},
    {"grandes_almacenes", "1F3EC"},
    {"fabrica", "1F3ED"},
    {"lampara_japonesa", "1F3EE"},
    {"castillo_japones", "1F3EF"},
    {"castillo_europeo", "1F3F0"},
    {"bandera_blanca", "1F3F3"},
    {"bandera_negra", "1F3F4"},
    {"roseta", "1F3F5"},
    {"etiqueta", "1F3F7"},
    {"badminton", "1F3F8"},
    {"arco_y_flecha", "1F3F9"},
    {"anfora", "1F3FA"},
    {"tono_de_piel_claro", "1F3FB"},
    {"tono_de_piel_claro_medio", "1F3FC"},
    {"tono_de_piel_medio", "1F3FD"},
    {"tono_de_piel_oscuro_medio", "1F3FE"},
    {"tono_de_piel_oscuro", "1F3FF"},
    {"rata", "1F400"},
    {"raton", "1F401"},
    {"buey", "1F402"},
    {"bufalo_de_agua", "1F403"},
    {"vaca", "1F404"},
    {"tigre", "1F405"},
    {"leopardo", "1F406"},
    {"conejo", "1F407"},
    {"gato", "1F408"},
    {"dragon", "1F409"},
    {"cocodrilo", "1F40A"},
    {"ballena", "1F40B"},
    {"caracol", "1F40C"},
    {"serpiente", "1F40D"},
    {"caballo", "1F40E"},
    {"carnero", "1F40F"},
    {"cabra", "1F410"},
    {"oveja", "1F411"},
    {"mono", "1F412"},
    {"gallo", "1F413"},
    {"gallina", "1F414"},
    {"perro", "1F415"},
    {"cerdo", "1F416"},
    {"jabali", "1F417"},
    {"elefante", "1F418"},
    {"pulpo", "1F419"},
    {"caracola", "1F41A"},
    {"bicho", "1F41B"},
    {"hormiga", "1F41C"},
    {"abeja", "1F41D"},
    {"mariquita", "1F41E"},
    {"pez", "1F41F"},
    {"pez_tropical", "1F420"},
    {"pez_globo", "1F421"},
    {"tortuga", "1F422"},
    {"pollito_rompiendo_el_cascaron", "1F423"},
    {"pollito", "1F424"},
    {"pollito_de_frente", "1F425"},
    {"pajaro", "1F426"},
    {"pinguino", "1F427"},
    {"koala", "1F428"},
    {"caniche", "1F429"},
    {"dromedario", "1F42A"},
    {"camello", "1F42B"},
    {"delfin", "1F42C"},
    {"cara_de_raton", "1F42D"},
    {"cara_de_vaca", "1F42E"},
    {"cara_de_tigre", "1F42F"},
    {"cara_de_conejo", "1F430"},
    {"cara_de_gato", "1F431"},
    {"cara_de_dragon", "1F432"},
    {"ballena_soltando_un_chorro", "1F433"},
    {"cara_de_caballo", "1F434"},
    {"cara_de_mono", "1F435"},
    {"cara_de_perro", "1F436"},
    {"cara_de_cerdo", "1F437"},
    {"rana", "1F438"},
    {"hamster", "1F439"},
    {"lobo", "1F43A"},
    {"oso", "1F43B"},
    {"panda", "1F43C"},
    {"nariz_de_cerdo", "1F43D"},
    {"huellas_de_pezunas", "1F43E"},
    {"ardilla", "1F43F"},
    {"ojos", "1F440"},
    {"ojo", "1F441"},
    {"oreja", "1F442"},
    {"oreja_color1", "1F442-1F3FB"},
    {"oreja_color2", "1F442-1F3FC"},
    {"oreja_color3", "1F442-1F3FD"},
    {"oreja_color4", "1F442-1F3FE"},
    {"oreja_color5", "1F442-1F3FF"},
    {"nariz", "1F443"},
    {"nariz_color1", "1F443-1F3FB"},
    {"nariz_color2", "1F443-1F3FC"},
    {"nariz_color3", "1F443-1F3FD"},
    {"nariz_color4", "1F443-1F3FE"},
    {"nariz_color5", "1F443-1F3FF"},
    {"boca", "1F444"},
    {"lengua", "1F445"},
    {"dorso_de_mano_con_indice_hacia_arriba", "1F446"},
    {"dorso_de_mano_con_indice_hacia_arriba_color1", "1F446-1F3FB"},
    {"dorso_de_mano_con_indice_hacia_arriba_color2", "1F446-1F3FC"},
    {"dorso_de_mano_con_indice_hacia_arriba_color3", "1F446-1F3FD"},
    {"dorso_de_mano_con_indice_hacia_arriba_color4", "1F446-1F3FE"},
    {"dorso_de_mano_con_indice_hacia_arriba_color5", "1F446-1F3FF"},
    {"dorso_de_mano_con_indice_hacia_abajo", "1F447"},
    {"dorso_de_mano_con_indice_hacia_abajo_color1", "1F447-1F3FB"},
    {"dorso_de_mano_con_indice_hacia_abajo_color2", "1F447-1F3FC"},
    {"dorso_de_mano_con_indice_hacia_abajo_color3", "1F447-1F3FD"},
    {"dorso_de_mano_con_indice_hacia_abajo_color4", "1F447-1F3FE"},
    {"dorso_de_mano_con_indice_hacia_abajo_color5", "1F447-1F3FF"},
    {"dorso_de_mano_con_indice_a_la_izquierda", "1F448"},
    {"dorso_de_mano_con_indice_a_la_izquierda_color1", "1F448-1F3FB"},
    {"dorso_de_mano_con_indice_a_la_izquierda_color2", "1F448-1F3FC"},
    {"dorso_de_mano_con_indice_a_la_izquierda_color3", "1F448-1F3FD"},
    {"dorso_de_mano_con_indice_a_la_izquierda_color4", "1F448-1F3FE"},
    {"dorso_de_mano_con_indice_a_la_izquierda_color5", "1F448-1F3FF"},
    {"dorso_de_mano_con_indice_a_la_derecha", "1F449"},
    {"dorso_de_mano_con_indice_a_la_derecha_color1", "1F449-1F3FB"},
    {"dorso_de_mano_con_indice_a_la_derecha_color2", "1F449-1F3FC"},
    {"dorso_de_mano_con_indice_a_la_derecha_color3", "1F449-1F3FD"},
    {"dorso_de_mano_con_indice_a_la_derecha_color4", "1F449-1F3FE"},
    {"dorso_de_mano_con_indice_a_la_derecha_color5", "1F449-1F3FF"},
    {"puno_cerrado", "1F44A"},
    {"puno_cerrado_color1", "1F44A-1F3FB"},
    {"puno_cerrado_color2", "1F44A-1F3FC"},
    {"puno_cerrado_color3", "1F44A-1F3FD"},
    {"puno_cerrado_color4", "1F44A-1F3FE"},
    {"puno_cerrado_color5", "1F44A-1F3FF"},
    {"mano_saludando", "1F44B"},
    {"mano_saludando_color1", "1F44B-1F3FB"},
    {"mano_saludando_color2", "1F44B-1F3FC"},
    {"mano_saludando_color3", "1F44B-1F3FD"},
    {"mano_saludando_color4", "1F44B-1F3FE"},
    {"mano_saludando_color5", "1F44B-1F3FF"},
    {"senal_de_aprobacion_con_la_mano", "1F44C"},
    {"senal_de_aprobacion_con_la_mano_color1", "1F44C-1F3FB"},
    {"senal_de_aprobacion_con_la_mano_color2", "1F44C-1F3FC"},
    {"senal_de_aprobacion_con_la_mano_color3", "1F44C-1F3FD"},
    {"senal_de_aprobacion_con_la_mano_color4", "1F44C-1F3FE"},
    {"senal_de_aprobacion_con_la_mano_color5", "1F44C-1F3FF"},
    {"pulgar_hacia_arriba", "1F44D"},
    {"pulgar_hacia_arriba_color1", "1F44D-1F3FB"},
    {"pulgar_hacia_arriba_color2", "1F44D-1F3FC"},
    {"pulgar_hacia_arriba_color3", "1F44D-1F3FD"},
    {"pulgar_hacia_arriba_color4", "1F44D-1F3FE"},
    {"pulgar_hacia_arriba_color5", "1F44D-1F3FF"},
    {"pulgar_hacia_abajo", "1F44E"},
    {"pulgar_hacia_abajo_color1", "1F44E-1F3FB"},
    {"pulgar_hacia_abajo_color2", "1F44E-1F3FC"},
    {"pulgar_hacia_abajo_color3", "1F44E-1F3FD"},
    {"pulgar_hacia_abajo_color4", "1F44E-1F3FE"},
    {"pulgar_hacia_abajo_color5", "1F44E-1F3FF"},
    {"manos_aplaudiendo", "1F44F"},
    {"manos_aplaudiendo_color1", "1F44F-1F3FB"},
    {"manos_aplaudiendo_color2", "1F44F-1F3FC"},
    {"manos_aplaudiendo_color3", "1F44F-1F3FD"},
    {"manos_aplaudiendo_color4", "1F44F-1F3FE"},
    {"manos_aplaudiendo_color5", "1F44F-1F3FF"},
    {"manos_abiertas", "1F450"},
    {"manos_abiertas_color1", "1F450-1F3FB"},
    {"manos_abiertas_color2", "1F450-1F3FC"},
    {"manos_abiertas_color3", "1F450-1F3FD"},
    {"manos_abiertas_color4", "1F450-1F3FE"},
    {"manos_abiertas_color5", "1F450-1F3FF"},
    {"corona", "1F451"},
    {"sombrero_de_mujer", "1F452"},
    {"gafas", "1F453"},
    {"corbata", "1F454"},
    {"camiseta", "1F455"},
    {"vaqueros", "1F456"},
    {"vestido", "1F457"},
    {"kimono", "1F458"},
    {"bikini", "1F459"},
    {"ropa_de_mujer", "1F45A"},
    {"monedero", "1F45B"},
    {"bolso", "1F45C"},
    {"bolso_de_mano", "1F45D"},
    {"zapato_de_hombre", "1F45E"},
    {"zapatilla_deportiva", "1F45F"},
    {"zapato_de_tacon", "1F460"},
    {"sandalia_de_mujer", "1F461"},
    {"bota_de_mujer", "1F462"},
    {"huellas_de_pies", "1F463"},
    {"silueta_de_busto", "1F464"},
    {"dos_siluetas_de_bustos", "1F465"},
    {"nino", "1F466"},
    {"nino_color1", "1F466-1F3FB"},
    {"nino_color2", "1F466-1F3FC"},
    {"nino_color3", "1F466-1F3FD"},
    {"nino_color4", "1F466-1F3FE"},
    {"nino_color5", "1F466-1F3FF"},
    {"nina", "1F467"},
    {"nina_color1", "1F467-1F3FB"},
    {"nina_color2", "1F467-1F3FC"},
    {"nina_color3", "1F467-1F3FD"},
    {"nina_color4", "1F467-1F3FE"},
    {"nina_color5", "1F467-1F3FF"},
    {"hombre", "1F468"},
    {"hombre_color1", "1F468-1F3FB"},
    {"hombre_color2", "1F468-1F3FC"},
    {"hombre_color3", "1F468-1F3FD"},
    {"hombre_color4", "1F468-1F3FE"},
    {"hombre_color5", "1F468-1F3FF"},
    {"mujer", "1F469"},
    {"mujer_color1", "1F469-1F3FB"},
    {"mujer_color2", "1F469-1F3FC"},
    {"mujer_color3", "1F469-1F3FD"},
    {"mujer_color4", "1F469-1F3FE"},
    {"mujer_color5", "1F469-1F3FF"},
    {"familia", "1F46A"},
    {"mujer_y_hombre_de_la_mano", "1F46B"},
    {"mujer_y_hombre_de_la_mano_color1", "1F46B-1F3FB"},
    {"mujer_y_hombre_de_la_mano_color2", "1F46B-1F3FC"},
    {"mujer_y_hombre_de_la_mano_color3", "1F46B-1F3FD"},
    {"mujer_y_hombre_de_la_mano_color4", "1F46B-1F3FE"},
    {"mujer_y_hombre_de_la_mano_color5", "1F46B-1F3FF"},
    {"mujer_y_hombre_de_la_mano_color1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mujer_y_hombre_de_la_mano_color1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mujer_y_hombre_de_la_mano_color1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mujer_y_hombre_de_la_mano_color1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mujer_y_hombre_de_la_mano_color2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mujer_y_hombre_de_la_mano_color2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mujer_y_hombre_de_la_mano_color2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mujer_y_hombre_de_la_mano_color2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mujer_y_hombre_de_la_mano_color3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mujer_y_hombre_de_la_mano_color3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mujer_y_hombre_de_la_mano_color3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mujer_y_hombre_de_la_mano_color3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mujer_y_hombre_de_la_mano_color4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mujer_y_hombre_de_la_mano_color4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mujer_y_hombre_de_la_mano_color4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mujer_y_hombre_de_la_mano_color4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mujer_y_hombre_de_la_mano_color5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mujer_y_hombre_de_la_mano_color5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mujer_y_hombre_de_la_mano_color5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mujer_y_hombre_de_la_mano_color5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"hombres_de_la_mano", "1F46C"},
    {"hombres_de_la_mano_color1", "1F46C-1F3FB"},
    {"hombres_de_la_mano_color2", "1F46C-1F3FC"},
    {"hombres_de_la_mano_color3", "1F46C-1F3FD"},
    {"hombres_de_la_mano_color4", "1F46C-1F3FE"},
    {"hombres_de_la_mano_color5", "1F46C-1F3FF"},
    {"hombres_de_la_mano_color1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"hombres_de_la_mano_color1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"hombres_de_la_mano_color1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"hombres_de_la_mano_color1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"hombres_de_la_mano_color2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"hombres_de_la_mano_color2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"hombres_de_la_mano_color2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"hombres_de_la_mano_color2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"hombres_de_la_mano_color3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"hombres_de_la_mano_color3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"hombres_de_la_mano_color3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"hombres_de_la_mano_color3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"hombres_de_la_mano_color4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"hombres_de_la_mano_color4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"hombres_de_la_mano_color4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"hombres_de_la_mano_color4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"hombres_de_la_mano_color5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"hombres_de_la_mano_color5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"hombres_de_la_mano_color5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"hombres_de_la_mano_color5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"mujeres_de_la_mano", "1F46D"},
    {"mujeres_de_la_mano_color1", "1F46D-1F3FB"},
    {"mujeres_de_la_mano_color2", "1F46D-1F3FC"},
    {"mujeres_de_la_mano_color3", "1F46D-1F3FD"},
    {"mujeres_de_la_mano_color4", "1F46D-1F3FE"},
    {"mujeres_de_la_mano_color5", "1F46D-1F3FF"},
    {"mujeres_de_la_mano_color1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"mujeres_de_la_mano_color1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"mujeres_de_la_mano_color1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"mujeres_de_la_mano_color1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"mujeres_de_la_mano_color2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"mujeres_de_la_mano_color2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"mujeres_de_la_mano_color2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"mujeres_de_la_mano_color2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"mujeres_de_la_mano_color3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"mujeres_de_la_mano_color3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"mujeres_de_la_mano_color3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"mujeres_de_la_mano_color3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"mujeres_de_la_mano_color4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"mujeres_de_la_mano_color4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"mujeres_de_la_mano_color4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"mujeres_de_la_mano_color4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"mujeres_de_la_mano_color5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"mujeres_de_la_mano_color5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"mujeres_de_la_mano_color5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"mujeres_de_la_mano_color5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"agente_de_policia", "1F46E"},
    {"agente_de_policia_color1", "1F46E-1F3FB"},
    {"agente_de_policia_color2", "1F46E-1F3FC"},
    {"agente_de_policia_color3", "1F46E-1F3FD"},
    {"agente_de_policia_color4", "1F46E-1F3FE"},
    {"agente_de_policia_color5", "1F46E-1F3FF"},
    {"personas_con_orejas_de_conejo", "1F46F"},
    {"personas_con_orejas_de_conejo_color1", "1F46F-1F3FB"},
    {"personas_con_orejas_de_conejo_color2", "1F46F-1F3FC"},
    {"personas_con_orejas_de_conejo_color3", "1F46F-1F3FD"},
    {"personas_con_orejas_de_conejo_color4", "1F46F-1F3FE"},
    {"personas_con_orejas_de_conejo_color5", "1F46F-1F3FF"},
    {"personas_con_orejas_de_conejo_color1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"personas_con_orejas_de_conejo_color1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"personas_con_orejas_de_conejo_color1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"personas_con_orejas_de_conejo_color1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"personas_con_orejas_de_conejo_color2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"personas_con_orejas_de_conejo_color2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"personas_con_orejas_de_conejo_color2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"personas_con_orejas_de_conejo_color2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"personas_con_orejas_de_conejo_color3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"personas_con_orejas_de_conejo_color3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"personas_con_orejas_de_conejo_color3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"personas_con_orejas_de_conejo_color3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"personas_con_orejas_de_conejo_color4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"personas_con_orejas_de_conejo_color4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"personas_con_orejas_de_conejo_color4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"personas_con_orejas_de_conejo_color4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"personas_con_orejas_de_conejo_color5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"personas_con_orejas_de_conejo_color5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"personas_con_orejas_de_conejo_color5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"personas_con_orejas_de_conejo_color5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"persona_con_velo", "1F470"},
    {"persona_con_velo_color1", "1F470-1F3FB"},
    {"persona_con_velo_color2", "1F470-1F3FC"},
    {"persona_con_velo_color3", "1F470-1F3FD"},
    {"persona_con_velo_color4", "1F470-1F3FE"},
    {"persona_con_velo_color5", "1F470-1F3FF"},
    {"persona_adulta_rubia", "1F471"},
    {"persona_adulta_rubia_color1", "1F471-1F3FB"},
    {"persona_adulta_rubia_color2", "1F471-1F3FC"},
    {"persona_adulta_rubia_color3", "1F471-1F3FD"},
    {"persona_adulta_rubia_color4", "1F471-1F3FE"},
    {"persona_adulta_rubia_color5", "1F471-1F3FF"},
    {"persona_con_gorro_chino", "1F472"},
    {"persona_con_gorro_chino_color1", "1F472-1F3FB"},
    {"persona_con_gorro_chino_color2", "1F472-1F3FC"},
    {"persona_con_gorro_chino_color3", "1F472-1F3FD"},
    {"persona_con_gorro_chino_color4", "1F472-1F3FE"},
    {"persona_con_gorro_chino_color5", "1F472-1F3FF"},
    {"persona_con_turbante", "1F473"},
    {"persona_con_turbante_color1", "1F473-1F3FB"},
    {"persona_con_turbante_color2", "1F473-1F3FC"},
    {"persona_con_turbante_color3", "1F473-1F3FD"},
    {"persona_con_turbante_color4", "1F473-1F3FE"},
    {"persona_con_turbante_color5", "1F473-1F3FF"},
    {"anciano", "1F474"},
    {"anciano_color1", "1F474-1F3FB"},
    {"anciano_color2", "1F474-1F3FC"},
    {"anciano_color3", "1F474-1F3FD"},
    {"anciano_color4", "1F474-1F3FE"},
    {"anciano_color5", "1F474-1F3FF"},
    {"anciana", "1F475"},
    {"anciana_color1", "1F475-1F3FB"},
    {"anciana_color2", "1F475-1F3FC"},
    {"anciana_color3", "1F475-1F3FD"},
    {"anciana_color4", "1F475-1F3FE"},
    {"anciana_color5", "1F475-1F3FF"},
    {"bebe", "1F476"},
    {"bebe_color1", "1F476-1F3FB"},
    {"bebe_color2", "1F476-1F3FC"},
    {"bebe_color3", "1F476-1F3FD"},
    {"bebe_color4", "1F476-1F3FE"},
    {"bebe_color5", "1F476-1F3FF"},
    {"profesional_de_la_construccion", "1F477"},
    {"profesional_de_la_construccion_color1", "1F477-1F3FB"},
    {"profesional_de_la_construccion_color2", "1F477-1F3FC"},
    {"profesional_de_la_construccion_color3", "1F477-1F3FD"},
    {"profesional_de_la_construccion_color4", "1F477-1F3FE"},
    {"profesional_de_la_construccion_color5", "1F477-1F3FF"},
    {"princesa", "1F478"},
    {"princesa_color1", "1F478-1F3FB"},
    {"princesa_color2", "1F478-1F3FC"},
    {"princesa_color3", "1F478-1F3FD"},
    {"princesa_color4", "1F478-1F3FE"},
    {"princesa_color5", "1F478-1F3FF"},
    {"demonio_japones_oni", "1F479"},
    {"demonio_japones_tengu", "1F47A"},
    {"fantasma", "1F47B"},
    {"bebe_angel", "1F47C"},
    {"bebe_angel_color1", "1F47C-1F3FB"},
    {"bebe_angel_color2", "1F47C-1F3FC"},
    {"bebe_angel_color3", "1F47C-1F3FD"},
    {"bebe_angel_color4", "1F47C-1F3FE"},
    {"bebe_angel_color5", "1F47C-1F3FF"},
    {"alienigena", "1F47D"},
    {"monstruo_alienigena", "1F47E"},
    {"cara_enfadada_con_cuernos", "1F47F"},
    {"calavera", "1F480"},
    {"persona_de_mostrador_de_informacion", "1F481"},
    {"persona_de_mostrador_de_informacion_color1", "1F481-1F3FB"},
    {"persona_de_mostrador_de_informacion_color2", "1F481-1F3FC"},
    {"persona_de_mostrador_de_informacion_color3", "1F481-1F3FD"},
    {"persona_de_mostrador_de_informacion_color4", "1F481-1F3FE"},
    {"persona_de_mostrador_de_informacion_color5", "1F481-1F3FF"},
    {"guardia", "1F482"},
    {"guardia_color1", "1F482-1F3FB"},
    {"guardia_color2", "1F482-1F3FC"},
    {"guardia_color3", "1F482-1F3FD"},
    {"guardia_color4", "1F482-1F3FE"},
    {"guardia_color5", "1F482-1F3FF"},
    {"mujer_bailando", "1F483"},
    {"mujer_bailando_color1", "1F483-1F3FB"},
    {"mujer_bailando_color2", "1F483-1F3FC"},
    {"mujer_bailando_color3", "1F483-1F3FD"},
    {"mujer_bailando_color4", "1F483-1F3FE"},
    {"mujer_bailando_color5", "1F483-1F3FF"},
    {"pintalabios", "1F484"},
    {"pintarse_las_unas", "1F485"},
    {"pintarse_las_unas_color1", "1F485-1F3FB"},
    {"pintarse_las_unas_color2", "1F485-1F3FC"},
    {"pintarse_las_unas_color3", "1F485-1F3FD"},
    {"pintarse_las_unas_color4", "1F485-1F3FE"},
    {"pintarse_las_unas_color5", "1F485-1F3FF"},
    {"persona_recibiendo_masaje", "1F486"},
    {"persona_recibiendo_masaje_color1", "1F486-1F3FB"},
    {"persona_recibiendo_masaje_color2", "1F486-1F3FC"},
    {"persona_recibiendo_masaje_color3", "1F486-1F3FD"},
    {"persona_recibiendo_masaje_color4", "1F486-1F3FE"},
    {"persona_recibiendo_masaje_color5", "1F486-1F3FF"},
    {"persona_cortandose_el_pelo", "1F487"},
    {"persona_cortandose_el_pelo_color1", "1F487-1F3FB"},
    {"persona_cortandose_el_pelo_color2", "1F487-1F3FC"},
    {"persona_cortandose_el_pelo_color3", "1F487-1F3FD"},
    {"persona_cortandose_el_pelo_color4", "1F487-1F3FE"},
    {"persona_cortandose_el_pelo_color5", "1F487-1F3FF"},
    {"poste_de_barbero", "1F488"},
    {"jeringuilla", "1F489"},
    {"pildora", "1F48A"},
    {"marca_de_beso", "1F48B"},
    {"carta_de_amor", "1F48C"},
    {"anillo", "1F48D"},
    {"piedra_preciosa", "1F48E"},
    {"beso", "1F48F"},
    {"beso_color1", "1F48F-1F3FB"},
    {"beso_color2", "1F48F-1F3FC"},
    {"beso_color3", "1F48F-1F3FD"},
    {"beso_color4", "1F48F-1F3FE"},
    {"beso_color5", "1F48F-1F3FF"},
    {"beso_color1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"beso_color1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"beso_color1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"beso_color1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"beso_color2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"beso_color2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"beso_color2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"beso_color2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"beso_color3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"beso_color3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"beso_color3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"beso_color3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"beso_color4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"beso_color4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"beso_color4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"beso_color4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"beso_color5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"beso_color5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"beso_color5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"beso_color5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"ramo_de_flores", "1F490"},
    {"pareja_enamorada", "1F491"},
    {"pareja_enamorada_color1", "1F491-1F3FB"},
    {"pareja_enamorada_color2", "1F491-1F3FC"},
    {"pareja_enamorada_color3", "1F491-1F3FD"},
    {"pareja_enamorada_color4", "1F491-1F3FE"},
    {"pareja_enamorada_color5", "1F491-1F3FF"},
    {"pareja_enamorada_color1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pareja_enamorada_color1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pareja_enamorada_color1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pareja_enamorada_color1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pareja_enamorada_color2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pareja_enamorada_color2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pareja_enamorada_color2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pareja_enamorada_color2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pareja_enamorada_color3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pareja_enamorada_color3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pareja_enamorada_color3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pareja_enamorada_color3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pareja_enamorada_color4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pareja_enamorada_color4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pareja_enamorada_color4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pareja_enamorada_color4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pareja_enamorada_color5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pareja_enamorada_color5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pareja_enamorada_color5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pareja_enamorada_color5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"iglesia_celebrando_boda", "1F492"},
    {"corazon_latiendo", "1F493"},
    {"corazon_roto", "1F494"},
    {"dos_corazones", "1F495"},
    {"corazon_brillante", "1F496"},
    {"corazon_creciente", "1F497"},
    {"corazon_con_flecha", "1F498"},
    {"corazon_azul", "1F499"},
    {"corazon_verde", "1F49A"},
    {"corazon_amarillo", "1F49B"},
    {"corazon_morado", "1F49C"},
    {"corazon_con_lazo", "1F49D"},
    {"corazones_giratorios", "1F49E"},
    {"adorno_de_corazon", "1F49F"},
    {"rombo_con_petalo", "1F4A0"},
    {"bombilla", "1F4A1"},
    {"simbolo_de_enfado", "1F4A2"},
    {"bomba", "1F4A3"},
    {"simbolo_de_sueno", "1F4A4"},
    {"colision", "1F4A5"},
    {"gotas_de_sudor", "1F4A6"},
    {"gota", "1F4A7"},
    {"salir_corriendo", "1F4A8"},
    {"caca_con_ojos", "1F4A9"},
    {"biceps_flexionado", "1F4AA"},
    {"biceps_flexionado_color1", "1F4AA-1F3FB"},
    {"biceps_flexionado_color2", "1F4AA-1F3FC"},
    {"biceps_flexionado_color3", "1F4AA-1F3FD"},
    {"biceps_flexionado_color4", "1F4AA-1F3FE"},
    {"biceps_flexionado_color5", "1F4AA-1F3FF"},
    {"simbolo_de_mareo", "1F4AB"},
    {"bocadillo_de_dialogo", "1F4AC"},
    {"bocadillo_de_pensamiento", "1F4AD"},
    {"flor_blanca", "1F4AE"},
    {"cien_puntos", "1F4AF"},
    {"bolsa_de_dinero", "1F4B0"},
    {"cambio_de_divisas", "1F4B1"},
    {"simbolo_de_dolar", "1F4B2"},
    {"tarjeta_de_credito", "1F4B3"},
    {"billete_de_yen", "1F4B4"},
    {"billete_de_dolar", "1F4B5"},
    {"billete_de_euro", "1F4B6"},
    {"billete_de_libra", "1F4B7"},
    {"billete_con_alas", "1F4B8"},
    {"mercado_al_alza", "1F4B9"},
    {"asiento_de_transporte", "1F4BA"},
    {"ordenador_portatil", "1F4BB"},
    {"maletin", "1F4BC"},
    {"minidisc", "1F4BD"},
    {"disquete", "1F4BE"},
    {"disco_optico", "1F4BF"},
    {"disco_dvd", "1F4C0"},
    {"carpeta_de_archivos", "1F4C1"},
    {"carpeta_de_archivos_abierta", "1F4C2"},
    {"pagina_doblada", "1F4C3"},
    {"pagina_hacia_arriba", "1F4C4"},
    {"calendario", "1F4C5"},
    {"calendario_recortable", "1F4C6"},
    {"organizador_de_fichas", "1F4C7"},
    {"grafica_de_evolucion_ascendente", "1F4C8"},
    {"grafica_de_evolucion_descendente", "1F4C9"},
    {"grafico_de_barras", "1F4CA"},
    {"portapapeles", "1F4CB"},
    {"chincheta", "1F4CC"},
    {"chincheta_redonda", "1F4CD"},
    {"clip", "1F4CE"},
    {"regla", "1F4CF"},
    {"escuadra", "1F4D0"},
    {"marcadores", "1F4D1"},
    {"libro_de_contabilidad", "1F4D2"},
    {"cuaderno", "1F4D3"},
    {"cuaderno_con_tapa_decorativa", "1F4D4"},
    {"libro_cerrado", "1F4D5"},
    {"libro_abierto", "1F4D6"},
    {"libro_verde", "1F4D7"},
    {"libro_azul", "1F4D8"},
    {"libro_naranja", "1F4D9"},
    {"libros", "1F4DA"},
    {"etiqueta_identificativa", "1F4DB"},
    {"pergamino", "1F4DC"},
    {"cuaderno_de_notas", "1F4DD"},
    {"auricular_de_telefono", "1F4DE"},
    {"busca", "1F4DF"},
    {"maquina_de_fax", "1F4E0"},
    {"antena_de_satelite", "1F4E1"},
    {"altavoz_de_mano", "1F4E2"},
    {"megafono", "1F4E3"},
    {"bandeja_de_salida", "1F4E4"},
    {"bandeja_de_entrada", "1F4E5"},
    {"paquete", "1F4E6"},
    {"correo_electronico", "1F4E7"},
    {"sobre_entrante", "1F4E8"},
    {"sobre_con_flecha", "1F4E9"},
    {"buzon_cerrado_con_la_bandera_bajada", "1F4EA"},
    {"buzon_cerrado_con_la_bandera_levantada", "1F4EB"},
    {"buzon_abierto_con_la_bandera_levantada", "1F4EC"},
    {"buzon_abierto_con_la_bandera_bajada", "1F4ED"},
    {"buzon", "1F4EE"},
    {"corneta_de_posta", "1F4EF"},
    {"periodico", "1F4F0"},
    {"telefono_movil", "1F4F1"},
    {"movil_con_una_flecha", "1F4F2"},
    {"modo_vibracion", "1F4F3"},
    {"telefono_movil_apagado", "1F4F4"},
    {"prohibido_el_uso_de_moviles", "1F4F5"},
    {"barras_de_cobertura", "1F4F6"},
    {"camara_de_fotos", "1F4F7"},
    {"camara_con_flash", "1F4F8"},
    {"videocamara", "1F4F9"},
    {"television", "1F4FA"},
    {"radio", "1F4FB"},
    {"cinta_de_video", "1F4FC"},
    {"proyector_de_cine", "1F4FD"},
    {"rosario", "1F4FF"},
    {"reproduccion_aleatoria", "1F500"},
    {"repetir", "1F501"},
    {"repetir_una_vez", "1F502"},
    {"flechas_verticales_en_sentido_horario", "1F503"},
    {"flechas_en_sentido_antihorario", "1F504"},
    {"brillo_bajo", "1F505"},
    {"brillo_alto", "1F506"},
    {"altavoz_silenciado", "1F507"},
    {"altavoz_a_volumen_bajo", "1F508"},
    {"altavoz_a_volumen_medio", "1F509"},
    {"altavoz_a_volumen_alto", "1F50A"},
    {"pila", "1F50B"},
    {"enchufe_electrico", "1F50C"},
    {"lupa_orientada_hacia_la_izquierda", "1F50D"},
    {"lupa_orientada_hacia_la_derecha", "1F50E"},
    {"candado_con_pluma_estilografica", "1F50F"},
    {"candado_cerrado_y_llave", "1F510"},
    {"llave", "1F511"},
    {"candado_cerrado", "1F512"},
    {"candado_abierto", "1F513"},
    {"campana", "1F514"},
    {"campana_con_signo_de_cancelacion", "1F515"},
    {"marcapaginas", "1F516"},
    {"eslabon", "1F517"},
    {"boton_de_opcion", "1F518"},
    {"flecha_back", "1F519"},
    {"flecha_end", "1F51A"},
    {"flecha_on", "1F51B"},
    {"flecha_soon", "1F51C"},
    {"flecha_top", "1F51D"},
    {"prohibido_para_menos_de_18_anos", "1F51E"},
    {"teclas_10", "1F51F"},
    {"letras_latinas_mayusculas", "1F520"},
    {"letras_latinas_minusculas", "1F521"},
    {"numeros", "1F522"},
    {"simbolos", "1F523"},
    {"alfabeto_latino", "1F524"},
    {"fuego", "1F525"},
    {"linterna", "1F526"},
    {"llave_inglesa", "1F527"},
    {"martillo", "1F528"},
    {"tornillo_y_tuerca", "1F529"},
    {"cuchillo_de_cocina", "1F52A"},
    {"pistola_de_agua", "1F52B"},
    {"microscopio", "1F52C"},
    {"telescopio", "1F52D"},
    {"bola_de_cristal", "1F52E"},
    {"estrella_de_seis_puntas", "1F52F"},
    {"simbolo_japones_para_principiante", "1F530"},
    {"emblema_de_tridente", "1F531"},
    {"boton_cuadrado_con_borde_negro", "1F532"},
    {"boton_cuadrado_con_borde_blanco", "1F533"},
    {"circulo_rojo", "1F534"},
    {"circulo_azul", "1F535"},
    {"rombo_naranja_grande", "1F536"},
    {"rombo_azul_grande", "1F537"},
    {"rombo_naranja_pequeno", "1F538"},
    {"rombo_azul_pequeno", "1F539"},
    {"triangulo_rojo_hacia_arriba", "1F53A"},
    {"triangulo_rojo_hacia_abajo", "1F53B"},
    {"triangulo_hacia_arriba", "1F53C"},
    {"triangulo_hacia_abajo", "1F53D"},
    {"om", "1F549"},
    {"paloma", "1F54A"},
    {"kaaba", "1F54B"},
    {"mezquita", "1F54C"},
    {"sinagoga", "1F54D"},
    {"menora", "1F54E"},
    {"1_en_punto", "1F550"},
    {"2_en_punto", "1F551"},
    {"3_en_punto", "1F552"},
    {"4_en_punto", "1F553"},
    {"5_en_punto", "1F554"},
    {"6_en_punto", "1F555"},
    {"7_en_punto", "1F556"},
    {"8_en_punto", "1F557"},
    {"9_en_punto", "1F558"},
    {"10_en_punto", "1F559"},
    {"11_en_punto", "1F55A"},
    {"12_en_punto", "1F55B"},
    {"una_y_media", "1F55C"},
    {"dos_y_media", "1F55D"},
    {"tres_y_media", "1F55E"},
    {"cuatro_y_media", "1F55F"},
    {"cinco_y_media", "1F560"},
    {"seis_y_media", "1F561"},
    {"siete_y_media", "1F562"},
    {"ocho_y_media", "1F563"},
    {"nueve_y_media", "1F564"},
    {"diez_y_media", "1F565"},
    {"once_y_media", "1F566"},
    {"doce_y_media", "1F567"},
    {"vela", "1F56F"},
    {"reloj_de_sobremesa", "1F570"},
    {"agujero", "1F573"},
    {"persona_trajeada_levitando", "1F574"},
    {"persona_trajeada_levitando_color1", "1F574-1F3FB"},
    {"persona_trajeada_levitando_color2", "1F574-1F3FC"},
    {"persona_trajeada_levitando_color3", "1F574-1F3FD"},
    {"persona_trajeada_levitando_color4", "1F574-1F3FE"},
    {"persona_trajeada_levitando_color5", "1F574-1F3FF"},
    {"detective", "1F575"},
    {"detective_color1", "1F575-1F3FB"},
    {"detective_color2", "1F575-1F3FC"},
    {"detective_color3", "1F575-1F3FD"},
    {"detective_color4", "1F575-1F3FE"},
    {"detective_color5", "1F575-1F3FF"},
    {"gafas_de_sol", "1F576"},
    {"arana", "1F577"},
    {"tela_de_arana", "1F578"},
    {"joystick", "1F579"},
    {"hombre_bailando", "1F57A"},
    {"hombre_bailando_color1", "1F57A-1F3FB"},
    {"hombre_bailando_color2", "1F57A-1F3FC"},
    {"hombre_bailando_color3", "1F57A-1F3FD"},
    {"hombre_bailando_color4", "1F57A-1F3FE"},
    {"hombre_bailando_color5", "1F57A-1F3FF"},
    {"clips_unidos", "1F587"},
    {"boligrafo", "1F58A"},
    {"estilografica", "1F58B"},
    {"pincel", "1F58C"},
    {"lapiz_de_cera", "1F58D"},
    {"mano_abierta", "1F590"},
    {"mano_abierta_color1", "1F590-1F3FB"},
    {"mano_abierta_color2", "1F590-1F3FC"},
    {"mano_abierta_color3", "1F590-1F3FD"},
    {"mano_abierta_color4", "1F590-1F3FE"},
    {"mano_abierta_color5", "1F590-1F3FF"},
    {"dedo_corazon_hacia_arriba", "1F595"},
    {"dedo_corazon_hacia_arriba_color1", "1F595-1F3FB"},
    {"dedo_corazon_hacia_arriba_color2", "1F595-1F3FC"},
    {"dedo_corazon_hacia_arriba_color3", "1F595-1F3FD"},
    {"dedo_corazon_hacia_arriba_color4", "1F595-1F3FE"},
    {"dedo_corazon_hacia_arriba_color5", "1F595-1F3FF"},
    {"saludo_vulcano", "1F596"},
    {"saludo_vulcano_color1", "1F596-1F3FB"},
    {"saludo_vulcano_color2", "1F596-1F3FC"},
    {"saludo_vulcano_color3", "1F596-1F3FD"},
    {"saludo_vulcano_color4", "1F596-1F3FE"},
    {"saludo_vulcano_color5", "1F596-1F3FF"},
    {"corazon_negro", "1F5A4"},
    {"ordenador_de_sobremesa", "1F5A5"},
    {"impresora", "1F5A8"},
    {"raton_de_ordenador", "1F5B1"},
    {"bola_de_desplazamiento", "1F5B2"},
    {"cuadro_enmarcado", "1F5BC"},
    {"separador_de_fichas", "1F5C2"},
    {"archivador_de_tarjetas", "1F5C3"},
    {"archivador", "1F5C4"},
    {"papelera", "1F5D1"},
    {"bloc_de_notas_de_espiral", "1F5D2"},
    {"calendario_de_espiral", "1F5D3"},
    {"tornillo_de_banco", "1F5DC"},
    {"llave_antigua", "1F5DD"},
    {"periodico_enrollado", "1F5DE"},
    {"punal", "1F5E1"},
    {"cabeza_parlante", "1F5E3"},
    {"bocadillo_de_dialogo_por_la_izquierda", "1F5E8"},
    {"bocadillo_de_enfado_por_la_derecha", "1F5EF"},
    {"urna_con_papeleta", "1F5F3"},
    {"mapa_mundial", "1F5FA"},
    {"monte_fuji", "1F5FB"},
    {"torre_de_tokio", "1F5FC"},
    {"estatua_de_la_libertad", "1F5FD"},
    {"mapa_de_japon", "1F5FE"},
    {"estatua_moai", "1F5FF"},
    {"cara_sonriendo", "1F600"},
    {"cara_radiante_con_ojos_sonrientes", "1F601"},
    {"cara_llorando_de_risa", "1F602"},
    {"cara_sonriendo_con_ojos_grandes", "1F603"},
    {"cara_sonriendo_con_ojos_sonrientes", "1F604"},
    {"cara_sonriendo_con_sudor_frio", "1F605"},
    {"cara_sonriendo_con_los_ojos_cerrados", "1F606"},
    {"cara_sonriendo_con_aureola", "1F607"},
    {"cara_sonriendo_con_cuernos", "1F608"},
    {"cara_guinando_el_ojo", "1F609"},
    {"cara_feliz_con_ojos_sonrientes", "1F60A"},
    {"cara_saboreando_comida", "1F60B"},
    {"cara_de_alivio", "1F60C"},
    {"cara_sonriendo_con_ojos_de_corazon", "1F60D"},
    {"cara_sonriendo_con_gafas_de_sol", "1F60E"},
    {"cara_sonriendo_con_superioridad", "1F60F"},
    {"cara_neutral", "1F610"},
    {"cara_sin_expresion", "1F611"},
    {"cara_de_desaprobacion", "1F612"},
    {"cara_con_sudor_frio", "1F613"},
    {"cara_desanimada", "1F614"},
    {"cara_de_confusion", "1F615"},
    {"cara_de_frustracion", "1F616"},
    {"cara_besando", "1F617"},
    {"cara_lanzando_un_beso", "1F618"},
    {"cara_besando_con_ojos_sonrientes", "1F619"},
    {"cara_besando_con_los_ojos_cerrados", "1F61A"},
    {"cara_sacando_la_lengua", "1F61B"},
    {"cara_sacando_la_lengua_y_guinando_un_ojo", "1F61C"},
    {"cara_con_ojos_cerrados_y_lengua_fuera", "1F61D"},
    {"cara_decepcionada", "1F61E"},
    {"cara_preocupada", "1F61F"},
    {"cara_enfadada", "1F620"},
    {"cara_cabreada", "1F621"},
    {"cara_llorando", "1F622"},
    {"cara_desesperada", "1F623"},
    {"cara_resoplando", "1F624"},
    {"cara_triste_pero_aliviada", "1F625"},
    {"cara_con_el_ceno_fruncido_y_la_boca_abierta", "1F626"},
    {"cara_angustiada", "1F627"},
    {"cara_asustada", "1F628"},
    {"cara_agotada", "1F629"},
    {"cara_de_sueno", "1F62A"},
    {"cara_cansada", "1F62B"},
    {"cara_haciendo_una_mueca", "1F62C"},
    {"cara_llorando_fuerte", "1F62D"},
    {"cara_con_la_boca_abierta", "1F62E"},
    {"cara_estupefacta", "1F62F"},
    {"cara_con_ansiedad_y_sudor", "1F630"},
    {"cara_gritando_de_miedo", "1F631"},
    {"cara_asombrada", "1F632"},
    {"cara_sonrojada", "1F633"},
    {"cara_durmiendo", "1F634"},
    {"cara_mareada", "1F635"},
    {"cara_sin_boca", "1F636"},
    {"cara_con_mascarilla_medica", "1F637"},
    {"gato_sonriendo_con_ojos_sonrientes", "1F638"},
    {"gato_llorando_de_risa", "1F639"},
    {"gato_sonriendo", "1F63A"},
    {"gato_sonriendo_con_ojos_de_corazon", "1F63B"},
    {"gato_haciendo_una_mueca", "1F63C"},
    {"gato_besando", "1F63D"},
    {"gato_enfadado", "1F63E"},
    {"gato_llorando", "1F63F"},
    {"gato_asustado", "1F640"},
    {"cara_con_el_ceno_ligeramente_fruncido", "1F641"},
    {"cara_sonriendo_ligeramente", "1F642"},
    {"cara_al_reves", "1F643"},
    {"cara_con_ojos_en_blanco", "1F644"},
    {"persona_haciendo_el_gesto_de_no", "1F645"},
    {"persona_haciendo_el_gesto_de_no_color1", "1F645-1F3FB"},
    {"persona_haciendo_el_gesto_de_no_color2", "1F645-1F3FC"},
    {"persona_haciendo_el_gesto_de_no_color3", "1F645-1F3FD"},
    {"persona_haciendo_el_gesto_de_no_color4", "1F645-1F3FE"},
    {"persona_haciendo_el_gesto_de_no_color5", "1F645-1F3FF"},
    {"persona_haciendo_el_gesto_de_de_acuerdo", "1F646"},
    {"persona_haciendo_el_gesto_de_de_acuerdo_color1", "1F646-1F3FB"},
    {"persona_haciendo_el_gesto_de_de_acuerdo_color2", "1F646-1F3FC"},
    {"persona_haciendo_el_gesto_de_de_acuerdo_color3", "1F646-1F3FD"},
    {"persona_haciendo_el_gesto_de_de_acuerdo_color4", "1F646-1F3FE"},
    {"persona_haciendo_el_gesto_de_de_acuerdo_color5", "1F646-1F3FF"},
    {"persona_haciendo_una_reverencia", "1F647"},
    {"persona_haciendo_una_reverencia_color1", "1F647-1F3FB"},
    {"persona_haciendo_una_reverencia_color2", "1F647-1F3FC"},
    {"persona_haciendo_una_reverencia_color3", "1F647-1F3FD"},
    {"persona_haciendo_una_reverencia_color4", "1F647-1F3FE"},
    {"persona_haciendo_una_reverencia_color5", "1F647-1F3FF"},
    {"mono_con_los_ojos_tapados", "1F648"},
    {"mono_con_los_oidos_tapados", "1F649"},
    {"mono_con_la_boca_tapada", "1F64A"},
    {"persona_con_la_mano_levantada", "1F64B"},
    {"persona_con_la_mano_levantada_color1", "1F64B-1F3FB"},
    {"persona_con_la_mano_levantada_color2", "1F64B-1F3FC"},
    {"persona_con_la_mano_levantada_color3", "1F64B-1F3FD"},
    {"persona_con_la_mano_levantada_color4", "1F64B-1F3FE"},
    {"persona_con_la_mano_levantada_color5", "1F64B-1F3FF"},
    {"manos_levantadas_celebrando", "1F64C"},
    {"manos_levantadas_celebrando_color1", "1F64C-1F3FB"},
    {"manos_levantadas_celebrando_color2", "1F64C-1F3FC"},
    {"manos_levantadas_celebrando_color3", "1F64C-1F3FD"},
    {"manos_levantadas_celebrando_color4", "1F64C-1F3FE"},
    {"manos_levantadas_celebrando_color5", "1F64C-1F3FF"},
    {"persona_frunciendo_el_ceno", "1F64D"},
    {"persona_frunciendo_el_ceno_color1", "1F64D-1F3FB"},
    {"persona_frunciendo_el_ceno_color2", "1F64D-1F3FC"},
    {"persona_frunciendo_el_ceno_color3", "1F64D-1F3FD"},
    {"persona_frunciendo_el_ceno_color4", "1F64D-1F3FE"},
    {"persona_frunciendo_el_ceno_color5", "1F64D-1F3FF"},
    {"persona_haciendo_pucheros", "1F64E"},
    {"persona_haciendo_pucheros_color1", "1F64E-1F3FB"},
    {"persona_haciendo_pucheros_color2", "1F64E-1F3FC"},
    {"persona_haciendo_pucheros_color3", "1F64E-1F3FD"},
    {"persona_haciendo_pucheros_color4", "1F64E-1F3FE"},
    {"persona_haciendo_pucheros_color5", "1F64E-1F3FF"},
    {"manos_en_oracion", "1F64F"},
    {"manos_en_oracion_color1", "1F64F-1F3FB"},
    {"manos_en_oracion_color2", "1F64F-1F3FC"},
    {"manos_en_oracion_color3", "1F64F-1F3FD"},
    {"manos_en_oracion_color4", "1F64F-1F3FE"},
    {"manos_en_oracion_color5", "1F64F-1F3FF"},
    {"cohete", "1F680"},
    {"helicoptero", "1F681"},
    {"locomotora_de_vapor", "1F682"},
    {"vagon", "1F683"},
    {"tren_de_alta_velocidad", "1F684"},
    {"tren_bala", "1F685"},
    {"tren", "1F686"},
    {"metro", "1F687"},
    {"tren_ligero", "1F688"},
    {"estacion_de_tren", "1F689"},
    {"tranvia", "1F68A"},
    {"vagon_de_tranvia", "1F68B"},
    {"autobus", "1F68C"},
    {"autobus_proximo", "1F68D"},
    {"trolebus", "1F68E"},
    {"parada_de_autobus", "1F68F"},
    {"minibus", "1F690"},
    {"ambulancia", "1F691"},
    {"coche_de_bomberos", "1F692"},
    {"coche_de_policia", "1F693"},
    {"coche_de_policia_proximo", "1F694"},
    {"taxi", "1F695"},
    {"taxi_proximo", "1F696"},
    {"coche", "1F697"},
    {"coche_proximo", "1F698"},
    {"vehiculo_deportivo_utilitario", "1F699"},
    {"camion_de_reparto", "1F69A"},
    {"camion_articulado", "1F69B"},
    {"tractor", "1F69C"},
    {"monorrail", "1F69D"},
    {"ferrocarril_de_montana", "1F69E"},
    {"ferrocarril_de_suspension", "1F69F"},
    {"teleferico_de_montana", "1F6A0"},
    {"teleferico", "1F6A1"},
    {"barco", "1F6A2"},
    {"persona_remando_en_un_bote", "1F6A3"},
    {"persona_remando_en_un_bote_color1", "1F6A3-1F3FB"},
    {"persona_remando_en_un_bote_color2", "1F6A3-1F3FC"},
    {"persona_remando_en_un_bote_color3", "1F6A3-1F3FD"},
    {"persona_remando_en_un_bote_color4", "1F6A3-1F3FE"},
    {"persona_remando_en_un_bote_color5", "1F6A3-1F3FF"},
    {"lancha_motora", "1F6A4"},
    {"semaforo_horizontal", "1F6A5"},
    {"semaforo", "1F6A6"},
    {"obras", "1F6A7"},
    {"luces_de_policia", "1F6A8"},
    {"bandera_triangular", "1F6A9"},
    {"puerta", "1F6AA"},
    {"prohibido", "1F6AB"},
    {"cigarrillo", "1F6AC"},
    {"prohibido_fumar", "1F6AD"},
    {"senal_de_usar_papelera", "1F6AE"},
    {"prohibido_tirar_basura", "1F6AF"},
    {"agua_potable", "1F6B0"},
    {"agua_no_potable", "1F6B1"},
    {"bicicleta", "1F6B2"},
    {"bicicletas_prohibidas", "1F6B3"},
    {"persona_en_bicicleta", "1F6B4"},
    {"persona_en_bicicleta_color1", "1F6B4-1F3FB"},
    {"persona_en_bicicleta_color2", "1F6B4-1F3FC"},
    {"persona_en_bicicleta_color3", "1F6B4-1F3FD"},
    {"persona_en_bicicleta_color4", "1F6B4-1F3FE"},
    {"persona_en_bicicleta_color5", "1F6B4-1F3FF"},
    {"persona_en_bicicleta_de_montana", "1F6B5"},
    {"persona_en_bicicleta_de_montana_color1", "1F6B5-1F3FB"},
    {"persona_en_bicicleta_de_montana_color2", "1F6B5-1F3FC"},
    {"persona_en_bicicleta_de_montana_color3", "1F6B5-1F3FD"},
    {"persona_en_bicicleta_de_montana_color4", "1F6B5-1F3FE"},
    {"persona_en_bicicleta_de_montana_color5", "1F6B5-1F3FF"},
    {"persona_caminando", "1F6B6"},
    {"persona_caminando_color1", "1F6B6-1F3FB"},
    {"persona_caminando_color2", "1F6B6-1F3FC"},
    {"persona_caminando_color3", "1F6B6-1F3FD"},
    {"persona_caminando_color4", "1F6B6-1F3FE"},
    {"persona_caminando_color5", "1F6B6-1F3FF"},
    {"prohibido_el_paso_de_peatones", "1F6B7"},
    {"ninos_cruzando", "1F6B8"},
    {"aseo_para_hombres", "1F6B9"},
    {"senal_de_aseo_para_mujeres", "1F6BA"},
    {"senal_de_aseos", "1F6BB"},
    {"senal_de_bebe", "1F6BC"},
    {"inodoro", "1F6BD"},
    {"aseos", "1F6BE"},
    {"ducha", "1F6BF"},
    {"persona_en_la_banera", "1F6C0"},
    {"persona_en_la_banera_color1", "1F6C0-1F3FB"},
    {"persona_en_la_banera_color2", "1F6C0-1F3FC"},
    {"persona_en_la_banera_color3", "1F6C0-1F3FD"},
    {"persona_en_la_banera_color4", "1F6C0-1F3FE"},
    {"persona_en_la_banera_color5", "1F6C0-1F3FF"},
    {"banera", "1F6C1"},
    {"control_de_pasaportes", "1F6C2"},
    {"aduana", "1F6C3"},
    {"recogida_de_equipajes", "1F6C4"},
    {"consigna", "1F6C5"},
    {"sofa_y_lampara", "1F6CB"},
    {"persona_en_la_cama", "1F6CC"},
    {"persona_en_la_cama_color1", "1F6CC-1F3FB"},
    {"persona_en_la_cama_color2", "1F6CC-1F3FC"},
    {"persona_en_la_cama_color3", "1F6CC-1F3FD"},
    {"persona_en_la_cama_color4", "1F6CC-1F3FE"},
    {"persona_en_la_cama_color5", "1F6CC-1F3FF"},
    {"bolsas_de_compras", "1F6CD"},
    {"timbre_de_hotel", "1F6CE"},
    {"cama", "1F6CF"},
    {"lugar_de_culto", "1F6D0"},
    {"senal_de_stop", "1F6D1"},
    {"carrito_de_la_compra", "1F6D2"},
    {"templo_hindu", "1F6D5"},
    {"cabana", "1F6D6"},
    {"ascensor", "1F6D7"},
    {"desprendimiento", "1F6D8"},
    {"wifi", "1F6DC"},
    {"tobogan", "1F6DD"},
    {"rueda", "1F6DE"},
    {"salvavidas", "1F6DF"},
    {"martillo_y_llave_inglesa", "1F6E0"},
    {"escudo", "1F6E1"},
    {"barril_de_petroleo", "1F6E2"},
    {"autopista", "1F6E3"},
    {"via_de_tren", "1F6E4"},
    {"barco_a_motor", "1F6E5"},
    {"avioneta", "1F6E9"},
    {"avion_despegando", "1F6EB"},
    {"avion_aterrizando", "1F6EC"},
    {"satelite", "1F6F0"},
    {"barco_de_pasajeros", "1F6F3"},
    {"patinete", "1F6F4"},
    {"scooter", "1F6F5"},
    {"canoa", "1F6F6"},
    {"trineo", "1F6F7"},
    {"platillo_volante", "1F6F8"},
    {"monopatin", "1F6F9"},
    {"mototaxi", "1F6FA"},
    {"camioneta", "1F6FB"},
    {"patines", "1F6FC"},
    {"circulo_naranja", "1F7E0"},
    {"circulo_amarillo", "1F7E1"},
    {"circulo_verde", "1F7E2"},
    {"circulo_morado", "1F7E3"},
    {"circulo_marron", "1F7E4"},
    {"cuadrado_rojo", "1F7E5"},
    {"cuadrado_azul", "1F7E6"},
    {"cuadrado_naranja", "1F7E7"},
    {"cuadrado_amarillo", "1F7E8"},
    {"cuadrado_verde", "1F7E9"},
    {"cuadrado_morado", "1F7EA"},
    {"cuadrado_marron", "1F7EB"},
    {"signo_igual_grueso", "1F7F0"},
    {"dedos_juntos_apuntando_hacia_arriba", "1F90C"},
    {"dedos_juntos_apuntando_hacia_arriba_color1", "1F90C-1F3FB"},
    {"dedos_juntos_apuntando_hacia_arriba_color2", "1F90C-1F3FC"},
    {"dedos_juntos_apuntando_hacia_arriba_color3", "1F90C-1F3FD"},
    {"dedos_juntos_apuntando_hacia_arriba_color4", "1F90C-1F3FE"},
    {"dedos_juntos_apuntando_hacia_arriba_color5", "1F90C-1F3FF"},
    {"corazon_blanco", "1F90D"},
    {"corazon_marron", "1F90E"},
    {"mano_pellizcando", "1F90F"},
    {"mano_pellizcando_color1", "1F90F-1F3FB"},
    {"mano_pellizcando_color2", "1F90F-1F3FC"},
    {"mano_pellizcando_color3", "1F90F-1F3FD"},
    {"mano_pellizcando_color4", "1F90F-1F3FE"},
    {"mano_pellizcando_color5", "1F90F-1F3FF"},
    {"cara_con_la_boca_cerrada_con_cremallera", "1F910"},
    {"cara_con_lengua_de_dinero", "1F911"},
    {"cara_con_termometro", "1F912"},
    {"cara_de_empollon", "1F913"},
    {"cara_pensativa", "1F914"},
    {"cara_con_la_cabeza_vendada", "1F915"},
    {"robot", "1F916"},
    {"cara_con_manos_abrazando", "1F917"},
    {"mano_haciendo_el_signo_de_cuernos", "1F918"},
    {"mano_haciendo_el_signo_de_cuernos_color1", "1F918-1F3FB"},
    {"mano_haciendo_el_signo_de_cuernos_color2", "1F918-1F3FC"},
    {"mano_haciendo_el_signo_de_cuernos_color3", "1F918-1F3FD"},
    {"mano_haciendo_el_signo_de_cuernos_color4", "1F918-1F3FE"},
    {"mano_haciendo_el_signo_de_cuernos_color5", "1F918-1F3FF"},
    {"mano_haciendo_el_gesto_de_llamar", "1F919"},
    {"mano_haciendo_el_gesto_de_llamar_color1", "1F919-1F3FB"},
    {"mano_haciendo_el_gesto_de_llamar_color2", "1F919-1F3FC"},
    {"mano_haciendo_el_gesto_de_llamar_color3", "1F919-1F3FD"},
    {"mano_haciendo_el_gesto_de_llamar_color4", "1F919-1F3FE"},
    {"mano_haciendo_el_gesto_de_llamar_color5", "1F919-1F3FF"},
    {"dorso_de_la_mano", "1F91A"},
    {"dorso_de_la_mano_color1", "1F91A-1F3FB"},
    {"dorso_de_la_mano_color2", "1F91A-1F3FC"},
    {"dorso_de_la_mano_color3", "1F91A-1F3FD"},
    {"dorso_de_la_mano_color4", "1F91A-1F3FE"},
    {"dorso_de_la_mano_color5", "1F91A-1F3FF"},
    {"puno_hacia_la_izquierda", "1F91B"},
    {"puno_hacia_la_izquierda_color1", "1F91B-1F3FB"},
    {"puno_hacia_la_izquierda_color2", "1F91B-1F3FC"},
    {"puno_hacia_la_izquierda_color3", "1F91B-1F3FD"},
    {"puno_hacia_la_izquierda_color4", "1F91B-1F3FE"},
    {"puno_hacia_la_izquierda_color5", "1F91B-1F3FF"},
    {"puno_hacia_la_derecha", "1F91C"},
    {"puno_hacia_la_derecha_color1", "1F91C-1F3FB"},
    {"puno_hacia_la_derecha_color2", "1F91C-1F3FC"},
    {"puno_hacia_la_derecha_color3", "1F91C-1F3FD"},
    {"puno_hacia_la_derecha_color4", "1F91C-1F3FE"},
    {"puno_hacia_la_derecha_color5", "1F91C-1F3FF"},
    {"apreton_de_manos", "1F91D"},
    {"apreton_de_manos_color1", "1F91D-1F3FB"},
    {"apreton_de_manos_color2", "1F91D-1F3FC"},
    {"apreton_de_manos_color3", "1F91D-1F3FD"},
    {"apreton_de_manos_color4", "1F91D-1F3FE"},
    {"apreton_de_manos_color5", "1F91D-1F3FF"},
    {"apreton_de_manos_color1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"apreton_de_manos_color1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"apreton_de_manos_color1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"apreton_de_manos_color1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"apreton_de_manos_color2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"apreton_de_manos_color2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"apreton_de_manos_color2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"apreton_de_manos_color2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"apreton_de_manos_color3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"apreton_de_manos_color3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"apreton_de_manos_color3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"apreton_de_manos_color3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"apreton_de_manos_color4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"apreton_de_manos_color4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"apreton_de_manos_color4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"apreton_de_manos_color4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"apreton_de_manos_color5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"apreton_de_manos_color5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"apreton_de_manos_color5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"apreton_de_manos_color5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"dedos_cruzados", "1F91E"},
    {"dedos_cruzados_color1", "1F91E-1F3FB"},
    {"dedos_cruzados_color2", "1F91E-1F3FC"},
    {"dedos_cruzados_color3", "1F91E-1F3FD"},
    {"dedos_cruzados_color4", "1F91E-1F3FE"},
    {"dedos_cruzados_color5", "1F91E-1F3FF"},
    {"gesto_de_te_quiero", "1F91F"},
    {"gesto_de_te_quiero_color1", "1F91F-1F3FB"},
    {"gesto_de_te_quiero_color2", "1F91F-1F3FC"},
    {"gesto_de_te_quiero_color3", "1F91F-1F3FD"},
    {"gesto_de_te_quiero_color4", "1F91F-1F3FE"},
    {"gesto_de_te_quiero_color5", "1F91F-1F3FF"},
    {"cara_con_sombrero_de_vaquero", "1F920"},
    {"cara_de_payaso", "1F921"},
    {"cara_de_nauseas", "1F922"},
    {"cara_revolviendose_de_la_risa", "1F923"},
    {"cara_babeando", "1F924"},
    {"cara_de_mentiroso", "1F925"},
    {"persona_con_la_mano_en_la_frente", "1F926"},
    {"persona_con_la_mano_en_la_frente_color1", "1F926-1F3FB"},
    {"persona_con_la_mano_en_la_frente_color2", "1F926-1F3FC"},
    {"persona_con_la_mano_en_la_frente_color3", "1F926-1F3FD"},
    {"persona_con_la_mano_en_la_frente_color4", "1F926-1F3FE"},
    {"persona_con_la_mano_en_la_frente_color5", "1F926-1F3FF"},
    {"cara_estornudando", "1F927"},
    {"cara_con_ceja_alzada", "1F928"},
    {"cara_sonriendo_con_estrellas", "1F929"},
    {"cara_de_loco", "1F92A"},
    {"cara_pidiendo_silencio", "1F92B"},
    {"cara_con_simbolos_en_la_boca", "1F92C"},
    {"cara_con_mano_sobre_la_boca", "1F92D"},
    {"cara_vomitando", "1F92E"},
    {"cabeza_explotando", "1F92F"},
    {"mujer_embarazada", "1F930"},
    {"mujer_embarazada_color1", "1F930-1F3FB"},
    {"mujer_embarazada_color2", "1F930-1F3FC"},
    {"mujer_embarazada_color3", "1F930-1F3FD"},
    {"mujer_embarazada_color4", "1F930-1F3FE"},
    {"mujer_embarazada_color5", "1F930-1F3FF"},
    {"lactancia_materna", "1F931"},
    {"lactancia_materna_color1", "1F931-1F3FB"},
    {"lactancia_materna_color2", "1F931-1F3FC"},
    {"lactancia_materna_color3", "1F931-1F3FD"},
    {"lactancia_materna_color4", "1F931-1F3FE"},
    {"lactancia_materna_color5", "1F931-1F3FF"},
    {"palmas_hacia_arriba_juntas", "1F932"},
    {"palmas_hacia_arriba_juntas_color1", "1F932-1F3FB"},
    {"palmas_hacia_arriba_juntas_color2", "1F932-1F3FC"},
    {"palmas_hacia_arriba_juntas_color3", "1F932-1F3FD"},
    {"palmas_hacia_arriba_juntas_color4", "1F932-1F3FE"},
    {"palmas_hacia_arriba_juntas_color5", "1F932-1F3FF"},
    {"selfi", "1F933"},
    {"selfi_color1", "1F933-1F3FB"},
    {"selfi_color2", "1F933-1F3FC"},
    {"selfi_color3", "1F933-1F3FD"},
    {"selfi_color4", "1F933-1F3FE"},
    {"selfi_color5", "1F933-1F3FF"},
    {"principe", "1F934"},
    {"principe_color1", "1F934-1F3FB"},
    {"principe_color2", "1F934-1F3FC"},
    {"principe_color3", "1F934-1F3FD"},
    {"principe_color4", "1F934-1F3FE"},
    {"principe_color5", "1F934-1F3FF"},
    {"persona_con_esmoquin", "1F935"},
    {"persona_con_esmoquin_color1", "1F935-1F3FB"},
    {"persona_con_esmoquin_color2", "1F935-1F3FC"},
    {"persona_con_esmoquin_color3", "1F935-1F3FD"},
    {"persona_con_esmoquin_color4", "1F935-1F3FE"},
    {"persona_con_esmoquin_color5", "1F935-1F3FF"},
    {"mama_noel", "1F936"},
    {"mama_noel_color1", "1F936-1F3FB"},
    {"mama_noel_color2", "1F936-1F3FC"},
    {"mama_noel_color3", "1F936-1F3FD"},
    {"mama_noel_color4", "1F936-1F3FE"},
    {"mama_noel_color5", "1F936-1F3FF"},
    {"persona_encogida_de_hombros", "1F937"},
    {"persona_encogida_de_hombros_color1", "1F937-1F3FB"},
    {"persona_encogida_de_hombros_color2", "1F937-1F3FC"},
    {"persona_encogida_de_hombros_color3", "1F937-1F3FD"},
    {"persona_encogida_de_hombros_color4", "1F937-1F3FE"},
    {"persona_encogida_de_hombros_color5", "1F937-1F3FF"},
    {"persona_haciendo_voltereta_lateral", "1F938"},
    {"persona_haciendo_voltereta_lateral_color1", "1F938-1F3FB"},
    {"persona_haciendo_voltereta_lateral_color2", "1F938-1F3FC"},
    {"persona_haciendo_voltereta_lateral_color3", "1F938-1F3FD"},
    {"persona_haciendo_voltereta_lateral_color4", "1F938-1F3FE"},
    {"persona_haciendo_voltereta_lateral_color5", "1F938-1F3FF"},
    {"persona_haciendo_malabares", "1F939"},
    {"persona_haciendo_malabares_color1", "1F939-1F3FB"},
    {"persona_haciendo_malabares_color2", "1F939-1F3FC"},
    {"persona_haciendo_malabares_color3", "1F939-1F3FD"},
    {"persona_haciendo_malabares_color4", "1F939-1F3FE"},
    {"persona_haciendo_malabares_color5", "1F939-1F3FF"},
    {"persona_haciendo_esgrima", "1F93A"},
    {"personas_luchando", "1F93C"},
    {"personas_luchando_color1", "1F93C-1F3FB"},
    {"personas_luchando_color2", "1F93C-1F3FC"},
    {"personas_luchando_color3", "1F93C-1F3FD"},
    {"personas_luchando_color4", "1F93C-1F3FE"},
    {"personas_luchando_color5", "1F93C-1F3FF"},
    {"personas_luchando_color1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"personas_luchando_color1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"personas_luchando_color1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"personas_luchando_color1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"personas_luchando_color2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"personas_luchando_color2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"personas_luchando_color2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"personas_luchando_color2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"personas_luchando_color3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"personas_luchando_color3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"personas_luchando_color3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"personas_luchando_color3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"personas_luchando_color4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"personas_luchando_color4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"personas_luchando_color4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"personas_luchando_color4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"personas_luchando_color5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"personas_luchando_color5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"personas_luchando_color5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"personas_luchando_color5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"persona_jugando_al_waterpolo", "1F93D"},
    {"persona_jugando_al_waterpolo_color1", "1F93D-1F3FB"},
    {"persona_jugando_al_waterpolo_color2", "1F93D-1F3FC"},
    {"persona_jugando_al_waterpolo_color3", "1F93D-1F3FD"},
    {"persona_jugando_al_waterpolo_color4", "1F93D-1F3FE"},
    {"persona_jugando_al_waterpolo_color5", "1F93D-1F3FF"},
    {"persona_jugando_al_balonmano", "1F93E"},
    {"persona_jugando_al_balonmano_color1", "1F93E-1F3FB"},
    {"persona_jugando_al_balonmano_color2", "1F93E-1F3FC"},
    {"persona_jugando_al_balonmano_color3", "1F93E-1F3FD"},
    {"persona_jugando_al_balonmano_color4", "1F93E-1F3FE"},
    {"persona_jugando_al_balonmano_color5", "1F93E-1F3FF"},
    {"mascara_de_buceo", "1F93F"},
    {"flor_marchita", "1F940"},
    {"tambor", "1F941"},
    {"copas_brindando", "1F942"},
    {"vaso_de_whisky", "1F943"},
    {"cuchara", "1F944"},
    {"porteria", "1F945"},
    {"medalla_de_oro", "1F947"},
    {"medalla_de_plata", "1F948"},
    {"medalla_de_bronce", "1F949"},
    {"guante_de_boxeo", "1F94A"},
    {"uniforme_de_artes_marciales", "1F94B"},
    {"piedra_de_curling", "1F94C"},
    {"lacrosse", "1F94D"},
    {"pelota_de_softball", "1F94E"},
    {"disco_volador", "1F94F"},
    {"cruasan", "1F950"},
    {"aguacate", "1F951"},
    {"pepino", "1F952"},
    {"beicon", "1F953"},
    {"patata", "1F954"},
    {"zanahoria", "1F955"},
    {"baguete", "1F956"},
    {"ensalada", "1F957"},
    {"paella", "1F958"},
    {"pan_relleno", "1F959"},
    {"huevo", "1F95A"},
    {"vaso_de_leche", "1F95B"},
    {"cacahuetes", "1F95C"},
    {"kiwi", "1F95D"},
    {"tortitas", "1F95E"},
    {"dumpling", "1F95F"},
    {"galleta_de_la_fortuna", "1F960"},
    {"caja_para_llevar", "1F961"},
    {"palillos", "1F962"},
    {"cuenco_con_cuchara", "1F963"},
    {"vaso_con_pajita", "1F964"},
    {"coco", "1F965"},
    {"brocoli", "1F966"},
    {"pastel", "1F967"},
    {"bretzel", "1F968"},
    {"corte_de_carne", "1F969"},
    {"sandwich", "1F96A"},
    {"comida_enlatada", "1F96B"},
    {"verdura_de_hoja_verde", "1F96C"},
    {"mango", "1F96D"},
    {"pastel_de_luna", "1F96E"},
    {"bagel", "1F96F"},
    {"cara_sonriendo_con_corazones", "1F970"},
    {"cara_de_bostezo", "1F971"},
    {"cara_sonriente_con_lagrima", "1F972"},
    {"cara_de_fiesta", "1F973"},
    {"cara_de_grogui", "1F974"},
    {"cara_con_calor", "1F975"},
    {"cara_con_frio", "1F976"},
    {"ninja", "1F977"},
    {"ninja_color1", "1F977-1F3FB"},
    {"ninja_color2", "1F977-1F3FC"},
    {"ninja_color3", "1F977-1F3FD"},
    {"ninja_color4", "1F977-1F3FE"},
    {"ninja_color5", "1F977-1F3FF"},
    {"cara_disfrazada", "1F978"},
    {"cara_aguantandose_las_lagrimas", "1F979"},
    {"cara_suplicante", "1F97A"},
    {"sari", "1F97B"},
    {"bata_de_laboratorio", "1F97C"},
    {"gafas_de_proteccion", "1F97D"},
    {"bota_de_senderismo", "1F97E"},
    {"bailarina", "1F97F"},
    {"cangrejo", "1F980"},
    {"leon", "1F981"},
    {"escorpion", "1F982"},
    {"pavo", "1F983"},
    {"unicornio", "1F984"},
    {"aguila", "1F985"},
    {"pato", "1F986"},
    {"murcielago", "1F987"},
    {"tiburon", "1F988"},
    {"buho", "1F989"},
    {"zorro", "1F98A"},
    {"mariposa", "1F98B"},
    {"ciervo", "1F98C"},
    {"gorila", "1F98D"},
    {"lagarto", "1F98E"},
    {"rinoceronte", "1F98F"},
    {"gamba", "1F990"},
    {"calamar", "1F991"},
    {"jirafa", "1F992"},
    {"cebra", "1F993"},
    {"erizo", "1F994"},
    {"sauropodo", "1F995"},
    {"trex", "1F996"},
    {"grillo", "1F997"},
    {"canguro", "1F998"},
    {"llama", "1F999"},
    {"pavo_real", "1F99A"},
    {"hipopotamo", "1F99B"},
    {"loro", "1F99C"},
    {"mapache", "1F99D"},
    {"bogavante", "1F99E"},
    {"mosquito", "1F99F"},
    {"microbio", "1F9A0"},
    {"tejon", "1F9A1"},
    {"cisne", "1F9A2"},
    {"mamut", "1F9A3"},
    {"dodo", "1F9A4"},
    {"perezoso", "1F9A5"},
    {"nutria", "1F9A6"},
    {"orangutan", "1F9A7"},
    {"mofeta", "1F9A8"},
    {"flamenco", "1F9A9"},
    {"ostra", "1F9AA"},
    {"castor", "1F9AB"},
    {"bisonte", "1F9AC"},
    {"foca", "1F9AD"},
    {"perro_guia", "1F9AE"},
    {"baston", "1F9AF"},
    {"pelo_pelirrojo", "1F9B0"},
    {"pelo_rizado", "1F9B1"},
    {"sin_pelo", "1F9B2"},
    {"pelo_blanco", "1F9B3"},
    {"hueso", "1F9B4"},
    {"pierna", "1F9B5"},
    {"pierna_color1", "1F9B5-1F3FB"},
    {"pierna_color2", "1F9B5-1F3FC"},
    {"pierna_color3", "1F9B5-1F3FD"},
    {"pierna_color4", "1F9B5-1F3FE"},
    {"pierna_color5", "1F9B5-1F3FF"},
    {"pie", "1F9B6"},
    {"pie_color1", "1F9B6-1F3FB"},
    {"pie_color2", "1F9B6-1F3FC"},
    {"pie_color3", "1F9B6-1F3FD"},
    {"pie_color4", "1F9B6-1F3FE"},
    {"pie_color5", "1F9B6-1F3FF"},
    {"diente", "1F9B7"},
    {"personaje_de_superheroe", "1F9B8"},
    {"personaje_de_superheroe_color1", "1F9B8-1F3FB"},
    {"personaje_de_superheroe_color2", "1F9B8-1F3FC"},
    {"personaje_de_superheroe_color3", "1F9B8-1F3FD"},
    {"personaje_de_superheroe_color4", "1F9B8-1F3FE"},
    {"personaje_de_superheroe_color5", "1F9B8-1F3FF"},
    {"personaje_de_supervillano", "1F9B9"},
    {"personaje_de_supervillano_color1", "1F9B9-1F3FB"},
    {"personaje_de_supervillano_color2", "1F9B9-1F3FC"},
    {"personaje_de_supervillano_color3", "1F9B9-1F3FD"},
    {"personaje_de_supervillano_color4", "1F9B9-1F3FE"},
    {"personaje_de_supervillano_color5", "1F9B9-1F3FF"},
    {"chaleco_de_seguridad", "1F9BA"},
    {"oreja_con_audifono", "1F9BB"},
    {"oreja_con_audifono_color1", "1F9BB-1F3FB"},
    {"oreja_con_audifono_color2", "1F9BB-1F3FC"},
    {"oreja_con_audifono_color3", "1F9BB-1F3FD"},
    {"oreja_con_audifono_color4", "1F9BB-1F3FE"},
    {"oreja_con_audifono_color5", "1F9BB-1F3FF"},
    {"silla_de_ruedas_electrica", "1F9BC"},
    {"silla_de_ruedas_manual", "1F9BD"},
    {"brazo_mecanico", "1F9BE"},
    {"pierna_mecanica", "1F9BF"},
    {"cuna_de_queso", "1F9C0"},
    {"magdalena", "1F9C1"},
    {"sal", "1F9C2"},
    {"tetrabrik", "1F9C3"},
    {"ajo", "1F9C4"},
    {"cebolla", "1F9C5"},
    {"falafel", "1F9C6"},
    {"gofre", "1F9C7"},
    {"mantequilla", "1F9C8"},
    {"mate", "1F9C9"},
    {"cubito_de_hielo", "1F9CA"},
    {"te_de_burbujas", "1F9CB"},
    {"trol", "1F9CC"},
    {"persona_de_pie", "1F9CD"},
    {"persona_de_pie_color1", "1F9CD-1F3FB"},
    {"persona_de_pie_color2", "1F9CD-1F3FC"},
    {"persona_de_pie_color3", "1F9CD-1F3FD"},
    {"persona_de_pie_color4", "1F9CD-1F3FE"},
    {"persona_de_pie_color5", "1F9CD-1F3FF"},
    {"persona_de_rodillas", "1F9CE"},
    {"persona_de_rodillas_color1", "1F9CE-1F3FB"},
    {"persona_de_rodillas_color2", "1F9CE-1F3FC"},
    {"persona_de_rodillas_color3", "1F9CE-1F3FD"},
    {"persona_de_rodillas_color4", "1F9CE-1F3FE"},
    {"persona_de_rodillas_color5", "1F9CE-1F3FF"},
    {"persona_sorda", "1F9CF"},
    {"persona_sorda_color1", "1F9CF-1F3FB"},
    {"persona_sorda_color2", "1F9CF-1F3FC"},
    {"persona_sorda_color3", "1F9CF-1F3FD"},
    {"persona_sorda_color4", "1F9CF-1F3FE"},
    {"persona_sorda_color5", "1F9CF-1F3FF"},
    {"cara_con_monoculo", "1F9D0"},
    {"persona_adulta", "1F9D1"},
    {"persona_adulta_color1", "1F9D1-1F3FB"},
    {"persona_adulta_color2", "1F9D1-1F3FC"},
    {"persona_adulta_color3", "1F9D1-1F3FD"},
    {"persona_adulta_color4", "1F9D1-1F3FE"},
    {"persona_adulta_color5", "1F9D1-1F3FF"},
    {"infante", "1F9D2"},
    {"infante_color1", "1F9D2-1F3FB"},
    {"infante_color2", "1F9D2-1F3FC"},
    {"infante_color3", "1F9D2-1F3FD"},
    {"infante_color4", "1F9D2-1F3FE"},
    {"infante_color5", "1F9D2-1F3FF"},
    {"persona_mayor", "1F9D3"},
    {"persona_mayor_color1", "1F9D3-1F3FB"},
    {"persona_mayor_color2", "1F9D3-1F3FC"},
    {"persona_mayor_color3", "1F9D3-1F3FD"},
    {"persona_mayor_color4", "1F9D3-1F3FE"},
    {"persona_mayor_color5", "1F9D3-1F3FF"},
    {"persona_con_barba", "1F9D4"},
    {"persona_con_barba_color1", "1F9D4-1F3FB"},
    {"persona_con_barba_color2", "1F9D4-1F3FC"},
    {"persona_con_barba_color3", "1F9D4-1F3FD"},
    {"persona_con_barba_color4", "1F9D4-1F3FE"},
    {"persona_con_barba_color5", "1F9D4-1F3FF"},
    {"mujer_con_hiyab", "1F9D5"},
    {"mujer_con_hiyab_color1", "1F9D5-1F3FB"},
    {"mujer_con_hiyab_color2", "1F9D5-1F3FC"},
    {"mujer_con_hiyab_color3", "1F9D5-1F3FD"},
    {"mujer_con_hiyab_color4", "1F9D5-1F3FE"},
    {"mujer_con_hiyab_color5", "1F9D5-1F3FF"},
    {"persona_en_una_sauna", "1F9D6"},
    {"persona_en_una_sauna_color1", "1F9D6-1F3FB"},
    {"persona_en_una_sauna_color2", "1F9D6-1F3FC"},
    {"persona_en_una_sauna_color3", "1F9D6-1F3FD"},
    {"persona_en_una_sauna_color4", "1F9D6-1F3FE"},
    {"persona_en_una_sauna_color5", "1F9D6-1F3FF"},
    {"persona_escalando", "1F9D7"},
    {"persona_escalando_color1", "1F9D7-1F3FB"},
    {"persona_escalando_color2", "1F9D7-1F3FC"},
    {"persona_escalando_color3", "1F9D7-1F3FD"},
    {"persona_escalando_color4", "1F9D7-1F3FE"},
    {"persona_escalando_color5", "1F9D7-1F3FF"},
    {"persona_en_posicion_de_loto", "1F9D8"},
    {"persona_en_posicion_de_loto_color1", "1F9D8-1F3FB"},
    {"persona_en_posicion_de_loto_color2", "1F9D8-1F3FC"},
    {"persona_en_posicion_de_loto_color3", "1F9D8-1F3FD"},
    {"persona_en_posicion_de_loto_color4", "1F9D8-1F3FE"},
    {"persona_en_posicion_de_loto_color5", "1F9D8-1F3FF"},
    {"persona_maga", "1F9D9"},
    {"persona_maga_color1", "1F9D9-1F3FB"},
    {"persona_maga_color2", "1F9D9-1F3FC"},
    {"persona_maga_color3", "1F9D9-1F3FD"},
    {"persona_maga_color4", "1F9D9-1F3FE"},
    {"persona_maga_color5", "1F9D9-1F3FF"},
    {"hada", "1F9DA"},
    {"hada_color1", "1F9DA-1F3FB"},
    {"hada_color2", "1F9DA-1F3FC"},
    {"hada_color3", "1F9DA-1F3FD"},
    {"hada_color4", "1F9DA-1F3FE"},
    {"hada_color5", "1F9DA-1F3FF"},
    {"vampiro", "1F9DB"},
    {"vampiro_color1", "1F9DB-1F3FB"},
    {"vampiro_color2", "1F9DB-1F3FC"},
    {"vampiro_color3", "1F9DB-1F3FD"},
    {"vampiro_color4", "1F9DB-1F3FE"},
    {"vampiro_color5", "1F9DB-1F3FF"},
    {"persona_sirena", "1F9DC"},
    {"persona_sirena_color1", "1F9DC-1F3FB"},
    {"persona_sirena_color2", "1F9DC-1F3FC"},
    {"persona_sirena_color3", "1F9DC-1F3FD"},
    {"persona_sirena_color4", "1F9DC-1F3FE"},
    {"persona_sirena_color5", "1F9DC-1F3FF"},
    {"elfo", "1F9DD"},
    {"elfo_color1", "1F9DD-1F3FB"},
    {"elfo_color2", "1F9DD-1F3FC"},
    {"elfo_color3", "1F9DD-1F3FD"},
    {"elfo_color4", "1F9DD-1F3FE"},
    {"elfo_color5", "1F9DD-1F3FF"},
    {"genio", "1F9DE"},
    {"zombi", "1F9DF"},
    {"cerebro", "1F9E0"},
    {"corazon_naranja", "1F9E1"},
    {"gorra_con_visera", "1F9E2"},
    {"bufanda", "1F9E3"},
    {"guantes", "1F9E4"},
    {"abrigo", "1F9E5"},
    {"calcetines", "1F9E6"},
    {"sobre_rojo", "1F9E7"},
    {"petardo", "1F9E8"},
    {"pieza_de_puzle", "1F9E9"},
    {"tubo_de_ensayo", "1F9EA"},
    {"placa_de_petri", "1F9EB"},
    {"adn", "1F9EC"},
    {"brujula", "1F9ED"},
    {"abaco", "1F9EE"},
    {"extintor", "1F9EF"},
    {"caja_de_herramientas", "1F9F0"},
    {"ladrillo", "1F9F1"},
    {"iman", "1F9F2"},
    {"equipaje", "1F9F3"},
    {"bote_de_crema", "1F9F4"},
    {"hilo", "1F9F5"},
    {"ovillo", "1F9F6"},
    {"imperdible", "1F9F7"},
    {"osito_de_peluche", "1F9F8"},
    {"escoba", "1F9F9"},
    {"cesta", "1F9FA"},
    {"rollo_de_papel", "1F9FB"},
    {"jabon", "1F9FC"},
    {"esponja", "1F9FD"},
    {"recibo", "1F9FE"},
    {"ojo_turco", "1F9FF"},
    {"zapatillas_de_ballet", "1FA70"},
    {"traje_de_bano_de_una_pieza", "1FA71"},
    {"ropa_interior", "1FA72"},
    {"pantalones_cortos", "1FA73"},
    {"chancla", "1FA74"},
    {"corazon_azul_claro", "1FA75"},
    {"corazon_gris", "1FA76"},
    {"corazon_rosa", "1FA77"},
    {"gota_de_sangre", "1FA78"},
    {"tirita", "1FA79"},
    {"estetoscopio", "1FA7A"},
    {"radiografia", "1FA7B"},
    {"muleta", "1FA7C"},
    {"yoyo", "1FA80"},
    {"cometa", "1FA81"},
    {"paracaidas", "1FA82"},
    {"bumeran", "1FA83"},
    {"varita_magica", "1FA84"},
    {"pinata", "1FA85"},
    {"muneca_rusa", "1FA86"},
    {"maracas", "1FA87"},
    {"flauta", "1FA88"},
    {"arpa", "1FA89"},
    {"trombon", "1FA8A"},
    {"cofre_de_tesoro", "1FA8E"},
    {"pala", "1FA8F"},
    {"planeta_con_anillos", "1FA90"},
    {"silla", "1FA91"},
    {"cuchilla_de_afeitar", "1FA92"},
    {"hacha", "1FA93"},
    {"lampara_de_aceite", "1FA94"},
    {"banjo", "1FA95"},
    {"casco_militar", "1FA96"},
    {"acordeon", "1FA97"},
    {"tamboril", "1FA98"},
    {"moneda", "1FA99"},
    {"sierra_de_carpinteria", "1FA9A"},
    {"destornillador", "1FA9B"},
    {"escalera", "1FA9C"},
    {"gancho", "1FA9D"},
    {"espejo", "1FA9E"},
    {"ventana", "1FA9F"},
    {"desatascador", "1FAA0"},
    {"aguja_de_coser", "1FAA1"},
    {"nudo", "1FAA2"},
    {"cubo", "1FAA3"},
    {"trampa_de_ratones", "1FAA4"},
    {"cepillo_de_dientes", "1FAA5"},
    {"lapida", "1FAA6"},
    {"letrero", "1FAA7"},
    {"piedra", "1FAA8"},
    {"bola_de_espejos", "1FAA9"},
    {"carne_de_identidad", "1FAAA"},
    {"bateria_baja", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"abanico_abierto", "1FAAD"},
    {"peineta", "1FAAE"},
    {"khanda", "1FAAF"},
    {"mosca", "1FAB0"},
    {"gusano", "1FAB1"},
    {"escarabajo", "1FAB2"},
    {"cucaracha", "1FAB3"},
    {"planta_de_maceta", "1FAB4"},
    {"madera", "1FAB5"},
    {"pluma", "1FAB6"},
    {"loto", "1FAB7"},
    {"coral", "1FAB8"},
    {"nido_vacio", "1FAB9"},
    {"nido_con_huevos", "1FABA"},
    {"campanilla", "1FABB"},
    {"medusa", "1FABC"},
    {"ala", "1FABD"},
    {"arbol_sin_hojas", "1FABE"},
    {"oca", "1FABF"},
    {"corazon_humano", "1FAC0"},
    {"pulmones", "1FAC1"},
    {"personas_abrazandose", "1FAC2"},
    {"hombre_embarazado", "1FAC3"},
    {"hombre_embarazado_color1", "1FAC3-1F3FB"},
    {"hombre_embarazado_color2", "1FAC3-1F3FC"},
    {"hombre_embarazado_color3", "1FAC3-1F3FD"},
    {"hombre_embarazado_color4", "1FAC3-1F3FE"},
    {"hombre_embarazado_color5", "1FAC3-1F3FF"},
    {"persona_embarazada", "1FAC4"},
    {"persona_embarazada_color1", "1FAC4-1F3FB"},
    {"persona_embarazada_color2", "1FAC4-1F3FC"},
    {"persona_embarazada_color3", "1FAC4-1F3FD"},
    {"persona_embarazada_color4", "1FAC4-1F3FE"},
    {"persona_embarazada_color5", "1FAC4-1F3FF"},
    {"persona_con_corona", "1FAC5"},
    {"persona_con_corona_color1", "1FAC5-1F3FB"},
    {"persona_con_corona_color2", "1FAC5-1F3FC"},
    {"persona_con_corona_color3", "1FAC5-1F3FD"},
    {"persona_con_corona_color4", "1FAC5-1F3FE"},
    {"persona_con_corona_color5", "1FAC5-1F3FF"},
    {"huella_dactilar", "1FAC6"},
    {"criatura_peluda", "1FAC8"},
    {"orca", "1FACD"},
    {"alce", "1FACE"},
    {"burro", "1FACF"},
    {"arandanos", "1FAD0"},
    {"pimiento", "1FAD1"},
    {"aceituna", "1FAD2"},
    {"pan_sin_levadura", "1FAD3"},
    {"tamal", "1FAD4"},
    {"fondue", "1FAD5"},
    {"tetera", "1FAD6"},
    {"liquido_derramandose", "1FAD7"},
    {"alubias", "1FAD8"},
    {"tarro", "1FAD9"},
    {"raiz_de_jengibre", "1FADA"},
    {"vaina", "1FADB"},
    {"tuberculo_comestible", "1FADC"},
    {"salpicadura", "1FADF"},
    {"cara_derritiendose", "1FAE0"},
    {"cara_saludando", "1FAE1"},
    {"cara_con_ojos_abiertos_y_boca_tapada", "1FAE2"},
    {"cara_tapada_con_ojo_espiando", "1FAE3"},
    {"cara_con_boca_diagonal", "1FAE4"},
    {"cara_con_linea_de_puntos", "1FAE5"},
    {"labio_mordido", "1FAE6"},
    {"burbujas", "1FAE7"},
    {"cara_temblorosa", "1FAE8"},
    {"cara_con_ojeras", "1FAE9"},
    {"cara_deformada", "1FAEA"},
    {"nube_de_pelea", "1FAEF"},
    {"mano_con_dedo_indice_y_pulgar_cruzados", "1FAF0"},
    {"mano_con_dedo_indice_y_pulgar_cruzados_color1", "1FAF0-1F3FB"},
    {"mano_con_dedo_indice_y_pulgar_cruzados_color2", "1FAF0-1F3FC"},
    {"mano_con_dedo_indice_y_pulgar_cruzados_color3", "1FAF0-1F3FD"},
    {"mano_con_dedo_indice_y_pulgar_cruzados_color4", "1FAF0-1F3FE"},
    {"mano_con_dedo_indice_y_pulgar_cruzados_color5", "1FAF0-1F3FF"},
    {"mano_hacia_la_derecha", "1FAF1"},
    {"mano_hacia_la_derecha_color1", "1FAF1-1F3FB"},
    {"mano_hacia_la_derecha_color2", "1FAF1-1F3FC"},
    {"mano_hacia_la_derecha_color3", "1FAF1-1F3FD"},
    {"mano_hacia_la_derecha_color4", "1FAF1-1F3FE"},
    {"mano_hacia_la_derecha_color5", "1FAF1-1F3FF"},
    {"mano_hacia_la_izquierda", "1FAF2"},
    {"mano_hacia_la_izquierda_color1", "1FAF2-1F3FB"},
    {"mano_hacia_la_izquierda_color2", "1FAF2-1F3FC"},
    {"mano_hacia_la_izquierda_color3", "1FAF2-1F3FD"},
    {"mano_hacia_la_izquierda_color4", "1FAF2-1F3FE"},
    {"mano_hacia_la_izquierda_color5", "1FAF2-1F3FF"},
    {"mano_con_la_palma_hacia_abajo", "1FAF3"},
    {"mano_con_la_palma_hacia_abajo_color1", "1FAF3-1F3FB"},
    {"mano_con_la_palma_hacia_abajo_color2", "1FAF3-1F3FC"},
    {"mano_con_la_palma_hacia_abajo_color3", "1FAF3-1F3FD"},
    {"mano_con_la_palma_hacia_abajo_color4", "1FAF3-1F3FE"},
    {"mano_con_la_palma_hacia_abajo_color5", "1FAF3-1F3FF"},
    {"mano_con_la_palma_hacia_arriba", "1FAF4"},
    {"mano_con_la_palma_hacia_arriba_color1", "1FAF4-1F3FB"},
    {"mano_con_la_palma_hacia_arriba_color2", "1FAF4-1F3FC"},
    {"mano_con_la_palma_hacia_arriba_color3", "1FAF4-1F3FD"},
    {"mano_con_la_palma_hacia_arriba_color4", "1FAF4-1F3FE"},
    {"mano_con_la_palma_hacia_arriba_color5", "1FAF4-1F3FF"},
    {"dedo_indice_apuntandote_a_ti", "1FAF5"},
    {"dedo_indice_apuntandote_a_ti_color1", "1FAF5-1F3FB"},
    {"dedo_indice_apuntandote_a_ti_color2", "1FAF5-1F3FC"},
    {"dedo_indice_apuntandote_a_ti_color3", "1FAF5-1F3FD"},
    {"dedo_indice_apuntandote_a_ti_color4", "1FAF5-1F3FE"},
    {"dedo_indice_apuntandote_a_ti_color5", "1FAF5-1F3FF"},
    {"manos_formando_un_corazon", "1FAF6"},
    {"manos_formando_un_corazon_color1", "1FAF6-1F3FB"},
    {"manos_formando_un_corazon_color2", "1FAF6-1F3FC"},
    {"manos_formando_un_corazon_color3", "1FAF6-1F3FD"},
    {"manos_formando_un_corazon_color4", "1FAF6-1F3FE"},
    {"manos_formando_un_corazon_color5", "1FAF6-1F3FF"},
    {"mano_empujando_hacia_la_izquierda", "1FAF7"},
    {"mano_empujando_hacia_la_izquierda_color1", "1FAF7-1F3FB"},
    {"mano_empujando_hacia_la_izquierda_color2", "1FAF7-1F3FC"},
    {"mano_empujando_hacia_la_izquierda_color3", "1FAF7-1F3FD"},
    {"mano_empujando_hacia_la_izquierda_color4", "1FAF7-1F3FE"},
    {"mano_empujando_hacia_la_izquierda_color5", "1FAF7-1F3FF"},
    {"mano_empujando_hacia_la_derecha", "1FAF8"},
    {"mano_empujando_hacia_la_derecha_color1", "1FAF8-1F3FB"},
    {"mano_empujando_hacia_la_derecha_color2", "1FAF8-1F3FC"},
    {"mano_empujando_hacia_la_derecha_color3", "1FAF8-1F3FD"},
    {"mano_empujando_hacia_la_derecha_color4", "1FAF8-1F3FE"},
    {"mano_empujando_hacia_la_derecha_color5", "1FAF8-1F3FF"},
    {"teclas_simbolo_de_numero", "0023-FE0F-20E3"},
    {"teclas_asterisco", "002A-FE0F-20E3"},
    {"tecla_0", "0030-FE0F-20E3"},
    {"tecla_1", "0031-FE0F-20E3"},
    {"tecla_2", "0032-FE0F-20E3"},
    {"tecla_3", "0033-FE0F-20E3"},
    {"tecla_4", "0034-FE0F-20E3"},
    {"tecla_5", "0035-FE0F-20E3"},
    {"tecla_6", "0036-FE0F-20E3"},
    {"tecla_7", "0037-FE0F-20E3"},
    {"tecla_8", "0038-FE0F-20E3"},
    {"tecla_9", "0039-FE0F-20E3"},
    {"bandera_isla_de_la_ascension", "1F1E6-1F1E8"},
    {"bandera_andorra", "1F1E6-1F1E9"},
    {"bandera_emiratos_arabes_unidos", "1F1E6-1F1EA"},
    {"bandera_afganistan", "1F1E6-1F1EB"},
    {"bandera_antigua_y_barbuda", "1F1E6-1F1EC"},
    {"bandera_anguila", "1F1E6-1F1EE"},
    {"bandera_albania", "1F1E6-1F1F1"},
    {"bandera_armenia", "1F1E6-1F1F2"},
    {"bandera_angola", "1F1E6-1F1F4"},
    {"bandera_antartida", "1F1E6-1F1F6"},
    {"bandera_argentina", "1F1E6-1F1F7"},
    {"bandera_samoa_americana", "1F1E6-1F1F8"},
    {"bandera_austria", "1F1E6-1F1F9"},
    {"bandera_australia", "1F1E6-1F1FA"},
    {"bandera_aruba", "1F1E6-1F1FC"},
    {"bandera_islas_aland", "1F1E6-1F1FD"},
    {"bandera_azerbaiyan", "1F1E6-1F1FF"},
    {"bandera_bosnia_y_herzegovina", "1F1E7-1F1E6"},
    {"bandera_barbados", "1F1E7-1F1E7"},
    {"bandera_banglades", "1F1E7-1F1E9"},
    {"bandera_belgica", "1F1E7-1F1EA"},
    {"bandera_burkina_faso", "1F1E7-1F1EB"},
    {"bandera_bulgaria", "1F1E7-1F1EC"},
    {"bandera_barein", "1F1E7-1F1ED"},
    {"bandera_burundi", "1F1E7-1F1EE"},
    {"bandera_benin", "1F1E7-1F1EF"},
    {"bandera_san_bartolome", "1F1E7-1F1F1"},
    {"bandera_bermudas", "1F1E7-1F1F2"},
    {"bandera_brunei", "1F1E7-1F1F3"},
    {"bandera_bolivia", "1F1E7-1F1F4"},
    {"bandera_caribe_neerlandes", "1F1E7-1F1F6"},
    {"bandera_brasil", "1F1E7-1F1F7"},
    {"bandera_bahamas", "1F1E7-1F1F8"},
    {"bandera_butan", "1F1E7-1F1F9"},
    {"bandera_isla_bouvet", "1F1E7-1F1FB"},
    {"bandera_botsuana", "1F1E7-1F1FC"},
    {"bandera_bielorrusia", "1F1E7-1F1FE"},
    {"bandera_belice", "1F1E7-1F1FF"},
    {"bandera_canada", "1F1E8-1F1E6"},
    {"bandera_islas_cocos", "1F1E8-1F1E8"},
    {"bandera_republica_democratica_del_congo", "1F1E8-1F1E9"},
    {"bandera_republica_centroafricana", "1F1E8-1F1EB"},
    {"bandera_congo", "1F1E8-1F1EC"},
    {"bandera_suiza", "1F1E8-1F1ED"},
    {"bandera_cote_divoire", "1F1E8-1F1EE"},
    {"bandera_islas_cook", "1F1E8-1F1F0"},
    {"bandera_chile", "1F1E8-1F1F1"},
    {"bandera_camerun", "1F1E8-1F1F2"},
    {"bandera_china", "1F1E8-1F1F3"},
    {"bandera_colombia", "1F1E8-1F1F4"},
    {"bandera_isla_clipperton", "1F1E8-1F1F5"},
    {"bandera_sark", "1F1E8-1F1F6"},
    {"bandera_costa_rica", "1F1E8-1F1F7"},
    {"bandera_cuba", "1F1E8-1F1FA"},
    {"bandera_cabo_verde", "1F1E8-1F1FB"},
    {"bandera_curazao", "1F1E8-1F1FC"},
    {"bandera_isla_de_navidad", "1F1E8-1F1FD"},
    {"bandera_chipre", "1F1E8-1F1FE"},
    {"bandera_chequia", "1F1E8-1F1FF"},
    {"bandera_alemania", "1F1E9-1F1EA"},
    {"bandera_diego_garcia", "1F1E9-1F1EC"},
    {"bandera_yibuti", "1F1E9-1F1EF"},
    {"bandera_dinamarca", "1F1E9-1F1F0"},
    {"bandera_dominica", "1F1E9-1F1F2"},
    {"bandera_republica_dominicana", "1F1E9-1F1F4"},
    {"bandera_argelia", "1F1E9-1F1FF"},
    {"bandera_ceuta_y_melilla", "1F1EA-1F1E6"},
    {"bandera_ecuador", "1F1EA-1F1E8"},
    {"bandera_estonia", "1F1EA-1F1EA"},
    {"bandera_egipto", "1F1EA-1F1EC"},
    {"bandera_sahara_occidental", "1F1EA-1F1ED"},
    {"bandera_eritrea", "1F1EA-1F1F7"},
    {"bandera_espana", "1F1EA-1F1F8"},
    {"bandera_etiopia", "1F1EA-1F1F9"},
    {"bandera_union_europea", "1F1EA-1F1FA"},
    {"bandera_finlandia", "1F1EB-1F1EE"},
    {"bandera_fiyi", "1F1EB-1F1EF"},
    {"bandera_islas_malvinas", "1F1EB-1F1F0"},
    {"bandera_micronesia", "1F1EB-1F1F2"},
    {"bandera_islas_feroe", "1F1EB-1F1F4"},
    {"bandera_francia", "1F1EB-1F1F7"},
    {"bandera_gabon", "1F1EC-1F1E6"},
    {"bandera_reino_unido", "1F1EC-1F1E7"},
    {"bandera_granada", "1F1EC-1F1E9"},
    {"bandera_georgia", "1F1EC-1F1EA"},
    {"bandera_guayana_francesa", "1F1EC-1F1EB"},
    {"bandera_guernesey", "1F1EC-1F1EC"},
    {"bandera_ghana", "1F1EC-1F1ED"},
    {"bandera_gibraltar", "1F1EC-1F1EE"},
    {"bandera_groenlandia", "1F1EC-1F1F1"},
    {"bandera_gambia", "1F1EC-1F1F2"},
    {"bandera_guinea", "1F1EC-1F1F3"},
    {"bandera_guadalupe", "1F1EC-1F1F5"},
    {"bandera_guinea_ecuatorial", "1F1EC-1F1F6"},
    {"bandera_grecia", "1F1EC-1F1F7"},
    {"bandera_islas_georgia_del_sur_y_sandwich_del_sur", "1F1EC-1F1F8"},
    {"bandera_guatemala", "1F1EC-1F1F9"},
    {"bandera_guam", "1F1EC-1F1FA"},
    {"bandera_guinea_bisau", "1F1EC-1F1FC"},
    {"bandera_guyana", "1F1EC-1F1FE"},
    {"bandera_rae_de_hong_kong_china", "1F1ED-1F1F0"},
    {"bandera_islas_heard_y_mcdonald", "1F1ED-1F1F2"},
    {"bandera_honduras", "1F1ED-1F1F3"},
    {"bandera_croacia", "1F1ED-1F1F7"},
    {"bandera_haiti", "1F1ED-1F1F9"},
    {"bandera_hungria", "1F1ED-1F1FA"},
    {"bandera_canarias", "1F1EE-1F1E8"},
    {"bandera_indonesia", "1F1EE-1F1E9"},
    {"bandera_irlanda", "1F1EE-1F1EA"},
    {"bandera_israel", "1F1EE-1F1F1"},
    {"bandera_isla_de_man", "1F1EE-1F1F2"},
    {"bandera_india", "1F1EE-1F1F3"},
    {"bandera_territorio_britanico_del_oceano_indico", "1F1EE-1F1F4"},
    {"bandera_irak", "1F1EE-1F1F6"},
    {"bandera_iran", "1F1EE-1F1F7"},
    {"bandera_islandia", "1F1EE-1F1F8"},
    {"bandera_italia", "1F1EE-1F1F9"},
    {"bandera_jersey", "1F1EF-1F1EA"},
    {"bandera_jamaica", "1F1EF-1F1F2"},
    {"bandera_jordania", "1F1EF-1F1F4"},
    {"bandera_japon", "1F1EF-1F1F5"},
    {"bandera_kenia", "1F1F0-1F1EA"},
    {"bandera_kirguistan", "1F1F0-1F1EC"},
    {"bandera_camboya", "1F1F0-1F1ED"},
    {"bandera_kiribati", "1F1F0-1F1EE"},
    {"bandera_comoras", "1F1F0-1F1F2"},
    {"bandera_san_cristobal_y_nieves", "1F1F0-1F1F3"},
    {"bandera_corea_del_norte", "1F1F0-1F1F5"},
    {"bandera_corea_del_sur", "1F1F0-1F1F7"},
    {"bandera_kuwait", "1F1F0-1F1FC"},
    {"bandera_islas_caiman", "1F1F0-1F1FE"},
    {"bandera_kazajistan", "1F1F0-1F1FF"},
    {"bandera_laos", "1F1F1-1F1E6"},
    {"bandera_libano", "1F1F1-1F1E7"},
    {"bandera_santa_lucia", "1F1F1-1F1E8"},
    {"bandera_liechtenstein", "1F1F1-1F1EE"},
    {"bandera_sri_lanka", "1F1F1-1F1F0"},
    {"bandera_liberia", "1F1F1-1F1F7"},
    {"bandera_lesoto", "1F1F1-1F1F8"},
    {"bandera_lituania", "1F1F1-1F1F9"},
    {"bandera_luxemburgo", "1F1F1-1F1FA"},
    {"bandera_letonia", "1F1F1-1F1FB"},
    {"bandera_libia", "1F1F1-1F1FE"},
    {"bandera_marruecos", "1F1F2-1F1E6"},
    {"bandera_monaco", "1F1F2-1F1E8"},
    {"bandera_moldavia", "1F1F2-1F1E9"},
    {"bandera_montenegro", "1F1F2-1F1EA"},
    {"bandera_san_martin", "1F1F2-1F1EB"},
    {"bandera_madagascar", "1F1F2-1F1EC"},
    {"bandera_islas_marshall", "1F1F2-1F1ED"},
    {"bandera_macedonia_del_norte", "1F1F2-1F1F0"},
    {"bandera_mali", "1F1F2-1F1F1"},
    {"bandera_myanmar_birmania", "1F1F2-1F1F2"},
    {"bandera_mongolia", "1F1F2-1F1F3"},
    {"bandera_rae_de_macao_china", "1F1F2-1F1F4"},
    {"bandera_islas_marianas_del_norte", "1F1F2-1F1F5"},
    {"bandera_martinica", "1F1F2-1F1F6"},
    {"bandera_mauritania", "1F1F2-1F1F7"},
    {"bandera_montserrat", "1F1F2-1F1F8"},
    {"bandera_malta", "1F1F2-1F1F9"},
    {"bandera_mauricio", "1F1F2-1F1FA"},
    {"bandera_maldivas", "1F1F2-1F1FB"},
    {"bandera_malaui", "1F1F2-1F1FC"},
    {"bandera_mexico", "1F1F2-1F1FD"},
    {"bandera_malasia", "1F1F2-1F1FE"},
    {"bandera_mozambique", "1F1F2-1F1FF"},
    {"bandera_namibia", "1F1F3-1F1E6"},
    {"bandera_nueva_caledonia", "1F1F3-1F1E8"},
    {"bandera_niger", "1F1F3-1F1EA"},
    {"bandera_isla_norfolk", "1F1F3-1F1EB"},
    {"bandera_nigeria", "1F1F3-1F1EC"},
    {"bandera_nicaragua", "1F1F3-1F1EE"},
    {"bandera_paises_bajos", "1F1F3-1F1F1"},
    {"bandera_noruega", "1F1F3-1F1F4"},
    {"bandera_nepal", "1F1F3-1F1F5"},
    {"bandera_nauru", "1F1F3-1F1F7"},
    {"bandera_niue", "1F1F3-1F1FA"},
    {"bandera_nueva_zelanda", "1F1F3-1F1FF"},
    {"bandera_oman", "1F1F4-1F1F2"},
    {"bandera_panama", "1F1F5-1F1E6"},
    {"bandera_peru", "1F1F5-1F1EA"},
    {"bandera_polinesia_francesa", "1F1F5-1F1EB"},
    {"bandera_papua_nueva_guinea", "1F1F5-1F1EC"},
    {"bandera_filipinas", "1F1F5-1F1ED"},
    {"bandera_pakistan", "1F1F5-1F1F0"},
    {"bandera_polonia", "1F1F5-1F1F1"},
    {"bandera_san_pedro_y_miquelon", "1F1F5-1F1F2"},
    {"bandera_islas_pitcairn", "1F1F5-1F1F3"},
    {"bandera_puerto_rico", "1F1F5-1F1F7"},
    {"bandera_territorios_palestinos", "1F1F5-1F1F8"},
    {"bandera_portugal", "1F1F5-1F1F9"},
    {"bandera_palaos", "1F1F5-1F1FC"},
    {"bandera_paraguay", "1F1F5-1F1FE"},
    {"bandera_catar", "1F1F6-1F1E6"},
    {"bandera_reunion", "1F1F7-1F1EA"},
    {"bandera_rumania", "1F1F7-1F1F4"},
    {"bandera_serbia", "1F1F7-1F1F8"},
    {"bandera_rusia", "1F1F7-1F1FA"},
    {"bandera_ruanda", "1F1F7-1F1FC"},
    {"bandera_arabia_saudi", "1F1F8-1F1E6"},
    {"bandera_islas_salomon", "1F1F8-1F1E7"},
    {"bandera_seychelles", "1F1F8-1F1E8"},
    {"bandera_sudan", "1F1F8-1F1E9"},
    {"bandera_suecia", "1F1F8-1F1EA"},
    {"bandera_singapur", "1F1F8-1F1EC"},
    {"bandera_santa_elena", "1F1F8-1F1ED"},
    {"bandera_eslovenia", "1F1F8-1F1EE"},
    {"bandera_svalbard_y_jan_mayen", "1F1F8-1F1EF"},
    {"bandera_eslovaquia", "1F1F8-1F1F0"},
    {"bandera_sierra_leona", "1F1F8-1F1F1"},
    {"bandera_san_marino", "1F1F8-1F1F2"},
    {"bandera_senegal", "1F1F8-1F1F3"},
    {"bandera_somalia", "1F1F8-1F1F4"},
    {"bandera_surinam", "1F1F8-1F1F7"},
    {"bandera_sudan_del_sur", "1F1F8-1F1F8"},
    {"bandera_santo_tome_y_principe", "1F1F8-1F1F9"},
    {"bandera_el_salvador", "1F1F8-1F1FB"},
    {"bandera_sint_maarten", "1F1F8-1F1FD"},
    {"bandera_siria", "1F1F8-1F1FE"},
    {"bandera_esuatini", "1F1F8-1F1FF"},
    {"bandera_tristan_de_acuna", "1F1F9-1F1E6"},
    {"bandera_islas_turcas_y_caicos", "1F1F9-1F1E8"},
    {"bandera_chad", "1F1F9-1F1E9"},
    {"bandera_territorios_australes_franceses", "1F1F9-1F1EB"},
    {"bandera_togo", "1F1F9-1F1EC"},
    {"bandera_tailandia", "1F1F9-1F1ED"},
    {"bandera_tayikistan", "1F1F9-1F1EF"},
    {"bandera_tokelau", "1F1F9-1F1F0"},
    {"bandera_timor_leste", "1F1F9-1F1F1"},
    {"bandera_turkmenistan", "1F1F9-1F1F2"},
    {"bandera_tunez", "1F1F9-1F1F3"},
    {"bandera_tonga", "1F1F9-1F1F4"},
    {"bandera_turquia", "1F1F9-1F1F7"},
    {"bandera_trinidad_y_tobago", "1F1F9-1F1F9"},
    {"bandera_tuvalu", "1F1F9-1F1FB"},
    {"bandera_taiwan", "1F1F9-1F1FC"},
    {"bandera_tanzania", "1F1F9-1F1FF"},
    {"bandera_ucrania", "1F1FA-1F1E6"},
    {"bandera_uganda", "1F1FA-1F1EC"},
    {"bandera_islas_menores_alejadas_de_ee_uu", "1F1FA-1F1F2"},
    {"bandera_naciones_unidas", "1F1FA-1F1F3"},
    {"bandera_estados_unidos", "1F1FA-1F1F8"},
    {"bandera_uruguay", "1F1FA-1F1FE"},
    {"bandera_uzbekistan", "1F1FA-1F1FF"},
    {"bandera_ciudad_del_vaticano", "1F1FB-1F1E6"},
    {"bandera_san_vicente_y_las_granadinas", "1F1FB-1F1E8"},
    {"bandera_venezuela", "1F1FB-1F1EA"},
    {"bandera_islas_virgenes_britanicas", "1F1FB-1F1EC"},
    {"bandera_islas_virgenes_de_ee_uu", "1F1FB-1F1EE"},
    {"bandera_vietnam", "1F1FB-1F1F3"},
    {"bandera_vanuatu", "1F1FB-1F1FA"},
    {"bandera_wallis_y_futuna", "1F1FC-1F1EB"},
    {"bandera_samoa", "1F1FC-1F1F8"},
    {"bandera_kosovo", "1F1FD-1F1F0"},
    {"bandera_yemen", "1F1FE-1F1EA"},
    {"bandera_mayotte", "1F1FE-1F1F9"},
    {"bandera_sudafrica", "1F1FF-1F1E6"},
    {"bandera_zambia", "1F1FF-1F1F2"},
    {"bandera_zimbabue", "1F1FF-1F1FC"},
    {"bandera_inglaterra", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"bandera_escocia", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"bandera_gales", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"pareja_enamorada_hombre_y_hombre", "1F468-200D-2764-FE0F-200D-1F468"},
    {"pareja_enamorada_hombre_y_hombre_color1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pareja_enamorada_hombre_y_hombre_color1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pareja_enamorada_hombre_y_hombre_color1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pareja_enamorada_hombre_y_hombre_color1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pareja_enamorada_hombre_y_hombre_color1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pareja_enamorada_hombre_y_hombre_color2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pareja_enamorada_hombre_y_hombre_color2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pareja_enamorada_hombre_y_hombre_color2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pareja_enamorada_hombre_y_hombre_color2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pareja_enamorada_hombre_y_hombre_color2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pareja_enamorada_hombre_y_hombre_color3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pareja_enamorada_hombre_y_hombre_color3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pareja_enamorada_hombre_y_hombre_color3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pareja_enamorada_hombre_y_hombre_color3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pareja_enamorada_hombre_y_hombre_color3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pareja_enamorada_hombre_y_hombre_color4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pareja_enamorada_hombre_y_hombre_color4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pareja_enamorada_hombre_y_hombre_color4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pareja_enamorada_hombre_y_hombre_color4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pareja_enamorada_hombre_y_hombre_color4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pareja_enamorada_hombre_y_hombre_color5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pareja_enamorada_hombre_y_hombre_color5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pareja_enamorada_hombre_y_hombre_color5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pareja_enamorada_hombre_y_hombre_color5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pareja_enamorada_hombre_y_hombre_color5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"beso_hombre_y_hombre", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"beso_hombre_y_hombre_color1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beso_hombre_y_hombre_color1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beso_hombre_y_hombre_color1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beso_hombre_y_hombre_color1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beso_hombre_y_hombre_color1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beso_hombre_y_hombre_color2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beso_hombre_y_hombre_color2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beso_hombre_y_hombre_color2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beso_hombre_y_hombre_color2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beso_hombre_y_hombre_color2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beso_hombre_y_hombre_color3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beso_hombre_y_hombre_color3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beso_hombre_y_hombre_color3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beso_hombre_y_hombre_color3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beso_hombre_y_hombre_color3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beso_hombre_y_hombre_color4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beso_hombre_y_hombre_color4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beso_hombre_y_hombre_color4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beso_hombre_y_hombre_color4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beso_hombre_y_hombre_color4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beso_hombre_y_hombre_color5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beso_hombre_y_hombre_color5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beso_hombre_y_hombre_color5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beso_hombre_y_hombre_color5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beso_hombre_y_hombre_color5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"familia_hombre_y_nino", "1F468-200D-1F466"},
    {"familia_hombre_nino_nino", "1F468-200D-1F466-200D-1F466"},
    {"familia_hombre_y_nina", "1F468-200D-1F467"},
    {"familia_hombre_nina_nino", "1F468-200D-1F467-200D-1F466"},
    {"familia_hombre_nina_nina", "1F468-200D-1F467-200D-1F467"},
    {"familia_hombre_hombre_nino", "1F468-200D-1F468-200D-1F466"},
    {"familia_hombre_hombre_nino_nino", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"familia_hombre_hombre_nina", "1F468-200D-1F468-200D-1F467"},
    {"familia_hombre_hombre_nina_nino", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"familia_hombre_hombre_nina_nina", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"familia_hombre_mujer_nino", "1F468-200D-1F469-200D-1F466"},
    {"familia_hombre_mujer_nino_nino", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"familia_hombre_mujer_nina", "1F468-200D-1F469-200D-1F467"},
    {"familia_hombre_mujer_nina_nino", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"familia_hombre_mujer_nina_nina", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"pareja_enamorada_mujer_y_hombre", "1F469-200D-2764-FE0F-200D-1F468"},
    {"pareja_enamorada_mujer_y_hombre_color1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pareja_enamorada_mujer_y_hombre_color1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pareja_enamorada_mujer_y_hombre_color1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pareja_enamorada_mujer_y_hombre_color1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pareja_enamorada_mujer_y_hombre_color1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pareja_enamorada_mujer_y_hombre_color2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pareja_enamorada_mujer_y_hombre_color2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pareja_enamorada_mujer_y_hombre_color2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pareja_enamorada_mujer_y_hombre_color2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pareja_enamorada_mujer_y_hombre_color2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pareja_enamorada_mujer_y_hombre_color3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pareja_enamorada_mujer_y_hombre_color3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pareja_enamorada_mujer_y_hombre_color3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pareja_enamorada_mujer_y_hombre_color3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pareja_enamorada_mujer_y_hombre_color3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pareja_enamorada_mujer_y_hombre_color4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pareja_enamorada_mujer_y_hombre_color4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pareja_enamorada_mujer_y_hombre_color4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pareja_enamorada_mujer_y_hombre_color4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pareja_enamorada_mujer_y_hombre_color4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pareja_enamorada_mujer_y_hombre_color5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pareja_enamorada_mujer_y_hombre_color5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pareja_enamorada_mujer_y_hombre_color5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pareja_enamorada_mujer_y_hombre_color5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pareja_enamorada_mujer_y_hombre_color5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pareja_enamorada_mujer_y_mujer", "1F469-200D-2764-FE0F-200D-1F469"},
    {"pareja_enamorada_mujer_y_mujer_color1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pareja_enamorada_mujer_y_mujer_color1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pareja_enamorada_mujer_y_mujer_color1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pareja_enamorada_mujer_y_mujer_color1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pareja_enamorada_mujer_y_mujer_color1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pareja_enamorada_mujer_y_mujer_color2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pareja_enamorada_mujer_y_mujer_color2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pareja_enamorada_mujer_y_mujer_color2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pareja_enamorada_mujer_y_mujer_color2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pareja_enamorada_mujer_y_mujer_color2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pareja_enamorada_mujer_y_mujer_color3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pareja_enamorada_mujer_y_mujer_color3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pareja_enamorada_mujer_y_mujer_color3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pareja_enamorada_mujer_y_mujer_color3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pareja_enamorada_mujer_y_mujer_color3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pareja_enamorada_mujer_y_mujer_color4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pareja_enamorada_mujer_y_mujer_color4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pareja_enamorada_mujer_y_mujer_color4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pareja_enamorada_mujer_y_mujer_color4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pareja_enamorada_mujer_y_mujer_color4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pareja_enamorada_mujer_y_mujer_color5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pareja_enamorada_mujer_y_mujer_color5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pareja_enamorada_mujer_y_mujer_color5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pareja_enamorada_mujer_y_mujer_color5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pareja_enamorada_mujer_y_mujer_color5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"beso_mujer_y_hombre", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"beso_mujer_y_hombre_color1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beso_mujer_y_hombre_color1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beso_mujer_y_hombre_color1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beso_mujer_y_hombre_color1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beso_mujer_y_hombre_color1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beso_mujer_y_hombre_color2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beso_mujer_y_hombre_color2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beso_mujer_y_hombre_color2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beso_mujer_y_hombre_color2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beso_mujer_y_hombre_color2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beso_mujer_y_hombre_color3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beso_mujer_y_hombre_color3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beso_mujer_y_hombre_color3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beso_mujer_y_hombre_color3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beso_mujer_y_hombre_color3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beso_mujer_y_hombre_color4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beso_mujer_y_hombre_color4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beso_mujer_y_hombre_color4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beso_mujer_y_hombre_color4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beso_mujer_y_hombre_color4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beso_mujer_y_hombre_color5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"beso_mujer_y_hombre_color5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"beso_mujer_y_hombre_color5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"beso_mujer_y_hombre_color5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"beso_mujer_y_hombre_color5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"beso_mujer_y_mujer", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"beso_mujer_y_mujer_color1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"beso_mujer_y_mujer_color1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"beso_mujer_y_mujer_color1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"beso_mujer_y_mujer_color1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"beso_mujer_y_mujer_color1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"beso_mujer_y_mujer_color2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"beso_mujer_y_mujer_color2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"beso_mujer_y_mujer_color2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"beso_mujer_y_mujer_color2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"beso_mujer_y_mujer_color2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"beso_mujer_y_mujer_color3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"beso_mujer_y_mujer_color3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"beso_mujer_y_mujer_color3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"beso_mujer_y_mujer_color3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"beso_mujer_y_mujer_color3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"beso_mujer_y_mujer_color4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"beso_mujer_y_mujer_color4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"beso_mujer_y_mujer_color4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"beso_mujer_y_mujer_color4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"beso_mujer_y_mujer_color4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"beso_mujer_y_mujer_color5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"beso_mujer_y_mujer_color5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"beso_mujer_y_mujer_color5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"beso_mujer_y_mujer_color5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"beso_mujer_y_mujer_color5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"familia_mujer_y_nino", "1F469-200D-1F466"},
    {"familia_mujer_nino_nino", "1F469-200D-1F466-200D-1F466"},
    {"familia_mujer_y_nina", "1F469-200D-1F467"},
    {"familia_mujer_nina_nino", "1F469-200D-1F467-200D-1F466"},
    {"familia_mujer_nina_nina", "1F469-200D-1F467-200D-1F467"},
    {"familia_mujer_mujer_nino", "1F469-200D-1F469-200D-1F466"},
    {"familia_mujer_mujer_nino_nino", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"familia_mujer_mujer_nina", "1F469-200D-1F469-200D-1F467"},
    {"familia_mujer_mujer_nina_nino", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"familia_mujer_mujer_nina_nina", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"dos_personas_de_la_mano", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"dos_personas_de_la_mano_color1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"dos_personas_de_la_mano_color1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"dos_personas_de_la_mano_color1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"dos_personas_de_la_mano_color1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"dos_personas_de_la_mano_color1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"dos_personas_de_la_mano_color2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"dos_personas_de_la_mano_color2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"dos_personas_de_la_mano_color2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"dos_personas_de_la_mano_color2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"dos_personas_de_la_mano_color2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"dos_personas_de_la_mano_color3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"dos_personas_de_la_mano_color3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"dos_personas_de_la_mano_color3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"dos_personas_de_la_mano_color3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"dos_personas_de_la_mano_color3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"dos_personas_de_la_mano_color4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"dos_personas_de_la_mano_color4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"dos_personas_de_la_mano_color4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"dos_personas_de_la_mano_color4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"dos_personas_de_la_mano_color4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"dos_personas_de_la_mano_color5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"dos_personas_de_la_mano_color5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"dos_personas_de_la_mano_color5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"dos_personas_de_la_mano_color5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"dos_personas_de_la_mano_color5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"familia_adulto_adulto_nino", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"familia_adulto_adulto_nino_nino", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"familia_adulto_nino", "1F9D1-200D-1F9D2"},
    {"familia_adulto_nino_nino", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"persona_corriendo_hacia_la_derecha", "1F3C3-200D-27A1-FE0F"},
    {"persona_corriendo_hacia_la_derecha_color1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"persona_corriendo_hacia_la_derecha_color2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"persona_corriendo_hacia_la_derecha_color3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"persona_corriendo_hacia_la_derecha_color4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"persona_corriendo_hacia_la_derecha_color5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"profesional_sanitario_hombre", "1F468-200D-2695-FE0F"},
    {"profesional_sanitario_hombre_color1", "1F468-1F3FB-200D-2695-FE0F"},
    {"profesional_sanitario_hombre_color2", "1F468-1F3FC-200D-2695-FE0F"},
    {"profesional_sanitario_hombre_color3", "1F468-1F3FD-200D-2695-FE0F"},
    {"profesional_sanitario_hombre_color4", "1F468-1F3FE-200D-2695-FE0F"},
    {"profesional_sanitario_hombre_color5", "1F468-1F3FF-200D-2695-FE0F"},
    {"fiscal_hombre", "1F468-200D-2696-FE0F"},
    {"fiscal_hombre_color1", "1F468-1F3FB-200D-2696-FE0F"},
    {"fiscal_hombre_color2", "1F468-1F3FC-200D-2696-FE0F"},
    {"fiscal_hombre_color3", "1F468-1F3FD-200D-2696-FE0F"},
    {"fiscal_hombre_color4", "1F468-1F3FE-200D-2696-FE0F"},
    {"fiscal_hombre_color5", "1F468-1F3FF-200D-2696-FE0F"},
    {"piloto_hombre", "1F468-200D-2708-FE0F"},
    {"piloto_hombre_color1", "1F468-1F3FB-200D-2708-FE0F"},
    {"piloto_hombre_color2", "1F468-1F3FC-200D-2708-FE0F"},
    {"piloto_hombre_color3", "1F468-1F3FD-200D-2708-FE0F"},
    {"piloto_hombre_color4", "1F468-1F3FE-200D-2708-FE0F"},
    {"piloto_hombre_color5", "1F468-1F3FF-200D-2708-FE0F"},
    {"profesional_de_la_agricultura_hombre", "1F468-200D-1F33E"},
    {"profesional_de_la_agricultura_hombre_color1", "1F468-1F3FB-200D-1F33E"},
    {"profesional_de_la_agricultura_hombre_color2", "1F468-1F3FC-200D-1F33E"},
    {"profesional_de_la_agricultura_hombre_color3", "1F468-1F3FD-200D-1F33E"},
    {"profesional_de_la_agricultura_hombre_color4", "1F468-1F3FE-200D-1F33E"},
    {"profesional_de_la_agricultura_hombre_color5", "1F468-1F3FF-200D-1F33E"},
    {"chef_hombre", "1F468-200D-1F373"},
    {"chef_hombre_color1", "1F468-1F3FB-200D-1F373"},
    {"chef_hombre_color2", "1F468-1F3FC-200D-1F373"},
    {"chef_hombre_color3", "1F468-1F3FD-200D-1F373"},
    {"chef_hombre_color4", "1F468-1F3FE-200D-1F373"},
    {"chef_hombre_color5", "1F468-1F3FF-200D-1F373"},
    {"hombre_alimentando_a_bebe", "1F468-200D-1F37C"},
    {"hombre_alimentando_a_bebe_color1", "1F468-1F3FB-200D-1F37C"},
    {"hombre_alimentando_a_bebe_color2", "1F468-1F3FC-200D-1F37C"},
    {"hombre_alimentando_a_bebe_color3", "1F468-1F3FD-200D-1F37C"},
    {"hombre_alimentando_a_bebe_color4", "1F468-1F3FE-200D-1F37C"},
    {"hombre_alimentando_a_bebe_color5", "1F468-1F3FF-200D-1F37C"},
    {"estudiante_hombre", "1F468-200D-1F393"},
    {"estudiante_hombre_color1", "1F468-1F3FB-200D-1F393"},
    {"estudiante_hombre_color2", "1F468-1F3FC-200D-1F393"},
    {"estudiante_hombre_color3", "1F468-1F3FD-200D-1F393"},
    {"estudiante_hombre_color4", "1F468-1F3FE-200D-1F393"},
    {"estudiante_hombre_color5", "1F468-1F3FF-200D-1F393"},
    {"cantante_hombre", "1F468-200D-1F3A4"},
    {"cantante_hombre_color1", "1F468-1F3FB-200D-1F3A4"},
    {"cantante_hombre_color2", "1F468-1F3FC-200D-1F3A4"},
    {"cantante_hombre_color3", "1F468-1F3FD-200D-1F3A4"},
    {"cantante_hombre_color4", "1F468-1F3FE-200D-1F3A4"},
    {"cantante_hombre_color5", "1F468-1F3FF-200D-1F3A4"},
    {"artista_hombre", "1F468-200D-1F3A8"},
    {"artista_hombre_color1", "1F468-1F3FB-200D-1F3A8"},
    {"artista_hombre_color2", "1F468-1F3FC-200D-1F3A8"},
    {"artista_hombre_color3", "1F468-1F3FD-200D-1F3A8"},
    {"artista_hombre_color4", "1F468-1F3FE-200D-1F3A8"},
    {"artista_hombre_color5", "1F468-1F3FF-200D-1F3A8"},
    {"docente_hombre", "1F468-200D-1F3EB"},
    {"docente_hombre_color1", "1F468-1F3FB-200D-1F3EB"},
    {"docente_hombre_color2", "1F468-1F3FC-200D-1F3EB"},
    {"docente_hombre_color3", "1F468-1F3FD-200D-1F3EB"},
    {"docente_hombre_color4", "1F468-1F3FE-200D-1F3EB"},
    {"docente_hombre_color5", "1F468-1F3FF-200D-1F3EB"},
    {"profesional_industrial_hombre", "1F468-200D-1F3ED"},
    {"profesional_industrial_hombre_color1", "1F468-1F3FB-200D-1F3ED"},
    {"profesional_industrial_hombre_color2", "1F468-1F3FC-200D-1F3ED"},
    {"profesional_industrial_hombre_color3", "1F468-1F3FD-200D-1F3ED"},
    {"profesional_industrial_hombre_color4", "1F468-1F3FE-200D-1F3ED"},
    {"profesional_industrial_hombre_color5", "1F468-1F3FF-200D-1F3ED"},
    {"profesional_de_la_tecnologia_hombre", "1F468-200D-1F4BB"},
    {"profesional_de_la_tecnologia_hombre_color1", "1F468-1F3FB-200D-1F4BB"},
    {"profesional_de_la_tecnologia_hombre_color2", "1F468-1F3FC-200D-1F4BB"},
    {"profesional_de_la_tecnologia_hombre_color3", "1F468-1F3FD-200D-1F4BB"},
    {"profesional_de_la_tecnologia_hombre_color4", "1F468-1F3FE-200D-1F4BB"},
    {"profesional_de_la_tecnologia_hombre_color5", "1F468-1F3FF-200D-1F4BB"},
    {"oficinista_hombre", "1F468-200D-1F4BC"},
    {"oficinista_hombre_color1", "1F468-1F3FB-200D-1F4BC"},
    {"oficinista_hombre_color2", "1F468-1F3FC-200D-1F4BC"},
    {"oficinista_hombre_color3", "1F468-1F3FD-200D-1F4BC"},
    {"oficinista_hombre_color4", "1F468-1F3FE-200D-1F4BC"},
    {"oficinista_hombre_color5", "1F468-1F3FF-200D-1F4BC"},
    {"profesional_de_la_mecanica_hombre", "1F468-200D-1F527"},
    {"profesional_de_la_mecanica_hombre_color1", "1F468-1F3FB-200D-1F527"},
    {"profesional_de_la_mecanica_hombre_color2", "1F468-1F3FC-200D-1F527"},
    {"profesional_de_la_mecanica_hombre_color3", "1F468-1F3FD-200D-1F527"},
    {"profesional_de_la_mecanica_hombre_color4", "1F468-1F3FE-200D-1F527"},
    {"profesional_de_la_mecanica_hombre_color5", "1F468-1F3FF-200D-1F527"},
    {"profesional_de_la_ciencia_hombre", "1F468-200D-1F52C"},
    {"profesional_de_la_ciencia_hombre_color1", "1F468-1F3FB-200D-1F52C"},
    {"profesional_de_la_ciencia_hombre_color2", "1F468-1F3FC-200D-1F52C"},
    {"profesional_de_la_ciencia_hombre_color3", "1F468-1F3FD-200D-1F52C"},
    {"profesional_de_la_ciencia_hombre_color4", "1F468-1F3FE-200D-1F52C"},
    {"profesional_de_la_ciencia_hombre_color5", "1F468-1F3FF-200D-1F52C"},
    {"astronauta_hombre", "1F468-200D-1F680"},
    {"astronauta_hombre_color1", "1F468-1F3FB-200D-1F680"},
    {"astronauta_hombre_color2", "1F468-1F3FC-200D-1F680"},
    {"astronauta_hombre_color3", "1F468-1F3FD-200D-1F680"},
    {"astronauta_hombre_color4", "1F468-1F3FE-200D-1F680"},
    {"astronauta_hombre_color5", "1F468-1F3FF-200D-1F680"},
    {"bombero_hombre", "1F468-200D-1F692"},
    {"bombero_hombre_color1", "1F468-1F3FB-200D-1F692"},
    {"bombero_hombre_color2", "1F468-1F3FC-200D-1F692"},
    {"bombero_hombre_color3", "1F468-1F3FD-200D-1F692"},
    {"bombero_hombre_color4", "1F468-1F3FE-200D-1F692"},
    {"bombero_hombre_color5", "1F468-1F3FF-200D-1F692"},
    {"hombre_con_baston", "1F468-200D-1F9AF"},
    {"hombre_con_baston_color1", "1F468-1F3FB-200D-1F9AF"},
    {"hombre_con_baston_color2", "1F468-1F3FC-200D-1F9AF"},
    {"hombre_con_baston_color3", "1F468-1F3FD-200D-1F9AF"},
    {"hombre_con_baston_color4", "1F468-1F3FE-200D-1F9AF"},
    {"hombre_con_baston_color5", "1F468-1F3FF-200D-1F9AF"},
    {"hombre_con_baston_hacia_la_derecha", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"hombre_con_baston_hacia_la_derecha_color1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"hombre_con_baston_hacia_la_derecha_color2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"hombre_con_baston_hacia_la_derecha_color3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"hombre_con_baston_hacia_la_derecha_color4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"hombre_con_baston_hacia_la_derecha_color5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_electrica", "1F468-200D-1F9BC"},
    {"hombre_en_silla_de_ruedas_electrica_color1", "1F468-1F3FB-200D-1F9BC"},
    {"hombre_en_silla_de_ruedas_electrica_color2", "1F468-1F3FC-200D-1F9BC"},
    {"hombre_en_silla_de_ruedas_electrica_color3", "1F468-1F3FD-200D-1F9BC"},
    {"hombre_en_silla_de_ruedas_electrica_color4", "1F468-1F3FE-200D-1F9BC"},
    {"hombre_en_silla_de_ruedas_electrica_color5", "1F468-1F3FF-200D-1F9BC"},
    {"hombre_en_silla_de_ruedas_electrica_hacia_la_derecha", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_electrica_hacia_la_derecha_color1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_electrica_hacia_la_derecha_color2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_electrica_hacia_la_derecha_color3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_electrica_hacia_la_derecha_color4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_electrica_hacia_la_derecha_color5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_manual", "1F468-200D-1F9BD"},
    {"hombre_en_silla_de_ruedas_manual_color1", "1F468-1F3FB-200D-1F9BD"},
    {"hombre_en_silla_de_ruedas_manual_color2", "1F468-1F3FC-200D-1F9BD"},
    {"hombre_en_silla_de_ruedas_manual_color3", "1F468-1F3FD-200D-1F9BD"},
    {"hombre_en_silla_de_ruedas_manual_color4", "1F468-1F3FE-200D-1F9BD"},
    {"hombre_en_silla_de_ruedas_manual_color5", "1F468-1F3FF-200D-1F9BD"},
    {"hombre_en_silla_de_ruedas_manual_hacia_la_derecha", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_manual_hacia_la_derecha_color1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_manual_hacia_la_derecha_color2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_manual_hacia_la_derecha_color3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_manual_hacia_la_derecha_color4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_manual_hacia_la_derecha_color5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"profesional_sanitario_mujer", "1F469-200D-2695-FE0F"},
    {"profesional_sanitario_mujer_color1", "1F469-1F3FB-200D-2695-FE0F"},
    {"profesional_sanitario_mujer_color2", "1F469-1F3FC-200D-2695-FE0F"},
    {"profesional_sanitario_mujer_color3", "1F469-1F3FD-200D-2695-FE0F"},
    {"profesional_sanitario_mujer_color4", "1F469-1F3FE-200D-2695-FE0F"},
    {"profesional_sanitario_mujer_color5", "1F469-1F3FF-200D-2695-FE0F"},
    {"fiscal_mujer", "1F469-200D-2696-FE0F"},
    {"fiscal_mujer_color1", "1F469-1F3FB-200D-2696-FE0F"},
    {"fiscal_mujer_color2", "1F469-1F3FC-200D-2696-FE0F"},
    {"fiscal_mujer_color3", "1F469-1F3FD-200D-2696-FE0F"},
    {"fiscal_mujer_color4", "1F469-1F3FE-200D-2696-FE0F"},
    {"fiscal_mujer_color5", "1F469-1F3FF-200D-2696-FE0F"},
    {"piloto_mujer", "1F469-200D-2708-FE0F"},
    {"piloto_mujer_color1", "1F469-1F3FB-200D-2708-FE0F"},
    {"piloto_mujer_color2", "1F469-1F3FC-200D-2708-FE0F"},
    {"piloto_mujer_color3", "1F469-1F3FD-200D-2708-FE0F"},
    {"piloto_mujer_color4", "1F469-1F3FE-200D-2708-FE0F"},
    {"piloto_mujer_color5", "1F469-1F3FF-200D-2708-FE0F"},
    {"profesional_de_la_agricultura_mujer", "1F469-200D-1F33E"},
    {"profesional_de_la_agricultura_mujer_color1", "1F469-1F3FB-200D-1F33E"},
    {"profesional_de_la_agricultura_mujer_color2", "1F469-1F3FC-200D-1F33E"},
    {"profesional_de_la_agricultura_mujer_color3", "1F469-1F3FD-200D-1F33E"},
    {"profesional_de_la_agricultura_mujer_color4", "1F469-1F3FE-200D-1F33E"},
    {"profesional_de_la_agricultura_mujer_color5", "1F469-1F3FF-200D-1F33E"},
    {"chef_mujer", "1F469-200D-1F373"},
    {"chef_mujer_color1", "1F469-1F3FB-200D-1F373"},
    {"chef_mujer_color2", "1F469-1F3FC-200D-1F373"},
    {"chef_mujer_color3", "1F469-1F3FD-200D-1F373"},
    {"chef_mujer_color4", "1F469-1F3FE-200D-1F373"},
    {"chef_mujer_color5", "1F469-1F3FF-200D-1F373"},
    {"mujer_alimentando_a_bebe", "1F469-200D-1F37C"},
    {"mujer_alimentando_a_bebe_color1", "1F469-1F3FB-200D-1F37C"},
    {"mujer_alimentando_a_bebe_color2", "1F469-1F3FC-200D-1F37C"},
    {"mujer_alimentando_a_bebe_color3", "1F469-1F3FD-200D-1F37C"},
    {"mujer_alimentando_a_bebe_color4", "1F469-1F3FE-200D-1F37C"},
    {"mujer_alimentando_a_bebe_color5", "1F469-1F3FF-200D-1F37C"},
    {"estudiante_mujer", "1F469-200D-1F393"},
    {"estudiante_mujer_color1", "1F469-1F3FB-200D-1F393"},
    {"estudiante_mujer_color2", "1F469-1F3FC-200D-1F393"},
    {"estudiante_mujer_color3", "1F469-1F3FD-200D-1F393"},
    {"estudiante_mujer_color4", "1F469-1F3FE-200D-1F393"},
    {"estudiante_mujer_color5", "1F469-1F3FF-200D-1F393"},
    {"cantante_mujer", "1F469-200D-1F3A4"},
    {"cantante_mujer_color1", "1F469-1F3FB-200D-1F3A4"},
    {"cantante_mujer_color2", "1F469-1F3FC-200D-1F3A4"},
    {"cantante_mujer_color3", "1F469-1F3FD-200D-1F3A4"},
    {"cantante_mujer_color4", "1F469-1F3FE-200D-1F3A4"},
    {"cantante_mujer_color5", "1F469-1F3FF-200D-1F3A4"},
    {"artista_mujer", "1F469-200D-1F3A8"},
    {"artista_mujer_color1", "1F469-1F3FB-200D-1F3A8"},
    {"artista_mujer_color2", "1F469-1F3FC-200D-1F3A8"},
    {"artista_mujer_color3", "1F469-1F3FD-200D-1F3A8"},
    {"artista_mujer_color4", "1F469-1F3FE-200D-1F3A8"},
    {"artista_mujer_color5", "1F469-1F3FF-200D-1F3A8"},
    {"docente_mujer", "1F469-200D-1F3EB"},
    {"docente_mujer_color1", "1F469-1F3FB-200D-1F3EB"},
    {"docente_mujer_color2", "1F469-1F3FC-200D-1F3EB"},
    {"docente_mujer_color3", "1F469-1F3FD-200D-1F3EB"},
    {"docente_mujer_color4", "1F469-1F3FE-200D-1F3EB"},
    {"docente_mujer_color5", "1F469-1F3FF-200D-1F3EB"},
    {"profesional_industrial_mujer", "1F469-200D-1F3ED"},
    {"profesional_industrial_mujer_color1", "1F469-1F3FB-200D-1F3ED"},
    {"profesional_industrial_mujer_color2", "1F469-1F3FC-200D-1F3ED"},
    {"profesional_industrial_mujer_color3", "1F469-1F3FD-200D-1F3ED"},
    {"profesional_industrial_mujer_color4", "1F469-1F3FE-200D-1F3ED"},
    {"profesional_industrial_mujer_color5", "1F469-1F3FF-200D-1F3ED"},
    {"profesional_de_la_tecnologia_mujer", "1F469-200D-1F4BB"},
    {"profesional_de_la_tecnologia_mujer_color1", "1F469-1F3FB-200D-1F4BB"},
    {"profesional_de_la_tecnologia_mujer_color2", "1F469-1F3FC-200D-1F4BB"},
    {"profesional_de_la_tecnologia_mujer_color3", "1F469-1F3FD-200D-1F4BB"},
    {"profesional_de_la_tecnologia_mujer_color4", "1F469-1F3FE-200D-1F4BB"},
    {"profesional_de_la_tecnologia_mujer_color5", "1F469-1F3FF-200D-1F4BB"},
    {"oficinista_mujer", "1F469-200D-1F4BC"},
    {"oficinista_mujer_color1", "1F469-1F3FB-200D-1F4BC"},
    {"oficinista_mujer_color2", "1F469-1F3FC-200D-1F4BC"},
    {"oficinista_mujer_color3", "1F469-1F3FD-200D-1F4BC"},
    {"oficinista_mujer_color4", "1F469-1F3FE-200D-1F4BC"},
    {"oficinista_mujer_color5", "1F469-1F3FF-200D-1F4BC"},
    {"profesional_de_la_mecanica_mujer", "1F469-200D-1F527"},
    {"profesional_de_la_mecanica_mujer_color1", "1F469-1F3FB-200D-1F527"},
    {"profesional_de_la_mecanica_mujer_color2", "1F469-1F3FC-200D-1F527"},
    {"profesional_de_la_mecanica_mujer_color3", "1F469-1F3FD-200D-1F527"},
    {"profesional_de_la_mecanica_mujer_color4", "1F469-1F3FE-200D-1F527"},
    {"profesional_de_la_mecanica_mujer_color5", "1F469-1F3FF-200D-1F527"},
    {"profesional_de_la_ciencia_mujer", "1F469-200D-1F52C"},
    {"profesional_de_la_ciencia_mujer_color1", "1F469-1F3FB-200D-1F52C"},
    {"profesional_de_la_ciencia_mujer_color2", "1F469-1F3FC-200D-1F52C"},
    {"profesional_de_la_ciencia_mujer_color3", "1F469-1F3FD-200D-1F52C"},
    {"profesional_de_la_ciencia_mujer_color4", "1F469-1F3FE-200D-1F52C"},
    {"profesional_de_la_ciencia_mujer_color5", "1F469-1F3FF-200D-1F52C"},
    {"astronauta_mujer", "1F469-200D-1F680"},
    {"astronauta_mujer_color1", "1F469-1F3FB-200D-1F680"},
    {"astronauta_mujer_color2", "1F469-1F3FC-200D-1F680"},
    {"astronauta_mujer_color3", "1F469-1F3FD-200D-1F680"},
    {"astronauta_mujer_color4", "1F469-1F3FE-200D-1F680"},
    {"astronauta_mujer_color5", "1F469-1F3FF-200D-1F680"},
    {"bombera", "1F469-200D-1F692"},
    {"bombera_color1", "1F469-1F3FB-200D-1F692"},
    {"bombera_color2", "1F469-1F3FC-200D-1F692"},
    {"bombera_color3", "1F469-1F3FD-200D-1F692"},
    {"bombera_color4", "1F469-1F3FE-200D-1F692"},
    {"bombera_color5", "1F469-1F3FF-200D-1F692"},
    {"mujer_con_baston", "1F469-200D-1F9AF"},
    {"mujer_con_baston_color1", "1F469-1F3FB-200D-1F9AF"},
    {"mujer_con_baston_color2", "1F469-1F3FC-200D-1F9AF"},
    {"mujer_con_baston_color3", "1F469-1F3FD-200D-1F9AF"},
    {"mujer_con_baston_color4", "1F469-1F3FE-200D-1F9AF"},
    {"mujer_con_baston_color5", "1F469-1F3FF-200D-1F9AF"},
    {"mujer_con_baston_hacia_la_derecha", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"mujer_con_baston_hacia_la_derecha_color1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"mujer_con_baston_hacia_la_derecha_color2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"mujer_con_baston_hacia_la_derecha_color3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"mujer_con_baston_hacia_la_derecha_color4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"mujer_con_baston_hacia_la_derecha_color5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_electrica", "1F469-200D-1F9BC"},
    {"mujer_en_silla_de_ruedas_electrica_color1", "1F469-1F3FB-200D-1F9BC"},
    {"mujer_en_silla_de_ruedas_electrica_color2", "1F469-1F3FC-200D-1F9BC"},
    {"mujer_en_silla_de_ruedas_electrica_color3", "1F469-1F3FD-200D-1F9BC"},
    {"mujer_en_silla_de_ruedas_electrica_color4", "1F469-1F3FE-200D-1F9BC"},
    {"mujer_en_silla_de_ruedas_electrica_color5", "1F469-1F3FF-200D-1F9BC"},
    {"mujer_en_silla_de_ruedas_electrica_hacia_la_derecha", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_electrica_hacia_la_derecha_color1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_electrica_hacia_la_derecha_color2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_electrica_hacia_la_derecha_color3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_electrica_hacia_la_derecha_color4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_electrica_hacia_la_derecha_color5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_manual", "1F469-200D-1F9BD"},
    {"mujer_en_silla_de_ruedas_manual_color1", "1F469-1F3FB-200D-1F9BD"},
    {"mujer_en_silla_de_ruedas_manual_color2", "1F469-1F3FC-200D-1F9BD"},
    {"mujer_en_silla_de_ruedas_manual_color3", "1F469-1F3FD-200D-1F9BD"},
    {"mujer_en_silla_de_ruedas_manual_color4", "1F469-1F3FE-200D-1F9BD"},
    {"mujer_en_silla_de_ruedas_manual_color5", "1F469-1F3FF-200D-1F9BD"},
    {"mujer_en_silla_de_ruedas_manual_hacia_la_derecha", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_manual_hacia_la_derecha_color1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_manual_hacia_la_derecha_color2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_manual_hacia_la_derecha_color3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_manual_hacia_la_derecha_color4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_manual_hacia_la_derecha_color5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"persona_caminando_hacia_la_derecha", "1F6B6-200D-27A1-FE0F"},
    {"persona_caminando_hacia_la_derecha_color1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"persona_caminando_hacia_la_derecha_color2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"persona_caminando_hacia_la_derecha_color3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"persona_caminando_hacia_la_derecha_color4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"persona_caminando_hacia_la_derecha_color5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"persona_de_rodillas_hacia_la_derecha", "1F9CE-200D-27A1-FE0F"},
    {"persona_de_rodillas_hacia_la_derecha_color1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"persona_de_rodillas_hacia_la_derecha_color2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"persona_de_rodillas_hacia_la_derecha_color3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"persona_de_rodillas_hacia_la_derecha_color4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"persona_de_rodillas_hacia_la_derecha_color5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"profesional_sanitario", "1F9D1-200D-2695-FE0F"},
    {"profesional_sanitario_color1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"profesional_sanitario_color2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"profesional_sanitario_color3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"profesional_sanitario_color4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"profesional_sanitario_color5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"fiscal", "1F9D1-200D-2696-FE0F"},
    {"fiscal_color1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"fiscal_color2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"fiscal_color3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"fiscal_color4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"fiscal_color5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"piloto", "1F9D1-200D-2708-FE0F"},
    {"piloto_color1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"piloto_color2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"piloto_color3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"piloto_color4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"piloto_color5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"profesional_de_la_agricultura", "1F9D1-200D-1F33E"},
    {"profesional_de_la_agricultura_color1", "1F9D1-1F3FB-200D-1F33E"},
    {"profesional_de_la_agricultura_color2", "1F9D1-1F3FC-200D-1F33E"},
    {"profesional_de_la_agricultura_color3", "1F9D1-1F3FD-200D-1F33E"},
    {"profesional_de_la_agricultura_color4", "1F9D1-1F3FE-200D-1F33E"},
    {"profesional_de_la_agricultura_color5", "1F9D1-1F3FF-200D-1F33E"},
    {"chef", "1F9D1-200D-1F373"},
    {"chef_color1", "1F9D1-1F3FB-200D-1F373"},
    {"chef_color2", "1F9D1-1F3FC-200D-1F373"},
    {"chef_color3", "1F9D1-1F3FD-200D-1F373"},
    {"chef_color4", "1F9D1-1F3FE-200D-1F373"},
    {"chef_color5", "1F9D1-1F3FF-200D-1F373"},
    {"persona_alimentando_a_bebe", "1F9D1-200D-1F37C"},
    {"persona_alimentando_a_bebe_color1", "1F9D1-1F3FB-200D-1F37C"},
    {"persona_alimentando_a_bebe_color2", "1F9D1-1F3FC-200D-1F37C"},
    {"persona_alimentando_a_bebe_color3", "1F9D1-1F3FD-200D-1F37C"},
    {"persona_alimentando_a_bebe_color4", "1F9D1-1F3FE-200D-1F37C"},
    {"persona_alimentando_a_bebe_color5", "1F9D1-1F3FF-200D-1F37C"},
    {"noel", "1F9D1-200D-1F384"},
    {"noel_color1", "1F9D1-1F3FB-200D-1F384"},
    {"noel_color2", "1F9D1-1F3FC-200D-1F384"},
    {"noel_color3", "1F9D1-1F3FD-200D-1F384"},
    {"noel_color4", "1F9D1-1F3FE-200D-1F384"},
    {"noel_color5", "1F9D1-1F3FF-200D-1F384"},
    {"estudiante", "1F9D1-200D-1F393"},
    {"estudiante_color1", "1F9D1-1F3FB-200D-1F393"},
    {"estudiante_color2", "1F9D1-1F3FC-200D-1F393"},
    {"estudiante_color3", "1F9D1-1F3FD-200D-1F393"},
    {"estudiante_color4", "1F9D1-1F3FE-200D-1F393"},
    {"estudiante_color5", "1F9D1-1F3FF-200D-1F393"},
    {"cantante", "1F9D1-200D-1F3A4"},
    {"cantante_color1", "1F9D1-1F3FB-200D-1F3A4"},
    {"cantante_color2", "1F9D1-1F3FC-200D-1F3A4"},
    {"cantante_color3", "1F9D1-1F3FD-200D-1F3A4"},
    {"cantante_color4", "1F9D1-1F3FE-200D-1F3A4"},
    {"cantante_color5", "1F9D1-1F3FF-200D-1F3A4"},
    {"artista", "1F9D1-200D-1F3A8"},
    {"artista_color1", "1F9D1-1F3FB-200D-1F3A8"},
    {"artista_color2", "1F9D1-1F3FC-200D-1F3A8"},
    {"artista_color3", "1F9D1-1F3FD-200D-1F3A8"},
    {"artista_color4", "1F9D1-1F3FE-200D-1F3A8"},
    {"artista_color5", "1F9D1-1F3FF-200D-1F3A8"},
    {"docente", "1F9D1-200D-1F3EB"},
    {"docente_color1", "1F9D1-1F3FB-200D-1F3EB"},
    {"docente_color2", "1F9D1-1F3FC-200D-1F3EB"},
    {"docente_color3", "1F9D1-1F3FD-200D-1F3EB"},
    {"docente_color4", "1F9D1-1F3FE-200D-1F3EB"},
    {"docente_color5", "1F9D1-1F3FF-200D-1F3EB"},
    {"profesional_industrial", "1F9D1-200D-1F3ED"},
    {"profesional_industrial_color1", "1F9D1-1F3FB-200D-1F3ED"},
    {"profesional_industrial_color2", "1F9D1-1F3FC-200D-1F3ED"},
    {"profesional_industrial_color3", "1F9D1-1F3FD-200D-1F3ED"},
    {"profesional_industrial_color4", "1F9D1-1F3FE-200D-1F3ED"},
    {"profesional_industrial_color5", "1F9D1-1F3FF-200D-1F3ED"},
    {"profesional_de_la_tecnologia", "1F9D1-200D-1F4BB"},
    {"profesional_de_la_tecnologia_color1", "1F9D1-1F3FB-200D-1F4BB"},
    {"profesional_de_la_tecnologia_color2", "1F9D1-1F3FC-200D-1F4BB"},
    {"profesional_de_la_tecnologia_color3", "1F9D1-1F3FD-200D-1F4BB"},
    {"profesional_de_la_tecnologia_color4", "1F9D1-1F3FE-200D-1F4BB"},
    {"profesional_de_la_tecnologia_color5", "1F9D1-1F3FF-200D-1F4BB"},
    {"oficinista", "1F9D1-200D-1F4BC"},
    {"oficinista_color1", "1F9D1-1F3FB-200D-1F4BC"},
    {"oficinista_color2", "1F9D1-1F3FC-200D-1F4BC"},
    {"oficinista_color3", "1F9D1-1F3FD-200D-1F4BC"},
    {"oficinista_color4", "1F9D1-1F3FE-200D-1F4BC"},
    {"oficinista_color5", "1F9D1-1F3FF-200D-1F4BC"},
    {"profesional_de_la_mecanica", "1F9D1-200D-1F527"},
    {"profesional_de_la_mecanica_color1", "1F9D1-1F3FB-200D-1F527"},
    {"profesional_de_la_mecanica_color2", "1F9D1-1F3FC-200D-1F527"},
    {"profesional_de_la_mecanica_color3", "1F9D1-1F3FD-200D-1F527"},
    {"profesional_de_la_mecanica_color4", "1F9D1-1F3FE-200D-1F527"},
    {"profesional_de_la_mecanica_color5", "1F9D1-1F3FF-200D-1F527"},
    {"profesional_de_la_ciencia", "1F9D1-200D-1F52C"},
    {"profesional_de_la_ciencia_color1", "1F9D1-1F3FB-200D-1F52C"},
    {"profesional_de_la_ciencia_color2", "1F9D1-1F3FC-200D-1F52C"},
    {"profesional_de_la_ciencia_color3", "1F9D1-1F3FD-200D-1F52C"},
    {"profesional_de_la_ciencia_color4", "1F9D1-1F3FE-200D-1F52C"},
    {"profesional_de_la_ciencia_color5", "1F9D1-1F3FF-200D-1F52C"},
    {"astronauta", "1F9D1-200D-1F680"},
    {"astronauta_color1", "1F9D1-1F3FB-200D-1F680"},
    {"astronauta_color2", "1F9D1-1F3FC-200D-1F680"},
    {"astronauta_color3", "1F9D1-1F3FD-200D-1F680"},
    {"astronauta_color4", "1F9D1-1F3FE-200D-1F680"},
    {"astronauta_color5", "1F9D1-1F3FF-200D-1F680"},
    {"bombero", "1F9D1-200D-1F692"},
    {"bombero_color1", "1F9D1-1F3FB-200D-1F692"},
    {"bombero_color2", "1F9D1-1F3FC-200D-1F692"},
    {"bombero_color3", "1F9D1-1F3FD-200D-1F692"},
    {"bombero_color4", "1F9D1-1F3FE-200D-1F692"},
    {"bombero_color5", "1F9D1-1F3FF-200D-1F692"},
    {"persona_con_baston", "1F9D1-200D-1F9AF"},
    {"persona_con_baston_color1", "1F9D1-1F3FB-200D-1F9AF"},
    {"persona_con_baston_color2", "1F9D1-1F3FC-200D-1F9AF"},
    {"persona_con_baston_color3", "1F9D1-1F3FD-200D-1F9AF"},
    {"persona_con_baston_color4", "1F9D1-1F3FE-200D-1F9AF"},
    {"persona_con_baston_color5", "1F9D1-1F3FF-200D-1F9AF"},
    {"persona_con_baston_hacia_la_derecha", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_baston_hacia_la_derecha_color1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_baston_hacia_la_derecha_color2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_baston_hacia_la_derecha_color3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_baston_hacia_la_derecha_color4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_baston_hacia_la_derecha_color5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_electrica", "1F9D1-200D-1F9BC"},
    {"persona_en_silla_de_ruedas_electrica_color1", "1F9D1-1F3FB-200D-1F9BC"},
    {"persona_en_silla_de_ruedas_electrica_color2", "1F9D1-1F3FC-200D-1F9BC"},
    {"persona_en_silla_de_ruedas_electrica_color3", "1F9D1-1F3FD-200D-1F9BC"},
    {"persona_en_silla_de_ruedas_electrica_color4", "1F9D1-1F3FE-200D-1F9BC"},
    {"persona_en_silla_de_ruedas_electrica_color5", "1F9D1-1F3FF-200D-1F9BC"},
    {"persona_en_silla_de_ruedas_electrica_hacia_la_derecha", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_electrica_hacia_la_derecha_color1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_electrica_hacia_la_derecha_color2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_electrica_hacia_la_derecha_color3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_electrica_hacia_la_derecha_color4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_electrica_hacia_la_derecha_color5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_manual", "1F9D1-200D-1F9BD"},
    {"persona_en_silla_de_ruedas_manual_color1", "1F9D1-1F3FB-200D-1F9BD"},
    {"persona_en_silla_de_ruedas_manual_color2", "1F9D1-1F3FC-200D-1F9BD"},
    {"persona_en_silla_de_ruedas_manual_color3", "1F9D1-1F3FD-200D-1F9BD"},
    {"persona_en_silla_de_ruedas_manual_color4", "1F9D1-1F3FE-200D-1F9BD"},
    {"persona_en_silla_de_ruedas_manual_color5", "1F9D1-1F3FF-200D-1F9BD"},
    {"persona_en_silla_de_ruedas_manual_hacia_la_derecha", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_manual_hacia_la_derecha_color1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_manual_hacia_la_derecha_color2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_manual_hacia_la_derecha_color3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_manual_hacia_la_derecha_color4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_manual_hacia_la_derecha_color5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"mujer_botando_un_balon", "26F9-FE0F-200D-2640-FE0F"},
    {"mujer_botando_un_balon_color1", "26F9-1F3FB-200D-2640-FE0F"},
    {"mujer_botando_un_balon_color2", "26F9-1F3FC-200D-2640-FE0F"},
    {"mujer_botando_un_balon_color3", "26F9-1F3FD-200D-2640-FE0F"},
    {"mujer_botando_un_balon_color4", "26F9-1F3FE-200D-2640-FE0F"},
    {"mujer_botando_un_balon_color5", "26F9-1F3FF-200D-2640-FE0F"},
    {"hombre_botando_un_balon", "26F9-FE0F-200D-2642-FE0F"},
    {"hombre_botando_un_balon_color1", "26F9-1F3FB-200D-2642-FE0F"},
    {"hombre_botando_un_balon_color2", "26F9-1F3FC-200D-2642-FE0F"},
    {"hombre_botando_un_balon_color3", "26F9-1F3FD-200D-2642-FE0F"},
    {"hombre_botando_un_balon_color4", "26F9-1F3FE-200D-2642-FE0F"},
    {"hombre_botando_un_balon_color5", "26F9-1F3FF-200D-2642-FE0F"},
    {"mujer_corriendo", "1F3C3-200D-2640-FE0F"},
    {"mujer_corriendo_color1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"mujer_corriendo_color2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"mujer_corriendo_color3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"mujer_corriendo_color4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"mujer_corriendo_color5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"mujer_corriendo_hacia_la_derecha", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_corriendo_hacia_la_derecha_color1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_corriendo_hacia_la_derecha_color2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_corriendo_hacia_la_derecha_color3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_corriendo_hacia_la_derecha_color4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_corriendo_hacia_la_derecha_color5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"hombre_corriendo", "1F3C3-200D-2642-FE0F"},
    {"hombre_corriendo_color1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"hombre_corriendo_color2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"hombre_corriendo_color3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"hombre_corriendo_color4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"hombre_corriendo_color5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"hombre_corriendo_hacia_la_derecha", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_corriendo_hacia_la_derecha_color1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_corriendo_hacia_la_derecha_color2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_corriendo_hacia_la_derecha_color3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_corriendo_hacia_la_derecha_color4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_corriendo_hacia_la_derecha_color5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mujer_haciendo_surf", "1F3C4-200D-2640-FE0F"},
    {"mujer_haciendo_surf_color1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"mujer_haciendo_surf_color2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"mujer_haciendo_surf_color3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"mujer_haciendo_surf_color4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"mujer_haciendo_surf_color5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"hombre_haciendo_surf", "1F3C4-200D-2642-FE0F"},
    {"hombre_haciendo_surf_color1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"hombre_haciendo_surf_color2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"hombre_haciendo_surf_color3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"hombre_haciendo_surf_color4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"hombre_haciendo_surf_color5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"mujer_nadando", "1F3CA-200D-2640-FE0F"},
    {"mujer_nadando_color1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"mujer_nadando_color2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"mujer_nadando_color3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"mujer_nadando_color4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"mujer_nadando_color5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"hombre_nadando", "1F3CA-200D-2642-FE0F"},
    {"hombre_nadando_color1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"hombre_nadando_color2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"hombre_nadando_color3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"hombre_nadando_color4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"hombre_nadando_color5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"mujer_levantando_pesas", "1F3CB-FE0F-200D-2640-FE0F"},
    {"mujer_levantando_pesas_color1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"mujer_levantando_pesas_color2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"mujer_levantando_pesas_color3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"mujer_levantando_pesas_color4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"mujer_levantando_pesas_color5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"hombre_levantando_pesas", "1F3CB-FE0F-200D-2642-FE0F"},
    {"hombre_levantando_pesas_color1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"hombre_levantando_pesas_color2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"hombre_levantando_pesas_color3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"hombre_levantando_pesas_color4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"hombre_levantando_pesas_color5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"mujer_jugando_al_golf", "1F3CC-FE0F-200D-2640-FE0F"},
    {"mujer_jugando_al_golf_color1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"mujer_jugando_al_golf_color2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"mujer_jugando_al_golf_color3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"mujer_jugando_al_golf_color4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"mujer_jugando_al_golf_color5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"hombre_jugando_al_golf", "1F3CC-FE0F-200D-2642-FE0F"},
    {"hombre_jugando_al_golf_color1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"hombre_jugando_al_golf_color2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"hombre_jugando_al_golf_color3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"hombre_jugando_al_golf_color4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"hombre_jugando_al_golf_color5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"agente_de_policia_mujer", "1F46E-200D-2640-FE0F"},
    {"agente_de_policia_mujer_color1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"agente_de_policia_mujer_color2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"agente_de_policia_mujer_color3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"agente_de_policia_mujer_color4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"agente_de_policia_mujer_color5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"agente_de_policia_hombre", "1F46E-200D-2642-FE0F"},
    {"agente_de_policia_hombre_color1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"agente_de_policia_hombre_color2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"agente_de_policia_hombre_color3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"agente_de_policia_hombre_color4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"agente_de_policia_hombre_color5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"mujeres_con_orejas_de_conejo", "1F46F-200D-2640-FE0F"},
    {"mujeres_con_orejas_de_conejo_color1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"mujeres_con_orejas_de_conejo_color1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"mujeres_con_orejas_de_conejo_color1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"mujeres_con_orejas_de_conejo_color1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"mujeres_con_orejas_de_conejo_color2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"mujeres_con_orejas_de_conejo_color2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"mujeres_con_orejas_de_conejo_color2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"mujeres_con_orejas_de_conejo_color2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"mujeres_con_orejas_de_conejo_color3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"mujeres_con_orejas_de_conejo_color3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"mujeres_con_orejas_de_conejo_color3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"mujeres_con_orejas_de_conejo_color3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"mujeres_con_orejas_de_conejo_color4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"mujeres_con_orejas_de_conejo_color4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"mujeres_con_orejas_de_conejo_color4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"mujeres_con_orejas_de_conejo_color4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"mujeres_con_orejas_de_conejo_color5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"mujeres_con_orejas_de_conejo_color5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"mujeres_con_orejas_de_conejo_color5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"mujeres_con_orejas_de_conejo_color5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"mujeres_con_orejas_de_conejo_color1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"mujeres_con_orejas_de_conejo_color2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"mujeres_con_orejas_de_conejo_color3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"mujeres_con_orejas_de_conejo_color4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"mujeres_con_orejas_de_conejo_color5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"hombres_con_orejas_de_conejo", "1F46F-200D-2642-FE0F"},
    {"hombres_con_orejas_de_conejo_color1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"hombres_con_orejas_de_conejo_color1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"hombres_con_orejas_de_conejo_color1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"hombres_con_orejas_de_conejo_color1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"hombres_con_orejas_de_conejo_color2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"hombres_con_orejas_de_conejo_color2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"hombres_con_orejas_de_conejo_color2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"hombres_con_orejas_de_conejo_color2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"hombres_con_orejas_de_conejo_color3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"hombres_con_orejas_de_conejo_color3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"hombres_con_orejas_de_conejo_color3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"hombres_con_orejas_de_conejo_color3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"hombres_con_orejas_de_conejo_color4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"hombres_con_orejas_de_conejo_color4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"hombres_con_orejas_de_conejo_color4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"hombres_con_orejas_de_conejo_color4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"hombres_con_orejas_de_conejo_color5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"hombres_con_orejas_de_conejo_color5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"hombres_con_orejas_de_conejo_color5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"hombres_con_orejas_de_conejo_color5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"hombres_con_orejas_de_conejo_color1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"hombres_con_orejas_de_conejo_color2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"hombres_con_orejas_de_conejo_color3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"hombres_con_orejas_de_conejo_color4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"hombres_con_orejas_de_conejo_color5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"mujer_con_velo", "1F470-200D-2640-FE0F"},
    {"mujer_con_velo_color1", "1F470-1F3FB-200D-2640-FE0F"},
    {"mujer_con_velo_color2", "1F470-1F3FC-200D-2640-FE0F"},
    {"mujer_con_velo_color3", "1F470-1F3FD-200D-2640-FE0F"},
    {"mujer_con_velo_color4", "1F470-1F3FE-200D-2640-FE0F"},
    {"mujer_con_velo_color5", "1F470-1F3FF-200D-2640-FE0F"},
    {"hombre_con_velo", "1F470-200D-2642-FE0F"},
    {"hombre_con_velo_color1", "1F470-1F3FB-200D-2642-FE0F"},
    {"hombre_con_velo_color2", "1F470-1F3FC-200D-2642-FE0F"},
    {"hombre_con_velo_color3", "1F470-1F3FD-200D-2642-FE0F"},
    {"hombre_con_velo_color4", "1F470-1F3FE-200D-2642-FE0F"},
    {"hombre_con_velo_color5", "1F470-1F3FF-200D-2642-FE0F"},
    {"mujer_rubia", "1F471-200D-2640-FE0F"},
    {"mujer_rubia_color1", "1F471-1F3FB-200D-2640-FE0F"},
    {"mujer_rubia_color2", "1F471-1F3FC-200D-2640-FE0F"},
    {"mujer_rubia_color3", "1F471-1F3FD-200D-2640-FE0F"},
    {"mujer_rubia_color4", "1F471-1F3FE-200D-2640-FE0F"},
    {"mujer_rubia_color5", "1F471-1F3FF-200D-2640-FE0F"},
    {"hombre_rubio", "1F471-200D-2642-FE0F"},
    {"hombre_rubio_color1", "1F471-1F3FB-200D-2642-FE0F"},
    {"hombre_rubio_color2", "1F471-1F3FC-200D-2642-FE0F"},
    {"hombre_rubio_color3", "1F471-1F3FD-200D-2642-FE0F"},
    {"hombre_rubio_color4", "1F471-1F3FE-200D-2642-FE0F"},
    {"hombre_rubio_color5", "1F471-1F3FF-200D-2642-FE0F"},
    {"mujer_con_turbante", "1F473-200D-2640-FE0F"},
    {"mujer_con_turbante_color1", "1F473-1F3FB-200D-2640-FE0F"},
    {"mujer_con_turbante_color2", "1F473-1F3FC-200D-2640-FE0F"},
    {"mujer_con_turbante_color3", "1F473-1F3FD-200D-2640-FE0F"},
    {"mujer_con_turbante_color4", "1F473-1F3FE-200D-2640-FE0F"},
    {"mujer_con_turbante_color5", "1F473-1F3FF-200D-2640-FE0F"},
    {"hombre_con_turbante", "1F473-200D-2642-FE0F"},
    {"hombre_con_turbante_color1", "1F473-1F3FB-200D-2642-FE0F"},
    {"hombre_con_turbante_color2", "1F473-1F3FC-200D-2642-FE0F"},
    {"hombre_con_turbante_color3", "1F473-1F3FD-200D-2642-FE0F"},
    {"hombre_con_turbante_color4", "1F473-1F3FE-200D-2642-FE0F"},
    {"hombre_con_turbante_color5", "1F473-1F3FF-200D-2642-FE0F"},
    {"profesional_de_la_construccion_mujer", "1F477-200D-2640-FE0F"},
    {"profesional_de_la_construccion_mujer_color1", "1F477-1F3FB-200D-2640-FE0F"},
    {"profesional_de_la_construccion_mujer_color2", "1F477-1F3FC-200D-2640-FE0F"},
    {"profesional_de_la_construccion_mujer_color3", "1F477-1F3FD-200D-2640-FE0F"},
    {"profesional_de_la_construccion_mujer_color4", "1F477-1F3FE-200D-2640-FE0F"},
    {"profesional_de_la_construccion_mujer_color5", "1F477-1F3FF-200D-2640-FE0F"},
    {"profesional_de_la_construccion_hombre", "1F477-200D-2642-FE0F"},
    {"profesional_de_la_construccion_hombre_color1", "1F477-1F3FB-200D-2642-FE0F"},
    {"profesional_de_la_construccion_hombre_color2", "1F477-1F3FC-200D-2642-FE0F"},
    {"profesional_de_la_construccion_hombre_color3", "1F477-1F3FD-200D-2642-FE0F"},
    {"profesional_de_la_construccion_hombre_color4", "1F477-1F3FE-200D-2642-FE0F"},
    {"profesional_de_la_construccion_hombre_color5", "1F477-1F3FF-200D-2642-FE0F"},
    {"empleada_de_mostrador_de_informacion", "1F481-200D-2640-FE0F"},
    {"empleada_de_mostrador_de_informacion_color1", "1F481-1F3FB-200D-2640-FE0F"},
    {"empleada_de_mostrador_de_informacion_color2", "1F481-1F3FC-200D-2640-FE0F"},
    {"empleada_de_mostrador_de_informacion_color3", "1F481-1F3FD-200D-2640-FE0F"},
    {"empleada_de_mostrador_de_informacion_color4", "1F481-1F3FE-200D-2640-FE0F"},
    {"empleada_de_mostrador_de_informacion_color5", "1F481-1F3FF-200D-2640-FE0F"},
    {"empleado_de_mostrador_de_informacion", "1F481-200D-2642-FE0F"},
    {"empleado_de_mostrador_de_informacion_color1", "1F481-1F3FB-200D-2642-FE0F"},
    {"empleado_de_mostrador_de_informacion_color2", "1F481-1F3FC-200D-2642-FE0F"},
    {"empleado_de_mostrador_de_informacion_color3", "1F481-1F3FD-200D-2642-FE0F"},
    {"empleado_de_mostrador_de_informacion_color4", "1F481-1F3FE-200D-2642-FE0F"},
    {"empleado_de_mostrador_de_informacion_color5", "1F481-1F3FF-200D-2642-FE0F"},
    {"guardia_mujer", "1F482-200D-2640-FE0F"},
    {"guardia_mujer_color1", "1F482-1F3FB-200D-2640-FE0F"},
    {"guardia_mujer_color2", "1F482-1F3FC-200D-2640-FE0F"},
    {"guardia_mujer_color3", "1F482-1F3FD-200D-2640-FE0F"},
    {"guardia_mujer_color4", "1F482-1F3FE-200D-2640-FE0F"},
    {"guardia_mujer_color5", "1F482-1F3FF-200D-2640-FE0F"},
    {"guardia_hombre", "1F482-200D-2642-FE0F"},
    {"guardia_hombre_color1", "1F482-1F3FB-200D-2642-FE0F"},
    {"guardia_hombre_color2", "1F482-1F3FC-200D-2642-FE0F"},
    {"guardia_hombre_color3", "1F482-1F3FD-200D-2642-FE0F"},
    {"guardia_hombre_color4", "1F482-1F3FE-200D-2642-FE0F"},
    {"guardia_hombre_color5", "1F482-1F3FF-200D-2642-FE0F"},
    {"mujer_recibiendo_masaje", "1F486-200D-2640-FE0F"},
    {"mujer_recibiendo_masaje_color1", "1F486-1F3FB-200D-2640-FE0F"},
    {"mujer_recibiendo_masaje_color2", "1F486-1F3FC-200D-2640-FE0F"},
    {"mujer_recibiendo_masaje_color3", "1F486-1F3FD-200D-2640-FE0F"},
    {"mujer_recibiendo_masaje_color4", "1F486-1F3FE-200D-2640-FE0F"},
    {"mujer_recibiendo_masaje_color5", "1F486-1F3FF-200D-2640-FE0F"},
    {"hombre_recibiendo_masaje", "1F486-200D-2642-FE0F"},
    {"hombre_recibiendo_masaje_color1", "1F486-1F3FB-200D-2642-FE0F"},
    {"hombre_recibiendo_masaje_color2", "1F486-1F3FC-200D-2642-FE0F"},
    {"hombre_recibiendo_masaje_color3", "1F486-1F3FD-200D-2642-FE0F"},
    {"hombre_recibiendo_masaje_color4", "1F486-1F3FE-200D-2642-FE0F"},
    {"hombre_recibiendo_masaje_color5", "1F486-1F3FF-200D-2642-FE0F"},
    {"mujer_cortandose_el_pelo", "1F487-200D-2640-FE0F"},
    {"mujer_cortandose_el_pelo_color1", "1F487-1F3FB-200D-2640-FE0F"},
    {"mujer_cortandose_el_pelo_color2", "1F487-1F3FC-200D-2640-FE0F"},
    {"mujer_cortandose_el_pelo_color3", "1F487-1F3FD-200D-2640-FE0F"},
    {"mujer_cortandose_el_pelo_color4", "1F487-1F3FE-200D-2640-FE0F"},
    {"mujer_cortandose_el_pelo_color5", "1F487-1F3FF-200D-2640-FE0F"},
    {"hombre_cortandose_el_pelo", "1F487-200D-2642-FE0F"},
    {"hombre_cortandose_el_pelo_color1", "1F487-1F3FB-200D-2642-FE0F"},
    {"hombre_cortandose_el_pelo_color2", "1F487-1F3FC-200D-2642-FE0F"},
    {"hombre_cortandose_el_pelo_color3", "1F487-1F3FD-200D-2642-FE0F"},
    {"hombre_cortandose_el_pelo_color4", "1F487-1F3FE-200D-2642-FE0F"},
    {"hombre_cortandose_el_pelo_color5", "1F487-1F3FF-200D-2642-FE0F"},
    {"detective_mujer", "1F575-FE0F-200D-2640-FE0F"},
    {"detective_mujer_color1", "1F575-1F3FB-200D-2640-FE0F"},
    {"detective_mujer_color2", "1F575-1F3FC-200D-2640-FE0F"},
    {"detective_mujer_color3", "1F575-1F3FD-200D-2640-FE0F"},
    {"detective_mujer_color4", "1F575-1F3FE-200D-2640-FE0F"},
    {"detective_mujer_color5", "1F575-1F3FF-200D-2640-FE0F"},
    {"detective_hombre", "1F575-FE0F-200D-2642-FE0F"},
    {"detective_hombre_color1", "1F575-1F3FB-200D-2642-FE0F"},
    {"detective_hombre_color2", "1F575-1F3FC-200D-2642-FE0F"},
    {"detective_hombre_color3", "1F575-1F3FD-200D-2642-FE0F"},
    {"detective_hombre_color4", "1F575-1F3FE-200D-2642-FE0F"},
    {"detective_hombre_color5", "1F575-1F3FF-200D-2642-FE0F"},
    {"mujer_haciendo_el_gesto_de_no", "1F645-200D-2640-FE0F"},
    {"mujer_haciendo_el_gesto_de_no_color1", "1F645-1F3FB-200D-2640-FE0F"},
    {"mujer_haciendo_el_gesto_de_no_color2", "1F645-1F3FC-200D-2640-FE0F"},
    {"mujer_haciendo_el_gesto_de_no_color3", "1F645-1F3FD-200D-2640-FE0F"},
    {"mujer_haciendo_el_gesto_de_no_color4", "1F645-1F3FE-200D-2640-FE0F"},
    {"mujer_haciendo_el_gesto_de_no_color5", "1F645-1F3FF-200D-2640-FE0F"},
    {"hombre_haciendo_el_gesto_de_no", "1F645-200D-2642-FE0F"},
    {"hombre_haciendo_el_gesto_de_no_color1", "1F645-1F3FB-200D-2642-FE0F"},
    {"hombre_haciendo_el_gesto_de_no_color2", "1F645-1F3FC-200D-2642-FE0F"},
    {"hombre_haciendo_el_gesto_de_no_color3", "1F645-1F3FD-200D-2642-FE0F"},
    {"hombre_haciendo_el_gesto_de_no_color4", "1F645-1F3FE-200D-2642-FE0F"},
    {"hombre_haciendo_el_gesto_de_no_color5", "1F645-1F3FF-200D-2642-FE0F"},
    {"mujer_haciendo_el_gesto_de_de_acuerdo", "1F646-200D-2640-FE0F"},
    {"mujer_haciendo_el_gesto_de_de_acuerdo_color1", "1F646-1F3FB-200D-2640-FE0F"},
    {"mujer_haciendo_el_gesto_de_de_acuerdo_color2", "1F646-1F3FC-200D-2640-FE0F"},
    {"mujer_haciendo_el_gesto_de_de_acuerdo_color3", "1F646-1F3FD-200D-2640-FE0F"},
    {"mujer_haciendo_el_gesto_de_de_acuerdo_color4", "1F646-1F3FE-200D-2640-FE0F"},
    {"mujer_haciendo_el_gesto_de_de_acuerdo_color5", "1F646-1F3FF-200D-2640-FE0F"},
    {"hombre_haciendo_el_gesto_de_de_acuerdo", "1F646-200D-2642-FE0F"},
    {"hombre_haciendo_el_gesto_de_de_acuerdo_color1", "1F646-1F3FB-200D-2642-FE0F"},
    {"hombre_haciendo_el_gesto_de_de_acuerdo_color2", "1F646-1F3FC-200D-2642-FE0F"},
    {"hombre_haciendo_el_gesto_de_de_acuerdo_color3", "1F646-1F3FD-200D-2642-FE0F"},
    {"hombre_haciendo_el_gesto_de_de_acuerdo_color4", "1F646-1F3FE-200D-2642-FE0F"},
    {"hombre_haciendo_el_gesto_de_de_acuerdo_color5", "1F646-1F3FF-200D-2642-FE0F"},
    {"mujer_haciendo_una_reverencia", "1F647-200D-2640-FE0F"},
    {"mujer_haciendo_una_reverencia_color1", "1F647-1F3FB-200D-2640-FE0F"},
    {"mujer_haciendo_una_reverencia_color2", "1F647-1F3FC-200D-2640-FE0F"},
    {"mujer_haciendo_una_reverencia_color3", "1F647-1F3FD-200D-2640-FE0F"},
    {"mujer_haciendo_una_reverencia_color4", "1F647-1F3FE-200D-2640-FE0F"},
    {"mujer_haciendo_una_reverencia_color5", "1F647-1F3FF-200D-2640-FE0F"},
    {"hombre_haciendo_una_reverencia", "1F647-200D-2642-FE0F"},
    {"hombre_haciendo_una_reverencia_color1", "1F647-1F3FB-200D-2642-FE0F"},
    {"hombre_haciendo_una_reverencia_color2", "1F647-1F3FC-200D-2642-FE0F"},
    {"hombre_haciendo_una_reverencia_color3", "1F647-1F3FD-200D-2642-FE0F"},
    {"hombre_haciendo_una_reverencia_color4", "1F647-1F3FE-200D-2642-FE0F"},
    {"hombre_haciendo_una_reverencia_color5", "1F647-1F3FF-200D-2642-FE0F"},
    {"mujer_con_la_mano_levantada", "1F64B-200D-2640-FE0F"},
    {"mujer_con_la_mano_levantada_color1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"mujer_con_la_mano_levantada_color2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"mujer_con_la_mano_levantada_color3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"mujer_con_la_mano_levantada_color4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"mujer_con_la_mano_levantada_color5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"hombre_con_la_mano_levantada", "1F64B-200D-2642-FE0F"},
    {"hombre_con_la_mano_levantada_color1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"hombre_con_la_mano_levantada_color2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"hombre_con_la_mano_levantada_color3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"hombre_con_la_mano_levantada_color4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"hombre_con_la_mano_levantada_color5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"mujer_frunciendo_el_ceno", "1F64D-200D-2640-FE0F"},
    {"mujer_frunciendo_el_ceno_color1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"mujer_frunciendo_el_ceno_color2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"mujer_frunciendo_el_ceno_color3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"mujer_frunciendo_el_ceno_color4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"mujer_frunciendo_el_ceno_color5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"hombre_frunciendo_el_ceno", "1F64D-200D-2642-FE0F"},
    {"hombre_frunciendo_el_ceno_color1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"hombre_frunciendo_el_ceno_color2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"hombre_frunciendo_el_ceno_color3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"hombre_frunciendo_el_ceno_color4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"hombre_frunciendo_el_ceno_color5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"mujer_haciendo_pucheros", "1F64E-200D-2640-FE0F"},
    {"mujer_haciendo_pucheros_color1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"mujer_haciendo_pucheros_color2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"mujer_haciendo_pucheros_color3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"mujer_haciendo_pucheros_color4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"mujer_haciendo_pucheros_color5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"hombre_haciendo_pucheros", "1F64E-200D-2642-FE0F"},
    {"hombre_haciendo_pucheros_color1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"hombre_haciendo_pucheros_color2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"hombre_haciendo_pucheros_color3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"hombre_haciendo_pucheros_color4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"hombre_haciendo_pucheros_color5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"mujer_remando_en_un_bote", "1F6A3-200D-2640-FE0F"},
    {"mujer_remando_en_un_bote_color1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"mujer_remando_en_un_bote_color2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"mujer_remando_en_un_bote_color3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"mujer_remando_en_un_bote_color4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"mujer_remando_en_un_bote_color5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"hombre_remando_en_un_bote", "1F6A3-200D-2642-FE0F"},
    {"hombre_remando_en_un_bote_color1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"hombre_remando_en_un_bote_color2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"hombre_remando_en_un_bote_color3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"hombre_remando_en_un_bote_color4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"hombre_remando_en_un_bote_color5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"mujer_en_bicicleta", "1F6B4-200D-2640-FE0F"},
    {"mujer_en_bicicleta_color1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"mujer_en_bicicleta_color2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"mujer_en_bicicleta_color3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"mujer_en_bicicleta_color4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"mujer_en_bicicleta_color5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"hombre_en_bicicleta", "1F6B4-200D-2642-FE0F"},
    {"hombre_en_bicicleta_color1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"hombre_en_bicicleta_color2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"hombre_en_bicicleta_color3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"hombre_en_bicicleta_color4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"hombre_en_bicicleta_color5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"mujer_en_bicicleta_de_montana", "1F6B5-200D-2640-FE0F"},
    {"mujer_en_bicicleta_de_montana_color1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"mujer_en_bicicleta_de_montana_color2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"mujer_en_bicicleta_de_montana_color3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"mujer_en_bicicleta_de_montana_color4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"mujer_en_bicicleta_de_montana_color5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"hombre_en_bicicleta_de_montana", "1F6B5-200D-2642-FE0F"},
    {"hombre_en_bicicleta_de_montana_color1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"hombre_en_bicicleta_de_montana_color2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"hombre_en_bicicleta_de_montana_color3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"hombre_en_bicicleta_de_montana_color4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"hombre_en_bicicleta_de_montana_color5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"mujer_caminando", "1F6B6-200D-2640-FE0F"},
    {"mujer_caminando_color1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"mujer_caminando_color2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"mujer_caminando_color3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"mujer_caminando_color4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"mujer_caminando_color5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"mujer_caminando_hacia_la_derecha", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_caminando_hacia_la_derecha_color1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_caminando_hacia_la_derecha_color2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_caminando_hacia_la_derecha_color3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_caminando_hacia_la_derecha_color4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_caminando_hacia_la_derecha_color5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"hombre_caminando", "1F6B6-200D-2642-FE0F"},
    {"hombre_caminando_color1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"hombre_caminando_color2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"hombre_caminando_color3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"hombre_caminando_color4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"hombre_caminando_color5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"hombre_caminando_hacia_la_derecha", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_caminando_hacia_la_derecha_color1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_caminando_hacia_la_derecha_color2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_caminando_hacia_la_derecha_color3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_caminando_hacia_la_derecha_color4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_caminando_hacia_la_derecha_color5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mujer_con_la_mano_en_la_frente", "1F926-200D-2640-FE0F"},
    {"mujer_con_la_mano_en_la_frente_color1", "1F926-1F3FB-200D-2640-FE0F"},
    {"mujer_con_la_mano_en_la_frente_color2", "1F926-1F3FC-200D-2640-FE0F"},
    {"mujer_con_la_mano_en_la_frente_color3", "1F926-1F3FD-200D-2640-FE0F"},
    {"mujer_con_la_mano_en_la_frente_color4", "1F926-1F3FE-200D-2640-FE0F"},
    {"mujer_con_la_mano_en_la_frente_color5", "1F926-1F3FF-200D-2640-FE0F"},
    {"hombre_con_la_mano_en_la_frente", "1F926-200D-2642-FE0F"},
    {"hombre_con_la_mano_en_la_frente_color1", "1F926-1F3FB-200D-2642-FE0F"},
    {"hombre_con_la_mano_en_la_frente_color2", "1F926-1F3FC-200D-2642-FE0F"},
    {"hombre_con_la_mano_en_la_frente_color3", "1F926-1F3FD-200D-2642-FE0F"},
    {"hombre_con_la_mano_en_la_frente_color4", "1F926-1F3FE-200D-2642-FE0F"},
    {"hombre_con_la_mano_en_la_frente_color5", "1F926-1F3FF-200D-2642-FE0F"},
    {"mujer_con_esmoquin", "1F935-200D-2640-FE0F"},
    {"mujer_con_esmoquin_color1", "1F935-1F3FB-200D-2640-FE0F"},
    {"mujer_con_esmoquin_color2", "1F935-1F3FC-200D-2640-FE0F"},
    {"mujer_con_esmoquin_color3", "1F935-1F3FD-200D-2640-FE0F"},
    {"mujer_con_esmoquin_color4", "1F935-1F3FE-200D-2640-FE0F"},
    {"mujer_con_esmoquin_color5", "1F935-1F3FF-200D-2640-FE0F"},
    {"hombre_con_esmoquin", "1F935-200D-2642-FE0F"},
    {"hombre_con_esmoquin_color1", "1F935-1F3FB-200D-2642-FE0F"},
    {"hombre_con_esmoquin_color2", "1F935-1F3FC-200D-2642-FE0F"},
    {"hombre_con_esmoquin_color3", "1F935-1F3FD-200D-2642-FE0F"},
    {"hombre_con_esmoquin_color4", "1F935-1F3FE-200D-2642-FE0F"},
    {"hombre_con_esmoquin_color5", "1F935-1F3FF-200D-2642-FE0F"},
    {"mujer_encogida_de_hombros", "1F937-200D-2640-FE0F"},
    {"mujer_encogida_de_hombros_color1", "1F937-1F3FB-200D-2640-FE0F"},
    {"mujer_encogida_de_hombros_color2", "1F937-1F3FC-200D-2640-FE0F"},
    {"mujer_encogida_de_hombros_color3", "1F937-1F3FD-200D-2640-FE0F"},
    {"mujer_encogida_de_hombros_color4", "1F937-1F3FE-200D-2640-FE0F"},
    {"mujer_encogida_de_hombros_color5", "1F937-1F3FF-200D-2640-FE0F"},
    {"hombre_encogido_de_hombros", "1F937-200D-2642-FE0F"},
    {"hombre_encogido_de_hombros_color1", "1F937-1F3FB-200D-2642-FE0F"},
    {"hombre_encogido_de_hombros_color2", "1F937-1F3FC-200D-2642-FE0F"},
    {"hombre_encogido_de_hombros_color3", "1F937-1F3FD-200D-2642-FE0F"},
    {"hombre_encogido_de_hombros_color4", "1F937-1F3FE-200D-2642-FE0F"},
    {"hombre_encogido_de_hombros_color5", "1F937-1F3FF-200D-2642-FE0F"},
    {"mujer_dando_una_voltereta_lateral", "1F938-200D-2640-FE0F"},
    {"mujer_dando_una_voltereta_lateral_color1", "1F938-1F3FB-200D-2640-FE0F"},
    {"mujer_dando_una_voltereta_lateral_color2", "1F938-1F3FC-200D-2640-FE0F"},
    {"mujer_dando_una_voltereta_lateral_color3", "1F938-1F3FD-200D-2640-FE0F"},
    {"mujer_dando_una_voltereta_lateral_color4", "1F938-1F3FE-200D-2640-FE0F"},
    {"mujer_dando_una_voltereta_lateral_color5", "1F938-1F3FF-200D-2640-FE0F"},
    {"hombre_dando_una_voltereta_lateral", "1F938-200D-2642-FE0F"},
    {"hombre_dando_una_voltereta_lateral_color1", "1F938-1F3FB-200D-2642-FE0F"},
    {"hombre_dando_una_voltereta_lateral_color2", "1F938-1F3FC-200D-2642-FE0F"},
    {"hombre_dando_una_voltereta_lateral_color3", "1F938-1F3FD-200D-2642-FE0F"},
    {"hombre_dando_una_voltereta_lateral_color4", "1F938-1F3FE-200D-2642-FE0F"},
    {"hombre_dando_una_voltereta_lateral_color5", "1F938-1F3FF-200D-2642-FE0F"},
    {"mujer_haciendo_malabares", "1F939-200D-2640-FE0F"},
    {"mujer_haciendo_malabares_color1", "1F939-1F3FB-200D-2640-FE0F"},
    {"mujer_haciendo_malabares_color2", "1F939-1F3FC-200D-2640-FE0F"},
    {"mujer_haciendo_malabares_color3", "1F939-1F3FD-200D-2640-FE0F"},
    {"mujer_haciendo_malabares_color4", "1F939-1F3FE-200D-2640-FE0F"},
    {"mujer_haciendo_malabares_color5", "1F939-1F3FF-200D-2640-FE0F"},
    {"hombre_haciendo_malabares", "1F939-200D-2642-FE0F"},
    {"hombre_haciendo_malabares_color1", "1F939-1F3FB-200D-2642-FE0F"},
    {"hombre_haciendo_malabares_color2", "1F939-1F3FC-200D-2642-FE0F"},
    {"hombre_haciendo_malabares_color3", "1F939-1F3FD-200D-2642-FE0F"},
    {"hombre_haciendo_malabares_color4", "1F939-1F3FE-200D-2642-FE0F"},
    {"hombre_haciendo_malabares_color5", "1F939-1F3FF-200D-2642-FE0F"},
    {"mujeres_luchando", "1F93C-200D-2640-FE0F"},
    {"mujeres_luchando_color1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"mujeres_luchando_color1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"mujeres_luchando_color1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"mujeres_luchando_color1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"mujeres_luchando_color2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"mujeres_luchando_color2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"mujeres_luchando_color2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"mujeres_luchando_color2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"mujeres_luchando_color3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"mujeres_luchando_color3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"mujeres_luchando_color3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"mujeres_luchando_color3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"mujeres_luchando_color4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"mujeres_luchando_color4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"mujeres_luchando_color4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"mujeres_luchando_color4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"mujeres_luchando_color5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"mujeres_luchando_color5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"mujeres_luchando_color5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"mujeres_luchando_color5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"mujeres_luchando_color1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"mujeres_luchando_color2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"mujeres_luchando_color3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"mujeres_luchando_color4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"mujeres_luchando_color5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"hombres_luchando", "1F93C-200D-2642-FE0F"},
    {"hombres_luchando_color1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"hombres_luchando_color1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"hombres_luchando_color1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"hombres_luchando_color1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"hombres_luchando_color2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"hombres_luchando_color2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"hombres_luchando_color2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"hombres_luchando_color2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"hombres_luchando_color3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"hombres_luchando_color3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"hombres_luchando_color3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"hombres_luchando_color3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"hombres_luchando_color4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"hombres_luchando_color4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"hombres_luchando_color4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"hombres_luchando_color4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"hombres_luchando_color5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"hombres_luchando_color5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"hombres_luchando_color5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"hombres_luchando_color5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"hombres_luchando_color1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"hombres_luchando_color2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"hombres_luchando_color3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"hombres_luchando_color4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"hombres_luchando_color5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"mujer_jugando_al_waterpolo", "1F93D-200D-2640-FE0F"},
    {"mujer_jugando_al_waterpolo_color1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"mujer_jugando_al_waterpolo_color2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"mujer_jugando_al_waterpolo_color3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"mujer_jugando_al_waterpolo_color4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"mujer_jugando_al_waterpolo_color5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"hombre_jugando_al_waterpolo", "1F93D-200D-2642-FE0F"},
    {"hombre_jugando_al_waterpolo_color1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"hombre_jugando_al_waterpolo_color2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"hombre_jugando_al_waterpolo_color3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"hombre_jugando_al_waterpolo_color4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"hombre_jugando_al_waterpolo_color5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"mujer_jugando_al_balonmano", "1F93E-200D-2640-FE0F"},
    {"mujer_jugando_al_balonmano_color1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"mujer_jugando_al_balonmano_color2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"mujer_jugando_al_balonmano_color3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"mujer_jugando_al_balonmano_color4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"mujer_jugando_al_balonmano_color5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"hombre_jugando_al_balonmano", "1F93E-200D-2642-FE0F"},
    {"hombre_jugando_al_balonmano_color1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"hombre_jugando_al_balonmano_color2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"hombre_jugando_al_balonmano_color3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"hombre_jugando_al_balonmano_color4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"hombre_jugando_al_balonmano_color5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"superheroina", "1F9B8-200D-2640-FE0F"},
    {"superheroina_color1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"superheroina_color2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"superheroina_color3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"superheroina_color4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"superheroina_color5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"superheroe", "1F9B8-200D-2642-FE0F"},
    {"superheroe_color1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"superheroe_color2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"superheroe_color3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"superheroe_color4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"superheroe_color5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"supervillana", "1F9B9-200D-2640-FE0F"},
    {"supervillana_color1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"supervillana_color2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"supervillana_color3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"supervillana_color4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"supervillana_color5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"supervillano", "1F9B9-200D-2642-FE0F"},
    {"supervillano_color1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"supervillano_color2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"supervillano_color3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"supervillano_color4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"supervillano_color5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"mujer_de_pie", "1F9CD-200D-2640-FE0F"},
    {"mujer_de_pie_color1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"mujer_de_pie_color2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"mujer_de_pie_color3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"mujer_de_pie_color4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"mujer_de_pie_color5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"hombre_de_pie", "1F9CD-200D-2642-FE0F"},
    {"hombre_de_pie_color1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"hombre_de_pie_color2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"hombre_de_pie_color3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"hombre_de_pie_color4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"hombre_de_pie_color5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"mujer_de_rodillas", "1F9CE-200D-2640-FE0F"},
    {"mujer_de_rodillas_color1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"mujer_de_rodillas_color2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"mujer_de_rodillas_color3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"mujer_de_rodillas_color4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"mujer_de_rodillas_color5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"mujer_de_rodillas_hacia_la_derecha", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_de_rodillas_hacia_la_derecha_color1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_de_rodillas_hacia_la_derecha_color2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_de_rodillas_hacia_la_derecha_color3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_de_rodillas_hacia_la_derecha_color4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mujer_de_rodillas_hacia_la_derecha_color5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"hombre_de_rodillas", "1F9CE-200D-2642-FE0F"},
    {"hombre_de_rodillas_color1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"hombre_de_rodillas_color2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"hombre_de_rodillas_color3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"hombre_de_rodillas_color4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"hombre_de_rodillas_color5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"hombre_de_rodillas_hacia_la_derecha", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_de_rodillas_hacia_la_derecha_color1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_de_rodillas_hacia_la_derecha_color2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_de_rodillas_hacia_la_derecha_color3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_de_rodillas_hacia_la_derecha_color4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hombre_de_rodillas_hacia_la_derecha_color5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mujer_sorda", "1F9CF-200D-2640-FE0F"},
    {"mujer_sorda_color1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"mujer_sorda_color2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"mujer_sorda_color3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"mujer_sorda_color4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"mujer_sorda_color5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"hombre_sordo", "1F9CF-200D-2642-FE0F"},
    {"hombre_sordo_color1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"hombre_sordo_color2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"hombre_sordo_color3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"hombre_sordo_color4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"hombre_sordo_color5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"mujer_barba", "1F9D4-200D-2640-FE0F"},
    {"mujer_barba_color1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"mujer_barba_color2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"mujer_barba_color3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"mujer_barba_color4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"mujer_barba_color5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"hombre_barba", "1F9D4-200D-2642-FE0F"},
    {"hombre_barba_color1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"hombre_barba_color2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"hombre_barba_color3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"hombre_barba_color4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"hombre_barba_color5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"mujer_en_una_sauna", "1F9D6-200D-2640-FE0F"},
    {"mujer_en_una_sauna_color1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"mujer_en_una_sauna_color2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"mujer_en_una_sauna_color3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"mujer_en_una_sauna_color4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"mujer_en_una_sauna_color5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"hombre_en_una_sauna", "1F9D6-200D-2642-FE0F"},
    {"hombre_en_una_sauna_color1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"hombre_en_una_sauna_color2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"hombre_en_una_sauna_color3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"hombre_en_una_sauna_color4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"hombre_en_una_sauna_color5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"mujer_escalando", "1F9D7-200D-2640-FE0F"},
    {"mujer_escalando_color1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"mujer_escalando_color2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"mujer_escalando_color3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"mujer_escalando_color4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"mujer_escalando_color5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"hombre_escalando", "1F9D7-200D-2642-FE0F"},
    {"hombre_escalando_color1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"hombre_escalando_color2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"hombre_escalando_color3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"hombre_escalando_color4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"hombre_escalando_color5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"mujer_en_posicion_de_loto", "1F9D8-200D-2640-FE0F"},
    {"mujer_en_posicion_de_loto_color1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"mujer_en_posicion_de_loto_color2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"mujer_en_posicion_de_loto_color3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"mujer_en_posicion_de_loto_color4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"mujer_en_posicion_de_loto_color5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"hombre_en_posicion_de_loto", "1F9D8-200D-2642-FE0F"},
    {"hombre_en_posicion_de_loto_color1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"hombre_en_posicion_de_loto_color2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"hombre_en_posicion_de_loto_color3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"hombre_en_posicion_de_loto_color4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"hombre_en_posicion_de_loto_color5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"maga", "1F9D9-200D-2640-FE0F"},
    {"maga_color1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"maga_color2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"maga_color3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"maga_color4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"maga_color5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"mago", "1F9D9-200D-2642-FE0F"},
    {"mago_color1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"mago_color2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"mago_color3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"mago_color4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"mago_color5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"hada_mujer", "1F9DA-200D-2640-FE0F"},
    {"hada_mujer_color1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"hada_mujer_color2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"hada_mujer_color3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"hada_mujer_color4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"hada_mujer_color5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"hada_hombre", "1F9DA-200D-2642-FE0F"},
    {"hada_hombre_color1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"hada_hombre_color2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"hada_hombre_color3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"hada_hombre_color4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"hada_hombre_color5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"vampiresa", "1F9DB-200D-2640-FE0F"},
    {"vampiresa_color1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"vampiresa_color2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"vampiresa_color3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"vampiresa_color4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"vampiresa_color5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"vampiro_hombre", "1F9DB-200D-2642-FE0F"},
    {"vampiro_hombre_color1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"vampiro_hombre_color2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"vampiro_hombre_color3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"vampiro_hombre_color4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"vampiro_hombre_color5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"sirena", "1F9DC-200D-2640-FE0F"},
    {"sirena_color1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"sirena_color2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"sirena_color3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"sirena_color4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"sirena_color5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"sirena_hombre", "1F9DC-200D-2642-FE0F"},
    {"sirena_hombre_color1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"sirena_hombre_color2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"sirena_hombre_color3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"sirena_hombre_color4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"sirena_hombre_color5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"elfa", "1F9DD-200D-2640-FE0F"},
    {"elfa_color1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"elfa_color2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"elfa_color3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"elfa_color4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"elfa_color5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"elfo_hombre", "1F9DD-200D-2642-FE0F"},
    {"elfo_hombre_color1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"elfo_hombre_color2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"elfo_hombre_color3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"elfo_hombre_color4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"elfo_hombre_color5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"genio_mujer", "1F9DE-200D-2640-FE0F"},
    {"genio_hombre", "1F9DE-200D-2642-FE0F"},
    {"zombi_mujer", "1F9DF-200D-2640-FE0F"},
    {"zombi_hombre", "1F9DF-200D-2642-FE0F"},
    {"hombre_pelo_pelirrojo", "1F468-200D-1F9B0"},
    {"hombre_pelo_pelirrojo_color1", "1F468-1F3FB-200D-1F9B0"},
    {"hombre_pelo_pelirrojo_color2", "1F468-1F3FC-200D-1F9B0"},
    {"hombre_pelo_pelirrojo_color3", "1F468-1F3FD-200D-1F9B0"},
    {"hombre_pelo_pelirrojo_color4", "1F468-1F3FE-200D-1F9B0"},
    {"hombre_pelo_pelirrojo_color5", "1F468-1F3FF-200D-1F9B0"},
    {"hombre_pelo_rizado", "1F468-200D-1F9B1"},
    {"hombre_pelo_rizado_color1", "1F468-1F3FB-200D-1F9B1"},
    {"hombre_pelo_rizado_color2", "1F468-1F3FC-200D-1F9B1"},
    {"hombre_pelo_rizado_color3", "1F468-1F3FD-200D-1F9B1"},
    {"hombre_pelo_rizado_color4", "1F468-1F3FE-200D-1F9B1"},
    {"hombre_pelo_rizado_color5", "1F468-1F3FF-200D-1F9B1"},
    {"hombre_sin_pelo", "1F468-200D-1F9B2"},
    {"hombre_sin_pelo_color1", "1F468-1F3FB-200D-1F9B2"},
    {"hombre_sin_pelo_color2", "1F468-1F3FC-200D-1F9B2"},
    {"hombre_sin_pelo_color3", "1F468-1F3FD-200D-1F9B2"},
    {"hombre_sin_pelo_color4", "1F468-1F3FE-200D-1F9B2"},
    {"hombre_sin_pelo_color5", "1F468-1F3FF-200D-1F9B2"},
    {"hombre_pelo_blanco", "1F468-200D-1F9B3"},
    {"hombre_pelo_blanco_color1", "1F468-1F3FB-200D-1F9B3"},
    {"hombre_pelo_blanco_color2", "1F468-1F3FC-200D-1F9B3"},
    {"hombre_pelo_blanco_color3", "1F468-1F3FD-200D-1F9B3"},
    {"hombre_pelo_blanco_color4", "1F468-1F3FE-200D-1F9B3"},
    {"hombre_pelo_blanco_color5", "1F468-1F3FF-200D-1F9B3"},
    {"mujer_pelo_pelirrojo", "1F469-200D-1F9B0"},
    {"mujer_pelo_pelirrojo_color1", "1F469-1F3FB-200D-1F9B0"},
    {"mujer_pelo_pelirrojo_color2", "1F469-1F3FC-200D-1F9B0"},
    {"mujer_pelo_pelirrojo_color3", "1F469-1F3FD-200D-1F9B0"},
    {"mujer_pelo_pelirrojo_color4", "1F469-1F3FE-200D-1F9B0"},
    {"mujer_pelo_pelirrojo_color5", "1F469-1F3FF-200D-1F9B0"},
    {"mujer_pelo_rizado", "1F469-200D-1F9B1"},
    {"mujer_pelo_rizado_color1", "1F469-1F3FB-200D-1F9B1"},
    {"mujer_pelo_rizado_color2", "1F469-1F3FC-200D-1F9B1"},
    {"mujer_pelo_rizado_color3", "1F469-1F3FD-200D-1F9B1"},
    {"mujer_pelo_rizado_color4", "1F469-1F3FE-200D-1F9B1"},
    {"mujer_pelo_rizado_color5", "1F469-1F3FF-200D-1F9B1"},
    {"mujer_sin_pelo", "1F469-200D-1F9B2"},
    {"mujer_sin_pelo_color1", "1F469-1F3FB-200D-1F9B2"},
    {"mujer_sin_pelo_color2", "1F469-1F3FC-200D-1F9B2"},
    {"mujer_sin_pelo_color3", "1F469-1F3FD-200D-1F9B2"},
    {"mujer_sin_pelo_color4", "1F469-1F3FE-200D-1F9B2"},
    {"mujer_sin_pelo_color5", "1F469-1F3FF-200D-1F9B2"},
    {"mujer_pelo_blanco", "1F469-200D-1F9B3"},
    {"mujer_pelo_blanco_color1", "1F469-1F3FB-200D-1F9B3"},
    {"mujer_pelo_blanco_color2", "1F469-1F3FC-200D-1F9B3"},
    {"mujer_pelo_blanco_color3", "1F469-1F3FD-200D-1F9B3"},
    {"mujer_pelo_blanco_color4", "1F469-1F3FE-200D-1F9B3"},
    {"mujer_pelo_blanco_color5", "1F469-1F3FF-200D-1F9B3"},
    {"persona_adulta_pelo_pelirrojo", "1F9D1-200D-1F9B0"},
    {"persona_adulta_pelo_pelirrojo_color1", "1F9D1-1F3FB-200D-1F9B0"},
    {"persona_adulta_pelo_pelirrojo_color2", "1F9D1-1F3FC-200D-1F9B0"},
    {"persona_adulta_pelo_pelirrojo_color3", "1F9D1-1F3FD-200D-1F9B0"},
    {"persona_adulta_pelo_pelirrojo_color4", "1F9D1-1F3FE-200D-1F9B0"},
    {"persona_adulta_pelo_pelirrojo_color5", "1F9D1-1F3FF-200D-1F9B0"},
    {"persona_adulta_pelo_rizado", "1F9D1-200D-1F9B1"},
    {"persona_adulta_pelo_rizado_color1", "1F9D1-1F3FB-200D-1F9B1"},
    {"persona_adulta_pelo_rizado_color2", "1F9D1-1F3FC-200D-1F9B1"},
    {"persona_adulta_pelo_rizado_color3", "1F9D1-1F3FD-200D-1F9B1"},
    {"persona_adulta_pelo_rizado_color4", "1F9D1-1F3FE-200D-1F9B1"},
    {"persona_adulta_pelo_rizado_color5", "1F9D1-1F3FF-200D-1F9B1"},
    {"persona_adulta_sin_pelo", "1F9D1-200D-1F9B2"},
    {"persona_adulta_sin_pelo_color1", "1F9D1-1F3FB-200D-1F9B2"},
    {"persona_adulta_sin_pelo_color2", "1F9D1-1F3FC-200D-1F9B2"},
    {"persona_adulta_sin_pelo_color3", "1F9D1-1F3FD-200D-1F9B2"},
    {"persona_adulta_sin_pelo_color4", "1F9D1-1F3FE-200D-1F9B2"},
    {"persona_adulta_sin_pelo_color5", "1F9D1-1F3FF-200D-1F9B2"},
    {"persona_adulta_pelo_blanco", "1F9D1-200D-1F9B3"},
    {"persona_adulta_pelo_blanco_color1", "1F9D1-1F3FB-200D-1F9B3"},
    {"persona_adulta_pelo_blanco_color2", "1F9D1-1F3FC-200D-1F9B3"},
    {"persona_adulta_pelo_blanco_color3", "1F9D1-1F3FD-200D-1F9B3"},
    {"persona_adulta_pelo_blanco_color4", "1F9D1-1F3FE-200D-1F9B3"},
    {"persona_adulta_pelo_blanco_color5", "1F9D1-1F3FF-200D-1F9B3"},
    {"cadena_rota", "26D3-FE0F-200D-1F4A5"},
    {"corazon_en_llamas", "2764-FE0F-200D-1F525"},
    {"corazon_vendado", "2764-FE0F-200D-1FA79"},
    {"champinon_marron", "1F344-200D-1F7EB"},
    {"lima", "1F34B-200D-1F7E9"},
    {"bandera_transgenero", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"bandera_del_arcoiris", "1F3F3-FE0F-200D-1F308"},
    {"bandera_pirata", "1F3F4-200D-2620-FE0F"},
    {"gato_negro", "1F408-200D-2B1B"},
    {"perro_de_servicio", "1F415-200D-1F9BA"},
    {"pajaro_negro", "1F426-200D-2B1B"},
    {"fenix", "1F426-200D-1F525"},
    {"oso_polar", "1F43B-200D-2744-FE0F"},
    {"ojo_en_bocadillo_de_texto", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"cara_exhalando", "1F62E-200D-1F4A8"},
    {"cara_con_ojos_en_espiral", "1F635-200D-1F4AB"},
    {"cara_entre_las_nubes", "1F636-200D-1F32B-FE0F"},
    {"cabeza_negando", "1F642-200D-2194-FE0F"},
    {"cabeza_asintiendo", "1F642-200D-2195-FE0F"},
    {"bailarina_de_ballet", "1F9D1-200D-1FA70"},
    {"bailarina_de_ballet_color1", "1F9D1-1F3FB-200D-1FA70"},
    {"bailarina_de_ballet_color2", "1F9D1-1F3FC-200D-1FA70"},
    {"bailarina_de_ballet_color3", "1F9D1-1F3FD-200D-1FA70"},
    {"bailarina_de_ballet_color4", "1F9D1-1F3FE-200D-1FA70"},
    {"bailarina_de_ballet_color5", "1F9D1-1F3FF-200D-1FA70"},
    {"exclamación_e_interrogación", "2049"},
    {"símbolo_de_marca_comercial", "2122"},
    {"información", "2139"},
    {"muñeco_de_nieve_con_nieve", "2603"},
    {"casilla_con_marca_de_verificación", "2611"},
    {"trébol", "2618"},
    {"riesgo_biológico", "2623"},
    {"cara_con_el_ceño_fruncido", "2639"},
    {"palo_de_tréboles", "2663"},
    {"símbolo_de_medicina", "2695"},
    {"botón_de_marca_de_verificación", "2705"},
    {"avión", "2708"},
    {"marca_de_verificación", "2714"},
    {"multiplicación", "2716"},
    {"interrogación_roja", "2753"},
    {"interrogación_blanca", "2754"},
    {"exclamación_blanca", "2755"},
    {"exclamación_roja", "2757"},
    {"exclamación_de_corazón", "2763"},
    {"corazón_rojo", "2764"},
    {"más", "2795"},
    {"división", "2797"},
    {"flecha_derecha_curvándose_hacia_arriba", "2934"},
    {"flecha_derecha_curvándose_hacia_abajo", "2935"},
    {"ideograma_japonés_para_enhorabuena", "3297"},
    {"ideograma_japonés_para_secreto", "3299"},
    {"exclamación_doble", "203C"},
    {"flecha_derecha_curvándose_a_la_izquierda", "21A9"},
    {"flecha_izquierda_curvándose_a_la_derecha", "21AA"},
    {"avance_rápido", "23E9"},
    {"retroceso_rápido", "23EA"},
    {"triángulo_doble_hacia_arriba", "23EB"},
    {"triángulo_doble_hacia_abajo", "23EC"},
    {"cronómetro", "23F1"},
    {"m_en_círculo", "24C2"},
    {"cuadrado_negro_pequeño", "25AA"},
    {"cuadrado_blanco_pequeño", "25AB"},
    {"cuadrado_blanco_mediano_pequeño", "25FD"},
    {"cuadrado_negro_mediano_pequeño", "25FE"},
    {"teléfono", "260E"},
    {"dedo_índice_hacia_arriba", "261D"},
    {"dedo_índice_hacia_arriba_color1", "261D-1F3FB"},
    {"dedo_índice_hacia_arriba_color2", "261D-1F3FC"},
    {"dedo_índice_hacia_arriba_color3", "261D-1F3FD"},
    {"dedo_índice_hacia_arriba_color4", "261D-1F3FE"},
    {"dedo_índice_hacia_arriba_color5", "261D-1F3FF"},
    {"símbolo_de_la_paz", "262E"},
    {"géminis", "264A"},
    {"cáncer", "264B"},
    {"peón_de_ajedrez", "265F"},
    {"símbolo_de_reciclaje", "267B"},
    {"símbolo_de_silla_de_ruedas", "267F"},
    {"símbolo_de_átomo", "269B"},
    {"símbolo_de_transgénero", "26A7"},
    {"círculo_blanco", "26AA"},
    {"círculo_negro", "26AB"},
    {"ataúd", "26B0"},
    {"balón_de_fútbol", "26BD"},
    {"béisbol", "26BE"},
    {"muñeco_de_nieve", "26C4"},
    {"sol_detrás_de_una_nube", "26C5"},
    {"dirección_prohibida", "26D4"},
    {"santuario_sintoísta", "26E9"},
    {"montaña", "26F0"},
    {"banderín_en_hoyo", "26F3"},
    {"patín_de_hielo", "26F8"},
    {"persona_botando_un_balón", "26F9"},
    {"persona_botando_un_balón_color1", "26F9-1F3FB"},
    {"persona_botando_un_balón_color2", "26F9-1F3FC"},
    {"persona_botando_un_balón_color3", "26F9-1F3FD"},
    {"persona_botando_un_balón_color4", "26F9-1F3FE"},
    {"persona_botando_un_balón_color5", "26F9-1F3FF"},
    {"tienda_de_campaña", "26FA"},
    {"puño_en_alto", "270A"},
    {"puño_en_alto_color1", "270A-1F3FB"},
    {"puño_en_alto_color2", "270A-1F3FC"},
    {"puño_en_alto_color3", "270A-1F3FD"},
    {"puño_en_alto_color4", "270A-1F3FE"},
    {"puño_en_alto_color5", "270A-1F3FF"},
    {"mano_con_señal_de_victoria", "270C"},
    {"mano_con_señal_de_victoria_color1", "270C-1F3FB"},
    {"mano_con_señal_de_victoria_color2", "270C-1F3FC"},
    {"mano_con_señal_de_victoria_color3", "270C-1F3FD"},
    {"mano_con_señal_de_victoria_color4", "270C-1F3FE"},
    {"mano_con_señal_de_victoria_color5", "270C-1F3FF"},
    {"lápiz", "270F"},
    {"botón_con_marca_de_cruz", "274E"},
    {"círculo_rojo_hueco", "2B55"},
    {"dragón_rojo_de_mahjong", "1F004"},
    {"comodín", "1F0CF"},
    {"grupo_sanguíneo_a", "1F170"},
    {"grupo_sanguíneo_b", "1F171"},
    {"grupo_sanguíneo_tipo_o", "1F17E"},
    {"grupo_sanguíneo_ab", "1F18E"},
    {"botón_cool", "1F192"},
    {"botón_free", "1F193"},
    {"símbolo_de_identificación", "1F194"},
    {"botón_new", "1F195"},
    {"botón_ng", "1F196"},
    {"botón_ok", "1F197"},
    {"símbolo_de_socorro", "1F198"},
    {"botón_up", "1F199"},
    {"botón_vs", "1F19A"},
    {"ideograma_japonés_para_aquí", "1F201"},
    {"ideograma_japonés_para_de_cortesía", "1F202"},
    {"ideograma_japonés_para_gratis", "1F21A"},
    {"ideograma_japonés_para_reservado", "1F22F"},
    {"ideograma_japonés_para_prohibido", "1F232"},
    {"ideograma_japonés_para_vacante", "1F233"},
    {"ideograma_japonés_para_aprobado", "1F234"},
    {"ideograma_japonés_para_completo", "1F235"},
    {"ideograma_japonés_para_de_pago", "1F236"},
    {"ideograma_japonés_para_cantidad_mensual", "1F237"},
    {"ideograma_japonés_para_aplicación", "1F238"},
    {"ideograma_japonés_para_descuento", "1F239"},
    {"ideograma_japonés_para_abierto", "1F23A"},
    {"ideograma_japonés_para_ganga", "1F250"},
    {"ideograma_japonés_para_aceptable", "1F251"},
    {"ciclón", "1F300"},
    {"amanecer_sobre_montañas", "1F304"},
    {"arcoíris", "1F308"},
    {"volcán", "1F30B"},
    {"vía_láctea", "1F30C"},
    {"globo_terráqueo_mostrando_europa_y_áfrica", "1F30D"},
    {"globo_terráqueo_mostrando_américa", "1F30E"},
    {"globo_terráqueo_mostrando_asia_y_australia", "1F30F"},
    {"globo_terráqueo_con_meridianos", "1F310"},
    {"termómetro", "1F321"},
    {"sol_detrás_de_una_nube_pequeña", "1F324"},
    {"sol_detrás_de_una_nube_grande", "1F325"},
    {"sol_detrás_de_una_nube_con_lluvia", "1F326"},
    {"castaña", "1F330"},
    {"árbol_de_hoja_perenne", "1F332"},
    {"árbol_de_hoja_caduca", "1F333"},
    {"tulipán", "1F337"},
    {"espiga_de_maíz", "1F33D"},
    {"trébol_de_cuatro_hojas", "1F340"},
    {"hojas_caídas", "1F342"},
    {"champiñón", "1F344"},
    {"melón", "1F348"},
    {"sandía", "1F349"},
    {"limón", "1F34B"},
    {"plátano", "1F34C"},
    {"piña", "1F34D"},
    {"melocotón", "1F351"},
    {"tazón_de_fideos", "1F35C"},
    {"pastel_de_pescado_japonés", "1F365"},
    {"dónut", "1F369"},
    {"tazón_de_té", "1F375"},
    {"copa_de_cóctel", "1F378"},
    {"biberón", "1F37C"},
    {"tarta_de_cumpleaños", "1F382"},
    {"árbol_de_navidad", "1F384"},
    {"papá_noel", "1F385"},
    {"papá_noel_color1", "1F385-1F3FB"},
    {"papá_noel_color2", "1F385-1F3FC"},
    {"papá_noel_color3", "1F385-1F3FD"},
    {"papá_noel_color4", "1F385-1F3FE"},
    {"papá_noel_color5", "1F385-1F3FF"},
    {"cañón_de_confeti", "1F389"},
    {"árbol_de_tanabata", "1F38B"},
    {"decoración_de_pino", "1F38D"},
    {"muñecas_japonesas", "1F38E"},
    {"banderín_de_carpas", "1F38F"},
    {"ceremonia_de_contemplación_de_la_luna", "1F391"},
    {"micrófono_de_estudio", "1F399"},
    {"fotograma_de_película", "1F39E"},
    {"montaña_rusa", "1F3A2"},
    {"caña_de_pescar", "1F3A3"},
    {"micrófono", "1F3A4"},
    {"cámara_de_cine", "1F3A5"},
    {"máscaras_de_teatro", "1F3AD"},
    {"máquina_tragaperras", "1F3B0"},
    {"saxofón", "1F3B7"},
    {"violín", "1F3BB"},
    {"esquís", "1F3BF"},
    {"balón_de_baloncesto", "1F3C0"},
    {"balón_de_fútbol_americano", "1F3C8"},
    {"balón_de_rugby", "1F3C9"},
    {"críquet", "1F3CF"},
    {"montaña_con_nieve", "1F3D4"},
    {"construcción", "1F3D7"},
    {"edificio_clásico", "1F3DB"},
    {"casa_con_jardín", "1F3E1"},
    {"señal_de_cajero_automático", "1F3E7"},
    {"fábrica", "1F3ED"},
    {"lámpara_japonesa", "1F3EE"},
    {"castillo_japonés", "1F3EF"},
    {"bádminton", "1F3F8"},
    {"ánfora", "1F3FA"},
    {"ratón", "1F401"},
    {"búfalo_de_agua", "1F403"},
    {"dragón", "1F409"},
    {"jabalí", "1F417"},
    {"pollito_rompiendo_el_cascarón", "1F423"},
    {"pájaro", "1F426"},
    {"pingüino", "1F427"},
    {"delfín", "1F42C"},
    {"cara_de_ratón", "1F42D"},
    {"cara_de_dragón", "1F432"},
    {"hámster", "1F439"},
    {"huellas_de_pezuñas", "1F43E"},
    {"dorso_de_mano_con_índice_hacia_arriba", "1F446"},
    {"dorso_de_mano_con_índice_hacia_arriba_color1", "1F446-1F3FB"},
    {"dorso_de_mano_con_índice_hacia_arriba_color2", "1F446-1F3FC"},
    {"dorso_de_mano_con_índice_hacia_arriba_color3", "1F446-1F3FD"},
    {"dorso_de_mano_con_índice_hacia_arriba_color4", "1F446-1F3FE"},
    {"dorso_de_mano_con_índice_hacia_arriba_color5", "1F446-1F3FF"},
    {"dorso_de_mano_con_índice_hacia_abajo", "1F447"},
    {"dorso_de_mano_con_índice_hacia_abajo_color1", "1F447-1F3FB"},
    {"dorso_de_mano_con_índice_hacia_abajo_color2", "1F447-1F3FC"},
    {"dorso_de_mano_con_índice_hacia_abajo_color3", "1F447-1F3FD"},
    {"dorso_de_mano_con_índice_hacia_abajo_color4", "1F447-1F3FE"},
    {"dorso_de_mano_con_índice_hacia_abajo_color5", "1F447-1F3FF"},
    {"dorso_de_mano_con_índice_a_la_izquierda", "1F448"},
    {"dorso_de_mano_con_índice_a_la_izquierda_color1", "1F448-1F3FB"},
    {"dorso_de_mano_con_índice_a_la_izquierda_color2", "1F448-1F3FC"},
    {"dorso_de_mano_con_índice_a_la_izquierda_color3", "1F448-1F3FD"},
    {"dorso_de_mano_con_índice_a_la_izquierda_color4", "1F448-1F3FE"},
    {"dorso_de_mano_con_índice_a_la_izquierda_color5", "1F448-1F3FF"},
    {"dorso_de_mano_con_índice_a_la_derecha", "1F449"},
    {"dorso_de_mano_con_índice_a_la_derecha_color1", "1F449-1F3FB"},
    {"dorso_de_mano_con_índice_a_la_derecha_color2", "1F449-1F3FC"},
    {"dorso_de_mano_con_índice_a_la_derecha_color3", "1F449-1F3FD"},
    {"dorso_de_mano_con_índice_a_la_derecha_color4", "1F449-1F3FE"},
    {"dorso_de_mano_con_índice_a_la_derecha_color5", "1F449-1F3FF"},
    {"puño_cerrado", "1F44A"},
    {"puño_cerrado_color1", "1F44A-1F3FB"},
    {"puño_cerrado_color2", "1F44A-1F3FC"},
    {"puño_cerrado_color3", "1F44A-1F3FD"},
    {"puño_cerrado_color4", "1F44A-1F3FE"},
    {"puño_cerrado_color5", "1F44A-1F3FF"},
    {"señal_de_aprobación_con_la_mano", "1F44C"},
    {"señal_de_aprobación_con_la_mano_color1", "1F44C-1F3FB"},
    {"señal_de_aprobación_con_la_mano_color2", "1F44C-1F3FC"},
    {"señal_de_aprobación_con_la_mano_color3", "1F44C-1F3FD"},
    {"señal_de_aprobación_con_la_mano_color4", "1F44C-1F3FE"},
    {"señal_de_aprobación_con_la_mano_color5", "1F44C-1F3FF"},
    {"zapato_de_tacón", "1F460"},
    {"niño", "1F466"},
    {"niño_color1", "1F466-1F3FB"},
    {"niño_color2", "1F466-1F3FC"},
    {"niño_color3", "1F466-1F3FD"},
    {"niño_color4", "1F466-1F3FE"},
    {"niño_color5", "1F466-1F3FF"},
    {"niña", "1F467"},
    {"niña_color1", "1F467-1F3FB"},
    {"niña_color2", "1F467-1F3FC"},
    {"niña_color3", "1F467-1F3FD"},
    {"niña_color4", "1F467-1F3FE"},
    {"niña_color5", "1F467-1F3FF"},
    {"agente_de_policía", "1F46E"},
    {"agente_de_policía_color1", "1F46E-1F3FB"},
    {"agente_de_policía_color2", "1F46E-1F3FC"},
    {"agente_de_policía_color3", "1F46E-1F3FD"},
    {"agente_de_policía_color4", "1F46E-1F3FE"},
    {"agente_de_policía_color5", "1F46E-1F3FF"},
    {"bebé", "1F476"},
    {"bebé_color1", "1F476-1F3FB"},
    {"bebé_color2", "1F476-1F3FC"},
    {"bebé_color3", "1F476-1F3FD"},
    {"bebé_color4", "1F476-1F3FE"},
    {"bebé_color5", "1F476-1F3FF"},
    {"profesional_de_la_construcción", "1F477"},
    {"profesional_de_la_construcción_color1", "1F477-1F3FB"},
    {"profesional_de_la_construcción_color2", "1F477-1F3FC"},
    {"profesional_de_la_construcción_color3", "1F477-1F3FD"},
    {"profesional_de_la_construcción_color4", "1F477-1F3FE"},
    {"profesional_de_la_construcción_color5", "1F477-1F3FF"},
    {"demonio_japonés_oni", "1F479"},
    {"demonio_japonés_tengu", "1F47A"},
    {"bebé_ángel", "1F47C"},
    {"bebé_ángel_color1", "1F47C-1F3FB"},
    {"bebé_ángel_color2", "1F47C-1F3FC"},
    {"bebé_ángel_color3", "1F47C-1F3FD"},
    {"bebé_ángel_color4", "1F47C-1F3FE"},
    {"bebé_ángel_color5", "1F47C-1F3FF"},
    {"alienígena", "1F47D"},
    {"monstruo_alienígena", "1F47E"},
    {"persona_de_mostrador_de_información", "1F481"},
    {"persona_de_mostrador_de_información_color1", "1F481-1F3FB"},
    {"persona_de_mostrador_de_información_color2", "1F481-1F3FC"},
    {"persona_de_mostrador_de_información_color3", "1F481-1F3FD"},
    {"persona_de_mostrador_de_información_color4", "1F481-1F3FE"},
    {"persona_de_mostrador_de_información_color5", "1F481-1F3FF"},
    {"pintarse_las_uñas", "1F485"},
    {"pintarse_las_uñas_color1", "1F485-1F3FB"},
    {"pintarse_las_uñas_color2", "1F485-1F3FC"},
    {"pintarse_las_uñas_color3", "1F485-1F3FD"},
    {"pintarse_las_uñas_color4", "1F485-1F3FE"},
    {"pintarse_las_uñas_color5", "1F485-1F3FF"},
    {"persona_cortándose_el_pelo", "1F487"},
    {"persona_cortándose_el_pelo_color1", "1F487-1F3FB"},
    {"persona_cortándose_el_pelo_color2", "1F487-1F3FC"},
    {"persona_cortándose_el_pelo_color3", "1F487-1F3FD"},
    {"persona_cortándose_el_pelo_color4", "1F487-1F3FE"},
    {"persona_cortándose_el_pelo_color5", "1F487-1F3FF"},
    {"píldora", "1F48A"},
    {"corazón_latiendo", "1F493"},
    {"corazón_roto", "1F494"},
    {"corazón_brillante", "1F496"},
    {"corazón_creciente", "1F497"},
    {"corazón_con_flecha", "1F498"},
    {"corazón_azul", "1F499"},
    {"corazón_verde", "1F49A"},
    {"corazón_amarillo", "1F49B"},
    {"corazón_morado", "1F49C"},
    {"corazón_con_lazo", "1F49D"},
    {"adorno_de_corazón", "1F49F"},
    {"rombo_con_pétalo", "1F4A0"},
    {"símbolo_de_enfado", "1F4A2"},
    {"símbolo_de_sueño", "1F4A4"},
    {"colisión", "1F4A5"},
    {"bíceps_flexionado", "1F4AA"},
    {"bíceps_flexionado_color1", "1F4AA-1F3FB"},
    {"bíceps_flexionado_color2", "1F4AA-1F3FC"},
    {"bíceps_flexionado_color3", "1F4AA-1F3FD"},
    {"bíceps_flexionado_color4", "1F4AA-1F3FE"},
    {"bíceps_flexionado_color5", "1F4AA-1F3FF"},
    {"símbolo_de_mareo", "1F4AB"},
    {"bocadillo_de_diálogo", "1F4AC"},
    {"símbolo_de_dólar", "1F4B2"},
    {"tarjeta_de_crédito", "1F4B3"},
    {"billete_de_dólar", "1F4B5"},
    {"ordenador_portátil", "1F4BB"},
    {"maletín", "1F4BC"},
    {"disco_óptico", "1F4BF"},
    {"página_doblada", "1F4C3"},
    {"página_hacia_arriba", "1F4C4"},
    {"gráfica_de_evolución_ascendente", "1F4C8"},
    {"gráfica_de_evolución_descendente", "1F4C9"},
    {"gráfico_de_barras", "1F4CA"},
    {"auricular_de_teléfono", "1F4DE"},
    {"máquina_de_fax", "1F4E0"},
    {"antena_de_satélite", "1F4E1"},
    {"megáfono", "1F4E3"},
    {"correo_electrónico", "1F4E7"},
    {"buzón_cerrado_con_la_bandera_bajada", "1F4EA"},
    {"buzón_cerrado_con_la_bandera_levantada", "1F4EB"},
    {"buzón_abierto_con_la_bandera_levantada", "1F4EC"},
    {"buzón_abierto_con_la_bandera_bajada", "1F4ED"},
    {"buzón", "1F4EE"},
    {"periódico", "1F4F0"},
    {"teléfono_móvil", "1F4F1"},
    {"móvil_con_una_flecha", "1F4F2"},
    {"modo_vibración", "1F4F3"},
    {"teléfono_móvil_apagado", "1F4F4"},
    {"prohibido_el_uso_de_móviles", "1F4F5"},
    {"cámara_de_fotos", "1F4F7"},
    {"cámara_con_flash", "1F4F8"},
    {"videocámara", "1F4F9"},
    {"televisión", "1F4FA"},
    {"cinta_de_vídeo", "1F4FC"},
    {"reproducción_aleatoria", "1F500"},
    {"enchufe_eléctrico", "1F50C"},
    {"candado_con_pluma_estilográfica", "1F50F"},
    {"campana_con_signo_de_cancelación", "1F515"},
    {"marcapáginas", "1F516"},
    {"eslabón", "1F517"},
    {"botón_de_opción", "1F518"},
    {"prohibido_para_menos_de_18_años", "1F51E"},
    {"letras_latinas_mayúsculas", "1F520"},
    {"letras_latinas_minúsculas", "1F521"},
    {"números", "1F522"},
    {"símbolos", "1F523"},
    {"símbolo_japonés_para_principiante", "1F530"},
    {"botón_cuadrado_con_borde_negro", "1F532"},
    {"botón_cuadrado_con_borde_blanco", "1F533"},
    {"círculo_rojo", "1F534"},
    {"círculo_azul", "1F535"},
    {"rombo_naranja_pequeño", "1F538"},
    {"rombo_azul_pequeño", "1F539"},
    {"triángulo_rojo_hacia_arriba", "1F53A"},
    {"triángulo_rojo_hacia_abajo", "1F53B"},
    {"triángulo_hacia_arriba", "1F53C"},
    {"triángulo_hacia_abajo", "1F53D"},
    {"menorá", "1F54E"},
    {"araña", "1F577"},
    {"tela_de_araña", "1F578"},
    {"bolígrafo", "1F58A"},
    {"estilográfica", "1F58B"},
    {"lápiz_de_cera", "1F58D"},
    {"dedo_corazón_hacia_arriba", "1F595"},
    {"dedo_corazón_hacia_arriba_color1", "1F595-1F3FB"},
    {"dedo_corazón_hacia_arriba_color2", "1F595-1F3FC"},
    {"dedo_corazón_hacia_arriba_color3", "1F595-1F3FD"},
    {"dedo_corazón_hacia_arriba_color4", "1F595-1F3FE"},
    {"dedo_corazón_hacia_arriba_color5", "1F595-1F3FF"},
    {"corazón_negro", "1F5A4"},
    {"ratón_de_ordenador", "1F5B1"},
    {"periódico_enrollado", "1F5DE"},
    {"puñal", "1F5E1"},
    {"bocadillo_de_diálogo_por_la_izquierda", "1F5E8"},
    {"mapa_de_japón", "1F5FE"},
    {"estatua_moái", "1F5FF"},
    {"cara_sonriendo_con_sudor_frío", "1F605"},
    {"cara_guiñando_el_ojo", "1F609"},
    {"cara_sonriendo_con_ojos_de_corazón", "1F60D"},
    {"cara_sin_expresión", "1F611"},
    {"cara_de_desaprobación", "1F612"},
    {"cara_con_sudor_frío", "1F613"},
    {"cara_de_confusión", "1F615"},
    {"cara_de_frustración", "1F616"},
    {"cara_sacando_la_lengua_y_guiñando_un_ojo", "1F61C"},
    {"cara_con_el_ceño_fruncido_y_la_boca_abierta", "1F626"},
    {"cara_de_sueño", "1F62A"},
    {"cara_con_mascarilla_médica", "1F637"},
    {"gato_sonriendo_con_ojos_de_corazón", "1F63B"},
    {"cara_con_el_ceño_ligeramente_fruncido", "1F641"},
    {"cara_al_revés", "1F643"},
    {"mono_con_los_oídos_tapados", "1F649"},
    {"persona_frunciendo_el_ceño", "1F64D"},
    {"persona_frunciendo_el_ceño_color1", "1F64D-1F3FB"},
    {"persona_frunciendo_el_ceño_color2", "1F64D-1F3FC"},
    {"persona_frunciendo_el_ceño_color3", "1F64D-1F3FD"},
    {"persona_frunciendo_el_ceño_color4", "1F64D-1F3FE"},
    {"persona_frunciendo_el_ceño_color5", "1F64D-1F3FF"},
    {"manos_en_oración", "1F64F"},
    {"manos_en_oración_color1", "1F64F-1F3FB"},
    {"manos_en_oración_color2", "1F64F-1F3FC"},
    {"manos_en_oración_color3", "1F64F-1F3FD"},
    {"manos_en_oración_color4", "1F64F-1F3FE"},
    {"manos_en_oración_color5", "1F64F-1F3FF"},
    {"helicóptero", "1F681"},
    {"vagón", "1F683"},
    {"estación_de_tren", "1F689"},
    {"tranvía", "1F68A"},
    {"vagón_de_tranvía", "1F68B"},
    {"autobús", "1F68C"},
    {"autobús_próximo", "1F68D"},
    {"trolebús", "1F68E"},
    {"parada_de_autobús", "1F68F"},
    {"minibús", "1F690"},
    {"coche_de_policía", "1F693"},
    {"coche_de_policía_próximo", "1F694"},
    {"taxi_próximo", "1F696"},
    {"coche_próximo", "1F698"},
    {"vehículo_deportivo_utilitario", "1F699"},
    {"camión_de_reparto", "1F69A"},
    {"camión_articulado", "1F69B"},
    {"monorraíl", "1F69D"},
    {"ferrocarril_de_montaña", "1F69E"},
    {"ferrocarril_de_suspensión", "1F69F"},
    {"teleférico_de_montaña", "1F6A0"},
    {"teleférico", "1F6A1"},
    {"semáforo_horizontal", "1F6A5"},
    {"semáforo", "1F6A6"},
    {"luces_de_policía", "1F6A8"},
    {"señal_de_usar_papelera", "1F6AE"},
    {"persona_en_bicicleta_de_montaña", "1F6B5"},
    {"persona_en_bicicleta_de_montaña_color1", "1F6B5-1F3FB"},
    {"persona_en_bicicleta_de_montaña_color2", "1F6B5-1F3FC"},
    {"persona_en_bicicleta_de_montaña_color3", "1F6B5-1F3FD"},
    {"persona_en_bicicleta_de_montaña_color4", "1F6B5-1F3FE"},
    {"persona_en_bicicleta_de_montaña_color5", "1F6B5-1F3FF"},
    {"niños_cruzando", "1F6B8"},
    {"señal_de_aseo_para_mujeres", "1F6BA"},
    {"señal_de_aseos", "1F6BB"},
    {"señal_de_bebé", "1F6BC"},
    {"persona_en_la_bañera", "1F6C0"},
    {"persona_en_la_bañera_color1", "1F6C0-1F3FB"},
    {"persona_en_la_bañera_color2", "1F6C0-1F3FC"},
    {"persona_en_la_bañera_color3", "1F6C0-1F3FD"},
    {"persona_en_la_bañera_color4", "1F6C0-1F3FE"},
    {"persona_en_la_bañera_color5", "1F6C0-1F3FF"},
    {"bañera", "1F6C1"},
    {"sofá_y_lámpara", "1F6CB"},
    {"señal_de_stop", "1F6D1"},
    {"templo_hindú", "1F6D5"},
    {"cabaña", "1F6D6"},
    {"tobogán", "1F6DD"},
    {"barril_de_petróleo", "1F6E2"},
    {"vía_de_tren", "1F6E4"},
    {"avión_despegando", "1F6EB"},
    {"avión_aterrizando", "1F6EC"},
    {"satélite", "1F6F0"},
    {"monopatín", "1F6F9"},
    {"círculo_naranja", "1F7E0"},
    {"círculo_amarillo", "1F7E1"},
    {"círculo_verde", "1F7E2"},
    {"círculo_morado", "1F7E3"},
    {"círculo_marrón", "1F7E4"},
    {"cuadrado_marrón", "1F7EB"},
    {"corazón_blanco", "1F90D"},
    {"corazón_marrón", "1F90E"},
    {"cara_con_termómetro", "1F912"},
    {"cara_de_empollón", "1F913"},
    {"puño_hacia_la_izquierda", "1F91B"},
    {"puño_hacia_la_izquierda_color1", "1F91B-1F3FB"},
    {"puño_hacia_la_izquierda_color2", "1F91B-1F3FC"},
    {"puño_hacia_la_izquierda_color3", "1F91B-1F3FD"},
    {"puño_hacia_la_izquierda_color4", "1F91B-1F3FE"},
    {"puño_hacia_la_izquierda_color5", "1F91B-1F3FF"},
    {"puño_hacia_la_derecha", "1F91C"},
    {"puño_hacia_la_derecha_color1", "1F91C-1F3FB"},
    {"puño_hacia_la_derecha_color2", "1F91C-1F3FC"},
    {"puño_hacia_la_derecha_color3", "1F91C-1F3FD"},
    {"puño_hacia_la_derecha_color4", "1F91C-1F3FE"},
    {"puño_hacia_la_derecha_color5", "1F91C-1F3FF"},
    {"apretón_de_manos", "1F91D"},
    {"apretón_de_manos_color1", "1F91D-1F3FB"},
    {"apretón_de_manos_color2", "1F91D-1F3FC"},
    {"apretón_de_manos_color3", "1F91D-1F3FD"},
    {"apretón_de_manos_color4", "1F91D-1F3FE"},
    {"apretón_de_manos_color5", "1F91D-1F3FF"},
    {"apretón_de_manos_color1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"apretón_de_manos_color1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"apretón_de_manos_color1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"apretón_de_manos_color1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"apretón_de_manos_color2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"apretón_de_manos_color2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"apretón_de_manos_color2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"apretón_de_manos_color2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"apretón_de_manos_color3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"apretón_de_manos_color3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"apretón_de_manos_color3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"apretón_de_manos_color3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"apretón_de_manos_color4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"apretón_de_manos_color4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"apretón_de_manos_color4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"apretón_de_manos_color4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"apretón_de_manos_color5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"apretón_de_manos_color5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"apretón_de_manos_color5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"apretón_de_manos_color5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"cara_de_náuseas", "1F922"},
    {"cara_revolviéndose_de_la_risa", "1F923"},
    {"cara_con_símbolos_en_la_boca", "1F92C"},
    {"príncipe", "1F934"},
    {"príncipe_color1", "1F934-1F3FB"},
    {"príncipe_color2", "1F934-1F3FC"},
    {"príncipe_color3", "1F934-1F3FD"},
    {"príncipe_color4", "1F934-1F3FE"},
    {"príncipe_color5", "1F934-1F3FF"},
    {"mamá_noel", "1F936"},
    {"mamá_noel_color1", "1F936-1F3FB"},
    {"mamá_noel_color2", "1F936-1F3FC"},
    {"mamá_noel_color3", "1F936-1F3FD"},
    {"mamá_noel_color4", "1F936-1F3FE"},
    {"mamá_noel_color5", "1F936-1F3FF"},
    {"máscara_de_buceo", "1F93F"},
    {"portería", "1F945"},
    {"cruasán", "1F950"},
    {"brócoli", "1F966"},
    {"sándwich", "1F96A"},
    {"cara_sonriente_con_lágrima", "1F972"},
    {"cara_con_frío", "1F976"},
    {"cara_aguantándose_las_lágrimas", "1F979"},
    {"gafas_de_protección", "1F97D"},
    {"león", "1F981"},
    {"escorpión", "1F982"},
    {"águila", "1F985"},
    {"murciélago", "1F987"},
    {"tiburón", "1F988"},
    {"búho", "1F989"},
    {"saurópodo", "1F995"},
    {"hipopótamo", "1F99B"},
    {"tejón", "1F9A1"},
    {"orangután", "1F9A7"},
    {"perro_guía", "1F9AE"},
    {"bastón", "1F9AF"},
    {"personaje_de_superhéroe", "1F9B8"},
    {"personaje_de_superhéroe_color1", "1F9B8-1F3FB"},
    {"personaje_de_superhéroe_color2", "1F9B8-1F3FC"},
    {"personaje_de_superhéroe_color3", "1F9B8-1F3FD"},
    {"personaje_de_superhéroe_color4", "1F9B8-1F3FE"},
    {"personaje_de_superhéroe_color5", "1F9B8-1F3FF"},
    {"oreja_con_audífono", "1F9BB"},
    {"oreja_con_audífono_color1", "1F9BB-1F3FB"},
    {"oreja_con_audífono_color2", "1F9BB-1F3FC"},
    {"oreja_con_audífono_color3", "1F9BB-1F3FD"},
    {"oreja_con_audífono_color4", "1F9BB-1F3FE"},
    {"oreja_con_audífono_color5", "1F9BB-1F3FF"},
    {"silla_de_ruedas_eléctrica", "1F9BC"},
    {"brazo_mecánico", "1F9BE"},
    {"pierna_mecánica", "1F9BF"},
    {"cuña_de_queso", "1F9C0"},
    {"té_de_burbujas", "1F9CB"},
    {"cara_con_monóculo", "1F9D0"},
    {"persona_en_posición_de_loto", "1F9D8"},
    {"persona_en_posición_de_loto_color1", "1F9D8-1F3FB"},
    {"persona_en_posición_de_loto_color2", "1F9D8-1F3FC"},
    {"persona_en_posición_de_loto_color3", "1F9D8-1F3FD"},
    {"persona_en_posición_de_loto_color4", "1F9D8-1F3FE"},
    {"persona_en_posición_de_loto_color5", "1F9D8-1F3FF"},
    {"corazón_naranja", "1F9E1"},
    {"brújula", "1F9ED"},
    {"ábaco", "1F9EE"},
    {"imán", "1F9F2"},
    {"jabón", "1F9FC"},
    {"traje_de_baño_de_una_pieza", "1FA71"},
    {"corazón_azul_claro", "1FA75"},
    {"corazón_gris", "1FA76"},
    {"corazón_rosa", "1FA77"},
    {"radiografía", "1FA7B"},
    {"yoyó", "1FA80"},
    {"paracaídas", "1FA82"},
    {"bumerán", "1FA83"},
    {"varita_mágica", "1FA84"},
    {"piñata", "1FA85"},
    {"muñeca_rusa", "1FA86"},
    {"trombón", "1FA8A"},
    {"lámpara_de_aceite", "1FA94"},
    {"acordeón", "1FA97"},
    {"sierra_de_carpintería", "1FA9A"},
    {"lápida", "1FAA6"},
    {"carné_de_identidad", "1FAAA"},
    {"batería_baja", "1FAAB"},
    {"nido_vacío", "1FAB9"},
    {"árbol_sin_hojas", "1FABE"},
    {"corazón_humano", "1FAC0"},
    {"personas_abrazándose", "1FAC2"},
    {"arándanos", "1FAD0"},
    {"líquido_derramándose", "1FAD7"},
    {"raíz_de_jengibre", "1FADA"},
    {"túberculo_comestible", "1FADC"},
    {"cara_derritiéndose", "1FAE0"},
    {"cara_con_línea_de_puntos", "1FAE5"},
    {"mano_con_dedo_índice_y_pulgar_cruzados", "1FAF0"},
    {"mano_con_dedo_índice_y_pulgar_cruzados_color1", "1FAF0-1F3FB"},
    {"mano_con_dedo_índice_y_pulgar_cruzados_color2", "1FAF0-1F3FC"},
    {"mano_con_dedo_índice_y_pulgar_cruzados_color3", "1FAF0-1F3FD"},
    {"mano_con_dedo_índice_y_pulgar_cruzados_color4", "1FAF0-1F3FE"},
    {"mano_con_dedo_índice_y_pulgar_cruzados_color5", "1FAF0-1F3FF"},
    {"dedo_índice_apuntándote_a_ti", "1FAF5"},
    {"dedo_índice_apuntándote_a_ti_color1", "1FAF5-1F3FB"},
    {"dedo_índice_apuntándote_a_ti_color2", "1FAF5-1F3FC"},
    {"dedo_índice_apuntándote_a_ti_color3", "1FAF5-1F3FD"},
    {"dedo_índice_apuntándote_a_ti_color4", "1FAF5-1F3FE"},
    {"dedo_índice_apuntándote_a_ti_color5", "1FAF5-1F3FF"},
    {"manos_formando_un_corazón", "1FAF6"},
    {"manos_formando_un_corazón_color1", "1FAF6-1F3FB"},
    {"manos_formando_un_corazón_color2", "1FAF6-1F3FC"},
    {"manos_formando_un_corazón_color3", "1FAF6-1F3FD"},
    {"manos_formando_un_corazón_color4", "1FAF6-1F3FE"},
    {"manos_formando_un_corazón_color5", "1FAF6-1F3FF"},
    {"teclas_símbolo_de_número", "0023-FE0F-20E3"},
    {"bandera_isla_de_la_ascensión", "1F1E6-1F1E8"},
    {"bandera_emiratos_árabes_unidos", "1F1E6-1F1EA"},
    {"bandera_afganistán", "1F1E6-1F1EB"},
    {"bandera_antártida", "1F1E6-1F1F6"},
    {"bandera_azerbaiyán", "1F1E6-1F1FF"},
    {"bandera_bangladés", "1F1E7-1F1E9"},
    {"bandera_bélgica", "1F1E7-1F1EA"},
    {"bandera_baréin", "1F1E7-1F1ED"},
    {"bandera_benín", "1F1E7-1F1EF"},
    {"bandera_san_bartolomé", "1F1E7-1F1F1"},
    {"bandera_brunéi", "1F1E7-1F1F3"},
    {"bandera_caribe_neerlandés", "1F1E7-1F1F6"},
    {"bandera_bután", "1F1E7-1F1F9"},
    {"bandera_canadá", "1F1E8-1F1E6"},
    {"bandera_república_democrática_del_congo", "1F1E8-1F1E9"},
    {"bandera_república_centroafricana", "1F1E8-1F1EB"},
    {"bandera_côte_divoire", "1F1E8-1F1EE"},
    {"bandera_camerún", "1F1E8-1F1F2"},
    {"bandera_diego_garcía", "1F1E9-1F1EC"},
    {"bandera_república_dominicana", "1F1E9-1F1F4"},
    {"bandera_sáhara_occidental", "1F1EA-1F1ED"},
    {"bandera_españa", "1F1EA-1F1F8"},
    {"bandera_etiopía", "1F1EA-1F1F9"},
    {"bandera_unión_europea", "1F1EA-1F1FA"},
    {"bandera_gabón", "1F1EC-1F1E6"},
    {"bandera_guinea_bisáu", "1F1EC-1F1FC"},
    {"bandera_haití", "1F1ED-1F1F9"},
    {"bandera_hungría", "1F1ED-1F1FA"},
    {"bandera_territorio_británico_del_océano_índico", "1F1EE-1F1F4"},
    {"bandera_irán", "1F1EE-1F1F7"},
    {"bandera_japón", "1F1EF-1F1F5"},
    {"bandera_kirguistán", "1F1F0-1F1EC"},
    {"bandera_san_cristóbal_y_nieves", "1F1F0-1F1F3"},
    {"bandera_islas_caimán", "1F1F0-1F1FE"},
    {"bandera_kazajistán", "1F1F0-1F1FF"},
    {"bandera_líbano", "1F1F1-1F1E7"},
    {"bandera_santa_lucía", "1F1F1-1F1E8"},
    {"bandera_mónaco", "1F1F2-1F1E8"},
    {"bandera_san_martín", "1F1F2-1F1EB"},
    {"bandera_méxico", "1F1F2-1F1FD"},
    {"bandera_níger", "1F1F3-1F1EA"},
    {"bandera_países_bajos", "1F1F3-1F1F1"},
    {"bandera_omán", "1F1F4-1F1F2"},
    {"bandera_panamá", "1F1F5-1F1E6"},
    {"bandera_perú", "1F1F5-1F1EA"},
    {"bandera_papúa_nueva_guinea", "1F1F5-1F1EC"},
    {"bandera_pakistán", "1F1F5-1F1F0"},
    {"bandera_san_pedro_y_miquelón", "1F1F5-1F1F2"},
    {"bandera_reunión", "1F1F7-1F1EA"},
    {"bandera_rumanía", "1F1F7-1F1F4"},
    {"bandera_arabia_saudí", "1F1F8-1F1E6"},
    {"bandera_islas_salomón", "1F1F8-1F1E7"},
    {"bandera_sudán", "1F1F8-1F1E9"},
    {"bandera_sudán_del_sur", "1F1F8-1F1F8"},
    {"bandera_santo_tomé_y_príncipe", "1F1F8-1F1F9"},
    {"bandera_tristán_de_acuña", "1F1F9-1F1E6"},
    {"bandera_tayikistán", "1F1F9-1F1EF"},
    {"bandera_turkmenistán", "1F1F9-1F1F2"},
    {"bandera_túnez", "1F1F9-1F1F3"},
    {"bandera_turquía", "1F1F9-1F1F7"},
    {"bandera_taiwán", "1F1F9-1F1FC"},
    {"bandera_uzbekistán", "1F1FA-1F1FF"},
    {"bandera_islas_vírgenes_británicas", "1F1FB-1F1EC"},
    {"bandera_islas_vírgenes_de_ee_uu", "1F1FB-1F1EE"},
    {"bandera_sudáfrica", "1F1FF-1F1E6"},
    {"familia_hombre_y_niño", "1F468-200D-1F466"},
    {"familia_hombre_niño_niño", "1F468-200D-1F466-200D-1F466"},
    {"familia_hombre_y_niña", "1F468-200D-1F467"},
    {"familia_hombre_niña_niño", "1F468-200D-1F467-200D-1F466"},
    {"familia_hombre_niña_niña", "1F468-200D-1F467-200D-1F467"},
    {"familia_hombre_hombre_niño", "1F468-200D-1F468-200D-1F466"},
    {"familia_hombre_hombre_niño_niño", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"familia_hombre_hombre_niña", "1F468-200D-1F468-200D-1F467"},
    {"familia_hombre_hombre_niña_niño", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"familia_hombre_hombre_niña_niña", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"familia_hombre_mujer_niño", "1F468-200D-1F469-200D-1F466"},
    {"familia_hombre_mujer_niño_niño", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"familia_hombre_mujer_niña", "1F468-200D-1F469-200D-1F467"},
    {"familia_hombre_mujer_niña_niño", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"familia_hombre_mujer_niña_niña", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"familia_mujer_y_niño", "1F469-200D-1F466"},
    {"familia_mujer_niño_niño", "1F469-200D-1F466-200D-1F466"},
    {"familia_mujer_y_niña", "1F469-200D-1F467"},
    {"familia_mujer_niña_niño", "1F469-200D-1F467-200D-1F466"},
    {"familia_mujer_niña_niña", "1F469-200D-1F467-200D-1F467"},
    {"familia_mujer_mujer_niño", "1F469-200D-1F469-200D-1F466"},
    {"familia_mujer_mujer_niño_niño", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"familia_mujer_mujer_niña", "1F469-200D-1F469-200D-1F467"},
    {"familia_mujer_mujer_niña_niño", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"familia_mujer_mujer_niña_niña", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"familia_adulto_adulto_niño", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"familia_adulto_adulto_niño_niño", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"familia_adulto_niño", "1F9D1-200D-1F9D2"},
    {"familia_adulto_niño_niño", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"hombre_alimentando_a_bebé", "1F468-200D-1F37C"},
    {"hombre_alimentando_a_bebé_color1", "1F468-1F3FB-200D-1F37C"},
    {"hombre_alimentando_a_bebé_color2", "1F468-1F3FC-200D-1F37C"},
    {"hombre_alimentando_a_bebé_color3", "1F468-1F3FD-200D-1F37C"},
    {"hombre_alimentando_a_bebé_color4", "1F468-1F3FE-200D-1F37C"},
    {"hombre_alimentando_a_bebé_color5", "1F468-1F3FF-200D-1F37C"},
    {"profesional_de_la_tecnología_hombre", "1F468-200D-1F4BB"},
    {"profesional_de_la_tecnología_hombre_color1", "1F468-1F3FB-200D-1F4BB"},
    {"profesional_de_la_tecnología_hombre_color2", "1F468-1F3FC-200D-1F4BB"},
    {"profesional_de_la_tecnología_hombre_color3", "1F468-1F3FD-200D-1F4BB"},
    {"profesional_de_la_tecnología_hombre_color4", "1F468-1F3FE-200D-1F4BB"},
    {"profesional_de_la_tecnología_hombre_color5", "1F468-1F3FF-200D-1F4BB"},
    {"profesional_de_la_mecánica_hombre", "1F468-200D-1F527"},
    {"profesional_de_la_mecánica_hombre_color1", "1F468-1F3FB-200D-1F527"},
    {"profesional_de_la_mecánica_hombre_color2", "1F468-1F3FC-200D-1F527"},
    {"profesional_de_la_mecánica_hombre_color3", "1F468-1F3FD-200D-1F527"},
    {"profesional_de_la_mecánica_hombre_color4", "1F468-1F3FE-200D-1F527"},
    {"profesional_de_la_mecánica_hombre_color5", "1F468-1F3FF-200D-1F527"},
    {"hombre_con_bastón", "1F468-200D-1F9AF"},
    {"hombre_con_bastón_color1", "1F468-1F3FB-200D-1F9AF"},
    {"hombre_con_bastón_color2", "1F468-1F3FC-200D-1F9AF"},
    {"hombre_con_bastón_color3", "1F468-1F3FD-200D-1F9AF"},
    {"hombre_con_bastón_color4", "1F468-1F3FE-200D-1F9AF"},
    {"hombre_con_bastón_color5", "1F468-1F3FF-200D-1F9AF"},
    {"hombre_con_bastón_hacia_la_derecha", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"hombre_con_bastón_hacia_la_derecha_color1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"hombre_con_bastón_hacia_la_derecha_color2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"hombre_con_bastón_hacia_la_derecha_color3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"hombre_con_bastón_hacia_la_derecha_color4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"hombre_con_bastón_hacia_la_derecha_color5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_eléctrica", "1F468-200D-1F9BC"},
    {"hombre_en_silla_de_ruedas_eléctrica_color1", "1F468-1F3FB-200D-1F9BC"},
    {"hombre_en_silla_de_ruedas_eléctrica_color2", "1F468-1F3FC-200D-1F9BC"},
    {"hombre_en_silla_de_ruedas_eléctrica_color3", "1F468-1F3FD-200D-1F9BC"},
    {"hombre_en_silla_de_ruedas_eléctrica_color4", "1F468-1F3FE-200D-1F9BC"},
    {"hombre_en_silla_de_ruedas_eléctrica_color5", "1F468-1F3FF-200D-1F9BC"},
    {"hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"hombre_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_alimentando_a_bebé", "1F469-200D-1F37C"},
    {"mujer_alimentando_a_bebé_color1", "1F469-1F3FB-200D-1F37C"},
    {"mujer_alimentando_a_bebé_color2", "1F469-1F3FC-200D-1F37C"},
    {"mujer_alimentando_a_bebé_color3", "1F469-1F3FD-200D-1F37C"},
    {"mujer_alimentando_a_bebé_color4", "1F469-1F3FE-200D-1F37C"},
    {"mujer_alimentando_a_bebé_color5", "1F469-1F3FF-200D-1F37C"},
    {"profesional_de_la_tecnología_mujer", "1F469-200D-1F4BB"},
    {"profesional_de_la_tecnología_mujer_color1", "1F469-1F3FB-200D-1F4BB"},
    {"profesional_de_la_tecnología_mujer_color2", "1F469-1F3FC-200D-1F4BB"},
    {"profesional_de_la_tecnología_mujer_color3", "1F469-1F3FD-200D-1F4BB"},
    {"profesional_de_la_tecnología_mujer_color4", "1F469-1F3FE-200D-1F4BB"},
    {"profesional_de_la_tecnología_mujer_color5", "1F469-1F3FF-200D-1F4BB"},
    {"profesional_de_la_mecánica_mujer", "1F469-200D-1F527"},
    {"profesional_de_la_mecánica_mujer_color1", "1F469-1F3FB-200D-1F527"},
    {"profesional_de_la_mecánica_mujer_color2", "1F469-1F3FC-200D-1F527"},
    {"profesional_de_la_mecánica_mujer_color3", "1F469-1F3FD-200D-1F527"},
    {"profesional_de_la_mecánica_mujer_color4", "1F469-1F3FE-200D-1F527"},
    {"profesional_de_la_mecánica_mujer_color5", "1F469-1F3FF-200D-1F527"},
    {"mujer_con_bastón", "1F469-200D-1F9AF"},
    {"mujer_con_bastón_color1", "1F469-1F3FB-200D-1F9AF"},
    {"mujer_con_bastón_color2", "1F469-1F3FC-200D-1F9AF"},
    {"mujer_con_bastón_color3", "1F469-1F3FD-200D-1F9AF"},
    {"mujer_con_bastón_color4", "1F469-1F3FE-200D-1F9AF"},
    {"mujer_con_bastón_color5", "1F469-1F3FF-200D-1F9AF"},
    {"mujer_con_bastón_hacia_la_derecha", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"mujer_con_bastón_hacia_la_derecha_color1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"mujer_con_bastón_hacia_la_derecha_color2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"mujer_con_bastón_hacia_la_derecha_color3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"mujer_con_bastón_hacia_la_derecha_color4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"mujer_con_bastón_hacia_la_derecha_color5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_eléctrica", "1F469-200D-1F9BC"},
    {"mujer_en_silla_de_ruedas_eléctrica_color1", "1F469-1F3FB-200D-1F9BC"},
    {"mujer_en_silla_de_ruedas_eléctrica_color2", "1F469-1F3FC-200D-1F9BC"},
    {"mujer_en_silla_de_ruedas_eléctrica_color3", "1F469-1F3FD-200D-1F9BC"},
    {"mujer_en_silla_de_ruedas_eléctrica_color4", "1F469-1F3FE-200D-1F9BC"},
    {"mujer_en_silla_de_ruedas_eléctrica_color5", "1F469-1F3FF-200D-1F9BC"},
    {"mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_alimentando_a_bebé", "1F9D1-200D-1F37C"},
    {"persona_alimentando_a_bebé_color1", "1F9D1-1F3FB-200D-1F37C"},
    {"persona_alimentando_a_bebé_color2", "1F9D1-1F3FC-200D-1F37C"},
    {"persona_alimentando_a_bebé_color3", "1F9D1-1F3FD-200D-1F37C"},
    {"persona_alimentando_a_bebé_color4", "1F9D1-1F3FE-200D-1F37C"},
    {"persona_alimentando_a_bebé_color5", "1F9D1-1F3FF-200D-1F37C"},
    {"profesional_de_la_tecnología", "1F9D1-200D-1F4BB"},
    {"profesional_de_la_tecnología_color1", "1F9D1-1F3FB-200D-1F4BB"},
    {"profesional_de_la_tecnología_color2", "1F9D1-1F3FC-200D-1F4BB"},
    {"profesional_de_la_tecnología_color3", "1F9D1-1F3FD-200D-1F4BB"},
    {"profesional_de_la_tecnología_color4", "1F9D1-1F3FE-200D-1F4BB"},
    {"profesional_de_la_tecnología_color5", "1F9D1-1F3FF-200D-1F4BB"},
    {"profesional_de_la_mecánica", "1F9D1-200D-1F527"},
    {"profesional_de_la_mecánica_color1", "1F9D1-1F3FB-200D-1F527"},
    {"profesional_de_la_mecánica_color2", "1F9D1-1F3FC-200D-1F527"},
    {"profesional_de_la_mecánica_color3", "1F9D1-1F3FD-200D-1F527"},
    {"profesional_de_la_mecánica_color4", "1F9D1-1F3FE-200D-1F527"},
    {"profesional_de_la_mecánica_color5", "1F9D1-1F3FF-200D-1F527"},
    {"persona_con_bastón", "1F9D1-200D-1F9AF"},
    {"persona_con_bastón_color1", "1F9D1-1F3FB-200D-1F9AF"},
    {"persona_con_bastón_color2", "1F9D1-1F3FC-200D-1F9AF"},
    {"persona_con_bastón_color3", "1F9D1-1F3FD-200D-1F9AF"},
    {"persona_con_bastón_color4", "1F9D1-1F3FE-200D-1F9AF"},
    {"persona_con_bastón_color5", "1F9D1-1F3FF-200D-1F9AF"},
    {"persona_con_bastón_hacia_la_derecha", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_bastón_hacia_la_derecha_color1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_bastón_hacia_la_derecha_color2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_bastón_hacia_la_derecha_color3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_bastón_hacia_la_derecha_color4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_bastón_hacia_la_derecha_color5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_eléctrica", "1F9D1-200D-1F9BC"},
    {"persona_en_silla_de_ruedas_eléctrica_color1", "1F9D1-1F3FB-200D-1F9BC"},
    {"persona_en_silla_de_ruedas_eléctrica_color2", "1F9D1-1F3FC-200D-1F9BC"},
    {"persona_en_silla_de_ruedas_eléctrica_color3", "1F9D1-1F3FD-200D-1F9BC"},
    {"persona_en_silla_de_ruedas_eléctrica_color4", "1F9D1-1F3FE-200D-1F9BC"},
    {"persona_en_silla_de_ruedas_eléctrica_color5", "1F9D1-1F3FF-200D-1F9BC"},
    {"persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_en_silla_de_ruedas_eléctrica_hacia_la_derecha_color5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mujer_botando_un_balón", "26F9-FE0F-200D-2640-FE0F"},
    {"mujer_botando_un_balón_color1", "26F9-1F3FB-200D-2640-FE0F"},
    {"mujer_botando_un_balón_color2", "26F9-1F3FC-200D-2640-FE0F"},
    {"mujer_botando_un_balón_color3", "26F9-1F3FD-200D-2640-FE0F"},
    {"mujer_botando_un_balón_color4", "26F9-1F3FE-200D-2640-FE0F"},
    {"mujer_botando_un_balón_color5", "26F9-1F3FF-200D-2640-FE0F"},
    {"hombre_botando_un_balón", "26F9-FE0F-200D-2642-FE0F"},
    {"hombre_botando_un_balón_color1", "26F9-1F3FB-200D-2642-FE0F"},
    {"hombre_botando_un_balón_color2", "26F9-1F3FC-200D-2642-FE0F"},
    {"hombre_botando_un_balón_color3", "26F9-1F3FD-200D-2642-FE0F"},
    {"hombre_botando_un_balón_color4", "26F9-1F3FE-200D-2642-FE0F"},
    {"hombre_botando_un_balón_color5", "26F9-1F3FF-200D-2642-FE0F"},
    {"agente_de_policía_mujer", "1F46E-200D-2640-FE0F"},
    {"agente_de_policía_mujer_color1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"agente_de_policía_mujer_color2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"agente_de_policía_mujer_color3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"agente_de_policía_mujer_color4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"agente_de_policía_mujer_color5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"agente_de_policía_hombre", "1F46E-200D-2642-FE0F"},
    {"agente_de_policía_hombre_color1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"agente_de_policía_hombre_color2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"agente_de_policía_hombre_color3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"agente_de_policía_hombre_color4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"agente_de_policía_hombre_color5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"profesional_de_la_construcción_mujer", "1F477-200D-2640-FE0F"},
    {"profesional_de_la_construcción_mujer_color1", "1F477-1F3FB-200D-2640-FE0F"},
    {"profesional_de_la_construcción_mujer_color2", "1F477-1F3FC-200D-2640-FE0F"},
    {"profesional_de_la_construcción_mujer_color3", "1F477-1F3FD-200D-2640-FE0F"},
    {"profesional_de_la_construcción_mujer_color4", "1F477-1F3FE-200D-2640-FE0F"},
    {"profesional_de_la_construcción_mujer_color5", "1F477-1F3FF-200D-2640-FE0F"},
    {"profesional_de_la_construcción_hombre", "1F477-200D-2642-FE0F"},
    {"profesional_de_la_construcción_hombre_color1", "1F477-1F3FB-200D-2642-FE0F"},
    {"profesional_de_la_construcción_hombre_color2", "1F477-1F3FC-200D-2642-FE0F"},
    {"profesional_de_la_construcción_hombre_color3", "1F477-1F3FD-200D-2642-FE0F"},
    {"profesional_de_la_construcción_hombre_color4", "1F477-1F3FE-200D-2642-FE0F"},
    {"profesional_de_la_construcción_hombre_color5", "1F477-1F3FF-200D-2642-FE0F"},
    {"empleada_de_mostrador_de_información", "1F481-200D-2640-FE0F"},
    {"empleada_de_mostrador_de_información_color1", "1F481-1F3FB-200D-2640-FE0F"},
    {"empleada_de_mostrador_de_información_color2", "1F481-1F3FC-200D-2640-FE0F"},
    {"empleada_de_mostrador_de_información_color3", "1F481-1F3FD-200D-2640-FE0F"},
    {"empleada_de_mostrador_de_información_color4", "1F481-1F3FE-200D-2640-FE0F"},
    {"empleada_de_mostrador_de_información_color5", "1F481-1F3FF-200D-2640-FE0F"},
    {"empleado_de_mostrador_de_información", "1F481-200D-2642-FE0F"},
    {"empleado_de_mostrador_de_información_color1", "1F481-1F3FB-200D-2642-FE0F"},
    {"empleado_de_mostrador_de_información_color2", "1F481-1F3FC-200D-2642-FE0F"},
    {"empleado_de_mostrador_de_información_color3", "1F481-1F3FD-200D-2642-FE0F"},
    {"empleado_de_mostrador_de_información_color4", "1F481-1F3FE-200D-2642-FE0F"},
    {"empleado_de_mostrador_de_información_color5", "1F481-1F3FF-200D-2642-FE0F"},
    {"mujer_cortándose_el_pelo", "1F487-200D-2640-FE0F"},
    {"mujer_cortándose_el_pelo_color1", "1F487-1F3FB-200D-2640-FE0F"},
    {"mujer_cortándose_el_pelo_color2", "1F487-1F3FC-200D-2640-FE0F"},
    {"mujer_cortándose_el_pelo_color3", "1F487-1F3FD-200D-2640-FE0F"},
    {"mujer_cortándose_el_pelo_color4", "1F487-1F3FE-200D-2640-FE0F"},
    {"mujer_cortándose_el_pelo_color5", "1F487-1F3FF-200D-2640-FE0F"},
    {"hombre_cortándose_el_pelo", "1F487-200D-2642-FE0F"},
    {"hombre_cortándose_el_pelo_color1", "1F487-1F3FB-200D-2642-FE0F"},
    {"hombre_cortándose_el_pelo_color2", "1F487-1F3FC-200D-2642-FE0F"},
    {"hombre_cortándose_el_pelo_color3", "1F487-1F3FD-200D-2642-FE0F"},
    {"hombre_cortándose_el_pelo_color4", "1F487-1F3FE-200D-2642-FE0F"},
    {"hombre_cortándose_el_pelo_color5", "1F487-1F3FF-200D-2642-FE0F"},
    {"mujer_frunciendo_el_ceño", "1F64D-200D-2640-FE0F"},
    {"mujer_frunciendo_el_ceño_color1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"mujer_frunciendo_el_ceño_color2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"mujer_frunciendo_el_ceño_color3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"mujer_frunciendo_el_ceño_color4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"mujer_frunciendo_el_ceño_color5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"hombre_frunciendo_el_ceño", "1F64D-200D-2642-FE0F"},
    {"hombre_frunciendo_el_ceño_color1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"hombre_frunciendo_el_ceño_color2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"hombre_frunciendo_el_ceño_color3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"hombre_frunciendo_el_ceño_color4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"hombre_frunciendo_el_ceño_color5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"mujer_en_bicicleta_de_montaña", "1F6B5-200D-2640-FE0F"},
    {"mujer_en_bicicleta_de_montaña_color1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"mujer_en_bicicleta_de_montaña_color2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"mujer_en_bicicleta_de_montaña_color3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"mujer_en_bicicleta_de_montaña_color4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"mujer_en_bicicleta_de_montaña_color5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"hombre_en_bicicleta_de_montaña", "1F6B5-200D-2642-FE0F"},
    {"hombre_en_bicicleta_de_montaña_color1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"hombre_en_bicicleta_de_montaña_color2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"hombre_en_bicicleta_de_montaña_color3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"hombre_en_bicicleta_de_montaña_color4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"hombre_en_bicicleta_de_montaña_color5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"superheroína", "1F9B8-200D-2640-FE0F"},
    {"superheroína_color1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"superheroína_color2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"superheroína_color3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"superheroína_color4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"superheroína_color5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"superhéroe", "1F9B8-200D-2642-FE0F"},
    {"superhéroe_color1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"superhéroe_color2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"superhéroe_color3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"superhéroe_color4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"superhéroe_color5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"mujer_en_posición_de_loto", "1F9D8-200D-2640-FE0F"},
    {"mujer_en_posición_de_loto_color1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"mujer_en_posición_de_loto_color2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"mujer_en_posición_de_loto_color3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"mujer_en_posición_de_loto_color4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"mujer_en_posición_de_loto_color5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"hombre_en_posición_de_loto", "1F9D8-200D-2642-FE0F"},
    {"hombre_en_posición_de_loto_color1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"hombre_en_posición_de_loto_color2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"hombre_en_posición_de_loto_color3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"hombre_en_posición_de_loto_color4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"hombre_en_posición_de_loto_color5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"corazón_en_llamas", "2764-FE0F-200D-1F525"},
    {"corazón_vendado", "2764-FE0F-200D-1FA79"},
    {"champiñón_marrón", "1F344-200D-1F7EB"},
    {"bandera_transgénero", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"bandera_del_arcoíris", "1F3F3-FE0F-200D-1F308"},
    {"pájaro_negro", "1F426-200D-2B1B"},
    {"fénix", "1F426-200D-1F525"},
};

