// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_it[] = {
    {"punto_esclamativo_e_interrogativo", "2049"},
    {"marchio", "2122"},
    {"punto_informazioni", "2139"},
    {"freccia_sinistra_destra", "2194"},
    {"freccia_su_giu", "2195"},
    {"freccia_in_alto_a_sinistra", "2196"},
    {"freccia_rivolta_verso_destra_che_punta_in_alto", "2197"},
    {"freccia_in_basso_a_destra", "2198"},
    {"freccia_in_basso_a_sinistra", "2199"},
    {"tastiera", "2328"},
    {"sole", "2600"},
    {"nuvola", "2601"},
    {"ombrello", "2602"},
    {"pupazzo_di_neve", "2603"},
    {"cometa", "2604"},
    {"riquadro_con_spunta", "2611"},
    {"ombrello_con_gocce_di_pioggia", "2614"},
    {"bevanda_calda", "2615"},
    {"trifoglio", "2618"},
    {"teschio_con_ossa_incrociate", "2620"},
    {"simbolo_della_radioattivita", "2622"},
    {"simbolo_del_rischio_biologico", "2623"},
    {"croce_ortodossa", "2626"},
    {"ruota_del_dharma", "2638"},
    {"faccina_imbronciata", "2639"},
    {"simbolo_genere_femminile", "2640"},
    {"simbolo_genere_maschile", "2642"},
    {"segno_zodiacale_dellariete", "2648"},
    {"segno_zodiacale_del_toro", "2649"},
    {"segno_zodiacale_del_saggitario", "2650"},
    {"segno_zodiacale_del_capricorno", "2651"},
    {"segno_zodiacale_dellacquario", "2652"},
    {"segno_zodiacale_dei_pesci", "2653"},
    {"picche", "2660"},
    {"fiori", "2663"},
    {"cuori", "2665"},
    {"quadri", "2666"},
    {"sorgenti_termali", "2668"},
    {"piccone_e_martello", "2692"},
    {"ancora", "2693"},
    {"spade_incrociate", "2694"},
    {"simbolo_della_medicina", "2695"},
    {"bilancia_a_doppio_piatto", "2696"},
    {"alambicco", "2697"},
    {"ingranaggio", "2699"},
    {"forbici", "2702"},
    {"pulsante_segno_di_spunta", "2705"},
    {"aeroplano", "2708"},
    {"busta", "2709"},
    {"pennino", "2712"},
    {"segno_di_spunta", "2714"},
    {"moltiplicazione", "2716"},
    {"stella_di_david", "2721"},
    {"stelline", "2728"},
    {"asterisco", "2733"},
    {"stella_stilizzata", "2734"},
    {"fiocco_di_neve", "2744"},
    {"scintilla_stilizzata", "2747"},
    {"punto_interrogativo_rosso", "2753"},
    {"punto_interrogativo_bianco", "2754"},
    {"punto_esclamativo_bianco", "2755"},
    {"punto_esclamativo_rosso", "2757"},
    {"punto_esclamativo_a_cuore", "2763"},
    {"cuore_rosso", "2764"},
    {"addizione", "2795"},
    {"sottrazione", "2796"},
    {"divisione", "2797"},
    {"freccia_curva_in_alto", "2934"},
    {"freccia_curva_in_basso", "2935"},
    {"trattino_ondulato", "3030"},
    {"ideogramma_giapponese_di_congratulazioni", "3297"},
    {"ideogramma_giapponese_di_segreto", "3299"},
    {"copyright", "00A9"},
    {"marchio_registrato", "00AE"},
    {"doppio_punto_esclamativo", "203C"},
    {"freccia_curva_a_sinistra", "21A9"},
    {"freccia_curva_a_destra", "21AA"},
    {"orologio", "231A"},
    {"clessidra", "231B"},
    {"pulsante_di_espulsione", "23CF"},
    {"pulsante_di_avanzamento_rapido", "23E9"},
    {"pulsante_di_riavvolgimento_rapido", "23EA"},
    {"pulsante_doppia_freccia_in_alto", "23EB"},
    {"doppia_freccia_in_basso", "23EC"},
    {"traccia_successiva", "23ED"},
    {"pulsante_traccia_precedente", "23EE"},
    {"pulsante_riproduci_pausa", "23EF"},
    {"sveglia", "23F0"},
    {"cronometro", "23F1"},
    {"timer", "23F2"},
    {"clessidra_che_scorre", "23F3"},
    {"pulsante_pausa", "23F8"},
    {"pulsante_stop", "23F9"},
    {"pulsante_di_registrazione", "23FA"},
    {"pulsante_m_cerchiata", "24C2"},
    {"quadrato_nero_piccolo", "25AA"},
    {"quadrato_bianco_piccolo", "25AB"},
    {"pulsante_di_riproduzione", "25B6"},
    {"pulsante_di_riavvolgimento", "25C0"},
    {"quadrato_bianco_medio", "25FB"},
    {"quadrato_nero_medio", "25FC"},
    {"quadrato_bianco_medio_piccolo", "25FD"},
    {"quadrato_nero_medio_piccolo", "25FE"},
    {"telefono_fisso", "260E"},
    {"indice_verso_lalto", "261D"},
    {"indice_verso_lalto_colore1", "261D-1F3FB"},
    {"indice_verso_lalto_colore2", "261D-1F3FC"},
    {"indice_verso_lalto_colore3", "261D-1F3FD"},
    {"indice_verso_lalto_colore4", "261D-1F3FE"},
    {"indice_verso_lalto_colore5", "261D-1F3FF"},
    {"mezzaluna_e_stella", "262A"},
    {"simbolo_della_pace", "262E"},
    {"yin_e_yang", "262F"},
    {"faccina_sorridente", "263A"},
    {"segno_zodiacale_dei_gemelli", "264A"},
    {"segno_zodiacale_del_cancro", "264B"},
    {"segno_zodiacale_del_leone", "264C"},
    {"segno_zodiacale_della_vergine", "264D"},
    {"segno_zodiacale_della_bilancia", "264E"},
    {"segno_zodiacale_dello_scorpione", "264F"},
    {"pedina_degli_scacchi", "265F"},
    {"simbolo_del_riciclaggio", "267B"},
    {"simbolo_dellinfinito", "267E"},
    {"simbolo_della_disabilita", "267F"},
    {"simbolo_dellatomo", "269B"},
    {"giglio_araldico", "269C"},
    {"simbolo_di_avviso", "26A0"},
    {"alta_tensione", "26A1"},
    {"simbolo_transgender", "26A7"},
    {"cerchio_bianco", "26AA"},
    {"cerchio_nero", "26AB"},
    {"bara", "26B0"},
    {"urna_funeraria", "26B1"},
    {"pallone_da_calcio", "26BD"},
    {"palla_da_baseball", "26BE"},
    {"pupazzo_di_neve_senza_neve", "26C4"},
    {"sole_coperto", "26C5"},
    {"temporale", "26C8"},
    {"segno_zodiacale_dellofiuco", "26CE"},
    {"piccone", "26CF"},
    {"elmetto_con_croce_bianca", "26D1"},
    {"catene", "26D3"},
    {"segnale_di_divieto_di_accesso", "26D4"},
    {"santuario_shintoista", "26E9"},
    {"chiesa", "26EA"},
    {"montagna", "26F0"},
    {"ombrellone", "26F1"},
    {"fontana", "26F2"},
    {"bandiera_in_buca", "26F3"},
    {"traghetto", "26F4"},
    {"barca_a_vela", "26F5"},
    {"sciatore", "26F7"},
    {"pattinaggio_su_ghiaccio", "26F8"},
    {"persona_che_fa_rimbalzare_una_palla", "26F9"},
    {"persona_che_fa_rimbalzare_una_palla_colore1", "26F9-1F3FB"},
    {"persona_che_fa_rimbalzare_una_palla_colore2", "26F9-1F3FC"},
    {"persona_che_fa_rimbalzare_una_palla_colore3", "26F9-1F3FD"},
    {"persona_che_fa_rimbalzare_una_palla_colore4", "26F9-1F3FE"},
    {"persona_che_fa_rimbalzare_una_palla_colore5", "26F9-1F3FF"},
    {"tenda", "26FA"},
    {"stazione_di_servizio", "26FD"},
    {"pugno", "270A"},
    {"pugno_colore1", "270A-1F3FB"},
    {"pugno_colore2", "270A-1F3FC"},
    {"pugno_colore3", "270A-1F3FD"},
    {"pugno_colore4", "270A-1F3FE"},
    {"pugno_colore5", "270A-1F3FF"},
    {"mano_alzata", "270B"},
    {"mano_alzata_colore1", "270B-1F3FB"},
    {"mano_alzata_colore2", "270B-1F3FC"},
    {"mano_alzata_colore3", "270B-1F3FD"},
    {"mano_alzata_colore4", "270B-1F3FE"},
    {"mano_alzata_colore5", "270B-1F3FF"},
    {"vittoria", "270C"},
    {"vittoria_colore1", "270C-1F3FB"},
    {"vittoria_colore2", "270C-1F3FC"},
    {"vittoria_colore3", "270C-1F3FD"},
    {"vittoria_colore4", "270C-1F3FE"},
    {"vittoria_colore5", "270C-1F3FF"},
    {"mano_che_scrive", "270D"},
    {"mano_che_scrive_colore1", "270D-1F3FB"},
    {"mano_che_scrive_colore2", "270D-1F3FC"},
    {"mano_che_scrive_colore3", "270D-1F3FD"},
    {"mano_che_scrive_colore4", "270D-1F3FE"},
    {"mano_che_scrive_colore5", "270D-1F3FF"},
    {"matita", "270F"},
    {"croce_latina", "271D"},
    {"croce", "274C"},
    {"croce_con_quadrato", "274E"},
    {"freccia_rivolta_verso_destra", "27A1"},
    {"occhiello", "27B0"},
    {"doppio_occhiello", "27BF"},
    {"freccia_rivolta_a_sinistra", "2B05"},
    {"freccia_rivolta_verso_lalto", "2B06"},
    {"freccia_rivolta_verso_il_basso", "2B07"},
    {"quadrato_nero_grande", "2B1B"},
    {"quadrato_bianco_grande", "2B1C"},
    {"stella", "2B50"},
    {"cerchio_rosso_vuoto", "2B55"},
    {"simbolo_di_alternanza_delle_parti", "303D"},
    {"tessera_mahjong", "1F004"},
    {"jolly", "1F0CF"},
    {"gruppo_sanguigno_a", "1F170"},
    {"gruppo_sanguigno_b", "1F171"},
    {"gruppo_sanguigno_0", "1F17E"},
    {"pulsante_p", "1F17F"},
    {"gruppo_sanguigno_ab", "1F18E"},
    {"pulsante_cl", "1F191"},
    {"pulsante_cool", "1F192"},
    {"pulsante_free", "1F193"},
    {"pulsante_id", "1F194"},
    {"pulsante_new", "1F195"},
    {"pulsante_ng", "1F196"},
    {"pulsante_ok", "1F197"},
    {"pulsante_sos", "1F198"},
    {"pulsante_up", "1F199"},
    {"pulsante_vs", "1F19A"},
    {"indicatore_regionale_a", "1F1E6"},
    {"indicatore_regionale_b", "1F1E7"},
    {"indicatore_regionale_c", "1F1E8"},
    {"indicatore_regionale_d", "1F1E9"},
    {"indicatore_regionale_e", "1F1EA"},
    {"indicatore_regionale_f", "1F1EB"},
    {"indicatore_regionale_g", "1F1EC"},
    {"indicatore_regionale_h", "1F1ED"},
    {"indicatore_regionale_i", "1F1EE"},
    {"indicatore_regionale_j", "1F1EF"},
    {"indicatore_regionale_k", "1F1F0"},
    {"indicatore_regionale_l", "1F1F1"},
    {"indicatore_regionale_m", "1F1F2"},
    {"indicatore_regionale_n", "1F1F3"},
    {"indicatore_regionale_o", "1F1F4"},
    {"indicatore_regionale_p", "1F1F5"},
    {"indicatore_regionale_q", "1F1F6"},
    {"indicatore_regionale_r", "1F1F7"},
    {"indicatore_regionale_s", "1F1F8"},
    {"indicatore_regionale_t", "1F1F9"},
    {"indicatore_regionale_u", "1F1FA"},
    {"indicatore_regionale_v", "1F1FB"},
    {"indicatore_regionale_w", "1F1FC"},
    {"indicatore_regionale_x", "1F1FD"},
    {"indicatore_regionale_y", "1F1FE"},
    {"indicatore_regionale_z", "1F1FF"},
    {"ideogramma_giapponese_per_qui", "1F201"},
    {"ideogramma_giapponese_di_costo_del_servizio", "1F202"},
    {"ideogramma_giapponese_di_gratis", "1F21A"},
    {"ideogramma_giapponese_di_riservato", "1F22F"},
    {"ideogramma_giapponese_di_proibito", "1F232"},
    {"ideogramma_giapponese_di_posto_libero", "1F233"},
    {"ideogramma_giapponese_di_voto_di_sufficienza", "1F234"},
    {"ideogramma_giapponese_di_nessun_posto_libero", "1F235"},
    {"ideogramma_giapponese_di_a_pagamento", "1F236"},
    {"ideogramma_giapponese_di_importo_mensile", "1F237"},
    {"ideogramma_giapponese_di_candidatura", "1F238"},
    {"ideogramma_giapponese_di_sconto", "1F239"},
    {"ideogramma_giapponese_di_aperto_al_pubblico", "1F23A"},
    {"ideogramma_giapponese_di_occasione", "1F250"},
    {"ideogramma_giapponese_di_accettabile", "1F251"},
    {"ciclone", "1F300"},
    {"nebbioso", "1F301"},
    {"ombrello_chiuso", "1F302"},
    {"notte_stellata", "1F303"},
    {"alba_sulle_montagne", "1F304"},
    {"alba", "1F305"},
    {"citta_al_tramonto", "1F306"},
    {"tramonto", "1F307"},
    {"arcobaleno", "1F308"},
    {"ponte_di_notte", "1F309"},
    {"onda", "1F30A"},
    {"vulcano", "1F30B"},
    {"via_lattea", "1F30C"},
    {"europa_e_africa", "1F30D"},
    {"america", "1F30E"},
    {"asia_e_australia", "1F30F"},
    {"globo_con_meridiani", "1F310"},
    {"luna_nuova", "1F311"},
    {"luna_crescente", "1F312"},
    {"primo_quarto_di_luna", "1F313"},
    {"luna_gibbosa_crescente", "1F314"},
    {"luna_piena", "1F315"},
    {"luna_gibbosa_calante", "1F316"},
    {"ultimo_quarto_di_luna", "1F317"},
    {"luna_calante", "1F318"},
    {"spicchio_di_luna", "1F319"},
    {"faccina_luna_nuova", "1F31A"},
    {"faccina_primo_quarto_di_luna", "1F31B"},
    {"faccina_ultimo_quarto_di_luna", "1F31C"},
    {"faccina_luna_piena", "1F31D"},
    {"faccina_sole", "1F31E"},
    {"stella_che_brilla", "1F31F"},
    {"stella_cadente", "1F320"},
    {"termometro", "1F321"},
    {"parzialmente_nuvoloso", "1F324"},
    {"molto_nuvoloso", "1F325"},
    {"sole_coperto_e_pioggia", "1F326"},
    {"pioggia", "1F327"},
    {"neve", "1F328"},
    {"fulmini", "1F329"},
    {"tornado", "1F32A"},
    {"nebbia", "1F32B"},
    {"vento", "1F32C"},
    {"hot_dog", "1F32D"},
    {"taco", "1F32E"},
    {"burrito", "1F32F"},
    {"castagna", "1F330"},
    {"germoglio", "1F331"},
    {"albero_sempreverde", "1F332"},
    {"albero_deciduo", "1F333"},
    {"palma", "1F334"},
    {"cactus", "1F335"},
    {"peperoncino", "1F336"},
    {"tulipano", "1F337"},
    {"fiore_di_ciliegio", "1F338"},
    {"rosa", "1F339"},
    {"ibisco", "1F33A"},
    {"girasole", "1F33B"},
    {"fiore", "1F33C"},
    {"pannocchia", "1F33D"},
    {"pianta_di_riso", "1F33E"},
    {"pianta", "1F33F"},
    {"quadrifoglio", "1F340"},
    {"foglia_dacero", "1F341"},
    {"foglia_caduta", "1F342"},
    {"foglia_al_vento", "1F343"},
    {"fungo", "1F344"},
    {"pomodoro", "1F345"},
    {"melanzana", "1F346"},
    {"uva", "1F347"},
    {"melone", "1F348"},
    {"anguria", "1F349"},
    {"mandarino", "1F34A"},
    {"limone", "1F34B"},
    {"banana", "1F34C"},
    {"ananas", "1F34D"},
    {"mela_rossa", "1F34E"},
    {"mela_verde", "1F34F"},
    {"pera", "1F350"},
    {"pesca", "1F351"},
    {"ciliegie", "1F352"},
    {"fragola", "1F353"},
    {"hamburger", "1F354"},
    {"pizza", "1F355"},
    {"carne", "1F356"},
    {"coscia_di_pollo", "1F357"},
    {"galletta_di_riso", "1F358"},
    {"onigiri", "1F359"},
    {"riso_bollito", "1F35A"},
    {"riso_al_curry", "1F35B"},
    {"ciotola_fumante", "1F35C"},
    {"spaghetti", "1F35D"},
    {"pane_in_cassetta", "1F35E"},
    {"patatine", "1F35F"},
    {"patata_dolce_arrosto", "1F360"},
    {"dango", "1F361"},
    {"oden_giapponese", "1F362"},
    {"sushi", "1F363"},
    {"gambero_fritto", "1F364"},
    {"tortino_di_pesce_a_spirale", "1F365"},
    {"cono_gelato", "1F366"},
    {"granita", "1F367"},
    {"coppa_di_gelato", "1F368"},
    {"ciambella", "1F369"},
    {"biscotto", "1F36A"},
    {"cioccolato", "1F36B"},
    {"caramella", "1F36C"},
    {"lecca_lecca", "1F36D"},
    {"budino", "1F36E"},
    {"barattolo_di_miele", "1F36F"},
    {"fetta_di_torta", "1F370"},
    {"bento_box", "1F371"},
    {"pentola_di_cibo", "1F372"},
    {"cucinare", "1F373"},
    {"forchetta_e_coltello", "1F374"},
    {"tazza_da_te_senza_manico", "1F375"},
    {"sake", "1F376"},
    {"bicchiere_di_vino", "1F377"},
    {"cocktail", "1F378"},
    {"cocktail_tropicale", "1F379"},
    {"boccale_di_birra", "1F37A"},
    {"boccali_di_birra", "1F37B"},
    {"biberon", "1F37C"},
    {"piatto_e_posate", "1F37D"},
    {"bottiglia_stappata", "1F37E"},
    {"popcorn", "1F37F"},
    {"fiocco", "1F380"},
    {"regalo", "1F381"},
    {"torta_di_compleanno", "1F382"},
    {"zucca_di_halloween", "1F383"},
    {"albero_di_natale", "1F384"},
    {"babbo_natale", "1F385"},
    {"babbo_natale_colore1", "1F385-1F3FB"},
    {"babbo_natale_colore2", "1F385-1F3FC"},
    {"babbo_natale_colore3", "1F385-1F3FD"},
    {"babbo_natale_colore4", "1F385-1F3FE"},
    {"babbo_natale_colore5", "1F385-1F3FF"},
    {"fuochi_dartificio", "1F386"},
    {"stellina_scintillante", "1F387"},
    {"palloncino", "1F388"},
    {"spara_coriandoli", "1F389"},
    {"coriandoli", "1F38A"},
    {"albero_di_tanabata", "1F38B"},
    {"bandiere_del_giappone_incrociate", "1F38C"},
    {"bambu_decorato", "1F38D"},
    {"bambole_giapponesi", "1F38E"},
    {"aquilone_a_forma_di_carpa", "1F38F"},
    {"campana_al_vento", "1F390"},
    {"contemplazione_della_luna", "1F391"},
    {"zaino", "1F392"},
    {"cappello_da_laureato", "1F393"},
    {"medaglia_militare", "1F396"},
    {"nastro", "1F397"},
    {"microfono_radiofonico", "1F399"},
    {"cursore_del_volume", "1F39A"},
    {"manopole_di_controllo", "1F39B"},
    {"pellicola_cinematografica", "1F39E"},
    {"biglietto_dingresso", "1F39F"},
    {"cavallo_da_giostra", "1F3A0"},
    {"ruota_panoramica", "1F3A1"},
    {"montagne_russe", "1F3A2"},
    {"canna_da_pesca", "1F3A3"},
    {"microfono", "1F3A4"},
    {"cinepresa", "1F3A5"},
    {"simbolo_del_cinema", "1F3A6"},
    {"cuffie", "1F3A7"},
    {"tavolozza", "1F3A8"},
    {"cilindro", "1F3A9"},
    {"circo", "1F3AA"},
    {"biglietto", "1F3AB"},
    {"ciak", "1F3AC"},
    {"maschere", "1F3AD"},
    {"gamepad", "1F3AE"},
    {"bersaglio", "1F3AF"},
    {"slot_machine", "1F3B0"},
    {"palla_da_biliardo", "1F3B1"},
    {"dado", "1F3B2"},
    {"bowling", "1F3B3"},
    {"carta_da_gioco_hanafuda", "1F3B4"},
    {"nota_musicale", "1F3B5"},
    {"note_musicali", "1F3B6"},
    {"sassofono", "1F3B7"},
    {"chitarra", "1F3B8"},
    {"piano", "1F3B9"},
    {"tromba", "1F3BA"},
    {"violino", "1F3BB"},
    {"pentagramma", "1F3BC"},
    {"maglia_da_corsa", "1F3BD"},
    {"tennis", "1F3BE"},
    {"sci", "1F3BF"},
    {"palla_da_pallacanestro", "1F3C0"},
    {"bandiera_a_scacchi", "1F3C1"},
    {"persona_sullo_snowboard", "1F3C2"},
    {"persona_sullo_snowboard_colore1", "1F3C2-1F3FB"},
    {"persona_sullo_snowboard_colore2", "1F3C2-1F3FC"},
    {"persona_sullo_snowboard_colore3", "1F3C2-1F3FD"},
    {"persona_sullo_snowboard_colore4", "1F3C2-1F3FE"},
    {"persona_sullo_snowboard_colore5", "1F3C2-1F3FF"},
    {"persona_che_corre", "1F3C3"},
    {"persona_che_corre_colore1", "1F3C3-1F3FB"},
    {"persona_che_corre_colore2", "1F3C3-1F3FC"},
    {"persona_che_corre_colore3", "1F3C3-1F3FD"},
    {"persona_che_corre_colore4", "1F3C3-1F3FE"},
    {"persona_che_corre_colore5", "1F3C3-1F3FF"},
    {"persona_che_fa_surf", "1F3C4"},
    {"persona_che_fa_surf_colore1", "1F3C4-1F3FB"},
    {"persona_che_fa_surf_colore2", "1F3C4-1F3FC"},
    {"persona_che_fa_surf_colore3", "1F3C4-1F3FD"},
    {"persona_che_fa_surf_colore4", "1F3C4-1F3FE"},
    {"persona_che_fa_surf_colore5", "1F3C4-1F3FF"},
    {"medaglia_sportiva", "1F3C5"},
    {"coppa", "1F3C6"},
    {"ippica", "1F3C7"},
    {"ippica_colore1", "1F3C7-1F3FB"},
    {"ippica_colore2", "1F3C7-1F3FC"},
    {"ippica_colore3", "1F3C7-1F3FD"},
    {"ippica_colore4", "1F3C7-1F3FE"},
    {"ippica_colore5", "1F3C7-1F3FF"},
    {"football_americano", "1F3C8"},
    {"pallone_da_rugby", "1F3C9"},
    {"persona_che_nuota", "1F3CA"},
    {"persona_che_nuota_colore1", "1F3CA-1F3FB"},
    {"persona_che_nuota_colore2", "1F3CA-1F3FC"},
    {"persona_che_nuota_colore3", "1F3CA-1F3FD"},
    {"persona_che_nuota_colore4", "1F3CA-1F3FE"},
    {"persona_che_nuota_colore5", "1F3CA-1F3FF"},
    {"persona_che_solleva_pesi", "1F3CB"},
    {"persona_che_solleva_pesi_colore1", "1F3CB-1F3FB"},
    {"persona_che_solleva_pesi_colore2", "1F3CB-1F3FC"},
    {"persona_che_solleva_pesi_colore3", "1F3CB-1F3FD"},
    {"persona_che_solleva_pesi_colore4", "1F3CB-1F3FE"},
    {"persona_che_solleva_pesi_colore5", "1F3CB-1F3FF"},
    {"persona_che_gioca_a_golf", "1F3CC"},
    {"persona_che_gioca_a_golf_colore1", "1F3CC-1F3FB"},
    {"persona_che_gioca_a_golf_colore2", "1F3CC-1F3FC"},
    {"persona_che_gioca_a_golf_colore3", "1F3CC-1F3FD"},
    {"persona_che_gioca_a_golf_colore4", "1F3CC-1F3FE"},
    {"persona_che_gioca_a_golf_colore5", "1F3CC-1F3FF"},
    {"motocicletta", "1F3CD"},
    {"auto_da_corsa", "1F3CE"},
    {"cricket", "1F3CF"},
    {"palla_da_pallavolo", "1F3D0"},
    {"hockey_su_prato", "1F3D1"},
    {"hockey_su_ghiaccio", "1F3D2"},
    {"ping_pong", "1F3D3"},
    {"montagna_innevata", "1F3D4"},
    {"campeggio", "1F3D5"},
    {"spiaggia_con_ombrellone", "1F3D6"},
    {"edificio_in_costruzione", "1F3D7"},
    {"case", "1F3D8"},
    {"paesaggio_urbano", "1F3D9"},
    {"casa_in_rovina", "1F3DA"},
    {"edificio_classico", "1F3DB"},
    {"deserto", "1F3DC"},
    {"isola_deserta", "1F3DD"},
    {"parco_nazionale", "1F3DE"},
    {"stadio", "1F3DF"},
    {"casa", "1F3E0"},
    {"casa_con_giardino", "1F3E1"},
    {"edificio", "1F3E2"},
    {"ufficio_postale_giapponese", "1F3E3"},
    {"ufficio_postale", "1F3E4"},
    {"ospedale", "1F3E5"},
    {"banca", "1F3E6"},
    {"simbolo_dello_sportello_bancomat", "1F3E7"},
    {"hotel", "1F3E8"},
    {"love_hotel", "1F3E9"},
    {"minimarket", "1F3EA"},
    {"scuola", "1F3EB"},
    {"grande_magazzino", "1F3EC"},
    {"fabbrica", "1F3ED"},
    {"lanterna_rossa", "1F3EE"},
    {"castello_giapponese", "1F3EF"},
    {"castello", "1F3F0"},
    {"bandiera_bianca", "1F3F3"},
    {"bandiera_nera", "1F3F4"},
    {"rosetta", "1F3F5"},
    {"etichetta", "1F3F7"},
    {"badminton", "1F3F8"},
    {"arco_e_freccia", "1F3F9"},
    {"anfora", "1F3FA"},
    {"carnagione_chiara", "1F3FB"},
    {"carnagione_abbastanza_chiara", "1F3FC"},
    {"carnagione_olivastra", "1F3FD"},
    {"carnagione_abbastanza_scura", "1F3FE"},
    {"carnagione_scura", "1F3FF"},
    {"ratto", "1F400"},
    {"topo", "1F401"},
    {"bue", "1F402"},
    {"bufalo_dacqua", "1F403"},
    {"mucca", "1F404"},
    {"tigre", "1F405"},
    {"leopardo", "1F406"},
    {"coniglio", "1F407"},
    {"gatto", "1F408"},
    {"drago", "1F409"},
    {"coccodrillo", "1F40A"},
    {"balena", "1F40B"},
    {"lumaca", "1F40C"},
    {"serpente", "1F40D"},
    {"cavallo", "1F40E"},
    {"montone", "1F40F"},
    {"capra", "1F410"},
    {"pecora", "1F411"},
    {"scimmia", "1F412"},
    {"gallo", "1F413"},
    {"gallina", "1F414"},
    {"cane", "1F415"},
    {"maiale", "1F416"},
    {"cinghiale", "1F417"},
    {"elefante", "1F418"},
    {"polpo", "1F419"},
    {"conchiglia", "1F41A"},
    {"insetto", "1F41B"},
    {"formica", "1F41C"},
    {"ape", "1F41D"},
    {"coccinella", "1F41E"},
    {"pesce", "1F41F"},
    {"pesce_tropicale", "1F420"},
    {"pesce_palla", "1F421"},
    {"tartaruga", "1F422"},
    {"pulcino_che_nasce", "1F423"},
    {"pulcino", "1F424"},
    {"pulcino_visto_di_fronte", "1F425"},
    {"uccello", "1F426"},
    {"pinguino", "1F427"},
    {"koala", "1F428"},
    {"barboncino", "1F429"},
    {"dromedario", "1F42A"},
    {"cammello", "1F42B"},
    {"delfino", "1F42C"},
    {"muso_di_topo", "1F42D"},
    {"muso_di_mucca", "1F42E"},
    {"muso_di_tigre", "1F42F"},
    {"muso_di_coniglio", "1F430"},
    {"muso_di_gatto", "1F431"},
    {"testa_di_drago", "1F432"},
    {"balena_che_spruzza_acqua", "1F433"},
    {"muso_di_cavallo", "1F434"},
    {"muso_di_scimmia", "1F435"},
    {"muso_di_cane", "1F436"},
    {"muso_di_maiale", "1F437"},
    {"rana", "1F438"},
    {"criceto", "1F439"},
    {"lupo", "1F43A"},
    {"orso", "1F43B"},
    {"panda", "1F43C"},
    {"naso_da_maiale", "1F43D"},
    {"impronte_di_zampe", "1F43E"},
    {"scoiattolo", "1F43F"},
    {"occhi", "1F440"},
    {"occhio", "1F441"},
    {"orecchio", "1F442"},
    {"orecchio_colore1", "1F442-1F3FB"},
    {"orecchio_colore2", "1F442-1F3FC"},
    {"orecchio_colore3", "1F442-1F3FD"},
    {"orecchio_colore4", "1F442-1F3FE"},
    {"orecchio_colore5", "1F442-1F3FF"},
    {"naso", "1F443"},
    {"naso_colore1", "1F443-1F3FB"},
    {"naso_colore2", "1F443-1F3FC"},
    {"naso_colore3", "1F443-1F3FD"},
    {"naso_colore4", "1F443-1F3FE"},
    {"naso_colore5", "1F443-1F3FF"},
    {"bocca", "1F444"},
    {"lingua", "1F445"},
    {"indice_alzato", "1F446"},
    {"indice_alzato_colore1", "1F446-1F3FB"},
    {"indice_alzato_colore2", "1F446-1F3FC"},
    {"indice_alzato_colore3", "1F446-1F3FD"},
    {"indice_alzato_colore4", "1F446-1F3FE"},
    {"indice_alzato_colore5", "1F446-1F3FF"},
    {"indice_abbassato", "1F447"},
    {"indice_abbassato_colore1", "1F447-1F3FB"},
    {"indice_abbassato_colore2", "1F447-1F3FC"},
    {"indice_abbassato_colore3", "1F447-1F3FD"},
    {"indice_abbassato_colore4", "1F447-1F3FE"},
    {"indice_abbassato_colore5", "1F447-1F3FF"},
    {"indice_verso_sinistra", "1F448"},
    {"indice_verso_sinistra_colore1", "1F448-1F3FB"},
    {"indice_verso_sinistra_colore2", "1F448-1F3FC"},
    {"indice_verso_sinistra_colore3", "1F448-1F3FD"},
    {"indice_verso_sinistra_colore4", "1F448-1F3FE"},
    {"indice_verso_sinistra_colore5", "1F448-1F3FF"},
    {"indice_verso_destra", "1F449"},
    {"indice_verso_destra_colore1", "1F449-1F3FB"},
    {"indice_verso_destra_colore2", "1F449-1F3FC"},
    {"indice_verso_destra_colore3", "1F449-1F3FD"},
    {"indice_verso_destra_colore4", "1F449-1F3FE"},
    {"indice_verso_destra_colore5", "1F449-1F3FF"},
    {"pugno_chiuso", "1F44A"},
    {"pugno_chiuso_colore1", "1F44A-1F3FB"},
    {"pugno_chiuso_colore2", "1F44A-1F3FC"},
    {"pugno_chiuso_colore3", "1F44A-1F3FD"},
    {"pugno_chiuso_colore4", "1F44A-1F3FE"},
    {"pugno_chiuso_colore5", "1F44A-1F3FF"},
    {"mano_che_saluta", "1F44B"},
    {"mano_che_saluta_colore1", "1F44B-1F3FB"},
    {"mano_che_saluta_colore2", "1F44B-1F3FC"},
    {"mano_che_saluta_colore3", "1F44B-1F3FD"},
    {"mano_che_saluta_colore4", "1F44B-1F3FE"},
    {"mano_che_saluta_colore5", "1F44B-1F3FF"},
    {"mano_che_fa_ok", "1F44C"},
    {"mano_che_fa_ok_colore1", "1F44C-1F3FB"},
    {"mano_che_fa_ok_colore2", "1F44C-1F3FC"},
    {"mano_che_fa_ok_colore3", "1F44C-1F3FD"},
    {"mano_che_fa_ok_colore4", "1F44C-1F3FE"},
    {"mano_che_fa_ok_colore5", "1F44C-1F3FF"},
    {"pollice_in_su", "1F44D"},
    {"pollice_in_su_colore1", "1F44D-1F3FB"},
    {"pollice_in_su_colore2", "1F44D-1F3FC"},
    {"pollice_in_su_colore3", "1F44D-1F3FD"},
    {"pollice_in_su_colore4", "1F44D-1F3FE"},
    {"pollice_in_su_colore5", "1F44D-1F3FF"},
    {"pollice_verso", "1F44E"},
    {"pollice_verso_colore1", "1F44E-1F3FB"},
    {"pollice_verso_colore2", "1F44E-1F3FC"},
    {"pollice_verso_colore3", "1F44E-1F3FD"},
    {"pollice_verso_colore4", "1F44E-1F3FE"},
    {"pollice_verso_colore5", "1F44E-1F3FF"},
    {"mani_che_applaudono", "1F44F"},
    {"mani_che_applaudono_colore1", "1F44F-1F3FB"},
    {"mani_che_applaudono_colore2", "1F44F-1F3FC"},
    {"mani_che_applaudono_colore3", "1F44F-1F3FD"},
    {"mani_che_applaudono_colore4", "1F44F-1F3FE"},
    {"mani_che_applaudono_colore5", "1F44F-1F3FF"},
    {"mani_aperte", "1F450"},
    {"mani_aperte_colore1", "1F450-1F3FB"},
    {"mani_aperte_colore2", "1F450-1F3FC"},
    {"mani_aperte_colore3", "1F450-1F3FD"},
    {"mani_aperte_colore4", "1F450-1F3FE"},
    {"mani_aperte_colore5", "1F450-1F3FF"},
    {"corona", "1F451"},
    {"cappello_da_donna", "1F452"},
    {"occhiali_da_vista", "1F453"},
    {"cravatta", "1F454"},
    {"t_shirt", "1F455"},
    {"jeans", "1F456"},
    {"vestito", "1F457"},
    {"kimono", "1F458"},
    {"bikini", "1F459"},
    {"maglietta_da_donna", "1F45A"},
    {"borsellino", "1F45B"},
    {"borsa", "1F45C"},
    {"pochette", "1F45D"},
    {"scarpa_da_uomo", "1F45E"},
    {"scarpa_sportiva", "1F45F"},
    {"scarpa_con_il_tacco", "1F460"},
    {"sandalo_da_donna", "1F461"},
    {"stivale_da_donna", "1F462"},
    {"impronta_di_piedi", "1F463"},
    {"profilo_di_persona", "1F464"},
    {"profilo_di_due_persone", "1F465"},
    {"bambino", "1F466"},
    {"bambino_colore1", "1F466-1F3FB"},
    {"bambino_colore2", "1F466-1F3FC"},
    {"bambino_colore3", "1F466-1F3FD"},
    {"bambino_colore4", "1F466-1F3FE"},
    {"bambino_colore5", "1F466-1F3FF"},
    {"bambina", "1F467"},
    {"bambina_colore1", "1F467-1F3FB"},
    {"bambina_colore2", "1F467-1F3FC"},
    {"bambina_colore3", "1F467-1F3FD"},
    {"bambina_colore4", "1F467-1F3FE"},
    {"bambina_colore5", "1F467-1F3FF"},
    {"uomo", "1F468"},
    {"uomo_colore1", "1F468-1F3FB"},
    {"uomo_colore2", "1F468-1F3FC"},
    {"uomo_colore3", "1F468-1F3FD"},
    {"uomo_colore4", "1F468-1F3FE"},
    {"uomo_colore5", "1F468-1F3FF"},
    {"donna", "1F469"},
    {"donna_colore1", "1F469-1F3FB"},
    {"donna_colore2", "1F469-1F3FC"},
    {"donna_colore3", "1F469-1F3FD"},
    {"donna_colore4", "1F469-1F3FE"},
    {"donna_colore5", "1F469-1F3FF"},
    {"famiglia", "1F46A"},
    {"uomo_e_donna_che_si_tengono_per_mano", "1F46B"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore1", "1F46B-1F3FB"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore2", "1F46B-1F3FC"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore3", "1F46B-1F3FD"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore4", "1F46B-1F3FE"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore5", "1F46B-1F3FF"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"uomo_e_donna_che_si_tengono_per_mano_colore5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"due_uomini_che_si_tengono_per_mano", "1F46C"},
    {"due_uomini_che_si_tengono_per_mano_colore1", "1F46C-1F3FB"},
    {"due_uomini_che_si_tengono_per_mano_colore2", "1F46C-1F3FC"},
    {"due_uomini_che_si_tengono_per_mano_colore3", "1F46C-1F3FD"},
    {"due_uomini_che_si_tengono_per_mano_colore4", "1F46C-1F3FE"},
    {"due_uomini_che_si_tengono_per_mano_colore5", "1F46C-1F3FF"},
    {"due_uomini_che_si_tengono_per_mano_colore1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"due_uomini_che_si_tengono_per_mano_colore1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"due_uomini_che_si_tengono_per_mano_colore1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"due_uomini_che_si_tengono_per_mano_colore1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"due_uomini_che_si_tengono_per_mano_colore2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"due_uomini_che_si_tengono_per_mano_colore2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"due_uomini_che_si_tengono_per_mano_colore2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"due_uomini_che_si_tengono_per_mano_colore2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"due_uomini_che_si_tengono_per_mano_colore3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"due_uomini_che_si_tengono_per_mano_colore3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"due_uomini_che_si_tengono_per_mano_colore3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"due_uomini_che_si_tengono_per_mano_colore3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"due_uomini_che_si_tengono_per_mano_colore4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"due_uomini_che_si_tengono_per_mano_colore4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"due_uomini_che_si_tengono_per_mano_colore4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"due_uomini_che_si_tengono_per_mano_colore4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"due_uomini_che_si_tengono_per_mano_colore5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"due_uomini_che_si_tengono_per_mano_colore5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"due_uomini_che_si_tengono_per_mano_colore5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"due_uomini_che_si_tengono_per_mano_colore5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"due_donne_che_si_tengono_per_mano", "1F46D"},
    {"due_donne_che_si_tengono_per_mano_colore1", "1F46D-1F3FB"},
    {"due_donne_che_si_tengono_per_mano_colore2", "1F46D-1F3FC"},
    {"due_donne_che_si_tengono_per_mano_colore3", "1F46D-1F3FD"},
    {"due_donne_che_si_tengono_per_mano_colore4", "1F46D-1F3FE"},
    {"due_donne_che_si_tengono_per_mano_colore5", "1F46D-1F3FF"},
    {"due_donne_che_si_tengono_per_mano_colore1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"due_donne_che_si_tengono_per_mano_colore1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"due_donne_che_si_tengono_per_mano_colore1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"due_donne_che_si_tengono_per_mano_colore1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"due_donne_che_si_tengono_per_mano_colore2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"due_donne_che_si_tengono_per_mano_colore2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"due_donne_che_si_tengono_per_mano_colore2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"due_donne_che_si_tengono_per_mano_colore2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"due_donne_che_si_tengono_per_mano_colore3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"due_donne_che_si_tengono_per_mano_colore3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"due_donne_che_si_tengono_per_mano_colore3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"due_donne_che_si_tengono_per_mano_colore3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"due_donne_che_si_tengono_per_mano_colore4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"due_donne_che_si_tengono_per_mano_colore4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"due_donne_che_si_tengono_per_mano_colore4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"due_donne_che_si_tengono_per_mano_colore4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"due_donne_che_si_tengono_per_mano_colore5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"due_donne_che_si_tengono_per_mano_colore5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"due_donne_che_si_tengono_per_mano_colore5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"due_donne_che_si_tengono_per_mano_colore5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"agente_di_polizia", "1F46E"},
    {"agente_di_polizia_colore1", "1F46E-1F3FB"},
    {"agente_di_polizia_colore2", "1F46E-1F3FC"},
    {"agente_di_polizia_colore3", "1F46E-1F3FD"},
    {"agente_di_polizia_colore4", "1F46E-1F3FE"},
    {"agente_di_polizia_colore5", "1F46E-1F3FF"},
    {"persone_con_orecchie_da_coniglio", "1F46F"},
    {"persone_con_orecchie_da_coniglio_colore1", "1F46F-1F3FB"},
    {"persone_con_orecchie_da_coniglio_colore2", "1F46F-1F3FC"},
    {"persone_con_orecchie_da_coniglio_colore3", "1F46F-1F3FD"},
    {"persone_con_orecchie_da_coniglio_colore4", "1F46F-1F3FE"},
    {"persone_con_orecchie_da_coniglio_colore5", "1F46F-1F3FF"},
    {"persone_con_orecchie_da_coniglio_colore1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"persone_con_orecchie_da_coniglio_colore1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"persone_con_orecchie_da_coniglio_colore1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"persone_con_orecchie_da_coniglio_colore1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"persone_con_orecchie_da_coniglio_colore2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"persone_con_orecchie_da_coniglio_colore2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"persone_con_orecchie_da_coniglio_colore2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"persone_con_orecchie_da_coniglio_colore2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"persone_con_orecchie_da_coniglio_colore3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"persone_con_orecchie_da_coniglio_colore3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"persone_con_orecchie_da_coniglio_colore3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"persone_con_orecchie_da_coniglio_colore3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"persone_con_orecchie_da_coniglio_colore4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"persone_con_orecchie_da_coniglio_colore4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"persone_con_orecchie_da_coniglio_colore4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"persone_con_orecchie_da_coniglio_colore4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"persone_con_orecchie_da_coniglio_colore5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"persone_con_orecchie_da_coniglio_colore5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"persone_con_orecchie_da_coniglio_colore5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"persone_con_orecchie_da_coniglio_colore5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"persona_con_velo", "1F470"},
    {"persona_con_velo_colore1", "1F470-1F3FB"},
    {"persona_con_velo_colore2", "1F470-1F3FC"},
    {"persona_con_velo_colore3", "1F470-1F3FD"},
    {"persona_con_velo_colore4", "1F470-1F3FE"},
    {"persona_con_velo_colore5", "1F470-1F3FF"},
    {"persona_bionda", "1F471"},
    {"persona_bionda_colore1", "1F471-1F3FB"},
    {"persona_bionda_colore2", "1F471-1F3FC"},
    {"persona_bionda_colore3", "1F471-1F3FD"},
    {"persona_bionda_colore4", "1F471-1F3FE"},
    {"persona_bionda_colore5", "1F471-1F3FF"},
    {"uomo_con_zucchetto_cinese", "1F472"},
    {"uomo_con_zucchetto_cinese_colore1", "1F472-1F3FB"},
    {"uomo_con_zucchetto_cinese_colore2", "1F472-1F3FC"},
    {"uomo_con_zucchetto_cinese_colore3", "1F472-1F3FD"},
    {"uomo_con_zucchetto_cinese_colore4", "1F472-1F3FE"},
    {"uomo_con_zucchetto_cinese_colore5", "1F472-1F3FF"},
    {"persona_con_turbante", "1F473"},
    {"persona_con_turbante_colore1", "1F473-1F3FB"},
    {"persona_con_turbante_colore2", "1F473-1F3FC"},
    {"persona_con_turbante_colore3", "1F473-1F3FD"},
    {"persona_con_turbante_colore4", "1F473-1F3FE"},
    {"persona_con_turbante_colore5", "1F473-1F3FF"},
    {"uomo_anziano", "1F474"},
    {"uomo_anziano_colore1", "1F474-1F3FB"},
    {"uomo_anziano_colore2", "1F474-1F3FC"},
    {"uomo_anziano_colore3", "1F474-1F3FD"},
    {"uomo_anziano_colore4", "1F474-1F3FE"},
    {"uomo_anziano_colore5", "1F474-1F3FF"},
    {"donna_anziana", "1F475"},
    {"donna_anziana_colore1", "1F475-1F3FB"},
    {"donna_anziana_colore2", "1F475-1F3FC"},
    {"donna_anziana_colore3", "1F475-1F3FD"},
    {"donna_anziana_colore4", "1F475-1F3FE"},
    {"donna_anziana_colore5", "1F475-1F3FF"},
    {"neonato", "1F476"},
    {"neonato_colore1", "1F476-1F3FB"},
    {"neonato_colore2", "1F476-1F3FC"},
    {"neonato_colore3", "1F476-1F3FD"},
    {"neonato_colore4", "1F476-1F3FE"},
    {"neonato_colore5", "1F476-1F3FF"},
    {"operaio_edile", "1F477"},
    {"operaio_edile_colore1", "1F477-1F3FB"},
    {"operaio_edile_colore2", "1F477-1F3FC"},
    {"operaio_edile_colore3", "1F477-1F3FD"},
    {"operaio_edile_colore4", "1F477-1F3FE"},
    {"operaio_edile_colore5", "1F477-1F3FF"},
    {"principessa", "1F478"},
    {"principessa_colore1", "1F478-1F3FB"},
    {"principessa_colore2", "1F478-1F3FC"},
    {"principessa_colore3", "1F478-1F3FD"},
    {"principessa_colore4", "1F478-1F3FE"},
    {"principessa_colore5", "1F478-1F3FF"},
    {"orco", "1F479"},
    {"goblin", "1F47A"},
    {"fantasma", "1F47B"},
    {"angioletto", "1F47C"},
    {"angioletto_colore1", "1F47C-1F3FB"},
    {"angioletto_colore2", "1F47C-1F3FC"},
    {"angioletto_colore3", "1F47C-1F3FD"},
    {"angioletto_colore4", "1F47C-1F3FE"},
    {"angioletto_colore5", "1F47C-1F3FF"},
    {"alieno", "1F47D"},
    {"mostro_alieno", "1F47E"},
    {"faccina_arrabbiata_con_corna", "1F47F"},
    {"teschio", "1F480"},
    {"persona_con_mano_capovolta", "1F481"},
    {"persona_con_mano_capovolta_colore1", "1F481-1F3FB"},
    {"persona_con_mano_capovolta_colore2", "1F481-1F3FC"},
    {"persona_con_mano_capovolta_colore3", "1F481-1F3FD"},
    {"persona_con_mano_capovolta_colore4", "1F481-1F3FE"},
    {"persona_con_mano_capovolta_colore5", "1F481-1F3FF"},
    {"guardia", "1F482"},
    {"guardia_colore1", "1F482-1F3FB"},
    {"guardia_colore2", "1F482-1F3FC"},
    {"guardia_colore3", "1F482-1F3FD"},
    {"guardia_colore4", "1F482-1F3FE"},
    {"guardia_colore5", "1F482-1F3FF"},
    {"donna_che_balla", "1F483"},
    {"donna_che_balla_colore1", "1F483-1F3FB"},
    {"donna_che_balla_colore2", "1F483-1F3FC"},
    {"donna_che_balla_colore3", "1F483-1F3FD"},
    {"donna_che_balla_colore4", "1F483-1F3FE"},
    {"donna_che_balla_colore5", "1F483-1F3FF"},
    {"rossetto", "1F484"},
    {"smalto_per_unghie", "1F485"},
    {"smalto_per_unghie_colore1", "1F485-1F3FB"},
    {"smalto_per_unghie_colore2", "1F485-1F3FC"},
    {"smalto_per_unghie_colore3", "1F485-1F3FD"},
    {"smalto_per_unghie_colore4", "1F485-1F3FE"},
    {"smalto_per_unghie_colore5", "1F485-1F3FF"},
    {"persona_che_riceve_un_massaggio", "1F486"},
    {"persona_che_riceve_un_massaggio_colore1", "1F486-1F3FB"},
    {"persona_che_riceve_un_massaggio_colore2", "1F486-1F3FC"},
    {"persona_che_riceve_un_massaggio_colore3", "1F486-1F3FD"},
    {"persona_che_riceve_un_massaggio_colore4", "1F486-1F3FE"},
    {"persona_che_riceve_un_massaggio_colore5", "1F486-1F3FF"},
    {"taglio_di_capelli", "1F487"},
    {"taglio_di_capelli_colore1", "1F487-1F3FB"},
    {"taglio_di_capelli_colore2", "1F487-1F3FC"},
    {"taglio_di_capelli_colore3", "1F487-1F3FD"},
    {"taglio_di_capelli_colore4", "1F487-1F3FE"},
    {"taglio_di_capelli_colore5", "1F487-1F3FF"},
    {"barbiere", "1F488"},
    {"siringa", "1F489"},
    {"pillola", "1F48A"},
    {"impronta_della_bocca", "1F48B"},
    {"lettera_damore", "1F48C"},
    {"anello", "1F48D"},
    {"gemma", "1F48E"},
    {"bacio_tra_coppia", "1F48F"},
    {"bacio_tra_coppia_colore1", "1F48F-1F3FB"},
    {"bacio_tra_coppia_colore2", "1F48F-1F3FC"},
    {"bacio_tra_coppia_colore3", "1F48F-1F3FD"},
    {"bacio_tra_coppia_colore4", "1F48F-1F3FE"},
    {"bacio_tra_coppia_colore5", "1F48F-1F3FF"},
    {"bacio_tra_coppia_colore1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bacio_tra_coppia_colore1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bacio_tra_coppia_colore1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bacio_tra_coppia_colore1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bacio_tra_coppia_colore2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bacio_tra_coppia_colore2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bacio_tra_coppia_colore2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bacio_tra_coppia_colore2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bacio_tra_coppia_colore3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bacio_tra_coppia_colore3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bacio_tra_coppia_colore3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bacio_tra_coppia_colore3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bacio_tra_coppia_colore4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bacio_tra_coppia_colore4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bacio_tra_coppia_colore4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bacio_tra_coppia_colore4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bacio_tra_coppia_colore5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bacio_tra_coppia_colore5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bacio_tra_coppia_colore5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bacio_tra_coppia_colore5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"mazzo_di_fiori", "1F490"},
    {"coppia_con_cuore", "1F491"},
    {"coppia_con_cuore_colore1", "1F491-1F3FB"},
    {"coppia_con_cuore_colore2", "1F491-1F3FC"},
    {"coppia_con_cuore_colore3", "1F491-1F3FD"},
    {"coppia_con_cuore_colore4", "1F491-1F3FE"},
    {"coppia_con_cuore_colore5", "1F491-1F3FF"},
    {"coppia_con_cuore_colore1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"coppia_con_cuore_colore1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"coppia_con_cuore_colore1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"coppia_con_cuore_colore1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"coppia_con_cuore_colore2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"coppia_con_cuore_colore2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"coppia_con_cuore_colore2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"coppia_con_cuore_colore2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"coppia_con_cuore_colore3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"coppia_con_cuore_colore3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"coppia_con_cuore_colore3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"coppia_con_cuore_colore3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"coppia_con_cuore_colore4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"coppia_con_cuore_colore4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"coppia_con_cuore_colore4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"coppia_con_cuore_colore4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"coppia_con_cuore_colore5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"coppia_con_cuore_colore5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"coppia_con_cuore_colore5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"coppia_con_cuore_colore5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"chiesa_per_matrimonio", "1F492"},
    {"cuore_che_batte", "1F493"},
    {"cuore_infranto", "1F494"},
    {"due_cuori", "1F495"},
    {"cuore_che_luccica", "1F496"},
    {"cuore_che_cresce", "1F497"},
    {"cuore_con_freccia", "1F498"},
    {"cuore_blu", "1F499"},
    {"cuore_verde", "1F49A"},
    {"cuore_giallo", "1F49B"},
    {"cuore_viola", "1F49C"},
    {"cuore_con_fiocco", "1F49D"},
    {"cuori_che_girano", "1F49E"},
    {"decorazione_con_cuore", "1F49F"},
    {"petalo_di_fiore", "1F4A0"},
    {"lampadina", "1F4A1"},
    {"rabbia", "1F4A2"},
    {"bomba", "1F4A3"},
    {"sonno", "1F4A4"},
    {"collisione", "1F4A5"},
    {"gocce_di_sudore", "1F4A6"},
    {"goccia", "1F4A7"},
    {"nuvola_di_polvere", "1F4A8"},
    {"cacca", "1F4A9"},
    {"bicipite", "1F4AA"},
    {"bicipite_colore1", "1F4AA-1F3FB"},
    {"bicipite_colore2", "1F4AA-1F3FC"},
    {"bicipite_colore3", "1F4AA-1F3FD"},
    {"bicipite_colore4", "1F4AA-1F3FE"},
    {"bicipite_colore5", "1F4AA-1F3FF"},
    {"stella_con_scia", "1F4AB"},
    {"fumetto", "1F4AC"},
    {"nuvoletta", "1F4AD"},
    {"fiore_bianco", "1F4AE"},
    {"100_punti", "1F4AF"},
    {"sacco_di_soldi", "1F4B0"},
    {"cambio_valuta", "1F4B1"},
    {"dollaro", "1F4B2"},
    {"carta_di_credito", "1F4B3"},
    {"banconota_yen", "1F4B4"},
    {"banconota_dollaro", "1F4B5"},
    {"banconota_euro", "1F4B6"},
    {"banconota_sterlina", "1F4B7"},
    {"soldi_con_le_ali", "1F4B8"},
    {"grafico_ascendente", "1F4B9"},
    {"sedile", "1F4BA"},
    {"computer_portatile", "1F4BB"},
    {"valigetta_24_ore", "1F4BC"},
    {"minidisc", "1F4BD"},
    {"floppy_disc", "1F4BE"},
    {"disco_ottico", "1F4BF"},
    {"dvd", "1F4C0"},
    {"cartella_file", "1F4C1"},
    {"cartella_aperta", "1F4C2"},
    {"pagina_con_piega", "1F4C3"},
    {"pagina", "1F4C4"},
    {"calendario", "1F4C5"},
    {"calendario_a_strappo", "1F4C6"},
    {"schedario", "1F4C7"},
    {"grafico_con_andamento_positivo", "1F4C8"},
    {"grafico_con_andamento_negativo", "1F4C9"},
    {"grafico_a_barre", "1F4CA"},
    {"portablocco", "1F4CB"},
    {"puntina", "1F4CC"},
    {"puntina_arrotondata", "1F4CD"},
    {"graffetta", "1F4CE"},
    {"righello", "1F4CF"},
    {"squadra", "1F4D0"},
    {"etichette_segnalibro", "1F4D1"},
    {"quaderno_ad_anelli", "1F4D2"},
    {"quaderno", "1F4D3"},
    {"quaderno_con_copertina_decorata", "1F4D4"},
    {"libro_chiuso", "1F4D5"},
    {"libro_aperto", "1F4D6"},
    {"libro_verde", "1F4D7"},
    {"libro_blu", "1F4D8"},
    {"libro_arancione", "1F4D9"},
    {"libri", "1F4DA"},
    {"tesserino_per_nome", "1F4DB"},
    {"pergamena", "1F4DC"},
    {"memo", "1F4DD"},
    {"cornetta", "1F4DE"},
    {"cercapersone", "1F4DF"},
    {"fax", "1F4E0"},
    {"antenna_satellitare", "1F4E1"},
    {"altoparlante", "1F4E2"},
    {"megafono", "1F4E3"},
    {"posta_inviata", "1F4E4"},
    {"posta_ricevuta", "1F4E5"},
    {"pacco", "1F4E6"},
    {"email", "1F4E7"},
    {"posta_in_arrivo", "1F4E8"},
    {"posta_in_uscita", "1F4E9"},
    {"cassetta_postale_chiusa_bandierina_abbassata", "1F4EA"},
    {"cassetta_postale_chiusa_bandierina_alzata", "1F4EB"},
    {"cassetta_postale_aperta_bandierina_alzata", "1F4EC"},
    {"cassetta_postale_aperta_bandierina_abbassata", "1F4ED"},
    {"cassetta_delle_lettere", "1F4EE"},
    {"corno_postale", "1F4EF"},
    {"quotidiano", "1F4F0"},
    {"telefono_cellulare", "1F4F1"},
    {"telefono_cellulare_con_freccia", "1F4F2"},
    {"modalita_vibrazione", "1F4F3"},
    {"cellulare_spento", "1F4F4"},
    {"simbolo_che_vieta_luso_dei_cellulari", "1F4F5"},
    {"segnale_cellulare", "1F4F6"},
    {"fotocamera", "1F4F7"},
    {"fotocamera_con_flash", "1F4F8"},
    {"videocamera", "1F4F9"},
    {"televisore", "1F4FA"},
    {"radio", "1F4FB"},
    {"videocassetta", "1F4FC"},
    {"proiettore_cinematografico", "1F4FD"},
    {"collana_da_preghiera", "1F4FF"},
    {"pulsante_di_riproduzione_casuale", "1F500"},
    {"pulsante_di_ripetizione_della_riproduzione", "1F501"},
    {"pulsante_di_ripetizione_della_riproduzione_di_una_traccia", "1F502"},
    {"frecce_verticali_che_ruotano_in_senso_orario", "1F503"},
    {"frecce_che_ruotano_in_senso_antiorario", "1F504"},
    {"luminosita_bassa", "1F505"},
    {"luminosita_elevata", "1F506"},
    {"altoparlante_disattivato", "1F507"},
    {"altoparlante_a_volume_basso", "1F508"},
    {"altoparlante_a_volume_intermedio", "1F509"},
    {"altoparlante_a_volume_alto", "1F50A"},
    {"batteria", "1F50B"},
    {"spina_elettrica", "1F50C"},
    {"lente_di_ingrandimento_rivolta_a_sinistra", "1F50D"},
    {"lente_di_ingrandimento_rivolta_a_destra", "1F50E"},
    {"lucchetto_con_penna_stilo", "1F50F"},
    {"lucchetto_chiuso_con_chiave", "1F510"},
    {"chiave", "1F511"},
    {"lucchetto_chiuso", "1F512"},
    {"lucchetto_aperto", "1F513"},
    {"campana", "1F514"},
    {"campana_sbarrata", "1F515"},
    {"segnalibro", "1F516"},
    {"collegamento", "1F517"},
    {"pulsante_rotondo", "1F518"},
    {"freccia_back", "1F519"},
    {"freccia_end", "1F51A"},
    {"freccia_on", "1F51B"},
    {"freccia_soon", "1F51C"},
    {"freccia_top", "1F51D"},
    {"simbolo_di_divieto_ai_minorenni", "1F51E"},
    {"tasto_10", "1F51F"},
    {"pulsante_con_lettere_latine_maiuscole", "1F520"},
    {"pulsante_con_lettere_latine_minuscole", "1F521"},
    {"pulsante_con_numeri", "1F522"},
    {"pulsante_con_simboli", "1F523"},
    {"pulsante_con_lettere_latine", "1F524"},
    {"fuoco", "1F525"},
    {"torcia", "1F526"},
    {"chiave_inglese", "1F527"},
    {"martello", "1F528"},
    {"vite_e_bullone", "1F529"},
    {"coltello_da_cucina", "1F52A"},
    {"pistola_ad_acqua", "1F52B"},
    {"microscopio", "1F52C"},
    {"telescopio", "1F52D"},
    {"sfera_di_cristallo", "1F52E"},
    {"stella_a_sei_punte", "1F52F"},
    {"ideogramma_giapponese_di_principiante", "1F530"},
    {"tridente", "1F531"},
    {"tasto_quadrato_bianco_con_bordo_nero", "1F532"},
    {"tasto_quadrato_nero_con_bordo_bianco", "1F533"},
    {"cerchio_rosso", "1F534"},
    {"cerchio_blu", "1F535"},
    {"rombo_arancione_grande", "1F536"},
    {"rombo_blu_grande", "1F537"},
    {"rombo_arancione_piccolo", "1F538"},
    {"rombo_blu_piccolo", "1F539"},
    {"triangolo_rosso_con_punta_verso_lalto", "1F53A"},
    {"triangolo_rosso_con_punta_verso_il_basso", "1F53B"},
    {"pulsante_a_triangolo_rivolto_verso_lalto", "1F53C"},
    {"pulsante_a_triangolo_rivolto_verso_il_basso", "1F53D"},
    {"om", "1F549"},
    {"colomba", "1F54A"},
    {"kaaba", "1F54B"},
    {"moschea", "1F54C"},
    {"sinagoga", "1F54D"},
    {"menorah", "1F54E"},
    {"ore_una", "1F550"},
    {"ore_due", "1F551"},
    {"ore_tre", "1F552"},
    {"ore_quattro", "1F553"},
    {"ore_cinque", "1F554"},
    {"ore_sei", "1F555"},
    {"ore_sette", "1F556"},
    {"ore_otto", "1F557"},
    {"ore_nove", "1F558"},
    {"ore_dieci", "1F559"},
    {"ore_undici", "1F55A"},
    {"ore_dodici", "1F55B"},
    {"ore_una_e_mezza", "1F55C"},
    {"ore_due_e_mezza", "1F55D"},
    {"ore_tre_e_mezza", "1F55E"},
    {"ore_quattro_e_mezza", "1F55F"},
    {"ore_cinque_e_mezza", "1F560"},
    {"ore_sei_e_mezza", "1F561"},
    {"ore_sette_e_mezza", "1F562"},
    {"ore_otto_e_mezza", "1F563"},
    {"ore_nove_e_mezza", "1F564"},
    {"ore_dieci_e_mezza", "1F565"},
    {"ore_undici_e_mezza", "1F566"},
    {"ore_dodici_e_mezza", "1F567"},
    {"candela", "1F56F"},
    {"orologio_da_mensola", "1F570"},
    {"buco", "1F573"},
    {"uomo_con_completo_che_levita", "1F574"},
    {"uomo_con_completo_che_levita_colore1", "1F574-1F3FB"},
    {"uomo_con_completo_che_levita_colore2", "1F574-1F3FC"},
    {"uomo_con_completo_che_levita_colore3", "1F574-1F3FD"},
    {"uomo_con_completo_che_levita_colore4", "1F574-1F3FE"},
    {"uomo_con_completo_che_levita_colore5", "1F574-1F3FF"},
    {"detective", "1F575"},
    {"detective_colore1", "1F575-1F3FB"},
    {"detective_colore2", "1F575-1F3FC"},
    {"detective_colore3", "1F575-1F3FD"},
    {"detective_colore4", "1F575-1F3FE"},
    {"detective_colore5", "1F575-1F3FF"},
    {"occhiali_da_sole", "1F576"},
    {"ragno", "1F577"},
    {"ragnatela", "1F578"},
    {"joystick", "1F579"},
    {"uomo_che_balla", "1F57A"},
    {"uomo_che_balla_colore1", "1F57A-1F3FB"},
    {"uomo_che_balla_colore2", "1F57A-1F3FC"},
    {"uomo_che_balla_colore3", "1F57A-1F3FD"},
    {"uomo_che_balla_colore4", "1F57A-1F3FE"},
    {"uomo_che_balla_colore5", "1F57A-1F3FF"},
    {"graffette_attaccate", "1F587"},
    {"penna_a_sfera", "1F58A"},
    {"penna_stilografica", "1F58B"},
    {"pennello", "1F58C"},
    {"pastello_a_cera", "1F58D"},
    {"mano_aperta", "1F590"},
    {"mano_aperta_colore1", "1F590-1F3FB"},
    {"mano_aperta_colore2", "1F590-1F3FC"},
    {"mano_aperta_colore3", "1F590-1F3FD"},
    {"mano_aperta_colore4", "1F590-1F3FE"},
    {"mano_aperta_colore5", "1F590-1F3FF"},
    {"dito_medio", "1F595"},
    {"dito_medio_colore1", "1F595-1F3FB"},
    {"dito_medio_colore2", "1F595-1F3FC"},
    {"dito_medio_colore3", "1F595-1F3FD"},
    {"dito_medio_colore4", "1F595-1F3FE"},
    {"dito_medio_colore5", "1F595-1F3FF"},
    {"saluto_vulcaniano", "1F596"},
    {"saluto_vulcaniano_colore1", "1F596-1F3FB"},
    {"saluto_vulcaniano_colore2", "1F596-1F3FC"},
    {"saluto_vulcaniano_colore3", "1F596-1F3FD"},
    {"saluto_vulcaniano_colore4", "1F596-1F3FE"},
    {"saluto_vulcaniano_colore5", "1F596-1F3FF"},
    {"cuore_nero", "1F5A4"},
    {"computer_fisso", "1F5A5"},
    {"stampante", "1F5A8"},
    {"mouse", "1F5B1"},
    {"trackball", "1F5B2"},
    {"quadro_incorniciato", "1F5BC"},
    {"divisori_per_schedario", "1F5C2"},
    {"schedario_da_tavolo", "1F5C3"},
    {"schedario_da_ufficio", "1F5C4"},
    {"cestino", "1F5D1"},
    {"blocco_per_appunti_con_spirale", "1F5D2"},
    {"calendario_a_spirale", "1F5D3"},
    {"morsetto", "1F5DC"},
    {"chiave_antica", "1F5DD"},
    {"quotidiano_arrotolato", "1F5DE"},
    {"pugnale", "1F5E1"},
    {"persona_che_parla", "1F5E3"},
    {"fumetto_a_sinistra", "1F5E8"},
    {"nuvoletta_rabbia", "1F5EF"},
    {"urna_elettorale_con_scheda", "1F5F3"},
    {"mappa_mondiale", "1F5FA"},
    {"monte_fuji", "1F5FB"},
    {"torre_di_tokyo", "1F5FC"},
    {"statua_della_liberta", "1F5FD"},
    {"mappa_del_giappone", "1F5FE"},
    {"moai", "1F5FF"},
    {"faccina_con_un_gran_sorriso", "1F600"},
    {"faccina_raggiante_con_occhi_felici", "1F601"},
    {"faccina_con_lacrime_di_gioia", "1F602"},
    {"faccina_con_sorriso_e_occhi_spalancati", "1F603"},
    {"faccina_con_sorriso_e_occhi_sorridenti", "1F604"},
    {"faccina_con_sorriso_e_goccia_di_sudore", "1F605"},
    {"sorriso_a_bocca_aperta_con_occhi_chiusi", "1F606"},
    {"faccina_con_sorriso_e_aureola", "1F607"},
    {"faccina_con_sorriso_e_corna", "1F608"},
    {"faccina_che_fa_locchiolino", "1F609"},
    {"faccina_con_occhi_sorridenti", "1F60A"},
    {"faccina_che_si_lecca_i_baffi", "1F60B"},
    {"faccina_sollevata", "1F60C"},
    {"faccina_con_sorriso_e_occhi_a_cuore", "1F60D"},
    {"faccina_con_sorriso_e_occhiali_da_sole", "1F60E"},
    {"faccina_con_sorrisetto", "1F60F"},
    {"faccina_neutra", "1F610"},
    {"faccina_inespressiva", "1F611"},
    {"faccina_contrariata", "1F612"},
    {"faccina_abbattuta_sudata", "1F613"},
    {"faccina_pensierosa", "1F614"},
    {"faccina_confusa", "1F615"},
    {"faccina_frustrata", "1F616"},
    {"faccina_che_bacia", "1F617"},
    {"faccina_che_manda_un_bacio", "1F618"},
    {"faccina_che_bacia_con_occhi_sorridenti", "1F619"},
    {"faccina_che_bacia_con_occhi_chiusi", "1F61A"},
    {"faccina_che_mostra_la_lingua", "1F61B"},
    {"faccina_che_fa_locchiolino_e_mostra_la_lingua", "1F61C"},
    {"faccina_che_strizza_gli_occhi_e_mostra_la_lingua", "1F61D"},
    {"faccina_delusa", "1F61E"},
    {"faccina_preoccupata", "1F61F"},
    {"faccina_arrabbiata", "1F620"},
    {"faccina_accigliata", "1F621"},
    {"faccina_che_piange", "1F622"},
    {"faccina_perseverante", "1F623"},
    {"faccina_che_sbuffa", "1F624"},
    {"faccina_delusa_ma_sollevata", "1F625"},
    {"faccina_imbronciata_con_bocca_aperta", "1F626"},
    {"faccina_angosciata", "1F627"},
    {"faccina_impaurita", "1F628"},
    {"faccina_esausta", "1F629"},
    {"faccina_assonnata", "1F62A"},
    {"faccina_stanca", "1F62B"},
    {"faccina_con_smorfia", "1F62C"},
    {"faccina_disperata", "1F62D"},
    {"faccina_con_bocca_aperta", "1F62E"},
    {"faccina_sorpresa", "1F62F"},
    {"faccina_sudata_in_ansia", "1F630"},
    {"faccina_terrorizzata", "1F631"},
    {"faccina_stupita", "1F632"},
    {"faccina_imbarazzata", "1F633"},
    {"faccina_che_dorme", "1F634"},
    {"faccina_frastornata", "1F635"},
    {"faccina_senza_bocca", "1F636"},
    {"faccina_con_mascherina", "1F637"},
    {"gatto_che_sogghigna", "1F638"},
    {"gatto_con_lacrime_di_gioia", "1F639"},
    {"gatto_che_sorride", "1F63A"},
    {"gatto_innamorato", "1F63B"},
    {"gatto_con_sorriso_sarcastico", "1F63C"},
    {"gatto_che_manda_baci", "1F63D"},
    {"gatto_imbronciato", "1F63E"},
    {"gatto_che_piange", "1F63F"},
    {"gatto_esterrefatto", "1F640"},
    {"faccina_leggermente_imbronciata", "1F641"},
    {"faccina_con_sorriso_accennato", "1F642"},
    {"faccina_sottosopra", "1F643"},
    {"faccina_con_occhi_al_cielo", "1F644"},
    {"persona_che_fa_segno_di_no", "1F645"},
    {"persona_che_fa_segno_di_no_colore1", "1F645-1F3FB"},
    {"persona_che_fa_segno_di_no_colore2", "1F645-1F3FC"},
    {"persona_che_fa_segno_di_no_colore3", "1F645-1F3FD"},
    {"persona_che_fa_segno_di_no_colore4", "1F645-1F3FE"},
    {"persona_che_fa_segno_di_no_colore5", "1F645-1F3FF"},
    {"persona_con_gesto_ok", "1F646"},
    {"persona_con_gesto_ok_colore1", "1F646-1F3FB"},
    {"persona_con_gesto_ok_colore2", "1F646-1F3FC"},
    {"persona_con_gesto_ok_colore3", "1F646-1F3FD"},
    {"persona_con_gesto_ok_colore4", "1F646-1F3FE"},
    {"persona_con_gesto_ok_colore5", "1F646-1F3FF"},
    {"persona_che_fa_un_inchino_profondo", "1F647"},
    {"persona_che_fa_un_inchino_profondo_colore1", "1F647-1F3FB"},
    {"persona_che_fa_un_inchino_profondo_colore2", "1F647-1F3FC"},
    {"persona_che_fa_un_inchino_profondo_colore3", "1F647-1F3FD"},
    {"persona_che_fa_un_inchino_profondo_colore4", "1F647-1F3FE"},
    {"persona_che_fa_un_inchino_profondo_colore5", "1F647-1F3FF"},
    {"non_vedo", "1F648"},
    {"non_sento", "1F649"},
    {"non_parlo", "1F64A"},
    {"persona_con_mano_alzata", "1F64B"},
    {"persona_con_mano_alzata_colore1", "1F64B-1F3FB"},
    {"persona_con_mano_alzata_colore2", "1F64B-1F3FC"},
    {"persona_con_mano_alzata_colore3", "1F64B-1F3FD"},
    {"persona_con_mano_alzata_colore4", "1F64B-1F3FE"},
    {"persona_con_mano_alzata_colore5", "1F64B-1F3FF"},
    {"mani_alzate", "1F64C"},
    {"mani_alzate_colore1", "1F64C-1F3FB"},
    {"mani_alzate_colore2", "1F64C-1F3FC"},
    {"mani_alzate_colore3", "1F64C-1F3FD"},
    {"mani_alzate_colore4", "1F64C-1F3FE"},
    {"mani_alzate_colore5", "1F64C-1F3FF"},
    {"persona_corrucciata", "1F64D"},
    {"persona_corrucciata_colore1", "1F64D-1F3FB"},
    {"persona_corrucciata_colore2", "1F64D-1F3FC"},
    {"persona_corrucciata_colore3", "1F64D-1F3FD"},
    {"persona_corrucciata_colore4", "1F64D-1F3FE"},
    {"persona_corrucciata_colore5", "1F64D-1F3FF"},
    {"persona_imbronciata", "1F64E"},
    {"persona_imbronciata_colore1", "1F64E-1F3FB"},
    {"persona_imbronciata_colore2", "1F64E-1F3FC"},
    {"persona_imbronciata_colore3", "1F64E-1F3FD"},
    {"persona_imbronciata_colore4", "1F64E-1F3FE"},
    {"persona_imbronciata_colore5", "1F64E-1F3FF"},
    {"mani_giunte", "1F64F"},
    {"mani_giunte_colore1", "1F64F-1F3FB"},
    {"mani_giunte_colore2", "1F64F-1F3FC"},
    {"mani_giunte_colore3", "1F64F-1F3FD"},
    {"mani_giunte_colore4", "1F64F-1F3FE"},
    {"mani_giunte_colore5", "1F64F-1F3FF"},
    {"razzo", "1F680"},
    {"elicottero", "1F681"},
    {"locomotiva", "1F682"},
    {"vagone", "1F683"},
    {"treno_alta_velocita", "1F684"},
    {"treno_alta_velocita_punta_arrotondata", "1F685"},
    {"treno", "1F686"},
    {"metropolitana", "1F687"},
    {"metrotranvia", "1F688"},
    {"stazione", "1F689"},
    {"tram", "1F68A"},
    {"vagone_del_tram", "1F68B"},
    {"bus", "1F68C"},
    {"bus_in_arrivo", "1F68D"},
    {"filobus", "1F68E"},
    {"fermata_dellautobus", "1F68F"},
    {"pulmino", "1F690"},
    {"ambulanza", "1F691"},
    {"camion_dei_pompieri", "1F692"},
    {"macchina_della_polizia", "1F693"},
    {"macchina_della_polizia_in_arrivo", "1F694"},
    {"taxi", "1F695"},
    {"taxi_in_arrivo", "1F696"},
    {"auto", "1F697"},
    {"auto_in_arrivo", "1F698"},
    {"suv", "1F699"},
    {"camion", "1F69A"},
    {"autoarticolato", "1F69B"},
    {"trattore", "1F69C"},
    {"monorotaia", "1F69D"},
    {"ferrovia_di_montagna", "1F69E"},
    {"ferrovia_sospesa", "1F69F"},
    {"funivia", "1F6A0"},
    {"funivia_aerea", "1F6A1"},
    {"nave", "1F6A2"},
    {"persona_in_barca_a_remi", "1F6A3"},
    {"persona_in_barca_a_remi_colore1", "1F6A3-1F3FB"},
    {"persona_in_barca_a_remi_colore2", "1F6A3-1F3FC"},
    {"persona_in_barca_a_remi_colore3", "1F6A3-1F3FD"},
    {"persona_in_barca_a_remi_colore4", "1F6A3-1F3FE"},
    {"persona_in_barca_a_remi_colore5", "1F6A3-1F3FF"},
    {"motoscafo", "1F6A4"},
    {"semaforo_orizzontale", "1F6A5"},
    {"semaforo_verticale", "1F6A6"},
    {"area_cantiere", "1F6A7"},
    {"lampeggiante", "1F6A8"},
    {"bandierina_rossa", "1F6A9"},
    {"porta", "1F6AA"},
    {"segnale_di_divieto", "1F6AB"},
    {"sigaretta", "1F6AC"},
    {"segnale_di_vietato_fumare", "1F6AD"},
    {"simbolo_per_la_raccolta_dei_rifiuti", "1F6AE"},
    {"simbolo_del_divieto_di_gettare_rifiuti", "1F6AF"},
    {"acqua_potabile", "1F6B0"},
    {"simbolo_di_acqua_non_potabile", "1F6B1"},
    {"bicicletta", "1F6B2"},
    {"segnale_di_divieto_di_transito_delle_biciclette", "1F6B3"},
    {"ciclista", "1F6B4"},
    {"ciclista_colore1", "1F6B4-1F3FB"},
    {"ciclista_colore2", "1F6B4-1F3FC"},
    {"ciclista_colore3", "1F6B4-1F3FD"},
    {"ciclista_colore4", "1F6B4-1F3FE"},
    {"ciclista_colore5", "1F6B4-1F3FF"},
    {"ciclista_di_mountain_bike", "1F6B5"},
    {"ciclista_di_mountain_bike_colore1", "1F6B5-1F3FB"},
    {"ciclista_di_mountain_bike_colore2", "1F6B5-1F3FC"},
    {"ciclista_di_mountain_bike_colore3", "1F6B5-1F3FD"},
    {"ciclista_di_mountain_bike_colore4", "1F6B5-1F3FE"},
    {"ciclista_di_mountain_bike_colore5", "1F6B5-1F3FF"},
    {"pedone", "1F6B6"},
    {"pedone_colore1", "1F6B6-1F3FB"},
    {"pedone_colore2", "1F6B6-1F3FC"},
    {"pedone_colore3", "1F6B6-1F3FD"},
    {"pedone_colore4", "1F6B6-1F3FE"},
    {"pedone_colore5", "1F6B6-1F3FF"},
    {"simbolo_del_divieto_di_transito_pedoni", "1F6B7"},
    {"attraversamento_bambini", "1F6B8"},
    {"simbolo_del_bagno_degli_uomini", "1F6B9"},
    {"simbolo_del_bagno_delle_donne", "1F6BA"},
    {"simbolo_dei_servizi_igienici", "1F6BB"},
    {"simbolo_con_immagine_di_bambino", "1F6BC"},
    {"toilette", "1F6BD"},
    {"simbolo_del_wc", "1F6BE"},
    {"doccia", "1F6BF"},
    {"persona_che_fa_il_bagno", "1F6C0"},
    {"persona_che_fa_il_bagno_colore1", "1F6C0-1F3FB"},
    {"persona_che_fa_il_bagno_colore2", "1F6C0-1F3FC"},
    {"persona_che_fa_il_bagno_colore3", "1F6C0-1F3FD"},
    {"persona_che_fa_il_bagno_colore4", "1F6C0-1F3FE"},
    {"persona_che_fa_il_bagno_colore5", "1F6C0-1F3FF"},
    {"vasca", "1F6C1"},
    {"simbolo_del_controllo_passaporti", "1F6C2"},
    {"simbolo_della_dogana", "1F6C3"},
    {"simbolo_del_ritiro_bagagli", "1F6C4"},
    {"simbolo_del_deposito_bagagli", "1F6C5"},
    {"divano_e_lampada", "1F6CB"},
    {"persona_a_letto", "1F6CC"},
    {"persona_a_letto_colore1", "1F6CC-1F3FB"},
    {"persona_a_letto_colore2", "1F6CC-1F3FC"},
    {"persona_a_letto_colore3", "1F6CC-1F3FD"},
    {"persona_a_letto_colore4", "1F6CC-1F3FE"},
    {"persona_a_letto_colore5", "1F6CC-1F3FF"},
    {"borse_della_spesa", "1F6CD"},
    {"campanello_da_hotel", "1F6CE"},
    {"letto", "1F6CF"},
    {"luogo_di_culto", "1F6D0"},
    {"segnale_di_stop", "1F6D1"},
    {"carrello", "1F6D2"},
    {"tempio_indu", "1F6D5"},
    {"capanna", "1F6D6"},
    {"ascensore", "1F6D7"},
    {"frana", "1F6D8"},
    {"wireless", "1F6DC"},
    {"scivolo", "1F6DD"},
    {"ruota", "1F6DE"},
    {"salvagente", "1F6DF"},
    {"martello_e_chiave_inglese", "1F6E0"},
    {"scudo", "1F6E1"},
    {"barile_di_petrolio", "1F6E2"},
    {"autostrada", "1F6E3"},
    {"binari", "1F6E4"},
    {"barca_a_motore", "1F6E5"},
    {"aereo_turistico", "1F6E9"},
    {"decollo", "1F6EB"},
    {"atterraggio", "1F6EC"},
    {"satellite", "1F6F0"},
    {"nave_da_crociera", "1F6F3"},
    {"monopattino", "1F6F4"},
    {"scooter", "1F6F5"},
    {"canoa", "1F6F6"},
    {"slitta", "1F6F7"},
    {"disco_volante", "1F6F8"},
    {"skateboard", "1F6F9"},
    {"riscio_a_motore", "1F6FA"},
    {"pickup", "1F6FB"},
    {"pattini_a_rotelle", "1F6FC"},
    {"cerchio_arancione", "1F7E0"},
    {"cerchio_giallo", "1F7E1"},
    {"cerchio_verde", "1F7E2"},
    {"cerchio_viola", "1F7E3"},
    {"cerchio_marrone", "1F7E4"},
    {"quadrato_rosso", "1F7E5"},
    {"quadrato_blu", "1F7E6"},
    {"quadrato_arancione", "1F7E7"},
    {"quadrato_giallo", "1F7E8"},
    {"quadrato_verde", "1F7E9"},
    {"quadrato_viola", "1F7EA"},
    {"quadrato_marrone", "1F7EB"},
    {"segno_di_uguaglianza", "1F7F0"},
    {"mano_a_pigna", "1F90C"},
    {"mano_a_pigna_colore1", "1F90C-1F3FB"},
    {"mano_a_pigna_colore2", "1F90C-1F3FC"},
    {"mano_a_pigna_colore3", "1F90C-1F3FD"},
    {"mano_a_pigna_colore4", "1F90C-1F3FE"},
    {"mano_a_pigna_colore5", "1F90C-1F3FF"},
    {"cuore_bianco", "1F90D"},
    {"cuore_marrone", "1F90E"},
    {"mano_che_fa_il_gesto_del_pizzico", "1F90F"},
    {"mano_che_fa_il_gesto_del_pizzico_colore1", "1F90F-1F3FB"},
    {"mano_che_fa_il_gesto_del_pizzico_colore2", "1F90F-1F3FC"},
    {"mano_che_fa_il_gesto_del_pizzico_colore3", "1F90F-1F3FD"},
    {"mano_che_fa_il_gesto_del_pizzico_colore4", "1F90F-1F3FE"},
    {"mano_che_fa_il_gesto_del_pizzico_colore5", "1F90F-1F3FF"},
    {"faccina_con_bocca_con_cerniera", "1F910"},
    {"faccina_avida_di_denaro", "1F911"},
    {"faccina_con_termometro", "1F912"},
    {"faccina_nerd", "1F913"},
    {"faccina_concentrata", "1F914"},
    {"faccina_con_la_testa_bendata", "1F915"},
    {"faccina_di_robot", "1F916"},
    {"faccina_che_abbraccia", "1F917"},
    {"segno_delle_corna", "1F918"},
    {"segno_delle_corna_colore1", "1F918-1F3FB"},
    {"segno_delle_corna_colore2", "1F918-1F3FC"},
    {"segno_delle_corna_colore3", "1F918-1F3FD"},
    {"segno_delle_corna_colore4", "1F918-1F3FE"},
    {"segno_delle_corna_colore5", "1F918-1F3FF"},
    {"mano_con_gesto_di_chiamata", "1F919"},
    {"mano_con_gesto_di_chiamata_colore1", "1F919-1F3FB"},
    {"mano_con_gesto_di_chiamata_colore2", "1F919-1F3FC"},
    {"mano_con_gesto_di_chiamata_colore3", "1F919-1F3FD"},
    {"mano_con_gesto_di_chiamata_colore4", "1F919-1F3FE"},
    {"mano_con_gesto_di_chiamata_colore5", "1F919-1F3FF"},
    {"dorso_mano_alzata", "1F91A"},
    {"dorso_mano_alzata_colore1", "1F91A-1F3FB"},
    {"dorso_mano_alzata_colore2", "1F91A-1F3FC"},
    {"dorso_mano_alzata_colore3", "1F91A-1F3FD"},
    {"dorso_mano_alzata_colore4", "1F91A-1F3FE"},
    {"dorso_mano_alzata_colore5", "1F91A-1F3FF"},
    {"pugno_a_sinistra", "1F91B"},
    {"pugno_a_sinistra_colore1", "1F91B-1F3FB"},
    {"pugno_a_sinistra_colore2", "1F91B-1F3FC"},
    {"pugno_a_sinistra_colore3", "1F91B-1F3FD"},
    {"pugno_a_sinistra_colore4", "1F91B-1F3FE"},
    {"pugno_a_sinistra_colore5", "1F91B-1F3FF"},
    {"pugno_a_destra", "1F91C"},
    {"pugno_a_destra_colore1", "1F91C-1F3FB"},
    {"pugno_a_destra_colore2", "1F91C-1F3FC"},
    {"pugno_a_destra_colore3", "1F91C-1F3FD"},
    {"pugno_a_destra_colore4", "1F91C-1F3FE"},
    {"pugno_a_destra_colore5", "1F91C-1F3FF"},
    {"stretta_di_mano", "1F91D"},
    {"stretta_di_mano_colore1", "1F91D-1F3FB"},
    {"stretta_di_mano_colore2", "1F91D-1F3FC"},
    {"stretta_di_mano_colore3", "1F91D-1F3FD"},
    {"stretta_di_mano_colore4", "1F91D-1F3FE"},
    {"stretta_di_mano_colore5", "1F91D-1F3FF"},
    {"stretta_di_mano_colore1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"stretta_di_mano_colore1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"stretta_di_mano_colore1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"stretta_di_mano_colore1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"stretta_di_mano_colore2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"stretta_di_mano_colore2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"stretta_di_mano_colore2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"stretta_di_mano_colore2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"stretta_di_mano_colore3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"stretta_di_mano_colore3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"stretta_di_mano_colore3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"stretta_di_mano_colore3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"stretta_di_mano_colore4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"stretta_di_mano_colore4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"stretta_di_mano_colore4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"stretta_di_mano_colore4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"stretta_di_mano_colore5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"stretta_di_mano_colore5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"stretta_di_mano_colore5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"stretta_di_mano_colore5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"dita_incrociate", "1F91E"},
    {"dita_incrociate_colore1", "1F91E-1F3FB"},
    {"dita_incrociate_colore2", "1F91E-1F3FC"},
    {"dita_incrociate_colore3", "1F91E-1F3FD"},
    {"dita_incrociate_colore4", "1F91E-1F3FE"},
    {"dita_incrociate_colore5", "1F91E-1F3FF"},
    {"gesto_ti_amo", "1F91F"},
    {"gesto_ti_amo_colore1", "1F91F-1F3FB"},
    {"gesto_ti_amo_colore2", "1F91F-1F3FC"},
    {"gesto_ti_amo_colore3", "1F91F-1F3FD"},
    {"gesto_ti_amo_colore4", "1F91F-1F3FE"},
    {"gesto_ti_amo_colore5", "1F91F-1F3FF"},
    {"faccina_con_cappello_da_cowboy", "1F920"},
    {"faccina_pagliaccio", "1F921"},
    {"faccina_nauseata", "1F922"},
    {"ridere_a_crepapelle", "1F923"},
    {"faccina_che_sbava", "1F924"},
    {"faccina_bugiarda", "1F925"},
    {"persona_esasperata", "1F926"},
    {"persona_esasperata_colore1", "1F926-1F3FB"},
    {"persona_esasperata_colore2", "1F926-1F3FC"},
    {"persona_esasperata_colore3", "1F926-1F3FD"},
    {"persona_esasperata_colore4", "1F926-1F3FE"},
    {"persona_esasperata_colore5", "1F926-1F3FF"},
    {"faccina_che_starnutisce", "1F927"},
    {"faccina_con_sopracciglia_alzate", "1F928"},
    {"colpo_di_fulmine", "1F929"},
    {"faccina_impazzita", "1F92A"},
    {"faccina_che_zittisce", "1F92B"},
    {"faccina_con_simboli_sulla_bocca", "1F92C"},
    {"faccina_con_mano_sulla_bocca", "1F92D"},
    {"faccina_che_vomita", "1F92E"},
    {"testa_che_esplode", "1F92F"},
    {"donna_incinta", "1F930"},
    {"donna_incinta_colore1", "1F930-1F3FB"},
    {"donna_incinta_colore2", "1F930-1F3FC"},
    {"donna_incinta_colore3", "1F930-1F3FD"},
    {"donna_incinta_colore4", "1F930-1F3FE"},
    {"donna_incinta_colore5", "1F930-1F3FF"},
    {"allattare", "1F931"},
    {"allattare_colore1", "1F931-1F3FB"},
    {"allattare_colore2", "1F931-1F3FC"},
    {"allattare_colore3", "1F931-1F3FD"},
    {"allattare_colore4", "1F931-1F3FE"},
    {"allattare_colore5", "1F931-1F3FF"},
    {"mani_unite_in_alto", "1F932"},
    {"mani_unite_in_alto_colore1", "1F932-1F3FB"},
    {"mani_unite_in_alto_colore2", "1F932-1F3FC"},
    {"mani_unite_in_alto_colore3", "1F932-1F3FD"},
    {"mani_unite_in_alto_colore4", "1F932-1F3FE"},
    {"mani_unite_in_alto_colore5", "1F932-1F3FF"},
    {"selfie", "1F933"},
    {"selfie_colore1", "1F933-1F3FB"},
    {"selfie_colore2", "1F933-1F3FC"},
    {"selfie_colore3", "1F933-1F3FD"},
    {"selfie_colore4", "1F933-1F3FE"},
    {"selfie_colore5", "1F933-1F3FF"},
    {"principe", "1F934"},
    {"principe_colore1", "1F934-1F3FB"},
    {"principe_colore2", "1F934-1F3FC"},
    {"principe_colore3", "1F934-1F3FD"},
    {"principe_colore4", "1F934-1F3FE"},
    {"principe_colore5", "1F934-1F3FF"},
    {"persona_in_smoking", "1F935"},
    {"persona_in_smoking_colore1", "1F935-1F3FB"},
    {"persona_in_smoking_colore2", "1F935-1F3FC"},
    {"persona_in_smoking_colore3", "1F935-1F3FD"},
    {"persona_in_smoking_colore4", "1F935-1F3FE"},
    {"persona_in_smoking_colore5", "1F935-1F3FF"},
    {"mamma_natale", "1F936"},
    {"mamma_natale_colore1", "1F936-1F3FB"},
    {"mamma_natale_colore2", "1F936-1F3FC"},
    {"mamma_natale_colore3", "1F936-1F3FD"},
    {"mamma_natale_colore4", "1F936-1F3FE"},
    {"mamma_natale_colore5", "1F936-1F3FF"},
    {"persona_che_scrolla_le_spalle", "1F937"},
    {"persona_che_scrolla_le_spalle_colore1", "1F937-1F3FB"},
    {"persona_che_scrolla_le_spalle_colore2", "1F937-1F3FC"},
    {"persona_che_scrolla_le_spalle_colore3", "1F937-1F3FD"},
    {"persona_che_scrolla_le_spalle_colore4", "1F937-1F3FE"},
    {"persona_che_scrolla_le_spalle_colore5", "1F937-1F3FF"},
    {"persona_che_fa_la_ruota", "1F938"},
    {"persona_che_fa_la_ruota_colore1", "1F938-1F3FB"},
    {"persona_che_fa_la_ruota_colore2", "1F938-1F3FC"},
    {"persona_che_fa_la_ruota_colore3", "1F938-1F3FD"},
    {"persona_che_fa_la_ruota_colore4", "1F938-1F3FE"},
    {"persona_che_fa_la_ruota_colore5", "1F938-1F3FF"},
    {"persona_che_fa_giocoleria", "1F939"},
    {"persona_che_fa_giocoleria_colore1", "1F939-1F3FB"},
    {"persona_che_fa_giocoleria_colore2", "1F939-1F3FC"},
    {"persona_che_fa_giocoleria_colore3", "1F939-1F3FD"},
    {"persona_che_fa_giocoleria_colore4", "1F939-1F3FE"},
    {"persona_che_fa_giocoleria_colore5", "1F939-1F3FF"},
    {"schermidore", "1F93A"},
    {"persone_che_fanno_la_lotta", "1F93C"},
    {"persone_che_fanno_la_lotta_colore1", "1F93C-1F3FB"},
    {"persone_che_fanno_la_lotta_colore2", "1F93C-1F3FC"},
    {"persone_che_fanno_la_lotta_colore3", "1F93C-1F3FD"},
    {"persone_che_fanno_la_lotta_colore4", "1F93C-1F3FE"},
    {"persone_che_fanno_la_lotta_colore5", "1F93C-1F3FF"},
    {"persone_che_fanno_la_lotta_colore1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"persone_che_fanno_la_lotta_colore1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"persone_che_fanno_la_lotta_colore1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"persone_che_fanno_la_lotta_colore1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"persone_che_fanno_la_lotta_colore2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"persone_che_fanno_la_lotta_colore2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"persone_che_fanno_la_lotta_colore2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"persone_che_fanno_la_lotta_colore2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"persone_che_fanno_la_lotta_colore3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"persone_che_fanno_la_lotta_colore3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"persone_che_fanno_la_lotta_colore3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"persone_che_fanno_la_lotta_colore3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"persone_che_fanno_la_lotta_colore4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"persone_che_fanno_la_lotta_colore4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"persone_che_fanno_la_lotta_colore4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"persone_che_fanno_la_lotta_colore4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"persone_che_fanno_la_lotta_colore5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"persone_che_fanno_la_lotta_colore5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"persone_che_fanno_la_lotta_colore5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"persone_che_fanno_la_lotta_colore5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"persona_che_gioca_a_pallanuoto", "1F93D"},
    {"persona_che_gioca_a_pallanuoto_colore1", "1F93D-1F3FB"},
    {"persona_che_gioca_a_pallanuoto_colore2", "1F93D-1F3FC"},
    {"persona_che_gioca_a_pallanuoto_colore3", "1F93D-1F3FD"},
    {"persona_che_gioca_a_pallanuoto_colore4", "1F93D-1F3FE"},
    {"persona_che_gioca_a_pallanuoto_colore5", "1F93D-1F3FF"},
    {"persona_che_gioca_a_pallamano", "1F93E"},
    {"persona_che_gioca_a_pallamano_colore1", "1F93E-1F3FB"},
    {"persona_che_gioca_a_pallamano_colore2", "1F93E-1F3FC"},
    {"persona_che_gioca_a_pallamano_colore3", "1F93E-1F3FD"},
    {"persona_che_gioca_a_pallamano_colore4", "1F93E-1F3FE"},
    {"persona_che_gioca_a_pallamano_colore5", "1F93E-1F3FF"},
    {"maschera_da_sub", "1F93F"},
    {"fiore_appassito", "1F940"},
    {"tamburo", "1F941"},
    {"brindisi", "1F942"},
    {"bicchiere_tumbler", "1F943"},
    {"cucchiaio", "1F944"},
    {"porta_da_calcio", "1F945"},
    {"medaglia_doro", "1F947"},
    {"medaglia_dargento", "1F948"},
    {"medaglia_di_bronzo", "1F949"},
    {"guantone_da_pugilato", "1F94A"},
    {"kimono_per_arti_marziali", "1F94B"},
    {"stone_da_curling", "1F94C"},
    {"lacrosse", "1F94D"},
    {"palla_da_softball", "1F94E"},
    {"frisbee", "1F94F"},
    {"croissant", "1F950"},
    {"avocado", "1F951"},
    {"cetriolo", "1F952"},
    {"pancetta", "1F953"},
    {"patata", "1F954"},
    {"carota", "1F955"},
    {"baguette", "1F956"},
    {"insalata_verde", "1F957"},
    {"padella_con_cibo", "1F958"},
    {"pita_farcita", "1F959"},
    {"uovo", "1F95A"},
    {"bicchiere_di_latte", "1F95B"},
    {"arachidi", "1F95C"},
    {"kiwi", "1F95D"},
    {"pancake", "1F95E"},
    {"raviolo", "1F95F"},
    {"biscotto_della_fortuna", "1F960"},
    {"confezione_da_asporto", "1F961"},
    {"bacchette", "1F962"},
    {"ciotola_con_cucchiaio", "1F963"},
    {"bicchiere_con_cannuccia", "1F964"},
    {"cocco", "1F965"},
    {"broccoli", "1F966"},
    {"torta", "1F967"},
    {"pretzel", "1F968"},
    {"taglio_di_carne", "1F969"},
    {"sandwich", "1F96A"},
    {"cibo_in_scatola", "1F96B"},
    {"verdure_a_foglia", "1F96C"},
    {"mango", "1F96D"},
    {"torta_della_luna", "1F96E"},
    {"bagel", "1F96F"},
    {"faccina_con_cuoricini", "1F970"},
    {"faccina_che_sbadiglia", "1F971"},
    {"faccina_sorridente_con_lacrima", "1F972"},
    {"faccina_che_festeggia", "1F973"},
    {"faccina_stordita", "1F974"},
    {"faccina_accaldata", "1F975"},
    {"faccina_congelata", "1F976"},
    {"ninja", "1F977"},
    {"ninja_colore1", "1F977-1F3FB"},
    {"ninja_colore2", "1F977-1F3FC"},
    {"ninja_colore3", "1F977-1F3FD"},
    {"ninja_colore4", "1F977-1F3FE"},
    {"ninja_colore5", "1F977-1F3FF"},
    {"faccina_travestita", "1F978"},
    {"faccina_che_trattiene_le_lacrime", "1F979"},
    {"faccina_supplichevole", "1F97A"},
    {"sari", "1F97B"},
    {"camice", "1F97C"},
    {"maschera", "1F97D"},
    {"stivale_da_trekking", "1F97E"},
    {"ballerina", "1F97F"},
    {"granchio", "1F980"},
    {"leone", "1F981"},
    {"scorpione", "1F982"},
    {"tacchino", "1F983"},
    {"unicorno", "1F984"},
    {"aquila", "1F985"},
    {"anatra", "1F986"},
    {"pipistrello", "1F987"},
    {"squalo", "1F988"},
    {"gufo", "1F989"},
    {"volpe", "1F98A"},
    {"farfalla", "1F98B"},
    {"cervo", "1F98C"},
    {"gorilla", "1F98D"},
    {"lucertola", "1F98E"},
    {"rinoceronte", "1F98F"},
    {"gamberetto", "1F990"},
    {"calamaro", "1F991"},
    {"giraffa", "1F992"},
    {"zebra", "1F993"},
    {"riccio", "1F994"},
    {"sauropode", "1F995"},
    {"trex", "1F996"},
    {"grillo", "1F997"},
    {"canguro", "1F998"},
    {"lama", "1F999"},
    {"pavone", "1F99A"},
    {"ippopotamo", "1F99B"},
    {"pappagallo", "1F99C"},
    {"procione", "1F99D"},
    {"aragosta", "1F99E"},
    {"zanzara", "1F99F"},
    {"microbo", "1F9A0"},
    {"tasso", "1F9A1"},
    {"cigno", "1F9A2"},
    {"mammut", "1F9A3"},
    {"dodo", "1F9A4"},
    {"bradipo", "1F9A5"},
    {"lontra", "1F9A6"},
    {"orangotango", "1F9A7"},
    {"puzzola", "1F9A8"},
    {"fenicottero", "1F9A9"},
    {"ostrica", "1F9AA"},
    {"castoro", "1F9AB"},
    {"bisonte", "1F9AC"},
    {"foca", "1F9AD"},
    {"cane_guida", "1F9AE"},
    {"bastone_per_non_vedenti", "1F9AF"},
    {"capelli_rossi", "1F9B0"},
    {"capelli_ricci", "1F9B1"},
    {"calvo", "1F9B2"},
    {"capelli_bianchi", "1F9B3"},
    {"osso", "1F9B4"},
    {"gamba", "1F9B5"},
    {"gamba_colore1", "1F9B5-1F3FB"},
    {"gamba_colore2", "1F9B5-1F3FC"},
    {"gamba_colore3", "1F9B5-1F3FD"},
    {"gamba_colore4", "1F9B5-1F3FE"},
    {"gamba_colore5", "1F9B5-1F3FF"},
    {"piede", "1F9B6"},
    {"piede_colore1", "1F9B6-1F3FB"},
    {"piede_colore2", "1F9B6-1F3FC"},
    {"piede_colore3", "1F9B6-1F3FD"},
    {"piede_colore4", "1F9B6-1F3FE"},
    {"piede_colore5", "1F9B6-1F3FF"},
    {"dente", "1F9B7"},
    {"supereroe", "1F9B8"},
    {"supereroe_colore1", "1F9B8-1F3FB"},
    {"supereroe_colore2", "1F9B8-1F3FC"},
    {"supereroe_colore3", "1F9B8-1F3FD"},
    {"supereroe_colore4", "1F9B8-1F3FE"},
    {"supereroe_colore5", "1F9B8-1F3FF"},
    {"supercattivo", "1F9B9"},
    {"supercattivo_colore1", "1F9B9-1F3FB"},
    {"supercattivo_colore2", "1F9B9-1F3FC"},
    {"supercattivo_colore3", "1F9B9-1F3FD"},
    {"supercattivo_colore4", "1F9B9-1F3FE"},
    {"supercattivo_colore5", "1F9B9-1F3FF"},
    {"gilet_di_sicurezza", "1F9BA"},
    {"orecchio_con_apparecchio_acustico", "1F9BB"},
    {"orecchio_con_apparecchio_acustico_colore1", "1F9BB-1F3FB"},
    {"orecchio_con_apparecchio_acustico_colore2", "1F9BB-1F3FC"},
    {"orecchio_con_apparecchio_acustico_colore3", "1F9BB-1F3FD"},
    {"orecchio_con_apparecchio_acustico_colore4", "1F9BB-1F3FE"},
    {"orecchio_con_apparecchio_acustico_colore5", "1F9BB-1F3FF"},
    {"sedia_a_rotelle_motorizzata", "1F9BC"},
    {"sedia_a_rotelle_manuale", "1F9BD"},
    {"protesi_robotica_per_il_braccio", "1F9BE"},
    {"protesi_robotica_per_la_gamba", "1F9BF"},
    {"fetta_di_formaggio", "1F9C0"},
    {"cupcake", "1F9C1"},
    {"sale", "1F9C2"},
    {"bevanda_monodose", "1F9C3"},
    {"aglio", "1F9C4"},
    {"cipolla", "1F9C5"},
    {"falafel", "1F9C6"},
    {"waffle", "1F9C7"},
    {"burro", "1F9C8"},
    {"mate", "1F9C9"},
    {"cubetto_di_ghiaccio", "1F9CA"},
    {"bubble_tea", "1F9CB"},
    {"troll", "1F9CC"},
    {"persona_in_piedi", "1F9CD"},
    {"persona_in_piedi_colore1", "1F9CD-1F3FB"},
    {"persona_in_piedi_colore2", "1F9CD-1F3FC"},
    {"persona_in_piedi_colore3", "1F9CD-1F3FD"},
    {"persona_in_piedi_colore4", "1F9CD-1F3FE"},
    {"persona_in_piedi_colore5", "1F9CD-1F3FF"},
    {"persona_inginocchiata", "1F9CE"},
    {"persona_inginocchiata_colore1", "1F9CE-1F3FB"},
    {"persona_inginocchiata_colore2", "1F9CE-1F3FC"},
    {"persona_inginocchiata_colore3", "1F9CE-1F3FD"},
    {"persona_inginocchiata_colore4", "1F9CE-1F3FE"},
    {"persona_inginocchiata_colore5", "1F9CE-1F3FF"},
    {"persona_con_problemi_di_udito", "1F9CF"},
    {"persona_con_problemi_di_udito_colore1", "1F9CF-1F3FB"},
    {"persona_con_problemi_di_udito_colore2", "1F9CF-1F3FC"},
    {"persona_con_problemi_di_udito_colore3", "1F9CF-1F3FD"},
    {"persona_con_problemi_di_udito_colore4", "1F9CF-1F3FE"},
    {"persona_con_problemi_di_udito_colore5", "1F9CF-1F3FF"},
    {"faccina_con_monocolo", "1F9D0"},
    {"persona", "1F9D1"},
    {"persona_colore1", "1F9D1-1F3FB"},
    {"persona_colore2", "1F9D1-1F3FC"},
    {"persona_colore3", "1F9D1-1F3FD"},
    {"persona_colore4", "1F9D1-1F3FE"},
    {"persona_colore5", "1F9D1-1F3FF"},
    {"bimbo", "1F9D2"},
    {"bimbo_colore1", "1F9D2-1F3FB"},
    {"bimbo_colore2", "1F9D2-1F3FC"},
    {"bimbo_colore3", "1F9D2-1F3FD"},
    {"bimbo_colore4", "1F9D2-1F3FE"},
    {"bimbo_colore5", "1F9D2-1F3FF"},
    {"persona_anziana", "1F9D3"},
    {"persona_anziana_colore1", "1F9D3-1F3FB"},
    {"persona_anziana_colore2", "1F9D3-1F3FC"},
    {"persona_anziana_colore3", "1F9D3-1F3FD"},
    {"persona_anziana_colore4", "1F9D3-1F3FE"},
    {"persona_anziana_colore5", "1F9D3-1F3FF"},
    {"persona_con_la_barba", "1F9D4"},
    {"persona_con_la_barba_colore1", "1F9D4-1F3FB"},
    {"persona_con_la_barba_colore2", "1F9D4-1F3FC"},
    {"persona_con_la_barba_colore3", "1F9D4-1F3FD"},
    {"persona_con_la_barba_colore4", "1F9D4-1F3FE"},
    {"persona_con_la_barba_colore5", "1F9D4-1F3FF"},
    {"donna_con_velo", "1F9D5"},
    {"donna_con_velo_colore1", "1F9D5-1F3FB"},
    {"donna_con_velo_colore2", "1F9D5-1F3FC"},
    {"donna_con_velo_colore3", "1F9D5-1F3FD"},
    {"donna_con_velo_colore4", "1F9D5-1F3FE"},
    {"donna_con_velo_colore5", "1F9D5-1F3FF"},
    {"persona_in_sauna", "1F9D6"},
    {"persona_in_sauna_colore1", "1F9D6-1F3FB"},
    {"persona_in_sauna_colore2", "1F9D6-1F3FC"},
    {"persona_in_sauna_colore3", "1F9D6-1F3FD"},
    {"persona_in_sauna_colore4", "1F9D6-1F3FE"},
    {"persona_in_sauna_colore5", "1F9D6-1F3FF"},
    {"persona_che_scala", "1F9D7"},
    {"persona_che_scala_colore1", "1F9D7-1F3FB"},
    {"persona_che_scala_colore2", "1F9D7-1F3FC"},
    {"persona_che_scala_colore3", "1F9D7-1F3FD"},
    {"persona_che_scala_colore4", "1F9D7-1F3FE"},
    {"persona_che_scala_colore5", "1F9D7-1F3FF"},
    {"persona_in_posizione_del_loto", "1F9D8"},
    {"persona_in_posizione_del_loto_colore1", "1F9D8-1F3FB"},
    {"persona_in_posizione_del_loto_colore2", "1F9D8-1F3FC"},
    {"persona_in_posizione_del_loto_colore3", "1F9D8-1F3FD"},
    {"persona_in_posizione_del_loto_colore4", "1F9D8-1F3FE"},
    {"persona_in_posizione_del_loto_colore5", "1F9D8-1F3FF"},
    {"mago", "1F9D9"},
    {"mago_colore1", "1F9D9-1F3FB"},
    {"mago_colore2", "1F9D9-1F3FC"},
    {"mago_colore3", "1F9D9-1F3FD"},
    {"mago_colore4", "1F9D9-1F3FE"},
    {"mago_colore5", "1F9D9-1F3FF"},
    {"fata", "1F9DA"},
    {"fata_colore1", "1F9DA-1F3FB"},
    {"fata_colore2", "1F9DA-1F3FC"},
    {"fata_colore3", "1F9DA-1F3FD"},
    {"fata_colore4", "1F9DA-1F3FE"},
    {"fata_colore5", "1F9DA-1F3FF"},
    {"vampiro", "1F9DB"},
    {"vampiro_colore1", "1F9DB-1F3FB"},
    {"vampiro_colore2", "1F9DB-1F3FC"},
    {"vampiro_colore3", "1F9DB-1F3FD"},
    {"vampiro_colore4", "1F9DB-1F3FE"},
    {"vampiro_colore5", "1F9DB-1F3FF"},
    {"sirena", "1F9DC"},
    {"sirena_colore1", "1F9DC-1F3FB"},
    {"sirena_colore2", "1F9DC-1F3FC"},
    {"sirena_colore3", "1F9DC-1F3FD"},
    {"sirena_colore4", "1F9DC-1F3FE"},
    {"sirena_colore5", "1F9DC-1F3FF"},
    {"elfo", "1F9DD"},
    {"elfo_colore1", "1F9DD-1F3FB"},
    {"elfo_colore2", "1F9DD-1F3FC"},
    {"elfo_colore3", "1F9DD-1F3FD"},
    {"elfo_colore4", "1F9DD-1F3FE"},
    {"elfo_colore5", "1F9DD-1F3FF"},
    {"genio", "1F9DE"},
    {"zombie", "1F9DF"},
    {"cervello", "1F9E0"},
    {"cuore_arancione", "1F9E1"},
    {"cappello_con_visiera", "1F9E2"},
    {"sciarpa", "1F9E3"},
    {"guanti", "1F9E4"},
    {"cappotto", "1F9E5"},
    {"calzini", "1F9E6"},
    {"busta_hongbao", "1F9E7"},
    {"petardo", "1F9E8"},
    {"pezzo_di_puzzle", "1F9E9"},
    {"provetta", "1F9EA"},
    {"piastra_di_petri", "1F9EB"},
    {"dna", "1F9EC"},
    {"bussola", "1F9ED"},
    {"abaco", "1F9EE"},
    {"estintore", "1F9EF"},
    {"cassetta_degli_attrezzi", "1F9F0"},
    {"mattoni", "1F9F1"},
    {"calamita", "1F9F2"},
    {"valigia", "1F9F3"},
    {"flacone", "1F9F4"},
    {"filo", "1F9F5"},
    {"gomitolo", "1F9F6"},
    {"spilla_da_balia", "1F9F7"},
    {"orsetto", "1F9F8"},
    {"scopa", "1F9F9"},
    {"cesto", "1F9FA"},
    {"rotolo_di_carta_igienica", "1F9FB"},
    {"sapone", "1F9FC"},
    {"spugna", "1F9FD"},
    {"scontrino", "1F9FE"},
    {"occhio_di_allah", "1F9FF"},
    {"scarpette_da_ballerina", "1FA70"},
    {"costume_da_bagno_intero", "1FA71"},
    {"slip", "1FA72"},
    {"pantaloncini", "1FA73"},
    {"infradito", "1FA74"},
    {"cuore_azzurro", "1FA75"},
    {"cuore_grigio", "1FA76"},
    {"cuore_rosa", "1FA77"},
    {"goccia_di_sangue", "1FA78"},
    {"cerotto", "1FA79"},
    {"stetoscopio", "1FA7A"},
    {"radiografia", "1FA7B"},
    {"stampella", "1FA7C"},
    {"yo_yo", "1FA80"},
    {"aquilone", "1FA81"},
    {"paracadute", "1FA82"},
    {"boomerang", "1FA83"},
    {"bacchetta_magica", "1FA84"},
    {"pentolaccia", "1FA85"},
    {"matrioska", "1FA86"},
    {"maracas", "1FA87"},
    {"flauto", "1FA88"},
    {"arpa", "1FA89"},
    {"trombone", "1FA8A"},
    {"forziere", "1FA8E"},
    {"vanga", "1FA8F"},
    {"pianeta_con_satellite", "1FA90"},
    {"sedia", "1FA91"},
    {"rasoio", "1FA92"},
    {"ascia", "1FA93"},
    {"diya", "1FA94"},
    {"banjo", "1FA95"},
    {"elmetto_militare", "1FA96"},
    {"fisarmonica", "1FA97"},
    {"tamburo_lungo", "1FA98"},
    {"moneta", "1FA99"},
    {"sega_da_falegname", "1FA9A"},
    {"cacciavite", "1FA9B"},
    {"scala", "1FA9C"},
    {"gancio", "1FA9D"},
    {"specchio", "1FA9E"},
    {"finestra", "1FA9F"},
    {"sturalavandini", "1FAA0"},
    {"ago_da_cucito", "1FAA1"},
    {"nodo", "1FAA2"},
    {"secchio", "1FAA3"},
    {"trappola_per_topi", "1FAA4"},
    {"spazzolino_da_denti", "1FAA5"},
    {"lapide", "1FAA6"},
    {"cartello", "1FAA7"},
    {"roccia", "1FAA8"},
    {"palla_da_discoteca", "1FAA9"},
    {"carta_didentita", "1FAAA"},
    {"batteria_scarica", "1FAAB"},
    {"mano_di_fatima", "1FAAC"},
    {"ventaglio", "1FAAD"},
    {"pettine_afro", "1FAAE"},
    {"khanda", "1FAAF"},
    {"mosca", "1FAB0"},
    {"verme", "1FAB1"},
    {"coleottero", "1FAB2"},
    {"scarafaggio", "1FAB3"},
    {"pianta_in_vaso", "1FAB4"},
    {"legna", "1FAB5"},
    {"piuma", "1FAB6"},
    {"loto", "1FAB7"},
    {"corallo", "1FAB8"},
    {"nido_vuoto", "1FAB9"},
    {"nido_con_uova", "1FABA"},
    {"giacinto", "1FABB"},
    {"medusa", "1FABC"},
    {"ala", "1FABD"},
    {"albero_spoglio", "1FABE"},
    {"oca", "1FABF"},
    {"organo_del_cuore", "1FAC0"},
    {"polmoni", "1FAC1"},
    {"persone_che_si_abbracciano", "1FAC2"},
    {"uomo_incinto", "1FAC3"},
    {"uomo_incinto_colore1", "1FAC3-1F3FB"},
    {"uomo_incinto_colore2", "1FAC3-1F3FC"},
    {"uomo_incinto_colore3", "1FAC3-1F3FD"},
    {"uomo_incinto_colore4", "1FAC3-1F3FE"},
    {"uomo_incinto_colore5", "1FAC3-1F3FF"},
    {"persona_incinta", "1FAC4"},
    {"persona_incinta_colore1", "1FAC4-1F3FB"},
    {"persona_incinta_colore2", "1FAC4-1F3FC"},
    {"persona_incinta_colore3", "1FAC4-1F3FD"},
    {"persona_incinta_colore4", "1FAC4-1F3FE"},
    {"persona_incinta_colore5", "1FAC4-1F3FF"},
    {"persona_con_corona", "1FAC5"},
    {"persona_con_corona_colore1", "1FAC5-1F3FB"},
    {"persona_con_corona_colore2", "1FAC5-1F3FC"},
    {"persona_con_corona_colore3", "1FAC5-1F3FD"},
    {"persona_con_corona_colore4", "1FAC5-1F3FE"},
    {"persona_con_corona_colore5", "1FAC5-1F3FF"},
    {"impronta_digitale", "1FAC6"},
    {"creatura_pelosa", "1FAC8"},
    {"orca", "1FACD"},
    {"alce", "1FACE"},
    {"asino", "1FACF"},
    {"mirtilli", "1FAD0"},
    {"peperone", "1FAD1"},
    {"oliva", "1FAD2"},
    {"focaccia", "1FAD3"},
    {"tamale", "1FAD4"},
    {"fonduta", "1FAD5"},
    {"teiera", "1FAD6"},
    {"liquido_versato", "1FAD7"},
    {"fagioli", "1FAD8"},
    {"barattolo", "1FAD9"},
    {"radice_di_zenzero", "1FADA"},
    {"baccello_di_piselli", "1FADB"},
    {"ortaggio_da_radice", "1FADC"},
    {"spruzzo", "1FADF"},
    {"faccina_che_si_scioglie", "1FAE0"},
    {"faccina_che_fa_il_saluto", "1FAE1"},
    {"faccina_con_occhi_aperti_e_mano_sulla_bocca", "1FAE2"},
    {"faccina_che_sbircia_tra_le_dita", "1FAE3"},
    {"faccina_perplessa", "1FAE4"},
    {"faccina_tratteggiata", "1FAE5"},
    {"bocca_che_morde_il_labbro", "1FAE6"},
    {"bolle", "1FAE7"},
    {"faccina_tremante", "1FAE8"},
    {"faccina_con_borse_sotto_gli_occhi", "1FAE9"},
    {"faccina_stravolta", "1FAEA"},
    {"nuvoletta_di_zuffa", "1FAEF"},
    {"mano_con_indice_e_pollice_incrociati", "1FAF0"},
    {"mano_con_indice_e_pollice_incrociati_colore1", "1FAF0-1F3FB"},
    {"mano_con_indice_e_pollice_incrociati_colore2", "1FAF0-1F3FC"},
    {"mano_con_indice_e_pollice_incrociati_colore3", "1FAF0-1F3FD"},
    {"mano_con_indice_e_pollice_incrociati_colore4", "1FAF0-1F3FE"},
    {"mano_con_indice_e_pollice_incrociati_colore5", "1FAF0-1F3FF"},
    {"mano_rivolta_a_destra", "1FAF1"},
    {"mano_rivolta_a_destra_colore1", "1FAF1-1F3FB"},
    {"mano_rivolta_a_destra_colore2", "1FAF1-1F3FC"},
    {"mano_rivolta_a_destra_colore3", "1FAF1-1F3FD"},
    {"mano_rivolta_a_destra_colore4", "1FAF1-1F3FE"},
    {"mano_rivolta_a_destra_colore5", "1FAF1-1F3FF"},
    {"mano_rivolta_a_sinistra", "1FAF2"},
    {"mano_rivolta_a_sinistra_colore1", "1FAF2-1F3FB"},
    {"mano_rivolta_a_sinistra_colore2", "1FAF2-1F3FC"},
    {"mano_rivolta_a_sinistra_colore3", "1FAF2-1F3FD"},
    {"mano_rivolta_a_sinistra_colore4", "1FAF2-1F3FE"},
    {"mano_rivolta_a_sinistra_colore5", "1FAF2-1F3FF"},
    {"mano_con_il_palmo_verso_il_basso", "1FAF3"},
    {"mano_con_il_palmo_verso_il_basso_colore1", "1FAF3-1F3FB"},
    {"mano_con_il_palmo_verso_il_basso_colore2", "1FAF3-1F3FC"},
    {"mano_con_il_palmo_verso_il_basso_colore3", "1FAF3-1F3FD"},
    {"mano_con_il_palmo_verso_il_basso_colore4", "1FAF3-1F3FE"},
    {"mano_con_il_palmo_verso_il_basso_colore5", "1FAF3-1F3FF"},
    {"mano_con_il_palmo_verso_lalto", "1FAF4"},
    {"mano_con_il_palmo_verso_lalto_colore1", "1FAF4-1F3FB"},
    {"mano_con_il_palmo_verso_lalto_colore2", "1FAF4-1F3FC"},
    {"mano_con_il_palmo_verso_lalto_colore3", "1FAF4-1F3FD"},
    {"mano_con_il_palmo_verso_lalto_colore4", "1FAF4-1F3FE"},
    {"mano_con_il_palmo_verso_lalto_colore5", "1FAF4-1F3FF"},
    {"indice_verso_losservatore", "1FAF5"},
    {"indice_verso_losservatore_colore1", "1FAF5-1F3FB"},
    {"indice_verso_losservatore_colore2", "1FAF5-1F3FC"},
    {"indice_verso_losservatore_colore3", "1FAF5-1F3FD"},
    {"indice_verso_losservatore_colore4", "1FAF5-1F3FE"},
    {"indice_verso_losservatore_colore5", "1FAF5-1F3FF"},
    {"mani_a_cuore", "1FAF6"},
    {"mani_a_cuore_colore1", "1FAF6-1F3FB"},
    {"mani_a_cuore_colore2", "1FAF6-1F3FC"},
    {"mani_a_cuore_colore3", "1FAF6-1F3FD"},
    {"mani_a_cuore_colore4", "1FAF6-1F3FE"},
    {"mani_a_cuore_colore5", "1FAF6-1F3FF"},
    {"mano_che_spinge_a_sinistra", "1FAF7"},
    {"mano_che_spinge_a_sinistra_colore1", "1FAF7-1F3FB"},
    {"mano_che_spinge_a_sinistra_colore2", "1FAF7-1F3FC"},
    {"mano_che_spinge_a_sinistra_colore3", "1FAF7-1F3FD"},
    {"mano_che_spinge_a_sinistra_colore4", "1FAF7-1F3FE"},
    {"mano_che_spinge_a_sinistra_colore5", "1FAF7-1F3FF"},
    {"mano_che_spinge_a_destra", "1FAF8"},
    {"mano_che_spinge_a_destra_colore1", "1FAF8-1F3FB"},
    {"mano_che_spinge_a_destra_colore2", "1FAF8-1F3FC"},
    {"mano_che_spinge_a_destra_colore3", "1FAF8-1F3FD"},
    {"mano_che_spinge_a_destra_colore4", "1FAF8-1F3FE"},
    {"mano_che_spinge_a_destra_colore5", "1FAF8-1F3FF"},
    {"tasto_segno_di_numero", "0023-FE0F-20E3"},
    {"tasto_asterisco", "002A-FE0F-20E3"},
    {"tasto_0", "0030-FE0F-20E3"},
    {"tasto_1", "0031-FE0F-20E3"},
    {"tasto_2", "0032-FE0F-20E3"},
    {"tasto_3", "0033-FE0F-20E3"},
    {"tasto_4", "0034-FE0F-20E3"},
    {"tasto_5", "0035-FE0F-20E3"},
    {"tasto_6", "0036-FE0F-20E3"},
    {"tasto_7", "0037-FE0F-20E3"},
    {"tasto_8", "0038-FE0F-20E3"},
    {"tasto_9", "0039-FE0F-20E3"},
    {"bandiera_isola_ascensione", "1F1E6-1F1E8"},
    {"bandiera_andorra", "1F1E6-1F1E9"},
    {"bandiera_emirati_arabi_uniti", "1F1E6-1F1EA"},
    {"bandiera_afghanistan", "1F1E6-1F1EB"},
    {"bandiera_antigua_e_barbuda", "1F1E6-1F1EC"},
    {"bandiera_anguilla", "1F1E6-1F1EE"},
    {"bandiera_albania", "1F1E6-1F1F1"},
    {"bandiera_armenia", "1F1E6-1F1F2"},
    {"bandiera_angola", "1F1E6-1F1F4"},
    {"bandiera_antartide", "1F1E6-1F1F6"},
    {"bandiera_argentina", "1F1E6-1F1F7"},
    {"bandiera_samoa_americane", "1F1E6-1F1F8"},
    {"bandiera_austria", "1F1E6-1F1F9"},
    {"bandiera_australia", "1F1E6-1F1FA"},
    {"bandiera_aruba", "1F1E6-1F1FC"},
    {"bandiera_isole_aland", "1F1E6-1F1FD"},
    {"bandiera_azerbaigian", "1F1E6-1F1FF"},
    {"bandiera_bosnia_ed_erzegovina", "1F1E7-1F1E6"},
    {"bandiera_barbados", "1F1E7-1F1E7"},
    {"bandiera_bangladesh", "1F1E7-1F1E9"},
    {"bandiera_belgio", "1F1E7-1F1EA"},
    {"bandiera_burkina_faso", "1F1E7-1F1EB"},
    {"bandiera_bulgaria", "1F1E7-1F1EC"},
    {"bandiera_bahrein", "1F1E7-1F1ED"},
    {"bandiera_burundi", "1F1E7-1F1EE"},
    {"bandiera_benin", "1F1E7-1F1EF"},
    {"bandiera_saint_barthelemy", "1F1E7-1F1F1"},
    {"bandiera_bermuda", "1F1E7-1F1F2"},
    {"bandiera_brunei", "1F1E7-1F1F3"},
    {"bandiera_bolivia", "1F1E7-1F1F4"},
    {"bandiera_caraibi_olandesi", "1F1E7-1F1F6"},
    {"bandiera_brasile", "1F1E7-1F1F7"},
    {"bandiera_bahamas", "1F1E7-1F1F8"},
    {"bandiera_bhutan", "1F1E7-1F1F9"},
    {"bandiera_isola_bouvet", "1F1E7-1F1FB"},
    {"bandiera_botswana", "1F1E7-1F1FC"},
    {"bandiera_bielorussia", "1F1E7-1F1FE"},
    {"bandiera_belize", "1F1E7-1F1FF"},
    {"bandiera_canada", "1F1E8-1F1E6"},
    {"bandiera_isole_cocos_keeling", "1F1E8-1F1E8"},
    {"bandiera_congo_kinshasa", "1F1E8-1F1E9"},
    {"bandiera_repubblica_centrafricana", "1F1E8-1F1EB"},
    {"bandiera_congo_brazzaville", "1F1E8-1F1EC"},
    {"bandiera_svizzera", "1F1E8-1F1ED"},
    {"bandiera_costa_davorio", "1F1E8-1F1EE"},
    {"bandiera_isole_cook", "1F1E8-1F1F0"},
    {"bandiera_cile", "1F1E8-1F1F1"},
    {"bandiera_camerun", "1F1E8-1F1F2"},
    {"bandiera_cina", "1F1E8-1F1F3"},
    {"bandiera_colombia", "1F1E8-1F1F4"},
    {"bandiera_isola_di_clipperton", "1F1E8-1F1F5"},
    {"bandiera_sark", "1F1E8-1F1F6"},
    {"bandiera_costa_rica", "1F1E8-1F1F7"},
    {"bandiera_cuba", "1F1E8-1F1FA"},
    {"bandiera_capo_verde", "1F1E8-1F1FB"},
    {"bandiera_curacao", "1F1E8-1F1FC"},
    {"bandiera_isola_christmas", "1F1E8-1F1FD"},
    {"bandiera_cipro", "1F1E8-1F1FE"},
    {"bandiera_cechia", "1F1E8-1F1FF"},
    {"bandiera_germania", "1F1E9-1F1EA"},
    {"bandiera_diego_garcia", "1F1E9-1F1EC"},
    {"bandiera_gibuti", "1F1E9-1F1EF"},
    {"bandiera_danimarca", "1F1E9-1F1F0"},
    {"bandiera_dominica", "1F1E9-1F1F2"},
    {"bandiera_repubblica_dominicana", "1F1E9-1F1F4"},
    {"bandiera_algeria", "1F1E9-1F1FF"},
    {"bandiera_ceuta_e_melilla", "1F1EA-1F1E6"},
    {"bandiera_ecuador", "1F1EA-1F1E8"},
    {"bandiera_estonia", "1F1EA-1F1EA"},
    {"bandiera_egitto", "1F1EA-1F1EC"},
    {"bandiera_sahara_occidentale", "1F1EA-1F1ED"},
    {"bandiera_eritrea", "1F1EA-1F1F7"},
    {"bandiera_spagna", "1F1EA-1F1F8"},
    {"bandiera_etiopia", "1F1EA-1F1F9"},
    {"bandiera_unione_europea", "1F1EA-1F1FA"},
    {"bandiera_finlandia", "1F1EB-1F1EE"},
    {"bandiera_figi", "1F1EB-1F1EF"},
    {"bandiera_isole_falkland", "1F1EB-1F1F0"},
    {"bandiera_micronesia", "1F1EB-1F1F2"},
    {"bandiera_isole_faer_oer", "1F1EB-1F1F4"},
    {"bandiera_francia", "1F1EB-1F1F7"},
    {"bandiera_gabon", "1F1EC-1F1E6"},
    {"bandiera_regno_unito", "1F1EC-1F1E7"},
    {"bandiera_grenada", "1F1EC-1F1E9"},
    {"bandiera_georgia", "1F1EC-1F1EA"},
    {"bandiera_guyana_francese", "1F1EC-1F1EB"},
    {"bandiera_guernsey", "1F1EC-1F1EC"},
    {"bandiera_ghana", "1F1EC-1F1ED"},
    {"bandiera_gibilterra", "1F1EC-1F1EE"},
    {"bandiera_groenlandia", "1F1EC-1F1F1"},
    {"bandiera_gambia", "1F1EC-1F1F2"},
    {"bandiera_guinea", "1F1EC-1F1F3"},
    {"bandiera_guadalupa", "1F1EC-1F1F5"},
    {"bandiera_guinea_equatoriale", "1F1EC-1F1F6"},
    {"bandiera_grecia", "1F1EC-1F1F7"},
    {"bandiera_georgia_del_sud_e_sandwich_australi", "1F1EC-1F1F8"},
    {"bandiera_guatemala", "1F1EC-1F1F9"},
    {"bandiera_guam", "1F1EC-1F1FA"},
    {"bandiera_guinea_bissau", "1F1EC-1F1FC"},
    {"bandiera_guyana", "1F1EC-1F1FE"},
    {"bandiera_ras_di_hong_kong", "1F1ED-1F1F0"},
    {"bandiera_isole_heard_e_mcdonald", "1F1ED-1F1F2"},
    {"bandiera_honduras", "1F1ED-1F1F3"},
    {"bandiera_croazia", "1F1ED-1F1F7"},
    {"bandiera_haiti", "1F1ED-1F1F9"},
    {"bandiera_ungheria", "1F1ED-1F1FA"},
    {"bandiera_isole_canarie", "1F1EE-1F1E8"},
    {"bandiera_indonesia", "1F1EE-1F1E9"},
    {"bandiera_irlanda", "1F1EE-1F1EA"},
    {"bandiera_israele", "1F1EE-1F1F1"},
    {"bandiera_isola_di_man", "1F1EE-1F1F2"},
    {"bandiera_india", "1F1EE-1F1F3"},
    {"bandiera_territorio_britannico_delloceano_indiano", "1F1EE-1F1F4"},
    {"bandiera_iraq", "1F1EE-1F1F6"},
    {"bandiera_iran", "1F1EE-1F1F7"},
    {"bandiera_islanda", "1F1EE-1F1F8"},
    {"bandiera_italia", "1F1EE-1F1F9"},
    {"bandiera_jersey", "1F1EF-1F1EA"},
    {"bandiera_giamaica", "1F1EF-1F1F2"},
    {"bandiera_giordania", "1F1EF-1F1F4"},
    {"bandiera_giappone", "1F1EF-1F1F5"},
    {"bandiera_kenya", "1F1F0-1F1EA"},
    {"bandiera_kirghizistan", "1F1F0-1F1EC"},
    {"bandiera_cambogia", "1F1F0-1F1ED"},
    {"bandiera_kiribati", "1F1F0-1F1EE"},
    {"bandiera_comore", "1F1F0-1F1F2"},
    {"bandiera_saint_kitts_e_nevis", "1F1F0-1F1F3"},
    {"bandiera_corea_del_nord", "1F1F0-1F1F5"},
    {"bandiera_corea_del_sud", "1F1F0-1F1F7"},
    {"bandiera_kuwait", "1F1F0-1F1FC"},
    {"bandiera_isole_cayman", "1F1F0-1F1FE"},
    {"bandiera_kazakistan", "1F1F0-1F1FF"},
    {"bandiera_laos", "1F1F1-1F1E6"},
    {"bandiera_libano", "1F1F1-1F1E7"},
    {"bandiera_saint_lucia", "1F1F1-1F1E8"},
    {"bandiera_liechtenstein", "1F1F1-1F1EE"},
    {"bandiera_sri_lanka", "1F1F1-1F1F0"},
    {"bandiera_liberia", "1F1F1-1F1F7"},
    {"bandiera_lesotho", "1F1F1-1F1F8"},
    {"bandiera_lituania", "1F1F1-1F1F9"},
    {"bandiera_lussemburgo", "1F1F1-1F1FA"},
    {"bandiera_lettonia", "1F1F1-1F1FB"},
    {"bandiera_libia", "1F1F1-1F1FE"},
    {"bandiera_marocco", "1F1F2-1F1E6"},
    {"bandiera_monaco", "1F1F2-1F1E8"},
    {"bandiera_moldavia", "1F1F2-1F1E9"},
    {"bandiera_montenegro", "1F1F2-1F1EA"},
    {"bandiera_saint_martin", "1F1F2-1F1EB"},
    {"bandiera_madagascar", "1F1F2-1F1EC"},
    {"bandiera_isole_marshall", "1F1F2-1F1ED"},
    {"bandiera_macedonia_del_nord", "1F1F2-1F1F0"},
    {"bandiera_mali", "1F1F2-1F1F1"},
    {"bandiera_myanmar_birmania", "1F1F2-1F1F2"},
    {"bandiera_mongolia", "1F1F2-1F1F3"},
    {"bandiera_ras_di_macao", "1F1F2-1F1F4"},
    {"bandiera_isole_marianne_settentrionali", "1F1F2-1F1F5"},
    {"bandiera_martinica", "1F1F2-1F1F6"},
    {"bandiera_mauritania", "1F1F2-1F1F7"},
    {"bandiera_montserrat", "1F1F2-1F1F8"},
    {"bandiera_malta", "1F1F2-1F1F9"},
    {"bandiera_mauritius", "1F1F2-1F1FA"},
    {"bandiera_maldive", "1F1F2-1F1FB"},
    {"bandiera_malawi", "1F1F2-1F1FC"},
    {"bandiera_messico", "1F1F2-1F1FD"},
    {"bandiera_malaysia", "1F1F2-1F1FE"},
    {"bandiera_mozambico", "1F1F2-1F1FF"},
    {"bandiera_namibia", "1F1F3-1F1E6"},
    {"bandiera_nuova_caledonia", "1F1F3-1F1E8"},
    {"bandiera_niger", "1F1F3-1F1EA"},
    {"bandiera_isola_norfolk", "1F1F3-1F1EB"},
    {"bandiera_nigeria", "1F1F3-1F1EC"},
    {"bandiera_nicaragua", "1F1F3-1F1EE"},
    {"bandiera_paesi_bassi", "1F1F3-1F1F1"},
    {"bandiera_norvegia", "1F1F3-1F1F4"},
    {"bandiera_nepal", "1F1F3-1F1F5"},
    {"bandiera_nauru", "1F1F3-1F1F7"},
    {"bandiera_niue", "1F1F3-1F1FA"},
    {"bandiera_nuova_zelanda", "1F1F3-1F1FF"},
    {"bandiera_oman", "1F1F4-1F1F2"},
    {"bandiera_panama", "1F1F5-1F1E6"},
    {"bandiera_peru", "1F1F5-1F1EA"},
    {"bandiera_polinesia_francese", "1F1F5-1F1EB"},
    {"bandiera_papua_nuova_guinea", "1F1F5-1F1EC"},
    {"bandiera_filippine", "1F1F5-1F1ED"},
    {"bandiera_pakistan", "1F1F5-1F1F0"},
    {"bandiera_polonia", "1F1F5-1F1F1"},
    {"bandiera_saint_pierre_e_miquelon", "1F1F5-1F1F2"},
    {"bandiera_isole_pitcairn", "1F1F5-1F1F3"},
    {"bandiera_portorico", "1F1F5-1F1F7"},
    {"bandiera_territori_palestinesi", "1F1F5-1F1F8"},
    {"bandiera_portogallo", "1F1F5-1F1F9"},
    {"bandiera_palau", "1F1F5-1F1FC"},
    {"bandiera_paraguay", "1F1F5-1F1FE"},
    {"bandiera_qatar", "1F1F6-1F1E6"},
    {"bandiera_riunione", "1F1F7-1F1EA"},
    {"bandiera_romania", "1F1F7-1F1F4"},
    {"bandiera_serbia", "1F1F7-1F1F8"},
    {"bandiera_russia", "1F1F7-1F1FA"},
    {"bandiera_ruanda", "1F1F7-1F1FC"},
    {"bandiera_arabia_saudita", "1F1F8-1F1E6"},
    {"bandiera_isole_salomone", "1F1F8-1F1E7"},
    {"bandiera_seychelles", "1F1F8-1F1E8"},
    {"bandiera_sudan", "1F1F8-1F1E9"},
    {"bandiera_svezia", "1F1F8-1F1EA"},
    {"bandiera_singapore", "1F1F8-1F1EC"},
    {"bandiera_santelena", "1F1F8-1F1ED"},
    {"bandiera_slovenia", "1F1F8-1F1EE"},
    {"bandiera_svalbard_e_jan_mayen", "1F1F8-1F1EF"},
    {"bandiera_slovacchia", "1F1F8-1F1F0"},
    {"bandiera_sierra_leone", "1F1F8-1F1F1"},
    {"bandiera_san_marino", "1F1F8-1F1F2"},
    {"bandiera_senegal", "1F1F8-1F1F3"},
    {"bandiera_somalia", "1F1F8-1F1F4"},
    {"bandiera_suriname", "1F1F8-1F1F7"},
    {"bandiera_sud_sudan", "1F1F8-1F1F8"},
    {"bandiera_sao_tome_e_principe", "1F1F8-1F1F9"},
    {"bandiera_el_salvador", "1F1F8-1F1FB"},
    {"bandiera_sint_maarten", "1F1F8-1F1FD"},
    {"bandiera_siria", "1F1F8-1F1FE"},
    {"bandiera_eswatini", "1F1F8-1F1FF"},
    {"bandiera_tristan_da_cunha", "1F1F9-1F1E6"},
    {"bandiera_isole_turks_e_caicos", "1F1F9-1F1E8"},
    {"bandiera_ciad", "1F1F9-1F1E9"},
    {"bandiera_terre_australi_francesi", "1F1F9-1F1EB"},
    {"bandiera_togo", "1F1F9-1F1EC"},
    {"bandiera_thailandia", "1F1F9-1F1ED"},
    {"bandiera_tagikistan", "1F1F9-1F1EF"},
    {"bandiera_tokelau", "1F1F9-1F1F0"},
    {"bandiera_timor_est", "1F1F9-1F1F1"},
    {"bandiera_turkmenistan", "1F1F9-1F1F2"},
    {"bandiera_tunisia", "1F1F9-1F1F3"},
    {"bandiera_tonga", "1F1F9-1F1F4"},
    {"bandiera_turchia", "1F1F9-1F1F7"},
    {"bandiera_trinidad_e_tobago", "1F1F9-1F1F9"},
    {"bandiera_tuvalu", "1F1F9-1F1FB"},
    {"bandiera_taiwan", "1F1F9-1F1FC"},
    {"bandiera_tanzania", "1F1F9-1F1FF"},
    {"bandiera_ucraina", "1F1FA-1F1E6"},
    {"bandiera_uganda", "1F1FA-1F1EC"},
    {"bandiera_isole_minori_esterne_degli_stati_uniti", "1F1FA-1F1F2"},
    {"bandiera_nazioni_unite", "1F1FA-1F1F3"},
    {"bandiera_stati_uniti", "1F1FA-1F1F8"},
    {"bandiera_uruguay", "1F1FA-1F1FE"},
    {"bandiera_uzbekistan", "1F1FA-1F1FF"},
    {"bandiera_citta_del_vaticano", "1F1FB-1F1E6"},
    {"bandiera_saint_vincent_e_grenadine", "1F1FB-1F1E8"},
    {"bandiera_venezuela", "1F1FB-1F1EA"},
    {"bandiera_isole_vergini_britanniche", "1F1FB-1F1EC"},
    {"bandiera_isole_vergini_americane", "1F1FB-1F1EE"},
    {"bandiera_vietnam", "1F1FB-1F1F3"},
    {"bandiera_vanuatu", "1F1FB-1F1FA"},
    {"bandiera_wallis_e_futuna", "1F1FC-1F1EB"},
    {"bandiera_samoa", "1F1FC-1F1F8"},
    {"bandiera_kosovo", "1F1FD-1F1F0"},
    {"bandiera_yemen", "1F1FE-1F1EA"},
    {"bandiera_mayotte", "1F1FE-1F1F9"},
    {"bandiera_sudafrica", "1F1FF-1F1E6"},
    {"bandiera_zambia", "1F1FF-1F1F2"},
    {"bandiera_zimbabwe", "1F1FF-1F1FC"},
    {"bandiera_inghilterra", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"bandiera_scozia", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"bandiera_galles", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"coppia_con_cuore_uomo_e_uomo", "1F468-200D-2764-FE0F-200D-1F468"},
    {"coppia_con_cuore_uomo_e_uomo_colore1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"coppia_con_cuore_uomo_e_uomo_colore1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"coppia_con_cuore_uomo_e_uomo_colore1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"coppia_con_cuore_uomo_e_uomo_colore1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"coppia_con_cuore_uomo_e_uomo_colore1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"coppia_con_cuore_uomo_e_uomo_colore2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"coppia_con_cuore_uomo_e_uomo_colore2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"coppia_con_cuore_uomo_e_uomo_colore2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"coppia_con_cuore_uomo_e_uomo_colore2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"coppia_con_cuore_uomo_e_uomo_colore2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"coppia_con_cuore_uomo_e_uomo_colore3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"coppia_con_cuore_uomo_e_uomo_colore3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"coppia_con_cuore_uomo_e_uomo_colore3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"coppia_con_cuore_uomo_e_uomo_colore3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"coppia_con_cuore_uomo_e_uomo_colore3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"coppia_con_cuore_uomo_e_uomo_colore4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"coppia_con_cuore_uomo_e_uomo_colore4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"coppia_con_cuore_uomo_e_uomo_colore4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"coppia_con_cuore_uomo_e_uomo_colore4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"coppia_con_cuore_uomo_e_uomo_colore4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"coppia_con_cuore_uomo_e_uomo_colore5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"coppia_con_cuore_uomo_e_uomo_colore5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"coppia_con_cuore_uomo_e_uomo_colore5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"coppia_con_cuore_uomo_e_uomo_colore5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"coppia_con_cuore_uomo_e_uomo_colore5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"bacio_tra_coppia_uomo_e_uomo", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"bacio_tra_coppia_uomo_e_uomo_colore1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bacio_tra_coppia_uomo_e_uomo_colore1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bacio_tra_coppia_uomo_e_uomo_colore1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bacio_tra_coppia_uomo_e_uomo_colore1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bacio_tra_coppia_uomo_e_uomo_colore1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bacio_tra_coppia_uomo_e_uomo_colore2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bacio_tra_coppia_uomo_e_uomo_colore2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bacio_tra_coppia_uomo_e_uomo_colore2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bacio_tra_coppia_uomo_e_uomo_colore2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bacio_tra_coppia_uomo_e_uomo_colore2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bacio_tra_coppia_uomo_e_uomo_colore3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bacio_tra_coppia_uomo_e_uomo_colore3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bacio_tra_coppia_uomo_e_uomo_colore3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bacio_tra_coppia_uomo_e_uomo_colore3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bacio_tra_coppia_uomo_e_uomo_colore3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bacio_tra_coppia_uomo_e_uomo_colore4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bacio_tra_coppia_uomo_e_uomo_colore4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bacio_tra_coppia_uomo_e_uomo_colore4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bacio_tra_coppia_uomo_e_uomo_colore4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bacio_tra_coppia_uomo_e_uomo_colore4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bacio_tra_coppia_uomo_e_uomo_colore5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bacio_tra_coppia_uomo_e_uomo_colore5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bacio_tra_coppia_uomo_e_uomo_colore5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bacio_tra_coppia_uomo_e_uomo_colore5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bacio_tra_coppia_uomo_e_uomo_colore5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"famiglia_uomo_e_bambino", "1F468-200D-1F466"},
    {"famiglia_uomo_bambino_e_bambino", "1F468-200D-1F466-200D-1F466"},
    {"famiglia_uomo_e_bambina", "1F468-200D-1F467"},
    {"famiglia_uomo_bambina_e_bambino", "1F468-200D-1F467-200D-1F466"},
    {"famiglia_uomo_bambina_e_bambina", "1F468-200D-1F467-200D-1F467"},
    {"famiglia_uomo_uomo_e_bambino", "1F468-200D-1F468-200D-1F466"},
    {"famiglia_uomo_uomo_bambino_e_bambino", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"famiglia_uomo_uomo_e_bambina", "1F468-200D-1F468-200D-1F467"},
    {"famiglia_uomo_uomo_bambina_e_bambino", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"famiglia_uomo_uomo_bambina_e_bambina", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"famiglia_uomo_donna_e_bambino", "1F468-200D-1F469-200D-1F466"},
    {"famiglia_uomo_donna_bambino_e_bambino", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"famiglia_uomo_donna_e_bambina", "1F468-200D-1F469-200D-1F467"},
    {"famiglia_uomo_donna_bambina_e_bambino", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"famiglia_uomo_donna_bambina_e_bambina", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"coppia_con_cuore_donna_e_uomo", "1F469-200D-2764-FE0F-200D-1F468"},
    {"coppia_con_cuore_donna_e_uomo_colore1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"coppia_con_cuore_donna_e_uomo_colore1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"coppia_con_cuore_donna_e_uomo_colore1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"coppia_con_cuore_donna_e_uomo_colore1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"coppia_con_cuore_donna_e_uomo_colore1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"coppia_con_cuore_donna_e_uomo_colore2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"coppia_con_cuore_donna_e_uomo_colore2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"coppia_con_cuore_donna_e_uomo_colore2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"coppia_con_cuore_donna_e_uomo_colore2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"coppia_con_cuore_donna_e_uomo_colore2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"coppia_con_cuore_donna_e_uomo_colore3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"coppia_con_cuore_donna_e_uomo_colore3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"coppia_con_cuore_donna_e_uomo_colore3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"coppia_con_cuore_donna_e_uomo_colore3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"coppia_con_cuore_donna_e_uomo_colore3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"coppia_con_cuore_donna_e_uomo_colore4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"coppia_con_cuore_donna_e_uomo_colore4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"coppia_con_cuore_donna_e_uomo_colore4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"coppia_con_cuore_donna_e_uomo_colore4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"coppia_con_cuore_donna_e_uomo_colore4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"coppia_con_cuore_donna_e_uomo_colore5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"coppia_con_cuore_donna_e_uomo_colore5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"coppia_con_cuore_donna_e_uomo_colore5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"coppia_con_cuore_donna_e_uomo_colore5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"coppia_con_cuore_donna_e_uomo_colore5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"coppia_con_cuore_donna_e_donna", "1F469-200D-2764-FE0F-200D-1F469"},
    {"coppia_con_cuore_donna_e_donna_colore1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"coppia_con_cuore_donna_e_donna_colore1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"coppia_con_cuore_donna_e_donna_colore1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"coppia_con_cuore_donna_e_donna_colore1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"coppia_con_cuore_donna_e_donna_colore1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"coppia_con_cuore_donna_e_donna_colore2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"coppia_con_cuore_donna_e_donna_colore2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"coppia_con_cuore_donna_e_donna_colore2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"coppia_con_cuore_donna_e_donna_colore2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"coppia_con_cuore_donna_e_donna_colore2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"coppia_con_cuore_donna_e_donna_colore3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"coppia_con_cuore_donna_e_donna_colore3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"coppia_con_cuore_donna_e_donna_colore3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"coppia_con_cuore_donna_e_donna_colore3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"coppia_con_cuore_donna_e_donna_colore3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"coppia_con_cuore_donna_e_donna_colore4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"coppia_con_cuore_donna_e_donna_colore4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"coppia_con_cuore_donna_e_donna_colore4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"coppia_con_cuore_donna_e_donna_colore4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"coppia_con_cuore_donna_e_donna_colore4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"coppia_con_cuore_donna_e_donna_colore5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"coppia_con_cuore_donna_e_donna_colore5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"coppia_con_cuore_donna_e_donna_colore5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"coppia_con_cuore_donna_e_donna_colore5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"coppia_con_cuore_donna_e_donna_colore5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"bacio_tra_coppia_donna_e_uomo", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"bacio_tra_coppia_donna_e_uomo_colore1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bacio_tra_coppia_donna_e_uomo_colore1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bacio_tra_coppia_donna_e_uomo_colore1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bacio_tra_coppia_donna_e_uomo_colore1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bacio_tra_coppia_donna_e_uomo_colore1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bacio_tra_coppia_donna_e_uomo_colore2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bacio_tra_coppia_donna_e_uomo_colore2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bacio_tra_coppia_donna_e_uomo_colore2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bacio_tra_coppia_donna_e_uomo_colore2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bacio_tra_coppia_donna_e_uomo_colore2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bacio_tra_coppia_donna_e_uomo_colore3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bacio_tra_coppia_donna_e_uomo_colore3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bacio_tra_coppia_donna_e_uomo_colore3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bacio_tra_coppia_donna_e_uomo_colore3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bacio_tra_coppia_donna_e_uomo_colore3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bacio_tra_coppia_donna_e_uomo_colore4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bacio_tra_coppia_donna_e_uomo_colore4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bacio_tra_coppia_donna_e_uomo_colore4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bacio_tra_coppia_donna_e_uomo_colore4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bacio_tra_coppia_donna_e_uomo_colore4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bacio_tra_coppia_donna_e_uomo_colore5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bacio_tra_coppia_donna_e_uomo_colore5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bacio_tra_coppia_donna_e_uomo_colore5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bacio_tra_coppia_donna_e_uomo_colore5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bacio_tra_coppia_donna_e_uomo_colore5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bacio_tra_coppia_donna_e_donna", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"bacio_tra_coppia_donna_e_donna_colore1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bacio_tra_coppia_donna_e_donna_colore1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bacio_tra_coppia_donna_e_donna_colore1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bacio_tra_coppia_donna_e_donna_colore1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bacio_tra_coppia_donna_e_donna_colore1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bacio_tra_coppia_donna_e_donna_colore2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bacio_tra_coppia_donna_e_donna_colore2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bacio_tra_coppia_donna_e_donna_colore2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bacio_tra_coppia_donna_e_donna_colore2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bacio_tra_coppia_donna_e_donna_colore2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bacio_tra_coppia_donna_e_donna_colore3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bacio_tra_coppia_donna_e_donna_colore3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bacio_tra_coppia_donna_e_donna_colore3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bacio_tra_coppia_donna_e_donna_colore3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bacio_tra_coppia_donna_e_donna_colore3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bacio_tra_coppia_donna_e_donna_colore4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bacio_tra_coppia_donna_e_donna_colore4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bacio_tra_coppia_donna_e_donna_colore4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bacio_tra_coppia_donna_e_donna_colore4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bacio_tra_coppia_donna_e_donna_colore4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bacio_tra_coppia_donna_e_donna_colore5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bacio_tra_coppia_donna_e_donna_colore5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bacio_tra_coppia_donna_e_donna_colore5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bacio_tra_coppia_donna_e_donna_colore5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bacio_tra_coppia_donna_e_donna_colore5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"famiglia_donna_e_bambino", "1F469-200D-1F466"},
    {"famiglia_donna_bambino_e_bambino", "1F469-200D-1F466-200D-1F466"},
    {"famiglia_donna_e_bambina", "1F469-200D-1F467"},
    {"famiglia_donna_bambina_e_bambino", "1F469-200D-1F467-200D-1F466"},
    {"famiglia_donna_bambina_e_bambina", "1F469-200D-1F467-200D-1F467"},
    {"famiglia_donna_donna_e_bambino", "1F469-200D-1F469-200D-1F466"},
    {"famiglia_donna_donna_bambino_e_bambino", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"famiglia_donna_donna_e_bambina", "1F469-200D-1F469-200D-1F467"},
    {"famiglia_donna_donna_bambina_e_bambino", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"famiglia_donna_donna_bambina_e_bambina", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"persone_che_si_tengono_per_mano", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"persone_che_si_tengono_per_mano_colore1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"persone_che_si_tengono_per_mano_colore1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"persone_che_si_tengono_per_mano_colore1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"persone_che_si_tengono_per_mano_colore1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"persone_che_si_tengono_per_mano_colore1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"persone_che_si_tengono_per_mano_colore2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"persone_che_si_tengono_per_mano_colore2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"persone_che_si_tengono_per_mano_colore2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"persone_che_si_tengono_per_mano_colore2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"persone_che_si_tengono_per_mano_colore2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"persone_che_si_tengono_per_mano_colore3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"persone_che_si_tengono_per_mano_colore3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"persone_che_si_tengono_per_mano_colore3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"persone_che_si_tengono_per_mano_colore3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"persone_che_si_tengono_per_mano_colore3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"persone_che_si_tengono_per_mano_colore4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"persone_che_si_tengono_per_mano_colore4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"persone_che_si_tengono_per_mano_colore4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"persone_che_si_tengono_per_mano_colore4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"persone_che_si_tengono_per_mano_colore4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"persone_che_si_tengono_per_mano_colore5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"persone_che_si_tengono_per_mano_colore5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"persone_che_si_tengono_per_mano_colore5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"persone_che_si_tengono_per_mano_colore5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"persone_che_si_tengono_per_mano_colore5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"famiglia_persone_adulte_minore", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"famiglia_persone_adulte_minori", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"famiglia_persona_adulta_minore", "1F9D1-200D-1F9D2"},
    {"famiglia_persona_adulta_minore_minore", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"persona_che_corre_a_destra", "1F3C3-200D-27A1-FE0F"},
    {"persona_che_corre_a_destra_colore1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"persona_che_corre_a_destra_colore2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"persona_che_corre_a_destra_colore3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"persona_che_corre_a_destra_colore4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"persona_che_corre_a_destra_colore5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"operatore_sanitario", "1F468-200D-2695-FE0F"},
    {"operatore_sanitario_colore1", "1F468-1F3FB-200D-2695-FE0F"},
    {"operatore_sanitario_colore2", "1F468-1F3FC-200D-2695-FE0F"},
    {"operatore_sanitario_colore3", "1F468-1F3FD-200D-2695-FE0F"},
    {"operatore_sanitario_colore4", "1F468-1F3FE-200D-2695-FE0F"},
    {"operatore_sanitario_colore5", "1F468-1F3FF-200D-2695-FE0F"},
    {"giudice_uomo", "1F468-200D-2696-FE0F"},
    {"giudice_uomo_colore1", "1F468-1F3FB-200D-2696-FE0F"},
    {"giudice_uomo_colore2", "1F468-1F3FC-200D-2696-FE0F"},
    {"giudice_uomo_colore3", "1F468-1F3FD-200D-2696-FE0F"},
    {"giudice_uomo_colore4", "1F468-1F3FE-200D-2696-FE0F"},
    {"giudice_uomo_colore5", "1F468-1F3FF-200D-2696-FE0F"},
    {"pilota_uomo", "1F468-200D-2708-FE0F"},
    {"pilota_uomo_colore1", "1F468-1F3FB-200D-2708-FE0F"},
    {"pilota_uomo_colore2", "1F468-1F3FC-200D-2708-FE0F"},
    {"pilota_uomo_colore3", "1F468-1F3FD-200D-2708-FE0F"},
    {"pilota_uomo_colore4", "1F468-1F3FE-200D-2708-FE0F"},
    {"pilota_uomo_colore5", "1F468-1F3FF-200D-2708-FE0F"},
    {"contadino", "1F468-200D-1F33E"},
    {"contadino_colore1", "1F468-1F3FB-200D-1F33E"},
    {"contadino_colore2", "1F468-1F3FC-200D-1F33E"},
    {"contadino_colore3", "1F468-1F3FD-200D-1F33E"},
    {"contadino_colore4", "1F468-1F3FE-200D-1F33E"},
    {"contadino_colore5", "1F468-1F3FF-200D-1F33E"},
    {"cuoco", "1F468-200D-1F373"},
    {"cuoco_colore1", "1F468-1F3FB-200D-1F373"},
    {"cuoco_colore2", "1F468-1F3FC-200D-1F373"},
    {"cuoco_colore3", "1F468-1F3FD-200D-1F373"},
    {"cuoco_colore4", "1F468-1F3FE-200D-1F373"},
    {"cuoco_colore5", "1F468-1F3FF-200D-1F373"},
    {"uomo_che_allatta", "1F468-200D-1F37C"},
    {"uomo_che_allatta_colore1", "1F468-1F3FB-200D-1F37C"},
    {"uomo_che_allatta_colore2", "1F468-1F3FC-200D-1F37C"},
    {"uomo_che_allatta_colore3", "1F468-1F3FD-200D-1F37C"},
    {"uomo_che_allatta_colore4", "1F468-1F3FE-200D-1F37C"},
    {"uomo_che_allatta_colore5", "1F468-1F3FF-200D-1F37C"},
    {"studente_maschio", "1F468-200D-1F393"},
    {"studente_maschio_colore1", "1F468-1F3FB-200D-1F393"},
    {"studente_maschio_colore2", "1F468-1F3FC-200D-1F393"},
    {"studente_maschio_colore3", "1F468-1F3FD-200D-1F393"},
    {"studente_maschio_colore4", "1F468-1F3FE-200D-1F393"},
    {"studente_maschio_colore5", "1F468-1F3FF-200D-1F393"},
    {"cantante_uomo", "1F468-200D-1F3A4"},
    {"cantante_uomo_colore1", "1F468-1F3FB-200D-1F3A4"},
    {"cantante_uomo_colore2", "1F468-1F3FC-200D-1F3A4"},
    {"cantante_uomo_colore3", "1F468-1F3FD-200D-1F3A4"},
    {"cantante_uomo_colore4", "1F468-1F3FE-200D-1F3A4"},
    {"cantante_uomo_colore5", "1F468-1F3FF-200D-1F3A4"},
    {"artista_uomo", "1F468-200D-1F3A8"},
    {"artista_uomo_colore1", "1F468-1F3FB-200D-1F3A8"},
    {"artista_uomo_colore2", "1F468-1F3FC-200D-1F3A8"},
    {"artista_uomo_colore3", "1F468-1F3FD-200D-1F3A8"},
    {"artista_uomo_colore4", "1F468-1F3FE-200D-1F3A8"},
    {"artista_uomo_colore5", "1F468-1F3FF-200D-1F3A8"},
    {"professore", "1F468-200D-1F3EB"},
    {"professore_colore1", "1F468-1F3FB-200D-1F3EB"},
    {"professore_colore2", "1F468-1F3FC-200D-1F3EB"},
    {"professore_colore3", "1F468-1F3FD-200D-1F3EB"},
    {"professore_colore4", "1F468-1F3FE-200D-1F3EB"},
    {"professore_colore5", "1F468-1F3FF-200D-1F3EB"},
    {"operaio", "1F468-200D-1F3ED"},
    {"operaio_colore1", "1F468-1F3FB-200D-1F3ED"},
    {"operaio_colore2", "1F468-1F3FC-200D-1F3ED"},
    {"operaio_colore3", "1F468-1F3FD-200D-1F3ED"},
    {"operaio_colore4", "1F468-1F3FE-200D-1F3ED"},
    {"operaio_colore5", "1F468-1F3FF-200D-1F3ED"},
    {"tecnologo", "1F468-200D-1F4BB"},
    {"tecnologo_colore1", "1F468-1F3FB-200D-1F4BB"},
    {"tecnologo_colore2", "1F468-1F3FC-200D-1F4BB"},
    {"tecnologo_colore3", "1F468-1F3FD-200D-1F4BB"},
    {"tecnologo_colore4", "1F468-1F3FE-200D-1F4BB"},
    {"tecnologo_colore5", "1F468-1F3FF-200D-1F4BB"},
    {"impiegato", "1F468-200D-1F4BC"},
    {"impiegato_colore1", "1F468-1F3FB-200D-1F4BC"},
    {"impiegato_colore2", "1F468-1F3FC-200D-1F4BC"},
    {"impiegato_colore3", "1F468-1F3FD-200D-1F4BC"},
    {"impiegato_colore4", "1F468-1F3FE-200D-1F4BC"},
    {"impiegato_colore5", "1F468-1F3FF-200D-1F4BC"},
    {"meccanico_uomo", "1F468-200D-1F527"},
    {"meccanico_uomo_colore1", "1F468-1F3FB-200D-1F527"},
    {"meccanico_uomo_colore2", "1F468-1F3FC-200D-1F527"},
    {"meccanico_uomo_colore3", "1F468-1F3FD-200D-1F527"},
    {"meccanico_uomo_colore4", "1F468-1F3FE-200D-1F527"},
    {"meccanico_uomo_colore5", "1F468-1F3FF-200D-1F527"},
    {"scienziato", "1F468-200D-1F52C"},
    {"scienziato_colore1", "1F468-1F3FB-200D-1F52C"},
    {"scienziato_colore2", "1F468-1F3FC-200D-1F52C"},
    {"scienziato_colore3", "1F468-1F3FD-200D-1F52C"},
    {"scienziato_colore4", "1F468-1F3FE-200D-1F52C"},
    {"scienziato_colore5", "1F468-1F3FF-200D-1F52C"},
    {"astronauta_uomo", "1F468-200D-1F680"},
    {"astronauta_uomo_colore1", "1F468-1F3FB-200D-1F680"},
    {"astronauta_uomo_colore2", "1F468-1F3FC-200D-1F680"},
    {"astronauta_uomo_colore3", "1F468-1F3FD-200D-1F680"},
    {"astronauta_uomo_colore4", "1F468-1F3FE-200D-1F680"},
    {"astronauta_uomo_colore5", "1F468-1F3FF-200D-1F680"},
    {"pompiere_uomo", "1F468-200D-1F692"},
    {"pompiere_uomo_colore1", "1F468-1F3FB-200D-1F692"},
    {"pompiere_uomo_colore2", "1F468-1F3FC-200D-1F692"},
    {"pompiere_uomo_colore3", "1F468-1F3FD-200D-1F692"},
    {"pompiere_uomo_colore4", "1F468-1F3FE-200D-1F692"},
    {"pompiere_uomo_colore5", "1F468-1F3FF-200D-1F692"},
    {"uomo_con_bastone_bianco_di_orientamento", "1F468-200D-1F9AF"},
    {"uomo_con_bastone_bianco_di_orientamento_colore1", "1F468-1F3FB-200D-1F9AF"},
    {"uomo_con_bastone_bianco_di_orientamento_colore2", "1F468-1F3FC-200D-1F9AF"},
    {"uomo_con_bastone_bianco_di_orientamento_colore3", "1F468-1F3FD-200D-1F9AF"},
    {"uomo_con_bastone_bianco_di_orientamento_colore4", "1F468-1F3FE-200D-1F9AF"},
    {"uomo_con_bastone_bianco_di_orientamento_colore5", "1F468-1F3FF-200D-1F9AF"},
    {"uomo_con_bastone_bianco_di_orientamento_a_destra", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"uomo_con_bastone_bianco_di_orientamento_a_destra_colore1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"uomo_con_bastone_bianco_di_orientamento_a_destra_colore2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"uomo_con_bastone_bianco_di_orientamento_a_destra_colore3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"uomo_con_bastone_bianco_di_orientamento_a_destra_colore4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"uomo_con_bastone_bianco_di_orientamento_a_destra_colore5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"uomo_su_sedia_a_rotelle_motorizzata", "1F468-200D-1F9BC"},
    {"uomo_su_sedia_a_rotelle_motorizzata_colore1", "1F468-1F3FB-200D-1F9BC"},
    {"uomo_su_sedia_a_rotelle_motorizzata_colore2", "1F468-1F3FC-200D-1F9BC"},
    {"uomo_su_sedia_a_rotelle_motorizzata_colore3", "1F468-1F3FD-200D-1F9BC"},
    {"uomo_su_sedia_a_rotelle_motorizzata_colore4", "1F468-1F3FE-200D-1F9BC"},
    {"uomo_su_sedia_a_rotelle_motorizzata_colore5", "1F468-1F3FF-200D-1F9BC"},
    {"uomo_su_sedia_a_rotelle_motorizzata_a_destra", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"uomo_su_sedia_a_rotelle_motorizzata_a_destra_colore1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"uomo_su_sedia_a_rotelle_motorizzata_a_destra_colore2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"uomo_su_sedia_a_rotelle_motorizzata_a_destra_colore3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"uomo_su_sedia_a_rotelle_motorizzata_a_destra_colore4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"uomo_su_sedia_a_rotelle_motorizzata_a_destra_colore5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"uomo_su_sedia_a_rotelle_manuale", "1F468-200D-1F9BD"},
    {"uomo_su_sedia_a_rotelle_manuale_colore1", "1F468-1F3FB-200D-1F9BD"},
    {"uomo_su_sedia_a_rotelle_manuale_colore2", "1F468-1F3FC-200D-1F9BD"},
    {"uomo_su_sedia_a_rotelle_manuale_colore3", "1F468-1F3FD-200D-1F9BD"},
    {"uomo_su_sedia_a_rotelle_manuale_colore4", "1F468-1F3FE-200D-1F9BD"},
    {"uomo_su_sedia_a_rotelle_manuale_colore5", "1F468-1F3FF-200D-1F9BD"},
    {"uomo_su_sedia_a_rotelle_manuale_a_destra", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"uomo_su_sedia_a_rotelle_manuale_a_destra_colore1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"uomo_su_sedia_a_rotelle_manuale_a_destra_colore2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"uomo_su_sedia_a_rotelle_manuale_a_destra_colore3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"uomo_su_sedia_a_rotelle_manuale_a_destra_colore4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"uomo_su_sedia_a_rotelle_manuale_a_destra_colore5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"operatrice_sanitaria", "1F469-200D-2695-FE0F"},
    {"operatrice_sanitaria_colore1", "1F469-1F3FB-200D-2695-FE0F"},
    {"operatrice_sanitaria_colore2", "1F469-1F3FC-200D-2695-FE0F"},
    {"operatrice_sanitaria_colore3", "1F469-1F3FD-200D-2695-FE0F"},
    {"operatrice_sanitaria_colore4", "1F469-1F3FE-200D-2695-FE0F"},
    {"operatrice_sanitaria_colore5", "1F469-1F3FF-200D-2695-FE0F"},
    {"giudice_donna", "1F469-200D-2696-FE0F"},
    {"giudice_donna_colore1", "1F469-1F3FB-200D-2696-FE0F"},
    {"giudice_donna_colore2", "1F469-1F3FC-200D-2696-FE0F"},
    {"giudice_donna_colore3", "1F469-1F3FD-200D-2696-FE0F"},
    {"giudice_donna_colore4", "1F469-1F3FE-200D-2696-FE0F"},
    {"giudice_donna_colore5", "1F469-1F3FF-200D-2696-FE0F"},
    {"pilota_donna", "1F469-200D-2708-FE0F"},
    {"pilota_donna_colore1", "1F469-1F3FB-200D-2708-FE0F"},
    {"pilota_donna_colore2", "1F469-1F3FC-200D-2708-FE0F"},
    {"pilota_donna_colore3", "1F469-1F3FD-200D-2708-FE0F"},
    {"pilota_donna_colore4", "1F469-1F3FE-200D-2708-FE0F"},
    {"pilota_donna_colore5", "1F469-1F3FF-200D-2708-FE0F"},
    {"contadina", "1F469-200D-1F33E"},
    {"contadina_colore1", "1F469-1F3FB-200D-1F33E"},
    {"contadina_colore2", "1F469-1F3FC-200D-1F33E"},
    {"contadina_colore3", "1F469-1F3FD-200D-1F33E"},
    {"contadina_colore4", "1F469-1F3FE-200D-1F33E"},
    {"contadina_colore5", "1F469-1F3FF-200D-1F33E"},
    {"cuoca", "1F469-200D-1F373"},
    {"cuoca_colore1", "1F469-1F3FB-200D-1F373"},
    {"cuoca_colore2", "1F469-1F3FC-200D-1F373"},
    {"cuoca_colore3", "1F469-1F3FD-200D-1F373"},
    {"cuoca_colore4", "1F469-1F3FE-200D-1F373"},
    {"cuoca_colore5", "1F469-1F3FF-200D-1F373"},
    {"donna_che_allatta", "1F469-200D-1F37C"},
    {"donna_che_allatta_colore1", "1F469-1F3FB-200D-1F37C"},
    {"donna_che_allatta_colore2", "1F469-1F3FC-200D-1F37C"},
    {"donna_che_allatta_colore3", "1F469-1F3FD-200D-1F37C"},
    {"donna_che_allatta_colore4", "1F469-1F3FE-200D-1F37C"},
    {"donna_che_allatta_colore5", "1F469-1F3FF-200D-1F37C"},
    {"studentessa", "1F469-200D-1F393"},
    {"studentessa_colore1", "1F469-1F3FB-200D-1F393"},
    {"studentessa_colore2", "1F469-1F3FC-200D-1F393"},
    {"studentessa_colore3", "1F469-1F3FD-200D-1F393"},
    {"studentessa_colore4", "1F469-1F3FE-200D-1F393"},
    {"studentessa_colore5", "1F469-1F3FF-200D-1F393"},
    {"cantante_donna", "1F469-200D-1F3A4"},
    {"cantante_donna_colore1", "1F469-1F3FB-200D-1F3A4"},
    {"cantante_donna_colore2", "1F469-1F3FC-200D-1F3A4"},
    {"cantante_donna_colore3", "1F469-1F3FD-200D-1F3A4"},
    {"cantante_donna_colore4", "1F469-1F3FE-200D-1F3A4"},
    {"cantante_donna_colore5", "1F469-1F3FF-200D-1F3A4"},
    {"artista_donna", "1F469-200D-1F3A8"},
    {"artista_donna_colore1", "1F469-1F3FB-200D-1F3A8"},
    {"artista_donna_colore2", "1F469-1F3FC-200D-1F3A8"},
    {"artista_donna_colore3", "1F469-1F3FD-200D-1F3A8"},
    {"artista_donna_colore4", "1F469-1F3FE-200D-1F3A8"},
    {"artista_donna_colore5", "1F469-1F3FF-200D-1F3A8"},
    {"professoressa", "1F469-200D-1F3EB"},
    {"professoressa_colore1", "1F469-1F3FB-200D-1F3EB"},
    {"professoressa_colore2", "1F469-1F3FC-200D-1F3EB"},
    {"professoressa_colore3", "1F469-1F3FD-200D-1F3EB"},
    {"professoressa_colore4", "1F469-1F3FE-200D-1F3EB"},
    {"professoressa_colore5", "1F469-1F3FF-200D-1F3EB"},
    {"operaia", "1F469-200D-1F3ED"},
    {"operaia_colore1", "1F469-1F3FB-200D-1F3ED"},
    {"operaia_colore2", "1F469-1F3FC-200D-1F3ED"},
    {"operaia_colore3", "1F469-1F3FD-200D-1F3ED"},
    {"operaia_colore4", "1F469-1F3FE-200D-1F3ED"},
    {"operaia_colore5", "1F469-1F3FF-200D-1F3ED"},
    {"tecnologa", "1F469-200D-1F4BB"},
    {"tecnologa_colore1", "1F469-1F3FB-200D-1F4BB"},
    {"tecnologa_colore2", "1F469-1F3FC-200D-1F4BB"},
    {"tecnologa_colore3", "1F469-1F3FD-200D-1F4BB"},
    {"tecnologa_colore4", "1F469-1F3FE-200D-1F4BB"},
    {"tecnologa_colore5", "1F469-1F3FF-200D-1F4BB"},
    {"impiegata", "1F469-200D-1F4BC"},
    {"impiegata_colore1", "1F469-1F3FB-200D-1F4BC"},
    {"impiegata_colore2", "1F469-1F3FC-200D-1F4BC"},
    {"impiegata_colore3", "1F469-1F3FD-200D-1F4BC"},
    {"impiegata_colore4", "1F469-1F3FE-200D-1F4BC"},
    {"impiegata_colore5", "1F469-1F3FF-200D-1F4BC"},
    {"meccanico_donna", "1F469-200D-1F527"},
    {"meccanico_donna_colore1", "1F469-1F3FB-200D-1F527"},
    {"meccanico_donna_colore2", "1F469-1F3FC-200D-1F527"},
    {"meccanico_donna_colore3", "1F469-1F3FD-200D-1F527"},
    {"meccanico_donna_colore4", "1F469-1F3FE-200D-1F527"},
    {"meccanico_donna_colore5", "1F469-1F3FF-200D-1F527"},
    {"scienziata", "1F469-200D-1F52C"},
    {"scienziata_colore1", "1F469-1F3FB-200D-1F52C"},
    {"scienziata_colore2", "1F469-1F3FC-200D-1F52C"},
    {"scienziata_colore3", "1F469-1F3FD-200D-1F52C"},
    {"scienziata_colore4", "1F469-1F3FE-200D-1F52C"},
    {"scienziata_colore5", "1F469-1F3FF-200D-1F52C"},
    {"astronauta_donna", "1F469-200D-1F680"},
    {"astronauta_donna_colore1", "1F469-1F3FB-200D-1F680"},
    {"astronauta_donna_colore2", "1F469-1F3FC-200D-1F680"},
    {"astronauta_donna_colore3", "1F469-1F3FD-200D-1F680"},
    {"astronauta_donna_colore4", "1F469-1F3FE-200D-1F680"},
    {"astronauta_donna_colore5", "1F469-1F3FF-200D-1F680"},
    {"pompiere_donna", "1F469-200D-1F692"},
    {"pompiere_donna_colore1", "1F469-1F3FB-200D-1F692"},
    {"pompiere_donna_colore2", "1F469-1F3FC-200D-1F692"},
    {"pompiere_donna_colore3", "1F469-1F3FD-200D-1F692"},
    {"pompiere_donna_colore4", "1F469-1F3FE-200D-1F692"},
    {"pompiere_donna_colore5", "1F469-1F3FF-200D-1F692"},
    {"donna_con_bastone_bianco_di_orientamento", "1F469-200D-1F9AF"},
    {"donna_con_bastone_bianco_di_orientamento_colore1", "1F469-1F3FB-200D-1F9AF"},
    {"donna_con_bastone_bianco_di_orientamento_colore2", "1F469-1F3FC-200D-1F9AF"},
    {"donna_con_bastone_bianco_di_orientamento_colore3", "1F469-1F3FD-200D-1F9AF"},
    {"donna_con_bastone_bianco_di_orientamento_colore4", "1F469-1F3FE-200D-1F9AF"},
    {"donna_con_bastone_bianco_di_orientamento_colore5", "1F469-1F3FF-200D-1F9AF"},
    {"donna_con_bastone_bianco_di_orientamento_a_destra", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"donna_con_bastone_bianco_di_orientamento_a_destra_colore1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"donna_con_bastone_bianco_di_orientamento_a_destra_colore2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"donna_con_bastone_bianco_di_orientamento_a_destra_colore3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"donna_con_bastone_bianco_di_orientamento_a_destra_colore4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"donna_con_bastone_bianco_di_orientamento_a_destra_colore5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"donna_su_sedia_a_rotelle_motorizzata", "1F469-200D-1F9BC"},
    {"donna_su_sedia_a_rotelle_motorizzata_colore1", "1F469-1F3FB-200D-1F9BC"},
    {"donna_su_sedia_a_rotelle_motorizzata_colore2", "1F469-1F3FC-200D-1F9BC"},
    {"donna_su_sedia_a_rotelle_motorizzata_colore3", "1F469-1F3FD-200D-1F9BC"},
    {"donna_su_sedia_a_rotelle_motorizzata_colore4", "1F469-1F3FE-200D-1F9BC"},
    {"donna_su_sedia_a_rotelle_motorizzata_colore5", "1F469-1F3FF-200D-1F9BC"},
    {"donna_su_sedia_a_rotelle_motorizzata_a_destra", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"donna_su_sedia_a_rotelle_motorizzata_a_destra_colore1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"donna_su_sedia_a_rotelle_motorizzata_a_destra_colore2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"donna_su_sedia_a_rotelle_motorizzata_a_destra_colore3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"donna_su_sedia_a_rotelle_motorizzata_a_destra_colore4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"donna_su_sedia_a_rotelle_motorizzata_a_destra_colore5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"donna_su_sedia_a_rotelle_manuale", "1F469-200D-1F9BD"},
    {"donna_su_sedia_a_rotelle_manuale_colore1", "1F469-1F3FB-200D-1F9BD"},
    {"donna_su_sedia_a_rotelle_manuale_colore2", "1F469-1F3FC-200D-1F9BD"},
    {"donna_su_sedia_a_rotelle_manuale_colore3", "1F469-1F3FD-200D-1F9BD"},
    {"donna_su_sedia_a_rotelle_manuale_colore4", "1F469-1F3FE-200D-1F9BD"},
    {"donna_su_sedia_a_rotelle_manuale_colore5", "1F469-1F3FF-200D-1F9BD"},
    {"donna_su_sedia_a_rotelle_manuale_a_destra", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"donna_su_sedia_a_rotelle_manuale_a_destra_colore1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"donna_su_sedia_a_rotelle_manuale_a_destra_colore2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"donna_su_sedia_a_rotelle_manuale_a_destra_colore3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"donna_su_sedia_a_rotelle_manuale_a_destra_colore4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"donna_su_sedia_a_rotelle_manuale_a_destra_colore5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"pedone_a_destra", "1F6B6-200D-27A1-FE0F"},
    {"pedone_a_destra_colore1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"pedone_a_destra_colore2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"pedone_a_destra_colore3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"pedone_a_destra_colore4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"pedone_a_destra_colore5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"persona_inginocchiata_a_destra", "1F9CE-200D-27A1-FE0F"},
    {"persona_inginocchiata_a_destra_colore1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"persona_inginocchiata_a_destra_colore2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"persona_inginocchiata_a_destra_colore3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"persona_inginocchiata_a_destra_colore4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"persona_inginocchiata_a_destra_colore5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"persona_che_lavora_nella_sanita", "1F9D1-200D-2695-FE0F"},
    {"persona_che_lavora_nella_sanita_colore1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"persona_che_lavora_nella_sanita_colore2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"persona_che_lavora_nella_sanita_colore3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"persona_che_lavora_nella_sanita_colore4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"persona_che_lavora_nella_sanita_colore5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"giudice", "1F9D1-200D-2696-FE0F"},
    {"giudice_colore1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"giudice_colore2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"giudice_colore3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"giudice_colore4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"giudice_colore5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"pilota", "1F9D1-200D-2708-FE0F"},
    {"pilota_colore1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"pilota_colore2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"pilota_colore3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"pilota_colore4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"pilota_colore5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"agricoltore", "1F9D1-200D-1F33E"},
    {"agricoltore_colore1", "1F9D1-1F3FB-200D-1F33E"},
    {"agricoltore_colore2", "1F9D1-1F3FC-200D-1F33E"},
    {"agricoltore_colore3", "1F9D1-1F3FD-200D-1F33E"},
    {"agricoltore_colore4", "1F9D1-1F3FE-200D-1F33E"},
    {"agricoltore_colore5", "1F9D1-1F3FF-200D-1F33E"},
    {"persona_che_cucina", "1F9D1-200D-1F373"},
    {"persona_che_cucina_colore1", "1F9D1-1F3FB-200D-1F373"},
    {"persona_che_cucina_colore2", "1F9D1-1F3FC-200D-1F373"},
    {"persona_che_cucina_colore3", "1F9D1-1F3FD-200D-1F373"},
    {"persona_che_cucina_colore4", "1F9D1-1F3FE-200D-1F373"},
    {"persona_che_cucina_colore5", "1F9D1-1F3FF-200D-1F373"},
    {"persona_che_allatta", "1F9D1-200D-1F37C"},
    {"persona_che_allatta_colore1", "1F9D1-1F3FB-200D-1F37C"},
    {"persona_che_allatta_colore2", "1F9D1-1F3FC-200D-1F37C"},
    {"persona_che_allatta_colore3", "1F9D1-1F3FD-200D-1F37C"},
    {"persona_che_allatta_colore4", "1F9D1-1F3FE-200D-1F37C"},
    {"persona_che_allatta_colore5", "1F9D1-1F3FF-200D-1F37C"},
    {"santa_claus", "1F9D1-200D-1F384"},
    {"santa_claus_colore1", "1F9D1-1F3FB-200D-1F384"},
    {"santa_claus_colore2", "1F9D1-1F3FC-200D-1F384"},
    {"santa_claus_colore3", "1F9D1-1F3FD-200D-1F384"},
    {"santa_claus_colore4", "1F9D1-1F3FE-200D-1F384"},
    {"santa_claus_colore5", "1F9D1-1F3FF-200D-1F384"},
    {"studente", "1F9D1-200D-1F393"},
    {"studente_colore1", "1F9D1-1F3FB-200D-1F393"},
    {"studente_colore2", "1F9D1-1F3FC-200D-1F393"},
    {"studente_colore3", "1F9D1-1F3FD-200D-1F393"},
    {"studente_colore4", "1F9D1-1F3FE-200D-1F393"},
    {"studente_colore5", "1F9D1-1F3FF-200D-1F393"},
    {"cantante", "1F9D1-200D-1F3A4"},
    {"cantante_colore1", "1F9D1-1F3FB-200D-1F3A4"},
    {"cantante_colore2", "1F9D1-1F3FC-200D-1F3A4"},
    {"cantante_colore3", "1F9D1-1F3FD-200D-1F3A4"},
    {"cantante_colore4", "1F9D1-1F3FE-200D-1F3A4"},
    {"cantante_colore5", "1F9D1-1F3FF-200D-1F3A4"},
    {"artista", "1F9D1-200D-1F3A8"},
    {"artista_colore1", "1F9D1-1F3FB-200D-1F3A8"},
    {"artista_colore2", "1F9D1-1F3FC-200D-1F3A8"},
    {"artista_colore3", "1F9D1-1F3FD-200D-1F3A8"},
    {"artista_colore4", "1F9D1-1F3FE-200D-1F3A8"},
    {"artista_colore5", "1F9D1-1F3FF-200D-1F3A8"},
    {"insegnante", "1F9D1-200D-1F3EB"},
    {"insegnante_colore1", "1F9D1-1F3FB-200D-1F3EB"},
    {"insegnante_colore2", "1F9D1-1F3FC-200D-1F3EB"},
    {"insegnante_colore3", "1F9D1-1F3FD-200D-1F3EB"},
    {"insegnante_colore4", "1F9D1-1F3FE-200D-1F3EB"},
    {"insegnante_colore5", "1F9D1-1F3FF-200D-1F3EB"},
    {"persona_che_lavora_in_fabbrica", "1F9D1-200D-1F3ED"},
    {"persona_che_lavora_in_fabbrica_colore1", "1F9D1-1F3FB-200D-1F3ED"},
    {"persona_che_lavora_in_fabbrica_colore2", "1F9D1-1F3FC-200D-1F3ED"},
    {"persona_che_lavora_in_fabbrica_colore3", "1F9D1-1F3FD-200D-1F3ED"},
    {"persona_che_lavora_in_fabbrica_colore4", "1F9D1-1F3FE-200D-1F3ED"},
    {"persona_che_lavora_in_fabbrica_colore5", "1F9D1-1F3FF-200D-1F3ED"},
    {"persona_esperta_di_tecnologia", "1F9D1-200D-1F4BB"},
    {"persona_esperta_di_tecnologia_colore1", "1F9D1-1F3FB-200D-1F4BB"},
    {"persona_esperta_di_tecnologia_colore2", "1F9D1-1F3FC-200D-1F4BB"},
    {"persona_esperta_di_tecnologia_colore3", "1F9D1-1F3FD-200D-1F4BB"},
    {"persona_esperta_di_tecnologia_colore4", "1F9D1-1F3FE-200D-1F4BB"},
    {"persona_esperta_di_tecnologia_colore5", "1F9D1-1F3FF-200D-1F4BB"},
    {"persona_che_fa_un_lavoro_dufficio", "1F9D1-200D-1F4BC"},
    {"persona_che_fa_un_lavoro_dufficio_colore1", "1F9D1-1F3FB-200D-1F4BC"},
    {"persona_che_fa_un_lavoro_dufficio_colore2", "1F9D1-1F3FC-200D-1F4BC"},
    {"persona_che_fa_un_lavoro_dufficio_colore3", "1F9D1-1F3FD-200D-1F4BC"},
    {"persona_che_fa_un_lavoro_dufficio_colore4", "1F9D1-1F3FE-200D-1F4BC"},
    {"persona_che_fa_un_lavoro_dufficio_colore5", "1F9D1-1F3FF-200D-1F4BC"},
    {"meccanico", "1F9D1-200D-1F527"},
    {"meccanico_colore1", "1F9D1-1F3FB-200D-1F527"},
    {"meccanico_colore2", "1F9D1-1F3FC-200D-1F527"},
    {"meccanico_colore3", "1F9D1-1F3FD-200D-1F527"},
    {"meccanico_colore4", "1F9D1-1F3FE-200D-1F527"},
    {"meccanico_colore5", "1F9D1-1F3FF-200D-1F527"},
    {"persona_che_lavora_in_campo_scientifico", "1F9D1-200D-1F52C"},
    {"persona_che_lavora_in_campo_scientifico_colore1", "1F9D1-1F3FB-200D-1F52C"},
    {"persona_che_lavora_in_campo_scientifico_colore2", "1F9D1-1F3FC-200D-1F52C"},
    {"persona_che_lavora_in_campo_scientifico_colore3", "1F9D1-1F3FD-200D-1F52C"},
    {"persona_che_lavora_in_campo_scientifico_colore4", "1F9D1-1F3FE-200D-1F52C"},
    {"persona_che_lavora_in_campo_scientifico_colore5", "1F9D1-1F3FF-200D-1F52C"},
    {"astronauta", "1F9D1-200D-1F680"},
    {"astronauta_colore1", "1F9D1-1F3FB-200D-1F680"},
    {"astronauta_colore2", "1F9D1-1F3FC-200D-1F680"},
    {"astronauta_colore3", "1F9D1-1F3FD-200D-1F680"},
    {"astronauta_colore4", "1F9D1-1F3FE-200D-1F680"},
    {"astronauta_colore5", "1F9D1-1F3FF-200D-1F680"},
    {"pompiere", "1F9D1-200D-1F692"},
    {"pompiere_colore1", "1F9D1-1F3FB-200D-1F692"},
    {"pompiere_colore2", "1F9D1-1F3FC-200D-1F692"},
    {"pompiere_colore3", "1F9D1-1F3FD-200D-1F692"},
    {"pompiere_colore4", "1F9D1-1F3FE-200D-1F692"},
    {"pompiere_colore5", "1F9D1-1F3FF-200D-1F692"},
    {"persona_con_bastone_di_orientamento", "1F9D1-200D-1F9AF"},
    {"persona_con_bastone_di_orientamento_colore1", "1F9D1-1F3FB-200D-1F9AF"},
    {"persona_con_bastone_di_orientamento_colore2", "1F9D1-1F3FC-200D-1F9AF"},
    {"persona_con_bastone_di_orientamento_colore3", "1F9D1-1F3FD-200D-1F9AF"},
    {"persona_con_bastone_di_orientamento_colore4", "1F9D1-1F3FE-200D-1F9AF"},
    {"persona_con_bastone_di_orientamento_colore5", "1F9D1-1F3FF-200D-1F9AF"},
    {"persona_con_bastone_di_orientamento_a_destra", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_bastone_di_orientamento_a_destra_colore1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_bastone_di_orientamento_a_destra_colore2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_bastone_di_orientamento_a_destra_colore3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_bastone_di_orientamento_a_destra_colore4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_con_bastone_di_orientamento_a_destra_colore5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"persona_su_sedia_a_rotelle_motorizzata", "1F9D1-200D-1F9BC"},
    {"persona_su_sedia_a_rotelle_motorizzata_colore1", "1F9D1-1F3FB-200D-1F9BC"},
    {"persona_su_sedia_a_rotelle_motorizzata_colore2", "1F9D1-1F3FC-200D-1F9BC"},
    {"persona_su_sedia_a_rotelle_motorizzata_colore3", "1F9D1-1F3FD-200D-1F9BC"},
    {"persona_su_sedia_a_rotelle_motorizzata_colore4", "1F9D1-1F3FE-200D-1F9BC"},
    {"persona_su_sedia_a_rotelle_motorizzata_colore5", "1F9D1-1F3FF-200D-1F9BC"},
    {"persona_su_sedia_a_rotelle_motorizzata_a_destra", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_su_sedia_a_rotelle_motorizzata_a_destra_colore1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_su_sedia_a_rotelle_motorizzata_a_destra_colore2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_su_sedia_a_rotelle_motorizzata_a_destra_colore3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_su_sedia_a_rotelle_motorizzata_a_destra_colore4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_su_sedia_a_rotelle_motorizzata_a_destra_colore5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"persona_su_sedia_a_rotelle_manuale", "1F9D1-200D-1F9BD"},
    {"persona_su_sedia_a_rotelle_manuale_colore1", "1F9D1-1F3FB-200D-1F9BD"},
    {"persona_su_sedia_a_rotelle_manuale_colore2", "1F9D1-1F3FC-200D-1F9BD"},
    {"persona_su_sedia_a_rotelle_manuale_colore3", "1F9D1-1F3FD-200D-1F9BD"},
    {"persona_su_sedia_a_rotelle_manuale_colore4", "1F9D1-1F3FE-200D-1F9BD"},
    {"persona_su_sedia_a_rotelle_manuale_colore5", "1F9D1-1F3FF-200D-1F9BD"},
    {"persona_su_sedia_a_rotelle_manuale_a_destra", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"persona_su_sedia_a_rotelle_manuale_a_destra_colore1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"persona_su_sedia_a_rotelle_manuale_a_destra_colore2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"persona_su_sedia_a_rotelle_manuale_a_destra_colore3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"persona_su_sedia_a_rotelle_manuale_a_destra_colore4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"persona_su_sedia_a_rotelle_manuale_a_destra_colore5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"donna_che_fa_rimbalzare_una_palla", "26F9-FE0F-200D-2640-FE0F"},
    {"donna_che_fa_rimbalzare_una_palla_colore1", "26F9-1F3FB-200D-2640-FE0F"},
    {"donna_che_fa_rimbalzare_una_palla_colore2", "26F9-1F3FC-200D-2640-FE0F"},
    {"donna_che_fa_rimbalzare_una_palla_colore3", "26F9-1F3FD-200D-2640-FE0F"},
    {"donna_che_fa_rimbalzare_una_palla_colore4", "26F9-1F3FE-200D-2640-FE0F"},
    {"donna_che_fa_rimbalzare_una_palla_colore5", "26F9-1F3FF-200D-2640-FE0F"},
    {"uomo_che_fa_rimbalzare_una_palla", "26F9-FE0F-200D-2642-FE0F"},
    {"uomo_che_fa_rimbalzare_una_palla_colore1", "26F9-1F3FB-200D-2642-FE0F"},
    {"uomo_che_fa_rimbalzare_una_palla_colore2", "26F9-1F3FC-200D-2642-FE0F"},
    {"uomo_che_fa_rimbalzare_una_palla_colore3", "26F9-1F3FD-200D-2642-FE0F"},
    {"uomo_che_fa_rimbalzare_una_palla_colore4", "26F9-1F3FE-200D-2642-FE0F"},
    {"uomo_che_fa_rimbalzare_una_palla_colore5", "26F9-1F3FF-200D-2642-FE0F"},
    {"donna_che_corre", "1F3C3-200D-2640-FE0F"},
    {"donna_che_corre_colore1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"donna_che_corre_colore2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"donna_che_corre_colore3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"donna_che_corre_colore4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"donna_che_corre_colore5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"donna_che_corre_a_destra", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_che_corre_a_destra_colore1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_che_corre_a_destra_colore2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_che_corre_a_destra_colore3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_che_corre_a_destra_colore4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_che_corre_a_destra_colore5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"uomo_che_corre", "1F3C3-200D-2642-FE0F"},
    {"uomo_che_corre_colore1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"uomo_che_corre_colore2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"uomo_che_corre_colore3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"uomo_che_corre_colore4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"uomo_che_corre_colore5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"uomo_che_corre_a_destra", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_che_corre_a_destra_colore1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_che_corre_a_destra_colore2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_che_corre_a_destra_colore3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_che_corre_a_destra_colore4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_che_corre_a_destra_colore5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"surfista_donna", "1F3C4-200D-2640-FE0F"},
    {"surfista_donna_colore1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"surfista_donna_colore2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"surfista_donna_colore3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"surfista_donna_colore4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"surfista_donna_colore5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"surfista_uomo", "1F3C4-200D-2642-FE0F"},
    {"surfista_uomo_colore1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"surfista_uomo_colore2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"surfista_uomo_colore3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"surfista_uomo_colore4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"surfista_uomo_colore5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"nuotatrice", "1F3CA-200D-2640-FE0F"},
    {"nuotatrice_colore1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"nuotatrice_colore2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"nuotatrice_colore3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"nuotatrice_colore4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"nuotatrice_colore5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"nuotatore", "1F3CA-200D-2642-FE0F"},
    {"nuotatore_colore1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"nuotatore_colore2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"nuotatore_colore3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"nuotatore_colore4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"nuotatore_colore5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"donna_che_solleva_pesi", "1F3CB-FE0F-200D-2640-FE0F"},
    {"donna_che_solleva_pesi_colore1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"donna_che_solleva_pesi_colore2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"donna_che_solleva_pesi_colore3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"donna_che_solleva_pesi_colore4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"donna_che_solleva_pesi_colore5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"uomo_che_solleva_pesi", "1F3CB-FE0F-200D-2642-FE0F"},
    {"uomo_che_solleva_pesi_colore1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"uomo_che_solleva_pesi_colore2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"uomo_che_solleva_pesi_colore3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"uomo_che_solleva_pesi_colore4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"uomo_che_solleva_pesi_colore5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"golfista_donna", "1F3CC-FE0F-200D-2640-FE0F"},
    {"golfista_donna_colore1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"golfista_donna_colore2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"golfista_donna_colore3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"golfista_donna_colore4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"golfista_donna_colore5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"golfista_uomo", "1F3CC-FE0F-200D-2642-FE0F"},
    {"golfista_uomo_colore1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"golfista_uomo_colore2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"golfista_uomo_colore3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"golfista_uomo_colore4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"golfista_uomo_colore5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"poliziotta", "1F46E-200D-2640-FE0F"},
    {"poliziotta_colore1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"poliziotta_colore2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"poliziotta_colore3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"poliziotta_colore4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"poliziotta_colore5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"poliziotto", "1F46E-200D-2642-FE0F"},
    {"poliziotto_colore1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"poliziotto_colore2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"poliziotto_colore3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"poliziotto_colore4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"poliziotto_colore5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"donne_con_orecchie_da_coniglio", "1F46F-200D-2640-FE0F"},
    {"donne_con_orecchie_da_coniglio_colore1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"donne_con_orecchie_da_coniglio_colore1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"donne_con_orecchie_da_coniglio_colore1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"donne_con_orecchie_da_coniglio_colore1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"donne_con_orecchie_da_coniglio_colore2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"donne_con_orecchie_da_coniglio_colore2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"donne_con_orecchie_da_coniglio_colore2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"donne_con_orecchie_da_coniglio_colore2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"donne_con_orecchie_da_coniglio_colore3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"donne_con_orecchie_da_coniglio_colore3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"donne_con_orecchie_da_coniglio_colore3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"donne_con_orecchie_da_coniglio_colore3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"donne_con_orecchie_da_coniglio_colore4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"donne_con_orecchie_da_coniglio_colore4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"donne_con_orecchie_da_coniglio_colore4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"donne_con_orecchie_da_coniglio_colore4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"donne_con_orecchie_da_coniglio_colore5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"donne_con_orecchie_da_coniglio_colore5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"donne_con_orecchie_da_coniglio_colore5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"donne_con_orecchie_da_coniglio_colore5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"donne_con_orecchie_da_coniglio_colore1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"donne_con_orecchie_da_coniglio_colore2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"donne_con_orecchie_da_coniglio_colore3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"donne_con_orecchie_da_coniglio_colore4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"donne_con_orecchie_da_coniglio_colore5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"uomini_con_orecchie_da_coniglio", "1F46F-200D-2642-FE0F"},
    {"uomini_con_orecchie_da_coniglio_colore1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"uomini_con_orecchie_da_coniglio_colore1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"uomini_con_orecchie_da_coniglio_colore1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"uomini_con_orecchie_da_coniglio_colore1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"uomini_con_orecchie_da_coniglio_colore2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"uomini_con_orecchie_da_coniglio_colore2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"uomini_con_orecchie_da_coniglio_colore2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"uomini_con_orecchie_da_coniglio_colore2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"uomini_con_orecchie_da_coniglio_colore3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"uomini_con_orecchie_da_coniglio_colore3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"uomini_con_orecchie_da_coniglio_colore3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"uomini_con_orecchie_da_coniglio_colore3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"uomini_con_orecchie_da_coniglio_colore4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"uomini_con_orecchie_da_coniglio_colore4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"uomini_con_orecchie_da_coniglio_colore4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"uomini_con_orecchie_da_coniglio_colore4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"uomini_con_orecchie_da_coniglio_colore5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"uomini_con_orecchie_da_coniglio_colore5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"uomini_con_orecchie_da_coniglio_colore5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"uomini_con_orecchie_da_coniglio_colore5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"uomini_con_orecchie_da_coniglio_colore1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"uomini_con_orecchie_da_coniglio_colore2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"uomini_con_orecchie_da_coniglio_colore3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"uomini_con_orecchie_da_coniglio_colore4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"uomini_con_orecchie_da_coniglio_colore5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"sposa_con_velo", "1F470-200D-2640-FE0F"},
    {"sposa_con_velo_colore1", "1F470-1F3FB-200D-2640-FE0F"},
    {"sposa_con_velo_colore2", "1F470-1F3FC-200D-2640-FE0F"},
    {"sposa_con_velo_colore3", "1F470-1F3FD-200D-2640-FE0F"},
    {"sposa_con_velo_colore4", "1F470-1F3FE-200D-2640-FE0F"},
    {"sposa_con_velo_colore5", "1F470-1F3FF-200D-2640-FE0F"},
    {"sposo_con_velo", "1F470-200D-2642-FE0F"},
    {"sposo_con_velo_colore1", "1F470-1F3FB-200D-2642-FE0F"},
    {"sposo_con_velo_colore2", "1F470-1F3FC-200D-2642-FE0F"},
    {"sposo_con_velo_colore3", "1F470-1F3FD-200D-2642-FE0F"},
    {"sposo_con_velo_colore4", "1F470-1F3FE-200D-2642-FE0F"},
    {"sposo_con_velo_colore5", "1F470-1F3FF-200D-2642-FE0F"},
    {"donna_bionda", "1F471-200D-2640-FE0F"},
    {"donna_bionda_colore1", "1F471-1F3FB-200D-2640-FE0F"},
    {"donna_bionda_colore2", "1F471-1F3FC-200D-2640-FE0F"},
    {"donna_bionda_colore3", "1F471-1F3FD-200D-2640-FE0F"},
    {"donna_bionda_colore4", "1F471-1F3FE-200D-2640-FE0F"},
    {"donna_bionda_colore5", "1F471-1F3FF-200D-2640-FE0F"},
    {"uomo_biondo", "1F471-200D-2642-FE0F"},
    {"uomo_biondo_colore1", "1F471-1F3FB-200D-2642-FE0F"},
    {"uomo_biondo_colore2", "1F471-1F3FC-200D-2642-FE0F"},
    {"uomo_biondo_colore3", "1F471-1F3FD-200D-2642-FE0F"},
    {"uomo_biondo_colore4", "1F471-1F3FE-200D-2642-FE0F"},
    {"uomo_biondo_colore5", "1F471-1F3FF-200D-2642-FE0F"},
    {"donna_con_turbante", "1F473-200D-2640-FE0F"},
    {"donna_con_turbante_colore1", "1F473-1F3FB-200D-2640-FE0F"},
    {"donna_con_turbante_colore2", "1F473-1F3FC-200D-2640-FE0F"},
    {"donna_con_turbante_colore3", "1F473-1F3FD-200D-2640-FE0F"},
    {"donna_con_turbante_colore4", "1F473-1F3FE-200D-2640-FE0F"},
    {"donna_con_turbante_colore5", "1F473-1F3FF-200D-2640-FE0F"},
    {"uomo_con_turbante", "1F473-200D-2642-FE0F"},
    {"uomo_con_turbante_colore1", "1F473-1F3FB-200D-2642-FE0F"},
    {"uomo_con_turbante_colore2", "1F473-1F3FC-200D-2642-FE0F"},
    {"uomo_con_turbante_colore3", "1F473-1F3FD-200D-2642-FE0F"},
    {"uomo_con_turbante_colore4", "1F473-1F3FE-200D-2642-FE0F"},
    {"uomo_con_turbante_colore5", "1F473-1F3FF-200D-2642-FE0F"},
    {"operaia_edile", "1F477-200D-2640-FE0F"},
    {"operaia_edile_colore1", "1F477-1F3FB-200D-2640-FE0F"},
    {"operaia_edile_colore2", "1F477-1F3FC-200D-2640-FE0F"},
    {"operaia_edile_colore3", "1F477-1F3FD-200D-2640-FE0F"},
    {"operaia_edile_colore4", "1F477-1F3FE-200D-2640-FE0F"},
    {"operaia_edile_colore5", "1F477-1F3FF-200D-2640-FE0F"},
    {"operaio_edile_uomo", "1F477-200D-2642-FE0F"},
    {"operaio_edile_uomo_colore1", "1F477-1F3FB-200D-2642-FE0F"},
    {"operaio_edile_uomo_colore2", "1F477-1F3FC-200D-2642-FE0F"},
    {"operaio_edile_uomo_colore3", "1F477-1F3FD-200D-2642-FE0F"},
    {"operaio_edile_uomo_colore4", "1F477-1F3FE-200D-2642-FE0F"},
    {"operaio_edile_uomo_colore5", "1F477-1F3FF-200D-2642-FE0F"},
    {"donna_con_mano_capovolta", "1F481-200D-2640-FE0F"},
    {"donna_con_mano_capovolta_colore1", "1F481-1F3FB-200D-2640-FE0F"},
    {"donna_con_mano_capovolta_colore2", "1F481-1F3FC-200D-2640-FE0F"},
    {"donna_con_mano_capovolta_colore3", "1F481-1F3FD-200D-2640-FE0F"},
    {"donna_con_mano_capovolta_colore4", "1F481-1F3FE-200D-2640-FE0F"},
    {"donna_con_mano_capovolta_colore5", "1F481-1F3FF-200D-2640-FE0F"},
    {"uomo_con_mano_capovolta", "1F481-200D-2642-FE0F"},
    {"uomo_con_mano_capovolta_colore1", "1F481-1F3FB-200D-2642-FE0F"},
    {"uomo_con_mano_capovolta_colore2", "1F481-1F3FC-200D-2642-FE0F"},
    {"uomo_con_mano_capovolta_colore3", "1F481-1F3FD-200D-2642-FE0F"},
    {"uomo_con_mano_capovolta_colore4", "1F481-1F3FE-200D-2642-FE0F"},
    {"uomo_con_mano_capovolta_colore5", "1F481-1F3FF-200D-2642-FE0F"},
    {"guardia_donna", "1F482-200D-2640-FE0F"},
    {"guardia_donna_colore1", "1F482-1F3FB-200D-2640-FE0F"},
    {"guardia_donna_colore2", "1F482-1F3FC-200D-2640-FE0F"},
    {"guardia_donna_colore3", "1F482-1F3FD-200D-2640-FE0F"},
    {"guardia_donna_colore4", "1F482-1F3FE-200D-2640-FE0F"},
    {"guardia_donna_colore5", "1F482-1F3FF-200D-2640-FE0F"},
    {"guardia_uomo", "1F482-200D-2642-FE0F"},
    {"guardia_uomo_colore1", "1F482-1F3FB-200D-2642-FE0F"},
    {"guardia_uomo_colore2", "1F482-1F3FC-200D-2642-FE0F"},
    {"guardia_uomo_colore3", "1F482-1F3FD-200D-2642-FE0F"},
    {"guardia_uomo_colore4", "1F482-1F3FE-200D-2642-FE0F"},
    {"guardia_uomo_colore5", "1F482-1F3FF-200D-2642-FE0F"},
    {"donna_che_riceve_un_massaggio", "1F486-200D-2640-FE0F"},
    {"donna_che_riceve_un_massaggio_colore1", "1F486-1F3FB-200D-2640-FE0F"},
    {"donna_che_riceve_un_massaggio_colore2", "1F486-1F3FC-200D-2640-FE0F"},
    {"donna_che_riceve_un_massaggio_colore3", "1F486-1F3FD-200D-2640-FE0F"},
    {"donna_che_riceve_un_massaggio_colore4", "1F486-1F3FE-200D-2640-FE0F"},
    {"donna_che_riceve_un_massaggio_colore5", "1F486-1F3FF-200D-2640-FE0F"},
    {"uomo_che_riceve_un_massaggio", "1F486-200D-2642-FE0F"},
    {"uomo_che_riceve_un_massaggio_colore1", "1F486-1F3FB-200D-2642-FE0F"},
    {"uomo_che_riceve_un_massaggio_colore2", "1F486-1F3FC-200D-2642-FE0F"},
    {"uomo_che_riceve_un_massaggio_colore3", "1F486-1F3FD-200D-2642-FE0F"},
    {"uomo_che_riceve_un_massaggio_colore4", "1F486-1F3FE-200D-2642-FE0F"},
    {"uomo_che_riceve_un_massaggio_colore5", "1F486-1F3FF-200D-2642-FE0F"},
    {"taglio_di_capelli_per_donna", "1F487-200D-2640-FE0F"},
    {"taglio_di_capelli_per_donna_colore1", "1F487-1F3FB-200D-2640-FE0F"},
    {"taglio_di_capelli_per_donna_colore2", "1F487-1F3FC-200D-2640-FE0F"},
    {"taglio_di_capelli_per_donna_colore3", "1F487-1F3FD-200D-2640-FE0F"},
    {"taglio_di_capelli_per_donna_colore4", "1F487-1F3FE-200D-2640-FE0F"},
    {"taglio_di_capelli_per_donna_colore5", "1F487-1F3FF-200D-2640-FE0F"},
    {"taglio_di_capelli_per_uomo", "1F487-200D-2642-FE0F"},
    {"taglio_di_capelli_per_uomo_colore1", "1F487-1F3FB-200D-2642-FE0F"},
    {"taglio_di_capelli_per_uomo_colore2", "1F487-1F3FC-200D-2642-FE0F"},
    {"taglio_di_capelli_per_uomo_colore3", "1F487-1F3FD-200D-2642-FE0F"},
    {"taglio_di_capelli_per_uomo_colore4", "1F487-1F3FE-200D-2642-FE0F"},
    {"taglio_di_capelli_per_uomo_colore5", "1F487-1F3FF-200D-2642-FE0F"},
    {"investigatrice", "1F575-FE0F-200D-2640-FE0F"},
    {"investigatrice_colore1", "1F575-1F3FB-200D-2640-FE0F"},
    {"investigatrice_colore2", "1F575-1F3FC-200D-2640-FE0F"},
    {"investigatrice_colore3", "1F575-1F3FD-200D-2640-FE0F"},
    {"investigatrice_colore4", "1F575-1F3FE-200D-2640-FE0F"},
    {"investigatrice_colore5", "1F575-1F3FF-200D-2640-FE0F"},
    {"investigatore", "1F575-FE0F-200D-2642-FE0F"},
    {"investigatore_colore1", "1F575-1F3FB-200D-2642-FE0F"},
    {"investigatore_colore2", "1F575-1F3FC-200D-2642-FE0F"},
    {"investigatore_colore3", "1F575-1F3FD-200D-2642-FE0F"},
    {"investigatore_colore4", "1F575-1F3FE-200D-2642-FE0F"},
    {"investigatore_colore5", "1F575-1F3FF-200D-2642-FE0F"},
    {"donna_con_gesto_di_rifiuto", "1F645-200D-2640-FE0F"},
    {"donna_con_gesto_di_rifiuto_colore1", "1F645-1F3FB-200D-2640-FE0F"},
    {"donna_con_gesto_di_rifiuto_colore2", "1F645-1F3FC-200D-2640-FE0F"},
    {"donna_con_gesto_di_rifiuto_colore3", "1F645-1F3FD-200D-2640-FE0F"},
    {"donna_con_gesto_di_rifiuto_colore4", "1F645-1F3FE-200D-2640-FE0F"},
    {"donna_con_gesto_di_rifiuto_colore5", "1F645-1F3FF-200D-2640-FE0F"},
    {"uomo_con_gesto_di_rifiuto", "1F645-200D-2642-FE0F"},
    {"uomo_con_gesto_di_rifiuto_colore1", "1F645-1F3FB-200D-2642-FE0F"},
    {"uomo_con_gesto_di_rifiuto_colore2", "1F645-1F3FC-200D-2642-FE0F"},
    {"uomo_con_gesto_di_rifiuto_colore3", "1F645-1F3FD-200D-2642-FE0F"},
    {"uomo_con_gesto_di_rifiuto_colore4", "1F645-1F3FE-200D-2642-FE0F"},
    {"uomo_con_gesto_di_rifiuto_colore5", "1F645-1F3FF-200D-2642-FE0F"},
    {"donna_con_gesto_ok", "1F646-200D-2640-FE0F"},
    {"donna_con_gesto_ok_colore1", "1F646-1F3FB-200D-2640-FE0F"},
    {"donna_con_gesto_ok_colore2", "1F646-1F3FC-200D-2640-FE0F"},
    {"donna_con_gesto_ok_colore3", "1F646-1F3FD-200D-2640-FE0F"},
    {"donna_con_gesto_ok_colore4", "1F646-1F3FE-200D-2640-FE0F"},
    {"donna_con_gesto_ok_colore5", "1F646-1F3FF-200D-2640-FE0F"},
    {"uomo_con_gesto_ok", "1F646-200D-2642-FE0F"},
    {"uomo_con_gesto_ok_colore1", "1F646-1F3FB-200D-2642-FE0F"},
    {"uomo_con_gesto_ok_colore2", "1F646-1F3FC-200D-2642-FE0F"},
    {"uomo_con_gesto_ok_colore3", "1F646-1F3FD-200D-2642-FE0F"},
    {"uomo_con_gesto_ok_colore4", "1F646-1F3FE-200D-2642-FE0F"},
    {"uomo_con_gesto_ok_colore5", "1F646-1F3FF-200D-2642-FE0F"},
    {"donna_che_fa_inchino_profondo", "1F647-200D-2640-FE0F"},
    {"donna_che_fa_inchino_profondo_colore1", "1F647-1F3FB-200D-2640-FE0F"},
    {"donna_che_fa_inchino_profondo_colore2", "1F647-1F3FC-200D-2640-FE0F"},
    {"donna_che_fa_inchino_profondo_colore3", "1F647-1F3FD-200D-2640-FE0F"},
    {"donna_che_fa_inchino_profondo_colore4", "1F647-1F3FE-200D-2640-FE0F"},
    {"donna_che_fa_inchino_profondo_colore5", "1F647-1F3FF-200D-2640-FE0F"},
    {"uomo_che_fa_inchino_profondo", "1F647-200D-2642-FE0F"},
    {"uomo_che_fa_inchino_profondo_colore1", "1F647-1F3FB-200D-2642-FE0F"},
    {"uomo_che_fa_inchino_profondo_colore2", "1F647-1F3FC-200D-2642-FE0F"},
    {"uomo_che_fa_inchino_profondo_colore3", "1F647-1F3FD-200D-2642-FE0F"},
    {"uomo_che_fa_inchino_profondo_colore4", "1F647-1F3FE-200D-2642-FE0F"},
    {"uomo_che_fa_inchino_profondo_colore5", "1F647-1F3FF-200D-2642-FE0F"},
    {"donna_con_mano_alzata", "1F64B-200D-2640-FE0F"},
    {"donna_con_mano_alzata_colore1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"donna_con_mano_alzata_colore2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"donna_con_mano_alzata_colore3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"donna_con_mano_alzata_colore4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"donna_con_mano_alzata_colore5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"uomo_con_mano_alzata", "1F64B-200D-2642-FE0F"},
    {"uomo_con_mano_alzata_colore1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"uomo_con_mano_alzata_colore2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"uomo_con_mano_alzata_colore3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"uomo_con_mano_alzata_colore4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"uomo_con_mano_alzata_colore5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"donna_corrucciata", "1F64D-200D-2640-FE0F"},
    {"donna_corrucciata_colore1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"donna_corrucciata_colore2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"donna_corrucciata_colore3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"donna_corrucciata_colore4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"donna_corrucciata_colore5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"uomo_corrucciato", "1F64D-200D-2642-FE0F"},
    {"uomo_corrucciato_colore1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"uomo_corrucciato_colore2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"uomo_corrucciato_colore3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"uomo_corrucciato_colore4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"uomo_corrucciato_colore5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"donna_imbronciata", "1F64E-200D-2640-FE0F"},
    {"donna_imbronciata_colore1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"donna_imbronciata_colore2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"donna_imbronciata_colore3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"donna_imbronciata_colore4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"donna_imbronciata_colore5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"uomo_imbronciato", "1F64E-200D-2642-FE0F"},
    {"uomo_imbronciato_colore1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"uomo_imbronciato_colore2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"uomo_imbronciato_colore3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"uomo_imbronciato_colore4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"uomo_imbronciato_colore5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"donna_in_barca_a_remi", "1F6A3-200D-2640-FE0F"},
    {"donna_in_barca_a_remi_colore1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"donna_in_barca_a_remi_colore2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"donna_in_barca_a_remi_colore3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"donna_in_barca_a_remi_colore4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"donna_in_barca_a_remi_colore5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"uomo_in_barca_a_remi", "1F6A3-200D-2642-FE0F"},
    {"uomo_in_barca_a_remi_colore1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"uomo_in_barca_a_remi_colore2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"uomo_in_barca_a_remi_colore3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"uomo_in_barca_a_remi_colore4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"uomo_in_barca_a_remi_colore5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"ciclista_donna", "1F6B4-200D-2640-FE0F"},
    {"ciclista_donna_colore1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"ciclista_donna_colore2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"ciclista_donna_colore3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"ciclista_donna_colore4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"ciclista_donna_colore5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"ciclista_uomo", "1F6B4-200D-2642-FE0F"},
    {"ciclista_uomo_colore1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"ciclista_uomo_colore2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"ciclista_uomo_colore3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"ciclista_uomo_colore4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"ciclista_uomo_colore5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"ciclista_donna_di_mountain_bike", "1F6B5-200D-2640-FE0F"},
    {"ciclista_donna_di_mountain_bike_colore1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"ciclista_donna_di_mountain_bike_colore2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"ciclista_donna_di_mountain_bike_colore3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"ciclista_donna_di_mountain_bike_colore4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"ciclista_donna_di_mountain_bike_colore5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"ciclista_uomo_di_mountain_bike", "1F6B5-200D-2642-FE0F"},
    {"ciclista_uomo_di_mountain_bike_colore1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"ciclista_uomo_di_mountain_bike_colore2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"ciclista_uomo_di_mountain_bike_colore3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"ciclista_uomo_di_mountain_bike_colore4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"ciclista_uomo_di_mountain_bike_colore5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"donna_che_cammina", "1F6B6-200D-2640-FE0F"},
    {"donna_che_cammina_colore1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"donna_che_cammina_colore2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"donna_che_cammina_colore3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"donna_che_cammina_colore4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"donna_che_cammina_colore5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"donna_che_cammina_a_destra", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_che_cammina_a_destra_colore1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_che_cammina_a_destra_colore2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_che_cammina_a_destra_colore3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_che_cammina_a_destra_colore4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_che_cammina_a_destra_colore5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"uomo_che_cammina", "1F6B6-200D-2642-FE0F"},
    {"uomo_che_cammina_colore1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"uomo_che_cammina_colore2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"uomo_che_cammina_colore3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"uomo_che_cammina_colore4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"uomo_che_cammina_colore5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"uomo_che_cammina_a_destra", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_che_cammina_a_destra_colore1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_che_cammina_a_destra_colore2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_che_cammina_a_destra_colore3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_che_cammina_a_destra_colore4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_che_cammina_a_destra_colore5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"donna_esasperata", "1F926-200D-2640-FE0F"},
    {"donna_esasperata_colore1", "1F926-1F3FB-200D-2640-FE0F"},
    {"donna_esasperata_colore2", "1F926-1F3FC-200D-2640-FE0F"},
    {"donna_esasperata_colore3", "1F926-1F3FD-200D-2640-FE0F"},
    {"donna_esasperata_colore4", "1F926-1F3FE-200D-2640-FE0F"},
    {"donna_esasperata_colore5", "1F926-1F3FF-200D-2640-FE0F"},
    {"uomo_esasperato", "1F926-200D-2642-FE0F"},
    {"uomo_esasperato_colore1", "1F926-1F3FB-200D-2642-FE0F"},
    {"uomo_esasperato_colore2", "1F926-1F3FC-200D-2642-FE0F"},
    {"uomo_esasperato_colore3", "1F926-1F3FD-200D-2642-FE0F"},
    {"uomo_esasperato_colore4", "1F926-1F3FE-200D-2642-FE0F"},
    {"uomo_esasperato_colore5", "1F926-1F3FF-200D-2642-FE0F"},
    {"donna_in_smoking", "1F935-200D-2640-FE0F"},
    {"donna_in_smoking_colore1", "1F935-1F3FB-200D-2640-FE0F"},
    {"donna_in_smoking_colore2", "1F935-1F3FC-200D-2640-FE0F"},
    {"donna_in_smoking_colore3", "1F935-1F3FD-200D-2640-FE0F"},
    {"donna_in_smoking_colore4", "1F935-1F3FE-200D-2640-FE0F"},
    {"donna_in_smoking_colore5", "1F935-1F3FF-200D-2640-FE0F"},
    {"uomo_in_smoking", "1F935-200D-2642-FE0F"},
    {"uomo_in_smoking_colore1", "1F935-1F3FB-200D-2642-FE0F"},
    {"uomo_in_smoking_colore2", "1F935-1F3FC-200D-2642-FE0F"},
    {"uomo_in_smoking_colore3", "1F935-1F3FD-200D-2642-FE0F"},
    {"uomo_in_smoking_colore4", "1F935-1F3FE-200D-2642-FE0F"},
    {"uomo_in_smoking_colore5", "1F935-1F3FF-200D-2642-FE0F"},
    {"donna_che_scrolla_le_spalle", "1F937-200D-2640-FE0F"},
    {"donna_che_scrolla_le_spalle_colore1", "1F937-1F3FB-200D-2640-FE0F"},
    {"donna_che_scrolla_le_spalle_colore2", "1F937-1F3FC-200D-2640-FE0F"},
    {"donna_che_scrolla_le_spalle_colore3", "1F937-1F3FD-200D-2640-FE0F"},
    {"donna_che_scrolla_le_spalle_colore4", "1F937-1F3FE-200D-2640-FE0F"},
    {"donna_che_scrolla_le_spalle_colore5", "1F937-1F3FF-200D-2640-FE0F"},
    {"uomo_che_scrolla_le_spalle", "1F937-200D-2642-FE0F"},
    {"uomo_che_scrolla_le_spalle_colore1", "1F937-1F3FB-200D-2642-FE0F"},
    {"uomo_che_scrolla_le_spalle_colore2", "1F937-1F3FC-200D-2642-FE0F"},
    {"uomo_che_scrolla_le_spalle_colore3", "1F937-1F3FD-200D-2642-FE0F"},
    {"uomo_che_scrolla_le_spalle_colore4", "1F937-1F3FE-200D-2642-FE0F"},
    {"uomo_che_scrolla_le_spalle_colore5", "1F937-1F3FF-200D-2642-FE0F"},
    {"donna_che_fa_la_ruota", "1F938-200D-2640-FE0F"},
    {"donna_che_fa_la_ruota_colore1", "1F938-1F3FB-200D-2640-FE0F"},
    {"donna_che_fa_la_ruota_colore2", "1F938-1F3FC-200D-2640-FE0F"},
    {"donna_che_fa_la_ruota_colore3", "1F938-1F3FD-200D-2640-FE0F"},
    {"donna_che_fa_la_ruota_colore4", "1F938-1F3FE-200D-2640-FE0F"},
    {"donna_che_fa_la_ruota_colore5", "1F938-1F3FF-200D-2640-FE0F"},
    {"uomo_che_fa_la_ruota", "1F938-200D-2642-FE0F"},
    {"uomo_che_fa_la_ruota_colore1", "1F938-1F3FB-200D-2642-FE0F"},
    {"uomo_che_fa_la_ruota_colore2", "1F938-1F3FC-200D-2642-FE0F"},
    {"uomo_che_fa_la_ruota_colore3", "1F938-1F3FD-200D-2642-FE0F"},
    {"uomo_che_fa_la_ruota_colore4", "1F938-1F3FE-200D-2642-FE0F"},
    {"uomo_che_fa_la_ruota_colore5", "1F938-1F3FF-200D-2642-FE0F"},
    {"giocoliere_donna", "1F939-200D-2640-FE0F"},
    {"giocoliere_donna_colore1", "1F939-1F3FB-200D-2640-FE0F"},
    {"giocoliere_donna_colore2", "1F939-1F3FC-200D-2640-FE0F"},
    {"giocoliere_donna_colore3", "1F939-1F3FD-200D-2640-FE0F"},
    {"giocoliere_donna_colore4", "1F939-1F3FE-200D-2640-FE0F"},
    {"giocoliere_donna_colore5", "1F939-1F3FF-200D-2640-FE0F"},
    {"giocoliere_uomo", "1F939-200D-2642-FE0F"},
    {"giocoliere_uomo_colore1", "1F939-1F3FB-200D-2642-FE0F"},
    {"giocoliere_uomo_colore2", "1F939-1F3FC-200D-2642-FE0F"},
    {"giocoliere_uomo_colore3", "1F939-1F3FD-200D-2642-FE0F"},
    {"giocoliere_uomo_colore4", "1F939-1F3FE-200D-2642-FE0F"},
    {"giocoliere_uomo_colore5", "1F939-1F3FF-200D-2642-FE0F"},
    {"lottatrici", "1F93C-200D-2640-FE0F"},
    {"lottatrici_colore1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"lottatrici_colore1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"lottatrici_colore1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"lottatrici_colore1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"lottatrici_colore2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"lottatrici_colore2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"lottatrici_colore2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"lottatrici_colore2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"lottatrici_colore3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"lottatrici_colore3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"lottatrici_colore3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"lottatrici_colore3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"lottatrici_colore4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"lottatrici_colore4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"lottatrici_colore4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"lottatrici_colore4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"lottatrici_colore5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"lottatrici_colore5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"lottatrici_colore5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"lottatrici_colore5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"lottatrici_colore1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"lottatrici_colore2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"lottatrici_colore3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"lottatrici_colore4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"lottatrici_colore5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"lottatori", "1F93C-200D-2642-FE0F"},
    {"lottatori_colore1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"lottatori_colore1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"lottatori_colore1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"lottatori_colore1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"lottatori_colore2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"lottatori_colore2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"lottatori_colore2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"lottatori_colore2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"lottatori_colore3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"lottatori_colore3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"lottatori_colore3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"lottatori_colore3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"lottatori_colore4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"lottatori_colore4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"lottatori_colore4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"lottatori_colore4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"lottatori_colore5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"lottatori_colore5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"lottatori_colore5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"lottatori_colore5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"lottatori_colore1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"lottatori_colore2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"lottatori_colore3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"lottatori_colore4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"lottatori_colore5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"pallanuotista_donna", "1F93D-200D-2640-FE0F"},
    {"pallanuotista_donna_colore1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"pallanuotista_donna_colore2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"pallanuotista_donna_colore3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"pallanuotista_donna_colore4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"pallanuotista_donna_colore5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"pallanuotista_uomo", "1F93D-200D-2642-FE0F"},
    {"pallanuotista_uomo_colore1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"pallanuotista_uomo_colore2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"pallanuotista_uomo_colore3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"pallanuotista_uomo_colore4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"pallanuotista_uomo_colore5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"pallamanista_donna", "1F93E-200D-2640-FE0F"},
    {"pallamanista_donna_colore1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"pallamanista_donna_colore2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"pallamanista_donna_colore3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"pallamanista_donna_colore4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"pallamanista_donna_colore5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"pallamanista_uomo", "1F93E-200D-2642-FE0F"},
    {"pallamanista_uomo_colore1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"pallamanista_uomo_colore2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"pallamanista_uomo_colore3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"pallamanista_uomo_colore4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"pallamanista_uomo_colore5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"supereroina", "1F9B8-200D-2640-FE0F"},
    {"supereroina_colore1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"supereroina_colore2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"supereroina_colore3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"supereroina_colore4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"supereroina_colore5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"supereroe_uomo", "1F9B8-200D-2642-FE0F"},
    {"supereroe_uomo_colore1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"supereroe_uomo_colore2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"supereroe_uomo_colore3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"supereroe_uomo_colore4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"supereroe_uomo_colore5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"supercattiva", "1F9B9-200D-2640-FE0F"},
    {"supercattiva_colore1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"supercattiva_colore2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"supercattiva_colore3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"supercattiva_colore4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"supercattiva_colore5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"supercattivo_uomo", "1F9B9-200D-2642-FE0F"},
    {"supercattivo_uomo_colore1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"supercattivo_uomo_colore2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"supercattivo_uomo_colore3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"supercattivo_uomo_colore4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"supercattivo_uomo_colore5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"donna_in_piedi", "1F9CD-200D-2640-FE0F"},
    {"donna_in_piedi_colore1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"donna_in_piedi_colore2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"donna_in_piedi_colore3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"donna_in_piedi_colore4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"donna_in_piedi_colore5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"uomo_in_piedi", "1F9CD-200D-2642-FE0F"},
    {"uomo_in_piedi_colore1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"uomo_in_piedi_colore2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"uomo_in_piedi_colore3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"uomo_in_piedi_colore4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"uomo_in_piedi_colore5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"donna_inginocchiata", "1F9CE-200D-2640-FE0F"},
    {"donna_inginocchiata_colore1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"donna_inginocchiata_colore2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"donna_inginocchiata_colore3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"donna_inginocchiata_colore4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"donna_inginocchiata_colore5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"donna_inginocchiata_a_destra", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_inginocchiata_a_destra_colore1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_inginocchiata_a_destra_colore2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_inginocchiata_a_destra_colore3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_inginocchiata_a_destra_colore4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"donna_inginocchiata_a_destra_colore5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"uomo_inginocchiato", "1F9CE-200D-2642-FE0F"},
    {"uomo_inginocchiato_colore1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"uomo_inginocchiato_colore2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"uomo_inginocchiato_colore3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"uomo_inginocchiato_colore4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"uomo_inginocchiato_colore5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"uomo_inginocchiato_a_destra", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_inginocchiato_a_destra_colore1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_inginocchiato_a_destra_colore2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_inginocchiato_a_destra_colore3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_inginocchiato_a_destra_colore4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"uomo_inginocchiato_a_destra_colore5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"donna_con_problemi_di_udito", "1F9CF-200D-2640-FE0F"},
    {"donna_con_problemi_di_udito_colore1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"donna_con_problemi_di_udito_colore2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"donna_con_problemi_di_udito_colore3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"donna_con_problemi_di_udito_colore4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"donna_con_problemi_di_udito_colore5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"uomo_con_problemi_di_udito", "1F9CF-200D-2642-FE0F"},
    {"uomo_con_problemi_di_udito_colore1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"uomo_con_problemi_di_udito_colore2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"uomo_con_problemi_di_udito_colore3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"uomo_con_problemi_di_udito_colore4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"uomo_con_problemi_di_udito_colore5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"donna_con_la_barba", "1F9D4-200D-2640-FE0F"},
    {"donna_con_la_barba_colore1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"donna_con_la_barba_colore2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"donna_con_la_barba_colore3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"donna_con_la_barba_colore4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"donna_con_la_barba_colore5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"uomo_con_la_barba", "1F9D4-200D-2642-FE0F"},
    {"uomo_con_la_barba_colore1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"uomo_con_la_barba_colore2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"uomo_con_la_barba_colore3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"uomo_con_la_barba_colore4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"uomo_con_la_barba_colore5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"donna_in_sauna", "1F9D6-200D-2640-FE0F"},
    {"donna_in_sauna_colore1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"donna_in_sauna_colore2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"donna_in_sauna_colore3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"donna_in_sauna_colore4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"donna_in_sauna_colore5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"uomo_in_sauna", "1F9D6-200D-2642-FE0F"},
    {"uomo_in_sauna_colore1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"uomo_in_sauna_colore2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"uomo_in_sauna_colore3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"uomo_in_sauna_colore4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"uomo_in_sauna_colore5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"scalatrice", "1F9D7-200D-2640-FE0F"},
    {"scalatrice_colore1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"scalatrice_colore2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"scalatrice_colore3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"scalatrice_colore4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"scalatrice_colore5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"scalatore", "1F9D7-200D-2642-FE0F"},
    {"scalatore_colore1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"scalatore_colore2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"scalatore_colore3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"scalatore_colore4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"scalatore_colore5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"donna_in_posizione_del_loto", "1F9D8-200D-2640-FE0F"},
    {"donna_in_posizione_del_loto_colore1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"donna_in_posizione_del_loto_colore2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"donna_in_posizione_del_loto_colore3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"donna_in_posizione_del_loto_colore4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"donna_in_posizione_del_loto_colore5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"uomo_in_posizione_del_loto", "1F9D8-200D-2642-FE0F"},
    {"uomo_in_posizione_del_loto_colore1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"uomo_in_posizione_del_loto_colore2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"uomo_in_posizione_del_loto_colore3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"uomo_in_posizione_del_loto_colore4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"uomo_in_posizione_del_loto_colore5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"maga", "1F9D9-200D-2640-FE0F"},
    {"maga_colore1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"maga_colore2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"maga_colore3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"maga_colore4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"maga_colore5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"mago_uomo", "1F9D9-200D-2642-FE0F"},
    {"mago_uomo_colore1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"mago_uomo_colore2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"mago_uomo_colore3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"mago_uomo_colore4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"mago_uomo_colore5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"fata_donna", "1F9DA-200D-2640-FE0F"},
    {"fata_donna_colore1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"fata_donna_colore2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"fata_donna_colore3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"fata_donna_colore4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"fata_donna_colore5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"folletto_alato", "1F9DA-200D-2642-FE0F"},
    {"folletto_alato_colore1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"folletto_alato_colore2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"folletto_alato_colore3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"folletto_alato_colore4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"folletto_alato_colore5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"vampira", "1F9DB-200D-2640-FE0F"},
    {"vampira_colore1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"vampira_colore2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"vampira_colore3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"vampira_colore4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"vampira_colore5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"vampiro_uomo", "1F9DB-200D-2642-FE0F"},
    {"vampiro_uomo_colore1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"vampiro_uomo_colore2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"vampiro_uomo_colore3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"vampiro_uomo_colore4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"vampiro_uomo_colore5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"sirena_donna", "1F9DC-200D-2640-FE0F"},
    {"sirena_donna_colore1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"sirena_donna_colore2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"sirena_donna_colore3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"sirena_donna_colore4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"sirena_donna_colore5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"tritone", "1F9DC-200D-2642-FE0F"},
    {"tritone_colore1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"tritone_colore2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"tritone_colore3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"tritone_colore4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"tritone_colore5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"elfo_donna", "1F9DD-200D-2640-FE0F"},
    {"elfo_donna_colore1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"elfo_donna_colore2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"elfo_donna_colore3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"elfo_donna_colore4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"elfo_donna_colore5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"elfo_uomo", "1F9DD-200D-2642-FE0F"},
    {"elfo_uomo_colore1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"elfo_uomo_colore2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"elfo_uomo_colore3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"elfo_uomo_colore4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"elfo_uomo_colore5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"genio_donna", "1F9DE-200D-2640-FE0F"},
    {"genio_uomo", "1F9DE-200D-2642-FE0F"},
    {"zombie_donna", "1F9DF-200D-2640-FE0F"},
    {"zombie_uomo", "1F9DF-200D-2642-FE0F"},
    {"uomo_capelli_rossi", "1F468-200D-1F9B0"},
    {"uomo_capelli_rossi_colore1", "1F468-1F3FB-200D-1F9B0"},
    {"uomo_capelli_rossi_colore2", "1F468-1F3FC-200D-1F9B0"},
    {"uomo_capelli_rossi_colore3", "1F468-1F3FD-200D-1F9B0"},
    {"uomo_capelli_rossi_colore4", "1F468-1F3FE-200D-1F9B0"},
    {"uomo_capelli_rossi_colore5", "1F468-1F3FF-200D-1F9B0"},
    {"uomo_capelli_ricci", "1F468-200D-1F9B1"},
    {"uomo_capelli_ricci_colore1", "1F468-1F3FB-200D-1F9B1"},
    {"uomo_capelli_ricci_colore2", "1F468-1F3FC-200D-1F9B1"},
    {"uomo_capelli_ricci_colore3", "1F468-1F3FD-200D-1F9B1"},
    {"uomo_capelli_ricci_colore4", "1F468-1F3FE-200D-1F9B1"},
    {"uomo_capelli_ricci_colore5", "1F468-1F3FF-200D-1F9B1"},
    {"uomo_calvo", "1F468-200D-1F9B2"},
    {"uomo_calvo_colore1", "1F468-1F3FB-200D-1F9B2"},
    {"uomo_calvo_colore2", "1F468-1F3FC-200D-1F9B2"},
    {"uomo_calvo_colore3", "1F468-1F3FD-200D-1F9B2"},
    {"uomo_calvo_colore4", "1F468-1F3FE-200D-1F9B2"},
    {"uomo_calvo_colore5", "1F468-1F3FF-200D-1F9B2"},
    {"uomo_capelli_bianchi", "1F468-200D-1F9B3"},
    {"uomo_capelli_bianchi_colore1", "1F468-1F3FB-200D-1F9B3"},
    {"uomo_capelli_bianchi_colore2", "1F468-1F3FC-200D-1F9B3"},
    {"uomo_capelli_bianchi_colore3", "1F468-1F3FD-200D-1F9B3"},
    {"uomo_capelli_bianchi_colore4", "1F468-1F3FE-200D-1F9B3"},
    {"uomo_capelli_bianchi_colore5", "1F468-1F3FF-200D-1F9B3"},
    {"donna_capelli_rossi", "1F469-200D-1F9B0"},
    {"donna_capelli_rossi_colore1", "1F469-1F3FB-200D-1F9B0"},
    {"donna_capelli_rossi_colore2", "1F469-1F3FC-200D-1F9B0"},
    {"donna_capelli_rossi_colore3", "1F469-1F3FD-200D-1F9B0"},
    {"donna_capelli_rossi_colore4", "1F469-1F3FE-200D-1F9B0"},
    {"donna_capelli_rossi_colore5", "1F469-1F3FF-200D-1F9B0"},
    {"donna_capelli_ricci", "1F469-200D-1F9B1"},
    {"donna_capelli_ricci_colore1", "1F469-1F3FB-200D-1F9B1"},
    {"donna_capelli_ricci_colore2", "1F469-1F3FC-200D-1F9B1"},
    {"donna_capelli_ricci_colore3", "1F469-1F3FD-200D-1F9B1"},
    {"donna_capelli_ricci_colore4", "1F469-1F3FE-200D-1F9B1"},
    {"donna_capelli_ricci_colore5", "1F469-1F3FF-200D-1F9B1"},
    {"donna_calvo", "1F469-200D-1F9B2"},
    {"donna_calvo_colore1", "1F469-1F3FB-200D-1F9B2"},
    {"donna_calvo_colore2", "1F469-1F3FC-200D-1F9B2"},
    {"donna_calvo_colore3", "1F469-1F3FD-200D-1F9B2"},
    {"donna_calvo_colore4", "1F469-1F3FE-200D-1F9B2"},
    {"donna_calvo_colore5", "1F469-1F3FF-200D-1F9B2"},
    {"donna_capelli_bianchi", "1F469-200D-1F9B3"},
    {"donna_capelli_bianchi_colore1", "1F469-1F3FB-200D-1F9B3"},
    {"donna_capelli_bianchi_colore2", "1F469-1F3FC-200D-1F9B3"},
    {"donna_capelli_bianchi_colore3", "1F469-1F3FD-200D-1F9B3"},
    {"donna_capelli_bianchi_colore4", "1F469-1F3FE-200D-1F9B3"},
    {"donna_capelli_bianchi_colore5", "1F469-1F3FF-200D-1F9B3"},
    {"persona_capelli_rossi", "1F9D1-200D-1F9B0"},
    {"persona_capelli_rossi_colore1", "1F9D1-1F3FB-200D-1F9B0"},
    {"persona_capelli_rossi_colore2", "1F9D1-1F3FC-200D-1F9B0"},
    {"persona_capelli_rossi_colore3", "1F9D1-1F3FD-200D-1F9B0"},
    {"persona_capelli_rossi_colore4", "1F9D1-1F3FE-200D-1F9B0"},
    {"persona_capelli_rossi_colore5", "1F9D1-1F3FF-200D-1F9B0"},
    {"persona_capelli_ricci", "1F9D1-200D-1F9B1"},
    {"persona_capelli_ricci_colore1", "1F9D1-1F3FB-200D-1F9B1"},
    {"persona_capelli_ricci_colore2", "1F9D1-1F3FC-200D-1F9B1"},
    {"persona_capelli_ricci_colore3", "1F9D1-1F3FD-200D-1F9B1"},
    {"persona_capelli_ricci_colore4", "1F9D1-1F3FE-200D-1F9B1"},
    {"persona_capelli_ricci_colore5", "1F9D1-1F3FF-200D-1F9B1"},
    {"persona_calvo", "1F9D1-200D-1F9B2"},
    {"persona_calvo_colore1", "1F9D1-1F3FB-200D-1F9B2"},
    {"persona_calvo_colore2", "1F9D1-1F3FC-200D-1F9B2"},
    {"persona_calvo_colore3", "1F9D1-1F3FD-200D-1F9B2"},
    {"persona_calvo_colore4", "1F9D1-1F3FE-200D-1F9B2"},
    {"persona_calvo_colore5", "1F9D1-1F3FF-200D-1F9B2"},
    {"persona_capelli_bianchi", "1F9D1-200D-1F9B3"},
    {"persona_capelli_bianchi_colore1", "1F9D1-1F3FB-200D-1F9B3"},
    {"persona_capelli_bianchi_colore2", "1F9D1-1F3FC-200D-1F9B3"},
    {"persona_capelli_bianchi_colore3", "1F9D1-1F3FD-200D-1F9B3"},
    {"persona_capelli_bianchi_colore4", "1F9D1-1F3FE-200D-1F9B3"},
    {"persona_capelli_bianchi_colore5", "1F9D1-1F3FF-200D-1F9B3"},
    {"catena_spezzata", "26D3-FE0F-200D-1F4A5"},
    {"cuore_in_fiamme", "2764-FE0F-200D-1F525"},
    {"cuore_guarito", "2764-FE0F-200D-1FA79"},
    {"fungo_marrone", "1F344-200D-1F7EB"},
    {"lime", "1F34B-200D-1F7E9"},
    {"bandiera_transgender", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"bandiera_arcobaleno", "1F3F3-FE0F-200D-1F308"},
    {"bandiera_dei_pirati", "1F3F4-200D-2620-FE0F"},
    {"gatto_nero", "1F408-200D-2B1B"},
    {"cane_da_assistenza", "1F415-200D-1F9BA"},
    {"merlo", "1F426-200D-2B1B"},
    {"fenice", "1F426-200D-1F525"},
    {"orso_polare", "1F43B-200D-2744-FE0F"},
    {"occhio_nel_fumetto", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"faccina_che_espira", "1F62E-200D-1F4A8"},
    {"faccina_con_occhi_a_spirale", "1F635-200D-1F4AB"},
    {"testa_tra_le_nuvole", "1F636-200D-1F32B-FE0F"},
    {"scuotere_la_testa_in_senso_orizzontale", "1F642-200D-2194-FE0F"},
    {"scuotere_la_testa_in_senso_verticale", "1F642-200D-2195-FE0F"},
    {"persona_che_danza", "1F9D1-200D-1FA70"},
    {"persona_che_danza_colore1", "1F9D1-1F3FB-200D-1FA70"},
    {"persona_che_danza_colore2", "1F9D1-1F3FC-200D-1FA70"},
    {"persona_che_danza_colore3", "1F9D1-1F3FD-200D-1FA70"},
    {"persona_che_danza_colore4", "1F9D1-1F3FE-200D-1FA70"},
    {"persona_che_danza_colore5", "1F9D1-1F3FF-200D-1FA70"},
    {"freccia_su_giù", "2195"},
    {"simbolo_della_radioattività", "2622"},
    {"simbolo_della_disabilità", "267F"},
    {"città_al_tramonto", "1F306"},
    {"tazza_da_tè_senza_manico", "1F375"},
    {"sakè", "1F376"},
    {"bambù_decorato", "1F38D"},
    {"modalità_vibrazione", "1F4F3"},
    {"luminosità_bassa", "1F505"},
    {"luminosità_elevata", "1F506"},
    {"statua_della_libertà", "1F5FD"},
    {"treno_alta_velocità", "1F684"},
    {"treno_alta_velocità_punta_arrotondata", "1F685"},
    {"tempio_indù", "1F6D5"},
    {"risciò_a_motore", "1F6FA"},
    {"carta_didentità", "1FAAA"},
    {"bandiera_isole_åland", "1F1E6-1F1FD"},
    {"bandiera_saint_barthélemy", "1F1E7-1F1F1"},
    {"bandiera_curaçao", "1F1E8-1F1FC"},
    {"bandiera_isole_fær_øer", "1F1EB-1F1F4"},
    {"bandiera_perù", "1F1F5-1F1EA"},
    {"bandiera_são_tomé_e_príncipe", "1F1F8-1F1F9"},
    {"bandiera_città_del_vaticano", "1F1FB-1F1E6"},
    {"persona_che_lavora_nella_sanità", "1F9D1-200D-2695-FE0F"},
    {"persona_che_lavora_nella_sanità_colore1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"persona_che_lavora_nella_sanità_colore2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"persona_che_lavora_nella_sanità_colore3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"persona_che_lavora_nella_sanità_colore4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"persona_che_lavora_nella_sanità_colore5", "1F9D1-1F3FF-200D-2695-FE0F"},
};

