// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_da[] = {
    {"udrabstegn_og_sporgsmalstegn", "2049"},
    {"varemaerke", "2122"},
    {"information", "2139"},
    {"pil_mod_venstre_og_hojre", "2194"},
    {"op_og_nedadvendt_pil", "2195"},
    {"opadvendt_pil_mod_venstre", "2196"},
    {"opadvendt_pil_mod_hojre", "2197"},
    {"nedadvendt_pil_mod_hojre", "2198"},
    {"nedadvendt_pil_mod_venstre", "2199"},
    {"tastatur", "2328"},
    {"sol", "2600"},
    {"sky", "2601"},
    {"paraply", "2602"},
    {"snemand", "2603"},
    {"komet", "2604"},
    {"afkrydsningsfelt_med_flueben", "2611"},
    {"paraply_med_regndraber", "2614"},
    {"varm_drik", "2615"},
    {"klover", "2618"},
    {"kranium_og_korslagte_knogler", "2620"},
    {"radioaktiv", "2622"},
    {"biologisk_fare", "2623"},
    {"ortodokst_kors", "2626"},
    {"dharma_hjul", "2638"},
    {"trist_ansigt", "2639"},
    {"kvindesymbol", "2640"},
    {"mandesymbol", "2642"},
    {"vaedderen", "2648"},
    {"tyren", "2649"},
    {"skytten", "2650"},
    {"stenbukken", "2651"},
    {"vandmanden", "2652"},
    {"fisken", "2653"},
    {"spar", "2660"},
    {"klor", "2663"},
    {"hjerter", "2665"},
    {"ruder", "2666"},
    {"varme_kilder", "2668"},
    {"hammer_og_hakke", "2692"},
    {"anker", "2693"},
    {"korslagte_svaerd", "2694"},
    {"medicinsymbol", "2695"},
    {"vaegt", "2696"},
    {"kolbe", "2697"},
    {"tandhjul", "2699"},
    {"saks", "2702"},
    {"hvidt_flueben", "2705"},
    {"fly", "2708"},
    {"konvolut", "2709"},
    {"sort_fyldepen", "2712"},
    {"flueben", "2714"},
    {"gange", "2716"},
    {"davidsstjerne", "2721"},
    {"stjerner", "2728"},
    {"ottetakket_spids_stjerne", "2733"},
    {"ottetakket_stjerne", "2734"},
    {"snefnug", "2744"},
    {"funkle", "2747"},
    {"rodt_sporgsmalstegn", "2753"},
    {"hvidt_sporgsmalstegn", "2754"},
    {"hvidt_udrabstegn", "2755"},
    {"rodt_udrabstegn", "2757"},
    {"hjerte_som_udrabstegn", "2763"},
    {"rodt_hjerte", "2764"},
    {"plus", "2795"},
    {"minus", "2796"},
    {"dividere", "2797"},
    {"pil_mod_hojre_med_sving_opad", "2934"},
    {"pil_mod_hojre_med_sving_nedad", "2935"},
    {"bolgestreg", "3030"},
    {"cirkulaert_ideogram_for_lykonskning", "3297"},
    {"cirkulaert_ideogram_for_hemmelighed", "3299"},
    {"copyright", "00A9"},
    {"registreret_varemaerke", "00AE"},
    {"dobbelt_udrabstegn", "203C"},
    {"pil_mod_hojre_med_sving_til_venstre", "21A9"},
    {"pil_mod_venstre_med_sving_til_hojre", "21AA"},
    {"ur", "231A"},
    {"udlobet_timeglas", "231B"},
    {"udskubningsknap", "23CF"},
    {"spol_frem_knap", "23E9"},
    {"spol_tilbage_knap", "23EA"},
    {"hurtigt_op_knap", "23EB"},
    {"hurtigt_ned_knap", "23EC"},
    {"knap_til_naeste_nummer", "23ED"},
    {"knap_til_forrige_nummer", "23EE"},
    {"knap_til_afspilning_og_pause", "23EF"},
    {"vaekkeur", "23F0"},
    {"stopur", "23F1"},
    {"aeggeur", "23F2"},
    {"timeglas_med_lobende_sand", "23F3"},
    {"pauseknap", "23F8"},
    {"stopknap", "23F9"},
    {"optageknap", "23FA"},
    {"m_i_cirkel", "24C2"},
    {"lille_sort_kvadrat", "25AA"},
    {"lille_hvidt_kvadrat", "25AB"},
    {"afspilningsknap", "25B6"},
    {"knap_til_baglaens_afspilning", "25C0"},
    {"mellemstort_hvidt_kvadrat", "25FB"},
    {"mellemstort_sort_kvadrat", "25FC"},
    {"halvlille_hvidt_kvadrat", "25FD"},
    {"halvlille_sort_kvadrat", "25FE"},
    {"telefon", "260E"},
    {"raekker_finger_op", "261D"},
    {"raekker_finger_op_farve1", "261D-1F3FB"},
    {"raekker_finger_op_farve2", "261D-1F3FC"},
    {"raekker_finger_op_farve3", "261D-1F3FD"},
    {"raekker_finger_op_farve4", "261D-1F3FE"},
    {"raekker_finger_op_farve5", "261D-1F3FF"},
    {"stjerne_og_halvmane", "262A"},
    {"fredssymbol", "262E"},
    {"yin_og_yang", "262F"},
    {"smilende_ansigt", "263A"},
    {"tvillingen", "264A"},
    {"krebsen", "264B"},
    {"loven", "264C"},
    {"jomfruen", "264D"},
    {"vaegten", "264E"},
    {"skorpionen", "264F"},
    {"skakbrik", "265F"},
    {"genbrug", "267B"},
    {"uendelig", "267E"},
    {"symbol_for_korestol", "267F"},
    {"atomsymbol", "269B"},
    {"fransk_lilje", "269C"},
    {"advarselsskilt", "26A0"},
    {"hojspaending", "26A1"},
    {"symbol_for_transkonnethed", "26A7"},
    {"hvid_cirkel", "26AA"},
    {"sort_cirkel", "26AB"},
    {"kiste", "26B0"},
    {"urne", "26B1"},
    {"fodbold", "26BD"},
    {"baseball", "26BE"},
    {"snemand_uden_snevejr", "26C4"},
    {"sol_bag_sky", "26C5"},
    {"sky_med_lyn_og_regn", "26C8"},
    {"slangebaereren", "26CE"},
    {"hakke", "26CF"},
    {"hjelm_med_hvidt_kors", "26D1"},
    {"kaeder", "26D3"},
    {"adgang_forbudt", "26D4"},
    {"shinto_helligdom", "26E9"},
    {"kirke", "26EA"},
    {"bjerg", "26F0"},
    {"parasol", "26F1"},
    {"springvand", "26F2"},
    {"golfhul_med_flag", "26F3"},
    {"faerge", "26F4"},
    {"sejlbad", "26F5"},
    {"skilober", "26F7"},
    {"skojte", "26F8"},
    {"person_der_dribler_med_en_bold", "26F9"},
    {"person_der_dribler_med_en_bold_farve1", "26F9-1F3FB"},
    {"person_der_dribler_med_en_bold_farve2", "26F9-1F3FC"},
    {"person_der_dribler_med_en_bold_farve3", "26F9-1F3FD"},
    {"person_der_dribler_med_en_bold_farve4", "26F9-1F3FE"},
    {"person_der_dribler_med_en_bold_farve5", "26F9-1F3FF"},
    {"telt", "26FA"},
    {"tankstation", "26FD"},
    {"loftet_knytnaeve", "270A"},
    {"loftet_knytnaeve_farve1", "270A-1F3FB"},
    {"loftet_knytnaeve_farve2", "270A-1F3FC"},
    {"loftet_knytnaeve_farve3", "270A-1F3FD"},
    {"loftet_knytnaeve_farve4", "270A-1F3FE"},
    {"loftet_knytnaeve_farve5", "270A-1F3FF"},
    {"loftet_hand", "270B"},
    {"loftet_hand_farve1", "270B-1F3FB"},
    {"loftet_hand_farve2", "270B-1F3FC"},
    {"loftet_hand_farve3", "270B-1F3FD"},
    {"loftet_hand_farve4", "270B-1F3FE"},
    {"loftet_hand_farve5", "270B-1F3FF"},
    {"v_tegn", "270C"},
    {"v_tegn_farve1", "270C-1F3FB"},
    {"v_tegn_farve2", "270C-1F3FC"},
    {"v_tegn_farve3", "270C-1F3FD"},
    {"v_tegn_farve4", "270C-1F3FE"},
    {"v_tegn_farve5", "270C-1F3FF"},
    {"skrivende_hand", "270D"},
    {"skrivende_hand_farve1", "270D-1F3FB"},
    {"skrivende_hand_farve2", "270D-1F3FC"},
    {"skrivende_hand_farve3", "270D-1F3FD"},
    {"skrivende_hand_farve4", "270D-1F3FE"},
    {"skrivende_hand_farve5", "270D-1F3FF"},
    {"blyant", "270F"},
    {"latinsk_kors", "271D"},
    {"kryds", "274C"},
    {"krydsknap", "274E"},
    {"pil_mod_hojre", "27A1"},
    {"krolle", "27B0"},
    {"dobbelt_krolle", "27BF"},
    {"pil_mod_venstre", "2B05"},
    {"opadvendt_pil", "2B06"},
    {"nedadvendt_pil", "2B07"},
    {"stort_sort_kvadrat", "2B1B"},
    {"stort_hvidt_kvadrat", "2B1C"},
    {"stjerne", "2B50"},
    {"stor_rod_cirkel", "2B55"},
    {"japansk_kommatering", "303D"},
    {"mahjong_brik", "1F004"},
    {"joker", "1F0CF"},
    {"blodtype_a", "1F170"},
    {"blodtype_b", "1F171"},
    {"blodtype_o", "1F17E"},
    {"parkering", "1F17F"},
    {"blodtype_ab", "1F18E"},
    {"cl_knap", "1F191"},
    {"cool_knap", "1F192"},
    {"free_knap", "1F193"},
    {"id_knap", "1F194"},
    {"new_knap", "1F195"},
    {"ng_knap", "1F196"},
    {"ok", "1F197"},
    {"sos", "1F198"},
    {"up_knap", "1F199"},
    {"versus", "1F19A"},
    {"regional_indikator_a", "1F1E6"},
    {"regional_indikator_b", "1F1E7"},
    {"regional_indikator_c", "1F1E8"},
    {"regional_indikator_d", "1F1E9"},
    {"regional_indikator_e", "1F1EA"},
    {"regional_indikator_f", "1F1EB"},
    {"regional_indikator_g", "1F1EC"},
    {"regional_indikator_h", "1F1ED"},
    {"regional_indikator_i", "1F1EE"},
    {"regional_indikator_j", "1F1EF"},
    {"regional_indikator_k", "1F1F0"},
    {"regional_indikator_l", "1F1F1"},
    {"regional_indikator_m", "1F1F2"},
    {"regional_indikator_n", "1F1F3"},
    {"regional_indikator_o", "1F1F4"},
    {"regional_indikator_p", "1F1F5"},
    {"regional_indikator_q", "1F1F6"},
    {"regional_indikator_r", "1F1F7"},
    {"regional_indikator_s", "1F1F8"},
    {"regional_indikator_t", "1F1F9"},
    {"regional_indikator_u", "1F1FA"},
    {"regional_indikator_v", "1F1FB"},
    {"regional_indikator_w", "1F1FC"},
    {"regional_indikator_x", "1F1FD"},
    {"regional_indikator_y", "1F1FE"},
    {"regional_indikator_z", "1F1FF"},
    {"kvadratisk_ideogram_for_her", "1F201"},
    {"kvadratisk_ideogram_for_serviceafgift", "1F202"},
    {"kvadratisk_ideogram_for_gratis", "1F21A"},
    {"kvadratisk_ideogram_for_reserveret", "1F22F"},
    {"kvadratisk_ideogram_for_forbud", "1F232"},
    {"kvadratisk_ideogram_for_ledig", "1F233"},
    {"kvadratisk_ideogram_for_bestaet", "1F234"},
    {"kvadratisk_ideogram_for_optaget", "1F235"},
    {"kvadratisk_ideogram_for_ikke_gratis", "1F236"},
    {"kvadratisk_ideogram_for_manedligt_belob", "1F237"},
    {"kvadratisk_ideogram_for_anvendelse", "1F238"},
    {"kvadratisk_ideogram_for_rabat", "1F239"},
    {"kvadratisk_ideogram_for_aben", "1F23A"},
    {"cirkulaert_ideogram_for_tilbud", "1F250"},
    {"cirkulaert_ideogram_for_accept", "1F251"},
    {"cyklon", "1F300"},
    {"taget", "1F301"},
    {"lukket_paraply", "1F302"},
    {"stjernehimmel", "1F303"},
    {"solopgang_over_bjerge", "1F304"},
    {"solopgang", "1F305"},
    {"by_i_solnedgang", "1F306"},
    {"solnedgang", "1F307"},
    {"regnbue", "1F308"},
    {"bro_i_morke", "1F309"},
    {"bolge", "1F30A"},
    {"vulkan", "1F30B"},
    {"maelkevejen", "1F30C"},
    {"globus_med_europa_og_afrika", "1F30D"},
    {"globus_med_nord_og_sydamerika", "1F30E"},
    {"globus_med_asien_og_australien", "1F30F"},
    {"globus_med_meridianer", "1F310"},
    {"nymane", "1F311"},
    {"tiltagende_mane", "1F312"},
    {"halvmane_1_fjerdedel", "1F313"},
    {"tiltagende_mane_naesten_fuld", "1F314"},
    {"fuldmane", "1F315"},
    {"aftagende_mane_naesten_fuld", "1F316"},
    {"halvmane_4_fjerdedel", "1F317"},
    {"aftagende_mane", "1F318"},
    {"halvmane", "1F319"},
    {"nymane_med_ansigt", "1F31A"},
    {"mane_med_ansigt_1_fjerdedel", "1F31B"},
    {"mane_med_ansigt_4_fjerdedel", "1F31C"},
    {"fuldmane_med_ansigt", "1F31D"},
    {"sol_med_ansigt", "1F31E"},
    {"blinkende_stjerne", "1F31F"},
    {"stjerneskud", "1F320"},
    {"termometer", "1F321"},
    {"sol_bag_lille_sky", "1F324"},
    {"sol_bag_stor_sky", "1F325"},
    {"sol_bag_sky_med_regn", "1F326"},
    {"sky_med_regn", "1F327"},
    {"sky_med_sne", "1F328"},
    {"tordensky", "1F329"},
    {"tornado", "1F32A"},
    {"tage", "1F32B"},
    {"vindansigt", "1F32C"},
    {"hotdog", "1F32D"},
    {"taco", "1F32E"},
    {"burrito", "1F32F"},
    {"kastanje", "1F330"},
    {"spire", "1F331"},
    {"naletrae", "1F332"},
    {"lovtrae", "1F333"},
    {"palme", "1F334"},
    {"kaktus", "1F335"},
    {"chili", "1F336"},
    {"tulipan", "1F337"},
    {"kirsebaerblomst", "1F338"},
    {"rose", "1F339"},
    {"hibiscus", "1F33A"},
    {"solsikke", "1F33B"},
    {"blomst", "1F33C"},
    {"majs", "1F33D"},
    {"risplante", "1F33E"},
    {"urt", "1F33F"},
    {"firklover", "1F340"},
    {"ahornblad", "1F341"},
    {"faldende_blade", "1F342"},
    {"blade_i_vinden", "1F343"},
    {"svamp", "1F344"},
    {"tomat", "1F345"},
    {"aubergine", "1F346"},
    {"vindruer", "1F347"},
    {"melon", "1F348"},
    {"vandmelon", "1F349"},
    {"appelsin", "1F34A"},
    {"citron", "1F34B"},
    {"banan", "1F34C"},
    {"ananas", "1F34D"},
    {"rodt_aeble", "1F34E"},
    {"gront_aeble", "1F34F"},
    {"paere", "1F350"},
    {"fersken", "1F351"},
    {"kirsebaer", "1F352"},
    {"jordbaer", "1F353"},
    {"hamburger", "1F354"},
    {"pizza", "1F355"},
    {"kod", "1F356"},
    {"kyllingelar", "1F357"},
    {"riskiks", "1F358"},
    {"riskugle", "1F359"},
    {"ris", "1F35A"},
    {"ris_og_karry", "1F35B"},
    {"nudler", "1F35C"},
    {"spaghetti", "1F35D"},
    {"brod", "1F35E"},
    {"pomfritter", "1F35F"},
    {"sod_kartoffel", "1F360"},
    {"dango", "1F361"},
    {"mad_pa_spyd", "1F362"},
    {"sushi", "1F363"},
    {"dybstegt_reje", "1F364"},
    {"fiskekage", "1F365"},
    {"softice", "1F366"},
    {"isdessert", "1F367"},
    {"is", "1F368"},
    {"donut", "1F369"},
    {"smakage", "1F36A"},
    {"chokolade", "1F36B"},
    {"bolsje", "1F36C"},
    {"slikkepind", "1F36D"},
    {"dessert", "1F36E"},
    {"honning", "1F36F"},
    {"lagkage", "1F370"},
    {"bento", "1F371"},
    {"gryde_med_mad", "1F372"},
    {"stegepande", "1F373"},
    {"kniv_og_gaffel", "1F374"},
    {"tekrus", "1F375"},
    {"sake", "1F376"},
    {"vinglas", "1F377"},
    {"cocktailglas", "1F378"},
    {"tropisk_drink", "1F379"},
    {"olkrus", "1F37A"},
    {"skalende_olkrus", "1F37B"},
    {"sutteflaske", "1F37C"},
    {"kniv_og_gaffel_med_tallerken", "1F37D"},
    {"flaske_med_flyvende_prop", "1F37E"},
    {"popcorn", "1F37F"},
    {"slojfe", "1F380"},
    {"gave", "1F381"},
    {"fodselsdagskage", "1F382"},
    {"graeskarmand", "1F383"},
    {"juletrae", "1F384"},
    {"julemand", "1F385"},
    {"julemand_farve1", "1F385-1F3FB"},
    {"julemand_farve2", "1F385-1F3FC"},
    {"julemand_farve3", "1F385-1F3FD"},
    {"julemand_farve4", "1F385-1F3FE"},
    {"julemand_farve5", "1F385-1F3FF"},
    {"fyrvaerkeri", "1F386"},
    {"stjernekaster", "1F387"},
    {"ballon", "1F388"},
    {"bordbombe", "1F389"},
    {"konfetti", "1F38A"},
    {"trae_med_papirpynt", "1F38B"},
    {"to_krydsede_flag", "1F38C"},
    {"fyrretraesdekoration", "1F38D"},
    {"japanske_dukker", "1F38E"},
    {"karpevimpel", "1F38F"},
    {"vindspil", "1F390"},
    {"maneceremoni", "1F391"},
    {"rygsaek", "1F392"},
    {"eksamenshat", "1F393"},
    {"militaermedalje", "1F396"},
    {"mindeband", "1F397"},
    {"studiemikrofon", "1F399"},
    {"volumenkontrol", "1F39A"},
    {"knapper", "1F39B"},
    {"filmstrimmel", "1F39E"},
    {"adgangsbillet", "1F39F"},
    {"karrusel", "1F3A0"},
    {"pariserhjul", "1F3A1"},
    {"rutsjebane", "1F3A2"},
    {"fiskestang", "1F3A3"},
    {"mikrofon", "1F3A4"},
    {"filmkamera", "1F3A5"},
    {"film", "1F3A6"},
    {"hovedtelefon", "1F3A7"},
    {"palet", "1F3A8"},
    {"hoj_hat", "1F3A9"},
    {"cirkus", "1F3AA"},
    {"billet", "1F3AB"},
    {"klaptrae", "1F3AC"},
    {"masker", "1F3AD"},
    {"videospil", "1F3AE"},
    {"pletskud", "1F3AF"},
    {"spillemaskine", "1F3B0"},
    {"billard", "1F3B1"},
    {"terning", "1F3B2"},
    {"bowling", "1F3B3"},
    {"spillekort_med_blomster", "1F3B4"},
    {"node", "1F3B5"},
    {"noder", "1F3B6"},
    {"saxofon", "1F3B7"},
    {"guitar", "1F3B8"},
    {"klaver", "1F3B9"},
    {"trompet", "1F3BA"},
    {"violin", "1F3BB"},
    {"partitur", "1F3BC"},
    {"lobetroje", "1F3BD"},
    {"tennis", "1F3BE"},
    {"ski", "1F3BF"},
    {"basketball", "1F3C0"},
    {"ternet_flag", "1F3C1"},
    {"snowboarder", "1F3C2"},
    {"snowboarder_farve1", "1F3C2-1F3FB"},
    {"snowboarder_farve2", "1F3C2-1F3FC"},
    {"snowboarder_farve3", "1F3C2-1F3FD"},
    {"snowboarder_farve4", "1F3C2-1F3FE"},
    {"snowboarder_farve5", "1F3C2-1F3FF"},
    {"lober", "1F3C3"},
    {"lober_farve1", "1F3C3-1F3FB"},
    {"lober_farve2", "1F3C3-1F3FC"},
    {"lober_farve3", "1F3C3-1F3FD"},
    {"lober_farve4", "1F3C3-1F3FE"},
    {"lober_farve5", "1F3C3-1F3FF"},
    {"surfer", "1F3C4"},
    {"surfer_farve1", "1F3C4-1F3FB"},
    {"surfer_farve2", "1F3C4-1F3FC"},
    {"surfer_farve3", "1F3C4-1F3FD"},
    {"surfer_farve4", "1F3C4-1F3FE"},
    {"surfer_farve5", "1F3C4-1F3FF"},
    {"sportsmedalje", "1F3C5"},
    {"pokal", "1F3C6"},
    {"hestevaeddelob", "1F3C7"},
    {"hestevaeddelob_farve1", "1F3C7-1F3FB"},
    {"hestevaeddelob_farve2", "1F3C7-1F3FC"},
    {"hestevaeddelob_farve3", "1F3C7-1F3FD"},
    {"hestevaeddelob_farve4", "1F3C7-1F3FE"},
    {"hestevaeddelob_farve5", "1F3C7-1F3FF"},
    {"amerikansk_fodbold", "1F3C8"},
    {"rugbybold", "1F3C9"},
    {"svommer", "1F3CA"},
    {"svommer_farve1", "1F3CA-1F3FB"},
    {"svommer_farve2", "1F3CA-1F3FC"},
    {"svommer_farve3", "1F3CA-1F3FD"},
    {"svommer_farve4", "1F3CA-1F3FE"},
    {"svommer_farve5", "1F3CA-1F3FF"},
    {"vaegtlofter", "1F3CB"},
    {"vaegtlofter_farve1", "1F3CB-1F3FB"},
    {"vaegtlofter_farve2", "1F3CB-1F3FC"},
    {"vaegtlofter_farve3", "1F3CB-1F3FD"},
    {"vaegtlofter_farve4", "1F3CB-1F3FE"},
    {"vaegtlofter_farve5", "1F3CB-1F3FF"},
    {"golfspiller", "1F3CC"},
    {"golfspiller_farve1", "1F3CC-1F3FB"},
    {"golfspiller_farve2", "1F3CC-1F3FC"},
    {"golfspiller_farve3", "1F3CC-1F3FD"},
    {"golfspiller_farve4", "1F3CC-1F3FE"},
    {"golfspiller_farve5", "1F3CC-1F3FF"},
    {"motorcykel", "1F3CD"},
    {"racerbil", "1F3CE"},
    {"cricket", "1F3CF"},
    {"volleyball", "1F3D0"},
    {"hockey", "1F3D1"},
    {"ishockey", "1F3D2"},
    {"bordtennis", "1F3D3"},
    {"sneklaedt_bjerg", "1F3D4"},
    {"camping", "1F3D5"},
    {"strand_med_parasol", "1F3D6"},
    {"byggeplads", "1F3D7"},
    {"huse", "1F3D8"},
    {"bylandskab", "1F3D9"},
    {"faldefaerdigt_hus", "1F3DA"},
    {"klassisk_bygning", "1F3DB"},
    {"orken", "1F3DC"},
    {"ode_o", "1F3DD"},
    {"nationalpark", "1F3DE"},
    {"stadion", "1F3DF"},
    {"hus", "1F3E0"},
    {"hus_med_have", "1F3E1"},
    {"kontorbygning", "1F3E2"},
    {"japansk_posthus", "1F3E3"},
    {"posthus", "1F3E4"},
    {"hospital", "1F3E5"},
    {"bank", "1F3E6"},
    {"haeveautomat", "1F3E7"},
    {"hotel", "1F3E8"},
    {"kaerlighedshotel", "1F3E9"},
    {"dognkiosk", "1F3EA"},
    {"skole", "1F3EB"},
    {"stormagasin", "1F3EC"},
    {"fabrik", "1F3ED"},
    {"papirlanterne", "1F3EE"},
    {"japansk_slot", "1F3EF"},
    {"slot", "1F3F0"},
    {"hvidt_flag", "1F3F3"},
    {"sort_flag", "1F3F4"},
    {"roset", "1F3F5"},
    {"maerkat", "1F3F7"},
    {"badminton", "1F3F8"},
    {"bue_og_pil", "1F3F9"},
    {"amfora", "1F3FA"},
    {"lys_teint", "1F3FB"},
    {"medium_til_lys_teint", "1F3FC"},
    {"medium_teint", "1F3FD"},
    {"medium_til_mork_teint", "1F3FE"},
    {"mork_teint", "1F3FF"},
    {"rotte", "1F400"},
    {"mus", "1F401"},
    {"okse", "1FA93"},
    {"vandboffel", "1F403"},
    {"ko", "1F404"},
    {"tiger", "1F405"},
    {"leopard", "1F406"},
    {"kanin", "1F407"},
    {"kat", "1F408"},
    {"drage", "1F409"},
    {"krokodille", "1F40A"},
    {"hval", "1F40B"},
    {"snegl", "1F40C"},
    {"slange", "1F40D"},
    {"hest", "1F40E"},
    {"vaedder", "1F40F"},
    {"ged", "1F410"},
    {"far", "1F411"},
    {"abe", "1F412"},
    {"hane", "1F413"},
    {"hone", "1F414"},
    {"hund", "1F415"},
    {"gris", "1F416"},
    {"vildsvin", "1F417"},
    {"elefant", "1F418"},
    {"blaeksprutte", "1F419"},
    {"konkylie", "1F41A"},
    {"larve", "1F41B"},
    {"myre", "1F41C"},
    {"honningbi", "1F41D"},
    {"mariehone", "1F41E"},
    {"fisk", "1F41F"},
    {"tropisk_fisk", "1F420"},
    {"kuglefisk", "1F421"},
    {"skildpadde", "1F422"},
    {"nyudklaekket_kylling", "1F423"},
    {"kylling", "1F424"},
    {"kylling_forfra", "1F425"},
    {"fugl", "1F426"},
    {"pingvin", "1F427"},
    {"koala", "1F428"},
    {"puddel", "1F429"},
    {"dromedar", "1F42A"},
    {"kamel", "1F42B"},
    {"delfin", "1F42C"},
    {"musehoved", "1F42D"},
    {"kohoved", "1F42E"},
    {"tigerhoved", "1F42F"},
    {"kaninhoved", "1F430"},
    {"kattehoved", "1F431"},
    {"dragehoved", "1F432"},
    {"hval_med_vandsprojt", "1F433"},
    {"hestehoved", "1F434"},
    {"abehoved", "1F435"},
    {"hundehoved", "1F436"},
    {"grisehoved", "1F437"},
    {"fro", "1F438"},
    {"hamsterhoved", "1F439"},
    {"ulvehoved", "1F43A"},
    {"bjornehoved", "1F43B"},
    {"pandahoved", "1F43C"},
    {"grisetryne", "1F43D"},
    {"poteaftryk", "1F43E"},
    {"jordegern", "1F43F"},
    {"ojne", "1F440"},
    {"oje", "1F441"},
    {"ore", "1F442"},
    {"ore_farve1", "1F442-1F3FB"},
    {"ore_farve2", "1F442-1F3FC"},
    {"ore_farve3", "1F442-1F3FD"},
    {"ore_farve4", "1F442-1F3FE"},
    {"ore_farve5", "1F442-1F3FF"},
    {"naese", "1F443"},
    {"naese_farve1", "1F443-1F3FB"},
    {"naese_farve2", "1F443-1F3FC"},
    {"naese_farve3", "1F443-1F3FD"},
    {"naese_farve4", "1F443-1F3FE"},
    {"naese_farve5", "1F443-1F3FF"},
    {"mund", "1F444"},
    {"tunge", "1F445"},
    {"peger_opad", "1F446"},
    {"peger_opad_farve1", "1F446-1F3FB"},
    {"peger_opad_farve2", "1F446-1F3FC"},
    {"peger_opad_farve3", "1F446-1F3FD"},
    {"peger_opad_farve4", "1F446-1F3FE"},
    {"peger_opad_farve5", "1F446-1F3FF"},
    {"peger_nedad", "1F447"},
    {"peger_nedad_farve1", "1F447-1F3FB"},
    {"peger_nedad_farve2", "1F447-1F3FC"},
    {"peger_nedad_farve3", "1F447-1F3FD"},
    {"peger_nedad_farve4", "1F447-1F3FE"},
    {"peger_nedad_farve5", "1F447-1F3FF"},
    {"peger_mod_venstre", "1F448"},
    {"peger_mod_venstre_farve1", "1F448-1F3FB"},
    {"peger_mod_venstre_farve2", "1F448-1F3FC"},
    {"peger_mod_venstre_farve3", "1F448-1F3FD"},
    {"peger_mod_venstre_farve4", "1F448-1F3FE"},
    {"peger_mod_venstre_farve5", "1F448-1F3FF"},
    {"peger_mod_hojre", "1F449"},
    {"peger_mod_hojre_farve1", "1F449-1F3FB"},
    {"peger_mod_hojre_farve2", "1F449-1F3FC"},
    {"peger_mod_hojre_farve3", "1F449-1F3FD"},
    {"peger_mod_hojre_farve4", "1F449-1F3FE"},
    {"peger_mod_hojre_farve5", "1F449-1F3FF"},
    {"knytnaeve", "1F44A"},
    {"knytnaeve_farve1", "1F44A-1F3FB"},
    {"knytnaeve_farve2", "1F44A-1F3FC"},
    {"knytnaeve_farve3", "1F44A-1F3FD"},
    {"knytnaeve_farve4", "1F44A-1F3FE"},
    {"knytnaeve_farve5", "1F44A-1F3FF"},
    {"vinker", "1F44B"},
    {"vinker_farve1", "1F44B-1F3FB"},
    {"vinker_farve2", "1F44B-1F3FC"},
    {"vinker_farve3", "1F44B-1F3FD"},
    {"vinker_farve4", "1F44B-1F3FE"},
    {"vinker_farve5", "1F44B-1F3FF"},
    {"ok_tegn", "1F44C"},
    {"ok_tegn_farve1", "1F44C-1F3FB"},
    {"ok_tegn_farve2", "1F44C-1F3FC"},
    {"ok_tegn_farve3", "1F44C-1F3FD"},
    {"ok_tegn_farve4", "1F44C-1F3FE"},
    {"ok_tegn_farve5", "1F44C-1F3FF"},
    {"tommelfinger_op", "1F44D"},
    {"tommelfinger_op_farve1", "1F44D-1F3FB"},
    {"tommelfinger_op_farve2", "1F44D-1F3FC"},
    {"tommelfinger_op_farve3", "1F44D-1F3FD"},
    {"tommelfinger_op_farve4", "1F44D-1F3FE"},
    {"tommelfinger_op_farve5", "1F44D-1F3FF"},
    {"tommelfinger_ned", "1F44E"},
    {"tommelfinger_ned_farve1", "1F44E-1F3FB"},
    {"tommelfinger_ned_farve2", "1F44E-1F3FC"},
    {"tommelfinger_ned_farve3", "1F44E-1F3FD"},
    {"tommelfinger_ned_farve4", "1F44E-1F3FE"},
    {"tommelfinger_ned_farve5", "1F44E-1F3FF"},
    {"klappende_haender", "1F44F"},
    {"klappende_haender_farve1", "1F44F-1F3FB"},
    {"klappende_haender_farve2", "1F44F-1F3FC"},
    {"klappende_haender_farve3", "1F44F-1F3FD"},
    {"klappende_haender_farve4", "1F44F-1F3FE"},
    {"klappende_haender_farve5", "1F44F-1F3FF"},
    {"abne_haender", "1F450"},
    {"abne_haender_farve1", "1F450-1F3FB"},
    {"abne_haender_farve2", "1F450-1F3FC"},
    {"abne_haender_farve3", "1F450-1F3FD"},
    {"abne_haender_farve4", "1F450-1F3FE"},
    {"abne_haender_farve5", "1F450-1F3FF"},
    {"krone", "1F451"},
    {"hat", "1F452"},
    {"briller", "1F453"},
    {"slips", "1F454"},
    {"t_shirt", "1F455"},
    {"bukser", "1F456"},
    {"kjole", "1F457"},
    {"kimono", "1F458"},
    {"bikini", "1F459"},
    {"bluse", "1F45A"},
    {"pung", "1F45B"},
    {"handtaske", "1F45C"},
    {"clutch", "1F45D"},
    {"herresko", "1F45E"},
    {"lobesko", "1F45F"},
    {"hoje_haele", "1F460"},
    {"damesandal", "1F461"},
    {"damestovle", "1F462"},
    {"fodspor", "1F463"},
    {"buste_i_silhuet", "1F464"},
    {"buster_i_silhuet", "1F465"},
    {"dreng", "1F466"},
    {"dreng_farve1", "1F466-1F3FB"},
    {"dreng_farve2", "1F466-1F3FC"},
    {"dreng_farve3", "1F466-1F3FD"},
    {"dreng_farve4", "1F466-1F3FE"},
    {"dreng_farve5", "1F466-1F3FF"},
    {"pige", "1F467"},
    {"pige_farve1", "1F467-1F3FB"},
    {"pige_farve2", "1F467-1F3FC"},
    {"pige_farve3", "1F467-1F3FD"},
    {"pige_farve4", "1F467-1F3FE"},
    {"pige_farve5", "1F467-1F3FF"},
    {"mand", "1F468"},
    {"mand_farve1", "1F468-1F3FB"},
    {"mand_farve2", "1F468-1F3FC"},
    {"mand_farve3", "1F468-1F3FD"},
    {"mand_farve4", "1F468-1F3FE"},
    {"mand_farve5", "1F468-1F3FF"},
    {"kvinde", "1F469"},
    {"kvinde_farve1", "1F469-1F3FB"},
    {"kvinde_farve2", "1F469-1F3FC"},
    {"kvinde_farve3", "1F469-1F3FD"},
    {"kvinde_farve4", "1F469-1F3FE"},
    {"kvinde_farve5", "1F469-1F3FF"},
    {"familie", "1F46A"},
    {"mand_og_kvinde_hand_i_hand", "1F46B"},
    {"mand_og_kvinde_hand_i_hand_farve1", "1F46B-1F3FB"},
    {"mand_og_kvinde_hand_i_hand_farve2", "1F46B-1F3FC"},
    {"mand_og_kvinde_hand_i_hand_farve3", "1F46B-1F3FD"},
    {"mand_og_kvinde_hand_i_hand_farve4", "1F46B-1F3FE"},
    {"mand_og_kvinde_hand_i_hand_farve5", "1F46B-1F3FF"},
    {"mand_og_kvinde_hand_i_hand_farve1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mand_og_kvinde_hand_i_hand_farve1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mand_og_kvinde_hand_i_hand_farve1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mand_og_kvinde_hand_i_hand_farve1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mand_og_kvinde_hand_i_hand_farve2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mand_og_kvinde_hand_i_hand_farve2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mand_og_kvinde_hand_i_hand_farve2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mand_og_kvinde_hand_i_hand_farve2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mand_og_kvinde_hand_i_hand_farve3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mand_og_kvinde_hand_i_hand_farve3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mand_og_kvinde_hand_i_hand_farve3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mand_og_kvinde_hand_i_hand_farve3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mand_og_kvinde_hand_i_hand_farve4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mand_og_kvinde_hand_i_hand_farve4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mand_og_kvinde_hand_i_hand_farve4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mand_og_kvinde_hand_i_hand_farve4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mand_og_kvinde_hand_i_hand_farve5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mand_og_kvinde_hand_i_hand_farve5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mand_og_kvinde_hand_i_hand_farve5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mand_og_kvinde_hand_i_hand_farve5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"to_maend_hand_i_hand", "1F46C"},
    {"to_maend_hand_i_hand_farve1", "1F46C-1F3FB"},
    {"to_maend_hand_i_hand_farve2", "1F46C-1F3FC"},
    {"to_maend_hand_i_hand_farve3", "1F46C-1F3FD"},
    {"to_maend_hand_i_hand_farve4", "1F46C-1F3FE"},
    {"to_maend_hand_i_hand_farve5", "1F46C-1F3FF"},
    {"to_maend_hand_i_hand_farve1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"to_maend_hand_i_hand_farve1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"to_maend_hand_i_hand_farve1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"to_maend_hand_i_hand_farve1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"to_maend_hand_i_hand_farve2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"to_maend_hand_i_hand_farve2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"to_maend_hand_i_hand_farve2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"to_maend_hand_i_hand_farve2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"to_maend_hand_i_hand_farve3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"to_maend_hand_i_hand_farve3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"to_maend_hand_i_hand_farve3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"to_maend_hand_i_hand_farve3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"to_maend_hand_i_hand_farve4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"to_maend_hand_i_hand_farve4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"to_maend_hand_i_hand_farve4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"to_maend_hand_i_hand_farve4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"to_maend_hand_i_hand_farve5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"to_maend_hand_i_hand_farve5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"to_maend_hand_i_hand_farve5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"to_maend_hand_i_hand_farve5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"to_kvinder_hand_i_hand", "1F46D"},
    {"to_kvinder_hand_i_hand_farve1", "1F46D-1F3FB"},
    {"to_kvinder_hand_i_hand_farve2", "1F46D-1F3FC"},
    {"to_kvinder_hand_i_hand_farve3", "1F46D-1F3FD"},
    {"to_kvinder_hand_i_hand_farve4", "1F46D-1F3FE"},
    {"to_kvinder_hand_i_hand_farve5", "1F46D-1F3FF"},
    {"to_kvinder_hand_i_hand_farve1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"to_kvinder_hand_i_hand_farve1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"to_kvinder_hand_i_hand_farve1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"to_kvinder_hand_i_hand_farve1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"to_kvinder_hand_i_hand_farve2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"to_kvinder_hand_i_hand_farve2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"to_kvinder_hand_i_hand_farve2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"to_kvinder_hand_i_hand_farve2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"to_kvinder_hand_i_hand_farve3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"to_kvinder_hand_i_hand_farve3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"to_kvinder_hand_i_hand_farve3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"to_kvinder_hand_i_hand_farve3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"to_kvinder_hand_i_hand_farve4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"to_kvinder_hand_i_hand_farve4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"to_kvinder_hand_i_hand_farve4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"to_kvinder_hand_i_hand_farve4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"to_kvinder_hand_i_hand_farve5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"to_kvinder_hand_i_hand_farve5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"to_kvinder_hand_i_hand_farve5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"to_kvinder_hand_i_hand_farve5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"politibetjent", "1F46E"},
    {"politibetjent_farve1", "1F46E-1F3FB"},
    {"politibetjent_farve2", "1F46E-1F3FC"},
    {"politibetjent_farve3", "1F46E-1F3FD"},
    {"politibetjent_farve4", "1F46E-1F3FE"},
    {"politibetjent_farve5", "1F46E-1F3FF"},
    {"personer_med_kaninorer", "1F46F"},
    {"personer_med_kaninorer_farve1", "1F46F-1F3FB"},
    {"personer_med_kaninorer_farve2", "1F46F-1F3FC"},
    {"personer_med_kaninorer_farve3", "1F46F-1F3FD"},
    {"personer_med_kaninorer_farve4", "1F46F-1F3FE"},
    {"personer_med_kaninorer_farve5", "1F46F-1F3FF"},
    {"personer_med_kaninorer_farve1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninorer_farve1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninorer_farve1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninorer_farve1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninorer_farve2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninorer_farve2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninorer_farve2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninorer_farve2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninorer_farve3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninorer_farve3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninorer_farve3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninorer_farve3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninorer_farve4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninorer_farve4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninorer_farve4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninorer_farve4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninorer_farve5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninorer_farve5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninorer_farve5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninorer_farve5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"person_med_brudeslor", "1F470"},
    {"person_med_brudeslor_farve1", "1F470-1F3FB"},
    {"person_med_brudeslor_farve2", "1F470-1F3FC"},
    {"person_med_brudeslor_farve3", "1F470-1F3FD"},
    {"person_med_brudeslor_farve4", "1F470-1F3FE"},
    {"person_med_brudeslor_farve5", "1F470-1F3FF"},
    {"blond_person", "1F471"},
    {"blond_person_farve1", "1F471-1F3FB"},
    {"blond_person_farve2", "1F471-1F3FC"},
    {"blond_person_farve3", "1F471-1F3FD"},
    {"blond_person_farve4", "1F471-1F3FE"},
    {"blond_person_farve5", "1F471-1F3FF"},
    {"mand_med_kinesisk_hat", "1F472"},
    {"mand_med_kinesisk_hat_farve1", "1F472-1F3FB"},
    {"mand_med_kinesisk_hat_farve2", "1F472-1F3FC"},
    {"mand_med_kinesisk_hat_farve3", "1F472-1F3FD"},
    {"mand_med_kinesisk_hat_farve4", "1F472-1F3FE"},
    {"mand_med_kinesisk_hat_farve5", "1F472-1F3FF"},
    {"person_med_turban", "1F473"},
    {"person_med_turban_farve1", "1F473-1F3FB"},
    {"person_med_turban_farve2", "1F473-1F3FC"},
    {"person_med_turban_farve3", "1F473-1F3FD"},
    {"person_med_turban_farve4", "1F473-1F3FE"},
    {"person_med_turban_farve5", "1F473-1F3FF"},
    {"gammel_mand", "1F474"},
    {"gammel_mand_farve1", "1F474-1F3FB"},
    {"gammel_mand_farve2", "1F474-1F3FC"},
    {"gammel_mand_farve3", "1F474-1F3FD"},
    {"gammel_mand_farve4", "1F474-1F3FE"},
    {"gammel_mand_farve5", "1F474-1F3FF"},
    {"gammel_kvinde", "1F475"},
    {"gammel_kvinde_farve1", "1F475-1F3FB"},
    {"gammel_kvinde_farve2", "1F475-1F3FC"},
    {"gammel_kvinde_farve3", "1F475-1F3FD"},
    {"gammel_kvinde_farve4", "1F475-1F3FE"},
    {"gammel_kvinde_farve5", "1F475-1F3FF"},
    {"baby", "1F476"},
    {"baby_farve1", "1F476-1F3FB"},
    {"baby_farve2", "1F476-1F3FC"},
    {"baby_farve3", "1F476-1F3FD"},
    {"baby_farve4", "1F476-1F3FE"},
    {"baby_farve5", "1F476-1F3FF"},
    {"bygningsarbejder", "1F477"},
    {"bygningsarbejder_farve1", "1F477-1F3FB"},
    {"bygningsarbejder_farve2", "1F477-1F3FC"},
    {"bygningsarbejder_farve3", "1F477-1F3FD"},
    {"bygningsarbejder_farve4", "1F477-1F3FE"},
    {"bygningsarbejder_farve5", "1F477-1F3FF"},
    {"prinsesse", "1F478"},
    {"prinsesse_farve1", "1F478-1F3FB"},
    {"prinsesse_farve2", "1F478-1F3FC"},
    {"prinsesse_farve3", "1F478-1F3FD"},
    {"prinsesse_farve4", "1F478-1F3FE"},
    {"prinsesse_farve5", "1F478-1F3FF"},
    {"uhyre", "1F479"},
    {"and", "1F986"},
    {"spogelse", "1F47B"},
    {"babyengel", "1F47C"},
    {"babyengel_farve1", "1F47C-1F3FB"},
    {"babyengel_farve2", "1F47C-1F3FC"},
    {"babyengel_farve3", "1F47C-1F3FD"},
    {"babyengel_farve4", "1F47C-1F3FE"},
    {"babyengel_farve5", "1F47C-1F3FF"},
    {"rumvaesen", "1F47D"},
    {"alien_monster", "1F47E"},
    {"sur_lilla_djaevel", "1F47F"},
    {"kranium", "1F480"},
    {"person_vender_handfladen_op", "1F481"},
    {"person_vender_handfladen_op_farve1", "1F481-1F3FB"},
    {"person_vender_handfladen_op_farve2", "1F481-1F3FC"},
    {"person_vender_handfladen_op_farve3", "1F481-1F3FD"},
    {"person_vender_handfladen_op_farve4", "1F481-1F3FE"},
    {"person_vender_handfladen_op_farve5", "1F481-1F3FF"},
    {"garder", "1F482"},
    {"garder_farve1", "1F482-1F3FB"},
    {"garder_farve2", "1F482-1F3FC"},
    {"garder_farve3", "1F482-1F3FD"},
    {"garder_farve4", "1F482-1F3FE"},
    {"garder_farve5", "1F482-1F3FF"},
    {"danser", "1F483"},
    {"danser_farve1", "1F483-1F3FB"},
    {"danser_farve2", "1F483-1F3FC"},
    {"danser_farve3", "1F483-1F3FD"},
    {"danser_farve4", "1F483-1F3FE"},
    {"danser_farve5", "1F483-1F3FF"},
    {"laebestift", "1F484"},
    {"neglelak", "1F485"},
    {"neglelak_farve1", "1F485-1F3FB"},
    {"neglelak_farve2", "1F485-1F3FC"},
    {"neglelak_farve3", "1F485-1F3FD"},
    {"neglelak_farve4", "1F485-1F3FE"},
    {"neglelak_farve5", "1F485-1F3FF"},
    {"ansigtsmassage", "1F486"},
    {"ansigtsmassage_farve1", "1F486-1F3FB"},
    {"ansigtsmassage_farve2", "1F486-1F3FC"},
    {"ansigtsmassage_farve3", "1F486-1F3FD"},
    {"ansigtsmassage_farve4", "1F486-1F3FE"},
    {"ansigtsmassage_farve5", "1F486-1F3FF"},
    {"bliver_klippet", "1F487"},
    {"bliver_klippet_farve1", "1F487-1F3FB"},
    {"bliver_klippet_farve2", "1F487-1F3FC"},
    {"bliver_klippet_farve3", "1F487-1F3FD"},
    {"bliver_klippet_farve4", "1F487-1F3FE"},
    {"bliver_klippet_farve5", "1F487-1F3FF"},
    {"barber", "1F488"},
    {"sprojte", "1F489"},
    {"pille", "1F48A"},
    {"laebeaftryk", "1F48B"},
    {"kaerlighedsbrev", "1F48C"},
    {"ring", "1F48D"},
    {"aedelsten", "1F48E"},
    {"kys", "1F48F"},
    {"kys_farve1", "1F48F-1F3FB"},
    {"kys_farve2", "1F48F-1F3FC"},
    {"kys_farve3", "1F48F-1F3FD"},
    {"kys_farve4", "1F48F-1F3FE"},
    {"kys_farve5", "1F48F-1F3FF"},
    {"kys_farve1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"kys_farve1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"kys_farve1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"kys_farve1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"kys_farve2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"kys_farve2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"kys_farve2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"kys_farve2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"kys_farve3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"kys_farve3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"kys_farve3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"kys_farve3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"kys_farve4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"kys_farve4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"kys_farve4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"kys_farve4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"kys_farve5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"kys_farve5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"kys_farve5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"kys_farve5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"blomsterbuket", "1F490"},
    {"par_med_hjerte", "1F491"},
    {"par_med_hjerte_farve1", "1F491-1F3FB"},
    {"par_med_hjerte_farve2", "1F491-1F3FC"},
    {"par_med_hjerte_farve3", "1F491-1F3FD"},
    {"par_med_hjerte_farve4", "1F491-1F3FE"},
    {"par_med_hjerte_farve5", "1F491-1F3FF"},
    {"par_med_hjerte_farve1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjerte_farve1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjerte_farve1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"par_med_hjerte_farve1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjerte_farve2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjerte_farve2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjerte_farve2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"par_med_hjerte_farve2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjerte_farve3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjerte_farve3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjerte_farve3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"par_med_hjerte_farve3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjerte_farve4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjerte_farve4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjerte_farve4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjerte_farve4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjerte_farve5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjerte_farve5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjerte_farve5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjerte_farve5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"bryllup", "1F492"},
    {"bankende_hjerte", "1F493"},
    {"knust_hjerte", "1F494"},
    {"to_hjerter", "1F495"},
    {"hjerte_med_stjerner", "1F496"},
    {"voksende_hjerte", "1F497"},
    {"hjerte_med_pil", "1F498"},
    {"blat_hjerte", "1F499"},
    {"gront_hjerte", "1F49A"},
    {"gult_hjerte", "1F49B"},
    {"violet_hjerte", "1F49C"},
    {"hjerte_med_band", "1F49D"},
    {"roterende_hjerter", "1F49E"},
    {"hjertedekoration", "1F49F"},
    {"ruder_med_prik", "1F4A0"},
    {"elpaere", "1F4A1"},
    {"vrede", "1F4A2"},
    {"bombe", "1F4A3"},
    {"sovnig", "1F4A4"},
    {"kollision", "1F4A5"},
    {"sveddraber", "1F4A6"},
    {"drabe", "1F4A7"},
    {"smutter", "1F4A8"},
    {"bae", "1F4A9"},
    {"armmuskler", "1F4AA"},
    {"armmuskler_farve1", "1F4AA-1F3FB"},
    {"armmuskler_farve2", "1F4AA-1F3FC"},
    {"armmuskler_farve3", "1F4AA-1F3FD"},
    {"armmuskler_farve4", "1F4AA-1F3FE"},
    {"armmuskler_farve5", "1F4AA-1F3FF"},
    {"svimmel", "1F4AB"},
    {"taleboble", "1F4AC"},
    {"tankeboble", "1F4AD"},
    {"hvid_blomst", "1F4AE"},
    {"100_point", "1F4AF"},
    {"pengepose", "1F4B0"},
    {"valutaveksling", "1F4B1"},
    {"dollartegn", "1F4B2"},
    {"kreditkort", "1F4B3"},
    {"yenseddel", "1F4B4"},
    {"dollarseddel", "1F4B5"},
    {"euroseddel", "1F4B6"},
    {"pundseddel", "1F4B7"},
    {"flyvende_penge", "1F4B8"},
    {"stigende_marked", "1F4B9"},
    {"saede", "1F4BA"},
    {"baerbar_computer", "1F4BB"},
    {"attachemappe", "1F4BC"},
    {"cd_rom", "1F4BD"},
    {"diskette", "1F4BE"},
    {"cd", "1F4BF"},
    {"dvd", "1F4C0"},
    {"mappe", "1F4C1"},
    {"aben_mappe", "1F4C2"},
    {"side_med_fold", "1F4C3"},
    {"dokument", "1F4C4"},
    {"kalender", "1F4C5"},
    {"kalenderblok", "1F4C6"},
    {"kartotek", "1F4C7"},
    {"opadgaende_graf", "1F4C8"},
    {"nedadgaende_graf", "1F4C9"},
    {"sojlediagram", "1F4CA"},
    {"clipboard", "1F4CB"},
    {"tegnestift", "1F4CC"},
    {"rund_tegnestift", "1F4CD"},
    {"papirclips", "1F4CE"},
    {"lineal", "1F4CF"},
    {"tegnetrekant", "1F4D0"},
    {"faner", "1F4D1"},
    {"regnskabsbog", "1F4D2"},
    {"notesbog", "1F4D3"},
    {"notesbog_med_udsmykning", "1F4D4"},
    {"lukket_bog", "1F4D5"},
    {"aben_bog", "1F4D6"},
    {"gron_bog", "1F4D7"},
    {"bla_bog", "1F4D8"},
    {"orange_bog", "1F4D9"},
    {"boger", "1F4DA"},
    {"navneskilt", "1F4DB"},
    {"skriftrulle", "1F4DC"},
    {"notat", "1F4DD"},
    {"telefonror", "1F4DE"},
    {"personsoger", "1F4DF"},
    {"fax", "1F4E0"},
    {"parabolantenne", "1F4E1"},
    {"elektrisk_megafon", "1F4E2"},
    {"megafon", "1F4E3"},
    {"udbakke", "1F4E4"},
    {"indbakke", "1F4E5"},
    {"pakke", "1F4E6"},
    {"mail", "1F4E7"},
    {"indgaende_post", "1F4E8"},
    {"udgaende_post", "1F4E9"},
    {"lukket_postkasse_med_saenket_flag", "1F4EA"},
    {"lukket_postkasse_med_haevet_flag", "1F4EB"},
    {"aben_postkasse_med_haevet_flag", "1F4EC"},
    {"aben_postkasse_med_saenket_flag", "1F4ED"},
    {"postkasse", "1F4EE"},
    {"posthorn", "1F4EF"},
    {"avis", "1F4F0"},
    {"mobil", "1F4F1"},
    {"mobil_med_pil", "1F4F2"},
    {"vibration", "1F4F3"},
    {"mobil_slaet_fra", "1F4F4"},
    {"mobiltelefoner_forbudt", "1F4F5"},
    {"mobilsignal", "1F4F6"},
    {"kamera", "1F4F7"},
    {"kamera_med_blitz", "1F4F8"},
    {"videokamera", "1F4F9"},
    {"tv", "1F4FA"},
    {"radio", "1F4FB"},
    {"videoband", "1F4FC"},
    {"filmprojektor", "1F4FD"},
    {"bedekrans", "1F4FF"},
    {"bland_sange_knap", "1F500"},
    {"gentageknap", "1F501"},
    {"knap_til_en_gentagelse", "1F502"},
    {"lodrette_cirkulaere_pile_med_uret", "1F503"},
    {"vandrette_cirkulaere_pile_mod_uret", "1F504"},
    {"lav_lysstyrke", "1F505"},
    {"hoj_lysstyrke", "1F506"},
    {"hojttaler_fra", "1F507"},
    {"hojttaler_lav_lydstyrke", "1F508"},
    {"hojttaler_mellem_lydstyrke", "1F509"},
    {"hojttaler_hoj_lydstyrke", "1F50A"},
    {"batteri", "1F50B"},
    {"stik", "1F50C"},
    {"forstorrelsesglas_til_venstre", "1F50D"},
    {"forstorrelsesglas_til_hojre", "1F50E"},
    {"lukket_haengelas_med_pen", "1F50F"},
    {"lukket_haengelas_med_nogle", "1F510"},
    {"nogle", "1F511"},
    {"lukket_haengelas", "1F512"},
    {"aben_haengelas", "1F513"},
    {"klokke", "1F514"},
    {"ingen_klokke", "1F515"},
    {"bogmaerke", "1F516"},
    {"link", "1F517"},
    {"alternativknap", "1F518"},
    {"back_pil", "1F519"},
    {"end_pil", "1F51A"},
    {"on_dobbeltpil", "1F51B"},
    {"soon_pil", "1F51C"},
    {"top_pil", "1F51D"},
    {"forbudt_for_under_18", "1F51E"},
    {"bookmarkwater_pistol", "1F51F"},
    {"store_bogstaver", "1F520"},
    {"sma_bogstaver", "1F521"},
    {"tal", "1F522"},
    {"symboler", "1F523"},
    {"latinske_alfabet", "1F524"},
    {"ild", "1F525"},
    {"lommelygte", "1F526"},
    {"skruenogle", "1F527"},
    {"hammer", "1F528"},
    {"motrik_og_bolt", "1F529"},
    {"kokkenkniv", "1F52A"},
    {"vandpistol", "1F52B"},
    {"mikroskop", "1F52C"},
    {"teleskop", "1F52D"},
    {"krystalkugle", "1F52E"},
    {"sekstakket_stjerne_med_prik", "1F52F"},
    {"japansk_begyndersymbol", "1F530"},
    {"trefork", "1F531"},
    {"sort_kvadratisk_knap", "1F532"},
    {"hvid_kvadratisk_knap", "1F533"},
    {"rod_cirkel", "1F534"},
    {"bla_cirkel", "1F535"},
    {"stor_orange_ruder", "1F536"},
    {"stor_bla_ruder", "1F537"},
    {"lille_orange_ruder", "1F538"},
    {"lille_bla_ruder", "1F539"},
    {"op_trekant", "1F53A"},
    {"ned_trekant", "1F53B"},
    {"op_knap", "1F53C"},
    {"ned_knap", "1F53D"},
    {"om", "1F549"},
    {"due", "1F54A"},
    {"kabaen", "1F54B"},
    {"moske", "1F54C"},
    {"synagoge", "1F54D"},
    {"syvarmet_lysestage", "1F54E"},
    {"kl_et", "1F550"},
    {"kl_to", "1F551"},
    {"kl_tre", "1F552"},
    {"kl_fire", "1F553"},
    {"kl_fem", "1F554"},
    {"kl_seks", "1F555"},
    {"kl_syv", "1F556"},
    {"kl_otte", "1F557"},
    {"kl_ni", "1F558"},
    {"kl_ti", "1F559"},
    {"kl_elleve", "1F55A"},
    {"kl_tolv", "1F55B"},
    {"halv_to", "1F55C"},
    {"halv_tre", "1F55D"},
    {"halv_fire", "1F55E"},
    {"halv_fem", "1F55F"},
    {"halv_seks", "1F560"},
    {"halv_syv", "1F561"},
    {"halv_otte", "1F562"},
    {"halv_ni", "1F563"},
    {"halv_ti", "1F564"},
    {"halv_elleve", "1F565"},
    {"halv_tolv", "1F566"},
    {"halv_et", "1F567"},
    {"stearinlys", "1F56F"},
    {"pynteur", "1F570"},
    {"hul", "1F573"},
    {"svaevende_forretningsmand", "1F574"},
    {"svaevende_forretningsmand_farve1", "1F574-1F3FB"},
    {"svaevende_forretningsmand_farve2", "1F574-1F3FC"},
    {"svaevende_forretningsmand_farve3", "1F574-1F3FD"},
    {"svaevende_forretningsmand_farve4", "1F574-1F3FE"},
    {"svaevende_forretningsmand_farve5", "1F574-1F3FF"},
    {"detektiv", "1F575"},
    {"detektiv_farve1", "1F575-1F3FB"},
    {"detektiv_farve2", "1F575-1F3FC"},
    {"detektiv_farve3", "1F575-1F3FD"},
    {"detektiv_farve4", "1F575-1F3FE"},
    {"detektiv_farve5", "1F575-1F3FF"},
    {"solbriller", "1F576"},
    {"edderkop", "1F577"},
    {"edderkoppespind", "1F578"},
    {"joystick", "1F579"},
    {"dansende_mand", "1F57A"},
    {"dansende_mand_farve1", "1F57A-1F3FB"},
    {"dansende_mand_farve2", "1F57A-1F3FC"},
    {"dansende_mand_farve3", "1F57A-1F3FD"},
    {"dansende_mand_farve4", "1F57A-1F3FE"},
    {"dansende_mand_farve5", "1F57A-1F3FF"},
    {"sammensatte_papirclips", "1F587"},
    {"kuglepen", "1F58A"},
    {"fyldepen", "1F58B"},
    {"malerpensel", "1F58C"},
    {"farvekridt", "1F58D"},
    {"hand_med_spredte_fingre", "1F590"},
    {"hand_med_spredte_fingre_farve1", "1F590-1F3FB"},
    {"hand_med_spredte_fingre_farve2", "1F590-1F3FC"},
    {"hand_med_spredte_fingre_farve3", "1F590-1F3FD"},
    {"hand_med_spredte_fingre_farve4", "1F590-1F3FE"},
    {"hand_med_spredte_fingre_farve5", "1F590-1F3FF"},
    {"langefinger", "1F595"},
    {"langefinger_farve1", "1F595-1F3FB"},
    {"langefinger_farve2", "1F595-1F3FC"},
    {"langefinger_farve3", "1F595-1F3FD"},
    {"langefinger_farve4", "1F595-1F3FE"},
    {"langefinger_farve5", "1F595-1F3FF"},
    {"vulcan_hilsen", "1F596"},
    {"vulcan_hilsen_farve1", "1F596-1F3FB"},
    {"vulcan_hilsen_farve2", "1F596-1F3FC"},
    {"vulcan_hilsen_farve3", "1F596-1F3FD"},
    {"vulcan_hilsen_farve4", "1F596-1F3FE"},
    {"vulcan_hilsen_farve5", "1F596-1F3FF"},
    {"sort_hjerte", "1F5A4"},
    {"stationaer_computer", "1F5A5"},
    {"printer", "1F5A8"},
    {"computermus", "1F5B1"},
    {"trackball", "1F5B2"},
    {"ramme_med_billede", "1F5BC"},
    {"kartotekskort", "1F5C2"},
    {"arkivkasse", "1F5C3"},
    {"arkivskab", "1F5C4"},
    {"papirkurv", "1F5D1"},
    {"notesblok", "1F5D2"},
    {"spiralkalender", "1F5D3"},
    {"skruetvinge", "1F5DC"},
    {"gammel_nogle", "1F5DD"},
    {"sammenrullet_avis", "1F5DE"},
    {"dolk", "1F5E1"},
    {"talende_hoved", "1F5E3"},
    {"taleboble_til_venstre", "1F5E8"},
    {"vred_taleboble_til_hojre", "1F5EF"},
    {"valgurne_med_stemmeseddel", "1F5F3"},
    {"verdenskort", "1F5FA"},
    {"fuji_bjerget", "1F5FB"},
    {"tokyo_tower", "1F5FC"},
    {"frihedsgudinden", "1F5FD"},
    {"kort_over_japan", "1F5FE"},
    {"statue", "1F5FF"},
    {"grinende_ansigt", "1F600"},
    {"stralende_ansigt_med_smilende_ojne", "1F601"},
    {"ansigt_med_glaedestarer", "1F602"},
    {"grinende_ansigt_med_store_ojne", "1F603"},
    {"grinende_ansigt_med_smilende_ojne", "1F604"},
    {"grinende_ansigt_med_sveddrabe", "1F605"},
    {"grinende_ansigt_med_lukkede_ojne", "1F606"},
    {"smilende_ansigt_med_glorie", "1F607"},
    {"glad_lilla_djaevel", "1F608"},
    {"blinkende_ansigt", "1F609"},
    {"smilende_ansigt_og_ojne", "1F60A"},
    {"nydende_ansigt", "1F60B"},
    {"lettet_ansigt", "1F60C"},
    {"smilende_ansigt_med_hjerteojne", "1F60D"},
    {"smilende_ansigt_med_solbriller", "1F60E"},
    {"ansigt_med_smorret_grin", "1F60F"},
    {"neutralt_ansigt", "1F610"},
    {"udtrykslost_ansigt", "1F611"},
    {"uimponeret_ansigt", "1F612"},
    {"ansigt_med_koldsved", "1F613"},
    {"eftertaenksomt_ansigt", "1F614"},
    {"forvirret_ansigt", "1F615"},
    {"forbistret_ansigt", "1F616"},
    {"kysseansigt", "1F617"},
    {"kyssende_ansigt", "1F618"},
    {"kysseansigt_med_smilende_ojne", "1F619"},
    {"kysseansigt_med_lukkede_ojne", "1F61A"},
    {"ansigt_med_tungen_ude", "1F61B"},
    {"ansigt_med_tungen_ude_og_blinkende_oje", "1F61C"},
    {"ansigt_med_tungen_ude_og_sammenknebne_ojne", "1F61D"},
    {"skuffet_ansigt", "1F61E"},
    {"bekymret_ansigt", "1F61F"},
    {"vredt_ansigt", "1F620"},
    {"surt_ansigt", "1F621"},
    {"graedende_ansigt", "1F622"},
    {"staedigt_ansigt", "1F623"},
    {"fnysende_ansigt", "1F624"},
    {"skuffet_men_lettet_ansigt", "1F625"},
    {"mabende_ansigt_med_aben_mund", "1F626"},
    {"forpint_ansigt", "1F627"},
    {"bange_ansigt", "1F628"},
    {"udmattet_ansigt", "1F629"},
    {"sovnigt_ansigt", "1F62A"},
    {"traet_ansigt", "1F62B"},
    {"sammenbidt_ansigt", "1F62C"},
    {"stortudende_ansigt", "1F62D"},
    {"ansigt_med_aben_mund", "1F62E"},
    {"tavst_ansigt", "1F62F"},
    {"ansigt_med_aben_mund_og_koldsved", "1F630"},
    {"skrigende_ansigt", "1F631"},
    {"overrasket_ansigt", "1F632"},
    {"rodmende_ansigt", "1F633"},
    {"sovende_ansigt", "1F634"},
    {"svimmelt_ansigt", "1F635"},
    {"ansigt_uden_mund", "1F636"},
    {"ansigt_med_mundbind", "1F637"},
    {"grinende_kat", "1F638"},
    {"kat_med_glaedestarer", "1F639"},
    {"smilende_kat", "1F63A"},
    {"glad_kat_med_hjerteojne", "1F63B"},
    {"kat_med_ironisk_smil", "1F63C"},
    {"kyssende_kat", "1F63D"},
    {"sur_kat", "1F63E"},
    {"graedende_kat", "1F63F"},
    {"overrasket_kat", "1F640"},
    {"lidt_trist_ansigt", "1F641"},
    {"let_smilende_ansigt", "1F642"},
    {"omvendt_ansigt", "1F643"},
    {"ansigt_med_rullende_ojne", "1F644"},
    {"signalerer_nej", "1F645"},
    {"signalerer_nej_farve1", "1F645-1F3FB"},
    {"signalerer_nej_farve2", "1F645-1F3FC"},
    {"signalerer_nej_farve3", "1F645-1F3FD"},
    {"signalerer_nej_farve4", "1F645-1F3FE"},
    {"signalerer_nej_farve5", "1F645-1F3FF"},
    {"signalerer_ok", "1F646"},
    {"signalerer_ok_farve1", "1F646-1F3FB"},
    {"signalerer_ok_farve2", "1F646-1F3FC"},
    {"signalerer_ok_farve3", "1F646-1F3FD"},
    {"signalerer_ok_farve4", "1F646-1F3FE"},
    {"signalerer_ok_farve5", "1F646-1F3FF"},
    {"person_bukker_dybt", "1F647"},
    {"person_bukker_dybt_farve1", "1F647-1F3FB"},
    {"person_bukker_dybt_farve2", "1F647-1F3FC"},
    {"person_bukker_dybt_farve3", "1F647-1F3FD"},
    {"person_bukker_dybt_farve4", "1F647-1F3FE"},
    {"person_bukker_dybt_farve5", "1F647-1F3FF"},
    {"ikke_se", "1F648"},
    {"ikke_hore", "1F649"},
    {"ikke_tale", "1F64A"},
    {"person_med_loftet_hand", "1F64B"},
    {"person_med_loftet_hand_farve1", "1F64B-1F3FB"},
    {"person_med_loftet_hand_farve2", "1F64B-1F3FC"},
    {"person_med_loftet_hand_farve3", "1F64B-1F3FD"},
    {"person_med_loftet_hand_farve4", "1F64B-1F3FE"},
    {"person_med_loftet_hand_farve5", "1F64B-1F3FF"},
    {"loftede_haender", "1F64C"},
    {"loftede_haender_farve1", "1F64C-1F3FB"},
    {"loftede_haender_farve2", "1F64C-1F3FC"},
    {"loftede_haender_farve3", "1F64C-1F3FD"},
    {"loftede_haender_farve4", "1F64C-1F3FE"},
    {"loftede_haender_farve5", "1F64C-1F3FF"},
    {"utilfreds_person", "1F64D"},
    {"utilfreds_person_farve1", "1F64D-1F3FB"},
    {"utilfreds_person_farve2", "1F64D-1F3FC"},
    {"utilfreds_person_farve3", "1F64D-1F3FD"},
    {"utilfreds_person_farve4", "1F64D-1F3FE"},
    {"utilfreds_person_farve5", "1F64D-1F3FF"},
    {"surmulende_person", "1F64E"},
    {"surmulende_person_farve1", "1F64E-1F3FB"},
    {"surmulende_person_farve2", "1F64E-1F3FC"},
    {"surmulende_person_farve3", "1F64E-1F3FD"},
    {"surmulende_person_farve4", "1F64E-1F3FE"},
    {"surmulende_person_farve5", "1F64E-1F3FF"},
    {"foldede_haender", "1F64F"},
    {"foldede_haender_farve1", "1F64F-1F3FB"},
    {"foldede_haender_farve2", "1F64F-1F3FC"},
    {"foldede_haender_farve3", "1F64F-1F3FD"},
    {"foldede_haender_farve4", "1F64F-1F3FE"},
    {"foldede_haender_farve5", "1F64F-1F3FF"},
    {"raket", "1F680"},
    {"helikopter", "1F681"},
    {"lokomotiv", "1F682"},
    {"togvogn", "1F683"},
    {"lyntog", "1F684"},
    {"hojhastighedstog", "1F685"},
    {"tog", "1F686"},
    {"metro", "1F687"},
    {"letbane", "1F688"},
    {"togstation", "1F689"},
    {"sporvogn", "1F68A"},
    {"vogn", "1F68B"},
    {"bus", "1F68C"},
    {"bus_forfra", "1F68D"},
    {"trolleybus", "1F68E"},
    {"busstoppested", "1F68F"},
    {"minibus", "1F690"},
    {"ambulance", "1F691"},
    {"brandbil", "1F692"},
    {"politibil", "1F693"},
    {"politibil_forfra", "1F694"},
    {"taxa", "1F695"},
    {"taxa_forfra", "1F696"},
    {"bil", "1F697"},
    {"bil_forfra", "1F698"},
    {"firehjulstraekker", "1F699"},
    {"lastbil", "1F69A"},
    {"lastvogn", "1F69B"},
    {"traktor", "1F69C"},
    {"monorail", "1F69D"},
    {"bjergbane", "1F69E"},
    {"luftbane", "1F69F"},
    {"kabelbane", "1F6A0"},
    {"svaevebane", "1F6A1"},
    {"skib", "1F6A2"},
    {"roer", "1F6A3"},
    {"roer_farve1", "1F6A3-1F3FB"},
    {"roer_farve2", "1F6A3-1F3FC"},
    {"roer_farve3", "1F6A3-1F3FD"},
    {"roer_farve4", "1F6A3-1F3FE"},
    {"roer_farve5", "1F6A3-1F3FF"},
    {"speedbad", "1F6A4"},
    {"liggende_trafiklys", "1F6A5"},
    {"trafiklys", "1F6A6"},
    {"byggeri", "1F6A7"},
    {"politiblink", "1F6A8"},
    {"trekantet_flag", "1F6A9"},
    {"dor", "1F6AA"},
    {"ingen_adgang", "1F6AB"},
    {"cigaret", "1F6AC"},
    {"rygning_forbudt", "1F6AD"},
    {"brug_affaldsspanden", "1F6AE"},
    {"smid_ikke_affald", "1F6AF"},
    {"drikkevand", "1F6B0"},
    {"ikke_drikkevand", "1F6B1"},
    {"cykel", "1F6B2"},
    {"cykler_forbudt", "1F6B3"},
    {"cyklist", "1F6B4"},
    {"cyklist_farve1", "1F6B4-1F3FB"},
    {"cyklist_farve2", "1F6B4-1F3FC"},
    {"cyklist_farve3", "1F6B4-1F3FD"},
    {"cyklist_farve4", "1F6B4-1F3FE"},
    {"cyklist_farve5", "1F6B4-1F3FF"},
    {"mountainbikerytter", "1F6B5"},
    {"mountainbikerytter_farve1", "1F6B5-1F3FB"},
    {"mountainbikerytter_farve2", "1F6B5-1F3FC"},
    {"mountainbikerytter_farve3", "1F6B5-1F3FD"},
    {"mountainbikerytter_farve4", "1F6B5-1F3FE"},
    {"mountainbikerytter_farve5", "1F6B5-1F3FF"},
    {"fodgaenger", "1F6B6"},
    {"fodgaenger_farve1", "1F6B6-1F3FB"},
    {"fodgaenger_farve2", "1F6B6-1F3FC"},
    {"fodgaenger_farve3", "1F6B6-1F3FD"},
    {"fodgaenger_farve4", "1F6B6-1F3FE"},
    {"fodgaenger_farve5", "1F6B6-1F3FF"},
    {"ingen_fodgaengere", "1F6B7"},
    {"born_krydser_vej", "1F6B8"},
    {"herretoilet", "1F6B9"},
    {"dametoilet", "1F6BA"},
    {"toiletskilt", "1F6BB"},
    {"babysymbol", "1F6BC"},
    {"toilet", "1F6BD"},
    {"wc", "1F6BE"},
    {"brusebad", "1F6BF"},
    {"bad", "1F6C0"},
    {"bad_farve1", "1F6C0-1F3FB"},
    {"bad_farve2", "1F6C0-1F3FC"},
    {"bad_farve3", "1F6C0-1F3FD"},
    {"bad_farve4", "1F6C0-1F3FE"},
    {"bad_farve5", "1F6C0-1F3FF"},
    {"badekar", "1F6C1"},
    {"paskontrol", "1F6C2"},
    {"told", "1F6C3"},
    {"bagageudlevering", "1F6C4"},
    {"bagageopbevaring", "1F6C5"},
    {"sofa_og_lampe", "1F6CB"},
    {"person_i_seng", "1F6CC"},
    {"person_i_seng_farve1", "1F6CC-1F3FB"},
    {"person_i_seng_farve2", "1F6CC-1F3FC"},
    {"person_i_seng_farve3", "1F6CC-1F3FD"},
    {"person_i_seng_farve4", "1F6CC-1F3FE"},
    {"person_i_seng_farve5", "1F6CC-1F3FF"},
    {"indkobsposer", "1F6CD"},
    {"hotelklokke", "1F6CE"},
    {"seng", "1F6CF"},
    {"bedested", "1F6D0"},
    {"stopskilt", "1F6D1"},
    {"indkobsvogn", "1F6D2"},
    {"hindutempel", "1F6D5"},
    {"hytte", "1F6D6"},
    {"elevator", "1F6D7"},
    {"jordskred", "1F6D8"},
    {"tradlos", "1F6DC"},
    {"legeplads_rutsjebane", "1F6DD"},
    {"hjul", "1F6DE"},
    {"redningskrans", "1F6DF"},
    {"hammer_og_skruenogle", "1F6E0"},
    {"skjold", "1F6E1"},
    {"olietonde", "1F6E2"},
    {"motorvej", "1F6E3"},
    {"jernbane", "1F6E4"},
    {"motorbad", "1F6E5"},
    {"lille_fly", "1F6E9"},
    {"flyafgang", "1F6EB"},
    {"flyankomst", "1F6EC"},
    {"satellit", "1F6F0"},
    {"passagerskib", "1F6F3"},
    {"lobehjul", "1F6F4"},
    {"scooter", "1F6F5"},
    {"kano", "1F6F6"},
    {"kaelk", "1F6F7"},
    {"ufo", "1F6F8"},
    {"skateboard", "1F6F9"},
    {"motor_rickshaw", "1F6FA"},
    {"pickup", "1F6FB"},
    {"rulleskojte", "1F6FC"},
    {"orange_cirkel", "1F7E0"},
    {"gul_cirkel", "1F7E1"},
    {"gron_cirkel", "1F7E2"},
    {"lilla_cirkel", "1F7E3"},
    {"brun_cirkel", "1F7E4"},
    {"rodt_kvadrat", "1F7E5"},
    {"blat_kvadrat", "1F7E6"},
    {"orange_kvadrat", "1F7E7"},
    {"gult_kvadrat", "1F7E8"},
    {"gront_kvadrat", "1F7E9"},
    {"lilla_kvadrat", "1F7EA"},
    {"brunt_kvadrat", "1F7EB"},
    {"fedt_lighedstegn", "1F7F0"},
    {"samlede_fingre", "1F90C"},
    {"samlede_fingre_farve1", "1F90C-1F3FB"},
    {"samlede_fingre_farve2", "1F90C-1F3FC"},
    {"samlede_fingre_farve3", "1F90C-1F3FD"},
    {"samlede_fingre_farve4", "1F90C-1F3FE"},
    {"samlede_fingre_farve5", "1F90C-1F3FF"},
    {"hvidt_hjerte", "1F90D"},
    {"brunt_hjerte", "1F90E"},
    {"lidt_vist_med_tommel_og_pegefinger", "1F90F"},
    {"lidt_vist_med_tommel_og_pegefinger_farve1", "1F90F-1F3FB"},
    {"lidt_vist_med_tommel_og_pegefinger_farve2", "1F90F-1F3FC"},
    {"lidt_vist_med_tommel_og_pegefinger_farve3", "1F90F-1F3FD"},
    {"lidt_vist_med_tommel_og_pegefinger_farve4", "1F90F-1F3FE"},
    {"lidt_vist_med_tommel_og_pegefinger_farve5", "1F90F-1F3FF"},
    {"ansigt_med_lynlasmund", "1F910"},
    {"ansigt_med_dollartegn", "1F911"},
    {"ansigt_med_termometer", "1F912"},
    {"nordansigt", "1F913"},
    {"taenkende_ansigt", "1F914"},
    {"ansigt_med_bandage", "1F915"},
    {"robot", "1F916"},
    {"krammende_ansigt", "1F917"},
    {"djaevletegn", "1F918"},
    {"djaevletegn_farve1", "1F918-1F3FB"},
    {"djaevletegn_farve2", "1F918-1F3FC"},
    {"djaevletegn_farve3", "1F918-1F3FD"},
    {"djaevletegn_farve4", "1F918-1F3FE"},
    {"djaevletegn_farve5", "1F918-1F3FF"},
    {"ring_til_mig_hand", "1F919"},
    {"ring_til_mig_hand_farve1", "1F919-1F3FB"},
    {"ring_til_mig_hand_farve2", "1F919-1F3FC"},
    {"ring_til_mig_hand_farve3", "1F919-1F3FD"},
    {"ring_til_mig_hand_farve4", "1F919-1F3FE"},
    {"ring_til_mig_hand_farve5", "1F919-1F3FF"},
    {"loftet_handryg", "1F91A"},
    {"loftet_handryg_farve1", "1F91A-1F3FB"},
    {"loftet_handryg_farve2", "1F91A-1F3FC"},
    {"loftet_handryg_farve3", "1F91A-1F3FD"},
    {"loftet_handryg_farve4", "1F91A-1F3FE"},
    {"loftet_handryg_farve5", "1F91A-1F3FF"},
    {"venstrevendt_knytnaeve", "1F91B"},
    {"venstrevendt_knytnaeve_farve1", "1F91B-1F3FB"},
    {"venstrevendt_knytnaeve_farve2", "1F91B-1F3FC"},
    {"venstrevendt_knytnaeve_farve3", "1F91B-1F3FD"},
    {"venstrevendt_knytnaeve_farve4", "1F91B-1F3FE"},
    {"venstrevendt_knytnaeve_farve5", "1F91B-1F3FF"},
    {"hojrevendt_knytnaeve", "1F91C"},
    {"hojrevendt_knytnaeve_farve1", "1F91C-1F3FB"},
    {"hojrevendt_knytnaeve_farve2", "1F91C-1F3FC"},
    {"hojrevendt_knytnaeve_farve3", "1F91C-1F3FD"},
    {"hojrevendt_knytnaeve_farve4", "1F91C-1F3FE"},
    {"hojrevendt_knytnaeve_farve5", "1F91C-1F3FF"},
    {"handtryk", "1F91D"},
    {"handtryk_farve1", "1F91D-1F3FB"},
    {"handtryk_farve2", "1F91D-1F3FC"},
    {"handtryk_farve3", "1F91D-1F3FD"},
    {"handtryk_farve4", "1F91D-1F3FE"},
    {"handtryk_farve5", "1F91D-1F3FF"},
    {"handtryk_farve1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"handtryk_farve1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"handtryk_farve1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"handtryk_farve1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"handtryk_farve2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"handtryk_farve2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"handtryk_farve2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"handtryk_farve2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"handtryk_farve3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"handtryk_farve3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"handtryk_farve3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"handtryk_farve3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"handtryk_farve4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"handtryk_farve4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"handtryk_farve4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"handtryk_farve4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"handtryk_farve5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"handtryk_farve5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"handtryk_farve5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"handtryk_farve5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"krydsede_fingre", "1F91E"},
    {"krydsede_fingre_farve1", "1F91E-1F3FB"},
    {"krydsede_fingre_farve2", "1F91E-1F3FC"},
    {"krydsede_fingre_farve3", "1F91E-1F3FD"},
    {"krydsede_fingre_farve4", "1F91E-1F3FE"},
    {"krydsede_fingre_farve5", "1F91E-1F3FF"},
    {"elsker_dig_gestus", "1F91F"},
    {"elsker_dig_gestus_farve1", "1F91F-1F3FB"},
    {"elsker_dig_gestus_farve2", "1F91F-1F3FC"},
    {"elsker_dig_gestus_farve3", "1F91F-1F3FD"},
    {"elsker_dig_gestus_farve4", "1F91F-1F3FE"},
    {"elsker_dig_gestus_farve5", "1F91F-1F3FF"},
    {"ansigt_med_cowboyhat", "1F920"},
    {"klovneansigt", "1F921"},
    {"ansigt_med_kvalme", "1F922"},
    {"ruller_rundt_pa_gulvet_af_grin", "1F923"},
    {"savlende_ansigt", "1F924"},
    {"logneransigt", "1F925"},
    {"sla_sig_for_panden", "1F926"},
    {"sla_sig_for_panden_farve1", "1F926-1F3FB"},
    {"sla_sig_for_panden_farve2", "1F926-1F3FC"},
    {"sla_sig_for_panden_farve3", "1F926-1F3FD"},
    {"sla_sig_for_panden_farve4", "1F926-1F3FE"},
    {"sla_sig_for_panden_farve5", "1F926-1F3FF"},
    {"nysende_ansigt", "1F927"},
    {"ansigt_med_loftet_ojenbryn", "1F928"},
    {"benovet", "1F929"},
    {"skort_ansigt", "1F92A"},
    {"tyssende_ansigt", "1F92B"},
    {"ansigt_med_symboler_pa_munden", "1F92C"},
    {"ansigt_med_hand_over_munden", "1F92D"},
    {"ansigt_der_kaster_op", "1F92E"},
    {"hoved_der_eksploderer", "1F92F"},
    {"gravid_kvinde", "1F930"},
    {"gravid_kvinde_farve1", "1F930-1F3FB"},
    {"gravid_kvinde_farve2", "1F930-1F3FC"},
    {"gravid_kvinde_farve3", "1F930-1F3FD"},
    {"gravid_kvinde_farve4", "1F930-1F3FE"},
    {"gravid_kvinde_farve5", "1F930-1F3FF"},
    {"kvinde_der_ammer", "1F931"},
    {"kvinde_der_ammer_farve1", "1F931-1F3FB"},
    {"kvinde_der_ammer_farve2", "1F931-1F3FC"},
    {"kvinde_der_ammer_farve3", "1F931-1F3FD"},
    {"kvinde_der_ammer_farve4", "1F931-1F3FE"},
    {"kvinde_der_ammer_farve5", "1F931-1F3FF"},
    {"handflader_samlet_og_loftet", "1F932"},
    {"handflader_samlet_og_loftet_farve1", "1F932-1F3FB"},
    {"handflader_samlet_og_loftet_farve2", "1F932-1F3FC"},
    {"handflader_samlet_og_loftet_farve3", "1F932-1F3FD"},
    {"handflader_samlet_og_loftet_farve4", "1F932-1F3FE"},
    {"handflader_samlet_og_loftet_farve5", "1F932-1F3FF"},
    {"selfie", "1F933"},
    {"selfie_farve1", "1F933-1F3FB"},
    {"selfie_farve2", "1F933-1F3FC"},
    {"selfie_farve3", "1F933-1F3FD"},
    {"selfie_farve4", "1F933-1F3FE"},
    {"selfie_farve5", "1F933-1F3FF"},
    {"prins", "1F934"},
    {"prins_farve1", "1F934-1F3FB"},
    {"prins_farve2", "1F934-1F3FC"},
    {"prins_farve3", "1F934-1F3FD"},
    {"prins_farve4", "1F934-1F3FE"},
    {"prins_farve5", "1F934-1F3FF"},
    {"person_i_smoking", "1F935"},
    {"person_i_smoking_farve1", "1F935-1F3FB"},
    {"person_i_smoking_farve2", "1F935-1F3FC"},
    {"person_i_smoking_farve3", "1F935-1F3FD"},
    {"person_i_smoking_farve4", "1F935-1F3FE"},
    {"person_i_smoking_farve5", "1F935-1F3FF"},
    {"julemandens_kone", "1F936"},
    {"julemandens_kone_farve1", "1F936-1F3FB"},
    {"julemandens_kone_farve2", "1F936-1F3FC"},
    {"julemandens_kone_farve3", "1F936-1F3FD"},
    {"julemandens_kone_farve4", "1F936-1F3FE"},
    {"julemandens_kone_farve5", "1F936-1F3FF"},
    {"traekker_pa_skuldrene", "1F937"},
    {"traekker_pa_skuldrene_farve1", "1F937-1F3FB"},
    {"traekker_pa_skuldrene_farve2", "1F937-1F3FC"},
    {"traekker_pa_skuldrene_farve3", "1F937-1F3FD"},
    {"traekker_pa_skuldrene_farve4", "1F937-1F3FE"},
    {"traekker_pa_skuldrene_farve5", "1F937-1F3FF"},
    {"person_slar_vejrmolle", "1F938"},
    {"person_slar_vejrmolle_farve1", "1F938-1F3FB"},
    {"person_slar_vejrmolle_farve2", "1F938-1F3FC"},
    {"person_slar_vejrmolle_farve3", "1F938-1F3FD"},
    {"person_slar_vejrmolle_farve4", "1F938-1F3FE"},
    {"person_slar_vejrmolle_farve5", "1F938-1F3FF"},
    {"jonglor", "1F939"},
    {"jonglor_farve1", "1F939-1F3FB"},
    {"jonglor_farve2", "1F939-1F3FC"},
    {"jonglor_farve3", "1F939-1F3FD"},
    {"jonglor_farve4", "1F939-1F3FE"},
    {"jonglor_farve5", "1F939-1F3FF"},
    {"faegter", "1F93A"},
    {"brydere", "1F93C"},
    {"brydere_farve1", "1F93C-1F3FB"},
    {"brydere_farve2", "1F93C-1F3FC"},
    {"brydere_farve3", "1F93C-1F3FD"},
    {"brydere_farve4", "1F93C-1F3FE"},
    {"brydere_farve5", "1F93C-1F3FF"},
    {"brydere_farve1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"brydere_farve1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"brydere_farve1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"brydere_farve1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"brydere_farve2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"brydere_farve2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"brydere_farve2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"brydere_farve2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"brydere_farve3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"brydere_farve3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"brydere_farve3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"brydere_farve3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"brydere_farve4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"brydere_farve4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"brydere_farve4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"brydere_farve4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"brydere_farve5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"brydere_farve5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"brydere_farve5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"brydere_farve5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"vandpolospiller", "1F93D"},
    {"vandpolospiller_farve1", "1F93D-1F3FB"},
    {"vandpolospiller_farve2", "1F93D-1F3FC"},
    {"vandpolospiller_farve3", "1F93D-1F3FD"},
    {"vandpolospiller_farve4", "1F93D-1F3FE"},
    {"vandpolospiller_farve5", "1F93D-1F3FF"},
    {"handboldspiller", "1F93E"},
    {"handboldspiller_farve1", "1F93E-1F3FB"},
    {"handboldspiller_farve2", "1F93E-1F3FC"},
    {"handboldspiller_farve3", "1F93E-1F3FD"},
    {"handboldspiller_farve4", "1F93E-1F3FE"},
    {"handboldspiller_farve5", "1F93E-1F3FF"},
    {"dykkermaske", "1F93F"},
    {"vissen_blomst", "1F940"},
    {"tromme", "1F941"},
    {"skal", "1F942"},
    {"fladbundet_glas", "1F943"},
    {"ske", "1F944"},
    {"malnet", "1F945"},
    {"guldmedalje", "1F947"},
    {"solvmedalje", "1F948"},
    {"bronzemedalje", "1F949"},
    {"boksehandske", "1F94A"},
    {"karatedragt", "1F94B"},
    {"curlingsten", "1F94C"},
    {"lacrosse", "1F94D"},
    {"softball", "1F94E"},
    {"frisbee", "1F94F"},
    {"croissant", "1F950"},
    {"avokado", "1F951"},
    {"agurk", "1F952"},
    {"bacon", "1F953"},
    {"kartoffel", "1F954"},
    {"gulerod", "1F955"},
    {"flute", "1F956"},
    {"gron_salat", "1F957"},
    {"pande_med_mad", "1F958"},
    {"fyldt_pitabrod", "1F959"},
    {"aeg", "1F95A"},
    {"glas_maelk", "1F95B"},
    {"jordnodder", "1F95C"},
    {"kiwifrugt", "1F95D"},
    {"pandekager", "1F95E"},
    {"dumpling", "1F95F"},
    {"lykkekage", "1F960"},
    {"kinesisk_takeaway", "1F961"},
    {"spisepinde", "1F962"},
    {"ske_i_skal", "1F963"},
    {"baeger_med_sugeror", "1F964"},
    {"kokosnod", "1F965"},
    {"broccoli", "1F966"},
    {"taerte", "1F967"},
    {"kringle", "1F968"},
    {"kodudskaering", "1F969"},
    {"sandwich", "1F96A"},
    {"konserves", "1F96B"},
    {"bladgront", "1F96C"},
    {"mango", "1F96D"},
    {"manekage", "1F96E"},
    {"bagel", "1F96F"},
    {"smilende_ansigt_med_tre_hjerter", "1F970"},
    {"gabende_ansigt", "1F971"},
    {"smilende_ansigt_med_tare", "1F972"},
    {"festansigt", "1F973"},
    {"ort_ansigt", "1F974"},
    {"svedende_ansigt", "1F975"},
    {"frysende_ansigt", "1F976"},
    {"ninja", "1F977"},
    {"ninja_farve1", "1F977-1F3FB"},
    {"ninja_farve2", "1F977-1F3FC"},
    {"ninja_farve3", "1F977-1F3FD"},
    {"ninja_farve4", "1F977-1F3FE"},
    {"ninja_farve5", "1F977-1F3FF"},
    {"maske", "1F978"},
    {"ansigt_der_holder_tarer_tilbage", "1F979"},
    {"bonfaldende_ansigt", "1F97A"},
    {"sari", "1F97B"},
    {"kittel", "1F97C"},
    {"sikkerhedsbriller", "1F97D"},
    {"vandrestovle", "1F97E"},
    {"flad_sko", "1F97F"},
    {"krabbe", "1F980"},
    {"lovehoved", "1F981"},
    {"skorpion", "1F982"},
    {"kalkun", "1F983"},
    {"enhjorningehoved", "1F984"},
    {"orn", "1F985"},
    {"flagermus", "1F987"},
    {"haj", "1F988"},
    {"ugle", "1F989"},
    {"raevehoved", "1F98A"},
    {"sommerfugl", "1F98B"},
    {"hjort", "1F98C"},
    {"gorilla", "1F98D"},
    {"firben", "1F98E"},
    {"naesehorn", "1F98F"},
    {"reje", "1F990"},
    {"sprutte", "1F991"},
    {"giraf", "1F992"},
    {"zebra", "1F993"},
    {"pindsvin", "1F994"},
    {"sauropod", "1F995"},
    {"trex", "1F996"},
    {"farekylling", "1F997"},
    {"kaenguru", "1F998"},
    {"lama", "1F999"},
    {"pafugl", "1F99A"},
    {"flodhest", "1F99B"},
    {"papegoje", "1F99C"},
    {"vaskebjorn", "1F99D"},
    {"hummer", "1F99E"},
    {"myg", "1F99F"},
    {"mikrobe", "1F9A0"},
    {"graevling", "1F9A1"},
    {"svane", "1F9A2"},
    {"mammut", "1F9A3"},
    {"dronte", "1F9A4"},
    {"dovendyr", "1F9A5"},
    {"odder", "1F9A6"},
    {"orangutang", "1F9A7"},
    {"stinkdyr", "1F9A8"},
    {"flamingo", "1F9A9"},
    {"osters", "1F9AA"},
    {"baever", "1F9AB"},
    {"bison", "1F9AC"},
    {"sael", "1F9AD"},
    {"forerhund", "1F9AE"},
    {"blindestok", "1F9AF"},
    {"rodt_har", "1F9B0"},
    {"krollet_har", "1F9B1"},
    {"skaldet", "1F9B2"},
    {"hvidt_har", "1F9B3"},
    {"knogle", "1F9B4"},
    {"ben", "1F9B5"},
    {"ben_farve1", "1F9B5-1F3FB"},
    {"ben_farve2", "1F9B5-1F3FC"},
    {"ben_farve3", "1F9B5-1F3FD"},
    {"ben_farve4", "1F9B5-1F3FE"},
    {"ben_farve5", "1F9B5-1F3FF"},
    {"fod", "1F9B6"},
    {"fod_farve1", "1F9B6-1F3FB"},
    {"fod_farve2", "1F9B6-1F3FC"},
    {"fod_farve3", "1F9B6-1F3FD"},
    {"fod_farve4", "1F9B6-1F3FE"},
    {"fod_farve5", "1F9B6-1F3FF"},
    {"tand", "1F9B7"},
    {"superhelt", "1F9B8"},
    {"superhelt_farve1", "1F9B8-1F3FB"},
    {"superhelt_farve2", "1F9B8-1F3FC"},
    {"superhelt_farve3", "1F9B8-1F3FD"},
    {"superhelt_farve4", "1F9B8-1F3FE"},
    {"superhelt_farve5", "1F9B8-1F3FF"},
    {"superskurk", "1F9B9"},
    {"superskurk_farve1", "1F9B9-1F3FB"},
    {"superskurk_farve2", "1F9B9-1F3FC"},
    {"superskurk_farve3", "1F9B9-1F3FD"},
    {"superskurk_farve4", "1F9B9-1F3FE"},
    {"superskurk_farve5", "1F9B9-1F3FF"},
    {"sikkerhedsvest", "1F9BA"},
    {"ore_med_horeapparat", "1F9BB"},
    {"ore_med_horeapparat_farve1", "1F9BB-1F3FB"},
    {"ore_med_horeapparat_farve2", "1F9BB-1F3FC"},
    {"ore_med_horeapparat_farve3", "1F9BB-1F3FD"},
    {"ore_med_horeapparat_farve4", "1F9BB-1F3FE"},
    {"ore_med_horeapparat_farve5", "1F9BB-1F3FF"},
    {"elektrisk_korestol", "1F9BC"},
    {"manuel_korestol", "1F9BD"},
    {"mekanisk_arm", "1F9BE"},
    {"mekanisk_ben", "1F9BF"},
    {"et_stykke_ost", "1F9C0"},
    {"cupcake", "1F9C1"},
    {"salt", "1F9C2"},
    {"brik", "1F9C3"},
    {"hvidlog", "1F9C4"},
    {"log", "1F9C5"},
    {"falafel", "1F9C6"},
    {"vaffel", "1F9C7"},
    {"smor", "1F9C8"},
    {"mate", "1F9C9"},
    {"isterning", "1F9CA"},
    {"bubble_tea", "1F9CB"},
    {"trold", "1F9CC"},
    {"staende_person", "1F9CD"},
    {"staende_person_farve1", "1F9CD-1F3FB"},
    {"staende_person_farve2", "1F9CD-1F3FC"},
    {"staende_person_farve3", "1F9CD-1F3FD"},
    {"staende_person_farve4", "1F9CD-1F3FE"},
    {"staende_person_farve5", "1F9CD-1F3FF"},
    {"knaelende_person", "1F9CE"},
    {"knaelende_person_farve1", "1F9CE-1F3FB"},
    {"knaelende_person_farve2", "1F9CE-1F3FC"},
    {"knaelende_person_farve3", "1F9CE-1F3FD"},
    {"knaelende_person_farve4", "1F9CE-1F3FE"},
    {"knaelende_person_farve5", "1F9CE-1F3FF"},
    {"dov_person", "1F9CF"},
    {"dov_person_farve1", "1F9CF-1F3FB"},
    {"dov_person_farve2", "1F9CF-1F3FC"},
    {"dov_person_farve3", "1F9CF-1F3FD"},
    {"dov_person_farve4", "1F9CF-1F3FE"},
    {"dov_person_farve5", "1F9CF-1F3FF"},
    {"ansigt_med_monokel", "1F9D0"},
    {"voksen", "1F9D1"},
    {"voksen_farve1", "1F9D1-1F3FB"},
    {"voksen_farve2", "1F9D1-1F3FC"},
    {"voksen_farve3", "1F9D1-1F3FD"},
    {"voksen_farve4", "1F9D1-1F3FE"},
    {"voksen_farve5", "1F9D1-1F3FF"},
    {"barn", "1F9D2"},
    {"barn_farve1", "1F9D2-1F3FB"},
    {"barn_farve2", "1F9D2-1F3FC"},
    {"barn_farve3", "1F9D2-1F3FD"},
    {"barn_farve4", "1F9D2-1F3FE"},
    {"barn_farve5", "1F9D2-1F3FF"},
    {"aeldre_menneske", "1F9D3"},
    {"aeldre_menneske_farve1", "1F9D3-1F3FB"},
    {"aeldre_menneske_farve2", "1F9D3-1F3FC"},
    {"aeldre_menneske_farve3", "1F9D3-1F3FD"},
    {"aeldre_menneske_farve4", "1F9D3-1F3FE"},
    {"aeldre_menneske_farve5", "1F9D3-1F3FF"},
    {"person_med_skaeg", "1F9D4"},
    {"person_med_skaeg_farve1", "1F9D4-1F3FB"},
    {"person_med_skaeg_farve2", "1F9D4-1F3FC"},
    {"person_med_skaeg_farve3", "1F9D4-1F3FD"},
    {"person_med_skaeg_farve4", "1F9D4-1F3FE"},
    {"person_med_skaeg_farve5", "1F9D4-1F3FF"},
    {"kvinde_med_torklaede", "1F9D5"},
    {"kvinde_med_torklaede_farve1", "1F9D5-1F3FB"},
    {"kvinde_med_torklaede_farve2", "1F9D5-1F3FC"},
    {"kvinde_med_torklaede_farve3", "1F9D5-1F3FD"},
    {"kvinde_med_torklaede_farve4", "1F9D5-1F3FE"},
    {"kvinde_med_torklaede_farve5", "1F9D5-1F3FF"},
    {"person_i_dampbad", "1F9D6"},
    {"person_i_dampbad_farve1", "1F9D6-1F3FB"},
    {"person_i_dampbad_farve2", "1F9D6-1F3FC"},
    {"person_i_dampbad_farve3", "1F9D6-1F3FD"},
    {"person_i_dampbad_farve4", "1F9D6-1F3FE"},
    {"person_i_dampbad_farve5", "1F9D6-1F3FF"},
    {"bjergbestiger", "1F9D7"},
    {"bjergbestiger_farve1", "1F9D7-1F3FB"},
    {"bjergbestiger_farve2", "1F9D7-1F3FC"},
    {"bjergbestiger_farve3", "1F9D7-1F3FD"},
    {"bjergbestiger_farve4", "1F9D7-1F3FE"},
    {"bjergbestiger_farve5", "1F9D7-1F3FF"},
    {"lotusstilling", "1F9D8"},
    {"lotusstilling_farve1", "1F9D8-1F3FB"},
    {"lotusstilling_farve2", "1F9D8-1F3FC"},
    {"lotusstilling_farve3", "1F9D8-1F3FD"},
    {"lotusstilling_farve4", "1F9D8-1F3FE"},
    {"lotusstilling_farve5", "1F9D8-1F3FF"},
    {"magiker", "1F9D9"},
    {"magiker_farve1", "1F9D9-1F3FB"},
    {"magiker_farve2", "1F9D9-1F3FC"},
    {"magiker_farve3", "1F9D9-1F3FD"},
    {"magiker_farve4", "1F9D9-1F3FE"},
    {"magiker_farve5", "1F9D9-1F3FF"},
    {"fe", "1F9DA"},
    {"fe_farve1", "1F9DA-1F3FB"},
    {"fe_farve2", "1F9DA-1F3FC"},
    {"fe_farve3", "1F9DA-1F3FD"},
    {"fe_farve4", "1F9DA-1F3FE"},
    {"fe_farve5", "1F9DA-1F3FF"},
    {"vampyr", "1F9DB"},
    {"vampyr_farve1", "1F9DB-1F3FB"},
    {"vampyr_farve2", "1F9DB-1F3FC"},
    {"vampyr_farve3", "1F9DB-1F3FD"},
    {"vampyr_farve4", "1F9DB-1F3FE"},
    {"vampyr_farve5", "1F9DB-1F3FF"},
    {"havvaesen", "1F9DC"},
    {"havvaesen_farve1", "1F9DC-1F3FB"},
    {"havvaesen_farve2", "1F9DC-1F3FC"},
    {"havvaesen_farve3", "1F9DC-1F3FD"},
    {"havvaesen_farve4", "1F9DC-1F3FE"},
    {"havvaesen_farve5", "1F9DC-1F3FF"},
    {"elver", "1F9DD"},
    {"elver_farve1", "1F9DD-1F3FB"},
    {"elver_farve2", "1F9DD-1F3FC"},
    {"elver_farve3", "1F9DD-1F3FD"},
    {"elver_farve4", "1F9DD-1F3FE"},
    {"elver_farve5", "1F9DD-1F3FF"},
    {"magisk_and", "1F9DE"},
    {"zombie", "1F9DF"},
    {"hjerne", "1F9E0"},
    {"orange_hjerte", "1F9E1"},
    {"kasket", "1F9E2"},
    {"halstorklaede", "1F9E3"},
    {"handsker", "1F9E4"},
    {"frakke", "1F9E5"},
    {"sokker", "1F9E6"},
    {"pengegave", "1F9E7"},
    {"kanonslag", "1F9E8"},
    {"brik_til_puslespil", "1F9E9"},
    {"reagensglas", "1F9EA"},
    {"petriskal", "1F9EB"},
    {"dna", "1F9EC"},
    {"kompas", "1F9ED"},
    {"kugleramme", "1F9EE"},
    {"brandslukker", "1F9EF"},
    {"vaerktojskasse", "1F9F0"},
    {"mursten", "1F9F1"},
    {"magnet", "1F9F2"},
    {"bagage", "1F9F3"},
    {"bodylotion", "1F9F4"},
    {"trad", "1F9F5"},
    {"garn", "1F9F6"},
    {"sikkerhedsnal", "1F9F7"},
    {"bamse", "1F9F8"},
    {"kost", "1F9F9"},
    {"kurv", "1F9FA"},
    {"papirrulle", "1F9FB"},
    {"saebe", "1F9FC"},
    {"vaskesvamp", "1F9FD"},
    {"kvittering", "1F9FE"},
    {"nazaramulet", "1F9FF"},
    {"balletsko", "1FA70"},
    {"badedragt", "1FA71"},
    {"badebukser", "1FA72"},
    {"shorts", "1FA73"},
    {"klipklapper", "1FA74"},
    {"lyseblat_hjerte", "1FA75"},
    {"grat_hjerte", "1FA76"},
    {"lyserodt_hjerte", "1FA77"},
    {"bloddrabe", "1FA78"},
    {"plaster", "1FA79"},
    {"stetoskop", "1FA7A"},
    {"rontgen", "1FA7B"},
    {"krykke", "1FA7C"},
    {"yoyo", "1FA80"},
    {"legetojsdrage", "1FA81"},
    {"faldskaerm", "1FA82"},
    {"boomerang", "1FA83"},
    {"tryllestav", "1FA84"},
    {"pinata", "1FA85"},
    {"russiske_dukker", "1FA86"},
    {"maracas", "1FA87"},
    {"flojte", "1FA88"},
    {"harpe", "1FA89"},
    {"basun", "1FA8A"},
    {"skattekiste", "1FA8E"},
    {"skovl", "1FA8F"},
    {"planet_med_ringe", "1FA90"},
    {"stol", "1FA91"},
    {"ragekniv", "1FA92"},
    {"diya_lampe", "1FA94"},
    {"banjo", "1FA95"},
    {"militaerhjelm", "1FA96"},
    {"akkordeon", "1FA97"},
    {"hoj_tromme", "1FA98"},
    {"mont", "1FA99"},
    {"sav", "1FA9A"},
    {"skruetraekker", "1FA9B"},
    {"stige", "1FA9C"},
    {"krog", "1FA9D"},
    {"spejl", "1FA9E"},
    {"vindue", "1FA9F"},
    {"svupper", "1FAA0"},
    {"synal", "1FAA1"},
    {"knude", "1FAA2"},
    {"spand", "1FAA3"},
    {"musefaelde", "1FAA4"},
    {"tandborste", "1FAA5"},
    {"gravsten", "1FAA6"},
    {"skilt", "1FAA7"},
    {"sten", "1FAA8"},
    {"diskokugle", "1FAA9"},
    {"legitimationskort", "1FAAA"},
    {"lavt_batteriniveau", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"sammenklappelig_handvifte", "1FAAD"},
    {"afrokam", "1FAAE"},
    {"khanda", "1FAAF"},
    {"flue", "1FAB0"},
    {"orm", "1FAB1"},
    {"bille", "1FAB2"},
    {"kakerlak", "1FAB3"},
    {"potteplante", "1FAB4"},
    {"trae", "1FAB5"},
    {"fjer", "1FAB6"},
    {"lotus", "1FAB7"},
    {"koral", "1FAB8"},
    {"tom_rede", "1FAB9"},
    {"rede_med_aeg", "1FABA"},
    {"hyacint", "1FABB"},
    {"gople", "1FABC"},
    {"vinge", "1FABD"},
    {"trae_uden_blade", "1FABE"},
    {"gas", "1FABF"},
    {"hjerte", "1FAC0"},
    {"lunger", "1FAC1"},
    {"knus", "1FAC2"},
    {"gravid_mand", "1FAC3"},
    {"gravid_mand_farve1", "1FAC3-1F3FB"},
    {"gravid_mand_farve2", "1FAC3-1F3FC"},
    {"gravid_mand_farve3", "1FAC3-1F3FD"},
    {"gravid_mand_farve4", "1FAC3-1F3FE"},
    {"gravid_mand_farve5", "1FAC3-1F3FF"},
    {"gravid_person", "1FAC4"},
    {"gravid_person_farve1", "1FAC4-1F3FB"},
    {"gravid_person_farve2", "1FAC4-1F3FC"},
    {"gravid_person_farve3", "1FAC4-1F3FD"},
    {"gravid_person_farve4", "1FAC4-1F3FE"},
    {"gravid_person_farve5", "1FAC4-1F3FF"},
    {"person_med_krone", "1FAC5"},
    {"person_med_krone_farve1", "1FAC5-1F3FB"},
    {"person_med_krone_farve2", "1FAC5-1F3FC"},
    {"person_med_krone_farve3", "1FAC5-1F3FD"},
    {"person_med_krone_farve4", "1FAC5-1F3FE"},
    {"person_med_krone_farve5", "1FAC5-1F3FF"},
    {"fingeraftryk", "1FAC6"},
    {"beharet_vaesen", "1FAC8"},
    {"spaekhugger", "1FACD"},
    {"elg", "1FACE"},
    {"aesel", "1FACF"},
    {"blabaer", "1FAD0"},
    {"peberfrugt", "1FAD1"},
    {"oliven", "1FAD2"},
    {"fladbrod", "1FAD3"},
    {"tamales", "1FAD4"},
    {"fondue", "1FAD5"},
    {"tekande", "1FAD6"},
    {"haelde_vand", "1FAD7"},
    {"bonner", "1FAD8"},
    {"krukke", "1FAD9"},
    {"ingefaerrod", "1FADA"},
    {"aertebaelg", "1FADB"},
    {"rodfrugt", "1FADC"},
    {"klat", "1FADF"},
    {"smeltende_ansigt", "1FAE0"},
    {"ansigt_der_gor_honnor", "1FAE1"},
    {"ansigt_med_abne_ojne_og_handen_over_munden", "1FAE2"},
    {"ansigt_der_smugkigger", "1FAE3"},
    {"ansigt_med_skra_mund", "1FAE4"},
    {"stiplet_linje_ansigt", "1FAE5"},
    {"bider_i_laeben", "1FAE6"},
    {"bobler", "1FAE7"},
    {"rystende_ansigt", "1FAE8"},
    {"ansigt_med_poser_under_ojnene", "1FAE9"},
    {"forvraenget_ansigt", "1FAEA"},
    {"slagsmalssky", "1FAEF"},
    {"krydset_pegefinger_og_tommelfinger", "1FAF0"},
    {"krydset_pegefinger_og_tommelfinger_farve1", "1FAF0-1F3FB"},
    {"krydset_pegefinger_og_tommelfinger_farve2", "1FAF0-1F3FC"},
    {"krydset_pegefinger_og_tommelfinger_farve3", "1FAF0-1F3FD"},
    {"krydset_pegefinger_og_tommelfinger_farve4", "1FAF0-1F3FE"},
    {"krydset_pegefinger_og_tommelfinger_farve5", "1FAF0-1F3FF"},
    {"hand_til_hojre", "1FAF1"},
    {"hand_til_hojre_farve1", "1FAF1-1F3FB"},
    {"hand_til_hojre_farve2", "1FAF1-1F3FC"},
    {"hand_til_hojre_farve3", "1FAF1-1F3FD"},
    {"hand_til_hojre_farve4", "1FAF1-1F3FE"},
    {"hand_til_hojre_farve5", "1FAF1-1F3FF"},
    {"hand_til_venstre", "1FAF2"},
    {"hand_til_venstre_farve1", "1FAF2-1F3FB"},
    {"hand_til_venstre_farve2", "1FAF2-1F3FC"},
    {"hand_til_venstre_farve3", "1FAF2-1F3FD"},
    {"hand_til_venstre_farve4", "1FAF2-1F3FE"},
    {"hand_til_venstre_farve5", "1FAF2-1F3FF"},
    {"handflade_nedad", "1FAF3"},
    {"handflade_nedad_farve1", "1FAF3-1F3FB"},
    {"handflade_nedad_farve2", "1FAF3-1F3FC"},
    {"handflade_nedad_farve3", "1FAF3-1F3FD"},
    {"handflade_nedad_farve4", "1FAF3-1F3FE"},
    {"handflade_nedad_farve5", "1FAF3-1F3FF"},
    {"handflade_opad", "1FAF4"},
    {"handflade_opad_farve1", "1FAF4-1F3FB"},
    {"handflade_opad_farve2", "1FAF4-1F3FC"},
    {"handflade_opad_farve3", "1FAF4-1F3FD"},
    {"handflade_opad_farve4", "1FAF4-1F3FE"},
    {"handflade_opad_farve5", "1FAF4-1F3FF"},
    {"peger_pa_seeren", "1FAF5"},
    {"peger_pa_seeren_farve1", "1FAF5-1F3FB"},
    {"peger_pa_seeren_farve2", "1FAF5-1F3FC"},
    {"peger_pa_seeren_farve3", "1FAF5-1F3FD"},
    {"peger_pa_seeren_farve4", "1FAF5-1F3FE"},
    {"peger_pa_seeren_farve5", "1FAF5-1F3FF"},
    {"hjertehaender", "1FAF6"},
    {"hjertehaender_farve1", "1FAF6-1F3FB"},
    {"hjertehaender_farve2", "1FAF6-1F3FC"},
    {"hjertehaender_farve3", "1FAF6-1F3FD"},
    {"hjertehaender_farve4", "1FAF6-1F3FE"},
    {"hjertehaender_farve5", "1FAF6-1F3FF"},
    {"hand_skubber_mod_venstre", "1FAF7"},
    {"hand_skubber_mod_venstre_farve1", "1FAF7-1F3FB"},
    {"hand_skubber_mod_venstre_farve2", "1FAF7-1F3FC"},
    {"hand_skubber_mod_venstre_farve3", "1FAF7-1F3FD"},
    {"hand_skubber_mod_venstre_farve4", "1FAF7-1F3FE"},
    {"hand_skubber_mod_venstre_farve5", "1FAF7-1F3FF"},
    {"hand_skubber_mod_hojre", "1FAF8"},
    {"hand_skubber_mod_hojre_farve1", "1FAF8-1F3FB"},
    {"hand_skubber_mod_hojre_farve2", "1FAF8-1F3FC"},
    {"hand_skubber_mod_hojre_farve3", "1FAF8-1F3FD"},
    {"hand_skubber_mod_hojre_farve4", "1FAF8-1F3FE"},
    {"hand_skubber_mod_hojre_farve5", "1FAF8-1F3FF"},
    {"keycap_nummertegn", "0023-FE0F-20E3"},
    {"keycap_stjerne", "002A-FE0F-20E3"},
    {"tast_0", "0030-FE0F-20E3"},
    {"tast_1", "0031-FE0F-20E3"},
    {"tast_2", "0032-FE0F-20E3"},
    {"tast_3", "0033-FE0F-20E3"},
    {"tast_4", "0034-FE0F-20E3"},
    {"tast_5", "0035-FE0F-20E3"},
    {"tast_6", "0036-FE0F-20E3"},
    {"tast_7", "0037-FE0F-20E3"},
    {"tast_8", "0038-FE0F-20E3"},
    {"tast_9", "0039-FE0F-20E3"},
    {"ascensionoen", "1F1E6-1F1E8"},
    {"andorra", "1F1E6-1F1E9"},
    {"de_forenede_arabiske_emirater", "1F1E6-1F1EA"},
    {"afghanistan", "1F1E6-1F1EB"},
    {"antigua_og_barbuda", "1F1E6-1F1EC"},
    {"anguilla", "1F1E6-1F1EE"},
    {"albanien", "1F1E6-1F1F1"},
    {"armenien", "1F1E6-1F1F2"},
    {"angola", "1F1E6-1F1F4"},
    {"antarktis", "1F1E6-1F1F6"},
    {"argentina", "1F1E6-1F1F7"},
    {"amerikansk_samoa", "1F1E6-1F1F8"},
    {"ostrig", "1F1E6-1F1F9"},
    {"australien", "1F1E6-1F1FA"},
    {"aruba", "1F1E6-1F1FC"},
    {"aland", "1F1E6-1F1FD"},
    {"aserbajdsjan", "1F1E6-1F1FF"},
    {"bosnien_hercegovina", "1F1E7-1F1E6"},
    {"barbados", "1F1E7-1F1E7"},
    {"bangladesh", "1F1E7-1F1E9"},
    {"belgien", "1F1E7-1F1EA"},
    {"burkina_faso", "1F1E7-1F1EB"},
    {"bulgarien", "1F1E7-1F1EC"},
    {"bahrain", "1F1E7-1F1ED"},
    {"burundi", "1F1E7-1F1EE"},
    {"benin", "1F1E7-1F1EF"},
    {"saint_barthelemy", "1F1E7-1F1F1"},
    {"bermuda", "1F1E7-1F1F2"},
    {"brunei", "1F1E7-1F1F3"},
    {"bolivia", "1F1E7-1F1F4"},
    {"de_tidligere_nederlandske_antiller", "1F1E7-1F1F6"},
    {"brasilien", "1F1E7-1F1F7"},
    {"bahamas", "1F1E7-1F1F8"},
    {"bhutan", "1F1E7-1F1F9"},
    {"bouvetoen", "1F1E7-1F1FB"},
    {"botswana", "1F1E7-1F1FC"},
    {"belarus", "1F1E7-1F1FE"},
    {"belize", "1F1E7-1F1FF"},
    {"canada", "1F1E8-1F1E6"},
    {"cocosoerne", "1F1E8-1F1E8"},
    {"congo_kinshasa", "1F1E8-1F1E9"},
    {"den_centralafrikanske_republik", "1F1E8-1F1EB"},
    {"congo_brazzaville", "1F1E8-1F1EC"},
    {"schweiz", "1F1E8-1F1ED"},
    {"elfenbenskysten", "1F1E8-1F1EE"},
    {"cookoerne", "1F1E8-1F1F0"},
    {"chile", "1F1E8-1F1F1"},
    {"cameroun", "1F1E8-1F1F2"},
    {"kina", "1F1E8-1F1F3"},
    {"colombia", "1F1E8-1F1F4"},
    {"clippertonoen", "1F1E8-1F1F5"},
    {"sark", "1F1E8-1F1F6"},
    {"costa_rica", "1F1E8-1F1F7"},
    {"cuba", "1F1E8-1F1FA"},
    {"kap_verde", "1F1E8-1F1FB"},
    {"curacao", "1F1E8-1F1FC"},
    {"juleoen", "1F1E8-1F1FD"},
    {"cypern", "1F1E8-1F1FE"},
    {"tjekkiet", "1F1E8-1F1FF"},
    {"tyskland", "1F1E9-1F1EA"},
    {"diego_garcia", "1F1E9-1F1EC"},
    {"djibouti", "1F1E9-1F1EF"},
    {"danmark", "1F1E9-1F1F0"},
    {"dominica", "1F1E9-1F1F2"},
    {"den_dominikanske_republik", "1F1E9-1F1F4"},
    {"algeriet", "1F1E9-1F1FF"},
    {"ceuta_og_melilla", "1F1EA-1F1E6"},
    {"ecuador", "1F1EA-1F1E8"},
    {"estland", "1F1EA-1F1EA"},
    {"egypten", "1F1EA-1F1EC"},
    {"vestsahara", "1F1EA-1F1ED"},
    {"eritrea", "1F1EA-1F1F7"},
    {"spanien", "1F1EA-1F1F8"},
    {"etiopien", "1F1EA-1F1F9"},
    {"den_europaeiske_union", "1F1EA-1F1FA"},
    {"finland", "1F1EB-1F1EE"},
    {"fiji", "1F1EB-1F1EF"},
    {"falklandsoerne", "1F1EB-1F1F0"},
    {"mikronesien", "1F1EB-1F1F2"},
    {"faeroerne", "1F1EB-1F1F4"},
    {"frankrig", "1F1EB-1F1F7"},
    {"gabon", "1F1EC-1F1E6"},
    {"storbritannien", "1F1EC-1F1E7"},
    {"grenada", "1F1EC-1F1E9"},
    {"georgien", "1F1EC-1F1EA"},
    {"fransk_guyana", "1F1EC-1F1EB"},
    {"guernsey", "1F1EC-1F1EC"},
    {"ghana", "1F1EC-1F1ED"},
    {"gibraltar", "1F1EC-1F1EE"},
    {"gronland", "1F1EC-1F1F1"},
    {"gambia", "1F1EC-1F1F2"},
    {"guinea", "1F1EC-1F1F3"},
    {"guadeloupe", "1F1EC-1F1F5"},
    {"aekvatorialguinea", "1F1EC-1F1F6"},
    {"graekenland", "1F1EC-1F1F7"},
    {"south_georgia_og_de_sydlige_sandwichoer", "1F1EC-1F1F8"},
    {"guatemala", "1F1EC-1F1F9"},
    {"guam", "1F1EC-1F1FA"},
    {"guinea_bissau", "1F1EC-1F1FC"},
    {"guyana", "1F1EC-1F1FE"},
    {"sar_hongkong", "1F1ED-1F1F0"},
    {"heard_island_og_mcdonald_islands", "1F1ED-1F1F2"},
    {"honduras", "1F1ED-1F1F3"},
    {"kroatien", "1F1ED-1F1F7"},
    {"haiti", "1F1ED-1F1F9"},
    {"ungarn", "1F1ED-1F1FA"},
    {"kanariske_oer", "1F1EE-1F1E8"},
    {"indonesien", "1F1EE-1F1E9"},
    {"irland", "1F1EE-1F1EA"},
    {"israel", "1F1EE-1F1F1"},
    {"isle_of_man", "1F1EE-1F1F2"},
    {"indien", "1F1EE-1F1F3"},
    {"det_britiske_territorium_i_det_indiske_ocean", "1F1EE-1F1F4"},
    {"irak", "1F1EE-1F1F6"},
    {"iran", "1F1EE-1F1F7"},
    {"island", "1F1EE-1F1F8"},
    {"italien", "1F1EE-1F1F9"},
    {"jersey", "1F1EF-1F1EA"},
    {"jamaica", "1F1EF-1F1F2"},
    {"jordan", "1F1EF-1F1F4"},
    {"japan", "1F1EF-1F1F5"},
    {"kenya", "1F1F0-1F1EA"},
    {"kirgisistan", "1F1F0-1F1EC"},
    {"cambodja", "1F1F0-1F1ED"},
    {"kiribati", "1F1F0-1F1EE"},
    {"comorerne", "1F1F0-1F1F2"},
    {"saint_kitts_og_nevis", "1F1F0-1F1F3"},
    {"nordkorea", "1F1F0-1F1F5"},
    {"sydkorea", "1F1F0-1F1F7"},
    {"kuwait", "1F1F0-1F1FC"},
    {"caymanoerne", "1F1F0-1F1FE"},
    {"kasakhstan", "1F1F0-1F1FF"},
    {"laos", "1F1F1-1F1E6"},
    {"libanon", "1F1F1-1F1E7"},
    {"saint_lucia", "1F1F1-1F1E8"},
    {"liechtenstein", "1F1F1-1F1EE"},
    {"sri_lanka", "1F1F1-1F1F0"},
    {"liberia", "1F1F1-1F1F7"},
    {"lesotho", "1F1F1-1F1F8"},
    {"litauen", "1F1F1-1F1F9"},
    {"luxembourg", "1F1F1-1F1FA"},
    {"letland", "1F1F1-1F1FB"},
    {"libyen", "1F1F1-1F1FE"},
    {"marokko", "1F1F2-1F1E6"},
    {"monaco", "1F1F2-1F1E8"},
    {"moldova", "1F1F2-1F1E9"},
    {"montenegro", "1F1F2-1F1EA"},
    {"saint_martin", "1F1F2-1F1EB"},
    {"madagaskar", "1F1F2-1F1EC"},
    {"marshalloerne", "1F1F2-1F1ED"},
    {"nordmakedonien", "1F1F2-1F1F0"},
    {"mali", "1F1F2-1F1F1"},
    {"myanmar_burma", "1F1F2-1F1F2"},
    {"mongoliet", "1F1F2-1F1F3"},
    {"sar_macao", "1F1F2-1F1F4"},
    {"nordmarianerne", "1F1F2-1F1F5"},
    {"martinique", "1F1F2-1F1F6"},
    {"mauretanien", "1F1F2-1F1F7"},
    {"montserrat", "1F1F2-1F1F8"},
    {"malta", "1F1F2-1F1F9"},
    {"mauritius", "1F1F2-1F1FA"},
    {"maldiverne", "1F1F2-1F1FB"},
    {"malawi", "1F1F2-1F1FC"},
    {"mexico", "1F1F2-1F1FD"},
    {"malaysia", "1F1F2-1F1FE"},
    {"mozambique", "1F1F2-1F1FF"},
    {"namibia", "1F1F3-1F1E6"},
    {"ny_kaledonien", "1F1F3-1F1E8"},
    {"niger", "1F1F3-1F1EA"},
    {"norfolk_island", "1F1F3-1F1EB"},
    {"nigeria", "1F1F3-1F1EC"},
    {"nicaragua", "1F1F3-1F1EE"},
    {"nederlandene", "1F1F3-1F1F1"},
    {"norge", "1F1F3-1F1F4"},
    {"nepal", "1F1F3-1F1F5"},
    {"nauru", "1F1F3-1F1F7"},
    {"niue", "1F1F3-1F1FA"},
    {"new_zealand", "1F1F3-1F1FF"},
    {"oman", "1F1F4-1F1F2"},
    {"panama", "1F1F5-1F1E6"},
    {"peru", "1F1F5-1F1EA"},
    {"fransk_polynesien", "1F1F5-1F1EB"},
    {"papua_ny_guinea", "1F1F5-1F1EC"},
    {"filippinerne", "1F1F5-1F1ED"},
    {"pakistan", "1F1F5-1F1F0"},
    {"polen", "1F1F5-1F1F1"},
    {"saint_pierre_og_miquelon", "1F1F5-1F1F2"},
    {"pitcairn", "1F1F5-1F1F3"},
    {"puerto_rico", "1F1F5-1F1F7"},
    {"de_palaestinensiske_omrader", "1F1F5-1F1F8"},
    {"portugal", "1F1F5-1F1F9"},
    {"palau", "1F1F5-1F1FC"},
    {"paraguay", "1F1F5-1F1FE"},
    {"qatar", "1F1F6-1F1E6"},
    {"reunion", "1F1F7-1F1EA"},
    {"rumaenien", "1F1F7-1F1F4"},
    {"serbien", "1F1F7-1F1F8"},
    {"rusland", "1F1F7-1F1FA"},
    {"rwanda", "1F1F7-1F1FC"},
    {"saudi_arabien", "1F1F8-1F1E6"},
    {"salomonoerne", "1F1F8-1F1E7"},
    {"seychellerne", "1F1F8-1F1E8"},
    {"sudan", "1F1F8-1F1E9"},
    {"sverige", "1F1F8-1F1EA"},
    {"singapore", "1F1F8-1F1EC"},
    {"st_helena", "1F1F8-1F1ED"},
    {"slovenien", "1F1F8-1F1EE"},
    {"svalbard_og_jan_mayen", "1F1F8-1F1EF"},
    {"slovakiet", "1F1F8-1F1F0"},
    {"sierra_leone", "1F1F8-1F1F1"},
    {"san_marino", "1F1F8-1F1F2"},
    {"senegal", "1F1F8-1F1F3"},
    {"somalia", "1F1F8-1F1F4"},
    {"surinam", "1F1F8-1F1F7"},
    {"sydsudan", "1F1F8-1F1F8"},
    {"sao_tome_og_principe", "1F1F8-1F1F9"},
    {"el_salvador", "1F1F8-1F1FB"},
    {"sint_maarten", "1F1F8-1F1FD"},
    {"syrien", "1F1F8-1F1FE"},
    {"eswatini", "1F1F8-1F1FF"},
    {"tristan_da_cunha", "1F1F9-1F1E6"},
    {"turks_og_caicosoerne", "1F1F9-1F1E8"},
    {"tchad", "1F1F9-1F1E9"},
    {"de_franske_besiddelser_i_det_sydlige_indiske_ocean_og_antarktis", "1F1F9-1F1EB"},
    {"togo", "1F1F9-1F1EC"},
    {"thailand", "1F1F9-1F1ED"},
    {"tadsjikistan", "1F1F9-1F1EF"},
    {"tokelau", "1F1F9-1F1F0"},
    {"timor_leste", "1F1F9-1F1F1"},
    {"turkmenistan", "1F1F9-1F1F2"},
    {"tunesien", "1F1F9-1F1F3"},
    {"tonga", "1F1F9-1F1F4"},
    {"tyrkiet", "1F1F9-1F1F7"},
    {"trinidad_og_tobago", "1F1F9-1F1F9"},
    {"tuvalu", "1F1F9-1F1FB"},
    {"taiwan", "1F1F9-1F1FC"},
    {"tanzania", "1F1F9-1F1FF"},
    {"ukraine", "1F1FA-1F1E6"},
    {"uganda", "1F1FA-1F1EC"},
    {"amerikanske_oversoiske_oer", "1F1FA-1F1F2"},
    {"de_forenede_nationer", "1F1FA-1F1F3"},
    {"usa", "1F1FA-1F1F8"},
    {"uruguay", "1F1FA-1F1FE"},
    {"usbekistan", "1F1FA-1F1FF"},
    {"vatikanstaten", "1F1FB-1F1E6"},
    {"saint_vincent_og_grenadinerne", "1F1FB-1F1E8"},
    {"venezuela", "1F1FB-1F1EA"},
    {"de_britiske_jomfruoer", "1F1FB-1F1EC"},
    {"de_amerikanske_jomfruoer", "1F1FB-1F1EE"},
    {"vietnam", "1F1FB-1F1F3"},
    {"vanuatu", "1F1FB-1F1FA"},
    {"wallis_og_futuna", "1F1FC-1F1EB"},
    {"samoa", "1F1FC-1F1F8"},
    {"kosovo", "1F1FD-1F1F0"},
    {"yemen", "1F1FE-1F1EA"},
    {"mayotte", "1F1FE-1F1F9"},
    {"sydafrika", "1F1FF-1F1E6"},
    {"zambia", "1F1FF-1F1F2"},
    {"zimbabwe", "1F1FF-1F1FC"},
    {"england", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"skotland", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"wales", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"par_med_hjerte_mand_og_mand", "1F468-200D-2764-FE0F-200D-1F468"},
    {"par_med_hjerte_mand_og_mand_farve1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_mand_og_mand_farve1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_mand_og_mand_farve1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_mand_og_mand_farve1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_mand_og_mand_farve1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_mand_og_mand_farve2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_mand_og_mand_farve2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_mand_og_mand_farve2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_mand_og_mand_farve2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_mand_og_mand_farve2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_mand_og_mand_farve3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_mand_og_mand_farve3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_mand_og_mand_farve3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_mand_og_mand_farve3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_mand_og_mand_farve3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_mand_og_mand_farve4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_mand_og_mand_farve4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_mand_og_mand_farve4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_mand_og_mand_farve4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_mand_og_mand_farve4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_mand_og_mand_farve5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_mand_og_mand_farve5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_mand_og_mand_farve5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_mand_og_mand_farve5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_mand_og_mand_farve5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"kys_mand_og_mand", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"kys_mand_og_mand_farve1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kys_mand_og_mand_farve1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kys_mand_og_mand_farve1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kys_mand_og_mand_farve1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kys_mand_og_mand_farve1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kys_mand_og_mand_farve2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kys_mand_og_mand_farve2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kys_mand_og_mand_farve2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kys_mand_og_mand_farve2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kys_mand_og_mand_farve2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kys_mand_og_mand_farve3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kys_mand_og_mand_farve3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kys_mand_og_mand_farve3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kys_mand_og_mand_farve3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kys_mand_og_mand_farve3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kys_mand_og_mand_farve4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kys_mand_og_mand_farve4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kys_mand_og_mand_farve4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kys_mand_og_mand_farve4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kys_mand_og_mand_farve4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kys_mand_og_mand_farve5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kys_mand_og_mand_farve5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kys_mand_og_mand_farve5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kys_mand_og_mand_farve5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kys_mand_og_mand_farve5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"familie_mand_og_dreng", "1F468-200D-1F466"},
    {"familie_mand_dreng_og_dreng", "1F468-200D-1F466-200D-1F466"},
    {"familie_mand_og_pige", "1F468-200D-1F467"},
    {"familie_mand_pige_og_dreng", "1F468-200D-1F467-200D-1F466"},
    {"familie_mand_pige_og_pige", "1F468-200D-1F467-200D-1F467"},
    {"familie_mand_mand_og_dreng", "1F468-200D-1F468-200D-1F466"},
    {"familie_mand_mand_dreng_og_dreng", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"familie_mand_mand_og_pige", "1F468-200D-1F468-200D-1F467"},
    {"familie_mand_mand_pige_og_dreng", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"familie_mand_mand_pige_og_pige", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"familie_mand_kvinde_og_dreng", "1F468-200D-1F469-200D-1F466"},
    {"familie_mand_kvinde_dreng_og_dreng", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"familie_mand_kvinde_og_pige", "1F468-200D-1F469-200D-1F467"},
    {"familie_mand_kvinde_pige_og_dreng", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"familie_mand_kvinde_pige_og_pige", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"par_med_hjerte_kvinde_og_mand", "1F469-200D-2764-FE0F-200D-1F468"},
    {"par_med_hjerte_kvinde_og_mand_farve1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_kvinde_og_mand_farve1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_kvinde_og_mand_farve1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_kvinde_og_mand_farve1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_kvinde_og_mand_farve1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_kvinde_og_mand_farve2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_kvinde_og_mand_farve2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_kvinde_og_mand_farve2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_kvinde_og_mand_farve2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_kvinde_og_mand_farve2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_kvinde_og_mand_farve3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_kvinde_og_mand_farve3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_kvinde_og_mand_farve3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_kvinde_og_mand_farve3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_kvinde_og_mand_farve3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_kvinde_og_mand_farve4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_kvinde_og_mand_farve4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_kvinde_og_mand_farve4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_kvinde_og_mand_farve4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_kvinde_og_mand_farve4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_kvinde_og_mand_farve5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjerte_kvinde_og_mand_farve5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjerte_kvinde_og_mand_farve5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjerte_kvinde_og_mand_farve5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjerte_kvinde_og_mand_farve5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjerte_kvinde_og_kvinde", "1F469-200D-2764-FE0F-200D-1F469"},
    {"par_med_hjerte_kvinde_og_kvinde_farve1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjerte_kvinde_og_kvinde_farve1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjerte_kvinde_og_kvinde_farve1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjerte_kvinde_og_kvinde_farve1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjerte_kvinde_og_kvinde_farve1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjerte_kvinde_og_kvinde_farve2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjerte_kvinde_og_kvinde_farve2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjerte_kvinde_og_kvinde_farve2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjerte_kvinde_og_kvinde_farve2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjerte_kvinde_og_kvinde_farve2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjerte_kvinde_og_kvinde_farve3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjerte_kvinde_og_kvinde_farve3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjerte_kvinde_og_kvinde_farve3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjerte_kvinde_og_kvinde_farve3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjerte_kvinde_og_kvinde_farve3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjerte_kvinde_og_kvinde_farve4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjerte_kvinde_og_kvinde_farve4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjerte_kvinde_og_kvinde_farve4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjerte_kvinde_og_kvinde_farve4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjerte_kvinde_og_kvinde_farve4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjerte_kvinde_og_kvinde_farve5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjerte_kvinde_og_kvinde_farve5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjerte_kvinde_og_kvinde_farve5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjerte_kvinde_og_kvinde_farve5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjerte_kvinde_og_kvinde_farve5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"kys_kvinde_og_mand", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"kys_kvinde_og_mand_farve1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kys_kvinde_og_mand_farve1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kys_kvinde_og_mand_farve1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kys_kvinde_og_mand_farve1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kys_kvinde_og_mand_farve1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kys_kvinde_og_mand_farve2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kys_kvinde_og_mand_farve2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kys_kvinde_og_mand_farve2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kys_kvinde_og_mand_farve2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kys_kvinde_og_mand_farve2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kys_kvinde_og_mand_farve3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kys_kvinde_og_mand_farve3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kys_kvinde_og_mand_farve3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kys_kvinde_og_mand_farve3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kys_kvinde_og_mand_farve3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kys_kvinde_og_mand_farve4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kys_kvinde_og_mand_farve4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kys_kvinde_og_mand_farve4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kys_kvinde_og_mand_farve4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kys_kvinde_og_mand_farve4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kys_kvinde_og_mand_farve5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kys_kvinde_og_mand_farve5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kys_kvinde_og_mand_farve5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kys_kvinde_og_mand_farve5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kys_kvinde_og_mand_farve5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kys_kvinde_og_kvinde", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"kys_kvinde_og_kvinde_farve1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kys_kvinde_og_kvinde_farve1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kys_kvinde_og_kvinde_farve1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kys_kvinde_og_kvinde_farve1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kys_kvinde_og_kvinde_farve1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"kys_kvinde_og_kvinde_farve2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kys_kvinde_og_kvinde_farve2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kys_kvinde_og_kvinde_farve2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kys_kvinde_og_kvinde_farve2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kys_kvinde_og_kvinde_farve2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"kys_kvinde_og_kvinde_farve3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kys_kvinde_og_kvinde_farve3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kys_kvinde_og_kvinde_farve3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kys_kvinde_og_kvinde_farve3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kys_kvinde_og_kvinde_farve3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"kys_kvinde_og_kvinde_farve4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kys_kvinde_og_kvinde_farve4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kys_kvinde_og_kvinde_farve4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kys_kvinde_og_kvinde_farve4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kys_kvinde_og_kvinde_farve4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"kys_kvinde_og_kvinde_farve5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kys_kvinde_og_kvinde_farve5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kys_kvinde_og_kvinde_farve5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kys_kvinde_og_kvinde_farve5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kys_kvinde_og_kvinde_farve5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"familie_kvinde_og_dreng", "1F469-200D-1F466"},
    {"familie_kvinde_dreng_og_dreng", "1F469-200D-1F466-200D-1F466"},
    {"familie_kvinde_og_pige", "1F469-200D-1F467"},
    {"familie_kvinde_pige_og_dreng", "1F469-200D-1F467-200D-1F466"},
    {"familie_kvinde_pige_og_pige", "1F469-200D-1F467-200D-1F467"},
    {"familie_kvinde_kvinde_og_dreng", "1F469-200D-1F469-200D-1F466"},
    {"familie_kvinde_kvinde_dreng_og_dreng", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"familie_kvinde_kvinde_og_pige", "1F469-200D-1F469-200D-1F467"},
    {"familie_kvinde_kvinde_pige_og_dreng", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"familie_kvinde_kvinde_pige_og_pige", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"personer_hand_i_hand", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"personer_hand_i_hand_farve1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_hand_i_hand_farve1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_hand_i_hand_farve1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_hand_i_hand_farve1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_hand_i_hand_farve1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_hand_i_hand_farve2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_hand_i_hand_farve2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_hand_i_hand_farve2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_hand_i_hand_farve2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_hand_i_hand_farve2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_hand_i_hand_farve3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_hand_i_hand_farve3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_hand_i_hand_farve3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_hand_i_hand_farve3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_hand_i_hand_farve3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_hand_i_hand_farve4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_hand_i_hand_farve4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_hand_i_hand_farve4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_hand_i_hand_farve4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_hand_i_hand_farve4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_hand_i_hand_farve5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_hand_i_hand_farve5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_hand_i_hand_farve5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_hand_i_hand_farve5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_hand_i_hand_farve5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"familie_voksen_voksen_barn", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"familie_voksen_voksen_barn_barn", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"familie_voksen_barn", "1F9D1-200D-1F9D2"},
    {"familie_voksen_barn_barn", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"lober_mod_hojre", "1F3C3-200D-27A1-FE0F"},
    {"lober_mod_hojre_farve1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"lober_mod_hojre_farve2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"lober_mod_hojre_farve3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"lober_mod_hojre_farve4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"lober_mod_hojre_farve5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"mandlig_sundhedsperson", "1F468-200D-2695-FE0F"},
    {"mandlig_sundhedsperson_farve1", "1F468-1F3FB-200D-2695-FE0F"},
    {"mandlig_sundhedsperson_farve2", "1F468-1F3FC-200D-2695-FE0F"},
    {"mandlig_sundhedsperson_farve3", "1F468-1F3FD-200D-2695-FE0F"},
    {"mandlig_sundhedsperson_farve4", "1F468-1F3FE-200D-2695-FE0F"},
    {"mandlig_sundhedsperson_farve5", "1F468-1F3FF-200D-2695-FE0F"},
    {"mandlig_dommer", "1F468-200D-2696-FE0F"},
    {"mandlig_dommer_farve1", "1F468-1F3FB-200D-2696-FE0F"},
    {"mandlig_dommer_farve2", "1F468-1F3FC-200D-2696-FE0F"},
    {"mandlig_dommer_farve3", "1F468-1F3FD-200D-2696-FE0F"},
    {"mandlig_dommer_farve4", "1F468-1F3FE-200D-2696-FE0F"},
    {"mandlig_dommer_farve5", "1F468-1F3FF-200D-2696-FE0F"},
    {"mandlig_pilot", "1F468-200D-2708-FE0F"},
    {"mandlig_pilot_farve1", "1F468-1F3FB-200D-2708-FE0F"},
    {"mandlig_pilot_farve2", "1F468-1F3FC-200D-2708-FE0F"},
    {"mandlig_pilot_farve3", "1F468-1F3FD-200D-2708-FE0F"},
    {"mandlig_pilot_farve4", "1F468-1F3FE-200D-2708-FE0F"},
    {"mandlig_pilot_farve5", "1F468-1F3FF-200D-2708-FE0F"},
    {"landmand", "1F468-200D-1F33E"},
    {"landmand_farve1", "1F468-1F3FB-200D-1F33E"},
    {"landmand_farve2", "1F468-1F3FC-200D-1F33E"},
    {"landmand_farve3", "1F468-1F3FD-200D-1F33E"},
    {"landmand_farve4", "1F468-1F3FE-200D-1F33E"},
    {"landmand_farve5", "1F468-1F3FF-200D-1F33E"},
    {"mandlig_kok", "1F468-200D-1F373"},
    {"mandlig_kok_farve1", "1F468-1F3FB-200D-1F373"},
    {"mandlig_kok_farve2", "1F468-1F3FC-200D-1F373"},
    {"mandlig_kok_farve3", "1F468-1F3FD-200D-1F373"},
    {"mandlig_kok_farve4", "1F468-1F3FE-200D-1F373"},
    {"mandlig_kok_farve5", "1F468-1F3FF-200D-1F373"},
    {"mand_der_mader_baby", "1F468-200D-1F37C"},
    {"mand_der_mader_baby_farve1", "1F468-1F3FB-200D-1F37C"},
    {"mand_der_mader_baby_farve2", "1F468-1F3FC-200D-1F37C"},
    {"mand_der_mader_baby_farve3", "1F468-1F3FD-200D-1F37C"},
    {"mand_der_mader_baby_farve4", "1F468-1F3FE-200D-1F37C"},
    {"mand_der_mader_baby_farve5", "1F468-1F3FF-200D-1F37C"},
    {"mandlig_studerende", "1F468-200D-1F393"},
    {"mandlig_studerende_farve1", "1F468-1F3FB-200D-1F393"},
    {"mandlig_studerende_farve2", "1F468-1F3FC-200D-1F393"},
    {"mandlig_studerende_farve3", "1F468-1F3FD-200D-1F393"},
    {"mandlig_studerende_farve4", "1F468-1F3FE-200D-1F393"},
    {"mandlig_studerende_farve5", "1F468-1F3FF-200D-1F393"},
    {"mandlig_sanger", "1F468-200D-1F3A4"},
    {"mandlig_sanger_farve1", "1F468-1F3FB-200D-1F3A4"},
    {"mandlig_sanger_farve2", "1F468-1F3FC-200D-1F3A4"},
    {"mandlig_sanger_farve3", "1F468-1F3FD-200D-1F3A4"},
    {"mandlig_sanger_farve4", "1F468-1F3FE-200D-1F3A4"},
    {"mandlig_sanger_farve5", "1F468-1F3FF-200D-1F3A4"},
    {"mandlig_kunstner", "1F468-200D-1F3A8"},
    {"mandlig_kunstner_farve1", "1F468-1F3FB-200D-1F3A8"},
    {"mandlig_kunstner_farve2", "1F468-1F3FC-200D-1F3A8"},
    {"mandlig_kunstner_farve3", "1F468-1F3FD-200D-1F3A8"},
    {"mandlig_kunstner_farve4", "1F468-1F3FE-200D-1F3A8"},
    {"mandlig_kunstner_farve5", "1F468-1F3FF-200D-1F3A8"},
    {"mandlig_laerer", "1F468-200D-1F3EB"},
    {"mandlig_laerer_farve1", "1F468-1F3FB-200D-1F3EB"},
    {"mandlig_laerer_farve2", "1F468-1F3FC-200D-1F3EB"},
    {"mandlig_laerer_farve3", "1F468-1F3FD-200D-1F3EB"},
    {"mandlig_laerer_farve4", "1F468-1F3FE-200D-1F3EB"},
    {"mandlig_laerer_farve5", "1F468-1F3FF-200D-1F3EB"},
    {"mandlig_fabriksarbejder", "1F468-200D-1F3ED"},
    {"mandlig_fabriksarbejder_farve1", "1F468-1F3FB-200D-1F3ED"},
    {"mandlig_fabriksarbejder_farve2", "1F468-1F3FC-200D-1F3ED"},
    {"mandlig_fabriksarbejder_farve3", "1F468-1F3FD-200D-1F3ED"},
    {"mandlig_fabriksarbejder_farve4", "1F468-1F3FE-200D-1F3ED"},
    {"mandlig_fabriksarbejder_farve5", "1F468-1F3FF-200D-1F3ED"},
    {"mandlig_it_medarbejder", "1F468-200D-1F4BB"},
    {"mandlig_it_medarbejder_farve1", "1F468-1F3FB-200D-1F4BB"},
    {"mandlig_it_medarbejder_farve2", "1F468-1F3FC-200D-1F4BB"},
    {"mandlig_it_medarbejder_farve3", "1F468-1F3FD-200D-1F4BB"},
    {"mandlig_it_medarbejder_farve4", "1F468-1F3FE-200D-1F4BB"},
    {"mandlig_it_medarbejder_farve5", "1F468-1F3FF-200D-1F4BB"},
    {"mandlig_kontoransat", "1F468-200D-1F4BC"},
    {"mandlig_kontoransat_farve1", "1F468-1F3FB-200D-1F4BC"},
    {"mandlig_kontoransat_farve2", "1F468-1F3FC-200D-1F4BC"},
    {"mandlig_kontoransat_farve3", "1F468-1F3FD-200D-1F4BC"},
    {"mandlig_kontoransat_farve4", "1F468-1F3FE-200D-1F4BC"},
    {"mandlig_kontoransat_farve5", "1F468-1F3FF-200D-1F4BC"},
    {"mandlig_mekaniker", "1F468-200D-1F527"},
    {"mandlig_mekaniker_farve1", "1F468-1F3FB-200D-1F527"},
    {"mandlig_mekaniker_farve2", "1F468-1F3FC-200D-1F527"},
    {"mandlig_mekaniker_farve3", "1F468-1F3FD-200D-1F527"},
    {"mandlig_mekaniker_farve4", "1F468-1F3FE-200D-1F527"},
    {"mandlig_mekaniker_farve5", "1F468-1F3FF-200D-1F527"},
    {"mandlig_forsker", "1F468-200D-1F52C"},
    {"mandlig_forsker_farve1", "1F468-1F3FB-200D-1F52C"},
    {"mandlig_forsker_farve2", "1F468-1F3FC-200D-1F52C"},
    {"mandlig_forsker_farve3", "1F468-1F3FD-200D-1F52C"},
    {"mandlig_forsker_farve4", "1F468-1F3FE-200D-1F52C"},
    {"mandlig_forsker_farve5", "1F468-1F3FF-200D-1F52C"},
    {"mandlig_astronaut", "1F468-200D-1F680"},
    {"mandlig_astronaut_farve1", "1F468-1F3FB-200D-1F680"},
    {"mandlig_astronaut_farve2", "1F468-1F3FC-200D-1F680"},
    {"mandlig_astronaut_farve3", "1F468-1F3FD-200D-1F680"},
    {"mandlig_astronaut_farve4", "1F468-1F3FE-200D-1F680"},
    {"mandlig_astronaut_farve5", "1F468-1F3FF-200D-1F680"},
    {"brandmand", "1F468-200D-1F692"},
    {"brandmand_farve1", "1F468-1F3FB-200D-1F692"},
    {"brandmand_farve2", "1F468-1F3FC-200D-1F692"},
    {"brandmand_farve3", "1F468-1F3FD-200D-1F692"},
    {"brandmand_farve4", "1F468-1F3FE-200D-1F692"},
    {"brandmand_farve5", "1F468-1F3FF-200D-1F692"},
    {"mand_med_blindestok", "1F468-200D-1F9AF"},
    {"mand_med_blindestok_farve1", "1F468-1F3FB-200D-1F9AF"},
    {"mand_med_blindestok_farve2", "1F468-1F3FC-200D-1F9AF"},
    {"mand_med_blindestok_farve3", "1F468-1F3FD-200D-1F9AF"},
    {"mand_med_blindestok_farve4", "1F468-1F3FE-200D-1F9AF"},
    {"mand_med_blindestok_farve5", "1F468-1F3FF-200D-1F9AF"},
    {"mand_med_blindestok_mod_hojre", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"mand_med_blindestok_mod_hojre_farve1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"mand_med_blindestok_mod_hojre_farve2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"mand_med_blindestok_mod_hojre_farve3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"mand_med_blindestok_mod_hojre_farve4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"mand_med_blindestok_mod_hojre_farve5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"mand_i_elektrisk_korestol", "1F468-200D-1F9BC"},
    {"mand_i_elektrisk_korestol_farve1", "1F468-1F3FB-200D-1F9BC"},
    {"mand_i_elektrisk_korestol_farve2", "1F468-1F3FC-200D-1F9BC"},
    {"mand_i_elektrisk_korestol_farve3", "1F468-1F3FD-200D-1F9BC"},
    {"mand_i_elektrisk_korestol_farve4", "1F468-1F3FE-200D-1F9BC"},
    {"mand_i_elektrisk_korestol_farve5", "1F468-1F3FF-200D-1F9BC"},
    {"mand_i_elektrisk_korestol_mod_hojre", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"mand_i_elektrisk_korestol_mod_hojre_farve1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"mand_i_elektrisk_korestol_mod_hojre_farve2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"mand_i_elektrisk_korestol_mod_hojre_farve3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"mand_i_elektrisk_korestol_mod_hojre_farve4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"mand_i_elektrisk_korestol_mod_hojre_farve5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mand_i_manuel_korestol", "1F468-200D-1F9BD"},
    {"mand_i_manuel_korestol_farve1", "1F468-1F3FB-200D-1F9BD"},
    {"mand_i_manuel_korestol_farve2", "1F468-1F3FC-200D-1F9BD"},
    {"mand_i_manuel_korestol_farve3", "1F468-1F3FD-200D-1F9BD"},
    {"mand_i_manuel_korestol_farve4", "1F468-1F3FE-200D-1F9BD"},
    {"mand_i_manuel_korestol_farve5", "1F468-1F3FF-200D-1F9BD"},
    {"mand_i_manuel_korestol_mod_hojre", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"mand_i_manuel_korestol_mod_hojre_farve1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"mand_i_manuel_korestol_mod_hojre_farve2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"mand_i_manuel_korestol_mod_hojre_farve3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"mand_i_manuel_korestol_mod_hojre_farve4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"mand_i_manuel_korestol_mod_hojre_farve5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kvindelig_sundhedsperson", "1F469-200D-2695-FE0F"},
    {"kvindelig_sundhedsperson_farve1", "1F469-1F3FB-200D-2695-FE0F"},
    {"kvindelig_sundhedsperson_farve2", "1F469-1F3FC-200D-2695-FE0F"},
    {"kvindelig_sundhedsperson_farve3", "1F469-1F3FD-200D-2695-FE0F"},
    {"kvindelig_sundhedsperson_farve4", "1F469-1F3FE-200D-2695-FE0F"},
    {"kvindelig_sundhedsperson_farve5", "1F469-1F3FF-200D-2695-FE0F"},
    {"kvindelig_dommer", "1F469-200D-2696-FE0F"},
    {"kvindelig_dommer_farve1", "1F469-1F3FB-200D-2696-FE0F"},
    {"kvindelig_dommer_farve2", "1F469-1F3FC-200D-2696-FE0F"},
    {"kvindelig_dommer_farve3", "1F469-1F3FD-200D-2696-FE0F"},
    {"kvindelig_dommer_farve4", "1F469-1F3FE-200D-2696-FE0F"},
    {"kvindelig_dommer_farve5", "1F469-1F3FF-200D-2696-FE0F"},
    {"kvindelig_pilot", "1F469-200D-2708-FE0F"},
    {"kvindelig_pilot_farve1", "1F469-1F3FB-200D-2708-FE0F"},
    {"kvindelig_pilot_farve2", "1F469-1F3FC-200D-2708-FE0F"},
    {"kvindelig_pilot_farve3", "1F469-1F3FD-200D-2708-FE0F"},
    {"kvindelig_pilot_farve4", "1F469-1F3FE-200D-2708-FE0F"},
    {"kvindelig_pilot_farve5", "1F469-1F3FF-200D-2708-FE0F"},
    {"kvindelig_landmand", "1F469-200D-1F33E"},
    {"kvindelig_landmand_farve1", "1F469-1F3FB-200D-1F33E"},
    {"kvindelig_landmand_farve2", "1F469-1F3FC-200D-1F33E"},
    {"kvindelig_landmand_farve3", "1F469-1F3FD-200D-1F33E"},
    {"kvindelig_landmand_farve4", "1F469-1F3FE-200D-1F33E"},
    {"kvindelig_landmand_farve5", "1F469-1F3FF-200D-1F33E"},
    {"kvindelig_kok", "1F469-200D-1F373"},
    {"kvindelig_kok_farve1", "1F469-1F3FB-200D-1F373"},
    {"kvindelig_kok_farve2", "1F469-1F3FC-200D-1F373"},
    {"kvindelig_kok_farve3", "1F469-1F3FD-200D-1F373"},
    {"kvindelig_kok_farve4", "1F469-1F3FE-200D-1F373"},
    {"kvindelig_kok_farve5", "1F469-1F3FF-200D-1F373"},
    {"kvinde_der_mader_baby", "1F469-200D-1F37C"},
    {"kvinde_der_mader_baby_farve1", "1F469-1F3FB-200D-1F37C"},
    {"kvinde_der_mader_baby_farve2", "1F469-1F3FC-200D-1F37C"},
    {"kvinde_der_mader_baby_farve3", "1F469-1F3FD-200D-1F37C"},
    {"kvinde_der_mader_baby_farve4", "1F469-1F3FE-200D-1F37C"},
    {"kvinde_der_mader_baby_farve5", "1F469-1F3FF-200D-1F37C"},
    {"kvindelig_studerende", "1F469-200D-1F393"},
    {"kvindelig_studerende_farve1", "1F469-1F3FB-200D-1F393"},
    {"kvindelig_studerende_farve2", "1F469-1F3FC-200D-1F393"},
    {"kvindelig_studerende_farve3", "1F469-1F3FD-200D-1F393"},
    {"kvindelig_studerende_farve4", "1F469-1F3FE-200D-1F393"},
    {"kvindelig_studerende_farve5", "1F469-1F3FF-200D-1F393"},
    {"sangerinde", "1F469-200D-1F3A4"},
    {"sangerinde_farve1", "1F469-1F3FB-200D-1F3A4"},
    {"sangerinde_farve2", "1F469-1F3FC-200D-1F3A4"},
    {"sangerinde_farve3", "1F469-1F3FD-200D-1F3A4"},
    {"sangerinde_farve4", "1F469-1F3FE-200D-1F3A4"},
    {"sangerinde_farve5", "1F469-1F3FF-200D-1F3A4"},
    {"kvindelig_kunstner", "1F469-200D-1F3A8"},
    {"kvindelig_kunstner_farve1", "1F469-1F3FB-200D-1F3A8"},
    {"kvindelig_kunstner_farve2", "1F469-1F3FC-200D-1F3A8"},
    {"kvindelig_kunstner_farve3", "1F469-1F3FD-200D-1F3A8"},
    {"kvindelig_kunstner_farve4", "1F469-1F3FE-200D-1F3A8"},
    {"kvindelig_kunstner_farve5", "1F469-1F3FF-200D-1F3A8"},
    {"kvindelig_laerer", "1F469-200D-1F3EB"},
    {"kvindelig_laerer_farve1", "1F469-1F3FB-200D-1F3EB"},
    {"kvindelig_laerer_farve2", "1F469-1F3FC-200D-1F3EB"},
    {"kvindelig_laerer_farve3", "1F469-1F3FD-200D-1F3EB"},
    {"kvindelig_laerer_farve4", "1F469-1F3FE-200D-1F3EB"},
    {"kvindelig_laerer_farve5", "1F469-1F3FF-200D-1F3EB"},
    {"kvindelig_fabriksarbejder", "1F469-200D-1F3ED"},
    {"kvindelig_fabriksarbejder_farve1", "1F469-1F3FB-200D-1F3ED"},
    {"kvindelig_fabriksarbejder_farve2", "1F469-1F3FC-200D-1F3ED"},
    {"kvindelig_fabriksarbejder_farve3", "1F469-1F3FD-200D-1F3ED"},
    {"kvindelig_fabriksarbejder_farve4", "1F469-1F3FE-200D-1F3ED"},
    {"kvindelig_fabriksarbejder_farve5", "1F469-1F3FF-200D-1F3ED"},
    {"kvindelig_it_medarbejder", "1F469-200D-1F4BB"},
    {"kvindelig_it_medarbejder_farve1", "1F469-1F3FB-200D-1F4BB"},
    {"kvindelig_it_medarbejder_farve2", "1F469-1F3FC-200D-1F4BB"},
    {"kvindelig_it_medarbejder_farve3", "1F469-1F3FD-200D-1F4BB"},
    {"kvindelig_it_medarbejder_farve4", "1F469-1F3FE-200D-1F4BB"},
    {"kvindelig_it_medarbejder_farve5", "1F469-1F3FF-200D-1F4BB"},
    {"kvindelig_kontoransat", "1F469-200D-1F4BC"},
    {"kvindelig_kontoransat_farve1", "1F469-1F3FB-200D-1F4BC"},
    {"kvindelig_kontoransat_farve2", "1F469-1F3FC-200D-1F4BC"},
    {"kvindelig_kontoransat_farve3", "1F469-1F3FD-200D-1F4BC"},
    {"kvindelig_kontoransat_farve4", "1F469-1F3FE-200D-1F4BC"},
    {"kvindelig_kontoransat_farve5", "1F469-1F3FF-200D-1F4BC"},
    {"kvindelig_mekaniker", "1F469-200D-1F527"},
    {"kvindelig_mekaniker_farve1", "1F469-1F3FB-200D-1F527"},
    {"kvindelig_mekaniker_farve2", "1F469-1F3FC-200D-1F527"},
    {"kvindelig_mekaniker_farve3", "1F469-1F3FD-200D-1F527"},
    {"kvindelig_mekaniker_farve4", "1F469-1F3FE-200D-1F527"},
    {"kvindelig_mekaniker_farve5", "1F469-1F3FF-200D-1F527"},
    {"kvindelig_forsker", "1F469-200D-1F52C"},
    {"kvindelig_forsker_farve1", "1F469-1F3FB-200D-1F52C"},
    {"kvindelig_forsker_farve2", "1F469-1F3FC-200D-1F52C"},
    {"kvindelig_forsker_farve3", "1F469-1F3FD-200D-1F52C"},
    {"kvindelig_forsker_farve4", "1F469-1F3FE-200D-1F52C"},
    {"kvindelig_forsker_farve5", "1F469-1F3FF-200D-1F52C"},
    {"kvindelig_astronaut", "1F469-200D-1F680"},
    {"kvindelig_astronaut_farve1", "1F469-1F3FB-200D-1F680"},
    {"kvindelig_astronaut_farve2", "1F469-1F3FC-200D-1F680"},
    {"kvindelig_astronaut_farve3", "1F469-1F3FD-200D-1F680"},
    {"kvindelig_astronaut_farve4", "1F469-1F3FE-200D-1F680"},
    {"kvindelig_astronaut_farve5", "1F469-1F3FF-200D-1F680"},
    {"kvindelig_brandmand", "1F469-200D-1F692"},
    {"kvindelig_brandmand_farve1", "1F469-1F3FB-200D-1F692"},
    {"kvindelig_brandmand_farve2", "1F469-1F3FC-200D-1F692"},
    {"kvindelig_brandmand_farve3", "1F469-1F3FD-200D-1F692"},
    {"kvindelig_brandmand_farve4", "1F469-1F3FE-200D-1F692"},
    {"kvindelig_brandmand_farve5", "1F469-1F3FF-200D-1F692"},
    {"kvinde_med_blindestok", "1F469-200D-1F9AF"},
    {"kvinde_med_blindestok_farve1", "1F469-1F3FB-200D-1F9AF"},
    {"kvinde_med_blindestok_farve2", "1F469-1F3FC-200D-1F9AF"},
    {"kvinde_med_blindestok_farve3", "1F469-1F3FD-200D-1F9AF"},
    {"kvinde_med_blindestok_farve4", "1F469-1F3FE-200D-1F9AF"},
    {"kvinde_med_blindestok_farve5", "1F469-1F3FF-200D-1F9AF"},
    {"kvinde_med_blindestok_mod_hojre", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinde_med_blindestok_mod_hojre_farve1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinde_med_blindestok_mod_hojre_farve2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinde_med_blindestok_mod_hojre_farve3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinde_med_blindestok_mod_hojre_farve4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinde_med_blindestok_mod_hojre_farve5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinde_i_elektrisk_korestol", "1F469-200D-1F9BC"},
    {"kvinde_i_elektrisk_korestol_farve1", "1F469-1F3FB-200D-1F9BC"},
    {"kvinde_i_elektrisk_korestol_farve2", "1F469-1F3FC-200D-1F9BC"},
    {"kvinde_i_elektrisk_korestol_farve3", "1F469-1F3FD-200D-1F9BC"},
    {"kvinde_i_elektrisk_korestol_farve4", "1F469-1F3FE-200D-1F9BC"},
    {"kvinde_i_elektrisk_korestol_farve5", "1F469-1F3FF-200D-1F9BC"},
    {"kvinde_i_elektrisk_korestol_mod_hojre", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinde_i_elektrisk_korestol_mod_hojre_farve1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinde_i_elektrisk_korestol_mod_hojre_farve2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinde_i_elektrisk_korestol_mod_hojre_farve3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinde_i_elektrisk_korestol_mod_hojre_farve4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinde_i_elektrisk_korestol_mod_hojre_farve5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinde_i_manuel_korestol", "1F469-200D-1F9BD"},
    {"kvinde_i_manuel_korestol_farve1", "1F469-1F3FB-200D-1F9BD"},
    {"kvinde_i_manuel_korestol_farve2", "1F469-1F3FC-200D-1F9BD"},
    {"kvinde_i_manuel_korestol_farve3", "1F469-1F3FD-200D-1F9BD"},
    {"kvinde_i_manuel_korestol_farve4", "1F469-1F3FE-200D-1F9BD"},
    {"kvinde_i_manuel_korestol_farve5", "1F469-1F3FF-200D-1F9BD"},
    {"kvinde_i_manuel_korestol_mod_hojre", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinde_i_manuel_korestol_mod_hojre_farve1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinde_i_manuel_korestol_mod_hojre_farve2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinde_i_manuel_korestol_mod_hojre_farve3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinde_i_manuel_korestol_mod_hojre_farve4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinde_i_manuel_korestol_mod_hojre_farve5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"fodgaenger_mod_hojre", "1F6B6-200D-27A1-FE0F"},
    {"fodgaenger_mod_hojre_farve1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"fodgaenger_mod_hojre_farve2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"fodgaenger_mod_hojre_farve3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"fodgaenger_mod_hojre_farve4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"fodgaenger_mod_hojre_farve5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"knaelende_person_mod_hojre", "1F9CE-200D-27A1-FE0F"},
    {"knaelende_person_mod_hojre_farve1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"knaelende_person_mod_hojre_farve2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"knaelende_person_mod_hojre_farve3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"knaelende_person_mod_hojre_farve4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"knaelende_person_mod_hojre_farve5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"sundhedspersonale", "1F9D1-200D-2695-FE0F"},
    {"sundhedspersonale_farve1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"sundhedspersonale_farve2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"sundhedspersonale_farve3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"sundhedspersonale_farve4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"sundhedspersonale_farve5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"dommer", "1F9D1-200D-2696-FE0F"},
    {"dommer_farve1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"dommer_farve2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"dommer_farve3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"dommer_farve4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"dommer_farve5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"pilot", "1F9D1-200D-2708-FE0F"},
    {"pilot_farve1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"pilot_farve2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"pilot_farve3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"pilot_farve4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"pilot_farve5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"bonde", "1F9D1-200D-1F33E"},
    {"bonde_farve1", "1F9D1-1F3FB-200D-1F33E"},
    {"bonde_farve2", "1F9D1-1F3FC-200D-1F33E"},
    {"bonde_farve3", "1F9D1-1F3FD-200D-1F33E"},
    {"bonde_farve4", "1F9D1-1F3FE-200D-1F33E"},
    {"bonde_farve5", "1F9D1-1F3FF-200D-1F33E"},
    {"kok", "1F9D1-200D-1F373"},
    {"kok_farve1", "1F9D1-1F3FB-200D-1F373"},
    {"kok_farve2", "1F9D1-1F3FC-200D-1F373"},
    {"kok_farve3", "1F9D1-1F3FD-200D-1F373"},
    {"kok_farve4", "1F9D1-1F3FE-200D-1F373"},
    {"kok_farve5", "1F9D1-1F3FF-200D-1F373"},
    {"person_der_mader_baby", "1F9D1-200D-1F37C"},
    {"person_der_mader_baby_farve1", "1F9D1-1F3FB-200D-1F37C"},
    {"person_der_mader_baby_farve2", "1F9D1-1F3FC-200D-1F37C"},
    {"person_der_mader_baby_farve3", "1F9D1-1F3FD-200D-1F37C"},
    {"person_der_mader_baby_farve4", "1F9D1-1F3FE-200D-1F37C"},
    {"person_der_mader_baby_farve5", "1F9D1-1F3FF-200D-1F37C"},
    {"julenisse", "1F9D1-200D-1F384"},
    {"julenisse_farve1", "1F9D1-1F3FB-200D-1F384"},
    {"julenisse_farve2", "1F9D1-1F3FC-200D-1F384"},
    {"julenisse_farve3", "1F9D1-1F3FD-200D-1F384"},
    {"julenisse_farve4", "1F9D1-1F3FE-200D-1F384"},
    {"julenisse_farve5", "1F9D1-1F3FF-200D-1F384"},
    {"studerende", "1F9D1-200D-1F393"},
    {"studerende_farve1", "1F9D1-1F3FB-200D-1F393"},
    {"studerende_farve2", "1F9D1-1F3FC-200D-1F393"},
    {"studerende_farve3", "1F9D1-1F3FD-200D-1F393"},
    {"studerende_farve4", "1F9D1-1F3FE-200D-1F393"},
    {"studerende_farve5", "1F9D1-1F3FF-200D-1F393"},
    {"sanger", "1F9D1-200D-1F3A4"},
    {"sanger_farve1", "1F9D1-1F3FB-200D-1F3A4"},
    {"sanger_farve2", "1F9D1-1F3FC-200D-1F3A4"},
    {"sanger_farve3", "1F9D1-1F3FD-200D-1F3A4"},
    {"sanger_farve4", "1F9D1-1F3FE-200D-1F3A4"},
    {"sanger_farve5", "1F9D1-1F3FF-200D-1F3A4"},
    {"kunstner", "1F9D1-200D-1F3A8"},
    {"kunstner_farve1", "1F9D1-1F3FB-200D-1F3A8"},
    {"kunstner_farve2", "1F9D1-1F3FC-200D-1F3A8"},
    {"kunstner_farve3", "1F9D1-1F3FD-200D-1F3A8"},
    {"kunstner_farve4", "1F9D1-1F3FE-200D-1F3A8"},
    {"kunstner_farve5", "1F9D1-1F3FF-200D-1F3A8"},
    {"laerer", "1F9D1-200D-1F3EB"},
    {"laerer_farve1", "1F9D1-1F3FB-200D-1F3EB"},
    {"laerer_farve2", "1F9D1-1F3FC-200D-1F3EB"},
    {"laerer_farve3", "1F9D1-1F3FD-200D-1F3EB"},
    {"laerer_farve4", "1F9D1-1F3FE-200D-1F3EB"},
    {"laerer_farve5", "1F9D1-1F3FF-200D-1F3EB"},
    {"fabriksarbejder", "1F9D1-200D-1F3ED"},
    {"fabriksarbejder_farve1", "1F9D1-1F3FB-200D-1F3ED"},
    {"fabriksarbejder_farve2", "1F9D1-1F3FC-200D-1F3ED"},
    {"fabriksarbejder_farve3", "1F9D1-1F3FD-200D-1F3ED"},
    {"fabriksarbejder_farve4", "1F9D1-1F3FE-200D-1F3ED"},
    {"fabriksarbejder_farve5", "1F9D1-1F3FF-200D-1F3ED"},
    {"it_medarbejder", "1F9D1-200D-1F4BB"},
    {"it_medarbejder_farve1", "1F9D1-1F3FB-200D-1F4BB"},
    {"it_medarbejder_farve2", "1F9D1-1F3FC-200D-1F4BB"},
    {"it_medarbejder_farve3", "1F9D1-1F3FD-200D-1F4BB"},
    {"it_medarbejder_farve4", "1F9D1-1F3FE-200D-1F4BB"},
    {"it_medarbejder_farve5", "1F9D1-1F3FF-200D-1F4BB"},
    {"kontoransat", "1F9D1-200D-1F4BC"},
    {"kontoransat_farve1", "1F9D1-1F3FB-200D-1F4BC"},
    {"kontoransat_farve2", "1F9D1-1F3FC-200D-1F4BC"},
    {"kontoransat_farve3", "1F9D1-1F3FD-200D-1F4BC"},
    {"kontoransat_farve4", "1F9D1-1F3FE-200D-1F4BC"},
    {"kontoransat_farve5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mekaniker", "1F9D1-200D-1F527"},
    {"mekaniker_farve1", "1F9D1-1F3FB-200D-1F527"},
    {"mekaniker_farve2", "1F9D1-1F3FC-200D-1F527"},
    {"mekaniker_farve3", "1F9D1-1F3FD-200D-1F527"},
    {"mekaniker_farve4", "1F9D1-1F3FE-200D-1F527"},
    {"mekaniker_farve5", "1F9D1-1F3FF-200D-1F527"},
    {"forsker", "1F9D1-200D-1F52C"},
    {"forsker_farve1", "1F9D1-1F3FB-200D-1F52C"},
    {"forsker_farve2", "1F9D1-1F3FC-200D-1F52C"},
    {"forsker_farve3", "1F9D1-1F3FD-200D-1F52C"},
    {"forsker_farve4", "1F9D1-1F3FE-200D-1F52C"},
    {"forsker_farve5", "1F9D1-1F3FF-200D-1F52C"},
    {"astronaut", "1F9D1-200D-1F680"},
    {"astronaut_farve1", "1F9D1-1F3FB-200D-1F680"},
    {"astronaut_farve2", "1F9D1-1F3FC-200D-1F680"},
    {"astronaut_farve3", "1F9D1-1F3FD-200D-1F680"},
    {"astronaut_farve4", "1F9D1-1F3FE-200D-1F680"},
    {"astronaut_farve5", "1F9D1-1F3FF-200D-1F680"},
    {"brandfolk", "1F9D1-200D-1F692"},
    {"brandfolk_farve1", "1F9D1-1F3FB-200D-1F692"},
    {"brandfolk_farve2", "1F9D1-1F3FC-200D-1F692"},
    {"brandfolk_farve3", "1F9D1-1F3FD-200D-1F692"},
    {"brandfolk_farve4", "1F9D1-1F3FE-200D-1F692"},
    {"brandfolk_farve5", "1F9D1-1F3FF-200D-1F692"},
    {"person_med_blindestok", "1F9D1-200D-1F9AF"},
    {"person_med_blindestok_farve1", "1F9D1-1F3FB-200D-1F9AF"},
    {"person_med_blindestok_farve2", "1F9D1-1F3FC-200D-1F9AF"},
    {"person_med_blindestok_farve3", "1F9D1-1F3FD-200D-1F9AF"},
    {"person_med_blindestok_farve4", "1F9D1-1F3FE-200D-1F9AF"},
    {"person_med_blindestok_farve5", "1F9D1-1F3FF-200D-1F9AF"},
    {"person_med_blindestok_mod_hojre", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestok_mod_hojre_farve1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestok_mod_hojre_farve2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestok_mod_hojre_farve3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestok_mod_hojre_farve4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestok_mod_hojre_farve5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"person_i_elkorestol", "1F9D1-200D-1F9BC"},
    {"person_i_elkorestol_farve1", "1F9D1-1F3FB-200D-1F9BC"},
    {"person_i_elkorestol_farve2", "1F9D1-1F3FC-200D-1F9BC"},
    {"person_i_elkorestol_farve3", "1F9D1-1F3FD-200D-1F9BC"},
    {"person_i_elkorestol_farve4", "1F9D1-1F3FE-200D-1F9BC"},
    {"person_i_elkorestol_farve5", "1F9D1-1F3FF-200D-1F9BC"},
    {"person_i_elkorestol_mod_hojre", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_elkorestol_mod_hojre_farve1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_elkorestol_mod_hojre_farve2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_elkorestol_mod_hojre_farve3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_elkorestol_mod_hojre_farve4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_elkorestol_mod_hojre_farve5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_manuel_korestol", "1F9D1-200D-1F9BD"},
    {"person_i_manuel_korestol_farve1", "1F9D1-1F3FB-200D-1F9BD"},
    {"person_i_manuel_korestol_farve2", "1F9D1-1F3FC-200D-1F9BD"},
    {"person_i_manuel_korestol_farve3", "1F9D1-1F3FD-200D-1F9BD"},
    {"person_i_manuel_korestol_farve4", "1F9D1-1F3FE-200D-1F9BD"},
    {"person_i_manuel_korestol_farve5", "1F9D1-1F3FF-200D-1F9BD"},
    {"person_i_manuel_korestol_mod_hojre", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuel_korestol_mod_hojre_farve1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuel_korestol_mod_hojre_farve2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuel_korestol_mod_hojre_farve3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuel_korestol_mod_hojre_farve4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuel_korestol_mod_hojre_farve5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinde_der_dribler_med_en_bold", "26F9-FE0F-200D-2640-FE0F"},
    {"kvinde_der_dribler_med_en_bold_farve1", "26F9-1F3FB-200D-2640-FE0F"},
    {"kvinde_der_dribler_med_en_bold_farve2", "26F9-1F3FC-200D-2640-FE0F"},
    {"kvinde_der_dribler_med_en_bold_farve3", "26F9-1F3FD-200D-2640-FE0F"},
    {"kvinde_der_dribler_med_en_bold_farve4", "26F9-1F3FE-200D-2640-FE0F"},
    {"kvinde_der_dribler_med_en_bold_farve5", "26F9-1F3FF-200D-2640-FE0F"},
    {"mand_der_dribler_med_en_bold", "26F9-FE0F-200D-2642-FE0F"},
    {"mand_der_dribler_med_en_bold_farve1", "26F9-1F3FB-200D-2642-FE0F"},
    {"mand_der_dribler_med_en_bold_farve2", "26F9-1F3FC-200D-2642-FE0F"},
    {"mand_der_dribler_med_en_bold_farve3", "26F9-1F3FD-200D-2642-FE0F"},
    {"mand_der_dribler_med_en_bold_farve4", "26F9-1F3FE-200D-2642-FE0F"},
    {"mand_der_dribler_med_en_bold_farve5", "26F9-1F3FF-200D-2642-FE0F"},
    {"kvinde_lober", "1F3C3-200D-2640-FE0F"},
    {"kvinde_lober_farve1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"kvinde_lober_farve2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"kvinde_lober_farve3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"kvinde_lober_farve4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"kvinde_lober_farve5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"kvinde_lober_mod_hojre", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_lober_mod_hojre_farve1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_lober_mod_hojre_farve2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_lober_mod_hojre_farve3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_lober_mod_hojre_farve4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_lober_mod_hojre_farve5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mand_lober", "1F3C3-200D-2642-FE0F"},
    {"mand_lober_farve1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"mand_lober_farve2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"mand_lober_farve3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"mand_lober_farve4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"mand_lober_farve5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"mand_lober_mod_hojre", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_lober_mod_hojre_farve1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_lober_mod_hojre_farve2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_lober_mod_hojre_farve3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_lober_mod_hojre_farve4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_lober_mod_hojre_farve5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kvindelig_surfer", "1F3C4-200D-2640-FE0F"},
    {"kvindelig_surfer_farve1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"kvindelig_surfer_farve2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"kvindelig_surfer_farve3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"kvindelig_surfer_farve4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"kvindelig_surfer_farve5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"mandlig_surfer", "1F3C4-200D-2642-FE0F"},
    {"mandlig_surfer_farve1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"mandlig_surfer_farve2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"mandlig_surfer_farve3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"mandlig_surfer_farve4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"mandlig_surfer_farve5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"kvindelig_svommer", "1F3CA-200D-2640-FE0F"},
    {"kvindelig_svommer_farve1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"kvindelig_svommer_farve2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"kvindelig_svommer_farve3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"kvindelig_svommer_farve4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"kvindelig_svommer_farve5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"mandlig_svommer", "1F3CA-200D-2642-FE0F"},
    {"mandlig_svommer_farve1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"mandlig_svommer_farve2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"mandlig_svommer_farve3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"mandlig_svommer_farve4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"mandlig_svommer_farve5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"kvindelig_vaegtlofter", "1F3CB-FE0F-200D-2640-FE0F"},
    {"kvindelig_vaegtlofter_farve1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"kvindelig_vaegtlofter_farve2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"kvindelig_vaegtlofter_farve3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"kvindelig_vaegtlofter_farve4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"kvindelig_vaegtlofter_farve5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"mandlig_vaegtlofter", "1F3CB-FE0F-200D-2642-FE0F"},
    {"mandlig_vaegtlofter_farve1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"mandlig_vaegtlofter_farve2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"mandlig_vaegtlofter_farve3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"mandlig_vaegtlofter_farve4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"mandlig_vaegtlofter_farve5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"kvindelig_golfspiller", "1F3CC-FE0F-200D-2640-FE0F"},
    {"kvindelig_golfspiller_farve1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"kvindelig_golfspiller_farve2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"kvindelig_golfspiller_farve3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"kvindelig_golfspiller_farve4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"kvindelig_golfspiller_farve5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"mandlig_golfspiller", "1F3CC-FE0F-200D-2642-FE0F"},
    {"mandlig_golfspiller_farve1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"mandlig_golfspiller_farve2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"mandlig_golfspiller_farve3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"mandlig_golfspiller_farve4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"mandlig_golfspiller_farve5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"kvindelig_politibetjent", "1F46E-200D-2640-FE0F"},
    {"kvindelig_politibetjent_farve1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"kvindelig_politibetjent_farve2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"kvindelig_politibetjent_farve3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"kvindelig_politibetjent_farve4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"kvindelig_politibetjent_farve5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"mandlig_politibetjent", "1F46E-200D-2642-FE0F"},
    {"mandlig_politibetjent_farve1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"mandlig_politibetjent_farve2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"mandlig_politibetjent_farve3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"mandlig_politibetjent_farve4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"mandlig_politibetjent_farve5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"kvinder_med_kaninorer", "1F46F-200D-2640-FE0F"},
    {"kvinder_med_kaninorer_farve1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"kvinder_med_kaninorer_farve1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"kvinder_med_kaninorer_farve1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"kvinder_med_kaninorer_farve1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"kvinder_med_kaninorer_farve2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"kvinder_med_kaninorer_farve2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"kvinder_med_kaninorer_farve2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"kvinder_med_kaninorer_farve2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"kvinder_med_kaninorer_farve3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"kvinder_med_kaninorer_farve3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"kvinder_med_kaninorer_farve3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"kvinder_med_kaninorer_farve3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"kvinder_med_kaninorer_farve4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"kvinder_med_kaninorer_farve4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"kvinder_med_kaninorer_farve4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"kvinder_med_kaninorer_farve4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"kvinder_med_kaninorer_farve5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"kvinder_med_kaninorer_farve5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"kvinder_med_kaninorer_farve5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"kvinder_med_kaninorer_farve5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"kvinder_med_kaninorer_farve1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"kvinder_med_kaninorer_farve2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"kvinder_med_kaninorer_farve3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"kvinder_med_kaninorer_farve4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"kvinder_med_kaninorer_farve5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"maend_med_kaninorer", "1F46F-200D-2642-FE0F"},
    {"maend_med_kaninorer_farve1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"maend_med_kaninorer_farve1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"maend_med_kaninorer_farve1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"maend_med_kaninorer_farve1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"maend_med_kaninorer_farve2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"maend_med_kaninorer_farve2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"maend_med_kaninorer_farve2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"maend_med_kaninorer_farve2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"maend_med_kaninorer_farve3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"maend_med_kaninorer_farve3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"maend_med_kaninorer_farve3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"maend_med_kaninorer_farve3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"maend_med_kaninorer_farve4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"maend_med_kaninorer_farve4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"maend_med_kaninorer_farve4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"maend_med_kaninorer_farve4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"maend_med_kaninorer_farve5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"maend_med_kaninorer_farve5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"maend_med_kaninorer_farve5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"maend_med_kaninorer_farve5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"maend_med_kaninorer_farve1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"maend_med_kaninorer_farve2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"maend_med_kaninorer_farve3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"maend_med_kaninorer_farve4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"maend_med_kaninorer_farve5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"kvinde_med_brudeslor", "1F470-200D-2640-FE0F"},
    {"kvinde_med_brudeslor_farve1", "1F470-1F3FB-200D-2640-FE0F"},
    {"kvinde_med_brudeslor_farve2", "1F470-1F3FC-200D-2640-FE0F"},
    {"kvinde_med_brudeslor_farve3", "1F470-1F3FD-200D-2640-FE0F"},
    {"kvinde_med_brudeslor_farve4", "1F470-1F3FE-200D-2640-FE0F"},
    {"kvinde_med_brudeslor_farve5", "1F470-1F3FF-200D-2640-FE0F"},
    {"mand_med_brudeslor", "1F470-200D-2642-FE0F"},
    {"mand_med_brudeslor_farve1", "1F470-1F3FB-200D-2642-FE0F"},
    {"mand_med_brudeslor_farve2", "1F470-1F3FC-200D-2642-FE0F"},
    {"mand_med_brudeslor_farve3", "1F470-1F3FD-200D-2642-FE0F"},
    {"mand_med_brudeslor_farve4", "1F470-1F3FE-200D-2642-FE0F"},
    {"mand_med_brudeslor_farve5", "1F470-1F3FF-200D-2642-FE0F"},
    {"blond_kvinde", "1F471-200D-2640-FE0F"},
    {"blond_kvinde_farve1", "1F471-1F3FB-200D-2640-FE0F"},
    {"blond_kvinde_farve2", "1F471-1F3FC-200D-2640-FE0F"},
    {"blond_kvinde_farve3", "1F471-1F3FD-200D-2640-FE0F"},
    {"blond_kvinde_farve4", "1F471-1F3FE-200D-2640-FE0F"},
    {"blond_kvinde_farve5", "1F471-1F3FF-200D-2640-FE0F"},
    {"blond_mand", "1F471-200D-2642-FE0F"},
    {"blond_mand_farve1", "1F471-1F3FB-200D-2642-FE0F"},
    {"blond_mand_farve2", "1F471-1F3FC-200D-2642-FE0F"},
    {"blond_mand_farve3", "1F471-1F3FD-200D-2642-FE0F"},
    {"blond_mand_farve4", "1F471-1F3FE-200D-2642-FE0F"},
    {"blond_mand_farve5", "1F471-1F3FF-200D-2642-FE0F"},
    {"kvinde_med_turban", "1F473-200D-2640-FE0F"},
    {"kvinde_med_turban_farve1", "1F473-1F3FB-200D-2640-FE0F"},
    {"kvinde_med_turban_farve2", "1F473-1F3FC-200D-2640-FE0F"},
    {"kvinde_med_turban_farve3", "1F473-1F3FD-200D-2640-FE0F"},
    {"kvinde_med_turban_farve4", "1F473-1F3FE-200D-2640-FE0F"},
    {"kvinde_med_turban_farve5", "1F473-1F3FF-200D-2640-FE0F"},
    {"mand_med_turban", "1F473-200D-2642-FE0F"},
    {"mand_med_turban_farve1", "1F473-1F3FB-200D-2642-FE0F"},
    {"mand_med_turban_farve2", "1F473-1F3FC-200D-2642-FE0F"},
    {"mand_med_turban_farve3", "1F473-1F3FD-200D-2642-FE0F"},
    {"mand_med_turban_farve4", "1F473-1F3FE-200D-2642-FE0F"},
    {"mand_med_turban_farve5", "1F473-1F3FF-200D-2642-FE0F"},
    {"kvindelig_bygningsarbejder", "1F477-200D-2640-FE0F"},
    {"kvindelig_bygningsarbejder_farve1", "1F477-1F3FB-200D-2640-FE0F"},
    {"kvindelig_bygningsarbejder_farve2", "1F477-1F3FC-200D-2640-FE0F"},
    {"kvindelig_bygningsarbejder_farve3", "1F477-1F3FD-200D-2640-FE0F"},
    {"kvindelig_bygningsarbejder_farve4", "1F477-1F3FE-200D-2640-FE0F"},
    {"kvindelig_bygningsarbejder_farve5", "1F477-1F3FF-200D-2640-FE0F"},
    {"mandlig_bygningsarbejder", "1F477-200D-2642-FE0F"},
    {"mandlig_bygningsarbejder_farve1", "1F477-1F3FB-200D-2642-FE0F"},
    {"mandlig_bygningsarbejder_farve2", "1F477-1F3FC-200D-2642-FE0F"},
    {"mandlig_bygningsarbejder_farve3", "1F477-1F3FD-200D-2642-FE0F"},
    {"mandlig_bygningsarbejder_farve4", "1F477-1F3FE-200D-2642-FE0F"},
    {"mandlig_bygningsarbejder_farve5", "1F477-1F3FF-200D-2642-FE0F"},
    {"kvinde_vender_handfladen_op", "1F481-200D-2640-FE0F"},
    {"kvinde_vender_handfladen_op_farve1", "1F481-1F3FB-200D-2640-FE0F"},
    {"kvinde_vender_handfladen_op_farve2", "1F481-1F3FC-200D-2640-FE0F"},
    {"kvinde_vender_handfladen_op_farve3", "1F481-1F3FD-200D-2640-FE0F"},
    {"kvinde_vender_handfladen_op_farve4", "1F481-1F3FE-200D-2640-FE0F"},
    {"kvinde_vender_handfladen_op_farve5", "1F481-1F3FF-200D-2640-FE0F"},
    {"mand_vender_handfladen_op", "1F481-200D-2642-FE0F"},
    {"mand_vender_handfladen_op_farve1", "1F481-1F3FB-200D-2642-FE0F"},
    {"mand_vender_handfladen_op_farve2", "1F481-1F3FC-200D-2642-FE0F"},
    {"mand_vender_handfladen_op_farve3", "1F481-1F3FD-200D-2642-FE0F"},
    {"mand_vender_handfladen_op_farve4", "1F481-1F3FE-200D-2642-FE0F"},
    {"mand_vender_handfladen_op_farve5", "1F481-1F3FF-200D-2642-FE0F"},
    {"kvindelig_garder", "1F482-200D-2640-FE0F"},
    {"kvindelig_garder_farve1", "1F482-1F3FB-200D-2640-FE0F"},
    {"kvindelig_garder_farve2", "1F482-1F3FC-200D-2640-FE0F"},
    {"kvindelig_garder_farve3", "1F482-1F3FD-200D-2640-FE0F"},
    {"kvindelig_garder_farve4", "1F482-1F3FE-200D-2640-FE0F"},
    {"kvindelig_garder_farve5", "1F482-1F3FF-200D-2640-FE0F"},
    {"mandlig_garder", "1F482-200D-2642-FE0F"},
    {"mandlig_garder_farve1", "1F482-1F3FB-200D-2642-FE0F"},
    {"mandlig_garder_farve2", "1F482-1F3FC-200D-2642-FE0F"},
    {"mandlig_garder_farve3", "1F482-1F3FD-200D-2642-FE0F"},
    {"mandlig_garder_farve4", "1F482-1F3FE-200D-2642-FE0F"},
    {"mandlig_garder_farve5", "1F482-1F3FF-200D-2642-FE0F"},
    {"kvinde_far_ansigtsmassage", "1F486-200D-2640-FE0F"},
    {"kvinde_far_ansigtsmassage_farve1", "1F486-1F3FB-200D-2640-FE0F"},
    {"kvinde_far_ansigtsmassage_farve2", "1F486-1F3FC-200D-2640-FE0F"},
    {"kvinde_far_ansigtsmassage_farve3", "1F486-1F3FD-200D-2640-FE0F"},
    {"kvinde_far_ansigtsmassage_farve4", "1F486-1F3FE-200D-2640-FE0F"},
    {"kvinde_far_ansigtsmassage_farve5", "1F486-1F3FF-200D-2640-FE0F"},
    {"mand_far_ansigtsmassage", "1F486-200D-2642-FE0F"},
    {"mand_far_ansigtsmassage_farve1", "1F486-1F3FB-200D-2642-FE0F"},
    {"mand_far_ansigtsmassage_farve2", "1F486-1F3FC-200D-2642-FE0F"},
    {"mand_far_ansigtsmassage_farve3", "1F486-1F3FD-200D-2642-FE0F"},
    {"mand_far_ansigtsmassage_farve4", "1F486-1F3FE-200D-2642-FE0F"},
    {"mand_far_ansigtsmassage_farve5", "1F486-1F3FF-200D-2642-FE0F"},
    {"kvinde_bliver_klippet", "1F487-200D-2640-FE0F"},
    {"kvinde_bliver_klippet_farve1", "1F487-1F3FB-200D-2640-FE0F"},
    {"kvinde_bliver_klippet_farve2", "1F487-1F3FC-200D-2640-FE0F"},
    {"kvinde_bliver_klippet_farve3", "1F487-1F3FD-200D-2640-FE0F"},
    {"kvinde_bliver_klippet_farve4", "1F487-1F3FE-200D-2640-FE0F"},
    {"kvinde_bliver_klippet_farve5", "1F487-1F3FF-200D-2640-FE0F"},
    {"mand_bliver_klippet", "1F487-200D-2642-FE0F"},
    {"mand_bliver_klippet_farve1", "1F487-1F3FB-200D-2642-FE0F"},
    {"mand_bliver_klippet_farve2", "1F487-1F3FC-200D-2642-FE0F"},
    {"mand_bliver_klippet_farve3", "1F487-1F3FD-200D-2642-FE0F"},
    {"mand_bliver_klippet_farve4", "1F487-1F3FE-200D-2642-FE0F"},
    {"mand_bliver_klippet_farve5", "1F487-1F3FF-200D-2642-FE0F"},
    {"kvindelig_detektiv", "1F575-FE0F-200D-2640-FE0F"},
    {"kvindelig_detektiv_farve1", "1F575-1F3FB-200D-2640-FE0F"},
    {"kvindelig_detektiv_farve2", "1F575-1F3FC-200D-2640-FE0F"},
    {"kvindelig_detektiv_farve3", "1F575-1F3FD-200D-2640-FE0F"},
    {"kvindelig_detektiv_farve4", "1F575-1F3FE-200D-2640-FE0F"},
    {"kvindelig_detektiv_farve5", "1F575-1F3FF-200D-2640-FE0F"},
    {"mandlig_detektiv", "1F575-FE0F-200D-2642-FE0F"},
    {"mandlig_detektiv_farve1", "1F575-1F3FB-200D-2642-FE0F"},
    {"mandlig_detektiv_farve2", "1F575-1F3FC-200D-2642-FE0F"},
    {"mandlig_detektiv_farve3", "1F575-1F3FD-200D-2642-FE0F"},
    {"mandlig_detektiv_farve4", "1F575-1F3FE-200D-2642-FE0F"},
    {"mandlig_detektiv_farve5", "1F575-1F3FF-200D-2642-FE0F"},
    {"kvinde_signalerer_nej", "1F645-200D-2640-FE0F"},
    {"kvinde_signalerer_nej_farve1", "1F645-1F3FB-200D-2640-FE0F"},
    {"kvinde_signalerer_nej_farve2", "1F645-1F3FC-200D-2640-FE0F"},
    {"kvinde_signalerer_nej_farve3", "1F645-1F3FD-200D-2640-FE0F"},
    {"kvinde_signalerer_nej_farve4", "1F645-1F3FE-200D-2640-FE0F"},
    {"kvinde_signalerer_nej_farve5", "1F645-1F3FF-200D-2640-FE0F"},
    {"mand_signalerer_nej", "1F645-200D-2642-FE0F"},
    {"mand_signalerer_nej_farve1", "1F645-1F3FB-200D-2642-FE0F"},
    {"mand_signalerer_nej_farve2", "1F645-1F3FC-200D-2642-FE0F"},
    {"mand_signalerer_nej_farve3", "1F645-1F3FD-200D-2642-FE0F"},
    {"mand_signalerer_nej_farve4", "1F645-1F3FE-200D-2642-FE0F"},
    {"mand_signalerer_nej_farve5", "1F645-1F3FF-200D-2642-FE0F"},
    {"kvinde_signalerer_ok", "1F646-200D-2640-FE0F"},
    {"kvinde_signalerer_ok_farve1", "1F646-1F3FB-200D-2640-FE0F"},
    {"kvinde_signalerer_ok_farve2", "1F646-1F3FC-200D-2640-FE0F"},
    {"kvinde_signalerer_ok_farve3", "1F646-1F3FD-200D-2640-FE0F"},
    {"kvinde_signalerer_ok_farve4", "1F646-1F3FE-200D-2640-FE0F"},
    {"kvinde_signalerer_ok_farve5", "1F646-1F3FF-200D-2640-FE0F"},
    {"mand_signalerer_ok", "1F646-200D-2642-FE0F"},
    {"mand_signalerer_ok_farve1", "1F646-1F3FB-200D-2642-FE0F"},
    {"mand_signalerer_ok_farve2", "1F646-1F3FC-200D-2642-FE0F"},
    {"mand_signalerer_ok_farve3", "1F646-1F3FD-200D-2642-FE0F"},
    {"mand_signalerer_ok_farve4", "1F646-1F3FE-200D-2642-FE0F"},
    {"mand_signalerer_ok_farve5", "1F646-1F3FF-200D-2642-FE0F"},
    {"kvinde_bukker_dybt", "1F647-200D-2640-FE0F"},
    {"kvinde_bukker_dybt_farve1", "1F647-1F3FB-200D-2640-FE0F"},
    {"kvinde_bukker_dybt_farve2", "1F647-1F3FC-200D-2640-FE0F"},
    {"kvinde_bukker_dybt_farve3", "1F647-1F3FD-200D-2640-FE0F"},
    {"kvinde_bukker_dybt_farve4", "1F647-1F3FE-200D-2640-FE0F"},
    {"kvinde_bukker_dybt_farve5", "1F647-1F3FF-200D-2640-FE0F"},
    {"mand_bukker_dybt", "1F647-200D-2642-FE0F"},
    {"mand_bukker_dybt_farve1", "1F647-1F3FB-200D-2642-FE0F"},
    {"mand_bukker_dybt_farve2", "1F647-1F3FC-200D-2642-FE0F"},
    {"mand_bukker_dybt_farve3", "1F647-1F3FD-200D-2642-FE0F"},
    {"mand_bukker_dybt_farve4", "1F647-1F3FE-200D-2642-FE0F"},
    {"mand_bukker_dybt_farve5", "1F647-1F3FF-200D-2642-FE0F"},
    {"kvinde_med_loftet_hand", "1F64B-200D-2640-FE0F"},
    {"kvinde_med_loftet_hand_farve1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"kvinde_med_loftet_hand_farve2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"kvinde_med_loftet_hand_farve3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"kvinde_med_loftet_hand_farve4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"kvinde_med_loftet_hand_farve5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"mand_med_loftet_hand", "1F64B-200D-2642-FE0F"},
    {"mand_med_loftet_hand_farve1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"mand_med_loftet_hand_farve2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"mand_med_loftet_hand_farve3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"mand_med_loftet_hand_farve4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"mand_med_loftet_hand_farve5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"utilfreds_kvinde", "1F64D-200D-2640-FE0F"},
    {"utilfreds_kvinde_farve1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"utilfreds_kvinde_farve2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"utilfreds_kvinde_farve3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"utilfreds_kvinde_farve4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"utilfreds_kvinde_farve5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"utilfreds_mand", "1F64D-200D-2642-FE0F"},
    {"utilfreds_mand_farve1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"utilfreds_mand_farve2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"utilfreds_mand_farve3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"utilfreds_mand_farve4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"utilfreds_mand_farve5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"surmulende_kvinde", "1F64E-200D-2640-FE0F"},
    {"surmulende_kvinde_farve1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"surmulende_kvinde_farve2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"surmulende_kvinde_farve3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"surmulende_kvinde_farve4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"surmulende_kvinde_farve5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"surmulende_mand", "1F64E-200D-2642-FE0F"},
    {"surmulende_mand_farve1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"surmulende_mand_farve2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"surmulende_mand_farve3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"surmulende_mand_farve4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"surmulende_mand_farve5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"kvinde_i_robad", "1F6A3-200D-2640-FE0F"},
    {"kvinde_i_robad_farve1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"kvinde_i_robad_farve2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"kvinde_i_robad_farve3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"kvinde_i_robad_farve4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"kvinde_i_robad_farve5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"mand_i_robad", "1F6A3-200D-2642-FE0F"},
    {"mand_i_robad_farve1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"mand_i_robad_farve2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"mand_i_robad_farve3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"mand_i_robad_farve4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"mand_i_robad_farve5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"kvindelig_cyklist", "1F6B4-200D-2640-FE0F"},
    {"kvindelig_cyklist_farve1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"kvindelig_cyklist_farve2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"kvindelig_cyklist_farve3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"kvindelig_cyklist_farve4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"kvindelig_cyklist_farve5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"mandlig_cyklist", "1F6B4-200D-2642-FE0F"},
    {"mandlig_cyklist_farve1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"mandlig_cyklist_farve2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"mandlig_cyklist_farve3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"mandlig_cyklist_farve4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"mandlig_cyklist_farve5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"kvinde_pa_mountainbike", "1F6B5-200D-2640-FE0F"},
    {"kvinde_pa_mountainbike_farve1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"kvinde_pa_mountainbike_farve2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"kvinde_pa_mountainbike_farve3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"kvinde_pa_mountainbike_farve4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"kvinde_pa_mountainbike_farve5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"mand_pa_mountainbike", "1F6B5-200D-2642-FE0F"},
    {"mand_pa_mountainbike_farve1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"mand_pa_mountainbike_farve2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"mand_pa_mountainbike_farve3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"mand_pa_mountainbike_farve4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"mand_pa_mountainbike_farve5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"kvinde_gar", "1F6B6-200D-2640-FE0F"},
    {"kvinde_gar_farve1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"kvinde_gar_farve2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"kvinde_gar_farve3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"kvinde_gar_farve4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"kvinde_gar_farve5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"kvinde_gar_mod_hojre", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_gar_mod_hojre_farve1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_gar_mod_hojre_farve2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_gar_mod_hojre_farve3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_gar_mod_hojre_farve4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_gar_mod_hojre_farve5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mand_gar", "1F6B6-200D-2642-FE0F"},
    {"mand_gar_farve1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"mand_gar_farve2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"mand_gar_farve3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"mand_gar_farve4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"mand_gar_farve5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"mand_gar_mod_hojre", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_gar_mod_hojre_farve1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_gar_mod_hojre_farve2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_gar_mod_hojre_farve3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_gar_mod_hojre_farve4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_gar_mod_hojre_farve5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kvinde_slar_sig_for_panden", "1F926-200D-2640-FE0F"},
    {"kvinde_slar_sig_for_panden_farve1", "1F926-1F3FB-200D-2640-FE0F"},
    {"kvinde_slar_sig_for_panden_farve2", "1F926-1F3FC-200D-2640-FE0F"},
    {"kvinde_slar_sig_for_panden_farve3", "1F926-1F3FD-200D-2640-FE0F"},
    {"kvinde_slar_sig_for_panden_farve4", "1F926-1F3FE-200D-2640-FE0F"},
    {"kvinde_slar_sig_for_panden_farve5", "1F926-1F3FF-200D-2640-FE0F"},
    {"mand_slar_sig_for_panden", "1F926-200D-2642-FE0F"},
    {"mand_slar_sig_for_panden_farve1", "1F926-1F3FB-200D-2642-FE0F"},
    {"mand_slar_sig_for_panden_farve2", "1F926-1F3FC-200D-2642-FE0F"},
    {"mand_slar_sig_for_panden_farve3", "1F926-1F3FD-200D-2642-FE0F"},
    {"mand_slar_sig_for_panden_farve4", "1F926-1F3FE-200D-2642-FE0F"},
    {"mand_slar_sig_for_panden_farve5", "1F926-1F3FF-200D-2642-FE0F"},
    {"kvinde_i_smoking", "1F935-200D-2640-FE0F"},
    {"kvinde_i_smoking_farve1", "1F935-1F3FB-200D-2640-FE0F"},
    {"kvinde_i_smoking_farve2", "1F935-1F3FC-200D-2640-FE0F"},
    {"kvinde_i_smoking_farve3", "1F935-1F3FD-200D-2640-FE0F"},
    {"kvinde_i_smoking_farve4", "1F935-1F3FE-200D-2640-FE0F"},
    {"kvinde_i_smoking_farve5", "1F935-1F3FF-200D-2640-FE0F"},
    {"mand_i_smoking", "1F935-200D-2642-FE0F"},
    {"mand_i_smoking_farve1", "1F935-1F3FB-200D-2642-FE0F"},
    {"mand_i_smoking_farve2", "1F935-1F3FC-200D-2642-FE0F"},
    {"mand_i_smoking_farve3", "1F935-1F3FD-200D-2642-FE0F"},
    {"mand_i_smoking_farve4", "1F935-1F3FE-200D-2642-FE0F"},
    {"mand_i_smoking_farve5", "1F935-1F3FF-200D-2642-FE0F"},
    {"kvinde_traekker_pa_skuldrene", "1F937-200D-2640-FE0F"},
    {"kvinde_traekker_pa_skuldrene_farve1", "1F937-1F3FB-200D-2640-FE0F"},
    {"kvinde_traekker_pa_skuldrene_farve2", "1F937-1F3FC-200D-2640-FE0F"},
    {"kvinde_traekker_pa_skuldrene_farve3", "1F937-1F3FD-200D-2640-FE0F"},
    {"kvinde_traekker_pa_skuldrene_farve4", "1F937-1F3FE-200D-2640-FE0F"},
    {"kvinde_traekker_pa_skuldrene_farve5", "1F937-1F3FF-200D-2640-FE0F"},
    {"mand_traekker_pa_skuldrene", "1F937-200D-2642-FE0F"},
    {"mand_traekker_pa_skuldrene_farve1", "1F937-1F3FB-200D-2642-FE0F"},
    {"mand_traekker_pa_skuldrene_farve2", "1F937-1F3FC-200D-2642-FE0F"},
    {"mand_traekker_pa_skuldrene_farve3", "1F937-1F3FD-200D-2642-FE0F"},
    {"mand_traekker_pa_skuldrene_farve4", "1F937-1F3FE-200D-2642-FE0F"},
    {"mand_traekker_pa_skuldrene_farve5", "1F937-1F3FF-200D-2642-FE0F"},
    {"kvinde_slar_vejrmoller", "1F938-200D-2640-FE0F"},
    {"kvinde_slar_vejrmoller_farve1", "1F938-1F3FB-200D-2640-FE0F"},
    {"kvinde_slar_vejrmoller_farve2", "1F938-1F3FC-200D-2640-FE0F"},
    {"kvinde_slar_vejrmoller_farve3", "1F938-1F3FD-200D-2640-FE0F"},
    {"kvinde_slar_vejrmoller_farve4", "1F938-1F3FE-200D-2640-FE0F"},
    {"kvinde_slar_vejrmoller_farve5", "1F938-1F3FF-200D-2640-FE0F"},
    {"mand_slar_vejrmoller", "1F938-200D-2642-FE0F"},
    {"mand_slar_vejrmoller_farve1", "1F938-1F3FB-200D-2642-FE0F"},
    {"mand_slar_vejrmoller_farve2", "1F938-1F3FC-200D-2642-FE0F"},
    {"mand_slar_vejrmoller_farve3", "1F938-1F3FD-200D-2642-FE0F"},
    {"mand_slar_vejrmoller_farve4", "1F938-1F3FE-200D-2642-FE0F"},
    {"mand_slar_vejrmoller_farve5", "1F938-1F3FF-200D-2642-FE0F"},
    {"kvindelig_jonglor", "1F939-200D-2640-FE0F"},
    {"kvindelig_jonglor_farve1", "1F939-1F3FB-200D-2640-FE0F"},
    {"kvindelig_jonglor_farve2", "1F939-1F3FC-200D-2640-FE0F"},
    {"kvindelig_jonglor_farve3", "1F939-1F3FD-200D-2640-FE0F"},
    {"kvindelig_jonglor_farve4", "1F939-1F3FE-200D-2640-FE0F"},
    {"kvindelig_jonglor_farve5", "1F939-1F3FF-200D-2640-FE0F"},
    {"mandlig_jonglor", "1F939-200D-2642-FE0F"},
    {"mandlig_jonglor_farve1", "1F939-1F3FB-200D-2642-FE0F"},
    {"mandlig_jonglor_farve2", "1F939-1F3FC-200D-2642-FE0F"},
    {"mandlig_jonglor_farve3", "1F939-1F3FD-200D-2642-FE0F"},
    {"mandlig_jonglor_farve4", "1F939-1F3FE-200D-2642-FE0F"},
    {"mandlig_jonglor_farve5", "1F939-1F3FF-200D-2642-FE0F"},
    {"kvindelige_brydere", "1F93C-200D-2640-FE0F"},
    {"kvindelige_brydere_farve1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"kvindelige_brydere_farve1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"kvindelige_brydere_farve1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"kvindelige_brydere_farve1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"kvindelige_brydere_farve2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"kvindelige_brydere_farve2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"kvindelige_brydere_farve2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"kvindelige_brydere_farve2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"kvindelige_brydere_farve3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"kvindelige_brydere_farve3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"kvindelige_brydere_farve3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"kvindelige_brydere_farve3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"kvindelige_brydere_farve4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"kvindelige_brydere_farve4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"kvindelige_brydere_farve4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"kvindelige_brydere_farve4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"kvindelige_brydere_farve5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"kvindelige_brydere_farve5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"kvindelige_brydere_farve5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"kvindelige_brydere_farve5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"kvindelige_brydere_farve1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"kvindelige_brydere_farve2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"kvindelige_brydere_farve3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"kvindelige_brydere_farve4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"kvindelige_brydere_farve5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"mandlige_brydere", "1F93C-200D-2642-FE0F"},
    {"mandlige_brydere_farve1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"mandlige_brydere_farve1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"mandlige_brydere_farve1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"mandlige_brydere_farve1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"mandlige_brydere_farve2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"mandlige_brydere_farve2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"mandlige_brydere_farve2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"mandlige_brydere_farve2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"mandlige_brydere_farve3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"mandlige_brydere_farve3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"mandlige_brydere_farve3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"mandlige_brydere_farve3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"mandlige_brydere_farve4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"mandlige_brydere_farve4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"mandlige_brydere_farve4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"mandlige_brydere_farve4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"mandlige_brydere_farve5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"mandlige_brydere_farve5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"mandlige_brydere_farve5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"mandlige_brydere_farve5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"mandlige_brydere_farve1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"mandlige_brydere_farve2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"mandlige_brydere_farve3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"mandlige_brydere_farve4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"mandlige_brydere_farve5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"kvindelig_vandpolospiller", "1F93D-200D-2640-FE0F"},
    {"kvindelig_vandpolospiller_farve1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"kvindelig_vandpolospiller_farve2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"kvindelig_vandpolospiller_farve3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"kvindelig_vandpolospiller_farve4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"kvindelig_vandpolospiller_farve5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"mandlig_vandpolospiller", "1F93D-200D-2642-FE0F"},
    {"mandlig_vandpolospiller_farve1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"mandlig_vandpolospiller_farve2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"mandlig_vandpolospiller_farve3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"mandlig_vandpolospiller_farve4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"mandlig_vandpolospiller_farve5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"kvindelig_handboldspiller", "1F93E-200D-2640-FE0F"},
    {"kvindelig_handboldspiller_farve1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"kvindelig_handboldspiller_farve2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"kvindelig_handboldspiller_farve3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"kvindelig_handboldspiller_farve4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"kvindelig_handboldspiller_farve5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"mandlig_handboldspiller", "1F93E-200D-2642-FE0F"},
    {"mandlig_handboldspiller_farve1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"mandlig_handboldspiller_farve2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"mandlig_handboldspiller_farve3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"mandlig_handboldspiller_farve4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"mandlig_handboldspiller_farve5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"kvindelig_superhelt", "1F9B8-200D-2640-FE0F"},
    {"kvindelig_superhelt_farve1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"kvindelig_superhelt_farve2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"kvindelig_superhelt_farve3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"kvindelig_superhelt_farve4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"kvindelig_superhelt_farve5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"mandlig_superhelt", "1F9B8-200D-2642-FE0F"},
    {"mandlig_superhelt_farve1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"mandlig_superhelt_farve2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"mandlig_superhelt_farve3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"mandlig_superhelt_farve4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"mandlig_superhelt_farve5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"kvindelig_superskurk", "1F9B9-200D-2640-FE0F"},
    {"kvindelig_superskurk_farve1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"kvindelig_superskurk_farve2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"kvindelig_superskurk_farve3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"kvindelig_superskurk_farve4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"kvindelig_superskurk_farve5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"mandlig_superskurk", "1F9B9-200D-2642-FE0F"},
    {"mandlig_superskurk_farve1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"mandlig_superskurk_farve2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"mandlig_superskurk_farve3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"mandlig_superskurk_farve4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"mandlig_superskurk_farve5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"staende_kvinde", "1F9CD-200D-2640-FE0F"},
    {"staende_kvinde_farve1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"staende_kvinde_farve2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"staende_kvinde_farve3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"staende_kvinde_farve4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"staende_kvinde_farve5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"staende_mand", "1F9CD-200D-2642-FE0F"},
    {"staende_mand_farve1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"staende_mand_farve2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"staende_mand_farve3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"staende_mand_farve4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"staende_mand_farve5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"knaelende_kvinde", "1F9CE-200D-2640-FE0F"},
    {"knaelende_kvinde_farve1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"knaelende_kvinde_farve2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"knaelende_kvinde_farve3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"knaelende_kvinde_farve4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"knaelende_kvinde_farve5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"knaelende_kvinde_mod_hojre", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knaelende_kvinde_mod_hojre_farve1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knaelende_kvinde_mod_hojre_farve2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knaelende_kvinde_mod_hojre_farve3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knaelende_kvinde_mod_hojre_farve4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knaelende_kvinde_mod_hojre_farve5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knaelende_mand", "1F9CE-200D-2642-FE0F"},
    {"knaelende_mand_farve1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"knaelende_mand_farve2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"knaelende_mand_farve3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"knaelende_mand_farve4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"knaelende_mand_farve5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"knaelende_mand_mod_hojre", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knaelende_mand_mod_hojre_farve1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knaelende_mand_mod_hojre_farve2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knaelende_mand_mod_hojre_farve3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knaelende_mand_mod_hojre_farve4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knaelende_mand_mod_hojre_farve5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"dov_kvinde", "1F9CF-200D-2640-FE0F"},
    {"dov_kvinde_farve1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"dov_kvinde_farve2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"dov_kvinde_farve3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"dov_kvinde_farve4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"dov_kvinde_farve5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"dov_mand", "1F9CF-200D-2642-FE0F"},
    {"dov_mand_farve1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"dov_mand_farve2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"dov_mand_farve3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"dov_mand_farve4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"dov_mand_farve5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"kvinde_med_skaeg", "1F9D4-200D-2640-FE0F"},
    {"kvinde_med_skaeg_farve1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"kvinde_med_skaeg_farve2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"kvinde_med_skaeg_farve3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"kvinde_med_skaeg_farve4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"kvinde_med_skaeg_farve5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"mand_med_skaeg", "1F9D4-200D-2642-FE0F"},
    {"mand_med_skaeg_farve1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"mand_med_skaeg_farve2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"mand_med_skaeg_farve3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"mand_med_skaeg_farve4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"mand_med_skaeg_farve5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"kvinde_i_dampbad", "1F9D6-200D-2640-FE0F"},
    {"kvinde_i_dampbad_farve1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"kvinde_i_dampbad_farve2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"kvinde_i_dampbad_farve3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"kvinde_i_dampbad_farve4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"kvinde_i_dampbad_farve5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"mand_i_dampbad", "1F9D6-200D-2642-FE0F"},
    {"mand_i_dampbad_farve1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"mand_i_dampbad_farve2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"mand_i_dampbad_farve3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"mand_i_dampbad_farve4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"mand_i_dampbad_farve5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"kvindelig_bjergbestiger", "1F9D7-200D-2640-FE0F"},
    {"kvindelig_bjergbestiger_farve1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"kvindelig_bjergbestiger_farve2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"kvindelig_bjergbestiger_farve3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"kvindelig_bjergbestiger_farve4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"kvindelig_bjergbestiger_farve5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"mandlig_bjergbestiger", "1F9D7-200D-2642-FE0F"},
    {"mandlig_bjergbestiger_farve1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"mandlig_bjergbestiger_farve2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"mandlig_bjergbestiger_farve3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"mandlig_bjergbestiger_farve4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"mandlig_bjergbestiger_farve5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"kvinde_i_lotusstilling", "1F9D8-200D-2640-FE0F"},
    {"kvinde_i_lotusstilling_farve1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"kvinde_i_lotusstilling_farve2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"kvinde_i_lotusstilling_farve3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"kvinde_i_lotusstilling_farve4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"kvinde_i_lotusstilling_farve5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"mand_i_lotusstilling", "1F9D8-200D-2642-FE0F"},
    {"mand_i_lotusstilling_farve1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"mand_i_lotusstilling_farve2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"mand_i_lotusstilling_farve3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"mand_i_lotusstilling_farve4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"mand_i_lotusstilling_farve5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"kvindelig_magiker", "1F9D9-200D-2640-FE0F"},
    {"kvindelig_magiker_farve1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"kvindelig_magiker_farve2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"kvindelig_magiker_farve3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"kvindelig_magiker_farve4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"kvindelig_magiker_farve5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"mandlig_magiker", "1F9D9-200D-2642-FE0F"},
    {"mandlig_magiker_farve1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"mandlig_magiker_farve2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"mandlig_magiker_farve3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"mandlig_magiker_farve4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"mandlig_magiker_farve5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"kvindelig_fe", "1F9DA-200D-2640-FE0F"},
    {"kvindelig_fe_farve1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"kvindelig_fe_farve2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"kvindelig_fe_farve3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"kvindelig_fe_farve4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"kvindelig_fe_farve5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"mandlig_fe", "1F9DA-200D-2642-FE0F"},
    {"mandlig_fe_farve1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"mandlig_fe_farve2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"mandlig_fe_farve3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"mandlig_fe_farve4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"mandlig_fe_farve5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"kvindelig_vampyr", "1F9DB-200D-2640-FE0F"},
    {"kvindelig_vampyr_farve1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"kvindelig_vampyr_farve2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"kvindelig_vampyr_farve3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"kvindelig_vampyr_farve4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"kvindelig_vampyr_farve5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"mandlig_vampyr", "1F9DB-200D-2642-FE0F"},
    {"mandlig_vampyr_farve1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"mandlig_vampyr_farve2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"mandlig_vampyr_farve3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"mandlig_vampyr_farve4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"mandlig_vampyr_farve5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"havfrue", "1F9DC-200D-2640-FE0F"},
    {"havfrue_farve1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"havfrue_farve2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"havfrue_farve3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"havfrue_farve4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"havfrue_farve5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"havmand", "1F9DC-200D-2642-FE0F"},
    {"havmand_farve1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"havmand_farve2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"havmand_farve3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"havmand_farve4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"havmand_farve5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"elverkvinde", "1F9DD-200D-2640-FE0F"},
    {"elverkvinde_farve1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"elverkvinde_farve2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"elverkvinde_farve3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"elverkvinde_farve4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"elverkvinde_farve5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"elvermand", "1F9DD-200D-2642-FE0F"},
    {"elvermand_farve1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"elvermand_farve2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"elvermand_farve3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"elvermand_farve4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"elvermand_farve5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"kvindelig_magisk_and", "1F9DE-200D-2640-FE0F"},
    {"mandlig_magisk_and", "1F9DE-200D-2642-FE0F"},
    {"kvindelig_zombie", "1F9DF-200D-2640-FE0F"},
    {"mandlig_zombie", "1F9DF-200D-2642-FE0F"},
    {"mand_rodt_har", "1F468-200D-1F9B0"},
    {"mand_rodt_har_farve1", "1F468-1F3FB-200D-1F9B0"},
    {"mand_rodt_har_farve2", "1F468-1F3FC-200D-1F9B0"},
    {"mand_rodt_har_farve3", "1F468-1F3FD-200D-1F9B0"},
    {"mand_rodt_har_farve4", "1F468-1F3FE-200D-1F9B0"},
    {"mand_rodt_har_farve5", "1F468-1F3FF-200D-1F9B0"},
    {"mand_krollet_har", "1F468-200D-1F9B1"},
    {"mand_krollet_har_farve1", "1F468-1F3FB-200D-1F9B1"},
    {"mand_krollet_har_farve2", "1F468-1F3FC-200D-1F9B1"},
    {"mand_krollet_har_farve3", "1F468-1F3FD-200D-1F9B1"},
    {"mand_krollet_har_farve4", "1F468-1F3FE-200D-1F9B1"},
    {"mand_krollet_har_farve5", "1F468-1F3FF-200D-1F9B1"},
    {"mand_skaldet", "1F468-200D-1F9B2"},
    {"mand_skaldet_farve1", "1F468-1F3FB-200D-1F9B2"},
    {"mand_skaldet_farve2", "1F468-1F3FC-200D-1F9B2"},
    {"mand_skaldet_farve3", "1F468-1F3FD-200D-1F9B2"},
    {"mand_skaldet_farve4", "1F468-1F3FE-200D-1F9B2"},
    {"mand_skaldet_farve5", "1F468-1F3FF-200D-1F9B2"},
    {"mand_hvidt_har", "1F468-200D-1F9B3"},
    {"mand_hvidt_har_farve1", "1F468-1F3FB-200D-1F9B3"},
    {"mand_hvidt_har_farve2", "1F468-1F3FC-200D-1F9B3"},
    {"mand_hvidt_har_farve3", "1F468-1F3FD-200D-1F9B3"},
    {"mand_hvidt_har_farve4", "1F468-1F3FE-200D-1F9B3"},
    {"mand_hvidt_har_farve5", "1F468-1F3FF-200D-1F9B3"},
    {"kvinde_rodt_har", "1F469-200D-1F9B0"},
    {"kvinde_rodt_har_farve1", "1F469-1F3FB-200D-1F9B0"},
    {"kvinde_rodt_har_farve2", "1F469-1F3FC-200D-1F9B0"},
    {"kvinde_rodt_har_farve3", "1F469-1F3FD-200D-1F9B0"},
    {"kvinde_rodt_har_farve4", "1F469-1F3FE-200D-1F9B0"},
    {"kvinde_rodt_har_farve5", "1F469-1F3FF-200D-1F9B0"},
    {"kvinde_krollet_har", "1F469-200D-1F9B1"},
    {"kvinde_krollet_har_farve1", "1F469-1F3FB-200D-1F9B1"},
    {"kvinde_krollet_har_farve2", "1F469-1F3FC-200D-1F9B1"},
    {"kvinde_krollet_har_farve3", "1F469-1F3FD-200D-1F9B1"},
    {"kvinde_krollet_har_farve4", "1F469-1F3FE-200D-1F9B1"},
    {"kvinde_krollet_har_farve5", "1F469-1F3FF-200D-1F9B1"},
    {"kvinde_skaldet", "1F469-200D-1F9B2"},
    {"kvinde_skaldet_farve1", "1F469-1F3FB-200D-1F9B2"},
    {"kvinde_skaldet_farve2", "1F469-1F3FC-200D-1F9B2"},
    {"kvinde_skaldet_farve3", "1F469-1F3FD-200D-1F9B2"},
    {"kvinde_skaldet_farve4", "1F469-1F3FE-200D-1F9B2"},
    {"kvinde_skaldet_farve5", "1F469-1F3FF-200D-1F9B2"},
    {"kvinde_hvidt_har", "1F469-200D-1F9B3"},
    {"kvinde_hvidt_har_farve1", "1F469-1F3FB-200D-1F9B3"},
    {"kvinde_hvidt_har_farve2", "1F469-1F3FC-200D-1F9B3"},
    {"kvinde_hvidt_har_farve3", "1F469-1F3FD-200D-1F9B3"},
    {"kvinde_hvidt_har_farve4", "1F469-1F3FE-200D-1F9B3"},
    {"kvinde_hvidt_har_farve5", "1F469-1F3FF-200D-1F9B3"},
    {"voksen_rodt_har", "1F9D1-200D-1F9B0"},
    {"voksen_rodt_har_farve1", "1F9D1-1F3FB-200D-1F9B0"},
    {"voksen_rodt_har_farve2", "1F9D1-1F3FC-200D-1F9B0"},
    {"voksen_rodt_har_farve3", "1F9D1-1F3FD-200D-1F9B0"},
    {"voksen_rodt_har_farve4", "1F9D1-1F3FE-200D-1F9B0"},
    {"voksen_rodt_har_farve5", "1F9D1-1F3FF-200D-1F9B0"},
    {"voksen_krollet_har", "1F9D1-200D-1F9B1"},
    {"voksen_krollet_har_farve1", "1F9D1-1F3FB-200D-1F9B1"},
    {"voksen_krollet_har_farve2", "1F9D1-1F3FC-200D-1F9B1"},
    {"voksen_krollet_har_farve3", "1F9D1-1F3FD-200D-1F9B1"},
    {"voksen_krollet_har_farve4", "1F9D1-1F3FE-200D-1F9B1"},
    {"voksen_krollet_har_farve5", "1F9D1-1F3FF-200D-1F9B1"},
    {"voksen_skaldet", "1F9D1-200D-1F9B2"},
    {"voksen_skaldet_farve1", "1F9D1-1F3FB-200D-1F9B2"},
    {"voksen_skaldet_farve2", "1F9D1-1F3FC-200D-1F9B2"},
    {"voksen_skaldet_farve3", "1F9D1-1F3FD-200D-1F9B2"},
    {"voksen_skaldet_farve4", "1F9D1-1F3FE-200D-1F9B2"},
    {"voksen_skaldet_farve5", "1F9D1-1F3FF-200D-1F9B2"},
    {"voksen_hvidt_har", "1F9D1-200D-1F9B3"},
    {"voksen_hvidt_har_farve1", "1F9D1-1F3FB-200D-1F9B3"},
    {"voksen_hvidt_har_farve2", "1F9D1-1F3FC-200D-1F9B3"},
    {"voksen_hvidt_har_farve3", "1F9D1-1F3FD-200D-1F9B3"},
    {"voksen_hvidt_har_farve4", "1F9D1-1F3FE-200D-1F9B3"},
    {"voksen_hvidt_har_farve5", "1F9D1-1F3FF-200D-1F9B3"},
    {"brudt_kaede", "26D3-FE0F-200D-1F4A5"},
    {"braendende_hjerte", "2764-FE0F-200D-1F525"},
    {"helende_hjerte", "2764-FE0F-200D-1FA79"},
    {"brun_svamp", "1F344-200D-1F7EB"},
    {"lime", "1F34B-200D-1F7E9"},
    {"transkonnet_flag", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"regnbueflag", "1F3F3-FE0F-200D-1F308"},
    {"piratflag", "1F3F4-200D-2620-FE0F"},
    {"sort_kat", "1F408-200D-2B1B"},
    {"servicehund", "1F415-200D-1F9BA"},
    {"sort_fugl", "1F426-200D-2B1B"},
    {"foniks", "1F426-200D-1F525"},
    {"isbjorn", "1F43B-200D-2744-FE0F"},
    {"oje_i_taleboble", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"udandende_ansigt", "1F62E-200D-1F4A8"},
    {"ansigt_med_spiralojne", "1F635-200D-1F4AB"},
    {"ansigt_i_sky", "1F636-200D-1F32B-FE0F"},
    {"ryster_pa_hovedet_vandret", "1F642-200D-2194-FE0F"},
    {"ryster_pa_hovedet_lodret", "1F642-200D-2195-FE0F"},
    {"balletdanser", "1F9D1-200D-1FA70"},
    {"balletdanser_farve1", "1F9D1-1F3FB-200D-1FA70"},
    {"balletdanser_farve2", "1F9D1-1F3FC-200D-1FA70"},
    {"balletdanser_farve3", "1F9D1-1F3FD-200D-1FA70"},
    {"balletdanser_farve4", "1F9D1-1F3FE-200D-1FA70"},
    {"balletdanser_farve5", "1F9D1-1F3FF-200D-1FA70"},
    {"udråbstegn_og_spørgsmålstegn", "2049"},
    {"varemærke", "2122"},
    {"pil_mod_venstre_og_højre", "2194"},
    {"opadvendt_pil_mod_højre", "2197"},
    {"nedadvendt_pil_mod_højre", "2198"},
    {"paraply_med_regndråber", "2614"},
    {"kløver", "2618"},
    {"vædderen", "2648"},
    {"klør", "2663"},
    {"korslagte_sværd", "2694"},
    {"vægt", "2696"},
    {"rødt_spørgsmålstegn", "2753"},
    {"hvidt_spørgsmålstegn", "2754"},
    {"hvidt_udråbstegn", "2755"},
    {"rødt_udråbstegn", "2757"},
    {"hjerte_som_udråbstegn", "2763"},
    {"rødt_hjerte", "2764"},
    {"pil_mod_højre_med_sving_opad", "2934"},
    {"pil_mod_højre_med_sving_nedad", "2935"},
    {"bølgestreg", "3030"},
    {"cirkulært_ideogram_for_lykønskning", "3297"},
    {"cirkulært_ideogram_for_hemmelighed", "3299"},
    {"registreret_varemærke", "00AE"},
    {"dobbelt_udråbstegn", "203C"},
    {"pil_mod_højre_med_sving_til_venstre", "21A9"},
    {"pil_mod_venstre_med_sving_til_højre", "21AA"},
    {"udløbet_timeglas", "231B"},
    {"knap_til_næste_nummer", "23ED"},
    {"vækkeur", "23F0"},
    {"æggeur", "23F2"},
    {"timeglas_med_løbende_sand", "23F3"},
    {"knap_til_baglæns_afspilning", "25C0"},
    {"rækker_finger_op", "261D"},
    {"rækker_finger_op_farve1", "261D-1F3FB"},
    {"rækker_finger_op_farve2", "261D-1F3FC"},
    {"rækker_finger_op_farve3", "261D-1F3FD"},
    {"rækker_finger_op_farve4", "261D-1F3FE"},
    {"rækker_finger_op_farve5", "261D-1F3FF"},
    {"stjerne_og_halvmåne", "262A"},
    {"løven", "264C"},
    {"vægten", "264E"},
    {"symbol_for_kørestol", "267F"},
    {"højspænding", "26A1"},
    {"symbol_for_transkønnethed", "26A7"},
    {"slangebæreren", "26CE"},
    {"kæder", "26D3"},
    {"færge", "26F4"},
    {"sejlbåd", "26F5"},
    {"skiløber", "26F7"},
    {"skøjte", "26F8"},
    {"løftet_knytnæve", "270A"},
    {"løftet_knytnæve_farve1", "270A-1F3FB"},
    {"løftet_knytnæve_farve2", "270A-1F3FC"},
    {"løftet_knytnæve_farve3", "270A-1F3FD"},
    {"løftet_knytnæve_farve4", "270A-1F3FE"},
    {"løftet_knytnæve_farve5", "270A-1F3FF"},
    {"løftet_hånd", "270B"},
    {"løftet_hånd_farve1", "270B-1F3FB"},
    {"løftet_hånd_farve2", "270B-1F3FC"},
    {"løftet_hånd_farve3", "270B-1F3FD"},
    {"løftet_hånd_farve4", "270B-1F3FE"},
    {"løftet_hånd_farve5", "270B-1F3FF"},
    {"skrivende_hånd", "270D"},
    {"skrivende_hånd_farve1", "270D-1F3FB"},
    {"skrivende_hånd_farve2", "270D-1F3FC"},
    {"skrivende_hånd_farve3", "270D-1F3FD"},
    {"skrivende_hånd_farve4", "270D-1F3FE"},
    {"skrivende_hånd_farve5", "270D-1F3FF"},
    {"pil_mod_højre", "27A1"},
    {"krølle", "27B0"},
    {"dobbelt_krølle", "27BF"},
    {"stor_rød_cirkel", "2B55"},
    {"kvadratisk_ideogram_for_bestået", "1F234"},
    {"kvadratisk_ideogram_for_månedligt_beløb", "1F237"},
    {"kvadratisk_ideogram_for_åben", "1F23A"},
    {"cirkulært_ideogram_for_tilbud", "1F250"},
    {"cirkulært_ideogram_for_accept", "1F251"},
    {"tåget", "1F301"},
    {"bro_i_mørke", "1F309"},
    {"bølge", "1F30A"},
    {"mælkevejen", "1F30C"},
    {"nymåne", "1F311"},
    {"tiltagende_måne", "1F312"},
    {"halvmåne_1_fjerdedel", "1F313"},
    {"tiltagende_måne_næsten_fuld", "1F314"},
    {"fuldmåne", "1F315"},
    {"aftagende_måne_næsten_fuld", "1F316"},
    {"halvmåne_4_fjerdedel", "1F317"},
    {"aftagende_måne", "1F318"},
    {"halvmåne", "1F319"},
    {"nymåne_med_ansigt", "1F31A"},
    {"måne_med_ansigt_1_fjerdedel", "1F31B"},
    {"måne_med_ansigt_4_fjerdedel", "1F31C"},
    {"fuldmåne_med_ansigt", "1F31D"},
    {"tåge", "1F32B"},
    {"nåletræ", "1F332"},
    {"løvtræ", "1F333"},
    {"kirsebærblomst", "1F338"},
    {"firkløver", "1F340"},
    {"rødt_æble", "1F34E"},
    {"grønt_æble", "1F34F"},
    {"pære", "1F350"},
    {"kirsebær", "1F352"},
    {"jordbær", "1F353"},
    {"kød", "1F356"},
    {"kyllingelår", "1F357"},
    {"brød", "1F35E"},
    {"sød_kartoffel", "1F360"},
    {"mad_på_spyd", "1F362"},
    {"småkage", "1F36A"},
    {"ølkrus", "1F37A"},
    {"skålende_ølkrus", "1F37B"},
    {"sløjfe", "1F380"},
    {"fødselsdagskage", "1F382"},
    {"græskarmand", "1F383"},
    {"juletræ", "1F384"},
    {"fyrværkeri", "1F386"},
    {"træ_med_papirpynt", "1F38B"},
    {"fyrretræsdekoration", "1F38D"},
    {"måneceremoni", "1F391"},
    {"rygsæk", "1F392"},
    {"militærmedalje", "1F396"},
    {"mindebånd", "1F397"},
    {"høj_hat", "1F3A9"},
    {"klaptræ", "1F3AC"},
    {"løbetrøje", "1F3BD"},
    {"løber", "1F3C3"},
    {"løber_farve1", "1F3C3-1F3FB"},
    {"løber_farve2", "1F3C3-1F3FC"},
    {"løber_farve3", "1F3C3-1F3FD"},
    {"løber_farve4", "1F3C3-1F3FE"},
    {"løber_farve5", "1F3C3-1F3FF"},
    {"hestevæddeløb", "1F3C7"},
    {"hestevæddeløb_farve1", "1F3C7-1F3FB"},
    {"hestevæddeløb_farve2", "1F3C7-1F3FC"},
    {"hestevæddeløb_farve3", "1F3C7-1F3FD"},
    {"hestevæddeløb_farve4", "1F3C7-1F3FE"},
    {"hestevæddeløb_farve5", "1F3C7-1F3FF"},
    {"svømmer", "1F3CA"},
    {"svømmer_farve1", "1F3CA-1F3FB"},
    {"svømmer_farve2", "1F3CA-1F3FC"},
    {"svømmer_farve3", "1F3CA-1F3FD"},
    {"svømmer_farve4", "1F3CA-1F3FE"},
    {"svømmer_farve5", "1F3CA-1F3FF"},
    {"vægtløfter", "1F3CB"},
    {"vægtløfter_farve1", "1F3CB-1F3FB"},
    {"vægtløfter_farve2", "1F3CB-1F3FC"},
    {"vægtløfter_farve3", "1F3CB-1F3FD"},
    {"vægtløfter_farve4", "1F3CB-1F3FE"},
    {"vægtløfter_farve5", "1F3CB-1F3FF"},
    {"sneklædt_bjerg", "1F3D4"},
    {"faldefærdigt_hus", "1F3DA"},
    {"ørken", "1F3DC"},
    {"øde_ø", "1F3DD"},
    {"hæveautomat", "1F3E7"},
    {"kærlighedshotel", "1F3E9"},
    {"døgnkiosk", "1F3EA"},
    {"mærkat", "1F3F7"},
    {"medium_til_mørk_teint", "1F3FE"},
    {"mørk_teint", "1F3FF"},
    {"vandbøffel", "1F403"},
    {"vædder", "1F40F"},
    {"får", "1F411"},
    {"høne", "1F414"},
    {"blæksprutte", "1F419"},
    {"mariehøne", "1F41E"},
    {"nyudklækket_kylling", "1F423"},
    {"hval_med_vandsprøjt", "1F433"},
    {"frø", "1F438"},
    {"bjørnehoved", "1F43B"},
    {"øjne", "1F440"},
    {"øje", "1F441"},
    {"øre", "1F442"},
    {"øre_farve1", "1F442-1F3FB"},
    {"øre_farve2", "1F442-1F3FC"},
    {"øre_farve3", "1F442-1F3FD"},
    {"øre_farve4", "1F442-1F3FE"},
    {"øre_farve5", "1F442-1F3FF"},
    {"næse", "1F443"},
    {"næse_farve1", "1F443-1F3FB"},
    {"næse_farve2", "1F443-1F3FC"},
    {"næse_farve3", "1F443-1F3FD"},
    {"næse_farve4", "1F443-1F3FE"},
    {"næse_farve5", "1F443-1F3FF"},
    {"peger_mod_højre", "1F449"},
    {"peger_mod_højre_farve1", "1F449-1F3FB"},
    {"peger_mod_højre_farve2", "1F449-1F3FC"},
    {"peger_mod_højre_farve3", "1F449-1F3FD"},
    {"peger_mod_højre_farve4", "1F449-1F3FE"},
    {"peger_mod_højre_farve5", "1F449-1F3FF"},
    {"knytnæve", "1F44A"},
    {"knytnæve_farve1", "1F44A-1F3FB"},
    {"knytnæve_farve2", "1F44A-1F3FC"},
    {"knytnæve_farve3", "1F44A-1F3FD"},
    {"knytnæve_farve4", "1F44A-1F3FE"},
    {"knytnæve_farve5", "1F44A-1F3FF"},
    {"klappende_hænder", "1F44F"},
    {"klappende_hænder_farve1", "1F44F-1F3FB"},
    {"klappende_hænder_farve2", "1F44F-1F3FC"},
    {"klappende_hænder_farve3", "1F44F-1F3FD"},
    {"klappende_hænder_farve4", "1F44F-1F3FE"},
    {"klappende_hænder_farve5", "1F44F-1F3FF"},
    {"åbne_hænder", "1F450"},
    {"åbne_hænder_farve1", "1F450-1F3FB"},
    {"åbne_hænder_farve2", "1F450-1F3FC"},
    {"åbne_hænder_farve3", "1F450-1F3FD"},
    {"åbne_hænder_farve4", "1F450-1F3FE"},
    {"åbne_hænder_farve5", "1F450-1F3FF"},
    {"håndtaske", "1F45C"},
    {"løbesko", "1F45F"},
    {"høje_hæle", "1F460"},
    {"damestøvle", "1F462"},
    {"mand_og_kvinde_hånd_i_hånd", "1F46B"},
    {"mand_og_kvinde_hånd_i_hånd_farve1", "1F46B-1F3FB"},
    {"mand_og_kvinde_hånd_i_hånd_farve2", "1F46B-1F3FC"},
    {"mand_og_kvinde_hånd_i_hånd_farve3", "1F46B-1F3FD"},
    {"mand_og_kvinde_hånd_i_hånd_farve4", "1F46B-1F3FE"},
    {"mand_og_kvinde_hånd_i_hånd_farve5", "1F46B-1F3FF"},
    {"mand_og_kvinde_hånd_i_hånd_farve1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mand_og_kvinde_hånd_i_hånd_farve1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mand_og_kvinde_hånd_i_hånd_farve1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mand_og_kvinde_hånd_i_hånd_farve1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mand_og_kvinde_hånd_i_hånd_farve2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mand_og_kvinde_hånd_i_hånd_farve2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mand_og_kvinde_hånd_i_hånd_farve2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mand_og_kvinde_hånd_i_hånd_farve2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mand_og_kvinde_hånd_i_hånd_farve3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mand_og_kvinde_hånd_i_hånd_farve3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mand_og_kvinde_hånd_i_hånd_farve3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mand_og_kvinde_hånd_i_hånd_farve3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mand_og_kvinde_hånd_i_hånd_farve4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mand_og_kvinde_hånd_i_hånd_farve4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mand_og_kvinde_hånd_i_hånd_farve4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mand_og_kvinde_hånd_i_hånd_farve4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mand_og_kvinde_hånd_i_hånd_farve5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mand_og_kvinde_hånd_i_hånd_farve5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mand_og_kvinde_hånd_i_hånd_farve5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mand_og_kvinde_hånd_i_hånd_farve5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"to_mænd_hånd_i_hånd", "1F46C"},
    {"to_mænd_hånd_i_hånd_farve1", "1F46C-1F3FB"},
    {"to_mænd_hånd_i_hånd_farve2", "1F46C-1F3FC"},
    {"to_mænd_hånd_i_hånd_farve3", "1F46C-1F3FD"},
    {"to_mænd_hånd_i_hånd_farve4", "1F46C-1F3FE"},
    {"to_mænd_hånd_i_hånd_farve5", "1F46C-1F3FF"},
    {"to_mænd_hånd_i_hånd_farve1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"to_mænd_hånd_i_hånd_farve1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"to_mænd_hånd_i_hånd_farve1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"to_mænd_hånd_i_hånd_farve1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"to_mænd_hånd_i_hånd_farve2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"to_mænd_hånd_i_hånd_farve2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"to_mænd_hånd_i_hånd_farve2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"to_mænd_hånd_i_hånd_farve2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"to_mænd_hånd_i_hånd_farve3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"to_mænd_hånd_i_hånd_farve3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"to_mænd_hånd_i_hånd_farve3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"to_mænd_hånd_i_hånd_farve3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"to_mænd_hånd_i_hånd_farve4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"to_mænd_hånd_i_hånd_farve4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"to_mænd_hånd_i_hånd_farve4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"to_mænd_hånd_i_hånd_farve4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"to_mænd_hånd_i_hånd_farve5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"to_mænd_hånd_i_hånd_farve5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"to_mænd_hånd_i_hånd_farve5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"to_mænd_hånd_i_hånd_farve5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"to_kvinder_hånd_i_hånd", "1F46D"},
    {"to_kvinder_hånd_i_hånd_farve1", "1F46D-1F3FB"},
    {"to_kvinder_hånd_i_hånd_farve2", "1F46D-1F3FC"},
    {"to_kvinder_hånd_i_hånd_farve3", "1F46D-1F3FD"},
    {"to_kvinder_hånd_i_hånd_farve4", "1F46D-1F3FE"},
    {"to_kvinder_hånd_i_hånd_farve5", "1F46D-1F3FF"},
    {"to_kvinder_hånd_i_hånd_farve1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"to_kvinder_hånd_i_hånd_farve1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"to_kvinder_hånd_i_hånd_farve1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"to_kvinder_hånd_i_hånd_farve1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"to_kvinder_hånd_i_hånd_farve2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"to_kvinder_hånd_i_hånd_farve2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"to_kvinder_hånd_i_hånd_farve2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"to_kvinder_hånd_i_hånd_farve2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"to_kvinder_hånd_i_hånd_farve3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"to_kvinder_hånd_i_hånd_farve3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"to_kvinder_hånd_i_hånd_farve3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"to_kvinder_hånd_i_hånd_farve3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"to_kvinder_hånd_i_hånd_farve4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"to_kvinder_hånd_i_hånd_farve4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"to_kvinder_hånd_i_hånd_farve4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"to_kvinder_hånd_i_hånd_farve4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"to_kvinder_hånd_i_hånd_farve5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"to_kvinder_hånd_i_hånd_farve5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"to_kvinder_hånd_i_hånd_farve5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"to_kvinder_hånd_i_hånd_farve5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"personer_med_kaninører", "1F46F"},
    {"personer_med_kaninører_farve1", "1F46F-1F3FB"},
    {"personer_med_kaninører_farve2", "1F46F-1F3FC"},
    {"personer_med_kaninører_farve3", "1F46F-1F3FD"},
    {"personer_med_kaninører_farve4", "1F46F-1F3FE"},
    {"personer_med_kaninører_farve5", "1F46F-1F3FF"},
    {"personer_med_kaninører_farve1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninører_farve1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninører_farve1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninører_farve1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninører_farve2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninører_farve2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninører_farve2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninører_farve2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninører_farve3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninører_farve3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninører_farve3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninører_farve3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninører_farve4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninører_farve4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninører_farve4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninører_farve4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninører_farve5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninører_farve5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninører_farve5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninører_farve5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"person_med_brudeslør", "1F470"},
    {"person_med_brudeslør_farve1", "1F470-1F3FB"},
    {"person_med_brudeslør_farve2", "1F470-1F3FC"},
    {"person_med_brudeslør_farve3", "1F470-1F3FD"},
    {"person_med_brudeslør_farve4", "1F470-1F3FE"},
    {"person_med_brudeslør_farve5", "1F470-1F3FF"},
    {"ånd", "1F47A"},
    {"spøgelse", "1F47B"},
    {"rumvæsen", "1F47D"},
    {"sur_lilla_djævel", "1F47F"},
    {"person_vender_håndfladen_op", "1F481"},
    {"person_vender_håndfladen_op_farve1", "1F481-1F3FB"},
    {"person_vender_håndfladen_op_farve2", "1F481-1F3FC"},
    {"person_vender_håndfladen_op_farve3", "1F481-1F3FD"},
    {"person_vender_håndfladen_op_farve4", "1F481-1F3FE"},
    {"person_vender_håndfladen_op_farve5", "1F481-1F3FF"},
    {"læbestift", "1F484"},
    {"sprøjte", "1F489"},
    {"læbeaftryk", "1F48B"},
    {"kærlighedsbrev", "1F48C"},
    {"ædelsten", "1F48E"},
    {"blåt_hjerte", "1F499"},
    {"grønt_hjerte", "1F49A"},
    {"hjerte_med_bånd", "1F49D"},
    {"elpære", "1F4A1"},
    {"søvnig", "1F4A4"},
    {"sveddråber", "1F4A6"},
    {"dråbe", "1F4A7"},
    {"bæ", "1F4A9"},
    {"sæde", "1F4BA"},
    {"bærbar_computer", "1F4BB"},
    {"attachémappe", "1F4BC"},
    {"åben_mappe", "1F4C2"},
    {"opadgående_graf", "1F4C8"},
    {"nedadgående_graf", "1F4C9"},
    {"søjlediagram", "1F4CA"},
    {"åben_bog", "1F4D6"},
    {"grøn_bog", "1F4D7"},
    {"blå_bog", "1F4D8"},
    {"bøger", "1F4DA"},
    {"telefonrør", "1F4DE"},
    {"personsøger", "1F4DF"},
    {"indgående_post", "1F4E8"},
    {"udgående_post", "1F4E9"},
    {"lukket_postkasse_med_sænket_flag", "1F4EA"},
    {"lukket_postkasse_med_hævet_flag", "1F4EB"},
    {"åben_postkasse_med_hævet_flag", "1F4EC"},
    {"åben_postkasse_med_sænket_flag", "1F4ED"},
    {"mobil_slået_fra", "1F4F4"},
    {"videobånd", "1F4FC"},
    {"knap_til_én_gentagelse", "1F502"},
    {"lodrette_cirkulære_pile_med_uret", "1F503"},
    {"vandrette_cirkulære_pile_mod_uret", "1F504"},
    {"høj_lysstyrke", "1F506"},
    {"højttaler_fra", "1F507"},
    {"højttaler_lav_lydstyrke", "1F508"},
    {"højttaler_mellem_lydstyrke", "1F509"},
    {"højttaler_høj_lydstyrke", "1F50A"},
    {"forstørrelsesglas_til_venstre", "1F50D"},
    {"forstørrelsesglas_til_højre", "1F50E"},
    {"lukket_hængelås_med_pen", "1F50F"},
    {"lukket_hængelås_med_nøgle", "1F510"},
    {"nøgle", "1F511"},
    {"lukket_hængelås", "1F512"},
    {"åben_hængelås", "1F513"},
    {"bogmærke", "1F516"},
    {"små_bogstaver", "1F521"},
    {"skruenøgle", "1F527"},
    {"møtrik_og_bolt", "1F529"},
    {"køkkenkniv", "1F52A"},
    {"rød_cirkel", "1F534"},
    {"blå_cirkel", "1F535"},
    {"stor_blå_ruder", "1F537"},
    {"lille_blå_ruder", "1F539"},
    {"moské", "1F54C"},
    {"svævende_forretningsmand", "1F574"},
    {"svævende_forretningsmand_farve1", "1F574-1F3FB"},
    {"svævende_forretningsmand_farve2", "1F574-1F3FC"},
    {"svævende_forretningsmand_farve3", "1F574-1F3FD"},
    {"svævende_forretningsmand_farve4", "1F574-1F3FE"},
    {"svævende_forretningsmand_farve5", "1F574-1F3FF"},
    {"hånd_med_spredte_fingre", "1F590"},
    {"hånd_med_spredte_fingre_farve1", "1F590-1F3FB"},
    {"hånd_med_spredte_fingre_farve2", "1F590-1F3FC"},
    {"hånd_med_spredte_fingre_farve3", "1F590-1F3FD"},
    {"hånd_med_spredte_fingre_farve4", "1F590-1F3FE"},
    {"hånd_med_spredte_fingre_farve5", "1F590-1F3FF"},
    {"stationær_computer", "1F5A5"},
    {"gammel_nøgle", "1F5DD"},
    {"vred_taleboble_til_højre", "1F5EF"},
    {"strålende_ansigt_med_smilende_øjne", "1F601"},
    {"ansigt_med_glædestårer", "1F602"},
    {"grinende_ansigt_med_store_øjne", "1F603"},
    {"grinende_ansigt_med_smilende_øjne", "1F604"},
    {"grinende_ansigt_med_sveddråbe", "1F605"},
    {"grinende_ansigt_med_lukkede_øjne", "1F606"},
    {"glad_lilla_djævel", "1F608"},
    {"smilende_ansigt_og_øjne", "1F60A"},
    {"smilende_ansigt_med_hjerteøjne", "1F60D"},
    {"ansigt_med_smørret_grin", "1F60F"},
    {"udtryksløst_ansigt", "1F611"},
    {"eftertænksomt_ansigt", "1F614"},
    {"kysseansigt_med_smilende_øjne", "1F619"},
    {"kysseansigt_med_lukkede_øjne", "1F61A"},
    {"ansigt_med_tungen_ude_og_blinkende_øje", "1F61C"},
    {"ansigt_med_tungen_ude_og_sammenknebne_øjne", "1F61D"},
    {"grædende_ansigt", "1F622"},
    {"stædigt_ansigt", "1F623"},
    {"måbende_ansigt_med_åben_mund", "1F626"},
    {"søvnigt_ansigt", "1F62A"},
    {"træt_ansigt", "1F62B"},
    {"ansigt_med_åben_mund", "1F62E"},
    {"ansigt_med_åben_mund_og_koldsved", "1F630"},
    {"rødmende_ansigt", "1F633"},
    {"kat_med_glædestårer", "1F639"},
    {"glad_kat_med_hjerteøjne", "1F63B"},
    {"grædende_kat", "1F63F"},
    {"ansigt_med_rullende_øjne", "1F644"},
    {"ikke_høre", "1F649"},
    {"person_med_løftet_hånd", "1F64B"},
    {"person_med_løftet_hånd_farve1", "1F64B-1F3FB"},
    {"person_med_løftet_hånd_farve2", "1F64B-1F3FC"},
    {"person_med_løftet_hånd_farve3", "1F64B-1F3FD"},
    {"person_med_løftet_hånd_farve4", "1F64B-1F3FE"},
    {"person_med_løftet_hånd_farve5", "1F64B-1F3FF"},
    {"løftede_hænder", "1F64C"},
    {"løftede_hænder_farve1", "1F64C-1F3FB"},
    {"løftede_hænder_farve2", "1F64C-1F3FC"},
    {"løftede_hænder_farve3", "1F64C-1F3FD"},
    {"løftede_hænder_farve4", "1F64C-1F3FE"},
    {"løftede_hænder_farve5", "1F64C-1F3FF"},
    {"foldede_hænder", "1F64F"},
    {"foldede_hænder_farve1", "1F64F-1F3FB"},
    {"foldede_hænder_farve2", "1F64F-1F3FC"},
    {"foldede_hænder_farve3", "1F64F-1F3FD"},
    {"foldede_hænder_farve4", "1F64F-1F3FE"},
    {"foldede_hænder_farve5", "1F64F-1F3FF"},
    {"højhastighedstog", "1F685"},
    {"firehjulstrækker", "1F699"},
    {"svævebane", "1F6A1"},
    {"speedbåd", "1F6A4"},
    {"dør", "1F6AA"},
    {"fodgænger", "1F6B6"},
    {"fodgænger_farve1", "1F6B6-1F3FB"},
    {"fodgænger_farve2", "1F6B6-1F3FC"},
    {"fodgænger_farve3", "1F6B6-1F3FD"},
    {"fodgænger_farve4", "1F6B6-1F3FE"},
    {"fodgænger_farve5", "1F6B6-1F3FF"},
    {"ingen_fodgængere", "1F6B7"},
    {"børn_krydser_vej", "1F6B8"},
    {"indkøbsposer", "1F6CD"},
    {"indkøbsvogn", "1F6D2"},
    {"trådløs", "1F6DC"},
    {"hammer_og_skruenøgle", "1F6E0"},
    {"olietønde", "1F6E2"},
    {"motorbåd", "1F6E5"},
    {"løbehjul", "1F6F4"},
    {"kælk", "1F6F7"},
    {"rulleskøjte", "1F6FC"},
    {"grøn_cirkel", "1F7E2"},
    {"rødt_kvadrat", "1F7E5"},
    {"blåt_kvadrat", "1F7E6"},
    {"grønt_kvadrat", "1F7E9"},
    {"ansigt_med_lynlåsmund", "1F910"},
    {"nørdansigt", "1F913"},
    {"tænkende_ansigt", "1F914"},
    {"djævletegn", "1F918"},
    {"djævletegn_farve1", "1F918-1F3FB"},
    {"djævletegn_farve2", "1F918-1F3FC"},
    {"djævletegn_farve3", "1F918-1F3FD"},
    {"djævletegn_farve4", "1F918-1F3FE"},
    {"djævletegn_farve5", "1F918-1F3FF"},
    {"ring_til_mig_hånd", "1F919"},
    {"ring_til_mig_hånd_farve1", "1F919-1F3FB"},
    {"ring_til_mig_hånd_farve2", "1F919-1F3FC"},
    {"ring_til_mig_hånd_farve3", "1F919-1F3FD"},
    {"ring_til_mig_hånd_farve4", "1F919-1F3FE"},
    {"ring_til_mig_hånd_farve5", "1F919-1F3FF"},
    {"løftet_håndryg", "1F91A"},
    {"løftet_håndryg_farve1", "1F91A-1F3FB"},
    {"løftet_håndryg_farve2", "1F91A-1F3FC"},
    {"løftet_håndryg_farve3", "1F91A-1F3FD"},
    {"løftet_håndryg_farve4", "1F91A-1F3FE"},
    {"løftet_håndryg_farve5", "1F91A-1F3FF"},
    {"venstrevendt_knytnæve", "1F91B"},
    {"venstrevendt_knytnæve_farve1", "1F91B-1F3FB"},
    {"venstrevendt_knytnæve_farve2", "1F91B-1F3FC"},
    {"venstrevendt_knytnæve_farve3", "1F91B-1F3FD"},
    {"venstrevendt_knytnæve_farve4", "1F91B-1F3FE"},
    {"venstrevendt_knytnæve_farve5", "1F91B-1F3FF"},
    {"højrevendt_knytnæve", "1F91C"},
    {"højrevendt_knytnæve_farve1", "1F91C-1F3FB"},
    {"højrevendt_knytnæve_farve2", "1F91C-1F3FC"},
    {"højrevendt_knytnæve_farve3", "1F91C-1F3FD"},
    {"højrevendt_knytnæve_farve4", "1F91C-1F3FE"},
    {"højrevendt_knytnæve_farve5", "1F91C-1F3FF"},
    {"håndtryk", "1F91D"},
    {"håndtryk_farve1", "1F91D-1F3FB"},
    {"håndtryk_farve2", "1F91D-1F3FC"},
    {"håndtryk_farve3", "1F91D-1F3FD"},
    {"håndtryk_farve4", "1F91D-1F3FE"},
    {"håndtryk_farve5", "1F91D-1F3FF"},
    {"håndtryk_farve1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"håndtryk_farve1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"håndtryk_farve1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"håndtryk_farve1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"håndtryk_farve2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"håndtryk_farve2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"håndtryk_farve2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"håndtryk_farve2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"håndtryk_farve3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"håndtryk_farve3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"håndtryk_farve3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"håndtryk_farve3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"håndtryk_farve4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"håndtryk_farve4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"håndtryk_farve4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"håndtryk_farve4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"håndtryk_farve5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"håndtryk_farve5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"håndtryk_farve5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"håndtryk_farve5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"ruller_rundt_på_gulvet_af_grin", "1F923"},
    {"løgneransigt", "1F925"},
    {"slå_sig_for_panden", "1F926"},
    {"slå_sig_for_panden_farve1", "1F926-1F3FB"},
    {"slå_sig_for_panden_farve2", "1F926-1F3FC"},
    {"slå_sig_for_panden_farve3", "1F926-1F3FD"},
    {"slå_sig_for_panden_farve4", "1F926-1F3FE"},
    {"slå_sig_for_panden_farve5", "1F926-1F3FF"},
    {"ansigt_med_løftet_øjenbryn", "1F928"},
    {"skørt_ansigt", "1F92A"},
    {"ansigt_med_symboler_på_munden", "1F92C"},
    {"ansigt_med_hånd_over_munden", "1F92D"},
    {"håndflader_samlet_og_løftet", "1F932"},
    {"håndflader_samlet_og_løftet_farve1", "1F932-1F3FB"},
    {"håndflader_samlet_og_løftet_farve2", "1F932-1F3FC"},
    {"håndflader_samlet_og_løftet_farve3", "1F932-1F3FD"},
    {"håndflader_samlet_og_løftet_farve4", "1F932-1F3FE"},
    {"håndflader_samlet_og_løftet_farve5", "1F932-1F3FF"},
    {"trækker_på_skuldrene", "1F937"},
    {"trækker_på_skuldrene_farve1", "1F937-1F3FB"},
    {"trækker_på_skuldrene_farve2", "1F937-1F3FC"},
    {"trækker_på_skuldrene_farve3", "1F937-1F3FD"},
    {"trækker_på_skuldrene_farve4", "1F937-1F3FE"},
    {"trækker_på_skuldrene_farve5", "1F937-1F3FF"},
    {"person_slår_vejrmølle", "1F938"},
    {"person_slår_vejrmølle_farve1", "1F938-1F3FB"},
    {"person_slår_vejrmølle_farve2", "1F938-1F3FC"},
    {"person_slår_vejrmølle_farve3", "1F938-1F3FD"},
    {"person_slår_vejrmølle_farve4", "1F938-1F3FE"},
    {"person_slår_vejrmølle_farve5", "1F938-1F3FF"},
    {"jonglør", "1F939"},
    {"jonglør_farve1", "1F939-1F3FB"},
    {"jonglør_farve2", "1F939-1F3FC"},
    {"jonglør_farve3", "1F939-1F3FD"},
    {"jonglør_farve4", "1F939-1F3FE"},
    {"jonglør_farve5", "1F939-1F3FF"},
    {"fægter", "1F93A"},
    {"håndboldspiller", "1F93E"},
    {"håndboldspiller_farve1", "1F93E-1F3FB"},
    {"håndboldspiller_farve2", "1F93E-1F3FC"},
    {"håndboldspiller_farve3", "1F93E-1F3FD"},
    {"håndboldspiller_farve4", "1F93E-1F3FE"},
    {"håndboldspiller_farve5", "1F93E-1F3FF"},
    {"skål", "1F942"},
    {"målnet", "1F945"},
    {"sølvmedalje", "1F948"},
    {"grøn_salat", "1F957"},
    {"fyldt_pitabrød", "1F959"},
    {"æg", "1F95A"},
    {"glas_mælk", "1F95B"},
    {"jordnødder", "1F95C"},
    {"ske_i_skål", "1F963"},
    {"bæger_med_sugerør", "1F964"},
    {"kokosnød", "1F965"},
    {"tærte", "1F967"},
    {"kødudskæring", "1F969"},
    {"bladgrønt", "1F96C"},
    {"månekage", "1F96E"},
    {"smilende_ansigt_med_tåre", "1F972"},
    {"ørt_ansigt", "1F974"},
    {"ansigt_der_holder_tårer_tilbage", "1F979"},
    {"bønfaldende_ansigt", "1F97A"},
    {"vandrestøvle", "1F97E"},
    {"løvehoved", "1F981"},
    {"enhjørningehoved", "1F984"},
    {"ørn", "1F985"},
    {"rævehoved", "1F98A"},
    {"næsehorn", "1F98F"},
    {"fårekylling", "1F997"},
    {"kænguru", "1F998"},
    {"påfugl", "1F99A"},
    {"papegøje", "1F99C"},
    {"vaskebjørn", "1F99D"},
    {"grævling", "1F9A1"},
    {"østers", "1F9AA"},
    {"bæver", "1F9AB"},
    {"sæl", "1F9AD"},
    {"førerhund", "1F9AE"},
    {"rødt_hår", "1F9B0"},
    {"krøllet_hår", "1F9B1"},
    {"hvidt_hår", "1F9B3"},
    {"øre_med_høreapparat", "1F9BB"},
    {"øre_med_høreapparat_farve1", "1F9BB-1F3FB"},
    {"øre_med_høreapparat_farve2", "1F9BB-1F3FC"},
    {"øre_med_høreapparat_farve3", "1F9BB-1F3FD"},
    {"øre_med_høreapparat_farve4", "1F9BB-1F3FE"},
    {"øre_med_høreapparat_farve5", "1F9BB-1F3FF"},
    {"elektrisk_kørestol", "1F9BC"},
    {"manuel_kørestol", "1F9BD"},
    {"hvidløg", "1F9C4"},
    {"løg", "1F9C5"},
    {"smør", "1F9C8"},
    {"stående_person", "1F9CD"},
    {"stående_person_farve1", "1F9CD-1F3FB"},
    {"stående_person_farve2", "1F9CD-1F3FC"},
    {"stående_person_farve3", "1F9CD-1F3FD"},
    {"stående_person_farve4", "1F9CD-1F3FE"},
    {"stående_person_farve5", "1F9CD-1F3FF"},
    {"knælende_person", "1F9CE"},
    {"knælende_person_farve1", "1F9CE-1F3FB"},
    {"knælende_person_farve2", "1F9CE-1F3FC"},
    {"knælende_person_farve3", "1F9CE-1F3FD"},
    {"knælende_person_farve4", "1F9CE-1F3FE"},
    {"knælende_person_farve5", "1F9CE-1F3FF"},
    {"døv_person", "1F9CF"},
    {"døv_person_farve1", "1F9CF-1F3FB"},
    {"døv_person_farve2", "1F9CF-1F3FC"},
    {"døv_person_farve3", "1F9CF-1F3FD"},
    {"døv_person_farve4", "1F9CF-1F3FE"},
    {"døv_person_farve5", "1F9CF-1F3FF"},
    {"ældre_menneske", "1F9D3"},
    {"ældre_menneske_farve1", "1F9D3-1F3FB"},
    {"ældre_menneske_farve2", "1F9D3-1F3FC"},
    {"ældre_menneske_farve3", "1F9D3-1F3FD"},
    {"ældre_menneske_farve4", "1F9D3-1F3FE"},
    {"ældre_menneske_farve5", "1F9D3-1F3FF"},
    {"person_med_skæg", "1F9D4"},
    {"person_med_skæg_farve1", "1F9D4-1F3FB"},
    {"person_med_skæg_farve2", "1F9D4-1F3FC"},
    {"person_med_skæg_farve3", "1F9D4-1F3FD"},
    {"person_med_skæg_farve4", "1F9D4-1F3FE"},
    {"person_med_skæg_farve5", "1F9D4-1F3FF"},
    {"kvinde_med_tørklæde", "1F9D5"},
    {"kvinde_med_tørklæde_farve1", "1F9D5-1F3FB"},
    {"kvinde_med_tørklæde_farve2", "1F9D5-1F3FC"},
    {"kvinde_med_tørklæde_farve3", "1F9D5-1F3FD"},
    {"kvinde_med_tørklæde_farve4", "1F9D5-1F3FE"},
    {"kvinde_med_tørklæde_farve5", "1F9D5-1F3FF"},
    {"havvæsen", "1F9DC"},
    {"havvæsen_farve1", "1F9DC-1F3FB"},
    {"havvæsen_farve2", "1F9DC-1F3FC"},
    {"havvæsen_farve3", "1F9DC-1F3FD"},
    {"havvæsen_farve4", "1F9DC-1F3FE"},
    {"havvæsen_farve5", "1F9DC-1F3FF"},
    {"magisk_ånd", "1F9DE"},
    {"halstørklæde", "1F9E3"},
    {"petriskål", "1F9EB"},
    {"værktøjskasse", "1F9F0"},
    {"tråd", "1F9F5"},
    {"sikkerhedsnål", "1F9F7"},
    {"sæbe", "1F9FC"},
    {"lyseblåt_hjerte", "1FA75"},
    {"gråt_hjerte", "1FA76"},
    {"lyserødt_hjerte", "1FA77"},
    {"bloddråbe", "1FA78"},
    {"røntgen", "1FA7B"},
    {"legetøjsdrage", "1FA81"},
    {"faldskærm", "1FA82"},
    {"piñata", "1FA85"},
    {"fløjte", "1FA88"},
    {"økse", "1FA93"},
    {"militærhjelm", "1FA96"},
    {"høj_tromme", "1FA98"},
    {"mønt", "1FA99"},
    {"skruetrækker", "1FA9B"},
    {"synål", "1FAA1"},
    {"musefælde", "1FAA4"},
    {"tandbørste", "1FAA5"},
    {"sammenklappelig_håndvifte", "1FAAD"},
    {"træ", "1FAB5"},
    {"rede_med_æg", "1FABA"},
    {"træ_uden_blade", "1FABE"},
    {"gås", "1FABF"},
    {"behåret_væsen", "1FAC8"},
    {"spækhugger", "1FACD"},
    {"æsel", "1FACF"},
    {"blåbær", "1FAD0"},
    {"fladbrød", "1FAD3"},
    {"hælde_vand", "1FAD7"},
    {"bønner", "1FAD8"},
    {"ingefærrod", "1FADA"},
    {"ærtebælg", "1FADB"},
    {"ansigt_der_gør_honnør", "1FAE1"},
    {"ansigt_med_åbne_øjne_og_hånden_over_munden", "1FAE2"},
    {"ansigt_med_skrå_mund", "1FAE4"},
    {"bider_i_læben", "1FAE6"},
    {"ansigt_med_poser_under_øjnene", "1FAE9"},
    {"forvrænget_ansigt", "1FAEA"},
    {"slagsmålssky", "1FAEF"},
    {"hånd_til_højre", "1FAF1"},
    {"hånd_til_højre_farve1", "1FAF1-1F3FB"},
    {"hånd_til_højre_farve2", "1FAF1-1F3FC"},
    {"hånd_til_højre_farve3", "1FAF1-1F3FD"},
    {"hånd_til_højre_farve4", "1FAF1-1F3FE"},
    {"hånd_til_højre_farve5", "1FAF1-1F3FF"},
    {"hånd_til_venstre", "1FAF2"},
    {"hånd_til_venstre_farve1", "1FAF2-1F3FB"},
    {"hånd_til_venstre_farve2", "1FAF2-1F3FC"},
    {"hånd_til_venstre_farve3", "1FAF2-1F3FD"},
    {"hånd_til_venstre_farve4", "1FAF2-1F3FE"},
    {"hånd_til_venstre_farve5", "1FAF2-1F3FF"},
    {"håndflade_nedad", "1FAF3"},
    {"håndflade_nedad_farve1", "1FAF3-1F3FB"},
    {"håndflade_nedad_farve2", "1FAF3-1F3FC"},
    {"håndflade_nedad_farve3", "1FAF3-1F3FD"},
    {"håndflade_nedad_farve4", "1FAF3-1F3FE"},
    {"håndflade_nedad_farve5", "1FAF3-1F3FF"},
    {"håndflade_opad", "1FAF4"},
    {"håndflade_opad_farve1", "1FAF4-1F3FB"},
    {"håndflade_opad_farve2", "1FAF4-1F3FC"},
    {"håndflade_opad_farve3", "1FAF4-1F3FD"},
    {"håndflade_opad_farve4", "1FAF4-1F3FE"},
    {"håndflade_opad_farve5", "1FAF4-1F3FF"},
    {"peger_på_seeren", "1FAF5"},
    {"peger_på_seeren_farve1", "1FAF5-1F3FB"},
    {"peger_på_seeren_farve2", "1FAF5-1F3FC"},
    {"peger_på_seeren_farve3", "1FAF5-1F3FD"},
    {"peger_på_seeren_farve4", "1FAF5-1F3FE"},
    {"peger_på_seeren_farve5", "1FAF5-1F3FF"},
    {"hjertehænder", "1FAF6"},
    {"hjertehænder_farve1", "1FAF6-1F3FB"},
    {"hjertehænder_farve2", "1FAF6-1F3FC"},
    {"hjertehænder_farve3", "1FAF6-1F3FD"},
    {"hjertehænder_farve4", "1FAF6-1F3FE"},
    {"hjertehænder_farve5", "1FAF6-1F3FF"},
    {"hånd_skubber_mod_venstre", "1FAF7"},
    {"hånd_skubber_mod_venstre_farve1", "1FAF7-1F3FB"},
    {"hånd_skubber_mod_venstre_farve2", "1FAF7-1F3FC"},
    {"hånd_skubber_mod_venstre_farve3", "1FAF7-1F3FD"},
    {"hånd_skubber_mod_venstre_farve4", "1FAF7-1F3FE"},
    {"hånd_skubber_mod_venstre_farve5", "1FAF7-1F3FF"},
    {"hånd_skubber_mod_højre", "1FAF8"},
    {"hånd_skubber_mod_højre_farve1", "1FAF8-1F3FB"},
    {"hånd_skubber_mod_højre_farve2", "1FAF8-1F3FC"},
    {"hånd_skubber_mod_højre_farve3", "1FAF8-1F3FD"},
    {"hånd_skubber_mod_højre_farve4", "1FAF8-1F3FE"},
    {"hånd_skubber_mod_højre_farve5", "1FAF8-1F3FF"},
    {"ascensionøen", "1F1E6-1F1E8"},
    {"østrig", "1F1E6-1F1F9"},
    {"åland", "1F1E6-1F1FD"},
    {"saint_barthélemy", "1F1E7-1F1F1"},
    {"bouvetøen", "1F1E7-1F1FB"},
    {"cocosøerne", "1F1E8-1F1E8"},
    {"cookøerne", "1F1E8-1F1F0"},
    {"clippertonøen", "1F1E8-1F1F5"},
    {"curaçao", "1F1E8-1F1FC"},
    {"juleøen", "1F1E8-1F1FD"},
    {"den_europæiske_union", "1F1EA-1F1FA"},
    {"falklandsøerne", "1F1EB-1F1F0"},
    {"færøerne", "1F1EB-1F1F4"},
    {"grønland", "1F1EC-1F1F1"},
    {"ækvatorialguinea", "1F1EC-1F1F6"},
    {"grækenland", "1F1EC-1F1F7"},
    {"south_georgia_og_de_sydlige_sandwichøer", "1F1EC-1F1F8"},
    {"kanariske_øer", "1F1EE-1F1E8"},
    {"caymanøerne", "1F1F0-1F1FE"},
    {"marshalløerne", "1F1F2-1F1ED"},
    {"de_palæstinensiske_områder", "1F1F5-1F1F8"},
    {"réunion", "1F1F7-1F1EA"},
    {"rumænien", "1F1F7-1F1F4"},
    {"salomonøerne", "1F1F8-1F1E7"},
    {"são_tomé_og_príncipe", "1F1F8-1F1F9"},
    {"turks_og_caicosøerne", "1F1F9-1F1E8"},
    {"amerikanske_oversøiske_øer", "1F1FA-1F1F2"},
    {"de_britiske_jomfruøer", "1F1FB-1F1EC"},
    {"de_amerikanske_jomfruøer", "1F1FB-1F1EE"},
    {"personer_hånd_i_hånd", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"personer_hånd_i_hånd_farve1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_hånd_i_hånd_farve1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_hånd_i_hånd_farve1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_hånd_i_hånd_farve1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_hånd_i_hånd_farve1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_hånd_i_hånd_farve2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_hånd_i_hånd_farve2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_hånd_i_hånd_farve2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_hånd_i_hånd_farve2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_hånd_i_hånd_farve2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_hånd_i_hånd_farve3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_hånd_i_hånd_farve3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_hånd_i_hånd_farve3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_hånd_i_hånd_farve3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_hånd_i_hånd_farve3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_hånd_i_hånd_farve4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_hånd_i_hånd_farve4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_hånd_i_hånd_farve4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_hånd_i_hånd_farve4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_hånd_i_hånd_farve4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_hånd_i_hånd_farve5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_hånd_i_hånd_farve5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_hånd_i_hånd_farve5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_hånd_i_hånd_farve5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_hånd_i_hånd_farve5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"løber_mod_højre", "1F3C3-200D-27A1-FE0F"},
    {"løber_mod_højre_farve1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"løber_mod_højre_farve2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"løber_mod_højre_farve3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"løber_mod_højre_farve4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"løber_mod_højre_farve5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"mandlig_lærer", "1F468-200D-1F3EB"},
    {"mandlig_lærer_farve1", "1F468-1F3FB-200D-1F3EB"},
    {"mandlig_lærer_farve2", "1F468-1F3FC-200D-1F3EB"},
    {"mandlig_lærer_farve3", "1F468-1F3FD-200D-1F3EB"},
    {"mandlig_lærer_farve4", "1F468-1F3FE-200D-1F3EB"},
    {"mandlig_lærer_farve5", "1F468-1F3FF-200D-1F3EB"},
    {"mand_med_blindestok_mod_højre", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"mand_med_blindestok_mod_højre_farve1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"mand_med_blindestok_mod_højre_farve2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"mand_med_blindestok_mod_højre_farve3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"mand_med_blindestok_mod_højre_farve4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"mand_med_blindestok_mod_højre_farve5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"mand_i_elektrisk_kørestol", "1F468-200D-1F9BC"},
    {"mand_i_elektrisk_kørestol_farve1", "1F468-1F3FB-200D-1F9BC"},
    {"mand_i_elektrisk_kørestol_farve2", "1F468-1F3FC-200D-1F9BC"},
    {"mand_i_elektrisk_kørestol_farve3", "1F468-1F3FD-200D-1F9BC"},
    {"mand_i_elektrisk_kørestol_farve4", "1F468-1F3FE-200D-1F9BC"},
    {"mand_i_elektrisk_kørestol_farve5", "1F468-1F3FF-200D-1F9BC"},
    {"mand_i_elektrisk_kørestol_mod_højre", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"mand_i_elektrisk_kørestol_mod_højre_farve1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"mand_i_elektrisk_kørestol_mod_højre_farve2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"mand_i_elektrisk_kørestol_mod_højre_farve3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"mand_i_elektrisk_kørestol_mod_højre_farve4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"mand_i_elektrisk_kørestol_mod_højre_farve5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mand_i_manuel_kørestol", "1F468-200D-1F9BD"},
    {"mand_i_manuel_kørestol_farve1", "1F468-1F3FB-200D-1F9BD"},
    {"mand_i_manuel_kørestol_farve2", "1F468-1F3FC-200D-1F9BD"},
    {"mand_i_manuel_kørestol_farve3", "1F468-1F3FD-200D-1F9BD"},
    {"mand_i_manuel_kørestol_farve4", "1F468-1F3FE-200D-1F9BD"},
    {"mand_i_manuel_kørestol_farve5", "1F468-1F3FF-200D-1F9BD"},
    {"mand_i_manuel_kørestol_mod_højre", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"mand_i_manuel_kørestol_mod_højre_farve1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"mand_i_manuel_kørestol_mod_højre_farve2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"mand_i_manuel_kørestol_mod_højre_farve3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"mand_i_manuel_kørestol_mod_højre_farve4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"mand_i_manuel_kørestol_mod_højre_farve5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kvindelig_lærer", "1F469-200D-1F3EB"},
    {"kvindelig_lærer_farve1", "1F469-1F3FB-200D-1F3EB"},
    {"kvindelig_lærer_farve2", "1F469-1F3FC-200D-1F3EB"},
    {"kvindelig_lærer_farve3", "1F469-1F3FD-200D-1F3EB"},
    {"kvindelig_lærer_farve4", "1F469-1F3FE-200D-1F3EB"},
    {"kvindelig_lærer_farve5", "1F469-1F3FF-200D-1F3EB"},
    {"kvinde_med_blindestok_mod_højre", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinde_med_blindestok_mod_højre_farve1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinde_med_blindestok_mod_højre_farve2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinde_med_blindestok_mod_højre_farve3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinde_med_blindestok_mod_højre_farve4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinde_med_blindestok_mod_højre_farve5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinde_i_elektrisk_kørestol", "1F469-200D-1F9BC"},
    {"kvinde_i_elektrisk_kørestol_farve1", "1F469-1F3FB-200D-1F9BC"},
    {"kvinde_i_elektrisk_kørestol_farve2", "1F469-1F3FC-200D-1F9BC"},
    {"kvinde_i_elektrisk_kørestol_farve3", "1F469-1F3FD-200D-1F9BC"},
    {"kvinde_i_elektrisk_kørestol_farve4", "1F469-1F3FE-200D-1F9BC"},
    {"kvinde_i_elektrisk_kørestol_farve5", "1F469-1F3FF-200D-1F9BC"},
    {"kvinde_i_elektrisk_kørestol_mod_højre", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinde_i_elektrisk_kørestol_mod_højre_farve1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinde_i_elektrisk_kørestol_mod_højre_farve2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinde_i_elektrisk_kørestol_mod_højre_farve3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinde_i_elektrisk_kørestol_mod_højre_farve4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinde_i_elektrisk_kørestol_mod_højre_farve5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinde_i_manuel_kørestol", "1F469-200D-1F9BD"},
    {"kvinde_i_manuel_kørestol_farve1", "1F469-1F3FB-200D-1F9BD"},
    {"kvinde_i_manuel_kørestol_farve2", "1F469-1F3FC-200D-1F9BD"},
    {"kvinde_i_manuel_kørestol_farve3", "1F469-1F3FD-200D-1F9BD"},
    {"kvinde_i_manuel_kørestol_farve4", "1F469-1F3FE-200D-1F9BD"},
    {"kvinde_i_manuel_kørestol_farve5", "1F469-1F3FF-200D-1F9BD"},
    {"kvinde_i_manuel_kørestol_mod_højre", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinde_i_manuel_kørestol_mod_højre_farve1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinde_i_manuel_kørestol_mod_højre_farve2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinde_i_manuel_kørestol_mod_højre_farve3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinde_i_manuel_kørestol_mod_højre_farve4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinde_i_manuel_kørestol_mod_højre_farve5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"fodgænger_mod_højre", "1F6B6-200D-27A1-FE0F"},
    {"fodgænger_mod_højre_farve1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"fodgænger_mod_højre_farve2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"fodgænger_mod_højre_farve3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"fodgænger_mod_højre_farve4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"fodgænger_mod_højre_farve5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"knælende_person_mod_højre", "1F9CE-200D-27A1-FE0F"},
    {"knælende_person_mod_højre_farve1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"knælende_person_mod_højre_farve2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"knælende_person_mod_højre_farve3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"knælende_person_mod_højre_farve4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"knælende_person_mod_højre_farve5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"lærer", "1F9D1-200D-1F3EB"},
    {"lærer_farve1", "1F9D1-1F3FB-200D-1F3EB"},
    {"lærer_farve2", "1F9D1-1F3FC-200D-1F3EB"},
    {"lærer_farve3", "1F9D1-1F3FD-200D-1F3EB"},
    {"lærer_farve4", "1F9D1-1F3FE-200D-1F3EB"},
    {"lærer_farve5", "1F9D1-1F3FF-200D-1F3EB"},
    {"person_med_blindestok_mod_højre", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestok_mod_højre_farve1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestok_mod_højre_farve2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestok_mod_højre_farve3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestok_mod_højre_farve4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_blindestok_mod_højre_farve5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"person_i_elkørestol", "1F9D1-200D-1F9BC"},
    {"person_i_elkørestol_farve1", "1F9D1-1F3FB-200D-1F9BC"},
    {"person_i_elkørestol_farve2", "1F9D1-1F3FC-200D-1F9BC"},
    {"person_i_elkørestol_farve3", "1F9D1-1F3FD-200D-1F9BC"},
    {"person_i_elkørestol_farve4", "1F9D1-1F3FE-200D-1F9BC"},
    {"person_i_elkørestol_farve5", "1F9D1-1F3FF-200D-1F9BC"},
    {"person_i_elkørestol_mod_højre", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_elkørestol_mod_højre_farve1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_elkørestol_mod_højre_farve2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_elkørestol_mod_højre_farve3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_elkørestol_mod_højre_farve4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_elkørestol_mod_højre_farve5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_manuel_kørestol", "1F9D1-200D-1F9BD"},
    {"person_i_manuel_kørestol_farve1", "1F9D1-1F3FB-200D-1F9BD"},
    {"person_i_manuel_kørestol_farve2", "1F9D1-1F3FC-200D-1F9BD"},
    {"person_i_manuel_kørestol_farve3", "1F9D1-1F3FD-200D-1F9BD"},
    {"person_i_manuel_kørestol_farve4", "1F9D1-1F3FE-200D-1F9BD"},
    {"person_i_manuel_kørestol_farve5", "1F9D1-1F3FF-200D-1F9BD"},
    {"person_i_manuel_kørestol_mod_højre", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuel_kørestol_mod_højre_farve1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuel_kørestol_mod_højre_farve2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuel_kørestol_mod_højre_farve3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuel_kørestol_mod_højre_farve4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuel_kørestol_mod_højre_farve5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinde_løber", "1F3C3-200D-2640-FE0F"},
    {"kvinde_løber_farve1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"kvinde_løber_farve2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"kvinde_løber_farve3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"kvinde_løber_farve4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"kvinde_løber_farve5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"kvinde_løber_mod_højre", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_løber_mod_højre_farve1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_løber_mod_højre_farve2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_løber_mod_højre_farve3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_løber_mod_højre_farve4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_løber_mod_højre_farve5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mand_løber", "1F3C3-200D-2642-FE0F"},
    {"mand_løber_farve1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"mand_løber_farve2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"mand_løber_farve3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"mand_løber_farve4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"mand_løber_farve5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"mand_løber_mod_højre", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_løber_mod_højre_farve1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_løber_mod_højre_farve2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_løber_mod_højre_farve3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_løber_mod_højre_farve4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_løber_mod_højre_farve5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kvindelig_svømmer", "1F3CA-200D-2640-FE0F"},
    {"kvindelig_svømmer_farve1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"kvindelig_svømmer_farve2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"kvindelig_svømmer_farve3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"kvindelig_svømmer_farve4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"kvindelig_svømmer_farve5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"mandlig_svømmer", "1F3CA-200D-2642-FE0F"},
    {"mandlig_svømmer_farve1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"mandlig_svømmer_farve2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"mandlig_svømmer_farve3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"mandlig_svømmer_farve4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"mandlig_svømmer_farve5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"kvindelig_vægtløfter", "1F3CB-FE0F-200D-2640-FE0F"},
    {"kvindelig_vægtløfter_farve1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"kvindelig_vægtløfter_farve2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"kvindelig_vægtløfter_farve3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"kvindelig_vægtløfter_farve4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"kvindelig_vægtløfter_farve5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"mandlig_vægtløfter", "1F3CB-FE0F-200D-2642-FE0F"},
    {"mandlig_vægtløfter_farve1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"mandlig_vægtløfter_farve2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"mandlig_vægtløfter_farve3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"mandlig_vægtløfter_farve4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"mandlig_vægtløfter_farve5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"kvinder_med_kaninører", "1F46F-200D-2640-FE0F"},
    {"kvinder_med_kaninører_farve1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"kvinder_med_kaninører_farve1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"kvinder_med_kaninører_farve1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"kvinder_med_kaninører_farve1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"kvinder_med_kaninører_farve2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"kvinder_med_kaninører_farve2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"kvinder_med_kaninører_farve2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"kvinder_med_kaninører_farve2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"kvinder_med_kaninører_farve3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"kvinder_med_kaninører_farve3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"kvinder_med_kaninører_farve3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"kvinder_med_kaninører_farve3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"kvinder_med_kaninører_farve4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"kvinder_med_kaninører_farve4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"kvinder_med_kaninører_farve4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"kvinder_med_kaninører_farve4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"kvinder_med_kaninører_farve5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"kvinder_med_kaninører_farve5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"kvinder_med_kaninører_farve5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"kvinder_med_kaninører_farve5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"kvinder_med_kaninører_farve1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"kvinder_med_kaninører_farve2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"kvinder_med_kaninører_farve3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"kvinder_med_kaninører_farve4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"kvinder_med_kaninører_farve5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"mænd_med_kaninører", "1F46F-200D-2642-FE0F"},
    {"mænd_med_kaninører_farve1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"mænd_med_kaninører_farve1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"mænd_med_kaninører_farve1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"mænd_med_kaninører_farve1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"mænd_med_kaninører_farve2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"mænd_med_kaninører_farve2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"mænd_med_kaninører_farve2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"mænd_med_kaninører_farve2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"mænd_med_kaninører_farve3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"mænd_med_kaninører_farve3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"mænd_med_kaninører_farve3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"mænd_med_kaninører_farve3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"mænd_med_kaninører_farve4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"mænd_med_kaninører_farve4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"mænd_med_kaninører_farve4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"mænd_med_kaninører_farve4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"mænd_med_kaninører_farve5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"mænd_med_kaninører_farve5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"mænd_med_kaninører_farve5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"mænd_med_kaninører_farve5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"mænd_med_kaninører_farve1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"mænd_med_kaninører_farve2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"mænd_med_kaninører_farve3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"mænd_med_kaninører_farve4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"mænd_med_kaninører_farve5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"kvinde_med_brudeslør", "1F470-200D-2640-FE0F"},
    {"kvinde_med_brudeslør_farve1", "1F470-1F3FB-200D-2640-FE0F"},
    {"kvinde_med_brudeslør_farve2", "1F470-1F3FC-200D-2640-FE0F"},
    {"kvinde_med_brudeslør_farve3", "1F470-1F3FD-200D-2640-FE0F"},
    {"kvinde_med_brudeslør_farve4", "1F470-1F3FE-200D-2640-FE0F"},
    {"kvinde_med_brudeslør_farve5", "1F470-1F3FF-200D-2640-FE0F"},
    {"mand_med_brudeslør", "1F470-200D-2642-FE0F"},
    {"mand_med_brudeslør_farve1", "1F470-1F3FB-200D-2642-FE0F"},
    {"mand_med_brudeslør_farve2", "1F470-1F3FC-200D-2642-FE0F"},
    {"mand_med_brudeslør_farve3", "1F470-1F3FD-200D-2642-FE0F"},
    {"mand_med_brudeslør_farve4", "1F470-1F3FE-200D-2642-FE0F"},
    {"mand_med_brudeslør_farve5", "1F470-1F3FF-200D-2642-FE0F"},
    {"kvinde_vender_håndfladen_op", "1F481-200D-2640-FE0F"},
    {"kvinde_vender_håndfladen_op_farve1", "1F481-1F3FB-200D-2640-FE0F"},
    {"kvinde_vender_håndfladen_op_farve2", "1F481-1F3FC-200D-2640-FE0F"},
    {"kvinde_vender_håndfladen_op_farve3", "1F481-1F3FD-200D-2640-FE0F"},
    {"kvinde_vender_håndfladen_op_farve4", "1F481-1F3FE-200D-2640-FE0F"},
    {"kvinde_vender_håndfladen_op_farve5", "1F481-1F3FF-200D-2640-FE0F"},
    {"mand_vender_håndfladen_op", "1F481-200D-2642-FE0F"},
    {"mand_vender_håndfladen_op_farve1", "1F481-1F3FB-200D-2642-FE0F"},
    {"mand_vender_håndfladen_op_farve2", "1F481-1F3FC-200D-2642-FE0F"},
    {"mand_vender_håndfladen_op_farve3", "1F481-1F3FD-200D-2642-FE0F"},
    {"mand_vender_håndfladen_op_farve4", "1F481-1F3FE-200D-2642-FE0F"},
    {"mand_vender_håndfladen_op_farve5", "1F481-1F3FF-200D-2642-FE0F"},
    {"kvinde_får_ansigtsmassage", "1F486-200D-2640-FE0F"},
    {"kvinde_får_ansigtsmassage_farve1", "1F486-1F3FB-200D-2640-FE0F"},
    {"kvinde_får_ansigtsmassage_farve2", "1F486-1F3FC-200D-2640-FE0F"},
    {"kvinde_får_ansigtsmassage_farve3", "1F486-1F3FD-200D-2640-FE0F"},
    {"kvinde_får_ansigtsmassage_farve4", "1F486-1F3FE-200D-2640-FE0F"},
    {"kvinde_får_ansigtsmassage_farve5", "1F486-1F3FF-200D-2640-FE0F"},
    {"mand_får_ansigtsmassage", "1F486-200D-2642-FE0F"},
    {"mand_får_ansigtsmassage_farve1", "1F486-1F3FB-200D-2642-FE0F"},
    {"mand_får_ansigtsmassage_farve2", "1F486-1F3FC-200D-2642-FE0F"},
    {"mand_får_ansigtsmassage_farve3", "1F486-1F3FD-200D-2642-FE0F"},
    {"mand_får_ansigtsmassage_farve4", "1F486-1F3FE-200D-2642-FE0F"},
    {"mand_får_ansigtsmassage_farve5", "1F486-1F3FF-200D-2642-FE0F"},
    {"kvinde_med_løftet_hånd", "1F64B-200D-2640-FE0F"},
    {"kvinde_med_løftet_hånd_farve1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"kvinde_med_løftet_hånd_farve2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"kvinde_med_løftet_hånd_farve3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"kvinde_med_løftet_hånd_farve4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"kvinde_med_løftet_hånd_farve5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"mand_med_løftet_hånd", "1F64B-200D-2642-FE0F"},
    {"mand_med_løftet_hånd_farve1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"mand_med_løftet_hånd_farve2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"mand_med_løftet_hånd_farve3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"mand_med_løftet_hånd_farve4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"mand_med_løftet_hånd_farve5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"kvinde_i_robåd", "1F6A3-200D-2640-FE0F"},
    {"kvinde_i_robåd_farve1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"kvinde_i_robåd_farve2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"kvinde_i_robåd_farve3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"kvinde_i_robåd_farve4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"kvinde_i_robåd_farve5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"mand_i_robåd", "1F6A3-200D-2642-FE0F"},
    {"mand_i_robåd_farve1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"mand_i_robåd_farve2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"mand_i_robåd_farve3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"mand_i_robåd_farve4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"mand_i_robåd_farve5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"kvinde_på_mountainbike", "1F6B5-200D-2640-FE0F"},
    {"kvinde_på_mountainbike_farve1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"kvinde_på_mountainbike_farve2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"kvinde_på_mountainbike_farve3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"kvinde_på_mountainbike_farve4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"kvinde_på_mountainbike_farve5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"mand_på_mountainbike", "1F6B5-200D-2642-FE0F"},
    {"mand_på_mountainbike_farve1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"mand_på_mountainbike_farve2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"mand_på_mountainbike_farve3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"mand_på_mountainbike_farve4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"mand_på_mountainbike_farve5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"kvinde_går", "1F6B6-200D-2640-FE0F"},
    {"kvinde_går_farve1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"kvinde_går_farve2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"kvinde_går_farve3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"kvinde_går_farve4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"kvinde_går_farve5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"kvinde_går_mod_højre", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_går_mod_højre_farve1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_går_mod_højre_farve2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_går_mod_højre_farve3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_går_mod_højre_farve4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinde_går_mod_højre_farve5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"mand_går", "1F6B6-200D-2642-FE0F"},
    {"mand_går_farve1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"mand_går_farve2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"mand_går_farve3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"mand_går_farve4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"mand_går_farve5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"mand_går_mod_højre", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_går_mod_højre_farve1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_går_mod_højre_farve2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_går_mod_højre_farve3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_går_mod_højre_farve4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"mand_går_mod_højre_farve5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kvinde_slår_sig_for_panden", "1F926-200D-2640-FE0F"},
    {"kvinde_slår_sig_for_panden_farve1", "1F926-1F3FB-200D-2640-FE0F"},
    {"kvinde_slår_sig_for_panden_farve2", "1F926-1F3FC-200D-2640-FE0F"},
    {"kvinde_slår_sig_for_panden_farve3", "1F926-1F3FD-200D-2640-FE0F"},
    {"kvinde_slår_sig_for_panden_farve4", "1F926-1F3FE-200D-2640-FE0F"},
    {"kvinde_slår_sig_for_panden_farve5", "1F926-1F3FF-200D-2640-FE0F"},
    {"mand_slår_sig_for_panden", "1F926-200D-2642-FE0F"},
    {"mand_slår_sig_for_panden_farve1", "1F926-1F3FB-200D-2642-FE0F"},
    {"mand_slår_sig_for_panden_farve2", "1F926-1F3FC-200D-2642-FE0F"},
    {"mand_slår_sig_for_panden_farve3", "1F926-1F3FD-200D-2642-FE0F"},
    {"mand_slår_sig_for_panden_farve4", "1F926-1F3FE-200D-2642-FE0F"},
    {"mand_slår_sig_for_panden_farve5", "1F926-1F3FF-200D-2642-FE0F"},
    {"kvinde_trækker_på_skuldrene", "1F937-200D-2640-FE0F"},
    {"kvinde_trækker_på_skuldrene_farve1", "1F937-1F3FB-200D-2640-FE0F"},
    {"kvinde_trækker_på_skuldrene_farve2", "1F937-1F3FC-200D-2640-FE0F"},
    {"kvinde_trækker_på_skuldrene_farve3", "1F937-1F3FD-200D-2640-FE0F"},
    {"kvinde_trækker_på_skuldrene_farve4", "1F937-1F3FE-200D-2640-FE0F"},
    {"kvinde_trækker_på_skuldrene_farve5", "1F937-1F3FF-200D-2640-FE0F"},
    {"mand_trækker_på_skuldrene", "1F937-200D-2642-FE0F"},
    {"mand_trækker_på_skuldrene_farve1", "1F937-1F3FB-200D-2642-FE0F"},
    {"mand_trækker_på_skuldrene_farve2", "1F937-1F3FC-200D-2642-FE0F"},
    {"mand_trækker_på_skuldrene_farve3", "1F937-1F3FD-200D-2642-FE0F"},
    {"mand_trækker_på_skuldrene_farve4", "1F937-1F3FE-200D-2642-FE0F"},
    {"mand_trækker_på_skuldrene_farve5", "1F937-1F3FF-200D-2642-FE0F"},
    {"kvinde_slår_vejrmøller", "1F938-200D-2640-FE0F"},
    {"kvinde_slår_vejrmøller_farve1", "1F938-1F3FB-200D-2640-FE0F"},
    {"kvinde_slår_vejrmøller_farve2", "1F938-1F3FC-200D-2640-FE0F"},
    {"kvinde_slår_vejrmøller_farve3", "1F938-1F3FD-200D-2640-FE0F"},
    {"kvinde_slår_vejrmøller_farve4", "1F938-1F3FE-200D-2640-FE0F"},
    {"kvinde_slår_vejrmøller_farve5", "1F938-1F3FF-200D-2640-FE0F"},
    {"mand_slår_vejrmøller", "1F938-200D-2642-FE0F"},
    {"mand_slår_vejrmøller_farve1", "1F938-1F3FB-200D-2642-FE0F"},
    {"mand_slår_vejrmøller_farve2", "1F938-1F3FC-200D-2642-FE0F"},
    {"mand_slår_vejrmøller_farve3", "1F938-1F3FD-200D-2642-FE0F"},
    {"mand_slår_vejrmøller_farve4", "1F938-1F3FE-200D-2642-FE0F"},
    {"mand_slår_vejrmøller_farve5", "1F938-1F3FF-200D-2642-FE0F"},
    {"kvindelig_jonglør", "1F939-200D-2640-FE0F"},
    {"kvindelig_jonglør_farve1", "1F939-1F3FB-200D-2640-FE0F"},
    {"kvindelig_jonglør_farve2", "1F939-1F3FC-200D-2640-FE0F"},
    {"kvindelig_jonglør_farve3", "1F939-1F3FD-200D-2640-FE0F"},
    {"kvindelig_jonglør_farve4", "1F939-1F3FE-200D-2640-FE0F"},
    {"kvindelig_jonglør_farve5", "1F939-1F3FF-200D-2640-FE0F"},
    {"mandlig_jonglør", "1F939-200D-2642-FE0F"},
    {"mandlig_jonglør_farve1", "1F939-1F3FB-200D-2642-FE0F"},
    {"mandlig_jonglør_farve2", "1F939-1F3FC-200D-2642-FE0F"},
    {"mandlig_jonglør_farve3", "1F939-1F3FD-200D-2642-FE0F"},
    {"mandlig_jonglør_farve4", "1F939-1F3FE-200D-2642-FE0F"},
    {"mandlig_jonglør_farve5", "1F939-1F3FF-200D-2642-FE0F"},
    {"kvindelig_håndboldspiller", "1F93E-200D-2640-FE0F"},
    {"kvindelig_håndboldspiller_farve1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"kvindelig_håndboldspiller_farve2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"kvindelig_håndboldspiller_farve3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"kvindelig_håndboldspiller_farve4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"kvindelig_håndboldspiller_farve5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"mandlig_håndboldspiller", "1F93E-200D-2642-FE0F"},
    {"mandlig_håndboldspiller_farve1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"mandlig_håndboldspiller_farve2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"mandlig_håndboldspiller_farve3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"mandlig_håndboldspiller_farve4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"mandlig_håndboldspiller_farve5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"stående_kvinde", "1F9CD-200D-2640-FE0F"},
    {"stående_kvinde_farve1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"stående_kvinde_farve2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"stående_kvinde_farve3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"stående_kvinde_farve4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"stående_kvinde_farve5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"stående_mand", "1F9CD-200D-2642-FE0F"},
    {"stående_mand_farve1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"stående_mand_farve2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"stående_mand_farve3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"stående_mand_farve4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"stående_mand_farve5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"knælende_kvinde", "1F9CE-200D-2640-FE0F"},
    {"knælende_kvinde_farve1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"knælende_kvinde_farve2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"knælende_kvinde_farve3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"knælende_kvinde_farve4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"knælende_kvinde_farve5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"knælende_kvinde_mod_højre", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knælende_kvinde_mod_højre_farve1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knælende_kvinde_mod_højre_farve2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knælende_kvinde_mod_højre_farve3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knælende_kvinde_mod_højre_farve4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knælende_kvinde_mod_højre_farve5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knælende_mand", "1F9CE-200D-2642-FE0F"},
    {"knælende_mand_farve1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"knælende_mand_farve2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"knælende_mand_farve3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"knælende_mand_farve4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"knælende_mand_farve5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"knælende_mand_mod_højre", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knælende_mand_mod_højre_farve1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knælende_mand_mod_højre_farve2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knælende_mand_mod_højre_farve3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knælende_mand_mod_højre_farve4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knælende_mand_mod_højre_farve5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"døv_kvinde", "1F9CF-200D-2640-FE0F"},
    {"døv_kvinde_farve1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"døv_kvinde_farve2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"døv_kvinde_farve3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"døv_kvinde_farve4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"døv_kvinde_farve5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"døv_mand", "1F9CF-200D-2642-FE0F"},
    {"døv_mand_farve1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"døv_mand_farve2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"døv_mand_farve3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"døv_mand_farve4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"døv_mand_farve5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"kvinde_med_skæg", "1F9D4-200D-2640-FE0F"},
    {"kvinde_med_skæg_farve1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"kvinde_med_skæg_farve2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"kvinde_med_skæg_farve3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"kvinde_med_skæg_farve4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"kvinde_med_skæg_farve5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"mand_med_skæg", "1F9D4-200D-2642-FE0F"},
    {"mand_med_skæg_farve1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"mand_med_skæg_farve2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"mand_med_skæg_farve3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"mand_med_skæg_farve4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"mand_med_skæg_farve5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"kvindelig_magisk_ånd", "1F9DE-200D-2640-FE0F"},
    {"mandlig_magisk_ånd", "1F9DE-200D-2642-FE0F"},
    {"mand_rødt_hår", "1F468-200D-1F9B0"},
    {"mand_rødt_hår_farve1", "1F468-1F3FB-200D-1F9B0"},
    {"mand_rødt_hår_farve2", "1F468-1F3FC-200D-1F9B0"},
    {"mand_rødt_hår_farve3", "1F468-1F3FD-200D-1F9B0"},
    {"mand_rødt_hår_farve4", "1F468-1F3FE-200D-1F9B0"},
    {"mand_rødt_hår_farve5", "1F468-1F3FF-200D-1F9B0"},
    {"mand_krøllet_hår", "1F468-200D-1F9B1"},
    {"mand_krøllet_hår_farve1", "1F468-1F3FB-200D-1F9B1"},
    {"mand_krøllet_hår_farve2", "1F468-1F3FC-200D-1F9B1"},
    {"mand_krøllet_hår_farve3", "1F468-1F3FD-200D-1F9B1"},
    {"mand_krøllet_hår_farve4", "1F468-1F3FE-200D-1F9B1"},
    {"mand_krøllet_hår_farve5", "1F468-1F3FF-200D-1F9B1"},
    {"mand_hvidt_hår", "1F468-200D-1F9B3"},
    {"mand_hvidt_hår_farve1", "1F468-1F3FB-200D-1F9B3"},
    {"mand_hvidt_hår_farve2", "1F468-1F3FC-200D-1F9B3"},
    {"mand_hvidt_hår_farve3", "1F468-1F3FD-200D-1F9B3"},
    {"mand_hvidt_hår_farve4", "1F468-1F3FE-200D-1F9B3"},
    {"mand_hvidt_hår_farve5", "1F468-1F3FF-200D-1F9B3"},
    {"kvinde_rødt_hår", "1F469-200D-1F9B0"},
    {"kvinde_rødt_hår_farve1", "1F469-1F3FB-200D-1F9B0"},
    {"kvinde_rødt_hår_farve2", "1F469-1F3FC-200D-1F9B0"},
    {"kvinde_rødt_hår_farve3", "1F469-1F3FD-200D-1F9B0"},
    {"kvinde_rødt_hår_farve4", "1F469-1F3FE-200D-1F9B0"},
    {"kvinde_rødt_hår_farve5", "1F469-1F3FF-200D-1F9B0"},
    {"kvinde_krøllet_hår", "1F469-200D-1F9B1"},
    {"kvinde_krøllet_hår_farve1", "1F469-1F3FB-200D-1F9B1"},
    {"kvinde_krøllet_hår_farve2", "1F469-1F3FC-200D-1F9B1"},
    {"kvinde_krøllet_hår_farve3", "1F469-1F3FD-200D-1F9B1"},
    {"kvinde_krøllet_hår_farve4", "1F469-1F3FE-200D-1F9B1"},
    {"kvinde_krøllet_hår_farve5", "1F469-1F3FF-200D-1F9B1"},
    {"kvinde_hvidt_hår", "1F469-200D-1F9B3"},
    {"kvinde_hvidt_hår_farve1", "1F469-1F3FB-200D-1F9B3"},
    {"kvinde_hvidt_hår_farve2", "1F469-1F3FC-200D-1F9B3"},
    {"kvinde_hvidt_hår_farve3", "1F469-1F3FD-200D-1F9B3"},
    {"kvinde_hvidt_hår_farve4", "1F469-1F3FE-200D-1F9B3"},
    {"kvinde_hvidt_hår_farve5", "1F469-1F3FF-200D-1F9B3"},
    {"voksen_rødt_hår", "1F9D1-200D-1F9B0"},
    {"voksen_rødt_hår_farve1", "1F9D1-1F3FB-200D-1F9B0"},
    {"voksen_rødt_hår_farve2", "1F9D1-1F3FC-200D-1F9B0"},
    {"voksen_rødt_hår_farve3", "1F9D1-1F3FD-200D-1F9B0"},
    {"voksen_rødt_hår_farve4", "1F9D1-1F3FE-200D-1F9B0"},
    {"voksen_rødt_hår_farve5", "1F9D1-1F3FF-200D-1F9B0"},
    {"voksen_krøllet_hår", "1F9D1-200D-1F9B1"},
    {"voksen_krøllet_hår_farve1", "1F9D1-1F3FB-200D-1F9B1"},
    {"voksen_krøllet_hår_farve2", "1F9D1-1F3FC-200D-1F9B1"},
    {"voksen_krøllet_hår_farve3", "1F9D1-1F3FD-200D-1F9B1"},
    {"voksen_krøllet_hår_farve4", "1F9D1-1F3FE-200D-1F9B1"},
    {"voksen_krøllet_hår_farve5", "1F9D1-1F3FF-200D-1F9B1"},
    {"voksen_hvidt_hår", "1F9D1-200D-1F9B3"},
    {"voksen_hvidt_hår_farve1", "1F9D1-1F3FB-200D-1F9B3"},
    {"voksen_hvidt_hår_farve2", "1F9D1-1F3FC-200D-1F9B3"},
    {"voksen_hvidt_hår_farve3", "1F9D1-1F3FD-200D-1F9B3"},
    {"voksen_hvidt_hår_farve4", "1F9D1-1F3FE-200D-1F9B3"},
    {"voksen_hvidt_hår_farve5", "1F9D1-1F3FF-200D-1F9B3"},
    {"brudt_kæde", "26D3-FE0F-200D-1F4A5"},
    {"brændende_hjerte", "2764-FE0F-200D-1F525"},
    {"transkønnet_flag", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"føniks", "1F426-200D-1F525"},
    {"isbjørn", "1F43B-200D-2744-FE0F"},
    {"øje_i_taleboble", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"udåndende_ansigt", "1F62E-200D-1F4A8"},
    {"ansigt_med_spiraløjne", "1F635-200D-1F4AB"},
    {"ryster_på_hovedet_vandret", "1F642-200D-2194-FE0F"},
    {"ryster_på_hovedet_lodret", "1F642-200D-2195-FE0F"},
};

