// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_de[] = {
    {"ausrufe_und_fragezeichen", "2049"},
    {"markenzeichen", "2122"},
    {"buchstabe_i_in_blauem_quadrat", "2139"},
    {"pfeil_nach_links_und_rechts", "2194"},
    {"pfeil_nach_oben_und_unten", "2195"},
    {"pfeil_nach_links_oben", "2196"},
    {"pfeil_nach_rechts_oben", "2197"},
    {"pfeil_nach_rechts_unten", "2198"},
    {"pfeil_nach_links_unten", "2199"},
    {"tastatur", "2328"},
    {"sonne", "2600"},
    {"wolke", "2601"},
    {"regenschirm", "2602"},
    {"schneemann_im_schnee", "2603"},
    {"komet", "2604"},
    {"abgehaktes_kastchen", "2611"},
    {"regenschirm_im_regen", "2614"},
    {"heissgetrank", "2615"},
    {"kleeblatt", "2618"},
    {"totenkopf_mit_gekreuzten_knochen", "2620"},
    {"radioaktiv", "2622"},
    {"biogefahrdung", "2623"},
    {"orthodoxes_kreuz", "2626"},
    {"dharma_rad", "2638"},
    {"dusteres_gesicht", "2639"},
    {"frauensymbol", "2640"},
    {"mannersymbol", "2642"},
    {"widder_sternzeichen", "2648"},
    {"stier_sternzeichen", "2649"},
    {"schutze_sternzeichen", "2650"},
    {"steinbock_sternzeichen", "2651"},
    {"wassermann_sternzeichen", "2652"},
    {"fische_sternzeichen", "2653"},
    {"pik", "2660"},
    {"kreuz", "2663"},
    {"herz", "2665"},
    {"karo", "2666"},
    {"heisse_quellen", "2668"},
    {"hammer_und_pickel", "2692"},
    {"anker", "2693"},
    {"gekreuzte_schwerter", "2694"},
    {"askulapstab", "2695"},
    {"waage", "2696"},
    {"destillierapparat", "2697"},
    {"zahnrad", "2699"},
    {"schere", "2702"},
    {"weisses_hakchen", "2705"},
    {"flugzeug", "2708"},
    {"briefumschlag", "2709"},
    {"schwarzer_federhalter", "2712"},
    {"kraftiges_hakchen", "2714"},
    {"mal", "2716"},
    {"davidstern", "2721"},
    {"funkelnde_sterne", "2728"},
    {"achtzackiger_stern", "2733"},
    {"achtstrahliger_stern", "2734"},
    {"schneeflocke", "2744"},
    {"funkeln", "2747"},
    {"rotes_fragezeichen", "2753"},
    {"weisses_fragezeichen", "2754"},
    {"weisses_ausrufezeichen", "2755"},
    {"rotes_ausrufezeichen", "2757"},
    {"herz_als_ausrufezeichen", "2763"},
    {"rotes_herz", "2764"},
    {"plus", "2795"},
    {"minus", "2796"},
    {"geteilt_durch", "2797"},
    {"geschwungener_pfeil_nach_oben", "2934"},
    {"geschwungener_pfeil_nach_unten", "2935"},
    {"wellenlinie", "3030"},
    {"schriftzeichen_fur_gratulation", "3297"},
    {"schriftzeichen_fur_geheimnis", "3299"},
    {"copyright", "00A9"},
    {"registered_trademark", "00AE"},
    {"doppeltes_ausrufezeichen", "203C"},
    {"geschwungener_pfeil_nach_links", "21A9"},
    {"geschwungener_pfeil_nach_rechts", "21AA"},
    {"armbanduhr", "231A"},
    {"sanduhr", "231B"},
    {"auswerfen", "23CF"},
    {"doppelpfeile_nach_rechts", "23E9"},
    {"zuruckspulen", "23EA"},
    {"doppelpfeile_nach_oben", "23EB"},
    {"doppelpfeile_nach_unten", "23EC"},
    {"nachster_titel", "23ED"},
    {"vorheriger_titel", "23EE"},
    {"wiedergabe_oder_pause", "23EF"},
    {"wecker", "23F0"},
    {"stoppuhr", "23F1"},
    {"zeitschaltuhr", "23F2"},
    {"laufende_sanduhr", "23F3"},
    {"pause", "23F8"},
    {"stopp", "23F9"},
    {"aufnehmen", "23FA"},
    {"buchstabe_m_in_kreis", "24C2"},
    {"kleines_schwarzes_quadrat", "25AA"},
    {"kleines_weisses_quadrat", "25AB"},
    {"wiedergabe", "25B6"},
    {"pfeil_zuruck", "25C0"},
    {"mittelgrosses_weisses_quadrat", "25FB"},
    {"mittelgrosses_schwarzes_quadrat", "25FC"},
    {"mittelkleines_weisses_quadrat", "25FD"},
    {"mittelkleines_schwarzes_quadrat", "25FE"},
    {"telefon", "260E"},
    {"nach_oben_weisender_zeigefinger_von_vorne", "261D"},
    {"nach_oben_weisender_zeigefinger_von_vorne_farbe1", "261D-1F3FB"},
    {"nach_oben_weisender_zeigefinger_von_vorne_farbe2", "261D-1F3FC"},
    {"nach_oben_weisender_zeigefinger_von_vorne_farbe3", "261D-1F3FD"},
    {"nach_oben_weisender_zeigefinger_von_vorne_farbe4", "261D-1F3FE"},
    {"nach_oben_weisender_zeigefinger_von_vorne_farbe5", "261D-1F3FF"},
    {"hilal_und_stern", "262A"},
    {"friedenszeichen", "262E"},
    {"yin_und_yang", "262F"},
    {"lachelndes_gesicht", "263A"},
    {"zwillinge_sternzeichen", "264A"},
    {"krebs_sternzeichen", "264B"},
    {"lowe_sternzeichen", "264C"},
    {"jungfrau_sternzeichen", "264D"},
    {"waage_sternzeichen", "264E"},
    {"skorpion_sternzeichen", "264F"},
    {"bauer_schach", "265F"},
    {"recycling_symbol", "267B"},
    {"unendlichkeit", "267E"},
    {"symbol_rollstuhl", "267F"},
    {"atomzeichen", "269B"},
    {"lilie", "269C"},
    {"warnung", "26A0"},
    {"hochspannung", "26A1"},
    {"transgender_symbol", "26A7"},
    {"weisser_punkt", "26AA"},
    {"schwarzer_punkt", "26AB"},
    {"sarg", "26B0"},
    {"urne", "26B1"},
    {"fussball", "26BD"},
    {"baseball", "26BE"},
    {"schneemann_ohne_schneeflocken", "26C4"},
    {"sonne_hinter_wolke", "26C5"},
    {"wolke_mit_blitz_und_regen", "26C8"},
    {"schlangentrager", "26CE"},
    {"pickel", "26CF"},
    {"rettungshelm", "26D1"},
    {"ketten", "26D3"},
    {"zutritt_verboten", "26D4"},
    {"shinto_schrein", "26E9"},
    {"kirche", "26EA"},
    {"berg", "26F0"},
    {"aufgestellter_sonnenschirm", "26F1"},
    {"springbrunnen", "26F2"},
    {"golffahne", "26F3"},
    {"fahre", "26F4"},
    {"segelboot", "26F5"},
    {"skifahrerin", "26F7"},
    {"schlittschuh", "26F8"},
    {"person_mit_ball", "26F9"},
    {"person_mit_ball_farbe1", "26F9-1F3FB"},
    {"person_mit_ball_farbe2", "26F9-1F3FC"},
    {"person_mit_ball_farbe3", "26F9-1F3FD"},
    {"person_mit_ball_farbe4", "26F9-1F3FE"},
    {"person_mit_ball_farbe5", "26F9-1F3FF"},
    {"zelt", "26FA"},
    {"tanksaule", "26FD"},
    {"erhobene_faust", "270A"},
    {"erhobene_faust_farbe1", "270A-1F3FB"},
    {"erhobene_faust_farbe2", "270A-1F3FC"},
    {"erhobene_faust_farbe3", "270A-1F3FD"},
    {"erhobene_faust_farbe4", "270A-1F3FE"},
    {"erhobene_faust_farbe5", "270A-1F3FF"},
    {"erhobene_hand", "270B"},
    {"erhobene_hand_farbe1", "270B-1F3FB"},
    {"erhobene_hand_farbe2", "270B-1F3FC"},
    {"erhobene_hand_farbe3", "270B-1F3FD"},
    {"erhobene_hand_farbe4", "270B-1F3FE"},
    {"erhobene_hand_farbe5", "270B-1F3FF"},
    {"victory_geste", "270C"},
    {"victory_geste_farbe1", "270C-1F3FB"},
    {"victory_geste_farbe2", "270C-1F3FC"},
    {"victory_geste_farbe3", "270C-1F3FD"},
    {"victory_geste_farbe4", "270C-1F3FE"},
    {"victory_geste_farbe5", "270C-1F3FF"},
    {"schreibende_hand", "270D"},
    {"schreibende_hand_farbe1", "270D-1F3FB"},
    {"schreibende_hand_farbe2", "270D-1F3FC"},
    {"schreibende_hand_farbe3", "270D-1F3FD"},
    {"schreibende_hand_farbe4", "270D-1F3FE"},
    {"schreibende_hand_farbe5", "270D-1F3FF"},
    {"bleistift", "270F"},
    {"lateinisches_kreuz", "271D"},
    {"kreuzzeichen", "274C"},
    {"angekreuztes_kastchen", "274E"},
    {"pfeil_nach_rechts", "27A1"},
    {"schleife", "27B0"},
    {"doppelschleife", "27BF"},
    {"pfeil_nach_links", "2B05"},
    {"pfeil_nach_oben", "2B06"},
    {"pfeil_nach_unten", "2B07"},
    {"grosses_schwarzes_quadrat", "2B1B"},
    {"grosses_weisses_quadrat", "2B1C"},
    {"weisser_mittelgrosser_stern", "2B50"},
    {"hohler_roter_kreis", "2B55"},
    {"teilalternationszeichen", "303D"},
    {"mahjong_stein", "1F004"},
    {"jokerkarte", "1F0CF"},
    {"grossbuchstabe_a_in_rotem_quadrat", "1F170"},
    {"grossbuchstabe_b_in_rotem_quadrat", "1F171"},
    {"grossbuchstabe_o_in_rotem_quadrat", "1F17E"},
    {"grossbuchstabe_p_in_blauem_quadrat", "1F17F"},
    {"grossbuchstaben_ab_in_rotem_quadrat", "1F18E"},
    {"grossbuchstaben_cl_in_rotem_quadrat", "1F191"},
    {"wort_cool_in_blauem_quadrat", "1F192"},
    {"wort_free_in_blauem_quadrat", "1F193"},
    {"grossbuchstaben_id_in_lila_quadrat", "1F194"},
    {"wort_new_in_blauem_quadrat", "1F195"},
    {"grossbuchstaben_ng_in_blauem_quadrat", "1F196"},
    {"grossbuchstaben_ok_in_blauem_quadrat", "1F197"},
    {"sos_zeichen", "1F198"},
    {"schriftzug_up_im_blauen_quadrat", "1F199"},
    {"schriftzug_vs_in_orangem_quadrat", "1F19A"},
    {"regionaler_indikator_a", "1F1E6"},
    {"regionaler_indikator_b", "1F1E7"},
    {"regionaler_indikator_c", "1F1E8"},
    {"regionaler_indikator_d", "1F1E9"},
    {"regionaler_indikator_e", "1F1EA"},
    {"regionaler_indikator_f", "1F1EB"},
    {"regionaler_indikator_g", "1F1EC"},
    {"regionaler_indikator_h", "1F1ED"},
    {"regionaler_indikator_i", "1F1EE"},
    {"regionaler_indikator_j", "1F1EF"},
    {"regionaler_indikator_k", "1F1F0"},
    {"regionaler_indikator_l", "1F1F1"},
    {"regionaler_indikator_m", "1F1F2"},
    {"regionaler_indikator_n", "1F1F3"},
    {"regionaler_indikator_o", "1F1F4"},
    {"regionaler_indikator_p", "1F1F5"},
    {"regionaler_indikator_q", "1F1F6"},
    {"regionaler_indikator_r", "1F1F7"},
    {"regionaler_indikator_s", "1F1F8"},
    {"regionaler_indikator_t", "1F1F9"},
    {"regionaler_indikator_u", "1F1FA"},
    {"regionaler_indikator_v", "1F1FB"},
    {"regionaler_indikator_w", "1F1FC"},
    {"regionaler_indikator_x", "1F1FD"},
    {"regionaler_indikator_y", "1F1FE"},
    {"regionaler_indikator_z", "1F1FF"},
    {"schriftzeichen_koko", "1F201"},
    {"schriftzeichen_sa", "1F202"},
    {"schriftzeichen_fur_gratis", "1F21A"},
    {"schriftzeichen_fur_reserviert", "1F22F"},
    {"schriftzeichen_fur_verbieten", "1F232"},
    {"schriftzeichen_fur_zimmer_frei", "1F233"},
    {"schriftzeichen_fur_note_zum_bestehen", "1F234"},
    {"schriftzeichen_fur_kein_zimmer_frei", "1F235"},
    {"schriftzeichen_fur_nicht_gratis", "1F236"},
    {"schriftzeichen_fur_monatsbetrag", "1F237"},
    {"schriftzeichen_fur_anwenden", "1F238"},
    {"schriftzeichen_fur_rabatt", "1F239"},
    {"schriftzeichen_fur_geoffnet", "1F23A"},
    {"schriftzeichen_fur_schnappchen", "1F250"},
    {"schriftzeichen_fur_akzeptieren", "1F251"},
    {"wirbel", "1F300"},
    {"neblig", "1F301"},
    {"geschlossener_regenschirm", "1F302"},
    {"sternenhimmel", "1F303"},
    {"sonnenaufgang_uber_bergen", "1F304"},
    {"sonnenaufgang_uber_dem_meer", "1F305"},
    {"abendstimmung_in_der_stadt", "1F306"},
    {"sonnenuntergang_in_der_stadt", "1F307"},
    {"regenbogen", "1F308"},
    {"brucke_vor_nachthimmel", "1F309"},
    {"welle", "1F30A"},
    {"vulkan", "1F30B"},
    {"milchstrasse", "1F30C"},
    {"globus_mit_europa_und_afrika", "1F30D"},
    {"globus_mit_amerika", "1F30E"},
    {"globus_mit_asien_und_australien", "1F30F"},
    {"globus_mit_meridianen", "1F310"},
    {"neumond", "1F311"},
    {"erstes_mondviertel", "1F312"},
    {"zunehmender_halbmond", "1F313"},
    {"zweites_mondviertel", "1F314"},
    {"vollmond", "1F315"},
    {"drittes_mondviertel", "1F316"},
    {"abnehmender_halbmond", "1F317"},
    {"letztes_mondviertel", "1F318"},
    {"mondsichel", "1F319"},
    {"neumond_mit_gesicht", "1F31A"},
    {"mondsichel_mit_gesicht_links", "1F31B"},
    {"mondsichel_mit_gesicht_rechts", "1F31C"},
    {"vollmond_mit_gesicht", "1F31D"},
    {"sonne_mit_gesicht", "1F31E"},
    {"funkelnder_stern", "1F31F"},
    {"sternschnuppe", "1F320"},
    {"thermometer", "1F321"},
    {"sonne_hinter_kleiner_wolke", "1F324"},
    {"sonne_hinter_grosser_wolke", "1F325"},
    {"sonne_hinter_regenwolke", "1F326"},
    {"wolke_mit_regen", "1F327"},
    {"wolke_mit_schnee", "1F328"},
    {"wolke_mit_blitz", "1F329"},
    {"wirbelsturm", "1F32A"},
    {"nebel", "1F32B"},
    {"wind", "1F32C"},
    {"hotdog", "1F32D"},
    {"taco", "1F32E"},
    {"burrito", "1F32F"},
    {"kastanie", "1F330"},
    {"spross", "1F331"},
    {"nadelbaum", "1F332"},
    {"laubbaum", "1F333"},
    {"palme", "1F334"},
    {"kaktus", "1F335"},
    {"peperoni", "1F336"},
    {"tulpe", "1F337"},
    {"kirschblute", "1F338"},
    {"rose", "1F339"},
    {"hibiskus", "1F33A"},
    {"sonnenblume", "1F33B"},
    {"gelbe_blute", "1F33C"},
    {"maiskolben", "1F33D"},
    {"reisahre", "1F33E"},
    {"krauter", "1F33F"},
    {"glucksklee", "1F340"},
    {"ahornblatt", "1F341"},
    {"laub", "1F342"},
    {"blatter_im_wind", "1F343"},
    {"fliegenpilz", "1F344"},
    {"tomate", "1F345"},
    {"aubergine", "1F346"},
    {"trauben", "1F347"},
    {"honigmelone", "1F348"},
    {"wassermelone", "1F349"},
    {"mandarine", "1F34A"},
    {"zitrone", "1F34B"},
    {"banane", "1F34C"},
    {"ananas", "1F34D"},
    {"roter_apfel", "1F34E"},
    {"gruner_apfel", "1F34F"},
    {"birne", "1F350"},
    {"pfirsich", "1F351"},
    {"kirschen", "1F352"},
    {"erdbeere", "1F353"},
    {"hamburger", "1F354"},
    {"pizza", "1F355"},
    {"fleischhachse", "1F356"},
    {"hahnchenschenkel", "1F357"},
    {"reiscracker", "1F358"},
    {"reisballchen", "1F359"},
    {"reis_in_schussel", "1F35A"},
    {"reis_mit_curry", "1F35B"},
    {"schussel_und_essstabchen", "1F35C"},
    {"spaghetti", "1F35D"},
    {"brot", "1F35E"},
    {"pommes_frites", "1F35F"},
    {"gerostete_susskartoffel", "1F360"},
    {"dango", "1F361"},
    {"oden", "1F362"},
    {"sushi", "1F363"},
    {"frittierte_garnele", "1F364"},
    {"fischfrikadelle", "1F365"},
    {"softeis", "1F366"},
    {"wassereis", "1F367"},
    {"eiscreme", "1F368"},
    {"donut", "1F369"},
    {"keks", "1F36A"},
    {"schokoladentafel", "1F36B"},
    {"bonbon", "1F36C"},
    {"lutscher", "1F36D"},
    {"pudding", "1F36E"},
    {"honigtopf", "1F36F"},
    {"torte", "1F370"},
    {"bento_box", "1F371"},
    {"topf_mit_essen", "1F372"},
    {"spiegelei_in_bratpfanne", "1F373"},
    {"messer_und_gabel", "1F374"},
    {"teetasse_ohne_henkel", "1F375"},
    {"sake_flasche_mit_tasse", "1F376"},
    {"weinglas", "1F377"},
    {"cocktailglas", "1F378"},
    {"cocktail", "1F379"},
    {"bierkrug", "1F37A"},
    {"bierkruge", "1F37B"},
    {"babyflasche", "1F37C"},
    {"teller_mit_messer_und_gabel", "1F37D"},
    {"flasche_mit_knallendem_korken", "1F37E"},
    {"popcorn", "1F37F"},
    {"pinke_schleife", "1F380"},
    {"geschenk", "1F381"},
    {"geburtstagskuchen", "1F382"},
    {"halloweenkurbis", "1F383"},
    {"weihnachtsbaum", "1F384"},
    {"weihnachtsmann", "1F385"},
    {"weihnachtsmann_farbe1", "1F385-1F3FB"},
    {"weihnachtsmann_farbe2", "1F385-1F3FC"},
    {"weihnachtsmann_farbe3", "1F385-1F3FD"},
    {"weihnachtsmann_farbe4", "1F385-1F3FE"},
    {"weihnachtsmann_farbe5", "1F385-1F3FF"},
    {"feuerwerk", "1F386"},
    {"wunderkerze", "1F387"},
    {"luftballon", "1F388"},
    {"konfettibombe", "1F389"},
    {"konfettiball", "1F38A"},
    {"tanabata_baum", "1F38B"},
    {"uberkreuzte_flaggen", "1F38C"},
    {"piniendekoration", "1F38D"},
    {"japanische_puppen", "1F38E"},
    {"traditionelle_japanische_windsacke", "1F38F"},
    {"japanisches_windspiel", "1F390"},
    {"traditionelles_mondfest", "1F391"},
    {"schulranzen", "1F392"},
    {"doktorhut", "1F393"},
    {"militarorden", "1F396"},
    {"gedenkschleife", "1F397"},
    {"studiomikrofon", "1F399"},
    {"schieberegler", "1F39A"},
    {"drehregler", "1F39B"},
    {"filmstreifen", "1F39E"},
    {"eintrittskarten", "1F39F"},
    {"karussellpferd", "1F3A0"},
    {"riesenrad", "1F3A1"},
    {"achterbahn", "1F3A2"},
    {"angel_mit_fisch", "1F3A3"},
    {"mikrofon", "1F3A4"},
    {"filmkamera", "1F3A5"},
    {"kinosymbol", "1F3A6"},
    {"kopfhorer", "1F3A7"},
    {"mischpalette", "1F3A8"},
    {"zylinder", "1F3A9"},
    {"zirkuszelt", "1F3AA"},
    {"ticket", "1F3AB"},
    {"filmklappe", "1F3AC"},
    {"masken", "1F3AD"},
    {"gamepad", "1F3AE"},
    {"darts", "1F3AF"},
    {"spielautomat", "1F3B0"},
    {"billardkugel", "1F3B1"},
    {"spielwurfel", "1F3B2"},
    {"bowling", "1F3B3"},
    {"japanische_blumenkarte", "1F3B4"},
    {"musiknote", "1F3B5"},
    {"musiknoten", "1F3B6"},
    {"saxofon", "1F3B7"},
    {"gitarre", "1F3B8"},
    {"klaviatur", "1F3B9"},
    {"trompete", "1F3BA"},
    {"geige", "1F3BB"},
    {"notenschlussel", "1F3BC"},
    {"laufshirt", "1F3BD"},
    {"tennisball", "1F3BE"},
    {"ski", "1F3BF"},
    {"basketball", "1F3C0"},
    {"zielflagge", "1F3C1"},
    {"snowboarderin", "1F3C2"},
    {"snowboarderin_farbe1", "1F3C2-1F3FB"},
    {"snowboarderin_farbe2", "1F3C2-1F3FC"},
    {"snowboarderin_farbe3", "1F3C2-1F3FD"},
    {"snowboarderin_farbe4", "1F3C2-1F3FE"},
    {"snowboarderin_farbe5", "1F3C2-1F3FF"},
    {"laufende_person", "1F3C3"},
    {"laufende_person_farbe1", "1F3C3-1F3FB"},
    {"laufende_person_farbe2", "1F3C3-1F3FC"},
    {"laufende_person_farbe3", "1F3C3-1F3FD"},
    {"laufende_person_farbe4", "1F3C3-1F3FE"},
    {"laufende_person_farbe5", "1F3C3-1F3FF"},
    {"surferin", "1F3C4-200D-2640-FE0F"},
    {"surferin_farbe1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"surferin_farbe2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"surferin_farbe3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"surferin_farbe4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"surferin_farbe5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"sportmedaille", "1F3C5"},
    {"pokal", "1F3C6"},
    {"pferderennen", "1F3C7"},
    {"pferderennen_farbe1", "1F3C7-1F3FB"},
    {"pferderennen_farbe2", "1F3C7-1F3FC"},
    {"pferderennen_farbe3", "1F3C7-1F3FD"},
    {"pferderennen_farbe4", "1F3C7-1F3FE"},
    {"pferderennen_farbe5", "1F3C7-1F3FF"},
    {"football", "1F3C8"},
    {"rugbyball", "1F3C9"},
    {"schwimmerin", "1F3CA-200D-2640-FE0F"},
    {"schwimmerin_farbe1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"schwimmerin_farbe2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"schwimmerin_farbe3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"schwimmerin_farbe4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"schwimmerin_farbe5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"gewichtheberin", "1F3CB-FE0F-200D-2640-FE0F"},
    {"gewichtheberin_farbe1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"gewichtheberin_farbe2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"gewichtheberin_farbe3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"gewichtheberin_farbe4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"gewichtheberin_farbe5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"golferin", "1F3CC-FE0F-200D-2640-FE0F"},
    {"golferin_farbe1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"golferin_farbe2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"golferin_farbe3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"golferin_farbe4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"golferin_farbe5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"motorrad", "1F3CD"},
    {"rennauto", "1F3CE"},
    {"kricket", "1F3CF"},
    {"volleyball", "1F3D0"},
    {"feldhockey", "1F3D1"},
    {"eishockey", "1F3D2"},
    {"tischtennis", "1F3D3"},
    {"schneebedeckter_berg", "1F3D4"},
    {"camping", "1F3D5"},
    {"strand_mit_sonnenschirm", "1F3D6"},
    {"kran", "1F3D7"},
    {"wohnhauser", "1F3D8"},
    {"skyline", "1F3D9"},
    {"verfallenes_haus", "1F3DA"},
    {"antikes_gebaude", "1F3DB"},
    {"wuste", "1F3DC"},
    {"einsame_insel", "1F3DD"},
    {"nationalpark", "1F3DE"},
    {"stadion", "1F3DF"},
    {"haus", "1F3E0"},
    {"haus_mit_garten", "1F3E1"},
    {"burogebaude", "1F3E2"},
    {"japanisches_postgebaude", "1F3E3"},
    {"postgebaude", "1F3E4"},
    {"krankenhaus", "1F3E5"},
    {"bank", "1F3E6"},
    {"symbol_geldautomat", "1F3E7"},
    {"hotel", "1F3E8"},
    {"stundenhotel", "1F3E9"},
    {"minimarkt", "1F3EA"},
    {"schule", "1F3EB"},
    {"kaufhaus", "1F3EC"},
    {"fabrik", "1F3ED"},
    {"rote_papierlaterne", "1F3EE"},
    {"japanisches_schloss", "1F3EF"},
    {"schloss", "1F3F0"},
    {"weisse_flagge", "1F3F3"},
    {"schwarze_flagge", "1F3F4"},
    {"rosette", "1F3F5"},
    {"etikett", "1F3F7"},
    {"badminton", "1F3F8"},
    {"pfeil_und_bogen", "1F3F9"},
    {"amphore", "1F3FA"},
    {"helle_hautfarbe", "1F3FB"},
    {"mittelhelle_hautfarbe", "1F3FC"},
    {"mittlere_hautfarbe", "1F3FD"},
    {"mitteldunkle_hautfarbe", "1F3FE"},
    {"dunkle_hautfarbe", "1F3FF"},
    {"ratte", "1F400"},
    {"maus", "1F401"},
    {"ochse", "1F402"},
    {"wasserbuffel", "1F403"},
    {"kuh", "1F404"},
    {"tiger", "1F405"},
    {"leopard", "1F406"},
    {"hase", "1F407"},
    {"katze", "1F408"},
    {"drache", "1F409"},
    {"krokodil", "1F40A"},
    {"wal", "1F40B"},
    {"schnecke", "1F40C"},
    {"schlange", "1F40D"},
    {"pferd", "1F40E"},
    {"widder", "1F40F"},
    {"ziege", "1F410"},
    {"schaf", "1F411"},
    {"affe", "1F412"},
    {"hahn", "1F413"},
    {"huhn", "1F414"},
    {"hund", "1F415"},
    {"schwein", "1F416"},
    {"wildschwein", "1F417"},
    {"elefant", "1F418"},
    {"oktopus", "1F419"},
    {"schneckenhaus", "1F41A"},
    {"raupe", "1F41B"},
    {"ameise", "1F41C"},
    {"biene", "1F41D"},
    {"marienkafer", "1F41E"},
    {"fisch", "1F41F"},
    {"tropenfisch", "1F420"},
    {"kugelfisch", "1F421"},
    {"schildkrote", "1F422"},
    {"schlupfendes_kuken", "1F423"},
    {"kuken", "1F424"},
    {"kuken_von_vorne", "1F425"},
    {"vogel", "1F426"},
    {"pinguin", "1F427"},
    {"koala", "1F428"},
    {"pudel", "1F429"},
    {"dromedar", "1F42A"},
    {"kamel", "1F42B"},
    {"delfin", "1F42C"},
    {"mausegesicht", "1F42D"},
    {"kuhgesicht", "1F42E"},
    {"tigergesicht", "1F42F"},
    {"hasengesicht", "1F430"},
    {"katzengesicht", "1F431"},
    {"drachengesicht", "1F432"},
    {"blasender_wal", "1F433"},
    {"pferdegesicht", "1F434"},
    {"affengesicht", "1F435"},
    {"hundegesicht", "1F436"},
    {"schweinegesicht", "1F437"},
    {"frosch", "1F438"},
    {"hamster", "1F439"},
    {"wolf", "1F43A"},
    {"bar", "1F43B"},
    {"panda", "1F43C"},
    {"schweinerussel", "1F43D"},
    {"tatzenabdrucke", "1F43E"},
    {"streifenhornchen", "1F43F"},
    {"augen", "1F440"},
    {"auge", "1F441"},
    {"ohr", "1F442"},
    {"ohr_farbe1", "1F442-1F3FB"},
    {"ohr_farbe2", "1F442-1F3FC"},
    {"ohr_farbe3", "1F442-1F3FD"},
    {"ohr_farbe4", "1F442-1F3FE"},
    {"ohr_farbe5", "1F442-1F3FF"},
    {"nase", "1F443"},
    {"nase_farbe1", "1F443-1F3FB"},
    {"nase_farbe2", "1F443-1F3FC"},
    {"nase_farbe3", "1F443-1F3FD"},
    {"nase_farbe4", "1F443-1F3FE"},
    {"nase_farbe5", "1F443-1F3FF"},
    {"mund", "1F444"},
    {"zunge", "1F445"},
    {"nach_oben_weisender_zeigefinger_von_hinten", "1F446"},
    {"nach_oben_weisender_zeigefinger_von_hinten_farbe1", "1F446-1F3FB"},
    {"nach_oben_weisender_zeigefinger_von_hinten_farbe2", "1F446-1F3FC"},
    {"nach_oben_weisender_zeigefinger_von_hinten_farbe3", "1F446-1F3FD"},
    {"nach_oben_weisender_zeigefinger_von_hinten_farbe4", "1F446-1F3FE"},
    {"nach_oben_weisender_zeigefinger_von_hinten_farbe5", "1F446-1F3FF"},
    {"nach_unten_weisender_zeigefinger", "1F447"},
    {"nach_unten_weisender_zeigefinger_farbe1", "1F447-1F3FB"},
    {"nach_unten_weisender_zeigefinger_farbe2", "1F447-1F3FC"},
    {"nach_unten_weisender_zeigefinger_farbe3", "1F447-1F3FD"},
    {"nach_unten_weisender_zeigefinger_farbe4", "1F447-1F3FE"},
    {"nach_unten_weisender_zeigefinger_farbe5", "1F447-1F3FF"},
    {"nach_links_weisender_zeigefinger", "1F448"},
    {"nach_links_weisender_zeigefinger_farbe1", "1F448-1F3FB"},
    {"nach_links_weisender_zeigefinger_farbe2", "1F448-1F3FC"},
    {"nach_links_weisender_zeigefinger_farbe3", "1F448-1F3FD"},
    {"nach_links_weisender_zeigefinger_farbe4", "1F448-1F3FE"},
    {"nach_links_weisender_zeigefinger_farbe5", "1F448-1F3FF"},
    {"nach_rechts_weisender_zeigefinger", "1F449"},
    {"nach_rechts_weisender_zeigefinger_farbe1", "1F449-1F3FB"},
    {"nach_rechts_weisender_zeigefinger_farbe2", "1F449-1F3FC"},
    {"nach_rechts_weisender_zeigefinger_farbe3", "1F449-1F3FD"},
    {"nach_rechts_weisender_zeigefinger_farbe4", "1F449-1F3FE"},
    {"nach_rechts_weisender_zeigefinger_farbe5", "1F449-1F3FF"},
    {"geballte_faust", "1F44A"},
    {"geballte_faust_farbe1", "1F44A-1F3FB"},
    {"geballte_faust_farbe2", "1F44A-1F3FC"},
    {"geballte_faust_farbe3", "1F44A-1F3FD"},
    {"geballte_faust_farbe4", "1F44A-1F3FE"},
    {"geballte_faust_farbe5", "1F44A-1F3FF"},
    {"winkende_hand", "1F44B"},
    {"winkende_hand_farbe1", "1F44B-1F3FB"},
    {"winkende_hand_farbe2", "1F44B-1F3FC"},
    {"winkende_hand_farbe3", "1F44B-1F3FD"},
    {"winkende_hand_farbe4", "1F44B-1F3FE"},
    {"winkende_hand_farbe5", "1F44B-1F3FF"},
    {"ok_zeichen", "1F44C"},
    {"ok_zeichen_farbe1", "1F44C-1F3FB"},
    {"ok_zeichen_farbe2", "1F44C-1F3FC"},
    {"ok_zeichen_farbe3", "1F44C-1F3FD"},
    {"ok_zeichen_farbe4", "1F44C-1F3FE"},
    {"ok_zeichen_farbe5", "1F44C-1F3FF"},
    {"daumen_hoch", "1F44D"},
    {"daumen_hoch_farbe1", "1F44D-1F3FB"},
    {"daumen_hoch_farbe2", "1F44D-1F3FC"},
    {"daumen_hoch_farbe3", "1F44D-1F3FD"},
    {"daumen_hoch_farbe4", "1F44D-1F3FE"},
    {"daumen_hoch_farbe5", "1F44D-1F3FF"},
    {"daumen_runter", "1F44E"},
    {"daumen_runter_farbe1", "1F44E-1F3FB"},
    {"daumen_runter_farbe2", "1F44E-1F3FC"},
    {"daumen_runter_farbe3", "1F44E-1F3FD"},
    {"daumen_runter_farbe4", "1F44E-1F3FE"},
    {"daumen_runter_farbe5", "1F44E-1F3FF"},
    {"klatschende_hande", "1F44F"},
    {"klatschende_hande_farbe1", "1F44F-1F3FB"},
    {"klatschende_hande_farbe2", "1F44F-1F3FC"},
    {"klatschende_hande_farbe3", "1F44F-1F3FD"},
    {"klatschende_hande_farbe4", "1F44F-1F3FE"},
    {"klatschende_hande_farbe5", "1F44F-1F3FF"},
    {"offene_hande", "1F450"},
    {"offene_hande_farbe1", "1F450-1F3FB"},
    {"offene_hande_farbe2", "1F450-1F3FC"},
    {"offene_hande_farbe3", "1F450-1F3FD"},
    {"offene_hande_farbe4", "1F450-1F3FE"},
    {"offene_hande_farbe5", "1F450-1F3FF"},
    {"krone", "1F451"},
    {"damenhut", "1F452"},
    {"brille", "1F453"},
    {"hemd_mit_krawatte", "1F454"},
    {"t_shirt", "1F455"},
    {"jeans", "1F456"},
    {"kleid", "1F457"},
    {"kimono", "1F458"},
    {"bikini", "1F459"},
    {"bluse", "1F45A"},
    {"geldborse", "1F45B"},
    {"handtasche", "1F45C"},
    {"clutch", "1F45D"},
    {"herrenschuh", "1F45E"},
    {"sportschuh", "1F45F"},
    {"stockelschuh", "1F460"},
    {"damensandale", "1F461"},
    {"damenstiefel", "1F462"},
    {"fussabdrucke", "1F463"},
    {"silhouette_einer_buste", "1F464"},
    {"silhouette_mehrerer_busten", "1F465"},
    {"junge", "1F466"},
    {"junge_farbe1", "1F466-1F3FB"},
    {"junge_farbe2", "1F466-1F3FC"},
    {"junge_farbe3", "1F466-1F3FD"},
    {"junge_farbe4", "1F466-1F3FE"},
    {"junge_farbe5", "1F466-1F3FF"},
    {"madchen", "1F467"},
    {"madchen_farbe1", "1F467-1F3FB"},
    {"madchen_farbe2", "1F467-1F3FC"},
    {"madchen_farbe3", "1F467-1F3FD"},
    {"madchen_farbe4", "1F467-1F3FE"},
    {"madchen_farbe5", "1F467-1F3FF"},
    {"mann", "1F468"},
    {"mann_farbe1", "1F468-1F3FB"},
    {"mann_farbe2", "1F468-1F3FC"},
    {"mann_farbe3", "1F468-1F3FD"},
    {"mann_farbe4", "1F468-1F3FE"},
    {"mann_farbe5", "1F468-1F3FF"},
    {"frau", "1F469"},
    {"frau_farbe1", "1F469-1F3FB"},
    {"frau_farbe2", "1F469-1F3FC"},
    {"frau_farbe3", "1F469-1F3FD"},
    {"frau_farbe4", "1F469-1F3FE"},
    {"frau_farbe5", "1F469-1F3FF"},
    {"familie", "1F46A"},
    {"mann_und_frau_halten_hande", "1F46B"},
    {"mann_und_frau_halten_hande_farbe1", "1F46B-1F3FB"},
    {"mann_und_frau_halten_hande_farbe2", "1F46B-1F3FC"},
    {"mann_und_frau_halten_hande_farbe3", "1F46B-1F3FD"},
    {"mann_und_frau_halten_hande_farbe4", "1F46B-1F3FE"},
    {"mann_und_frau_halten_hande_farbe5", "1F46B-1F3FF"},
    {"mann_und_frau_halten_hande_farbe1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mann_und_frau_halten_hande_farbe1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mann_und_frau_halten_hande_farbe1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mann_und_frau_halten_hande_farbe1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mann_und_frau_halten_hande_farbe2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mann_und_frau_halten_hande_farbe2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mann_und_frau_halten_hande_farbe2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mann_und_frau_halten_hande_farbe2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mann_und_frau_halten_hande_farbe3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mann_und_frau_halten_hande_farbe3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mann_und_frau_halten_hande_farbe3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mann_und_frau_halten_hande_farbe3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mann_und_frau_halten_hande_farbe4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mann_und_frau_halten_hande_farbe4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mann_und_frau_halten_hande_farbe4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mann_und_frau_halten_hande_farbe4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mann_und_frau_halten_hande_farbe5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mann_und_frau_halten_hande_farbe5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mann_und_frau_halten_hande_farbe5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mann_und_frau_halten_hande_farbe5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"handchen_haltende_manner", "1F46C"},
    {"handchen_haltende_manner_farbe1", "1F46C-1F3FB"},
    {"handchen_haltende_manner_farbe2", "1F46C-1F3FC"},
    {"handchen_haltende_manner_farbe3", "1F46C-1F3FD"},
    {"handchen_haltende_manner_farbe4", "1F46C-1F3FE"},
    {"handchen_haltende_manner_farbe5", "1F46C-1F3FF"},
    {"handchen_haltende_manner_farbe1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"handchen_haltende_manner_farbe1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"handchen_haltende_manner_farbe1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"handchen_haltende_manner_farbe1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"handchen_haltende_manner_farbe2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"handchen_haltende_manner_farbe2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"handchen_haltende_manner_farbe2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"handchen_haltende_manner_farbe2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"handchen_haltende_manner_farbe3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"handchen_haltende_manner_farbe3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"handchen_haltende_manner_farbe3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"handchen_haltende_manner_farbe3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"handchen_haltende_manner_farbe4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"handchen_haltende_manner_farbe4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"handchen_haltende_manner_farbe4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"handchen_haltende_manner_farbe4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"handchen_haltende_manner_farbe5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"handchen_haltende_manner_farbe5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"handchen_haltende_manner_farbe5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"handchen_haltende_manner_farbe5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"handchen_haltende_frauen", "1F46D"},
    {"handchen_haltende_frauen_farbe1", "1F46D-1F3FB"},
    {"handchen_haltende_frauen_farbe2", "1F46D-1F3FC"},
    {"handchen_haltende_frauen_farbe3", "1F46D-1F3FD"},
    {"handchen_haltende_frauen_farbe4", "1F46D-1F3FE"},
    {"handchen_haltende_frauen_farbe5", "1F46D-1F3FF"},
    {"handchen_haltende_frauen_farbe1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"handchen_haltende_frauen_farbe1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"handchen_haltende_frauen_farbe1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"handchen_haltende_frauen_farbe1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"handchen_haltende_frauen_farbe2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"handchen_haltende_frauen_farbe2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"handchen_haltende_frauen_farbe2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"handchen_haltende_frauen_farbe2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"handchen_haltende_frauen_farbe3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"handchen_haltende_frauen_farbe3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"handchen_haltende_frauen_farbe3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"handchen_haltende_frauen_farbe3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"handchen_haltende_frauen_farbe4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"handchen_haltende_frauen_farbe4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"handchen_haltende_frauen_farbe4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"handchen_haltende_frauen_farbe4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"handchen_haltende_frauen_farbe5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"handchen_haltende_frauen_farbe5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"handchen_haltende_frauen_farbe5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"handchen_haltende_frauen_farbe5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"polizistin", "1F46E-200D-2640-FE0F"},
    {"polizistin_farbe1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"polizistin_farbe2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"polizistin_farbe3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"polizistin_farbe4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"polizistin_farbe5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"personen_mit_hasenohren", "1F46F"},
    {"personen_mit_hasenohren_farbe1", "1F46F-1F3FB"},
    {"personen_mit_hasenohren_farbe2", "1F46F-1F3FC"},
    {"personen_mit_hasenohren_farbe3", "1F46F-1F3FD"},
    {"personen_mit_hasenohren_farbe4", "1F46F-1F3FE"},
    {"personen_mit_hasenohren_farbe5", "1F46F-1F3FF"},
    {"personen_mit_hasenohren_farbe1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"personen_mit_hasenohren_farbe1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"personen_mit_hasenohren_farbe1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"personen_mit_hasenohren_farbe1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"personen_mit_hasenohren_farbe2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"personen_mit_hasenohren_farbe2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"personen_mit_hasenohren_farbe2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"personen_mit_hasenohren_farbe2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"personen_mit_hasenohren_farbe3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"personen_mit_hasenohren_farbe3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"personen_mit_hasenohren_farbe3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"personen_mit_hasenohren_farbe3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"personen_mit_hasenohren_farbe4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"personen_mit_hasenohren_farbe4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"personen_mit_hasenohren_farbe4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"personen_mit_hasenohren_farbe4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"personen_mit_hasenohren_farbe5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"personen_mit_hasenohren_farbe5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"personen_mit_hasenohren_farbe5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"personen_mit_hasenohren_farbe5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"person_mit_schleier", "1F470"},
    {"person_mit_schleier_farbe1", "1F470-1F3FB"},
    {"person_mit_schleier_farbe2", "1F470-1F3FC"},
    {"person_mit_schleier_farbe3", "1F470-1F3FD"},
    {"person_mit_schleier_farbe4", "1F470-1F3FE"},
    {"person_mit_schleier_farbe5", "1F470-1F3FF"},
    {"person_blondes_haar", "1F471"},
    {"person_blondes_haar_farbe1", "1F471-1F3FB"},
    {"person_blondes_haar_farbe2", "1F471-1F3FC"},
    {"person_blondes_haar_farbe3", "1F471-1F3FD"},
    {"person_blondes_haar_farbe4", "1F471-1F3FE"},
    {"person_blondes_haar_farbe5", "1F471-1F3FF"},
    {"mann_mit_chinesischem_hut", "1F472"},
    {"mann_mit_chinesischem_hut_farbe1", "1F472-1F3FB"},
    {"mann_mit_chinesischem_hut_farbe2", "1F472-1F3FC"},
    {"mann_mit_chinesischem_hut_farbe3", "1F472-1F3FD"},
    {"mann_mit_chinesischem_hut_farbe4", "1F472-1F3FE"},
    {"mann_mit_chinesischem_hut_farbe5", "1F472-1F3FF"},
    {"person_mit_turban", "1F473"},
    {"person_mit_turban_farbe1", "1F473-1F3FB"},
    {"person_mit_turban_farbe2", "1F473-1F3FC"},
    {"person_mit_turban_farbe3", "1F473-1F3FD"},
    {"person_mit_turban_farbe4", "1F473-1F3FE"},
    {"person_mit_turban_farbe5", "1F473-1F3FF"},
    {"alterer_mann", "1F474"},
    {"alterer_mann_farbe1", "1F474-1F3FB"},
    {"alterer_mann_farbe2", "1F474-1F3FC"},
    {"alterer_mann_farbe3", "1F474-1F3FD"},
    {"alterer_mann_farbe4", "1F474-1F3FE"},
    {"alterer_mann_farbe5", "1F474-1F3FF"},
    {"altere_frau", "1F475"},
    {"altere_frau_farbe1", "1F475-1F3FB"},
    {"altere_frau_farbe2", "1F475-1F3FC"},
    {"altere_frau_farbe3", "1F475-1F3FD"},
    {"altere_frau_farbe4", "1F475-1F3FE"},
    {"altere_frau_farbe5", "1F475-1F3FF"},
    {"baby", "1F476"},
    {"baby_farbe1", "1F476-1F3FB"},
    {"baby_farbe2", "1F476-1F3FC"},
    {"baby_farbe3", "1F476-1F3FD"},
    {"baby_farbe4", "1F476-1F3FE"},
    {"baby_farbe5", "1F476-1F3FF"},
    {"bauarbeiterin", "1F477-200D-2640-FE0F"},
    {"bauarbeiterin_farbe1", "1F477-1F3FB-200D-2640-FE0F"},
    {"bauarbeiterin_farbe2", "1F477-1F3FC-200D-2640-FE0F"},
    {"bauarbeiterin_farbe3", "1F477-1F3FD-200D-2640-FE0F"},
    {"bauarbeiterin_farbe4", "1F477-1F3FE-200D-2640-FE0F"},
    {"bauarbeiterin_farbe5", "1F477-1F3FF-200D-2640-FE0F"},
    {"prinzessin", "1F478"},
    {"prinzessin_farbe1", "1F478-1F3FB"},
    {"prinzessin_farbe2", "1F478-1F3FC"},
    {"prinzessin_farbe3", "1F478-1F3FD"},
    {"prinzessin_farbe4", "1F478-1F3FE"},
    {"prinzessin_farbe5", "1F478-1F3FF"},
    {"ungeheuer", "1F479"},
    {"kobold", "1F47A"},
    {"gespenst", "1F47B"},
    {"putte", "1F47C"},
    {"putte_farbe1", "1F47C-1F3FB"},
    {"putte_farbe2", "1F47C-1F3FC"},
    {"putte_farbe3", "1F47C-1F3FD"},
    {"putte_farbe4", "1F47C-1F3FE"},
    {"putte_farbe5", "1F47C-1F3FF"},
    {"ausserirdischer", "1F47D"},
    {"computerspiel_monster", "1F47E"},
    {"wutendes_gesicht_mit_hornern", "1F47F"},
    {"totenkopf", "1F480"},
    {"infoschalter_mitarbeiterin", "1F481-200D-2640-FE0F"},
    {"infoschalter_mitarbeiterin_farbe1", "1F481-1F3FB-200D-2640-FE0F"},
    {"infoschalter_mitarbeiterin_farbe2", "1F481-1F3FC-200D-2640-FE0F"},
    {"infoschalter_mitarbeiterin_farbe3", "1F481-1F3FD-200D-2640-FE0F"},
    {"infoschalter_mitarbeiterin_farbe4", "1F481-1F3FE-200D-2640-FE0F"},
    {"infoschalter_mitarbeiterin_farbe5", "1F481-1F3FF-200D-2640-FE0F"},
    {"wache", "1F482"},
    {"wache_farbe1", "1F482-1F3FB"},
    {"wache_farbe2", "1F482-1F3FC"},
    {"wache_farbe3", "1F482-1F3FD"},
    {"wache_farbe4", "1F482-1F3FE"},
    {"wache_farbe5", "1F482-1F3FF"},
    {"tanzende_frau", "1F483"},
    {"tanzende_frau_farbe1", "1F483-1F3FB"},
    {"tanzende_frau_farbe2", "1F483-1F3FC"},
    {"tanzende_frau_farbe3", "1F483-1F3FD"},
    {"tanzende_frau_farbe4", "1F483-1F3FE"},
    {"tanzende_frau_farbe5", "1F483-1F3FF"},
    {"lippenstift", "1F484"},
    {"nagellack", "1F485"},
    {"nagellack_farbe1", "1F485-1F3FB"},
    {"nagellack_farbe2", "1F485-1F3FC"},
    {"nagellack_farbe3", "1F485-1F3FD"},
    {"nagellack_farbe4", "1F485-1F3FE"},
    {"nagellack_farbe5", "1F485-1F3FF"},
    {"person_die_eine_kopfmassage_bekommt", "1F486"},
    {"person_die_eine_kopfmassage_bekommt_farbe1", "1F486-1F3FB"},
    {"person_die_eine_kopfmassage_bekommt_farbe2", "1F486-1F3FC"},
    {"person_die_eine_kopfmassage_bekommt_farbe3", "1F486-1F3FD"},
    {"person_die_eine_kopfmassage_bekommt_farbe4", "1F486-1F3FE"},
    {"person_die_eine_kopfmassage_bekommt_farbe5", "1F486-1F3FF"},
    {"person_beim_haareschneiden", "1F487"},
    {"person_beim_haareschneiden_farbe1", "1F487-1F3FB"},
    {"person_beim_haareschneiden_farbe2", "1F487-1F3FC"},
    {"person_beim_haareschneiden_farbe3", "1F487-1F3FD"},
    {"person_beim_haareschneiden_farbe4", "1F487-1F3FE"},
    {"person_beim_haareschneiden_farbe5", "1F487-1F3FF"},
    {"barbershop_saule", "1F488"},
    {"spritze", "1F489"},
    {"kapsel", "1F48A"},
    {"kussabdruck", "1F48B"},
    {"liebesbrief", "1F48C"},
    {"ring", "1F48D"},
    {"edelstein", "1F48E"},
    {"sich_kussendes_paar", "1F48F"},
    {"sich_kussendes_paar_farbe1", "1F48F-1F3FB"},
    {"sich_kussendes_paar_farbe2", "1F48F-1F3FC"},
    {"sich_kussendes_paar_farbe3", "1F48F-1F3FD"},
    {"sich_kussendes_paar_farbe4", "1F48F-1F3FE"},
    {"sich_kussendes_paar_farbe5", "1F48F-1F3FF"},
    {"sich_kussendes_paar_farbe1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"sich_kussendes_paar_farbe1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"sich_kussendes_paar_farbe1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"sich_kussendes_paar_farbe1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"sich_kussendes_paar_farbe2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"sich_kussendes_paar_farbe2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"sich_kussendes_paar_farbe2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"sich_kussendes_paar_farbe2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"sich_kussendes_paar_farbe3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"sich_kussendes_paar_farbe3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"sich_kussendes_paar_farbe3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"sich_kussendes_paar_farbe3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"sich_kussendes_paar_farbe4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"sich_kussendes_paar_farbe4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"sich_kussendes_paar_farbe4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"sich_kussendes_paar_farbe4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"sich_kussendes_paar_farbe5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"sich_kussendes_paar_farbe5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"sich_kussendes_paar_farbe5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"sich_kussendes_paar_farbe5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"blumenstrauss", "1F490"},
    {"liebespaar", "1F491"},
    {"liebespaar_farbe1", "1F491-1F3FB"},
    {"liebespaar_farbe2", "1F491-1F3FC"},
    {"liebespaar_farbe3", "1F491-1F3FD"},
    {"liebespaar_farbe4", "1F491-1F3FE"},
    {"liebespaar_farbe5", "1F491-1F3FF"},
    {"liebespaar_farbe1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"liebespaar_farbe1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"liebespaar_farbe1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"liebespaar_farbe1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"liebespaar_farbe2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"liebespaar_farbe2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"liebespaar_farbe2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"liebespaar_farbe2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"liebespaar_farbe3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"liebespaar_farbe3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"liebespaar_farbe3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"liebespaar_farbe3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"liebespaar_farbe4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"liebespaar_farbe4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"liebespaar_farbe4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"liebespaar_farbe4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"liebespaar_farbe5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"liebespaar_farbe5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"liebespaar_farbe5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"liebespaar_farbe5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"hochzeit", "1F492"},
    {"schlagendes_herz", "1F493"},
    {"gebrochenes_herz", "1F494"},
    {"zwei_herzen", "1F495"},
    {"funkelndes_herz", "1F496"},
    {"wachsendes_herz", "1F497"},
    {"herz_mit_pfeil", "1F498"},
    {"blaues_herz", "1F499"},
    {"grunes_herz", "1F49A"},
    {"gelbes_herz", "1F49B"},
    {"lila_herz", "1F49C"},
    {"herz_mit_schleife", "1F49D"},
    {"kreisende_herzen", "1F49E"},
    {"herzdekoration", "1F49F"},
    {"rautenform_mit_punkt", "1F4A0"},
    {"gluhbirne", "1F4A1"},
    {"arger", "1F4A2"},
    {"bombe", "1F4A3"},
    {"schlafen", "1F4A4"},
    {"zusammenstoss", "1F4A5"},
    {"schweisstropfen", "1F4A6"},
    {"tropfen", "1F4A7"},
    {"staubwolke", "1F4A8"},
    {"kothaufen", "1F4A9"},
    {"angespannter_bizeps", "1F4AA"},
    {"angespannter_bizeps_farbe1", "1F4AA-1F3FB"},
    {"angespannter_bizeps_farbe2", "1F4AA-1F3FC"},
    {"angespannter_bizeps_farbe3", "1F4AA-1F3FD"},
    {"angespannter_bizeps_farbe4", "1F4AA-1F3FE"},
    {"angespannter_bizeps_farbe5", "1F4AA-1F3FF"},
    {"schwindlig", "1F4AB"},
    {"sprechblase_mit_drei_punkten", "1F4AC"},
    {"gedankenblase", "1F4AD"},
    {"blumenstempel", "1F4AE"},
    {"100_punkte", "1F4AF"},
    {"geldsack", "1F4B0"},
    {"geldwechsel", "1F4B1"},
    {"dollarzeichen_extrafett", "1F4B2"},
    {"kreditkarte", "1F4B3"},
    {"yen_banknote", "1F4B4"},
    {"dollar_banknote", "1F4B5"},
    {"euro_banknote", "1F4B6"},
    {"pfund_banknote", "1F4B7"},
    {"geldschein_mit_flugeln", "1F4B8"},
    {"steigende_kurve_mit_yen_zeichen", "1F4B9"},
    {"sitzplatz", "1F4BA"},
    {"laptop", "1F4BB"},
    {"aktentasche", "1F4BC"},
    {"minidisc", "1F4BD"},
    {"diskette", "1F4BE"},
    {"cd", "1F4BF"},
    {"dvd", "1F4C0"},
    {"ordner", "1F4C1"},
    {"geoffneter_ordner", "1F4C2"},
    {"teilweise_eingerolltes_blatt", "1F4C3"},
    {"vorderseite_eines_blattes", "1F4C4"},
    {"kalender", "1F4C5"},
    {"abreisskalender", "1F4C6"},
    {"rotationskartei", "1F4C7"},
    {"aufwartstrend", "1F4C8"},
    {"abwartstrend", "1F4C9"},
    {"balkendiagramm", "1F4CA"},
    {"klemmbrett", "1F4CB"},
    {"reisszwecke", "1F4CC"},
    {"stecknadel", "1F4CD"},
    {"buroklammer", "1F4CE"},
    {"lineal", "1F4CF"},
    {"dreieckiges_lineal", "1F4D0"},
    {"pagemarker", "1F4D1"},
    {"spiralblock", "1F4D2"},
    {"notizbuch", "1F4D3"},
    {"notizbuch_mit_dekorativem_einband", "1F4D4"},
    {"geschlossenes_buch", "1F4D5"},
    {"offenes_buch", "1F4D6"},
    {"grunes_buch", "1F4D7"},
    {"blaues_buch", "1F4D8"},
    {"orangefarbenes_buch", "1F4D9"},
    {"bucherstapel", "1F4DA"},
    {"namensschild", "1F4DB"},
    {"schriftrolle", "1F4DC"},
    {"papier_und_bleistift", "1F4DD"},
    {"telefonhorer", "1F4DE"},
    {"pager", "1F4DF"},
    {"faxgerat", "1F4E0"},
    {"satellitenschussel", "1F4E1"},
    {"lautsprecher", "1F4E2"},
    {"megafon", "1F4E3"},
    {"postausgang", "1F4E4"},
    {"posteingang", "1F4E5"},
    {"paket", "1F4E6"},
    {"email", "1F4E7"},
    {"eingehender_briefumschlag", "1F4E8"},
    {"umschlag_mit_pfeil", "1F4E9"},
    {"geschlossener_briefkasten_ohne_post", "1F4EA"},
    {"geschlossener_briefkasten_mit_post", "1F4EB"},
    {"offener_briefkasten_mit_post", "1F4EC"},
    {"offener_briefkasten_ohne_post", "1F4ED"},
    {"briefkasten", "1F4EE"},
    {"posthorn", "1F4EF"},
    {"zeitung", "1F4F0"},
    {"mobiltelefon", "1F4F1"},
    {"mobiltelefon_mit_pfeil", "1F4F2"},
    {"vibrationsmodus", "1F4F3"},
    {"mobiltelefon_aus", "1F4F4"},
    {"mobiltelefone_verboten", "1F4F5"},
    {"balkenformige_signalstarkenanzeige", "1F4F6"},
    {"fotoapparat", "1F4F7"},
    {"fotoapparat_mit_blitz", "1F4F8"},
    {"videokamera", "1F4F9"},
    {"fernseher", "1F4FA"},
    {"radio", "1F4FB"},
    {"videokassette", "1F4FC"},
    {"filmprojektor", "1F4FD"},
    {"gebetskette", "1F4FF"},
    {"zufallsmodus", "1F500"},
    {"wiederholen", "1F501"},
    {"titel_wiederholen", "1F502"},
    {"kreisformige_pfeile_im_uhrzeigersinn", "1F503"},
    {"pfeile_gegen_den_uhrzeigersinn", "1F504"},
    {"taste_dimmen", "1F505"},
    {"heller_taste", "1F506"},
    {"durchgestrichener_lautsprecher", "1F507"},
    {"lautsprecher_mit_geringer_lautstarke", "1F508"},
    {"lautsprecher_mit_mittlerer_lautstarke", "1F509"},
    {"lautsprecher_mit_hoher_lautstarke", "1F50A"},
    {"batterie", "1F50B"},
    {"netzstecker", "1F50C"},
    {"lupe_nach_links", "1F50D"},
    {"lupe_nach_rechts", "1F50E"},
    {"schloss_mit_fuller", "1F50F"},
    {"schloss_mit_schlussel", "1F510"},
    {"schlussel", "1F511"},
    {"geschlossenes_schloss", "1F512"},
    {"offenes_schloss", "1F513"},
    {"glocke", "1F514"},
    {"durchgestrichene_glocke", "1F515"},
    {"lesezeichen", "1F516"},
    {"linksymbol", "1F517"},
    {"optionsfeld", "1F518"},
    {"back_pfeil", "1F519"},
    {"end_pfeil", "1F51A"},
    {"on_pfeil", "1F51B"},
    {"soon_pfeil", "1F51C"},
    {"top_pfeil", "1F51D"},
    {"minderjahrige_verboten", "1F51E"},
    {"taste_10", "1F51F"},
    {"eingabesymbol_lateinische_grossbuchstaben", "1F520"},
    {"eingabesymbol_lateinische_kleinbuchstaben", "1F521"},
    {"eingabesymbol_zahlen", "1F522"},
    {"eingabesymbol_sonderzeichen", "1F523"},
    {"eingabesymbol_lateinische_buchstaben", "1F524"},
    {"feuer", "1F525"},
    {"taschenlampe", "1F526"},
    {"schraubenschlussel", "1F527"},
    {"hammer", "1F528"},
    {"mutter_und_schraube", "1F529"},
    {"kuchenmesser", "1F52A"},
    {"wasserpistole", "1F52B"},
    {"mikroskop", "1F52C"},
    {"teleskop", "1F52D"},
    {"kristallkugel", "1F52E"},
    {"hexagramm_mit_punkt", "1F52F"},
    {"japanisches_anfanger_zeichen", "1F530"},
    {"dreizack", "1F531"},
    {"schwarze_quadratische_schaltflache", "1F532"},
    {"weisse_quadratische_schaltflache", "1F533"},
    {"roter_punkt", "1F534"},
    {"blauer_punkt", "1F535"},
    {"grosse_orangefarbene_raute", "1F536"},
    {"grosse_blaue_raute", "1F537"},
    {"kleine_orangefarbene_raute", "1F538"},
    {"kleine_blaue_raute", "1F539"},
    {"rotes_dreieck_mit_der_spitze_nach_oben", "1F53A"},
    {"rotes_dreieck_mit_der_spitze_nach_unten", "1F53B"},
    {"aufwarts_schaltflache", "1F53C"},
    {"abwarts_schaltflache", "1F53D"},
    {"om", "1F549"},
    {"taube", "1F54A"},
    {"kaaba", "1F54B"},
    {"moschee", "1F54C"},
    {"synagoge", "1F54D"},
    {"menora", "1F54E"},
    {"100_uhr", "1F550"},
    {"200_uhr", "1F551"},
    {"300_uhr", "1F552"},
    {"400_uhr", "1F553"},
    {"500_uhr", "1F554"},
    {"600_uhr", "1F555"},
    {"700_uhr", "1F556"},
    {"800_uhr", "1F557"},
    {"900_uhr", "1F558"},
    {"1000_uhr", "1F559"},
    {"1100_uhr", "1F55A"},
    {"ziffernblatt_1200_uhr", "1F55B"},
    {"130_uhr", "1F55C"},
    {"230_uhr", "1F55D"},
    {"330_uhr", "1F55E"},
    {"430_uhr", "1F55F"},
    {"530_uhr", "1F560"},
    {"630_uhr", "1F561"},
    {"730_uhr", "1F562"},
    {"830_uhr", "1F563"},
    {"930_uhr", "1F564"},
    {"1030_uhr", "1F565"},
    {"1130_uhr", "1F566"},
    {"1230_uhr", "1F567"},
    {"kerze", "1F56F"},
    {"kaminuhr", "1F570"},
    {"loch", "1F573"},
    {"schwebender_mann_im_anzug", "1F574"},
    {"schwebender_mann_im_anzug_farbe1", "1F574-1F3FB"},
    {"schwebender_mann_im_anzug_farbe2", "1F574-1F3FC"},
    {"schwebender_mann_im_anzug_farbe3", "1F574-1F3FD"},
    {"schwebender_mann_im_anzug_farbe4", "1F574-1F3FE"},
    {"schwebender_mann_im_anzug_farbe5", "1F574-1F3FF"},
    {"detektivin", "1F575-FE0F-200D-2640-FE0F"},
    {"detektivin_farbe1", "1F575-1F3FB-200D-2640-FE0F"},
    {"detektivin_farbe2", "1F575-1F3FC-200D-2640-FE0F"},
    {"detektivin_farbe3", "1F575-1F3FD-200D-2640-FE0F"},
    {"detektivin_farbe4", "1F575-1F3FE-200D-2640-FE0F"},
    {"detektivin_farbe5", "1F575-1F3FF-200D-2640-FE0F"},
    {"sonnenbrille", "1F576"},
    {"spinne", "1F577"},
    {"spinnennetz", "1F578"},
    {"joystick", "1F579"},
    {"tanzender_mann", "1F57A"},
    {"tanzender_mann_farbe1", "1F57A-1F3FB"},
    {"tanzender_mann_farbe2", "1F57A-1F3FC"},
    {"tanzender_mann_farbe3", "1F57A-1F3FD"},
    {"tanzender_mann_farbe4", "1F57A-1F3FE"},
    {"tanzender_mann_farbe5", "1F57A-1F3FF"},
    {"verhakte_buroklammern", "1F587"},
    {"kugelschreiber", "1F58A"},
    {"fullhalter", "1F58B"},
    {"pinsel", "1F58C"},
    {"wachsmalstift", "1F58D"},
    {"hand_mit_gespreizten_fingern", "1F590"},
    {"hand_mit_gespreizten_fingern_farbe1", "1F590-1F3FB"},
    {"hand_mit_gespreizten_fingern_farbe2", "1F590-1F3FC"},
    {"hand_mit_gespreizten_fingern_farbe3", "1F590-1F3FD"},
    {"hand_mit_gespreizten_fingern_farbe4", "1F590-1F3FE"},
    {"hand_mit_gespreizten_fingern_farbe5", "1F590-1F3FF"},
    {"mittelfinger", "1F595"},
    {"mittelfinger_farbe1", "1F595-1F3FB"},
    {"mittelfinger_farbe2", "1F595-1F3FC"},
    {"mittelfinger_farbe3", "1F595-1F3FD"},
    {"mittelfinger_farbe4", "1F595-1F3FE"},
    {"mittelfinger_farbe5", "1F595-1F3FF"},
    {"vulkanischer_gruss", "1F596"},
    {"vulkanischer_gruss_farbe1", "1F596-1F3FB"},
    {"vulkanischer_gruss_farbe2", "1F596-1F3FC"},
    {"vulkanischer_gruss_farbe3", "1F596-1F3FD"},
    {"vulkanischer_gruss_farbe4", "1F596-1F3FE"},
    {"vulkanischer_gruss_farbe5", "1F596-1F3FF"},
    {"schwarzes_herz", "1F5A4"},
    {"desktopcomputer", "1F5A5"},
    {"drucker", "1F5A8"},
    {"computermaus", "1F5B1"},
    {"trackball", "1F5B2"},
    {"gerahmtes_bild", "1F5BC"},
    {"karteireiter", "1F5C2"},
    {"karteikasten", "1F5C3"},
    {"aktenschrank", "1F5C4"},
    {"papierkorb", "1F5D1"},
    {"notizblock", "1F5D2"},
    {"spiralkalender", "1F5D3"},
    {"schraubzwinge", "1F5DC"},
    {"alter_schlussel", "1F5DD"},
    {"zusammengerollte_zeitung", "1F5DE"},
    {"dolch", "1F5E1"},
    {"sprechender_kopf", "1F5E3"},
    {"sprechblase_links", "1F5E8"},
    {"sprechblase_fur_wutende_aussage_rechts", "1F5EF"},
    {"urne_mit_wahlzettel", "1F5F3"},
    {"weltkarte", "1F5FA"},
    {"fuji", "1F5FB"},
    {"tokyo_tower", "1F5FC"},
    {"freiheitsstatue", "1F5FD"},
    {"umriss_von_japan", "1F5FE"},
    {"statue", "1F5FF"},
    {"grinsendes_gesicht", "1F600"},
    {"strahlendes_gesicht_mit_lachenden_augen", "1F601"},
    {"gesicht_mit_freudentranen", "1F602"},
    {"grinsendes_gesicht_mit_grossen_augen", "1F603"},
    {"grinsendes_gesicht_mit_lachenden_augen", "1F604"},
    {"grinsendes_gesicht_mit_schweisstropfen", "1F605"},
    {"grinsegesicht_mit_zugekniffenen_augen", "1F606"},
    {"lachelndes_gesicht_mit_heiligenschein", "1F607"},
    {"grinsendes_gesicht_mit_hornern", "1F608"},
    {"zwinkerndes_gesicht", "1F609"},
    {"lachelndes_gesicht_mit_lachenden_augen", "1F60A"},
    {"sich_die_lippen_leckendes_gesicht", "1F60B"},
    {"erleichtertes_gesicht", "1F60C"},
    {"lachelndes_gesicht_mit_herzformigen_augen", "1F60D"},
    {"lachelndes_gesicht_mit_sonnenbrille", "1F60E"},
    {"suffisant_lachelndes_gesicht", "1F60F"},
    {"neutrales_gesicht", "1F610"},
    {"ausdrucksloses_gesicht", "1F611"},
    {"verstimmtes_gesicht", "1F612"},
    {"bedrucktes_gesicht_mit_schweiss", "1F613"},
    {"nachdenkliches_gesicht", "1F614"},
    {"verwundertes_gesicht", "1F615"},
    {"verwirrtes_gesicht", "1F616"},
    {"kussendes_gesicht", "1F617"},
    {"kuss_zuwerfendes_gesicht", "1F618"},
    {"kussendes_gesicht_mit_lachelnden_augen", "1F619"},
    {"kussendes_gesicht_mit_geschlossenen_augen", "1F61A"},
    {"gesicht_mit_herausgestreckter_zunge", "1F61B"},
    {"zwinkerndes_gesicht_mit_herausgestreckter_zunge", "1F61C"},
    {"gesicht_mit_herausgestreckter_zunge_und_zusammengekniffenen_augen", "1F61D"},
    {"enttauschtes_gesicht", "1F61E"},
    {"besorgtes_gesicht", "1F61F"},
    {"verargertes_gesicht", "1F620"},
    {"schmollendes_gesicht", "1F621"},
    {"weinendes_gesicht", "1F622"},
    {"entschlossenes_gesicht", "1F623"},
    {"schnaubendes_gesicht", "1F624"},
    {"trauriges_aber_erleichtertes_gesicht", "1F625"},
    {"entsetztes_gesicht", "1F626"},
    {"qualvolles_gesicht", "1F627"},
    {"angstliches_gesicht", "1F628"},
    {"erschopftes_gesicht", "1F629"},
    {"schlafriges_gesicht", "1F62A"},
    {"mudes_gesicht", "1F62B"},
    {"grimassen_schneidendes_gesicht", "1F62C"},
    {"heulendes_gesicht", "1F62D"},
    {"gesicht_mit_offenem_mund", "1F62E"},
    {"verdutztes_gesicht", "1F62F"},
    {"besorgtes_gesicht_mit_schweisstropfen", "1F630"},
    {"vor_angst_schreiendes_gesicht", "1F631"},
    {"erstauntes_gesicht", "1F632"},
    {"errotetes_gesicht_mit_grossen_augen", "1F633"},
    {"schlafendes_gesicht", "1F634"},
    {"benommenes_gesicht", "1F635"},
    {"gesicht_ohne_mund", "1F636"},
    {"gesicht_mit_atemschutzmaske", "1F637"},
    {"grinsende_katze_mit_lachenden_augen", "1F638"},
    {"katze_mit_freudentranen", "1F639"},
    {"grinsende_katze", "1F63A"},
    {"lachende_katze_mit_herzen_als_augen", "1F63B"},
    {"verwegen_lachelnde_katze", "1F63C"},
    {"kussende_katze", "1F63D"},
    {"schmollende_katze", "1F63E"},
    {"weinende_katze", "1F63F"},
    {"erschopfte_katze", "1F640"},
    {"betrubtes_gesicht", "1F641"},
    {"leicht_lachelndes_gesicht", "1F642"},
    {"umgekehrtes_gesicht", "1F643"},
    {"augen_verdrehendes_gesicht", "1F644"},
    {"person_mit_uberkreuzten_armen", "1F645"},
    {"person_mit_uberkreuzten_armen_farbe1", "1F645-1F3FB"},
    {"person_mit_uberkreuzten_armen_farbe2", "1F645-1F3FC"},
    {"person_mit_uberkreuzten_armen_farbe3", "1F645-1F3FD"},
    {"person_mit_uberkreuzten_armen_farbe4", "1F645-1F3FE"},
    {"person_mit_uberkreuzten_armen_farbe5", "1F645-1F3FF"},
    {"person_mit_handen_auf_dem_kopf", "1F646"},
    {"person_mit_handen_auf_dem_kopf_farbe1", "1F646-1F3FB"},
    {"person_mit_handen_auf_dem_kopf_farbe2", "1F646-1F3FC"},
    {"person_mit_handen_auf_dem_kopf_farbe3", "1F646-1F3FD"},
    {"person_mit_handen_auf_dem_kopf_farbe4", "1F646-1F3FE"},
    {"person_mit_handen_auf_dem_kopf_farbe5", "1F646-1F3FF"},
    {"sich_verbeugende_person", "1F647"},
    {"sich_verbeugende_person_farbe1", "1F647-1F3FB"},
    {"sich_verbeugende_person_farbe2", "1F647-1F3FC"},
    {"sich_verbeugende_person_farbe3", "1F647-1F3FD"},
    {"sich_verbeugende_person_farbe4", "1F647-1F3FE"},
    {"sich_verbeugende_person_farbe5", "1F647-1F3FF"},
    {"sich_die_augen_zuhaltendes_affengesicht", "1F648"},
    {"sich_die_ohren_zuhaltendes_affengesicht", "1F649"},
    {"sich_den_mund_zuhaltendes_affengesicht", "1F64A"},
    {"person_mit_erhobenem_arm", "1F64B"},
    {"person_mit_erhobenem_arm_farbe1", "1F64B-1F3FB"},
    {"person_mit_erhobenem_arm_farbe2", "1F64B-1F3FC"},
    {"person_mit_erhobenem_arm_farbe3", "1F64B-1F3FD"},
    {"person_mit_erhobenem_arm_farbe4", "1F64B-1F3FE"},
    {"person_mit_erhobenem_arm_farbe5", "1F64B-1F3FF"},
    {"zwei_erhobene_handflachen", "1F64C"},
    {"zwei_erhobene_handflachen_farbe1", "1F64C-1F3FB"},
    {"zwei_erhobene_handflachen_farbe2", "1F64C-1F3FC"},
    {"zwei_erhobene_handflachen_farbe3", "1F64C-1F3FD"},
    {"zwei_erhobene_handflachen_farbe4", "1F64C-1F3FE"},
    {"zwei_erhobene_handflachen_farbe5", "1F64C-1F3FF"},
    {"missmutige_person", "1F64D"},
    {"missmutige_person_farbe1", "1F64D-1F3FB"},
    {"missmutige_person_farbe2", "1F64D-1F3FC"},
    {"missmutige_person_farbe3", "1F64D-1F3FD"},
    {"missmutige_person_farbe4", "1F64D-1F3FE"},
    {"missmutige_person_farbe5", "1F64D-1F3FF"},
    {"schmollende_person", "1F64E"},
    {"schmollende_person_farbe1", "1F64E-1F3FB"},
    {"schmollende_person_farbe2", "1F64E-1F3FC"},
    {"schmollende_person_farbe3", "1F64E-1F3FD"},
    {"schmollende_person_farbe4", "1F64E-1F3FE"},
    {"schmollende_person_farbe5", "1F64E-1F3FF"},
    {"zusammengelegte_handflachen", "1F64F"},
    {"zusammengelegte_handflachen_farbe1", "1F64F-1F3FB"},
    {"zusammengelegte_handflachen_farbe2", "1F64F-1F3FC"},
    {"zusammengelegte_handflachen_farbe3", "1F64F-1F3FD"},
    {"zusammengelegte_handflachen_farbe4", "1F64F-1F3FE"},
    {"zusammengelegte_handflachen_farbe5", "1F64F-1F3FF"},
    {"rakete", "1F680"},
    {"hubschrauber", "1F681"},
    {"dampflokomotive", "1F682"},
    {"eisenbahnwagen", "1F683"},
    {"hochgeschwindigkeitszug_mit_spitzer_nase", "1F684"},
    {"hochgeschwindigkeitszug", "1F685"},
    {"zug", "1F686"},
    {"u_bahn", "1F687"},
    {"s_bahn", "1F688"},
    {"bahnhof", "1F689"},
    {"strassenbahn", "1F68A"},
    {"strassenbahnwagen", "1F68B"},
    {"bus", "1F68C"},
    {"bus_von_vorne", "1F68D"},
    {"oberleitungsbus", "1F68E"},
    {"bushaltestelle", "1F68F"},
    {"kleinbus", "1F690"},
    {"krankenwagen", "1F691"},
    {"feuerwehrauto", "1F692"},
    {"polizeiwagen", "1F693"},
    {"polizeiwagen_von_vorne", "1F694"},
    {"taxi", "1F695"},
    {"taxi_von_vorne", "1F696"},
    {"auto", "1F697"},
    {"auto_von_vorne", "1F698"},
    {"wohnmobil", "1F699"},
    {"lieferwagen", "1F69A"},
    {"sattelzug", "1F69B"},
    {"traktor", "1F69C"},
    {"einschienenbahn", "1F69D"},
    {"bergbahn", "1F69E"},
    {"schwebebahn", "1F69F"},
    {"bergschwebebahn", "1F6A0"},
    {"bergseilbahn", "1F6A1"},
    {"schiff", "1F6A2"},
    {"person_im_ruderboot", "1F6A3"},
    {"person_im_ruderboot_farbe1", "1F6A3-1F3FB"},
    {"person_im_ruderboot_farbe2", "1F6A3-1F3FC"},
    {"person_im_ruderboot_farbe3", "1F6A3-1F3FD"},
    {"person_im_ruderboot_farbe4", "1F6A3-1F3FE"},
    {"person_im_ruderboot_farbe5", "1F6A3-1F3FF"},
    {"schnellboot", "1F6A4"},
    {"horizontale_verkehrsampel", "1F6A5"},
    {"vertikale_verkehrsampel", "1F6A6"},
    {"baustellenabsperrung", "1F6A7"},
    {"polizeilicht", "1F6A8"},
    {"wimpel", "1F6A9"},
    {"tur", "1F6AA"},
    {"verboten", "1F6AB"},
    {"zigarette", "1F6AC"},
    {"rauchverbot", "1F6AD"},
    {"symbol_papierkorb", "1F6AE"},
    {"abfall_verboten", "1F6AF"},
    {"trinkwasser", "1F6B0"},
    {"kein_trinkwasser", "1F6B1"},
    {"fahrrad", "1F6B2"},
    {"fahrrader_verboten", "1F6B3"},
    {"radfahrerin", "1F6B4-200D-2640-FE0F"},
    {"radfahrerin_farbe1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"radfahrerin_farbe2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"radfahrerin_farbe3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"radfahrerin_farbe4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"radfahrerin_farbe5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"mountainbikerin", "1F6B5-200D-2640-FE0F"},
    {"mountainbikerin_farbe1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"mountainbikerin_farbe2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"mountainbikerin_farbe3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"mountainbikerin_farbe4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"mountainbikerin_farbe5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"fussgangerin", "1F6B6-200D-2640-FE0F"},
    {"fussgangerin_farbe1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"fussgangerin_farbe2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"fussgangerin_farbe3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"fussgangerin_farbe4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"fussgangerin_farbe5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"fussganger_verboten", "1F6B7"},
    {"kinder_uberqueren_die_strasse", "1F6B8"},
    {"herren", "1F6B9"},
    {"damen", "1F6BA"},
    {"toiletten", "1F6BB"},
    {"symbol_baby", "1F6BC"},
    {"toilette", "1F6BD"},
    {"wc", "1F6BE"},
    {"dusche", "1F6BF"},
    {"badende_person", "1F6C0"},
    {"badende_person_farbe1", "1F6C0-1F3FB"},
    {"badende_person_farbe2", "1F6C0-1F3FC"},
    {"badende_person_farbe3", "1F6C0-1F3FD"},
    {"badende_person_farbe4", "1F6C0-1F3FE"},
    {"badende_person_farbe5", "1F6C0-1F3FF"},
    {"badewanne", "1F6C1"},
    {"passkontrolle", "1F6C2"},
    {"zollkontrolle", "1F6C3"},
    {"gepackausgabe", "1F6C4"},
    {"gepackaufbewahrung", "1F6C5"},
    {"sofa_und_lampe", "1F6CB"},
    {"im_bett_liegende_person", "1F6CC"},
    {"im_bett_liegende_person_farbe1", "1F6CC-1F3FB"},
    {"im_bett_liegende_person_farbe2", "1F6CC-1F3FC"},
    {"im_bett_liegende_person_farbe3", "1F6CC-1F3FD"},
    {"im_bett_liegende_person_farbe4", "1F6CC-1F3FE"},
    {"im_bett_liegende_person_farbe5", "1F6CC-1F3FF"},
    {"einkaufstuten", "1F6CD"},
    {"rezeptionsklingel", "1F6CE"},
    {"bett", "1F6CF"},
    {"religiose_statte", "1F6D0"},
    {"stoppschild", "1F6D1"},
    {"einkaufswagen", "1F6D2"},
    {"hindutempel", "1F6D5"},
    {"hutte", "1F6D6"},
    {"fahrstuhl", "1F6D7"},
    {"erdrutsch", "1F6D8"},
    {"wlan", "1F6DC"},
    {"spielplatzrutsche", "1F6DD"},
    {"autorad", "1F6DE"},
    {"rettungsring", "1F6DF"},
    {"hammer_und_schraubenschlussel", "1F6E0"},
    {"schutzschild", "1F6E1"},
    {"olfass", "1F6E2"},
    {"autobahn", "1F6E3"},
    {"bahngleis", "1F6E4"},
    {"motorboot", "1F6E5"},
    {"kleines_flugzeug", "1F6E9"},
    {"abflug", "1F6EB"},
    {"landung_eines_flugzeugs", "1F6EC"},
    {"satellit", "1F6F0"},
    {"passagierschiff", "1F6F3"},
    {"tretroller", "1F6F4"},
    {"motorroller", "1F6F5"},
    {"kanu", "1F6F6"},
    {"schlitten", "1F6F7"},
    {"fliegende_untertasse", "1F6F8"},
    {"skateboard", "1F6F9"},
    {"autorikscha", "1F6FA"},
    {"pick_up", "1F6FB"},
    {"rollschuh", "1F6FC"},
    {"oranger_punkt", "1F7E0"},
    {"gelber_punkt", "1F7E1"},
    {"gruner_punkt", "1F7E2"},
    {"lila_punkt", "1F7E3"},
    {"brauner_punkt", "1F7E4"},
    {"rotes_quadrat", "1F7E5"},
    {"blaues_quadrat", "1F7E6"},
    {"oranges_quadrat", "1F7E7"},
    {"gelbes_quadrat", "1F7E8"},
    {"grunes_quadrat", "1F7E9"},
    {"lila_quadrat", "1F7EA"},
    {"braunes_quadrat", "1F7EB"},
    {"gleichheitszeichen_extrafett", "1F7F0"},
    {"zusammengedruckte_finger", "1F90C"},
    {"zusammengedruckte_finger_farbe1", "1F90C-1F3FB"},
    {"zusammengedruckte_finger_farbe2", "1F90C-1F3FC"},
    {"zusammengedruckte_finger_farbe3", "1F90C-1F3FD"},
    {"zusammengedruckte_finger_farbe4", "1F90C-1F3FE"},
    {"zusammengedruckte_finger_farbe5", "1F90C-1F3FF"},
    {"weisses_herz", "1F90D"},
    {"braunes_herz", "1F90E"},
    {"wenig_geste", "1F90F"},
    {"wenig_geste_farbe1", "1F90F-1F3FB"},
    {"wenig_geste_farbe2", "1F90F-1F3FC"},
    {"wenig_geste_farbe3", "1F90F-1F3FD"},
    {"wenig_geste_farbe4", "1F90F-1F3FE"},
    {"wenig_geste_farbe5", "1F90F-1F3FF"},
    {"gesicht_mit_reissverschlussmund", "1F910"},
    {"gesicht_mit_dollarzeichen", "1F911"},
    {"gesicht_mit_fieberthermometer", "1F912"},
    {"strebergesicht", "1F913"},
    {"nachdenkendes_gesicht", "1F914"},
    {"gesicht_mit_kopfverband", "1F915"},
    {"roboter", "1F916"},
    {"gesicht_mit_umarmenden_handen", "1F917"},
    {"teufelsgruss", "1F918"},
    {"teufelsgruss_farbe1", "1F918-1F3FB"},
    {"teufelsgruss_farbe2", "1F918-1F3FC"},
    {"teufelsgruss_farbe3", "1F918-1F3FD"},
    {"teufelsgruss_farbe4", "1F918-1F3FE"},
    {"teufelsgruss_farbe5", "1F918-1F3FF"},
    {"ruf_mich_an_handzeichen", "1F919"},
    {"ruf_mich_an_handzeichen_farbe1", "1F919-1F3FB"},
    {"ruf_mich_an_handzeichen_farbe2", "1F919-1F3FC"},
    {"ruf_mich_an_handzeichen_farbe3", "1F919-1F3FD"},
    {"ruf_mich_an_handzeichen_farbe4", "1F919-1F3FE"},
    {"ruf_mich_an_handzeichen_farbe5", "1F919-1F3FF"},
    {"erhobene_hand_von_hinten", "1F91A"},
    {"erhobene_hand_von_hinten_farbe1", "1F91A-1F3FB"},
    {"erhobene_hand_von_hinten_farbe2", "1F91A-1F3FC"},
    {"erhobene_hand_von_hinten_farbe3", "1F91A-1F3FD"},
    {"erhobene_hand_von_hinten_farbe4", "1F91A-1F3FE"},
    {"erhobene_hand_von_hinten_farbe5", "1F91A-1F3FF"},
    {"faust_nach_links", "1F91B"},
    {"faust_nach_links_farbe1", "1F91B-1F3FB"},
    {"faust_nach_links_farbe2", "1F91B-1F3FC"},
    {"faust_nach_links_farbe3", "1F91B-1F3FD"},
    {"faust_nach_links_farbe4", "1F91B-1F3FE"},
    {"faust_nach_links_farbe5", "1F91B-1F3FF"},
    {"faust_nach_rechts", "1F91C"},
    {"faust_nach_rechts_farbe1", "1F91C-1F3FB"},
    {"faust_nach_rechts_farbe2", "1F91C-1F3FC"},
    {"faust_nach_rechts_farbe3", "1F91C-1F3FD"},
    {"faust_nach_rechts_farbe4", "1F91C-1F3FE"},
    {"faust_nach_rechts_farbe5", "1F91C-1F3FF"},
    {"handschlag", "1F91D"},
    {"handschlag_farbe1", "1F91D-1F3FB"},
    {"handschlag_farbe2", "1F91D-1F3FC"},
    {"handschlag_farbe3", "1F91D-1F3FD"},
    {"handschlag_farbe4", "1F91D-1F3FE"},
    {"handschlag_farbe5", "1F91D-1F3FF"},
    {"handschlag_farbe1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"handschlag_farbe1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"handschlag_farbe1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"handschlag_farbe1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"handschlag_farbe2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"handschlag_farbe2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"handschlag_farbe2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"handschlag_farbe2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"handschlag_farbe3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"handschlag_farbe3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"handschlag_farbe3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"handschlag_farbe3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"handschlag_farbe4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"handschlag_farbe4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"handschlag_farbe4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"handschlag_farbe4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"handschlag_farbe5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"handschlag_farbe5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"handschlag_farbe5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"handschlag_farbe5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"hand_mit_gekreuzten_fingern", "1F91E"},
    {"hand_mit_gekreuzten_fingern_farbe1", "1F91E-1F3FB"},
    {"hand_mit_gekreuzten_fingern_farbe2", "1F91E-1F3FC"},
    {"hand_mit_gekreuzten_fingern_farbe3", "1F91E-1F3FD"},
    {"hand_mit_gekreuzten_fingern_farbe4", "1F91E-1F3FE"},
    {"hand_mit_gekreuzten_fingern_farbe5", "1F91E-1F3FF"},
    {"ich_liebe_dich_geste", "1F91F"},
    {"ich_liebe_dich_geste_farbe1", "1F91F-1F3FB"},
    {"ich_liebe_dich_geste_farbe2", "1F91F-1F3FC"},
    {"ich_liebe_dich_geste_farbe3", "1F91F-1F3FD"},
    {"ich_liebe_dich_geste_farbe4", "1F91F-1F3FE"},
    {"ich_liebe_dich_geste_farbe5", "1F91F-1F3FF"},
    {"gesicht_mit_cowboyhut", "1F920"},
    {"clown_gesicht", "1F921"},
    {"wurgendes_gesicht", "1F922"},
    {"sich_vor_lachen_auf_dem_boden_walzen", "1F923"},
    {"sabberndes_gesicht", "1F924"},
    {"lugendes_gesicht", "1F925"},
    {"sich_an_den_kopf_fassende_person", "1F926"},
    {"sich_an_den_kopf_fassende_person_farbe1", "1F926-1F3FB"},
    {"sich_an_den_kopf_fassende_person_farbe2", "1F926-1F3FC"},
    {"sich_an_den_kopf_fassende_person_farbe3", "1F926-1F3FD"},
    {"sich_an_den_kopf_fassende_person_farbe4", "1F926-1F3FE"},
    {"sich_an_den_kopf_fassende_person_farbe5", "1F926-1F3FF"},
    {"niesendes_gesicht", "1F927"},
    {"gesicht_mit_hochgezogenen_augenbrauen", "1F928"},
    {"uberwaltigt", "1F929"},
    {"irres_gesicht", "1F92A"},
    {"ermahnendes_gesicht", "1F92B"},
    {"gesicht_mit_symbolen_uber_dem_mund", "1F92C"},
    {"verlegen_kicherndes_gesicht", "1F92D"},
    {"kotzendes_gesicht", "1F92E"},
    {"explodierender_kopf", "1F92F"},
    {"schwangere_frau", "1F930"},
    {"schwangere_frau_farbe1", "1F930-1F3FB"},
    {"schwangere_frau_farbe2", "1F930-1F3FC"},
    {"schwangere_frau_farbe3", "1F930-1F3FD"},
    {"schwangere_frau_farbe4", "1F930-1F3FE"},
    {"schwangere_frau_farbe5", "1F930-1F3FF"},
    {"stillen", "1F931"},
    {"stillen_farbe1", "1F931-1F3FB"},
    {"stillen_farbe2", "1F931-1F3FC"},
    {"stillen_farbe3", "1F931-1F3FD"},
    {"stillen_farbe4", "1F931-1F3FE"},
    {"stillen_farbe5", "1F931-1F3FF"},
    {"handflachen_nach_oben", "1F932"},
    {"handflachen_nach_oben_farbe1", "1F932-1F3FB"},
    {"handflachen_nach_oben_farbe2", "1F932-1F3FC"},
    {"handflachen_nach_oben_farbe3", "1F932-1F3FD"},
    {"handflachen_nach_oben_farbe4", "1F932-1F3FE"},
    {"handflachen_nach_oben_farbe5", "1F932-1F3FF"},
    {"selfie", "1F933"},
    {"selfie_farbe1", "1F933-1F3FB"},
    {"selfie_farbe2", "1F933-1F3FC"},
    {"selfie_farbe3", "1F933-1F3FD"},
    {"selfie_farbe4", "1F933-1F3FE"},
    {"selfie_farbe5", "1F933-1F3FF"},
    {"prinz", "1F934"},
    {"prinz_farbe1", "1F934-1F3FB"},
    {"prinz_farbe2", "1F934-1F3FC"},
    {"prinz_farbe3", "1F934-1F3FD"},
    {"prinz_farbe4", "1F934-1F3FE"},
    {"prinz_farbe5", "1F934-1F3FF"},
    {"person_im_smoking", "1F935"},
    {"person_im_smoking_farbe1", "1F935-1F3FB"},
    {"person_im_smoking_farbe2", "1F935-1F3FC"},
    {"person_im_smoking_farbe3", "1F935-1F3FD"},
    {"person_im_smoking_farbe4", "1F935-1F3FE"},
    {"person_im_smoking_farbe5", "1F935-1F3FF"},
    {"weihnachtsfrau", "1F936"},
    {"weihnachtsfrau_farbe1", "1F936-1F3FB"},
    {"weihnachtsfrau_farbe2", "1F936-1F3FC"},
    {"weihnachtsfrau_farbe3", "1F936-1F3FD"},
    {"weihnachtsfrau_farbe4", "1F936-1F3FE"},
    {"weihnachtsfrau_farbe5", "1F936-1F3FF"},
    {"schulterzuckende_person", "1F937"},
    {"schulterzuckende_person_farbe1", "1F937-1F3FB"},
    {"schulterzuckende_person_farbe2", "1F937-1F3FC"},
    {"schulterzuckende_person_farbe3", "1F937-1F3FD"},
    {"schulterzuckende_person_farbe4", "1F937-1F3FE"},
    {"schulterzuckende_person_farbe5", "1F937-1F3FF"},
    {"rad_schlagende_person", "1F938"},
    {"rad_schlagende_person_farbe1", "1F938-1F3FB"},
    {"rad_schlagende_person_farbe2", "1F938-1F3FC"},
    {"rad_schlagende_person_farbe3", "1F938-1F3FD"},
    {"rad_schlagende_person_farbe4", "1F938-1F3FE"},
    {"rad_schlagende_person_farbe5", "1F938-1F3FF"},
    {"jongleurin", "1F939-200D-2640-FE0F"},
    {"jongleurin_farbe1", "1F939-1F3FB-200D-2640-FE0F"},
    {"jongleurin_farbe2", "1F939-1F3FC-200D-2640-FE0F"},
    {"jongleurin_farbe3", "1F939-1F3FD-200D-2640-FE0F"},
    {"jongleurin_farbe4", "1F939-1F3FE-200D-2640-FE0F"},
    {"jongleurin_farbe5", "1F939-1F3FF-200D-2640-FE0F"},
    {"fechterin", "1F93A"},
    {"ringerin", "1F93C"},
    {"ringerin_farbe1", "1F93C-1F3FB"},
    {"ringerin_farbe2", "1F93C-1F3FC"},
    {"ringerin_farbe3", "1F93C-1F3FD"},
    {"ringerin_farbe4", "1F93C-1F3FE"},
    {"ringerin_farbe5", "1F93C-1F3FF"},
    {"ringerin_farbe1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"ringerin_farbe1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"ringerin_farbe1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"ringerin_farbe1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"ringerin_farbe2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"ringerin_farbe2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"ringerin_farbe2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"ringerin_farbe2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"ringerin_farbe3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"ringerin_farbe3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"ringerin_farbe3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"ringerin_farbe3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"ringerin_farbe4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"ringerin_farbe4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"ringerin_farbe4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"ringerin_farbe4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"ringerin_farbe5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"ringerin_farbe5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"ringerin_farbe5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"ringerin_farbe5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"wasserballspielerin", "1F93D-200D-2640-FE0F"},
    {"wasserballspielerin_farbe1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"wasserballspielerin_farbe2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"wasserballspielerin_farbe3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"wasserballspielerin_farbe4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"wasserballspielerin_farbe5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"handballspielerin", "1F93E-200D-2640-FE0F"},
    {"handballspielerin_farbe1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"handballspielerin_farbe2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"handballspielerin_farbe3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"handballspielerin_farbe4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"handballspielerin_farbe5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"tauchmaske", "1F93F"},
    {"welke_blume", "1F940"},
    {"trommel", "1F941"},
    {"sektglaser", "1F942"},
    {"trinkglas", "1F943"},
    {"loffel", "1F944"},
    {"tor", "1F945"},
    {"goldmedaille", "1F947"},
    {"silbermedaille", "1F948"},
    {"bronzemedaille", "1F949"},
    {"boxhandschuh", "1F94A"},
    {"kampfsportanzug", "1F94B"},
    {"curlingstein", "1F94C"},
    {"lacrosse", "1F94D"},
    {"softball", "1F94E"},
    {"frisbee", "1F94F"},
    {"croissant", "1F950"},
    {"avocado", "1F951"},
    {"gurke", "1F952"},
    {"bacon", "1F953"},
    {"kartoffel", "1F954"},
    {"karotte", "1F955"},
    {"baguette", "1F956"},
    {"salat", "1F957"},
    {"pfannengericht", "1F958"},
    {"doner", "1F959"},
    {"ei", "1F95A"},
    {"glas_milch", "1F95B"},
    {"erdnuss", "1F95C"},
    {"kiwi", "1F95D"},
    {"pfannkuchen", "1F95E"},
    {"teigtasche", "1F95F"},
    {"gluckskeks", "1F960"},
    {"takeaway_schachtel", "1F961"},
    {"essstabchen", "1F962"},
    {"schussel_mit_loffel", "1F963"},
    {"becher_mit_strohhalm", "1F964"},
    {"kokosnuss", "1F965"},
    {"brokkoli", "1F966"},
    {"kuchen", "1F967"},
    {"brezel", "1F968"},
    {"fleischstuck", "1F969"},
    {"sandwich", "1F96A"},
    {"konserve", "1F96B"},
    {"blattgemuse", "1F96C"},
    {"mango", "1F96D"},
    {"mondkuchen", "1F96E"},
    {"bagel", "1F96F"},
    {"lachelndes_gesicht_mit_herzen", "1F970"},
    {"gahnendes_gesicht", "1F971"},
    {"lachendes_gesicht_mit_trane", "1F972"},
    {"partygesicht", "1F973"},
    {"schwindeliges_gesicht", "1F974"},
    {"schwitzendes_gesicht", "1F975"},
    {"frierendes_gesicht", "1F976"},
    {"ninja", "1F977"},
    {"ninja_farbe1", "1F977-1F3FB"},
    {"ninja_farbe2", "1F977-1F3FC"},
    {"ninja_farbe3", "1F977-1F3FD"},
    {"ninja_farbe4", "1F977-1F3FE"},
    {"ninja_farbe5", "1F977-1F3FF"},
    {"verkleidet", "1F978"},
    {"gesicht_das_tranen_zuruckhalt", "1F979"},
    {"bettelndes_gesicht", "1F97A"},
    {"sari", "1F97B"},
    {"laborkittel", "1F97C"},
    {"schutzbrille", "1F97D"},
    {"wanderstiefel", "1F97E"},
    {"flacher_schuh", "1F97F"},
    {"krebs", "1F980"},
    {"lowe", "1F981"},
    {"skorpion", "1F982"},
    {"truthahn", "1F983"},
    {"einhorn", "1F984"},
    {"adler", "1F985"},
    {"ente", "1F986"},
    {"fledermaus", "1F987"},
    {"hai", "1F988"},
    {"eule", "1F989"},
    {"fuchs", "1F98A"},
    {"schmetterling", "1F98B"},
    {"hirsch", "1F98C"},
    {"gorilla", "1F98D"},
    {"eidechse", "1F98E"},
    {"nashorn", "1F98F"},
    {"garnele", "1F990"},
    {"tintenfisch", "1F991"},
    {"giraffe", "1F992"},
    {"zebra", "1F993"},
    {"igel", "1F994"},
    {"sauropode", "1F995"},
    {"trex", "1F996"},
    {"grille", "1F997"},
    {"kanguru", "1F998"},
    {"lama", "1F999"},
    {"pfau", "1F99A"},
    {"nilpferd", "1F99B"},
    {"papagei", "1F99C"},
    {"waschbar", "1F99D"},
    {"hummer", "1F99E"},
    {"mucke", "1F99F"},
    {"mikrobe", "1F9A0"},
    {"dachs", "1F9A1"},
    {"schwan", "1F9A2"},
    {"mammut", "1F9A3"},
    {"dodo", "1F9A4"},
    {"faultier", "1F9A5"},
    {"otter", "1F9A6"},
    {"orang_utan", "1F9A7"},
    {"stinktier", "1F9A8"},
    {"flamingo", "1F9A9"},
    {"auster", "1F9AA"},
    {"biber", "1F9AB"},
    {"bison", "1F9AC"},
    {"seehund", "1F9AD"},
    {"blindenhund", "1F9AE"},
    {"blindenstock", "1F9AF"},
    {"rotes_haar", "1F9B0"},
    {"lockiges_haar", "1F9B1"},
    {"glatze", "1F9B2"},
    {"weisses_haar", "1F9B3"},
    {"knochen", "1F9B4"},
    {"bein", "1F9B5"},
    {"bein_farbe1", "1F9B5-1F3FB"},
    {"bein_farbe2", "1F9B5-1F3FC"},
    {"bein_farbe3", "1F9B5-1F3FD"},
    {"bein_farbe4", "1F9B5-1F3FE"},
    {"bein_farbe5", "1F9B5-1F3FF"},
    {"fuss", "1F9B6"},
    {"fuss_farbe1", "1F9B6-1F3FB"},
    {"fuss_farbe2", "1F9B6-1F3FC"},
    {"fuss_farbe3", "1F9B6-1F3FD"},
    {"fuss_farbe4", "1F9B6-1F3FE"},
    {"fuss_farbe5", "1F9B6-1F3FF"},
    {"zahn", "1F9B7"},
    {"superheldin", "1F9B8"},
    {"superheldin_farbe1", "1F9B8-1F3FB"},
    {"superheldin_farbe2", "1F9B8-1F3FC"},
    {"superheldin_farbe3", "1F9B8-1F3FD"},
    {"superheldin_farbe4", "1F9B8-1F3FE"},
    {"superheldin_farbe5", "1F9B8-1F3FF"},
    {"bosewicht", "1F9B9"},
    {"bosewicht_farbe1", "1F9B9-1F3FB"},
    {"bosewicht_farbe2", "1F9B9-1F3FC"},
    {"bosewicht_farbe3", "1F9B9-1F3FD"},
    {"bosewicht_farbe4", "1F9B9-1F3FE"},
    {"bosewicht_farbe5", "1F9B9-1F3FF"},
    {"sicherheitsweste", "1F9BA"},
    {"ohr_mit_horgerat", "1F9BB"},
    {"ohr_mit_horgerat_farbe1", "1F9BB-1F3FB"},
    {"ohr_mit_horgerat_farbe2", "1F9BB-1F3FC"},
    {"ohr_mit_horgerat_farbe3", "1F9BB-1F3FD"},
    {"ohr_mit_horgerat_farbe4", "1F9BB-1F3FE"},
    {"ohr_mit_horgerat_farbe5", "1F9BB-1F3FF"},
    {"elektrischer_rollstuhl", "1F9BC"},
    {"manueller_rollstuhl", "1F9BD"},
    {"armprothese", "1F9BE"},
    {"beinprothese", "1F9BF"},
    {"kasestuck", "1F9C0"},
    {"cupcake", "1F9C1"},
    {"salz", "1F9C2"},
    {"trinkpackchen", "1F9C3"},
    {"knoblauch", "1F9C4"},
    {"zwiebel", "1F9C5"},
    {"falafel", "1F9C6"},
    {"waffel", "1F9C7"},
    {"butter", "1F9C8"},
    {"mate_tee", "1F9C9"},
    {"eiswurfel", "1F9CA"},
    {"bubble_tea", "1F9CB"},
    {"troll", "1F9CC"},
    {"stehende_person", "1F9CD"},
    {"stehende_person_farbe1", "1F9CD-1F3FB"},
    {"stehende_person_farbe2", "1F9CD-1F3FC"},
    {"stehende_person_farbe3", "1F9CD-1F3FD"},
    {"stehende_person_farbe4", "1F9CD-1F3FE"},
    {"stehende_person_farbe5", "1F9CD-1F3FF"},
    {"kniende_person", "1F9CE"},
    {"kniende_person_farbe1", "1F9CE-1F3FB"},
    {"kniende_person_farbe2", "1F9CE-1F3FC"},
    {"kniende_person_farbe3", "1F9CE-1F3FD"},
    {"kniende_person_farbe4", "1F9CE-1F3FE"},
    {"kniende_person_farbe5", "1F9CE-1F3FF"},
    {"gehorlose_person", "1F9CF"},
    {"gehorlose_person_farbe1", "1F9CF-1F3FB"},
    {"gehorlose_person_farbe2", "1F9CF-1F3FC"},
    {"gehorlose_person_farbe3", "1F9CF-1F3FD"},
    {"gehorlose_person_farbe4", "1F9CF-1F3FE"},
    {"gehorlose_person_farbe5", "1F9CF-1F3FF"},
    {"gesicht_mit_monokel", "1F9D0"},
    {"person", "1F9D1"},
    {"person_farbe1", "1F9D1-1F3FB"},
    {"person_farbe2", "1F9D1-1F3FC"},
    {"person_farbe3", "1F9D1-1F3FD"},
    {"person_farbe4", "1F9D1-1F3FE"},
    {"person_farbe5", "1F9D1-1F3FF"},
    {"kind", "1F9D2"},
    {"kind_farbe1", "1F9D2-1F3FB"},
    {"kind_farbe2", "1F9D2-1F3FC"},
    {"kind_farbe3", "1F9D2-1F3FD"},
    {"kind_farbe4", "1F9D2-1F3FE"},
    {"kind_farbe5", "1F9D2-1F3FF"},
    {"altere_person", "1F9D3"},
    {"altere_person_farbe1", "1F9D3-1F3FB"},
    {"altere_person_farbe2", "1F9D3-1F3FC"},
    {"altere_person_farbe3", "1F9D3-1F3FD"},
    {"altere_person_farbe4", "1F9D3-1F3FE"},
    {"altere_person_farbe5", "1F9D3-1F3FF"},
    {"person_bart", "1F9D4"},
    {"person_bart_farbe1", "1F9D4-1F3FB"},
    {"person_bart_farbe2", "1F9D4-1F3FC"},
    {"person_bart_farbe3", "1F9D4-1F3FD"},
    {"person_bart_farbe4", "1F9D4-1F3FE"},
    {"person_bart_farbe5", "1F9D4-1F3FF"},
    {"frau_mit_kopftuch", "1F9D5"},
    {"frau_mit_kopftuch_farbe1", "1F9D5-1F3FB"},
    {"frau_mit_kopftuch_farbe2", "1F9D5-1F3FC"},
    {"frau_mit_kopftuch_farbe3", "1F9D5-1F3FD"},
    {"frau_mit_kopftuch_farbe4", "1F9D5-1F3FE"},
    {"frau_mit_kopftuch_farbe5", "1F9D5-1F3FF"},
    {"person_in_dampfsauna", "1F9D6"},
    {"person_in_dampfsauna_farbe1", "1F9D6-1F3FB"},
    {"person_in_dampfsauna_farbe2", "1F9D6-1F3FC"},
    {"person_in_dampfsauna_farbe3", "1F9D6-1F3FD"},
    {"person_in_dampfsauna_farbe4", "1F9D6-1F3FE"},
    {"person_in_dampfsauna_farbe5", "1F9D6-1F3FF"},
    {"bergsteigerin", "1F9D7-200D-2640-FE0F"},
    {"bergsteigerin_farbe1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"bergsteigerin_farbe2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"bergsteigerin_farbe3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"bergsteigerin_farbe4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"bergsteigerin_farbe5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"person_im_lotossitz", "1F9D8"},
    {"person_im_lotossitz_farbe1", "1F9D8-1F3FB"},
    {"person_im_lotossitz_farbe2", "1F9D8-1F3FC"},
    {"person_im_lotossitz_farbe3", "1F9D8-1F3FD"},
    {"person_im_lotossitz_farbe4", "1F9D8-1F3FE"},
    {"person_im_lotossitz_farbe5", "1F9D8-1F3FF"},
    {"magierin", "1F9D9-200D-2640-FE0F"},
    {"magierin_farbe1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"magierin_farbe2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"magierin_farbe3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"magierin_farbe4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"magierin_farbe5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"marchenfee", "1F9DA"},
    {"marchenfee_farbe1", "1F9DA-1F3FB"},
    {"marchenfee_farbe2", "1F9DA-1F3FC"},
    {"marchenfee_farbe3", "1F9DA-1F3FD"},
    {"marchenfee_farbe4", "1F9DA-1F3FE"},
    {"marchenfee_farbe5", "1F9DA-1F3FF"},
    {"vampir", "1F9DB"},
    {"vampir_farbe1", "1F9DB-1F3FB"},
    {"vampir_farbe2", "1F9DB-1F3FC"},
    {"vampir_farbe3", "1F9DB-1F3FD"},
    {"vampir_farbe4", "1F9DB-1F3FE"},
    {"vampir_farbe5", "1F9DB-1F3FF"},
    {"wassermensch", "1F9DC"},
    {"wassermensch_farbe1", "1F9DC-1F3FB"},
    {"wassermensch_farbe2", "1F9DC-1F3FC"},
    {"wassermensch_farbe3", "1F9DC-1F3FD"},
    {"wassermensch_farbe4", "1F9DC-1F3FE"},
    {"wassermensch_farbe5", "1F9DC-1F3FF"},
    {"elfe", "1F9DD-200D-2640-FE0F"},
    {"elfe_farbe1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"elfe_farbe2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"elfe_farbe3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"elfe_farbe4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"elfe_farbe5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"flaschengeist", "1F9DE"},
    {"zombie", "1F9DF"},
    {"gehirn", "1F9E0"},
    {"oranges_herz", "1F9E1"},
    {"baseballmutze", "1F9E2"},
    {"schal", "1F9E3"},
    {"handschuhe", "1F9E4"},
    {"mantel", "1F9E5"},
    {"socken", "1F9E6"},
    {"roter_umschlag", "1F9E7"},
    {"feuerwerkskorper", "1F9E8"},
    {"puzzleteil", "1F9E9"},
    {"reagenzglas", "1F9EA"},
    {"petrischale", "1F9EB"},
    {"dna", "1F9EC"},
    {"kompass", "1F9ED"},
    {"abakus", "1F9EE"},
    {"feuerloscher", "1F9EF"},
    {"werkzeugkasten", "1F9F0"},
    {"ziegelstein", "1F9F1"},
    {"magnet", "1F9F2"},
    {"gepack", "1F9F3"},
    {"creme", "1F9F4"},
    {"faden", "1F9F5"},
    {"wollknauel", "1F9F6"},
    {"sicherheitsnadel", "1F9F7"},
    {"teddybar", "1F9F8"},
    {"besen", "1F9F9"},
    {"korb", "1F9FA"},
    {"kuchenrolle", "1F9FB"},
    {"seife", "1F9FC"},
    {"schwamm", "1F9FD"},
    {"beleg", "1F9FE"},
    {"nazar_amulett", "1F9FF"},
    {"ballettschuhe", "1FA70"},
    {"einteiliger_badeanzug", "1FA71"},
    {"slip", "1FA72"},
    {"shorts", "1FA73"},
    {"zehensandale", "1FA74"},
    {"hellblaues_herz", "1FA75"},
    {"graues_herz", "1FA76"},
    {"pinkes_herz", "1FA77"},
    {"blutstropfen", "1FA78"},
    {"heftpflaster", "1FA79"},
    {"stethoskop", "1FA7A"},
    {"rontgenbild", "1FA7B"},
    {"krucke", "1FA7C"},
    {"jo_jo", "1FA80"},
    {"drachen", "1FA81"},
    {"fallschirm", "1FA82"},
    {"bumerang", "1FA83"},
    {"zauberstab", "1FA84"},
    {"pinata", "1FA85"},
    {"matroschka", "1FA86"},
    {"maracas", "1FA87"},
    {"flote", "1FA88"},
    {"harfe", "1FA89"},
    {"posaune", "1FA8A"},
    {"schatztruhe", "1FA8E"},
    {"schaufel", "1FA8F"},
    {"ringplanet", "1FA90"},
    {"stuhl", "1FA91"},
    {"rasierer", "1FA92"},
    {"axt", "1FA93"},
    {"ollampe", "1FA94"},
    {"banjo", "1FA95"},
    {"militarhelm", "1FA96"},
    {"akkordeon", "1FA97"},
    {"afrikanische_trommel", "1FA98"},
    {"munze", "1FA99"},
    {"handsage", "1FA9A"},
    {"schraubenzieher", "1FA9B"},
    {"leiter", "1FA9C"},
    {"haken", "1FA9D"},
    {"spiegel", "1FA9E"},
    {"fenster", "1FA9F"},
    {"saugglocke", "1FAA0"},
    {"nahnadel", "1FAA1"},
    {"knoten", "1FAA2"},
    {"eimer", "1FAA3"},
    {"mausefalle", "1FAA4"},
    {"zahnburste", "1FAA5"},
    {"grabstein", "1FAA6"},
    {"protestschild", "1FAA7"},
    {"felsen", "1FAA8"},
    {"discokugel", "1FAA9"},
    {"ausweis", "1FAAA"},
    {"schwache_batterie", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"faltfacher", "1FAAD"},
    {"haarkamm", "1FAAE"},
    {"khanda", "1FAAF"},
    {"fliege", "1FAB0"},
    {"wurm", "1FAB1"},
    {"kafer", "1FAB2"},
    {"kakerlake", "1FAB3"},
    {"topfpflanze", "1FAB4"},
    {"holz", "1FAB5"},
    {"feder", "1FAB6"},
    {"lotusblute", "1FAB7"},
    {"koralle", "1FAB8"},
    {"leeres_nest", "1FAB9"},
    {"nest_mit_eiern", "1FABA"},
    {"hyazinthe", "1FABB"},
    {"qualle", "1FABC"},
    {"flugel", "1FABD"},
    {"kahler_baum", "1FABE"},
    {"gans", "1FABF"},
    {"herz_organ", "1FAC0"},
    {"lunge", "1FAC1"},
    {"sich_umarmende_personen", "1FAC2"},
    {"schwangerer_mann", "1FAC3"},
    {"schwangerer_mann_farbe1", "1FAC3-1F3FB"},
    {"schwangerer_mann_farbe2", "1FAC3-1F3FC"},
    {"schwangerer_mann_farbe3", "1FAC3-1F3FD"},
    {"schwangerer_mann_farbe4", "1FAC3-1F3FE"},
    {"schwangerer_mann_farbe5", "1FAC3-1F3FF"},
    {"schwangere_person", "1FAC4"},
    {"schwangere_person_farbe1", "1FAC4-1F3FB"},
    {"schwangere_person_farbe2", "1FAC4-1F3FC"},
    {"schwangere_person_farbe3", "1FAC4-1F3FD"},
    {"schwangere_person_farbe4", "1FAC4-1F3FE"},
    {"schwangere_person_farbe5", "1FAC4-1F3FF"},
    {"person_mit_krone", "1FAC5"},
    {"person_mit_krone_farbe1", "1FAC5-1F3FB"},
    {"person_mit_krone_farbe2", "1FAC5-1F3FC"},
    {"person_mit_krone_farbe3", "1FAC5-1F3FD"},
    {"person_mit_krone_farbe4", "1FAC5-1F3FE"},
    {"person_mit_krone_farbe5", "1FAC5-1F3FF"},
    {"fingerabdruck", "1FAC6"},
    {"haariges_wesen", "1FAC8"},
    {"orca", "1FACD"},
    {"elch", "1FACE"},
    {"esel", "1FACF"},
    {"blaubeeren", "1FAD0"},
    {"paprika", "1FAD1"},
    {"olive", "1FAD2"},
    {"fladenbrot", "1FAD3"},
    {"tamale", "1FAD4"},
    {"fondue", "1FAD5"},
    {"teekanne", "1FAD6"},
    {"flussigkeit_ausgiessen", "1FAD7"},
    {"bohnen", "1FAD8"},
    {"einmachglas", "1FAD9"},
    {"ingwer", "1FADA"},
    {"erbsenschote", "1FADB"},
    {"wurzelgemuse", "1FADC"},
    {"spritzer", "1FADF"},
    {"schmelzendes_gesicht", "1FAE0"},
    {"salutierendes_gesicht", "1FAE1"},
    {"gesicht_mit_offenen_augen_und_hand_uber_dem_mund", "1FAE2"},
    {"gesicht_mit_durch_die_finger_linsendem_auge", "1FAE3"},
    {"gesicht_mit_schragem_mund", "1FAE4"},
    {"gestricheltes_gesicht", "1FAE5"},
    {"auf_lippe_beissen", "1FAE6"},
    {"blasen", "1FAE7"},
    {"zitterndes_doppelgesicht", "1FAE8"},
    {"gesicht_mit_augenringen", "1FAE9"},
    {"verzerrtes_gesicht", "1FAEA"},
    {"kampfwolke", "1FAEF"},
    {"hand_mit_gekreuztem_zeigefinger_und_daumen", "1FAF0"},
    {"hand_mit_gekreuztem_zeigefinger_und_daumen_farbe1", "1FAF0-1F3FB"},
    {"hand_mit_gekreuztem_zeigefinger_und_daumen_farbe2", "1FAF0-1F3FC"},
    {"hand_mit_gekreuztem_zeigefinger_und_daumen_farbe3", "1FAF0-1F3FD"},
    {"hand_mit_gekreuztem_zeigefinger_und_daumen_farbe4", "1FAF0-1F3FE"},
    {"hand_mit_gekreuztem_zeigefinger_und_daumen_farbe5", "1FAF0-1F3FF"},
    {"nach_rechts_weisende_hand", "1FAF1"},
    {"nach_rechts_weisende_hand_farbe1", "1FAF1-1F3FB"},
    {"nach_rechts_weisende_hand_farbe2", "1FAF1-1F3FC"},
    {"nach_rechts_weisende_hand_farbe3", "1FAF1-1F3FD"},
    {"nach_rechts_weisende_hand_farbe4", "1FAF1-1F3FE"},
    {"nach_rechts_weisende_hand_farbe5", "1FAF1-1F3FF"},
    {"nach_links_weisende_hand", "1FAF2"},
    {"nach_links_weisende_hand_farbe1", "1FAF2-1F3FB"},
    {"nach_links_weisende_hand_farbe2", "1FAF2-1F3FC"},
    {"nach_links_weisende_hand_farbe3", "1FAF2-1F3FD"},
    {"nach_links_weisende_hand_farbe4", "1FAF2-1F3FE"},
    {"nach_links_weisende_hand_farbe5", "1FAF2-1F3FF"},
    {"hand_mit_handflache_nach_unten", "1FAF3"},
    {"hand_mit_handflache_nach_unten_farbe1", "1FAF3-1F3FB"},
    {"hand_mit_handflache_nach_unten_farbe2", "1FAF3-1F3FC"},
    {"hand_mit_handflache_nach_unten_farbe3", "1FAF3-1F3FD"},
    {"hand_mit_handflache_nach_unten_farbe4", "1FAF3-1F3FE"},
    {"hand_mit_handflache_nach_unten_farbe5", "1FAF3-1F3FF"},
    {"hand_mit_handflache_nach_oben", "1FAF4"},
    {"hand_mit_handflache_nach_oben_farbe1", "1FAF4-1F3FB"},
    {"hand_mit_handflache_nach_oben_farbe2", "1FAF4-1F3FC"},
    {"hand_mit_handflache_nach_oben_farbe3", "1FAF4-1F3FD"},
    {"hand_mit_handflache_nach_oben_farbe4", "1FAF4-1F3FE"},
    {"hand_mit_handflache_nach_oben_farbe5", "1FAF4-1F3FF"},
    {"auf_betrachter_zeigender_zeigefinger", "1FAF5"},
    {"auf_betrachter_zeigender_zeigefinger_farbe1", "1FAF5-1F3FB"},
    {"auf_betrachter_zeigender_zeigefinger_farbe2", "1FAF5-1F3FC"},
    {"auf_betrachter_zeigender_zeigefinger_farbe3", "1FAF5-1F3FD"},
    {"auf_betrachter_zeigender_zeigefinger_farbe4", "1FAF5-1F3FE"},
    {"auf_betrachter_zeigender_zeigefinger_farbe5", "1FAF5-1F3FF"},
    {"hande_die_herz_bilden", "1FAF6"},
    {"hande_die_herz_bilden_farbe1", "1FAF6-1F3FB"},
    {"hande_die_herz_bilden_farbe2", "1FAF6-1F3FC"},
    {"hande_die_herz_bilden_farbe3", "1FAF6-1F3FD"},
    {"hande_die_herz_bilden_farbe4", "1FAF6-1F3FE"},
    {"hande_die_herz_bilden_farbe5", "1FAF6-1F3FF"},
    {"nach_links_schiebende_hand", "1FAF7"},
    {"nach_links_schiebende_hand_farbe1", "1FAF7-1F3FB"},
    {"nach_links_schiebende_hand_farbe2", "1FAF7-1F3FC"},
    {"nach_links_schiebende_hand_farbe3", "1FAF7-1F3FD"},
    {"nach_links_schiebende_hand_farbe4", "1FAF7-1F3FE"},
    {"nach_links_schiebende_hand_farbe5", "1FAF7-1F3FF"},
    {"nach_rechts_schiebende_hand", "1FAF8"},
    {"nach_rechts_schiebende_hand_farbe1", "1FAF8-1F3FB"},
    {"nach_rechts_schiebende_hand_farbe2", "1FAF8-1F3FC"},
    {"nach_rechts_schiebende_hand_farbe3", "1FAF8-1F3FD"},
    {"nach_rechts_schiebende_hand_farbe4", "1FAF8-1F3FE"},
    {"nach_rechts_schiebende_hand_farbe5", "1FAF8-1F3FF"},
    {"taste_nummernschild", "0023-FE0F-20E3"},
    {"taste_sternchen", "002A-FE0F-20E3"},
    {"taste_0", "0030-FE0F-20E3"},
    {"taste_1", "0031-FE0F-20E3"},
    {"taste_2", "0032-FE0F-20E3"},
    {"taste_3", "0033-FE0F-20E3"},
    {"taste_4", "0034-FE0F-20E3"},
    {"taste_5", "0035-FE0F-20E3"},
    {"taste_6", "0036-FE0F-20E3"},
    {"taste_7", "0037-FE0F-20E3"},
    {"taste_8", "0038-FE0F-20E3"},
    {"taste_9", "0039-FE0F-20E3"},
    {"flagge_ascension", "1F1E6-1F1E8"},
    {"flagge_andorra", "1F1E6-1F1E9"},
    {"flagge_vereinigte_arabische_emirate", "1F1E6-1F1EA"},
    {"flagge_afghanistan", "1F1E6-1F1EB"},
    {"flagge_antigua_und_barbuda", "1F1E6-1F1EC"},
    {"flagge_anguilla", "1F1E6-1F1EE"},
    {"flagge_albanien", "1F1E6-1F1F1"},
    {"flagge_armenien", "1F1E6-1F1F2"},
    {"flagge_angola", "1F1E6-1F1F4"},
    {"flagge_antarktis", "1F1E6-1F1F6"},
    {"flagge_argentinien", "1F1E6-1F1F7"},
    {"flagge_amerikanisch_samoa", "1F1E6-1F1F8"},
    {"flagge_osterreich", "1F1E6-1F1F9"},
    {"flagge_australien", "1F1E6-1F1FA"},
    {"flagge_aruba", "1F1E6-1F1FC"},
    {"flagge_alandinseln", "1F1E6-1F1FD"},
    {"flagge_aserbaidschan", "1F1E6-1F1FF"},
    {"flagge_bosnien_und_herzegowina", "1F1E7-1F1E6"},
    {"flagge_barbados", "1F1E7-1F1E7"},
    {"flagge_bangladesch", "1F1E7-1F1E9"},
    {"flagge_belgien", "1F1E7-1F1EA"},
    {"flagge_burkina_faso", "1F1E7-1F1EB"},
    {"flagge_bulgarien", "1F1E7-1F1EC"},
    {"flagge_bahrain", "1F1E7-1F1ED"},
    {"flagge_burundi", "1F1E7-1F1EE"},
    {"flagge_benin", "1F1E7-1F1EF"},
    {"flagge_st_barthelemy", "1F1E7-1F1F1"},
    {"flagge_bermuda", "1F1E7-1F1F2"},
    {"flagge_brunei_darussalam", "1F1E7-1F1F3"},
    {"flagge_bolivien", "1F1E7-1F1F4"},
    {"flagge_karibische_niederlande", "1F1E7-1F1F6"},
    {"flagge_brasilien", "1F1E7-1F1F7"},
    {"flagge_bahamas", "1F1E7-1F1F8"},
    {"flagge_bhutan", "1F1E7-1F1F9"},
    {"flagge_bouvetinsel", "1F1E7-1F1FB"},
    {"flagge_botsuana", "1F1E7-1F1FC"},
    {"flagge_belarus", "1F1E7-1F1FE"},
    {"flagge_belize", "1F1E7-1F1FF"},
    {"flagge_kanada", "1F1E8-1F1E6"},
    {"flagge_kokosinseln", "1F1E8-1F1E8"},
    {"flagge_kongo_kinshasa", "1F1E8-1F1E9"},
    {"flagge_zentralafrikanische_republik", "1F1E8-1F1EB"},
    {"flagge_kongo_brazzaville", "1F1E8-1F1EC"},
    {"flagge_schweiz", "1F1E8-1F1ED"},
    {"flagge_cote_divoire", "1F1E8-1F1EE"},
    {"flagge_cookinseln", "1F1E8-1F1F0"},
    {"flagge_chile", "1F1E8-1F1F1"},
    {"flagge_kamerun", "1F1E8-1F1F2"},
    {"flagge_china", "1F1E8-1F1F3"},
    {"flagge_kolumbien", "1F1E8-1F1F4"},
    {"flagge_clipperton_insel", "1F1E8-1F1F5"},
    {"flagge_sark", "1F1E8-1F1F6"},
    {"flagge_costa_rica", "1F1E8-1F1F7"},
    {"flagge_kuba", "1F1E8-1F1FA"},
    {"flagge_cabo_verde", "1F1E8-1F1FB"},
    {"flagge_curacao", "1F1E8-1F1FC"},
    {"flagge_weihnachtsinsel", "1F1E8-1F1FD"},
    {"flagge_zypern", "1F1E8-1F1FE"},
    {"flagge_tschechien", "1F1E8-1F1FF"},
    {"flagge_deutschland", "1F1E9-1F1EA"},
    {"flagge_diego_garcia", "1F1E9-1F1EC"},
    {"flagge_dschibuti", "1F1E9-1F1EF"},
    {"flagge_danemark", "1F1E9-1F1F0"},
    {"flagge_dominica", "1F1E9-1F1F2"},
    {"flagge_dominikanische_republik", "1F1E9-1F1F4"},
    {"flagge_algerien", "1F1E9-1F1FF"},
    {"flagge_ceuta_und_melilla", "1F1EA-1F1E6"},
    {"flagge_ecuador", "1F1EA-1F1E8"},
    {"flagge_estland", "1F1EA-1F1EA"},
    {"flagge_agypten", "1F1EA-1F1EC"},
    {"flagge_westsahara", "1F1EA-1F1ED"},
    {"flagge_eritrea", "1F1EA-1F1F7"},
    {"flagge_spanien", "1F1EA-1F1F8"},
    {"flagge_athiopien", "1F1EA-1F1F9"},
    {"flagge_europaische_union", "1F1EA-1F1FA"},
    {"flagge_finnland", "1F1EB-1F1EE"},
    {"flagge_fidschi", "1F1EB-1F1EF"},
    {"flagge_falklandinseln", "1F1EB-1F1F0"},
    {"flagge_mikronesien", "1F1EB-1F1F2"},
    {"flagge_faroer", "1F1EB-1F1F4"},
    {"flagge_frankreich", "1F1EB-1F1F7"},
    {"flagge_gabun", "1F1EC-1F1E6"},
    {"flagge_vereinigtes_konigreich", "1F1EC-1F1E7"},
    {"flagge_grenada", "1F1EC-1F1E9"},
    {"flagge_georgien", "1F1EC-1F1EA"},
    {"flagge_franzosisch_guayana", "1F1EC-1F1EB"},
    {"flagge_guernsey", "1F1EC-1F1EC"},
    {"flagge_ghana", "1F1EC-1F1ED"},
    {"flagge_gibraltar", "1F1EC-1F1EE"},
    {"flagge_gronland", "1F1EC-1F1F1"},
    {"flagge_gambia", "1F1EC-1F1F2"},
    {"flagge_guinea", "1F1EC-1F1F3"},
    {"flagge_guadeloupe", "1F1EC-1F1F5"},
    {"flagge_aquatorialguinea", "1F1EC-1F1F6"},
    {"flagge_griechenland", "1F1EC-1F1F7"},
    {"flagge_sudgeorgien_und_die_sudlichen_sandwichinseln", "1F1EC-1F1F8"},
    {"flagge_guatemala", "1F1EC-1F1F9"},
    {"flagge_guam", "1F1EC-1F1FA"},
    {"flagge_guinea_bissau", "1F1EC-1F1FC"},
    {"flagge_guyana", "1F1EC-1F1FE"},
    {"flagge_sonderverwaltungsregion_hongkong", "1F1ED-1F1F0"},
    {"flagge_heard_und_mcdonaldinseln", "1F1ED-1F1F2"},
    {"flagge_honduras", "1F1ED-1F1F3"},
    {"flagge_kroatien", "1F1ED-1F1F7"},
    {"flagge_haiti", "1F1ED-1F1F9"},
    {"flagge_ungarn", "1F1ED-1F1FA"},
    {"flagge_kanarische_inseln", "1F1EE-1F1E8"},
    {"flagge_indonesien", "1F1EE-1F1E9"},
    {"flagge_irland", "1F1EE-1F1EA"},
    {"flagge_israel", "1F1EE-1F1F1"},
    {"flagge_isle_of_man", "1F1EE-1F1F2"},
    {"flagge_indien", "1F1EE-1F1F3"},
    {"flagge_britisches_territorium_im_indischen_ozean", "1F1EE-1F1F4"},
    {"flagge_irak", "1F1EE-1F1F6"},
    {"flagge_iran", "1F1EE-1F1F7"},
    {"flagge_island", "1F1EE-1F1F8"},
    {"flagge_italien", "1F1EE-1F1F9"},
    {"flagge_jersey", "1F1EF-1F1EA"},
    {"flagge_jamaika", "1F1EF-1F1F2"},
    {"flagge_jordanien", "1F1EF-1F1F4"},
    {"flagge_japan", "1F1EF-1F1F5"},
    {"flagge_kenia", "1F1F0-1F1EA"},
    {"flagge_kirgisistan", "1F1F0-1F1EC"},
    {"flagge_kambodscha", "1F1F0-1F1ED"},
    {"flagge_kiribati", "1F1F0-1F1EE"},
    {"flagge_komoren", "1F1F0-1F1F2"},
    {"flagge_st_kitts_und_nevis", "1F1F0-1F1F3"},
    {"flagge_nordkorea", "1F1F0-1F1F5"},
    {"flagge_sudkorea", "1F1F0-1F1F7"},
    {"flagge_kuwait", "1F1F0-1F1FC"},
    {"flagge_kaimaninseln", "1F1F0-1F1FE"},
    {"flagge_kasachstan", "1F1F0-1F1FF"},
    {"flagge_laos", "1F1F1-1F1E6"},
    {"flagge_libanon", "1F1F1-1F1E7"},
    {"flagge_st_lucia", "1F1F1-1F1E8"},
    {"flagge_liechtenstein", "1F1F1-1F1EE"},
    {"flagge_sri_lanka", "1F1F1-1F1F0"},
    {"flagge_liberia", "1F1F1-1F1F7"},
    {"flagge_lesotho", "1F1F1-1F1F8"},
    {"flagge_litauen", "1F1F1-1F1F9"},
    {"flagge_luxemburg", "1F1F1-1F1FA"},
    {"flagge_lettland", "1F1F1-1F1FB"},
    {"flagge_libyen", "1F1F1-1F1FE"},
    {"flagge_marokko", "1F1F2-1F1E6"},
    {"flagge_monaco", "1F1F2-1F1E8"},
    {"flagge_republik_moldau", "1F1F2-1F1E9"},
    {"flagge_montenegro", "1F1F2-1F1EA"},
    {"flagge_st_martin", "1F1F2-1F1EB"},
    {"flagge_madagaskar", "1F1F2-1F1EC"},
    {"flagge_marshallinseln", "1F1F2-1F1ED"},
    {"flagge_nordmazedonien", "1F1F2-1F1F0"},
    {"flagge_mali", "1F1F2-1F1F1"},
    {"flagge_myanmar", "1F1F2-1F1F2"},
    {"flagge_mongolei", "1F1F2-1F1F3"},
    {"flagge_sonderverwaltungsregion_macau", "1F1F2-1F1F4"},
    {"flagge_nordliche_marianen", "1F1F2-1F1F5"},
    {"flagge_martinique", "1F1F2-1F1F6"},
    {"flagge_mauretanien", "1F1F2-1F1F7"},
    {"flagge_montserrat", "1F1F2-1F1F8"},
    {"flagge_malta", "1F1F2-1F1F9"},
    {"flagge_mauritius", "1F1F2-1F1FA"},
    {"flagge_malediven", "1F1F2-1F1FB"},
    {"flagge_malawi", "1F1F2-1F1FC"},
    {"flagge_mexiko", "1F1F2-1F1FD"},
    {"flagge_malaysia", "1F1F2-1F1FE"},
    {"flagge_mosambik", "1F1F2-1F1FF"},
    {"flagge_namibia", "1F1F3-1F1E6"},
    {"flagge_neukaledonien", "1F1F3-1F1E8"},
    {"flagge_niger", "1F1F3-1F1EA"},
    {"flagge_norfolkinsel", "1F1F3-1F1EB"},
    {"flagge_nigeria", "1F1F3-1F1EC"},
    {"flagge_nicaragua", "1F1F3-1F1EE"},
    {"flagge_niederlande", "1F1F3-1F1F1"},
    {"flagge_norwegen", "1F1F3-1F1F4"},
    {"flagge_nepal", "1F1F3-1F1F5"},
    {"flagge_nauru", "1F1F3-1F1F7"},
    {"flagge_niue", "1F1F3-1F1FA"},
    {"flagge_neuseeland", "1F1F3-1F1FF"},
    {"flagge_oman", "1F1F4-1F1F2"},
    {"flagge_panama", "1F1F5-1F1E6"},
    {"flagge_peru", "1F1F5-1F1EA"},
    {"flagge_franzosisch_polynesien", "1F1F5-1F1EB"},
    {"flagge_papua_neuguinea", "1F1F5-1F1EC"},
    {"flagge_philippinen", "1F1F5-1F1ED"},
    {"flagge_pakistan", "1F1F5-1F1F0"},
    {"flagge_polen", "1F1F5-1F1F1"},
    {"flagge_st_pierre_und_miquelon", "1F1F5-1F1F2"},
    {"flagge_pitcairninseln", "1F1F5-1F1F3"},
    {"flagge_puerto_rico", "1F1F5-1F1F7"},
    {"flagge_palastinensische_autonomiegebiete", "1F1F5-1F1F8"},
    {"flagge_portugal", "1F1F5-1F1F9"},
    {"flagge_palau", "1F1F5-1F1FC"},
    {"flagge_paraguay", "1F1F5-1F1FE"},
    {"flagge_katar", "1F1F6-1F1E6"},
    {"flagge_reunion", "1F1F7-1F1EA"},
    {"flagge_rumanien", "1F1F7-1F1F4"},
    {"flagge_serbien", "1F1F7-1F1F8"},
    {"flagge_russland", "1F1F7-1F1FA"},
    {"flagge_ruanda", "1F1F7-1F1FC"},
    {"flagge_saudi_arabien", "1F1F8-1F1E6"},
    {"flagge_salomonen", "1F1F8-1F1E7"},
    {"flagge_seychellen", "1F1F8-1F1E8"},
    {"flagge_sudan", "1F1F8-1F1E9"},
    {"flagge_schweden", "1F1F8-1F1EA"},
    {"flagge_singapur", "1F1F8-1F1EC"},
    {"flagge_st_helena", "1F1F8-1F1ED"},
    {"flagge_slowenien", "1F1F8-1F1EE"},
    {"flagge_spitzbergen_und_jan_mayen", "1F1F8-1F1EF"},
    {"flagge_slowakei", "1F1F8-1F1F0"},
    {"flagge_sierra_leone", "1F1F8-1F1F1"},
    {"flagge_san_marino", "1F1F8-1F1F2"},
    {"flagge_senegal", "1F1F8-1F1F3"},
    {"flagge_somalia", "1F1F8-1F1F4"},
    {"flagge_suriname", "1F1F8-1F1F7"},
    {"flagge_sudsudan", "1F1F8-1F1F8"},
    {"flagge_sao_tome_und_principe", "1F1F8-1F1F9"},
    {"flagge_el_salvador", "1F1F8-1F1FB"},
    {"flagge_sint_maarten", "1F1F8-1F1FD"},
    {"flagge_syrien", "1F1F8-1F1FE"},
    {"flagge_eswatini", "1F1F8-1F1FF"},
    {"flagge_tristan_da_cunha", "1F1F9-1F1E6"},
    {"flagge_turks_und_caicosinseln", "1F1F9-1F1E8"},
    {"flagge_tschad", "1F1F9-1F1E9"},
    {"flagge_franzosische_sud_und_antarktisgebiete", "1F1F9-1F1EB"},
    {"flagge_togo", "1F1F9-1F1EC"},
    {"flagge_thailand", "1F1F9-1F1ED"},
    {"flagge_tadschikistan", "1F1F9-1F1EF"},
    {"flagge_tokelau", "1F1F9-1F1F0"},
    {"flagge_timor_leste", "1F1F9-1F1F1"},
    {"flagge_turkmenistan", "1F1F9-1F1F2"},
    {"flagge_tunesien", "1F1F9-1F1F3"},
    {"flagge_tonga", "1F1F9-1F1F4"},
    {"flagge_turkei", "1F1F9-1F1F7"},
    {"flagge_trinidad_und_tobago", "1F1F9-1F1F9"},
    {"flagge_tuvalu", "1F1F9-1F1FB"},
    {"flagge_taiwan", "1F1F9-1F1FC"},
    {"flagge_tansania", "1F1F9-1F1FF"},
    {"flagge_ukraine", "1F1FA-1F1E6"},
    {"flagge_uganda", "1F1FA-1F1EC"},
    {"flagge_amerikanische_uberseeinseln", "1F1FA-1F1F2"},
    {"flagge_vereinte_nationen", "1F1FA-1F1F3"},
    {"flagge_vereinigte_staaten", "1F1FA-1F1F8"},
    {"flagge_uruguay", "1F1FA-1F1FE"},
    {"flagge_usbekistan", "1F1FA-1F1FF"},
    {"flagge_vatikanstadt", "1F1FB-1F1E6"},
    {"flagge_st_vincent_und_die_grenadinen", "1F1FB-1F1E8"},
    {"flagge_venezuela", "1F1FB-1F1EA"},
    {"flagge_britische_jungferninseln", "1F1FB-1F1EC"},
    {"flagge_amerikanische_jungferninseln", "1F1FB-1F1EE"},
    {"flagge_vietnam", "1F1FB-1F1F3"},
    {"flagge_vanuatu", "1F1FB-1F1FA"},
    {"flagge_wallis_und_futuna", "1F1FC-1F1EB"},
    {"flagge_samoa", "1F1FC-1F1F8"},
    {"flagge_kosovo", "1F1FD-1F1F0"},
    {"flagge_jemen", "1F1FE-1F1EA"},
    {"flagge_mayotte", "1F1FE-1F1F9"},
    {"flagge_sudafrika", "1F1FF-1F1E6"},
    {"flagge_sambia", "1F1FF-1F1F2"},
    {"flagge_simbabwe", "1F1FF-1F1FC"},
    {"flagge_england", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"flagge_schottland", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"flagge_wales", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"liebespaar_mann_mann", "1F468-200D-2764-FE0F-200D-1F468"},
    {"liebespaar_mann_mann_farbe1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"liebespaar_mann_mann_farbe1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"liebespaar_mann_mann_farbe1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"liebespaar_mann_mann_farbe1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"liebespaar_mann_mann_farbe1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"liebespaar_mann_mann_farbe2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"liebespaar_mann_mann_farbe2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"liebespaar_mann_mann_farbe2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"liebespaar_mann_mann_farbe2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"liebespaar_mann_mann_farbe2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"liebespaar_mann_mann_farbe3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"liebespaar_mann_mann_farbe3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"liebespaar_mann_mann_farbe3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"liebespaar_mann_mann_farbe3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"liebespaar_mann_mann_farbe3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"liebespaar_mann_mann_farbe4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"liebespaar_mann_mann_farbe4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"liebespaar_mann_mann_farbe4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"liebespaar_mann_mann_farbe4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"liebespaar_mann_mann_farbe4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"liebespaar_mann_mann_farbe5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"liebespaar_mann_mann_farbe5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"liebespaar_mann_mann_farbe5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"liebespaar_mann_mann_farbe5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"liebespaar_mann_mann_farbe5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"sich_kussendes_paar_mann_mann", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"sich_kussendes_paar_mann_mann_farbe1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_kussendes_paar_mann_mann_farbe1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_kussendes_paar_mann_mann_farbe1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_kussendes_paar_mann_mann_farbe1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_kussendes_paar_mann_mann_farbe1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_kussendes_paar_mann_mann_farbe2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_kussendes_paar_mann_mann_farbe2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_kussendes_paar_mann_mann_farbe2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_kussendes_paar_mann_mann_farbe2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_kussendes_paar_mann_mann_farbe2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_kussendes_paar_mann_mann_farbe3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_kussendes_paar_mann_mann_farbe3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_kussendes_paar_mann_mann_farbe3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_kussendes_paar_mann_mann_farbe3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_kussendes_paar_mann_mann_farbe3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_kussendes_paar_mann_mann_farbe4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_kussendes_paar_mann_mann_farbe4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_kussendes_paar_mann_mann_farbe4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_kussendes_paar_mann_mann_farbe4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_kussendes_paar_mann_mann_farbe4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_kussendes_paar_mann_mann_farbe5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_kussendes_paar_mann_mann_farbe5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_kussendes_paar_mann_mann_farbe5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_kussendes_paar_mann_mann_farbe5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_kussendes_paar_mann_mann_farbe5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"familie_mann_junge", "1F468-200D-1F466"},
    {"familie_mann_junge_und_junge", "1F468-200D-1F466-200D-1F466"},
    {"familie_mann_madchen", "1F468-200D-1F467"},
    {"familie_mann_madchen_und_junge", "1F468-200D-1F467-200D-1F466"},
    {"familie_mann_madchen_und_madchen", "1F468-200D-1F467-200D-1F467"},
    {"familie_mann_mann_und_junge", "1F468-200D-1F468-200D-1F466"},
    {"familie_mann_mann_junge_und_junge", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"familie_mann_mann_und_madchen", "1F468-200D-1F468-200D-1F467"},
    {"familie_mann_mann_madchen_und_junge", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"familie_mann_mann_madchen_und_madchen", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"familie_mann_frau_und_junge", "1F468-200D-1F469-200D-1F466"},
    {"familie_mann_frau_junge_und_junge", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"familie_mann_frau_und_madchen", "1F468-200D-1F469-200D-1F467"},
    {"familie_mann_frau_madchen_und_junge", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"familie_mann_frau_madchen_und_madchen", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"liebespaar_frau_mann", "1F469-200D-2764-FE0F-200D-1F468"},
    {"liebespaar_frau_mann_farbe1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"liebespaar_frau_mann_farbe1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"liebespaar_frau_mann_farbe1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"liebespaar_frau_mann_farbe1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"liebespaar_frau_mann_farbe1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"liebespaar_frau_mann_farbe2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"liebespaar_frau_mann_farbe2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"liebespaar_frau_mann_farbe2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"liebespaar_frau_mann_farbe2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"liebespaar_frau_mann_farbe2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"liebespaar_frau_mann_farbe3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"liebespaar_frau_mann_farbe3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"liebespaar_frau_mann_farbe3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"liebespaar_frau_mann_farbe3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"liebespaar_frau_mann_farbe3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"liebespaar_frau_mann_farbe4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"liebespaar_frau_mann_farbe4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"liebespaar_frau_mann_farbe4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"liebespaar_frau_mann_farbe4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"liebespaar_frau_mann_farbe4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"liebespaar_frau_mann_farbe5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"liebespaar_frau_mann_farbe5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"liebespaar_frau_mann_farbe5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"liebespaar_frau_mann_farbe5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"liebespaar_frau_mann_farbe5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"liebespaar_frau_frau", "1F469-200D-2764-FE0F-200D-1F469"},
    {"liebespaar_frau_frau_farbe1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"liebespaar_frau_frau_farbe1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"liebespaar_frau_frau_farbe1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"liebespaar_frau_frau_farbe1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"liebespaar_frau_frau_farbe1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"liebespaar_frau_frau_farbe2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"liebespaar_frau_frau_farbe2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"liebespaar_frau_frau_farbe2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"liebespaar_frau_frau_farbe2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"liebespaar_frau_frau_farbe2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"liebespaar_frau_frau_farbe3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"liebespaar_frau_frau_farbe3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"liebespaar_frau_frau_farbe3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"liebespaar_frau_frau_farbe3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"liebespaar_frau_frau_farbe3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"liebespaar_frau_frau_farbe4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"liebespaar_frau_frau_farbe4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"liebespaar_frau_frau_farbe4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"liebespaar_frau_frau_farbe4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"liebespaar_frau_frau_farbe4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"liebespaar_frau_frau_farbe5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"liebespaar_frau_frau_farbe5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"liebespaar_frau_frau_farbe5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"liebespaar_frau_frau_farbe5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"liebespaar_frau_frau_farbe5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"sich_kussendes_paar_frau_mann", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"sich_kussendes_paar_frau_mann_farbe1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_kussendes_paar_frau_mann_farbe1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_kussendes_paar_frau_mann_farbe1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_kussendes_paar_frau_mann_farbe1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_kussendes_paar_frau_mann_farbe1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_kussendes_paar_frau_mann_farbe2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_kussendes_paar_frau_mann_farbe2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_kussendes_paar_frau_mann_farbe2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_kussendes_paar_frau_mann_farbe2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_kussendes_paar_frau_mann_farbe2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_kussendes_paar_frau_mann_farbe3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_kussendes_paar_frau_mann_farbe3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_kussendes_paar_frau_mann_farbe3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_kussendes_paar_frau_mann_farbe3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_kussendes_paar_frau_mann_farbe3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_kussendes_paar_frau_mann_farbe4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_kussendes_paar_frau_mann_farbe4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_kussendes_paar_frau_mann_farbe4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_kussendes_paar_frau_mann_farbe4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_kussendes_paar_frau_mann_farbe4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_kussendes_paar_frau_mann_farbe5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_kussendes_paar_frau_mann_farbe5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_kussendes_paar_frau_mann_farbe5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_kussendes_paar_frau_mann_farbe5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_kussendes_paar_frau_mann_farbe5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_kussendes_paar_frau_frau", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"sich_kussendes_paar_frau_frau_farbe1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"sich_kussendes_paar_frau_frau_farbe1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"sich_kussendes_paar_frau_frau_farbe1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"sich_kussendes_paar_frau_frau_farbe1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"sich_kussendes_paar_frau_frau_farbe1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"sich_kussendes_paar_frau_frau_farbe2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"sich_kussendes_paar_frau_frau_farbe2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"sich_kussendes_paar_frau_frau_farbe2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"sich_kussendes_paar_frau_frau_farbe2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"sich_kussendes_paar_frau_frau_farbe2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"sich_kussendes_paar_frau_frau_farbe3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"sich_kussendes_paar_frau_frau_farbe3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"sich_kussendes_paar_frau_frau_farbe3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"sich_kussendes_paar_frau_frau_farbe3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"sich_kussendes_paar_frau_frau_farbe3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"sich_kussendes_paar_frau_frau_farbe4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"sich_kussendes_paar_frau_frau_farbe4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"sich_kussendes_paar_frau_frau_farbe4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"sich_kussendes_paar_frau_frau_farbe4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"sich_kussendes_paar_frau_frau_farbe4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"sich_kussendes_paar_frau_frau_farbe5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"sich_kussendes_paar_frau_frau_farbe5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"sich_kussendes_paar_frau_frau_farbe5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"sich_kussendes_paar_frau_frau_farbe5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"sich_kussendes_paar_frau_frau_farbe5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"familie_frau_junge", "1F469-200D-1F466"},
    {"familie_frau_junge_und_junge", "1F469-200D-1F466-200D-1F466"},
    {"familie_frau_madchen", "1F469-200D-1F467"},
    {"familie_frau_madchen_und_junge", "1F469-200D-1F467-200D-1F466"},
    {"familie_frau_madchen_und_madchen", "1F469-200D-1F467-200D-1F467"},
    {"familie_frau_frau_und_junge", "1F469-200D-1F469-200D-1F466"},
    {"familie_frau_frau_junge_und_junge", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"familie_frau_frau_und_madchen", "1F469-200D-1F469-200D-1F467"},
    {"familie_frau_frau_madchen_und_junge", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"familie_frau_frau_madchen_und_madchen", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"sich_an_den_handen_haltende_personen", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"sich_an_den_handen_haltende_personen_farbe1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"sich_an_den_handen_haltende_personen_farbe1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"sich_an_den_handen_haltende_personen_farbe1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"sich_an_den_handen_haltende_personen_farbe1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"sich_an_den_handen_haltende_personen_farbe1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"sich_an_den_handen_haltende_personen_farbe2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"sich_an_den_handen_haltende_personen_farbe2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"sich_an_den_handen_haltende_personen_farbe2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"sich_an_den_handen_haltende_personen_farbe2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"sich_an_den_handen_haltende_personen_farbe2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"sich_an_den_handen_haltende_personen_farbe3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"sich_an_den_handen_haltende_personen_farbe3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"sich_an_den_handen_haltende_personen_farbe3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"sich_an_den_handen_haltende_personen_farbe3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"sich_an_den_handen_haltende_personen_farbe3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"sich_an_den_handen_haltende_personen_farbe4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"sich_an_den_handen_haltende_personen_farbe4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"sich_an_den_handen_haltende_personen_farbe4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"sich_an_den_handen_haltende_personen_farbe4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"sich_an_den_handen_haltende_personen_farbe4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"sich_an_den_handen_haltende_personen_farbe5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"sich_an_den_handen_haltende_personen_farbe5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"sich_an_den_handen_haltende_personen_farbe5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"sich_an_den_handen_haltende_personen_farbe5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"sich_an_den_handen_haltende_personen_farbe5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"familie_2_erwachsene_1_kind", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"familie_2_erwachsene_2_kinder", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"familie_1_erwachsener_1_kind", "1F9D1-200D-1F9D2"},
    {"familie_1_erwachsener_2_kinder", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"laufende_person_nach_rechts", "1F3C3-200D-27A1-FE0F"},
    {"laufende_person_nach_rechts_farbe1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"laufende_person_nach_rechts_farbe2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"laufende_person_nach_rechts_farbe3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"laufende_person_nach_rechts_farbe4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"laufende_person_nach_rechts_farbe5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"arzt", "1F468-200D-2695-FE0F"},
    {"arzt_farbe1", "1F468-1F3FB-200D-2695-FE0F"},
    {"arzt_farbe2", "1F468-1F3FC-200D-2695-FE0F"},
    {"arzt_farbe3", "1F468-1F3FD-200D-2695-FE0F"},
    {"arzt_farbe4", "1F468-1F3FE-200D-2695-FE0F"},
    {"arzt_farbe5", "1F468-1F3FF-200D-2695-FE0F"},
    {"richter", "1F468-200D-2696-FE0F"},
    {"richter_farbe1", "1F468-1F3FB-200D-2696-FE0F"},
    {"richter_farbe2", "1F468-1F3FC-200D-2696-FE0F"},
    {"richter_farbe3", "1F468-1F3FD-200D-2696-FE0F"},
    {"richter_farbe4", "1F468-1F3FE-200D-2696-FE0F"},
    {"richter_farbe5", "1F468-1F3FF-200D-2696-FE0F"},
    {"pilot", "1F468-200D-2708-FE0F"},
    {"pilot_farbe1", "1F468-1F3FB-200D-2708-FE0F"},
    {"pilot_farbe2", "1F468-1F3FC-200D-2708-FE0F"},
    {"pilot_farbe3", "1F468-1F3FD-200D-2708-FE0F"},
    {"pilot_farbe4", "1F468-1F3FE-200D-2708-FE0F"},
    {"pilot_farbe5", "1F468-1F3FF-200D-2708-FE0F"},
    {"bauer", "1F468-200D-1F33E"},
    {"bauer_farbe1", "1F468-1F3FB-200D-1F33E"},
    {"bauer_farbe2", "1F468-1F3FC-200D-1F33E"},
    {"bauer_farbe3", "1F468-1F3FD-200D-1F33E"},
    {"bauer_farbe4", "1F468-1F3FE-200D-1F33E"},
    {"bauer_farbe5", "1F468-1F3FF-200D-1F33E"},
    {"koch", "1F468-200D-1F373"},
    {"koch_farbe1", "1F468-1F3FB-200D-1F373"},
    {"koch_farbe2", "1F468-1F3FC-200D-1F373"},
    {"koch_farbe3", "1F468-1F3FD-200D-1F373"},
    {"koch_farbe4", "1F468-1F3FE-200D-1F373"},
    {"koch_farbe5", "1F468-1F3FF-200D-1F373"},
    {"mann_der_baby_die_flasche_gibt", "1F468-200D-1F37C"},
    {"mann_der_baby_die_flasche_gibt_farbe1", "1F468-1F3FB-200D-1F37C"},
    {"mann_der_baby_die_flasche_gibt_farbe2", "1F468-1F3FC-200D-1F37C"},
    {"mann_der_baby_die_flasche_gibt_farbe3", "1F468-1F3FD-200D-1F37C"},
    {"mann_der_baby_die_flasche_gibt_farbe4", "1F468-1F3FE-200D-1F37C"},
    {"mann_der_baby_die_flasche_gibt_farbe5", "1F468-1F3FF-200D-1F37C"},
    {"student", "1F468-200D-1F393"},
    {"student_farbe1", "1F468-1F3FB-200D-1F393"},
    {"student_farbe2", "1F468-1F3FC-200D-1F393"},
    {"student_farbe3", "1F468-1F3FD-200D-1F393"},
    {"student_farbe4", "1F468-1F3FE-200D-1F393"},
    {"student_farbe5", "1F468-1F3FF-200D-1F393"},
    {"sanger", "1F468-200D-1F3A4"},
    {"sanger_farbe1", "1F468-1F3FB-200D-1F3A4"},
    {"sanger_farbe2", "1F468-1F3FC-200D-1F3A4"},
    {"sanger_farbe3", "1F468-1F3FD-200D-1F3A4"},
    {"sanger_farbe4", "1F468-1F3FE-200D-1F3A4"},
    {"sanger_farbe5", "1F468-1F3FF-200D-1F3A4"},
    {"kunstler", "1F468-200D-1F3A8"},
    {"kunstler_farbe1", "1F468-1F3FB-200D-1F3A8"},
    {"kunstler_farbe2", "1F468-1F3FC-200D-1F3A8"},
    {"kunstler_farbe3", "1F468-1F3FD-200D-1F3A8"},
    {"kunstler_farbe4", "1F468-1F3FE-200D-1F3A8"},
    {"kunstler_farbe5", "1F468-1F3FF-200D-1F3A8"},
    {"lehrer", "1F468-200D-1F3EB"},
    {"lehrer_farbe1", "1F468-1F3FB-200D-1F3EB"},
    {"lehrer_farbe2", "1F468-1F3FC-200D-1F3EB"},
    {"lehrer_farbe3", "1F468-1F3FD-200D-1F3EB"},
    {"lehrer_farbe4", "1F468-1F3FE-200D-1F3EB"},
    {"lehrer_farbe5", "1F468-1F3FF-200D-1F3EB"},
    {"fabrikarbeiter", "1F468-200D-1F3ED"},
    {"fabrikarbeiter_farbe1", "1F468-1F3FB-200D-1F3ED"},
    {"fabrikarbeiter_farbe2", "1F468-1F3FC-200D-1F3ED"},
    {"fabrikarbeiter_farbe3", "1F468-1F3FD-200D-1F3ED"},
    {"fabrikarbeiter_farbe4", "1F468-1F3FE-200D-1F3ED"},
    {"fabrikarbeiter_farbe5", "1F468-1F3FF-200D-1F3ED"},
    {"it_experte", "1F468-200D-1F4BB"},
    {"it_experte_farbe1", "1F468-1F3FB-200D-1F4BB"},
    {"it_experte_farbe2", "1F468-1F3FC-200D-1F4BB"},
    {"it_experte_farbe3", "1F468-1F3FD-200D-1F4BB"},
    {"it_experte_farbe4", "1F468-1F3FE-200D-1F4BB"},
    {"it_experte_farbe5", "1F468-1F3FF-200D-1F4BB"},
    {"buroangestellter", "1F9D1-200D-1F4BC"},
    {"buroangestellter_farbe1", "1F9D1-1F3FB-200D-1F4BC"},
    {"buroangestellter_farbe2", "1F9D1-1F3FC-200D-1F4BC"},
    {"buroangestellter_farbe3", "1F9D1-1F3FD-200D-1F4BC"},
    {"buroangestellter_farbe4", "1F9D1-1F3FE-200D-1F4BC"},
    {"buroangestellter_farbe5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mechaniker", "1F468-200D-1F527"},
    {"mechaniker_farbe1", "1F468-1F3FB-200D-1F527"},
    {"mechaniker_farbe2", "1F468-1F3FC-200D-1F527"},
    {"mechaniker_farbe3", "1F468-1F3FD-200D-1F527"},
    {"mechaniker_farbe4", "1F468-1F3FE-200D-1F527"},
    {"mechaniker_farbe5", "1F468-1F3FF-200D-1F527"},
    {"wissenschaftler", "1F468-200D-1F52C"},
    {"wissenschaftler_farbe1", "1F468-1F3FB-200D-1F52C"},
    {"wissenschaftler_farbe2", "1F468-1F3FC-200D-1F52C"},
    {"wissenschaftler_farbe3", "1F468-1F3FD-200D-1F52C"},
    {"wissenschaftler_farbe4", "1F468-1F3FE-200D-1F52C"},
    {"wissenschaftler_farbe5", "1F468-1F3FF-200D-1F52C"},
    {"astronaut", "1F468-200D-1F680"},
    {"astronaut_farbe1", "1F468-1F3FB-200D-1F680"},
    {"astronaut_farbe2", "1F468-1F3FC-200D-1F680"},
    {"astronaut_farbe3", "1F468-1F3FD-200D-1F680"},
    {"astronaut_farbe4", "1F468-1F3FE-200D-1F680"},
    {"astronaut_farbe5", "1F468-1F3FF-200D-1F680"},
    {"feuerwehrmann", "1F468-200D-1F692"},
    {"feuerwehrmann_farbe1", "1F468-1F3FB-200D-1F692"},
    {"feuerwehrmann_farbe2", "1F468-1F3FC-200D-1F692"},
    {"feuerwehrmann_farbe3", "1F468-1F3FD-200D-1F692"},
    {"feuerwehrmann_farbe4", "1F468-1F3FE-200D-1F692"},
    {"feuerwehrmann_farbe5", "1F468-1F3FF-200D-1F692"},
    {"mann_mit_langstock", "1F468-200D-1F9AF"},
    {"mann_mit_langstock_farbe1", "1F468-1F3FB-200D-1F9AF"},
    {"mann_mit_langstock_farbe2", "1F468-1F3FC-200D-1F9AF"},
    {"mann_mit_langstock_farbe3", "1F468-1F3FD-200D-1F9AF"},
    {"mann_mit_langstock_farbe4", "1F468-1F3FE-200D-1F9AF"},
    {"mann_mit_langstock_farbe5", "1F468-1F3FF-200D-1F9AF"},
    {"mann_mit_langstock_nach_rechts", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_mit_langstock_nach_rechts_farbe1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_mit_langstock_nach_rechts_farbe2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_mit_langstock_nach_rechts_farbe3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_mit_langstock_nach_rechts_farbe4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_mit_langstock_nach_rechts_farbe5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"mann_in_elektrischem_rollstuhl", "1F468-200D-1F9BC"},
    {"mann_in_elektrischem_rollstuhl_farbe1", "1F468-1F3FB-200D-1F9BC"},
    {"mann_in_elektrischem_rollstuhl_farbe2", "1F468-1F3FC-200D-1F9BC"},
    {"mann_in_elektrischem_rollstuhl_farbe3", "1F468-1F3FD-200D-1F9BC"},
    {"mann_in_elektrischem_rollstuhl_farbe4", "1F468-1F3FE-200D-1F9BC"},
    {"mann_in_elektrischem_rollstuhl_farbe5", "1F468-1F3FF-200D-1F9BC"},
    {"mann_in_elektrischem_rollstuhl_nach_rechts", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_in_elektrischem_rollstuhl_nach_rechts_farbe1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_in_elektrischem_rollstuhl_nach_rechts_farbe2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_in_elektrischem_rollstuhl_nach_rechts_farbe3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_in_elektrischem_rollstuhl_nach_rechts_farbe4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_in_elektrischem_rollstuhl_nach_rechts_farbe5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mann_in_manuellem_rollstuhl", "1F468-200D-1F9BD"},
    {"mann_in_manuellem_rollstuhl_farbe1", "1F468-1F3FB-200D-1F9BD"},
    {"mann_in_manuellem_rollstuhl_farbe2", "1F468-1F3FC-200D-1F9BD"},
    {"mann_in_manuellem_rollstuhl_farbe3", "1F468-1F3FD-200D-1F9BD"},
    {"mann_in_manuellem_rollstuhl_farbe4", "1F468-1F3FE-200D-1F9BD"},
    {"mann_in_manuellem_rollstuhl_farbe5", "1F468-1F3FF-200D-1F9BD"},
    {"mann_in_manuellem_rollstuhl_nach_rechts", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_in_manuellem_rollstuhl_nach_rechts_farbe1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_in_manuellem_rollstuhl_nach_rechts_farbe2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_in_manuellem_rollstuhl_nach_rechts_farbe3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_in_manuellem_rollstuhl_nach_rechts_farbe4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"mann_in_manuellem_rollstuhl_nach_rechts_farbe5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"arztin", "1F469-200D-2695-FE0F"},
    {"arztin_farbe1", "1F469-1F3FB-200D-2695-FE0F"},
    {"arztin_farbe2", "1F469-1F3FC-200D-2695-FE0F"},
    {"arztin_farbe3", "1F469-1F3FD-200D-2695-FE0F"},
    {"arztin_farbe4", "1F469-1F3FE-200D-2695-FE0F"},
    {"arztin_farbe5", "1F469-1F3FF-200D-2695-FE0F"},
    {"richterin", "1F9D1-200D-2696-FE0F"},
    {"richterin_farbe1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"richterin_farbe2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"richterin_farbe3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"richterin_farbe4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"richterin_farbe5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"pilotin", "1F9D1-200D-2708-FE0F"},
    {"pilotin_farbe1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"pilotin_farbe2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"pilotin_farbe3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"pilotin_farbe4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"pilotin_farbe5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"bauerin", "1F469-200D-1F33E"},
    {"bauerin_farbe1", "1F469-1F3FB-200D-1F33E"},
    {"bauerin_farbe2", "1F469-1F3FC-200D-1F33E"},
    {"bauerin_farbe3", "1F469-1F3FD-200D-1F33E"},
    {"bauerin_farbe4", "1F469-1F3FE-200D-1F33E"},
    {"bauerin_farbe5", "1F469-1F3FF-200D-1F33E"},
    {"kochin", "1F469-200D-1F373"},
    {"kochin_farbe1", "1F469-1F3FB-200D-1F373"},
    {"kochin_farbe2", "1F469-1F3FC-200D-1F373"},
    {"kochin_farbe3", "1F469-1F3FD-200D-1F373"},
    {"kochin_farbe4", "1F469-1F3FE-200D-1F373"},
    {"kochin_farbe5", "1F469-1F3FF-200D-1F373"},
    {"frau_die_baby_die_flasche_gibt", "1F469-200D-1F37C"},
    {"frau_die_baby_die_flasche_gibt_farbe1", "1F469-1F3FB-200D-1F37C"},
    {"frau_die_baby_die_flasche_gibt_farbe2", "1F469-1F3FC-200D-1F37C"},
    {"frau_die_baby_die_flasche_gibt_farbe3", "1F469-1F3FD-200D-1F37C"},
    {"frau_die_baby_die_flasche_gibt_farbe4", "1F469-1F3FE-200D-1F37C"},
    {"frau_die_baby_die_flasche_gibt_farbe5", "1F469-1F3FF-200D-1F37C"},
    {"studentin", "1F9D1-200D-1F393"},
    {"studentin_farbe1", "1F9D1-1F3FB-200D-1F393"},
    {"studentin_farbe2", "1F9D1-1F3FC-200D-1F393"},
    {"studentin_farbe3", "1F9D1-1F3FD-200D-1F393"},
    {"studentin_farbe4", "1F9D1-1F3FE-200D-1F393"},
    {"studentin_farbe5", "1F9D1-1F3FF-200D-1F393"},
    {"sangerin", "1F9D1-200D-1F3A4"},
    {"sangerin_farbe1", "1F9D1-1F3FB-200D-1F3A4"},
    {"sangerin_farbe2", "1F9D1-1F3FC-200D-1F3A4"},
    {"sangerin_farbe3", "1F9D1-1F3FD-200D-1F3A4"},
    {"sangerin_farbe4", "1F9D1-1F3FE-200D-1F3A4"},
    {"sangerin_farbe5", "1F9D1-1F3FF-200D-1F3A4"},
    {"kunstlerin", "1F9D1-200D-1F3A8"},
    {"kunstlerin_farbe1", "1F9D1-1F3FB-200D-1F3A8"},
    {"kunstlerin_farbe2", "1F9D1-1F3FC-200D-1F3A8"},
    {"kunstlerin_farbe3", "1F9D1-1F3FD-200D-1F3A8"},
    {"kunstlerin_farbe4", "1F9D1-1F3FE-200D-1F3A8"},
    {"kunstlerin_farbe5", "1F9D1-1F3FF-200D-1F3A8"},
    {"lehrerin", "1F9D1-200D-1F3EB"},
    {"lehrerin_farbe1", "1F9D1-1F3FB-200D-1F3EB"},
    {"lehrerin_farbe2", "1F9D1-1F3FC-200D-1F3EB"},
    {"lehrerin_farbe3", "1F9D1-1F3FD-200D-1F3EB"},
    {"lehrerin_farbe4", "1F9D1-1F3FE-200D-1F3EB"},
    {"lehrerin_farbe5", "1F9D1-1F3FF-200D-1F3EB"},
    {"fabrikarbeiterin", "1F9D1-200D-1F3ED"},
    {"fabrikarbeiterin_farbe1", "1F9D1-1F3FB-200D-1F3ED"},
    {"fabrikarbeiterin_farbe2", "1F9D1-1F3FC-200D-1F3ED"},
    {"fabrikarbeiterin_farbe3", "1F9D1-1F3FD-200D-1F3ED"},
    {"fabrikarbeiterin_farbe4", "1F9D1-1F3FE-200D-1F3ED"},
    {"fabrikarbeiterin_farbe5", "1F9D1-1F3FF-200D-1F3ED"},
    {"it_expertin", "1F469-200D-1F4BB"},
    {"it_expertin_farbe1", "1F469-1F3FB-200D-1F4BB"},
    {"it_expertin_farbe2", "1F469-1F3FC-200D-1F4BB"},
    {"it_expertin_farbe3", "1F469-1F3FD-200D-1F4BB"},
    {"it_expertin_farbe4", "1F469-1F3FE-200D-1F4BB"},
    {"it_expertin_farbe5", "1F469-1F3FF-200D-1F4BB"},
    {"buroangestellte", "1F469-200D-1F4BC"},
    {"buroangestellte_farbe1", "1F469-1F3FB-200D-1F4BC"},
    {"buroangestellte_farbe2", "1F469-1F3FC-200D-1F4BC"},
    {"buroangestellte_farbe3", "1F469-1F3FD-200D-1F4BC"},
    {"buroangestellte_farbe4", "1F469-1F3FE-200D-1F4BC"},
    {"buroangestellte_farbe5", "1F469-1F3FF-200D-1F4BC"},
    {"mechanikerin", "1F9D1-200D-1F527"},
    {"mechanikerin_farbe1", "1F9D1-1F3FB-200D-1F527"},
    {"mechanikerin_farbe2", "1F9D1-1F3FC-200D-1F527"},
    {"mechanikerin_farbe3", "1F9D1-1F3FD-200D-1F527"},
    {"mechanikerin_farbe4", "1F9D1-1F3FE-200D-1F527"},
    {"mechanikerin_farbe5", "1F9D1-1F3FF-200D-1F527"},
    {"wissenschaftlerin", "1F9D1-200D-1F52C"},
    {"wissenschaftlerin_farbe1", "1F9D1-1F3FB-200D-1F52C"},
    {"wissenschaftlerin_farbe2", "1F9D1-1F3FC-200D-1F52C"},
    {"wissenschaftlerin_farbe3", "1F9D1-1F3FD-200D-1F52C"},
    {"wissenschaftlerin_farbe4", "1F9D1-1F3FE-200D-1F52C"},
    {"wissenschaftlerin_farbe5", "1F9D1-1F3FF-200D-1F52C"},
    {"astronautin", "1F9D1-200D-1F680"},
    {"astronautin_farbe1", "1F9D1-1F3FB-200D-1F680"},
    {"astronautin_farbe2", "1F9D1-1F3FC-200D-1F680"},
    {"astronautin_farbe3", "1F9D1-1F3FD-200D-1F680"},
    {"astronautin_farbe4", "1F9D1-1F3FE-200D-1F680"},
    {"astronautin_farbe5", "1F9D1-1F3FF-200D-1F680"},
    {"feuerwehrfrau", "1F469-200D-1F692"},
    {"feuerwehrfrau_farbe1", "1F469-1F3FB-200D-1F692"},
    {"feuerwehrfrau_farbe2", "1F469-1F3FC-200D-1F692"},
    {"feuerwehrfrau_farbe3", "1F469-1F3FD-200D-1F692"},
    {"feuerwehrfrau_farbe4", "1F469-1F3FE-200D-1F692"},
    {"feuerwehrfrau_farbe5", "1F469-1F3FF-200D-1F692"},
    {"frau_mit_langstock", "1F469-200D-1F9AF"},
    {"frau_mit_langstock_farbe1", "1F469-1F3FB-200D-1F9AF"},
    {"frau_mit_langstock_farbe2", "1F469-1F3FC-200D-1F9AF"},
    {"frau_mit_langstock_farbe3", "1F469-1F3FD-200D-1F9AF"},
    {"frau_mit_langstock_farbe4", "1F469-1F3FE-200D-1F9AF"},
    {"frau_mit_langstock_farbe5", "1F469-1F3FF-200D-1F9AF"},
    {"frau_mit_langstock_nach_rechts", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"frau_mit_langstock_nach_rechts_farbe1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"frau_mit_langstock_nach_rechts_farbe2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"frau_mit_langstock_nach_rechts_farbe3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"frau_mit_langstock_nach_rechts_farbe4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"frau_mit_langstock_nach_rechts_farbe5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"frau_in_elektrischem_rollstuhl", "1F469-200D-1F9BC"},
    {"frau_in_elektrischem_rollstuhl_farbe1", "1F469-1F3FB-200D-1F9BC"},
    {"frau_in_elektrischem_rollstuhl_farbe2", "1F469-1F3FC-200D-1F9BC"},
    {"frau_in_elektrischem_rollstuhl_farbe3", "1F469-1F3FD-200D-1F9BC"},
    {"frau_in_elektrischem_rollstuhl_farbe4", "1F469-1F3FE-200D-1F9BC"},
    {"frau_in_elektrischem_rollstuhl_farbe5", "1F469-1F3FF-200D-1F9BC"},
    {"frau_in_elektrischem_rollstuhl_nach_rechts", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"frau_in_elektrischem_rollstuhl_nach_rechts_farbe1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"frau_in_elektrischem_rollstuhl_nach_rechts_farbe2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"frau_in_elektrischem_rollstuhl_nach_rechts_farbe3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"frau_in_elektrischem_rollstuhl_nach_rechts_farbe4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"frau_in_elektrischem_rollstuhl_nach_rechts_farbe5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"frau_in_manuellem_rollstuhl", "1F469-200D-1F9BD"},
    {"frau_in_manuellem_rollstuhl_farbe1", "1F469-1F3FB-200D-1F9BD"},
    {"frau_in_manuellem_rollstuhl_farbe2", "1F469-1F3FC-200D-1F9BD"},
    {"frau_in_manuellem_rollstuhl_farbe3", "1F469-1F3FD-200D-1F9BD"},
    {"frau_in_manuellem_rollstuhl_farbe4", "1F469-1F3FE-200D-1F9BD"},
    {"frau_in_manuellem_rollstuhl_farbe5", "1F469-1F3FF-200D-1F9BD"},
    {"frau_in_manuellem_rollstuhl_nach_rechts", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"frau_in_manuellem_rollstuhl_nach_rechts_farbe1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"frau_in_manuellem_rollstuhl_nach_rechts_farbe2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"frau_in_manuellem_rollstuhl_nach_rechts_farbe3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"frau_in_manuellem_rollstuhl_nach_rechts_farbe4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"frau_in_manuellem_rollstuhl_nach_rechts_farbe5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"fussgangerin_nach_rechts", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"fussgangerin_nach_rechts_farbe1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"fussgangerin_nach_rechts_farbe2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"fussgangerin_nach_rechts_farbe3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"fussgangerin_nach_rechts_farbe4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"fussgangerin_nach_rechts_farbe5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kniende_person_nach_rechts", "1F9CE-200D-27A1-FE0F"},
    {"kniende_person_nach_rechts_farbe1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"kniende_person_nach_rechts_farbe2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"kniende_person_nach_rechts_farbe3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"kniende_person_nach_rechts_farbe4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"kniende_person_nach_rechts_farbe5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"arzt_arztin", "1F9D1-200D-2695-FE0F"},
    {"arzt_arztin_farbe1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"arzt_arztin_farbe2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"arzt_arztin_farbe3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"arzt_arztin_farbe4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"arzt_arztin_farbe5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"bauer_bauerin", "1F9D1-200D-1F33E"},
    {"bauer_bauerin_farbe1", "1F9D1-1F3FB-200D-1F33E"},
    {"bauer_bauerin_farbe2", "1F9D1-1F3FC-200D-1F33E"},
    {"bauer_bauerin_farbe3", "1F9D1-1F3FD-200D-1F33E"},
    {"bauer_bauerin_farbe4", "1F9D1-1F3FE-200D-1F33E"},
    {"bauer_bauerin_farbe5", "1F9D1-1F3FF-200D-1F33E"},
    {"koch_kochin", "1F9D1-200D-1F373"},
    {"koch_kochin_farbe1", "1F9D1-1F3FB-200D-1F373"},
    {"koch_kochin_farbe2", "1F9D1-1F3FC-200D-1F373"},
    {"koch_kochin_farbe3", "1F9D1-1F3FD-200D-1F373"},
    {"koch_kochin_farbe4", "1F9D1-1F3FE-200D-1F373"},
    {"koch_kochin_farbe5", "1F9D1-1F3FF-200D-1F373"},
    {"person_die_baby_die_flasche_gibt", "1F9D1-200D-1F37C"},
    {"person_die_baby_die_flasche_gibt_farbe1", "1F9D1-1F3FB-200D-1F37C"},
    {"person_die_baby_die_flasche_gibt_farbe2", "1F9D1-1F3FC-200D-1F37C"},
    {"person_die_baby_die_flasche_gibt_farbe3", "1F9D1-1F3FD-200D-1F37C"},
    {"person_die_baby_die_flasche_gibt_farbe4", "1F9D1-1F3FE-200D-1F37C"},
    {"person_die_baby_die_flasche_gibt_farbe5", "1F9D1-1F3FF-200D-1F37C"},
    {"weihnachtsperson", "1F9D1-200D-1F384"},
    {"weihnachtsperson_farbe1", "1F9D1-1F3FB-200D-1F384"},
    {"weihnachtsperson_farbe2", "1F9D1-1F3FC-200D-1F384"},
    {"weihnachtsperson_farbe3", "1F9D1-1F3FD-200D-1F384"},
    {"weihnachtsperson_farbe4", "1F9D1-1F3FE-200D-1F384"},
    {"weihnachtsperson_farbe5", "1F9D1-1F3FF-200D-1F384"},
    {"it_experte_it_expertin", "1F9D1-200D-1F4BB"},
    {"it_experte_it_expertin_farbe1", "1F9D1-1F3FB-200D-1F4BB"},
    {"it_experte_it_expertin_farbe2", "1F9D1-1F3FC-200D-1F4BB"},
    {"it_experte_it_expertin_farbe3", "1F9D1-1F3FD-200D-1F4BB"},
    {"it_experte_it_expertin_farbe4", "1F9D1-1F3FE-200D-1F4BB"},
    {"it_experte_it_expertin_farbe5", "1F9D1-1F3FF-200D-1F4BB"},
    {"feuerwehrmann_frau", "1F9D1-200D-1F692"},
    {"feuerwehrmann_frau_farbe1", "1F9D1-1F3FB-200D-1F692"},
    {"feuerwehrmann_frau_farbe2", "1F9D1-1F3FC-200D-1F692"},
    {"feuerwehrmann_frau_farbe3", "1F9D1-1F3FD-200D-1F692"},
    {"feuerwehrmann_frau_farbe4", "1F9D1-1F3FE-200D-1F692"},
    {"feuerwehrmann_frau_farbe5", "1F9D1-1F3FF-200D-1F692"},
    {"person_mit_langstock", "1F9D1-200D-1F9AF"},
    {"person_mit_langstock_farbe1", "1F9D1-1F3FB-200D-1F9AF"},
    {"person_mit_langstock_farbe2", "1F9D1-1F3FC-200D-1F9AF"},
    {"person_mit_langstock_farbe3", "1F9D1-1F3FD-200D-1F9AF"},
    {"person_mit_langstock_farbe4", "1F9D1-1F3FE-200D-1F9AF"},
    {"person_mit_langstock_farbe5", "1F9D1-1F3FF-200D-1F9AF"},
    {"person_mit_langstock_nach_rechts", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"person_mit_langstock_nach_rechts_farbe1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"person_mit_langstock_nach_rechts_farbe2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"person_mit_langstock_nach_rechts_farbe3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"person_mit_langstock_nach_rechts_farbe4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"person_mit_langstock_nach_rechts_farbe5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"person_in_elektrischem_rollstuhl", "1F9D1-200D-1F9BC"},
    {"person_in_elektrischem_rollstuhl_farbe1", "1F9D1-1F3FB-200D-1F9BC"},
    {"person_in_elektrischem_rollstuhl_farbe2", "1F9D1-1F3FC-200D-1F9BC"},
    {"person_in_elektrischem_rollstuhl_farbe3", "1F9D1-1F3FD-200D-1F9BC"},
    {"person_in_elektrischem_rollstuhl_farbe4", "1F9D1-1F3FE-200D-1F9BC"},
    {"person_in_elektrischem_rollstuhl_farbe5", "1F9D1-1F3FF-200D-1F9BC"},
    {"person_in_elektrischem_rollstuhl_nach_rechts", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"person_in_elektrischem_rollstuhl_nach_rechts_farbe1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"person_in_elektrischem_rollstuhl_nach_rechts_farbe2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"person_in_elektrischem_rollstuhl_nach_rechts_farbe3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"person_in_elektrischem_rollstuhl_nach_rechts_farbe4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"person_in_elektrischem_rollstuhl_nach_rechts_farbe5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"person_in_manuellem_rollstuhl", "1F9D1-200D-1F9BD"},
    {"person_in_manuellem_rollstuhl_farbe1", "1F9D1-1F3FB-200D-1F9BD"},
    {"person_in_manuellem_rollstuhl_farbe2", "1F9D1-1F3FC-200D-1F9BD"},
    {"person_in_manuellem_rollstuhl_farbe3", "1F9D1-1F3FD-200D-1F9BD"},
    {"person_in_manuellem_rollstuhl_farbe4", "1F9D1-1F3FE-200D-1F9BD"},
    {"person_in_manuellem_rollstuhl_farbe5", "1F9D1-1F3FF-200D-1F9BD"},
    {"person_in_manuellem_rollstuhl_nach_rechts", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"person_in_manuellem_rollstuhl_nach_rechts_farbe1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"person_in_manuellem_rollstuhl_nach_rechts_farbe2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"person_in_manuellem_rollstuhl_nach_rechts_farbe3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"person_in_manuellem_rollstuhl_nach_rechts_farbe4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"person_in_manuellem_rollstuhl_nach_rechts_farbe5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"frau_mit_ball", "26F9-FE0F-200D-2640-FE0F"},
    {"frau_mit_ball_farbe1", "26F9-1F3FB-200D-2640-FE0F"},
    {"frau_mit_ball_farbe2", "26F9-1F3FC-200D-2640-FE0F"},
    {"frau_mit_ball_farbe3", "26F9-1F3FD-200D-2640-FE0F"},
    {"frau_mit_ball_farbe4", "26F9-1F3FE-200D-2640-FE0F"},
    {"frau_mit_ball_farbe5", "26F9-1F3FF-200D-2640-FE0F"},
    {"mann_mit_ball", "26F9-FE0F-200D-2642-FE0F"},
    {"mann_mit_ball_farbe1", "26F9-1F3FB-200D-2642-FE0F"},
    {"mann_mit_ball_farbe2", "26F9-1F3FC-200D-2642-FE0F"},
    {"mann_mit_ball_farbe3", "26F9-1F3FD-200D-2642-FE0F"},
    {"mann_mit_ball_farbe4", "26F9-1F3FE-200D-2642-FE0F"},
    {"mann_mit_ball_farbe5", "26F9-1F3FF-200D-2642-FE0F"},
    {"laufende_frau", "1F3C3-200D-2640-FE0F"},
    {"laufende_frau_farbe1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"laufende_frau_farbe2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"laufende_frau_farbe3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"laufende_frau_farbe4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"laufende_frau_farbe5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"laufende_frau_nach_rechts", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"laufende_frau_nach_rechts_farbe1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"laufende_frau_nach_rechts_farbe2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"laufende_frau_nach_rechts_farbe3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"laufende_frau_nach_rechts_farbe4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"laufende_frau_nach_rechts_farbe5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"laufender_mann", "1F3C3-200D-2642-FE0F"},
    {"laufender_mann_farbe1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"laufender_mann_farbe2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"laufender_mann_farbe3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"laufender_mann_farbe4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"laufender_mann_farbe5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"laufender_mann_nach_rechts", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"laufender_mann_nach_rechts_farbe1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"laufender_mann_nach_rechts_farbe2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"laufender_mann_nach_rechts_farbe3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"laufender_mann_nach_rechts_farbe4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"laufender_mann_nach_rechts_farbe5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"surfer", "1F3C4-200D-2642-FE0F"},
    {"surfer_farbe1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"surfer_farbe2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"surfer_farbe3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"surfer_farbe4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"surfer_farbe5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"schwimmer", "1F3CA-200D-2642-FE0F"},
    {"schwimmer_farbe1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"schwimmer_farbe2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"schwimmer_farbe3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"schwimmer_farbe4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"schwimmer_farbe5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"gewichtheber", "1F3CB-FE0F-200D-2642-FE0F"},
    {"gewichtheber_farbe1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"gewichtheber_farbe2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"gewichtheber_farbe3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"gewichtheber_farbe4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"gewichtheber_farbe5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"golfer", "1F3CC-FE0F-200D-2642-FE0F"},
    {"golfer_farbe1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"golfer_farbe2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"golfer_farbe3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"golfer_farbe4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"golfer_farbe5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"polizist", "1F46E-200D-2642-FE0F"},
    {"polizist_farbe1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"polizist_farbe2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"polizist_farbe3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"polizist_farbe4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"polizist_farbe5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"frauen_mit_hasenohren", "1F46F-200D-2640-FE0F"},
    {"frauen_mit_hasenohren_farbe1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"frauen_mit_hasenohren_farbe1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"frauen_mit_hasenohren_farbe1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"frauen_mit_hasenohren_farbe1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"frauen_mit_hasenohren_farbe2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"frauen_mit_hasenohren_farbe2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"frauen_mit_hasenohren_farbe2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"frauen_mit_hasenohren_farbe2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"frauen_mit_hasenohren_farbe3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"frauen_mit_hasenohren_farbe3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"frauen_mit_hasenohren_farbe3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"frauen_mit_hasenohren_farbe3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"frauen_mit_hasenohren_farbe4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"frauen_mit_hasenohren_farbe4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"frauen_mit_hasenohren_farbe4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"frauen_mit_hasenohren_farbe4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"frauen_mit_hasenohren_farbe5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"frauen_mit_hasenohren_farbe5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"frauen_mit_hasenohren_farbe5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"frauen_mit_hasenohren_farbe5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"frauen_mit_hasenohren_farbe1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"frauen_mit_hasenohren_farbe2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"frauen_mit_hasenohren_farbe3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"frauen_mit_hasenohren_farbe4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"frauen_mit_hasenohren_farbe5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"manner_mit_hasenohren", "1F46F-200D-2642-FE0F"},
    {"manner_mit_hasenohren_farbe1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"manner_mit_hasenohren_farbe1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"manner_mit_hasenohren_farbe1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"manner_mit_hasenohren_farbe1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"manner_mit_hasenohren_farbe2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"manner_mit_hasenohren_farbe2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"manner_mit_hasenohren_farbe2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"manner_mit_hasenohren_farbe2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"manner_mit_hasenohren_farbe3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"manner_mit_hasenohren_farbe3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"manner_mit_hasenohren_farbe3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"manner_mit_hasenohren_farbe3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"manner_mit_hasenohren_farbe4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"manner_mit_hasenohren_farbe4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"manner_mit_hasenohren_farbe4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"manner_mit_hasenohren_farbe4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"manner_mit_hasenohren_farbe5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"manner_mit_hasenohren_farbe5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"manner_mit_hasenohren_farbe5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"manner_mit_hasenohren_farbe5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"manner_mit_hasenohren_farbe1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"manner_mit_hasenohren_farbe2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"manner_mit_hasenohren_farbe3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"manner_mit_hasenohren_farbe4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"manner_mit_hasenohren_farbe5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"frau_mit_schleier", "1F470-200D-2640-FE0F"},
    {"frau_mit_schleier_farbe1", "1F470-1F3FB-200D-2640-FE0F"},
    {"frau_mit_schleier_farbe2", "1F470-1F3FC-200D-2640-FE0F"},
    {"frau_mit_schleier_farbe3", "1F470-1F3FD-200D-2640-FE0F"},
    {"frau_mit_schleier_farbe4", "1F470-1F3FE-200D-2640-FE0F"},
    {"frau_mit_schleier_farbe5", "1F470-1F3FF-200D-2640-FE0F"},
    {"mann_mit_schleier", "1F470-200D-2642-FE0F"},
    {"mann_mit_schleier_farbe1", "1F470-1F3FB-200D-2642-FE0F"},
    {"mann_mit_schleier_farbe2", "1F470-1F3FC-200D-2642-FE0F"},
    {"mann_mit_schleier_farbe3", "1F470-1F3FD-200D-2642-FE0F"},
    {"mann_mit_schleier_farbe4", "1F470-1F3FE-200D-2642-FE0F"},
    {"mann_mit_schleier_farbe5", "1F470-1F3FF-200D-2642-FE0F"},
    {"frau_blond", "1F471-200D-2640-FE0F"},
    {"frau_blond_farbe1", "1F471-1F3FB-200D-2640-FE0F"},
    {"frau_blond_farbe2", "1F471-1F3FC-200D-2640-FE0F"},
    {"frau_blond_farbe3", "1F471-1F3FD-200D-2640-FE0F"},
    {"frau_blond_farbe4", "1F471-1F3FE-200D-2640-FE0F"},
    {"frau_blond_farbe5", "1F471-1F3FF-200D-2640-FE0F"},
    {"mann_blond", "1F471-200D-2642-FE0F"},
    {"mann_blond_farbe1", "1F471-1F3FB-200D-2642-FE0F"},
    {"mann_blond_farbe2", "1F471-1F3FC-200D-2642-FE0F"},
    {"mann_blond_farbe3", "1F471-1F3FD-200D-2642-FE0F"},
    {"mann_blond_farbe4", "1F471-1F3FE-200D-2642-FE0F"},
    {"mann_blond_farbe5", "1F471-1F3FF-200D-2642-FE0F"},
    {"frau_mit_turban", "1F473-200D-2640-FE0F"},
    {"frau_mit_turban_farbe1", "1F473-1F3FB-200D-2640-FE0F"},
    {"frau_mit_turban_farbe2", "1F473-1F3FC-200D-2640-FE0F"},
    {"frau_mit_turban_farbe3", "1F473-1F3FD-200D-2640-FE0F"},
    {"frau_mit_turban_farbe4", "1F473-1F3FE-200D-2640-FE0F"},
    {"frau_mit_turban_farbe5", "1F473-1F3FF-200D-2640-FE0F"},
    {"mann_mit_turban", "1F473-200D-2642-FE0F"},
    {"mann_mit_turban_farbe1", "1F473-1F3FB-200D-2642-FE0F"},
    {"mann_mit_turban_farbe2", "1F473-1F3FC-200D-2642-FE0F"},
    {"mann_mit_turban_farbe3", "1F473-1F3FD-200D-2642-FE0F"},
    {"mann_mit_turban_farbe4", "1F473-1F3FE-200D-2642-FE0F"},
    {"mann_mit_turban_farbe5", "1F473-1F3FF-200D-2642-FE0F"},
    {"bauarbeiter", "1F477-200D-2642-FE0F"},
    {"bauarbeiter_farbe1", "1F477-1F3FB-200D-2642-FE0F"},
    {"bauarbeiter_farbe2", "1F477-1F3FC-200D-2642-FE0F"},
    {"bauarbeiter_farbe3", "1F477-1F3FD-200D-2642-FE0F"},
    {"bauarbeiter_farbe4", "1F477-1F3FE-200D-2642-FE0F"},
    {"bauarbeiter_farbe5", "1F477-1F3FF-200D-2642-FE0F"},
    {"infoschalter_mitarbeiter", "1F481-200D-2642-FE0F"},
    {"infoschalter_mitarbeiter_farbe1", "1F481-1F3FB-200D-2642-FE0F"},
    {"infoschalter_mitarbeiter_farbe2", "1F481-1F3FC-200D-2642-FE0F"},
    {"infoschalter_mitarbeiter_farbe3", "1F481-1F3FD-200D-2642-FE0F"},
    {"infoschalter_mitarbeiter_farbe4", "1F481-1F3FE-200D-2642-FE0F"},
    {"infoschalter_mitarbeiter_farbe5", "1F481-1F3FF-200D-2642-FE0F"},
    {"wachsoldatin", "1F482-200D-2640-FE0F"},
    {"wachsoldatin_farbe1", "1F482-1F3FB-200D-2640-FE0F"},
    {"wachsoldatin_farbe2", "1F482-1F3FC-200D-2640-FE0F"},
    {"wachsoldatin_farbe3", "1F482-1F3FD-200D-2640-FE0F"},
    {"wachsoldatin_farbe4", "1F482-1F3FE-200D-2640-FE0F"},
    {"wachsoldatin_farbe5", "1F482-1F3FF-200D-2640-FE0F"},
    {"wachsoldat", "1F482-200D-2642-FE0F"},
    {"wachsoldat_farbe1", "1F482-1F3FB-200D-2642-FE0F"},
    {"wachsoldat_farbe2", "1F482-1F3FC-200D-2642-FE0F"},
    {"wachsoldat_farbe3", "1F482-1F3FD-200D-2642-FE0F"},
    {"wachsoldat_farbe4", "1F482-1F3FE-200D-2642-FE0F"},
    {"wachsoldat_farbe5", "1F482-1F3FF-200D-2642-FE0F"},
    {"frau_die_eine_kopfmassage_bekommt", "1F486-200D-2640-FE0F"},
    {"frau_die_eine_kopfmassage_bekommt_farbe1", "1F486-1F3FB-200D-2640-FE0F"},
    {"frau_die_eine_kopfmassage_bekommt_farbe2", "1F486-1F3FC-200D-2640-FE0F"},
    {"frau_die_eine_kopfmassage_bekommt_farbe3", "1F486-1F3FD-200D-2640-FE0F"},
    {"frau_die_eine_kopfmassage_bekommt_farbe4", "1F486-1F3FE-200D-2640-FE0F"},
    {"frau_die_eine_kopfmassage_bekommt_farbe5", "1F486-1F3FF-200D-2640-FE0F"},
    {"mann_der_eine_kopfmassage_bekommt", "1F486-200D-2642-FE0F"},
    {"mann_der_eine_kopfmassage_bekommt_farbe1", "1F486-1F3FB-200D-2642-FE0F"},
    {"mann_der_eine_kopfmassage_bekommt_farbe2", "1F486-1F3FC-200D-2642-FE0F"},
    {"mann_der_eine_kopfmassage_bekommt_farbe3", "1F486-1F3FD-200D-2642-FE0F"},
    {"mann_der_eine_kopfmassage_bekommt_farbe4", "1F486-1F3FE-200D-2642-FE0F"},
    {"mann_der_eine_kopfmassage_bekommt_farbe5", "1F486-1F3FF-200D-2642-FE0F"},
    {"frau_beim_haareschneiden", "1F487-200D-2640-FE0F"},
    {"frau_beim_haareschneiden_farbe1", "1F487-1F3FB-200D-2640-FE0F"},
    {"frau_beim_haareschneiden_farbe2", "1F487-1F3FC-200D-2640-FE0F"},
    {"frau_beim_haareschneiden_farbe3", "1F487-1F3FD-200D-2640-FE0F"},
    {"frau_beim_haareschneiden_farbe4", "1F487-1F3FE-200D-2640-FE0F"},
    {"frau_beim_haareschneiden_farbe5", "1F487-1F3FF-200D-2640-FE0F"},
    {"mann_beim_haareschneiden", "1F487-200D-2642-FE0F"},
    {"mann_beim_haareschneiden_farbe1", "1F487-1F3FB-200D-2642-FE0F"},
    {"mann_beim_haareschneiden_farbe2", "1F487-1F3FC-200D-2642-FE0F"},
    {"mann_beim_haareschneiden_farbe3", "1F487-1F3FD-200D-2642-FE0F"},
    {"mann_beim_haareschneiden_farbe4", "1F487-1F3FE-200D-2642-FE0F"},
    {"mann_beim_haareschneiden_farbe5", "1F487-1F3FF-200D-2642-FE0F"},
    {"detektiv", "1F575-FE0F-200D-2642-FE0F"},
    {"detektiv_farbe1", "1F575-1F3FB-200D-2642-FE0F"},
    {"detektiv_farbe2", "1F575-1F3FC-200D-2642-FE0F"},
    {"detektiv_farbe3", "1F575-1F3FD-200D-2642-FE0F"},
    {"detektiv_farbe4", "1F575-1F3FE-200D-2642-FE0F"},
    {"detektiv_farbe5", "1F575-1F3FF-200D-2642-FE0F"},
    {"frau_mit_uberkreuzten_armen", "1F645-200D-2640-FE0F"},
    {"frau_mit_uberkreuzten_armen_farbe1", "1F645-1F3FB-200D-2640-FE0F"},
    {"frau_mit_uberkreuzten_armen_farbe2", "1F645-1F3FC-200D-2640-FE0F"},
    {"frau_mit_uberkreuzten_armen_farbe3", "1F645-1F3FD-200D-2640-FE0F"},
    {"frau_mit_uberkreuzten_armen_farbe4", "1F645-1F3FE-200D-2640-FE0F"},
    {"frau_mit_uberkreuzten_armen_farbe5", "1F645-1F3FF-200D-2640-FE0F"},
    {"mann_mit_uberkreuzten_armen", "1F645-200D-2642-FE0F"},
    {"mann_mit_uberkreuzten_armen_farbe1", "1F645-1F3FB-200D-2642-FE0F"},
    {"mann_mit_uberkreuzten_armen_farbe2", "1F645-1F3FC-200D-2642-FE0F"},
    {"mann_mit_uberkreuzten_armen_farbe3", "1F645-1F3FD-200D-2642-FE0F"},
    {"mann_mit_uberkreuzten_armen_farbe4", "1F645-1F3FE-200D-2642-FE0F"},
    {"mann_mit_uberkreuzten_armen_farbe5", "1F645-1F3FF-200D-2642-FE0F"},
    {"frau_mit_handen_auf_dem_kopf", "1F646-200D-2640-FE0F"},
    {"frau_mit_handen_auf_dem_kopf_farbe1", "1F646-1F3FB-200D-2640-FE0F"},
    {"frau_mit_handen_auf_dem_kopf_farbe2", "1F646-1F3FC-200D-2640-FE0F"},
    {"frau_mit_handen_auf_dem_kopf_farbe3", "1F646-1F3FD-200D-2640-FE0F"},
    {"frau_mit_handen_auf_dem_kopf_farbe4", "1F646-1F3FE-200D-2640-FE0F"},
    {"frau_mit_handen_auf_dem_kopf_farbe5", "1F646-1F3FF-200D-2640-FE0F"},
    {"mann_mit_handen_auf_dem_kopf", "1F646-200D-2642-FE0F"},
    {"mann_mit_handen_auf_dem_kopf_farbe1", "1F646-1F3FB-200D-2642-FE0F"},
    {"mann_mit_handen_auf_dem_kopf_farbe2", "1F646-1F3FC-200D-2642-FE0F"},
    {"mann_mit_handen_auf_dem_kopf_farbe3", "1F646-1F3FD-200D-2642-FE0F"},
    {"mann_mit_handen_auf_dem_kopf_farbe4", "1F646-1F3FE-200D-2642-FE0F"},
    {"mann_mit_handen_auf_dem_kopf_farbe5", "1F646-1F3FF-200D-2642-FE0F"},
    {"sich_verbeugende_frau", "1F647-200D-2640-FE0F"},
    {"sich_verbeugende_frau_farbe1", "1F647-1F3FB-200D-2640-FE0F"},
    {"sich_verbeugende_frau_farbe2", "1F647-1F3FC-200D-2640-FE0F"},
    {"sich_verbeugende_frau_farbe3", "1F647-1F3FD-200D-2640-FE0F"},
    {"sich_verbeugende_frau_farbe4", "1F647-1F3FE-200D-2640-FE0F"},
    {"sich_verbeugende_frau_farbe5", "1F647-1F3FF-200D-2640-FE0F"},
    {"sich_verbeugender_mann", "1F647-200D-2642-FE0F"},
    {"sich_verbeugender_mann_farbe1", "1F647-1F3FB-200D-2642-FE0F"},
    {"sich_verbeugender_mann_farbe2", "1F647-1F3FC-200D-2642-FE0F"},
    {"sich_verbeugender_mann_farbe3", "1F647-1F3FD-200D-2642-FE0F"},
    {"sich_verbeugender_mann_farbe4", "1F647-1F3FE-200D-2642-FE0F"},
    {"sich_verbeugender_mann_farbe5", "1F647-1F3FF-200D-2642-FE0F"},
    {"frau_mit_erhobenem_arm", "1F64B-200D-2640-FE0F"},
    {"frau_mit_erhobenem_arm_farbe1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"frau_mit_erhobenem_arm_farbe2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"frau_mit_erhobenem_arm_farbe3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"frau_mit_erhobenem_arm_farbe4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"frau_mit_erhobenem_arm_farbe5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"mann_mit_erhobenem_arm", "1F64B-200D-2642-FE0F"},
    {"mann_mit_erhobenem_arm_farbe1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"mann_mit_erhobenem_arm_farbe2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"mann_mit_erhobenem_arm_farbe3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"mann_mit_erhobenem_arm_farbe4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"mann_mit_erhobenem_arm_farbe5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"missmutige_frau", "1F64D-200D-2640-FE0F"},
    {"missmutige_frau_farbe1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"missmutige_frau_farbe2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"missmutige_frau_farbe3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"missmutige_frau_farbe4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"missmutige_frau_farbe5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"missmutiger_mann", "1F64D-200D-2642-FE0F"},
    {"missmutiger_mann_farbe1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"missmutiger_mann_farbe2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"missmutiger_mann_farbe3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"missmutiger_mann_farbe4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"missmutiger_mann_farbe5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"schmollende_frau", "1F64E-200D-2640-FE0F"},
    {"schmollende_frau_farbe1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"schmollende_frau_farbe2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"schmollende_frau_farbe3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"schmollende_frau_farbe4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"schmollende_frau_farbe5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"schmollender_mann", "1F64E-200D-2642-FE0F"},
    {"schmollender_mann_farbe1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"schmollender_mann_farbe2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"schmollender_mann_farbe3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"schmollender_mann_farbe4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"schmollender_mann_farbe5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"frau_im_ruderboot", "1F6A3-200D-2640-FE0F"},
    {"frau_im_ruderboot_farbe1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"frau_im_ruderboot_farbe2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"frau_im_ruderboot_farbe3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"frau_im_ruderboot_farbe4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"frau_im_ruderboot_farbe5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"mann_im_ruderboot", "1F6A3-200D-2642-FE0F"},
    {"mann_im_ruderboot_farbe1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"mann_im_ruderboot_farbe2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"mann_im_ruderboot_farbe3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"mann_im_ruderboot_farbe4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"mann_im_ruderboot_farbe5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"radfahrer", "1F6B4-200D-2642-FE0F"},
    {"radfahrer_farbe1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"radfahrer_farbe2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"radfahrer_farbe3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"radfahrer_farbe4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"radfahrer_farbe5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"mountainbiker", "1F6B5-200D-2642-FE0F"},
    {"mountainbiker_farbe1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"mountainbiker_farbe2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"mountainbiker_farbe3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"mountainbiker_farbe4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"mountainbiker_farbe5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"fussganger", "1F6B6-200D-2642-FE0F"},
    {"fussganger_farbe1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"fussganger_farbe2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"fussganger_farbe3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"fussganger_farbe4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"fussganger_farbe5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"fussganger_nach_rechts", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"fussganger_nach_rechts_farbe1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"fussganger_nach_rechts_farbe2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"fussganger_nach_rechts_farbe3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"fussganger_nach_rechts_farbe4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"fussganger_nach_rechts_farbe5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"sich_an_den_kopf_fassende_frau", "1F926-200D-2640-FE0F"},
    {"sich_an_den_kopf_fassende_frau_farbe1", "1F926-1F3FB-200D-2640-FE0F"},
    {"sich_an_den_kopf_fassende_frau_farbe2", "1F926-1F3FC-200D-2640-FE0F"},
    {"sich_an_den_kopf_fassende_frau_farbe3", "1F926-1F3FD-200D-2640-FE0F"},
    {"sich_an_den_kopf_fassende_frau_farbe4", "1F926-1F3FE-200D-2640-FE0F"},
    {"sich_an_den_kopf_fassende_frau_farbe5", "1F926-1F3FF-200D-2640-FE0F"},
    {"sich_an_den_kopf_fassender_mann", "1F926-200D-2642-FE0F"},
    {"sich_an_den_kopf_fassender_mann_farbe1", "1F926-1F3FB-200D-2642-FE0F"},
    {"sich_an_den_kopf_fassender_mann_farbe2", "1F926-1F3FC-200D-2642-FE0F"},
    {"sich_an_den_kopf_fassender_mann_farbe3", "1F926-1F3FD-200D-2642-FE0F"},
    {"sich_an_den_kopf_fassender_mann_farbe4", "1F926-1F3FE-200D-2642-FE0F"},
    {"sich_an_den_kopf_fassender_mann_farbe5", "1F926-1F3FF-200D-2642-FE0F"},
    {"frau_im_smoking", "1F935-200D-2640-FE0F"},
    {"frau_im_smoking_farbe1", "1F935-1F3FB-200D-2640-FE0F"},
    {"frau_im_smoking_farbe2", "1F935-1F3FC-200D-2640-FE0F"},
    {"frau_im_smoking_farbe3", "1F935-1F3FD-200D-2640-FE0F"},
    {"frau_im_smoking_farbe4", "1F935-1F3FE-200D-2640-FE0F"},
    {"frau_im_smoking_farbe5", "1F935-1F3FF-200D-2640-FE0F"},
    {"mann_im_smoking", "1F935-200D-2642-FE0F"},
    {"mann_im_smoking_farbe1", "1F935-1F3FB-200D-2642-FE0F"},
    {"mann_im_smoking_farbe2", "1F935-1F3FC-200D-2642-FE0F"},
    {"mann_im_smoking_farbe3", "1F935-1F3FD-200D-2642-FE0F"},
    {"mann_im_smoking_farbe4", "1F935-1F3FE-200D-2642-FE0F"},
    {"mann_im_smoking_farbe5", "1F935-1F3FF-200D-2642-FE0F"},
    {"schulterzuckende_frau", "1F937-200D-2640-FE0F"},
    {"schulterzuckende_frau_farbe1", "1F937-1F3FB-200D-2640-FE0F"},
    {"schulterzuckende_frau_farbe2", "1F937-1F3FC-200D-2640-FE0F"},
    {"schulterzuckende_frau_farbe3", "1F937-1F3FD-200D-2640-FE0F"},
    {"schulterzuckende_frau_farbe4", "1F937-1F3FE-200D-2640-FE0F"},
    {"schulterzuckende_frau_farbe5", "1F937-1F3FF-200D-2640-FE0F"},
    {"schulterzuckender_mann", "1F937-200D-2642-FE0F"},
    {"schulterzuckender_mann_farbe1", "1F937-1F3FB-200D-2642-FE0F"},
    {"schulterzuckender_mann_farbe2", "1F937-1F3FC-200D-2642-FE0F"},
    {"schulterzuckender_mann_farbe3", "1F937-1F3FD-200D-2642-FE0F"},
    {"schulterzuckender_mann_farbe4", "1F937-1F3FE-200D-2642-FE0F"},
    {"schulterzuckender_mann_farbe5", "1F937-1F3FF-200D-2642-FE0F"},
    {"rad_schlagende_frau", "1F938-200D-2640-FE0F"},
    {"rad_schlagende_frau_farbe1", "1F938-1F3FB-200D-2640-FE0F"},
    {"rad_schlagende_frau_farbe2", "1F938-1F3FC-200D-2640-FE0F"},
    {"rad_schlagende_frau_farbe3", "1F938-1F3FD-200D-2640-FE0F"},
    {"rad_schlagende_frau_farbe4", "1F938-1F3FE-200D-2640-FE0F"},
    {"rad_schlagende_frau_farbe5", "1F938-1F3FF-200D-2640-FE0F"},
    {"rad_schlagender_mann", "1F938-200D-2642-FE0F"},
    {"rad_schlagender_mann_farbe1", "1F938-1F3FB-200D-2642-FE0F"},
    {"rad_schlagender_mann_farbe2", "1F938-1F3FC-200D-2642-FE0F"},
    {"rad_schlagender_mann_farbe3", "1F938-1F3FD-200D-2642-FE0F"},
    {"rad_schlagender_mann_farbe4", "1F938-1F3FE-200D-2642-FE0F"},
    {"rad_schlagender_mann_farbe5", "1F938-1F3FF-200D-2642-FE0F"},
    {"jongleur", "1F939-200D-2642-FE0F"},
    {"jongleur_farbe1", "1F939-1F3FB-200D-2642-FE0F"},
    {"jongleur_farbe2", "1F939-1F3FC-200D-2642-FE0F"},
    {"jongleur_farbe3", "1F939-1F3FD-200D-2642-FE0F"},
    {"jongleur_farbe4", "1F939-1F3FE-200D-2642-FE0F"},
    {"jongleur_farbe5", "1F939-1F3FF-200D-2642-FE0F"},
    {"ringende_frauen", "1F93C-200D-2640-FE0F"},
    {"ringende_frauen_farbe1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"ringende_frauen_farbe1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"ringende_frauen_farbe1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"ringende_frauen_farbe1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"ringende_frauen_farbe2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"ringende_frauen_farbe2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"ringende_frauen_farbe2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"ringende_frauen_farbe2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"ringende_frauen_farbe3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"ringende_frauen_farbe3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"ringende_frauen_farbe3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"ringende_frauen_farbe3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"ringende_frauen_farbe4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"ringende_frauen_farbe4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"ringende_frauen_farbe4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"ringende_frauen_farbe4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"ringende_frauen_farbe5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"ringende_frauen_farbe5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"ringende_frauen_farbe5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"ringende_frauen_farbe5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"ringende_frauen_farbe1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"ringende_frauen_farbe2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"ringende_frauen_farbe3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"ringende_frauen_farbe4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"ringende_frauen_farbe5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"ringende_manner", "1F93C-200D-2642-FE0F"},
    {"ringende_manner_farbe1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"ringende_manner_farbe1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"ringende_manner_farbe1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"ringende_manner_farbe1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"ringende_manner_farbe2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"ringende_manner_farbe2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"ringende_manner_farbe2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"ringende_manner_farbe2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"ringende_manner_farbe3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"ringende_manner_farbe3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"ringende_manner_farbe3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"ringende_manner_farbe3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"ringende_manner_farbe4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"ringende_manner_farbe4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"ringende_manner_farbe4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"ringende_manner_farbe4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"ringende_manner_farbe5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"ringende_manner_farbe5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"ringende_manner_farbe5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"ringende_manner_farbe5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"ringende_manner_farbe1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"ringende_manner_farbe2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"ringende_manner_farbe3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"ringende_manner_farbe4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"ringende_manner_farbe5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"wasserballspieler", "1F93D-200D-2642-FE0F"},
    {"wasserballspieler_farbe1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"wasserballspieler_farbe2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"wasserballspieler_farbe3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"wasserballspieler_farbe4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"wasserballspieler_farbe5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"handballspieler", "1F93E-200D-2642-FE0F"},
    {"handballspieler_farbe1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"handballspieler_farbe2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"handballspieler_farbe3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"handballspieler_farbe4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"handballspieler_farbe5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"heldin", "1F9B8-200D-2640-FE0F"},
    {"heldin_farbe1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"heldin_farbe2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"heldin_farbe3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"heldin_farbe4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"heldin_farbe5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"superheld", "1F9B8-200D-2642-FE0F"},
    {"superheld_farbe1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"superheld_farbe2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"superheld_farbe3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"superheld_farbe4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"superheld_farbe5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"weiblicher_bosewicht", "1F9B9-200D-2640-FE0F"},
    {"weiblicher_bosewicht_farbe1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"weiblicher_bosewicht_farbe2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"weiblicher_bosewicht_farbe3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"weiblicher_bosewicht_farbe4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"weiblicher_bosewicht_farbe5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"mannlicher_bosewicht", "1F9B9-200D-2642-FE0F"},
    {"mannlicher_bosewicht_farbe1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"mannlicher_bosewicht_farbe2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"mannlicher_bosewicht_farbe3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"mannlicher_bosewicht_farbe4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"mannlicher_bosewicht_farbe5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"stehende_frau", "1F9CD-200D-2640-FE0F"},
    {"stehende_frau_farbe1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"stehende_frau_farbe2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"stehende_frau_farbe3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"stehende_frau_farbe4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"stehende_frau_farbe5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"stehender_mann", "1F9CD-200D-2642-FE0F"},
    {"stehender_mann_farbe1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"stehender_mann_farbe2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"stehender_mann_farbe3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"stehender_mann_farbe4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"stehender_mann_farbe5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"kniende_frau", "1F9CE-200D-2640-FE0F"},
    {"kniende_frau_farbe1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"kniende_frau_farbe2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"kniende_frau_farbe3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"kniende_frau_farbe4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"kniende_frau_farbe5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"kniende_frau_nach_rechts", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kniende_frau_nach_rechts_farbe1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kniende_frau_nach_rechts_farbe2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kniende_frau_nach_rechts_farbe3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kniende_frau_nach_rechts_farbe4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kniende_frau_nach_rechts_farbe5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kniender_mann", "1F9CE-200D-2642-FE0F"},
    {"kniender_mann_farbe1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"kniender_mann_farbe2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"kniender_mann_farbe3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"kniender_mann_farbe4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"kniender_mann_farbe5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"kniender_mann_nach_rechts", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kniender_mann_nach_rechts_farbe1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kniender_mann_nach_rechts_farbe2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kniender_mann_nach_rechts_farbe3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kniender_mann_nach_rechts_farbe4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kniender_mann_nach_rechts_farbe5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"gehorlose_frau", "1F9CF-200D-2640-FE0F"},
    {"gehorlose_frau_farbe1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"gehorlose_frau_farbe2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"gehorlose_frau_farbe3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"gehorlose_frau_farbe4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"gehorlose_frau_farbe5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"gehorloser_mann", "1F9CF-200D-2642-FE0F"},
    {"gehorloser_mann_farbe1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"gehorloser_mann_farbe2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"gehorloser_mann_farbe3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"gehorloser_mann_farbe4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"gehorloser_mann_farbe5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"frau_bart", "1F9D4-200D-2640-FE0F"},
    {"frau_bart_farbe1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"frau_bart_farbe2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"frau_bart_farbe3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"frau_bart_farbe4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"frau_bart_farbe5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"mann_bart", "1F9D4-200D-2642-FE0F"},
    {"mann_bart_farbe1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"mann_bart_farbe2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"mann_bart_farbe3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"mann_bart_farbe4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"mann_bart_farbe5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"frau_in_dampfsauna", "1F9D6-200D-2640-FE0F"},
    {"frau_in_dampfsauna_farbe1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"frau_in_dampfsauna_farbe2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"frau_in_dampfsauna_farbe3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"frau_in_dampfsauna_farbe4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"frau_in_dampfsauna_farbe5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"mann_in_dampfsauna", "1F9D6-200D-2642-FE0F"},
    {"mann_in_dampfsauna_farbe1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"mann_in_dampfsauna_farbe2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"mann_in_dampfsauna_farbe3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"mann_in_dampfsauna_farbe4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"mann_in_dampfsauna_farbe5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"bergsteiger", "1F9D7-200D-2642-FE0F"},
    {"bergsteiger_farbe1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"bergsteiger_farbe2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"bergsteiger_farbe3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"bergsteiger_farbe4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"bergsteiger_farbe5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"frau_im_lotossitz", "1F9D8-200D-2640-FE0F"},
    {"frau_im_lotossitz_farbe1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"frau_im_lotossitz_farbe2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"frau_im_lotossitz_farbe3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"frau_im_lotossitz_farbe4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"frau_im_lotossitz_farbe5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"mann_im_lotossitz", "1F9D8-200D-2642-FE0F"},
    {"mann_im_lotossitz_farbe1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"mann_im_lotossitz_farbe2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"mann_im_lotossitz_farbe3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"mann_im_lotossitz_farbe4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"mann_im_lotossitz_farbe5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"magier", "1F9D9-200D-2642-FE0F"},
    {"magier_farbe1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"magier_farbe2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"magier_farbe3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"magier_farbe4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"magier_farbe5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"fee", "1F9DA-200D-2640-FE0F"},
    {"fee_farbe1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"fee_farbe2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"fee_farbe3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"fee_farbe4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"fee_farbe5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"mannliche_fee", "1F9DA-200D-2642-FE0F"},
    {"mannliche_fee_farbe1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"mannliche_fee_farbe2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"mannliche_fee_farbe3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"mannliche_fee_farbe4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"mannliche_fee_farbe5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"weiblicher_vampir", "1F9DB-200D-2640-FE0F"},
    {"weiblicher_vampir_farbe1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"weiblicher_vampir_farbe2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"weiblicher_vampir_farbe3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"weiblicher_vampir_farbe4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"weiblicher_vampir_farbe5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"mannlicher_vampir", "1F9DB-200D-2642-FE0F"},
    {"mannlicher_vampir_farbe1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"mannlicher_vampir_farbe2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"mannlicher_vampir_farbe3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"mannlicher_vampir_farbe4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"mannlicher_vampir_farbe5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"meerjungfrau", "1F9DC-200D-2640-FE0F"},
    {"meerjungfrau_farbe1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"meerjungfrau_farbe2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"meerjungfrau_farbe3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"meerjungfrau_farbe4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"meerjungfrau_farbe5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"wassermann", "1F9DC-200D-2642-FE0F"},
    {"wassermann_farbe1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"wassermann_farbe2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"wassermann_farbe3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"wassermann_farbe4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"wassermann_farbe5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"elf", "1F9DD-200D-2642-FE0F"},
    {"elf_farbe1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"elf_farbe2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"elf_farbe3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"elf_farbe4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"elf_farbe5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"weiblicher_flaschengeist", "1F9DE-200D-2640-FE0F"},
    {"mannlicher_flaschengeist", "1F9DE-200D-2642-FE0F"},
    {"weiblicher_zombie", "1F9DF-200D-2640-FE0F"},
    {"mannlicher_zombie", "1F9DF-200D-2642-FE0F"},
    {"mann_rotes_haar", "1F468-200D-1F9B0"},
    {"mann_rotes_haar_farbe1", "1F468-1F3FB-200D-1F9B0"},
    {"mann_rotes_haar_farbe2", "1F468-1F3FC-200D-1F9B0"},
    {"mann_rotes_haar_farbe3", "1F468-1F3FD-200D-1F9B0"},
    {"mann_rotes_haar_farbe4", "1F468-1F3FE-200D-1F9B0"},
    {"mann_rotes_haar_farbe5", "1F468-1F3FF-200D-1F9B0"},
    {"mann_lockiges_haar", "1F468-200D-1F9B1"},
    {"mann_lockiges_haar_farbe1", "1F468-1F3FB-200D-1F9B1"},
    {"mann_lockiges_haar_farbe2", "1F468-1F3FC-200D-1F9B1"},
    {"mann_lockiges_haar_farbe3", "1F468-1F3FD-200D-1F9B1"},
    {"mann_lockiges_haar_farbe4", "1F468-1F3FE-200D-1F9B1"},
    {"mann_lockiges_haar_farbe5", "1F468-1F3FF-200D-1F9B1"},
    {"mann_glatze", "1F468-200D-1F9B2"},
    {"mann_glatze_farbe1", "1F468-1F3FB-200D-1F9B2"},
    {"mann_glatze_farbe2", "1F468-1F3FC-200D-1F9B2"},
    {"mann_glatze_farbe3", "1F468-1F3FD-200D-1F9B2"},
    {"mann_glatze_farbe4", "1F468-1F3FE-200D-1F9B2"},
    {"mann_glatze_farbe5", "1F468-1F3FF-200D-1F9B2"},
    {"mann_weisses_haar", "1F468-200D-1F9B3"},
    {"mann_weisses_haar_farbe1", "1F468-1F3FB-200D-1F9B3"},
    {"mann_weisses_haar_farbe2", "1F468-1F3FC-200D-1F9B3"},
    {"mann_weisses_haar_farbe3", "1F468-1F3FD-200D-1F9B3"},
    {"mann_weisses_haar_farbe4", "1F468-1F3FE-200D-1F9B3"},
    {"mann_weisses_haar_farbe5", "1F468-1F3FF-200D-1F9B3"},
    {"frau_rotes_haar", "1F469-200D-1F9B0"},
    {"frau_rotes_haar_farbe1", "1F469-1F3FB-200D-1F9B0"},
    {"frau_rotes_haar_farbe2", "1F469-1F3FC-200D-1F9B0"},
    {"frau_rotes_haar_farbe3", "1F469-1F3FD-200D-1F9B0"},
    {"frau_rotes_haar_farbe4", "1F469-1F3FE-200D-1F9B0"},
    {"frau_rotes_haar_farbe5", "1F469-1F3FF-200D-1F9B0"},
    {"frau_lockiges_haar", "1F469-200D-1F9B1"},
    {"frau_lockiges_haar_farbe1", "1F469-1F3FB-200D-1F9B1"},
    {"frau_lockiges_haar_farbe2", "1F469-1F3FC-200D-1F9B1"},
    {"frau_lockiges_haar_farbe3", "1F469-1F3FD-200D-1F9B1"},
    {"frau_lockiges_haar_farbe4", "1F469-1F3FE-200D-1F9B1"},
    {"frau_lockiges_haar_farbe5", "1F469-1F3FF-200D-1F9B1"},
    {"frau_glatze", "1F469-200D-1F9B2"},
    {"frau_glatze_farbe1", "1F469-1F3FB-200D-1F9B2"},
    {"frau_glatze_farbe2", "1F469-1F3FC-200D-1F9B2"},
    {"frau_glatze_farbe3", "1F469-1F3FD-200D-1F9B2"},
    {"frau_glatze_farbe4", "1F469-1F3FE-200D-1F9B2"},
    {"frau_glatze_farbe5", "1F469-1F3FF-200D-1F9B2"},
    {"frau_weisses_haar", "1F469-200D-1F9B3"},
    {"frau_weisses_haar_farbe1", "1F469-1F3FB-200D-1F9B3"},
    {"frau_weisses_haar_farbe2", "1F469-1F3FC-200D-1F9B3"},
    {"frau_weisses_haar_farbe3", "1F469-1F3FD-200D-1F9B3"},
    {"frau_weisses_haar_farbe4", "1F469-1F3FE-200D-1F9B3"},
    {"frau_weisses_haar_farbe5", "1F469-1F3FF-200D-1F9B3"},
    {"person_rotes_haar", "1F9D1-200D-1F9B0"},
    {"person_rotes_haar_farbe1", "1F9D1-1F3FB-200D-1F9B0"},
    {"person_rotes_haar_farbe2", "1F9D1-1F3FC-200D-1F9B0"},
    {"person_rotes_haar_farbe3", "1F9D1-1F3FD-200D-1F9B0"},
    {"person_rotes_haar_farbe4", "1F9D1-1F3FE-200D-1F9B0"},
    {"person_rotes_haar_farbe5", "1F9D1-1F3FF-200D-1F9B0"},
    {"person_lockiges_haar", "1F9D1-200D-1F9B1"},
    {"person_lockiges_haar_farbe1", "1F9D1-1F3FB-200D-1F9B1"},
    {"person_lockiges_haar_farbe2", "1F9D1-1F3FC-200D-1F9B1"},
    {"person_lockiges_haar_farbe3", "1F9D1-1F3FD-200D-1F9B1"},
    {"person_lockiges_haar_farbe4", "1F9D1-1F3FE-200D-1F9B1"},
    {"person_lockiges_haar_farbe5", "1F9D1-1F3FF-200D-1F9B1"},
    {"person_glatze", "1F9D1-200D-1F9B2"},
    {"person_glatze_farbe1", "1F9D1-1F3FB-200D-1F9B2"},
    {"person_glatze_farbe2", "1F9D1-1F3FC-200D-1F9B2"},
    {"person_glatze_farbe3", "1F9D1-1F3FD-200D-1F9B2"},
    {"person_glatze_farbe4", "1F9D1-1F3FE-200D-1F9B2"},
    {"person_glatze_farbe5", "1F9D1-1F3FF-200D-1F9B2"},
    {"person_weisses_haar", "1F9D1-200D-1F9B3"},
    {"person_weisses_haar_farbe1", "1F9D1-1F3FB-200D-1F9B3"},
    {"person_weisses_haar_farbe2", "1F9D1-1F3FC-200D-1F9B3"},
    {"person_weisses_haar_farbe3", "1F9D1-1F3FD-200D-1F9B3"},
    {"person_weisses_haar_farbe4", "1F9D1-1F3FE-200D-1F9B3"},
    {"person_weisses_haar_farbe5", "1F9D1-1F3FF-200D-1F9B3"},
    {"gebrochene_kette", "26D3-FE0F-200D-1F4A5"},
    {"herz_in_flammen", "2764-FE0F-200D-1F525"},
    {"herz_mit_verband", "2764-FE0F-200D-1FA79"},
    {"brauner_pilz", "1F344-200D-1F7EB"},
    {"limette", "1F34B-200D-1F7E9"},
    {"transgender_flagge", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"regenbogenflagge", "1F3F3-FE0F-200D-1F308"},
    {"piratenflagge", "1F3F4-200D-2620-FE0F"},
    {"schwarze_katze", "1F408-200D-2B1B"},
    {"assistenzhund", "1F415-200D-1F9BA"},
    {"schwarzer_vogel", "1F426-200D-2B1B"},
    {"phonix", "1F426-200D-1F525"},
    {"eisbar", "1F43B-200D-2744-FE0F"},
    {"auge_in_sprechblase", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"gesicht_das_ausatmet", "1F62E-200D-1F4A8"},
    {"gesicht_mit_spiralen_als_augen", "1F635-200D-1F4AB"},
    {"gesicht_in_wolken", "1F636-200D-1F32B-FE0F"},
    {"kopfschutteln", "1F642-200D-2194-FE0F"},
    {"kopfnicken", "1F642-200D-2195-FE0F"},
    {"balletttanzerin", "1F9D1-200D-1FA70"},
    {"balletttanzerin_farbe1", "1F9D1-1F3FB-200D-1FA70"},
    {"balletttanzerin_farbe2", "1F9D1-1F3FC-200D-1FA70"},
    {"balletttanzerin_farbe3", "1F9D1-1F3FD-200D-1FA70"},
    {"balletttanzerin_farbe4", "1F9D1-1F3FE-200D-1FA70"},
    {"balletttanzerin_farbe5", "1F9D1-1F3FF-200D-1FA70"},
    {"abgehaktes_kästchen", "2611"},
    {"heißgetränk", "2615"},
    {"biogefährdung", "2623"},
    {"düsteres_gesicht", "2639"},
    {"männersymbol", "2642"},
    {"schütze_sternzeichen", "2650"},
    {"heiße_quellen", "2668"},
    {"äskulapstab", "2695"},
    {"weißes_häkchen", "2705"},
    {"kräftiges_häkchen", "2714"},
    {"weißes_fragezeichen", "2754"},
    {"weißes_ausrufezeichen", "2755"},
    {"schriftzeichen_für_gratulation", "3297"},
    {"schriftzeichen_für_geheimnis", "3299"},
    {"zurückspulen", "23EA"},
    {"nächster_titel", "23ED"},
    {"kleines_weißes_quadrat", "25AB"},
    {"pfeil_zurück", "25C0"},
    {"mittelgroßes_weißes_quadrat", "25FB"},
    {"mittelgroßes_schwarzes_quadrat", "25FC"},
    {"mittelkleines_weißes_quadrat", "25FD"},
    {"lächelndes_gesicht", "263A"},
    {"löwe_sternzeichen", "264C"},
    {"weißer_punkt", "26AA"},
    {"fußball", "26BD"},
    {"schlangenträger", "26CE"},
    {"fähre", "26F4"},
    {"tanksäule", "26FD"},
    {"angekreuztes_kästchen", "274E"},
    {"großes_schwarzes_quadrat", "2B1B"},
    {"großes_weißes_quadrat", "2B1C"},
    {"weißer_mittelgroßer_stern", "2B50"},
    {"großbuchstabe_a_in_rotem_quadrat", "1F170"},
    {"großbuchstabe_b_in_rotem_quadrat", "1F171"},
    {"großbuchstabe_o_in_rotem_quadrat", "1F17E"},
    {"großbuchstabe_p_in_blauem_quadrat", "1F17F"},
    {"großbuchstaben_ab_in_rotem_quadrat", "1F18E"},
    {"großbuchstaben_cl_in_rotem_quadrat", "1F191"},
    {"großbuchstaben_id_in_lila_quadrat", "1F194"},
    {"großbuchstaben_ng_in_blauem_quadrat", "1F196"},
    {"großbuchstaben_ok_in_blauem_quadrat", "1F197"},
    {"schriftzeichen_für_gratis", "1F21A"},
    {"schriftzeichen_für_reserviert", "1F22F"},
    {"schriftzeichen_für_verbieten", "1F232"},
    {"schriftzeichen_für_zimmer_frei", "1F233"},
    {"schriftzeichen_für_note_zum_bestehen", "1F234"},
    {"schriftzeichen_für_kein_zimmer_frei", "1F235"},
    {"schriftzeichen_für_nicht_gratis", "1F236"},
    {"schriftzeichen_für_monatsbetrag", "1F237"},
    {"schriftzeichen_für_anwenden", "1F238"},
    {"schriftzeichen_für_rabatt", "1F239"},
    {"schriftzeichen_für_geöffnet", "1F23A"},
    {"schriftzeichen_für_schnäppchen", "1F250"},
    {"schriftzeichen_für_akzeptieren", "1F251"},
    {"sonnenaufgang_über_bergen", "1F304"},
    {"sonnenaufgang_über_dem_meer", "1F305"},
    {"brücke_vor_nachthimmel", "1F309"},
    {"milchstraße", "1F30C"},
    {"sonne_hinter_großer_wolke", "1F325"},
    {"kirschblüte", "1F338"},
    {"gelbe_blüte", "1F33C"},
    {"reisähre", "1F33E"},
    {"kräuter", "1F33F"},
    {"glücksklee", "1F340"},
    {"blätter_im_wind", "1F343"},
    {"grüner_apfel", "1F34F"},
    {"hähnchenschenkel", "1F357"},
    {"reisbällchen", "1F359"},
    {"reis_in_schüssel", "1F35A"},
    {"schüssel_und_essstäbchen", "1F35C"},
    {"geröstete_süßkartoffel", "1F360"},
    {"bierkrüge", "1F37B"},
    {"halloweenkürbis", "1F383"},
    {"überkreuzte_flaggen", "1F38C"},
    {"traditionelle_japanische_windsäcke", "1F38F"},
    {"militärorden", "1F396"},
    {"kopfhörer", "1F3A7"},
    {"spielwürfel", "1F3B2"},
    {"notenschlüssel", "1F3BC"},
    {"wohnhäuser", "1F3D8"},
    {"antikes_gebäude", "1F3DB"},
    {"wüste", "1F3DC"},
    {"bürogebäude", "1F3E2"},
    {"japanisches_postgebäude", "1F3E3"},
    {"postgebäude", "1F3E4"},
    {"weiße_flagge", "1F3F3"},
    {"wasserbüffel", "1F403"},
    {"marienkäfer", "1F41E"},
    {"schildkröte", "1F422"},
    {"schlüpfendes_küken", "1F423"},
    {"küken", "1F424"},
    {"küken_von_vorne", "1F425"},
    {"mäusegesicht", "1F42D"},
    {"bär", "1F43B"},
    {"schweinerüssel", "1F43D"},
    {"tatzenabdrücke", "1F43E"},
    {"streifenhörnchen", "1F43F"},
    {"klatschende_hände", "1F44F"},
    {"klatschende_hände_farbe1", "1F44F-1F3FB"},
    {"klatschende_hände_farbe2", "1F44F-1F3FC"},
    {"klatschende_hände_farbe3", "1F44F-1F3FD"},
    {"klatschende_hände_farbe4", "1F44F-1F3FE"},
    {"klatschende_hände_farbe5", "1F44F-1F3FF"},
    {"offene_hände", "1F450"},
    {"offene_hände_farbe1", "1F450-1F3FB"},
    {"offene_hände_farbe2", "1F450-1F3FC"},
    {"offene_hände_farbe3", "1F450-1F3FD"},
    {"offene_hände_farbe4", "1F450-1F3FE"},
    {"offene_hände_farbe5", "1F450-1F3FF"},
    {"geldbörse", "1F45B"},
    {"stöckelschuh", "1F460"},
    {"fußabdrücke", "1F463"},
    {"silhouette_einer_büste", "1F464"},
    {"silhouette_mehrerer_büsten", "1F465"},
    {"mädchen", "1F467"},
    {"mädchen_farbe1", "1F467-1F3FB"},
    {"mädchen_farbe2", "1F467-1F3FC"},
    {"mädchen_farbe3", "1F467-1F3FD"},
    {"mädchen_farbe4", "1F467-1F3FE"},
    {"mädchen_farbe5", "1F467-1F3FF"},
    {"mann_und_frau_halten_hände", "1F46B"},
    {"mann_und_frau_halten_hände_farbe1", "1F46B-1F3FB"},
    {"mann_und_frau_halten_hände_farbe2", "1F46B-1F3FC"},
    {"mann_und_frau_halten_hände_farbe3", "1F46B-1F3FD"},
    {"mann_und_frau_halten_hände_farbe4", "1F46B-1F3FE"},
    {"mann_und_frau_halten_hände_farbe5", "1F46B-1F3FF"},
    {"mann_und_frau_halten_hände_farbe1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mann_und_frau_halten_hände_farbe1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mann_und_frau_halten_hände_farbe1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mann_und_frau_halten_hände_farbe1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mann_und_frau_halten_hände_farbe2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mann_und_frau_halten_hände_farbe2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mann_und_frau_halten_hände_farbe2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mann_und_frau_halten_hände_farbe2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mann_und_frau_halten_hände_farbe3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mann_und_frau_halten_hände_farbe3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mann_und_frau_halten_hände_farbe3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mann_und_frau_halten_hände_farbe3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mann_und_frau_halten_hände_farbe4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mann_und_frau_halten_hände_farbe4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mann_und_frau_halten_hände_farbe4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mann_und_frau_halten_hände_farbe4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mann_und_frau_halten_hände_farbe5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mann_und_frau_halten_hände_farbe5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mann_und_frau_halten_hände_farbe5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mann_und_frau_halten_hände_farbe5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"händchen_haltende_männer", "1F46C"},
    {"händchen_haltende_männer_farbe1", "1F46C-1F3FB"},
    {"händchen_haltende_männer_farbe2", "1F46C-1F3FC"},
    {"händchen_haltende_männer_farbe3", "1F46C-1F3FD"},
    {"händchen_haltende_männer_farbe4", "1F46C-1F3FE"},
    {"händchen_haltende_männer_farbe5", "1F46C-1F3FF"},
    {"händchen_haltende_männer_farbe1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"händchen_haltende_männer_farbe1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"händchen_haltende_männer_farbe1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"händchen_haltende_männer_farbe1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"händchen_haltende_männer_farbe2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"händchen_haltende_männer_farbe2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"händchen_haltende_männer_farbe2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"händchen_haltende_männer_farbe2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"händchen_haltende_männer_farbe3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"händchen_haltende_männer_farbe3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"händchen_haltende_männer_farbe3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"händchen_haltende_männer_farbe3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"händchen_haltende_männer_farbe4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"händchen_haltende_männer_farbe4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"händchen_haltende_männer_farbe4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"händchen_haltende_männer_farbe4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"händchen_haltende_männer_farbe5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"händchen_haltende_männer_farbe5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"händchen_haltende_männer_farbe5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"händchen_haltende_männer_farbe5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"händchen_haltende_frauen", "1F46D"},
    {"händchen_haltende_frauen_farbe1", "1F46D-1F3FB"},
    {"händchen_haltende_frauen_farbe2", "1F46D-1F3FC"},
    {"händchen_haltende_frauen_farbe3", "1F46D-1F3FD"},
    {"händchen_haltende_frauen_farbe4", "1F46D-1F3FE"},
    {"händchen_haltende_frauen_farbe5", "1F46D-1F3FF"},
    {"händchen_haltende_frauen_farbe1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"händchen_haltende_frauen_farbe1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"händchen_haltende_frauen_farbe1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"händchen_haltende_frauen_farbe1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"händchen_haltende_frauen_farbe2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"händchen_haltende_frauen_farbe2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"händchen_haltende_frauen_farbe2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"händchen_haltende_frauen_farbe2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"händchen_haltende_frauen_farbe3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"händchen_haltende_frauen_farbe3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"händchen_haltende_frauen_farbe3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"händchen_haltende_frauen_farbe3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"händchen_haltende_frauen_farbe4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"händchen_haltende_frauen_farbe4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"händchen_haltende_frauen_farbe4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"händchen_haltende_frauen_farbe4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"händchen_haltende_frauen_farbe5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"händchen_haltende_frauen_farbe5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"händchen_haltende_frauen_farbe5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"händchen_haltende_frauen_farbe5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"älterer_mann", "1F474"},
    {"älterer_mann_farbe1", "1F474-1F3FB"},
    {"älterer_mann_farbe2", "1F474-1F3FC"},
    {"älterer_mann_farbe3", "1F474-1F3FD"},
    {"älterer_mann_farbe4", "1F474-1F3FE"},
    {"älterer_mann_farbe5", "1F474-1F3FF"},
    {"ältere_frau", "1F475"},
    {"ältere_frau_farbe1", "1F475-1F3FB"},
    {"ältere_frau_farbe2", "1F475-1F3FC"},
    {"ältere_frau_farbe3", "1F475-1F3FD"},
    {"ältere_frau_farbe4", "1F475-1F3FE"},
    {"ältere_frau_farbe5", "1F475-1F3FF"},
    {"außerirdischer", "1F47D"},
    {"wütendes_gesicht_mit_hörnern", "1F47F"},
    {"barbershop_säule", "1F488"},
    {"sich_küssendes_paar", "1F48F"},
    {"sich_küssendes_paar_farbe1", "1F48F-1F3FB"},
    {"sich_küssendes_paar_farbe2", "1F48F-1F3FC"},
    {"sich_küssendes_paar_farbe3", "1F48F-1F3FD"},
    {"sich_küssendes_paar_farbe4", "1F48F-1F3FE"},
    {"sich_küssendes_paar_farbe5", "1F48F-1F3FF"},
    {"sich_küssendes_paar_farbe1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"sich_küssendes_paar_farbe1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"sich_küssendes_paar_farbe1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"sich_küssendes_paar_farbe1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"sich_küssendes_paar_farbe2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"sich_küssendes_paar_farbe2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"sich_küssendes_paar_farbe2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"sich_küssendes_paar_farbe2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"sich_küssendes_paar_farbe3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"sich_küssendes_paar_farbe3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"sich_küssendes_paar_farbe3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"sich_küssendes_paar_farbe3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"sich_küssendes_paar_farbe4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"sich_küssendes_paar_farbe4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"sich_küssendes_paar_farbe4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"sich_küssendes_paar_farbe4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"sich_küssendes_paar_farbe5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"sich_küssendes_paar_farbe5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"sich_küssendes_paar_farbe5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"sich_küssendes_paar_farbe5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"blumenstrauß", "1F490"},
    {"grünes_herz", "1F49A"},
    {"glühbirne", "1F4A1"},
    {"ärger", "1F4A2"},
    {"zusammenstoß", "1F4A5"},
    {"schweißtropfen", "1F4A6"},
    {"geldschein_mit_flügeln", "1F4B8"},
    {"geöffneter_ordner", "1F4C2"},
    {"abreißkalender", "1F4C6"},
    {"aufwärtstrend", "1F4C8"},
    {"abwärtstrend", "1F4C9"},
    {"reißzwecke", "1F4CC"},
    {"büroklammer", "1F4CE"},
    {"grünes_buch", "1F4D7"},
    {"bücherstapel", "1F4DA"},
    {"telefonhörer", "1F4DE"},
    {"faxgerät", "1F4E0"},
    {"satellitenschüssel", "1F4E1"},
    {"balkenförmige_signalstärkenanzeige", "1F4F6"},
    {"kreisförmige_pfeile_im_uhrzeigersinn", "1F503"},
    {"lautsprecher_mit_geringer_lautstärke", "1F508"},
    {"lautsprecher_mit_mittlerer_lautstärke", "1F509"},
    {"lautsprecher_mit_hoher_lautstärke", "1F50A"},
    {"schloss_mit_füller", "1F50F"},
    {"schloss_mit_schlüssel", "1F510"},
    {"schlüssel", "1F511"},
    {"minderjährige_verboten", "1F51E"},
    {"eingabesymbol_lateinische_großbuchstaben", "1F520"},
    {"schraubenschlüssel", "1F527"},
    {"küchenmesser", "1F52A"},
    {"japanisches_anfänger_zeichen", "1F530"},
    {"schwarze_quadratische_schaltfläche", "1F532"},
    {"weiße_quadratische_schaltfläche", "1F533"},
    {"große_orangefarbene_raute", "1F536"},
    {"große_blaue_raute", "1F537"},
    {"aufwärts_schaltfläche", "1F53C"},
    {"abwärts_schaltfläche", "1F53D"},
    {"verhakte_büroklammern", "1F587"},
    {"füllhalter", "1F58B"},
    {"vulkanischer_gruß", "1F596"},
    {"vulkanischer_gruß_farbe1", "1F596-1F3FB"},
    {"vulkanischer_gruß_farbe2", "1F596-1F3FC"},
    {"vulkanischer_gruß_farbe3", "1F596-1F3FD"},
    {"vulkanischer_gruß_farbe4", "1F596-1F3FE"},
    {"vulkanischer_gruß_farbe5", "1F596-1F3FF"},
    {"alter_schlüssel", "1F5DD"},
    {"sprechblase_für_wütende_aussage_rechts", "1F5EF"},
    {"gesicht_mit_freudentränen", "1F602"},
    {"grinsendes_gesicht_mit_großen_augen", "1F603"},
    {"grinsendes_gesicht_mit_schweißtropfen", "1F605"},
    {"lächelndes_gesicht_mit_heiligenschein", "1F607"},
    {"grinsendes_gesicht_mit_hörnern", "1F608"},
    {"lächelndes_gesicht_mit_lachenden_augen", "1F60A"},
    {"lächelndes_gesicht_mit_herzförmigen_augen", "1F60D"},
    {"lächelndes_gesicht_mit_sonnenbrille", "1F60E"},
    {"süffisant_lächelndes_gesicht", "1F60F"},
    {"bedrücktes_gesicht_mit_schweiß", "1F613"},
    {"küssendes_gesicht", "1F617"},
    {"küssendes_gesicht_mit_lächelnden_augen", "1F619"},
    {"küssendes_gesicht_mit_geschlossenen_augen", "1F61A"},
    {"enttäuschtes_gesicht", "1F61E"},
    {"verärgertes_gesicht", "1F620"},
    {"ängstliches_gesicht", "1F628"},
    {"erschöpftes_gesicht", "1F629"},
    {"schläfriges_gesicht", "1F62A"},
    {"müdes_gesicht", "1F62B"},
    {"besorgtes_gesicht_mit_schweißtropfen", "1F630"},
    {"errötetes_gesicht_mit_großen_augen", "1F633"},
    {"katze_mit_freudentränen", "1F639"},
    {"verwegen_lächelnde_katze", "1F63C"},
    {"küssende_katze", "1F63D"},
    {"erschöpfte_katze", "1F640"},
    {"betrübtes_gesicht", "1F641"},
    {"leicht_lächelndes_gesicht", "1F642"},
    {"person_mit_überkreuzten_armen", "1F645"},
    {"person_mit_überkreuzten_armen_farbe1", "1F645-1F3FB"},
    {"person_mit_überkreuzten_armen_farbe2", "1F645-1F3FC"},
    {"person_mit_überkreuzten_armen_farbe3", "1F645-1F3FD"},
    {"person_mit_überkreuzten_armen_farbe4", "1F645-1F3FE"},
    {"person_mit_überkreuzten_armen_farbe5", "1F645-1F3FF"},
    {"person_mit_händen_auf_dem_kopf", "1F646"},
    {"person_mit_händen_auf_dem_kopf_farbe1", "1F646-1F3FB"},
    {"person_mit_händen_auf_dem_kopf_farbe2", "1F646-1F3FC"},
    {"person_mit_händen_auf_dem_kopf_farbe3", "1F646-1F3FD"},
    {"person_mit_händen_auf_dem_kopf_farbe4", "1F646-1F3FE"},
    {"person_mit_händen_auf_dem_kopf_farbe5", "1F646-1F3FF"},
    {"zwei_erhobene_handflächen", "1F64C"},
    {"zwei_erhobene_handflächen_farbe1", "1F64C-1F3FB"},
    {"zwei_erhobene_handflächen_farbe2", "1F64C-1F3FC"},
    {"zwei_erhobene_handflächen_farbe3", "1F64C-1F3FD"},
    {"zwei_erhobene_handflächen_farbe4", "1F64C-1F3FE"},
    {"zwei_erhobene_handflächen_farbe5", "1F64C-1F3FF"},
    {"zusammengelegte_handflächen", "1F64F"},
    {"zusammengelegte_handflächen_farbe1", "1F64F-1F3FB"},
    {"zusammengelegte_handflächen_farbe2", "1F64F-1F3FC"},
    {"zusammengelegte_handflächen_farbe3", "1F64F-1F3FD"},
    {"zusammengelegte_handflächen_farbe4", "1F64F-1F3FE"},
    {"zusammengelegte_handflächen_farbe5", "1F64F-1F3FF"},
    {"straßenbahn", "1F68A"},
    {"straßenbahnwagen", "1F68B"},
    {"tür", "1F6AA"},
    {"fahrräder_verboten", "1F6B3"},
    {"fußgängerin", "1F6B6-200D-2640-FE0F"},
    {"fußgängerin_farbe1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"fußgängerin_farbe2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"fußgängerin_farbe3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"fußgängerin_farbe4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"fußgängerin_farbe5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"fußgänger_verboten", "1F6B7"},
    {"kinder_überqueren_die_straße", "1F6B8"},
    {"gepäckausgabe", "1F6C4"},
    {"gepäckaufbewahrung", "1F6C5"},
    {"einkaufstüten", "1F6CD"},
    {"religiöse_stätte", "1F6D0"},
    {"hütte", "1F6D6"},
    {"hammer_und_schraubenschlüssel", "1F6E0"},
    {"ölfass", "1F6E2"},
    {"grüner_punkt", "1F7E2"},
    {"grünes_quadrat", "1F7E9"},
    {"zusammengedrückte_finger", "1F90C"},
    {"zusammengedrückte_finger_farbe1", "1F90C-1F3FB"},
    {"zusammengedrückte_finger_farbe2", "1F90C-1F3FC"},
    {"zusammengedrückte_finger_farbe3", "1F90C-1F3FD"},
    {"zusammengedrückte_finger_farbe4", "1F90C-1F3FE"},
    {"zusammengedrückte_finger_farbe5", "1F90C-1F3FF"},
    {"weißes_herz", "1F90D"},
    {"gesicht_mit_reißverschlussmund", "1F910"},
    {"gesicht_mit_umarmenden_händen", "1F917"},
    {"teufelsgruß", "1F918"},
    {"teufelsgruß_farbe1", "1F918-1F3FB"},
    {"teufelsgruß_farbe2", "1F918-1F3FC"},
    {"teufelsgruß_farbe3", "1F918-1F3FD"},
    {"teufelsgruß_farbe4", "1F918-1F3FE"},
    {"teufelsgruß_farbe5", "1F918-1F3FF"},
    {"würgendes_gesicht", "1F922"},
    {"sich_vor_lachen_auf_dem_boden_wälzen", "1F923"},
    {"lügendes_gesicht", "1F925"},
    {"überwältigt", "1F929"},
    {"gesicht_mit_symbolen_über_dem_mund", "1F92C"},
    {"handflächen_nach_oben", "1F932"},
    {"handflächen_nach_oben_farbe1", "1F932-1F3FB"},
    {"handflächen_nach_oben_farbe2", "1F932-1F3FC"},
    {"handflächen_nach_oben_farbe3", "1F932-1F3FD"},
    {"handflächen_nach_oben_farbe4", "1F932-1F3FE"},
    {"handflächen_nach_oben_farbe5", "1F932-1F3FF"},
    {"sektgläser", "1F942"},
    {"löffel", "1F944"},
    {"döner", "1F959"},
    {"glückskeks", "1F960"},
    {"essstäbchen", "1F962"},
    {"schüssel_mit_löffel", "1F963"},
    {"fleischstück", "1F969"},
    {"blattgemüse", "1F96C"},
    {"lächelndes_gesicht_mit_herzen", "1F970"},
    {"gähnendes_gesicht", "1F971"},
    {"lachendes_gesicht_mit_träne", "1F972"},
    {"gesicht_das_tränen_zurückhält", "1F979"},
    {"löwe", "1F981"},
    {"känguru", "1F998"},
    {"waschbär", "1F99D"},
    {"mücke", "1F99F"},
    {"weißes_haar", "1F9B3"},
    {"fuß", "1F9B6"},
    {"fuß_farbe1", "1F9B6-1F3FB"},
    {"fuß_farbe2", "1F9B6-1F3FC"},
    {"fuß_farbe3", "1F9B6-1F3FD"},
    {"fuß_farbe4", "1F9B6-1F3FE"},
    {"fuß_farbe5", "1F9B6-1F3FF"},
    {"bösewicht", "1F9B9"},
    {"bösewicht_farbe1", "1F9B9-1F3FB"},
    {"bösewicht_farbe2", "1F9B9-1F3FC"},
    {"bösewicht_farbe3", "1F9B9-1F3FD"},
    {"bösewicht_farbe4", "1F9B9-1F3FE"},
    {"bösewicht_farbe5", "1F9B9-1F3FF"},
    {"ohr_mit_hörgerät", "1F9BB"},
    {"ohr_mit_hörgerät_farbe1", "1F9BB-1F3FB"},
    {"ohr_mit_hörgerät_farbe2", "1F9BB-1F3FC"},
    {"ohr_mit_hörgerät_farbe3", "1F9BB-1F3FD"},
    {"ohr_mit_hörgerät_farbe4", "1F9BB-1F3FE"},
    {"ohr_mit_hörgerät_farbe5", "1F9BB-1F3FF"},
    {"käsestück", "1F9C0"},
    {"trinkpäckchen", "1F9C3"},
    {"eiswürfel", "1F9CA"},
    {"gehörlose_person", "1F9CF"},
    {"gehörlose_person_farbe1", "1F9CF-1F3FB"},
    {"gehörlose_person_farbe2", "1F9CF-1F3FC"},
    {"gehörlose_person_farbe3", "1F9CF-1F3FD"},
    {"gehörlose_person_farbe4", "1F9CF-1F3FE"},
    {"gehörlose_person_farbe5", "1F9CF-1F3FF"},
    {"ältere_person", "1F9D3"},
    {"ältere_person_farbe1", "1F9D3-1F3FB"},
    {"ältere_person_farbe2", "1F9D3-1F3FC"},
    {"ältere_person_farbe3", "1F9D3-1F3FD"},
    {"ältere_person_farbe4", "1F9D3-1F3FE"},
    {"ältere_person_farbe5", "1F9D3-1F3FF"},
    {"märchenfee", "1F9DA"},
    {"märchenfee_farbe1", "1F9DA-1F3FB"},
    {"märchenfee_farbe2", "1F9DA-1F3FC"},
    {"märchenfee_farbe3", "1F9DA-1F3FD"},
    {"märchenfee_farbe4", "1F9DA-1F3FE"},
    {"märchenfee_farbe5", "1F9DA-1F3FF"},
    {"baseballmütze", "1F9E2"},
    {"feuerwerkskörper", "1F9E8"},
    {"feuerlöscher", "1F9EF"},
    {"gepäck", "1F9F3"},
    {"wollknäuel", "1F9F6"},
    {"teddybär", "1F9F8"},
    {"küchenrolle", "1F9FB"},
    {"röntgenbild", "1FA7B"},
    {"krücke", "1FA7C"},
    {"piñata", "1FA85"},
    {"flöte", "1FA88"},
    {"öllampe", "1FA94"},
    {"militärhelm", "1FA96"},
    {"münze", "1FA99"},
    {"handsäge", "1FA9A"},
    {"nähnadel", "1FAA1"},
    {"zahnbürste", "1FAA5"},
    {"faltfächer", "1FAAD"},
    {"käfer", "1FAB2"},
    {"lotusblüte", "1FAB7"},
    {"flügel", "1FABD"},
    {"flüssigkeit_ausgießen", "1FAD7"},
    {"wurzelgemüse", "1FADC"},
    {"gesicht_mit_offenen_augen_und_hand_über_dem_mund", "1FAE2"},
    {"gesicht_mit_schrägem_mund", "1FAE4"},
    {"auf_lippe_beißen", "1FAE6"},
    {"hand_mit_handfläche_nach_unten", "1FAF3"},
    {"hand_mit_handfläche_nach_unten_farbe1", "1FAF3-1F3FB"},
    {"hand_mit_handfläche_nach_unten_farbe2", "1FAF3-1F3FC"},
    {"hand_mit_handfläche_nach_unten_farbe3", "1FAF3-1F3FD"},
    {"hand_mit_handfläche_nach_unten_farbe4", "1FAF3-1F3FE"},
    {"hand_mit_handfläche_nach_unten_farbe5", "1FAF3-1F3FF"},
    {"hand_mit_handfläche_nach_oben", "1FAF4"},
    {"hand_mit_handfläche_nach_oben_farbe1", "1FAF4-1F3FB"},
    {"hand_mit_handfläche_nach_oben_farbe2", "1FAF4-1F3FC"},
    {"hand_mit_handfläche_nach_oben_farbe3", "1FAF4-1F3FD"},
    {"hand_mit_handfläche_nach_oben_farbe4", "1FAF4-1F3FE"},
    {"hand_mit_handfläche_nach_oben_farbe5", "1FAF4-1F3FF"},
    {"hände_die_herz_bilden", "1FAF6"},
    {"hände_die_herz_bilden_farbe1", "1FAF6-1F3FB"},
    {"hände_die_herz_bilden_farbe2", "1FAF6-1F3FC"},
    {"hände_die_herz_bilden_farbe3", "1FAF6-1F3FD"},
    {"hände_die_herz_bilden_farbe4", "1FAF6-1F3FE"},
    {"hände_die_herz_bilden_farbe5", "1FAF6-1F3FF"},
    {"flagge_österreich", "1F1E6-1F1F9"},
    {"flagge_ålandinseln", "1F1E6-1F1FD"},
    {"flagge_st_barthélemy", "1F1E7-1F1F1"},
    {"flagge_côte_divoire", "1F1E8-1F1EE"},
    {"flagge_curaçao", "1F1E8-1F1FC"},
    {"flagge_dänemark", "1F1E9-1F1F0"},
    {"flagge_ägypten", "1F1EA-1F1EC"},
    {"flagge_äthiopien", "1F1EA-1F1F9"},
    {"flagge_europäische_union", "1F1EA-1F1FA"},
    {"flagge_färöer", "1F1EB-1F1F4"},
    {"flagge_vereinigtes_königreich", "1F1EC-1F1E7"},
    {"flagge_französisch_guayana", "1F1EC-1F1EB"},
    {"flagge_grönland", "1F1EC-1F1F1"},
    {"flagge_äquatorialguinea", "1F1EC-1F1F6"},
    {"flagge_südgeorgien_und_die_südlichen_sandwichinseln", "1F1EC-1F1F8"},
    {"flagge_südkorea", "1F1F0-1F1F7"},
    {"flagge_nördliche_marianen", "1F1F2-1F1F5"},
    {"flagge_französisch_polynesien", "1F1F5-1F1EB"},
    {"flagge_palästinensische_autonomiegebiete", "1F1F5-1F1F8"},
    {"flagge_réunion", "1F1F7-1F1EA"},
    {"flagge_rumänien", "1F1F7-1F1F4"},
    {"flagge_südsudan", "1F1F8-1F1F8"},
    {"flagge_são_tomé_und_príncipe", "1F1F8-1F1F9"},
    {"flagge_französische_süd_und_antarktisgebiete", "1F1F9-1F1EB"},
    {"flagge_türkei", "1F1F9-1F1F7"},
    {"flagge_amerikanische_überseeinseln", "1F1FA-1F1F2"},
    {"flagge_südafrika", "1F1FF-1F1E6"},
    {"sich_küssendes_paar_mann_mann", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"sich_küssendes_paar_mann_mann_farbe1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_küssendes_paar_mann_mann_farbe1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_küssendes_paar_mann_mann_farbe1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_küssendes_paar_mann_mann_farbe1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_küssendes_paar_mann_mann_farbe1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_küssendes_paar_mann_mann_farbe2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_küssendes_paar_mann_mann_farbe2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_küssendes_paar_mann_mann_farbe2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_küssendes_paar_mann_mann_farbe2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_küssendes_paar_mann_mann_farbe2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_küssendes_paar_mann_mann_farbe3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_küssendes_paar_mann_mann_farbe3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_küssendes_paar_mann_mann_farbe3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_küssendes_paar_mann_mann_farbe3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_küssendes_paar_mann_mann_farbe3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_küssendes_paar_mann_mann_farbe4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_küssendes_paar_mann_mann_farbe4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_küssendes_paar_mann_mann_farbe4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_küssendes_paar_mann_mann_farbe4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_küssendes_paar_mann_mann_farbe4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_küssendes_paar_mann_mann_farbe5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_küssendes_paar_mann_mann_farbe5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_küssendes_paar_mann_mann_farbe5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_küssendes_paar_mann_mann_farbe5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_küssendes_paar_mann_mann_farbe5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"familie_mann_mädchen", "1F468-200D-1F467"},
    {"familie_mann_mädchen_und_junge", "1F468-200D-1F467-200D-1F466"},
    {"familie_mann_mädchen_und_mädchen", "1F468-200D-1F467-200D-1F467"},
    {"familie_mann_mann_und_mädchen", "1F468-200D-1F468-200D-1F467"},
    {"familie_mann_mann_mädchen_und_junge", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"familie_mann_mann_mädchen_und_mädchen", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"familie_mann_frau_und_mädchen", "1F468-200D-1F469-200D-1F467"},
    {"familie_mann_frau_mädchen_und_junge", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"familie_mann_frau_mädchen_und_mädchen", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"sich_küssendes_paar_frau_mann", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"sich_küssendes_paar_frau_mann_farbe1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_küssendes_paar_frau_mann_farbe1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_küssendes_paar_frau_mann_farbe1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_küssendes_paar_frau_mann_farbe1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_küssendes_paar_frau_mann_farbe1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_küssendes_paar_frau_mann_farbe2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_küssendes_paar_frau_mann_farbe2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_küssendes_paar_frau_mann_farbe2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_küssendes_paar_frau_mann_farbe2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_küssendes_paar_frau_mann_farbe2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_küssendes_paar_frau_mann_farbe3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_küssendes_paar_frau_mann_farbe3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_küssendes_paar_frau_mann_farbe3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_küssendes_paar_frau_mann_farbe3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_küssendes_paar_frau_mann_farbe3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_küssendes_paar_frau_mann_farbe4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_küssendes_paar_frau_mann_farbe4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_küssendes_paar_frau_mann_farbe4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_küssendes_paar_frau_mann_farbe4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_küssendes_paar_frau_mann_farbe4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_küssendes_paar_frau_mann_farbe5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"sich_küssendes_paar_frau_mann_farbe5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"sich_küssendes_paar_frau_mann_farbe5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"sich_küssendes_paar_frau_mann_farbe5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"sich_küssendes_paar_frau_mann_farbe5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"sich_küssendes_paar_frau_frau", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"sich_küssendes_paar_frau_frau_farbe1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"sich_küssendes_paar_frau_frau_farbe1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"sich_küssendes_paar_frau_frau_farbe1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"sich_küssendes_paar_frau_frau_farbe1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"sich_küssendes_paar_frau_frau_farbe1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"sich_küssendes_paar_frau_frau_farbe2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"sich_küssendes_paar_frau_frau_farbe2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"sich_küssendes_paar_frau_frau_farbe2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"sich_küssendes_paar_frau_frau_farbe2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"sich_küssendes_paar_frau_frau_farbe2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"sich_küssendes_paar_frau_frau_farbe3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"sich_küssendes_paar_frau_frau_farbe3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"sich_küssendes_paar_frau_frau_farbe3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"sich_küssendes_paar_frau_frau_farbe3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"sich_küssendes_paar_frau_frau_farbe3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"sich_küssendes_paar_frau_frau_farbe4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"sich_küssendes_paar_frau_frau_farbe4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"sich_küssendes_paar_frau_frau_farbe4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"sich_küssendes_paar_frau_frau_farbe4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"sich_küssendes_paar_frau_frau_farbe4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"sich_küssendes_paar_frau_frau_farbe5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"sich_küssendes_paar_frau_frau_farbe5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"sich_küssendes_paar_frau_frau_farbe5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"sich_küssendes_paar_frau_frau_farbe5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"sich_küssendes_paar_frau_frau_farbe5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"familie_frau_mädchen", "1F469-200D-1F467"},
    {"familie_frau_mädchen_und_junge", "1F469-200D-1F467-200D-1F466"},
    {"familie_frau_mädchen_und_mädchen", "1F469-200D-1F467-200D-1F467"},
    {"familie_frau_frau_und_mädchen", "1F469-200D-1F469-200D-1F467"},
    {"familie_frau_frau_mädchen_und_junge", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"familie_frau_frau_mädchen_und_mädchen", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"sich_an_den_händen_haltende_personen", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"sich_an_den_händen_haltende_personen_farbe1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"sich_an_den_händen_haltende_personen_farbe1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"sich_an_den_händen_haltende_personen_farbe1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"sich_an_den_händen_haltende_personen_farbe1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"sich_an_den_händen_haltende_personen_farbe1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"sich_an_den_händen_haltende_personen_farbe2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"sich_an_den_händen_haltende_personen_farbe2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"sich_an_den_händen_haltende_personen_farbe2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"sich_an_den_händen_haltende_personen_farbe2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"sich_an_den_händen_haltende_personen_farbe2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"sich_an_den_händen_haltende_personen_farbe3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"sich_an_den_händen_haltende_personen_farbe3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"sich_an_den_händen_haltende_personen_farbe3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"sich_an_den_händen_haltende_personen_farbe3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"sich_an_den_händen_haltende_personen_farbe3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"sich_an_den_händen_haltende_personen_farbe4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"sich_an_den_händen_haltende_personen_farbe4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"sich_an_den_händen_haltende_personen_farbe4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"sich_an_den_händen_haltende_personen_farbe4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"sich_an_den_händen_haltende_personen_farbe4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"sich_an_den_händen_haltende_personen_farbe5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"sich_an_den_händen_haltende_personen_farbe5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"sich_an_den_händen_haltende_personen_farbe5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"sich_an_den_händen_haltende_personen_farbe5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"sich_an_den_händen_haltende_personen_farbe5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"sänger", "1F468-200D-1F3A4"},
    {"sänger_farbe1", "1F468-1F3FB-200D-1F3A4"},
    {"sänger_farbe2", "1F468-1F3FC-200D-1F3A4"},
    {"sänger_farbe3", "1F468-1F3FD-200D-1F3A4"},
    {"sänger_farbe4", "1F468-1F3FE-200D-1F3A4"},
    {"sänger_farbe5", "1F468-1F3FF-200D-1F3A4"},
    {"künstler", "1F468-200D-1F3A8"},
    {"künstler_farbe1", "1F468-1F3FB-200D-1F3A8"},
    {"künstler_farbe2", "1F468-1F3FC-200D-1F3A8"},
    {"künstler_farbe3", "1F468-1F3FD-200D-1F3A8"},
    {"künstler_farbe4", "1F468-1F3FE-200D-1F3A8"},
    {"künstler_farbe5", "1F468-1F3FF-200D-1F3A8"},
    {"büroangestellter", "1F9D1-200D-1F4BC"},
    {"büroangestellter_farbe1", "1F9D1-1F3FB-200D-1F4BC"},
    {"büroangestellter_farbe2", "1F9D1-1F3FC-200D-1F4BC"},
    {"büroangestellter_farbe3", "1F9D1-1F3FD-200D-1F4BC"},
    {"büroangestellter_farbe4", "1F9D1-1F3FE-200D-1F4BC"},
    {"büroangestellter_farbe5", "1F9D1-1F3FF-200D-1F4BC"},
    {"ärztin", "1F469-200D-2695-FE0F"},
    {"ärztin_farbe1", "1F469-1F3FB-200D-2695-FE0F"},
    {"ärztin_farbe2", "1F469-1F3FC-200D-2695-FE0F"},
    {"ärztin_farbe3", "1F469-1F3FD-200D-2695-FE0F"},
    {"ärztin_farbe4", "1F469-1F3FE-200D-2695-FE0F"},
    {"ärztin_farbe5", "1F469-1F3FF-200D-2695-FE0F"},
    {"bäuerin", "1F469-200D-1F33E"},
    {"bäuerin_farbe1", "1F469-1F3FB-200D-1F33E"},
    {"bäuerin_farbe2", "1F469-1F3FC-200D-1F33E"},
    {"bäuerin_farbe3", "1F469-1F3FD-200D-1F33E"},
    {"bäuerin_farbe4", "1F469-1F3FE-200D-1F33E"},
    {"bäuerin_farbe5", "1F469-1F3FF-200D-1F33E"},
    {"köchin", "1F469-200D-1F373"},
    {"köchin_farbe1", "1F469-1F3FB-200D-1F373"},
    {"köchin_farbe2", "1F469-1F3FC-200D-1F373"},
    {"köchin_farbe3", "1F469-1F3FD-200D-1F373"},
    {"köchin_farbe4", "1F469-1F3FE-200D-1F373"},
    {"köchin_farbe5", "1F469-1F3FF-200D-1F373"},
    {"sängerin", "1F9D1-200D-1F3A4"},
    {"sängerin_farbe1", "1F9D1-1F3FB-200D-1F3A4"},
    {"sängerin_farbe2", "1F9D1-1F3FC-200D-1F3A4"},
    {"sängerin_farbe3", "1F9D1-1F3FD-200D-1F3A4"},
    {"sängerin_farbe4", "1F9D1-1F3FE-200D-1F3A4"},
    {"sängerin_farbe5", "1F9D1-1F3FF-200D-1F3A4"},
    {"künstlerin", "1F9D1-200D-1F3A8"},
    {"künstlerin_farbe1", "1F9D1-1F3FB-200D-1F3A8"},
    {"künstlerin_farbe2", "1F9D1-1F3FC-200D-1F3A8"},
    {"künstlerin_farbe3", "1F9D1-1F3FD-200D-1F3A8"},
    {"künstlerin_farbe4", "1F9D1-1F3FE-200D-1F3A8"},
    {"künstlerin_farbe5", "1F9D1-1F3FF-200D-1F3A8"},
    {"büroangestellte", "1F469-200D-1F4BC"},
    {"büroangestellte_farbe1", "1F469-1F3FB-200D-1F4BC"},
    {"büroangestellte_farbe2", "1F469-1F3FC-200D-1F4BC"},
    {"büroangestellte_farbe3", "1F469-1F3FD-200D-1F4BC"},
    {"büroangestellte_farbe4", "1F469-1F3FE-200D-1F4BC"},
    {"büroangestellte_farbe5", "1F469-1F3FF-200D-1F4BC"},
    {"fußgängerin_nach_rechts", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"fußgängerin_nach_rechts_farbe1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"fußgängerin_nach_rechts_farbe2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"fußgängerin_nach_rechts_farbe3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"fußgängerin_nach_rechts_farbe4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"fußgängerin_nach_rechts_farbe5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"arzt_ärztin", "1F9D1-200D-2695-FE0F"},
    {"arzt_ärztin_farbe1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"arzt_ärztin_farbe2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"arzt_ärztin_farbe3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"arzt_ärztin_farbe4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"arzt_ärztin_farbe5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"bauer_bäuerin", "1F9D1-200D-1F33E"},
    {"bauer_bäuerin_farbe1", "1F9D1-1F3FB-200D-1F33E"},
    {"bauer_bäuerin_farbe2", "1F9D1-1F3FC-200D-1F33E"},
    {"bauer_bäuerin_farbe3", "1F9D1-1F3FD-200D-1F33E"},
    {"bauer_bäuerin_farbe4", "1F9D1-1F3FE-200D-1F33E"},
    {"bauer_bäuerin_farbe5", "1F9D1-1F3FF-200D-1F33E"},
    {"koch_köchin", "1F9D1-200D-1F373"},
    {"koch_köchin_farbe1", "1F9D1-1F3FB-200D-1F373"},
    {"koch_köchin_farbe2", "1F9D1-1F3FC-200D-1F373"},
    {"koch_köchin_farbe3", "1F9D1-1F3FD-200D-1F373"},
    {"koch_köchin_farbe4", "1F9D1-1F3FE-200D-1F373"},
    {"koch_köchin_farbe5", "1F9D1-1F3FF-200D-1F373"},
    {"männer_mit_hasenohren", "1F46F-200D-2642-FE0F"},
    {"männer_mit_hasenohren_farbe1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"männer_mit_hasenohren_farbe1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"männer_mit_hasenohren_farbe1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"männer_mit_hasenohren_farbe1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"männer_mit_hasenohren_farbe2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"männer_mit_hasenohren_farbe2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"männer_mit_hasenohren_farbe2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"männer_mit_hasenohren_farbe2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"männer_mit_hasenohren_farbe3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"männer_mit_hasenohren_farbe3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"männer_mit_hasenohren_farbe3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"männer_mit_hasenohren_farbe3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"männer_mit_hasenohren_farbe4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"männer_mit_hasenohren_farbe4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"männer_mit_hasenohren_farbe4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"männer_mit_hasenohren_farbe4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"männer_mit_hasenohren_farbe5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"männer_mit_hasenohren_farbe5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"männer_mit_hasenohren_farbe5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"männer_mit_hasenohren_farbe5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"männer_mit_hasenohren_farbe1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"männer_mit_hasenohren_farbe2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"männer_mit_hasenohren_farbe3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"männer_mit_hasenohren_farbe4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"männer_mit_hasenohren_farbe5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"frau_mit_überkreuzten_armen", "1F645-200D-2640-FE0F"},
    {"frau_mit_überkreuzten_armen_farbe1", "1F645-1F3FB-200D-2640-FE0F"},
    {"frau_mit_überkreuzten_armen_farbe2", "1F645-1F3FC-200D-2640-FE0F"},
    {"frau_mit_überkreuzten_armen_farbe3", "1F645-1F3FD-200D-2640-FE0F"},
    {"frau_mit_überkreuzten_armen_farbe4", "1F645-1F3FE-200D-2640-FE0F"},
    {"frau_mit_überkreuzten_armen_farbe5", "1F645-1F3FF-200D-2640-FE0F"},
    {"mann_mit_überkreuzten_armen", "1F645-200D-2642-FE0F"},
    {"mann_mit_überkreuzten_armen_farbe1", "1F645-1F3FB-200D-2642-FE0F"},
    {"mann_mit_überkreuzten_armen_farbe2", "1F645-1F3FC-200D-2642-FE0F"},
    {"mann_mit_überkreuzten_armen_farbe3", "1F645-1F3FD-200D-2642-FE0F"},
    {"mann_mit_überkreuzten_armen_farbe4", "1F645-1F3FE-200D-2642-FE0F"},
    {"mann_mit_überkreuzten_armen_farbe5", "1F645-1F3FF-200D-2642-FE0F"},
    {"frau_mit_händen_auf_dem_kopf", "1F646-200D-2640-FE0F"},
    {"frau_mit_händen_auf_dem_kopf_farbe1", "1F646-1F3FB-200D-2640-FE0F"},
    {"frau_mit_händen_auf_dem_kopf_farbe2", "1F646-1F3FC-200D-2640-FE0F"},
    {"frau_mit_händen_auf_dem_kopf_farbe3", "1F646-1F3FD-200D-2640-FE0F"},
    {"frau_mit_händen_auf_dem_kopf_farbe4", "1F646-1F3FE-200D-2640-FE0F"},
    {"frau_mit_händen_auf_dem_kopf_farbe5", "1F646-1F3FF-200D-2640-FE0F"},
    {"mann_mit_händen_auf_dem_kopf", "1F646-200D-2642-FE0F"},
    {"mann_mit_händen_auf_dem_kopf_farbe1", "1F646-1F3FB-200D-2642-FE0F"},
    {"mann_mit_händen_auf_dem_kopf_farbe2", "1F646-1F3FC-200D-2642-FE0F"},
    {"mann_mit_händen_auf_dem_kopf_farbe3", "1F646-1F3FD-200D-2642-FE0F"},
    {"mann_mit_händen_auf_dem_kopf_farbe4", "1F646-1F3FE-200D-2642-FE0F"},
    {"mann_mit_händen_auf_dem_kopf_farbe5", "1F646-1F3FF-200D-2642-FE0F"},
    {"fußgänger", "1F6B6-200D-2642-FE0F"},
    {"fußgänger_farbe1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"fußgänger_farbe2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"fußgänger_farbe3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"fußgänger_farbe4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"fußgänger_farbe5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"fußgänger_nach_rechts", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"fußgänger_nach_rechts_farbe1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"fußgänger_nach_rechts_farbe2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"fußgänger_nach_rechts_farbe3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"fußgänger_nach_rechts_farbe4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"fußgänger_nach_rechts_farbe5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"ringende_männer", "1F93C-200D-2642-FE0F"},
    {"ringende_männer_farbe1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"ringende_männer_farbe1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"ringende_männer_farbe1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"ringende_männer_farbe1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"ringende_männer_farbe2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"ringende_männer_farbe2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"ringende_männer_farbe2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"ringende_männer_farbe2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"ringende_männer_farbe3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"ringende_männer_farbe3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"ringende_männer_farbe3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"ringende_männer_farbe3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"ringende_männer_farbe4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"ringende_männer_farbe4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"ringende_männer_farbe4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"ringende_männer_farbe4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"ringende_männer_farbe5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"ringende_männer_farbe5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"ringende_männer_farbe5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"ringende_männer_farbe5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"ringende_männer_farbe1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"ringende_männer_farbe2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"ringende_männer_farbe3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"ringende_männer_farbe4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"ringende_männer_farbe5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"weiblicher_bösewicht", "1F9B9-200D-2640-FE0F"},
    {"weiblicher_bösewicht_farbe1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"weiblicher_bösewicht_farbe2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"weiblicher_bösewicht_farbe3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"weiblicher_bösewicht_farbe4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"weiblicher_bösewicht_farbe5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"männlicher_bösewicht", "1F9B9-200D-2642-FE0F"},
    {"männlicher_bösewicht_farbe1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"männlicher_bösewicht_farbe2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"männlicher_bösewicht_farbe3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"männlicher_bösewicht_farbe4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"männlicher_bösewicht_farbe5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"gehörlose_frau", "1F9CF-200D-2640-FE0F"},
    {"gehörlose_frau_farbe1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"gehörlose_frau_farbe2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"gehörlose_frau_farbe3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"gehörlose_frau_farbe4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"gehörlose_frau_farbe5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"gehörloser_mann", "1F9CF-200D-2642-FE0F"},
    {"gehörloser_mann_farbe1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"gehörloser_mann_farbe2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"gehörloser_mann_farbe3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"gehörloser_mann_farbe4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"gehörloser_mann_farbe5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"männliche_fee", "1F9DA-200D-2642-FE0F"},
    {"männliche_fee_farbe1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"männliche_fee_farbe2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"männliche_fee_farbe3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"männliche_fee_farbe4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"männliche_fee_farbe5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"männlicher_vampir", "1F9DB-200D-2642-FE0F"},
    {"männlicher_vampir_farbe1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"männlicher_vampir_farbe2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"männlicher_vampir_farbe3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"männlicher_vampir_farbe4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"männlicher_vampir_farbe5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"männlicher_flaschengeist", "1F9DE-200D-2642-FE0F"},
    {"männlicher_zombie", "1F9DF-200D-2642-FE0F"},
    {"mann_weißes_haar", "1F468-200D-1F9B3"},
    {"mann_weißes_haar_farbe1", "1F468-1F3FB-200D-1F9B3"},
    {"mann_weißes_haar_farbe2", "1F468-1F3FC-200D-1F9B3"},
    {"mann_weißes_haar_farbe3", "1F468-1F3FD-200D-1F9B3"},
    {"mann_weißes_haar_farbe4", "1F468-1F3FE-200D-1F9B3"},
    {"mann_weißes_haar_farbe5", "1F468-1F3FF-200D-1F9B3"},
    {"frau_weißes_haar", "1F469-200D-1F9B3"},
    {"frau_weißes_haar_farbe1", "1F469-1F3FB-200D-1F9B3"},
    {"frau_weißes_haar_farbe2", "1F469-1F3FC-200D-1F9B3"},
    {"frau_weißes_haar_farbe3", "1F469-1F3FD-200D-1F9B3"},
    {"frau_weißes_haar_farbe4", "1F469-1F3FE-200D-1F9B3"},
    {"frau_weißes_haar_farbe5", "1F469-1F3FF-200D-1F9B3"},
    {"person_weißes_haar", "1F9D1-200D-1F9B3"},
    {"person_weißes_haar_farbe1", "1F9D1-1F3FB-200D-1F9B3"},
    {"person_weißes_haar_farbe2", "1F9D1-1F3FC-200D-1F9B3"},
    {"person_weißes_haar_farbe3", "1F9D1-1F3FD-200D-1F9B3"},
    {"person_weißes_haar_farbe4", "1F9D1-1F3FE-200D-1F9B3"},
    {"person_weißes_haar_farbe5", "1F9D1-1F3FF-200D-1F9B3"},
    {"phönix", "1F426-200D-1F525"},
    {"eisbär", "1F43B-200D-2744-FE0F"},
    {"kopfschütteln", "1F642-200D-2194-FE0F"},
    {"balletttänzerin", "1F9D1-200D-1FA70"},
    {"balletttänzerin_farbe1", "1F9D1-1F3FB-200D-1FA70"},
    {"balletttänzerin_farbe2", "1F9D1-1F3FC-200D-1FA70"},
    {"balletttänzerin_farbe3", "1F9D1-1F3FD-200D-1FA70"},
    {"balletttänzerin_farbe4", "1F9D1-1F3FE-200D-1FA70"},
    {"balletttänzerin_farbe5", "1F9D1-1F3FF-200D-1FA70"},
};

