// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_pl[] = {
    {"wykrzyknik_ze_znakiem_zapytania", "2049"},
    {"znak_towarowy", "2122"},
    {"informacja", "2139"},
    {"strzalka_w_lewo_i_w_prawo", "2194"},
    {"strzalka_w_gore_i_w_dol", "2195"},
    {"strzalka_w_gore_w_lewo", "2196"},
    {"strzalka_w_gore_w_prawo", "2197"},
    {"strzalka_w_dol_w_prawo", "2198"},
    {"strzalka_w_dol_w_lewo", "2199"},
    {"klawiatura", "2328"},
    {"slonce", "2600"},
    {"chmura", "2601"},
    {"parasol", "2602"},
    {"balwanek_i_platki_sniegu", "2603"},
    {"kometa", "2604"},
    {"pole_ze_znacznikiem_wyboru", "2611"},
    {"parasol_z_kroplami_deszczu", "2614"},
    {"goracy_napoj", "2615"},
    {"koniczyna", "2618"},
    {"czaszka_z_piszczelami", "2620"},
    {"promieniowanie_radioaktywne", "2622"},
    {"zagrozenie_ze_strony_organizmow_zywych", "2623"},
    {"krzyz_prawoslawny", "2626"},
    {"kolo_dharmy", "2638"},
    {"zachmurzona_twarz", "2639"},
    {"znak_kobiety", "2640"},
    {"znak_mezczyzny", "2642"},
    {"baran", "2648"},
    {"byk", "2649"},
    {"strzelec", "2650"},
    {"koziorozec", "2651"},
    {"wodnik", "2652"},
    {"ryby", "2653"},
    {"pik", "2660"},
    {"trefl", "2663"},
    {"kier", "2665"},
    {"karo", "2666"},
    {"gorace_zrodla", "2668"},
    {"mloty", "2692"},
    {"kotwica", "2693"},
    {"skrzyzowane_miecze", "2694"},
    {"symbol_medycyny", "2695"},
    {"waga_szalkowa", "2696"},
    {"alembik", "2697"},
    {"kolo_zebate", "2699"},
    {"nozyczki", "2702"},
    {"przycisk_ze_znacznikiem_wyboru", "2705"},
    {"samolot", "2708"},
    {"koperta", "2709"},
    {"pioro_z_czarna_stalowka", "2712"},
    {"znacznik_wyboru", "2714"},
    {"znak_mnozenia", "2716"},
    {"gwiazda_dawida", "2721"},
    {"gwiazdki", "2728"},
    {"gwiazdka_osmioramienna", "2733"},
    {"gwiazda_osmioramienna", "2734"},
    {"platek_sniegu", "2744"},
    {"iskra", "2747"},
    {"czerwony_znak_zapytania", "2753"},
    {"bialy_znak_zapytania", "2754"},
    {"bialy_wykrzyknik", "2755"},
    {"czerwony_wykrzyknik", "2757"},
    {"wykrzyknik_w_ksztalcie_serca", "2763"},
    {"czerwone_serce", "2764"},
    {"znak_plus", "2795"},
    {"znak_minus", "2796"},
    {"znak_dzielenia", "2797"},
    {"strzalka_w_prawo_skrecajaca_w_gore", "2934"},
    {"strzalka_w_prawo_skrecajaca_w_dol", "2935"},
    {"falista_kreska", "3030"},
    {"japonski_przycisk_gratulacje", "3297"},
    {"japonski_przycisk_tajemnica", "3299"},
    {"znak_praw_autorskich", "00A9"},
    {"zarejestrowany_znak_towarowy", "00AE"},
    {"podwojny_wykrzyknik", "203C"},
    {"strzalka_zakrecona_w_lewo", "21A9"},
    {"strzalka_zakrecona_w_prawo", "21AA"},
    {"zegarek", "231A"},
    {"klepsydra", "231B"},
    {"wysun", "23CF"},
    {"przycisk_szybkiego_przewijania_do_przodu", "23E9"},
    {"przycisk_szybkiego_przewijania_do_tylu", "23EA"},
    {"przycisk_szybko_w_gore", "23EB"},
    {"przycisk_szybko_w_dol", "23EC"},
    {"nastepny_utwor", "23ED"},
    {"poprzedni_utwor", "23EE"},
    {"odtworz_lub_wstrzymaj", "23EF"},
    {"budzik", "23F0"},
    {"stoper", "23F1"},
    {"minutnik", "23F2"},
    {"klepsydra_z_przesypujacym_sie_piaskiem", "23F3"},
    {"pauza", "23F8"},
    {"zatrzymaj", "23F9"},
    {"nagraj", "23FA"},
    {"kolo_z_litera_m", "24C2"},
    {"maly_czarny_kwadrat", "25AA"},
    {"maly_bialy_kwadrat", "25AB"},
    {"odtworz", "25B6"},
    {"wstecz", "25C0"},
    {"sredni_bialy_kwadrat", "25FB"},
    {"sredni_czarny_kwadrat", "25FC"},
    {"maly_sredni_bialy_kwadrat", "25FD"},
    {"maly_sredni_czarny_kwadrat", "25FE"},
    {"telefon", "260E"},
    {"palec_wskazujacy_w_gore", "261D"},
    {"palec_wskazujacy_w_gore_kolor1", "261D-1F3FB"},
    {"palec_wskazujacy_w_gore_kolor2", "261D-1F3FC"},
    {"palec_wskazujacy_w_gore_kolor3", "261D-1F3FD"},
    {"palec_wskazujacy_w_gore_kolor4", "261D-1F3FE"},
    {"palec_wskazujacy_w_gore_kolor5", "261D-1F3FF"},
    {"gwiazda_i_polksiezyc", "262A"},
    {"symbol_pokoju", "262E"},
    {"yin_yang", "262F"},
    {"usmiechnieta_twarz", "263A"},
    {"bliznieta", "264A"},
    {"rak", "264B"},
    {"lew", "264C"},
    {"panna", "264D"},
    {"waga", "264E"},
    {"skorpion", "264F"},
    {"pionek", "265F"},
    {"recykling", "267B"},
    {"nieskonczonosc", "267E"},
    {"symbol_wozka_inwalidzkiego", "267F"},
    {"symbol_atomu", "269B"},
    {"lilijka", "269C"},
    {"ostrzezenie", "26A0"},
    {"wysokie_napiecie", "26A1"},
    {"symbol_transplciowosci", "26A7"},
    {"biale_kolo", "26AA"},
    {"czarne_kolo", "26AB"},
    {"trumna", "26B0"},
    {"urna_na_prochy", "26B1"},
    {"pilka_nozna", "26BD"},
    {"baseball", "26BE"},
    {"balwanek_bez_sniegu", "26C4"},
    {"slonce_zza_chmury", "26C5"},
    {"chmura_z_piorunem_i_deszczem", "26C8"},
    {"wezownik", "26CE"},
    {"kilof", "26CF"},
    {"kask_z_krzyzykiem", "26D1"},
    {"lancuchy", "26D3"},
    {"zakaz_wjazdu", "26D4"},
    {"swiatynia_shinto", "26E9"},
    {"kosciol", "26EA"},
    {"gora", "26F0"},
    {"parasol_plazowy_wbity_w_ziemie", "26F1"},
    {"fontanna", "26F2"},
    {"flaga_w_dolku", "26F3"},
    {"prom", "26F4"},
    {"zaglowka", "26F5"},
    {"narciarz", "26F7"},
    {"but_z_lyzwa", "26F8"},
    {"osoba_kozlujaca_pilke", "26F9"},
    {"osoba_kozlujaca_pilke_kolor1", "26F9-1F3FB"},
    {"osoba_kozlujaca_pilke_kolor2", "26F9-1F3FC"},
    {"osoba_kozlujaca_pilke_kolor3", "26F9-1F3FD"},
    {"osoba_kozlujaca_pilke_kolor4", "26F9-1F3FE"},
    {"osoba_kozlujaca_pilke_kolor5", "26F9-1F3FF"},
    {"namiot", "26FA"},
    {"dystrybutor_paliwa", "26FD"},
    {"wzniesiona_piesc", "270A"},
    {"wzniesiona_piesc_kolor1", "270A-1F3FB"},
    {"wzniesiona_piesc_kolor2", "270A-1F3FC"},
    {"wzniesiona_piesc_kolor3", "270A-1F3FD"},
    {"wzniesiona_piesc_kolor4", "270A-1F3FE"},
    {"wzniesiona_piesc_kolor5", "270A-1F3FF"},
    {"wzniesiona_dlon", "270B"},
    {"wzniesiona_dlon_kolor1", "270B-1F3FB"},
    {"wzniesiona_dlon_kolor2", "270B-1F3FC"},
    {"wzniesiona_dlon_kolor3", "270B-1F3FD"},
    {"wzniesiona_dlon_kolor4", "270B-1F3FE"},
    {"wzniesiona_dlon_kolor5", "270B-1F3FF"},
    {"gest_v", "270C"},
    {"gest_v_kolor1", "270C-1F3FB"},
    {"gest_v_kolor2", "270C-1F3FC"},
    {"gest_v_kolor3", "270C-1F3FD"},
    {"gest_v_kolor4", "270C-1F3FE"},
    {"gest_v_kolor5", "270C-1F3FF"},
    {"piszaca_dlon", "270D"},
    {"piszaca_dlon_kolor1", "270D-1F3FB"},
    {"piszaca_dlon_kolor2", "270D-1F3FC"},
    {"piszaca_dlon_kolor3", "270D-1F3FD"},
    {"piszaca_dlon_kolor4", "270D-1F3FE"},
    {"piszaca_dlon_kolor5", "270D-1F3FF"},
    {"olowek", "270F"},
    {"krzyz_lacinski", "271D"},
    {"znak_krzyzyka", "274C"},
    {"przycisk_z_krzyzykiem", "274E"},
    {"strzalka_w_prawo", "27A1"},
    {"petla", "27B0"},
    {"podwojna_petla", "27BF"},
    {"strzalka_w_lewo", "2B05"},
    {"strzalka_w_gore", "2B06"},
    {"strzalka_w_dol", "2B07"},
    {"duzy_czarny_kwadrat", "2B1B"},
    {"duzy_bialy_kwadrat", "2B1C"},
    {"gwiazda", "2B50"},
    {"gruby_czerwony_okrag", "2B55"},
    {"poczatek_partii_wokalnej", "303D"},
    {"madzong_czerwony_smok", "1F004"},
    {"dzoker", "1F0CF"},
    {"grupa_krwi_a", "1F170"},
    {"grupa_krwi_b", "1F171"},
    {"grupa_krwi_0", "1F17E"},
    {"parking", "1F17F"},
    {"grupa_krwi_ab", "1F18E"},
    {"przycisk_cl", "1F191"},
    {"przycisk_cool", "1F192"},
    {"przycisk_free", "1F193"},
    {"przycisk_id", "1F194"},
    {"przycisk_new", "1F195"},
    {"przycisk_ng", "1F196"},
    {"przycisk_ok", "1F197"},
    {"przycisk_sos", "1F198"},
    {"przycisk_up", "1F199"},
    {"przycisk_vs", "1F19A"},
    {"wskaznik_regionalny_a", "1F1E6"},
    {"wskaznik_regionalny_b", "1F1E7"},
    {"wskaznik_regionalny_c", "1F1E8"},
    {"wskaznik_regionalny_d", "1F1E9"},
    {"wskaznik_regionalny_e", "1F1EA"},
    {"wskaznik_regionalny_f", "1F1EB"},
    {"wskaznik_regionalny_g", "1F1EC"},
    {"wskaznik_regionalny_h", "1F1ED"},
    {"wskaznik_regionalny_i", "1F1EE"},
    {"wskaznik_regionalny_j", "1F1EF"},
    {"wskaznik_regionalny_k", "1F1F0"},
    {"wskaznik_regionalny_l", "1F1F1"},
    {"wskaznik_regionalny_m", "1F1F2"},
    {"wskaznik_regionalny_n", "1F1F3"},
    {"wskaznik_regionalny_o", "1F1F4"},
    {"wskaznik_regionalny_p", "1F1F5"},
    {"wskaznik_regionalny_q", "1F1F6"},
    {"wskaznik_regionalny_r", "1F1F7"},
    {"wskaznik_regionalny_s", "1F1F8"},
    {"wskaznik_regionalny_t", "1F1F9"},
    {"wskaznik_regionalny_u", "1F1FA"},
    {"wskaznik_regionalny_v", "1F1FB"},
    {"wskaznik_regionalny_w", "1F1FC"},
    {"wskaznik_regionalny_x", "1F1FD"},
    {"wskaznik_regionalny_y", "1F1FE"},
    {"wskaznik_regionalny_z", "1F1FF"},
    {"japonski_przycisk_tutaj", "1F201"},
    {"japonski_przycisk_usluga", "1F202"},
    {"japonski_przycisk_bezplatne", "1F21A"},
    {"japonski_przycisk_zarezerwowane", "1F22F"},
    {"japonski_przycisk_zabronione", "1F232"},
    {"japonski_przycisk_wolne_miejsce", "1F233"},
    {"japonski_przycisk_ocena_pozytywna", "1F234"},
    {"japonski_przycisk_brak_wolnych_miejsc", "1F235"},
    {"japonski_przycisk_platne", "1F236"},
    {"ideogram_ksiezyca", "1F237"},
    {"japonski_przycisk_wniosek", "1F238"},
    {"japonski_przycisk_rabat", "1F239"},
    {"japonski_przycisk_czynne", "1F23A"},
    {"japonski_przycisk_okazja", "1F250"},
    {"japonski_przycisk_dozwolone", "1F251"},
    {"cyklon", "1F300"},
    {"zamglenie", "1F301"},
    {"zlozony_parasol", "1F302"},
    {"noc_z_gwiazdami", "1F303"},
    {"wschod_slonca_zza_gor", "1F304"},
    {"wschod_slonca", "1F305"},
    {"miasto_o_zmierzchu", "1F306"},
    {"zachod_slonca", "1F307"},
    {"tecza", "1F308"},
    {"most_noca", "1F309"},
    {"fala", "1F30A"},
    {"wulkan", "1F30B"},
    {"droga_mleczna", "1F30C"},
    {"kula_ziemska_przedstawiajaca_europe_i_afryke", "1F30D"},
    {"kula_ziemska_przedstawiajaca_ameryki", "1F30E"},
    {"kula_ziemska_przedstawiajaca_azje_i_australie", "1F30F"},
    {"kula_ziemska_z_poludnikami", "1F310"},
    {"now", "1F311"},
    {"narastajacy_polksiezyc", "1F312"},
    {"pierwsza_kwadra_ksiezyca", "1F313"},
    {"narastajacy_ksiezyc_miedzy_pierwsza_kwadra_a_pelnia", "1F314"},
    {"pelnia", "1F315"},
    {"zanikajacy_ksiezyc_miedzy_pelnia_a_ostatnia_kwadra", "1F316"},
    {"ostatnia_kwadra_ksiezyca", "1F317"},
    {"zanikajacy_polksiezyc", "1F318"},
    {"polksiezyc", "1F319"},
    {"now_z_twarza", "1F31A"},
    {"pierwsza_kwadra_ksiezyca_z_twarza", "1F31B"},
    {"ostatnia_kwadra_ksiezyca_z_twarza", "1F31C"},
    {"pelnia_z_twarza", "1F31D"},
    {"slonce_z_twarza", "1F31E"},
    {"blyszczaca_gwiazda", "1F31F"},
    {"spadajaca_gwiazda", "1F320"},
    {"termometr", "1F321"},
    {"slonce_za_chmurka", "1F324"},
    {"slonce_za_chmura", "1F325"},
    {"slonce_za_chmura_i_deszcz", "1F326"},
    {"chmura_i_deszcz", "1F327"},
    {"chmura_i_snieg", "1F328"},
    {"chmura_i_piorun", "1F329"},
    {"tornado", "1F32A"},
    {"mgla", "1F32B"},
    {"dmuchajaca_twarz_wiatru", "1F32C"},
    {"hot_dog", "1F32D"},
    {"taco", "1F32E"},
    {"burrito", "1F32F"},
    {"kasztan", "1F330"},
    {"sadzonka", "1F331"},
    {"wiecznie_zielone_drzewo", "1F332"},
    {"drzewo_lisciaste", "1F333"},
    {"palma", "1F334"},
    {"kaktus", "1F335"},
    {"ostra_papryka", "1F336"},
    {"tulipan", "1F337"},
    {"kwiat_wisni", "1F338"},
    {"roza", "1F339"},
    {"kwiat_hibiskusa", "1F33A"},
    {"slonecznik", "1F33B"},
    {"kwiat", "1F33C"},
    {"kolba_kukurydzy", "1F33D"},
    {"pedy_ryzu", "1F33E"},
    {"ziolo", "1F33F"},
    {"czterolistna_koniczyna", "1F340"},
    {"lisc_klonowy", "1F341"},
    {"spadajace_liscie", "1F342"},
    {"liscie_na_wietrze", "1F343"},
    {"grzyb", "1F344"},
    {"pomidor", "1F345"},
    {"baklazan", "1F346"},
    {"winogrona", "1F347"},
    {"melon", "1F348"},
    {"arbuz", "1F349"},
    {"mandarynka", "1F34A"},
    {"cytryna", "1F34B"},
    {"banan", "1F34C"},
    {"ananas", "1F34D"},
    {"czerwone_jablko", "1F34E"},
    {"zielone_jablko", "1F34F"},
    {"gruszka", "1F350"},
    {"brzoskwinia", "1F351"},
    {"wisnie", "1F352"},
    {"truskawka", "1F353"},
    {"hamburger", "1F354"},
    {"pizza", "1F355"},
    {"mieso_z_koscia", "1F356"},
    {"udko_kurczaka", "1F357"},
    {"krakers_ryzowy", "1F358"},
    {"kulka_ryzowa", "1F359"},
    {"gotowany_ryz", "1F35A"},
    {"ryz_curry", "1F35B"},
    {"miska_parujacego_jedzenia", "1F35C"},
    {"spaghetti", "1F35D"},
    {"chleb", "1F35E"},
    {"frytki", "1F35F"},
    {"pieczony_batat", "1F360"},
    {"dango", "1F361"},
    {"oden", "1F362"},
    {"sushi", "1F363"},
    {"smazona_krewetka", "1F364"},
    {"ciastko_rybne", "1F365"},
    {"lody_wloskie", "1F366"},
    {"lody_hawajskie", "1F367"},
    {"lody", "1F368"},
    {"paczek_z_dziurka", "1F369"},
    {"ciastko", "1F36A"},
    {"tabliczka_czekolady", "1F36B"},
    {"cukierek", "1F36C"},
    {"lizak", "1F36D"},
    {"deser_typu_krem_angielski", "1F36E"},
    {"garnek_miodu", "1F36F"},
    {"tort", "1F370"},
    {"pudelko_bento", "1F371"},
    {"garnek_z_jedzeniem", "1F372"},
    {"gotowanie", "1F373"},
    {"widelec_i_noz", "1F374"},
    {"filizanka_bez_uszka", "1F375"},
    {"sake", "1F376"},
    {"kieliszek_wina", "1F377"},
    {"kieliszek_z_koktajlem", "1F378"},
    {"drink_tropikalny", "1F379"},
    {"kufel_piwa", "1F37A"},
    {"stukajace_sie_kufle_piwa", "1F37B"},
    {"butelka_ze_smoczkiem", "1F37C"},
    {"talerz_z_nozem_i_widelcem", "1F37D"},
    {"butelka_z_wystrzelajacym_korkiem", "1F37E"},
    {"popcorn", "1F37F"},
    {"wstazka", "1F380"},
    {"zapakowany_prezent", "1F381"},
    {"tort_urodzinowy", "1F382"},
    {"dynia_na_halloween", "1F383"},
    {"choinka_bozonarodzeniowa", "1F384"},
    {"swiety_mikolaj", "1F385"},
    {"swiety_mikolaj_kolor1", "1F385-1F3FB"},
    {"swiety_mikolaj_kolor2", "1F385-1F3FC"},
    {"swiety_mikolaj_kolor3", "1F385-1F3FD"},
    {"swiety_mikolaj_kolor4", "1F385-1F3FE"},
    {"swiety_mikolaj_kolor5", "1F385-1F3FF"},
    {"fajerwerki", "1F386"},
    {"zimne_ognie", "1F387"},
    {"balon", "1F388"},
    {"tuba_z_konfetti", "1F389"},
    {"kula_z_konfetti", "1F38A"},
    {"drzewko_na_tanabata", "1F38B"},
    {"skrzyzowane_flagi", "1F38C"},
    {"japonska_dekoracja_kadomatsu", "1F38D"},
    {"japonskie_lalki", "1F38E"},
    {"flaga_w_ksztalcie_karpia", "1F38F"},
    {"dzwonek_wietrzny", "1F390"},
    {"ceremonia_ogladania_ksiezyca", "1F391"},
    {"tornister", "1F392"},
    {"biret", "1F393"},
    {"order_wojskowy", "1F396"},
    {"wstazka_pamieci", "1F397"},
    {"mikrofon_studyjny", "1F399"},
    {"suwak_miksera_dzwieku", "1F39A"},
    {"pokretla", "1F39B"},
    {"tasma_filmowa", "1F39E"},
    {"bilety_wstepu", "1F39F"},
    {"kon_z_karuzeli", "1F3A0"},
    {"diabelski_mlyn", "1F3A1"},
    {"kolejka_gorska", "1F3A2"},
    {"wedka", "1F3A3"},
    {"mikrofon", "1F3A4"},
    {"kamera_filmowa", "1F3A5"},
    {"kino", "1F3A6"},
    {"sluchawki", "1F3A7"},
    {"paleta_malarska", "1F3A8"},
    {"cylinder", "1F3A9"},
    {"namiot_cyrkowy", "1F3AA"},
    {"bilet", "1F3AB"},
    {"klaps", "1F3AC"},
    {"maski_teatralne", "1F3AD"},
    {"gra_wideo", "1F3AE"},
    {"strzal_w_10", "1F3AF"},
    {"automat_do_gier", "1F3B0"},
    {"bila_8", "1F3B1"},
    {"kostka_do_gry", "1F3B2"},
    {"kregle", "1F3B3"},
    {"gra_karciana_hanafuda", "1F3B4"},
    {"nuta", "1F3B5"},
    {"nuty", "1F3B6"},
    {"saksofon", "1F3B7"},
    {"gitara", "1F3B8"},
    {"klawisze", "1F3B9"},
    {"trabka", "1F3BA"},
    {"skrzypce", "1F3BB"},
    {"partytura", "1F3BC"},
    {"koszulka_do_biegania", "1F3BD"},
    {"tenis", "1F3BE"},
    {"narty", "1F3BF"},
    {"koszykowka", "1F3C0"},
    {"flaga_w_szachownice", "1F3C1"},
    {"snowboardzista", "1F3C2"},
    {"snowboardzista_kolor1", "1F3C2-1F3FB"},
    {"snowboardzista_kolor2", "1F3C2-1F3FC"},
    {"snowboardzista_kolor3", "1F3C2-1F3FD"},
    {"snowboardzista_kolor4", "1F3C2-1F3FE"},
    {"snowboardzista_kolor5", "1F3C2-1F3FF"},
    {"biegnaca_osoba", "1F3C3"},
    {"biegnaca_osoba_kolor1", "1F3C3-1F3FB"},
    {"biegnaca_osoba_kolor2", "1F3C3-1F3FC"},
    {"biegnaca_osoba_kolor3", "1F3C3-1F3FD"},
    {"biegnaca_osoba_kolor4", "1F3C3-1F3FE"},
    {"biegnaca_osoba_kolor5", "1F3C3-1F3FF"},
    {"surfujaca_osoba", "1F3C4"},
    {"surfujaca_osoba_kolor1", "1F3C4-1F3FB"},
    {"surfujaca_osoba_kolor2", "1F3C4-1F3FC"},
    {"surfujaca_osoba_kolor3", "1F3C4-1F3FD"},
    {"surfujaca_osoba_kolor4", "1F3C4-1F3FE"},
    {"surfujaca_osoba_kolor5", "1F3C4-1F3FF"},
    {"medal_sportowy", "1F3C5"},
    {"puchar", "1F3C6"},
    {"wyscigi_konne", "1F3C7"},
    {"wyscigi_konne_kolor1", "1F3C7-1F3FB"},
    {"wyscigi_konne_kolor2", "1F3C7-1F3FC"},
    {"wyscigi_konne_kolor3", "1F3C7-1F3FD"},
    {"wyscigi_konne_kolor4", "1F3C7-1F3FE"},
    {"wyscigi_konne_kolor5", "1F3C7-1F3FF"},
    {"futbol_amerykanski", "1F3C8"},
    {"pilka_do_rugby", "1F3C9"},
    {"plywajaca_osoba", "1F3CA"},
    {"plywajaca_osoba_kolor1", "1F3CA-1F3FB"},
    {"plywajaca_osoba_kolor2", "1F3CA-1F3FC"},
    {"plywajaca_osoba_kolor3", "1F3CA-1F3FD"},
    {"plywajaca_osoba_kolor4", "1F3CA-1F3FE"},
    {"plywajaca_osoba_kolor5", "1F3CA-1F3FF"},
    {"osoba_podnoszaca_ciezary", "1F3CB"},
    {"osoba_podnoszaca_ciezary_kolor1", "1F3CB-1F3FB"},
    {"osoba_podnoszaca_ciezary_kolor2", "1F3CB-1F3FC"},
    {"osoba_podnoszaca_ciezary_kolor3", "1F3CB-1F3FD"},
    {"osoba_podnoszaca_ciezary_kolor4", "1F3CB-1F3FE"},
    {"osoba_podnoszaca_ciezary_kolor5", "1F3CB-1F3FF"},
    {"osoba_grajaca_w_golfa", "1F3CC"},
    {"osoba_grajaca_w_golfa_kolor1", "1F3CC-1F3FB"},
    {"osoba_grajaca_w_golfa_kolor2", "1F3CC-1F3FC"},
    {"osoba_grajaca_w_golfa_kolor3", "1F3CC-1F3FD"},
    {"osoba_grajaca_w_golfa_kolor4", "1F3CC-1F3FE"},
    {"osoba_grajaca_w_golfa_kolor5", "1F3CC-1F3FF"},
    {"motocykl", "1F3CD"},
    {"samochod_wyscigowy", "1F3CE"},
    {"krykiet", "1F3CF"},
    {"siatkowka", "1F3D0"},
    {"hokej_na_trawie", "1F3D1"},
    {"hokej_na_lodzie", "1F3D2"},
    {"tenis_stolowy", "1F3D3"},
    {"gora_z_czapa_lodowa", "1F3D4"},
    {"kemping", "1F3D5"},
    {"parasol_na_plazy", "1F3D6"},
    {"dzwig_budowlany", "1F3D7"},
    {"domki", "1F3D8"},
    {"drapacze_chmur", "1F3D9"},
    {"stary_dom", "1F3DA"},
    {"klasyczna_budowla", "1F3DB"},
    {"pustynia", "1F3DC"},
    {"bezludna_wyspa", "1F3DD"},
    {"park_narodowy", "1F3DE"},
    {"stadion", "1F3DF"},
    {"dom", "1F3E0"},
    {"dom_z_ogrodem", "1F3E1"},
    {"biurowiec", "1F3E2"},
    {"japonska_poczta", "1F3E3"},
    {"urzad_pocztowy", "1F3E4"},
    {"szpital", "1F3E5"},
    {"bank", "1F3E6"},
    {"znak_bankomatu", "1F3E7"},
    {"hotel", "1F3E8"},
    {"hotel_milosci", "1F3E9"},
    {"sklep_calodobowy", "1F3EA"},
    {"szkola", "1F3EB"},
    {"dom_towarowy", "1F3EC"},
    {"fabryka", "1F3ED"},
    {"czerwony_lampion", "1F3EE"},
    {"japonski_zamek", "1F3EF"},
    {"zamek", "1F3F0"},
    {"biala_flaga", "1F3F3"},
    {"czarna_flaga", "1F3F4"},
    {"rozeta", "1F3F5"},
    {"przywieszka_do_kluczy", "1F3F7"},
    {"badminton", "1F3F8"},
    {"luk_i_strzala", "1F3F9"},
    {"amfora", "1F3FA"},
    {"karnacja_jasna", "1F3FB"},
    {"karnacja_srednio_jasna", "1F3FC"},
    {"karnacja_srednia", "1F3FD"},
    {"karnacja_srednio_ciemna", "1F3FE"},
    {"karnacja_ciemna", "1F3FF"},
    {"szczur", "1F400"},
    {"mysz", "1F401"},
    {"wol", "1F402"},
    {"wol_domowy", "1F403"},
    {"krowa", "1F404"},
    {"tygrys", "1F405"},
    {"lampart", "1F406"},
    {"krolik", "1F407"},
    {"kot", "1F408"},
    {"smok", "1F409"},
    {"krokodyl", "1F40A"},
    {"wieloryb", "1F40B"},
    {"slimak", "1F40C"},
    {"waz", "1F40D"},
    {"kon", "1F40E"},
    {"bialy_baran", "1F40F"},
    {"koza", "1F410"},
    {"owca", "1F411"},
    {"malpa", "1F412"},
    {"kogut", "1F413"},
    {"kura", "1F414"},
    {"pies", "1F415"},
    {"swinia", "1F416"},
    {"dzik", "1F417"},
    {"slon", "1F418"},
    {"osmiornica", "1F419"},
    {"muszla_spiralna", "1F41A"},
    {"gasienica", "1F41B"},
    {"mrowka", "1F41C"},
    {"pszczola", "1F41D"},
    {"biedronka", "1F41E"},
    {"ryba", "1F41F"},
    {"ryba_tropikalna", "1F420"},
    {"ryba_rozdymka", "1F421"},
    {"zolw", "1F422"},
    {"wykluwajace_sie_kurczatko", "1F423"},
    {"kurczatko", "1F424"},
    {"kurczatko_od_przodu", "1F425"},
    {"ptak", "1F426"},
    {"pingwin", "1F427"},
    {"koala", "1F428"},
    {"pudel", "1F429"},
    {"wielblad", "1F42A"},
    {"wielblad_dwugarbny", "1F42B"},
    {"delfin", "1F42C"},
    {"glowa_myszy", "1F42D"},
    {"glowa_krowy", "1F42E"},
    {"glowa_tygrysa", "1F42F"},
    {"glowa_krolika", "1F430"},
    {"glowa_kota", "1F431"},
    {"glowa_smoka", "1F432"},
    {"wieloryb_tryskajacy_woda", "1F433"},
    {"glowa_konia", "1F434"},
    {"glowa_malpy", "1F435"},
    {"glowa_psa", "1F436"},
    {"glowa_swini", "1F437"},
    {"glowa_zaby", "1F438"},
    {"glowa_chomika", "1F439"},
    {"glowa_wilka", "1F43A"},
    {"glowa_niedzwiedzia", "1F43B"},
    {"glowa_pandy", "1F43C"},
    {"ryj_swini", "1F43D"},
    {"slady_lap", "1F43E"},
    {"pregowiec", "1F43F"},
    {"oczy", "1F440"},
    {"oko", "1F441"},
    {"ucho", "1F442"},
    {"ucho_kolor1", "1F442-1F3FB"},
    {"ucho_kolor2", "1F442-1F3FC"},
    {"ucho_kolor3", "1F442-1F3FD"},
    {"ucho_kolor4", "1F442-1F3FE"},
    {"ucho_kolor5", "1F442-1F3FF"},
    {"nos", "1F443"},
    {"nos_kolor1", "1F443-1F3FB"},
    {"nos_kolor2", "1F443-1F3FC"},
    {"nos_kolor3", "1F443-1F3FD"},
    {"nos_kolor4", "1F443-1F3FE"},
    {"nos_kolor5", "1F443-1F3FF"},
    {"usta", "1F444"},
    {"jezyk", "1F445"},
    {"dlon_z_palcem_wskazujacym_w_gore", "1F446"},
    {"dlon_z_palcem_wskazujacym_w_gore_kolor1", "1F446-1F3FB"},
    {"dlon_z_palcem_wskazujacym_w_gore_kolor2", "1F446-1F3FC"},
    {"dlon_z_palcem_wskazujacym_w_gore_kolor3", "1F446-1F3FD"},
    {"dlon_z_palcem_wskazujacym_w_gore_kolor4", "1F446-1F3FE"},
    {"dlon_z_palcem_wskazujacym_w_gore_kolor5", "1F446-1F3FF"},
    {"dlon_z_palcem_wskazujacym_w_dol", "1F447"},
    {"dlon_z_palcem_wskazujacym_w_dol_kolor1", "1F447-1F3FB"},
    {"dlon_z_palcem_wskazujacym_w_dol_kolor2", "1F447-1F3FC"},
    {"dlon_z_palcem_wskazujacym_w_dol_kolor3", "1F447-1F3FD"},
    {"dlon_z_palcem_wskazujacym_w_dol_kolor4", "1F447-1F3FE"},
    {"dlon_z_palcem_wskazujacym_w_dol_kolor5", "1F447-1F3FF"},
    {"dlon_z_palcem_wskazujacym_w_lewo", "1F448"},
    {"dlon_z_palcem_wskazujacym_w_lewo_kolor1", "1F448-1F3FB"},
    {"dlon_z_palcem_wskazujacym_w_lewo_kolor2", "1F448-1F3FC"},
    {"dlon_z_palcem_wskazujacym_w_lewo_kolor3", "1F448-1F3FD"},
    {"dlon_z_palcem_wskazujacym_w_lewo_kolor4", "1F448-1F3FE"},
    {"dlon_z_palcem_wskazujacym_w_lewo_kolor5", "1F448-1F3FF"},
    {"dlon_z_palcem_wskazujacym_w_prawo", "1F449"},
    {"dlon_z_palcem_wskazujacym_w_prawo_kolor1", "1F449-1F3FB"},
    {"dlon_z_palcem_wskazujacym_w_prawo_kolor2", "1F449-1F3FC"},
    {"dlon_z_palcem_wskazujacym_w_prawo_kolor3", "1F449-1F3FD"},
    {"dlon_z_palcem_wskazujacym_w_prawo_kolor4", "1F449-1F3FE"},
    {"dlon_z_palcem_wskazujacym_w_prawo_kolor5", "1F449-1F3FF"},
    {"piesc_od_przodu", "1F44A"},
    {"piesc_od_przodu_kolor1", "1F44A-1F3FB"},
    {"piesc_od_przodu_kolor2", "1F44A-1F3FC"},
    {"piesc_od_przodu_kolor3", "1F44A-1F3FD"},
    {"piesc_od_przodu_kolor4", "1F44A-1F3FE"},
    {"piesc_od_przodu_kolor5", "1F44A-1F3FF"},
    {"machajaca_dlon", "1F44B"},
    {"machajaca_dlon_kolor1", "1F44B-1F3FB"},
    {"machajaca_dlon_kolor2", "1F44B-1F3FC"},
    {"machajaca_dlon_kolor3", "1F44B-1F3FD"},
    {"machajaca_dlon_kolor4", "1F44B-1F3FE"},
    {"machajaca_dlon_kolor5", "1F44B-1F3FF"},
    {"dlon_z_gestem_ok", "1F44C"},
    {"dlon_z_gestem_ok_kolor1", "1F44C-1F3FB"},
    {"dlon_z_gestem_ok_kolor2", "1F44C-1F3FC"},
    {"dlon_z_gestem_ok_kolor3", "1F44C-1F3FD"},
    {"dlon_z_gestem_ok_kolor4", "1F44C-1F3FE"},
    {"dlon_z_gestem_ok_kolor5", "1F44C-1F3FF"},
    {"kciuk_w_gore", "1F44D"},
    {"kciuk_w_gore_kolor1", "1F44D-1F3FB"},
    {"kciuk_w_gore_kolor2", "1F44D-1F3FC"},
    {"kciuk_w_gore_kolor3", "1F44D-1F3FD"},
    {"kciuk_w_gore_kolor4", "1F44D-1F3FE"},
    {"kciuk_w_gore_kolor5", "1F44D-1F3FF"},
    {"kciuk_w_dol", "1F44E"},
    {"kciuk_w_dol_kolor1", "1F44E-1F3FB"},
    {"kciuk_w_dol_kolor2", "1F44E-1F3FC"},
    {"kciuk_w_dol_kolor3", "1F44E-1F3FD"},
    {"kciuk_w_dol_kolor4", "1F44E-1F3FE"},
    {"kciuk_w_dol_kolor5", "1F44E-1F3FF"},
    {"klaskanie", "1F44F"},
    {"klaskanie_kolor1", "1F44F-1F3FB"},
    {"klaskanie_kolor2", "1F44F-1F3FC"},
    {"klaskanie_kolor3", "1F44F-1F3FD"},
    {"klaskanie_kolor4", "1F44F-1F3FE"},
    {"klaskanie_kolor5", "1F44F-1F3FF"},
    {"otwarte_dlonie", "1F450"},
    {"otwarte_dlonie_kolor1", "1F450-1F3FB"},
    {"otwarte_dlonie_kolor2", "1F450-1F3FC"},
    {"otwarte_dlonie_kolor3", "1F450-1F3FD"},
    {"otwarte_dlonie_kolor4", "1F450-1F3FE"},
    {"otwarte_dlonie_kolor5", "1F450-1F3FF"},
    {"korona", "1F451"},
    {"kapelusz_damski", "1F452"},
    {"okulary", "1F453"},
    {"krawat", "1F454"},
    {"koszulka", "1F455"},
    {"dzinsy", "1F456"},
    {"sukienka", "1F457"},
    {"kimono", "1F458"},
    {"bikini", "1F459"},
    {"ubranie_damskie", "1F45A"},
    {"portmonetka", "1F45B"},
    {"torebka", "1F45C"},
    {"saszetka", "1F45D"},
    {"but_meski", "1F45E"},
    {"but_do_biegania", "1F45F"},
    {"but_na_wysokim_obcasie", "1F460"},
    {"sandal_damski", "1F461"},
    {"kozak_damski", "1F462"},
    {"slady_stop", "1F463"},
    {"sylwetka_popiersia", "1F464"},
    {"sylwetki_popiersi", "1F465"},
    {"chlopiec", "1F466"},
    {"chlopiec_kolor1", "1F466-1F3FB"},
    {"chlopiec_kolor2", "1F466-1F3FC"},
    {"chlopiec_kolor3", "1F466-1F3FD"},
    {"chlopiec_kolor4", "1F466-1F3FE"},
    {"chlopiec_kolor5", "1F466-1F3FF"},
    {"dziewczynka", "1F467"},
    {"dziewczynka_kolor1", "1F467-1F3FB"},
    {"dziewczynka_kolor2", "1F467-1F3FC"},
    {"dziewczynka_kolor3", "1F467-1F3FD"},
    {"dziewczynka_kolor4", "1F467-1F3FE"},
    {"dziewczynka_kolor5", "1F467-1F3FF"},
    {"mezczyzna", "1F468"},
    {"mezczyzna_kolor1", "1F468-1F3FB"},
    {"mezczyzna_kolor2", "1F468-1F3FC"},
    {"mezczyzna_kolor3", "1F468-1F3FD"},
    {"mezczyzna_kolor4", "1F468-1F3FE"},
    {"mezczyzna_kolor5", "1F468-1F3FF"},
    {"kobieta", "1F469"},
    {"kobieta_kolor1", "1F469-1F3FB"},
    {"kobieta_kolor2", "1F469-1F3FC"},
    {"kobieta_kolor3", "1F469-1F3FD"},
    {"kobieta_kolor4", "1F469-1F3FE"},
    {"kobieta_kolor5", "1F469-1F3FF"},
    {"rodzina", "1F46A"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece", "1F46B"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor1", "1F46B-1F3FB"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor2", "1F46B-1F3FC"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor3", "1F46B-1F3FD"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor4", "1F46B-1F3FE"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor5", "1F46B-1F3FF"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mezczyzna_i_kobieta_trzymajacy_sie_za_rece_kolor5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece", "1F46C"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor1", "1F46C-1F3FB"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor2", "1F46C-1F3FC"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor3", "1F46C-1F3FD"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor4", "1F46C-1F3FE"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor5", "1F46C-1F3FF"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"dwoch_mezczyzn_trzymajacych_sie_za_rece_kolor5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"kobiety_trzymajace_sie_za_rece", "1F46D"},
    {"kobiety_trzymajace_sie_za_rece_kolor1", "1F46D-1F3FB"},
    {"kobiety_trzymajace_sie_za_rece_kolor2", "1F46D-1F3FC"},
    {"kobiety_trzymajace_sie_za_rece_kolor3", "1F46D-1F3FD"},
    {"kobiety_trzymajace_sie_za_rece_kolor4", "1F46D-1F3FE"},
    {"kobiety_trzymajace_sie_za_rece_kolor5", "1F46D-1F3FF"},
    {"kobiety_trzymajace_sie_za_rece_kolor1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"kobiety_trzymajace_sie_za_rece_kolor1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"kobiety_trzymajace_sie_za_rece_kolor1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"kobiety_trzymajace_sie_za_rece_kolor1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"kobiety_trzymajace_sie_za_rece_kolor2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"kobiety_trzymajace_sie_za_rece_kolor2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"kobiety_trzymajace_sie_za_rece_kolor2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"kobiety_trzymajace_sie_za_rece_kolor2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"kobiety_trzymajace_sie_za_rece_kolor3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"kobiety_trzymajace_sie_za_rece_kolor3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"kobiety_trzymajace_sie_za_rece_kolor3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"kobiety_trzymajace_sie_za_rece_kolor3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"kobiety_trzymajace_sie_za_rece_kolor4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"kobiety_trzymajace_sie_za_rece_kolor4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"kobiety_trzymajace_sie_za_rece_kolor4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"kobiety_trzymajace_sie_za_rece_kolor4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"kobiety_trzymajace_sie_za_rece_kolor5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"kobiety_trzymajace_sie_za_rece_kolor5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"kobiety_trzymajace_sie_za_rece_kolor5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"kobiety_trzymajace_sie_za_rece_kolor5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"policjant", "1F46E"},
    {"policjant_kolor1", "1F46E-1F3FB"},
    {"policjant_kolor2", "1F46E-1F3FC"},
    {"policjant_kolor3", "1F46E-1F3FD"},
    {"policjant_kolor4", "1F46E-1F3FE"},
    {"policjant_kolor5", "1F46E-1F3FF"},
    {"osoby_z_uszami_krolika_na_przyjeciu", "1F46F"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor1", "1F46F-1F3FB"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor2", "1F46F-1F3FC"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor3", "1F46F-1F3FD"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor4", "1F46F-1F3FE"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor5", "1F46F-1F3FF"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"osoby_z_uszami_krolika_na_przyjeciu_kolor5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"osoba_w_welonie", "1F470"},
    {"osoba_w_welonie_kolor1", "1F470-1F3FB"},
    {"osoba_w_welonie_kolor2", "1F470-1F3FC"},
    {"osoba_w_welonie_kolor3", "1F470-1F3FD"},
    {"osoba_w_welonie_kolor4", "1F470-1F3FE"},
    {"osoba_w_welonie_kolor5", "1F470-1F3FF"},
    {"osoba_z_wlosami_blond", "1F471"},
    {"osoba_z_wlosami_blond_kolor1", "1F471-1F3FB"},
    {"osoba_z_wlosami_blond_kolor2", "1F471-1F3FC"},
    {"osoba_z_wlosami_blond_kolor3", "1F471-1F3FD"},
    {"osoba_z_wlosami_blond_kolor4", "1F471-1F3FE"},
    {"osoba_z_wlosami_blond_kolor5", "1F471-1F3FF"},
    {"osoba_w_chinskiej_czapce", "1F472"},
    {"osoba_w_chinskiej_czapce_kolor1", "1F472-1F3FB"},
    {"osoba_w_chinskiej_czapce_kolor2", "1F472-1F3FC"},
    {"osoba_w_chinskiej_czapce_kolor3", "1F472-1F3FD"},
    {"osoba_w_chinskiej_czapce_kolor4", "1F472-1F3FE"},
    {"osoba_w_chinskiej_czapce_kolor5", "1F472-1F3FF"},
    {"osoba_w_turbanie", "1F473"},
    {"osoba_w_turbanie_kolor1", "1F473-1F3FB"},
    {"osoba_w_turbanie_kolor2", "1F473-1F3FC"},
    {"osoba_w_turbanie_kolor3", "1F473-1F3FD"},
    {"osoba_w_turbanie_kolor4", "1F473-1F3FE"},
    {"osoba_w_turbanie_kolor5", "1F473-1F3FF"},
    {"starszy_mezczyzna", "1F474"},
    {"starszy_mezczyzna_kolor1", "1F474-1F3FB"},
    {"starszy_mezczyzna_kolor2", "1F474-1F3FC"},
    {"starszy_mezczyzna_kolor3", "1F474-1F3FD"},
    {"starszy_mezczyzna_kolor4", "1F474-1F3FE"},
    {"starszy_mezczyzna_kolor5", "1F474-1F3FF"},
    {"starsza_kobieta", "1F475"},
    {"starsza_kobieta_kolor1", "1F475-1F3FB"},
    {"starsza_kobieta_kolor2", "1F475-1F3FC"},
    {"starsza_kobieta_kolor3", "1F475-1F3FD"},
    {"starsza_kobieta_kolor4", "1F475-1F3FE"},
    {"starsza_kobieta_kolor5", "1F475-1F3FF"},
    {"niemowle", "1F476"},
    {"niemowle_kolor1", "1F476-1F3FB"},
    {"niemowle_kolor2", "1F476-1F3FC"},
    {"niemowle_kolor3", "1F476-1F3FD"},
    {"niemowle_kolor4", "1F476-1F3FE"},
    {"niemowle_kolor5", "1F476-1F3FF"},
    {"budowlaniec", "1F477"},
    {"budowlaniec_kolor1", "1F477-1F3FB"},
    {"budowlaniec_kolor2", "1F477-1F3FC"},
    {"budowlaniec_kolor3", "1F477-1F3FD"},
    {"budowlaniec_kolor4", "1F477-1F3FE"},
    {"budowlaniec_kolor5", "1F477-1F3FF"},
    {"ksiezniczka", "1F478"},
    {"ksiezniczka_kolor1", "1F478-1F3FB"},
    {"ksiezniczka_kolor2", "1F478-1F3FC"},
    {"ksiezniczka_kolor3", "1F478-1F3FD"},
    {"ksiezniczka_kolor4", "1F478-1F3FE"},
    {"ksiezniczka_kolor5", "1F478-1F3FF"},
    {"ogr", "1F479"},
    {"goblin", "1F47A"},
    {"duch", "1F47B"},
    {"aniolek", "1F47C"},
    {"aniolek_kolor1", "1F47C-1F3FB"},
    {"aniolek_kolor2", "1F47C-1F3FC"},
    {"aniolek_kolor3", "1F47C-1F3FD"},
    {"aniolek_kolor4", "1F47C-1F3FE"},
    {"aniolek_kolor5", "1F47C-1F3FF"},
    {"ufoludek", "1F47D"},
    {"kosmiczny_potwor", "1F47E"},
    {"zagniewana_twarz_z_rogami", "1F47F"},
    {"czaszka", "1F480"},
    {"osoba_z_wystawiona_dlonia", "1F481"},
    {"osoba_z_wystawiona_dlonia_kolor1", "1F481-1F3FB"},
    {"osoba_z_wystawiona_dlonia_kolor2", "1F481-1F3FC"},
    {"osoba_z_wystawiona_dlonia_kolor3", "1F481-1F3FD"},
    {"osoba_z_wystawiona_dlonia_kolor4", "1F481-1F3FE"},
    {"osoba_z_wystawiona_dlonia_kolor5", "1F481-1F3FF"},
    {"gwardzista", "1F482"},
    {"gwardzista_kolor1", "1F482-1F3FB"},
    {"gwardzista_kolor2", "1F482-1F3FC"},
    {"gwardzista_kolor3", "1F482-1F3FD"},
    {"gwardzista_kolor4", "1F482-1F3FE"},
    {"gwardzista_kolor5", "1F482-1F3FF"},
    {"tanczaca_kobieta", "1F483"},
    {"tanczaca_kobieta_kolor1", "1F483-1F3FB"},
    {"tanczaca_kobieta_kolor2", "1F483-1F3FC"},
    {"tanczaca_kobieta_kolor3", "1F483-1F3FD"},
    {"tanczaca_kobieta_kolor4", "1F483-1F3FE"},
    {"tanczaca_kobieta_kolor5", "1F483-1F3FF"},
    {"szminka", "1F484"},
    {"lakier_do_paznokci", "1F485"},
    {"lakier_do_paznokci_kolor1", "1F485-1F3FB"},
    {"lakier_do_paznokci_kolor2", "1F485-1F3FC"},
    {"lakier_do_paznokci_kolor3", "1F485-1F3FD"},
    {"lakier_do_paznokci_kolor4", "1F485-1F3FE"},
    {"lakier_do_paznokci_kolor5", "1F485-1F3FF"},
    {"osoba_podczas_masazu", "1F486"},
    {"osoba_podczas_masazu_kolor1", "1F486-1F3FB"},
    {"osoba_podczas_masazu_kolor2", "1F486-1F3FC"},
    {"osoba_podczas_masazu_kolor3", "1F486-1F3FD"},
    {"osoba_podczas_masazu_kolor4", "1F486-1F3FE"},
    {"osoba_podczas_masazu_kolor5", "1F486-1F3FF"},
    {"osoba_podczas_strzyzenia", "1F487"},
    {"osoba_podczas_strzyzenia_kolor1", "1F487-1F3FB"},
    {"osoba_podczas_strzyzenia_kolor2", "1F487-1F3FC"},
    {"osoba_podczas_strzyzenia_kolor3", "1F487-1F3FD"},
    {"osoba_podczas_strzyzenia_kolor4", "1F487-1F3FE"},
    {"osoba_podczas_strzyzenia_kolor5", "1F487-1F3FF"},
    {"slup_fryzjerski", "1F488"},
    {"strzykawka", "1F489"},
    {"kapsulka", "1F48A"},
    {"slad_po_pocalunku", "1F48B"},
    {"list_milosny", "1F48C"},
    {"pierscionek", "1F48D"},
    {"kamien_szlachetny", "1F48E"},
    {"pocalunek", "1F48F"},
    {"pocalunek_kolor1", "1F48F-1F3FB"},
    {"pocalunek_kolor2", "1F48F-1F3FC"},
    {"pocalunek_kolor3", "1F48F-1F3FD"},
    {"pocalunek_kolor4", "1F48F-1F3FE"},
    {"pocalunek_kolor5", "1F48F-1F3FF"},
    {"pocalunek_kolor1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"pocalunek_kolor1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"pocalunek_kolor1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"pocalunek_kolor1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"pocalunek_kolor2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"pocalunek_kolor2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"pocalunek_kolor2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"pocalunek_kolor2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"pocalunek_kolor3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"pocalunek_kolor3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"pocalunek_kolor3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"pocalunek_kolor3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"pocalunek_kolor4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"pocalunek_kolor4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"pocalunek_kolor4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"pocalunek_kolor4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"pocalunek_kolor5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"pocalunek_kolor5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"pocalunek_kolor5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"pocalunek_kolor5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bukiet_kwiatow", "1F490"},
    {"para_z_sercem", "1F491"},
    {"para_z_sercem_kolor1", "1F491-1F3FB"},
    {"para_z_sercem_kolor2", "1F491-1F3FC"},
    {"para_z_sercem_kolor3", "1F491-1F3FD"},
    {"para_z_sercem_kolor4", "1F491-1F3FE"},
    {"para_z_sercem_kolor5", "1F491-1F3FF"},
    {"para_z_sercem_kolor1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"para_z_sercem_kolor1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"para_z_sercem_kolor1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"para_z_sercem_kolor1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"para_z_sercem_kolor2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"para_z_sercem_kolor2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"para_z_sercem_kolor2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"para_z_sercem_kolor2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"para_z_sercem_kolor3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"para_z_sercem_kolor3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"para_z_sercem_kolor3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"para_z_sercem_kolor3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"para_z_sercem_kolor4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"para_z_sercem_kolor4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"para_z_sercem_kolor4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"para_z_sercem_kolor4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"para_z_sercem_kolor5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"para_z_sercem_kolor5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"para_z_sercem_kolor5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"para_z_sercem_kolor5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"slub", "1F492"},
    {"bijace_serce", "1F493"},
    {"zlamane_serce", "1F494"},
    {"dwa_serca", "1F495"},
    {"migoczace_serce", "1F496"},
    {"rosnace_serce", "1F497"},
    {"serce_przebite_strzala", "1F498"},
    {"niebieskie_serce", "1F499"},
    {"zielone_serce", "1F49A"},
    {"zolte_serce", "1F49B"},
    {"purpurowe_serce", "1F49C"},
    {"serce_ze_wstazka", "1F49D"},
    {"wirujace_serca", "1F49E"},
    {"dekoracja_z_sercem", "1F49F"},
    {"romb_z_kropka", "1F4A0"},
    {"zarowka", "1F4A1"},
    {"symbol_gniewu", "1F4A2"},
    {"bomba", "1F4A3"},
    {"chrapanie", "1F4A4"},
    {"kolizja", "1F4A5"},
    {"krople_potu", "1F4A6"},
    {"kropla", "1F4A7"},
    {"szybkie_poruszanie_sie", "1F4A8"},
    {"kupa", "1F4A9"},
    {"napiety_biceps", "1F4AA"},
    {"napiety_biceps_kolor1", "1F4AA-1F3FB"},
    {"napiety_biceps_kolor2", "1F4AA-1F3FC"},
    {"napiety_biceps_kolor3", "1F4AA-1F3FD"},
    {"napiety_biceps_kolor4", "1F4AA-1F3FE"},
    {"napiety_biceps_kolor5", "1F4AA-1F3FF"},
    {"zawroty_glowy", "1F4AB"},
    {"dymek", "1F4AC"},
    {"dymek_mysli", "1F4AD"},
    {"bialy_kwiat", "1F4AE"},
    {"sto_punktow", "1F4AF"},
    {"worek_z_pieniedzmi", "1F4B0"},
    {"wymiana_walut", "1F4B1"},
    {"gruby_symbol_dolara", "1F4B2"},
    {"karta_kredytowa", "1F4B3"},
    {"banknot_jena", "1F4B4"},
    {"banknot_dolara", "1F4B5"},
    {"banknot_euro", "1F4B6"},
    {"banknot_funta", "1F4B7"},
    {"uskrzydlone_pieniadze", "1F4B8"},
    {"wykres_wzrostu_z_symbolem_jena", "1F4B9"},
    {"fotel", "1F4BA"},
    {"laptop", "1F4BB"},
    {"teczka", "1F4BC"},
    {"dysk_komputerowy", "1F4BD"},
    {"dyskietka", "1F4BE"},
    {"dysk_optyczny", "1F4BF"},
    {"dvd", "1F4C0"},
    {"folder", "1F4C1"},
    {"otwarty_folder", "1F4C2"},
    {"zawinieta_strona", "1F4C3"},
    {"strona", "1F4C4"},
    {"kalendarz", "1F4C5"},
    {"kalendarz_z_wyrywanymi_kartkami", "1F4C6"},
    {"wizytownik", "1F4C7"},
    {"wykres_wzrostu", "1F4C8"},
    {"wykres_spadku", "1F4C9"},
    {"wykres_slupkowy", "1F4CA"},
    {"podkladka_do_pisania", "1F4CB"},
    {"pinezka", "1F4CC"},
    {"okragla_pinezka", "1F4CD"},
    {"spinacz", "1F4CE"},
    {"linijka", "1F4CF"},
    {"ekierka", "1F4D0"},
    {"karty_zakladek", "1F4D1"},
    {"skoroszyt", "1F4D2"},
    {"notes", "1F4D3"},
    {"notes_z_dekoracyjna_okladka", "1F4D4"},
    {"zamknieta_ksiazka", "1F4D5"},
    {"otwarta_ksiazka", "1F4D6"},
    {"zielona_ksiazka", "1F4D7"},
    {"niebieska_ksiazka", "1F4D8"},
    {"pomaranczowa_ksiazka", "1F4D9"},
    {"ksiazki", "1F4DA"},
    {"plakietka", "1F4DB"},
    {"zwoj", "1F4DC"},
    {"notatka", "1F4DD"},
    {"sluchawka_telefoniczna", "1F4DE"},
    {"pager", "1F4DF"},
    {"faks", "1F4E0"},
    {"antena_satelitarna", "1F4E1"},
    {"glosnik", "1F4E2"},
    {"megafon", "1F4E3"},
    {"skrzynka_nadawcza", "1F4E4"},
    {"skrzynka_odbiorcza", "1F4E5"},
    {"paczka", "1F4E6"},
    {"email", "1F4E7"},
    {"koperta_przychodzaca", "1F4E8"},
    {"koperta_ze_strzalka", "1F4E9"},
    {"zamknieta_skrzynka_pocztowa_z_opuszczona_flaga", "1F4EA"},
    {"zamknieta_skrzynka_pocztowa_z_podniesiona_flaga", "1F4EB"},
    {"otwarta_skrzynka_pocztowa_z_podniesiona_flaga", "1F4EC"},
    {"otwarta_skrzynka_pocztowa_z_opuszczona_flaga", "1F4ED"},
    {"skrzynka_na_listy", "1F4EE"},
    {"trabka_pocztowa", "1F4EF"},
    {"gazeta", "1F4F0"},
    {"telefon_komorkowy", "1F4F1"},
    {"telefon_komorkowy_ze_strzalka", "1F4F2"},
    {"tryb_wibracji", "1F4F3"},
    {"wylaczony_telefon_komorkowy", "1F4F4"},
    {"zakaz_korzystania_z_telefonow_komorkowych", "1F4F5"},
    {"sila_sygnalu", "1F4F6"},
    {"aparat_fotograficzny", "1F4F7"},
    {"aparat_fotograficzny_z_lampa_blyskowa", "1F4F8"},
    {"kamera_wideo", "1F4F9"},
    {"telewizja", "1F4FA"},
    {"radio", "1F4FB"},
    {"kaseta_wideo", "1F4FC"},
    {"projektor_filmowy", "1F4FD"},
    {"sznur_modlitewny", "1F4FF"},
    {"przycisk_losowego_odtwarzania_utworow", "1F500"},
    {"przycisk_powtarzania", "1F501"},
    {"przycisk_powtarzania_jednokrotnego", "1F502"},
    {"pionowe_strzalki_zgodne_z_ruchem_wskazowek_zegara", "1F503"},
    {"przycisk_ze_strzalkami_przeciwnie_do_ruchu_wskazowek_zegara", "1F504"},
    {"przycisk_malej_jasnosci", "1F505"},
    {"przycisk_duzej_jasnosci", "1F506"},
    {"wyciszony_glosnik", "1F507"},
    {"niska_glosnosc_glosnika", "1F508"},
    {"srednia_glosnosc_glosnika", "1F509"},
    {"wysoka_glosnosc_glosnika", "1F50A"},
    {"bateria", "1F50B"},
    {"wtyczka_elektryczna", "1F50C"},
    {"lupa_pochylona_w_lewo", "1F50D"},
    {"lupa_pochylona_w_prawo", "1F50E"},
    {"zamknieta_klodka_z_piorem", "1F50F"},
    {"zamknieta_klodka_z_kluczem", "1F510"},
    {"klucz", "1F511"},
    {"zamknieta_klodka", "1F512"},
    {"otwarta_klodka", "1F513"},
    {"dzwonek", "1F514"},
    {"przekreslony_dzwonek", "1F515"},
    {"zakladka", "1F516"},
    {"ogniwa", "1F517"},
    {"przycisk_radiowy", "1F518"},
    {"strzalka_z_napisem_back", "1F519"},
    {"strzalka_z_napisem_end", "1F51A"},
    {"strzalka_z_napisem_on", "1F51B"},
    {"strzalka_z_napisem_soon", "1F51C"},
    {"strzalka_z_napisem_top", "1F51D"},
    {"zakaz_ponizej_osiemnastego_roku_zycia", "1F51E"},
    {"klawisz_10", "1F51F"},
    {"wielkie_litery_alfabet_lacinski", "1F520"},
    {"male_litery_alfabet_lacinski", "1F521"},
    {"cyfry", "1F522"},
    {"symbole", "1F523"},
    {"litery_alfabet_lacinski", "1F524"},
    {"ogien", "1F525"},
    {"latarka", "1F526"},
    {"klucz_warsztatowy", "1F527"},
    {"mlotek", "1F528"},
    {"sruba_i_nakretka", "1F529"},
    {"noz_kuchenny", "1F52A"},
    {"pistolet_na_wode", "1F52B"},
    {"mikroskop", "1F52C"},
    {"teleskop", "1F52D"},
    {"krysztalowa_kula", "1F52E"},
    {"gwiazda_szescioramienna_z_kropka", "1F52F"},
    {"japonski_symbol_poczatkujacego_kierowcy", "1F530"},
    {"emblemat_z_trojzebem", "1F531"},
    {"czarny_kwadratowy_przycisk", "1F532"},
    {"bialy_kwadratowy_przycisk", "1F533"},
    {"czerwone_kolo", "1F534"},
    {"niebieskie_kolo", "1F535"},
    {"duzy_pomaranczowy_romb", "1F536"},
    {"duzy_niebieski_romb", "1F537"},
    {"maly_pomaranczowy_romb", "1F538"},
    {"maly_niebieski_romb", "1F539"},
    {"czerwony_trojkat_skierowany_w_gore", "1F53A"},
    {"czerwony_trojkat_skierowany_w_dol", "1F53B"},
    {"przycisk_w_gore", "1F53C"},
    {"przycisk_w_dol", "1F53D"},
    {"om", "1F549"},
    {"golab", "1F54A"},
    {"kaaba", "1F54B"},
    {"meczet", "1F54C"},
    {"synagoga", "1F54D"},
    {"menora", "1F54E"},
    {"godzina_100", "1F550"},
    {"godzina_200", "1F551"},
    {"godzina_300", "1F552"},
    {"godzina_400", "1F553"},
    {"godzina_500", "1F554"},
    {"godzina_600", "1F555"},
    {"godzina_700", "1F556"},
    {"godzina_800", "1F557"},
    {"godzina_900", "1F558"},
    {"godzina_1000", "1F559"},
    {"godzina_1100", "1F55A"},
    {"godzina_1200", "1F55B"},
    {"godzina_130", "1F55C"},
    {"godzina_230", "1F55D"},
    {"godzina_330", "1F55E"},
    {"godzina_430", "1F55F"},
    {"godzina_530", "1F560"},
    {"godzina_630", "1F561"},
    {"godzina_730", "1F562"},
    {"godzina_830", "1F563"},
    {"godzina_930", "1F564"},
    {"godzina_1030", "1F565"},
    {"godzina_1130", "1F566"},
    {"godzina_1230", "1F567"},
    {"swieca", "1F56F"},
    {"zegar_kominkowy", "1F570"},
    {"otwor", "1F573"},
    {"lewitujacy_mezczyzna_w_garniturze", "1F574"},
    {"lewitujacy_mezczyzna_w_garniturze_kolor1", "1F574-1F3FB"},
    {"lewitujacy_mezczyzna_w_garniturze_kolor2", "1F574-1F3FC"},
    {"lewitujacy_mezczyzna_w_garniturze_kolor3", "1F574-1F3FD"},
    {"lewitujacy_mezczyzna_w_garniturze_kolor4", "1F574-1F3FE"},
    {"lewitujacy_mezczyzna_w_garniturze_kolor5", "1F574-1F3FF"},
    {"detektyw", "1F575"},
    {"detektyw_kolor1", "1F575-1F3FB"},
    {"detektyw_kolor2", "1F575-1F3FC"},
    {"detektyw_kolor3", "1F575-1F3FD"},
    {"detektyw_kolor4", "1F575-1F3FE"},
    {"detektyw_kolor5", "1F575-1F3FF"},
    {"okulary_sloneczne", "1F576"},
    {"pajak", "1F577"},
    {"pajeczyna", "1F578"},
    {"dzojstik", "1F579"},
    {"tanczacy_mezczyzna", "1F57A"},
    {"tanczacy_mezczyzna_kolor1", "1F57A-1F3FB"},
    {"tanczacy_mezczyzna_kolor2", "1F57A-1F3FC"},
    {"tanczacy_mezczyzna_kolor3", "1F57A-1F3FD"},
    {"tanczacy_mezczyzna_kolor4", "1F57A-1F3FE"},
    {"tanczacy_mezczyzna_kolor5", "1F57A-1F3FF"},
    {"zlaczone_spinacze", "1F587"},
    {"dlugopis", "1F58A"},
    {"wieczne_pioro", "1F58B"},
    {"pedzel", "1F58C"},
    {"kredka", "1F58D"},
    {"uniesiona_otwarta_dlon", "1F590"},
    {"uniesiona_otwarta_dlon_kolor1", "1F590-1F3FB"},
    {"uniesiona_otwarta_dlon_kolor2", "1F590-1F3FC"},
    {"uniesiona_otwarta_dlon_kolor3", "1F590-1F3FD"},
    {"uniesiona_otwarta_dlon_kolor4", "1F590-1F3FE"},
    {"uniesiona_otwarta_dlon_kolor5", "1F590-1F3FF"},
    {"srodkowy_palec", "1F595"},
    {"srodkowy_palec_kolor1", "1F595-1F3FB"},
    {"srodkowy_palec_kolor2", "1F595-1F3FC"},
    {"srodkowy_palec_kolor3", "1F595-1F3FD"},
    {"srodkowy_palec_kolor4", "1F595-1F3FE"},
    {"srodkowy_palec_kolor5", "1F595-1F3FF"},
    {"salut_wolkanski", "1F596"},
    {"salut_wolkanski_kolor1", "1F596-1F3FB"},
    {"salut_wolkanski_kolor2", "1F596-1F3FC"},
    {"salut_wolkanski_kolor3", "1F596-1F3FD"},
    {"salut_wolkanski_kolor4", "1F596-1F3FE"},
    {"salut_wolkanski_kolor5", "1F596-1F3FF"},
    {"czarne_serce", "1F5A4"},
    {"komputer", "1F5A5"},
    {"drukarka", "1F5A8"},
    {"mysz_komputerowa", "1F5B1"},
    {"trackball", "1F5B2"},
    {"obrazek_w_ramce", "1F5BC"},
    {"rozdzielacze_kartoteki", "1F5C2"},
    {"pudelko_kartoteka", "1F5C3"},
    {"szafka_kartoteka", "1F5C4"},
    {"kosz_na_smieci", "1F5D1"},
    {"kolonotatnik", "1F5D2"},
    {"kalendarz_na_spirali", "1F5D3"},
    {"scisk", "1F5DC"},
    {"staroswiecki_klucz", "1F5DD"},
    {"zwinieta_gazeta", "1F5DE"},
    {"sztylet", "1F5E1"},
    {"mowiaca_glowa", "1F5E3"},
    {"lewostronny_dymek", "1F5E8"},
    {"prawostronny_dymek_zlosci", "1F5EF"},
    {"urna_wyborcza_z_glosem", "1F5F3"},
    {"mapa_swiata", "1F5FA"},
    {"gora_fudzi", "1F5FB"},
    {"tokyo_tower", "1F5FC"},
    {"statua_wolnosci", "1F5FD"},
    {"mapa_japonii", "1F5FE"},
    {"moai", "1F5FF"},
    {"szeroko_usmiechnieta_twarz", "1F600"},
    {"szeroko_usmiechnieta_twarz_o_rozesmianych_oczach", "1F601"},
    {"twarz_ze_lzami_radosci", "1F602"},
    {"usmiechnieta_twarz_z_otwartymi_ustami", "1F603"},
    {"usmiechnieta_twarz_z_otwartymi_ustami_i_rozesmianymi_oczami", "1F604"},
    {"usmiechnieta_twarz_z_otwartymi_ustami_oblana_zimnym_potem", "1F605"},
    {"usmiechnieta_twarz_z_przymruzonymi_oczami", "1F606"},
    {"usmiechnieta_twarz_z_aureola", "1F607"},
    {"usmiechnieta_twarz_z_rogami", "1F608"},
    {"twarz_puszczajaca_oko", "1F609"},
    {"usmiechnieta_twarz_o_rozesmianych_oczach", "1F60A"},
    {"twarz_delektujaca_sie_pysznym_jedzeniem", "1F60B"},
    {"twarz_z_wyrazem_ulgi", "1F60C"},
    {"usmiechnieta_twarz_z_oczami_w_ksztalcie_serca", "1F60D"},
    {"usmiechnieta_twarz_w_okularach_przeciwslonecznych", "1F60E"},
    {"twarz_z_usmieszkiem", "1F60F"},
    {"neutralna_twarz", "1F610"},
    {"twarz_bez_wyrazu", "1F611"},
    {"niezadowolona_twarz", "1F612"},
    {"twarz_ze_spuszczonymi_oczami_oblana_potem", "1F613"},
    {"zamyslona_twarz", "1F614"},
    {"zdezorientowana_twarz", "1F615"},
    {"zaklopotana_twarz", "1F616"},
    {"calujaca_twarz", "1F617"},
    {"twarz_przesylajaca_calusa", "1F618"},
    {"calujaca_twarz_o_rozesmianych_oczach", "1F619"},
    {"calujaca_twarz_z_zamknietymi_oczami", "1F61A"},
    {"twarz_wystawiajaca_jezyk", "1F61B"},
    {"twarz_wystawiajaca_jezyk_i_puszczajaca_oko", "1F61C"},
    {"twarz_ze_zmruzonymi_oczami_wystawiajaca_jezyk", "1F61D"},
    {"rozczarowana_twarz", "1F61E"},
    {"zmartwiona_twarz", "1F61F"},
    {"zagniewana_twarz", "1F620"},
    {"nadasana_twarz", "1F621"},
    {"placzaca_twarz", "1F622"},
    {"uparta_twarz", "1F623"},
    {"prychajaca_twarz", "1F624"},
    {"smutna_twarz_z_wyrazem_ulgi", "1F625"},
    {"zachmurzona_twarz_z_otwartymi_ustami", "1F626"},
    {"udreczona_twarz", "1F627"},
    {"przestraszona_twarz", "1F628"},
    {"znuzona_twarz", "1F629"},
    {"senna_twarz", "1F62A"},
    {"zmeczona_twarz", "1F62B"},
    {"twarz_z_grymasem", "1F62C"},
    {"glosno_placzaca_twarz", "1F62D"},
    {"twarz_z_otwartymi_ustami", "1F62E"},
    {"zdumiona_twarz", "1F62F"},
    {"zaniepokojona_twarz_oblana_potem", "1F630"},
    {"twarz_krzyczaca_ze_strachu", "1F631"},
    {"zadziwiona_twarz", "1F632"},
    {"twarz_z_rumiencami", "1F633"},
    {"spiaca_twarz", "1F634"},
    {"oszolomiona_twarz", "1F635"},
    {"twarz_bez_ust", "1F636"},
    {"twarz_w_masce_medycznej", "1F637"},
    {"usmiechniety_kot_o_wesolych_oczach", "1F638"},
    {"kot_ze_lzami_radosci", "1F639"},
    {"usmiechniety_kot", "1F63A"},
    {"usmiechniety_kot_z_oczami_w_ksztalcie_serca", "1F63B"},
    {"kot_z_drwiacym_usmiechem", "1F63C"},
    {"kot_przesylajacy_calusa", "1F63D"},
    {"nadasany_kot", "1F63E"},
    {"placzacy_kot", "1F63F"},
    {"przestraszony_kot", "1F640"},
    {"lekko_zachmurzona_twarz", "1F641"},
    {"lekko_usmiechnieta_twarz", "1F642"},
    {"odwrocona_twarz", "1F643"},
    {"twarz_przewracajaca_oczami", "1F644"},
    {"osoba_pokazujaca_gest_nie", "1F645"},
    {"osoba_pokazujaca_gest_nie_kolor1", "1F645-1F3FB"},
    {"osoba_pokazujaca_gest_nie_kolor2", "1F645-1F3FC"},
    {"osoba_pokazujaca_gest_nie_kolor3", "1F645-1F3FD"},
    {"osoba_pokazujaca_gest_nie_kolor4", "1F645-1F3FE"},
    {"osoba_pokazujaca_gest_nie_kolor5", "1F645-1F3FF"},
    {"osoba_pokazujaca_gest_ok", "1F646"},
    {"osoba_pokazujaca_gest_ok_kolor1", "1F646-1F3FB"},
    {"osoba_pokazujaca_gest_ok_kolor2", "1F646-1F3FC"},
    {"osoba_pokazujaca_gest_ok_kolor3", "1F646-1F3FD"},
    {"osoba_pokazujaca_gest_ok_kolor4", "1F646-1F3FE"},
    {"osoba_pokazujaca_gest_ok_kolor5", "1F646-1F3FF"},
    {"osoba_w_uklonie", "1F647"},
    {"osoba_w_uklonie_kolor1", "1F647-1F3FB"},
    {"osoba_w_uklonie_kolor2", "1F647-1F3FC"},
    {"osoba_w_uklonie_kolor3", "1F647-1F3FD"},
    {"osoba_w_uklonie_kolor4", "1F647-1F3FE"},
    {"osoba_w_uklonie_kolor5", "1F647-1F3FF"},
    {"malpa_zaslaniajaca_oczy", "1F648"},
    {"malpa_zaslaniajaca_uszy", "1F649"},
    {"malpa_zaslaniajaca_pyszczek", "1F64A"},
    {"osoba_podnoszaca_reke", "1F64B"},
    {"osoba_podnoszaca_reke_kolor1", "1F64B-1F3FB"},
    {"osoba_podnoszaca_reke_kolor2", "1F64B-1F3FC"},
    {"osoba_podnoszaca_reke_kolor3", "1F64B-1F3FD"},
    {"osoba_podnoszaca_reke_kolor4", "1F64B-1F3FE"},
    {"osoba_podnoszaca_reke_kolor5", "1F64B-1F3FF"},
    {"wzniesione_rece", "1F64C"},
    {"wzniesione_rece_kolor1", "1F64C-1F3FB"},
    {"wzniesione_rece_kolor2", "1F64C-1F3FC"},
    {"wzniesione_rece_kolor3", "1F64C-1F3FD"},
    {"wzniesione_rece_kolor4", "1F64C-1F3FE"},
    {"wzniesione_rece_kolor5", "1F64C-1F3FF"},
    {"zachmurzona_osoba", "1F64D"},
    {"zachmurzona_osoba_kolor1", "1F64D-1F3FB"},
    {"zachmurzona_osoba_kolor2", "1F64D-1F3FC"},
    {"zachmurzona_osoba_kolor3", "1F64D-1F3FD"},
    {"zachmurzona_osoba_kolor4", "1F64D-1F3FE"},
    {"zachmurzona_osoba_kolor5", "1F64D-1F3FF"},
    {"nadasana_osoba", "1F64E"},
    {"nadasana_osoba_kolor1", "1F64E-1F3FB"},
    {"nadasana_osoba_kolor2", "1F64E-1F3FC"},
    {"nadasana_osoba_kolor3", "1F64E-1F3FD"},
    {"nadasana_osoba_kolor4", "1F64E-1F3FE"},
    {"nadasana_osoba_kolor5", "1F64E-1F3FF"},
    {"zlozone_rece", "1F64F"},
    {"zlozone_rece_kolor1", "1F64F-1F3FB"},
    {"zlozone_rece_kolor2", "1F64F-1F3FC"},
    {"zlozone_rece_kolor3", "1F64F-1F3FD"},
    {"zlozone_rece_kolor4", "1F64F-1F3FE"},
    {"zlozone_rece_kolor5", "1F64F-1F3FF"},
    {"rakieta", "1F680"},
    {"helikopter", "1F681"},
    {"lokomotywa", "1F682"},
    {"wagon_kolejowy", "1F683"},
    {"szybki_pociag", "1F684"},
    {"szybki_pociag_z_zaokraglonym_przodem", "1F685"},
    {"pociag", "1F686"},
    {"metro", "1F687"},
    {"kolej_miejska", "1F688"},
    {"stacja", "1F689"},
    {"tramwaj", "1F68A"},
    {"wagon_tramwajowy", "1F68B"},
    {"autobus", "1F68C"},
    {"nadjezdzajacy_autobus", "1F68D"},
    {"trolejbus", "1F68E"},
    {"przystanek_autobusowy", "1F68F"},
    {"minibus", "1F690"},
    {"karetka_pogotowia", "1F691"},
    {"samochod_strazacki", "1F692"},
    {"samochod_policyjny", "1F693"},
    {"nadjezdzajacy_radiowoz", "1F694"},
    {"taksowka", "1F695"},
    {"nadjezdzajaca_taksowka", "1F696"},
    {"samochod", "1F697"},
    {"nadjezdzajacy_samochod", "1F698"},
    {"samochod_suv", "1F699"},
    {"samochod_dostawczy", "1F69A"},
    {"samochod_ciezarowy", "1F69B"},
    {"ciagnik", "1F69C"},
    {"kolej_jednoszynowa", "1F69D"},
    {"kolej_gorska", "1F69E"},
    {"kolej_podwieszana", "1F69F"},
    {"gorska_kolej_linowa", "1F6A0"},
    {"kolej_linowa", "1F6A1"},
    {"statek", "1F6A2"},
    {"osoba_wioslujaca_w_lodzi", "1F6A3"},
    {"osoba_wioslujaca_w_lodzi_kolor1", "1F6A3-1F3FB"},
    {"osoba_wioslujaca_w_lodzi_kolor2", "1F6A3-1F3FC"},
    {"osoba_wioslujaca_w_lodzi_kolor3", "1F6A3-1F3FD"},
    {"osoba_wioslujaca_w_lodzi_kolor4", "1F6A3-1F3FE"},
    {"osoba_wioslujaca_w_lodzi_kolor5", "1F6A3-1F3FF"},
    {"scigacz_wodny", "1F6A4"},
    {"pozioma_sygnalizacja_uliczna", "1F6A5"},
    {"pionowa_sygnalizacja_uliczna", "1F6A6"},
    {"prace_drogowe", "1F6A7"},
    {"kogut_policyjny", "1F6A8"},
    {"flaga_trojkatna", "1F6A9"},
    {"drzwi", "1F6AA"},
    {"zakaz", "1F6AB"},
    {"papieros", "1F6AC"},
    {"zakaz_palenia", "1F6AD"},
    {"znak_kosza_na_smieci", "1F6AE"},
    {"zakaz_smiecenia", "1F6AF"},
    {"woda_pitna", "1F6B0"},
    {"woda_niezdatna_do_picia", "1F6B1"},
    {"rower", "1F6B2"},
    {"zakaz_wjazdu_rowerow", "1F6B3"},
    {"osoba_na_rowerze", "1F6B4"},
    {"osoba_na_rowerze_kolor1", "1F6B4-1F3FB"},
    {"osoba_na_rowerze_kolor2", "1F6B4-1F3FC"},
    {"osoba_na_rowerze_kolor3", "1F6B4-1F3FD"},
    {"osoba_na_rowerze_kolor4", "1F6B4-1F3FE"},
    {"osoba_na_rowerze_kolor5", "1F6B4-1F3FF"},
    {"osoba_na_rowerze_gorskim", "1F6B5"},
    {"osoba_na_rowerze_gorskim_kolor1", "1F6B5-1F3FB"},
    {"osoba_na_rowerze_gorskim_kolor2", "1F6B5-1F3FC"},
    {"osoba_na_rowerze_gorskim_kolor3", "1F6B5-1F3FD"},
    {"osoba_na_rowerze_gorskim_kolor4", "1F6B5-1F3FE"},
    {"osoba_na_rowerze_gorskim_kolor5", "1F6B5-1F3FF"},
    {"idaca_osoba", "1F6B6"},
    {"idaca_osoba_kolor1", "1F6B6-1F3FB"},
    {"idaca_osoba_kolor2", "1F6B6-1F3FC"},
    {"idaca_osoba_kolor3", "1F6B6-1F3FD"},
    {"idaca_osoba_kolor4", "1F6B6-1F3FE"},
    {"idaca_osoba_kolor5", "1F6B6-1F3FF"},
    {"zakaz_ruchu_pieszych", "1F6B7"},
    {"dzieci_przechodzace_przez_jezdnie", "1F6B8"},
    {"toaleta_meska", "1F6B9"},
    {"toaleta_damska", "1F6BA"},
    {"toalety", "1F6BB"},
    {"symbol_niemowlecia", "1F6BC"},
    {"muszla_klozetowa", "1F6BD"},
    {"wc", "1F6BE"},
    {"prysznic", "1F6BF"},
    {"osoba_bioraca_kapiel", "1F6C0"},
    {"osoba_bioraca_kapiel_kolor1", "1F6C0-1F3FB"},
    {"osoba_bioraca_kapiel_kolor2", "1F6C0-1F3FC"},
    {"osoba_bioraca_kapiel_kolor3", "1F6C0-1F3FD"},
    {"osoba_bioraca_kapiel_kolor4", "1F6C0-1F3FE"},
    {"osoba_bioraca_kapiel_kolor5", "1F6C0-1F3FF"},
    {"wanna", "1F6C1"},
    {"kontrola_paszportowa", "1F6C2"},
    {"kontrola_celna", "1F6C3"},
    {"odbior_bagazu", "1F6C4"},
    {"przechowalnia_bagazu", "1F6C5"},
    {"kanapa_i_lampa", "1F6CB"},
    {"osoba_w_lozku", "1F6CC"},
    {"osoba_w_lozku_kolor1", "1F6CC-1F3FB"},
    {"osoba_w_lozku_kolor2", "1F6CC-1F3FC"},
    {"osoba_w_lozku_kolor3", "1F6CC-1F3FD"},
    {"osoba_w_lozku_kolor4", "1F6CC-1F3FE"},
    {"osoba_w_lozku_kolor5", "1F6CC-1F3FF"},
    {"torby_na_zakupy", "1F6CD"},
    {"dzwonek_w_recepcji", "1F6CE"},
    {"lozko", "1F6CF"},
    {"miejsce_kultu", "1F6D0"},
    {"znak_stopu", "1F6D1"},
    {"wozek_sklepowy", "1F6D2"},
    {"swiatynia_hinduistyczna", "1F6D5"},
    {"chata", "1F6D6"},
    {"winda", "1F6D7"},
    {"osuwisko", "1F6D8"},
    {"bezprzewodowy", "1F6DC"},
    {"zjezdzalnia", "1F6DD"},
    {"kolo_pojazdu", "1F6DE"},
    {"kolo_ratunkowe", "1F6DF"},
    {"mlot_i_klucz", "1F6E0"},
    {"tarcza", "1F6E1"},
    {"beczka_oleju", "1F6E2"},
    {"autostrada", "1F6E3"},
    {"tor_kolejowy", "1F6E4"},
    {"motorowka", "1F6E5"},
    {"maly_samolot", "1F6E9"},
    {"odlot_samolotu", "1F6EB"},
    {"przylot_samolotu", "1F6EC"},
    {"satelita", "1F6F0"},
    {"statek_pasazerski", "1F6F3"},
    {"hulajnoga", "1F6F4"},
    {"skuter", "1F6F5"},
    {"kajak", "1F6F6"},
    {"sanie", "1F6F7"},
    {"latajacy_talerz", "1F6F8"},
    {"deskorolka", "1F6F9"},
    {"autoriksza", "1F6FA"},
    {"pick_up", "1F6FB"},
    {"wrotka", "1F6FC"},
    {"pomaranczowe_kolo", "1F7E0"},
    {"zolte_kolo", "1F7E1"},
    {"zielone_kolo", "1F7E2"},
    {"fioletowe_kolo", "1F7E3"},
    {"brazowe_kolo", "1F7E4"},
    {"czerwony_kwadrat", "1F7E5"},
    {"niebieski_kwadrat", "1F7E6"},
    {"pomaranczowy_kwadrat", "1F7E7"},
    {"zolty_kwadrat", "1F7E8"},
    {"zielony_kwadrat", "1F7E9"},
    {"fioletowy_kwadrat", "1F7EA"},
    {"brazowy_kwadrat", "1F7EB"},
    {"gruby_znak_rownosci", "1F7F0"},
    {"zlaczone_palce", "1F90C"},
    {"zlaczone_palce_kolor1", "1F90C-1F3FB"},
    {"zlaczone_palce_kolor2", "1F90C-1F3FC"},
    {"zlaczone_palce_kolor3", "1F90C-1F3FD"},
    {"zlaczone_palce_kolor4", "1F90C-1F3FE"},
    {"zlaczone_palce_kolor5", "1F90C-1F3FF"},
    {"biale_serce", "1F90D"},
    {"brazowe_serce", "1F90E"},
    {"dlon_z_gestem_troche", "1F90F"},
    {"dlon_z_gestem_troche_kolor1", "1F90F-1F3FB"},
    {"dlon_z_gestem_troche_kolor2", "1F90F-1F3FC"},
    {"dlon_z_gestem_troche_kolor3", "1F90F-1F3FD"},
    {"dlon_z_gestem_troche_kolor4", "1F90F-1F3FE"},
    {"dlon_z_gestem_troche_kolor5", "1F90F-1F3FF"},
    {"twarz_z_zamkiem_blyskawicznym_na_ustach", "1F910"},
    {"twarz_z_pieniedzmi_na_ustach", "1F911"},
    {"twarz_z_termometrem", "1F912"},
    {"kujon", "1F913"},
    {"myslaca_twarz", "1F914"},
    {"twarz_z_bandazem", "1F915"},
    {"glowa_robota", "1F916"},
    {"twarz_z_gestem_przytulania", "1F917"},
    {"gest_rogow", "1F918"},
    {"gest_rogow_kolor1", "1F918-1F3FB"},
    {"gest_rogow_kolor2", "1F918-1F3FC"},
    {"gest_rogow_kolor3", "1F918-1F3FD"},
    {"gest_rogow_kolor4", "1F918-1F3FE"},
    {"gest_rogow_kolor5", "1F918-1F3FF"},
    {"dlon_w_gescie_zadzwon_do_mnie", "1F919"},
    {"dlon_w_gescie_zadzwon_do_mnie_kolor1", "1F919-1F3FB"},
    {"dlon_w_gescie_zadzwon_do_mnie_kolor2", "1F919-1F3FC"},
    {"dlon_w_gescie_zadzwon_do_mnie_kolor3", "1F919-1F3FD"},
    {"dlon_w_gescie_zadzwon_do_mnie_kolor4", "1F919-1F3FE"},
    {"dlon_w_gescie_zadzwon_do_mnie_kolor5", "1F919-1F3FF"},
    {"wzniesiony_grzbiet_dloni", "1F91A"},
    {"wzniesiony_grzbiet_dloni_kolor1", "1F91A-1F3FB"},
    {"wzniesiony_grzbiet_dloni_kolor2", "1F91A-1F3FC"},
    {"wzniesiony_grzbiet_dloni_kolor3", "1F91A-1F3FD"},
    {"wzniesiony_grzbiet_dloni_kolor4", "1F91A-1F3FE"},
    {"wzniesiony_grzbiet_dloni_kolor5", "1F91A-1F3FF"},
    {"piesc_skierowana_w_lewo", "1F91B"},
    {"piesc_skierowana_w_lewo_kolor1", "1F91B-1F3FB"},
    {"piesc_skierowana_w_lewo_kolor2", "1F91B-1F3FC"},
    {"piesc_skierowana_w_lewo_kolor3", "1F91B-1F3FD"},
    {"piesc_skierowana_w_lewo_kolor4", "1F91B-1F3FE"},
    {"piesc_skierowana_w_lewo_kolor5", "1F91B-1F3FF"},
    {"piesc_skierowana_w_prawo", "1F91C"},
    {"piesc_skierowana_w_prawo_kolor1", "1F91C-1F3FB"},
    {"piesc_skierowana_w_prawo_kolor2", "1F91C-1F3FC"},
    {"piesc_skierowana_w_prawo_kolor3", "1F91C-1F3FD"},
    {"piesc_skierowana_w_prawo_kolor4", "1F91C-1F3FE"},
    {"piesc_skierowana_w_prawo_kolor5", "1F91C-1F3FF"},
    {"uscisk_dloni", "1F91D"},
    {"uscisk_dloni_kolor1", "1F91D-1F3FB"},
    {"uscisk_dloni_kolor2", "1F91D-1F3FC"},
    {"uscisk_dloni_kolor3", "1F91D-1F3FD"},
    {"uscisk_dloni_kolor4", "1F91D-1F3FE"},
    {"uscisk_dloni_kolor5", "1F91D-1F3FF"},
    {"uscisk_dloni_kolor1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"uscisk_dloni_kolor1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"uscisk_dloni_kolor1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"uscisk_dloni_kolor1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"uscisk_dloni_kolor2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"uscisk_dloni_kolor2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"uscisk_dloni_kolor2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"uscisk_dloni_kolor2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"uscisk_dloni_kolor3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"uscisk_dloni_kolor3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"uscisk_dloni_kolor3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"uscisk_dloni_kolor3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"uscisk_dloni_kolor4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"uscisk_dloni_kolor4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"uscisk_dloni_kolor4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"uscisk_dloni_kolor4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"uscisk_dloni_kolor5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"uscisk_dloni_kolor5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"uscisk_dloni_kolor5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"uscisk_dloni_kolor5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"skrzyzowane_palce", "1F91E"},
    {"skrzyzowane_palce_kolor1", "1F91E-1F3FB"},
    {"skrzyzowane_palce_kolor2", "1F91E-1F3FC"},
    {"skrzyzowane_palce_kolor3", "1F91E-1F3FD"},
    {"skrzyzowane_palce_kolor4", "1F91E-1F3FE"},
    {"skrzyzowane_palce_kolor5", "1F91E-1F3FF"},
    {"gest_kocham_cie", "1F91F"},
    {"gest_kocham_cie_kolor1", "1F91F-1F3FB"},
    {"gest_kocham_cie_kolor2", "1F91F-1F3FC"},
    {"gest_kocham_cie_kolor3", "1F91F-1F3FD"},
    {"gest_kocham_cie_kolor4", "1F91F-1F3FE"},
    {"gest_kocham_cie_kolor5", "1F91F-1F3FF"},
    {"glowa_w_kapeluszu_kowbojskim", "1F920"},
    {"twarz_klauna", "1F921"},
    {"twarz_z_mdlosciami", "1F922"},
    {"tarza_sie_ze_smiechu", "1F923"},
    {"sliniaca_sie_twarz", "1F924"},
    {"twarz_klamcy", "1F925"},
    {"osoba_trzymajaca_sie_za_czolo", "1F926"},
    {"osoba_trzymajaca_sie_za_czolo_kolor1", "1F926-1F3FB"},
    {"osoba_trzymajaca_sie_za_czolo_kolor2", "1F926-1F3FC"},
    {"osoba_trzymajaca_sie_za_czolo_kolor3", "1F926-1F3FD"},
    {"osoba_trzymajaca_sie_za_czolo_kolor4", "1F926-1F3FE"},
    {"osoba_trzymajaca_sie_za_czolo_kolor5", "1F926-1F3FF"},
    {"kichajaca_twarz", "1F927"},
    {"twarz_z_podniesiona_brwia", "1F928"},
    {"zafascynowany", "1F929"},
    {"szalona_twarz", "1F92A"},
    {"uciszajaca_twarz", "1F92B"},
    {"twarz_z_symbolami_na_ustach", "1F92C"},
    {"twarz_z_reka_na_ustach", "1F92D"},
    {"wymiotujaca_twarz", "1F92E"},
    {"eksplodujaca_glowa", "1F92F"},
    {"kobieta_w_ciazy", "1F930"},
    {"kobieta_w_ciazy_kolor1", "1F930-1F3FB"},
    {"kobieta_w_ciazy_kolor2", "1F930-1F3FC"},
    {"kobieta_w_ciazy_kolor3", "1F930-1F3FD"},
    {"kobieta_w_ciazy_kolor4", "1F930-1F3FE"},
    {"kobieta_w_ciazy_kolor5", "1F930-1F3FF"},
    {"karmienie_piersia", "1F931"},
    {"karmienie_piersia_kolor1", "1F931-1F3FB"},
    {"karmienie_piersia_kolor2", "1F931-1F3FC"},
    {"karmienie_piersia_kolor3", "1F931-1F3FD"},
    {"karmienie_piersia_kolor4", "1F931-1F3FE"},
    {"karmienie_piersia_kolor5", "1F931-1F3FF"},
    {"dlonie_do_gory_razem", "1F932"},
    {"dlonie_do_gory_razem_kolor1", "1F932-1F3FB"},
    {"dlonie_do_gory_razem_kolor2", "1F932-1F3FC"},
    {"dlonie_do_gory_razem_kolor3", "1F932-1F3FD"},
    {"dlonie_do_gory_razem_kolor4", "1F932-1F3FE"},
    {"dlonie_do_gory_razem_kolor5", "1F932-1F3FF"},
    {"selfie", "1F933"},
    {"selfie_kolor1", "1F933-1F3FB"},
    {"selfie_kolor2", "1F933-1F3FC"},
    {"selfie_kolor3", "1F933-1F3FD"},
    {"selfie_kolor4", "1F933-1F3FE"},
    {"selfie_kolor5", "1F933-1F3FF"},
    {"ksiaze", "1F934"},
    {"ksiaze_kolor1", "1F934-1F3FB"},
    {"ksiaze_kolor2", "1F934-1F3FC"},
    {"ksiaze_kolor3", "1F934-1F3FD"},
    {"ksiaze_kolor4", "1F934-1F3FE"},
    {"ksiaze_kolor5", "1F934-1F3FF"},
    {"osoba_w_smokingu", "1F935"},
    {"osoba_w_smokingu_kolor1", "1F935-1F3FB"},
    {"osoba_w_smokingu_kolor2", "1F935-1F3FC"},
    {"osoba_w_smokingu_kolor3", "1F935-1F3FD"},
    {"osoba_w_smokingu_kolor4", "1F935-1F3FE"},
    {"osoba_w_smokingu_kolor5", "1F935-1F3FF"},
    {"zona_sw_mikolaja", "1F936"},
    {"zona_sw_mikolaja_kolor1", "1F936-1F3FB"},
    {"zona_sw_mikolaja_kolor2", "1F936-1F3FC"},
    {"zona_sw_mikolaja_kolor3", "1F936-1F3FD"},
    {"zona_sw_mikolaja_kolor4", "1F936-1F3FE"},
    {"zona_sw_mikolaja_kolor5", "1F936-1F3FF"},
    {"osoba_wzruszajaca_ramionami", "1F937"},
    {"osoba_wzruszajaca_ramionami_kolor1", "1F937-1F3FB"},
    {"osoba_wzruszajaca_ramionami_kolor2", "1F937-1F3FC"},
    {"osoba_wzruszajaca_ramionami_kolor3", "1F937-1F3FD"},
    {"osoba_wzruszajaca_ramionami_kolor4", "1F937-1F3FE"},
    {"osoba_wzruszajaca_ramionami_kolor5", "1F937-1F3FF"},
    {"osoba_robiaca_gwiazde", "1F938"},
    {"osoba_robiaca_gwiazde_kolor1", "1F938-1F3FB"},
    {"osoba_robiaca_gwiazde_kolor2", "1F938-1F3FC"},
    {"osoba_robiaca_gwiazde_kolor3", "1F938-1F3FD"},
    {"osoba_robiaca_gwiazde_kolor4", "1F938-1F3FE"},
    {"osoba_robiaca_gwiazde_kolor5", "1F938-1F3FF"},
    {"zonglujaca_osoba", "1F939"},
    {"zonglujaca_osoba_kolor1", "1F939-1F3FB"},
    {"zonglujaca_osoba_kolor2", "1F939-1F3FC"},
    {"zonglujaca_osoba_kolor3", "1F939-1F3FD"},
    {"zonglujaca_osoba_kolor4", "1F939-1F3FE"},
    {"zonglujaca_osoba_kolor5", "1F939-1F3FF"},
    {"osoba_uprawiajaca_szermierke", "1F93A"},
    {"osoby_uprawiajace_zapasy", "1F93C"},
    {"osoby_uprawiajace_zapasy_kolor1", "1F93C-1F3FB"},
    {"osoby_uprawiajace_zapasy_kolor2", "1F93C-1F3FC"},
    {"osoby_uprawiajace_zapasy_kolor3", "1F93C-1F3FD"},
    {"osoby_uprawiajace_zapasy_kolor4", "1F93C-1F3FE"},
    {"osoby_uprawiajace_zapasy_kolor5", "1F93C-1F3FF"},
    {"osoby_uprawiajace_zapasy_kolor1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"osoby_uprawiajace_zapasy_kolor1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"osoby_uprawiajace_zapasy_kolor1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"osoby_uprawiajace_zapasy_kolor1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"osoby_uprawiajace_zapasy_kolor2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"osoby_uprawiajace_zapasy_kolor2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"osoby_uprawiajace_zapasy_kolor2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"osoby_uprawiajace_zapasy_kolor2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"osoby_uprawiajace_zapasy_kolor3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"osoby_uprawiajace_zapasy_kolor3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"osoby_uprawiajace_zapasy_kolor3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"osoby_uprawiajace_zapasy_kolor3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"osoby_uprawiajace_zapasy_kolor4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"osoby_uprawiajace_zapasy_kolor4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"osoby_uprawiajace_zapasy_kolor4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"osoby_uprawiajace_zapasy_kolor4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"osoby_uprawiajace_zapasy_kolor5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"osoby_uprawiajace_zapasy_kolor5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"osoby_uprawiajace_zapasy_kolor5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"osoby_uprawiajace_zapasy_kolor5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"osoba_grajaca_w_pilke_wodna", "1F93D"},
    {"osoba_grajaca_w_pilke_wodna_kolor1", "1F93D-1F3FB"},
    {"osoba_grajaca_w_pilke_wodna_kolor2", "1F93D-1F3FC"},
    {"osoba_grajaca_w_pilke_wodna_kolor3", "1F93D-1F3FD"},
    {"osoba_grajaca_w_pilke_wodna_kolor4", "1F93D-1F3FE"},
    {"osoba_grajaca_w_pilke_wodna_kolor5", "1F93D-1F3FF"},
    {"osoba_grajaca_w_pilke_reczna", "1F93E"},
    {"osoba_grajaca_w_pilke_reczna_kolor1", "1F93E-1F3FB"},
    {"osoba_grajaca_w_pilke_reczna_kolor2", "1F93E-1F3FC"},
    {"osoba_grajaca_w_pilke_reczna_kolor3", "1F93E-1F3FD"},
    {"osoba_grajaca_w_pilke_reczna_kolor4", "1F93E-1F3FE"},
    {"osoba_grajaca_w_pilke_reczna_kolor5", "1F93E-1F3FF"},
    {"maska_do_nurkowania", "1F93F"},
    {"zwiedly_kwiat", "1F940"},
    {"beben", "1F941"},
    {"stukajace_sie_kieliszki", "1F942"},
    {"szklanka_z_grubym_dnem", "1F943"},
    {"lyzka", "1F944"},
    {"bramka", "1F945"},
    {"medal_za_1_miejsce", "1F947"},
    {"medal_za_2_miejsce", "1F948"},
    {"medal_za_3_miejsce", "1F949"},
    {"rekawica_bokserska", "1F94A"},
    {"stroj_do_sztuk_walki", "1F94B"},
    {"kamien_do_curlingu", "1F94C"},
    {"lacrosse", "1F94D"},
    {"softball", "1F94E"},
    {"frisbee", "1F94F"},
    {"rogalik", "1F950"},
    {"awokado", "1F951"},
    {"ogorek", "1F952"},
    {"boczek", "1F953"},
    {"ziemniak", "1F954"},
    {"marchew", "1F955"},
    {"bagietka", "1F956"},
    {"salatka", "1F957"},
    {"danie_z_patelni", "1F958"},
    {"wrap", "1F959"},
    {"jajko", "1F95A"},
    {"szklanka_mleka", "1F95B"},
    {"orzeszki_ziemne", "1F95C"},
    {"owoc_kiwi", "1F95D"},
    {"nalesniki", "1F95E"},
    {"pierog", "1F95F"},
    {"ciastko_z_wrozba", "1F960"},
    {"pudelko_na_wynos", "1F961"},
    {"paleczki", "1F962"},
    {"miska_z_lyzka", "1F963"},
    {"szklanka_ze_slomka", "1F964"},
    {"kokos", "1F965"},
    {"brokul", "1F966"},
    {"ciasto", "1F967"},
    {"precel", "1F968"},
    {"kawalek_miesa", "1F969"},
    {"kanapka", "1F96A"},
    {"jedzenie_w_puszce", "1F96B"},
    {"warzywo_lisciaste", "1F96C"},
    {"mango", "1F96D"},
    {"ciasteczko_ksiezycowe", "1F96E"},
    {"bajgiel", "1F96F"},
    {"usmiechnieta_buzka_z_sercami", "1F970"},
    {"ziewajaca_twarz", "1F971"},
    {"usmiechnieta_buzka_ze_lza", "1F972"},
    {"swietujaca_twarz", "1F973"},
    {"zamroczona_twarz", "1F974"},
    {"rozgrzana_twarz", "1F975"},
    {"zmarznieta_twarz", "1F976"},
    {"ninja", "1F977"},
    {"ninja_kolor1", "1F977-1F3FB"},
    {"ninja_kolor2", "1F977-1F3FC"},
    {"ninja_kolor3", "1F977-1F3FD"},
    {"ninja_kolor4", "1F977-1F3FE"},
    {"ninja_kolor5", "1F977-1F3FF"},
    {"buzka_w_przebraniu", "1F978"},
    {"twarz_powstrzymujaca_lzy", "1F979"},
    {"blagajaca_twarz", "1F97A"},
    {"sari", "1F97B"},
    {"bialy_fartuch", "1F97C"},
    {"okulary_ochronne", "1F97D"},
    {"but_turystyczny", "1F97E"},
    {"baleriny", "1F97F"},
    {"krab", "1F980"},
    {"glowa_lwa", "1F981"},
    {"brazowy_skorpion", "1F982"},
    {"indyk", "1F983"},
    {"glowa_jednorozca", "1F984"},
    {"orzel", "1F985"},
    {"kaczka", "1F986"},
    {"nietoperz", "1F987"},
    {"rekin", "1F988"},
    {"sowa", "1F989"},
    {"glowa_lisa", "1F98A"},
    {"motyl", "1F98B"},
    {"jelen", "1F98C"},
    {"goryl", "1F98D"},
    {"jaszczurka", "1F98E"},
    {"nosorozec", "1F98F"},
    {"krewetka", "1F990"},
    {"kalamarnica", "1F991"},
    {"zyrafa", "1F992"},
    {"zebra", "1F993"},
    {"jez", "1F994"},
    {"zauropod", "1F995"},
    {"trex", "1F996"},
    {"swierszcz", "1F997"},
    {"kangur", "1F998"},
    {"lama", "1F999"},
    {"paw", "1F99A"},
    {"hipopotam", "1F99B"},
    {"papuga", "1F99C"},
    {"szop", "1F99D"},
    {"homar", "1F99E"},
    {"komar", "1F99F"},
    {"zarazek", "1F9A0"},
    {"borsuk", "1F9A1"},
    {"labedz", "1F9A2"},
    {"mamut", "1F9A3"},
    {"dodo", "1F9A4"},
    {"leniwiec", "1F9A5"},
    {"wydra", "1F9A6"},
    {"orangutan", "1F9A7"},
    {"skunks", "1F9A8"},
    {"flaming", "1F9A9"},
    {"ostryga", "1F9AA"},
    {"bobr", "1F9AB"},
    {"zubr", "1F9AC"},
    {"foka", "1F9AD"},
    {"pies_przewodnik", "1F9AE"},
    {"biala_laska", "1F9AF"},
    {"rude_wlosy", "1F9B0"},
    {"krecone_wlosy", "1F9B1"},
    {"lysy", "1F9B2"},
    {"biale_wlosy", "1F9B3"},
    {"kosc", "1F9B4"},
    {"noga", "1F9B5"},
    {"noga_kolor1", "1F9B5-1F3FB"},
    {"noga_kolor2", "1F9B5-1F3FC"},
    {"noga_kolor3", "1F9B5-1F3FD"},
    {"noga_kolor4", "1F9B5-1F3FE"},
    {"noga_kolor5", "1F9B5-1F3FF"},
    {"stopa", "1F9B6"},
    {"stopa_kolor1", "1F9B6-1F3FB"},
    {"stopa_kolor2", "1F9B6-1F3FC"},
    {"stopa_kolor3", "1F9B6-1F3FD"},
    {"stopa_kolor4", "1F9B6-1F3FE"},
    {"stopa_kolor5", "1F9B6-1F3FF"},
    {"zab", "1F9B7"},
    {"superbohater", "1F9B8"},
    {"superbohater_kolor1", "1F9B8-1F3FB"},
    {"superbohater_kolor2", "1F9B8-1F3FC"},
    {"superbohater_kolor3", "1F9B8-1F3FD"},
    {"superbohater_kolor4", "1F9B8-1F3FE"},
    {"superbohater_kolor5", "1F9B8-1F3FF"},
    {"superlotr", "1F9B9"},
    {"superlotr_kolor1", "1F9B9-1F3FB"},
    {"superlotr_kolor2", "1F9B9-1F3FC"},
    {"superlotr_kolor3", "1F9B9-1F3FD"},
    {"superlotr_kolor4", "1F9B9-1F3FE"},
    {"superlotr_kolor5", "1F9B9-1F3FF"},
    {"kamizelka_ratunkowa", "1F9BA"},
    {"ucho_z_aparatem_sluchowym", "1F9BB"},
    {"ucho_z_aparatem_sluchowym_kolor1", "1F9BB-1F3FB"},
    {"ucho_z_aparatem_sluchowym_kolor2", "1F9BB-1F3FC"},
    {"ucho_z_aparatem_sluchowym_kolor3", "1F9BB-1F3FD"},
    {"ucho_z_aparatem_sluchowym_kolor4", "1F9BB-1F3FE"},
    {"ucho_z_aparatem_sluchowym_kolor5", "1F9BB-1F3FF"},
    {"wozek_inwalidzki_elektryczny", "1F9BC"},
    {"wozek_inwalidzki_reczny", "1F9BD"},
    {"mechaniczne_ramie", "1F9BE"},
    {"proteza_nogi", "1F9BF"},
    {"kawalek_sera", "1F9C0"},
    {"babeczka", "1F9C1"},
    {"sol", "1F9C2"},
    {"napoj_w_kartoniku", "1F9C3"},
    {"czosnek", "1F9C4"},
    {"cebula", "1F9C5"},
    {"falafel", "1F9C6"},
    {"gofr", "1F9C7"},
    {"maslo", "1F9C8"},
    {"yerba_mate", "1F9C9"},
    {"kostka_lodu", "1F9CA"},
    {"napoj_bubble_tea", "1F9CB"},
    {"troll", "1F9CC"},
    {"stojaca_osoba", "1F9CD"},
    {"stojaca_osoba_kolor1", "1F9CD-1F3FB"},
    {"stojaca_osoba_kolor2", "1F9CD-1F3FC"},
    {"stojaca_osoba_kolor3", "1F9CD-1F3FD"},
    {"stojaca_osoba_kolor4", "1F9CD-1F3FE"},
    {"stojaca_osoba_kolor5", "1F9CD-1F3FF"},
    {"kleczaca_osoba", "1F9CE"},
    {"kleczaca_osoba_kolor1", "1F9CE-1F3FB"},
    {"kleczaca_osoba_kolor2", "1F9CE-1F3FC"},
    {"kleczaca_osoba_kolor3", "1F9CE-1F3FD"},
    {"kleczaca_osoba_kolor4", "1F9CE-1F3FE"},
    {"kleczaca_osoba_kolor5", "1F9CE-1F3FF"},
    {"osoba_glucha", "1F9CF"},
    {"osoba_glucha_kolor1", "1F9CF-1F3FB"},
    {"osoba_glucha_kolor2", "1F9CF-1F3FC"},
    {"osoba_glucha_kolor3", "1F9CF-1F3FD"},
    {"osoba_glucha_kolor4", "1F9CF-1F3FE"},
    {"osoba_glucha_kolor5", "1F9CF-1F3FF"},
    {"twarz_z_monoklem", "1F9D0"},
    {"dorosly", "1F9D1"},
    {"dorosly_kolor1", "1F9D1-1F3FB"},
    {"dorosly_kolor2", "1F9D1-1F3FC"},
    {"dorosly_kolor3", "1F9D1-1F3FD"},
    {"dorosly_kolor4", "1F9D1-1F3FE"},
    {"dorosly_kolor5", "1F9D1-1F3FF"},
    {"dziecko", "1F9D2"},
    {"dziecko_kolor1", "1F9D2-1F3FB"},
    {"dziecko_kolor2", "1F9D2-1F3FC"},
    {"dziecko_kolor3", "1F9D2-1F3FD"},
    {"dziecko_kolor4", "1F9D2-1F3FE"},
    {"dziecko_kolor5", "1F9D2-1F3FF"},
    {"starsza_osoba", "1F9D3"},
    {"starsza_osoba_kolor1", "1F9D3-1F3FB"},
    {"starsza_osoba_kolor2", "1F9D3-1F3FC"},
    {"starsza_osoba_kolor3", "1F9D3-1F3FD"},
    {"starsza_osoba_kolor4", "1F9D3-1F3FE"},
    {"starsza_osoba_kolor5", "1F9D3-1F3FF"},
    {"osoba_z_broda", "1F9D4"},
    {"osoba_z_broda_kolor1", "1F9D4-1F3FB"},
    {"osoba_z_broda_kolor2", "1F9D4-1F3FC"},
    {"osoba_z_broda_kolor3", "1F9D4-1F3FD"},
    {"osoba_z_broda_kolor4", "1F9D4-1F3FE"},
    {"osoba_z_broda_kolor5", "1F9D4-1F3FF"},
    {"kobieta_w_chuscie", "1F9D5"},
    {"kobieta_w_chuscie_kolor1", "1F9D5-1F3FB"},
    {"kobieta_w_chuscie_kolor2", "1F9D5-1F3FC"},
    {"kobieta_w_chuscie_kolor3", "1F9D5-1F3FD"},
    {"kobieta_w_chuscie_kolor4", "1F9D5-1F3FE"},
    {"kobieta_w_chuscie_kolor5", "1F9D5-1F3FF"},
    {"osoba_w_zaparowanym_pomieszczeniu", "1F9D6"},
    {"osoba_w_zaparowanym_pomieszczeniu_kolor1", "1F9D6-1F3FB"},
    {"osoba_w_zaparowanym_pomieszczeniu_kolor2", "1F9D6-1F3FC"},
    {"osoba_w_zaparowanym_pomieszczeniu_kolor3", "1F9D6-1F3FD"},
    {"osoba_w_zaparowanym_pomieszczeniu_kolor4", "1F9D6-1F3FE"},
    {"osoba_w_zaparowanym_pomieszczeniu_kolor5", "1F9D6-1F3FF"},
    {"osoba_wspinajaca_sie", "1F9D7"},
    {"osoba_wspinajaca_sie_kolor1", "1F9D7-1F3FB"},
    {"osoba_wspinajaca_sie_kolor2", "1F9D7-1F3FC"},
    {"osoba_wspinajaca_sie_kolor3", "1F9D7-1F3FD"},
    {"osoba_wspinajaca_sie_kolor4", "1F9D7-1F3FE"},
    {"osoba_wspinajaca_sie_kolor5", "1F9D7-1F3FF"},
    {"osoba_w_pozycji_lotosu", "1F9D8"},
    {"osoba_w_pozycji_lotosu_kolor1", "1F9D8-1F3FB"},
    {"osoba_w_pozycji_lotosu_kolor2", "1F9D8-1F3FC"},
    {"osoba_w_pozycji_lotosu_kolor3", "1F9D8-1F3FD"},
    {"osoba_w_pozycji_lotosu_kolor4", "1F9D8-1F3FE"},
    {"osoba_w_pozycji_lotosu_kolor5", "1F9D8-1F3FF"},
    {"mag", "1F9D9"},
    {"mag_kolor1", "1F9D9-1F3FB"},
    {"mag_kolor2", "1F9D9-1F3FC"},
    {"mag_kolor3", "1F9D9-1F3FD"},
    {"mag_kolor4", "1F9D9-1F3FE"},
    {"mag_kolor5", "1F9D9-1F3FF"},
    {"wrozka", "1F9DA"},
    {"wrozka_kolor1", "1F9DA-1F3FB"},
    {"wrozka_kolor2", "1F9DA-1F3FC"},
    {"wrozka_kolor3", "1F9DA-1F3FD"},
    {"wrozka_kolor4", "1F9DA-1F3FE"},
    {"wrozka_kolor5", "1F9DA-1F3FF"},
    {"wampir", "1F9DB"},
    {"wampir_kolor1", "1F9DB-1F3FB"},
    {"wampir_kolor2", "1F9DB-1F3FC"},
    {"wampir_kolor3", "1F9DB-1F3FD"},
    {"wampir_kolor4", "1F9DB-1F3FE"},
    {"wampir_kolor5", "1F9DB-1F3FF"},
    {"syrena", "1F9DC"},
    {"syrena_kolor1", "1F9DC-1F3FB"},
    {"syrena_kolor2", "1F9DC-1F3FC"},
    {"syrena_kolor3", "1F9DC-1F3FD"},
    {"syrena_kolor4", "1F9DC-1F3FE"},
    {"syrena_kolor5", "1F9DC-1F3FF"},
    {"elf", "1F9DD"},
    {"elf_kolor1", "1F9DD-1F3FB"},
    {"elf_kolor2", "1F9DD-1F3FC"},
    {"elf_kolor3", "1F9DD-1F3FD"},
    {"elf_kolor4", "1F9DD-1F3FE"},
    {"elf_kolor5", "1F9DD-1F3FF"},
    {"dzin", "1F9DE"},
    {"zombie", "1F9DF"},
    {"mozg", "1F9E0"},
    {"pomaranczowe_serce", "1F9E1"},
    {"bejsbolowka", "1F9E2"},
    {"szalik", "1F9E3"},
    {"rekawiczki", "1F9E4"},
    {"plaszcz", "1F9E5"},
    {"skarpetki", "1F9E6"},
    {"czerwona_koperta", "1F9E7"},
    {"petarda", "1F9E8"},
    {"element_ukladanki", "1F9E9"},
    {"probowka", "1F9EA"},
    {"plytka_petriego", "1F9EB"},
    {"dna", "1F9EC"},
    {"kompas", "1F9ED"},
    {"liczydlo", "1F9EE"},
    {"gasnica", "1F9EF"},
    {"skrzynka_na_narzedzia", "1F9F0"},
    {"cegla", "1F9F1"},
    {"magnes", "1F9F2"},
    {"bagaz", "1F9F3"},
    {"butelka_z_plynem", "1F9F4"},
    {"nic", "1F9F5"},
    {"wloczka", "1F9F6"},
    {"agrafka", "1F9F7"},
    {"mis_pluszowy", "1F9F8"},
    {"miotla", "1F9F9"},
    {"kosz", "1F9FA"},
    {"rolka_papieru", "1F9FB"},
    {"mydlo", "1F9FC"},
    {"gabka", "1F9FD"},
    {"paragon", "1F9FE"},
    {"amulet_nazar", "1F9FF"},
    {"baletki", "1FA70"},
    {"stroj_kapielowy_jednoczesciowy", "1FA71"},
    {"kapielowki", "1FA72"},
    {"szorty", "1FA73"},
    {"japonka", "1FA74"},
    {"jasnoniebieskie_serce", "1FA75"},
    {"szare_serce", "1FA76"},
    {"rozowe_serce", "1FA77"},
    {"kropla_krwi", "1FA78"},
    {"plaster", "1FA79"},
    {"stetoskop", "1FA7A"},
    {"zdjecie_rentgenowskie", "1FA7B"},
    {"kula", "1FA7C"},
    {"jojo", "1FA80"},
    {"latawiec", "1FA81"},
    {"spadochron", "1FA82"},
    {"bumerang", "1FA83"},
    {"czarodziejska_rozdzka", "1FA84"},
    {"piniata", "1FA85"},
    {"matrioszki", "1FA86"},
    {"marakasy", "1FA87"},
    {"flet", "1FA88"},
    {"harfa", "1FA89"},
    {"puzon", "1FA8A"},
    {"skrzynia_skarbow", "1FA8E"},
    {"lopata", "1FA8F"},
    {"planeta_z_pierscieniami", "1FA90"},
    {"krzeslo", "1FA91"},
    {"brzytwa", "1FA92"},
    {"siekiera", "1FA93"},
    {"dipa", "1FA94"},
    {"banjo", "1FA95"},
    {"helm_wojskowy", "1FA96"},
    {"akordeon", "1FA97"},
    {"konga", "1FA98"},
    {"moneta", "1FA99"},
    {"pila_do_drewna", "1FA9A"},
    {"srubokret", "1FA9B"},
    {"drabina", "1FA9C"},
    {"hak", "1FA9D"},
    {"lustro", "1FA9E"},
    {"okno", "1FA9F"},
    {"przepychacz", "1FAA0"},
    {"igla_krawiecka", "1FAA1"},
    {"wezel", "1FAA2"},
    {"wiadro", "1FAA3"},
    {"pulapka_na_myszy", "1FAA4"},
    {"szczoteczka_do_zebow", "1FAA5"},
    {"nagrobek", "1FAA6"},
    {"tabliczka", "1FAA7"},
    {"skala", "1FAA8"},
    {"kula_dyskotekowa", "1FAA9"},
    {"dowod_tozsamosci", "1FAAA"},
    {"slaba_bateria", "1FAAB"},
    {"chamsa", "1FAAC"},
    {"wachlarz", "1FAAD"},
    {"grzebien_fryzjerski", "1FAAE"},
    {"khanda", "1FAAF"},
    {"mucha", "1FAB0"},
    {"robak", "1FAB1"},
    {"chrzaszcz", "1FAB2"},
    {"karaluch", "1FAB3"},
    {"roslina_doniczkowa", "1FAB4"},
    {"drewno", "1FAB5"},
    {"pioro", "1FAB6"},
    {"lotos", "1FAB7"},
    {"koralowiec", "1FAB8"},
    {"puste_gniazdo", "1FAB9"},
    {"gniazdo_z_jajkami", "1FABA"},
    {"hiacynt", "1FABB"},
    {"meduza", "1FABC"},
    {"skrzydlo", "1FABD"},
    {"drzewo_bez_lisci", "1FABE"},
    {"ges", "1FABF"},
    {"serce", "1FAC0"},
    {"pluca", "1FAC1"},
    {"obejmujace_sie_osoby", "1FAC2"},
    {"mezczyzna_w_ciazy", "1FAC3"},
    {"mezczyzna_w_ciazy_kolor1", "1FAC3-1F3FB"},
    {"mezczyzna_w_ciazy_kolor2", "1FAC3-1F3FC"},
    {"mezczyzna_w_ciazy_kolor3", "1FAC3-1F3FD"},
    {"mezczyzna_w_ciazy_kolor4", "1FAC3-1F3FE"},
    {"mezczyzna_w_ciazy_kolor5", "1FAC3-1F3FF"},
    {"osoba_w_ciazy", "1FAC4"},
    {"osoba_w_ciazy_kolor1", "1FAC4-1F3FB"},
    {"osoba_w_ciazy_kolor2", "1FAC4-1F3FC"},
    {"osoba_w_ciazy_kolor3", "1FAC4-1F3FD"},
    {"osoba_w_ciazy_kolor4", "1FAC4-1F3FE"},
    {"osoba_w_ciazy_kolor5", "1FAC4-1F3FF"},
    {"osoba_w_koronie", "1FAC5"},
    {"osoba_w_koronie_kolor1", "1FAC5-1F3FB"},
    {"osoba_w_koronie_kolor2", "1FAC5-1F3FC"},
    {"osoba_w_koronie_kolor3", "1FAC5-1F3FD"},
    {"osoba_w_koronie_kolor4", "1FAC5-1F3FE"},
    {"osoba_w_koronie_kolor5", "1FAC5-1F3FF"},
    {"odcisk_palca", "1FAC6"},
    {"wlochate_stworzenie", "1FAC8"},
    {"orka", "1FACD"},
    {"los", "1FACE"},
    {"osiol", "1FACF"},
    {"borowka_amerykanska", "1FAD0"},
    {"papryka", "1FAD1"},
    {"oliwka", "1FAD2"},
    {"pita", "1FAD3"},
    {"tamales", "1FAD4"},
    {"fondue", "1FAD5"},
    {"czajniczek", "1FAD6"},
    {"nalewanie_plynu", "1FAD7"},
    {"fasola", "1FAD8"},
    {"sloik", "1FAD9"},
    {"korzen_imbiru", "1FADA"},
    {"strak_grochu", "1FADB"},
    {"warzywo_korzeniowe", "1FADC"},
    {"rozbryzg", "1FADF"},
    {"roztapiajaca_sie_twarz", "1FAE0"},
    {"salutujaca_twarz", "1FAE1"},
    {"twarz_z_otwartymi_oczami_i_reka_na_ustach", "1FAE2"},
    {"twarz_z_podgladajacym_okiem", "1FAE3"},
    {"twarz_z_ukosnymi_ustami", "1FAE4"},
    {"twarz_otoczona_linia_przerywana", "1FAE5"},
    {"przygryzanie_wargi", "1FAE6"},
    {"babelki", "1FAE7"},
    {"wstrzasnieta_twarz", "1FAE8"},
    {"twarz_z_workami_pod_oczami", "1FAE9"},
    {"wykrzywiona_twarz", "1FAEA"},
    {"chmura_walki", "1FAEF"},
    {"dlon_ze_skrzyzowanym_palcem_wskazujacym_i_kciukiem", "1FAF0"},
    {"dlon_ze_skrzyzowanym_palcem_wskazujacym_i_kciukiem_kolor1", "1FAF0-1F3FB"},
    {"dlon_ze_skrzyzowanym_palcem_wskazujacym_i_kciukiem_kolor2", "1FAF0-1F3FC"},
    {"dlon_ze_skrzyzowanym_palcem_wskazujacym_i_kciukiem_kolor3", "1FAF0-1F3FD"},
    {"dlon_ze_skrzyzowanym_palcem_wskazujacym_i_kciukiem_kolor4", "1FAF0-1F3FE"},
    {"dlon_ze_skrzyzowanym_palcem_wskazujacym_i_kciukiem_kolor5", "1FAF0-1F3FF"},
    {"dlon_skierowana_w_prawo", "1FAF1"},
    {"dlon_skierowana_w_prawo_kolor1", "1FAF1-1F3FB"},
    {"dlon_skierowana_w_prawo_kolor2", "1FAF1-1F3FC"},
    {"dlon_skierowana_w_prawo_kolor3", "1FAF1-1F3FD"},
    {"dlon_skierowana_w_prawo_kolor4", "1FAF1-1F3FE"},
    {"dlon_skierowana_w_prawo_kolor5", "1FAF1-1F3FF"},
    {"dlon_skierowana_w_lewo", "1FAF2"},
    {"dlon_skierowana_w_lewo_kolor1", "1FAF2-1F3FB"},
    {"dlon_skierowana_w_lewo_kolor2", "1FAF2-1F3FC"},
    {"dlon_skierowana_w_lewo_kolor3", "1FAF2-1F3FD"},
    {"dlon_skierowana_w_lewo_kolor4", "1FAF2-1F3FE"},
    {"dlon_skierowana_w_lewo_kolor5", "1FAF2-1F3FF"},
    {"dlon_skierowana_w_dol", "1FAF3"},
    {"dlon_skierowana_w_dol_kolor1", "1FAF3-1F3FB"},
    {"dlon_skierowana_w_dol_kolor2", "1FAF3-1F3FC"},
    {"dlon_skierowana_w_dol_kolor3", "1FAF3-1F3FD"},
    {"dlon_skierowana_w_dol_kolor4", "1FAF3-1F3FE"},
    {"dlon_skierowana_w_dol_kolor5", "1FAF3-1F3FF"},
    {"dlon_skierowana_w_gore", "1FAF4"},
    {"dlon_skierowana_w_gore_kolor1", "1FAF4-1F3FB"},
    {"dlon_skierowana_w_gore_kolor2", "1FAF4-1F3FC"},
    {"dlon_skierowana_w_gore_kolor3", "1FAF4-1F3FD"},
    {"dlon_skierowana_w_gore_kolor4", "1FAF4-1F3FE"},
    {"dlon_skierowana_w_gore_kolor5", "1FAF4-1F3FF"},
    {"palec_wskazujacy_skierowany_na_patrzacego", "1FAF5"},
    {"palec_wskazujacy_skierowany_na_patrzacego_kolor1", "1FAF5-1F3FB"},
    {"palec_wskazujacy_skierowany_na_patrzacego_kolor2", "1FAF5-1F3FC"},
    {"palec_wskazujacy_skierowany_na_patrzacego_kolor3", "1FAF5-1F3FD"},
    {"palec_wskazujacy_skierowany_na_patrzacego_kolor4", "1FAF5-1F3FE"},
    {"palec_wskazujacy_skierowany_na_patrzacego_kolor5", "1FAF5-1F3FF"},
    {"dlonie_tworzace_serce", "1FAF6"},
    {"dlonie_tworzace_serce_kolor1", "1FAF6-1F3FB"},
    {"dlonie_tworzace_serce_kolor2", "1FAF6-1F3FC"},
    {"dlonie_tworzace_serce_kolor3", "1FAF6-1F3FD"},
    {"dlonie_tworzace_serce_kolor4", "1FAF6-1F3FE"},
    {"dlonie_tworzace_serce_kolor5", "1FAF6-1F3FF"},
    {"dlon_pchajaca_w_lewo", "1FAF7"},
    {"dlon_pchajaca_w_lewo_kolor1", "1FAF7-1F3FB"},
    {"dlon_pchajaca_w_lewo_kolor2", "1FAF7-1F3FC"},
    {"dlon_pchajaca_w_lewo_kolor3", "1FAF7-1F3FD"},
    {"dlon_pchajaca_w_lewo_kolor4", "1FAF7-1F3FE"},
    {"dlon_pchajaca_w_lewo_kolor5", "1FAF7-1F3FF"},
    {"dlon_pchajaca_w_prawo", "1FAF8"},
    {"dlon_pchajaca_w_prawo_kolor1", "1FAF8-1F3FB"},
    {"dlon_pchajaca_w_prawo_kolor2", "1FAF8-1F3FC"},
    {"dlon_pchajaca_w_prawo_kolor3", "1FAF8-1F3FD"},
    {"dlon_pchajaca_w_prawo_kolor4", "1FAF8-1F3FE"},
    {"dlon_pchajaca_w_prawo_kolor5", "1FAF8-1F3FF"},
    {"klawisz_znak_liczbowy", "0023-FE0F-20E3"},
    {"klawisz_gwiazdka", "002A-FE0F-20E3"},
    {"klawisz_0", "0030-FE0F-20E3"},
    {"klawisz_1", "0031-FE0F-20E3"},
    {"klawisz_2", "0032-FE0F-20E3"},
    {"klawisz_3", "0033-FE0F-20E3"},
    {"klawisz_4", "0034-FE0F-20E3"},
    {"klawisz_5", "0035-FE0F-20E3"},
    {"klawisz_6", "0036-FE0F-20E3"},
    {"klawisz_7", "0037-FE0F-20E3"},
    {"klawisz_8", "0038-FE0F-20E3"},
    {"klawisz_9", "0039-FE0F-20E3"},
    {"flaga_wyspa_wniebowstapienia", "1F1E6-1F1E8"},
    {"flaga_andora", "1F1E6-1F1E9"},
    {"flaga_zjednoczone_emiraty_arabskie", "1F1E6-1F1EA"},
    {"flaga_afganistan", "1F1E6-1F1EB"},
    {"flaga_antigua_i_barbuda", "1F1E6-1F1EC"},
    {"flaga_anguilla", "1F1E6-1F1EE"},
    {"flaga_albania", "1F1E6-1F1F1"},
    {"flaga_armenia", "1F1E6-1F1F2"},
    {"flaga_angola", "1F1E6-1F1F4"},
    {"flaga_antarktyda", "1F1E6-1F1F6"},
    {"flaga_argentyna", "1F1E6-1F1F7"},
    {"flaga_samoa_amerykanskie", "1F1E6-1F1F8"},
    {"flaga_austria", "1F1E6-1F1F9"},
    {"flaga_australia", "1F1E6-1F1FA"},
    {"flaga_aruba", "1F1E6-1F1FC"},
    {"flaga_wyspy_alandzkie", "1F1E6-1F1FD"},
    {"flaga_azerbejdzan", "1F1E6-1F1FF"},
    {"flaga_bosnia_i_hercegowina", "1F1E7-1F1E6"},
    {"flaga_barbados", "1F1E7-1F1E7"},
    {"flaga_bangladesz", "1F1E7-1F1E9"},
    {"flaga_belgia", "1F1E7-1F1EA"},
    {"flaga_burkina_faso", "1F1E7-1F1EB"},
    {"flaga_bulgaria", "1F1E7-1F1EC"},
    {"flaga_bahrajn", "1F1E7-1F1ED"},
    {"flaga_burundi", "1F1E7-1F1EE"},
    {"flaga_benin", "1F1E7-1F1EF"},
    {"flaga_saint_barthelemy", "1F1E7-1F1F1"},
    {"flaga_bermudy", "1F1E7-1F1F2"},
    {"flaga_brunei", "1F1E7-1F1F3"},
    {"flaga_boliwia", "1F1E7-1F1F4"},
    {"flaga_niderlandy_karaibskie", "1F1E7-1F1F6"},
    {"flaga_brazylia", "1F1E7-1F1F7"},
    {"flaga_bahamy", "1F1E7-1F1F8"},
    {"flaga_bhutan", "1F1E7-1F1F9"},
    {"flaga_wyspa_bouveta", "1F1E7-1F1FB"},
    {"flaga_botswana", "1F1E7-1F1FC"},
    {"flaga_bialorus", "1F1E7-1F1FE"},
    {"flaga_belize", "1F1E7-1F1FF"},
    {"flaga_kanada", "1F1E8-1F1E6"},
    {"flaga_wyspy_kokosowe", "1F1E8-1F1E8"},
    {"flaga_demokratyczna_republika_konga", "1F1E8-1F1E9"},
    {"flaga_republika_srodkowoafrykanska", "1F1E8-1F1EB"},
    {"flaga_kongo", "1F1E8-1F1EC"},
    {"flaga_szwajcaria", "1F1E8-1F1ED"},
    {"flaga_cote_divoire", "1F1E8-1F1EE"},
    {"flaga_wyspy_cooka", "1F1E8-1F1F0"},
    {"flaga_chile", "1F1E8-1F1F1"},
    {"flaga_kamerun", "1F1E8-1F1F2"},
    {"flaga_chiny", "1F1E8-1F1F3"},
    {"flaga_kolumbia", "1F1E8-1F1F4"},
    {"flaga_wyspa_clippertona", "1F1E8-1F1F5"},
    {"flaga_sark", "1F1E8-1F1F6"},
    {"flaga_kostaryka", "1F1E8-1F1F7"},
    {"flaga_kuba", "1F1E8-1F1FA"},
    {"flaga_republika_zielonego_przyladka", "1F1E8-1F1FB"},
    {"flaga_curacao", "1F1E8-1F1FC"},
    {"flaga_wyspa_bozego_narodzenia", "1F1E8-1F1FD"},
    {"flaga_cypr", "1F1E8-1F1FE"},
    {"flaga_czechy", "1F1E8-1F1FF"},
    {"flaga_niemcy", "1F1E9-1F1EA"},
    {"flaga_diego_garcia", "1F1E9-1F1EC"},
    {"flaga_dzibuti", "1F1E9-1F1EF"},
    {"flaga_dania", "1F1E9-1F1F0"},
    {"flaga_dominika", "1F1E9-1F1F2"},
    {"flaga_dominikana", "1F1E9-1F1F4"},
    {"flaga_algieria", "1F1E9-1F1FF"},
    {"flaga_ceuta_i_melilla", "1F1EA-1F1E6"},
    {"flaga_ekwador", "1F1EA-1F1E8"},
    {"flaga_estonia", "1F1EA-1F1EA"},
    {"flaga_egipt", "1F1EA-1F1EC"},
    {"flaga_sahara_zachodnia", "1F1EA-1F1ED"},
    {"flaga_erytrea", "1F1EA-1F1F7"},
    {"flaga_hiszpania", "1F1EA-1F1F8"},
    {"flaga_etiopia", "1F1EA-1F1F9"},
    {"flaga_unia_europejska", "1F1EA-1F1FA"},
    {"flaga_finlandia", "1F1EB-1F1EE"},
    {"flaga_fidzi", "1F1EB-1F1EF"},
    {"flaga_falklandy", "1F1EB-1F1F0"},
    {"flaga_mikronezja", "1F1EB-1F1F2"},
    {"flaga_wyspy_owcze", "1F1EB-1F1F4"},
    {"flaga_francja", "1F1EB-1F1F7"},
    {"flaga_gabon", "1F1EC-1F1E6"},
    {"flaga_wielka_brytania", "1F1EC-1F1E7"},
    {"flaga_grenada", "1F1EC-1F1E9"},
    {"flaga_gruzja", "1F1EC-1F1EA"},
    {"flaga_gujana_francuska", "1F1EC-1F1EB"},
    {"flaga_guernsey", "1F1EC-1F1EC"},
    {"flaga_ghana", "1F1EC-1F1ED"},
    {"flaga_gibraltar", "1F1EC-1F1EE"},
    {"flaga_grenlandia", "1F1EC-1F1F1"},
    {"flaga_gambia", "1F1EC-1F1F2"},
    {"flaga_gwinea", "1F1EC-1F1F3"},
    {"flaga_gwadelupa", "1F1EC-1F1F5"},
    {"flaga_gwinea_rownikowa", "1F1EC-1F1F6"},
    {"flaga_grecja", "1F1EC-1F1F7"},
    {"flaga_georgia_poludniowa_i_sandwich_poludniowy", "1F1EC-1F1F8"},
    {"flaga_gwatemala", "1F1EC-1F1F9"},
    {"flaga_guam", "1F1EC-1F1FA"},
    {"flaga_gwinea_bissau", "1F1EC-1F1FC"},
    {"flaga_gujana", "1F1EC-1F1FE"},
    {"flaga_sra_hongkong_chiny", "1F1ED-1F1F0"},
    {"flaga_wyspy_heard_i_mcdonalda", "1F1ED-1F1F2"},
    {"flaga_honduras", "1F1ED-1F1F3"},
    {"flaga_chorwacja", "1F1ED-1F1F7"},
    {"flaga_haiti", "1F1ED-1F1F9"},
    {"flaga_wegry", "1F1ED-1F1FA"},
    {"flaga_wyspy_kanaryjskie", "1F1EE-1F1E8"},
    {"flaga_indonezja", "1F1EE-1F1E9"},
    {"flaga_irlandia", "1F1EE-1F1EA"},
    {"flaga_izrael", "1F1EE-1F1F1"},
    {"flaga_wyspa_man", "1F1EE-1F1F2"},
    {"flaga_indie", "1F1EE-1F1F3"},
    {"flaga_brytyjskie_terytorium_oceanu_indyjskiego", "1F1EE-1F1F4"},
    {"flaga_irak", "1F1EE-1F1F6"},
    {"flaga_iran", "1F1EE-1F1F7"},
    {"flaga_islandia", "1F1EE-1F1F8"},
    {"flaga_wlochy", "1F1EE-1F1F9"},
    {"flaga_jersey", "1F1EF-1F1EA"},
    {"flaga_jamajka", "1F1EF-1F1F2"},
    {"flaga_jordania", "1F1EF-1F1F4"},
    {"flaga_japonia", "1F1EF-1F1F5"},
    {"flaga_kenia", "1F1F0-1F1EA"},
    {"flaga_kirgistan", "1F1F0-1F1EC"},
    {"flaga_kambodza", "1F1F0-1F1ED"},
    {"flaga_kiribati", "1F1F0-1F1EE"},
    {"flaga_komory", "1F1F0-1F1F2"},
    {"flaga_saint_kitts_i_nevis", "1F1F0-1F1F3"},
    {"flaga_korea_polnocna", "1F1F0-1F1F5"},
    {"flaga_korea_poludniowa", "1F1F0-1F1F7"},
    {"flaga_kuwejt", "1F1F0-1F1FC"},
    {"flaga_kajmany", "1F1F0-1F1FE"},
    {"flaga_kazachstan", "1F1F0-1F1FF"},
    {"flaga_laos", "1F1F1-1F1E6"},
    {"flaga_liban", "1F1F1-1F1E7"},
    {"flaga_saint_lucia", "1F1F1-1F1E8"},
    {"flaga_liechtenstein", "1F1F1-1F1EE"},
    {"flaga_sri_lanka", "1F1F1-1F1F0"},
    {"flaga_liberia", "1F1F1-1F1F7"},
    {"flaga_lesotho", "1F1F1-1F1F8"},
    {"flaga_litwa", "1F1F1-1F1F9"},
    {"flaga_luksemburg", "1F1F1-1F1FA"},
    {"flaga_lotwa", "1F1F1-1F1FB"},
    {"flaga_libia", "1F1F1-1F1FE"},
    {"flaga_maroko", "1F1F2-1F1E6"},
    {"flaga_monako", "1F1F2-1F1E8"},
    {"flaga_moldawia", "1F1F2-1F1E9"},
    {"flaga_czarnogora", "1F1F2-1F1EA"},
    {"flaga_saint_martin", "1F1F2-1F1EB"},
    {"flaga_madagaskar", "1F1F2-1F1EC"},
    {"flaga_wyspy_marshalla", "1F1F2-1F1ED"},
    {"flaga_macedonia_polnocna", "1F1F2-1F1F0"},
    {"flaga_mali", "1F1F2-1F1F1"},
    {"flaga_mjanma_birma", "1F1F2-1F1F2"},
    {"flaga_mongolia", "1F1F2-1F1F3"},
    {"flaga_sra_makau_chiny", "1F1F2-1F1F4"},
    {"flaga_mariany_polnocne", "1F1F2-1F1F5"},
    {"flaga_martynika", "1F1F2-1F1F6"},
    {"flaga_mauretania", "1F1F2-1F1F7"},
    {"flaga_montserrat", "1F1F2-1F1F8"},
    {"flaga_malta", "1F1F2-1F1F9"},
    {"flaga_mauritius", "1F1F2-1F1FA"},
    {"flaga_malediwy", "1F1F2-1F1FB"},
    {"flaga_malawi", "1F1F2-1F1FC"},
    {"flaga_meksyk", "1F1F2-1F1FD"},
    {"flaga_malezja", "1F1F2-1F1FE"},
    {"flaga_mozambik", "1F1F2-1F1FF"},
    {"flaga_namibia", "1F1F3-1F1E6"},
    {"flaga_nowa_kaledonia", "1F1F3-1F1E8"},
    {"flaga_niger", "1F1F3-1F1EA"},
    {"flaga_norfolk", "1F1F3-1F1EB"},
    {"flaga_nigeria", "1F1F3-1F1EC"},
    {"flaga_nikaragua", "1F1F3-1F1EE"},
    {"flaga_holandia", "1F1F3-1F1F1"},
    {"flaga_norwegia", "1F1F3-1F1F4"},
    {"flaga_nepal", "1F1F3-1F1F5"},
    {"flaga_nauru", "1F1F3-1F1F7"},
    {"flaga_niue", "1F1F3-1F1FA"},
    {"flaga_nowa_zelandia", "1F1F3-1F1FF"},
    {"flaga_oman", "1F1F4-1F1F2"},
    {"flaga_panama", "1F1F5-1F1E6"},
    {"flaga_peru", "1F1F5-1F1EA"},
    {"flaga_polinezja_francuska", "1F1F5-1F1EB"},
    {"flaga_papua_nowa_gwinea", "1F1F5-1F1EC"},
    {"flaga_filipiny", "1F1F5-1F1ED"},
    {"flaga_pakistan", "1F1F5-1F1F0"},
    {"flaga_polska", "1F1F5-1F1F1"},
    {"flaga_saint_pierre_i_miquelon", "1F1F5-1F1F2"},
    {"flaga_pitcairn", "1F1F5-1F1F3"},
    {"flaga_portoryko", "1F1F5-1F1F7"},
    {"flaga_terytoria_palestynskie", "1F1F5-1F1F8"},
    {"flaga_portugalia", "1F1F5-1F1F9"},
    {"flaga_palau", "1F1F5-1F1FC"},
    {"flaga_paragwaj", "1F1F5-1F1FE"},
    {"flaga_katar", "1F1F6-1F1E6"},
    {"flaga_reunion", "1F1F7-1F1EA"},
    {"flaga_rumunia", "1F1F7-1F1F4"},
    {"flaga_serbia", "1F1F7-1F1F8"},
    {"flaga_rosja", "1F1F7-1F1FA"},
    {"flaga_rwanda", "1F1F7-1F1FC"},
    {"flaga_arabia_saudyjska", "1F1F8-1F1E6"},
    {"flaga_wyspy_salomona", "1F1F8-1F1E7"},
    {"flaga_seszele", "1F1F8-1F1E8"},
    {"flaga_sudan", "1F1F8-1F1E9"},
    {"flaga_szwecja", "1F1F8-1F1EA"},
    {"flaga_singapur", "1F1F8-1F1EC"},
    {"flaga_wyspa_swietej_heleny", "1F1F8-1F1ED"},
    {"flaga_slowenia", "1F1F8-1F1EE"},
    {"flaga_svalbard_i_jan_mayen", "1F1F8-1F1EF"},
    {"flaga_slowacja", "1F1F8-1F1F0"},
    {"flaga_sierra_leone", "1F1F8-1F1F1"},
    {"flaga_san_marino", "1F1F8-1F1F2"},
    {"flaga_senegal", "1F1F8-1F1F3"},
    {"flaga_somalia", "1F1F8-1F1F4"},
    {"flaga_surinam", "1F1F8-1F1F7"},
    {"flaga_sudan_poludniowy", "1F1F8-1F1F8"},
    {"flaga_wyspy_swietego_tomasza_i_ksiazeca", "1F1F8-1F1F9"},
    {"flaga_salwador", "1F1F8-1F1FB"},
    {"flaga_sint_maarten", "1F1F8-1F1FD"},
    {"flaga_syria", "1F1F8-1F1FE"},
    {"flaga_eswatini", "1F1F8-1F1FF"},
    {"flaga_tristan_da_cunha", "1F1F9-1F1E6"},
    {"flaga_turks_i_caicos", "1F1F9-1F1E8"},
    {"flaga_czad", "1F1F9-1F1E9"},
    {"flaga_francuskie_terytoria_poludniowe_i_antarktyczne", "1F1F9-1F1EB"},
    {"flaga_togo", "1F1F9-1F1EC"},
    {"flaga_tajlandia", "1F1F9-1F1ED"},
    {"flaga_tadzykistan", "1F1F9-1F1EF"},
    {"flaga_tokelau", "1F1F9-1F1F0"},
    {"flaga_timor_wschodni", "1F1F9-1F1F1"},
    {"flaga_turkmenistan", "1F1F9-1F1F2"},
    {"flaga_tunezja", "1F1F9-1F1F3"},
    {"flaga_tonga", "1F1F9-1F1F4"},
    {"flaga_turcja", "1F1F9-1F1F7"},
    {"flaga_trynidad_i_tobago", "1F1F9-1F1F9"},
    {"flaga_tuvalu", "1F1F9-1F1FB"},
    {"flaga_tajwan", "1F1F9-1F1FC"},
    {"flaga_tanzania", "1F1F9-1F1FF"},
    {"flaga_ukraina", "1F1FA-1F1E6"},
    {"flaga_uganda", "1F1FA-1F1EC"},
    {"flaga_dalekie_wyspy_mniejsze_stanow_zjednoczonych", "1F1FA-1F1F2"},
    {"flaga_organizacja_narodow_zjednoczonych", "1F1FA-1F1F3"},
    {"flaga_stany_zjednoczone", "1F1FA-1F1F8"},
    {"flaga_urugwaj", "1F1FA-1F1FE"},
    {"flaga_uzbekistan", "1F1FA-1F1FF"},
    {"flaga_watykan", "1F1FB-1F1E6"},
    {"flaga_saint_vincent_i_grenadyny", "1F1FB-1F1E8"},
    {"flaga_wenezuela", "1F1FB-1F1EA"},
    {"flaga_brytyjskie_wyspy_dziewicze", "1F1FB-1F1EC"},
    {"flaga_wyspy_dziewicze_stanow_zjednoczonych", "1F1FB-1F1EE"},
    {"flaga_wietnam", "1F1FB-1F1F3"},
    {"flaga_vanuatu", "1F1FB-1F1FA"},
    {"flaga_wallis_i_futuna", "1F1FC-1F1EB"},
    {"flaga_samoa", "1F1FC-1F1F8"},
    {"flaga_kosowo", "1F1FD-1F1F0"},
    {"flaga_jemen", "1F1FE-1F1EA"},
    {"flaga_majotta", "1F1FE-1F1F9"},
    {"flaga_republika_poludniowej_afryki", "1F1FF-1F1E6"},
    {"flaga_zambia", "1F1FF-1F1F2"},
    {"flaga_zimbabwe", "1F1FF-1F1FC"},
    {"flaga_anglia", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"flaga_szkocja", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"flaga_walia", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"para_z_sercem_mezczyzna_i_mezczyzna", "1F468-200D-2764-FE0F-200D-1F468"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_mezczyzna_i_mezczyzna_kolor5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pocalunek_mezczyzna_i_mezczyzna", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocalunek_mezczyzna_i_mezczyzna_kolor5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"rodzina_mezczyzna_i_chlopiec", "1F468-200D-1F466"},
    {"rodzina_mezczyzna_chlopiec_i_chlopiec", "1F468-200D-1F466-200D-1F466"},
    {"rodzina_mezczyzna_i_dziewczynka", "1F468-200D-1F467"},
    {"rodzina_mezczyzna_dziewczynka_i_chlopiec", "1F468-200D-1F467-200D-1F466"},
    {"rodzina_mezczyzna_dziewczynka_i_dziewczynka", "1F468-200D-1F467-200D-1F467"},
    {"rodzina_mezczyzna_mezczyzna_i_chlopiec", "1F468-200D-1F468-200D-1F466"},
    {"rodzina_mezczyzna_mezczyzna_chlopiec_i_chlopiec", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"rodzina_mezczyzna_mezczyzna_i_dziewczynka", "1F468-200D-1F468-200D-1F467"},
    {"rodzina_mezczyzna_mezczyzna_dziewczynka_i_chlopiec", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"rodzina_mezczyzna_mezczyzna_dziewczynka_i_dziewczynka", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"rodzina_mezczyzna_kobieta_i_chlopiec", "1F468-200D-1F469-200D-1F466"},
    {"rodzina_mezczyzna_kobieta_chlopiec_i_chlopiec", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"rodzina_mezczyzna_kobieta_i_dziewczynka", "1F468-200D-1F469-200D-1F467"},
    {"rodzina_mezczyzna_kobieta_dziewczynka_i_chlopiec", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"rodzina_mezczyzna_kobieta_dziewczynka_i_dziewczynka", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"para_z_sercem_kobieta_i_mezczyzna", "1F469-200D-2764-FE0F-200D-1F468"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_kobieta_i_mezczyzna_kolor5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_kobieta_i_kobieta", "1F469-200D-2764-FE0F-200D-1F469"},
    {"para_z_sercem_kobieta_i_kobieta_kolor1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"para_z_sercem_kobieta_i_kobieta_kolor1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"para_z_sercem_kobieta_i_kobieta_kolor1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"para_z_sercem_kobieta_i_kobieta_kolor1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"para_z_sercem_kobieta_i_kobieta_kolor1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"para_z_sercem_kobieta_i_kobieta_kolor2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"para_z_sercem_kobieta_i_kobieta_kolor2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"para_z_sercem_kobieta_i_kobieta_kolor2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"para_z_sercem_kobieta_i_kobieta_kolor2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"para_z_sercem_kobieta_i_kobieta_kolor2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"para_z_sercem_kobieta_i_kobieta_kolor3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"para_z_sercem_kobieta_i_kobieta_kolor3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"para_z_sercem_kobieta_i_kobieta_kolor3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"para_z_sercem_kobieta_i_kobieta_kolor3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"para_z_sercem_kobieta_i_kobieta_kolor3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"para_z_sercem_kobieta_i_kobieta_kolor4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"para_z_sercem_kobieta_i_kobieta_kolor4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"para_z_sercem_kobieta_i_kobieta_kolor4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"para_z_sercem_kobieta_i_kobieta_kolor4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"para_z_sercem_kobieta_i_kobieta_kolor4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"para_z_sercem_kobieta_i_kobieta_kolor5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"para_z_sercem_kobieta_i_kobieta_kolor5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"para_z_sercem_kobieta_i_kobieta_kolor5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"para_z_sercem_kobieta_i_kobieta_kolor5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"para_z_sercem_kobieta_i_kobieta_kolor5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pocalunek_kobieta_i_mezczyzna", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"pocalunek_kobieta_i_mezczyzna_kolor1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocalunek_kobieta_i_mezczyzna_kolor1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocalunek_kobieta_i_mezczyzna_kolor1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocalunek_kobieta_i_mezczyzna_kolor1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocalunek_kobieta_i_mezczyzna_kolor1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocalunek_kobieta_i_mezczyzna_kolor2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocalunek_kobieta_i_mezczyzna_kolor2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocalunek_kobieta_i_mezczyzna_kolor2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocalunek_kobieta_i_mezczyzna_kolor2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocalunek_kobieta_i_mezczyzna_kolor2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocalunek_kobieta_i_mezczyzna_kolor3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocalunek_kobieta_i_mezczyzna_kolor3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocalunek_kobieta_i_mezczyzna_kolor3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocalunek_kobieta_i_mezczyzna_kolor3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocalunek_kobieta_i_mezczyzna_kolor3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocalunek_kobieta_i_mezczyzna_kolor4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocalunek_kobieta_i_mezczyzna_kolor4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocalunek_kobieta_i_mezczyzna_kolor4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocalunek_kobieta_i_mezczyzna_kolor4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocalunek_kobieta_i_mezczyzna_kolor4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocalunek_kobieta_i_mezczyzna_kolor5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocalunek_kobieta_i_mezczyzna_kolor5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocalunek_kobieta_i_mezczyzna_kolor5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocalunek_kobieta_i_mezczyzna_kolor5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocalunek_kobieta_i_mezczyzna_kolor5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocalunek_kobieta_i_kobieta", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"pocalunek_kobieta_i_kobieta_kolor1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pocalunek_kobieta_i_kobieta_kolor1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pocalunek_kobieta_i_kobieta_kolor1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pocalunek_kobieta_i_kobieta_kolor1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pocalunek_kobieta_i_kobieta_kolor1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"pocalunek_kobieta_i_kobieta_kolor2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pocalunek_kobieta_i_kobieta_kolor2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pocalunek_kobieta_i_kobieta_kolor2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pocalunek_kobieta_i_kobieta_kolor2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pocalunek_kobieta_i_kobieta_kolor2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"pocalunek_kobieta_i_kobieta_kolor3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pocalunek_kobieta_i_kobieta_kolor3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pocalunek_kobieta_i_kobieta_kolor3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pocalunek_kobieta_i_kobieta_kolor3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pocalunek_kobieta_i_kobieta_kolor3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"pocalunek_kobieta_i_kobieta_kolor4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pocalunek_kobieta_i_kobieta_kolor4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pocalunek_kobieta_i_kobieta_kolor4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pocalunek_kobieta_i_kobieta_kolor4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pocalunek_kobieta_i_kobieta_kolor4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"pocalunek_kobieta_i_kobieta_kolor5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pocalunek_kobieta_i_kobieta_kolor5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pocalunek_kobieta_i_kobieta_kolor5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pocalunek_kobieta_i_kobieta_kolor5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pocalunek_kobieta_i_kobieta_kolor5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"rodzina_kobieta_i_chlopiec", "1F469-200D-1F466"},
    {"rodzina_kobieta_chlopiec_i_chlopiec", "1F469-200D-1F466-200D-1F466"},
    {"rodzina_kobieta_i_dziewczynka", "1F469-200D-1F467"},
    {"rodzina_kobieta_dziewczynka_i_chlopiec", "1F469-200D-1F467-200D-1F466"},
    {"rodzina_kobieta_dziewczynka_i_dziewczynka", "1F469-200D-1F467-200D-1F467"},
    {"rodzina_kobieta_kobieta_i_chlopiec", "1F469-200D-1F469-200D-1F466"},
    {"rodzina_kobieta_kobieta_chlopiec_i_chlopiec", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"rodzina_kobieta_kobieta_i_dziewczynka", "1F469-200D-1F469-200D-1F467"},
    {"rodzina_kobieta_kobieta_dziewczynka_i_chlopiec", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"rodzina_kobieta_kobieta_dziewczynka_i_dziewczynka", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"osoby_trzymajace_sie_za_rece", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"osoby_trzymajace_sie_za_rece_kolor1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"osoby_trzymajace_sie_za_rece_kolor1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"osoby_trzymajace_sie_za_rece_kolor1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"osoby_trzymajace_sie_za_rece_kolor1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"osoby_trzymajace_sie_za_rece_kolor1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"osoby_trzymajace_sie_za_rece_kolor2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"osoby_trzymajace_sie_za_rece_kolor2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"osoby_trzymajace_sie_za_rece_kolor2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"osoby_trzymajace_sie_za_rece_kolor2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"osoby_trzymajace_sie_za_rece_kolor2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"osoby_trzymajace_sie_za_rece_kolor3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"osoby_trzymajace_sie_za_rece_kolor3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"osoby_trzymajace_sie_za_rece_kolor3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"osoby_trzymajace_sie_za_rece_kolor3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"osoby_trzymajace_sie_za_rece_kolor3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"osoby_trzymajace_sie_za_rece_kolor4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"osoby_trzymajace_sie_za_rece_kolor4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"osoby_trzymajace_sie_za_rece_kolor4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"osoby_trzymajace_sie_za_rece_kolor4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"osoby_trzymajace_sie_za_rece_kolor4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"osoby_trzymajace_sie_za_rece_kolor5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"osoby_trzymajace_sie_za_rece_kolor5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"osoby_trzymajace_sie_za_rece_kolor5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"osoby_trzymajace_sie_za_rece_kolor5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"osoby_trzymajace_sie_za_rece_kolor5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"rodzina_dorosly_dorosly_dziecko", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"rodzina_dorosly_dorosly_dziecko_dziecko", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"rodzina_dorosly_dziecko", "1F9D1-200D-1F9D2"},
    {"rodzina_dorosly_dziecko_dziecko", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"biegnaca_osoba_zwrocony_w_prawo", "1F3C3-200D-27A1-FE0F"},
    {"biegnaca_osoba_zwrocony_w_prawo_kolor1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"biegnaca_osoba_zwrocony_w_prawo_kolor2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"biegnaca_osoba_zwrocony_w_prawo_kolor3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"biegnaca_osoba_zwrocony_w_prawo_kolor4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"biegnaca_osoba_zwrocony_w_prawo_kolor5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"lekarz", "1F468-200D-2695-FE0F"},
    {"lekarz_kolor1", "1F468-1F3FB-200D-2695-FE0F"},
    {"lekarz_kolor2", "1F468-1F3FC-200D-2695-FE0F"},
    {"lekarz_kolor3", "1F468-1F3FD-200D-2695-FE0F"},
    {"lekarz_kolor4", "1F468-1F3FE-200D-2695-FE0F"},
    {"lekarz_kolor5", "1F468-1F3FF-200D-2695-FE0F"},
    {"mezczyzna_sedzia", "1F468-200D-2696-FE0F"},
    {"mezczyzna_sedzia_kolor1", "1F468-1F3FB-200D-2696-FE0F"},
    {"mezczyzna_sedzia_kolor2", "1F468-1F3FC-200D-2696-FE0F"},
    {"mezczyzna_sedzia_kolor3", "1F468-1F3FD-200D-2696-FE0F"},
    {"mezczyzna_sedzia_kolor4", "1F468-1F3FE-200D-2696-FE0F"},
    {"mezczyzna_sedzia_kolor5", "1F468-1F3FF-200D-2696-FE0F"},
    {"mezczyzna_pilot", "1F468-200D-2708-FE0F"},
    {"mezczyzna_pilot_kolor1", "1F468-1F3FB-200D-2708-FE0F"},
    {"mezczyzna_pilot_kolor2", "1F468-1F3FC-200D-2708-FE0F"},
    {"mezczyzna_pilot_kolor3", "1F468-1F3FD-200D-2708-FE0F"},
    {"mezczyzna_pilot_kolor4", "1F468-1F3FE-200D-2708-FE0F"},
    {"mezczyzna_pilot_kolor5", "1F468-1F3FF-200D-2708-FE0F"},
    {"mezczyzna_rolnik", "1F468-200D-1F33E"},
    {"mezczyzna_rolnik_kolor1", "1F468-1F3FB-200D-1F33E"},
    {"mezczyzna_rolnik_kolor2", "1F468-1F3FC-200D-1F33E"},
    {"mezczyzna_rolnik_kolor3", "1F468-1F3FD-200D-1F33E"},
    {"mezczyzna_rolnik_kolor4", "1F468-1F3FE-200D-1F33E"},
    {"mezczyzna_rolnik_kolor5", "1F468-1F3FF-200D-1F33E"},
    {"mezczyzna_kucharz", "1F468-200D-1F373"},
    {"mezczyzna_kucharz_kolor1", "1F468-1F3FB-200D-1F373"},
    {"mezczyzna_kucharz_kolor2", "1F468-1F3FC-200D-1F373"},
    {"mezczyzna_kucharz_kolor3", "1F468-1F3FD-200D-1F373"},
    {"mezczyzna_kucharz_kolor4", "1F468-1F3FE-200D-1F373"},
    {"mezczyzna_kucharz_kolor5", "1F468-1F3FF-200D-1F373"},
    {"mezczyzna_karmiacy_niemowle", "1F468-200D-1F37C"},
    {"mezczyzna_karmiacy_niemowle_kolor1", "1F468-1F3FB-200D-1F37C"},
    {"mezczyzna_karmiacy_niemowle_kolor2", "1F468-1F3FC-200D-1F37C"},
    {"mezczyzna_karmiacy_niemowle_kolor3", "1F468-1F3FD-200D-1F37C"},
    {"mezczyzna_karmiacy_niemowle_kolor4", "1F468-1F3FE-200D-1F37C"},
    {"mezczyzna_karmiacy_niemowle_kolor5", "1F468-1F3FF-200D-1F37C"},
    {"mezczyzna_student", "1F468-200D-1F393"},
    {"mezczyzna_student_kolor1", "1F468-1F3FB-200D-1F393"},
    {"mezczyzna_student_kolor2", "1F468-1F3FC-200D-1F393"},
    {"mezczyzna_student_kolor3", "1F468-1F3FD-200D-1F393"},
    {"mezczyzna_student_kolor4", "1F468-1F3FE-200D-1F393"},
    {"mezczyzna_student_kolor5", "1F468-1F3FF-200D-1F393"},
    {"mezczyzna_piosenkarz", "1F468-200D-1F3A4"},
    {"mezczyzna_piosenkarz_kolor1", "1F468-1F3FB-200D-1F3A4"},
    {"mezczyzna_piosenkarz_kolor2", "1F468-1F3FC-200D-1F3A4"},
    {"mezczyzna_piosenkarz_kolor3", "1F468-1F3FD-200D-1F3A4"},
    {"mezczyzna_piosenkarz_kolor4", "1F468-1F3FE-200D-1F3A4"},
    {"mezczyzna_piosenkarz_kolor5", "1F468-1F3FF-200D-1F3A4"},
    {"mezczyzna_artysta", "1F468-200D-1F3A8"},
    {"mezczyzna_artysta_kolor1", "1F468-1F3FB-200D-1F3A8"},
    {"mezczyzna_artysta_kolor2", "1F468-1F3FC-200D-1F3A8"},
    {"mezczyzna_artysta_kolor3", "1F468-1F3FD-200D-1F3A8"},
    {"mezczyzna_artysta_kolor4", "1F468-1F3FE-200D-1F3A8"},
    {"mezczyzna_artysta_kolor5", "1F468-1F3FF-200D-1F3A8"},
    {"mezczyzna_nauczyciel", "1F468-200D-1F3EB"},
    {"mezczyzna_nauczyciel_kolor1", "1F468-1F3FB-200D-1F3EB"},
    {"mezczyzna_nauczyciel_kolor2", "1F468-1F3FC-200D-1F3EB"},
    {"mezczyzna_nauczyciel_kolor3", "1F468-1F3FD-200D-1F3EB"},
    {"mezczyzna_nauczyciel_kolor4", "1F468-1F3FE-200D-1F3EB"},
    {"mezczyzna_nauczyciel_kolor5", "1F468-1F3FF-200D-1F3EB"},
    {"mezczyzna_pracownik_fabryki", "1F468-200D-1F3ED"},
    {"mezczyzna_pracownik_fabryki_kolor1", "1F468-1F3FB-200D-1F3ED"},
    {"mezczyzna_pracownik_fabryki_kolor2", "1F468-1F3FC-200D-1F3ED"},
    {"mezczyzna_pracownik_fabryki_kolor3", "1F468-1F3FD-200D-1F3ED"},
    {"mezczyzna_pracownik_fabryki_kolor4", "1F468-1F3FE-200D-1F3ED"},
    {"mezczyzna_pracownik_fabryki_kolor5", "1F468-1F3FF-200D-1F3ED"},
    {"mezczyzna_technolog", "1F468-200D-1F4BB"},
    {"mezczyzna_technolog_kolor1", "1F468-1F3FB-200D-1F4BB"},
    {"mezczyzna_technolog_kolor2", "1F468-1F3FC-200D-1F4BB"},
    {"mezczyzna_technolog_kolor3", "1F468-1F3FD-200D-1F4BB"},
    {"mezczyzna_technolog_kolor4", "1F468-1F3FE-200D-1F4BB"},
    {"mezczyzna_technolog_kolor5", "1F468-1F3FF-200D-1F4BB"},
    {"mezczyzna_pracownik_biurowy", "1F468-200D-1F4BC"},
    {"mezczyzna_pracownik_biurowy_kolor1", "1F468-1F3FB-200D-1F4BC"},
    {"mezczyzna_pracownik_biurowy_kolor2", "1F468-1F3FC-200D-1F4BC"},
    {"mezczyzna_pracownik_biurowy_kolor3", "1F468-1F3FD-200D-1F4BC"},
    {"mezczyzna_pracownik_biurowy_kolor4", "1F468-1F3FE-200D-1F4BC"},
    {"mezczyzna_pracownik_biurowy_kolor5", "1F468-1F3FF-200D-1F4BC"},
    {"mezczyzna_elektryk", "1F468-200D-1F527"},
    {"mezczyzna_elektryk_kolor1", "1F468-1F3FB-200D-1F527"},
    {"mezczyzna_elektryk_kolor2", "1F468-1F3FC-200D-1F527"},
    {"mezczyzna_elektryk_kolor3", "1F468-1F3FD-200D-1F527"},
    {"mezczyzna_elektryk_kolor4", "1F468-1F3FE-200D-1F527"},
    {"mezczyzna_elektryk_kolor5", "1F468-1F3FF-200D-1F527"},
    {"mezczyzna_naukowiec", "1F468-200D-1F52C"},
    {"mezczyzna_naukowiec_kolor1", "1F468-1F3FB-200D-1F52C"},
    {"mezczyzna_naukowiec_kolor2", "1F468-1F3FC-200D-1F52C"},
    {"mezczyzna_naukowiec_kolor3", "1F468-1F3FD-200D-1F52C"},
    {"mezczyzna_naukowiec_kolor4", "1F468-1F3FE-200D-1F52C"},
    {"mezczyzna_naukowiec_kolor5", "1F468-1F3FF-200D-1F52C"},
    {"mezczyzna_astronauta", "1F468-200D-1F680"},
    {"mezczyzna_astronauta_kolor1", "1F468-1F3FB-200D-1F680"},
    {"mezczyzna_astronauta_kolor2", "1F468-1F3FC-200D-1F680"},
    {"mezczyzna_astronauta_kolor3", "1F468-1F3FD-200D-1F680"},
    {"mezczyzna_astronauta_kolor4", "1F468-1F3FE-200D-1F680"},
    {"mezczyzna_astronauta_kolor5", "1F468-1F3FF-200D-1F680"},
    {"mezczyzna_strazak", "1F468-200D-1F692"},
    {"mezczyzna_strazak_kolor1", "1F468-1F3FB-200D-1F692"},
    {"mezczyzna_strazak_kolor2", "1F468-1F3FC-200D-1F692"},
    {"mezczyzna_strazak_kolor3", "1F468-1F3FD-200D-1F692"},
    {"mezczyzna_strazak_kolor4", "1F468-1F3FE-200D-1F692"},
    {"mezczyzna_strazak_kolor5", "1F468-1F3FF-200D-1F692"},
    {"mezczyzna_z_biala_laska", "1F468-200D-1F9AF"},
    {"mezczyzna_z_biala_laska_kolor1", "1F468-1F3FB-200D-1F9AF"},
    {"mezczyzna_z_biala_laska_kolor2", "1F468-1F3FC-200D-1F9AF"},
    {"mezczyzna_z_biala_laska_kolor3", "1F468-1F3FD-200D-1F9AF"},
    {"mezczyzna_z_biala_laska_kolor4", "1F468-1F3FE-200D-1F9AF"},
    {"mezczyzna_z_biala_laska_kolor5", "1F468-1F3FF-200D-1F9AF"},
    {"mezczyzna_z_biala_laska_zwrocony_w_prawo", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"mezczyzna_z_biala_laska_zwrocony_w_prawo_kolor1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"mezczyzna_z_biala_laska_zwrocony_w_prawo_kolor2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"mezczyzna_z_biala_laska_zwrocony_w_prawo_kolor3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"mezczyzna_z_biala_laska_zwrocony_w_prawo_kolor4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"mezczyzna_z_biala_laska_zwrocony_w_prawo_kolor5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"mezczyzna_na_elektrycznym_wozku_inwalidzkim", "1F468-200D-1F9BC"},
    {"mezczyzna_na_elektrycznym_wozku_inwalidzkim_kolor1", "1F468-1F3FB-200D-1F9BC"},
    {"mezczyzna_na_elektrycznym_wozku_inwalidzkim_kolor2", "1F468-1F3FC-200D-1F9BC"},
    {"mezczyzna_na_elektrycznym_wozku_inwalidzkim_kolor3", "1F468-1F3FD-200D-1F9BC"},
    {"mezczyzna_na_elektrycznym_wozku_inwalidzkim_kolor4", "1F468-1F3FE-200D-1F9BC"},
    {"mezczyzna_na_elektrycznym_wozku_inwalidzkim_kolor5", "1F468-1F3FF-200D-1F9BC"},
    {"mezczyzna_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"mezczyzna_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"mezczyzna_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"mezczyzna_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"mezczyzna_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"mezczyzna_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mezczyzna_na_wozku_inwalidzkim", "1F468-200D-1F9BD"},
    {"mezczyzna_na_wozku_inwalidzkim_kolor1", "1F468-1F3FB-200D-1F9BD"},
    {"mezczyzna_na_wozku_inwalidzkim_kolor2", "1F468-1F3FC-200D-1F9BD"},
    {"mezczyzna_na_wozku_inwalidzkim_kolor3", "1F468-1F3FD-200D-1F9BD"},
    {"mezczyzna_na_wozku_inwalidzkim_kolor4", "1F468-1F3FE-200D-1F9BD"},
    {"mezczyzna_na_wozku_inwalidzkim_kolor5", "1F468-1F3FF-200D-1F9BD"},
    {"mezczyzna_na_wozku_inwalidzkim_zwrocony_w_prawo", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"mezczyzna_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"mezczyzna_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"mezczyzna_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"mezczyzna_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"mezczyzna_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"lekarka", "1F469-200D-2695-FE0F"},
    {"lekarka_kolor1", "1F469-1F3FB-200D-2695-FE0F"},
    {"lekarka_kolor2", "1F469-1F3FC-200D-2695-FE0F"},
    {"lekarka_kolor3", "1F469-1F3FD-200D-2695-FE0F"},
    {"lekarka_kolor4", "1F469-1F3FE-200D-2695-FE0F"},
    {"lekarka_kolor5", "1F469-1F3FF-200D-2695-FE0F"},
    {"kobieta_sedzia", "1F469-200D-2696-FE0F"},
    {"kobieta_sedzia_kolor1", "1F469-1F3FB-200D-2696-FE0F"},
    {"kobieta_sedzia_kolor2", "1F469-1F3FC-200D-2696-FE0F"},
    {"kobieta_sedzia_kolor3", "1F469-1F3FD-200D-2696-FE0F"},
    {"kobieta_sedzia_kolor4", "1F469-1F3FE-200D-2696-FE0F"},
    {"kobieta_sedzia_kolor5", "1F469-1F3FF-200D-2696-FE0F"},
    {"kobieta_pilot", "1F469-200D-2708-FE0F"},
    {"kobieta_pilot_kolor1", "1F469-1F3FB-200D-2708-FE0F"},
    {"kobieta_pilot_kolor2", "1F469-1F3FC-200D-2708-FE0F"},
    {"kobieta_pilot_kolor3", "1F469-1F3FD-200D-2708-FE0F"},
    {"kobieta_pilot_kolor4", "1F469-1F3FE-200D-2708-FE0F"},
    {"kobieta_pilot_kolor5", "1F469-1F3FF-200D-2708-FE0F"},
    {"rolniczka", "1F469-200D-1F33E"},
    {"rolniczka_kolor1", "1F469-1F3FB-200D-1F33E"},
    {"rolniczka_kolor2", "1F469-1F3FC-200D-1F33E"},
    {"rolniczka_kolor3", "1F469-1F3FD-200D-1F33E"},
    {"rolniczka_kolor4", "1F469-1F3FE-200D-1F33E"},
    {"rolniczka_kolor5", "1F469-1F3FF-200D-1F33E"},
    {"kucharka", "1F469-200D-1F373"},
    {"kucharka_kolor1", "1F469-1F3FB-200D-1F373"},
    {"kucharka_kolor2", "1F469-1F3FC-200D-1F373"},
    {"kucharka_kolor3", "1F469-1F3FD-200D-1F373"},
    {"kucharka_kolor4", "1F469-1F3FE-200D-1F373"},
    {"kucharka_kolor5", "1F469-1F3FF-200D-1F373"},
    {"kobieta_karmiaca_niemowle", "1F469-200D-1F37C"},
    {"kobieta_karmiaca_niemowle_kolor1", "1F469-1F3FB-200D-1F37C"},
    {"kobieta_karmiaca_niemowle_kolor2", "1F469-1F3FC-200D-1F37C"},
    {"kobieta_karmiaca_niemowle_kolor3", "1F469-1F3FD-200D-1F37C"},
    {"kobieta_karmiaca_niemowle_kolor4", "1F469-1F3FE-200D-1F37C"},
    {"kobieta_karmiaca_niemowle_kolor5", "1F469-1F3FF-200D-1F37C"},
    {"studentka", "1F469-200D-1F393"},
    {"studentka_kolor1", "1F469-1F3FB-200D-1F393"},
    {"studentka_kolor2", "1F469-1F3FC-200D-1F393"},
    {"studentka_kolor3", "1F469-1F3FD-200D-1F393"},
    {"studentka_kolor4", "1F469-1F3FE-200D-1F393"},
    {"studentka_kolor5", "1F469-1F3FF-200D-1F393"},
    {"piosenkarka", "1F469-200D-1F3A4"},
    {"piosenkarka_kolor1", "1F469-1F3FB-200D-1F3A4"},
    {"piosenkarka_kolor2", "1F469-1F3FC-200D-1F3A4"},
    {"piosenkarka_kolor3", "1F469-1F3FD-200D-1F3A4"},
    {"piosenkarka_kolor4", "1F469-1F3FE-200D-1F3A4"},
    {"piosenkarka_kolor5", "1F469-1F3FF-200D-1F3A4"},
    {"artystka", "1F469-200D-1F3A8"},
    {"artystka_kolor1", "1F469-1F3FB-200D-1F3A8"},
    {"artystka_kolor2", "1F469-1F3FC-200D-1F3A8"},
    {"artystka_kolor3", "1F469-1F3FD-200D-1F3A8"},
    {"artystka_kolor4", "1F469-1F3FE-200D-1F3A8"},
    {"artystka_kolor5", "1F469-1F3FF-200D-1F3A8"},
    {"nauczycielka", "1F469-200D-1F3EB"},
    {"nauczycielka_kolor1", "1F469-1F3FB-200D-1F3EB"},
    {"nauczycielka_kolor2", "1F469-1F3FC-200D-1F3EB"},
    {"nauczycielka_kolor3", "1F469-1F3FD-200D-1F3EB"},
    {"nauczycielka_kolor4", "1F469-1F3FE-200D-1F3EB"},
    {"nauczycielka_kolor5", "1F469-1F3FF-200D-1F3EB"},
    {"pracownica_fabryki", "1F469-200D-1F3ED"},
    {"pracownica_fabryki_kolor1", "1F469-1F3FB-200D-1F3ED"},
    {"pracownica_fabryki_kolor2", "1F469-1F3FC-200D-1F3ED"},
    {"pracownica_fabryki_kolor3", "1F469-1F3FD-200D-1F3ED"},
    {"pracownica_fabryki_kolor4", "1F469-1F3FE-200D-1F3ED"},
    {"pracownica_fabryki_kolor5", "1F469-1F3FF-200D-1F3ED"},
    {"kobieta_technolog", "1F469-200D-1F4BB"},
    {"kobieta_technolog_kolor1", "1F469-1F3FB-200D-1F4BB"},
    {"kobieta_technolog_kolor2", "1F469-1F3FC-200D-1F4BB"},
    {"kobieta_technolog_kolor3", "1F469-1F3FD-200D-1F4BB"},
    {"kobieta_technolog_kolor4", "1F469-1F3FE-200D-1F4BB"},
    {"kobieta_technolog_kolor5", "1F469-1F3FF-200D-1F4BB"},
    {"pracownica_biurowa", "1F469-200D-1F4BC"},
    {"pracownica_biurowa_kolor1", "1F469-1F3FB-200D-1F4BC"},
    {"pracownica_biurowa_kolor2", "1F469-1F3FC-200D-1F4BC"},
    {"pracownica_biurowa_kolor3", "1F469-1F3FD-200D-1F4BC"},
    {"pracownica_biurowa_kolor4", "1F469-1F3FE-200D-1F4BC"},
    {"pracownica_biurowa_kolor5", "1F469-1F3FF-200D-1F4BC"},
    {"kobieta_elektryk", "1F469-200D-1F527"},
    {"kobieta_elektryk_kolor1", "1F469-1F3FB-200D-1F527"},
    {"kobieta_elektryk_kolor2", "1F469-1F3FC-200D-1F527"},
    {"kobieta_elektryk_kolor3", "1F469-1F3FD-200D-1F527"},
    {"kobieta_elektryk_kolor4", "1F469-1F3FE-200D-1F527"},
    {"kobieta_elektryk_kolor5", "1F469-1F3FF-200D-1F527"},
    {"kobieta_naukowiec", "1F469-200D-1F52C"},
    {"kobieta_naukowiec_kolor1", "1F469-1F3FB-200D-1F52C"},
    {"kobieta_naukowiec_kolor2", "1F469-1F3FC-200D-1F52C"},
    {"kobieta_naukowiec_kolor3", "1F469-1F3FD-200D-1F52C"},
    {"kobieta_naukowiec_kolor4", "1F469-1F3FE-200D-1F52C"},
    {"kobieta_naukowiec_kolor5", "1F469-1F3FF-200D-1F52C"},
    {"astronautka", "1F469-200D-1F680"},
    {"astronautka_kolor1", "1F469-1F3FB-200D-1F680"},
    {"astronautka_kolor2", "1F469-1F3FC-200D-1F680"},
    {"astronautka_kolor3", "1F469-1F3FD-200D-1F680"},
    {"astronautka_kolor4", "1F469-1F3FE-200D-1F680"},
    {"astronautka_kolor5", "1F469-1F3FF-200D-1F680"},
    {"kobieta_strazak", "1F469-200D-1F692"},
    {"kobieta_strazak_kolor1", "1F469-1F3FB-200D-1F692"},
    {"kobieta_strazak_kolor2", "1F469-1F3FC-200D-1F692"},
    {"kobieta_strazak_kolor3", "1F469-1F3FD-200D-1F692"},
    {"kobieta_strazak_kolor4", "1F469-1F3FE-200D-1F692"},
    {"kobieta_strazak_kolor5", "1F469-1F3FF-200D-1F692"},
    {"kobieta_z_biala_laska", "1F469-200D-1F9AF"},
    {"kobieta_z_biala_laska_kolor1", "1F469-1F3FB-200D-1F9AF"},
    {"kobieta_z_biala_laska_kolor2", "1F469-1F3FC-200D-1F9AF"},
    {"kobieta_z_biala_laska_kolor3", "1F469-1F3FD-200D-1F9AF"},
    {"kobieta_z_biala_laska_kolor4", "1F469-1F3FE-200D-1F9AF"},
    {"kobieta_z_biala_laska_kolor5", "1F469-1F3FF-200D-1F9AF"},
    {"kobieta_z_biala_laska_zwrocony_w_prawo", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"kobieta_z_biala_laska_zwrocony_w_prawo_kolor1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"kobieta_z_biala_laska_zwrocony_w_prawo_kolor2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"kobieta_z_biala_laska_zwrocony_w_prawo_kolor3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"kobieta_z_biala_laska_zwrocony_w_prawo_kolor4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"kobieta_z_biala_laska_zwrocony_w_prawo_kolor5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"kobieta_na_elektrycznym_wozku_inwalidzkim", "1F469-200D-1F9BC"},
    {"kobieta_na_elektrycznym_wozku_inwalidzkim_kolor1", "1F469-1F3FB-200D-1F9BC"},
    {"kobieta_na_elektrycznym_wozku_inwalidzkim_kolor2", "1F469-1F3FC-200D-1F9BC"},
    {"kobieta_na_elektrycznym_wozku_inwalidzkim_kolor3", "1F469-1F3FD-200D-1F9BC"},
    {"kobieta_na_elektrycznym_wozku_inwalidzkim_kolor4", "1F469-1F3FE-200D-1F9BC"},
    {"kobieta_na_elektrycznym_wozku_inwalidzkim_kolor5", "1F469-1F3FF-200D-1F9BC"},
    {"kobieta_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"kobieta_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"kobieta_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"kobieta_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"kobieta_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"kobieta_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"kobieta_na_wozku_inwalidzkim", "1F469-200D-1F9BD"},
    {"kobieta_na_wozku_inwalidzkim_kolor1", "1F469-1F3FB-200D-1F9BD"},
    {"kobieta_na_wozku_inwalidzkim_kolor2", "1F469-1F3FC-200D-1F9BD"},
    {"kobieta_na_wozku_inwalidzkim_kolor3", "1F469-1F3FD-200D-1F9BD"},
    {"kobieta_na_wozku_inwalidzkim_kolor4", "1F469-1F3FE-200D-1F9BD"},
    {"kobieta_na_wozku_inwalidzkim_kolor5", "1F469-1F3FF-200D-1F9BD"},
    {"kobieta_na_wozku_inwalidzkim_zwrocony_w_prawo", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"idaca_osoba_zwrocony_w_prawo", "1F6B6-200D-27A1-FE0F"},
    {"idaca_osoba_zwrocony_w_prawo_kolor1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"idaca_osoba_zwrocony_w_prawo_kolor2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"idaca_osoba_zwrocony_w_prawo_kolor3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"idaca_osoba_zwrocony_w_prawo_kolor4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"idaca_osoba_zwrocony_w_prawo_kolor5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"kleczaca_osoba_zwrocony_w_prawo", "1F9CE-200D-27A1-FE0F"},
    {"kleczaca_osoba_zwrocony_w_prawo_kolor1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"kleczaca_osoba_zwrocony_w_prawo_kolor2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"kleczaca_osoba_zwrocony_w_prawo_kolor3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"kleczaca_osoba_zwrocony_w_prawo_kolor4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"kleczaca_osoba_zwrocony_w_prawo_kolor5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"pracownik_sluzby_zdrowia", "1F9D1-200D-2695-FE0F"},
    {"pracownik_sluzby_zdrowia_kolor1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"pracownik_sluzby_zdrowia_kolor2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"pracownik_sluzby_zdrowia_kolor3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"pracownik_sluzby_zdrowia_kolor4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"pracownik_sluzby_zdrowia_kolor5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"sedzia", "1F9D1-200D-2696-FE0F"},
    {"sedzia_kolor1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"sedzia_kolor2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"sedzia_kolor3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"sedzia_kolor4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"sedzia_kolor5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"pilot", "1F9D1-200D-2708-FE0F"},
    {"pilot_kolor1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"pilot_kolor2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"pilot_kolor3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"pilot_kolor4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"pilot_kolor5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"rolnik", "1F9D1-200D-1F33E"},
    {"rolnik_kolor1", "1F9D1-1F3FB-200D-1F33E"},
    {"rolnik_kolor2", "1F9D1-1F3FC-200D-1F33E"},
    {"rolnik_kolor3", "1F9D1-1F3FD-200D-1F33E"},
    {"rolnik_kolor4", "1F9D1-1F3FE-200D-1F33E"},
    {"rolnik_kolor5", "1F9D1-1F3FF-200D-1F33E"},
    {"kucharz", "1F9D1-200D-1F373"},
    {"kucharz_kolor1", "1F9D1-1F3FB-200D-1F373"},
    {"kucharz_kolor2", "1F9D1-1F3FC-200D-1F373"},
    {"kucharz_kolor3", "1F9D1-1F3FD-200D-1F373"},
    {"kucharz_kolor4", "1F9D1-1F3FE-200D-1F373"},
    {"kucharz_kolor5", "1F9D1-1F3FF-200D-1F373"},
    {"osoba_karmiaca_niemowle", "1F9D1-200D-1F37C"},
    {"osoba_karmiaca_niemowle_kolor1", "1F9D1-1F3FB-200D-1F37C"},
    {"osoba_karmiaca_niemowle_kolor2", "1F9D1-1F3FC-200D-1F37C"},
    {"osoba_karmiaca_niemowle_kolor3", "1F9D1-1F3FD-200D-1F37C"},
    {"osoba_karmiaca_niemowle_kolor4", "1F9D1-1F3FE-200D-1F37C"},
    {"osoba_karmiaca_niemowle_kolor5", "1F9D1-1F3FF-200D-1F37C"},
    {"mikolajka", "1F9D1-200D-1F384"},
    {"mikolajka_kolor1", "1F9D1-1F3FB-200D-1F384"},
    {"mikolajka_kolor2", "1F9D1-1F3FC-200D-1F384"},
    {"mikolajka_kolor3", "1F9D1-1F3FD-200D-1F384"},
    {"mikolajka_kolor4", "1F9D1-1F3FE-200D-1F384"},
    {"mikolajka_kolor5", "1F9D1-1F3FF-200D-1F384"},
    {"student", "1F9D1-200D-1F393"},
    {"student_kolor1", "1F9D1-1F3FB-200D-1F393"},
    {"student_kolor2", "1F9D1-1F3FC-200D-1F393"},
    {"student_kolor3", "1F9D1-1F3FD-200D-1F393"},
    {"student_kolor4", "1F9D1-1F3FE-200D-1F393"},
    {"student_kolor5", "1F9D1-1F3FF-200D-1F393"},
    {"piosenkarz", "1F9D1-200D-1F3A4"},
    {"piosenkarz_kolor1", "1F9D1-1F3FB-200D-1F3A4"},
    {"piosenkarz_kolor2", "1F9D1-1F3FC-200D-1F3A4"},
    {"piosenkarz_kolor3", "1F9D1-1F3FD-200D-1F3A4"},
    {"piosenkarz_kolor4", "1F9D1-1F3FE-200D-1F3A4"},
    {"piosenkarz_kolor5", "1F9D1-1F3FF-200D-1F3A4"},
    {"artysta", "1F9D1-200D-1F3A8"},
    {"artysta_kolor1", "1F9D1-1F3FB-200D-1F3A8"},
    {"artysta_kolor2", "1F9D1-1F3FC-200D-1F3A8"},
    {"artysta_kolor3", "1F9D1-1F3FD-200D-1F3A8"},
    {"artysta_kolor4", "1F9D1-1F3FE-200D-1F3A8"},
    {"artysta_kolor5", "1F9D1-1F3FF-200D-1F3A8"},
    {"nauczyciel", "1F9D1-200D-1F3EB"},
    {"nauczyciel_kolor1", "1F9D1-1F3FB-200D-1F3EB"},
    {"nauczyciel_kolor2", "1F9D1-1F3FC-200D-1F3EB"},
    {"nauczyciel_kolor3", "1F9D1-1F3FD-200D-1F3EB"},
    {"nauczyciel_kolor4", "1F9D1-1F3FE-200D-1F3EB"},
    {"nauczyciel_kolor5", "1F9D1-1F3FF-200D-1F3EB"},
    {"pracownik_fabryki", "1F9D1-200D-1F3ED"},
    {"pracownik_fabryki_kolor1", "1F9D1-1F3FB-200D-1F3ED"},
    {"pracownik_fabryki_kolor2", "1F9D1-1F3FC-200D-1F3ED"},
    {"pracownik_fabryki_kolor3", "1F9D1-1F3FD-200D-1F3ED"},
    {"pracownik_fabryki_kolor4", "1F9D1-1F3FE-200D-1F3ED"},
    {"pracownik_fabryki_kolor5", "1F9D1-1F3FF-200D-1F3ED"},
    {"technolog", "1F9D1-200D-1F4BB"},
    {"technolog_kolor1", "1F9D1-1F3FB-200D-1F4BB"},
    {"technolog_kolor2", "1F9D1-1F3FC-200D-1F4BB"},
    {"technolog_kolor3", "1F9D1-1F3FD-200D-1F4BB"},
    {"technolog_kolor4", "1F9D1-1F3FE-200D-1F4BB"},
    {"technolog_kolor5", "1F9D1-1F3FF-200D-1F4BB"},
    {"pracownik_biurowy", "1F9D1-200D-1F4BC"},
    {"pracownik_biurowy_kolor1", "1F9D1-1F3FB-200D-1F4BC"},
    {"pracownik_biurowy_kolor2", "1F9D1-1F3FC-200D-1F4BC"},
    {"pracownik_biurowy_kolor3", "1F9D1-1F3FD-200D-1F4BC"},
    {"pracownik_biurowy_kolor4", "1F9D1-1F3FE-200D-1F4BC"},
    {"pracownik_biurowy_kolor5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mechanik", "1F9D1-200D-1F527"},
    {"mechanik_kolor1", "1F9D1-1F3FB-200D-1F527"},
    {"mechanik_kolor2", "1F9D1-1F3FC-200D-1F527"},
    {"mechanik_kolor3", "1F9D1-1F3FD-200D-1F527"},
    {"mechanik_kolor4", "1F9D1-1F3FE-200D-1F527"},
    {"mechanik_kolor5", "1F9D1-1F3FF-200D-1F527"},
    {"naukowiec", "1F9D1-200D-1F52C"},
    {"naukowiec_kolor1", "1F9D1-1F3FB-200D-1F52C"},
    {"naukowiec_kolor2", "1F9D1-1F3FC-200D-1F52C"},
    {"naukowiec_kolor3", "1F9D1-1F3FD-200D-1F52C"},
    {"naukowiec_kolor4", "1F9D1-1F3FE-200D-1F52C"},
    {"naukowiec_kolor5", "1F9D1-1F3FF-200D-1F52C"},
    {"astronauta", "1F9D1-200D-1F680"},
    {"astronauta_kolor1", "1F9D1-1F3FB-200D-1F680"},
    {"astronauta_kolor2", "1F9D1-1F3FC-200D-1F680"},
    {"astronauta_kolor3", "1F9D1-1F3FD-200D-1F680"},
    {"astronauta_kolor4", "1F9D1-1F3FE-200D-1F680"},
    {"astronauta_kolor5", "1F9D1-1F3FF-200D-1F680"},
    {"strazak", "1F9D1-200D-1F692"},
    {"strazak_kolor1", "1F9D1-1F3FB-200D-1F692"},
    {"strazak_kolor2", "1F9D1-1F3FC-200D-1F692"},
    {"strazak_kolor3", "1F9D1-1F3FD-200D-1F692"},
    {"strazak_kolor4", "1F9D1-1F3FE-200D-1F692"},
    {"strazak_kolor5", "1F9D1-1F3FF-200D-1F692"},
    {"osoba_z_biala_laska", "1F9D1-200D-1F9AF"},
    {"osoba_z_biala_laska_kolor1", "1F9D1-1F3FB-200D-1F9AF"},
    {"osoba_z_biala_laska_kolor2", "1F9D1-1F3FC-200D-1F9AF"},
    {"osoba_z_biala_laska_kolor3", "1F9D1-1F3FD-200D-1F9AF"},
    {"osoba_z_biala_laska_kolor4", "1F9D1-1F3FE-200D-1F9AF"},
    {"osoba_z_biala_laska_kolor5", "1F9D1-1F3FF-200D-1F9AF"},
    {"osoba_z_biala_laska_zwrocony_w_prawo", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"osoba_z_biala_laska_zwrocony_w_prawo_kolor1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"osoba_z_biala_laska_zwrocony_w_prawo_kolor2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"osoba_z_biala_laska_zwrocony_w_prawo_kolor3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"osoba_z_biala_laska_zwrocony_w_prawo_kolor4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"osoba_z_biala_laska_zwrocony_w_prawo_kolor5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"osoba_na_elektrycznym_wozku_inwalidzkim", "1F9D1-200D-1F9BC"},
    {"osoba_na_elektrycznym_wozku_inwalidzkim_kolor1", "1F9D1-1F3FB-200D-1F9BC"},
    {"osoba_na_elektrycznym_wozku_inwalidzkim_kolor2", "1F9D1-1F3FC-200D-1F9BC"},
    {"osoba_na_elektrycznym_wozku_inwalidzkim_kolor3", "1F9D1-1F3FD-200D-1F9BC"},
    {"osoba_na_elektrycznym_wozku_inwalidzkim_kolor4", "1F9D1-1F3FE-200D-1F9BC"},
    {"osoba_na_elektrycznym_wozku_inwalidzkim_kolor5", "1F9D1-1F3FF-200D-1F9BC"},
    {"osoba_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"osoba_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"osoba_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"osoba_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"osoba_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"osoba_na_elektrycznym_wozku_inwalidzkim_zwrocony_w_prawo_kolor5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"osoba_na_wozku_inwalidzkim", "1F9D1-200D-1F9BD"},
    {"osoba_na_wozku_inwalidzkim_kolor1", "1F9D1-1F3FB-200D-1F9BD"},
    {"osoba_na_wozku_inwalidzkim_kolor2", "1F9D1-1F3FC-200D-1F9BD"},
    {"osoba_na_wozku_inwalidzkim_kolor3", "1F9D1-1F3FD-200D-1F9BD"},
    {"osoba_na_wozku_inwalidzkim_kolor4", "1F9D1-1F3FE-200D-1F9BD"},
    {"osoba_na_wozku_inwalidzkim_kolor5", "1F9D1-1F3FF-200D-1F9BD"},
    {"osoba_na_wozku_inwalidzkim_zwrocony_w_prawo", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"osoba_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"osoba_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"osoba_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"osoba_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"osoba_na_wozku_inwalidzkim_zwrocony_w_prawo_kolor5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_kozlujaca_pilke", "26F9-FE0F-200D-2640-FE0F"},
    {"kobieta_kozlujaca_pilke_kolor1", "26F9-1F3FB-200D-2640-FE0F"},
    {"kobieta_kozlujaca_pilke_kolor2", "26F9-1F3FC-200D-2640-FE0F"},
    {"kobieta_kozlujaca_pilke_kolor3", "26F9-1F3FD-200D-2640-FE0F"},
    {"kobieta_kozlujaca_pilke_kolor4", "26F9-1F3FE-200D-2640-FE0F"},
    {"kobieta_kozlujaca_pilke_kolor5", "26F9-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_kozlujacy_pilke", "26F9-FE0F-200D-2642-FE0F"},
    {"mezczyzna_kozlujacy_pilke_kolor1", "26F9-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_kozlujacy_pilke_kolor2", "26F9-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_kozlujacy_pilke_kolor3", "26F9-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_kozlujacy_pilke_kolor4", "26F9-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_kozlujacy_pilke_kolor5", "26F9-1F3FF-200D-2642-FE0F"},
    {"biegnaca_kobieta", "1F3C3-200D-2640-FE0F"},
    {"biegnaca_kobieta_kolor1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"biegnaca_kobieta_kolor2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"biegnaca_kobieta_kolor3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"biegnaca_kobieta_kolor4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"biegnaca_kobieta_kolor5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"biegnaca_kobieta_zwrocony_w_prawo", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"biegnaca_kobieta_zwrocony_w_prawo_kolor1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"biegnaca_kobieta_zwrocony_w_prawo_kolor2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"biegnaca_kobieta_zwrocony_w_prawo_kolor3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"biegnaca_kobieta_zwrocony_w_prawo_kolor4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"biegnaca_kobieta_zwrocony_w_prawo_kolor5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"biegnacy_mezczyzna", "1F3C3-200D-2642-FE0F"},
    {"biegnacy_mezczyzna_kolor1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"biegnacy_mezczyzna_kolor2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"biegnacy_mezczyzna_kolor3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"biegnacy_mezczyzna_kolor4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"biegnacy_mezczyzna_kolor5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"biegnacy_mezczyzna_zwrocony_w_prawo", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"biegnacy_mezczyzna_zwrocony_w_prawo_kolor1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"biegnacy_mezczyzna_zwrocony_w_prawo_kolor2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"biegnacy_mezczyzna_zwrocony_w_prawo_kolor3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"biegnacy_mezczyzna_zwrocony_w_prawo_kolor4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"biegnacy_mezczyzna_zwrocony_w_prawo_kolor5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"surfujaca_kobieta", "1F3C4-200D-2640-FE0F"},
    {"surfujaca_kobieta_kolor1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"surfujaca_kobieta_kolor2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"surfujaca_kobieta_kolor3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"surfujaca_kobieta_kolor4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"surfujaca_kobieta_kolor5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"surfujacy_mezczyzna", "1F3C4-200D-2642-FE0F"},
    {"surfujacy_mezczyzna_kolor1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"surfujacy_mezczyzna_kolor2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"surfujacy_mezczyzna_kolor3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"surfujacy_mezczyzna_kolor4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"surfujacy_mezczyzna_kolor5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"plywajaca_kobieta", "1F3CA-200D-2640-FE0F"},
    {"plywajaca_kobieta_kolor1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"plywajaca_kobieta_kolor2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"plywajaca_kobieta_kolor3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"plywajaca_kobieta_kolor4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"plywajaca_kobieta_kolor5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"plywajacy_mezczyzna", "1F3CA-200D-2642-FE0F"},
    {"plywajacy_mezczyzna_kolor1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"plywajacy_mezczyzna_kolor2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"plywajacy_mezczyzna_kolor3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"plywajacy_mezczyzna_kolor4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"plywajacy_mezczyzna_kolor5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"kobieta_podnoszaca_ciezary", "1F3CB-FE0F-200D-2640-FE0F"},
    {"kobieta_podnoszaca_ciezary_kolor1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"kobieta_podnoszaca_ciezary_kolor2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"kobieta_podnoszaca_ciezary_kolor3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"kobieta_podnoszaca_ciezary_kolor4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"kobieta_podnoszaca_ciezary_kolor5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_podnoszacy_ciezary", "1F3CB-FE0F-200D-2642-FE0F"},
    {"mezczyzna_podnoszacy_ciezary_kolor1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_podnoszacy_ciezary_kolor2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_podnoszacy_ciezary_kolor3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_podnoszacy_ciezary_kolor4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_podnoszacy_ciezary_kolor5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"kobieta_grajaca_w_golfa", "1F3CC-FE0F-200D-2640-FE0F"},
    {"kobieta_grajaca_w_golfa_kolor1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"kobieta_grajaca_w_golfa_kolor2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"kobieta_grajaca_w_golfa_kolor3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"kobieta_grajaca_w_golfa_kolor4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"kobieta_grajaca_w_golfa_kolor5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_grajacy_w_golfa", "1F3CC-FE0F-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_golfa_kolor1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_golfa_kolor2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_golfa_kolor3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_golfa_kolor4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_golfa_kolor5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"kobieta_policjant", "1F46E-200D-2640-FE0F"},
    {"kobieta_policjant_kolor1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"kobieta_policjant_kolor2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"kobieta_policjant_kolor3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"kobieta_policjant_kolor4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"kobieta_policjant_kolor5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_policjant", "1F46E-200D-2642-FE0F"},
    {"mezczyzna_policjant_kolor1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_policjant_kolor2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_policjant_kolor3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_policjant_kolor4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_policjant_kolor5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"kobiety_z_uszami_krolika_na_przyjeciu", "1F46F-200D-2640-FE0F"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"kobiety_z_uszami_krolika_na_przyjeciu_kolor5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu", "1F46F-200D-2642-FE0F"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"mezczyzni_z_uszami_krolika_na_przyjeciu_kolor5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"kobieta_w_welonie", "1F470-200D-2640-FE0F"},
    {"kobieta_w_welonie_kolor1", "1F470-1F3FB-200D-2640-FE0F"},
    {"kobieta_w_welonie_kolor2", "1F470-1F3FC-200D-2640-FE0F"},
    {"kobieta_w_welonie_kolor3", "1F470-1F3FD-200D-2640-FE0F"},
    {"kobieta_w_welonie_kolor4", "1F470-1F3FE-200D-2640-FE0F"},
    {"kobieta_w_welonie_kolor5", "1F470-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_w_welonie", "1F470-200D-2642-FE0F"},
    {"mezczyzna_w_welonie_kolor1", "1F470-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_w_welonie_kolor2", "1F470-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_w_welonie_kolor3", "1F470-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_w_welonie_kolor4", "1F470-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_w_welonie_kolor5", "1F470-1F3FF-200D-2642-FE0F"},
    {"blondynka", "1F471-200D-2640-FE0F"},
    {"blondynka_kolor1", "1F471-1F3FB-200D-2640-FE0F"},
    {"blondynka_kolor2", "1F471-1F3FC-200D-2640-FE0F"},
    {"blondynka_kolor3", "1F471-1F3FD-200D-2640-FE0F"},
    {"blondynka_kolor4", "1F471-1F3FE-200D-2640-FE0F"},
    {"blondynka_kolor5", "1F471-1F3FF-200D-2640-FE0F"},
    {"blondyn", "1F471-200D-2642-FE0F"},
    {"blondyn_kolor1", "1F471-1F3FB-200D-2642-FE0F"},
    {"blondyn_kolor2", "1F471-1F3FC-200D-2642-FE0F"},
    {"blondyn_kolor3", "1F471-1F3FD-200D-2642-FE0F"},
    {"blondyn_kolor4", "1F471-1F3FE-200D-2642-FE0F"},
    {"blondyn_kolor5", "1F471-1F3FF-200D-2642-FE0F"},
    {"kobieta_w_turbanie", "1F473-200D-2640-FE0F"},
    {"kobieta_w_turbanie_kolor1", "1F473-1F3FB-200D-2640-FE0F"},
    {"kobieta_w_turbanie_kolor2", "1F473-1F3FC-200D-2640-FE0F"},
    {"kobieta_w_turbanie_kolor3", "1F473-1F3FD-200D-2640-FE0F"},
    {"kobieta_w_turbanie_kolor4", "1F473-1F3FE-200D-2640-FE0F"},
    {"kobieta_w_turbanie_kolor5", "1F473-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_w_turbanie", "1F473-200D-2642-FE0F"},
    {"mezczyzna_w_turbanie_kolor1", "1F473-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_w_turbanie_kolor2", "1F473-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_w_turbanie_kolor3", "1F473-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_w_turbanie_kolor4", "1F473-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_w_turbanie_kolor5", "1F473-1F3FF-200D-2642-FE0F"},
    {"kobieta_budowlaniec", "1F477-200D-2640-FE0F"},
    {"kobieta_budowlaniec_kolor1", "1F477-1F3FB-200D-2640-FE0F"},
    {"kobieta_budowlaniec_kolor2", "1F477-1F3FC-200D-2640-FE0F"},
    {"kobieta_budowlaniec_kolor3", "1F477-1F3FD-200D-2640-FE0F"},
    {"kobieta_budowlaniec_kolor4", "1F477-1F3FE-200D-2640-FE0F"},
    {"kobieta_budowlaniec_kolor5", "1F477-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_budowlaniec", "1F477-200D-2642-FE0F"},
    {"mezczyzna_budowlaniec_kolor1", "1F477-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_budowlaniec_kolor2", "1F477-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_budowlaniec_kolor3", "1F477-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_budowlaniec_kolor4", "1F477-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_budowlaniec_kolor5", "1F477-1F3FF-200D-2642-FE0F"},
    {"kobieta_z_wystawiona_dlonia", "1F481-200D-2640-FE0F"},
    {"kobieta_z_wystawiona_dlonia_kolor1", "1F481-1F3FB-200D-2640-FE0F"},
    {"kobieta_z_wystawiona_dlonia_kolor2", "1F481-1F3FC-200D-2640-FE0F"},
    {"kobieta_z_wystawiona_dlonia_kolor3", "1F481-1F3FD-200D-2640-FE0F"},
    {"kobieta_z_wystawiona_dlonia_kolor4", "1F481-1F3FE-200D-2640-FE0F"},
    {"kobieta_z_wystawiona_dlonia_kolor5", "1F481-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_z_wystawiona_dlonia", "1F481-200D-2642-FE0F"},
    {"mezczyzna_z_wystawiona_dlonia_kolor1", "1F481-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_z_wystawiona_dlonia_kolor2", "1F481-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_z_wystawiona_dlonia_kolor3", "1F481-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_z_wystawiona_dlonia_kolor4", "1F481-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_z_wystawiona_dlonia_kolor5", "1F481-1F3FF-200D-2642-FE0F"},
    {"kobieta_gwardzista", "1F482-200D-2640-FE0F"},
    {"kobieta_gwardzista_kolor1", "1F482-1F3FB-200D-2640-FE0F"},
    {"kobieta_gwardzista_kolor2", "1F482-1F3FC-200D-2640-FE0F"},
    {"kobieta_gwardzista_kolor3", "1F482-1F3FD-200D-2640-FE0F"},
    {"kobieta_gwardzista_kolor4", "1F482-1F3FE-200D-2640-FE0F"},
    {"kobieta_gwardzista_kolor5", "1F482-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_gwardzista", "1F482-200D-2642-FE0F"},
    {"mezczyzna_gwardzista_kolor1", "1F482-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_gwardzista_kolor2", "1F482-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_gwardzista_kolor3", "1F482-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_gwardzista_kolor4", "1F482-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_gwardzista_kolor5", "1F482-1F3FF-200D-2642-FE0F"},
    {"kobieta_podczas_masazu", "1F486-200D-2640-FE0F"},
    {"kobieta_podczas_masazu_kolor1", "1F486-1F3FB-200D-2640-FE0F"},
    {"kobieta_podczas_masazu_kolor2", "1F486-1F3FC-200D-2640-FE0F"},
    {"kobieta_podczas_masazu_kolor3", "1F486-1F3FD-200D-2640-FE0F"},
    {"kobieta_podczas_masazu_kolor4", "1F486-1F3FE-200D-2640-FE0F"},
    {"kobieta_podczas_masazu_kolor5", "1F486-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_podczas_masazu", "1F486-200D-2642-FE0F"},
    {"mezczyzna_podczas_masazu_kolor1", "1F486-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_podczas_masazu_kolor2", "1F486-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_podczas_masazu_kolor3", "1F486-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_podczas_masazu_kolor4", "1F486-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_podczas_masazu_kolor5", "1F486-1F3FF-200D-2642-FE0F"},
    {"kobieta_podczas_strzyzenia", "1F487-200D-2640-FE0F"},
    {"kobieta_podczas_strzyzenia_kolor1", "1F487-1F3FB-200D-2640-FE0F"},
    {"kobieta_podczas_strzyzenia_kolor2", "1F487-1F3FC-200D-2640-FE0F"},
    {"kobieta_podczas_strzyzenia_kolor3", "1F487-1F3FD-200D-2640-FE0F"},
    {"kobieta_podczas_strzyzenia_kolor4", "1F487-1F3FE-200D-2640-FE0F"},
    {"kobieta_podczas_strzyzenia_kolor5", "1F487-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_podczas_strzyzenia", "1F487-200D-2642-FE0F"},
    {"mezczyzna_podczas_strzyzenia_kolor1", "1F487-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_podczas_strzyzenia_kolor2", "1F487-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_podczas_strzyzenia_kolor3", "1F487-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_podczas_strzyzenia_kolor4", "1F487-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_podczas_strzyzenia_kolor5", "1F487-1F3FF-200D-2642-FE0F"},
    {"kobieta_detektyw", "1F575-FE0F-200D-2640-FE0F"},
    {"kobieta_detektyw_kolor1", "1F575-1F3FB-200D-2640-FE0F"},
    {"kobieta_detektyw_kolor2", "1F575-1F3FC-200D-2640-FE0F"},
    {"kobieta_detektyw_kolor3", "1F575-1F3FD-200D-2640-FE0F"},
    {"kobieta_detektyw_kolor4", "1F575-1F3FE-200D-2640-FE0F"},
    {"kobieta_detektyw_kolor5", "1F575-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_detektyw", "1F575-FE0F-200D-2642-FE0F"},
    {"mezczyzna_detektyw_kolor1", "1F575-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_detektyw_kolor2", "1F575-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_detektyw_kolor3", "1F575-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_detektyw_kolor4", "1F575-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_detektyw_kolor5", "1F575-1F3FF-200D-2642-FE0F"},
    {"kobieta_pokazujaca_gest_nie", "1F645-200D-2640-FE0F"},
    {"kobieta_pokazujaca_gest_nie_kolor1", "1F645-1F3FB-200D-2640-FE0F"},
    {"kobieta_pokazujaca_gest_nie_kolor2", "1F645-1F3FC-200D-2640-FE0F"},
    {"kobieta_pokazujaca_gest_nie_kolor3", "1F645-1F3FD-200D-2640-FE0F"},
    {"kobieta_pokazujaca_gest_nie_kolor4", "1F645-1F3FE-200D-2640-FE0F"},
    {"kobieta_pokazujaca_gest_nie_kolor5", "1F645-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_pokazujacy_gest_nie", "1F645-200D-2642-FE0F"},
    {"mezczyzna_pokazujacy_gest_nie_kolor1", "1F645-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_pokazujacy_gest_nie_kolor2", "1F645-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_pokazujacy_gest_nie_kolor3", "1F645-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_pokazujacy_gest_nie_kolor4", "1F645-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_pokazujacy_gest_nie_kolor5", "1F645-1F3FF-200D-2642-FE0F"},
    {"kobieta_pokazujaca_gest_ok", "1F646-200D-2640-FE0F"},
    {"kobieta_pokazujaca_gest_ok_kolor1", "1F646-1F3FB-200D-2640-FE0F"},
    {"kobieta_pokazujaca_gest_ok_kolor2", "1F646-1F3FC-200D-2640-FE0F"},
    {"kobieta_pokazujaca_gest_ok_kolor3", "1F646-1F3FD-200D-2640-FE0F"},
    {"kobieta_pokazujaca_gest_ok_kolor4", "1F646-1F3FE-200D-2640-FE0F"},
    {"kobieta_pokazujaca_gest_ok_kolor5", "1F646-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_pokazujacy_gest_ok", "1F646-200D-2642-FE0F"},
    {"mezczyzna_pokazujacy_gest_ok_kolor1", "1F646-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_pokazujacy_gest_ok_kolor2", "1F646-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_pokazujacy_gest_ok_kolor3", "1F646-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_pokazujacy_gest_ok_kolor4", "1F646-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_pokazujacy_gest_ok_kolor5", "1F646-1F3FF-200D-2642-FE0F"},
    {"kobieta_w_uklonie", "1F647-200D-2640-FE0F"},
    {"kobieta_w_uklonie_kolor1", "1F647-1F3FB-200D-2640-FE0F"},
    {"kobieta_w_uklonie_kolor2", "1F647-1F3FC-200D-2640-FE0F"},
    {"kobieta_w_uklonie_kolor3", "1F647-1F3FD-200D-2640-FE0F"},
    {"kobieta_w_uklonie_kolor4", "1F647-1F3FE-200D-2640-FE0F"},
    {"kobieta_w_uklonie_kolor5", "1F647-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_w_uklonie", "1F647-200D-2642-FE0F"},
    {"mezczyzna_w_uklonie_kolor1", "1F647-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_w_uklonie_kolor2", "1F647-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_w_uklonie_kolor3", "1F647-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_w_uklonie_kolor4", "1F647-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_w_uklonie_kolor5", "1F647-1F3FF-200D-2642-FE0F"},
    {"kobieta_podnoszaca_reke", "1F64B-200D-2640-FE0F"},
    {"kobieta_podnoszaca_reke_kolor1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"kobieta_podnoszaca_reke_kolor2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"kobieta_podnoszaca_reke_kolor3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"kobieta_podnoszaca_reke_kolor4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"kobieta_podnoszaca_reke_kolor5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_podnoszacy_reke", "1F64B-200D-2642-FE0F"},
    {"mezczyzna_podnoszacy_reke_kolor1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_podnoszacy_reke_kolor2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_podnoszacy_reke_kolor3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_podnoszacy_reke_kolor4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_podnoszacy_reke_kolor5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"zachmurzona_kobieta", "1F64D-200D-2640-FE0F"},
    {"zachmurzona_kobieta_kolor1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"zachmurzona_kobieta_kolor2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"zachmurzona_kobieta_kolor3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"zachmurzona_kobieta_kolor4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"zachmurzona_kobieta_kolor5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"zachmurzony_mezczyzna", "1F64D-200D-2642-FE0F"},
    {"zachmurzony_mezczyzna_kolor1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"zachmurzony_mezczyzna_kolor2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"zachmurzony_mezczyzna_kolor3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"zachmurzony_mezczyzna_kolor4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"zachmurzony_mezczyzna_kolor5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"nadasana_kobieta", "1F64E-200D-2640-FE0F"},
    {"nadasana_kobieta_kolor1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"nadasana_kobieta_kolor2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"nadasana_kobieta_kolor3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"nadasana_kobieta_kolor4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"nadasana_kobieta_kolor5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"nadasany_mezczyzna", "1F64E-200D-2642-FE0F"},
    {"nadasany_mezczyzna_kolor1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"nadasany_mezczyzna_kolor2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"nadasany_mezczyzna_kolor3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"nadasany_mezczyzna_kolor4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"nadasany_mezczyzna_kolor5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"kobieta_wioslujaca_w_lodzi", "1F6A3-200D-2640-FE0F"},
    {"kobieta_wioslujaca_w_lodzi_kolor1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"kobieta_wioslujaca_w_lodzi_kolor2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"kobieta_wioslujaca_w_lodzi_kolor3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"kobieta_wioslujaca_w_lodzi_kolor4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"kobieta_wioslujaca_w_lodzi_kolor5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_wioslujacy_w_lodzi", "1F6A3-200D-2642-FE0F"},
    {"mezczyzna_wioslujacy_w_lodzi_kolor1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_wioslujacy_w_lodzi_kolor2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_wioslujacy_w_lodzi_kolor3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_wioslujacy_w_lodzi_kolor4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_wioslujacy_w_lodzi_kolor5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"kobieta_na_rowerze", "1F6B4-200D-2640-FE0F"},
    {"kobieta_na_rowerze_kolor1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"kobieta_na_rowerze_kolor2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"kobieta_na_rowerze_kolor3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"kobieta_na_rowerze_kolor4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"kobieta_na_rowerze_kolor5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_na_rowerze", "1F6B4-200D-2642-FE0F"},
    {"mezczyzna_na_rowerze_kolor1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_na_rowerze_kolor2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_na_rowerze_kolor3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_na_rowerze_kolor4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_na_rowerze_kolor5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"kobieta_na_rowerze_gorskim", "1F6B5-200D-2640-FE0F"},
    {"kobieta_na_rowerze_gorskim_kolor1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"kobieta_na_rowerze_gorskim_kolor2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"kobieta_na_rowerze_gorskim_kolor3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"kobieta_na_rowerze_gorskim_kolor4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"kobieta_na_rowerze_gorskim_kolor5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_na_rowerze_gorskim", "1F6B5-200D-2642-FE0F"},
    {"mezczyzna_na_rowerze_gorskim_kolor1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_na_rowerze_gorskim_kolor2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_na_rowerze_gorskim_kolor3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_na_rowerze_gorskim_kolor4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_na_rowerze_gorskim_kolor5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"idaca_kobieta", "1F6B6-200D-2640-FE0F"},
    {"idaca_kobieta_kolor1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"idaca_kobieta_kolor2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"idaca_kobieta_kolor3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"idaca_kobieta_kolor4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"idaca_kobieta_kolor5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"idaca_kobieta_zwrocony_w_prawo", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"idaca_kobieta_zwrocony_w_prawo_kolor1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"idaca_kobieta_zwrocony_w_prawo_kolor2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"idaca_kobieta_zwrocony_w_prawo_kolor3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"idaca_kobieta_zwrocony_w_prawo_kolor4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"idaca_kobieta_zwrocony_w_prawo_kolor5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"idacy_mezczyzna", "1F6B6-200D-2642-FE0F"},
    {"idacy_mezczyzna_kolor1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"idacy_mezczyzna_kolor2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"idacy_mezczyzna_kolor3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"idacy_mezczyzna_kolor4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"idacy_mezczyzna_kolor5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"idacy_mezczyzna_zwrocony_w_prawo", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"idacy_mezczyzna_zwrocony_w_prawo_kolor1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"idacy_mezczyzna_zwrocony_w_prawo_kolor2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"idacy_mezczyzna_zwrocony_w_prawo_kolor3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"idacy_mezczyzna_zwrocony_w_prawo_kolor4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"idacy_mezczyzna_zwrocony_w_prawo_kolor5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kobieta_trzymajaca_sie_za_czolo", "1F926-200D-2640-FE0F"},
    {"kobieta_trzymajaca_sie_za_czolo_kolor1", "1F926-1F3FB-200D-2640-FE0F"},
    {"kobieta_trzymajaca_sie_za_czolo_kolor2", "1F926-1F3FC-200D-2640-FE0F"},
    {"kobieta_trzymajaca_sie_za_czolo_kolor3", "1F926-1F3FD-200D-2640-FE0F"},
    {"kobieta_trzymajaca_sie_za_czolo_kolor4", "1F926-1F3FE-200D-2640-FE0F"},
    {"kobieta_trzymajaca_sie_za_czolo_kolor5", "1F926-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_trzymajacy_sie_za_czolo", "1F926-200D-2642-FE0F"},
    {"mezczyzna_trzymajacy_sie_za_czolo_kolor1", "1F926-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_trzymajacy_sie_za_czolo_kolor2", "1F926-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_trzymajacy_sie_za_czolo_kolor3", "1F926-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_trzymajacy_sie_za_czolo_kolor4", "1F926-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_trzymajacy_sie_za_czolo_kolor5", "1F926-1F3FF-200D-2642-FE0F"},
    {"kobieta_w_smokingu", "1F935-200D-2640-FE0F"},
    {"kobieta_w_smokingu_kolor1", "1F935-1F3FB-200D-2640-FE0F"},
    {"kobieta_w_smokingu_kolor2", "1F935-1F3FC-200D-2640-FE0F"},
    {"kobieta_w_smokingu_kolor3", "1F935-1F3FD-200D-2640-FE0F"},
    {"kobieta_w_smokingu_kolor4", "1F935-1F3FE-200D-2640-FE0F"},
    {"kobieta_w_smokingu_kolor5", "1F935-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_w_smokingu", "1F935-200D-2642-FE0F"},
    {"mezczyzna_w_smokingu_kolor1", "1F935-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_w_smokingu_kolor2", "1F935-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_w_smokingu_kolor3", "1F935-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_w_smokingu_kolor4", "1F935-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_w_smokingu_kolor5", "1F935-1F3FF-200D-2642-FE0F"},
    {"kobieta_wzruszajaca_ramionami", "1F937-200D-2640-FE0F"},
    {"kobieta_wzruszajaca_ramionami_kolor1", "1F937-1F3FB-200D-2640-FE0F"},
    {"kobieta_wzruszajaca_ramionami_kolor2", "1F937-1F3FC-200D-2640-FE0F"},
    {"kobieta_wzruszajaca_ramionami_kolor3", "1F937-1F3FD-200D-2640-FE0F"},
    {"kobieta_wzruszajaca_ramionami_kolor4", "1F937-1F3FE-200D-2640-FE0F"},
    {"kobieta_wzruszajaca_ramionami_kolor5", "1F937-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_wzruszajacy_ramionami", "1F937-200D-2642-FE0F"},
    {"mezczyzna_wzruszajacy_ramionami_kolor1", "1F937-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_wzruszajacy_ramionami_kolor2", "1F937-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_wzruszajacy_ramionami_kolor3", "1F937-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_wzruszajacy_ramionami_kolor4", "1F937-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_wzruszajacy_ramionami_kolor5", "1F937-1F3FF-200D-2642-FE0F"},
    {"kobieta_robiaca_gwiazde", "1F938-200D-2640-FE0F"},
    {"kobieta_robiaca_gwiazde_kolor1", "1F938-1F3FB-200D-2640-FE0F"},
    {"kobieta_robiaca_gwiazde_kolor2", "1F938-1F3FC-200D-2640-FE0F"},
    {"kobieta_robiaca_gwiazde_kolor3", "1F938-1F3FD-200D-2640-FE0F"},
    {"kobieta_robiaca_gwiazde_kolor4", "1F938-1F3FE-200D-2640-FE0F"},
    {"kobieta_robiaca_gwiazde_kolor5", "1F938-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_robiacy_gwiazde", "1F938-200D-2642-FE0F"},
    {"mezczyzna_robiacy_gwiazde_kolor1", "1F938-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_robiacy_gwiazde_kolor2", "1F938-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_robiacy_gwiazde_kolor3", "1F938-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_robiacy_gwiazde_kolor4", "1F938-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_robiacy_gwiazde_kolor5", "1F938-1F3FF-200D-2642-FE0F"},
    {"zonglujaca_kobieta", "1F939-200D-2640-FE0F"},
    {"zonglujaca_kobieta_kolor1", "1F939-1F3FB-200D-2640-FE0F"},
    {"zonglujaca_kobieta_kolor2", "1F939-1F3FC-200D-2640-FE0F"},
    {"zonglujaca_kobieta_kolor3", "1F939-1F3FD-200D-2640-FE0F"},
    {"zonglujaca_kobieta_kolor4", "1F939-1F3FE-200D-2640-FE0F"},
    {"zonglujaca_kobieta_kolor5", "1F939-1F3FF-200D-2640-FE0F"},
    {"zonglujacy_mezczyzna", "1F939-200D-2642-FE0F"},
    {"zonglujacy_mezczyzna_kolor1", "1F939-1F3FB-200D-2642-FE0F"},
    {"zonglujacy_mezczyzna_kolor2", "1F939-1F3FC-200D-2642-FE0F"},
    {"zonglujacy_mezczyzna_kolor3", "1F939-1F3FD-200D-2642-FE0F"},
    {"zonglujacy_mezczyzna_kolor4", "1F939-1F3FE-200D-2642-FE0F"},
    {"zonglujacy_mezczyzna_kolor5", "1F939-1F3FF-200D-2642-FE0F"},
    {"kobiety_uprawiajace_zapasy", "1F93C-200D-2640-FE0F"},
    {"kobiety_uprawiajace_zapasy_kolor1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"kobiety_uprawiajace_zapasy_kolor1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"kobiety_uprawiajace_zapasy_kolor1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"kobiety_uprawiajace_zapasy_kolor1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"kobiety_uprawiajace_zapasy_kolor2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"kobiety_uprawiajace_zapasy_kolor2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"kobiety_uprawiajace_zapasy_kolor2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"kobiety_uprawiajace_zapasy_kolor2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"kobiety_uprawiajace_zapasy_kolor3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"kobiety_uprawiajace_zapasy_kolor3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"kobiety_uprawiajace_zapasy_kolor3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"kobiety_uprawiajace_zapasy_kolor3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"kobiety_uprawiajace_zapasy_kolor4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"kobiety_uprawiajace_zapasy_kolor4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"kobiety_uprawiajace_zapasy_kolor4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"kobiety_uprawiajace_zapasy_kolor4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"kobiety_uprawiajace_zapasy_kolor5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"kobiety_uprawiajace_zapasy_kolor5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"kobiety_uprawiajace_zapasy_kolor5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"kobiety_uprawiajace_zapasy_kolor5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"kobiety_uprawiajace_zapasy_kolor1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"kobiety_uprawiajace_zapasy_kolor2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"kobiety_uprawiajace_zapasy_kolor3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"kobiety_uprawiajace_zapasy_kolor4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"kobiety_uprawiajace_zapasy_kolor5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"mezczyzni_uprawiajacy_zapasy", "1F93C-200D-2642-FE0F"},
    {"mezczyzni_uprawiajacy_zapasy_kolor1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"mezczyzni_uprawiajacy_zapasy_kolor1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"mezczyzni_uprawiajacy_zapasy_kolor1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"mezczyzni_uprawiajacy_zapasy_kolor1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"mezczyzni_uprawiajacy_zapasy_kolor2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"mezczyzni_uprawiajacy_zapasy_kolor2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"mezczyzni_uprawiajacy_zapasy_kolor2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"mezczyzni_uprawiajacy_zapasy_kolor2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"mezczyzni_uprawiajacy_zapasy_kolor3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"mezczyzni_uprawiajacy_zapasy_kolor3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"mezczyzni_uprawiajacy_zapasy_kolor3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"mezczyzni_uprawiajacy_zapasy_kolor3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"mezczyzni_uprawiajacy_zapasy_kolor4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"mezczyzni_uprawiajacy_zapasy_kolor4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"mezczyzni_uprawiajacy_zapasy_kolor4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"mezczyzni_uprawiajacy_zapasy_kolor4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"mezczyzni_uprawiajacy_zapasy_kolor5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"mezczyzni_uprawiajacy_zapasy_kolor5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"mezczyzni_uprawiajacy_zapasy_kolor5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"mezczyzni_uprawiajacy_zapasy_kolor5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"mezczyzni_uprawiajacy_zapasy_kolor1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"mezczyzni_uprawiajacy_zapasy_kolor2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"mezczyzni_uprawiajacy_zapasy_kolor3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"mezczyzni_uprawiajacy_zapasy_kolor4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"mezczyzni_uprawiajacy_zapasy_kolor5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"kobieta_grajaca_w_pilke_wodna", "1F93D-200D-2640-FE0F"},
    {"kobieta_grajaca_w_pilke_wodna_kolor1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"kobieta_grajaca_w_pilke_wodna_kolor2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"kobieta_grajaca_w_pilke_wodna_kolor3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"kobieta_grajaca_w_pilke_wodna_kolor4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"kobieta_grajaca_w_pilke_wodna_kolor5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_grajacy_w_pilke_wodna", "1F93D-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_pilke_wodna_kolor1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_pilke_wodna_kolor2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_pilke_wodna_kolor3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_pilke_wodna_kolor4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_pilke_wodna_kolor5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"kobieta_grajaca_w_pilke_reczna", "1F93E-200D-2640-FE0F"},
    {"kobieta_grajaca_w_pilke_reczna_kolor1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"kobieta_grajaca_w_pilke_reczna_kolor2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"kobieta_grajaca_w_pilke_reczna_kolor3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"kobieta_grajaca_w_pilke_reczna_kolor4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"kobieta_grajaca_w_pilke_reczna_kolor5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_grajacy_w_pilke_reczna", "1F93E-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_pilke_reczna_kolor1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_pilke_reczna_kolor2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_pilke_reczna_kolor3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_pilke_reczna_kolor4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_grajacy_w_pilke_reczna_kolor5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"kobieta_superbohater", "1F9B8-200D-2640-FE0F"},
    {"kobieta_superbohater_kolor1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"kobieta_superbohater_kolor2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"kobieta_superbohater_kolor3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"kobieta_superbohater_kolor4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"kobieta_superbohater_kolor5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_superbohater", "1F9B8-200D-2642-FE0F"},
    {"mezczyzna_superbohater_kolor1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_superbohater_kolor2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_superbohater_kolor3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_superbohater_kolor4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_superbohater_kolor5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"kobieta_superlotr", "1F9B9-200D-2640-FE0F"},
    {"kobieta_superlotr_kolor1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"kobieta_superlotr_kolor2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"kobieta_superlotr_kolor3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"kobieta_superlotr_kolor4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"kobieta_superlotr_kolor5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_superlotr", "1F9B9-200D-2642-FE0F"},
    {"mezczyzna_superlotr_kolor1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_superlotr_kolor2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_superlotr_kolor3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_superlotr_kolor4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_superlotr_kolor5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"stojaca_kobieta", "1F9CD-200D-2640-FE0F"},
    {"stojaca_kobieta_kolor1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"stojaca_kobieta_kolor2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"stojaca_kobieta_kolor3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"stojaca_kobieta_kolor4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"stojaca_kobieta_kolor5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"stojacy_mezczyzna", "1F9CD-200D-2642-FE0F"},
    {"stojacy_mezczyzna_kolor1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"stojacy_mezczyzna_kolor2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"stojacy_mezczyzna_kolor3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"stojacy_mezczyzna_kolor4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"stojacy_mezczyzna_kolor5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"kleczaca_kobieta", "1F9CE-200D-2640-FE0F"},
    {"kleczaca_kobieta_kolor1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"kleczaca_kobieta_kolor2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"kleczaca_kobieta_kolor3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"kleczaca_kobieta_kolor4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"kleczaca_kobieta_kolor5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"kleczaca_kobieta_zwrocony_w_prawo", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kleczaca_kobieta_zwrocony_w_prawo_kolor1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kleczaca_kobieta_zwrocony_w_prawo_kolor2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kleczaca_kobieta_zwrocony_w_prawo_kolor3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kleczaca_kobieta_zwrocony_w_prawo_kolor4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kleczaca_kobieta_zwrocony_w_prawo_kolor5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kleczacy_mezczyzna", "1F9CE-200D-2642-FE0F"},
    {"kleczacy_mezczyzna_kolor1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"kleczacy_mezczyzna_kolor2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"kleczacy_mezczyzna_kolor3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"kleczacy_mezczyzna_kolor4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"kleczacy_mezczyzna_kolor5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"kleczacy_mezczyzna_zwrocony_w_prawo", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kleczacy_mezczyzna_zwrocony_w_prawo_kolor1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kleczacy_mezczyzna_zwrocony_w_prawo_kolor2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kleczacy_mezczyzna_zwrocony_w_prawo_kolor3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kleczacy_mezczyzna_zwrocony_w_prawo_kolor4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kleczacy_mezczyzna_zwrocony_w_prawo_kolor5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"glucha_kobieta", "1F9CF-200D-2640-FE0F"},
    {"glucha_kobieta_kolor1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"glucha_kobieta_kolor2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"glucha_kobieta_kolor3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"glucha_kobieta_kolor4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"glucha_kobieta_kolor5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"gluchy_mezczyzna", "1F9CF-200D-2642-FE0F"},
    {"gluchy_mezczyzna_kolor1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"gluchy_mezczyzna_kolor2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"gluchy_mezczyzna_kolor3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"gluchy_mezczyzna_kolor4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"gluchy_mezczyzna_kolor5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"kobieta_z_broda", "1F9D4-200D-2640-FE0F"},
    {"kobieta_z_broda_kolor1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"kobieta_z_broda_kolor2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"kobieta_z_broda_kolor3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"kobieta_z_broda_kolor4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"kobieta_z_broda_kolor5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_z_broda", "1F9D4-200D-2642-FE0F"},
    {"mezczyzna_z_broda_kolor1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_z_broda_kolor2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_z_broda_kolor3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_z_broda_kolor4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_z_broda_kolor5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"kobieta_w_zaparowanym_pomieszczeniu", "1F9D6-200D-2640-FE0F"},
    {"kobieta_w_zaparowanym_pomieszczeniu_kolor1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"kobieta_w_zaparowanym_pomieszczeniu_kolor2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"kobieta_w_zaparowanym_pomieszczeniu_kolor3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"kobieta_w_zaparowanym_pomieszczeniu_kolor4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"kobieta_w_zaparowanym_pomieszczeniu_kolor5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_w_zaparowanym_pomieszczeniu", "1F9D6-200D-2642-FE0F"},
    {"mezczyzna_w_zaparowanym_pomieszczeniu_kolor1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_w_zaparowanym_pomieszczeniu_kolor2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_w_zaparowanym_pomieszczeniu_kolor3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_w_zaparowanym_pomieszczeniu_kolor4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_w_zaparowanym_pomieszczeniu_kolor5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"kobieta_wspinajaca_sie", "1F9D7-200D-2640-FE0F"},
    {"kobieta_wspinajaca_sie_kolor1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"kobieta_wspinajaca_sie_kolor2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"kobieta_wspinajaca_sie_kolor3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"kobieta_wspinajaca_sie_kolor4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"kobieta_wspinajaca_sie_kolor5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_wspinajacy_sie", "1F9D7-200D-2642-FE0F"},
    {"mezczyzna_wspinajacy_sie_kolor1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_wspinajacy_sie_kolor2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_wspinajacy_sie_kolor3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_wspinajacy_sie_kolor4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_wspinajacy_sie_kolor5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"kobieta_w_pozycji_lotosu", "1F9D8-200D-2640-FE0F"},
    {"kobieta_w_pozycji_lotosu_kolor1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"kobieta_w_pozycji_lotosu_kolor2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"kobieta_w_pozycji_lotosu_kolor3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"kobieta_w_pozycji_lotosu_kolor4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"kobieta_w_pozycji_lotosu_kolor5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_w_pozycji_lotosu", "1F9D8-200D-2642-FE0F"},
    {"mezczyzna_w_pozycji_lotosu_kolor1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_w_pozycji_lotosu_kolor2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_w_pozycji_lotosu_kolor3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_w_pozycji_lotosu_kolor4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_w_pozycji_lotosu_kolor5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"kobieta_mag", "1F9D9-200D-2640-FE0F"},
    {"kobieta_mag_kolor1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"kobieta_mag_kolor2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"kobieta_mag_kolor3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"kobieta_mag_kolor4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"kobieta_mag_kolor5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_mag", "1F9D9-200D-2642-FE0F"},
    {"mezczyzna_mag_kolor1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_mag_kolor2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_mag_kolor3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_mag_kolor4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_mag_kolor5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"kobieta_wrozka", "1F9DA-200D-2640-FE0F"},
    {"kobieta_wrozka_kolor1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"kobieta_wrozka_kolor2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"kobieta_wrozka_kolor3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"kobieta_wrozka_kolor4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"kobieta_wrozka_kolor5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_wrozka", "1F9DA-200D-2642-FE0F"},
    {"mezczyzna_wrozka_kolor1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_wrozka_kolor2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_wrozka_kolor3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_wrozka_kolor4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_wrozka_kolor5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"kobieta_wampir", "1F9DB-200D-2640-FE0F"},
    {"kobieta_wampir_kolor1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"kobieta_wampir_kolor2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"kobieta_wampir_kolor3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"kobieta_wampir_kolor4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"kobieta_wampir_kolor5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_wampir", "1F9DB-200D-2642-FE0F"},
    {"mezczyzna_wampir_kolor1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_wampir_kolor2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_wampir_kolor3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_wampir_kolor4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_wampir_kolor5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"kobieta_syrena", "1F9DC-200D-2640-FE0F"},
    {"kobieta_syrena_kolor1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"kobieta_syrena_kolor2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"kobieta_syrena_kolor3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"kobieta_syrena_kolor4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"kobieta_syrena_kolor5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_syrena", "1F9DC-200D-2642-FE0F"},
    {"mezczyzna_syrena_kolor1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_syrena_kolor2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_syrena_kolor3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_syrena_kolor4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_syrena_kolor5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"kobieta_elf", "1F9DD-200D-2640-FE0F"},
    {"kobieta_elf_kolor1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"kobieta_elf_kolor2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"kobieta_elf_kolor3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"kobieta_elf_kolor4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"kobieta_elf_kolor5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"mezczyzna_elf", "1F9DD-200D-2642-FE0F"},
    {"mezczyzna_elf_kolor1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"mezczyzna_elf_kolor2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"mezczyzna_elf_kolor3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"mezczyzna_elf_kolor4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"mezczyzna_elf_kolor5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"kobieta_dzin", "1F9DE-200D-2640-FE0F"},
    {"mezczyzna_dzin", "1F9DE-200D-2642-FE0F"},
    {"kobieta_zombie", "1F9DF-200D-2640-FE0F"},
    {"mezczyzna_zombie", "1F9DF-200D-2642-FE0F"},
    {"mezczyzna_rude_wlosy", "1F468-200D-1F9B0"},
    {"mezczyzna_rude_wlosy_kolor1", "1F468-1F3FB-200D-1F9B0"},
    {"mezczyzna_rude_wlosy_kolor2", "1F468-1F3FC-200D-1F9B0"},
    {"mezczyzna_rude_wlosy_kolor3", "1F468-1F3FD-200D-1F9B0"},
    {"mezczyzna_rude_wlosy_kolor4", "1F468-1F3FE-200D-1F9B0"},
    {"mezczyzna_rude_wlosy_kolor5", "1F468-1F3FF-200D-1F9B0"},
    {"mezczyzna_krecone_wlosy", "1F468-200D-1F9B1"},
    {"mezczyzna_krecone_wlosy_kolor1", "1F468-1F3FB-200D-1F9B1"},
    {"mezczyzna_krecone_wlosy_kolor2", "1F468-1F3FC-200D-1F9B1"},
    {"mezczyzna_krecone_wlosy_kolor3", "1F468-1F3FD-200D-1F9B1"},
    {"mezczyzna_krecone_wlosy_kolor4", "1F468-1F3FE-200D-1F9B1"},
    {"mezczyzna_krecone_wlosy_kolor5", "1F468-1F3FF-200D-1F9B1"},
    {"mezczyzna_lysy", "1F468-200D-1F9B2"},
    {"mezczyzna_lysy_kolor1", "1F468-1F3FB-200D-1F9B2"},
    {"mezczyzna_lysy_kolor2", "1F468-1F3FC-200D-1F9B2"},
    {"mezczyzna_lysy_kolor3", "1F468-1F3FD-200D-1F9B2"},
    {"mezczyzna_lysy_kolor4", "1F468-1F3FE-200D-1F9B2"},
    {"mezczyzna_lysy_kolor5", "1F468-1F3FF-200D-1F9B2"},
    {"mezczyzna_biale_wlosy", "1F468-200D-1F9B3"},
    {"mezczyzna_biale_wlosy_kolor1", "1F468-1F3FB-200D-1F9B3"},
    {"mezczyzna_biale_wlosy_kolor2", "1F468-1F3FC-200D-1F9B3"},
    {"mezczyzna_biale_wlosy_kolor3", "1F468-1F3FD-200D-1F9B3"},
    {"mezczyzna_biale_wlosy_kolor4", "1F468-1F3FE-200D-1F9B3"},
    {"mezczyzna_biale_wlosy_kolor5", "1F468-1F3FF-200D-1F9B3"},
    {"kobieta_rude_wlosy", "1F469-200D-1F9B0"},
    {"kobieta_rude_wlosy_kolor1", "1F469-1F3FB-200D-1F9B0"},
    {"kobieta_rude_wlosy_kolor2", "1F469-1F3FC-200D-1F9B0"},
    {"kobieta_rude_wlosy_kolor3", "1F469-1F3FD-200D-1F9B0"},
    {"kobieta_rude_wlosy_kolor4", "1F469-1F3FE-200D-1F9B0"},
    {"kobieta_rude_wlosy_kolor5", "1F469-1F3FF-200D-1F9B0"},
    {"kobieta_krecone_wlosy", "1F469-200D-1F9B1"},
    {"kobieta_krecone_wlosy_kolor1", "1F469-1F3FB-200D-1F9B1"},
    {"kobieta_krecone_wlosy_kolor2", "1F469-1F3FC-200D-1F9B1"},
    {"kobieta_krecone_wlosy_kolor3", "1F469-1F3FD-200D-1F9B1"},
    {"kobieta_krecone_wlosy_kolor4", "1F469-1F3FE-200D-1F9B1"},
    {"kobieta_krecone_wlosy_kolor5", "1F469-1F3FF-200D-1F9B1"},
    {"kobieta_lysy", "1F469-200D-1F9B2"},
    {"kobieta_lysy_kolor1", "1F469-1F3FB-200D-1F9B2"},
    {"kobieta_lysy_kolor2", "1F469-1F3FC-200D-1F9B2"},
    {"kobieta_lysy_kolor3", "1F469-1F3FD-200D-1F9B2"},
    {"kobieta_lysy_kolor4", "1F469-1F3FE-200D-1F9B2"},
    {"kobieta_lysy_kolor5", "1F469-1F3FF-200D-1F9B2"},
    {"kobieta_biale_wlosy", "1F469-200D-1F9B3"},
    {"kobieta_biale_wlosy_kolor1", "1F469-1F3FB-200D-1F9B3"},
    {"kobieta_biale_wlosy_kolor2", "1F469-1F3FC-200D-1F9B3"},
    {"kobieta_biale_wlosy_kolor3", "1F469-1F3FD-200D-1F9B3"},
    {"kobieta_biale_wlosy_kolor4", "1F469-1F3FE-200D-1F9B3"},
    {"kobieta_biale_wlosy_kolor5", "1F469-1F3FF-200D-1F9B3"},
    {"dorosly_rude_wlosy", "1F9D1-200D-1F9B0"},
    {"dorosly_rude_wlosy_kolor1", "1F9D1-1F3FB-200D-1F9B0"},
    {"dorosly_rude_wlosy_kolor2", "1F9D1-1F3FC-200D-1F9B0"},
    {"dorosly_rude_wlosy_kolor3", "1F9D1-1F3FD-200D-1F9B0"},
    {"dorosly_rude_wlosy_kolor4", "1F9D1-1F3FE-200D-1F9B0"},
    {"dorosly_rude_wlosy_kolor5", "1F9D1-1F3FF-200D-1F9B0"},
    {"dorosly_krecone_wlosy", "1F9D1-200D-1F9B1"},
    {"dorosly_krecone_wlosy_kolor1", "1F9D1-1F3FB-200D-1F9B1"},
    {"dorosly_krecone_wlosy_kolor2", "1F9D1-1F3FC-200D-1F9B1"},
    {"dorosly_krecone_wlosy_kolor3", "1F9D1-1F3FD-200D-1F9B1"},
    {"dorosly_krecone_wlosy_kolor4", "1F9D1-1F3FE-200D-1F9B1"},
    {"dorosly_krecone_wlosy_kolor5", "1F9D1-1F3FF-200D-1F9B1"},
    {"dorosly_lysy", "1F9D1-200D-1F9B2"},
    {"dorosly_lysy_kolor1", "1F9D1-1F3FB-200D-1F9B2"},
    {"dorosly_lysy_kolor2", "1F9D1-1F3FC-200D-1F9B2"},
    {"dorosly_lysy_kolor3", "1F9D1-1F3FD-200D-1F9B2"},
    {"dorosly_lysy_kolor4", "1F9D1-1F3FE-200D-1F9B2"},
    {"dorosly_lysy_kolor5", "1F9D1-1F3FF-200D-1F9B2"},
    {"dorosly_biale_wlosy", "1F9D1-200D-1F9B3"},
    {"dorosly_biale_wlosy_kolor1", "1F9D1-1F3FB-200D-1F9B3"},
    {"dorosly_biale_wlosy_kolor2", "1F9D1-1F3FC-200D-1F9B3"},
    {"dorosly_biale_wlosy_kolor3", "1F9D1-1F3FD-200D-1F9B3"},
    {"dorosly_biale_wlosy_kolor4", "1F9D1-1F3FE-200D-1F9B3"},
    {"dorosly_biale_wlosy_kolor5", "1F9D1-1F3FF-200D-1F9B3"},
    {"zerwany_lancuch", "26D3-FE0F-200D-1F4A5"},
    {"serce_w_plomieniach", "2764-FE0F-200D-1F525"},
    {"uleczone_serce", "2764-FE0F-200D-1FA79"},
    {"brazowy_grzyb", "1F344-200D-1F7EB"},
    {"limonka", "1F34B-200D-1F7E9"},
    {"flaga_osob_transplciowych", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"teczowa_flaga", "1F3F3-FE0F-200D-1F308"},
    {"piracka_flaga", "1F3F4-200D-2620-FE0F"},
    {"czarny_kot", "1F408-200D-2B1B"},
    {"pies_pomocnik", "1F415-200D-1F9BA"},
    {"czarny_ptak", "1F426-200D-2B1B"},
    {"feniks", "1F426-200D-1F525"},
    {"niedzwiedz_polarny", "1F43B-200D-2744-FE0F"},
    {"oko_w_dymku", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"twarz_wypuszczajaca_powietrze", "1F62E-200D-1F4A8"},
    {"twarz_ze_spiralnymi_oczami", "1F635-200D-1F4AB"},
    {"twarz_w_chmurach", "1F636-200D-1F32B-FE0F"},
    {"glowa_krecaca_sie_poziomo", "1F642-200D-2194-FE0F"},
    {"glowa_kiwajaca_sie_pionowo", "1F642-200D-2195-FE0F"},
    {"baletnica", "1F9D1-200D-1FA70"},
    {"baletnica_kolor1", "1F9D1-1F3FB-200D-1FA70"},
    {"baletnica_kolor2", "1F9D1-1F3FC-200D-1FA70"},
    {"baletnica_kolor3", "1F9D1-1F3FD-200D-1FA70"},
    {"baletnica_kolor4", "1F9D1-1F3FE-200D-1FA70"},
    {"baletnica_kolor5", "1F9D1-1F3FF-200D-1FA70"},
    {"strzałka_w_lewo_i_w_prawo", "2194"},
    {"strzałka_w_górę_i_w_dół", "2195"},
    {"strzałka_w_górę_w_lewo", "2196"},
    {"strzałka_w_górę_w_prawo", "2197"},
    {"strzałka_w_dół_w_prawo", "2198"},
    {"strzałka_w_dół_w_lewo", "2199"},
    {"słońce", "2600"},
    {"bałwanek_i_płatki_śniegu", "2603"},
    {"gorący_napój", "2615"},
    {"zagrożenie_ze_strony_organizmów_żywych", "2623"},
    {"krzyż_prawosławny", "2626"},
    {"koło_dharmy", "2638"},
    {"znak_mężczyzny", "2642"},
    {"koziorożec", "2651"},
    {"gorące_źródła", "2668"},
    {"młoty", "2692"},
    {"skrzyżowane_miecze", "2694"},
    {"koło_zębate", "2699"},
    {"nożyczki", "2702"},
    {"pióro_z_czarną_stalówką", "2712"},
    {"znak_mnożenia", "2716"},
    {"gwiazdka_ośmioramienna", "2733"},
    {"gwiazda_ośmioramienna", "2734"},
    {"płatek_śniegu", "2744"},
    {"biały_znak_zapytania", "2754"},
    {"biały_wykrzyknik", "2755"},
    {"wykrzyknik_w_kształcie_serca", "2763"},
    {"strzałka_w_prawo_skręcająca_w_górę", "2934"},
    {"strzałka_w_prawo_skręcająca_w_dół", "2935"},
    {"japoński_przycisk_gratulacje", "3297"},
    {"japoński_przycisk_tajemnica", "3299"},
    {"podwójny_wykrzyknik", "203C"},
    {"strzałka_zakręcona_w_lewo", "21A9"},
    {"strzałka_zakręcona_w_prawo", "21AA"},
    {"wysuń", "23CF"},
    {"przycisk_szybkiego_przewijania_do_tyłu", "23EA"},
    {"przycisk_szybko_w_górę", "23EB"},
    {"przycisk_szybko_w_dół", "23EC"},
    {"następny_utwór", "23ED"},
    {"poprzedni_utwór", "23EE"},
    {"odtwórz_lub_wstrzymaj", "23EF"},
    {"klepsydra_z_przesypującym_się_piaskiem", "23F3"},
    {"koło_z_literą_m", "24C2"},
    {"mały_czarny_kwadrat", "25AA"},
    {"mały_biały_kwadrat", "25AB"},
    {"odtwórz", "25B6"},
    {"średni_biały_kwadrat", "25FB"},
    {"średni_czarny_kwadrat", "25FC"},
    {"mały_średni_biały_kwadrat", "25FD"},
    {"mały_średni_czarny_kwadrat", "25FE"},
    {"palec_wskazujący_w_górę", "261D"},
    {"palec_wskazujący_w_górę_kolor1", "261D-1F3FB"},
    {"palec_wskazujący_w_górę_kolor2", "261D-1F3FC"},
    {"palec_wskazujący_w_górę_kolor3", "261D-1F3FD"},
    {"palec_wskazujący_w_górę_kolor4", "261D-1F3FE"},
    {"palec_wskazujący_w_górę_kolor5", "261D-1F3FF"},
    {"gwiazda_i_półksiężyc", "262A"},
    {"uśmiechnięta_twarz", "263A"},
    {"bliźnięta", "264A"},
    {"nieskończoność", "267E"},
    {"symbol_wózka_inwalidzkiego", "267F"},
    {"ostrzeżenie", "26A0"},
    {"wysokie_napięcie", "26A1"},
    {"symbol_transpłciowości", "26A7"},
    {"białe_koło", "26AA"},
    {"czarne_koło", "26AB"},
    {"piłka_nożna", "26BD"},
    {"bałwanek_bez_śniegu", "26C4"},
    {"słońce_zza_chmury", "26C5"},
    {"wężownik", "26CE"},
    {"kask_z_krzyżykiem", "26D1"},
    {"łańcuchy", "26D3"},
    {"świątynia_shinto", "26E9"},
    {"kościół", "26EA"},
    {"góra", "26F0"},
    {"parasol_plażowy_wbity_w_ziemię", "26F1"},
    {"flaga_w_dołku", "26F3"},
    {"żaglówka", "26F5"},
    {"but_z_łyżwą", "26F8"},
    {"osoba_kozłująca_piłkę", "26F9"},
    {"osoba_kozłująca_piłkę_kolor1", "26F9-1F3FB"},
    {"osoba_kozłująca_piłkę_kolor2", "26F9-1F3FC"},
    {"osoba_kozłująca_piłkę_kolor3", "26F9-1F3FD"},
    {"osoba_kozłująca_piłkę_kolor4", "26F9-1F3FE"},
    {"osoba_kozłująca_piłkę_kolor5", "26F9-1F3FF"},
    {"wzniesiona_pięść", "270A"},
    {"wzniesiona_pięść_kolor1", "270A-1F3FB"},
    {"wzniesiona_pięść_kolor2", "270A-1F3FC"},
    {"wzniesiona_pięść_kolor3", "270A-1F3FD"},
    {"wzniesiona_pięść_kolor4", "270A-1F3FE"},
    {"wzniesiona_pięść_kolor5", "270A-1F3FF"},
    {"wzniesiona_dłoń", "270B"},
    {"wzniesiona_dłoń_kolor1", "270B-1F3FB"},
    {"wzniesiona_dłoń_kolor2", "270B-1F3FC"},
    {"wzniesiona_dłoń_kolor3", "270B-1F3FD"},
    {"wzniesiona_dłoń_kolor4", "270B-1F3FE"},
    {"wzniesiona_dłoń_kolor5", "270B-1F3FF"},
    {"pisząca_dłoń", "270D"},
    {"pisząca_dłoń_kolor1", "270D-1F3FB"},
    {"pisząca_dłoń_kolor2", "270D-1F3FC"},
    {"pisząca_dłoń_kolor3", "270D-1F3FD"},
    {"pisząca_dłoń_kolor4", "270D-1F3FE"},
    {"pisząca_dłoń_kolor5", "270D-1F3FF"},
    {"ołówek", "270F"},
    {"krzyż_łaciński", "271D"},
    {"znak_krzyżyka", "274C"},
    {"przycisk_z_krzyżykiem", "274E"},
    {"strzałka_w_prawo", "27A1"},
    {"pętla", "27B0"},
    {"podwójna_pętla", "27BF"},
    {"strzałka_w_lewo", "2B05"},
    {"strzałka_w_górę", "2B06"},
    {"strzałka_w_dół", "2B07"},
    {"duży_czarny_kwadrat", "2B1B"},
    {"duży_biały_kwadrat", "2B1C"},
    {"gruby_czerwony_okrąg", "2B55"},
    {"początek_partii_wokalnej", "303D"},
    {"madżong_czerwony_smok", "1F004"},
    {"dżoker", "1F0CF"},
    {"wskaźnik_regionalny_a", "1F1E6"},
    {"wskaźnik_regionalny_b", "1F1E7"},
    {"wskaźnik_regionalny_c", "1F1E8"},
    {"wskaźnik_regionalny_d", "1F1E9"},
    {"wskaźnik_regionalny_e", "1F1EA"},
    {"wskaźnik_regionalny_f", "1F1EB"},
    {"wskaźnik_regionalny_g", "1F1EC"},
    {"wskaźnik_regionalny_h", "1F1ED"},
    {"wskaźnik_regionalny_i", "1F1EE"},
    {"wskaźnik_regionalny_j", "1F1EF"},
    {"wskaźnik_regionalny_k", "1F1F0"},
    {"wskaźnik_regionalny_l", "1F1F1"},
    {"wskaźnik_regionalny_m", "1F1F2"},
    {"wskaźnik_regionalny_n", "1F1F3"},
    {"wskaźnik_regionalny_o", "1F1F4"},
    {"wskaźnik_regionalny_p", "1F1F5"},
    {"wskaźnik_regionalny_q", "1F1F6"},
    {"wskaźnik_regionalny_r", "1F1F7"},
    {"wskaźnik_regionalny_s", "1F1F8"},
    {"wskaźnik_regionalny_t", "1F1F9"},
    {"wskaźnik_regionalny_u", "1F1FA"},
    {"wskaźnik_regionalny_v", "1F1FB"},
    {"wskaźnik_regionalny_w", "1F1FC"},
    {"wskaźnik_regionalny_x", "1F1FD"},
    {"wskaźnik_regionalny_y", "1F1FE"},
    {"wskaźnik_regionalny_z", "1F1FF"},
    {"japoński_przycisk_tutaj", "1F201"},
    {"japoński_przycisk_usługa", "1F202"},
    {"japoński_przycisk_bezpłatne", "1F21A"},
    {"japoński_przycisk_zarezerwowane", "1F22F"},
    {"japoński_przycisk_zabronione", "1F232"},
    {"japoński_przycisk_wolne_miejsce", "1F233"},
    {"japoński_przycisk_ocena_pozytywna", "1F234"},
    {"japoński_przycisk_brak_wolnych_miejsc", "1F235"},
    {"japoński_przycisk_płatne", "1F236"},
    {"ideogram_księżyca", "1F237"},
    {"japoński_przycisk_wniosek", "1F238"},
    {"japoński_przycisk_rabat", "1F239"},
    {"japoński_przycisk_czynne", "1F23A"},
    {"japoński_przycisk_okazja", "1F250"},
    {"japoński_przycisk_dozwolone", "1F251"},
    {"złożony_parasol", "1F302"},
    {"wschód_słońca_zza_gór", "1F304"},
    {"wschód_słońca", "1F305"},
    {"zachód_słońca", "1F307"},
    {"tęcza", "1F308"},
    {"most_nocą", "1F309"},
    {"kula_ziemska_przedstawiająca_europę_i_afrykę", "1F30D"},
    {"kula_ziemska_przedstawiająca_ameryki", "1F30E"},
    {"kula_ziemska_przedstawiająca_azję_i_australię", "1F30F"},
    {"kula_ziemska_z_południkami", "1F310"},
    {"nów", "1F311"},
    {"narastający_półksiężyc", "1F312"},
    {"pierwsza_kwadra_księżyca", "1F313"},
    {"narastający_księżyc_między_pierwszą_kwadrą_a_pełnią", "1F314"},
    {"pełnia", "1F315"},
    {"zanikający_księżyc_między_pełnią_a_ostatnią_kwadrą", "1F316"},
    {"ostatnia_kwadra_księżyca", "1F317"},
    {"zanikający_półksiężyc", "1F318"},
    {"półksiężyc", "1F319"},
    {"nów_z_twarzą", "1F31A"},
    {"pierwsza_kwadra_księżyca_z_twarzą", "1F31B"},
    {"ostatnia_kwadra_księżyca_z_twarzą", "1F31C"},
    {"pełnia_z_twarzą", "1F31D"},
    {"słońce_z_twarzą", "1F31E"},
    {"błyszcząca_gwiazda", "1F31F"},
    {"spadająca_gwiazda", "1F320"},
    {"słońce_za_chmurką", "1F324"},
    {"słońce_za_chmurą", "1F325"},
    {"słońce_za_chmurą_i_deszcz", "1F326"},
    {"chmura_i_śnieg", "1F328"},
    {"mgła", "1F32B"},
    {"dmuchająca_twarz_wiatru", "1F32C"},
    {"drzewo_liściaste", "1F333"},
    {"kwiat_wiśni", "1F338"},
    {"róża", "1F339"},
    {"słonecznik", "1F33B"},
    {"pędy_ryżu", "1F33E"},
    {"zioło", "1F33F"},
    {"liść_klonowy", "1F341"},
    {"spadające_liście", "1F342"},
    {"liście_na_wietrze", "1F343"},
    {"bakłażan", "1F346"},
    {"czerwone_jabłko", "1F34E"},
    {"zielone_jabłko", "1F34F"},
    {"wiśnie", "1F352"},
    {"mięso_z_kością", "1F356"},
    {"krakers_ryżowy", "1F358"},
    {"kulka_ryżowa", "1F359"},
    {"gotowany_ryż", "1F35A"},
    {"ryż_curry", "1F35B"},
    {"miska_parującego_jedzenia", "1F35C"},
    {"smażona_krewetka", "1F364"},
    {"lody_włoskie", "1F366"},
    {"pączek_z_dziurką", "1F369"},
    {"pudełko_bento", "1F371"},
    {"widelec_i_nóż", "1F374"},
    {"filiżanka_bez_uszka", "1F375"},
    {"stukające_się_kufle_piwa", "1F37B"},
    {"talerz_z_nożem_i_widelcem", "1F37D"},
    {"butelka_z_wystrzelającym_korkiem", "1F37E"},
    {"wstążka", "1F380"},
    {"choinka_bożonarodzeniowa", "1F384"},
    {"święty_mikołaj", "1F385"},
    {"święty_mikołaj_kolor1", "1F385-1F3FB"},
    {"święty_mikołaj_kolor2", "1F385-1F3FC"},
    {"święty_mikołaj_kolor3", "1F385-1F3FD"},
    {"święty_mikołaj_kolor4", "1F385-1F3FE"},
    {"święty_mikołaj_kolor5", "1F385-1F3FF"},
    {"skrzyżowane_flagi", "1F38C"},
    {"japońska_dekoracja_kadomatsu", "1F38D"},
    {"japońskie_lalki", "1F38E"},
    {"flaga_w_kształcie_karpia", "1F38F"},
    {"ceremonia_oglądania_księżyca", "1F391"},
    {"wstążka_pamięci", "1F397"},
    {"suwak_miksera_dźwięku", "1F39A"},
    {"pokrętła", "1F39B"},
    {"taśma_filmowa", "1F39E"},
    {"bilety_wstępu", "1F39F"},
    {"koń_z_karuzeli", "1F3A0"},
    {"diabelski_młyn", "1F3A1"},
    {"kolejka_górska", "1F3A2"},
    {"wędka", "1F3A3"},
    {"słuchawki", "1F3A7"},
    {"strzał_w_10", "1F3AF"},
    {"kręgle", "1F3B3"},
    {"trąbka", "1F3BA"},
    {"koszykówka", "1F3C0"},
    {"flaga_w_szachownicę", "1F3C1"},
    {"biegnąca_osoba", "1F3C3"},
    {"biegnąca_osoba_kolor1", "1F3C3-1F3FB"},
    {"biegnąca_osoba_kolor2", "1F3C3-1F3FC"},
    {"biegnąca_osoba_kolor3", "1F3C3-1F3FD"},
    {"biegnąca_osoba_kolor4", "1F3C3-1F3FE"},
    {"biegnąca_osoba_kolor5", "1F3C3-1F3FF"},
    {"surfująca_osoba", "1F3C4"},
    {"surfująca_osoba_kolor1", "1F3C4-1F3FB"},
    {"surfująca_osoba_kolor2", "1F3C4-1F3FC"},
    {"surfująca_osoba_kolor3", "1F3C4-1F3FD"},
    {"surfująca_osoba_kolor4", "1F3C4-1F3FE"},
    {"surfująca_osoba_kolor5", "1F3C4-1F3FF"},
    {"wyścigi_konne", "1F3C7"},
    {"wyścigi_konne_kolor1", "1F3C7-1F3FB"},
    {"wyścigi_konne_kolor2", "1F3C7-1F3FC"},
    {"wyścigi_konne_kolor3", "1F3C7-1F3FD"},
    {"wyścigi_konne_kolor4", "1F3C7-1F3FE"},
    {"wyścigi_konne_kolor5", "1F3C7-1F3FF"},
    {"futbol_amerykański", "1F3C8"},
    {"piłka_do_rugby", "1F3C9"},
    {"pływająca_osoba", "1F3CA"},
    {"pływająca_osoba_kolor1", "1F3CA-1F3FB"},
    {"pływająca_osoba_kolor2", "1F3CA-1F3FC"},
    {"pływająca_osoba_kolor3", "1F3CA-1F3FD"},
    {"pływająca_osoba_kolor4", "1F3CA-1F3FE"},
    {"pływająca_osoba_kolor5", "1F3CA-1F3FF"},
    {"osoba_podnosząca_ciężary", "1F3CB"},
    {"osoba_podnosząca_ciężary_kolor1", "1F3CB-1F3FB"},
    {"osoba_podnosząca_ciężary_kolor2", "1F3CB-1F3FC"},
    {"osoba_podnosząca_ciężary_kolor3", "1F3CB-1F3FD"},
    {"osoba_podnosząca_ciężary_kolor4", "1F3CB-1F3FE"},
    {"osoba_podnosząca_ciężary_kolor5", "1F3CB-1F3FF"},
    {"osoba_grająca_w_golfa", "1F3CC"},
    {"osoba_grająca_w_golfa_kolor1", "1F3CC-1F3FB"},
    {"osoba_grająca_w_golfa_kolor2", "1F3CC-1F3FC"},
    {"osoba_grająca_w_golfa_kolor3", "1F3CC-1F3FD"},
    {"osoba_grająca_w_golfa_kolor4", "1F3CC-1F3FE"},
    {"osoba_grająca_w_golfa_kolor5", "1F3CC-1F3FF"},
    {"samochód_wyścigowy", "1F3CE"},
    {"siatkówka", "1F3D0"},
    {"tenis_stołowy", "1F3D3"},
    {"góra_z_czapą_lodową", "1F3D4"},
    {"parasol_na_plaży", "1F3D6"},
    {"dźwig_budowlany", "1F3D7"},
    {"japońska_poczta", "1F3E3"},
    {"urząd_pocztowy", "1F3E4"},
    {"hotel_miłości", "1F3E9"},
    {"sklep_całodobowy", "1F3EA"},
    {"szkoła", "1F3EB"},
    {"japoński_zamek", "1F3EF"},
    {"biała_flaga", "1F3F3"},
    {"łuk_i_strzała", "1F3F9"},
    {"karnacja_średnio_jasna", "1F3FC"},
    {"karnacja_średnia", "1F3FD"},
    {"karnacja_średnio_ciemna", "1F3FE"},
    {"wół", "1F402"},
    {"wół_domowy", "1F403"},
    {"królik", "1F407"},
    {"ślimak", "1F40C"},
    {"wąż", "1F40D"},
    {"koń", "1F40E"},
    {"biały_baran", "1F40F"},
    {"małpa", "1F412"},
    {"świnia", "1F416"},
    {"słoń", "1F418"},
    {"ośmiornica", "1F419"},
    {"gąsienica", "1F41B"},
    {"mrówka", "1F41C"},
    {"pszczoła", "1F41D"},
    {"żółw", "1F422"},
    {"wykluwające_się_kurczątko", "1F423"},
    {"kurczątko", "1F424"},
    {"kurczątko_od_przodu", "1F425"},
    {"wielbłąd", "1F42A"},
    {"wielbłąd_dwugarbny", "1F42B"},
    {"głowa_myszy", "1F42D"},
    {"głowa_krowy", "1F42E"},
    {"głowa_tygrysa", "1F42F"},
    {"głowa_królika", "1F430"},
    {"głowa_kota", "1F431"},
    {"głowa_smoka", "1F432"},
    {"wieloryb_tryskający_wodą", "1F433"},
    {"głowa_konia", "1F434"},
    {"głowa_małpy", "1F435"},
    {"głowa_psa", "1F436"},
    {"głowa_świni", "1F437"},
    {"głowa_żaby", "1F438"},
    {"głowa_chomika", "1F439"},
    {"głowa_wilka", "1F43A"},
    {"głowa_niedźwiedzia", "1F43B"},
    {"głowa_pandy", "1F43C"},
    {"ryj_świni", "1F43D"},
    {"ślady_łap", "1F43E"},
    {"pręgowiec", "1F43F"},
    {"język", "1F445"},
    {"dłoń_z_palcem_wskazującym_w_górę", "1F446"},
    {"dłoń_z_palcem_wskazującym_w_górę_kolor1", "1F446-1F3FB"},
    {"dłoń_z_palcem_wskazującym_w_górę_kolor2", "1F446-1F3FC"},
    {"dłoń_z_palcem_wskazującym_w_górę_kolor3", "1F446-1F3FD"},
    {"dłoń_z_palcem_wskazującym_w_górę_kolor4", "1F446-1F3FE"},
    {"dłoń_z_palcem_wskazującym_w_górę_kolor5", "1F446-1F3FF"},
    {"dłoń_z_palcem_wskazującym_w_dół", "1F447"},
    {"dłoń_z_palcem_wskazującym_w_dół_kolor1", "1F447-1F3FB"},
    {"dłoń_z_palcem_wskazującym_w_dół_kolor2", "1F447-1F3FC"},
    {"dłoń_z_palcem_wskazującym_w_dół_kolor3", "1F447-1F3FD"},
    {"dłoń_z_palcem_wskazującym_w_dół_kolor4", "1F447-1F3FE"},
    {"dłoń_z_palcem_wskazującym_w_dół_kolor5", "1F447-1F3FF"},
    {"dłoń_z_palcem_wskazującym_w_lewo", "1F448"},
    {"dłoń_z_palcem_wskazującym_w_lewo_kolor1", "1F448-1F3FB"},
    {"dłoń_z_palcem_wskazującym_w_lewo_kolor2", "1F448-1F3FC"},
    {"dłoń_z_palcem_wskazującym_w_lewo_kolor3", "1F448-1F3FD"},
    {"dłoń_z_palcem_wskazującym_w_lewo_kolor4", "1F448-1F3FE"},
    {"dłoń_z_palcem_wskazującym_w_lewo_kolor5", "1F448-1F3FF"},
    {"dłoń_z_palcem_wskazującym_w_prawo", "1F449"},
    {"dłoń_z_palcem_wskazującym_w_prawo_kolor1", "1F449-1F3FB"},
    {"dłoń_z_palcem_wskazującym_w_prawo_kolor2", "1F449-1F3FC"},
    {"dłoń_z_palcem_wskazującym_w_prawo_kolor3", "1F449-1F3FD"},
    {"dłoń_z_palcem_wskazującym_w_prawo_kolor4", "1F449-1F3FE"},
    {"dłoń_z_palcem_wskazującym_w_prawo_kolor5", "1F449-1F3FF"},
    {"pięść_od_przodu", "1F44A"},
    {"pięść_od_przodu_kolor1", "1F44A-1F3FB"},
    {"pięść_od_przodu_kolor2", "1F44A-1F3FC"},
    {"pięść_od_przodu_kolor3", "1F44A-1F3FD"},
    {"pięść_od_przodu_kolor4", "1F44A-1F3FE"},
    {"pięść_od_przodu_kolor5", "1F44A-1F3FF"},
    {"machająca_dłoń", "1F44B"},
    {"machająca_dłoń_kolor1", "1F44B-1F3FB"},
    {"machająca_dłoń_kolor2", "1F44B-1F3FC"},
    {"machająca_dłoń_kolor3", "1F44B-1F3FD"},
    {"machająca_dłoń_kolor4", "1F44B-1F3FE"},
    {"machająca_dłoń_kolor5", "1F44B-1F3FF"},
    {"dłoń_z_gestem_ok", "1F44C"},
    {"dłoń_z_gestem_ok_kolor1", "1F44C-1F3FB"},
    {"dłoń_z_gestem_ok_kolor2", "1F44C-1F3FC"},
    {"dłoń_z_gestem_ok_kolor3", "1F44C-1F3FD"},
    {"dłoń_z_gestem_ok_kolor4", "1F44C-1F3FE"},
    {"dłoń_z_gestem_ok_kolor5", "1F44C-1F3FF"},
    {"kciuk_w_górę", "1F44D"},
    {"kciuk_w_górę_kolor1", "1F44D-1F3FB"},
    {"kciuk_w_górę_kolor2", "1F44D-1F3FC"},
    {"kciuk_w_górę_kolor3", "1F44D-1F3FD"},
    {"kciuk_w_górę_kolor4", "1F44D-1F3FE"},
    {"kciuk_w_górę_kolor5", "1F44D-1F3FF"},
    {"kciuk_w_dół", "1F44E"},
    {"kciuk_w_dół_kolor1", "1F44E-1F3FB"},
    {"kciuk_w_dół_kolor2", "1F44E-1F3FC"},
    {"kciuk_w_dół_kolor3", "1F44E-1F3FD"},
    {"kciuk_w_dół_kolor4", "1F44E-1F3FE"},
    {"kciuk_w_dół_kolor5", "1F44E-1F3FF"},
    {"otwarte_dłonie", "1F450"},
    {"otwarte_dłonie_kolor1", "1F450-1F3FB"},
    {"otwarte_dłonie_kolor2", "1F450-1F3FC"},
    {"otwarte_dłonie_kolor3", "1F450-1F3FD"},
    {"otwarte_dłonie_kolor4", "1F450-1F3FE"},
    {"otwarte_dłonie_kolor5", "1F450-1F3FF"},
    {"dżinsy", "1F456"},
    {"but_męski", "1F45E"},
    {"sandał_damski", "1F461"},
    {"ślady_stóp", "1F463"},
    {"chłopiec", "1F466"},
    {"chłopiec_kolor1", "1F466-1F3FB"},
    {"chłopiec_kolor2", "1F466-1F3FC"},
    {"chłopiec_kolor3", "1F466-1F3FD"},
    {"chłopiec_kolor4", "1F466-1F3FE"},
    {"chłopiec_kolor5", "1F466-1F3FF"},
    {"mężczyzna", "1F468"},
    {"mężczyzna_kolor1", "1F468-1F3FB"},
    {"mężczyzna_kolor2", "1F468-1F3FC"},
    {"mężczyzna_kolor3", "1F468-1F3FD"},
    {"mężczyzna_kolor4", "1F468-1F3FE"},
    {"mężczyzna_kolor5", "1F468-1F3FF"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce", "1F46B"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor1", "1F46B-1F3FB"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor2", "1F46B-1F3FC"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor3", "1F46B-1F3FD"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor4", "1F46B-1F3FE"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor5", "1F46B-1F3FF"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mężczyzna_i_kobieta_trzymający_się_za_ręce_kolor5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce", "1F46C"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor1", "1F46C-1F3FB"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor2", "1F46C-1F3FC"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor3", "1F46C-1F3FD"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor4", "1F46C-1F3FE"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor5", "1F46C-1F3FF"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"dwóch_mężczyzn_trzymających_się_za_ręce_kolor5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"kobiety_trzymające_się_za_ręce", "1F46D"},
    {"kobiety_trzymające_się_za_ręce_kolor1", "1F46D-1F3FB"},
    {"kobiety_trzymające_się_za_ręce_kolor2", "1F46D-1F3FC"},
    {"kobiety_trzymające_się_za_ręce_kolor3", "1F46D-1F3FD"},
    {"kobiety_trzymające_się_za_ręce_kolor4", "1F46D-1F3FE"},
    {"kobiety_trzymające_się_za_ręce_kolor5", "1F46D-1F3FF"},
    {"kobiety_trzymające_się_za_ręce_kolor1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"kobiety_trzymające_się_za_ręce_kolor1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"kobiety_trzymające_się_za_ręce_kolor1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"kobiety_trzymające_się_za_ręce_kolor1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"kobiety_trzymające_się_za_ręce_kolor2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"kobiety_trzymające_się_za_ręce_kolor2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"kobiety_trzymające_się_za_ręce_kolor2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"kobiety_trzymające_się_za_ręce_kolor2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"kobiety_trzymające_się_za_ręce_kolor3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"kobiety_trzymające_się_za_ręce_kolor3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"kobiety_trzymające_się_za_ręce_kolor3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"kobiety_trzymające_się_za_ręce_kolor3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"kobiety_trzymające_się_za_ręce_kolor4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"kobiety_trzymające_się_za_ręce_kolor4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"kobiety_trzymające_się_za_ręce_kolor4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"kobiety_trzymające_się_za_ręce_kolor4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"kobiety_trzymające_się_za_ręce_kolor5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"kobiety_trzymające_się_za_ręce_kolor5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"kobiety_trzymające_się_za_ręce_kolor5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"kobiety_trzymające_się_za_ręce_kolor5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"osoby_z_uszami_królika_na_przyjęciu", "1F46F"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor1", "1F46F-1F3FB"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor2", "1F46F-1F3FC"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor3", "1F46F-1F3FD"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor4", "1F46F-1F3FE"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor5", "1F46F-1F3FF"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"osoby_z_uszami_królika_na_przyjęciu_kolor5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"osoba_z_włosami_blond", "1F471"},
    {"osoba_z_włosami_blond_kolor1", "1F471-1F3FB"},
    {"osoba_z_włosami_blond_kolor2", "1F471-1F3FC"},
    {"osoba_z_włosami_blond_kolor3", "1F471-1F3FD"},
    {"osoba_z_włosami_blond_kolor4", "1F471-1F3FE"},
    {"osoba_z_włosami_blond_kolor5", "1F471-1F3FF"},
    {"osoba_w_chińskiej_czapce", "1F472"},
    {"osoba_w_chińskiej_czapce_kolor1", "1F472-1F3FB"},
    {"osoba_w_chińskiej_czapce_kolor2", "1F472-1F3FC"},
    {"osoba_w_chińskiej_czapce_kolor3", "1F472-1F3FD"},
    {"osoba_w_chińskiej_czapce_kolor4", "1F472-1F3FE"},
    {"osoba_w_chińskiej_czapce_kolor5", "1F472-1F3FF"},
    {"starszy_mężczyzna", "1F474"},
    {"starszy_mężczyzna_kolor1", "1F474-1F3FB"},
    {"starszy_mężczyzna_kolor2", "1F474-1F3FC"},
    {"starszy_mężczyzna_kolor3", "1F474-1F3FD"},
    {"starszy_mężczyzna_kolor4", "1F474-1F3FE"},
    {"starszy_mężczyzna_kolor5", "1F474-1F3FF"},
    {"niemowlę", "1F476"},
    {"niemowlę_kolor1", "1F476-1F3FB"},
    {"niemowlę_kolor2", "1F476-1F3FC"},
    {"niemowlę_kolor3", "1F476-1F3FD"},
    {"niemowlę_kolor4", "1F476-1F3FE"},
    {"niemowlę_kolor5", "1F476-1F3FF"},
    {"księżniczka", "1F478"},
    {"księżniczka_kolor1", "1F478-1F3FB"},
    {"księżniczka_kolor2", "1F478-1F3FC"},
    {"księżniczka_kolor3", "1F478-1F3FD"},
    {"księżniczka_kolor4", "1F478-1F3FE"},
    {"księżniczka_kolor5", "1F478-1F3FF"},
    {"aniołek", "1F47C"},
    {"aniołek_kolor1", "1F47C-1F3FB"},
    {"aniołek_kolor2", "1F47C-1F3FC"},
    {"aniołek_kolor3", "1F47C-1F3FD"},
    {"aniołek_kolor4", "1F47C-1F3FE"},
    {"aniołek_kolor5", "1F47C-1F3FF"},
    {"kosmiczny_potwór", "1F47E"},
    {"osoba_z_wystawioną_dłonią", "1F481"},
    {"osoba_z_wystawioną_dłonią_kolor1", "1F481-1F3FB"},
    {"osoba_z_wystawioną_dłonią_kolor2", "1F481-1F3FC"},
    {"osoba_z_wystawioną_dłonią_kolor3", "1F481-1F3FD"},
    {"osoba_z_wystawioną_dłonią_kolor4", "1F481-1F3FE"},
    {"osoba_z_wystawioną_dłonią_kolor5", "1F481-1F3FF"},
    {"tańcząca_kobieta", "1F483"},
    {"tańcząca_kobieta_kolor1", "1F483-1F3FB"},
    {"tańcząca_kobieta_kolor2", "1F483-1F3FC"},
    {"tańcząca_kobieta_kolor3", "1F483-1F3FD"},
    {"tańcząca_kobieta_kolor4", "1F483-1F3FE"},
    {"tańcząca_kobieta_kolor5", "1F483-1F3FF"},
    {"osoba_podczas_masażu", "1F486"},
    {"osoba_podczas_masażu_kolor1", "1F486-1F3FB"},
    {"osoba_podczas_masażu_kolor2", "1F486-1F3FC"},
    {"osoba_podczas_masażu_kolor3", "1F486-1F3FD"},
    {"osoba_podczas_masażu_kolor4", "1F486-1F3FE"},
    {"osoba_podczas_masażu_kolor5", "1F486-1F3FF"},
    {"osoba_podczas_strzyżenia", "1F487"},
    {"osoba_podczas_strzyżenia_kolor1", "1F487-1F3FB"},
    {"osoba_podczas_strzyżenia_kolor2", "1F487-1F3FC"},
    {"osoba_podczas_strzyżenia_kolor3", "1F487-1F3FD"},
    {"osoba_podczas_strzyżenia_kolor4", "1F487-1F3FE"},
    {"osoba_podczas_strzyżenia_kolor5", "1F487-1F3FF"},
    {"słup_fryzjerski", "1F488"},
    {"kapsułka", "1F48A"},
    {"ślad_po_pocałunku", "1F48B"},
    {"list_miłosny", "1F48C"},
    {"pierścionek", "1F48D"},
    {"kamień_szlachetny", "1F48E"},
    {"pocałunek", "1F48F"},
    {"pocałunek_kolor1", "1F48F-1F3FB"},
    {"pocałunek_kolor2", "1F48F-1F3FC"},
    {"pocałunek_kolor3", "1F48F-1F3FD"},
    {"pocałunek_kolor4", "1F48F-1F3FE"},
    {"pocałunek_kolor5", "1F48F-1F3FF"},
    {"pocałunek_kolor1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"pocałunek_kolor1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"pocałunek_kolor1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"pocałunek_kolor1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"pocałunek_kolor2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"pocałunek_kolor2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"pocałunek_kolor2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"pocałunek_kolor2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"pocałunek_kolor3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"pocałunek_kolor3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"pocałunek_kolor3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"pocałunek_kolor3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"pocałunek_kolor4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"pocałunek_kolor4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"pocałunek_kolor4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"pocałunek_kolor4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"pocałunek_kolor5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"pocałunek_kolor5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"pocałunek_kolor5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"pocałunek_kolor5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bukiet_kwiatów", "1F490"},
    {"ślub", "1F492"},
    {"bijące_serce", "1F493"},
    {"złamane_serce", "1F494"},
    {"migoczące_serce", "1F496"},
    {"rosnące_serce", "1F497"},
    {"serce_przebite_strzałą", "1F498"},
    {"żółte_serce", "1F49B"},
    {"serce_ze_wstążką", "1F49D"},
    {"wirujące_serca", "1F49E"},
    {"romb_z_kropką", "1F4A0"},
    {"żarówka", "1F4A1"},
    {"szybkie_poruszanie_się", "1F4A8"},
    {"napięty_biceps", "1F4AA"},
    {"napięty_biceps_kolor1", "1F4AA-1F3FB"},
    {"napięty_biceps_kolor2", "1F4AA-1F3FC"},
    {"napięty_biceps_kolor3", "1F4AA-1F3FD"},
    {"napięty_biceps_kolor4", "1F4AA-1F3FE"},
    {"napięty_biceps_kolor5", "1F4AA-1F3FF"},
    {"zawroty_głowy", "1F4AB"},
    {"dymek_myśli", "1F4AD"},
    {"biały_kwiat", "1F4AE"},
    {"sto_punktów", "1F4AF"},
    {"worek_z_pieniędzmi", "1F4B0"},
    {"uskrzydlone_pieniądze", "1F4B8"},
    {"zawinięta_strona", "1F4C3"},
    {"wykres_słupkowy", "1F4CA"},
    {"podkładka_do_pisania", "1F4CB"},
    {"okrągła_pinezka", "1F4CD"},
    {"karty_zakładek", "1F4D1"},
    {"notes_z_dekoracyjną_okładką", "1F4D4"},
    {"zamknięta_książka", "1F4D5"},
    {"otwarta_książka", "1F4D6"},
    {"zielona_książka", "1F4D7"},
    {"niebieska_książka", "1F4D8"},
    {"pomarańczowa_książka", "1F4D9"},
    {"książki", "1F4DA"},
    {"zwój", "1F4DC"},
    {"słuchawka_telefoniczna", "1F4DE"},
    {"głośnik", "1F4E2"},
    {"koperta_przychodząca", "1F4E8"},
    {"koperta_ze_strzałką", "1F4E9"},
    {"zamknięta_skrzynka_pocztowa_z_opuszczoną_flagą", "1F4EA"},
    {"zamknięta_skrzynka_pocztowa_z_podniesioną_flagą", "1F4EB"},
    {"otwarta_skrzynka_pocztowa_z_podniesioną_flagą", "1F4EC"},
    {"otwarta_skrzynka_pocztowa_z_opuszczoną_flagą", "1F4ED"},
    {"trąbka_pocztowa", "1F4EF"},
    {"telefon_komórkowy", "1F4F1"},
    {"telefon_komórkowy_ze_strzałką", "1F4F2"},
    {"wyłączony_telefon_komórkowy", "1F4F4"},
    {"zakaz_korzystania_z_telefonów_komórkowych", "1F4F5"},
    {"siła_sygnału", "1F4F6"},
    {"aparat_fotograficzny_z_lampą_błyskową", "1F4F8"},
    {"przycisk_losowego_odtwarzania_utworów", "1F500"},
    {"pionowe_strzałki_zgodne_z_ruchem_wskazówek_zegara", "1F503"},
    {"przycisk_ze_strzałkami_przeciwnie_do_ruchu_wskazówek_zegara", "1F504"},
    {"przycisk_małej_jasności", "1F505"},
    {"przycisk_dużej_jasności", "1F506"},
    {"wyciszony_głośnik", "1F507"},
    {"niska_głośność_głośnika", "1F508"},
    {"średnia_głośność_głośnika", "1F509"},
    {"wysoka_głośność_głośnika", "1F50A"},
    {"zamknięta_kłódka_z_piórem", "1F50F"},
    {"zamknięta_kłódka_z_kluczem", "1F510"},
    {"zamknięta_kłódka", "1F512"},
    {"otwarta_kłódka", "1F513"},
    {"przekreślony_dzwonek", "1F515"},
    {"zakładka", "1F516"},
    {"strzałka_z_napisem_back", "1F519"},
    {"strzałka_z_napisem_end", "1F51A"},
    {"strzałka_z_napisem_on", "1F51B"},
    {"strzałka_z_napisem_soon", "1F51C"},
    {"strzałka_z_napisem_top", "1F51D"},
    {"zakaz_poniżej_osiemnastego_roku_życia", "1F51E"},
    {"wielkie_litery_alfabet_łaciński", "1F520"},
    {"małe_litery_alfabet_łaciński", "1F521"},
    {"litery_alfabet_łaciński", "1F524"},
    {"ogień", "1F525"},
    {"młotek", "1F528"},
    {"śruba_i_nakrętka", "1F529"},
    {"nóż_kuchenny", "1F52A"},
    {"pistolet_na_wodę", "1F52B"},
    {"kryształowa_kula", "1F52E"},
    {"gwiazda_sześcioramienna_z_kropką", "1F52F"},
    {"japoński_symbol_początkującego_kierowcy", "1F530"},
    {"emblemat_z_trójzębem", "1F531"},
    {"biały_kwadratowy_przycisk", "1F533"},
    {"czerwone_koło", "1F534"},
    {"niebieskie_koło", "1F535"},
    {"duży_pomarańczowy_romb", "1F536"},
    {"duży_niebieski_romb", "1F537"},
    {"mały_pomarańczowy_romb", "1F538"},
    {"mały_niebieski_romb", "1F539"},
    {"czerwony_trójkąt_skierowany_w_górę", "1F53A"},
    {"czerwony_trójkąt_skierowany_w_dół", "1F53B"},
    {"przycisk_w_górę", "1F53C"},
    {"przycisk_w_dół", "1F53D"},
    {"gołąb", "1F54A"},
    {"świeca", "1F56F"},
    {"otwór", "1F573"},
    {"lewitujący_mężczyzna_w_garniturze", "1F574"},
    {"lewitujący_mężczyzna_w_garniturze_kolor1", "1F574-1F3FB"},
    {"lewitujący_mężczyzna_w_garniturze_kolor2", "1F574-1F3FC"},
    {"lewitujący_mężczyzna_w_garniturze_kolor3", "1F574-1F3FD"},
    {"lewitujący_mężczyzna_w_garniturze_kolor4", "1F574-1F3FE"},
    {"lewitujący_mężczyzna_w_garniturze_kolor5", "1F574-1F3FF"},
    {"okulary_słoneczne", "1F576"},
    {"pająk", "1F577"},
    {"pajęczyna", "1F578"},
    {"dżojstik", "1F579"},
    {"tańczący_mężczyzna", "1F57A"},
    {"tańczący_mężczyzna_kolor1", "1F57A-1F3FB"},
    {"tańczący_mężczyzna_kolor2", "1F57A-1F3FC"},
    {"tańczący_mężczyzna_kolor3", "1F57A-1F3FD"},
    {"tańczący_mężczyzna_kolor4", "1F57A-1F3FE"},
    {"tańczący_mężczyzna_kolor5", "1F57A-1F3FF"},
    {"złączone_spinacze", "1F587"},
    {"długopis", "1F58A"},
    {"wieczne_pióro", "1F58B"},
    {"pędzel", "1F58C"},
    {"uniesiona_otwarta_dłoń", "1F590"},
    {"uniesiona_otwarta_dłoń_kolor1", "1F590-1F3FB"},
    {"uniesiona_otwarta_dłoń_kolor2", "1F590-1F3FC"},
    {"uniesiona_otwarta_dłoń_kolor3", "1F590-1F3FD"},
    {"uniesiona_otwarta_dłoń_kolor4", "1F590-1F3FE"},
    {"uniesiona_otwarta_dłoń_kolor5", "1F590-1F3FF"},
    {"środkowy_palec", "1F595"},
    {"środkowy_palec_kolor1", "1F595-1F3FB"},
    {"środkowy_palec_kolor2", "1F595-1F3FC"},
    {"środkowy_palec_kolor3", "1F595-1F3FD"},
    {"środkowy_palec_kolor4", "1F595-1F3FE"},
    {"środkowy_palec_kolor5", "1F595-1F3FF"},
    {"salut_wolkański", "1F596"},
    {"salut_wolkański_kolor1", "1F596-1F3FB"},
    {"salut_wolkański_kolor2", "1F596-1F3FC"},
    {"salut_wolkański_kolor3", "1F596-1F3FD"},
    {"salut_wolkański_kolor4", "1F596-1F3FE"},
    {"salut_wolkański_kolor5", "1F596-1F3FF"},
    {"pudełko_kartoteka", "1F5C3"},
    {"kosz_na_śmieci", "1F5D1"},
    {"kołonotatnik", "1F5D2"},
    {"ścisk", "1F5DC"},
    {"staroświecki_klucz", "1F5DD"},
    {"zwinięta_gazeta", "1F5DE"},
    {"mówiąca_głowa", "1F5E3"},
    {"prawostronny_dymek_złości", "1F5EF"},
    {"urna_wyborcza_z_głosem", "1F5F3"},
    {"mapa_świata", "1F5FA"},
    {"góra_fudżi", "1F5FB"},
    {"statua_wolności", "1F5FD"},
    {"szeroko_uśmiechnięta_twarz", "1F600"},
    {"szeroko_uśmiechnięta_twarz_o_roześmianych_oczach", "1F601"},
    {"twarz_ze_łzami_radości", "1F602"},
    {"uśmiechnięta_twarz_z_otwartymi_ustami", "1F603"},
    {"uśmiechnięta_twarz_z_otwartymi_ustami_i_roześmianymi_oczami", "1F604"},
    {"uśmiechnięta_twarz_z_otwartymi_ustami_oblana_zimnym_potem", "1F605"},
    {"uśmiechnięta_twarz_z_przymrużonymi_oczami", "1F606"},
    {"uśmiechnięta_twarz_z_aureolą", "1F607"},
    {"uśmiechnięta_twarz_z_rogami", "1F608"},
    {"twarz_puszczająca_oko", "1F609"},
    {"uśmiechnięta_twarz_o_roześmianych_oczach", "1F60A"},
    {"twarz_delektująca_się_pysznym_jedzeniem", "1F60B"},
    {"uśmiechnięta_twarz_z_oczami_w_kształcie_serca", "1F60D"},
    {"uśmiechnięta_twarz_w_okularach_przeciwsłonecznych", "1F60E"},
    {"twarz_z_uśmieszkiem", "1F60F"},
    {"zamyślona_twarz", "1F614"},
    {"zakłopotana_twarz", "1F616"},
    {"całująca_twarz", "1F617"},
    {"twarz_przesyłająca_całusa", "1F618"},
    {"całująca_twarz_o_roześmianych_oczach", "1F619"},
    {"całująca_twarz_z_zamkniętymi_oczami", "1F61A"},
    {"twarz_wystawiająca_język", "1F61B"},
    {"twarz_wystawiająca_język_i_puszczająca_oko", "1F61C"},
    {"twarz_ze_zmrużonymi_oczami_wystawiająca_język", "1F61D"},
    {"nadąsana_twarz", "1F621"},
    {"płacząca_twarz", "1F622"},
    {"prychająca_twarz", "1F624"},
    {"udręczona_twarz", "1F627"},
    {"znużona_twarz", "1F629"},
    {"zmęczona_twarz", "1F62B"},
    {"głośno_płacząca_twarz", "1F62D"},
    {"twarz_krzycząca_ze_strachu", "1F631"},
    {"twarz_z_rumieńcami", "1F633"},
    {"śpiąca_twarz", "1F634"},
    {"oszołomiona_twarz", "1F635"},
    {"uśmiechnięty_kot_o_wesołych_oczach", "1F638"},
    {"kot_ze_łzami_radości", "1F639"},
    {"uśmiechnięty_kot", "1F63A"},
    {"uśmiechnięty_kot_z_oczami_w_kształcie_serca", "1F63B"},
    {"kot_z_drwiącym_uśmiechem", "1F63C"},
    {"kot_przesyłający_całusa", "1F63D"},
    {"nadąsany_kot", "1F63E"},
    {"płaczący_kot", "1F63F"},
    {"lekko_uśmiechnięta_twarz", "1F642"},
    {"odwrócona_twarz", "1F643"},
    {"twarz_przewracająca_oczami", "1F644"},
    {"osoba_pokazująca_gest_nie", "1F645"},
    {"osoba_pokazująca_gest_nie_kolor1", "1F645-1F3FB"},
    {"osoba_pokazująca_gest_nie_kolor2", "1F645-1F3FC"},
    {"osoba_pokazująca_gest_nie_kolor3", "1F645-1F3FD"},
    {"osoba_pokazująca_gest_nie_kolor4", "1F645-1F3FE"},
    {"osoba_pokazująca_gest_nie_kolor5", "1F645-1F3FF"},
    {"osoba_pokazująca_gest_ok", "1F646"},
    {"osoba_pokazująca_gest_ok_kolor1", "1F646-1F3FB"},
    {"osoba_pokazująca_gest_ok_kolor2", "1F646-1F3FC"},
    {"osoba_pokazująca_gest_ok_kolor3", "1F646-1F3FD"},
    {"osoba_pokazująca_gest_ok_kolor4", "1F646-1F3FE"},
    {"osoba_pokazująca_gest_ok_kolor5", "1F646-1F3FF"},
    {"osoba_w_ukłonie", "1F647"},
    {"osoba_w_ukłonie_kolor1", "1F647-1F3FB"},
    {"osoba_w_ukłonie_kolor2", "1F647-1F3FC"},
    {"osoba_w_ukłonie_kolor3", "1F647-1F3FD"},
    {"osoba_w_ukłonie_kolor4", "1F647-1F3FE"},
    {"osoba_w_ukłonie_kolor5", "1F647-1F3FF"},
    {"małpa_zasłaniająca_oczy", "1F648"},
    {"małpa_zasłaniająca_uszy", "1F649"},
    {"małpa_zasłaniająca_pyszczek", "1F64A"},
    {"osoba_podnosząca_rękę", "1F64B"},
    {"osoba_podnosząca_rękę_kolor1", "1F64B-1F3FB"},
    {"osoba_podnosząca_rękę_kolor2", "1F64B-1F3FC"},
    {"osoba_podnosząca_rękę_kolor3", "1F64B-1F3FD"},
    {"osoba_podnosząca_rękę_kolor4", "1F64B-1F3FE"},
    {"osoba_podnosząca_rękę_kolor5", "1F64B-1F3FF"},
    {"wzniesione_ręce", "1F64C"},
    {"wzniesione_ręce_kolor1", "1F64C-1F3FB"},
    {"wzniesione_ręce_kolor2", "1F64C-1F3FC"},
    {"wzniesione_ręce_kolor3", "1F64C-1F3FD"},
    {"wzniesione_ręce_kolor4", "1F64C-1F3FE"},
    {"wzniesione_ręce_kolor5", "1F64C-1F3FF"},
    {"nadąsana_osoba", "1F64E"},
    {"nadąsana_osoba_kolor1", "1F64E-1F3FB"},
    {"nadąsana_osoba_kolor2", "1F64E-1F3FC"},
    {"nadąsana_osoba_kolor3", "1F64E-1F3FD"},
    {"nadąsana_osoba_kolor4", "1F64E-1F3FE"},
    {"nadąsana_osoba_kolor5", "1F64E-1F3FF"},
    {"złożone_ręce", "1F64F"},
    {"złożone_ręce_kolor1", "1F64F-1F3FB"},
    {"złożone_ręce_kolor2", "1F64F-1F3FC"},
    {"złożone_ręce_kolor3", "1F64F-1F3FD"},
    {"złożone_ręce_kolor4", "1F64F-1F3FE"},
    {"złożone_ręce_kolor5", "1F64F-1F3FF"},
    {"szybki_pociąg", "1F684"},
    {"szybki_pociąg_z_zaokrąglonym_przodem", "1F685"},
    {"pociąg", "1F686"},
    {"nadjeżdżający_autobus", "1F68D"},
    {"samochód_strażacki", "1F692"},
    {"samochód_policyjny", "1F693"},
    {"nadjeżdżający_radiowóz", "1F694"},
    {"taksówka", "1F695"},
    {"nadjeżdżająca_taksówka", "1F696"},
    {"samochód", "1F697"},
    {"nadjeżdżający_samochód", "1F698"},
    {"samochód_suv", "1F699"},
    {"samochód_dostawczy", "1F69A"},
    {"samochód_ciężarowy", "1F69B"},
    {"ciągnik", "1F69C"},
    {"kolej_górska", "1F69E"},
    {"górska_kolej_linowa", "1F6A0"},
    {"osoba_wiosłująca_w_łodzi", "1F6A3"},
    {"osoba_wiosłująca_w_łodzi_kolor1", "1F6A3-1F3FB"},
    {"osoba_wiosłująca_w_łodzi_kolor2", "1F6A3-1F3FC"},
    {"osoba_wiosłująca_w_łodzi_kolor3", "1F6A3-1F3FD"},
    {"osoba_wiosłująca_w_łodzi_kolor4", "1F6A3-1F3FE"},
    {"osoba_wiosłująca_w_łodzi_kolor5", "1F6A3-1F3FF"},
    {"ścigacz_wodny", "1F6A4"},
    {"flaga_trójkątna", "1F6A9"},
    {"znak_kosza_na_śmieci", "1F6AE"},
    {"zakaz_śmiecenia", "1F6AF"},
    {"zakaz_wjazdu_rowerów", "1F6B3"},
    {"osoba_na_rowerze_górskim", "1F6B5"},
    {"osoba_na_rowerze_górskim_kolor1", "1F6B5-1F3FB"},
    {"osoba_na_rowerze_górskim_kolor2", "1F6B5-1F3FC"},
    {"osoba_na_rowerze_górskim_kolor3", "1F6B5-1F3FD"},
    {"osoba_na_rowerze_górskim_kolor4", "1F6B5-1F3FE"},
    {"osoba_na_rowerze_górskim_kolor5", "1F6B5-1F3FF"},
    {"idąca_osoba", "1F6B6"},
    {"idąca_osoba_kolor1", "1F6B6-1F3FB"},
    {"idąca_osoba_kolor2", "1F6B6-1F3FC"},
    {"idąca_osoba_kolor3", "1F6B6-1F3FD"},
    {"idąca_osoba_kolor4", "1F6B6-1F3FE"},
    {"idąca_osoba_kolor5", "1F6B6-1F3FF"},
    {"dzieci_przechodzące_przez_jezdnię", "1F6B8"},
    {"toaleta_męska", "1F6B9"},
    {"symbol_niemowlęcia", "1F6BC"},
    {"osoba_biorąca_kąpiel", "1F6C0"},
    {"osoba_biorąca_kąpiel_kolor1", "1F6C0-1F3FB"},
    {"osoba_biorąca_kąpiel_kolor2", "1F6C0-1F3FC"},
    {"osoba_biorąca_kąpiel_kolor3", "1F6C0-1F3FD"},
    {"osoba_biorąca_kąpiel_kolor4", "1F6C0-1F3FE"},
    {"osoba_biorąca_kąpiel_kolor5", "1F6C0-1F3FF"},
    {"odbiór_bagażu", "1F6C4"},
    {"przechowalnia_bagażu", "1F6C5"},
    {"osoba_w_łóżku", "1F6CC"},
    {"osoba_w_łóżku_kolor1", "1F6CC-1F3FB"},
    {"osoba_w_łóżku_kolor2", "1F6CC-1F3FC"},
    {"osoba_w_łóżku_kolor3", "1F6CC-1F3FD"},
    {"osoba_w_łóżku_kolor4", "1F6CC-1F3FE"},
    {"osoba_w_łóżku_kolor5", "1F6CC-1F3FF"},
    {"łóżko", "1F6CF"},
    {"wózek_sklepowy", "1F6D2"},
    {"świątynia_hinduistyczna", "1F6D5"},
    {"zjeżdżalnia", "1F6DD"},
    {"koło_pojazdu", "1F6DE"},
    {"koło_ratunkowe", "1F6DF"},
    {"młot_i_klucz", "1F6E0"},
    {"motorówka", "1F6E5"},
    {"mały_samolot", "1F6E9"},
    {"statek_pasażerski", "1F6F3"},
    {"latający_talerz", "1F6F8"},
    {"pomarańczowe_koło", "1F7E0"},
    {"żółte_koło", "1F7E1"},
    {"zielone_koło", "1F7E2"},
    {"fioletowe_koło", "1F7E3"},
    {"brązowe_koło", "1F7E4"},
    {"pomarańczowy_kwadrat", "1F7E7"},
    {"żółty_kwadrat", "1F7E8"},
    {"brązowy_kwadrat", "1F7EB"},
    {"gruby_znak_równości", "1F7F0"},
    {"złączone_palce", "1F90C"},
    {"złączone_palce_kolor1", "1F90C-1F3FB"},
    {"złączone_palce_kolor2", "1F90C-1F3FC"},
    {"złączone_palce_kolor3", "1F90C-1F3FD"},
    {"złączone_palce_kolor4", "1F90C-1F3FE"},
    {"złączone_palce_kolor5", "1F90C-1F3FF"},
    {"białe_serce", "1F90D"},
    {"brązowe_serce", "1F90E"},
    {"dłoń_z_gestem_trochę", "1F90F"},
    {"dłoń_z_gestem_trochę_kolor1", "1F90F-1F3FB"},
    {"dłoń_z_gestem_trochę_kolor2", "1F90F-1F3FC"},
    {"dłoń_z_gestem_trochę_kolor3", "1F90F-1F3FD"},
    {"dłoń_z_gestem_trochę_kolor4", "1F90F-1F3FE"},
    {"dłoń_z_gestem_trochę_kolor5", "1F90F-1F3FF"},
    {"twarz_z_zamkiem_błyskawicznym_na_ustach", "1F910"},
    {"twarz_z_pieniędzmi_na_ustach", "1F911"},
    {"myśląca_twarz", "1F914"},
    {"twarz_z_bandażem", "1F915"},
    {"głowa_robota", "1F916"},
    {"gest_rogów", "1F918"},
    {"gest_rogów_kolor1", "1F918-1F3FB"},
    {"gest_rogów_kolor2", "1F918-1F3FC"},
    {"gest_rogów_kolor3", "1F918-1F3FD"},
    {"gest_rogów_kolor4", "1F918-1F3FE"},
    {"gest_rogów_kolor5", "1F918-1F3FF"},
    {"dłoń_w_geście_zadzwoń_do_mnie", "1F919"},
    {"dłoń_w_geście_zadzwoń_do_mnie_kolor1", "1F919-1F3FB"},
    {"dłoń_w_geście_zadzwoń_do_mnie_kolor2", "1F919-1F3FC"},
    {"dłoń_w_geście_zadzwoń_do_mnie_kolor3", "1F919-1F3FD"},
    {"dłoń_w_geście_zadzwoń_do_mnie_kolor4", "1F919-1F3FE"},
    {"dłoń_w_geście_zadzwoń_do_mnie_kolor5", "1F919-1F3FF"},
    {"wzniesiony_grzbiet_dłoni", "1F91A"},
    {"wzniesiony_grzbiet_dłoni_kolor1", "1F91A-1F3FB"},
    {"wzniesiony_grzbiet_dłoni_kolor2", "1F91A-1F3FC"},
    {"wzniesiony_grzbiet_dłoni_kolor3", "1F91A-1F3FD"},
    {"wzniesiony_grzbiet_dłoni_kolor4", "1F91A-1F3FE"},
    {"wzniesiony_grzbiet_dłoni_kolor5", "1F91A-1F3FF"},
    {"pięść_skierowana_w_lewo", "1F91B"},
    {"pięść_skierowana_w_lewo_kolor1", "1F91B-1F3FB"},
    {"pięść_skierowana_w_lewo_kolor2", "1F91B-1F3FC"},
    {"pięść_skierowana_w_lewo_kolor3", "1F91B-1F3FD"},
    {"pięść_skierowana_w_lewo_kolor4", "1F91B-1F3FE"},
    {"pięść_skierowana_w_lewo_kolor5", "1F91B-1F3FF"},
    {"pięść_skierowana_w_prawo", "1F91C"},
    {"pięść_skierowana_w_prawo_kolor1", "1F91C-1F3FB"},
    {"pięść_skierowana_w_prawo_kolor2", "1F91C-1F3FC"},
    {"pięść_skierowana_w_prawo_kolor3", "1F91C-1F3FD"},
    {"pięść_skierowana_w_prawo_kolor4", "1F91C-1F3FE"},
    {"pięść_skierowana_w_prawo_kolor5", "1F91C-1F3FF"},
    {"uścisk_dłoni", "1F91D"},
    {"uścisk_dłoni_kolor1", "1F91D-1F3FB"},
    {"uścisk_dłoni_kolor2", "1F91D-1F3FC"},
    {"uścisk_dłoni_kolor3", "1F91D-1F3FD"},
    {"uścisk_dłoni_kolor4", "1F91D-1F3FE"},
    {"uścisk_dłoni_kolor5", "1F91D-1F3FF"},
    {"uścisk_dłoni_kolor1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"uścisk_dłoni_kolor1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"uścisk_dłoni_kolor1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"uścisk_dłoni_kolor1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"uścisk_dłoni_kolor2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"uścisk_dłoni_kolor2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"uścisk_dłoni_kolor2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"uścisk_dłoni_kolor2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"uścisk_dłoni_kolor3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"uścisk_dłoni_kolor3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"uścisk_dłoni_kolor3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"uścisk_dłoni_kolor3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"uścisk_dłoni_kolor4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"uścisk_dłoni_kolor4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"uścisk_dłoni_kolor4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"uścisk_dłoni_kolor4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"uścisk_dłoni_kolor5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"uścisk_dłoni_kolor5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"uścisk_dłoni_kolor5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"uścisk_dłoni_kolor5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"skrzyżowane_palce", "1F91E"},
    {"skrzyżowane_palce_kolor1", "1F91E-1F3FB"},
    {"skrzyżowane_palce_kolor2", "1F91E-1F3FC"},
    {"skrzyżowane_palce_kolor3", "1F91E-1F3FD"},
    {"skrzyżowane_palce_kolor4", "1F91E-1F3FE"},
    {"skrzyżowane_palce_kolor5", "1F91E-1F3FF"},
    {"gest_kocham_cię", "1F91F"},
    {"gest_kocham_cię_kolor1", "1F91F-1F3FB"},
    {"gest_kocham_cię_kolor2", "1F91F-1F3FC"},
    {"gest_kocham_cię_kolor3", "1F91F-1F3FD"},
    {"gest_kocham_cię_kolor4", "1F91F-1F3FE"},
    {"gest_kocham_cię_kolor5", "1F91F-1F3FF"},
    {"głowa_w_kapeluszu_kowbojskim", "1F920"},
    {"twarz_z_mdłościami", "1F922"},
    {"tarza_się_ze_śmiechu", "1F923"},
    {"śliniąca_się_twarz", "1F924"},
    {"twarz_kłamcy", "1F925"},
    {"osoba_trzymająca_się_za_czoło", "1F926"},
    {"osoba_trzymająca_się_za_czoło_kolor1", "1F926-1F3FB"},
    {"osoba_trzymająca_się_za_czoło_kolor2", "1F926-1F3FC"},
    {"osoba_trzymająca_się_za_czoło_kolor3", "1F926-1F3FD"},
    {"osoba_trzymająca_się_za_czoło_kolor4", "1F926-1F3FE"},
    {"osoba_trzymająca_się_za_czoło_kolor5", "1F926-1F3FF"},
    {"kichająca_twarz", "1F927"},
    {"twarz_z_podniesioną_brwią", "1F928"},
    {"uciszająca_twarz", "1F92B"},
    {"twarz_z_ręką_na_ustach", "1F92D"},
    {"wymiotująca_twarz", "1F92E"},
    {"eksplodująca_głowa", "1F92F"},
    {"kobieta_w_ciąży", "1F930"},
    {"kobieta_w_ciąży_kolor1", "1F930-1F3FB"},
    {"kobieta_w_ciąży_kolor2", "1F930-1F3FC"},
    {"kobieta_w_ciąży_kolor3", "1F930-1F3FD"},
    {"kobieta_w_ciąży_kolor4", "1F930-1F3FE"},
    {"kobieta_w_ciąży_kolor5", "1F930-1F3FF"},
    {"karmienie_piersią", "1F931"},
    {"karmienie_piersią_kolor1", "1F931-1F3FB"},
    {"karmienie_piersią_kolor2", "1F931-1F3FC"},
    {"karmienie_piersią_kolor3", "1F931-1F3FD"},
    {"karmienie_piersią_kolor4", "1F931-1F3FE"},
    {"karmienie_piersią_kolor5", "1F931-1F3FF"},
    {"dłonie_do_góry_razem", "1F932"},
    {"dłonie_do_góry_razem_kolor1", "1F932-1F3FB"},
    {"dłonie_do_góry_razem_kolor2", "1F932-1F3FC"},
    {"dłonie_do_góry_razem_kolor3", "1F932-1F3FD"},
    {"dłonie_do_góry_razem_kolor4", "1F932-1F3FE"},
    {"dłonie_do_góry_razem_kolor5", "1F932-1F3FF"},
    {"książę", "1F934"},
    {"książę_kolor1", "1F934-1F3FB"},
    {"książę_kolor2", "1F934-1F3FC"},
    {"książę_kolor3", "1F934-1F3FD"},
    {"książę_kolor4", "1F934-1F3FE"},
    {"książę_kolor5", "1F934-1F3FF"},
    {"żona_św_mikołaja", "1F936"},
    {"żona_św_mikołaja_kolor1", "1F936-1F3FB"},
    {"żona_św_mikołaja_kolor2", "1F936-1F3FC"},
    {"żona_św_mikołaja_kolor3", "1F936-1F3FD"},
    {"żona_św_mikołaja_kolor4", "1F936-1F3FE"},
    {"żona_św_mikołaja_kolor5", "1F936-1F3FF"},
    {"osoba_wzruszająca_ramionami", "1F937"},
    {"osoba_wzruszająca_ramionami_kolor1", "1F937-1F3FB"},
    {"osoba_wzruszająca_ramionami_kolor2", "1F937-1F3FC"},
    {"osoba_wzruszająca_ramionami_kolor3", "1F937-1F3FD"},
    {"osoba_wzruszająca_ramionami_kolor4", "1F937-1F3FE"},
    {"osoba_wzruszająca_ramionami_kolor5", "1F937-1F3FF"},
    {"osoba_robiąca_gwiazdę", "1F938"},
    {"osoba_robiąca_gwiazdę_kolor1", "1F938-1F3FB"},
    {"osoba_robiąca_gwiazdę_kolor2", "1F938-1F3FC"},
    {"osoba_robiąca_gwiazdę_kolor3", "1F938-1F3FD"},
    {"osoba_robiąca_gwiazdę_kolor4", "1F938-1F3FE"},
    {"osoba_robiąca_gwiazdę_kolor5", "1F938-1F3FF"},
    {"żonglująca_osoba", "1F939"},
    {"żonglująca_osoba_kolor1", "1F939-1F3FB"},
    {"żonglująca_osoba_kolor2", "1F939-1F3FC"},
    {"żonglująca_osoba_kolor3", "1F939-1F3FD"},
    {"żonglująca_osoba_kolor4", "1F939-1F3FE"},
    {"żonglująca_osoba_kolor5", "1F939-1F3FF"},
    {"osoba_uprawiająca_szermierkę", "1F93A"},
    {"osoby_uprawiające_zapasy", "1F93C"},
    {"osoby_uprawiające_zapasy_kolor1", "1F93C-1F3FB"},
    {"osoby_uprawiające_zapasy_kolor2", "1F93C-1F3FC"},
    {"osoby_uprawiające_zapasy_kolor3", "1F93C-1F3FD"},
    {"osoby_uprawiające_zapasy_kolor4", "1F93C-1F3FE"},
    {"osoby_uprawiające_zapasy_kolor5", "1F93C-1F3FF"},
    {"osoby_uprawiające_zapasy_kolor1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"osoby_uprawiające_zapasy_kolor1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"osoby_uprawiające_zapasy_kolor1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"osoby_uprawiające_zapasy_kolor1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"osoby_uprawiające_zapasy_kolor2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"osoby_uprawiające_zapasy_kolor2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"osoby_uprawiające_zapasy_kolor2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"osoby_uprawiające_zapasy_kolor2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"osoby_uprawiające_zapasy_kolor3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"osoby_uprawiające_zapasy_kolor3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"osoby_uprawiające_zapasy_kolor3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"osoby_uprawiające_zapasy_kolor3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"osoby_uprawiające_zapasy_kolor4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"osoby_uprawiające_zapasy_kolor4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"osoby_uprawiające_zapasy_kolor4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"osoby_uprawiające_zapasy_kolor4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"osoby_uprawiające_zapasy_kolor5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"osoby_uprawiające_zapasy_kolor5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"osoby_uprawiające_zapasy_kolor5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"osoby_uprawiające_zapasy_kolor5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"osoba_grająca_w_piłkę_wodną", "1F93D"},
    {"osoba_grająca_w_piłkę_wodną_kolor1", "1F93D-1F3FB"},
    {"osoba_grająca_w_piłkę_wodną_kolor2", "1F93D-1F3FC"},
    {"osoba_grająca_w_piłkę_wodną_kolor3", "1F93D-1F3FD"},
    {"osoba_grająca_w_piłkę_wodną_kolor4", "1F93D-1F3FE"},
    {"osoba_grająca_w_piłkę_wodną_kolor5", "1F93D-1F3FF"},
    {"osoba_grająca_w_piłkę_ręczną", "1F93E"},
    {"osoba_grająca_w_piłkę_ręczną_kolor1", "1F93E-1F3FB"},
    {"osoba_grająca_w_piłkę_ręczną_kolor2", "1F93E-1F3FC"},
    {"osoba_grająca_w_piłkę_ręczną_kolor3", "1F93E-1F3FD"},
    {"osoba_grająca_w_piłkę_ręczną_kolor4", "1F93E-1F3FE"},
    {"osoba_grająca_w_piłkę_ręczną_kolor5", "1F93E-1F3FF"},
    {"zwiędły_kwiat", "1F940"},
    {"bęben", "1F941"},
    {"stukające_się_kieliszki", "1F942"},
    {"łyżka", "1F944"},
    {"rękawica_bokserska", "1F94A"},
    {"strój_do_sztuk_walki", "1F94B"},
    {"kamień_do_curlingu", "1F94C"},
    {"ogórek", "1F952"},
    {"sałatka", "1F957"},
    {"naleśniki", "1F95E"},
    {"pieróg", "1F95F"},
    {"ciastko_z_wróżbą", "1F960"},
    {"pudełko_na_wynos", "1F961"},
    {"pałeczki", "1F962"},
    {"miska_z_łyżką", "1F963"},
    {"szklanka_ze_słomką", "1F964"},
    {"brokuł", "1F966"},
    {"kawałek_mięsa", "1F969"},
    {"warzywo_liściaste", "1F96C"},
    {"ciasteczko_księżycowe", "1F96E"},
    {"uśmiechnięta_buźka_z_sercami", "1F970"},
    {"ziewająca_twarz", "1F971"},
    {"uśmiechnięta_buźka_ze_łzą", "1F972"},
    {"świętująca_twarz", "1F973"},
    {"zmarznięta_twarz", "1F976"},
    {"buźka_w_przebraniu", "1F978"},
    {"twarz_powstrzymująca_łzy", "1F979"},
    {"błagająca_twarz", "1F97A"},
    {"biały_fartuch", "1F97C"},
    {"głowa_lwa", "1F981"},
    {"brązowy_skorpion", "1F982"},
    {"głowa_jednorożca", "1F984"},
    {"orzeł", "1F985"},
    {"głowa_lisa", "1F98A"},
    {"jeleń", "1F98C"},
    {"nosorożec", "1F98F"},
    {"kałamarnica", "1F991"},
    {"żyrafa", "1F992"},
    {"jeż", "1F994"},
    {"świerszcz", "1F997"},
    {"łabędź", "1F9A2"},
    {"bóbr", "1F9AB"},
    {"żubr", "1F9AC"},
    {"biała_laska", "1F9AF"},
    {"rude_włosy", "1F9B0"},
    {"kręcone_włosy", "1F9B1"},
    {"łysy", "1F9B2"},
    {"białe_włosy", "1F9B3"},
    {"kość", "1F9B4"},
    {"ząb", "1F9B7"},
    {"superłotr", "1F9B9"},
    {"superłotr_kolor1", "1F9B9-1F3FB"},
    {"superłotr_kolor2", "1F9B9-1F3FC"},
    {"superłotr_kolor3", "1F9B9-1F3FD"},
    {"superłotr_kolor4", "1F9B9-1F3FE"},
    {"superłotr_kolor5", "1F9B9-1F3FF"},
    {"ucho_z_aparatem_słuchowym", "1F9BB"},
    {"ucho_z_aparatem_słuchowym_kolor1", "1F9BB-1F3FB"},
    {"ucho_z_aparatem_słuchowym_kolor2", "1F9BB-1F3FC"},
    {"ucho_z_aparatem_słuchowym_kolor3", "1F9BB-1F3FD"},
    {"ucho_z_aparatem_słuchowym_kolor4", "1F9BB-1F3FE"},
    {"ucho_z_aparatem_słuchowym_kolor5", "1F9BB-1F3FF"},
    {"wózek_inwalidzki_elektryczny", "1F9BC"},
    {"wózek_inwalidzki_ręczny", "1F9BD"},
    {"mechaniczne_ramię", "1F9BE"},
    {"kawałek_sera", "1F9C0"},
    {"sól", "1F9C2"},
    {"napój_w_kartoniku", "1F9C3"},
    {"masło", "1F9C8"},
    {"napój_bubble_tea", "1F9CB"},
    {"stojąca_osoba", "1F9CD"},
    {"stojąca_osoba_kolor1", "1F9CD-1F3FB"},
    {"stojąca_osoba_kolor2", "1F9CD-1F3FC"},
    {"stojąca_osoba_kolor3", "1F9CD-1F3FD"},
    {"stojąca_osoba_kolor4", "1F9CD-1F3FE"},
    {"stojąca_osoba_kolor5", "1F9CD-1F3FF"},
    {"klęcząca_osoba", "1F9CE"},
    {"klęcząca_osoba_kolor1", "1F9CE-1F3FB"},
    {"klęcząca_osoba_kolor2", "1F9CE-1F3FC"},
    {"klęcząca_osoba_kolor3", "1F9CE-1F3FD"},
    {"klęcząca_osoba_kolor4", "1F9CE-1F3FE"},
    {"klęcząca_osoba_kolor5", "1F9CE-1F3FF"},
    {"osoba_głucha", "1F9CF"},
    {"osoba_głucha_kolor1", "1F9CF-1F3FB"},
    {"osoba_głucha_kolor2", "1F9CF-1F3FC"},
    {"osoba_głucha_kolor3", "1F9CF-1F3FD"},
    {"osoba_głucha_kolor4", "1F9CF-1F3FE"},
    {"osoba_głucha_kolor5", "1F9CF-1F3FF"},
    {"dorosły", "1F9D1"},
    {"dorosły_kolor1", "1F9D1-1F3FB"},
    {"dorosły_kolor2", "1F9D1-1F3FC"},
    {"dorosły_kolor3", "1F9D1-1F3FD"},
    {"dorosły_kolor4", "1F9D1-1F3FE"},
    {"dorosły_kolor5", "1F9D1-1F3FF"},
    {"osoba_z_brodą", "1F9D4"},
    {"osoba_z_brodą_kolor1", "1F9D4-1F3FB"},
    {"osoba_z_brodą_kolor2", "1F9D4-1F3FC"},
    {"osoba_z_brodą_kolor3", "1F9D4-1F3FD"},
    {"osoba_z_brodą_kolor4", "1F9D4-1F3FE"},
    {"osoba_z_brodą_kolor5", "1F9D4-1F3FF"},
    {"kobieta_w_chuście", "1F9D5"},
    {"kobieta_w_chuście_kolor1", "1F9D5-1F3FB"},
    {"kobieta_w_chuście_kolor2", "1F9D5-1F3FC"},
    {"kobieta_w_chuście_kolor3", "1F9D5-1F3FD"},
    {"kobieta_w_chuście_kolor4", "1F9D5-1F3FE"},
    {"kobieta_w_chuście_kolor5", "1F9D5-1F3FF"},
    {"osoba_wspinająca_się", "1F9D7"},
    {"osoba_wspinająca_się_kolor1", "1F9D7-1F3FB"},
    {"osoba_wspinająca_się_kolor2", "1F9D7-1F3FC"},
    {"osoba_wspinająca_się_kolor3", "1F9D7-1F3FD"},
    {"osoba_wspinająca_się_kolor4", "1F9D7-1F3FE"},
    {"osoba_wspinająca_się_kolor5", "1F9D7-1F3FF"},
    {"wróżka", "1F9DA"},
    {"wróżka_kolor1", "1F9DA-1F3FB"},
    {"wróżka_kolor2", "1F9DA-1F3FC"},
    {"wróżka_kolor3", "1F9DA-1F3FD"},
    {"wróżka_kolor4", "1F9DA-1F3FE"},
    {"wróżka_kolor5", "1F9DA-1F3FF"},
    {"dżin", "1F9DE"},
    {"mózg", "1F9E0"},
    {"pomarańczowe_serce", "1F9E1"},
    {"bejsbolówka", "1F9E2"},
    {"rękawiczki", "1F9E4"},
    {"płaszcz", "1F9E5"},
    {"element_układanki", "1F9E9"},
    {"probówka", "1F9EA"},
    {"płytka_petriego", "1F9EB"},
    {"liczydło", "1F9EE"},
    {"gaśnica", "1F9EF"},
    {"skrzynka_na_narzędzia", "1F9F0"},
    {"cegła", "1F9F1"},
    {"bagaż", "1F9F3"},
    {"butelka_z_płynem", "1F9F4"},
    {"nić", "1F9F5"},
    {"włóczka", "1F9F6"},
    {"miś_pluszowy", "1F9F8"},
    {"miotła", "1F9F9"},
    {"mydło", "1F9FC"},
    {"gąbka", "1F9FD"},
    {"strój_kąpielowy_jednoczęściowy", "1FA71"},
    {"kąpielówki", "1FA72"},
    {"różowe_serce", "1FA77"},
    {"zdjęcie_rentgenowskie", "1FA7B"},
    {"czarodziejska_różdżka", "1FA84"},
    {"skrzynia_skarbów", "1FA8E"},
    {"łopata", "1FA8F"},
    {"planeta_z_pierścieniami", "1FA90"},
    {"krzesło", "1FA91"},
    {"hełm_wojskowy", "1FA96"},
    {"piła_do_drewna", "1FA9A"},
    {"śrubokręt", "1FA9B"},
    {"igła_krawiecka", "1FAA1"},
    {"węzeł", "1FAA2"},
    {"pułapka_na_myszy", "1FAA4"},
    {"szczoteczka_do_zębów", "1FAA5"},
    {"skała", "1FAA8"},
    {"dowód_tożsamości", "1FAAA"},
    {"słaba_bateria", "1FAAB"},
    {"grzebień_fryzjerski", "1FAAE"},
    {"chrząszcz", "1FAB2"},
    {"roślina_doniczkowa", "1FAB4"},
    {"pióro", "1FAB6"},
    {"skrzydło", "1FABD"},
    {"drzewo_bez_liści", "1FABE"},
    {"gęś", "1FABF"},
    {"płuca", "1FAC1"},
    {"obejmujące_się_osoby", "1FAC2"},
    {"mężczyzna_w_ciąży", "1FAC3"},
    {"mężczyzna_w_ciąży_kolor1", "1FAC3-1F3FB"},
    {"mężczyzna_w_ciąży_kolor2", "1FAC3-1F3FC"},
    {"mężczyzna_w_ciąży_kolor3", "1FAC3-1F3FD"},
    {"mężczyzna_w_ciąży_kolor4", "1FAC3-1F3FE"},
    {"mężczyzna_w_ciąży_kolor5", "1FAC3-1F3FF"},
    {"osoba_w_ciąży", "1FAC4"},
    {"osoba_w_ciąży_kolor1", "1FAC4-1F3FB"},
    {"osoba_w_ciąży_kolor2", "1FAC4-1F3FC"},
    {"osoba_w_ciąży_kolor3", "1FAC4-1F3FD"},
    {"osoba_w_ciąży_kolor4", "1FAC4-1F3FE"},
    {"osoba_w_ciąży_kolor5", "1FAC4-1F3FF"},
    {"włochate_stworzenie", "1FAC8"},
    {"łoś", "1FACE"},
    {"osioł", "1FACF"},
    {"borówka_amerykańska", "1FAD0"},
    {"nalewanie_płynu", "1FAD7"},
    {"słoik", "1FAD9"},
    {"korzeń_imbiru", "1FADA"},
    {"strąk_grochu", "1FADB"},
    {"roztapiająca_się_twarz", "1FAE0"},
    {"salutująca_twarz", "1FAE1"},
    {"twarz_z_otwartymi_oczami_i_ręką_na_ustach", "1FAE2"},
    {"twarz_z_podglądającym_okiem", "1FAE3"},
    {"twarz_z_ukośnymi_ustami", "1FAE4"},
    {"twarz_otoczona_linią_przerywaną", "1FAE5"},
    {"bąbelki", "1FAE7"},
    {"wstrząśnięta_twarz", "1FAE8"},
    {"dłoń_ze_skrzyżowanym_palcem_wskazującym_i_kciukiem", "1FAF0"},
    {"dłoń_ze_skrzyżowanym_palcem_wskazującym_i_kciukiem_kolor1", "1FAF0-1F3FB"},
    {"dłoń_ze_skrzyżowanym_palcem_wskazującym_i_kciukiem_kolor2", "1FAF0-1F3FC"},
    {"dłoń_ze_skrzyżowanym_palcem_wskazującym_i_kciukiem_kolor3", "1FAF0-1F3FD"},
    {"dłoń_ze_skrzyżowanym_palcem_wskazującym_i_kciukiem_kolor4", "1FAF0-1F3FE"},
    {"dłoń_ze_skrzyżowanym_palcem_wskazującym_i_kciukiem_kolor5", "1FAF0-1F3FF"},
    {"dłoń_skierowana_w_prawo", "1FAF1"},
    {"dłoń_skierowana_w_prawo_kolor1", "1FAF1-1F3FB"},
    {"dłoń_skierowana_w_prawo_kolor2", "1FAF1-1F3FC"},
    {"dłoń_skierowana_w_prawo_kolor3", "1FAF1-1F3FD"},
    {"dłoń_skierowana_w_prawo_kolor4", "1FAF1-1F3FE"},
    {"dłoń_skierowana_w_prawo_kolor5", "1FAF1-1F3FF"},
    {"dłoń_skierowana_w_lewo", "1FAF2"},
    {"dłoń_skierowana_w_lewo_kolor1", "1FAF2-1F3FB"},
    {"dłoń_skierowana_w_lewo_kolor2", "1FAF2-1F3FC"},
    {"dłoń_skierowana_w_lewo_kolor3", "1FAF2-1F3FD"},
    {"dłoń_skierowana_w_lewo_kolor4", "1FAF2-1F3FE"},
    {"dłoń_skierowana_w_lewo_kolor5", "1FAF2-1F3FF"},
    {"dłoń_skierowana_w_dół", "1FAF3"},
    {"dłoń_skierowana_w_dół_kolor1", "1FAF3-1F3FB"},
    {"dłoń_skierowana_w_dół_kolor2", "1FAF3-1F3FC"},
    {"dłoń_skierowana_w_dół_kolor3", "1FAF3-1F3FD"},
    {"dłoń_skierowana_w_dół_kolor4", "1FAF3-1F3FE"},
    {"dłoń_skierowana_w_dół_kolor5", "1FAF3-1F3FF"},
    {"dłoń_skierowana_w_górę", "1FAF4"},
    {"dłoń_skierowana_w_górę_kolor1", "1FAF4-1F3FB"},
    {"dłoń_skierowana_w_górę_kolor2", "1FAF4-1F3FC"},
    {"dłoń_skierowana_w_górę_kolor3", "1FAF4-1F3FD"},
    {"dłoń_skierowana_w_górę_kolor4", "1FAF4-1F3FE"},
    {"dłoń_skierowana_w_górę_kolor5", "1FAF4-1F3FF"},
    {"palec_wskazujący_skierowany_na_patrzącego", "1FAF5"},
    {"palec_wskazujący_skierowany_na_patrzącego_kolor1", "1FAF5-1F3FB"},
    {"palec_wskazujący_skierowany_na_patrzącego_kolor2", "1FAF5-1F3FC"},
    {"palec_wskazujący_skierowany_na_patrzącego_kolor3", "1FAF5-1F3FD"},
    {"palec_wskazujący_skierowany_na_patrzącego_kolor4", "1FAF5-1F3FE"},
    {"palec_wskazujący_skierowany_na_patrzącego_kolor5", "1FAF5-1F3FF"},
    {"dłonie_tworzące_serce", "1FAF6"},
    {"dłonie_tworzące_serce_kolor1", "1FAF6-1F3FB"},
    {"dłonie_tworzące_serce_kolor2", "1FAF6-1F3FC"},
    {"dłonie_tworzące_serce_kolor3", "1FAF6-1F3FD"},
    {"dłonie_tworzące_serce_kolor4", "1FAF6-1F3FE"},
    {"dłonie_tworzące_serce_kolor5", "1FAF6-1F3FF"},
    {"dłoń_pchająca_w_lewo", "1FAF7"},
    {"dłoń_pchająca_w_lewo_kolor1", "1FAF7-1F3FB"},
    {"dłoń_pchająca_w_lewo_kolor2", "1FAF7-1F3FC"},
    {"dłoń_pchająca_w_lewo_kolor3", "1FAF7-1F3FD"},
    {"dłoń_pchająca_w_lewo_kolor4", "1FAF7-1F3FE"},
    {"dłoń_pchająca_w_lewo_kolor5", "1FAF7-1F3FF"},
    {"dłoń_pchająca_w_prawo", "1FAF8"},
    {"dłoń_pchająca_w_prawo_kolor1", "1FAF8-1F3FB"},
    {"dłoń_pchająca_w_prawo_kolor2", "1FAF8-1F3FC"},
    {"dłoń_pchająca_w_prawo_kolor3", "1FAF8-1F3FD"},
    {"dłoń_pchająca_w_prawo_kolor4", "1FAF8-1F3FE"},
    {"dłoń_pchająca_w_prawo_kolor5", "1FAF8-1F3FF"},
    {"flaga_wyspa_wniebowstąpienia", "1F1E6-1F1E8"},
    {"flaga_samoa_amerykańskie", "1F1E6-1F1F8"},
    {"flaga_azerbejdżan", "1F1E6-1F1FF"},
    {"flaga_bośnia_i_hercegowina", "1F1E7-1F1E6"},
    {"flaga_bułgaria", "1F1E7-1F1EC"},
    {"flaga_saint_barthélemy", "1F1E7-1F1F1"},
    {"flaga_białoruś", "1F1E7-1F1FE"},
    {"flaga_republika_środkowoafrykańska", "1F1E8-1F1EB"},
    {"flaga_côte_divoire", "1F1E8-1F1EE"},
    {"flaga_republika_zielonego_przylądka", "1F1E8-1F1FB"},
    {"flaga_curaçao", "1F1E8-1F1FC"},
    {"flaga_wyspa_bożego_narodzenia", "1F1E8-1F1FD"},
    {"flaga_dżibuti", "1F1E9-1F1EF"},
    {"flaga_fidżi", "1F1EB-1F1EF"},
    {"flaga_gwinea_równikowa", "1F1EC-1F1F6"},
    {"flaga_georgia_południowa_i_sandwich_południowy", "1F1EC-1F1F8"},
    {"flaga_węgry", "1F1ED-1F1FA"},
    {"flaga_włochy", "1F1EE-1F1F9"},
    {"flaga_kambodża", "1F1F0-1F1ED"},
    {"flaga_korea_północna", "1F1F0-1F1F5"},
    {"flaga_korea_południowa", "1F1F0-1F1F7"},
    {"flaga_łotwa", "1F1F1-1F1FB"},
    {"flaga_mołdawia", "1F1F2-1F1E9"},
    {"flaga_czarnogóra", "1F1F2-1F1EA"},
    {"flaga_macedonia_północna", "1F1F2-1F1F0"},
    {"flaga_mariany_północne", "1F1F2-1F1F5"},
    {"flaga_terytoria_palestyńskie", "1F1F5-1F1F8"},
    {"flaga_wyspa_świętej_heleny", "1F1F8-1F1ED"},
    {"flaga_słowenia", "1F1F8-1F1EE"},
    {"flaga_słowacja", "1F1F8-1F1F0"},
    {"flaga_sudan_południowy", "1F1F8-1F1F8"},
    {"flaga_wyspy_świętego_tomasza_i_książęca", "1F1F8-1F1F9"},
    {"flaga_francuskie_terytoria_południowe_i_antarktyczne", "1F1F9-1F1EB"},
    {"flaga_tadżykistan", "1F1F9-1F1EF"},
    {"flaga_dalekie_wyspy_mniejsze_stanów_zjednoczonych", "1F1FA-1F1F2"},
    {"flaga_organizacja_narodów_zjednoczonych", "1F1FA-1F1F3"},
    {"flaga_wyspy_dziewicze_stanów_zjednoczonych", "1F1FB-1F1EE"},
    {"flaga_republika_południowej_afryki", "1F1FF-1F1E6"},
    {"para_z_sercem_mężczyzna_i_mężczyzna", "1F468-200D-2764-FE0F-200D-1F468"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_mężczyzna_i_mężczyzna_kolor5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pocałunek_mężczyzna_i_mężczyzna", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocałunek_mężczyzna_i_mężczyzna_kolor5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"rodzina_mężczyzna_i_chłopiec", "1F468-200D-1F466"},
    {"rodzina_mężczyzna_chłopiec_i_chłopiec", "1F468-200D-1F466-200D-1F466"},
    {"rodzina_mężczyzna_i_dziewczynka", "1F468-200D-1F467"},
    {"rodzina_mężczyzna_dziewczynka_i_chłopiec", "1F468-200D-1F467-200D-1F466"},
    {"rodzina_mężczyzna_dziewczynka_i_dziewczynka", "1F468-200D-1F467-200D-1F467"},
    {"rodzina_mężczyzna_mężczyzna_i_chłopiec", "1F468-200D-1F468-200D-1F466"},
    {"rodzina_mężczyzna_mężczyzna_chłopiec_i_chłopiec", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"rodzina_mężczyzna_mężczyzna_i_dziewczynka", "1F468-200D-1F468-200D-1F467"},
    {"rodzina_mężczyzna_mężczyzna_dziewczynka_i_chłopiec", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"rodzina_mężczyzna_mężczyzna_dziewczynka_i_dziewczynka", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"rodzina_mężczyzna_kobieta_i_chłopiec", "1F468-200D-1F469-200D-1F466"},
    {"rodzina_mężczyzna_kobieta_chłopiec_i_chłopiec", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"rodzina_mężczyzna_kobieta_i_dziewczynka", "1F468-200D-1F469-200D-1F467"},
    {"rodzina_mężczyzna_kobieta_dziewczynka_i_chłopiec", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"rodzina_mężczyzna_kobieta_dziewczynka_i_dziewczynka", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"para_z_sercem_kobieta_i_mężczyzna", "1F469-200D-2764-FE0F-200D-1F468"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"para_z_sercem_kobieta_i_mężczyzna_kolor5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pocałunek_kobieta_i_mężczyzna", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"pocałunek_kobieta_i_mężczyzna_kolor1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocałunek_kobieta_i_mężczyzna_kolor1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocałunek_kobieta_i_mężczyzna_kolor1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocałunek_kobieta_i_mężczyzna_kolor1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocałunek_kobieta_i_mężczyzna_kolor1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocałunek_kobieta_i_mężczyzna_kolor2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocałunek_kobieta_i_mężczyzna_kolor2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocałunek_kobieta_i_mężczyzna_kolor2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocałunek_kobieta_i_mężczyzna_kolor2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocałunek_kobieta_i_mężczyzna_kolor2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocałunek_kobieta_i_mężczyzna_kolor3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocałunek_kobieta_i_mężczyzna_kolor3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocałunek_kobieta_i_mężczyzna_kolor3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocałunek_kobieta_i_mężczyzna_kolor3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocałunek_kobieta_i_mężczyzna_kolor3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocałunek_kobieta_i_mężczyzna_kolor4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocałunek_kobieta_i_mężczyzna_kolor4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocałunek_kobieta_i_mężczyzna_kolor4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocałunek_kobieta_i_mężczyzna_kolor4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocałunek_kobieta_i_mężczyzna_kolor4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocałunek_kobieta_i_mężczyzna_kolor5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pocałunek_kobieta_i_mężczyzna_kolor5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pocałunek_kobieta_i_mężczyzna_kolor5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pocałunek_kobieta_i_mężczyzna_kolor5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pocałunek_kobieta_i_mężczyzna_kolor5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pocałunek_kobieta_i_kobieta", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"pocałunek_kobieta_i_kobieta_kolor1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pocałunek_kobieta_i_kobieta_kolor1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pocałunek_kobieta_i_kobieta_kolor1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pocałunek_kobieta_i_kobieta_kolor1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pocałunek_kobieta_i_kobieta_kolor1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"pocałunek_kobieta_i_kobieta_kolor2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pocałunek_kobieta_i_kobieta_kolor2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pocałunek_kobieta_i_kobieta_kolor2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pocałunek_kobieta_i_kobieta_kolor2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pocałunek_kobieta_i_kobieta_kolor2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"pocałunek_kobieta_i_kobieta_kolor3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pocałunek_kobieta_i_kobieta_kolor3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pocałunek_kobieta_i_kobieta_kolor3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pocałunek_kobieta_i_kobieta_kolor3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pocałunek_kobieta_i_kobieta_kolor3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"pocałunek_kobieta_i_kobieta_kolor4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pocałunek_kobieta_i_kobieta_kolor4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pocałunek_kobieta_i_kobieta_kolor4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pocałunek_kobieta_i_kobieta_kolor4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pocałunek_kobieta_i_kobieta_kolor4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"pocałunek_kobieta_i_kobieta_kolor5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pocałunek_kobieta_i_kobieta_kolor5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pocałunek_kobieta_i_kobieta_kolor5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pocałunek_kobieta_i_kobieta_kolor5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pocałunek_kobieta_i_kobieta_kolor5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"rodzina_kobieta_i_chłopiec", "1F469-200D-1F466"},
    {"rodzina_kobieta_chłopiec_i_chłopiec", "1F469-200D-1F466-200D-1F466"},
    {"rodzina_kobieta_dziewczynka_i_chłopiec", "1F469-200D-1F467-200D-1F466"},
    {"rodzina_kobieta_kobieta_i_chłopiec", "1F469-200D-1F469-200D-1F466"},
    {"rodzina_kobieta_kobieta_chłopiec_i_chłopiec", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"rodzina_kobieta_kobieta_dziewczynka_i_chłopiec", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"osoby_trzymające_się_za_ręce", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"osoby_trzymające_się_za_ręce_kolor1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"osoby_trzymające_się_za_ręce_kolor1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"osoby_trzymające_się_za_ręce_kolor1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"osoby_trzymające_się_za_ręce_kolor1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"osoby_trzymające_się_za_ręce_kolor1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"osoby_trzymające_się_za_ręce_kolor2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"osoby_trzymające_się_za_ręce_kolor2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"osoby_trzymające_się_za_ręce_kolor2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"osoby_trzymające_się_za_ręce_kolor2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"osoby_trzymające_się_za_ręce_kolor2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"osoby_trzymające_się_za_ręce_kolor3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"osoby_trzymające_się_za_ręce_kolor3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"osoby_trzymające_się_za_ręce_kolor3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"osoby_trzymające_się_za_ręce_kolor3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"osoby_trzymające_się_za_ręce_kolor3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"osoby_trzymające_się_za_ręce_kolor4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"osoby_trzymające_się_za_ręce_kolor4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"osoby_trzymające_się_za_ręce_kolor4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"osoby_trzymające_się_za_ręce_kolor4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"osoby_trzymające_się_za_ręce_kolor4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"osoby_trzymające_się_za_ręce_kolor5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"osoby_trzymające_się_za_ręce_kolor5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"osoby_trzymające_się_za_ręce_kolor5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"osoby_trzymające_się_za_ręce_kolor5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"osoby_trzymające_się_za_ręce_kolor5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"rodzina_dorosły_dorosły_dziecko", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"rodzina_dorosły_dorosły_dziecko_dziecko", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"rodzina_dorosły_dziecko", "1F9D1-200D-1F9D2"},
    {"rodzina_dorosły_dziecko_dziecko", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"biegnąca_osoba_zwrócony_w_prawo", "1F3C3-200D-27A1-FE0F"},
    {"biegnąca_osoba_zwrócony_w_prawo_kolor1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"biegnąca_osoba_zwrócony_w_prawo_kolor2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"biegnąca_osoba_zwrócony_w_prawo_kolor3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"biegnąca_osoba_zwrócony_w_prawo_kolor4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"biegnąca_osoba_zwrócony_w_prawo_kolor5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"mężczyzna_sędzia", "1F468-200D-2696-FE0F"},
    {"mężczyzna_sędzia_kolor1", "1F468-1F3FB-200D-2696-FE0F"},
    {"mężczyzna_sędzia_kolor2", "1F468-1F3FC-200D-2696-FE0F"},
    {"mężczyzna_sędzia_kolor3", "1F468-1F3FD-200D-2696-FE0F"},
    {"mężczyzna_sędzia_kolor4", "1F468-1F3FE-200D-2696-FE0F"},
    {"mężczyzna_sędzia_kolor5", "1F468-1F3FF-200D-2696-FE0F"},
    {"mężczyzna_pilot", "1F468-200D-2708-FE0F"},
    {"mężczyzna_pilot_kolor1", "1F468-1F3FB-200D-2708-FE0F"},
    {"mężczyzna_pilot_kolor2", "1F468-1F3FC-200D-2708-FE0F"},
    {"mężczyzna_pilot_kolor3", "1F468-1F3FD-200D-2708-FE0F"},
    {"mężczyzna_pilot_kolor4", "1F468-1F3FE-200D-2708-FE0F"},
    {"mężczyzna_pilot_kolor5", "1F468-1F3FF-200D-2708-FE0F"},
    {"mężczyzna_rolnik", "1F468-200D-1F33E"},
    {"mężczyzna_rolnik_kolor1", "1F468-1F3FB-200D-1F33E"},
    {"mężczyzna_rolnik_kolor2", "1F468-1F3FC-200D-1F33E"},
    {"mężczyzna_rolnik_kolor3", "1F468-1F3FD-200D-1F33E"},
    {"mężczyzna_rolnik_kolor4", "1F468-1F3FE-200D-1F33E"},
    {"mężczyzna_rolnik_kolor5", "1F468-1F3FF-200D-1F33E"},
    {"mężczyzna_kucharz", "1F468-200D-1F373"},
    {"mężczyzna_kucharz_kolor1", "1F468-1F3FB-200D-1F373"},
    {"mężczyzna_kucharz_kolor2", "1F468-1F3FC-200D-1F373"},
    {"mężczyzna_kucharz_kolor3", "1F468-1F3FD-200D-1F373"},
    {"mężczyzna_kucharz_kolor4", "1F468-1F3FE-200D-1F373"},
    {"mężczyzna_kucharz_kolor5", "1F468-1F3FF-200D-1F373"},
    {"mężczyzna_karmiący_niemowlę", "1F468-200D-1F37C"},
    {"mężczyzna_karmiący_niemowlę_kolor1", "1F468-1F3FB-200D-1F37C"},
    {"mężczyzna_karmiący_niemowlę_kolor2", "1F468-1F3FC-200D-1F37C"},
    {"mężczyzna_karmiący_niemowlę_kolor3", "1F468-1F3FD-200D-1F37C"},
    {"mężczyzna_karmiący_niemowlę_kolor4", "1F468-1F3FE-200D-1F37C"},
    {"mężczyzna_karmiący_niemowlę_kolor5", "1F468-1F3FF-200D-1F37C"},
    {"mężczyzna_student", "1F468-200D-1F393"},
    {"mężczyzna_student_kolor1", "1F468-1F3FB-200D-1F393"},
    {"mężczyzna_student_kolor2", "1F468-1F3FC-200D-1F393"},
    {"mężczyzna_student_kolor3", "1F468-1F3FD-200D-1F393"},
    {"mężczyzna_student_kolor4", "1F468-1F3FE-200D-1F393"},
    {"mężczyzna_student_kolor5", "1F468-1F3FF-200D-1F393"},
    {"mężczyzna_piosenkarz", "1F468-200D-1F3A4"},
    {"mężczyzna_piosenkarz_kolor1", "1F468-1F3FB-200D-1F3A4"},
    {"mężczyzna_piosenkarz_kolor2", "1F468-1F3FC-200D-1F3A4"},
    {"mężczyzna_piosenkarz_kolor3", "1F468-1F3FD-200D-1F3A4"},
    {"mężczyzna_piosenkarz_kolor4", "1F468-1F3FE-200D-1F3A4"},
    {"mężczyzna_piosenkarz_kolor5", "1F468-1F3FF-200D-1F3A4"},
    {"mężczyzna_artysta", "1F468-200D-1F3A8"},
    {"mężczyzna_artysta_kolor1", "1F468-1F3FB-200D-1F3A8"},
    {"mężczyzna_artysta_kolor2", "1F468-1F3FC-200D-1F3A8"},
    {"mężczyzna_artysta_kolor3", "1F468-1F3FD-200D-1F3A8"},
    {"mężczyzna_artysta_kolor4", "1F468-1F3FE-200D-1F3A8"},
    {"mężczyzna_artysta_kolor5", "1F468-1F3FF-200D-1F3A8"},
    {"mężczyzna_nauczyciel", "1F468-200D-1F3EB"},
    {"mężczyzna_nauczyciel_kolor1", "1F468-1F3FB-200D-1F3EB"},
    {"mężczyzna_nauczyciel_kolor2", "1F468-1F3FC-200D-1F3EB"},
    {"mężczyzna_nauczyciel_kolor3", "1F468-1F3FD-200D-1F3EB"},
    {"mężczyzna_nauczyciel_kolor4", "1F468-1F3FE-200D-1F3EB"},
    {"mężczyzna_nauczyciel_kolor5", "1F468-1F3FF-200D-1F3EB"},
    {"mężczyzna_pracownik_fabryki", "1F468-200D-1F3ED"},
    {"mężczyzna_pracownik_fabryki_kolor1", "1F468-1F3FB-200D-1F3ED"},
    {"mężczyzna_pracownik_fabryki_kolor2", "1F468-1F3FC-200D-1F3ED"},
    {"mężczyzna_pracownik_fabryki_kolor3", "1F468-1F3FD-200D-1F3ED"},
    {"mężczyzna_pracownik_fabryki_kolor4", "1F468-1F3FE-200D-1F3ED"},
    {"mężczyzna_pracownik_fabryki_kolor5", "1F468-1F3FF-200D-1F3ED"},
    {"mężczyzna_technolog", "1F468-200D-1F4BB"},
    {"mężczyzna_technolog_kolor1", "1F468-1F3FB-200D-1F4BB"},
    {"mężczyzna_technolog_kolor2", "1F468-1F3FC-200D-1F4BB"},
    {"mężczyzna_technolog_kolor3", "1F468-1F3FD-200D-1F4BB"},
    {"mężczyzna_technolog_kolor4", "1F468-1F3FE-200D-1F4BB"},
    {"mężczyzna_technolog_kolor5", "1F468-1F3FF-200D-1F4BB"},
    {"mężczyzna_pracownik_biurowy", "1F468-200D-1F4BC"},
    {"mężczyzna_pracownik_biurowy_kolor1", "1F468-1F3FB-200D-1F4BC"},
    {"mężczyzna_pracownik_biurowy_kolor2", "1F468-1F3FC-200D-1F4BC"},
    {"mężczyzna_pracownik_biurowy_kolor3", "1F468-1F3FD-200D-1F4BC"},
    {"mężczyzna_pracownik_biurowy_kolor4", "1F468-1F3FE-200D-1F4BC"},
    {"mężczyzna_pracownik_biurowy_kolor5", "1F468-1F3FF-200D-1F4BC"},
    {"mężczyzna_elektryk", "1F468-200D-1F527"},
    {"mężczyzna_elektryk_kolor1", "1F468-1F3FB-200D-1F527"},
    {"mężczyzna_elektryk_kolor2", "1F468-1F3FC-200D-1F527"},
    {"mężczyzna_elektryk_kolor3", "1F468-1F3FD-200D-1F527"},
    {"mężczyzna_elektryk_kolor4", "1F468-1F3FE-200D-1F527"},
    {"mężczyzna_elektryk_kolor5", "1F468-1F3FF-200D-1F527"},
    {"mężczyzna_naukowiec", "1F468-200D-1F52C"},
    {"mężczyzna_naukowiec_kolor1", "1F468-1F3FB-200D-1F52C"},
    {"mężczyzna_naukowiec_kolor2", "1F468-1F3FC-200D-1F52C"},
    {"mężczyzna_naukowiec_kolor3", "1F468-1F3FD-200D-1F52C"},
    {"mężczyzna_naukowiec_kolor4", "1F468-1F3FE-200D-1F52C"},
    {"mężczyzna_naukowiec_kolor5", "1F468-1F3FF-200D-1F52C"},
    {"mężczyzna_astronauta", "1F468-200D-1F680"},
    {"mężczyzna_astronauta_kolor1", "1F468-1F3FB-200D-1F680"},
    {"mężczyzna_astronauta_kolor2", "1F468-1F3FC-200D-1F680"},
    {"mężczyzna_astronauta_kolor3", "1F468-1F3FD-200D-1F680"},
    {"mężczyzna_astronauta_kolor4", "1F468-1F3FE-200D-1F680"},
    {"mężczyzna_astronauta_kolor5", "1F468-1F3FF-200D-1F680"},
    {"mężczyzna_strażak", "1F468-200D-1F692"},
    {"mężczyzna_strażak_kolor1", "1F468-1F3FB-200D-1F692"},
    {"mężczyzna_strażak_kolor2", "1F468-1F3FC-200D-1F692"},
    {"mężczyzna_strażak_kolor3", "1F468-1F3FD-200D-1F692"},
    {"mężczyzna_strażak_kolor4", "1F468-1F3FE-200D-1F692"},
    {"mężczyzna_strażak_kolor5", "1F468-1F3FF-200D-1F692"},
    {"mężczyzna_z_białą_laską", "1F468-200D-1F9AF"},
    {"mężczyzna_z_białą_laską_kolor1", "1F468-1F3FB-200D-1F9AF"},
    {"mężczyzna_z_białą_laską_kolor2", "1F468-1F3FC-200D-1F9AF"},
    {"mężczyzna_z_białą_laską_kolor3", "1F468-1F3FD-200D-1F9AF"},
    {"mężczyzna_z_białą_laską_kolor4", "1F468-1F3FE-200D-1F9AF"},
    {"mężczyzna_z_białą_laską_kolor5", "1F468-1F3FF-200D-1F9AF"},
    {"mężczyzna_z_białą_laską_zwrócony_w_prawo", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"mężczyzna_z_białą_laską_zwrócony_w_prawo_kolor1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"mężczyzna_z_białą_laską_zwrócony_w_prawo_kolor2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"mężczyzna_z_białą_laską_zwrócony_w_prawo_kolor3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"mężczyzna_z_białą_laską_zwrócony_w_prawo_kolor4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"mężczyzna_z_białą_laską_zwrócony_w_prawo_kolor5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"mężczyzna_na_elektrycznym_wózku_inwalidzkim", "1F468-200D-1F9BC"},
    {"mężczyzna_na_elektrycznym_wózku_inwalidzkim_kolor1", "1F468-1F3FB-200D-1F9BC"},
    {"mężczyzna_na_elektrycznym_wózku_inwalidzkim_kolor2", "1F468-1F3FC-200D-1F9BC"},
    {"mężczyzna_na_elektrycznym_wózku_inwalidzkim_kolor3", "1F468-1F3FD-200D-1F9BC"},
    {"mężczyzna_na_elektrycznym_wózku_inwalidzkim_kolor4", "1F468-1F3FE-200D-1F9BC"},
    {"mężczyzna_na_elektrycznym_wózku_inwalidzkim_kolor5", "1F468-1F3FF-200D-1F9BC"},
    {"mężczyzna_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"mężczyzna_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"mężczyzna_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"mężczyzna_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"mężczyzna_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"mężczyzna_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mężczyzna_na_wózku_inwalidzkim", "1F468-200D-1F9BD"},
    {"mężczyzna_na_wózku_inwalidzkim_kolor1", "1F468-1F3FB-200D-1F9BD"},
    {"mężczyzna_na_wózku_inwalidzkim_kolor2", "1F468-1F3FC-200D-1F9BD"},
    {"mężczyzna_na_wózku_inwalidzkim_kolor3", "1F468-1F3FD-200D-1F9BD"},
    {"mężczyzna_na_wózku_inwalidzkim_kolor4", "1F468-1F3FE-200D-1F9BD"},
    {"mężczyzna_na_wózku_inwalidzkim_kolor5", "1F468-1F3FF-200D-1F9BD"},
    {"mężczyzna_na_wózku_inwalidzkim_zwrócony_w_prawo", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"mężczyzna_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"mężczyzna_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"mężczyzna_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"mężczyzna_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"mężczyzna_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_sędzia", "1F469-200D-2696-FE0F"},
    {"kobieta_sędzia_kolor1", "1F469-1F3FB-200D-2696-FE0F"},
    {"kobieta_sędzia_kolor2", "1F469-1F3FC-200D-2696-FE0F"},
    {"kobieta_sędzia_kolor3", "1F469-1F3FD-200D-2696-FE0F"},
    {"kobieta_sędzia_kolor4", "1F469-1F3FE-200D-2696-FE0F"},
    {"kobieta_sędzia_kolor5", "1F469-1F3FF-200D-2696-FE0F"},
    {"kobieta_karmiąca_niemowlę", "1F469-200D-1F37C"},
    {"kobieta_karmiąca_niemowlę_kolor1", "1F469-1F3FB-200D-1F37C"},
    {"kobieta_karmiąca_niemowlę_kolor2", "1F469-1F3FC-200D-1F37C"},
    {"kobieta_karmiąca_niemowlę_kolor3", "1F469-1F3FD-200D-1F37C"},
    {"kobieta_karmiąca_niemowlę_kolor4", "1F469-1F3FE-200D-1F37C"},
    {"kobieta_karmiąca_niemowlę_kolor5", "1F469-1F3FF-200D-1F37C"},
    {"kobieta_strażak", "1F469-200D-1F692"},
    {"kobieta_strażak_kolor1", "1F469-1F3FB-200D-1F692"},
    {"kobieta_strażak_kolor2", "1F469-1F3FC-200D-1F692"},
    {"kobieta_strażak_kolor3", "1F469-1F3FD-200D-1F692"},
    {"kobieta_strażak_kolor4", "1F469-1F3FE-200D-1F692"},
    {"kobieta_strażak_kolor5", "1F469-1F3FF-200D-1F692"},
    {"kobieta_z_białą_laską", "1F469-200D-1F9AF"},
    {"kobieta_z_białą_laską_kolor1", "1F469-1F3FB-200D-1F9AF"},
    {"kobieta_z_białą_laską_kolor2", "1F469-1F3FC-200D-1F9AF"},
    {"kobieta_z_białą_laską_kolor3", "1F469-1F3FD-200D-1F9AF"},
    {"kobieta_z_białą_laską_kolor4", "1F469-1F3FE-200D-1F9AF"},
    {"kobieta_z_białą_laską_kolor5", "1F469-1F3FF-200D-1F9AF"},
    {"kobieta_z_białą_laską_zwrócony_w_prawo", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"kobieta_z_białą_laską_zwrócony_w_prawo_kolor1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"kobieta_z_białą_laską_zwrócony_w_prawo_kolor2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"kobieta_z_białą_laską_zwrócony_w_prawo_kolor3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"kobieta_z_białą_laską_zwrócony_w_prawo_kolor4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"kobieta_z_białą_laską_zwrócony_w_prawo_kolor5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"kobieta_na_elektrycznym_wózku_inwalidzkim", "1F469-200D-1F9BC"},
    {"kobieta_na_elektrycznym_wózku_inwalidzkim_kolor1", "1F469-1F3FB-200D-1F9BC"},
    {"kobieta_na_elektrycznym_wózku_inwalidzkim_kolor2", "1F469-1F3FC-200D-1F9BC"},
    {"kobieta_na_elektrycznym_wózku_inwalidzkim_kolor3", "1F469-1F3FD-200D-1F9BC"},
    {"kobieta_na_elektrycznym_wózku_inwalidzkim_kolor4", "1F469-1F3FE-200D-1F9BC"},
    {"kobieta_na_elektrycznym_wózku_inwalidzkim_kolor5", "1F469-1F3FF-200D-1F9BC"},
    {"kobieta_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"kobieta_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"kobieta_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"kobieta_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"kobieta_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"kobieta_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"kobieta_na_wózku_inwalidzkim", "1F469-200D-1F9BD"},
    {"kobieta_na_wózku_inwalidzkim_kolor1", "1F469-1F3FB-200D-1F9BD"},
    {"kobieta_na_wózku_inwalidzkim_kolor2", "1F469-1F3FC-200D-1F9BD"},
    {"kobieta_na_wózku_inwalidzkim_kolor3", "1F469-1F3FD-200D-1F9BD"},
    {"kobieta_na_wózku_inwalidzkim_kolor4", "1F469-1F3FE-200D-1F9BD"},
    {"kobieta_na_wózku_inwalidzkim_kolor5", "1F469-1F3FF-200D-1F9BD"},
    {"kobieta_na_wózku_inwalidzkim_zwrócony_w_prawo", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"idąca_osoba_zwrócony_w_prawo", "1F6B6-200D-27A1-FE0F"},
    {"idąca_osoba_zwrócony_w_prawo_kolor1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"idąca_osoba_zwrócony_w_prawo_kolor2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"idąca_osoba_zwrócony_w_prawo_kolor3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"idąca_osoba_zwrócony_w_prawo_kolor4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"idąca_osoba_zwrócony_w_prawo_kolor5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"klęcząca_osoba_zwrócony_w_prawo", "1F9CE-200D-27A1-FE0F"},
    {"klęcząca_osoba_zwrócony_w_prawo_kolor1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"klęcząca_osoba_zwrócony_w_prawo_kolor2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"klęcząca_osoba_zwrócony_w_prawo_kolor3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"klęcząca_osoba_zwrócony_w_prawo_kolor4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"klęcząca_osoba_zwrócony_w_prawo_kolor5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"pracownik_służby_zdrowia", "1F9D1-200D-2695-FE0F"},
    {"pracownik_służby_zdrowia_kolor1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"pracownik_służby_zdrowia_kolor2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"pracownik_służby_zdrowia_kolor3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"pracownik_służby_zdrowia_kolor4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"pracownik_służby_zdrowia_kolor5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"sędzia", "1F9D1-200D-2696-FE0F"},
    {"sędzia_kolor1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"sędzia_kolor2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"sędzia_kolor3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"sędzia_kolor4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"sędzia_kolor5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"osoba_karmiąca_niemowlę", "1F9D1-200D-1F37C"},
    {"osoba_karmiąca_niemowlę_kolor1", "1F9D1-1F3FB-200D-1F37C"},
    {"osoba_karmiąca_niemowlę_kolor2", "1F9D1-1F3FC-200D-1F37C"},
    {"osoba_karmiąca_niemowlę_kolor3", "1F9D1-1F3FD-200D-1F37C"},
    {"osoba_karmiąca_niemowlę_kolor4", "1F9D1-1F3FE-200D-1F37C"},
    {"osoba_karmiąca_niemowlę_kolor5", "1F9D1-1F3FF-200D-1F37C"},
    {"mikołajka", "1F9D1-200D-1F384"},
    {"mikołajka_kolor1", "1F9D1-1F3FB-200D-1F384"},
    {"mikołajka_kolor2", "1F9D1-1F3FC-200D-1F384"},
    {"mikołajka_kolor3", "1F9D1-1F3FD-200D-1F384"},
    {"mikołajka_kolor4", "1F9D1-1F3FE-200D-1F384"},
    {"mikołajka_kolor5", "1F9D1-1F3FF-200D-1F384"},
    {"strażak", "1F9D1-200D-1F692"},
    {"strażak_kolor1", "1F9D1-1F3FB-200D-1F692"},
    {"strażak_kolor2", "1F9D1-1F3FC-200D-1F692"},
    {"strażak_kolor3", "1F9D1-1F3FD-200D-1F692"},
    {"strażak_kolor4", "1F9D1-1F3FE-200D-1F692"},
    {"strażak_kolor5", "1F9D1-1F3FF-200D-1F692"},
    {"osoba_z_białą_laską", "1F9D1-200D-1F9AF"},
    {"osoba_z_białą_laską_kolor1", "1F9D1-1F3FB-200D-1F9AF"},
    {"osoba_z_białą_laską_kolor2", "1F9D1-1F3FC-200D-1F9AF"},
    {"osoba_z_białą_laską_kolor3", "1F9D1-1F3FD-200D-1F9AF"},
    {"osoba_z_białą_laską_kolor4", "1F9D1-1F3FE-200D-1F9AF"},
    {"osoba_z_białą_laską_kolor5", "1F9D1-1F3FF-200D-1F9AF"},
    {"osoba_z_białą_laską_zwrócony_w_prawo", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"osoba_z_białą_laską_zwrócony_w_prawo_kolor1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"osoba_z_białą_laską_zwrócony_w_prawo_kolor2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"osoba_z_białą_laską_zwrócony_w_prawo_kolor3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"osoba_z_białą_laską_zwrócony_w_prawo_kolor4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"osoba_z_białą_laską_zwrócony_w_prawo_kolor5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"osoba_na_elektrycznym_wózku_inwalidzkim", "1F9D1-200D-1F9BC"},
    {"osoba_na_elektrycznym_wózku_inwalidzkim_kolor1", "1F9D1-1F3FB-200D-1F9BC"},
    {"osoba_na_elektrycznym_wózku_inwalidzkim_kolor2", "1F9D1-1F3FC-200D-1F9BC"},
    {"osoba_na_elektrycznym_wózku_inwalidzkim_kolor3", "1F9D1-1F3FD-200D-1F9BC"},
    {"osoba_na_elektrycznym_wózku_inwalidzkim_kolor4", "1F9D1-1F3FE-200D-1F9BC"},
    {"osoba_na_elektrycznym_wózku_inwalidzkim_kolor5", "1F9D1-1F3FF-200D-1F9BC"},
    {"osoba_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"osoba_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"osoba_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"osoba_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"osoba_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"osoba_na_elektrycznym_wózku_inwalidzkim_zwrócony_w_prawo_kolor5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"osoba_na_wózku_inwalidzkim", "1F9D1-200D-1F9BD"},
    {"osoba_na_wózku_inwalidzkim_kolor1", "1F9D1-1F3FB-200D-1F9BD"},
    {"osoba_na_wózku_inwalidzkim_kolor2", "1F9D1-1F3FC-200D-1F9BD"},
    {"osoba_na_wózku_inwalidzkim_kolor3", "1F9D1-1F3FD-200D-1F9BD"},
    {"osoba_na_wózku_inwalidzkim_kolor4", "1F9D1-1F3FE-200D-1F9BD"},
    {"osoba_na_wózku_inwalidzkim_kolor5", "1F9D1-1F3FF-200D-1F9BD"},
    {"osoba_na_wózku_inwalidzkim_zwrócony_w_prawo", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"osoba_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"osoba_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"osoba_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"osoba_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"osoba_na_wózku_inwalidzkim_zwrócony_w_prawo_kolor5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kobieta_kozłująca_piłkę", "26F9-FE0F-200D-2640-FE0F"},
    {"kobieta_kozłująca_piłkę_kolor1", "26F9-1F3FB-200D-2640-FE0F"},
    {"kobieta_kozłująca_piłkę_kolor2", "26F9-1F3FC-200D-2640-FE0F"},
    {"kobieta_kozłująca_piłkę_kolor3", "26F9-1F3FD-200D-2640-FE0F"},
    {"kobieta_kozłująca_piłkę_kolor4", "26F9-1F3FE-200D-2640-FE0F"},
    {"kobieta_kozłująca_piłkę_kolor5", "26F9-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_kozłujący_piłkę", "26F9-FE0F-200D-2642-FE0F"},
    {"mężczyzna_kozłujący_piłkę_kolor1", "26F9-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_kozłujący_piłkę_kolor2", "26F9-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_kozłujący_piłkę_kolor3", "26F9-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_kozłujący_piłkę_kolor4", "26F9-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_kozłujący_piłkę_kolor5", "26F9-1F3FF-200D-2642-FE0F"},
    {"biegnąca_kobieta", "1F3C3-200D-2640-FE0F"},
    {"biegnąca_kobieta_kolor1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"biegnąca_kobieta_kolor2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"biegnąca_kobieta_kolor3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"biegnąca_kobieta_kolor4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"biegnąca_kobieta_kolor5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"biegnąca_kobieta_zwrócony_w_prawo", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"biegnąca_kobieta_zwrócony_w_prawo_kolor1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"biegnąca_kobieta_zwrócony_w_prawo_kolor2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"biegnąca_kobieta_zwrócony_w_prawo_kolor3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"biegnąca_kobieta_zwrócony_w_prawo_kolor4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"biegnąca_kobieta_zwrócony_w_prawo_kolor5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"biegnący_mężczyzna", "1F3C3-200D-2642-FE0F"},
    {"biegnący_mężczyzna_kolor1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"biegnący_mężczyzna_kolor2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"biegnący_mężczyzna_kolor3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"biegnący_mężczyzna_kolor4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"biegnący_mężczyzna_kolor5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"biegnący_mężczyzna_zwrócony_w_prawo", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"biegnący_mężczyzna_zwrócony_w_prawo_kolor1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"biegnący_mężczyzna_zwrócony_w_prawo_kolor2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"biegnący_mężczyzna_zwrócony_w_prawo_kolor3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"biegnący_mężczyzna_zwrócony_w_prawo_kolor4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"biegnący_mężczyzna_zwrócony_w_prawo_kolor5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"surfująca_kobieta", "1F3C4-200D-2640-FE0F"},
    {"surfująca_kobieta_kolor1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"surfująca_kobieta_kolor2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"surfująca_kobieta_kolor3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"surfująca_kobieta_kolor4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"surfująca_kobieta_kolor5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"surfujący_mężczyzna", "1F3C4-200D-2642-FE0F"},
    {"surfujący_mężczyzna_kolor1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"surfujący_mężczyzna_kolor2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"surfujący_mężczyzna_kolor3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"surfujący_mężczyzna_kolor4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"surfujący_mężczyzna_kolor5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"pływająca_kobieta", "1F3CA-200D-2640-FE0F"},
    {"pływająca_kobieta_kolor1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"pływająca_kobieta_kolor2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"pływająca_kobieta_kolor3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"pływająca_kobieta_kolor4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"pływająca_kobieta_kolor5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"pływający_mężczyzna", "1F3CA-200D-2642-FE0F"},
    {"pływający_mężczyzna_kolor1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"pływający_mężczyzna_kolor2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"pływający_mężczyzna_kolor3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"pływający_mężczyzna_kolor4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"pływający_mężczyzna_kolor5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"kobieta_podnosząca_ciężary", "1F3CB-FE0F-200D-2640-FE0F"},
    {"kobieta_podnosząca_ciężary_kolor1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"kobieta_podnosząca_ciężary_kolor2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"kobieta_podnosząca_ciężary_kolor3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"kobieta_podnosząca_ciężary_kolor4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"kobieta_podnosząca_ciężary_kolor5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_podnoszący_ciężary", "1F3CB-FE0F-200D-2642-FE0F"},
    {"mężczyzna_podnoszący_ciężary_kolor1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_podnoszący_ciężary_kolor2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_podnoszący_ciężary_kolor3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_podnoszący_ciężary_kolor4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_podnoszący_ciężary_kolor5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"kobieta_grająca_w_golfa", "1F3CC-FE0F-200D-2640-FE0F"},
    {"kobieta_grająca_w_golfa_kolor1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"kobieta_grająca_w_golfa_kolor2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"kobieta_grająca_w_golfa_kolor3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"kobieta_grająca_w_golfa_kolor4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"kobieta_grająca_w_golfa_kolor5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_grający_w_golfa", "1F3CC-FE0F-200D-2642-FE0F"},
    {"mężczyzna_grający_w_golfa_kolor1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_grający_w_golfa_kolor2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_grający_w_golfa_kolor3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_grający_w_golfa_kolor4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_grający_w_golfa_kolor5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_policjant", "1F46E-200D-2642-FE0F"},
    {"mężczyzna_policjant_kolor1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_policjant_kolor2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_policjant_kolor3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_policjant_kolor4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_policjant_kolor5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"kobiety_z_uszami_królika_na_przyjęciu", "1F46F-200D-2640-FE0F"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"kobiety_z_uszami_królika_na_przyjęciu_kolor5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu", "1F46F-200D-2642-FE0F"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"mężczyźni_z_uszami_królika_na_przyjęciu_kolor5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_w_welonie", "1F470-200D-2642-FE0F"},
    {"mężczyzna_w_welonie_kolor1", "1F470-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_w_welonie_kolor2", "1F470-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_w_welonie_kolor3", "1F470-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_w_welonie_kolor4", "1F470-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_w_welonie_kolor5", "1F470-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_w_turbanie", "1F473-200D-2642-FE0F"},
    {"mężczyzna_w_turbanie_kolor1", "1F473-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_w_turbanie_kolor2", "1F473-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_w_turbanie_kolor3", "1F473-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_w_turbanie_kolor4", "1F473-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_w_turbanie_kolor5", "1F473-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_budowlaniec", "1F477-200D-2642-FE0F"},
    {"mężczyzna_budowlaniec_kolor1", "1F477-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_budowlaniec_kolor2", "1F477-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_budowlaniec_kolor3", "1F477-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_budowlaniec_kolor4", "1F477-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_budowlaniec_kolor5", "1F477-1F3FF-200D-2642-FE0F"},
    {"kobieta_z_wystawioną_dłonią", "1F481-200D-2640-FE0F"},
    {"kobieta_z_wystawioną_dłonią_kolor1", "1F481-1F3FB-200D-2640-FE0F"},
    {"kobieta_z_wystawioną_dłonią_kolor2", "1F481-1F3FC-200D-2640-FE0F"},
    {"kobieta_z_wystawioną_dłonią_kolor3", "1F481-1F3FD-200D-2640-FE0F"},
    {"kobieta_z_wystawioną_dłonią_kolor4", "1F481-1F3FE-200D-2640-FE0F"},
    {"kobieta_z_wystawioną_dłonią_kolor5", "1F481-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_z_wystawioną_dłonią", "1F481-200D-2642-FE0F"},
    {"mężczyzna_z_wystawioną_dłonią_kolor1", "1F481-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_z_wystawioną_dłonią_kolor2", "1F481-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_z_wystawioną_dłonią_kolor3", "1F481-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_z_wystawioną_dłonią_kolor4", "1F481-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_z_wystawioną_dłonią_kolor5", "1F481-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_gwardzista", "1F482-200D-2642-FE0F"},
    {"mężczyzna_gwardzista_kolor1", "1F482-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_gwardzista_kolor2", "1F482-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_gwardzista_kolor3", "1F482-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_gwardzista_kolor4", "1F482-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_gwardzista_kolor5", "1F482-1F3FF-200D-2642-FE0F"},
    {"kobieta_podczas_masażu", "1F486-200D-2640-FE0F"},
    {"kobieta_podczas_masażu_kolor1", "1F486-1F3FB-200D-2640-FE0F"},
    {"kobieta_podczas_masażu_kolor2", "1F486-1F3FC-200D-2640-FE0F"},
    {"kobieta_podczas_masażu_kolor3", "1F486-1F3FD-200D-2640-FE0F"},
    {"kobieta_podczas_masażu_kolor4", "1F486-1F3FE-200D-2640-FE0F"},
    {"kobieta_podczas_masażu_kolor5", "1F486-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_podczas_masażu", "1F486-200D-2642-FE0F"},
    {"mężczyzna_podczas_masażu_kolor1", "1F486-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_podczas_masażu_kolor2", "1F486-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_podczas_masażu_kolor3", "1F486-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_podczas_masażu_kolor4", "1F486-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_podczas_masażu_kolor5", "1F486-1F3FF-200D-2642-FE0F"},
    {"kobieta_podczas_strzyżenia", "1F487-200D-2640-FE0F"},
    {"kobieta_podczas_strzyżenia_kolor1", "1F487-1F3FB-200D-2640-FE0F"},
    {"kobieta_podczas_strzyżenia_kolor2", "1F487-1F3FC-200D-2640-FE0F"},
    {"kobieta_podczas_strzyżenia_kolor3", "1F487-1F3FD-200D-2640-FE0F"},
    {"kobieta_podczas_strzyżenia_kolor4", "1F487-1F3FE-200D-2640-FE0F"},
    {"kobieta_podczas_strzyżenia_kolor5", "1F487-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_podczas_strzyżenia", "1F487-200D-2642-FE0F"},
    {"mężczyzna_podczas_strzyżenia_kolor1", "1F487-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_podczas_strzyżenia_kolor2", "1F487-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_podczas_strzyżenia_kolor3", "1F487-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_podczas_strzyżenia_kolor4", "1F487-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_podczas_strzyżenia_kolor5", "1F487-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_detektyw", "1F575-FE0F-200D-2642-FE0F"},
    {"mężczyzna_detektyw_kolor1", "1F575-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_detektyw_kolor2", "1F575-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_detektyw_kolor3", "1F575-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_detektyw_kolor4", "1F575-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_detektyw_kolor5", "1F575-1F3FF-200D-2642-FE0F"},
    {"kobieta_pokazująca_gest_nie", "1F645-200D-2640-FE0F"},
    {"kobieta_pokazująca_gest_nie_kolor1", "1F645-1F3FB-200D-2640-FE0F"},
    {"kobieta_pokazująca_gest_nie_kolor2", "1F645-1F3FC-200D-2640-FE0F"},
    {"kobieta_pokazująca_gest_nie_kolor3", "1F645-1F3FD-200D-2640-FE0F"},
    {"kobieta_pokazująca_gest_nie_kolor4", "1F645-1F3FE-200D-2640-FE0F"},
    {"kobieta_pokazująca_gest_nie_kolor5", "1F645-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_pokazujący_gest_nie", "1F645-200D-2642-FE0F"},
    {"mężczyzna_pokazujący_gest_nie_kolor1", "1F645-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_pokazujący_gest_nie_kolor2", "1F645-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_pokazujący_gest_nie_kolor3", "1F645-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_pokazujący_gest_nie_kolor4", "1F645-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_pokazujący_gest_nie_kolor5", "1F645-1F3FF-200D-2642-FE0F"},
    {"kobieta_pokazująca_gest_ok", "1F646-200D-2640-FE0F"},
    {"kobieta_pokazująca_gest_ok_kolor1", "1F646-1F3FB-200D-2640-FE0F"},
    {"kobieta_pokazująca_gest_ok_kolor2", "1F646-1F3FC-200D-2640-FE0F"},
    {"kobieta_pokazująca_gest_ok_kolor3", "1F646-1F3FD-200D-2640-FE0F"},
    {"kobieta_pokazująca_gest_ok_kolor4", "1F646-1F3FE-200D-2640-FE0F"},
    {"kobieta_pokazująca_gest_ok_kolor5", "1F646-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_pokazujący_gest_ok", "1F646-200D-2642-FE0F"},
    {"mężczyzna_pokazujący_gest_ok_kolor1", "1F646-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_pokazujący_gest_ok_kolor2", "1F646-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_pokazujący_gest_ok_kolor3", "1F646-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_pokazujący_gest_ok_kolor4", "1F646-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_pokazujący_gest_ok_kolor5", "1F646-1F3FF-200D-2642-FE0F"},
    {"kobieta_w_ukłonie", "1F647-200D-2640-FE0F"},
    {"kobieta_w_ukłonie_kolor1", "1F647-1F3FB-200D-2640-FE0F"},
    {"kobieta_w_ukłonie_kolor2", "1F647-1F3FC-200D-2640-FE0F"},
    {"kobieta_w_ukłonie_kolor3", "1F647-1F3FD-200D-2640-FE0F"},
    {"kobieta_w_ukłonie_kolor4", "1F647-1F3FE-200D-2640-FE0F"},
    {"kobieta_w_ukłonie_kolor5", "1F647-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_w_ukłonie", "1F647-200D-2642-FE0F"},
    {"mężczyzna_w_ukłonie_kolor1", "1F647-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_w_ukłonie_kolor2", "1F647-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_w_ukłonie_kolor3", "1F647-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_w_ukłonie_kolor4", "1F647-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_w_ukłonie_kolor5", "1F647-1F3FF-200D-2642-FE0F"},
    {"kobieta_podnosząca_rękę", "1F64B-200D-2640-FE0F"},
    {"kobieta_podnosząca_rękę_kolor1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"kobieta_podnosząca_rękę_kolor2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"kobieta_podnosząca_rękę_kolor3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"kobieta_podnosząca_rękę_kolor4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"kobieta_podnosząca_rękę_kolor5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_podnoszący_rękę", "1F64B-200D-2642-FE0F"},
    {"mężczyzna_podnoszący_rękę_kolor1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_podnoszący_rękę_kolor2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_podnoszący_rękę_kolor3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_podnoszący_rękę_kolor4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_podnoszący_rękę_kolor5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"zachmurzony_mężczyzna", "1F64D-200D-2642-FE0F"},
    {"zachmurzony_mężczyzna_kolor1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"zachmurzony_mężczyzna_kolor2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"zachmurzony_mężczyzna_kolor3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"zachmurzony_mężczyzna_kolor4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"zachmurzony_mężczyzna_kolor5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"nadąsana_kobieta", "1F64E-200D-2640-FE0F"},
    {"nadąsana_kobieta_kolor1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"nadąsana_kobieta_kolor2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"nadąsana_kobieta_kolor3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"nadąsana_kobieta_kolor4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"nadąsana_kobieta_kolor5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"nadąsany_mężczyzna", "1F64E-200D-2642-FE0F"},
    {"nadąsany_mężczyzna_kolor1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"nadąsany_mężczyzna_kolor2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"nadąsany_mężczyzna_kolor3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"nadąsany_mężczyzna_kolor4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"nadąsany_mężczyzna_kolor5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"kobieta_wiosłująca_w_łodzi", "1F6A3-200D-2640-FE0F"},
    {"kobieta_wiosłująca_w_łodzi_kolor1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"kobieta_wiosłująca_w_łodzi_kolor2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"kobieta_wiosłująca_w_łodzi_kolor3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"kobieta_wiosłująca_w_łodzi_kolor4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"kobieta_wiosłująca_w_łodzi_kolor5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_wiosłujący_w_łodzi", "1F6A3-200D-2642-FE0F"},
    {"mężczyzna_wiosłujący_w_łodzi_kolor1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_wiosłujący_w_łodzi_kolor2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_wiosłujący_w_łodzi_kolor3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_wiosłujący_w_łodzi_kolor4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_wiosłujący_w_łodzi_kolor5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_na_rowerze", "1F6B4-200D-2642-FE0F"},
    {"mężczyzna_na_rowerze_kolor1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_na_rowerze_kolor2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_na_rowerze_kolor3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_na_rowerze_kolor4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_na_rowerze_kolor5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"kobieta_na_rowerze_górskim", "1F6B5-200D-2640-FE0F"},
    {"kobieta_na_rowerze_górskim_kolor1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"kobieta_na_rowerze_górskim_kolor2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"kobieta_na_rowerze_górskim_kolor3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"kobieta_na_rowerze_górskim_kolor4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"kobieta_na_rowerze_górskim_kolor5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_na_rowerze_górskim", "1F6B5-200D-2642-FE0F"},
    {"mężczyzna_na_rowerze_górskim_kolor1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_na_rowerze_górskim_kolor2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_na_rowerze_górskim_kolor3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_na_rowerze_górskim_kolor4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_na_rowerze_górskim_kolor5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"idąca_kobieta", "1F6B6-200D-2640-FE0F"},
    {"idąca_kobieta_kolor1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"idąca_kobieta_kolor2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"idąca_kobieta_kolor3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"idąca_kobieta_kolor4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"idąca_kobieta_kolor5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"idąca_kobieta_zwrócony_w_prawo", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"idąca_kobieta_zwrócony_w_prawo_kolor1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"idąca_kobieta_zwrócony_w_prawo_kolor2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"idąca_kobieta_zwrócony_w_prawo_kolor3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"idąca_kobieta_zwrócony_w_prawo_kolor4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"idąca_kobieta_zwrócony_w_prawo_kolor5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"idący_mężczyzna", "1F6B6-200D-2642-FE0F"},
    {"idący_mężczyzna_kolor1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"idący_mężczyzna_kolor2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"idący_mężczyzna_kolor3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"idący_mężczyzna_kolor4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"idący_mężczyzna_kolor5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"idący_mężczyzna_zwrócony_w_prawo", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"idący_mężczyzna_zwrócony_w_prawo_kolor1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"idący_mężczyzna_zwrócony_w_prawo_kolor2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"idący_mężczyzna_zwrócony_w_prawo_kolor3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"idący_mężczyzna_zwrócony_w_prawo_kolor4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"idący_mężczyzna_zwrócony_w_prawo_kolor5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kobieta_trzymająca_się_za_czoło", "1F926-200D-2640-FE0F"},
    {"kobieta_trzymająca_się_za_czoło_kolor1", "1F926-1F3FB-200D-2640-FE0F"},
    {"kobieta_trzymająca_się_za_czoło_kolor2", "1F926-1F3FC-200D-2640-FE0F"},
    {"kobieta_trzymająca_się_za_czoło_kolor3", "1F926-1F3FD-200D-2640-FE0F"},
    {"kobieta_trzymająca_się_za_czoło_kolor4", "1F926-1F3FE-200D-2640-FE0F"},
    {"kobieta_trzymająca_się_za_czoło_kolor5", "1F926-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_trzymający_się_za_czoło", "1F926-200D-2642-FE0F"},
    {"mężczyzna_trzymający_się_za_czoło_kolor1", "1F926-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_trzymający_się_za_czoło_kolor2", "1F926-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_trzymający_się_za_czoło_kolor3", "1F926-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_trzymający_się_za_czoło_kolor4", "1F926-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_trzymający_się_za_czoło_kolor5", "1F926-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_w_smokingu", "1F935-200D-2642-FE0F"},
    {"mężczyzna_w_smokingu_kolor1", "1F935-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_w_smokingu_kolor2", "1F935-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_w_smokingu_kolor3", "1F935-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_w_smokingu_kolor4", "1F935-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_w_smokingu_kolor5", "1F935-1F3FF-200D-2642-FE0F"},
    {"kobieta_wzruszająca_ramionami", "1F937-200D-2640-FE0F"},
    {"kobieta_wzruszająca_ramionami_kolor1", "1F937-1F3FB-200D-2640-FE0F"},
    {"kobieta_wzruszająca_ramionami_kolor2", "1F937-1F3FC-200D-2640-FE0F"},
    {"kobieta_wzruszająca_ramionami_kolor3", "1F937-1F3FD-200D-2640-FE0F"},
    {"kobieta_wzruszająca_ramionami_kolor4", "1F937-1F3FE-200D-2640-FE0F"},
    {"kobieta_wzruszająca_ramionami_kolor5", "1F937-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_wzruszający_ramionami", "1F937-200D-2642-FE0F"},
    {"mężczyzna_wzruszający_ramionami_kolor1", "1F937-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_wzruszający_ramionami_kolor2", "1F937-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_wzruszający_ramionami_kolor3", "1F937-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_wzruszający_ramionami_kolor4", "1F937-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_wzruszający_ramionami_kolor5", "1F937-1F3FF-200D-2642-FE0F"},
    {"kobieta_robiąca_gwiazdę", "1F938-200D-2640-FE0F"},
    {"kobieta_robiąca_gwiazdę_kolor1", "1F938-1F3FB-200D-2640-FE0F"},
    {"kobieta_robiąca_gwiazdę_kolor2", "1F938-1F3FC-200D-2640-FE0F"},
    {"kobieta_robiąca_gwiazdę_kolor3", "1F938-1F3FD-200D-2640-FE0F"},
    {"kobieta_robiąca_gwiazdę_kolor4", "1F938-1F3FE-200D-2640-FE0F"},
    {"kobieta_robiąca_gwiazdę_kolor5", "1F938-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_robiący_gwiazdę", "1F938-200D-2642-FE0F"},
    {"mężczyzna_robiący_gwiazdę_kolor1", "1F938-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_robiący_gwiazdę_kolor2", "1F938-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_robiący_gwiazdę_kolor3", "1F938-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_robiący_gwiazdę_kolor4", "1F938-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_robiący_gwiazdę_kolor5", "1F938-1F3FF-200D-2642-FE0F"},
    {"żonglująca_kobieta", "1F939-200D-2640-FE0F"},
    {"żonglująca_kobieta_kolor1", "1F939-1F3FB-200D-2640-FE0F"},
    {"żonglująca_kobieta_kolor2", "1F939-1F3FC-200D-2640-FE0F"},
    {"żonglująca_kobieta_kolor3", "1F939-1F3FD-200D-2640-FE0F"},
    {"żonglująca_kobieta_kolor4", "1F939-1F3FE-200D-2640-FE0F"},
    {"żonglująca_kobieta_kolor5", "1F939-1F3FF-200D-2640-FE0F"},
    {"żonglujący_mężczyzna", "1F939-200D-2642-FE0F"},
    {"żonglujący_mężczyzna_kolor1", "1F939-1F3FB-200D-2642-FE0F"},
    {"żonglujący_mężczyzna_kolor2", "1F939-1F3FC-200D-2642-FE0F"},
    {"żonglujący_mężczyzna_kolor3", "1F939-1F3FD-200D-2642-FE0F"},
    {"żonglujący_mężczyzna_kolor4", "1F939-1F3FE-200D-2642-FE0F"},
    {"żonglujący_mężczyzna_kolor5", "1F939-1F3FF-200D-2642-FE0F"},
    {"kobiety_uprawiające_zapasy", "1F93C-200D-2640-FE0F"},
    {"kobiety_uprawiające_zapasy_kolor1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"kobiety_uprawiające_zapasy_kolor1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"kobiety_uprawiające_zapasy_kolor1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"kobiety_uprawiające_zapasy_kolor1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"kobiety_uprawiające_zapasy_kolor2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"kobiety_uprawiające_zapasy_kolor2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"kobiety_uprawiające_zapasy_kolor2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"kobiety_uprawiające_zapasy_kolor2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"kobiety_uprawiające_zapasy_kolor3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"kobiety_uprawiające_zapasy_kolor3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"kobiety_uprawiające_zapasy_kolor3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"kobiety_uprawiające_zapasy_kolor3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"kobiety_uprawiające_zapasy_kolor4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"kobiety_uprawiające_zapasy_kolor4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"kobiety_uprawiające_zapasy_kolor4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"kobiety_uprawiające_zapasy_kolor4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"kobiety_uprawiające_zapasy_kolor5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"kobiety_uprawiające_zapasy_kolor5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"kobiety_uprawiające_zapasy_kolor5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"kobiety_uprawiające_zapasy_kolor5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"kobiety_uprawiające_zapasy_kolor1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"kobiety_uprawiające_zapasy_kolor2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"kobiety_uprawiające_zapasy_kolor3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"kobiety_uprawiające_zapasy_kolor4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"kobiety_uprawiające_zapasy_kolor5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"mężczyźni_uprawiający_zapasy", "1F93C-200D-2642-FE0F"},
    {"mężczyźni_uprawiający_zapasy_kolor1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"mężczyźni_uprawiający_zapasy_kolor1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"mężczyźni_uprawiający_zapasy_kolor1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"mężczyźni_uprawiający_zapasy_kolor1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"mężczyźni_uprawiający_zapasy_kolor2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"mężczyźni_uprawiający_zapasy_kolor2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"mężczyźni_uprawiający_zapasy_kolor2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"mężczyźni_uprawiający_zapasy_kolor2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"mężczyźni_uprawiający_zapasy_kolor3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"mężczyźni_uprawiający_zapasy_kolor3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"mężczyźni_uprawiający_zapasy_kolor3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"mężczyźni_uprawiający_zapasy_kolor3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"mężczyźni_uprawiający_zapasy_kolor4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"mężczyźni_uprawiający_zapasy_kolor4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"mężczyźni_uprawiający_zapasy_kolor4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"mężczyźni_uprawiający_zapasy_kolor4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"mężczyźni_uprawiający_zapasy_kolor5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"mężczyźni_uprawiający_zapasy_kolor5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"mężczyźni_uprawiający_zapasy_kolor5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"mężczyźni_uprawiający_zapasy_kolor5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"mężczyźni_uprawiający_zapasy_kolor1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"mężczyźni_uprawiający_zapasy_kolor2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"mężczyźni_uprawiający_zapasy_kolor3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"mężczyźni_uprawiający_zapasy_kolor4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"mężczyźni_uprawiający_zapasy_kolor5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"kobieta_grająca_w_piłkę_wodną", "1F93D-200D-2640-FE0F"},
    {"kobieta_grająca_w_piłkę_wodną_kolor1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"kobieta_grająca_w_piłkę_wodną_kolor2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"kobieta_grająca_w_piłkę_wodną_kolor3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"kobieta_grająca_w_piłkę_wodną_kolor4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"kobieta_grająca_w_piłkę_wodną_kolor5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_grający_w_piłkę_wodną", "1F93D-200D-2642-FE0F"},
    {"mężczyzna_grający_w_piłkę_wodną_kolor1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_grający_w_piłkę_wodną_kolor2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_grający_w_piłkę_wodną_kolor3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_grający_w_piłkę_wodną_kolor4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_grający_w_piłkę_wodną_kolor5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"kobieta_grająca_w_piłkę_ręczną", "1F93E-200D-2640-FE0F"},
    {"kobieta_grająca_w_piłkę_ręczną_kolor1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"kobieta_grająca_w_piłkę_ręczną_kolor2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"kobieta_grająca_w_piłkę_ręczną_kolor3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"kobieta_grająca_w_piłkę_ręczną_kolor4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"kobieta_grająca_w_piłkę_ręczną_kolor5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_grający_w_piłkę_ręczną", "1F93E-200D-2642-FE0F"},
    {"mężczyzna_grający_w_piłkę_ręczną_kolor1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_grający_w_piłkę_ręczną_kolor2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_grający_w_piłkę_ręczną_kolor3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_grający_w_piłkę_ręczną_kolor4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_grający_w_piłkę_ręczną_kolor5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_superbohater", "1F9B8-200D-2642-FE0F"},
    {"mężczyzna_superbohater_kolor1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_superbohater_kolor2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_superbohater_kolor3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_superbohater_kolor4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_superbohater_kolor5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"kobieta_superłotr", "1F9B9-200D-2640-FE0F"},
    {"kobieta_superłotr_kolor1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"kobieta_superłotr_kolor2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"kobieta_superłotr_kolor3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"kobieta_superłotr_kolor4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"kobieta_superłotr_kolor5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_superłotr", "1F9B9-200D-2642-FE0F"},
    {"mężczyzna_superłotr_kolor1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_superłotr_kolor2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_superłotr_kolor3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_superłotr_kolor4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_superłotr_kolor5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"stojąca_kobieta", "1F9CD-200D-2640-FE0F"},
    {"stojąca_kobieta_kolor1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"stojąca_kobieta_kolor2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"stojąca_kobieta_kolor3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"stojąca_kobieta_kolor4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"stojąca_kobieta_kolor5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"stojący_mężczyzna", "1F9CD-200D-2642-FE0F"},
    {"stojący_mężczyzna_kolor1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"stojący_mężczyzna_kolor2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"stojący_mężczyzna_kolor3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"stojący_mężczyzna_kolor4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"stojący_mężczyzna_kolor5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"klęcząca_kobieta", "1F9CE-200D-2640-FE0F"},
    {"klęcząca_kobieta_kolor1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"klęcząca_kobieta_kolor2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"klęcząca_kobieta_kolor3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"klęcząca_kobieta_kolor4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"klęcząca_kobieta_kolor5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"klęcząca_kobieta_zwrócony_w_prawo", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klęcząca_kobieta_zwrócony_w_prawo_kolor1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klęcząca_kobieta_zwrócony_w_prawo_kolor2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klęcząca_kobieta_zwrócony_w_prawo_kolor3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klęcząca_kobieta_zwrócony_w_prawo_kolor4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klęcząca_kobieta_zwrócony_w_prawo_kolor5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klęczący_mężczyzna", "1F9CE-200D-2642-FE0F"},
    {"klęczący_mężczyzna_kolor1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"klęczący_mężczyzna_kolor2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"klęczący_mężczyzna_kolor3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"klęczący_mężczyzna_kolor4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"klęczący_mężczyzna_kolor5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"klęczący_mężczyzna_zwrócony_w_prawo", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klęczący_mężczyzna_zwrócony_w_prawo_kolor1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klęczący_mężczyzna_zwrócony_w_prawo_kolor2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klęczący_mężczyzna_zwrócony_w_prawo_kolor3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klęczący_mężczyzna_zwrócony_w_prawo_kolor4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klęczący_mężczyzna_zwrócony_w_prawo_kolor5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"głucha_kobieta", "1F9CF-200D-2640-FE0F"},
    {"głucha_kobieta_kolor1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"głucha_kobieta_kolor2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"głucha_kobieta_kolor3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"głucha_kobieta_kolor4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"głucha_kobieta_kolor5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"głuchy_mężczyzna", "1F9CF-200D-2642-FE0F"},
    {"głuchy_mężczyzna_kolor1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"głuchy_mężczyzna_kolor2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"głuchy_mężczyzna_kolor3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"głuchy_mężczyzna_kolor4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"głuchy_mężczyzna_kolor5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"kobieta_z_brodą", "1F9D4-200D-2640-FE0F"},
    {"kobieta_z_brodą_kolor1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"kobieta_z_brodą_kolor2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"kobieta_z_brodą_kolor3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"kobieta_z_brodą_kolor4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"kobieta_z_brodą_kolor5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_z_brodą", "1F9D4-200D-2642-FE0F"},
    {"mężczyzna_z_brodą_kolor1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_z_brodą_kolor2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_z_brodą_kolor3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_z_brodą_kolor4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_z_brodą_kolor5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_w_zaparowanym_pomieszczeniu", "1F9D6-200D-2642-FE0F"},
    {"mężczyzna_w_zaparowanym_pomieszczeniu_kolor1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_w_zaparowanym_pomieszczeniu_kolor2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_w_zaparowanym_pomieszczeniu_kolor3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_w_zaparowanym_pomieszczeniu_kolor4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_w_zaparowanym_pomieszczeniu_kolor5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"kobieta_wspinająca_się", "1F9D7-200D-2640-FE0F"},
    {"kobieta_wspinająca_się_kolor1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"kobieta_wspinająca_się_kolor2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"kobieta_wspinająca_się_kolor3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"kobieta_wspinająca_się_kolor4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"kobieta_wspinająca_się_kolor5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_wspinający_się", "1F9D7-200D-2642-FE0F"},
    {"mężczyzna_wspinający_się_kolor1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_wspinający_się_kolor2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_wspinający_się_kolor3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_wspinający_się_kolor4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_wspinający_się_kolor5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_w_pozycji_lotosu", "1F9D8-200D-2642-FE0F"},
    {"mężczyzna_w_pozycji_lotosu_kolor1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_w_pozycji_lotosu_kolor2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_w_pozycji_lotosu_kolor3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_w_pozycji_lotosu_kolor4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_w_pozycji_lotosu_kolor5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_mag", "1F9D9-200D-2642-FE0F"},
    {"mężczyzna_mag_kolor1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_mag_kolor2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_mag_kolor3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_mag_kolor4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_mag_kolor5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"kobieta_wróżka", "1F9DA-200D-2640-FE0F"},
    {"kobieta_wróżka_kolor1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"kobieta_wróżka_kolor2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"kobieta_wróżka_kolor3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"kobieta_wróżka_kolor4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"kobieta_wróżka_kolor5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"mężczyzna_wróżka", "1F9DA-200D-2642-FE0F"},
    {"mężczyzna_wróżka_kolor1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_wróżka_kolor2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_wróżka_kolor3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_wróżka_kolor4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_wróżka_kolor5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_wampir", "1F9DB-200D-2642-FE0F"},
    {"mężczyzna_wampir_kolor1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_wampir_kolor2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_wampir_kolor3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_wampir_kolor4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_wampir_kolor5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_syrena", "1F9DC-200D-2642-FE0F"},
    {"mężczyzna_syrena_kolor1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_syrena_kolor2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_syrena_kolor3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_syrena_kolor4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_syrena_kolor5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"mężczyzna_elf", "1F9DD-200D-2642-FE0F"},
    {"mężczyzna_elf_kolor1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"mężczyzna_elf_kolor2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"mężczyzna_elf_kolor3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"mężczyzna_elf_kolor4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"mężczyzna_elf_kolor5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"kobieta_dżin", "1F9DE-200D-2640-FE0F"},
    {"mężczyzna_dżin", "1F9DE-200D-2642-FE0F"},
    {"mężczyzna_zombie", "1F9DF-200D-2642-FE0F"},
    {"mężczyzna_rude_włosy", "1F468-200D-1F9B0"},
    {"mężczyzna_rude_włosy_kolor1", "1F468-1F3FB-200D-1F9B0"},
    {"mężczyzna_rude_włosy_kolor2", "1F468-1F3FC-200D-1F9B0"},
    {"mężczyzna_rude_włosy_kolor3", "1F468-1F3FD-200D-1F9B0"},
    {"mężczyzna_rude_włosy_kolor4", "1F468-1F3FE-200D-1F9B0"},
    {"mężczyzna_rude_włosy_kolor5", "1F468-1F3FF-200D-1F9B0"},
    {"mężczyzna_kręcone_włosy", "1F468-200D-1F9B1"},
    {"mężczyzna_kręcone_włosy_kolor1", "1F468-1F3FB-200D-1F9B1"},
    {"mężczyzna_kręcone_włosy_kolor2", "1F468-1F3FC-200D-1F9B1"},
    {"mężczyzna_kręcone_włosy_kolor3", "1F468-1F3FD-200D-1F9B1"},
    {"mężczyzna_kręcone_włosy_kolor4", "1F468-1F3FE-200D-1F9B1"},
    {"mężczyzna_kręcone_włosy_kolor5", "1F468-1F3FF-200D-1F9B1"},
    {"mężczyzna_łysy", "1F468-200D-1F9B2"},
    {"mężczyzna_łysy_kolor1", "1F468-1F3FB-200D-1F9B2"},
    {"mężczyzna_łysy_kolor2", "1F468-1F3FC-200D-1F9B2"},
    {"mężczyzna_łysy_kolor3", "1F468-1F3FD-200D-1F9B2"},
    {"mężczyzna_łysy_kolor4", "1F468-1F3FE-200D-1F9B2"},
    {"mężczyzna_łysy_kolor5", "1F468-1F3FF-200D-1F9B2"},
    {"mężczyzna_białe_włosy", "1F468-200D-1F9B3"},
    {"mężczyzna_białe_włosy_kolor1", "1F468-1F3FB-200D-1F9B3"},
    {"mężczyzna_białe_włosy_kolor2", "1F468-1F3FC-200D-1F9B3"},
    {"mężczyzna_białe_włosy_kolor3", "1F468-1F3FD-200D-1F9B3"},
    {"mężczyzna_białe_włosy_kolor4", "1F468-1F3FE-200D-1F9B3"},
    {"mężczyzna_białe_włosy_kolor5", "1F468-1F3FF-200D-1F9B3"},
    {"kobieta_rude_włosy", "1F469-200D-1F9B0"},
    {"kobieta_rude_włosy_kolor1", "1F469-1F3FB-200D-1F9B0"},
    {"kobieta_rude_włosy_kolor2", "1F469-1F3FC-200D-1F9B0"},
    {"kobieta_rude_włosy_kolor3", "1F469-1F3FD-200D-1F9B0"},
    {"kobieta_rude_włosy_kolor4", "1F469-1F3FE-200D-1F9B0"},
    {"kobieta_rude_włosy_kolor5", "1F469-1F3FF-200D-1F9B0"},
    {"kobieta_kręcone_włosy", "1F469-200D-1F9B1"},
    {"kobieta_kręcone_włosy_kolor1", "1F469-1F3FB-200D-1F9B1"},
    {"kobieta_kręcone_włosy_kolor2", "1F469-1F3FC-200D-1F9B1"},
    {"kobieta_kręcone_włosy_kolor3", "1F469-1F3FD-200D-1F9B1"},
    {"kobieta_kręcone_włosy_kolor4", "1F469-1F3FE-200D-1F9B1"},
    {"kobieta_kręcone_włosy_kolor5", "1F469-1F3FF-200D-1F9B1"},
    {"kobieta_łysy", "1F469-200D-1F9B2"},
    {"kobieta_łysy_kolor1", "1F469-1F3FB-200D-1F9B2"},
    {"kobieta_łysy_kolor2", "1F469-1F3FC-200D-1F9B2"},
    {"kobieta_łysy_kolor3", "1F469-1F3FD-200D-1F9B2"},
    {"kobieta_łysy_kolor4", "1F469-1F3FE-200D-1F9B2"},
    {"kobieta_łysy_kolor5", "1F469-1F3FF-200D-1F9B2"},
    {"kobieta_białe_włosy", "1F469-200D-1F9B3"},
    {"kobieta_białe_włosy_kolor1", "1F469-1F3FB-200D-1F9B3"},
    {"kobieta_białe_włosy_kolor2", "1F469-1F3FC-200D-1F9B3"},
    {"kobieta_białe_włosy_kolor3", "1F469-1F3FD-200D-1F9B3"},
    {"kobieta_białe_włosy_kolor4", "1F469-1F3FE-200D-1F9B3"},
    {"kobieta_białe_włosy_kolor5", "1F469-1F3FF-200D-1F9B3"},
    {"dorosły_rude_włosy", "1F9D1-200D-1F9B0"},
    {"dorosły_rude_włosy_kolor1", "1F9D1-1F3FB-200D-1F9B0"},
    {"dorosły_rude_włosy_kolor2", "1F9D1-1F3FC-200D-1F9B0"},
    {"dorosły_rude_włosy_kolor3", "1F9D1-1F3FD-200D-1F9B0"},
    {"dorosły_rude_włosy_kolor4", "1F9D1-1F3FE-200D-1F9B0"},
    {"dorosły_rude_włosy_kolor5", "1F9D1-1F3FF-200D-1F9B0"},
    {"dorosły_kręcone_włosy", "1F9D1-200D-1F9B1"},
    {"dorosły_kręcone_włosy_kolor1", "1F9D1-1F3FB-200D-1F9B1"},
    {"dorosły_kręcone_włosy_kolor2", "1F9D1-1F3FC-200D-1F9B1"},
    {"dorosły_kręcone_włosy_kolor3", "1F9D1-1F3FD-200D-1F9B1"},
    {"dorosły_kręcone_włosy_kolor4", "1F9D1-1F3FE-200D-1F9B1"},
    {"dorosły_kręcone_włosy_kolor5", "1F9D1-1F3FF-200D-1F9B1"},
    {"dorosły_łysy", "1F9D1-200D-1F9B2"},
    {"dorosły_łysy_kolor1", "1F9D1-1F3FB-200D-1F9B2"},
    {"dorosły_łysy_kolor2", "1F9D1-1F3FC-200D-1F9B2"},
    {"dorosły_łysy_kolor3", "1F9D1-1F3FD-200D-1F9B2"},
    {"dorosły_łysy_kolor4", "1F9D1-1F3FE-200D-1F9B2"},
    {"dorosły_łysy_kolor5", "1F9D1-1F3FF-200D-1F9B2"},
    {"dorosły_białe_włosy", "1F9D1-200D-1F9B3"},
    {"dorosły_białe_włosy_kolor1", "1F9D1-1F3FB-200D-1F9B3"},
    {"dorosły_białe_włosy_kolor2", "1F9D1-1F3FC-200D-1F9B3"},
    {"dorosły_białe_włosy_kolor3", "1F9D1-1F3FD-200D-1F9B3"},
    {"dorosły_białe_włosy_kolor4", "1F9D1-1F3FE-200D-1F9B3"},
    {"dorosły_białe_włosy_kolor5", "1F9D1-1F3FF-200D-1F9B3"},
    {"zerwany_łańcuch", "26D3-FE0F-200D-1F4A5"},
    {"serce_w_płomieniach", "2764-FE0F-200D-1F525"},
    {"brązowy_grzyb", "1F344-200D-1F7EB"},
    {"flaga_osób_transpłciowych", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"tęczowa_flaga", "1F3F3-FE0F-200D-1F308"},
    {"niedźwiedź_polarny", "1F43B-200D-2744-FE0F"},
    {"twarz_wypuszczająca_powietrze", "1F62E-200D-1F4A8"},
    {"głowa_kręcąca_się_poziomo", "1F642-200D-2194-FE0F"},
    {"głowa_kiwająca_się_pionowo", "1F642-200D-2195-FE0F"},
};

