// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_ru[] = {
    {"vosklicatelnyy_i_voprositelnyy_znaki", "2049"},
    {"tovarnyy_znak", "2122"},
    {"znachok_informacii", "2139"},
    {"strelka_vlevo_vpravo", "2194"},
    {"strelka_vverh_vniz", "2195"},
    {"strelka_vlevo_vverh", "2196"},
    {"strelka_vpravo_vverh", "2197"},
    {"strelka_vpravo_vniz", "2198"},
    {"strelka_vlevo_vniz", "2199"},
    {"klaviatura", "2328"},
    {"solnce", "2600"},
    {"oblako", "2601"},
    {"zont", "2602"},
    {"snegovik_pod_snegom", "2603"},
    {"kometa", "2604"},
    {"galochka_v_kvadrate", "2611"},
    {"zont_pod_dozhdem", "2614"},
    {"goryachiy_napitok", "2615"},
    {"trilistnik", "2618"},
    {"cherep_i_kosti", "2620"},
    {"radiaciya", "2622"},
    {"biologicheskaya_ugroza", "2623"},
    {"pravoslavnyy_krest", "2626"},
    {"koleso_dharmy", "2638"},
    {"grustit", "2639"},
    {"zhenskiy_simvol", "2640"},
    {"muzhskoy_simvol", "2642"},
    {"znak_zodiaka_oven", "2648"},
    {"znak_zodiaka_telec", "2649"},
    {"znak_zodiaka_strelec", "2650"},
    {"znak_zodiaka_kozerog", "2651"},
    {"znak_zodiaka_vodoley", "2652"},
    {"znak_zodiaka_ryby", "2653"},
    {"piki", "2660"},
    {"trefy", "2663"},
    {"chervy", "2665"},
    {"bubny", "2666"},
    {"goryachie_istochniki", "2668"},
    {"kirka_i_molot", "2692"},
    {"yakor", "2693"},
    {"skreshchennye_mechi", "2694"},
    {"medicinskiy_simvol", "2695"},
    {"vesy", "2696"},
    {"peregonnyy_sosud", "2697"},
    {"shesterenka", "2699"},
    {"nozhnicy", "2702"},
    {"belaya_galochka", "2705"},
    {"samolet", "2708"},
    {"konvert", "2709"},
    {"perevoy_nakonechnik", "2712"},
    {"galochka", "2714"},
    {"znak_umnozheniya", "2716"},
    {"zvezda_davida", "2721"},
    {"blestki", "2728"},
    {"vosmikonechnaya_zvezdochka", "2733"},
    {"vosmikonechnaya_zvezda", "2734"},
    {"snezhinka", "2744"},
    {"iskra", "2747"},
    {"krasnyy_voprositelnyy_znak", "2753"},
    {"belyy_voprositelnyy_znak", "2754"},
    {"belyy_vosklicatelnyy_znak", "2755"},
    {"krasnyy_vosklicatelnyy_znak", "2757"},
    {"vosklicatelnyy_znak_v_vide_serdca", "2763"},
    {"aloe_serdce", "2764"},
    {"znak_slozheniya", "2795"},
    {"znak_vychitaniya", "2796"},
    {"znak_deleniya", "2797"},
    {"strelka_vpravo_s_povorotom_vverh", "2934"},
    {"strelka_vpravo_s_povorotom_vniz", "2935"},
    {"volnistaya_liniya", "3030"},
    {"pozdravlenie", "3297"},
    {"sekret", "3299"},
    {"znachok_zashchity_avtorskogo_prava", "00A9"},
    {"zaregistrirovannyy_tovarnyy_znak", "00AE"},
    {"dva_vosklicatelnyh_znaka", "203C"},
    {"strelka_vpravo_s_povorotom_vlevo", "21A9"},
    {"strelka_vlevo_s_povorotom_vpravo", "21AA"},
    {"naruchnye_chasy", "231A"},
    {"pesochnye_chasy", "231B"},
    {"vybros", "23CF"},
    {"peremotka_vpered", "23E9"},
    {"peremotka_nazad", "23EA"},
    {"dva_treugolnika_ostriem_vverh", "23EB"},
    {"dva_treugolnika_ostriem_vniz", "23EC"},
    {"sleduyushchiy_trek", "23ED"},
    {"predydushchiy_trek", "23EE"},
    {"vosproizvedenie_pauza", "23EF"},
    {"budilnik", "23F0"},
    {"sekundomer", "23F1"},
    {"taymer", "23F2"},
    {"vremya_poshlo", "23F3"},
    {"pauza", "23F8"},
    {"stop", "23F9"},
    {"zapis", "23FA"},
    {"bukva_m_v_kruzhke", "24C2"},
    {"chernyy_kvadratik", "25AA"},
    {"belyy_kvadratik", "25AB"},
    {"vosproizvedenie", "25B6"},
    {"revers", "25C0"},
    {"belyy_kvadrat", "25FB"},
    {"chernyy_kvadrat", "25FC"},
    {"nebolshoy_belyy_kvadrat", "25FD"},
    {"nebolshoy_chernyy_kvadrat", "25FE"},
    {"telefon", "260E"},
    {"ukazatelnyy_palec_vverh", "261D"},
    {"ukazatelnyy_palec_vverh_cvet1", "261D-1F3FB"},
    {"ukazatelnyy_palec_vverh_cvet2", "261D-1F3FC"},
    {"ukazatelnyy_palec_vverh_cvet3", "261D-1F3FD"},
    {"ukazatelnyy_palec_vverh_cvet4", "261D-1F3FE"},
    {"ukazatelnyy_palec_vverh_cvet5", "261D-1F3FF"},
    {"zvezda_i_polumesyac", "262A"},
    {"pacifik", "262E"},
    {"in_yan", "262F"},
    {"ulybaetsya", "263A"},
    {"znak_zodiaka_bliznecy", "264A"},
    {"znak_zodiaka_rak", "264B"},
    {"znak_zodiaka_lev", "264C"},
    {"znak_zodiaka_deva", "264D"},
    {"znak_zodiaka_vesy", "264E"},
    {"znak_zodiaka_skorpion", "264F"},
    {"peshka", "265F"},
    {"utilizaciya", "267B"},
    {"beskonechnost", "267E"},
    {"znachok_dlya_invalidov", "267F"},
    {"simvol_atoma", "269B"},
    {"geraldicheskaya_liliya", "269C"},
    {"znachok_preduprezhdeniya", "26A0"},
    {"vysokoe_napryazhenie", "26A1"},
    {"transgender", "26A7"},
    {"belyy_krug", "26AA"},
    {"chernyy_krug", "26AB"},
    {"grob", "26B0"},
    {"urna_s_prahom", "26B1"},
    {"futbol", "26BD"},
    {"beysbol", "26BE"},
    {"snegovik", "26C4"},
    {"solnce_za_oblakami", "26C5"},
    {"groza", "26C8"},
    {"znak_zodiaka_zmeenosec", "26CE"},
    {"kirka", "26CF"},
    {"kaska_s_belym_krestom", "26D1"},
    {"cep", "26D3"},
    {"vezd_zapreshchen", "26D4"},
    {"sintoistskiy_hram", "26E9"},
    {"cerkov", "26EA"},
    {"gora", "26F0"},
    {"plyazhnyy_zont", "26F1"},
    {"fontan", "26F2"},
    {"golf", "26F3"},
    {"parom", "26F4"},
    {"parusnik", "26F5"},
    {"gornye_lyzhi", "26F7"},
    {"konki", "26F8"},
    {"basketbolist", "26F9"},
    {"basketbolist_cvet1", "26F9-1F3FB"},
    {"basketbolist_cvet2", "26F9-1F3FC"},
    {"basketbolist_cvet3", "26F9-1F3FD"},
    {"basketbolist_cvet4", "26F9-1F3FE"},
    {"basketbolist_cvet5", "26F9-1F3FF"},
    {"palatka", "26FA"},
    {"zapravka", "26FD"},
    {"podnyatyy_kulak", "270A"},
    {"podnyatyy_kulak_cvet1", "270A-1F3FB"},
    {"podnyatyy_kulak_cvet2", "270A-1F3FC"},
    {"podnyatyy_kulak_cvet3", "270A-1F3FD"},
    {"podnyatyy_kulak_cvet4", "270A-1F3FE"},
    {"podnyatyy_kulak_cvet5", "270A-1F3FF"},
    {"podnyataya_ladon", "270B"},
    {"podnyataya_ladon_cvet1", "270B-1F3FB"},
    {"podnyataya_ladon_cvet2", "270B-1F3FC"},
    {"podnyataya_ladon_cvet3", "270B-1F3FD"},
    {"podnyataya_ladon_cvet4", "270B-1F3FE"},
    {"podnyataya_ladon_cvet5", "270B-1F3FF"},
    {"zhest_v", "270C"},
    {"zhest_v_cvet1", "270C-1F3FB"},
    {"zhest_v_cvet2", "270C-1F3FC"},
    {"zhest_v_cvet3", "270C-1F3FD"},
    {"zhest_v_cvet4", "270C-1F3FE"},
    {"zhest_v_cvet5", "270C-1F3FF"},
    {"pishushchaya_ruka", "270D"},
    {"pishushchaya_ruka_cvet1", "270D-1F3FB"},
    {"pishushchaya_ruka_cvet2", "270D-1F3FC"},
    {"pishushchaya_ruka_cvet3", "270D-1F3FD"},
    {"pishushchaya_ruka_cvet4", "270D-1F3FE"},
    {"pishushchaya_ruka_cvet5", "270D-1F3FF"},
    {"karandash", "270F"},
    {"latinskiy_krest", "271D"},
    {"krestik", "274C"},
    {"knopka_krestik", "274E"},
    {"strelka_vpravo", "27A1"},
    {"petlya", "27B0"},
    {"dvoynaya_petlya", "27BF"},
    {"strelka_vlevo", "2B05"},
    {"strelka_vverh", "2B06"},
    {"strelka_vniz", "2B07"},
    {"bolshoy_chernyy_kvadrat", "2B1B"},
    {"bolshoy_belyy_kvadrat", "2B1C"},
    {"zheltaya_zvezda", "2B50"},
    {"krasnoe_kolco", "2B55"},
    {"vstupaet_drugoy_golos", "303D"},
    {"madzhong", "1F004"},
    {"dzhoker", "1F0CF"},
    {"ii_gruppa_krovi", "1F170"},
    {"iii_gruppa_krovi", "1F171"},
    {"i_gruppa_krovi", "1F17E"},
    {"parkovka", "1F17F"},
    {"iv_gruppa_krovi", "1F18E"},
    {"knopka_ochistit", "1F191"},
    {"znachok_kruto", "1F192"},
    {"znachok_besplatno", "1F193"},
    {"znachok_identifikaciya", "1F194"},
    {"znachok_novinka", "1F195"},
    {"znachok_ne_goditsya", "1F196"},
    {"znachok_ok", "1F197"},
    {"znachok_sos", "1F198"},
    {"znachok_obnovlenie", "1F199"},
    {"znachok_protiv", "1F19A"},
    {"regionalnyy_pokazatel_a", "1F1E6"},
    {"regionalnyy_pokazatel_b", "1F1E7"},
    {"regionalnyy_pokazatel_c", "1F1E8"},
    {"regionalnyy_pokazatel_d", "1F1E9"},
    {"regionalnyy_pokazatel_e", "1F1EA"},
    {"regionalnyy_pokazatel_f", "1F1EB"},
    {"regionalnyy_pokazatel_g", "1F1EC"},
    {"regionalnyy_pokazatel_h", "1F1ED"},
    {"regionalnyy_pokazatel_i", "1F1EE"},
    {"regionalnyy_pokazatel_j", "1F1EF"},
    {"regionalnyy_pokazatel_k", "1F1F0"},
    {"regionalnyy_pokazatel_l", "1F1F1"},
    {"regionalnyy_pokazatel_m", "1F1F2"},
    {"regionalnyy_pokazatel_n", "1F1F3"},
    {"regionalnyy_pokazatel_o", "1F1F4"},
    {"regionalnyy_pokazatel_p", "1F1F5"},
    {"regionalnyy_pokazatel_q", "1F1F6"},
    {"regionalnyy_pokazatel_r", "1F1F7"},
    {"regionalnyy_pokazatel_s", "1F1F8"},
    {"regionalnyy_pokazatel_t", "1F1F9"},
    {"regionalnyy_pokazatel_u", "1F1FA"},
    {"regionalnyy_pokazatel_v", "1F1FB"},
    {"regionalnyy_pokazatel_w", "1F1FC"},
    {"regionalnyy_pokazatel_x", "1F1FD"},
    {"regionalnyy_pokazatel_y", "1F1FE"},
    {"regionalnyy_pokazatel_z", "1F1FF"},
    {"znachok_zdes", "1F201"},
    {"cena_uslugi", "1F202"},
    {"besplatno", "1F21A"},
    {"zabronirovano", "1F22F"},
    {"zapret", "1F232"},
    {"est_mesta", "1F233"},
    {"prohodnoy_ball", "1F234"},
    {"net_mest", "1F235"},
    {"platno", "1F236"},
    {"mesyac", "1F237"},
    {"zayavka", "1F238"},
    {"skidka", "1F239"},
    {"otkryto", "1F23A"},
    {"udachnaya_sdelka", "1F250"},
    {"dopustimost", "1F251"},
    {"ciklon", "1F300"},
    {"tuman", "1F301"},
    {"zakrytyy_zont", "1F302"},
    {"zvezdnaya_noch", "1F303"},
    {"voshod_solnca_nad_gorami", "1F304"},
    {"voshod", "1F305"},
    {"zakat_v_gorode", "1F306"},
    {"zakat", "1F307"},
    {"raduga", "1F308"},
    {"most_nochyu", "1F309"},
    {"volna", "1F30A"},
    {"vulkan", "1F30B"},
    {"mlechnyy_put", "1F30C"},
    {"evropa_i_afrika_na_zemnom_share", "1F30D"},
    {"amerika_na_zemnom_share", "1F30E"},
    {"aziya_i_avstraliya_na_zemnom_share", "1F30F"},
    {"meridiany", "1F310"},
    {"novolunie", "1F311"},
    {"molodaya_luna", "1F312"},
    {"luna_v_pervoy_chetverti", "1F313"},
    {"pribyvayushchaya_luna", "1F314"},
    {"polnolunie", "1F315"},
    {"ubyvayushchaya_luna", "1F316"},
    {"luna_v_posledney_chetverti", "1F317"},
    {"staraya_luna", "1F318"},
    {"polumesyac", "1F319"},
    {"novaya_luna_s_licom", "1F31A"},
    {"luna_v_pervoy_chetverti_s_licom", "1F31B"},
    {"luna_v_posledney_chetverti_s_licom", "1F31C"},
    {"polnaya_luna_s_licom", "1F31D"},
    {"solnce_s_licom", "1F31E"},
    {"siyayushchaya_zvezda", "1F31F"},
    {"padayushchaya_zvezda", "1F320"},
    {"termometr", "1F321"},
    {"solnce_za_oblachkom", "1F324"},
    {"solnce_za_oblakom", "1F325"},
    {"slepoy_dozhd", "1F326"},
    {"dozhd", "1F327"},
    {"sneg", "1F328"},
    {"molniya", "1F329"},
    {"tornado", "1F32A"},
    {"gustoy_tuman", "1F32B"},
    {"duet_veter", "1F32C"},
    {"hot_dog", "1F32D"},
    {"tako", "1F32E"},
    {"burrito", "1F32F"},
    {"kashtan", "1F330"},
    {"rostok", "1F331"},
    {"elka", "1F332"},
    {"derevo", "1F333"},
    {"palma", "1F334"},
    {"kaktus", "1F335"},
    {"ostryy_perec", "1F336"},
    {"tyulpan", "1F337"},
    {"cvetok_vishni", "1F338"},
    {"roza", "1F339"},
    {"gibiskus", "1F33A"},
    {"podsolnuh", "1F33B"},
    {"cvetok", "1F33C"},
    {"kukuruznyy_pochatok", "1F33D"},
    {"kolos", "1F33E"},
    {"vetochka", "1F33F"},
    {"chetyrehlistnyy_klever", "1F340"},
    {"klenovyy_list", "1F341"},
    {"padayushchie_listya", "1F342"},
    {"listya_na_vetru", "1F343"},
    {"grib", "1F344"},
    {"pomidor", "1F345"},
    {"baklazhan", "1F346"},
    {"vinograd", "1F347"},
    {"dynya", "1F348"},
    {"arbuz", "1F349"},
    {"mandarin", "1F34A"},
    {"limon", "1F34B"},
    {"banan", "1F34C"},
    {"ananas", "1F34D"},
    {"krasnoe_yabloko", "1F34E"},
    {"zelenoe_yabloko", "1F34F"},
    {"grusha", "1F350"},
    {"persik", "1F351"},
    {"vishnya", "1F352"},
    {"klubnika", "1F353"},
    {"gamburger", "1F354"},
    {"picca", "1F355"},
    {"myaso_na_kostochke", "1F356"},
    {"kurinaya_nozhka", "1F357"},
    {"risovyy_kreker", "1F358"},
    {"onigiri", "1F359"},
    {"ris", "1F35A"},
    {"ris_s_karri", "1F35B"},
    {"lapsha", "1F35C"},
    {"spagetti", "1F35D"},
    {"hleb", "1F35E"},
    {"kartofel_fri", "1F35F"},
    {"pechenyy_sladkiy_kartofel", "1F360"},
    {"dango", "1F361"},
    {"oden", "1F362"},
    {"sushi", "1F363"},
    {"zharenaya_krevetka", "1F364"},
    {"kamaboko", "1F365"},
    {"morozhenoe_v_stakanchike", "1F366"},
    {"morozhenoe_v_kremanke", "1F367"},
    {"morozhenoe", "1F368"},
    {"ponchik", "1F369"},
    {"pechene", "1F36A"},
    {"shokolad", "1F36B"},
    {"konfeta", "1F36C"},
    {"ledenec", "1F36D"},
    {"puding", "1F36E"},
    {"med", "1F36F"},
    {"kusochek_torta", "1F370"},
    {"bento", "1F371"},
    {"kastryulya_s_goryachey_edoy", "1F372"},
    {"yaichnica_na_skovorode", "1F373"},
    {"vilka_i_nozh", "1F374"},
    {"chashka_bez_ruchki", "1F375"},
    {"butylka_sake", "1F376"},
    {"bokal_vina", "1F377"},
    {"kokteyl", "1F378"},
    {"tropicheskiy_kokteyl", "1F379"},
    {"kruzhka_piva", "1F37A"},
    {"chokayushchiesya_kruzhki", "1F37B"},
    {"butylochka_s_soskoy", "1F37C"},
    {"tarelka_s_priborami", "1F37D"},
    {"shampanskoe", "1F37E"},
    {"popkorn", "1F37F"},
    {"bantik", "1F380"},
    {"podarok", "1F381"},
    {"tort_ko_dnyu_rozhdeniya", "1F382"},
    {"tykva_fonar", "1F383"},
    {"novogodnyaya_elka", "1F384"},
    {"santa_klaus", "1F385"},
    {"santa_klaus_cvet1", "1F385-1F3FB"},
    {"santa_klaus_cvet2", "1F385-1F3FC"},
    {"santa_klaus_cvet3", "1F385-1F3FD"},
    {"santa_klaus_cvet4", "1F385-1F3FE"},
    {"santa_klaus_cvet5", "1F385-1F3FF"},
    {"feyerverk", "1F386"},
    {"bengalskiy_ogon", "1F387"},
    {"vozdushnyy_sharik", "1F388"},
    {"hlopushka", "1F389"},
    {"konfetti", "1F38A"},
    {"bambuk_s_bumazhnymi_lentami", "1F38B"},
    {"skreshchennye_flagi", "1F38C"},
    {"ukrashennyy_bambuk", "1F38D"},
    {"yaponskie_kukly", "1F38E"},
    {"vympely_v_vide_karpov", "1F38F"},
    {"muzyka_vetra", "1F390"},
    {"prazdnik_luny", "1F391"},
    {"ranec", "1F392"},
    {"shapochka_vypusknika", "1F393"},
    {"voennaya_medal", "1F396"},
    {"pamyatnaya_lenta", "1F397"},
    {"studiynyy_mikrofon", "1F399"},
    {"slayder", "1F39A"},
    {"regulyatory", "1F39B"},
    {"kinoplenka", "1F39E"},
    {"bilety", "1F39F"},
    {"loshad_na_karuseli", "1F3A0"},
    {"koleso_obozreniya", "1F3A1"},
    {"amerikanskie_gorki", "1F3A2"},
    {"udochka_s_ryboy", "1F3A3"},
    {"mikrofon", "1F3A4"},
    {"kinokamera", "1F3A5"},
    {"videozapis", "1F3A6"},
    {"naushniki", "1F3A7"},
    {"palitra_s_kraskami", "1F3A8"},
    {"cilindr", "1F3A9"},
    {"cirkovoy_shater", "1F3AA"},
    {"bilet", "1F3AB"},
    {"hlopushka_numerator", "1F3AC"},
    {"maski", "1F3AD"},
    {"videoigry", "1F3AE"},
    {"mishen", "1F3AF"},
    {"igrovoy_avtomat", "1F3B0"},
    {"bilyard", "1F3B1"},
    {"igralnaya_kost", "1F3B2"},
    {"bouling", "1F3B3"},
    {"cvetochnaya_karta", "1F3B4"},
    {"nota", "1F3B5"},
    {"noty", "1F3B6"},
    {"saksofon", "1F3B7"},
    {"gitara", "1F3B8"},
    {"klavishnye", "1F3B9"},
    {"truba", "1F3BA"},
    {"skripka", "1F3BB"},
    {"notnyy_stan", "1F3BC"},
    {"futbolka_dlya_bega", "1F3BD"},
    {"tennis", "1F3BE"},
    {"lyzhi", "1F3BF"},
    {"basketbol", "1F3C0"},
    {"kletchatyy_flag", "1F3C1"},
    {"snoubordist", "1F3C2"},
    {"snoubordist_cvet1", "1F3C2-1F3FB"},
    {"snoubordist_cvet2", "1F3C2-1F3FC"},
    {"snoubordist_cvet3", "1F3C2-1F3FD"},
    {"snoubordist_cvet4", "1F3C2-1F3FE"},
    {"snoubordist_cvet5", "1F3C2-1F3FF"},
    {"chelovek_bezhit", "1F3C3"},
    {"chelovek_bezhit_cvet1", "1F3C3-1F3FB"},
    {"chelovek_bezhit_cvet2", "1F3C3-1F3FC"},
    {"chelovek_bezhit_cvet3", "1F3C3-1F3FD"},
    {"chelovek_bezhit_cvet4", "1F3C3-1F3FE"},
    {"chelovek_bezhit_cvet5", "1F3C3-1F3FF"},
    {"serfing", "1F3C4"},
    {"serfing_cvet1", "1F3C4-1F3FB"},
    {"serfing_cvet2", "1F3C4-1F3FC"},
    {"serfing_cvet3", "1F3C4-1F3FD"},
    {"serfing_cvet4", "1F3C4-1F3FE"},
    {"serfing_cvet5", "1F3C4-1F3FF"},
    {"sportivnaya_medal", "1F3C5"},
    {"kubok", "1F3C6"},
    {"skachki", "1F3C7"},
    {"skachki_cvet1", "1F3C7-1F3FB"},
    {"skachki_cvet2", "1F3C7-1F3FC"},
    {"skachki_cvet3", "1F3C7-1F3FD"},
    {"skachki_cvet4", "1F3C7-1F3FE"},
    {"skachki_cvet5", "1F3C7-1F3FF"},
    {"amerikanskiy_futbol", "1F3C8"},
    {"regbi", "1F3C9"},
    {"plavanie", "1F3CA"},
    {"plavanie_cvet1", "1F3CA-1F3FB"},
    {"plavanie_cvet2", "1F3CA-1F3FC"},
    {"plavanie_cvet3", "1F3CA-1F3FD"},
    {"plavanie_cvet4", "1F3CA-1F3FE"},
    {"plavanie_cvet5", "1F3CA-1F3FF"},
    {"tyazheloatlet", "1F3CB"},
    {"tyazheloatlet_cvet1", "1F3CB-1F3FB"},
    {"tyazheloatlet_cvet2", "1F3CB-1F3FC"},
    {"tyazheloatlet_cvet3", "1F3CB-1F3FD"},
    {"tyazheloatlet_cvet4", "1F3CB-1F3FE"},
    {"tyazheloatlet_cvet5", "1F3CB-1F3FF"},
    {"golfist", "1F3CC"},
    {"golfist_cvet1", "1F3CC-1F3FB"},
    {"golfist_cvet2", "1F3CC-1F3FC"},
    {"golfist_cvet3", "1F3CC-1F3FD"},
    {"golfist_cvet4", "1F3CC-1F3FE"},
    {"golfist_cvet5", "1F3CC-1F3FF"},
    {"motocikl", "1F3CD"},
    {"gonochnyy_avtomobil", "1F3CE"},
    {"kriket", "1F3CF"},
    {"voleybol", "1F3D0"},
    {"hokkey_na_trave", "1F3D1"},
    {"hokkey", "1F3D2"},
    {"nastolnyy_tennis", "1F3D3"},
    {"gora_so_snezhnoy_shapkoy", "1F3D4"},
    {"kemping", "1F3D5"},
    {"plyazh", "1F3D6"},
    {"stroyka", "1F3D7"},
    {"doma", "1F3D8"},
    {"vid_na_gorod", "1F3D9"},
    {"zabroshennyy_dom", "1F3DA"},
    {"antichnoe_zdanie", "1F3DB"},
    {"pustynya", "1F3DC"},
    {"ostrov", "1F3DD"},
    {"nacionalnyy_park", "1F3DE"},
    {"stadion", "1F3DF"},
    {"dom", "1F3E0"},
    {"dom_s_sadom", "1F3E1"},
    {"ofisnoe_zdanie", "1F3E2"},
    {"yaponskaya_pochta", "1F3E3"},
    {"pochta", "1F3E4"},
    {"bolnica", "1F3E5"},
    {"bank", "1F3E6"},
    {"znachok_bankomata", "1F3E7"},
    {"gostinica", "1F3E8"},
    {"gostinica_dlya_svidaniy", "1F3E9"},
    {"kruglosutochnyy_magazin", "1F3EA"},
    {"shkola", "1F3EB"},
    {"univermag", "1F3EC"},
    {"zavod", "1F3ED"},
    {"krasnyy_bumazhnyy_fonarik", "1F3EE"},
    {"yaponskiy_zamok", "1F3EF"},
    {"zamok", "1F3F0"},
    {"belyy_flag", "1F3F3"},
    {"chernyy_flag", "1F3F4"},
    {"rozetka", "1F3F5"},
    {"birka", "1F3F7"},
    {"badminton", "1F3F8"},
    {"luk_so_streloy", "1F3F9"},
    {"amfora", "1F3FA"},
    {"ochen_svetlyy_ton_kozhi", "1F3FB"},
    {"svetlyy_ton_kozhi", "1F3FC"},
    {"sredniy_ton_kozhi", "1F3FD"},
    {"temnyy_ton_kozhi", "1F3FE"},
    {"ochen_temnyy_ton_kozhi", "1F3FF"},
    {"krysa", "1F400"},
    {"mysh", "1F401"},
    {"byk", "1F402"},
    {"buyvol", "1F403"},
    {"korova", "1F404"},
    {"tigr", "1F405"},
    {"leopard", "1F406"},
    {"krolik", "1F407"},
    {"koshka", "1F408"},
    {"drakon", "1F409"},
    {"krokodil", "1F40A"},
    {"kit", "1F40B"},
    {"ulitka", "1F40C"},
    {"zmeya", "1F40D"},
    {"loshad", "1F40E"},
    {"baran", "1F40F"},
    {"koza", "1F410"},
    {"ovca", "1F411"},
    {"obezyana", "1F412"},
    {"petuh", "1F413"},
    {"kurica", "1F414"},
    {"sobaka", "1F415"},
    {"svinya", "1F416"},
    {"kaban", "1F417"},
    {"slon", "1F418"},
    {"osminog", "1F419"},
    {"rakovina", "1F41A"},
    {"gusenica", "1F41B"},
    {"muravey", "1F41C"},
    {"pchela", "1F41D"},
    {"bozhya_korovka", "1F41E"},
    {"ryba", "1F41F"},
    {"tropicheskaya_ryba", "1F420"},
    {"ryba_ezh", "1F421"},
    {"cherepaha", "1F422"},
    {"cyplenok_v_yayce", "1F423"},
    {"cyplenok_bokom", "1F424"},
    {"cyplenok", "1F425"},
    {"ptica", "1F426"},
    {"pingvin", "1F427"},
    {"morda_koaly", "1F428"},
    {"pudel", "1F429"},
    {"odnogorbyy_verblyud", "1F42A"},
    {"dvugorbyy_verblyud", "1F42B"},
    {"delfin", "1F42C"},
    {"morda_myshi", "1F42D"},
    {"morda_korovy", "1F42E"},
    {"morda_tigra", "1F42F"},
    {"morda_krolika", "1F430"},
    {"morda_koshki", "1F431"},
    {"golova_drakona", "1F432"},
    {"kit_s_fontanchikom", "1F433"},
    {"morda_loshadi", "1F434"},
    {"morda_obezyany", "1F435"},
    {"morda_sobaki", "1F436"},
    {"morda_svini", "1F437"},
    {"golova_lyagushki", "1F438"},
    {"morda_homyaka", "1F439"},
    {"morda_volka", "1F43A"},
    {"morda_medvedya", "1F43B"},
    {"morda_pandy", "1F43C"},
    {"svinoy_pyatachok", "1F43D"},
    {"sledy_zhivotnogo", "1F43E"},
    {"burunduk", "1F43F"},
    {"glaza", "1F440"},
    {"glaz", "1F441"},
    {"uho", "1F442"},
    {"uho_cvet1", "1F442-1F3FB"},
    {"uho_cvet2", "1F442-1F3FC"},
    {"uho_cvet3", "1F442-1F3FD"},
    {"uho_cvet4", "1F442-1F3FE"},
    {"uho_cvet5", "1F442-1F3FF"},
    {"nos", "1F443"},
    {"nos_cvet1", "1F443-1F3FB"},
    {"nos_cvet2", "1F443-1F3FC"},
    {"nos_cvet3", "1F443-1F3FD"},
    {"nos_cvet4", "1F443-1F3FE"},
    {"nos_cvet5", "1F443-1F3FF"},
    {"rot", "1F444"},
    {"yazyk", "1F445"},
    {"ukazyvayushchiy_vverh_palec", "1F446"},
    {"ukazyvayushchiy_vverh_palec_cvet1", "1F446-1F3FB"},
    {"ukazyvayushchiy_vverh_palec_cvet2", "1F446-1F3FC"},
    {"ukazyvayushchiy_vverh_palec_cvet3", "1F446-1F3FD"},
    {"ukazyvayushchiy_vverh_palec_cvet4", "1F446-1F3FE"},
    {"ukazyvayushchiy_vverh_palec_cvet5", "1F446-1F3FF"},
    {"ukazyvayushchiy_vniz_palec", "1F447"},
    {"ukazyvayushchiy_vniz_palec_cvet1", "1F447-1F3FB"},
    {"ukazyvayushchiy_vniz_palec_cvet2", "1F447-1F3FC"},
    {"ukazyvayushchiy_vniz_palec_cvet3", "1F447-1F3FD"},
    {"ukazyvayushchiy_vniz_palec_cvet4", "1F447-1F3FE"},
    {"ukazyvayushchiy_vniz_palec_cvet5", "1F447-1F3FF"},
    {"ukazyvayushchiy_nalevo_palec", "1F448"},
    {"ukazyvayushchiy_nalevo_palec_cvet1", "1F448-1F3FB"},
    {"ukazyvayushchiy_nalevo_palec_cvet2", "1F448-1F3FC"},
    {"ukazyvayushchiy_nalevo_palec_cvet3", "1F448-1F3FD"},
    {"ukazyvayushchiy_nalevo_palec_cvet4", "1F448-1F3FE"},
    {"ukazyvayushchiy_nalevo_palec_cvet5", "1F448-1F3FF"},
    {"ukazyvayushchiy_napravo_palec", "1F449"},
    {"ukazyvayushchiy_napravo_palec_cvet1", "1F449-1F3FB"},
    {"ukazyvayushchiy_napravo_palec_cvet2", "1F449-1F3FC"},
    {"ukazyvayushchiy_napravo_palec_cvet3", "1F449-1F3FD"},
    {"ukazyvayushchiy_napravo_palec_cvet4", "1F449-1F3FE"},
    {"ukazyvayushchiy_napravo_palec_cvet5", "1F449-1F3FF"},
    {"kulak", "1F44A"},
    {"kulak_cvet1", "1F44A-1F3FB"},
    {"kulak_cvet2", "1F44A-1F3FC"},
    {"kulak_cvet3", "1F44A-1F3FD"},
    {"kulak_cvet4", "1F44A-1F3FE"},
    {"kulak_cvet5", "1F44A-1F3FF"},
    {"mashet_rukoy", "1F44B"},
    {"mashet_rukoy_cvet1", "1F44B-1F3FB"},
    {"mashet_rukoy_cvet2", "1F44B-1F3FC"},
    {"mashet_rukoy_cvet3", "1F44B-1F3FD"},
    {"mashet_rukoy_cvet4", "1F44B-1F3FE"},
    {"mashet_rukoy_cvet5", "1F44B-1F3FF"},
    {"zhest_vse_horosho", "1F44C"},
    {"zhest_vse_horosho_cvet1", "1F44C-1F3FB"},
    {"zhest_vse_horosho_cvet2", "1F44C-1F3FC"},
    {"zhest_vse_horosho_cvet3", "1F44C-1F3FD"},
    {"zhest_vse_horosho_cvet4", "1F44C-1F3FE"},
    {"zhest_vse_horosho_cvet5", "1F44C-1F3FF"},
    {"bolshoy_palec_vverh", "1F44D"},
    {"bolshoy_palec_vverh_cvet1", "1F44D-1F3FB"},
    {"bolshoy_palec_vverh_cvet2", "1F44D-1F3FC"},
    {"bolshoy_palec_vverh_cvet3", "1F44D-1F3FD"},
    {"bolshoy_palec_vverh_cvet4", "1F44D-1F3FE"},
    {"bolshoy_palec_vverh_cvet5", "1F44D-1F3FF"},
    {"bolshoy_palec_vniz", "1F44E"},
    {"bolshoy_palec_vniz_cvet1", "1F44E-1F3FB"},
    {"bolshoy_palec_vniz_cvet2", "1F44E-1F3FC"},
    {"bolshoy_palec_vniz_cvet3", "1F44E-1F3FD"},
    {"bolshoy_palec_vniz_cvet4", "1F44E-1F3FE"},
    {"bolshoy_palec_vniz_cvet5", "1F44E-1F3FF"},
    {"aplodismenty", "1F44F"},
    {"aplodismenty_cvet1", "1F44F-1F3FB"},
    {"aplodismenty_cvet2", "1F44F-1F3FC"},
    {"aplodismenty_cvet3", "1F44F-1F3FD"},
    {"aplodismenty_cvet4", "1F44F-1F3FE"},
    {"aplodismenty_cvet5", "1F44F-1F3FF"},
    {"ladoni_v_storony", "1F450"},
    {"ladoni_v_storony_cvet1", "1F450-1F3FB"},
    {"ladoni_v_storony_cvet2", "1F450-1F3FC"},
    {"ladoni_v_storony_cvet3", "1F450-1F3FD"},
    {"ladoni_v_storony_cvet4", "1F450-1F3FE"},
    {"ladoni_v_storony_cvet5", "1F450-1F3FF"},
    {"korona", "1F451"},
    {"shlyapka", "1F452"},
    {"ochki", "1F453"},
    {"galstuk", "1F454"},
    {"futbolka", "1F455"},
    {"dzhinsy", "1F456"},
    {"plate", "1F457"},
    {"kimono", "1F458"},
    {"bikini", "1F459"},
    {"bluzka", "1F45A"},
    {"koshelek", "1F45B"},
    {"sumka", "1F45C"},
    {"klatch", "1F45D"},
    {"botinok", "1F45E"},
    {"krossovki", "1F45F"},
    {"tufli_na_kabluke", "1F460"},
    {"bosonozhki", "1F461"},
    {"sapog", "1F462"},
    {"sledy", "1F463"},
    {"siluet_cheloveka", "1F464"},
    {"dva_silueta", "1F465"},
    {"malchik", "1F466"},
    {"malchik_cvet1", "1F466-1F3FB"},
    {"malchik_cvet2", "1F466-1F3FC"},
    {"malchik_cvet3", "1F466-1F3FD"},
    {"malchik_cvet4", "1F466-1F3FE"},
    {"malchik_cvet5", "1F466-1F3FF"},
    {"devochka", "1F467"},
    {"devochka_cvet1", "1F467-1F3FB"},
    {"devochka_cvet2", "1F467-1F3FC"},
    {"devochka_cvet3", "1F467-1F3FD"},
    {"devochka_cvet4", "1F467-1F3FE"},
    {"devochka_cvet5", "1F467-1F3FF"},
    {"muzhchina", "1F468"},
    {"muzhchina_cvet1", "1F468-1F3FB"},
    {"muzhchina_cvet2", "1F468-1F3FC"},
    {"muzhchina_cvet3", "1F468-1F3FD"},
    {"muzhchina_cvet4", "1F468-1F3FE"},
    {"muzhchina_cvet5", "1F468-1F3FF"},
    {"zhenshchina", "1F469"},
    {"zhenshchina_cvet1", "1F469-1F3FB"},
    {"zhenshchina_cvet2", "1F469-1F3FC"},
    {"zhenshchina_cvet3", "1F469-1F3FD"},
    {"zhenshchina_cvet4", "1F469-1F3FE"},
    {"zhenshchina_cvet5", "1F469-1F3FF"},
    {"semya", "1F46A"},
    {"muzhchina_i_zhenshchina", "1F46B"},
    {"muzhchina_i_zhenshchina_cvet1", "1F46B-1F3FB"},
    {"muzhchina_i_zhenshchina_cvet2", "1F46B-1F3FC"},
    {"muzhchina_i_zhenshchina_cvet3", "1F46B-1F3FD"},
    {"muzhchina_i_zhenshchina_cvet4", "1F46B-1F3FE"},
    {"muzhchina_i_zhenshchina_cvet5", "1F46B-1F3FF"},
    {"muzhchina_i_zhenshchina_cvet1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"muzhchina_i_zhenshchina_cvet1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"muzhchina_i_zhenshchina_cvet1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"muzhchina_i_zhenshchina_cvet1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"muzhchina_i_zhenshchina_cvet2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"muzhchina_i_zhenshchina_cvet2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"muzhchina_i_zhenshchina_cvet2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"muzhchina_i_zhenshchina_cvet2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"muzhchina_i_zhenshchina_cvet3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"muzhchina_i_zhenshchina_cvet3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"muzhchina_i_zhenshchina_cvet3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"muzhchina_i_zhenshchina_cvet3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"muzhchina_i_zhenshchina_cvet4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"muzhchina_i_zhenshchina_cvet4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"muzhchina_i_zhenshchina_cvet4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"muzhchina_i_zhenshchina_cvet4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"muzhchina_i_zhenshchina_cvet5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"muzhchina_i_zhenshchina_cvet5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"muzhchina_i_zhenshchina_cvet5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"muzhchina_i_zhenshchina_cvet5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"dvoe_muzhchin", "1F46C"},
    {"dvoe_muzhchin_cvet1", "1F46C-1F3FB"},
    {"dvoe_muzhchin_cvet2", "1F46C-1F3FC"},
    {"dvoe_muzhchin_cvet3", "1F46C-1F3FD"},
    {"dvoe_muzhchin_cvet4", "1F46C-1F3FE"},
    {"dvoe_muzhchin_cvet5", "1F46C-1F3FF"},
    {"dvoe_muzhchin_cvet1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"dvoe_muzhchin_cvet1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"dvoe_muzhchin_cvet1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"dvoe_muzhchin_cvet1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"dvoe_muzhchin_cvet2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"dvoe_muzhchin_cvet2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"dvoe_muzhchin_cvet2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"dvoe_muzhchin_cvet2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"dvoe_muzhchin_cvet3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"dvoe_muzhchin_cvet3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"dvoe_muzhchin_cvet3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"dvoe_muzhchin_cvet3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"dvoe_muzhchin_cvet4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"dvoe_muzhchin_cvet4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"dvoe_muzhchin_cvet4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"dvoe_muzhchin_cvet4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"dvoe_muzhchin_cvet5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"dvoe_muzhchin_cvet5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"dvoe_muzhchin_cvet5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"dvoe_muzhchin_cvet5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"dve_zhenshchiny", "1F46D"},
    {"dve_zhenshchiny_cvet1", "1F46D-1F3FB"},
    {"dve_zhenshchiny_cvet2", "1F46D-1F3FC"},
    {"dve_zhenshchiny_cvet3", "1F46D-1F3FD"},
    {"dve_zhenshchiny_cvet4", "1F46D-1F3FE"},
    {"dve_zhenshchiny_cvet5", "1F46D-1F3FF"},
    {"dve_zhenshchiny_cvet1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"dve_zhenshchiny_cvet1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"dve_zhenshchiny_cvet1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"dve_zhenshchiny_cvet1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"dve_zhenshchiny_cvet2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"dve_zhenshchiny_cvet2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"dve_zhenshchiny_cvet2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"dve_zhenshchiny_cvet2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"dve_zhenshchiny_cvet3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"dve_zhenshchiny_cvet3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"dve_zhenshchiny_cvet3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"dve_zhenshchiny_cvet3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"dve_zhenshchiny_cvet4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"dve_zhenshchiny_cvet4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"dve_zhenshchiny_cvet4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"dve_zhenshchiny_cvet4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"dve_zhenshchiny_cvet5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"dve_zhenshchiny_cvet5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"dve_zhenshchiny_cvet5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"dve_zhenshchiny_cvet5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"policeyskiy", "1F46E"},
    {"policeyskiy_cvet1", "1F46E-1F3FB"},
    {"policeyskiy_cvet2", "1F46E-1F3FC"},
    {"policeyskiy_cvet3", "1F46E-1F3FD"},
    {"policeyskiy_cvet4", "1F46E-1F3FE"},
    {"policeyskiy_cvet5", "1F46E-1F3FF"},
    {"lyudi_s_ushami_krolika", "1F46F"},
    {"lyudi_s_ushami_krolika_cvet1", "1F46F-1F3FB"},
    {"lyudi_s_ushami_krolika_cvet2", "1F46F-1F3FC"},
    {"lyudi_s_ushami_krolika_cvet3", "1F46F-1F3FD"},
    {"lyudi_s_ushami_krolika_cvet4", "1F46F-1F3FE"},
    {"lyudi_s_ushami_krolika_cvet5", "1F46F-1F3FF"},
    {"lyudi_s_ushami_krolika_cvet1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"lyudi_s_ushami_krolika_cvet1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"lyudi_s_ushami_krolika_cvet1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"lyudi_s_ushami_krolika_cvet1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"lyudi_s_ushami_krolika_cvet2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"lyudi_s_ushami_krolika_cvet2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"lyudi_s_ushami_krolika_cvet2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"lyudi_s_ushami_krolika_cvet2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"lyudi_s_ushami_krolika_cvet3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"lyudi_s_ushami_krolika_cvet3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"lyudi_s_ushami_krolika_cvet3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"lyudi_s_ushami_krolika_cvet3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"lyudi_s_ushami_krolika_cvet4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"lyudi_s_ushami_krolika_cvet4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"lyudi_s_ushami_krolika_cvet4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"lyudi_s_ushami_krolika_cvet4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"lyudi_s_ushami_krolika_cvet5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"lyudi_s_ushami_krolika_cvet5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"lyudi_s_ushami_krolika_cvet5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"lyudi_s_ushami_krolika_cvet5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"nevesta", "1F470"},
    {"nevesta_cvet1", "1F470-1F3FB"},
    {"nevesta_cvet2", "1F470-1F3FC"},
    {"nevesta_cvet3", "1F470-1F3FD"},
    {"nevesta_cvet4", "1F470-1F3FE"},
    {"nevesta_cvet5", "1F470-1F3FF"},
    {"chelovek_so_svetlymi_volosami", "1F471"},
    {"chelovek_so_svetlymi_volosami_cvet1", "1F471-1F3FB"},
    {"chelovek_so_svetlymi_volosami_cvet2", "1F471-1F3FC"},
    {"chelovek_so_svetlymi_volosami_cvet3", "1F471-1F3FD"},
    {"chelovek_so_svetlymi_volosami_cvet4", "1F471-1F3FE"},
    {"chelovek_so_svetlymi_volosami_cvet5", "1F471-1F3FF"},
    {"muzhchina_v_kitayskoy_shapke", "1F472"},
    {"muzhchina_v_kitayskoy_shapke_cvet1", "1F472-1F3FB"},
    {"muzhchina_v_kitayskoy_shapke_cvet2", "1F472-1F3FC"},
    {"muzhchina_v_kitayskoy_shapke_cvet3", "1F472-1F3FD"},
    {"muzhchina_v_kitayskoy_shapke_cvet4", "1F472-1F3FE"},
    {"muzhchina_v_kitayskoy_shapke_cvet5", "1F472-1F3FF"},
    {"chelovek_v_tyurbane", "1F473"},
    {"chelovek_v_tyurbane_cvet1", "1F473-1F3FB"},
    {"chelovek_v_tyurbane_cvet2", "1F473-1F3FC"},
    {"chelovek_v_tyurbane_cvet3", "1F473-1F3FD"},
    {"chelovek_v_tyurbane_cvet4", "1F473-1F3FE"},
    {"chelovek_v_tyurbane_cvet5", "1F473-1F3FF"},
    {"pozhiloy_muzhchina", "1F474"},
    {"pozhiloy_muzhchina_cvet1", "1F474-1F3FB"},
    {"pozhiloy_muzhchina_cvet2", "1F474-1F3FC"},
    {"pozhiloy_muzhchina_cvet3", "1F474-1F3FD"},
    {"pozhiloy_muzhchina_cvet4", "1F474-1F3FE"},
    {"pozhiloy_muzhchina_cvet5", "1F474-1F3FF"},
    {"pozhilaya_zhenshchina", "1F475"},
    {"pozhilaya_zhenshchina_cvet1", "1F475-1F3FB"},
    {"pozhilaya_zhenshchina_cvet2", "1F475-1F3FC"},
    {"pozhilaya_zhenshchina_cvet3", "1F475-1F3FD"},
    {"pozhilaya_zhenshchina_cvet4", "1F475-1F3FE"},
    {"pozhilaya_zhenshchina_cvet5", "1F475-1F3FF"},
    {"mladenec", "1F476"},
    {"mladenec_cvet1", "1F476-1F3FB"},
    {"mladenec_cvet2", "1F476-1F3FC"},
    {"mladenec_cvet3", "1F476-1F3FD"},
    {"mladenec_cvet4", "1F476-1F3FE"},
    {"mladenec_cvet5", "1F476-1F3FF"},
    {"stroitel", "1F477"},
    {"stroitel_cvet1", "1F477-1F3FB"},
    {"stroitel_cvet2", "1F477-1F3FC"},
    {"stroitel_cvet3", "1F477-1F3FD"},
    {"stroitel_cvet4", "1F477-1F3FE"},
    {"stroitel_cvet5", "1F477-1F3FF"},
    {"princessa", "1F478"},
    {"princessa_cvet1", "1F478-1F3FB"},
    {"princessa_cvet2", "1F478-1F3FC"},
    {"princessa_cvet3", "1F478-1F3FD"},
    {"princessa_cvet4", "1F478-1F3FE"},
    {"princessa_cvet5", "1F478-1F3FF"},
    {"chudovishche", "1F479"},
    {"yaponskiy_goblin", "1F47A"},
    {"prividenie", "1F47B"},
    {"angelochek", "1F47C"},
    {"angelochek_cvet1", "1F47C-1F3FB"},
    {"angelochek_cvet2", "1F47C-1F3FC"},
    {"angelochek_cvet3", "1F47C-1F3FD"},
    {"angelochek_cvet4", "1F47C-1F3FE"},
    {"angelochek_cvet5", "1F47C-1F3FF"},
    {"inoplanetyanin", "1F47D"},
    {"kosmicheskiy_monstr", "1F47E"},
    {"zloy_chertenok", "1F47F"},
    {"cherep", "1F480"},
    {"chelovek_tyanet_ruku", "1F481"},
    {"chelovek_tyanet_ruku_cvet1", "1F481-1F3FB"},
    {"chelovek_tyanet_ruku_cvet2", "1F481-1F3FC"},
    {"chelovek_tyanet_ruku_cvet3", "1F481-1F3FD"},
    {"chelovek_tyanet_ruku_cvet4", "1F481-1F3FE"},
    {"chelovek_tyanet_ruku_cvet5", "1F481-1F3FF"},
    {"gvardeec", "1F482"},
    {"gvardeec_cvet1", "1F482-1F3FB"},
    {"gvardeec_cvet2", "1F482-1F3FC"},
    {"gvardeec_cvet3", "1F482-1F3FD"},
    {"gvardeec_cvet4", "1F482-1F3FE"},
    {"gvardeec_cvet5", "1F482-1F3FF"},
    {"tancuyushchaya_zhenshchina", "1F483"},
    {"tancuyushchaya_zhenshchina_cvet1", "1F483-1F3FB"},
    {"tancuyushchaya_zhenshchina_cvet2", "1F483-1F3FC"},
    {"tancuyushchaya_zhenshchina_cvet3", "1F483-1F3FD"},
    {"tancuyushchaya_zhenshchina_cvet4", "1F483-1F3FE"},
    {"tancuyushchaya_zhenshchina_cvet5", "1F483-1F3FF"},
    {"pomada", "1F484"},
    {"manikyur", "1F485"},
    {"manikyur_cvet1", "1F485-1F3FB"},
    {"manikyur_cvet2", "1F485-1F3FC"},
    {"manikyur_cvet3", "1F485-1F3FD"},
    {"manikyur_cvet4", "1F485-1F3FE"},
    {"manikyur_cvet5", "1F485-1F3FF"},
    {"cheloveku_massiruyut_lico", "1F486"},
    {"cheloveku_massiruyut_lico_cvet1", "1F486-1F3FB"},
    {"cheloveku_massiruyut_lico_cvet2", "1F486-1F3FC"},
    {"cheloveku_massiruyut_lico_cvet3", "1F486-1F3FD"},
    {"cheloveku_massiruyut_lico_cvet4", "1F486-1F3FE"},
    {"cheloveku_massiruyut_lico_cvet5", "1F486-1F3FF"},
    {"chelovek_u_parikmahera", "1F487"},
    {"chelovek_u_parikmahera_cvet1", "1F487-1F3FB"},
    {"chelovek_u_parikmahera_cvet2", "1F487-1F3FC"},
    {"chelovek_u_parikmahera_cvet3", "1F487-1F3FD"},
    {"chelovek_u_parikmahera_cvet4", "1F487-1F3FE"},
    {"chelovek_u_parikmahera_cvet5", "1F487-1F3FF"},
    {"parikmaherskaya", "1F488"},
    {"shpric", "1F489"},
    {"pilyulya", "1F48A"},
    {"sled_ot_poceluya", "1F48B"},
    {"lyubovnoe_pismo", "1F48C"},
    {"kolco", "1F48D"},
    {"dragocennyy_kamen", "1F48E"},
    {"poceluy", "1F48F"},
    {"poceluy_cvet1", "1F48F-1F3FB"},
    {"poceluy_cvet2", "1F48F-1F3FC"},
    {"poceluy_cvet3", "1F48F-1F3FD"},
    {"poceluy_cvet4", "1F48F-1F3FE"},
    {"poceluy_cvet5", "1F48F-1F3FF"},
    {"poceluy_cvet1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"poceluy_cvet1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"poceluy_cvet1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"poceluy_cvet1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"poceluy_cvet2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"poceluy_cvet2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"poceluy_cvet2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"poceluy_cvet2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"poceluy_cvet3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"poceluy_cvet3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"poceluy_cvet3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"poceluy_cvet3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"poceluy_cvet4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"poceluy_cvet4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"poceluy_cvet4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"poceluy_cvet4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"poceluy_cvet5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"poceluy_cvet5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"poceluy_cvet5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"poceluy_cvet5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"buket", "1F490"},
    {"vlyublennaya_para", "1F491"},
    {"vlyublennaya_para_cvet1", "1F491-1F3FB"},
    {"vlyublennaya_para_cvet2", "1F491-1F3FC"},
    {"vlyublennaya_para_cvet3", "1F491-1F3FD"},
    {"vlyublennaya_para_cvet4", "1F491-1F3FE"},
    {"vlyublennaya_para_cvet5", "1F491-1F3FF"},
    {"vlyublennaya_para_cvet1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"vlyublennaya_para_cvet1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"vlyublennaya_para_cvet1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"vlyublennaya_para_cvet1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"vlyublennaya_para_cvet2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"vlyublennaya_para_cvet2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"vlyublennaya_para_cvet2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"vlyublennaya_para_cvet2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"vlyublennaya_para_cvet3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"vlyublennaya_para_cvet3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"vlyublennaya_para_cvet3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"vlyublennaya_para_cvet3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"vlyublennaya_para_cvet4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"vlyublennaya_para_cvet4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"vlyublennaya_para_cvet4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"vlyublennaya_para_cvet4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"vlyublennaya_para_cvet5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"vlyublennaya_para_cvet5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"vlyublennaya_para_cvet5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"vlyublennaya_para_cvet5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"svadba", "1F492"},
    {"pulsiruyushchee_serdce", "1F493"},
    {"razbitoe_serdce", "1F494"},
    {"dva_serdca", "1F495"},
    {"sverkayushchee_serdce", "1F496"},
    {"rastushchee_serdce", "1F497"},
    {"serdce_pronzennoe_streloy", "1F498"},
    {"sinee_serdce", "1F499"},
    {"zelenoe_serdce", "1F49A"},
    {"zheltoe_serdce", "1F49B"},
    {"fioletovoe_serdce", "1F49C"},
    {"serdce_s_lentoy", "1F49D"},
    {"vrashchayushchiesya_serdca", "1F49E"},
    {"znachok_serdce", "1F49F"},
    {"romb_s_tochkoy", "1F4A0"},
    {"lampochka", "1F4A1"},
    {"znachok_gnev", "1F4A2"},
    {"bomba", "1F4A3"},
    {"son", "1F4A4"},
    {"vzryv", "1F4A5"},
    {"kapli_pota", "1F4A6"},
    {"kaplya", "1F4A7"},
    {"ischeznovenie", "1F4A8"},
    {"kakashka", "1F4A9"},
    {"biceps", "1F4AA"},
    {"biceps_cvet1", "1F4AA-1F3FB"},
    {"biceps_cvet2", "1F4AA-1F3FC"},
    {"biceps_cvet3", "1F4AA-1F3FD"},
    {"biceps_cvet4", "1F4AA-1F3FE"},
    {"biceps_cvet5", "1F4AA-1F3FF"},
    {"kruzhashchayasya_zvezda", "1F4AB"},
    {"oblachko_s_mnogotochiem", "1F4AC"},
    {"oblachko_s_myslyami", "1F4AD"},
    {"belyy_cvetok", "1F4AE"},
    {"sto_ballov", "1F4AF"},
    {"meshok_deneg", "1F4B0"},
    {"obmen_valyuty", "1F4B1"},
    {"zhirnyy_znak_dollara", "1F4B2"},
    {"kreditnaya_karta", "1F4B3"},
    {"banknota_ieny", "1F4B4"},
    {"banknota_dollara", "1F4B5"},
    {"banknota_evro", "1F4B6"},
    {"banknota_funta", "1F4B7"},
    {"banknota_s_krylyshkami", "1F4B8"},
    {"birzha", "1F4B9"},
    {"kreslo", "1F4BA"},
    {"noutbuk", "1F4BB"},
    {"portfel", "1F4BC"},
    {"disk", "1F4BD"},
    {"disketa", "1F4BE"},
    {"cd", "1F4BF"},
    {"dvd", "1F4C0"},
    {"papka", "1F4C1"},
    {"otkrytaya_papka", "1F4C2"},
    {"stranica_s_zagibom", "1F4C3"},
    {"stranica", "1F4C4"},
    {"kalendar", "1F4C5"},
    {"otryvnoy_kalendar", "1F4C6"},
    {"organayzer", "1F4C7"},
    {"diagramma_rosta", "1F4C8"},
    {"diagramma_padeniya", "1F4C9"},
    {"gistogramma", "1F4CA"},
    {"planshet_s_zazhimom", "1F4CB"},
    {"kancelyarskaya_knopka", "1F4CC"},
    {"kancelyarskaya_knopka_s_sharikom", "1F4CD"},
    {"skrepka", "1F4CE"},
    {"lineyka", "1F4CF"},
    {"lineyka_treugolnik", "1F4D0"},
    {"stikery_zakladki", "1F4D1"},
    {"bloknot", "1F4D2"},
    {"tetrad", "1F4D3"},
    {"zapisnaya_knizhka", "1F4D4"},
    {"zakrytaya_kniga", "1F4D5"},
    {"otkrytaya_kniga", "1F4D6"},
    {"zelenaya_kniga", "1F4D7"},
    {"sinyaya_kniga", "1F4D8"},
    {"oranzhevaya_kniga", "1F4D9"},
    {"knigi", "1F4DA"},
    {"beydzh", "1F4DB"},
    {"svitok", "1F4DC"},
    {"zametka", "1F4DD"},
    {"telefonnaya_trubka", "1F4DE"},
    {"peydzher", "1F4DF"},
    {"faks", "1F4E0"},
    {"sputnikovaya_antenna", "1F4E1"},
    {"gromkogovoritel", "1F4E2"},
    {"megafon", "1F4E3"},
    {"ishodyashchie", "1F4E4"},
    {"vhodyashchie", "1F4E5"},
    {"posylka", "1F4E6"},
    {"elektronnaya_pochta", "1F4E7"},
    {"vhodyashchee_pismo", "1F4E8"},
    {"ishodyashchee_pismo", "1F4E9"},
    {"zakrytyy_pochtovyy_yashchik_s_opushchennym_flazhkom", "1F4EA"},
    {"zakrytyy_pochtovyy_yashchik_s_podnyatym_flazhkom", "1F4EB"},
    {"otkrytyy_pochtovyy_yashchik_s_podnyatym_flazhkom", "1F4EC"},
    {"otkrytyy_pochtovyy_yashchik_s_opushchennym_flazhkom", "1F4ED"},
    {"pochtovyy_yashchik", "1F4EE"},
    {"pochtovyy_rozhok", "1F4EF"},
    {"gazeta", "1F4F0"},
    {"mobilnyy_telefon", "1F4F1"},
    {"vhodyashchiy_vyzov", "1F4F2"},
    {"rezhim_vibracii", "1F4F3"},
    {"vyklyuchennyy_telefon", "1F4F4"},
    {"mobilnye_telefony_zapreshcheny", "1F4F5"},
    {"signal_seti", "1F4F6"},
    {"fotoapparat", "1F4F7"},
    {"fotoapparat_so_vspyshkoy", "1F4F8"},
    {"videokamera", "1F4F9"},
    {"televizor", "1F4FA"},
    {"radio", "1F4FB"},
    {"videokasseta", "1F4FC"},
    {"kinoproektor", "1F4FD"},
    {"chetki", "1F4FF"},
    {"sluchaynyy_vybor", "1F500"},
    {"povtor", "1F501"},
    {"povtor_odnoy_pesni", "1F502"},
    {"po_chasovoy_strelke", "1F503"},
    {"protiv_chasovoy_strelki", "1F504"},
    {"nizkaya_yarkost", "1F505"},
    {"vysokaya_yarkost", "1F506"},
    {"bez_zvuka", "1F507"},
    {"nizkaya_gromkost", "1F508"},
    {"srednyaya_gromkost", "1F509"},
    {"vysokaya_gromkost", "1F50A"},
    {"batareyka", "1F50B"},
    {"vilka", "1F50C"},
    {"lupa_naklonennaya_vlevo", "1F50D"},
    {"lupa_naklonennaya_vpravo", "1F50E"},
    {"zakrytyy_zamok_s_perevoy_ruchkoy", "1F50F"},
    {"zakrytyy_zamok_s_klyuchom", "1F510"},
    {"klyuch", "1F511"},
    {"zakrytyy_zamok", "1F512"},
    {"otkrytyy_zamok", "1F513"},
    {"kolokolchik", "1F514"},
    {"perecherknutyy_kolokolchik", "1F515"},
    {"zakladka", "1F516"},
    {"giperssylka", "1F517"},
    {"kruglaya_knopka", "1F518"},
    {"strelka_nazad", "1F519"},
    {"strelka_konec", "1F51A"},
    {"strelka_aktivno", "1F51B"},
    {"strelka_skoro", "1F51C"},
    {"strelka_verh", "1F51D"},
    {"dlya_lyudey_starshe_vosemnadcati_let", "1F51E"},
    {"klavishi_10", "1F51F"},
    {"vvod_propisnymi", "1F520"},
    {"vvod_strochnymi", "1F521"},
    {"vvod_cifr", "1F522"},
    {"vvod_simvolov", "1F523"},
    {"vvod_latinicey", "1F524"},
    {"ogon", "1F525"},
    {"fonarik", "1F526"},
    {"gaechnyy_klyuch", "1F527"},
    {"molotok", "1F528"},
    {"bolt_i_gayka", "1F529"},
    {"nozh", "1F52A"},
    {"vodyanoy_pistolet", "1F52B"},
    {"mikroskop", "1F52C"},
    {"teleskop", "1F52D"},
    {"hrustalnyy_shar", "1F52E"},
    {"shestikonechnaya_zvezda", "1F52F"},
    {"simvol_novichka", "1F530"},
    {"trezubec", "1F531"},
    {"knopka_s_chernoy_ramkoy", "1F532"},
    {"knopka_s_beloy_ramkoy", "1F533"},
    {"krasnyy_krug", "1F534"},
    {"goluboy_krug", "1F535"},
    {"bolshoy_oranzhevyy_romb", "1F536"},
    {"bolshoy_goluboy_romb", "1F537"},
    {"nebolshoy_oranzhevyy_romb", "1F538"},
    {"nebolshoy_goluboy_romb", "1F539"},
    {"treugolnik_vershinoy_vverh", "1F53A"},
    {"treugolnik_vershinoy_vniz", "1F53B"},
    {"treugolnik_ostriem_vverh", "1F53C"},
    {"treugolnik_ostriem_vniz", "1F53D"},
    {"om", "1F549"},
    {"golub", "1F54A"},
    {"kaaba", "1F54B"},
    {"mechet", "1F54C"},
    {"sinagoga", "1F54D"},
    {"menora", "1F54E"},
    {"chas", "1F550"},
    {"dva_chasa", "1F551"},
    {"tri_chasa", "1F552"},
    {"chetyre_chasa", "1F553"},
    {"pyat_chasov", "1F554"},
    {"shest_chasov", "1F555"},
    {"sem_chasov", "1F556"},
    {"vosem_chasov", "1F557"},
    {"devyat_chasov", "1F558"},
    {"desyat_chasov", "1F559"},
    {"odinnadcat_chasov", "1F55A"},
    {"dvenadcat_chasov", "1F55B"},
    {"polvtorogo", "1F55C"},
    {"poltretego", "1F55D"},
    {"polchetvertogo", "1F55E"},
    {"polpyatogo", "1F55F"},
    {"polshestogo", "1F560"},
    {"polsedmogo", "1F561"},
    {"polvosmogo", "1F562"},
    {"poldevyatogo", "1F563"},
    {"poldesyatogo", "1F564"},
    {"pol_odinnadcatogo", "1F565"},
    {"poldvenadcatogo", "1F566"},
    {"polpervogo", "1F567"},
    {"svecha", "1F56F"},
    {"kaminnye_chasy", "1F570"},
    {"dyra", "1F573"},
    {"biznesmen_v_vozduhe", "1F574"},
    {"biznesmen_v_vozduhe_cvet1", "1F574-1F3FB"},
    {"biznesmen_v_vozduhe_cvet2", "1F574-1F3FC"},
    {"biznesmen_v_vozduhe_cvet3", "1F574-1F3FD"},
    {"biznesmen_v_vozduhe_cvet4", "1F574-1F3FE"},
    {"biznesmen_v_vozduhe_cvet5", "1F574-1F3FF"},
    {"detektiv", "1F575"},
    {"detektiv_cvet1", "1F575-1F3FB"},
    {"detektiv_cvet2", "1F575-1F3FC"},
    {"detektiv_cvet3", "1F575-1F3FD"},
    {"detektiv_cvet4", "1F575-1F3FE"},
    {"detektiv_cvet5", "1F575-1F3FF"},
    {"solnechnye_ochki", "1F576"},
    {"pauk", "1F577"},
    {"pautina", "1F578"},
    {"dzhoystik", "1F579"},
    {"tancuyushchiy_muzhchina", "1F57A"},
    {"tancuyushchiy_muzhchina_cvet1", "1F57A-1F3FB"},
    {"tancuyushchiy_muzhchina_cvet2", "1F57A-1F3FC"},
    {"tancuyushchiy_muzhchina_cvet3", "1F57A-1F3FD"},
    {"tancuyushchiy_muzhchina_cvet4", "1F57A-1F3FE"},
    {"tancuyushchiy_muzhchina_cvet5", "1F57A-1F3FF"},
    {"skrepki", "1F587"},
    {"sharikovaya_ruchka", "1F58A"},
    {"perevaya_ruchka", "1F58B"},
    {"kistochka", "1F58C"},
    {"voskovoy_melok", "1F58D"},
    {"raskrytaya_ladon", "1F590"},
    {"raskrytaya_ladon_cvet1", "1F590-1F3FB"},
    {"raskrytaya_ladon_cvet2", "1F590-1F3FC"},
    {"raskrytaya_ladon_cvet3", "1F590-1F3FD"},
    {"raskrytaya_ladon_cvet4", "1F590-1F3FE"},
    {"raskrytaya_ladon_cvet5", "1F590-1F3FF"},
    {"sredniy_palec", "1F595"},
    {"sredniy_palec_cvet1", "1F595-1F3FB"},
    {"sredniy_palec_cvet2", "1F595-1F3FC"},
    {"sredniy_palec_cvet3", "1F595-1F3FD"},
    {"sredniy_palec_cvet4", "1F595-1F3FE"},
    {"sredniy_palec_cvet5", "1F595-1F3FF"},
    {"vulkanskiy_salyut", "1F596"},
    {"vulkanskiy_salyut_cvet1", "1F596-1F3FB"},
    {"vulkanskiy_salyut_cvet2", "1F596-1F3FC"},
    {"vulkanskiy_salyut_cvet3", "1F596-1F3FD"},
    {"vulkanskiy_salyut_cvet4", "1F596-1F3FE"},
    {"vulkanskiy_salyut_cvet5", "1F596-1F3FF"},
    {"chernoe_serdce", "1F5A4"},
    {"kompyuter", "1F5A5"},
    {"printer", "1F5A8"},
    {"myshka", "1F5B1"},
    {"trekbol", "1F5B2"},
    {"kartina", "1F5BC"},
    {"markirovannye_papki", "1F5C2"},
    {"kartoteka", "1F5C3"},
    {"arhivnyy_shkaf", "1F5C4"},
    {"korzina_dlya_musora", "1F5D1"},
    {"perekidnoy_bloknot", "1F5D2"},
    {"perekidnoy_kalendar", "1F5D3"},
    {"kompressor", "1F5DC"},
    {"starinnyy_klyuch", "1F5DD"},
    {"svernutaya_gazeta", "1F5DE"},
    {"kinzhal", "1F5E1"},
    {"govoryashchaya_golova", "1F5E3"},
    {"pustoe_oblachko", "1F5E8"},
    {"gnevnoe_oblachko", "1F5EF"},
    {"izbiratelnaya_urna", "1F5F3"},
    {"karta_mira", "1F5FA"},
    {"fudziyama", "1F5FB"},
    {"tokiyskaya_bashnya", "1F5FC"},
    {"statuya_svobody", "1F5FD"},
    {"karta_yaponii", "1F5FE"},
    {"istukan", "1F5FF"},
    {"shiroko_ulybaetsya", "1F600"},
    {"svetitsya_ot_schastya", "1F601"},
    {"smeetsya_do_slez", "1F602"},
    {"smeetsya", "1F603"},
    {"smeetsya_s_zakrytymi_glazami", "1F604"},
    {"nervno_smeetsya", "1F605"},
    {"smeetsya_s_prishchurennymi_glazami", "1F606"},
    {"s_nimbom", "1F607"},
    {"ulybayushchiysya_chertenok", "1F608"},
    {"podmigivaet", "1F609"},
    {"dovolno_ulybaetsya", "1F60A"},
    {"vkusno", "1F60B"},
    {"ispytyvaet_oblegchenie", "1F60C"},
    {"vlyublennoe_lico", "1F60D"},
    {"lico_v_temnyh_ochkah", "1F60E"},
    {"uhmylyaetsya", "1F60F"},
    {"vse_ravno", "1F610"},
    {"bez_emociy", "1F611"},
    {"skuchaet", "1F612"},
    {"v_holodnom_potu", "1F613"},
    {"razmyshlyaet", "1F614"},
    {"v_zameshatelstve", "1F615"},
    {"v_rasteryannosti", "1F616"},
    {"celuet", "1F617"},
    {"vozdushnyy_poceluy", "1F618"},
    {"schastlivyy_poceluy", "1F619"},
    {"celuet_s_zakrytymi_glazami", "1F61A"},
    {"pokazyvaet_yazyk", "1F61B"},
    {"pokazyvaet_yazyk_i_podmigivaet", "1F61C"},
    {"morshchitsya_i_pokazyvaet_yazyk", "1F61D"},
    {"razocharovanie", "1F61E"},
    {"bespokoitsya", "1F61F"},
    {"serditsya", "1F620"},
    {"zlitsya", "1F621"},
    {"plachet", "1F622"},
    {"uporstvo", "1F623"},
    {"v_ozhidanii_uspeha", "1F624"},
    {"oboshlos", "1F625"},
    {"v_izumlenii", "1F626"},
    {"stradaet", "1F627"},
    {"boitsya", "1F628"},
    {"ustalost", "1F629"},
    {"hochet_spat", "1F62A"},
    {"silnaya_ustalost", "1F62B"},
    {"grimasa", "1F62C"},
    {"slezy_rekoy", "1F62D"},
    {"udivlenie", "1F62E"},
    {"nepriyatnyy_syurpriz", "1F62F"},
    {"silno_bespokoitsya", "1F630"},
    {"v_uzhase", "1F631"},
    {"v_shoke", "1F632"},
    {"krasneet", "1F633"},
    {"spit", "1F634"},
    {"golovokruzhenie", "1F635"},
    {"lico_bez_rta", "1F636"},
    {"v_medicinskoy_maske", "1F637"},
    {"smeyushchiysya_kot", "1F638"},
    {"smeyushchiysya_do_slez_kot", "1F639"},
    {"ulybayushchiysya_kot", "1F63A"},
    {"vlyublennyy_kot", "1F63B"},
    {"uhmylyayushchiysya_kot", "1F63C"},
    {"celuyushchiy_kot", "1F63D"},
    {"zloy_kot", "1F63E"},
    {"plachushchiy_kot", "1F63F"},
    {"kot_v_shoke", "1F640"},
    {"nemnogo_grustit", "1F641"},
    {"slegka_ulybaetsya", "1F642"},
    {"vverh_tormashkami", "1F643"},
    {"zakatyvaet_glaza", "1F644"},
    {"chelovek_protiv", "1F645"},
    {"chelovek_protiv_cvet1", "1F645-1F3FB"},
    {"chelovek_protiv_cvet2", "1F645-1F3FC"},
    {"chelovek_protiv_cvet3", "1F645-1F3FD"},
    {"chelovek_protiv_cvet4", "1F645-1F3FE"},
    {"chelovek_protiv_cvet5", "1F645-1F3FF"},
    {"chelovek_za", "1F646"},
    {"chelovek_za_cvet1", "1F646-1F3FB"},
    {"chelovek_za_cvet2", "1F646-1F3FC"},
    {"chelovek_za_cvet3", "1F646-1F3FD"},
    {"chelovek_za_cvet4", "1F646-1F3FE"},
    {"chelovek_za_cvet5", "1F646-1F3FF"},
    {"chelovek_klanyaetsya", "1F647"},
    {"chelovek_klanyaetsya_cvet1", "1F647-1F3FB"},
    {"chelovek_klanyaetsya_cvet2", "1F647-1F3FC"},
    {"chelovek_klanyaetsya_cvet3", "1F647-1F3FD"},
    {"chelovek_klanyaetsya_cvet4", "1F647-1F3FE"},
    {"chelovek_klanyaetsya_cvet5", "1F647-1F3FF"},
    {"nichego_ne_vizhu", "1F648"},
    {"nichego_ne_slyshu", "1F649"},
    {"nichego_nikomu_ne_skazhu", "1F64A"},
    {"chelovek_s_podnyatoy_rukoy", "1F64B"},
    {"chelovek_s_podnyatoy_rukoy_cvet1", "1F64B-1F3FB"},
    {"chelovek_s_podnyatoy_rukoy_cvet2", "1F64B-1F3FC"},
    {"chelovek_s_podnyatoy_rukoy_cvet3", "1F64B-1F3FD"},
    {"chelovek_s_podnyatoy_rukoy_cvet4", "1F64B-1F3FE"},
    {"chelovek_s_podnyatoy_rukoy_cvet5", "1F64B-1F3FF"},
    {"ura", "1F64C"},
    {"ura_cvet1", "1F64C-1F3FB"},
    {"ura_cvet2", "1F64C-1F3FC"},
    {"ura_cvet3", "1F64C-1F3FD"},
    {"ura_cvet4", "1F64C-1F3FE"},
    {"ura_cvet5", "1F64C-1F3FF"},
    {"nahmurivshiysya_chelovek", "1F64D"},
    {"nahmurivshiysya_chelovek_cvet1", "1F64D-1F3FB"},
    {"nahmurivshiysya_chelovek_cvet2", "1F64D-1F3FC"},
    {"nahmurivshiysya_chelovek_cvet3", "1F64D-1F3FD"},
    {"nahmurivshiysya_chelovek_cvet4", "1F64D-1F3FE"},
    {"nahmurivshiysya_chelovek_cvet5", "1F64D-1F3FF"},
    {"obizhennyy_chelovek", "1F64E"},
    {"obizhennyy_chelovek_cvet1", "1F64E-1F3FB"},
    {"obizhennyy_chelovek_cvet2", "1F64E-1F3FC"},
    {"obizhennyy_chelovek_cvet3", "1F64E-1F3FD"},
    {"obizhennyy_chelovek_cvet4", "1F64E-1F3FE"},
    {"obizhennyy_chelovek_cvet5", "1F64E-1F3FF"},
    {"slozhennye_ruki", "1F64F"},
    {"slozhennye_ruki_cvet1", "1F64F-1F3FB"},
    {"slozhennye_ruki_cvet2", "1F64F-1F3FC"},
    {"slozhennye_ruki_cvet3", "1F64F-1F3FD"},
    {"slozhennye_ruki_cvet4", "1F64F-1F3FE"},
    {"slozhennye_ruki_cvet5", "1F64F-1F3FF"},
    {"raketa", "1F680"},
    {"vertolet", "1F681"},
    {"parovoz", "1F682"},
    {"vagon", "1F683"},
    {"skorostnoy_poezd", "1F684"},
    {"poezd_pulya", "1F685"},
    {"poezd", "1F686"},
    {"metro", "1F687"},
    {"legkiy_relsovyy_transport", "1F688"},
    {"zheleznodorozhnyy_vokzal", "1F689"},
    {"tramvay", "1F68A"},
    {"tramvaynyy_vagon", "1F68B"},
    {"avtobus", "1F68C"},
    {"avtobus_speredi", "1F68D"},
    {"trolleybus", "1F68E"},
    {"avtobusnaya_ostanovka", "1F68F"},
    {"mikroavtobus", "1F690"},
    {"mashina_skoroy_pomoshchi", "1F691"},
    {"pozharnaya_mashina", "1F692"},
    {"policeyskaya_mashina", "1F693"},
    {"policeyskaya_mashina_speredi", "1F694"},
    {"taksi", "1F695"},
    {"taksi_speredi", "1F696"},
    {"avtomobil", "1F697"},
    {"avtomobil_speredi", "1F698"},
    {"dzhip", "1F699"},
    {"gruzovik", "1F69A"},
    {"fura", "1F69B"},
    {"traktor", "1F69C"},
    {"monorels", "1F69D"},
    {"poezd_v_gorah", "1F69E"},
    {"podvesnaya_zheleznaya_doroga", "1F69F"},
    {"vagon_funikulera", "1F6A0"},
    {"kanatnaya_doroga", "1F6A1"},
    {"korabl", "1F6A2"},
    {"greblya", "1F6A3"},
    {"greblya_cvet1", "1F6A3-1F3FB"},
    {"greblya_cvet2", "1F6A3-1F3FC"},
    {"greblya_cvet3", "1F6A3-1F3FD"},
    {"greblya_cvet4", "1F6A3-1F3FE"},
    {"greblya_cvet5", "1F6A3-1F3FF"},
    {"kater", "1F6A4"},
    {"gorizontalnyy_svetofor", "1F6A5"},
    {"vertikalnyy_svetofor", "1F6A6"},
    {"stroitelnye_raboty", "1F6A7"},
    {"migalka", "1F6A8"},
    {"treugolnyy_flag", "1F6A9"},
    {"dver", "1F6AA"},
    {"zapreshcheno", "1F6AB"},
    {"sigareta", "1F6AC"},
    {"kurenie_zapreshcheno", "1F6AD"},
    {"brosayte_musor_v_urnu", "1F6AE"},
    {"ne_musorte", "1F6AF"},
    {"pitevaya_voda", "1F6B0"},
    {"nepitevaya_voda", "1F6B1"},
    {"velosiped", "1F6B2"},
    {"perecherknutyy_velosiped", "1F6B3"},
    {"velosipedist", "1F6B4"},
    {"velosipedist_cvet1", "1F6B4-1F3FB"},
    {"velosipedist_cvet2", "1F6B4-1F3FC"},
    {"velosipedist_cvet3", "1F6B4-1F3FD"},
    {"velosipedist_cvet4", "1F6B4-1F3FE"},
    {"velosipedist_cvet5", "1F6B4-1F3FF"},
    {"chelovek_na_gornom_velosipede", "1F6B5"},
    {"chelovek_na_gornom_velosipede_cvet1", "1F6B5-1F3FB"},
    {"chelovek_na_gornom_velosipede_cvet2", "1F6B5-1F3FC"},
    {"chelovek_na_gornom_velosipede_cvet3", "1F6B5-1F3FD"},
    {"chelovek_na_gornom_velosipede_cvet4", "1F6B5-1F3FE"},
    {"chelovek_na_gornom_velosipede_cvet5", "1F6B5-1F3FF"},
    {"chelovek_idet", "1F6B6"},
    {"chelovek_idet_cvet1", "1F6B6-1F3FB"},
    {"chelovek_idet_cvet2", "1F6B6-1F3FC"},
    {"chelovek_idet_cvet3", "1F6B6-1F3FD"},
    {"chelovek_idet_cvet4", "1F6B6-1F3FE"},
    {"chelovek_idet_cvet5", "1F6B6-1F3FF"},
    {"perehod_zapreshchen", "1F6B7"},
    {"znak_deti", "1F6B8"},
    {"muzhskoy_tualet", "1F6B9"},
    {"zhenskiy_tualet", "1F6BA"},
    {"ubornaya", "1F6BB"},
    {"pelenalnaya", "1F6BC"},
    {"unitaz", "1F6BD"},
    {"tualet", "1F6BE"},
    {"dush", "1F6BF"},
    {"chelovek_v_vanne", "1F6C0"},
    {"chelovek_v_vanne_cvet1", "1F6C0-1F3FB"},
    {"chelovek_v_vanne_cvet2", "1F6C0-1F3FC"},
    {"chelovek_v_vanne_cvet3", "1F6C0-1F3FD"},
    {"chelovek_v_vanne_cvet4", "1F6C0-1F3FE"},
    {"chelovek_v_vanne_cvet5", "1F6C0-1F3FF"},
    {"vanna", "1F6C1"},
    {"pasportnyy_kontrol", "1F6C2"},
    {"tamozhnya", "1F6C3"},
    {"poluchenie_bagazha", "1F6C4"},
    {"zabytyy_bagazh", "1F6C5"},
    {"divan_i_torsher", "1F6CB"},
    {"chelovek_v_krovati", "1F6CC"},
    {"chelovek_v_krovati_cvet1", "1F6CC-1F3FB"},
    {"chelovek_v_krovati_cvet2", "1F6CC-1F3FC"},
    {"chelovek_v_krovati_cvet3", "1F6CC-1F3FD"},
    {"chelovek_v_krovati_cvet4", "1F6CC-1F3FE"},
    {"chelovek_v_krovati_cvet5", "1F6CC-1F3FF"},
    {"pakety", "1F6CD"},
    {"gostinichnyy_zvonok", "1F6CE"},
    {"krovat", "1F6CF"},
    {"religiya", "1F6D0"},
    {"znak_stop", "1F6D1"},
    {"telezhka_dlya_pokupok", "1F6D2"},
    {"induistskiy_hram", "1F6D5"},
    {"hizhina", "1F6D6"},
    {"lift", "1F6D7"},
    {"opolzen", "1F6D8"},
    {"besprovodnaya_set", "1F6DC"},
    {"detskaya_gorka", "1F6DD"},
    {"avtomobilnoe_koleso", "1F6DE"},
    {"spasatelnyy_krug", "1F6DF"},
    {"molot_i_gaechnyy_klyuch", "1F6E0"},
    {"shchit", "1F6E1"},
    {"bochka", "1F6E2"},
    {"shosse", "1F6E3"},
    {"zheleznaya_doroga", "1F6E4"},
    {"motornaya_lodka", "1F6E5"},
    {"malenkiy_samolet", "1F6E9"},
    {"vzlet", "1F6EB"},
    {"posadka", "1F6EC"},
    {"sputnik", "1F6F0"},
    {"passazhirskiy_layner", "1F6F3"},
    {"samokat", "1F6F4"},
    {"skuter", "1F6F5"},
    {"kanoe", "1F6F6"},
    {"sanki", "1F6F7"},
    {"letayushchaya_tarelka", "1F6F8"},
    {"skeytbord", "1F6F9"},
    {"avtoriksha", "1F6FA"},
    {"pikap", "1F6FB"},
    {"rolikovye_konki", "1F6FC"},
    {"oranzhevyy_krug", "1F7E0"},
    {"zheltyy_krug", "1F7E1"},
    {"zelenyy_krug", "1F7E2"},
    {"fioletovyy_krug", "1F7E3"},
    {"korichnevyy_krug", "1F7E4"},
    {"krasnyy_kvadrat", "1F7E5"},
    {"siniy_kvadrat", "1F7E6"},
    {"oranzhevyy_kvadrat", "1F7E7"},
    {"zheltyy_kvadrat", "1F7E8"},
    {"zelenyy_kvadrat", "1F7E9"},
    {"fioletovyy_kvadrat", "1F7EA"},
    {"korichnevyy_kvadrat", "1F7EB"},
    {"zhirnyy_znak_ravenstva", "1F7F0"},
    {"shchepot", "1F90C"},
    {"shchepot_cvet1", "1F90C-1F3FB"},
    {"shchepot_cvet2", "1F90C-1F3FC"},
    {"shchepot_cvet3", "1F90C-1F3FD"},
    {"shchepot_cvet4", "1F90C-1F3FE"},
    {"shchepot_cvet5", "1F90C-1F3FF"},
    {"beloe_serdce", "1F90D"},
    {"korichnevoe_serdce", "1F90E"},
    {"zhest_malo", "1F90F"},
    {"zhest_malo_cvet1", "1F90F-1F3FB"},
    {"zhest_malo_cvet2", "1F90F-1F3FC"},
    {"zhest_malo_cvet3", "1F90F-1F3FD"},
    {"zhest_malo_cvet4", "1F90F-1F3FE"},
    {"zhest_malo_cvet5", "1F90F-1F3FF"},
    {"rot_na_zamke", "1F910"},
    {"dengi_v_glazah", "1F911"},
    {"s_gradusnikom_vo_rtu", "1F912"},
    {"ochkarik", "1F913"},
    {"dumaet", "1F914"},
    {"s_perevyazannoy_golovoy", "1F915"},
    {"robot", "1F916"},
    {"obnimaet", "1F917"},
    {"rokerskiy_zhest", "1F918"},
    {"rokerskiy_zhest_cvet1", "1F918-1F3FB"},
    {"rokerskiy_zhest_cvet2", "1F918-1F3FC"},
    {"rokerskiy_zhest_cvet3", "1F918-1F3FD"},
    {"rokerskiy_zhest_cvet4", "1F918-1F3FE"},
    {"rokerskiy_zhest_cvet5", "1F918-1F3FF"},
    {"zhest_pozvoni_mne", "1F919"},
    {"zhest_pozvoni_mne_cvet1", "1F919-1F3FB"},
    {"zhest_pozvoni_mne_cvet2", "1F919-1F3FC"},
    {"zhest_pozvoni_mne_cvet3", "1F919-1F3FD"},
    {"zhest_pozvoni_mne_cvet4", "1F919-1F3FE"},
    {"zhest_pozvoni_mne_cvet5", "1F919-1F3FF"},
    {"podnyataya_ruka", "1F91A"},
    {"podnyataya_ruka_cvet1", "1F91A-1F3FB"},
    {"podnyataya_ruka_cvet2", "1F91A-1F3FC"},
    {"podnyataya_ruka_cvet3", "1F91A-1F3FD"},
    {"podnyataya_ruka_cvet4", "1F91A-1F3FE"},
    {"podnyataya_ruka_cvet5", "1F91A-1F3FF"},
    {"kulak_vlevo", "1F91B"},
    {"kulak_vlevo_cvet1", "1F91B-1F3FB"},
    {"kulak_vlevo_cvet2", "1F91B-1F3FC"},
    {"kulak_vlevo_cvet3", "1F91B-1F3FD"},
    {"kulak_vlevo_cvet4", "1F91B-1F3FE"},
    {"kulak_vlevo_cvet5", "1F91B-1F3FF"},
    {"kulak_vpravo", "1F91C"},
    {"kulak_vpravo_cvet1", "1F91C-1F3FB"},
    {"kulak_vpravo_cvet2", "1F91C-1F3FC"},
    {"kulak_vpravo_cvet3", "1F91C-1F3FD"},
    {"kulak_vpravo_cvet4", "1F91C-1F3FE"},
    {"kulak_vpravo_cvet5", "1F91C-1F3FF"},
    {"rukopozhatie", "1F91D"},
    {"rukopozhatie_cvet1", "1F91D-1F3FB"},
    {"rukopozhatie_cvet2", "1F91D-1F3FC"},
    {"rukopozhatie_cvet3", "1F91D-1F3FD"},
    {"rukopozhatie_cvet4", "1F91D-1F3FE"},
    {"rukopozhatie_cvet5", "1F91D-1F3FF"},
    {"rukopozhatie_cvet1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"rukopozhatie_cvet1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"rukopozhatie_cvet1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"rukopozhatie_cvet1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"rukopozhatie_cvet2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"rukopozhatie_cvet2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"rukopozhatie_cvet2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"rukopozhatie_cvet2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"rukopozhatie_cvet3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"rukopozhatie_cvet3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"rukopozhatie_cvet3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"rukopozhatie_cvet3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"rukopozhatie_cvet4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"rukopozhatie_cvet4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"rukopozhatie_cvet4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"rukopozhatie_cvet4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"rukopozhatie_cvet5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"rukopozhatie_cvet5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"rukopozhatie_cvet5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"rukopozhatie_cvet5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"skreshchennye_palcy", "1F91E"},
    {"skreshchennye_palcy_cvet1", "1F91E-1F3FB"},
    {"skreshchennye_palcy_cvet2", "1F91E-1F3FC"},
    {"skreshchennye_palcy_cvet3", "1F91E-1F3FD"},
    {"skreshchennye_palcy_cvet4", "1F91E-1F3FE"},
    {"skreshchennye_palcy_cvet5", "1F91E-1F3FF"},
    {"ya_tebya_lyublyu", "1F91F"},
    {"ya_tebya_lyublyu_cvet1", "1F91F-1F3FB"},
    {"ya_tebya_lyublyu_cvet2", "1F91F-1F3FC"},
    {"ya_tebya_lyublyu_cvet3", "1F91F-1F3FD"},
    {"ya_tebya_lyublyu_cvet4", "1F91F-1F3FE"},
    {"ya_tebya_lyublyu_cvet5", "1F91F-1F3FF"},
    {"v_kovboyskoy_shlyape", "1F920"},
    {"kloun", "1F921"},
    {"toshnit", "1F922"},
    {"kataetsya_so_smehu", "1F923"},
    {"puskaet_slyuni", "1F924"},
    {"lzhec", "1F925"},
    {"chelovek_poteryal_dar_rechi", "1F926"},
    {"chelovek_poteryal_dar_rechi_cvet1", "1F926-1F3FB"},
    {"chelovek_poteryal_dar_rechi_cvet2", "1F926-1F3FC"},
    {"chelovek_poteryal_dar_rechi_cvet3", "1F926-1F3FD"},
    {"chelovek_poteryal_dar_rechi_cvet4", "1F926-1F3FE"},
    {"chelovek_poteryal_dar_rechi_cvet5", "1F926-1F3FF"},
    {"chihaet", "1F927"},
    {"ne_verit", "1F928"},
    {"zvezdy_v_glazah", "1F929"},
    {"sumasshedshee_lico", "1F92A"},
    {"ts_s_s", "1F92B"},
    {"cenzura", "1F92C"},
    {"prikryvaet_rot_rukoy", "1F92D"},
    {"rvota", "1F92E"},
    {"vzryv_mozga", "1F92F"},
    {"beremennaya_zhenshchina", "1F930"},
    {"beremennaya_zhenshchina_cvet1", "1F930-1F3FB"},
    {"beremennaya_zhenshchina_cvet2", "1F930-1F3FC"},
    {"beremennaya_zhenshchina_cvet3", "1F930-1F3FD"},
    {"beremennaya_zhenshchina_cvet4", "1F930-1F3FE"},
    {"beremennaya_zhenshchina_cvet5", "1F930-1F3FF"},
    {"kormlenie_grudyu", "1F931"},
    {"kormlenie_grudyu_cvet1", "1F931-1F3FB"},
    {"kormlenie_grudyu_cvet2", "1F931-1F3FC"},
    {"kormlenie_grudyu_cvet3", "1F931-1F3FD"},
    {"kormlenie_grudyu_cvet4", "1F931-1F3FE"},
    {"kormlenie_grudyu_cvet5", "1F931-1F3FF"},
    {"ladoni_lodochkoy", "1F932"},
    {"ladoni_lodochkoy_cvet1", "1F932-1F3FB"},
    {"ladoni_lodochkoy_cvet2", "1F932-1F3FC"},
    {"ladoni_lodochkoy_cvet3", "1F932-1F3FD"},
    {"ladoni_lodochkoy_cvet4", "1F932-1F3FE"},
    {"ladoni_lodochkoy_cvet5", "1F932-1F3FF"},
    {"selfi", "1F933"},
    {"selfi_cvet1", "1F933-1F3FB"},
    {"selfi_cvet2", "1F933-1F3FC"},
    {"selfi_cvet3", "1F933-1F3FD"},
    {"selfi_cvet4", "1F933-1F3FE"},
    {"selfi_cvet5", "1F933-1F3FF"},
    {"princ", "1F934"},
    {"princ_cvet1", "1F934-1F3FB"},
    {"princ_cvet2", "1F934-1F3FC"},
    {"princ_cvet3", "1F934-1F3FD"},
    {"princ_cvet4", "1F934-1F3FE"},
    {"princ_cvet5", "1F934-1F3FF"},
    {"zhenih", "1F935"},
    {"zhenih_cvet1", "1F935-1F3FB"},
    {"zhenih_cvet2", "1F935-1F3FC"},
    {"zhenih_cvet3", "1F935-1F3FD"},
    {"zhenih_cvet4", "1F935-1F3FE"},
    {"zhenih_cvet5", "1F935-1F3FF"},
    {"missis_klaus", "1F936"},
    {"missis_klaus_cvet1", "1F936-1F3FB"},
    {"missis_klaus_cvet2", "1F936-1F3FC"},
    {"missis_klaus_cvet3", "1F936-1F3FD"},
    {"missis_klaus_cvet4", "1F936-1F3FE"},
    {"missis_klaus_cvet5", "1F936-1F3FF"},
    {"chelovek_pozhimaet_plechami", "1F937"},
    {"chelovek_pozhimaet_plechami_cvet1", "1F937-1F3FB"},
    {"chelovek_pozhimaet_plechami_cvet2", "1F937-1F3FC"},
    {"chelovek_pozhimaet_plechami_cvet3", "1F937-1F3FD"},
    {"chelovek_pozhimaet_plechami_cvet4", "1F937-1F3FE"},
    {"chelovek_pozhimaet_plechami_cvet5", "1F937-1F3FF"},
    {"akrobaticheskiy_tryuk", "1F938"},
    {"akrobaticheskiy_tryuk_cvet1", "1F938-1F3FB"},
    {"akrobaticheskiy_tryuk_cvet2", "1F938-1F3FC"},
    {"akrobaticheskiy_tryuk_cvet3", "1F938-1F3FD"},
    {"akrobaticheskiy_tryuk_cvet4", "1F938-1F3FE"},
    {"akrobaticheskiy_tryuk_cvet5", "1F938-1F3FF"},
    {"zhongler", "1F939"},
    {"zhongler_cvet1", "1F939-1F3FB"},
    {"zhongler_cvet2", "1F939-1F3FC"},
    {"zhongler_cvet3", "1F939-1F3FD"},
    {"zhongler_cvet4", "1F939-1F3FE"},
    {"zhongler_cvet5", "1F939-1F3FF"},
    {"fehtovalshchik", "1F93A"},
    {"borcy", "1F93C"},
    {"borcy_cvet1", "1F93C-1F3FB"},
    {"borcy_cvet2", "1F93C-1F3FC"},
    {"borcy_cvet3", "1F93C-1F3FD"},
    {"borcy_cvet4", "1F93C-1F3FE"},
    {"borcy_cvet5", "1F93C-1F3FF"},
    {"borcy_cvet1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"borcy_cvet1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"borcy_cvet1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"borcy_cvet1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"borcy_cvet2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"borcy_cvet2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"borcy_cvet2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"borcy_cvet2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"borcy_cvet3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"borcy_cvet3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"borcy_cvet3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"borcy_cvet3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"borcy_cvet4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"borcy_cvet4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"borcy_cvet4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"borcy_cvet4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"borcy_cvet5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"borcy_cvet5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"borcy_cvet5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"borcy_cvet5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"vodnoe_polo", "1F93D"},
    {"vodnoe_polo_cvet1", "1F93D-1F3FB"},
    {"vodnoe_polo_cvet2", "1F93D-1F3FC"},
    {"vodnoe_polo_cvet3", "1F93D-1F3FD"},
    {"vodnoe_polo_cvet4", "1F93D-1F3FE"},
    {"vodnoe_polo_cvet5", "1F93D-1F3FF"},
    {"gandbol", "1F93E"},
    {"gandbol_cvet1", "1F93E-1F3FB"},
    {"gandbol_cvet2", "1F93E-1F3FC"},
    {"gandbol_cvet3", "1F93E-1F3FD"},
    {"gandbol_cvet4", "1F93E-1F3FE"},
    {"gandbol_cvet5", "1F93E-1F3FF"},
    {"maska_s_trubkoy", "1F93F"},
    {"uvyadshiy_cvetok", "1F940"},
    {"baraban", "1F941"},
    {"chokayushchiesya_bokaly", "1F942"},
    {"stakan_s_viski", "1F943"},
    {"lozhka", "1F944"},
    {"sportivnye_vorota", "1F945"},
    {"zolotaya_medal", "1F947"},
    {"serebryanaya_medal", "1F948"},
    {"bronzovaya_medal", "1F949"},
    {"bokserskaya_perchatka", "1F94A"},
    {"sportivnoe_kimono", "1F94B"},
    {"kerling", "1F94C"},
    {"lakross", "1F94D"},
    {"softbol", "1F94E"},
    {"letayushchiy_disk", "1F94F"},
    {"kruassan", "1F950"},
    {"avokado", "1F951"},
    {"ogurec", "1F952"},
    {"bekon", "1F953"},
    {"kartofel", "1F954"},
    {"morkov", "1F955"},
    {"baget", "1F956"},
    {"ovoshchnoy_salat", "1F957"},
    {"ragu_v_skovorode", "1F958"},
    {"shaurma", "1F959"},
    {"yayco", "1F95A"},
    {"stakan_moloka", "1F95B"},
    {"arahis", "1F95C"},
    {"kivi", "1F95D"},
    {"bliny", "1F95E"},
    {"varenik", "1F95F"},
    {"pechene_s_predskazaniem", "1F960"},
    {"korobka_dlya_edy_na_vynos", "1F961"},
    {"palochki_dlya_edy", "1F962"},
    {"miska_s_lozhkoy", "1F963"},
    {"stakan_s_solominkoy", "1F964"},
    {"kokos", "1F965"},
    {"brokkoli", "1F966"},
    {"pirog", "1F967"},
    {"krendel", "1F968"},
    {"kusok_myasa", "1F969"},
    {"sendvich", "1F96A"},
    {"konservy", "1F96B"},
    {"zelenyy_salat", "1F96C"},
    {"mango", "1F96D"},
    {"lunnyy_pirog", "1F96E"},
    {"bublik", "1F96F"},
    {"ulybayushcheesya_lico_s_serdechkami", "1F970"},
    {"zevaet", "1F971"},
    {"ulybayushcheesya_lico_so_slezoy", "1F972"},
    {"na_vecherinke", "1F973"},
    {"chuvstvuet_golovokruzhenie", "1F974"},
    {"zhar", "1F975"},
    {"merznet", "1F976"},
    {"nindzya", "1F977"},
    {"nindzya_cvet1", "1F977-1F3FB"},
    {"nindzya_cvet2", "1F977-1F3FC"},
    {"nindzya_cvet3", "1F977-1F3FD"},
    {"nindzya_cvet4", "1F977-1F3FE"},
    {"nindzya_cvet5", "1F977-1F3FF"},
    {"maskirovka", "1F978"},
    {"ele_sderzhivaet_slezy", "1F979"},
    {"umolyaet", "1F97A"},
    {"sari", "1F97B"},
    {"laboratornyy_halat", "1F97C"},
    {"zashchitnye_ochki", "1F97D"},
    {"trekkingovyy_botinok", "1F97E"},
    {"baletki", "1F97F"},
    {"krab", "1F980"},
    {"morda_lva", "1F981"},
    {"skorpion", "1F982"},
    {"indeyka", "1F983"},
    {"golova_edinoroga", "1F984"},
    {"orel", "1F985"},
    {"utka", "1F986"},
    {"letuchaya_mysh", "1F987"},
    {"akula", "1F988"},
    {"sova", "1F989"},
    {"morda_lisicy", "1F98A"},
    {"babochka", "1F98B"},
    {"olen", "1F98C"},
    {"gorilla", "1F98D"},
    {"yashcherica", "1F98E"},
    {"nosorog", "1F98F"},
    {"krevetka", "1F990"},
    {"kalmar", "1F991"},
    {"zhiraf", "1F992"},
    {"zebra", "1F993"},
    {"ezh", "1F994"},
    {"zauropod", "1F995"},
    {"ti_reks", "1F996"},
    {"sverchok", "1F997"},
    {"kenguru", "1F998"},
    {"lama", "1F999"},
    {"pavlin", "1F99A"},
    {"gippopotam", "1F99B"},
    {"popugay", "1F99C"},
    {"morda_enota", "1F99D"},
    {"omar", "1F99E"},
    {"komar", "1F99F"},
    {"mikrob", "1F9A0"},
    {"barsuk", "1F9A1"},
    {"lebed", "1F9A2"},
    {"mamont", "1F9A3"},
    {"dront", "1F9A4"},
    {"lenivec", "1F9A5"},
    {"vydra", "1F9A6"},
    {"orangutan", "1F9A7"},
    {"skuns", "1F9A8"},
    {"flamingo", "1F9A9"},
    {"ustrica", "1F9AA"},
    {"bobr", "1F9AB"},
    {"bizon", "1F9AC"},
    {"tyulen", "1F9AD"},
    {"sobaka_povodyr", "1F9AE"},
    {"belaya_trost", "1F9AF"},
    {"ryzhie_volosy", "1F9B0"},
    {"kudryavye_volosy", "1F9B1"},
    {"bez_volos", "1F9B2"},
    {"sedye_volosy", "1F9B3"},
    {"kost", "1F9B4"},
    {"noga", "1F9B5"},
    {"noga_cvet1", "1F9B5-1F3FB"},
    {"noga_cvet2", "1F9B5-1F3FC"},
    {"noga_cvet3", "1F9B5-1F3FD"},
    {"noga_cvet4", "1F9B5-1F3FE"},
    {"noga_cvet5", "1F9B5-1F3FF"},
    {"shchikolotka", "1F9B6"},
    {"shchikolotka_cvet1", "1F9B6-1F3FB"},
    {"shchikolotka_cvet2", "1F9B6-1F3FC"},
    {"shchikolotka_cvet3", "1F9B6-1F3FD"},
    {"shchikolotka_cvet4", "1F9B6-1F3FE"},
    {"shchikolotka_cvet5", "1F9B6-1F3FF"},
    {"zub", "1F9B7"},
    {"supergeroy", "1F9B8"},
    {"supergeroy_cvet1", "1F9B8-1F3FB"},
    {"supergeroy_cvet2", "1F9B8-1F3FC"},
    {"supergeroy_cvet3", "1F9B8-1F3FD"},
    {"supergeroy_cvet4", "1F9B8-1F3FE"},
    {"supergeroy_cvet5", "1F9B8-1F3FF"},
    {"superzlodey", "1F9B9"},
    {"superzlodey_cvet1", "1F9B9-1F3FB"},
    {"superzlodey_cvet2", "1F9B9-1F3FC"},
    {"superzlodey_cvet3", "1F9B9-1F3FD"},
    {"superzlodey_cvet4", "1F9B9-1F3FE"},
    {"superzlodey_cvet5", "1F9B9-1F3FF"},
    {"spasatelnyy_zhilet", "1F9BA"},
    {"uho_so_sluhovym_apparatom", "1F9BB"},
    {"uho_so_sluhovym_apparatom_cvet1", "1F9BB-1F3FB"},
    {"uho_so_sluhovym_apparatom_cvet2", "1F9BB-1F3FC"},
    {"uho_so_sluhovym_apparatom_cvet3", "1F9BB-1F3FD"},
    {"uho_so_sluhovym_apparatom_cvet4", "1F9BB-1F3FE"},
    {"uho_so_sluhovym_apparatom_cvet5", "1F9BB-1F3FF"},
    {"kreslo_kolyaska_s_motorom", "1F9BC"},
    {"kreslo_kolyaska_s_ruchnym_privodom", "1F9BD"},
    {"mehanicheskaya_ruka", "1F9BE"},
    {"mehanicheskaya_noga", "1F9BF"},
    {"syr", "1F9C0"},
    {"kapkeyk", "1F9C1"},
    {"solonka", "1F9C2"},
    {"paketik_soka", "1F9C3"},
    {"chesnok", "1F9C4"},
    {"luk", "1F9C5"},
    {"falafel", "1F9C6"},
    {"vaflya", "1F9C7"},
    {"maslo", "1F9C8"},
    {"mate", "1F9C9"},
    {"kubik_lda", "1F9CA"},
    {"chay_s_sharikami", "1F9CB"},
    {"troll", "1F9CC"},
    {"chelovek_stoit", "1F9CD"},
    {"chelovek_stoit_cvet1", "1F9CD-1F3FB"},
    {"chelovek_stoit_cvet2", "1F9CD-1F3FC"},
    {"chelovek_stoit_cvet3", "1F9CD-1F3FD"},
    {"chelovek_stoit_cvet4", "1F9CD-1F3FE"},
    {"chelovek_stoit_cvet5", "1F9CD-1F3FF"},
    {"chelovek_na_kolenyah", "1F9CE"},
    {"chelovek_na_kolenyah_cvet1", "1F9CE-1F3FB"},
    {"chelovek_na_kolenyah_cvet2", "1F9CE-1F3FC"},
    {"chelovek_na_kolenyah_cvet3", "1F9CE-1F3FD"},
    {"chelovek_na_kolenyah_cvet4", "1F9CE-1F3FE"},
    {"chelovek_na_kolenyah_cvet5", "1F9CE-1F3FF"},
    {"gluhoy_chelovek", "1F9CF"},
    {"gluhoy_chelovek_cvet1", "1F9CF-1F3FB"},
    {"gluhoy_chelovek_cvet2", "1F9CF-1F3FC"},
    {"gluhoy_chelovek_cvet3", "1F9CF-1F3FD"},
    {"gluhoy_chelovek_cvet4", "1F9CF-1F3FE"},
    {"gluhoy_chelovek_cvet5", "1F9CF-1F3FF"},
    {"s_monoklem", "1F9D0"},
    {"vzroslyy", "1F9D1"},
    {"vzroslyy_cvet1", "1F9D1-1F3FB"},
    {"vzroslyy_cvet2", "1F9D1-1F3FC"},
    {"vzroslyy_cvet3", "1F9D1-1F3FD"},
    {"vzroslyy_cvet4", "1F9D1-1F3FE"},
    {"vzroslyy_cvet5", "1F9D1-1F3FF"},
    {"rebenok", "1F9D2"},
    {"rebenok_cvet1", "1F9D2-1F3FB"},
    {"rebenok_cvet2", "1F9D2-1F3FC"},
    {"rebenok_cvet3", "1F9D2-1F3FD"},
    {"rebenok_cvet4", "1F9D2-1F3FE"},
    {"rebenok_cvet5", "1F9D2-1F3FF"},
    {"pozhiloy_chelovek", "1F9D3"},
    {"pozhiloy_chelovek_cvet1", "1F9D3-1F3FB"},
    {"pozhiloy_chelovek_cvet2", "1F9D3-1F3FC"},
    {"pozhiloy_chelovek_cvet3", "1F9D3-1F3FD"},
    {"pozhiloy_chelovek_cvet4", "1F9D3-1F3FE"},
    {"pozhiloy_chelovek_cvet5", "1F9D3-1F3FF"},
    {"borodatyy_chelovek", "1F9D4"},
    {"borodatyy_chelovek_cvet1", "1F9D4-1F3FB"},
    {"borodatyy_chelovek_cvet2", "1F9D4-1F3FC"},
    {"borodatyy_chelovek_cvet3", "1F9D4-1F3FD"},
    {"borodatyy_chelovek_cvet4", "1F9D4-1F3FE"},
    {"borodatyy_chelovek_cvet5", "1F9D4-1F3FF"},
    {"zhenshchina_v_platke", "1F9D5"},
    {"zhenshchina_v_platke_cvet1", "1F9D5-1F3FB"},
    {"zhenshchina_v_platke_cvet2", "1F9D5-1F3FC"},
    {"zhenshchina_v_platke_cvet3", "1F9D5-1F3FD"},
    {"zhenshchina_v_platke_cvet4", "1F9D5-1F3FE"},
    {"zhenshchina_v_platke_cvet5", "1F9D5-1F3FF"},
    {"chelovek_v_bane", "1F9D6"},
    {"chelovek_v_bane_cvet1", "1F9D6-1F3FB"},
    {"chelovek_v_bane_cvet2", "1F9D6-1F3FC"},
    {"chelovek_v_bane_cvet3", "1F9D6-1F3FD"},
    {"chelovek_v_bane_cvet4", "1F9D6-1F3FE"},
    {"chelovek_v_bane_cvet5", "1F9D6-1F3FF"},
    {"chelovek_vzbiraetsya_po_skale", "1F9D7"},
    {"chelovek_vzbiraetsya_po_skale_cvet1", "1F9D7-1F3FB"},
    {"chelovek_vzbiraetsya_po_skale_cvet2", "1F9D7-1F3FC"},
    {"chelovek_vzbiraetsya_po_skale_cvet3", "1F9D7-1F3FD"},
    {"chelovek_vzbiraetsya_po_skale_cvet4", "1F9D7-1F3FE"},
    {"chelovek_vzbiraetsya_po_skale_cvet5", "1F9D7-1F3FF"},
    {"chelovek_v_poze_lotosa", "1F9D8"},
    {"chelovek_v_poze_lotosa_cvet1", "1F9D8-1F3FB"},
    {"chelovek_v_poze_lotosa_cvet2", "1F9D8-1F3FC"},
    {"chelovek_v_poze_lotosa_cvet3", "1F9D8-1F3FD"},
    {"chelovek_v_poze_lotosa_cvet4", "1F9D8-1F3FE"},
    {"chelovek_v_poze_lotosa_cvet5", "1F9D8-1F3FF"},
    {"mag", "1F9D9"},
    {"mag_cvet1", "1F9D9-1F3FB"},
    {"mag_cvet2", "1F9D9-1F3FC"},
    {"mag_cvet3", "1F9D9-1F3FD"},
    {"mag_cvet4", "1F9D9-1F3FE"},
    {"mag_cvet5", "1F9D9-1F3FF"},
    {"feya", "1F9DA"},
    {"feya_cvet1", "1F9DA-1F3FB"},
    {"feya_cvet2", "1F9DA-1F3FC"},
    {"feya_cvet3", "1F9DA-1F3FD"},
    {"feya_cvet4", "1F9DA-1F3FE"},
    {"feya_cvet5", "1F9DA-1F3FF"},
    {"vampir", "1F9DB"},
    {"vampir_cvet1", "1F9DB-1F3FB"},
    {"vampir_cvet2", "1F9DB-1F3FC"},
    {"vampir_cvet3", "1F9DB-1F3FD"},
    {"vampir_cvet4", "1F9DB-1F3FE"},
    {"vampir_cvet5", "1F9DB-1F3FF"},
    {"rusalka", "1F9DC"},
    {"rusalka_cvet1", "1F9DC-1F3FB"},
    {"rusalka_cvet2", "1F9DC-1F3FC"},
    {"rusalka_cvet3", "1F9DC-1F3FD"},
    {"rusalka_cvet4", "1F9DC-1F3FE"},
    {"rusalka_cvet5", "1F9DC-1F3FF"},
    {"elf", "1F9DD"},
    {"elf_cvet1", "1F9DD-1F3FB"},
    {"elf_cvet2", "1F9DD-1F3FC"},
    {"elf_cvet3", "1F9DD-1F3FD"},
    {"elf_cvet4", "1F9DD-1F3FE"},
    {"elf_cvet5", "1F9DD-1F3FF"},
    {"dzhinn", "1F9DE"},
    {"zombi", "1F9DF"},
    {"mozg", "1F9E0"},
    {"oranzhevoe_serdce", "1F9E1"},
    {"beysbolka", "1F9E2"},
    {"sharf", "1F9E3"},
    {"perchatki", "1F9E4"},
    {"palto", "1F9E5"},
    {"noski", "1F9E6"},
    {"krasnyy_konvert", "1F9E7"},
    {"dinamitnaya_shashka", "1F9E8"},
    {"pazl", "1F9E9"},
    {"probirka", "1F9EA"},
    {"chashka_petri", "1F9EB"},
    {"dnk", "1F9EC"},
    {"kompas", "1F9ED"},
    {"schety", "1F9EE"},
    {"ognetushitel", "1F9EF"},
    {"yashchik_dlya_instrumentov", "1F9F0"},
    {"kirpichi", "1F9F1"},
    {"magnit", "1F9F2"},
    {"bagazh", "1F9F3"},
    {"butylka_losona", "1F9F4"},
    {"katushka_nitok", "1F9F5"},
    {"klubok", "1F9F6"},
    {"angliyskaya_bulavka", "1F9F7"},
    {"plyushevyy_mishka", "1F9F8"},
    {"metla", "1F9F9"},
    {"korzina", "1F9FA"},
    {"rulon_bumagi", "1F9FB"},
    {"mylo", "1F9FC"},
    {"gubka", "1F9FD"},
    {"chek", "1F9FE"},
    {"nazar_bondzhuk", "1F9FF"},
    {"puanty", "1FA70"},
    {"celnyy_kupalnik", "1FA71"},
    {"plavki", "1FA72"},
    {"shorty", "1FA73"},
    {"vetnamki", "1FA74"},
    {"goluboe_serdce", "1FA75"},
    {"seroe_serdce", "1FA76"},
    {"rozovoe_serdce", "1FA77"},
    {"kaplya_krovi", "1FA78"},
    {"plastyr", "1FA79"},
    {"stetoskop", "1FA7A"},
    {"rentgen", "1FA7B"},
    {"kostyl", "1FA7C"},
    {"yo_yo", "1FA80"},
    {"vozdushnyy_zmey", "1FA81"},
    {"parashyut", "1FA82"},
    {"bumerang", "1FA83"},
    {"volshebnaya_palochka", "1FA84"},
    {"pinyata", "1FA85"},
    {"matreshka", "1FA86"},
    {"marakasy", "1FA87"},
    {"fleyta", "1FA88"},
    {"arfa", "1FA89"},
    {"trombon", "1FA8A"},
    {"sunduk_s_sokrovishchami", "1FA8E"},
    {"lopata", "1FA8F"},
    {"planeta_s_kolcom", "1FA90"},
    {"stul", "1FA91"},
    {"britva", "1FA92"},
    {"topor", "1FA93"},
    {"maslyanaya_lampa", "1FA94"},
    {"bandzho", "1FA95"},
    {"armeyskaya_kaska", "1FA96"},
    {"akkordeon", "1FA97"},
    {"konga", "1FA98"},
    {"moneta", "1FA99"},
    {"pila_po_derevu", "1FA9A"},
    {"otvertka", "1FA9B"},
    {"lestnica", "1FA9C"},
    {"kryuk", "1FA9D"},
    {"zerkalo", "1FA9E"},
    {"okno", "1FA9F"},
    {"vantuz", "1FAA0"},
    {"shveynaya_igla", "1FAA1"},
    {"uzel", "1FAA2"},
    {"vedro", "1FAA3"},
    {"myshelovka", "1FAA4"},
    {"zubnaya_shchetka", "1FAA5"},
    {"nadgrobie", "1FAA6"},
    {"plakat", "1FAA7"},
    {"kamen", "1FAA8"},
    {"disko_shar", "1FAA9"},
    {"udostoverenie_lichnosti", "1FAAA"},
    {"nizkiy_uroven_zaryada", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"veer", "1FAAD"},
    {"afro_greben", "1FAAE"},
    {"khanda", "1FAAF"},
    {"muha", "1FAB0"},
    {"chervyak", "1FAB1"},
    {"zhuk", "1FAB2"},
    {"tarakan", "1FAB3"},
    {"rastenie_v_gorshke", "1FAB4"},
    {"drevesina", "1FAB5"},
    {"pero", "1FAB6"},
    {"lotos", "1FAB7"},
    {"korall", "1FAB8"},
    {"pustoe_gnezdo", "1FAB9"},
    {"gnezdo_s_yaycami", "1FABA"},
    {"giacint", "1FABB"},
    {"meduza", "1FABC"},
    {"krylo", "1FABD"},
    {"derevo_bez_listvy", "1FABE"},
    {"gus", "1FABF"},
    {"serdce_cheloveka", "1FAC0"},
    {"legkie", "1FAC1"},
    {"obnimayushchiesya_lyudi", "1FAC2"},
    {"beremennyy_muzhchina", "1FAC3"},
    {"beremennyy_muzhchina_cvet1", "1FAC3-1F3FB"},
    {"beremennyy_muzhchina_cvet2", "1FAC3-1F3FC"},
    {"beremennyy_muzhchina_cvet3", "1FAC3-1F3FD"},
    {"beremennyy_muzhchina_cvet4", "1FAC3-1F3FE"},
    {"beremennyy_muzhchina_cvet5", "1FAC3-1F3FF"},
    {"beremennyy_chelovek", "1FAC4"},
    {"beremennyy_chelovek_cvet1", "1FAC4-1F3FB"},
    {"beremennyy_chelovek_cvet2", "1FAC4-1F3FC"},
    {"beremennyy_chelovek_cvet3", "1FAC4-1F3FD"},
    {"beremennyy_chelovek_cvet4", "1FAC4-1F3FE"},
    {"beremennyy_chelovek_cvet5", "1FAC4-1F3FF"},
    {"chelovek_s_koronoy", "1FAC5"},
    {"chelovek_s_koronoy_cvet1", "1FAC5-1F3FB"},
    {"chelovek_s_koronoy_cvet2", "1FAC5-1F3FC"},
    {"chelovek_s_koronoy_cvet3", "1FAC5-1F3FD"},
    {"chelovek_s_koronoy_cvet4", "1FAC5-1F3FE"},
    {"chelovek_s_koronoy_cvet5", "1FAC5-1F3FF"},
    {"otpechatok_palca", "1FAC6"},
    {"snezhnyy_chelovek", "1FAC8"},
    {"kosatka", "1FACD"},
    {"los", "1FACE"},
    {"osel", "1FACF"},
    {"golubika", "1FAD0"},
    {"bolgarskiy_perec", "1FAD1"},
    {"olivka", "1FAD2"},
    {"lepeshka", "1FAD3"},
    {"tamale", "1FAD4"},
    {"fondyu", "1FAD5"},
    {"chaynik", "1FAD6"},
    {"stakan_s_zhidkostyu", "1FAD7"},
    {"fasol", "1FAD8"},
    {"banka", "1FAD9"},
    {"koren_imbirya", "1FADA"},
    {"struchok_goroha", "1FADB"},
    {"korneplod", "1FADC"},
    {"klyaksa", "1FADF"},
    {"taet", "1FAE0"},
    {"voinskoe_privetstvie", "1FAE1"},
    {"prikryvaet_rot_rukoy_s_otkrytymi_glazami", "1FAE2"},
    {"podsmatrivaet", "1FAE3"},
    {"s_perekoshennym_rtom", "1FAE4"},
    {"lico_punktirom", "1FAE5"},
    {"kusat_guby", "1FAE6"},
    {"puzyri", "1FAE7"},
    {"tryasushcheesya_lico", "1FAE8"},
    {"lico_s_meshkami_pod_glazami", "1FAE9"},
    {"lico_perekosilo", "1FAEA"},
    {"draka", "1FAEF"},
    {"zhest_dengi", "1FAF0"},
    {"zhest_dengi_cvet1", "1FAF0-1F3FB"},
    {"zhest_dengi_cvet2", "1FAF0-1F3FC"},
    {"zhest_dengi_cvet3", "1FAF0-1F3FD"},
    {"zhest_dengi_cvet4", "1FAF0-1F3FE"},
    {"zhest_dengi_cvet5", "1FAF0-1F3FF"},
    {"ruka_vpravo", "1FAF1"},
    {"ruka_vpravo_cvet1", "1FAF1-1F3FB"},
    {"ruka_vpravo_cvet2", "1FAF1-1F3FC"},
    {"ruka_vpravo_cvet3", "1FAF1-1F3FD"},
    {"ruka_vpravo_cvet4", "1FAF1-1F3FE"},
    {"ruka_vpravo_cvet5", "1FAF1-1F3FF"},
    {"ruka_vlevo", "1FAF2"},
    {"ruka_vlevo_cvet1", "1FAF2-1F3FB"},
    {"ruka_vlevo_cvet2", "1FAF2-1F3FC"},
    {"ruka_vlevo_cvet3", "1FAF2-1F3FD"},
    {"ruka_vlevo_cvet4", "1FAF2-1F3FE"},
    {"ruka_vlevo_cvet5", "1FAF2-1F3FF"},
    {"ruka_ladonyu_vniz", "1FAF3"},
    {"ruka_ladonyu_vniz_cvet1", "1FAF3-1F3FB"},
    {"ruka_ladonyu_vniz_cvet2", "1FAF3-1F3FC"},
    {"ruka_ladonyu_vniz_cvet3", "1FAF3-1F3FD"},
    {"ruka_ladonyu_vniz_cvet4", "1FAF3-1F3FE"},
    {"ruka_ladonyu_vniz_cvet5", "1FAF3-1F3FF"},
    {"ruka_ladonyu_vverh", "1FAF4"},
    {"ruka_ladonyu_vverh_cvet1", "1FAF4-1F3FB"},
    {"ruka_ladonyu_vverh_cvet2", "1FAF4-1F3FC"},
    {"ruka_ladonyu_vverh_cvet3", "1FAF4-1F3FD"},
    {"ruka_ladonyu_vverh_cvet4", "1FAF4-1F3FE"},
    {"ruka_ladonyu_vverh_cvet5", "1FAF4-1F3FF"},
    {"ukazatelnyy_palec_na_sebya", "1FAF5"},
    {"ukazatelnyy_palec_na_sebya_cvet1", "1FAF5-1F3FB"},
    {"ukazatelnyy_palec_na_sebya_cvet2", "1FAF5-1F3FC"},
    {"ukazatelnyy_palec_na_sebya_cvet3", "1FAF5-1F3FD"},
    {"ukazatelnyy_palec_na_sebya_cvet4", "1FAF5-1F3FE"},
    {"ukazatelnyy_palec_na_sebya_cvet5", "1FAF5-1F3FF"},
    {"zhest_serdce", "1FAF6"},
    {"zhest_serdce_cvet1", "1FAF6-1F3FB"},
    {"zhest_serdce_cvet2", "1FAF6-1F3FC"},
    {"zhest_serdce_cvet3", "1FAF6-1F3FD"},
    {"zhest_serdce_cvet4", "1FAF6-1F3FE"},
    {"zhest_serdce_cvet5", "1FAF6-1F3FF"},
    {"ruka_ladonyu_vlevo", "1FAF7"},
    {"ruka_ladonyu_vlevo_cvet1", "1FAF7-1F3FB"},
    {"ruka_ladonyu_vlevo_cvet2", "1FAF7-1F3FC"},
    {"ruka_ladonyu_vlevo_cvet3", "1FAF7-1F3FD"},
    {"ruka_ladonyu_vlevo_cvet4", "1FAF7-1F3FE"},
    {"ruka_ladonyu_vlevo_cvet5", "1FAF7-1F3FF"},
    {"ruka_ladonyu_vpravo", "1FAF8"},
    {"ruka_ladonyu_vpravo_cvet1", "1FAF8-1F3FB"},
    {"ruka_ladonyu_vpravo_cvet2", "1FAF8-1F3FC"},
    {"ruka_ladonyu_vpravo_cvet3", "1FAF8-1F3FD"},
    {"ruka_ladonyu_vpravo_cvet4", "1FAF8-1F3FE"},
    {"ruka_ladonyu_vpravo_cvet5", "1FAF8-1F3FF"},
    {"klavishi_chislovoy_znak", "0023-FE0F-20E3"},
    {"klavishi_zvezdochka", "002A-FE0F-20E3"},
    {"klavisha_0", "0030-FE0F-20E3"},
    {"klavisha_1", "0031-FE0F-20E3"},
    {"klavisha_2", "0032-FE0F-20E3"},
    {"klavisha_3", "0033-FE0F-20E3"},
    {"klavisha_4", "0034-FE0F-20E3"},
    {"klavisha_5", "0035-FE0F-20E3"},
    {"klavisha_6", "0036-FE0F-20E3"},
    {"klavisha_7", "0037-FE0F-20E3"},
    {"klavisha_8", "0038-FE0F-20E3"},
    {"klavisha_9", "0039-FE0F-20E3"},
    {"flag_o_v_vozneseniya", "1F1E6-1F1E8"},
    {"flag_andorra", "1F1E6-1F1E9"},
    {"flag_oae", "1F1E6-1F1EA"},
    {"flag_afganistan", "1F1E6-1F1EB"},
    {"flag_antigua_i_barbuda", "1F1E6-1F1EC"},
    {"flag_angilya", "1F1E6-1F1EE"},
    {"flag_albaniya", "1F1E6-1F1F1"},
    {"flag_armeniya", "1F1E6-1F1F2"},
    {"flag_angola", "1F1E6-1F1F4"},
    {"flag_antarktida", "1F1E6-1F1F6"},
    {"flag_argentina", "1F1E6-1F1F7"},
    {"flag_amerikanskoe_samoa", "1F1E6-1F1F8"},
    {"flag_avstriya", "1F1E6-1F1F9"},
    {"flag_avstraliya", "1F1E6-1F1FA"},
    {"flag_aruba", "1F1E6-1F1FC"},
    {"flag_alandskie_o_va", "1F1E6-1F1FD"},
    {"flag_azerbaydzhan", "1F1E6-1F1FF"},
    {"flag_bosniya_i_gercegovina", "1F1E7-1F1E6"},
    {"flag_barbados", "1F1E7-1F1E7"},
    {"flag_bangladesh", "1F1E7-1F1E9"},
    {"flag_belgiya", "1F1E7-1F1EA"},
    {"flag_burkina_faso", "1F1E7-1F1EB"},
    {"flag_bolgariya", "1F1E7-1F1EC"},
    {"flag_bahreyn", "1F1E7-1F1ED"},
    {"flag_burundi", "1F1E7-1F1EE"},
    {"flag_benin", "1F1E7-1F1EF"},
    {"flag_sen_bartelemi", "1F1E7-1F1F1"},
    {"flag_bermudskie_o_va", "1F1E7-1F1F2"},
    {"flag_bruney", "1F1E7-1F1F3"},
    {"flag_boliviya", "1F1E7-1F1F4"},
    {"flag_boneyr_sint_estatius_i_saba", "1F1E7-1F1F6"},
    {"flag_braziliya", "1F1E7-1F1F7"},
    {"flag_bagamy", "1F1E7-1F1F8"},
    {"flag_butan", "1F1E7-1F1F9"},
    {"flag_o_v_buve", "1F1E7-1F1FB"},
    {"flag_botsvana", "1F1E7-1F1FC"},
    {"flag_belarus", "1F1E7-1F1FE"},
    {"flag_beliz", "1F1E7-1F1FF"},
    {"flag_kanada", "1F1E8-1F1E6"},
    {"flag_kokosovye_o_va", "1F1E8-1F1E8"},
    {"flag_kongo_kinshasa", "1F1E8-1F1E9"},
    {"flag_centralno_afrikanskaya_respublika", "1F1E8-1F1EB"},
    {"flag_kongo_brazzavil", "1F1E8-1F1EC"},
    {"flag_shveycariya", "1F1E8-1F1ED"},
    {"flag_kot_divuar", "1F1E8-1F1EE"},
    {"flag_o_va_kuka", "1F1E8-1F1F0"},
    {"flag_chili", "1F1E8-1F1F1"},
    {"flag_kamerun", "1F1E8-1F1F2"},
    {"flag_kitay", "1F1E8-1F1F3"},
    {"flag_kolumbiya", "1F1E8-1F1F4"},
    {"flag_o_v_klipperton", "1F1E8-1F1F5"},
    {"flag_sark", "1F1E8-1F1F6"},
    {"flag_kosta_rika", "1F1E8-1F1F7"},
    {"flag_kuba", "1F1E8-1F1FA"},
    {"flag_kabo_verde", "1F1E8-1F1FB"},
    {"flag_kyurasao", "1F1E8-1F1FC"},
    {"flag_o_v_rozhdestva", "1F1E8-1F1FD"},
    {"flag_kipr", "1F1E8-1F1FE"},
    {"flag_chehiya", "1F1E8-1F1FF"},
    {"flag_germaniya", "1F1E9-1F1EA"},
    {"flag_diego_garsiya", "1F1E9-1F1EC"},
    {"flag_dzhibuti", "1F1E9-1F1EF"},
    {"flag_daniya", "1F1E9-1F1F0"},
    {"flag_dominika", "1F1E9-1F1F2"},
    {"flag_dominikanskaya_respublika", "1F1E9-1F1F4"},
    {"flag_alzhir", "1F1E9-1F1FF"},
    {"flag_seuta_i_melilya", "1F1EA-1F1E6"},
    {"flag_ekvador", "1F1EA-1F1E8"},
    {"flag_estoniya", "1F1EA-1F1EA"},
    {"flag_egipet", "1F1EA-1F1EC"},
    {"flag_zapadnaya_sahara", "1F1EA-1F1ED"},
    {"flag_eritreya", "1F1EA-1F1F7"},
    {"flag_ispaniya", "1F1EA-1F1F8"},
    {"flag_efiopiya", "1F1EA-1F1F9"},
    {"flag_evropeyskiy_soyuz", "1F1EA-1F1FA"},
    {"flag_finlyandiya", "1F1EB-1F1EE"},
    {"flag_fidzhi", "1F1EB-1F1EF"},
    {"flag_folklendskie_o_va", "1F1EB-1F1F0"},
    {"flag_federativnye_shtaty_mikronezii", "1F1EB-1F1F2"},
    {"flag_farerskie_o_va", "1F1EB-1F1F4"},
    {"flag_franciya", "1F1EB-1F1F7"},
    {"flag_gabon", "1F1EC-1F1E6"},
    {"flag_velikobritaniya", "1F1EC-1F1E7"},
    {"flag_grenada", "1F1EC-1F1E9"},
    {"flag_gruziya", "1F1EC-1F1EA"},
    {"flag_francuzskaya_gviana", "1F1EC-1F1EB"},
    {"flag_gernsi", "1F1EC-1F1EC"},
    {"flag_gana", "1F1EC-1F1ED"},
    {"flag_gibraltar", "1F1EC-1F1EE"},
    {"flag_grenlandiya", "1F1EC-1F1F1"},
    {"flag_gambiya", "1F1EC-1F1F2"},
    {"flag_gvineya", "1F1EC-1F1F3"},
    {"flag_gvadelupa", "1F1EC-1F1F5"},
    {"flag_ekvatorialnaya_gvineya", "1F1EC-1F1F6"},
    {"flag_greciya", "1F1EC-1F1F7"},
    {"flag_yuzhnaya_georgiya_i_yuzhnye_sandvichevy_o_va", "1F1EC-1F1F8"},
    {"flag_gvatemala", "1F1EC-1F1F9"},
    {"flag_guam", "1F1EC-1F1FA"},
    {"flag_gvineya_bisau", "1F1EC-1F1FC"},
    {"flag_gayana", "1F1EC-1F1FE"},
    {"flag_gonkong_sar", "1F1ED-1F1F0"},
    {"flag_o_va_herd_i_makdonald", "1F1ED-1F1F2"},
    {"flag_gonduras", "1F1ED-1F1F3"},
    {"flag_horvatiya", "1F1ED-1F1F7"},
    {"flag_gaiti", "1F1ED-1F1F9"},
    {"flag_vengriya", "1F1ED-1F1FA"},
    {"flag_kanarskie_o_va", "1F1EE-1F1E8"},
    {"flag_indoneziya", "1F1EE-1F1E9"},
    {"flag_irlandiya", "1F1EE-1F1EA"},
    {"flag_izrail", "1F1EE-1F1F1"},
    {"flag_o_v_men", "1F1EE-1F1F2"},
    {"flag_indiya", "1F1EE-1F1F3"},
    {"flag_britanskaya_territoriya_v_indiyskom_okeane", "1F1EE-1F1F4"},
    {"flag_irak", "1F1EE-1F1F6"},
    {"flag_iran", "1F1EE-1F1F7"},
    {"flag_islandiya", "1F1EE-1F1F8"},
    {"flag_italiya", "1F1EE-1F1F9"},
    {"flag_dzhersi", "1F1EF-1F1EA"},
    {"flag_yamayka", "1F1EF-1F1F2"},
    {"flag_iordaniya", "1F1EF-1F1F4"},
    {"flag_yaponiya", "1F1EF-1F1F5"},
    {"flag_keniya", "1F1F0-1F1EA"},
    {"flag_kirgiziya", "1F1F0-1F1EC"},
    {"flag_kambodzha", "1F1F0-1F1ED"},
    {"flag_kiribati", "1F1F0-1F1EE"},
    {"flag_komory", "1F1F0-1F1F2"},
    {"flag_sent_kits_i_nevis", "1F1F0-1F1F3"},
    {"flag_kndr", "1F1F0-1F1F5"},
    {"flag_respublika_koreya", "1F1F0-1F1F7"},
    {"flag_kuveyt", "1F1F0-1F1FC"},
    {"flag_o_va_kayman", "1F1F0-1F1FE"},
    {"flag_kazahstan", "1F1F0-1F1FF"},
    {"flag_laos", "1F1F1-1F1E6"},
    {"flag_livan", "1F1F1-1F1E7"},
    {"flag_sent_lyusiya", "1F1F1-1F1E8"},
    {"flag_lihtenshteyn", "1F1F1-1F1EE"},
    {"flag_shri_lanka", "1F1F1-1F1F0"},
    {"flag_liberiya", "1F1F1-1F1F7"},
    {"flag_lesoto", "1F1F1-1F1F8"},
    {"flag_litva", "1F1F1-1F1F9"},
    {"flag_lyuksemburg", "1F1F1-1F1FA"},
    {"flag_latviya", "1F1F1-1F1FB"},
    {"flag_liviya", "1F1F1-1F1FE"},
    {"flag_marokko", "1F1F2-1F1E6"},
    {"flag_monako", "1F1F2-1F1E8"},
    {"flag_moldova", "1F1F2-1F1E9"},
    {"flag_chernogoriya", "1F1F2-1F1EA"},
    {"flag_sen_marten", "1F1F2-1F1EB"},
    {"flag_madagaskar", "1F1F2-1F1EC"},
    {"flag_marshallovy_o_va", "1F1F2-1F1ED"},
    {"flag_severnaya_makedoniya", "1F1F2-1F1F0"},
    {"flag_mali", "1F1F2-1F1F1"},
    {"flag_myanma_birma", "1F1F2-1F1F2"},
    {"flag_mongoliya", "1F1F2-1F1F3"},
    {"flag_makao_sar", "1F1F2-1F1F4"},
    {"flag_severnye_marianskie_o_va", "1F1F2-1F1F5"},
    {"flag_martinika", "1F1F2-1F1F6"},
    {"flag_mavritaniya", "1F1F2-1F1F7"},
    {"flag_montserrat", "1F1F2-1F1F8"},
    {"flag_malta", "1F1F2-1F1F9"},
    {"flag_mavrikiy", "1F1F2-1F1FA"},
    {"flag_maldivy", "1F1F2-1F1FB"},
    {"flag_malavi", "1F1F2-1F1FC"},
    {"flag_meksika", "1F1F2-1F1FD"},
    {"flag_malayziya", "1F1F2-1F1FE"},
    {"flag_mozambik", "1F1F2-1F1FF"},
    {"flag_namibiya", "1F1F3-1F1E6"},
    {"flag_novaya_kaledoniya", "1F1F3-1F1E8"},
    {"flag_niger", "1F1F3-1F1EA"},
    {"flag_o_v_norfolk", "1F1F3-1F1EB"},
    {"flag_nigeriya", "1F1F3-1F1EC"},
    {"flag_nikaragua", "1F1F3-1F1EE"},
    {"flag_niderlandy", "1F1F3-1F1F1"},
    {"flag_norvegiya", "1F1F3-1F1F4"},
    {"flag_nepal", "1F1F3-1F1F5"},
    {"flag_nauru", "1F1F3-1F1F7"},
    {"flag_niue", "1F1F3-1F1FA"},
    {"flag_novaya_zelandiya", "1F1F3-1F1FF"},
    {"flag_oman", "1F1F4-1F1F2"},
    {"flag_panama", "1F1F5-1F1E6"},
    {"flag_peru", "1F1F5-1F1EA"},
    {"flag_francuzskaya_polineziya", "1F1F5-1F1EB"},
    {"flag_papua_novaya_gvineya", "1F1F5-1F1EC"},
    {"flag_filippiny", "1F1F5-1F1ED"},
    {"flag_pakistan", "1F1F5-1F1F0"},
    {"flag_polsha", "1F1F5-1F1F1"},
    {"flag_sen_per_i_mikelon", "1F1F5-1F1F2"},
    {"flag_o_va_pitkern", "1F1F5-1F1F3"},
    {"flag_puerto_riko", "1F1F5-1F1F7"},
    {"flag_palestinskie_territorii", "1F1F5-1F1F8"},
    {"flag_portugaliya", "1F1F5-1F1F9"},
    {"flag_palau", "1F1F5-1F1FC"},
    {"flag_paragvay", "1F1F5-1F1FE"},
    {"flag_katar", "1F1F6-1F1E6"},
    {"flag_reyunon", "1F1F7-1F1EA"},
    {"flag_rumyniya", "1F1F7-1F1F4"},
    {"flag_serbiya", "1F1F7-1F1F8"},
    {"flag_rossiya", "1F1F7-1F1FA"},
    {"flag_ruanda", "1F1F7-1F1FC"},
    {"flag_saudovskaya_araviya", "1F1F8-1F1E6"},
    {"flag_solomonovy_o_va", "1F1F8-1F1E7"},
    {"flag_seyshelskie_o_va", "1F1F8-1F1E8"},
    {"flag_sudan", "1F1F8-1F1E9"},
    {"flag_shveciya", "1F1F8-1F1EA"},
    {"flag_singapur", "1F1F8-1F1EC"},
    {"flag_o_v_sv_eleny", "1F1F8-1F1ED"},
    {"flag_sloveniya", "1F1F8-1F1EE"},
    {"flag_shpicbergen_i_yan_mayen", "1F1F8-1F1EF"},
    {"flag_slovakiya", "1F1F8-1F1F0"},
    {"flag_serra_leone", "1F1F8-1F1F1"},
    {"flag_san_marino", "1F1F8-1F1F2"},
    {"flag_senegal", "1F1F8-1F1F3"},
    {"flag_somali", "1F1F8-1F1F4"},
    {"flag_surinam", "1F1F8-1F1F7"},
    {"flag_yuzhnyy_sudan", "1F1F8-1F1F8"},
    {"flag_san_tome_i_prinsipi", "1F1F8-1F1F9"},
    {"flag_salvador", "1F1F8-1F1FB"},
    {"flag_sint_marten", "1F1F8-1F1FD"},
    {"flag_siriya", "1F1F8-1F1FE"},
    {"flag_esvatini", "1F1F8-1F1FF"},
    {"flag_tristan_da_kunya", "1F1F9-1F1E6"},
    {"flag_tyorks_i_kaykos", "1F1F9-1F1E8"},
    {"flag_chad", "1F1F9-1F1E9"},
    {"flag_francuzskie_yuzhnye_territorii", "1F1F9-1F1EB"},
    {"flag_togo", "1F1F9-1F1EC"},
    {"flag_tailand", "1F1F9-1F1ED"},
    {"flag_tadzhikistan", "1F1F9-1F1EF"},
    {"flag_tokelau", "1F1F9-1F1F0"},
    {"flag_vostochnyy_timor", "1F1F9-1F1F1"},
    {"flag_turkmenistan", "1F1F9-1F1F2"},
    {"flag_tunis", "1F1F9-1F1F3"},
    {"flag_tonga", "1F1F9-1F1F4"},
    {"flag_turciya", "1F1F9-1F1F7"},
    {"flag_trinidad_i_tobago", "1F1F9-1F1F9"},
    {"flag_tuvalu", "1F1F9-1F1FB"},
    {"flag_tayvan", "1F1F9-1F1FC"},
    {"flag_tanzaniya", "1F1F9-1F1FF"},
    {"flag_ukraina", "1F1FA-1F1E6"},
    {"flag_uganda", "1F1FA-1F1EC"},
    {"flag_vneshnie_malye_o_va_ssha", "1F1FA-1F1F2"},
    {"flag_organizaciya_obedinennyh_naciy", "1F1FA-1F1F3"},
    {"flag_soedinennye_shtaty", "1F1FA-1F1F8"},
    {"flag_urugvay", "1F1FA-1F1FE"},
    {"flag_uzbekistan", "1F1FA-1F1FF"},
    {"flag_vatikan", "1F1FB-1F1E6"},
    {"flag_sent_vinsent_i_grenadiny", "1F1FB-1F1E8"},
    {"flag_venesuela", "1F1FB-1F1EA"},
    {"flag_virginskie_o_va_velikobritaniya", "1F1FB-1F1EC"},
    {"flag_virginskie_o_va_ssha", "1F1FB-1F1EE"},
    {"flag_vetnam", "1F1FB-1F1F3"},
    {"flag_vanuatu", "1F1FB-1F1FA"},
    {"flag_uollis_i_futuna", "1F1FC-1F1EB"},
    {"flag_samoa", "1F1FC-1F1F8"},
    {"flag_kosovo", "1F1FD-1F1F0"},
    {"flag_yemen", "1F1FE-1F1EA"},
    {"flag_mayotta", "1F1FE-1F1F9"},
    {"flag_yuzhno_afrikanskaya_respublika", "1F1FF-1F1E6"},
    {"flag_zambiya", "1F1FF-1F1F2"},
    {"flag_zimbabve", "1F1FF-1F1FC"},
    {"flag_angliya", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"flag_shotlandiya", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"flag_uels", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"vlyublennaya_para_muzhchina_muzhchina", "1F468-200D-2764-FE0F-200D-1F468"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"vlyublennaya_para_muzhchina_muzhchina_cvet5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"poceluy_muzhchina_muzhchina", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"poceluy_muzhchina_muzhchina_cvet1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"poceluy_muzhchina_muzhchina_cvet1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"poceluy_muzhchina_muzhchina_cvet1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"poceluy_muzhchina_muzhchina_cvet1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"poceluy_muzhchina_muzhchina_cvet1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"poceluy_muzhchina_muzhchina_cvet2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"poceluy_muzhchina_muzhchina_cvet2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"poceluy_muzhchina_muzhchina_cvet2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"poceluy_muzhchina_muzhchina_cvet2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"poceluy_muzhchina_muzhchina_cvet2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"poceluy_muzhchina_muzhchina_cvet3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"poceluy_muzhchina_muzhchina_cvet3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"poceluy_muzhchina_muzhchina_cvet3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"poceluy_muzhchina_muzhchina_cvet3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"poceluy_muzhchina_muzhchina_cvet3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"poceluy_muzhchina_muzhchina_cvet4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"poceluy_muzhchina_muzhchina_cvet4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"poceluy_muzhchina_muzhchina_cvet4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"poceluy_muzhchina_muzhchina_cvet4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"poceluy_muzhchina_muzhchina_cvet4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"poceluy_muzhchina_muzhchina_cvet5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"poceluy_muzhchina_muzhchina_cvet5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"poceluy_muzhchina_muzhchina_cvet5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"poceluy_muzhchina_muzhchina_cvet5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"poceluy_muzhchina_muzhchina_cvet5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"semya_muzhchina_malchik", "1F468-200D-1F466"},
    {"semya_muzhchina_malchik_malchik", "1F468-200D-1F466-200D-1F466"},
    {"semya_muzhchina_devochka", "1F468-200D-1F467"},
    {"semya_muzhchina_devochka_malchik", "1F468-200D-1F467-200D-1F466"},
    {"semya_muzhchina_devochka_devochka", "1F468-200D-1F467-200D-1F467"},
    {"semya_muzhchina_muzhchina_malchik", "1F468-200D-1F468-200D-1F466"},
    {"semya_muzhchina_muzhchina_malchik_malchik", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"semya_muzhchina_muzhchina_devochka", "1F468-200D-1F468-200D-1F467"},
    {"semya_muzhchina_muzhchina_devochka_malchik", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"semya_muzhchina_muzhchina_devochka_devochka", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"semya_muzhchina_zhenshchina_malchik", "1F468-200D-1F469-200D-1F466"},
    {"semya_muzhchina_zhenshchina_malchik_malchik", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"semya_muzhchina_zhenshchina_devochka", "1F468-200D-1F469-200D-1F467"},
    {"semya_muzhchina_zhenshchina_devochka_malchik", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"semya_muzhchina_zhenshchina_devochka_devochka", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"vlyublennaya_para_zhenshchina_muzhchina", "1F469-200D-2764-FE0F-200D-1F468"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"vlyublennaya_para_zhenshchina_muzhchina_cvet5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"vlyublennaya_para_zhenshchina_zhenshchina", "1F469-200D-2764-FE0F-200D-1F469"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"vlyublennaya_para_zhenshchina_zhenshchina_cvet5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"poceluy_zhenshchina_muzhchina", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"poceluy_zhenshchina_muzhchina_cvet1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"poceluy_zhenshchina_muzhchina_cvet1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"poceluy_zhenshchina_muzhchina_cvet1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"poceluy_zhenshchina_muzhchina_cvet1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"poceluy_zhenshchina_muzhchina_cvet1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"poceluy_zhenshchina_muzhchina_cvet2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"poceluy_zhenshchina_muzhchina_cvet2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"poceluy_zhenshchina_muzhchina_cvet2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"poceluy_zhenshchina_muzhchina_cvet2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"poceluy_zhenshchina_muzhchina_cvet2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"poceluy_zhenshchina_muzhchina_cvet3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"poceluy_zhenshchina_muzhchina_cvet3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"poceluy_zhenshchina_muzhchina_cvet3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"poceluy_zhenshchina_muzhchina_cvet3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"poceluy_zhenshchina_muzhchina_cvet3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"poceluy_zhenshchina_muzhchina_cvet4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"poceluy_zhenshchina_muzhchina_cvet4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"poceluy_zhenshchina_muzhchina_cvet4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"poceluy_zhenshchina_muzhchina_cvet4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"poceluy_zhenshchina_muzhchina_cvet4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"poceluy_zhenshchina_muzhchina_cvet5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"poceluy_zhenshchina_muzhchina_cvet5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"poceluy_zhenshchina_muzhchina_cvet5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"poceluy_zhenshchina_muzhchina_cvet5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"poceluy_zhenshchina_muzhchina_cvet5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"poceluy_zhenshchina_zhenshchina", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"poceluy_zhenshchina_zhenshchina_cvet1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"poceluy_zhenshchina_zhenshchina_cvet1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"poceluy_zhenshchina_zhenshchina_cvet1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"poceluy_zhenshchina_zhenshchina_cvet1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"poceluy_zhenshchina_zhenshchina_cvet1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"poceluy_zhenshchina_zhenshchina_cvet2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"poceluy_zhenshchina_zhenshchina_cvet2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"poceluy_zhenshchina_zhenshchina_cvet2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"poceluy_zhenshchina_zhenshchina_cvet2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"poceluy_zhenshchina_zhenshchina_cvet2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"poceluy_zhenshchina_zhenshchina_cvet3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"poceluy_zhenshchina_zhenshchina_cvet3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"poceluy_zhenshchina_zhenshchina_cvet3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"poceluy_zhenshchina_zhenshchina_cvet3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"poceluy_zhenshchina_zhenshchina_cvet3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"poceluy_zhenshchina_zhenshchina_cvet4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"poceluy_zhenshchina_zhenshchina_cvet4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"poceluy_zhenshchina_zhenshchina_cvet4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"poceluy_zhenshchina_zhenshchina_cvet4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"poceluy_zhenshchina_zhenshchina_cvet4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"poceluy_zhenshchina_zhenshchina_cvet5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"poceluy_zhenshchina_zhenshchina_cvet5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"poceluy_zhenshchina_zhenshchina_cvet5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"poceluy_zhenshchina_zhenshchina_cvet5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"poceluy_zhenshchina_zhenshchina_cvet5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"semya_zhenshchina_malchik", "1F469-200D-1F466"},
    {"semya_zhenshchina_malchik_malchik", "1F469-200D-1F466-200D-1F466"},
    {"semya_zhenshchina_devochka", "1F469-200D-1F467"},
    {"semya_zhenshchina_devochka_malchik", "1F469-200D-1F467-200D-1F466"},
    {"semya_zhenshchina_devochka_devochka", "1F469-200D-1F467-200D-1F467"},
    {"semya_zhenshchina_zhenshchina_malchik", "1F469-200D-1F469-200D-1F466"},
    {"semya_zhenshchina_zhenshchina_malchik_malchik", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"semya_zhenshchina_zhenshchina_devochka", "1F469-200D-1F469-200D-1F467"},
    {"semya_zhenshchina_zhenshchina_devochka_malchik", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"semya_zhenshchina_zhenshchina_devochka_devochka", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"lyudi_derzhashchiesya_za_ruki", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"lyudi_derzhashchiesya_za_ruki_cvet1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"lyudi_derzhashchiesya_za_ruki_cvet1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"lyudi_derzhashchiesya_za_ruki_cvet1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"lyudi_derzhashchiesya_za_ruki_cvet1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"lyudi_derzhashchiesya_za_ruki_cvet1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"lyudi_derzhashchiesya_za_ruki_cvet2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"lyudi_derzhashchiesya_za_ruki_cvet2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"lyudi_derzhashchiesya_za_ruki_cvet2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"lyudi_derzhashchiesya_za_ruki_cvet2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"lyudi_derzhashchiesya_za_ruki_cvet2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"lyudi_derzhashchiesya_za_ruki_cvet3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"lyudi_derzhashchiesya_za_ruki_cvet3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"lyudi_derzhashchiesya_za_ruki_cvet3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"lyudi_derzhashchiesya_za_ruki_cvet3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"lyudi_derzhashchiesya_za_ruki_cvet3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"lyudi_derzhashchiesya_za_ruki_cvet4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"lyudi_derzhashchiesya_za_ruki_cvet4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"lyudi_derzhashchiesya_za_ruki_cvet4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"lyudi_derzhashchiesya_za_ruki_cvet4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"lyudi_derzhashchiesya_za_ruki_cvet4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"lyudi_derzhashchiesya_za_ruki_cvet5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"lyudi_derzhashchiesya_za_ruki_cvet5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"lyudi_derzhashchiesya_za_ruki_cvet5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"lyudi_derzhashchiesya_za_ruki_cvet5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"lyudi_derzhashchiesya_za_ruki_cvet5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"semya_dvoe_vzroslyh_i_rebenok", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"semya_dvoe_vzroslyh_i_dvoe_detey", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"semya_vzroslyy_i_rebenok", "1F9D1-200D-1F9D2"},
    {"semya_vzroslyy_i_dvoe_detey", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"chelovek_bezhit_napravlennye_vpravo", "1F3C3-200D-27A1-FE0F"},
    {"chelovek_bezhit_napravlennye_vpravo_cvet1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"chelovek_bezhit_napravlennye_vpravo_cvet2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"chelovek_bezhit_napravlennye_vpravo_cvet3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"chelovek_bezhit_napravlennye_vpravo_cvet4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"chelovek_bezhit_napravlennye_vpravo_cvet5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"vrach", "1F468-200D-2695-FE0F"},
    {"vrach_cvet1", "1F468-1F3FB-200D-2695-FE0F"},
    {"vrach_cvet2", "1F468-1F3FC-200D-2695-FE0F"},
    {"vrach_cvet3", "1F468-1F3FD-200D-2695-FE0F"},
    {"vrach_cvet4", "1F468-1F3FE-200D-2695-FE0F"},
    {"vrach_cvet5", "1F468-1F3FF-200D-2695-FE0F"},
    {"muzhchina_sudya", "1F468-200D-2696-FE0F"},
    {"muzhchina_sudya_cvet1", "1F468-1F3FB-200D-2696-FE0F"},
    {"muzhchina_sudya_cvet2", "1F468-1F3FC-200D-2696-FE0F"},
    {"muzhchina_sudya_cvet3", "1F468-1F3FD-200D-2696-FE0F"},
    {"muzhchina_sudya_cvet4", "1F468-1F3FE-200D-2696-FE0F"},
    {"muzhchina_sudya_cvet5", "1F468-1F3FF-200D-2696-FE0F"},
    {"muzhchina_pilot", "1F468-200D-2708-FE0F"},
    {"muzhchina_pilot_cvet1", "1F468-1F3FB-200D-2708-FE0F"},
    {"muzhchina_pilot_cvet2", "1F468-1F3FC-200D-2708-FE0F"},
    {"muzhchina_pilot_cvet3", "1F468-1F3FD-200D-2708-FE0F"},
    {"muzhchina_pilot_cvet4", "1F468-1F3FE-200D-2708-FE0F"},
    {"muzhchina_pilot_cvet5", "1F468-1F3FF-200D-2708-FE0F"},
    {"muzhchina_fermer", "1F468-200D-1F33E"},
    {"muzhchina_fermer_cvet1", "1F468-1F3FB-200D-1F33E"},
    {"muzhchina_fermer_cvet2", "1F468-1F3FC-200D-1F33E"},
    {"muzhchina_fermer_cvet3", "1F468-1F3FD-200D-1F33E"},
    {"muzhchina_fermer_cvet4", "1F468-1F3FE-200D-1F33E"},
    {"muzhchina_fermer_cvet5", "1F468-1F3FF-200D-1F33E"},
    {"muzhchina_povar", "1F468-200D-1F373"},
    {"muzhchina_povar_cvet1", "1F468-1F3FB-200D-1F373"},
    {"muzhchina_povar_cvet2", "1F468-1F3FC-200D-1F373"},
    {"muzhchina_povar_cvet3", "1F468-1F3FD-200D-1F373"},
    {"muzhchina_povar_cvet4", "1F468-1F3FE-200D-1F373"},
    {"muzhchina_povar_cvet5", "1F468-1F3FF-200D-1F373"},
    {"kormyashchiy_muzhchina", "1F468-200D-1F37C"},
    {"kormyashchiy_muzhchina_cvet1", "1F468-1F3FB-200D-1F37C"},
    {"kormyashchiy_muzhchina_cvet2", "1F468-1F3FC-200D-1F37C"},
    {"kormyashchiy_muzhchina_cvet3", "1F468-1F3FD-200D-1F37C"},
    {"kormyashchiy_muzhchina_cvet4", "1F468-1F3FE-200D-1F37C"},
    {"kormyashchiy_muzhchina_cvet5", "1F468-1F3FF-200D-1F37C"},
    {"student", "1F468-200D-1F393"},
    {"student_cvet1", "1F468-1F3FB-200D-1F393"},
    {"student_cvet2", "1F468-1F3FC-200D-1F393"},
    {"student_cvet3", "1F468-1F3FD-200D-1F393"},
    {"student_cvet4", "1F468-1F3FE-200D-1F393"},
    {"student_cvet5", "1F468-1F3FF-200D-1F393"},
    {"muzhchina_pevec", "1F468-200D-1F3A4"},
    {"muzhchina_pevec_cvet1", "1F468-1F3FB-200D-1F3A4"},
    {"muzhchina_pevec_cvet2", "1F468-1F3FC-200D-1F3A4"},
    {"muzhchina_pevec_cvet3", "1F468-1F3FD-200D-1F3A4"},
    {"muzhchina_pevec_cvet4", "1F468-1F3FE-200D-1F3A4"},
    {"muzhchina_pevec_cvet5", "1F468-1F3FF-200D-1F3A4"},
    {"muzhchina_hudozhnik", "1F468-200D-1F3A8"},
    {"muzhchina_hudozhnik_cvet1", "1F468-1F3FB-200D-1F3A8"},
    {"muzhchina_hudozhnik_cvet2", "1F468-1F3FC-200D-1F3A8"},
    {"muzhchina_hudozhnik_cvet3", "1F468-1F3FD-200D-1F3A8"},
    {"muzhchina_hudozhnik_cvet4", "1F468-1F3FE-200D-1F3A8"},
    {"muzhchina_hudozhnik_cvet5", "1F468-1F3FF-200D-1F3A8"},
    {"uchitel", "1F468-200D-1F3EB"},
    {"uchitel_cvet1", "1F468-1F3FB-200D-1F3EB"},
    {"uchitel_cvet2", "1F468-1F3FC-200D-1F3EB"},
    {"uchitel_cvet3", "1F468-1F3FD-200D-1F3EB"},
    {"uchitel_cvet4", "1F468-1F3FE-200D-1F3EB"},
    {"uchitel_cvet5", "1F468-1F3FF-200D-1F3EB"},
    {"muzhchina_rabochiy", "1F468-200D-1F3ED"},
    {"muzhchina_rabochiy_cvet1", "1F468-1F3FB-200D-1F3ED"},
    {"muzhchina_rabochiy_cvet2", "1F468-1F3FC-200D-1F3ED"},
    {"muzhchina_rabochiy_cvet3", "1F468-1F3FD-200D-1F3ED"},
    {"muzhchina_rabochiy_cvet4", "1F468-1F3FE-200D-1F3ED"},
    {"muzhchina_rabochiy_cvet5", "1F468-1F3FF-200D-1F3ED"},
    {"muzhchina_inzhener", "1F468-200D-1F4BB"},
    {"muzhchina_inzhener_cvet1", "1F468-1F3FB-200D-1F4BB"},
    {"muzhchina_inzhener_cvet2", "1F468-1F3FC-200D-1F4BB"},
    {"muzhchina_inzhener_cvet3", "1F468-1F3FD-200D-1F4BB"},
    {"muzhchina_inzhener_cvet4", "1F468-1F3FE-200D-1F4BB"},
    {"muzhchina_inzhener_cvet5", "1F468-1F3FF-200D-1F4BB"},
    {"ofisnyy_rabotnik_muzhchina", "1F468-200D-1F4BC"},
    {"ofisnyy_rabotnik_muzhchina_cvet1", "1F468-1F3FB-200D-1F4BC"},
    {"ofisnyy_rabotnik_muzhchina_cvet2", "1F468-1F3FC-200D-1F4BC"},
    {"ofisnyy_rabotnik_muzhchina_cvet3", "1F468-1F3FD-200D-1F4BC"},
    {"ofisnyy_rabotnik_muzhchina_cvet4", "1F468-1F3FE-200D-1F4BC"},
    {"ofisnyy_rabotnik_muzhchina_cvet5", "1F468-1F3FF-200D-1F4BC"},
    {"muzhchina_mehanik", "1F468-200D-1F527"},
    {"muzhchina_mehanik_cvet1", "1F468-1F3FB-200D-1F527"},
    {"muzhchina_mehanik_cvet2", "1F468-1F3FC-200D-1F527"},
    {"muzhchina_mehanik_cvet3", "1F468-1F3FD-200D-1F527"},
    {"muzhchina_mehanik_cvet4", "1F468-1F3FE-200D-1F527"},
    {"muzhchina_mehanik_cvet5", "1F468-1F3FF-200D-1F527"},
    {"muzhchina_uchenyy", "1F468-200D-1F52C"},
    {"muzhchina_uchenyy_cvet1", "1F468-1F3FB-200D-1F52C"},
    {"muzhchina_uchenyy_cvet2", "1F468-1F3FC-200D-1F52C"},
    {"muzhchina_uchenyy_cvet3", "1F468-1F3FD-200D-1F52C"},
    {"muzhchina_uchenyy_cvet4", "1F468-1F3FE-200D-1F52C"},
    {"muzhchina_uchenyy_cvet5", "1F468-1F3FF-200D-1F52C"},
    {"muzhchina_kosmonavt", "1F468-200D-1F680"},
    {"muzhchina_kosmonavt_cvet1", "1F468-1F3FB-200D-1F680"},
    {"muzhchina_kosmonavt_cvet2", "1F468-1F3FC-200D-1F680"},
    {"muzhchina_kosmonavt_cvet3", "1F468-1F3FD-200D-1F680"},
    {"muzhchina_kosmonavt_cvet4", "1F468-1F3FE-200D-1F680"},
    {"muzhchina_kosmonavt_cvet5", "1F468-1F3FF-200D-1F680"},
    {"muzhchina_pozharnyy", "1F468-200D-1F692"},
    {"muzhchina_pozharnyy_cvet1", "1F468-1F3FB-200D-1F692"},
    {"muzhchina_pozharnyy_cvet2", "1F468-1F3FC-200D-1F692"},
    {"muzhchina_pozharnyy_cvet3", "1F468-1F3FD-200D-1F692"},
    {"muzhchina_pozharnyy_cvet4", "1F468-1F3FE-200D-1F692"},
    {"muzhchina_pozharnyy_cvet5", "1F468-1F3FF-200D-1F692"},
    {"muzhchina_s_trostyu_dlya_hodby", "1F468-200D-1F9AF"},
    {"muzhchina_s_trostyu_dlya_hodby_cvet1", "1F468-1F3FB-200D-1F9AF"},
    {"muzhchina_s_trostyu_dlya_hodby_cvet2", "1F468-1F3FC-200D-1F9AF"},
    {"muzhchina_s_trostyu_dlya_hodby_cvet3", "1F468-1F3FD-200D-1F9AF"},
    {"muzhchina_s_trostyu_dlya_hodby_cvet4", "1F468-1F3FE-200D-1F9AF"},
    {"muzhchina_s_trostyu_dlya_hodby_cvet5", "1F468-1F3FF-200D-1F9AF"},
    {"muzhchina_s_trostyu_dlya_hodby_napravlennye_vpravo", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"muzhchina_s_trostyu_dlya_hodby_napravlennye_vpravo_cvet1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"muzhchina_s_trostyu_dlya_hodby_napravlennye_vpravo_cvet2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"muzhchina_s_trostyu_dlya_hodby_napravlennye_vpravo_cvet3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"muzhchina_s_trostyu_dlya_hodby_napravlennye_vpravo_cvet4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"muzhchina_s_trostyu_dlya_hodby_napravlennye_vpravo_cvet5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"muzhchina_v_elektricheskom_kresle_kolyaske", "1F468-200D-1F9BC"},
    {"muzhchina_v_elektricheskom_kresle_kolyaske_cvet1", "1F468-1F3FB-200D-1F9BC"},
    {"muzhchina_v_elektricheskom_kresle_kolyaske_cvet2", "1F468-1F3FC-200D-1F9BC"},
    {"muzhchina_v_elektricheskom_kresle_kolyaske_cvet3", "1F468-1F3FD-200D-1F9BC"},
    {"muzhchina_v_elektricheskom_kresle_kolyaske_cvet4", "1F468-1F3FE-200D-1F9BC"},
    {"muzhchina_v_elektricheskom_kresle_kolyaske_cvet5", "1F468-1F3FF-200D-1F9BC"},
    {"muzhchina_v_elektricheskom_kresle_kolyaske_napravlennye_vpravo", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"muzhchina_v_elektricheskom_kresle_kolyaske_napravlennye_vpravo_cvet1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"muzhchina_v_elektricheskom_kresle_kolyaske_napravlennye_vpravo_cvet2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"muzhchina_v_elektricheskom_kresle_kolyaske_napravlennye_vpravo_cvet3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"muzhchina_v_elektricheskom_kresle_kolyaske_napravlennye_vpravo_cvet4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"muzhchina_v_elektricheskom_kresle_kolyaske_napravlennye_vpravo_cvet5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"muzhchina_v_ruchnom_kresle_kolyaske", "1F468-200D-1F9BD"},
    {"muzhchina_v_ruchnom_kresle_kolyaske_cvet1", "1F468-1F3FB-200D-1F9BD"},
    {"muzhchina_v_ruchnom_kresle_kolyaske_cvet2", "1F468-1F3FC-200D-1F9BD"},
    {"muzhchina_v_ruchnom_kresle_kolyaske_cvet3", "1F468-1F3FD-200D-1F9BD"},
    {"muzhchina_v_ruchnom_kresle_kolyaske_cvet4", "1F468-1F3FE-200D-1F9BD"},
    {"muzhchina_v_ruchnom_kresle_kolyaske_cvet5", "1F468-1F3FF-200D-1F9BD"},
    {"muzhchina_v_ruchnom_kresle_kolyaske_napravlennye_vpravo", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"muzhchina_v_ruchnom_kresle_kolyaske_napravlennye_vpravo_cvet1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"muzhchina_v_ruchnom_kresle_kolyaske_napravlennye_vpravo_cvet2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"muzhchina_v_ruchnom_kresle_kolyaske_napravlennye_vpravo_cvet3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"muzhchina_v_ruchnom_kresle_kolyaske_napravlennye_vpravo_cvet4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"muzhchina_v_ruchnom_kresle_kolyaske_napravlennye_vpravo_cvet5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"zhenshchina_vrach", "1F469-200D-2695-FE0F"},
    {"zhenshchina_vrach_cvet1", "1F469-1F3FB-200D-2695-FE0F"},
    {"zhenshchina_vrach_cvet2", "1F469-1F3FC-200D-2695-FE0F"},
    {"zhenshchina_vrach_cvet3", "1F469-1F3FD-200D-2695-FE0F"},
    {"zhenshchina_vrach_cvet4", "1F469-1F3FE-200D-2695-FE0F"},
    {"zhenshchina_vrach_cvet5", "1F469-1F3FF-200D-2695-FE0F"},
    {"zhenshchina_sudya", "1F469-200D-2696-FE0F"},
    {"zhenshchina_sudya_cvet1", "1F469-1F3FB-200D-2696-FE0F"},
    {"zhenshchina_sudya_cvet2", "1F469-1F3FC-200D-2696-FE0F"},
    {"zhenshchina_sudya_cvet3", "1F469-1F3FD-200D-2696-FE0F"},
    {"zhenshchina_sudya_cvet4", "1F469-1F3FE-200D-2696-FE0F"},
    {"zhenshchina_sudya_cvet5", "1F469-1F3FF-200D-2696-FE0F"},
    {"zhenshchina_pilot", "1F469-200D-2708-FE0F"},
    {"zhenshchina_pilot_cvet1", "1F469-1F3FB-200D-2708-FE0F"},
    {"zhenshchina_pilot_cvet2", "1F469-1F3FC-200D-2708-FE0F"},
    {"zhenshchina_pilot_cvet3", "1F469-1F3FD-200D-2708-FE0F"},
    {"zhenshchina_pilot_cvet4", "1F469-1F3FE-200D-2708-FE0F"},
    {"zhenshchina_pilot_cvet5", "1F469-1F3FF-200D-2708-FE0F"},
    {"zhenshchina_fermer", "1F469-200D-1F33E"},
    {"zhenshchina_fermer_cvet1", "1F469-1F3FB-200D-1F33E"},
    {"zhenshchina_fermer_cvet2", "1F469-1F3FC-200D-1F33E"},
    {"zhenshchina_fermer_cvet3", "1F469-1F3FD-200D-1F33E"},
    {"zhenshchina_fermer_cvet4", "1F469-1F3FE-200D-1F33E"},
    {"zhenshchina_fermer_cvet5", "1F469-1F3FF-200D-1F33E"},
    {"povariha", "1F469-200D-1F373"},
    {"povariha_cvet1", "1F469-1F3FB-200D-1F373"},
    {"povariha_cvet2", "1F469-1F3FC-200D-1F373"},
    {"povariha_cvet3", "1F469-1F3FD-200D-1F373"},
    {"povariha_cvet4", "1F469-1F3FE-200D-1F373"},
    {"povariha_cvet5", "1F469-1F3FF-200D-1F373"},
    {"kormyashchaya_zhenshchina", "1F469-200D-1F37C"},
    {"kormyashchaya_zhenshchina_cvet1", "1F469-1F3FB-200D-1F37C"},
    {"kormyashchaya_zhenshchina_cvet2", "1F469-1F3FC-200D-1F37C"},
    {"kormyashchaya_zhenshchina_cvet3", "1F469-1F3FD-200D-1F37C"},
    {"kormyashchaya_zhenshchina_cvet4", "1F469-1F3FE-200D-1F37C"},
    {"kormyashchaya_zhenshchina_cvet5", "1F469-1F3FF-200D-1F37C"},
    {"studentka", "1F469-200D-1F393"},
    {"studentka_cvet1", "1F469-1F3FB-200D-1F393"},
    {"studentka_cvet2", "1F469-1F3FC-200D-1F393"},
    {"studentka_cvet3", "1F469-1F3FD-200D-1F393"},
    {"studentka_cvet4", "1F469-1F3FE-200D-1F393"},
    {"studentka_cvet5", "1F469-1F3FF-200D-1F393"},
    {"pevica", "1F469-200D-1F3A4"},
    {"pevica_cvet1", "1F469-1F3FB-200D-1F3A4"},
    {"pevica_cvet2", "1F469-1F3FC-200D-1F3A4"},
    {"pevica_cvet3", "1F469-1F3FD-200D-1F3A4"},
    {"pevica_cvet4", "1F469-1F3FE-200D-1F3A4"},
    {"pevica_cvet5", "1F469-1F3FF-200D-1F3A4"},
    {"hudozhnica", "1F469-200D-1F3A8"},
    {"hudozhnica_cvet1", "1F469-1F3FB-200D-1F3A8"},
    {"hudozhnica_cvet2", "1F469-1F3FC-200D-1F3A8"},
    {"hudozhnica_cvet3", "1F469-1F3FD-200D-1F3A8"},
    {"hudozhnica_cvet4", "1F469-1F3FE-200D-1F3A8"},
    {"hudozhnica_cvet5", "1F469-1F3FF-200D-1F3A8"},
    {"uchitelnica", "1F469-200D-1F3EB"},
    {"uchitelnica_cvet1", "1F469-1F3FB-200D-1F3EB"},
    {"uchitelnica_cvet2", "1F469-1F3FC-200D-1F3EB"},
    {"uchitelnica_cvet3", "1F469-1F3FD-200D-1F3EB"},
    {"uchitelnica_cvet4", "1F469-1F3FE-200D-1F3EB"},
    {"uchitelnica_cvet5", "1F469-1F3FF-200D-1F3EB"},
    {"rabotnica", "1F469-200D-1F3ED"},
    {"rabotnica_cvet1", "1F469-1F3FB-200D-1F3ED"},
    {"rabotnica_cvet2", "1F469-1F3FC-200D-1F3ED"},
    {"rabotnica_cvet3", "1F469-1F3FD-200D-1F3ED"},
    {"rabotnica_cvet4", "1F469-1F3FE-200D-1F3ED"},
    {"rabotnica_cvet5", "1F469-1F3FF-200D-1F3ED"},
    {"zhenshchina_inzhener", "1F469-200D-1F4BB"},
    {"zhenshchina_inzhener_cvet1", "1F469-1F3FB-200D-1F4BB"},
    {"zhenshchina_inzhener_cvet2", "1F469-1F3FC-200D-1F4BB"},
    {"zhenshchina_inzhener_cvet3", "1F469-1F3FD-200D-1F4BB"},
    {"zhenshchina_inzhener_cvet4", "1F469-1F3FE-200D-1F4BB"},
    {"zhenshchina_inzhener_cvet5", "1F469-1F3FF-200D-1F4BB"},
    {"ofisnaya_rabotnica", "1F469-200D-1F4BC"},
    {"ofisnaya_rabotnica_cvet1", "1F469-1F3FB-200D-1F4BC"},
    {"ofisnaya_rabotnica_cvet2", "1F469-1F3FC-200D-1F4BC"},
    {"ofisnaya_rabotnica_cvet3", "1F469-1F3FD-200D-1F4BC"},
    {"ofisnaya_rabotnica_cvet4", "1F469-1F3FE-200D-1F4BC"},
    {"ofisnaya_rabotnica_cvet5", "1F469-1F3FF-200D-1F4BC"},
    {"zhenshchina_mehanik", "1F469-200D-1F527"},
    {"zhenshchina_mehanik_cvet1", "1F469-1F3FB-200D-1F527"},
    {"zhenshchina_mehanik_cvet2", "1F469-1F3FC-200D-1F527"},
    {"zhenshchina_mehanik_cvet3", "1F469-1F3FD-200D-1F527"},
    {"zhenshchina_mehanik_cvet4", "1F469-1F3FE-200D-1F527"},
    {"zhenshchina_mehanik_cvet5", "1F469-1F3FF-200D-1F527"},
    {"uchenaya", "1F469-200D-1F52C"},
    {"uchenaya_cvet1", "1F469-1F3FB-200D-1F52C"},
    {"uchenaya_cvet2", "1F469-1F3FC-200D-1F52C"},
    {"uchenaya_cvet3", "1F469-1F3FD-200D-1F52C"},
    {"uchenaya_cvet4", "1F469-1F3FE-200D-1F52C"},
    {"uchenaya_cvet5", "1F469-1F3FF-200D-1F52C"},
    {"zhenshchina_kosmonavt", "1F469-200D-1F680"},
    {"zhenshchina_kosmonavt_cvet1", "1F469-1F3FB-200D-1F680"},
    {"zhenshchina_kosmonavt_cvet2", "1F469-1F3FC-200D-1F680"},
    {"zhenshchina_kosmonavt_cvet3", "1F469-1F3FD-200D-1F680"},
    {"zhenshchina_kosmonavt_cvet4", "1F469-1F3FE-200D-1F680"},
    {"zhenshchina_kosmonavt_cvet5", "1F469-1F3FF-200D-1F680"},
    {"zhenshchina_pozharnyy", "1F469-200D-1F692"},
    {"zhenshchina_pozharnyy_cvet1", "1F469-1F3FB-200D-1F692"},
    {"zhenshchina_pozharnyy_cvet2", "1F469-1F3FC-200D-1F692"},
    {"zhenshchina_pozharnyy_cvet3", "1F469-1F3FD-200D-1F692"},
    {"zhenshchina_pozharnyy_cvet4", "1F469-1F3FE-200D-1F692"},
    {"zhenshchina_pozharnyy_cvet5", "1F469-1F3FF-200D-1F692"},
    {"zhenshchina_s_trostyu_dlya_hodby", "1F469-200D-1F9AF"},
    {"zhenshchina_s_trostyu_dlya_hodby_cvet1", "1F469-1F3FB-200D-1F9AF"},
    {"zhenshchina_s_trostyu_dlya_hodby_cvet2", "1F469-1F3FC-200D-1F9AF"},
    {"zhenshchina_s_trostyu_dlya_hodby_cvet3", "1F469-1F3FD-200D-1F9AF"},
    {"zhenshchina_s_trostyu_dlya_hodby_cvet4", "1F469-1F3FE-200D-1F9AF"},
    {"zhenshchina_s_trostyu_dlya_hodby_cvet5", "1F469-1F3FF-200D-1F9AF"},
    {"zhenshchina_s_trostyu_dlya_hodby_napravlennye_vpravo", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"zhenshchina_s_trostyu_dlya_hodby_napravlennye_vpravo_cvet1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"zhenshchina_s_trostyu_dlya_hodby_napravlennye_vpravo_cvet2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"zhenshchina_s_trostyu_dlya_hodby_napravlennye_vpravo_cvet3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"zhenshchina_s_trostyu_dlya_hodby_napravlennye_vpravo_cvet4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"zhenshchina_s_trostyu_dlya_hodby_napravlennye_vpravo_cvet5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"zhenshchina_v_elektricheskom_kresle_kolyaske", "1F469-200D-1F9BC"},
    {"zhenshchina_v_elektricheskom_kresle_kolyaske_cvet1", "1F469-1F3FB-200D-1F9BC"},
    {"zhenshchina_v_elektricheskom_kresle_kolyaske_cvet2", "1F469-1F3FC-200D-1F9BC"},
    {"zhenshchina_v_elektricheskom_kresle_kolyaske_cvet3", "1F469-1F3FD-200D-1F9BC"},
    {"zhenshchina_v_elektricheskom_kresle_kolyaske_cvet4", "1F469-1F3FE-200D-1F9BC"},
    {"zhenshchina_v_elektricheskom_kresle_kolyaske_cvet5", "1F469-1F3FF-200D-1F9BC"},
    {"zhenshchina_v_elektricheskom_kresle_kolyaske_napravlennye_vpravo", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"zhenshchina_v_elektricheskom_kresle_kolyaske_napravlennye_vpravo_cvet1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"zhenshchina_v_elektricheskom_kresle_kolyaske_napravlennye_vpravo_cvet2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"zhenshchina_v_elektricheskom_kresle_kolyaske_napravlennye_vpravo_cvet3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"zhenshchina_v_elektricheskom_kresle_kolyaske_napravlennye_vpravo_cvet4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"zhenshchina_v_elektricheskom_kresle_kolyaske_napravlennye_vpravo_cvet5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"zhenshchina_v_ruchnom_kresle_kolyaske", "1F469-200D-1F9BD"},
    {"zhenshchina_v_ruchnom_kresle_kolyaske_cvet1", "1F469-1F3FB-200D-1F9BD"},
    {"zhenshchina_v_ruchnom_kresle_kolyaske_cvet2", "1F469-1F3FC-200D-1F9BD"},
    {"zhenshchina_v_ruchnom_kresle_kolyaske_cvet3", "1F469-1F3FD-200D-1F9BD"},
    {"zhenshchina_v_ruchnom_kresle_kolyaske_cvet4", "1F469-1F3FE-200D-1F9BD"},
    {"zhenshchina_v_ruchnom_kresle_kolyaske_cvet5", "1F469-1F3FF-200D-1F9BD"},
    {"zhenshchina_v_ruchnom_kresle_kolyaske_napravlennye_vpravo", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"zhenshchina_v_ruchnom_kresle_kolyaske_napravlennye_vpravo_cvet1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"zhenshchina_v_ruchnom_kresle_kolyaske_napravlennye_vpravo_cvet2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"zhenshchina_v_ruchnom_kresle_kolyaske_napravlennye_vpravo_cvet3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"zhenshchina_v_ruchnom_kresle_kolyaske_napravlennye_vpravo_cvet4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"zhenshchina_v_ruchnom_kresle_kolyaske_napravlennye_vpravo_cvet5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"chelovek_idet_napravlennye_vpravo", "1F6B6-200D-27A1-FE0F"},
    {"chelovek_idet_napravlennye_vpravo_cvet1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"chelovek_idet_napravlennye_vpravo_cvet2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"chelovek_idet_napravlennye_vpravo_cvet3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"chelovek_idet_napravlennye_vpravo_cvet4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"chelovek_idet_napravlennye_vpravo_cvet5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"chelovek_na_kolenyah_napravlennye_vpravo", "1F9CE-200D-27A1-FE0F"},
    {"chelovek_na_kolenyah_napravlennye_vpravo_cvet1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"chelovek_na_kolenyah_napravlennye_vpravo_cvet2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"chelovek_na_kolenyah_napravlennye_vpravo_cvet3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"chelovek_na_kolenyah_napravlennye_vpravo_cvet4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"chelovek_na_kolenyah_napravlennye_vpravo_cvet5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"medicinskiy_rabotnik", "1F9D1-200D-2695-FE0F"},
    {"medicinskiy_rabotnik_cvet1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"medicinskiy_rabotnik_cvet2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"medicinskiy_rabotnik_cvet3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"medicinskiy_rabotnik_cvet4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"medicinskiy_rabotnik_cvet5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"sudya", "1F9D1-200D-2696-FE0F"},
    {"sudya_cvet1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"sudya_cvet2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"sudya_cvet3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"sudya_cvet4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"sudya_cvet5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"pilot", "1F9D1-200D-2708-FE0F"},
    {"pilot_cvet1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"pilot_cvet2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"pilot_cvet3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"pilot_cvet4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"pilot_cvet5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"fermer", "1F9D1-200D-1F33E"},
    {"fermer_cvet1", "1F9D1-1F3FB-200D-1F33E"},
    {"fermer_cvet2", "1F9D1-1F3FC-200D-1F33E"},
    {"fermer_cvet3", "1F9D1-1F3FD-200D-1F33E"},
    {"fermer_cvet4", "1F9D1-1F3FE-200D-1F33E"},
    {"fermer_cvet5", "1F9D1-1F3FF-200D-1F33E"},
    {"povar", "1F9D1-200D-1F373"},
    {"povar_cvet1", "1F9D1-1F3FB-200D-1F373"},
    {"povar_cvet2", "1F9D1-1F3FC-200D-1F373"},
    {"povar_cvet3", "1F9D1-1F3FD-200D-1F373"},
    {"povar_cvet4", "1F9D1-1F3FE-200D-1F373"},
    {"povar_cvet5", "1F9D1-1F3FF-200D-1F373"},
    {"kormyashchiy_chelovek", "1F9D1-200D-1F37C"},
    {"kormyashchiy_chelovek_cvet1", "1F9D1-1F3FB-200D-1F37C"},
    {"kormyashchiy_chelovek_cvet2", "1F9D1-1F3FC-200D-1F37C"},
    {"kormyashchiy_chelovek_cvet3", "1F9D1-1F3FD-200D-1F37C"},
    {"kormyashchiy_chelovek_cvet4", "1F9D1-1F3FE-200D-1F37C"},
    {"kormyashchiy_chelovek_cvet5", "1F9D1-1F3FF-200D-1F37C"},
    {"santa", "1F9D1-200D-1F384"},
    {"santa_cvet1", "1F9D1-1F3FB-200D-1F384"},
    {"santa_cvet2", "1F9D1-1F3FC-200D-1F384"},
    {"santa_cvet3", "1F9D1-1F3FD-200D-1F384"},
    {"santa_cvet4", "1F9D1-1F3FE-200D-1F384"},
    {"santa_cvet5", "1F9D1-1F3FF-200D-1F384"},
    {"uchashchiysya", "1F9D1-200D-1F393"},
    {"uchashchiysya_cvet1", "1F9D1-1F3FB-200D-1F393"},
    {"uchashchiysya_cvet2", "1F9D1-1F3FC-200D-1F393"},
    {"uchashchiysya_cvet3", "1F9D1-1F3FD-200D-1F393"},
    {"uchashchiysya_cvet4", "1F9D1-1F3FE-200D-1F393"},
    {"uchashchiysya_cvet5", "1F9D1-1F3FF-200D-1F393"},
    {"pevec", "1F9D1-200D-1F3A4"},
    {"pevec_cvet1", "1F9D1-1F3FB-200D-1F3A4"},
    {"pevec_cvet2", "1F9D1-1F3FC-200D-1F3A4"},
    {"pevec_cvet3", "1F9D1-1F3FD-200D-1F3A4"},
    {"pevec_cvet4", "1F9D1-1F3FE-200D-1F3A4"},
    {"pevec_cvet5", "1F9D1-1F3FF-200D-1F3A4"},
    {"hudozhnik", "1F9D1-200D-1F3A8"},
    {"hudozhnik_cvet1", "1F9D1-1F3FB-200D-1F3A8"},
    {"hudozhnik_cvet2", "1F9D1-1F3FC-200D-1F3A8"},
    {"hudozhnik_cvet3", "1F9D1-1F3FD-200D-1F3A8"},
    {"hudozhnik_cvet4", "1F9D1-1F3FE-200D-1F3A8"},
    {"hudozhnik_cvet5", "1F9D1-1F3FF-200D-1F3A8"},
    {"pedagog", "1F9D1-200D-1F3EB"},
    {"pedagog_cvet1", "1F9D1-1F3FB-200D-1F3EB"},
    {"pedagog_cvet2", "1F9D1-1F3FC-200D-1F3EB"},
    {"pedagog_cvet3", "1F9D1-1F3FD-200D-1F3EB"},
    {"pedagog_cvet4", "1F9D1-1F3FE-200D-1F3EB"},
    {"pedagog_cvet5", "1F9D1-1F3FF-200D-1F3EB"},
    {"rabochiy", "1F9D1-200D-1F3ED"},
    {"rabochiy_cvet1", "1F9D1-1F3FB-200D-1F3ED"},
    {"rabochiy_cvet2", "1F9D1-1F3FC-200D-1F3ED"},
    {"rabochiy_cvet3", "1F9D1-1F3FD-200D-1F3ED"},
    {"rabochiy_cvet4", "1F9D1-1F3FE-200D-1F3ED"},
    {"rabochiy_cvet5", "1F9D1-1F3FF-200D-1F3ED"},
    {"inzhener", "1F9D1-200D-1F4BB"},
    {"inzhener_cvet1", "1F9D1-1F3FB-200D-1F4BB"},
    {"inzhener_cvet2", "1F9D1-1F3FC-200D-1F4BB"},
    {"inzhener_cvet3", "1F9D1-1F3FD-200D-1F4BB"},
    {"inzhener_cvet4", "1F9D1-1F3FE-200D-1F4BB"},
    {"inzhener_cvet5", "1F9D1-1F3FF-200D-1F4BB"},
    {"ofisnyy_rabotnik", "1F9D1-200D-1F4BC"},
    {"ofisnyy_rabotnik_cvet1", "1F9D1-1F3FB-200D-1F4BC"},
    {"ofisnyy_rabotnik_cvet2", "1F9D1-1F3FC-200D-1F4BC"},
    {"ofisnyy_rabotnik_cvet3", "1F9D1-1F3FD-200D-1F4BC"},
    {"ofisnyy_rabotnik_cvet4", "1F9D1-1F3FE-200D-1F4BC"},
    {"ofisnyy_rabotnik_cvet5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mehanik", "1F9D1-200D-1F527"},
    {"mehanik_cvet1", "1F9D1-1F3FB-200D-1F527"},
    {"mehanik_cvet2", "1F9D1-1F3FC-200D-1F527"},
    {"mehanik_cvet3", "1F9D1-1F3FD-200D-1F527"},
    {"mehanik_cvet4", "1F9D1-1F3FE-200D-1F527"},
    {"mehanik_cvet5", "1F9D1-1F3FF-200D-1F527"},
    {"uchenyy", "1F9D1-200D-1F52C"},
    {"uchenyy_cvet1", "1F9D1-1F3FB-200D-1F52C"},
    {"uchenyy_cvet2", "1F9D1-1F3FC-200D-1F52C"},
    {"uchenyy_cvet3", "1F9D1-1F3FD-200D-1F52C"},
    {"uchenyy_cvet4", "1F9D1-1F3FE-200D-1F52C"},
    {"uchenyy_cvet5", "1F9D1-1F3FF-200D-1F52C"},
    {"kosmonavt", "1F9D1-200D-1F680"},
    {"kosmonavt_cvet1", "1F9D1-1F3FB-200D-1F680"},
    {"kosmonavt_cvet2", "1F9D1-1F3FC-200D-1F680"},
    {"kosmonavt_cvet3", "1F9D1-1F3FD-200D-1F680"},
    {"kosmonavt_cvet4", "1F9D1-1F3FE-200D-1F680"},
    {"kosmonavt_cvet5", "1F9D1-1F3FF-200D-1F680"},
    {"pozharnyy", "1F9D1-200D-1F692"},
    {"pozharnyy_cvet1", "1F9D1-1F3FB-200D-1F692"},
    {"pozharnyy_cvet2", "1F9D1-1F3FC-200D-1F692"},
    {"pozharnyy_cvet3", "1F9D1-1F3FD-200D-1F692"},
    {"pozharnyy_cvet4", "1F9D1-1F3FE-200D-1F692"},
    {"pozharnyy_cvet5", "1F9D1-1F3FF-200D-1F692"},
    {"chelovek_s_beloy_trostyu", "1F9D1-200D-1F9AF"},
    {"chelovek_s_beloy_trostyu_cvet1", "1F9D1-1F3FB-200D-1F9AF"},
    {"chelovek_s_beloy_trostyu_cvet2", "1F9D1-1F3FC-200D-1F9AF"},
    {"chelovek_s_beloy_trostyu_cvet3", "1F9D1-1F3FD-200D-1F9AF"},
    {"chelovek_s_beloy_trostyu_cvet4", "1F9D1-1F3FE-200D-1F9AF"},
    {"chelovek_s_beloy_trostyu_cvet5", "1F9D1-1F3FF-200D-1F9AF"},
    {"chelovek_s_beloy_trostyu_napravlennye_vpravo", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"chelovek_s_beloy_trostyu_napravlennye_vpravo_cvet1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"chelovek_s_beloy_trostyu_napravlennye_vpravo_cvet2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"chelovek_s_beloy_trostyu_napravlennye_vpravo_cvet3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"chelovek_s_beloy_trostyu_napravlennye_vpravo_cvet4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"chelovek_s_beloy_trostyu_napravlennye_vpravo_cvet5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"chelovek_v_kresle_kolyaske_s_motorom", "1F9D1-200D-1F9BC"},
    {"chelovek_v_kresle_kolyaske_s_motorom_cvet1", "1F9D1-1F3FB-200D-1F9BC"},
    {"chelovek_v_kresle_kolyaske_s_motorom_cvet2", "1F9D1-1F3FC-200D-1F9BC"},
    {"chelovek_v_kresle_kolyaske_s_motorom_cvet3", "1F9D1-1F3FD-200D-1F9BC"},
    {"chelovek_v_kresle_kolyaske_s_motorom_cvet4", "1F9D1-1F3FE-200D-1F9BC"},
    {"chelovek_v_kresle_kolyaske_s_motorom_cvet5", "1F9D1-1F3FF-200D-1F9BC"},
    {"chelovek_v_kresle_kolyaske_s_motorom_napravlennye_vpravo", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"chelovek_v_kresle_kolyaske_s_motorom_napravlennye_vpravo_cvet1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"chelovek_v_kresle_kolyaske_s_motorom_napravlennye_vpravo_cvet2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"chelovek_v_kresle_kolyaske_s_motorom_napravlennye_vpravo_cvet3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"chelovek_v_kresle_kolyaske_s_motorom_napravlennye_vpravo_cvet4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"chelovek_v_kresle_kolyaske_s_motorom_napravlennye_vpravo_cvet5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"chelovek_v_kresle_kolyaske_s_ruchnym_privodom", "1F9D1-200D-1F9BD"},
    {"chelovek_v_kresle_kolyaske_s_ruchnym_privodom_cvet1", "1F9D1-1F3FB-200D-1F9BD"},
    {"chelovek_v_kresle_kolyaske_s_ruchnym_privodom_cvet2", "1F9D1-1F3FC-200D-1F9BD"},
    {"chelovek_v_kresle_kolyaske_s_ruchnym_privodom_cvet3", "1F9D1-1F3FD-200D-1F9BD"},
    {"chelovek_v_kresle_kolyaske_s_ruchnym_privodom_cvet4", "1F9D1-1F3FE-200D-1F9BD"},
    {"chelovek_v_kresle_kolyaske_s_ruchnym_privodom_cvet5", "1F9D1-1F3FF-200D-1F9BD"},
    {"chelovek_v_kresle_kolyaske_s_ruchnym_privodom_napravlennye_vpravo", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"chelovek_v_kresle_kolyaske_s_ruchnym_privodom_napravlennye_vpravo_cvet1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"chelovek_v_kresle_kolyaske_s_ruchnym_privodom_napravlennye_vpravo_cvet2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"chelovek_v_kresle_kolyaske_s_ruchnym_privodom_napravlennye_vpravo_cvet3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"chelovek_v_kresle_kolyaske_s_ruchnym_privodom_napravlennye_vpravo_cvet4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"chelovek_v_kresle_kolyaske_s_ruchnym_privodom_napravlennye_vpravo_cvet5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"zhenshchina_s_myachom", "26F9-FE0F-200D-2640-FE0F"},
    {"zhenshchina_s_myachom_cvet1", "26F9-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_s_myachom_cvet2", "26F9-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_s_myachom_cvet3", "26F9-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_s_myachom_cvet4", "26F9-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_s_myachom_cvet5", "26F9-1F3FF-200D-2640-FE0F"},
    {"muzhchina_s_myachom", "26F9-FE0F-200D-2642-FE0F"},
    {"muzhchina_s_myachom_cvet1", "26F9-1F3FB-200D-2642-FE0F"},
    {"muzhchina_s_myachom_cvet2", "26F9-1F3FC-200D-2642-FE0F"},
    {"muzhchina_s_myachom_cvet3", "26F9-1F3FD-200D-2642-FE0F"},
    {"muzhchina_s_myachom_cvet4", "26F9-1F3FE-200D-2642-FE0F"},
    {"muzhchina_s_myachom_cvet5", "26F9-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_bezhit", "1F3C3-200D-2640-FE0F"},
    {"zhenshchina_bezhit_cvet1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_bezhit_cvet2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_bezhit_cvet3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_bezhit_cvet4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_bezhit_cvet5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"zhenshchina_bezhit_napravlennye_vpravo", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_bezhit_napravlennye_vpravo_cvet1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_bezhit_napravlennye_vpravo_cvet2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_bezhit_napravlennye_vpravo_cvet3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_bezhit_napravlennye_vpravo_cvet4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_bezhit_napravlennye_vpravo_cvet5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"muzhchina_bezhit", "1F3C3-200D-2642-FE0F"},
    {"muzhchina_bezhit_cvet1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"muzhchina_bezhit_cvet2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"muzhchina_bezhit_cvet3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"muzhchina_bezhit_cvet4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"muzhchina_bezhit_cvet5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"muzhchina_bezhit_napravlennye_vpravo", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_bezhit_napravlennye_vpravo_cvet1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_bezhit_napravlennye_vpravo_cvet2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_bezhit_napravlennye_vpravo_cvet3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_bezhit_napravlennye_vpravo_cvet4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_bezhit_napravlennye_vpravo_cvet5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"serfingistka", "1F3C4-200D-2640-FE0F"},
    {"serfingistka_cvet1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"serfingistka_cvet2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"serfingistka_cvet3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"serfingistka_cvet4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"serfingistka_cvet5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"serfingist", "1F3C4-200D-2642-FE0F"},
    {"serfingist_cvet1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"serfingist_cvet2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"serfingist_cvet3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"serfingist_cvet4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"serfingist_cvet5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"plovchiha", "1F3CA-200D-2640-FE0F"},
    {"plovchiha_cvet1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"plovchiha_cvet2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"plovchiha_cvet3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"plovchiha_cvet4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"plovchiha_cvet5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"plovec", "1F3CA-200D-2642-FE0F"},
    {"plovec_cvet1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"plovec_cvet2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"plovec_cvet3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"plovec_cvet4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"plovec_cvet5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_so_shtangoy", "1F3CB-FE0F-200D-2640-FE0F"},
    {"zhenshchina_so_shtangoy_cvet1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_so_shtangoy_cvet2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_so_shtangoy_cvet3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_so_shtangoy_cvet4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_so_shtangoy_cvet5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"muzhchina_so_shtangoy", "1F3CB-FE0F-200D-2642-FE0F"},
    {"muzhchina_so_shtangoy_cvet1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"muzhchina_so_shtangoy_cvet2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"muzhchina_so_shtangoy_cvet3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"muzhchina_so_shtangoy_cvet4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"muzhchina_so_shtangoy_cvet5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_igraet_v_golf", "1F3CC-FE0F-200D-2640-FE0F"},
    {"zhenshchina_igraet_v_golf_cvet1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_igraet_v_golf_cvet2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_igraet_v_golf_cvet3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_igraet_v_golf_cvet4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_igraet_v_golf_cvet5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"muzhchina_igraet_v_golf", "1F3CC-FE0F-200D-2642-FE0F"},
    {"muzhchina_igraet_v_golf_cvet1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"muzhchina_igraet_v_golf_cvet2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"muzhchina_igraet_v_golf_cvet3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"muzhchina_igraet_v_golf_cvet4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"muzhchina_igraet_v_golf_cvet5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_policeyskiy", "1F46E-200D-2640-FE0F"},
    {"zhenshchina_policeyskiy_cvet1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_policeyskiy_cvet2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_policeyskiy_cvet3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_policeyskiy_cvet4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_policeyskiy_cvet5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"muzhchina_policeyskiy", "1F46E-200D-2642-FE0F"},
    {"muzhchina_policeyskiy_cvet1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"muzhchina_policeyskiy_cvet2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"muzhchina_policeyskiy_cvet3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"muzhchina_policeyskiy_cvet4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"muzhchina_policeyskiy_cvet5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"zhenshchiny_s_ushami_krolika", "1F46F-200D-2640-FE0F"},
    {"zhenshchiny_s_ushami_krolika_cvet1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"zhenshchiny_s_ushami_krolika_cvet1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"zhenshchiny_s_ushami_krolika_cvet1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"zhenshchiny_s_ushami_krolika_cvet1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"zhenshchiny_s_ushami_krolika_cvet2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"zhenshchiny_s_ushami_krolika_cvet2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"zhenshchiny_s_ushami_krolika_cvet2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"zhenshchiny_s_ushami_krolika_cvet2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"zhenshchiny_s_ushami_krolika_cvet3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"zhenshchiny_s_ushami_krolika_cvet3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"zhenshchiny_s_ushami_krolika_cvet3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"zhenshchiny_s_ushami_krolika_cvet3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"zhenshchiny_s_ushami_krolika_cvet4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"zhenshchiny_s_ushami_krolika_cvet4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"zhenshchiny_s_ushami_krolika_cvet4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"zhenshchiny_s_ushami_krolika_cvet4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"zhenshchiny_s_ushami_krolika_cvet5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"zhenshchiny_s_ushami_krolika_cvet5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"zhenshchiny_s_ushami_krolika_cvet5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"zhenshchiny_s_ushami_krolika_cvet5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"zhenshchiny_s_ushami_krolika_cvet1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"zhenshchiny_s_ushami_krolika_cvet2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"zhenshchiny_s_ushami_krolika_cvet3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"zhenshchiny_s_ushami_krolika_cvet4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"zhenshchiny_s_ushami_krolika_cvet5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"muzhchiny_s_ushami_krolika", "1F46F-200D-2642-FE0F"},
    {"muzhchiny_s_ushami_krolika_cvet1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"muzhchiny_s_ushami_krolika_cvet1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"muzhchiny_s_ushami_krolika_cvet1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"muzhchiny_s_ushami_krolika_cvet1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"muzhchiny_s_ushami_krolika_cvet2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"muzhchiny_s_ushami_krolika_cvet2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"muzhchiny_s_ushami_krolika_cvet2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"muzhchiny_s_ushami_krolika_cvet2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"muzhchiny_s_ushami_krolika_cvet3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"muzhchiny_s_ushami_krolika_cvet3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"muzhchiny_s_ushami_krolika_cvet3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"muzhchiny_s_ushami_krolika_cvet3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"muzhchiny_s_ushami_krolika_cvet4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"muzhchiny_s_ushami_krolika_cvet4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"muzhchiny_s_ushami_krolika_cvet4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"muzhchiny_s_ushami_krolika_cvet4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"muzhchiny_s_ushami_krolika_cvet5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"muzhchiny_s_ushami_krolika_cvet5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"muzhchiny_s_ushami_krolika_cvet5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"muzhchiny_s_ushami_krolika_cvet5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"muzhchiny_s_ushami_krolika_cvet1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"muzhchiny_s_ushami_krolika_cvet2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"muzhchiny_s_ushami_krolika_cvet3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"muzhchiny_s_ushami_krolika_cvet4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"muzhchiny_s_ushami_krolika_cvet5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_v_fate", "1F470-200D-2640-FE0F"},
    {"zhenshchina_v_fate_cvet1", "1F470-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_v_fate_cvet2", "1F470-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_v_fate_cvet3", "1F470-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_v_fate_cvet4", "1F470-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_v_fate_cvet5", "1F470-1F3FF-200D-2640-FE0F"},
    {"muzhchina_v_fate", "1F470-200D-2642-FE0F"},
    {"muzhchina_v_fate_cvet1", "1F470-1F3FB-200D-2642-FE0F"},
    {"muzhchina_v_fate_cvet2", "1F470-1F3FC-200D-2642-FE0F"},
    {"muzhchina_v_fate_cvet3", "1F470-1F3FD-200D-2642-FE0F"},
    {"muzhchina_v_fate_cvet4", "1F470-1F3FE-200D-2642-FE0F"},
    {"muzhchina_v_fate_cvet5", "1F470-1F3FF-200D-2642-FE0F"},
    {"blondinka", "1F471-200D-2640-FE0F"},
    {"blondinka_cvet1", "1F471-1F3FB-200D-2640-FE0F"},
    {"blondinka_cvet2", "1F471-1F3FC-200D-2640-FE0F"},
    {"blondinka_cvet3", "1F471-1F3FD-200D-2640-FE0F"},
    {"blondinka_cvet4", "1F471-1F3FE-200D-2640-FE0F"},
    {"blondinka_cvet5", "1F471-1F3FF-200D-2640-FE0F"},
    {"blondin", "1F471-200D-2642-FE0F"},
    {"blondin_cvet1", "1F471-1F3FB-200D-2642-FE0F"},
    {"blondin_cvet2", "1F471-1F3FC-200D-2642-FE0F"},
    {"blondin_cvet3", "1F471-1F3FD-200D-2642-FE0F"},
    {"blondin_cvet4", "1F471-1F3FE-200D-2642-FE0F"},
    {"blondin_cvet5", "1F471-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_v_tyurbane", "1F473-200D-2640-FE0F"},
    {"zhenshchina_v_tyurbane_cvet1", "1F473-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_v_tyurbane_cvet2", "1F473-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_v_tyurbane_cvet3", "1F473-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_v_tyurbane_cvet4", "1F473-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_v_tyurbane_cvet5", "1F473-1F3FF-200D-2640-FE0F"},
    {"muzhchina_v_tyurbane", "1F473-200D-2642-FE0F"},
    {"muzhchina_v_tyurbane_cvet1", "1F473-1F3FB-200D-2642-FE0F"},
    {"muzhchina_v_tyurbane_cvet2", "1F473-1F3FC-200D-2642-FE0F"},
    {"muzhchina_v_tyurbane_cvet3", "1F473-1F3FD-200D-2642-FE0F"},
    {"muzhchina_v_tyurbane_cvet4", "1F473-1F3FE-200D-2642-FE0F"},
    {"muzhchina_v_tyurbane_cvet5", "1F473-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_stroitel", "1F477-200D-2640-FE0F"},
    {"zhenshchina_stroitel_cvet1", "1F477-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_stroitel_cvet2", "1F477-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_stroitel_cvet3", "1F477-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_stroitel_cvet4", "1F477-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_stroitel_cvet5", "1F477-1F3FF-200D-2640-FE0F"},
    {"muzhchina_stroitel", "1F477-200D-2642-FE0F"},
    {"muzhchina_stroitel_cvet1", "1F477-1F3FB-200D-2642-FE0F"},
    {"muzhchina_stroitel_cvet2", "1F477-1F3FC-200D-2642-FE0F"},
    {"muzhchina_stroitel_cvet3", "1F477-1F3FD-200D-2642-FE0F"},
    {"muzhchina_stroitel_cvet4", "1F477-1F3FE-200D-2642-FE0F"},
    {"muzhchina_stroitel_cvet5", "1F477-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_tyanet_ruku", "1F481-200D-2640-FE0F"},
    {"zhenshchina_tyanet_ruku_cvet1", "1F481-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_tyanet_ruku_cvet2", "1F481-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_tyanet_ruku_cvet3", "1F481-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_tyanet_ruku_cvet4", "1F481-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_tyanet_ruku_cvet5", "1F481-1F3FF-200D-2640-FE0F"},
    {"muzhchina_tyanet_ruku", "1F481-200D-2642-FE0F"},
    {"muzhchina_tyanet_ruku_cvet1", "1F481-1F3FB-200D-2642-FE0F"},
    {"muzhchina_tyanet_ruku_cvet2", "1F481-1F3FC-200D-2642-FE0F"},
    {"muzhchina_tyanet_ruku_cvet3", "1F481-1F3FD-200D-2642-FE0F"},
    {"muzhchina_tyanet_ruku_cvet4", "1F481-1F3FE-200D-2642-FE0F"},
    {"muzhchina_tyanet_ruku_cvet5", "1F481-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_gvardeec", "1F482-200D-2640-FE0F"},
    {"zhenshchina_gvardeec_cvet1", "1F482-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_gvardeec_cvet2", "1F482-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_gvardeec_cvet3", "1F482-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_gvardeec_cvet4", "1F482-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_gvardeec_cvet5", "1F482-1F3FF-200D-2640-FE0F"},
    {"muzhchina_gvardeec", "1F482-200D-2642-FE0F"},
    {"muzhchina_gvardeec_cvet1", "1F482-1F3FB-200D-2642-FE0F"},
    {"muzhchina_gvardeec_cvet2", "1F482-1F3FC-200D-2642-FE0F"},
    {"muzhchina_gvardeec_cvet3", "1F482-1F3FD-200D-2642-FE0F"},
    {"muzhchina_gvardeec_cvet4", "1F482-1F3FE-200D-2642-FE0F"},
    {"muzhchina_gvardeec_cvet5", "1F482-1F3FF-200D-2642-FE0F"},
    {"zhenshchine_massiruyut_lico", "1F486-200D-2640-FE0F"},
    {"zhenshchine_massiruyut_lico_cvet1", "1F486-1F3FB-200D-2640-FE0F"},
    {"zhenshchine_massiruyut_lico_cvet2", "1F486-1F3FC-200D-2640-FE0F"},
    {"zhenshchine_massiruyut_lico_cvet3", "1F486-1F3FD-200D-2640-FE0F"},
    {"zhenshchine_massiruyut_lico_cvet4", "1F486-1F3FE-200D-2640-FE0F"},
    {"zhenshchine_massiruyut_lico_cvet5", "1F486-1F3FF-200D-2640-FE0F"},
    {"muzhchine_massiruyut_lico", "1F486-200D-2642-FE0F"},
    {"muzhchine_massiruyut_lico_cvet1", "1F486-1F3FB-200D-2642-FE0F"},
    {"muzhchine_massiruyut_lico_cvet2", "1F486-1F3FC-200D-2642-FE0F"},
    {"muzhchine_massiruyut_lico_cvet3", "1F486-1F3FD-200D-2642-FE0F"},
    {"muzhchine_massiruyut_lico_cvet4", "1F486-1F3FE-200D-2642-FE0F"},
    {"muzhchine_massiruyut_lico_cvet5", "1F486-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_u_parikmahera", "1F487-200D-2640-FE0F"},
    {"zhenshchina_u_parikmahera_cvet1", "1F487-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_u_parikmahera_cvet2", "1F487-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_u_parikmahera_cvet3", "1F487-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_u_parikmahera_cvet4", "1F487-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_u_parikmahera_cvet5", "1F487-1F3FF-200D-2640-FE0F"},
    {"muzhchina_u_parikmahera", "1F487-200D-2642-FE0F"},
    {"muzhchina_u_parikmahera_cvet1", "1F487-1F3FB-200D-2642-FE0F"},
    {"muzhchina_u_parikmahera_cvet2", "1F487-1F3FC-200D-2642-FE0F"},
    {"muzhchina_u_parikmahera_cvet3", "1F487-1F3FD-200D-2642-FE0F"},
    {"muzhchina_u_parikmahera_cvet4", "1F487-1F3FE-200D-2642-FE0F"},
    {"muzhchina_u_parikmahera_cvet5", "1F487-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_detektiv", "1F575-FE0F-200D-2640-FE0F"},
    {"zhenshchina_detektiv_cvet1", "1F575-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_detektiv_cvet2", "1F575-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_detektiv_cvet3", "1F575-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_detektiv_cvet4", "1F575-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_detektiv_cvet5", "1F575-1F3FF-200D-2640-FE0F"},
    {"muzhchina_detektiv", "1F575-FE0F-200D-2642-FE0F"},
    {"muzhchina_detektiv_cvet1", "1F575-1F3FB-200D-2642-FE0F"},
    {"muzhchina_detektiv_cvet2", "1F575-1F3FC-200D-2642-FE0F"},
    {"muzhchina_detektiv_cvet3", "1F575-1F3FD-200D-2642-FE0F"},
    {"muzhchina_detektiv_cvet4", "1F575-1F3FE-200D-2642-FE0F"},
    {"muzhchina_detektiv_cvet5", "1F575-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_protiv", "1F645-200D-2640-FE0F"},
    {"zhenshchina_protiv_cvet1", "1F645-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_protiv_cvet2", "1F645-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_protiv_cvet3", "1F645-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_protiv_cvet4", "1F645-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_protiv_cvet5", "1F645-1F3FF-200D-2640-FE0F"},
    {"muzhchina_protiv", "1F645-200D-2642-FE0F"},
    {"muzhchina_protiv_cvet1", "1F645-1F3FB-200D-2642-FE0F"},
    {"muzhchina_protiv_cvet2", "1F645-1F3FC-200D-2642-FE0F"},
    {"muzhchina_protiv_cvet3", "1F645-1F3FD-200D-2642-FE0F"},
    {"muzhchina_protiv_cvet4", "1F645-1F3FE-200D-2642-FE0F"},
    {"muzhchina_protiv_cvet5", "1F645-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_za", "1F646-200D-2640-FE0F"},
    {"zhenshchina_za_cvet1", "1F646-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_za_cvet2", "1F646-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_za_cvet3", "1F646-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_za_cvet4", "1F646-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_za_cvet5", "1F646-1F3FF-200D-2640-FE0F"},
    {"muzhchina_za", "1F646-200D-2642-FE0F"},
    {"muzhchina_za_cvet1", "1F646-1F3FB-200D-2642-FE0F"},
    {"muzhchina_za_cvet2", "1F646-1F3FC-200D-2642-FE0F"},
    {"muzhchina_za_cvet3", "1F646-1F3FD-200D-2642-FE0F"},
    {"muzhchina_za_cvet4", "1F646-1F3FE-200D-2642-FE0F"},
    {"muzhchina_za_cvet5", "1F646-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_klanyaetsya", "1F647-200D-2640-FE0F"},
    {"zhenshchina_klanyaetsya_cvet1", "1F647-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_klanyaetsya_cvet2", "1F647-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_klanyaetsya_cvet3", "1F647-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_klanyaetsya_cvet4", "1F647-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_klanyaetsya_cvet5", "1F647-1F3FF-200D-2640-FE0F"},
    {"muzhchina_klanyaetsya", "1F647-200D-2642-FE0F"},
    {"muzhchina_klanyaetsya_cvet1", "1F647-1F3FB-200D-2642-FE0F"},
    {"muzhchina_klanyaetsya_cvet2", "1F647-1F3FC-200D-2642-FE0F"},
    {"muzhchina_klanyaetsya_cvet3", "1F647-1F3FD-200D-2642-FE0F"},
    {"muzhchina_klanyaetsya_cvet4", "1F647-1F3FE-200D-2642-FE0F"},
    {"muzhchina_klanyaetsya_cvet5", "1F647-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_s_podnyatoy_rukoy", "1F64B-200D-2640-FE0F"},
    {"zhenshchina_s_podnyatoy_rukoy_cvet1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_s_podnyatoy_rukoy_cvet2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_s_podnyatoy_rukoy_cvet3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_s_podnyatoy_rukoy_cvet4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_s_podnyatoy_rukoy_cvet5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"muzhchina_s_podnyatoy_rukoy", "1F64B-200D-2642-FE0F"},
    {"muzhchina_s_podnyatoy_rukoy_cvet1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"muzhchina_s_podnyatoy_rukoy_cvet2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"muzhchina_s_podnyatoy_rukoy_cvet3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"muzhchina_s_podnyatoy_rukoy_cvet4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"muzhchina_s_podnyatoy_rukoy_cvet5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"nahmurivshayasya_zhenshchina", "1F64D-200D-2640-FE0F"},
    {"nahmurivshayasya_zhenshchina_cvet1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"nahmurivshayasya_zhenshchina_cvet2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"nahmurivshayasya_zhenshchina_cvet3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"nahmurivshayasya_zhenshchina_cvet4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"nahmurivshayasya_zhenshchina_cvet5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"nahmurivshiysya_muzhchina", "1F64D-200D-2642-FE0F"},
    {"nahmurivshiysya_muzhchina_cvet1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"nahmurivshiysya_muzhchina_cvet2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"nahmurivshiysya_muzhchina_cvet3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"nahmurivshiysya_muzhchina_cvet4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"nahmurivshiysya_muzhchina_cvet5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"obizhennaya_zhenshchina", "1F64E-200D-2640-FE0F"},
    {"obizhennaya_zhenshchina_cvet1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"obizhennaya_zhenshchina_cvet2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"obizhennaya_zhenshchina_cvet3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"obizhennaya_zhenshchina_cvet4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"obizhennaya_zhenshchina_cvet5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"obizhennyy_muzhchina", "1F64E-200D-2642-FE0F"},
    {"obizhennyy_muzhchina_cvet1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"obizhennyy_muzhchina_cvet2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"obizhennyy_muzhchina_cvet3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"obizhennyy_muzhchina_cvet4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"obizhennyy_muzhchina_cvet5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"grebchiha", "1F6A3-200D-2640-FE0F"},
    {"grebchiha_cvet1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"grebchiha_cvet2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"grebchiha_cvet3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"grebchiha_cvet4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"grebchiha_cvet5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"grebec", "1F6A3-200D-2642-FE0F"},
    {"grebec_cvet1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"grebec_cvet2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"grebec_cvet3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"grebec_cvet4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"grebec_cvet5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_na_velosipede", "1F6B4-200D-2640-FE0F"},
    {"zhenshchina_na_velosipede_cvet1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_na_velosipede_cvet2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_na_velosipede_cvet3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_na_velosipede_cvet4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_na_velosipede_cvet5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"muzhchina_na_velosipede", "1F6B4-200D-2642-FE0F"},
    {"muzhchina_na_velosipede_cvet1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"muzhchina_na_velosipede_cvet2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"muzhchina_na_velosipede_cvet3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"muzhchina_na_velosipede_cvet4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"muzhchina_na_velosipede_cvet5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_na_gornom_velosipede", "1F6B5-200D-2640-FE0F"},
    {"zhenshchina_na_gornom_velosipede_cvet1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_na_gornom_velosipede_cvet2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_na_gornom_velosipede_cvet3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_na_gornom_velosipede_cvet4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_na_gornom_velosipede_cvet5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"muzhchina_na_gornom_velosipede", "1F6B5-200D-2642-FE0F"},
    {"muzhchina_na_gornom_velosipede_cvet1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"muzhchina_na_gornom_velosipede_cvet2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"muzhchina_na_gornom_velosipede_cvet3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"muzhchina_na_gornom_velosipede_cvet4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"muzhchina_na_gornom_velosipede_cvet5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_idet", "1F6B6-200D-2640-FE0F"},
    {"zhenshchina_idet_cvet1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_idet_cvet2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_idet_cvet3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_idet_cvet4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_idet_cvet5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"zhenshchina_idet_napravlennye_vpravo", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_idet_napravlennye_vpravo_cvet1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_idet_napravlennye_vpravo_cvet2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_idet_napravlennye_vpravo_cvet3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_idet_napravlennye_vpravo_cvet4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_idet_napravlennye_vpravo_cvet5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"muzhchina_idet", "1F6B6-200D-2642-FE0F"},
    {"muzhchina_idet_cvet1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"muzhchina_idet_cvet2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"muzhchina_idet_cvet3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"muzhchina_idet_cvet4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"muzhchina_idet_cvet5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"muzhchina_idet_napravlennye_vpravo", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_idet_napravlennye_vpravo_cvet1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_idet_napravlennye_vpravo_cvet2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_idet_napravlennye_vpravo_cvet3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_idet_napravlennye_vpravo_cvet4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_idet_napravlennye_vpravo_cvet5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_poteryala_dar_rechi", "1F926-200D-2640-FE0F"},
    {"zhenshchina_poteryala_dar_rechi_cvet1", "1F926-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_poteryala_dar_rechi_cvet2", "1F926-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_poteryala_dar_rechi_cvet3", "1F926-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_poteryala_dar_rechi_cvet4", "1F926-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_poteryala_dar_rechi_cvet5", "1F926-1F3FF-200D-2640-FE0F"},
    {"muzhchina_poteryal_dar_rechi", "1F926-200D-2642-FE0F"},
    {"muzhchina_poteryal_dar_rechi_cvet1", "1F926-1F3FB-200D-2642-FE0F"},
    {"muzhchina_poteryal_dar_rechi_cvet2", "1F926-1F3FC-200D-2642-FE0F"},
    {"muzhchina_poteryal_dar_rechi_cvet3", "1F926-1F3FD-200D-2642-FE0F"},
    {"muzhchina_poteryal_dar_rechi_cvet4", "1F926-1F3FE-200D-2642-FE0F"},
    {"muzhchina_poteryal_dar_rechi_cvet5", "1F926-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_v_smokinge", "1F935-200D-2640-FE0F"},
    {"zhenshchina_v_smokinge_cvet1", "1F935-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_v_smokinge_cvet2", "1F935-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_v_smokinge_cvet3", "1F935-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_v_smokinge_cvet4", "1F935-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_v_smokinge_cvet5", "1F935-1F3FF-200D-2640-FE0F"},
    {"muzhchina_v_smokinge", "1F935-200D-2642-FE0F"},
    {"muzhchina_v_smokinge_cvet1", "1F935-1F3FB-200D-2642-FE0F"},
    {"muzhchina_v_smokinge_cvet2", "1F935-1F3FC-200D-2642-FE0F"},
    {"muzhchina_v_smokinge_cvet3", "1F935-1F3FD-200D-2642-FE0F"},
    {"muzhchina_v_smokinge_cvet4", "1F935-1F3FE-200D-2642-FE0F"},
    {"muzhchina_v_smokinge_cvet5", "1F935-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_pozhimaet_plechami", "1F937-200D-2640-FE0F"},
    {"zhenshchina_pozhimaet_plechami_cvet1", "1F937-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_pozhimaet_plechami_cvet2", "1F937-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_pozhimaet_plechami_cvet3", "1F937-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_pozhimaet_plechami_cvet4", "1F937-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_pozhimaet_plechami_cvet5", "1F937-1F3FF-200D-2640-FE0F"},
    {"muzhchina_pozhimaet_plechami", "1F937-200D-2642-FE0F"},
    {"muzhchina_pozhimaet_plechami_cvet1", "1F937-1F3FB-200D-2642-FE0F"},
    {"muzhchina_pozhimaet_plechami_cvet2", "1F937-1F3FC-200D-2642-FE0F"},
    {"muzhchina_pozhimaet_plechami_cvet3", "1F937-1F3FD-200D-2642-FE0F"},
    {"muzhchina_pozhimaet_plechami_cvet4", "1F937-1F3FE-200D-2642-FE0F"},
    {"muzhchina_pozhimaet_plechami_cvet5", "1F937-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_akrobat", "1F938-200D-2640-FE0F"},
    {"zhenshchina_akrobat_cvet1", "1F938-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_akrobat_cvet2", "1F938-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_akrobat_cvet3", "1F938-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_akrobat_cvet4", "1F938-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_akrobat_cvet5", "1F938-1F3FF-200D-2640-FE0F"},
    {"akrobat", "1F938-200D-2642-FE0F"},
    {"akrobat_cvet1", "1F938-1F3FB-200D-2642-FE0F"},
    {"akrobat_cvet2", "1F938-1F3FC-200D-2642-FE0F"},
    {"akrobat_cvet3", "1F938-1F3FD-200D-2642-FE0F"},
    {"akrobat_cvet4", "1F938-1F3FE-200D-2642-FE0F"},
    {"akrobat_cvet5", "1F938-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_zhongler", "1F939-200D-2640-FE0F"},
    {"zhenshchina_zhongler_cvet1", "1F939-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_zhongler_cvet2", "1F939-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_zhongler_cvet3", "1F939-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_zhongler_cvet4", "1F939-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_zhongler_cvet5", "1F939-1F3FF-200D-2640-FE0F"},
    {"muzhchina_zhongler", "1F939-200D-2642-FE0F"},
    {"muzhchina_zhongler_cvet1", "1F939-1F3FB-200D-2642-FE0F"},
    {"muzhchina_zhongler_cvet2", "1F939-1F3FC-200D-2642-FE0F"},
    {"muzhchina_zhongler_cvet3", "1F939-1F3FD-200D-2642-FE0F"},
    {"muzhchina_zhongler_cvet4", "1F939-1F3FE-200D-2642-FE0F"},
    {"muzhchina_zhongler_cvet5", "1F939-1F3FF-200D-2642-FE0F"},
    {"zhenshchiny_borcy", "1F93C-200D-2640-FE0F"},
    {"zhenshchiny_borcy_cvet1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"zhenshchiny_borcy_cvet1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"zhenshchiny_borcy_cvet1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"zhenshchiny_borcy_cvet1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"zhenshchiny_borcy_cvet2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"zhenshchiny_borcy_cvet2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"zhenshchiny_borcy_cvet2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"zhenshchiny_borcy_cvet2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"zhenshchiny_borcy_cvet3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"zhenshchiny_borcy_cvet3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"zhenshchiny_borcy_cvet3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"zhenshchiny_borcy_cvet3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"zhenshchiny_borcy_cvet4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"zhenshchiny_borcy_cvet4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"zhenshchiny_borcy_cvet4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"zhenshchiny_borcy_cvet4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"zhenshchiny_borcy_cvet5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"zhenshchiny_borcy_cvet5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"zhenshchiny_borcy_cvet5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"zhenshchiny_borcy_cvet5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"zhenshchiny_borcy_cvet1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"zhenshchiny_borcy_cvet2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"zhenshchiny_borcy_cvet3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"zhenshchiny_borcy_cvet4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"zhenshchiny_borcy_cvet5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"muzhchiny_borcy", "1F93C-200D-2642-FE0F"},
    {"muzhchiny_borcy_cvet1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"muzhchiny_borcy_cvet1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"muzhchiny_borcy_cvet1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"muzhchiny_borcy_cvet1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"muzhchiny_borcy_cvet2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"muzhchiny_borcy_cvet2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"muzhchiny_borcy_cvet2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"muzhchiny_borcy_cvet2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"muzhchiny_borcy_cvet3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"muzhchiny_borcy_cvet3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"muzhchiny_borcy_cvet3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"muzhchiny_borcy_cvet3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"muzhchiny_borcy_cvet4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"muzhchiny_borcy_cvet4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"muzhchiny_borcy_cvet4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"muzhchiny_borcy_cvet4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"muzhchiny_borcy_cvet5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"muzhchiny_borcy_cvet5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"muzhchiny_borcy_cvet5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"muzhchiny_borcy_cvet5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"muzhchiny_borcy_cvet1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"muzhchiny_borcy_cvet2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"muzhchiny_borcy_cvet3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"muzhchiny_borcy_cvet4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"muzhchiny_borcy_cvet5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_igraet_v_vodnoe_polo", "1F93D-200D-2640-FE0F"},
    {"zhenshchina_igraet_v_vodnoe_polo_cvet1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_igraet_v_vodnoe_polo_cvet2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_igraet_v_vodnoe_polo_cvet3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_igraet_v_vodnoe_polo_cvet4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_igraet_v_vodnoe_polo_cvet5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"muzhchina_igraet_v_vodnoe_polo", "1F93D-200D-2642-FE0F"},
    {"muzhchina_igraet_v_vodnoe_polo_cvet1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"muzhchina_igraet_v_vodnoe_polo_cvet2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"muzhchina_igraet_v_vodnoe_polo_cvet3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"muzhchina_igraet_v_vodnoe_polo_cvet4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"muzhchina_igraet_v_vodnoe_polo_cvet5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"gandbolistka", "1F93E-200D-2640-FE0F"},
    {"gandbolistka_cvet1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"gandbolistka_cvet2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"gandbolistka_cvet3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"gandbolistka_cvet4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"gandbolistka_cvet5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"gandbolist", "1F93E-200D-2642-FE0F"},
    {"gandbolist_cvet1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"gandbolist_cvet2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"gandbolist_cvet3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"gandbolist_cvet4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"gandbolist_cvet5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_supergeroy", "1F9B8-200D-2640-FE0F"},
    {"zhenshchina_supergeroy_cvet1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_supergeroy_cvet2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_supergeroy_cvet3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_supergeroy_cvet4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_supergeroy_cvet5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"muzhchina_supergeroy", "1F9B8-200D-2642-FE0F"},
    {"muzhchina_supergeroy_cvet1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"muzhchina_supergeroy_cvet2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"muzhchina_supergeroy_cvet3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"muzhchina_supergeroy_cvet4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"muzhchina_supergeroy_cvet5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_superzlodey", "1F9B9-200D-2640-FE0F"},
    {"zhenshchina_superzlodey_cvet1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_superzlodey_cvet2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_superzlodey_cvet3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_superzlodey_cvet4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_superzlodey_cvet5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"muzhchina_superzlodey", "1F9B9-200D-2642-FE0F"},
    {"muzhchina_superzlodey_cvet1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"muzhchina_superzlodey_cvet2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"muzhchina_superzlodey_cvet3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"muzhchina_superzlodey_cvet4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"muzhchina_superzlodey_cvet5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_stoit", "1F9CD-200D-2640-FE0F"},
    {"zhenshchina_stoit_cvet1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_stoit_cvet2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_stoit_cvet3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_stoit_cvet4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_stoit_cvet5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"muzhchina_stoit", "1F9CD-200D-2642-FE0F"},
    {"muzhchina_stoit_cvet1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"muzhchina_stoit_cvet2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"muzhchina_stoit_cvet3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"muzhchina_stoit_cvet4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"muzhchina_stoit_cvet5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_na_kolenyah", "1F9CE-200D-2640-FE0F"},
    {"zhenshchina_na_kolenyah_cvet1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_na_kolenyah_cvet2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_na_kolenyah_cvet3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_na_kolenyah_cvet4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_na_kolenyah_cvet5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"zhenshchina_na_kolenyah_napravlennye_vpravo", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_na_kolenyah_napravlennye_vpravo_cvet1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_na_kolenyah_napravlennye_vpravo_cvet2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_na_kolenyah_napravlennye_vpravo_cvet3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_na_kolenyah_napravlennye_vpravo_cvet4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"zhenshchina_na_kolenyah_napravlennye_vpravo_cvet5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"muzhchina_na_kolenyah", "1F9CE-200D-2642-FE0F"},
    {"muzhchina_na_kolenyah_cvet1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"muzhchina_na_kolenyah_cvet2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"muzhchina_na_kolenyah_cvet3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"muzhchina_na_kolenyah_cvet4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"muzhchina_na_kolenyah_cvet5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"muzhchina_na_kolenyah_napravlennye_vpravo", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_na_kolenyah_napravlennye_vpravo_cvet1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_na_kolenyah_napravlennye_vpravo_cvet2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_na_kolenyah_napravlennye_vpravo_cvet3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_na_kolenyah_napravlennye_vpravo_cvet4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"muzhchina_na_kolenyah_napravlennye_vpravo_cvet5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"gluhaya_zhenshchina", "1F9CF-200D-2640-FE0F"},
    {"gluhaya_zhenshchina_cvet1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"gluhaya_zhenshchina_cvet2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"gluhaya_zhenshchina_cvet3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"gluhaya_zhenshchina_cvet4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"gluhaya_zhenshchina_cvet5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"gluhoy_muzhchina", "1F9CF-200D-2642-FE0F"},
    {"gluhoy_muzhchina_cvet1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"gluhoy_muzhchina_cvet2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"gluhoy_muzhchina_cvet3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"gluhoy_muzhchina_cvet4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"gluhoy_muzhchina_cvet5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"borodataya_zhenshchina", "1F9D4-200D-2640-FE0F"},
    {"borodataya_zhenshchina_cvet1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"borodataya_zhenshchina_cvet2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"borodataya_zhenshchina_cvet3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"borodataya_zhenshchina_cvet4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"borodataya_zhenshchina_cvet5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"borodatyy_muzhchina", "1F9D4-200D-2642-FE0F"},
    {"borodatyy_muzhchina_cvet1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"borodatyy_muzhchina_cvet2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"borodatyy_muzhchina_cvet3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"borodatyy_muzhchina_cvet4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"borodatyy_muzhchina_cvet5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_v_bane", "1F9D6-200D-2640-FE0F"},
    {"zhenshchina_v_bane_cvet1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_v_bane_cvet2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_v_bane_cvet3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_v_bane_cvet4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_v_bane_cvet5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"muzhchina_v_bane", "1F9D6-200D-2642-FE0F"},
    {"muzhchina_v_bane_cvet1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"muzhchina_v_bane_cvet2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"muzhchina_v_bane_cvet3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"muzhchina_v_bane_cvet4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"muzhchina_v_bane_cvet5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_vzbiraetsya_po_skale", "1F9D7-200D-2640-FE0F"},
    {"zhenshchina_vzbiraetsya_po_skale_cvet1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_vzbiraetsya_po_skale_cvet2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_vzbiraetsya_po_skale_cvet3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_vzbiraetsya_po_skale_cvet4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_vzbiraetsya_po_skale_cvet5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"muzhchina_vzbiraetsya_po_skale", "1F9D7-200D-2642-FE0F"},
    {"muzhchina_vzbiraetsya_po_skale_cvet1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"muzhchina_vzbiraetsya_po_skale_cvet2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"muzhchina_vzbiraetsya_po_skale_cvet3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"muzhchina_vzbiraetsya_po_skale_cvet4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"muzhchina_vzbiraetsya_po_skale_cvet5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_v_poze_lotosa", "1F9D8-200D-2640-FE0F"},
    {"zhenshchina_v_poze_lotosa_cvet1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_v_poze_lotosa_cvet2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_v_poze_lotosa_cvet3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_v_poze_lotosa_cvet4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_v_poze_lotosa_cvet5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"muzhchina_v_poze_lotosa", "1F9D8-200D-2642-FE0F"},
    {"muzhchina_v_poze_lotosa_cvet1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"muzhchina_v_poze_lotosa_cvet2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"muzhchina_v_poze_lotosa_cvet3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"muzhchina_v_poze_lotosa_cvet4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"muzhchina_v_poze_lotosa_cvet5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"volshebnica", "1F9D9-200D-2640-FE0F"},
    {"volshebnica_cvet1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"volshebnica_cvet2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"volshebnica_cvet3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"volshebnica_cvet4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"volshebnica_cvet5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"volshebnik", "1F9D9-200D-2642-FE0F"},
    {"volshebnik_cvet1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"volshebnik_cvet2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"volshebnik_cvet3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"volshebnik_cvet4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"volshebnik_cvet5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_feya", "1F9DA-200D-2640-FE0F"},
    {"zhenshchina_feya_cvet1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_feya_cvet2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_feya_cvet3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_feya_cvet4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_feya_cvet5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"muzhchina_feya", "1F9DA-200D-2642-FE0F"},
    {"muzhchina_feya_cvet1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"muzhchina_feya_cvet2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"muzhchina_feya_cvet3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"muzhchina_feya_cvet4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"muzhchina_feya_cvet5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_vampir", "1F9DB-200D-2640-FE0F"},
    {"zhenshchina_vampir_cvet1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_vampir_cvet2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_vampir_cvet3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_vampir_cvet4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_vampir_cvet5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"muzhchina_vampir", "1F9DB-200D-2642-FE0F"},
    {"muzhchina_vampir_cvet1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"muzhchina_vampir_cvet2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"muzhchina_vampir_cvet3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"muzhchina_vampir_cvet4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"muzhchina_vampir_cvet5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_rusalka", "1F9DC-200D-2640-FE0F"},
    {"zhenshchina_rusalka_cvet1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_rusalka_cvet2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_rusalka_cvet3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_rusalka_cvet4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_rusalka_cvet5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"triton", "1F9DC-200D-2642-FE0F"},
    {"triton_cvet1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"triton_cvet2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"triton_cvet3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"triton_cvet4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"triton_cvet5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_elf", "1F9DD-200D-2640-FE0F"},
    {"zhenshchina_elf_cvet1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"zhenshchina_elf_cvet2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"zhenshchina_elf_cvet3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"zhenshchina_elf_cvet4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"zhenshchina_elf_cvet5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"muzhchina_elf", "1F9DD-200D-2642-FE0F"},
    {"muzhchina_elf_cvet1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"muzhchina_elf_cvet2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"muzhchina_elf_cvet3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"muzhchina_elf_cvet4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"muzhchina_elf_cvet5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"zhenshchina_dzhinn", "1F9DE-200D-2640-FE0F"},
    {"muzhchina_dzhinn", "1F9DE-200D-2642-FE0F"},
    {"zhenshchina_zombi", "1F9DF-200D-2640-FE0F"},
    {"muzhchina_zombi", "1F9DF-200D-2642-FE0F"},
    {"muzhchina_ryzhie_volosy", "1F468-200D-1F9B0"},
    {"muzhchina_ryzhie_volosy_cvet1", "1F468-1F3FB-200D-1F9B0"},
    {"muzhchina_ryzhie_volosy_cvet2", "1F468-1F3FC-200D-1F9B0"},
    {"muzhchina_ryzhie_volosy_cvet3", "1F468-1F3FD-200D-1F9B0"},
    {"muzhchina_ryzhie_volosy_cvet4", "1F468-1F3FE-200D-1F9B0"},
    {"muzhchina_ryzhie_volosy_cvet5", "1F468-1F3FF-200D-1F9B0"},
    {"muzhchina_kudryavye_volosy", "1F468-200D-1F9B1"},
    {"muzhchina_kudryavye_volosy_cvet1", "1F468-1F3FB-200D-1F9B1"},
    {"muzhchina_kudryavye_volosy_cvet2", "1F468-1F3FC-200D-1F9B1"},
    {"muzhchina_kudryavye_volosy_cvet3", "1F468-1F3FD-200D-1F9B1"},
    {"muzhchina_kudryavye_volosy_cvet4", "1F468-1F3FE-200D-1F9B1"},
    {"muzhchina_kudryavye_volosy_cvet5", "1F468-1F3FF-200D-1F9B1"},
    {"muzhchina_bez_volos", "1F468-200D-1F9B2"},
    {"muzhchina_bez_volos_cvet1", "1F468-1F3FB-200D-1F9B2"},
    {"muzhchina_bez_volos_cvet2", "1F468-1F3FC-200D-1F9B2"},
    {"muzhchina_bez_volos_cvet3", "1F468-1F3FD-200D-1F9B2"},
    {"muzhchina_bez_volos_cvet4", "1F468-1F3FE-200D-1F9B2"},
    {"muzhchina_bez_volos_cvet5", "1F468-1F3FF-200D-1F9B2"},
    {"muzhchina_sedye_volosy", "1F468-200D-1F9B3"},
    {"muzhchina_sedye_volosy_cvet1", "1F468-1F3FB-200D-1F9B3"},
    {"muzhchina_sedye_volosy_cvet2", "1F468-1F3FC-200D-1F9B3"},
    {"muzhchina_sedye_volosy_cvet3", "1F468-1F3FD-200D-1F9B3"},
    {"muzhchina_sedye_volosy_cvet4", "1F468-1F3FE-200D-1F9B3"},
    {"muzhchina_sedye_volosy_cvet5", "1F468-1F3FF-200D-1F9B3"},
    {"zhenshchina_ryzhie_volosy", "1F469-200D-1F9B0"},
    {"zhenshchina_ryzhie_volosy_cvet1", "1F469-1F3FB-200D-1F9B0"},
    {"zhenshchina_ryzhie_volosy_cvet2", "1F469-1F3FC-200D-1F9B0"},
    {"zhenshchina_ryzhie_volosy_cvet3", "1F469-1F3FD-200D-1F9B0"},
    {"zhenshchina_ryzhie_volosy_cvet4", "1F469-1F3FE-200D-1F9B0"},
    {"zhenshchina_ryzhie_volosy_cvet5", "1F469-1F3FF-200D-1F9B0"},
    {"zhenshchina_kudryavye_volosy", "1F469-200D-1F9B1"},
    {"zhenshchina_kudryavye_volosy_cvet1", "1F469-1F3FB-200D-1F9B1"},
    {"zhenshchina_kudryavye_volosy_cvet2", "1F469-1F3FC-200D-1F9B1"},
    {"zhenshchina_kudryavye_volosy_cvet3", "1F469-1F3FD-200D-1F9B1"},
    {"zhenshchina_kudryavye_volosy_cvet4", "1F469-1F3FE-200D-1F9B1"},
    {"zhenshchina_kudryavye_volosy_cvet5", "1F469-1F3FF-200D-1F9B1"},
    {"zhenshchina_bez_volos", "1F469-200D-1F9B2"},
    {"zhenshchina_bez_volos_cvet1", "1F469-1F3FB-200D-1F9B2"},
    {"zhenshchina_bez_volos_cvet2", "1F469-1F3FC-200D-1F9B2"},
    {"zhenshchina_bez_volos_cvet3", "1F469-1F3FD-200D-1F9B2"},
    {"zhenshchina_bez_volos_cvet4", "1F469-1F3FE-200D-1F9B2"},
    {"zhenshchina_bez_volos_cvet5", "1F469-1F3FF-200D-1F9B2"},
    {"zhenshchina_sedye_volosy", "1F469-200D-1F9B3"},
    {"zhenshchina_sedye_volosy_cvet1", "1F469-1F3FB-200D-1F9B3"},
    {"zhenshchina_sedye_volosy_cvet2", "1F469-1F3FC-200D-1F9B3"},
    {"zhenshchina_sedye_volosy_cvet3", "1F469-1F3FD-200D-1F9B3"},
    {"zhenshchina_sedye_volosy_cvet4", "1F469-1F3FE-200D-1F9B3"},
    {"zhenshchina_sedye_volosy_cvet5", "1F469-1F3FF-200D-1F9B3"},
    {"vzroslyy_ryzhie_volosy", "1F9D1-200D-1F9B0"},
    {"vzroslyy_ryzhie_volosy_cvet1", "1F9D1-1F3FB-200D-1F9B0"},
    {"vzroslyy_ryzhie_volosy_cvet2", "1F9D1-1F3FC-200D-1F9B0"},
    {"vzroslyy_ryzhie_volosy_cvet3", "1F9D1-1F3FD-200D-1F9B0"},
    {"vzroslyy_ryzhie_volosy_cvet4", "1F9D1-1F3FE-200D-1F9B0"},
    {"vzroslyy_ryzhie_volosy_cvet5", "1F9D1-1F3FF-200D-1F9B0"},
    {"vzroslyy_kudryavye_volosy", "1F9D1-200D-1F9B1"},
    {"vzroslyy_kudryavye_volosy_cvet1", "1F9D1-1F3FB-200D-1F9B1"},
    {"vzroslyy_kudryavye_volosy_cvet2", "1F9D1-1F3FC-200D-1F9B1"},
    {"vzroslyy_kudryavye_volosy_cvet3", "1F9D1-1F3FD-200D-1F9B1"},
    {"vzroslyy_kudryavye_volosy_cvet4", "1F9D1-1F3FE-200D-1F9B1"},
    {"vzroslyy_kudryavye_volosy_cvet5", "1F9D1-1F3FF-200D-1F9B1"},
    {"vzroslyy_bez_volos", "1F9D1-200D-1F9B2"},
    {"vzroslyy_bez_volos_cvet1", "1F9D1-1F3FB-200D-1F9B2"},
    {"vzroslyy_bez_volos_cvet2", "1F9D1-1F3FC-200D-1F9B2"},
    {"vzroslyy_bez_volos_cvet3", "1F9D1-1F3FD-200D-1F9B2"},
    {"vzroslyy_bez_volos_cvet4", "1F9D1-1F3FE-200D-1F9B2"},
    {"vzroslyy_bez_volos_cvet5", "1F9D1-1F3FF-200D-1F9B2"},
    {"vzroslyy_sedye_volosy", "1F9D1-200D-1F9B3"},
    {"vzroslyy_sedye_volosy_cvet1", "1F9D1-1F3FB-200D-1F9B3"},
    {"vzroslyy_sedye_volosy_cvet2", "1F9D1-1F3FC-200D-1F9B3"},
    {"vzroslyy_sedye_volosy_cvet3", "1F9D1-1F3FD-200D-1F9B3"},
    {"vzroslyy_sedye_volosy_cvet4", "1F9D1-1F3FE-200D-1F9B3"},
    {"vzroslyy_sedye_volosy_cvet5", "1F9D1-1F3FF-200D-1F9B3"},
    {"razorvannaya_cep", "26D3-FE0F-200D-1F4A5"},
    {"goryashchee_serdce", "2764-FE0F-200D-1F525"},
    {"zazhivayushchee_serdce", "2764-FE0F-200D-1FA79"},
    {"buryy_grib", "1F344-200D-1F7EB"},
    {"laym", "1F34B-200D-1F7E9"},
    {"flag_transgendernogo_soobshchestva", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"raduzhnyy_flag", "1F3F3-FE0F-200D-1F308"},
    {"piratskiy_flag", "1F3F4-200D-2620-FE0F"},
    {"chernaya_koshka", "1F408-200D-2B1B"},
    {"sluzhebnaya_sobaka", "1F415-200D-1F9BA"},
    {"chernaya_ptica", "1F426-200D-2B1B"},
    {"feniks", "1F426-200D-1F525"},
    {"morda_belogo_medvedya", "1F43B-200D-2744-FE0F"},
    {"znachok_svidetelya", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"vydyhaet", "1F62E-200D-1F4A8"},
    {"oshelomlenie", "1F635-200D-1F4AB"},
    {"lico_v_oblakah", "1F636-200D-1F32B-FE0F"},
    {"otricatelno_kachaet_golovoy", "1F642-200D-2194-FE0F"},
    {"polozhitelno_kachaet_golovoy", "1F642-200D-2195-FE0F"},
    {"artist_baleta", "1F9D1-200D-1FA70"},
    {"artist_baleta_cvet1", "1F9D1-1F3FB-200D-1FA70"},
    {"artist_baleta_cvet2", "1F9D1-1F3FC-200D-1FA70"},
    {"artist_baleta_cvet3", "1F9D1-1F3FD-200D-1FA70"},
    {"artist_baleta_cvet4", "1F9D1-1F3FE-200D-1FA70"},
    {"artist_baleta_cvet5", "1F9D1-1F3FF-200D-1FA70"},
    {"восклицательный_и_вопросительный_знаки", "2049"},
    {"товарный_знак", "2122"},
    {"значок_информации", "2139"},
    {"стрелка_влево_вправо", "2194"},
    {"стрелка_вверх_вниз", "2195"},
    {"стрелка_влево_вверх", "2196"},
    {"стрелка_вправо_вверх", "2197"},
    {"стрелка_вправо_вниз", "2198"},
    {"стрелка_влево_вниз", "2199"},
    {"клавиатура", "2328"},
    {"солнце", "2600"},
    {"облако", "2601"},
    {"зонт", "2602"},
    {"снеговик_под_снегом", "2603"},
    {"комета", "2604"},
    {"галочка_в_квадрате", "2611"},
    {"зонт_под_дождем", "2614"},
    {"горячий_напиток", "2615"},
    {"трилистник", "2618"},
    {"череп_и_кости", "2620"},
    {"радиация", "2622"},
    {"биологическая_угроза", "2623"},
    {"православный_крест", "2626"},
    {"колесо_дхармы", "2638"},
    {"грустит", "2639"},
    {"женский_символ", "2640"},
    {"мужской_символ", "2642"},
    {"знак_зодиака_овен", "2648"},
    {"знак_зодиака_телец", "2649"},
    {"знак_зодиака_стрелец", "2650"},
    {"знак_зодиака_козерог", "2651"},
    {"знак_зодиака_водолей", "2652"},
    {"знак_зодиака_рыбы", "2653"},
    {"пики", "2660"},
    {"трефы", "2663"},
    {"червы", "2665"},
    {"бубны", "2666"},
    {"горячие_источники", "2668"},
    {"кирка_и_молот", "2692"},
    {"якорь", "2693"},
    {"скрещенные_мечи", "2694"},
    {"медицинский_символ", "2695"},
    {"весы", "2696"},
    {"перегонный_сосуд", "2697"},
    {"шестеренка", "2699"},
    {"ножницы", "2702"},
    {"белая_галочка", "2705"},
    {"самолет", "2708"},
    {"конверт", "2709"},
    {"перьевой_наконечник", "2712"},
    {"галочка", "2714"},
    {"знак_умножения", "2716"},
    {"звезда_давида", "2721"},
    {"блестки", "2728"},
    {"восьмиконечная_звездочка", "2733"},
    {"восьмиконечная_звезда", "2734"},
    {"снежинка", "2744"},
    {"искра", "2747"},
    {"красный_вопросительный_знак", "2753"},
    {"белый_вопросительный_знак", "2754"},
    {"белый_восклицательный_знак", "2755"},
    {"красный_восклицательный_знак", "2757"},
    {"восклицательный_знак_в_виде_сердца", "2763"},
    {"алое_сердце", "2764"},
    {"знак_сложения", "2795"},
    {"знак_вычитания", "2796"},
    {"знак_деления", "2797"},
    {"стрелка_вправо_с_поворотом_вверх", "2934"},
    {"стрелка_вправо_с_поворотом_вниз", "2935"},
    {"волнистая_линия", "3030"},
    {"поздравление", "3297"},
    {"секрет", "3299"},
    {"значок_защиты_авторского_права", "00A9"},
    {"зарегистрированный_товарный_знак", "00AE"},
    {"два_восклицательных_знака", "203C"},
    {"стрелка_вправо_с_поворотом_влево", "21A9"},
    {"стрелка_влево_с_поворотом_вправо", "21AA"},
    {"наручные_часы", "231A"},
    {"песочные_часы", "231B"},
    {"выброс", "23CF"},
    {"перемотка_вперед", "23E9"},
    {"перемотка_назад", "23EA"},
    {"два_треугольника_острием_вверх", "23EB"},
    {"два_треугольника_острием_вниз", "23EC"},
    {"следующий_трек", "23ED"},
    {"предыдущий_трек", "23EE"},
    {"воспроизведение_пауза", "23EF"},
    {"будильник", "23F0"},
    {"секундомер", "23F1"},
    {"таймер", "23F2"},
    {"время_пошло", "23F3"},
    {"пауза", "23F8"},
    {"стоп", "23F9"},
    {"запись", "23FA"},
    {"буква_м_в_кружке", "24C2"},
    {"черный_квадратик", "25AA"},
    {"белый_квадратик", "25AB"},
    {"воспроизведение", "25B6"},
    {"реверс", "25C0"},
    {"белый_квадрат", "25FB"},
    {"черный_квадрат", "25FC"},
    {"небольшой_белый_квадрат", "25FD"},
    {"небольшой_черный_квадрат", "25FE"},
    {"телефон", "260E"},
    {"указательный_палец_вверх", "261D"},
    {"указательный_палец_вверх_цвет1", "261D-1F3FB"},
    {"указательный_палец_вверх_цвет2", "261D-1F3FC"},
    {"указательный_палец_вверх_цвет3", "261D-1F3FD"},
    {"указательный_палец_вверх_цвет4", "261D-1F3FE"},
    {"указательный_палец_вверх_цвет5", "261D-1F3FF"},
    {"звезда_и_полумесяц", "262A"},
    {"пацифик", "262E"},
    {"инь_ян", "262F"},
    {"улыбается", "263A"},
    {"знак_зодиака_близнецы", "264A"},
    {"знак_зодиака_рак", "264B"},
    {"знак_зодиака_лев", "264C"},
    {"знак_зодиака_дева", "264D"},
    {"знак_зодиака_весы", "264E"},
    {"знак_зодиака_скорпион", "264F"},
    {"пешка", "265F"},
    {"утилизация", "267B"},
    {"бесконечность", "267E"},
    {"значок_для_инвалидов", "267F"},
    {"символ_атома", "269B"},
    {"геральдическая_лилия", "269C"},
    {"значок_предупреждения", "26A0"},
    {"высокое_напряжение", "26A1"},
    {"трансгендер", "26A7"},
    {"белый_круг", "26AA"},
    {"черный_круг", "26AB"},
    {"гроб", "26B0"},
    {"урна_с_прахом", "26B1"},
    {"футбол", "26BD"},
    {"бейсбол", "26BE"},
    {"снеговик", "26C4"},
    {"солнце_за_облаками", "26C5"},
    {"гроза", "26C8"},
    {"знак_зодиака_змееносец", "26CE"},
    {"кирка", "26CF"},
    {"каска_с_белым_крестом", "26D1"},
    {"цепь", "26D3"},
    {"въезд_запрещен", "26D4"},
    {"синтоистский_храм", "26E9"},
    {"церковь", "26EA"},
    {"гора", "26F0"},
    {"пляжный_зонт", "26F1"},
    {"фонтан", "26F2"},
    {"гольф", "26F3"},
    {"паром", "26F4"},
    {"парусник", "26F5"},
    {"горные_лыжи", "26F7"},
    {"коньки", "26F8"},
    {"баскетболист", "26F9"},
    {"баскетболист_цвет1", "26F9-1F3FB"},
    {"баскетболист_цвет2", "26F9-1F3FC"},
    {"баскетболист_цвет3", "26F9-1F3FD"},
    {"баскетболист_цвет4", "26F9-1F3FE"},
    {"баскетболист_цвет5", "26F9-1F3FF"},
    {"палатка", "26FA"},
    {"заправка", "26FD"},
    {"поднятый_кулак", "270A"},
    {"поднятый_кулак_цвет1", "270A-1F3FB"},
    {"поднятый_кулак_цвет2", "270A-1F3FC"},
    {"поднятый_кулак_цвет3", "270A-1F3FD"},
    {"поднятый_кулак_цвет4", "270A-1F3FE"},
    {"поднятый_кулак_цвет5", "270A-1F3FF"},
    {"поднятая_ладонь", "270B"},
    {"поднятая_ладонь_цвет1", "270B-1F3FB"},
    {"поднятая_ладонь_цвет2", "270B-1F3FC"},
    {"поднятая_ладонь_цвет3", "270B-1F3FD"},
    {"поднятая_ладонь_цвет4", "270B-1F3FE"},
    {"поднятая_ладонь_цвет5", "270B-1F3FF"},
    {"жест_v", "270C"},
    {"жест_v_цвет1", "270C-1F3FB"},
    {"жест_v_цвет2", "270C-1F3FC"},
    {"жест_v_цвет3", "270C-1F3FD"},
    {"жест_v_цвет4", "270C-1F3FE"},
    {"жест_v_цвет5", "270C-1F3FF"},
    {"пишущая_рука", "270D"},
    {"пишущая_рука_цвет1", "270D-1F3FB"},
    {"пишущая_рука_цвет2", "270D-1F3FC"},
    {"пишущая_рука_цвет3", "270D-1F3FD"},
    {"пишущая_рука_цвет4", "270D-1F3FE"},
    {"пишущая_рука_цвет5", "270D-1F3FF"},
    {"карандаш", "270F"},
    {"латинский_крест", "271D"},
    {"крестик", "274C"},
    {"кнопка_крестик", "274E"},
    {"стрелка_вправо", "27A1"},
    {"петля", "27B0"},
    {"двойная_петля", "27BF"},
    {"стрелка_влево", "2B05"},
    {"стрелка_вверх", "2B06"},
    {"стрелка_вниз", "2B07"},
    {"большой_черный_квадрат", "2B1B"},
    {"большой_белый_квадрат", "2B1C"},
    {"желтая_звезда", "2B50"},
    {"красное_кольцо", "2B55"},
    {"вступает_другой_голос", "303D"},
    {"маджонг", "1F004"},
    {"джокер", "1F0CF"},
    {"ii_группа_крови", "1F170"},
    {"iii_группа_крови", "1F171"},
    {"i_группа_крови", "1F17E"},
    {"парковка", "1F17F"},
    {"iv_группа_крови", "1F18E"},
    {"кнопка_очистить", "1F191"},
    {"значок_круто", "1F192"},
    {"значок_бесплатно", "1F193"},
    {"значок_идентификация", "1F194"},
    {"значок_новинка", "1F195"},
    {"значок_не_годится", "1F196"},
    {"значок_ок", "1F197"},
    {"значок_sos", "1F198"},
    {"значок_обновление", "1F199"},
    {"значок_против", "1F19A"},
    {"региональный_показатель_a", "1F1E6"},
    {"региональный_показатель_b", "1F1E7"},
    {"региональный_показатель_c", "1F1E8"},
    {"региональный_показатель_d", "1F1E9"},
    {"региональный_показатель_e", "1F1EA"},
    {"региональный_показатель_f", "1F1EB"},
    {"региональный_показатель_g", "1F1EC"},
    {"региональный_показатель_h", "1F1ED"},
    {"региональный_показатель_i", "1F1EE"},
    {"региональный_показатель_j", "1F1EF"},
    {"региональный_показатель_k", "1F1F0"},
    {"региональный_показатель_l", "1F1F1"},
    {"региональный_показатель_m", "1F1F2"},
    {"региональный_показатель_n", "1F1F3"},
    {"региональный_показатель_o", "1F1F4"},
    {"региональный_показатель_p", "1F1F5"},
    {"региональный_показатель_q", "1F1F6"},
    {"региональный_показатель_r", "1F1F7"},
    {"региональный_показатель_s", "1F1F8"},
    {"региональный_показатель_t", "1F1F9"},
    {"региональный_показатель_u", "1F1FA"},
    {"региональный_показатель_v", "1F1FB"},
    {"региональный_показатель_w", "1F1FC"},
    {"региональный_показатель_x", "1F1FD"},
    {"региональный_показатель_y", "1F1FE"},
    {"региональный_показатель_z", "1F1FF"},
    {"значок_здесь", "1F201"},
    {"цена_услуги", "1F202"},
    {"бесплатно", "1F21A"},
    {"забронировано", "1F22F"},
    {"запрет", "1F232"},
    {"есть_места", "1F233"},
    {"проходной_балл", "1F234"},
    {"нет_мест", "1F235"},
    {"платно", "1F236"},
    {"месяц", "1F237"},
    {"заявка", "1F238"},
    {"скидка", "1F239"},
    {"открыто", "1F23A"},
    {"удачная_сделка", "1F250"},
    {"допустимость", "1F251"},
    {"циклон", "1F300"},
    {"туман", "1F301"},
    {"закрытый_зонт", "1F302"},
    {"звездная_ночь", "1F303"},
    {"восход_солнца_над_горами", "1F304"},
    {"восход", "1F305"},
    {"закат_в_городе", "1F306"},
    {"закат", "1F307"},
    {"радуга", "1F308"},
    {"мост_ночью", "1F309"},
    {"волна", "1F30A"},
    {"вулкан", "1F30B"},
    {"млечный_путь", "1F30C"},
    {"европа_и_африка_на_земном_шаре", "1F30D"},
    {"америка_на_земном_шаре", "1F30E"},
    {"азия_и_австралия_на_земном_шаре", "1F30F"},
    {"меридианы", "1F310"},
    {"новолуние", "1F311"},
    {"молодая_луна", "1F312"},
    {"луна_в_первой_четверти", "1F313"},
    {"прибывающая_луна", "1F314"},
    {"полнолуние", "1F315"},
    {"убывающая_луна", "1F316"},
    {"луна_в_последней_четверти", "1F317"},
    {"старая_луна", "1F318"},
    {"полумесяц", "1F319"},
    {"новая_луна_с_лицом", "1F31A"},
    {"луна_в_первой_четверти_с_лицом", "1F31B"},
    {"луна_в_последней_четверти_с_лицом", "1F31C"},
    {"полная_луна_с_лицом", "1F31D"},
    {"солнце_с_лицом", "1F31E"},
    {"сияющая_звезда", "1F31F"},
    {"падающая_звезда", "1F320"},
    {"термометр", "1F321"},
    {"солнце_за_облачком", "1F324"},
    {"солнце_за_облаком", "1F325"},
    {"слепой_дождь", "1F326"},
    {"дождь", "1F327"},
    {"снег", "1F328"},
    {"молния", "1F329"},
    {"торнадо", "1F32A"},
    {"густой_туман", "1F32B"},
    {"дует_ветер", "1F32C"},
    {"хот_дог", "1F32D"},
    {"тако", "1F32E"},
    {"буррито", "1F32F"},
    {"каштан", "1F330"},
    {"росток", "1F331"},
    {"елка", "1F332"},
    {"дерево", "1F333"},
    {"пальма", "1F334"},
    {"кактус", "1F335"},
    {"острый_перец", "1F336"},
    {"тюльпан", "1F337"},
    {"цветок_вишни", "1F338"},
    {"роза", "1F339"},
    {"гибискус", "1F33A"},
    {"подсолнух", "1F33B"},
    {"цветок", "1F33C"},
    {"кукурузный_початок", "1F33D"},
    {"колос", "1F33E"},
    {"веточка", "1F33F"},
    {"четырехлистный_клевер", "1F340"},
    {"кленовый_лист", "1F341"},
    {"падающие_листья", "1F342"},
    {"листья_на_ветру", "1F343"},
    {"гриб", "1F344"},
    {"помидор", "1F345"},
    {"баклажан", "1F346"},
    {"виноград", "1F347"},
    {"дыня", "1F348"},
    {"арбуз", "1F349"},
    {"мандарин", "1F34A"},
    {"лимон", "1F34B"},
    {"банан", "1F34C"},
    {"ананас", "1F34D"},
    {"красное_яблоко", "1F34E"},
    {"зеленое_яблоко", "1F34F"},
    {"груша", "1F350"},
    {"персик", "1F351"},
    {"вишня", "1F352"},
    {"клубника", "1F353"},
    {"гамбургер", "1F354"},
    {"пицца", "1F355"},
    {"мясо_на_косточке", "1F356"},
    {"куриная_ножка", "1F357"},
    {"рисовый_крекер", "1F358"},
    {"онигири", "1F359"},
    {"рис", "1F35A"},
    {"рис_с_карри", "1F35B"},
    {"лапша", "1F35C"},
    {"спагетти", "1F35D"},
    {"хлеб", "1F35E"},
    {"картофель_фри", "1F35F"},
    {"печеный_сладкий_картофель", "1F360"},
    {"данго", "1F361"},
    {"одэн", "1F362"},
    {"суши", "1F363"},
    {"жареная_креветка", "1F364"},
    {"камабоко", "1F365"},
    {"мороженое_в_стаканчике", "1F366"},
    {"мороженое_в_креманке", "1F367"},
    {"мороженое", "1F368"},
    {"пончик", "1F369"},
    {"печенье", "1F36A"},
    {"шоколад", "1F36B"},
    {"конфета", "1F36C"},
    {"леденец", "1F36D"},
    {"пудинг", "1F36E"},
    {"мед", "1F36F"},
    {"кусочек_торта", "1F370"},
    {"бэнто", "1F371"},
    {"кастрюля_с_горячей_едой", "1F372"},
    {"яичница_на_сковороде", "1F373"},
    {"вилка_и_нож", "1F374"},
    {"чашка_без_ручки", "1F375"},
    {"бутылка_саке", "1F376"},
    {"бокал_вина", "1F377"},
    {"коктейль", "1F378"},
    {"тропический_коктейль", "1F379"},
    {"кружка_пива", "1F37A"},
    {"чокающиеся_кружки", "1F37B"},
    {"бутылочка_с_соской", "1F37C"},
    {"тарелка_с_приборами", "1F37D"},
    {"шампанское", "1F37E"},
    {"попкорн", "1F37F"},
    {"бантик", "1F380"},
    {"подарок", "1F381"},
    {"торт_ко_дню_рождения", "1F382"},
    {"тыква_фонарь", "1F383"},
    {"новогодняя_елка", "1F384"},
    {"санта_клаус", "1F385"},
    {"санта_клаус_цвет1", "1F385-1F3FB"},
    {"санта_клаус_цвет2", "1F385-1F3FC"},
    {"санта_клаус_цвет3", "1F385-1F3FD"},
    {"санта_клаус_цвет4", "1F385-1F3FE"},
    {"санта_клаус_цвет5", "1F385-1F3FF"},
    {"фейерверк", "1F386"},
    {"бенгальский_огонь", "1F387"},
    {"воздушный_шарик", "1F388"},
    {"хлопушка", "1F389"},
    {"конфетти", "1F38A"},
    {"бамбук_с_бумажными_лентами", "1F38B"},
    {"скрещенные_флаги", "1F38C"},
    {"украшенный_бамбук", "1F38D"},
    {"японские_куклы", "1F38E"},
    {"вымпелы_в_виде_карпов", "1F38F"},
    {"музыка_ветра", "1F390"},
    {"праздник_луны", "1F391"},
    {"ранец", "1F392"},
    {"шапочка_выпускника", "1F393"},
    {"военная_медаль", "1F396"},
    {"памятная_лента", "1F397"},
    {"студийный_микрофон", "1F399"},
    {"слайдер", "1F39A"},
    {"регуляторы", "1F39B"},
    {"кинопленка", "1F39E"},
    {"билеты", "1F39F"},
    {"лошадь_на_карусели", "1F3A0"},
    {"колесо_обозрения", "1F3A1"},
    {"американские_горки", "1F3A2"},
    {"удочка_с_рыбой", "1F3A3"},
    {"микрофон", "1F3A4"},
    {"кинокамера", "1F3A5"},
    {"видеозапись", "1F3A6"},
    {"наушники", "1F3A7"},
    {"палитра_с_красками", "1F3A8"},
    {"цилиндр", "1F3A9"},
    {"цирковой_шатер", "1F3AA"},
    {"билет", "1F3AB"},
    {"хлопушка_нумератор", "1F3AC"},
    {"маски", "1F3AD"},
    {"видеоигры", "1F3AE"},
    {"мишень", "1F3AF"},
    {"игровой_автомат", "1F3B0"},
    {"бильярд", "1F3B1"},
    {"игральная_кость", "1F3B2"},
    {"боулинг", "1F3B3"},
    {"цветочная_карта", "1F3B4"},
    {"нота", "1F3B5"},
    {"ноты", "1F3B6"},
    {"саксофон", "1F3B7"},
    {"гитара", "1F3B8"},
    {"клавишные", "1F3B9"},
    {"труба", "1F3BA"},
    {"скрипка", "1F3BB"},
    {"нотный_стан", "1F3BC"},
    {"футболка_для_бега", "1F3BD"},
    {"теннис", "1F3BE"},
    {"лыжи", "1F3BF"},
    {"баскетбол", "1F3C0"},
    {"клетчатый_флаг", "1F3C1"},
    {"сноубордист", "1F3C2"},
    {"сноубордист_цвет1", "1F3C2-1F3FB"},
    {"сноубордист_цвет2", "1F3C2-1F3FC"},
    {"сноубордист_цвет3", "1F3C2-1F3FD"},
    {"сноубордист_цвет4", "1F3C2-1F3FE"},
    {"сноубордист_цвет5", "1F3C2-1F3FF"},
    {"человек_бежит", "1F3C3"},
    {"человек_бежит_цвет1", "1F3C3-1F3FB"},
    {"человек_бежит_цвет2", "1F3C3-1F3FC"},
    {"человек_бежит_цвет3", "1F3C3-1F3FD"},
    {"человек_бежит_цвет4", "1F3C3-1F3FE"},
    {"человек_бежит_цвет5", "1F3C3-1F3FF"},
    {"серфинг", "1F3C4"},
    {"серфинг_цвет1", "1F3C4-1F3FB"},
    {"серфинг_цвет2", "1F3C4-1F3FC"},
    {"серфинг_цвет3", "1F3C4-1F3FD"},
    {"серфинг_цвет4", "1F3C4-1F3FE"},
    {"серфинг_цвет5", "1F3C4-1F3FF"},
    {"спортивная_медаль", "1F3C5"},
    {"кубок", "1F3C6"},
    {"скачки", "1F3C7"},
    {"скачки_цвет1", "1F3C7-1F3FB"},
    {"скачки_цвет2", "1F3C7-1F3FC"},
    {"скачки_цвет3", "1F3C7-1F3FD"},
    {"скачки_цвет4", "1F3C7-1F3FE"},
    {"скачки_цвет5", "1F3C7-1F3FF"},
    {"американский_футбол", "1F3C8"},
    {"регби", "1F3C9"},
    {"плавание", "1F3CA"},
    {"плавание_цвет1", "1F3CA-1F3FB"},
    {"плавание_цвет2", "1F3CA-1F3FC"},
    {"плавание_цвет3", "1F3CA-1F3FD"},
    {"плавание_цвет4", "1F3CA-1F3FE"},
    {"плавание_цвет5", "1F3CA-1F3FF"},
    {"тяжелоатлет", "1F3CB"},
    {"тяжелоатлет_цвет1", "1F3CB-1F3FB"},
    {"тяжелоатлет_цвет2", "1F3CB-1F3FC"},
    {"тяжелоатлет_цвет3", "1F3CB-1F3FD"},
    {"тяжелоатлет_цвет4", "1F3CB-1F3FE"},
    {"тяжелоатлет_цвет5", "1F3CB-1F3FF"},
    {"гольфист", "1F3CC"},
    {"гольфист_цвет1", "1F3CC-1F3FB"},
    {"гольфист_цвет2", "1F3CC-1F3FC"},
    {"гольфист_цвет3", "1F3CC-1F3FD"},
    {"гольфист_цвет4", "1F3CC-1F3FE"},
    {"гольфист_цвет5", "1F3CC-1F3FF"},
    {"мотоцикл", "1F3CD"},
    {"гоночный_автомобиль", "1F3CE"},
    {"крикет", "1F3CF"},
    {"волейбол", "1F3D0"},
    {"хоккей_на_траве", "1F3D1"},
    {"хоккей", "1F3D2"},
    {"настольный_теннис", "1F3D3"},
    {"гора_со_снежной_шапкой", "1F3D4"},
    {"кемпинг", "1F3D5"},
    {"пляж", "1F3D6"},
    {"стройка", "1F3D7"},
    {"дома", "1F3D8"},
    {"вид_на_город", "1F3D9"},
    {"заброшенный_дом", "1F3DA"},
    {"античное_здание", "1F3DB"},
    {"пустыня", "1F3DC"},
    {"остров", "1F3DD"},
    {"национальный_парк", "1F3DE"},
    {"стадион", "1F3DF"},
    {"дом", "1F3E0"},
    {"дом_с_садом", "1F3E1"},
    {"офисное_здание", "1F3E2"},
    {"японская_почта", "1F3E3"},
    {"почта", "1F3E4"},
    {"больница", "1F3E5"},
    {"банк", "1F3E6"},
    {"значок_банкомата", "1F3E7"},
    {"гостиница", "1F3E8"},
    {"гостиница_для_свиданий", "1F3E9"},
    {"круглосуточный_магазин", "1F3EA"},
    {"школа", "1F3EB"},
    {"универмаг", "1F3EC"},
    {"завод", "1F3ED"},
    {"красный_бумажный_фонарик", "1F3EE"},
    {"японский_замок", "1F3EF"},
    {"замок", "1F3F0"},
    {"белый_флаг", "1F3F3"},
    {"черный_флаг", "1F3F4"},
    {"розетка", "1F3F5"},
    {"бирка", "1F3F7"},
    {"бадминтон", "1F3F8"},
    {"лук_со_стрелой", "1F3F9"},
    {"амфора", "1F3FA"},
    {"очень_светлый_тон_кожи", "1F3FB"},
    {"светлый_тон_кожи", "1F3FC"},
    {"средний_тон_кожи", "1F3FD"},
    {"темный_тон_кожи", "1F3FE"},
    {"очень_темный_тон_кожи", "1F3FF"},
    {"крыса", "1F400"},
    {"мышь", "1F401"},
    {"бык", "1F402"},
    {"буйвол", "1F403"},
    {"корова", "1F404"},
    {"тигр", "1F405"},
    {"леопард", "1F406"},
    {"кролик", "1F407"},
    {"кошка", "1F408"},
    {"дракон", "1F409"},
    {"крокодил", "1F40A"},
    {"кит", "1F40B"},
    {"улитка", "1F40C"},
    {"змея", "1F40D"},
    {"лошадь", "1F40E"},
    {"баран", "1F40F"},
    {"коза", "1F410"},
    {"овца", "1F411"},
    {"обезьяна", "1F412"},
    {"петух", "1F413"},
    {"курица", "1F414"},
    {"собака", "1F415"},
    {"свинья", "1F416"},
    {"кабан", "1F417"},
    {"слон", "1F418"},
    {"осьминог", "1F419"},
    {"раковина", "1F41A"},
    {"гусеница", "1F41B"},
    {"муравей", "1F41C"},
    {"пчела", "1F41D"},
    {"божья_коровка", "1F41E"},
    {"рыба", "1F41F"},
    {"тропическая_рыба", "1F420"},
    {"рыба_еж", "1F421"},
    {"черепаха", "1F422"},
    {"цыпленок_в_яйце", "1F423"},
    {"цыпленок_боком", "1F424"},
    {"цыпленок", "1F425"},
    {"птица", "1F426"},
    {"пингвин", "1F427"},
    {"морда_коалы", "1F428"},
    {"пудель", "1F429"},
    {"одногорбый_верблюд", "1F42A"},
    {"двугорбый_верблюд", "1F42B"},
    {"дельфин", "1F42C"},
    {"морда_мыши", "1F42D"},
    {"морда_коровы", "1F42E"},
    {"морда_тигра", "1F42F"},
    {"морда_кролика", "1F430"},
    {"морда_кошки", "1F431"},
    {"голова_дракона", "1F432"},
    {"кит_с_фонтанчиком", "1F433"},
    {"морда_лошади", "1F434"},
    {"морда_обезьяны", "1F435"},
    {"морда_собаки", "1F436"},
    {"морда_свиньи", "1F437"},
    {"голова_лягушки", "1F438"},
    {"морда_хомяка", "1F439"},
    {"морда_волка", "1F43A"},
    {"морда_медведя", "1F43B"},
    {"морда_панды", "1F43C"},
    {"свиной_пятачок", "1F43D"},
    {"следы_животного", "1F43E"},
    {"бурундук", "1F43F"},
    {"глаза", "1F440"},
    {"глаз", "1F441"},
    {"ухо", "1F442"},
    {"ухо_цвет1", "1F442-1F3FB"},
    {"ухо_цвет2", "1F442-1F3FC"},
    {"ухо_цвет3", "1F442-1F3FD"},
    {"ухо_цвет4", "1F442-1F3FE"},
    {"ухо_цвет5", "1F442-1F3FF"},
    {"нос", "1F443"},
    {"нос_цвет1", "1F443-1F3FB"},
    {"нос_цвет2", "1F443-1F3FC"},
    {"нос_цвет3", "1F443-1F3FD"},
    {"нос_цвет4", "1F443-1F3FE"},
    {"нос_цвет5", "1F443-1F3FF"},
    {"рот", "1F444"},
    {"язык", "1F445"},
    {"указывающий_вверх_палец", "1F446"},
    {"указывающий_вверх_палец_цвет1", "1F446-1F3FB"},
    {"указывающий_вверх_палец_цвет2", "1F446-1F3FC"},
    {"указывающий_вверх_палец_цвет3", "1F446-1F3FD"},
    {"указывающий_вверх_палец_цвет4", "1F446-1F3FE"},
    {"указывающий_вверх_палец_цвет5", "1F446-1F3FF"},
    {"указывающий_вниз_палец", "1F447"},
    {"указывающий_вниз_палец_цвет1", "1F447-1F3FB"},
    {"указывающий_вниз_палец_цвет2", "1F447-1F3FC"},
    {"указывающий_вниз_палец_цвет3", "1F447-1F3FD"},
    {"указывающий_вниз_палец_цвет4", "1F447-1F3FE"},
    {"указывающий_вниз_палец_цвет5", "1F447-1F3FF"},
    {"указывающий_налево_палец", "1F448"},
    {"указывающий_налево_палец_цвет1", "1F448-1F3FB"},
    {"указывающий_налево_палец_цвет2", "1F448-1F3FC"},
    {"указывающий_налево_палец_цвет3", "1F448-1F3FD"},
    {"указывающий_налево_палец_цвет4", "1F448-1F3FE"},
    {"указывающий_налево_палец_цвет5", "1F448-1F3FF"},
    {"указывающий_направо_палец", "1F449"},
    {"указывающий_направо_палец_цвет1", "1F449-1F3FB"},
    {"указывающий_направо_палец_цвет2", "1F449-1F3FC"},
    {"указывающий_направо_палец_цвет3", "1F449-1F3FD"},
    {"указывающий_направо_палец_цвет4", "1F449-1F3FE"},
    {"указывающий_направо_палец_цвет5", "1F449-1F3FF"},
    {"кулак", "1F44A"},
    {"кулак_цвет1", "1F44A-1F3FB"},
    {"кулак_цвет2", "1F44A-1F3FC"},
    {"кулак_цвет3", "1F44A-1F3FD"},
    {"кулак_цвет4", "1F44A-1F3FE"},
    {"кулак_цвет5", "1F44A-1F3FF"},
    {"машет_рукой", "1F44B"},
    {"машет_рукой_цвет1", "1F44B-1F3FB"},
    {"машет_рукой_цвет2", "1F44B-1F3FC"},
    {"машет_рукой_цвет3", "1F44B-1F3FD"},
    {"машет_рукой_цвет4", "1F44B-1F3FE"},
    {"машет_рукой_цвет5", "1F44B-1F3FF"},
    {"жест_все_хорошо", "1F44C"},
    {"жест_все_хорошо_цвет1", "1F44C-1F3FB"},
    {"жест_все_хорошо_цвет2", "1F44C-1F3FC"},
    {"жест_все_хорошо_цвет3", "1F44C-1F3FD"},
    {"жест_все_хорошо_цвет4", "1F44C-1F3FE"},
    {"жест_все_хорошо_цвет5", "1F44C-1F3FF"},
    {"большой_палец_вверх", "1F44D"},
    {"большой_палец_вверх_цвет1", "1F44D-1F3FB"},
    {"большой_палец_вверх_цвет2", "1F44D-1F3FC"},
    {"большой_палец_вверх_цвет3", "1F44D-1F3FD"},
    {"большой_палец_вверх_цвет4", "1F44D-1F3FE"},
    {"большой_палец_вверх_цвет5", "1F44D-1F3FF"},
    {"большой_палец_вниз", "1F44E"},
    {"большой_палец_вниз_цвет1", "1F44E-1F3FB"},
    {"большой_палец_вниз_цвет2", "1F44E-1F3FC"},
    {"большой_палец_вниз_цвет3", "1F44E-1F3FD"},
    {"большой_палец_вниз_цвет4", "1F44E-1F3FE"},
    {"большой_палец_вниз_цвет5", "1F44E-1F3FF"},
    {"аплодисменты", "1F44F"},
    {"аплодисменты_цвет1", "1F44F-1F3FB"},
    {"аплодисменты_цвет2", "1F44F-1F3FC"},
    {"аплодисменты_цвет3", "1F44F-1F3FD"},
    {"аплодисменты_цвет4", "1F44F-1F3FE"},
    {"аплодисменты_цвет5", "1F44F-1F3FF"},
    {"ладони_в_стороны", "1F450"},
    {"ладони_в_стороны_цвет1", "1F450-1F3FB"},
    {"ладони_в_стороны_цвет2", "1F450-1F3FC"},
    {"ладони_в_стороны_цвет3", "1F450-1F3FD"},
    {"ладони_в_стороны_цвет4", "1F450-1F3FE"},
    {"ладони_в_стороны_цвет5", "1F450-1F3FF"},
    {"корона", "1F451"},
    {"шляпка", "1F452"},
    {"очки", "1F453"},
    {"галстук", "1F454"},
    {"футболка", "1F455"},
    {"джинсы", "1F456"},
    {"платье", "1F457"},
    {"кимоно", "1F458"},
    {"бикини", "1F459"},
    {"блузка", "1F45A"},
    {"кошелек", "1F45B"},
    {"сумка", "1F45C"},
    {"клатч", "1F45D"},
    {"ботинок", "1F45E"},
    {"кроссовки", "1F45F"},
    {"туфли_на_каблуке", "1F460"},
    {"босоножки", "1F461"},
    {"сапог", "1F462"},
    {"следы", "1F463"},
    {"силуэт_человека", "1F464"},
    {"два_силуэта", "1F465"},
    {"мальчик", "1F466"},
    {"мальчик_цвет1", "1F466-1F3FB"},
    {"мальчик_цвет2", "1F466-1F3FC"},
    {"мальчик_цвет3", "1F466-1F3FD"},
    {"мальчик_цвет4", "1F466-1F3FE"},
    {"мальчик_цвет5", "1F466-1F3FF"},
    {"девочка", "1F467"},
    {"девочка_цвет1", "1F467-1F3FB"},
    {"девочка_цвет2", "1F467-1F3FC"},
    {"девочка_цвет3", "1F467-1F3FD"},
    {"девочка_цвет4", "1F467-1F3FE"},
    {"девочка_цвет5", "1F467-1F3FF"},
    {"мужчина", "1F468"},
    {"мужчина_цвет1", "1F468-1F3FB"},
    {"мужчина_цвет2", "1F468-1F3FC"},
    {"мужчина_цвет3", "1F468-1F3FD"},
    {"мужчина_цвет4", "1F468-1F3FE"},
    {"мужчина_цвет5", "1F468-1F3FF"},
    {"женщина", "1F469"},
    {"женщина_цвет1", "1F469-1F3FB"},
    {"женщина_цвет2", "1F469-1F3FC"},
    {"женщина_цвет3", "1F469-1F3FD"},
    {"женщина_цвет4", "1F469-1F3FE"},
    {"женщина_цвет5", "1F469-1F3FF"},
    {"семья", "1F46A"},
    {"мужчина_и_женщина", "1F46B"},
    {"мужчина_и_женщина_цвет1", "1F46B-1F3FB"},
    {"мужчина_и_женщина_цвет2", "1F46B-1F3FC"},
    {"мужчина_и_женщина_цвет3", "1F46B-1F3FD"},
    {"мужчина_и_женщина_цвет4", "1F46B-1F3FE"},
    {"мужчина_и_женщина_цвет5", "1F46B-1F3FF"},
    {"мужчина_и_женщина_цвет1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"мужчина_и_женщина_цвет1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"мужчина_и_женщина_цвет1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"мужчина_и_женщина_цвет1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"мужчина_и_женщина_цвет2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"мужчина_и_женщина_цвет2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"мужчина_и_женщина_цвет2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"мужчина_и_женщина_цвет2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"мужчина_и_женщина_цвет3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"мужчина_и_женщина_цвет3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"мужчина_и_женщина_цвет3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"мужчина_и_женщина_цвет3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"мужчина_и_женщина_цвет4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"мужчина_и_женщина_цвет4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"мужчина_и_женщина_цвет4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"мужчина_и_женщина_цвет4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"мужчина_и_женщина_цвет5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"мужчина_и_женщина_цвет5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"мужчина_и_женщина_цвет5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"мужчина_и_женщина_цвет5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"двое_мужчин", "1F46C"},
    {"двое_мужчин_цвет1", "1F46C-1F3FB"},
    {"двое_мужчин_цвет2", "1F46C-1F3FC"},
    {"двое_мужчин_цвет3", "1F46C-1F3FD"},
    {"двое_мужчин_цвет4", "1F46C-1F3FE"},
    {"двое_мужчин_цвет5", "1F46C-1F3FF"},
    {"двое_мужчин_цвет1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"двое_мужчин_цвет1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"двое_мужчин_цвет1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"двое_мужчин_цвет1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"двое_мужчин_цвет2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"двое_мужчин_цвет2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"двое_мужчин_цвет2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"двое_мужчин_цвет2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"двое_мужчин_цвет3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"двое_мужчин_цвет3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"двое_мужчин_цвет3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"двое_мужчин_цвет3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"двое_мужчин_цвет4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"двое_мужчин_цвет4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"двое_мужчин_цвет4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"двое_мужчин_цвет4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"двое_мужчин_цвет5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"двое_мужчин_цвет5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"двое_мужчин_цвет5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"двое_мужчин_цвет5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"две_женщины", "1F46D"},
    {"две_женщины_цвет1", "1F46D-1F3FB"},
    {"две_женщины_цвет2", "1F46D-1F3FC"},
    {"две_женщины_цвет3", "1F46D-1F3FD"},
    {"две_женщины_цвет4", "1F46D-1F3FE"},
    {"две_женщины_цвет5", "1F46D-1F3FF"},
    {"две_женщины_цвет1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"две_женщины_цвет1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"две_женщины_цвет1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"две_женщины_цвет1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"две_женщины_цвет2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"две_женщины_цвет2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"две_женщины_цвет2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"две_женщины_цвет2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"две_женщины_цвет3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"две_женщины_цвет3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"две_женщины_цвет3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"две_женщины_цвет3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"две_женщины_цвет4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"две_женщины_цвет4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"две_женщины_цвет4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"две_женщины_цвет4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"две_женщины_цвет5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"две_женщины_цвет5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"две_женщины_цвет5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"две_женщины_цвет5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"полицейский", "1F46E"},
    {"полицейский_цвет1", "1F46E-1F3FB"},
    {"полицейский_цвет2", "1F46E-1F3FC"},
    {"полицейский_цвет3", "1F46E-1F3FD"},
    {"полицейский_цвет4", "1F46E-1F3FE"},
    {"полицейский_цвет5", "1F46E-1F3FF"},
    {"люди_с_ушами_кролика", "1F46F"},
    {"люди_с_ушами_кролика_цвет1", "1F46F-1F3FB"},
    {"люди_с_ушами_кролика_цвет2", "1F46F-1F3FC"},
    {"люди_с_ушами_кролика_цвет3", "1F46F-1F3FD"},
    {"люди_с_ушами_кролика_цвет4", "1F46F-1F3FE"},
    {"люди_с_ушами_кролика_цвет5", "1F46F-1F3FF"},
    {"люди_с_ушами_кролика_цвет1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"люди_с_ушами_кролика_цвет1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"люди_с_ушами_кролика_цвет1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"люди_с_ушами_кролика_цвет1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"люди_с_ушами_кролика_цвет2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"люди_с_ушами_кролика_цвет2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"люди_с_ушами_кролика_цвет2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"люди_с_ушами_кролика_цвет2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"люди_с_ушами_кролика_цвет3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"люди_с_ушами_кролика_цвет3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"люди_с_ушами_кролика_цвет3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"люди_с_ушами_кролика_цвет3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"люди_с_ушами_кролика_цвет4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"люди_с_ушами_кролика_цвет4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"люди_с_ушами_кролика_цвет4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"люди_с_ушами_кролика_цвет4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"люди_с_ушами_кролика_цвет5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"люди_с_ушами_кролика_цвет5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"люди_с_ушами_кролика_цвет5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"люди_с_ушами_кролика_цвет5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"невеста", "1F470"},
    {"невеста_цвет1", "1F470-1F3FB"},
    {"невеста_цвет2", "1F470-1F3FC"},
    {"невеста_цвет3", "1F470-1F3FD"},
    {"невеста_цвет4", "1F470-1F3FE"},
    {"невеста_цвет5", "1F470-1F3FF"},
    {"человек_со_светлыми_волосами", "1F471"},
    {"человек_со_светлыми_волосами_цвет1", "1F471-1F3FB"},
    {"человек_со_светлыми_волосами_цвет2", "1F471-1F3FC"},
    {"человек_со_светлыми_волосами_цвет3", "1F471-1F3FD"},
    {"человек_со_светлыми_волосами_цвет4", "1F471-1F3FE"},
    {"человек_со_светлыми_волосами_цвет5", "1F471-1F3FF"},
    {"мужчина_в_китайской_шапке", "1F472"},
    {"мужчина_в_китайской_шапке_цвет1", "1F472-1F3FB"},
    {"мужчина_в_китайской_шапке_цвет2", "1F472-1F3FC"},
    {"мужчина_в_китайской_шапке_цвет3", "1F472-1F3FD"},
    {"мужчина_в_китайской_шапке_цвет4", "1F472-1F3FE"},
    {"мужчина_в_китайской_шапке_цвет5", "1F472-1F3FF"},
    {"человек_в_тюрбане", "1F473"},
    {"человек_в_тюрбане_цвет1", "1F473-1F3FB"},
    {"человек_в_тюрбане_цвет2", "1F473-1F3FC"},
    {"человек_в_тюрбане_цвет3", "1F473-1F3FD"},
    {"человек_в_тюрбане_цвет4", "1F473-1F3FE"},
    {"человек_в_тюрбане_цвет5", "1F473-1F3FF"},
    {"пожилой_мужчина", "1F474"},
    {"пожилой_мужчина_цвет1", "1F474-1F3FB"},
    {"пожилой_мужчина_цвет2", "1F474-1F3FC"},
    {"пожилой_мужчина_цвет3", "1F474-1F3FD"},
    {"пожилой_мужчина_цвет4", "1F474-1F3FE"},
    {"пожилой_мужчина_цвет5", "1F474-1F3FF"},
    {"пожилая_женщина", "1F475"},
    {"пожилая_женщина_цвет1", "1F475-1F3FB"},
    {"пожилая_женщина_цвет2", "1F475-1F3FC"},
    {"пожилая_женщина_цвет3", "1F475-1F3FD"},
    {"пожилая_женщина_цвет4", "1F475-1F3FE"},
    {"пожилая_женщина_цвет5", "1F475-1F3FF"},
    {"младенец", "1F476"},
    {"младенец_цвет1", "1F476-1F3FB"},
    {"младенец_цвет2", "1F476-1F3FC"},
    {"младенец_цвет3", "1F476-1F3FD"},
    {"младенец_цвет4", "1F476-1F3FE"},
    {"младенец_цвет5", "1F476-1F3FF"},
    {"строитель", "1F477"},
    {"строитель_цвет1", "1F477-1F3FB"},
    {"строитель_цвет2", "1F477-1F3FC"},
    {"строитель_цвет3", "1F477-1F3FD"},
    {"строитель_цвет4", "1F477-1F3FE"},
    {"строитель_цвет5", "1F477-1F3FF"},
    {"принцесса", "1F478"},
    {"принцесса_цвет1", "1F478-1F3FB"},
    {"принцесса_цвет2", "1F478-1F3FC"},
    {"принцесса_цвет3", "1F478-1F3FD"},
    {"принцесса_цвет4", "1F478-1F3FE"},
    {"принцесса_цвет5", "1F478-1F3FF"},
    {"чудовище", "1F479"},
    {"японский_гоблин", "1F47A"},
    {"привидение", "1F47B"},
    {"ангелочек", "1F47C"},
    {"ангелочек_цвет1", "1F47C-1F3FB"},
    {"ангелочек_цвет2", "1F47C-1F3FC"},
    {"ангелочек_цвет3", "1F47C-1F3FD"},
    {"ангелочек_цвет4", "1F47C-1F3FE"},
    {"ангелочек_цвет5", "1F47C-1F3FF"},
    {"инопланетянин", "1F47D"},
    {"космический_монстр", "1F47E"},
    {"злой_чертенок", "1F47F"},
    {"череп", "1F480"},
    {"человек_тянет_руку", "1F481"},
    {"человек_тянет_руку_цвет1", "1F481-1F3FB"},
    {"человек_тянет_руку_цвет2", "1F481-1F3FC"},
    {"человек_тянет_руку_цвет3", "1F481-1F3FD"},
    {"человек_тянет_руку_цвет4", "1F481-1F3FE"},
    {"человек_тянет_руку_цвет5", "1F481-1F3FF"},
    {"гвардеец", "1F482"},
    {"гвардеец_цвет1", "1F482-1F3FB"},
    {"гвардеец_цвет2", "1F482-1F3FC"},
    {"гвардеец_цвет3", "1F482-1F3FD"},
    {"гвардеец_цвет4", "1F482-1F3FE"},
    {"гвардеец_цвет5", "1F482-1F3FF"},
    {"танцующая_женщина", "1F483"},
    {"танцующая_женщина_цвет1", "1F483-1F3FB"},
    {"танцующая_женщина_цвет2", "1F483-1F3FC"},
    {"танцующая_женщина_цвет3", "1F483-1F3FD"},
    {"танцующая_женщина_цвет4", "1F483-1F3FE"},
    {"танцующая_женщина_цвет5", "1F483-1F3FF"},
    {"помада", "1F484"},
    {"маникюр", "1F485"},
    {"маникюр_цвет1", "1F485-1F3FB"},
    {"маникюр_цвет2", "1F485-1F3FC"},
    {"маникюр_цвет3", "1F485-1F3FD"},
    {"маникюр_цвет4", "1F485-1F3FE"},
    {"маникюр_цвет5", "1F485-1F3FF"},
    {"человеку_массируют_лицо", "1F486"},
    {"человеку_массируют_лицо_цвет1", "1F486-1F3FB"},
    {"человеку_массируют_лицо_цвет2", "1F486-1F3FC"},
    {"человеку_массируют_лицо_цвет3", "1F486-1F3FD"},
    {"человеку_массируют_лицо_цвет4", "1F486-1F3FE"},
    {"человеку_массируют_лицо_цвет5", "1F486-1F3FF"},
    {"человек_у_парикмахера", "1F487"},
    {"человек_у_парикмахера_цвет1", "1F487-1F3FB"},
    {"человек_у_парикмахера_цвет2", "1F487-1F3FC"},
    {"человек_у_парикмахера_цвет3", "1F487-1F3FD"},
    {"человек_у_парикмахера_цвет4", "1F487-1F3FE"},
    {"человек_у_парикмахера_цвет5", "1F487-1F3FF"},
    {"парикмахерская", "1F488"},
    {"шприц", "1F489"},
    {"пилюля", "1F48A"},
    {"след_от_поцелуя", "1F48B"},
    {"любовное_письмо", "1F48C"},
    {"кольцо", "1F48D"},
    {"драгоценный_камень", "1F48E"},
    {"поцелуй", "1F48F"},
    {"поцелуй_цвет1", "1F48F-1F3FB"},
    {"поцелуй_цвет2", "1F48F-1F3FC"},
    {"поцелуй_цвет3", "1F48F-1F3FD"},
    {"поцелуй_цвет4", "1F48F-1F3FE"},
    {"поцелуй_цвет5", "1F48F-1F3FF"},
    {"поцелуй_цвет1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"поцелуй_цвет1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"поцелуй_цвет1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"поцелуй_цвет1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"поцелуй_цвет2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"поцелуй_цвет2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"поцелуй_цвет2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"поцелуй_цвет2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"поцелуй_цвет3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"поцелуй_цвет3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"поцелуй_цвет3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"поцелуй_цвет3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"поцелуй_цвет4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"поцелуй_цвет4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"поцелуй_цвет4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"поцелуй_цвет4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"поцелуй_цвет5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"поцелуй_цвет5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"поцелуй_цвет5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"поцелуй_цвет5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"букет", "1F490"},
    {"влюбленная_пара", "1F491"},
    {"влюбленная_пара_цвет1", "1F491-1F3FB"},
    {"влюбленная_пара_цвет2", "1F491-1F3FC"},
    {"влюбленная_пара_цвет3", "1F491-1F3FD"},
    {"влюбленная_пара_цвет4", "1F491-1F3FE"},
    {"влюбленная_пара_цвет5", "1F491-1F3FF"},
    {"влюбленная_пара_цвет1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"влюбленная_пара_цвет1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"влюбленная_пара_цвет1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"влюбленная_пара_цвет1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"влюбленная_пара_цвет2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"влюбленная_пара_цвет2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"влюбленная_пара_цвет2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"влюбленная_пара_цвет2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"влюбленная_пара_цвет3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"влюбленная_пара_цвет3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"влюбленная_пара_цвет3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"влюбленная_пара_цвет3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"влюбленная_пара_цвет4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"влюбленная_пара_цвет4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"влюбленная_пара_цвет4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"влюбленная_пара_цвет4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"влюбленная_пара_цвет5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"влюбленная_пара_цвет5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"влюбленная_пара_цвет5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"влюбленная_пара_цвет5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"свадьба", "1F492"},
    {"пульсирующее_сердце", "1F493"},
    {"разбитое_сердце", "1F494"},
    {"два_сердца", "1F495"},
    {"сверкающее_сердце", "1F496"},
    {"растущее_сердце", "1F497"},
    {"сердце_пронзенное_стрелой", "1F498"},
    {"синее_сердце", "1F499"},
    {"зеленое_сердце", "1F49A"},
    {"желтое_сердце", "1F49B"},
    {"фиолетовое_сердце", "1F49C"},
    {"сердце_с_лентой", "1F49D"},
    {"вращающиеся_сердца", "1F49E"},
    {"значок_сердце", "1F49F"},
    {"ромб_с_точкой", "1F4A0"},
    {"лампочка", "1F4A1"},
    {"значок_гнев", "1F4A2"},
    {"бомба", "1F4A3"},
    {"сон", "1F4A4"},
    {"взрыв", "1F4A5"},
    {"капли_пота", "1F4A6"},
    {"капля", "1F4A7"},
    {"исчезновение", "1F4A8"},
    {"какашка", "1F4A9"},
    {"бицепс", "1F4AA"},
    {"бицепс_цвет1", "1F4AA-1F3FB"},
    {"бицепс_цвет2", "1F4AA-1F3FC"},
    {"бицепс_цвет3", "1F4AA-1F3FD"},
    {"бицепс_цвет4", "1F4AA-1F3FE"},
    {"бицепс_цвет5", "1F4AA-1F3FF"},
    {"кружащаяся_звезда", "1F4AB"},
    {"облачко_с_многоточием", "1F4AC"},
    {"облачко_с_мыслями", "1F4AD"},
    {"белый_цветок", "1F4AE"},
    {"сто_баллов", "1F4AF"},
    {"мешок_денег", "1F4B0"},
    {"обмен_валюты", "1F4B1"},
    {"жирный_знак_доллара", "1F4B2"},
    {"кредитная_карта", "1F4B3"},
    {"банкнота_иены", "1F4B4"},
    {"банкнота_доллара", "1F4B5"},
    {"банкнота_евро", "1F4B6"},
    {"банкнота_фунта", "1F4B7"},
    {"банкнота_с_крылышками", "1F4B8"},
    {"биржа", "1F4B9"},
    {"кресло", "1F4BA"},
    {"ноутбук", "1F4BB"},
    {"портфель", "1F4BC"},
    {"диск", "1F4BD"},
    {"дискета", "1F4BE"},
    {"папка", "1F4C1"},
    {"открытая_папка", "1F4C2"},
    {"страница_с_загибом", "1F4C3"},
    {"страница", "1F4C4"},
    {"календарь", "1F4C5"},
    {"отрывной_календарь", "1F4C6"},
    {"органайзер", "1F4C7"},
    {"диаграмма_роста", "1F4C8"},
    {"диаграмма_падения", "1F4C9"},
    {"гистограмма", "1F4CA"},
    {"планшет_с_зажимом", "1F4CB"},
    {"канцелярская_кнопка", "1F4CC"},
    {"канцелярская_кнопка_с_шариком", "1F4CD"},
    {"скрепка", "1F4CE"},
    {"линейка", "1F4CF"},
    {"линейка_треугольник", "1F4D0"},
    {"стикеры_закладки", "1F4D1"},
    {"блокнот", "1F4D2"},
    {"тетрадь", "1F4D3"},
    {"записная_книжка", "1F4D4"},
    {"закрытая_книга", "1F4D5"},
    {"открытая_книга", "1F4D6"},
    {"зеленая_книга", "1F4D7"},
    {"синяя_книга", "1F4D8"},
    {"оранжевая_книга", "1F4D9"},
    {"книги", "1F4DA"},
    {"бейдж", "1F4DB"},
    {"свиток", "1F4DC"},
    {"заметка", "1F4DD"},
    {"телефонная_трубка", "1F4DE"},
    {"пейджер", "1F4DF"},
    {"факс", "1F4E0"},
    {"спутниковая_антенна", "1F4E1"},
    {"громкоговоритель", "1F4E2"},
    {"мегафон", "1F4E3"},
    {"исходящие", "1F4E4"},
    {"входящие", "1F4E5"},
    {"посылка", "1F4E6"},
    {"электронная_почта", "1F4E7"},
    {"входящее_письмо", "1F4E8"},
    {"исходящее_письмо", "1F4E9"},
    {"закрытый_почтовый_ящик_с_опущенным_флажком", "1F4EA"},
    {"закрытый_почтовый_ящик_с_поднятым_флажком", "1F4EB"},
    {"открытый_почтовый_ящик_с_поднятым_флажком", "1F4EC"},
    {"открытый_почтовый_ящик_с_опущенным_флажком", "1F4ED"},
    {"почтовый_ящик", "1F4EE"},
    {"почтовый_рожок", "1F4EF"},
    {"газета", "1F4F0"},
    {"мобильный_телефон", "1F4F1"},
    {"входящий_вызов", "1F4F2"},
    {"режим_вибрации", "1F4F3"},
    {"выключенный_телефон", "1F4F4"},
    {"мобильные_телефоны_запрещены", "1F4F5"},
    {"сигнал_сети", "1F4F6"},
    {"фотоаппарат", "1F4F7"},
    {"фотоаппарат_со_вспышкой", "1F4F8"},
    {"видеокамера", "1F4F9"},
    {"телевизор", "1F4FA"},
    {"радио", "1F4FB"},
    {"видеокассета", "1F4FC"},
    {"кинопроектор", "1F4FD"},
    {"четки", "1F4FF"},
    {"случайный_выбор", "1F500"},
    {"повтор", "1F501"},
    {"повтор_одной_песни", "1F502"},
    {"по_часовой_стрелке", "1F503"},
    {"против_часовой_стрелки", "1F504"},
    {"низкая_яркость", "1F505"},
    {"высокая_яркость", "1F506"},
    {"без_звука", "1F507"},
    {"низкая_громкость", "1F508"},
    {"средняя_громкость", "1F509"},
    {"высокая_громкость", "1F50A"},
    {"батарейка", "1F50B"},
    {"вилка", "1F50C"},
    {"лупа_наклоненная_влево", "1F50D"},
    {"лупа_наклоненная_вправо", "1F50E"},
    {"закрытый_замок_с_перьевой_ручкой", "1F50F"},
    {"закрытый_замок_с_ключом", "1F510"},
    {"ключ", "1F511"},
    {"закрытый_замок", "1F512"},
    {"открытый_замок", "1F513"},
    {"колокольчик", "1F514"},
    {"перечеркнутый_колокольчик", "1F515"},
    {"закладка", "1F516"},
    {"гиперссылка", "1F517"},
    {"круглая_кнопка", "1F518"},
    {"стрелка_назад", "1F519"},
    {"стрелка_конец", "1F51A"},
    {"стрелка_активно", "1F51B"},
    {"стрелка_скоро", "1F51C"},
    {"стрелка_верх", "1F51D"},
    {"для_людей_старше_восемнадцати_лет", "1F51E"},
    {"клавиши_10", "1F51F"},
    {"ввод_прописными", "1F520"},
    {"ввод_строчными", "1F521"},
    {"ввод_цифр", "1F522"},
    {"ввод_символов", "1F523"},
    {"ввод_латиницей", "1F524"},
    {"огонь", "1F525"},
    {"фонарик", "1F526"},
    {"гаечный_ключ", "1F527"},
    {"молоток", "1F528"},
    {"болт_и_гайка", "1F529"},
    {"нож", "1F52A"},
    {"водяной_пистолет", "1F52B"},
    {"микроскоп", "1F52C"},
    {"телескоп", "1F52D"},
    {"хрустальный_шар", "1F52E"},
    {"шестиконечная_звезда", "1F52F"},
    {"символ_новичка", "1F530"},
    {"трезубец", "1F531"},
    {"кнопка_с_черной_рамкой", "1F532"},
    {"кнопка_с_белой_рамкой", "1F533"},
    {"красный_круг", "1F534"},
    {"голубой_круг", "1F535"},
    {"большой_оранжевый_ромб", "1F536"},
    {"большой_голубой_ромб", "1F537"},
    {"небольшой_оранжевый_ромб", "1F538"},
    {"небольшой_голубой_ромб", "1F539"},
    {"треугольник_вершиной_вверх", "1F53A"},
    {"треугольник_вершиной_вниз", "1F53B"},
    {"треугольник_острием_вверх", "1F53C"},
    {"треугольник_острием_вниз", "1F53D"},
    {"ом", "1F549"},
    {"голубь", "1F54A"},
    {"кааба", "1F54B"},
    {"мечеть", "1F54C"},
    {"синагога", "1F54D"},
    {"менора", "1F54E"},
    {"час", "1F550"},
    {"два_часа", "1F551"},
    {"три_часа", "1F552"},
    {"четыре_часа", "1F553"},
    {"пять_часов", "1F554"},
    {"шесть_часов", "1F555"},
    {"семь_часов", "1F556"},
    {"восемь_часов", "1F557"},
    {"девять_часов", "1F558"},
    {"десять_часов", "1F559"},
    {"одиннадцать_часов", "1F55A"},
    {"двенадцать_часов", "1F55B"},
    {"полвторого", "1F55C"},
    {"полтретьего", "1F55D"},
    {"полчетвертого", "1F55E"},
    {"полпятого", "1F55F"},
    {"полшестого", "1F560"},
    {"полседьмого", "1F561"},
    {"полвосьмого", "1F562"},
    {"полдевятого", "1F563"},
    {"полдесятого", "1F564"},
    {"пол_одиннадцатого", "1F565"},
    {"полдвенадцатого", "1F566"},
    {"полпервого", "1F567"},
    {"свеча", "1F56F"},
    {"каминные_часы", "1F570"},
    {"дыра", "1F573"},
    {"бизнесмен_в_воздухе", "1F574"},
    {"бизнесмен_в_воздухе_цвет1", "1F574-1F3FB"},
    {"бизнесмен_в_воздухе_цвет2", "1F574-1F3FC"},
    {"бизнесмен_в_воздухе_цвет3", "1F574-1F3FD"},
    {"бизнесмен_в_воздухе_цвет4", "1F574-1F3FE"},
    {"бизнесмен_в_воздухе_цвет5", "1F574-1F3FF"},
    {"детектив", "1F575"},
    {"детектив_цвет1", "1F575-1F3FB"},
    {"детектив_цвет2", "1F575-1F3FC"},
    {"детектив_цвет3", "1F575-1F3FD"},
    {"детектив_цвет4", "1F575-1F3FE"},
    {"детектив_цвет5", "1F575-1F3FF"},
    {"солнечные_очки", "1F576"},
    {"паук", "1F577"},
    {"паутина", "1F578"},
    {"джойстик", "1F579"},
    {"танцующий_мужчина", "1F57A"},
    {"танцующий_мужчина_цвет1", "1F57A-1F3FB"},
    {"танцующий_мужчина_цвет2", "1F57A-1F3FC"},
    {"танцующий_мужчина_цвет3", "1F57A-1F3FD"},
    {"танцующий_мужчина_цвет4", "1F57A-1F3FE"},
    {"танцующий_мужчина_цвет5", "1F57A-1F3FF"},
    {"скрепки", "1F587"},
    {"шариковая_ручка", "1F58A"},
    {"перьевая_ручка", "1F58B"},
    {"кисточка", "1F58C"},
    {"восковой_мелок", "1F58D"},
    {"раскрытая_ладонь", "1F590"},
    {"раскрытая_ладонь_цвет1", "1F590-1F3FB"},
    {"раскрытая_ладонь_цвет2", "1F590-1F3FC"},
    {"раскрытая_ладонь_цвет3", "1F590-1F3FD"},
    {"раскрытая_ладонь_цвет4", "1F590-1F3FE"},
    {"раскрытая_ладонь_цвет5", "1F590-1F3FF"},
    {"средний_палец", "1F595"},
    {"средний_палец_цвет1", "1F595-1F3FB"},
    {"средний_палец_цвет2", "1F595-1F3FC"},
    {"средний_палец_цвет3", "1F595-1F3FD"},
    {"средний_палец_цвет4", "1F595-1F3FE"},
    {"средний_палец_цвет5", "1F595-1F3FF"},
    {"вулканский_салют", "1F596"},
    {"вулканский_салют_цвет1", "1F596-1F3FB"},
    {"вулканский_салют_цвет2", "1F596-1F3FC"},
    {"вулканский_салют_цвет3", "1F596-1F3FD"},
    {"вулканский_салют_цвет4", "1F596-1F3FE"},
    {"вулканский_салют_цвет5", "1F596-1F3FF"},
    {"черное_сердце", "1F5A4"},
    {"компьютер", "1F5A5"},
    {"принтер", "1F5A8"},
    {"мышка", "1F5B1"},
    {"трекбол", "1F5B2"},
    {"картина", "1F5BC"},
    {"маркированные_папки", "1F5C2"},
    {"картотека", "1F5C3"},
    {"архивный_шкаф", "1F5C4"},
    {"корзина_для_мусора", "1F5D1"},
    {"перекидной_блокнот", "1F5D2"},
    {"перекидной_календарь", "1F5D3"},
    {"компрессор", "1F5DC"},
    {"старинный_ключ", "1F5DD"},
    {"свернутая_газета", "1F5DE"},
    {"кинжал", "1F5E1"},
    {"говорящая_голова", "1F5E3"},
    {"пустое_облачко", "1F5E8"},
    {"гневное_облачко", "1F5EF"},
    {"избирательная_урна", "1F5F3"},
    {"карта_мира", "1F5FA"},
    {"фудзияма", "1F5FB"},
    {"токийская_башня", "1F5FC"},
    {"статуя_свободы", "1F5FD"},
    {"карта_японии", "1F5FE"},
    {"истукан", "1F5FF"},
    {"широко_улыбается", "1F600"},
    {"светится_от_счастья", "1F601"},
    {"смеется_до_слез", "1F602"},
    {"смеется", "1F603"},
    {"смеется_с_закрытыми_глазами", "1F604"},
    {"нервно_смеется", "1F605"},
    {"смеется_с_прищуренными_глазами", "1F606"},
    {"с_нимбом", "1F607"},
    {"улыбающийся_чертенок", "1F608"},
    {"подмигивает", "1F609"},
    {"довольно_улыбается", "1F60A"},
    {"вкусно", "1F60B"},
    {"испытывает_облегчение", "1F60C"},
    {"влюбленное_лицо", "1F60D"},
    {"лицо_в_темных_очках", "1F60E"},
    {"ухмыляется", "1F60F"},
    {"все_равно", "1F610"},
    {"без_эмоций", "1F611"},
    {"скучает", "1F612"},
    {"в_холодном_поту", "1F613"},
    {"размышляет", "1F614"},
    {"в_замешательстве", "1F615"},
    {"в_растерянности", "1F616"},
    {"целует", "1F617"},
    {"воздушный_поцелуй", "1F618"},
    {"счастливый_поцелуй", "1F619"},
    {"целует_с_закрытыми_глазами", "1F61A"},
    {"показывает_язык", "1F61B"},
    {"показывает_язык_и_подмигивает", "1F61C"},
    {"морщится_и_показывает_язык", "1F61D"},
    {"разочарование", "1F61E"},
    {"беспокоится", "1F61F"},
    {"сердится", "1F620"},
    {"злится", "1F621"},
    {"плачет", "1F622"},
    {"упорство", "1F623"},
    {"в_ожидании_успеха", "1F624"},
    {"обошлось", "1F625"},
    {"в_изумлении", "1F626"},
    {"страдает", "1F627"},
    {"боится", "1F628"},
    {"усталость", "1F629"},
    {"хочет_спать", "1F62A"},
    {"сильная_усталость", "1F62B"},
    {"гримаса", "1F62C"},
    {"слезы_рекой", "1F62D"},
    {"удивление", "1F62E"},
    {"неприятный_сюрприз", "1F62F"},
    {"сильно_беспокоится", "1F630"},
    {"в_ужасе", "1F631"},
    {"в_шоке", "1F632"},
    {"краснеет", "1F633"},
    {"спит", "1F634"},
    {"головокружение", "1F635"},
    {"лицо_без_рта", "1F636"},
    {"в_медицинской_маске", "1F637"},
    {"смеющийся_кот", "1F638"},
    {"смеющийся_до_слез_кот", "1F639"},
    {"улыбающийся_кот", "1F63A"},
    {"влюбленный_кот", "1F63B"},
    {"ухмыляющийся_кот", "1F63C"},
    {"целующий_кот", "1F63D"},
    {"злой_кот", "1F63E"},
    {"плачущий_кот", "1F63F"},
    {"кот_в_шоке", "1F640"},
    {"немного_грустит", "1F641"},
    {"слегка_улыбается", "1F642"},
    {"вверх_тормашками", "1F643"},
    {"закатывает_глаза", "1F644"},
    {"человек_против", "1F645"},
    {"человек_против_цвет1", "1F645-1F3FB"},
    {"человек_против_цвет2", "1F645-1F3FC"},
    {"человек_против_цвет3", "1F645-1F3FD"},
    {"человек_против_цвет4", "1F645-1F3FE"},
    {"человек_против_цвет5", "1F645-1F3FF"},
    {"человек_за", "1F646"},
    {"человек_за_цвет1", "1F646-1F3FB"},
    {"человек_за_цвет2", "1F646-1F3FC"},
    {"человек_за_цвет3", "1F646-1F3FD"},
    {"человек_за_цвет4", "1F646-1F3FE"},
    {"человек_за_цвет5", "1F646-1F3FF"},
    {"человек_кланяется", "1F647"},
    {"человек_кланяется_цвет1", "1F647-1F3FB"},
    {"человек_кланяется_цвет2", "1F647-1F3FC"},
    {"человек_кланяется_цвет3", "1F647-1F3FD"},
    {"человек_кланяется_цвет4", "1F647-1F3FE"},
    {"человек_кланяется_цвет5", "1F647-1F3FF"},
    {"ничего_не_вижу", "1F648"},
    {"ничего_не_слышу", "1F649"},
    {"ничего_никому_не_скажу", "1F64A"},
    {"человек_с_поднятой_рукой", "1F64B"},
    {"человек_с_поднятой_рукой_цвет1", "1F64B-1F3FB"},
    {"человек_с_поднятой_рукой_цвет2", "1F64B-1F3FC"},
    {"человек_с_поднятой_рукой_цвет3", "1F64B-1F3FD"},
    {"человек_с_поднятой_рукой_цвет4", "1F64B-1F3FE"},
    {"человек_с_поднятой_рукой_цвет5", "1F64B-1F3FF"},
    {"ура", "1F64C"},
    {"ура_цвет1", "1F64C-1F3FB"},
    {"ура_цвет2", "1F64C-1F3FC"},
    {"ура_цвет3", "1F64C-1F3FD"},
    {"ура_цвет4", "1F64C-1F3FE"},
    {"ура_цвет5", "1F64C-1F3FF"},
    {"нахмурившийся_человек", "1F64D"},
    {"нахмурившийся_человек_цвет1", "1F64D-1F3FB"},
    {"нахмурившийся_человек_цвет2", "1F64D-1F3FC"},
    {"нахмурившийся_человек_цвет3", "1F64D-1F3FD"},
    {"нахмурившийся_человек_цвет4", "1F64D-1F3FE"},
    {"нахмурившийся_человек_цвет5", "1F64D-1F3FF"},
    {"обиженный_человек", "1F64E"},
    {"обиженный_человек_цвет1", "1F64E-1F3FB"},
    {"обиженный_человек_цвет2", "1F64E-1F3FC"},
    {"обиженный_человек_цвет3", "1F64E-1F3FD"},
    {"обиженный_человек_цвет4", "1F64E-1F3FE"},
    {"обиженный_человек_цвет5", "1F64E-1F3FF"},
    {"сложенные_руки", "1F64F"},
    {"сложенные_руки_цвет1", "1F64F-1F3FB"},
    {"сложенные_руки_цвет2", "1F64F-1F3FC"},
    {"сложенные_руки_цвет3", "1F64F-1F3FD"},
    {"сложенные_руки_цвет4", "1F64F-1F3FE"},
    {"сложенные_руки_цвет5", "1F64F-1F3FF"},
    {"ракета", "1F680"},
    {"вертолет", "1F681"},
    {"паровоз", "1F682"},
    {"вагон", "1F683"},
    {"скоростной_поезд", "1F684"},
    {"поезд_пуля", "1F685"},
    {"поезд", "1F686"},
    {"метро", "1F687"},
    {"легкий_рельсовый_транспорт", "1F688"},
    {"железнодорожный_вокзал", "1F689"},
    {"трамвай", "1F68A"},
    {"трамвайный_вагон", "1F68B"},
    {"автобус", "1F68C"},
    {"автобус_спереди", "1F68D"},
    {"троллейбус", "1F68E"},
    {"автобусная_остановка", "1F68F"},
    {"микроавтобус", "1F690"},
    {"машина_скорой_помощи", "1F691"},
    {"пожарная_машина", "1F692"},
    {"полицейская_машина", "1F693"},
    {"полицейская_машина_спереди", "1F694"},
    {"такси", "1F695"},
    {"такси_спереди", "1F696"},
    {"автомобиль", "1F697"},
    {"автомобиль_спереди", "1F698"},
    {"джип", "1F699"},
    {"грузовик", "1F69A"},
    {"фура", "1F69B"},
    {"трактор", "1F69C"},
    {"монорельс", "1F69D"},
    {"поезд_в_горах", "1F69E"},
    {"подвесная_железная_дорога", "1F69F"},
    {"вагон_фуникулера", "1F6A0"},
    {"канатная_дорога", "1F6A1"},
    {"корабль", "1F6A2"},
    {"гребля", "1F6A3"},
    {"гребля_цвет1", "1F6A3-1F3FB"},
    {"гребля_цвет2", "1F6A3-1F3FC"},
    {"гребля_цвет3", "1F6A3-1F3FD"},
    {"гребля_цвет4", "1F6A3-1F3FE"},
    {"гребля_цвет5", "1F6A3-1F3FF"},
    {"катер", "1F6A4"},
    {"горизонтальный_светофор", "1F6A5"},
    {"вертикальный_светофор", "1F6A6"},
    {"строительные_работы", "1F6A7"},
    {"мигалка", "1F6A8"},
    {"треугольный_флаг", "1F6A9"},
    {"дверь", "1F6AA"},
    {"запрещено", "1F6AB"},
    {"сигарета", "1F6AC"},
    {"курение_запрещено", "1F6AD"},
    {"бросайте_мусор_в_урну", "1F6AE"},
    {"не_мусорьте", "1F6AF"},
    {"питьевая_вода", "1F6B0"},
    {"непитьевая_вода", "1F6B1"},
    {"велосипед", "1F6B2"},
    {"перечеркнутый_велосипед", "1F6B3"},
    {"велосипедист", "1F6B4"},
    {"велосипедист_цвет1", "1F6B4-1F3FB"},
    {"велосипедист_цвет2", "1F6B4-1F3FC"},
    {"велосипедист_цвет3", "1F6B4-1F3FD"},
    {"велосипедист_цвет4", "1F6B4-1F3FE"},
    {"велосипедист_цвет5", "1F6B4-1F3FF"},
    {"человек_на_горном_велосипеде", "1F6B5"},
    {"человек_на_горном_велосипеде_цвет1", "1F6B5-1F3FB"},
    {"человек_на_горном_велосипеде_цвет2", "1F6B5-1F3FC"},
    {"человек_на_горном_велосипеде_цвет3", "1F6B5-1F3FD"},
    {"человек_на_горном_велосипеде_цвет4", "1F6B5-1F3FE"},
    {"человек_на_горном_велосипеде_цвет5", "1F6B5-1F3FF"},
    {"человек_идет", "1F6B6"},
    {"человек_идет_цвет1", "1F6B6-1F3FB"},
    {"человек_идет_цвет2", "1F6B6-1F3FC"},
    {"человек_идет_цвет3", "1F6B6-1F3FD"},
    {"человек_идет_цвет4", "1F6B6-1F3FE"},
    {"человек_идет_цвет5", "1F6B6-1F3FF"},
    {"переход_запрещен", "1F6B7"},
    {"знак_дети", "1F6B8"},
    {"мужской_туалет", "1F6B9"},
    {"женский_туалет", "1F6BA"},
    {"уборная", "1F6BB"},
    {"пеленальная", "1F6BC"},
    {"унитаз", "1F6BD"},
    {"туалет", "1F6BE"},
    {"душ", "1F6BF"},
    {"человек_в_ванне", "1F6C0"},
    {"человек_в_ванне_цвет1", "1F6C0-1F3FB"},
    {"человек_в_ванне_цвет2", "1F6C0-1F3FC"},
    {"человек_в_ванне_цвет3", "1F6C0-1F3FD"},
    {"человек_в_ванне_цвет4", "1F6C0-1F3FE"},
    {"человек_в_ванне_цвет5", "1F6C0-1F3FF"},
    {"ванна", "1F6C1"},
    {"паспортный_контроль", "1F6C2"},
    {"таможня", "1F6C3"},
    {"получение_багажа", "1F6C4"},
    {"забытый_багаж", "1F6C5"},
    {"диван_и_торшер", "1F6CB"},
    {"человек_в_кровати", "1F6CC"},
    {"человек_в_кровати_цвет1", "1F6CC-1F3FB"},
    {"человек_в_кровати_цвет2", "1F6CC-1F3FC"},
    {"человек_в_кровати_цвет3", "1F6CC-1F3FD"},
    {"человек_в_кровати_цвет4", "1F6CC-1F3FE"},
    {"человек_в_кровати_цвет5", "1F6CC-1F3FF"},
    {"пакеты", "1F6CD"},
    {"гостиничный_звонок", "1F6CE"},
    {"кровать", "1F6CF"},
    {"религия", "1F6D0"},
    {"знак_стоп", "1F6D1"},
    {"тележка_для_покупок", "1F6D2"},
    {"индуистский_храм", "1F6D5"},
    {"хижина", "1F6D6"},
    {"лифт", "1F6D7"},
    {"оползень", "1F6D8"},
    {"беспроводная_сеть", "1F6DC"},
    {"детская_горка", "1F6DD"},
    {"автомобильное_колесо", "1F6DE"},
    {"спасательный_круг", "1F6DF"},
    {"молот_и_гаечный_ключ", "1F6E0"},
    {"щит", "1F6E1"},
    {"бочка", "1F6E2"},
    {"шоссе", "1F6E3"},
    {"железная_дорога", "1F6E4"},
    {"моторная_лодка", "1F6E5"},
    {"маленький_самолет", "1F6E9"},
    {"взлет", "1F6EB"},
    {"посадка", "1F6EC"},
    {"спутник", "1F6F0"},
    {"пассажирский_лайнер", "1F6F3"},
    {"самокат", "1F6F4"},
    {"скутер", "1F6F5"},
    {"каноэ", "1F6F6"},
    {"санки", "1F6F7"},
    {"летающая_тарелка", "1F6F8"},
    {"скейтборд", "1F6F9"},
    {"авторикша", "1F6FA"},
    {"пикап", "1F6FB"},
    {"роликовые_коньки", "1F6FC"},
    {"оранжевый_круг", "1F7E0"},
    {"желтый_круг", "1F7E1"},
    {"зеленый_круг", "1F7E2"},
    {"фиолетовый_круг", "1F7E3"},
    {"коричневый_круг", "1F7E4"},
    {"красный_квадрат", "1F7E5"},
    {"синий_квадрат", "1F7E6"},
    {"оранжевый_квадрат", "1F7E7"},
    {"желтый_квадрат", "1F7E8"},
    {"зеленый_квадрат", "1F7E9"},
    {"фиолетовый_квадрат", "1F7EA"},
    {"коричневый_квадрат", "1F7EB"},
    {"жирный_знак_равенства", "1F7F0"},
    {"щепоть", "1F90C"},
    {"щепоть_цвет1", "1F90C-1F3FB"},
    {"щепоть_цвет2", "1F90C-1F3FC"},
    {"щепоть_цвет3", "1F90C-1F3FD"},
    {"щепоть_цвет4", "1F90C-1F3FE"},
    {"щепоть_цвет5", "1F90C-1F3FF"},
    {"белое_сердце", "1F90D"},
    {"коричневое_сердце", "1F90E"},
    {"жест_мало", "1F90F"},
    {"жест_мало_цвет1", "1F90F-1F3FB"},
    {"жест_мало_цвет2", "1F90F-1F3FC"},
    {"жест_мало_цвет3", "1F90F-1F3FD"},
    {"жест_мало_цвет4", "1F90F-1F3FE"},
    {"жест_мало_цвет5", "1F90F-1F3FF"},
    {"рот_на_замке", "1F910"},
    {"деньги_в_глазах", "1F911"},
    {"с_градусником_во_рту", "1F912"},
    {"очкарик", "1F913"},
    {"думает", "1F914"},
    {"с_перевязанной_головой", "1F915"},
    {"робот", "1F916"},
    {"обнимает", "1F917"},
    {"рокерский_жест", "1F918"},
    {"рокерский_жест_цвет1", "1F918-1F3FB"},
    {"рокерский_жест_цвет2", "1F918-1F3FC"},
    {"рокерский_жест_цвет3", "1F918-1F3FD"},
    {"рокерский_жест_цвет4", "1F918-1F3FE"},
    {"рокерский_жест_цвет5", "1F918-1F3FF"},
    {"жест_позвони_мне", "1F919"},
    {"жест_позвони_мне_цвет1", "1F919-1F3FB"},
    {"жест_позвони_мне_цвет2", "1F919-1F3FC"},
    {"жест_позвони_мне_цвет3", "1F919-1F3FD"},
    {"жест_позвони_мне_цвет4", "1F919-1F3FE"},
    {"жест_позвони_мне_цвет5", "1F919-1F3FF"},
    {"поднятая_рука", "1F91A"},
    {"поднятая_рука_цвет1", "1F91A-1F3FB"},
    {"поднятая_рука_цвет2", "1F91A-1F3FC"},
    {"поднятая_рука_цвет3", "1F91A-1F3FD"},
    {"поднятая_рука_цвет4", "1F91A-1F3FE"},
    {"поднятая_рука_цвет5", "1F91A-1F3FF"},
    {"кулак_влево", "1F91B"},
    {"кулак_влево_цвет1", "1F91B-1F3FB"},
    {"кулак_влево_цвет2", "1F91B-1F3FC"},
    {"кулак_влево_цвет3", "1F91B-1F3FD"},
    {"кулак_влево_цвет4", "1F91B-1F3FE"},
    {"кулак_влево_цвет5", "1F91B-1F3FF"},
    {"кулак_вправо", "1F91C"},
    {"кулак_вправо_цвет1", "1F91C-1F3FB"},
    {"кулак_вправо_цвет2", "1F91C-1F3FC"},
    {"кулак_вправо_цвет3", "1F91C-1F3FD"},
    {"кулак_вправо_цвет4", "1F91C-1F3FE"},
    {"кулак_вправо_цвет5", "1F91C-1F3FF"},
    {"рукопожатие", "1F91D"},
    {"рукопожатие_цвет1", "1F91D-1F3FB"},
    {"рукопожатие_цвет2", "1F91D-1F3FC"},
    {"рукопожатие_цвет3", "1F91D-1F3FD"},
    {"рукопожатие_цвет4", "1F91D-1F3FE"},
    {"рукопожатие_цвет5", "1F91D-1F3FF"},
    {"рукопожатие_цвет1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"рукопожатие_цвет1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"рукопожатие_цвет1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"рукопожатие_цвет1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"рукопожатие_цвет2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"рукопожатие_цвет2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"рукопожатие_цвет2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"рукопожатие_цвет2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"рукопожатие_цвет3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"рукопожатие_цвет3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"рукопожатие_цвет3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"рукопожатие_цвет3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"рукопожатие_цвет4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"рукопожатие_цвет4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"рукопожатие_цвет4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"рукопожатие_цвет4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"рукопожатие_цвет5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"рукопожатие_цвет5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"рукопожатие_цвет5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"рукопожатие_цвет5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"скрещенные_пальцы", "1F91E"},
    {"скрещенные_пальцы_цвет1", "1F91E-1F3FB"},
    {"скрещенные_пальцы_цвет2", "1F91E-1F3FC"},
    {"скрещенные_пальцы_цвет3", "1F91E-1F3FD"},
    {"скрещенные_пальцы_цвет4", "1F91E-1F3FE"},
    {"скрещенные_пальцы_цвет5", "1F91E-1F3FF"},
    {"я_тебя_люблю", "1F91F"},
    {"я_тебя_люблю_цвет1", "1F91F-1F3FB"},
    {"я_тебя_люблю_цвет2", "1F91F-1F3FC"},
    {"я_тебя_люблю_цвет3", "1F91F-1F3FD"},
    {"я_тебя_люблю_цвет4", "1F91F-1F3FE"},
    {"я_тебя_люблю_цвет5", "1F91F-1F3FF"},
    {"в_ковбойской_шляпе", "1F920"},
    {"клоун", "1F921"},
    {"тошнит", "1F922"},
    {"катается_со_смеху", "1F923"},
    {"пускает_слюни", "1F924"},
    {"лжец", "1F925"},
    {"человек_потерял_дар_речи", "1F926"},
    {"человек_потерял_дар_речи_цвет1", "1F926-1F3FB"},
    {"человек_потерял_дар_речи_цвет2", "1F926-1F3FC"},
    {"человек_потерял_дар_речи_цвет3", "1F926-1F3FD"},
    {"человек_потерял_дар_речи_цвет4", "1F926-1F3FE"},
    {"человек_потерял_дар_речи_цвет5", "1F926-1F3FF"},
    {"чихает", "1F927"},
    {"не_верит", "1F928"},
    {"звезды_в_глазах", "1F929"},
    {"сумасшедшее_лицо", "1F92A"},
    {"тс_с_с", "1F92B"},
    {"цензура", "1F92C"},
    {"прикрывает_рот_рукой", "1F92D"},
    {"рвота", "1F92E"},
    {"взрыв_мозга", "1F92F"},
    {"беременная_женщина", "1F930"},
    {"беременная_женщина_цвет1", "1F930-1F3FB"},
    {"беременная_женщина_цвет2", "1F930-1F3FC"},
    {"беременная_женщина_цвет3", "1F930-1F3FD"},
    {"беременная_женщина_цвет4", "1F930-1F3FE"},
    {"беременная_женщина_цвет5", "1F930-1F3FF"},
    {"кормление_грудью", "1F931"},
    {"кормление_грудью_цвет1", "1F931-1F3FB"},
    {"кормление_грудью_цвет2", "1F931-1F3FC"},
    {"кормление_грудью_цвет3", "1F931-1F3FD"},
    {"кормление_грудью_цвет4", "1F931-1F3FE"},
    {"кормление_грудью_цвет5", "1F931-1F3FF"},
    {"ладони_лодочкой", "1F932"},
    {"ладони_лодочкой_цвет1", "1F932-1F3FB"},
    {"ладони_лодочкой_цвет2", "1F932-1F3FC"},
    {"ладони_лодочкой_цвет3", "1F932-1F3FD"},
    {"ладони_лодочкой_цвет4", "1F932-1F3FE"},
    {"ладони_лодочкой_цвет5", "1F932-1F3FF"},
    {"селфи", "1F933"},
    {"селфи_цвет1", "1F933-1F3FB"},
    {"селфи_цвет2", "1F933-1F3FC"},
    {"селфи_цвет3", "1F933-1F3FD"},
    {"селфи_цвет4", "1F933-1F3FE"},
    {"селфи_цвет5", "1F933-1F3FF"},
    {"принц", "1F934"},
    {"принц_цвет1", "1F934-1F3FB"},
    {"принц_цвет2", "1F934-1F3FC"},
    {"принц_цвет3", "1F934-1F3FD"},
    {"принц_цвет4", "1F934-1F3FE"},
    {"принц_цвет5", "1F934-1F3FF"},
    {"жених", "1F935"},
    {"жених_цвет1", "1F935-1F3FB"},
    {"жених_цвет2", "1F935-1F3FC"},
    {"жених_цвет3", "1F935-1F3FD"},
    {"жених_цвет4", "1F935-1F3FE"},
    {"жених_цвет5", "1F935-1F3FF"},
    {"миссис_клаус", "1F936"},
    {"миссис_клаус_цвет1", "1F936-1F3FB"},
    {"миссис_клаус_цвет2", "1F936-1F3FC"},
    {"миссис_клаус_цвет3", "1F936-1F3FD"},
    {"миссис_клаус_цвет4", "1F936-1F3FE"},
    {"миссис_клаус_цвет5", "1F936-1F3FF"},
    {"человек_пожимает_плечами", "1F937"},
    {"человек_пожимает_плечами_цвет1", "1F937-1F3FB"},
    {"человек_пожимает_плечами_цвет2", "1F937-1F3FC"},
    {"человек_пожимает_плечами_цвет3", "1F937-1F3FD"},
    {"человек_пожимает_плечами_цвет4", "1F937-1F3FE"},
    {"человек_пожимает_плечами_цвет5", "1F937-1F3FF"},
    {"акробатический_трюк", "1F938"},
    {"акробатический_трюк_цвет1", "1F938-1F3FB"},
    {"акробатический_трюк_цвет2", "1F938-1F3FC"},
    {"акробатический_трюк_цвет3", "1F938-1F3FD"},
    {"акробатический_трюк_цвет4", "1F938-1F3FE"},
    {"акробатический_трюк_цвет5", "1F938-1F3FF"},
    {"жонглер", "1F939"},
    {"жонглер_цвет1", "1F939-1F3FB"},
    {"жонглер_цвет2", "1F939-1F3FC"},
    {"жонглер_цвет3", "1F939-1F3FD"},
    {"жонглер_цвет4", "1F939-1F3FE"},
    {"жонглер_цвет5", "1F939-1F3FF"},
    {"фехтовальщик", "1F93A"},
    {"борцы", "1F93C"},
    {"борцы_цвет1", "1F93C-1F3FB"},
    {"борцы_цвет2", "1F93C-1F3FC"},
    {"борцы_цвет3", "1F93C-1F3FD"},
    {"борцы_цвет4", "1F93C-1F3FE"},
    {"борцы_цвет5", "1F93C-1F3FF"},
    {"борцы_цвет1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"борцы_цвет1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"борцы_цвет1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"борцы_цвет1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"борцы_цвет2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"борцы_цвет2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"борцы_цвет2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"борцы_цвет2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"борцы_цвет3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"борцы_цвет3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"борцы_цвет3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"борцы_цвет3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"борцы_цвет4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"борцы_цвет4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"борцы_цвет4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"борцы_цвет4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"борцы_цвет5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"борцы_цвет5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"борцы_цвет5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"борцы_цвет5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"водное_поло", "1F93D"},
    {"водное_поло_цвет1", "1F93D-1F3FB"},
    {"водное_поло_цвет2", "1F93D-1F3FC"},
    {"водное_поло_цвет3", "1F93D-1F3FD"},
    {"водное_поло_цвет4", "1F93D-1F3FE"},
    {"водное_поло_цвет5", "1F93D-1F3FF"},
    {"гандбол", "1F93E"},
    {"гандбол_цвет1", "1F93E-1F3FB"},
    {"гандбол_цвет2", "1F93E-1F3FC"},
    {"гандбол_цвет3", "1F93E-1F3FD"},
    {"гандбол_цвет4", "1F93E-1F3FE"},
    {"гандбол_цвет5", "1F93E-1F3FF"},
    {"маска_с_трубкой", "1F93F"},
    {"увядший_цветок", "1F940"},
    {"барабан", "1F941"},
    {"чокающиеся_бокалы", "1F942"},
    {"стакан_с_виски", "1F943"},
    {"ложка", "1F944"},
    {"спортивные_ворота", "1F945"},
    {"золотая_медаль", "1F947"},
    {"серебряная_медаль", "1F948"},
    {"бронзовая_медаль", "1F949"},
    {"боксерская_перчатка", "1F94A"},
    {"спортивное_кимоно", "1F94B"},
    {"керлинг", "1F94C"},
    {"лакросс", "1F94D"},
    {"софтбол", "1F94E"},
    {"летающий_диск", "1F94F"},
    {"круассан", "1F950"},
    {"авокадо", "1F951"},
    {"огурец", "1F952"},
    {"бекон", "1F953"},
    {"картофель", "1F954"},
    {"морковь", "1F955"},
    {"багет", "1F956"},
    {"овощной_салат", "1F957"},
    {"рагу_в_сковороде", "1F958"},
    {"шаурма", "1F959"},
    {"яйцо", "1F95A"},
    {"стакан_молока", "1F95B"},
    {"арахис", "1F95C"},
    {"киви", "1F95D"},
    {"блины", "1F95E"},
    {"вареник", "1F95F"},
    {"печенье_с_предсказанием", "1F960"},
    {"коробка_для_еды_на_вынос", "1F961"},
    {"палочки_для_еды", "1F962"},
    {"миска_с_ложкой", "1F963"},
    {"стакан_с_соломинкой", "1F964"},
    {"кокос", "1F965"},
    {"брокколи", "1F966"},
    {"пирог", "1F967"},
    {"крендель", "1F968"},
    {"кусок_мяса", "1F969"},
    {"сэндвич", "1F96A"},
    {"консервы", "1F96B"},
    {"зеленый_салат", "1F96C"},
    {"манго", "1F96D"},
    {"лунный_пирог", "1F96E"},
    {"бублик", "1F96F"},
    {"улыбающееся_лицо_с_сердечками", "1F970"},
    {"зевает", "1F971"},
    {"улыбающееся_лицо_со_слезой", "1F972"},
    {"на_вечеринке", "1F973"},
    {"чувствует_головокружение", "1F974"},
    {"жар", "1F975"},
    {"мерзнет", "1F976"},
    {"ниндзя", "1F977"},
    {"ниндзя_цвет1", "1F977-1F3FB"},
    {"ниндзя_цвет2", "1F977-1F3FC"},
    {"ниндзя_цвет3", "1F977-1F3FD"},
    {"ниндзя_цвет4", "1F977-1F3FE"},
    {"ниндзя_цвет5", "1F977-1F3FF"},
    {"маскировка", "1F978"},
    {"еле_сдерживает_слезы", "1F979"},
    {"умоляет", "1F97A"},
    {"сари", "1F97B"},
    {"лабораторный_халат", "1F97C"},
    {"защитные_очки", "1F97D"},
    {"треккинговый_ботинок", "1F97E"},
    {"балетки", "1F97F"},
    {"краб", "1F980"},
    {"морда_льва", "1F981"},
    {"скорпион", "1F982"},
    {"индейка", "1F983"},
    {"голова_единорога", "1F984"},
    {"орел", "1F985"},
    {"утка", "1F986"},
    {"летучая_мышь", "1F987"},
    {"акула", "1F988"},
    {"сова", "1F989"},
    {"морда_лисицы", "1F98A"},
    {"бабочка", "1F98B"},
    {"олень", "1F98C"},
    {"горилла", "1F98D"},
    {"ящерица", "1F98E"},
    {"носорог", "1F98F"},
    {"креветка", "1F990"},
    {"кальмар", "1F991"},
    {"жираф", "1F992"},
    {"зебра", "1F993"},
    {"еж", "1F994"},
    {"зауропод", "1F995"},
    {"ти_рекс", "1F996"},
    {"сверчок", "1F997"},
    {"кенгуру", "1F998"},
    {"лама", "1F999"},
    {"павлин", "1F99A"},
    {"гиппопотам", "1F99B"},
    {"попугай", "1F99C"},
    {"морда_енота", "1F99D"},
    {"омар", "1F99E"},
    {"комар", "1F99F"},
    {"микроб", "1F9A0"},
    {"барсук", "1F9A1"},
    {"лебедь", "1F9A2"},
    {"мамонт", "1F9A3"},
    {"дронт", "1F9A4"},
    {"ленивец", "1F9A5"},
    {"выдра", "1F9A6"},
    {"орангутан", "1F9A7"},
    {"скунс", "1F9A8"},
    {"фламинго", "1F9A9"},
    {"устрица", "1F9AA"},
    {"бобр", "1F9AB"},
    {"бизон", "1F9AC"},
    {"тюлень", "1F9AD"},
    {"собака_поводырь", "1F9AE"},
    {"белая_трость", "1F9AF"},
    {"рыжие_волосы", "1F9B0"},
    {"кудрявые_волосы", "1F9B1"},
    {"без_волос", "1F9B2"},
    {"седые_волосы", "1F9B3"},
    {"кость", "1F9B4"},
    {"нога", "1F9B5"},
    {"нога_цвет1", "1F9B5-1F3FB"},
    {"нога_цвет2", "1F9B5-1F3FC"},
    {"нога_цвет3", "1F9B5-1F3FD"},
    {"нога_цвет4", "1F9B5-1F3FE"},
    {"нога_цвет5", "1F9B5-1F3FF"},
    {"щиколотка", "1F9B6"},
    {"щиколотка_цвет1", "1F9B6-1F3FB"},
    {"щиколотка_цвет2", "1F9B6-1F3FC"},
    {"щиколотка_цвет3", "1F9B6-1F3FD"},
    {"щиколотка_цвет4", "1F9B6-1F3FE"},
    {"щиколотка_цвет5", "1F9B6-1F3FF"},
    {"зуб", "1F9B7"},
    {"супергерой", "1F9B8"},
    {"супергерой_цвет1", "1F9B8-1F3FB"},
    {"супергерой_цвет2", "1F9B8-1F3FC"},
    {"супергерой_цвет3", "1F9B8-1F3FD"},
    {"супергерой_цвет4", "1F9B8-1F3FE"},
    {"супергерой_цвет5", "1F9B8-1F3FF"},
    {"суперзлодей", "1F9B9"},
    {"суперзлодей_цвет1", "1F9B9-1F3FB"},
    {"суперзлодей_цвет2", "1F9B9-1F3FC"},
    {"суперзлодей_цвет3", "1F9B9-1F3FD"},
    {"суперзлодей_цвет4", "1F9B9-1F3FE"},
    {"суперзлодей_цвет5", "1F9B9-1F3FF"},
    {"спасательный_жилет", "1F9BA"},
    {"ухо_со_слуховым_аппаратом", "1F9BB"},
    {"ухо_со_слуховым_аппаратом_цвет1", "1F9BB-1F3FB"},
    {"ухо_со_слуховым_аппаратом_цвет2", "1F9BB-1F3FC"},
    {"ухо_со_слуховым_аппаратом_цвет3", "1F9BB-1F3FD"},
    {"ухо_со_слуховым_аппаратом_цвет4", "1F9BB-1F3FE"},
    {"ухо_со_слуховым_аппаратом_цвет5", "1F9BB-1F3FF"},
    {"кресло_коляска_с_мотором", "1F9BC"},
    {"кресло_коляска_с_ручным_приводом", "1F9BD"},
    {"механическая_рука", "1F9BE"},
    {"механическая_нога", "1F9BF"},
    {"сыр", "1F9C0"},
    {"капкейк", "1F9C1"},
    {"солонка", "1F9C2"},
    {"пакетик_сока", "1F9C3"},
    {"чеснок", "1F9C4"},
    {"лук", "1F9C5"},
    {"фалафель", "1F9C6"},
    {"вафля", "1F9C7"},
    {"масло", "1F9C8"},
    {"мате", "1F9C9"},
    {"кубик_льда", "1F9CA"},
    {"чай_с_шариками", "1F9CB"},
    {"тролль", "1F9CC"},
    {"человек_стоит", "1F9CD"},
    {"человек_стоит_цвет1", "1F9CD-1F3FB"},
    {"человек_стоит_цвет2", "1F9CD-1F3FC"},
    {"человек_стоит_цвет3", "1F9CD-1F3FD"},
    {"человек_стоит_цвет4", "1F9CD-1F3FE"},
    {"человек_стоит_цвет5", "1F9CD-1F3FF"},
    {"человек_на_коленях", "1F9CE"},
    {"человек_на_коленях_цвет1", "1F9CE-1F3FB"},
    {"человек_на_коленях_цвет2", "1F9CE-1F3FC"},
    {"человек_на_коленях_цвет3", "1F9CE-1F3FD"},
    {"человек_на_коленях_цвет4", "1F9CE-1F3FE"},
    {"человек_на_коленях_цвет5", "1F9CE-1F3FF"},
    {"глухой_человек", "1F9CF"},
    {"глухой_человек_цвет1", "1F9CF-1F3FB"},
    {"глухой_человек_цвет2", "1F9CF-1F3FC"},
    {"глухой_человек_цвет3", "1F9CF-1F3FD"},
    {"глухой_человек_цвет4", "1F9CF-1F3FE"},
    {"глухой_человек_цвет5", "1F9CF-1F3FF"},
    {"с_моноклем", "1F9D0"},
    {"взрослый", "1F9D1"},
    {"взрослый_цвет1", "1F9D1-1F3FB"},
    {"взрослый_цвет2", "1F9D1-1F3FC"},
    {"взрослый_цвет3", "1F9D1-1F3FD"},
    {"взрослый_цвет4", "1F9D1-1F3FE"},
    {"взрослый_цвет5", "1F9D1-1F3FF"},
    {"ребенок", "1F9D2"},
    {"ребенок_цвет1", "1F9D2-1F3FB"},
    {"ребенок_цвет2", "1F9D2-1F3FC"},
    {"ребенок_цвет3", "1F9D2-1F3FD"},
    {"ребенок_цвет4", "1F9D2-1F3FE"},
    {"ребенок_цвет5", "1F9D2-1F3FF"},
    {"пожилой_человек", "1F9D3"},
    {"пожилой_человек_цвет1", "1F9D3-1F3FB"},
    {"пожилой_человек_цвет2", "1F9D3-1F3FC"},
    {"пожилой_человек_цвет3", "1F9D3-1F3FD"},
    {"пожилой_человек_цвет4", "1F9D3-1F3FE"},
    {"пожилой_человек_цвет5", "1F9D3-1F3FF"},
    {"бородатый_человек", "1F9D4"},
    {"бородатый_человек_цвет1", "1F9D4-1F3FB"},
    {"бородатый_человек_цвет2", "1F9D4-1F3FC"},
    {"бородатый_человек_цвет3", "1F9D4-1F3FD"},
    {"бородатый_человек_цвет4", "1F9D4-1F3FE"},
    {"бородатый_человек_цвет5", "1F9D4-1F3FF"},
    {"женщина_в_платке", "1F9D5"},
    {"женщина_в_платке_цвет1", "1F9D5-1F3FB"},
    {"женщина_в_платке_цвет2", "1F9D5-1F3FC"},
    {"женщина_в_платке_цвет3", "1F9D5-1F3FD"},
    {"женщина_в_платке_цвет4", "1F9D5-1F3FE"},
    {"женщина_в_платке_цвет5", "1F9D5-1F3FF"},
    {"человек_в_бане", "1F9D6"},
    {"человек_в_бане_цвет1", "1F9D6-1F3FB"},
    {"человек_в_бане_цвет2", "1F9D6-1F3FC"},
    {"человек_в_бане_цвет3", "1F9D6-1F3FD"},
    {"человек_в_бане_цвет4", "1F9D6-1F3FE"},
    {"человек_в_бане_цвет5", "1F9D6-1F3FF"},
    {"человек_взбирается_по_скале", "1F9D7"},
    {"человек_взбирается_по_скале_цвет1", "1F9D7-1F3FB"},
    {"человек_взбирается_по_скале_цвет2", "1F9D7-1F3FC"},
    {"человек_взбирается_по_скале_цвет3", "1F9D7-1F3FD"},
    {"человек_взбирается_по_скале_цвет4", "1F9D7-1F3FE"},
    {"человек_взбирается_по_скале_цвет5", "1F9D7-1F3FF"},
    {"человек_в_позе_лотоса", "1F9D8"},
    {"человек_в_позе_лотоса_цвет1", "1F9D8-1F3FB"},
    {"человек_в_позе_лотоса_цвет2", "1F9D8-1F3FC"},
    {"человек_в_позе_лотоса_цвет3", "1F9D8-1F3FD"},
    {"человек_в_позе_лотоса_цвет4", "1F9D8-1F3FE"},
    {"человек_в_позе_лотоса_цвет5", "1F9D8-1F3FF"},
    {"маг", "1F9D9"},
    {"маг_цвет1", "1F9D9-1F3FB"},
    {"маг_цвет2", "1F9D9-1F3FC"},
    {"маг_цвет3", "1F9D9-1F3FD"},
    {"маг_цвет4", "1F9D9-1F3FE"},
    {"маг_цвет5", "1F9D9-1F3FF"},
    {"фея", "1F9DA"},
    {"фея_цвет1", "1F9DA-1F3FB"},
    {"фея_цвет2", "1F9DA-1F3FC"},
    {"фея_цвет3", "1F9DA-1F3FD"},
    {"фея_цвет4", "1F9DA-1F3FE"},
    {"фея_цвет5", "1F9DA-1F3FF"},
    {"вампир", "1F9DB"},
    {"вампир_цвет1", "1F9DB-1F3FB"},
    {"вампир_цвет2", "1F9DB-1F3FC"},
    {"вампир_цвет3", "1F9DB-1F3FD"},
    {"вампир_цвет4", "1F9DB-1F3FE"},
    {"вампир_цвет5", "1F9DB-1F3FF"},
    {"русалка", "1F9DC"},
    {"русалка_цвет1", "1F9DC-1F3FB"},
    {"русалка_цвет2", "1F9DC-1F3FC"},
    {"русалка_цвет3", "1F9DC-1F3FD"},
    {"русалка_цвет4", "1F9DC-1F3FE"},
    {"русалка_цвет5", "1F9DC-1F3FF"},
    {"эльф", "1F9DD"},
    {"эльф_цвет1", "1F9DD-1F3FB"},
    {"эльф_цвет2", "1F9DD-1F3FC"},
    {"эльф_цвет3", "1F9DD-1F3FD"},
    {"эльф_цвет4", "1F9DD-1F3FE"},
    {"эльф_цвет5", "1F9DD-1F3FF"},
    {"джинн", "1F9DE"},
    {"зомби", "1F9DF"},
    {"мозг", "1F9E0"},
    {"оранжевое_сердце", "1F9E1"},
    {"бейсболка", "1F9E2"},
    {"шарф", "1F9E3"},
    {"перчатки", "1F9E4"},
    {"пальто", "1F9E5"},
    {"носки", "1F9E6"},
    {"красный_конверт", "1F9E7"},
    {"динамитная_шашка", "1F9E8"},
    {"пазл", "1F9E9"},
    {"пробирка", "1F9EA"},
    {"чашка_петри", "1F9EB"},
    {"днк", "1F9EC"},
    {"компас", "1F9ED"},
    {"счеты", "1F9EE"},
    {"огнетушитель", "1F9EF"},
    {"ящик_для_инструментов", "1F9F0"},
    {"кирпичи", "1F9F1"},
    {"магнит", "1F9F2"},
    {"багаж", "1F9F3"},
    {"бутылка_лосьона", "1F9F4"},
    {"катушка_ниток", "1F9F5"},
    {"клубок", "1F9F6"},
    {"английская_булавка", "1F9F7"},
    {"плюшевый_мишка", "1F9F8"},
    {"метла", "1F9F9"},
    {"корзина", "1F9FA"},
    {"рулон_бумаги", "1F9FB"},
    {"мыло", "1F9FC"},
    {"губка", "1F9FD"},
    {"чек", "1F9FE"},
    {"назар_бонджук", "1F9FF"},
    {"пуанты", "1FA70"},
    {"цельный_купальник", "1FA71"},
    {"плавки", "1FA72"},
    {"шорты", "1FA73"},
    {"вьетнамки", "1FA74"},
    {"голубое_сердце", "1FA75"},
    {"серое_сердце", "1FA76"},
    {"розовое_сердце", "1FA77"},
    {"капля_крови", "1FA78"},
    {"пластырь", "1FA79"},
    {"стетоскоп", "1FA7A"},
    {"рентген", "1FA7B"},
    {"костыль", "1FA7C"},
    {"йо_йо", "1FA80"},
    {"воздушный_змей", "1FA81"},
    {"парашют", "1FA82"},
    {"бумеранг", "1FA83"},
    {"волшебная_палочка", "1FA84"},
    {"пиньята", "1FA85"},
    {"матрешка", "1FA86"},
    {"маракасы", "1FA87"},
    {"флейта", "1FA88"},
    {"арфа", "1FA89"},
    {"тромбон", "1FA8A"},
    {"сундук_с_сокровищами", "1FA8E"},
    {"лопата", "1FA8F"},
    {"планета_с_кольцом", "1FA90"},
    {"стул", "1FA91"},
    {"бритва", "1FA92"},
    {"топор", "1FA93"},
    {"масляная_лампа", "1FA94"},
    {"банджо", "1FA95"},
    {"армейская_каска", "1FA96"},
    {"аккордеон", "1FA97"},
    {"конга", "1FA98"},
    {"монета", "1FA99"},
    {"пила_по_дереву", "1FA9A"},
    {"отвертка", "1FA9B"},
    {"лестница", "1FA9C"},
    {"крюк", "1FA9D"},
    {"зеркало", "1FA9E"},
    {"окно", "1FA9F"},
    {"вантуз", "1FAA0"},
    {"швейная_игла", "1FAA1"},
    {"узел", "1FAA2"},
    {"ведро", "1FAA3"},
    {"мышеловка", "1FAA4"},
    {"зубная_щетка", "1FAA5"},
    {"надгробие", "1FAA6"},
    {"плакат", "1FAA7"},
    {"камень", "1FAA8"},
    {"диско_шар", "1FAA9"},
    {"удостоверение_личности", "1FAAA"},
    {"низкий_уровень_заряда", "1FAAB"},
    {"хамса", "1FAAC"},
    {"веер", "1FAAD"},
    {"афро_гребень", "1FAAE"},
    {"кханда", "1FAAF"},
    {"муха", "1FAB0"},
    {"червяк", "1FAB1"},
    {"жук", "1FAB2"},
    {"таракан", "1FAB3"},
    {"растение_в_горшке", "1FAB4"},
    {"древесина", "1FAB5"},
    {"перо", "1FAB6"},
    {"лотос", "1FAB7"},
    {"коралл", "1FAB8"},
    {"пустое_гнездо", "1FAB9"},
    {"гнездо_с_яйцами", "1FABA"},
    {"гиацинт", "1FABB"},
    {"медуза", "1FABC"},
    {"крыло", "1FABD"},
    {"дерево_без_листвы", "1FABE"},
    {"гусь", "1FABF"},
    {"сердце_человека", "1FAC0"},
    {"легкие", "1FAC1"},
    {"обнимающиеся_люди", "1FAC2"},
    {"беременный_мужчина", "1FAC3"},
    {"беременный_мужчина_цвет1", "1FAC3-1F3FB"},
    {"беременный_мужчина_цвет2", "1FAC3-1F3FC"},
    {"беременный_мужчина_цвет3", "1FAC3-1F3FD"},
    {"беременный_мужчина_цвет4", "1FAC3-1F3FE"},
    {"беременный_мужчина_цвет5", "1FAC3-1F3FF"},
    {"беременный_человек", "1FAC4"},
    {"беременный_человек_цвет1", "1FAC4-1F3FB"},
    {"беременный_человек_цвет2", "1FAC4-1F3FC"},
    {"беременный_человек_цвет3", "1FAC4-1F3FD"},
    {"беременный_человек_цвет4", "1FAC4-1F3FE"},
    {"беременный_человек_цвет5", "1FAC4-1F3FF"},
    {"человек_с_короной", "1FAC5"},
    {"человек_с_короной_цвет1", "1FAC5-1F3FB"},
    {"человек_с_короной_цвет2", "1FAC5-1F3FC"},
    {"человек_с_короной_цвет3", "1FAC5-1F3FD"},
    {"человек_с_короной_цвет4", "1FAC5-1F3FE"},
    {"человек_с_короной_цвет5", "1FAC5-1F3FF"},
    {"отпечаток_пальца", "1FAC6"},
    {"снежный_человек", "1FAC8"},
    {"косатка", "1FACD"},
    {"лось", "1FACE"},
    {"осел", "1FACF"},
    {"голубика", "1FAD0"},
    {"болгарский_перец", "1FAD1"},
    {"оливка", "1FAD2"},
    {"лепешка", "1FAD3"},
    {"тамале", "1FAD4"},
    {"фондю", "1FAD5"},
    {"чайник", "1FAD6"},
    {"стакан_с_жидкостью", "1FAD7"},
    {"фасоль", "1FAD8"},
    {"банка", "1FAD9"},
    {"корень_имбиря", "1FADA"},
    {"стручок_гороха", "1FADB"},
    {"корнеплод", "1FADC"},
    {"клякса", "1FADF"},
    {"тает", "1FAE0"},
    {"воинское_приветствие", "1FAE1"},
    {"прикрывает_рот_рукой_с_открытыми_глазами", "1FAE2"},
    {"подсматривает", "1FAE3"},
    {"с_перекошенным_ртом", "1FAE4"},
    {"лицо_пунктиром", "1FAE5"},
    {"кусать_губы", "1FAE6"},
    {"пузыри", "1FAE7"},
    {"трясущееся_лицо", "1FAE8"},
    {"лицо_с_мешками_под_глазами", "1FAE9"},
    {"лицо_перекосило", "1FAEA"},
    {"драка", "1FAEF"},
    {"жест_деньги", "1FAF0"},
    {"жест_деньги_цвет1", "1FAF0-1F3FB"},
    {"жест_деньги_цвет2", "1FAF0-1F3FC"},
    {"жест_деньги_цвет3", "1FAF0-1F3FD"},
    {"жест_деньги_цвет4", "1FAF0-1F3FE"},
    {"жест_деньги_цвет5", "1FAF0-1F3FF"},
    {"рука_вправо", "1FAF1"},
    {"рука_вправо_цвет1", "1FAF1-1F3FB"},
    {"рука_вправо_цвет2", "1FAF1-1F3FC"},
    {"рука_вправо_цвет3", "1FAF1-1F3FD"},
    {"рука_вправо_цвет4", "1FAF1-1F3FE"},
    {"рука_вправо_цвет5", "1FAF1-1F3FF"},
    {"рука_влево", "1FAF2"},
    {"рука_влево_цвет1", "1FAF2-1F3FB"},
    {"рука_влево_цвет2", "1FAF2-1F3FC"},
    {"рука_влево_цвет3", "1FAF2-1F3FD"},
    {"рука_влево_цвет4", "1FAF2-1F3FE"},
    {"рука_влево_цвет5", "1FAF2-1F3FF"},
    {"рука_ладонью_вниз", "1FAF3"},
    {"рука_ладонью_вниз_цвет1", "1FAF3-1F3FB"},
    {"рука_ладонью_вниз_цвет2", "1FAF3-1F3FC"},
    {"рука_ладонью_вниз_цвет3", "1FAF3-1F3FD"},
    {"рука_ладонью_вниз_цвет4", "1FAF3-1F3FE"},
    {"рука_ладонью_вниз_цвет5", "1FAF3-1F3FF"},
    {"рука_ладонью_вверх", "1FAF4"},
    {"рука_ладонью_вверх_цвет1", "1FAF4-1F3FB"},
    {"рука_ладонью_вверх_цвет2", "1FAF4-1F3FC"},
    {"рука_ладонью_вверх_цвет3", "1FAF4-1F3FD"},
    {"рука_ладонью_вверх_цвет4", "1FAF4-1F3FE"},
    {"рука_ладонью_вверх_цвет5", "1FAF4-1F3FF"},
    {"указательный_палец_на_себя", "1FAF5"},
    {"указательный_палец_на_себя_цвет1", "1FAF5-1F3FB"},
    {"указательный_палец_на_себя_цвет2", "1FAF5-1F3FC"},
    {"указательный_палец_на_себя_цвет3", "1FAF5-1F3FD"},
    {"указательный_палец_на_себя_цвет4", "1FAF5-1F3FE"},
    {"указательный_палец_на_себя_цвет5", "1FAF5-1F3FF"},
    {"жест_сердце", "1FAF6"},
    {"жест_сердце_цвет1", "1FAF6-1F3FB"},
    {"жест_сердце_цвет2", "1FAF6-1F3FC"},
    {"жест_сердце_цвет3", "1FAF6-1F3FD"},
    {"жест_сердце_цвет4", "1FAF6-1F3FE"},
    {"жест_сердце_цвет5", "1FAF6-1F3FF"},
    {"рука_ладонью_влево", "1FAF7"},
    {"рука_ладонью_влево_цвет1", "1FAF7-1F3FB"},
    {"рука_ладонью_влево_цвет2", "1FAF7-1F3FC"},
    {"рука_ладонью_влево_цвет3", "1FAF7-1F3FD"},
    {"рука_ладонью_влево_цвет4", "1FAF7-1F3FE"},
    {"рука_ладонью_влево_цвет5", "1FAF7-1F3FF"},
    {"рука_ладонью_вправо", "1FAF8"},
    {"рука_ладонью_вправо_цвет1", "1FAF8-1F3FB"},
    {"рука_ладонью_вправо_цвет2", "1FAF8-1F3FC"},
    {"рука_ладонью_вправо_цвет3", "1FAF8-1F3FD"},
    {"рука_ладонью_вправо_цвет4", "1FAF8-1F3FE"},
    {"рука_ладонью_вправо_цвет5", "1FAF8-1F3FF"},
    {"клавиши_числовой_знак", "0023-FE0F-20E3"},
    {"клавиши_звездочка", "002A-FE0F-20E3"},
    {"клавиша_0", "0030-FE0F-20E3"},
    {"клавиша_1", "0031-FE0F-20E3"},
    {"клавиша_2", "0032-FE0F-20E3"},
    {"клавиша_3", "0033-FE0F-20E3"},
    {"клавиша_4", "0034-FE0F-20E3"},
    {"клавиша_5", "0035-FE0F-20E3"},
    {"клавиша_6", "0036-FE0F-20E3"},
    {"клавиша_7", "0037-FE0F-20E3"},
    {"клавиша_8", "0038-FE0F-20E3"},
    {"клавиша_9", "0039-FE0F-20E3"},
    {"флаг_о_в_вознесения", "1F1E6-1F1E8"},
    {"флаг_андорра", "1F1E6-1F1E9"},
    {"флаг_оаэ", "1F1E6-1F1EA"},
    {"флаг_афганистан", "1F1E6-1F1EB"},
    {"флаг_антигуа_и_барбуда", "1F1E6-1F1EC"},
    {"флаг_ангилья", "1F1E6-1F1EE"},
    {"флаг_албания", "1F1E6-1F1F1"},
    {"флаг_армения", "1F1E6-1F1F2"},
    {"флаг_ангола", "1F1E6-1F1F4"},
    {"флаг_антарктида", "1F1E6-1F1F6"},
    {"флаг_аргентина", "1F1E6-1F1F7"},
    {"флаг_американское_самоа", "1F1E6-1F1F8"},
    {"флаг_австрия", "1F1E6-1F1F9"},
    {"флаг_австралия", "1F1E6-1F1FA"},
    {"флаг_аруба", "1F1E6-1F1FC"},
    {"флаг_аландские_о_ва", "1F1E6-1F1FD"},
    {"флаг_азербайджан", "1F1E6-1F1FF"},
    {"флаг_босния_и_герцеговина", "1F1E7-1F1E6"},
    {"флаг_барбадос", "1F1E7-1F1E7"},
    {"флаг_бангладеш", "1F1E7-1F1E9"},
    {"флаг_бельгия", "1F1E7-1F1EA"},
    {"флаг_буркина_фасо", "1F1E7-1F1EB"},
    {"флаг_болгария", "1F1E7-1F1EC"},
    {"флаг_бахрейн", "1F1E7-1F1ED"},
    {"флаг_бурунди", "1F1E7-1F1EE"},
    {"флаг_бенин", "1F1E7-1F1EF"},
    {"флаг_сен_бартелеми", "1F1E7-1F1F1"},
    {"флаг_бермудские_о_ва", "1F1E7-1F1F2"},
    {"флаг_бруней", "1F1E7-1F1F3"},
    {"флаг_боливия", "1F1E7-1F1F4"},
    {"флаг_бонэйр_синт_эстатиус_и_саба", "1F1E7-1F1F6"},
    {"флаг_бразилия", "1F1E7-1F1F7"},
    {"флаг_багамы", "1F1E7-1F1F8"},
    {"флаг_бутан", "1F1E7-1F1F9"},
    {"флаг_о_в_буве", "1F1E7-1F1FB"},
    {"флаг_ботсвана", "1F1E7-1F1FC"},
    {"флаг_беларусь", "1F1E7-1F1FE"},
    {"флаг_белиз", "1F1E7-1F1FF"},
    {"флаг_канада", "1F1E8-1F1E6"},
    {"флаг_кокосовые_о_ва", "1F1E8-1F1E8"},
    {"флаг_конго_киншаса", "1F1E8-1F1E9"},
    {"флаг_центрально_африканская_республика", "1F1E8-1F1EB"},
    {"флаг_конго_браззавиль", "1F1E8-1F1EC"},
    {"флаг_швейцария", "1F1E8-1F1ED"},
    {"флаг_кот_дивуар", "1F1E8-1F1EE"},
    {"флаг_о_ва_кука", "1F1E8-1F1F0"},
    {"флаг_чили", "1F1E8-1F1F1"},
    {"флаг_камерун", "1F1E8-1F1F2"},
    {"флаг_китай", "1F1E8-1F1F3"},
    {"флаг_колумбия", "1F1E8-1F1F4"},
    {"флаг_о_в_клиппертон", "1F1E8-1F1F5"},
    {"флаг_сарк", "1F1E8-1F1F6"},
    {"флаг_коста_рика", "1F1E8-1F1F7"},
    {"флаг_куба", "1F1E8-1F1FA"},
    {"флаг_кабо_верде", "1F1E8-1F1FB"},
    {"флаг_кюрасао", "1F1E8-1F1FC"},
    {"флаг_о_в_рождества", "1F1E8-1F1FD"},
    {"флаг_кипр", "1F1E8-1F1FE"},
    {"флаг_чехия", "1F1E8-1F1FF"},
    {"флаг_германия", "1F1E9-1F1EA"},
    {"флаг_диего_гарсия", "1F1E9-1F1EC"},
    {"флаг_джибути", "1F1E9-1F1EF"},
    {"флаг_дания", "1F1E9-1F1F0"},
    {"флаг_доминика", "1F1E9-1F1F2"},
    {"флаг_доминиканская_республика", "1F1E9-1F1F4"},
    {"флаг_алжир", "1F1E9-1F1FF"},
    {"флаг_сеута_и_мелилья", "1F1EA-1F1E6"},
    {"флаг_эквадор", "1F1EA-1F1E8"},
    {"флаг_эстония", "1F1EA-1F1EA"},
    {"флаг_египет", "1F1EA-1F1EC"},
    {"флаг_западная_сахара", "1F1EA-1F1ED"},
    {"флаг_эритрея", "1F1EA-1F1F7"},
    {"флаг_испания", "1F1EA-1F1F8"},
    {"флаг_эфиопия", "1F1EA-1F1F9"},
    {"флаг_европейский_союз", "1F1EA-1F1FA"},
    {"флаг_финляндия", "1F1EB-1F1EE"},
    {"флаг_фиджи", "1F1EB-1F1EF"},
    {"флаг_фолклендские_о_ва", "1F1EB-1F1F0"},
    {"флаг_федеративные_штаты_микронезии", "1F1EB-1F1F2"},
    {"флаг_фарерские_о_ва", "1F1EB-1F1F4"},
    {"флаг_франция", "1F1EB-1F1F7"},
    {"флаг_габон", "1F1EC-1F1E6"},
    {"флаг_великобритания", "1F1EC-1F1E7"},
    {"флаг_гренада", "1F1EC-1F1E9"},
    {"флаг_грузия", "1F1EC-1F1EA"},
    {"флаг_французская_гвиана", "1F1EC-1F1EB"},
    {"флаг_гернси", "1F1EC-1F1EC"},
    {"флаг_гана", "1F1EC-1F1ED"},
    {"флаг_гибралтар", "1F1EC-1F1EE"},
    {"флаг_гренландия", "1F1EC-1F1F1"},
    {"флаг_гамбия", "1F1EC-1F1F2"},
    {"флаг_гвинея", "1F1EC-1F1F3"},
    {"флаг_гваделупа", "1F1EC-1F1F5"},
    {"флаг_экваториальная_гвинея", "1F1EC-1F1F6"},
    {"флаг_греция", "1F1EC-1F1F7"},
    {"флаг_южная_георгия_и_южные_сандвичевы_о_ва", "1F1EC-1F1F8"},
    {"флаг_гватемала", "1F1EC-1F1F9"},
    {"флаг_гуам", "1F1EC-1F1FA"},
    {"флаг_гвинея_бисау", "1F1EC-1F1FC"},
    {"флаг_гайана", "1F1EC-1F1FE"},
    {"флаг_гонконг_сар", "1F1ED-1F1F0"},
    {"флаг_о_ва_херд_и_макдональд", "1F1ED-1F1F2"},
    {"флаг_гондурас", "1F1ED-1F1F3"},
    {"флаг_хорватия", "1F1ED-1F1F7"},
    {"флаг_гаити", "1F1ED-1F1F9"},
    {"флаг_венгрия", "1F1ED-1F1FA"},
    {"флаг_канарские_о_ва", "1F1EE-1F1E8"},
    {"флаг_индонезия", "1F1EE-1F1E9"},
    {"флаг_ирландия", "1F1EE-1F1EA"},
    {"флаг_израиль", "1F1EE-1F1F1"},
    {"флаг_о_в_мэн", "1F1EE-1F1F2"},
    {"флаг_индия", "1F1EE-1F1F3"},
    {"флаг_британская_территория_в_индийском_океане", "1F1EE-1F1F4"},
    {"флаг_ирак", "1F1EE-1F1F6"},
    {"флаг_иран", "1F1EE-1F1F7"},
    {"флаг_исландия", "1F1EE-1F1F8"},
    {"флаг_италия", "1F1EE-1F1F9"},
    {"флаг_джерси", "1F1EF-1F1EA"},
    {"флаг_ямайка", "1F1EF-1F1F2"},
    {"флаг_иордания", "1F1EF-1F1F4"},
    {"флаг_япония", "1F1EF-1F1F5"},
    {"флаг_кения", "1F1F0-1F1EA"},
    {"флаг_киргизия", "1F1F0-1F1EC"},
    {"флаг_камбоджа", "1F1F0-1F1ED"},
    {"флаг_кирибати", "1F1F0-1F1EE"},
    {"флаг_коморы", "1F1F0-1F1F2"},
    {"флаг_сент_китс_и_невис", "1F1F0-1F1F3"},
    {"флаг_кндр", "1F1F0-1F1F5"},
    {"флаг_республика_корея", "1F1F0-1F1F7"},
    {"флаг_кувейт", "1F1F0-1F1FC"},
    {"флаг_о_ва_кайман", "1F1F0-1F1FE"},
    {"флаг_казахстан", "1F1F0-1F1FF"},
    {"флаг_лаос", "1F1F1-1F1E6"},
    {"флаг_ливан", "1F1F1-1F1E7"},
    {"флаг_сент_люсия", "1F1F1-1F1E8"},
    {"флаг_лихтенштейн", "1F1F1-1F1EE"},
    {"флаг_шри_ланка", "1F1F1-1F1F0"},
    {"флаг_либерия", "1F1F1-1F1F7"},
    {"флаг_лесото", "1F1F1-1F1F8"},
    {"флаг_литва", "1F1F1-1F1F9"},
    {"флаг_люксембург", "1F1F1-1F1FA"},
    {"флаг_латвия", "1F1F1-1F1FB"},
    {"флаг_ливия", "1F1F1-1F1FE"},
    {"флаг_марокко", "1F1F2-1F1E6"},
    {"флаг_монако", "1F1F2-1F1E8"},
    {"флаг_молдова", "1F1F2-1F1E9"},
    {"флаг_черногория", "1F1F2-1F1EA"},
    {"флаг_сен_мартен", "1F1F2-1F1EB"},
    {"флаг_мадагаскар", "1F1F2-1F1EC"},
    {"флаг_маршалловы_о_ва", "1F1F2-1F1ED"},
    {"флаг_северная_македония", "1F1F2-1F1F0"},
    {"флаг_мали", "1F1F2-1F1F1"},
    {"флаг_мьянма_бирма", "1F1F2-1F1F2"},
    {"флаг_монголия", "1F1F2-1F1F3"},
    {"флаг_макао_сар", "1F1F2-1F1F4"},
    {"флаг_северные_марианские_о_ва", "1F1F2-1F1F5"},
    {"флаг_мартиника", "1F1F2-1F1F6"},
    {"флаг_мавритания", "1F1F2-1F1F7"},
    {"флаг_монтсеррат", "1F1F2-1F1F8"},
    {"флаг_мальта", "1F1F2-1F1F9"},
    {"флаг_маврикий", "1F1F2-1F1FA"},
    {"флаг_мальдивы", "1F1F2-1F1FB"},
    {"флаг_малави", "1F1F2-1F1FC"},
    {"флаг_мексика", "1F1F2-1F1FD"},
    {"флаг_малайзия", "1F1F2-1F1FE"},
    {"флаг_мозамбик", "1F1F2-1F1FF"},
    {"флаг_намибия", "1F1F3-1F1E6"},
    {"флаг_новая_каледония", "1F1F3-1F1E8"},
    {"флаг_нигер", "1F1F3-1F1EA"},
    {"флаг_о_в_норфолк", "1F1F3-1F1EB"},
    {"флаг_нигерия", "1F1F3-1F1EC"},
    {"флаг_никарагуа", "1F1F3-1F1EE"},
    {"флаг_нидерланды", "1F1F3-1F1F1"},
    {"флаг_норвегия", "1F1F3-1F1F4"},
    {"флаг_непал", "1F1F3-1F1F5"},
    {"флаг_науру", "1F1F3-1F1F7"},
    {"флаг_ниуэ", "1F1F3-1F1FA"},
    {"флаг_новая_зеландия", "1F1F3-1F1FF"},
    {"флаг_оман", "1F1F4-1F1F2"},
    {"флаг_панама", "1F1F5-1F1E6"},
    {"флаг_перу", "1F1F5-1F1EA"},
    {"флаг_французская_полинезия", "1F1F5-1F1EB"},
    {"флаг_папуа_новая_гвинея", "1F1F5-1F1EC"},
    {"флаг_филиппины", "1F1F5-1F1ED"},
    {"флаг_пакистан", "1F1F5-1F1F0"},
    {"флаг_польша", "1F1F5-1F1F1"},
    {"флаг_сен_пьер_и_микелон", "1F1F5-1F1F2"},
    {"флаг_о_ва_питкэрн", "1F1F5-1F1F3"},
    {"флаг_пуэрто_рико", "1F1F5-1F1F7"},
    {"флаг_палестинские_территории", "1F1F5-1F1F8"},
    {"флаг_португалия", "1F1F5-1F1F9"},
    {"флаг_палау", "1F1F5-1F1FC"},
    {"флаг_парагвай", "1F1F5-1F1FE"},
    {"флаг_катар", "1F1F6-1F1E6"},
    {"флаг_реюньон", "1F1F7-1F1EA"},
    {"флаг_румыния", "1F1F7-1F1F4"},
    {"флаг_сербия", "1F1F7-1F1F8"},
    {"флаг_россия", "1F1F7-1F1FA"},
    {"флаг_руанда", "1F1F7-1F1FC"},
    {"флаг_саудовская_аравия", "1F1F8-1F1E6"},
    {"флаг_соломоновы_о_ва", "1F1F8-1F1E7"},
    {"флаг_сейшельские_о_ва", "1F1F8-1F1E8"},
    {"флаг_судан", "1F1F8-1F1E9"},
    {"флаг_швеция", "1F1F8-1F1EA"},
    {"флаг_сингапур", "1F1F8-1F1EC"},
    {"флаг_о_в_св_елены", "1F1F8-1F1ED"},
    {"флаг_словения", "1F1F8-1F1EE"},
    {"флаг_шпицберген_и_ян_майен", "1F1F8-1F1EF"},
    {"флаг_словакия", "1F1F8-1F1F0"},
    {"флаг_сьерра_леоне", "1F1F8-1F1F1"},
    {"флаг_сан_марино", "1F1F8-1F1F2"},
    {"флаг_сенегал", "1F1F8-1F1F3"},
    {"флаг_сомали", "1F1F8-1F1F4"},
    {"флаг_суринам", "1F1F8-1F1F7"},
    {"флаг_южный_судан", "1F1F8-1F1F8"},
    {"флаг_сан_томе_и_принсипи", "1F1F8-1F1F9"},
    {"флаг_сальвадор", "1F1F8-1F1FB"},
    {"флаг_синт_мартен", "1F1F8-1F1FD"},
    {"флаг_сирия", "1F1F8-1F1FE"},
    {"флаг_эсватини", "1F1F8-1F1FF"},
    {"флаг_тристан_да_кунья", "1F1F9-1F1E6"},
    {"флаг_тёркс_и_кайкос", "1F1F9-1F1E8"},
    {"флаг_чад", "1F1F9-1F1E9"},
    {"флаг_французские_южные_территории", "1F1F9-1F1EB"},
    {"флаг_того", "1F1F9-1F1EC"},
    {"флаг_таиланд", "1F1F9-1F1ED"},
    {"флаг_таджикистан", "1F1F9-1F1EF"},
    {"флаг_токелау", "1F1F9-1F1F0"},
    {"флаг_восточный_тимор", "1F1F9-1F1F1"},
    {"флаг_туркменистан", "1F1F9-1F1F2"},
    {"флаг_тунис", "1F1F9-1F1F3"},
    {"флаг_тонга", "1F1F9-1F1F4"},
    {"флаг_турция", "1F1F9-1F1F7"},
    {"флаг_тринидад_и_тобаго", "1F1F9-1F1F9"},
    {"флаг_тувалу", "1F1F9-1F1FB"},
    {"флаг_тайвань", "1F1F9-1F1FC"},
    {"флаг_танзания", "1F1F9-1F1FF"},
    {"флаг_украина", "1F1FA-1F1E6"},
    {"флаг_уганда", "1F1FA-1F1EC"},
    {"флаг_внешние_малые_о_ва_сша", "1F1FA-1F1F2"},
    {"флаг_организация_объединенных_наций", "1F1FA-1F1F3"},
    {"флаг_соединенные_штаты", "1F1FA-1F1F8"},
    {"флаг_уругвай", "1F1FA-1F1FE"},
    {"флаг_узбекистан", "1F1FA-1F1FF"},
    {"флаг_ватикан", "1F1FB-1F1E6"},
    {"флаг_сент_винсент_и_гренадины", "1F1FB-1F1E8"},
    {"флаг_венесуэла", "1F1FB-1F1EA"},
    {"флаг_виргинские_о_ва_великобритания", "1F1FB-1F1EC"},
    {"флаг_виргинские_о_ва_сша", "1F1FB-1F1EE"},
    {"флаг_вьетнам", "1F1FB-1F1F3"},
    {"флаг_вануату", "1F1FB-1F1FA"},
    {"флаг_уоллис_и_футуна", "1F1FC-1F1EB"},
    {"флаг_самоа", "1F1FC-1F1F8"},
    {"флаг_косово", "1F1FD-1F1F0"},
    {"флаг_йемен", "1F1FE-1F1EA"},
    {"флаг_майотта", "1F1FE-1F1F9"},
    {"флаг_южно_африканская_республика", "1F1FF-1F1E6"},
    {"флаг_замбия", "1F1FF-1F1F2"},
    {"флаг_зимбабве", "1F1FF-1F1FC"},
    {"флаг_англия", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"флаг_шотландия", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"флаг_уэльс", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"влюбленная_пара_мужчина_мужчина", "1F468-200D-2764-FE0F-200D-1F468"},
    {"влюбленная_пара_мужчина_мужчина_цвет1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"влюбленная_пара_мужчина_мужчина_цвет1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"влюбленная_пара_мужчина_мужчина_цвет1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"влюбленная_пара_мужчина_мужчина_цвет1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"влюбленная_пара_мужчина_мужчина_цвет1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"влюбленная_пара_мужчина_мужчина_цвет2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"влюбленная_пара_мужчина_мужчина_цвет2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"влюбленная_пара_мужчина_мужчина_цвет2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"влюбленная_пара_мужчина_мужчина_цвет2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"влюбленная_пара_мужчина_мужчина_цвет2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"влюбленная_пара_мужчина_мужчина_цвет3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"влюбленная_пара_мужчина_мужчина_цвет3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"влюбленная_пара_мужчина_мужчина_цвет3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"влюбленная_пара_мужчина_мужчина_цвет3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"влюбленная_пара_мужчина_мужчина_цвет3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"влюбленная_пара_мужчина_мужчина_цвет4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"влюбленная_пара_мужчина_мужчина_цвет4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"влюбленная_пара_мужчина_мужчина_цвет4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"влюбленная_пара_мужчина_мужчина_цвет4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"влюбленная_пара_мужчина_мужчина_цвет4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"влюбленная_пара_мужчина_мужчина_цвет5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"влюбленная_пара_мужчина_мужчина_цвет5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"влюбленная_пара_мужчина_мужчина_цвет5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"влюбленная_пара_мужчина_мужчина_цвет5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"влюбленная_пара_мужчина_мужчина_цвет5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"поцелуй_мужчина_мужчина", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"поцелуй_мужчина_мужчина_цвет1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"поцелуй_мужчина_мужчина_цвет1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"поцелуй_мужчина_мужчина_цвет1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"поцелуй_мужчина_мужчина_цвет1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"поцелуй_мужчина_мужчина_цвет1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"поцелуй_мужчина_мужчина_цвет2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"поцелуй_мужчина_мужчина_цвет2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"поцелуй_мужчина_мужчина_цвет2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"поцелуй_мужчина_мужчина_цвет2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"поцелуй_мужчина_мужчина_цвет2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"поцелуй_мужчина_мужчина_цвет3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"поцелуй_мужчина_мужчина_цвет3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"поцелуй_мужчина_мужчина_цвет3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"поцелуй_мужчина_мужчина_цвет3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"поцелуй_мужчина_мужчина_цвет3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"поцелуй_мужчина_мужчина_цвет4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"поцелуй_мужчина_мужчина_цвет4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"поцелуй_мужчина_мужчина_цвет4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"поцелуй_мужчина_мужчина_цвет4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"поцелуй_мужчина_мужчина_цвет4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"поцелуй_мужчина_мужчина_цвет5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"поцелуй_мужчина_мужчина_цвет5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"поцелуй_мужчина_мужчина_цвет5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"поцелуй_мужчина_мужчина_цвет5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"поцелуй_мужчина_мужчина_цвет5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"семья_мужчина_мальчик", "1F468-200D-1F466"},
    {"семья_мужчина_мальчик_мальчик", "1F468-200D-1F466-200D-1F466"},
    {"семья_мужчина_девочка", "1F468-200D-1F467"},
    {"семья_мужчина_девочка_мальчик", "1F468-200D-1F467-200D-1F466"},
    {"семья_мужчина_девочка_девочка", "1F468-200D-1F467-200D-1F467"},
    {"семья_мужчина_мужчина_мальчик", "1F468-200D-1F468-200D-1F466"},
    {"семья_мужчина_мужчина_мальчик_мальчик", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"семья_мужчина_мужчина_девочка", "1F468-200D-1F468-200D-1F467"},
    {"семья_мужчина_мужчина_девочка_мальчик", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"семья_мужчина_мужчина_девочка_девочка", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"семья_мужчина_женщина_мальчик", "1F468-200D-1F469-200D-1F466"},
    {"семья_мужчина_женщина_мальчик_мальчик", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"семья_мужчина_женщина_девочка", "1F468-200D-1F469-200D-1F467"},
    {"семья_мужчина_женщина_девочка_мальчик", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"семья_мужчина_женщина_девочка_девочка", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"влюбленная_пара_женщина_мужчина", "1F469-200D-2764-FE0F-200D-1F468"},
    {"влюбленная_пара_женщина_мужчина_цвет1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"влюбленная_пара_женщина_мужчина_цвет1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"влюбленная_пара_женщина_мужчина_цвет1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"влюбленная_пара_женщина_мужчина_цвет1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"влюбленная_пара_женщина_мужчина_цвет1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"влюбленная_пара_женщина_мужчина_цвет2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"влюбленная_пара_женщина_мужчина_цвет2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"влюбленная_пара_женщина_мужчина_цвет2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"влюбленная_пара_женщина_мужчина_цвет2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"влюбленная_пара_женщина_мужчина_цвет2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"влюбленная_пара_женщина_мужчина_цвет3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"влюбленная_пара_женщина_мужчина_цвет3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"влюбленная_пара_женщина_мужчина_цвет3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"влюбленная_пара_женщина_мужчина_цвет3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"влюбленная_пара_женщина_мужчина_цвет3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"влюбленная_пара_женщина_мужчина_цвет4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"влюбленная_пара_женщина_мужчина_цвет4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"влюбленная_пара_женщина_мужчина_цвет4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"влюбленная_пара_женщина_мужчина_цвет4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"влюбленная_пара_женщина_мужчина_цвет4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"влюбленная_пара_женщина_мужчина_цвет5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"влюбленная_пара_женщина_мужчина_цвет5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"влюбленная_пара_женщина_мужчина_цвет5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"влюбленная_пара_женщина_мужчина_цвет5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"влюбленная_пара_женщина_мужчина_цвет5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"влюбленная_пара_женщина_женщина", "1F469-200D-2764-FE0F-200D-1F469"},
    {"влюбленная_пара_женщина_женщина_цвет1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"влюбленная_пара_женщина_женщина_цвет1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"влюбленная_пара_женщина_женщина_цвет1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"влюбленная_пара_женщина_женщина_цвет1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"влюбленная_пара_женщина_женщина_цвет1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"влюбленная_пара_женщина_женщина_цвет2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"влюбленная_пара_женщина_женщина_цвет2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"влюбленная_пара_женщина_женщина_цвет2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"влюбленная_пара_женщина_женщина_цвет2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"влюбленная_пара_женщина_женщина_цвет2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"влюбленная_пара_женщина_женщина_цвет3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"влюбленная_пара_женщина_женщина_цвет3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"влюбленная_пара_женщина_женщина_цвет3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"влюбленная_пара_женщина_женщина_цвет3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"влюбленная_пара_женщина_женщина_цвет3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"влюбленная_пара_женщина_женщина_цвет4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"влюбленная_пара_женщина_женщина_цвет4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"влюбленная_пара_женщина_женщина_цвет4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"влюбленная_пара_женщина_женщина_цвет4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"влюбленная_пара_женщина_женщина_цвет4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"влюбленная_пара_женщина_женщина_цвет5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"влюбленная_пара_женщина_женщина_цвет5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"влюбленная_пара_женщина_женщина_цвет5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"влюбленная_пара_женщина_женщина_цвет5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"влюбленная_пара_женщина_женщина_цвет5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"поцелуй_женщина_мужчина", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"поцелуй_женщина_мужчина_цвет1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"поцелуй_женщина_мужчина_цвет1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"поцелуй_женщина_мужчина_цвет1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"поцелуй_женщина_мужчина_цвет1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"поцелуй_женщина_мужчина_цвет1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"поцелуй_женщина_мужчина_цвет2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"поцелуй_женщина_мужчина_цвет2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"поцелуй_женщина_мужчина_цвет2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"поцелуй_женщина_мужчина_цвет2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"поцелуй_женщина_мужчина_цвет2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"поцелуй_женщина_мужчина_цвет3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"поцелуй_женщина_мужчина_цвет3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"поцелуй_женщина_мужчина_цвет3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"поцелуй_женщина_мужчина_цвет3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"поцелуй_женщина_мужчина_цвет3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"поцелуй_женщина_мужчина_цвет4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"поцелуй_женщина_мужчина_цвет4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"поцелуй_женщина_мужчина_цвет4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"поцелуй_женщина_мужчина_цвет4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"поцелуй_женщина_мужчина_цвет4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"поцелуй_женщина_мужчина_цвет5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"поцелуй_женщина_мужчина_цвет5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"поцелуй_женщина_мужчина_цвет5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"поцелуй_женщина_мужчина_цвет5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"поцелуй_женщина_мужчина_цвет5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"поцелуй_женщина_женщина", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"поцелуй_женщина_женщина_цвет1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"поцелуй_женщина_женщина_цвет1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"поцелуй_женщина_женщина_цвет1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"поцелуй_женщина_женщина_цвет1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"поцелуй_женщина_женщина_цвет1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"поцелуй_женщина_женщина_цвет2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"поцелуй_женщина_женщина_цвет2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"поцелуй_женщина_женщина_цвет2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"поцелуй_женщина_женщина_цвет2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"поцелуй_женщина_женщина_цвет2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"поцелуй_женщина_женщина_цвет3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"поцелуй_женщина_женщина_цвет3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"поцелуй_женщина_женщина_цвет3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"поцелуй_женщина_женщина_цвет3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"поцелуй_женщина_женщина_цвет3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"поцелуй_женщина_женщина_цвет4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"поцелуй_женщина_женщина_цвет4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"поцелуй_женщина_женщина_цвет4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"поцелуй_женщина_женщина_цвет4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"поцелуй_женщина_женщина_цвет4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"поцелуй_женщина_женщина_цвет5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"поцелуй_женщина_женщина_цвет5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"поцелуй_женщина_женщина_цвет5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"поцелуй_женщина_женщина_цвет5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"поцелуй_женщина_женщина_цвет5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"семья_женщина_мальчик", "1F469-200D-1F466"},
    {"семья_женщина_мальчик_мальчик", "1F469-200D-1F466-200D-1F466"},
    {"семья_женщина_девочка", "1F469-200D-1F467"},
    {"семья_женщина_девочка_мальчик", "1F469-200D-1F467-200D-1F466"},
    {"семья_женщина_девочка_девочка", "1F469-200D-1F467-200D-1F467"},
    {"семья_женщина_женщина_мальчик", "1F469-200D-1F469-200D-1F466"},
    {"семья_женщина_женщина_мальчик_мальчик", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"семья_женщина_женщина_девочка", "1F469-200D-1F469-200D-1F467"},
    {"семья_женщина_женщина_девочка_мальчик", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"семья_женщина_женщина_девочка_девочка", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"люди_держащиеся_за_руки", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"люди_держащиеся_за_руки_цвет1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"люди_держащиеся_за_руки_цвет1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"люди_держащиеся_за_руки_цвет1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"люди_держащиеся_за_руки_цвет1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"люди_держащиеся_за_руки_цвет1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"люди_держащиеся_за_руки_цвет2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"люди_держащиеся_за_руки_цвет2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"люди_держащиеся_за_руки_цвет2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"люди_держащиеся_за_руки_цвет2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"люди_держащиеся_за_руки_цвет2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"люди_держащиеся_за_руки_цвет3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"люди_держащиеся_за_руки_цвет3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"люди_держащиеся_за_руки_цвет3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"люди_держащиеся_за_руки_цвет3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"люди_держащиеся_за_руки_цвет3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"люди_держащиеся_за_руки_цвет4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"люди_держащиеся_за_руки_цвет4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"люди_держащиеся_за_руки_цвет4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"люди_держащиеся_за_руки_цвет4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"люди_держащиеся_за_руки_цвет4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"люди_держащиеся_за_руки_цвет5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"люди_держащиеся_за_руки_цвет5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"люди_держащиеся_за_руки_цвет5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"люди_держащиеся_за_руки_цвет5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"люди_держащиеся_за_руки_цвет5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"семья_двое_взрослых_и_ребенок", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"семья_двое_взрослых_и_двое_детей", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"семья_взрослый_и_ребенок", "1F9D1-200D-1F9D2"},
    {"семья_взрослый_и_двое_детей", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"человек_бежит_направленные_вправо", "1F3C3-200D-27A1-FE0F"},
    {"человек_бежит_направленные_вправо_цвет1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"человек_бежит_направленные_вправо_цвет2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"человек_бежит_направленные_вправо_цвет3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"человек_бежит_направленные_вправо_цвет4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"человек_бежит_направленные_вправо_цвет5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"врач", "1F468-200D-2695-FE0F"},
    {"врач_цвет1", "1F468-1F3FB-200D-2695-FE0F"},
    {"врач_цвет2", "1F468-1F3FC-200D-2695-FE0F"},
    {"врач_цвет3", "1F468-1F3FD-200D-2695-FE0F"},
    {"врач_цвет4", "1F468-1F3FE-200D-2695-FE0F"},
    {"врач_цвет5", "1F468-1F3FF-200D-2695-FE0F"},
    {"мужчина_судья", "1F468-200D-2696-FE0F"},
    {"мужчина_судья_цвет1", "1F468-1F3FB-200D-2696-FE0F"},
    {"мужчина_судья_цвет2", "1F468-1F3FC-200D-2696-FE0F"},
    {"мужчина_судья_цвет3", "1F468-1F3FD-200D-2696-FE0F"},
    {"мужчина_судья_цвет4", "1F468-1F3FE-200D-2696-FE0F"},
    {"мужчина_судья_цвет5", "1F468-1F3FF-200D-2696-FE0F"},
    {"мужчина_пилот", "1F468-200D-2708-FE0F"},
    {"мужчина_пилот_цвет1", "1F468-1F3FB-200D-2708-FE0F"},
    {"мужчина_пилот_цвет2", "1F468-1F3FC-200D-2708-FE0F"},
    {"мужчина_пилот_цвет3", "1F468-1F3FD-200D-2708-FE0F"},
    {"мужчина_пилот_цвет4", "1F468-1F3FE-200D-2708-FE0F"},
    {"мужчина_пилот_цвет5", "1F468-1F3FF-200D-2708-FE0F"},
    {"мужчина_фермер", "1F468-200D-1F33E"},
    {"мужчина_фермер_цвет1", "1F468-1F3FB-200D-1F33E"},
    {"мужчина_фермер_цвет2", "1F468-1F3FC-200D-1F33E"},
    {"мужчина_фермер_цвет3", "1F468-1F3FD-200D-1F33E"},
    {"мужчина_фермер_цвет4", "1F468-1F3FE-200D-1F33E"},
    {"мужчина_фермер_цвет5", "1F468-1F3FF-200D-1F33E"},
    {"мужчина_повар", "1F468-200D-1F373"},
    {"мужчина_повар_цвет1", "1F468-1F3FB-200D-1F373"},
    {"мужчина_повар_цвет2", "1F468-1F3FC-200D-1F373"},
    {"мужчина_повар_цвет3", "1F468-1F3FD-200D-1F373"},
    {"мужчина_повар_цвет4", "1F468-1F3FE-200D-1F373"},
    {"мужчина_повар_цвет5", "1F468-1F3FF-200D-1F373"},
    {"кормящий_мужчина", "1F468-200D-1F37C"},
    {"кормящий_мужчина_цвет1", "1F468-1F3FB-200D-1F37C"},
    {"кормящий_мужчина_цвет2", "1F468-1F3FC-200D-1F37C"},
    {"кормящий_мужчина_цвет3", "1F468-1F3FD-200D-1F37C"},
    {"кормящий_мужчина_цвет4", "1F468-1F3FE-200D-1F37C"},
    {"кормящий_мужчина_цвет5", "1F468-1F3FF-200D-1F37C"},
    {"студент", "1F468-200D-1F393"},
    {"студент_цвет1", "1F468-1F3FB-200D-1F393"},
    {"студент_цвет2", "1F468-1F3FC-200D-1F393"},
    {"студент_цвет3", "1F468-1F3FD-200D-1F393"},
    {"студент_цвет4", "1F468-1F3FE-200D-1F393"},
    {"студент_цвет5", "1F468-1F3FF-200D-1F393"},
    {"мужчина_певец", "1F468-200D-1F3A4"},
    {"мужчина_певец_цвет1", "1F468-1F3FB-200D-1F3A4"},
    {"мужчина_певец_цвет2", "1F468-1F3FC-200D-1F3A4"},
    {"мужчина_певец_цвет3", "1F468-1F3FD-200D-1F3A4"},
    {"мужчина_певец_цвет4", "1F468-1F3FE-200D-1F3A4"},
    {"мужчина_певец_цвет5", "1F468-1F3FF-200D-1F3A4"},
    {"мужчина_художник", "1F468-200D-1F3A8"},
    {"мужчина_художник_цвет1", "1F468-1F3FB-200D-1F3A8"},
    {"мужчина_художник_цвет2", "1F468-1F3FC-200D-1F3A8"},
    {"мужчина_художник_цвет3", "1F468-1F3FD-200D-1F3A8"},
    {"мужчина_художник_цвет4", "1F468-1F3FE-200D-1F3A8"},
    {"мужчина_художник_цвет5", "1F468-1F3FF-200D-1F3A8"},
    {"учитель", "1F468-200D-1F3EB"},
    {"учитель_цвет1", "1F468-1F3FB-200D-1F3EB"},
    {"учитель_цвет2", "1F468-1F3FC-200D-1F3EB"},
    {"учитель_цвет3", "1F468-1F3FD-200D-1F3EB"},
    {"учитель_цвет4", "1F468-1F3FE-200D-1F3EB"},
    {"учитель_цвет5", "1F468-1F3FF-200D-1F3EB"},
    {"мужчина_рабочий", "1F468-200D-1F3ED"},
    {"мужчина_рабочий_цвет1", "1F468-1F3FB-200D-1F3ED"},
    {"мужчина_рабочий_цвет2", "1F468-1F3FC-200D-1F3ED"},
    {"мужчина_рабочий_цвет3", "1F468-1F3FD-200D-1F3ED"},
    {"мужчина_рабочий_цвет4", "1F468-1F3FE-200D-1F3ED"},
    {"мужчина_рабочий_цвет5", "1F468-1F3FF-200D-1F3ED"},
    {"мужчина_инженер", "1F468-200D-1F4BB"},
    {"мужчина_инженер_цвет1", "1F468-1F3FB-200D-1F4BB"},
    {"мужчина_инженер_цвет2", "1F468-1F3FC-200D-1F4BB"},
    {"мужчина_инженер_цвет3", "1F468-1F3FD-200D-1F4BB"},
    {"мужчина_инженер_цвет4", "1F468-1F3FE-200D-1F4BB"},
    {"мужчина_инженер_цвет5", "1F468-1F3FF-200D-1F4BB"},
    {"офисный_работник_мужчина", "1F468-200D-1F4BC"},
    {"офисный_работник_мужчина_цвет1", "1F468-1F3FB-200D-1F4BC"},
    {"офисный_работник_мужчина_цвет2", "1F468-1F3FC-200D-1F4BC"},
    {"офисный_работник_мужчина_цвет3", "1F468-1F3FD-200D-1F4BC"},
    {"офисный_работник_мужчина_цвет4", "1F468-1F3FE-200D-1F4BC"},
    {"офисный_работник_мужчина_цвет5", "1F468-1F3FF-200D-1F4BC"},
    {"мужчина_механик", "1F468-200D-1F527"},
    {"мужчина_механик_цвет1", "1F468-1F3FB-200D-1F527"},
    {"мужчина_механик_цвет2", "1F468-1F3FC-200D-1F527"},
    {"мужчина_механик_цвет3", "1F468-1F3FD-200D-1F527"},
    {"мужчина_механик_цвет4", "1F468-1F3FE-200D-1F527"},
    {"мужчина_механик_цвет5", "1F468-1F3FF-200D-1F527"},
    {"мужчина_ученый", "1F468-200D-1F52C"},
    {"мужчина_ученый_цвет1", "1F468-1F3FB-200D-1F52C"},
    {"мужчина_ученый_цвет2", "1F468-1F3FC-200D-1F52C"},
    {"мужчина_ученый_цвет3", "1F468-1F3FD-200D-1F52C"},
    {"мужчина_ученый_цвет4", "1F468-1F3FE-200D-1F52C"},
    {"мужчина_ученый_цвет5", "1F468-1F3FF-200D-1F52C"},
    {"мужчина_космонавт", "1F468-200D-1F680"},
    {"мужчина_космонавт_цвет1", "1F468-1F3FB-200D-1F680"},
    {"мужчина_космонавт_цвет2", "1F468-1F3FC-200D-1F680"},
    {"мужчина_космонавт_цвет3", "1F468-1F3FD-200D-1F680"},
    {"мужчина_космонавт_цвет4", "1F468-1F3FE-200D-1F680"},
    {"мужчина_космонавт_цвет5", "1F468-1F3FF-200D-1F680"},
    {"мужчина_пожарный", "1F468-200D-1F692"},
    {"мужчина_пожарный_цвет1", "1F468-1F3FB-200D-1F692"},
    {"мужчина_пожарный_цвет2", "1F468-1F3FC-200D-1F692"},
    {"мужчина_пожарный_цвет3", "1F468-1F3FD-200D-1F692"},
    {"мужчина_пожарный_цвет4", "1F468-1F3FE-200D-1F692"},
    {"мужчина_пожарный_цвет5", "1F468-1F3FF-200D-1F692"},
    {"мужчина_с_тростью_для_ходьбы", "1F468-200D-1F9AF"},
    {"мужчина_с_тростью_для_ходьбы_цвет1", "1F468-1F3FB-200D-1F9AF"},
    {"мужчина_с_тростью_для_ходьбы_цвет2", "1F468-1F3FC-200D-1F9AF"},
    {"мужчина_с_тростью_для_ходьбы_цвет3", "1F468-1F3FD-200D-1F9AF"},
    {"мужчина_с_тростью_для_ходьбы_цвет4", "1F468-1F3FE-200D-1F9AF"},
    {"мужчина_с_тростью_для_ходьбы_цвет5", "1F468-1F3FF-200D-1F9AF"},
    {"мужчина_с_тростью_для_ходьбы_направленные_вправо", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"мужчина_с_тростью_для_ходьбы_направленные_вправо_цвет1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"мужчина_с_тростью_для_ходьбы_направленные_вправо_цвет2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"мужчина_с_тростью_для_ходьбы_направленные_вправо_цвет3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"мужчина_с_тростью_для_ходьбы_направленные_вправо_цвет4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"мужчина_с_тростью_для_ходьбы_направленные_вправо_цвет5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"мужчина_в_электрическом_кресле_коляске", "1F468-200D-1F9BC"},
    {"мужчина_в_электрическом_кресле_коляске_цвет1", "1F468-1F3FB-200D-1F9BC"},
    {"мужчина_в_электрическом_кресле_коляске_цвет2", "1F468-1F3FC-200D-1F9BC"},
    {"мужчина_в_электрическом_кресле_коляске_цвет3", "1F468-1F3FD-200D-1F9BC"},
    {"мужчина_в_электрическом_кресле_коляске_цвет4", "1F468-1F3FE-200D-1F9BC"},
    {"мужчина_в_электрическом_кресле_коляске_цвет5", "1F468-1F3FF-200D-1F9BC"},
    {"мужчина_в_электрическом_кресле_коляске_направленные_вправо", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"мужчина_в_электрическом_кресле_коляске_направленные_вправо_цвет1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"мужчина_в_электрическом_кресле_коляске_направленные_вправо_цвет2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"мужчина_в_электрическом_кресле_коляске_направленные_вправо_цвет3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"мужчина_в_электрическом_кресле_коляске_направленные_вправо_цвет4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"мужчина_в_электрическом_кресле_коляске_направленные_вправо_цвет5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"мужчина_в_ручном_кресле_коляске", "1F468-200D-1F9BD"},
    {"мужчина_в_ручном_кресле_коляске_цвет1", "1F468-1F3FB-200D-1F9BD"},
    {"мужчина_в_ручном_кресле_коляске_цвет2", "1F468-1F3FC-200D-1F9BD"},
    {"мужчина_в_ручном_кресле_коляске_цвет3", "1F468-1F3FD-200D-1F9BD"},
    {"мужчина_в_ручном_кресле_коляске_цвет4", "1F468-1F3FE-200D-1F9BD"},
    {"мужчина_в_ручном_кресле_коляске_цвет5", "1F468-1F3FF-200D-1F9BD"},
    {"мужчина_в_ручном_кресле_коляске_направленные_вправо", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"мужчина_в_ручном_кресле_коляске_направленные_вправо_цвет1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"мужчина_в_ручном_кресле_коляске_направленные_вправо_цвет2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"мужчина_в_ручном_кресле_коляске_направленные_вправо_цвет3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"мужчина_в_ручном_кресле_коляске_направленные_вправо_цвет4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"мужчина_в_ручном_кресле_коляске_направленные_вправо_цвет5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"женщина_врач", "1F469-200D-2695-FE0F"},
    {"женщина_врач_цвет1", "1F469-1F3FB-200D-2695-FE0F"},
    {"женщина_врач_цвет2", "1F469-1F3FC-200D-2695-FE0F"},
    {"женщина_врач_цвет3", "1F469-1F3FD-200D-2695-FE0F"},
    {"женщина_врач_цвет4", "1F469-1F3FE-200D-2695-FE0F"},
    {"женщина_врач_цвет5", "1F469-1F3FF-200D-2695-FE0F"},
    {"женщина_судья", "1F469-200D-2696-FE0F"},
    {"женщина_судья_цвет1", "1F469-1F3FB-200D-2696-FE0F"},
    {"женщина_судья_цвет2", "1F469-1F3FC-200D-2696-FE0F"},
    {"женщина_судья_цвет3", "1F469-1F3FD-200D-2696-FE0F"},
    {"женщина_судья_цвет4", "1F469-1F3FE-200D-2696-FE0F"},
    {"женщина_судья_цвет5", "1F469-1F3FF-200D-2696-FE0F"},
    {"женщина_пилот", "1F469-200D-2708-FE0F"},
    {"женщина_пилот_цвет1", "1F469-1F3FB-200D-2708-FE0F"},
    {"женщина_пилот_цвет2", "1F469-1F3FC-200D-2708-FE0F"},
    {"женщина_пилот_цвет3", "1F469-1F3FD-200D-2708-FE0F"},
    {"женщина_пилот_цвет4", "1F469-1F3FE-200D-2708-FE0F"},
    {"женщина_пилот_цвет5", "1F469-1F3FF-200D-2708-FE0F"},
    {"женщина_фермер", "1F469-200D-1F33E"},
    {"женщина_фермер_цвет1", "1F469-1F3FB-200D-1F33E"},
    {"женщина_фермер_цвет2", "1F469-1F3FC-200D-1F33E"},
    {"женщина_фермер_цвет3", "1F469-1F3FD-200D-1F33E"},
    {"женщина_фермер_цвет4", "1F469-1F3FE-200D-1F33E"},
    {"женщина_фермер_цвет5", "1F469-1F3FF-200D-1F33E"},
    {"повариха", "1F469-200D-1F373"},
    {"повариха_цвет1", "1F469-1F3FB-200D-1F373"},
    {"повариха_цвет2", "1F469-1F3FC-200D-1F373"},
    {"повариха_цвет3", "1F469-1F3FD-200D-1F373"},
    {"повариха_цвет4", "1F469-1F3FE-200D-1F373"},
    {"повариха_цвет5", "1F469-1F3FF-200D-1F373"},
    {"кормящая_женщина", "1F469-200D-1F37C"},
    {"кормящая_женщина_цвет1", "1F469-1F3FB-200D-1F37C"},
    {"кормящая_женщина_цвет2", "1F469-1F3FC-200D-1F37C"},
    {"кормящая_женщина_цвет3", "1F469-1F3FD-200D-1F37C"},
    {"кормящая_женщина_цвет4", "1F469-1F3FE-200D-1F37C"},
    {"кормящая_женщина_цвет5", "1F469-1F3FF-200D-1F37C"},
    {"студентка", "1F469-200D-1F393"},
    {"студентка_цвет1", "1F469-1F3FB-200D-1F393"},
    {"студентка_цвет2", "1F469-1F3FC-200D-1F393"},
    {"студентка_цвет3", "1F469-1F3FD-200D-1F393"},
    {"студентка_цвет4", "1F469-1F3FE-200D-1F393"},
    {"студентка_цвет5", "1F469-1F3FF-200D-1F393"},
    {"певица", "1F469-200D-1F3A4"},
    {"певица_цвет1", "1F469-1F3FB-200D-1F3A4"},
    {"певица_цвет2", "1F469-1F3FC-200D-1F3A4"},
    {"певица_цвет3", "1F469-1F3FD-200D-1F3A4"},
    {"певица_цвет4", "1F469-1F3FE-200D-1F3A4"},
    {"певица_цвет5", "1F469-1F3FF-200D-1F3A4"},
    {"художница", "1F469-200D-1F3A8"},
    {"художница_цвет1", "1F469-1F3FB-200D-1F3A8"},
    {"художница_цвет2", "1F469-1F3FC-200D-1F3A8"},
    {"художница_цвет3", "1F469-1F3FD-200D-1F3A8"},
    {"художница_цвет4", "1F469-1F3FE-200D-1F3A8"},
    {"художница_цвет5", "1F469-1F3FF-200D-1F3A8"},
    {"учительница", "1F469-200D-1F3EB"},
    {"учительница_цвет1", "1F469-1F3FB-200D-1F3EB"},
    {"учительница_цвет2", "1F469-1F3FC-200D-1F3EB"},
    {"учительница_цвет3", "1F469-1F3FD-200D-1F3EB"},
    {"учительница_цвет4", "1F469-1F3FE-200D-1F3EB"},
    {"учительница_цвет5", "1F469-1F3FF-200D-1F3EB"},
    {"работница", "1F469-200D-1F3ED"},
    {"работница_цвет1", "1F469-1F3FB-200D-1F3ED"},
    {"работница_цвет2", "1F469-1F3FC-200D-1F3ED"},
    {"работница_цвет3", "1F469-1F3FD-200D-1F3ED"},
    {"работница_цвет4", "1F469-1F3FE-200D-1F3ED"},
    {"работница_цвет5", "1F469-1F3FF-200D-1F3ED"},
    {"женщина_инженер", "1F469-200D-1F4BB"},
    {"женщина_инженер_цвет1", "1F469-1F3FB-200D-1F4BB"},
    {"женщина_инженер_цвет2", "1F469-1F3FC-200D-1F4BB"},
    {"женщина_инженер_цвет3", "1F469-1F3FD-200D-1F4BB"},
    {"женщина_инженер_цвет4", "1F469-1F3FE-200D-1F4BB"},
    {"женщина_инженер_цвет5", "1F469-1F3FF-200D-1F4BB"},
    {"офисная_работница", "1F469-200D-1F4BC"},
    {"офисная_работница_цвет1", "1F469-1F3FB-200D-1F4BC"},
    {"офисная_работница_цвет2", "1F469-1F3FC-200D-1F4BC"},
    {"офисная_работница_цвет3", "1F469-1F3FD-200D-1F4BC"},
    {"офисная_работница_цвет4", "1F469-1F3FE-200D-1F4BC"},
    {"офисная_работница_цвет5", "1F469-1F3FF-200D-1F4BC"},
    {"женщина_механик", "1F469-200D-1F527"},
    {"женщина_механик_цвет1", "1F469-1F3FB-200D-1F527"},
    {"женщина_механик_цвет2", "1F469-1F3FC-200D-1F527"},
    {"женщина_механик_цвет3", "1F469-1F3FD-200D-1F527"},
    {"женщина_механик_цвет4", "1F469-1F3FE-200D-1F527"},
    {"женщина_механик_цвет5", "1F469-1F3FF-200D-1F527"},
    {"ученая", "1F469-200D-1F52C"},
    {"ученая_цвет1", "1F469-1F3FB-200D-1F52C"},
    {"ученая_цвет2", "1F469-1F3FC-200D-1F52C"},
    {"ученая_цвет3", "1F469-1F3FD-200D-1F52C"},
    {"ученая_цвет4", "1F469-1F3FE-200D-1F52C"},
    {"ученая_цвет5", "1F469-1F3FF-200D-1F52C"},
    {"женщина_космонавт", "1F469-200D-1F680"},
    {"женщина_космонавт_цвет1", "1F469-1F3FB-200D-1F680"},
    {"женщина_космонавт_цвет2", "1F469-1F3FC-200D-1F680"},
    {"женщина_космонавт_цвет3", "1F469-1F3FD-200D-1F680"},
    {"женщина_космонавт_цвет4", "1F469-1F3FE-200D-1F680"},
    {"женщина_космонавт_цвет5", "1F469-1F3FF-200D-1F680"},
    {"женщина_пожарный", "1F469-200D-1F692"},
    {"женщина_пожарный_цвет1", "1F469-1F3FB-200D-1F692"},
    {"женщина_пожарный_цвет2", "1F469-1F3FC-200D-1F692"},
    {"женщина_пожарный_цвет3", "1F469-1F3FD-200D-1F692"},
    {"женщина_пожарный_цвет4", "1F469-1F3FE-200D-1F692"},
    {"женщина_пожарный_цвет5", "1F469-1F3FF-200D-1F692"},
    {"женщина_с_тростью_для_ходьбы", "1F469-200D-1F9AF"},
    {"женщина_с_тростью_для_ходьбы_цвет1", "1F469-1F3FB-200D-1F9AF"},
    {"женщина_с_тростью_для_ходьбы_цвет2", "1F469-1F3FC-200D-1F9AF"},
    {"женщина_с_тростью_для_ходьбы_цвет3", "1F469-1F3FD-200D-1F9AF"},
    {"женщина_с_тростью_для_ходьбы_цвет4", "1F469-1F3FE-200D-1F9AF"},
    {"женщина_с_тростью_для_ходьбы_цвет5", "1F469-1F3FF-200D-1F9AF"},
    {"женщина_с_тростью_для_ходьбы_направленные_вправо", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"женщина_с_тростью_для_ходьбы_направленные_вправо_цвет1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"женщина_с_тростью_для_ходьбы_направленные_вправо_цвет2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"женщина_с_тростью_для_ходьбы_направленные_вправо_цвет3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"женщина_с_тростью_для_ходьбы_направленные_вправо_цвет4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"женщина_с_тростью_для_ходьбы_направленные_вправо_цвет5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"женщина_в_электрическом_кресле_коляске", "1F469-200D-1F9BC"},
    {"женщина_в_электрическом_кресле_коляске_цвет1", "1F469-1F3FB-200D-1F9BC"},
    {"женщина_в_электрическом_кресле_коляске_цвет2", "1F469-1F3FC-200D-1F9BC"},
    {"женщина_в_электрическом_кресле_коляске_цвет3", "1F469-1F3FD-200D-1F9BC"},
    {"женщина_в_электрическом_кресле_коляске_цвет4", "1F469-1F3FE-200D-1F9BC"},
    {"женщина_в_электрическом_кресле_коляске_цвет5", "1F469-1F3FF-200D-1F9BC"},
    {"женщина_в_электрическом_кресле_коляске_направленные_вправо", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"женщина_в_электрическом_кресле_коляске_направленные_вправо_цвет1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"женщина_в_электрическом_кресле_коляске_направленные_вправо_цвет2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"женщина_в_электрическом_кресле_коляске_направленные_вправо_цвет3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"женщина_в_электрическом_кресле_коляске_направленные_вправо_цвет4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"женщина_в_электрическом_кресле_коляске_направленные_вправо_цвет5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"женщина_в_ручном_кресле_коляске", "1F469-200D-1F9BD"},
    {"женщина_в_ручном_кресле_коляске_цвет1", "1F469-1F3FB-200D-1F9BD"},
    {"женщина_в_ручном_кресле_коляске_цвет2", "1F469-1F3FC-200D-1F9BD"},
    {"женщина_в_ручном_кресле_коляске_цвет3", "1F469-1F3FD-200D-1F9BD"},
    {"женщина_в_ручном_кресле_коляске_цвет4", "1F469-1F3FE-200D-1F9BD"},
    {"женщина_в_ручном_кресле_коляске_цвет5", "1F469-1F3FF-200D-1F9BD"},
    {"женщина_в_ручном_кресле_коляске_направленные_вправо", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"женщина_в_ручном_кресле_коляске_направленные_вправо_цвет1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"женщина_в_ручном_кресле_коляске_направленные_вправо_цвет2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"женщина_в_ручном_кресле_коляске_направленные_вправо_цвет3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"женщина_в_ручном_кресле_коляске_направленные_вправо_цвет4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"женщина_в_ручном_кресле_коляске_направленные_вправо_цвет5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"человек_идет_направленные_вправо", "1F6B6-200D-27A1-FE0F"},
    {"человек_идет_направленные_вправо_цвет1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"человек_идет_направленные_вправо_цвет2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"человек_идет_направленные_вправо_цвет3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"человек_идет_направленные_вправо_цвет4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"человек_идет_направленные_вправо_цвет5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"человек_на_коленях_направленные_вправо", "1F9CE-200D-27A1-FE0F"},
    {"человек_на_коленях_направленные_вправо_цвет1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"человек_на_коленях_направленные_вправо_цвет2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"человек_на_коленях_направленные_вправо_цвет3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"человек_на_коленях_направленные_вправо_цвет4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"человек_на_коленях_направленные_вправо_цвет5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"медицинский_работник", "1F9D1-200D-2695-FE0F"},
    {"медицинский_работник_цвет1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"медицинский_работник_цвет2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"медицинский_работник_цвет3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"медицинский_работник_цвет4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"медицинский_работник_цвет5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"судья", "1F9D1-200D-2696-FE0F"},
    {"судья_цвет1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"судья_цвет2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"судья_цвет3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"судья_цвет4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"судья_цвет5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"пилот", "1F9D1-200D-2708-FE0F"},
    {"пилот_цвет1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"пилот_цвет2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"пилот_цвет3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"пилот_цвет4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"пилот_цвет5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"фермер", "1F9D1-200D-1F33E"},
    {"фермер_цвет1", "1F9D1-1F3FB-200D-1F33E"},
    {"фермер_цвет2", "1F9D1-1F3FC-200D-1F33E"},
    {"фермер_цвет3", "1F9D1-1F3FD-200D-1F33E"},
    {"фермер_цвет4", "1F9D1-1F3FE-200D-1F33E"},
    {"фермер_цвет5", "1F9D1-1F3FF-200D-1F33E"},
    {"повар", "1F9D1-200D-1F373"},
    {"повар_цвет1", "1F9D1-1F3FB-200D-1F373"},
    {"повар_цвет2", "1F9D1-1F3FC-200D-1F373"},
    {"повар_цвет3", "1F9D1-1F3FD-200D-1F373"},
    {"повар_цвет4", "1F9D1-1F3FE-200D-1F373"},
    {"повар_цвет5", "1F9D1-1F3FF-200D-1F373"},
    {"кормящий_человек", "1F9D1-200D-1F37C"},
    {"кормящий_человек_цвет1", "1F9D1-1F3FB-200D-1F37C"},
    {"кормящий_человек_цвет2", "1F9D1-1F3FC-200D-1F37C"},
    {"кормящий_человек_цвет3", "1F9D1-1F3FD-200D-1F37C"},
    {"кормящий_человек_цвет4", "1F9D1-1F3FE-200D-1F37C"},
    {"кормящий_человек_цвет5", "1F9D1-1F3FF-200D-1F37C"},
    {"санта", "1F9D1-200D-1F384"},
    {"санта_цвет1", "1F9D1-1F3FB-200D-1F384"},
    {"санта_цвет2", "1F9D1-1F3FC-200D-1F384"},
    {"санта_цвет3", "1F9D1-1F3FD-200D-1F384"},
    {"санта_цвет4", "1F9D1-1F3FE-200D-1F384"},
    {"санта_цвет5", "1F9D1-1F3FF-200D-1F384"},
    {"учащийся", "1F9D1-200D-1F393"},
    {"учащийся_цвет1", "1F9D1-1F3FB-200D-1F393"},
    {"учащийся_цвет2", "1F9D1-1F3FC-200D-1F393"},
    {"учащийся_цвет3", "1F9D1-1F3FD-200D-1F393"},
    {"учащийся_цвет4", "1F9D1-1F3FE-200D-1F393"},
    {"учащийся_цвет5", "1F9D1-1F3FF-200D-1F393"},
    {"певец", "1F9D1-200D-1F3A4"},
    {"певец_цвет1", "1F9D1-1F3FB-200D-1F3A4"},
    {"певец_цвет2", "1F9D1-1F3FC-200D-1F3A4"},
    {"певец_цвет3", "1F9D1-1F3FD-200D-1F3A4"},
    {"певец_цвет4", "1F9D1-1F3FE-200D-1F3A4"},
    {"певец_цвет5", "1F9D1-1F3FF-200D-1F3A4"},
    {"художник", "1F9D1-200D-1F3A8"},
    {"художник_цвет1", "1F9D1-1F3FB-200D-1F3A8"},
    {"художник_цвет2", "1F9D1-1F3FC-200D-1F3A8"},
    {"художник_цвет3", "1F9D1-1F3FD-200D-1F3A8"},
    {"художник_цвет4", "1F9D1-1F3FE-200D-1F3A8"},
    {"художник_цвет5", "1F9D1-1F3FF-200D-1F3A8"},
    {"педагог", "1F9D1-200D-1F3EB"},
    {"педагог_цвет1", "1F9D1-1F3FB-200D-1F3EB"},
    {"педагог_цвет2", "1F9D1-1F3FC-200D-1F3EB"},
    {"педагог_цвет3", "1F9D1-1F3FD-200D-1F3EB"},
    {"педагог_цвет4", "1F9D1-1F3FE-200D-1F3EB"},
    {"педагог_цвет5", "1F9D1-1F3FF-200D-1F3EB"},
    {"рабочий", "1F9D1-200D-1F3ED"},
    {"рабочий_цвет1", "1F9D1-1F3FB-200D-1F3ED"},
    {"рабочий_цвет2", "1F9D1-1F3FC-200D-1F3ED"},
    {"рабочий_цвет3", "1F9D1-1F3FD-200D-1F3ED"},
    {"рабочий_цвет4", "1F9D1-1F3FE-200D-1F3ED"},
    {"рабочий_цвет5", "1F9D1-1F3FF-200D-1F3ED"},
    {"инженер", "1F9D1-200D-1F4BB"},
    {"инженер_цвет1", "1F9D1-1F3FB-200D-1F4BB"},
    {"инженер_цвет2", "1F9D1-1F3FC-200D-1F4BB"},
    {"инженер_цвет3", "1F9D1-1F3FD-200D-1F4BB"},
    {"инженер_цвет4", "1F9D1-1F3FE-200D-1F4BB"},
    {"инженер_цвет5", "1F9D1-1F3FF-200D-1F4BB"},
    {"офисный_работник", "1F9D1-200D-1F4BC"},
    {"офисный_работник_цвет1", "1F9D1-1F3FB-200D-1F4BC"},
    {"офисный_работник_цвет2", "1F9D1-1F3FC-200D-1F4BC"},
    {"офисный_работник_цвет3", "1F9D1-1F3FD-200D-1F4BC"},
    {"офисный_работник_цвет4", "1F9D1-1F3FE-200D-1F4BC"},
    {"офисный_работник_цвет5", "1F9D1-1F3FF-200D-1F4BC"},
    {"механик", "1F9D1-200D-1F527"},
    {"механик_цвет1", "1F9D1-1F3FB-200D-1F527"},
    {"механик_цвет2", "1F9D1-1F3FC-200D-1F527"},
    {"механик_цвет3", "1F9D1-1F3FD-200D-1F527"},
    {"механик_цвет4", "1F9D1-1F3FE-200D-1F527"},
    {"механик_цвет5", "1F9D1-1F3FF-200D-1F527"},
    {"ученый", "1F9D1-200D-1F52C"},
    {"ученый_цвет1", "1F9D1-1F3FB-200D-1F52C"},
    {"ученый_цвет2", "1F9D1-1F3FC-200D-1F52C"},
    {"ученый_цвет3", "1F9D1-1F3FD-200D-1F52C"},
    {"ученый_цвет4", "1F9D1-1F3FE-200D-1F52C"},
    {"ученый_цвет5", "1F9D1-1F3FF-200D-1F52C"},
    {"космонавт", "1F9D1-200D-1F680"},
    {"космонавт_цвет1", "1F9D1-1F3FB-200D-1F680"},
    {"космонавт_цвет2", "1F9D1-1F3FC-200D-1F680"},
    {"космонавт_цвет3", "1F9D1-1F3FD-200D-1F680"},
    {"космонавт_цвет4", "1F9D1-1F3FE-200D-1F680"},
    {"космонавт_цвет5", "1F9D1-1F3FF-200D-1F680"},
    {"пожарный", "1F9D1-200D-1F692"},
    {"пожарный_цвет1", "1F9D1-1F3FB-200D-1F692"},
    {"пожарный_цвет2", "1F9D1-1F3FC-200D-1F692"},
    {"пожарный_цвет3", "1F9D1-1F3FD-200D-1F692"},
    {"пожарный_цвет4", "1F9D1-1F3FE-200D-1F692"},
    {"пожарный_цвет5", "1F9D1-1F3FF-200D-1F692"},
    {"человек_с_белой_тростью", "1F9D1-200D-1F9AF"},
    {"человек_с_белой_тростью_цвет1", "1F9D1-1F3FB-200D-1F9AF"},
    {"человек_с_белой_тростью_цвет2", "1F9D1-1F3FC-200D-1F9AF"},
    {"человек_с_белой_тростью_цвет3", "1F9D1-1F3FD-200D-1F9AF"},
    {"человек_с_белой_тростью_цвет4", "1F9D1-1F3FE-200D-1F9AF"},
    {"человек_с_белой_тростью_цвет5", "1F9D1-1F3FF-200D-1F9AF"},
    {"человек_с_белой_тростью_направленные_вправо", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"человек_с_белой_тростью_направленные_вправо_цвет1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"человек_с_белой_тростью_направленные_вправо_цвет2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"человек_с_белой_тростью_направленные_вправо_цвет3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"человек_с_белой_тростью_направленные_вправо_цвет4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"человек_с_белой_тростью_направленные_вправо_цвет5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"человек_в_кресле_коляске_с_мотором", "1F9D1-200D-1F9BC"},
    {"человек_в_кресле_коляске_с_мотором_цвет1", "1F9D1-1F3FB-200D-1F9BC"},
    {"человек_в_кресле_коляске_с_мотором_цвет2", "1F9D1-1F3FC-200D-1F9BC"},
    {"человек_в_кресле_коляске_с_мотором_цвет3", "1F9D1-1F3FD-200D-1F9BC"},
    {"человек_в_кресле_коляске_с_мотором_цвет4", "1F9D1-1F3FE-200D-1F9BC"},
    {"человек_в_кресле_коляске_с_мотором_цвет5", "1F9D1-1F3FF-200D-1F9BC"},
    {"человек_в_кресле_коляске_с_мотором_направленные_вправо", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"человек_в_кресле_коляске_с_мотором_направленные_вправо_цвет1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"человек_в_кресле_коляске_с_мотором_направленные_вправо_цвет2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"человек_в_кресле_коляске_с_мотором_направленные_вправо_цвет3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"человек_в_кресле_коляске_с_мотором_направленные_вправо_цвет4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"человек_в_кресле_коляске_с_мотором_направленные_вправо_цвет5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"человек_в_кресле_коляске_с_ручным_приводом", "1F9D1-200D-1F9BD"},
    {"человек_в_кресле_коляске_с_ручным_приводом_цвет1", "1F9D1-1F3FB-200D-1F9BD"},
    {"человек_в_кресле_коляске_с_ручным_приводом_цвет2", "1F9D1-1F3FC-200D-1F9BD"},
    {"человек_в_кресле_коляске_с_ручным_приводом_цвет3", "1F9D1-1F3FD-200D-1F9BD"},
    {"человек_в_кресле_коляске_с_ручным_приводом_цвет4", "1F9D1-1F3FE-200D-1F9BD"},
    {"человек_в_кресле_коляске_с_ручным_приводом_цвет5", "1F9D1-1F3FF-200D-1F9BD"},
    {"человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_цвет1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_цвет2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_цвет3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_цвет4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"человек_в_кресле_коляске_с_ручным_приводом_направленные_вправо_цвет5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"женщина_с_мячом", "26F9-FE0F-200D-2640-FE0F"},
    {"женщина_с_мячом_цвет1", "26F9-1F3FB-200D-2640-FE0F"},
    {"женщина_с_мячом_цвет2", "26F9-1F3FC-200D-2640-FE0F"},
    {"женщина_с_мячом_цвет3", "26F9-1F3FD-200D-2640-FE0F"},
    {"женщина_с_мячом_цвет4", "26F9-1F3FE-200D-2640-FE0F"},
    {"женщина_с_мячом_цвет5", "26F9-1F3FF-200D-2640-FE0F"},
    {"мужчина_с_мячом", "26F9-FE0F-200D-2642-FE0F"},
    {"мужчина_с_мячом_цвет1", "26F9-1F3FB-200D-2642-FE0F"},
    {"мужчина_с_мячом_цвет2", "26F9-1F3FC-200D-2642-FE0F"},
    {"мужчина_с_мячом_цвет3", "26F9-1F3FD-200D-2642-FE0F"},
    {"мужчина_с_мячом_цвет4", "26F9-1F3FE-200D-2642-FE0F"},
    {"мужчина_с_мячом_цвет5", "26F9-1F3FF-200D-2642-FE0F"},
    {"женщина_бежит", "1F3C3-200D-2640-FE0F"},
    {"женщина_бежит_цвет1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"женщина_бежит_цвет2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"женщина_бежит_цвет3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"женщина_бежит_цвет4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"женщина_бежит_цвет5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"женщина_бежит_направленные_вправо", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_бежит_направленные_вправо_цвет1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_бежит_направленные_вправо_цвет2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_бежит_направленные_вправо_цвет3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_бежит_направленные_вправо_цвет4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_бежит_направленные_вправо_цвет5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"мужчина_бежит", "1F3C3-200D-2642-FE0F"},
    {"мужчина_бежит_цвет1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"мужчина_бежит_цвет2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"мужчина_бежит_цвет3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"мужчина_бежит_цвет4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"мужчина_бежит_цвет5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"мужчина_бежит_направленные_вправо", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_бежит_направленные_вправо_цвет1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_бежит_направленные_вправо_цвет2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_бежит_направленные_вправо_цвет3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_бежит_направленные_вправо_цвет4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_бежит_направленные_вправо_цвет5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"серфингистка", "1F3C4-200D-2640-FE0F"},
    {"серфингистка_цвет1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"серфингистка_цвет2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"серфингистка_цвет3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"серфингистка_цвет4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"серфингистка_цвет5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"серфингист", "1F3C4-200D-2642-FE0F"},
    {"серфингист_цвет1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"серфингист_цвет2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"серфингист_цвет3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"серфингист_цвет4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"серфингист_цвет5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"пловчиха", "1F3CA-200D-2640-FE0F"},
    {"пловчиха_цвет1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"пловчиха_цвет2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"пловчиха_цвет3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"пловчиха_цвет4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"пловчиха_цвет5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"пловец", "1F3CA-200D-2642-FE0F"},
    {"пловец_цвет1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"пловец_цвет2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"пловец_цвет3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"пловец_цвет4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"пловец_цвет5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"женщина_со_штангой", "1F3CB-FE0F-200D-2640-FE0F"},
    {"женщина_со_штангой_цвет1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"женщина_со_штангой_цвет2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"женщина_со_штангой_цвет3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"женщина_со_штангой_цвет4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"женщина_со_штангой_цвет5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"мужчина_со_штангой", "1F3CB-FE0F-200D-2642-FE0F"},
    {"мужчина_со_штангой_цвет1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"мужчина_со_штангой_цвет2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"мужчина_со_штангой_цвет3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"мужчина_со_штангой_цвет4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"мужчина_со_штангой_цвет5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"женщина_играет_в_гольф", "1F3CC-FE0F-200D-2640-FE0F"},
    {"женщина_играет_в_гольф_цвет1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"женщина_играет_в_гольф_цвет2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"женщина_играет_в_гольф_цвет3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"женщина_играет_в_гольф_цвет4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"женщина_играет_в_гольф_цвет5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"мужчина_играет_в_гольф", "1F3CC-FE0F-200D-2642-FE0F"},
    {"мужчина_играет_в_гольф_цвет1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"мужчина_играет_в_гольф_цвет2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"мужчина_играет_в_гольф_цвет3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"мужчина_играет_в_гольф_цвет4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"мужчина_играет_в_гольф_цвет5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"женщина_полицейский", "1F46E-200D-2640-FE0F"},
    {"женщина_полицейский_цвет1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"женщина_полицейский_цвет2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"женщина_полицейский_цвет3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"женщина_полицейский_цвет4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"женщина_полицейский_цвет5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"мужчина_полицейский", "1F46E-200D-2642-FE0F"},
    {"мужчина_полицейский_цвет1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"мужчина_полицейский_цвет2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"мужчина_полицейский_цвет3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"мужчина_полицейский_цвет4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"мужчина_полицейский_цвет5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"женщины_с_ушами_кролика", "1F46F-200D-2640-FE0F"},
    {"женщины_с_ушами_кролика_цвет1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"женщины_с_ушами_кролика_цвет1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"женщины_с_ушами_кролика_цвет1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"женщины_с_ушами_кролика_цвет1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"женщины_с_ушами_кролика_цвет2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"женщины_с_ушами_кролика_цвет2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"женщины_с_ушами_кролика_цвет2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"женщины_с_ушами_кролика_цвет2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"женщины_с_ушами_кролика_цвет3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"женщины_с_ушами_кролика_цвет3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"женщины_с_ушами_кролика_цвет3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"женщины_с_ушами_кролика_цвет3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"женщины_с_ушами_кролика_цвет4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"женщины_с_ушами_кролика_цвет4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"женщины_с_ушами_кролика_цвет4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"женщины_с_ушами_кролика_цвет4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"женщины_с_ушами_кролика_цвет5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"женщины_с_ушами_кролика_цвет5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"женщины_с_ушами_кролика_цвет5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"женщины_с_ушами_кролика_цвет5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"женщины_с_ушами_кролика_цвет1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"женщины_с_ушами_кролика_цвет2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"женщины_с_ушами_кролика_цвет3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"женщины_с_ушами_кролика_цвет4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"женщины_с_ушами_кролика_цвет5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"мужчины_с_ушами_кролика", "1F46F-200D-2642-FE0F"},
    {"мужчины_с_ушами_кролика_цвет1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"мужчины_с_ушами_кролика_цвет1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"мужчины_с_ушами_кролика_цвет1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"мужчины_с_ушами_кролика_цвет1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"мужчины_с_ушами_кролика_цвет2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"мужчины_с_ушами_кролика_цвет2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"мужчины_с_ушами_кролика_цвет2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"мужчины_с_ушами_кролика_цвет2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"мужчины_с_ушами_кролика_цвет3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"мужчины_с_ушами_кролика_цвет3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"мужчины_с_ушами_кролика_цвет3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"мужчины_с_ушами_кролика_цвет3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"мужчины_с_ушами_кролика_цвет4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"мужчины_с_ушами_кролика_цвет4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"мужчины_с_ушами_кролика_цвет4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"мужчины_с_ушами_кролика_цвет4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"мужчины_с_ушами_кролика_цвет5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"мужчины_с_ушами_кролика_цвет5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"мужчины_с_ушами_кролика_цвет5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"мужчины_с_ушами_кролика_цвет5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"мужчины_с_ушами_кролика_цвет1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"мужчины_с_ушами_кролика_цвет2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"мужчины_с_ушами_кролика_цвет3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"мужчины_с_ушами_кролика_цвет4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"мужчины_с_ушами_кролика_цвет5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"женщина_в_фате", "1F470-200D-2640-FE0F"},
    {"женщина_в_фате_цвет1", "1F470-1F3FB-200D-2640-FE0F"},
    {"женщина_в_фате_цвет2", "1F470-1F3FC-200D-2640-FE0F"},
    {"женщина_в_фате_цвет3", "1F470-1F3FD-200D-2640-FE0F"},
    {"женщина_в_фате_цвет4", "1F470-1F3FE-200D-2640-FE0F"},
    {"женщина_в_фате_цвет5", "1F470-1F3FF-200D-2640-FE0F"},
    {"мужчина_в_фате", "1F470-200D-2642-FE0F"},
    {"мужчина_в_фате_цвет1", "1F470-1F3FB-200D-2642-FE0F"},
    {"мужчина_в_фате_цвет2", "1F470-1F3FC-200D-2642-FE0F"},
    {"мужчина_в_фате_цвет3", "1F470-1F3FD-200D-2642-FE0F"},
    {"мужчина_в_фате_цвет4", "1F470-1F3FE-200D-2642-FE0F"},
    {"мужчина_в_фате_цвет5", "1F470-1F3FF-200D-2642-FE0F"},
    {"блондинка", "1F471-200D-2640-FE0F"},
    {"блондинка_цвет1", "1F471-1F3FB-200D-2640-FE0F"},
    {"блондинка_цвет2", "1F471-1F3FC-200D-2640-FE0F"},
    {"блондинка_цвет3", "1F471-1F3FD-200D-2640-FE0F"},
    {"блондинка_цвет4", "1F471-1F3FE-200D-2640-FE0F"},
    {"блондинка_цвет5", "1F471-1F3FF-200D-2640-FE0F"},
    {"блондин", "1F471-200D-2642-FE0F"},
    {"блондин_цвет1", "1F471-1F3FB-200D-2642-FE0F"},
    {"блондин_цвет2", "1F471-1F3FC-200D-2642-FE0F"},
    {"блондин_цвет3", "1F471-1F3FD-200D-2642-FE0F"},
    {"блондин_цвет4", "1F471-1F3FE-200D-2642-FE0F"},
    {"блондин_цвет5", "1F471-1F3FF-200D-2642-FE0F"},
    {"женщина_в_тюрбане", "1F473-200D-2640-FE0F"},
    {"женщина_в_тюрбане_цвет1", "1F473-1F3FB-200D-2640-FE0F"},
    {"женщина_в_тюрбане_цвет2", "1F473-1F3FC-200D-2640-FE0F"},
    {"женщина_в_тюрбане_цвет3", "1F473-1F3FD-200D-2640-FE0F"},
    {"женщина_в_тюрбане_цвет4", "1F473-1F3FE-200D-2640-FE0F"},
    {"женщина_в_тюрбане_цвет5", "1F473-1F3FF-200D-2640-FE0F"},
    {"мужчина_в_тюрбане", "1F473-200D-2642-FE0F"},
    {"мужчина_в_тюрбане_цвет1", "1F473-1F3FB-200D-2642-FE0F"},
    {"мужчина_в_тюрбане_цвет2", "1F473-1F3FC-200D-2642-FE0F"},
    {"мужчина_в_тюрбане_цвет3", "1F473-1F3FD-200D-2642-FE0F"},
    {"мужчина_в_тюрбане_цвет4", "1F473-1F3FE-200D-2642-FE0F"},
    {"мужчина_в_тюрбане_цвет5", "1F473-1F3FF-200D-2642-FE0F"},
    {"женщина_строитель", "1F477-200D-2640-FE0F"},
    {"женщина_строитель_цвет1", "1F477-1F3FB-200D-2640-FE0F"},
    {"женщина_строитель_цвет2", "1F477-1F3FC-200D-2640-FE0F"},
    {"женщина_строитель_цвет3", "1F477-1F3FD-200D-2640-FE0F"},
    {"женщина_строитель_цвет4", "1F477-1F3FE-200D-2640-FE0F"},
    {"женщина_строитель_цвет5", "1F477-1F3FF-200D-2640-FE0F"},
    {"мужчина_строитель", "1F477-200D-2642-FE0F"},
    {"мужчина_строитель_цвет1", "1F477-1F3FB-200D-2642-FE0F"},
    {"мужчина_строитель_цвет2", "1F477-1F3FC-200D-2642-FE0F"},
    {"мужчина_строитель_цвет3", "1F477-1F3FD-200D-2642-FE0F"},
    {"мужчина_строитель_цвет4", "1F477-1F3FE-200D-2642-FE0F"},
    {"мужчина_строитель_цвет5", "1F477-1F3FF-200D-2642-FE0F"},
    {"женщина_тянет_руку", "1F481-200D-2640-FE0F"},
    {"женщина_тянет_руку_цвет1", "1F481-1F3FB-200D-2640-FE0F"},
    {"женщина_тянет_руку_цвет2", "1F481-1F3FC-200D-2640-FE0F"},
    {"женщина_тянет_руку_цвет3", "1F481-1F3FD-200D-2640-FE0F"},
    {"женщина_тянет_руку_цвет4", "1F481-1F3FE-200D-2640-FE0F"},
    {"женщина_тянет_руку_цвет5", "1F481-1F3FF-200D-2640-FE0F"},
    {"мужчина_тянет_руку", "1F481-200D-2642-FE0F"},
    {"мужчина_тянет_руку_цвет1", "1F481-1F3FB-200D-2642-FE0F"},
    {"мужчина_тянет_руку_цвет2", "1F481-1F3FC-200D-2642-FE0F"},
    {"мужчина_тянет_руку_цвет3", "1F481-1F3FD-200D-2642-FE0F"},
    {"мужчина_тянет_руку_цвет4", "1F481-1F3FE-200D-2642-FE0F"},
    {"мужчина_тянет_руку_цвет5", "1F481-1F3FF-200D-2642-FE0F"},
    {"женщина_гвардеец", "1F482-200D-2640-FE0F"},
    {"женщина_гвардеец_цвет1", "1F482-1F3FB-200D-2640-FE0F"},
    {"женщина_гвардеец_цвет2", "1F482-1F3FC-200D-2640-FE0F"},
    {"женщина_гвардеец_цвет3", "1F482-1F3FD-200D-2640-FE0F"},
    {"женщина_гвардеец_цвет4", "1F482-1F3FE-200D-2640-FE0F"},
    {"женщина_гвардеец_цвет5", "1F482-1F3FF-200D-2640-FE0F"},
    {"мужчина_гвардеец", "1F482-200D-2642-FE0F"},
    {"мужчина_гвардеец_цвет1", "1F482-1F3FB-200D-2642-FE0F"},
    {"мужчина_гвардеец_цвет2", "1F482-1F3FC-200D-2642-FE0F"},
    {"мужчина_гвардеец_цвет3", "1F482-1F3FD-200D-2642-FE0F"},
    {"мужчина_гвардеец_цвет4", "1F482-1F3FE-200D-2642-FE0F"},
    {"мужчина_гвардеец_цвет5", "1F482-1F3FF-200D-2642-FE0F"},
    {"женщине_массируют_лицо", "1F486-200D-2640-FE0F"},
    {"женщине_массируют_лицо_цвет1", "1F486-1F3FB-200D-2640-FE0F"},
    {"женщине_массируют_лицо_цвет2", "1F486-1F3FC-200D-2640-FE0F"},
    {"женщине_массируют_лицо_цвет3", "1F486-1F3FD-200D-2640-FE0F"},
    {"женщине_массируют_лицо_цвет4", "1F486-1F3FE-200D-2640-FE0F"},
    {"женщине_массируют_лицо_цвет5", "1F486-1F3FF-200D-2640-FE0F"},
    {"мужчине_массируют_лицо", "1F486-200D-2642-FE0F"},
    {"мужчине_массируют_лицо_цвет1", "1F486-1F3FB-200D-2642-FE0F"},
    {"мужчине_массируют_лицо_цвет2", "1F486-1F3FC-200D-2642-FE0F"},
    {"мужчине_массируют_лицо_цвет3", "1F486-1F3FD-200D-2642-FE0F"},
    {"мужчине_массируют_лицо_цвет4", "1F486-1F3FE-200D-2642-FE0F"},
    {"мужчине_массируют_лицо_цвет5", "1F486-1F3FF-200D-2642-FE0F"},
    {"женщина_у_парикмахера", "1F487-200D-2640-FE0F"},
    {"женщина_у_парикмахера_цвет1", "1F487-1F3FB-200D-2640-FE0F"},
    {"женщина_у_парикмахера_цвет2", "1F487-1F3FC-200D-2640-FE0F"},
    {"женщина_у_парикмахера_цвет3", "1F487-1F3FD-200D-2640-FE0F"},
    {"женщина_у_парикмахера_цвет4", "1F487-1F3FE-200D-2640-FE0F"},
    {"женщина_у_парикмахера_цвет5", "1F487-1F3FF-200D-2640-FE0F"},
    {"мужчина_у_парикмахера", "1F487-200D-2642-FE0F"},
    {"мужчина_у_парикмахера_цвет1", "1F487-1F3FB-200D-2642-FE0F"},
    {"мужчина_у_парикмахера_цвет2", "1F487-1F3FC-200D-2642-FE0F"},
    {"мужчина_у_парикмахера_цвет3", "1F487-1F3FD-200D-2642-FE0F"},
    {"мужчина_у_парикмахера_цвет4", "1F487-1F3FE-200D-2642-FE0F"},
    {"мужчина_у_парикмахера_цвет5", "1F487-1F3FF-200D-2642-FE0F"},
    {"женщина_детектив", "1F575-FE0F-200D-2640-FE0F"},
    {"женщина_детектив_цвет1", "1F575-1F3FB-200D-2640-FE0F"},
    {"женщина_детектив_цвет2", "1F575-1F3FC-200D-2640-FE0F"},
    {"женщина_детектив_цвет3", "1F575-1F3FD-200D-2640-FE0F"},
    {"женщина_детектив_цвет4", "1F575-1F3FE-200D-2640-FE0F"},
    {"женщина_детектив_цвет5", "1F575-1F3FF-200D-2640-FE0F"},
    {"мужчина_детектив", "1F575-FE0F-200D-2642-FE0F"},
    {"мужчина_детектив_цвет1", "1F575-1F3FB-200D-2642-FE0F"},
    {"мужчина_детектив_цвет2", "1F575-1F3FC-200D-2642-FE0F"},
    {"мужчина_детектив_цвет3", "1F575-1F3FD-200D-2642-FE0F"},
    {"мужчина_детектив_цвет4", "1F575-1F3FE-200D-2642-FE0F"},
    {"мужчина_детектив_цвет5", "1F575-1F3FF-200D-2642-FE0F"},
    {"женщина_против", "1F645-200D-2640-FE0F"},
    {"женщина_против_цвет1", "1F645-1F3FB-200D-2640-FE0F"},
    {"женщина_против_цвет2", "1F645-1F3FC-200D-2640-FE0F"},
    {"женщина_против_цвет3", "1F645-1F3FD-200D-2640-FE0F"},
    {"женщина_против_цвет4", "1F645-1F3FE-200D-2640-FE0F"},
    {"женщина_против_цвет5", "1F645-1F3FF-200D-2640-FE0F"},
    {"мужчина_против", "1F645-200D-2642-FE0F"},
    {"мужчина_против_цвет1", "1F645-1F3FB-200D-2642-FE0F"},
    {"мужчина_против_цвет2", "1F645-1F3FC-200D-2642-FE0F"},
    {"мужчина_против_цвет3", "1F645-1F3FD-200D-2642-FE0F"},
    {"мужчина_против_цвет4", "1F645-1F3FE-200D-2642-FE0F"},
    {"мужчина_против_цвет5", "1F645-1F3FF-200D-2642-FE0F"},
    {"женщина_за", "1F646-200D-2640-FE0F"},
    {"женщина_за_цвет1", "1F646-1F3FB-200D-2640-FE0F"},
    {"женщина_за_цвет2", "1F646-1F3FC-200D-2640-FE0F"},
    {"женщина_за_цвет3", "1F646-1F3FD-200D-2640-FE0F"},
    {"женщина_за_цвет4", "1F646-1F3FE-200D-2640-FE0F"},
    {"женщина_за_цвет5", "1F646-1F3FF-200D-2640-FE0F"},
    {"мужчина_за", "1F646-200D-2642-FE0F"},
    {"мужчина_за_цвет1", "1F646-1F3FB-200D-2642-FE0F"},
    {"мужчина_за_цвет2", "1F646-1F3FC-200D-2642-FE0F"},
    {"мужчина_за_цвет3", "1F646-1F3FD-200D-2642-FE0F"},
    {"мужчина_за_цвет4", "1F646-1F3FE-200D-2642-FE0F"},
    {"мужчина_за_цвет5", "1F646-1F3FF-200D-2642-FE0F"},
    {"женщина_кланяется", "1F647-200D-2640-FE0F"},
    {"женщина_кланяется_цвет1", "1F647-1F3FB-200D-2640-FE0F"},
    {"женщина_кланяется_цвет2", "1F647-1F3FC-200D-2640-FE0F"},
    {"женщина_кланяется_цвет3", "1F647-1F3FD-200D-2640-FE0F"},
    {"женщина_кланяется_цвет4", "1F647-1F3FE-200D-2640-FE0F"},
    {"женщина_кланяется_цвет5", "1F647-1F3FF-200D-2640-FE0F"},
    {"мужчина_кланяется", "1F647-200D-2642-FE0F"},
    {"мужчина_кланяется_цвет1", "1F647-1F3FB-200D-2642-FE0F"},
    {"мужчина_кланяется_цвет2", "1F647-1F3FC-200D-2642-FE0F"},
    {"мужчина_кланяется_цвет3", "1F647-1F3FD-200D-2642-FE0F"},
    {"мужчина_кланяется_цвет4", "1F647-1F3FE-200D-2642-FE0F"},
    {"мужчина_кланяется_цвет5", "1F647-1F3FF-200D-2642-FE0F"},
    {"женщина_с_поднятой_рукой", "1F64B-200D-2640-FE0F"},
    {"женщина_с_поднятой_рукой_цвет1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"женщина_с_поднятой_рукой_цвет2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"женщина_с_поднятой_рукой_цвет3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"женщина_с_поднятой_рукой_цвет4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"женщина_с_поднятой_рукой_цвет5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"мужчина_с_поднятой_рукой", "1F64B-200D-2642-FE0F"},
    {"мужчина_с_поднятой_рукой_цвет1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"мужчина_с_поднятой_рукой_цвет2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"мужчина_с_поднятой_рукой_цвет3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"мужчина_с_поднятой_рукой_цвет4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"мужчина_с_поднятой_рукой_цвет5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"нахмурившаяся_женщина", "1F64D-200D-2640-FE0F"},
    {"нахмурившаяся_женщина_цвет1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"нахмурившаяся_женщина_цвет2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"нахмурившаяся_женщина_цвет3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"нахмурившаяся_женщина_цвет4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"нахмурившаяся_женщина_цвет5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"нахмурившийся_мужчина", "1F64D-200D-2642-FE0F"},
    {"нахмурившийся_мужчина_цвет1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"нахмурившийся_мужчина_цвет2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"нахмурившийся_мужчина_цвет3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"нахмурившийся_мужчина_цвет4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"нахмурившийся_мужчина_цвет5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"обиженная_женщина", "1F64E-200D-2640-FE0F"},
    {"обиженная_женщина_цвет1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"обиженная_женщина_цвет2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"обиженная_женщина_цвет3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"обиженная_женщина_цвет4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"обиженная_женщина_цвет5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"обиженный_мужчина", "1F64E-200D-2642-FE0F"},
    {"обиженный_мужчина_цвет1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"обиженный_мужчина_цвет2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"обиженный_мужчина_цвет3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"обиженный_мужчина_цвет4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"обиженный_мужчина_цвет5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"гребчиха", "1F6A3-200D-2640-FE0F"},
    {"гребчиха_цвет1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"гребчиха_цвет2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"гребчиха_цвет3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"гребчиха_цвет4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"гребчиха_цвет5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"гребец", "1F6A3-200D-2642-FE0F"},
    {"гребец_цвет1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"гребец_цвет2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"гребец_цвет3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"гребец_цвет4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"гребец_цвет5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"женщина_на_велосипеде", "1F6B4-200D-2640-FE0F"},
    {"женщина_на_велосипеде_цвет1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"женщина_на_велосипеде_цвет2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"женщина_на_велосипеде_цвет3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"женщина_на_велосипеде_цвет4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"женщина_на_велосипеде_цвет5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"мужчина_на_велосипеде", "1F6B4-200D-2642-FE0F"},
    {"мужчина_на_велосипеде_цвет1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"мужчина_на_велосипеде_цвет2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"мужчина_на_велосипеде_цвет3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"мужчина_на_велосипеде_цвет4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"мужчина_на_велосипеде_цвет5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"женщина_на_горном_велосипеде", "1F6B5-200D-2640-FE0F"},
    {"женщина_на_горном_велосипеде_цвет1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"женщина_на_горном_велосипеде_цвет2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"женщина_на_горном_велосипеде_цвет3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"женщина_на_горном_велосипеде_цвет4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"женщина_на_горном_велосипеде_цвет5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"мужчина_на_горном_велосипеде", "1F6B5-200D-2642-FE0F"},
    {"мужчина_на_горном_велосипеде_цвет1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"мужчина_на_горном_велосипеде_цвет2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"мужчина_на_горном_велосипеде_цвет3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"мужчина_на_горном_велосипеде_цвет4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"мужчина_на_горном_велосипеде_цвет5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"женщина_идет", "1F6B6-200D-2640-FE0F"},
    {"женщина_идет_цвет1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"женщина_идет_цвет2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"женщина_идет_цвет3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"женщина_идет_цвет4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"женщина_идет_цвет5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"женщина_идет_направленные_вправо", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_идет_направленные_вправо_цвет1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_идет_направленные_вправо_цвет2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_идет_направленные_вправо_цвет3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_идет_направленные_вправо_цвет4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_идет_направленные_вправо_цвет5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"мужчина_идет", "1F6B6-200D-2642-FE0F"},
    {"мужчина_идет_цвет1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"мужчина_идет_цвет2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"мужчина_идет_цвет3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"мужчина_идет_цвет4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"мужчина_идет_цвет5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"мужчина_идет_направленные_вправо", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_идет_направленные_вправо_цвет1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_идет_направленные_вправо_цвет2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_идет_направленные_вправо_цвет3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_идет_направленные_вправо_цвет4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_идет_направленные_вправо_цвет5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"женщина_потеряла_дар_речи", "1F926-200D-2640-FE0F"},
    {"женщина_потеряла_дар_речи_цвет1", "1F926-1F3FB-200D-2640-FE0F"},
    {"женщина_потеряла_дар_речи_цвет2", "1F926-1F3FC-200D-2640-FE0F"},
    {"женщина_потеряла_дар_речи_цвет3", "1F926-1F3FD-200D-2640-FE0F"},
    {"женщина_потеряла_дар_речи_цвет4", "1F926-1F3FE-200D-2640-FE0F"},
    {"женщина_потеряла_дар_речи_цвет5", "1F926-1F3FF-200D-2640-FE0F"},
    {"мужчина_потерял_дар_речи", "1F926-200D-2642-FE0F"},
    {"мужчина_потерял_дар_речи_цвет1", "1F926-1F3FB-200D-2642-FE0F"},
    {"мужчина_потерял_дар_речи_цвет2", "1F926-1F3FC-200D-2642-FE0F"},
    {"мужчина_потерял_дар_речи_цвет3", "1F926-1F3FD-200D-2642-FE0F"},
    {"мужчина_потерял_дар_речи_цвет4", "1F926-1F3FE-200D-2642-FE0F"},
    {"мужчина_потерял_дар_речи_цвет5", "1F926-1F3FF-200D-2642-FE0F"},
    {"женщина_в_смокинге", "1F935-200D-2640-FE0F"},
    {"женщина_в_смокинге_цвет1", "1F935-1F3FB-200D-2640-FE0F"},
    {"женщина_в_смокинге_цвет2", "1F935-1F3FC-200D-2640-FE0F"},
    {"женщина_в_смокинге_цвет3", "1F935-1F3FD-200D-2640-FE0F"},
    {"женщина_в_смокинге_цвет4", "1F935-1F3FE-200D-2640-FE0F"},
    {"женщина_в_смокинге_цвет5", "1F935-1F3FF-200D-2640-FE0F"},
    {"мужчина_в_смокинге", "1F935-200D-2642-FE0F"},
    {"мужчина_в_смокинге_цвет1", "1F935-1F3FB-200D-2642-FE0F"},
    {"мужчина_в_смокинге_цвет2", "1F935-1F3FC-200D-2642-FE0F"},
    {"мужчина_в_смокинге_цвет3", "1F935-1F3FD-200D-2642-FE0F"},
    {"мужчина_в_смокинге_цвет4", "1F935-1F3FE-200D-2642-FE0F"},
    {"мужчина_в_смокинге_цвет5", "1F935-1F3FF-200D-2642-FE0F"},
    {"женщина_пожимает_плечами", "1F937-200D-2640-FE0F"},
    {"женщина_пожимает_плечами_цвет1", "1F937-1F3FB-200D-2640-FE0F"},
    {"женщина_пожимает_плечами_цвет2", "1F937-1F3FC-200D-2640-FE0F"},
    {"женщина_пожимает_плечами_цвет3", "1F937-1F3FD-200D-2640-FE0F"},
    {"женщина_пожимает_плечами_цвет4", "1F937-1F3FE-200D-2640-FE0F"},
    {"женщина_пожимает_плечами_цвет5", "1F937-1F3FF-200D-2640-FE0F"},
    {"мужчина_пожимает_плечами", "1F937-200D-2642-FE0F"},
    {"мужчина_пожимает_плечами_цвет1", "1F937-1F3FB-200D-2642-FE0F"},
    {"мужчина_пожимает_плечами_цвет2", "1F937-1F3FC-200D-2642-FE0F"},
    {"мужчина_пожимает_плечами_цвет3", "1F937-1F3FD-200D-2642-FE0F"},
    {"мужчина_пожимает_плечами_цвет4", "1F937-1F3FE-200D-2642-FE0F"},
    {"мужчина_пожимает_плечами_цвет5", "1F937-1F3FF-200D-2642-FE0F"},
    {"женщина_акробат", "1F938-200D-2640-FE0F"},
    {"женщина_акробат_цвет1", "1F938-1F3FB-200D-2640-FE0F"},
    {"женщина_акробат_цвет2", "1F938-1F3FC-200D-2640-FE0F"},
    {"женщина_акробат_цвет3", "1F938-1F3FD-200D-2640-FE0F"},
    {"женщина_акробат_цвет4", "1F938-1F3FE-200D-2640-FE0F"},
    {"женщина_акробат_цвет5", "1F938-1F3FF-200D-2640-FE0F"},
    {"акробат", "1F938-200D-2642-FE0F"},
    {"акробат_цвет1", "1F938-1F3FB-200D-2642-FE0F"},
    {"акробат_цвет2", "1F938-1F3FC-200D-2642-FE0F"},
    {"акробат_цвет3", "1F938-1F3FD-200D-2642-FE0F"},
    {"акробат_цвет4", "1F938-1F3FE-200D-2642-FE0F"},
    {"акробат_цвет5", "1F938-1F3FF-200D-2642-FE0F"},
    {"женщина_жонглер", "1F939-200D-2640-FE0F"},
    {"женщина_жонглер_цвет1", "1F939-1F3FB-200D-2640-FE0F"},
    {"женщина_жонглер_цвет2", "1F939-1F3FC-200D-2640-FE0F"},
    {"женщина_жонглер_цвет3", "1F939-1F3FD-200D-2640-FE0F"},
    {"женщина_жонглер_цвет4", "1F939-1F3FE-200D-2640-FE0F"},
    {"женщина_жонглер_цвет5", "1F939-1F3FF-200D-2640-FE0F"},
    {"мужчина_жонглер", "1F939-200D-2642-FE0F"},
    {"мужчина_жонглер_цвет1", "1F939-1F3FB-200D-2642-FE0F"},
    {"мужчина_жонглер_цвет2", "1F939-1F3FC-200D-2642-FE0F"},
    {"мужчина_жонглер_цвет3", "1F939-1F3FD-200D-2642-FE0F"},
    {"мужчина_жонглер_цвет4", "1F939-1F3FE-200D-2642-FE0F"},
    {"мужчина_жонглер_цвет5", "1F939-1F3FF-200D-2642-FE0F"},
    {"женщины_борцы", "1F93C-200D-2640-FE0F"},
    {"женщины_борцы_цвет1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"женщины_борцы_цвет1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"женщины_борцы_цвет1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"женщины_борцы_цвет1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"женщины_борцы_цвет2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"женщины_борцы_цвет2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"женщины_борцы_цвет2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"женщины_борцы_цвет2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"женщины_борцы_цвет3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"женщины_борцы_цвет3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"женщины_борцы_цвет3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"женщины_борцы_цвет3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"женщины_борцы_цвет4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"женщины_борцы_цвет4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"женщины_борцы_цвет4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"женщины_борцы_цвет4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"женщины_борцы_цвет5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"женщины_борцы_цвет5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"женщины_борцы_цвет5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"женщины_борцы_цвет5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"женщины_борцы_цвет1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"женщины_борцы_цвет2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"женщины_борцы_цвет3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"женщины_борцы_цвет4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"женщины_борцы_цвет5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"мужчины_борцы", "1F93C-200D-2642-FE0F"},
    {"мужчины_борцы_цвет1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"мужчины_борцы_цвет1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"мужчины_борцы_цвет1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"мужчины_борцы_цвет1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"мужчины_борцы_цвет2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"мужчины_борцы_цвет2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"мужчины_борцы_цвет2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"мужчины_борцы_цвет2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"мужчины_борцы_цвет3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"мужчины_борцы_цвет3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"мужчины_борцы_цвет3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"мужчины_борцы_цвет3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"мужчины_борцы_цвет4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"мужчины_борцы_цвет4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"мужчины_борцы_цвет4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"мужчины_борцы_цвет4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"мужчины_борцы_цвет5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"мужчины_борцы_цвет5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"мужчины_борцы_цвет5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"мужчины_борцы_цвет5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"мужчины_борцы_цвет1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"мужчины_борцы_цвет2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"мужчины_борцы_цвет3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"мужчины_борцы_цвет4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"мужчины_борцы_цвет5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"женщина_играет_в_водное_поло", "1F93D-200D-2640-FE0F"},
    {"женщина_играет_в_водное_поло_цвет1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"женщина_играет_в_водное_поло_цвет2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"женщина_играет_в_водное_поло_цвет3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"женщина_играет_в_водное_поло_цвет4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"женщина_играет_в_водное_поло_цвет5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"мужчина_играет_в_водное_поло", "1F93D-200D-2642-FE0F"},
    {"мужчина_играет_в_водное_поло_цвет1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"мужчина_играет_в_водное_поло_цвет2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"мужчина_играет_в_водное_поло_цвет3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"мужчина_играет_в_водное_поло_цвет4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"мужчина_играет_в_водное_поло_цвет5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"гандболистка", "1F93E-200D-2640-FE0F"},
    {"гандболистка_цвет1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"гандболистка_цвет2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"гандболистка_цвет3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"гандболистка_цвет4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"гандболистка_цвет5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"гандболист", "1F93E-200D-2642-FE0F"},
    {"гандболист_цвет1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"гандболист_цвет2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"гандболист_цвет3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"гандболист_цвет4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"гандболист_цвет5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"женщина_супергерой", "1F9B8-200D-2640-FE0F"},
    {"женщина_супергерой_цвет1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"женщина_супергерой_цвет2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"женщина_супергерой_цвет3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"женщина_супергерой_цвет4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"женщина_супергерой_цвет5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"мужчина_супергерой", "1F9B8-200D-2642-FE0F"},
    {"мужчина_супергерой_цвет1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"мужчина_супергерой_цвет2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"мужчина_супергерой_цвет3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"мужчина_супергерой_цвет4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"мужчина_супергерой_цвет5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"женщина_суперзлодей", "1F9B9-200D-2640-FE0F"},
    {"женщина_суперзлодей_цвет1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"женщина_суперзлодей_цвет2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"женщина_суперзлодей_цвет3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"женщина_суперзлодей_цвет4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"женщина_суперзлодей_цвет5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"мужчина_суперзлодей", "1F9B9-200D-2642-FE0F"},
    {"мужчина_суперзлодей_цвет1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"мужчина_суперзлодей_цвет2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"мужчина_суперзлодей_цвет3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"мужчина_суперзлодей_цвет4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"мужчина_суперзлодей_цвет5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"женщина_стоит", "1F9CD-200D-2640-FE0F"},
    {"женщина_стоит_цвет1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"женщина_стоит_цвет2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"женщина_стоит_цвет3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"женщина_стоит_цвет4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"женщина_стоит_цвет5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"мужчина_стоит", "1F9CD-200D-2642-FE0F"},
    {"мужчина_стоит_цвет1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"мужчина_стоит_цвет2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"мужчина_стоит_цвет3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"мужчина_стоит_цвет4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"мужчина_стоит_цвет5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"женщина_на_коленях", "1F9CE-200D-2640-FE0F"},
    {"женщина_на_коленях_цвет1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"женщина_на_коленях_цвет2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"женщина_на_коленях_цвет3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"женщина_на_коленях_цвет4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"женщина_на_коленях_цвет5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"женщина_на_коленях_направленные_вправо", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_на_коленях_направленные_вправо_цвет1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_на_коленях_направленные_вправо_цвет2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_на_коленях_направленные_вправо_цвет3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_на_коленях_направленные_вправо_цвет4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"женщина_на_коленях_направленные_вправо_цвет5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"мужчина_на_коленях", "1F9CE-200D-2642-FE0F"},
    {"мужчина_на_коленях_цвет1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"мужчина_на_коленях_цвет2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"мужчина_на_коленях_цвет3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"мужчина_на_коленях_цвет4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"мужчина_на_коленях_цвет5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"мужчина_на_коленях_направленные_вправо", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_на_коленях_направленные_вправо_цвет1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_на_коленях_направленные_вправо_цвет2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_на_коленях_направленные_вправо_цвет3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_на_коленях_направленные_вправо_цвет4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"мужчина_на_коленях_направленные_вправо_цвет5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"глухая_женщина", "1F9CF-200D-2640-FE0F"},
    {"глухая_женщина_цвет1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"глухая_женщина_цвет2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"глухая_женщина_цвет3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"глухая_женщина_цвет4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"глухая_женщина_цвет5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"глухой_мужчина", "1F9CF-200D-2642-FE0F"},
    {"глухой_мужчина_цвет1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"глухой_мужчина_цвет2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"глухой_мужчина_цвет3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"глухой_мужчина_цвет4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"глухой_мужчина_цвет5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"бородатая_женщина", "1F9D4-200D-2640-FE0F"},
    {"бородатая_женщина_цвет1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"бородатая_женщина_цвет2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"бородатая_женщина_цвет3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"бородатая_женщина_цвет4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"бородатая_женщина_цвет5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"бородатый_мужчина", "1F9D4-200D-2642-FE0F"},
    {"бородатый_мужчина_цвет1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"бородатый_мужчина_цвет2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"бородатый_мужчина_цвет3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"бородатый_мужчина_цвет4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"бородатый_мужчина_цвет5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"женщина_в_бане", "1F9D6-200D-2640-FE0F"},
    {"женщина_в_бане_цвет1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"женщина_в_бане_цвет2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"женщина_в_бане_цвет3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"женщина_в_бане_цвет4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"женщина_в_бане_цвет5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"мужчина_в_бане", "1F9D6-200D-2642-FE0F"},
    {"мужчина_в_бане_цвет1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"мужчина_в_бане_цвет2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"мужчина_в_бане_цвет3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"мужчина_в_бане_цвет4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"мужчина_в_бане_цвет5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"женщина_взбирается_по_скале", "1F9D7-200D-2640-FE0F"},
    {"женщина_взбирается_по_скале_цвет1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"женщина_взбирается_по_скале_цвет2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"женщина_взбирается_по_скале_цвет3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"женщина_взбирается_по_скале_цвет4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"женщина_взбирается_по_скале_цвет5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"мужчина_взбирается_по_скале", "1F9D7-200D-2642-FE0F"},
    {"мужчина_взбирается_по_скале_цвет1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"мужчина_взбирается_по_скале_цвет2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"мужчина_взбирается_по_скале_цвет3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"мужчина_взбирается_по_скале_цвет4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"мужчина_взбирается_по_скале_цвет5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"женщина_в_позе_лотоса", "1F9D8-200D-2640-FE0F"},
    {"женщина_в_позе_лотоса_цвет1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"женщина_в_позе_лотоса_цвет2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"женщина_в_позе_лотоса_цвет3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"женщина_в_позе_лотоса_цвет4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"женщина_в_позе_лотоса_цвет5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"мужчина_в_позе_лотоса", "1F9D8-200D-2642-FE0F"},
    {"мужчина_в_позе_лотоса_цвет1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"мужчина_в_позе_лотоса_цвет2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"мужчина_в_позе_лотоса_цвет3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"мужчина_в_позе_лотоса_цвет4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"мужчина_в_позе_лотоса_цвет5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"волшебница", "1F9D9-200D-2640-FE0F"},
    {"волшебница_цвет1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"волшебница_цвет2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"волшебница_цвет3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"волшебница_цвет4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"волшебница_цвет5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"волшебник", "1F9D9-200D-2642-FE0F"},
    {"волшебник_цвет1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"волшебник_цвет2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"волшебник_цвет3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"волшебник_цвет4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"волшебник_цвет5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"женщина_фея", "1F9DA-200D-2640-FE0F"},
    {"женщина_фея_цвет1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"женщина_фея_цвет2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"женщина_фея_цвет3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"женщина_фея_цвет4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"женщина_фея_цвет5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"мужчина_фея", "1F9DA-200D-2642-FE0F"},
    {"мужчина_фея_цвет1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"мужчина_фея_цвет2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"мужчина_фея_цвет3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"мужчина_фея_цвет4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"мужчина_фея_цвет5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"женщина_вампир", "1F9DB-200D-2640-FE0F"},
    {"женщина_вампир_цвет1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"женщина_вампир_цвет2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"женщина_вампир_цвет3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"женщина_вампир_цвет4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"женщина_вампир_цвет5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"мужчина_вампир", "1F9DB-200D-2642-FE0F"},
    {"мужчина_вампир_цвет1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"мужчина_вампир_цвет2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"мужчина_вампир_цвет3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"мужчина_вампир_цвет4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"мужчина_вампир_цвет5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"женщина_русалка", "1F9DC-200D-2640-FE0F"},
    {"женщина_русалка_цвет1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"женщина_русалка_цвет2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"женщина_русалка_цвет3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"женщина_русалка_цвет4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"женщина_русалка_цвет5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"тритон", "1F9DC-200D-2642-FE0F"},
    {"тритон_цвет1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"тритон_цвет2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"тритон_цвет3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"тритон_цвет4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"тритон_цвет5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"женщина_эльф", "1F9DD-200D-2640-FE0F"},
    {"женщина_эльф_цвет1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"женщина_эльф_цвет2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"женщина_эльф_цвет3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"женщина_эльф_цвет4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"женщина_эльф_цвет5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"мужчина_эльф", "1F9DD-200D-2642-FE0F"},
    {"мужчина_эльф_цвет1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"мужчина_эльф_цвет2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"мужчина_эльф_цвет3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"мужчина_эльф_цвет4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"мужчина_эльф_цвет5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"женщина_джинн", "1F9DE-200D-2640-FE0F"},
    {"мужчина_джинн", "1F9DE-200D-2642-FE0F"},
    {"женщина_зомби", "1F9DF-200D-2640-FE0F"},
    {"мужчина_зомби", "1F9DF-200D-2642-FE0F"},
    {"мужчина_рыжие_волосы", "1F468-200D-1F9B0"},
    {"мужчина_рыжие_волосы_цвет1", "1F468-1F3FB-200D-1F9B0"},
    {"мужчина_рыжие_волосы_цвет2", "1F468-1F3FC-200D-1F9B0"},
    {"мужчина_рыжие_волосы_цвет3", "1F468-1F3FD-200D-1F9B0"},
    {"мужчина_рыжие_волосы_цвет4", "1F468-1F3FE-200D-1F9B0"},
    {"мужчина_рыжие_волосы_цвет5", "1F468-1F3FF-200D-1F9B0"},
    {"мужчина_кудрявые_волосы", "1F468-200D-1F9B1"},
    {"мужчина_кудрявые_волосы_цвет1", "1F468-1F3FB-200D-1F9B1"},
    {"мужчина_кудрявые_волосы_цвет2", "1F468-1F3FC-200D-1F9B1"},
    {"мужчина_кудрявые_волосы_цвет3", "1F468-1F3FD-200D-1F9B1"},
    {"мужчина_кудрявые_волосы_цвет4", "1F468-1F3FE-200D-1F9B1"},
    {"мужчина_кудрявые_волосы_цвет5", "1F468-1F3FF-200D-1F9B1"},
    {"мужчина_без_волос", "1F468-200D-1F9B2"},
    {"мужчина_без_волос_цвет1", "1F468-1F3FB-200D-1F9B2"},
    {"мужчина_без_волос_цвет2", "1F468-1F3FC-200D-1F9B2"},
    {"мужчина_без_волос_цвет3", "1F468-1F3FD-200D-1F9B2"},
    {"мужчина_без_волос_цвет4", "1F468-1F3FE-200D-1F9B2"},
    {"мужчина_без_волос_цвет5", "1F468-1F3FF-200D-1F9B2"},
    {"мужчина_седые_волосы", "1F468-200D-1F9B3"},
    {"мужчина_седые_волосы_цвет1", "1F468-1F3FB-200D-1F9B3"},
    {"мужчина_седые_волосы_цвет2", "1F468-1F3FC-200D-1F9B3"},
    {"мужчина_седые_волосы_цвет3", "1F468-1F3FD-200D-1F9B3"},
    {"мужчина_седые_волосы_цвет4", "1F468-1F3FE-200D-1F9B3"},
    {"мужчина_седые_волосы_цвет5", "1F468-1F3FF-200D-1F9B3"},
    {"женщина_рыжие_волосы", "1F469-200D-1F9B0"},
    {"женщина_рыжие_волосы_цвет1", "1F469-1F3FB-200D-1F9B0"},
    {"женщина_рыжие_волосы_цвет2", "1F469-1F3FC-200D-1F9B0"},
    {"женщина_рыжие_волосы_цвет3", "1F469-1F3FD-200D-1F9B0"},
    {"женщина_рыжие_волосы_цвет4", "1F469-1F3FE-200D-1F9B0"},
    {"женщина_рыжие_волосы_цвет5", "1F469-1F3FF-200D-1F9B0"},
    {"женщина_кудрявые_волосы", "1F469-200D-1F9B1"},
    {"женщина_кудрявые_волосы_цвет1", "1F469-1F3FB-200D-1F9B1"},
    {"женщина_кудрявые_волосы_цвет2", "1F469-1F3FC-200D-1F9B1"},
    {"женщина_кудрявые_волосы_цвет3", "1F469-1F3FD-200D-1F9B1"},
    {"женщина_кудрявые_волосы_цвет4", "1F469-1F3FE-200D-1F9B1"},
    {"женщина_кудрявые_волосы_цвет5", "1F469-1F3FF-200D-1F9B1"},
    {"женщина_без_волос", "1F469-200D-1F9B2"},
    {"женщина_без_волос_цвет1", "1F469-1F3FB-200D-1F9B2"},
    {"женщина_без_волос_цвет2", "1F469-1F3FC-200D-1F9B2"},
    {"женщина_без_волос_цвет3", "1F469-1F3FD-200D-1F9B2"},
    {"женщина_без_волос_цвет4", "1F469-1F3FE-200D-1F9B2"},
    {"женщина_без_волос_цвет5", "1F469-1F3FF-200D-1F9B2"},
    {"женщина_седые_волосы", "1F469-200D-1F9B3"},
    {"женщина_седые_волосы_цвет1", "1F469-1F3FB-200D-1F9B3"},
    {"женщина_седые_волосы_цвет2", "1F469-1F3FC-200D-1F9B3"},
    {"женщина_седые_волосы_цвет3", "1F469-1F3FD-200D-1F9B3"},
    {"женщина_седые_волосы_цвет4", "1F469-1F3FE-200D-1F9B3"},
    {"женщина_седые_волосы_цвет5", "1F469-1F3FF-200D-1F9B3"},
    {"взрослый_рыжие_волосы", "1F9D1-200D-1F9B0"},
    {"взрослый_рыжие_волосы_цвет1", "1F9D1-1F3FB-200D-1F9B0"},
    {"взрослый_рыжие_волосы_цвет2", "1F9D1-1F3FC-200D-1F9B0"},
    {"взрослый_рыжие_волосы_цвет3", "1F9D1-1F3FD-200D-1F9B0"},
    {"взрослый_рыжие_волосы_цвет4", "1F9D1-1F3FE-200D-1F9B0"},
    {"взрослый_рыжие_волосы_цвет5", "1F9D1-1F3FF-200D-1F9B0"},
    {"взрослый_кудрявые_волосы", "1F9D1-200D-1F9B1"},
    {"взрослый_кудрявые_волосы_цвет1", "1F9D1-1F3FB-200D-1F9B1"},
    {"взрослый_кудрявые_волосы_цвет2", "1F9D1-1F3FC-200D-1F9B1"},
    {"взрослый_кудрявые_волосы_цвет3", "1F9D1-1F3FD-200D-1F9B1"},
    {"взрослый_кудрявые_волосы_цвет4", "1F9D1-1F3FE-200D-1F9B1"},
    {"взрослый_кудрявые_волосы_цвет5", "1F9D1-1F3FF-200D-1F9B1"},
    {"взрослый_без_волос", "1F9D1-200D-1F9B2"},
    {"взрослый_без_волос_цвет1", "1F9D1-1F3FB-200D-1F9B2"},
    {"взрослый_без_волос_цвет2", "1F9D1-1F3FC-200D-1F9B2"},
    {"взрослый_без_волос_цвет3", "1F9D1-1F3FD-200D-1F9B2"},
    {"взрослый_без_волос_цвет4", "1F9D1-1F3FE-200D-1F9B2"},
    {"взрослый_без_волос_цвет5", "1F9D1-1F3FF-200D-1F9B2"},
    {"взрослый_седые_волосы", "1F9D1-200D-1F9B3"},
    {"взрослый_седые_волосы_цвет1", "1F9D1-1F3FB-200D-1F9B3"},
    {"взрослый_седые_волосы_цвет2", "1F9D1-1F3FC-200D-1F9B3"},
    {"взрослый_седые_волосы_цвет3", "1F9D1-1F3FD-200D-1F9B3"},
    {"взрослый_седые_волосы_цвет4", "1F9D1-1F3FE-200D-1F9B3"},
    {"взрослый_седые_волосы_цвет5", "1F9D1-1F3FF-200D-1F9B3"},
    {"разорванная_цепь", "26D3-FE0F-200D-1F4A5"},
    {"горящее_сердце", "2764-FE0F-200D-1F525"},
    {"заживающее_сердце", "2764-FE0F-200D-1FA79"},
    {"бурый_гриб", "1F344-200D-1F7EB"},
    {"лайм", "1F34B-200D-1F7E9"},
    {"флаг_трансгендерного_сообщества", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"радужный_флаг", "1F3F3-FE0F-200D-1F308"},
    {"пиратский_флаг", "1F3F4-200D-2620-FE0F"},
    {"черная_кошка", "1F408-200D-2B1B"},
    {"служебная_собака", "1F415-200D-1F9BA"},
    {"черная_птица", "1F426-200D-2B1B"},
    {"феникс", "1F426-200D-1F525"},
    {"морда_белого_медведя", "1F43B-200D-2744-FE0F"},
    {"значок_свидетеля", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"выдыхает", "1F62E-200D-1F4A8"},
    {"ошеломление", "1F635-200D-1F4AB"},
    {"лицо_в_облаках", "1F636-200D-1F32B-FE0F"},
    {"отрицательно_качает_головой", "1F642-200D-2194-FE0F"},
    {"положительно_качает_головой", "1F642-200D-2195-FE0F"},
    {"артист_балета", "1F9D1-200D-1FA70"},
    {"артист_балета_цвет1", "1F9D1-1F3FB-200D-1FA70"},
    {"артист_балета_цвет2", "1F9D1-1F3FC-200D-1FA70"},
    {"артист_балета_цвет3", "1F9D1-1F3FD-200D-1FA70"},
    {"артист_балета_цвет4", "1F9D1-1F3FE-200D-1FA70"},
    {"артист_балета_цвет5", "1F9D1-1F3FF-200D-1FA70"},
};

