// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_nl[] = {
    {"uitroepteken_en_vraagteken", "2049"},
    {"handelsmerksymbool", "2122"},
    {"informatie", "2139"},
    {"pijl_naar_links_en_rechts", "2194"},
    {"pijl_omhoog_en_omlaag", "2195"},
    {"pijl_linksomhoog", "2196"},
    {"pijl_rechtsomhoog", "2197"},
    {"pijl_rechtsomlaag", "2198"},
    {"pijl_linksomlaag", "2199"},
    {"toetsenbord", "2328"},
    {"zon", "2600"},
    {"wolk", "2601"},
    {"paraplu", "2602"},
    {"sneeuwpop", "2603"},
    {"komeet", "2604"},
    {"vinkje_in_vak", "2611"},
    {"paraplu_met_regendruppels", "2614"},
    {"warme_drank", "2615"},
    {"klavertjedrie", "2618"},
    {"doodshoofd_met_gekruiste_beenderen", "2620"},
    {"radioactief", "2622"},
    {"biologisch_gevaar", "2623"},
    {"russisch_kruis", "2626"},
    {"wiel_van_dharma", "2638"},
    {"fronsend_gezicht", "2639"},
    {"vrouwelijkheidssymbool", "2640"},
    {"mannelijkheidssymbool", "2642"},
    {"ram_sterrenbeeld", "2648"},
    {"stier_sterrenbeeld", "2649"},
    {"boogschutter_sterrenbeeld", "2650"},
    {"steenbok_sterrenbeeld", "2651"},
    {"waterman_sterrenbeeld", "2652"},
    {"vissen_sterrenbeeld", "2653"},
    {"schoppen", "2660"},
    {"klaveren", "2663"},
    {"harten", "2665"},
    {"ruiten", "2666"},
    {"warmwaterbronnen", "2668"},
    {"twee_hamers", "2692"},
    {"anker", "2693"},
    {"gekruiste_zwaarden", "2694"},
    {"esculaap", "2695"},
    {"weegschaal", "2696"},
    {"distilleerkolf", "2697"},
    {"tandwiel", "2699"},
    {"schaar", "2702"},
    {"knop_voor_vinkje", "2705"},
    {"vliegtuig", "2708"},
    {"envelop", "2709"},
    {"kroontjespen_met_zwarte_punt", "2712"},
    {"vinkje", "2714"},
    {"vermenigvuldigen", "2716"},
    {"davidster", "2721"},
    {"sterretjes", "2728"},
    {"achtpuntige_asterisk", "2733"},
    {"achtpuntige_ster", "2734"},
    {"sneeuwvlok", "2744"},
    {"sprankeling", "2747"},
    {"rood_vraagteken", "2753"},
    {"wit_vraagteken", "2754"},
    {"wit_uitroepteken", "2755"},
    {"rood_uitroepteken", "2757"},
    {"hart_als_uitroepteken", "2763"},
    {"rood_hart", "2764"},
    {"plus", "2795"},
    {"min", "2796"},
    {"delen", "2797"},
    {"pijl_naar_rechts_die_omhoog_draait", "2934"},
    {"pijl_naar_rechts_die_omlaag_draait", "2935"},
    {"golvend_streepje", "3030"},
    {"japans_teken_voor_gefeliciteerd", "3297"},
    {"japans_teken_voor_geheim", "3299"},
    {"copyright", "00A9"},
    {"gedeponeerd", "00AE"},
    {"dubbel_uitroepteken", "203C"},
    {"pijl_naar_rechts_die_naar_links_draait", "21A9"},
    {"pijl_naar_links_die_naar_rechts_draait", "21AA"},
    {"horloge", "231A"},
    {"zandloper", "231B"},
    {"knop_voor_uitwerpen", "23CF"},
    {"knop_voor_snel_vooruitspoelen", "23E9"},
    {"knop_voor_versneld_terugspoelen", "23EA"},
    {"knop_voor_versneld_omhoog", "23EB"},
    {"knop_voor_versneld_omlaag", "23EC"},
    {"knop_voor_volgende_nummer", "23ED"},
    {"knop_voor_vorige_nummer", "23EE"},
    {"knop_voor_afspelen_of_pauzeren", "23EF"},
    {"wekker", "23F0"},
    {"stopwatch", "23F1"},
    {"keukenwekker", "23F2"},
    {"zandloper_met_stromend_zand", "23F3"},
    {"pauzeknop", "23F8"},
    {"knop_voor_stoppen", "23F9"},
    {"knop_voor_opnemen", "23FA"},
    {"m_in_cirkel", "24C2"},
    {"klein_zwart_vierkant", "25AA"},
    {"klein_wit_vierkant", "25AB"},
    {"knop_voor_afspelen", "25B6"},
    {"knop_voor_terugspoelen", "25C0"},
    {"middelgroot_wit_vierkant", "25FB"},
    {"middelgroot_zwart_vierkant", "25FC"},
    {"klein_tot_middelgroot_wit_vierkant", "25FD"},
    {"klein_tot_middelgroot_zwart_vierkant", "25FE"},
    {"telefoon", "260E"},
    {"omhoog_wijzende_wijsvinger", "261D"},
    {"omhoog_wijzende_wijsvinger_kleur1", "261D-1F3FB"},
    {"omhoog_wijzende_wijsvinger_kleur2", "261D-1F3FC"},
    {"omhoog_wijzende_wijsvinger_kleur3", "261D-1F3FD"},
    {"omhoog_wijzende_wijsvinger_kleur4", "261D-1F3FE"},
    {"omhoog_wijzende_wijsvinger_kleur5", "261D-1F3FF"},
    {"wassende_maan_en_ster", "262A"},
    {"vredessymbool", "262E"},
    {"yin_en_yang", "262F"},
    {"lachend_gezicht", "263A"},
    {"tweelingen_sterrenbeeld", "264A"},
    {"kreeft_sterrenbeeld", "264B"},
    {"leeuw_sterrenbeeld", "264C"},
    {"maagd_sterrenbeeld", "264D"},
    {"weegschaal_sterrenbeeld", "264E"},
    {"schorpioen_sterrenbeeld", "264F"},
    {"pion", "265F"},
    {"recycling_symbool", "267B"},
    {"oneindig", "267E"},
    {"rolstoelsymbool", "267F"},
    {"atoomsymbool", "269B"},
    {"fleur_de_lis", "269C"},
    {"waarschuwing", "26A0"},
    {"hoog_voltage", "26A1"},
    {"transgendersymbool", "26A7"},
    {"witte_cirkel", "26AA"},
    {"zwarte_cirkel", "26AB"},
    {"doodskist", "26B0"},
    {"urn", "26B1"},
    {"voetbal", "26BD"},
    {"honkbal", "26BE"},
    {"sneeuwpop_zonder_sneeuwvlokken", "26C4"},
    {"zon_achter_wolk", "26C5"},
    {"wolk_met_bliksem_en_regen", "26C8"},
    {"slangendrager_sterrenbeeld", "26CE"},
    {"pikhouweel", "26CF"},
    {"helm_van_reddingswerker", "26D1"},
    {"kettingen", "26D3"},
    {"geen_toegang", "26D4"},
    {"shinto_tempel", "26E9"},
    {"kerk", "26EA"},
    {"berg", "26F0"},
    {"strandparasol", "26F1"},
    {"fontein", "26F2"},
    {"vlag_in_hole", "26F3"},
    {"veerboot", "26F4"},
    {"zeilboot", "26F5"},
    {"skier", "26F7"},
    {"schaats", "26F8"},
    {"basketbalspeler", "26F9"},
    {"basketbalspeler_kleur1", "26F9-1F3FB"},
    {"basketbalspeler_kleur2", "26F9-1F3FC"},
    {"basketbalspeler_kleur3", "26F9-1F3FD"},
    {"basketbalspeler_kleur4", "26F9-1F3FE"},
    {"basketbalspeler_kleur5", "26F9-1F3FF"},
    {"tent", "26FA"},
    {"benzinestation", "26FD"},
    {"opgeheven_vuist", "270A"},
    {"opgeheven_vuist_kleur1", "270A-1F3FB"},
    {"opgeheven_vuist_kleur2", "270A-1F3FC"},
    {"opgeheven_vuist_kleur3", "270A-1F3FD"},
    {"opgeheven_vuist_kleur4", "270A-1F3FE"},
    {"opgeheven_vuist_kleur5", "270A-1F3FF"},
    {"opgeheven_hand", "270B"},
    {"opgeheven_hand_kleur1", "270B-1F3FB"},
    {"opgeheven_hand_kleur2", "270B-1F3FC"},
    {"opgeheven_hand_kleur3", "270B-1F3FD"},
    {"opgeheven_hand_kleur4", "270B-1F3FE"},
    {"opgeheven_hand_kleur5", "270B-1F3FF"},
    {"winnende_hand", "270C"},
    {"winnende_hand_kleur1", "270C-1F3FB"},
    {"winnende_hand_kleur2", "270C-1F3FC"},
    {"winnende_hand_kleur3", "270C-1F3FD"},
    {"winnende_hand_kleur4", "270C-1F3FE"},
    {"winnende_hand_kleur5", "270C-1F3FF"},
    {"schrijvende_hand", "270D"},
    {"schrijvende_hand_kleur1", "270D-1F3FB"},
    {"schrijvende_hand_kleur2", "270D-1F3FC"},
    {"schrijvende_hand_kleur3", "270D-1F3FD"},
    {"schrijvende_hand_kleur4", "270D-1F3FE"},
    {"schrijvende_hand_kleur5", "270D-1F3FF"},
    {"potlood", "270F"},
    {"christelijk_kruis", "271D"},
    {"kruisteken", "274C"},
    {"knop_voor_kruis", "274E"},
    {"pijl_naar_rechts", "27A1"},
    {"gekrulde_lus", "27B0"},
    {"dubbele_gekrulde_lus", "27BF"},
    {"pijl_naar_links", "2B05"},
    {"pijl_omhoog", "2B06"},
    {"pijl_omlaag", "2B07"},
    {"groot_zwart_vierkant", "2B1B"},
    {"groot_wit_vierkant", "2B1C"},
    {"witte_middelgrote_ster", "2B50"},
    {"lege_rode_cirkel", "2B55"},
    {"rolwisselingsmarkering", "303D"},
    {"mahjong_rode_draak", "1F004"},
    {"jokerkaart", "1F0CF"},
    {"bloedgroep_a", "1F170"},
    {"bloedgroep_b", "1F171"},
    {"bloedgroep_o", "1F17E"},
    {"p_knop", "1F17F"},
    {"bloedgroep_ab", "1F18E"},
    {"cl_knop", "1F191"},
    {"cool_knop", "1F192"},
    {"free_knop", "1F193"},
    {"id_knop", "1F194"},
    {"new_knop", "1F195"},
    {"ng_knop", "1F196"},
    {"ok_knop", "1F197"},
    {"sos_knop", "1F198"},
    {"up_knop", "1F199"},
    {"vs_knop", "1F19A"},
    {"regionale_indicator_a", "1F1E6"},
    {"regionale_indicator_b", "1F1E7"},
    {"regionale_indicator_c", "1F1E8"},
    {"regionale_indicator_d", "1F1E9"},
    {"regionale_indicator_e", "1F1EA"},
    {"regionale_indicator_f", "1F1EB"},
    {"regionale_indicator_g", "1F1EC"},
    {"regionale_indicator_h", "1F1ED"},
    {"regionale_indicator_i", "1F1EE"},
    {"regionale_indicator_j", "1F1EF"},
    {"regionale_indicator_k", "1F1F0"},
    {"regionale_indicator_l", "1F1F1"},
    {"regionale_indicator_m", "1F1F2"},
    {"regionale_indicator_n", "1F1F3"},
    {"regionale_indicator_o", "1F1F4"},
    {"regionale_indicator_p", "1F1F5"},
    {"regionale_indicator_q", "1F1F6"},
    {"regionale_indicator_r", "1F1F7"},
    {"regionale_indicator_s", "1F1F8"},
    {"regionale_indicator_t", "1F1F9"},
    {"regionale_indicator_u", "1F1FA"},
    {"regionale_indicator_v", "1F1FB"},
    {"regionale_indicator_w", "1F1FC"},
    {"regionale_indicator_x", "1F1FD"},
    {"regionale_indicator_y", "1F1FE"},
    {"regionale_indicator_z", "1F1FF"},
    {"japans_teken_voor_hier", "1F201"},
    {"japans_teken_voor_dienstenheffing", "1F202"},
    {"japans_teken_voor_gratis", "1F21A"},
    {"japans_teken_voor_gereserveerd", "1F22F"},
    {"japans_teken_voor_verboden", "1F232"},
    {"japans_teken_voor_plaats_vrij", "1F233"},
    {"japans_teken_voor_voldoende_cijfer", "1F234"},
    {"japans_teken_voor_vol_geen_plaats", "1F235"},
    {"japans_teken_voor_niet_gratis", "1F236"},
    {"japans_teken_voor_maandelijks_bedrag", "1F237"},
    {"japans_teken_voor_toepassing", "1F238"},
    {"japans_teken_voor_korting", "1F239"},
    {"japans_teken_voor_geopend", "1F23A"},
    {"japans_teken_voor_koopje", "1F250"},
    {"japans_teken_voor_acceptabel", "1F251"},
    {"cycloon", "1F300"},
    {"mistig", "1F301"},
    {"ingevouwen_paraplu", "1F302"},
    {"sterrenhemel", "1F303"},
    {"zonsopgang_in_de_bergen", "1F304"},
    {"zonsopgang", "1F305"},
    {"stad_bij_schemering", "1F306"},
    {"zonsondergang", "1F307"},
    {"regenboog", "1F308"},
    {"brug_bij_nacht", "1F309"},
    {"golf", "1F30A"},
    {"vulkaan", "1F30B"},
    {"melkweg", "1F30C"},
    {"wereldbol_met_europa_afrika", "1F30D"},
    {"wereldbol_met_noord_en_zuid_amerika", "1F30E"},
    {"wereldbol_met_azie_australie", "1F30F"},
    {"wereldbol_met_meridianen", "1F310"},
    {"nieuwe_maan", "1F311"},
    {"jonge_maansikkel", "1F312"},
    {"maan_in_eerste_kwartier", "1F313"},
    {"wassende_maan", "1F314"},
    {"volle_maan", "1F315"},
    {"afnemende_maan", "1F316"},
    {"maan_in_laatste_kwartier", "1F317"},
    {"asgrauwe_maan", "1F318"},
    {"maansikkel", "1F319"},
    {"nieuwe_maan_met_gezicht", "1F31A"},
    {"maan_met_gezicht_in_eerste_kwartier", "1F31B"},
    {"maan_met_gezicht_in_laatste_kwartier", "1F31C"},
    {"volle_maan_met_gezicht", "1F31D"},
    {"zon_met_gezicht", "1F31E"},
    {"stralende_ster", "1F31F"},
    {"vallende_ster", "1F320"},
    {"thermometer", "1F321"},
    {"zon_achter_kleine_wolk", "1F324"},
    {"zon_achter_grote_wolk", "1F325"},
    {"zon_achter_regenwolk", "1F326"},
    {"wolk_met_regen", "1F327"},
    {"wolk_met_sneeuw", "1F328"},
    {"wolk_met_bliksem", "1F329"},
    {"tornado", "1F32A"},
    {"mist", "1F32B"},
    {"gezicht_dat_wind_blaast", "1F32C"},
    {"hotdog", "1F32D"},
    {"taco", "1F32E"},
    {"burrito", "1F32F"},
    {"kastanje", "1F330"},
    {"zaailing", "1F331"},
    {"naaldboom", "1F332"},
    {"loofboom", "1F333"},
    {"palmboom", "1F334"},
    {"cactus", "1F335"},
    {"hete_peper", "1F336"},
    {"tulp", "1F337"},
    {"kersenbloesem", "1F338"},
    {"roos", "1F339"},
    {"hibiscus", "1F33A"},
    {"zonnebloem", "1F33B"},
    {"bloesem", "1F33C"},
    {"maiskolf", "1F33D"},
    {"rijstplant", "1F33E"},
    {"kruid", "1F33F"},
    {"klavertjevier", "1F340"},
    {"esdoornblad", "1F341"},
    {"vallende_bladeren", "1F342"},
    {"blad_in_de_wind", "1F343"},
    {"paddenstoel", "1F344"},
    {"tomaat", "1F345"},
    {"aubergine", "1F346"},
    {"druiven", "1F347"},
    {"meloen", "1F348"},
    {"watermeloen", "1F349"},
    {"mandarijn", "1F34A"},
    {"citroen", "1F34B"},
    {"banaan", "1F34C"},
    {"ananas", "1F34D"},
    {"rode_appel", "1F34E"},
    {"groene_appel", "1F34F"},
    {"peer", "1F350"},
    {"perzik", "1F351"},
    {"kersen", "1F352"},
    {"aardbei", "1F353"},
    {"hamburger", "1F354"},
    {"pizzapunt", "1F355"},
    {"vlees_met_bot", "1F356"},
    {"kippenpoot", "1F357"},
    {"rijstwafel", "1F358"},
    {"rijstballetje", "1F359"},
    {"gekookte_rijst", "1F35A"},
    {"curry_en_rijst", "1F35B"},
    {"dampende_kom", "1F35C"},
    {"spaghetti", "1F35D"},
    {"brood", "1F35E"},
    {"patat", "1F35F"},
    {"gepofte_zoete_aardappel", "1F360"},
    {"dango", "1F361"},
    {"oden", "1F362"},
    {"sushi", "1F363"},
    {"gefrituurde_garnaal", "1F364"},
    {"viskoekje_met_krul", "1F365"},
    {"softijs", "1F366"},
    {"geschaafd_ijs", "1F367"},
    {"ijs", "1F368"},
    {"donut", "1F369"},
    {"koekje", "1F36A"},
    {"chocoladereep", "1F36B"},
    {"snoep", "1F36C"},
    {"lolly", "1F36D"},
    {"vla", "1F36E"},
    {"honingpot", "1F36F"},
    {"cake", "1F370"},
    {"bentobox", "1F371"},
    {"stoofschotel", "1F372"},
    {"koken", "1F373"},
    {"mes_en_vork", "1F374"},
    {"theekopje_zonder_oor", "1F375"},
    {"sake", "1F376"},
    {"wijnglas", "1F377"},
    {"cocktailglas", "1F378"},
    {"tropisch_drankje", "1F379"},
    {"bierglas", "1F37A"},
    {"klinkende_bierglazen", "1F37B"},
    {"babyflesje", "1F37C"},
    {"mes_en_vork_met_bord", "1F37D"},
    {"fles_met_knallende_kurk", "1F37E"},
    {"popcorn", "1F37F"},
    {"strik", "1F380"},
    {"ingepakt_cadeau", "1F381"},
    {"verjaardagstaart", "1F382"},
    {"halloweenlantaarn", "1F383"},
    {"kerstboom", "1F384"},
    {"kerstman", "1F385"},
    {"kerstman_kleur1", "1F385-1F3FB"},
    {"kerstman_kleur2", "1F385-1F3FC"},
    {"kerstman_kleur3", "1F385-1F3FD"},
    {"kerstman_kleur4", "1F385-1F3FE"},
    {"kerstman_kleur5", "1F385-1F3FF"},
    {"vuurwerk", "1F386"},
    {"sterretje", "1F387"},
    {"ballon", "1F388"},
    {"feestknaller", "1F389"},
    {"confettibal", "1F38A"},
    {"tanabataboom", "1F38B"},
    {"gekruiste_vlaggen", "1F38C"},
    {"dennenboomversiering", "1F38D"},
    {"japanse_poppen", "1F38E"},
    {"karperslinger", "1F38F"},
    {"windorgel", "1F390"},
    {"maanobservatieceremonie", "1F391"},
    {"schooltas", "1F392"},
    {"academische_baret", "1F393"},
    {"militaire_medaille", "1F396"},
    {"bewustzijnslintje", "1F397"},
    {"studiomicrofoon", "1F399"},
    {"schuifregelaar", "1F39A"},
    {"controleknoppen", "1F39B"},
    {"filmframes", "1F39E"},
    {"entreekaartjes", "1F39F"},
    {"draaimolenpaard", "1F3A0"},
    {"reuzenrad", "1F3A1"},
    {"achtbaan", "1F3A2"},
    {"vishengel", "1F3A3"},
    {"microfoon", "1F3A4"},
    {"filmcamera", "1F3A5"},
    {"knop_voor_filmen", "1F3A6"},
    {"hoofdtelefoon", "1F3A7"},
    {"schilderspalet", "1F3A8"},
    {"hoge_hoed", "1F3A9"},
    {"circustent", "1F3AA"},
    {"ticket", "1F3AB"},
    {"klapbord", "1F3AC"},
    {"toneelkunsten", "1F3AD"},
    {"videogame", "1F3AE"},
    {"directe_treffer", "1F3AF"},
    {"fruitautomaat", "1F3B0"},
    {"biljart", "1F3B1"},
    {"dobbelsteen", "1F3B2"},
    {"bowlen", "1F3B3"},
    {"speelkaarten", "1F3B4"},
    {"muzieknoot", "1F3B5"},
    {"muzieknoten", "1F3B6"},
    {"saxofoon", "1F3B7"},
    {"gitaar", "1F3B8"},
    {"piano", "1F3B9"},
    {"trompet", "1F3BA"},
    {"viool", "1F3BB"},
    {"notenschift", "1F3BC"},
    {"hardloopshirt", "1F3BD"},
    {"tennis", "1F3BE"},
    {"skis", "1F3BF"},
    {"basketbal", "1F3C0"},
    {"geblokte_vlag", "1F3C1"},
    {"snowboarder", "1F3C2"},
    {"snowboarder_kleur1", "1F3C2-1F3FB"},
    {"snowboarder_kleur2", "1F3C2-1F3FC"},
    {"snowboarder_kleur3", "1F3C2-1F3FD"},
    {"snowboarder_kleur4", "1F3C2-1F3FE"},
    {"snowboarder_kleur5", "1F3C2-1F3FF"},
    {"hardloper", "1F3C3"},
    {"hardloper_kleur1", "1F3C3-1F3FB"},
    {"hardloper_kleur2", "1F3C3-1F3FC"},
    {"hardloper_kleur3", "1F3C3-1F3FD"},
    {"hardloper_kleur4", "1F3C3-1F3FE"},
    {"hardloper_kleur5", "1F3C3-1F3FF"},
    {"surfer", "1F3C4"},
    {"surfer_kleur1", "1F3C4-1F3FB"},
    {"surfer_kleur2", "1F3C4-1F3FC"},
    {"surfer_kleur3", "1F3C4-1F3FD"},
    {"surfer_kleur4", "1F3C4-1F3FE"},
    {"surfer_kleur5", "1F3C4-1F3FF"},
    {"sportmedaille", "1F3C5"},
    {"trofee", "1F3C6"},
    {"jockey_op_renpaard", "1F3C7"},
    {"jockey_op_renpaard_kleur1", "1F3C7-1F3FB"},
    {"jockey_op_renpaard_kleur2", "1F3C7-1F3FC"},
    {"jockey_op_renpaard_kleur3", "1F3C7-1F3FD"},
    {"jockey_op_renpaard_kleur4", "1F3C7-1F3FE"},
    {"jockey_op_renpaard_kleur5", "1F3C7-1F3FF"},
    {"american_football", "1F3C8"},
    {"rugby", "1F3C9"},
    {"zwemmende_persoon", "1F3CA"},
    {"zwemmende_persoon_kleur1", "1F3CA-1F3FB"},
    {"zwemmende_persoon_kleur2", "1F3CA-1F3FC"},
    {"zwemmende_persoon_kleur3", "1F3CA-1F3FD"},
    {"zwemmende_persoon_kleur4", "1F3CA-1F3FE"},
    {"zwemmende_persoon_kleur5", "1F3CA-1F3FF"},
    {"gewichtheffer", "1F3CB"},
    {"gewichtheffer_kleur1", "1F3CB-1F3FB"},
    {"gewichtheffer_kleur2", "1F3CB-1F3FC"},
    {"gewichtheffer_kleur3", "1F3CB-1F3FD"},
    {"gewichtheffer_kleur4", "1F3CB-1F3FE"},
    {"gewichtheffer_kleur5", "1F3CB-1F3FF"},
    {"golfer", "1F3CC"},
    {"golfer_kleur1", "1F3CC-1F3FB"},
    {"golfer_kleur2", "1F3CC-1F3FC"},
    {"golfer_kleur3", "1F3CC-1F3FD"},
    {"golfer_kleur4", "1F3CC-1F3FE"},
    {"golfer_kleur5", "1F3CC-1F3FF"},
    {"motor", "1F3CD"},
    {"raceauto", "1F3CE"},
    {"cricket", "1F3CF"},
    {"volleybal", "1F3D0"},
    {"hockey", "1F3D1"},
    {"ijshockeystick_en_puck", "1F3D2"},
    {"pingpong", "1F3D3"},
    {"berg_met_besneeuwde_toppen", "1F3D4"},
    {"kamperen", "1F3D5"},
    {"strand_met_parasol", "1F3D6"},
    {"woningbouw", "1F3D7"},
    {"huizen", "1F3D8"},
    {"stadsgezicht", "1F3D9"},
    {"vervallen_huis", "1F3DA"},
    {"klassiek_gebouw", "1F3DB"},
    {"woestijn", "1F3DC"},
    {"onbewoond_eiland", "1F3DD"},
    {"nationaal_park", "1F3DE"},
    {"stadion", "1F3DF"},
    {"huis", "1F3E0"},
    {"huis_met_tuin", "1F3E1"},
    {"kantoorgebouw", "1F3E2"},
    {"japans_postkantoor", "1F3E3"},
    {"postkantoor", "1F3E4"},
    {"ziekenhuis", "1F3E5"},
    {"bank", "1F3E6"},
    {"pinautomaat", "1F3E7"},
    {"hotel", "1F3E8"},
    {"love_hotel", "1F3E9"},
    {"supermarkt", "1F3EA"},
    {"school", "1F3EB"},
    {"warenhuis", "1F3EC"},
    {"fabriek", "1F3ED"},
    {"rode_papieren_lantaarn", "1F3EE"},
    {"japans_kasteel", "1F3EF"},
    {"kasteel", "1F3F0"},
    {"witte_vlag", "1F3F3"},
    {"zwarte_vlag", "1F3F4"},
    {"rozet", "1F3F5"},
    {"label", "1F3F7"},
    {"badminton", "1F3F8"},
    {"pijl_en_boog", "1F3F9"},
    {"amfora", "1F3FA"},
    {"lichte_huidskleur", "1F3FB"},
    {"lichtgetinte_huidskleur", "1F3FC"},
    {"getinte_huidskleur", "1F3FD"},
    {"donkergetinte_huidskleur", "1F3FE"},
    {"donkere_huidskleur", "1F3FF"},
    {"rat", "1F400"},
    {"muis", "1F401"},
    {"os", "1F402"},
    {"waterbuffel", "1F403"},
    {"koe", "1F404"},
    {"tijger", "1F405"},
    {"luipaard", "1F406"},
    {"konijn", "1F407"},
    {"kat", "1F408"},
    {"draak", "1F409"},
    {"krokodil", "1F40A"},
    {"walvis", "1F40B"},
    {"slak", "1F40C"},
    {"slang", "1F40D"},
    {"paard", "1F40E"},
    {"ram", "1F40F"},
    {"geit", "1F410"},
    {"ooi", "1F411"},
    {"aap", "1F412"},
    {"haan", "1F413"},
    {"kip", "1F414"},
    {"hond", "1F415"},
    {"varken", "1F416"},
    {"zwijn", "1F417"},
    {"olifant", "1F418"},
    {"octopus", "1F419"},
    {"schelp", "1F41A"},
    {"insect", "1F41B"},
    {"mier", "1F41C"},
    {"honingbij", "1F41D"},
    {"lieveheersbeestje", "1F41E"},
    {"vis", "1F41F"},
    {"tropische_vis", "1F420"},
    {"kogelvis", "1F421"},
    {"schildpad", "1F422"},
    {"kuiken_uit_ei", "1F423"},
    {"kuikentje", "1F424"},
    {"naar_voren_kijkend_kuikentje", "1F425"},
    {"vogel", "1F426"},
    {"pinguin", "1F427"},
    {"koala", "1F428"},
    {"poedel", "1F429"},
    {"dromedaris", "1F42A"},
    {"kameel", "1F42B"},
    {"dolfijn", "1F42C"},
    {"muizengezicht", "1F42D"},
    {"koeiengezicht", "1F42E"},
    {"tijgergezicht", "1F42F"},
    {"konijnengezicht", "1F430"},
    {"kattengezicht", "1F431"},
    {"drakengezicht", "1F432"},
    {"spuitende_walvis", "1F433"},
    {"paardengezicht", "1F434"},
    {"apengezicht", "1F435"},
    {"hondengezicht", "1F436"},
    {"varkensgezicht", "1F437"},
    {"kikker", "1F438"},
    {"hamster", "1F439"},
    {"wolf", "1F43A"},
    {"beer", "1F43B"},
    {"panda", "1F43C"},
    {"varkensneus", "1F43D"},
    {"pootafdrukken", "1F43E"},
    {"eekhoorn", "1F43F"},
    {"ogen", "1F440"},
    {"oog", "1F441"},
    {"oor", "1F442"},
    {"oor_kleur1", "1F442-1F3FB"},
    {"oor_kleur2", "1F442-1F3FC"},
    {"oor_kleur3", "1F442-1F3FD"},
    {"oor_kleur4", "1F442-1F3FE"},
    {"oor_kleur5", "1F442-1F3FF"},
    {"neus", "1F443"},
    {"neus_kleur1", "1F443-1F3FB"},
    {"neus_kleur2", "1F443-1F3FC"},
    {"neus_kleur3", "1F443-1F3FD"},
    {"neus_kleur4", "1F443-1F3FE"},
    {"neus_kleur5", "1F443-1F3FF"},
    {"mond", "1F444"},
    {"tong", "1F445"},
    {"achterkant_van_hand_met_omhoog_wijzende_wijsvinger", "1F446"},
    {"achterkant_van_hand_met_omhoog_wijzende_wijsvinger_kleur1", "1F446-1F3FB"},
    {"achterkant_van_hand_met_omhoog_wijzende_wijsvinger_kleur2", "1F446-1F3FC"},
    {"achterkant_van_hand_met_omhoog_wijzende_wijsvinger_kleur3", "1F446-1F3FD"},
    {"achterkant_van_hand_met_omhoog_wijzende_wijsvinger_kleur4", "1F446-1F3FE"},
    {"achterkant_van_hand_met_omhoog_wijzende_wijsvinger_kleur5", "1F446-1F3FF"},
    {"achterkant_van_hand_met_omlaag_wijzende_wijsvinger", "1F447"},
    {"achterkant_van_hand_met_omlaag_wijzende_wijsvinger_kleur1", "1F447-1F3FB"},
    {"achterkant_van_hand_met_omlaag_wijzende_wijsvinger_kleur2", "1F447-1F3FC"},
    {"achterkant_van_hand_met_omlaag_wijzende_wijsvinger_kleur3", "1F447-1F3FD"},
    {"achterkant_van_hand_met_omlaag_wijzende_wijsvinger_kleur4", "1F447-1F3FE"},
    {"achterkant_van_hand_met_omlaag_wijzende_wijsvinger_kleur5", "1F447-1F3FF"},
    {"achterkant_van_hand_met_naar_links_wijzende_wijsvinger", "1F448"},
    {"achterkant_van_hand_met_naar_links_wijzende_wijsvinger_kleur1", "1F448-1F3FB"},
    {"achterkant_van_hand_met_naar_links_wijzende_wijsvinger_kleur2", "1F448-1F3FC"},
    {"achterkant_van_hand_met_naar_links_wijzende_wijsvinger_kleur3", "1F448-1F3FD"},
    {"achterkant_van_hand_met_naar_links_wijzende_wijsvinger_kleur4", "1F448-1F3FE"},
    {"achterkant_van_hand_met_naar_links_wijzende_wijsvinger_kleur5", "1F448-1F3FF"},
    {"achterkant_van_hand_met_naar_rechts_wijzende_wijsvinger", "1F449"},
    {"achterkant_van_hand_met_naar_rechts_wijzende_wijsvinger_kleur1", "1F449-1F3FB"},
    {"achterkant_van_hand_met_naar_rechts_wijzende_wijsvinger_kleur2", "1F449-1F3FC"},
    {"achterkant_van_hand_met_naar_rechts_wijzende_wijsvinger_kleur3", "1F449-1F3FD"},
    {"achterkant_van_hand_met_naar_rechts_wijzende_wijsvinger_kleur4", "1F449-1F3FE"},
    {"achterkant_van_hand_met_naar_rechts_wijzende_wijsvinger_kleur5", "1F449-1F3FF"},
    {"stotende_vuist", "1F44A"},
    {"stotende_vuist_kleur1", "1F44A-1F3FB"},
    {"stotende_vuist_kleur2", "1F44A-1F3FC"},
    {"stotende_vuist_kleur3", "1F44A-1F3FD"},
    {"stotende_vuist_kleur4", "1F44A-1F3FE"},
    {"stotende_vuist_kleur5", "1F44A-1F3FF"},
    {"zwaaiende_hand", "1F44B"},
    {"zwaaiende_hand_kleur1", "1F44B-1F3FB"},
    {"zwaaiende_hand_kleur2", "1F44B-1F3FC"},
    {"zwaaiende_hand_kleur3", "1F44B-1F3FD"},
    {"zwaaiende_hand_kleur4", "1F44B-1F3FE"},
    {"zwaaiende_hand_kleur5", "1F44B-1F3FF"},
    {"ok_handgebaar", "1F44C"},
    {"ok_handgebaar_kleur1", "1F44C-1F3FB"},
    {"ok_handgebaar_kleur2", "1F44C-1F3FC"},
    {"ok_handgebaar_kleur3", "1F44C-1F3FD"},
    {"ok_handgebaar_kleur4", "1F44C-1F3FE"},
    {"ok_handgebaar_kleur5", "1F44C-1F3FF"},
    {"duim_omhoog", "1F44D"},
    {"duim_omhoog_kleur1", "1F44D-1F3FB"},
    {"duim_omhoog_kleur2", "1F44D-1F3FC"},
    {"duim_omhoog_kleur3", "1F44D-1F3FD"},
    {"duim_omhoog_kleur4", "1F44D-1F3FE"},
    {"duim_omhoog_kleur5", "1F44D-1F3FF"},
    {"duim_omlaag", "1F44E"},
    {"duim_omlaag_kleur1", "1F44E-1F3FB"},
    {"duim_omlaag_kleur2", "1F44E-1F3FC"},
    {"duim_omlaag_kleur3", "1F44E-1F3FD"},
    {"duim_omlaag_kleur4", "1F44E-1F3FE"},
    {"duim_omlaag_kleur5", "1F44E-1F3FF"},
    {"klappende_handen", "1F44F"},
    {"klappende_handen_kleur1", "1F44F-1F3FB"},
    {"klappende_handen_kleur2", "1F44F-1F3FC"},
    {"klappende_handen_kleur3", "1F44F-1F3FD"},
    {"klappende_handen_kleur4", "1F44F-1F3FE"},
    {"klappende_handen_kleur5", "1F44F-1F3FF"},
    {"open_handen", "1F450"},
    {"open_handen_kleur1", "1F450-1F3FB"},
    {"open_handen_kleur2", "1F450-1F3FC"},
    {"open_handen_kleur3", "1F450-1F3FD"},
    {"open_handen_kleur4", "1F450-1F3FE"},
    {"open_handen_kleur5", "1F450-1F3FF"},
    {"kroon", "1F451"},
    {"dameshoed", "1F452"},
    {"bril", "1F453"},
    {"stropdas", "1F454"},
    {"t_shirt", "1F455"},
    {"spijkerbroek", "1F456"},
    {"jurk", "1F457"},
    {"kimono", "1F458"},
    {"bikini", "1F459"},
    {"dameskleding", "1F45A"},
    {"portemonnee", "1F45B"},
    {"handtas", "1F45C"},
    {"tasje", "1F45D"},
    {"herenschoen", "1F45E"},
    {"sportschoen", "1F45F"},
    {"schoen_met_hoge_hak", "1F460"},
    {"damessandaal", "1F461"},
    {"dameslaars", "1F462"},
    {"voetafdrukken", "1F463"},
    {"silhouet_van_persoon", "1F464"},
    {"silhouet_van_twee_personen", "1F465"},
    {"jongen", "1F466"},
    {"jongen_kleur1", "1F466-1F3FB"},
    {"jongen_kleur2", "1F466-1F3FC"},
    {"jongen_kleur3", "1F466-1F3FD"},
    {"jongen_kleur4", "1F466-1F3FE"},
    {"jongen_kleur5", "1F466-1F3FF"},
    {"meisje", "1F467"},
    {"meisje_kleur1", "1F467-1F3FB"},
    {"meisje_kleur2", "1F467-1F3FC"},
    {"meisje_kleur3", "1F467-1F3FD"},
    {"meisje_kleur4", "1F467-1F3FE"},
    {"meisje_kleur5", "1F467-1F3FF"},
    {"man", "1F468"},
    {"man_kleur1", "1F468-1F3FB"},
    {"man_kleur2", "1F468-1F3FC"},
    {"man_kleur3", "1F468-1F3FD"},
    {"man_kleur4", "1F468-1F3FE"},
    {"man_kleur5", "1F468-1F3FF"},
    {"vrouw", "1F469"},
    {"vrouw_kleur1", "1F469-1F3FB"},
    {"vrouw_kleur2", "1F469-1F3FC"},
    {"vrouw_kleur3", "1F469-1F3FD"},
    {"vrouw_kleur4", "1F469-1F3FE"},
    {"vrouw_kleur5", "1F469-1F3FF"},
    {"gezin", "1F46A"},
    {"man_en_vrouw_hand_in_hand", "1F46B"},
    {"man_en_vrouw_hand_in_hand_kleur1", "1F46B-1F3FB"},
    {"man_en_vrouw_hand_in_hand_kleur2", "1F46B-1F3FC"},
    {"man_en_vrouw_hand_in_hand_kleur3", "1F46B-1F3FD"},
    {"man_en_vrouw_hand_in_hand_kleur4", "1F46B-1F3FE"},
    {"man_en_vrouw_hand_in_hand_kleur5", "1F46B-1F3FF"},
    {"man_en_vrouw_hand_in_hand_kleur1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"man_en_vrouw_hand_in_hand_kleur1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"man_en_vrouw_hand_in_hand_kleur1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"man_en_vrouw_hand_in_hand_kleur1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"man_en_vrouw_hand_in_hand_kleur2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"man_en_vrouw_hand_in_hand_kleur2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"man_en_vrouw_hand_in_hand_kleur2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"man_en_vrouw_hand_in_hand_kleur2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"man_en_vrouw_hand_in_hand_kleur3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"man_en_vrouw_hand_in_hand_kleur3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"man_en_vrouw_hand_in_hand_kleur3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"man_en_vrouw_hand_in_hand_kleur3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"man_en_vrouw_hand_in_hand_kleur4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"man_en_vrouw_hand_in_hand_kleur4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"man_en_vrouw_hand_in_hand_kleur4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"man_en_vrouw_hand_in_hand_kleur4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"man_en_vrouw_hand_in_hand_kleur5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"man_en_vrouw_hand_in_hand_kleur5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"man_en_vrouw_hand_in_hand_kleur5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"man_en_vrouw_hand_in_hand_kleur5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"mannen_hand_in_hand", "1F46C"},
    {"mannen_hand_in_hand_kleur1", "1F46C-1F3FB"},
    {"mannen_hand_in_hand_kleur2", "1F46C-1F3FC"},
    {"mannen_hand_in_hand_kleur3", "1F46C-1F3FD"},
    {"mannen_hand_in_hand_kleur4", "1F46C-1F3FE"},
    {"mannen_hand_in_hand_kleur5", "1F46C-1F3FF"},
    {"mannen_hand_in_hand_kleur1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mannen_hand_in_hand_kleur1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mannen_hand_in_hand_kleur1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mannen_hand_in_hand_kleur1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mannen_hand_in_hand_kleur2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mannen_hand_in_hand_kleur2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mannen_hand_in_hand_kleur2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mannen_hand_in_hand_kleur2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mannen_hand_in_hand_kleur3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mannen_hand_in_hand_kleur3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mannen_hand_in_hand_kleur3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mannen_hand_in_hand_kleur3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mannen_hand_in_hand_kleur4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mannen_hand_in_hand_kleur4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mannen_hand_in_hand_kleur4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mannen_hand_in_hand_kleur4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mannen_hand_in_hand_kleur5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mannen_hand_in_hand_kleur5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mannen_hand_in_hand_kleur5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mannen_hand_in_hand_kleur5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"vrouwen_hand_in_hand", "1F46D"},
    {"vrouwen_hand_in_hand_kleur1", "1F46D-1F3FB"},
    {"vrouwen_hand_in_hand_kleur2", "1F46D-1F3FC"},
    {"vrouwen_hand_in_hand_kleur3", "1F46D-1F3FD"},
    {"vrouwen_hand_in_hand_kleur4", "1F46D-1F3FE"},
    {"vrouwen_hand_in_hand_kleur5", "1F46D-1F3FF"},
    {"vrouwen_hand_in_hand_kleur1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"vrouwen_hand_in_hand_kleur1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"vrouwen_hand_in_hand_kleur1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"vrouwen_hand_in_hand_kleur1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"vrouwen_hand_in_hand_kleur2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"vrouwen_hand_in_hand_kleur2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"vrouwen_hand_in_hand_kleur2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"vrouwen_hand_in_hand_kleur2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"vrouwen_hand_in_hand_kleur3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"vrouwen_hand_in_hand_kleur3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"vrouwen_hand_in_hand_kleur3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"vrouwen_hand_in_hand_kleur3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"vrouwen_hand_in_hand_kleur4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"vrouwen_hand_in_hand_kleur4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"vrouwen_hand_in_hand_kleur4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"vrouwen_hand_in_hand_kleur4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"vrouwen_hand_in_hand_kleur5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"vrouwen_hand_in_hand_kleur5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"vrouwen_hand_in_hand_kleur5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"vrouwen_hand_in_hand_kleur5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"politieagent", "1F46E"},
    {"politieagent_kleur1", "1F46E-1F3FB"},
    {"politieagent_kleur2", "1F46E-1F3FC"},
    {"politieagent_kleur3", "1F46E-1F3FD"},
    {"politieagent_kleur4", "1F46E-1F3FE"},
    {"politieagent_kleur5", "1F46E-1F3FF"},
    {"mensen_met_konijnenoren", "1F46F"},
    {"mensen_met_konijnenoren_kleur1", "1F46F-1F3FB"},
    {"mensen_met_konijnenoren_kleur2", "1F46F-1F3FC"},
    {"mensen_met_konijnenoren_kleur3", "1F46F-1F3FD"},
    {"mensen_met_konijnenoren_kleur4", "1F46F-1F3FE"},
    {"mensen_met_konijnenoren_kleur5", "1F46F-1F3FF"},
    {"mensen_met_konijnenoren_kleur1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"mensen_met_konijnenoren_kleur1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"mensen_met_konijnenoren_kleur1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"mensen_met_konijnenoren_kleur1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"mensen_met_konijnenoren_kleur2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"mensen_met_konijnenoren_kleur2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"mensen_met_konijnenoren_kleur2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"mensen_met_konijnenoren_kleur2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"mensen_met_konijnenoren_kleur3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"mensen_met_konijnenoren_kleur3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"mensen_met_konijnenoren_kleur3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"mensen_met_konijnenoren_kleur3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"mensen_met_konijnenoren_kleur4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"mensen_met_konijnenoren_kleur4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"mensen_met_konijnenoren_kleur4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"mensen_met_konijnenoren_kleur4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"mensen_met_konijnenoren_kleur5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"mensen_met_konijnenoren_kleur5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"mensen_met_konijnenoren_kleur5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"mensen_met_konijnenoren_kleur5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"persoon_met_sluier", "1F470"},
    {"persoon_met_sluier_kleur1", "1F470-1F3FB"},
    {"persoon_met_sluier_kleur2", "1F470-1F3FC"},
    {"persoon_met_sluier_kleur3", "1F470-1F3FD"},
    {"persoon_met_sluier_kleur4", "1F470-1F3FE"},
    {"persoon_met_sluier_kleur5", "1F470-1F3FF"},
    {"persoon_met_blond_haar", "1F471"},
    {"persoon_met_blond_haar_kleur1", "1F471-1F3FB"},
    {"persoon_met_blond_haar_kleur2", "1F471-1F3FC"},
    {"persoon_met_blond_haar_kleur3", "1F471-1F3FD"},
    {"persoon_met_blond_haar_kleur4", "1F471-1F3FE"},
    {"persoon_met_blond_haar_kleur5", "1F471-1F3FF"},
    {"man_met_chinees_petje", "1F472"},
    {"man_met_chinees_petje_kleur1", "1F472-1F3FB"},
    {"man_met_chinees_petje_kleur2", "1F472-1F3FC"},
    {"man_met_chinees_petje_kleur3", "1F472-1F3FD"},
    {"man_met_chinees_petje_kleur4", "1F472-1F3FE"},
    {"man_met_chinees_petje_kleur5", "1F472-1F3FF"},
    {"persoon_met_tulband", "1F473"},
    {"persoon_met_tulband_kleur1", "1F473-1F3FB"},
    {"persoon_met_tulband_kleur2", "1F473-1F3FC"},
    {"persoon_met_tulband_kleur3", "1F473-1F3FD"},
    {"persoon_met_tulband_kleur4", "1F473-1F3FE"},
    {"persoon_met_tulband_kleur5", "1F473-1F3FF"},
    {"oude_man", "1F474"},
    {"oude_man_kleur1", "1F474-1F3FB"},
    {"oude_man_kleur2", "1F474-1F3FC"},
    {"oude_man_kleur3", "1F474-1F3FD"},
    {"oude_man_kleur4", "1F474-1F3FE"},
    {"oude_man_kleur5", "1F474-1F3FF"},
    {"oude_vrouw", "1F475"},
    {"oude_vrouw_kleur1", "1F475-1F3FB"},
    {"oude_vrouw_kleur2", "1F475-1F3FC"},
    {"oude_vrouw_kleur3", "1F475-1F3FD"},
    {"oude_vrouw_kleur4", "1F475-1F3FE"},
    {"oude_vrouw_kleur5", "1F475-1F3FF"},
    {"baby", "1F476"},
    {"baby_kleur1", "1F476-1F3FB"},
    {"baby_kleur2", "1F476-1F3FC"},
    {"baby_kleur3", "1F476-1F3FD"},
    {"baby_kleur4", "1F476-1F3FE"},
    {"baby_kleur5", "1F476-1F3FF"},
    {"bouwvakker", "1F477"},
    {"bouwvakker_kleur1", "1F477-1F3FB"},
    {"bouwvakker_kleur2", "1F477-1F3FC"},
    {"bouwvakker_kleur3", "1F477-1F3FD"},
    {"bouwvakker_kleur4", "1F477-1F3FE"},
    {"bouwvakker_kleur5", "1F477-1F3FF"},
    {"prinses", "1F478"},
    {"prinses_kleur1", "1F478-1F3FB"},
    {"prinses_kleur2", "1F478-1F3FC"},
    {"prinses_kleur3", "1F478-1F3FD"},
    {"prinses_kleur4", "1F478-1F3FE"},
    {"prinses_kleur5", "1F478-1F3FF"},
    {"boeman", "1F479"},
    {"kobold", "1F47A"},
    {"spook", "1F47B"},
    {"baby_engel", "1F47C"},
    {"baby_engel_kleur1", "1F47C-1F3FB"},
    {"baby_engel_kleur2", "1F47C-1F3FC"},
    {"baby_engel_kleur3", "1F47C-1F3FD"},
    {"baby_engel_kleur4", "1F47C-1F3FE"},
    {"baby_engel_kleur5", "1F47C-1F3FF"},
    {"buitenaards_wezen", "1F47D"},
    {"buitenaards_monster", "1F47E"},
    {"boos_gezicht_met_hoorns", "1F47F"},
    {"schedel", "1F480"},
    {"persoon_achter_informatiebalie", "1F481"},
    {"persoon_achter_informatiebalie_kleur1", "1F481-1F3FB"},
    {"persoon_achter_informatiebalie_kleur2", "1F481-1F3FC"},
    {"persoon_achter_informatiebalie_kleur3", "1F481-1F3FD"},
    {"persoon_achter_informatiebalie_kleur4", "1F481-1F3FE"},
    {"persoon_achter_informatiebalie_kleur5", "1F481-1F3FF"},
    {"wachter", "1F482"},
    {"wachter_kleur1", "1F482-1F3FB"},
    {"wachter_kleur2", "1F482-1F3FC"},
    {"wachter_kleur3", "1F482-1F3FD"},
    {"wachter_kleur4", "1F482-1F3FE"},
    {"wachter_kleur5", "1F482-1F3FF"},
    {"dansende_vrouw", "1F483"},
    {"dansende_vrouw_kleur1", "1F483-1F3FB"},
    {"dansende_vrouw_kleur2", "1F483-1F3FC"},
    {"dansende_vrouw_kleur3", "1F483-1F3FD"},
    {"dansende_vrouw_kleur4", "1F483-1F3FE"},
    {"dansende_vrouw_kleur5", "1F483-1F3FF"},
    {"lippenstift", "1F484"},
    {"nagellak", "1F485"},
    {"nagellak_kleur1", "1F485-1F3FB"},
    {"nagellak_kleur2", "1F485-1F3FC"},
    {"nagellak_kleur3", "1F485-1F3FD"},
    {"nagellak_kleur4", "1F485-1F3FE"},
    {"nagellak_kleur5", "1F485-1F3FF"},
    {"persoon_die_hoofdmassage_krijgt", "1F486"},
    {"persoon_die_hoofdmassage_krijgt_kleur1", "1F486-1F3FB"},
    {"persoon_die_hoofdmassage_krijgt_kleur2", "1F486-1F3FC"},
    {"persoon_die_hoofdmassage_krijgt_kleur3", "1F486-1F3FD"},
    {"persoon_die_hoofdmassage_krijgt_kleur4", "1F486-1F3FE"},
    {"persoon_die_hoofdmassage_krijgt_kleur5", "1F486-1F3FF"},
    {"persoon_die_wordt_geknipt", "1F487"},
    {"persoon_die_wordt_geknipt_kleur1", "1F487-1F3FB"},
    {"persoon_die_wordt_geknipt_kleur2", "1F487-1F3FC"},
    {"persoon_die_wordt_geknipt_kleur3", "1F487-1F3FD"},
    {"persoon_die_wordt_geknipt_kleur4", "1F487-1F3FE"},
    {"persoon_die_wordt_geknipt_kleur5", "1F487-1F3FF"},
    {"kapperspaal", "1F488"},
    {"injectiespuit", "1F489"},
    {"pil", "1F48A"},
    {"lipafdruk", "1F48B"},
    {"liefdesbrief", "1F48C"},
    {"ring", "1F48D"},
    {"edelsteen", "1F48E"},
    {"kus", "1F48F"},
    {"kus_kleur1", "1F48F-1F3FB"},
    {"kus_kleur2", "1F48F-1F3FC"},
    {"kus_kleur3", "1F48F-1F3FD"},
    {"kus_kleur4", "1F48F-1F3FE"},
    {"kus_kleur5", "1F48F-1F3FF"},
    {"kus_kleur1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"kus_kleur1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"kus_kleur1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"kus_kleur1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"kus_kleur2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"kus_kleur2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"kus_kleur2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"kus_kleur2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"kus_kleur3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"kus_kleur3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"kus_kleur3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"kus_kleur3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"kus_kleur4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"kus_kleur4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"kus_kleur4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"kus_kleur4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"kus_kleur5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"kus_kleur5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"kus_kleur5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"kus_kleur5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"boeket", "1F490"},
    {"stel_met_hart", "1F491"},
    {"stel_met_hart_kleur1", "1F491-1F3FB"},
    {"stel_met_hart_kleur2", "1F491-1F3FC"},
    {"stel_met_hart_kleur3", "1F491-1F3FD"},
    {"stel_met_hart_kleur4", "1F491-1F3FE"},
    {"stel_met_hart_kleur5", "1F491-1F3FF"},
    {"stel_met_hart_kleur1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"stel_met_hart_kleur1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"stel_met_hart_kleur1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"stel_met_hart_kleur1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"stel_met_hart_kleur2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"stel_met_hart_kleur2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"stel_met_hart_kleur2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"stel_met_hart_kleur2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"stel_met_hart_kleur3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"stel_met_hart_kleur3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"stel_met_hart_kleur3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"stel_met_hart_kleur3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"stel_met_hart_kleur4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"stel_met_hart_kleur4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"stel_met_hart_kleur4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"stel_met_hart_kleur4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"stel_met_hart_kleur5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"stel_met_hart_kleur5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"stel_met_hart_kleur5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"stel_met_hart_kleur5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"bruiloft", "1F492"},
    {"kloppend_hart", "1F493"},
    {"gebroken_hart", "1F494"},
    {"twee_harten", "1F495"},
    {"fonkelend_hart", "1F496"},
    {"groeiend_hart", "1F497"},
    {"hart_met_pijl", "1F498"},
    {"blauw_hart", "1F499"},
    {"groen_hart", "1F49A"},
    {"geel_hart", "1F49B"},
    {"paars_hart", "1F49C"},
    {"hart_met_strik", "1F49D"},
    {"ronddraaiende_harten", "1F49E"},
    {"decoratief_hart", "1F49F"},
    {"ruitvorm_met_stip_erin", "1F4A0"},
    {"gloeilamp", "1F4A1"},
    {"woedesymbool", "1F4A2"},
    {"bom", "1F4A3"},
    {"slaapsymbool", "1F4A4"},
    {"botsing", "1F4A5"},
    {"zweetdruppels", "1F4A6"},
    {"druppel", "1F4A7"},
    {"wegrennen", "1F4A8"},
    {"drol", "1F4A9"},
    {"spierballen", "1F4AA"},
    {"spierballen_kleur1", "1F4AA-1F3FB"},
    {"spierballen_kleur2", "1F4AA-1F3FC"},
    {"spierballen_kleur3", "1F4AA-1F3FD"},
    {"spierballen_kleur4", "1F4AA-1F3FE"},
    {"spierballen_kleur5", "1F4AA-1F3FF"},
    {"duizelig", "1F4AB"},
    {"spraakballon", "1F4AC"},
    {"gedachteballon", "1F4AD"},
    {"witte_bloem", "1F4AE"},
    {"maximumscore", "1F4AF"},
    {"zak_met_geld", "1F4B0"},
    {"geld_wisselen", "1F4B1"},
    {"vetgedrukt_dollarteken", "1F4B2"},
    {"creditcard", "1F4B3"},
    {"yenbiljet", "1F4B4"},
    {"dollarbiljet", "1F4B5"},
    {"eurobiljet", "1F4B6"},
    {"pondbiljet", "1F4B7"},
    {"vliegend_geld", "1F4B8"},
    {"stijgende_grafiek_met_yenteken", "1F4B9"},
    {"stoel", "1F4BA"},
    {"laptop", "1F4BB"},
    {"aktentas", "1F4BC"},
    {"harddisk", "1F4BD"},
    {"floppydisk", "1F4BE"},
    {"optische_schijf", "1F4BF"},
    {"dvd", "1F4C0"},
    {"bestandsmap", "1F4C1"},
    {"geopende_map", "1F4C2"},
    {"omkrullende_pagina", "1F4C3"},
    {"pagina_met_bovenzijde_omhoog", "1F4C4"},
    {"kalender", "1F4C5"},
    {"scheurkalender", "1F4C6"},
    {"kaartindex", "1F4C7"},
    {"stijgende_trend", "1F4C8"},
    {"dalende_trend", "1F4C9"},
    {"staafdiagram", "1F4CA"},
    {"klembord", "1F4CB"},
    {"punaise", "1F4CC"},
    {"ronde_speld", "1F4CD"},
    {"paperclip", "1F4CE"},
    {"liniaal", "1F4CF"},
    {"geodriehoek", "1F4D0"},
    {"bladwijzertabs", "1F4D1"},
    {"grootboek", "1F4D2"},
    {"schrift", "1F4D3"},
    {"versierd_schrift", "1F4D4"},
    {"gesloten_boek", "1F4D5"},
    {"geopend_boek", "1F4D6"},
    {"groen_boek", "1F4D7"},
    {"blauw_boek", "1F4D8"},
    {"oranje_boek", "1F4D9"},
    {"boeken", "1F4DA"},
    {"naamkaartje", "1F4DB"},
    {"rol", "1F4DC"},
    {"memo", "1F4DD"},
    {"telefoonhoorn", "1F4DE"},
    {"pager", "1F4DF"},
    {"fax", "1F4E0"},
    {"satellietschotel", "1F4E1"},
    {"luidspreker", "1F4E2"},
    {"megafoon", "1F4E3"},
    {"postvak_uit", "1F4E4"},
    {"postvak_in", "1F4E5"},
    {"pakket", "1F4E6"},
    {"email", "1F4E7"},
    {"inkomende_post", "1F4E8"},
    {"envelop_met_pijl", "1F4E9"},
    {"gesloten_brievenbus_met_vlaggetje_omlaag", "1F4EA"},
    {"gesloten_brievenbus_met_vlaggetje_omhoog", "1F4EB"},
    {"open_brievenbus_met_vlaggetje_omhoog", "1F4EC"},
    {"open_brievenbus_met_vlaggetje_omlaag", "1F4ED"},
    {"brievenbus", "1F4EE"},
    {"posthoorn", "1F4EF"},
    {"krant", "1F4F0"},
    {"mobiele_telefoon", "1F4F1"},
    {"mobiel_met_pijl", "1F4F2"},
    {"trilstand", "1F4F3"},
    {"telefoon_uit", "1F4F4"},
    {"geen_mobiele_telefoon", "1F4F5"},
    {"mobiel_signaal", "1F4F6"},
    {"camera", "1F4F7"},
    {"camera_met_flitser", "1F4F8"},
    {"videocamera", "1F4F9"},
    {"tv", "1F4FA"},
    {"radio", "1F4FB"},
    {"videoband", "1F4FC"},
    {"filmprojector", "1F4FD"},
    {"gebedssnoer", "1F4FF"},
    {"knop_voor_shuffle", "1F500"},
    {"knop_voor_herhalen", "1F501"},
    {"knop_voor_een_herhalen", "1F502"},
    {"verticale_pijlen_met_de_klok_mee", "1F503"},
    {"pijlen_tegen_de_klok_in", "1F504"},
    {"knop_voor_lagere_helderheid", "1F505"},
    {"knop_voor_hogere_helderheid", "1F506"},
    {"speaker_uit", "1F507"},
    {"speaker", "1F508"},
    {"speaker_aan", "1F509"},
    {"speaker_luid", "1F50A"},
    {"batterij", "1F50B"},
    {"stekker", "1F50C"},
    {"vergrootglas_naar_links_gericht", "1F50D"},
    {"vergrootglas_naar_rechts_gericht", "1F50E"},
    {"slot_met_pen", "1F50F"},
    {"gesloten_slot_met_sleutel", "1F510"},
    {"sleutel", "1F511"},
    {"slot", "1F512"},
    {"geopend_hangslot", "1F513"},
    {"bel", "1F514"},
    {"doorgestreepte_bel", "1F515"},
    {"bladwijzer", "1F516"},
    {"schakels", "1F517"},
    {"keuzerondje", "1F518"},
    {"back_pijl", "1F519"},
    {"end_pijl", "1F51A"},
    {"on_pijl", "1F51B"},
    {"soon_pijl", "1F51C"},
    {"top_pijl", "1F51D"},
    {"niemand_onder_de_18", "1F51E"},
    {"toets_10", "1F51F"},
    {"hoofdletters", "1F520"},
    {"kleine_letters", "1F521"},
    {"cijfers", "1F522"},
    {"symbolen", "1F523"},
    {"latijns_alfabet", "1F524"},
    {"vuur", "1F525"},
    {"zaklamp", "1F526"},
    {"moersleutel", "1F527"},
    {"hamer", "1F528"},
    {"bout_en_moer", "1F529"},
    {"keukenmes", "1F52A"},
    {"waterpistool", "1F52B"},
    {"microscoop", "1F52C"},
    {"telescoop", "1F52D"},
    {"kristallen_bol", "1F52E"},
    {"zespuntige_ster", "1F52F"},
    {"japans_symbool_voor_beginner", "1F530"},
    {"drietand_symbool", "1F531"},
    {"zwarte_vierkante_knop", "1F532"},
    {"witte_vierkante_knop", "1F533"},
    {"rode_cirkel", "1F534"},
    {"blauwe_cirkel", "1F535"},
    {"grote_oranje_ruit", "1F536"},
    {"grote_blauwe_ruit", "1F537"},
    {"kleine_oranje_ruit", "1F538"},
    {"kleine_blauwe_ruit", "1F539"},
    {"rode_omhoogwijzende_driehoek", "1F53A"},
    {"rode_omlaagwijzende_driehoek", "1F53B"},
    {"knop_voor_omhoog", "1F53C"},
    {"knop_voor_omlaag", "1F53D"},
    {"om_symbool", "1F549"},
    {"duif", "1F54A"},
    {"kaaba", "1F54B"},
    {"moskee", "1F54C"},
    {"synagoge", "1F54D"},
    {"menora", "1F54E"},
    {"een_uur", "1F550"},
    {"twee_uur", "1F551"},
    {"drie_uur", "1F552"},
    {"vier_uur", "1F553"},
    {"vijf_uur", "1F554"},
    {"zes_uur", "1F555"},
    {"zeven_uur", "1F556"},
    {"acht_uur", "1F557"},
    {"negen_uur", "1F558"},
    {"tien_uur", "1F559"},
    {"elf_uur", "1F55A"},
    {"twaalf_uur", "1F55B"},
    {"half_twee", "1F55C"},
    {"half_drie", "1F55D"},
    {"half_vier", "1F55E"},
    {"half_vijf", "1F55F"},
    {"half_zes", "1F560"},
    {"half_zeven", "1F561"},
    {"half_acht", "1F562"},
    {"half_negen", "1F563"},
    {"half_tien", "1F564"},
    {"half_elf", "1F565"},
    {"half_twaalf", "1F566"},
    {"half_een", "1F567"},
    {"kaars", "1F56F"},
    {"klok", "1F570"},
    {"gat", "1F573"},
    {"zwevende_man_in_pak", "1F574"},
    {"zwevende_man_in_pak_kleur1", "1F574-1F3FB"},
    {"zwevende_man_in_pak_kleur2", "1F574-1F3FC"},
    {"zwevende_man_in_pak_kleur3", "1F574-1F3FD"},
    {"zwevende_man_in_pak_kleur4", "1F574-1F3FE"},
    {"zwevende_man_in_pak_kleur5", "1F574-1F3FF"},
    {"detective", "1F575"},
    {"detective_kleur1", "1F575-1F3FB"},
    {"detective_kleur2", "1F575-1F3FC"},
    {"detective_kleur3", "1F575-1F3FD"},
    {"detective_kleur4", "1F575-1F3FE"},
    {"detective_kleur5", "1F575-1F3FF"},
    {"zonnebril", "1F576"},
    {"spin", "1F577"},
    {"spinnenweb", "1F578"},
    {"joystick", "1F579"},
    {"dansende_man", "1F57A"},
    {"dansende_man_kleur1", "1F57A-1F3FB"},
    {"dansende_man_kleur2", "1F57A-1F3FC"},
    {"dansende_man_kleur3", "1F57A-1F3FD"},
    {"dansende_man_kleur4", "1F57A-1F3FE"},
    {"dansende_man_kleur5", "1F57A-1F3FF"},
    {"twee_paperclips_aan_elkaar", "1F587"},
    {"pen", "1F58A"},
    {"vulpen", "1F58B"},
    {"kwast", "1F58C"},
    {"krijtje", "1F58D"},
    {"opgeheven_hand_met_uitgestoken_vingers", "1F590"},
    {"opgeheven_hand_met_uitgestoken_vingers_kleur1", "1F590-1F3FB"},
    {"opgeheven_hand_met_uitgestoken_vingers_kleur2", "1F590-1F3FC"},
    {"opgeheven_hand_met_uitgestoken_vingers_kleur3", "1F590-1F3FD"},
    {"opgeheven_hand_met_uitgestoken_vingers_kleur4", "1F590-1F3FE"},
    {"opgeheven_hand_met_uitgestoken_vingers_kleur5", "1F590-1F3FF"},
    {"middelvinger", "1F595"},
    {"middelvinger_kleur1", "1F595-1F3FB"},
    {"middelvinger_kleur2", "1F595-1F3FC"},
    {"middelvinger_kleur3", "1F595-1F3FD"},
    {"middelvinger_kleur4", "1F595-1F3FE"},
    {"middelvinger_kleur5", "1F595-1F3FF"},
    {"vulcaanse_groet", "1F596"},
    {"vulcaanse_groet_kleur1", "1F596-1F3FB"},
    {"vulcaanse_groet_kleur2", "1F596-1F3FC"},
    {"vulcaanse_groet_kleur3", "1F596-1F3FD"},
    {"vulcaanse_groet_kleur4", "1F596-1F3FE"},
    {"vulcaanse_groet_kleur5", "1F596-1F3FF"},
    {"zwart_hart", "1F5A4"},
    {"desktop", "1F5A5"},
    {"printer", "1F5A8"},
    {"computermuis", "1F5B1"},
    {"trackball", "1F5B2"},
    {"lijst_met_afbeelding", "1F5BC"},
    {"tabbladen", "1F5C2"},
    {"kaartenbak", "1F5C3"},
    {"archiefkast", "1F5C4"},
    {"prullenmand", "1F5D1"},
    {"schrijfblok_met_spiraal", "1F5D2"},
    {"kalender_met_spiraal", "1F5D3"},
    {"lijmklem", "1F5DC"},
    {"oude_sleutel", "1F5DD"},
    {"opgerolde_krant", "1F5DE"},
    {"dolk", "1F5E1"},
    {"silhouet_van_sprekend_hoofd", "1F5E3"},
    {"spraakballon_naar_links", "1F5E8"},
    {"boze_spraakballon_naar_rechts", "1F5EF"},
    {"stembus_met_stembiljet", "1F5F3"},
    {"wereldkaart", "1F5FA"},
    {"berg_fuji", "1F5FB"},
    {"tokio_toren", "1F5FC"},
    {"vrijheidsbeeld", "1F5FD"},
    {"kaart_van_japan", "1F5FE"},
    {"moai_beeld", "1F5FF"},
    {"grijnzend_gezicht", "1F600"},
    {"stralend_gezicht_met_lachende_ogen", "1F601"},
    {"gezicht_met_tranen_van_vreugde", "1F602"},
    {"grijnzend_gezicht_met_grote_ogen", "1F603"},
    {"grijnzend_gezicht_met_lachende_ogen", "1F604"},
    {"grijnzend_gezicht_met_zweetdruppels", "1F605"},
    {"grijnzend_gezicht_met_stijf_dichtgeknepen_ogen", "1F606"},
    {"lachend_gezicht_met_halo", "1F607"},
    {"lachend_gezicht_met_hoorns", "1F608"},
    {"knipogend_gezicht", "1F609"},
    {"lachend_gezicht_met_lachende_ogen", "1F60A"},
    {"lachend_gezicht_dat_lippen_likt", "1F60B"},
    {"opgelucht_gezicht", "1F60C"},
    {"lachend_gezicht_met_hartvormige_ogen", "1F60D"},
    {"lachend_gezicht_met_zonnebril", "1F60E"},
    {"gniffelend_gezicht", "1F60F"},
    {"neutraal_gezicht", "1F610"},
    {"uitdrukkingsloos_gezicht", "1F611"},
    {"chagrijnig_gezicht", "1F612"},
    {"neerslachtig_gezicht_met_zweetdruppel", "1F613"},
    {"peinzend_gezicht", "1F614"},
    {"verward_gezicht", "1F615"},
    {"onthutst_gezicht", "1F616"},
    {"kussend_gezicht", "1F617"},
    {"gezicht_dat_kus_toeblaast", "1F618"},
    {"kussend_gezicht_met_lachende_ogen", "1F619"},
    {"kussend_gezicht_met_gesloten_ogen", "1F61A"},
    {"gezicht_met_uitgestoken_tong", "1F61B"},
    {"knipoog_met_uitgestoken_tong", "1F61C"},
    {"gezicht_met_uitgestoken_tong_en_dichtgeknepen_ogen", "1F61D"},
    {"teleurgesteld_gezicht", "1F61E"},
    {"bezorgd_gezicht", "1F61F"},
    {"boos_gezicht", "1F620"},
    {"pruilend_gezicht", "1F621"},
    {"huilend_gezicht", "1F622"},
    {"volhardend_gezicht", "1F623"},
    {"gezicht_blazend_van_woede", "1F624"},
    {"teleurgesteld_maar_opgelucht_gezicht", "1F625"},
    {"fronsend_gezicht_met_open_mond", "1F626"},
    {"gekweld_gezicht", "1F627"},
    {"bang_gezicht", "1F628"},
    {"moe_gezicht", "1F629"},
    {"slaperig_gezicht", "1F62A"},
    {"vermoeid_gezicht", "1F62B"},
    {"gezicht_met_grimas", "1F62C"},
    {"hard_huilend_gezicht", "1F62D"},
    {"gezicht_met_open_mond", "1F62E"},
    {"verstomd_gezicht", "1F62F"},
    {"angstig_gezicht_met_zweetdruppel", "1F630"},
    {"angstig_schreeuwend_gezicht", "1F631"},
    {"verbaasd_gezicht", "1F632"},
    {"blozend_gezicht", "1F633"},
    {"slapend_gezicht", "1F634"},
    {"duizelig_gezicht", "1F635"},
    {"gezicht_zonder_mond", "1F636"},
    {"gezicht_met_doktersmasker", "1F637"},
    {"grijnzende_kat_met_lachende_ogen", "1F638"},
    {"kat_met_tranen_van_vreugde", "1F639"},
    {"grijnzende_kat", "1F63A"},
    {"lachende_kat_met_hartvormige_ogen", "1F63B"},
    {"gniffelende_kat", "1F63C"},
    {"kussende_kat", "1F63D"},
    {"pruilende_kat", "1F63E"},
    {"huilende_kat", "1F63F"},
    {"vermoeide_kat", "1F640"},
    {"licht_fronsend_gezicht", "1F641"},
    {"enigszins_lachend_gezicht", "1F642"},
    {"omgekeerd_gezicht", "1F643"},
    {"gezicht_met_rollende_ogen", "1F644"},
    {"persoon_die_nee_gebaar_maakt", "1F645"},
    {"persoon_die_nee_gebaar_maakt_kleur1", "1F645-1F3FB"},
    {"persoon_die_nee_gebaar_maakt_kleur2", "1F645-1F3FC"},
    {"persoon_die_nee_gebaar_maakt_kleur3", "1F645-1F3FD"},
    {"persoon_die_nee_gebaar_maakt_kleur4", "1F645-1F3FE"},
    {"persoon_die_nee_gebaar_maakt_kleur5", "1F645-1F3FF"},
    {"persoon_die_ok_gebaar_maakt", "1F646"},
    {"persoon_die_ok_gebaar_maakt_kleur1", "1F646-1F3FB"},
    {"persoon_die_ok_gebaar_maakt_kleur2", "1F646-1F3FC"},
    {"persoon_die_ok_gebaar_maakt_kleur3", "1F646-1F3FD"},
    {"persoon_die_ok_gebaar_maakt_kleur4", "1F646-1F3FE"},
    {"persoon_die_ok_gebaar_maakt_kleur5", "1F646-1F3FF"},
    {"buigende_persoon", "1F647"},
    {"buigende_persoon_kleur1", "1F647-1F3FB"},
    {"buigende_persoon_kleur2", "1F647-1F3FC"},
    {"buigende_persoon_kleur3", "1F647-1F3FD"},
    {"buigende_persoon_kleur4", "1F647-1F3FE"},
    {"buigende_persoon_kleur5", "1F647-1F3FF"},
    {"geen_kwaad_zien", "1F648"},
    {"geen_kwaad_horen", "1F649"},
    {"geen_kwaad_spreken", "1F64A"},
    {"persoon_met_opgeheven_hand", "1F64B"},
    {"persoon_met_opgeheven_hand_kleur1", "1F64B-1F3FB"},
    {"persoon_met_opgeheven_hand_kleur2", "1F64B-1F3FC"},
    {"persoon_met_opgeheven_hand_kleur3", "1F64B-1F3FD"},
    {"persoon_met_opgeheven_hand_kleur4", "1F64B-1F3FE"},
    {"persoon_met_opgeheven_hand_kleur5", "1F64B-1F3FF"},
    {"juichend_omhooggestoken_handen", "1F64C"},
    {"juichend_omhooggestoken_handen_kleur1", "1F64C-1F3FB"},
    {"juichend_omhooggestoken_handen_kleur2", "1F64C-1F3FC"},
    {"juichend_omhooggestoken_handen_kleur3", "1F64C-1F3FD"},
    {"juichend_omhooggestoken_handen_kleur4", "1F64C-1F3FE"},
    {"juichend_omhooggestoken_handen_kleur5", "1F64C-1F3FF"},
    {"fronsend_persoon", "1F64D"},
    {"fronsend_persoon_kleur1", "1F64D-1F3FB"},
    {"fronsend_persoon_kleur2", "1F64D-1F3FC"},
    {"fronsend_persoon_kleur3", "1F64D-1F3FD"},
    {"fronsend_persoon_kleur4", "1F64D-1F3FE"},
    {"fronsend_persoon_kleur5", "1F64D-1F3FF"},
    {"pruilend_persoon", "1F64E"},
    {"pruilend_persoon_kleur1", "1F64E-1F3FB"},
    {"pruilend_persoon_kleur2", "1F64E-1F3FC"},
    {"pruilend_persoon_kleur3", "1F64E-1F3FD"},
    {"pruilend_persoon_kleur4", "1F64E-1F3FE"},
    {"pruilend_persoon_kleur5", "1F64E-1F3FF"},
    {"gevouwen_handen", "1F64F"},
    {"gevouwen_handen_kleur1", "1F64F-1F3FB"},
    {"gevouwen_handen_kleur2", "1F64F-1F3FC"},
    {"gevouwen_handen_kleur3", "1F64F-1F3FD"},
    {"gevouwen_handen_kleur4", "1F64F-1F3FE"},
    {"gevouwen_handen_kleur5", "1F64F-1F3FF"},
    {"raket", "1F680"},
    {"helikopter", "1F681"},
    {"locomotief", "1F682"},
    {"treinwagon", "1F683"},
    {"hogesnelheidstrein", "1F684"},
    {"hogesnelheidstrein_met_kogelneus", "1F685"},
    {"trein", "1F686"},
    {"metro", "1F687"},
    {"lightrail", "1F688"},
    {"treinstation", "1F689"},
    {"tram", "1F68A"},
    {"tramwagon", "1F68B"},
    {"bus", "1F68C"},
    {"naderende_bus", "1F68D"},
    {"trolleybus", "1F68E"},
    {"bushalte", "1F68F"},
    {"minibus", "1F690"},
    {"ambulance", "1F691"},
    {"brandweerauto", "1F692"},
    {"politieauto", "1F693"},
    {"naderende_politieauto", "1F694"},
    {"taxi", "1F695"},
    {"naderende_taxi", "1F696"},
    {"auto", "1F697"},
    {"naderende_auto", "1F698"},
    {"suv", "1F699"},
    {"bestelbus", "1F69A"},
    {"vrachtwagen", "1F69B"},
    {"tractor", "1F69C"},
    {"monorail", "1F69D"},
    {"bergspoor", "1F69E"},
    {"zweeftrein", "1F69F"},
    {"bergkabelbaan", "1F6A0"},
    {"kabelbaan", "1F6A1"},
    {"schip", "1F6A2"},
    {"persoon_in_roeiboot", "1F6A3"},
    {"persoon_in_roeiboot_kleur1", "1F6A3-1F3FB"},
    {"persoon_in_roeiboot_kleur2", "1F6A3-1F3FC"},
    {"persoon_in_roeiboot_kleur3", "1F6A3-1F3FD"},
    {"persoon_in_roeiboot_kleur4", "1F6A3-1F3FE"},
    {"persoon_in_roeiboot_kleur5", "1F6A3-1F3FF"},
    {"speedboot", "1F6A4"},
    {"horizontaal_verkeerslicht", "1F6A5"},
    {"verticaal_verkeerslicht", "1F6A6"},
    {"bouwwerkzaamheden", "1F6A7"},
    {"zwaailicht_van_politieauto", "1F6A8"},
    {"driehoekige_vlag", "1F6A9"},
    {"deur", "1F6AA"},
    {"verboden", "1F6AB"},
    {"sigaret", "1F6AC"},
    {"niet_roken", "1F6AD"},
    {"prullenbaksymbool", "1F6AE"},
    {"geen_afval_weggooien", "1F6AF"},
    {"drinkwater", "1F6B0"},
    {"geen_drinkwater", "1F6B1"},
    {"fiets", "1F6B2"},
    {"geen_fietsen", "1F6B3"},
    {"fietsende_persoon", "1F6B4"},
    {"fietsende_persoon_kleur1", "1F6B4-1F3FB"},
    {"fietsende_persoon_kleur2", "1F6B4-1F3FC"},
    {"fietsende_persoon_kleur3", "1F6B4-1F3FD"},
    {"fietsende_persoon_kleur4", "1F6B4-1F3FE"},
    {"fietsende_persoon_kleur5", "1F6B4-1F3FF"},
    {"persoon_op_mountainbike", "1F6B5"},
    {"persoon_op_mountainbike_kleur1", "1F6B5-1F3FB"},
    {"persoon_op_mountainbike_kleur2", "1F6B5-1F3FC"},
    {"persoon_op_mountainbike_kleur3", "1F6B5-1F3FD"},
    {"persoon_op_mountainbike_kleur4", "1F6B5-1F3FE"},
    {"persoon_op_mountainbike_kleur5", "1F6B5-1F3FF"},
    {"voetganger", "1F6B6"},
    {"voetganger_kleur1", "1F6B6-1F3FB"},
    {"voetganger_kleur2", "1F6B6-1F3FC"},
    {"voetganger_kleur3", "1F6B6-1F3FD"},
    {"voetganger_kleur4", "1F6B6-1F3FE"},
    {"voetganger_kleur5", "1F6B6-1F3FF"},
    {"geen_voetgangers", "1F6B7"},
    {"overstekende_kinderen", "1F6B8"},
    {"herentoilet", "1F6B9"},
    {"damestoilet", "1F6BA"},
    {"toiletten", "1F6BB"},
    {"babysymbool", "1F6BC"},
    {"toilet", "1F6BD"},
    {"wc", "1F6BE"},
    {"douche", "1F6BF"},
    {"persoon_in_badkuip", "1F6C0"},
    {"persoon_in_badkuip_kleur1", "1F6C0-1F3FB"},
    {"persoon_in_badkuip_kleur2", "1F6C0-1F3FC"},
    {"persoon_in_badkuip_kleur3", "1F6C0-1F3FD"},
    {"persoon_in_badkuip_kleur4", "1F6C0-1F3FE"},
    {"persoon_in_badkuip_kleur5", "1F6C0-1F3FF"},
    {"badkuip", "1F6C1"},
    {"paspoortcontrole", "1F6C2"},
    {"douane", "1F6C3"},
    {"bagage_afhalen", "1F6C4"},
    {"bagagekluis", "1F6C5"},
    {"bank_en_lamp", "1F6CB"},
    {"persoon_in_bed", "1F6CC"},
    {"persoon_in_bed_kleur1", "1F6CC-1F3FB"},
    {"persoon_in_bed_kleur2", "1F6CC-1F3FC"},
    {"persoon_in_bed_kleur3", "1F6CC-1F3FD"},
    {"persoon_in_bed_kleur4", "1F6CC-1F3FE"},
    {"persoon_in_bed_kleur5", "1F6CC-1F3FF"},
    {"boodschappentassen", "1F6CD"},
    {"piccolobel", "1F6CE"},
    {"bed", "1F6CF"},
    {"gebedsruimte", "1F6D0"},
    {"stopteken", "1F6D1"},
    {"winkelwagen", "1F6D2"},
    {"hindoetempel", "1F6D5"},
    {"hut", "1F6D6"},
    {"lift", "1F6D7"},
    {"aardverschuiving", "1F6D8"},
    {"draadloos", "1F6DC"},
    {"glijbaan", "1F6DD"},
    {"wiel", "1F6DE"},
    {"reddingsboei", "1F6DF"},
    {"hamer_en_moersleutel", "1F6E0"},
    {"schild", "1F6E1"},
    {"olievat", "1F6E2"},
    {"snelweg", "1F6E3"},
    {"treinspoor", "1F6E4"},
    {"motorboot", "1F6E5"},
    {"vliegtuigje", "1F6E9"},
    {"vertrek_van_vliegtuig", "1F6EB"},
    {"aankomst_van_vliegtuig", "1F6EC"},
    {"satelliet", "1F6F0"},
    {"passagiersschip", "1F6F3"},
    {"step", "1F6F4"},
    {"scooter", "1F6F5"},
    {"kano", "1F6F6"},
    {"slee", "1F6F7"},
    {"vliegende_schotel", "1F6F8"},
    {"skateboard", "1F6F9"},
    {"riksja", "1F6FA"},
    {"pick_up", "1F6FB"},
    {"rolschaats", "1F6FC"},
    {"oranje_cirkel", "1F7E0"},
    {"gele_cirkel", "1F7E1"},
    {"groene_cirkel", "1F7E2"},
    {"paarse_cirkel", "1F7E3"},
    {"bruine_cirkel", "1F7E4"},
    {"rood_vierkant", "1F7E5"},
    {"blauw_vierkant", "1F7E6"},
    {"oranje_vierkant", "1F7E7"},
    {"geel_vierkant", "1F7E8"},
    {"groen_vierkant", "1F7E9"},
    {"paars_vierkant", "1F7EA"},
    {"bruin_vierkant", "1F7EB"},
    {"vetgedrukt_gelijkteken", "1F7F0"},
    {"samengeknepen_vingers", "1F90C"},
    {"samengeknepen_vingers_kleur1", "1F90C-1F3FB"},
    {"samengeknepen_vingers_kleur2", "1F90C-1F3FC"},
    {"samengeknepen_vingers_kleur3", "1F90C-1F3FD"},
    {"samengeknepen_vingers_kleur4", "1F90C-1F3FE"},
    {"samengeknepen_vingers_kleur5", "1F90C-1F3FF"},
    {"wit_hart", "1F90D"},
    {"bruin_hart", "1F90E"},
    {"hand_met_samengeknepen_vingers", "1F90F"},
    {"hand_met_samengeknepen_vingers_kleur1", "1F90F-1F3FB"},
    {"hand_met_samengeknepen_vingers_kleur2", "1F90F-1F3FC"},
    {"hand_met_samengeknepen_vingers_kleur3", "1F90F-1F3FD"},
    {"hand_met_samengeknepen_vingers_kleur4", "1F90F-1F3FE"},
    {"hand_met_samengeknepen_vingers_kleur5", "1F90F-1F3FF"},
    {"gezicht_met_een_rits_als_mond", "1F910"},
    {"gezicht_met_geld_in_de_mond", "1F911"},
    {"gezicht_met_thermometer", "1F912"},
    {"nerdgezicht", "1F913"},
    {"nadenkend_gezicht", "1F914"},
    {"gezicht_met_een_verband_om_het_hoofd", "1F915"},
    {"robot", "1F916"},
    {"blij_gezicht_met_uitgestoken_handen", "1F917"},
    {"duivelshorens", "1F918"},
    {"duivelshorens_kleur1", "1F918-1F3FB"},
    {"duivelshorens_kleur2", "1F918-1F3FC"},
    {"duivelshorens_kleur3", "1F918-1F3FD"},
    {"duivelshorens_kleur4", "1F918-1F3FE"},
    {"duivelshorens_kleur5", "1F918-1F3FF"},
    {"bel_me_gebaar", "1F919"},
    {"bel_me_gebaar_kleur1", "1F919-1F3FB"},
    {"bel_me_gebaar_kleur2", "1F919-1F3FC"},
    {"bel_me_gebaar_kleur3", "1F919-1F3FD"},
    {"bel_me_gebaar_kleur4", "1F919-1F3FE"},
    {"bel_me_gebaar_kleur5", "1F919-1F3FF"},
    {"achterkant_van_opgeheven_hand", "1F91A"},
    {"achterkant_van_opgeheven_hand_kleur1", "1F91A-1F3FB"},
    {"achterkant_van_opgeheven_hand_kleur2", "1F91A-1F3FC"},
    {"achterkant_van_opgeheven_hand_kleur3", "1F91A-1F3FD"},
    {"achterkant_van_opgeheven_hand_kleur4", "1F91A-1F3FE"},
    {"achterkant_van_opgeheven_hand_kleur5", "1F91A-1F3FF"},
    {"naar_links_gerichte_vuist", "1F91B"},
    {"naar_links_gerichte_vuist_kleur1", "1F91B-1F3FB"},
    {"naar_links_gerichte_vuist_kleur2", "1F91B-1F3FC"},
    {"naar_links_gerichte_vuist_kleur3", "1F91B-1F3FD"},
    {"naar_links_gerichte_vuist_kleur4", "1F91B-1F3FE"},
    {"naar_links_gerichte_vuist_kleur5", "1F91B-1F3FF"},
    {"naar_rechts_gerichte_vuist", "1F91C"},
    {"naar_rechts_gerichte_vuist_kleur1", "1F91C-1F3FB"},
    {"naar_rechts_gerichte_vuist_kleur2", "1F91C-1F3FC"},
    {"naar_rechts_gerichte_vuist_kleur3", "1F91C-1F3FD"},
    {"naar_rechts_gerichte_vuist_kleur4", "1F91C-1F3FE"},
    {"naar_rechts_gerichte_vuist_kleur5", "1F91C-1F3FF"},
    {"handdruk", "1F91D"},
    {"handdruk_kleur1", "1F91D-1F3FB"},
    {"handdruk_kleur2", "1F91D-1F3FC"},
    {"handdruk_kleur3", "1F91D-1F3FD"},
    {"handdruk_kleur4", "1F91D-1F3FE"},
    {"handdruk_kleur5", "1F91D-1F3FF"},
    {"handdruk_kleur1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"handdruk_kleur1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"handdruk_kleur1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"handdruk_kleur1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"handdruk_kleur2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"handdruk_kleur2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"handdruk_kleur2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"handdruk_kleur2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"handdruk_kleur3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"handdruk_kleur3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"handdruk_kleur3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"handdruk_kleur3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"handdruk_kleur4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"handdruk_kleur4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"handdruk_kleur4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"handdruk_kleur4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"handdruk_kleur5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"handdruk_kleur5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"handdruk_kleur5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"handdruk_kleur5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"gekruiste_vingers", "1F91E"},
    {"gekruiste_vingers_kleur1", "1F91E-1F3FB"},
    {"gekruiste_vingers_kleur2", "1F91E-1F3FC"},
    {"gekruiste_vingers_kleur3", "1F91E-1F3FD"},
    {"gekruiste_vingers_kleur4", "1F91E-1F3FE"},
    {"gekruiste_vingers_kleur5", "1F91E-1F3FF"},
    {"ik_hou_van_je_gebaar", "1F91F"},
    {"ik_hou_van_je_gebaar_kleur1", "1F91F-1F3FB"},
    {"ik_hou_van_je_gebaar_kleur2", "1F91F-1F3FC"},
    {"ik_hou_van_je_gebaar_kleur3", "1F91F-1F3FD"},
    {"ik_hou_van_je_gebaar_kleur4", "1F91F-1F3FE"},
    {"ik_hou_van_je_gebaar_kleur5", "1F91F-1F3FF"},
    {"gezicht_met_cowboyhoed", "1F920"},
    {"clownsgezicht", "1F921"},
    {"misselijk_gezicht", "1F922"},
    {"gezicht_dat_op_de_grond_rolt_van_het_lachen", "1F923"},
    {"kwijlend_gezicht", "1F924"},
    {"liegend_gezicht", "1F925"},
    {"persoon_die_het_gezicht_met_de_hand_bedekt", "1F926"},
    {"persoon_die_het_gezicht_met_de_hand_bedekt_kleur1", "1F926-1F3FB"},
    {"persoon_die_het_gezicht_met_de_hand_bedekt_kleur2", "1F926-1F3FC"},
    {"persoon_die_het_gezicht_met_de_hand_bedekt_kleur3", "1F926-1F3FD"},
    {"persoon_die_het_gezicht_met_de_hand_bedekt_kleur4", "1F926-1F3FE"},
    {"persoon_die_het_gezicht_met_de_hand_bedekt_kleur5", "1F926-1F3FF"},
    {"niezend_gezicht", "1F927"},
    {"gezicht_met_opgeheven_wenkbrauw", "1F928"},
    {"sterren_in_de_ogen", "1F929"},
    {"gek_gezicht", "1F92A"},
    {"gezicht_dat_tot_stilte_maant", "1F92B"},
    {"gezicht_met_symbolen_op_mond", "1F92C"},
    {"gezicht_met_hand_over_de_mond", "1F92D"},
    {"brakend_gezicht", "1F92E"},
    {"ontploffend_hoofd", "1F92F"},
    {"zwangere_vrouw", "1F930"},
    {"zwangere_vrouw_kleur1", "1F930-1F3FB"},
    {"zwangere_vrouw_kleur2", "1F930-1F3FC"},
    {"zwangere_vrouw_kleur3", "1F930-1F3FD"},
    {"zwangere_vrouw_kleur4", "1F930-1F3FE"},
    {"zwangere_vrouw_kleur5", "1F930-1F3FF"},
    {"borstvoeding", "1F931"},
    {"borstvoeding_kleur1", "1F931-1F3FB"},
    {"borstvoeding_kleur2", "1F931-1F3FC"},
    {"borstvoeding_kleur3", "1F931-1F3FD"},
    {"borstvoeding_kleur4", "1F931-1F3FE"},
    {"borstvoeding_kleur5", "1F931-1F3FF"},
    {"beide_handpalmen_omhoog", "1F932"},
    {"beide_handpalmen_omhoog_kleur1", "1F932-1F3FB"},
    {"beide_handpalmen_omhoog_kleur2", "1F932-1F3FC"},
    {"beide_handpalmen_omhoog_kleur3", "1F932-1F3FD"},
    {"beide_handpalmen_omhoog_kleur4", "1F932-1F3FE"},
    {"beide_handpalmen_omhoog_kleur5", "1F932-1F3FF"},
    {"selfie", "1F933"},
    {"selfie_kleur1", "1F933-1F3FB"},
    {"selfie_kleur2", "1F933-1F3FC"},
    {"selfie_kleur3", "1F933-1F3FD"},
    {"selfie_kleur4", "1F933-1F3FE"},
    {"selfie_kleur5", "1F933-1F3FF"},
    {"prins", "1F934"},
    {"prins_kleur1", "1F934-1F3FB"},
    {"prins_kleur2", "1F934-1F3FC"},
    {"prins_kleur3", "1F934-1F3FD"},
    {"prins_kleur4", "1F934-1F3FE"},
    {"prins_kleur5", "1F934-1F3FF"},
    {"persoon_in_smoking", "1F935"},
    {"persoon_in_smoking_kleur1", "1F935-1F3FB"},
    {"persoon_in_smoking_kleur2", "1F935-1F3FC"},
    {"persoon_in_smoking_kleur3", "1F935-1F3FD"},
    {"persoon_in_smoking_kleur4", "1F935-1F3FE"},
    {"persoon_in_smoking_kleur5", "1F935-1F3FF"},
    {"kerstvrouw", "1F936"},
    {"kerstvrouw_kleur1", "1F936-1F3FB"},
    {"kerstvrouw_kleur2", "1F936-1F3FC"},
    {"kerstvrouw_kleur3", "1F936-1F3FD"},
    {"kerstvrouw_kleur4", "1F936-1F3FE"},
    {"kerstvrouw_kleur5", "1F936-1F3FF"},
    {"persoon_die_schouders_ophaalt", "1F937"},
    {"persoon_die_schouders_ophaalt_kleur1", "1F937-1F3FB"},
    {"persoon_die_schouders_ophaalt_kleur2", "1F937-1F3FC"},
    {"persoon_die_schouders_ophaalt_kleur3", "1F937-1F3FD"},
    {"persoon_die_schouders_ophaalt_kleur4", "1F937-1F3FE"},
    {"persoon_die_schouders_ophaalt_kleur5", "1F937-1F3FF"},
    {"persoon_die_een_radslag_doet", "1F938"},
    {"persoon_die_een_radslag_doet_kleur1", "1F938-1F3FB"},
    {"persoon_die_een_radslag_doet_kleur2", "1F938-1F3FC"},
    {"persoon_die_een_radslag_doet_kleur3", "1F938-1F3FD"},
    {"persoon_die_een_radslag_doet_kleur4", "1F938-1F3FE"},
    {"persoon_die_een_radslag_doet_kleur5", "1F938-1F3FF"},
    {"persoon_die_jongleert", "1F939"},
    {"persoon_die_jongleert_kleur1", "1F939-1F3FB"},
    {"persoon_die_jongleert_kleur2", "1F939-1F3FC"},
    {"persoon_die_jongleert_kleur3", "1F939-1F3FD"},
    {"persoon_die_jongleert_kleur4", "1F939-1F3FE"},
    {"persoon_die_jongleert_kleur5", "1F939-1F3FF"},
    {"schermer", "1F93A"},
    {"personen_die_worstelen", "1F93C"},
    {"personen_die_worstelen_kleur1", "1F93C-1F3FB"},
    {"personen_die_worstelen_kleur2", "1F93C-1F3FC"},
    {"personen_die_worstelen_kleur3", "1F93C-1F3FD"},
    {"personen_die_worstelen_kleur4", "1F93C-1F3FE"},
    {"personen_die_worstelen_kleur5", "1F93C-1F3FF"},
    {"personen_die_worstelen_kleur1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"personen_die_worstelen_kleur1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"personen_die_worstelen_kleur1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"personen_die_worstelen_kleur1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"personen_die_worstelen_kleur2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"personen_die_worstelen_kleur2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"personen_die_worstelen_kleur2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"personen_die_worstelen_kleur2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"personen_die_worstelen_kleur3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"personen_die_worstelen_kleur3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"personen_die_worstelen_kleur3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"personen_die_worstelen_kleur3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"personen_die_worstelen_kleur4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"personen_die_worstelen_kleur4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"personen_die_worstelen_kleur4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"personen_die_worstelen_kleur4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"personen_die_worstelen_kleur5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"personen_die_worstelen_kleur5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"personen_die_worstelen_kleur5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"personen_die_worstelen_kleur5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"persoon_die_waterpolo_speelt", "1F93D"},
    {"persoon_die_waterpolo_speelt_kleur1", "1F93D-1F3FB"},
    {"persoon_die_waterpolo_speelt_kleur2", "1F93D-1F3FC"},
    {"persoon_die_waterpolo_speelt_kleur3", "1F93D-1F3FD"},
    {"persoon_die_waterpolo_speelt_kleur4", "1F93D-1F3FE"},
    {"persoon_die_waterpolo_speelt_kleur5", "1F93D-1F3FF"},
    {"persoon_die_handbal_speelt", "1F93E"},
    {"persoon_die_handbal_speelt_kleur1", "1F93E-1F3FB"},
    {"persoon_die_handbal_speelt_kleur2", "1F93E-1F3FC"},
    {"persoon_die_handbal_speelt_kleur3", "1F93E-1F3FD"},
    {"persoon_die_handbal_speelt_kleur4", "1F93E-1F3FE"},
    {"persoon_die_handbal_speelt_kleur5", "1F93E-1F3FF"},
    {"duikbril", "1F93F"},
    {"verwelkte_bloem", "1F940"},
    {"trommel", "1F941"},
    {"klinkende_glazen", "1F942"},
    {"tumblerglas", "1F943"},
    {"lepel", "1F944"},
    {"doelnet", "1F945"},
    {"medaille_1e_plaats", "1F947"},
    {"medaille_2e_plaats", "1F948"},
    {"medaille_3e_plaats", "1F949"},
    {"bokshandschoen", "1F94A"},
    {"pak_voor_vechtsporten", "1F94B"},
    {"curlingsteen", "1F94C"},
    {"lacrosse", "1F94D"},
    {"softbal", "1F94E"},
    {"frisbee", "1F94F"},
    {"croissant", "1F950"},
    {"avocado", "1F951"},
    {"komkommer", "1F952"},
    {"bacon", "1F953"},
    {"aardappel", "1F954"},
    {"wortel", "1F955"},
    {"stokbrood", "1F956"},
    {"groene_salade", "1F957"},
    {"paellapan", "1F958"},
    {"gevuld_pitabroodje", "1F959"},
    {"ei", "1F95A"},
    {"glas_melk", "1F95B"},
    {"pindas", "1F95C"},
    {"kiwi", "1F95D"},
    {"pannenkoeken", "1F95E"},
    {"knoedel", "1F95F"},
    {"gelukskoekje", "1F960"},
    {"afhaalmaaltijd", "1F961"},
    {"chopsticks", "1F962"},
    {"kom_met_lepel", "1F963"},
    {"beker_met_rietje", "1F964"},
    {"kokosnoot", "1F965"},
    {"broccoli", "1F966"},
    {"taart", "1F967"},
    {"pretzel", "1F968"},
    {"vlees", "1F969"},
    {"boterham", "1F96A"},
    {"blikvoedsel", "1F96B"},
    {"bladgroente", "1F96C"},
    {"mango", "1F96D"},
    {"maancake", "1F96E"},
    {"bagel", "1F96F"},
    {"lachend_gezicht_met_hartjes", "1F970"},
    {"gapend_gezicht", "1F971"},
    {"lachend_gezicht_met_traan", "1F972"},
    {"feestend_gezicht", "1F973"},
    {"wazig_gezicht", "1F974"},
    {"heet_gezicht", "1F975"},
    {"koud_gezicht", "1F976"},
    {"ninja", "1F977"},
    {"ninja_kleur1", "1F977-1F3FB"},
    {"ninja_kleur2", "1F977-1F3FC"},
    {"ninja_kleur3", "1F977-1F3FD"},
    {"ninja_kleur4", "1F977-1F3FE"},
    {"ninja_kleur5", "1F977-1F3FF"},
    {"vermomd_gezicht", "1F978"},
    {"gezicht_dat_tranen_tegenhoudt", "1F979"},
    {"smekend_gezicht", "1F97A"},
    {"sari", "1F97B"},
    {"labjas", "1F97C"},
    {"oogscherm", "1F97D"},
    {"wandelschoen", "1F97E"},
    {"platte_schoen", "1F97F"},
    {"krab", "1F980"},
    {"leeuw", "1F981"},
    {"schorpioen", "1F982"},
    {"kalkoen", "1F983"},
    {"eenhoorn", "1F984"},
    {"adelaar", "1F985"},
    {"eend", "1F986"},
    {"vleermuis", "1F987"},
    {"haai", "1F988"},
    {"uil", "1F989"},
    {"vos", "1F98A"},
    {"vlinder", "1F98B"},
    {"hert", "1F98C"},
    {"gorilla", "1F98D"},
    {"hagedis", "1F98E"},
    {"neushoorn", "1F98F"},
    {"garnaal", "1F990"},
    {"inktvis", "1F991"},
    {"giraffe", "1F992"},
    {"zebra", "1F993"},
    {"egel", "1F994"},
    {"sauropode", "1F995"},
    {"trex", "1F996"},
    {"krekel", "1F997"},
    {"kangoeroe", "1F998"},
    {"lama", "1F999"},
    {"pauw", "1F99A"},
    {"nijlpaard", "1F99B"},
    {"papegaai", "1F99C"},
    {"wasbeer", "1F99D"},
    {"kreeft", "1F99E"},
    {"mug", "1F99F"},
    {"microbe", "1F9A0"},
    {"das", "1F9A1"},
    {"zwaan", "1F9A2"},
    {"mammoet", "1F9A3"},
    {"dodo", "1F9A4"},
    {"luiaard", "1F9A5"},
    {"otter", "1F9A6"},
    {"orang_oetang", "1F9A7"},
    {"stinkdier", "1F9A8"},
    {"flamingo", "1F9A9"},
    {"oester", "1F9AA"},
    {"bever", "1F9AB"},
    {"bizon", "1F9AC"},
    {"zeehond", "1F9AD"},
    {"blindengeleidehond", "1F9AE"},
    {"taststok", "1F9AF"},
    {"rood_haar", "1F9B0"},
    {"krullend_haar", "1F9B1"},
    {"kaal", "1F9B2"},
    {"wit_haar", "1F9B3"},
    {"bot", "1F9B4"},
    {"been", "1F9B5"},
    {"been_kleur1", "1F9B5-1F3FB"},
    {"been_kleur2", "1F9B5-1F3FC"},
    {"been_kleur3", "1F9B5-1F3FD"},
    {"been_kleur4", "1F9B5-1F3FE"},
    {"been_kleur5", "1F9B5-1F3FF"},
    {"voet", "1F9B6"},
    {"voet_kleur1", "1F9B6-1F3FB"},
    {"voet_kleur2", "1F9B6-1F3FC"},
    {"voet_kleur3", "1F9B6-1F3FD"},
    {"voet_kleur4", "1F9B6-1F3FE"},
    {"voet_kleur5", "1F9B6-1F3FF"},
    {"tand", "1F9B7"},
    {"superheld", "1F9B8"},
    {"superheld_kleur1", "1F9B8-1F3FB"},
    {"superheld_kleur2", "1F9B8-1F3FC"},
    {"superheld_kleur3", "1F9B8-1F3FD"},
    {"superheld_kleur4", "1F9B8-1F3FE"},
    {"superheld_kleur5", "1F9B8-1F3FF"},
    {"superschurk", "1F9B9"},
    {"superschurk_kleur1", "1F9B9-1F3FB"},
    {"superschurk_kleur2", "1F9B9-1F3FC"},
    {"superschurk_kleur3", "1F9B9-1F3FD"},
    {"superschurk_kleur4", "1F9B9-1F3FE"},
    {"superschurk_kleur5", "1F9B9-1F3FF"},
    {"veiligheidsvest", "1F9BA"},
    {"oor_met_gehoorapparaat", "1F9BB"},
    {"oor_met_gehoorapparaat_kleur1", "1F9BB-1F3FB"},
    {"oor_met_gehoorapparaat_kleur2", "1F9BB-1F3FC"},
    {"oor_met_gehoorapparaat_kleur3", "1F9BB-1F3FD"},
    {"oor_met_gehoorapparaat_kleur4", "1F9BB-1F3FE"},
    {"oor_met_gehoorapparaat_kleur5", "1F9BB-1F3FF"},
    {"elektrische_rolstoel", "1F9BC"},
    {"rolstoel", "1F9BD"},
    {"armprothese", "1F9BE"},
    {"beenprothese", "1F9BF"},
    {"stuk_kaas", "1F9C0"},
    {"cupcake", "1F9C1"},
    {"zout", "1F9C2"},
    {"pakje_drinken", "1F9C3"},
    {"knoflook", "1F9C4"},
    {"ui", "1F9C5"},
    {"falafel", "1F9C6"},
    {"wafel", "1F9C7"},
    {"boter", "1F9C8"},
    {"mate", "1F9C9"},
    {"ijsklontje", "1F9CA"},
    {"bubbelthee", "1F9CB"},
    {"trol", "1F9CC"},
    {"staande_persoon", "1F9CD"},
    {"staande_persoon_kleur1", "1F9CD-1F3FB"},
    {"staande_persoon_kleur2", "1F9CD-1F3FC"},
    {"staande_persoon_kleur3", "1F9CD-1F3FD"},
    {"staande_persoon_kleur4", "1F9CD-1F3FE"},
    {"staande_persoon_kleur5", "1F9CD-1F3FF"},
    {"knielende_persoon", "1F9CE"},
    {"knielende_persoon_kleur1", "1F9CE-1F3FB"},
    {"knielende_persoon_kleur2", "1F9CE-1F3FC"},
    {"knielende_persoon_kleur3", "1F9CE-1F3FD"},
    {"knielende_persoon_kleur4", "1F9CE-1F3FE"},
    {"knielende_persoon_kleur5", "1F9CE-1F3FF"},
    {"doof_persoon", "1F9CF"},
    {"doof_persoon_kleur1", "1F9CF-1F3FB"},
    {"doof_persoon_kleur2", "1F9CF-1F3FC"},
    {"doof_persoon_kleur3", "1F9CF-1F3FD"},
    {"doof_persoon_kleur4", "1F9CF-1F3FE"},
    {"doof_persoon_kleur5", "1F9CF-1F3FF"},
    {"gezicht_met_monocle", "1F9D0"},
    {"volwassene", "1F9D1"},
    {"volwassene_kleur1", "1F9D1-1F3FB"},
    {"volwassene_kleur2", "1F9D1-1F3FC"},
    {"volwassene_kleur3", "1F9D1-1F3FD"},
    {"volwassene_kleur4", "1F9D1-1F3FE"},
    {"volwassene_kleur5", "1F9D1-1F3FF"},
    {"kind", "1F9D2"},
    {"kind_kleur1", "1F9D2-1F3FB"},
    {"kind_kleur2", "1F9D2-1F3FC"},
    {"kind_kleur3", "1F9D2-1F3FD"},
    {"kind_kleur4", "1F9D2-1F3FE"},
    {"kind_kleur5", "1F9D2-1F3FF"},
    {"oudere_persoon", "1F9D3"},
    {"oudere_persoon_kleur1", "1F9D3-1F3FB"},
    {"oudere_persoon_kleur2", "1F9D3-1F3FC"},
    {"oudere_persoon_kleur3", "1F9D3-1F3FD"},
    {"oudere_persoon_kleur4", "1F9D3-1F3FE"},
    {"oudere_persoon_kleur5", "1F9D3-1F3FF"},
    {"persoon_baard", "1F9D4"},
    {"persoon_baard_kleur1", "1F9D4-1F3FB"},
    {"persoon_baard_kleur2", "1F9D4-1F3FC"},
    {"persoon_baard_kleur3", "1F9D4-1F3FD"},
    {"persoon_baard_kleur4", "1F9D4-1F3FE"},
    {"persoon_baard_kleur5", "1F9D4-1F3FF"},
    {"vrouw_met_hoofddoek", "1F9D5"},
    {"vrouw_met_hoofddoek_kleur1", "1F9D5-1F3FB"},
    {"vrouw_met_hoofddoek_kleur2", "1F9D5-1F3FC"},
    {"vrouw_met_hoofddoek_kleur3", "1F9D5-1F3FD"},
    {"vrouw_met_hoofddoek_kleur4", "1F9D5-1F3FE"},
    {"vrouw_met_hoofddoek_kleur5", "1F9D5-1F3FF"},
    {"persoon_in_ruimte_vol_stoom", "1F9D6"},
    {"persoon_in_ruimte_vol_stoom_kleur1", "1F9D6-1F3FB"},
    {"persoon_in_ruimte_vol_stoom_kleur2", "1F9D6-1F3FC"},
    {"persoon_in_ruimte_vol_stoom_kleur3", "1F9D6-1F3FD"},
    {"persoon_in_ruimte_vol_stoom_kleur4", "1F9D6-1F3FE"},
    {"persoon_in_ruimte_vol_stoom_kleur5", "1F9D6-1F3FF"},
    {"klimmende_persoon", "1F9D7"},
    {"klimmende_persoon_kleur1", "1F9D7-1F3FB"},
    {"klimmende_persoon_kleur2", "1F9D7-1F3FC"},
    {"klimmende_persoon_kleur3", "1F9D7-1F3FD"},
    {"klimmende_persoon_kleur4", "1F9D7-1F3FE"},
    {"klimmende_persoon_kleur5", "1F9D7-1F3FF"},
    {"persoon_in_lotushouding", "1F9D8"},
    {"persoon_in_lotushouding_kleur1", "1F9D8-1F3FB"},
    {"persoon_in_lotushouding_kleur2", "1F9D8-1F3FC"},
    {"persoon_in_lotushouding_kleur3", "1F9D8-1F3FD"},
    {"persoon_in_lotushouding_kleur4", "1F9D8-1F3FE"},
    {"persoon_in_lotushouding_kleur5", "1F9D8-1F3FF"},
    {"magier", "1F9D9"},
    {"magier_kleur1", "1F9D9-1F3FB"},
    {"magier_kleur2", "1F9D9-1F3FC"},
    {"magier_kleur3", "1F9D9-1F3FD"},
    {"magier_kleur4", "1F9D9-1F3FE"},
    {"magier_kleur5", "1F9D9-1F3FF"},
    {"elfje", "1F9DA"},
    {"elfje_kleur1", "1F9DA-1F3FB"},
    {"elfje_kleur2", "1F9DA-1F3FC"},
    {"elfje_kleur3", "1F9DA-1F3FD"},
    {"elfje_kleur4", "1F9DA-1F3FE"},
    {"elfje_kleur5", "1F9DA-1F3FF"},
    {"vampier", "1F9DB"},
    {"vampier_kleur1", "1F9DB-1F3FB"},
    {"vampier_kleur2", "1F9DB-1F3FC"},
    {"vampier_kleur3", "1F9DB-1F3FD"},
    {"vampier_kleur4", "1F9DB-1F3FE"},
    {"vampier_kleur5", "1F9DB-1F3FF"},
    {"zeemeermin", "1F9DC"},
    {"zeemeermin_kleur1", "1F9DC-1F3FB"},
    {"zeemeermin_kleur2", "1F9DC-1F3FC"},
    {"zeemeermin_kleur3", "1F9DC-1F3FD"},
    {"zeemeermin_kleur4", "1F9DC-1F3FE"},
    {"zeemeermin_kleur5", "1F9DC-1F3FF"},
    {"elf", "1F9DD"},
    {"elf_kleur1", "1F9DD-1F3FB"},
    {"elf_kleur2", "1F9DD-1F3FC"},
    {"elf_kleur3", "1F9DD-1F3FD"},
    {"elf_kleur4", "1F9DD-1F3FE"},
    {"elf_kleur5", "1F9DD-1F3FF"},
    {"geest_in_de_fles", "1F9DE"},
    {"zombie", "1F9DF"},
    {"brein", "1F9E0"},
    {"oranje_hart", "1F9E1"},
    {"pet", "1F9E2"},
    {"sjaal", "1F9E3"},
    {"handschoenen", "1F9E4"},
    {"jas", "1F9E5"},
    {"sokken", "1F9E6"},
    {"rode_envelop", "1F9E7"},
    {"rotje", "1F9E8"},
    {"puzzelstukje", "1F9E9"},
    {"reageerbuis", "1F9EA"},
    {"petrischaal", "1F9EB"},
    {"dna", "1F9EC"},
    {"kompas", "1F9ED"},
    {"abacus", "1F9EE"},
    {"brandblusapparaat", "1F9EF"},
    {"gereedschapskist", "1F9F0"},
    {"baksteen", "1F9F1"},
    {"magneet", "1F9F2"},
    {"bagage", "1F9F3"},
    {"lotionfles", "1F9F4"},
    {"garen", "1F9F5"},
    {"wol", "1F9F6"},
    {"veiligheidsspeld", "1F9F7"},
    {"teddybeer", "1F9F8"},
    {"bezem", "1F9F9"},
    {"mand", "1F9FA"},
    {"rol_papier", "1F9FB"},
    {"zeep", "1F9FC"},
    {"spons", "1F9FD"},
    {"kassabon", "1F9FE"},
    {"nazar_amulet", "1F9FF"},
    {"balletschoenen", "1FA70"},
    {"badpak", "1FA71"},
    {"slip", "1FA72"},
    {"korte_broek", "1FA73"},
    {"teenslipper", "1FA74"},
    {"lichtblauw_hart", "1FA75"},
    {"grijs_hart", "1FA76"},
    {"roze_hart", "1FA77"},
    {"bloeddruppel", "1FA78"},
    {"pleister", "1FA79"},
    {"stethoscoop", "1FA7A"},
    {"rontgenfoto", "1FA7B"},
    {"kruk", "1FA7C"},
    {"jojo", "1FA80"},
    {"vlieger", "1FA81"},
    {"parachute", "1FA82"},
    {"boemerang", "1FA83"},
    {"toverstok", "1FA84"},
    {"pinata", "1FA85"},
    {"matroesjka", "1FA86"},
    {"sambaballen", "1FA87"},
    {"fluit", "1FA88"},
    {"harp", "1FA89"},
    {"trombone", "1FA8A"},
    {"schatkist", "1FA8E"},
    {"schop", "1FA8F"},
    {"planeet_met_ringen", "1FA90"},
    {"hoge_stoel", "1FA91"},
    {"scheermes", "1FA92"},
    {"bijl", "1FA93"},
    {"diya_lamp", "1FA94"},
    {"banjo", "1FA95"},
    {"legerhelm", "1FA96"},
    {"accordeon", "1FA97"},
    {"hoge_trom", "1FA98"},
    {"munt", "1FA99"},
    {"houtzaag", "1FA9A"},
    {"schroevendraaier", "1FA9B"},
    {"ladder", "1FA9C"},
    {"haak", "1FA9D"},
    {"spiegel", "1FA9E"},
    {"raam", "1FA9F"},
    {"ontstopper", "1FAA0"},
    {"naald", "1FAA1"},
    {"knoop", "1FAA2"},
    {"emmer", "1FAA3"},
    {"muizenval", "1FAA4"},
    {"tandenborstel", "1FAA5"},
    {"grafsteen", "1FAA6"},
    {"plakkaat", "1FAA7"},
    {"rots", "1FAA8"},
    {"spiegelbal", "1FAA9"},
    {"legitimatiebewijs", "1FAAA"},
    {"batterij_bijna_leeg", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"opvouwbare_waaier", "1FAAD"},
    {"haarkam", "1FAAE"},
    {"khanda", "1FAAF"},
    {"vlieg", "1FAB0"},
    {"worm", "1FAB1"},
    {"kever", "1FAB2"},
    {"kakkerlak", "1FAB3"},
    {"potplant", "1FAB4"},
    {"hout", "1FAB5"},
    {"veer", "1FAB6"},
    {"lotus", "1FAB7"},
    {"koraal", "1FAB8"},
    {"leeg_nest", "1FAB9"},
    {"nest_met_eieren", "1FABA"},
    {"hyacint", "1FABB"},
    {"kwal", "1FABC"},
    {"vleugel", "1FABD"},
    {"boom_zonder_bladeren", "1FABE"},
    {"gans", "1FABF"},
    {"hart", "1FAC0"},
    {"longen", "1FAC1"},
    {"personen_in_een_omhelzing", "1FAC2"},
    {"zwangere_man", "1FAC3"},
    {"zwangere_man_kleur1", "1FAC3-1F3FB"},
    {"zwangere_man_kleur2", "1FAC3-1F3FC"},
    {"zwangere_man_kleur3", "1FAC3-1F3FD"},
    {"zwangere_man_kleur4", "1FAC3-1F3FE"},
    {"zwangere_man_kleur5", "1FAC3-1F3FF"},
    {"zwanger_persoon", "1FAC4"},
    {"zwanger_persoon_kleur1", "1FAC4-1F3FB"},
    {"zwanger_persoon_kleur2", "1FAC4-1F3FC"},
    {"zwanger_persoon_kleur3", "1FAC4-1F3FD"},
    {"zwanger_persoon_kleur4", "1FAC4-1F3FE"},
    {"zwanger_persoon_kleur5", "1FAC4-1F3FF"},
    {"persoon_met_kroon", "1FAC5"},
    {"persoon_met_kroon_kleur1", "1FAC5-1F3FB"},
    {"persoon_met_kroon_kleur2", "1FAC5-1F3FC"},
    {"persoon_met_kroon_kleur3", "1FAC5-1F3FD"},
    {"persoon_met_kroon_kleur4", "1FAC5-1F3FE"},
    {"persoon_met_kroon_kleur5", "1FAC5-1F3FF"},
    {"vingerafdruk", "1FAC6"},
    {"harig_wezen", "1FAC8"},
    {"orka", "1FACD"},
    {"eland", "1FACE"},
    {"ezel", "1FACF"},
    {"bosbessen", "1FAD0"},
    {"paprika", "1FAD1"},
    {"olijf", "1FAD2"},
    {"platbrood", "1FAD3"},
    {"tamale", "1FAD4"},
    {"fondue", "1FAD5"},
    {"theepot", "1FAD6"},
    {"vloeistof_uitgieten", "1FAD7"},
    {"bonen", "1FAD8"},
    {"pot", "1FAD9"},
    {"gemberwortel", "1FADA"},
    {"erwtendop", "1FADB"},
    {"knol", "1FADC"},
    {"gespat", "1FADF"},
    {"smeltend_gezicht", "1FAE0"},
    {"saluerend_gezicht", "1FAE1"},
    {"gezicht_met_open_ogen_en_hand_voor_mond", "1FAE2"},
    {"gezicht_met_glurend_oog", "1FAE3"},
    {"gezicht_met_schuine_mond", "1FAE4"},
    {"gezicht_in_stippellijn", "1FAE5"},
    {"mond_die_op_lip_bijt", "1FAE6"},
    {"zeepbellen", "1FAE7"},
    {"schuddend_gezicht", "1FAE8"},
    {"gezicht_met_wallen_onder_de_ogen", "1FAE9"},
    {"verwrongen_gezicht", "1FAEA"},
    {"vechtwolk", "1FAEF"},
    {"hand_met_wijsvinger_en_duim_gekruist", "1FAF0"},
    {"hand_met_wijsvinger_en_duim_gekruist_kleur1", "1FAF0-1F3FB"},
    {"hand_met_wijsvinger_en_duim_gekruist_kleur2", "1FAF0-1F3FC"},
    {"hand_met_wijsvinger_en_duim_gekruist_kleur3", "1FAF0-1F3FD"},
    {"hand_met_wijsvinger_en_duim_gekruist_kleur4", "1FAF0-1F3FE"},
    {"hand_met_wijsvinger_en_duim_gekruist_kleur5", "1FAF0-1F3FF"},
    {"hand_naar_rechts", "1FAF1"},
    {"hand_naar_rechts_kleur1", "1FAF1-1F3FB"},
    {"hand_naar_rechts_kleur2", "1FAF1-1F3FC"},
    {"hand_naar_rechts_kleur3", "1FAF1-1F3FD"},
    {"hand_naar_rechts_kleur4", "1FAF1-1F3FE"},
    {"hand_naar_rechts_kleur5", "1FAF1-1F3FF"},
    {"hand_naar_links", "1FAF2"},
    {"hand_naar_links_kleur1", "1FAF2-1F3FB"},
    {"hand_naar_links_kleur2", "1FAF2-1F3FC"},
    {"hand_naar_links_kleur3", "1FAF2-1F3FD"},
    {"hand_naar_links_kleur4", "1FAF2-1F3FE"},
    {"hand_naar_links_kleur5", "1FAF2-1F3FF"},
    {"handpalm_omlaag", "1FAF3"},
    {"handpalm_omlaag_kleur1", "1FAF3-1F3FB"},
    {"handpalm_omlaag_kleur2", "1FAF3-1F3FC"},
    {"handpalm_omlaag_kleur3", "1FAF3-1F3FD"},
    {"handpalm_omlaag_kleur4", "1FAF3-1F3FE"},
    {"handpalm_omlaag_kleur5", "1FAF3-1F3FF"},
    {"handpalm_omhoog", "1FAF4"},
    {"handpalm_omhoog_kleur1", "1FAF4-1F3FB"},
    {"handpalm_omhoog_kleur2", "1FAF4-1F3FC"},
    {"handpalm_omhoog_kleur3", "1FAF4-1F3FD"},
    {"handpalm_omhoog_kleur4", "1FAF4-1F3FE"},
    {"handpalm_omhoog_kleur5", "1FAF4-1F3FF"},
    {"wijsvinger_die_naar_de_kijker_wijst", "1FAF5"},
    {"wijsvinger_die_naar_de_kijker_wijst_kleur1", "1FAF5-1F3FB"},
    {"wijsvinger_die_naar_de_kijker_wijst_kleur2", "1FAF5-1F3FC"},
    {"wijsvinger_die_naar_de_kijker_wijst_kleur3", "1FAF5-1F3FD"},
    {"wijsvinger_die_naar_de_kijker_wijst_kleur4", "1FAF5-1F3FE"},
    {"wijsvinger_die_naar_de_kijker_wijst_kleur5", "1FAF5-1F3FF"},
    {"handen_in_de_vorm_van_een_hart", "1FAF6"},
    {"handen_in_de_vorm_van_een_hart_kleur1", "1FAF6-1F3FB"},
    {"handen_in_de_vorm_van_een_hart_kleur2", "1FAF6-1F3FC"},
    {"handen_in_de_vorm_van_een_hart_kleur3", "1FAF6-1F3FD"},
    {"handen_in_de_vorm_van_een_hart_kleur4", "1FAF6-1F3FE"},
    {"handen_in_de_vorm_van_een_hart_kleur5", "1FAF6-1F3FF"},
    {"hand_die_naar_links_duwt", "1FAF7"},
    {"hand_die_naar_links_duwt_kleur1", "1FAF7-1F3FB"},
    {"hand_die_naar_links_duwt_kleur2", "1FAF7-1F3FC"},
    {"hand_die_naar_links_duwt_kleur3", "1FAF7-1F3FD"},
    {"hand_die_naar_links_duwt_kleur4", "1FAF7-1F3FE"},
    {"hand_die_naar_links_duwt_kleur5", "1FAF7-1F3FF"},
    {"hand_die_naar_rechts_duwt", "1FAF8"},
    {"hand_die_naar_rechts_duwt_kleur1", "1FAF8-1F3FB"},
    {"hand_die_naar_rechts_duwt_kleur2", "1FAF8-1F3FC"},
    {"hand_die_naar_rechts_duwt_kleur3", "1FAF8-1F3FD"},
    {"hand_die_naar_rechts_duwt_kleur4", "1FAF8-1F3FE"},
    {"hand_die_naar_rechts_duwt_kleur5", "1FAF8-1F3FF"},
    {"toets_nummer_teken", "0023-FE0F-20E3"},
    {"toets_asterisk", "002A-FE0F-20E3"},
    {"toets_0", "0030-FE0F-20E3"},
    {"toets_1", "0031-FE0F-20E3"},
    {"toets_2", "0032-FE0F-20E3"},
    {"toets_3", "0033-FE0F-20E3"},
    {"toets_4", "0034-FE0F-20E3"},
    {"toets_5", "0035-FE0F-20E3"},
    {"toets_6", "0036-FE0F-20E3"},
    {"toets_7", "0037-FE0F-20E3"},
    {"toets_8", "0038-FE0F-20E3"},
    {"toets_9", "0039-FE0F-20E3"},
    {"vlag_ascension", "1F1E6-1F1E8"},
    {"vlag_andorra", "1F1E6-1F1E9"},
    {"vlag_verenigde_arabische_emiraten", "1F1E6-1F1EA"},
    {"vlag_afghanistan", "1F1E6-1F1EB"},
    {"vlag_antigua_en_barbuda", "1F1E6-1F1EC"},
    {"vlag_anguilla", "1F1E6-1F1EE"},
    {"vlag_albanie", "1F1E6-1F1F1"},
    {"vlag_armenie", "1F1E6-1F1F2"},
    {"vlag_angola", "1F1E6-1F1F4"},
    {"vlag_antarctica", "1F1E6-1F1F6"},
    {"vlag_argentinie", "1F1E6-1F1F7"},
    {"vlag_amerikaans_samoa", "1F1E6-1F1F8"},
    {"vlag_oostenrijk", "1F1E6-1F1F9"},
    {"vlag_australie", "1F1E6-1F1FA"},
    {"vlag_aruba", "1F1E6-1F1FC"},
    {"vlag_aland", "1F1E6-1F1FD"},
    {"vlag_azerbeidzjan", "1F1E6-1F1FF"},
    {"vlag_bosnie_en_herzegovina", "1F1E7-1F1E6"},
    {"vlag_barbados", "1F1E7-1F1E7"},
    {"vlag_bangladesh", "1F1E7-1F1E9"},
    {"vlag_belgie", "1F1E7-1F1EA"},
    {"vlag_burkina_faso", "1F1E7-1F1EB"},
    {"vlag_bulgarije", "1F1E7-1F1EC"},
    {"vlag_bahrein", "1F1E7-1F1ED"},
    {"vlag_burundi", "1F1E7-1F1EE"},
    {"vlag_benin", "1F1E7-1F1EF"},
    {"vlag_saint_barthelemy", "1F1E7-1F1F1"},
    {"vlag_bermuda", "1F1E7-1F1F2"},
    {"vlag_brunei", "1F1E7-1F1F3"},
    {"vlag_bolivia", "1F1E7-1F1F4"},
    {"vlag_caribisch_nederland", "1F1E7-1F1F6"},
    {"vlag_brazilie", "1F1E7-1F1F7"},
    {"vlag_bahamas", "1F1E7-1F1F8"},
    {"vlag_bhutan", "1F1E7-1F1F9"},
    {"vlag_bouveteiland", "1F1E7-1F1FB"},
    {"vlag_botswana", "1F1E7-1F1FC"},
    {"vlag_belarus", "1F1E7-1F1FE"},
    {"vlag_belize", "1F1E7-1F1FF"},
    {"vlag_canada", "1F1E8-1F1E6"},
    {"vlag_cocoseilanden", "1F1E8-1F1E8"},
    {"vlag_congo_kinshasa", "1F1E8-1F1E9"},
    {"vlag_centraal_afrikaanse_republiek", "1F1E8-1F1EB"},
    {"vlag_congo_brazzaville", "1F1E8-1F1EC"},
    {"vlag_zwitserland", "1F1E8-1F1ED"},
    {"vlag_ivoorkust", "1F1E8-1F1EE"},
    {"vlag_cookeilanden", "1F1E8-1F1F0"},
    {"vlag_chili", "1F1E8-1F1F1"},
    {"vlag_kameroen", "1F1E8-1F1F2"},
    {"vlag_china", "1F1E8-1F1F3"},
    {"vlag_colombia", "1F1E8-1F1F4"},
    {"vlag_clipperton", "1F1E8-1F1F5"},
    {"vlag_sark", "1F1E8-1F1F6"},
    {"vlag_costa_rica", "1F1E8-1F1F7"},
    {"vlag_cuba", "1F1E8-1F1FA"},
    {"vlag_kaapverdie", "1F1E8-1F1FB"},
    {"vlag_curacao", "1F1E8-1F1FC"},
    {"vlag_christmaseiland", "1F1E8-1F1FD"},
    {"vlag_cyprus", "1F1E8-1F1FE"},
    {"vlag_tsjechie", "1F1E8-1F1FF"},
    {"vlag_duitsland", "1F1E9-1F1EA"},
    {"vlag_diego_garcia", "1F1E9-1F1EC"},
    {"vlag_djibouti", "1F1E9-1F1EF"},
    {"vlag_denemarken", "1F1E9-1F1F0"},
    {"vlag_dominica", "1F1E9-1F1F2"},
    {"vlag_dominicaanse_republiek", "1F1E9-1F1F4"},
    {"vlag_algerije", "1F1E9-1F1FF"},
    {"vlag_ceuta_en_melilla", "1F1EA-1F1E6"},
    {"vlag_ecuador", "1F1EA-1F1E8"},
    {"vlag_estland", "1F1EA-1F1EA"},
    {"vlag_egypte", "1F1EA-1F1EC"},
    {"vlag_westelijke_sahara", "1F1EA-1F1ED"},
    {"vlag_eritrea", "1F1EA-1F1F7"},
    {"vlag_spanje", "1F1EA-1F1F8"},
    {"vlag_ethiopie", "1F1EA-1F1F9"},
    {"vlag_europese_unie", "1F1EA-1F1FA"},
    {"vlag_finland", "1F1EB-1F1EE"},
    {"vlag_fiji", "1F1EB-1F1EF"},
    {"vlag_falklandeilanden", "1F1EB-1F1F0"},
    {"vlag_micronesia", "1F1EB-1F1F2"},
    {"vlag_faeroer", "1F1EB-1F1F4"},
    {"vlag_frankrijk", "1F1EB-1F1F7"},
    {"vlag_gabon", "1F1EC-1F1E6"},
    {"vlag_verenigd_koninkrijk", "1F1EC-1F1E7"},
    {"vlag_grenada", "1F1EC-1F1E9"},
    {"vlag_georgie", "1F1EC-1F1EA"},
    {"vlag_frans_guyana", "1F1EC-1F1EB"},
    {"vlag_guernsey", "1F1EC-1F1EC"},
    {"vlag_ghana", "1F1EC-1F1ED"},
    {"vlag_gibraltar", "1F1EC-1F1EE"},
    {"vlag_groenland", "1F1EC-1F1F1"},
    {"vlag_gambia", "1F1EC-1F1F2"},
    {"vlag_guinee", "1F1EC-1F1F3"},
    {"vlag_guadeloupe", "1F1EC-1F1F5"},
    {"vlag_equatoriaal_guinea", "1F1EC-1F1F6"},
    {"vlag_griekenland", "1F1EC-1F1F7"},
    {"vlag_zuid_georgia_en_zuidelijke_sandwicheilanden", "1F1EC-1F1F8"},
    {"vlag_guatemala", "1F1EC-1F1F9"},
    {"vlag_guam", "1F1EC-1F1FA"},
    {"vlag_guinee_bissau", "1F1EC-1F1FC"},
    {"vlag_guyana", "1F1EC-1F1FE"},
    {"vlag_hongkong_sar_van_china", "1F1ED-1F1F0"},
    {"vlag_heard_en_mcdonaldeilanden", "1F1ED-1F1F2"},
    {"vlag_honduras", "1F1ED-1F1F3"},
    {"vlag_kroatie", "1F1ED-1F1F7"},
    {"vlag_haiti", "1F1ED-1F1F9"},
    {"vlag_hongarije", "1F1ED-1F1FA"},
    {"vlag_canarische_eilanden", "1F1EE-1F1E8"},
    {"vlag_indonesie", "1F1EE-1F1E9"},
    {"vlag_ierland", "1F1EE-1F1EA"},
    {"vlag_israel", "1F1EE-1F1F1"},
    {"vlag_isle_of_man", "1F1EE-1F1F2"},
    {"vlag_india", "1F1EE-1F1F3"},
    {"vlag_brits_indische_oceaanterritorium", "1F1EE-1F1F4"},
    {"vlag_irak", "1F1EE-1F1F6"},
    {"vlag_iran", "1F1EE-1F1F7"},
    {"vlag_ijsland", "1F1EE-1F1F8"},
    {"vlag_italie", "1F1EE-1F1F9"},
    {"vlag_jersey", "1F1EF-1F1EA"},
    {"vlag_jamaica", "1F1EF-1F1F2"},
    {"vlag_jordanie", "1F1EF-1F1F4"},
    {"vlag_japan", "1F1EF-1F1F5"},
    {"vlag_kenia", "1F1F0-1F1EA"},
    {"vlag_kirgizie", "1F1F0-1F1EC"},
    {"vlag_cambodja", "1F1F0-1F1ED"},
    {"vlag_kiribati", "1F1F0-1F1EE"},
    {"vlag_comoren", "1F1F0-1F1F2"},
    {"vlag_saint_kitts_en_nevis", "1F1F0-1F1F3"},
    {"vlag_noord_korea", "1F1F0-1F1F5"},
    {"vlag_zuid_korea", "1F1F0-1F1F7"},
    {"vlag_koeweit", "1F1F0-1F1FC"},
    {"vlag_kaaimaneilanden", "1F1F0-1F1FE"},
    {"vlag_kazachstan", "1F1F0-1F1FF"},
    {"vlag_laos", "1F1F1-1F1E6"},
    {"vlag_libanon", "1F1F1-1F1E7"},
    {"vlag_saint_lucia", "1F1F1-1F1E8"},
    {"vlag_liechtenstein", "1F1F1-1F1EE"},
    {"vlag_sri_lanka", "1F1F1-1F1F0"},
    {"vlag_liberia", "1F1F1-1F1F7"},
    {"vlag_lesotho", "1F1F1-1F1F8"},
    {"vlag_litouwen", "1F1F1-1F1F9"},
    {"vlag_luxemburg", "1F1F1-1F1FA"},
    {"vlag_letland", "1F1F1-1F1FB"},
    {"vlag_libie", "1F1F1-1F1FE"},
    {"vlag_marokko", "1F1F2-1F1E6"},
    {"vlag_monaco", "1F1F2-1F1E8"},
    {"vlag_moldavie", "1F1F2-1F1E9"},
    {"vlag_montenegro", "1F1F2-1F1EA"},
    {"vlag_saint_martin", "1F1F2-1F1EB"},
    {"vlag_madagaskar", "1F1F2-1F1EC"},
    {"vlag_marshalleilanden", "1F1F2-1F1ED"},
    {"vlag_noord_macedonie", "1F1F2-1F1F0"},
    {"vlag_mali", "1F1F2-1F1F1"},
    {"vlag_myanmar_birma", "1F1F2-1F1F2"},
    {"vlag_mongolie", "1F1F2-1F1F3"},
    {"vlag_macau_sar_van_china", "1F1F2-1F1F4"},
    {"vlag_noordelijke_marianen", "1F1F2-1F1F5"},
    {"vlag_martinique", "1F1F2-1F1F6"},
    {"vlag_mauritanie", "1F1F2-1F1F7"},
    {"vlag_montserrat", "1F1F2-1F1F8"},
    {"vlag_malta", "1F1F2-1F1F9"},
    {"vlag_mauritius", "1F1F2-1F1FA"},
    {"vlag_maldiven", "1F1F2-1F1FB"},
    {"vlag_malawi", "1F1F2-1F1FC"},
    {"vlag_mexico", "1F1F2-1F1FD"},
    {"vlag_maleisie", "1F1F2-1F1FE"},
    {"vlag_mozambique", "1F1F2-1F1FF"},
    {"vlag_namibie", "1F1F3-1F1E6"},
    {"vlag_nieuw_caledonie", "1F1F3-1F1E8"},
    {"vlag_niger", "1F1F3-1F1EA"},
    {"vlag_norfolk", "1F1F3-1F1EB"},
    {"vlag_nigeria", "1F1F3-1F1EC"},
    {"vlag_nicaragua", "1F1F3-1F1EE"},
    {"vlag_nederland", "1F1F3-1F1F1"},
    {"vlag_noorwegen", "1F1F3-1F1F4"},
    {"vlag_nepal", "1F1F3-1F1F5"},
    {"vlag_nauru", "1F1F3-1F1F7"},
    {"vlag_niue", "1F1F3-1F1FA"},
    {"vlag_nieuw_zeeland", "1F1F3-1F1FF"},
    {"vlag_oman", "1F1F4-1F1F2"},
    {"vlag_panama", "1F1F5-1F1E6"},
    {"vlag_peru", "1F1F5-1F1EA"},
    {"vlag_frans_polynesie", "1F1F5-1F1EB"},
    {"vlag_papoea_nieuw_guinea", "1F1F5-1F1EC"},
    {"vlag_filipijnen", "1F1F5-1F1ED"},
    {"vlag_pakistan", "1F1F5-1F1F0"},
    {"vlag_polen", "1F1F5-1F1F1"},
    {"vlag_saint_pierre_en_miquelon", "1F1F5-1F1F2"},
    {"vlag_pitcairneilanden", "1F1F5-1F1F3"},
    {"vlag_puerto_rico", "1F1F5-1F1F7"},
    {"vlag_palestijnse_gebieden", "1F1F5-1F1F8"},
    {"vlag_portugal", "1F1F5-1F1F9"},
    {"vlag_palau", "1F1F5-1F1FC"},
    {"vlag_paraguay", "1F1F5-1F1FE"},
    {"vlag_qatar", "1F1F6-1F1E6"},
    {"vlag_reunion", "1F1F7-1F1EA"},
    {"vlag_roemenie", "1F1F7-1F1F4"},
    {"vlag_servie", "1F1F7-1F1F8"},
    {"vlag_rusland", "1F1F7-1F1FA"},
    {"vlag_rwanda", "1F1F7-1F1FC"},
    {"vlag_saoedi_arabie", "1F1F8-1F1E6"},
    {"vlag_salomonseilanden", "1F1F8-1F1E7"},
    {"vlag_seychellen", "1F1F8-1F1E8"},
    {"vlag_soedan", "1F1F8-1F1E9"},
    {"vlag_zweden", "1F1F8-1F1EA"},
    {"vlag_singapore", "1F1F8-1F1EC"},
    {"vlag_sint_helena", "1F1F8-1F1ED"},
    {"vlag_slovenie", "1F1F8-1F1EE"},
    {"vlag_spitsbergen_en_jan_mayen", "1F1F8-1F1EF"},
    {"vlag_slowakije", "1F1F8-1F1F0"},
    {"vlag_sierra_leone", "1F1F8-1F1F1"},
    {"vlag_san_marino", "1F1F8-1F1F2"},
    {"vlag_senegal", "1F1F8-1F1F3"},
    {"vlag_somalie", "1F1F8-1F1F4"},
    {"vlag_suriname", "1F1F8-1F1F7"},
    {"vlag_zuid_soedan", "1F1F8-1F1F8"},
    {"vlag_sao_tome_en_principe", "1F1F8-1F1F9"},
    {"vlag_el_salvador", "1F1F8-1F1FB"},
    {"vlag_sint_maarten", "1F1F8-1F1FD"},
    {"vlag_syrie", "1F1F8-1F1FE"},
    {"vlag_eswatini", "1F1F8-1F1FF"},
    {"vlag_tristan_da_cunha", "1F1F9-1F1E6"},
    {"vlag_turks_en_caicoseilanden", "1F1F9-1F1E8"},
    {"vlag_tsjaad", "1F1F9-1F1E9"},
    {"vlag_franse_gebieden_in_de_zuidelijke_indische_oceaan", "1F1F9-1F1EB"},
    {"vlag_togo", "1F1F9-1F1EC"},
    {"vlag_thailand", "1F1F9-1F1ED"},
    {"vlag_tadzjikistan", "1F1F9-1F1EF"},
    {"vlag_tokelau", "1F1F9-1F1F0"},
    {"vlag_oost_timor", "1F1F9-1F1F1"},
    {"vlag_turkmenistan", "1F1F9-1F1F2"},
    {"vlag_tunesie", "1F1F9-1F1F3"},
    {"vlag_tonga", "1F1F9-1F1F4"},
    {"vlag_turkije", "1F1F9-1F1F7"},
    {"vlag_trinidad_en_tobago", "1F1F9-1F1F9"},
    {"vlag_tuvalu", "1F1F9-1F1FB"},
    {"vlag_taiwan", "1F1F9-1F1FC"},
    {"vlag_tanzania", "1F1F9-1F1FF"},
    {"vlag_oekraine", "1F1FA-1F1E6"},
    {"vlag_oeganda", "1F1FA-1F1EC"},
    {"vlag_kleine_afgelegen_eilanden_van_de_verenigde_staten", "1F1FA-1F1F2"},
    {"vlag_verenigde_naties", "1F1FA-1F1F3"},
    {"vlag_verenigde_staten", "1F1FA-1F1F8"},
    {"vlag_uruguay", "1F1FA-1F1FE"},
    {"vlag_oezbekistan", "1F1FA-1F1FF"},
    {"vlag_vaticaanstad", "1F1FB-1F1E6"},
    {"vlag_saint_vincent_en_de_grenadines", "1F1FB-1F1E8"},
    {"vlag_venezuela", "1F1FB-1F1EA"},
    {"vlag_britse_maagdeneilanden", "1F1FB-1F1EC"},
    {"vlag_amerikaanse_maagdeneilanden", "1F1FB-1F1EE"},
    {"vlag_vietnam", "1F1FB-1F1F3"},
    {"vlag_vanuatu", "1F1FB-1F1FA"},
    {"vlag_wallis_en_futuna", "1F1FC-1F1EB"},
    {"vlag_samoa", "1F1FC-1F1F8"},
    {"vlag_kosovo", "1F1FD-1F1F0"},
    {"vlag_jemen", "1F1FE-1F1EA"},
    {"vlag_mayotte", "1F1FE-1F1F9"},
    {"vlag_zuid_afrika", "1F1FF-1F1E6"},
    {"vlag_zambia", "1F1FF-1F1F2"},
    {"vlag_zimbabwe", "1F1FF-1F1FC"},
    {"vlag_engeland", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"vlag_schotland", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"vlag_wales", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"stel_met_hart_man_man", "1F468-200D-2764-FE0F-200D-1F468"},
    {"stel_met_hart_man_man_kleur1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"stel_met_hart_man_man_kleur1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"stel_met_hart_man_man_kleur1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"stel_met_hart_man_man_kleur1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"stel_met_hart_man_man_kleur1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"stel_met_hart_man_man_kleur2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"stel_met_hart_man_man_kleur2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"stel_met_hart_man_man_kleur2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"stel_met_hart_man_man_kleur2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"stel_met_hart_man_man_kleur2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"stel_met_hart_man_man_kleur3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"stel_met_hart_man_man_kleur3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"stel_met_hart_man_man_kleur3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"stel_met_hart_man_man_kleur3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"stel_met_hart_man_man_kleur3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"stel_met_hart_man_man_kleur4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"stel_met_hart_man_man_kleur4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"stel_met_hart_man_man_kleur4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"stel_met_hart_man_man_kleur4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"stel_met_hart_man_man_kleur4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"stel_met_hart_man_man_kleur5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"stel_met_hart_man_man_kleur5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"stel_met_hart_man_man_kleur5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"stel_met_hart_man_man_kleur5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"stel_met_hart_man_man_kleur5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"kus_man_man", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"kus_man_man_kleur1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kus_man_man_kleur1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kus_man_man_kleur1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kus_man_man_kleur1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kus_man_man_kleur1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kus_man_man_kleur2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kus_man_man_kleur2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kus_man_man_kleur2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kus_man_man_kleur2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kus_man_man_kleur2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kus_man_man_kleur3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kus_man_man_kleur3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kus_man_man_kleur3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kus_man_man_kleur3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kus_man_man_kleur3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kus_man_man_kleur4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kus_man_man_kleur4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kus_man_man_kleur4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kus_man_man_kleur4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kus_man_man_kleur4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kus_man_man_kleur5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kus_man_man_kleur5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kus_man_man_kleur5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kus_man_man_kleur5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kus_man_man_kleur5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"gezin_man_jongen", "1F468-200D-1F466"},
    {"gezin_man_jongen_jongen", "1F468-200D-1F466-200D-1F466"},
    {"gezin_man_meisje", "1F468-200D-1F467"},
    {"gezin_man_meisje_jongen", "1F468-200D-1F467-200D-1F466"},
    {"gezin_man_meisje_meisje", "1F468-200D-1F467-200D-1F467"},
    {"gezin_man_man_jongen", "1F468-200D-1F468-200D-1F466"},
    {"gezin_man_man_jongen_jongen", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"gezin_man_man_meisje", "1F468-200D-1F468-200D-1F467"},
    {"gezin_man_man_meisje_jongen", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"gezin_man_man_meisje_meisje", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"gezin_man_vrouw_jongen", "1F468-200D-1F469-200D-1F466"},
    {"gezin_man_vrouw_jongen_jongen", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"gezin_man_vrouw_meisje", "1F468-200D-1F469-200D-1F467"},
    {"gezin_man_vrouw_meisje_jongen", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"gezin_man_vrouw_meisje_meisje", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"stel_met_hart_vrouw_man", "1F469-200D-2764-FE0F-200D-1F468"},
    {"stel_met_hart_vrouw_man_kleur1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"stel_met_hart_vrouw_man_kleur1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"stel_met_hart_vrouw_man_kleur1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"stel_met_hart_vrouw_man_kleur1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"stel_met_hart_vrouw_man_kleur1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"stel_met_hart_vrouw_man_kleur2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"stel_met_hart_vrouw_man_kleur2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"stel_met_hart_vrouw_man_kleur2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"stel_met_hart_vrouw_man_kleur2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"stel_met_hart_vrouw_man_kleur2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"stel_met_hart_vrouw_man_kleur3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"stel_met_hart_vrouw_man_kleur3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"stel_met_hart_vrouw_man_kleur3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"stel_met_hart_vrouw_man_kleur3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"stel_met_hart_vrouw_man_kleur3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"stel_met_hart_vrouw_man_kleur4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"stel_met_hart_vrouw_man_kleur4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"stel_met_hart_vrouw_man_kleur4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"stel_met_hart_vrouw_man_kleur4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"stel_met_hart_vrouw_man_kleur4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"stel_met_hart_vrouw_man_kleur5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"stel_met_hart_vrouw_man_kleur5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"stel_met_hart_vrouw_man_kleur5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"stel_met_hart_vrouw_man_kleur5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"stel_met_hart_vrouw_man_kleur5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"stel_met_hart_vrouw_vrouw", "1F469-200D-2764-FE0F-200D-1F469"},
    {"stel_met_hart_vrouw_vrouw_kleur1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"stel_met_hart_vrouw_vrouw_kleur1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"stel_met_hart_vrouw_vrouw_kleur1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"stel_met_hart_vrouw_vrouw_kleur1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"stel_met_hart_vrouw_vrouw_kleur1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"stel_met_hart_vrouw_vrouw_kleur2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"stel_met_hart_vrouw_vrouw_kleur2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"stel_met_hart_vrouw_vrouw_kleur2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"stel_met_hart_vrouw_vrouw_kleur2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"stel_met_hart_vrouw_vrouw_kleur2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"stel_met_hart_vrouw_vrouw_kleur3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"stel_met_hart_vrouw_vrouw_kleur3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"stel_met_hart_vrouw_vrouw_kleur3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"stel_met_hart_vrouw_vrouw_kleur3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"stel_met_hart_vrouw_vrouw_kleur3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"stel_met_hart_vrouw_vrouw_kleur4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"stel_met_hart_vrouw_vrouw_kleur4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"stel_met_hart_vrouw_vrouw_kleur4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"stel_met_hart_vrouw_vrouw_kleur4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"stel_met_hart_vrouw_vrouw_kleur4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"stel_met_hart_vrouw_vrouw_kleur5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"stel_met_hart_vrouw_vrouw_kleur5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"stel_met_hart_vrouw_vrouw_kleur5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"stel_met_hart_vrouw_vrouw_kleur5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"stel_met_hart_vrouw_vrouw_kleur5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"kus_vrouw_man", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"kus_vrouw_man_kleur1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kus_vrouw_man_kleur1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kus_vrouw_man_kleur1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kus_vrouw_man_kleur1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kus_vrouw_man_kleur1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kus_vrouw_man_kleur2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kus_vrouw_man_kleur2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kus_vrouw_man_kleur2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kus_vrouw_man_kleur2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kus_vrouw_man_kleur2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kus_vrouw_man_kleur3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kus_vrouw_man_kleur3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kus_vrouw_man_kleur3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kus_vrouw_man_kleur3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kus_vrouw_man_kleur3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kus_vrouw_man_kleur4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kus_vrouw_man_kleur4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kus_vrouw_man_kleur4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kus_vrouw_man_kleur4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kus_vrouw_man_kleur4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kus_vrouw_man_kleur5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"kus_vrouw_man_kleur5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"kus_vrouw_man_kleur5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"kus_vrouw_man_kleur5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"kus_vrouw_man_kleur5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"kus_vrouw_vrouw", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"kus_vrouw_vrouw_kleur1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kus_vrouw_vrouw_kleur1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kus_vrouw_vrouw_kleur1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kus_vrouw_vrouw_kleur1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kus_vrouw_vrouw_kleur1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"kus_vrouw_vrouw_kleur2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kus_vrouw_vrouw_kleur2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kus_vrouw_vrouw_kleur2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kus_vrouw_vrouw_kleur2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kus_vrouw_vrouw_kleur2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"kus_vrouw_vrouw_kleur3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kus_vrouw_vrouw_kleur3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kus_vrouw_vrouw_kleur3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kus_vrouw_vrouw_kleur3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kus_vrouw_vrouw_kleur3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"kus_vrouw_vrouw_kleur4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kus_vrouw_vrouw_kleur4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kus_vrouw_vrouw_kleur4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kus_vrouw_vrouw_kleur4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kus_vrouw_vrouw_kleur4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"kus_vrouw_vrouw_kleur5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"kus_vrouw_vrouw_kleur5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"kus_vrouw_vrouw_kleur5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"kus_vrouw_vrouw_kleur5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"kus_vrouw_vrouw_kleur5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"gezin_vrouw_jongen", "1F469-200D-1F466"},
    {"gezin_vrouw_jongen_jongen", "1F469-200D-1F466-200D-1F466"},
    {"gezin_vrouw_meisje", "1F469-200D-1F467"},
    {"gezin_vrouw_meisje_jongen", "1F469-200D-1F467-200D-1F466"},
    {"gezin_vrouw_meisje_meisje", "1F469-200D-1F467-200D-1F467"},
    {"gezin_vrouw_vrouw_jongen", "1F469-200D-1F469-200D-1F466"},
    {"gezin_vrouw_vrouw_jongen_jongen", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"gezin_vrouw_vrouw_meisje", "1F469-200D-1F469-200D-1F467"},
    {"gezin_vrouw_vrouw_meisje_jongen", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"gezin_vrouw_vrouw_meisje_meisje", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"personen_hand_in_hand", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"personen_hand_in_hand_kleur1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personen_hand_in_hand_kleur1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personen_hand_in_hand_kleur1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personen_hand_in_hand_kleur1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personen_hand_in_hand_kleur1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personen_hand_in_hand_kleur2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personen_hand_in_hand_kleur2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personen_hand_in_hand_kleur2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personen_hand_in_hand_kleur2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personen_hand_in_hand_kleur2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personen_hand_in_hand_kleur3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personen_hand_in_hand_kleur3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personen_hand_in_hand_kleur3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personen_hand_in_hand_kleur3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personen_hand_in_hand_kleur3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personen_hand_in_hand_kleur4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personen_hand_in_hand_kleur4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personen_hand_in_hand_kleur4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personen_hand_in_hand_kleur4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personen_hand_in_hand_kleur4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personen_hand_in_hand_kleur5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personen_hand_in_hand_kleur5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personen_hand_in_hand_kleur5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personen_hand_in_hand_kleur5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personen_hand_in_hand_kleur5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"gezin_volwassene_volwassene_kind", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"gezin_volwassene_volwassene_kind_kind", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"gezin_volwassene_kind", "1F9D1-200D-1F9D2"},
    {"gezin_volwassene_kind_kind", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"hardloper_naar_rechts_gericht", "1F3C3-200D-27A1-FE0F"},
    {"hardloper_naar_rechts_gericht_kleur1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"hardloper_naar_rechts_gericht_kleur2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"hardloper_naar_rechts_gericht_kleur3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"hardloper_naar_rechts_gericht_kleur4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"hardloper_naar_rechts_gericht_kleur5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"mannelijke_gezondheidsmedewerker", "1F468-200D-2695-FE0F"},
    {"mannelijke_gezondheidsmedewerker_kleur1", "1F468-1F3FB-200D-2695-FE0F"},
    {"mannelijke_gezondheidsmedewerker_kleur2", "1F468-1F3FC-200D-2695-FE0F"},
    {"mannelijke_gezondheidsmedewerker_kleur3", "1F468-1F3FD-200D-2695-FE0F"},
    {"mannelijke_gezondheidsmedewerker_kleur4", "1F468-1F3FE-200D-2695-FE0F"},
    {"mannelijke_gezondheidsmedewerker_kleur5", "1F468-1F3FF-200D-2695-FE0F"},
    {"mannelijke_rechter", "1F468-200D-2696-FE0F"},
    {"mannelijke_rechter_kleur1", "1F468-1F3FB-200D-2696-FE0F"},
    {"mannelijke_rechter_kleur2", "1F468-1F3FC-200D-2696-FE0F"},
    {"mannelijke_rechter_kleur3", "1F468-1F3FD-200D-2696-FE0F"},
    {"mannelijke_rechter_kleur4", "1F468-1F3FE-200D-2696-FE0F"},
    {"mannelijke_rechter_kleur5", "1F468-1F3FF-200D-2696-FE0F"},
    {"mannelijke_piloot", "1F468-200D-2708-FE0F"},
    {"mannelijke_piloot_kleur1", "1F468-1F3FB-200D-2708-FE0F"},
    {"mannelijke_piloot_kleur2", "1F468-1F3FC-200D-2708-FE0F"},
    {"mannelijke_piloot_kleur3", "1F468-1F3FD-200D-2708-FE0F"},
    {"mannelijke_piloot_kleur4", "1F468-1F3FE-200D-2708-FE0F"},
    {"mannelijke_piloot_kleur5", "1F468-1F3FF-200D-2708-FE0F"},
    {"mannelijke_boer", "1F468-200D-1F33E"},
    {"mannelijke_boer_kleur1", "1F468-1F3FB-200D-1F33E"},
    {"mannelijke_boer_kleur2", "1F468-1F3FC-200D-1F33E"},
    {"mannelijke_boer_kleur3", "1F468-1F3FD-200D-1F33E"},
    {"mannelijke_boer_kleur4", "1F468-1F3FE-200D-1F33E"},
    {"mannelijke_boer_kleur5", "1F468-1F3FF-200D-1F33E"},
    {"mannelijke_kok", "1F468-200D-1F373"},
    {"mannelijke_kok_kleur1", "1F468-1F3FB-200D-1F373"},
    {"mannelijke_kok_kleur2", "1F468-1F3FC-200D-1F373"},
    {"mannelijke_kok_kleur3", "1F468-1F3FD-200D-1F373"},
    {"mannelijke_kok_kleur4", "1F468-1F3FE-200D-1F373"},
    {"mannelijke_kok_kleur5", "1F468-1F3FF-200D-1F373"},
    {"man_die_een_baby_voedt", "1F468-200D-1F37C"},
    {"man_die_een_baby_voedt_kleur1", "1F468-1F3FB-200D-1F37C"},
    {"man_die_een_baby_voedt_kleur2", "1F468-1F3FC-200D-1F37C"},
    {"man_die_een_baby_voedt_kleur3", "1F468-1F3FD-200D-1F37C"},
    {"man_die_een_baby_voedt_kleur4", "1F468-1F3FE-200D-1F37C"},
    {"man_die_een_baby_voedt_kleur5", "1F468-1F3FF-200D-1F37C"},
    {"mannelijke_student", "1F468-200D-1F393"},
    {"mannelijke_student_kleur1", "1F468-1F3FB-200D-1F393"},
    {"mannelijke_student_kleur2", "1F468-1F3FC-200D-1F393"},
    {"mannelijke_student_kleur3", "1F468-1F3FD-200D-1F393"},
    {"mannelijke_student_kleur4", "1F468-1F3FE-200D-1F393"},
    {"mannelijke_student_kleur5", "1F468-1F3FF-200D-1F393"},
    {"mannelijke_zanger", "1F468-200D-1F3A4"},
    {"mannelijke_zanger_kleur1", "1F468-1F3FB-200D-1F3A4"},
    {"mannelijke_zanger_kleur2", "1F468-1F3FC-200D-1F3A4"},
    {"mannelijke_zanger_kleur3", "1F468-1F3FD-200D-1F3A4"},
    {"mannelijke_zanger_kleur4", "1F468-1F3FE-200D-1F3A4"},
    {"mannelijke_zanger_kleur5", "1F468-1F3FF-200D-1F3A4"},
    {"mannelijke_kunstenaar", "1F468-200D-1F3A8"},
    {"mannelijke_kunstenaar_kleur1", "1F468-1F3FB-200D-1F3A8"},
    {"mannelijke_kunstenaar_kleur2", "1F468-1F3FC-200D-1F3A8"},
    {"mannelijke_kunstenaar_kleur3", "1F468-1F3FD-200D-1F3A8"},
    {"mannelijke_kunstenaar_kleur4", "1F468-1F3FE-200D-1F3A8"},
    {"mannelijke_kunstenaar_kleur5", "1F468-1F3FF-200D-1F3A8"},
    {"mannelijke_docent", "1F468-200D-1F3EB"},
    {"mannelijke_docent_kleur1", "1F468-1F3FB-200D-1F3EB"},
    {"mannelijke_docent_kleur2", "1F468-1F3FC-200D-1F3EB"},
    {"mannelijke_docent_kleur3", "1F468-1F3FD-200D-1F3EB"},
    {"mannelijke_docent_kleur4", "1F468-1F3FE-200D-1F3EB"},
    {"mannelijke_docent_kleur5", "1F468-1F3FF-200D-1F3EB"},
    {"mannelijke_fabrieksarbeider", "1F468-200D-1F3ED"},
    {"mannelijke_fabrieksarbeider_kleur1", "1F468-1F3FB-200D-1F3ED"},
    {"mannelijke_fabrieksarbeider_kleur2", "1F468-1F3FC-200D-1F3ED"},
    {"mannelijke_fabrieksarbeider_kleur3", "1F468-1F3FD-200D-1F3ED"},
    {"mannelijke_fabrieksarbeider_kleur4", "1F468-1F3FE-200D-1F3ED"},
    {"mannelijke_fabrieksarbeider_kleur5", "1F468-1F3FF-200D-1F3ED"},
    {"mannelijke_technoloog", "1F468-200D-1F4BB"},
    {"mannelijke_technoloog_kleur1", "1F468-1F3FB-200D-1F4BB"},
    {"mannelijke_technoloog_kleur2", "1F468-1F3FC-200D-1F4BB"},
    {"mannelijke_technoloog_kleur3", "1F468-1F3FD-200D-1F4BB"},
    {"mannelijke_technoloog_kleur4", "1F468-1F3FE-200D-1F4BB"},
    {"mannelijke_technoloog_kleur5", "1F468-1F3FF-200D-1F4BB"},
    {"mannelijke_kantoormedewerker", "1F468-200D-1F4BC"},
    {"mannelijke_kantoormedewerker_kleur1", "1F468-1F3FB-200D-1F4BC"},
    {"mannelijke_kantoormedewerker_kleur2", "1F468-1F3FC-200D-1F4BC"},
    {"mannelijke_kantoormedewerker_kleur3", "1F468-1F3FD-200D-1F4BC"},
    {"mannelijke_kantoormedewerker_kleur4", "1F468-1F3FE-200D-1F4BC"},
    {"mannelijke_kantoormedewerker_kleur5", "1F468-1F3FF-200D-1F4BC"},
    {"mannelijke_monteur", "1F468-200D-1F527"},
    {"mannelijke_monteur_kleur1", "1F468-1F3FB-200D-1F527"},
    {"mannelijke_monteur_kleur2", "1F468-1F3FC-200D-1F527"},
    {"mannelijke_monteur_kleur3", "1F468-1F3FD-200D-1F527"},
    {"mannelijke_monteur_kleur4", "1F468-1F3FE-200D-1F527"},
    {"mannelijke_monteur_kleur5", "1F468-1F3FF-200D-1F527"},
    {"mannelijke_wetenschapper", "1F468-200D-1F52C"},
    {"mannelijke_wetenschapper_kleur1", "1F468-1F3FB-200D-1F52C"},
    {"mannelijke_wetenschapper_kleur2", "1F468-1F3FC-200D-1F52C"},
    {"mannelijke_wetenschapper_kleur3", "1F468-1F3FD-200D-1F52C"},
    {"mannelijke_wetenschapper_kleur4", "1F468-1F3FE-200D-1F52C"},
    {"mannelijke_wetenschapper_kleur5", "1F468-1F3FF-200D-1F52C"},
    {"mannelijke_astronaut", "1F468-200D-1F680"},
    {"mannelijke_astronaut_kleur1", "1F468-1F3FB-200D-1F680"},
    {"mannelijke_astronaut_kleur2", "1F468-1F3FC-200D-1F680"},
    {"mannelijke_astronaut_kleur3", "1F468-1F3FD-200D-1F680"},
    {"mannelijke_astronaut_kleur4", "1F468-1F3FE-200D-1F680"},
    {"mannelijke_astronaut_kleur5", "1F468-1F3FF-200D-1F680"},
    {"mannelijke_brandweerman", "1F468-200D-1F692"},
    {"mannelijke_brandweerman_kleur1", "1F468-1F3FB-200D-1F692"},
    {"mannelijke_brandweerman_kleur2", "1F468-1F3FC-200D-1F692"},
    {"mannelijke_brandweerman_kleur3", "1F468-1F3FD-200D-1F692"},
    {"mannelijke_brandweerman_kleur4", "1F468-1F3FE-200D-1F692"},
    {"mannelijke_brandweerman_kleur5", "1F468-1F3FF-200D-1F692"},
    {"man_met_taststok", "1F468-200D-1F9AF"},
    {"man_met_taststok_kleur1", "1F468-1F3FB-200D-1F9AF"},
    {"man_met_taststok_kleur2", "1F468-1F3FC-200D-1F9AF"},
    {"man_met_taststok_kleur3", "1F468-1F3FD-200D-1F9AF"},
    {"man_met_taststok_kleur4", "1F468-1F3FE-200D-1F9AF"},
    {"man_met_taststok_kleur5", "1F468-1F3FF-200D-1F9AF"},
    {"man_met_taststok_naar_rechts_gericht", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"man_met_taststok_naar_rechts_gericht_kleur1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"man_met_taststok_naar_rechts_gericht_kleur2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"man_met_taststok_naar_rechts_gericht_kleur3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"man_met_taststok_naar_rechts_gericht_kleur4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"man_met_taststok_naar_rechts_gericht_kleur5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"man_in_elektrische_rolstoel", "1F468-200D-1F9BC"},
    {"man_in_elektrische_rolstoel_kleur1", "1F468-1F3FB-200D-1F9BC"},
    {"man_in_elektrische_rolstoel_kleur2", "1F468-1F3FC-200D-1F9BC"},
    {"man_in_elektrische_rolstoel_kleur3", "1F468-1F3FD-200D-1F9BC"},
    {"man_in_elektrische_rolstoel_kleur4", "1F468-1F3FE-200D-1F9BC"},
    {"man_in_elektrische_rolstoel_kleur5", "1F468-1F3FF-200D-1F9BC"},
    {"man_in_elektrische_rolstoel_naar_rechts_gericht", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"man_in_elektrische_rolstoel_naar_rechts_gericht_kleur1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"man_in_elektrische_rolstoel_naar_rechts_gericht_kleur2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"man_in_elektrische_rolstoel_naar_rechts_gericht_kleur3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"man_in_elektrische_rolstoel_naar_rechts_gericht_kleur4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"man_in_elektrische_rolstoel_naar_rechts_gericht_kleur5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"man_in_rolstoel", "1F468-200D-1F9BD"},
    {"man_in_rolstoel_kleur1", "1F468-1F3FB-200D-1F9BD"},
    {"man_in_rolstoel_kleur2", "1F468-1F3FC-200D-1F9BD"},
    {"man_in_rolstoel_kleur3", "1F468-1F3FD-200D-1F9BD"},
    {"man_in_rolstoel_kleur4", "1F468-1F3FE-200D-1F9BD"},
    {"man_in_rolstoel_kleur5", "1F468-1F3FF-200D-1F9BD"},
    {"man_in_rolstoel_naar_rechts_gericht", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"man_in_rolstoel_naar_rechts_gericht_kleur1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"man_in_rolstoel_naar_rechts_gericht_kleur2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"man_in_rolstoel_naar_rechts_gericht_kleur3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"man_in_rolstoel_naar_rechts_gericht_kleur4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"man_in_rolstoel_naar_rechts_gericht_kleur5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"gezondheidsmedewerkster", "1F469-200D-2695-FE0F"},
    {"gezondheidsmedewerkster_kleur1", "1F469-1F3FB-200D-2695-FE0F"},
    {"gezondheidsmedewerkster_kleur2", "1F469-1F3FC-200D-2695-FE0F"},
    {"gezondheidsmedewerkster_kleur3", "1F469-1F3FD-200D-2695-FE0F"},
    {"gezondheidsmedewerkster_kleur4", "1F469-1F3FE-200D-2695-FE0F"},
    {"gezondheidsmedewerkster_kleur5", "1F469-1F3FF-200D-2695-FE0F"},
    {"vrouwelijke_rechter", "1F469-200D-2696-FE0F"},
    {"vrouwelijke_rechter_kleur1", "1F469-1F3FB-200D-2696-FE0F"},
    {"vrouwelijke_rechter_kleur2", "1F469-1F3FC-200D-2696-FE0F"},
    {"vrouwelijke_rechter_kleur3", "1F469-1F3FD-200D-2696-FE0F"},
    {"vrouwelijke_rechter_kleur4", "1F469-1F3FE-200D-2696-FE0F"},
    {"vrouwelijke_rechter_kleur5", "1F469-1F3FF-200D-2696-FE0F"},
    {"pilote", "1F469-200D-2708-FE0F"},
    {"pilote_kleur1", "1F469-1F3FB-200D-2708-FE0F"},
    {"pilote_kleur2", "1F469-1F3FC-200D-2708-FE0F"},
    {"pilote_kleur3", "1F469-1F3FD-200D-2708-FE0F"},
    {"pilote_kleur4", "1F469-1F3FE-200D-2708-FE0F"},
    {"pilote_kleur5", "1F469-1F3FF-200D-2708-FE0F"},
    {"boerin", "1F469-200D-1F33E"},
    {"boerin_kleur1", "1F469-1F3FB-200D-1F33E"},
    {"boerin_kleur2", "1F469-1F3FC-200D-1F33E"},
    {"boerin_kleur3", "1F469-1F3FD-200D-1F33E"},
    {"boerin_kleur4", "1F469-1F3FE-200D-1F33E"},
    {"boerin_kleur5", "1F469-1F3FF-200D-1F33E"},
    {"kokkin", "1F469-200D-1F373"},
    {"kokkin_kleur1", "1F469-1F3FB-200D-1F373"},
    {"kokkin_kleur2", "1F469-1F3FC-200D-1F373"},
    {"kokkin_kleur3", "1F469-1F3FD-200D-1F373"},
    {"kokkin_kleur4", "1F469-1F3FE-200D-1F373"},
    {"kokkin_kleur5", "1F469-1F3FF-200D-1F373"},
    {"vrouw_die_een_baby_voedt", "1F469-200D-1F37C"},
    {"vrouw_die_een_baby_voedt_kleur1", "1F469-1F3FB-200D-1F37C"},
    {"vrouw_die_een_baby_voedt_kleur2", "1F469-1F3FC-200D-1F37C"},
    {"vrouw_die_een_baby_voedt_kleur3", "1F469-1F3FD-200D-1F37C"},
    {"vrouw_die_een_baby_voedt_kleur4", "1F469-1F3FE-200D-1F37C"},
    {"vrouw_die_een_baby_voedt_kleur5", "1F469-1F3FF-200D-1F37C"},
    {"studente", "1F469-200D-1F393"},
    {"studente_kleur1", "1F469-1F3FB-200D-1F393"},
    {"studente_kleur2", "1F469-1F3FC-200D-1F393"},
    {"studente_kleur3", "1F469-1F3FD-200D-1F393"},
    {"studente_kleur4", "1F469-1F3FE-200D-1F393"},
    {"studente_kleur5", "1F469-1F3FF-200D-1F393"},
    {"zangeres", "1F469-200D-1F3A4"},
    {"zangeres_kleur1", "1F469-1F3FB-200D-1F3A4"},
    {"zangeres_kleur2", "1F469-1F3FC-200D-1F3A4"},
    {"zangeres_kleur3", "1F469-1F3FD-200D-1F3A4"},
    {"zangeres_kleur4", "1F469-1F3FE-200D-1F3A4"},
    {"zangeres_kleur5", "1F469-1F3FF-200D-1F3A4"},
    {"kunstenares", "1F469-200D-1F3A8"},
    {"kunstenares_kleur1", "1F469-1F3FB-200D-1F3A8"},
    {"kunstenares_kleur2", "1F469-1F3FC-200D-1F3A8"},
    {"kunstenares_kleur3", "1F469-1F3FD-200D-1F3A8"},
    {"kunstenares_kleur4", "1F469-1F3FE-200D-1F3A8"},
    {"kunstenares_kleur5", "1F469-1F3FF-200D-1F3A8"},
    {"docente", "1F469-200D-1F3EB"},
    {"docente_kleur1", "1F469-1F3FB-200D-1F3EB"},
    {"docente_kleur2", "1F469-1F3FC-200D-1F3EB"},
    {"docente_kleur3", "1F469-1F3FD-200D-1F3EB"},
    {"docente_kleur4", "1F469-1F3FE-200D-1F3EB"},
    {"docente_kleur5", "1F469-1F3FF-200D-1F3EB"},
    {"fabrieksarbeidster", "1F469-200D-1F3ED"},
    {"fabrieksarbeidster_kleur1", "1F469-1F3FB-200D-1F3ED"},
    {"fabrieksarbeidster_kleur2", "1F469-1F3FC-200D-1F3ED"},
    {"fabrieksarbeidster_kleur3", "1F469-1F3FD-200D-1F3ED"},
    {"fabrieksarbeidster_kleur4", "1F469-1F3FE-200D-1F3ED"},
    {"fabrieksarbeidster_kleur5", "1F469-1F3FF-200D-1F3ED"},
    {"technologe", "1F469-200D-1F4BB"},
    {"technologe_kleur1", "1F469-1F3FB-200D-1F4BB"},
    {"technologe_kleur2", "1F469-1F3FC-200D-1F4BB"},
    {"technologe_kleur3", "1F469-1F3FD-200D-1F4BB"},
    {"technologe_kleur4", "1F469-1F3FE-200D-1F4BB"},
    {"technologe_kleur5", "1F469-1F3FF-200D-1F4BB"},
    {"kantoormedewerkster", "1F469-200D-1F4BC"},
    {"kantoormedewerkster_kleur1", "1F469-1F3FB-200D-1F4BC"},
    {"kantoormedewerkster_kleur2", "1F469-1F3FC-200D-1F4BC"},
    {"kantoormedewerkster_kleur3", "1F469-1F3FD-200D-1F4BC"},
    {"kantoormedewerkster_kleur4", "1F469-1F3FE-200D-1F4BC"},
    {"kantoormedewerkster_kleur5", "1F469-1F3FF-200D-1F4BC"},
    {"monteuse", "1F469-200D-1F527"},
    {"monteuse_kleur1", "1F469-1F3FB-200D-1F527"},
    {"monteuse_kleur2", "1F469-1F3FC-200D-1F527"},
    {"monteuse_kleur3", "1F469-1F3FD-200D-1F527"},
    {"monteuse_kleur4", "1F469-1F3FE-200D-1F527"},
    {"monteuse_kleur5", "1F469-1F3FF-200D-1F527"},
    {"wetenschapster", "1F469-200D-1F52C"},
    {"wetenschapster_kleur1", "1F469-1F3FB-200D-1F52C"},
    {"wetenschapster_kleur2", "1F469-1F3FC-200D-1F52C"},
    {"wetenschapster_kleur3", "1F469-1F3FD-200D-1F52C"},
    {"wetenschapster_kleur4", "1F469-1F3FE-200D-1F52C"},
    {"wetenschapster_kleur5", "1F469-1F3FF-200D-1F52C"},
    {"astronaute", "1F469-200D-1F680"},
    {"astronaute_kleur1", "1F469-1F3FB-200D-1F680"},
    {"astronaute_kleur2", "1F469-1F3FC-200D-1F680"},
    {"astronaute_kleur3", "1F469-1F3FD-200D-1F680"},
    {"astronaute_kleur4", "1F469-1F3FE-200D-1F680"},
    {"astronaute_kleur5", "1F469-1F3FF-200D-1F680"},
    {"brandweervrouw", "1F469-200D-1F692"},
    {"brandweervrouw_kleur1", "1F469-1F3FB-200D-1F692"},
    {"brandweervrouw_kleur2", "1F469-1F3FC-200D-1F692"},
    {"brandweervrouw_kleur3", "1F469-1F3FD-200D-1F692"},
    {"brandweervrouw_kleur4", "1F469-1F3FE-200D-1F692"},
    {"brandweervrouw_kleur5", "1F469-1F3FF-200D-1F692"},
    {"vrouw_met_taststok", "1F469-200D-1F9AF"},
    {"vrouw_met_taststok_kleur1", "1F469-1F3FB-200D-1F9AF"},
    {"vrouw_met_taststok_kleur2", "1F469-1F3FC-200D-1F9AF"},
    {"vrouw_met_taststok_kleur3", "1F469-1F3FD-200D-1F9AF"},
    {"vrouw_met_taststok_kleur4", "1F469-1F3FE-200D-1F9AF"},
    {"vrouw_met_taststok_kleur5", "1F469-1F3FF-200D-1F9AF"},
    {"vrouw_met_taststok_naar_rechts_gericht", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"vrouw_met_taststok_naar_rechts_gericht_kleur1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"vrouw_met_taststok_naar_rechts_gericht_kleur2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"vrouw_met_taststok_naar_rechts_gericht_kleur3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"vrouw_met_taststok_naar_rechts_gericht_kleur4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"vrouw_met_taststok_naar_rechts_gericht_kleur5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"vrouw_in_elektrische_rolstoel", "1F469-200D-1F9BC"},
    {"vrouw_in_elektrische_rolstoel_kleur1", "1F469-1F3FB-200D-1F9BC"},
    {"vrouw_in_elektrische_rolstoel_kleur2", "1F469-1F3FC-200D-1F9BC"},
    {"vrouw_in_elektrische_rolstoel_kleur3", "1F469-1F3FD-200D-1F9BC"},
    {"vrouw_in_elektrische_rolstoel_kleur4", "1F469-1F3FE-200D-1F9BC"},
    {"vrouw_in_elektrische_rolstoel_kleur5", "1F469-1F3FF-200D-1F9BC"},
    {"vrouw_in_elektrische_rolstoel_naar_rechts_gericht", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"vrouw_in_elektrische_rolstoel_naar_rechts_gericht_kleur1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"vrouw_in_elektrische_rolstoel_naar_rechts_gericht_kleur2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"vrouw_in_elektrische_rolstoel_naar_rechts_gericht_kleur3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"vrouw_in_elektrische_rolstoel_naar_rechts_gericht_kleur4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"vrouw_in_elektrische_rolstoel_naar_rechts_gericht_kleur5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"vrouw_in_rolstoel", "1F469-200D-1F9BD"},
    {"vrouw_in_rolstoel_kleur1", "1F469-1F3FB-200D-1F9BD"},
    {"vrouw_in_rolstoel_kleur2", "1F469-1F3FC-200D-1F9BD"},
    {"vrouw_in_rolstoel_kleur3", "1F469-1F3FD-200D-1F9BD"},
    {"vrouw_in_rolstoel_kleur4", "1F469-1F3FE-200D-1F9BD"},
    {"vrouw_in_rolstoel_kleur5", "1F469-1F3FF-200D-1F9BD"},
    {"vrouw_in_rolstoel_naar_rechts_gericht", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"vrouw_in_rolstoel_naar_rechts_gericht_kleur1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"vrouw_in_rolstoel_naar_rechts_gericht_kleur2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"vrouw_in_rolstoel_naar_rechts_gericht_kleur3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"vrouw_in_rolstoel_naar_rechts_gericht_kleur4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"vrouw_in_rolstoel_naar_rechts_gericht_kleur5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"voetganger_naar_rechts_gericht", "1F6B6-200D-27A1-FE0F"},
    {"voetganger_naar_rechts_gericht_kleur1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"voetganger_naar_rechts_gericht_kleur2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"voetganger_naar_rechts_gericht_kleur3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"voetganger_naar_rechts_gericht_kleur4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"voetganger_naar_rechts_gericht_kleur5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"knielende_persoon_naar_rechts_gericht", "1F9CE-200D-27A1-FE0F"},
    {"knielende_persoon_naar_rechts_gericht_kleur1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"knielende_persoon_naar_rechts_gericht_kleur2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"knielende_persoon_naar_rechts_gericht_kleur3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"knielende_persoon_naar_rechts_gericht_kleur4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"knielende_persoon_naar_rechts_gericht_kleur5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"gezondheidsmedewerker", "1F9D1-200D-2695-FE0F"},
    {"gezondheidsmedewerker_kleur1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"gezondheidsmedewerker_kleur2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"gezondheidsmedewerker_kleur3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"gezondheidsmedewerker_kleur4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"gezondheidsmedewerker_kleur5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"rechter", "1F9D1-200D-2696-FE0F"},
    {"rechter_kleur1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"rechter_kleur2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"rechter_kleur3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"rechter_kleur4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"rechter_kleur5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"piloot", "1F9D1-200D-2708-FE0F"},
    {"piloot_kleur1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"piloot_kleur2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"piloot_kleur3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"piloot_kleur4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"piloot_kleur5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"boer", "1F9D1-200D-1F33E"},
    {"boer_kleur1", "1F9D1-1F3FB-200D-1F33E"},
    {"boer_kleur2", "1F9D1-1F3FC-200D-1F33E"},
    {"boer_kleur3", "1F9D1-1F3FD-200D-1F33E"},
    {"boer_kleur4", "1F9D1-1F3FE-200D-1F33E"},
    {"boer_kleur5", "1F9D1-1F3FF-200D-1F33E"},
    {"kok", "1F9D1-200D-1F373"},
    {"kok_kleur1", "1F9D1-1F3FB-200D-1F373"},
    {"kok_kleur2", "1F9D1-1F3FC-200D-1F373"},
    {"kok_kleur3", "1F9D1-1F3FD-200D-1F373"},
    {"kok_kleur4", "1F9D1-1F3FE-200D-1F373"},
    {"kok_kleur5", "1F9D1-1F3FF-200D-1F373"},
    {"persoon_die_een_baby_voedt", "1F9D1-200D-1F37C"},
    {"persoon_die_een_baby_voedt_kleur1", "1F9D1-1F3FB-200D-1F37C"},
    {"persoon_die_een_baby_voedt_kleur2", "1F9D1-1F3FC-200D-1F37C"},
    {"persoon_die_een_baby_voedt_kleur3", "1F9D1-1F3FD-200D-1F37C"},
    {"persoon_die_een_baby_voedt_kleur4", "1F9D1-1F3FE-200D-1F37C"},
    {"persoon_die_een_baby_voedt_kleur5", "1F9D1-1F3FF-200D-1F37C"},
    {"kersttransgender", "1F9D1-200D-1F384"},
    {"kersttransgender_kleur1", "1F9D1-1F3FB-200D-1F384"},
    {"kersttransgender_kleur2", "1F9D1-1F3FC-200D-1F384"},
    {"kersttransgender_kleur3", "1F9D1-1F3FD-200D-1F384"},
    {"kersttransgender_kleur4", "1F9D1-1F3FE-200D-1F384"},
    {"kersttransgender_kleur5", "1F9D1-1F3FF-200D-1F384"},
    {"student", "1F9D1-200D-1F393"},
    {"student_kleur1", "1F9D1-1F3FB-200D-1F393"},
    {"student_kleur2", "1F9D1-1F3FC-200D-1F393"},
    {"student_kleur3", "1F9D1-1F3FD-200D-1F393"},
    {"student_kleur4", "1F9D1-1F3FE-200D-1F393"},
    {"student_kleur5", "1F9D1-1F3FF-200D-1F393"},
    {"zanger", "1F9D1-200D-1F3A4"},
    {"zanger_kleur1", "1F9D1-1F3FB-200D-1F3A4"},
    {"zanger_kleur2", "1F9D1-1F3FC-200D-1F3A4"},
    {"zanger_kleur3", "1F9D1-1F3FD-200D-1F3A4"},
    {"zanger_kleur4", "1F9D1-1F3FE-200D-1F3A4"},
    {"zanger_kleur5", "1F9D1-1F3FF-200D-1F3A4"},
    {"kunstenaar", "1F9D1-200D-1F3A8"},
    {"kunstenaar_kleur1", "1F9D1-1F3FB-200D-1F3A8"},
    {"kunstenaar_kleur2", "1F9D1-1F3FC-200D-1F3A8"},
    {"kunstenaar_kleur3", "1F9D1-1F3FD-200D-1F3A8"},
    {"kunstenaar_kleur4", "1F9D1-1F3FE-200D-1F3A8"},
    {"kunstenaar_kleur5", "1F9D1-1F3FF-200D-1F3A8"},
    {"docent", "1F9D1-200D-1F3EB"},
    {"docent_kleur1", "1F9D1-1F3FB-200D-1F3EB"},
    {"docent_kleur2", "1F9D1-1F3FC-200D-1F3EB"},
    {"docent_kleur3", "1F9D1-1F3FD-200D-1F3EB"},
    {"docent_kleur4", "1F9D1-1F3FE-200D-1F3EB"},
    {"docent_kleur5", "1F9D1-1F3FF-200D-1F3EB"},
    {"fabrieksarbeider", "1F9D1-200D-1F3ED"},
    {"fabrieksarbeider_kleur1", "1F9D1-1F3FB-200D-1F3ED"},
    {"fabrieksarbeider_kleur2", "1F9D1-1F3FC-200D-1F3ED"},
    {"fabrieksarbeider_kleur3", "1F9D1-1F3FD-200D-1F3ED"},
    {"fabrieksarbeider_kleur4", "1F9D1-1F3FE-200D-1F3ED"},
    {"fabrieksarbeider_kleur5", "1F9D1-1F3FF-200D-1F3ED"},
    {"technoloog", "1F9D1-200D-1F4BB"},
    {"technoloog_kleur1", "1F9D1-1F3FB-200D-1F4BB"},
    {"technoloog_kleur2", "1F9D1-1F3FC-200D-1F4BB"},
    {"technoloog_kleur3", "1F9D1-1F3FD-200D-1F4BB"},
    {"technoloog_kleur4", "1F9D1-1F3FE-200D-1F4BB"},
    {"technoloog_kleur5", "1F9D1-1F3FF-200D-1F4BB"},
    {"kantoormedewerker", "1F9D1-200D-1F4BC"},
    {"kantoormedewerker_kleur1", "1F9D1-1F3FB-200D-1F4BC"},
    {"kantoormedewerker_kleur2", "1F9D1-1F3FC-200D-1F4BC"},
    {"kantoormedewerker_kleur3", "1F9D1-1F3FD-200D-1F4BC"},
    {"kantoormedewerker_kleur4", "1F9D1-1F3FE-200D-1F4BC"},
    {"kantoormedewerker_kleur5", "1F9D1-1F3FF-200D-1F4BC"},
    {"monteur", "1F9D1-200D-1F527"},
    {"monteur_kleur1", "1F9D1-1F3FB-200D-1F527"},
    {"monteur_kleur2", "1F9D1-1F3FC-200D-1F527"},
    {"monteur_kleur3", "1F9D1-1F3FD-200D-1F527"},
    {"monteur_kleur4", "1F9D1-1F3FE-200D-1F527"},
    {"monteur_kleur5", "1F9D1-1F3FF-200D-1F527"},
    {"wetenschapper", "1F9D1-200D-1F52C"},
    {"wetenschapper_kleur1", "1F9D1-1F3FB-200D-1F52C"},
    {"wetenschapper_kleur2", "1F9D1-1F3FC-200D-1F52C"},
    {"wetenschapper_kleur3", "1F9D1-1F3FD-200D-1F52C"},
    {"wetenschapper_kleur4", "1F9D1-1F3FE-200D-1F52C"},
    {"wetenschapper_kleur5", "1F9D1-1F3FF-200D-1F52C"},
    {"astronaut", "1F9D1-200D-1F680"},
    {"astronaut_kleur1", "1F9D1-1F3FB-200D-1F680"},
    {"astronaut_kleur2", "1F9D1-1F3FC-200D-1F680"},
    {"astronaut_kleur3", "1F9D1-1F3FD-200D-1F680"},
    {"astronaut_kleur4", "1F9D1-1F3FE-200D-1F680"},
    {"astronaut_kleur5", "1F9D1-1F3FF-200D-1F680"},
    {"brandweerman", "1F9D1-200D-1F692"},
    {"brandweerman_kleur1", "1F9D1-1F3FB-200D-1F692"},
    {"brandweerman_kleur2", "1F9D1-1F3FC-200D-1F692"},
    {"brandweerman_kleur3", "1F9D1-1F3FD-200D-1F692"},
    {"brandweerman_kleur4", "1F9D1-1F3FE-200D-1F692"},
    {"brandweerman_kleur5", "1F9D1-1F3FF-200D-1F692"},
    {"persoon_met_taststok", "1F9D1-200D-1F9AF"},
    {"persoon_met_taststok_kleur1", "1F9D1-1F3FB-200D-1F9AF"},
    {"persoon_met_taststok_kleur2", "1F9D1-1F3FC-200D-1F9AF"},
    {"persoon_met_taststok_kleur3", "1F9D1-1F3FD-200D-1F9AF"},
    {"persoon_met_taststok_kleur4", "1F9D1-1F3FE-200D-1F9AF"},
    {"persoon_met_taststok_kleur5", "1F9D1-1F3FF-200D-1F9AF"},
    {"persoon_met_taststok_naar_rechts_gericht", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"persoon_met_taststok_naar_rechts_gericht_kleur1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"persoon_met_taststok_naar_rechts_gericht_kleur2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"persoon_met_taststok_naar_rechts_gericht_kleur3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"persoon_met_taststok_naar_rechts_gericht_kleur4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"persoon_met_taststok_naar_rechts_gericht_kleur5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"persoon_in_elektrische_rolstoel", "1F9D1-200D-1F9BC"},
    {"persoon_in_elektrische_rolstoel_kleur1", "1F9D1-1F3FB-200D-1F9BC"},
    {"persoon_in_elektrische_rolstoel_kleur2", "1F9D1-1F3FC-200D-1F9BC"},
    {"persoon_in_elektrische_rolstoel_kleur3", "1F9D1-1F3FD-200D-1F9BC"},
    {"persoon_in_elektrische_rolstoel_kleur4", "1F9D1-1F3FE-200D-1F9BC"},
    {"persoon_in_elektrische_rolstoel_kleur5", "1F9D1-1F3FF-200D-1F9BC"},
    {"persoon_in_elektrische_rolstoel_naar_rechts_gericht", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"persoon_in_elektrische_rolstoel_naar_rechts_gericht_kleur1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"persoon_in_elektrische_rolstoel_naar_rechts_gericht_kleur2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"persoon_in_elektrische_rolstoel_naar_rechts_gericht_kleur3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"persoon_in_elektrische_rolstoel_naar_rechts_gericht_kleur4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"persoon_in_elektrische_rolstoel_naar_rechts_gericht_kleur5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"persoon_in_rolstoel", "1F9D1-200D-1F9BD"},
    {"persoon_in_rolstoel_kleur1", "1F9D1-1F3FB-200D-1F9BD"},
    {"persoon_in_rolstoel_kleur2", "1F9D1-1F3FC-200D-1F9BD"},
    {"persoon_in_rolstoel_kleur3", "1F9D1-1F3FD-200D-1F9BD"},
    {"persoon_in_rolstoel_kleur4", "1F9D1-1F3FE-200D-1F9BD"},
    {"persoon_in_rolstoel_kleur5", "1F9D1-1F3FF-200D-1F9BD"},
    {"persoon_in_rolstoel_naar_rechts_gericht", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"persoon_in_rolstoel_naar_rechts_gericht_kleur1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"persoon_in_rolstoel_naar_rechts_gericht_kleur2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"persoon_in_rolstoel_naar_rechts_gericht_kleur3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"persoon_in_rolstoel_naar_rechts_gericht_kleur4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"persoon_in_rolstoel_naar_rechts_gericht_kleur5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"basketbalster", "26F9-FE0F-200D-2640-FE0F"},
    {"basketbalster_kleur1", "26F9-1F3FB-200D-2640-FE0F"},
    {"basketbalster_kleur2", "26F9-1F3FC-200D-2640-FE0F"},
    {"basketbalster_kleur3", "26F9-1F3FD-200D-2640-FE0F"},
    {"basketbalster_kleur4", "26F9-1F3FE-200D-2640-FE0F"},
    {"basketbalster_kleur5", "26F9-1F3FF-200D-2640-FE0F"},
    {"basketballer", "26F9-FE0F-200D-2642-FE0F"},
    {"basketballer_kleur1", "26F9-1F3FB-200D-2642-FE0F"},
    {"basketballer_kleur2", "26F9-1F3FC-200D-2642-FE0F"},
    {"basketballer_kleur3", "26F9-1F3FD-200D-2642-FE0F"},
    {"basketballer_kleur4", "26F9-1F3FE-200D-2642-FE0F"},
    {"basketballer_kleur5", "26F9-1F3FF-200D-2642-FE0F"},
    {"rennende_vrouw", "1F3C3-200D-2640-FE0F"},
    {"rennende_vrouw_kleur1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"rennende_vrouw_kleur2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"rennende_vrouw_kleur3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"rennende_vrouw_kleur4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"rennende_vrouw_kleur5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"rennende_vrouw_naar_rechts_gericht", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"rennende_vrouw_naar_rechts_gericht_kleur1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"rennende_vrouw_naar_rechts_gericht_kleur2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"rennende_vrouw_naar_rechts_gericht_kleur3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"rennende_vrouw_naar_rechts_gericht_kleur4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"rennende_vrouw_naar_rechts_gericht_kleur5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"rennende_man", "1F3C3-200D-2642-FE0F"},
    {"rennende_man_kleur1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"rennende_man_kleur2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"rennende_man_kleur3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"rennende_man_kleur4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"rennende_man_kleur5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"rennende_man_naar_rechts_gericht", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"rennende_man_naar_rechts_gericht_kleur1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"rennende_man_naar_rechts_gericht_kleur2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"rennende_man_naar_rechts_gericht_kleur3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"rennende_man_naar_rechts_gericht_kleur4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"rennende_man_naar_rechts_gericht_kleur5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"vrouwelijke_surfer", "1F3C4-200D-2640-FE0F"},
    {"vrouwelijke_surfer_kleur1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"vrouwelijke_surfer_kleur2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"vrouwelijke_surfer_kleur3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"vrouwelijke_surfer_kleur4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"vrouwelijke_surfer_kleur5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"mannelijke_surfer", "1F3C4-200D-2642-FE0F"},
    {"mannelijke_surfer_kleur1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"mannelijke_surfer_kleur2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"mannelijke_surfer_kleur3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"mannelijke_surfer_kleur4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"mannelijke_surfer_kleur5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"zwemmende_vrouw", "1F3CA-200D-2640-FE0F"},
    {"zwemmende_vrouw_kleur1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"zwemmende_vrouw_kleur2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"zwemmende_vrouw_kleur3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"zwemmende_vrouw_kleur4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"zwemmende_vrouw_kleur5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"zwemmende_man", "1F3CA-200D-2642-FE0F"},
    {"zwemmende_man_kleur1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"zwemmende_man_kleur2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"zwemmende_man_kleur3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"zwemmende_man_kleur4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"zwemmende_man_kleur5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"vrouwelijke_gewichtheffer", "1F3CB-FE0F-200D-2640-FE0F"},
    {"vrouwelijke_gewichtheffer_kleur1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"vrouwelijke_gewichtheffer_kleur2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"vrouwelijke_gewichtheffer_kleur3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"vrouwelijke_gewichtheffer_kleur4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"vrouwelijke_gewichtheffer_kleur5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"mannelijke_gewichtheffer", "1F3CB-FE0F-200D-2642-FE0F"},
    {"mannelijke_gewichtheffer_kleur1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"mannelijke_gewichtheffer_kleur2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"mannelijke_gewichtheffer_kleur3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"mannelijke_gewichtheffer_kleur4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"mannelijke_gewichtheffer_kleur5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"golfende_vrouw", "1F3CC-FE0F-200D-2640-FE0F"},
    {"golfende_vrouw_kleur1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"golfende_vrouw_kleur2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"golfende_vrouw_kleur3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"golfende_vrouw_kleur4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"golfende_vrouw_kleur5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"golfende_man", "1F3CC-FE0F-200D-2642-FE0F"},
    {"golfende_man_kleur1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"golfende_man_kleur2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"golfende_man_kleur3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"golfende_man_kleur4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"golfende_man_kleur5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"politievrouw", "1F46E-200D-2640-FE0F"},
    {"politievrouw_kleur1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"politievrouw_kleur2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"politievrouw_kleur3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"politievrouw_kleur4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"politievrouw_kleur5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"politieman", "1F46E-200D-2642-FE0F"},
    {"politieman_kleur1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"politieman_kleur2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"politieman_kleur3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"politieman_kleur4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"politieman_kleur5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"vrouwen_met_konijnenoren", "1F46F-200D-2640-FE0F"},
    {"vrouwen_met_konijnenoren_kleur1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"vrouwen_met_konijnenoren_kleur1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"vrouwen_met_konijnenoren_kleur1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"vrouwen_met_konijnenoren_kleur1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"vrouwen_met_konijnenoren_kleur2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"vrouwen_met_konijnenoren_kleur2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"vrouwen_met_konijnenoren_kleur2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"vrouwen_met_konijnenoren_kleur2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"vrouwen_met_konijnenoren_kleur3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"vrouwen_met_konijnenoren_kleur3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"vrouwen_met_konijnenoren_kleur3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"vrouwen_met_konijnenoren_kleur3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"vrouwen_met_konijnenoren_kleur4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"vrouwen_met_konijnenoren_kleur4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"vrouwen_met_konijnenoren_kleur4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"vrouwen_met_konijnenoren_kleur4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"vrouwen_met_konijnenoren_kleur5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"vrouwen_met_konijnenoren_kleur5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"vrouwen_met_konijnenoren_kleur5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"vrouwen_met_konijnenoren_kleur5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"vrouwen_met_konijnenoren_kleur1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"vrouwen_met_konijnenoren_kleur2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"vrouwen_met_konijnenoren_kleur3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"vrouwen_met_konijnenoren_kleur4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"vrouwen_met_konijnenoren_kleur5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"mannen_met_konijnenoren", "1F46F-200D-2642-FE0F"},
    {"mannen_met_konijnenoren_kleur1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"mannen_met_konijnenoren_kleur1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"mannen_met_konijnenoren_kleur1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"mannen_met_konijnenoren_kleur1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"mannen_met_konijnenoren_kleur2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"mannen_met_konijnenoren_kleur2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"mannen_met_konijnenoren_kleur2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"mannen_met_konijnenoren_kleur2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"mannen_met_konijnenoren_kleur3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"mannen_met_konijnenoren_kleur3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"mannen_met_konijnenoren_kleur3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"mannen_met_konijnenoren_kleur3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"mannen_met_konijnenoren_kleur4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"mannen_met_konijnenoren_kleur4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"mannen_met_konijnenoren_kleur4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"mannen_met_konijnenoren_kleur4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"mannen_met_konijnenoren_kleur5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"mannen_met_konijnenoren_kleur5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"mannen_met_konijnenoren_kleur5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"mannen_met_konijnenoren_kleur5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"mannen_met_konijnenoren_kleur1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"mannen_met_konijnenoren_kleur2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"mannen_met_konijnenoren_kleur3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"mannen_met_konijnenoren_kleur4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"mannen_met_konijnenoren_kleur5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"vrouw_met_sluier", "1F470-200D-2640-FE0F"},
    {"vrouw_met_sluier_kleur1", "1F470-1F3FB-200D-2640-FE0F"},
    {"vrouw_met_sluier_kleur2", "1F470-1F3FC-200D-2640-FE0F"},
    {"vrouw_met_sluier_kleur3", "1F470-1F3FD-200D-2640-FE0F"},
    {"vrouw_met_sluier_kleur4", "1F470-1F3FE-200D-2640-FE0F"},
    {"vrouw_met_sluier_kleur5", "1F470-1F3FF-200D-2640-FE0F"},
    {"man_met_sluier", "1F470-200D-2642-FE0F"},
    {"man_met_sluier_kleur1", "1F470-1F3FB-200D-2642-FE0F"},
    {"man_met_sluier_kleur2", "1F470-1F3FC-200D-2642-FE0F"},
    {"man_met_sluier_kleur3", "1F470-1F3FD-200D-2642-FE0F"},
    {"man_met_sluier_kleur4", "1F470-1F3FE-200D-2642-FE0F"},
    {"man_met_sluier_kleur5", "1F470-1F3FF-200D-2642-FE0F"},
    {"vrouw_met_blond_haar", "1F471-200D-2640-FE0F"},
    {"vrouw_met_blond_haar_kleur1", "1F471-1F3FB-200D-2640-FE0F"},
    {"vrouw_met_blond_haar_kleur2", "1F471-1F3FC-200D-2640-FE0F"},
    {"vrouw_met_blond_haar_kleur3", "1F471-1F3FD-200D-2640-FE0F"},
    {"vrouw_met_blond_haar_kleur4", "1F471-1F3FE-200D-2640-FE0F"},
    {"vrouw_met_blond_haar_kleur5", "1F471-1F3FF-200D-2640-FE0F"},
    {"man_met_blond_haar", "1F471-200D-2642-FE0F"},
    {"man_met_blond_haar_kleur1", "1F471-1F3FB-200D-2642-FE0F"},
    {"man_met_blond_haar_kleur2", "1F471-1F3FC-200D-2642-FE0F"},
    {"man_met_blond_haar_kleur3", "1F471-1F3FD-200D-2642-FE0F"},
    {"man_met_blond_haar_kleur4", "1F471-1F3FE-200D-2642-FE0F"},
    {"man_met_blond_haar_kleur5", "1F471-1F3FF-200D-2642-FE0F"},
    {"vrouw_met_tulband", "1F473-200D-2640-FE0F"},
    {"vrouw_met_tulband_kleur1", "1F473-1F3FB-200D-2640-FE0F"},
    {"vrouw_met_tulband_kleur2", "1F473-1F3FC-200D-2640-FE0F"},
    {"vrouw_met_tulband_kleur3", "1F473-1F3FD-200D-2640-FE0F"},
    {"vrouw_met_tulband_kleur4", "1F473-1F3FE-200D-2640-FE0F"},
    {"vrouw_met_tulband_kleur5", "1F473-1F3FF-200D-2640-FE0F"},
    {"man_met_tulband", "1F473-200D-2642-FE0F"},
    {"man_met_tulband_kleur1", "1F473-1F3FB-200D-2642-FE0F"},
    {"man_met_tulband_kleur2", "1F473-1F3FC-200D-2642-FE0F"},
    {"man_met_tulband_kleur3", "1F473-1F3FD-200D-2642-FE0F"},
    {"man_met_tulband_kleur4", "1F473-1F3FE-200D-2642-FE0F"},
    {"man_met_tulband_kleur5", "1F473-1F3FF-200D-2642-FE0F"},
    {"vrouwelijke_bouwvakker", "1F477-200D-2640-FE0F"},
    {"vrouwelijke_bouwvakker_kleur1", "1F477-1F3FB-200D-2640-FE0F"},
    {"vrouwelijke_bouwvakker_kleur2", "1F477-1F3FC-200D-2640-FE0F"},
    {"vrouwelijke_bouwvakker_kleur3", "1F477-1F3FD-200D-2640-FE0F"},
    {"vrouwelijke_bouwvakker_kleur4", "1F477-1F3FE-200D-2640-FE0F"},
    {"vrouwelijke_bouwvakker_kleur5", "1F477-1F3FF-200D-2640-FE0F"},
    {"mannelijke_bouwvakker", "1F477-200D-2642-FE0F"},
    {"mannelijke_bouwvakker_kleur1", "1F477-1F3FB-200D-2642-FE0F"},
    {"mannelijke_bouwvakker_kleur2", "1F477-1F3FC-200D-2642-FE0F"},
    {"mannelijke_bouwvakker_kleur3", "1F477-1F3FD-200D-2642-FE0F"},
    {"mannelijke_bouwvakker_kleur4", "1F477-1F3FE-200D-2642-FE0F"},
    {"mannelijke_bouwvakker_kleur5", "1F477-1F3FF-200D-2642-FE0F"},
    {"vrouw_achter_informatiebalie", "1F481-200D-2640-FE0F"},
    {"vrouw_achter_informatiebalie_kleur1", "1F481-1F3FB-200D-2640-FE0F"},
    {"vrouw_achter_informatiebalie_kleur2", "1F481-1F3FC-200D-2640-FE0F"},
    {"vrouw_achter_informatiebalie_kleur3", "1F481-1F3FD-200D-2640-FE0F"},
    {"vrouw_achter_informatiebalie_kleur4", "1F481-1F3FE-200D-2640-FE0F"},
    {"vrouw_achter_informatiebalie_kleur5", "1F481-1F3FF-200D-2640-FE0F"},
    {"man_achter_informatiebalie", "1F481-200D-2642-FE0F"},
    {"man_achter_informatiebalie_kleur1", "1F481-1F3FB-200D-2642-FE0F"},
    {"man_achter_informatiebalie_kleur2", "1F481-1F3FC-200D-2642-FE0F"},
    {"man_achter_informatiebalie_kleur3", "1F481-1F3FD-200D-2642-FE0F"},
    {"man_achter_informatiebalie_kleur4", "1F481-1F3FE-200D-2642-FE0F"},
    {"man_achter_informatiebalie_kleur5", "1F481-1F3FF-200D-2642-FE0F"},
    {"vrouwelijke_wachter", "1F482-200D-2640-FE0F"},
    {"vrouwelijke_wachter_kleur1", "1F482-1F3FB-200D-2640-FE0F"},
    {"vrouwelijke_wachter_kleur2", "1F482-1F3FC-200D-2640-FE0F"},
    {"vrouwelijke_wachter_kleur3", "1F482-1F3FD-200D-2640-FE0F"},
    {"vrouwelijke_wachter_kleur4", "1F482-1F3FE-200D-2640-FE0F"},
    {"vrouwelijke_wachter_kleur5", "1F482-1F3FF-200D-2640-FE0F"},
    {"mannelijke_wachter", "1F482-200D-2642-FE0F"},
    {"mannelijke_wachter_kleur1", "1F482-1F3FB-200D-2642-FE0F"},
    {"mannelijke_wachter_kleur2", "1F482-1F3FC-200D-2642-FE0F"},
    {"mannelijke_wachter_kleur3", "1F482-1F3FD-200D-2642-FE0F"},
    {"mannelijke_wachter_kleur4", "1F482-1F3FE-200D-2642-FE0F"},
    {"mannelijke_wachter_kleur5", "1F482-1F3FF-200D-2642-FE0F"},
    {"vrouw_die_hoofdmassage_krijgt", "1F486-200D-2640-FE0F"},
    {"vrouw_die_hoofdmassage_krijgt_kleur1", "1F486-1F3FB-200D-2640-FE0F"},
    {"vrouw_die_hoofdmassage_krijgt_kleur2", "1F486-1F3FC-200D-2640-FE0F"},
    {"vrouw_die_hoofdmassage_krijgt_kleur3", "1F486-1F3FD-200D-2640-FE0F"},
    {"vrouw_die_hoofdmassage_krijgt_kleur4", "1F486-1F3FE-200D-2640-FE0F"},
    {"vrouw_die_hoofdmassage_krijgt_kleur5", "1F486-1F3FF-200D-2640-FE0F"},
    {"man_die_hoofdmassage_krijgt", "1F486-200D-2642-FE0F"},
    {"man_die_hoofdmassage_krijgt_kleur1", "1F486-1F3FB-200D-2642-FE0F"},
    {"man_die_hoofdmassage_krijgt_kleur2", "1F486-1F3FC-200D-2642-FE0F"},
    {"man_die_hoofdmassage_krijgt_kleur3", "1F486-1F3FD-200D-2642-FE0F"},
    {"man_die_hoofdmassage_krijgt_kleur4", "1F486-1F3FE-200D-2642-FE0F"},
    {"man_die_hoofdmassage_krijgt_kleur5", "1F486-1F3FF-200D-2642-FE0F"},
    {"vrouw_die_wordt_geknipt", "1F487-200D-2640-FE0F"},
    {"vrouw_die_wordt_geknipt_kleur1", "1F487-1F3FB-200D-2640-FE0F"},
    {"vrouw_die_wordt_geknipt_kleur2", "1F487-1F3FC-200D-2640-FE0F"},
    {"vrouw_die_wordt_geknipt_kleur3", "1F487-1F3FD-200D-2640-FE0F"},
    {"vrouw_die_wordt_geknipt_kleur4", "1F487-1F3FE-200D-2640-FE0F"},
    {"vrouw_die_wordt_geknipt_kleur5", "1F487-1F3FF-200D-2640-FE0F"},
    {"man_die_wordt_geknipt", "1F487-200D-2642-FE0F"},
    {"man_die_wordt_geknipt_kleur1", "1F487-1F3FB-200D-2642-FE0F"},
    {"man_die_wordt_geknipt_kleur2", "1F487-1F3FC-200D-2642-FE0F"},
    {"man_die_wordt_geknipt_kleur3", "1F487-1F3FD-200D-2642-FE0F"},
    {"man_die_wordt_geknipt_kleur4", "1F487-1F3FE-200D-2642-FE0F"},
    {"man_die_wordt_geknipt_kleur5", "1F487-1F3FF-200D-2642-FE0F"},
    {"vrouwelijke_detective", "1F575-FE0F-200D-2640-FE0F"},
    {"vrouwelijke_detective_kleur1", "1F575-1F3FB-200D-2640-FE0F"},
    {"vrouwelijke_detective_kleur2", "1F575-1F3FC-200D-2640-FE0F"},
    {"vrouwelijke_detective_kleur3", "1F575-1F3FD-200D-2640-FE0F"},
    {"vrouwelijke_detective_kleur4", "1F575-1F3FE-200D-2640-FE0F"},
    {"vrouwelijke_detective_kleur5", "1F575-1F3FF-200D-2640-FE0F"},
    {"mannelijke_detective", "1F575-FE0F-200D-2642-FE0F"},
    {"mannelijke_detective_kleur1", "1F575-1F3FB-200D-2642-FE0F"},
    {"mannelijke_detective_kleur2", "1F575-1F3FC-200D-2642-FE0F"},
    {"mannelijke_detective_kleur3", "1F575-1F3FD-200D-2642-FE0F"},
    {"mannelijke_detective_kleur4", "1F575-1F3FE-200D-2642-FE0F"},
    {"mannelijke_detective_kleur5", "1F575-1F3FF-200D-2642-FE0F"},
    {"vrouw_die_nee_gebaar_maakt", "1F645-200D-2640-FE0F"},
    {"vrouw_die_nee_gebaar_maakt_kleur1", "1F645-1F3FB-200D-2640-FE0F"},
    {"vrouw_die_nee_gebaar_maakt_kleur2", "1F645-1F3FC-200D-2640-FE0F"},
    {"vrouw_die_nee_gebaar_maakt_kleur3", "1F645-1F3FD-200D-2640-FE0F"},
    {"vrouw_die_nee_gebaar_maakt_kleur4", "1F645-1F3FE-200D-2640-FE0F"},
    {"vrouw_die_nee_gebaar_maakt_kleur5", "1F645-1F3FF-200D-2640-FE0F"},
    {"man_die_nee_gebaar_maakt", "1F645-200D-2642-FE0F"},
    {"man_die_nee_gebaar_maakt_kleur1", "1F645-1F3FB-200D-2642-FE0F"},
    {"man_die_nee_gebaar_maakt_kleur2", "1F645-1F3FC-200D-2642-FE0F"},
    {"man_die_nee_gebaar_maakt_kleur3", "1F645-1F3FD-200D-2642-FE0F"},
    {"man_die_nee_gebaar_maakt_kleur4", "1F645-1F3FE-200D-2642-FE0F"},
    {"man_die_nee_gebaar_maakt_kleur5", "1F645-1F3FF-200D-2642-FE0F"},
    {"vrouw_die_ok_gebaar_maakt", "1F646-200D-2640-FE0F"},
    {"vrouw_die_ok_gebaar_maakt_kleur1", "1F646-1F3FB-200D-2640-FE0F"},
    {"vrouw_die_ok_gebaar_maakt_kleur2", "1F646-1F3FC-200D-2640-FE0F"},
    {"vrouw_die_ok_gebaar_maakt_kleur3", "1F646-1F3FD-200D-2640-FE0F"},
    {"vrouw_die_ok_gebaar_maakt_kleur4", "1F646-1F3FE-200D-2640-FE0F"},
    {"vrouw_die_ok_gebaar_maakt_kleur5", "1F646-1F3FF-200D-2640-FE0F"},
    {"man_die_ok_gebaar_maakt", "1F646-200D-2642-FE0F"},
    {"man_die_ok_gebaar_maakt_kleur1", "1F646-1F3FB-200D-2642-FE0F"},
    {"man_die_ok_gebaar_maakt_kleur2", "1F646-1F3FC-200D-2642-FE0F"},
    {"man_die_ok_gebaar_maakt_kleur3", "1F646-1F3FD-200D-2642-FE0F"},
    {"man_die_ok_gebaar_maakt_kleur4", "1F646-1F3FE-200D-2642-FE0F"},
    {"man_die_ok_gebaar_maakt_kleur5", "1F646-1F3FF-200D-2642-FE0F"},
    {"buigende_vrouw", "1F647-200D-2640-FE0F"},
    {"buigende_vrouw_kleur1", "1F647-1F3FB-200D-2640-FE0F"},
    {"buigende_vrouw_kleur2", "1F647-1F3FC-200D-2640-FE0F"},
    {"buigende_vrouw_kleur3", "1F647-1F3FD-200D-2640-FE0F"},
    {"buigende_vrouw_kleur4", "1F647-1F3FE-200D-2640-FE0F"},
    {"buigende_vrouw_kleur5", "1F647-1F3FF-200D-2640-FE0F"},
    {"buigende_man", "1F647-200D-2642-FE0F"},
    {"buigende_man_kleur1", "1F647-1F3FB-200D-2642-FE0F"},
    {"buigende_man_kleur2", "1F647-1F3FC-200D-2642-FE0F"},
    {"buigende_man_kleur3", "1F647-1F3FD-200D-2642-FE0F"},
    {"buigende_man_kleur4", "1F647-1F3FE-200D-2642-FE0F"},
    {"buigende_man_kleur5", "1F647-1F3FF-200D-2642-FE0F"},
    {"vrouw_met_opgeheven_hand", "1F64B-200D-2640-FE0F"},
    {"vrouw_met_opgeheven_hand_kleur1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"vrouw_met_opgeheven_hand_kleur2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"vrouw_met_opgeheven_hand_kleur3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"vrouw_met_opgeheven_hand_kleur4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"vrouw_met_opgeheven_hand_kleur5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"man_met_opgeheven_hand", "1F64B-200D-2642-FE0F"},
    {"man_met_opgeheven_hand_kleur1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"man_met_opgeheven_hand_kleur2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"man_met_opgeheven_hand_kleur3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"man_met_opgeheven_hand_kleur4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"man_met_opgeheven_hand_kleur5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"fronsende_vrouw", "1F64D-200D-2640-FE0F"},
    {"fronsende_vrouw_kleur1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"fronsende_vrouw_kleur2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"fronsende_vrouw_kleur3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"fronsende_vrouw_kleur4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"fronsende_vrouw_kleur5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"fronsende_man", "1F64D-200D-2642-FE0F"},
    {"fronsende_man_kleur1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"fronsende_man_kleur2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"fronsende_man_kleur3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"fronsende_man_kleur4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"fronsende_man_kleur5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"pruilende_vrouw", "1F64E-200D-2640-FE0F"},
    {"pruilende_vrouw_kleur1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"pruilende_vrouw_kleur2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"pruilende_vrouw_kleur3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"pruilende_vrouw_kleur4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"pruilende_vrouw_kleur5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"pruilende_man", "1F64E-200D-2642-FE0F"},
    {"pruilende_man_kleur1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"pruilende_man_kleur2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"pruilende_man_kleur3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"pruilende_man_kleur4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"pruilende_man_kleur5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"vrouw_in_roeiboot", "1F6A3-200D-2640-FE0F"},
    {"vrouw_in_roeiboot_kleur1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"vrouw_in_roeiboot_kleur2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"vrouw_in_roeiboot_kleur3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"vrouw_in_roeiboot_kleur4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"vrouw_in_roeiboot_kleur5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"man_in_roeiboot", "1F6A3-200D-2642-FE0F"},
    {"man_in_roeiboot_kleur1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"man_in_roeiboot_kleur2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"man_in_roeiboot_kleur3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"man_in_roeiboot_kleur4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"man_in_roeiboot_kleur5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"fietsende_vrouw", "1F6B4-200D-2640-FE0F"},
    {"fietsende_vrouw_kleur1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"fietsende_vrouw_kleur2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"fietsende_vrouw_kleur3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"fietsende_vrouw_kleur4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"fietsende_vrouw_kleur5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"fietsende_man", "1F6B4-200D-2642-FE0F"},
    {"fietsende_man_kleur1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"fietsende_man_kleur2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"fietsende_man_kleur3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"fietsende_man_kleur4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"fietsende_man_kleur5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"vrouw_op_mountainbike", "1F6B5-200D-2640-FE0F"},
    {"vrouw_op_mountainbike_kleur1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"vrouw_op_mountainbike_kleur2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"vrouw_op_mountainbike_kleur3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"vrouw_op_mountainbike_kleur4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"vrouw_op_mountainbike_kleur5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"man_op_mountainbike", "1F6B5-200D-2642-FE0F"},
    {"man_op_mountainbike_kleur1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"man_op_mountainbike_kleur2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"man_op_mountainbike_kleur3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"man_op_mountainbike_kleur4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"man_op_mountainbike_kleur5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"wandelende_vrouw", "1F6B6-200D-2640-FE0F"},
    {"wandelende_vrouw_kleur1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"wandelende_vrouw_kleur2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"wandelende_vrouw_kleur3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"wandelende_vrouw_kleur4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"wandelende_vrouw_kleur5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"wandelende_vrouw_naar_rechts_gericht", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"wandelende_vrouw_naar_rechts_gericht_kleur1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"wandelende_vrouw_naar_rechts_gericht_kleur2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"wandelende_vrouw_naar_rechts_gericht_kleur3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"wandelende_vrouw_naar_rechts_gericht_kleur4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"wandelende_vrouw_naar_rechts_gericht_kleur5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"wandelende_man", "1F6B6-200D-2642-FE0F"},
    {"wandelende_man_kleur1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"wandelende_man_kleur2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"wandelende_man_kleur3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"wandelende_man_kleur4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"wandelende_man_kleur5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"wandelende_man_naar_rechts_gericht", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"wandelende_man_naar_rechts_gericht_kleur1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"wandelende_man_naar_rechts_gericht_kleur2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"wandelende_man_naar_rechts_gericht_kleur3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"wandelende_man_naar_rechts_gericht_kleur4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"wandelende_man_naar_rechts_gericht_kleur5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"vrouw_die_het_gezicht_met_de_hand_bedekt", "1F926-200D-2640-FE0F"},
    {"vrouw_die_het_gezicht_met_de_hand_bedekt_kleur1", "1F926-1F3FB-200D-2640-FE0F"},
    {"vrouw_die_het_gezicht_met_de_hand_bedekt_kleur2", "1F926-1F3FC-200D-2640-FE0F"},
    {"vrouw_die_het_gezicht_met_de_hand_bedekt_kleur3", "1F926-1F3FD-200D-2640-FE0F"},
    {"vrouw_die_het_gezicht_met_de_hand_bedekt_kleur4", "1F926-1F3FE-200D-2640-FE0F"},
    {"vrouw_die_het_gezicht_met_de_hand_bedekt_kleur5", "1F926-1F3FF-200D-2640-FE0F"},
    {"man_die_het_gezicht_met_de_hand_bedekt", "1F926-200D-2642-FE0F"},
    {"man_die_het_gezicht_met_de_hand_bedekt_kleur1", "1F926-1F3FB-200D-2642-FE0F"},
    {"man_die_het_gezicht_met_de_hand_bedekt_kleur2", "1F926-1F3FC-200D-2642-FE0F"},
    {"man_die_het_gezicht_met_de_hand_bedekt_kleur3", "1F926-1F3FD-200D-2642-FE0F"},
    {"man_die_het_gezicht_met_de_hand_bedekt_kleur4", "1F926-1F3FE-200D-2642-FE0F"},
    {"man_die_het_gezicht_met_de_hand_bedekt_kleur5", "1F926-1F3FF-200D-2642-FE0F"},
    {"vrouw_in_smoking", "1F935-200D-2640-FE0F"},
    {"vrouw_in_smoking_kleur1", "1F935-1F3FB-200D-2640-FE0F"},
    {"vrouw_in_smoking_kleur2", "1F935-1F3FC-200D-2640-FE0F"},
    {"vrouw_in_smoking_kleur3", "1F935-1F3FD-200D-2640-FE0F"},
    {"vrouw_in_smoking_kleur4", "1F935-1F3FE-200D-2640-FE0F"},
    {"vrouw_in_smoking_kleur5", "1F935-1F3FF-200D-2640-FE0F"},
    {"man_in_smoking", "1F935-200D-2642-FE0F"},
    {"man_in_smoking_kleur1", "1F935-1F3FB-200D-2642-FE0F"},
    {"man_in_smoking_kleur2", "1F935-1F3FC-200D-2642-FE0F"},
    {"man_in_smoking_kleur3", "1F935-1F3FD-200D-2642-FE0F"},
    {"man_in_smoking_kleur4", "1F935-1F3FE-200D-2642-FE0F"},
    {"man_in_smoking_kleur5", "1F935-1F3FF-200D-2642-FE0F"},
    {"vrouw_die_schouders_ophaalt", "1F937-200D-2640-FE0F"},
    {"vrouw_die_schouders_ophaalt_kleur1", "1F937-1F3FB-200D-2640-FE0F"},
    {"vrouw_die_schouders_ophaalt_kleur2", "1F937-1F3FC-200D-2640-FE0F"},
    {"vrouw_die_schouders_ophaalt_kleur3", "1F937-1F3FD-200D-2640-FE0F"},
    {"vrouw_die_schouders_ophaalt_kleur4", "1F937-1F3FE-200D-2640-FE0F"},
    {"vrouw_die_schouders_ophaalt_kleur5", "1F937-1F3FF-200D-2640-FE0F"},
    {"man_die_schouders_ophaalt", "1F937-200D-2642-FE0F"},
    {"man_die_schouders_ophaalt_kleur1", "1F937-1F3FB-200D-2642-FE0F"},
    {"man_die_schouders_ophaalt_kleur2", "1F937-1F3FC-200D-2642-FE0F"},
    {"man_die_schouders_ophaalt_kleur3", "1F937-1F3FD-200D-2642-FE0F"},
    {"man_die_schouders_ophaalt_kleur4", "1F937-1F3FE-200D-2642-FE0F"},
    {"man_die_schouders_ophaalt_kleur5", "1F937-1F3FF-200D-2642-FE0F"},
    {"vrouw_die_een_radslag_doet", "1F938-200D-2640-FE0F"},
    {"vrouw_die_een_radslag_doet_kleur1", "1F938-1F3FB-200D-2640-FE0F"},
    {"vrouw_die_een_radslag_doet_kleur2", "1F938-1F3FC-200D-2640-FE0F"},
    {"vrouw_die_een_radslag_doet_kleur3", "1F938-1F3FD-200D-2640-FE0F"},
    {"vrouw_die_een_radslag_doet_kleur4", "1F938-1F3FE-200D-2640-FE0F"},
    {"vrouw_die_een_radslag_doet_kleur5", "1F938-1F3FF-200D-2640-FE0F"},
    {"man_die_een_radslag_doet", "1F938-200D-2642-FE0F"},
    {"man_die_een_radslag_doet_kleur1", "1F938-1F3FB-200D-2642-FE0F"},
    {"man_die_een_radslag_doet_kleur2", "1F938-1F3FC-200D-2642-FE0F"},
    {"man_die_een_radslag_doet_kleur3", "1F938-1F3FD-200D-2642-FE0F"},
    {"man_die_een_radslag_doet_kleur4", "1F938-1F3FE-200D-2642-FE0F"},
    {"man_die_een_radslag_doet_kleur5", "1F938-1F3FF-200D-2642-FE0F"},
    {"vrouw_die_jongleert", "1F939-200D-2640-FE0F"},
    {"vrouw_die_jongleert_kleur1", "1F939-1F3FB-200D-2640-FE0F"},
    {"vrouw_die_jongleert_kleur2", "1F939-1F3FC-200D-2640-FE0F"},
    {"vrouw_die_jongleert_kleur3", "1F939-1F3FD-200D-2640-FE0F"},
    {"vrouw_die_jongleert_kleur4", "1F939-1F3FE-200D-2640-FE0F"},
    {"vrouw_die_jongleert_kleur5", "1F939-1F3FF-200D-2640-FE0F"},
    {"man_die_jongleert", "1F939-200D-2642-FE0F"},
    {"man_die_jongleert_kleur1", "1F939-1F3FB-200D-2642-FE0F"},
    {"man_die_jongleert_kleur2", "1F939-1F3FC-200D-2642-FE0F"},
    {"man_die_jongleert_kleur3", "1F939-1F3FD-200D-2642-FE0F"},
    {"man_die_jongleert_kleur4", "1F939-1F3FE-200D-2642-FE0F"},
    {"man_die_jongleert_kleur5", "1F939-1F3FF-200D-2642-FE0F"},
    {"worstelende_vrouwen", "1F93C-200D-2640-FE0F"},
    {"worstelende_vrouwen_kleur1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"worstelende_vrouwen_kleur1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"worstelende_vrouwen_kleur1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"worstelende_vrouwen_kleur1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"worstelende_vrouwen_kleur2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"worstelende_vrouwen_kleur2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"worstelende_vrouwen_kleur2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"worstelende_vrouwen_kleur2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"worstelende_vrouwen_kleur3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"worstelende_vrouwen_kleur3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"worstelende_vrouwen_kleur3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"worstelende_vrouwen_kleur3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"worstelende_vrouwen_kleur4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"worstelende_vrouwen_kleur4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"worstelende_vrouwen_kleur4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"worstelende_vrouwen_kleur4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"worstelende_vrouwen_kleur5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"worstelende_vrouwen_kleur5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"worstelende_vrouwen_kleur5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"worstelende_vrouwen_kleur5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"worstelende_vrouwen_kleur1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"worstelende_vrouwen_kleur2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"worstelende_vrouwen_kleur3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"worstelende_vrouwen_kleur4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"worstelende_vrouwen_kleur5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"worstelende_mannen", "1F93C-200D-2642-FE0F"},
    {"worstelende_mannen_kleur1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"worstelende_mannen_kleur1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"worstelende_mannen_kleur1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"worstelende_mannen_kleur1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"worstelende_mannen_kleur2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"worstelende_mannen_kleur2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"worstelende_mannen_kleur2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"worstelende_mannen_kleur2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"worstelende_mannen_kleur3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"worstelende_mannen_kleur3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"worstelende_mannen_kleur3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"worstelende_mannen_kleur3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"worstelende_mannen_kleur4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"worstelende_mannen_kleur4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"worstelende_mannen_kleur4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"worstelende_mannen_kleur4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"worstelende_mannen_kleur5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"worstelende_mannen_kleur5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"worstelende_mannen_kleur5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"worstelende_mannen_kleur5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"worstelende_mannen_kleur1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"worstelende_mannen_kleur2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"worstelende_mannen_kleur3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"worstelende_mannen_kleur4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"worstelende_mannen_kleur5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"vrouw_die_waterpolo_speelt", "1F93D-200D-2640-FE0F"},
    {"vrouw_die_waterpolo_speelt_kleur1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"vrouw_die_waterpolo_speelt_kleur2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"vrouw_die_waterpolo_speelt_kleur3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"vrouw_die_waterpolo_speelt_kleur4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"vrouw_die_waterpolo_speelt_kleur5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"man_die_waterpolo_speelt", "1F93D-200D-2642-FE0F"},
    {"man_die_waterpolo_speelt_kleur1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"man_die_waterpolo_speelt_kleur2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"man_die_waterpolo_speelt_kleur3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"man_die_waterpolo_speelt_kleur4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"man_die_waterpolo_speelt_kleur5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"vrouw_die_handbal_speelt", "1F93E-200D-2640-FE0F"},
    {"vrouw_die_handbal_speelt_kleur1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"vrouw_die_handbal_speelt_kleur2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"vrouw_die_handbal_speelt_kleur3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"vrouw_die_handbal_speelt_kleur4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"vrouw_die_handbal_speelt_kleur5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"man_die_handbal_speelt", "1F93E-200D-2642-FE0F"},
    {"man_die_handbal_speelt_kleur1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"man_die_handbal_speelt_kleur2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"man_die_handbal_speelt_kleur3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"man_die_handbal_speelt_kleur4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"man_die_handbal_speelt_kleur5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"vrouwelijke_superheld", "1F9B8-200D-2640-FE0F"},
    {"vrouwelijke_superheld_kleur1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"vrouwelijke_superheld_kleur2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"vrouwelijke_superheld_kleur3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"vrouwelijke_superheld_kleur4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"vrouwelijke_superheld_kleur5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"mannelijke_superheld", "1F9B8-200D-2642-FE0F"},
    {"mannelijke_superheld_kleur1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"mannelijke_superheld_kleur2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"mannelijke_superheld_kleur3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"mannelijke_superheld_kleur4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"mannelijke_superheld_kleur5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"vrouwelijke_superschurk", "1F9B9-200D-2640-FE0F"},
    {"vrouwelijke_superschurk_kleur1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"vrouwelijke_superschurk_kleur2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"vrouwelijke_superschurk_kleur3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"vrouwelijke_superschurk_kleur4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"vrouwelijke_superschurk_kleur5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"mannelijke_superschurk", "1F9B9-200D-2642-FE0F"},
    {"mannelijke_superschurk_kleur1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"mannelijke_superschurk_kleur2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"mannelijke_superschurk_kleur3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"mannelijke_superschurk_kleur4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"mannelijke_superschurk_kleur5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"staande_vrouw", "1F9CD-200D-2640-FE0F"},
    {"staande_vrouw_kleur1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"staande_vrouw_kleur2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"staande_vrouw_kleur3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"staande_vrouw_kleur4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"staande_vrouw_kleur5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"staande_man", "1F9CD-200D-2642-FE0F"},
    {"staande_man_kleur1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"staande_man_kleur2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"staande_man_kleur3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"staande_man_kleur4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"staande_man_kleur5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"knielende_vrouw", "1F9CE-200D-2640-FE0F"},
    {"knielende_vrouw_kleur1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"knielende_vrouw_kleur2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"knielende_vrouw_kleur3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"knielende_vrouw_kleur4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"knielende_vrouw_kleur5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"knielende_vrouw_naar_rechts_gericht", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knielende_vrouw_naar_rechts_gericht_kleur1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knielende_vrouw_naar_rechts_gericht_kleur2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knielende_vrouw_naar_rechts_gericht_kleur3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knielende_vrouw_naar_rechts_gericht_kleur4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knielende_vrouw_naar_rechts_gericht_kleur5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knielende_man", "1F9CE-200D-2642-FE0F"},
    {"knielende_man_kleur1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"knielende_man_kleur2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"knielende_man_kleur3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"knielende_man_kleur4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"knielende_man_kleur5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"knielende_man_naar_rechts_gericht", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knielende_man_naar_rechts_gericht_kleur1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knielende_man_naar_rechts_gericht_kleur2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knielende_man_naar_rechts_gericht_kleur3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knielende_man_naar_rechts_gericht_kleur4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knielende_man_naar_rechts_gericht_kleur5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"dove_vrouw", "1F9CF-200D-2640-FE0F"},
    {"dove_vrouw_kleur1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"dove_vrouw_kleur2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"dove_vrouw_kleur3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"dove_vrouw_kleur4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"dove_vrouw_kleur5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"dove_man", "1F9CF-200D-2642-FE0F"},
    {"dove_man_kleur1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"dove_man_kleur2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"dove_man_kleur3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"dove_man_kleur4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"dove_man_kleur5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"vrouw_baard", "1F9D4-200D-2640-FE0F"},
    {"vrouw_baard_kleur1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"vrouw_baard_kleur2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"vrouw_baard_kleur3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"vrouw_baard_kleur4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"vrouw_baard_kleur5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"man_baard", "1F9D4-200D-2642-FE0F"},
    {"man_baard_kleur1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"man_baard_kleur2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"man_baard_kleur3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"man_baard_kleur4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"man_baard_kleur5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"vrouw_in_ruimte_vol_stoom", "1F9D6-200D-2640-FE0F"},
    {"vrouw_in_ruimte_vol_stoom_kleur1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"vrouw_in_ruimte_vol_stoom_kleur2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"vrouw_in_ruimte_vol_stoom_kleur3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"vrouw_in_ruimte_vol_stoom_kleur4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"vrouw_in_ruimte_vol_stoom_kleur5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"man_in_ruimte_vol_stoom", "1F9D6-200D-2642-FE0F"},
    {"man_in_ruimte_vol_stoom_kleur1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"man_in_ruimte_vol_stoom_kleur2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"man_in_ruimte_vol_stoom_kleur3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"man_in_ruimte_vol_stoom_kleur4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"man_in_ruimte_vol_stoom_kleur5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"klimmende_vrouw", "1F9D7-200D-2640-FE0F"},
    {"klimmende_vrouw_kleur1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"klimmende_vrouw_kleur2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"klimmende_vrouw_kleur3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"klimmende_vrouw_kleur4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"klimmende_vrouw_kleur5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"klimmende_man", "1F9D7-200D-2642-FE0F"},
    {"klimmende_man_kleur1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"klimmende_man_kleur2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"klimmende_man_kleur3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"klimmende_man_kleur4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"klimmende_man_kleur5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"vrouw_in_lotushouding", "1F9D8-200D-2640-FE0F"},
    {"vrouw_in_lotushouding_kleur1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"vrouw_in_lotushouding_kleur2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"vrouw_in_lotushouding_kleur3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"vrouw_in_lotushouding_kleur4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"vrouw_in_lotushouding_kleur5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"man_in_lotushouding", "1F9D8-200D-2642-FE0F"},
    {"man_in_lotushouding_kleur1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"man_in_lotushouding_kleur2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"man_in_lotushouding_kleur3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"man_in_lotushouding_kleur4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"man_in_lotushouding_kleur5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"vrouwelijke_magier", "1F9D9-200D-2640-FE0F"},
    {"vrouwelijke_magier_kleur1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"vrouwelijke_magier_kleur2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"vrouwelijke_magier_kleur3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"vrouwelijke_magier_kleur4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"vrouwelijke_magier_kleur5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"mannelijke_magier", "1F9D9-200D-2642-FE0F"},
    {"mannelijke_magier_kleur1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"mannelijke_magier_kleur2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"mannelijke_magier_kleur3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"mannelijke_magier_kleur4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"mannelijke_magier_kleur5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"vrouwelijk_elfje", "1F9DA-200D-2640-FE0F"},
    {"vrouwelijk_elfje_kleur1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"vrouwelijk_elfje_kleur2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"vrouwelijk_elfje_kleur3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"vrouwelijk_elfje_kleur4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"vrouwelijk_elfje_kleur5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"mannelijk_elfje", "1F9DA-200D-2642-FE0F"},
    {"mannelijk_elfje_kleur1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"mannelijk_elfje_kleur2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"mannelijk_elfje_kleur3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"mannelijk_elfje_kleur4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"mannelijk_elfje_kleur5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"vrouwelijke_vampier", "1F9DB-200D-2640-FE0F"},
    {"vrouwelijke_vampier_kleur1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"vrouwelijke_vampier_kleur2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"vrouwelijke_vampier_kleur3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"vrouwelijke_vampier_kleur4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"vrouwelijke_vampier_kleur5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"mannelijke_vampier", "1F9DB-200D-2642-FE0F"},
    {"mannelijke_vampier_kleur1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"mannelijke_vampier_kleur2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"mannelijke_vampier_kleur3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"mannelijke_vampier_kleur4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"mannelijke_vampier_kleur5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"sirene", "1F9DC-200D-2640-FE0F"},
    {"sirene_kleur1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"sirene_kleur2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"sirene_kleur3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"sirene_kleur4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"sirene_kleur5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"zeemeerman", "1F9DC-200D-2642-FE0F"},
    {"zeemeerman_kleur1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"zeemeerman_kleur2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"zeemeerman_kleur3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"zeemeerman_kleur4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"zeemeerman_kleur5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"vrouwelijke_elf", "1F9DD-200D-2640-FE0F"},
    {"vrouwelijke_elf_kleur1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"vrouwelijke_elf_kleur2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"vrouwelijke_elf_kleur3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"vrouwelijke_elf_kleur4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"vrouwelijke_elf_kleur5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"mannelijke_elf", "1F9DD-200D-2642-FE0F"},
    {"mannelijke_elf_kleur1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"mannelijke_elf_kleur2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"mannelijke_elf_kleur3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"mannelijke_elf_kleur4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"mannelijke_elf_kleur5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"vrouwelijke_geest_in_de_fles", "1F9DE-200D-2640-FE0F"},
    {"mannelijke_geest_in_de_fles", "1F9DE-200D-2642-FE0F"},
    {"vrouwelijke_zombie", "1F9DF-200D-2640-FE0F"},
    {"mannelijke_zombie", "1F9DF-200D-2642-FE0F"},
    {"man_rood_haar", "1F468-200D-1F9B0"},
    {"man_rood_haar_kleur1", "1F468-1F3FB-200D-1F9B0"},
    {"man_rood_haar_kleur2", "1F468-1F3FC-200D-1F9B0"},
    {"man_rood_haar_kleur3", "1F468-1F3FD-200D-1F9B0"},
    {"man_rood_haar_kleur4", "1F468-1F3FE-200D-1F9B0"},
    {"man_rood_haar_kleur5", "1F468-1F3FF-200D-1F9B0"},
    {"man_krullend_haar", "1F468-200D-1F9B1"},
    {"man_krullend_haar_kleur1", "1F468-1F3FB-200D-1F9B1"},
    {"man_krullend_haar_kleur2", "1F468-1F3FC-200D-1F9B1"},
    {"man_krullend_haar_kleur3", "1F468-1F3FD-200D-1F9B1"},
    {"man_krullend_haar_kleur4", "1F468-1F3FE-200D-1F9B1"},
    {"man_krullend_haar_kleur5", "1F468-1F3FF-200D-1F9B1"},
    {"man_kaal", "1F468-200D-1F9B2"},
    {"man_kaal_kleur1", "1F468-1F3FB-200D-1F9B2"},
    {"man_kaal_kleur2", "1F468-1F3FC-200D-1F9B2"},
    {"man_kaal_kleur3", "1F468-1F3FD-200D-1F9B2"},
    {"man_kaal_kleur4", "1F468-1F3FE-200D-1F9B2"},
    {"man_kaal_kleur5", "1F468-1F3FF-200D-1F9B2"},
    {"man_wit_haar", "1F468-200D-1F9B3"},
    {"man_wit_haar_kleur1", "1F468-1F3FB-200D-1F9B3"},
    {"man_wit_haar_kleur2", "1F468-1F3FC-200D-1F9B3"},
    {"man_wit_haar_kleur3", "1F468-1F3FD-200D-1F9B3"},
    {"man_wit_haar_kleur4", "1F468-1F3FE-200D-1F9B3"},
    {"man_wit_haar_kleur5", "1F468-1F3FF-200D-1F9B3"},
    {"vrouw_rood_haar", "1F469-200D-1F9B0"},
    {"vrouw_rood_haar_kleur1", "1F469-1F3FB-200D-1F9B0"},
    {"vrouw_rood_haar_kleur2", "1F469-1F3FC-200D-1F9B0"},
    {"vrouw_rood_haar_kleur3", "1F469-1F3FD-200D-1F9B0"},
    {"vrouw_rood_haar_kleur4", "1F469-1F3FE-200D-1F9B0"},
    {"vrouw_rood_haar_kleur5", "1F469-1F3FF-200D-1F9B0"},
    {"vrouw_krullend_haar", "1F469-200D-1F9B1"},
    {"vrouw_krullend_haar_kleur1", "1F469-1F3FB-200D-1F9B1"},
    {"vrouw_krullend_haar_kleur2", "1F469-1F3FC-200D-1F9B1"},
    {"vrouw_krullend_haar_kleur3", "1F469-1F3FD-200D-1F9B1"},
    {"vrouw_krullend_haar_kleur4", "1F469-1F3FE-200D-1F9B1"},
    {"vrouw_krullend_haar_kleur5", "1F469-1F3FF-200D-1F9B1"},
    {"vrouw_kaal", "1F469-200D-1F9B2"},
    {"vrouw_kaal_kleur1", "1F469-1F3FB-200D-1F9B2"},
    {"vrouw_kaal_kleur2", "1F469-1F3FC-200D-1F9B2"},
    {"vrouw_kaal_kleur3", "1F469-1F3FD-200D-1F9B2"},
    {"vrouw_kaal_kleur4", "1F469-1F3FE-200D-1F9B2"},
    {"vrouw_kaal_kleur5", "1F469-1F3FF-200D-1F9B2"},
    {"vrouw_wit_haar", "1F469-200D-1F9B3"},
    {"vrouw_wit_haar_kleur1", "1F469-1F3FB-200D-1F9B3"},
    {"vrouw_wit_haar_kleur2", "1F469-1F3FC-200D-1F9B3"},
    {"vrouw_wit_haar_kleur3", "1F469-1F3FD-200D-1F9B3"},
    {"vrouw_wit_haar_kleur4", "1F469-1F3FE-200D-1F9B3"},
    {"vrouw_wit_haar_kleur5", "1F469-1F3FF-200D-1F9B3"},
    {"volwassene_rood_haar", "1F9D1-200D-1F9B0"},
    {"volwassene_rood_haar_kleur1", "1F9D1-1F3FB-200D-1F9B0"},
    {"volwassene_rood_haar_kleur2", "1F9D1-1F3FC-200D-1F9B0"},
    {"volwassene_rood_haar_kleur3", "1F9D1-1F3FD-200D-1F9B0"},
    {"volwassene_rood_haar_kleur4", "1F9D1-1F3FE-200D-1F9B0"},
    {"volwassene_rood_haar_kleur5", "1F9D1-1F3FF-200D-1F9B0"},
    {"volwassene_krullend_haar", "1F9D1-200D-1F9B1"},
    {"volwassene_krullend_haar_kleur1", "1F9D1-1F3FB-200D-1F9B1"},
    {"volwassene_krullend_haar_kleur2", "1F9D1-1F3FC-200D-1F9B1"},
    {"volwassene_krullend_haar_kleur3", "1F9D1-1F3FD-200D-1F9B1"},
    {"volwassene_krullend_haar_kleur4", "1F9D1-1F3FE-200D-1F9B1"},
    {"volwassene_krullend_haar_kleur5", "1F9D1-1F3FF-200D-1F9B1"},
    {"volwassene_kaal", "1F9D1-200D-1F9B2"},
    {"volwassene_kaal_kleur1", "1F9D1-1F3FB-200D-1F9B2"},
    {"volwassene_kaal_kleur2", "1F9D1-1F3FC-200D-1F9B2"},
    {"volwassene_kaal_kleur3", "1F9D1-1F3FD-200D-1F9B2"},
    {"volwassene_kaal_kleur4", "1F9D1-1F3FE-200D-1F9B2"},
    {"volwassene_kaal_kleur5", "1F9D1-1F3FF-200D-1F9B2"},
    {"volwassene_wit_haar", "1F9D1-200D-1F9B3"},
    {"volwassene_wit_haar_kleur1", "1F9D1-1F3FB-200D-1F9B3"},
    {"volwassene_wit_haar_kleur2", "1F9D1-1F3FC-200D-1F9B3"},
    {"volwassene_wit_haar_kleur3", "1F9D1-1F3FD-200D-1F9B3"},
    {"volwassene_wit_haar_kleur4", "1F9D1-1F3FE-200D-1F9B3"},
    {"volwassene_wit_haar_kleur5", "1F9D1-1F3FF-200D-1F9B3"},
    {"gebroken_ketting", "26D3-FE0F-200D-1F4A5"},
    {"hart_in_vuur_en_vlam", "2764-FE0F-200D-1F525"},
    {"helend_hart", "2764-FE0F-200D-1FA79"},
    {"bruine_paddestoel", "1F344-200D-1F7EB"},
    {"limoen", "1F34B-200D-1F7E9"},
    {"transgendervlag", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"regenboogvlag", "1F3F3-FE0F-200D-1F308"},
    {"piratenvlag", "1F3F4-200D-2620-FE0F"},
    {"zwarte_kat", "1F408-200D-2B1B"},
    {"assistentiehond", "1F415-200D-1F9BA"},
    {"merel", "1F426-200D-2B1B"},
    {"feniks", "1F426-200D-1F525"},
    {"ijsbeer", "1F43B-200D-2744-FE0F"},
    {"spraakballon_met_oog", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"gezicht_dat_uitademt", "1F62E-200D-1F4A8"},
    {"gezicht_met_spiraalvormige_ogen", "1F635-200D-1F4AB"},
    {"gezicht_in_de_wolken", "1F636-200D-1F32B-FE0F"},
    {"hoofd_dat_horizontaal_beweegt", "1F642-200D-2194-FE0F"},
    {"hoofd_dat_verticaal_beweegt", "1F642-200D-2195-FE0F"},
    {"balletdanser", "1F9D1-200D-1FA70"},
    {"balletdanser_kleur1", "1F9D1-1F3FB-200D-1FA70"},
    {"balletdanser_kleur2", "1F9D1-1F3FC-200D-1FA70"},
    {"balletdanser_kleur3", "1F9D1-1F3FD-200D-1FA70"},
    {"balletdanser_kleur4", "1F9D1-1F3FE-200D-1FA70"},
    {"balletdanser_kleur5", "1F9D1-1F3FF-200D-1FA70"},
    {"skiër", "26F7"},
    {"wereldbol_met_azië_australië", "1F30F"},
    {"maïskolf", "1F33D"},
    {"pinguïn", "1F427"},
    {"knop_voor_één_herhalen", "1F502"},
    {"kaäba", "1F54B"},
    {"één_uur", "1F550"},
    {"half_één", "1F567"},
    {"maté", "1F9C9"},
    {"magiër", "1F9D9"},
    {"magiër_kleur1", "1F9D9-1F3FB"},
    {"magiër_kleur2", "1F9D9-1F3FC"},
    {"magiër_kleur3", "1F9D9-1F3FD"},
    {"magiër_kleur4", "1F9D9-1F3FE"},
    {"magiër_kleur5", "1F9D9-1F3FF"},
    {"röntgenfoto", "1FA7B"},
    {"piñata", "1FA85"},
    {"vlag_albanië", "1F1E6-1F1F1"},
    {"vlag_armenië", "1F1E6-1F1F2"},
    {"vlag_argentinië", "1F1E6-1F1F7"},
    {"vlag_australië", "1F1E6-1F1FA"},
    {"vlag_åland", "1F1E6-1F1FD"},
    {"vlag_bosnië_en_herzegovina", "1F1E7-1F1E6"},
    {"vlag_belgië", "1F1E7-1F1EA"},
    {"vlag_saint_barthélemy", "1F1E7-1F1F1"},
    {"vlag_brazilië", "1F1E7-1F1F7"},
    {"vlag_kaapverdië", "1F1E8-1F1FB"},
    {"vlag_curaçao", "1F1E8-1F1FC"},
    {"vlag_tsjechië", "1F1E8-1F1FF"},
    {"vlag_ethiopië", "1F1EA-1F1F9"},
    {"vlag_faeröer", "1F1EB-1F1F4"},
    {"vlag_georgië", "1F1EC-1F1EA"},
    {"vlag_kroatië", "1F1ED-1F1F7"},
    {"vlag_haïti", "1F1ED-1F1F9"},
    {"vlag_indonesië", "1F1EE-1F1E9"},
    {"vlag_israël", "1F1EE-1F1F1"},
    {"vlag_italië", "1F1EE-1F1F9"},
    {"vlag_jordanië", "1F1EF-1F1F4"},
    {"vlag_kirgizië", "1F1F0-1F1EC"},
    {"vlag_libië", "1F1F1-1F1FE"},
    {"vlag_moldavië", "1F1F2-1F1E9"},
    {"vlag_noord_macedonië", "1F1F2-1F1F0"},
    {"vlag_mongolië", "1F1F2-1F1F3"},
    {"vlag_mauritanië", "1F1F2-1F1F7"},
    {"vlag_maleisië", "1F1F2-1F1FE"},
    {"vlag_namibië", "1F1F3-1F1E6"},
    {"vlag_nieuw_caledonië", "1F1F3-1F1E8"},
    {"vlag_frans_polynesië", "1F1F5-1F1EB"},
    {"vlag_réunion", "1F1F7-1F1EA"},
    {"vlag_roemenië", "1F1F7-1F1F4"},
    {"vlag_servië", "1F1F7-1F1F8"},
    {"vlag_saoedi_arabië", "1F1F8-1F1E6"},
    {"vlag_slovenië", "1F1F8-1F1EE"},
    {"vlag_somalië", "1F1F8-1F1F4"},
    {"vlag_sao_tomé_en_principe", "1F1F8-1F1F9"},
    {"vlag_syrië", "1F1F8-1F1FE"},
    {"vlag_tunesië", "1F1F9-1F1F3"},
    {"vlag_oekraïne", "1F1FA-1F1E6"},
    {"vrouwelijke_magiër", "1F9D9-200D-2640-FE0F"},
    {"vrouwelijke_magiër_kleur1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"vrouwelijke_magiër_kleur2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"vrouwelijke_magiër_kleur3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"vrouwelijke_magiër_kleur4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"vrouwelijke_magiër_kleur5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"mannelijke_magiër", "1F9D9-200D-2642-FE0F"},
    {"mannelijke_magiër_kleur1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"mannelijke_magiër_kleur2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"mannelijke_magiër_kleur3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"mannelijke_magiër_kleur4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"mannelijke_magiër_kleur5", "1F9D9-1F3FF-200D-2642-FE0F"},
};

