// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_sv[] = {
    {"utropstecken_och_fragetecken", "2049"},
    {"skyddat_varumarke", "2122"},
    {"information", "2139"},
    {"pil_hoger_och_vanster", "2194"},
    {"pil_upp_och_ned", "2195"},
    {"uppatpil_vanster", "2196"},
    {"uppatpil_hoger", "2197"},
    {"nedatpil_hoger", "2198"},
    {"nedatpil_vanster", "2199"},
    {"tangentbord", "2328"},
    {"sol", "2600"},
    {"moln", "2601"},
    {"paraply", "2602"},
    {"snogubbe", "2603"},
    {"komet", "2604"},
    {"kryssruta_med_bockmarkering", "2611"},
    {"paraply_med_regndroppar", "2614"},
    {"varm_dryck", "2615"},
    {"treklover", "2618"},
    {"dodskalle_med_ben_i_kors", "2620"},
    {"varning_for_stralrisk", "2622"},
    {"biologisk_fara", "2623"},
    {"ortodoxt_kors", "2626"},
    {"livets_hjul", "2638"},
    {"ansikte_med_sur_mun", "2639"},
    {"kvinnosymbol", "2640"},
    {"manssymbol", "2642"},
    {"vaduren", "2648"},
    {"oxen", "2649"},
    {"skytten", "2650"},
    {"stenbocken", "2651"},
    {"vattumannen", "2652"},
    {"fiskarna", "2653"},
    {"spader", "2660"},
    {"klover", "2663"},
    {"hjarter", "2665"},
    {"ruter", "2666"},
    {"varma_kallor", "2668"},
    {"hammare_och_hacka", "2692"},
    {"ankare", "2693"},
    {"korslagda_svard", "2694"},
    {"medicinsk_symbol", "2695"},
    {"balansvag", "2696"},
    {"retort", "2697"},
    {"kugghjul", "2699"},
    {"sax", "2702"},
    {"vit_bockmarkering", "2705"},
    {"flygplan", "2708"},
    {"kuvert", "2709"},
    {"stiftpenna", "2712"},
    {"bockmarkering", "2714"},
    {"multiplicera", "2716"},
    {"davidsstjarna", "2721"},
    {"gnistrande_stjarnor", "2728"},
    {"attauddig_spetsig_stjarna", "2733"},
    {"attauddig_stjarna", "2734"},
    {"snoflinga", "2744"},
    {"glittra", "2747"},
    {"rott_fragetecken", "2753"},
    {"vitt_fragetecken", "2754"},
    {"vitt_utropstecken", "2755"},
    {"rott_utropstecken", "2757"},
    {"utropstecken_i_form_av_ett_hjarta", "2763"},
    {"rott_hjarta", "2764"},
    {"addera", "2795"},
    {"subtrahera", "2796"},
    {"dividera", "2797"},
    {"svangd_uppatpil", "2934"},
    {"svangd_nedatpil", "2935"},
    {"vagigt_tankstreck", "3030"},
    {"japansk_grattis_knapp", "3297"},
    {"japansk_hemligt_knapp", "3299"},
    {"upphovsrattssymbol", "00A9"},
    {"registrerat_varumarke", "00AE"},
    {"dubbla_utropstecken", "203C"},
    {"svangd_vansterpil", "21A9"},
    {"svangd_hogerpil", "21AA"},
    {"armbandsur", "231A"},
    {"utrunnet_timglas", "231B"},
    {"utmatningsknapp", "23CF"},
    {"framatspolningsknapp", "23E9"},
    {"knapp_for_snabbspola_tillbaka", "23EA"},
    {"dubbla_uppatpilar", "23EB"},
    {"dubbla_nedatpilar", "23EC"},
    {"knapp_for_nasta_spar", "23ED"},
    {"knapp_for_sista_sparet", "23EE"},
    {"knapp_for_spela_eller_pausa", "23EF"},
    {"vackarklocka", "23F0"},
    {"tidtagarur", "23F1"},
    {"timer", "23F2"},
    {"timglas_med_rinnande_sand", "23F3"},
    {"pausknapp", "23F8"},
    {"stoppknapp", "23F9"},
    {"inspelningsknapp", "23FA"},
    {"m_i_en_cirkel", "24C2"},
    {"liten_svart_ruta", "25AA"},
    {"liten_vit_ruta", "25AB"},
    {"uppspelningsknapp", "25B6"},
    {"bakatspolningsknapp", "25C0"},
    {"mellanstor_vit_ruta", "25FB"},
    {"mellanstor_svart_ruta", "25FC"},
    {"mellanliten_vit_ruta", "25FD"},
    {"mellanliten_svart_ruta", "25FE"},
    {"telefon", "260E"},
    {"finger_upp", "261D"},
    {"finger_upp_farg1", "261D-1F3FB"},
    {"finger_upp_farg2", "261D-1F3FC"},
    {"finger_upp_farg3", "261D-1F3FD"},
    {"finger_upp_farg4", "261D-1F3FE"},
    {"finger_upp_farg5", "261D-1F3FF"},
    {"stjarna_och_halvmane", "262A"},
    {"fredssymbol", "262E"},
    {"yin_och_yang", "262F"},
    {"leende_ansikte", "263A"},
    {"tvillingarna", "264A"},
    {"kraftan", "264B"},
    {"lejonet", "264C"},
    {"jungfrun", "264D"},
    {"vagen", "264E"},
    {"skorpionen", "264F"},
    {"schackbonde", "265F"},
    {"atervinningssymbol", "267B"},
    {"oandlighet", "267E"},
    {"rullstolssymbol", "267F"},
    {"symbol_for_atom", "269B"},
    {"fransk_lilja", "269C"},
    {"varning", "26A0"},
    {"hogspanning", "26A1"},
    {"transgendersymbol", "26A7"},
    {"vit_cirkel", "26AA"},
    {"svart_cirkel", "26AB"},
    {"kista", "26B0"},
    {"begravningsurna", "26B1"},
    {"fotboll", "26BD"},
    {"baseboll", "26BE"},
    {"snogubbe_utan_sno", "26C4"},
    {"sol_bakom_moln", "26C5"},
    {"moln_med_regn_och_blixtar", "26C8"},
    {"ormbararen", "26CE"},
    {"hacka", "26CF"},
    {"raddningsarbetarhjalm", "26D1"},
    {"kedjor", "26D3"},
    {"forbud_mot_infart_med_fordon", "26D4"},
    {"shintohelgedom", "26E9"},
    {"kyrka", "26EA"},
    {"berg", "26F0"},
    {"parasoll_pa_marken", "26F1"},
    {"fontan", "26F2"},
    {"flagga_i_hal", "26F3"},
    {"farja", "26F4"},
    {"segelbat", "26F5"},
    {"skidakare", "26F7"},
    {"skridsko", "26F8"},
    {"person_med_boll", "26F9"},
    {"person_med_boll_farg1", "26F9-1F3FB"},
    {"person_med_boll_farg2", "26F9-1F3FC"},
    {"person_med_boll_farg3", "26F9-1F3FD"},
    {"person_med_boll_farg4", "26F9-1F3FE"},
    {"person_med_boll_farg5", "26F9-1F3FF"},
    {"talt", "26FA"},
    {"branslepump", "26FD"},
    {"knytnave_i_luften", "270A"},
    {"knytnave_i_luften_farg1", "270A-1F3FB"},
    {"knytnave_i_luften_farg2", "270A-1F3FC"},
    {"knytnave_i_luften_farg3", "270A-1F3FD"},
    {"knytnave_i_luften_farg4", "270A-1F3FE"},
    {"knytnave_i_luften_farg5", "270A-1F3FF"},
    {"hand_i_luften", "270B"},
    {"hand_i_luften_farg1", "270B-1F3FB"},
    {"hand_i_luften_farg2", "270B-1F3FC"},
    {"hand_i_luften_farg3", "270B-1F3FD"},
    {"hand_i_luften_farg4", "270B-1F3FE"},
    {"hand_i_luften_farg5", "270B-1F3FF"},
    {"v_tecken", "270C"},
    {"v_tecken_farg1", "270C-1F3FB"},
    {"v_tecken_farg2", "270C-1F3FC"},
    {"v_tecken_farg3", "270C-1F3FD"},
    {"v_tecken_farg4", "270C-1F3FE"},
    {"v_tecken_farg5", "270C-1F3FF"},
    {"hand_som_skriver", "270D"},
    {"hand_som_skriver_farg1", "270D-1F3FB"},
    {"hand_som_skriver_farg2", "270D-1F3FC"},
    {"hand_som_skriver_farg3", "270D-1F3FD"},
    {"hand_som_skriver_farg4", "270D-1F3FE"},
    {"hand_som_skriver_farg5", "270D-1F3FF"},
    {"blyertspenna", "270F"},
    {"latinskt_kors", "271D"},
    {"rott_kryss", "274C"},
    {"vitt_kryss_i_ruta", "274E"},
    {"hogerpil", "27A1"},
    {"ogla", "27B0"},
    {"dubbel_ogla", "27BF"},
    {"vansterpil", "2B05"},
    {"uppatpil", "2B06"},
    {"nedatpil", "2B07"},
    {"stor_svart_ruta", "2B1B"},
    {"stor_vit_ruta", "2B1C"},
    {"stjarna", "2B50"},
    {"tung_stor_cirkel", "2B55"},
    {"japansk_kommatering", "303D"},
    {"mahjong_bricka", "1F004"},
    {"joker", "1F0CF"},
    {"blodgrupp_a", "1F170"},
    {"blodgrupp_b", "1F171"},
    {"blodgrupp_0", "1F17E"},
    {"p_knapp", "1F17F"},
    {"blodgrupp_ab", "1F18E"},
    {"rensa_symbol", "1F191"},
    {"cool_knapp", "1F192"},
    {"free_knapp", "1F193"},
    {"id_knapp", "1F194"},
    {"new_knapp", "1F195"},
    {"ng_knapp", "1F196"},
    {"ok_knapp", "1F197"},
    {"sos_knapp", "1F198"},
    {"up_knapp", "1F199"},
    {"vs_knapp", "1F19A"},
    {"regional_indikator_a", "1F1E6"},
    {"regional_indikator_b", "1F1E7"},
    {"regional_indikator_c", "1F1E8"},
    {"regional_indikator_d", "1F1E9"},
    {"regional_indikator_e", "1F1EA"},
    {"regional_indikator_f", "1F1EB"},
    {"regional_indikator_g", "1F1EC"},
    {"regional_indikator_h", "1F1ED"},
    {"regional_indikator_i", "1F1EE"},
    {"regional_indikator_j", "1F1EF"},
    {"regional_indikator_k", "1F1F0"},
    {"regional_indikator_l", "1F1F1"},
    {"regional_indikator_m", "1F1F2"},
    {"regional_indikator_n", "1F1F3"},
    {"regional_indikator_o", "1F1F4"},
    {"regional_indikator_p", "1F1F5"},
    {"regional_indikator_q", "1F1F6"},
    {"regional_indikator_r", "1F1F7"},
    {"regional_indikator_s", "1F1F8"},
    {"regional_indikator_t", "1F1F9"},
    {"regional_indikator_u", "1F1FA"},
    {"regional_indikator_v", "1F1FB"},
    {"regional_indikator_w", "1F1FC"},
    {"regional_indikator_x", "1F1FD"},
    {"regional_indikator_y", "1F1FE"},
    {"regional_indikator_z", "1F1FF"},
    {"japansk_har_knapp", "1F201"},
    {"japansk_serviceavgift_knapp", "1F202"},
    {"japansk_gratis_knapp", "1F21A"},
    {"japansk_reserverad_knapp", "1F22F"},
    {"japansk_forbjudet_knapp", "1F232"},
    {"japansk_lediga_rum_knapp", "1F233"},
    {"japansk_godkant_betyg_knapp", "1F234"},
    {"japansk_fullbelagt_knapp", "1F235"},
    {"japansk_inte_gratis_knapp", "1F236"},
    {"japansk_manadsbelopp_knapp", "1F237"},
    {"japansk_anvandning_knapp", "1F238"},
    {"japansk_rabatt_knapp", "1F239"},
    {"japansk_oppet_knapp", "1F23A"},
    {"japansk_fynd_knapp", "1F250"},
    {"japansk_godtagbart_knapp", "1F251"},
    {"cyklon", "1F300"},
    {"dimmigt", "1F301"},
    {"hopfallt_paraply", "1F302"},
    {"stjarnklar_natt", "1F303"},
    {"soluppgang_over_berg", "1F304"},
    {"soluppgang", "1F305"},
    {"stad_i_skymning", "1F306"},
    {"solnedgang", "1F307"},
    {"regnbage", "1F308"},
    {"bro_pa_natten", "1F309"},
    {"vag", "1F30A"},
    {"vulkan", "1F30B"},
    {"vintergatan", "1F30C"},
    {"jordglob_europa_afrika", "1F30D"},
    {"jordglob_nord_och_sydamerika", "1F30E"},
    {"jordglob_asien_australien", "1F30F"},
    {"jordglob_med_meridianer", "1F310"},
    {"nymane", "1F311"},
    {"mane_i_forsta_kvarteret", "1F312"},
    {"tilltagande_halvmane", "1F313"},
    {"mane_i_andra_kvarteret", "1F314"},
    {"fullmane", "1F315"},
    {"mane_i_tredje_kvarteret", "1F316"},
    {"avtagande_halvmane", "1F317"},
    {"mane_i_fjarde_kvarteret", "1F318"},
    {"manskara", "1F319"},
    {"nymane_med_ansikte", "1F31A"},
    {"tilltagande_halvmane_med_ansikte", "1F31B"},
    {"avtagande_halvmane_med_ansikte", "1F31C"},
    {"fullmane_med_ansikte", "1F31D"},
    {"sol_med_ansikte", "1F31E"},
    {"blankande_stjarna", "1F31F"},
    {"stjarnfall", "1F320"},
    {"termometer", "1F321"},
    {"sol_bakom_litet_moln", "1F324"},
    {"sol_bakom_stort_moln", "1F325"},
    {"sol_bakom_moln_med_regn", "1F326"},
    {"regnmoln", "1F327"},
    {"moln_med_sno", "1F328"},
    {"moln_med_blixt", "1F329"},
    {"tornado", "1F32A"},
    {"dimma", "1F32B"},
    {"blaser_vind", "1F32C"},
    {"varmkorv", "1F32D"},
    {"taco", "1F32E"},
    {"burrito", "1F32F"},
    {"kastanj", "1F330"},
    {"skott", "1F331"},
    {"barrtrad", "1F332"},
    {"lovtrad", "1F333"},
    {"palm", "1F334"},
    {"kaktus", "1F335"},
    {"chili", "1F336"},
    {"tulpan", "1F337"},
    {"korsbarsblomma", "1F338"},
    {"ros", "1F339"},
    {"hibiskus", "1F33A"},
    {"solros", "1F33B"},
    {"blomma", "1F33C"},
    {"majskolv", "1F33D"},
    {"risax", "1F33E"},
    {"ort", "1F33F"},
    {"fyrklover", "1F340"},
    {"lonnlov", "1F341"},
    {"fallande_lov", "1F342"},
    {"lov_i_vinden", "1F343"},
    {"svamp", "1F344"},
    {"tomat", "1F345"},
    {"aubergine", "1F346"},
    {"vindruvor", "1F347"},
    {"melon", "1F348"},
    {"vattenmelon", "1F349"},
    {"mandarin", "1F34A"},
    {"citron", "1F34B"},
    {"banan", "1F34C"},
    {"ananas", "1F34D"},
    {"rott_apple", "1F34E"},
    {"gront_apple", "1F34F"},
    {"paron", "1F350"},
    {"persika", "1F351"},
    {"korsbar", "1F352"},
    {"jordgubbe", "1F353"},
    {"hamburgare", "1F354"},
    {"pizzabit", "1F355"},
    {"kottben", "1F356"},
    {"kycklinglar", "1F357"},
    {"riskaka", "1F358"},
    {"risboll", "1F359"},
    {"kokt_ris", "1F35A"},
    {"curry_och_ris", "1F35B"},
    {"skal_nudelsoppa", "1F35C"},
    {"spagetti", "1F35D"},
    {"brod", "1F35E"},
    {"pommes_frites", "1F35F"},
    {"rostad_sotpotatis", "1F360"},
    {"dango", "1F361"},
    {"skaldjursspett", "1F362"},
    {"sushi", "1F363"},
    {"friterad_raka", "1F364"},
    {"fiskkaka", "1F365"},
    {"mjukglass", "1F366"},
    {"kakigori", "1F367"},
    {"glass", "1F368"},
    {"munk", "1F369"},
    {"kaka", "1F36A"},
    {"chokladkaka", "1F36B"},
    {"godis", "1F36C"},
    {"klubba", "1F36D"},
    {"brylepudding", "1F36E"},
    {"honungsburk", "1F36F"},
    {"tartbit", "1F370"},
    {"bentolada", "1F371"},
    {"gryta", "1F372"},
    {"matlagning", "1F373"},
    {"kniv_och_gaffel", "1F374"},
    {"tekopp", "1F375"},
    {"sake", "1F376"},
    {"vinglas", "1F377"},
    {"cocktailglas", "1F378"},
    {"tropisk_drink", "1F379"},
    {"olstop", "1F37A"},
    {"skalande_olstop", "1F37B"},
    {"nappflaska", "1F37C"},
    {"tallrik_kniv_och_gaffel", "1F37D"},
    {"champagne", "1F37E"},
    {"popcorn", "1F37F"},
    {"rosett", "1F380"},
    {"inslagen_present", "1F381"},
    {"fodelsedagstarta", "1F382"},
    {"lyktgubbe", "1F383"},
    {"julgran", "1F384"},
    {"jultomte", "1F385"},
    {"jultomte_farg1", "1F385-1F3FB"},
    {"jultomte_farg2", "1F385-1F3FC"},
    {"jultomte_farg3", "1F385-1F3FD"},
    {"jultomte_farg4", "1F385-1F3FE"},
    {"jultomte_farg5", "1F385-1F3FF"},
    {"fyrverkerier", "1F386"},
    {"glittrande_fyrverkeri", "1F387"},
    {"ballong", "1F388"},
    {"konfettismallare", "1F389"},
    {"konfettiboll", "1F38A"},
    {"tanabatatrad", "1F38B"},
    {"korsade_flaggor", "1F38C"},
    {"pinjedekoration", "1F38D"},
    {"japanska_dockor", "1F38E"},
    {"karpvindstrut", "1F38F"},
    {"vindspel", "1F390"},
    {"manskadning", "1F391"},
    {"ryggsack", "1F392"},
    {"examenshatt", "1F393"},
    {"militar_medalj", "1F396"},
    {"paminnelseband", "1F397"},
    {"studiomikrofon", "1F399"},
    {"nivareglage", "1F39A"},
    {"reglagevred", "1F39B"},
    {"filmnegativ", "1F39E"},
    {"intradesbiljetter", "1F39F"},
    {"karusellhast", "1F3A0"},
    {"pariserhjul", "1F3A1"},
    {"berg_och_dalbana", "1F3A2"},
    {"fiskespo", "1F3A3"},
    {"mikrofon", "1F3A4"},
    {"filmkamera", "1F3A5"},
    {"bio", "1F3A6"},
    {"horlur", "1F3A7"},
    {"fargpalett", "1F3A8"},
    {"hog_hatt", "1F3A9"},
    {"cirkustalt", "1F3AA"},
    {"biljett", "1F3AB"},
    {"filmklappa", "1F3AC"},
    {"dans_och_teater", "1F3AD"},
    {"tv_spel", "1F3AE"},
    {"mitt_i_prick", "1F3AF"},
    {"enarmad_bandit", "1F3B0"},
    {"biljardboll", "1F3B1"},
    {"speltarning", "1F3B2"},
    {"bowling", "1F3B3"},
    {"spelkort_med_blomma", "1F3B4"},
    {"musiknot", "1F3B5"},
    {"musiknoter", "1F3B6"},
    {"saxofon", "1F3B7"},
    {"gitarr", "1F3B8"},
    {"klaviatur", "1F3B9"},
    {"trumpet", "1F3BA"},
    {"fiol", "1F3BB"},
    {"g_klav", "1F3BC"},
    {"loparlinne", "1F3BD"},
    {"tennis", "1F3BE"},
    {"skidor", "1F3BF"},
    {"basketboll", "1F3C0"},
    {"schackrutig_flagga", "1F3C1"},
    {"snowboardakare", "1F3C2"},
    {"snowboardakare_farg1", "1F3C2-1F3FB"},
    {"snowboardakare_farg2", "1F3C2-1F3FC"},
    {"snowboardakare_farg3", "1F3C2-1F3FD"},
    {"snowboardakare_farg4", "1F3C2-1F3FE"},
    {"snowboardakare_farg5", "1F3C2-1F3FF"},
    {"person_som_springer", "1F3C3"},
    {"person_som_springer_farg1", "1F3C3-1F3FB"},
    {"person_som_springer_farg2", "1F3C3-1F3FC"},
    {"person_som_springer_farg3", "1F3C3-1F3FD"},
    {"person_som_springer_farg4", "1F3C3-1F3FE"},
    {"person_som_springer_farg5", "1F3C3-1F3FF"},
    {"person_som_surfar", "1F3C4"},
    {"person_som_surfar_farg1", "1F3C4-1F3FB"},
    {"person_som_surfar_farg2", "1F3C4-1F3FC"},
    {"person_som_surfar_farg3", "1F3C4-1F3FD"},
    {"person_som_surfar_farg4", "1F3C4-1F3FE"},
    {"person_som_surfar_farg5", "1F3C4-1F3FF"},
    {"sportmedalj", "1F3C5"},
    {"pokal", "1F3C6"},
    {"hastkapplopning", "1F3C7"},
    {"hastkapplopning_farg1", "1F3C7-1F3FB"},
    {"hastkapplopning_farg2", "1F3C7-1F3FC"},
    {"hastkapplopning_farg3", "1F3C7-1F3FD"},
    {"hastkapplopning_farg4", "1F3C7-1F3FE"},
    {"hastkapplopning_farg5", "1F3C7-1F3FF"},
    {"amerikansk_fotboll", "1F3C8"},
    {"rugbyboll", "1F3C9"},
    {"person_som_simmar", "1F3CA"},
    {"person_som_simmar_farg1", "1F3CA-1F3FB"},
    {"person_som_simmar_farg2", "1F3CA-1F3FC"},
    {"person_som_simmar_farg3", "1F3CA-1F3FD"},
    {"person_som_simmar_farg4", "1F3CA-1F3FE"},
    {"person_som_simmar_farg5", "1F3CA-1F3FF"},
    {"person_som_lyfter_tyngder", "1F3CB"},
    {"person_som_lyfter_tyngder_farg1", "1F3CB-1F3FB"},
    {"person_som_lyfter_tyngder_farg2", "1F3CB-1F3FC"},
    {"person_som_lyfter_tyngder_farg3", "1F3CB-1F3FD"},
    {"person_som_lyfter_tyngder_farg4", "1F3CB-1F3FE"},
    {"person_som_lyfter_tyngder_farg5", "1F3CB-1F3FF"},
    {"person_som_spelar_golf", "1F3CC"},
    {"person_som_spelar_golf_farg1", "1F3CC-1F3FB"},
    {"person_som_spelar_golf_farg2", "1F3CC-1F3FC"},
    {"person_som_spelar_golf_farg3", "1F3CC-1F3FD"},
    {"person_som_spelar_golf_farg4", "1F3CC-1F3FE"},
    {"person_som_spelar_golf_farg5", "1F3CC-1F3FF"},
    {"motorcykel", "1F3CD"},
    {"racerbil", "1F3CE"},
    {"cricket", "1F3CF"},
    {"volleyboll", "1F3D0"},
    {"landhockey", "1F3D1"},
    {"ishockey", "1F3D2"},
    {"bordtennis", "1F3D3"},
    {"snoklatt_berg", "1F3D4"},
    {"camping", "1F3D5"},
    {"strand_med_parasoll", "1F3D6"},
    {"byggarbetsplats", "1F3D7"},
    {"flera_hus", "1F3D8"},
    {"stadsbild", "1F3D9"},
    {"fallfardigt_hus", "1F3DA"},
    {"klassisk_byggnad", "1F3DB"},
    {"oken", "1F3DC"},
    {"ode_o", "1F3DD"},
    {"nationalpark", "1F3DE"},
    {"stadion", "1F3DF"},
    {"hus", "1F3E0"},
    {"hus_med_tradgard", "1F3E1"},
    {"kontorsbyggnad", "1F3E2"},
    {"japanskt_postkontor", "1F3E3"},
    {"postkontor", "1F3E4"},
    {"sjukhus", "1F3E5"},
    {"bank", "1F3E6"},
    {"bankomat", "1F3E7"},
    {"hotell", "1F3E8"},
    {"karlekshotell", "1F3E9"},
    {"jourbutik", "1F3EA"},
    {"skola", "1F3EB"},
    {"varuhus", "1F3EC"},
    {"fabrik", "1F3ED"},
    {"rod_papperslykta", "1F3EE"},
    {"japanskt_slott", "1F3EF"},
    {"slott", "1F3F0"},
    {"vit_flagga", "1F3F3"},
    {"svart_flagga", "1F3F4"},
    {"bandros", "1F3F5"},
    {"etikett", "1F3F7"},
    {"badminton", "1F3F8"},
    {"pil_och_bage", "1F3F9"},
    {"amfora", "1F3FA"},
    {"ljus_hy", "1F3FB"},
    {"mellanljus_hy", "1F3FC"},
    {"mellanfargad_hy", "1F3FD"},
    {"mellanmork_hy", "1F3FE"},
    {"mork_hy", "1F3FF"},
    {"ratta", "1F400"},
    {"mus", "1F401"},
    {"oxe", "1F402"},
    {"vattenbuffel", "1F403"},
    {"ko", "1F404"},
    {"tiger", "1F405"},
    {"leopard", "1F406"},
    {"kanin", "1F407"},
    {"katt", "1F408"},
    {"drake", "1F409"},
    {"krokodil", "1F40A"},
    {"val", "1F40B"},
    {"snigel", "1F40C"},
    {"orm", "1F40D"},
    {"hast", "1F40E"},
    {"vadur", "1F40F"},
    {"get", "1F410"},
    {"tacka", "1F411"},
    {"apa", "1F412"},
    {"tupp", "1F413"},
    {"hona", "1F414"},
    {"hund", "1F415"},
    {"gris", "1F416"},
    {"vildsvin", "1F417"},
    {"elefant", "1F418"},
    {"blackfisk", "1F419"},
    {"snacka", "1F41A"},
    {"tusenfoting", "1F41B"},
    {"myra", "1F41C"},
    {"bi", "1F41D"},
    {"nyckelpiga", "1F41E"},
    {"fisk", "1F41F"},
    {"tropisk_fisk", "1F420"},
    {"blasfisk", "1F421"},
    {"skoldpadda", "1F422"},
    {"nyklackt_kyckling", "1F423"},
    {"kyckling", "1F424"},
    {"kyckling_framifran", "1F425"},
    {"fagel", "1F426"},
    {"pingvin", "1F427"},
    {"koala", "1F428"},
    {"pudel", "1F429"},
    {"dromedar", "1F42A"},
    {"kamel", "1F42B"},
    {"delfin", "1F42C"},
    {"musansikte", "1F42D"},
    {"koansikte", "1F42E"},
    {"tigeransikte", "1F42F"},
    {"kaninansikte", "1F430"},
    {"kattansikte", "1F431"},
    {"drakansikte", "1F432"},
    {"val_som_sprutar_vatten", "1F433"},
    {"hastansikte", "1F434"},
    {"apansikte", "1F435"},
    {"hundansikte", "1F436"},
    {"grisansikte", "1F437"},
    {"grodansikte", "1F438"},
    {"hamsteransikte", "1F439"},
    {"vargansikte", "1F43A"},
    {"bjornansikte", "1F43B"},
    {"pandaansikte", "1F43C"},
    {"gristryne", "1F43D"},
    {"tassavtryck", "1F43E"},
    {"jordekorre", "1F43F"},
    {"ogon", "1F440"},
    {"oga", "1F441"},
    {"ora", "1F442"},
    {"ora_farg1", "1F442-1F3FB"},
    {"ora_farg2", "1F442-1F3FC"},
    {"ora_farg3", "1F442-1F3FD"},
    {"ora_farg4", "1F442-1F3FE"},
    {"ora_farg5", "1F442-1F3FF"},
    {"nasa", "1F443"},
    {"nasa_farg1", "1F443-1F3FB"},
    {"nasa_farg2", "1F443-1F3FC"},
    {"nasa_farg3", "1F443-1F3FD"},
    {"nasa_farg4", "1F443-1F3FE"},
    {"nasa_farg5", "1F443-1F3FF"},
    {"mun", "1F444"},
    {"tunga", "1F445"},
    {"pekfinger_upp", "1F446"},
    {"pekfinger_upp_farg1", "1F446-1F3FB"},
    {"pekfinger_upp_farg2", "1F446-1F3FC"},
    {"pekfinger_upp_farg3", "1F446-1F3FD"},
    {"pekfinger_upp_farg4", "1F446-1F3FE"},
    {"pekfinger_upp_farg5", "1F446-1F3FF"},
    {"nedatpekande_finger", "1F447"},
    {"nedatpekande_finger_farg1", "1F447-1F3FB"},
    {"nedatpekande_finger_farg2", "1F447-1F3FC"},
    {"nedatpekande_finger_farg3", "1F447-1F3FD"},
    {"nedatpekande_finger_farg4", "1F447-1F3FE"},
    {"nedatpekande_finger_farg5", "1F447-1F3FF"},
    {"vansterpekande_finger", "1F448"},
    {"vansterpekande_finger_farg1", "1F448-1F3FB"},
    {"vansterpekande_finger_farg2", "1F448-1F3FC"},
    {"vansterpekande_finger_farg3", "1F448-1F3FD"},
    {"vansterpekande_finger_farg4", "1F448-1F3FE"},
    {"vansterpekande_finger_farg5", "1F448-1F3FF"},
    {"hogerpekande_finger", "1F449"},
    {"hogerpekande_finger_farg1", "1F449-1F3FB"},
    {"hogerpekande_finger_farg2", "1F449-1F3FC"},
    {"hogerpekande_finger_farg3", "1F449-1F3FD"},
    {"hogerpekande_finger_farg4", "1F449-1F3FE"},
    {"hogerpekande_finger_farg5", "1F449-1F3FF"},
    {"knuten_hand", "1F44A"},
    {"knuten_hand_farg1", "1F44A-1F3FB"},
    {"knuten_hand_farg2", "1F44A-1F3FC"},
    {"knuten_hand_farg3", "1F44A-1F3FD"},
    {"knuten_hand_farg4", "1F44A-1F3FE"},
    {"knuten_hand_farg5", "1F44A-1F3FF"},
    {"vinkande_hand", "1F44B"},
    {"vinkande_hand_farg1", "1F44B-1F3FB"},
    {"vinkande_hand_farg2", "1F44B-1F3FC"},
    {"vinkande_hand_farg3", "1F44B-1F3FD"},
    {"vinkande_hand_farg4", "1F44B-1F3FE"},
    {"vinkande_hand_farg5", "1F44B-1F3FF"},
    {"ok_tecken", "1F44C"},
    {"ok_tecken_farg1", "1F44C-1F3FB"},
    {"ok_tecken_farg2", "1F44C-1F3FC"},
    {"ok_tecken_farg3", "1F44C-1F3FD"},
    {"ok_tecken_farg4", "1F44C-1F3FE"},
    {"ok_tecken_farg5", "1F44C-1F3FF"},
    {"tummen_upp", "1F44D"},
    {"tummen_upp_farg1", "1F44D-1F3FB"},
    {"tummen_upp_farg2", "1F44D-1F3FC"},
    {"tummen_upp_farg3", "1F44D-1F3FD"},
    {"tummen_upp_farg4", "1F44D-1F3FE"},
    {"tummen_upp_farg5", "1F44D-1F3FF"},
    {"tummen_ned", "1F44E"},
    {"tummen_ned_farg1", "1F44E-1F3FB"},
    {"tummen_ned_farg2", "1F44E-1F3FC"},
    {"tummen_ned_farg3", "1F44E-1F3FD"},
    {"tummen_ned_farg4", "1F44E-1F3FE"},
    {"tummen_ned_farg5", "1F44E-1F3FF"},
    {"klappande_hander", "1F44F"},
    {"klappande_hander_farg1", "1F44F-1F3FB"},
    {"klappande_hander_farg2", "1F44F-1F3FC"},
    {"klappande_hander_farg3", "1F44F-1F3FD"},
    {"klappande_hander_farg4", "1F44F-1F3FE"},
    {"klappande_hander_farg5", "1F44F-1F3FF"},
    {"oppna_hander", "1F450"},
    {"oppna_hander_farg1", "1F450-1F3FB"},
    {"oppna_hander_farg2", "1F450-1F3FC"},
    {"oppna_hander_farg3", "1F450-1F3FD"},
    {"oppna_hander_farg4", "1F450-1F3FE"},
    {"oppna_hander_farg5", "1F450-1F3FF"},
    {"krona", "1F451"},
    {"damhatt", "1F452"},
    {"glasogon", "1F453"},
    {"skjorta_med_slips", "1F454"},
    {"t_shirt", "1F455"},
    {"jeans", "1F456"},
    {"klanning", "1F457"},
    {"kimono", "1F458"},
    {"bikini", "1F459"},
    {"blus", "1F45A"},
    {"portmonna", "1F45B"},
    {"handvaska", "1F45C"},
    {"kuvertvaska", "1F45D"},
    {"herrsko", "1F45E"},
    {"gympasko", "1F45F"},
    {"hogklackad_sko", "1F460"},
    {"damsandal", "1F461"},
    {"damstovel", "1F462"},
    {"fotavtryck", "1F463"},
    {"figur_i_silhuett", "1F464"},
    {"figurer_i_silhuett", "1F465"},
    {"pojke", "1F466"},
    {"pojke_farg1", "1F466-1F3FB"},
    {"pojke_farg2", "1F466-1F3FC"},
    {"pojke_farg3", "1F466-1F3FD"},
    {"pojke_farg4", "1F466-1F3FE"},
    {"pojke_farg5", "1F466-1F3FF"},
    {"flicka", "1F467"},
    {"flicka_farg1", "1F467-1F3FB"},
    {"flicka_farg2", "1F467-1F3FC"},
    {"flicka_farg3", "1F467-1F3FD"},
    {"flicka_farg4", "1F467-1F3FE"},
    {"flicka_farg5", "1F467-1F3FF"},
    {"man", "1F468"},
    {"man_farg1", "1F468-1F3FB"},
    {"man_farg2", "1F468-1F3FC"},
    {"man_farg3", "1F468-1F3FD"},
    {"man_farg4", "1F468-1F3FE"},
    {"man_farg5", "1F468-1F3FF"},
    {"kvinna", "1F469"},
    {"kvinna_farg1", "1F469-1F3FB"},
    {"kvinna_farg2", "1F469-1F3FC"},
    {"kvinna_farg3", "1F469-1F3FD"},
    {"kvinna_farg4", "1F469-1F3FE"},
    {"kvinna_farg5", "1F469-1F3FF"},
    {"familj", "1F46A"},
    {"man_och_kvinna_haller_hand", "1F46B"},
    {"man_och_kvinna_haller_hand_farg1", "1F46B-1F3FB"},
    {"man_och_kvinna_haller_hand_farg2", "1F46B-1F3FC"},
    {"man_och_kvinna_haller_hand_farg3", "1F46B-1F3FD"},
    {"man_och_kvinna_haller_hand_farg4", "1F46B-1F3FE"},
    {"man_och_kvinna_haller_hand_farg5", "1F46B-1F3FF"},
    {"man_och_kvinna_haller_hand_farg1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"man_och_kvinna_haller_hand_farg1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"man_och_kvinna_haller_hand_farg1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"man_och_kvinna_haller_hand_farg1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"man_och_kvinna_haller_hand_farg2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"man_och_kvinna_haller_hand_farg2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"man_och_kvinna_haller_hand_farg2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"man_och_kvinna_haller_hand_farg2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"man_och_kvinna_haller_hand_farg3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"man_och_kvinna_haller_hand_farg3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"man_och_kvinna_haller_hand_farg3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"man_och_kvinna_haller_hand_farg3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"man_och_kvinna_haller_hand_farg4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"man_och_kvinna_haller_hand_farg4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"man_och_kvinna_haller_hand_farg4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"man_och_kvinna_haller_hand_farg4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"man_och_kvinna_haller_hand_farg5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"man_och_kvinna_haller_hand_farg5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"man_och_kvinna_haller_hand_farg5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"man_och_kvinna_haller_hand_farg5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"tva_man_haller_hand", "1F46C"},
    {"tva_man_haller_hand_farg1", "1F46C-1F3FB"},
    {"tva_man_haller_hand_farg2", "1F46C-1F3FC"},
    {"tva_man_haller_hand_farg3", "1F46C-1F3FD"},
    {"tva_man_haller_hand_farg4", "1F46C-1F3FE"},
    {"tva_man_haller_hand_farg5", "1F46C-1F3FF"},
    {"tva_man_haller_hand_farg1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"tva_man_haller_hand_farg1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"tva_man_haller_hand_farg1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"tva_man_haller_hand_farg1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"tva_man_haller_hand_farg2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"tva_man_haller_hand_farg2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"tva_man_haller_hand_farg2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"tva_man_haller_hand_farg2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"tva_man_haller_hand_farg3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"tva_man_haller_hand_farg3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"tva_man_haller_hand_farg3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"tva_man_haller_hand_farg3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"tva_man_haller_hand_farg4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"tva_man_haller_hand_farg4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"tva_man_haller_hand_farg4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"tva_man_haller_hand_farg4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"tva_man_haller_hand_farg5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"tva_man_haller_hand_farg5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"tva_man_haller_hand_farg5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"tva_man_haller_hand_farg5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"tva_kvinnor_haller_hand", "1F46D"},
    {"tva_kvinnor_haller_hand_farg1", "1F46D-1F3FB"},
    {"tva_kvinnor_haller_hand_farg2", "1F46D-1F3FC"},
    {"tva_kvinnor_haller_hand_farg3", "1F46D-1F3FD"},
    {"tva_kvinnor_haller_hand_farg4", "1F46D-1F3FE"},
    {"tva_kvinnor_haller_hand_farg5", "1F46D-1F3FF"},
    {"tva_kvinnor_haller_hand_farg1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"tva_kvinnor_haller_hand_farg1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"tva_kvinnor_haller_hand_farg1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"tva_kvinnor_haller_hand_farg1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"tva_kvinnor_haller_hand_farg2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"tva_kvinnor_haller_hand_farg2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"tva_kvinnor_haller_hand_farg2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"tva_kvinnor_haller_hand_farg2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"tva_kvinnor_haller_hand_farg3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"tva_kvinnor_haller_hand_farg3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"tva_kvinnor_haller_hand_farg3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"tva_kvinnor_haller_hand_farg3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"tva_kvinnor_haller_hand_farg4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"tva_kvinnor_haller_hand_farg4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"tva_kvinnor_haller_hand_farg4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"tva_kvinnor_haller_hand_farg4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"tva_kvinnor_haller_hand_farg5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"tva_kvinnor_haller_hand_farg5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"tva_kvinnor_haller_hand_farg5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"tva_kvinnor_haller_hand_farg5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"polis", "1F46E"},
    {"polis_farg1", "1F46E-1F3FB"},
    {"polis_farg2", "1F46E-1F3FC"},
    {"polis_farg3", "1F46E-1F3FD"},
    {"polis_farg4", "1F46E-1F3FE"},
    {"polis_farg5", "1F46E-1F3FF"},
    {"personer_med_kaninoron", "1F46F"},
    {"personer_med_kaninoron_farg1", "1F46F-1F3FB"},
    {"personer_med_kaninoron_farg2", "1F46F-1F3FC"},
    {"personer_med_kaninoron_farg3", "1F46F-1F3FD"},
    {"personer_med_kaninoron_farg4", "1F46F-1F3FE"},
    {"personer_med_kaninoron_farg5", "1F46F-1F3FF"},
    {"personer_med_kaninoron_farg1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninoron_farg1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninoron_farg1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninoron_farg1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninoron_farg2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninoron_farg2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninoron_farg2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninoron_farg2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninoron_farg3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninoron_farg3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninoron_farg3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninoron_farg3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninoron_farg4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninoron_farg4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninoron_farg4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninoron_farg4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninoron_farg5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninoron_farg5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninoron_farg5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninoron_farg5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"person_i_brudsloja", "1F470"},
    {"person_i_brudsloja_farg1", "1F470-1F3FB"},
    {"person_i_brudsloja_farg2", "1F470-1F3FC"},
    {"person_i_brudsloja_farg3", "1F470-1F3FD"},
    {"person_i_brudsloja_farg4", "1F470-1F3FE"},
    {"person_i_brudsloja_farg5", "1F470-1F3FF"},
    {"blond_person", "1F471"},
    {"blond_person_farg1", "1F471-1F3FB"},
    {"blond_person_farg2", "1F471-1F3FC"},
    {"blond_person_farg3", "1F471-1F3FD"},
    {"blond_person_farg4", "1F471-1F3FE"},
    {"blond_person_farg5", "1F471-1F3FF"},
    {"man_med_kinesisk_hatt", "1F472"},
    {"man_med_kinesisk_hatt_farg1", "1F472-1F3FB"},
    {"man_med_kinesisk_hatt_farg2", "1F472-1F3FC"},
    {"man_med_kinesisk_hatt_farg3", "1F472-1F3FD"},
    {"man_med_kinesisk_hatt_farg4", "1F472-1F3FE"},
    {"man_med_kinesisk_hatt_farg5", "1F472-1F3FF"},
    {"man_med_turban", "1F473"},
    {"man_med_turban_farg1", "1F473-1F3FB"},
    {"man_med_turban_farg2", "1F473-1F3FC"},
    {"man_med_turban_farg3", "1F473-1F3FD"},
    {"man_med_turban_farg4", "1F473-1F3FE"},
    {"man_med_turban_farg5", "1F473-1F3FF"},
    {"gammal_man", "1F474"},
    {"gammal_man_farg1", "1F474-1F3FB"},
    {"gammal_man_farg2", "1F474-1F3FC"},
    {"gammal_man_farg3", "1F474-1F3FD"},
    {"gammal_man_farg4", "1F474-1F3FE"},
    {"gammal_man_farg5", "1F474-1F3FF"},
    {"gammal_kvinna", "1F475"},
    {"gammal_kvinna_farg1", "1F475-1F3FB"},
    {"gammal_kvinna_farg2", "1F475-1F3FC"},
    {"gammal_kvinna_farg3", "1F475-1F3FD"},
    {"gammal_kvinna_farg4", "1F475-1F3FE"},
    {"gammal_kvinna_farg5", "1F475-1F3FF"},
    {"bebis", "1F476"},
    {"bebis_farg1", "1F476-1F3FB"},
    {"bebis_farg2", "1F476-1F3FC"},
    {"bebis_farg3", "1F476-1F3FD"},
    {"bebis_farg4", "1F476-1F3FE"},
    {"bebis_farg5", "1F476-1F3FF"},
    {"byggarbetare", "1F477"},
    {"byggarbetare_farg1", "1F477-1F3FB"},
    {"byggarbetare_farg2", "1F477-1F3FC"},
    {"byggarbetare_farg3", "1F477-1F3FD"},
    {"byggarbetare_farg4", "1F477-1F3FE"},
    {"byggarbetare_farg5", "1F477-1F3FF"},
    {"prinsessa", "1F478"},
    {"prinsessa_farg1", "1F478-1F3FB"},
    {"prinsessa_farg2", "1F478-1F3FC"},
    {"prinsessa_farg3", "1F478-1F3FD"},
    {"prinsessa_farg4", "1F478-1F3FE"},
    {"prinsessa_farg5", "1F478-1F3FF"},
    {"japanskt_monster", "1F479"},
    {"japanskt_troll", "1F47A"},
    {"spoke", "1F47B"},
    {"bebisangel", "1F47C"},
    {"bebisangel_farg1", "1F47C-1F3FB"},
    {"bebisangel_farg2", "1F47C-1F3FC"},
    {"bebisangel_farg3", "1F47C-1F3FD"},
    {"bebisangel_farg4", "1F47C-1F3FE"},
    {"bebisangel_farg5", "1F47C-1F3FF"},
    {"utomjording", "1F47D"},
    {"utomjordiskt_monster", "1F47E"},
    {"argt_ansikte_med_horn", "1F47F"},
    {"dodskalle", "1F480"},
    {"receptionist", "1F481"},
    {"receptionist_farg1", "1F481-1F3FB"},
    {"receptionist_farg2", "1F481-1F3FC"},
    {"receptionist_farg3", "1F481-1F3FD"},
    {"receptionist_farg4", "1F481-1F3FE"},
    {"receptionist_farg5", "1F481-1F3FF"},
    {"vakt", "1F482"},
    {"vakt_farg1", "1F482-1F3FB"},
    {"vakt_farg2", "1F482-1F3FC"},
    {"vakt_farg3", "1F482-1F3FD"},
    {"vakt_farg4", "1F482-1F3FE"},
    {"vakt_farg5", "1F482-1F3FF"},
    {"dansande_kvinna", "1F483"},
    {"dansande_kvinna_farg1", "1F483-1F3FB"},
    {"dansande_kvinna_farg2", "1F483-1F3FC"},
    {"dansande_kvinna_farg3", "1F483-1F3FD"},
    {"dansande_kvinna_farg4", "1F483-1F3FE"},
    {"dansande_kvinna_farg5", "1F483-1F3FF"},
    {"lappstift", "1F484"},
    {"nagellack", "1F485"},
    {"nagellack_farg1", "1F485-1F3FB"},
    {"nagellack_farg2", "1F485-1F3FC"},
    {"nagellack_farg3", "1F485-1F3FD"},
    {"nagellack_farg4", "1F485-1F3FE"},
    {"nagellack_farg5", "1F485-1F3FF"},
    {"person_som_blir_masserad", "1F486"},
    {"person_som_blir_masserad_farg1", "1F486-1F3FB"},
    {"person_som_blir_masserad_farg2", "1F486-1F3FC"},
    {"person_som_blir_masserad_farg3", "1F486-1F3FD"},
    {"person_som_blir_masserad_farg4", "1F486-1F3FE"},
    {"person_som_blir_masserad_farg5", "1F486-1F3FF"},
    {"person_som_far_haret_klippt", "1F487"},
    {"person_som_far_haret_klippt_farg1", "1F487-1F3FB"},
    {"person_som_far_haret_klippt_farg2", "1F487-1F3FC"},
    {"person_som_far_haret_klippt_farg3", "1F487-1F3FD"},
    {"person_som_far_haret_klippt_farg4", "1F487-1F3FE"},
    {"person_som_far_haret_klippt_farg5", "1F487-1F3FF"},
    {"barberarstolpe", "1F488"},
    {"spruta", "1F489"},
    {"piller", "1F48A"},
    {"lappavtryck", "1F48B"},
    {"karleksbrev", "1F48C"},
    {"ring", "1F48D"},
    {"adelsten", "1F48E"},
    {"puss", "1F48F"},
    {"puss_farg1", "1F48F-1F3FB"},
    {"puss_farg2", "1F48F-1F3FC"},
    {"puss_farg3", "1F48F-1F3FD"},
    {"puss_farg4", "1F48F-1F3FE"},
    {"puss_farg5", "1F48F-1F3FF"},
    {"puss_farg1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"puss_farg1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"puss_farg1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"puss_farg1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"puss_farg2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"puss_farg2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"puss_farg2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"puss_farg2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"puss_farg3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"puss_farg3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"puss_farg3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"puss_farg3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"puss_farg4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"puss_farg4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"puss_farg4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"puss_farg4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"puss_farg5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"puss_farg5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"puss_farg5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"puss_farg5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"blombukett", "1F490"},
    {"par_med_hjarta", "1F491"},
    {"par_med_hjarta_farg1", "1F491-1F3FB"},
    {"par_med_hjarta_farg2", "1F491-1F3FC"},
    {"par_med_hjarta_farg3", "1F491-1F3FD"},
    {"par_med_hjarta_farg4", "1F491-1F3FE"},
    {"par_med_hjarta_farg5", "1F491-1F3FF"},
    {"par_med_hjarta_farg1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjarta_farg1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjarta_farg1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"par_med_hjarta_farg1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjarta_farg2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjarta_farg2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjarta_farg2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"par_med_hjarta_farg2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjarta_farg3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjarta_farg3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjarta_farg3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"par_med_hjarta_farg3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjarta_farg4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjarta_farg4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjarta_farg4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjarta_farg4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjarta_farg5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjarta_farg5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjarta_farg5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjarta_farg5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"brollop", "1F492"},
    {"bultande_hjarta", "1F493"},
    {"brustet_hjarta", "1F494"},
    {"tva_hjartan", "1F495"},
    {"gnistrande_hjarta", "1F496"},
    {"vaxande_hjarta", "1F497"},
    {"hjarta_med_pil", "1F498"},
    {"blatt_hjarta", "1F499"},
    {"gront_hjarta", "1F49A"},
    {"gult_hjarta", "1F49B"},
    {"lila_hjarta", "1F49C"},
    {"hjarta_med_rosett", "1F49D"},
    {"snurrande_hjartan", "1F49E"},
    {"hjartdekoration", "1F49F"},
    {"diamantform_med_en_prick", "1F4A0"},
    {"glodlampa", "1F4A1"},
    {"symbol_for_ilska", "1F4A2"},
    {"bomb", "1F4A3"},
    {"snarkningar", "1F4A4"},
    {"explosion", "1F4A5"},
    {"stankande_svett", "1F4A6"},
    {"droppe", "1F4A7"},
    {"fartsymbol", "1F4A8"},
    {"bajshog", "1F4A9"},
    {"spand_biceps", "1F4AA"},
    {"spand_biceps_farg1", "1F4AA-1F3FB"},
    {"spand_biceps_farg2", "1F4AA-1F3FC"},
    {"spand_biceps_farg3", "1F4AA-1F3FD"},
    {"spand_biceps_farg4", "1F4AA-1F3FE"},
    {"spand_biceps_farg5", "1F4AA-1F3FF"},
    {"omtumlad", "1F4AB"},
    {"pratbubbla", "1F4AC"},
    {"tankebubbla", "1F4AD"},
    {"vit_blomma", "1F4AE"},
    {"100_poang", "1F4AF"},
    {"pengapase", "1F4B0"},
    {"valutavaxling", "1F4B1"},
    {"dollartecken", "1F4B2"},
    {"kreditkort", "1F4B3"},
    {"yensedel", "1F4B4"},
    {"dollarsedel", "1F4B5"},
    {"eurosedel", "1F4B6"},
    {"pundsedel", "1F4B7"},
    {"pengar_med_vingar", "1F4B8"},
    {"stigande_diagram_med_yen", "1F4B9"},
    {"flygstol", "1F4BA"},
    {"barbar_dator", "1F4BB"},
    {"portfolj", "1F4BC"},
    {"datordiskett", "1F4BD"},
    {"diskett", "1F4BE"},
    {"optisk_skiva", "1F4BF"},
    {"dvd_skiva", "1F4C0"},
    {"arkivmapp", "1F4C1"},
    {"oppen_mapp", "1F4C2"},
    {"upprullad_sida", "1F4C3"},
    {"sida_med_text_uppat", "1F4C4"},
    {"kalender", "1F4C5"},
    {"avdragbar_kalender", "1F4C6"},
    {"kortregister", "1F4C7"},
    {"uppatgaende_trend", "1F4C8"},
    {"nedatgaende_trend", "1F4C9"},
    {"stapeldiagram", "1F4CA"},
    {"skrivplatta", "1F4CB"},
    {"kartnal", "1F4CC"},
    {"knappnal", "1F4CD"},
    {"gem", "1F4CE"},
    {"linjal", "1F4CF"},
    {"ratvinklig_linjal", "1F4D0"},
    {"bokmarkesflikar", "1F4D1"},
    {"liggare", "1F4D2"},
    {"anteckningsbok", "1F4D3"},
    {"anteckningsbok_med_dekorativt_omslag", "1F4D4"},
    {"stangd_bok", "1F4D5"},
    {"oppen_bok", "1F4D6"},
    {"gron_bok", "1F4D7"},
    {"bla_bok", "1F4D8"},
    {"orange_bok", "1F4D9"},
    {"bocker", "1F4DA"},
    {"namnskylt", "1F4DB"},
    {"bokrulle", "1F4DC"},
    {"kom_ihag_block", "1F4DD"},
    {"telefonlur", "1F4DE"},
    {"personsokare", "1F4DF"},
    {"fax", "1F4E0"},
    {"tv_satellit", "1F4E1"},
    {"handhogtalare", "1F4E2"},
    {"megafon", "1F4E3"},
    {"utkorg", "1F4E4"},
    {"inkorg", "1F4E5"},
    {"paket", "1F4E6"},
    {"e_post", "1F4E7"},
    {"inkommande_brev", "1F4E8"},
    {"kuvert_med_pil", "1F4E9"},
    {"stangd_brevlada_med_sankt_flagga", "1F4EA"},
    {"stangd_brevlada_med_hissad_flagga", "1F4EB"},
    {"oppen_brevlada_med_hissad_flagga", "1F4EC"},
    {"oppen_brevlada_med_sankt_flagga", "1F4ED"},
    {"postlada", "1F4EE"},
    {"postsymbol", "1F4EF"},
    {"tidning", "1F4F0"},
    {"mobiltelefon", "1F4F1"},
    {"mobiltelefon_med_pil", "1F4F2"},
    {"vibrationslage", "1F4F3"},
    {"avstangd_mobil", "1F4F4"},
    {"mobil_forbjuden", "1F4F5"},
    {"stapel_signalstyrka", "1F4F6"},
    {"kamera", "1F4F7"},
    {"kamera_med_blixt", "1F4F8"},
    {"videokamera", "1F4F9"},
    {"tv", "1F4FA"},
    {"radio", "1F4FB"},
    {"videokassett", "1F4FC"},
    {"filmprojektor", "1F4FD"},
    {"radband", "1F4FF"},
    {"blanda_latar", "1F500"},
    {"upprepning", "1F501"},
    {"upprepa_en_lat", "1F502"},
    {"vertikala_pilar_medurs", "1F503"},
    {"pilar_moturs", "1F504"},
    {"lag_ljusstyrka", "1F505"},
    {"hog_ljusstyrka", "1F506"},
    {"avstangd_hogtalare", "1F507"},
    {"hogtalare", "1F508"},
    {"hogtalare_med_medium_volym", "1F509"},
    {"hogtalare_med_hog_volym", "1F50A"},
    {"batteri", "1F50B"},
    {"elkontakt", "1F50C"},
    {"forstoringsglas_som_lutar_at_vanster", "1F50D"},
    {"forstoringsglas_som_lutar_at_hoger", "1F50E"},
    {"las_med_blackpenna", "1F50F"},
    {"las_med_nyckel", "1F510"},
    {"nyckel", "1F511"},
    {"last_las", "1F512"},
    {"oppet_las", "1F513"},
    {"ringklocka", "1F514"},
    {"overstruken_ringklocka", "1F515"},
    {"bokmarke", "1F516"},
    {"lank", "1F517"},
    {"alternativknapp", "1F518"},
    {"back_med_vansterpil", "1F519"},
    {"end_med_hogerpil", "1F51A"},
    {"on_med_pil_hoger_och_vanster_ovanfor", "1F51B"},
    {"soon_med_hogerpil_ovanfor", "1F51C"},
    {"top_med_uppatpil", "1F51D"},
    {"ej_for_minderariga", "1F51E"},
    {"bookmarkwater_pistol", "1F51F"},
    {"versaler", "1F520"},
    {"gemener", "1F521"},
    {"siffror", "1F522"},
    {"specialtecken", "1F523"},
    {"bokstaver", "1F524"},
    {"eld", "1F525"},
    {"ficklampa", "1F526"},
    {"skiftnyckel", "1F527"},
    {"hammare", "1F528"},
    {"skruv_och_mutter", "1F529"},
    {"kokskniv", "1F52A"},
    {"vattenpistol", "1F52B"},
    {"mikroskop", "1F52C"},
    {"teleskop", "1F52D"},
    {"kristallkula", "1F52E"},
    {"sexuddig_stjarna_med_prick", "1F52F"},
    {"japansk_symbol_for_nyborjare", "1F530"},
    {"treudd", "1F531"},
    {"svart_fyrkantig_knapp", "1F532"},
    {"vit_fyrkantig_knapp", "1F533"},
    {"rod_cirkel", "1F534"},
    {"bla_cirkel", "1F535"},
    {"stor_orange_diamantform", "1F536"},
    {"stor_bla_diamantform", "1F537"},
    {"liten_orange_diamantform", "1F538"},
    {"liten_bla_romb", "1F539"},
    {"uppatpekande_rod_triangel", "1F53A"},
    {"nedatpekande_rod_triangel", "1F53B"},
    {"uppatknapp", "1F53C"},
    {"nedatknapp", "1F53D"},
    {"aum_tecken", "1F549"},
    {"duva", "1F54A"},
    {"kaba", "1F54B"},
    {"moske", "1F54C"},
    {"synagoga", "1F54D"},
    {"menora", "1F54E"},
    {"klockan_ett", "1F550"},
    {"klockan_tva", "1F551"},
    {"klockan_tre", "1F552"},
    {"klockan_fyra", "1F553"},
    {"klockan_fem", "1F554"},
    {"klockan_sex", "1F555"},
    {"klockan_sju", "1F556"},
    {"klockan_atta", "1F557"},
    {"klockan_nio", "1F558"},
    {"klockan_tio", "1F559"},
    {"klockan_elva", "1F55A"},
    {"klockan_tolv", "1F55B"},
    {"halv_tva", "1F55C"},
    {"halv_tre", "1F55D"},
    {"halv_fyra", "1F55E"},
    {"halv_fem", "1F55F"},
    {"halv_sex", "1F560"},
    {"halv_sju", "1F561"},
    {"halv_atta", "1F562"},
    {"halv_nio", "1F563"},
    {"halv_tio", "1F564"},
    {"halv_elva", "1F565"},
    {"halv_tolv", "1F566"},
    {"halv_ett", "1F567"},
    {"stearinljus", "1F56F"},
    {"mantelklocka", "1F570"},
    {"hal", "1F573"},
    {"svavande_man_i_kostym", "1F574"},
    {"svavande_man_i_kostym_farg1", "1F574-1F3FB"},
    {"svavande_man_i_kostym_farg2", "1F574-1F3FC"},
    {"svavande_man_i_kostym_farg3", "1F574-1F3FD"},
    {"svavande_man_i_kostym_farg4", "1F574-1F3FE"},
    {"svavande_man_i_kostym_farg5", "1F574-1F3FF"},
    {"detektiv", "1F575"},
    {"detektiv_farg1", "1F575-1F3FB"},
    {"detektiv_farg2", "1F575-1F3FC"},
    {"detektiv_farg3", "1F575-1F3FD"},
    {"detektiv_farg4", "1F575-1F3FE"},
    {"detektiv_farg5", "1F575-1F3FF"},
    {"solglasogon", "1F576"},
    {"spindel", "1F577"},
    {"spindelnat", "1F578"},
    {"joystick", "1F579"},
    {"dansande_man", "1F57A"},
    {"dansande_man_farg1", "1F57A-1F3FB"},
    {"dansande_man_farg2", "1F57A-1F3FC"},
    {"dansande_man_farg3", "1F57A-1F3FD"},
    {"dansande_man_farg4", "1F57A-1F3FE"},
    {"dansande_man_farg5", "1F57A-1F3FF"},
    {"lankade_gem", "1F587"},
    {"kulspetspenna", "1F58A"},
    {"reservoarpenna", "1F58B"},
    {"malarpensel", "1F58C"},
    {"krita", "1F58D"},
    {"hand_med_utspretade_fingrar", "1F590"},
    {"hand_med_utspretade_fingrar_farg1", "1F590-1F3FB"},
    {"hand_med_utspretade_fingrar_farg2", "1F590-1F3FC"},
    {"hand_med_utspretade_fingrar_farg3", "1F590-1F3FD"},
    {"hand_med_utspretade_fingrar_farg4", "1F590-1F3FE"},
    {"hand_med_utspretade_fingrar_farg5", "1F590-1F3FF"},
    {"langfinger_upp", "1F595"},
    {"langfinger_upp_farg1", "1F595-1F3FB"},
    {"langfinger_upp_farg2", "1F595-1F3FC"},
    {"langfinger_upp_farg3", "1F595-1F3FD"},
    {"langfinger_upp_farg4", "1F595-1F3FE"},
    {"langfinger_upp_farg5", "1F595-1F3FF"},
    {"vulcan_halsning", "1F596"},
    {"vulcan_halsning_farg1", "1F596-1F3FB"},
    {"vulcan_halsning_farg2", "1F596-1F3FC"},
    {"vulcan_halsning_farg3", "1F596-1F3FD"},
    {"vulcan_halsning_farg4", "1F596-1F3FE"},
    {"vulcan_halsning_farg5", "1F596-1F3FF"},
    {"svart_hjarta", "1F5A4"},
    {"stationar_dator", "1F5A5"},
    {"skrivare", "1F5A8"},
    {"datormus", "1F5B1"},
    {"styrkula", "1F5B2"},
    {"tavla", "1F5BC"},
    {"indexflikar", "1F5C2"},
    {"kartotek", "1F5C3"},
    {"dokumentskap", "1F5C4"},
    {"papperskorg", "1F5D1"},
    {"anteckningsblock", "1F5D2"},
    {"spiralbunden_kalender", "1F5D3"},
    {"skruvtving", "1F5DC"},
    {"gammal_nyckel", "1F5DD"},
    {"ihoprullad_tidning", "1F5DE"},
    {"dolk", "1F5E1"},
    {"silhuett_som_talar", "1F5E3"},
    {"pratbubbla_at_vanster", "1F5E8"},
    {"arg_pratbubbla_at_hoger", "1F5EF"},
    {"valurna_med_rostsedel", "1F5F3"},
    {"varldskarta", "1F5FA"},
    {"fuji", "1F5FB"},
    {"tokyo_tower", "1F5FC"},
    {"frihetsgudinnan", "1F5FD"},
    {"karta_over_japan", "1F5FE"},
    {"staty", "1F5FF"},
    {"ansikte_med_stort_leende", "1F600"},
    {"stralande_ansikte_med_leende_ogon", "1F601"},
    {"ansikte_med_gladjetarar", "1F602"},
    {"flinande_ansikte_med_stora_ogon", "1F603"},
    {"flinande_ansikte_med_leende_ogon", "1F604"},
    {"flinande_ansikte_med_svettdroppe", "1F605"},
    {"flinande_ansikte_med_kisande_ogon", "1F606"},
    {"leende_ansikte_med_gloria", "1F607"},
    {"leende_ansikte_med_horn", "1F608"},
    {"ansikte_som_blinkar_med_ett_oga", "1F609"},
    {"leende_ansikte_och_ogon", "1F60A"},
    {"ansikte_som_njuter_av_god_mat", "1F60B"},
    {"lattat_ansikte", "1F60C"},
    {"leende_ansikte_med_hjartformade_ogon", "1F60D"},
    {"leende_ansikte_med_solglasogon", "1F60E"},
    {"halvleende_ansikte", "1F60F"},
    {"neutralt_ansikte", "1F610"},
    {"uttryckslost_ansikte", "1F611"},
    {"missnojt_ansikte", "1F612"},
    {"nedslaget_kallsvettigt_ansikte", "1F613"},
    {"funderande_ansikte", "1F614"},
    {"forbryllat_ansikte", "1F615"},
    {"forvirrat_ansikte", "1F616"},
    {"pussande_ansikte", "1F617"},
    {"ansikte_ger_en_slangkyss", "1F618"},
    {"pussande_ansikte_med_leende_ogon", "1F619"},
    {"pussande_ansikte_med_slutna_ogon", "1F61A"},
    {"ansikte_racker_ut_tungan", "1F61B"},
    {"blinkande_ansikte_racker_ut_tungan", "1F61C"},
    {"ansikte_racker_ut_tungan_med_stangda_ogon", "1F61D"},
    {"besviket_ansikte", "1F61E"},
    {"bekymrat_ansikte", "1F61F"},
    {"argt_ansikte", "1F620"},
    {"rasande_ansikte", "1F621"},
    {"gratande_ansikte", "1F622"},
    {"uthalligt_ansikte", "1F623"},
    {"ansikte_med_anga_ur_nasan", "1F624"},
    {"besviket_men_lattat_ansikte", "1F625"},
    {"ogillande_ansikte_med_oppen_mun", "1F626"},
    {"plagat_ansikte", "1F627"},
    {"skramt_ansikte", "1F628"},
    {"uttrottat_ansikte", "1F629"},
    {"somnigt_ansikte", "1F62A"},
    {"trott_ansikte", "1F62B"},
    {"grimaserande_ansikte", "1F62C"},
    {"storgratande_ansikte", "1F62D"},
    {"forvanat_ansikte", "1F62E"},
    {"hyssjat_ansikte", "1F62F"},
    {"oroligt_svettigt_ansikte", "1F630"},
    {"ansikte_som_skriker_av_fasa", "1F631"},
    {"chockat_ansikte", "1F632"},
    {"rodnande_ansikte", "1F633"},
    {"sovande_ansikte", "1F634"},
    {"omtumlat_ansikte", "1F635"},
    {"ansikte_utan_mun", "1F636"},
    {"ansikte_med_munskydd", "1F637"},
    {"flinande_kattansikte_med_leende_ogon", "1F638"},
    {"kattansikte_med_gladjetarar", "1F639"},
    {"flinande_kattansikte", "1F63A"},
    {"leende_kattansikte_med_hjartformade_ogon", "1F63B"},
    {"kattansikte_med_snett_leende", "1F63C"},
    {"kyssande_kattansikte", "1F63D"},
    {"tjurande_katt", "1F63E"},
    {"gratande_kattansikte", "1F63F"},
    {"uttrottat_kattansikte", "1F640"},
    {"ansikte_med_lite_sur_mun", "1F641"},
    {"smaleende_ansikte", "1F642"},
    {"upp_och_nervant_ansikte", "1F643"},
    {"ansikte_som_himlar_med_ogonen", "1F644"},
    {"person_gestikulerar_nej", "1F645"},
    {"person_gestikulerar_nej_farg1", "1F645-1F3FB"},
    {"person_gestikulerar_nej_farg2", "1F645-1F3FC"},
    {"person_gestikulerar_nej_farg3", "1F645-1F3FD"},
    {"person_gestikulerar_nej_farg4", "1F645-1F3FE"},
    {"person_gestikulerar_nej_farg5", "1F645-1F3FF"},
    {"person_gestikulerar_ok", "1F646"},
    {"person_gestikulerar_ok_farg1", "1F646-1F3FB"},
    {"person_gestikulerar_ok_farg2", "1F646-1F3FC"},
    {"person_gestikulerar_ok_farg3", "1F646-1F3FD"},
    {"person_gestikulerar_ok_farg4", "1F646-1F3FE"},
    {"person_gestikulerar_ok_farg5", "1F646-1F3FF"},
    {"bugande_person", "1F647"},
    {"bugande_person_farg1", "1F647-1F3FB"},
    {"bugande_person_farg2", "1F647-1F3FC"},
    {"bugande_person_farg3", "1F647-1F3FD"},
    {"bugande_person_farg4", "1F647-1F3FE"},
    {"bugande_person_farg5", "1F647-1F3FF"},
    {"apa_ser_inget", "1F648"},
    {"apa_hor_inget", "1F649"},
    {"apa_sager_inget", "1F64A"},
    {"person_som_racker_upp_handen", "1F64B"},
    {"person_som_racker_upp_handen_farg1", "1F64B-1F3FB"},
    {"person_som_racker_upp_handen_farg2", "1F64B-1F3FC"},
    {"person_som_racker_upp_handen_farg3", "1F64B-1F3FD"},
    {"person_som_racker_upp_handen_farg4", "1F64B-1F3FE"},
    {"person_som_racker_upp_handen_farg5", "1F64B-1F3FF"},
    {"bada_handerna_i_luften", "1F64C"},
    {"bada_handerna_i_luften_farg1", "1F64C-1F3FB"},
    {"bada_handerna_i_luften_farg2", "1F64C-1F3FC"},
    {"bada_handerna_i_luften_farg3", "1F64C-1F3FD"},
    {"bada_handerna_i_luften_farg4", "1F64C-1F3FE"},
    {"bada_handerna_i_luften_farg5", "1F64C-1F3FF"},
    {"sur_person", "1F64D"},
    {"sur_person_farg1", "1F64D-1F3FB"},
    {"sur_person_farg2", "1F64D-1F3FC"},
    {"sur_person_farg3", "1F64D-1F3FD"},
    {"sur_person_farg4", "1F64D-1F3FE"},
    {"sur_person_farg5", "1F64D-1F3FF"},
    {"plutande_person", "1F64E"},
    {"plutande_person_farg1", "1F64E-1F3FB"},
    {"plutande_person_farg2", "1F64E-1F3FC"},
    {"plutande_person_farg3", "1F64E-1F3FD"},
    {"plutande_person_farg4", "1F64E-1F3FE"},
    {"plutande_person_farg5", "1F64E-1F3FF"},
    {"beende_hander", "1F64F"},
    {"beende_hander_farg1", "1F64F-1F3FB"},
    {"beende_hander_farg2", "1F64F-1F3FC"},
    {"beende_hander_farg3", "1F64F-1F3FD"},
    {"beende_hander_farg4", "1F64F-1F3FE"},
    {"beende_hander_farg5", "1F64F-1F3FF"},
    {"raket", "1F680"},
    {"helikopter", "1F681"},
    {"anglok", "1F682"},
    {"tagvagn", "1F683"},
    {"snabbtag", "1F684"},
    {"hoghastighetstag", "1F685"},
    {"tag", "1F686"},
    {"tunnelbana", "1F687"},
    {"snabbsparvag", "1F688"},
    {"tagstation", "1F689"},
    {"sparvagn", "1F68A"},
    {"sparvagnsvagn", "1F68B"},
    {"buss", "1F68C"},
    {"motande_buss", "1F68D"},
    {"tradbuss", "1F68E"},
    {"busshallplats", "1F68F"},
    {"minibuss", "1F690"},
    {"ambulans", "1F691"},
    {"brandbil", "1F692"},
    {"polisbil", "1F693"},
    {"motande_polisbil", "1F694"},
    {"taxi", "1F695"},
    {"motande_taxi", "1F696"},
    {"bil", "1F697"},
    {"motande_bil", "1F698"},
    {"suv", "1F699"},
    {"budbil", "1F69A"},
    {"lastbil", "1F69B"},
    {"traktor", "1F69C"},
    {"monorail", "1F69D"},
    {"bergbana", "1F69E"},
    {"hangbana", "1F69F"},
    {"linbana", "1F6A0"},
    {"kabinbana", "1F6A1"},
    {"fartyg", "1F6A2"},
    {"person_som_ror_en_bat", "1F6A3"},
    {"person_som_ror_en_bat_farg1", "1F6A3-1F3FB"},
    {"person_som_ror_en_bat_farg2", "1F6A3-1F3FC"},
    {"person_som_ror_en_bat_farg3", "1F6A3-1F3FD"},
    {"person_som_ror_en_bat_farg4", "1F6A3-1F3FE"},
    {"person_som_ror_en_bat_farg5", "1F6A3-1F3FF"},
    {"racerbat", "1F6A4"},
    {"vagrata_trafikljus", "1F6A5"},
    {"lodrata_trafikljus", "1F6A6"},
    {"skylt_for_byggomrade", "1F6A7"},
    {"polisbilslampa", "1F6A8"},
    {"trekantig_flagga", "1F6A9"},
    {"dorr", "1F6AA"},
    {"forbjudet", "1F6AB"},
    {"cigarett", "1F6AC"},
    {"rokning_forbjuden", "1F6AD"},
    {"soptunna", "1F6AE"},
    {"skrapa_inte_ner", "1F6AF"},
    {"dricksvatten", "1F6B0"},
    {"ej_dricksvatten", "1F6B1"},
    {"cykel", "1F6B2"},
    {"forbud_mot_trafik_med_cykel", "1F6B3"},
    {"person_som_cyklar", "1F6B4"},
    {"person_som_cyklar_farg1", "1F6B4-1F3FB"},
    {"person_som_cyklar_farg2", "1F6B4-1F3FC"},
    {"person_som_cyklar_farg3", "1F6B4-1F3FD"},
    {"person_som_cyklar_farg4", "1F6B4-1F3FE"},
    {"person_som_cyklar_farg5", "1F6B4-1F3FF"},
    {"person_som_cyklar_pa_en_mountainbike", "1F6B5"},
    {"person_som_cyklar_pa_en_mountainbike_farg1", "1F6B5-1F3FB"},
    {"person_som_cyklar_pa_en_mountainbike_farg2", "1F6B5-1F3FC"},
    {"person_som_cyklar_pa_en_mountainbike_farg3", "1F6B5-1F3FD"},
    {"person_som_cyklar_pa_en_mountainbike_farg4", "1F6B5-1F3FE"},
    {"person_som_cyklar_pa_en_mountainbike_farg5", "1F6B5-1F3FF"},
    {"fotgangare", "1F6B6"},
    {"fotgangare_farg1", "1F6B6-1F3FB"},
    {"fotgangare_farg2", "1F6B6-1F3FC"},
    {"fotgangare_farg3", "1F6B6-1F3FD"},
    {"fotgangare_farg4", "1F6B6-1F3FE"},
    {"fotgangare_farg5", "1F6B6-1F3FF"},
    {"forbud_mot_gangtrafik", "1F6B7"},
    {"passerande_barn", "1F6B8"},
    {"herrtoalett", "1F6B9"},
    {"damtoalett", "1F6BA"},
    {"wc_skylt", "1F6BB"},
    {"bebissymbol", "1F6BC"},
    {"toalett", "1F6BD"},
    {"wc", "1F6BE"},
    {"dusch", "1F6BF"},
    {"person_som_badar", "1F6C0"},
    {"person_som_badar_farg1", "1F6C0-1F3FB"},
    {"person_som_badar_farg2", "1F6C0-1F3FC"},
    {"person_som_badar_farg3", "1F6C0-1F3FD"},
    {"person_som_badar_farg4", "1F6C0-1F3FE"},
    {"person_som_badar_farg5", "1F6C0-1F3FF"},
    {"badkar", "1F6C1"},
    {"passkontroll", "1F6C2"},
    {"tull", "1F6C3"},
    {"bagagehamtning", "1F6C4"},
    {"kvarlamnat_bagage", "1F6C5"},
    {"soffa_och_lampa", "1F6CB"},
    {"person_i_en_sang", "1F6CC"},
    {"person_i_en_sang_farg1", "1F6CC-1F3FB"},
    {"person_i_en_sang_farg2", "1F6CC-1F3FC"},
    {"person_i_en_sang_farg3", "1F6CC-1F3FD"},
    {"person_i_en_sang_farg4", "1F6CC-1F3FE"},
    {"person_i_en_sang_farg5", "1F6CC-1F3FF"},
    {"shoppingpasar", "1F6CD"},
    {"receptionsklocka", "1F6CE"},
    {"sang", "1F6CF"},
    {"plats_for_tillbedjan", "1F6D0"},
    {"stoppskylt", "1F6D1"},
    {"shoppingvagn", "1F6D2"},
    {"hindutempel", "1F6D5"},
    {"koja", "1F6D6"},
    {"hiss", "1F6D7"},
    {"jordskred", "1F6D8"},
    {"tradlos", "1F6DC"},
    {"rutschbana", "1F6DD"},
    {"hjul", "1F6DE"},
    {"livboj", "1F6DF"},
    {"hammare_och_skiftnyckel", "1F6E0"},
    {"skold", "1F6E1"},
    {"oljefat", "1F6E2"},
    {"motorvag", "1F6E3"},
    {"rals", "1F6E4"},
    {"motorbat", "1F6E5"},
    {"litet_flygplan", "1F6E9"},
    {"avgang_med_flyg", "1F6EB"},
    {"ankomst_med_flyg", "1F6EC"},
    {"satellit", "1F6F0"},
    {"passagerarfartyg", "1F6F3"},
    {"sparkcykel", "1F6F4"},
    {"skoter", "1F6F5"},
    {"kanot", "1F6F6"},
    {"kalke", "1F6F7"},
    {"flygande_tefat", "1F6F8"},
    {"skateboard", "1F6F9"},
    {"automatisk_riksha", "1F6FA"},
    {"pickup", "1F6FB"},
    {"rullskridsko", "1F6FC"},
    {"orange_cirkel", "1F7E0"},
    {"gul_cirkel", "1F7E1"},
    {"gron_cirkel", "1F7E2"},
    {"lila_cirkel", "1F7E3"},
    {"brun_cirkel", "1F7E4"},
    {"rod_kvadrat", "1F7E5"},
    {"bla_kvadrat", "1F7E6"},
    {"orange_kvadrat", "1F7E7"},
    {"gul_kvadrat", "1F7E8"},
    {"gron_kvadrat", "1F7E9"},
    {"lila_kvadrat", "1F7EA"},
    {"brun_kvadrat", "1F7EB"},
    {"fetstilt_likamedtecken", "1F7F0"},
    {"fingerportmonna", "1F90C"},
    {"fingerportmonna_farg1", "1F90C-1F3FB"},
    {"fingerportmonna_farg2", "1F90C-1F3FC"},
    {"fingerportmonna_farg3", "1F90C-1F3FD"},
    {"fingerportmonna_farg4", "1F90C-1F3FE"},
    {"fingerportmonna_farg5", "1F90C-1F3FF"},
    {"vitt_hjarta", "1F90D"},
    {"brunt_hjarta", "1F90E"},
    {"nypande_hand", "1F90F"},
    {"nypande_hand_farg1", "1F90F-1F3FB"},
    {"nypande_hand_farg2", "1F90F-1F3FC"},
    {"nypande_hand_farg3", "1F90F-1F3FD"},
    {"nypande_hand_farg4", "1F90F-1F3FE"},
    {"nypande_hand_farg5", "1F90F-1F3FF"},
    {"ansikte_som_tiger", "1F910"},
    {"ansikte_med_dollarogon", "1F911"},
    {"ansikte_med_termometer", "1F912"},
    {"nordansikte", "1F913"},
    {"ansikte_som_tanker", "1F914"},
    {"ansikte_med_bandage_pa_huvudet", "1F915"},
    {"robotansikte", "1F916"},
    {"ansikte_som_kramas", "1F917"},
    {"djavulstecknet", "1F918"},
    {"djavulstecknet_farg1", "1F918-1F3FB"},
    {"djavulstecknet_farg2", "1F918-1F3FC"},
    {"djavulstecknet_farg3", "1F918-1F3FD"},
    {"djavulstecknet_farg4", "1F918-1F3FE"},
    {"djavulstecknet_farg5", "1F918-1F3FF"},
    {"ring_mig_hand", "1F919"},
    {"ring_mig_hand_farg1", "1F919-1F3FB"},
    {"ring_mig_hand_farg2", "1F919-1F3FC"},
    {"ring_mig_hand_farg3", "1F919-1F3FD"},
    {"ring_mig_hand_farg4", "1F919-1F3FE"},
    {"ring_mig_hand_farg5", "1F919-1F3FF"},
    {"hojd_handrygg", "1F91A"},
    {"hojd_handrygg_farg1", "1F91A-1F3FB"},
    {"hojd_handrygg_farg2", "1F91A-1F3FC"},
    {"hojd_handrygg_farg3", "1F91A-1F3FD"},
    {"hojd_handrygg_farg4", "1F91A-1F3FE"},
    {"hojd_handrygg_farg5", "1F91A-1F3FF"},
    {"vanstervand_nave", "1F91B"},
    {"vanstervand_nave_farg1", "1F91B-1F3FB"},
    {"vanstervand_nave_farg2", "1F91B-1F3FC"},
    {"vanstervand_nave_farg3", "1F91B-1F3FD"},
    {"vanstervand_nave_farg4", "1F91B-1F3FE"},
    {"vanstervand_nave_farg5", "1F91B-1F3FF"},
    {"hogervand_nave", "1F91C"},
    {"hogervand_nave_farg1", "1F91C-1F3FB"},
    {"hogervand_nave_farg2", "1F91C-1F3FC"},
    {"hogervand_nave_farg3", "1F91C-1F3FD"},
    {"hogervand_nave_farg4", "1F91C-1F3FE"},
    {"hogervand_nave_farg5", "1F91C-1F3FF"},
    {"handskakning", "1F91D"},
    {"handskakning_farg1", "1F91D-1F3FB"},
    {"handskakning_farg2", "1F91D-1F3FC"},
    {"handskakning_farg3", "1F91D-1F3FD"},
    {"handskakning_farg4", "1F91D-1F3FE"},
    {"handskakning_farg5", "1F91D-1F3FF"},
    {"handskakning_farg1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"handskakning_farg1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"handskakning_farg1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"handskakning_farg1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"handskakning_farg2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"handskakning_farg2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"handskakning_farg2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"handskakning_farg2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"handskakning_farg3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"handskakning_farg3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"handskakning_farg3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"handskakning_farg3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"handskakning_farg4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"handskakning_farg4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"handskakning_farg4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"handskakning_farg4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"handskakning_farg5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"handskakning_farg5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"handskakning_farg5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"handskakning_farg5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"korsade_fingrar", "1F91E"},
    {"korsade_fingrar_farg1", "1F91E-1F3FB"},
    {"korsade_fingrar_farg2", "1F91E-1F3FC"},
    {"korsade_fingrar_farg3", "1F91E-1F3FD"},
    {"korsade_fingrar_farg4", "1F91E-1F3FE"},
    {"korsade_fingrar_farg5", "1F91E-1F3FF"},
    {"jag_alskar_dig_gest", "1F91F"},
    {"jag_alskar_dig_gest_farg1", "1F91F-1F3FB"},
    {"jag_alskar_dig_gest_farg2", "1F91F-1F3FC"},
    {"jag_alskar_dig_gest_farg3", "1F91F-1F3FD"},
    {"jag_alskar_dig_gest_farg4", "1F91F-1F3FE"},
    {"jag_alskar_dig_gest_farg5", "1F91F-1F3FF"},
    {"ansikte_med_cowboyhatt", "1F920"},
    {"clownansikte", "1F921"},
    {"acklat_ansikte", "1F922"},
    {"rullar_runt_pa_golvet_och_skrattar", "1F923"},
    {"dreglande_ansikte", "1F924"},
    {"ljugande_ansikte", "1F925"},
    {"person_som_slar_sig_for_pannan", "1F926"},
    {"person_som_slar_sig_for_pannan_farg1", "1F926-1F3FB"},
    {"person_som_slar_sig_for_pannan_farg2", "1F926-1F3FC"},
    {"person_som_slar_sig_for_pannan_farg3", "1F926-1F3FD"},
    {"person_som_slar_sig_for_pannan_farg4", "1F926-1F3FE"},
    {"person_som_slar_sig_for_pannan_farg5", "1F926-1F3FF"},
    {"nysande_ansikte", "1F927"},
    {"ansikte_med_hojt_ogonbryn", "1F928"},
    {"stjarnogt_ansikte", "1F929"},
    {"tokigt_ansikte", "1F92A"},
    {"hyssjande_ansikte", "1F92B"},
    {"ansikte_med_symboler_pa_munnen", "1F92C"},
    {"ansikte_med_handen_for_munnen", "1F92D"},
    {"krakande_ansikte", "1F92E"},
    {"exploderande_huvud", "1F92F"},
    {"gravid_kvinna", "1F930"},
    {"gravid_kvinna_farg1", "1F930-1F3FB"},
    {"gravid_kvinna_farg2", "1F930-1F3FC"},
    {"gravid_kvinna_farg3", "1F930-1F3FD"},
    {"gravid_kvinna_farg4", "1F930-1F3FE"},
    {"gravid_kvinna_farg5", "1F930-1F3FF"},
    {"amning", "1F931"},
    {"amning_farg1", "1F931-1F3FB"},
    {"amning_farg2", "1F931-1F3FC"},
    {"amning_farg3", "1F931-1F3FD"},
    {"amning_farg4", "1F931-1F3FE"},
    {"amning_farg5", "1F931-1F3FF"},
    {"handflatorna_upp", "1F932"},
    {"handflatorna_upp_farg1", "1F932-1F3FB"},
    {"handflatorna_upp_farg2", "1F932-1F3FC"},
    {"handflatorna_upp_farg3", "1F932-1F3FD"},
    {"handflatorna_upp_farg4", "1F932-1F3FE"},
    {"handflatorna_upp_farg5", "1F932-1F3FF"},
    {"selfie", "1F933"},
    {"selfie_farg1", "1F933-1F3FB"},
    {"selfie_farg2", "1F933-1F3FC"},
    {"selfie_farg3", "1F933-1F3FD"},
    {"selfie_farg4", "1F933-1F3FE"},
    {"selfie_farg5", "1F933-1F3FF"},
    {"prins", "1F934"},
    {"prins_farg1", "1F934-1F3FB"},
    {"prins_farg2", "1F934-1F3FC"},
    {"prins_farg3", "1F934-1F3FD"},
    {"prins_farg4", "1F934-1F3FE"},
    {"prins_farg5", "1F934-1F3FF"},
    {"person_i_smoking", "1F935"},
    {"person_i_smoking_farg1", "1F935-1F3FB"},
    {"person_i_smoking_farg2", "1F935-1F3FC"},
    {"person_i_smoking_farg3", "1F935-1F3FD"},
    {"person_i_smoking_farg4", "1F935-1F3FE"},
    {"person_i_smoking_farg5", "1F935-1F3FF"},
    {"tomtemor", "1F936"},
    {"tomtemor_farg1", "1F936-1F3FB"},
    {"tomtemor_farg2", "1F936-1F3FC"},
    {"tomtemor_farg3", "1F936-1F3FD"},
    {"tomtemor_farg4", "1F936-1F3FE"},
    {"tomtemor_farg5", "1F936-1F3FF"},
    {"person_som_rycker_pa_axlarna", "1F937"},
    {"person_som_rycker_pa_axlarna_farg1", "1F937-1F3FB"},
    {"person_som_rycker_pa_axlarna_farg2", "1F937-1F3FC"},
    {"person_som_rycker_pa_axlarna_farg3", "1F937-1F3FD"},
    {"person_som_rycker_pa_axlarna_farg4", "1F937-1F3FE"},
    {"person_som_rycker_pa_axlarna_farg5", "1F937-1F3FF"},
    {"person_som_hjular", "1F938"},
    {"person_som_hjular_farg1", "1F938-1F3FB"},
    {"person_som_hjular_farg2", "1F938-1F3FC"},
    {"person_som_hjular_farg3", "1F938-1F3FD"},
    {"person_som_hjular_farg4", "1F938-1F3FE"},
    {"person_som_hjular_farg5", "1F938-1F3FF"},
    {"person_som_jonglerar", "1F939"},
    {"person_som_jonglerar_farg1", "1F939-1F3FB"},
    {"person_som_jonglerar_farg2", "1F939-1F3FC"},
    {"person_som_jonglerar_farg3", "1F939-1F3FD"},
    {"person_som_jonglerar_farg4", "1F939-1F3FE"},
    {"person_som_jonglerar_farg5", "1F939-1F3FF"},
    {"person_som_faktas", "1F93A"},
    {"brottande_personer", "1F93C"},
    {"brottande_personer_farg1", "1F93C-1F3FB"},
    {"brottande_personer_farg2", "1F93C-1F3FC"},
    {"brottande_personer_farg3", "1F93C-1F3FD"},
    {"brottande_personer_farg4", "1F93C-1F3FE"},
    {"brottande_personer_farg5", "1F93C-1F3FF"},
    {"brottande_personer_farg1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"brottande_personer_farg1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"brottande_personer_farg1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"brottande_personer_farg1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"brottande_personer_farg2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"brottande_personer_farg2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"brottande_personer_farg2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"brottande_personer_farg2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"brottande_personer_farg3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"brottande_personer_farg3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"brottande_personer_farg3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"brottande_personer_farg3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"brottande_personer_farg4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"brottande_personer_farg4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"brottande_personer_farg4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"brottande_personer_farg4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"brottande_personer_farg5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"brottande_personer_farg5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"brottande_personer_farg5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"brottande_personer_farg5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"person_som_spelar_vattenpolo", "1F93D"},
    {"person_som_spelar_vattenpolo_farg1", "1F93D-1F3FB"},
    {"person_som_spelar_vattenpolo_farg2", "1F93D-1F3FC"},
    {"person_som_spelar_vattenpolo_farg3", "1F93D-1F3FD"},
    {"person_som_spelar_vattenpolo_farg4", "1F93D-1F3FE"},
    {"person_som_spelar_vattenpolo_farg5", "1F93D-1F3FF"},
    {"person_som_spelar_handboll", "1F93E"},
    {"person_som_spelar_handboll_farg1", "1F93E-1F3FB"},
    {"person_som_spelar_handboll_farg2", "1F93E-1F3FC"},
    {"person_som_spelar_handboll_farg3", "1F93E-1F3FD"},
    {"person_som_spelar_handboll_farg4", "1F93E-1F3FE"},
    {"person_som_spelar_handboll_farg5", "1F93E-1F3FF"},
    {"dykmask", "1F93F"},
    {"vissnad_blomma", "1F940"},
    {"trumma", "1F941"},
    {"skalande_glas", "1F942"},
    {"whiskyglas", "1F943"},
    {"sked", "1F944"},
    {"malnat", "1F945"},
    {"guldmedalj", "1F947"},
    {"silvermedalj", "1F948"},
    {"bronsmedalj", "1F949"},
    {"boxningshandske", "1F94A"},
    {"kampsportdrakt", "1F94B"},
    {"curlingsten", "1F94C"},
    {"lacrosse", "1F94D"},
    {"softboll", "1F94E"},
    {"frisbee", "1F94F"},
    {"croissant", "1F950"},
    {"avokado", "1F951"},
    {"gurka", "1F952"},
    {"bacon", "1F953"},
    {"potatis", "1F954"},
    {"morot", "1F955"},
    {"baguette", "1F956"},
    {"gronsallad", "1F957"},
    {"paella", "1F958"},
    {"pitabrod", "1F959"},
    {"agg", "1F95A"},
    {"mjolkglas", "1F95B"},
    {"jordnotter", "1F95C"},
    {"kiwifrukt", "1F95D"},
    {"pannkakor", "1F95E"},
    {"dumpling", "1F95F"},
    {"lyckokaka", "1F960"},
    {"hamtmatkartong", "1F961"},
    {"atpinnar", "1F962"},
    {"skal_och_sked", "1F963"},
    {"mugg_med_sugror", "1F964"},
    {"kokosnot", "1F965"},
    {"broccoli", "1F966"},
    {"paj", "1F967"},
    {"pretzel", "1F968"},
    {"styckdel", "1F969"},
    {"smorgas", "1F96A"},
    {"konserv", "1F96B"},
    {"sallat", "1F96C"},
    {"mango", "1F96D"},
    {"mankaka", "1F96E"},
    {"bagel", "1F96F"},
    {"leende_ansikte_med_tre_hjartan", "1F970"},
    {"gaspande_ansikte", "1F971"},
    {"tarogt_leende_ansikte", "1F972"},
    {"festande_ansikte", "1F973"},
    {"yrt_ansikte", "1F974"},
    {"hett_ansikte", "1F975"},
    {"kallt_ansikte", "1F976"},
    {"ninja", "1F977"},
    {"ninja_farg1", "1F977-1F3FB"},
    {"ninja_farg2", "1F977-1F3FC"},
    {"ninja_farg3", "1F977-1F3FD"},
    {"ninja_farg4", "1F977-1F3FE"},
    {"ninja_farg5", "1F977-1F3FF"},
    {"maskerat_ansikte", "1F978"},
    {"ansikte_som_haller_tillbaka_tararna", "1F979"},
    {"bonfallande_ansikte", "1F97A"},
    {"sari", "1F97B"},
    {"labbrock", "1F97C"},
    {"simglasogon", "1F97D"},
    {"vandringskanga", "1F97E"},
    {"ballerinasko", "1F97F"},
    {"krabba", "1F980"},
    {"lejonansikte", "1F981"},
    {"skorpion", "1F982"},
    {"kalkon", "1F983"},
    {"enhorningsansikte", "1F984"},
    {"orn", "1F985"},
    {"and", "1F986"},
    {"fladdermus", "1F987"},
    {"haj", "1F988"},
    {"uggla", "1F989"},
    {"ravansikte", "1F98A"},
    {"fjaril", "1F98B"},
    {"hjort", "1F98C"},
    {"gorilla", "1F98D"},
    {"odla", "1F98E"},
    {"noshorning", "1F98F"},
    {"raka", "1F990"},
    {"mindre_blackfisk", "1F991"},
    {"giraff", "1F992"},
    {"zebra", "1F993"},
    {"igelkott", "1F994"},
    {"sauropod", "1F995"},
    {"trex", "1F996"},
    {"syrsa", "1F997"},
    {"kanguru", "1F998"},
    {"lama", "1F999"},
    {"pafagel", "1F99A"},
    {"flodhast", "1F99B"},
    {"papegoja", "1F99C"},
    {"tvattbjorn", "1F99D"},
    {"hummer", "1F99E"},
    {"mygga", "1F99F"},
    {"mikrob", "1F9A0"},
    {"gravling", "1F9A1"},
    {"svan", "1F9A2"},
    {"mammut", "1F9A3"},
    {"dront", "1F9A4"},
    {"sengangare", "1F9A5"},
    {"utter", "1F9A6"},
    {"orangutang", "1F9A7"},
    {"skunk", "1F9A8"},
    {"flamingo", "1F9A9"},
    {"ostron", "1F9AA"},
    {"baver", "1F9AB"},
    {"bisonoxe", "1F9AC"},
    {"sal", "1F9AD"},
    {"ledarhund", "1F9AE"},
    {"blindkapp", "1F9AF"},
    {"rodharig", "1F9B0"},
    {"lockigt_har", "1F9B1"},
    {"skallig", "1F9B2"},
    {"vitharig", "1F9B3"},
    {"benknota", "1F9B4"},
    {"ben", "1F9B5"},
    {"ben_farg1", "1F9B5-1F3FB"},
    {"ben_farg2", "1F9B5-1F3FC"},
    {"ben_farg3", "1F9B5-1F3FD"},
    {"ben_farg4", "1F9B5-1F3FE"},
    {"ben_farg5", "1F9B5-1F3FF"},
    {"fot", "1F9B6"},
    {"fot_farg1", "1F9B6-1F3FB"},
    {"fot_farg2", "1F9B6-1F3FC"},
    {"fot_farg3", "1F9B6-1F3FD"},
    {"fot_farg4", "1F9B6-1F3FE"},
    {"fot_farg5", "1F9B6-1F3FF"},
    {"tand", "1F9B7"},
    {"superhjalte", "1F9B8"},
    {"superhjalte_farg1", "1F9B8-1F3FB"},
    {"superhjalte_farg2", "1F9B8-1F3FC"},
    {"superhjalte_farg3", "1F9B8-1F3FD"},
    {"superhjalte_farg4", "1F9B8-1F3FE"},
    {"superhjalte_farg5", "1F9B8-1F3FF"},
    {"superskurk", "1F9B9"},
    {"superskurk_farg1", "1F9B9-1F3FB"},
    {"superskurk_farg2", "1F9B9-1F3FC"},
    {"superskurk_farg3", "1F9B9-1F3FD"},
    {"superskurk_farg4", "1F9B9-1F3FE"},
    {"superskurk_farg5", "1F9B9-1F3FF"},
    {"varselvast", "1F9BA"},
    {"ora_med_horapparat", "1F9BB"},
    {"ora_med_horapparat_farg1", "1F9BB-1F3FB"},
    {"ora_med_horapparat_farg2", "1F9BB-1F3FC"},
    {"ora_med_horapparat_farg3", "1F9BB-1F3FD"},
    {"ora_med_horapparat_farg4", "1F9BB-1F3FE"},
    {"ora_med_horapparat_farg5", "1F9BB-1F3FF"},
    {"eldriven_rullstol", "1F9BC"},
    {"manuell_rullstol", "1F9BD"},
    {"mekanisk_arm", "1F9BE"},
    {"mekaniskt_ben", "1F9BF"},
    {"ostbit", "1F9C0"},
    {"muffin", "1F9C1"},
    {"salt", "1F9C2"},
    {"dryckeskartong", "1F9C3"},
    {"vitlok", "1F9C4"},
    {"lok", "1F9C5"},
    {"falafel", "1F9C6"},
    {"vaffla", "1F9C7"},
    {"smor", "1F9C8"},
    {"mate_te", "1F9C9"},
    {"isbit", "1F9CA"},
    {"bubbelte", "1F9CB"},
    {"troll", "1F9CC"},
    {"staende_person", "1F9CD"},
    {"staende_person_farg1", "1F9CD-1F3FB"},
    {"staende_person_farg2", "1F9CD-1F3FC"},
    {"staende_person_farg3", "1F9CD-1F3FD"},
    {"staende_person_farg4", "1F9CD-1F3FE"},
    {"staende_person_farg5", "1F9CD-1F3FF"},
    {"knasittande_person", "1F9CE"},
    {"knasittande_person_farg1", "1F9CE-1F3FB"},
    {"knasittande_person_farg2", "1F9CE-1F3FC"},
    {"knasittande_person_farg3", "1F9CE-1F3FD"},
    {"knasittande_person_farg4", "1F9CE-1F3FE"},
    {"knasittande_person_farg5", "1F9CE-1F3FF"},
    {"horselskadad_person", "1F9CF"},
    {"horselskadad_person_farg1", "1F9CF-1F3FB"},
    {"horselskadad_person_farg2", "1F9CF-1F3FC"},
    {"horselskadad_person_farg3", "1F9CF-1F3FD"},
    {"horselskadad_person_farg4", "1F9CF-1F3FE"},
    {"horselskadad_person_farg5", "1F9CF-1F3FF"},
    {"ansikte_med_monokel", "1F9D0"},
    {"person", "1F9D1"},
    {"person_farg1", "1F9D1-1F3FB"},
    {"person_farg2", "1F9D1-1F3FC"},
    {"person_farg3", "1F9D1-1F3FD"},
    {"person_farg4", "1F9D1-1F3FE"},
    {"person_farg5", "1F9D1-1F3FF"},
    {"barn", "1F9D2"},
    {"barn_farg1", "1F9D2-1F3FB"},
    {"barn_farg2", "1F9D2-1F3FC"},
    {"barn_farg3", "1F9D2-1F3FD"},
    {"barn_farg4", "1F9D2-1F3FE"},
    {"barn_farg5", "1F9D2-1F3FF"},
    {"aldre_person", "1F9D3"},
    {"aldre_person_farg1", "1F9D3-1F3FB"},
    {"aldre_person_farg2", "1F9D3-1F3FC"},
    {"aldre_person_farg3", "1F9D3-1F3FD"},
    {"aldre_person_farg4", "1F9D3-1F3FE"},
    {"aldre_person_farg5", "1F9D3-1F3FF"},
    {"man_med_skagg", "1F9D4"},
    {"man_med_skagg_farg1", "1F9D4-1F3FB"},
    {"man_med_skagg_farg2", "1F9D4-1F3FC"},
    {"man_med_skagg_farg3", "1F9D4-1F3FD"},
    {"man_med_skagg_farg4", "1F9D4-1F3FE"},
    {"man_med_skagg_farg5", "1F9D4-1F3FF"},
    {"kvinna_i_sjal", "1F9D5"},
    {"kvinna_i_sjal_farg1", "1F9D5-1F3FB"},
    {"kvinna_i_sjal_farg2", "1F9D5-1F3FC"},
    {"kvinna_i_sjal_farg3", "1F9D5-1F3FD"},
    {"kvinna_i_sjal_farg4", "1F9D5-1F3FE"},
    {"kvinna_i_sjal_farg5", "1F9D5-1F3FF"},
    {"person_i_angbastu", "1F9D6"},
    {"person_i_angbastu_farg1", "1F9D6-1F3FB"},
    {"person_i_angbastu_farg2", "1F9D6-1F3FC"},
    {"person_i_angbastu_farg3", "1F9D6-1F3FD"},
    {"person_i_angbastu_farg4", "1F9D6-1F3FE"},
    {"person_i_angbastu_farg5", "1F9D6-1F3FF"},
    {"person_som_klattrar", "1F9D7"},
    {"person_som_klattrar_farg1", "1F9D7-1F3FB"},
    {"person_som_klattrar_farg2", "1F9D7-1F3FC"},
    {"person_som_klattrar_farg3", "1F9D7-1F3FD"},
    {"person_som_klattrar_farg4", "1F9D7-1F3FE"},
    {"person_som_klattrar_farg5", "1F9D7-1F3FF"},
    {"person_i_lotusposition", "1F9D8"},
    {"person_i_lotusposition_farg1", "1F9D8-1F3FB"},
    {"person_i_lotusposition_farg2", "1F9D8-1F3FC"},
    {"person_i_lotusposition_farg3", "1F9D8-1F3FD"},
    {"person_i_lotusposition_farg4", "1F9D8-1F3FE"},
    {"person_i_lotusposition_farg5", "1F9D8-1F3FF"},
    {"magiker", "1F9D9"},
    {"magiker_farg1", "1F9D9-1F3FB"},
    {"magiker_farg2", "1F9D9-1F3FC"},
    {"magiker_farg3", "1F9D9-1F3FD"},
    {"magiker_farg4", "1F9D9-1F3FE"},
    {"magiker_farg5", "1F9D9-1F3FF"},
    {"fe", "1F9DA"},
    {"fe_farg1", "1F9DA-1F3FB"},
    {"fe_farg2", "1F9DA-1F3FC"},
    {"fe_farg3", "1F9DA-1F3FD"},
    {"fe_farg4", "1F9DA-1F3FE"},
    {"fe_farg5", "1F9DA-1F3FF"},
    {"vampyr", "1F9DB"},
    {"vampyr_farg1", "1F9DB-1F3FB"},
    {"vampyr_farg2", "1F9DB-1F3FC"},
    {"vampyr_farg3", "1F9DB-1F3FD"},
    {"vampyr_farg4", "1F9DB-1F3FE"},
    {"vampyr_farg5", "1F9DB-1F3FF"},
    {"vattenvasen", "1F9DC"},
    {"vattenvasen_farg1", "1F9DC-1F3FB"},
    {"vattenvasen_farg2", "1F9DC-1F3FC"},
    {"vattenvasen_farg3", "1F9DC-1F3FD"},
    {"vattenvasen_farg4", "1F9DC-1F3FE"},
    {"vattenvasen_farg5", "1F9DC-1F3FF"},
    {"alv", "1F9DD"},
    {"alv_farg1", "1F9DD-1F3FB"},
    {"alv_farg2", "1F9DD-1F3FC"},
    {"alv_farg3", "1F9DD-1F3FD"},
    {"alv_farg4", "1F9DD-1F3FE"},
    {"alv_farg5", "1F9DD-1F3FF"},
    {"ande", "1F9DE"},
    {"zombie", "1F9DF"},
    {"hjarna", "1F9E0"},
    {"hjarta_i_orange", "1F9E1"},
    {"skarmmossa", "1F9E2"},
    {"halsduk", "1F9E3"},
    {"handske", "1F9E4"},
    {"jacka", "1F9E5"},
    {"sockor", "1F9E6"},
    {"rott_kuvert", "1F9E7"},
    {"smallare", "1F9E8"},
    {"pusselbit", "1F9E9"},
    {"provror", "1F9EA"},
    {"petriskal", "1F9EB"},
    {"dna", "1F9EC"},
    {"kompass", "1F9ED"},
    {"kulram", "1F9EE"},
    {"brandslackare", "1F9EF"},
    {"verktygslada", "1F9F0"},
    {"tegelsten", "1F9F1"},
    {"magnet", "1F9F2"},
    {"bagage", "1F9F3"},
    {"lotionflaska", "1F9F4"},
    {"trad", "1F9F5"},
    {"garn", "1F9F6"},
    {"sakerhetsnal", "1F9F7"},
    {"nallebjorn", "1F9F8"},
    {"kvast", "1F9F9"},
    {"korg", "1F9FA"},
    {"toarulle", "1F9FB"},
    {"tval", "1F9FC"},
    {"tvattsvamp", "1F9FD"},
    {"kvitto", "1F9FE"},
    {"onda_ogat_sten", "1F9FF"},
    {"ballettskor", "1FA70"},
    {"baddrakt", "1FA71"},
    {"badbyxor", "1FA72"},
    {"shorts", "1FA73"},
    {"flip_flops", "1FA74"},
    {"ljusblatt_hjarta", "1FA75"},
    {"gratt_hjarta", "1FA76"},
    {"rosa_hjarta", "1FA77"},
    {"blodsdroppe", "1FA78"},
    {"plaster", "1FA79"},
    {"stetoskop", "1FA7A"},
    {"rontgen", "1FA7B"},
    {"krycka", "1FA7C"},
    {"jojo", "1FA80"},
    {"flygande_drake", "1FA81"},
    {"fallskarm", "1FA82"},
    {"boomerang", "1FA83"},
    {"trollspo", "1FA84"},
    {"pinata", "1FA85"},
    {"ryska_dockor", "1FA86"},
    {"maracas", "1FA87"},
    {"flojt", "1FA88"},
    {"harpa", "1FA89"},
    {"trombon", "1FA8A"},
    {"skattkista", "1FA8E"},
    {"spade", "1FA8F"},
    {"planet_med_ringar", "1FA90"},
    {"stol", "1FA91"},
    {"rakhyvel", "1FA92"},
    {"yxa", "1FA93"},
    {"diyalampa", "1FA94"},
    {"banjo", "1FA95"},
    {"militarhjalm", "1FA96"},
    {"dragspel", "1FA97"},
    {"congatrumma", "1FA98"},
    {"mynt", "1FA99"},
    {"fogsvans", "1FA9A"},
    {"skruvmejsel", "1FA9B"},
    {"stege", "1FA9C"},
    {"krok", "1FA9D"},
    {"spegel", "1FA9E"},
    {"fonster", "1FA9F"},
    {"vaskrensare", "1FAA0"},
    {"synal", "1FAA1"},
    {"knut", "1FAA2"},
    {"spann", "1FAA3"},
    {"musfalla", "1FAA4"},
    {"tandborste", "1FAA5"},
    {"gravsten", "1FAA6"},
    {"plakat", "1FAA7"},
    {"sten", "1FAA8"},
    {"discokula", "1FAA9"},
    {"id_kort", "1FAAA"},
    {"lag_batteriniva", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"solfjader", "1FAAD"},
    {"afrokam", "1FAAE"},
    {"khanda", "1FAAF"},
    {"fluga", "1FAB0"},
    {"mask", "1FAB1"},
    {"skalbagge", "1FAB2"},
    {"kackerlacka", "1FAB3"},
    {"krukvaxt", "1FAB4"},
    {"trastockar", "1FAB5"},
    {"fagelfjader", "1FAB6"},
    {"lotus", "1FAB7"},
    {"korall", "1FAB8"},
    {"tomt_bo", "1FAB9"},
    {"bo_med_agg", "1FABA"},
    {"hyacint", "1FABB"},
    {"manet", "1FABC"},
    {"vinge", "1FABD"},
    {"trad_utan_lov", "1FABE"},
    {"gas", "1FABF"},
    {"hjarta", "1FAC0"},
    {"lungor", "1FAC1"},
    {"personer_kramas", "1FAC2"},
    {"gravid_man", "1FAC3"},
    {"gravid_man_farg1", "1FAC3-1F3FB"},
    {"gravid_man_farg2", "1FAC3-1F3FC"},
    {"gravid_man_farg3", "1FAC3-1F3FD"},
    {"gravid_man_farg4", "1FAC3-1F3FE"},
    {"gravid_man_farg5", "1FAC3-1F3FF"},
    {"gravid_person", "1FAC4"},
    {"gravid_person_farg1", "1FAC4-1F3FB"},
    {"gravid_person_farg2", "1FAC4-1F3FC"},
    {"gravid_person_farg3", "1FAC4-1F3FD"},
    {"gravid_person_farg4", "1FAC4-1F3FE"},
    {"gravid_person_farg5", "1FAC4-1F3FF"},
    {"person_med_krona", "1FAC5"},
    {"person_med_krona_farg1", "1FAC5-1F3FB"},
    {"person_med_krona_farg2", "1FAC5-1F3FC"},
    {"person_med_krona_farg3", "1FAC5-1F3FD"},
    {"person_med_krona_farg4", "1FAC5-1F3FE"},
    {"person_med_krona_farg5", "1FAC5-1F3FF"},
    {"fingeravtryck", "1FAC6"},
    {"harig_varelse", "1FAC8"},
    {"spackhuggare", "1FACD"},
    {"alg", "1FACE"},
    {"asna", "1FACF"},
    {"blabar", "1FAD0"},
    {"paprika", "1FAD1"},
    {"oliv", "1FAD2"},
    {"tunnbrod", "1FAD3"},
    {"tamale", "1FAD4"},
    {"fondue", "1FAD5"},
    {"tekanna", "1FAD6"},
    {"halla_vatska", "1FAD7"},
    {"bonor", "1FAD8"},
    {"burk", "1FAD9"},
    {"ingefara", "1FADA"},
    {"artskida", "1FADB"},
    {"rotfrukt", "1FADC"},
    {"stank", "1FADF"},
    {"smaltande_ansikte", "1FAE0"},
    {"ansikte_som_gor_honnor", "1FAE1"},
    {"ansikte_med_oppna_ogon_och_hand_for_munnen", "1FAE2"},
    {"ansikte_med_kikande_oga", "1FAE3"},
    {"ansikte_med_diagonal_mun", "1FAE4"},
    {"osynligt_ansikte", "1FAE5"},
    {"biter_i_lappen", "1FAE6"},
    {"bubblor", "1FAE7"},
    {"skakande_ansikte", "1FAE8"},
    {"ansikte_med_pasar_under_ogonen", "1FAE9"},
    {"ansikte_i_narbild", "1FAEA"},
    {"slagsmal", "1FAEF"},
    {"hand_med_tummen_och_pekfingret_korsade", "1FAF0"},
    {"hand_med_tummen_och_pekfingret_korsade_farg1", "1FAF0-1F3FB"},
    {"hand_med_tummen_och_pekfingret_korsade_farg2", "1FAF0-1F3FC"},
    {"hand_med_tummen_och_pekfingret_korsade_farg3", "1FAF0-1F3FD"},
    {"hand_med_tummen_och_pekfingret_korsade_farg4", "1FAF0-1F3FE"},
    {"hand_med_tummen_och_pekfingret_korsade_farg5", "1FAF0-1F3FF"},
    {"hand_som_pekar_till_hoger", "1FAF1"},
    {"hand_som_pekar_till_hoger_farg1", "1FAF1-1F3FB"},
    {"hand_som_pekar_till_hoger_farg2", "1FAF1-1F3FC"},
    {"hand_som_pekar_till_hoger_farg3", "1FAF1-1F3FD"},
    {"hand_som_pekar_till_hoger_farg4", "1FAF1-1F3FE"},
    {"hand_som_pekar_till_hoger_farg5", "1FAF1-1F3FF"},
    {"hand_som_pekar_till_vanster", "1FAF2"},
    {"hand_som_pekar_till_vanster_farg1", "1FAF2-1F3FB"},
    {"hand_som_pekar_till_vanster_farg2", "1FAF2-1F3FC"},
    {"hand_som_pekar_till_vanster_farg3", "1FAF2-1F3FD"},
    {"hand_som_pekar_till_vanster_farg4", "1FAF2-1F3FE"},
    {"hand_som_pekar_till_vanster_farg5", "1FAF2-1F3FF"},
    {"handflatan_nedat", "1FAF3"},
    {"handflatan_nedat_farg1", "1FAF3-1F3FB"},
    {"handflatan_nedat_farg2", "1FAF3-1F3FC"},
    {"handflatan_nedat_farg3", "1FAF3-1F3FD"},
    {"handflatan_nedat_farg4", "1FAF3-1F3FE"},
    {"handflatan_nedat_farg5", "1FAF3-1F3FF"},
    {"handflatan_uppat", "1FAF4"},
    {"handflatan_uppat_farg1", "1FAF4-1F3FB"},
    {"handflatan_uppat_farg2", "1FAF4-1F3FC"},
    {"handflatan_uppat_farg3", "1FAF4-1F3FD"},
    {"handflatan_uppat_farg4", "1FAF4-1F3FE"},
    {"handflatan_uppat_farg5", "1FAF4-1F3FF"},
    {"finger_som_pekar_pa_mig", "1FAF5"},
    {"finger_som_pekar_pa_mig_farg1", "1FAF5-1F3FB"},
    {"finger_som_pekar_pa_mig_farg2", "1FAF5-1F3FC"},
    {"finger_som_pekar_pa_mig_farg3", "1FAF5-1F3FD"},
    {"finger_som_pekar_pa_mig_farg4", "1FAF5-1F3FE"},
    {"finger_som_pekar_pa_mig_farg5", "1FAF5-1F3FF"},
    {"handhjarta", "1FAF6"},
    {"handhjarta_farg1", "1FAF6-1F3FB"},
    {"handhjarta_farg2", "1FAF6-1F3FC"},
    {"handhjarta_farg3", "1FAF6-1F3FD"},
    {"handhjarta_farg4", "1FAF6-1F3FE"},
    {"handhjarta_farg5", "1FAF6-1F3FF"},
    {"hand_som_knuffar_at_vanster", "1FAF7"},
    {"hand_som_knuffar_at_vanster_farg1", "1FAF7-1F3FB"},
    {"hand_som_knuffar_at_vanster_farg2", "1FAF7-1F3FC"},
    {"hand_som_knuffar_at_vanster_farg3", "1FAF7-1F3FD"},
    {"hand_som_knuffar_at_vanster_farg4", "1FAF7-1F3FE"},
    {"hand_som_knuffar_at_vanster_farg5", "1FAF7-1F3FF"},
    {"hand_som_knuffar_at_hoger", "1FAF8"},
    {"hand_som_knuffar_at_hoger_farg1", "1FAF8-1F3FB"},
    {"hand_som_knuffar_at_hoger_farg2", "1FAF8-1F3FC"},
    {"hand_som_knuffar_at_hoger_farg3", "1FAF8-1F3FD"},
    {"hand_som_knuffar_at_hoger_farg4", "1FAF8-1F3FE"},
    {"hand_som_knuffar_at_hoger_farg5", "1FAF8-1F3FF"},
    {"keycap_nummertecken", "0023-FE0F-20E3"},
    {"keycap_asterisk", "002A-FE0F-20E3"},
    {"tangent_0", "0030-FE0F-20E3"},
    {"tangent_1", "0031-FE0F-20E3"},
    {"tangent_2", "0032-FE0F-20E3"},
    {"tangent_3", "0033-FE0F-20E3"},
    {"tangent_4", "0034-FE0F-20E3"},
    {"tangent_5", "0035-FE0F-20E3"},
    {"tangent_6", "0036-FE0F-20E3"},
    {"tangent_7", "0037-FE0F-20E3"},
    {"tangent_8", "0038-FE0F-20E3"},
    {"tangent_9", "0039-FE0F-20E3"},
    {"flagga_ascension", "1F1E6-1F1E8"},
    {"flagga_andorra", "1F1E6-1F1E9"},
    {"flagga_forenade_arabemiraten", "1F1E6-1F1EA"},
    {"flagga_afghanistan", "1F1E6-1F1EB"},
    {"flagga_antigua_och_barbuda", "1F1E6-1F1EC"},
    {"flagga_anguilla", "1F1E6-1F1EE"},
    {"flagga_albanien", "1F1E6-1F1F1"},
    {"flagga_armenien", "1F1E6-1F1F2"},
    {"flagga_angola", "1F1E6-1F1F4"},
    {"flagga_antarktis", "1F1E6-1F1F6"},
    {"flagga_argentina", "1F1E6-1F1F7"},
    {"flagga_amerikanska_samoa", "1F1E6-1F1F8"},
    {"flagga_osterrike", "1F1E6-1F1F9"},
    {"flagga_australien", "1F1E6-1F1FA"},
    {"flagga_aruba", "1F1E6-1F1FC"},
    {"flagga_aland", "1F1E6-1F1FD"},
    {"flagga_azerbajdzjan", "1F1E6-1F1FF"},
    {"flagga_bosnien_och_hercegovina", "1F1E7-1F1E6"},
    {"flagga_barbados", "1F1E7-1F1E7"},
    {"flagga_bangladesh", "1F1E7-1F1E9"},
    {"flagga_belgien", "1F1E7-1F1EA"},
    {"flagga_burkina_faso", "1F1E7-1F1EB"},
    {"flagga_bulgarien", "1F1E7-1F1EC"},
    {"flagga_bahrain", "1F1E7-1F1ED"},
    {"flagga_burundi", "1F1E7-1F1EE"},
    {"flagga_benin", "1F1E7-1F1EF"},
    {"flagga_st_barthelemy", "1F1E7-1F1F1"},
    {"flagga_bermuda", "1F1E7-1F1F2"},
    {"flagga_brunei", "1F1E7-1F1F3"},
    {"flagga_bolivia", "1F1E7-1F1F4"},
    {"flagga_karibiska_nederlanderna", "1F1E7-1F1F6"},
    {"flagga_brasilien", "1F1E7-1F1F7"},
    {"flagga_bahamas", "1F1E7-1F1F8"},
    {"flagga_bhutan", "1F1E7-1F1F9"},
    {"flagga_bouveton", "1F1E7-1F1FB"},
    {"flagga_botswana", "1F1E7-1F1FC"},
    {"flagga_belarus", "1F1E7-1F1FE"},
    {"flagga_belize", "1F1E7-1F1FF"},
    {"flagga_kanada", "1F1E8-1F1E6"},
    {"flagga_kokosoarna", "1F1E8-1F1E8"},
    {"flagga_kongo_kinshasa", "1F1E8-1F1E9"},
    {"flagga_centralafrikanska_republiken", "1F1E8-1F1EB"},
    {"flagga_kongo_brazzaville", "1F1E8-1F1EC"},
    {"flagga_schweiz", "1F1E8-1F1ED"},
    {"flagga_elfenbenskusten", "1F1E8-1F1EE"},
    {"flagga_cookoarna", "1F1E8-1F1F0"},
    {"flagga_chile", "1F1E8-1F1F1"},
    {"flagga_kamerun", "1F1E8-1F1F2"},
    {"flagga_kina", "1F1E8-1F1F3"},
    {"flagga_colombia", "1F1E8-1F1F4"},
    {"flagga_clippertonon", "1F1E8-1F1F5"},
    {"flagga_sark", "1F1E8-1F1F6"},
    {"flagga_costa_rica", "1F1E8-1F1F7"},
    {"flagga_kuba", "1F1E8-1F1FA"},
    {"flagga_kap_verde", "1F1E8-1F1FB"},
    {"flagga_curacao", "1F1E8-1F1FC"},
    {"flagga_julon", "1F1E8-1F1FD"},
    {"flagga_cypern", "1F1E8-1F1FE"},
    {"flagga_tjeckien", "1F1E8-1F1FF"},
    {"flagga_tyskland", "1F1E9-1F1EA"},
    {"flagga_diego_garcia", "1F1E9-1F1EC"},
    {"flagga_djibouti", "1F1E9-1F1EF"},
    {"flagga_danmark", "1F1E9-1F1F0"},
    {"flagga_dominica", "1F1E9-1F1F2"},
    {"flagga_dominikanska_republiken", "1F1E9-1F1F4"},
    {"flagga_algeriet", "1F1E9-1F1FF"},
    {"flagga_ceuta_och_melilla", "1F1EA-1F1E6"},
    {"flagga_ecuador", "1F1EA-1F1E8"},
    {"flagga_estland", "1F1EA-1F1EA"},
    {"flagga_egypten", "1F1EA-1F1EC"},
    {"flagga_vastsahara", "1F1EA-1F1ED"},
    {"flagga_eritrea", "1F1EA-1F1F7"},
    {"flagga_spanien", "1F1EA-1F1F8"},
    {"flagga_etiopien", "1F1EA-1F1F9"},
    {"flagga_europeiska_unionen", "1F1EA-1F1FA"},
    {"flagga_finland", "1F1EB-1F1EE"},
    {"flagga_fiji", "1F1EB-1F1EF"},
    {"flagga_falklandsoarna", "1F1EB-1F1F0"},
    {"flagga_mikronesien", "1F1EB-1F1F2"},
    {"flagga_faroarna", "1F1EB-1F1F4"},
    {"flagga_frankrike", "1F1EB-1F1F7"},
    {"flagga_gabon", "1F1EC-1F1E6"},
    {"flagga_storbritannien", "1F1EC-1F1E7"},
    {"flagga_grenada", "1F1EC-1F1E9"},
    {"flagga_georgien", "1F1EC-1F1EA"},
    {"flagga_franska_guyana", "1F1EC-1F1EB"},
    {"flagga_guernsey", "1F1EC-1F1EC"},
    {"flagga_ghana", "1F1EC-1F1ED"},
    {"flagga_gibraltar", "1F1EC-1F1EE"},
    {"flagga_gronland", "1F1EC-1F1F1"},
    {"flagga_gambia", "1F1EC-1F1F2"},
    {"flagga_guinea", "1F1EC-1F1F3"},
    {"flagga_guadeloupe", "1F1EC-1F1F5"},
    {"flagga_ekvatorialguinea", "1F1EC-1F1F6"},
    {"flagga_grekland", "1F1EC-1F1F7"},
    {"flagga_sydgeorgien_och_sydsandwichoarna", "1F1EC-1F1F8"},
    {"flagga_guatemala", "1F1EC-1F1F9"},
    {"flagga_guam", "1F1EC-1F1FA"},
    {"flagga_guinea_bissau", "1F1EC-1F1FC"},
    {"flagga_guyana", "1F1EC-1F1FE"},
    {"flagga_hongkong_sar", "1F1ED-1F1F0"},
    {"flagga_heardon_och_mcdonaldoarna", "1F1ED-1F1F2"},
    {"flagga_honduras", "1F1ED-1F1F3"},
    {"flagga_kroatien", "1F1ED-1F1F7"},
    {"flagga_haiti", "1F1ED-1F1F9"},
    {"flagga_ungern", "1F1ED-1F1FA"},
    {"flagga_kanarieoarna", "1F1EE-1F1E8"},
    {"flagga_indonesien", "1F1EE-1F1E9"},
    {"flagga_irland", "1F1EE-1F1EA"},
    {"flagga_israel", "1F1EE-1F1F1"},
    {"flagga_isle_of_man", "1F1EE-1F1F2"},
    {"flagga_indien", "1F1EE-1F1F3"},
    {"flagga_brittiska_territoriet_i_indiska_oceanen", "1F1EE-1F1F4"},
    {"flagga_irak", "1F1EE-1F1F6"},
    {"flagga_iran", "1F1EE-1F1F7"},
    {"flagga_island", "1F1EE-1F1F8"},
    {"flagga_italien", "1F1EE-1F1F9"},
    {"flagga_jersey", "1F1EF-1F1EA"},
    {"flagga_jamaica", "1F1EF-1F1F2"},
    {"flagga_jordanien", "1F1EF-1F1F4"},
    {"flagga_japan", "1F1EF-1F1F5"},
    {"flagga_kenya", "1F1F0-1F1EA"},
    {"flagga_kirgizistan", "1F1F0-1F1EC"},
    {"flagga_kambodja", "1F1F0-1F1ED"},
    {"flagga_kiribati", "1F1F0-1F1EE"},
    {"flagga_komorerna", "1F1F0-1F1F2"},
    {"flagga_st_kitts_och_nevis", "1F1F0-1F1F3"},
    {"flagga_nordkorea", "1F1F0-1F1F5"},
    {"flagga_sydkorea", "1F1F0-1F1F7"},
    {"flagga_kuwait", "1F1F0-1F1FC"},
    {"flagga_caymanoarna", "1F1F0-1F1FE"},
    {"flagga_kazakstan", "1F1F0-1F1FF"},
    {"flagga_laos", "1F1F1-1F1E6"},
    {"flagga_libanon", "1F1F1-1F1E7"},
    {"flagga_st_lucia", "1F1F1-1F1E8"},
    {"flagga_liechtenstein", "1F1F1-1F1EE"},
    {"flagga_sri_lanka", "1F1F1-1F1F0"},
    {"flagga_liberia", "1F1F1-1F1F7"},
    {"flagga_lesotho", "1F1F1-1F1F8"},
    {"flagga_litauen", "1F1F1-1F1F9"},
    {"flagga_luxemburg", "1F1F1-1F1FA"},
    {"flagga_lettland", "1F1F1-1F1FB"},
    {"flagga_libyen", "1F1F1-1F1FE"},
    {"flagga_marocko", "1F1F2-1F1E6"},
    {"flagga_monaco", "1F1F2-1F1E8"},
    {"flagga_moldavien", "1F1F2-1F1E9"},
    {"flagga_montenegro", "1F1F2-1F1EA"},
    {"flagga_saint_martin", "1F1F2-1F1EB"},
    {"flagga_madagaskar", "1F1F2-1F1EC"},
    {"flagga_marshalloarna", "1F1F2-1F1ED"},
    {"flagga_nordmakedonien", "1F1F2-1F1F0"},
    {"flagga_mali", "1F1F2-1F1F1"},
    {"flagga_myanmar_burma", "1F1F2-1F1F2"},
    {"flagga_mongoliet", "1F1F2-1F1F3"},
    {"flagga_macao_sar", "1F1F2-1F1F4"},
    {"flagga_nordmarianerna", "1F1F2-1F1F5"},
    {"flagga_martinique", "1F1F2-1F1F6"},
    {"flagga_mauretanien", "1F1F2-1F1F7"},
    {"flagga_montserrat", "1F1F2-1F1F8"},
    {"flagga_malta", "1F1F2-1F1F9"},
    {"flagga_mauritius", "1F1F2-1F1FA"},
    {"flagga_maldiverna", "1F1F2-1F1FB"},
    {"flagga_malawi", "1F1F2-1F1FC"},
    {"flagga_mexiko", "1F1F2-1F1FD"},
    {"flagga_malaysia", "1F1F2-1F1FE"},
    {"flagga_mocambique", "1F1F2-1F1FF"},
    {"flagga_namibia", "1F1F3-1F1E6"},
    {"flagga_nya_kaledonien", "1F1F3-1F1E8"},
    {"flagga_niger", "1F1F3-1F1EA"},
    {"flagga_norfolkon", "1F1F3-1F1EB"},
    {"flagga_nigeria", "1F1F3-1F1EC"},
    {"flagga_nicaragua", "1F1F3-1F1EE"},
    {"flagga_nederlanderna", "1F1F3-1F1F1"},
    {"flagga_norge", "1F1F3-1F1F4"},
    {"flagga_nepal", "1F1F3-1F1F5"},
    {"flagga_nauru", "1F1F3-1F1F7"},
    {"flagga_niue", "1F1F3-1F1FA"},
    {"flagga_nya_zeeland", "1F1F3-1F1FF"},
    {"flagga_oman", "1F1F4-1F1F2"},
    {"flagga_panama", "1F1F5-1F1E6"},
    {"flagga_peru", "1F1F5-1F1EA"},
    {"flagga_franska_polynesien", "1F1F5-1F1EB"},
    {"flagga_papua_nya_guinea", "1F1F5-1F1EC"},
    {"flagga_filippinerna", "1F1F5-1F1ED"},
    {"flagga_pakistan", "1F1F5-1F1F0"},
    {"flagga_polen", "1F1F5-1F1F1"},
    {"flagga_st_pierre_och_miquelon", "1F1F5-1F1F2"},
    {"flagga_pitcairnoarna", "1F1F5-1F1F3"},
    {"flagga_puerto_rico", "1F1F5-1F1F7"},
    {"flagga_palestinska_territorierna", "1F1F5-1F1F8"},
    {"flagga_portugal", "1F1F5-1F1F9"},
    {"flagga_palau", "1F1F5-1F1FC"},
    {"flagga_paraguay", "1F1F5-1F1FE"},
    {"flagga_qatar", "1F1F6-1F1E6"},
    {"flagga_reunion", "1F1F7-1F1EA"},
    {"flagga_rumanien", "1F1F7-1F1F4"},
    {"flagga_serbien", "1F1F7-1F1F8"},
    {"flagga_ryssland", "1F1F7-1F1FA"},
    {"flagga_rwanda", "1F1F7-1F1FC"},
    {"flagga_saudiarabien", "1F1F8-1F1E6"},
    {"flagga_salomonoarna", "1F1F8-1F1E7"},
    {"flagga_seychellerna", "1F1F8-1F1E8"},
    {"flagga_sudan", "1F1F8-1F1E9"},
    {"flagga_sverige", "1F1F8-1F1EA"},
    {"flagga_singapore", "1F1F8-1F1EC"},
    {"flagga_st_helena", "1F1F8-1F1ED"},
    {"flagga_slovenien", "1F1F8-1F1EE"},
    {"flagga_svalbard_och_jan_mayen", "1F1F8-1F1EF"},
    {"flagga_slovakien", "1F1F8-1F1F0"},
    {"flagga_sierra_leone", "1F1F8-1F1F1"},
    {"flagga_san_marino", "1F1F8-1F1F2"},
    {"flagga_senegal", "1F1F8-1F1F3"},
    {"flagga_somalia", "1F1F8-1F1F4"},
    {"flagga_surinam", "1F1F8-1F1F7"},
    {"flagga_sydsudan", "1F1F8-1F1F8"},
    {"flagga_sao_tome_och_principe", "1F1F8-1F1F9"},
    {"flagga_el_salvador", "1F1F8-1F1FB"},
    {"flagga_sint_maarten", "1F1F8-1F1FD"},
    {"flagga_syrien", "1F1F8-1F1FE"},
    {"flagga_eswatini", "1F1F8-1F1FF"},
    {"flagga_tristan_da_cunha", "1F1F9-1F1E6"},
    {"flagga_turks_och_caicosoarna", "1F1F9-1F1E8"},
    {"flagga_tchad", "1F1F9-1F1E9"},
    {"flagga_franska_sydterritorierna", "1F1F9-1F1EB"},
    {"flagga_togo", "1F1F9-1F1EC"},
    {"flagga_thailand", "1F1F9-1F1ED"},
    {"flagga_tadzjikistan", "1F1F9-1F1EF"},
    {"flagga_tokelauoarna", "1F1F9-1F1F0"},
    {"flagga_osttimor", "1F1F9-1F1F1"},
    {"flagga_turkmenistan", "1F1F9-1F1F2"},
    {"flagga_tunisien", "1F1F9-1F1F3"},
    {"flagga_tonga", "1F1F9-1F1F4"},
    {"flagga_turkiet", "1F1F9-1F1F7"},
    {"flagga_trinidad_och_tobago", "1F1F9-1F1F9"},
    {"flagga_tuvalu", "1F1F9-1F1FB"},
    {"flagga_taiwan", "1F1F9-1F1FC"},
    {"flagga_tanzania", "1F1F9-1F1FF"},
    {"flagga_ukraina", "1F1FA-1F1E6"},
    {"flagga_uganda", "1F1FA-1F1EC"},
    {"flagga_usas_yttre_oar", "1F1FA-1F1F2"},
    {"flagga_forenta_nationerna", "1F1FA-1F1F3"},
    {"flagga_usa", "1F1FA-1F1F8"},
    {"flagga_uruguay", "1F1FA-1F1FE"},
    {"flagga_uzbekistan", "1F1FA-1F1FF"},
    {"flagga_vatikanstaten", "1F1FB-1F1E6"},
    {"flagga_st_vincent_och_grenadinerna", "1F1FB-1F1E8"},
    {"flagga_venezuela", "1F1FB-1F1EA"},
    {"flagga_brittiska_jungfruoarna", "1F1FB-1F1EC"},
    {"flagga_amerikanska_jungfruoarna", "1F1FB-1F1EE"},
    {"flagga_vietnam", "1F1FB-1F1F3"},
    {"flagga_vanuatu", "1F1FB-1F1FA"},
    {"flagga_wallis_och_futunaoarna", "1F1FC-1F1EB"},
    {"flagga_samoa", "1F1FC-1F1F8"},
    {"flagga_kosovo", "1F1FD-1F1F0"},
    {"flagga_jemen", "1F1FE-1F1EA"},
    {"flagga_mayotte", "1F1FE-1F1F9"},
    {"flagga_sydafrika", "1F1FF-1F1E6"},
    {"flagga_zambia", "1F1FF-1F1F2"},
    {"flagga_zimbabwe", "1F1FF-1F1FC"},
    {"flagga_england", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"flagga_skottland", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"flagga_wales", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"par_med_hjarta_man_man", "1F468-200D-2764-FE0F-200D-1F468"},
    {"par_med_hjarta_man_man_farg1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjarta_man_man_farg1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjarta_man_man_farg1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjarta_man_man_farg1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjarta_man_man_farg1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjarta_man_man_farg2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjarta_man_man_farg2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjarta_man_man_farg2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjarta_man_man_farg2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjarta_man_man_farg2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjarta_man_man_farg3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjarta_man_man_farg3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjarta_man_man_farg3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjarta_man_man_farg3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjarta_man_man_farg3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjarta_man_man_farg4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjarta_man_man_farg4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjarta_man_man_farg4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjarta_man_man_farg4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjarta_man_man_farg4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjarta_man_man_farg5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjarta_man_man_farg5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjarta_man_man_farg5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjarta_man_man_farg5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjarta_man_man_farg5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"puss_man_man", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"puss_man_man_farg1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"puss_man_man_farg1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"puss_man_man_farg1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"puss_man_man_farg1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"puss_man_man_farg1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"puss_man_man_farg2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"puss_man_man_farg2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"puss_man_man_farg2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"puss_man_man_farg2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"puss_man_man_farg2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"puss_man_man_farg3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"puss_man_man_farg3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"puss_man_man_farg3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"puss_man_man_farg3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"puss_man_man_farg3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"puss_man_man_farg4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"puss_man_man_farg4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"puss_man_man_farg4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"puss_man_man_farg4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"puss_man_man_farg4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"puss_man_man_farg5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"puss_man_man_farg5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"puss_man_man_farg5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"puss_man_man_farg5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"puss_man_man_farg5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"familj_man_pojke", "1F468-200D-1F466"},
    {"familj_man_pojke_pojke", "1F468-200D-1F466-200D-1F466"},
    {"familj_man_flicka", "1F468-200D-1F467"},
    {"familj_man_flicka_pojke", "1F468-200D-1F467-200D-1F466"},
    {"familj_man_flicka_flicka", "1F468-200D-1F467-200D-1F467"},
    {"familj_man_man_pojke", "1F468-200D-1F468-200D-1F466"},
    {"familj_man_man_pojke_pojke", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"familj_man_man_flicka", "1F468-200D-1F468-200D-1F467"},
    {"familj_man_man_flicka_pojke", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"familj_man_man_flicka_flicka", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"familj_man_kvinna_pojke", "1F468-200D-1F469-200D-1F466"},
    {"familj_man_kvinna_pojke_pojke", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"familj_man_kvinna_flicka", "1F468-200D-1F469-200D-1F467"},
    {"familj_man_kvinna_flicka_pojke", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"familj_man_kvinna_flicka_flicka", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"par_med_hjarta_kvinna_man", "1F469-200D-2764-FE0F-200D-1F468"},
    {"par_med_hjarta_kvinna_man_farg1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjarta_kvinna_man_farg1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjarta_kvinna_man_farg1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjarta_kvinna_man_farg1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjarta_kvinna_man_farg1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjarta_kvinna_man_farg2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjarta_kvinna_man_farg2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjarta_kvinna_man_farg2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjarta_kvinna_man_farg2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjarta_kvinna_man_farg2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjarta_kvinna_man_farg3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjarta_kvinna_man_farg3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjarta_kvinna_man_farg3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjarta_kvinna_man_farg3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjarta_kvinna_man_farg3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjarta_kvinna_man_farg4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjarta_kvinna_man_farg4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjarta_kvinna_man_farg4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjarta_kvinna_man_farg4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjarta_kvinna_man_farg4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjarta_kvinna_man_farg5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjarta_kvinna_man_farg5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjarta_kvinna_man_farg5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjarta_kvinna_man_farg5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjarta_kvinna_man_farg5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjarta_kvinna_kvinna", "1F469-200D-2764-FE0F-200D-1F469"},
    {"par_med_hjarta_kvinna_kvinna_farg1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjarta_kvinna_kvinna_farg1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjarta_kvinna_kvinna_farg1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjarta_kvinna_kvinna_farg1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjarta_kvinna_kvinna_farg1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjarta_kvinna_kvinna_farg2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjarta_kvinna_kvinna_farg2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjarta_kvinna_kvinna_farg2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjarta_kvinna_kvinna_farg2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjarta_kvinna_kvinna_farg2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjarta_kvinna_kvinna_farg3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjarta_kvinna_kvinna_farg3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjarta_kvinna_kvinna_farg3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjarta_kvinna_kvinna_farg3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjarta_kvinna_kvinna_farg3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjarta_kvinna_kvinna_farg4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjarta_kvinna_kvinna_farg4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjarta_kvinna_kvinna_farg4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjarta_kvinna_kvinna_farg4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjarta_kvinna_kvinna_farg4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjarta_kvinna_kvinna_farg5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjarta_kvinna_kvinna_farg5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjarta_kvinna_kvinna_farg5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjarta_kvinna_kvinna_farg5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjarta_kvinna_kvinna_farg5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"puss_kvinna_man", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"puss_kvinna_man_farg1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"puss_kvinna_man_farg1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"puss_kvinna_man_farg1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"puss_kvinna_man_farg1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"puss_kvinna_man_farg1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"puss_kvinna_man_farg2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"puss_kvinna_man_farg2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"puss_kvinna_man_farg2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"puss_kvinna_man_farg2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"puss_kvinna_man_farg2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"puss_kvinna_man_farg3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"puss_kvinna_man_farg3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"puss_kvinna_man_farg3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"puss_kvinna_man_farg3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"puss_kvinna_man_farg3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"puss_kvinna_man_farg4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"puss_kvinna_man_farg4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"puss_kvinna_man_farg4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"puss_kvinna_man_farg4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"puss_kvinna_man_farg4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"puss_kvinna_man_farg5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"puss_kvinna_man_farg5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"puss_kvinna_man_farg5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"puss_kvinna_man_farg5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"puss_kvinna_man_farg5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"puss_kvinna_kvinna", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"puss_kvinna_kvinna_farg1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"puss_kvinna_kvinna_farg1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"puss_kvinna_kvinna_farg1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"puss_kvinna_kvinna_farg1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"puss_kvinna_kvinna_farg1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"puss_kvinna_kvinna_farg2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"puss_kvinna_kvinna_farg2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"puss_kvinna_kvinna_farg2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"puss_kvinna_kvinna_farg2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"puss_kvinna_kvinna_farg2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"puss_kvinna_kvinna_farg3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"puss_kvinna_kvinna_farg3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"puss_kvinna_kvinna_farg3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"puss_kvinna_kvinna_farg3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"puss_kvinna_kvinna_farg3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"puss_kvinna_kvinna_farg4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"puss_kvinna_kvinna_farg4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"puss_kvinna_kvinna_farg4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"puss_kvinna_kvinna_farg4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"puss_kvinna_kvinna_farg4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"puss_kvinna_kvinna_farg5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"puss_kvinna_kvinna_farg5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"puss_kvinna_kvinna_farg5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"puss_kvinna_kvinna_farg5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"puss_kvinna_kvinna_farg5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"familj_kvinna_pojke", "1F469-200D-1F466"},
    {"familj_kvinna_pojke_pojke", "1F469-200D-1F466-200D-1F466"},
    {"familj_kvinna_flicka", "1F469-200D-1F467"},
    {"familj_kvinna_flicka_pojke", "1F469-200D-1F467-200D-1F466"},
    {"familj_kvinna_flicka_flicka", "1F469-200D-1F467-200D-1F467"},
    {"familj_kvinna_kvinna_pojke", "1F469-200D-1F469-200D-1F466"},
    {"familj_kvinna_kvinna_pojke_pojke", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"familj_kvinna_kvinna_flicka", "1F469-200D-1F469-200D-1F467"},
    {"familj_kvinna_kvinna_flicka_pojke", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"familj_kvinna_kvinna_flicka_flicka", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"personer_som_haller_hand", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"personer_som_haller_hand_farg1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_haller_hand_farg1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_haller_hand_farg1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_haller_hand_farg1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_haller_hand_farg1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_som_haller_hand_farg2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_haller_hand_farg2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_haller_hand_farg2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_haller_hand_farg2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_haller_hand_farg2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_som_haller_hand_farg3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_haller_hand_farg3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_haller_hand_farg3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_haller_hand_farg3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_haller_hand_farg3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_som_haller_hand_farg4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_haller_hand_farg4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_haller_hand_farg4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_haller_hand_farg4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_haller_hand_farg4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_som_haller_hand_farg5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_haller_hand_farg5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_haller_hand_farg5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_haller_hand_farg5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_haller_hand_farg5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"familj_vuxen_vuxen_barn", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"familj_vuxen_vuxen_barn_barn", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"familj_vuxen_barn", "1F9D1-200D-1F9D2"},
    {"familj_vuxen_barn_barn", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"person_som_springer_at_hoger", "1F3C3-200D-27A1-FE0F"},
    {"person_som_springer_at_hoger_farg1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"person_som_springer_at_hoger_farg2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"person_som_springer_at_hoger_farg3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"person_som_springer_at_hoger_farg4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"person_som_springer_at_hoger_farg5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"manlig_lakare", "1F468-200D-2695-FE0F"},
    {"manlig_lakare_farg1", "1F468-1F3FB-200D-2695-FE0F"},
    {"manlig_lakare_farg2", "1F468-1F3FC-200D-2695-FE0F"},
    {"manlig_lakare_farg3", "1F468-1F3FD-200D-2695-FE0F"},
    {"manlig_lakare_farg4", "1F468-1F3FE-200D-2695-FE0F"},
    {"manlig_lakare_farg5", "1F468-1F3FF-200D-2695-FE0F"},
    {"manlig_domare", "1F468-200D-2696-FE0F"},
    {"manlig_domare_farg1", "1F468-1F3FB-200D-2696-FE0F"},
    {"manlig_domare_farg2", "1F468-1F3FC-200D-2696-FE0F"},
    {"manlig_domare_farg3", "1F468-1F3FD-200D-2696-FE0F"},
    {"manlig_domare_farg4", "1F468-1F3FE-200D-2696-FE0F"},
    {"manlig_domare_farg5", "1F468-1F3FF-200D-2696-FE0F"},
    {"manlig_pilot", "1F468-200D-2708-FE0F"},
    {"manlig_pilot_farg1", "1F468-1F3FB-200D-2708-FE0F"},
    {"manlig_pilot_farg2", "1F468-1F3FC-200D-2708-FE0F"},
    {"manlig_pilot_farg3", "1F468-1F3FD-200D-2708-FE0F"},
    {"manlig_pilot_farg4", "1F468-1F3FE-200D-2708-FE0F"},
    {"manlig_pilot_farg5", "1F468-1F3FF-200D-2708-FE0F"},
    {"manlig_bonde", "1F468-200D-1F33E"},
    {"manlig_bonde_farg1", "1F468-1F3FB-200D-1F33E"},
    {"manlig_bonde_farg2", "1F468-1F3FC-200D-1F33E"},
    {"manlig_bonde_farg3", "1F468-1F3FD-200D-1F33E"},
    {"manlig_bonde_farg4", "1F468-1F3FE-200D-1F33E"},
    {"manlig_bonde_farg5", "1F468-1F3FF-200D-1F33E"},
    {"manlig_kock", "1F468-200D-1F373"},
    {"manlig_kock_farg1", "1F468-1F3FB-200D-1F373"},
    {"manlig_kock_farg2", "1F468-1F3FC-200D-1F373"},
    {"manlig_kock_farg3", "1F468-1F3FD-200D-1F373"},
    {"manlig_kock_farg4", "1F468-1F3FE-200D-1F373"},
    {"manlig_kock_farg5", "1F468-1F3FF-200D-1F373"},
    {"man_matar_spadbarn", "1F468-200D-1F37C"},
    {"man_matar_spadbarn_farg1", "1F468-1F3FB-200D-1F37C"},
    {"man_matar_spadbarn_farg2", "1F468-1F3FC-200D-1F37C"},
    {"man_matar_spadbarn_farg3", "1F468-1F3FD-200D-1F37C"},
    {"man_matar_spadbarn_farg4", "1F468-1F3FE-200D-1F37C"},
    {"man_matar_spadbarn_farg5", "1F468-1F3FF-200D-1F37C"},
    {"manlig_student", "1F468-200D-1F393"},
    {"manlig_student_farg1", "1F468-1F3FB-200D-1F393"},
    {"manlig_student_farg2", "1F468-1F3FC-200D-1F393"},
    {"manlig_student_farg3", "1F468-1F3FD-200D-1F393"},
    {"manlig_student_farg4", "1F468-1F3FE-200D-1F393"},
    {"manlig_student_farg5", "1F468-1F3FF-200D-1F393"},
    {"manlig_sangare", "1F468-200D-1F3A4"},
    {"manlig_sangare_farg1", "1F468-1F3FB-200D-1F3A4"},
    {"manlig_sangare_farg2", "1F468-1F3FC-200D-1F3A4"},
    {"manlig_sangare_farg3", "1F468-1F3FD-200D-1F3A4"},
    {"manlig_sangare_farg4", "1F468-1F3FE-200D-1F3A4"},
    {"manlig_sangare_farg5", "1F468-1F3FF-200D-1F3A4"},
    {"manlig_konstnar", "1F468-200D-1F3A8"},
    {"manlig_konstnar_farg1", "1F468-1F3FB-200D-1F3A8"},
    {"manlig_konstnar_farg2", "1F468-1F3FC-200D-1F3A8"},
    {"manlig_konstnar_farg3", "1F468-1F3FD-200D-1F3A8"},
    {"manlig_konstnar_farg4", "1F468-1F3FE-200D-1F3A8"},
    {"manlig_konstnar_farg5", "1F468-1F3FF-200D-1F3A8"},
    {"manlig_larare", "1F468-200D-1F3EB"},
    {"manlig_larare_farg1", "1F468-1F3FB-200D-1F3EB"},
    {"manlig_larare_farg2", "1F468-1F3FC-200D-1F3EB"},
    {"manlig_larare_farg3", "1F468-1F3FD-200D-1F3EB"},
    {"manlig_larare_farg4", "1F468-1F3FE-200D-1F3EB"},
    {"manlig_larare_farg5", "1F468-1F3FF-200D-1F3EB"},
    {"manlig_fabriksarbetare", "1F468-200D-1F3ED"},
    {"manlig_fabriksarbetare_farg1", "1F468-1F3FB-200D-1F3ED"},
    {"manlig_fabriksarbetare_farg2", "1F468-1F3FC-200D-1F3ED"},
    {"manlig_fabriksarbetare_farg3", "1F468-1F3FD-200D-1F3ED"},
    {"manlig_fabriksarbetare_farg4", "1F468-1F3FE-200D-1F3ED"},
    {"manlig_fabriksarbetare_farg5", "1F468-1F3FF-200D-1F3ED"},
    {"manlig_it_expert", "1F468-200D-1F4BB"},
    {"manlig_it_expert_farg1", "1F468-1F3FB-200D-1F4BB"},
    {"manlig_it_expert_farg2", "1F468-1F3FC-200D-1F4BB"},
    {"manlig_it_expert_farg3", "1F468-1F3FD-200D-1F4BB"},
    {"manlig_it_expert_farg4", "1F468-1F3FE-200D-1F4BB"},
    {"manlig_it_expert_farg5", "1F468-1F3FF-200D-1F4BB"},
    {"kontorstjansteman", "1F468-200D-1F4BC"},
    {"kontorstjansteman_farg1", "1F468-1F3FB-200D-1F4BC"},
    {"kontorstjansteman_farg2", "1F468-1F3FC-200D-1F4BC"},
    {"kontorstjansteman_farg3", "1F468-1F3FD-200D-1F4BC"},
    {"kontorstjansteman_farg4", "1F468-1F3FE-200D-1F4BC"},
    {"kontorstjansteman_farg5", "1F468-1F3FF-200D-1F4BC"},
    {"manlig_mekaniker", "1F468-200D-1F527"},
    {"manlig_mekaniker_farg1", "1F468-1F3FB-200D-1F527"},
    {"manlig_mekaniker_farg2", "1F468-1F3FC-200D-1F527"},
    {"manlig_mekaniker_farg3", "1F468-1F3FD-200D-1F527"},
    {"manlig_mekaniker_farg4", "1F468-1F3FE-200D-1F527"},
    {"manlig_mekaniker_farg5", "1F468-1F3FF-200D-1F527"},
    {"manlig_forskare", "1F468-200D-1F52C"},
    {"manlig_forskare_farg1", "1F468-1F3FB-200D-1F52C"},
    {"manlig_forskare_farg2", "1F468-1F3FC-200D-1F52C"},
    {"manlig_forskare_farg3", "1F468-1F3FD-200D-1F52C"},
    {"manlig_forskare_farg4", "1F468-1F3FE-200D-1F52C"},
    {"manlig_forskare_farg5", "1F468-1F3FF-200D-1F52C"},
    {"manlig_astronaut", "1F468-200D-1F680"},
    {"manlig_astronaut_farg1", "1F468-1F3FB-200D-1F680"},
    {"manlig_astronaut_farg2", "1F468-1F3FC-200D-1F680"},
    {"manlig_astronaut_farg3", "1F468-1F3FD-200D-1F680"},
    {"manlig_astronaut_farg4", "1F468-1F3FE-200D-1F680"},
    {"manlig_astronaut_farg5", "1F468-1F3FF-200D-1F680"},
    {"manlig_brandman", "1F468-200D-1F692"},
    {"manlig_brandman_farg1", "1F468-1F3FB-200D-1F692"},
    {"manlig_brandman_farg2", "1F468-1F3FC-200D-1F692"},
    {"manlig_brandman_farg3", "1F468-1F3FD-200D-1F692"},
    {"manlig_brandman_farg4", "1F468-1F3FE-200D-1F692"},
    {"manlig_brandman_farg5", "1F468-1F3FF-200D-1F692"},
    {"man_med_vit_kapp", "1F468-200D-1F9AF"},
    {"man_med_vit_kapp_farg1", "1F468-1F3FB-200D-1F9AF"},
    {"man_med_vit_kapp_farg2", "1F468-1F3FC-200D-1F9AF"},
    {"man_med_vit_kapp_farg3", "1F468-1F3FD-200D-1F9AF"},
    {"man_med_vit_kapp_farg4", "1F468-1F3FE-200D-1F9AF"},
    {"man_med_vit_kapp_farg5", "1F468-1F3FF-200D-1F9AF"},
    {"man_med_vit_kapp_at_hoger", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"man_med_vit_kapp_at_hoger_farg1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"man_med_vit_kapp_at_hoger_farg2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"man_med_vit_kapp_at_hoger_farg3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"man_med_vit_kapp_at_hoger_farg4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"man_med_vit_kapp_at_hoger_farg5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"man_i_permobil", "1F468-200D-1F9BC"},
    {"man_i_permobil_farg1", "1F468-1F3FB-200D-1F9BC"},
    {"man_i_permobil_farg2", "1F468-1F3FC-200D-1F9BC"},
    {"man_i_permobil_farg3", "1F468-1F3FD-200D-1F9BC"},
    {"man_i_permobil_farg4", "1F468-1F3FE-200D-1F9BC"},
    {"man_i_permobil_farg5", "1F468-1F3FF-200D-1F9BC"},
    {"man_i_permobil_at_hoger", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"man_i_permobil_at_hoger_farg1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"man_i_permobil_at_hoger_farg2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"man_i_permobil_at_hoger_farg3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"man_i_permobil_at_hoger_farg4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"man_i_permobil_at_hoger_farg5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"man_i_manuell_rullstol", "1F468-200D-1F9BD"},
    {"man_i_manuell_rullstol_farg1", "1F468-1F3FB-200D-1F9BD"},
    {"man_i_manuell_rullstol_farg2", "1F468-1F3FC-200D-1F9BD"},
    {"man_i_manuell_rullstol_farg3", "1F468-1F3FD-200D-1F9BD"},
    {"man_i_manuell_rullstol_farg4", "1F468-1F3FE-200D-1F9BD"},
    {"man_i_manuell_rullstol_farg5", "1F468-1F3FF-200D-1F9BD"},
    {"man_i_manuell_rullstol_at_hoger", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"man_i_manuell_rullstol_at_hoger_farg1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"man_i_manuell_rullstol_at_hoger_farg2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"man_i_manuell_rullstol_at_hoger_farg3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"man_i_manuell_rullstol_at_hoger_farg4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"man_i_manuell_rullstol_at_hoger_farg5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinnlig_lakare", "1F469-200D-2695-FE0F"},
    {"kvinnlig_lakare_farg1", "1F469-1F3FB-200D-2695-FE0F"},
    {"kvinnlig_lakare_farg2", "1F469-1F3FC-200D-2695-FE0F"},
    {"kvinnlig_lakare_farg3", "1F469-1F3FD-200D-2695-FE0F"},
    {"kvinnlig_lakare_farg4", "1F469-1F3FE-200D-2695-FE0F"},
    {"kvinnlig_lakare_farg5", "1F469-1F3FF-200D-2695-FE0F"},
    {"kvinnlig_domare", "1F469-200D-2696-FE0F"},
    {"kvinnlig_domare_farg1", "1F469-1F3FB-200D-2696-FE0F"},
    {"kvinnlig_domare_farg2", "1F469-1F3FC-200D-2696-FE0F"},
    {"kvinnlig_domare_farg3", "1F469-1F3FD-200D-2696-FE0F"},
    {"kvinnlig_domare_farg4", "1F469-1F3FE-200D-2696-FE0F"},
    {"kvinnlig_domare_farg5", "1F469-1F3FF-200D-2696-FE0F"},
    {"kvinnlig_pilot", "1F469-200D-2708-FE0F"},
    {"kvinnlig_pilot_farg1", "1F469-1F3FB-200D-2708-FE0F"},
    {"kvinnlig_pilot_farg2", "1F469-1F3FC-200D-2708-FE0F"},
    {"kvinnlig_pilot_farg3", "1F469-1F3FD-200D-2708-FE0F"},
    {"kvinnlig_pilot_farg4", "1F469-1F3FE-200D-2708-FE0F"},
    {"kvinnlig_pilot_farg5", "1F469-1F3FF-200D-2708-FE0F"},
    {"kvinnlig_bonde", "1F469-200D-1F33E"},
    {"kvinnlig_bonde_farg1", "1F469-1F3FB-200D-1F33E"},
    {"kvinnlig_bonde_farg2", "1F469-1F3FC-200D-1F33E"},
    {"kvinnlig_bonde_farg3", "1F469-1F3FD-200D-1F33E"},
    {"kvinnlig_bonde_farg4", "1F469-1F3FE-200D-1F33E"},
    {"kvinnlig_bonde_farg5", "1F469-1F3FF-200D-1F33E"},
    {"kvinnlig_kock", "1F469-200D-1F373"},
    {"kvinnlig_kock_farg1", "1F469-1F3FB-200D-1F373"},
    {"kvinnlig_kock_farg2", "1F469-1F3FC-200D-1F373"},
    {"kvinnlig_kock_farg3", "1F469-1F3FD-200D-1F373"},
    {"kvinnlig_kock_farg4", "1F469-1F3FE-200D-1F373"},
    {"kvinnlig_kock_farg5", "1F469-1F3FF-200D-1F373"},
    {"kvinna_matar_spadbarn", "1F469-200D-1F37C"},
    {"kvinna_matar_spadbarn_farg1", "1F469-1F3FB-200D-1F37C"},
    {"kvinna_matar_spadbarn_farg2", "1F469-1F3FC-200D-1F37C"},
    {"kvinna_matar_spadbarn_farg3", "1F469-1F3FD-200D-1F37C"},
    {"kvinna_matar_spadbarn_farg4", "1F469-1F3FE-200D-1F37C"},
    {"kvinna_matar_spadbarn_farg5", "1F469-1F3FF-200D-1F37C"},
    {"kvinnlig_student", "1F469-200D-1F393"},
    {"kvinnlig_student_farg1", "1F469-1F3FB-200D-1F393"},
    {"kvinnlig_student_farg2", "1F469-1F3FC-200D-1F393"},
    {"kvinnlig_student_farg3", "1F469-1F3FD-200D-1F393"},
    {"kvinnlig_student_farg4", "1F469-1F3FE-200D-1F393"},
    {"kvinnlig_student_farg5", "1F469-1F3FF-200D-1F393"},
    {"kvinnlig_sangare", "1F469-200D-1F3A4"},
    {"kvinnlig_sangare_farg1", "1F469-1F3FB-200D-1F3A4"},
    {"kvinnlig_sangare_farg2", "1F469-1F3FC-200D-1F3A4"},
    {"kvinnlig_sangare_farg3", "1F469-1F3FD-200D-1F3A4"},
    {"kvinnlig_sangare_farg4", "1F469-1F3FE-200D-1F3A4"},
    {"kvinnlig_sangare_farg5", "1F469-1F3FF-200D-1F3A4"},
    {"kvinnlig_konstnar", "1F469-200D-1F3A8"},
    {"kvinnlig_konstnar_farg1", "1F469-1F3FB-200D-1F3A8"},
    {"kvinnlig_konstnar_farg2", "1F469-1F3FC-200D-1F3A8"},
    {"kvinnlig_konstnar_farg3", "1F469-1F3FD-200D-1F3A8"},
    {"kvinnlig_konstnar_farg4", "1F469-1F3FE-200D-1F3A8"},
    {"kvinnlig_konstnar_farg5", "1F469-1F3FF-200D-1F3A8"},
    {"kvinnlig_larare", "1F469-200D-1F3EB"},
    {"kvinnlig_larare_farg1", "1F469-1F3FB-200D-1F3EB"},
    {"kvinnlig_larare_farg2", "1F469-1F3FC-200D-1F3EB"},
    {"kvinnlig_larare_farg3", "1F469-1F3FD-200D-1F3EB"},
    {"kvinnlig_larare_farg4", "1F469-1F3FE-200D-1F3EB"},
    {"kvinnlig_larare_farg5", "1F469-1F3FF-200D-1F3EB"},
    {"kvinnlig_fabriksarbetare", "1F469-200D-1F3ED"},
    {"kvinnlig_fabriksarbetare_farg1", "1F469-1F3FB-200D-1F3ED"},
    {"kvinnlig_fabriksarbetare_farg2", "1F469-1F3FC-200D-1F3ED"},
    {"kvinnlig_fabriksarbetare_farg3", "1F469-1F3FD-200D-1F3ED"},
    {"kvinnlig_fabriksarbetare_farg4", "1F469-1F3FE-200D-1F3ED"},
    {"kvinnlig_fabriksarbetare_farg5", "1F469-1F3FF-200D-1F3ED"},
    {"kvinnlig_it_expert", "1F469-200D-1F4BB"},
    {"kvinnlig_it_expert_farg1", "1F469-1F3FB-200D-1F4BB"},
    {"kvinnlig_it_expert_farg2", "1F469-1F3FC-200D-1F4BB"},
    {"kvinnlig_it_expert_farg3", "1F469-1F3FD-200D-1F4BB"},
    {"kvinnlig_it_expert_farg4", "1F469-1F3FE-200D-1F4BB"},
    {"kvinnlig_it_expert_farg5", "1F469-1F3FF-200D-1F4BB"},
    {"kontorstjanstekvinna", "1F469-200D-1F4BC"},
    {"kontorstjanstekvinna_farg1", "1F469-1F3FB-200D-1F4BC"},
    {"kontorstjanstekvinna_farg2", "1F469-1F3FC-200D-1F4BC"},
    {"kontorstjanstekvinna_farg3", "1F469-1F3FD-200D-1F4BC"},
    {"kontorstjanstekvinna_farg4", "1F469-1F3FE-200D-1F4BC"},
    {"kontorstjanstekvinna_farg5", "1F469-1F3FF-200D-1F4BC"},
    {"kvinnlig_mekaniker", "1F469-200D-1F527"},
    {"kvinnlig_mekaniker_farg1", "1F469-1F3FB-200D-1F527"},
    {"kvinnlig_mekaniker_farg2", "1F469-1F3FC-200D-1F527"},
    {"kvinnlig_mekaniker_farg3", "1F469-1F3FD-200D-1F527"},
    {"kvinnlig_mekaniker_farg4", "1F469-1F3FE-200D-1F527"},
    {"kvinnlig_mekaniker_farg5", "1F469-1F3FF-200D-1F527"},
    {"kvinnlig_forskare", "1F469-200D-1F52C"},
    {"kvinnlig_forskare_farg1", "1F469-1F3FB-200D-1F52C"},
    {"kvinnlig_forskare_farg2", "1F469-1F3FC-200D-1F52C"},
    {"kvinnlig_forskare_farg3", "1F469-1F3FD-200D-1F52C"},
    {"kvinnlig_forskare_farg4", "1F469-1F3FE-200D-1F52C"},
    {"kvinnlig_forskare_farg5", "1F469-1F3FF-200D-1F52C"},
    {"kvinnlig_astronaut", "1F469-200D-1F680"},
    {"kvinnlig_astronaut_farg1", "1F469-1F3FB-200D-1F680"},
    {"kvinnlig_astronaut_farg2", "1F469-1F3FC-200D-1F680"},
    {"kvinnlig_astronaut_farg3", "1F469-1F3FD-200D-1F680"},
    {"kvinnlig_astronaut_farg4", "1F469-1F3FE-200D-1F680"},
    {"kvinnlig_astronaut_farg5", "1F469-1F3FF-200D-1F680"},
    {"kvinnlig_brandman", "1F469-200D-1F692"},
    {"kvinnlig_brandman_farg1", "1F469-1F3FB-200D-1F692"},
    {"kvinnlig_brandman_farg2", "1F469-1F3FC-200D-1F692"},
    {"kvinnlig_brandman_farg3", "1F469-1F3FD-200D-1F692"},
    {"kvinnlig_brandman_farg4", "1F469-1F3FE-200D-1F692"},
    {"kvinnlig_brandman_farg5", "1F469-1F3FF-200D-1F692"},
    {"kvinna_med_vit_kapp", "1F469-200D-1F9AF"},
    {"kvinna_med_vit_kapp_farg1", "1F469-1F3FB-200D-1F9AF"},
    {"kvinna_med_vit_kapp_farg2", "1F469-1F3FC-200D-1F9AF"},
    {"kvinna_med_vit_kapp_farg3", "1F469-1F3FD-200D-1F9AF"},
    {"kvinna_med_vit_kapp_farg4", "1F469-1F3FE-200D-1F9AF"},
    {"kvinna_med_vit_kapp_farg5", "1F469-1F3FF-200D-1F9AF"},
    {"kvinna_med_vit_kapp_at_hoger", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinna_med_vit_kapp_at_hoger_farg1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinna_med_vit_kapp_at_hoger_farg2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinna_med_vit_kapp_at_hoger_farg3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinna_med_vit_kapp_at_hoger_farg4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinna_med_vit_kapp_at_hoger_farg5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinna_i_permobil", "1F469-200D-1F9BC"},
    {"kvinna_i_permobil_farg1", "1F469-1F3FB-200D-1F9BC"},
    {"kvinna_i_permobil_farg2", "1F469-1F3FC-200D-1F9BC"},
    {"kvinna_i_permobil_farg3", "1F469-1F3FD-200D-1F9BC"},
    {"kvinna_i_permobil_farg4", "1F469-1F3FE-200D-1F9BC"},
    {"kvinna_i_permobil_farg5", "1F469-1F3FF-200D-1F9BC"},
    {"kvinna_i_permobil_at_hoger", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinna_i_permobil_at_hoger_farg1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinna_i_permobil_at_hoger_farg2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinna_i_permobil_at_hoger_farg3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinna_i_permobil_at_hoger_farg4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinna_i_permobil_at_hoger_farg5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinna_i_manuell_rullstol", "1F469-200D-1F9BD"},
    {"kvinna_i_manuell_rullstol_farg1", "1F469-1F3FB-200D-1F9BD"},
    {"kvinna_i_manuell_rullstol_farg2", "1F469-1F3FC-200D-1F9BD"},
    {"kvinna_i_manuell_rullstol_farg3", "1F469-1F3FD-200D-1F9BD"},
    {"kvinna_i_manuell_rullstol_farg4", "1F469-1F3FE-200D-1F9BD"},
    {"kvinna_i_manuell_rullstol_farg5", "1F469-1F3FF-200D-1F9BD"},
    {"kvinna_i_manuell_rullstol_at_hoger", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinna_i_manuell_rullstol_at_hoger_farg1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinna_i_manuell_rullstol_at_hoger_farg2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinna_i_manuell_rullstol_at_hoger_farg3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinna_i_manuell_rullstol_at_hoger_farg4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinna_i_manuell_rullstol_at_hoger_farg5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"fotgangare_at_hoger", "1F6B6-200D-27A1-FE0F"},
    {"fotgangare_at_hoger_farg1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"fotgangare_at_hoger_farg2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"fotgangare_at_hoger_farg3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"fotgangare_at_hoger_farg4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"fotgangare_at_hoger_farg5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"knasittande_person_at_hoger", "1F9CE-200D-27A1-FE0F"},
    {"knasittande_person_at_hoger_farg1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"knasittande_person_at_hoger_farg2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"knasittande_person_at_hoger_farg3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"knasittande_person_at_hoger_farg4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"knasittande_person_at_hoger_farg5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"lakare", "1F9D1-200D-2695-FE0F"},
    {"lakare_farg1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"lakare_farg2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"lakare_farg3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"lakare_farg4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"lakare_farg5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"domare", "1F9D1-200D-2696-FE0F"},
    {"domare_farg1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"domare_farg2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"domare_farg3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"domare_farg4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"domare_farg5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"pilot", "1F9D1-200D-2708-FE0F"},
    {"pilot_farg1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"pilot_farg2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"pilot_farg3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"pilot_farg4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"pilot_farg5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"bonde", "1F9D1-200D-1F33E"},
    {"bonde_farg1", "1F9D1-1F3FB-200D-1F33E"},
    {"bonde_farg2", "1F9D1-1F3FC-200D-1F33E"},
    {"bonde_farg3", "1F9D1-1F3FD-200D-1F33E"},
    {"bonde_farg4", "1F9D1-1F3FE-200D-1F33E"},
    {"bonde_farg5", "1F9D1-1F3FF-200D-1F33E"},
    {"kock", "1F9D1-200D-1F373"},
    {"kock_farg1", "1F9D1-1F3FB-200D-1F373"},
    {"kock_farg2", "1F9D1-1F3FC-200D-1F373"},
    {"kock_farg3", "1F9D1-1F3FD-200D-1F373"},
    {"kock_farg4", "1F9D1-1F3FE-200D-1F373"},
    {"kock_farg5", "1F9D1-1F3FF-200D-1F373"},
    {"person_ammar_spadbarn", "1F9D1-200D-1F37C"},
    {"person_ammar_spadbarn_farg1", "1F9D1-1F3FB-200D-1F37C"},
    {"person_ammar_spadbarn_farg2", "1F9D1-1F3FC-200D-1F37C"},
    {"person_ammar_spadbarn_farg3", "1F9D1-1F3FD-200D-1F37C"},
    {"person_ammar_spadbarn_farg4", "1F9D1-1F3FE-200D-1F37C"},
    {"person_ammar_spadbarn_farg5", "1F9D1-1F3FF-200D-1F37C"},
    {"konsneutral_tomte", "1F9D1-200D-1F384"},
    {"konsneutral_tomte_farg1", "1F9D1-1F3FB-200D-1F384"},
    {"konsneutral_tomte_farg2", "1F9D1-1F3FC-200D-1F384"},
    {"konsneutral_tomte_farg3", "1F9D1-1F3FD-200D-1F384"},
    {"konsneutral_tomte_farg4", "1F9D1-1F3FE-200D-1F384"},
    {"konsneutral_tomte_farg5", "1F9D1-1F3FF-200D-1F384"},
    {"student", "1F9D1-200D-1F393"},
    {"student_farg1", "1F9D1-1F3FB-200D-1F393"},
    {"student_farg2", "1F9D1-1F3FC-200D-1F393"},
    {"student_farg3", "1F9D1-1F3FD-200D-1F393"},
    {"student_farg4", "1F9D1-1F3FE-200D-1F393"},
    {"student_farg5", "1F9D1-1F3FF-200D-1F393"},
    {"sangare", "1F9D1-200D-1F3A4"},
    {"sangare_farg1", "1F9D1-1F3FB-200D-1F3A4"},
    {"sangare_farg2", "1F9D1-1F3FC-200D-1F3A4"},
    {"sangare_farg3", "1F9D1-1F3FD-200D-1F3A4"},
    {"sangare_farg4", "1F9D1-1F3FE-200D-1F3A4"},
    {"sangare_farg5", "1F9D1-1F3FF-200D-1F3A4"},
    {"konstnar", "1F9D1-200D-1F3A8"},
    {"konstnar_farg1", "1F9D1-1F3FB-200D-1F3A8"},
    {"konstnar_farg2", "1F9D1-1F3FC-200D-1F3A8"},
    {"konstnar_farg3", "1F9D1-1F3FD-200D-1F3A8"},
    {"konstnar_farg4", "1F9D1-1F3FE-200D-1F3A8"},
    {"konstnar_farg5", "1F9D1-1F3FF-200D-1F3A8"},
    {"larare", "1F9D1-200D-1F3EB"},
    {"larare_farg1", "1F9D1-1F3FB-200D-1F3EB"},
    {"larare_farg2", "1F9D1-1F3FC-200D-1F3EB"},
    {"larare_farg3", "1F9D1-1F3FD-200D-1F3EB"},
    {"larare_farg4", "1F9D1-1F3FE-200D-1F3EB"},
    {"larare_farg5", "1F9D1-1F3FF-200D-1F3EB"},
    {"fabriksarbetare", "1F9D1-200D-1F3ED"},
    {"fabriksarbetare_farg1", "1F9D1-1F3FB-200D-1F3ED"},
    {"fabriksarbetare_farg2", "1F9D1-1F3FC-200D-1F3ED"},
    {"fabriksarbetare_farg3", "1F9D1-1F3FD-200D-1F3ED"},
    {"fabriksarbetare_farg4", "1F9D1-1F3FE-200D-1F3ED"},
    {"fabriksarbetare_farg5", "1F9D1-1F3FF-200D-1F3ED"},
    {"it_tekniker", "1F9D1-200D-1F4BB"},
    {"it_tekniker_farg1", "1F9D1-1F3FB-200D-1F4BB"},
    {"it_tekniker_farg2", "1F9D1-1F3FC-200D-1F4BB"},
    {"it_tekniker_farg3", "1F9D1-1F3FD-200D-1F4BB"},
    {"it_tekniker_farg4", "1F9D1-1F3FE-200D-1F4BB"},
    {"it_tekniker_farg5", "1F9D1-1F3FF-200D-1F4BB"},
    {"kontorist", "1F9D1-200D-1F4BC"},
    {"kontorist_farg1", "1F9D1-1F3FB-200D-1F4BC"},
    {"kontorist_farg2", "1F9D1-1F3FC-200D-1F4BC"},
    {"kontorist_farg3", "1F9D1-1F3FD-200D-1F4BC"},
    {"kontorist_farg4", "1F9D1-1F3FE-200D-1F4BC"},
    {"kontorist_farg5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mekaniker", "1F9D1-200D-1F527"},
    {"mekaniker_farg1", "1F9D1-1F3FB-200D-1F527"},
    {"mekaniker_farg2", "1F9D1-1F3FC-200D-1F527"},
    {"mekaniker_farg3", "1F9D1-1F3FD-200D-1F527"},
    {"mekaniker_farg4", "1F9D1-1F3FE-200D-1F527"},
    {"mekaniker_farg5", "1F9D1-1F3FF-200D-1F527"},
    {"forskare", "1F9D1-200D-1F52C"},
    {"forskare_farg1", "1F9D1-1F3FB-200D-1F52C"},
    {"forskare_farg2", "1F9D1-1F3FC-200D-1F52C"},
    {"forskare_farg3", "1F9D1-1F3FD-200D-1F52C"},
    {"forskare_farg4", "1F9D1-1F3FE-200D-1F52C"},
    {"forskare_farg5", "1F9D1-1F3FF-200D-1F52C"},
    {"astronaut", "1F9D1-200D-1F680"},
    {"astronaut_farg1", "1F9D1-1F3FB-200D-1F680"},
    {"astronaut_farg2", "1F9D1-1F3FC-200D-1F680"},
    {"astronaut_farg3", "1F9D1-1F3FD-200D-1F680"},
    {"astronaut_farg4", "1F9D1-1F3FE-200D-1F680"},
    {"astronaut_farg5", "1F9D1-1F3FF-200D-1F680"},
    {"brandman", "1F9D1-200D-1F692"},
    {"brandman_farg1", "1F9D1-1F3FB-200D-1F692"},
    {"brandman_farg2", "1F9D1-1F3FC-200D-1F692"},
    {"brandman_farg3", "1F9D1-1F3FD-200D-1F692"},
    {"brandman_farg4", "1F9D1-1F3FE-200D-1F692"},
    {"brandman_farg5", "1F9D1-1F3FF-200D-1F692"},
    {"person_med_vit_kapp", "1F9D1-200D-1F9AF"},
    {"person_med_vit_kapp_farg1", "1F9D1-1F3FB-200D-1F9AF"},
    {"person_med_vit_kapp_farg2", "1F9D1-1F3FC-200D-1F9AF"},
    {"person_med_vit_kapp_farg3", "1F9D1-1F3FD-200D-1F9AF"},
    {"person_med_vit_kapp_farg4", "1F9D1-1F3FE-200D-1F9AF"},
    {"person_med_vit_kapp_farg5", "1F9D1-1F3FF-200D-1F9AF"},
    {"person_med_vit_kapp_at_hoger", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_vit_kapp_at_hoger_farg1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_vit_kapp_at_hoger_farg2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_vit_kapp_at_hoger_farg3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_vit_kapp_at_hoger_farg4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_vit_kapp_at_hoger_farg5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"person_i_permobil", "1F9D1-200D-1F9BC"},
    {"person_i_permobil_farg1", "1F9D1-1F3FB-200D-1F9BC"},
    {"person_i_permobil_farg2", "1F9D1-1F3FC-200D-1F9BC"},
    {"person_i_permobil_farg3", "1F9D1-1F3FD-200D-1F9BC"},
    {"person_i_permobil_farg4", "1F9D1-1F3FE-200D-1F9BC"},
    {"person_i_permobil_farg5", "1F9D1-1F3FF-200D-1F9BC"},
    {"person_i_permobil_at_hoger", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_permobil_at_hoger_farg1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_permobil_at_hoger_farg2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_permobil_at_hoger_farg3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_permobil_at_hoger_farg4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_permobil_at_hoger_farg5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_manuell_rullstol", "1F9D1-200D-1F9BD"},
    {"person_i_manuell_rullstol_farg1", "1F9D1-1F3FB-200D-1F9BD"},
    {"person_i_manuell_rullstol_farg2", "1F9D1-1F3FC-200D-1F9BD"},
    {"person_i_manuell_rullstol_farg3", "1F9D1-1F3FD-200D-1F9BD"},
    {"person_i_manuell_rullstol_farg4", "1F9D1-1F3FE-200D-1F9BD"},
    {"person_i_manuell_rullstol_farg5", "1F9D1-1F3FF-200D-1F9BD"},
    {"person_i_manuell_rullstol_at_hoger", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullstol_at_hoger_farg1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullstol_at_hoger_farg2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullstol_at_hoger_farg3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullstol_at_hoger_farg4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullstol_at_hoger_farg5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinna_med_boll", "26F9-FE0F-200D-2640-FE0F"},
    {"kvinna_med_boll_farg1", "26F9-1F3FB-200D-2640-FE0F"},
    {"kvinna_med_boll_farg2", "26F9-1F3FC-200D-2640-FE0F"},
    {"kvinna_med_boll_farg3", "26F9-1F3FD-200D-2640-FE0F"},
    {"kvinna_med_boll_farg4", "26F9-1F3FE-200D-2640-FE0F"},
    {"kvinna_med_boll_farg5", "26F9-1F3FF-200D-2640-FE0F"},
    {"man_med_boll", "26F9-FE0F-200D-2642-FE0F"},
    {"man_med_boll_farg1", "26F9-1F3FB-200D-2642-FE0F"},
    {"man_med_boll_farg2", "26F9-1F3FC-200D-2642-FE0F"},
    {"man_med_boll_farg3", "26F9-1F3FD-200D-2642-FE0F"},
    {"man_med_boll_farg4", "26F9-1F3FE-200D-2642-FE0F"},
    {"man_med_boll_farg5", "26F9-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_springer", "1F3C3-200D-2640-FE0F"},
    {"kvinna_som_springer_farg1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_springer_farg2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_springer_farg3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_springer_farg4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_springer_farg5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"kvinna_som_springer_at_hoger", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_springer_at_hoger_farg1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_springer_at_hoger_farg2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_springer_at_hoger_farg3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_springer_at_hoger_farg4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_springer_at_hoger_farg5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"man_som_springer", "1F3C3-200D-2642-FE0F"},
    {"man_som_springer_farg1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"man_som_springer_farg2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"man_som_springer_farg3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"man_som_springer_farg4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"man_som_springer_farg5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"man_som_springer_at_hoger", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_springer_at_hoger_farg1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_springer_at_hoger_farg2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_springer_at_hoger_farg3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_springer_at_hoger_farg4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_springer_at_hoger_farg5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_surfar", "1F3C4-200D-2640-FE0F"},
    {"kvinna_som_surfar_farg1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_surfar_farg2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_surfar_farg3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_surfar_farg4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_surfar_farg5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"man_som_surfar", "1F3C4-200D-2642-FE0F"},
    {"man_som_surfar_farg1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"man_som_surfar_farg2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"man_som_surfar_farg3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"man_som_surfar_farg4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"man_som_surfar_farg5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_simmar", "1F3CA-200D-2640-FE0F"},
    {"kvinna_som_simmar_farg1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_simmar_farg2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_simmar_farg3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_simmar_farg4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_simmar_farg5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"man_som_simmar", "1F3CA-200D-2642-FE0F"},
    {"man_som_simmar_farg1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"man_som_simmar_farg2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"man_som_simmar_farg3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"man_som_simmar_farg4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"man_som_simmar_farg5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_lyfter_tyngder", "1F3CB-FE0F-200D-2640-FE0F"},
    {"kvinna_som_lyfter_tyngder_farg1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_lyfter_tyngder_farg2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_lyfter_tyngder_farg3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_lyfter_tyngder_farg4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_lyfter_tyngder_farg5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"man_som_lyfter_tyngder", "1F3CB-FE0F-200D-2642-FE0F"},
    {"man_som_lyfter_tyngder_farg1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"man_som_lyfter_tyngder_farg2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"man_som_lyfter_tyngder_farg3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"man_som_lyfter_tyngder_farg4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"man_som_lyfter_tyngder_farg5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_spelar_golf", "1F3CC-FE0F-200D-2640-FE0F"},
    {"kvinna_som_spelar_golf_farg1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_spelar_golf_farg2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_spelar_golf_farg3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_spelar_golf_farg4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_spelar_golf_farg5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"man_som_spelar_golf", "1F3CC-FE0F-200D-2642-FE0F"},
    {"man_som_spelar_golf_farg1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"man_som_spelar_golf_farg2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"man_som_spelar_golf_farg3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"man_som_spelar_golf_farg4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"man_som_spelar_golf_farg5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_polis", "1F46E-200D-2640-FE0F"},
    {"kvinnlig_polis_farg1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_polis_farg2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_polis_farg3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_polis_farg4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_polis_farg5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"manlig_polis", "1F46E-200D-2642-FE0F"},
    {"manlig_polis_farg1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"manlig_polis_farg2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"manlig_polis_farg3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"manlig_polis_farg4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"manlig_polis_farg5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"kvinnor_med_kaninoron", "1F46F-200D-2640-FE0F"},
    {"kvinnor_med_kaninoron_farg1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"kvinnor_med_kaninoron_farg1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"kvinnor_med_kaninoron_farg1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"kvinnor_med_kaninoron_farg1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"kvinnor_med_kaninoron_farg2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"kvinnor_med_kaninoron_farg2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"kvinnor_med_kaninoron_farg2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"kvinnor_med_kaninoron_farg2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"kvinnor_med_kaninoron_farg3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"kvinnor_med_kaninoron_farg3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"kvinnor_med_kaninoron_farg3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"kvinnor_med_kaninoron_farg3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"kvinnor_med_kaninoron_farg4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"kvinnor_med_kaninoron_farg4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"kvinnor_med_kaninoron_farg4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"kvinnor_med_kaninoron_farg4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"kvinnor_med_kaninoron_farg5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"kvinnor_med_kaninoron_farg5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"kvinnor_med_kaninoron_farg5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"kvinnor_med_kaninoron_farg5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"kvinnor_med_kaninoron_farg1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"kvinnor_med_kaninoron_farg2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"kvinnor_med_kaninoron_farg3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"kvinnor_med_kaninoron_farg4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"kvinnor_med_kaninoron_farg5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"man_med_kaninoron", "1F46F-200D-2642-FE0F"},
    {"man_med_kaninoron_farg1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"man_med_kaninoron_farg1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"man_med_kaninoron_farg1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"man_med_kaninoron_farg1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"man_med_kaninoron_farg2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"man_med_kaninoron_farg2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"man_med_kaninoron_farg2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"man_med_kaninoron_farg2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"man_med_kaninoron_farg3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"man_med_kaninoron_farg3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"man_med_kaninoron_farg3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"man_med_kaninoron_farg3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"man_med_kaninoron_farg4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"man_med_kaninoron_farg4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"man_med_kaninoron_farg4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"man_med_kaninoron_farg4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"man_med_kaninoron_farg5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"man_med_kaninoron_farg5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"man_med_kaninoron_farg5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"man_med_kaninoron_farg5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"man_med_kaninoron_farg1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"man_med_kaninoron_farg2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"man_med_kaninoron_farg3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"man_med_kaninoron_farg4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"man_med_kaninoron_farg5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"kvinna_i_brudsloja", "1F470-200D-2640-FE0F"},
    {"kvinna_i_brudsloja_farg1", "1F470-1F3FB-200D-2640-FE0F"},
    {"kvinna_i_brudsloja_farg2", "1F470-1F3FC-200D-2640-FE0F"},
    {"kvinna_i_brudsloja_farg3", "1F470-1F3FD-200D-2640-FE0F"},
    {"kvinna_i_brudsloja_farg4", "1F470-1F3FE-200D-2640-FE0F"},
    {"kvinna_i_brudsloja_farg5", "1F470-1F3FF-200D-2640-FE0F"},
    {"man_i_brudsloja", "1F470-200D-2642-FE0F"},
    {"man_i_brudsloja_farg1", "1F470-1F3FB-200D-2642-FE0F"},
    {"man_i_brudsloja_farg2", "1F470-1F3FC-200D-2642-FE0F"},
    {"man_i_brudsloja_farg3", "1F470-1F3FD-200D-2642-FE0F"},
    {"man_i_brudsloja_farg4", "1F470-1F3FE-200D-2642-FE0F"},
    {"man_i_brudsloja_farg5", "1F470-1F3FF-200D-2642-FE0F"},
    {"blond_kvinna", "1F471-200D-2640-FE0F"},
    {"blond_kvinna_farg1", "1F471-1F3FB-200D-2640-FE0F"},
    {"blond_kvinna_farg2", "1F471-1F3FC-200D-2640-FE0F"},
    {"blond_kvinna_farg3", "1F471-1F3FD-200D-2640-FE0F"},
    {"blond_kvinna_farg4", "1F471-1F3FE-200D-2640-FE0F"},
    {"blond_kvinna_farg5", "1F471-1F3FF-200D-2640-FE0F"},
    {"blond_man", "1F471-200D-2642-FE0F"},
    {"blond_man_farg1", "1F471-1F3FB-200D-2642-FE0F"},
    {"blond_man_farg2", "1F471-1F3FC-200D-2642-FE0F"},
    {"blond_man_farg3", "1F471-1F3FD-200D-2642-FE0F"},
    {"blond_man_farg4", "1F471-1F3FE-200D-2642-FE0F"},
    {"blond_man_farg5", "1F471-1F3FF-200D-2642-FE0F"},
    {"kvinna_i_turban", "1F473-200D-2640-FE0F"},
    {"kvinna_i_turban_farg1", "1F473-1F3FB-200D-2640-FE0F"},
    {"kvinna_i_turban_farg2", "1F473-1F3FC-200D-2640-FE0F"},
    {"kvinna_i_turban_farg3", "1F473-1F3FD-200D-2640-FE0F"},
    {"kvinna_i_turban_farg4", "1F473-1F3FE-200D-2640-FE0F"},
    {"kvinna_i_turban_farg5", "1F473-1F3FF-200D-2640-FE0F"},
    {"man_i_turban", "1F473-200D-2642-FE0F"},
    {"man_i_turban_farg1", "1F473-1F3FB-200D-2642-FE0F"},
    {"man_i_turban_farg2", "1F473-1F3FC-200D-2642-FE0F"},
    {"man_i_turban_farg3", "1F473-1F3FD-200D-2642-FE0F"},
    {"man_i_turban_farg4", "1F473-1F3FE-200D-2642-FE0F"},
    {"man_i_turban_farg5", "1F473-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_byggarbetare", "1F477-200D-2640-FE0F"},
    {"kvinnlig_byggarbetare_farg1", "1F477-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_byggarbetare_farg2", "1F477-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_byggarbetare_farg3", "1F477-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_byggarbetare_farg4", "1F477-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_byggarbetare_farg5", "1F477-1F3FF-200D-2640-FE0F"},
    {"manlig_byggarbetare", "1F477-200D-2642-FE0F"},
    {"manlig_byggarbetare_farg1", "1F477-1F3FB-200D-2642-FE0F"},
    {"manlig_byggarbetare_farg2", "1F477-1F3FC-200D-2642-FE0F"},
    {"manlig_byggarbetare_farg3", "1F477-1F3FD-200D-2642-FE0F"},
    {"manlig_byggarbetare_farg4", "1F477-1F3FE-200D-2642-FE0F"},
    {"manlig_byggarbetare_farg5", "1F477-1F3FF-200D-2642-FE0F"},
    {"hjalpande_kvinna", "1F481-200D-2640-FE0F"},
    {"hjalpande_kvinna_farg1", "1F481-1F3FB-200D-2640-FE0F"},
    {"hjalpande_kvinna_farg2", "1F481-1F3FC-200D-2640-FE0F"},
    {"hjalpande_kvinna_farg3", "1F481-1F3FD-200D-2640-FE0F"},
    {"hjalpande_kvinna_farg4", "1F481-1F3FE-200D-2640-FE0F"},
    {"hjalpande_kvinna_farg5", "1F481-1F3FF-200D-2640-FE0F"},
    {"hjalpande_man", "1F481-200D-2642-FE0F"},
    {"hjalpande_man_farg1", "1F481-1F3FB-200D-2642-FE0F"},
    {"hjalpande_man_farg2", "1F481-1F3FC-200D-2642-FE0F"},
    {"hjalpande_man_farg3", "1F481-1F3FD-200D-2642-FE0F"},
    {"hjalpande_man_farg4", "1F481-1F3FE-200D-2642-FE0F"},
    {"hjalpande_man_farg5", "1F481-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_grenadjar", "1F482-200D-2640-FE0F"},
    {"kvinnlig_grenadjar_farg1", "1F482-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_grenadjar_farg2", "1F482-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_grenadjar_farg3", "1F482-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_grenadjar_farg4", "1F482-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_grenadjar_farg5", "1F482-1F3FF-200D-2640-FE0F"},
    {"manlig_grenadjar", "1F482-200D-2642-FE0F"},
    {"manlig_grenadjar_farg1", "1F482-1F3FB-200D-2642-FE0F"},
    {"manlig_grenadjar_farg2", "1F482-1F3FC-200D-2642-FE0F"},
    {"manlig_grenadjar_farg3", "1F482-1F3FD-200D-2642-FE0F"},
    {"manlig_grenadjar_farg4", "1F482-1F3FE-200D-2642-FE0F"},
    {"manlig_grenadjar_farg5", "1F482-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_blir_masserad", "1F486-200D-2640-FE0F"},
    {"kvinna_som_blir_masserad_farg1", "1F486-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_blir_masserad_farg2", "1F486-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_blir_masserad_farg3", "1F486-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_blir_masserad_farg4", "1F486-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_blir_masserad_farg5", "1F486-1F3FF-200D-2640-FE0F"},
    {"man_som_blir_masserad", "1F486-200D-2642-FE0F"},
    {"man_som_blir_masserad_farg1", "1F486-1F3FB-200D-2642-FE0F"},
    {"man_som_blir_masserad_farg2", "1F486-1F3FC-200D-2642-FE0F"},
    {"man_som_blir_masserad_farg3", "1F486-1F3FD-200D-2642-FE0F"},
    {"man_som_blir_masserad_farg4", "1F486-1F3FE-200D-2642-FE0F"},
    {"man_som_blir_masserad_farg5", "1F486-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_far_haret_klippt", "1F487-200D-2640-FE0F"},
    {"kvinna_som_far_haret_klippt_farg1", "1F487-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_far_haret_klippt_farg2", "1F487-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_far_haret_klippt_farg3", "1F487-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_far_haret_klippt_farg4", "1F487-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_far_haret_klippt_farg5", "1F487-1F3FF-200D-2640-FE0F"},
    {"man_som_far_haret_klippt", "1F487-200D-2642-FE0F"},
    {"man_som_far_haret_klippt_farg1", "1F487-1F3FB-200D-2642-FE0F"},
    {"man_som_far_haret_klippt_farg2", "1F487-1F3FC-200D-2642-FE0F"},
    {"man_som_far_haret_klippt_farg3", "1F487-1F3FD-200D-2642-FE0F"},
    {"man_som_far_haret_klippt_farg4", "1F487-1F3FE-200D-2642-FE0F"},
    {"man_som_far_haret_klippt_farg5", "1F487-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_detektiv", "1F575-FE0F-200D-2640-FE0F"},
    {"kvinnlig_detektiv_farg1", "1F575-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_detektiv_farg2", "1F575-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_detektiv_farg3", "1F575-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_detektiv_farg4", "1F575-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_detektiv_farg5", "1F575-1F3FF-200D-2640-FE0F"},
    {"manlig_detektiv", "1F575-FE0F-200D-2642-FE0F"},
    {"manlig_detektiv_farg1", "1F575-1F3FB-200D-2642-FE0F"},
    {"manlig_detektiv_farg2", "1F575-1F3FC-200D-2642-FE0F"},
    {"manlig_detektiv_farg3", "1F575-1F3FD-200D-2642-FE0F"},
    {"manlig_detektiv_farg4", "1F575-1F3FE-200D-2642-FE0F"},
    {"manlig_detektiv_farg5", "1F575-1F3FF-200D-2642-FE0F"},
    {"kvinna_gestikulerar_nej", "1F645-200D-2640-FE0F"},
    {"kvinna_gestikulerar_nej_farg1", "1F645-1F3FB-200D-2640-FE0F"},
    {"kvinna_gestikulerar_nej_farg2", "1F645-1F3FC-200D-2640-FE0F"},
    {"kvinna_gestikulerar_nej_farg3", "1F645-1F3FD-200D-2640-FE0F"},
    {"kvinna_gestikulerar_nej_farg4", "1F645-1F3FE-200D-2640-FE0F"},
    {"kvinna_gestikulerar_nej_farg5", "1F645-1F3FF-200D-2640-FE0F"},
    {"man_gestikulerar_nej", "1F645-200D-2642-FE0F"},
    {"man_gestikulerar_nej_farg1", "1F645-1F3FB-200D-2642-FE0F"},
    {"man_gestikulerar_nej_farg2", "1F645-1F3FC-200D-2642-FE0F"},
    {"man_gestikulerar_nej_farg3", "1F645-1F3FD-200D-2642-FE0F"},
    {"man_gestikulerar_nej_farg4", "1F645-1F3FE-200D-2642-FE0F"},
    {"man_gestikulerar_nej_farg5", "1F645-1F3FF-200D-2642-FE0F"},
    {"kvinna_gestikulerar_ok", "1F646-200D-2640-FE0F"},
    {"kvinna_gestikulerar_ok_farg1", "1F646-1F3FB-200D-2640-FE0F"},
    {"kvinna_gestikulerar_ok_farg2", "1F646-1F3FC-200D-2640-FE0F"},
    {"kvinna_gestikulerar_ok_farg3", "1F646-1F3FD-200D-2640-FE0F"},
    {"kvinna_gestikulerar_ok_farg4", "1F646-1F3FE-200D-2640-FE0F"},
    {"kvinna_gestikulerar_ok_farg5", "1F646-1F3FF-200D-2640-FE0F"},
    {"man_gestikulerar_ok", "1F646-200D-2642-FE0F"},
    {"man_gestikulerar_ok_farg1", "1F646-1F3FB-200D-2642-FE0F"},
    {"man_gestikulerar_ok_farg2", "1F646-1F3FC-200D-2642-FE0F"},
    {"man_gestikulerar_ok_farg3", "1F646-1F3FD-200D-2642-FE0F"},
    {"man_gestikulerar_ok_farg4", "1F646-1F3FE-200D-2642-FE0F"},
    {"man_gestikulerar_ok_farg5", "1F646-1F3FF-200D-2642-FE0F"},
    {"bugande_kvinna", "1F647-200D-2640-FE0F"},
    {"bugande_kvinna_farg1", "1F647-1F3FB-200D-2640-FE0F"},
    {"bugande_kvinna_farg2", "1F647-1F3FC-200D-2640-FE0F"},
    {"bugande_kvinna_farg3", "1F647-1F3FD-200D-2640-FE0F"},
    {"bugande_kvinna_farg4", "1F647-1F3FE-200D-2640-FE0F"},
    {"bugande_kvinna_farg5", "1F647-1F3FF-200D-2640-FE0F"},
    {"bugande_man", "1F647-200D-2642-FE0F"},
    {"bugande_man_farg1", "1F647-1F3FB-200D-2642-FE0F"},
    {"bugande_man_farg2", "1F647-1F3FC-200D-2642-FE0F"},
    {"bugande_man_farg3", "1F647-1F3FD-200D-2642-FE0F"},
    {"bugande_man_farg4", "1F647-1F3FE-200D-2642-FE0F"},
    {"bugande_man_farg5", "1F647-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_racker_upp_handen", "1F64B-200D-2640-FE0F"},
    {"kvinna_som_racker_upp_handen_farg1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_racker_upp_handen_farg2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_racker_upp_handen_farg3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_racker_upp_handen_farg4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_racker_upp_handen_farg5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"man_som_racker_upp_handen", "1F64B-200D-2642-FE0F"},
    {"man_som_racker_upp_handen_farg1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"man_som_racker_upp_handen_farg2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"man_som_racker_upp_handen_farg3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"man_som_racker_upp_handen_farg4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"man_som_racker_upp_handen_farg5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"sur_kvinna", "1F64D-200D-2640-FE0F"},
    {"sur_kvinna_farg1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"sur_kvinna_farg2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"sur_kvinna_farg3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"sur_kvinna_farg4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"sur_kvinna_farg5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"sur_man", "1F64D-200D-2642-FE0F"},
    {"sur_man_farg1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"sur_man_farg2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"sur_man_farg3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"sur_man_farg4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"sur_man_farg5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"plutande_kvinna", "1F64E-200D-2640-FE0F"},
    {"plutande_kvinna_farg1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"plutande_kvinna_farg2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"plutande_kvinna_farg3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"plutande_kvinna_farg4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"plutande_kvinna_farg5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"plutande_man", "1F64E-200D-2642-FE0F"},
    {"plutande_man_farg1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"plutande_man_farg2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"plutande_man_farg3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"plutande_man_farg4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"plutande_man_farg5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_ror_en_bat", "1F6A3-200D-2640-FE0F"},
    {"kvinna_som_ror_en_bat_farg1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_ror_en_bat_farg2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_ror_en_bat_farg3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_ror_en_bat_farg4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_ror_en_bat_farg5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"man_som_ror_en_bat", "1F6A3-200D-2642-FE0F"},
    {"man_som_ror_en_bat_farg1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"man_som_ror_en_bat_farg2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"man_som_ror_en_bat_farg3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"man_som_ror_en_bat_farg4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"man_som_ror_en_bat_farg5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_cyklar", "1F6B4-200D-2640-FE0F"},
    {"kvinna_som_cyklar_farg1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_cyklar_farg2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_cyklar_farg3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_cyklar_farg4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_cyklar_farg5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"man_som_cyklar", "1F6B4-200D-2642-FE0F"},
    {"man_som_cyklar_farg1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"man_som_cyklar_farg2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"man_som_cyklar_farg3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"man_som_cyklar_farg4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"man_som_cyklar_farg5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_cyklar_pa_en_mountainbike", "1F6B5-200D-2640-FE0F"},
    {"kvinna_som_cyklar_pa_en_mountainbike_farg1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_cyklar_pa_en_mountainbike_farg2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_cyklar_pa_en_mountainbike_farg3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_cyklar_pa_en_mountainbike_farg4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_cyklar_pa_en_mountainbike_farg5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"man_som_cyklar_pa_en_mountainbike", "1F6B5-200D-2642-FE0F"},
    {"man_som_cyklar_pa_en_mountainbike_farg1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"man_som_cyklar_pa_en_mountainbike_farg2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"man_som_cyklar_pa_en_mountainbike_farg3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"man_som_cyklar_pa_en_mountainbike_farg4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"man_som_cyklar_pa_en_mountainbike_farg5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_gar", "1F6B6-200D-2640-FE0F"},
    {"kvinna_som_gar_farg1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_gar_farg2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_gar_farg3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_gar_farg4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_gar_farg5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"kvinna_som_gar_at_hoger", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_gar_at_hoger_farg1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_gar_at_hoger_farg2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_gar_at_hoger_farg3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_gar_at_hoger_farg4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_gar_at_hoger_farg5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"man_som_gar", "1F6B6-200D-2642-FE0F"},
    {"man_som_gar_farg1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"man_som_gar_farg2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"man_som_gar_farg3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"man_som_gar_farg4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"man_som_gar_farg5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"man_som_gar_at_hoger", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_gar_at_hoger_farg1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_gar_at_hoger_farg2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_gar_at_hoger_farg3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_gar_at_hoger_farg4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_gar_at_hoger_farg5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_slar_sig_for_pannan", "1F926-200D-2640-FE0F"},
    {"kvinna_som_slar_sig_for_pannan_farg1", "1F926-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_slar_sig_for_pannan_farg2", "1F926-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_slar_sig_for_pannan_farg3", "1F926-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_slar_sig_for_pannan_farg4", "1F926-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_slar_sig_for_pannan_farg5", "1F926-1F3FF-200D-2640-FE0F"},
    {"man_som_slar_sig_for_pannan", "1F926-200D-2642-FE0F"},
    {"man_som_slar_sig_for_pannan_farg1", "1F926-1F3FB-200D-2642-FE0F"},
    {"man_som_slar_sig_for_pannan_farg2", "1F926-1F3FC-200D-2642-FE0F"},
    {"man_som_slar_sig_for_pannan_farg3", "1F926-1F3FD-200D-2642-FE0F"},
    {"man_som_slar_sig_for_pannan_farg4", "1F926-1F3FE-200D-2642-FE0F"},
    {"man_som_slar_sig_for_pannan_farg5", "1F926-1F3FF-200D-2642-FE0F"},
    {"kvinna_i_smoking", "1F935-200D-2640-FE0F"},
    {"kvinna_i_smoking_farg1", "1F935-1F3FB-200D-2640-FE0F"},
    {"kvinna_i_smoking_farg2", "1F935-1F3FC-200D-2640-FE0F"},
    {"kvinna_i_smoking_farg3", "1F935-1F3FD-200D-2640-FE0F"},
    {"kvinna_i_smoking_farg4", "1F935-1F3FE-200D-2640-FE0F"},
    {"kvinna_i_smoking_farg5", "1F935-1F3FF-200D-2640-FE0F"},
    {"man_i_smoking", "1F935-200D-2642-FE0F"},
    {"man_i_smoking_farg1", "1F935-1F3FB-200D-2642-FE0F"},
    {"man_i_smoking_farg2", "1F935-1F3FC-200D-2642-FE0F"},
    {"man_i_smoking_farg3", "1F935-1F3FD-200D-2642-FE0F"},
    {"man_i_smoking_farg4", "1F935-1F3FE-200D-2642-FE0F"},
    {"man_i_smoking_farg5", "1F935-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_rycker_pa_axlarna", "1F937-200D-2640-FE0F"},
    {"kvinna_som_rycker_pa_axlarna_farg1", "1F937-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_rycker_pa_axlarna_farg2", "1F937-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_rycker_pa_axlarna_farg3", "1F937-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_rycker_pa_axlarna_farg4", "1F937-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_rycker_pa_axlarna_farg5", "1F937-1F3FF-200D-2640-FE0F"},
    {"man_som_rycker_pa_axlarna", "1F937-200D-2642-FE0F"},
    {"man_som_rycker_pa_axlarna_farg1", "1F937-1F3FB-200D-2642-FE0F"},
    {"man_som_rycker_pa_axlarna_farg2", "1F937-1F3FC-200D-2642-FE0F"},
    {"man_som_rycker_pa_axlarna_farg3", "1F937-1F3FD-200D-2642-FE0F"},
    {"man_som_rycker_pa_axlarna_farg4", "1F937-1F3FE-200D-2642-FE0F"},
    {"man_som_rycker_pa_axlarna_farg5", "1F937-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_hjular", "1F938-200D-2640-FE0F"},
    {"kvinna_som_hjular_farg1", "1F938-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_hjular_farg2", "1F938-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_hjular_farg3", "1F938-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_hjular_farg4", "1F938-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_hjular_farg5", "1F938-1F3FF-200D-2640-FE0F"},
    {"man_som_hjular", "1F938-200D-2642-FE0F"},
    {"man_som_hjular_farg1", "1F938-1F3FB-200D-2642-FE0F"},
    {"man_som_hjular_farg2", "1F938-1F3FC-200D-2642-FE0F"},
    {"man_som_hjular_farg3", "1F938-1F3FD-200D-2642-FE0F"},
    {"man_som_hjular_farg4", "1F938-1F3FE-200D-2642-FE0F"},
    {"man_som_hjular_farg5", "1F938-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_jonglerar", "1F939-200D-2640-FE0F"},
    {"kvinna_som_jonglerar_farg1", "1F939-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_jonglerar_farg2", "1F939-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_jonglerar_farg3", "1F939-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_jonglerar_farg4", "1F939-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_jonglerar_farg5", "1F939-1F3FF-200D-2640-FE0F"},
    {"man_som_jonglerar", "1F939-200D-2642-FE0F"},
    {"man_som_jonglerar_farg1", "1F939-1F3FB-200D-2642-FE0F"},
    {"man_som_jonglerar_farg2", "1F939-1F3FC-200D-2642-FE0F"},
    {"man_som_jonglerar_farg3", "1F939-1F3FD-200D-2642-FE0F"},
    {"man_som_jonglerar_farg4", "1F939-1F3FE-200D-2642-FE0F"},
    {"man_som_jonglerar_farg5", "1F939-1F3FF-200D-2642-FE0F"},
    {"brottande_kvinnor", "1F93C-200D-2640-FE0F"},
    {"brottande_kvinnor_farg1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"brottande_kvinnor_farg1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"brottande_kvinnor_farg1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"brottande_kvinnor_farg1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"brottande_kvinnor_farg2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"brottande_kvinnor_farg2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"brottande_kvinnor_farg2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"brottande_kvinnor_farg2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"brottande_kvinnor_farg3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"brottande_kvinnor_farg3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"brottande_kvinnor_farg3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"brottande_kvinnor_farg3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"brottande_kvinnor_farg4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"brottande_kvinnor_farg4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"brottande_kvinnor_farg4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"brottande_kvinnor_farg4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"brottande_kvinnor_farg5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"brottande_kvinnor_farg5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"brottande_kvinnor_farg5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"brottande_kvinnor_farg5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"brottande_kvinnor_farg1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"brottande_kvinnor_farg2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"brottande_kvinnor_farg3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"brottande_kvinnor_farg4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"brottande_kvinnor_farg5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"brottande_man", "1F93C-200D-2642-FE0F"},
    {"brottande_man_farg1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"brottande_man_farg1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"brottande_man_farg1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"brottande_man_farg1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"brottande_man_farg2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"brottande_man_farg2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"brottande_man_farg2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"brottande_man_farg2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"brottande_man_farg3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"brottande_man_farg3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"brottande_man_farg3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"brottande_man_farg3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"brottande_man_farg4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"brottande_man_farg4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"brottande_man_farg4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"brottande_man_farg4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"brottande_man_farg5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"brottande_man_farg5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"brottande_man_farg5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"brottande_man_farg5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"brottande_man_farg1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"brottande_man_farg2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"brottande_man_farg3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"brottande_man_farg4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"brottande_man_farg5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_spelar_vattenpolo", "1F93D-200D-2640-FE0F"},
    {"kvinna_som_spelar_vattenpolo_farg1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_spelar_vattenpolo_farg2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_spelar_vattenpolo_farg3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_spelar_vattenpolo_farg4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_spelar_vattenpolo_farg5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"man_som_spelar_vattenpolo", "1F93D-200D-2642-FE0F"},
    {"man_som_spelar_vattenpolo_farg1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"man_som_spelar_vattenpolo_farg2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"man_som_spelar_vattenpolo_farg3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"man_som_spelar_vattenpolo_farg4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"man_som_spelar_vattenpolo_farg5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_spelar_handboll", "1F93E-200D-2640-FE0F"},
    {"kvinna_som_spelar_handboll_farg1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_spelar_handboll_farg2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_spelar_handboll_farg3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_spelar_handboll_farg4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_spelar_handboll_farg5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"man_som_spelar_handboll", "1F93E-200D-2642-FE0F"},
    {"man_som_spelar_handboll_farg1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"man_som_spelar_handboll_farg2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"man_som_spelar_handboll_farg3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"man_som_spelar_handboll_farg4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"man_som_spelar_handboll_farg5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_superhjalte", "1F9B8-200D-2640-FE0F"},
    {"kvinnlig_superhjalte_farg1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_superhjalte_farg2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_superhjalte_farg3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_superhjalte_farg4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_superhjalte_farg5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"manlig_superhjalte", "1F9B8-200D-2642-FE0F"},
    {"manlig_superhjalte_farg1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"manlig_superhjalte_farg2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"manlig_superhjalte_farg3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"manlig_superhjalte_farg4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"manlig_superhjalte_farg5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_superskurk", "1F9B9-200D-2640-FE0F"},
    {"kvinnlig_superskurk_farg1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_superskurk_farg2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_superskurk_farg3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_superskurk_farg4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_superskurk_farg5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"manlig_superskurk", "1F9B9-200D-2642-FE0F"},
    {"manlig_superskurk_farg1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"manlig_superskurk_farg2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"manlig_superskurk_farg3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"manlig_superskurk_farg4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"manlig_superskurk_farg5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"staende_kvinna", "1F9CD-200D-2640-FE0F"},
    {"staende_kvinna_farg1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"staende_kvinna_farg2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"staende_kvinna_farg3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"staende_kvinna_farg4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"staende_kvinna_farg5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"staende_man", "1F9CD-200D-2642-FE0F"},
    {"staende_man_farg1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"staende_man_farg2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"staende_man_farg3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"staende_man_farg4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"staende_man_farg5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"knasittande_kvinna", "1F9CE-200D-2640-FE0F"},
    {"knasittande_kvinna_farg1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"knasittande_kvinna_farg2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"knasittande_kvinna_farg3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"knasittande_kvinna_farg4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"knasittande_kvinna_farg5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"knasittande_kvinna_at_hoger", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knasittande_kvinna_at_hoger_farg1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knasittande_kvinna_at_hoger_farg2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knasittande_kvinna_at_hoger_farg3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knasittande_kvinna_at_hoger_farg4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knasittande_kvinna_at_hoger_farg5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knasittande_man", "1F9CE-200D-2642-FE0F"},
    {"knasittande_man_farg1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"knasittande_man_farg2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"knasittande_man_farg3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"knasittande_man_farg4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"knasittande_man_farg5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"knasittande_man_at_hoger", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knasittande_man_at_hoger_farg1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knasittande_man_at_hoger_farg2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knasittande_man_at_hoger_farg3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knasittande_man_at_hoger_farg4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knasittande_man_at_hoger_farg5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"horselskadad_kvinna", "1F9CF-200D-2640-FE0F"},
    {"horselskadad_kvinna_farg1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"horselskadad_kvinna_farg2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"horselskadad_kvinna_farg3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"horselskadad_kvinna_farg4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"horselskadad_kvinna_farg5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"horselskadad_man", "1F9CF-200D-2642-FE0F"},
    {"horselskadad_man_farg1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"horselskadad_man_farg2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"horselskadad_man_farg3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"horselskadad_man_farg4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"horselskadad_man_farg5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"kvinna_skagg", "1F9D4-200D-2640-FE0F"},
    {"kvinna_skagg_farg1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"kvinna_skagg_farg2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"kvinna_skagg_farg3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"kvinna_skagg_farg4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"kvinna_skagg_farg5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"man_skagg", "1F9D4-200D-2642-FE0F"},
    {"man_skagg_farg1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"man_skagg_farg2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"man_skagg_farg3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"man_skagg_farg4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"man_skagg_farg5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"kvinna_i_angbastu", "1F9D6-200D-2640-FE0F"},
    {"kvinna_i_angbastu_farg1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"kvinna_i_angbastu_farg2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"kvinna_i_angbastu_farg3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"kvinna_i_angbastu_farg4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"kvinna_i_angbastu_farg5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"man_i_angbastu", "1F9D6-200D-2642-FE0F"},
    {"man_i_angbastu_farg1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"man_i_angbastu_farg2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"man_i_angbastu_farg3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"man_i_angbastu_farg4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"man_i_angbastu_farg5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_klattrar", "1F9D7-200D-2640-FE0F"},
    {"kvinna_som_klattrar_farg1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_klattrar_farg2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_klattrar_farg3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_klattrar_farg4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_klattrar_farg5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"man_som_klattrar", "1F9D7-200D-2642-FE0F"},
    {"man_som_klattrar_farg1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"man_som_klattrar_farg2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"man_som_klattrar_farg3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"man_som_klattrar_farg4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"man_som_klattrar_farg5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"kvinna_i_lotusposition", "1F9D8-200D-2640-FE0F"},
    {"kvinna_i_lotusposition_farg1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"kvinna_i_lotusposition_farg2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"kvinna_i_lotusposition_farg3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"kvinna_i_lotusposition_farg4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"kvinna_i_lotusposition_farg5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"man_i_lotusposition", "1F9D8-200D-2642-FE0F"},
    {"man_i_lotusposition_farg1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"man_i_lotusposition_farg2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"man_i_lotusposition_farg3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"man_i_lotusposition_farg4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"man_i_lotusposition_farg5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_magiker", "1F9D9-200D-2640-FE0F"},
    {"kvinnlig_magiker_farg1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_magiker_farg2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_magiker_farg3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_magiker_farg4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_magiker_farg5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"manlig_magiker", "1F9D9-200D-2642-FE0F"},
    {"manlig_magiker_farg1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"manlig_magiker_farg2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"manlig_magiker_farg3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"manlig_magiker_farg4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"manlig_magiker_farg5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_fe", "1F9DA-200D-2640-FE0F"},
    {"kvinnlig_fe_farg1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_fe_farg2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_fe_farg3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_fe_farg4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_fe_farg5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"manlig_alva", "1F9DA-200D-2642-FE0F"},
    {"manlig_alva_farg1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"manlig_alva_farg2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"manlig_alva_farg3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"manlig_alva_farg4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"manlig_alva_farg5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_vampyr", "1F9DB-200D-2640-FE0F"},
    {"kvinnlig_vampyr_farg1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_vampyr_farg2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_vampyr_farg3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_vampyr_farg4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_vampyr_farg5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"manlig_vampyr", "1F9DB-200D-2642-FE0F"},
    {"manlig_vampyr_farg1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"manlig_vampyr_farg2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"manlig_vampyr_farg3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"manlig_vampyr_farg4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"manlig_vampyr_farg5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"sjojungfru", "1F9DC-200D-2640-FE0F"},
    {"sjojungfru_farg1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"sjojungfru_farg2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"sjojungfru_farg3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"sjojungfru_farg4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"sjojungfru_farg5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"stromkarl", "1F9DC-200D-2642-FE0F"},
    {"stromkarl_farg1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"stromkarl_farg2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"stromkarl_farg3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"stromkarl_farg4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"stromkarl_farg5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_alv", "1F9DD-200D-2640-FE0F"},
    {"kvinnlig_alv_farg1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_alv_farg2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_alv_farg3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_alv_farg4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_alv_farg5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"manlig_alv", "1F9DD-200D-2642-FE0F"},
    {"manlig_alv_farg1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"manlig_alv_farg2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"manlig_alv_farg3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"manlig_alv_farg4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"manlig_alv_farg5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_ande", "1F9DE-200D-2640-FE0F"},
    {"manlig_ande", "1F9DE-200D-2642-FE0F"},
    {"kvinnlig_zombie", "1F9DF-200D-2640-FE0F"},
    {"manlig_zombie", "1F9DF-200D-2642-FE0F"},
    {"man_rodharig", "1F468-200D-1F9B0"},
    {"man_rodharig_farg1", "1F468-1F3FB-200D-1F9B0"},
    {"man_rodharig_farg2", "1F468-1F3FC-200D-1F9B0"},
    {"man_rodharig_farg3", "1F468-1F3FD-200D-1F9B0"},
    {"man_rodharig_farg4", "1F468-1F3FE-200D-1F9B0"},
    {"man_rodharig_farg5", "1F468-1F3FF-200D-1F9B0"},
    {"man_lockigt_har", "1F468-200D-1F9B1"},
    {"man_lockigt_har_farg1", "1F468-1F3FB-200D-1F9B1"},
    {"man_lockigt_har_farg2", "1F468-1F3FC-200D-1F9B1"},
    {"man_lockigt_har_farg3", "1F468-1F3FD-200D-1F9B1"},
    {"man_lockigt_har_farg4", "1F468-1F3FE-200D-1F9B1"},
    {"man_lockigt_har_farg5", "1F468-1F3FF-200D-1F9B1"},
    {"man_skallig", "1F468-200D-1F9B2"},
    {"man_skallig_farg1", "1F468-1F3FB-200D-1F9B2"},
    {"man_skallig_farg2", "1F468-1F3FC-200D-1F9B2"},
    {"man_skallig_farg3", "1F468-1F3FD-200D-1F9B2"},
    {"man_skallig_farg4", "1F468-1F3FE-200D-1F9B2"},
    {"man_skallig_farg5", "1F468-1F3FF-200D-1F9B2"},
    {"man_vitharig", "1F468-200D-1F9B3"},
    {"man_vitharig_farg1", "1F468-1F3FB-200D-1F9B3"},
    {"man_vitharig_farg2", "1F468-1F3FC-200D-1F9B3"},
    {"man_vitharig_farg3", "1F468-1F3FD-200D-1F9B3"},
    {"man_vitharig_farg4", "1F468-1F3FE-200D-1F9B3"},
    {"man_vitharig_farg5", "1F468-1F3FF-200D-1F9B3"},
    {"kvinna_rodharig", "1F469-200D-1F9B0"},
    {"kvinna_rodharig_farg1", "1F469-1F3FB-200D-1F9B0"},
    {"kvinna_rodharig_farg2", "1F469-1F3FC-200D-1F9B0"},
    {"kvinna_rodharig_farg3", "1F469-1F3FD-200D-1F9B0"},
    {"kvinna_rodharig_farg4", "1F469-1F3FE-200D-1F9B0"},
    {"kvinna_rodharig_farg5", "1F469-1F3FF-200D-1F9B0"},
    {"kvinna_lockigt_har", "1F469-200D-1F9B1"},
    {"kvinna_lockigt_har_farg1", "1F469-1F3FB-200D-1F9B1"},
    {"kvinna_lockigt_har_farg2", "1F469-1F3FC-200D-1F9B1"},
    {"kvinna_lockigt_har_farg3", "1F469-1F3FD-200D-1F9B1"},
    {"kvinna_lockigt_har_farg4", "1F469-1F3FE-200D-1F9B1"},
    {"kvinna_lockigt_har_farg5", "1F469-1F3FF-200D-1F9B1"},
    {"kvinna_skallig", "1F469-200D-1F9B2"},
    {"kvinna_skallig_farg1", "1F469-1F3FB-200D-1F9B2"},
    {"kvinna_skallig_farg2", "1F469-1F3FC-200D-1F9B2"},
    {"kvinna_skallig_farg3", "1F469-1F3FD-200D-1F9B2"},
    {"kvinna_skallig_farg4", "1F469-1F3FE-200D-1F9B2"},
    {"kvinna_skallig_farg5", "1F469-1F3FF-200D-1F9B2"},
    {"kvinna_vitharig", "1F469-200D-1F9B3"},
    {"kvinna_vitharig_farg1", "1F469-1F3FB-200D-1F9B3"},
    {"kvinna_vitharig_farg2", "1F469-1F3FC-200D-1F9B3"},
    {"kvinna_vitharig_farg3", "1F469-1F3FD-200D-1F9B3"},
    {"kvinna_vitharig_farg4", "1F469-1F3FE-200D-1F9B3"},
    {"kvinna_vitharig_farg5", "1F469-1F3FF-200D-1F9B3"},
    {"person_rodharig", "1F9D1-200D-1F9B0"},
    {"person_rodharig_farg1", "1F9D1-1F3FB-200D-1F9B0"},
    {"person_rodharig_farg2", "1F9D1-1F3FC-200D-1F9B0"},
    {"person_rodharig_farg3", "1F9D1-1F3FD-200D-1F9B0"},
    {"person_rodharig_farg4", "1F9D1-1F3FE-200D-1F9B0"},
    {"person_rodharig_farg5", "1F9D1-1F3FF-200D-1F9B0"},
    {"person_lockigt_har", "1F9D1-200D-1F9B1"},
    {"person_lockigt_har_farg1", "1F9D1-1F3FB-200D-1F9B1"},
    {"person_lockigt_har_farg2", "1F9D1-1F3FC-200D-1F9B1"},
    {"person_lockigt_har_farg3", "1F9D1-1F3FD-200D-1F9B1"},
    {"person_lockigt_har_farg4", "1F9D1-1F3FE-200D-1F9B1"},
    {"person_lockigt_har_farg5", "1F9D1-1F3FF-200D-1F9B1"},
    {"person_skallig", "1F9D1-200D-1F9B2"},
    {"person_skallig_farg1", "1F9D1-1F3FB-200D-1F9B2"},
    {"person_skallig_farg2", "1F9D1-1F3FC-200D-1F9B2"},
    {"person_skallig_farg3", "1F9D1-1F3FD-200D-1F9B2"},
    {"person_skallig_farg4", "1F9D1-1F3FE-200D-1F9B2"},
    {"person_skallig_farg5", "1F9D1-1F3FF-200D-1F9B2"},
    {"person_vitharig", "1F9D1-200D-1F9B3"},
    {"person_vitharig_farg1", "1F9D1-1F3FB-200D-1F9B3"},
    {"person_vitharig_farg2", "1F9D1-1F3FC-200D-1F9B3"},
    {"person_vitharig_farg3", "1F9D1-1F3FD-200D-1F9B3"},
    {"person_vitharig_farg4", "1F9D1-1F3FE-200D-1F9B3"},
    {"person_vitharig_farg5", "1F9D1-1F3FF-200D-1F9B3"},
    {"bruten_lank", "26D3-FE0F-200D-1F4A5"},
    {"brinnande_hjarta", "2764-FE0F-200D-1F525"},
    {"lakande_hjarta", "2764-FE0F-200D-1FA79"},
    {"brun_svamp", "1F344-200D-1F7EB"},
    {"lime", "1F34B-200D-1F7E9"},
    {"transflagga", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"regnbagsflagga", "1F3F3-FE0F-200D-1F308"},
    {"sjorovarflagga", "1F3F4-200D-2620-FE0F"},
    {"svart_katt", "1F408-200D-2B1B"},
    {"servicehund", "1F415-200D-1F9BA"},
    {"koltrast", "1F426-200D-2B1B"},
    {"fenix", "1F426-200D-1F525"},
    {"isbjorn", "1F43B-200D-2744-FE0F"},
    {"oga_i_pratbubbla", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"ansikte_andas_ut", "1F62E-200D-1F4A8"},
    {"ansikte_med_spiralogon", "1F635-200D-1F4AB"},
    {"ansikte_i_moln", "1F636-200D-1F32B-FE0F"},
    {"huvud_skakar_horisontellt", "1F642-200D-2194-FE0F"},
    {"huvud_skakar_vertikalt", "1F642-200D-2195-FE0F"},
    {"balettdansare", "1F9D1-200D-1FA70"},
    {"balettdansare_farg1", "1F9D1-1F3FB-200D-1FA70"},
    {"balettdansare_farg2", "1F9D1-1F3FC-200D-1FA70"},
    {"balettdansare_farg3", "1F9D1-1F3FD-200D-1FA70"},
    {"balettdansare_farg4", "1F9D1-1F3FE-200D-1FA70"},
    {"balettdansare_farg5", "1F9D1-1F3FF-200D-1FA70"},
    {"utropstecken_och_frågetecken", "2049"},
    {"skyddat_varumärke", "2122"},
    {"pil_höger_och_vänster", "2194"},
    {"uppåtpil_vänster", "2196"},
    {"uppåtpil_höger", "2197"},
    {"nedåtpil_höger", "2198"},
    {"nedåtpil_vänster", "2199"},
    {"snögubbe", "2603"},
    {"treklöver", "2618"},
    {"dödskalle_med_ben_i_kors", "2620"},
    {"varning_för_strålrisk", "2622"},
    {"väduren", "2648"},
    {"klöver", "2663"},
    {"hjärter", "2665"},
    {"varma_källor", "2668"},
    {"korslagda_svärd", "2694"},
    {"balansvåg", "2696"},
    {"davidsstjärna", "2721"},
    {"gnistrande_stjärnor", "2728"},
    {"åttauddig_spetsig_stjärna", "2733"},
    {"åttauddig_stjärna", "2734"},
    {"snöflinga", "2744"},
    {"rött_frågetecken", "2753"},
    {"vitt_frågetecken", "2754"},
    {"rött_utropstecken", "2757"},
    {"utropstecken_i_form_av_ett_hjärta", "2763"},
    {"rött_hjärta", "2764"},
    {"svängd_uppåtpil", "2934"},
    {"svängd_nedåtpil", "2935"},
    {"vågigt_tankstreck", "3030"},
    {"upphovsrättssymbol", "00A9"},
    {"registrerat_varumärke", "00AE"},
    {"svängd_vänsterpil", "21A9"},
    {"svängd_högerpil", "21AA"},
    {"framåtspolningsknapp", "23E9"},
    {"knapp_för_snabbspola_tillbaka", "23EA"},
    {"dubbla_uppåtpilar", "23EB"},
    {"dubbla_nedåtpilar", "23EC"},
    {"knapp_för_nästa_spår", "23ED"},
    {"knapp_för_sista_spåret", "23EE"},
    {"knapp_för_spela_eller_pausa", "23EF"},
    {"väckarklocka", "23F0"},
    {"bakåtspolningsknapp", "25C0"},
    {"stjärna_och_halvmåne", "262A"},
    {"kräftan", "264B"},
    {"vågen", "264E"},
    {"återvinningssymbol", "267B"},
    {"oändlighet", "267E"},
    {"symbol_för_atom", "269B"},
    {"högspänning", "26A1"},
    {"snögubbe_utan_snö", "26C4"},
    {"ormbäraren", "26CE"},
    {"räddningsarbetarhjälm", "26D1"},
    {"förbud_mot_infart_med_fordon", "26D4"},
    {"parasoll_på_marken", "26F1"},
    {"fontän", "26F2"},
    {"flagga_i_hål", "26F3"},
    {"färja", "26F4"},
    {"segelbåt", "26F5"},
    {"skidåkare", "26F7"},
    {"tält", "26FA"},
    {"bränslepump", "26FD"},
    {"knytnäve_i_luften", "270A"},
    {"knytnäve_i_luften_färg1", "270A-1F3FB"},
    {"knytnäve_i_luften_färg2", "270A-1F3FC"},
    {"knytnäve_i_luften_färg3", "270A-1F3FD"},
    {"knytnäve_i_luften_färg4", "270A-1F3FE"},
    {"knytnäve_i_luften_färg5", "270A-1F3FF"},
    {"rött_kryss", "274C"},
    {"högerpil", "27A1"},
    {"ögla", "27B0"},
    {"dubbel_ögla", "27BF"},
    {"vänsterpil", "2B05"},
    {"uppåtpil", "2B06"},
    {"nedåtpil", "2B07"},
    {"stjärna", "2B50"},
    {"japansk_här_knapp", "1F201"},
    {"japansk_förbjudet_knapp", "1F232"},
    {"japansk_godkänt_betyg_knapp", "1F234"},
    {"japansk_månadsbelopp_knapp", "1F237"},
    {"japansk_användning_knapp", "1F238"},
    {"japansk_öppet_knapp", "1F23A"},
    {"hopfällt_paraply", "1F302"},
    {"stjärnklar_natt", "1F303"},
    {"soluppgång_över_berg", "1F304"},
    {"soluppgång", "1F305"},
    {"solnedgång", "1F307"},
    {"regnbåge", "1F308"},
    {"bro_på_natten", "1F309"},
    {"våg", "1F30A"},
    {"jordglob_europaafrika", "1F30D"},
    {"jordglob_asienaustralien", "1F30F"},
    {"nymåne", "1F311"},
    {"måne_i_första_kvarteret", "1F312"},
    {"tilltagande_halvmåne", "1F313"},
    {"måne_i_andra_kvarteret", "1F314"},
    {"fullmåne", "1F315"},
    {"måne_i_tredje_kvarteret", "1F316"},
    {"avtagande_halvmåne", "1F317"},
    {"måne_i_fjärde_kvarteret", "1F318"},
    {"månskära", "1F319"},
    {"nymåne_med_ansikte", "1F31A"},
    {"tilltagande_halvmåne_med_ansikte", "1F31B"},
    {"avtagande_halvmåne_med_ansikte", "1F31C"},
    {"fullmåne_med_ansikte", "1F31D"},
    {"blänkande_stjärna", "1F31F"},
    {"stjärnfall", "1F320"},
    {"moln_med_snö", "1F328"},
    {"blåser_vind", "1F32C"},
    {"barrträd", "1F332"},
    {"lövträd", "1F333"},
    {"körsbärsblomma", "1F338"},
    {"ört", "1F33F"},
    {"fyrklöver", "1F340"},
    {"lönnlöv", "1F341"},
    {"fallande_löv", "1F342"},
    {"löv_i_vinden", "1F343"},
    {"rött_äpple", "1F34E"},
    {"grönt_äpple", "1F34F"},
    {"päron", "1F350"},
    {"körsbär", "1F352"},
    {"köttben", "1F356"},
    {"kycklinglår", "1F357"},
    {"skål_nudelsoppa", "1F35C"},
    {"bröd", "1F35E"},
    {"rostad_sötpotatis", "1F360"},
    {"friterad_räka", "1F364"},
    {"kakigōri", "1F367"},
    {"brylépudding", "1F36E"},
    {"tårtbit", "1F370"},
    {"bentolåda", "1F371"},
    {"ölstop", "1F37A"},
    {"skålande_ölstop", "1F37B"},
    {"födelsedagstårta", "1F382"},
    {"konfettismällare", "1F389"},
    {"tanabataträd", "1F38B"},
    {"månskådning", "1F391"},
    {"ryggsäck", "1F392"},
    {"militär_medalj", "1F396"},
    {"påminnelseband", "1F397"},
    {"nivåreglage", "1F39A"},
    {"inträdesbiljetter", "1F39F"},
    {"karusellhäst", "1F3A0"},
    {"fiskespö", "1F3A3"},
    {"hörlur", "1F3A7"},
    {"färgpalett", "1F3A8"},
    {"hög_hatt", "1F3A9"},
    {"cirkustält", "1F3AA"},
    {"speltärning", "1F3B2"},
    {"löparlinne", "1F3BD"},
    {"snowboardåkare", "1F3C2"},
    {"snowboardåkare_färg1", "1F3C2-1F3FB"},
    {"snowboardåkare_färg2", "1F3C2-1F3FC"},
    {"snowboardåkare_färg3", "1F3C2-1F3FD"},
    {"snowboardåkare_färg4", "1F3C2-1F3FE"},
    {"snowboardåkare_färg5", "1F3C2-1F3FF"},
    {"hästkapplöpning", "1F3C7"},
    {"hästkapplöpning_färg1", "1F3C7-1F3FB"},
    {"hästkapplöpning_färg2", "1F3C7-1F3FC"},
    {"hästkapplöpning_färg3", "1F3C7-1F3FD"},
    {"hästkapplöpning_färg4", "1F3C7-1F3FE"},
    {"hästkapplöpning_färg5", "1F3C7-1F3FF"},
    {"snöklätt_berg", "1F3D4"},
    {"fallfärdigt_hus", "1F3DA"},
    {"öken", "1F3DC"},
    {"öde_ö", "1F3DD"},
    {"hus_med_trädgård", "1F3E1"},
    {"kärlekshotell", "1F3E9"},
    {"röd_papperslykta", "1F3EE"},
    {"pil_och_båge", "1F3F9"},
    {"mellanfärgad_hy", "1F3FD"},
    {"mellanmörk_hy", "1F3FE"},
    {"mörk_hy", "1F3FF"},
    {"råtta", "1F400"},
    {"häst", "1F40E"},
    {"vädur", "1F40F"},
    {"höna", "1F414"},
    {"bläckfisk", "1F419"},
    {"snäcka", "1F41A"},
    {"blåsfisk", "1F421"},
    {"sköldpadda", "1F422"},
    {"nykläckt_kyckling", "1F423"},
    {"kyckling_framifrån", "1F425"},
    {"fågel", "1F426"},
    {"hästansikte", "1F434"},
    {"björnansikte", "1F43B"},
    {"ögon", "1F440"},
    {"öga", "1F441"},
    {"öra", "1F442"},
    {"öra_färg1", "1F442-1F3FB"},
    {"öra_färg2", "1F442-1F3FC"},
    {"öra_färg3", "1F442-1F3FD"},
    {"öra_färg4", "1F442-1F3FE"},
    {"öra_färg5", "1F442-1F3FF"},
    {"näsa", "1F443"},
    {"näsa_färg1", "1F443-1F3FB"},
    {"näsa_färg2", "1F443-1F3FC"},
    {"näsa_färg3", "1F443-1F3FD"},
    {"näsa_färg4", "1F443-1F3FE"},
    {"näsa_färg5", "1F443-1F3FF"},
    {"nedåtpekande_finger", "1F447"},
    {"nedåtpekande_finger_färg1", "1F447-1F3FB"},
    {"nedåtpekande_finger_färg2", "1F447-1F3FC"},
    {"nedåtpekande_finger_färg3", "1F447-1F3FD"},
    {"nedåtpekande_finger_färg4", "1F447-1F3FE"},
    {"nedåtpekande_finger_färg5", "1F447-1F3FF"},
    {"vänsterpekande_finger", "1F448"},
    {"vänsterpekande_finger_färg1", "1F448-1F3FB"},
    {"vänsterpekande_finger_färg2", "1F448-1F3FC"},
    {"vänsterpekande_finger_färg3", "1F448-1F3FD"},
    {"vänsterpekande_finger_färg4", "1F448-1F3FE"},
    {"vänsterpekande_finger_färg5", "1F448-1F3FF"},
    {"högerpekande_finger", "1F449"},
    {"högerpekande_finger_färg1", "1F449-1F3FB"},
    {"högerpekande_finger_färg2", "1F449-1F3FC"},
    {"högerpekande_finger_färg3", "1F449-1F3FD"},
    {"högerpekande_finger_färg4", "1F449-1F3FE"},
    {"högerpekande_finger_färg5", "1F449-1F3FF"},
    {"klappande_händer", "1F44F"},
    {"klappande_händer_färg1", "1F44F-1F3FB"},
    {"klappande_händer_färg2", "1F44F-1F3FC"},
    {"klappande_händer_färg3", "1F44F-1F3FD"},
    {"klappande_händer_färg4", "1F44F-1F3FE"},
    {"klappande_händer_färg5", "1F44F-1F3FF"},
    {"öppna_händer", "1F450"},
    {"öppna_händer_färg1", "1F450-1F3FB"},
    {"öppna_händer_färg2", "1F450-1F3FC"},
    {"öppna_händer_färg3", "1F450-1F3FD"},
    {"öppna_händer_färg4", "1F450-1F3FE"},
    {"öppna_händer_färg5", "1F450-1F3FF"},
    {"glasögon", "1F453"},
    {"klänning", "1F457"},
    {"portmonnä", "1F45B"},
    {"handväska", "1F45C"},
    {"kuvertväska", "1F45D"},
    {"högklackad_sko", "1F460"},
    {"damstövel", "1F462"},
    {"man_och_kvinna_håller_hand", "1F46B"},
    {"man_och_kvinna_håller_hand_färg1", "1F46B-1F3FB"},
    {"man_och_kvinna_håller_hand_färg2", "1F46B-1F3FC"},
    {"man_och_kvinna_håller_hand_färg3", "1F46B-1F3FD"},
    {"man_och_kvinna_håller_hand_färg4", "1F46B-1F3FE"},
    {"man_och_kvinna_håller_hand_färg5", "1F46B-1F3FF"},
    {"man_och_kvinna_håller_hand_färg1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"man_och_kvinna_håller_hand_färg1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"man_och_kvinna_håller_hand_färg1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"man_och_kvinna_håller_hand_färg1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"man_och_kvinna_håller_hand_färg2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"man_och_kvinna_håller_hand_färg2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"man_och_kvinna_håller_hand_färg2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"man_och_kvinna_håller_hand_färg2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"man_och_kvinna_håller_hand_färg3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"man_och_kvinna_håller_hand_färg3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"man_och_kvinna_håller_hand_färg3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"man_och_kvinna_håller_hand_färg3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"man_och_kvinna_håller_hand_färg4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"man_och_kvinna_håller_hand_färg4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"man_och_kvinna_håller_hand_färg4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"man_och_kvinna_håller_hand_färg4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"man_och_kvinna_håller_hand_färg5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"man_och_kvinna_håller_hand_färg5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"man_och_kvinna_håller_hand_färg5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"man_och_kvinna_håller_hand_färg5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"två_män_håller_hand", "1F46C"},
    {"två_män_håller_hand_färg1", "1F46C-1F3FB"},
    {"två_män_håller_hand_färg2", "1F46C-1F3FC"},
    {"två_män_håller_hand_färg3", "1F46C-1F3FD"},
    {"två_män_håller_hand_färg4", "1F46C-1F3FE"},
    {"två_män_håller_hand_färg5", "1F46C-1F3FF"},
    {"två_män_håller_hand_färg1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"två_män_håller_hand_färg1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"två_män_håller_hand_färg1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"två_män_håller_hand_färg1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"två_män_håller_hand_färg2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"två_män_håller_hand_färg2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"två_män_håller_hand_färg2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"två_män_håller_hand_färg2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"två_män_håller_hand_färg3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"två_män_håller_hand_färg3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"två_män_håller_hand_färg3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"två_män_håller_hand_färg3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"två_män_håller_hand_färg4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"två_män_håller_hand_färg4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"två_män_håller_hand_färg4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"två_män_håller_hand_färg4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"två_män_håller_hand_färg5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"två_män_håller_hand_färg5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"två_män_håller_hand_färg5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"två_män_håller_hand_färg5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"två_kvinnor_håller_hand", "1F46D"},
    {"två_kvinnor_håller_hand_färg1", "1F46D-1F3FB"},
    {"två_kvinnor_håller_hand_färg2", "1F46D-1F3FC"},
    {"två_kvinnor_håller_hand_färg3", "1F46D-1F3FD"},
    {"två_kvinnor_håller_hand_färg4", "1F46D-1F3FE"},
    {"två_kvinnor_håller_hand_färg5", "1F46D-1F3FF"},
    {"två_kvinnor_håller_hand_färg1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"två_kvinnor_håller_hand_färg1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"två_kvinnor_håller_hand_färg1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"två_kvinnor_håller_hand_färg1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"två_kvinnor_håller_hand_färg2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"två_kvinnor_håller_hand_färg2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"två_kvinnor_håller_hand_färg2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"två_kvinnor_håller_hand_färg2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"två_kvinnor_håller_hand_färg3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"två_kvinnor_håller_hand_färg3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"två_kvinnor_håller_hand_färg3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"två_kvinnor_håller_hand_färg3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"två_kvinnor_håller_hand_färg4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"två_kvinnor_håller_hand_färg4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"två_kvinnor_håller_hand_färg4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"två_kvinnor_håller_hand_färg4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"två_kvinnor_håller_hand_färg5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"två_kvinnor_håller_hand_färg5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"två_kvinnor_håller_hand_färg5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"två_kvinnor_håller_hand_färg5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"personer_med_kaninöron", "1F46F"},
    {"personer_med_kaninöron_färg1", "1F46F-1F3FB"},
    {"personer_med_kaninöron_färg2", "1F46F-1F3FC"},
    {"personer_med_kaninöron_färg3", "1F46F-1F3FD"},
    {"personer_med_kaninöron_färg4", "1F46F-1F3FE"},
    {"personer_med_kaninöron_färg5", "1F46F-1F3FF"},
    {"personer_med_kaninöron_färg1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninöron_färg1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninöron_färg1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninöron_färg1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninöron_färg2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninöron_färg2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninöron_färg2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninöron_färg2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninöron_färg3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninöron_färg3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninöron_färg3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"personer_med_kaninöron_färg3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninöron_färg4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninöron_färg4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninöron_färg4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninöron_färg4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"personer_med_kaninöron_färg5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"personer_med_kaninöron_färg5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"personer_med_kaninöron_färg5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"personer_med_kaninöron_färg5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"person_i_brudslöja", "1F470"},
    {"person_i_brudslöja_färg1", "1F470-1F3FB"},
    {"person_i_brudslöja_färg2", "1F470-1F3FC"},
    {"person_i_brudslöja_färg3", "1F470-1F3FD"},
    {"person_i_brudslöja_färg4", "1F470-1F3FE"},
    {"person_i_brudslöja_färg5", "1F470-1F3FF"},
    {"spöke", "1F47B"},
    {"bebisängel", "1F47C"},
    {"bebisängel_färg1", "1F47C-1F3FB"},
    {"bebisängel_färg2", "1F47C-1F3FC"},
    {"bebisängel_färg3", "1F47C-1F3FD"},
    {"bebisängel_färg4", "1F47C-1F3FE"},
    {"bebisängel_färg5", "1F47C-1F3FF"},
    {"dödskalle", "1F480"},
    {"läppstift", "1F484"},
    {"person_som_får_håret_klippt", "1F487"},
    {"person_som_får_håret_klippt_färg1", "1F487-1F3FB"},
    {"person_som_får_håret_klippt_färg2", "1F487-1F3FC"},
    {"person_som_får_håret_klippt_färg3", "1F487-1F3FD"},
    {"person_som_får_håret_klippt_färg4", "1F487-1F3FE"},
    {"person_som_får_håret_klippt_färg5", "1F487-1F3FF"},
    {"läppavtryck", "1F48B"},
    {"kärleksbrev", "1F48C"},
    {"ädelsten", "1F48E"},
    {"par_med_hjärta", "1F491"},
    {"par_med_hjärta_färg1", "1F491-1F3FB"},
    {"par_med_hjärta_färg2", "1F491-1F3FC"},
    {"par_med_hjärta_färg3", "1F491-1F3FD"},
    {"par_med_hjärta_färg4", "1F491-1F3FE"},
    {"par_med_hjärta_färg5", "1F491-1F3FF"},
    {"par_med_hjärta_färg1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjärta_färg1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjärta_färg1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"par_med_hjärta_färg1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjärta_färg2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjärta_färg2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjärta_färg2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"par_med_hjärta_färg2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjärta_färg3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjärta_färg3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjärta_färg3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"par_med_hjärta_färg3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjärta_färg4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjärta_färg4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjärta_färg4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjärta_färg4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"par_med_hjärta_färg5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"par_med_hjärta_färg5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"par_med_hjärta_färg5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"par_med_hjärta_färg5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"bröllop", "1F492"},
    {"bultande_hjärta", "1F493"},
    {"brustet_hjärta", "1F494"},
    {"två_hjärtan", "1F495"},
    {"gnistrande_hjärta", "1F496"},
    {"växande_hjärta", "1F497"},
    {"hjärta_med_pil", "1F498"},
    {"blått_hjärta", "1F499"},
    {"grönt_hjärta", "1F49A"},
    {"gult_hjärta", "1F49B"},
    {"lila_hjärta", "1F49C"},
    {"hjärta_med_rosett", "1F49D"},
    {"snurrande_hjärtan", "1F49E"},
    {"hjärtdekoration", "1F49F"},
    {"glödlampa", "1F4A1"},
    {"symbol_för_ilska", "1F4A2"},
    {"stänkande_svett", "1F4A6"},
    {"bajshög", "1F4A9"},
    {"spänd_biceps", "1F4AA"},
    {"spänd_biceps_färg1", "1F4AA-1F3FB"},
    {"spänd_biceps_färg2", "1F4AA-1F3FC"},
    {"spänd_biceps_färg3", "1F4AA-1F3FD"},
    {"spänd_biceps_färg4", "1F4AA-1F3FE"},
    {"spänd_biceps_färg5", "1F4AA-1F3FF"},
    {"100_poäng", "1F4AF"},
    {"pengapåse", "1F4B0"},
    {"valutaväxling", "1F4B1"},
    {"bärbar_dator", "1F4BB"},
    {"portfölj", "1F4BC"},
    {"öppen_mapp", "1F4C2"},
    {"sida_med_text_uppåt", "1F4C4"},
    {"uppåtgående_trend", "1F4C8"},
    {"nedåtgående_trend", "1F4C9"},
    {"kartnål", "1F4CC"},
    {"knappnål", "1F4CD"},
    {"rätvinklig_linjal", "1F4D0"},
    {"bokmärkesflikar", "1F4D1"},
    {"stängd_bok", "1F4D5"},
    {"öppen_bok", "1F4D6"},
    {"grön_bok", "1F4D7"},
    {"blå_bok", "1F4D8"},
    {"böcker", "1F4DA"},
    {"kom_ihåg_block", "1F4DD"},
    {"personsökare", "1F4DF"},
    {"handhögtalare", "1F4E2"},
    {"stängd_brevlåda_med_sänkt_flagga", "1F4EA"},
    {"stängd_brevlåda_med_hissad_flagga", "1F4EB"},
    {"öppen_brevlåda_med_hissad_flagga", "1F4EC"},
    {"öppen_brevlåda_med_sänkt_flagga", "1F4ED"},
    {"postlåda", "1F4EE"},
    {"vibrationsläge", "1F4F3"},
    {"avstängd_mobil", "1F4F4"},
    {"mobil_förbjuden", "1F4F5"},
    {"blanda_låtar", "1F500"},
    {"upprepa_en_låt", "1F502"},
    {"låg_ljusstyrka", "1F505"},
    {"hög_ljusstyrka", "1F506"},
    {"avstängd_högtalare", "1F507"},
    {"högtalare", "1F508"},
    {"högtalare_med_medium_volym", "1F509"},
    {"högtalare_med_hög_volym", "1F50A"},
    {"förstoringsglas_som_lutar_åt_vänster", "1F50D"},
    {"förstoringsglas_som_lutar_åt_höger", "1F50E"},
    {"lås_med_bläckpenna", "1F50F"},
    {"lås_med_nyckel", "1F510"},
    {"låst_lås", "1F512"},
    {"öppet_lås", "1F513"},
    {"överstruken_ringklocka", "1F515"},
    {"bokmärke", "1F516"},
    {"länk", "1F517"},
    {"back_med_vänsterpil", "1F519"},
    {"end_med_högerpil", "1F51A"},
    {"on_med_pil_höger_och_vänster_ovanför", "1F51B"},
    {"soon_med_högerpil_ovanför", "1F51C"},
    {"top_med_uppåtpil", "1F51D"},
    {"ej_för_minderåriga", "1F51E"},
    {"bokstäver", "1F524"},
    {"kökskniv", "1F52A"},
    {"sexuddig_stjärna_med_prick", "1F52F"},
    {"japansk_symbol_för_nybörjare", "1F530"},
    {"röd_cirkel", "1F534"},
    {"blå_cirkel", "1F535"},
    {"stor_blå_diamantform", "1F537"},
    {"liten_blå_romb", "1F539"},
    {"uppåtpekande_röd_triangel", "1F53A"},
    {"nedåtpekande_röd_triangel", "1F53B"},
    {"uppåtknapp", "1F53C"},
    {"nedåtknapp", "1F53D"},
    {"moské", "1F54C"},
    {"klockan_två", "1F551"},
    {"klockan_åtta", "1F557"},
    {"halv_två", "1F55C"},
    {"halv_åtta", "1F562"},
    {"hål", "1F573"},
    {"svävande_man_i_kostym", "1F574"},
    {"svävande_man_i_kostym_färg1", "1F574-1F3FB"},
    {"svävande_man_i_kostym_färg2", "1F574-1F3FC"},
    {"svävande_man_i_kostym_färg3", "1F574-1F3FD"},
    {"svävande_man_i_kostym_färg4", "1F574-1F3FE"},
    {"svävande_man_i_kostym_färg5", "1F574-1F3FF"},
    {"solglasögon", "1F576"},
    {"spindelnät", "1F578"},
    {"länkade_gem", "1F587"},
    {"målarpensel", "1F58C"},
    {"långfinger_upp", "1F595"},
    {"långfinger_upp_färg1", "1F595-1F3FB"},
    {"långfinger_upp_färg2", "1F595-1F3FC"},
    {"långfinger_upp_färg3", "1F595-1F3FD"},
    {"långfinger_upp_färg4", "1F595-1F3FE"},
    {"långfinger_upp_färg5", "1F595-1F3FF"},
    {"vulcan_hälsning", "1F596"},
    {"vulcan_hälsning_färg1", "1F596-1F3FB"},
    {"vulcan_hälsning_färg2", "1F596-1F3FC"},
    {"vulcan_hälsning_färg3", "1F596-1F3FD"},
    {"vulcan_hälsning_färg4", "1F596-1F3FE"},
    {"vulcan_hälsning_färg5", "1F596-1F3FF"},
    {"svart_hjärta", "1F5A4"},
    {"stationär_dator", "1F5A5"},
    {"dokumentskåp", "1F5C4"},
    {"pratbubbla_åt_vänster", "1F5E8"},
    {"arg_pratbubbla_åt_höger", "1F5EF"},
    {"valurna_med_röstsedel", "1F5F3"},
    {"världskarta", "1F5FA"},
    {"karta_över_japan", "1F5FE"},
    {"strålande_ansikte_med_leende_ögon", "1F601"},
    {"ansikte_med_glädjetårar", "1F602"},
    {"flinande_ansikte_med_stora_ögon", "1F603"},
    {"flinande_ansikte_med_leende_ögon", "1F604"},
    {"flinande_ansikte_med_kisande_ögon", "1F606"},
    {"ansikte_som_blinkar_med_ett_öga", "1F609"},
    {"leende_ansikte_och_ögon", "1F60A"},
    {"lättat_ansikte", "1F60C"},
    {"leende_ansikte_med_hjärtformade_ögon", "1F60D"},
    {"leende_ansikte_med_solglasögon", "1F60E"},
    {"uttryckslöst_ansikte", "1F611"},
    {"missnöjt_ansikte", "1F612"},
    {"förbryllat_ansikte", "1F615"},
    {"förvirrat_ansikte", "1F616"},
    {"ansikte_ger_en_slängkyss", "1F618"},
    {"pussande_ansikte_med_leende_ögon", "1F619"},
    {"pussande_ansikte_med_slutna_ögon", "1F61A"},
    {"ansikte_räcker_ut_tungan", "1F61B"},
    {"blinkande_ansikte_räcker_ut_tungan", "1F61C"},
    {"ansikte_räcker_ut_tungan_med_stängda_ögon", "1F61D"},
    {"gråtande_ansikte", "1F622"},
    {"uthålligt_ansikte", "1F623"},
    {"ansikte_med_ånga_ur_näsan", "1F624"},
    {"besviket_men_lättat_ansikte", "1F625"},
    {"ogillande_ansikte_med_öppen_mun", "1F626"},
    {"plågat_ansikte", "1F627"},
    {"skrämt_ansikte", "1F628"},
    {"uttröttat_ansikte", "1F629"},
    {"sömnigt_ansikte", "1F62A"},
    {"trött_ansikte", "1F62B"},
    {"storgråtande_ansikte", "1F62D"},
    {"förvånat_ansikte", "1F62E"},
    {"flinande_kattansikte_med_leende_ögon", "1F638"},
    {"kattansikte_med_glädjetårar", "1F639"},
    {"leende_kattansikte_med_hjärtformade_ögon", "1F63B"},
    {"gråtande_kattansikte", "1F63F"},
    {"uttröttat_kattansikte", "1F640"},
    {"småleende_ansikte", "1F642"},
    {"upp_och_nervänt_ansikte", "1F643"},
    {"ansikte_som_himlar_med_ögonen", "1F644"},
    {"apa_hör_inget", "1F649"},
    {"apa_säger_inget", "1F64A"},
    {"person_som_räcker_upp_handen", "1F64B"},
    {"person_som_räcker_upp_handen_färg1", "1F64B-1F3FB"},
    {"person_som_räcker_upp_handen_färg2", "1F64B-1F3FC"},
    {"person_som_räcker_upp_handen_färg3", "1F64B-1F3FD"},
    {"person_som_räcker_upp_handen_färg4", "1F64B-1F3FE"},
    {"person_som_räcker_upp_handen_färg5", "1F64B-1F3FF"},
    {"båda_händerna_i_luften", "1F64C"},
    {"båda_händerna_i_luften_färg1", "1F64C-1F3FB"},
    {"båda_händerna_i_luften_färg2", "1F64C-1F3FC"},
    {"båda_händerna_i_luften_färg3", "1F64C-1F3FD"},
    {"båda_händerna_i_luften_färg4", "1F64C-1F3FE"},
    {"båda_händerna_i_luften_färg5", "1F64C-1F3FF"},
    {"beende_händer", "1F64F"},
    {"beende_händer_färg1", "1F64F-1F3FB"},
    {"beende_händer_färg2", "1F64F-1F3FC"},
    {"beende_händer_färg3", "1F64F-1F3FD"},
    {"beende_händer_färg4", "1F64F-1F3FE"},
    {"beende_händer_färg5", "1F64F-1F3FF"},
    {"ånglok", "1F682"},
    {"tågvagn", "1F683"},
    {"snabbtåg", "1F684"},
    {"höghastighetståg", "1F685"},
    {"tåg", "1F686"},
    {"snabbspårväg", "1F688"},
    {"tågstation", "1F689"},
    {"spårvagn", "1F68A"},
    {"spårvagnsvagn", "1F68B"},
    {"mötande_buss", "1F68D"},
    {"trådbuss", "1F68E"},
    {"busshållplats", "1F68F"},
    {"mötande_polisbil", "1F694"},
    {"mötande_taxi", "1F696"},
    {"mötande_bil", "1F698"},
    {"hängbana", "1F69F"},
    {"person_som_ror_en_båt", "1F6A3"},
    {"person_som_ror_en_båt_färg1", "1F6A3-1F3FB"},
    {"person_som_ror_en_båt_färg2", "1F6A3-1F3FC"},
    {"person_som_ror_en_båt_färg3", "1F6A3-1F3FD"},
    {"person_som_ror_en_båt_färg4", "1F6A3-1F3FE"},
    {"person_som_ror_en_båt_färg5", "1F6A3-1F3FF"},
    {"racerbåt", "1F6A4"},
    {"vågräta_trafikljus", "1F6A5"},
    {"lodräta_trafikljus", "1F6A6"},
    {"skylt_för_byggområde", "1F6A7"},
    {"dörr", "1F6AA"},
    {"förbjudet", "1F6AB"},
    {"rökning_förbjuden", "1F6AD"},
    {"skräpa_inte_ner", "1F6AF"},
    {"förbud_mot_trafik_med_cykel", "1F6B3"},
    {"person_som_cyklar_på_en_mountainbike", "1F6B5"},
    {"person_som_cyklar_på_en_mountainbike_färg1", "1F6B5-1F3FB"},
    {"person_som_cyklar_på_en_mountainbike_färg2", "1F6B5-1F3FC"},
    {"person_som_cyklar_på_en_mountainbike_färg3", "1F6B5-1F3FD"},
    {"person_som_cyklar_på_en_mountainbike_färg4", "1F6B5-1F3FE"},
    {"person_som_cyklar_på_en_mountainbike_färg5", "1F6B5-1F3FF"},
    {"fotgängare", "1F6B6"},
    {"fotgängare_färg1", "1F6B6-1F3FB"},
    {"fotgängare_färg2", "1F6B6-1F3FC"},
    {"fotgängare_färg3", "1F6B6-1F3FD"},
    {"fotgängare_färg4", "1F6B6-1F3FE"},
    {"fotgängare_färg5", "1F6B6-1F3FF"},
    {"förbud_mot_gångtrafik", "1F6B7"},
    {"bagagehämtning", "1F6C4"},
    {"kvarlämnat_bagage", "1F6C5"},
    {"person_i_en_säng", "1F6CC"},
    {"person_i_en_säng_färg1", "1F6CC-1F3FB"},
    {"person_i_en_säng_färg2", "1F6CC-1F3FC"},
    {"person_i_en_säng_färg3", "1F6CC-1F3FD"},
    {"person_i_en_säng_färg4", "1F6CC-1F3FE"},
    {"person_i_en_säng_färg5", "1F6CC-1F3FF"},
    {"shoppingpåsar", "1F6CD"},
    {"säng", "1F6CF"},
    {"plats_för_tillbedjan", "1F6D0"},
    {"trådlös", "1F6DC"},
    {"sköld", "1F6E1"},
    {"motorväg", "1F6E3"},
    {"räls", "1F6E4"},
    {"motorbåt", "1F6E5"},
    {"avgång_med_flyg", "1F6EB"},
    {"kälke", "1F6F7"},
    {"grön_cirkel", "1F7E2"},
    {"röd_kvadrat", "1F7E5"},
    {"blå_kvadrat", "1F7E6"},
    {"grön_kvadrat", "1F7E9"},
    {"fingerportmonnä", "1F90C"},
    {"fingerportmonnä_färg1", "1F90C-1F3FB"},
    {"fingerportmonnä_färg2", "1F90C-1F3FC"},
    {"fingerportmonnä_färg3", "1F90C-1F3FD"},
    {"fingerportmonnä_färg4", "1F90C-1F3FE"},
    {"fingerportmonnä_färg5", "1F90C-1F3FF"},
    {"vitt_hjärta", "1F90D"},
    {"brunt_hjärta", "1F90E"},
    {"ansikte_med_dollarögon", "1F911"},
    {"nördansikte", "1F913"},
    {"ansikte_som_tänker", "1F914"},
    {"ansikte_med_bandage_på_huvudet", "1F915"},
    {"djävulstecknet", "1F918"},
    {"djävulstecknet_färg1", "1F918-1F3FB"},
    {"djävulstecknet_färg2", "1F918-1F3FC"},
    {"djävulstecknet_färg3", "1F918-1F3FD"},
    {"djävulstecknet_färg4", "1F918-1F3FE"},
    {"djävulstecknet_färg5", "1F918-1F3FF"},
    {"höjd_handrygg", "1F91A"},
    {"höjd_handrygg_färg1", "1F91A-1F3FB"},
    {"höjd_handrygg_färg2", "1F91A-1F3FC"},
    {"höjd_handrygg_färg3", "1F91A-1F3FD"},
    {"höjd_handrygg_färg4", "1F91A-1F3FE"},
    {"höjd_handrygg_färg5", "1F91A-1F3FF"},
    {"vänstervänd_näve", "1F91B"},
    {"vänstervänd_näve_färg1", "1F91B-1F3FB"},
    {"vänstervänd_näve_färg2", "1F91B-1F3FC"},
    {"vänstervänd_näve_färg3", "1F91B-1F3FD"},
    {"vänstervänd_näve_färg4", "1F91B-1F3FE"},
    {"vänstervänd_näve_färg5", "1F91B-1F3FF"},
    {"högervänd_näve", "1F91C"},
    {"högervänd_näve_färg1", "1F91C-1F3FB"},
    {"högervänd_näve_färg2", "1F91C-1F3FC"},
    {"högervänd_näve_färg3", "1F91C-1F3FD"},
    {"högervänd_näve_färg4", "1F91C-1F3FE"},
    {"högervänd_näve_färg5", "1F91C-1F3FF"},
    {"jag_älskar_dig_gest", "1F91F"},
    {"jag_älskar_dig_gest_färg1", "1F91F-1F3FB"},
    {"jag_älskar_dig_gest_färg2", "1F91F-1F3FC"},
    {"jag_älskar_dig_gest_färg3", "1F91F-1F3FD"},
    {"jag_älskar_dig_gest_färg4", "1F91F-1F3FE"},
    {"jag_älskar_dig_gest_färg5", "1F91F-1F3FF"},
    {"äcklat_ansikte", "1F922"},
    {"rullar_runt_på_golvet_och_skrattar", "1F923"},
    {"person_som_slår_sig_för_pannan", "1F926"},
    {"person_som_slår_sig_för_pannan_färg1", "1F926-1F3FB"},
    {"person_som_slår_sig_för_pannan_färg2", "1F926-1F3FC"},
    {"person_som_slår_sig_för_pannan_färg3", "1F926-1F3FD"},
    {"person_som_slår_sig_för_pannan_färg4", "1F926-1F3FE"},
    {"person_som_slår_sig_för_pannan_färg5", "1F926-1F3FF"},
    {"ansikte_med_höjt_ögonbryn", "1F928"},
    {"stjärnögt_ansikte", "1F929"},
    {"ansikte_med_symboler_på_munnen", "1F92C"},
    {"ansikte_med_handen_för_munnen", "1F92D"},
    {"kräkande_ansikte", "1F92E"},
    {"person_som_rycker_på_axlarna", "1F937"},
    {"person_som_rycker_på_axlarna_färg1", "1F937-1F3FB"},
    {"person_som_rycker_på_axlarna_färg2", "1F937-1F3FC"},
    {"person_som_rycker_på_axlarna_färg3", "1F937-1F3FD"},
    {"person_som_rycker_på_axlarna_färg4", "1F937-1F3FE"},
    {"person_som_rycker_på_axlarna_färg5", "1F937-1F3FF"},
    {"person_som_fäktas", "1F93A"},
    {"skålande_glas", "1F942"},
    {"målnät", "1F945"},
    {"kampsportdräkt", "1F94B"},
    {"grönsallad", "1F957"},
    {"pitabröd", "1F959"},
    {"ägg", "1F95A"},
    {"mjölkglas", "1F95B"},
    {"jordnötter", "1F95C"},
    {"hämtmatkartong", "1F961"},
    {"ätpinnar", "1F962"},
    {"skål_och_sked", "1F963"},
    {"mugg_med_sugrör", "1F964"},
    {"kokosnöt", "1F965"},
    {"smörgås", "1F96A"},
    {"månkaka", "1F96E"},
    {"leende_ansikte_med_tre_hjärtan", "1F970"},
    {"gäspande_ansikte", "1F971"},
    {"tårögt_leende_ansikte", "1F972"},
    {"ansikte_som_håller_tillbaka_tårarna", "1F979"},
    {"bönfallande_ansikte", "1F97A"},
    {"simglasögon", "1F97D"},
    {"vandringskänga", "1F97E"},
    {"enhörningsansikte", "1F984"},
    {"örn", "1F985"},
    {"rävansikte", "1F98A"},
    {"fjäril", "1F98B"},
    {"ödla", "1F98E"},
    {"noshörning", "1F98F"},
    {"räka", "1F990"},
    {"mindre_bläckfisk", "1F991"},
    {"känguru", "1F998"},
    {"påfågel", "1F99A"},
    {"flodhäst", "1F99B"},
    {"tvättbjörn", "1F99D"},
    {"grävling", "1F9A1"},
    {"sengångare", "1F9A5"},
    {"bäver", "1F9AB"},
    {"säl", "1F9AD"},
    {"blindkäpp", "1F9AF"},
    {"rödhårig", "1F9B0"},
    {"lockigt_hår", "1F9B1"},
    {"vithårig", "1F9B3"},
    {"superhjälte", "1F9B8"},
    {"superhjälte_färg1", "1F9B8-1F3FB"},
    {"superhjälte_färg2", "1F9B8-1F3FC"},
    {"superhjälte_färg3", "1F9B8-1F3FD"},
    {"superhjälte_färg4", "1F9B8-1F3FE"},
    {"superhjälte_färg5", "1F9B8-1F3FF"},
    {"varselväst", "1F9BA"},
    {"öra_med_hörapparat", "1F9BB"},
    {"öra_med_hörapparat_färg1", "1F9BB-1F3FB"},
    {"öra_med_hörapparat_färg2", "1F9BB-1F3FC"},
    {"öra_med_hörapparat_färg3", "1F9BB-1F3FD"},
    {"öra_med_hörapparat_färg4", "1F9BB-1F3FE"},
    {"öra_med_hörapparat_färg5", "1F9BB-1F3FF"},
    {"vitlök", "1F9C4"},
    {"lök", "1F9C5"},
    {"våffla", "1F9C7"},
    {"smör", "1F9C8"},
    {"stående_person", "1F9CD"},
    {"stående_person_färg1", "1F9CD-1F3FB"},
    {"stående_person_färg2", "1F9CD-1F3FC"},
    {"stående_person_färg3", "1F9CD-1F3FD"},
    {"stående_person_färg4", "1F9CD-1F3FE"},
    {"stående_person_färg5", "1F9CD-1F3FF"},
    {"knäsittande_person", "1F9CE"},
    {"knäsittande_person_färg1", "1F9CE-1F3FB"},
    {"knäsittande_person_färg2", "1F9CE-1F3FC"},
    {"knäsittande_person_färg3", "1F9CE-1F3FD"},
    {"knäsittande_person_färg4", "1F9CE-1F3FE"},
    {"knäsittande_person_färg5", "1F9CE-1F3FF"},
    {"hörselskadad_person", "1F9CF"},
    {"hörselskadad_person_färg1", "1F9CF-1F3FB"},
    {"hörselskadad_person_färg2", "1F9CF-1F3FC"},
    {"hörselskadad_person_färg3", "1F9CF-1F3FD"},
    {"hörselskadad_person_färg4", "1F9CF-1F3FE"},
    {"hörselskadad_person_färg5", "1F9CF-1F3FF"},
    {"äldre_person", "1F9D3"},
    {"äldre_person_färg1", "1F9D3-1F3FB"},
    {"äldre_person_färg2", "1F9D3-1F3FC"},
    {"äldre_person_färg3", "1F9D3-1F3FD"},
    {"äldre_person_färg4", "1F9D3-1F3FE"},
    {"äldre_person_färg5", "1F9D3-1F3FF"},
    {"man_med_skägg", "1F9D4"},
    {"man_med_skägg_färg1", "1F9D4-1F3FB"},
    {"man_med_skägg_färg2", "1F9D4-1F3FC"},
    {"man_med_skägg_färg3", "1F9D4-1F3FD"},
    {"man_med_skägg_färg4", "1F9D4-1F3FE"},
    {"man_med_skägg_färg5", "1F9D4-1F3FF"},
    {"person_i_ångbastu", "1F9D6"},
    {"person_i_ångbastu_färg1", "1F9D6-1F3FB"},
    {"person_i_ångbastu_färg2", "1F9D6-1F3FC"},
    {"person_i_ångbastu_färg3", "1F9D6-1F3FD"},
    {"person_i_ångbastu_färg4", "1F9D6-1F3FE"},
    {"person_i_ångbastu_färg5", "1F9D6-1F3FF"},
    {"person_som_klättrar", "1F9D7"},
    {"person_som_klättrar_färg1", "1F9D7-1F3FB"},
    {"person_som_klättrar_färg2", "1F9D7-1F3FC"},
    {"person_som_klättrar_färg3", "1F9D7-1F3FD"},
    {"person_som_klättrar_färg4", "1F9D7-1F3FE"},
    {"person_som_klättrar_färg5", "1F9D7-1F3FF"},
    {"fé", "1F9DA"},
    {"fé_färg1", "1F9DA-1F3FB"},
    {"fé_färg2", "1F9DA-1F3FC"},
    {"fé_färg3", "1F9DA-1F3FD"},
    {"fé_färg4", "1F9DA-1F3FE"},
    {"fé_färg5", "1F9DA-1F3FF"},
    {"vattenväsen", "1F9DC"},
    {"vattenväsen_färg1", "1F9DC-1F3FB"},
    {"vattenväsen_färg2", "1F9DC-1F3FC"},
    {"vattenväsen_färg3", "1F9DC-1F3FD"},
    {"vattenväsen_färg4", "1F9DC-1F3FE"},
    {"vattenväsen_färg5", "1F9DC-1F3FF"},
    {"hjärna", "1F9E0"},
    {"hjärta_i_orange", "1F9E1"},
    {"skärmmössa", "1F9E2"},
    {"rött_kuvert", "1F9E7"},
    {"smällare", "1F9E8"},
    {"provrör", "1F9EA"},
    {"petriskål", "1F9EB"},
    {"brandsläckare", "1F9EF"},
    {"verktygslåda", "1F9F0"},
    {"tråd", "1F9F5"},
    {"säkerhetsnål", "1F9F7"},
    {"nallebjörn", "1F9F8"},
    {"tvål", "1F9FC"},
    {"tvättsvamp", "1F9FD"},
    {"onda_ögat_sten", "1F9FF"},
    {"baddräkt", "1FA71"},
    {"ljusblått_hjärta", "1FA75"},
    {"grått_hjärta", "1FA76"},
    {"rosa_hjärta", "1FA77"},
    {"plåster", "1FA79"},
    {"röntgen", "1FA7B"},
    {"fallskärm", "1FA82"},
    {"trollspö", "1FA84"},
    {"piñata", "1FA85"},
    {"flöjt", "1FA88"},
    {"militärhjälm", "1FA96"},
    {"fönster", "1FA9F"},
    {"synål", "1FAA1"},
    {"musfälla", "1FAA4"},
    {"låg_batterinivå", "1FAAB"},
    {"solfjäder", "1FAAD"},
    {"krukväxt", "1FAB4"},
    {"trästockar", "1FAB5"},
    {"fågelfjäder", "1FAB6"},
    {"bo_med_ägg", "1FABA"},
    {"träd_utan_löv", "1FABE"},
    {"gås", "1FABF"},
    {"hjärta", "1FAC0"},
    {"hårig_varelse", "1FAC8"},
    {"späckhuggare", "1FACD"},
    {"älg", "1FACE"},
    {"åsna", "1FACF"},
    {"blåbär", "1FAD0"},
    {"tunnbröd", "1FAD3"},
    {"hälla_vätska", "1FAD7"},
    {"bönor", "1FAD8"},
    {"ingefära", "1FADA"},
    {"ärtskida", "1FADB"},
    {"stänk", "1FADF"},
    {"smältande_ansikte", "1FAE0"},
    {"ansikte_som_gör_honnör", "1FAE1"},
    {"ansikte_med_öppna_ögon_och_hand_för_munnen", "1FAE2"},
    {"ansikte_med_kikande_öga", "1FAE3"},
    {"biter_i_läppen", "1FAE6"},
    {"ansikte_med_påsar_under_ögonen", "1FAE9"},
    {"ansikte_i_närbild", "1FAEA"},
    {"slagsmål", "1FAEF"},
    {"hand_som_pekar_till_höger", "1FAF1"},
    {"hand_som_pekar_till_höger_färg1", "1FAF1-1F3FB"},
    {"hand_som_pekar_till_höger_färg2", "1FAF1-1F3FC"},
    {"hand_som_pekar_till_höger_färg3", "1FAF1-1F3FD"},
    {"hand_som_pekar_till_höger_färg4", "1FAF1-1F3FE"},
    {"hand_som_pekar_till_höger_färg5", "1FAF1-1F3FF"},
    {"hand_som_pekar_till_vänster", "1FAF2"},
    {"hand_som_pekar_till_vänster_färg1", "1FAF2-1F3FB"},
    {"hand_som_pekar_till_vänster_färg2", "1FAF2-1F3FC"},
    {"hand_som_pekar_till_vänster_färg3", "1FAF2-1F3FD"},
    {"hand_som_pekar_till_vänster_färg4", "1FAF2-1F3FE"},
    {"hand_som_pekar_till_vänster_färg5", "1FAF2-1F3FF"},
    {"handflatan_nedåt", "1FAF3"},
    {"handflatan_nedåt_färg1", "1FAF3-1F3FB"},
    {"handflatan_nedåt_färg2", "1FAF3-1F3FC"},
    {"handflatan_nedåt_färg3", "1FAF3-1F3FD"},
    {"handflatan_nedåt_färg4", "1FAF3-1F3FE"},
    {"handflatan_nedåt_färg5", "1FAF3-1F3FF"},
    {"handflatan_uppåt", "1FAF4"},
    {"handflatan_uppåt_färg1", "1FAF4-1F3FB"},
    {"handflatan_uppåt_färg2", "1FAF4-1F3FC"},
    {"handflatan_uppåt_färg3", "1FAF4-1F3FD"},
    {"handflatan_uppåt_färg4", "1FAF4-1F3FE"},
    {"handflatan_uppåt_färg5", "1FAF4-1F3FF"},
    {"finger_som_pekar_på_mig", "1FAF5"},
    {"finger_som_pekar_på_mig_färg1", "1FAF5-1F3FB"},
    {"finger_som_pekar_på_mig_färg2", "1FAF5-1F3FC"},
    {"finger_som_pekar_på_mig_färg3", "1FAF5-1F3FD"},
    {"finger_som_pekar_på_mig_färg4", "1FAF5-1F3FE"},
    {"finger_som_pekar_på_mig_färg5", "1FAF5-1F3FF"},
    {"handhjärta", "1FAF6"},
    {"handhjärta_färg1", "1FAF6-1F3FB"},
    {"handhjärta_färg2", "1FAF6-1F3FC"},
    {"handhjärta_färg3", "1FAF6-1F3FD"},
    {"handhjärta_färg4", "1FAF6-1F3FE"},
    {"handhjärta_färg5", "1FAF6-1F3FF"},
    {"hand_som_knuffar_åt_vänster", "1FAF7"},
    {"hand_som_knuffar_åt_vänster_färg1", "1FAF7-1F3FB"},
    {"hand_som_knuffar_åt_vänster_färg2", "1FAF7-1F3FC"},
    {"hand_som_knuffar_åt_vänster_färg3", "1FAF7-1F3FD"},
    {"hand_som_knuffar_åt_vänster_färg4", "1FAF7-1F3FE"},
    {"hand_som_knuffar_åt_vänster_färg5", "1FAF7-1F3FF"},
    {"hand_som_knuffar_åt_höger", "1FAF8"},
    {"hand_som_knuffar_åt_höger_färg1", "1FAF8-1F3FB"},
    {"hand_som_knuffar_åt_höger_färg2", "1FAF8-1F3FC"},
    {"hand_som_knuffar_åt_höger_färg3", "1FAF8-1F3FD"},
    {"hand_som_knuffar_åt_höger_färg4", "1FAF8-1F3FE"},
    {"hand_som_knuffar_åt_höger_färg5", "1FAF8-1F3FF"},
    {"flagga_förenade_arabemiraten", "1F1E6-1F1EA"},
    {"flagga_österrike", "1F1E6-1F1F9"},
    {"flagga_åland", "1F1E6-1F1FD"},
    {"flagga_st_barthélemy", "1F1E7-1F1F1"},
    {"flagga_karibiska_nederländerna", "1F1E7-1F1F6"},
    {"flagga_bouvetön", "1F1E7-1F1FB"},
    {"flagga_kokosöarna", "1F1E8-1F1E8"},
    {"flagga_cooköarna", "1F1E8-1F1F0"},
    {"flagga_clippertonön", "1F1E8-1F1F5"},
    {"flagga_curaçao", "1F1E8-1F1FC"},
    {"flagga_julön", "1F1E8-1F1FD"},
    {"flagga_västsahara", "1F1EA-1F1ED"},
    {"flagga_falklandsöarna", "1F1EB-1F1F0"},
    {"flagga_färöarna", "1F1EB-1F1F4"},
    {"flagga_grönland", "1F1EC-1F1F1"},
    {"flagga_sydgeorgien_och_sydsandwichöarna", "1F1EC-1F1F8"},
    {"flagga_heardön_och_mcdonaldöarna", "1F1ED-1F1F2"},
    {"flagga_kanarieöarna", "1F1EE-1F1E8"},
    {"flagga_caymanöarna", "1F1F0-1F1FE"},
    {"flagga_marshallöarna", "1F1F2-1F1ED"},
    {"flagga_moçambique", "1F1F2-1F1FF"},
    {"flagga_norfolkön", "1F1F3-1F1EB"},
    {"flagga_nederländerna", "1F1F3-1F1F1"},
    {"flagga_pitcairnöarna", "1F1F5-1F1F3"},
    {"flagga_réunion", "1F1F7-1F1EA"},
    {"flagga_rumänien", "1F1F7-1F1F4"},
    {"flagga_salomonöarna", "1F1F8-1F1E7"},
    {"flagga_são_tomé_och_príncipe", "1F1F8-1F1F9"},
    {"flagga_turks_och_caicosöarna", "1F1F9-1F1E8"},
    {"flagga_tokelauöarna", "1F1F9-1F1F0"},
    {"flagga_östtimor", "1F1F9-1F1F1"},
    {"flagga_usas_yttre_öar", "1F1FA-1F1F2"},
    {"flagga_förenta_nationerna", "1F1FA-1F1F3"},
    {"flagga_brittiska_jungfruöarna", "1F1FB-1F1EC"},
    {"flagga_amerikanska_jungfruöarna", "1F1FB-1F1EE"},
    {"flagga_wallis_och_futunaöarna", "1F1FC-1F1EB"},
    {"par_med_hjärta_man_man", "1F468-200D-2764-FE0F-200D-1F468"},
    {"par_med_hjärta_man_man_färg1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjärta_man_man_färg1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjärta_man_man_färg1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjärta_man_man_färg1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjärta_man_man_färg1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjärta_man_man_färg2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjärta_man_man_färg2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjärta_man_man_färg2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjärta_man_man_färg2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjärta_man_man_färg2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjärta_man_man_färg3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjärta_man_man_färg3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjärta_man_man_färg3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjärta_man_man_färg3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjärta_man_man_färg3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjärta_man_man_färg4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjärta_man_man_färg4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjärta_man_man_färg4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjärta_man_man_färg4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjärta_man_man_färg4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjärta_man_man_färg5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjärta_man_man_färg5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjärta_man_man_färg5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjärta_man_man_färg5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjärta_man_man_färg5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjärta_kvinna_man", "1F469-200D-2764-FE0F-200D-1F468"},
    {"par_med_hjärta_kvinna_man_färg1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjärta_kvinna_man_färg1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjärta_kvinna_man_färg1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjärta_kvinna_man_färg1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjärta_kvinna_man_färg1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjärta_kvinna_man_färg2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjärta_kvinna_man_färg2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjärta_kvinna_man_färg2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjärta_kvinna_man_färg2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjärta_kvinna_man_färg2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjärta_kvinna_man_färg3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjärta_kvinna_man_färg3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjärta_kvinna_man_färg3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjärta_kvinna_man_färg3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjärta_kvinna_man_färg3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjärta_kvinna_man_färg4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjärta_kvinna_man_färg4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjärta_kvinna_man_färg4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjärta_kvinna_man_färg4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjärta_kvinna_man_färg4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjärta_kvinna_man_färg5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"par_med_hjärta_kvinna_man_färg5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"par_med_hjärta_kvinna_man_färg5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"par_med_hjärta_kvinna_man_färg5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"par_med_hjärta_kvinna_man_färg5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"par_med_hjärta_kvinna_kvinna", "1F469-200D-2764-FE0F-200D-1F469"},
    {"par_med_hjärta_kvinna_kvinna_färg1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjärta_kvinna_kvinna_färg1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjärta_kvinna_kvinna_färg1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjärta_kvinna_kvinna_färg1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjärta_kvinna_kvinna_färg1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjärta_kvinna_kvinna_färg2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjärta_kvinna_kvinna_färg2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjärta_kvinna_kvinna_färg2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjärta_kvinna_kvinna_färg2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjärta_kvinna_kvinna_färg2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjärta_kvinna_kvinna_färg3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjärta_kvinna_kvinna_färg3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjärta_kvinna_kvinna_färg3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjärta_kvinna_kvinna_färg3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjärta_kvinna_kvinna_färg3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjärta_kvinna_kvinna_färg4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjärta_kvinna_kvinna_färg4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjärta_kvinna_kvinna_färg4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjärta_kvinna_kvinna_färg4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjärta_kvinna_kvinna_färg4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"par_med_hjärta_kvinna_kvinna_färg5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"par_med_hjärta_kvinna_kvinna_färg5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"par_med_hjärta_kvinna_kvinna_färg5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"par_med_hjärta_kvinna_kvinna_färg5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"par_med_hjärta_kvinna_kvinna_färg5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"personer_som_håller_hand", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"personer_som_håller_hand_färg1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_håller_hand_färg1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_håller_hand_färg1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_håller_hand_färg1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_håller_hand_färg1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_som_håller_hand_färg2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_håller_hand_färg2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_håller_hand_färg2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_håller_hand_färg2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_håller_hand_färg2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_som_håller_hand_färg3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_håller_hand_färg3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_håller_hand_färg3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_håller_hand_färg3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_håller_hand_färg3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_som_håller_hand_färg4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_håller_hand_färg4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_håller_hand_färg4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_håller_hand_färg4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_håller_hand_färg4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"personer_som_håller_hand_färg5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"personer_som_håller_hand_färg5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"personer_som_håller_hand_färg5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"personer_som_håller_hand_färg5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"personer_som_håller_hand_färg5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"person_som_springer_åt_höger", "1F3C3-200D-27A1-FE0F"},
    {"person_som_springer_åt_höger_färg1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"person_som_springer_åt_höger_färg2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"person_som_springer_åt_höger_färg3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"person_som_springer_åt_höger_färg4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"person_som_springer_åt_höger_färg5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"manlig_läkare", "1F468-200D-2695-FE0F"},
    {"manlig_läkare_färg1", "1F468-1F3FB-200D-2695-FE0F"},
    {"manlig_läkare_färg2", "1F468-1F3FC-200D-2695-FE0F"},
    {"manlig_läkare_färg3", "1F468-1F3FD-200D-2695-FE0F"},
    {"manlig_läkare_färg4", "1F468-1F3FE-200D-2695-FE0F"},
    {"manlig_läkare_färg5", "1F468-1F3FF-200D-2695-FE0F"},
    {"man_matar_spädbarn", "1F468-200D-1F37C"},
    {"man_matar_spädbarn_färg1", "1F468-1F3FB-200D-1F37C"},
    {"man_matar_spädbarn_färg2", "1F468-1F3FC-200D-1F37C"},
    {"man_matar_spädbarn_färg3", "1F468-1F3FD-200D-1F37C"},
    {"man_matar_spädbarn_färg4", "1F468-1F3FE-200D-1F37C"},
    {"man_matar_spädbarn_färg5", "1F468-1F3FF-200D-1F37C"},
    {"manlig_sångare", "1F468-200D-1F3A4"},
    {"manlig_sångare_färg1", "1F468-1F3FB-200D-1F3A4"},
    {"manlig_sångare_färg2", "1F468-1F3FC-200D-1F3A4"},
    {"manlig_sångare_färg3", "1F468-1F3FD-200D-1F3A4"},
    {"manlig_sångare_färg4", "1F468-1F3FE-200D-1F3A4"},
    {"manlig_sångare_färg5", "1F468-1F3FF-200D-1F3A4"},
    {"manlig_konstnär", "1F468-200D-1F3A8"},
    {"manlig_konstnär_färg1", "1F468-1F3FB-200D-1F3A8"},
    {"manlig_konstnär_färg2", "1F468-1F3FC-200D-1F3A8"},
    {"manlig_konstnär_färg3", "1F468-1F3FD-200D-1F3A8"},
    {"manlig_konstnär_färg4", "1F468-1F3FE-200D-1F3A8"},
    {"manlig_konstnär_färg5", "1F468-1F3FF-200D-1F3A8"},
    {"manlig_lärare", "1F468-200D-1F3EB"},
    {"manlig_lärare_färg1", "1F468-1F3FB-200D-1F3EB"},
    {"manlig_lärare_färg2", "1F468-1F3FC-200D-1F3EB"},
    {"manlig_lärare_färg3", "1F468-1F3FD-200D-1F3EB"},
    {"manlig_lärare_färg4", "1F468-1F3FE-200D-1F3EB"},
    {"manlig_lärare_färg5", "1F468-1F3FF-200D-1F3EB"},
    {"kontorstjänsteman", "1F468-200D-1F4BC"},
    {"kontorstjänsteman_färg1", "1F468-1F3FB-200D-1F4BC"},
    {"kontorstjänsteman_färg2", "1F468-1F3FC-200D-1F4BC"},
    {"kontorstjänsteman_färg3", "1F468-1F3FD-200D-1F4BC"},
    {"kontorstjänsteman_färg4", "1F468-1F3FE-200D-1F4BC"},
    {"kontorstjänsteman_färg5", "1F468-1F3FF-200D-1F4BC"},
    {"man_med_vit_käpp", "1F468-200D-1F9AF"},
    {"man_med_vit_käpp_färg1", "1F468-1F3FB-200D-1F9AF"},
    {"man_med_vit_käpp_färg2", "1F468-1F3FC-200D-1F9AF"},
    {"man_med_vit_käpp_färg3", "1F468-1F3FD-200D-1F9AF"},
    {"man_med_vit_käpp_färg4", "1F468-1F3FE-200D-1F9AF"},
    {"man_med_vit_käpp_färg5", "1F468-1F3FF-200D-1F9AF"},
    {"man_med_vit_käpp_åt_höger", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"man_med_vit_käpp_åt_höger_färg1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"man_med_vit_käpp_åt_höger_färg2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"man_med_vit_käpp_åt_höger_färg3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"man_med_vit_käpp_åt_höger_färg4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"man_med_vit_käpp_åt_höger_färg5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"man_i_permobil_åt_höger", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"man_i_permobil_åt_höger_färg1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"man_i_permobil_åt_höger_färg2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"man_i_permobil_åt_höger_färg3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"man_i_permobil_åt_höger_färg4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"man_i_permobil_åt_höger_färg5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"man_i_manuell_rullstol_åt_höger", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"man_i_manuell_rullstol_åt_höger_färg1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"man_i_manuell_rullstol_åt_höger_färg2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"man_i_manuell_rullstol_åt_höger_färg3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"man_i_manuell_rullstol_åt_höger_färg4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"man_i_manuell_rullstol_åt_höger_färg5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinnlig_läkare", "1F469-200D-2695-FE0F"},
    {"kvinnlig_läkare_färg1", "1F469-1F3FB-200D-2695-FE0F"},
    {"kvinnlig_läkare_färg2", "1F469-1F3FC-200D-2695-FE0F"},
    {"kvinnlig_läkare_färg3", "1F469-1F3FD-200D-2695-FE0F"},
    {"kvinnlig_läkare_färg4", "1F469-1F3FE-200D-2695-FE0F"},
    {"kvinnlig_läkare_färg5", "1F469-1F3FF-200D-2695-FE0F"},
    {"kvinna_matar_spädbarn", "1F469-200D-1F37C"},
    {"kvinna_matar_spädbarn_färg1", "1F469-1F3FB-200D-1F37C"},
    {"kvinna_matar_spädbarn_färg2", "1F469-1F3FC-200D-1F37C"},
    {"kvinna_matar_spädbarn_färg3", "1F469-1F3FD-200D-1F37C"},
    {"kvinna_matar_spädbarn_färg4", "1F469-1F3FE-200D-1F37C"},
    {"kvinna_matar_spädbarn_färg5", "1F469-1F3FF-200D-1F37C"},
    {"kvinnlig_sångare", "1F469-200D-1F3A4"},
    {"kvinnlig_sångare_färg1", "1F469-1F3FB-200D-1F3A4"},
    {"kvinnlig_sångare_färg2", "1F469-1F3FC-200D-1F3A4"},
    {"kvinnlig_sångare_färg3", "1F469-1F3FD-200D-1F3A4"},
    {"kvinnlig_sångare_färg4", "1F469-1F3FE-200D-1F3A4"},
    {"kvinnlig_sångare_färg5", "1F469-1F3FF-200D-1F3A4"},
    {"kvinnlig_konstnär", "1F469-200D-1F3A8"},
    {"kvinnlig_konstnär_färg1", "1F469-1F3FB-200D-1F3A8"},
    {"kvinnlig_konstnär_färg2", "1F469-1F3FC-200D-1F3A8"},
    {"kvinnlig_konstnär_färg3", "1F469-1F3FD-200D-1F3A8"},
    {"kvinnlig_konstnär_färg4", "1F469-1F3FE-200D-1F3A8"},
    {"kvinnlig_konstnär_färg5", "1F469-1F3FF-200D-1F3A8"},
    {"kvinnlig_lärare", "1F469-200D-1F3EB"},
    {"kvinnlig_lärare_färg1", "1F469-1F3FB-200D-1F3EB"},
    {"kvinnlig_lärare_färg2", "1F469-1F3FC-200D-1F3EB"},
    {"kvinnlig_lärare_färg3", "1F469-1F3FD-200D-1F3EB"},
    {"kvinnlig_lärare_färg4", "1F469-1F3FE-200D-1F3EB"},
    {"kvinnlig_lärare_färg5", "1F469-1F3FF-200D-1F3EB"},
    {"kontorstjänstekvinna", "1F469-200D-1F4BC"},
    {"kontorstjänstekvinna_färg1", "1F469-1F3FB-200D-1F4BC"},
    {"kontorstjänstekvinna_färg2", "1F469-1F3FC-200D-1F4BC"},
    {"kontorstjänstekvinna_färg3", "1F469-1F3FD-200D-1F4BC"},
    {"kontorstjänstekvinna_färg4", "1F469-1F3FE-200D-1F4BC"},
    {"kontorstjänstekvinna_färg5", "1F469-1F3FF-200D-1F4BC"},
    {"kvinna_med_vit_käpp", "1F469-200D-1F9AF"},
    {"kvinna_med_vit_käpp_färg1", "1F469-1F3FB-200D-1F9AF"},
    {"kvinna_med_vit_käpp_färg2", "1F469-1F3FC-200D-1F9AF"},
    {"kvinna_med_vit_käpp_färg3", "1F469-1F3FD-200D-1F9AF"},
    {"kvinna_med_vit_käpp_färg4", "1F469-1F3FE-200D-1F9AF"},
    {"kvinna_med_vit_käpp_färg5", "1F469-1F3FF-200D-1F9AF"},
    {"kvinna_med_vit_käpp_åt_höger", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinna_med_vit_käpp_åt_höger_färg1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinna_med_vit_käpp_åt_höger_färg2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinna_med_vit_käpp_åt_höger_färg3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinna_med_vit_käpp_åt_höger_färg4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinna_med_vit_käpp_åt_höger_färg5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"kvinna_i_permobil_åt_höger", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinna_i_permobil_åt_höger_färg1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinna_i_permobil_åt_höger_färg2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinna_i_permobil_åt_höger_färg3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinna_i_permobil_åt_höger_färg4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinna_i_permobil_åt_höger_färg5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"kvinna_i_manuell_rullstol_åt_höger", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinna_i_manuell_rullstol_åt_höger_färg1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinna_i_manuell_rullstol_åt_höger_färg2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinna_i_manuell_rullstol_åt_höger_färg3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinna_i_manuell_rullstol_åt_höger_färg4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinna_i_manuell_rullstol_åt_höger_färg5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"fotgängare_åt_höger", "1F6B6-200D-27A1-FE0F"},
    {"fotgängare_åt_höger_färg1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"fotgängare_åt_höger_färg2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"fotgängare_åt_höger_färg3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"fotgängare_åt_höger_färg4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"fotgängare_åt_höger_färg5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"knäsittande_person_åt_höger", "1F9CE-200D-27A1-FE0F"},
    {"knäsittande_person_åt_höger_färg1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"knäsittande_person_åt_höger_färg2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"knäsittande_person_åt_höger_färg3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"knäsittande_person_åt_höger_färg4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"knäsittande_person_åt_höger_färg5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"läkare", "1F9D1-200D-2695-FE0F"},
    {"läkare_färg1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"läkare_färg2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"läkare_färg3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"läkare_färg4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"läkare_färg5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"person_ammar_spädbarn", "1F9D1-200D-1F37C"},
    {"person_ammar_spädbarn_färg1", "1F9D1-1F3FB-200D-1F37C"},
    {"person_ammar_spädbarn_färg2", "1F9D1-1F3FC-200D-1F37C"},
    {"person_ammar_spädbarn_färg3", "1F9D1-1F3FD-200D-1F37C"},
    {"person_ammar_spädbarn_färg4", "1F9D1-1F3FE-200D-1F37C"},
    {"person_ammar_spädbarn_färg5", "1F9D1-1F3FF-200D-1F37C"},
    {"könsneutral_tomte", "1F9D1-200D-1F384"},
    {"könsneutral_tomte_färg1", "1F9D1-1F3FB-200D-1F384"},
    {"könsneutral_tomte_färg2", "1F9D1-1F3FC-200D-1F384"},
    {"könsneutral_tomte_färg3", "1F9D1-1F3FD-200D-1F384"},
    {"könsneutral_tomte_färg4", "1F9D1-1F3FE-200D-1F384"},
    {"könsneutral_tomte_färg5", "1F9D1-1F3FF-200D-1F384"},
    {"sångare", "1F9D1-200D-1F3A4"},
    {"sångare_färg1", "1F9D1-1F3FB-200D-1F3A4"},
    {"sångare_färg2", "1F9D1-1F3FC-200D-1F3A4"},
    {"sångare_färg3", "1F9D1-1F3FD-200D-1F3A4"},
    {"sångare_färg4", "1F9D1-1F3FE-200D-1F3A4"},
    {"sångare_färg5", "1F9D1-1F3FF-200D-1F3A4"},
    {"konstnär", "1F9D1-200D-1F3A8"},
    {"konstnär_färg1", "1F9D1-1F3FB-200D-1F3A8"},
    {"konstnär_färg2", "1F9D1-1F3FC-200D-1F3A8"},
    {"konstnär_färg3", "1F9D1-1F3FD-200D-1F3A8"},
    {"konstnär_färg4", "1F9D1-1F3FE-200D-1F3A8"},
    {"konstnär_färg5", "1F9D1-1F3FF-200D-1F3A8"},
    {"lärare", "1F9D1-200D-1F3EB"},
    {"lärare_färg1", "1F9D1-1F3FB-200D-1F3EB"},
    {"lärare_färg2", "1F9D1-1F3FC-200D-1F3EB"},
    {"lärare_färg3", "1F9D1-1F3FD-200D-1F3EB"},
    {"lärare_färg4", "1F9D1-1F3FE-200D-1F3EB"},
    {"lärare_färg5", "1F9D1-1F3FF-200D-1F3EB"},
    {"person_med_vit_käpp", "1F9D1-200D-1F9AF"},
    {"person_med_vit_käpp_färg1", "1F9D1-1F3FB-200D-1F9AF"},
    {"person_med_vit_käpp_färg2", "1F9D1-1F3FC-200D-1F9AF"},
    {"person_med_vit_käpp_färg3", "1F9D1-1F3FD-200D-1F9AF"},
    {"person_med_vit_käpp_färg4", "1F9D1-1F3FE-200D-1F9AF"},
    {"person_med_vit_käpp_färg5", "1F9D1-1F3FF-200D-1F9AF"},
    {"person_med_vit_käpp_åt_höger", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_vit_käpp_åt_höger_färg1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_vit_käpp_åt_höger_färg2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_vit_käpp_åt_höger_färg3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_vit_käpp_åt_höger_färg4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"person_med_vit_käpp_åt_höger_färg5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"person_i_permobil_åt_höger", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_permobil_åt_höger_färg1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_permobil_åt_höger_färg2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_permobil_åt_höger_färg3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_permobil_åt_höger_färg4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_permobil_åt_höger_färg5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"person_i_manuell_rullstol_åt_höger", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullstol_åt_höger_färg1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullstol_åt_höger_färg2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullstol_åt_höger_färg3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullstol_åt_höger_färg4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"person_i_manuell_rullstol_åt_höger_färg5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"kvinna_som_springer_åt_höger", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_springer_åt_höger_färg1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_springer_åt_höger_färg2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_springer_åt_höger_färg3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_springer_åt_höger_färg4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_springer_åt_höger_färg5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"man_som_springer_åt_höger", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_springer_åt_höger_färg1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_springer_åt_höger_färg2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_springer_åt_höger_färg3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_springer_åt_höger_färg4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_springer_åt_höger_färg5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kvinnor_med_kaninöron", "1F46F-200D-2640-FE0F"},
    {"kvinnor_med_kaninöron_färg1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"kvinnor_med_kaninöron_färg1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"kvinnor_med_kaninöron_färg1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"kvinnor_med_kaninöron_färg1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"kvinnor_med_kaninöron_färg2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"kvinnor_med_kaninöron_färg2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"kvinnor_med_kaninöron_färg2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"kvinnor_med_kaninöron_färg2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"kvinnor_med_kaninöron_färg3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"kvinnor_med_kaninöron_färg3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"kvinnor_med_kaninöron_färg3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"kvinnor_med_kaninöron_färg3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"kvinnor_med_kaninöron_färg4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"kvinnor_med_kaninöron_färg4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"kvinnor_med_kaninöron_färg4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"kvinnor_med_kaninöron_färg4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"kvinnor_med_kaninöron_färg5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"kvinnor_med_kaninöron_färg5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"kvinnor_med_kaninöron_färg5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"kvinnor_med_kaninöron_färg5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"kvinnor_med_kaninöron_färg1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"kvinnor_med_kaninöron_färg2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"kvinnor_med_kaninöron_färg3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"kvinnor_med_kaninöron_färg4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"kvinnor_med_kaninöron_färg5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"män_med_kaninöron", "1F46F-200D-2642-FE0F"},
    {"män_med_kaninöron_färg1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"män_med_kaninöron_färg1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"män_med_kaninöron_färg1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"män_med_kaninöron_färg1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"män_med_kaninöron_färg2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"män_med_kaninöron_färg2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"män_med_kaninöron_färg2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"män_med_kaninöron_färg2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"män_med_kaninöron_färg3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"män_med_kaninöron_färg3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"män_med_kaninöron_färg3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"män_med_kaninöron_färg3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"män_med_kaninöron_färg4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"män_med_kaninöron_färg4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"män_med_kaninöron_färg4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"män_med_kaninöron_färg4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"män_med_kaninöron_färg5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"män_med_kaninöron_färg5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"män_med_kaninöron_färg5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"män_med_kaninöron_färg5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"män_med_kaninöron_färg1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"män_med_kaninöron_färg2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"män_med_kaninöron_färg3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"män_med_kaninöron_färg4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"män_med_kaninöron_färg5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"kvinna_i_brudslöja", "1F470-200D-2640-FE0F"},
    {"kvinna_i_brudslöja_färg1", "1F470-1F3FB-200D-2640-FE0F"},
    {"kvinna_i_brudslöja_färg2", "1F470-1F3FC-200D-2640-FE0F"},
    {"kvinna_i_brudslöja_färg3", "1F470-1F3FD-200D-2640-FE0F"},
    {"kvinna_i_brudslöja_färg4", "1F470-1F3FE-200D-2640-FE0F"},
    {"kvinna_i_brudslöja_färg5", "1F470-1F3FF-200D-2640-FE0F"},
    {"man_i_brudslöja", "1F470-200D-2642-FE0F"},
    {"man_i_brudslöja_färg1", "1F470-1F3FB-200D-2642-FE0F"},
    {"man_i_brudslöja_färg2", "1F470-1F3FC-200D-2642-FE0F"},
    {"man_i_brudslöja_färg3", "1F470-1F3FD-200D-2642-FE0F"},
    {"man_i_brudslöja_färg4", "1F470-1F3FE-200D-2642-FE0F"},
    {"man_i_brudslöja_färg5", "1F470-1F3FF-200D-2642-FE0F"},
    {"hjälpande_kvinna", "1F481-200D-2640-FE0F"},
    {"hjälpande_kvinna_färg1", "1F481-1F3FB-200D-2640-FE0F"},
    {"hjälpande_kvinna_färg2", "1F481-1F3FC-200D-2640-FE0F"},
    {"hjälpande_kvinna_färg3", "1F481-1F3FD-200D-2640-FE0F"},
    {"hjälpande_kvinna_färg4", "1F481-1F3FE-200D-2640-FE0F"},
    {"hjälpande_kvinna_färg5", "1F481-1F3FF-200D-2640-FE0F"},
    {"hjälpande_man", "1F481-200D-2642-FE0F"},
    {"hjälpande_man_färg1", "1F481-1F3FB-200D-2642-FE0F"},
    {"hjälpande_man_färg2", "1F481-1F3FC-200D-2642-FE0F"},
    {"hjälpande_man_färg3", "1F481-1F3FD-200D-2642-FE0F"},
    {"hjälpande_man_färg4", "1F481-1F3FE-200D-2642-FE0F"},
    {"hjälpande_man_färg5", "1F481-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_grenadjär", "1F482-200D-2640-FE0F"},
    {"kvinnlig_grenadjär_färg1", "1F482-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_grenadjär_färg2", "1F482-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_grenadjär_färg3", "1F482-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_grenadjär_färg4", "1F482-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_grenadjär_färg5", "1F482-1F3FF-200D-2640-FE0F"},
    {"manlig_grenadjär", "1F482-200D-2642-FE0F"},
    {"manlig_grenadjär_färg1", "1F482-1F3FB-200D-2642-FE0F"},
    {"manlig_grenadjär_färg2", "1F482-1F3FC-200D-2642-FE0F"},
    {"manlig_grenadjär_färg3", "1F482-1F3FD-200D-2642-FE0F"},
    {"manlig_grenadjär_färg4", "1F482-1F3FE-200D-2642-FE0F"},
    {"manlig_grenadjär_färg5", "1F482-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_får_håret_klippt", "1F487-200D-2640-FE0F"},
    {"kvinna_som_får_håret_klippt_färg1", "1F487-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_får_håret_klippt_färg2", "1F487-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_får_håret_klippt_färg3", "1F487-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_får_håret_klippt_färg4", "1F487-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_får_håret_klippt_färg5", "1F487-1F3FF-200D-2640-FE0F"},
    {"man_som_får_håret_klippt", "1F487-200D-2642-FE0F"},
    {"man_som_får_håret_klippt_färg1", "1F487-1F3FB-200D-2642-FE0F"},
    {"man_som_får_håret_klippt_färg2", "1F487-1F3FC-200D-2642-FE0F"},
    {"man_som_får_håret_klippt_färg3", "1F487-1F3FD-200D-2642-FE0F"},
    {"man_som_får_håret_klippt_färg4", "1F487-1F3FE-200D-2642-FE0F"},
    {"man_som_får_håret_klippt_färg5", "1F487-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_räcker_upp_handen", "1F64B-200D-2640-FE0F"},
    {"kvinna_som_räcker_upp_handen_färg1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_räcker_upp_handen_färg2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_räcker_upp_handen_färg3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_räcker_upp_handen_färg4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_räcker_upp_handen_färg5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"man_som_räcker_upp_handen", "1F64B-200D-2642-FE0F"},
    {"man_som_räcker_upp_handen_färg1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"man_som_räcker_upp_handen_färg2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"man_som_räcker_upp_handen_färg3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"man_som_räcker_upp_handen_färg4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"man_som_räcker_upp_handen_färg5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_ror_en_båt", "1F6A3-200D-2640-FE0F"},
    {"kvinna_som_ror_en_båt_färg1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_ror_en_båt_färg2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_ror_en_båt_färg3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_ror_en_båt_färg4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_ror_en_båt_färg5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"man_som_ror_en_båt", "1F6A3-200D-2642-FE0F"},
    {"man_som_ror_en_båt_färg1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"man_som_ror_en_båt_färg2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"man_som_ror_en_båt_färg3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"man_som_ror_en_båt_färg4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"man_som_ror_en_båt_färg5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_cyklar_på_en_mountainbike", "1F6B5-200D-2640-FE0F"},
    {"kvinna_som_cyklar_på_en_mountainbike_färg1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_cyklar_på_en_mountainbike_färg2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_cyklar_på_en_mountainbike_färg3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_cyklar_på_en_mountainbike_färg4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_cyklar_på_en_mountainbike_färg5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"man_som_cyklar_på_en_mountainbike", "1F6B5-200D-2642-FE0F"},
    {"man_som_cyklar_på_en_mountainbike_färg1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"man_som_cyklar_på_en_mountainbike_färg2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"man_som_cyklar_på_en_mountainbike_färg3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"man_som_cyklar_på_en_mountainbike_färg4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"man_som_cyklar_på_en_mountainbike_färg5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_går", "1F6B6-200D-2640-FE0F"},
    {"kvinna_som_går_färg1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_går_färg2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_går_färg3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_går_färg4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_går_färg5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"kvinna_som_går_åt_höger", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_går_åt_höger_färg1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_går_åt_höger_färg2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_går_åt_höger_färg3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_går_åt_höger_färg4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_går_åt_höger_färg5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"man_som_går", "1F6B6-200D-2642-FE0F"},
    {"man_som_går_färg1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"man_som_går_färg2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"man_som_går_färg3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"man_som_går_färg4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"man_som_går_färg5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"man_som_går_åt_höger", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_går_åt_höger_färg1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_går_åt_höger_färg2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_går_åt_höger_färg3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_går_åt_höger_färg4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"man_som_går_åt_höger_färg5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kvinna_som_slår_sig_för_pannan", "1F926-200D-2640-FE0F"},
    {"kvinna_som_slår_sig_för_pannan_färg1", "1F926-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_slår_sig_för_pannan_färg2", "1F926-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_slår_sig_för_pannan_färg3", "1F926-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_slår_sig_för_pannan_färg4", "1F926-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_slår_sig_för_pannan_färg5", "1F926-1F3FF-200D-2640-FE0F"},
    {"man_som_slår_sig_för_pannan", "1F926-200D-2642-FE0F"},
    {"man_som_slår_sig_för_pannan_färg1", "1F926-1F3FB-200D-2642-FE0F"},
    {"man_som_slår_sig_för_pannan_färg2", "1F926-1F3FC-200D-2642-FE0F"},
    {"man_som_slår_sig_för_pannan_färg3", "1F926-1F3FD-200D-2642-FE0F"},
    {"man_som_slår_sig_för_pannan_färg4", "1F926-1F3FE-200D-2642-FE0F"},
    {"man_som_slår_sig_för_pannan_färg5", "1F926-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_rycker_på_axlarna", "1F937-200D-2640-FE0F"},
    {"kvinna_som_rycker_på_axlarna_färg1", "1F937-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_rycker_på_axlarna_färg2", "1F937-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_rycker_på_axlarna_färg3", "1F937-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_rycker_på_axlarna_färg4", "1F937-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_rycker_på_axlarna_färg5", "1F937-1F3FF-200D-2640-FE0F"},
    {"man_som_rycker_på_axlarna", "1F937-200D-2642-FE0F"},
    {"man_som_rycker_på_axlarna_färg1", "1F937-1F3FB-200D-2642-FE0F"},
    {"man_som_rycker_på_axlarna_färg2", "1F937-1F3FC-200D-2642-FE0F"},
    {"man_som_rycker_på_axlarna_färg3", "1F937-1F3FD-200D-2642-FE0F"},
    {"man_som_rycker_på_axlarna_färg4", "1F937-1F3FE-200D-2642-FE0F"},
    {"man_som_rycker_på_axlarna_färg5", "1F937-1F3FF-200D-2642-FE0F"},
    {"brottande_män", "1F93C-200D-2642-FE0F"},
    {"brottande_män_färg1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"brottande_män_färg1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"brottande_män_färg1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"brottande_män_färg1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"brottande_män_färg2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"brottande_män_färg2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"brottande_män_färg2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"brottande_män_färg2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"brottande_män_färg3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"brottande_män_färg3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"brottande_män_färg3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"brottande_män_färg3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"brottande_män_färg4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"brottande_män_färg4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"brottande_män_färg4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"brottande_män_färg4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"brottande_män_färg5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"brottande_män_färg5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"brottande_män_färg5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"brottande_män_färg5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"brottande_män_färg1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"brottande_män_färg2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"brottande_män_färg3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"brottande_män_färg4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"brottande_män_färg5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_superhjälte", "1F9B8-200D-2640-FE0F"},
    {"kvinnlig_superhjälte_färg1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_superhjälte_färg2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_superhjälte_färg3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_superhjälte_färg4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_superhjälte_färg5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"manlig_superhjälte", "1F9B8-200D-2642-FE0F"},
    {"manlig_superhjälte_färg1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"manlig_superhjälte_färg2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"manlig_superhjälte_färg3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"manlig_superhjälte_färg4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"manlig_superhjälte_färg5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"stående_kvinna", "1F9CD-200D-2640-FE0F"},
    {"stående_kvinna_färg1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"stående_kvinna_färg2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"stående_kvinna_färg3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"stående_kvinna_färg4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"stående_kvinna_färg5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"stående_man", "1F9CD-200D-2642-FE0F"},
    {"stående_man_färg1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"stående_man_färg2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"stående_man_färg3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"stående_man_färg4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"stående_man_färg5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"knäsittande_kvinna", "1F9CE-200D-2640-FE0F"},
    {"knäsittande_kvinna_färg1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"knäsittande_kvinna_färg2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"knäsittande_kvinna_färg3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"knäsittande_kvinna_färg4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"knäsittande_kvinna_färg5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"knäsittande_kvinna_åt_höger", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knäsittande_kvinna_åt_höger_färg1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knäsittande_kvinna_åt_höger_färg2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knäsittande_kvinna_åt_höger_färg3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knäsittande_kvinna_åt_höger_färg4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knäsittande_kvinna_åt_höger_färg5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"knäsittande_man", "1F9CE-200D-2642-FE0F"},
    {"knäsittande_man_färg1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"knäsittande_man_färg2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"knäsittande_man_färg3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"knäsittande_man_färg4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"knäsittande_man_färg5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"knäsittande_man_åt_höger", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knäsittande_man_åt_höger_färg1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knäsittande_man_åt_höger_färg2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knäsittande_man_åt_höger_färg3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knäsittande_man_åt_höger_färg4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"knäsittande_man_åt_höger_färg5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"hörselskadad_kvinna", "1F9CF-200D-2640-FE0F"},
    {"hörselskadad_kvinna_färg1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"hörselskadad_kvinna_färg2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"hörselskadad_kvinna_färg3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"hörselskadad_kvinna_färg4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"hörselskadad_kvinna_färg5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"hörselskadad_man", "1F9CF-200D-2642-FE0F"},
    {"hörselskadad_man_färg1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"hörselskadad_man_färg2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"hörselskadad_man_färg3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"hörselskadad_man_färg4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"hörselskadad_man_färg5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"kvinna_skägg", "1F9D4-200D-2640-FE0F"},
    {"kvinna_skägg_färg1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"kvinna_skägg_färg2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"kvinna_skägg_färg3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"kvinna_skägg_färg4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"kvinna_skägg_färg5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"man_skägg", "1F9D4-200D-2642-FE0F"},
    {"man_skägg_färg1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"man_skägg_färg2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"man_skägg_färg3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"man_skägg_färg4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"man_skägg_färg5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"kvinna_i_ångbastu", "1F9D6-200D-2640-FE0F"},
    {"kvinna_i_ångbastu_färg1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"kvinna_i_ångbastu_färg2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"kvinna_i_ångbastu_färg3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"kvinna_i_ångbastu_färg4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"kvinna_i_ångbastu_färg5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"man_i_ångbastu", "1F9D6-200D-2642-FE0F"},
    {"man_i_ångbastu_färg1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"man_i_ångbastu_färg2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"man_i_ångbastu_färg3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"man_i_ångbastu_färg4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"man_i_ångbastu_färg5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"kvinna_som_klättrar", "1F9D7-200D-2640-FE0F"},
    {"kvinna_som_klättrar_färg1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"kvinna_som_klättrar_färg2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"kvinna_som_klättrar_färg3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"kvinna_som_klättrar_färg4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"kvinna_som_klättrar_färg5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"man_som_klättrar", "1F9D7-200D-2642-FE0F"},
    {"man_som_klättrar_färg1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"man_som_klättrar_färg2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"man_som_klättrar_färg3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"man_som_klättrar_färg4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"man_som_klättrar_färg5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"kvinnlig_fé", "1F9DA-200D-2640-FE0F"},
    {"kvinnlig_fé_färg1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"kvinnlig_fé_färg2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"kvinnlig_fé_färg3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"kvinnlig_fé_färg4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"kvinnlig_fé_färg5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"manlig_älva", "1F9DA-200D-2642-FE0F"},
    {"manlig_älva_färg1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"manlig_älva_färg2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"manlig_älva_färg3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"manlig_älva_färg4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"manlig_älva_färg5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"sjöjungfru", "1F9DC-200D-2640-FE0F"},
    {"sjöjungfru_färg1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"sjöjungfru_färg2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"sjöjungfru_färg3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"sjöjungfru_färg4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"sjöjungfru_färg5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"strömkarl", "1F9DC-200D-2642-FE0F"},
    {"strömkarl_färg1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"strömkarl_färg2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"strömkarl_färg3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"strömkarl_färg4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"strömkarl_färg5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"man_rödhårig", "1F468-200D-1F9B0"},
    {"man_rödhårig_färg1", "1F468-1F3FB-200D-1F9B0"},
    {"man_rödhårig_färg2", "1F468-1F3FC-200D-1F9B0"},
    {"man_rödhårig_färg3", "1F468-1F3FD-200D-1F9B0"},
    {"man_rödhårig_färg4", "1F468-1F3FE-200D-1F9B0"},
    {"man_rödhårig_färg5", "1F468-1F3FF-200D-1F9B0"},
    {"man_lockigt_hår", "1F468-200D-1F9B1"},
    {"man_lockigt_hår_färg1", "1F468-1F3FB-200D-1F9B1"},
    {"man_lockigt_hår_färg2", "1F468-1F3FC-200D-1F9B1"},
    {"man_lockigt_hår_färg3", "1F468-1F3FD-200D-1F9B1"},
    {"man_lockigt_hår_färg4", "1F468-1F3FE-200D-1F9B1"},
    {"man_lockigt_hår_färg5", "1F468-1F3FF-200D-1F9B1"},
    {"man_vithårig", "1F468-200D-1F9B3"},
    {"man_vithårig_färg1", "1F468-1F3FB-200D-1F9B3"},
    {"man_vithårig_färg2", "1F468-1F3FC-200D-1F9B3"},
    {"man_vithårig_färg3", "1F468-1F3FD-200D-1F9B3"},
    {"man_vithårig_färg4", "1F468-1F3FE-200D-1F9B3"},
    {"man_vithårig_färg5", "1F468-1F3FF-200D-1F9B3"},
    {"kvinna_rödhårig", "1F469-200D-1F9B0"},
    {"kvinna_rödhårig_färg1", "1F469-1F3FB-200D-1F9B0"},
    {"kvinna_rödhårig_färg2", "1F469-1F3FC-200D-1F9B0"},
    {"kvinna_rödhårig_färg3", "1F469-1F3FD-200D-1F9B0"},
    {"kvinna_rödhårig_färg4", "1F469-1F3FE-200D-1F9B0"},
    {"kvinna_rödhårig_färg5", "1F469-1F3FF-200D-1F9B0"},
    {"kvinna_lockigt_hår", "1F469-200D-1F9B1"},
    {"kvinna_lockigt_hår_färg1", "1F469-1F3FB-200D-1F9B1"},
    {"kvinna_lockigt_hår_färg2", "1F469-1F3FC-200D-1F9B1"},
    {"kvinna_lockigt_hår_färg3", "1F469-1F3FD-200D-1F9B1"},
    {"kvinna_lockigt_hår_färg4", "1F469-1F3FE-200D-1F9B1"},
    {"kvinna_lockigt_hår_färg5", "1F469-1F3FF-200D-1F9B1"},
    {"kvinna_vithårig", "1F469-200D-1F9B3"},
    {"kvinna_vithårig_färg1", "1F469-1F3FB-200D-1F9B3"},
    {"kvinna_vithårig_färg2", "1F469-1F3FC-200D-1F9B3"},
    {"kvinna_vithårig_färg3", "1F469-1F3FD-200D-1F9B3"},
    {"kvinna_vithårig_färg4", "1F469-1F3FE-200D-1F9B3"},
    {"kvinna_vithårig_färg5", "1F469-1F3FF-200D-1F9B3"},
    {"person_rödhårig", "1F9D1-200D-1F9B0"},
    {"person_rödhårig_färg1", "1F9D1-1F3FB-200D-1F9B0"},
    {"person_rödhårig_färg2", "1F9D1-1F3FC-200D-1F9B0"},
    {"person_rödhårig_färg3", "1F9D1-1F3FD-200D-1F9B0"},
    {"person_rödhårig_färg4", "1F9D1-1F3FE-200D-1F9B0"},
    {"person_rödhårig_färg5", "1F9D1-1F3FF-200D-1F9B0"},
    {"person_lockigt_hår", "1F9D1-200D-1F9B1"},
    {"person_lockigt_hår_färg1", "1F9D1-1F3FB-200D-1F9B1"},
    {"person_lockigt_hår_färg2", "1F9D1-1F3FC-200D-1F9B1"},
    {"person_lockigt_hår_färg3", "1F9D1-1F3FD-200D-1F9B1"},
    {"person_lockigt_hår_färg4", "1F9D1-1F3FE-200D-1F9B1"},
    {"person_lockigt_hår_färg5", "1F9D1-1F3FF-200D-1F9B1"},
    {"person_vithårig", "1F9D1-200D-1F9B3"},
    {"person_vithårig_färg1", "1F9D1-1F3FB-200D-1F9B3"},
    {"person_vithårig_färg2", "1F9D1-1F3FC-200D-1F9B3"},
    {"person_vithårig_färg3", "1F9D1-1F3FD-200D-1F9B3"},
    {"person_vithårig_färg4", "1F9D1-1F3FE-200D-1F9B3"},
    {"person_vithårig_färg5", "1F9D1-1F3FF-200D-1F9B3"},
    {"bruten_länk", "26D3-FE0F-200D-1F4A5"},
    {"brinnande_hjärta", "2764-FE0F-200D-1F525"},
    {"läkande_hjärta", "2764-FE0F-200D-1FA79"},
    {"regnbågsflagga", "1F3F3-FE0F-200D-1F308"},
    {"sjörövarflagga", "1F3F4-200D-2620-FE0F"},
    {"isbjörn", "1F43B-200D-2744-FE0F"},
    {"öga_i_pratbubbla", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"ansikte_med_spiralögon", "1F635-200D-1F4AB"},
};

