// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_fi[] = {
    {"huuto_kysymysmerkki", "2049"},
    {"tavaramerkki", "2122"},
    {"info", "2139"},
    {"nuoli_oikealle_ja_vasemmalle", "2194"},
    {"nuoli_ylos_ja_alas", "2195"},
    {"nuoli_ylavasemmalle", "2196"},
    {"nuoli_ylaoikealle", "2197"},
    {"nuoli_alaoikealle", "2198"},
    {"nuoli_alavasemmalle", "2199"},
    {"nappaimisto", "2328"},
    {"aurinko", "2600"},
    {"pilvi", "2601"},
    {"sateenvarjo", "2602"},
    {"lumiukko", "2603"},
    {"komeetta", "2604"},
    {"punainen_valintamerkki", "2611"},
    {"sateenvarjo_ja_pisarat", "2614"},
    {"kuuma_juoma", "2615"},
    {"kolmiapila", "2618"},
    {"paakallo_ja_reisiluut", "2620"},
    {"sateilyvaara", "2622"},
    {"biologinen_vaara", "2623"},
    {"ortodoksiristi", "2626"},
    {"dharma_pyora", "2638"},
    {"surullinen", "2639"},
    {"venus_symboli", "2640"},
    {"mars_symboli", "2642"},
    {"oinas_merkki", "2648"},
    {"harka_merkki", "2649"},
    {"jousimies_merkki", "2650"},
    {"kauris_merkki", "2651"},
    {"vesimies_merkki", "2652"},
    {"kalat_merkki", "2653"},
    {"pata", "2660"},
    {"risti", "2663"},
    {"hertta", "2665"},
    {"ruutu", "2666"},
    {"kuumat_lahteet", "2668"},
    {"hakku_ja_vasara", "2692"},
    {"ankkuri", "2693"},
    {"miekat_ristissa", "2694"},
    {"laaketieteen_symboli", "2695"},
    {"vaaka", "2696"},
    {"tislauskolvi", "2697"},
    {"hammasratas", "2699"},
    {"sakset", "2702"},
    {"valkoinen_valintamerkki", "2705"},
    {"lentokone", "2708"},
    {"kirjekuori", "2709"},
    {"musta_kynankarki", "2712"},
    {"paksu_valintamerkki", "2714"},
    {"kertomerkki", "2716"},
    {"daavidin_tahti", "2721"},
    {"saihke", "2728"},
    {"kahdeksanpiikkinen_tahtimerkki", "2733"},
    {"kahdeksankarkinen_tahti", "2734"},
    {"lumihiutale", "2744"},
    {"kipina", "2747"},
    {"kysymysmerkki", "2753"},
    {"valkoinen_kysymysmerkki", "2754"},
    {"valkoinen_huutomerkki", "2755"},
    {"huutomerkki", "2757"},
    {"sydanhuutomerkki", "2763"},
    {"punainen_sydan", "2764"},
    {"plusmerkki", "2795"},
    {"miinusmerkki", "2796"},
    {"jakomerkki", "2797"},
    {"ylos_kaantyva_nuoli", "2934"},
    {"alas_kaantyva_nuoli", "2935"},
    {"aaltoviiva", "3030"},
    {"onnittelut_kirjoitusmerkki", "3297"},
    {"salainen_kirjoitusmerkki", "3299"},
    {"tekijanoikeusmerkki", "00A9"},
    {"rekisteroity_merkki", "00AE"},
    {"kaksoishuutomerkki", "203C"},
    {"vasemmalle_kaantyva_nuoli", "21A9"},
    {"oikealle_kaantyva_nuoli", "21AA"},
    {"kello", "231A"},
    {"tiimalasi", "231B"},
    {"poista", "23CF"},
    {"kelaa_eteenpain", "23E9"},
    {"kelaa_taaksepain", "23EA"},
    {"nopeasti_ylos", "23EB"},
    {"nopeasti_alas", "23EC"},
    {"seuraava_kappale", "23ED"},
    {"edellinen_kappale", "23EE"},
    {"toista_pysayta", "23EF"},
    {"heratyskello", "23F0"},
    {"sekuntikello", "23F1"},
    {"ajastin", "23F2"},
    {"valuva_tiimalasi", "23F3"},
    {"pysayta", "23F8"},
    {"lopeta", "23F9"},
    {"tallenna", "23FA"},
    {"m_ympyrassa", "24C2"},
    {"pieni_musta_nelio", "25AA"},
    {"pieni_valkoinen_nelio", "25AB"},
    {"toista", "25B6"},
    {"kelaa_hitaasti_taakse", "25C0"},
    {"keskikokoinen_valkoinen_nelio", "25FB"},
    {"keskikokoinen_musta_nelio", "25FC"},
    {"pienehko_valkoinen_nelio", "25FD"},
    {"pienehko_musta_nelio", "25FE"},
    {"puhelin", "260E"},
    {"ylos_osoittava_etusormi", "261D"},
    {"ylos_osoittava_etusormi_vari1", "261D-1F3FB"},
    {"ylos_osoittava_etusormi_vari2", "261D-1F3FC"},
    {"ylos_osoittava_etusormi_vari3", "261D-1F3FD"},
    {"ylos_osoittava_etusormi_vari4", "261D-1F3FE"},
    {"ylos_osoittava_etusormi_vari5", "261D-1F3FF"},
    {"kuunsirppi_ja_tahti", "262A"},
    {"rauhansymboli", "262E"},
    {"jin_ja_jang", "262F"},
    {"hymynaama", "263A"},
    {"kaksonen_merkki", "264A"},
    {"rapu_merkki", "264B"},
    {"leijona_merkki", "264C"},
    {"neitsyt_merkki", "264D"},
    {"vaaka_merkki", "264E"},
    {"skorpioni_merkki", "264F"},
    {"sakkipelin_sotilas", "265F"},
    {"kierratyssymboli", "267B"},
    {"aarettomyys", "267E"},
    {"pyoratuoli", "267F"},
    {"atomi", "269B"},
    {"ranskan_lilja", "269C"},
    {"varoitus", "26A0"},
    {"korkeajannite", "26A1"},
    {"transsymboli", "26A7"},
    {"valkoinen_ympyra", "26AA"},
    {"musta_ympyra", "26AB"},
    {"arkku", "26B0"},
    {"tuhkauurna", "26B1"},
    {"jalkapallo", "26BD"},
    {"pesapallo", "26BE"},
    {"lumiukko_ilman_lunta", "26C4"},
    {"aurinko_pilvessa", "26C5"},
    {"ukkospilvi_ja_sade", "26C8"},
    {"kaarmeenkantaja", "26CE"},
    {"hakku", "26CF"},
    {"kypara_jossa_valkoinen_risti", "26D1"},
    {"ketjut", "26D3"},
    {"kielletty_ajosuunta", "26D4"},
    {"shinto_temppeli", "26E9"},
    {"kirkko", "26EA"},
    {"vuori", "26F0"},
    {"aurinkovarjo", "26F1"},
    {"suihkulahde", "26F2"},
    {"lippu_reian_paalla", "26F3"},
    {"lautta", "26F4"},
    {"purjevene", "26F5"},
    {"hiihtaja", "26F7"},
    {"luistin", "26F8"},
    {"pallottelija", "26F9"},
    {"pallottelija_vari1", "26F9-1F3FB"},
    {"pallottelija_vari2", "26F9-1F3FC"},
    {"pallottelija_vari3", "26F9-1F3FD"},
    {"pallottelija_vari4", "26F9-1F3FE"},
    {"pallottelija_vari5", "26F9-1F3FF"},
    {"teltta", "26FA"},
    {"bensapumppu", "26FD"},
    {"nostettu_nyrkki", "270A"},
    {"nostettu_nyrkki_vari1", "270A-1F3FB"},
    {"nostettu_nyrkki_vari2", "270A-1F3FC"},
    {"nostettu_nyrkki_vari3", "270A-1F3FD"},
    {"nostettu_nyrkki_vari4", "270A-1F3FE"},
    {"nostettu_nyrkki_vari5", "270A-1F3FF"},
    {"nostettu_kasi", "270B"},
    {"nostettu_kasi_vari1", "270B-1F3FB"},
    {"nostettu_kasi_vari2", "270B-1F3FC"},
    {"nostettu_kasi_vari3", "270B-1F3FD"},
    {"nostettu_kasi_vari4", "270B-1F3FE"},
    {"nostettu_kasi_vari5", "270B-1F3FF"},
    {"voitonmerkki", "270C"},
    {"voitonmerkki_vari1", "270C-1F3FB"},
    {"voitonmerkki_vari2", "270C-1F3FC"},
    {"voitonmerkki_vari3", "270C-1F3FD"},
    {"voitonmerkki_vari4", "270C-1F3FE"},
    {"voitonmerkki_vari5", "270C-1F3FF"},
    {"kirjoittava_kasi", "270D"},
    {"kirjoittava_kasi_vari1", "270D-1F3FB"},
    {"kirjoittava_kasi_vari2", "270D-1F3FC"},
    {"kirjoittava_kasi_vari3", "270D-1F3FD"},
    {"kirjoittava_kasi_vari4", "270D-1F3FE"},
    {"kirjoittava_kasi_vari5", "270D-1F3FF"},
    {"lyijykyna", "270F"},
    {"latinalainen_risti", "271D"},
    {"punainen_rastimerkki", "274C"},
    {"rastimerkki", "274E"},
    {"nuoli_oikealle", "27A1"},
    {"silmukka", "27B0"},
    {"kaksoissilmukka", "27BF"},
    {"nuoli_vasemmalle", "2B05"},
    {"nuoli_ylos", "2B06"},
    {"nuoli_alas", "2B07"},
    {"iso_musta_nelio", "2B1B"},
    {"iso_valkoinen_nelio", "2B1C"},
    {"valkoinen_tahti", "2B50"},
    {"suuri_ympyra", "2B55"},
    {"osanvaihtomerkki", "303D"},
    {"mahjong", "1F004"},
    {"jokeri", "1F0CF"},
    {"a_veriryhma", "1F170"},
    {"b_veriryhma", "1F171"},
    {"o_veriryhma", "1F17E"},
    {"p", "1F17F"},
    {"ab_veriryhma", "1F18E"},
    {"pyyhi_nappain", "1F191"},
    {"cool", "1F192"},
    {"free", "1F193"},
    {"id", "1F194"},
    {"new", "1F195"},
    {"ng", "1F196"},
    {"ok", "1F197"},
    {"sos", "1F198"},
    {"up", "1F199"},
    {"vs", "1F19A"},
    {"alueellinen_indikaattori_a", "1F1E6"},
    {"alueellinen_indikaattori_b", "1F1E7"},
    {"alueellinen_indikaattori_c", "1F1E8"},
    {"alueellinen_indikaattori_d", "1F1E9"},
    {"alueellinen_indikaattori_e", "1F1EA"},
    {"alueellinen_indikaattori_f", "1F1EB"},
    {"alueellinen_indikaattori_g", "1F1EC"},
    {"alueellinen_indikaattori_h", "1F1ED"},
    {"alueellinen_indikaattori_i", "1F1EE"},
    {"alueellinen_indikaattori_j", "1F1EF"},
    {"alueellinen_indikaattori_k", "1F1F0"},
    {"alueellinen_indikaattori_l", "1F1F1"},
    {"alueellinen_indikaattori_m", "1F1F2"},
    {"alueellinen_indikaattori_n", "1F1F3"},
    {"alueellinen_indikaattori_o", "1F1F4"},
    {"alueellinen_indikaattori_p", "1F1F5"},
    {"alueellinen_indikaattori_q", "1F1F6"},
    {"alueellinen_indikaattori_r", "1F1F7"},
    {"alueellinen_indikaattori_s", "1F1F8"},
    {"alueellinen_indikaattori_t", "1F1F9"},
    {"alueellinen_indikaattori_u", "1F1FA"},
    {"alueellinen_indikaattori_v", "1F1FB"},
    {"alueellinen_indikaattori_w", "1F1FC"},
    {"alueellinen_indikaattori_x", "1F1FD"},
    {"alueellinen_indikaattori_y", "1F1FE"},
    {"alueellinen_indikaattori_z", "1F1FF"},
    {"taalla_kirjainmerkki", "1F201"},
    {"palvelumaksu_kirjainmerkki", "1F202"},
    {"maksuton_kirjainmerkki", "1F21A"},
    {"varattu_kirjainmerkki", "1F22F"},
    {"kielletty_kirjoitusmerkki", "1F232"},
    {"tyhja_kirjoitusmerkki", "1F233"},
    {"hyvaksytty_arvosana_kirjoitusmerkki", "1F234"},
    {"taynna_kirjoitusmerkki", "1F235"},
    {"maksullinen_kirjainmerkki", "1F236"},
    {"kuukausittainen_maara_kirjainmerkki", "1F237"},
    {"hakemus_kirjoitusmerkki", "1F238"},
    {"alennus_kirjoitusmerkki", "1F239"},
    {"avoinna_kirjoitusmerkki", "1F23A"},
    {"hyva_ostos_kirjainmerkki", "1F250"},
    {"hyvaksyttava_kirjoitusmerkki", "1F251"},
    {"pyorremyrsky", "1F300"},
    {"sumuinen", "1F301"},
    {"suljettu_sateenvarjo", "1F302"},
    {"tahtitaivas", "1F303"},
    {"auringonnousu_vuorten_takaa", "1F304"},
    {"auringonnousu", "1F305"},
    {"kaupunkimaisema_illanhamarassa", "1F306"},
    {"auringonlasku", "1F307"},
    {"sateenkaari", "1F308"},
    {"silta_yolla", "1F309"},
    {"aalto", "1F30A"},
    {"tulivuori", "1F30B"},
    {"linnunrata", "1F30C"},
    {"maapallo_jossa_nakyy_eurooppa_ja_afrikka", "1F30D"},
    {"maapallo_jossa_nakyy_amerikka", "1F30E"},
    {"maapallo_jossa_nakyy_aasia_ja_australia", "1F30F"},
    {"maapallo_jossa_nakyy_pituuspiirit", "1F310"},
    {"uusikuu", "1F311"},
    {"kasvava_kuunsirppi", "1F312"},
    {"ensimmainen_neljanneskuu", "1F313"},
    {"kasvava_puolikuu", "1F314"},
    {"taysikuu", "1F315"},
    {"vaheneva_puolikuu", "1F316"},
    {"viimeinen_neljanneskuu", "1F317"},
    {"vaheneva_kuunsirppi", "1F318"},
    {"kuunsirppi", "1F319"},
    {"uudenkuun_kasvot", "1F31A"},
    {"kasvava_puolikuu_kasvoilla", "1F31B"},
    {"vaheneva_puolikuu_kasvoilla", "1F31C"},
    {"taysikuu_kasvoilla", "1F31D"},
    {"aurinko_kasvoilla", "1F31E"},
    {"loistava_tahti", "1F31F"},
    {"tahdenlento", "1F320"},
    {"lampomittari", "1F321"},
    {"aurinko_ja_pieni_pilvi", "1F324"},
    {"aurinko_ja_suuri_pilvi", "1F325"},
    {"aurinko_sadepilvessa", "1F326"},
    {"sadepilvi", "1F327"},
    {"lumisadepilvi", "1F328"},
    {"ukkospilvi", "1F329"},
    {"hurrikaani", "1F32A"},
    {"sumu", "1F32B"},
    {"puhuri", "1F32C"},
    {"hodari", "1F32D"},
    {"taco", "1F32E"},
    {"burrito", "1F32F"},
    {"kastanja", "1F330"},
    {"taimi", "1F331"},
    {"ikivihrea", "1F332"},
    {"lehtipuu", "1F333"},
    {"palmu", "1F334"},
    {"kaktus", "1F335"},
    {"jalapeno", "1F336"},
    {"tulppaani", "1F337"},
    {"kirsikankukka", "1F338"},
    {"ruusu", "1F339"},
    {"kiinanruusu", "1F33A"},
    {"auringonkukka", "1F33B"},
    {"kukinta", "1F33C"},
    {"maissintahka", "1F33D"},
    {"riisikasvi", "1F33E"},
    {"yrtti", "1F33F"},
    {"neliapila", "1F340"},
    {"vaahteranlehti", "1F341"},
    {"pudonnut_lehti", "1F342"},
    {"lehti_tuulessa", "1F343"},
    {"sieni", "1F344"},
    {"tomaatti", "1F345"},
    {"munakoiso", "1F346"},
    {"viinirypaleet", "1F347"},
    {"meloni", "1F348"},
    {"vesimeloni", "1F349"},
    {"mandariini", "1F34A"},
    {"sitruuna", "1F34B"},
    {"banaani", "1F34C"},
    {"ananas", "1F34D"},
    {"punainen_omena", "1F34E"},
    {"vihrea_omena", "1F34F"},
    {"paaryna", "1F350"},
    {"persikka", "1F351"},
    {"kirsikat", "1F352"},
    {"mansikka", "1F353"},
    {"hampurilainen", "1F354"},
    {"pizza", "1F355"},
    {"lihaa_luineen", "1F356"},
    {"kanankoipi", "1F357"},
    {"riisikakku", "1F358"},
    {"riisipallo", "1F359"},
    {"keitetty_riisi", "1F35A"},
    {"curry_riisi", "1F35B"},
    {"hoyryava_kulho", "1F35C"},
    {"spagetti", "1F35D"},
    {"leipa", "1F35E"},
    {"ranskalaiset", "1F35F"},
    {"paahdettu_bataatti", "1F360"},
    {"dango", "1F361"},
    {"oden", "1F362"},
    {"sushi", "1F363"},
    {"paistettu_katkarapu", "1F364"},
    {"kalakakku", "1F365"},
    {"pehmis", "1F366"},
    {"jaahilejalkiruoka", "1F367"},
    {"jaatelo", "1F368"},
    {"donitsi", "1F369"},
    {"keksi", "1F36A"},
    {"suklaapatukka", "1F36B"},
    {"karkki", "1F36C"},
    {"tikkari", "1F36D"},
    {"vanukas", "1F36E"},
    {"hunaja", "1F36F"},
    {"kakkupala", "1F370"},
    {"bento_rasia", "1F371"},
    {"ruokakulho", "1F372"},
    {"kokkaus", "1F373"},
    {"haarukka_ja_veitsi", "1F374"},
    {"korvaton_teekuppi", "1F375"},
    {"sake", "1F376"},
    {"viinilasi", "1F377"},
    {"cocktaillasi", "1F378"},
    {"trooppinen_drinkki", "1F379"},
    {"oluttuoppi", "1F37A"},
    {"kilisevat_oluttuopit", "1F37B"},
    {"tuttipullo", "1F37C"},
    {"haarukka_veitsi_ja_lautanen", "1F37D"},
    {"pullo_ja_korkin_poksautus", "1F37E"},
    {"popcorn", "1F37F"},
    {"rusetti", "1F380"},
    {"lahjapaketti", "1F381"},
    {"syntymapaivakakku", "1F382"},
    {"kurpitsalyhty", "1F383"},
    {"joulukuusi", "1F384"},
    {"joulupukki", "1F385"},
    {"joulupukki_vari1", "1F385-1F3FB"},
    {"joulupukki_vari2", "1F385-1F3FC"},
    {"joulupukki_vari3", "1F385-1F3FD"},
    {"joulupukki_vari4", "1F385-1F3FE"},
    {"joulupukki_vari5", "1F385-1F3FF"},
    {"ilotulitus", "1F386"},
    {"tahtisadetikku", "1F387"},
    {"ilmapallo", "1F388"},
    {"paukkuserpentiini", "1F389"},
    {"konfettipallo", "1F38A"},
    {"tanabatapuu", "1F38B"},
    {"liput_ristissa", "1F38C"},
    {"puukoriste", "1F38D"},
    {"japanilaiset_nuket", "1F38E"},
    {"karppiviiri", "1F38F"},
    {"tuulikello", "1F390"},
    {"kuuseremonia", "1F391"},
    {"koulureppu", "1F392"},
    {"valmistujaislakki", "1F393"},
    {"kunniamerkki", "1F396"},
    {"tietoisuusnauha", "1F397"},
    {"studiomikrofoni", "1F399"},
    {"liukusaadin", "1F39A"},
    {"saatimet", "1F39B"},
    {"filminauha", "1F39E"},
    {"paasylippu", "1F39F"},
    {"karusellihevonen", "1F3A0"},
    {"maailmanpyora", "1F3A1"},
    {"vuoristorata", "1F3A2"},
    {"onkivapa", "1F3A3"},
    {"mikrofoni", "1F3A4"},
    {"elokuvakamera", "1F3A5"},
    {"videokuva", "1F3A6"},
    {"kuuloke", "1F3A7"},
    {"taiteilijan_paletti", "1F3A8"},
    {"silinteri", "1F3A9"},
    {"sirkusteltta", "1F3AA"},
    {"lippu", "1F3AB"},
    {"klaffitaulu", "1F3AC"},
    {"nayttamotaiteet", "1F3AD"},
    {"videopeli", "1F3AE"},
    {"napakymppi", "1F3AF"},
    {"pelikone", "1F3B0"},
    {"biljardi", "1F3B1"},
    {"noppa", "1F3B2"},
    {"keilailu", "1F3B3"},
    {"kukkapelikortit", "1F3B4"},
    {"nuotti", "1F3B5"},
    {"nuotteja", "1F3B6"},
    {"saksofoni", "1F3B7"},
    {"kitara", "1F3B8"},
    {"koskettimet", "1F3B9"},
    {"trumpetti", "1F3BA"},
    {"viulu", "1F3BB"},
    {"nuottiviivasto", "1F3BC"},
    {"juoksupaita", "1F3BD"},
    {"tennis", "1F3BE"},
    {"sukset", "1F3BF"},
    {"koripallo", "1F3C0"},
    {"ruutulippu", "1F3C1"},
    {"lumilautailija", "1F3C2"},
    {"lumilautailija_vari1", "1F3C2-1F3FB"},
    {"lumilautailija_vari2", "1F3C2-1F3FC"},
    {"lumilautailija_vari3", "1F3C2-1F3FD"},
    {"lumilautailija_vari4", "1F3C2-1F3FE"},
    {"lumilautailija_vari5", "1F3C2-1F3FF"},
    {"juoksija", "1F3C3"},
    {"juoksija_vari1", "1F3C3-1F3FB"},
    {"juoksija_vari2", "1F3C3-1F3FC"},
    {"juoksija_vari3", "1F3C3-1F3FD"},
    {"juoksija_vari4", "1F3C3-1F3FE"},
    {"juoksija_vari5", "1F3C3-1F3FF"},
    {"surffaaja", "1F3C4"},
    {"surffaaja_vari1", "1F3C4-1F3FB"},
    {"surffaaja_vari2", "1F3C4-1F3FC"},
    {"surffaaja_vari3", "1F3C4-1F3FD"},
    {"surffaaja_vari4", "1F3C4-1F3FE"},
    {"surffaaja_vari5", "1F3C4-1F3FF"},
    {"urheilumitali", "1F3C5"},
    {"pokaali", "1F3C6"},
    {"ratsastus", "1F3C7"},
    {"ratsastus_vari1", "1F3C7-1F3FB"},
    {"ratsastus_vari2", "1F3C7-1F3FC"},
    {"ratsastus_vari3", "1F3C7-1F3FD"},
    {"ratsastus_vari4", "1F3C7-1F3FE"},
    {"ratsastus_vari5", "1F3C7-1F3FF"},
    {"amerikkalainen_jalkapallo", "1F3C8"},
    {"rugby_pallo", "1F3C9"},
    {"uimari", "1F3CA"},
    {"uimari_vari1", "1F3CA-1F3FB"},
    {"uimari_vari2", "1F3CA-1F3FC"},
    {"uimari_vari3", "1F3CA-1F3FD"},
    {"uimari_vari4", "1F3CA-1F3FE"},
    {"uimari_vari5", "1F3CA-1F3FF"},
    {"painonnostaja", "1F3CB"},
    {"painonnostaja_vari1", "1F3CB-1F3FB"},
    {"painonnostaja_vari2", "1F3CB-1F3FC"},
    {"painonnostaja_vari3", "1F3CB-1F3FD"},
    {"painonnostaja_vari4", "1F3CB-1F3FE"},
    {"painonnostaja_vari5", "1F3CB-1F3FF"},
    {"golfaaja", "1F3CC"},
    {"golfaaja_vari1", "1F3CC-1F3FB"},
    {"golfaaja_vari2", "1F3CC-1F3FC"},
    {"golfaaja_vari3", "1F3CC-1F3FD"},
    {"golfaaja_vari4", "1F3CC-1F3FE"},
    {"golfaaja_vari5", "1F3CC-1F3FF"},
    {"moottoripyora", "1F3CD"},
    {"kilpa_auto", "1F3CE"},
    {"kriketti", "1F3CF"},
    {"lentopallo", "1F3D0"},
    {"maahockey", "1F3D1"},
    {"jaakiekko_ja_maila", "1F3D2"},
    {"pingis", "1F3D3"},
    {"lumihuippuinen_vuori", "1F3D4"},
    {"telttailu", "1F3D5"},
    {"ranta_ja_aurinkovarjo", "1F3D6"},
    {"rakennustyomaa", "1F3D7"},
    {"taloryhma", "1F3D8"},
    {"kaupunkimaisema", "1F3D9"},
    {"autiotalo", "1F3DA"},
    {"klassinen_rakennus", "1F3DB"},
    {"autiomaa", "1F3DC"},
    {"autiosaari", "1F3DD"},
    {"kansallispuisto", "1F3DE"},
    {"stadion", "1F3DF"},
    {"talo", "1F3E0"},
    {"talo_ja_puutarha", "1F3E1"},
    {"toimistorakennus", "1F3E2"},
    {"japanilainen_postitoimisto", "1F3E3"},
    {"postitoimisto", "1F3E4"},
    {"sairaala", "1F3E5"},
    {"pankki", "1F3E6"},
    {"pankkiautomaatti", "1F3E7"},
    {"hotelli", "1F3E8"},
    {"tuntihotelli", "1F3E9"},
    {"lahikauppa", "1F3EA"},
    {"koulu", "1F3EB"},
    {"tavaratalo", "1F3EC"},
    {"tehdas", "1F3ED"},
    {"punainen_paperilyhty", "1F3EE"},
    {"japanilainen_linna", "1F3EF"},
    {"linna", "1F3F0"},
    {"valkoinen_lippu", "1F3F3"},
    {"musta_lippu", "1F3F4"},
    {"ruusuke", "1F3F5"},
    {"lappu", "1F3F7"},
    {"sulkapallo", "1F3F8"},
    {"jousi_ja_nuoli", "1F3F9"},
    {"amfora", "1F3FA"},
    {"vaalea_iho", "1F3FB"},
    {"keskivaalea_iho", "1F3FC"},
    {"tummanvaalea_iho", "1F3FD"},
    {"keskitumma_iho", "1F3FE"},
    {"tumma_iho", "1F3FF"},
    {"rotta", "1F400"},
    {"hiiri", "1F401"},
    {"harka", "1F402"},
    {"vesipuhveli", "1F403"},
    {"lehma", "1F404"},
    {"tiikeri", "1F405"},
    {"leopardi", "1F406"},
    {"kani", "1F407"},
    {"kissa", "1F408"},
    {"lohikaarme", "1F409"},
    {"krokotiili", "1F40A"},
    {"valas", "1F40B"},
    {"etana", "1F40C"},
    {"kaarme", "1F40D"},
    {"hevonen", "1F40E"},
    {"passi", "1F40F"},
    {"vuohi", "1F410"},
    {"lammas", "1F411"},
    {"apina", "1F412"},
    {"kukko", "1F413"},
    {"kana", "1F414"},
    {"koira", "1F415"},
    {"sika", "1F416"},
    {"villisika", "1F417"},
    {"elefantti", "1F418"},
    {"mustekala", "1F419"},
    {"kotilo", "1F41A"},
    {"otokka", "1F41B"},
    {"muurahainen", "1F41C"},
    {"mehilainen", "1F41D"},
    {"leppakerttu", "1F41E"},
    {"kala", "1F41F"},
    {"trooppinen_kala", "1F420"},
    {"pallokala", "1F421"},
    {"kilpikonna", "1F422"},
    {"kuoriutuva_tipu", "1F423"},
    {"tipu", "1F424"},
    {"tipu_edestapain", "1F425"},
    {"lintu", "1F426"},
    {"pingviini", "1F427"},
    {"koala", "1F428"},
    {"villakoira", "1F429"},
    {"kameli", "1F42A"},
    {"kaksikyttyrainen_kameli", "1F42B"},
    {"delfiini", "1F42C"},
    {"hiiren_naama", "1F42D"},
    {"lehman_naama", "1F42E"},
    {"tiikerin_naama", "1F42F"},
    {"janiksen_naama", "1F430"},
    {"kissan_naama", "1F431"},
    {"lohikaarmeen_naama", "1F432"},
    {"suihkuttava_valas", "1F433"},
    {"hevosen_naama", "1F434"},
    {"apinan_naama", "1F435"},
    {"koiran_naama", "1F436"},
    {"sian_naama", "1F437"},
    {"sammakko", "1F438"},
    {"hamsteri", "1F439"},
    {"susi", "1F43A"},
    {"karhu", "1F43B"},
    {"panda", "1F43C"},
    {"sian_karsa", "1F43D"},
    {"tassunjaljet", "1F43E"},
    {"maaorava", "1F43F"},
    {"silmat", "1F440"},
    {"silma", "1F441"},
    {"korva", "1F442"},
    {"korva_vari1", "1F442-1F3FB"},
    {"korva_vari2", "1F442-1F3FC"},
    {"korva_vari3", "1F442-1F3FD"},
    {"korva_vari4", "1F442-1F3FE"},
    {"korva_vari5", "1F442-1F3FF"},
    {"nena", "1F443"},
    {"nena_vari1", "1F443-1F3FB"},
    {"nena_vari2", "1F443-1F3FC"},
    {"nena_vari3", "1F443-1F3FD"},
    {"nena_vari4", "1F443-1F3FE"},
    {"nena_vari5", "1F443-1F3FF"},
    {"suu", "1F444"},
    {"kieli", "1F445"},
    {"kammenselka_ja_etusormi_ylos", "1F446"},
    {"kammenselka_ja_etusormi_ylos_vari1", "1F446-1F3FB"},
    {"kammenselka_ja_etusormi_ylos_vari2", "1F446-1F3FC"},
    {"kammenselka_ja_etusormi_ylos_vari3", "1F446-1F3FD"},
    {"kammenselka_ja_etusormi_ylos_vari4", "1F446-1F3FE"},
    {"kammenselka_ja_etusormi_ylos_vari5", "1F446-1F3FF"},
    {"kammenselka_ja_etusormi_alas", "1F447"},
    {"kammenselka_ja_etusormi_alas_vari1", "1F447-1F3FB"},
    {"kammenselka_ja_etusormi_alas_vari2", "1F447-1F3FC"},
    {"kammenselka_ja_etusormi_alas_vari3", "1F447-1F3FD"},
    {"kammenselka_ja_etusormi_alas_vari4", "1F447-1F3FE"},
    {"kammenselka_ja_etusormi_alas_vari5", "1F447-1F3FF"},
    {"kammenselka_ja_etusormi_vasemmalle", "1F448"},
    {"kammenselka_ja_etusormi_vasemmalle_vari1", "1F448-1F3FB"},
    {"kammenselka_ja_etusormi_vasemmalle_vari2", "1F448-1F3FC"},
    {"kammenselka_ja_etusormi_vasemmalle_vari3", "1F448-1F3FD"},
    {"kammenselka_ja_etusormi_vasemmalle_vari4", "1F448-1F3FE"},
    {"kammenselka_ja_etusormi_vasemmalle_vari5", "1F448-1F3FF"},
    {"kammenselka_ja_etusormi_oikealle", "1F449"},
    {"kammenselka_ja_etusormi_oikealle_vari1", "1F449-1F3FB"},
    {"kammenselka_ja_etusormi_oikealle_vari2", "1F449-1F3FC"},
    {"kammenselka_ja_etusormi_oikealle_vari3", "1F449-1F3FD"},
    {"kammenselka_ja_etusormi_oikealle_vari4", "1F449-1F3FE"},
    {"kammenselka_ja_etusormi_oikealle_vari5", "1F449-1F3FF"},
    {"kohti_tuleva_nyrkki", "1F44A"},
    {"kohti_tuleva_nyrkki_vari1", "1F44A-1F3FB"},
    {"kohti_tuleva_nyrkki_vari2", "1F44A-1F3FC"},
    {"kohti_tuleva_nyrkki_vari3", "1F44A-1F3FD"},
    {"kohti_tuleva_nyrkki_vari4", "1F44A-1F3FE"},
    {"kohti_tuleva_nyrkki_vari5", "1F44A-1F3FF"},
    {"kaden_heilautus", "1F44B"},
    {"kaden_heilautus_vari1", "1F44B-1F3FB"},
    {"kaden_heilautus_vari2", "1F44B-1F3FC"},
    {"kaden_heilautus_vari3", "1F44B-1F3FD"},
    {"kaden_heilautus_vari4", "1F44B-1F3FE"},
    {"kaden_heilautus_vari5", "1F44B-1F3FF"},
    {"ok_sormiele", "1F44C"},
    {"ok_sormiele_vari1", "1F44C-1F3FB"},
    {"ok_sormiele_vari2", "1F44C-1F3FC"},
    {"ok_sormiele_vari3", "1F44C-1F3FD"},
    {"ok_sormiele_vari4", "1F44C-1F3FE"},
    {"ok_sormiele_vari5", "1F44C-1F3FF"},
    {"peukalo_ylos", "1F44D"},
    {"peukalo_ylos_vari1", "1F44D-1F3FB"},
    {"peukalo_ylos_vari2", "1F44D-1F3FC"},
    {"peukalo_ylos_vari3", "1F44D-1F3FD"},
    {"peukalo_ylos_vari4", "1F44D-1F3FE"},
    {"peukalo_ylos_vari5", "1F44D-1F3FF"},
    {"peukalo_alas", "1F44E"},
    {"peukalo_alas_vari1", "1F44E-1F3FB"},
    {"peukalo_alas_vari2", "1F44E-1F3FC"},
    {"peukalo_alas_vari3", "1F44E-1F3FD"},
    {"peukalo_alas_vari4", "1F44E-1F3FE"},
    {"peukalo_alas_vari5", "1F44E-1F3FF"},
    {"kasien_taputus", "1F44F"},
    {"kasien_taputus_vari1", "1F44F-1F3FB"},
    {"kasien_taputus_vari2", "1F44F-1F3FC"},
    {"kasien_taputus_vari3", "1F44F-1F3FD"},
    {"kasien_taputus_vari4", "1F44F-1F3FE"},
    {"kasien_taputus_vari5", "1F44F-1F3FF"},
    {"avoimet_kadet", "1F450"},
    {"avoimet_kadet_vari1", "1F450-1F3FB"},
    {"avoimet_kadet_vari2", "1F450-1F3FC"},
    {"avoimet_kadet_vari3", "1F450-1F3FD"},
    {"avoimet_kadet_vari4", "1F450-1F3FE"},
    {"avoimet_kadet_vari5", "1F450-1F3FF"},
    {"kruunu", "1F451"},
    {"naisen_hattu", "1F452"},
    {"silmalasit", "1F453"},
    {"solmio", "1F454"},
    {"t_paita", "1F455"},
    {"farkut", "1F456"},
    {"mekko", "1F457"},
    {"kimono", "1F458"},
    {"bikinit", "1F459"},
    {"naistenvaatteet", "1F45A"},
    {"rahakukkaro", "1F45B"},
    {"kasilaukku", "1F45C"},
    {"kirjekuorilaukku", "1F45D"},
    {"miehen_kenka", "1F45E"},
    {"juoksukenka", "1F45F"},
    {"korkokenka", "1F460"},
    {"naisen_sandaali", "1F461"},
    {"naisen_saapas", "1F462"},
    {"jalanjaljet", "1F463"},
    {"rintasiluetti", "1F464"},
    {"kaksi_rintasiluettia", "1F465"},
    {"poika", "1F466"},
    {"poika_vari1", "1F466-1F3FB"},
    {"poika_vari2", "1F466-1F3FC"},
    {"poika_vari3", "1F466-1F3FD"},
    {"poika_vari4", "1F466-1F3FE"},
    {"poika_vari5", "1F466-1F3FF"},
    {"tytto", "1F467"},
    {"tytto_vari1", "1F467-1F3FB"},
    {"tytto_vari2", "1F467-1F3FC"},
    {"tytto_vari3", "1F467-1F3FD"},
    {"tytto_vari4", "1F467-1F3FE"},
    {"tytto_vari5", "1F467-1F3FF"},
    {"mies", "1F468"},
    {"mies_vari1", "1F468-1F3FB"},
    {"mies_vari2", "1F468-1F3FC"},
    {"mies_vari3", "1F468-1F3FD"},
    {"mies_vari4", "1F468-1F3FE"},
    {"mies_vari5", "1F468-1F3FF"},
    {"nainen", "1F469"},
    {"nainen_vari1", "1F469-1F3FB"},
    {"nainen_vari2", "1F469-1F3FC"},
    {"nainen_vari3", "1F469-1F3FD"},
    {"nainen_vari4", "1F469-1F3FE"},
    {"nainen_vari5", "1F469-1F3FF"},
    {"perhe", "1F46A"},
    {"mies_ja_nainen_kasi_kadessa", "1F46B"},
    {"mies_ja_nainen_kasi_kadessa_vari1", "1F46B-1F3FB"},
    {"mies_ja_nainen_kasi_kadessa_vari2", "1F46B-1F3FC"},
    {"mies_ja_nainen_kasi_kadessa_vari3", "1F46B-1F3FD"},
    {"mies_ja_nainen_kasi_kadessa_vari4", "1F46B-1F3FE"},
    {"mies_ja_nainen_kasi_kadessa_vari5", "1F46B-1F3FF"},
    {"mies_ja_nainen_kasi_kadessa_vari1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mies_ja_nainen_kasi_kadessa_vari1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mies_ja_nainen_kasi_kadessa_vari1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mies_ja_nainen_kasi_kadessa_vari1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mies_ja_nainen_kasi_kadessa_vari2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mies_ja_nainen_kasi_kadessa_vari2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mies_ja_nainen_kasi_kadessa_vari2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mies_ja_nainen_kasi_kadessa_vari2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mies_ja_nainen_kasi_kadessa_vari3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mies_ja_nainen_kasi_kadessa_vari3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mies_ja_nainen_kasi_kadessa_vari3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mies_ja_nainen_kasi_kadessa_vari3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mies_ja_nainen_kasi_kadessa_vari4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mies_ja_nainen_kasi_kadessa_vari4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mies_ja_nainen_kasi_kadessa_vari4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mies_ja_nainen_kasi_kadessa_vari4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mies_ja_nainen_kasi_kadessa_vari5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mies_ja_nainen_kasi_kadessa_vari5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mies_ja_nainen_kasi_kadessa_vari5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mies_ja_nainen_kasi_kadessa_vari5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"kaksi_miesta_kasi_kadessa", "1F46C"},
    {"kaksi_miesta_kasi_kadessa_vari1", "1F46C-1F3FB"},
    {"kaksi_miesta_kasi_kadessa_vari2", "1F46C-1F3FC"},
    {"kaksi_miesta_kasi_kadessa_vari3", "1F46C-1F3FD"},
    {"kaksi_miesta_kasi_kadessa_vari4", "1F46C-1F3FE"},
    {"kaksi_miesta_kasi_kadessa_vari5", "1F46C-1F3FF"},
    {"kaksi_miesta_kasi_kadessa_vari1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"kaksi_miesta_kasi_kadessa_vari1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"kaksi_miesta_kasi_kadessa_vari1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"kaksi_miesta_kasi_kadessa_vari1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"kaksi_miesta_kasi_kadessa_vari2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"kaksi_miesta_kasi_kadessa_vari2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"kaksi_miesta_kasi_kadessa_vari2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"kaksi_miesta_kasi_kadessa_vari2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"kaksi_miesta_kasi_kadessa_vari3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"kaksi_miesta_kasi_kadessa_vari3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"kaksi_miesta_kasi_kadessa_vari3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"kaksi_miesta_kasi_kadessa_vari3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"kaksi_miesta_kasi_kadessa_vari4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"kaksi_miesta_kasi_kadessa_vari4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"kaksi_miesta_kasi_kadessa_vari4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"kaksi_miesta_kasi_kadessa_vari4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"kaksi_miesta_kasi_kadessa_vari5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"kaksi_miesta_kasi_kadessa_vari5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"kaksi_miesta_kasi_kadessa_vari5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"kaksi_miesta_kasi_kadessa_vari5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"kaksi_naista_kasi_kadessa", "1F46D"},
    {"kaksi_naista_kasi_kadessa_vari1", "1F46D-1F3FB"},
    {"kaksi_naista_kasi_kadessa_vari2", "1F46D-1F3FC"},
    {"kaksi_naista_kasi_kadessa_vari3", "1F46D-1F3FD"},
    {"kaksi_naista_kasi_kadessa_vari4", "1F46D-1F3FE"},
    {"kaksi_naista_kasi_kadessa_vari5", "1F46D-1F3FF"},
    {"kaksi_naista_kasi_kadessa_vari1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"kaksi_naista_kasi_kadessa_vari1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"kaksi_naista_kasi_kadessa_vari1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"kaksi_naista_kasi_kadessa_vari1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"kaksi_naista_kasi_kadessa_vari2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"kaksi_naista_kasi_kadessa_vari2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"kaksi_naista_kasi_kadessa_vari2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"kaksi_naista_kasi_kadessa_vari2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"kaksi_naista_kasi_kadessa_vari3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"kaksi_naista_kasi_kadessa_vari3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"kaksi_naista_kasi_kadessa_vari3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"kaksi_naista_kasi_kadessa_vari3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"kaksi_naista_kasi_kadessa_vari4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"kaksi_naista_kasi_kadessa_vari4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"kaksi_naista_kasi_kadessa_vari4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"kaksi_naista_kasi_kadessa_vari4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"kaksi_naista_kasi_kadessa_vari5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"kaksi_naista_kasi_kadessa_vari5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"kaksi_naista_kasi_kadessa_vari5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"kaksi_naista_kasi_kadessa_vari5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"poliisi", "1F46E"},
    {"poliisi_vari1", "1F46E-1F3FB"},
    {"poliisi_vari2", "1F46E-1F3FC"},
    {"poliisi_vari3", "1F46E-1F3FD"},
    {"poliisi_vari4", "1F46E-1F3FE"},
    {"poliisi_vari5", "1F46E-1F3FF"},
    {"puputytot", "1F46F"},
    {"puputytot_vari1", "1F46F-1F3FB"},
    {"puputytot_vari2", "1F46F-1F3FC"},
    {"puputytot_vari3", "1F46F-1F3FD"},
    {"puputytot_vari4", "1F46F-1F3FE"},
    {"puputytot_vari5", "1F46F-1F3FF"},
    {"puputytot_vari1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"puputytot_vari1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"puputytot_vari1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"puputytot_vari1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"puputytot_vari2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"puputytot_vari2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"puputytot_vari2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"puputytot_vari2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"puputytot_vari3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"puputytot_vari3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"puputytot_vari3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"puputytot_vari3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"puputytot_vari4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"puputytot_vari4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"puputytot_vari4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"puputytot_vari4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"puputytot_vari5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"puputytot_vari5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"puputytot_vari5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"puputytot_vari5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"huntupainen_henkilo", "1F470"},
    {"huntupainen_henkilo_vari1", "1F470-1F3FB"},
    {"huntupainen_henkilo_vari2", "1F470-1F3FC"},
    {"huntupainen_henkilo_vari3", "1F470-1F3FD"},
    {"huntupainen_henkilo_vari4", "1F470-1F3FE"},
    {"huntupainen_henkilo_vari5", "1F470-1F3FF"},
    {"vaaleatukkainen_henkilo", "1F471"},
    {"vaaleatukkainen_henkilo_vari1", "1F471-1F3FB"},
    {"vaaleatukkainen_henkilo_vari2", "1F471-1F3FC"},
    {"vaaleatukkainen_henkilo_vari3", "1F471-1F3FD"},
    {"vaaleatukkainen_henkilo_vari4", "1F471-1F3FE"},
    {"vaaleatukkainen_henkilo_vari5", "1F471-1F3FF"},
    {"mies_ja_kiinalainen_hattu", "1F472"},
    {"mies_ja_kiinalainen_hattu_vari1", "1F472-1F3FB"},
    {"mies_ja_kiinalainen_hattu_vari2", "1F472-1F3FC"},
    {"mies_ja_kiinalainen_hattu_vari3", "1F472-1F3FD"},
    {"mies_ja_kiinalainen_hattu_vari4", "1F472-1F3FE"},
    {"mies_ja_kiinalainen_hattu_vari5", "1F472-1F3FF"},
    {"turbaanipainen_henkilo", "1F473"},
    {"turbaanipainen_henkilo_vari1", "1F473-1F3FB"},
    {"turbaanipainen_henkilo_vari2", "1F473-1F3FC"},
    {"turbaanipainen_henkilo_vari3", "1F473-1F3FD"},
    {"turbaanipainen_henkilo_vari4", "1F473-1F3FE"},
    {"turbaanipainen_henkilo_vari5", "1F473-1F3FF"},
    {"vanha_mies", "1F474"},
    {"vanha_mies_vari1", "1F474-1F3FB"},
    {"vanha_mies_vari2", "1F474-1F3FC"},
    {"vanha_mies_vari3", "1F474-1F3FD"},
    {"vanha_mies_vari4", "1F474-1F3FE"},
    {"vanha_mies_vari5", "1F474-1F3FF"},
    {"vanha_nainen", "1F475"},
    {"vanha_nainen_vari1", "1F475-1F3FB"},
    {"vanha_nainen_vari2", "1F475-1F3FC"},
    {"vanha_nainen_vari3", "1F475-1F3FD"},
    {"vanha_nainen_vari4", "1F475-1F3FE"},
    {"vanha_nainen_vari5", "1F475-1F3FF"},
    {"vauva", "1F476"},
    {"vauva_vari1", "1F476-1F3FB"},
    {"vauva_vari2", "1F476-1F3FC"},
    {"vauva_vari3", "1F476-1F3FD"},
    {"vauva_vari4", "1F476-1F3FE"},
    {"vauva_vari5", "1F476-1F3FF"},
    {"rakennustyolainen", "1F477"},
    {"rakennustyolainen_vari1", "1F477-1F3FB"},
    {"rakennustyolainen_vari2", "1F477-1F3FC"},
    {"rakennustyolainen_vari3", "1F477-1F3FD"},
    {"rakennustyolainen_vari4", "1F477-1F3FE"},
    {"rakennustyolainen_vari5", "1F477-1F3FF"},
    {"prinsessa", "1F478"},
    {"prinsessa_vari1", "1F478-1F3FB"},
    {"prinsessa_vari2", "1F478-1F3FC"},
    {"prinsessa_vari3", "1F478-1F3FD"},
    {"prinsessa_vari4", "1F478-1F3FE"},
    {"prinsessa_vari5", "1F478-1F3FF"},
    {"peikko", "1F479"},
    {"hiisi", "1F47A"},
    {"kummitus", "1F47B"},
    {"vauvaenkeli", "1F47C"},
    {"vauvaenkeli_vari1", "1F47C-1F3FB"},
    {"vauvaenkeli_vari2", "1F47C-1F3FC"},
    {"vauvaenkeli_vari3", "1F47C-1F3FD"},
    {"vauvaenkeli_vari4", "1F47C-1F3FE"},
    {"vauvaenkeli_vari5", "1F47C-1F3FF"},
    {"avaruusolio", "1F47D"},
    {"avaruushirvio", "1F47E"},
    {"pikkupaholainen", "1F47F"},
    {"paakallo", "1F480"},
    {"kammenta_ylospain_nayttava_henkilo", "1F481"},
    {"kammenta_ylospain_nayttava_henkilo_vari1", "1F481-1F3FB"},
    {"kammenta_ylospain_nayttava_henkilo_vari2", "1F481-1F3FC"},
    {"kammenta_ylospain_nayttava_henkilo_vari3", "1F481-1F3FD"},
    {"kammenta_ylospain_nayttava_henkilo_vari4", "1F481-1F3FE"},
    {"kammenta_ylospain_nayttava_henkilo_vari5", "1F481-1F3FF"},
    {"vartija", "1F482"},
    {"vartija_vari1", "1F482-1F3FB"},
    {"vartija_vari2", "1F482-1F3FC"},
    {"vartija_vari3", "1F482-1F3FD"},
    {"vartija_vari4", "1F482-1F3FE"},
    {"vartija_vari5", "1F482-1F3FF"},
    {"tanssiva_nainen", "1F483"},
    {"tanssiva_nainen_vari1", "1F483-1F3FB"},
    {"tanssiva_nainen_vari2", "1F483-1F3FC"},
    {"tanssiva_nainen_vari3", "1F483-1F3FD"},
    {"tanssiva_nainen_vari4", "1F483-1F3FE"},
    {"tanssiva_nainen_vari5", "1F483-1F3FF"},
    {"huulipuna", "1F484"},
    {"kynsilakka", "1F485"},
    {"kynsilakka_vari1", "1F485-1F3FB"},
    {"kynsilakka_vari2", "1F485-1F3FC"},
    {"kynsilakka_vari3", "1F485-1F3FD"},
    {"kynsilakka_vari4", "1F485-1F3FE"},
    {"kynsilakka_vari5", "1F485-1F3FF"},
    {"hieronta", "1F486"},
    {"hieronta_vari1", "1F486-1F3FB"},
    {"hieronta_vari2", "1F486-1F3FC"},
    {"hieronta_vari3", "1F486-1F3FD"},
    {"hieronta_vari4", "1F486-1F3FE"},
    {"hieronta_vari5", "1F486-1F3FF"},
    {"hiustenleikkaus", "1F487"},
    {"hiustenleikkaus_vari1", "1F487-1F3FB"},
    {"hiustenleikkaus_vari2", "1F487-1F3FC"},
    {"hiustenleikkaus_vari3", "1F487-1F3FD"},
    {"hiustenleikkaus_vari4", "1F487-1F3FE"},
    {"hiustenleikkaus_vari5", "1F487-1F3FF"},
    {"punasiniraitainen_pylvas", "1F488"},
    {"ruisku", "1F489"},
    {"pilleri", "1F48A"},
    {"huulipunajalki", "1F48B"},
    {"rakkauskirje", "1F48C"},
    {"sormus", "1F48D"},
    {"jalokivi", "1F48E"},
    {"pusu", "1F48F"},
    {"pusu_vari1", "1F48F-1F3FB"},
    {"pusu_vari2", "1F48F-1F3FC"},
    {"pusu_vari3", "1F48F-1F3FD"},
    {"pusu_vari4", "1F48F-1F3FE"},
    {"pusu_vari5", "1F48F-1F3FF"},
    {"pusu_vari1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"pusu_vari1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"pusu_vari1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"pusu_vari1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"pusu_vari2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"pusu_vari2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"pusu_vari2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"pusu_vari2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"pusu_vari3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"pusu_vari3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"pusu_vari3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"pusu_vari3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"pusu_vari4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"pusu_vari4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"pusu_vari4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"pusu_vari4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"pusu_vari5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"pusu_vari5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"pusu_vari5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"pusu_vari5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"kukkakimppu", "1F490"},
    {"pariskunta_ja_sydan", "1F491"},
    {"pariskunta_ja_sydan_vari1", "1F491-1F3FB"},
    {"pariskunta_ja_sydan_vari2", "1F491-1F3FC"},
    {"pariskunta_ja_sydan_vari3", "1F491-1F3FD"},
    {"pariskunta_ja_sydan_vari4", "1F491-1F3FE"},
    {"pariskunta_ja_sydan_vari5", "1F491-1F3FF"},
    {"pariskunta_ja_sydan_vari1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pariskunta_ja_sydan_vari1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pariskunta_ja_sydan_vari1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pariskunta_ja_sydan_vari1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pariskunta_ja_sydan_vari2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pariskunta_ja_sydan_vari2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pariskunta_ja_sydan_vari2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pariskunta_ja_sydan_vari2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pariskunta_ja_sydan_vari3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pariskunta_ja_sydan_vari3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pariskunta_ja_sydan_vari3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pariskunta_ja_sydan_vari3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pariskunta_ja_sydan_vari4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pariskunta_ja_sydan_vari4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pariskunta_ja_sydan_vari4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pariskunta_ja_sydan_vari4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pariskunta_ja_sydan_vari5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pariskunta_ja_sydan_vari5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pariskunta_ja_sydan_vari5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pariskunta_ja_sydan_vari5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"haat", "1F492"},
    {"sykkiva_sydan", "1F493"},
    {"sarkynyt_sydan", "1F494"},
    {"kaksi_sydanta", "1F495"},
    {"kimaltava_sydan", "1F496"},
    {"kasvava_sydan", "1F497"},
    {"sydan_ja_nuoli", "1F498"},
    {"sininen_sydan", "1F499"},
    {"vihrea_sydan", "1F49A"},
    {"keltainen_sydan", "1F49B"},
    {"violetti_sydan", "1F49C"},
    {"sydan_ja_lahjanauha", "1F49D"},
    {"toisiaan_kiertavat_sydamet", "1F49E"},
    {"sydankoriste", "1F49F"},
    {"vinonelio_kukka", "1F4A0"},
    {"hehkulamppu", "1F4A1"},
    {"viha", "1F4A2"},
    {"pommi", "1F4A3"},
    {"zzz", "1F4A4"},
    {"tormays", "1F4A5"},
    {"hikipisarat", "1F4A6"},
    {"pisara", "1F4A7"},
    {"pikalahto", "1F4A8"},
    {"kakkakasa", "1F4A9"},
    {"pullistettu_hauis", "1F4AA"},
    {"pullistettu_hauis_vari1", "1F4AA-1F3FB"},
    {"pullistettu_hauis_vari2", "1F4AA-1F3FC"},
    {"pullistettu_hauis_vari3", "1F4AA-1F3FD"},
    {"pullistettu_hauis_vari4", "1F4AA-1F3FE"},
    {"pullistettu_hauis_vari5", "1F4AA-1F3FF"},
    {"pyorryttava", "1F4AB"},
    {"puhekupla", "1F4AC"},
    {"ajatuskupla", "1F4AD"},
    {"valkoinen_kukka", "1F4AE"},
    {"sata_pistetta", "1F4AF"},
    {"rahasakki", "1F4B0"},
    {"valuutanvaihto", "1F4B1"},
    {"paksu_dollarimerkki", "1F4B2"},
    {"luottokortti", "1F4B3"},
    {"yen_seteli", "1F4B4"},
    {"dollariseteli", "1F4B5"},
    {"euroseteli", "1F4B6"},
    {"puntaseteli", "1F4B7"},
    {"rahalla_on_siivet", "1F4B8"},
    {"yen_ja_nouseva_kayra", "1F4B9"},
    {"istuin", "1F4BA"},
    {"kannettava_tietokone", "1F4BB"},
    {"salkku", "1F4BC"},
    {"minidisk", "1F4BD"},
    {"disketti", "1F4BE"},
    {"cd", "1F4BF"},
    {"dvd", "1F4C0"},
    {"tiedostokansio", "1F4C1"},
    {"avoin_tiedostokansio", "1F4C2"},
    {"kaannetty_sivu", "1F4C3"},
    {"sivu", "1F4C4"},
    {"kalenteri", "1F4C5"},
    {"paivakalenteri", "1F4C6"},
    {"kortisto", "1F4C7"},
    {"nouseva_kayra", "1F4C8"},
    {"laskeva_kayra", "1F4C9"},
    {"pylvaskaavio", "1F4CA"},
    {"leikepoyta", "1F4CB"},
    {"nuppineula", "1F4CC"},
    {"pyorea_nasta", "1F4CD"},
    {"paperiliitin", "1F4CE"},
    {"viivoitin", "1F4CF"},
    {"kolmioviivain", "1F4D0"},
    {"kirjanmerkkivalilehdet", "1F4D1"},
    {"tilikirja", "1F4D2"},
    {"muistikirja", "1F4D3"},
    {"koristeellinen_muistikirja", "1F4D4"},
    {"suljettu_kirja", "1F4D5"},
    {"avoin_kirja", "1F4D6"},
    {"vihrea_kirja", "1F4D7"},
    {"sininen_kirja", "1F4D8"},
    {"oranssi_kirja", "1F4D9"},
    {"kirjoja", "1F4DA"},
    {"nimikyltti", "1F4DB"},
    {"kirjakaaro", "1F4DC"},
    {"muistio", "1F4DD"},
    {"luuri", "1F4DE"},
    {"hakulaite", "1F4DF"},
    {"faksi", "1F4E0"},
    {"lautasantenni", "1F4E1"},
    {"kovaaaninen", "1F4E2"},
    {"megafoni", "1F4E3"},
    {"lahtevat", "1F4E4"},
    {"saapuvat", "1F4E5"},
    {"paketti", "1F4E6"},
    {"sahkoposti", "1F4E7"},
    {"saapuva_kirjekuori", "1F4E8"},
    {"kirjekuori_ja_nuoli", "1F4E9"},
    {"suljettu_postilaatikko_lippu_alhaalla", "1F4EA"},
    {"suljettu_postilaatikko_lippu_ylhaalla", "1F4EB"},
    {"avoin_postilaatikko_lippu_ylhaalla", "1F4EC"},
    {"avoin_postilaatikko_lippu_alhaalla", "1F4ED"},
    {"postilaatikko", "1F4EE"},
    {"postitorvi", "1F4EF"},
    {"sanomalehti", "1F4F0"},
    {"kannykka", "1F4F1"},
    {"kannykka_ja_nuoli", "1F4F2"},
    {"varinatila", "1F4F3"},
    {"matkapuhelin_suljettava", "1F4F4"},
    {"ei_matkapuhelimia", "1F4F5"},
    {"kuuluvuuspalkit", "1F4F6"},
    {"kamera", "1F4F7"},
    {"kamera_ja_salama", "1F4F8"},
    {"videokamera", "1F4F9"},
    {"televisio", "1F4FA"},
    {"radio", "1F4FB"},
    {"videokasetti", "1F4FC"},
    {"elokuvaprojektori", "1F4FD"},
    {"rukoushelmet", "1F4FF"},
    {"sekoitus", "1F500"},
    {"toista_uudelleen", "1F501"},
    {"toista_kerran", "1F502"},
    {"pystynuolet_myotapaivaan", "1F503"},
    {"nuolet_vastapaivaan", "1F504"},
    {"himmenna", "1F505"},
    {"kirkasta", "1F506"},
    {"mykistetty_kaiutin", "1F507"},
    {"hiljainen_kaiutin", "1F508"},
    {"kaiutin_normaalilla_aanenvoimakkuudella", "1F509"},
    {"aanekas_kaiutin", "1F50A"},
    {"akku", "1F50B"},
    {"pistoke", "1F50C"},
    {"suurennuslasi_vasemmalle", "1F50D"},
    {"suurennuslasi_oikealle", "1F50E"},
    {"lukko_ja_kyna", "1F50F"},
    {"avain_ja_lukko_kiinni", "1F510"},
    {"avain", "1F511"},
    {"lukko", "1F512"},
    {"lukko_auki", "1F513"},
    {"kirkonkello", "1F514"},
    {"mykistetty_kello", "1F515"},
    {"kirjanmerkki", "1F516"},
    {"lenkki", "1F517"},
    {"valintanappi", "1F518"},
    {"back_nuoli", "1F519"},
    {"end_nuoli", "1F51A"},
    {"on_nuoli", "1F51B"},
    {"soon_nuoli", "1F51C"},
    {"top_nuoli", "1F51D"},
    {"kielletty_alaikaisilta", "1F51E"},
    {"nappain_10", "1F51F"},
    {"syota_isoja_kirjaimia", "1F520"},
    {"syota_pienia_kirjaimia", "1F521"},
    {"syota_numeroja", "1F522"},
    {"syota_merkkeja", "1F523"},
    {"syota_kirjaimia", "1F524"},
    {"tuli", "1F525"},
    {"taskulamppu", "1F526"},
    {"jakoavain", "1F527"},
    {"vasara", "1F528"},
    {"ruuvi_ja_mutteri", "1F529"},
    {"keittioveitsi", "1F52A"},
    {"vesipyssy", "1F52B"},
    {"mikroskooppi", "1F52C"},
    {"kaukoputki", "1F52D"},
    {"kristallipallo", "1F52E"},
    {"kuusisakarainen_tahti_pisteella", "1F52F"},
    {"japanilainen_symboli_aloittelijalle", "1F530"},
    {"kolmikarki", "1F531"},
    {"musta_nelioruutu", "1F532"},
    {"valkoinen_nelioruutu", "1F533"},
    {"punainen_ympyra", "1F534"},
    {"sininen_ympyra", "1F535"},
    {"iso_oranssi_vinonelio", "1F536"},
    {"iso_sininen_vinonelio", "1F537"},
    {"pieni_oranssi_vinonelio", "1F538"},
    {"pieni_sininen_vinonelio", "1F539"},
    {"punainen_kolmio_karki_ylos", "1F53A"},
    {"punainen_kolmio_karki_alas", "1F53B"},
    {"ylos", "1F53C"},
    {"alas", "1F53D"},
    {"om_tavu", "1F549"},
    {"kyyhky", "1F54A"},
    {"kaaba", "1F54B"},
    {"moskeija", "1F54C"},
    {"synagoga", "1F54D"},
    {"menora", "1F54E"},
    {"kello_yksi", "1F550"},
    {"kello_kaksi", "1F551"},
    {"kello_kolme", "1F552"},
    {"kello_nelja", "1F553"},
    {"kello_viisi", "1F554"},
    {"kello_kuusi", "1F555"},
    {"kello_seitseman", "1F556"},
    {"kello_kahdeksan", "1F557"},
    {"kello_yhdeksan", "1F558"},
    {"kello_kymmenen", "1F559"},
    {"kello_yksitoista", "1F55A"},
    {"kello_kaksitoista", "1F55B"},
    {"puoli_kaksi", "1F55C"},
    {"puoli_kolme", "1F55D"},
    {"puoli_nelja", "1F55E"},
    {"puoli_viisi", "1F55F"},
    {"puoli_kuusi", "1F560"},
    {"puoli_seitseman", "1F561"},
    {"puoli_kahdeksan", "1F562"},
    {"puoli_yhdeksan", "1F563"},
    {"puoli_kymmenen", "1F564"},
    {"puoli_yksitoista", "1F565"},
    {"puoli_kaksitoista", "1F566"},
    {"puoli_yksi", "1F567"},
    {"kynttila", "1F56F"},
    {"poytakello", "1F570"},
    {"musta_aukko", "1F573"},
    {"levitoiva_mies_puvussa", "1F574"},
    {"levitoiva_mies_puvussa_vari1", "1F574-1F3FB"},
    {"levitoiva_mies_puvussa_vari2", "1F574-1F3FC"},
    {"levitoiva_mies_puvussa_vari3", "1F574-1F3FD"},
    {"levitoiva_mies_puvussa_vari4", "1F574-1F3FE"},
    {"levitoiva_mies_puvussa_vari5", "1F574-1F3FF"},
    {"etsiva", "1F575"},
    {"etsiva_vari1", "1F575-1F3FB"},
    {"etsiva_vari2", "1F575-1F3FC"},
    {"etsiva_vari3", "1F575-1F3FD"},
    {"etsiva_vari4", "1F575-1F3FE"},
    {"etsiva_vari5", "1F575-1F3FF"},
    {"aurinkolasit", "1F576"},
    {"hamahakki", "1F577"},
    {"hamahakin_seitti", "1F578"},
    {"joystick", "1F579"},
    {"tanssiva_mies", "1F57A"},
    {"tanssiva_mies_vari1", "1F57A-1F3FB"},
    {"tanssiva_mies_vari2", "1F57A-1F3FC"},
    {"tanssiva_mies_vari3", "1F57A-1F3FD"},
    {"tanssiva_mies_vari4", "1F57A-1F3FE"},
    {"tanssiva_mies_vari5", "1F57A-1F3FF"},
    {"klemmarit_yhdessa", "1F587"},
    {"kuulakarkikyna", "1F58A"},
    {"taytekyna", "1F58B"},
    {"sivellin", "1F58C"},
    {"variliitu", "1F58D"},
    {"nostettu_kasi_ja_sormet_erillaan", "1F590"},
    {"nostettu_kasi_ja_sormet_erillaan_vari1", "1F590-1F3FB"},
    {"nostettu_kasi_ja_sormet_erillaan_vari2", "1F590-1F3FC"},
    {"nostettu_kasi_ja_sormet_erillaan_vari3", "1F590-1F3FD"},
    {"nostettu_kasi_ja_sormet_erillaan_vari4", "1F590-1F3FE"},
    {"nostettu_kasi_ja_sormet_erillaan_vari5", "1F590-1F3FF"},
    {"keskisormi", "1F595"},
    {"keskisormi_vari1", "1F595-1F3FB"},
    {"keskisormi_vari2", "1F595-1F3FC"},
    {"keskisormi_vari3", "1F595-1F3FD"},
    {"keskisormi_vari4", "1F595-1F3FE"},
    {"keskisormi_vari5", "1F595-1F3FF"},
    {"vulkanilaisten_tervehdys", "1F596"},
    {"vulkanilaisten_tervehdys_vari1", "1F596-1F3FB"},
    {"vulkanilaisten_tervehdys_vari2", "1F596-1F3FC"},
    {"vulkanilaisten_tervehdys_vari3", "1F596-1F3FD"},
    {"vulkanilaisten_tervehdys_vari4", "1F596-1F3FE"},
    {"vulkanilaisten_tervehdys_vari5", "1F596-1F3FF"},
    {"musta_sydan", "1F5A4"},
    {"poytakone", "1F5A5"},
    {"tulostin", "1F5A8"},
    {"tietokoneen_hiiri", "1F5B1"},
    {"ohjauspallo", "1F5B2"},
    {"kuva_kehyksissa", "1F5BC"},
    {"kortiston_valilehdet", "1F5C2"},
    {"arkistolaatikko", "1F5C3"},
    {"arkistokaappi", "1F5C4"},
    {"roskakori", "1F5D1"},
    {"kierrelehtio", "1F5D2"},
    {"kierrekalenteri", "1F5D3"},
    {"ruuvipuristin", "1F5DC"},
    {"vanha_avain", "1F5DD"},
    {"sanomalehti_rullalla", "1F5DE"},
    {"tikari", "1F5E1"},
    {"puhuva_paa", "1F5E3"},
    {"vasen_puhekupla", "1F5E8"},
    {"oikea_vihainen_puhekupla", "1F5EF"},
    {"aaniuurna", "1F5F3"},
    {"maailmankartta", "1F5FA"},
    {"fuji_vuori", "1F5FB"},
    {"tokyo_tower", "1F5FC"},
    {"vapaudenpatsas", "1F5FD"},
    {"japanin_kartta", "1F5FE"},
    {"moai", "1F5FF"},
    {"levea_hymy", "1F600"},
    {"virnistys_ja_hymyilevat_silmat", "1F601"},
    {"kasvot_naurunkyynelissa", "1F602"},
    {"hymy_suu_auki", "1F603"},
    {"hymyilevat_silmat_ja_suu", "1F604"},
    {"hymy_ja_kylmanhiki", "1F605"},
    {"hymy_ja_tiukasti_suljetut_silmat", "1F606"},
    {"hymy_ja_sadekeha", "1F607"},
    {"pirullinen_hymy_ja_sarvet", "1F608"},
    {"silmanisku", "1F609"},
    {"punastunut_hymy", "1F60A"},
    {"herkullista_ruokaa_nauttinut", "1F60B"},
    {"helpottunut", "1F60C"},
    {"hymy_ja_sydansilmat", "1F60D"},
    {"aurinkolasihymy", "1F60E"},
    {"virnistys", "1F60F"},
    {"naama_peruslukemilla", "1F610"},
    {"ilmeeton", "1F611"},
    {"ei_naurata", "1F612"},
    {"kylmanhiki", "1F613"},
    {"pohdiskeleva", "1F614"},
    {"hammentynyt", "1F615"},
    {"tyrmistynyt", "1F616"},
    {"pusunaama", "1F617"},
    {"lentosuukko", "1F618"},
    {"pusunaama_ja_hymyilevat_silmat", "1F619"},
    {"pusunaama_ja_suljetut_silmat", "1F61A"},
    {"kieli_ulkona", "1F61B"},
    {"kieli_ulkona_ja_silmanisku", "1F61C"},
    {"kieli_ulkona_ja_silmat_tiukasti_kiinni", "1F61D"},
    {"pettynyt", "1F61E"},
    {"huolestunut", "1F61F"},
    {"vihainen", "1F620"},
    {"mokottava", "1F621"},
    {"itkeva", "1F622"},
    {"karsiva", "1F623"},
    {"hoyrya_nenasta", "1F624"},
    {"pettynyt_mutta_helpottunut", "1F625"},
    {"surullinen_ja_suu_auki", "1F626"},
    {"ahdistunut", "1F627"},
    {"pelokas", "1F628"},
    {"uupunut", "1F629"},
    {"unelias", "1F62A"},
    {"vasynyt", "1F62B"},
    {"irvistava", "1F62C"},
    {"aaneen_itkeva", "1F62D"},
    {"yllattynyt", "1F62E"},
    {"jarkyttynyt", "1F62F"},
    {"avoin_suu_ja_kylmanhiki", "1F630"},
    {"pelokas_huuto", "1F631"},
    {"allistynyt", "1F632"},
    {"punastunut_naama", "1F633"},
    {"nukkuva", "1F634"},
    {"tajuton", "1F635"},
    {"naama_ilman_suuta", "1F636"},
    {"naama_ja_hengityssuoja", "1F637"},
    {"virnistava_kissa_ja_hymyilevat_silmat", "1F638"},
    {"kissa_ja_naurunkyyneleet", "1F639"},
    {"leveasti_hymyileva_kissa", "1F63A"},
    {"kissa_ja_sydansilmat", "1F63B"},
    {"vinosti_hymyileva_kissa", "1F63C"},
    {"kissan_pusunaama_ja_suljetut_silmat", "1F63D"},
    {"mokottava_kissa", "1F63E"},
    {"itkeva_kissa", "1F63F"},
    {"yllattynyt_kissa", "1F640"},
    {"hieman_surullinen", "1F641"},
    {"pieni_hymy", "1F642"},
    {"ylosalainen_naama", "1F643"},
    {"silmien_pyoritys", "1F644"},
    {"ei_ele", "1F645"},
    {"ei_ele_vari1", "1F645-1F3FB"},
    {"ei_ele_vari2", "1F645-1F3FC"},
    {"ei_ele_vari3", "1F645-1F3FD"},
    {"ei_ele_vari4", "1F645-1F3FE"},
    {"ei_ele_vari5", "1F645-1F3FF"},
    {"ok_ele", "1F646"},
    {"ok_ele_vari1", "1F646-1F3FB"},
    {"ok_ele_vari2", "1F646-1F3FC"},
    {"ok_ele_vari3", "1F646-1F3FD"},
    {"ok_ele_vari4", "1F646-1F3FE"},
    {"ok_ele_vari5", "1F646-1F3FF"},
    {"kumartava_henkilo", "1F647"},
    {"kumartava_henkilo_vari1", "1F647-1F3FB"},
    {"kumartava_henkilo_vari2", "1F647-1F3FC"},
    {"kumartava_henkilo_vari3", "1F647-1F3FD"},
    {"kumartava_henkilo_vari4", "1F647-1F3FE"},
    {"kumartava_henkilo_vari5", "1F647-1F3FF"},
    {"en_nae_pahaa", "1F648"},
    {"en_kuule_pahaa", "1F649"},
    {"en_puhu_pahaa", "1F64A"},
    {"iloinen_henkilo_viittaa", "1F64B"},
    {"iloinen_henkilo_viittaa_vari1", "1F64B-1F3FB"},
    {"iloinen_henkilo_viittaa_vari2", "1F64B-1F3FC"},
    {"iloinen_henkilo_viittaa_vari3", "1F64B-1F3FD"},
    {"iloinen_henkilo_viittaa_vari4", "1F64B-1F3FE"},
    {"iloinen_henkilo_viittaa_vari5", "1F64B-1F3FF"},
    {"nousevat_kadet", "1F64C"},
    {"nousevat_kadet_vari1", "1F64C-1F3FB"},
    {"nousevat_kadet_vari2", "1F64C-1F3FC"},
    {"nousevat_kadet_vari3", "1F64C-1F3FD"},
    {"nousevat_kadet_vari4", "1F64C-1F3FE"},
    {"nousevat_kadet_vari5", "1F64C-1F3FF"},
    {"surullinen_henkilo", "1F64D"},
    {"surullinen_henkilo_vari1", "1F64D-1F3FB"},
    {"surullinen_henkilo_vari2", "1F64D-1F3FC"},
    {"surullinen_henkilo_vari3", "1F64D-1F3FD"},
    {"surullinen_henkilo_vari4", "1F64D-1F3FE"},
    {"surullinen_henkilo_vari5", "1F64D-1F3FF"},
    {"mokottava_henkilo", "1F64E"},
    {"mokottava_henkilo_vari1", "1F64E-1F3FB"},
    {"mokottava_henkilo_vari2", "1F64E-1F3FC"},
    {"mokottava_henkilo_vari3", "1F64E-1F3FD"},
    {"mokottava_henkilo_vari4", "1F64E-1F3FE"},
    {"mokottava_henkilo_vari5", "1F64E-1F3FF"},
    {"kammenet_yhdessa", "1F64F"},
    {"kammenet_yhdessa_vari1", "1F64F-1F3FB"},
    {"kammenet_yhdessa_vari2", "1F64F-1F3FC"},
    {"kammenet_yhdessa_vari3", "1F64F-1F3FD"},
    {"kammenet_yhdessa_vari4", "1F64F-1F3FE"},
    {"kammenet_yhdessa_vari5", "1F64F-1F3FF"},
    {"raketti", "1F680"},
    {"helikopteri", "1F681"},
    {"veturi", "1F682"},
    {"junavaunu", "1F683"},
    {"pikajuna", "1F684"},
    {"luotipikajuna", "1F685"},
    {"juna", "1F686"},
    {"metro", "1F687"},
    {"kevytraidejuna", "1F688"},
    {"asema", "1F689"},
    {"raitioliikenne", "1F68A"},
    {"raitiovaunu", "1F68B"},
    {"bussi", "1F68C"},
    {"saapuva_bussi", "1F68D"},
    {"johdinbussi", "1F68E"},
    {"bussipysakki", "1F68F"},
    {"minibussi", "1F690"},
    {"ambulanssi", "1F691"},
    {"paloauto", "1F692"},
    {"poliisiauto", "1F693"},
    {"saapuva_poliisiauto", "1F694"},
    {"taksi", "1F695"},
    {"saapuva_taksi", "1F696"},
    {"auto", "1F697"},
    {"saapuva_auto", "1F698"},
    {"katumaasturi", "1F699"},
    {"kuljetusauto", "1F69A"},
    {"rekka_auto", "1F69B"},
    {"traktori", "1F69C"},
    {"monorail", "1F69D"},
    {"rinnerata", "1F69E"},
    {"riippurata", "1F69F"},
    {"gondolihissi", "1F6A0"},
    {"koysirata", "1F6A1"},
    {"laiva", "1F6A2"},
    {"soutaja", "1F6A3"},
    {"soutaja_vari1", "1F6A3-1F3FB"},
    {"soutaja_vari2", "1F6A3-1F3FC"},
    {"soutaja_vari3", "1F6A3-1F3FD"},
    {"soutaja_vari4", "1F6A3-1F3FE"},
    {"soutaja_vari5", "1F6A3-1F3FF"},
    {"pikavene", "1F6A4"},
    {"vaakasuuntaiset_liikennevalot", "1F6A5"},
    {"pystysuuntaiset_liikennevalot", "1F6A6"},
    {"rakennustyomaan_merkki", "1F6A7"},
    {"poliisiauton_valo", "1F6A8"},
    {"kolmiolippu", "1F6A9"},
    {"ovi", "1F6AA"},
    {"paasy_kielletty", "1F6AB"},
    {"savuke", "1F6AC"},
    {"tupakointi_kielletty", "1F6AD"},
    {"roskat_merkki", "1F6AE"},
    {"roskaaminen_kielletty", "1F6AF"},
    {"juomakelpoista_vetta", "1F6B0"},
    {"vesi_ei_juomakelpoista", "1F6B1"},
    {"polkupyora", "1F6B2"},
    {"polkupyoralla_ajo_kielletty", "1F6B3"},
    {"pyorailija", "1F6B4"},
    {"pyorailija_vari1", "1F6B4-1F3FB"},
    {"pyorailija_vari2", "1F6B4-1F3FC"},
    {"pyorailija_vari3", "1F6B4-1F3FD"},
    {"pyorailija_vari4", "1F6B4-1F3FE"},
    {"pyorailija_vari5", "1F6B4-1F3FF"},
    {"maastopyorailija", "1F6B5"},
    {"maastopyorailija_vari1", "1F6B5-1F3FB"},
    {"maastopyorailija_vari2", "1F6B5-1F3FC"},
    {"maastopyorailija_vari3", "1F6B5-1F3FD"},
    {"maastopyorailija_vari4", "1F6B5-1F3FE"},
    {"maastopyorailija_vari5", "1F6B5-1F3FF"},
    {"jalankulkija", "1F6B6"},
    {"jalankulkija_vari1", "1F6B6-1F3FB"},
    {"jalankulkija_vari2", "1F6B6-1F3FC"},
    {"jalankulkija_vari3", "1F6B6-1F3FD"},
    {"jalankulkija_vari4", "1F6B6-1F3FE"},
    {"jalankulkija_vari5", "1F6B6-1F3FF"},
    {"jalankulku_kielletty", "1F6B7"},
    {"lapsia_liikenteessa", "1F6B8"},
    {"miesten_wc", "1F6B9"},
    {"naisten_wc", "1F6BA"},
    {"wc", "1F6BB"},
    {"vauva_merkki", "1F6BC"},
    {"vessanpontto", "1F6BD"},
    {"vessa", "1F6BE"},
    {"suihku", "1F6BF"},
    {"kylpija", "1F6C0"},
    {"kylpija_vari1", "1F6C0-1F3FB"},
    {"kylpija_vari2", "1F6C0-1F3FC"},
    {"kylpija_vari3", "1F6C0-1F3FD"},
    {"kylpija_vari4", "1F6C0-1F3FE"},
    {"kylpija_vari5", "1F6C0-1F3FF"},
    {"kylpyamme", "1F6C1"},
    {"passintarkastus", "1F6C2"},
    {"tulli", "1F6C3"},
    {"matkatavarat", "1F6C4"},
    {"sailytyslokero", "1F6C5"},
    {"sohva_ja_lamppu", "1F6CB"},
    {"nukkuja", "1F6CC"},
    {"nukkuja_vari1", "1F6CC-1F3FB"},
    {"nukkuja_vari2", "1F6CC-1F3FC"},
    {"nukkuja_vari3", "1F6CC-1F3FD"},
    {"nukkuja_vari4", "1F6CC-1F3FE"},
    {"nukkuja_vari5", "1F6CC-1F3FF"},
    {"ostoskassit", "1F6CD"},
    {"soittokello", "1F6CE"},
    {"sanky", "1F6CF"},
    {"pyha_rakennus", "1F6D0"},
    {"pysahtymismerkki", "1F6D1"},
    {"ostoskarryt", "1F6D2"},
    {"hindutemppeli", "1F6D5"},
    {"maja", "1F6D6"},
    {"hissi", "1F6D7"},
    {"maanvyoryma", "1F6D8"},
    {"langaton", "1F6DC"},
    {"liukumaki", "1F6DD"},
    {"rengas", "1F6DE"},
    {"pelastusrengas", "1F6DF"},
    {"jakoavain_ja_vasara", "1F6E0"},
    {"kilpi", "1F6E1"},
    {"oljytynnyri", "1F6E2"},
    {"moottoritie", "1F6E3"},
    {"junaraide", "1F6E4"},
    {"moottorivene", "1F6E5"},
    {"pienkone", "1F6E9"},
    {"lahteva_lentokone", "1F6EB"},
    {"saapuva_lentokone", "1F6EC"},
    {"satelliitti", "1F6F0"},
    {"matkustajalaiva", "1F6F3"},
    {"potkulauta", "1F6F4"},
    {"skootteri", "1F6F5"},
    {"kanootti", "1F6F6"},
    {"kelkka", "1F6F7"},
    {"lentava_lautanen", "1F6F8"},
    {"rullalauta", "1F6F9"},
    {"autoriksa", "1F6FA"},
    {"lava_auto", "1F6FB"},
    {"rullaluistin", "1F6FC"},
    {"oranssi_ympyra", "1F7E0"},
    {"keltainen_ympyra", "1F7E1"},
    {"vihrea_ympyra", "1F7E2"},
    {"violetti_ympyra", "1F7E3"},
    {"ruskea_ympyra", "1F7E4"},
    {"punainen_nelio", "1F7E5"},
    {"sininen_nelio", "1F7E6"},
    {"oranssi_nelio", "1F7E7"},
    {"keltainen_nelio", "1F7E8"},
    {"vihrea_nelio", "1F7E9"},
    {"violetti_nelio", "1F7EA"},
    {"ruskea_nelio", "1F7EB"},
    {"paksu_yhtasuuruusmerkki", "1F7F0"},
    {"sormet_supussa", "1F90C"},
    {"sormet_supussa_vari1", "1F90C-1F3FB"},
    {"sormet_supussa_vari2", "1F90C-1F3FC"},
    {"sormet_supussa_vari3", "1F90C-1F3FD"},
    {"sormet_supussa_vari4", "1F90C-1F3FE"},
    {"sormet_supussa_vari5", "1F90C-1F3FF"},
    {"valkoinen_sydan", "1F90D"},
    {"ruskea_sydan", "1F90E"},
    {"nipistava_kasi", "1F90F"},
    {"nipistava_kasi_vari1", "1F90F-1F3FB"},
    {"nipistava_kasi_vari2", "1F90F-1F3FC"},
    {"nipistava_kasi_vari3", "1F90F-1F3FD"},
    {"nipistava_kasi_vari4", "1F90F-1F3FE"},
    {"nipistava_kasi_vari5", "1F90F-1F3FF"},
    {"vetoketjusuu", "1F910"},
    {"dollarisilmat_ja_rahasuu", "1F911"},
    {"naama_ja_kuumemittari", "1F912"},
    {"nortti", "1F913"},
    {"miettelias", "1F914"},
    {"naama_ja_paaside", "1F915"},
    {"robotti", "1F916"},
    {"halaava_naama", "1F917"},
    {"sarviele", "1F918"},
    {"sarviele_vari1", "1F918-1F3FB"},
    {"sarviele_vari2", "1F918-1F3FC"},
    {"sarviele_vari3", "1F918-1F3FD"},
    {"sarviele_vari4", "1F918-1F3FE"},
    {"sarviele_vari5", "1F918-1F3FF"},
    {"soita_minulle", "1F919"},
    {"soita_minulle_vari1", "1F919-1F3FB"},
    {"soita_minulle_vari2", "1F919-1F3FC"},
    {"soita_minulle_vari3", "1F919-1F3FD"},
    {"soita_minulle_vari4", "1F919-1F3FE"},
    {"soita_minulle_vari5", "1F919-1F3FF"},
    {"kammenselka_koholla", "1F91A"},
    {"kammenselka_koholla_vari1", "1F91A-1F3FB"},
    {"kammenselka_koholla_vari2", "1F91A-1F3FC"},
    {"kammenselka_koholla_vari3", "1F91A-1F3FD"},
    {"kammenselka_koholla_vari4", "1F91A-1F3FE"},
    {"kammenselka_koholla_vari5", "1F91A-1F3FF"},
    {"nyrkki_vasemmalle", "1F91B"},
    {"nyrkki_vasemmalle_vari1", "1F91B-1F3FB"},
    {"nyrkki_vasemmalle_vari2", "1F91B-1F3FC"},
    {"nyrkki_vasemmalle_vari3", "1F91B-1F3FD"},
    {"nyrkki_vasemmalle_vari4", "1F91B-1F3FE"},
    {"nyrkki_vasemmalle_vari5", "1F91B-1F3FF"},
    {"nyrkki_oikealle", "1F91C"},
    {"nyrkki_oikealle_vari1", "1F91C-1F3FB"},
    {"nyrkki_oikealle_vari2", "1F91C-1F3FC"},
    {"nyrkki_oikealle_vari3", "1F91C-1F3FD"},
    {"nyrkki_oikealle_vari4", "1F91C-1F3FE"},
    {"nyrkki_oikealle_vari5", "1F91C-1F3FF"},
    {"kadenpuristus", "1F91D"},
    {"kadenpuristus_vari1", "1F91D-1F3FB"},
    {"kadenpuristus_vari2", "1F91D-1F3FC"},
    {"kadenpuristus_vari3", "1F91D-1F3FD"},
    {"kadenpuristus_vari4", "1F91D-1F3FE"},
    {"kadenpuristus_vari5", "1F91D-1F3FF"},
    {"kadenpuristus_vari1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"kadenpuristus_vari1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"kadenpuristus_vari1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"kadenpuristus_vari1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"kadenpuristus_vari2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"kadenpuristus_vari2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"kadenpuristus_vari2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"kadenpuristus_vari2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"kadenpuristus_vari3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"kadenpuristus_vari3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"kadenpuristus_vari3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"kadenpuristus_vari3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"kadenpuristus_vari4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"kadenpuristus_vari4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"kadenpuristus_vari4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"kadenpuristus_vari4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"kadenpuristus_vari5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"kadenpuristus_vari5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"kadenpuristus_vari5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"kadenpuristus_vari5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"sormet_ristissa", "1F91E"},
    {"sormet_ristissa_vari1", "1F91E-1F3FB"},
    {"sormet_ristissa_vari2", "1F91E-1F3FC"},
    {"sormet_ristissa_vari3", "1F91E-1F3FD"},
    {"sormet_ristissa_vari4", "1F91E-1F3FE"},
    {"sormet_ristissa_vari5", "1F91E-1F3FF"},
    {"rakastan_sinua_ele", "1F91F"},
    {"rakastan_sinua_ele_vari1", "1F91F-1F3FB"},
    {"rakastan_sinua_ele_vari2", "1F91F-1F3FC"},
    {"rakastan_sinua_ele_vari3", "1F91F-1F3FD"},
    {"rakastan_sinua_ele_vari4", "1F91F-1F3FE"},
    {"rakastan_sinua_ele_vari5", "1F91F-1F3FF"},
    {"cowboy_hattu", "1F920"},
    {"pelle", "1F921"},
    {"pahoinvoiva", "1F922"},
    {"nauraa_kippurassa", "1F923"},
    {"kuolaava", "1F924"},
    {"valehteleva", "1F925"},
    {"peittaa_kasvot_kadella", "1F926"},
    {"peittaa_kasvot_kadella_vari1", "1F926-1F3FB"},
    {"peittaa_kasvot_kadella_vari2", "1F926-1F3FC"},
    {"peittaa_kasvot_kadella_vari3", "1F926-1F3FD"},
    {"peittaa_kasvot_kadella_vari4", "1F926-1F3FE"},
    {"peittaa_kasvot_kadella_vari5", "1F926-1F3FF"},
    {"aivastava", "1F927"},
    {"kohotetut_kulmakarvat", "1F928"},
    {"tahtisilma", "1F929"},
    {"mielipuoli", "1F92A"},
    {"hyssytteleva", "1F92B"},
    {"sensuroitu_suu", "1F92C"},
    {"kasi_suun_edessa", "1F92D"},
    {"oksentava", "1F92E"},
    {"rajahdys_paassa", "1F92F"},
    {"raskaana_oleva_nainen", "1F930"},
    {"raskaana_oleva_nainen_vari1", "1F930-1F3FB"},
    {"raskaana_oleva_nainen_vari2", "1F930-1F3FC"},
    {"raskaana_oleva_nainen_vari3", "1F930-1F3FD"},
    {"raskaana_oleva_nainen_vari4", "1F930-1F3FE"},
    {"raskaana_oleva_nainen_vari5", "1F930-1F3FF"},
    {"imetys", "1F931"},
    {"imetys_vari1", "1F931-1F3FB"},
    {"imetys_vari2", "1F931-1F3FC"},
    {"imetys_vari3", "1F931-1F3FD"},
    {"imetys_vari4", "1F931-1F3FE"},
    {"imetys_vari5", "1F931-1F3FF"},
    {"kammenet_vierekkain", "1F932"},
    {"kammenet_vierekkain_vari1", "1F932-1F3FB"},
    {"kammenet_vierekkain_vari2", "1F932-1F3FC"},
    {"kammenet_vierekkain_vari3", "1F932-1F3FD"},
    {"kammenet_vierekkain_vari4", "1F932-1F3FE"},
    {"kammenet_vierekkain_vari5", "1F932-1F3FF"},
    {"selfie", "1F933"},
    {"selfie_vari1", "1F933-1F3FB"},
    {"selfie_vari2", "1F933-1F3FC"},
    {"selfie_vari3", "1F933-1F3FD"},
    {"selfie_vari4", "1F933-1F3FE"},
    {"selfie_vari5", "1F933-1F3FF"},
    {"prinssi", "1F934"},
    {"prinssi_vari1", "1F934-1F3FB"},
    {"prinssi_vari2", "1F934-1F3FC"},
    {"prinssi_vari3", "1F934-1F3FD"},
    {"prinssi_vari4", "1F934-1F3FE"},
    {"prinssi_vari5", "1F934-1F3FF"},
    {"henkilo_smokissa", "1F935"},
    {"henkilo_smokissa_vari1", "1F935-1F3FB"},
    {"henkilo_smokissa_vari2", "1F935-1F3FC"},
    {"henkilo_smokissa_vari3", "1F935-1F3FD"},
    {"henkilo_smokissa_vari4", "1F935-1F3FE"},
    {"henkilo_smokissa_vari5", "1F935-1F3FF"},
    {"joulupukin_muori", "1F936"},
    {"joulupukin_muori_vari1", "1F936-1F3FB"},
    {"joulupukin_muori_vari2", "1F936-1F3FC"},
    {"joulupukin_muori_vari3", "1F936-1F3FD"},
    {"joulupukin_muori_vari4", "1F936-1F3FE"},
    {"joulupukin_muori_vari5", "1F936-1F3FF"},
    {"olkapaitaan_kohauttava_henkilo", "1F937"},
    {"olkapaitaan_kohauttava_henkilo_vari1", "1F937-1F3FB"},
    {"olkapaitaan_kohauttava_henkilo_vari2", "1F937-1F3FC"},
    {"olkapaitaan_kohauttava_henkilo_vari3", "1F937-1F3FD"},
    {"olkapaitaan_kohauttava_henkilo_vari4", "1F937-1F3FE"},
    {"olkapaitaan_kohauttava_henkilo_vari5", "1F937-1F3FF"},
    {"karrynpyora", "1F938"},
    {"karrynpyora_vari1", "1F938-1F3FB"},
    {"karrynpyora_vari2", "1F938-1F3FC"},
    {"karrynpyora_vari3", "1F938-1F3FD"},
    {"karrynpyora_vari4", "1F938-1F3FE"},
    {"karrynpyora_vari5", "1F938-1F3FF"},
    {"jongleeraaja", "1F939"},
    {"jongleeraaja_vari1", "1F939-1F3FB"},
    {"jongleeraaja_vari2", "1F939-1F3FC"},
    {"jongleeraaja_vari3", "1F939-1F3FD"},
    {"jongleeraaja_vari4", "1F939-1F3FE"},
    {"jongleeraaja_vari5", "1F939-1F3FF"},
    {"miekkailija", "1F93A"},
    {"painijat", "1F93C"},
    {"painijat_vari1", "1F93C-1F3FB"},
    {"painijat_vari2", "1F93C-1F3FC"},
    {"painijat_vari3", "1F93C-1F3FD"},
    {"painijat_vari4", "1F93C-1F3FE"},
    {"painijat_vari5", "1F93C-1F3FF"},
    {"painijat_vari1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"painijat_vari1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"painijat_vari1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"painijat_vari1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"painijat_vari2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"painijat_vari2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"painijat_vari2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"painijat_vari2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"painijat_vari3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"painijat_vari3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"painijat_vari3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"painijat_vari3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"painijat_vari4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"painijat_vari4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"painijat_vari4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"painijat_vari4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"painijat_vari5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"painijat_vari5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"painijat_vari5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"painijat_vari5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"vesipalloilija", "1F93D"},
    {"vesipalloilija_vari1", "1F93D-1F3FB"},
    {"vesipalloilija_vari2", "1F93D-1F3FC"},
    {"vesipalloilija_vari3", "1F93D-1F3FD"},
    {"vesipalloilija_vari4", "1F93D-1F3FE"},
    {"vesipalloilija_vari5", "1F93D-1F3FF"},
    {"kasipalloilija", "1F93E"},
    {"kasipalloilija_vari1", "1F93E-1F3FB"},
    {"kasipalloilija_vari2", "1F93E-1F3FC"},
    {"kasipalloilija_vari3", "1F93E-1F3FD"},
    {"kasipalloilija_vari4", "1F93E-1F3FE"},
    {"kasipalloilija_vari5", "1F93E-1F3FF"},
    {"sukellusmaski", "1F93F"},
    {"nuutunut_kukka", "1F940"},
    {"rumpu", "1F941"},
    {"kilisevat_lasit", "1F942"},
    {"juomalasi", "1F943"},
    {"lusikka", "1F944"},
    {"maaliverkko", "1F945"},
    {"kultamitali", "1F947"},
    {"hopeamitali", "1F948"},
    {"pronssimitali", "1F949"},
    {"nyrkkeilyhanska", "1F94A"},
    {"kamppailulajiasu", "1F94B"},
    {"curling_kivi", "1F94C"},
    {"lacrosse", "1F94D"},
    {"softball", "1F94E"},
    {"liitokiekko", "1F94F"},
    {"voisarvi", "1F950"},
    {"avokado", "1F951"},
    {"kurkku", "1F952"},
    {"pekoni", "1F953"},
    {"peruna", "1F954"},
    {"porkkana", "1F955"},
    {"patonki", "1F956"},
    {"vihrea_salaatti", "1F957"},
    {"paellapannu", "1F958"},
    {"taytetty_pita", "1F959"},
    {"kananmuna", "1F95A"},
    {"maitolasi", "1F95B"},
    {"maapahkina", "1F95C"},
    {"kiivi", "1F95D"},
    {"letut", "1F95E"},
    {"taikinanyytti", "1F95F"},
    {"onnenkeksi", "1F960"},
    {"ateriarasia", "1F961"},
    {"syomapuikot", "1F962"},
    {"kulho_ja_lusikka", "1F963"},
    {"lasi_ja_pilli", "1F964"},
    {"kookospahkina", "1F965"},
    {"parsakaali", "1F966"},
    {"piirakka", "1F967"},
    {"rinkila", "1F968"},
    {"lihanpala", "1F969"},
    {"voileipa", "1F96A"},
    {"sailyketolkki", "1F96B"},
    {"lehtivihannes", "1F96C"},
    {"mango", "1F96D"},
    {"kuukakku", "1F96E"},
    {"vesirinkeli", "1F96F"},
    {"hymy_ja_kolme_sydanta", "1F970"},
    {"haukotteleva_naama", "1F971"},
    {"hymyilevat_kasvot_ja_kyynel", "1F972"},
    {"juhliva", "1F973"},
    {"tokkurainen_naama", "1F974"},
    {"kuumottava", "1F975"},
    {"paleleva", "1F976"},
    {"ninja", "1F977"},
    {"ninja_vari1", "1F977-1F3FB"},
    {"ninja_vari2", "1F977-1F3FC"},
    {"ninja_vari3", "1F977-1F3FD"},
    {"ninja_vari4", "1F977-1F3FE"},
    {"ninja_vari5", "1F977-1F3FF"},
    {"naamioitunut", "1F978"},
    {"kyynelia_pidatteleva_naama", "1F979"},
    {"anova", "1F97A"},
    {"sari", "1F97B"},
    {"laboratoriotakki", "1F97C"},
    {"suojalasit", "1F97D"},
    {"vaelluskenka", "1F97E"},
    {"balleriina", "1F97F"},
    {"rapu", "1F980"},
    {"leijona", "1F981"},
    {"skorpioni", "1F982"},
    {"kalkkuna", "1F983"},
    {"yksisarvinen", "1F984"},
    {"kotka", "1F985"},
    {"sorsa", "1F986"},
    {"lepakko", "1F987"},
    {"hai", "1F988"},
    {"pollo", "1F989"},
    {"kettu", "1F98A"},
    {"perhonen", "1F98B"},
    {"peura", "1F98C"},
    {"gorilla", "1F98D"},
    {"lisko", "1F98E"},
    {"sarvikuono", "1F98F"},
    {"katkarapu", "1F990"},
    {"kalmari", "1F991"},
    {"kirahvi", "1F992"},
    {"seepra", "1F993"},
    {"siili", "1F994"},
    {"sauropodi", "1F995"},
    {"tyrannosaurus_rex", "1F996"},
    {"sirkka", "1F997"},
    {"kenguru", "1F998"},
    {"laama", "1F999"},
    {"riikinkukko", "1F99A"},
    {"virtahepo", "1F99B"},
    {"papukaija", "1F99C"},
    {"pesukarhu", "1F99D"},
    {"hummeri", "1F99E"},
    {"hyttynen", "1F99F"},
    {"mikrobi", "1F9A0"},
    {"mayra", "1F9A1"},
    {"joutsen", "1F9A2"},
    {"mammutti", "1F9A3"},
    {"dodo", "1F9A4"},
    {"laiskiainen", "1F9A5"},
    {"saukko", "1F9A6"},
    {"oranki", "1F9A7"},
    {"haisunaata", "1F9A8"},
    {"flamingo", "1F9A9"},
    {"osteri", "1F9AA"},
    {"majava", "1F9AB"},
    {"biisoni", "1F9AC"},
    {"hylje", "1F9AD"},
    {"opaskoira", "1F9AE"},
    {"valkoinen_keppi", "1F9AF"},
    {"punatukkainen", "1F9B0"},
    {"kiharatukkainen", "1F9B1"},
    {"kalju", "1F9B2"},
    {"valkotukkainen", "1F9B3"},
    {"luu", "1F9B4"},
    {"jalka", "1F9B5"},
    {"jalka_vari1", "1F9B5-1F3FB"},
    {"jalka_vari2", "1F9B5-1F3FC"},
    {"jalka_vari3", "1F9B5-1F3FD"},
    {"jalka_vari4", "1F9B5-1F3FE"},
    {"jalka_vari5", "1F9B5-1F3FF"},
    {"jalkatera", "1F9B6"},
    {"jalkatera_vari1", "1F9B6-1F3FB"},
    {"jalkatera_vari2", "1F9B6-1F3FC"},
    {"jalkatera_vari3", "1F9B6-1F3FD"},
    {"jalkatera_vari4", "1F9B6-1F3FE"},
    {"jalkatera_vari5", "1F9B6-1F3FF"},
    {"hammas", "1F9B7"},
    {"supersankari", "1F9B8"},
    {"supersankari_vari1", "1F9B8-1F3FB"},
    {"supersankari_vari2", "1F9B8-1F3FC"},
    {"supersankari_vari3", "1F9B8-1F3FD"},
    {"supersankari_vari4", "1F9B8-1F3FE"},
    {"supersankari_vari5", "1F9B8-1F3FF"},
    {"superpahis", "1F9B9"},
    {"superpahis_vari1", "1F9B9-1F3FB"},
    {"superpahis_vari2", "1F9B9-1F3FC"},
    {"superpahis_vari3", "1F9B9-1F3FD"},
    {"superpahis_vari4", "1F9B9-1F3FE"},
    {"superpahis_vari5", "1F9B9-1F3FF"},
    {"turvaliivi", "1F9BA"},
    {"kuulolaite_korvassa", "1F9BB"},
    {"kuulolaite_korvassa_vari1", "1F9BB-1F3FB"},
    {"kuulolaite_korvassa_vari2", "1F9BB-1F3FC"},
    {"kuulolaite_korvassa_vari3", "1F9BB-1F3FD"},
    {"kuulolaite_korvassa_vari4", "1F9BB-1F3FE"},
    {"kuulolaite_korvassa_vari5", "1F9BB-1F3FF"},
    {"moottoroitu_pyoratuoli", "1F9BC"},
    {"kasikayttoinen_pyoratuoli", "1F9BD"},
    {"tekokasi", "1F9BE"},
    {"tekojalka", "1F9BF"},
    {"juustopala", "1F9C0"},
    {"vuokaleivos", "1F9C1"},
    {"suola", "1F9C2"},
    {"juomatetra", "1F9C3"},
    {"valkosipuli", "1F9C4"},
    {"sipuli", "1F9C5"},
    {"falafel", "1F9C6"},
    {"vohveli", "1F9C7"},
    {"voi", "1F9C8"},
    {"mate", "1F9C9"},
    {"jaakuutio", "1F9CA"},
    {"kuplatee", "1F9CB"},
    {"trolli", "1F9CC"},
    {"seisova_henkilo", "1F9CD"},
    {"seisova_henkilo_vari1", "1F9CD-1F3FB"},
    {"seisova_henkilo_vari2", "1F9CD-1F3FC"},
    {"seisova_henkilo_vari3", "1F9CD-1F3FD"},
    {"seisova_henkilo_vari4", "1F9CD-1F3FE"},
    {"seisova_henkilo_vari5", "1F9CD-1F3FF"},
    {"polvistuva_ihminen", "1F9CE"},
    {"polvistuva_ihminen_vari1", "1F9CE-1F3FB"},
    {"polvistuva_ihminen_vari2", "1F9CE-1F3FC"},
    {"polvistuva_ihminen_vari3", "1F9CE-1F3FD"},
    {"polvistuva_ihminen_vari4", "1F9CE-1F3FE"},
    {"polvistuva_ihminen_vari5", "1F9CE-1F3FF"},
    {"kuuro_henkilo", "1F9CF"},
    {"kuuro_henkilo_vari1", "1F9CF-1F3FB"},
    {"kuuro_henkilo_vari2", "1F9CF-1F3FC"},
    {"kuuro_henkilo_vari3", "1F9CF-1F3FD"},
    {"kuuro_henkilo_vari4", "1F9CF-1F3FE"},
    {"kuuro_henkilo_vari5", "1F9CF-1F3FF"},
    {"monokkeli_silmassa", "1F9D0"},
    {"aikuinen", "1F9D1"},
    {"aikuinen_vari1", "1F9D1-1F3FB"},
    {"aikuinen_vari2", "1F9D1-1F3FC"},
    {"aikuinen_vari3", "1F9D1-1F3FD"},
    {"aikuinen_vari4", "1F9D1-1F3FE"},
    {"aikuinen_vari5", "1F9D1-1F3FF"},
    {"lapsi", "1F9D2"},
    {"lapsi_vari1", "1F9D2-1F3FB"},
    {"lapsi_vari2", "1F9D2-1F3FC"},
    {"lapsi_vari3", "1F9D2-1F3FD"},
    {"lapsi_vari4", "1F9D2-1F3FE"},
    {"lapsi_vari5", "1F9D2-1F3FF"},
    {"vanhus", "1F9D3"},
    {"vanhus_vari1", "1F9D3-1F3FB"},
    {"vanhus_vari2", "1F9D3-1F3FC"},
    {"vanhus_vari3", "1F9D3-1F3FD"},
    {"vanhus_vari4", "1F9D3-1F3FE"},
    {"vanhus_vari5", "1F9D3-1F3FF"},
    {"parrakas_henkilo", "1F9D4"},
    {"parrakas_henkilo_vari1", "1F9D4-1F3FB"},
    {"parrakas_henkilo_vari2", "1F9D4-1F3FC"},
    {"parrakas_henkilo_vari3", "1F9D4-1F3FD"},
    {"parrakas_henkilo_vari4", "1F9D4-1F3FE"},
    {"parrakas_henkilo_vari5", "1F9D4-1F3FF"},
    {"huivipainen_nainen", "1F9D5"},
    {"huivipainen_nainen_vari1", "1F9D5-1F3FB"},
    {"huivipainen_nainen_vari2", "1F9D5-1F3FC"},
    {"huivipainen_nainen_vari3", "1F9D5-1F3FD"},
    {"huivipainen_nainen_vari4", "1F9D5-1F3FE"},
    {"huivipainen_nainen_vari5", "1F9D5-1F3FF"},
    {"henkilo_saunassa", "1F9D6"},
    {"henkilo_saunassa_vari1", "1F9D6-1F3FB"},
    {"henkilo_saunassa_vari2", "1F9D6-1F3FC"},
    {"henkilo_saunassa_vari3", "1F9D6-1F3FD"},
    {"henkilo_saunassa_vari4", "1F9D6-1F3FE"},
    {"henkilo_saunassa_vari5", "1F9D6-1F3FF"},
    {"kiipeilija", "1F9D7"},
    {"kiipeilija_vari1", "1F9D7-1F3FB"},
    {"kiipeilija_vari2", "1F9D7-1F3FC"},
    {"kiipeilija_vari3", "1F9D7-1F3FD"},
    {"kiipeilija_vari4", "1F9D7-1F3FE"},
    {"kiipeilija_vari5", "1F9D7-1F3FF"},
    {"henkilo_lootusasennossa", "1F9D8"},
    {"henkilo_lootusasennossa_vari1", "1F9D8-1F3FB"},
    {"henkilo_lootusasennossa_vari2", "1F9D8-1F3FC"},
    {"henkilo_lootusasennossa_vari3", "1F9D8-1F3FD"},
    {"henkilo_lootusasennossa_vari4", "1F9D8-1F3FE"},
    {"henkilo_lootusasennossa_vari5", "1F9D8-1F3FF"},
    {"velho", "1F9D9"},
    {"velho_vari1", "1F9D9-1F3FB"},
    {"velho_vari2", "1F9D9-1F3FC"},
    {"velho_vari3", "1F9D9-1F3FD"},
    {"velho_vari4", "1F9D9-1F3FE"},
    {"velho_vari5", "1F9D9-1F3FF"},
    {"keiju", "1F9DA"},
    {"keiju_vari1", "1F9DA-1F3FB"},
    {"keiju_vari2", "1F9DA-1F3FC"},
    {"keiju_vari3", "1F9DA-1F3FD"},
    {"keiju_vari4", "1F9DA-1F3FE"},
    {"keiju_vari5", "1F9DA-1F3FF"},
    {"vampyyri", "1F9DB"},
    {"vampyyri_vari1", "1F9DB-1F3FB"},
    {"vampyyri_vari2", "1F9DB-1F3FC"},
    {"vampyyri_vari3", "1F9DB-1F3FD"},
    {"vampyyri_vari4", "1F9DB-1F3FE"},
    {"vampyyri_vari5", "1F9DB-1F3FF"},
    {"merenvaki", "1F9DC"},
    {"merenvaki_vari1", "1F9DC-1F3FB"},
    {"merenvaki_vari2", "1F9DC-1F3FC"},
    {"merenvaki_vari3", "1F9DC-1F3FD"},
    {"merenvaki_vari4", "1F9DC-1F3FE"},
    {"merenvaki_vari5", "1F9DC-1F3FF"},
    {"haltia", "1F9DD"},
    {"haltia_vari1", "1F9DD-1F3FB"},
    {"haltia_vari2", "1F9DD-1F3FC"},
    {"haltia_vari3", "1F9DD-1F3FD"},
    {"haltia_vari4", "1F9DD-1F3FE"},
    {"haltia_vari5", "1F9DD-1F3FF"},
    {"henki", "1F9DE"},
    {"zombi", "1F9DF"},
    {"aivot", "1F9E0"},
    {"oranssi_sydan", "1F9E1"},
    {"lippalakki", "1F9E2"},
    {"huivi", "1F9E3"},
    {"hansikkaat", "1F9E4"},
    {"takki", "1F9E5"},
    {"sukat", "1F9E6"},
    {"punainen_kirjekuori", "1F9E7"},
    {"sahikainen", "1F9E8"},
    {"palapeli", "1F9E9"},
    {"koeputki", "1F9EA"},
    {"petrimalja", "1F9EB"},
    {"dna", "1F9EC"},
    {"kompassi", "1F9ED"},
    {"helmitaulu", "1F9EE"},
    {"sammutin", "1F9EF"},
    {"tyokalulaatikko", "1F9F0"},
    {"tiili", "1F9F1"},
    {"magneetti", "1F9F2"},
    {"matkatavara", "1F9F3"},
    {"hoitoainepullo", "1F9F4"},
    {"lankarulla", "1F9F5"},
    {"lankakera", "1F9F6"},
    {"hakaneula", "1F9F7"},
    {"nalle", "1F9F8"},
    {"luuta", "1F9F9"},
    {"kori", "1F9FA"},
    {"paperirulla", "1F9FB"},
    {"saippua", "1F9FC"},
    {"pesusieni", "1F9FD"},
    {"kuitti", "1F9FE"},
    {"nazar_amuletti", "1F9FF"},
    {"balettitossut", "1FA70"},
    {"uimapuku", "1FA71"},
    {"uimahousut", "1FA72"},
    {"uimashortsit", "1FA73"},
    {"varvassandaali", "1FA74"},
    {"vaaleansininen_sydan", "1FA75"},
    {"harmaa_sydan", "1FA76"},
    {"pinkki_sydan", "1FA77"},
    {"veripisara", "1FA78"},
    {"laastari", "1FA79"},
    {"stetoskooppi", "1FA7A"},
    {"rontgen", "1FA7B"},
    {"kainalosauva", "1FA7C"},
    {"jojo", "1FA80"},
    {"leija", "1FA81"},
    {"laskuvarjo", "1FA82"},
    {"bumerangi", "1FA83"},
    {"taikasauva", "1FA84"},
    {"pinata", "1FA85"},
    {"maatuskanuket", "1FA86"},
    {"marakassit", "1FA87"},
    {"huilu", "1FA88"},
    {"harppu", "1FA89"},
    {"pasuuna", "1FA8A"},
    {"aarrearkku", "1FA8E"},
    {"lapio", "1FA8F"},
    {"rengasplaneetta", "1FA90"},
    {"tuoli", "1FA91"},
    {"partaveitsi", "1FA92"},
    {"kirves", "1FA93"},
    {"savilamppu", "1FA94"},
    {"banjo", "1FA95"},
    {"sotilaskypara", "1FA96"},
    {"harmonikka", "1FA97"},
    {"congarumpu", "1FA98"},
    {"kolikko", "1FA99"},
    {"kasisaha", "1FA9A"},
    {"ruuvimeisseli", "1FA9B"},
    {"tikkaat", "1FA9C"},
    {"koukku", "1FA9D"},
    {"peili", "1FA9E"},
    {"ikkuna", "1FA9F"},
    {"viemaripumppu", "1FAA0"},
    {"ompeluneula", "1FAA1"},
    {"solmu", "1FAA2"},
    {"ampari", "1FAA3"},
    {"hiirenloukku", "1FAA4"},
    {"hammasharja", "1FAA5"},
    {"hautakivi", "1FAA6"},
    {"kyltti", "1FAA7"},
    {"kivi", "1FAA8"},
    {"peilipallo", "1FAA9"},
    {"henkilokortti", "1FAAA"},
    {"akku_vahissa", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"taittuva_kasiviuhka", "1FAAD"},
    {"selvityskampa", "1FAAE"},
    {"khanda", "1FAAF"},
    {"karpanen", "1FAB0"},
    {"mato", "1FAB1"},
    {"kuoriainen", "1FAB2"},
    {"torakka", "1FAB3"},
    {"ruukkukasvi", "1FAB4"},
    {"halko", "1FAB5"},
    {"hoyhen", "1FAB6"},
    {"lootus", "1FAB7"},
    {"koralli", "1FAB8"},
    {"tyhja_pesa", "1FAB9"},
    {"munia_pesassa", "1FABA"},
    {"hyasintti", "1FABB"},
    {"meduusa", "1FABC"},
    {"siipi", "1FABD"},
    {"lehdeton_puu", "1FABE"},
    {"hanhi", "1FABF"},
    {"sydan", "1FAC0"},
    {"keuhkot", "1FAC1"},
    {"halaus", "1FAC2"},
    {"raskaana_oleva_mies", "1FAC3"},
    {"raskaana_oleva_mies_vari1", "1FAC3-1F3FB"},
    {"raskaana_oleva_mies_vari2", "1FAC3-1F3FC"},
    {"raskaana_oleva_mies_vari3", "1FAC3-1F3FD"},
    {"raskaana_oleva_mies_vari4", "1FAC3-1F3FE"},
    {"raskaana_oleva_mies_vari5", "1FAC3-1F3FF"},
    {"raskaana_oleva_henkilo", "1FAC4"},
    {"raskaana_oleva_henkilo_vari1", "1FAC4-1F3FB"},
    {"raskaana_oleva_henkilo_vari2", "1FAC4-1F3FC"},
    {"raskaana_oleva_henkilo_vari3", "1FAC4-1F3FD"},
    {"raskaana_oleva_henkilo_vari4", "1FAC4-1F3FE"},
    {"raskaana_oleva_henkilo_vari5", "1FAC4-1F3FF"},
    {"kruunua_kantava_henkilo", "1FAC5"},
    {"kruunua_kantava_henkilo_vari1", "1FAC5-1F3FB"},
    {"kruunua_kantava_henkilo_vari2", "1FAC5-1F3FC"},
    {"kruunua_kantava_henkilo_vari3", "1FAC5-1F3FD"},
    {"kruunua_kantava_henkilo_vari4", "1FAC5-1F3FE"},
    {"kruunua_kantava_henkilo_vari5", "1FAC5-1F3FF"},
    {"sormenjalki", "1FAC6"},
    {"karvainen_otus", "1FAC8"},
    {"miekkavalas", "1FACD"},
    {"hirvi", "1FACE"},
    {"aasi", "1FACF"},
    {"mustikat", "1FAD0"},
    {"paprika", "1FAD1"},
    {"oliivi", "1FAD2"},
    {"rieska", "1FAD3"},
    {"tamal", "1FAD4"},
    {"fondue", "1FAD5"},
    {"teepannu", "1FAD6"},
    {"nesteen_kaataminen", "1FAD7"},
    {"pavut", "1FAD8"},
    {"purkki", "1FAD9"},
    {"inkivaari", "1FADA"},
    {"herneenpalko", "1FADB"},
    {"juures", "1FADC"},
    {"roiske", "1FADF"},
    {"sulava_naama", "1FAE0"},
    {"kunnioittava_tervehdys", "1FAE1"},
    {"silmat_pyoreina_kasi_suun_edessa", "1FAE2"},
    {"kurkistus_sormien_takaa", "1FAE3"},
    {"naama_suu_vinossa", "1FAE4"},
    {"katkoviivalla_piirretty_naama", "1FAE5"},
    {"purra_huulta", "1FAE6"},
    {"kuplat", "1FAE7"},
    {"tariseva", "1FAE8"},
    {"silmapussit", "1FAE9"},
    {"vaaristyneet_kasvot", "1FAEA"},
    {"tappelupilvi", "1FAEF"},
    {"etusormi_ja_peukalo_ristissa", "1FAF0"},
    {"etusormi_ja_peukalo_ristissa_vari1", "1FAF0-1F3FB"},
    {"etusormi_ja_peukalo_ristissa_vari2", "1FAF0-1F3FC"},
    {"etusormi_ja_peukalo_ristissa_vari3", "1FAF0-1F3FD"},
    {"etusormi_ja_peukalo_ristissa_vari4", "1FAF0-1F3FE"},
    {"etusormi_ja_peukalo_ristissa_vari5", "1FAF0-1F3FF"},
    {"oikealle_ojentuva_kasi", "1FAF1"},
    {"oikealle_ojentuva_kasi_vari1", "1FAF1-1F3FB"},
    {"oikealle_ojentuva_kasi_vari2", "1FAF1-1F3FC"},
    {"oikealle_ojentuva_kasi_vari3", "1FAF1-1F3FD"},
    {"oikealle_ojentuva_kasi_vari4", "1FAF1-1F3FE"},
    {"oikealle_ojentuva_kasi_vari5", "1FAF1-1F3FF"},
    {"vasemmalle_ojentuva_kasi", "1FAF2"},
    {"vasemmalle_ojentuva_kasi_vari1", "1FAF2-1F3FB"},
    {"vasemmalle_ojentuva_kasi_vari2", "1FAF2-1F3FC"},
    {"vasemmalle_ojentuva_kasi_vari3", "1FAF2-1F3FD"},
    {"vasemmalle_ojentuva_kasi_vari4", "1FAF2-1F3FE"},
    {"vasemmalle_ojentuva_kasi_vari5", "1FAF2-1F3FF"},
    {"kasi_kammen_alaspain", "1FAF3"},
    {"kasi_kammen_alaspain_vari1", "1FAF3-1F3FB"},
    {"kasi_kammen_alaspain_vari2", "1FAF3-1F3FC"},
    {"kasi_kammen_alaspain_vari3", "1FAF3-1F3FD"},
    {"kasi_kammen_alaspain_vari4", "1FAF3-1F3FE"},
    {"kasi_kammen_alaspain_vari5", "1FAF3-1F3FF"},
    {"kasi_kammen_ylospain", "1FAF4"},
    {"kasi_kammen_ylospain_vari1", "1FAF4-1F3FB"},
    {"kasi_kammen_ylospain_vari2", "1FAF4-1F3FC"},
    {"kasi_kammen_ylospain_vari3", "1FAF4-1F3FD"},
    {"kasi_kammen_ylospain_vari4", "1FAF4-1F3FE"},
    {"kasi_kammen_ylospain_vari5", "1FAF4-1F3FF"},
    {"etusormi_osoittaa_katsojaa", "1FAF5"},
    {"etusormi_osoittaa_katsojaa_vari1", "1FAF5-1F3FB"},
    {"etusormi_osoittaa_katsojaa_vari2", "1FAF5-1F3FC"},
    {"etusormi_osoittaa_katsojaa_vari3", "1FAF5-1F3FD"},
    {"etusormi_osoittaa_katsojaa_vari4", "1FAF5-1F3FE"},
    {"etusormi_osoittaa_katsojaa_vari5", "1FAF5-1F3FF"},
    {"sydankuvio_kasilla", "1FAF6"},
    {"sydankuvio_kasilla_vari1", "1FAF6-1F3FB"},
    {"sydankuvio_kasilla_vari2", "1FAF6-1F3FC"},
    {"sydankuvio_kasilla_vari3", "1FAF6-1F3FD"},
    {"sydankuvio_kasilla_vari4", "1FAF6-1F3FE"},
    {"sydankuvio_kasilla_vari5", "1FAF6-1F3FF"},
    {"vasemmalle_tyontava_kasi", "1FAF7"},
    {"vasemmalle_tyontava_kasi_vari1", "1FAF7-1F3FB"},
    {"vasemmalle_tyontava_kasi_vari2", "1FAF7-1F3FC"},
    {"vasemmalle_tyontava_kasi_vari3", "1FAF7-1F3FD"},
    {"vasemmalle_tyontava_kasi_vari4", "1FAF7-1F3FE"},
    {"vasemmalle_tyontava_kasi_vari5", "1FAF7-1F3FF"},
    {"oikealle_tyontava_kasi", "1FAF8"},
    {"oikealle_tyontava_kasi_vari1", "1FAF8-1F3FB"},
    {"oikealle_tyontava_kasi_vari2", "1FAF8-1F3FC"},
    {"oikealle_tyontava_kasi_vari3", "1FAF8-1F3FD"},
    {"oikealle_tyontava_kasi_vari4", "1FAF8-1F3FE"},
    {"oikealle_tyontava_kasi_vari5", "1FAF8-1F3FF"},
    {"nappain_numeromerkki", "0023-FE0F-20E3"},
    {"nappain_tahtimerkki", "002A-FE0F-20E3"},
    {"nappain_0", "0030-FE0F-20E3"},
    {"nappain_1", "0031-FE0F-20E3"},
    {"nappain_2", "0032-FE0F-20E3"},
    {"nappain_3", "0033-FE0F-20E3"},
    {"nappain_4", "0034-FE0F-20E3"},
    {"nappain_5", "0035-FE0F-20E3"},
    {"nappain_6", "0036-FE0F-20E3"},
    {"nappain_7", "0037-FE0F-20E3"},
    {"nappain_8", "0038-FE0F-20E3"},
    {"nappain_9", "0039-FE0F-20E3"},
    {"lippu_ascension_saari", "1F1E6-1F1E8"},
    {"lippu_andorra", "1F1E6-1F1E9"},
    {"lippu_arabiemiirikunnat", "1F1E6-1F1EA"},
    {"lippu_afganistan", "1F1E6-1F1EB"},
    {"lippu_antigua_ja_barbuda", "1F1E6-1F1EC"},
    {"lippu_anguilla", "1F1E6-1F1EE"},
    {"lippu_albania", "1F1E6-1F1F1"},
    {"lippu_armenia", "1F1E6-1F1F2"},
    {"lippu_angola", "1F1E6-1F1F4"},
    {"lippu_antarktis", "1F1E6-1F1F6"},
    {"lippu_argentiina", "1F1E6-1F1F7"},
    {"lippu_amerikan_samoa", "1F1E6-1F1F8"},
    {"lippu_itavalta", "1F1E6-1F1F9"},
    {"lippu_australia", "1F1E6-1F1FA"},
    {"lippu_aruba", "1F1E6-1F1FC"},
    {"lippu_ahvenanmaa", "1F1E6-1F1FD"},
    {"lippu_azerbaidzan", "1F1E6-1F1FF"},
    {"lippu_bosnia_ja_hertsegovina", "1F1E7-1F1E6"},
    {"lippu_barbados", "1F1E7-1F1E7"},
    {"lippu_bangladesh", "1F1E7-1F1E9"},
    {"lippu_belgia", "1F1E7-1F1EA"},
    {"lippu_burkina_faso", "1F1E7-1F1EB"},
    {"lippu_bulgaria", "1F1E7-1F1EC"},
    {"lippu_bahrain", "1F1E7-1F1ED"},
    {"lippu_burundi", "1F1E7-1F1EE"},
    {"lippu_benin", "1F1E7-1F1EF"},
    {"lippu_saint_barthelemy", "1F1E7-1F1F1"},
    {"lippu_bermuda", "1F1E7-1F1F2"},
    {"lippu_brunei", "1F1E7-1F1F3"},
    {"lippu_bolivia", "1F1E7-1F1F4"},
    {"lippu_karibian_alankomaat", "1F1E7-1F1F6"},
    {"lippu_brasilia", "1F1E7-1F1F7"},
    {"lippu_bahama", "1F1E7-1F1F8"},
    {"lippu_bhutan", "1F1E7-1F1F9"},
    {"lippu_bouvetnsaari", "1F1E7-1F1FB"},
    {"lippu_botswana", "1F1E7-1F1FC"},
    {"lippu_valko_venaja", "1F1E7-1F1FE"},
    {"lippu_belize", "1F1E7-1F1FF"},
    {"lippu_kanada", "1F1E8-1F1E6"},
    {"lippu_kookossaaret_keelingsaaret", "1F1E8-1F1E8"},
    {"lippu_kongon_demokraattinen_tasavalta", "1F1E8-1F1E9"},
    {"lippu_keski_afrikan_tasavalta", "1F1E8-1F1EB"},
    {"lippu_kongon_tasavalta", "1F1E8-1F1EC"},
    {"lippu_sveitsi", "1F1E8-1F1ED"},
    {"lippu_norsunluurannikko", "1F1E8-1F1EE"},
    {"lippu_cookinsaaret", "1F1E8-1F1F0"},
    {"lippu_chile", "1F1E8-1F1F1"},
    {"lippu_kamerun", "1F1E8-1F1F2"},
    {"lippu_kiina", "1F1E8-1F1F3"},
    {"lippu_kolumbia", "1F1E8-1F1F4"},
    {"lippu_clippertoninsaari", "1F1E8-1F1F5"},
    {"lippu_sark", "1F1E8-1F1F6"},
    {"lippu_costa_rica", "1F1E8-1F1F7"},
    {"lippu_kuuba", "1F1E8-1F1FA"},
    {"lippu_kap_verde", "1F1E8-1F1FB"},
    {"lippu_curacao", "1F1E8-1F1FC"},
    {"lippu_joulusaari", "1F1E8-1F1FD"},
    {"lippu_kypros", "1F1E8-1F1FE"},
    {"lippu_tsekki", "1F1E8-1F1FF"},
    {"lippu_saksa", "1F1E9-1F1EA"},
    {"lippu_diego_garcia", "1F1E9-1F1EC"},
    {"lippu_djibouti", "1F1E9-1F1EF"},
    {"lippu_tanska", "1F1E9-1F1F0"},
    {"lippu_dominica", "1F1E9-1F1F2"},
    {"lippu_dominikaaninen_tasavalta", "1F1E9-1F1F4"},
    {"lippu_algeria", "1F1E9-1F1FF"},
    {"lippu_ceuta_ja_melilla", "1F1EA-1F1E6"},
    {"lippu_ecuador", "1F1EA-1F1E8"},
    {"lippu_viro", "1F1EA-1F1EA"},
    {"lippu_egypti", "1F1EA-1F1EC"},
    {"lippu_lansi_sahara", "1F1EA-1F1ED"},
    {"lippu_eritrea", "1F1EA-1F1F7"},
    {"lippu_espanja", "1F1EA-1F1F8"},
    {"lippu_etiopia", "1F1EA-1F1F9"},
    {"lippu_euroopan_unioni", "1F1EA-1F1FA"},
    {"lippu_suomi", "1F1EB-1F1EE"},
    {"lippu_fidzi", "1F1EB-1F1EF"},
    {"lippu_falklandinsaaret", "1F1EB-1F1F0"},
    {"lippu_mikronesia", "1F1EB-1F1F2"},
    {"lippu_farsaaret", "1F1EB-1F1F4"},
    {"lippu_ranska", "1F1EB-1F1F7"},
    {"lippu_gabon", "1F1EC-1F1E6"},
    {"lippu_iso_britannia", "1F1EC-1F1E7"},
    {"lippu_grenada", "1F1EC-1F1E9"},
    {"lippu_georgia", "1F1EC-1F1EA"},
    {"lippu_ranskan_guayana", "1F1EC-1F1EB"},
    {"lippu_guernsey", "1F1EC-1F1EC"},
    {"lippu_ghana", "1F1EC-1F1ED"},
    {"lippu_gibraltar", "1F1EC-1F1EE"},
    {"lippu_gronlanti", "1F1EC-1F1F1"},
    {"lippu_gambia", "1F1EC-1F1F2"},
    {"lippu_guinea", "1F1EC-1F1F3"},
    {"lippu_guadeloupe", "1F1EC-1F1F5"},
    {"lippu_paivantasaajan_guinea", "1F1EC-1F1F6"},
    {"lippu_kreikka", "1F1EC-1F1F7"},
    {"lippu_etela_georgia_ja_etelaiset_sandwichinsaaret", "1F1EC-1F1F8"},
    {"lippu_guatemala", "1F1EC-1F1F9"},
    {"lippu_guam", "1F1EC-1F1FA"},
    {"lippu_guinea_bissau", "1F1EC-1F1FC"},
    {"lippu_guyana", "1F1EC-1F1FE"},
    {"lippu_hongkong_kiinan_erityishallintoalue", "1F1ED-1F1F0"},
    {"lippu_heard_ja_mcdonaldinsaaret", "1F1ED-1F1F2"},
    {"lippu_honduras", "1F1ED-1F1F3"},
    {"lippu_kroatia", "1F1ED-1F1F7"},
    {"lippu_haiti", "1F1ED-1F1F9"},
    {"lippu_unkari", "1F1ED-1F1FA"},
    {"lippu_kanariansaaret", "1F1EE-1F1E8"},
    {"lippu_indonesia", "1F1EE-1F1E9"},
    {"lippu_irlanti", "1F1EE-1F1EA"},
    {"lippu_israel", "1F1EE-1F1F1"},
    {"lippu_mansaari", "1F1EE-1F1F2"},
    {"lippu_intia", "1F1EE-1F1F3"},
    {"lippu_brittilainen_intian_valtameren_alue", "1F1EE-1F1F4"},
    {"lippu_irak", "1F1EE-1F1F6"},
    {"lippu_iran", "1F1EE-1F1F7"},
    {"lippu_islanti", "1F1EE-1F1F8"},
    {"lippu_italia", "1F1EE-1F1F9"},
    {"lippu_jersey", "1F1EF-1F1EA"},
    {"lippu_jamaika", "1F1EF-1F1F2"},
    {"lippu_jordania", "1F1EF-1F1F4"},
    {"lippu_japani", "1F1EF-1F1F5"},
    {"lippu_kenia", "1F1F0-1F1EA"},
    {"lippu_kirgisia", "1F1F0-1F1EC"},
    {"lippu_kambodza", "1F1F0-1F1ED"},
    {"lippu_kiribati", "1F1F0-1F1EE"},
    {"lippu_komorit", "1F1F0-1F1F2"},
    {"lippu_saint_kitts_ja_nevis", "1F1F0-1F1F3"},
    {"lippu_pohjois_korea", "1F1F0-1F1F5"},
    {"lippu_etela_korea", "1F1F0-1F1F7"},
    {"lippu_kuwait", "1F1F0-1F1FC"},
    {"lippu_caymansaaret", "1F1F0-1F1FE"},
    {"lippu_kazakstan", "1F1F0-1F1FF"},
    {"lippu_laos", "1F1F1-1F1E6"},
    {"lippu_libanon", "1F1F1-1F1E7"},
    {"lippu_saint_lucia", "1F1F1-1F1E8"},
    {"lippu_liechtenstein", "1F1F1-1F1EE"},
    {"lippu_sri_lanka", "1F1F1-1F1F0"},
    {"lippu_liberia", "1F1F1-1F1F7"},
    {"lippu_lesotho", "1F1F1-1F1F8"},
    {"lippu_liettua", "1F1F1-1F1F9"},
    {"lippu_luxemburg", "1F1F1-1F1FA"},
    {"lippu_latvia", "1F1F1-1F1FB"},
    {"lippu_libya", "1F1F1-1F1FE"},
    {"lippu_marokko", "1F1F2-1F1E6"},
    {"lippu_monaco", "1F1F2-1F1E8"},
    {"lippu_moldova", "1F1F2-1F1E9"},
    {"lippu_montenegro", "1F1F2-1F1EA"},
    {"lippu_saint_martin", "1F1F2-1F1EB"},
    {"lippu_madagaskar", "1F1F2-1F1EC"},
    {"lippu_marshallinsaaret", "1F1F2-1F1ED"},
    {"lippu_pohjois_makedonia", "1F1F2-1F1F0"},
    {"lippu_mali", "1F1F2-1F1F1"},
    {"lippu_myanmar_burma", "1F1F2-1F1F2"},
    {"lippu_mongolia", "1F1F2-1F1F3"},
    {"lippu_macao_kiinan_erityishallintoalue", "1F1F2-1F1F4"},
    {"lippu_pohjois_mariaanit", "1F1F2-1F1F5"},
    {"lippu_martinique", "1F1F2-1F1F6"},
    {"lippu_mauritania", "1F1F2-1F1F7"},
    {"lippu_montserrat", "1F1F2-1F1F8"},
    {"lippu_malta", "1F1F2-1F1F9"},
    {"lippu_mauritius", "1F1F2-1F1FA"},
    {"lippu_malediivit", "1F1F2-1F1FB"},
    {"lippu_malawi", "1F1F2-1F1FC"},
    {"lippu_meksiko", "1F1F2-1F1FD"},
    {"lippu_malesia", "1F1F2-1F1FE"},
    {"lippu_mosambik", "1F1F2-1F1FF"},
    {"lippu_namibia", "1F1F3-1F1E6"},
    {"lippu_uusi_kaledonia", "1F1F3-1F1E8"},
    {"lippu_niger", "1F1F3-1F1EA"},
    {"lippu_norfolkinsaari", "1F1F3-1F1EB"},
    {"lippu_nigeria", "1F1F3-1F1EC"},
    {"lippu_nicaragua", "1F1F3-1F1EE"},
    {"lippu_alankomaat", "1F1F3-1F1F1"},
    {"lippu_norja", "1F1F3-1F1F4"},
    {"lippu_nepal", "1F1F3-1F1F5"},
    {"lippu_nauru", "1F1F3-1F1F7"},
    {"lippu_niue", "1F1F3-1F1FA"},
    {"lippu_uusi_seelanti", "1F1F3-1F1FF"},
    {"lippu_oman", "1F1F4-1F1F2"},
    {"lippu_panama", "1F1F5-1F1E6"},
    {"lippu_peru", "1F1F5-1F1EA"},
    {"lippu_ranskan_polynesia", "1F1F5-1F1EB"},
    {"lippu_papua_uusi_guinea", "1F1F5-1F1EC"},
    {"lippu_filippiinit", "1F1F5-1F1ED"},
    {"lippu_pakistan", "1F1F5-1F1F0"},
    {"lippu_puola", "1F1F5-1F1F1"},
    {"lippu_saint_pierre_ja_miquelon", "1F1F5-1F1F2"},
    {"lippu_pitcairn", "1F1F5-1F1F3"},
    {"lippu_puerto_rico", "1F1F5-1F1F7"},
    {"lippu_palestiinalaisalue", "1F1F5-1F1F8"},
    {"lippu_portugali", "1F1F5-1F1F9"},
    {"lippu_palau", "1F1F5-1F1FC"},
    {"lippu_paraguay", "1F1F5-1F1FE"},
    {"lippu_qatar", "1F1F6-1F1E6"},
    {"lippu_reunion", "1F1F7-1F1EA"},
    {"lippu_romania", "1F1F7-1F1F4"},
    {"lippu_serbia", "1F1F7-1F1F8"},
    {"lippu_venaja", "1F1F7-1F1FA"},
    {"lippu_ruanda", "1F1F7-1F1FC"},
    {"lippu_saudi_arabia", "1F1F8-1F1E6"},
    {"lippu_salomonsaaret", "1F1F8-1F1E7"},
    {"lippu_seychellit", "1F1F8-1F1E8"},
    {"lippu_sudan", "1F1F8-1F1E9"},
    {"lippu_ruotsi", "1F1F8-1F1EA"},
    {"lippu_singapore", "1F1F8-1F1EC"},
    {"lippu_saint_helena", "1F1F8-1F1ED"},
    {"lippu_slovenia", "1F1F8-1F1EE"},
    {"lippu_huippuvuoret_ja_jan_mayen", "1F1F8-1F1EF"},
    {"lippu_slovakia", "1F1F8-1F1F0"},
    {"lippu_sierra_leone", "1F1F8-1F1F1"},
    {"lippu_san_marino", "1F1F8-1F1F2"},
    {"lippu_senegal", "1F1F8-1F1F3"},
    {"lippu_somalia", "1F1F8-1F1F4"},
    {"lippu_suriname", "1F1F8-1F1F7"},
    {"lippu_etela_sudan", "1F1F8-1F1F8"},
    {"lippu_sao_tome_ja_principe", "1F1F8-1F1F9"},
    {"lippu_el_salvador", "1F1F8-1F1FB"},
    {"lippu_sint_maarten", "1F1F8-1F1FD"},
    {"lippu_syyria", "1F1F8-1F1FE"},
    {"lippu_eswatini", "1F1F8-1F1FF"},
    {"lippu_tristan_da_cunha", "1F1F9-1F1E6"},
    {"lippu_turks_ja_caicossaaret", "1F1F9-1F1E8"},
    {"lippu_tsad", "1F1F9-1F1E9"},
    {"lippu_ranskan_etelaiset_ja_antarktiset_alueet", "1F1F9-1F1EB"},
    {"lippu_togo", "1F1F9-1F1EC"},
    {"lippu_thaimaa", "1F1F9-1F1ED"},
    {"lippu_tadzikistan", "1F1F9-1F1EF"},
    {"lippu_tokelau", "1F1F9-1F1F0"},
    {"lippu_ita_timor", "1F1F9-1F1F1"},
    {"lippu_turkmenistan", "1F1F9-1F1F2"},
    {"lippu_tunisia", "1F1F9-1F1F3"},
    {"lippu_tonga", "1F1F9-1F1F4"},
    {"lippu_turkki", "1F1F9-1F1F7"},
    {"lippu_trinidad_ja_tobago", "1F1F9-1F1F9"},
    {"lippu_tuvalu", "1F1F9-1F1FB"},
    {"lippu_taiwan", "1F1F9-1F1FC"},
    {"lippu_tansania", "1F1F9-1F1FF"},
    {"lippu_ukraina", "1F1FA-1F1E6"},
    {"lippu_uganda", "1F1FA-1F1EC"},
    {"lippu_yhdysvaltain_erillissaaret", "1F1FA-1F1F2"},
    {"lippu_yhdistyneet_kansakunnat", "1F1FA-1F1F3"},
    {"lippu_yhdysvallat", "1F1FA-1F1F8"},
    {"lippu_uruguay", "1F1FA-1F1FE"},
    {"lippu_uzbekistan", "1F1FA-1F1FF"},
    {"lippu_vatikaani", "1F1FB-1F1E6"},
    {"lippu_saint_vincent_ja_grenadiinit", "1F1FB-1F1E8"},
    {"lippu_venezuela", "1F1FB-1F1EA"},
    {"lippu_brittilaiset_neitsytsaaret", "1F1FB-1F1EC"},
    {"lippu_yhdysvaltain_neitsytsaaret", "1F1FB-1F1EE"},
    {"lippu_vietnam", "1F1FB-1F1F3"},
    {"lippu_vanuatu", "1F1FB-1F1FA"},
    {"lippu_wallis_ja_futuna", "1F1FC-1F1EB"},
    {"lippu_samoa", "1F1FC-1F1F8"},
    {"lippu_kosovo", "1F1FD-1F1F0"},
    {"lippu_jemen", "1F1FE-1F1EA"},
    {"lippu_mayotte", "1F1FE-1F1F9"},
    {"lippu_etela_afrikka", "1F1FF-1F1E6"},
    {"lippu_sambia", "1F1FF-1F1F2"},
    {"lippu_zimbabwe", "1F1FF-1F1FC"},
    {"lippu_englanti", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"lippu_skotlanti", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"lippu_wales", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"pariskunta_ja_sydan_mies_mies", "1F468-200D-2764-FE0F-200D-1F468"},
    {"pariskunta_ja_sydan_mies_mies_vari1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydan_mies_mies_vari1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydan_mies_mies_vari1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydan_mies_mies_vari1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydan_mies_mies_vari1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydan_mies_mies_vari2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydan_mies_mies_vari2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydan_mies_mies_vari2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydan_mies_mies_vari2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydan_mies_mies_vari2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydan_mies_mies_vari3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydan_mies_mies_vari3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydan_mies_mies_vari3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydan_mies_mies_vari3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydan_mies_mies_vari3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydan_mies_mies_vari4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydan_mies_mies_vari4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydan_mies_mies_vari4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydan_mies_mies_vari4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydan_mies_mies_vari4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydan_mies_mies_vari5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydan_mies_mies_vari5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydan_mies_mies_vari5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydan_mies_mies_vari5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydan_mies_mies_vari5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pusu_mies_mies", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"pusu_mies_mies_vari1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pusu_mies_mies_vari1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pusu_mies_mies_vari1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pusu_mies_mies_vari1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pusu_mies_mies_vari1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pusu_mies_mies_vari2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pusu_mies_mies_vari2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pusu_mies_mies_vari2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pusu_mies_mies_vari2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pusu_mies_mies_vari2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pusu_mies_mies_vari3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pusu_mies_mies_vari3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pusu_mies_mies_vari3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pusu_mies_mies_vari3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pusu_mies_mies_vari3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pusu_mies_mies_vari4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pusu_mies_mies_vari4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pusu_mies_mies_vari4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pusu_mies_mies_vari4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pusu_mies_mies_vari4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pusu_mies_mies_vari5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pusu_mies_mies_vari5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pusu_mies_mies_vari5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pusu_mies_mies_vari5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pusu_mies_mies_vari5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"perhe_mies_poika", "1F468-200D-1F466"},
    {"perhe_mies_poika_poika", "1F468-200D-1F466-200D-1F466"},
    {"perhe_mies_tytto", "1F468-200D-1F467"},
    {"perhe_mies_tytto_poika", "1F468-200D-1F467-200D-1F466"},
    {"perhe_mies_tytto_tytto", "1F468-200D-1F467-200D-1F467"},
    {"perhe_mies_mies_poika", "1F468-200D-1F468-200D-1F466"},
    {"perhe_mies_mies_poika_poika", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"perhe_mies_mies_tytto", "1F468-200D-1F468-200D-1F467"},
    {"perhe_mies_mies_tytto_poika", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"perhe_mies_mies_tytto_tytto", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"perhe_mies_nainen_poika", "1F468-200D-1F469-200D-1F466"},
    {"perhe_mies_nainen_poika_poika", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"perhe_mies_nainen_tytto", "1F468-200D-1F469-200D-1F467"},
    {"perhe_mies_nainen_tytto_poika", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"perhe_mies_nainen_tytto_tytto", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"pariskunta_ja_sydan_nainen_mies", "1F469-200D-2764-FE0F-200D-1F468"},
    {"pariskunta_ja_sydan_nainen_mies_vari1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydan_nainen_mies_vari1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydan_nainen_mies_vari1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydan_nainen_mies_vari1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydan_nainen_mies_vari1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydan_nainen_mies_vari2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydan_nainen_mies_vari2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydan_nainen_mies_vari2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydan_nainen_mies_vari2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydan_nainen_mies_vari2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydan_nainen_mies_vari3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydan_nainen_mies_vari3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydan_nainen_mies_vari3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydan_nainen_mies_vari3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydan_nainen_mies_vari3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydan_nainen_mies_vari4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydan_nainen_mies_vari4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydan_nainen_mies_vari4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydan_nainen_mies_vari4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydan_nainen_mies_vari4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydan_nainen_mies_vari5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydan_nainen_mies_vari5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydan_nainen_mies_vari5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydan_nainen_mies_vari5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydan_nainen_mies_vari5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydan_nainen_nainen", "1F469-200D-2764-FE0F-200D-1F469"},
    {"pariskunta_ja_sydan_nainen_nainen_vari1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pariskunta_ja_sydan_nainen_nainen_vari1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pariskunta_ja_sydan_nainen_nainen_vari1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pariskunta_ja_sydan_nainen_nainen_vari1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pariskunta_ja_sydan_nainen_nainen_vari1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pariskunta_ja_sydan_nainen_nainen_vari2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pariskunta_ja_sydan_nainen_nainen_vari2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pariskunta_ja_sydan_nainen_nainen_vari2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pariskunta_ja_sydan_nainen_nainen_vari2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pariskunta_ja_sydan_nainen_nainen_vari2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pariskunta_ja_sydan_nainen_nainen_vari3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pariskunta_ja_sydan_nainen_nainen_vari3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pariskunta_ja_sydan_nainen_nainen_vari3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pariskunta_ja_sydan_nainen_nainen_vari3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pariskunta_ja_sydan_nainen_nainen_vari3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pariskunta_ja_sydan_nainen_nainen_vari4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pariskunta_ja_sydan_nainen_nainen_vari4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pariskunta_ja_sydan_nainen_nainen_vari4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pariskunta_ja_sydan_nainen_nainen_vari4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pariskunta_ja_sydan_nainen_nainen_vari4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pariskunta_ja_sydan_nainen_nainen_vari5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pariskunta_ja_sydan_nainen_nainen_vari5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pariskunta_ja_sydan_nainen_nainen_vari5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pariskunta_ja_sydan_nainen_nainen_vari5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pariskunta_ja_sydan_nainen_nainen_vari5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pusu_nainen_mies", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"pusu_nainen_mies_vari1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pusu_nainen_mies_vari1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pusu_nainen_mies_vari1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pusu_nainen_mies_vari1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pusu_nainen_mies_vari1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pusu_nainen_mies_vari2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pusu_nainen_mies_vari2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pusu_nainen_mies_vari2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pusu_nainen_mies_vari2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pusu_nainen_mies_vari2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pusu_nainen_mies_vari3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pusu_nainen_mies_vari3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pusu_nainen_mies_vari3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pusu_nainen_mies_vari3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pusu_nainen_mies_vari3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pusu_nainen_mies_vari4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pusu_nainen_mies_vari4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pusu_nainen_mies_vari4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pusu_nainen_mies_vari4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pusu_nainen_mies_vari4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pusu_nainen_mies_vari5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"pusu_nainen_mies_vari5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"pusu_nainen_mies_vari5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"pusu_nainen_mies_vari5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"pusu_nainen_mies_vari5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"pusu_nainen_nainen", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"pusu_nainen_nainen_vari1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pusu_nainen_nainen_vari1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pusu_nainen_nainen_vari1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pusu_nainen_nainen_vari1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pusu_nainen_nainen_vari1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"pusu_nainen_nainen_vari2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pusu_nainen_nainen_vari2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pusu_nainen_nainen_vari2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pusu_nainen_nainen_vari2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pusu_nainen_nainen_vari2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"pusu_nainen_nainen_vari3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pusu_nainen_nainen_vari3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pusu_nainen_nainen_vari3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pusu_nainen_nainen_vari3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pusu_nainen_nainen_vari3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"pusu_nainen_nainen_vari4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pusu_nainen_nainen_vari4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pusu_nainen_nainen_vari4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pusu_nainen_nainen_vari4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pusu_nainen_nainen_vari4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"pusu_nainen_nainen_vari5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"pusu_nainen_nainen_vari5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"pusu_nainen_nainen_vari5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"pusu_nainen_nainen_vari5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"pusu_nainen_nainen_vari5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"perhe_nainen_poika", "1F469-200D-1F466"},
    {"perhe_nainen_poika_poika", "1F469-200D-1F466-200D-1F466"},
    {"perhe_nainen_tytto", "1F469-200D-1F467"},
    {"perhe_nainen_tytto_poika", "1F469-200D-1F467-200D-1F466"},
    {"perhe_nainen_tytto_tytto", "1F469-200D-1F467-200D-1F467"},
    {"perhe_nainen_nainen_poika", "1F469-200D-1F469-200D-1F466"},
    {"perhe_nainen_nainen_poika_poika", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"perhe_nainen_nainen_tytto", "1F469-200D-1F469-200D-1F467"},
    {"perhe_nainen_nainen_tytto_poika", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"perhe_nainen_nainen_tytto_tytto", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"ihmiset_kasi_kadessa", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"ihmiset_kasi_kadessa_vari1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"ihmiset_kasi_kadessa_vari1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"ihmiset_kasi_kadessa_vari1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"ihmiset_kasi_kadessa_vari1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"ihmiset_kasi_kadessa_vari1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"ihmiset_kasi_kadessa_vari2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"ihmiset_kasi_kadessa_vari2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"ihmiset_kasi_kadessa_vari2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"ihmiset_kasi_kadessa_vari2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"ihmiset_kasi_kadessa_vari2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"ihmiset_kasi_kadessa_vari3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"ihmiset_kasi_kadessa_vari3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"ihmiset_kasi_kadessa_vari3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"ihmiset_kasi_kadessa_vari3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"ihmiset_kasi_kadessa_vari3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"ihmiset_kasi_kadessa_vari4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"ihmiset_kasi_kadessa_vari4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"ihmiset_kasi_kadessa_vari4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"ihmiset_kasi_kadessa_vari4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"ihmiset_kasi_kadessa_vari4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"ihmiset_kasi_kadessa_vari5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"ihmiset_kasi_kadessa_vari5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"ihmiset_kasi_kadessa_vari5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"ihmiset_kasi_kadessa_vari5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"ihmiset_kasi_kadessa_vari5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"perhe_aikuinen_aikuinen_lapsi", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"perhe_aikuinen_aikuinen_lapsi_lapsi", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"perhe_aikuinen_lapsi", "1F9D1-200D-1F9D2"},
    {"perhe_aikuinen_lapsi_lapsi", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"juoksija_kohti_oikeaa", "1F3C3-200D-27A1-FE0F"},
    {"juoksija_kohti_oikeaa_vari1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"juoksija_kohti_oikeaa_vari2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"juoksija_kohti_oikeaa_vari3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"juoksija_kohti_oikeaa_vari4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"juoksija_kohti_oikeaa_vari5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"mieshoitotyontekija", "1F468-200D-2695-FE0F"},
    {"mieshoitotyontekija_vari1", "1F468-1F3FB-200D-2695-FE0F"},
    {"mieshoitotyontekija_vari2", "1F468-1F3FC-200D-2695-FE0F"},
    {"mieshoitotyontekija_vari3", "1F468-1F3FD-200D-2695-FE0F"},
    {"mieshoitotyontekija_vari4", "1F468-1F3FE-200D-2695-FE0F"},
    {"mieshoitotyontekija_vari5", "1F468-1F3FF-200D-2695-FE0F"},
    {"miestuomari", "1F468-200D-2696-FE0F"},
    {"miestuomari_vari1", "1F468-1F3FB-200D-2696-FE0F"},
    {"miestuomari_vari2", "1F468-1F3FC-200D-2696-FE0F"},
    {"miestuomari_vari3", "1F468-1F3FD-200D-2696-FE0F"},
    {"miestuomari_vari4", "1F468-1F3FE-200D-2696-FE0F"},
    {"miestuomari_vari5", "1F468-1F3FF-200D-2696-FE0F"},
    {"miespilotti", "1F468-200D-2708-FE0F"},
    {"miespilotti_vari1", "1F468-1F3FB-200D-2708-FE0F"},
    {"miespilotti_vari2", "1F468-1F3FC-200D-2708-FE0F"},
    {"miespilotti_vari3", "1F468-1F3FD-200D-2708-FE0F"},
    {"miespilotti_vari4", "1F468-1F3FE-200D-2708-FE0F"},
    {"miespilotti_vari5", "1F468-1F3FF-200D-2708-FE0F"},
    {"miesmaanviljelija", "1F468-200D-1F33E"},
    {"miesmaanviljelija_vari1", "1F468-1F3FB-200D-1F33E"},
    {"miesmaanviljelija_vari2", "1F468-1F3FC-200D-1F33E"},
    {"miesmaanviljelija_vari3", "1F468-1F3FD-200D-1F33E"},
    {"miesmaanviljelija_vari4", "1F468-1F3FE-200D-1F33E"},
    {"miesmaanviljelija_vari5", "1F468-1F3FF-200D-1F33E"},
    {"mieskokki", "1F468-200D-1F373"},
    {"mieskokki_vari1", "1F468-1F3FB-200D-1F373"},
    {"mieskokki_vari2", "1F468-1F3FC-200D-1F373"},
    {"mieskokki_vari3", "1F468-1F3FD-200D-1F373"},
    {"mieskokki_vari4", "1F468-1F3FE-200D-1F373"},
    {"mieskokki_vari5", "1F468-1F3FF-200D-1F373"},
    {"vauvaa_syottava_mies", "1F468-200D-1F37C"},
    {"vauvaa_syottava_mies_vari1", "1F468-1F3FB-200D-1F37C"},
    {"vauvaa_syottava_mies_vari2", "1F468-1F3FC-200D-1F37C"},
    {"vauvaa_syottava_mies_vari3", "1F468-1F3FD-200D-1F37C"},
    {"vauvaa_syottava_mies_vari4", "1F468-1F3FE-200D-1F37C"},
    {"vauvaa_syottava_mies_vari5", "1F468-1F3FF-200D-1F37C"},
    {"miesopiskelija", "1F468-200D-1F393"},
    {"miesopiskelija_vari1", "1F468-1F3FB-200D-1F393"},
    {"miesopiskelija_vari2", "1F468-1F3FC-200D-1F393"},
    {"miesopiskelija_vari3", "1F468-1F3FD-200D-1F393"},
    {"miesopiskelija_vari4", "1F468-1F3FE-200D-1F393"},
    {"miesopiskelija_vari5", "1F468-1F3FF-200D-1F393"},
    {"mieslaulaja", "1F468-200D-1F3A4"},
    {"mieslaulaja_vari1", "1F468-1F3FB-200D-1F3A4"},
    {"mieslaulaja_vari2", "1F468-1F3FC-200D-1F3A4"},
    {"mieslaulaja_vari3", "1F468-1F3FD-200D-1F3A4"},
    {"mieslaulaja_vari4", "1F468-1F3FE-200D-1F3A4"},
    {"mieslaulaja_vari5", "1F468-1F3FF-200D-1F3A4"},
    {"miestaiteilija", "1F468-200D-1F3A8"},
    {"miestaiteilija_vari1", "1F468-1F3FB-200D-1F3A8"},
    {"miestaiteilija_vari2", "1F468-1F3FC-200D-1F3A8"},
    {"miestaiteilija_vari3", "1F468-1F3FD-200D-1F3A8"},
    {"miestaiteilija_vari4", "1F468-1F3FE-200D-1F3A8"},
    {"miestaiteilija_vari5", "1F468-1F3FF-200D-1F3A8"},
    {"miesopettaja", "1F468-200D-1F3EB"},
    {"miesopettaja_vari1", "1F468-1F3FB-200D-1F3EB"},
    {"miesopettaja_vari2", "1F468-1F3FC-200D-1F3EB"},
    {"miesopettaja_vari3", "1F468-1F3FD-200D-1F3EB"},
    {"miesopettaja_vari4", "1F468-1F3FE-200D-1F3EB"},
    {"miesopettaja_vari5", "1F468-1F3FF-200D-1F3EB"},
    {"miestehdastyontekija", "1F468-200D-1F3ED"},
    {"miestehdastyontekija_vari1", "1F468-1F3FB-200D-1F3ED"},
    {"miestehdastyontekija_vari2", "1F468-1F3FC-200D-1F3ED"},
    {"miestehdastyontekija_vari3", "1F468-1F3FD-200D-1F3ED"},
    {"miestehdastyontekija_vari4", "1F468-1F3FE-200D-1F3ED"},
    {"miestehdastyontekija_vari5", "1F468-1F3FF-200D-1F3ED"},
    {"it_mies", "1F468-200D-1F4BB"},
    {"it_mies_vari1", "1F468-1F3FB-200D-1F4BB"},
    {"it_mies_vari2", "1F468-1F3FC-200D-1F4BB"},
    {"it_mies_vari3", "1F468-1F3FD-200D-1F4BB"},
    {"it_mies_vari4", "1F468-1F3FE-200D-1F4BB"},
    {"it_mies_vari5", "1F468-1F3FF-200D-1F4BB"},
    {"miestoimistotyontekija", "1F468-200D-1F4BC"},
    {"miestoimistotyontekija_vari1", "1F468-1F3FB-200D-1F4BC"},
    {"miestoimistotyontekija_vari2", "1F468-1F3FC-200D-1F4BC"},
    {"miestoimistotyontekija_vari3", "1F468-1F3FD-200D-1F4BC"},
    {"miestoimistotyontekija_vari4", "1F468-1F3FE-200D-1F4BC"},
    {"miestoimistotyontekija_vari5", "1F468-1F3FF-200D-1F4BC"},
    {"miesmekaanikko", "1F468-200D-1F527"},
    {"miesmekaanikko_vari1", "1F468-1F3FB-200D-1F527"},
    {"miesmekaanikko_vari2", "1F468-1F3FC-200D-1F527"},
    {"miesmekaanikko_vari3", "1F468-1F3FD-200D-1F527"},
    {"miesmekaanikko_vari4", "1F468-1F3FE-200D-1F527"},
    {"miesmekaanikko_vari5", "1F468-1F3FF-200D-1F527"},
    {"miestutkija", "1F468-200D-1F52C"},
    {"miestutkija_vari1", "1F468-1F3FB-200D-1F52C"},
    {"miestutkija_vari2", "1F468-1F3FC-200D-1F52C"},
    {"miestutkija_vari3", "1F468-1F3FD-200D-1F52C"},
    {"miestutkija_vari4", "1F468-1F3FE-200D-1F52C"},
    {"miestutkija_vari5", "1F468-1F3FF-200D-1F52C"},
    {"miesastronautti", "1F468-200D-1F680"},
    {"miesastronautti_vari1", "1F468-1F3FB-200D-1F680"},
    {"miesastronautti_vari2", "1F468-1F3FC-200D-1F680"},
    {"miesastronautti_vari3", "1F468-1F3FD-200D-1F680"},
    {"miesastronautti_vari4", "1F468-1F3FE-200D-1F680"},
    {"miesastronautti_vari5", "1F468-1F3FF-200D-1F680"},
    {"palomies", "1F468-200D-1F692"},
    {"palomies_vari1", "1F468-1F3FB-200D-1F692"},
    {"palomies_vari2", "1F468-1F3FC-200D-1F692"},
    {"palomies_vari3", "1F468-1F3FD-200D-1F692"},
    {"palomies_vari4", "1F468-1F3FE-200D-1F692"},
    {"palomies_vari5", "1F468-1F3FF-200D-1F692"},
    {"mies_ja_valkoinen_keppi", "1F468-200D-1F9AF"},
    {"mies_ja_valkoinen_keppi_vari1", "1F468-1F3FB-200D-1F9AF"},
    {"mies_ja_valkoinen_keppi_vari2", "1F468-1F3FC-200D-1F9AF"},
    {"mies_ja_valkoinen_keppi_vari3", "1F468-1F3FD-200D-1F9AF"},
    {"mies_ja_valkoinen_keppi_vari4", "1F468-1F3FE-200D-1F9AF"},
    {"mies_ja_valkoinen_keppi_vari5", "1F468-1F3FF-200D-1F9AF"},
    {"mies_ja_valkoinen_keppi_kohti_oikeaa", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"mies_ja_valkoinen_keppi_kohti_oikeaa_vari1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"mies_ja_valkoinen_keppi_kohti_oikeaa_vari2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"mies_ja_valkoinen_keppi_kohti_oikeaa_vari3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"mies_ja_valkoinen_keppi_kohti_oikeaa_vari4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"mies_ja_valkoinen_keppi_kohti_oikeaa_vari5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"mies_moottoroidussa_pyoratuolissa", "1F468-200D-1F9BC"},
    {"mies_moottoroidussa_pyoratuolissa_vari1", "1F468-1F3FB-200D-1F9BC"},
    {"mies_moottoroidussa_pyoratuolissa_vari2", "1F468-1F3FC-200D-1F9BC"},
    {"mies_moottoroidussa_pyoratuolissa_vari3", "1F468-1F3FD-200D-1F9BC"},
    {"mies_moottoroidussa_pyoratuolissa_vari4", "1F468-1F3FE-200D-1F9BC"},
    {"mies_moottoroidussa_pyoratuolissa_vari5", "1F468-1F3FF-200D-1F9BC"},
    {"mies_moottoroidussa_pyoratuolissa_kohti_oikeaa", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"mies_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"mies_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"mies_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"mies_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"mies_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mies_kasikayttoisessa_pyoratuolissa", "1F468-200D-1F9BD"},
    {"mies_kasikayttoisessa_pyoratuolissa_vari1", "1F468-1F3FB-200D-1F9BD"},
    {"mies_kasikayttoisessa_pyoratuolissa_vari2", "1F468-1F3FC-200D-1F9BD"},
    {"mies_kasikayttoisessa_pyoratuolissa_vari3", "1F468-1F3FD-200D-1F9BD"},
    {"mies_kasikayttoisessa_pyoratuolissa_vari4", "1F468-1F3FE-200D-1F9BD"},
    {"mies_kasikayttoisessa_pyoratuolissa_vari5", "1F468-1F3FF-200D-1F9BD"},
    {"mies_kasikayttoisessa_pyoratuolissa_kohti_oikeaa", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"mies_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"mies_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"mies_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"mies_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"mies_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"naishoitotyontekija", "1F469-200D-2695-FE0F"},
    {"naishoitotyontekija_vari1", "1F469-1F3FB-200D-2695-FE0F"},
    {"naishoitotyontekija_vari2", "1F469-1F3FC-200D-2695-FE0F"},
    {"naishoitotyontekija_vari3", "1F469-1F3FD-200D-2695-FE0F"},
    {"naishoitotyontekija_vari4", "1F469-1F3FE-200D-2695-FE0F"},
    {"naishoitotyontekija_vari5", "1F469-1F3FF-200D-2695-FE0F"},
    {"naistuomari", "1F469-200D-2696-FE0F"},
    {"naistuomari_vari1", "1F469-1F3FB-200D-2696-FE0F"},
    {"naistuomari_vari2", "1F469-1F3FC-200D-2696-FE0F"},
    {"naistuomari_vari3", "1F469-1F3FD-200D-2696-FE0F"},
    {"naistuomari_vari4", "1F469-1F3FE-200D-2696-FE0F"},
    {"naistuomari_vari5", "1F469-1F3FF-200D-2696-FE0F"},
    {"naispilotti", "1F469-200D-2708-FE0F"},
    {"naispilotti_vari1", "1F469-1F3FB-200D-2708-FE0F"},
    {"naispilotti_vari2", "1F469-1F3FC-200D-2708-FE0F"},
    {"naispilotti_vari3", "1F469-1F3FD-200D-2708-FE0F"},
    {"naispilotti_vari4", "1F469-1F3FE-200D-2708-FE0F"},
    {"naispilotti_vari5", "1F469-1F3FF-200D-2708-FE0F"},
    {"naismaanviljelija", "1F469-200D-1F33E"},
    {"naismaanviljelija_vari1", "1F469-1F3FB-200D-1F33E"},
    {"naismaanviljelija_vari2", "1F469-1F3FC-200D-1F33E"},
    {"naismaanviljelija_vari3", "1F469-1F3FD-200D-1F33E"},
    {"naismaanviljelija_vari4", "1F469-1F3FE-200D-1F33E"},
    {"naismaanviljelija_vari5", "1F469-1F3FF-200D-1F33E"},
    {"naiskokki", "1F469-200D-1F373"},
    {"naiskokki_vari1", "1F469-1F3FB-200D-1F373"},
    {"naiskokki_vari2", "1F469-1F3FC-200D-1F373"},
    {"naiskokki_vari3", "1F469-1F3FD-200D-1F373"},
    {"naiskokki_vari4", "1F469-1F3FE-200D-1F373"},
    {"naiskokki_vari5", "1F469-1F3FF-200D-1F373"},
    {"vauvaa_syottava_nainen", "1F469-200D-1F37C"},
    {"vauvaa_syottava_nainen_vari1", "1F469-1F3FB-200D-1F37C"},
    {"vauvaa_syottava_nainen_vari2", "1F469-1F3FC-200D-1F37C"},
    {"vauvaa_syottava_nainen_vari3", "1F469-1F3FD-200D-1F37C"},
    {"vauvaa_syottava_nainen_vari4", "1F469-1F3FE-200D-1F37C"},
    {"vauvaa_syottava_nainen_vari5", "1F469-1F3FF-200D-1F37C"},
    {"naisopiskelija", "1F469-200D-1F393"},
    {"naisopiskelija_vari1", "1F469-1F3FB-200D-1F393"},
    {"naisopiskelija_vari2", "1F469-1F3FC-200D-1F393"},
    {"naisopiskelija_vari3", "1F469-1F3FD-200D-1F393"},
    {"naisopiskelija_vari4", "1F469-1F3FE-200D-1F393"},
    {"naisopiskelija_vari5", "1F469-1F3FF-200D-1F393"},
    {"naislaulaja", "1F469-200D-1F3A4"},
    {"naislaulaja_vari1", "1F469-1F3FB-200D-1F3A4"},
    {"naislaulaja_vari2", "1F469-1F3FC-200D-1F3A4"},
    {"naislaulaja_vari3", "1F469-1F3FD-200D-1F3A4"},
    {"naislaulaja_vari4", "1F469-1F3FE-200D-1F3A4"},
    {"naislaulaja_vari5", "1F469-1F3FF-200D-1F3A4"},
    {"naistaiteilija", "1F469-200D-1F3A8"},
    {"naistaiteilija_vari1", "1F469-1F3FB-200D-1F3A8"},
    {"naistaiteilija_vari2", "1F469-1F3FC-200D-1F3A8"},
    {"naistaiteilija_vari3", "1F469-1F3FD-200D-1F3A8"},
    {"naistaiteilija_vari4", "1F469-1F3FE-200D-1F3A8"},
    {"naistaiteilija_vari5", "1F469-1F3FF-200D-1F3A8"},
    {"naisopettaja", "1F469-200D-1F3EB"},
    {"naisopettaja_vari1", "1F469-1F3FB-200D-1F3EB"},
    {"naisopettaja_vari2", "1F469-1F3FC-200D-1F3EB"},
    {"naisopettaja_vari3", "1F469-1F3FD-200D-1F3EB"},
    {"naisopettaja_vari4", "1F469-1F3FE-200D-1F3EB"},
    {"naisopettaja_vari5", "1F469-1F3FF-200D-1F3EB"},
    {"naistehdastyontekija", "1F469-200D-1F3ED"},
    {"naistehdastyontekija_vari1", "1F469-1F3FB-200D-1F3ED"},
    {"naistehdastyontekija_vari2", "1F469-1F3FC-200D-1F3ED"},
    {"naistehdastyontekija_vari3", "1F469-1F3FD-200D-1F3ED"},
    {"naistehdastyontekija_vari4", "1F469-1F3FE-200D-1F3ED"},
    {"naistehdastyontekija_vari5", "1F469-1F3FF-200D-1F3ED"},
    {"it_nainen", "1F469-200D-1F4BB"},
    {"it_nainen_vari1", "1F469-1F3FB-200D-1F4BB"},
    {"it_nainen_vari2", "1F469-1F3FC-200D-1F4BB"},
    {"it_nainen_vari3", "1F469-1F3FD-200D-1F4BB"},
    {"it_nainen_vari4", "1F469-1F3FE-200D-1F4BB"},
    {"it_nainen_vari5", "1F469-1F3FF-200D-1F4BB"},
    {"naistoimistotyontekija", "1F469-200D-1F4BC"},
    {"naistoimistotyontekija_vari1", "1F469-1F3FB-200D-1F4BC"},
    {"naistoimistotyontekija_vari2", "1F469-1F3FC-200D-1F4BC"},
    {"naistoimistotyontekija_vari3", "1F469-1F3FD-200D-1F4BC"},
    {"naistoimistotyontekija_vari4", "1F469-1F3FE-200D-1F4BC"},
    {"naistoimistotyontekija_vari5", "1F469-1F3FF-200D-1F4BC"},
    {"naismekaanikko", "1F469-200D-1F527"},
    {"naismekaanikko_vari1", "1F469-1F3FB-200D-1F527"},
    {"naismekaanikko_vari2", "1F469-1F3FC-200D-1F527"},
    {"naismekaanikko_vari3", "1F469-1F3FD-200D-1F527"},
    {"naismekaanikko_vari4", "1F469-1F3FE-200D-1F527"},
    {"naismekaanikko_vari5", "1F469-1F3FF-200D-1F527"},
    {"naistutkija", "1F469-200D-1F52C"},
    {"naistutkija_vari1", "1F469-1F3FB-200D-1F52C"},
    {"naistutkija_vari2", "1F469-1F3FC-200D-1F52C"},
    {"naistutkija_vari3", "1F469-1F3FD-200D-1F52C"},
    {"naistutkija_vari4", "1F469-1F3FE-200D-1F52C"},
    {"naistutkija_vari5", "1F469-1F3FF-200D-1F52C"},
    {"naisastronautti", "1F469-200D-1F680"},
    {"naisastronautti_vari1", "1F469-1F3FB-200D-1F680"},
    {"naisastronautti_vari2", "1F469-1F3FC-200D-1F680"},
    {"naisastronautti_vari3", "1F469-1F3FD-200D-1F680"},
    {"naisastronautti_vari4", "1F469-1F3FE-200D-1F680"},
    {"naisastronautti_vari5", "1F469-1F3FF-200D-1F680"},
    {"palonainen", "1F469-200D-1F692"},
    {"palonainen_vari1", "1F469-1F3FB-200D-1F692"},
    {"palonainen_vari2", "1F469-1F3FC-200D-1F692"},
    {"palonainen_vari3", "1F469-1F3FD-200D-1F692"},
    {"palonainen_vari4", "1F469-1F3FE-200D-1F692"},
    {"palonainen_vari5", "1F469-1F3FF-200D-1F692"},
    {"nainen_ja_valkoinen_keppi", "1F469-200D-1F9AF"},
    {"nainen_ja_valkoinen_keppi_vari1", "1F469-1F3FB-200D-1F9AF"},
    {"nainen_ja_valkoinen_keppi_vari2", "1F469-1F3FC-200D-1F9AF"},
    {"nainen_ja_valkoinen_keppi_vari3", "1F469-1F3FD-200D-1F9AF"},
    {"nainen_ja_valkoinen_keppi_vari4", "1F469-1F3FE-200D-1F9AF"},
    {"nainen_ja_valkoinen_keppi_vari5", "1F469-1F3FF-200D-1F9AF"},
    {"nainen_ja_valkoinen_keppi_kohti_oikeaa", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"nainen_ja_valkoinen_keppi_kohti_oikeaa_vari1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"nainen_ja_valkoinen_keppi_kohti_oikeaa_vari2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"nainen_ja_valkoinen_keppi_kohti_oikeaa_vari3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"nainen_ja_valkoinen_keppi_kohti_oikeaa_vari4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"nainen_ja_valkoinen_keppi_kohti_oikeaa_vari5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"nainen_moottoroidussa_pyoratuolissa", "1F469-200D-1F9BC"},
    {"nainen_moottoroidussa_pyoratuolissa_vari1", "1F469-1F3FB-200D-1F9BC"},
    {"nainen_moottoroidussa_pyoratuolissa_vari2", "1F469-1F3FC-200D-1F9BC"},
    {"nainen_moottoroidussa_pyoratuolissa_vari3", "1F469-1F3FD-200D-1F9BC"},
    {"nainen_moottoroidussa_pyoratuolissa_vari4", "1F469-1F3FE-200D-1F9BC"},
    {"nainen_moottoroidussa_pyoratuolissa_vari5", "1F469-1F3FF-200D-1F9BC"},
    {"nainen_moottoroidussa_pyoratuolissa_kohti_oikeaa", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"nainen_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"nainen_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"nainen_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"nainen_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"nainen_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"nainen_kasikayttoisessa_pyoratuolissa", "1F469-200D-1F9BD"},
    {"nainen_kasikayttoisessa_pyoratuolissa_vari1", "1F469-1F3FB-200D-1F9BD"},
    {"nainen_kasikayttoisessa_pyoratuolissa_vari2", "1F469-1F3FC-200D-1F9BD"},
    {"nainen_kasikayttoisessa_pyoratuolissa_vari3", "1F469-1F3FD-200D-1F9BD"},
    {"nainen_kasikayttoisessa_pyoratuolissa_vari4", "1F469-1F3FE-200D-1F9BD"},
    {"nainen_kasikayttoisessa_pyoratuolissa_vari5", "1F469-1F3FF-200D-1F9BD"},
    {"nainen_kasikayttoisessa_pyoratuolissa_kohti_oikeaa", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"nainen_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"nainen_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"nainen_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"nainen_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"nainen_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"jalankulkija_kohti_oikeaa", "1F6B6-200D-27A1-FE0F"},
    {"jalankulkija_kohti_oikeaa_vari1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"jalankulkija_kohti_oikeaa_vari2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"jalankulkija_kohti_oikeaa_vari3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"jalankulkija_kohti_oikeaa_vari4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"jalankulkija_kohti_oikeaa_vari5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"polvistuva_ihminen_kohti_oikeaa", "1F9CE-200D-27A1-FE0F"},
    {"polvistuva_ihminen_kohti_oikeaa_vari1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"polvistuva_ihminen_kohti_oikeaa_vari2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"polvistuva_ihminen_kohti_oikeaa_vari3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"polvistuva_ihminen_kohti_oikeaa_vari4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"polvistuva_ihminen_kohti_oikeaa_vari5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"hoitotyontekija", "1F9D1-200D-2695-FE0F"},
    {"hoitotyontekija_vari1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"hoitotyontekija_vari2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"hoitotyontekija_vari3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"hoitotyontekija_vari4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"hoitotyontekija_vari5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"tuomari", "1F9D1-200D-2696-FE0F"},
    {"tuomari_vari1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"tuomari_vari2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"tuomari_vari3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"tuomari_vari4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"tuomari_vari5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"pilotti", "1F9D1-200D-2708-FE0F"},
    {"pilotti_vari1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"pilotti_vari2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"pilotti_vari3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"pilotti_vari4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"pilotti_vari5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"maanviljelija", "1F9D1-200D-1F33E"},
    {"maanviljelija_vari1", "1F9D1-1F3FB-200D-1F33E"},
    {"maanviljelija_vari2", "1F9D1-1F3FC-200D-1F33E"},
    {"maanviljelija_vari3", "1F9D1-1F3FD-200D-1F33E"},
    {"maanviljelija_vari4", "1F9D1-1F3FE-200D-1F33E"},
    {"maanviljelija_vari5", "1F9D1-1F3FF-200D-1F33E"},
    {"kokki", "1F9D1-200D-1F373"},
    {"kokki_vari1", "1F9D1-1F3FB-200D-1F373"},
    {"kokki_vari2", "1F9D1-1F3FC-200D-1F373"},
    {"kokki_vari3", "1F9D1-1F3FD-200D-1F373"},
    {"kokki_vari4", "1F9D1-1F3FE-200D-1F373"},
    {"kokki_vari5", "1F9D1-1F3FF-200D-1F373"},
    {"vauvaa_syottava_henkilo", "1F9D1-200D-1F37C"},
    {"vauvaa_syottava_henkilo_vari1", "1F9D1-1F3FB-200D-1F37C"},
    {"vauvaa_syottava_henkilo_vari2", "1F9D1-1F3FC-200D-1F37C"},
    {"vauvaa_syottava_henkilo_vari3", "1F9D1-1F3FD-200D-1F37C"},
    {"vauvaa_syottava_henkilo_vari4", "1F9D1-1F3FE-200D-1F37C"},
    {"vauvaa_syottava_henkilo_vari5", "1F9D1-1F3FF-200D-1F37C"},
    {"tonttu", "1F9D1-200D-1F384"},
    {"tonttu_vari1", "1F9D1-1F3FB-200D-1F384"},
    {"tonttu_vari2", "1F9D1-1F3FC-200D-1F384"},
    {"tonttu_vari3", "1F9D1-1F3FD-200D-1F384"},
    {"tonttu_vari4", "1F9D1-1F3FE-200D-1F384"},
    {"tonttu_vari5", "1F9D1-1F3FF-200D-1F384"},
    {"opiskelija", "1F9D1-200D-1F393"},
    {"opiskelija_vari1", "1F9D1-1F3FB-200D-1F393"},
    {"opiskelija_vari2", "1F9D1-1F3FC-200D-1F393"},
    {"opiskelija_vari3", "1F9D1-1F3FD-200D-1F393"},
    {"opiskelija_vari4", "1F9D1-1F3FE-200D-1F393"},
    {"opiskelija_vari5", "1F9D1-1F3FF-200D-1F393"},
    {"laulaja", "1F9D1-200D-1F3A4"},
    {"laulaja_vari1", "1F9D1-1F3FB-200D-1F3A4"},
    {"laulaja_vari2", "1F9D1-1F3FC-200D-1F3A4"},
    {"laulaja_vari3", "1F9D1-1F3FD-200D-1F3A4"},
    {"laulaja_vari4", "1F9D1-1F3FE-200D-1F3A4"},
    {"laulaja_vari5", "1F9D1-1F3FF-200D-1F3A4"},
    {"artisti", "1F9D1-200D-1F3A8"},
    {"artisti_vari1", "1F9D1-1F3FB-200D-1F3A8"},
    {"artisti_vari2", "1F9D1-1F3FC-200D-1F3A8"},
    {"artisti_vari3", "1F9D1-1F3FD-200D-1F3A8"},
    {"artisti_vari4", "1F9D1-1F3FE-200D-1F3A8"},
    {"artisti_vari5", "1F9D1-1F3FF-200D-1F3A8"},
    {"opettaja", "1F9D1-200D-1F3EB"},
    {"opettaja_vari1", "1F9D1-1F3FB-200D-1F3EB"},
    {"opettaja_vari2", "1F9D1-1F3FC-200D-1F3EB"},
    {"opettaja_vari3", "1F9D1-1F3FD-200D-1F3EB"},
    {"opettaja_vari4", "1F9D1-1F3FE-200D-1F3EB"},
    {"opettaja_vari5", "1F9D1-1F3FF-200D-1F3EB"},
    {"tehdastyontekija", "1F9D1-200D-1F3ED"},
    {"tehdastyontekija_vari1", "1F9D1-1F3FB-200D-1F3ED"},
    {"tehdastyontekija_vari2", "1F9D1-1F3FC-200D-1F3ED"},
    {"tehdastyontekija_vari3", "1F9D1-1F3FD-200D-1F3ED"},
    {"tehdastyontekija_vari4", "1F9D1-1F3FE-200D-1F3ED"},
    {"tehdastyontekija_vari5", "1F9D1-1F3FF-200D-1F3ED"},
    {"it_tyontekija", "1F9D1-200D-1F4BB"},
    {"it_tyontekija_vari1", "1F9D1-1F3FB-200D-1F4BB"},
    {"it_tyontekija_vari2", "1F9D1-1F3FC-200D-1F4BB"},
    {"it_tyontekija_vari3", "1F9D1-1F3FD-200D-1F4BB"},
    {"it_tyontekija_vari4", "1F9D1-1F3FE-200D-1F4BB"},
    {"it_tyontekija_vari5", "1F9D1-1F3FF-200D-1F4BB"},
    {"toimistotyontekija", "1F9D1-200D-1F4BC"},
    {"toimistotyontekija_vari1", "1F9D1-1F3FB-200D-1F4BC"},
    {"toimistotyontekija_vari2", "1F9D1-1F3FC-200D-1F4BC"},
    {"toimistotyontekija_vari3", "1F9D1-1F3FD-200D-1F4BC"},
    {"toimistotyontekija_vari4", "1F9D1-1F3FE-200D-1F4BC"},
    {"toimistotyontekija_vari5", "1F9D1-1F3FF-200D-1F4BC"},
    {"mekaanikko", "1F9D1-200D-1F527"},
    {"mekaanikko_vari1", "1F9D1-1F3FB-200D-1F527"},
    {"mekaanikko_vari2", "1F9D1-1F3FC-200D-1F527"},
    {"mekaanikko_vari3", "1F9D1-1F3FD-200D-1F527"},
    {"mekaanikko_vari4", "1F9D1-1F3FE-200D-1F527"},
    {"mekaanikko_vari5", "1F9D1-1F3FF-200D-1F527"},
    {"tutkija", "1F9D1-200D-1F52C"},
    {"tutkija_vari1", "1F9D1-1F3FB-200D-1F52C"},
    {"tutkija_vari2", "1F9D1-1F3FC-200D-1F52C"},
    {"tutkija_vari3", "1F9D1-1F3FD-200D-1F52C"},
    {"tutkija_vari4", "1F9D1-1F3FE-200D-1F52C"},
    {"tutkija_vari5", "1F9D1-1F3FF-200D-1F52C"},
    {"astronautti", "1F9D1-200D-1F680"},
    {"astronautti_vari1", "1F9D1-1F3FB-200D-1F680"},
    {"astronautti_vari2", "1F9D1-1F3FC-200D-1F680"},
    {"astronautti_vari3", "1F9D1-1F3FD-200D-1F680"},
    {"astronautti_vari4", "1F9D1-1F3FE-200D-1F680"},
    {"astronautti_vari5", "1F9D1-1F3FF-200D-1F680"},
    {"pelastaja", "1F9D1-200D-1F692"},
    {"pelastaja_vari1", "1F9D1-1F3FB-200D-1F692"},
    {"pelastaja_vari2", "1F9D1-1F3FC-200D-1F692"},
    {"pelastaja_vari3", "1F9D1-1F3FD-200D-1F692"},
    {"pelastaja_vari4", "1F9D1-1F3FE-200D-1F692"},
    {"pelastaja_vari5", "1F9D1-1F3FF-200D-1F692"},
    {"henkilo_ja_valkoinen_keppi", "1F9D1-200D-1F9AF"},
    {"henkilo_ja_valkoinen_keppi_vari1", "1F9D1-1F3FB-200D-1F9AF"},
    {"henkilo_ja_valkoinen_keppi_vari2", "1F9D1-1F3FC-200D-1F9AF"},
    {"henkilo_ja_valkoinen_keppi_vari3", "1F9D1-1F3FD-200D-1F9AF"},
    {"henkilo_ja_valkoinen_keppi_vari4", "1F9D1-1F3FE-200D-1F9AF"},
    {"henkilo_ja_valkoinen_keppi_vari5", "1F9D1-1F3FF-200D-1F9AF"},
    {"henkilo_ja_valkoinen_keppi_kohti_oikeaa", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"henkilo_ja_valkoinen_keppi_kohti_oikeaa_vari1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"henkilo_ja_valkoinen_keppi_kohti_oikeaa_vari2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"henkilo_ja_valkoinen_keppi_kohti_oikeaa_vari3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"henkilo_ja_valkoinen_keppi_kohti_oikeaa_vari4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"henkilo_ja_valkoinen_keppi_kohti_oikeaa_vari5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"henkilo_moottoroidussa_pyoratuolissa", "1F9D1-200D-1F9BC"},
    {"henkilo_moottoroidussa_pyoratuolissa_vari1", "1F9D1-1F3FB-200D-1F9BC"},
    {"henkilo_moottoroidussa_pyoratuolissa_vari2", "1F9D1-1F3FC-200D-1F9BC"},
    {"henkilo_moottoroidussa_pyoratuolissa_vari3", "1F9D1-1F3FD-200D-1F9BC"},
    {"henkilo_moottoroidussa_pyoratuolissa_vari4", "1F9D1-1F3FE-200D-1F9BC"},
    {"henkilo_moottoroidussa_pyoratuolissa_vari5", "1F9D1-1F3FF-200D-1F9BC"},
    {"henkilo_moottoroidussa_pyoratuolissa_kohti_oikeaa", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"henkilo_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"henkilo_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"henkilo_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"henkilo_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"henkilo_moottoroidussa_pyoratuolissa_kohti_oikeaa_vari5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"henkilo_kasikayttoisessa_pyoratuolissa", "1F9D1-200D-1F9BD"},
    {"henkilo_kasikayttoisessa_pyoratuolissa_vari1", "1F9D1-1F3FB-200D-1F9BD"},
    {"henkilo_kasikayttoisessa_pyoratuolissa_vari2", "1F9D1-1F3FC-200D-1F9BD"},
    {"henkilo_kasikayttoisessa_pyoratuolissa_vari3", "1F9D1-1F3FD-200D-1F9BD"},
    {"henkilo_kasikayttoisessa_pyoratuolissa_vari4", "1F9D1-1F3FE-200D-1F9BD"},
    {"henkilo_kasikayttoisessa_pyoratuolissa_vari5", "1F9D1-1F3FF-200D-1F9BD"},
    {"henkilo_kasikayttoisessa_pyoratuolissa_kohti_oikeaa", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"henkilo_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"henkilo_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"henkilo_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"henkilo_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"henkilo_kasikayttoisessa_pyoratuolissa_kohti_oikeaa_vari5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"pallotteleva_nainen", "26F9-FE0F-200D-2640-FE0F"},
    {"pallotteleva_nainen_vari1", "26F9-1F3FB-200D-2640-FE0F"},
    {"pallotteleva_nainen_vari2", "26F9-1F3FC-200D-2640-FE0F"},
    {"pallotteleva_nainen_vari3", "26F9-1F3FD-200D-2640-FE0F"},
    {"pallotteleva_nainen_vari4", "26F9-1F3FE-200D-2640-FE0F"},
    {"pallotteleva_nainen_vari5", "26F9-1F3FF-200D-2640-FE0F"},
    {"pallotteleva_mies", "26F9-FE0F-200D-2642-FE0F"},
    {"pallotteleva_mies_vari1", "26F9-1F3FB-200D-2642-FE0F"},
    {"pallotteleva_mies_vari2", "26F9-1F3FC-200D-2642-FE0F"},
    {"pallotteleva_mies_vari3", "26F9-1F3FD-200D-2642-FE0F"},
    {"pallotteleva_mies_vari4", "26F9-1F3FE-200D-2642-FE0F"},
    {"pallotteleva_mies_vari5", "26F9-1F3FF-200D-2642-FE0F"},
    {"juokseva_nainen", "1F3C3-200D-2640-FE0F"},
    {"juokseva_nainen_vari1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"juokseva_nainen_vari2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"juokseva_nainen_vari3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"juokseva_nainen_vari4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"juokseva_nainen_vari5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"juokseva_nainen_kohti_oikeaa", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"juokseva_nainen_kohti_oikeaa_vari1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"juokseva_nainen_kohti_oikeaa_vari2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"juokseva_nainen_kohti_oikeaa_vari3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"juokseva_nainen_kohti_oikeaa_vari4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"juokseva_nainen_kohti_oikeaa_vari5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"juokseva_mies", "1F3C3-200D-2642-FE0F"},
    {"juokseva_mies_vari1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"juokseva_mies_vari2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"juokseva_mies_vari3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"juokseva_mies_vari4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"juokseva_mies_vari5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"juokseva_mies_kohti_oikeaa", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"juokseva_mies_kohti_oikeaa_vari1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"juokseva_mies_kohti_oikeaa_vari2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"juokseva_mies_kohti_oikeaa_vari3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"juokseva_mies_kohti_oikeaa_vari4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"juokseva_mies_kohti_oikeaa_vari5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"surffaajanainen", "1F3C4-200D-2640-FE0F"},
    {"surffaajanainen_vari1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"surffaajanainen_vari2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"surffaajanainen_vari3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"surffaajanainen_vari4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"surffaajanainen_vari5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"surffaajamies", "1F3C4-200D-2642-FE0F"},
    {"surffaajamies_vari1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"surffaajamies_vari2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"surffaajamies_vari3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"surffaajamies_vari4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"surffaajamies_vari5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"uiva_nainen", "1F3CA-200D-2640-FE0F"},
    {"uiva_nainen_vari1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"uiva_nainen_vari2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"uiva_nainen_vari3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"uiva_nainen_vari4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"uiva_nainen_vari5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"uiva_mies", "1F3CA-200D-2642-FE0F"},
    {"uiva_mies_vari1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"uiva_mies_vari2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"uiva_mies_vari3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"uiva_mies_vari4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"uiva_mies_vari5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"painonnostajanainen", "1F3CB-FE0F-200D-2640-FE0F"},
    {"painonnostajanainen_vari1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"painonnostajanainen_vari2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"painonnostajanainen_vari3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"painonnostajanainen_vari4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"painonnostajanainen_vari5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"painonnostajamies", "1F3CB-FE0F-200D-2642-FE0F"},
    {"painonnostajamies_vari1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"painonnostajamies_vari2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"painonnostajamies_vari3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"painonnostajamies_vari4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"painonnostajamies_vari5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"golfaajanainen", "1F3CC-FE0F-200D-2640-FE0F"},
    {"golfaajanainen_vari1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"golfaajanainen_vari2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"golfaajanainen_vari3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"golfaajanainen_vari4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"golfaajanainen_vari5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"golfaajamies", "1F3CC-FE0F-200D-2642-FE0F"},
    {"golfaajamies_vari1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"golfaajamies_vari2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"golfaajamies_vari3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"golfaajamies_vari4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"golfaajamies_vari5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"naispoliisi", "1F46E-200D-2640-FE0F"},
    {"naispoliisi_vari1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"naispoliisi_vari2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"naispoliisi_vari3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"naispoliisi_vari4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"naispoliisi_vari5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"miespoliisi", "1F46E-200D-2642-FE0F"},
    {"miespoliisi_vari1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"miespoliisi_vari2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"miespoliisi_vari3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"miespoliisi_vari4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"miespoliisi_vari5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"tanssivat_puputytot", "1F46F-200D-2640-FE0F"},
    {"tanssivat_puputytot_vari1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"tanssivat_puputytot_vari1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"tanssivat_puputytot_vari1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"tanssivat_puputytot_vari1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"tanssivat_puputytot_vari2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"tanssivat_puputytot_vari2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"tanssivat_puputytot_vari2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"tanssivat_puputytot_vari2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"tanssivat_puputytot_vari3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"tanssivat_puputytot_vari3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"tanssivat_puputytot_vari3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"tanssivat_puputytot_vari3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"tanssivat_puputytot_vari4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"tanssivat_puputytot_vari4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"tanssivat_puputytot_vari4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"tanssivat_puputytot_vari4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"tanssivat_puputytot_vari5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"tanssivat_puputytot_vari5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"tanssivat_puputytot_vari5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"tanssivat_puputytot_vari5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"tanssivat_puputytot_vari1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"tanssivat_puputytot_vari2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"tanssivat_puputytot_vari3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"tanssivat_puputytot_vari4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"tanssivat_puputytot_vari5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"tanssivat_pupupojat", "1F46F-200D-2642-FE0F"},
    {"tanssivat_pupupojat_vari1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"tanssivat_pupupojat_vari1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"tanssivat_pupupojat_vari1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"tanssivat_pupupojat_vari1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"tanssivat_pupupojat_vari2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"tanssivat_pupupojat_vari2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"tanssivat_pupupojat_vari2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"tanssivat_pupupojat_vari2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"tanssivat_pupupojat_vari3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"tanssivat_pupupojat_vari3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"tanssivat_pupupojat_vari3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"tanssivat_pupupojat_vari3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"tanssivat_pupupojat_vari4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"tanssivat_pupupojat_vari4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"tanssivat_pupupojat_vari4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"tanssivat_pupupojat_vari4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"tanssivat_pupupojat_vari5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"tanssivat_pupupojat_vari5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"tanssivat_pupupojat_vari5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"tanssivat_pupupojat_vari5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"tanssivat_pupupojat_vari1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"tanssivat_pupupojat_vari2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"tanssivat_pupupojat_vari3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"tanssivat_pupupojat_vari4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"tanssivat_pupupojat_vari5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"huntupainen_nainen", "1F470-200D-2640-FE0F"},
    {"huntupainen_nainen_vari1", "1F470-1F3FB-200D-2640-FE0F"},
    {"huntupainen_nainen_vari2", "1F470-1F3FC-200D-2640-FE0F"},
    {"huntupainen_nainen_vari3", "1F470-1F3FD-200D-2640-FE0F"},
    {"huntupainen_nainen_vari4", "1F470-1F3FE-200D-2640-FE0F"},
    {"huntupainen_nainen_vari5", "1F470-1F3FF-200D-2640-FE0F"},
    {"huntupainen_mies", "1F470-200D-2642-FE0F"},
    {"huntupainen_mies_vari1", "1F470-1F3FB-200D-2642-FE0F"},
    {"huntupainen_mies_vari2", "1F470-1F3FC-200D-2642-FE0F"},
    {"huntupainen_mies_vari3", "1F470-1F3FD-200D-2642-FE0F"},
    {"huntupainen_mies_vari4", "1F470-1F3FE-200D-2642-FE0F"},
    {"huntupainen_mies_vari5", "1F470-1F3FF-200D-2642-FE0F"},
    {"vaaleatukkainen_nainen", "1F471-200D-2640-FE0F"},
    {"vaaleatukkainen_nainen_vari1", "1F471-1F3FB-200D-2640-FE0F"},
    {"vaaleatukkainen_nainen_vari2", "1F471-1F3FC-200D-2640-FE0F"},
    {"vaaleatukkainen_nainen_vari3", "1F471-1F3FD-200D-2640-FE0F"},
    {"vaaleatukkainen_nainen_vari4", "1F471-1F3FE-200D-2640-FE0F"},
    {"vaaleatukkainen_nainen_vari5", "1F471-1F3FF-200D-2640-FE0F"},
    {"vaaleatukkainen_mies", "1F471-200D-2642-FE0F"},
    {"vaaleatukkainen_mies_vari1", "1F471-1F3FB-200D-2642-FE0F"},
    {"vaaleatukkainen_mies_vari2", "1F471-1F3FC-200D-2642-FE0F"},
    {"vaaleatukkainen_mies_vari3", "1F471-1F3FD-200D-2642-FE0F"},
    {"vaaleatukkainen_mies_vari4", "1F471-1F3FE-200D-2642-FE0F"},
    {"vaaleatukkainen_mies_vari5", "1F471-1F3FF-200D-2642-FE0F"},
    {"turbaanipainen_nainen", "1F473-200D-2640-FE0F"},
    {"turbaanipainen_nainen_vari1", "1F473-1F3FB-200D-2640-FE0F"},
    {"turbaanipainen_nainen_vari2", "1F473-1F3FC-200D-2640-FE0F"},
    {"turbaanipainen_nainen_vari3", "1F473-1F3FD-200D-2640-FE0F"},
    {"turbaanipainen_nainen_vari4", "1F473-1F3FE-200D-2640-FE0F"},
    {"turbaanipainen_nainen_vari5", "1F473-1F3FF-200D-2640-FE0F"},
    {"turbaanipainen_mies", "1F473-200D-2642-FE0F"},
    {"turbaanipainen_mies_vari1", "1F473-1F3FB-200D-2642-FE0F"},
    {"turbaanipainen_mies_vari2", "1F473-1F3FC-200D-2642-FE0F"},
    {"turbaanipainen_mies_vari3", "1F473-1F3FD-200D-2642-FE0F"},
    {"turbaanipainen_mies_vari4", "1F473-1F3FE-200D-2642-FE0F"},
    {"turbaanipainen_mies_vari5", "1F473-1F3FF-200D-2642-FE0F"},
    {"rakennusnainen", "1F477-200D-2640-FE0F"},
    {"rakennusnainen_vari1", "1F477-1F3FB-200D-2640-FE0F"},
    {"rakennusnainen_vari2", "1F477-1F3FC-200D-2640-FE0F"},
    {"rakennusnainen_vari3", "1F477-1F3FD-200D-2640-FE0F"},
    {"rakennusnainen_vari4", "1F477-1F3FE-200D-2640-FE0F"},
    {"rakennusnainen_vari5", "1F477-1F3FF-200D-2640-FE0F"},
    {"rakennusmies", "1F477-200D-2642-FE0F"},
    {"rakennusmies_vari1", "1F477-1F3FB-200D-2642-FE0F"},
    {"rakennusmies_vari2", "1F477-1F3FC-200D-2642-FE0F"},
    {"rakennusmies_vari3", "1F477-1F3FD-200D-2642-FE0F"},
    {"rakennusmies_vari4", "1F477-1F3FE-200D-2642-FE0F"},
    {"rakennusmies_vari5", "1F477-1F3FF-200D-2642-FE0F"},
    {"kammenta_ylospain_nayttava_nainen", "1F481-200D-2640-FE0F"},
    {"kammenta_ylospain_nayttava_nainen_vari1", "1F481-1F3FB-200D-2640-FE0F"},
    {"kammenta_ylospain_nayttava_nainen_vari2", "1F481-1F3FC-200D-2640-FE0F"},
    {"kammenta_ylospain_nayttava_nainen_vari3", "1F481-1F3FD-200D-2640-FE0F"},
    {"kammenta_ylospain_nayttava_nainen_vari4", "1F481-1F3FE-200D-2640-FE0F"},
    {"kammenta_ylospain_nayttava_nainen_vari5", "1F481-1F3FF-200D-2640-FE0F"},
    {"kammenta_ylospain_nayttava_mies", "1F481-200D-2642-FE0F"},
    {"kammenta_ylospain_nayttava_mies_vari1", "1F481-1F3FB-200D-2642-FE0F"},
    {"kammenta_ylospain_nayttava_mies_vari2", "1F481-1F3FC-200D-2642-FE0F"},
    {"kammenta_ylospain_nayttava_mies_vari3", "1F481-1F3FD-200D-2642-FE0F"},
    {"kammenta_ylospain_nayttava_mies_vari4", "1F481-1F3FE-200D-2642-FE0F"},
    {"kammenta_ylospain_nayttava_mies_vari5", "1F481-1F3FF-200D-2642-FE0F"},
    {"naisvartija", "1F482-200D-2640-FE0F"},
    {"naisvartija_vari1", "1F482-1F3FB-200D-2640-FE0F"},
    {"naisvartija_vari2", "1F482-1F3FC-200D-2640-FE0F"},
    {"naisvartija_vari3", "1F482-1F3FD-200D-2640-FE0F"},
    {"naisvartija_vari4", "1F482-1F3FE-200D-2640-FE0F"},
    {"naisvartija_vari5", "1F482-1F3FF-200D-2640-FE0F"},
    {"miesvartija", "1F482-200D-2642-FE0F"},
    {"miesvartija_vari1", "1F482-1F3FB-200D-2642-FE0F"},
    {"miesvartija_vari2", "1F482-1F3FC-200D-2642-FE0F"},
    {"miesvartija_vari3", "1F482-1F3FD-200D-2642-FE0F"},
    {"miesvartija_vari4", "1F482-1F3FE-200D-2642-FE0F"},
    {"miesvartija_vari5", "1F482-1F3FF-200D-2642-FE0F"},
    {"hieronnassa_oleva_nainen", "1F486-200D-2640-FE0F"},
    {"hieronnassa_oleva_nainen_vari1", "1F486-1F3FB-200D-2640-FE0F"},
    {"hieronnassa_oleva_nainen_vari2", "1F486-1F3FC-200D-2640-FE0F"},
    {"hieronnassa_oleva_nainen_vari3", "1F486-1F3FD-200D-2640-FE0F"},
    {"hieronnassa_oleva_nainen_vari4", "1F486-1F3FE-200D-2640-FE0F"},
    {"hieronnassa_oleva_nainen_vari5", "1F486-1F3FF-200D-2640-FE0F"},
    {"hieronnassa_oleva_mies", "1F486-200D-2642-FE0F"},
    {"hieronnassa_oleva_mies_vari1", "1F486-1F3FB-200D-2642-FE0F"},
    {"hieronnassa_oleva_mies_vari2", "1F486-1F3FC-200D-2642-FE0F"},
    {"hieronnassa_oleva_mies_vari3", "1F486-1F3FD-200D-2642-FE0F"},
    {"hieronnassa_oleva_mies_vari4", "1F486-1F3FE-200D-2642-FE0F"},
    {"hieronnassa_oleva_mies_vari5", "1F486-1F3FF-200D-2642-FE0F"},
    {"kampaajalla_oleva_nainen", "1F487-200D-2640-FE0F"},
    {"kampaajalla_oleva_nainen_vari1", "1F487-1F3FB-200D-2640-FE0F"},
    {"kampaajalla_oleva_nainen_vari2", "1F487-1F3FC-200D-2640-FE0F"},
    {"kampaajalla_oleva_nainen_vari3", "1F487-1F3FD-200D-2640-FE0F"},
    {"kampaajalla_oleva_nainen_vari4", "1F487-1F3FE-200D-2640-FE0F"},
    {"kampaajalla_oleva_nainen_vari5", "1F487-1F3FF-200D-2640-FE0F"},
    {"parturissa_oleva_mies", "1F487-200D-2642-FE0F"},
    {"parturissa_oleva_mies_vari1", "1F487-1F3FB-200D-2642-FE0F"},
    {"parturissa_oleva_mies_vari2", "1F487-1F3FC-200D-2642-FE0F"},
    {"parturissa_oleva_mies_vari3", "1F487-1F3FD-200D-2642-FE0F"},
    {"parturissa_oleva_mies_vari4", "1F487-1F3FE-200D-2642-FE0F"},
    {"parturissa_oleva_mies_vari5", "1F487-1F3FF-200D-2642-FE0F"},
    {"naisetsiva", "1F575-FE0F-200D-2640-FE0F"},
    {"naisetsiva_vari1", "1F575-1F3FB-200D-2640-FE0F"},
    {"naisetsiva_vari2", "1F575-1F3FC-200D-2640-FE0F"},
    {"naisetsiva_vari3", "1F575-1F3FD-200D-2640-FE0F"},
    {"naisetsiva_vari4", "1F575-1F3FE-200D-2640-FE0F"},
    {"naisetsiva_vari5", "1F575-1F3FF-200D-2640-FE0F"},
    {"miesetsiva", "1F575-FE0F-200D-2642-FE0F"},
    {"miesetsiva_vari1", "1F575-1F3FB-200D-2642-FE0F"},
    {"miesetsiva_vari2", "1F575-1F3FC-200D-2642-FE0F"},
    {"miesetsiva_vari3", "1F575-1F3FD-200D-2642-FE0F"},
    {"miesetsiva_vari4", "1F575-1F3FE-200D-2642-FE0F"},
    {"miesetsiva_vari5", "1F575-1F3FF-200D-2642-FE0F"},
    {"ei_kay_merkkia_nayttava_nainen", "1F645-200D-2640-FE0F"},
    {"ei_kay_merkkia_nayttava_nainen_vari1", "1F645-1F3FB-200D-2640-FE0F"},
    {"ei_kay_merkkia_nayttava_nainen_vari2", "1F645-1F3FC-200D-2640-FE0F"},
    {"ei_kay_merkkia_nayttava_nainen_vari3", "1F645-1F3FD-200D-2640-FE0F"},
    {"ei_kay_merkkia_nayttava_nainen_vari4", "1F645-1F3FE-200D-2640-FE0F"},
    {"ei_kay_merkkia_nayttava_nainen_vari5", "1F645-1F3FF-200D-2640-FE0F"},
    {"ei_kay_merkkia_nayttava_mies", "1F645-200D-2642-FE0F"},
    {"ei_kay_merkkia_nayttava_mies_vari1", "1F645-1F3FB-200D-2642-FE0F"},
    {"ei_kay_merkkia_nayttava_mies_vari2", "1F645-1F3FC-200D-2642-FE0F"},
    {"ei_kay_merkkia_nayttava_mies_vari3", "1F645-1F3FD-200D-2642-FE0F"},
    {"ei_kay_merkkia_nayttava_mies_vari4", "1F645-1F3FE-200D-2642-FE0F"},
    {"ei_kay_merkkia_nayttava_mies_vari5", "1F645-1F3FF-200D-2642-FE0F"},
    {"ok_merkkia_nayttava_nainen", "1F646-200D-2640-FE0F"},
    {"ok_merkkia_nayttava_nainen_vari1", "1F646-1F3FB-200D-2640-FE0F"},
    {"ok_merkkia_nayttava_nainen_vari2", "1F646-1F3FC-200D-2640-FE0F"},
    {"ok_merkkia_nayttava_nainen_vari3", "1F646-1F3FD-200D-2640-FE0F"},
    {"ok_merkkia_nayttava_nainen_vari4", "1F646-1F3FE-200D-2640-FE0F"},
    {"ok_merkkia_nayttava_nainen_vari5", "1F646-1F3FF-200D-2640-FE0F"},
    {"ok_merkkia_nayttava_mies", "1F646-200D-2642-FE0F"},
    {"ok_merkkia_nayttava_mies_vari1", "1F646-1F3FB-200D-2642-FE0F"},
    {"ok_merkkia_nayttava_mies_vari2", "1F646-1F3FC-200D-2642-FE0F"},
    {"ok_merkkia_nayttava_mies_vari3", "1F646-1F3FD-200D-2642-FE0F"},
    {"ok_merkkia_nayttava_mies_vari4", "1F646-1F3FE-200D-2642-FE0F"},
    {"ok_merkkia_nayttava_mies_vari5", "1F646-1F3FF-200D-2642-FE0F"},
    {"syvaan_kumartava_nainen", "1F647-200D-2640-FE0F"},
    {"syvaan_kumartava_nainen_vari1", "1F647-1F3FB-200D-2640-FE0F"},
    {"syvaan_kumartava_nainen_vari2", "1F647-1F3FC-200D-2640-FE0F"},
    {"syvaan_kumartava_nainen_vari3", "1F647-1F3FD-200D-2640-FE0F"},
    {"syvaan_kumartava_nainen_vari4", "1F647-1F3FE-200D-2640-FE0F"},
    {"syvaan_kumartava_nainen_vari5", "1F647-1F3FF-200D-2640-FE0F"},
    {"syvaan_kumartava_mies", "1F647-200D-2642-FE0F"},
    {"syvaan_kumartava_mies_vari1", "1F647-1F3FB-200D-2642-FE0F"},
    {"syvaan_kumartava_mies_vari2", "1F647-1F3FC-200D-2642-FE0F"},
    {"syvaan_kumartava_mies_vari3", "1F647-1F3FD-200D-2642-FE0F"},
    {"syvaan_kumartava_mies_vari4", "1F647-1F3FE-200D-2642-FE0F"},
    {"syvaan_kumartava_mies_vari5", "1F647-1F3FF-200D-2642-FE0F"},
    {"viittaava_nainen", "1F64B-200D-2640-FE0F"},
    {"viittaava_nainen_vari1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"viittaava_nainen_vari2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"viittaava_nainen_vari3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"viittaava_nainen_vari4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"viittaava_nainen_vari5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"viittaava_mies", "1F64B-200D-2642-FE0F"},
    {"viittaava_mies_vari1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"viittaava_mies_vari2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"viittaava_mies_vari3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"viittaava_mies_vari4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"viittaava_mies_vari5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"surullinen_nainen", "1F64D-200D-2640-FE0F"},
    {"surullinen_nainen_vari1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"surullinen_nainen_vari2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"surullinen_nainen_vari3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"surullinen_nainen_vari4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"surullinen_nainen_vari5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"surullinen_mies", "1F64D-200D-2642-FE0F"},
    {"surullinen_mies_vari1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"surullinen_mies_vari2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"surullinen_mies_vari3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"surullinen_mies_vari4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"surullinen_mies_vari5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"mokottava_nainen", "1F64E-200D-2640-FE0F"},
    {"mokottava_nainen_vari1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"mokottava_nainen_vari2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"mokottava_nainen_vari3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"mokottava_nainen_vari4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"mokottava_nainen_vari5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"mokottava_mies", "1F64E-200D-2642-FE0F"},
    {"mokottava_mies_vari1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"mokottava_mies_vari2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"mokottava_mies_vari3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"mokottava_mies_vari4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"mokottava_mies_vari5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"soutava_nainen", "1F6A3-200D-2640-FE0F"},
    {"soutava_nainen_vari1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"soutava_nainen_vari2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"soutava_nainen_vari3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"soutava_nainen_vari4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"soutava_nainen_vari5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"soutava_mies", "1F6A3-200D-2642-FE0F"},
    {"soutava_mies_vari1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"soutava_mies_vari2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"soutava_mies_vari3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"soutava_mies_vari4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"soutava_mies_vari5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"pyoraileva_nainen", "1F6B4-200D-2640-FE0F"},
    {"pyoraileva_nainen_vari1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"pyoraileva_nainen_vari2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"pyoraileva_nainen_vari3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"pyoraileva_nainen_vari4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"pyoraileva_nainen_vari5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"pyoraileva_mies", "1F6B4-200D-2642-FE0F"},
    {"pyoraileva_mies_vari1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"pyoraileva_mies_vari2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"pyoraileva_mies_vari3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"pyoraileva_mies_vari4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"pyoraileva_mies_vari5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"maastopyoraileva_nainen", "1F6B5-200D-2640-FE0F"},
    {"maastopyoraileva_nainen_vari1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"maastopyoraileva_nainen_vari2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"maastopyoraileva_nainen_vari3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"maastopyoraileva_nainen_vari4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"maastopyoraileva_nainen_vari5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"maastopyoraileva_mies", "1F6B5-200D-2642-FE0F"},
    {"maastopyoraileva_mies_vari1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"maastopyoraileva_mies_vari2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"maastopyoraileva_mies_vari3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"maastopyoraileva_mies_vari4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"maastopyoraileva_mies_vari5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"kaveleva_nainen", "1F6B6-200D-2640-FE0F"},
    {"kaveleva_nainen_vari1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"kaveleva_nainen_vari2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"kaveleva_nainen_vari3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"kaveleva_nainen_vari4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"kaveleva_nainen_vari5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"kaveleva_nainen_kohti_oikeaa", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kaveleva_nainen_kohti_oikeaa_vari1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kaveleva_nainen_kohti_oikeaa_vari2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kaveleva_nainen_kohti_oikeaa_vari3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kaveleva_nainen_kohti_oikeaa_vari4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kaveleva_nainen_kohti_oikeaa_vari5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kaveleva_mies", "1F6B6-200D-2642-FE0F"},
    {"kaveleva_mies_vari1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"kaveleva_mies_vari2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"kaveleva_mies_vari3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"kaveleva_mies_vari4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"kaveleva_mies_vari5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"kaveleva_mies_kohti_oikeaa", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kaveleva_mies_kohti_oikeaa_vari1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kaveleva_mies_kohti_oikeaa_vari2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kaveleva_mies_kohti_oikeaa_vari3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kaveleva_mies_kohti_oikeaa_vari4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kaveleva_mies_kohti_oikeaa_vari5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kasvonsa_kadella_peittava_nainen", "1F926-200D-2640-FE0F"},
    {"kasvonsa_kadella_peittava_nainen_vari1", "1F926-1F3FB-200D-2640-FE0F"},
    {"kasvonsa_kadella_peittava_nainen_vari2", "1F926-1F3FC-200D-2640-FE0F"},
    {"kasvonsa_kadella_peittava_nainen_vari3", "1F926-1F3FD-200D-2640-FE0F"},
    {"kasvonsa_kadella_peittava_nainen_vari4", "1F926-1F3FE-200D-2640-FE0F"},
    {"kasvonsa_kadella_peittava_nainen_vari5", "1F926-1F3FF-200D-2640-FE0F"},
    {"kasvonsa_kadella_peittava_mies", "1F926-200D-2642-FE0F"},
    {"kasvonsa_kadella_peittava_mies_vari1", "1F926-1F3FB-200D-2642-FE0F"},
    {"kasvonsa_kadella_peittava_mies_vari2", "1F926-1F3FC-200D-2642-FE0F"},
    {"kasvonsa_kadella_peittava_mies_vari3", "1F926-1F3FD-200D-2642-FE0F"},
    {"kasvonsa_kadella_peittava_mies_vari4", "1F926-1F3FE-200D-2642-FE0F"},
    {"kasvonsa_kadella_peittava_mies_vari5", "1F926-1F3FF-200D-2642-FE0F"},
    {"nainen_smokissa", "1F935-200D-2640-FE0F"},
    {"nainen_smokissa_vari1", "1F935-1F3FB-200D-2640-FE0F"},
    {"nainen_smokissa_vari2", "1F935-1F3FC-200D-2640-FE0F"},
    {"nainen_smokissa_vari3", "1F935-1F3FD-200D-2640-FE0F"},
    {"nainen_smokissa_vari4", "1F935-1F3FE-200D-2640-FE0F"},
    {"nainen_smokissa_vari5", "1F935-1F3FF-200D-2640-FE0F"},
    {"mies_smokissa", "1F935-200D-2642-FE0F"},
    {"mies_smokissa_vari1", "1F935-1F3FB-200D-2642-FE0F"},
    {"mies_smokissa_vari2", "1F935-1F3FC-200D-2642-FE0F"},
    {"mies_smokissa_vari3", "1F935-1F3FD-200D-2642-FE0F"},
    {"mies_smokissa_vari4", "1F935-1F3FE-200D-2642-FE0F"},
    {"mies_smokissa_vari5", "1F935-1F3FF-200D-2642-FE0F"},
    {"olkapaitaan_kohauttava_nainen", "1F937-200D-2640-FE0F"},
    {"olkapaitaan_kohauttava_nainen_vari1", "1F937-1F3FB-200D-2640-FE0F"},
    {"olkapaitaan_kohauttava_nainen_vari2", "1F937-1F3FC-200D-2640-FE0F"},
    {"olkapaitaan_kohauttava_nainen_vari3", "1F937-1F3FD-200D-2640-FE0F"},
    {"olkapaitaan_kohauttava_nainen_vari4", "1F937-1F3FE-200D-2640-FE0F"},
    {"olkapaitaan_kohauttava_nainen_vari5", "1F937-1F3FF-200D-2640-FE0F"},
    {"olkapaitaan_kohauttava_mies", "1F937-200D-2642-FE0F"},
    {"olkapaitaan_kohauttava_mies_vari1", "1F937-1F3FB-200D-2642-FE0F"},
    {"olkapaitaan_kohauttava_mies_vari2", "1F937-1F3FC-200D-2642-FE0F"},
    {"olkapaitaan_kohauttava_mies_vari3", "1F937-1F3FD-200D-2642-FE0F"},
    {"olkapaitaan_kohauttava_mies_vari4", "1F937-1F3FE-200D-2642-FE0F"},
    {"olkapaitaan_kohauttava_mies_vari5", "1F937-1F3FF-200D-2642-FE0F"},
    {"karrynpyoraa_tekeva_nainen", "1F938-200D-2640-FE0F"},
    {"karrynpyoraa_tekeva_nainen_vari1", "1F938-1F3FB-200D-2640-FE0F"},
    {"karrynpyoraa_tekeva_nainen_vari2", "1F938-1F3FC-200D-2640-FE0F"},
    {"karrynpyoraa_tekeva_nainen_vari3", "1F938-1F3FD-200D-2640-FE0F"},
    {"karrynpyoraa_tekeva_nainen_vari4", "1F938-1F3FE-200D-2640-FE0F"},
    {"karrynpyoraa_tekeva_nainen_vari5", "1F938-1F3FF-200D-2640-FE0F"},
    {"karrynpyoraa_tekeva_mies", "1F938-200D-2642-FE0F"},
    {"karrynpyoraa_tekeva_mies_vari1", "1F938-1F3FB-200D-2642-FE0F"},
    {"karrynpyoraa_tekeva_mies_vari2", "1F938-1F3FC-200D-2642-FE0F"},
    {"karrynpyoraa_tekeva_mies_vari3", "1F938-1F3FD-200D-2642-FE0F"},
    {"karrynpyoraa_tekeva_mies_vari4", "1F938-1F3FE-200D-2642-FE0F"},
    {"karrynpyoraa_tekeva_mies_vari5", "1F938-1F3FF-200D-2642-FE0F"},
    {"jongleeraava_nainen", "1F939-200D-2640-FE0F"},
    {"jongleeraava_nainen_vari1", "1F939-1F3FB-200D-2640-FE0F"},
    {"jongleeraava_nainen_vari2", "1F939-1F3FC-200D-2640-FE0F"},
    {"jongleeraava_nainen_vari3", "1F939-1F3FD-200D-2640-FE0F"},
    {"jongleeraava_nainen_vari4", "1F939-1F3FE-200D-2640-FE0F"},
    {"jongleeraava_nainen_vari5", "1F939-1F3FF-200D-2640-FE0F"},
    {"jongleeraava_mies", "1F939-200D-2642-FE0F"},
    {"jongleeraava_mies_vari1", "1F939-1F3FB-200D-2642-FE0F"},
    {"jongleeraava_mies_vari2", "1F939-1F3FC-200D-2642-FE0F"},
    {"jongleeraava_mies_vari3", "1F939-1F3FD-200D-2642-FE0F"},
    {"jongleeraava_mies_vari4", "1F939-1F3FE-200D-2642-FE0F"},
    {"jongleeraava_mies_vari5", "1F939-1F3FF-200D-2642-FE0F"},
    {"painivat_naiset", "1F93C-200D-2640-FE0F"},
    {"painivat_naiset_vari1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"painivat_naiset_vari1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"painivat_naiset_vari1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"painivat_naiset_vari1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"painivat_naiset_vari2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"painivat_naiset_vari2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"painivat_naiset_vari2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"painivat_naiset_vari2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"painivat_naiset_vari3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"painivat_naiset_vari3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"painivat_naiset_vari3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"painivat_naiset_vari3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"painivat_naiset_vari4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"painivat_naiset_vari4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"painivat_naiset_vari4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"painivat_naiset_vari4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"painivat_naiset_vari5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"painivat_naiset_vari5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"painivat_naiset_vari5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"painivat_naiset_vari5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"painivat_naiset_vari1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"painivat_naiset_vari2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"painivat_naiset_vari3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"painivat_naiset_vari4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"painivat_naiset_vari5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"painivat_miehet", "1F93C-200D-2642-FE0F"},
    {"painivat_miehet_vari1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"painivat_miehet_vari1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"painivat_miehet_vari1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"painivat_miehet_vari1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"painivat_miehet_vari2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"painivat_miehet_vari2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"painivat_miehet_vari2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"painivat_miehet_vari2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"painivat_miehet_vari3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"painivat_miehet_vari3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"painivat_miehet_vari3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"painivat_miehet_vari3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"painivat_miehet_vari4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"painivat_miehet_vari4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"painivat_miehet_vari4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"painivat_miehet_vari4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"painivat_miehet_vari5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"painivat_miehet_vari5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"painivat_miehet_vari5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"painivat_miehet_vari5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"painivat_miehet_vari1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"painivat_miehet_vari2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"painivat_miehet_vari3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"painivat_miehet_vari4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"painivat_miehet_vari5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"vesipalloa_pelaava_nainen", "1F93D-200D-2640-FE0F"},
    {"vesipalloa_pelaava_nainen_vari1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"vesipalloa_pelaava_nainen_vari2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"vesipalloa_pelaava_nainen_vari3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"vesipalloa_pelaava_nainen_vari4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"vesipalloa_pelaava_nainen_vari5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"vesipalloa_pelaava_mies", "1F93D-200D-2642-FE0F"},
    {"vesipalloa_pelaava_mies_vari1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"vesipalloa_pelaava_mies_vari2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"vesipalloa_pelaava_mies_vari3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"vesipalloa_pelaava_mies_vari4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"vesipalloa_pelaava_mies_vari5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"kasipalloa_pelaava_nainen", "1F93E-200D-2640-FE0F"},
    {"kasipalloa_pelaava_nainen_vari1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"kasipalloa_pelaava_nainen_vari2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"kasipalloa_pelaava_nainen_vari3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"kasipalloa_pelaava_nainen_vari4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"kasipalloa_pelaava_nainen_vari5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"kasipalloa_pelaava_mies", "1F93E-200D-2642-FE0F"},
    {"kasipalloa_pelaava_mies_vari1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"kasipalloa_pelaava_mies_vari2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"kasipalloa_pelaava_mies_vari3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"kasipalloa_pelaava_mies_vari4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"kasipalloa_pelaava_mies_vari5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"naissupersankari", "1F9B8-200D-2640-FE0F"},
    {"naissupersankari_vari1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"naissupersankari_vari2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"naissupersankari_vari3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"naissupersankari_vari4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"naissupersankari_vari5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"miessupersankari", "1F9B8-200D-2642-FE0F"},
    {"miessupersankari_vari1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"miessupersankari_vari2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"miessupersankari_vari3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"miessupersankari_vari4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"miessupersankari_vari5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"naissuperpahis", "1F9B9-200D-2640-FE0F"},
    {"naissuperpahis_vari1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"naissuperpahis_vari2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"naissuperpahis_vari3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"naissuperpahis_vari4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"naissuperpahis_vari5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"miessuperpahis", "1F9B9-200D-2642-FE0F"},
    {"miessuperpahis_vari1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"miessuperpahis_vari2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"miessuperpahis_vari3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"miessuperpahis_vari4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"miessuperpahis_vari5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"seisova_nainen", "1F9CD-200D-2640-FE0F"},
    {"seisova_nainen_vari1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"seisova_nainen_vari2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"seisova_nainen_vari3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"seisova_nainen_vari4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"seisova_nainen_vari5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"seisova_mies", "1F9CD-200D-2642-FE0F"},
    {"seisova_mies_vari1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"seisova_mies_vari2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"seisova_mies_vari3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"seisova_mies_vari4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"seisova_mies_vari5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"polvistuva_nainen", "1F9CE-200D-2640-FE0F"},
    {"polvistuva_nainen_vari1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"polvistuva_nainen_vari2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"polvistuva_nainen_vari3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"polvistuva_nainen_vari4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"polvistuva_nainen_vari5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"polvistuva_nainen_kohti_oikeaa", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"polvistuva_nainen_kohti_oikeaa_vari1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"polvistuva_nainen_kohti_oikeaa_vari2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"polvistuva_nainen_kohti_oikeaa_vari3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"polvistuva_nainen_kohti_oikeaa_vari4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"polvistuva_nainen_kohti_oikeaa_vari5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"polvistuva_mies", "1F9CE-200D-2642-FE0F"},
    {"polvistuva_mies_vari1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"polvistuva_mies_vari2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"polvistuva_mies_vari3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"polvistuva_mies_vari4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"polvistuva_mies_vari5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"polvistuva_mies_kohti_oikeaa", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"polvistuva_mies_kohti_oikeaa_vari1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"polvistuva_mies_kohti_oikeaa_vari2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"polvistuva_mies_kohti_oikeaa_vari3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"polvistuva_mies_kohti_oikeaa_vari4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"polvistuva_mies_kohti_oikeaa_vari5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kuuro_nainen", "1F9CF-200D-2640-FE0F"},
    {"kuuro_nainen_vari1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"kuuro_nainen_vari2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"kuuro_nainen_vari3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"kuuro_nainen_vari4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"kuuro_nainen_vari5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"kuuro_mies", "1F9CF-200D-2642-FE0F"},
    {"kuuro_mies_vari1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"kuuro_mies_vari2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"kuuro_mies_vari3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"kuuro_mies_vari4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"kuuro_mies_vari5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"parrakas_nainen", "1F9D4-200D-2640-FE0F"},
    {"parrakas_nainen_vari1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"parrakas_nainen_vari2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"parrakas_nainen_vari3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"parrakas_nainen_vari4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"parrakas_nainen_vari5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"parrakas_mies", "1F9D4-200D-2642-FE0F"},
    {"parrakas_mies_vari1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"parrakas_mies_vari2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"parrakas_mies_vari3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"parrakas_mies_vari4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"parrakas_mies_vari5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"nainen_saunassa", "1F9D6-200D-2640-FE0F"},
    {"nainen_saunassa_vari1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"nainen_saunassa_vari2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"nainen_saunassa_vari3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"nainen_saunassa_vari4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"nainen_saunassa_vari5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"mies_saunassa", "1F9D6-200D-2642-FE0F"},
    {"mies_saunassa_vari1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"mies_saunassa_vari2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"mies_saunassa_vari3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"mies_saunassa_vari4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"mies_saunassa_vari5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"naiskiipeilija", "1F9D7-200D-2640-FE0F"},
    {"naiskiipeilija_vari1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"naiskiipeilija_vari2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"naiskiipeilija_vari3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"naiskiipeilija_vari4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"naiskiipeilija_vari5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"mieskiipeilija", "1F9D7-200D-2642-FE0F"},
    {"mieskiipeilija_vari1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"mieskiipeilija_vari2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"mieskiipeilija_vari3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"mieskiipeilija_vari4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"mieskiipeilija_vari5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"nainen_lootusasennossa", "1F9D8-200D-2640-FE0F"},
    {"nainen_lootusasennossa_vari1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"nainen_lootusasennossa_vari2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"nainen_lootusasennossa_vari3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"nainen_lootusasennossa_vari4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"nainen_lootusasennossa_vari5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"mies_lootusasennossa", "1F9D8-200D-2642-FE0F"},
    {"mies_lootusasennossa_vari1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"mies_lootusasennossa_vari2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"mies_lootusasennossa_vari3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"mies_lootusasennossa_vari4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"mies_lootusasennossa_vari5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"naisvelho", "1F9D9-200D-2640-FE0F"},
    {"naisvelho_vari1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"naisvelho_vari2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"naisvelho_vari3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"naisvelho_vari4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"naisvelho_vari5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"miesvelho", "1F9D9-200D-2642-FE0F"},
    {"miesvelho_vari1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"miesvelho_vari2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"miesvelho_vari3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"miesvelho_vari4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"miesvelho_vari5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"naiskeiju", "1F9DA-200D-2640-FE0F"},
    {"naiskeiju_vari1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"naiskeiju_vari2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"naiskeiju_vari3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"naiskeiju_vari4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"naiskeiju_vari5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"mieskeiju", "1F9DA-200D-2642-FE0F"},
    {"mieskeiju_vari1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"mieskeiju_vari2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"mieskeiju_vari3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"mieskeiju_vari4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"mieskeiju_vari5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"naisvampyyri", "1F9DB-200D-2640-FE0F"},
    {"naisvampyyri_vari1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"naisvampyyri_vari2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"naisvampyyri_vari3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"naisvampyyri_vari4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"naisvampyyri_vari5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"miesvampyyri", "1F9DB-200D-2642-FE0F"},
    {"miesvampyyri_vari1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"miesvampyyri_vari2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"miesvampyyri_vari3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"miesvampyyri_vari4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"miesvampyyri_vari5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"merenneito", "1F9DC-200D-2640-FE0F"},
    {"merenneito_vari1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"merenneito_vari2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"merenneito_vari3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"merenneito_vari4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"merenneito_vari5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"merenmies", "1F9DC-200D-2642-FE0F"},
    {"merenmies_vari1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"merenmies_vari2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"merenmies_vari3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"merenmies_vari4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"merenmies_vari5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"haltiatar", "1F9DD-200D-2640-FE0F"},
    {"haltiatar_vari1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"haltiatar_vari2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"haltiatar_vari3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"haltiatar_vari4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"haltiatar_vari5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"mieshaltia", "1F9DD-200D-2642-FE0F"},
    {"mieshaltia_vari1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"mieshaltia_vari2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"mieshaltia_vari3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"mieshaltia_vari4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"mieshaltia_vari5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"naishenki", "1F9DE-200D-2640-FE0F"},
    {"mieshenki", "1F9DE-200D-2642-FE0F"},
    {"naiszombi", "1F9DF-200D-2640-FE0F"},
    {"mieszombi", "1F9DF-200D-2642-FE0F"},
    {"mies_punatukkainen", "1F468-200D-1F9B0"},
    {"mies_punatukkainen_vari1", "1F468-1F3FB-200D-1F9B0"},
    {"mies_punatukkainen_vari2", "1F468-1F3FC-200D-1F9B0"},
    {"mies_punatukkainen_vari3", "1F468-1F3FD-200D-1F9B0"},
    {"mies_punatukkainen_vari4", "1F468-1F3FE-200D-1F9B0"},
    {"mies_punatukkainen_vari5", "1F468-1F3FF-200D-1F9B0"},
    {"mies_kiharatukkainen", "1F468-200D-1F9B1"},
    {"mies_kiharatukkainen_vari1", "1F468-1F3FB-200D-1F9B1"},
    {"mies_kiharatukkainen_vari2", "1F468-1F3FC-200D-1F9B1"},
    {"mies_kiharatukkainen_vari3", "1F468-1F3FD-200D-1F9B1"},
    {"mies_kiharatukkainen_vari4", "1F468-1F3FE-200D-1F9B1"},
    {"mies_kiharatukkainen_vari5", "1F468-1F3FF-200D-1F9B1"},
    {"mies_kalju", "1F468-200D-1F9B2"},
    {"mies_kalju_vari1", "1F468-1F3FB-200D-1F9B2"},
    {"mies_kalju_vari2", "1F468-1F3FC-200D-1F9B2"},
    {"mies_kalju_vari3", "1F468-1F3FD-200D-1F9B2"},
    {"mies_kalju_vari4", "1F468-1F3FE-200D-1F9B2"},
    {"mies_kalju_vari5", "1F468-1F3FF-200D-1F9B2"},
    {"mies_valkotukkainen", "1F468-200D-1F9B3"},
    {"mies_valkotukkainen_vari1", "1F468-1F3FB-200D-1F9B3"},
    {"mies_valkotukkainen_vari2", "1F468-1F3FC-200D-1F9B3"},
    {"mies_valkotukkainen_vari3", "1F468-1F3FD-200D-1F9B3"},
    {"mies_valkotukkainen_vari4", "1F468-1F3FE-200D-1F9B3"},
    {"mies_valkotukkainen_vari5", "1F468-1F3FF-200D-1F9B3"},
    {"nainen_punatukkainen", "1F469-200D-1F9B0"},
    {"nainen_punatukkainen_vari1", "1F469-1F3FB-200D-1F9B0"},
    {"nainen_punatukkainen_vari2", "1F469-1F3FC-200D-1F9B0"},
    {"nainen_punatukkainen_vari3", "1F469-1F3FD-200D-1F9B0"},
    {"nainen_punatukkainen_vari4", "1F469-1F3FE-200D-1F9B0"},
    {"nainen_punatukkainen_vari5", "1F469-1F3FF-200D-1F9B0"},
    {"nainen_kiharatukkainen", "1F469-200D-1F9B1"},
    {"nainen_kiharatukkainen_vari1", "1F469-1F3FB-200D-1F9B1"},
    {"nainen_kiharatukkainen_vari2", "1F469-1F3FC-200D-1F9B1"},
    {"nainen_kiharatukkainen_vari3", "1F469-1F3FD-200D-1F9B1"},
    {"nainen_kiharatukkainen_vari4", "1F469-1F3FE-200D-1F9B1"},
    {"nainen_kiharatukkainen_vari5", "1F469-1F3FF-200D-1F9B1"},
    {"nainen_kalju", "1F469-200D-1F9B2"},
    {"nainen_kalju_vari1", "1F469-1F3FB-200D-1F9B2"},
    {"nainen_kalju_vari2", "1F469-1F3FC-200D-1F9B2"},
    {"nainen_kalju_vari3", "1F469-1F3FD-200D-1F9B2"},
    {"nainen_kalju_vari4", "1F469-1F3FE-200D-1F9B2"},
    {"nainen_kalju_vari5", "1F469-1F3FF-200D-1F9B2"},
    {"nainen_valkotukkainen", "1F469-200D-1F9B3"},
    {"nainen_valkotukkainen_vari1", "1F469-1F3FB-200D-1F9B3"},
    {"nainen_valkotukkainen_vari2", "1F469-1F3FC-200D-1F9B3"},
    {"nainen_valkotukkainen_vari3", "1F469-1F3FD-200D-1F9B3"},
    {"nainen_valkotukkainen_vari4", "1F469-1F3FE-200D-1F9B3"},
    {"nainen_valkotukkainen_vari5", "1F469-1F3FF-200D-1F9B3"},
    {"aikuinen_punatukkainen", "1F9D1-200D-1F9B0"},
    {"aikuinen_punatukkainen_vari1", "1F9D1-1F3FB-200D-1F9B0"},
    {"aikuinen_punatukkainen_vari2", "1F9D1-1F3FC-200D-1F9B0"},
    {"aikuinen_punatukkainen_vari3", "1F9D1-1F3FD-200D-1F9B0"},
    {"aikuinen_punatukkainen_vari4", "1F9D1-1F3FE-200D-1F9B0"},
    {"aikuinen_punatukkainen_vari5", "1F9D1-1F3FF-200D-1F9B0"},
    {"aikuinen_kiharatukkainen", "1F9D1-200D-1F9B1"},
    {"aikuinen_kiharatukkainen_vari1", "1F9D1-1F3FB-200D-1F9B1"},
    {"aikuinen_kiharatukkainen_vari2", "1F9D1-1F3FC-200D-1F9B1"},
    {"aikuinen_kiharatukkainen_vari3", "1F9D1-1F3FD-200D-1F9B1"},
    {"aikuinen_kiharatukkainen_vari4", "1F9D1-1F3FE-200D-1F9B1"},
    {"aikuinen_kiharatukkainen_vari5", "1F9D1-1F3FF-200D-1F9B1"},
    {"aikuinen_kalju", "1F9D1-200D-1F9B2"},
    {"aikuinen_kalju_vari1", "1F9D1-1F3FB-200D-1F9B2"},
    {"aikuinen_kalju_vari2", "1F9D1-1F3FC-200D-1F9B2"},
    {"aikuinen_kalju_vari3", "1F9D1-1F3FD-200D-1F9B2"},
    {"aikuinen_kalju_vari4", "1F9D1-1F3FE-200D-1F9B2"},
    {"aikuinen_kalju_vari5", "1F9D1-1F3FF-200D-1F9B2"},
    {"aikuinen_valkotukkainen", "1F9D1-200D-1F9B3"},
    {"aikuinen_valkotukkainen_vari1", "1F9D1-1F3FB-200D-1F9B3"},
    {"aikuinen_valkotukkainen_vari2", "1F9D1-1F3FC-200D-1F9B3"},
    {"aikuinen_valkotukkainen_vari3", "1F9D1-1F3FD-200D-1F9B3"},
    {"aikuinen_valkotukkainen_vari4", "1F9D1-1F3FE-200D-1F9B3"},
    {"aikuinen_valkotukkainen_vari5", "1F9D1-1F3FF-200D-1F9B3"},
    {"katkennut_ketju", "26D3-FE0F-200D-1F4A5"},
    {"palava_sydan", "2764-FE0F-200D-1F525"},
    {"parantuva_sydan", "2764-FE0F-200D-1FA79"},
    {"ruskea_herkkusieni", "1F344-200D-1F7EB"},
    {"limetti", "1F34B-200D-1F7E9"},
    {"translippu", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"sateenkaarilippu", "1F3F3-FE0F-200D-1F308"},
    {"merirosvolippu", "1F3F4-200D-2620-FE0F"},
    {"musta_kissa", "1F408-200D-2B1B"},
    {"palveluskoira", "1F415-200D-1F9BA"},
    {"mustarastas", "1F426-200D-2B1B"},
    {"feeniks", "1F426-200D-1F525"},
    {"jaakarhu", "1F43B-200D-2744-FE0F"},
    {"puhekuplasilma", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"hammastynyt", "1F62E-200D-1F4A8"},
    {"naama_jolla_spiraalisilmat", "1F635-200D-1F4AB"},
    {"naama_pilvissa", "1F636-200D-1F32B-FE0F"},
    {"ravistaa_paata_vaakasuunnassa", "1F642-200D-2194-FE0F"},
    {"ravistaa_paata_pystysuunnassa", "1F642-200D-2195-FE0F"},
    {"balettitanssija", "1F9D1-200D-1FA70"},
    {"balettitanssija_vari1", "1F9D1-1F3FB-200D-1FA70"},
    {"balettitanssija_vari2", "1F9D1-1F3FC-200D-1FA70"},
    {"balettitanssija_vari3", "1F9D1-1F3FD-200D-1FA70"},
    {"balettitanssija_vari4", "1F9D1-1F3FE-200D-1FA70"},
    {"balettitanssija_vari5", "1F9D1-1F3FF-200D-1FA70"},
    {"nuoli_ylös_ja_alas", "2195"},
    {"nuoli_ylävasemmalle", "2196"},
    {"nuoli_yläoikealle", "2197"},
    {"näppäimistö", "2328"},
    {"pääkallo_ja_reisiluut", "2620"},
    {"säteilyvaara", "2622"},
    {"dharma_pyörä", "2638"},
    {"härkä_merkki", "2649"},
    {"kuumat_lähteet", "2668"},
    {"miekat_ristissä", "2694"},
    {"lääketieteen_symboli", "2695"},
    {"musta_kynänkärki", "2712"},
    {"daavidin_tähti", "2721"},
    {"säihke", "2728"},
    {"kahdeksanpiikkinen_tähtimerkki", "2733"},
    {"kahdeksankärkinen_tähti", "2734"},
    {"kipinä", "2747"},
    {"sydänhuutomerkki", "2763"},
    {"punainen_sydän", "2764"},
    {"ylös_kääntyvä_nuoli", "2934"},
    {"alas_kääntyvä_nuoli", "2935"},
    {"tekijänoikeusmerkki", "00A9"},
    {"rekisteröity_merkki", "00AE"},
    {"vasemmalle_kääntyvä_nuoli", "21A9"},
    {"oikealle_kääntyvä_nuoli", "21AA"},
    {"kelaa_eteenpäin", "23E9"},
    {"kelaa_taaksepäin", "23EA"},
    {"nopeasti_ylös", "23EB"},
    {"toista_pysäytä", "23EF"},
    {"herätyskello", "23F0"},
    {"pysäytä", "23F8"},
    {"m_ympyrässä", "24C2"},
    {"pieni_musta_neliö", "25AA"},
    {"pieni_valkoinen_neliö", "25AB"},
    {"keskikokoinen_valkoinen_neliö", "25FB"},
    {"keskikokoinen_musta_neliö", "25FC"},
    {"pienehkö_valkoinen_neliö", "25FD"},
    {"pienehkö_musta_neliö", "25FE"},
    {"ylös_osoittava_etusormi", "261D"},
    {"ylös_osoittava_etusormi_väri1", "261D-1F3FB"},
    {"ylös_osoittava_etusormi_väri2", "261D-1F3FC"},
    {"ylös_osoittava_etusormi_väri3", "261D-1F3FD"},
    {"ylös_osoittava_etusormi_väri4", "261D-1F3FE"},
    {"ylös_osoittava_etusormi_väri5", "261D-1F3FF"},
    {"kuunsirppi_ja_tähti", "262A"},
    {"šakkipelin_sotilas", "265F"},
    {"kierrätyssymboli", "267B"},
    {"äärettömyys", "267E"},
    {"pyörätuoli", "267F"},
    {"korkeajännite", "26A1"},
    {"valkoinen_ympyrä", "26AA"},
    {"musta_ympyrä", "26AB"},
    {"pesäpallo", "26BE"},
    {"aurinko_pilvessä", "26C5"},
    {"käärmeenkantaja", "26CE"},
    {"kypärä_jossa_valkoinen_risti", "26D1"},
    {"suihkulähde", "26F2"},
    {"lippu_reiän_päällä", "26F3"},
    {"hiihtäjä", "26F7"},
    {"nostettu_käsi", "270B"},
    {"nostettu_käsi_väri1", "270B-1F3FB"},
    {"nostettu_käsi_väri2", "270B-1F3FC"},
    {"nostettu_käsi_väri3", "270B-1F3FD"},
    {"nostettu_käsi_väri4", "270B-1F3FE"},
    {"nostettu_käsi_väri5", "270B-1F3FF"},
    {"kirjoittava_käsi", "270D"},
    {"kirjoittava_käsi_väri1", "270D-1F3FB"},
    {"kirjoittava_käsi_väri2", "270D-1F3FC"},
    {"kirjoittava_käsi_väri3", "270D-1F3FD"},
    {"kirjoittava_käsi_väri4", "270D-1F3FE"},
    {"kirjoittava_käsi_väri5", "270D-1F3FF"},
    {"lyijykynä", "270F"},
    {"nuoli_ylös", "2B06"},
    {"iso_musta_neliö", "2B1B"},
    {"iso_valkoinen_neliö", "2B1C"},
    {"valkoinen_tähti", "2B50"},
    {"suuri_ympyrä", "2B55"},
    {"a_veriryhmä", "1F170"},
    {"b_veriryhmä", "1F171"},
    {"o_veriryhmä", "1F17E"},
    {"ab_veriryhmä", "1F18E"},
    {"pyyhi_näppäin", "1F191"},
    {"täällä_kirjainmerkki", "1F201"},
    {"tyhjä_kirjoitusmerkki", "1F233"},
    {"hyväksytty_arvosana_kirjoitusmerkki", "1F234"},
    {"täynnä_kirjoitusmerkki", "1F235"},
    {"kuukausittainen_määrä_kirjainmerkki", "1F237"},
    {"hyvä_ostos_kirjainmerkki", "1F250"},
    {"hyväksyttävä_kirjoitusmerkki", "1F251"},
    {"pyörremyrsky", "1F300"},
    {"tähtitaivas", "1F303"},
    {"kaupunkimaisema_illanhämärässä", "1F306"},
    {"silta_yöllä", "1F309"},
    {"maapallo_jossa_näkyy_eurooppa_ja_afrikka", "1F30D"},
    {"maapallo_jossa_näkyy_amerikka", "1F30E"},
    {"maapallo_jossa_näkyy_aasia_ja_australia", "1F30F"},
    {"maapallo_jossa_näkyy_pituuspiirit", "1F310"},
    {"ensimmäinen_neljänneskuu", "1F313"},
    {"täysikuu", "1F315"},
    {"vähenevä_puolikuu", "1F316"},
    {"viimeinen_neljänneskuu", "1F317"},
    {"vähenevä_kuunsirppi", "1F318"},
    {"vähenevä_puolikuu_kasvoilla", "1F31C"},
    {"täysikuu_kasvoilla", "1F31D"},
    {"loistava_tähti", "1F31F"},
    {"tähdenlento", "1F320"},
    {"lämpömittari", "1F321"},
    {"aurinko_sadepilvessä", "1F326"},
    {"ikivihreä", "1F332"},
    {"maissintähkä", "1F33D"},
    {"viinirypäleet", "1F347"},
    {"vihreä_omena", "1F34F"},
    {"päärynä", "1F350"},
    {"höyryävä_kulho", "1F35C"},
    {"leipä", "1F35E"},
    {"jäähilejälkiruoka", "1F367"},
    {"jäätelö", "1F368"},
    {"kilisevät_oluttuopit", "1F37B"},
    {"syntymäpäiväkakku", "1F382"},
    {"tähtisadetikku", "1F387"},
    {"liput_ristissä", "1F38C"},
    {"liukusäädin", "1F39A"},
    {"säätimet", "1F39B"},
    {"pääsylippu", "1F39F"},
    {"maailmanpyörä", "1F3A1"},
    {"näyttämötaiteet", "1F3AD"},
    {"moottoripyörä", "1F3CD"},
    {"jääkiekko_ja_maila", "1F3D2"},
    {"rakennustyömaa", "1F3D7"},
    {"taloryhmä", "1F3D8"},
    {"lähikauppa", "1F3EA"},
    {"härkä", "1F402"},
    {"lehmä", "1F404"},
    {"lohikäärme", "1F409"},
    {"käärme", "1F40D"},
    {"pässi", "1F40F"},
    {"ötökkä", "1F41B"},
    {"mehiläinen", "1F41D"},
    {"leppäkerttu", "1F41E"},
    {"tipu_edestäpäin", "1F425"},
    {"kaksikyttyräinen_kameli", "1F42B"},
    {"lehmän_naama", "1F42E"},
    {"jäniksen_naama", "1F430"},
    {"lohikäärmeen_naama", "1F432"},
    {"sian_kärsä", "1F43D"},
    {"tassunjäljet", "1F43E"},
    {"silmät", "1F440"},
    {"silmä", "1F441"},
    {"nenä", "1F443"},
    {"nenä_väri1", "1F443-1F3FB"},
    {"nenä_väri2", "1F443-1F3FC"},
    {"nenä_väri3", "1F443-1F3FD"},
    {"nenä_väri4", "1F443-1F3FE"},
    {"nenä_väri5", "1F443-1F3FF"},
    {"kämmenselkä_ja_etusormi_ylös", "1F446"},
    {"kämmenselkä_ja_etusormi_ylös_väri1", "1F446-1F3FB"},
    {"kämmenselkä_ja_etusormi_ylös_väri2", "1F446-1F3FC"},
    {"kämmenselkä_ja_etusormi_ylös_väri3", "1F446-1F3FD"},
    {"kämmenselkä_ja_etusormi_ylös_väri4", "1F446-1F3FE"},
    {"kämmenselkä_ja_etusormi_ylös_väri5", "1F446-1F3FF"},
    {"kämmenselkä_ja_etusormi_alas", "1F447"},
    {"kämmenselkä_ja_etusormi_alas_väri1", "1F447-1F3FB"},
    {"kämmenselkä_ja_etusormi_alas_väri2", "1F447-1F3FC"},
    {"kämmenselkä_ja_etusormi_alas_väri3", "1F447-1F3FD"},
    {"kämmenselkä_ja_etusormi_alas_väri4", "1F447-1F3FE"},
    {"kämmenselkä_ja_etusormi_alas_väri5", "1F447-1F3FF"},
    {"kämmenselkä_ja_etusormi_vasemmalle", "1F448"},
    {"kämmenselkä_ja_etusormi_vasemmalle_väri1", "1F448-1F3FB"},
    {"kämmenselkä_ja_etusormi_vasemmalle_väri2", "1F448-1F3FC"},
    {"kämmenselkä_ja_etusormi_vasemmalle_väri3", "1F448-1F3FD"},
    {"kämmenselkä_ja_etusormi_vasemmalle_väri4", "1F448-1F3FE"},
    {"kämmenselkä_ja_etusormi_vasemmalle_väri5", "1F448-1F3FF"},
    {"kämmenselkä_ja_etusormi_oikealle", "1F449"},
    {"kämmenselkä_ja_etusormi_oikealle_väri1", "1F449-1F3FB"},
    {"kämmenselkä_ja_etusormi_oikealle_väri2", "1F449-1F3FC"},
    {"kämmenselkä_ja_etusormi_oikealle_väri3", "1F449-1F3FD"},
    {"kämmenselkä_ja_etusormi_oikealle_väri4", "1F449-1F3FE"},
    {"kämmenselkä_ja_etusormi_oikealle_väri5", "1F449-1F3FF"},
    {"käden_heilautus", "1F44B"},
    {"käden_heilautus_väri1", "1F44B-1F3FB"},
    {"käden_heilautus_väri2", "1F44B-1F3FC"},
    {"käden_heilautus_väri3", "1F44B-1F3FD"},
    {"käden_heilautus_väri4", "1F44B-1F3FE"},
    {"käden_heilautus_väri5", "1F44B-1F3FF"},
    {"peukalo_ylös", "1F44D"},
    {"peukalo_ylös_väri1", "1F44D-1F3FB"},
    {"peukalo_ylös_väri2", "1F44D-1F3FC"},
    {"peukalo_ylös_väri3", "1F44D-1F3FD"},
    {"peukalo_ylös_väri4", "1F44D-1F3FE"},
    {"peukalo_ylös_väri5", "1F44D-1F3FF"},
    {"käsien_taputus", "1F44F"},
    {"käsien_taputus_väri1", "1F44F-1F3FB"},
    {"käsien_taputus_väri2", "1F44F-1F3FC"},
    {"käsien_taputus_väri3", "1F44F-1F3FD"},
    {"käsien_taputus_väri4", "1F44F-1F3FE"},
    {"käsien_taputus_väri5", "1F44F-1F3FF"},
    {"avoimet_kädet", "1F450"},
    {"avoimet_kädet_väri1", "1F450-1F3FB"},
    {"avoimet_kädet_väri2", "1F450-1F3FC"},
    {"avoimet_kädet_väri3", "1F450-1F3FD"},
    {"avoimet_kädet_väri4", "1F450-1F3FE"},
    {"avoimet_kädet_väri5", "1F450-1F3FF"},
    {"silmälasit", "1F453"},
    {"käsilaukku", "1F45C"},
    {"miehen_kenkä", "1F45E"},
    {"juoksukenkä", "1F45F"},
    {"korkokenkä", "1F460"},
    {"jalanjäljet", "1F463"},
    {"tyttö", "1F467"},
    {"tyttö_väri1", "1F467-1F3FB"},
    {"tyttö_väri2", "1F467-1F3FC"},
    {"tyttö_väri3", "1F467-1F3FD"},
    {"tyttö_väri4", "1F467-1F3FE"},
    {"tyttö_väri5", "1F467-1F3FF"},
    {"mies_ja_nainen_käsi_kädessä", "1F46B"},
    {"mies_ja_nainen_käsi_kädessä_väri1", "1F46B-1F3FB"},
    {"mies_ja_nainen_käsi_kädessä_väri2", "1F46B-1F3FC"},
    {"mies_ja_nainen_käsi_kädessä_väri3", "1F46B-1F3FD"},
    {"mies_ja_nainen_käsi_kädessä_väri4", "1F46B-1F3FE"},
    {"mies_ja_nainen_käsi_kädessä_väri5", "1F46B-1F3FF"},
    {"mies_ja_nainen_käsi_kädessä_väri1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"mies_ja_nainen_käsi_kädessä_väri1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"mies_ja_nainen_käsi_kädessä_väri1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"mies_ja_nainen_käsi_kädessä_väri1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"mies_ja_nainen_käsi_kädessä_väri2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"mies_ja_nainen_käsi_kädessä_väri2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"mies_ja_nainen_käsi_kädessä_väri2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"mies_ja_nainen_käsi_kädessä_väri2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"mies_ja_nainen_käsi_kädessä_väri3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"mies_ja_nainen_käsi_kädessä_väri3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"mies_ja_nainen_käsi_kädessä_väri3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"mies_ja_nainen_käsi_kädessä_väri3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"mies_ja_nainen_käsi_kädessä_väri4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"mies_ja_nainen_käsi_kädessä_väri4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"mies_ja_nainen_käsi_kädessä_väri4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"mies_ja_nainen_käsi_kädessä_väri4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"mies_ja_nainen_käsi_kädessä_väri5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"mies_ja_nainen_käsi_kädessä_väri5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"mies_ja_nainen_käsi_kädessä_väri5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"mies_ja_nainen_käsi_kädessä_väri5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"kaksi_miestä_käsi_kädessä", "1F46C"},
    {"kaksi_miestä_käsi_kädessä_väri1", "1F46C-1F3FB"},
    {"kaksi_miestä_käsi_kädessä_väri2", "1F46C-1F3FC"},
    {"kaksi_miestä_käsi_kädessä_väri3", "1F46C-1F3FD"},
    {"kaksi_miestä_käsi_kädessä_väri4", "1F46C-1F3FE"},
    {"kaksi_miestä_käsi_kädessä_väri5", "1F46C-1F3FF"},
    {"kaksi_miestä_käsi_kädessä_väri1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"kaksi_miestä_käsi_kädessä_väri1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"kaksi_miestä_käsi_kädessä_väri1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"kaksi_miestä_käsi_kädessä_väri1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"kaksi_miestä_käsi_kädessä_väri2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"kaksi_miestä_käsi_kädessä_väri2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"kaksi_miestä_käsi_kädessä_väri2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"kaksi_miestä_käsi_kädessä_väri2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"kaksi_miestä_käsi_kädessä_väri3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"kaksi_miestä_käsi_kädessä_väri3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"kaksi_miestä_käsi_kädessä_väri3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"kaksi_miestä_käsi_kädessä_väri3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"kaksi_miestä_käsi_kädessä_väri4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"kaksi_miestä_käsi_kädessä_väri4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"kaksi_miestä_käsi_kädessä_väri4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"kaksi_miestä_käsi_kädessä_väri4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"kaksi_miestä_käsi_kädessä_väri5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"kaksi_miestä_käsi_kädessä_väri5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"kaksi_miestä_käsi_kädessä_väri5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"kaksi_miestä_käsi_kädessä_väri5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"kaksi_naista_käsi_kädessä", "1F46D"},
    {"kaksi_naista_käsi_kädessä_väri1", "1F46D-1F3FB"},
    {"kaksi_naista_käsi_kädessä_väri2", "1F46D-1F3FC"},
    {"kaksi_naista_käsi_kädessä_väri3", "1F46D-1F3FD"},
    {"kaksi_naista_käsi_kädessä_väri4", "1F46D-1F3FE"},
    {"kaksi_naista_käsi_kädessä_väri5", "1F46D-1F3FF"},
    {"kaksi_naista_käsi_kädessä_väri1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"kaksi_naista_käsi_kädessä_väri1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"kaksi_naista_käsi_kädessä_väri1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"kaksi_naista_käsi_kädessä_väri1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"kaksi_naista_käsi_kädessä_väri2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"kaksi_naista_käsi_kädessä_väri2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"kaksi_naista_käsi_kädessä_väri2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"kaksi_naista_käsi_kädessä_väri2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"kaksi_naista_käsi_kädessä_väri3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"kaksi_naista_käsi_kädessä_väri3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"kaksi_naista_käsi_kädessä_väri3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"kaksi_naista_käsi_kädessä_väri3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"kaksi_naista_käsi_kädessä_väri4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"kaksi_naista_käsi_kädessä_väri4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"kaksi_naista_käsi_kädessä_väri4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"kaksi_naista_käsi_kädessä_väri4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"kaksi_naista_käsi_kädessä_väri5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"kaksi_naista_käsi_kädessä_väri5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"kaksi_naista_käsi_kädessä_väri5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"kaksi_naista_käsi_kädessä_väri5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"puputytöt", "1F46F"},
    {"puputytöt_väri1", "1F46F-1F3FB"},
    {"puputytöt_väri2", "1F46F-1F3FC"},
    {"puputytöt_väri3", "1F46F-1F3FD"},
    {"puputytöt_väri4", "1F46F-1F3FE"},
    {"puputytöt_väri5", "1F46F-1F3FF"},
    {"puputytöt_väri1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"puputytöt_väri1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"puputytöt_väri1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"puputytöt_väri1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"puputytöt_väri2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"puputytöt_väri2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"puputytöt_väri2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"puputytöt_väri2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"puputytöt_väri3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"puputytöt_väri3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"puputytöt_väri3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"puputytöt_väri3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"puputytöt_väri4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"puputytöt_väri4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"puputytöt_väri4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"puputytöt_väri4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"puputytöt_väri5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"puputytöt_väri5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"puputytöt_väri5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"puputytöt_väri5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"huntupäinen_henkilö", "1F470"},
    {"huntupäinen_henkilö_väri1", "1F470-1F3FB"},
    {"huntupäinen_henkilö_väri2", "1F470-1F3FC"},
    {"huntupäinen_henkilö_väri3", "1F470-1F3FD"},
    {"huntupäinen_henkilö_väri4", "1F470-1F3FE"},
    {"huntupäinen_henkilö_väri5", "1F470-1F3FF"},
    {"vaaleatukkainen_henkilö", "1F471"},
    {"vaaleatukkainen_henkilö_väri1", "1F471-1F3FB"},
    {"vaaleatukkainen_henkilö_väri2", "1F471-1F3FC"},
    {"vaaleatukkainen_henkilö_väri3", "1F471-1F3FD"},
    {"vaaleatukkainen_henkilö_väri4", "1F471-1F3FE"},
    {"vaaleatukkainen_henkilö_väri5", "1F471-1F3FF"},
    {"turbaanipäinen_henkilö", "1F473"},
    {"turbaanipäinen_henkilö_väri1", "1F473-1F3FB"},
    {"turbaanipäinen_henkilö_väri2", "1F473-1F3FC"},
    {"turbaanipäinen_henkilö_väri3", "1F473-1F3FD"},
    {"turbaanipäinen_henkilö_väri4", "1F473-1F3FE"},
    {"turbaanipäinen_henkilö_väri5", "1F473-1F3FF"},
    {"rakennustyöläinen", "1F477"},
    {"rakennustyöläinen_väri1", "1F477-1F3FB"},
    {"rakennustyöläinen_väri2", "1F477-1F3FC"},
    {"rakennustyöläinen_väri3", "1F477-1F3FD"},
    {"rakennustyöläinen_väri4", "1F477-1F3FE"},
    {"rakennustyöläinen_väri5", "1F477-1F3FF"},
    {"avaruushirviö", "1F47E"},
    {"pääkallo", "1F480"},
    {"kämmentä_ylöspäin_näyttävä_henkilö", "1F481"},
    {"kämmentä_ylöspäin_näyttävä_henkilö_väri1", "1F481-1F3FB"},
    {"kämmentä_ylöspäin_näyttävä_henkilö_väri2", "1F481-1F3FC"},
    {"kämmentä_ylöspäin_näyttävä_henkilö_väri3", "1F481-1F3FD"},
    {"kämmentä_ylöspäin_näyttävä_henkilö_väri4", "1F481-1F3FE"},
    {"kämmentä_ylöspäin_näyttävä_henkilö_väri5", "1F481-1F3FF"},
    {"punasiniraitainen_pylväs", "1F488"},
    {"huulipunajälki", "1F48B"},
    {"pariskunta_ja_sydän", "1F491"},
    {"pariskunta_ja_sydän_väri1", "1F491-1F3FB"},
    {"pariskunta_ja_sydän_väri2", "1F491-1F3FC"},
    {"pariskunta_ja_sydän_väri3", "1F491-1F3FD"},
    {"pariskunta_ja_sydän_väri4", "1F491-1F3FE"},
    {"pariskunta_ja_sydän_väri5", "1F491-1F3FF"},
    {"pariskunta_ja_sydän_väri1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pariskunta_ja_sydän_väri1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pariskunta_ja_sydän_väri1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pariskunta_ja_sydän_väri1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pariskunta_ja_sydän_väri2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pariskunta_ja_sydän_väri2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pariskunta_ja_sydän_väri2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pariskunta_ja_sydän_väri2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pariskunta_ja_sydän_väri3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pariskunta_ja_sydän_väri3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pariskunta_ja_sydän_väri3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pariskunta_ja_sydän_väri3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pariskunta_ja_sydän_väri4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pariskunta_ja_sydän_väri4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pariskunta_ja_sydän_väri4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pariskunta_ja_sydän_väri4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pariskunta_ja_sydän_väri5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pariskunta_ja_sydän_väri5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pariskunta_ja_sydän_väri5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pariskunta_ja_sydän_väri5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"häät", "1F492"},
    {"sykkivä_sydän", "1F493"},
    {"särkynyt_sydän", "1F494"},
    {"kaksi_sydäntä", "1F495"},
    {"kimaltava_sydän", "1F496"},
    {"kasvava_sydän", "1F497"},
    {"sydän_ja_nuoli", "1F498"},
    {"sininen_sydän", "1F499"},
    {"vihreä_sydän", "1F49A"},
    {"keltainen_sydän", "1F49B"},
    {"violetti_sydän", "1F49C"},
    {"sydän_ja_lahjanauha", "1F49D"},
    {"toisiaan_kiertävät_sydämet", "1F49E"},
    {"sydänkoriste", "1F49F"},
    {"vinoneliö_kukka", "1F4A0"},
    {"törmäys", "1F4A5"},
    {"pikalähtö", "1F4A8"},
    {"pyörryttävä", "1F4AB"},
    {"sata_pistettä", "1F4AF"},
    {"rahasäkki", "1F4B0"},
    {"yen_ja_nouseva_käyrä", "1F4B9"},
    {"käännetty_sivu", "1F4C3"},
    {"päiväkalenteri", "1F4C6"},
    {"nouseva_käyrä", "1F4C8"},
    {"laskeva_käyrä", "1F4C9"},
    {"pylväskaavio", "1F4CA"},
    {"leikepöytä", "1F4CB"},
    {"pyöreä_nasta", "1F4CD"},
    {"kirjanmerkkivälilehdet", "1F4D1"},
    {"vihreä_kirja", "1F4D7"},
    {"kirjakäärö", "1F4DC"},
    {"kovaääninen", "1F4E2"},
    {"lähtevät", "1F4E4"},
    {"sähköposti", "1F4E7"},
    {"suljettu_postilaatikko_lippu_ylhäällä", "1F4EB"},
    {"avoin_postilaatikko_lippu_ylhäällä", "1F4EC"},
    {"kännykkä", "1F4F1"},
    {"kännykkä_ja_nuoli", "1F4F2"},
    {"värinätila", "1F4F3"},
    {"pystynuolet_myötäpäivään", "1F503"},
    {"nuolet_vastapäivään", "1F504"},
    {"himmennä", "1F505"},
    {"kaiutin_normaalilla_äänenvoimakkuudella", "1F509"},
    {"äänekäs_kaiutin", "1F50A"},
    {"lukko_ja_kynä", "1F50F"},
    {"kielletty_alaikäisiltä", "1F51E"},
    {"näppäin_10", "1F51F"},
    {"syötä_isoja_kirjaimia", "1F520"},
    {"syötä_pieniä_kirjaimia", "1F521"},
    {"syötä_numeroja", "1F522"},
    {"syötä_merkkejä", "1F523"},
    {"syötä_kirjaimia", "1F524"},
    {"keittiöveitsi", "1F52A"},
    {"kuusisakarainen_tähti_pisteellä", "1F52F"},
    {"kolmikärki", "1F531"},
    {"musta_neliöruutu", "1F532"},
    {"valkoinen_neliöruutu", "1F533"},
    {"punainen_ympyrä", "1F534"},
    {"sininen_ympyrä", "1F535"},
    {"iso_oranssi_vinoneliö", "1F536"},
    {"iso_sininen_vinoneliö", "1F537"},
    {"pieni_oranssi_vinoneliö", "1F538"},
    {"pieni_sininen_vinoneliö", "1F539"},
    {"punainen_kolmio_kärki_ylös", "1F53A"},
    {"punainen_kolmio_kärki_alas", "1F53B"},
    {"ylös", "1F53C"},
    {"kello_neljä", "1F553"},
    {"kello_seitsemän", "1F556"},
    {"kello_yhdeksän", "1F558"},
    {"puoli_neljä", "1F55E"},
    {"puoli_seitsemän", "1F561"},
    {"puoli_yhdeksän", "1F563"},
    {"kynttilä", "1F56F"},
    {"pöytäkello", "1F570"},
    {"etsivä", "1F575"},
    {"etsivä_väri1", "1F575-1F3FB"},
    {"etsivä_väri2", "1F575-1F3FC"},
    {"etsivä_väri3", "1F575-1F3FD"},
    {"etsivä_väri4", "1F575-1F3FE"},
    {"etsivä_väri5", "1F575-1F3FF"},
    {"hämähäkki", "1F577"},
    {"hämähäkin_seitti", "1F578"},
    {"klemmarit_yhdessä", "1F587"},
    {"kuulakärkikynä", "1F58A"},
    {"täytekynä", "1F58B"},
    {"väriliitu", "1F58D"},
    {"nostettu_käsi_ja_sormet_erillään", "1F590"},
    {"nostettu_käsi_ja_sormet_erillään_väri1", "1F590-1F3FB"},
    {"nostettu_käsi_ja_sormet_erillään_väri2", "1F590-1F3FC"},
    {"nostettu_käsi_ja_sormet_erillään_väri3", "1F590-1F3FD"},
    {"nostettu_käsi_ja_sormet_erillään_väri4", "1F590-1F3FE"},
    {"nostettu_käsi_ja_sormet_erillään_väri5", "1F590-1F3FF"},
    {"musta_sydän", "1F5A4"},
    {"pöytäkone", "1F5A5"},
    {"kuva_kehyksissä", "1F5BC"},
    {"kortiston_välilehdet", "1F5C2"},
    {"kierrelehtiö", "1F5D2"},
    {"puhuva_pää", "1F5E3"},
    {"ääniuurna", "1F5F3"},
    {"leveä_hymy", "1F600"},
    {"virnistys_ja_hymyilevät_silmät", "1F601"},
    {"kasvot_naurunkyynelissä", "1F602"},
    {"hymyilevät_silmät_ja_suu", "1F604"},
    {"hymy_ja_kylmänhiki", "1F605"},
    {"hymy_ja_tiukasti_suljetut_silmät", "1F606"},
    {"hymy_ja_sädekehä", "1F607"},
    {"silmänisku", "1F609"},
    {"hymy_ja_sydänsilmät", "1F60D"},
    {"ilmeetön", "1F611"},
    {"kylmänhiki", "1F613"},
    {"hämmentynyt", "1F615"},
    {"pusunaama_ja_hymyilevät_silmät", "1F619"},
    {"pusunaama_ja_suljetut_silmät", "1F61A"},
    {"kieli_ulkona_ja_silmänisku", "1F61C"},
    {"kieli_ulkona_ja_silmät_tiukasti_kiinni", "1F61D"},
    {"mököttävä", "1F621"},
    {"itkevä", "1F622"},
    {"kärsivä", "1F623"},
    {"höyryä_nenästä", "1F624"},
    {"väsynyt", "1F62B"},
    {"irvistävä", "1F62C"},
    {"ääneen_itkevä", "1F62D"},
    {"yllättynyt", "1F62E"},
    {"järkyttynyt", "1F62F"},
    {"avoin_suu_ja_kylmänhiki", "1F630"},
    {"ällistynyt", "1F632"},
    {"virnistävä_kissa_ja_hymyilevät_silmät", "1F638"},
    {"leveästi_hymyilevä_kissa", "1F63A"},
    {"kissa_ja_sydänsilmät", "1F63B"},
    {"vinosti_hymyilevä_kissa", "1F63C"},
    {"kissan_pusunaama_ja_suljetut_silmät", "1F63D"},
    {"mököttävä_kissa", "1F63E"},
    {"itkevä_kissa", "1F63F"},
    {"yllättynyt_kissa", "1F640"},
    {"ylösalainen_naama", "1F643"},
    {"silmien_pyöritys", "1F644"},
    {"kumartava_henkilö", "1F647"},
    {"kumartava_henkilö_väri1", "1F647-1F3FB"},
    {"kumartava_henkilö_väri2", "1F647-1F3FC"},
    {"kumartava_henkilö_väri3", "1F647-1F3FD"},
    {"kumartava_henkilö_väri4", "1F647-1F3FE"},
    {"kumartava_henkilö_väri5", "1F647-1F3FF"},
    {"en_näe_pahaa", "1F648"},
    {"iloinen_henkilö_viittaa", "1F64B"},
    {"iloinen_henkilö_viittaa_väri1", "1F64B-1F3FB"},
    {"iloinen_henkilö_viittaa_väri2", "1F64B-1F3FC"},
    {"iloinen_henkilö_viittaa_väri3", "1F64B-1F3FD"},
    {"iloinen_henkilö_viittaa_väri4", "1F64B-1F3FE"},
    {"iloinen_henkilö_viittaa_väri5", "1F64B-1F3FF"},
    {"nousevat_kädet", "1F64C"},
    {"nousevat_kädet_väri1", "1F64C-1F3FB"},
    {"nousevat_kädet_väri2", "1F64C-1F3FC"},
    {"nousevat_kädet_väri3", "1F64C-1F3FD"},
    {"nousevat_kädet_väri4", "1F64C-1F3FE"},
    {"nousevat_kädet_väri5", "1F64C-1F3FF"},
    {"surullinen_henkilö", "1F64D"},
    {"surullinen_henkilö_väri1", "1F64D-1F3FB"},
    {"surullinen_henkilö_väri2", "1F64D-1F3FC"},
    {"surullinen_henkilö_väri3", "1F64D-1F3FD"},
    {"surullinen_henkilö_väri4", "1F64D-1F3FE"},
    {"surullinen_henkilö_väri5", "1F64D-1F3FF"},
    {"mököttävä_henkilö", "1F64E"},
    {"mököttävä_henkilö_väri1", "1F64E-1F3FB"},
    {"mököttävä_henkilö_väri2", "1F64E-1F3FC"},
    {"mököttävä_henkilö_väri3", "1F64E-1F3FD"},
    {"mököttävä_henkilö_väri4", "1F64E-1F3FE"},
    {"mököttävä_henkilö_väri5", "1F64E-1F3FF"},
    {"kämmenet_yhdessä", "1F64F"},
    {"kämmenet_yhdessä_väri1", "1F64F-1F3FB"},
    {"kämmenet_yhdessä_väri2", "1F64F-1F3FC"},
    {"kämmenet_yhdessä_väri3", "1F64F-1F3FD"},
    {"kämmenet_yhdessä_väri4", "1F64F-1F3FE"},
    {"kämmenet_yhdessä_väri5", "1F64F-1F3FF"},
    {"bussipysäkki", "1F68F"},
    {"köysirata", "1F6A1"},
    {"rakennustyömaan_merkki", "1F6A7"},
    {"pääsy_kielletty", "1F6AB"},
    {"juomakelpoista_vettä", "1F6B0"},
    {"polkupyörä", "1F6B2"},
    {"polkupyörällä_ajo_kielletty", "1F6B3"},
    {"pyöräilijä", "1F6B4"},
    {"pyöräilijä_väri1", "1F6B4-1F3FB"},
    {"pyöräilijä_väri2", "1F6B4-1F3FC"},
    {"pyöräilijä_väri3", "1F6B4-1F3FD"},
    {"pyöräilijä_väri4", "1F6B4-1F3FE"},
    {"pyöräilijä_väri5", "1F6B4-1F3FF"},
    {"maastopyöräilijä", "1F6B5"},
    {"maastopyöräilijä_väri1", "1F6B5-1F3FB"},
    {"maastopyöräilijä_väri2", "1F6B5-1F3FC"},
    {"maastopyöräilijä_väri3", "1F6B5-1F3FD"},
    {"maastopyöräilijä_väri4", "1F6B5-1F3FE"},
    {"maastopyöräilijä_väri5", "1F6B5-1F3FF"},
    {"lapsia_liikenteessä", "1F6B8"},
    {"vessanpönttö", "1F6BD"},
    {"kylpijä", "1F6C0"},
    {"kylpijä_väri1", "1F6C0-1F3FB"},
    {"kylpijä_väri2", "1F6C0-1F3FC"},
    {"kylpijä_väri3", "1F6C0-1F3FD"},
    {"kylpijä_väri4", "1F6C0-1F3FE"},
    {"kylpijä_väri5", "1F6C0-1F3FF"},
    {"säilytyslokero", "1F6C5"},
    {"sänky", "1F6CF"},
    {"pyhä_rakennus", "1F6D0"},
    {"pysähtymismerkki", "1F6D1"},
    {"ostoskärryt", "1F6D2"},
    {"maanvyörymä", "1F6D8"},
    {"liukumäki", "1F6DD"},
    {"öljytynnyri", "1F6E2"},
    {"lähtevä_lentokone", "1F6EB"},
    {"lentävä_lautanen", "1F6F8"},
    {"autorikša", "1F6FA"},
    {"oranssi_ympyrä", "1F7E0"},
    {"keltainen_ympyrä", "1F7E1"},
    {"vihreä_ympyrä", "1F7E2"},
    {"violetti_ympyrä", "1F7E3"},
    {"ruskea_ympyrä", "1F7E4"},
    {"punainen_neliö", "1F7E5"},
    {"sininen_neliö", "1F7E6"},
    {"oranssi_neliö", "1F7E7"},
    {"keltainen_neliö", "1F7E8"},
    {"vihreä_neliö", "1F7E9"},
    {"violetti_neliö", "1F7EA"},
    {"ruskea_neliö", "1F7EB"},
    {"paksu_yhtäsuuruusmerkki", "1F7F0"},
    {"valkoinen_sydän", "1F90D"},
    {"ruskea_sydän", "1F90E"},
    {"nipistävä_käsi", "1F90F"},
    {"nipistävä_käsi_väri1", "1F90F-1F3FB"},
    {"nipistävä_käsi_väri2", "1F90F-1F3FC"},
    {"nipistävä_käsi_väri3", "1F90F-1F3FD"},
    {"nipistävä_käsi_väri4", "1F90F-1F3FE"},
    {"nipistävä_käsi_väri5", "1F90F-1F3FF"},
    {"dollarisilmät_ja_rahasuu", "1F911"},
    {"nörtti", "1F913"},
    {"mietteliäs", "1F914"},
    {"naama_ja_pääside", "1F915"},
    {"kämmenselkä_koholla", "1F91A"},
    {"kämmenselkä_koholla_väri1", "1F91A-1F3FB"},
    {"kämmenselkä_koholla_väri2", "1F91A-1F3FC"},
    {"kämmenselkä_koholla_väri3", "1F91A-1F3FD"},
    {"kämmenselkä_koholla_väri4", "1F91A-1F3FE"},
    {"kämmenselkä_koholla_väri5", "1F91A-1F3FF"},
    {"kädenpuristus", "1F91D"},
    {"kädenpuristus_väri1", "1F91D-1F3FB"},
    {"kädenpuristus_väri2", "1F91D-1F3FC"},
    {"kädenpuristus_väri3", "1F91D-1F3FD"},
    {"kädenpuristus_väri4", "1F91D-1F3FE"},
    {"kädenpuristus_väri5", "1F91D-1F3FF"},
    {"kädenpuristus_väri1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"kädenpuristus_väri1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"kädenpuristus_väri1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"kädenpuristus_väri1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"kädenpuristus_väri2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"kädenpuristus_väri2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"kädenpuristus_väri2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"kädenpuristus_väri2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"kädenpuristus_väri3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"kädenpuristus_väri3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"kädenpuristus_väri3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"kädenpuristus_väri3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"kädenpuristus_väri4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"kädenpuristus_väri4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"kädenpuristus_väri4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"kädenpuristus_väri4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"kädenpuristus_väri5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"kädenpuristus_väri5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"kädenpuristus_väri5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"kädenpuristus_väri5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"sormet_ristissä", "1F91E"},
    {"sormet_ristissä_väri1", "1F91E-1F3FB"},
    {"sormet_ristissä_väri2", "1F91E-1F3FC"},
    {"sormet_ristissä_väri3", "1F91E-1F3FD"},
    {"sormet_ristissä_väri4", "1F91E-1F3FE"},
    {"sormet_ristissä_väri5", "1F91E-1F3FF"},
    {"peittää_kasvot_kädellä", "1F926"},
    {"peittää_kasvot_kädellä_väri1", "1F926-1F3FB"},
    {"peittää_kasvot_kädellä_väri2", "1F926-1F3FC"},
    {"peittää_kasvot_kädellä_väri3", "1F926-1F3FD"},
    {"peittää_kasvot_kädellä_väri4", "1F926-1F3FE"},
    {"peittää_kasvot_kädellä_väri5", "1F926-1F3FF"},
    {"tähtisilmä", "1F929"},
    {"hyssyttelevä", "1F92B"},
    {"käsi_suun_edessä", "1F92D"},
    {"räjähdys_päässä", "1F92F"},
    {"kämmenet_vierekkäin", "1F932"},
    {"kämmenet_vierekkäin_väri1", "1F932-1F3FB"},
    {"kämmenet_vierekkäin_väri2", "1F932-1F3FC"},
    {"kämmenet_vierekkäin_väri3", "1F932-1F3FD"},
    {"kämmenet_vierekkäin_väri4", "1F932-1F3FE"},
    {"kämmenet_vierekkäin_väri5", "1F932-1F3FF"},
    {"henkilö_smokissa", "1F935"},
    {"henkilö_smokissa_väri1", "1F935-1F3FB"},
    {"henkilö_smokissa_väri2", "1F935-1F3FC"},
    {"henkilö_smokissa_väri3", "1F935-1F3FD"},
    {"henkilö_smokissa_väri4", "1F935-1F3FE"},
    {"henkilö_smokissa_väri5", "1F935-1F3FF"},
    {"olkapäitään_kohauttava_henkilö", "1F937"},
    {"olkapäitään_kohauttava_henkilö_väri1", "1F937-1F3FB"},
    {"olkapäitään_kohauttava_henkilö_väri2", "1F937-1F3FC"},
    {"olkapäitään_kohauttava_henkilö_väri3", "1F937-1F3FD"},
    {"olkapäitään_kohauttava_henkilö_väri4", "1F937-1F3FE"},
    {"olkapäitään_kohauttava_henkilö_väri5", "1F937-1F3FF"},
    {"kärrynpyörä", "1F938"},
    {"kärrynpyörä_väri1", "1F938-1F3FB"},
    {"kärrynpyörä_väri2", "1F938-1F3FC"},
    {"kärrynpyörä_väri3", "1F938-1F3FD"},
    {"kärrynpyörä_väri4", "1F938-1F3FE"},
    {"kärrynpyörä_väri5", "1F938-1F3FF"},
    {"käsipalloilija", "1F93E"},
    {"käsipalloilija_väri1", "1F93E-1F3FB"},
    {"käsipalloilija_väri2", "1F93E-1F3FC"},
    {"käsipalloilija_väri3", "1F93E-1F3FD"},
    {"käsipalloilija_väri4", "1F93E-1F3FE"},
    {"käsipalloilija_väri5", "1F93E-1F3FF"},
    {"kilisevät_lasit", "1F942"},
    {"vihreä_salaatti", "1F957"},
    {"täytetty_pita", "1F959"},
    {"maapähkinä", "1F95C"},
    {"syömäpuikot", "1F962"},
    {"kookospähkinä", "1F965"},
    {"rinkilä", "1F968"},
    {"voileipä", "1F96A"},
    {"säilyketölkki", "1F96B"},
    {"hymy_ja_kolme_sydäntä", "1F970"},
    {"hymyilevät_kasvot_ja_kyynel", "1F972"},
    {"kyyneliä_pidättelevä_naama", "1F979"},
    {"vaelluskenkä", "1F97E"},
    {"pöllö", "1F989"},
    {"mäyrä", "1F9A1"},
    {"haisunäätä", "1F9A8"},
    {"jalkaterä", "1F9B6"},
    {"jalkaterä_väri1", "1F9B6-1F3FB"},
    {"jalkaterä_väri2", "1F9B6-1F3FC"},
    {"jalkaterä_väri3", "1F9B6-1F3FD"},
    {"jalkaterä_väri4", "1F9B6-1F3FE"},
    {"jalkaterä_väri5", "1F9B6-1F3FF"},
    {"moottoroitu_pyörätuoli", "1F9BC"},
    {"käsikäyttöinen_pyörätuoli", "1F9BD"},
    {"tekokäsi", "1F9BE"},
    {"jääkuutio", "1F9CA"},
    {"seisova_henkilö", "1F9CD"},
    {"seisova_henkilö_väri1", "1F9CD-1F3FB"},
    {"seisova_henkilö_väri2", "1F9CD-1F3FC"},
    {"seisova_henkilö_väri3", "1F9CD-1F3FD"},
    {"seisova_henkilö_väri4", "1F9CD-1F3FE"},
    {"seisova_henkilö_väri5", "1F9CD-1F3FF"},
    {"kuuro_henkilö", "1F9CF"},
    {"kuuro_henkilö_väri1", "1F9CF-1F3FB"},
    {"kuuro_henkilö_väri2", "1F9CF-1F3FC"},
    {"kuuro_henkilö_väri3", "1F9CF-1F3FD"},
    {"kuuro_henkilö_väri4", "1F9CF-1F3FE"},
    {"kuuro_henkilö_väri5", "1F9CF-1F3FF"},
    {"monokkeli_silmässä", "1F9D0"},
    {"parrakas_henkilö", "1F9D4"},
    {"parrakas_henkilö_väri1", "1F9D4-1F3FB"},
    {"parrakas_henkilö_väri2", "1F9D4-1F3FC"},
    {"parrakas_henkilö_väri3", "1F9D4-1F3FD"},
    {"parrakas_henkilö_väri4", "1F9D4-1F3FE"},
    {"parrakas_henkilö_väri5", "1F9D4-1F3FF"},
    {"huivipäinen_nainen", "1F9D5"},
    {"huivipäinen_nainen_väri1", "1F9D5-1F3FB"},
    {"huivipäinen_nainen_väri2", "1F9D5-1F3FC"},
    {"huivipäinen_nainen_väri3", "1F9D5-1F3FD"},
    {"huivipäinen_nainen_väri4", "1F9D5-1F3FE"},
    {"huivipäinen_nainen_väri5", "1F9D5-1F3FF"},
    {"henkilö_saunassa", "1F9D6"},
    {"henkilö_saunassa_väri1", "1F9D6-1F3FB"},
    {"henkilö_saunassa_väri2", "1F9D6-1F3FC"},
    {"henkilö_saunassa_väri3", "1F9D6-1F3FD"},
    {"henkilö_saunassa_väri4", "1F9D6-1F3FE"},
    {"henkilö_saunassa_väri5", "1F9D6-1F3FF"},
    {"kiipeilijä", "1F9D7"},
    {"kiipeilijä_väri1", "1F9D7-1F3FB"},
    {"kiipeilijä_väri2", "1F9D7-1F3FC"},
    {"kiipeilijä_väri3", "1F9D7-1F3FD"},
    {"kiipeilijä_väri4", "1F9D7-1F3FE"},
    {"kiipeilijä_väri5", "1F9D7-1F3FF"},
    {"henkilö_lootusasennossa", "1F9D8"},
    {"henkilö_lootusasennossa_väri1", "1F9D8-1F3FB"},
    {"henkilö_lootusasennossa_väri2", "1F9D8-1F3FC"},
    {"henkilö_lootusasennossa_väri3", "1F9D8-1F3FD"},
    {"henkilö_lootusasennossa_väri4", "1F9D8-1F3FE"},
    {"henkilö_lootusasennossa_väri5", "1F9D8-1F3FF"},
    {"merenväki", "1F9DC"},
    {"merenväki_väri1", "1F9DC-1F3FB"},
    {"merenväki_väri2", "1F9DC-1F3FC"},
    {"merenväki_väri3", "1F9DC-1F3FD"},
    {"merenväki_väri4", "1F9DC-1F3FE"},
    {"merenväki_väri5", "1F9DC-1F3FF"},
    {"oranssi_sydän", "1F9E1"},
    {"sähikäinen", "1F9E8"},
    {"työkalulaatikko", "1F9F0"},
    {"lankakerä", "1F9F6"},
    {"vaaleansininen_sydän", "1FA75"},
    {"harmaa_sydän", "1FA76"},
    {"pinkki_sydän", "1FA77"},
    {"röntgen", "1FA7B"},
    {"piñata", "1FA85"},
    {"sotilaskypärä", "1FA96"},
    {"käsisaha", "1FA9A"},
    {"viemäripumppu", "1FAA0"},
    {"ämpäri", "1FAA3"},
    {"henkilökortti", "1FAAA"},
    {"akku_vähissä", "1FAAB"},
    {"taittuva_käsiviuhka", "1FAAD"},
    {"kärpänen", "1FAB0"},
    {"höyhen", "1FAB6"},
    {"tyhjä_pesä", "1FAB9"},
    {"munia_pesässä", "1FABA"},
    {"lehdetön_puu", "1FABE"},
    {"sydän", "1FAC0"},
    {"raskaana_oleva_henkilö", "1FAC4"},
    {"raskaana_oleva_henkilö_väri1", "1FAC4-1F3FB"},
    {"raskaana_oleva_henkilö_väri2", "1FAC4-1F3FC"},
    {"raskaana_oleva_henkilö_väri3", "1FAC4-1F3FD"},
    {"raskaana_oleva_henkilö_väri4", "1FAC4-1F3FE"},
    {"raskaana_oleva_henkilö_väri5", "1FAC4-1F3FF"},
    {"kruunua_kantava_henkilö", "1FAC5"},
    {"kruunua_kantava_henkilö_väri1", "1FAC5-1F3FB"},
    {"kruunua_kantava_henkilö_väri2", "1FAC5-1F3FC"},
    {"kruunua_kantava_henkilö_väri3", "1FAC5-1F3FD"},
    {"kruunua_kantava_henkilö_väri4", "1FAC5-1F3FE"},
    {"kruunua_kantava_henkilö_väri5", "1FAC5-1F3FF"},
    {"sormenjälki", "1FAC6"},
    {"inkivääri", "1FADA"},
    {"silmät_pyöreinä_käsi_suun_edessä", "1FAE2"},
    {"tärisevä", "1FAE8"},
    {"silmäpussit", "1FAE9"},
    {"vääristyneet_kasvot", "1FAEA"},
    {"etusormi_ja_peukalo_ristissä", "1FAF0"},
    {"etusormi_ja_peukalo_ristissä_väri1", "1FAF0-1F3FB"},
    {"etusormi_ja_peukalo_ristissä_väri2", "1FAF0-1F3FC"},
    {"etusormi_ja_peukalo_ristissä_väri3", "1FAF0-1F3FD"},
    {"etusormi_ja_peukalo_ristissä_väri4", "1FAF0-1F3FE"},
    {"etusormi_ja_peukalo_ristissä_väri5", "1FAF0-1F3FF"},
    {"oikealle_ojentuva_käsi", "1FAF1"},
    {"oikealle_ojentuva_käsi_väri1", "1FAF1-1F3FB"},
    {"oikealle_ojentuva_käsi_väri2", "1FAF1-1F3FC"},
    {"oikealle_ojentuva_käsi_väri3", "1FAF1-1F3FD"},
    {"oikealle_ojentuva_käsi_väri4", "1FAF1-1F3FE"},
    {"oikealle_ojentuva_käsi_väri5", "1FAF1-1F3FF"},
    {"vasemmalle_ojentuva_käsi", "1FAF2"},
    {"vasemmalle_ojentuva_käsi_väri1", "1FAF2-1F3FB"},
    {"vasemmalle_ojentuva_käsi_väri2", "1FAF2-1F3FC"},
    {"vasemmalle_ojentuva_käsi_väri3", "1FAF2-1F3FD"},
    {"vasemmalle_ojentuva_käsi_väri4", "1FAF2-1F3FE"},
    {"vasemmalle_ojentuva_käsi_väri5", "1FAF2-1F3FF"},
    {"käsi_kämmen_alaspäin", "1FAF3"},
    {"käsi_kämmen_alaspäin_väri1", "1FAF3-1F3FB"},
    {"käsi_kämmen_alaspäin_väri2", "1FAF3-1F3FC"},
    {"käsi_kämmen_alaspäin_väri3", "1FAF3-1F3FD"},
    {"käsi_kämmen_alaspäin_väri4", "1FAF3-1F3FE"},
    {"käsi_kämmen_alaspäin_väri5", "1FAF3-1F3FF"},
    {"käsi_kämmen_ylöspäin", "1FAF4"},
    {"käsi_kämmen_ylöspäin_väri1", "1FAF4-1F3FB"},
    {"käsi_kämmen_ylöspäin_väri2", "1FAF4-1F3FC"},
    {"käsi_kämmen_ylöspäin_väri3", "1FAF4-1F3FD"},
    {"käsi_kämmen_ylöspäin_väri4", "1FAF4-1F3FE"},
    {"käsi_kämmen_ylöspäin_väri5", "1FAF4-1F3FF"},
    {"sydänkuvio_käsillä", "1FAF6"},
    {"sydänkuvio_käsillä_väri1", "1FAF6-1F3FB"},
    {"sydänkuvio_käsillä_väri2", "1FAF6-1F3FC"},
    {"sydänkuvio_käsillä_väri3", "1FAF6-1F3FD"},
    {"sydänkuvio_käsillä_väri4", "1FAF6-1F3FE"},
    {"sydänkuvio_käsillä_väri5", "1FAF6-1F3FF"},
    {"vasemmalle_työntävä_käsi", "1FAF7"},
    {"vasemmalle_työntävä_käsi_väri1", "1FAF7-1F3FB"},
    {"vasemmalle_työntävä_käsi_väri2", "1FAF7-1F3FC"},
    {"vasemmalle_työntävä_käsi_väri3", "1FAF7-1F3FD"},
    {"vasemmalle_työntävä_käsi_väri4", "1FAF7-1F3FE"},
    {"vasemmalle_työntävä_käsi_väri5", "1FAF7-1F3FF"},
    {"oikealle_työntävä_käsi", "1FAF8"},
    {"oikealle_työntävä_käsi_väri1", "1FAF8-1F3FB"},
    {"oikealle_työntävä_käsi_väri2", "1FAF8-1F3FC"},
    {"oikealle_työntävä_käsi_väri3", "1FAF8-1F3FD"},
    {"oikealle_työntävä_käsi_väri4", "1FAF8-1F3FE"},
    {"oikealle_työntävä_käsi_väri5", "1FAF8-1F3FF"},
    {"näppäin_numeromerkki", "0023-FE0F-20E3"},
    {"näppäin_tähtimerkki", "002A-FE0F-20E3"},
    {"näppäin_0", "0030-FE0F-20E3"},
    {"näppäin_1", "0031-FE0F-20E3"},
    {"näppäin_2", "0032-FE0F-20E3"},
    {"näppäin_3", "0033-FE0F-20E3"},
    {"näppäin_4", "0034-FE0F-20E3"},
    {"näppäin_5", "0035-FE0F-20E3"},
    {"näppäin_6", "0036-FE0F-20E3"},
    {"näppäin_7", "0037-FE0F-20E3"},
    {"näppäin_8", "0038-FE0F-20E3"},
    {"näppäin_9", "0039-FE0F-20E3"},
    {"lippu_itävalta", "1F1E6-1F1F9"},
    {"lippu_azerbaidžan", "1F1E6-1F1FF"},
    {"lippu_saint_barthélemy", "1F1E7-1F1F1"},
    {"lippu_valko_venäjä", "1F1E7-1F1FE"},
    {"lippu_curaçao", "1F1E8-1F1FC"},
    {"lippu_tšekki", "1F1E8-1F1FF"},
    {"lippu_länsi_sahara", "1F1EA-1F1ED"},
    {"lippu_fidži", "1F1EB-1F1EF"},
    {"lippu_färsaaret", "1F1EB-1F1F4"},
    {"lippu_grönlanti", "1F1EC-1F1F1"},
    {"lippu_päiväntasaajan_guinea", "1F1EC-1F1F6"},
    {"lippu_etelä_georgia_ja_eteläiset_sandwichinsaaret", "1F1EC-1F1F8"},
    {"lippu_brittiläinen_intian_valtameren_alue", "1F1EE-1F1F4"},
    {"lippu_kambodža", "1F1F0-1F1ED"},
    {"lippu_etelä_korea", "1F1F0-1F1F7"},
    {"lippu_réunion", "1F1F7-1F1EA"},
    {"lippu_venäjä", "1F1F7-1F1FA"},
    {"lippu_etelä_sudan", "1F1F8-1F1F8"},
    {"lippu_são_tomé_ja_príncipe", "1F1F8-1F1F9"},
    {"lippu_tšad", "1F1F9-1F1E9"},
    {"lippu_ranskan_eteläiset_ja_antarktiset_alueet", "1F1F9-1F1EB"},
    {"lippu_tadžikistan", "1F1F9-1F1EF"},
    {"lippu_itä_timor", "1F1F9-1F1F1"},
    {"lippu_brittiläiset_neitsytsaaret", "1F1FB-1F1EC"},
    {"lippu_etelä_afrikka", "1F1FF-1F1E6"},
    {"pariskunta_ja_sydän_mies_mies", "1F468-200D-2764-FE0F-200D-1F468"},
    {"pariskunta_ja_sydän_mies_mies_väri1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydän_mies_mies_väri1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydän_mies_mies_väri1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydän_mies_mies_väri1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydän_mies_mies_väri1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydän_mies_mies_väri2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydän_mies_mies_väri2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydän_mies_mies_väri2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydän_mies_mies_väri2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydän_mies_mies_väri2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydän_mies_mies_väri3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydän_mies_mies_väri3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydän_mies_mies_väri3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydän_mies_mies_väri3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydän_mies_mies_väri3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydän_mies_mies_väri4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydän_mies_mies_väri4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydän_mies_mies_väri4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydän_mies_mies_väri4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydän_mies_mies_väri4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydän_mies_mies_väri5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydän_mies_mies_väri5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydän_mies_mies_väri5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydän_mies_mies_väri5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydän_mies_mies_väri5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"perhe_mies_tyttö", "1F468-200D-1F467"},
    {"perhe_mies_tyttö_poika", "1F468-200D-1F467-200D-1F466"},
    {"perhe_mies_tyttö_tyttö", "1F468-200D-1F467-200D-1F467"},
    {"perhe_mies_mies_tyttö", "1F468-200D-1F468-200D-1F467"},
    {"perhe_mies_mies_tyttö_poika", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"perhe_mies_mies_tyttö_tyttö", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"perhe_mies_nainen_tyttö", "1F468-200D-1F469-200D-1F467"},
    {"perhe_mies_nainen_tyttö_poika", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"perhe_mies_nainen_tyttö_tyttö", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"pariskunta_ja_sydän_nainen_mies", "1F469-200D-2764-FE0F-200D-1F468"},
    {"pariskunta_ja_sydän_nainen_mies_väri1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydän_nainen_mies_väri1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydän_nainen_mies_väri1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydän_nainen_mies_väri1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydän_nainen_mies_väri1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydän_nainen_mies_väri2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydän_nainen_mies_väri2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydän_nainen_mies_väri2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydän_nainen_mies_väri2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydän_nainen_mies_väri2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydän_nainen_mies_väri3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydän_nainen_mies_väri3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydän_nainen_mies_väri3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydän_nainen_mies_väri3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydän_nainen_mies_väri3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydän_nainen_mies_väri4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydän_nainen_mies_väri4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydän_nainen_mies_väri4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydän_nainen_mies_väri4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydän_nainen_mies_väri4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydän_nainen_mies_väri5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pariskunta_ja_sydän_nainen_mies_väri5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pariskunta_ja_sydän_nainen_mies_väri5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pariskunta_ja_sydän_nainen_mies_väri5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pariskunta_ja_sydän_nainen_mies_väri5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pariskunta_ja_sydän_nainen_nainen", "1F469-200D-2764-FE0F-200D-1F469"},
    {"pariskunta_ja_sydän_nainen_nainen_väri1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pariskunta_ja_sydän_nainen_nainen_väri1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pariskunta_ja_sydän_nainen_nainen_väri1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pariskunta_ja_sydän_nainen_nainen_väri1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pariskunta_ja_sydän_nainen_nainen_väri1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pariskunta_ja_sydän_nainen_nainen_väri2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pariskunta_ja_sydän_nainen_nainen_väri2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pariskunta_ja_sydän_nainen_nainen_väri2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pariskunta_ja_sydän_nainen_nainen_väri2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pariskunta_ja_sydän_nainen_nainen_väri2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pariskunta_ja_sydän_nainen_nainen_väri3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pariskunta_ja_sydän_nainen_nainen_väri3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pariskunta_ja_sydän_nainen_nainen_väri3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pariskunta_ja_sydän_nainen_nainen_väri3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pariskunta_ja_sydän_nainen_nainen_väri3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pariskunta_ja_sydän_nainen_nainen_väri4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pariskunta_ja_sydän_nainen_nainen_väri4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pariskunta_ja_sydän_nainen_nainen_väri4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pariskunta_ja_sydän_nainen_nainen_väri4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pariskunta_ja_sydän_nainen_nainen_väri4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pariskunta_ja_sydän_nainen_nainen_väri5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pariskunta_ja_sydän_nainen_nainen_väri5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pariskunta_ja_sydän_nainen_nainen_väri5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pariskunta_ja_sydän_nainen_nainen_väri5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pariskunta_ja_sydän_nainen_nainen_väri5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"perhe_nainen_tyttö", "1F469-200D-1F467"},
    {"perhe_nainen_tyttö_poika", "1F469-200D-1F467-200D-1F466"},
    {"perhe_nainen_tyttö_tyttö", "1F469-200D-1F467-200D-1F467"},
    {"perhe_nainen_nainen_tyttö", "1F469-200D-1F469-200D-1F467"},
    {"perhe_nainen_nainen_tyttö_poika", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"perhe_nainen_nainen_tyttö_tyttö", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"ihmiset_käsi_kädessä", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"ihmiset_käsi_kädessä_väri1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"ihmiset_käsi_kädessä_väri1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"ihmiset_käsi_kädessä_väri1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"ihmiset_käsi_kädessä_väri1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"ihmiset_käsi_kädessä_väri1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"ihmiset_käsi_kädessä_väri2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"ihmiset_käsi_kädessä_väri2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"ihmiset_käsi_kädessä_väri2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"ihmiset_käsi_kädessä_väri2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"ihmiset_käsi_kädessä_väri2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"ihmiset_käsi_kädessä_väri3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"ihmiset_käsi_kädessä_väri3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"ihmiset_käsi_kädessä_väri3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"ihmiset_käsi_kädessä_väri3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"ihmiset_käsi_kädessä_väri3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"ihmiset_käsi_kädessä_väri4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"ihmiset_käsi_kädessä_väri4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"ihmiset_käsi_kädessä_väri4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"ihmiset_käsi_kädessä_väri4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"ihmiset_käsi_kädessä_väri4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"ihmiset_käsi_kädessä_väri5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"ihmiset_käsi_kädessä_väri5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"ihmiset_käsi_kädessä_väri5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"ihmiset_käsi_kädessä_väri5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"ihmiset_käsi_kädessä_väri5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"mieshoitotyöntekijä", "1F468-200D-2695-FE0F"},
    {"mieshoitotyöntekijä_väri1", "1F468-1F3FB-200D-2695-FE0F"},
    {"mieshoitotyöntekijä_väri2", "1F468-1F3FC-200D-2695-FE0F"},
    {"mieshoitotyöntekijä_väri3", "1F468-1F3FD-200D-2695-FE0F"},
    {"mieshoitotyöntekijä_väri4", "1F468-1F3FE-200D-2695-FE0F"},
    {"mieshoitotyöntekijä_väri5", "1F468-1F3FF-200D-2695-FE0F"},
    {"miesmaanviljelijä", "1F468-200D-1F33E"},
    {"miesmaanviljelijä_väri1", "1F468-1F3FB-200D-1F33E"},
    {"miesmaanviljelijä_väri2", "1F468-1F3FC-200D-1F33E"},
    {"miesmaanviljelijä_väri3", "1F468-1F3FD-200D-1F33E"},
    {"miesmaanviljelijä_väri4", "1F468-1F3FE-200D-1F33E"},
    {"miesmaanviljelijä_väri5", "1F468-1F3FF-200D-1F33E"},
    {"vauvaa_syöttävä_mies", "1F468-200D-1F37C"},
    {"vauvaa_syöttävä_mies_väri1", "1F468-1F3FB-200D-1F37C"},
    {"vauvaa_syöttävä_mies_väri2", "1F468-1F3FC-200D-1F37C"},
    {"vauvaa_syöttävä_mies_väri3", "1F468-1F3FD-200D-1F37C"},
    {"vauvaa_syöttävä_mies_väri4", "1F468-1F3FE-200D-1F37C"},
    {"vauvaa_syöttävä_mies_väri5", "1F468-1F3FF-200D-1F37C"},
    {"miestehdastyöntekijä", "1F468-200D-1F3ED"},
    {"miestehdastyöntekijä_väri1", "1F468-1F3FB-200D-1F3ED"},
    {"miestehdastyöntekijä_väri2", "1F468-1F3FC-200D-1F3ED"},
    {"miestehdastyöntekijä_väri3", "1F468-1F3FD-200D-1F3ED"},
    {"miestehdastyöntekijä_väri4", "1F468-1F3FE-200D-1F3ED"},
    {"miestehdastyöntekijä_väri5", "1F468-1F3FF-200D-1F3ED"},
    {"miestoimistotyöntekijä", "1F468-200D-1F4BC"},
    {"miestoimistotyöntekijä_väri1", "1F468-1F3FB-200D-1F4BC"},
    {"miestoimistotyöntekijä_väri2", "1F468-1F3FC-200D-1F4BC"},
    {"miestoimistotyöntekijä_väri3", "1F468-1F3FD-200D-1F4BC"},
    {"miestoimistotyöntekijä_väri4", "1F468-1F3FE-200D-1F4BC"},
    {"miestoimistotyöntekijä_väri5", "1F468-1F3FF-200D-1F4BC"},
    {"mies_moottoroidussa_pyörätuolissa", "1F468-200D-1F9BC"},
    {"mies_moottoroidussa_pyörätuolissa_väri1", "1F468-1F3FB-200D-1F9BC"},
    {"mies_moottoroidussa_pyörätuolissa_väri2", "1F468-1F3FC-200D-1F9BC"},
    {"mies_moottoroidussa_pyörätuolissa_väri3", "1F468-1F3FD-200D-1F9BC"},
    {"mies_moottoroidussa_pyörätuolissa_väri4", "1F468-1F3FE-200D-1F9BC"},
    {"mies_moottoroidussa_pyörätuolissa_väri5", "1F468-1F3FF-200D-1F9BC"},
    {"mies_moottoroidussa_pyörätuolissa_kohti_oikeaa", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"mies_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"mies_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"mies_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"mies_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"mies_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"mies_käsikäyttöisessä_pyörätuolissa", "1F468-200D-1F9BD"},
    {"mies_käsikäyttöisessä_pyörätuolissa_väri1", "1F468-1F3FB-200D-1F9BD"},
    {"mies_käsikäyttöisessä_pyörätuolissa_väri2", "1F468-1F3FC-200D-1F9BD"},
    {"mies_käsikäyttöisessä_pyörätuolissa_väri3", "1F468-1F3FD-200D-1F9BD"},
    {"mies_käsikäyttöisessä_pyörätuolissa_väri4", "1F468-1F3FE-200D-1F9BD"},
    {"mies_käsikäyttöisessä_pyörätuolissa_väri5", "1F468-1F3FF-200D-1F9BD"},
    {"mies_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"mies_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"mies_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"mies_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"mies_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"mies_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"naishoitotyöntekijä", "1F469-200D-2695-FE0F"},
    {"naishoitotyöntekijä_väri1", "1F469-1F3FB-200D-2695-FE0F"},
    {"naishoitotyöntekijä_väri2", "1F469-1F3FC-200D-2695-FE0F"},
    {"naishoitotyöntekijä_väri3", "1F469-1F3FD-200D-2695-FE0F"},
    {"naishoitotyöntekijä_väri4", "1F469-1F3FE-200D-2695-FE0F"},
    {"naishoitotyöntekijä_väri5", "1F469-1F3FF-200D-2695-FE0F"},
    {"naismaanviljelijä", "1F469-200D-1F33E"},
    {"naismaanviljelijä_väri1", "1F469-1F3FB-200D-1F33E"},
    {"naismaanviljelijä_väri2", "1F469-1F3FC-200D-1F33E"},
    {"naismaanviljelijä_väri3", "1F469-1F3FD-200D-1F33E"},
    {"naismaanviljelijä_väri4", "1F469-1F3FE-200D-1F33E"},
    {"naismaanviljelijä_väri5", "1F469-1F3FF-200D-1F33E"},
    {"vauvaa_syöttävä_nainen", "1F469-200D-1F37C"},
    {"vauvaa_syöttävä_nainen_väri1", "1F469-1F3FB-200D-1F37C"},
    {"vauvaa_syöttävä_nainen_väri2", "1F469-1F3FC-200D-1F37C"},
    {"vauvaa_syöttävä_nainen_väri3", "1F469-1F3FD-200D-1F37C"},
    {"vauvaa_syöttävä_nainen_väri4", "1F469-1F3FE-200D-1F37C"},
    {"vauvaa_syöttävä_nainen_väri5", "1F469-1F3FF-200D-1F37C"},
    {"naistehdastyöntekijä", "1F469-200D-1F3ED"},
    {"naistehdastyöntekijä_väri1", "1F469-1F3FB-200D-1F3ED"},
    {"naistehdastyöntekijä_väri2", "1F469-1F3FC-200D-1F3ED"},
    {"naistehdastyöntekijä_väri3", "1F469-1F3FD-200D-1F3ED"},
    {"naistehdastyöntekijä_väri4", "1F469-1F3FE-200D-1F3ED"},
    {"naistehdastyöntekijä_väri5", "1F469-1F3FF-200D-1F3ED"},
    {"naistoimistotyöntekijä", "1F469-200D-1F4BC"},
    {"naistoimistotyöntekijä_väri1", "1F469-1F3FB-200D-1F4BC"},
    {"naistoimistotyöntekijä_väri2", "1F469-1F3FC-200D-1F4BC"},
    {"naistoimistotyöntekijä_väri3", "1F469-1F3FD-200D-1F4BC"},
    {"naistoimistotyöntekijä_väri4", "1F469-1F3FE-200D-1F4BC"},
    {"naistoimistotyöntekijä_väri5", "1F469-1F3FF-200D-1F4BC"},
    {"nainen_moottoroidussa_pyörätuolissa", "1F469-200D-1F9BC"},
    {"nainen_moottoroidussa_pyörätuolissa_väri1", "1F469-1F3FB-200D-1F9BC"},
    {"nainen_moottoroidussa_pyörätuolissa_väri2", "1F469-1F3FC-200D-1F9BC"},
    {"nainen_moottoroidussa_pyörätuolissa_väri3", "1F469-1F3FD-200D-1F9BC"},
    {"nainen_moottoroidussa_pyörätuolissa_väri4", "1F469-1F3FE-200D-1F9BC"},
    {"nainen_moottoroidussa_pyörätuolissa_väri5", "1F469-1F3FF-200D-1F9BC"},
    {"nainen_moottoroidussa_pyörätuolissa_kohti_oikeaa", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"nainen_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"nainen_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"nainen_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"nainen_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"nainen_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"nainen_käsikäyttöisessä_pyörätuolissa", "1F469-200D-1F9BD"},
    {"nainen_käsikäyttöisessä_pyörätuolissa_väri1", "1F469-1F3FB-200D-1F9BD"},
    {"nainen_käsikäyttöisessä_pyörätuolissa_väri2", "1F469-1F3FC-200D-1F9BD"},
    {"nainen_käsikäyttöisessä_pyörätuolissa_väri3", "1F469-1F3FD-200D-1F9BD"},
    {"nainen_käsikäyttöisessä_pyörätuolissa_väri4", "1F469-1F3FE-200D-1F9BD"},
    {"nainen_käsikäyttöisessä_pyörätuolissa_väri5", "1F469-1F3FF-200D-1F9BD"},
    {"nainen_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"nainen_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"nainen_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"nainen_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"nainen_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"nainen_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"hoitotyöntekijä", "1F9D1-200D-2695-FE0F"},
    {"hoitotyöntekijä_väri1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"hoitotyöntekijä_väri2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"hoitotyöntekijä_väri3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"hoitotyöntekijä_väri4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"hoitotyöntekijä_väri5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"maanviljelijä", "1F9D1-200D-1F33E"},
    {"maanviljelijä_väri1", "1F9D1-1F3FB-200D-1F33E"},
    {"maanviljelijä_väri2", "1F9D1-1F3FC-200D-1F33E"},
    {"maanviljelijä_väri3", "1F9D1-1F3FD-200D-1F33E"},
    {"maanviljelijä_väri4", "1F9D1-1F3FE-200D-1F33E"},
    {"maanviljelijä_väri5", "1F9D1-1F3FF-200D-1F33E"},
    {"vauvaa_syöttävä_henkilö", "1F9D1-200D-1F37C"},
    {"vauvaa_syöttävä_henkilö_väri1", "1F9D1-1F3FB-200D-1F37C"},
    {"vauvaa_syöttävä_henkilö_väri2", "1F9D1-1F3FC-200D-1F37C"},
    {"vauvaa_syöttävä_henkilö_väri3", "1F9D1-1F3FD-200D-1F37C"},
    {"vauvaa_syöttävä_henkilö_väri4", "1F9D1-1F3FE-200D-1F37C"},
    {"vauvaa_syöttävä_henkilö_väri5", "1F9D1-1F3FF-200D-1F37C"},
    {"tehdastyöntekijä", "1F9D1-200D-1F3ED"},
    {"tehdastyöntekijä_väri1", "1F9D1-1F3FB-200D-1F3ED"},
    {"tehdastyöntekijä_väri2", "1F9D1-1F3FC-200D-1F3ED"},
    {"tehdastyöntekijä_väri3", "1F9D1-1F3FD-200D-1F3ED"},
    {"tehdastyöntekijä_väri4", "1F9D1-1F3FE-200D-1F3ED"},
    {"tehdastyöntekijä_väri5", "1F9D1-1F3FF-200D-1F3ED"},
    {"it_työntekijä", "1F9D1-200D-1F4BB"},
    {"it_työntekijä_väri1", "1F9D1-1F3FB-200D-1F4BB"},
    {"it_työntekijä_väri2", "1F9D1-1F3FC-200D-1F4BB"},
    {"it_työntekijä_väri3", "1F9D1-1F3FD-200D-1F4BB"},
    {"it_työntekijä_väri4", "1F9D1-1F3FE-200D-1F4BB"},
    {"it_työntekijä_väri5", "1F9D1-1F3FF-200D-1F4BB"},
    {"toimistotyöntekijä", "1F9D1-200D-1F4BC"},
    {"toimistotyöntekijä_väri1", "1F9D1-1F3FB-200D-1F4BC"},
    {"toimistotyöntekijä_väri2", "1F9D1-1F3FC-200D-1F4BC"},
    {"toimistotyöntekijä_väri3", "1F9D1-1F3FD-200D-1F4BC"},
    {"toimistotyöntekijä_väri4", "1F9D1-1F3FE-200D-1F4BC"},
    {"toimistotyöntekijä_väri5", "1F9D1-1F3FF-200D-1F4BC"},
    {"henkilö_ja_valkoinen_keppi", "1F9D1-200D-1F9AF"},
    {"henkilö_ja_valkoinen_keppi_väri1", "1F9D1-1F3FB-200D-1F9AF"},
    {"henkilö_ja_valkoinen_keppi_väri2", "1F9D1-1F3FC-200D-1F9AF"},
    {"henkilö_ja_valkoinen_keppi_väri3", "1F9D1-1F3FD-200D-1F9AF"},
    {"henkilö_ja_valkoinen_keppi_väri4", "1F9D1-1F3FE-200D-1F9AF"},
    {"henkilö_ja_valkoinen_keppi_väri5", "1F9D1-1F3FF-200D-1F9AF"},
    {"henkilö_ja_valkoinen_keppi_kohti_oikeaa", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"henkilö_ja_valkoinen_keppi_kohti_oikeaa_väri1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"henkilö_ja_valkoinen_keppi_kohti_oikeaa_väri2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"henkilö_ja_valkoinen_keppi_kohti_oikeaa_väri3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"henkilö_ja_valkoinen_keppi_kohti_oikeaa_väri4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"henkilö_ja_valkoinen_keppi_kohti_oikeaa_väri5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"henkilö_moottoroidussa_pyörätuolissa", "1F9D1-200D-1F9BC"},
    {"henkilö_moottoroidussa_pyörätuolissa_väri1", "1F9D1-1F3FB-200D-1F9BC"},
    {"henkilö_moottoroidussa_pyörätuolissa_väri2", "1F9D1-1F3FC-200D-1F9BC"},
    {"henkilö_moottoroidussa_pyörätuolissa_väri3", "1F9D1-1F3FD-200D-1F9BC"},
    {"henkilö_moottoroidussa_pyörätuolissa_väri4", "1F9D1-1F3FE-200D-1F9BC"},
    {"henkilö_moottoroidussa_pyörätuolissa_väri5", "1F9D1-1F3FF-200D-1F9BC"},
    {"henkilö_moottoroidussa_pyörätuolissa_kohti_oikeaa", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"henkilö_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"henkilö_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"henkilö_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"henkilö_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"henkilö_moottoroidussa_pyörätuolissa_kohti_oikeaa_väri5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"henkilö_käsikäyttöisessä_pyörätuolissa", "1F9D1-200D-1F9BD"},
    {"henkilö_käsikäyttöisessä_pyörätuolissa_väri1", "1F9D1-1F3FB-200D-1F9BD"},
    {"henkilö_käsikäyttöisessä_pyörätuolissa_väri2", "1F9D1-1F3FC-200D-1F9BD"},
    {"henkilö_käsikäyttöisessä_pyörätuolissa_väri3", "1F9D1-1F3FD-200D-1F9BD"},
    {"henkilö_käsikäyttöisessä_pyörätuolissa_väri4", "1F9D1-1F3FE-200D-1F9BD"},
    {"henkilö_käsikäyttöisessä_pyörätuolissa_väri5", "1F9D1-1F3FF-200D-1F9BD"},
    {"henkilö_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"henkilö_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"henkilö_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"henkilö_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"henkilö_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"henkilö_käsikäyttöisessä_pyörätuolissa_kohti_oikeaa_väri5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"tanssivat_puputytöt", "1F46F-200D-2640-FE0F"},
    {"tanssivat_puputytöt_väri1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"tanssivat_puputytöt_väri1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"tanssivat_puputytöt_väri1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"tanssivat_puputytöt_väri1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"tanssivat_puputytöt_väri2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"tanssivat_puputytöt_väri2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"tanssivat_puputytöt_väri2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"tanssivat_puputytöt_väri2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"tanssivat_puputytöt_väri3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"tanssivat_puputytöt_väri3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"tanssivat_puputytöt_väri3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"tanssivat_puputytöt_väri3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"tanssivat_puputytöt_väri4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"tanssivat_puputytöt_väri4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"tanssivat_puputytöt_väri4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"tanssivat_puputytöt_väri4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"tanssivat_puputytöt_väri5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"tanssivat_puputytöt_väri5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"tanssivat_puputytöt_väri5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"tanssivat_puputytöt_väri5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"tanssivat_puputytöt_väri1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"tanssivat_puputytöt_väri2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"tanssivat_puputytöt_väri3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"tanssivat_puputytöt_väri4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"tanssivat_puputytöt_väri5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"huntupäinen_nainen", "1F470-200D-2640-FE0F"},
    {"huntupäinen_nainen_väri1", "1F470-1F3FB-200D-2640-FE0F"},
    {"huntupäinen_nainen_väri2", "1F470-1F3FC-200D-2640-FE0F"},
    {"huntupäinen_nainen_väri3", "1F470-1F3FD-200D-2640-FE0F"},
    {"huntupäinen_nainen_väri4", "1F470-1F3FE-200D-2640-FE0F"},
    {"huntupäinen_nainen_väri5", "1F470-1F3FF-200D-2640-FE0F"},
    {"huntupäinen_mies", "1F470-200D-2642-FE0F"},
    {"huntupäinen_mies_väri1", "1F470-1F3FB-200D-2642-FE0F"},
    {"huntupäinen_mies_väri2", "1F470-1F3FC-200D-2642-FE0F"},
    {"huntupäinen_mies_väri3", "1F470-1F3FD-200D-2642-FE0F"},
    {"huntupäinen_mies_väri4", "1F470-1F3FE-200D-2642-FE0F"},
    {"huntupäinen_mies_väri5", "1F470-1F3FF-200D-2642-FE0F"},
    {"turbaanipäinen_nainen", "1F473-200D-2640-FE0F"},
    {"turbaanipäinen_nainen_väri1", "1F473-1F3FB-200D-2640-FE0F"},
    {"turbaanipäinen_nainen_väri2", "1F473-1F3FC-200D-2640-FE0F"},
    {"turbaanipäinen_nainen_väri3", "1F473-1F3FD-200D-2640-FE0F"},
    {"turbaanipäinen_nainen_väri4", "1F473-1F3FE-200D-2640-FE0F"},
    {"turbaanipäinen_nainen_väri5", "1F473-1F3FF-200D-2640-FE0F"},
    {"turbaanipäinen_mies", "1F473-200D-2642-FE0F"},
    {"turbaanipäinen_mies_väri1", "1F473-1F3FB-200D-2642-FE0F"},
    {"turbaanipäinen_mies_väri2", "1F473-1F3FC-200D-2642-FE0F"},
    {"turbaanipäinen_mies_väri3", "1F473-1F3FD-200D-2642-FE0F"},
    {"turbaanipäinen_mies_väri4", "1F473-1F3FE-200D-2642-FE0F"},
    {"turbaanipäinen_mies_väri5", "1F473-1F3FF-200D-2642-FE0F"},
    {"kämmentä_ylöspäin_näyttävä_nainen", "1F481-200D-2640-FE0F"},
    {"kämmentä_ylöspäin_näyttävä_nainen_väri1", "1F481-1F3FB-200D-2640-FE0F"},
    {"kämmentä_ylöspäin_näyttävä_nainen_väri2", "1F481-1F3FC-200D-2640-FE0F"},
    {"kämmentä_ylöspäin_näyttävä_nainen_väri3", "1F481-1F3FD-200D-2640-FE0F"},
    {"kämmentä_ylöspäin_näyttävä_nainen_väri4", "1F481-1F3FE-200D-2640-FE0F"},
    {"kämmentä_ylöspäin_näyttävä_nainen_väri5", "1F481-1F3FF-200D-2640-FE0F"},
    {"kämmentä_ylöspäin_näyttävä_mies", "1F481-200D-2642-FE0F"},
    {"kämmentä_ylöspäin_näyttävä_mies_väri1", "1F481-1F3FB-200D-2642-FE0F"},
    {"kämmentä_ylöspäin_näyttävä_mies_väri2", "1F481-1F3FC-200D-2642-FE0F"},
    {"kämmentä_ylöspäin_näyttävä_mies_väri3", "1F481-1F3FD-200D-2642-FE0F"},
    {"kämmentä_ylöspäin_näyttävä_mies_väri4", "1F481-1F3FE-200D-2642-FE0F"},
    {"kämmentä_ylöspäin_näyttävä_mies_väri5", "1F481-1F3FF-200D-2642-FE0F"},
    {"naisetsivä", "1F575-FE0F-200D-2640-FE0F"},
    {"naisetsivä_väri1", "1F575-1F3FB-200D-2640-FE0F"},
    {"naisetsivä_väri2", "1F575-1F3FC-200D-2640-FE0F"},
    {"naisetsivä_väri3", "1F575-1F3FD-200D-2640-FE0F"},
    {"naisetsivä_väri4", "1F575-1F3FE-200D-2640-FE0F"},
    {"naisetsivä_väri5", "1F575-1F3FF-200D-2640-FE0F"},
    {"miesetsivä", "1F575-FE0F-200D-2642-FE0F"},
    {"miesetsivä_väri1", "1F575-1F3FB-200D-2642-FE0F"},
    {"miesetsivä_väri2", "1F575-1F3FC-200D-2642-FE0F"},
    {"miesetsivä_väri3", "1F575-1F3FD-200D-2642-FE0F"},
    {"miesetsivä_väri4", "1F575-1F3FE-200D-2642-FE0F"},
    {"miesetsivä_väri5", "1F575-1F3FF-200D-2642-FE0F"},
    {"ei_käy_merkkiä_näyttävä_nainen", "1F645-200D-2640-FE0F"},
    {"ei_käy_merkkiä_näyttävä_nainen_väri1", "1F645-1F3FB-200D-2640-FE0F"},
    {"ei_käy_merkkiä_näyttävä_nainen_väri2", "1F645-1F3FC-200D-2640-FE0F"},
    {"ei_käy_merkkiä_näyttävä_nainen_väri3", "1F645-1F3FD-200D-2640-FE0F"},
    {"ei_käy_merkkiä_näyttävä_nainen_väri4", "1F645-1F3FE-200D-2640-FE0F"},
    {"ei_käy_merkkiä_näyttävä_nainen_väri5", "1F645-1F3FF-200D-2640-FE0F"},
    {"ei_käy_merkkiä_näyttävä_mies", "1F645-200D-2642-FE0F"},
    {"ei_käy_merkkiä_näyttävä_mies_väri1", "1F645-1F3FB-200D-2642-FE0F"},
    {"ei_käy_merkkiä_näyttävä_mies_väri2", "1F645-1F3FC-200D-2642-FE0F"},
    {"ei_käy_merkkiä_näyttävä_mies_väri3", "1F645-1F3FD-200D-2642-FE0F"},
    {"ei_käy_merkkiä_näyttävä_mies_väri4", "1F645-1F3FE-200D-2642-FE0F"},
    {"ei_käy_merkkiä_näyttävä_mies_väri5", "1F645-1F3FF-200D-2642-FE0F"},
    {"ok_merkkiä_näyttävä_nainen", "1F646-200D-2640-FE0F"},
    {"ok_merkkiä_näyttävä_nainen_väri1", "1F646-1F3FB-200D-2640-FE0F"},
    {"ok_merkkiä_näyttävä_nainen_väri2", "1F646-1F3FC-200D-2640-FE0F"},
    {"ok_merkkiä_näyttävä_nainen_väri3", "1F646-1F3FD-200D-2640-FE0F"},
    {"ok_merkkiä_näyttävä_nainen_väri4", "1F646-1F3FE-200D-2640-FE0F"},
    {"ok_merkkiä_näyttävä_nainen_väri5", "1F646-1F3FF-200D-2640-FE0F"},
    {"ok_merkkiä_näyttävä_mies", "1F646-200D-2642-FE0F"},
    {"ok_merkkiä_näyttävä_mies_väri1", "1F646-1F3FB-200D-2642-FE0F"},
    {"ok_merkkiä_näyttävä_mies_väri2", "1F646-1F3FC-200D-2642-FE0F"},
    {"ok_merkkiä_näyttävä_mies_väri3", "1F646-1F3FD-200D-2642-FE0F"},
    {"ok_merkkiä_näyttävä_mies_väri4", "1F646-1F3FE-200D-2642-FE0F"},
    {"ok_merkkiä_näyttävä_mies_väri5", "1F646-1F3FF-200D-2642-FE0F"},
    {"syvään_kumartava_nainen", "1F647-200D-2640-FE0F"},
    {"syvään_kumartava_nainen_väri1", "1F647-1F3FB-200D-2640-FE0F"},
    {"syvään_kumartava_nainen_väri2", "1F647-1F3FC-200D-2640-FE0F"},
    {"syvään_kumartava_nainen_väri3", "1F647-1F3FD-200D-2640-FE0F"},
    {"syvään_kumartava_nainen_väri4", "1F647-1F3FE-200D-2640-FE0F"},
    {"syvään_kumartava_nainen_väri5", "1F647-1F3FF-200D-2640-FE0F"},
    {"syvään_kumartava_mies", "1F647-200D-2642-FE0F"},
    {"syvään_kumartava_mies_väri1", "1F647-1F3FB-200D-2642-FE0F"},
    {"syvään_kumartava_mies_väri2", "1F647-1F3FC-200D-2642-FE0F"},
    {"syvään_kumartava_mies_väri3", "1F647-1F3FD-200D-2642-FE0F"},
    {"syvään_kumartava_mies_väri4", "1F647-1F3FE-200D-2642-FE0F"},
    {"syvään_kumartava_mies_väri5", "1F647-1F3FF-200D-2642-FE0F"},
    {"mököttävä_nainen", "1F64E-200D-2640-FE0F"},
    {"mököttävä_nainen_väri1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"mököttävä_nainen_väri2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"mököttävä_nainen_väri3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"mököttävä_nainen_väri4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"mököttävä_nainen_väri5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"mököttävä_mies", "1F64E-200D-2642-FE0F"},
    {"mököttävä_mies_väri1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"mököttävä_mies_väri2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"mököttävä_mies_väri3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"mököttävä_mies_väri4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"mököttävä_mies_väri5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"pyöräileva_nainen", "1F6B4-200D-2640-FE0F"},
    {"pyöräileva_nainen_väri1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"pyöräileva_nainen_väri2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"pyöräileva_nainen_väri3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"pyöräileva_nainen_väri4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"pyöräileva_nainen_väri5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"pyöräilevä_mies", "1F6B4-200D-2642-FE0F"},
    {"pyöräilevä_mies_väri1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"pyöräilevä_mies_väri2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"pyöräilevä_mies_väri3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"pyöräilevä_mies_väri4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"pyöräilevä_mies_väri5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"maastopyöräilevä_nainen", "1F6B5-200D-2640-FE0F"},
    {"maastopyöräilevä_nainen_väri1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"maastopyöräilevä_nainen_väri2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"maastopyöräilevä_nainen_väri3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"maastopyöräilevä_nainen_väri4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"maastopyöräilevä_nainen_väri5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"maastopyöräilevä_mies", "1F6B5-200D-2642-FE0F"},
    {"maastopyöräilevä_mies_väri1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"maastopyöräilevä_mies_väri2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"maastopyöräilevä_mies_väri3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"maastopyöräilevä_mies_väri4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"maastopyöräilevä_mies_väri5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"kävelevä_nainen", "1F6B6-200D-2640-FE0F"},
    {"kävelevä_nainen_väri1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"kävelevä_nainen_väri2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"kävelevä_nainen_väri3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"kävelevä_nainen_väri4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"kävelevä_nainen_väri5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"kävelevä_nainen_kohti_oikeaa", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kävelevä_nainen_kohti_oikeaa_väri1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kävelevä_nainen_kohti_oikeaa_väri2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kävelevä_nainen_kohti_oikeaa_väri3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kävelevä_nainen_kohti_oikeaa_väri4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kävelevä_nainen_kohti_oikeaa_väri5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"kävelevä_mies", "1F6B6-200D-2642-FE0F"},
    {"kävelevä_mies_väri1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"kävelevä_mies_väri2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"kävelevä_mies_väri3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"kävelevä_mies_väri4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"kävelevä_mies_väri5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"kävelevä_mies_kohti_oikeaa", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kävelevä_mies_kohti_oikeaa_väri1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kävelevä_mies_kohti_oikeaa_väri2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kävelevä_mies_kohti_oikeaa_väri3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kävelevä_mies_kohti_oikeaa_väri4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kävelevä_mies_kohti_oikeaa_väri5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kasvonsa_kädellä_peittävä_nainen", "1F926-200D-2640-FE0F"},
    {"kasvonsa_kädellä_peittävä_nainen_väri1", "1F926-1F3FB-200D-2640-FE0F"},
    {"kasvonsa_kädellä_peittävä_nainen_väri2", "1F926-1F3FC-200D-2640-FE0F"},
    {"kasvonsa_kädellä_peittävä_nainen_väri3", "1F926-1F3FD-200D-2640-FE0F"},
    {"kasvonsa_kädellä_peittävä_nainen_väri4", "1F926-1F3FE-200D-2640-FE0F"},
    {"kasvonsa_kädellä_peittävä_nainen_väri5", "1F926-1F3FF-200D-2640-FE0F"},
    {"kasvonsa_kädellä_peittävä_mies", "1F926-200D-2642-FE0F"},
    {"kasvonsa_kädellä_peittävä_mies_väri1", "1F926-1F3FB-200D-2642-FE0F"},
    {"kasvonsa_kädellä_peittävä_mies_väri2", "1F926-1F3FC-200D-2642-FE0F"},
    {"kasvonsa_kädellä_peittävä_mies_väri3", "1F926-1F3FD-200D-2642-FE0F"},
    {"kasvonsa_kädellä_peittävä_mies_väri4", "1F926-1F3FE-200D-2642-FE0F"},
    {"kasvonsa_kädellä_peittävä_mies_väri5", "1F926-1F3FF-200D-2642-FE0F"},
    {"olkapäitään_kohauttava_nainen", "1F937-200D-2640-FE0F"},
    {"olkapäitään_kohauttava_nainen_väri1", "1F937-1F3FB-200D-2640-FE0F"},
    {"olkapäitään_kohauttava_nainen_väri2", "1F937-1F3FC-200D-2640-FE0F"},
    {"olkapäitään_kohauttava_nainen_väri3", "1F937-1F3FD-200D-2640-FE0F"},
    {"olkapäitään_kohauttava_nainen_väri4", "1F937-1F3FE-200D-2640-FE0F"},
    {"olkapäitään_kohauttava_nainen_väri5", "1F937-1F3FF-200D-2640-FE0F"},
    {"olkapäitään_kohauttava_mies", "1F937-200D-2642-FE0F"},
    {"olkapäitään_kohauttava_mies_väri1", "1F937-1F3FB-200D-2642-FE0F"},
    {"olkapäitään_kohauttava_mies_väri2", "1F937-1F3FC-200D-2642-FE0F"},
    {"olkapäitään_kohauttava_mies_väri3", "1F937-1F3FD-200D-2642-FE0F"},
    {"olkapäitään_kohauttava_mies_väri4", "1F937-1F3FE-200D-2642-FE0F"},
    {"olkapäitään_kohauttava_mies_väri5", "1F937-1F3FF-200D-2642-FE0F"},
    {"kärrynpyörää_tekevä_nainen", "1F938-200D-2640-FE0F"},
    {"kärrynpyörää_tekevä_nainen_väri1", "1F938-1F3FB-200D-2640-FE0F"},
    {"kärrynpyörää_tekevä_nainen_väri2", "1F938-1F3FC-200D-2640-FE0F"},
    {"kärrynpyörää_tekevä_nainen_väri3", "1F938-1F3FD-200D-2640-FE0F"},
    {"kärrynpyörää_tekevä_nainen_väri4", "1F938-1F3FE-200D-2640-FE0F"},
    {"kärrynpyörää_tekevä_nainen_väri5", "1F938-1F3FF-200D-2640-FE0F"},
    {"kärrynpyörää_tekevä_mies", "1F938-200D-2642-FE0F"},
    {"kärrynpyörää_tekevä_mies_väri1", "1F938-1F3FB-200D-2642-FE0F"},
    {"kärrynpyörää_tekevä_mies_väri2", "1F938-1F3FC-200D-2642-FE0F"},
    {"kärrynpyörää_tekevä_mies_väri3", "1F938-1F3FD-200D-2642-FE0F"},
    {"kärrynpyörää_tekevä_mies_väri4", "1F938-1F3FE-200D-2642-FE0F"},
    {"kärrynpyörää_tekevä_mies_väri5", "1F938-1F3FF-200D-2642-FE0F"},
    {"käsipalloa_pelaava_nainen", "1F93E-200D-2640-FE0F"},
    {"käsipalloa_pelaava_nainen_väri1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"käsipalloa_pelaava_nainen_väri2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"käsipalloa_pelaava_nainen_väri3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"käsipalloa_pelaava_nainen_väri4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"käsipalloa_pelaava_nainen_väri5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"käsipalloa_pelaava_mies", "1F93E-200D-2642-FE0F"},
    {"käsipalloa_pelaava_mies_väri1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"käsipalloa_pelaava_mies_väri2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"käsipalloa_pelaava_mies_väri3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"käsipalloa_pelaava_mies_väri4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"käsipalloa_pelaava_mies_väri5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"naiskiipeilijä", "1F9D7-200D-2640-FE0F"},
    {"naiskiipeilijä_väri1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"naiskiipeilijä_väri2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"naiskiipeilijä_väri3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"naiskiipeilijä_väri4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"naiskiipeilijä_väri5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"mieskiipeilijä", "1F9D7-200D-2642-FE0F"},
    {"mieskiipeilijä_väri1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"mieskiipeilijä_väri2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"mieskiipeilijä_väri3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"mieskiipeilijä_väri4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"mieskiipeilijä_väri5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"palava_sydän", "2764-FE0F-200D-1F525"},
    {"parantuva_sydän", "2764-FE0F-200D-1FA79"},
    {"jääkarhu", "1F43B-200D-2744-FE0F"},
    {"puhekuplasilmä", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"hämmästynyt", "1F62E-200D-1F4A8"},
    {"naama_jolla_spiraalisilmät", "1F635-200D-1F4AB"},
    {"naama_pilvissä", "1F636-200D-1F32B-FE0F"},
    {"ravistaa_päätä_vaakasuunnassa", "1F642-200D-2194-FE0F"},
    {"ravistaa_päätä_pystysuunnassa", "1F642-200D-2195-FE0F"},
};

