// SPDX-FileCopyrightText: None
// SPDX-License-Identifier: LGPL-2.0-or-later
// THIS FILE WAS AUTO-GENERATED
// clang-format off
inline constexpr Emoji emojis_lt[] = {
    {"sauktukas_ir_klaustukas", "2049"},
    {"prekes_zenklas", "2122"},
    {"informacija", "2139"},
    {"rodykle_i_kaire_ir_i_desine", "2194"},
    {"rodykle_i_virsu_ir_i_apacia", "2195"},
    {"rodykle_i_virsu_ir_i_kaire", "2196"},
    {"rodykle_i_virsu_ir_i_desine", "2197"},
    {"rodykle_i_apacia_ir_i_desine", "2198"},
    {"rodykle_i_apacia_ir_i_kaire", "2199"},
    {"klaviatura", "2328"},
    {"saule", "2600"},
    {"debesis", "2601"},
    {"sketis", "2602"},
    {"sniego_senis", "2603"},
    {"kometa", "2604"},
    {"balsavimo_laukelis_su_varnele", "2611"},
    {"sketis_su_lietaus_lasais", "2614"},
    {"karstas_gerimas", "2615"},
    {"trilapis_dobilas", "2618"},
    {"kaukole_ir_sukryziuoti_kaulai", "2620"},
    {"radioaktyvu", "2622"},
    {"biorizika", "2623"},
    {"ortodoksu_kryzius", "2626"},
    {"dharmos_ratas", "2638"},
    {"paniures_veidas", "2639"},
    {"moteriskos_lyties_zenklas", "2640"},
    {"vyriskos_lyties_zenklas", "2642"},
    {"avinas", "2648"},
    {"tauras", "2649"},
    {"saulys", "2650"},
    {"oziaragis", "2651"},
    {"vandenio_zenklas", "2652"},
    {"zuvys", "2653"},
    {"pikai", "2660"},
    {"kryziai", "2663"},
    {"sirdys", "2665"},
    {"bugnai", "2666"},
    {"karstosios_versmes", "2668"},
    {"plaktuvas_ir_kirstuvas", "2692"},
    {"inkaras", "2693"},
    {"sukryziuoti_kardai", "2694"},
    {"medicinos_simbolis", "2695"},
    {"svarstykles_su_lekstemis", "2696"},
    {"distiliavimo_indas", "2697"},
    {"krumpliaratis", "2699"},
    {"zirkles", "2702"},
    {"balta_paryskinta_varnele", "2705"},
    {"lektuvas", "2708"},
    {"vokas", "2709"},
    {"juoda_plunksna", "2712"},
    {"paryskinta_varnele", "2714"},
    {"paryskintas_daugybos_zenklas", "2716"},
    {"dovydo_zvaigzde", "2721"},
    {"kibirksteles", "2728"},
    {"astuoniakampe_zvaigzdute", "2733"},
    {"astuoniakampe_zvaigzde", "2734"},
    {"snaige", "2744"},
    {"kibirkstele", "2747"},
    {"raudonas_klaustukas", "2753"},
    {"baltas_klaustukas", "2754"},
    {"baltas_sauktukas", "2755"},
    {"raudonas_sauktukas", "2757"},
    {"storas_sirdeles_formos_sauktukas", "2763"},
    {"raudona_sirdele", "2764"},
    {"paryskintas_pliusas", "2795"},
    {"paryskintas_minusas", "2796"},
    {"paryskintas_dalybos_zenklas", "2797"},
    {"rodykle_i_desine_uzlenkta_i_virsu", "2934"},
    {"rodykle_i_desine_uzlenkta_i_apacia", "2935"},
    {"vingiuotas_bruksnys", "3030"},
    {"pasveikinima_reiskiantis_hieroglifas", "3297"},
    {"paslapti_reiskiantis_hieroglifas", "3299"},
    {"autoriu_teises", "00A9"},
    {"uzregistruota", "00AE"},
    {"du_sauktukai", "203C"},
    {"rodykle_i_desine_uzlenkta_i_kaire", "21A9"},
    {"rodykle_i_kaire_uzlenkta_i_desine", "21AA"},
    {"laikrodis", "231A"},
    {"smelio_laikrodis", "231B"},
    {"ismetimo_mygtukas", "23CF"},
    {"sukti_pirmyn", "23E9"},
    {"atsukti_atgal", "23EA"},
    {"greitai_i_virsu", "23EB"},
    {"greitai_i_apacia", "23EC"},
    {"kitas_takelis", "23ED"},
    {"paskutinis_takelis", "23EE"},
    {"paleisti_arba_pristabdyti", "23EF"},
    {"zadintuvas", "23F0"},
    {"chronometras", "23F1"},
    {"laikmatis", "23F2"},
    {"smelio_laikrodis_su_krentanciu_smeliu", "23F3"},
    {"pristabdyti", "23F8"},
    {"sustabdymo_mygtukas", "23F9"},
    {"irasymo_mygtukas", "23FA"},
    {"raide_m", "24C2"},
    {"juodas_mazas_kvadratas", "25AA"},
    {"baltas_mazas_kvadratas", "25AB"},
    {"paleisti", "25B6"},
    {"atvirkstine_tvarka", "25C0"},
    {"baltas_vidutinis_kvadratas", "25FB"},
    {"juodas_vidutinis_kvadratas", "25FC"},
    {"baltas_vidutiniskai_mazas_kvadratas", "25FD"},
    {"juodas_vidutiniskai_mazas_kvadratas", "25FE"},
    {"telefonas", "260E"},
    {"delnu_atsukta_aukstyn_pirstu_rodanti_ranka", "261D"},
    {"delnu_atsukta_aukstyn_pirstu_rodanti_ranka_spalva1", "261D-1F3FB"},
    {"delnu_atsukta_aukstyn_pirstu_rodanti_ranka_spalva2", "261D-1F3FC"},
    {"delnu_atsukta_aukstyn_pirstu_rodanti_ranka_spalva3", "261D-1F3FD"},
    {"delnu_atsukta_aukstyn_pirstu_rodanti_ranka_spalva4", "261D-1F3FE"},
    {"delnu_atsukta_aukstyn_pirstu_rodanti_ranka_spalva5", "261D-1F3FF"},
    {"zvaigzde_ir_pusmenulis", "262A"},
    {"taikos_simbolis", "262E"},
    {"in_ir_jang", "262F"},
    {"besisypsantis_veidas", "263A"},
    {"dvyniai", "264A"},
    {"vezys", "264B"},
    {"liutas", "264C"},
    {"mergele", "264D"},
    {"svarstykles", "264E"},
    {"skorpionas", "264F"},
    {"sachmatu_pestininkas", "265F"},
    {"perdirbimo_simbolis", "267B"},
    {"begalybe", "267E"},
    {"neigaliojo_vezimelio_simbolis", "267F"},
    {"atomo_simbolis", "269B"},
    {"lelijos_simbolis", "269C"},
    {"ispejimas", "26A0"},
    {"aukstoji_itampa", "26A1"},
    {"translyciu_simbolis", "26A7"},
    {"baltas_apskritimas", "26AA"},
    {"juodas_apskritimas", "26AB"},
    {"karstas", "26B0"},
    {"laidojimo_urna", "26B1"},
    {"futbolo_kamuolys", "26BD"},
    {"beisbolo_kamuoliukas", "26BE"},
    {"sniego_senis_be_sniego", "26C4"},
    {"saule_uz_debesu", "26C5"},
    {"debesis_su_zaibu_ir_lietumi", "26C8"},
    {"gyvatnesis", "26CE"},
    {"kirstuvas", "26CF"},
    {"salmas_su_baltu_kryziumi", "26D1"},
    {"grandines", "26D3"},
    {"iejimo_nera", "26D4"},
    {"sintoizmo_sventykla", "26E9"},
    {"baznycia", "26EA"},
    {"kalnas", "26F0"},
    {"sketis_ant_zemes", "26F1"},
    {"fontanas", "26F2"},
    {"veliava_skyluteje", "26F3"},
    {"keltas", "26F4"},
    {"burlaivis", "26F5"},
    {"slidininkas", "26F7"},
    {"paciuzos", "26F8"},
    {"zmogus_su_kamuoliu", "26F9"},
    {"zmogus_su_kamuoliu_spalva1", "26F9-1F3FB"},
    {"zmogus_su_kamuoliu_spalva2", "26F9-1F3FC"},
    {"zmogus_su_kamuoliu_spalva3", "26F9-1F3FD"},
    {"zmogus_su_kamuoliu_spalva4", "26F9-1F3FE"},
    {"zmogus_su_kamuoliu_spalva5", "26F9-1F3FF"},
    {"palapine", "26FA"},
    {"degalu_siurblys", "26FD"},
    {"iskeltas_kumstis", "270A"},
    {"iskeltas_kumstis_spalva1", "270A-1F3FB"},
    {"iskeltas_kumstis_spalva2", "270A-1F3FC"},
    {"iskeltas_kumstis_spalva3", "270A-1F3FD"},
    {"iskeltas_kumstis_spalva4", "270A-1F3FE"},
    {"iskeltas_kumstis_spalva5", "270A-1F3FF"},
    {"iskelta_ranka", "270B"},
    {"iskelta_ranka_spalva1", "270B-1F3FB"},
    {"iskelta_ranka_spalva2", "270B-1F3FC"},
    {"iskelta_ranka_spalva3", "270B-1F3FD"},
    {"iskelta_ranka_spalva4", "270B-1F3FE"},
    {"iskelta_ranka_spalva5", "270B-1F3FF"},
    {"pergales_zenkla_rodanti_ranka", "270C"},
    {"pergales_zenkla_rodanti_ranka_spalva1", "270C-1F3FB"},
    {"pergales_zenkla_rodanti_ranka_spalva2", "270C-1F3FC"},
    {"pergales_zenkla_rodanti_ranka_spalva3", "270C-1F3FD"},
    {"pergales_zenkla_rodanti_ranka_spalva4", "270C-1F3FE"},
    {"pergales_zenkla_rodanti_ranka_spalva5", "270C-1F3FF"},
    {"rasanti_ranka", "270D"},
    {"rasanti_ranka_spalva1", "270D-1F3FB"},
    {"rasanti_ranka_spalva2", "270D-1F3FC"},
    {"rasanti_ranka_spalva3", "270D-1F3FD"},
    {"rasanti_ranka_spalva4", "270D-1F3FE"},
    {"rasanti_ranka_spalva5", "270D-1F3FF"},
    {"piestukas", "270F"},
    {"romos_kataliku_kryzius", "271D"},
    {"kryziukas", "274C"},
    {"kryziuko_mygtukas", "274E"},
    {"rodykle_i_desine", "27A1"},
    {"kilpele", "27B0"},
    {"dviguba_kilpele", "27BF"},
    {"rodykle_i_kaire", "2B05"},
    {"rodykle_i_virsu", "2B06"},
    {"rodykle_i_apacia", "2B07"},
    {"juodas_didelis_kvadratas", "2B1B"},
    {"baltas_didelis_kvadratas", "2B1C"},
    {"balta_vidutine_zvaigzde", "2B50"},
    {"ryskus_didelis_ratas", "2B55"},
    {"daliu_apkeitimo_zenklas", "303D"},
    {"madzongo_raudonas_drakonas", "1F004"},
    {"juokdarys", "1F0CF"},
    {"mygtukas_a_raudona", "1F170"},
    {"mygtukas_b_raudona", "1F171"},
    {"simbolis_o_raudona", "1F17E"},
    {"simbolis_p", "1F17F"},
    {"mygtukas_ab_raudona", "1F18E"},
    {"simbolis_cl", "1F191"},
    {"simbolis_cool", "1F192"},
    {"simbolis_free", "1F193"},
    {"simbolis_id", "1F194"},
    {"simbolis_new", "1F195"},
    {"simbolis_ng", "1F196"},
    {"simbolis_ok", "1F197"},
    {"simbolis_sos", "1F198"},
    {"mygtukas_up", "1F199"},
    {"simbolis_vs", "1F19A"},
    {"regioninis_rodiklis_a", "1F1E6"},
    {"regioninis_rodiklis_b", "1F1E7"},
    {"regioninis_rodiklis_c", "1F1E8"},
    {"regioninis_rodiklis_d", "1F1E9"},
    {"regioninis_rodiklis_e", "1F1EA"},
    {"regioninis_rodiklis_f", "1F1EB"},
    {"regioninis_rodiklis_g", "1F1EC"},
    {"regioninis_rodiklis_h", "1F1ED"},
    {"regioninis_rodiklis_i", "1F1EE"},
    {"regioninis_rodiklis_j", "1F1EF"},
    {"regioninis_rodiklis_k", "1F1F0"},
    {"regioninis_rodiklis_l", "1F1F1"},
    {"regioninis_rodiklis_m", "1F1F2"},
    {"regioninis_rodiklis_n", "1F1F3"},
    {"regioninis_rodiklis_o", "1F1F4"},
    {"regioninis_rodiklis_p", "1F1F5"},
    {"regioninis_rodiklis_q", "1F1F6"},
    {"regioninis_rodiklis_r", "1F1F7"},
    {"regioninis_rodiklis_s", "1F1F8"},
    {"regioninis_rodiklis_t", "1F1F9"},
    {"regioninis_rodiklis_u", "1F1FA"},
    {"regioninis_rodiklis_v", "1F1FB"},
    {"regioninis_rodiklis_w", "1F1FC"},
    {"regioninis_rodiklis_x", "1F1FD"},
    {"regioninis_rodiklis_y", "1F1FE"},
    {"regioninis_rodiklis_z", "1F1FF"},
    {"cia_reiskiantis_hieroglifas", "1F201"},
    {"paslaugos_mokesti_reiskiantis_hieroglifas", "1F202"},
    {"nemokamuma_reiskiantis_hieroglifas", "1F21A"},
    {"rezervavima_reiskiantis_hieroglifas", "1F22F"},
    {"draudima_reiskiantis_hieroglifas", "1F232"},
    {"tustuma_reiskiantis_hieroglifas", "1F233"},
    {"bendruma_reiskiantis_hieroglifas", "1F234"},
    {"pilnatve_reiskiantis_hieroglifas", "1F235"},
    {"ne_nemokama_reiskiantis_hieroglifas", "1F236"},
    {"menesio_suma_reiskiantis_hieroglifas", "1F237"},
    {"taikyma_reiskiantis_hieroglifas", "1F238"},
    {"nuolaida_reiskiantis_hieroglifas", "1F239"},
    {"veikima_reiskiantis_hieroglifas", "1F23A"},
    {"susitarima_reiskiantis_hieroglifas", "1F250"},
    {"sutikima_reiskiantis_hieroglifas", "1F251"},
    {"ciklonas", "1F300"},
    {"rukas", "1F301"},
    {"suskleistas_sketis", "1F302"},
    {"zvaigzdeta_naktis", "1F303"},
    {"sauletekis_virs_kalnu", "1F304"},
    {"sauletekis", "1F305"},
    {"sutemos_virs_miesto", "1F306"},
    {"saulelydis", "1F307"},
    {"vaivorykste", "1F308"},
    {"tiltas_nakti", "1F309"},
    {"vandens_banga", "1F30A"},
    {"ugnikalnis", "1F30B"},
    {"pauksciu_takas", "1F30C"},
    {"zemes_rutulys_su_europos_ir_afrikos_zemynais", "1F30D"},
    {"zemes_rutulys_su_siaures_ir_pietu_amerikos_zemynais", "1F30E"},
    {"zemes_rutulys_su_azijos_ir_australijos_zemynais", "1F30F"},
    {"zemes_rutulys_su_dienovidiniais", "1F310"},
    {"jaunas_menulis", "1F311"},
    {"priespilnis_menulis", "1F312"},
    {"pirmas_fazes_ketvirtis", "1F313"},
    {"priespilnis_pabaiga", "1F314"},
    {"pilnatis", "1F315"},
    {"delcia", "1F316"},
    {"paskutinis_fazes_ketvirtis", "1F317"},
    {"delcia_pabaiga", "1F318"},
    {"pusmenulis", "1F319"},
    {"jaunas_menulis_su_veidu", "1F31A"},
    {"priespilnio_menulis_su_veidu", "1F31B"},
    {"delcios_menulis_su_veidu", "1F31C"},
    {"pilnatis_su_veidu", "1F31D"},
    {"saule_su_veidu", "1F31E"},
    {"zerinti_zvaigzde", "1F31F"},
    {"krintanti_zvaigzde", "1F320"},
    {"termometras", "1F321"},
    {"saule_uz_nedidelio_debesies", "1F324"},
    {"saule_uz_didelio_debesies", "1F325"},
    {"saule_uz_debesies_is_kurio_lyja", "1F326"},
    {"debesis_is_kurio_lyja", "1F327"},
    {"debesis_is_kurio_sninga", "1F328"},
    {"debesis_su_zaibu", "1F329"},
    {"tornadas", "1F32A"},
    {"ukana", "1F32B"},
    {"vejo_veidas", "1F32C"},
    {"desrainis", "1F32D"},
    {"meksikietiskas_paplotelis", "1F32E"},
    {"buritas", "1F32F"},
    {"lazdyno_riesutas", "1F330"},
    {"daigas", "1F331"},
    {"amzinai_zaliuojantis_medis", "1F332"},
    {"lapuotis", "1F333"},
    {"palme", "1F334"},
    {"kaktusas", "1F335"},
    {"astrusis_pipiras", "1F336"},
    {"tulpe", "1F337"},
    {"vysnios_ziedas", "1F338"},
    {"roze", "1F339"},
    {"kinroze", "1F33A"},
    {"saulegraza", "1F33B"},
    {"geles_ziedas", "1F33C"},
    {"kukuruzo_burbuole", "1F33D"},
    {"ryziu_pedas", "1F33E"},
    {"zolele", "1F33F"},
    {"keturlapis_dobilas", "1F340"},
    {"klevo_lapas", "1F341"},
    {"nukrites_lapas", "1F342"},
    {"lapas_vejyje", "1F343"},
    {"grybas", "1F344"},
    {"pomidoras", "1F345"},
    {"baklazanas", "1F346"},
    {"vynuoges", "1F347"},
    {"melionas", "1F348"},
    {"arbuzas", "1F349"},
    {"mandarinas", "1F34A"},
    {"citrina", "1F34B"},
    {"bananas", "1F34C"},
    {"ananasas", "1F34D"},
    {"raudonas_obuolys", "1F34E"},
    {"zalias_obuolys", "1F34F"},
    {"kriause", "1F350"},
    {"persikas", "1F351"},
    {"vysnios", "1F352"},
    {"braske", "1F353"},
    {"mesainis", "1F354"},
    {"pica", "1F355"},
    {"mesa_ant_kaulo", "1F356"},
    {"paukscio_slaunele", "1F357"},
    {"ryziu_paplotelis", "1F358"},
    {"ryziu_rutuliukas", "1F359"},
    {"virti_ryziai", "1F35A"},
    {"ryziai_su_kariu", "1F35B"},
    {"makaronu_sriubos_dubenelis", "1F35C"},
    {"spageciai", "1F35D"},
    {"duona", "1F35E"},
    {"gruzdintos_bulvytes", "1F35F"},
    {"kepta_saldzioji_bulve", "1F360"},
    {"dango", "1F361"},
    {"odenas", "1F362"},
    {"susis", "1F363"},
    {"kepta_krevete", "1F364"},
    {"zuvies_pyragelis_su_ispiestu_sukurio_simboliu", "1F365"},
    {"minksti_ledai", "1F366"},
    {"susmulkintas_ledas", "1F367"},
    {"ledai", "1F368"},
    {"spurga", "1F369"},
    {"sausainis", "1F36A"},
    {"sokolado_batonelis", "1F36B"},
    {"saldainis", "1F36C"},
    {"ledinukas_ant_pagaliuko", "1F36D"},
    {"saldus_kremas", "1F36E"},
    {"medaus_puodyne", "1F36F"},
    {"trapus_pyragas", "1F370"},
    {"bento_dezute", "1F371"},
    {"puodas_su_maistu", "1F372"},
    {"valgio_gaminimas", "1F373"},
    {"sakute_ir_peilis", "1F374"},
    {"arbatos_puodelis_be_rankenos", "1F375"},
    {"sake", "1F376"},
    {"vyno_taure", "1F377"},
    {"kokteilio_taure", "1F378"},
    {"tropinis_gerimas", "1F379"},
    {"alaus_bokalas", "1F37A"},
    {"susidauziantys_alaus_bokalai", "1F37B"},
    {"kudikio_buteliukas", "1F37C"},
    {"sakute_ir_peilis_su_lekste", "1F37D"},
    {"butelis_su_issokusiu_kamsciu", "1F37E"},
    {"spragesiai", "1F37F"},
    {"kaspinas", "1F380"},
    {"supakuota_dovana", "1F381"},
    {"gimtadienio_tortas", "1F382"},
    {"zibintas_is_moliugo", "1F383"},
    {"kaledu_eglute", "1F384"},
    {"kaledu_senelis", "1F385"},
    {"kaledu_senelis_spalva1", "1F385-1F3FB"},
    {"kaledu_senelis_spalva2", "1F385-1F3FC"},
    {"kaledu_senelis_spalva3", "1F385-1F3FD"},
    {"kaledu_senelis_spalva4", "1F385-1F3FE"},
    {"kaledu_senelis_spalva5", "1F385-1F3FF"},
    {"fejerverkai", "1F386"},
    {"saltoji_ugnele", "1F387"},
    {"balionas", "1F388"},
    {"konfeti_saudykle", "1F389"},
    {"konfeti_kamuolys", "1F38A"},
    {"tanabata_medis", "1F38B"},
    {"sukryziuotos_veliavos", "1F38C"},
    {"dekoracija_is_pusies", "1F38D"},
    {"japoniskos_leles", "1F38E"},
    {"karpio_vetrunge", "1F38F"},
    {"vejo_varpeliai", "1F390"},
    {"menesienos_ceremonija", "1F391"},
    {"mokykline_kuprine", "1F392"},
    {"mokyklos_baigimo_kepurele", "1F393"},
    {"kariuomenes_medalis", "1F396"},
    {"atminimo_kaspinas", "1F397"},
    {"studijos_mikrofonas", "1F399"},
    {"lygio_slankiklis", "1F39A"},
    {"valdymo_rankenos", "1F39B"},
    {"filmo_kadrai", "1F39E"},
    {"iejimo_bilietai", "1F39F"},
    {"karuseles_arkliukas", "1F3A0"},
    {"apzvalgos_ratas", "1F3A1"},
    {"linksmieji_kalneliai", "1F3A2"},
    {"meskere", "1F3A3"},
    {"mikrofonas", "1F3A4"},
    {"filmavimo_kamera", "1F3A5"},
    {"kinas", "1F3A6"},
    {"ausines", "1F3A7"},
    {"dailininko_palete", "1F3A8"},
    {"cilindras", "1F3A9"},
    {"cirko_palapine", "1F3AA"},
    {"bilietas", "1F3AB"},
    {"pliauske", "1F3AC"},
    {"scenos_menas", "1F3AD"},
    {"vaizdo_zaidimas", "1F3AE"},
    {"smugis_tiesiai_i_taikini", "1F3AF"},
    {"zaidimu_automatas", "1F3B0"},
    {"biliardo_kamuoliukas", "1F3B1"},
    {"kauliukas", "1F3B2"},
    {"boulingas", "1F3B3"},
    {"kortos_su_gelemis", "1F3B4"},
    {"muzikos_nata", "1F3B5"},
    {"muzikos_natos", "1F3B6"},
    {"saksofonas", "1F3B7"},
    {"gitara", "1F3B8"},
    {"muzikos_klaviatura", "1F3B9"},
    {"trimitas", "1F3BA"},
    {"smuikas", "1F3BB"},
    {"partitura", "1F3BC"},
    {"begimo_marskineliai", "1F3BD"},
    {"teniso_kamuoliukas", "1F3BE"},
    {"slides", "1F3BF"},
    {"krepsinio_kamuolys", "1F3C0"},
    {"veliava_langeliais", "1F3C1"},
    {"snieglenciu_sporto_megejas", "1F3C2"},
    {"snieglenciu_sporto_megejas_spalva1", "1F3C2-1F3FB"},
    {"snieglenciu_sporto_megejas_spalva2", "1F3C2-1F3FC"},
    {"snieglenciu_sporto_megejas_spalva3", "1F3C2-1F3FD"},
    {"snieglenciu_sporto_megejas_spalva4", "1F3C2-1F3FE"},
    {"snieglenciu_sporto_megejas_spalva5", "1F3C2-1F3FF"},
    {"begikas", "1F3C3"},
    {"begikas_spalva1", "1F3C3-1F3FB"},
    {"begikas_spalva2", "1F3C3-1F3FC"},
    {"begikas_spalva3", "1F3C3-1F3FD"},
    {"begikas_spalva4", "1F3C3-1F3FE"},
    {"begikas_spalva5", "1F3C3-1F3FF"},
    {"banglentininkas", "1F3C4"},
    {"banglentininkas_spalva1", "1F3C4-1F3FB"},
    {"banglentininkas_spalva2", "1F3C4-1F3FC"},
    {"banglentininkas_spalva3", "1F3C4-1F3FD"},
    {"banglentininkas_spalva4", "1F3C4-1F3FE"},
    {"banglentininkas_spalva5", "1F3C4-1F3FF"},
    {"sporto_medalis", "1F3C5"},
    {"trofejus", "1F3C6"},
    {"arkliu_lenktynes", "1F3C7"},
    {"arkliu_lenktynes_spalva1", "1F3C7-1F3FB"},
    {"arkliu_lenktynes_spalva2", "1F3C7-1F3FC"},
    {"arkliu_lenktynes_spalva3", "1F3C7-1F3FD"},
    {"arkliu_lenktynes_spalva4", "1F3C7-1F3FE"},
    {"arkliu_lenktynes_spalva5", "1F3C7-1F3FF"},
    {"amerikietiskojo_futbolo_kamuolys", "1F3C8"},
    {"regbio_kamuolys", "1F3C9"},
    {"plaukikas", "1F3CA"},
    {"plaukikas_spalva1", "1F3CA-1F3FB"},
    {"plaukikas_spalva2", "1F3CA-1F3FC"},
    {"plaukikas_spalva3", "1F3CA-1F3FD"},
    {"plaukikas_spalva4", "1F3CA-1F3FE"},
    {"plaukikas_spalva5", "1F3CA-1F3FF"},
    {"svoriu_kilnotojas", "1F3CB"},
    {"svoriu_kilnotojas_spalva1", "1F3CB-1F3FB"},
    {"svoriu_kilnotojas_spalva2", "1F3CB-1F3FC"},
    {"svoriu_kilnotojas_spalva3", "1F3CB-1F3FD"},
    {"svoriu_kilnotojas_spalva4", "1F3CB-1F3FE"},
    {"svoriu_kilnotojas_spalva5", "1F3CB-1F3FF"},
    {"golfo_zaidejas", "1F3CC"},
    {"golfo_zaidejas_spalva1", "1F3CC-1F3FB"},
    {"golfo_zaidejas_spalva2", "1F3CC-1F3FC"},
    {"golfo_zaidejas_spalva3", "1F3CC-1F3FD"},
    {"golfo_zaidejas_spalva4", "1F3CC-1F3FE"},
    {"golfo_zaidejas_spalva5", "1F3CC-1F3FF"},
    {"motociklas", "1F3CD"},
    {"lenktyninis_automobilis", "1F3CE"},
    {"kriketas", "1F3CF"},
    {"tinklinio_kamuolys", "1F3D0"},
    {"zoles_riedulys", "1F3D1"},
    {"ledo_ritulio_lazda_ir_ritulys", "1F3D2"},
    {"stalo_tenisas", "1F3D3"},
    {"kalnas_su_snieguota_virsune", "1F3D4"},
    {"stovyklavimas", "1F3D5"},
    {"papludimys_ir_sketis", "1F3D6"},
    {"pastato_konstrukcija", "1F3D7"},
    {"namai", "1F3D8"},
    {"miesto_vaizdas", "1F3D9"},
    {"apleistas_pastatas", "1F3DA"},
    {"klasikinis_pastatas", "1F3DB"},
    {"dykuma", "1F3DC"},
    {"atoki_sala", "1F3DD"},
    {"nacionalinis_parkas", "1F3DE"},
    {"stadionas", "1F3DF"},
    {"namas", "1F3E0"},
    {"namas_su_sodu", "1F3E1"},
    {"biuru_pastatas", "1F3E2"},
    {"japoniskas_pastas", "1F3E3"},
    {"pasto_pastatas", "1F3E4"},
    {"ligonine", "1F3E5"},
    {"bankas", "1F3E6"},
    {"bankomato_zenklas", "1F3E7"},
    {"viesbutis", "1F3E8"},
    {"meiles_viesbutis", "1F3E9"},
    {"visa_para_dirbanti_parduotuve", "1F3EA"},
    {"mokykla", "1F3EB"},
    {"prekybos_centras", "1F3EC"},
    {"gamykla", "1F3ED"},
    {"raudonas_popierinis_zibintas", "1F3EE"},
    {"japoniska_pilis", "1F3EF"},
    {"pilis", "1F3F0"},
    {"balta_veliava", "1F3F3"},
    {"juoda_veliava", "1F3F4"},
    {"rozete", "1F3F5"},
    {"etikete", "1F3F7"},
    {"badmintonas", "1F3F8"},
    {"lankas_ir_strele", "1F3F9"},
    {"senovine_vaza", "1F3FA"},
    {"sviesios_odos", "1F3FB"},
    {"vidutinio_gymio", "1F3FC"},
    {"rusvos_odos", "1F3FD"},
    {"tamsios_odos", "1F3FE"},
    {"itin_tamsios_odos", "1F3FF"},
    {"ziurke", "1F400"},
    {"pele", "1F401"},
    {"jautis", "1F402"},
    {"azijinis_buivolas", "1F403"},
    {"karve", "1F404"},
    {"tigras", "1F405"},
    {"leopardas", "1F406"},
    {"triusis", "1F407"},
    {"kate", "1F408"},
    {"drakonas", "1F409"},
    {"krokodilas", "1F40A"},
    {"banginis", "1F40B"},
    {"sraige", "1F40C"},
    {"gyvate", "1F40D"},
    {"arklys", "1F40E"},
    {"avinas_gyvunas", "1F40F"},
    {"ozys", "1F410"},
    {"avis", "1F411"},
    {"bezdzione", "1F412"},
    {"gaidys", "1F413"},
    {"vista", "1F414"},
    {"suo", "1F415"},
    {"kiaule", "1F416"},
    {"sernas", "1F417"},
    {"dramblys", "1F418"},
    {"astuonkojis", "1F419"},
    {"susisukusi_kriaukle", "1F41A"},
    {"vabalas", "1F41B"},
    {"skruzdele", "1F41C"},
    {"bite", "1F41D"},
    {"boruze", "1F41E"},
    {"zuvis", "1F41F"},
    {"tropine_zuvis", "1F420"},
    {"fugu", "1F421"},
    {"vezlys", "1F422"},
    {"issirites_visciukas", "1F423"},
    {"visciukas", "1F424"},
    {"atsisukes_snapu_visciukas", "1F425"},
    {"paukstis", "1F426"},
    {"pingvinas", "1F427"},
    {"koala", "1F428"},
    {"pudelis", "1F429"},
    {"kupranugaris", "1F42A"},
    {"dvikupris_kupranugaris", "1F42B"},
    {"delfinas", "1F42C"},
    {"peles_snukutis", "1F42D"},
    {"karves_snukis", "1F42E"},
    {"tigro_snukis", "1F42F"},
    {"triusio_snukutis", "1F430"},
    {"kates_snukutis", "1F431"},
    {"drakono_snukis", "1F432"},
    {"purskiantis_banginis", "1F433"},
    {"arklio_snukis", "1F434"},
    {"bezdziones_snukutis", "1F435"},
    {"suns_snukutis", "1F436"},
    {"kiaules_snukis", "1F437"},
    {"varles_snukutis", "1F438"},
    {"ziurkeno_snukutis", "1F439"},
    {"vilko_snukis", "1F43A"},
    {"meskos_snukis", "1F43B"},
    {"pandos_snukis", "1F43C"},
    {"kiaules_nosis", "1F43D"},
    {"pedutes", "1F43E"},
    {"vovere", "1F43F"},
    {"akys", "1F440"},
    {"akis", "1F441"},
    {"ausis", "1F442"},
    {"ausis_spalva1", "1F442-1F3FB"},
    {"ausis_spalva2", "1F442-1F3FC"},
    {"ausis_spalva3", "1F442-1F3FD"},
    {"ausis_spalva4", "1F442-1F3FE"},
    {"ausis_spalva5", "1F442-1F3FF"},
    {"nosis", "1F443"},
    {"nosis_spalva1", "1F443-1F3FB"},
    {"nosis_spalva2", "1F443-1F3FC"},
    {"nosis_spalva3", "1F443-1F3FD"},
    {"nosis_spalva4", "1F443-1F3FE"},
    {"nosis_spalva5", "1F443-1F3FF"},
    {"burna", "1F444"},
    {"liezuvis", "1F445"},
    {"delnu_nusukta_aukstyn_pirstu_rodanti_ranka", "1F446"},
    {"delnu_nusukta_aukstyn_pirstu_rodanti_ranka_spalva1", "1F446-1F3FB"},
    {"delnu_nusukta_aukstyn_pirstu_rodanti_ranka_spalva2", "1F446-1F3FC"},
    {"delnu_nusukta_aukstyn_pirstu_rodanti_ranka_spalva3", "1F446-1F3FD"},
    {"delnu_nusukta_aukstyn_pirstu_rodanti_ranka_spalva4", "1F446-1F3FE"},
    {"delnu_nusukta_aukstyn_pirstu_rodanti_ranka_spalva5", "1F446-1F3FF"},
    {"delnu_nusukta_zemyn_pirstu_rodanti_ranka", "1F447"},
    {"delnu_nusukta_zemyn_pirstu_rodanti_ranka_spalva1", "1F447-1F3FB"},
    {"delnu_nusukta_zemyn_pirstu_rodanti_ranka_spalva2", "1F447-1F3FC"},
    {"delnu_nusukta_zemyn_pirstu_rodanti_ranka_spalva3", "1F447-1F3FD"},
    {"delnu_nusukta_zemyn_pirstu_rodanti_ranka_spalva4", "1F447-1F3FE"},
    {"delnu_nusukta_zemyn_pirstu_rodanti_ranka_spalva5", "1F447-1F3FF"},
    {"i_kaire_pirstu_rodanti_ranka", "1F448"},
    {"i_kaire_pirstu_rodanti_ranka_spalva1", "1F448-1F3FB"},
    {"i_kaire_pirstu_rodanti_ranka_spalva2", "1F448-1F3FC"},
    {"i_kaire_pirstu_rodanti_ranka_spalva3", "1F448-1F3FD"},
    {"i_kaire_pirstu_rodanti_ranka_spalva4", "1F448-1F3FE"},
    {"i_kaire_pirstu_rodanti_ranka_spalva5", "1F448-1F3FF"},
    {"i_desine_pirstu_rodanti_ranka", "1F449"},
    {"i_desine_pirstu_rodanti_ranka_spalva1", "1F449-1F3FB"},
    {"i_desine_pirstu_rodanti_ranka_spalva2", "1F449-1F3FC"},
    {"i_desine_pirstu_rodanti_ranka_spalva3", "1F449-1F3FD"},
    {"i_desine_pirstu_rodanti_ranka_spalva4", "1F449-1F3FE"},
    {"i_desine_pirstu_rodanti_ranka_spalva5", "1F449-1F3FF"},
    {"artejantis_kumstis", "1F44A"},
    {"artejantis_kumstis_spalva1", "1F44A-1F3FB"},
    {"artejantis_kumstis_spalva2", "1F44A-1F3FC"},
    {"artejantis_kumstis_spalva3", "1F44A-1F3FD"},
    {"artejantis_kumstis_spalva4", "1F44A-1F3FE"},
    {"artejantis_kumstis_spalva5", "1F44A-1F3FF"},
    {"mojuojanti_ranka", "1F44B"},
    {"mojuojanti_ranka_spalva1", "1F44B-1F3FB"},
    {"mojuojanti_ranka_spalva2", "1F44B-1F3FC"},
    {"mojuojanti_ranka_spalva3", "1F44B-1F3FD"},
    {"mojuojanti_ranka_spalva4", "1F44B-1F3FE"},
    {"mojuojanti_ranka_spalva5", "1F44B-1F3FF"},
    {"gerai_zenklas_rankomis", "1F44C"},
    {"gerai_zenklas_rankomis_spalva1", "1F44C-1F3FB"},
    {"gerai_zenklas_rankomis_spalva2", "1F44C-1F3FC"},
    {"gerai_zenklas_rankomis_spalva3", "1F44C-1F3FD"},
    {"gerai_zenklas_rankomis_spalva4", "1F44C-1F3FE"},
    {"gerai_zenklas_rankomis_spalva5", "1F44C-1F3FF"},
    {"nykstys_i_virsu", "1F44D"},
    {"nykstys_i_virsu_spalva1", "1F44D-1F3FB"},
    {"nykstys_i_virsu_spalva2", "1F44D-1F3FC"},
    {"nykstys_i_virsu_spalva3", "1F44D-1F3FD"},
    {"nykstys_i_virsu_spalva4", "1F44D-1F3FE"},
    {"nykstys_i_virsu_spalva5", "1F44D-1F3FF"},
    {"nykstys_i_apacia", "1F44E"},
    {"nykstys_i_apacia_spalva1", "1F44E-1F3FB"},
    {"nykstys_i_apacia_spalva2", "1F44E-1F3FC"},
    {"nykstys_i_apacia_spalva3", "1F44E-1F3FD"},
    {"nykstys_i_apacia_spalva4", "1F44E-1F3FE"},
    {"nykstys_i_apacia_spalva5", "1F44E-1F3FF"},
    {"plojancios_rankos", "1F44F"},
    {"plojancios_rankos_spalva1", "1F44F-1F3FB"},
    {"plojancios_rankos_spalva2", "1F44F-1F3FC"},
    {"plojancios_rankos_spalva3", "1F44F-1F3FD"},
    {"plojancios_rankos_spalva4", "1F44F-1F3FE"},
    {"plojancios_rankos_spalva5", "1F44F-1F3FF"},
    {"atviros_rankos", "1F450"},
    {"atviros_rankos_spalva1", "1F450-1F3FB"},
    {"atviros_rankos_spalva2", "1F450-1F3FC"},
    {"atviros_rankos_spalva3", "1F450-1F3FD"},
    {"atviros_rankos_spalva4", "1F450-1F3FE"},
    {"atviros_rankos_spalva5", "1F450-1F3FF"},
    {"karuna", "1F451"},
    {"moteriska_skrybele", "1F452"},
    {"akiniai", "1F453"},
    {"kaklaraistis", "1F454"},
    {"marskineliai", "1F455"},
    {"dzinsai", "1F456"},
    {"suknele", "1F457"},
    {"kimono", "1F458"},
    {"bikinis", "1F459"},
    {"moteriski_drabuziai", "1F45A"},
    {"pinigine", "1F45B"},
    {"rankine", "1F45C"},
    {"maiselis", "1F45D"},
    {"vyriskas_batas", "1F45E"},
    {"begimo_batelis", "1F45F"},
    {"aukstakulnis_batelis", "1F460"},
    {"moteriska_basute", "1F461"},
    {"moteriskas_batas", "1F462"},
    {"pedsakai", "1F463"},
    {"biusto_siluetas", "1F464"},
    {"biustu_siluetai", "1F465"},
    {"berniukas", "1F466"},
    {"berniukas_spalva1", "1F466-1F3FB"},
    {"berniukas_spalva2", "1F466-1F3FC"},
    {"berniukas_spalva3", "1F466-1F3FD"},
    {"berniukas_spalva4", "1F466-1F3FE"},
    {"berniukas_spalva5", "1F466-1F3FF"},
    {"mergaite", "1F467"},
    {"mergaite_spalva1", "1F467-1F3FB"},
    {"mergaite_spalva2", "1F467-1F3FC"},
    {"mergaite_spalva3", "1F467-1F3FD"},
    {"mergaite_spalva4", "1F467-1F3FE"},
    {"mergaite_spalva5", "1F467-1F3FF"},
    {"vyras", "1F468"},
    {"vyras_spalva1", "1F468-1F3FB"},
    {"vyras_spalva2", "1F468-1F3FC"},
    {"vyras_spalva3", "1F468-1F3FD"},
    {"vyras_spalva4", "1F468-1F3FE"},
    {"vyras_spalva5", "1F468-1F3FF"},
    {"moteris", "1F469"},
    {"moteris_spalva1", "1F469-1F3FB"},
    {"moteris_spalva2", "1F469-1F3FC"},
    {"moteris_spalva3", "1F469-1F3FD"},
    {"moteris_spalva4", "1F469-1F3FE"},
    {"moteris_spalva5", "1F469-1F3FF"},
    {"seima", "1F46A"},
    {"vyras_ir_moteris_susikibe_uz_ranku", "1F46B"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva1", "1F46B-1F3FB"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva2", "1F46B-1F3FC"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva3", "1F46B-1F3FD"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva4", "1F46B-1F3FE"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva5", "1F46B-1F3FF"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"vyras_ir_moteris_susikibe_uz_ranku_spalva5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"du_vyrai_susikibe_uz_ranku", "1F46C"},
    {"du_vyrai_susikibe_uz_ranku_spalva1", "1F46C-1F3FB"},
    {"du_vyrai_susikibe_uz_ranku_spalva2", "1F46C-1F3FC"},
    {"du_vyrai_susikibe_uz_ranku_spalva3", "1F46C-1F3FD"},
    {"du_vyrai_susikibe_uz_ranku_spalva4", "1F46C-1F3FE"},
    {"du_vyrai_susikibe_uz_ranku_spalva5", "1F46C-1F3FF"},
    {"du_vyrai_susikibe_uz_ranku_spalva1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"du_vyrai_susikibe_uz_ranku_spalva1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"du_vyrai_susikibe_uz_ranku_spalva1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"du_vyrai_susikibe_uz_ranku_spalva1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"du_vyrai_susikibe_uz_ranku_spalva2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"du_vyrai_susikibe_uz_ranku_spalva2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"du_vyrai_susikibe_uz_ranku_spalva2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"du_vyrai_susikibe_uz_ranku_spalva2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"du_vyrai_susikibe_uz_ranku_spalva3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"du_vyrai_susikibe_uz_ranku_spalva3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"du_vyrai_susikibe_uz_ranku_spalva3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"du_vyrai_susikibe_uz_ranku_spalva3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"du_vyrai_susikibe_uz_ranku_spalva4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"du_vyrai_susikibe_uz_ranku_spalva4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"du_vyrai_susikibe_uz_ranku_spalva4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"du_vyrai_susikibe_uz_ranku_spalva4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"du_vyrai_susikibe_uz_ranku_spalva5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"du_vyrai_susikibe_uz_ranku_spalva5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"du_vyrai_susikibe_uz_ranku_spalva5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"du_vyrai_susikibe_uz_ranku_spalva5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"dvi_moterys_susikibusios_uz_ranku", "1F46D"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva1", "1F46D-1F3FB"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva2", "1F46D-1F3FC"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva3", "1F46D-1F3FD"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva4", "1F46D-1F3FE"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva5", "1F46D-1F3FF"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"dvi_moterys_susikibusios_uz_ranku_spalva5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"policininkas", "1F46E"},
    {"policininkas_spalva1", "1F46E-1F3FB"},
    {"policininkas_spalva2", "1F46E-1F3FC"},
    {"policininkas_spalva3", "1F46E-1F3FD"},
    {"policininkas_spalva4", "1F46E-1F3FE"},
    {"policininkas_spalva5", "1F46E-1F3FF"},
    {"besilinksminancios_moterys", "1F46F"},
    {"besilinksminancios_moterys_spalva1", "1F46F-1F3FB"},
    {"besilinksminancios_moterys_spalva2", "1F46F-1F3FC"},
    {"besilinksminancios_moterys_spalva3", "1F46F-1F3FD"},
    {"besilinksminancios_moterys_spalva4", "1F46F-1F3FE"},
    {"besilinksminancios_moterys_spalva5", "1F46F-1F3FF"},
    {"besilinksminancios_moterys_spalva1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"besilinksminancios_moterys_spalva1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"besilinksminancios_moterys_spalva1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"besilinksminancios_moterys_spalva1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"besilinksminancios_moterys_spalva2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"besilinksminancios_moterys_spalva2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"besilinksminancios_moterys_spalva2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"besilinksminancios_moterys_spalva2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"besilinksminancios_moterys_spalva3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"besilinksminancios_moterys_spalva3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"besilinksminancios_moterys_spalva3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"besilinksminancios_moterys_spalva3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"besilinksminancios_moterys_spalva4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"besilinksminancios_moterys_spalva4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"besilinksminancios_moterys_spalva4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"besilinksminancios_moterys_spalva4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"besilinksminancios_moterys_spalva5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"besilinksminancios_moterys_spalva5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"besilinksminancios_moterys_spalva5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"besilinksminancios_moterys_spalva5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"nuotaka_su_veliumu", "1F470"},
    {"nuotaka_su_veliumu_spalva1", "1F470-1F3FB"},
    {"nuotaka_su_veliumu_spalva2", "1F470-1F3FC"},
    {"nuotaka_su_veliumu_spalva3", "1F470-1F3FD"},
    {"nuotaka_su_veliumu_spalva4", "1F470-1F3FE"},
    {"nuotaka_su_veliumu_spalva5", "1F470-1F3FF"},
    {"sviesiaplaukis_zmogus", "1F471"},
    {"sviesiaplaukis_zmogus_spalva1", "1F471-1F3FB"},
    {"sviesiaplaukis_zmogus_spalva2", "1F471-1F3FC"},
    {"sviesiaplaukis_zmogus_spalva3", "1F471-1F3FD"},
    {"sviesiaplaukis_zmogus_spalva4", "1F471-1F3FE"},
    {"sviesiaplaukis_zmogus_spalva5", "1F471-1F3FF"},
    {"vyras_su_kinietiska_kepurele", "1F472"},
    {"vyras_su_kinietiska_kepurele_spalva1", "1F472-1F3FB"},
    {"vyras_su_kinietiska_kepurele_spalva2", "1F472-1F3FC"},
    {"vyras_su_kinietiska_kepurele_spalva3", "1F472-1F3FD"},
    {"vyras_su_kinietiska_kepurele_spalva4", "1F472-1F3FE"},
    {"vyras_su_kinietiska_kepurele_spalva5", "1F472-1F3FF"},
    {"vyras_su_turbanu", "1F473"},
    {"vyras_su_turbanu_spalva1", "1F473-1F3FB"},
    {"vyras_su_turbanu_spalva2", "1F473-1F3FC"},
    {"vyras_su_turbanu_spalva3", "1F473-1F3FD"},
    {"vyras_su_turbanu_spalva4", "1F473-1F3FE"},
    {"vyras_su_turbanu_spalva5", "1F473-1F3FF"},
    {"senas_vyras", "1F474"},
    {"senas_vyras_spalva1", "1F474-1F3FB"},
    {"senas_vyras_spalva2", "1F474-1F3FC"},
    {"senas_vyras_spalva3", "1F474-1F3FD"},
    {"senas_vyras_spalva4", "1F474-1F3FE"},
    {"senas_vyras_spalva5", "1F474-1F3FF"},
    {"sena_moteris", "1F475"},
    {"sena_moteris_spalva1", "1F475-1F3FB"},
    {"sena_moteris_spalva2", "1F475-1F3FC"},
    {"sena_moteris_spalva3", "1F475-1F3FD"},
    {"sena_moteris_spalva4", "1F475-1F3FE"},
    {"sena_moteris_spalva5", "1F475-1F3FF"},
    {"kudikis", "1F476"},
    {"kudikis_spalva1", "1F476-1F3FB"},
    {"kudikis_spalva2", "1F476-1F3FC"},
    {"kudikis_spalva3", "1F476-1F3FD"},
    {"kudikis_spalva4", "1F476-1F3FE"},
    {"kudikis_spalva5", "1F476-1F3FF"},
    {"statybininkas", "1F477"},
    {"statybininkas_spalva1", "1F477-1F3FB"},
    {"statybininkas_spalva2", "1F477-1F3FC"},
    {"statybininkas_spalva3", "1F477-1F3FD"},
    {"statybininkas_spalva4", "1F477-1F3FE"},
    {"statybininkas_spalva5", "1F477-1F3FF"},
    {"princese", "1F478"},
    {"princese_spalva1", "1F478-1F3FB"},
    {"princese_spalva2", "1F478-1F3FC"},
    {"princese_spalva3", "1F478-1F3FD"},
    {"princese_spalva4", "1F478-1F3FE"},
    {"princese_spalva5", "1F478-1F3FF"},
    {"pabaisa", "1F479"},
    {"demonas", "1F47A"},
    {"vaiduoklis", "1F47B"},
    {"angeliukas", "1F47C"},
    {"angeliukas_spalva1", "1F47C-1F3FB"},
    {"angeliukas_spalva2", "1F47C-1F3FC"},
    {"angeliukas_spalva3", "1F47C-1F3FD"},
    {"angeliukas_spalva4", "1F47C-1F3FE"},
    {"angeliukas_spalva5", "1F47C-1F3FF"},
    {"ateivis", "1F47D"},
    {"monstras_ateivis", "1F47E"},
    {"velniukstis", "1F47F"},
    {"kaukole", "1F480"},
    {"informacijos_skyriaus_darbuotojas", "1F481"},
    {"informacijos_skyriaus_darbuotojas_spalva1", "1F481-1F3FB"},
    {"informacijos_skyriaus_darbuotojas_spalva2", "1F481-1F3FC"},
    {"informacijos_skyriaus_darbuotojas_spalva3", "1F481-1F3FD"},
    {"informacijos_skyriaus_darbuotojas_spalva4", "1F481-1F3FE"},
    {"informacijos_skyriaus_darbuotojas_spalva5", "1F481-1F3FF"},
    {"sargybinis", "1F482"},
    {"sargybinis_spalva1", "1F482-1F3FB"},
    {"sargybinis_spalva2", "1F482-1F3FC"},
    {"sargybinis_spalva3", "1F482-1F3FD"},
    {"sargybinis_spalva4", "1F482-1F3FE"},
    {"sargybinis_spalva5", "1F482-1F3FF"},
    {"sokanti_moteris", "1F483"},
    {"sokanti_moteris_spalva1", "1F483-1F3FB"},
    {"sokanti_moteris_spalva2", "1F483-1F3FC"},
    {"sokanti_moteris_spalva3", "1F483-1F3FD"},
    {"sokanti_moteris_spalva4", "1F483-1F3FE"},
    {"sokanti_moteris_spalva5", "1F483-1F3FF"},
    {"lupu_dazai", "1F484"},
    {"nagu_lakas", "1F485"},
    {"nagu_lakas_spalva1", "1F485-1F3FB"},
    {"nagu_lakas_spalva2", "1F485-1F3FC"},
    {"nagu_lakas_spalva3", "1F485-1F3FD"},
    {"nagu_lakas_spalva4", "1F485-1F3FE"},
    {"nagu_lakas_spalva5", "1F485-1F3FF"},
    {"veido_masazas", "1F486"},
    {"veido_masazas_spalva1", "1F486-1F3FB"},
    {"veido_masazas_spalva2", "1F486-1F3FC"},
    {"veido_masazas_spalva3", "1F486-1F3FD"},
    {"veido_masazas_spalva4", "1F486-1F3FE"},
    {"veido_masazas_spalva5", "1F486-1F3FF"},
    {"apkerpamas_zmogus", "1F487"},
    {"apkerpamas_zmogus_spalva1", "1F487-1F3FB"},
    {"apkerpamas_zmogus_spalva2", "1F487-1F3FC"},
    {"apkerpamas_zmogus_spalva3", "1F487-1F3FD"},
    {"apkerpamas_zmogus_spalva4", "1F487-1F3FE"},
    {"apkerpamas_zmogus_spalva5", "1F487-1F3FF"},
    {"kirpejo_suktukas", "1F488"},
    {"svirkstas", "1F489"},
    {"piliule", "1F48A"},
    {"bucinio_zyme", "1F48B"},
    {"meiles_laiskas", "1F48C"},
    {"ziedas", "1F48D"},
    {"brangakmenis", "1F48E"},
    {"bucinys", "1F48F"},
    {"bucinys_spalva1", "1F48F-1F3FB"},
    {"bucinys_spalva2", "1F48F-1F3FC"},
    {"bucinys_spalva3", "1F48F-1F3FD"},
    {"bucinys_spalva4", "1F48F-1F3FE"},
    {"bucinys_spalva5", "1F48F-1F3FF"},
    {"bucinys_spalva1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bucinys_spalva1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bucinys_spalva1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bucinys_spalva1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bucinys_spalva2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bucinys_spalva2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bucinys_spalva2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bucinys_spalva2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bucinys_spalva3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bucinys_spalva3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bucinys_spalva3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bucinys_spalva3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bucinys_spalva4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bucinys_spalva4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bucinys_spalva4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bucinys_spalva4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bucinys_spalva5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bucinys_spalva5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bucinys_spalva5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bucinys_spalva5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"puokste", "1F490"},
    {"pora_su_sirdele", "1F491"},
    {"pora_su_sirdele_spalva1", "1F491-1F3FB"},
    {"pora_su_sirdele_spalva2", "1F491-1F3FC"},
    {"pora_su_sirdele_spalva3", "1F491-1F3FD"},
    {"pora_su_sirdele_spalva4", "1F491-1F3FE"},
    {"pora_su_sirdele_spalva5", "1F491-1F3FF"},
    {"pora_su_sirdele_spalva1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pora_su_sirdele_spalva1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pora_su_sirdele_spalva1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pora_su_sirdele_spalva1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pora_su_sirdele_spalva2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pora_su_sirdele_spalva2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pora_su_sirdele_spalva2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pora_su_sirdele_spalva2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pora_su_sirdele_spalva3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pora_su_sirdele_spalva3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pora_su_sirdele_spalva3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pora_su_sirdele_spalva3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pora_su_sirdele_spalva4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pora_su_sirdele_spalva4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pora_su_sirdele_spalva4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pora_su_sirdele_spalva4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pora_su_sirdele_spalva5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pora_su_sirdele_spalva5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pora_su_sirdele_spalva5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pora_su_sirdele_spalva5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"vestuves", "1F492"},
    {"plakanti_sirdele", "1F493"},
    {"sudauzyta_sirdele", "1F494"},
    {"dvi_sirdeles", "1F495"},
    {"zerinti_sirdele", "1F496"},
    {"auganti_sirdele", "1F497"},
    {"sirdele_su_strele", "1F498"},
    {"melyna_sirdele", "1F499"},
    {"zalia_sirdele", "1F49A"},
    {"geltona_sirdele", "1F49B"},
    {"purpurine_sirdele", "1F49C"},
    {"sirdele_su_kaspinu", "1F49D"},
    {"besisukancios_sirdeles", "1F49E"},
    {"sirdeles_dekoracija", "1F49F"},
    {"deimantas_su_tasku", "1F4A0"},
    {"lempute", "1F4A1"},
    {"pykcio_simbolis", "1F4A2"},
    {"bomba", "1F4A3"},
    {"miego_simbolis", "1F4A4"},
    {"susidurimas", "1F4A5"},
    {"prakaito_laseliai", "1F4A6"},
    {"lasas", "1F4A7"},
    {"sukeltos_dulkes", "1F4A8"},
    {"ismatu_kruvele", "1F4A9"},
    {"paryskinti_bicepsai", "1F4AA"},
    {"paryskinti_bicepsai_spalva1", "1F4AA-1F3FB"},
    {"paryskinti_bicepsai_spalva2", "1F4AA-1F3FC"},
    {"paryskinti_bicepsai_spalva3", "1F4AA-1F3FD"},
    {"paryskinti_bicepsai_spalva4", "1F4AA-1F3FE"},
    {"paryskinti_bicepsai_spalva5", "1F4AA-1F3FF"},
    {"apsvaiges", "1F4AB"},
    {"kalbos_debeselis", "1F4AC"},
    {"minties_debeselis", "1F4AD"},
    {"balta_gele", "1F4AE"},
    {"simtas_tasku", "1F4AF"},
    {"pinigu_maisas", "1F4B0"},
    {"valiutos_keitimas", "1F4B1"},
    {"paryskintas_dolerio_zenklas", "1F4B2"},
    {"kredito_kortele", "1F4B3"},
    {"jenos_banknotas", "1F4B4"},
    {"dolerio_banknotas", "1F4B5"},
    {"euro_banknotas", "1F4B6"},
    {"svaro_sterlingu_banknotas", "1F4B7"},
    {"pinigai_su_sparnais", "1F4B8"},
    {"didejimo_diagrama_su_jenos_simboliu", "1F4B9"},
    {"vieta", "1F4BA"},
    {"skreitinis_kompiuteris", "1F4BB"},
    {"portfelis", "1F4BC"},
    {"kompiuterio_diskas", "1F4BD"},
    {"diskelis", "1F4BE"},
    {"optinis_diskas", "1F4BF"},
    {"skaitmeninis_vaizdo_diskas", "1F4C0"},
    {"failu_aplankas", "1F4C1"},
    {"atidarytas_failu_aplankas", "1F4C2"},
    {"uzsirietes_puslapis", "1F4C3"},
    {"puslapis_virsutine_puse", "1F4C4"},
    {"kalendorius", "1F4C5"},
    {"nuplesiamas_kalendorius", "1F4C6"},
    {"korteliu_deklas", "1F4C7"},
    {"didejimo_tendencijos_diagrama", "1F4C8"},
    {"mazejimo_tendencijos_diagrama", "1F4C9"},
    {"juostine_diagrama", "1F4CA"},
    {"segtuvas", "1F4CB"},
    {"smeigtukas", "1F4CC"},
    {"apvalus_smeigtukas", "1F4CD"},
    {"savarzele", "1F4CE"},
    {"tiesi_liniuote", "1F4CF"},
    {"trikampe_liniuote", "1F4D0"},
    {"zymiu_skirtukai", "1F4D1"},
    {"saskaitu_knyga", "1F4D2"},
    {"uzrasine", "1F4D3"},
    {"uzrasu_knygele_su_dekoratyviniu_virseliu", "1F4D4"},
    {"uzversta_knyga", "1F4D5"},
    {"atversta_knyga", "1F4D6"},
    {"zalia_knyga", "1F4D7"},
    {"melyna_knyga", "1F4D8"},
    {"oranzine_knyga", "1F4D9"},
    {"knygos", "1F4DA"},
    {"vardo_zenklelis", "1F4DB"},
    {"susuktas_lapas", "1F4DC"},
    {"atmintine", "1F4DD"},
    {"telefono_ragelis", "1F4DE"},
    {"gaviklis", "1F4DF"},
    {"fakso_aparatas", "1F4E0"},
    {"palydovine_antena", "1F4E1"},
    {"garsiakalbio_simbolis", "1F4E2"},
    {"garsintuvas", "1F4E3"},
    {"siunciamu_laisku_deklas", "1F4E4"},
    {"gaunamu_laisku_deklas", "1F4E5"},
    {"paketas", "1F4E6"},
    {"el_laiskas", "1F4E7"},
    {"gaunamas_vokas", "1F4E8"},
    {"vokas_su_rodykle", "1F4E9"},
    {"uzdaryta_pasto_dezute_su_nuleista_veliava", "1F4EA"},
    {"uzdaryta_pasto_dezute_su_pakelta_veliava", "1F4EB"},
    {"atidaryta_pasto_dezute_su_pakelta_veliava", "1F4EC"},
    {"atidaryta_pasto_dezute_su_nuleista_veliava", "1F4ED"},
    {"pasto_dezute", "1F4EE"},
    {"pasto_ragas", "1F4EF"},
    {"laikrastis", "1F4F0"},
    {"mobilusis_telefonas", "1F4F1"},
    {"mobilusis_telefonas_su_rodykle", "1F4F2"},
    {"vibravimo_rezimas", "1F4F3"},
    {"isjungti_mobiluji_telefona", "1F4F4"},
    {"jokiu_mobiliuju_telefonu", "1F4F5"},
    {"antenos_juostos", "1F4F6"},
    {"fotoaparatas", "1F4F7"},
    {"fotoaparatas_su_blykste", "1F4F8"},
    {"vaizdo_kamera", "1F4F9"},
    {"televizorius", "1F4FA"},
    {"radijas", "1F4FB"},
    {"vaizdo_irasu_kasete", "1F4FC"},
    {"filmu_projektorius", "1F4FD"},
    {"rozinis", "1F4FF"},
    {"maisyti_takelius", "1F500"},
    {"kartoti", "1F501"},
    {"kartoti_viena", "1F502"},
    {"vertikalios_laikrodzio_rodykles_kryptimis_nukreiptos_rodykles", "1F503"},
    {"pries_laikrodzio_rodykle_nukreiptu_rodykliu_mygtukas", "1F504"},
    {"uztemdymo_mygtukas", "1F505"},
    {"ryskumo_mygtukas", "1F506"},
    {"garsiakalbis_isjungtas", "1F507"},
    {"tylus_garsiakalbis", "1F508"},
    {"vidutinio_garso_garsiakalbis", "1F509"},
    {"didelis_garsiakalbio_garsumas", "1F50A"},
    {"baterija", "1F50B"},
    {"elektros_kistukas", "1F50C"},
    {"didinimo_stiklas_i_kaire", "1F50D"},
    {"didinimo_stiklas_i_desine", "1F50E"},
    {"spyna_ir_rasiklis", "1F50F"},
    {"uzrakinta_spyna_su_raktu", "1F510"},
    {"raktas", "1F511"},
    {"spyna", "1F512"},
    {"atrakinta_spyna", "1F513"},
    {"varpelis", "1F514"},
    {"perbrauktas_varpelis", "1F515"},
    {"zyme", "1F516"},
    {"jungtis", "1F517"},
    {"akute", "1F518"},
    {"rodykle_su_uzrasu_back", "1F519"},
    {"rodykle_su_uzrasu_end", "1F51A"},
    {"rodykle_su_uzrasu_on", "1F51B"},
    {"rodykle_su_uzrasu_soon", "1F51C"},
    {"rodykle_su_uzrasu_top", "1F51D"},
    {"tik_pilnameciams", "1F51E"},
    {"mygtukas_10", "1F51F"},
    {"ivestis_didziosios_lotyniskos_raides", "1F520"},
    {"ivestis_mazosios_lotyniskos_raides", "1F521"},
    {"ivesties_skaiciai", "1F522"},
    {"ivesties_simboliai", "1F523"},
    {"ivestis_lotyniskos_raides", "1F524"},
    {"ugnis", "1F525"},
    {"zibintuvelis", "1F526"},
    {"verzliaraktis", "1F527"},
    {"plaktukas", "1F528"},
    {"verzle_ir_varztas", "1F529"},
    {"virtuvinis_peilis", "1F52A"},
    {"vandens_sautuvas", "1F52B"},
    {"mikroskopas", "1F52C"},
    {"teleskopas", "1F52D"},
    {"kristolinis_rutulys", "1F52E"},
    {"sesiakampe_zvaigzde_su_tasku", "1F52F"},
    {"japoniskas_pradedanciojo_simbolis", "1F530"},
    {"trisake_emblema", "1F531"},
    {"juodas_kvadratinis_mygtukas", "1F532"},
    {"baltas_kvadratinis_mygtukas", "1F533"},
    {"raudonas_apskritimas", "1F534"},
    {"melynas_apskritimas", "1F535"},
    {"didelis_oranzinis_deimantas", "1F536"},
    {"didelis_melynas_deimantas", "1F537"},
    {"mazas_oranzinis_deimantas", "1F538"},
    {"mazas_melynas_deimantas", "1F539"},
    {"raudonas_i_virsu_nukreiptas_trikampis", "1F53A"},
    {"raudonas_i_apacia_nukreiptas_trikampis", "1F53B"},
    {"i_virsu", "1F53C"},
    {"i_apacia", "1F53D"},
    {"om", "1F549"},
    {"balandis", "1F54A"},
    {"kaaba", "1F54B"},
    {"mecete", "1F54C"},
    {"sinagoga", "1F54D"},
    {"menora", "1F54E"},
    {"pirma_valanda", "1F550"},
    {"dvi_valandos", "1F551"},
    {"trys_valandos", "1F552"},
    {"keturios_valandos", "1F553"},
    {"penkios_valandos", "1F554"},
    {"sesios_valandos", "1F555"},
    {"septynios_valandos", "1F556"},
    {"astuonios_valandos", "1F557"},
    {"devinta_valanda", "1F558"},
    {"desimt_valandu", "1F559"},
    {"vienuolika_valandu", "1F55A"},
    {"dvylika_valandu", "1F55B"},
    {"puse_dvieju", "1F55C"},
    {"puse_triju", "1F55D"},
    {"puse_keturiu", "1F55E"},
    {"puse_penkiu", "1F55F"},
    {"puse_sesiu", "1F560"},
    {"puse_septyniu", "1F561"},
    {"puse_astuoniu", "1F562"},
    {"puse_devyniu", "1F563"},
    {"puse_desimt", "1F564"},
    {"puse_vienuolikos", "1F565"},
    {"puse_dvylikos", "1F566"},
    {"dvylika_trisdesimt", "1F567"},
    {"zvake", "1F56F"},
    {"virs_zidinio_laikomas_laikrodis", "1F570"},
    {"duobe", "1F573"},
    {"i_ora_kylantis_vyras_su_kostiumu", "1F574"},
    {"i_ora_kylantis_vyras_su_kostiumu_spalva1", "1F574-1F3FB"},
    {"i_ora_kylantis_vyras_su_kostiumu_spalva2", "1F574-1F3FC"},
    {"i_ora_kylantis_vyras_su_kostiumu_spalva3", "1F574-1F3FD"},
    {"i_ora_kylantis_vyras_su_kostiumu_spalva4", "1F574-1F3FE"},
    {"i_ora_kylantis_vyras_su_kostiumu_spalva5", "1F574-1F3FF"},
    {"detektyvas", "1F575"},
    {"detektyvas_spalva1", "1F575-1F3FB"},
    {"detektyvas_spalva2", "1F575-1F3FC"},
    {"detektyvas_spalva3", "1F575-1F3FD"},
    {"detektyvas_spalva4", "1F575-1F3FE"},
    {"detektyvas_spalva5", "1F575-1F3FF"},
    {"akiniai_nuo_saules", "1F576"},
    {"voras", "1F577"},
    {"voratinklis", "1F578"},
    {"valdymo_svirtis", "1F579"},
    {"sokantis_vyras", "1F57A"},
    {"sokantis_vyras_spalva1", "1F57A-1F3FB"},
    {"sokantis_vyras_spalva2", "1F57A-1F3FC"},
    {"sokantis_vyras_spalva3", "1F57A-1F3FD"},
    {"sokantis_vyras_spalva4", "1F57A-1F3FE"},
    {"sokantis_vyras_spalva5", "1F57A-1F3FF"},
    {"sukabintos_savarzeles", "1F587"},
    {"rasiklis", "1F58A"},
    {"automatinis_plunksnakotis", "1F58B"},
    {"teptukas", "1F58C"},
    {"kreidele", "1F58D"},
    {"iskelta_ranka_su_isskestais_pirstais", "1F590"},
    {"iskelta_ranka_su_isskestais_pirstais_spalva1", "1F590-1F3FB"},
    {"iskelta_ranka_su_isskestais_pirstais_spalva2", "1F590-1F3FC"},
    {"iskelta_ranka_su_isskestais_pirstais_spalva3", "1F590-1F3FD"},
    {"iskelta_ranka_su_isskestais_pirstais_spalva4", "1F590-1F3FE"},
    {"iskelta_ranka_su_isskestais_pirstais_spalva5", "1F590-1F3FF"},
    {"vidurinis_pirstas", "1F595"},
    {"vidurinis_pirstas_spalva1", "1F595-1F3FB"},
    {"vidurinis_pirstas_spalva2", "1F595-1F3FC"},
    {"vidurinis_pirstas_spalva3", "1F595-1F3FD"},
    {"vidurinis_pirstas_spalva4", "1F595-1F3FE"},
    {"vidurinis_pirstas_spalva5", "1F595-1F3FF"},
    {"vulkanu_rases_pasisveikinimo_zenklas", "1F596"},
    {"vulkanu_rases_pasisveikinimo_zenklas_spalva1", "1F596-1F3FB"},
    {"vulkanu_rases_pasisveikinimo_zenklas_spalva2", "1F596-1F3FC"},
    {"vulkanu_rases_pasisveikinimo_zenklas_spalva3", "1F596-1F3FD"},
    {"vulkanu_rases_pasisveikinimo_zenklas_spalva4", "1F596-1F3FE"},
    {"vulkanu_rases_pasisveikinimo_zenklas_spalva5", "1F596-1F3FF"},
    {"juoda_sirdis", "1F5A4"},
    {"stalinis_kompiuteris", "1F5A5"},
    {"spausdintuvas", "1F5A8"},
    {"kompiuterio_pele", "1F5B1"},
    {"rutulinis_manipuliatorius", "1F5B2"},
    {"remas_su_paveikslu", "1F5BC"},
    {"korteliu_laikiklio_skirtukai", "1F5C2"},
    {"korteliu_saugojimo_deze", "1F5C3"},
    {"bylu_spintele_su_stalciais", "1F5C4"},
    {"siuksliadeze", "1F5D1"},
    {"uzrasine_su_spirale", "1F5D2"},
    {"kalendorius_su_spirale", "1F5D3"},
    {"suspaudimas", "1F5DC"},
    {"senas_raktas", "1F5DD"},
    {"susuktas_laikrastis", "1F5DE"},
    {"durklas", "1F5E1"},
    {"kalbanti_galva", "1F5E3"},
    {"kairysis_kalbos_debeselis", "1F5E8"},
    {"desinysis_pykcio_debeselis", "1F5EF"},
    {"balsavimo_dezute_su_balsavimo_biuleteniu", "1F5F3"},
    {"pasaulio_zemelapis", "1F5FA"},
    {"fudzio_kalnas", "1F5FB"},
    {"tokijo_bokstas", "1F5FC"},
    {"laisves_statula", "1F5FD"},
    {"japonijos_zemelapis", "1F5FE"},
    {"moai", "1F5FF"},
    {"issisiepes_veidas", "1F600"},
    {"issisiepes_veidas_su_besisypsanciomis_akimis", "1F601"},
    {"veidas_su_dziaugsmo_asaromis", "1F602"},
    {"besisypsantis_issiziojes_veidas", "1F603"},
    {"besisypsantis_issiziojes_veidas_besisypsanciomis_akimis", "1F604"},
    {"besisypsantis_issiziojes_veidas_ispiltas_salto_prakaito", "1F605"},
    {"besisypsantis_issiziojes_veidas_su_smarkiai_uzmerktomis_akimis", "1F606"},
    {"besisypsantis_veidas_su_aureole", "1F607"},
    {"besisypsantis_veidas_su_ragais", "1F608"},
    {"mirksintis_veidas", "1F609"},
    {"besisypsantis_veidas_su_besisypsanciomis_akimis", "1F60A"},
    {"skaniu_maistu_besimegaujantis_veidas", "1F60B"},
    {"veidas_su_palengvejimu", "1F60C"},
    {"besisypsantis_veidas_su_sirdeles_formos_akimis", "1F60D"},
    {"besisypsantis_veidas_su_akiniais_nuo_saules", "1F60E"},
    {"is_pasitenkinimo_besisypsantis_veidas", "1F60F"},
    {"neutralus_veidas", "1F610"},
    {"veidas_be_jokios_israiskos", "1F611"},
    {"nelinksmas_veidas", "1F612"},
    {"veidas_ispiltas_salto_prakaito", "1F613"},
    {"susimastes_veidas", "1F614"},
    {"sutrikes_veidas", "1F615"},
    {"suglumintas_veidas", "1F616"},
    {"buciuojantis_veidas", "1F617"},
    {"bucini_siunciantis_veidas", "1F618"},
    {"buciuojantis_su_besisypsanciomis_akimis", "1F619"},
    {"buciuojantis_su_uzmerktomis_akimis", "1F61A"},
    {"veidas_su_iskistu_liezuviu", "1F61B"},
    {"veidas_su_iskistu_liezuviu_ir_viena_primerkta_akimi", "1F61C"},
    {"veidas_su_iskistu_liezuviu_ir_stipriai_uzmerktomis_akimis", "1F61D"},
    {"nusivyles_veidas", "1F61E"},
    {"susirupines_veidas", "1F61F"},
    {"piktas_veidas", "1F620"},
    {"susiraukes_veidas", "1F621"},
    {"verkiantis_veidas", "1F622"},
    {"atkaklus_veidas", "1F623"},
    {"veidas_su_is_nosies_einanciais_garais", "1F624"},
    {"nusivyles_veidas_su_palengvejimu", "1F625"},
    {"paniures_issiziojes_veidas", "1F626"},
    {"kenciantis_veidas", "1F627"},
    {"baimes_pilnas_veidas", "1F628"},
    {"nuvarges_veidas", "1F629"},
    {"mieguistas_veidas", "1F62A"},
    {"pavarges_veidas", "1F62B"},
    {"besivaipantis_veidas", "1F62C"},
    {"garsiai_raudantis_veidas", "1F62D"},
    {"issiziojes_veidas", "1F62E"},
    {"nusciuves_veidas", "1F62F"},
    {"issiziojes_veidas_ispiltas_salto_prakaito", "1F630"},
    {"is_baimes_saukiantis_veidas", "1F631"},
    {"apstulbes_veidas", "1F632"},
    {"nuraudes_veidas", "1F633"},
    {"miegantis_veidas", "1F634"},
    {"apsvaiges_veidas", "1F635"},
    {"veidas_be_burnos", "1F636"},
    {"veidas_su_medicinine_kauke", "1F637"},
    {"issisiepusios_kates_snukutis_su_besisypsanciomis_akimis", "1F638"},
    {"kates_snukutis_su_dziaugsmo_asaromis", "1F639"},
    {"besisypsancios_kates_snukutis", "1F63A"},
    {"besisypsancios_kates_snukutis_su_sirdeles_formos_akimis", "1F63B"},
    {"nuvargusios_besisypsancios_kates_snukutis", "1F63C"},
    {"buciuojancios_kates_snukutis", "1F63D"},
    {"susiraukusios_kates_snukutis", "1F63E"},
    {"verkiancios_kates_snukutis", "1F63F"},
    {"nustebusios_kates_snukutis", "1F640"},
    {"siek_tiek_paniures_veidas", "1F641"},
    {"siek_tiek_besisypsantis_veidas", "1F642"},
    {"apverstas_veidas", "1F643"},
    {"veidas_vartantis_akis", "1F644"},
    {"neigiamas_gestas", "1F645"},
    {"neigiamas_gestas_spalva1", "1F645-1F3FB"},
    {"neigiamas_gestas_spalva2", "1F645-1F3FC"},
    {"neigiamas_gestas_spalva3", "1F645-1F3FD"},
    {"neigiamas_gestas_spalva4", "1F645-1F3FE"},
    {"neigiamas_gestas_spalva5", "1F645-1F3FF"},
    {"teigiamas_gestas", "1F646"},
    {"teigiamas_gestas_spalva1", "1F646-1F3FB"},
    {"teigiamas_gestas_spalva2", "1F646-1F3FC"},
    {"teigiamas_gestas_spalva3", "1F646-1F3FD"},
    {"teigiamas_gestas_spalva4", "1F646-1F3FE"},
    {"teigiamas_gestas_spalva5", "1F646-1F3FF"},
    {"nusilenkes_zmogus", "1F647"},
    {"nusilenkes_zmogus_spalva1", "1F647-1F3FB"},
    {"nusilenkes_zmogus_spalva2", "1F647-1F3FC"},
    {"nusilenkes_zmogus_spalva3", "1F647-1F3FD"},
    {"nusilenkes_zmogus_spalva4", "1F647-1F3FE"},
    {"nusilenkes_zmogus_spalva5", "1F647-1F3FF"},
    {"uzsidengusi_akis_bezdzione", "1F648"},
    {"uzsidengusi_ausis_bezdzione", "1F649"},
    {"uzsidengusi_burna_bezdzione", "1F64A"},
    {"zmogus_iskeles_ranka", "1F64B"},
    {"zmogus_iskeles_ranka_spalva1", "1F64B-1F3FB"},
    {"zmogus_iskeles_ranka_spalva2", "1F64B-1F3FC"},
    {"zmogus_iskeles_ranka_spalva3", "1F64B-1F3FD"},
    {"zmogus_iskeles_ranka_spalva4", "1F64B-1F3FE"},
    {"zmogus_iskeles_ranka_spalva5", "1F64B-1F3FF"},
    {"zmogus_iskeles_rankas", "1F64C"},
    {"zmogus_iskeles_rankas_spalva1", "1F64C-1F3FB"},
    {"zmogus_iskeles_rankas_spalva2", "1F64C-1F3FC"},
    {"zmogus_iskeles_rankas_spalva3", "1F64C-1F3FD"},
    {"zmogus_iskeles_rankas_spalva4", "1F64C-1F3FE"},
    {"zmogus_iskeles_rankas_spalva5", "1F64C-1F3FF"},
    {"paniures_zmogus", "1F64D"},
    {"paniures_zmogus_spalva1", "1F64D-1F3FB"},
    {"paniures_zmogus_spalva2", "1F64D-1F3FC"},
    {"paniures_zmogus_spalva3", "1F64D-1F3FD"},
    {"paniures_zmogus_spalva4", "1F64D-1F3FE"},
    {"paniures_zmogus_spalva5", "1F64D-1F3FF"},
    {"susiraukes_zmogus", "1F64E"},
    {"susiraukes_zmogus_spalva1", "1F64E-1F3FB"},
    {"susiraukes_zmogus_spalva2", "1F64E-1F3FC"},
    {"susiraukes_zmogus_spalva3", "1F64E-1F3FD"},
    {"susiraukes_zmogus_spalva4", "1F64E-1F3FE"},
    {"susiraukes_zmogus_spalva5", "1F64E-1F3FF"},
    {"suglausti_delnai", "1F64F"},
    {"suglausti_delnai_spalva1", "1F64F-1F3FB"},
    {"suglausti_delnai_spalva2", "1F64F-1F3FC"},
    {"suglausti_delnai_spalva3", "1F64F-1F3FD"},
    {"suglausti_delnai_spalva4", "1F64F-1F3FE"},
    {"suglausti_delnai_spalva5", "1F64F-1F3FF"},
    {"raketa", "1F680"},
    {"sraigstasparnis", "1F681"},
    {"lokomotyvas", "1F682"},
    {"traukinio_vagonas", "1F683"},
    {"greitasis_traukinys", "1F684"},
    {"greitasis_traukinys_su_smailu_priekiu", "1F685"},
    {"traukinys", "1F686"},
    {"metro", "1F687"},
    {"miesto_tramvajus", "1F688"},
    {"stotis", "1F689"},
    {"tramvajus", "1F68A"},
    {"tramvajaus_vagonas", "1F68B"},
    {"autobusas", "1F68C"},
    {"artejantis_autobusas", "1F68D"},
    {"troleibusas", "1F68E"},
    {"autobusu_stotele", "1F68F"},
    {"mikroautobusas", "1F690"},
    {"greitosios_pagalbos_automobilis", "1F691"},
    {"ugniagesiu_automobilis", "1F692"},
    {"policijos_automobilis", "1F693"},
    {"artejantis_policijos_automobilis", "1F694"},
    {"taksi", "1F695"},
    {"artejantis_taksi", "1F696"},
    {"automobilis", "1F697"},
    {"artejantis_automobilis", "1F698"},
    {"rekreacijos_automobilis", "1F699"},
    {"krovininis_sunkvezimis", "1F69A"},
    {"sunkvezimis_su_priekaba", "1F69B"},
    {"traktorius", "1F69C"},
    {"vienbegis_gelezinkelis", "1F69D"},
    {"kalnu_traukinys", "1F69E"},
    {"kabanti_gelezinkelio_linija", "1F69F"},
    {"kalnu_keltuvas", "1F6A0"},
    {"lynu_keltuvas", "1F6A1"},
    {"laivas", "1F6A2"},
    {"irkluojantis_zmogus", "1F6A3"},
    {"irkluojantis_zmogus_spalva1", "1F6A3-1F3FB"},
    {"irkluojantis_zmogus_spalva2", "1F6A3-1F3FC"},
    {"irkluojantis_zmogus_spalva3", "1F6A3-1F3FD"},
    {"irkluojantis_zmogus_spalva4", "1F6A3-1F3FE"},
    {"irkluojantis_zmogus_spalva5", "1F6A3-1F3FF"},
    {"motorinis_kateris", "1F6A4"},
    {"horizontalus_sviesoforas", "1F6A5"},
    {"vertikalus_sviesoforas", "1F6A6"},
    {"statybos", "1F6A7"},
    {"policijos_automobilio_lempa", "1F6A8"},
    {"trikampe_veliava", "1F6A9"},
    {"durys", "1F6AA"},
    {"draudziama", "1F6AB"},
    {"rukymas", "1F6AC"},
    {"rukyti_draudziama", "1F6AD"},
    {"siuksliu_metimo_i_siuksliadezes_zenklas", "1F6AE"},
    {"siukslinti_draudziama", "1F6AF"},
    {"geriamasis_vanduo", "1F6B0"},
    {"draudziama_gerti_vandeni", "1F6B1"},
    {"dviratis", "1F6B2"},
    {"dviraciai_draudziami", "1F6B3"},
    {"dviratininkas", "1F6B4"},
    {"dviratininkas_spalva1", "1F6B4-1F3FB"},
    {"dviratininkas_spalva2", "1F6B4-1F3FC"},
    {"dviratininkas_spalva3", "1F6B4-1F3FD"},
    {"dviratininkas_spalva4", "1F6B4-1F3FE"},
    {"dviratininkas_spalva5", "1F6B4-1F3FF"},
    {"kalnu_dviratininkas", "1F6B5"},
    {"kalnu_dviratininkas_spalva1", "1F6B5-1F3FB"},
    {"kalnu_dviratininkas_spalva2", "1F6B5-1F3FC"},
    {"kalnu_dviratininkas_spalva3", "1F6B5-1F3FD"},
    {"kalnu_dviratininkas_spalva4", "1F6B5-1F3FE"},
    {"kalnu_dviratininkas_spalva5", "1F6B5-1F3FF"},
    {"pestysis", "1F6B6"},
    {"pestysis_spalva1", "1F6B6-1F3FB"},
    {"pestysis_spalva2", "1F6B6-1F3FC"},
    {"pestysis_spalva3", "1F6B6-1F3FD"},
    {"pestysis_spalva4", "1F6B6-1F3FE"},
    {"pestysis_spalva5", "1F6B6-1F3FF"},
    {"pestiesiems_eiti_draudziama", "1F6B7"},
    {"vaiku_pereja", "1F6B8"},
    {"vyru_tualetas", "1F6B9"},
    {"moteru_tualetas", "1F6BA"},
    {"tualeto_kambarys", "1F6BB"},
    {"kudikio_simbolis", "1F6BC"},
    {"tualetas", "1F6BD"},
    {"tualeto_patalpa", "1F6BE"},
    {"dusas", "1F6BF"},
    {"zmogus_prausiasi_vonioje", "1F6C0"},
    {"zmogus_prausiasi_vonioje_spalva1", "1F6C0-1F3FB"},
    {"zmogus_prausiasi_vonioje_spalva2", "1F6C0-1F3FC"},
    {"zmogus_prausiasi_vonioje_spalva3", "1F6C0-1F3FD"},
    {"zmogus_prausiasi_vonioje_spalva4", "1F6C0-1F3FE"},
    {"zmogus_prausiasi_vonioje_spalva5", "1F6C0-1F3FF"},
    {"vonia", "1F6C1"},
    {"pasu_kontrole", "1F6C2"},
    {"muitine", "1F6C3"},
    {"bagazo_skyrius", "1F6C4"},
    {"paliktas_bagazas", "1F6C5"},
    {"sofa_ir_lempa", "1F6CB"},
    {"zmogus_lovoje", "1F6CC"},
    {"zmogus_lovoje_spalva1", "1F6CC-1F3FB"},
    {"zmogus_lovoje_spalva2", "1F6CC-1F3FC"},
    {"zmogus_lovoje_spalva3", "1F6CC-1F3FD"},
    {"zmogus_lovoje_spalva4", "1F6CC-1F3FE"},
    {"zmogus_lovoje_spalva5", "1F6CC-1F3FF"},
    {"pirkiniu_krepseliai", "1F6CD"},
    {"skambutis", "1F6CE"},
    {"lova", "1F6CF"},
    {"garbinimo_vieta", "1F6D0"},
    {"sustojimo_zenklas", "1F6D1"},
    {"pirkiniu_vezimelis", "1F6D2"},
    {"induistu_sventykla", "1F6D5"},
    {"namelis", "1F6D6"},
    {"elevatorius", "1F6D7"},
    {"nuosliauza", "1F6D8"},
    {"belaidis_rysys", "1F6DC"},
    {"ciuozykla", "1F6DD"},
    {"ratas", "1F6DE"},
    {"gelbejimo_ratas", "1F6DF"},
    {"plaktukas_ir_verzliaraktis", "1F6E0"},
    {"skydas", "1F6E1"},
    {"degalu_statine", "1F6E2"},
    {"greitkelis", "1F6E3"},
    {"gelezinkelio_begiai", "1F6E4"},
    {"motorine_valtis", "1F6E5"},
    {"mazas_lektuvas", "1F6E9"},
    {"lektuvo_pakilimas", "1F6EB"},
    {"lektuvo_nusileidimas", "1F6EC"},
    {"palydovas", "1F6F0"},
    {"keleivinis_laivas", "1F6F3"},
    {"paspirtukas", "1F6F4"},
    {"motoroleris", "1F6F5"},
    {"kanoja", "1F6F6"},
    {"roges", "1F6F7"},
    {"skraidanti_lekste", "1F6F8"},
    {"riedlente", "1F6F9"},
    {"autoriksa", "1F6FA"},
    {"pikapas", "1F6FB"},
    {"rieduciai", "1F6FC"},
    {"oranzinis_apskritimas", "1F7E0"},
    {"geltonas_apskritimas", "1F7E1"},
    {"zalias_apskritimas", "1F7E2"},
    {"violetinis_apskritimas", "1F7E3"},
    {"rudas_apskritimas", "1F7E4"},
    {"raudonas_kvadratas", "1F7E5"},
    {"melynas_kvadratas", "1F7E6"},
    {"oranzinis_kvadratas", "1F7E7"},
    {"geltonas_kvadratas", "1F7E8"},
    {"zalias_kvadratas", "1F7E9"},
    {"violetinis_kvadratas", "1F7EA"},
    {"rudas_kvadratas", "1F7EB"},
    {"paryskintas_lygybes_zenklas", "1F7F0"},
    {"suspausti_pirstai", "1F90C"},
    {"suspausti_pirstai_spalva1", "1F90C-1F3FB"},
    {"suspausti_pirstai_spalva2", "1F90C-1F3FC"},
    {"suspausti_pirstai_spalva3", "1F90C-1F3FD"},
    {"suspausti_pirstai_spalva4", "1F90C-1F3FE"},
    {"suspausti_pirstai_spalva5", "1F90C-1F3FF"},
    {"balta_sirdele", "1F90D"},
    {"ruda_sirdele", "1F90E"},
    {"znybianti_ranka", "1F90F"},
    {"znybianti_ranka_spalva1", "1F90F-1F3FB"},
    {"znybianti_ranka_spalva2", "1F90F-1F3FC"},
    {"znybianti_ranka_spalva3", "1F90F-1F3FD"},
    {"znybianti_ranka_spalva4", "1F90F-1F3FE"},
    {"znybianti_ranka_spalva5", "1F90F-1F3FF"},
    {"veidas_su_uztrauktuku_vietoje_burnos", "1F910"},
    {"veidas_su_pinigu_vietoje_liezuvio", "1F911"},
    {"veidas_su_termometru", "1F912"},
    {"moksliuko_veidas", "1F913"},
    {"mastantis_veidas", "1F914"},
    {"veidas_su_galvos_tvarsciu", "1F915"},
    {"roboto_veidas", "1F916"},
    {"apkabinantis_veidas", "1F917"},
    {"ragu_zenklas", "1F918"},
    {"ragu_zenklas_spalva1", "1F918-1F3FB"},
    {"ragu_zenklas_spalva2", "1F918-1F3FC"},
    {"ragu_zenklas_spalva3", "1F918-1F3FD"},
    {"ragu_zenklas_spalva4", "1F918-1F3FE"},
    {"ragu_zenklas_spalva5", "1F918-1F3FF"},
    {"gestas_paskambink_man", "1F919"},
    {"gestas_paskambink_man_spalva1", "1F919-1F3FB"},
    {"gestas_paskambink_man_spalva2", "1F919-1F3FC"},
    {"gestas_paskambink_man_spalva3", "1F919-1F3FD"},
    {"gestas_paskambink_man_spalva4", "1F919-1F3FE"},
    {"gestas_paskambink_man_spalva5", "1F919-1F3FF"},
    {"pakelta_virsutine_plastakos_puse", "1F91A"},
    {"pakelta_virsutine_plastakos_puse_spalva1", "1F91A-1F3FB"},
    {"pakelta_virsutine_plastakos_puse_spalva2", "1F91A-1F3FC"},
    {"pakelta_virsutine_plastakos_puse_spalva3", "1F91A-1F3FD"},
    {"pakelta_virsutine_plastakos_puse_spalva4", "1F91A-1F3FE"},
    {"pakelta_virsutine_plastakos_puse_spalva5", "1F91A-1F3FF"},
    {"i_kaire_nukreiptas_kumstis", "1F91B"},
    {"i_kaire_nukreiptas_kumstis_spalva1", "1F91B-1F3FB"},
    {"i_kaire_nukreiptas_kumstis_spalva2", "1F91B-1F3FC"},
    {"i_kaire_nukreiptas_kumstis_spalva3", "1F91B-1F3FD"},
    {"i_kaire_nukreiptas_kumstis_spalva4", "1F91B-1F3FE"},
    {"i_kaire_nukreiptas_kumstis_spalva5", "1F91B-1F3FF"},
    {"i_desine_nukreiptas_kumstis", "1F91C"},
    {"i_desine_nukreiptas_kumstis_spalva1", "1F91C-1F3FB"},
    {"i_desine_nukreiptas_kumstis_spalva2", "1F91C-1F3FC"},
    {"i_desine_nukreiptas_kumstis_spalva3", "1F91C-1F3FD"},
    {"i_desine_nukreiptas_kumstis_spalva4", "1F91C-1F3FE"},
    {"i_desine_nukreiptas_kumstis_spalva5", "1F91C-1F3FF"},
    {"rankos_paspaudimas", "1F91D"},
    {"rankos_paspaudimas_spalva1", "1F91D-1F3FB"},
    {"rankos_paspaudimas_spalva2", "1F91D-1F3FC"},
    {"rankos_paspaudimas_spalva3", "1F91D-1F3FD"},
    {"rankos_paspaudimas_spalva4", "1F91D-1F3FE"},
    {"rankos_paspaudimas_spalva5", "1F91D-1F3FF"},
    {"rankos_paspaudimas_spalva1-2", "1FAF1-1F3FB-200D-1FAF2-1F3FC"},
    {"rankos_paspaudimas_spalva1-3", "1FAF1-1F3FB-200D-1FAF2-1F3FD"},
    {"rankos_paspaudimas_spalva1-4", "1FAF1-1F3FB-200D-1FAF2-1F3FE"},
    {"rankos_paspaudimas_spalva1-5", "1FAF1-1F3FB-200D-1FAF2-1F3FF"},
    {"rankos_paspaudimas_spalva2-1", "1FAF1-1F3FC-200D-1FAF2-1F3FB"},
    {"rankos_paspaudimas_spalva2-3", "1FAF1-1F3FC-200D-1FAF2-1F3FD"},
    {"rankos_paspaudimas_spalva2-4", "1FAF1-1F3FC-200D-1FAF2-1F3FE"},
    {"rankos_paspaudimas_spalva2-5", "1FAF1-1F3FC-200D-1FAF2-1F3FF"},
    {"rankos_paspaudimas_spalva3-1", "1FAF1-1F3FD-200D-1FAF2-1F3FB"},
    {"rankos_paspaudimas_spalva3-2", "1FAF1-1F3FD-200D-1FAF2-1F3FC"},
    {"rankos_paspaudimas_spalva3-4", "1FAF1-1F3FD-200D-1FAF2-1F3FE"},
    {"rankos_paspaudimas_spalva3-5", "1FAF1-1F3FD-200D-1FAF2-1F3FF"},
    {"rankos_paspaudimas_spalva4-1", "1FAF1-1F3FE-200D-1FAF2-1F3FB"},
    {"rankos_paspaudimas_spalva4-2", "1FAF1-1F3FE-200D-1FAF2-1F3FC"},
    {"rankos_paspaudimas_spalva4-3", "1FAF1-1F3FE-200D-1FAF2-1F3FD"},
    {"rankos_paspaudimas_spalva4-5", "1FAF1-1F3FE-200D-1FAF2-1F3FF"},
    {"rankos_paspaudimas_spalva5-1", "1FAF1-1F3FF-200D-1FAF2-1F3FB"},
    {"rankos_paspaudimas_spalva5-2", "1FAF1-1F3FF-200D-1FAF2-1F3FC"},
    {"rankos_paspaudimas_spalva5-3", "1FAF1-1F3FF-200D-1FAF2-1F3FD"},
    {"rankos_paspaudimas_spalva5-4", "1FAF1-1F3FF-200D-1FAF2-1F3FE"},
    {"sukryziuoti_pirstai", "1F91E"},
    {"sukryziuoti_pirstai_spalva1", "1F91E-1F3FB"},
    {"sukryziuoti_pirstai_spalva2", "1F91E-1F3FC"},
    {"sukryziuoti_pirstai_spalva3", "1F91E-1F3FD"},
    {"sukryziuoti_pirstai_spalva4", "1F91E-1F3FE"},
    {"sukryziuoti_pirstai_spalva5", "1F91E-1F3FF"},
    {"myliu_tave_gestas", "1F91F"},
    {"myliu_tave_gestas_spalva1", "1F91F-1F3FB"},
    {"myliu_tave_gestas_spalva2", "1F91F-1F3FC"},
    {"myliu_tave_gestas_spalva3", "1F91F-1F3FD"},
    {"myliu_tave_gestas_spalva4", "1F91F-1F3FE"},
    {"myliu_tave_gestas_spalva5", "1F91F-1F3FF"},
    {"kaubojaus_veidas_su_skrybele", "1F920"},
    {"klouno_veidas", "1F921"},
    {"besiruosiantis_vemti_veidas", "1F922"},
    {"raitausi_is_juoko", "1F923"},
    {"besiseilejantis_veidas", "1F924"},
    {"meluojantis_veidas", "1F925"},
    {"o_varge", "1F926"},
    {"o_varge_spalva1", "1F926-1F3FB"},
    {"o_varge_spalva2", "1F926-1F3FC"},
    {"o_varge_spalva3", "1F926-1F3FD"},
    {"o_varge_spalva4", "1F926-1F3FE"},
    {"o_varge_spalva5", "1F926-1F3FF"},
    {"ciaudantis_veidas", "1F927"},
    {"veidas_su_pakeltais_antakiais", "1F928"},
    {"zvaigzdziu_liga", "1F929"},
    {"pamises_veidas", "1F92A"},
    {"tildantis_veidas", "1F92B"},
    {"veidas_su_simboliais_ant_burnos", "1F92C"},
    {"veidas_su_ranka_ant_burnos", "1F92D"},
    {"vemiantis_veidas", "1F92E"},
    {"sprogstanti_galva", "1F92F"},
    {"nescia_moteris", "1F930"},
    {"nescia_moteris_spalva1", "1F930-1F3FB"},
    {"nescia_moteris_spalva2", "1F930-1F3FC"},
    {"nescia_moteris_spalva3", "1F930-1F3FD"},
    {"nescia_moteris_spalva4", "1F930-1F3FE"},
    {"nescia_moteris_spalva5", "1F930-1F3FF"},
    {"maitinimas_krutimi", "1F931"},
    {"maitinimas_krutimi_spalva1", "1F931-1F3FB"},
    {"maitinimas_krutimi_spalva2", "1F931-1F3FC"},
    {"maitinimas_krutimi_spalva3", "1F931-1F3FD"},
    {"maitinimas_krutimi_spalva4", "1F931-1F3FE"},
    {"maitinimas_krutimi_spalva5", "1F931-1F3FF"},
    {"delnai_suglausti", "1F932"},
    {"delnai_suglausti_spalva1", "1F932-1F3FB"},
    {"delnai_suglausti_spalva2", "1F932-1F3FC"},
    {"delnai_suglausti_spalva3", "1F932-1F3FD"},
    {"delnai_suglausti_spalva4", "1F932-1F3FE"},
    {"delnai_suglausti_spalva5", "1F932-1F3FF"},
    {"asmenuke", "1F933"},
    {"asmenuke_spalva1", "1F933-1F3FB"},
    {"asmenuke_spalva2", "1F933-1F3FC"},
    {"asmenuke_spalva3", "1F933-1F3FD"},
    {"asmenuke_spalva4", "1F933-1F3FE"},
    {"asmenuke_spalva5", "1F933-1F3FF"},
    {"princas", "1F934"},
    {"princas_spalva1", "1F934-1F3FB"},
    {"princas_spalva2", "1F934-1F3FC"},
    {"princas_spalva3", "1F934-1F3FD"},
    {"princas_spalva4", "1F934-1F3FE"},
    {"princas_spalva5", "1F934-1F3FF"},
    {"asmuo_su_smokingu", "1F935"},
    {"asmuo_su_smokingu_spalva1", "1F935-1F3FB"},
    {"asmuo_su_smokingu_spalva2", "1F935-1F3FC"},
    {"asmuo_su_smokingu_spalva3", "1F935-1F3FD"},
    {"asmuo_su_smokingu_spalva4", "1F935-1F3FE"},
    {"asmuo_su_smokingu_spalva5", "1F935-1F3FF"},
    {"ponia_claus", "1F936"},
    {"ponia_claus_spalva1", "1F936-1F3FB"},
    {"ponia_claus_spalva2", "1F936-1F3FC"},
    {"ponia_claus_spalva3", "1F936-1F3FD"},
    {"ponia_claus_spalva4", "1F936-1F3FE"},
    {"ponia_claus_spalva5", "1F936-1F3FF"},
    {"guztelejimas_peciais", "1F937"},
    {"guztelejimas_peciais_spalva1", "1F937-1F3FB"},
    {"guztelejimas_peciais_spalva2", "1F937-1F3FC"},
    {"guztelejimas_peciais_spalva3", "1F937-1F3FD"},
    {"guztelejimas_peciais_spalva4", "1F937-1F3FE"},
    {"guztelejimas_peciais_spalva5", "1F937-1F3FF"},
    {"persivertimas_sonu", "1F938"},
    {"persivertimas_sonu_spalva1", "1F938-1F3FB"},
    {"persivertimas_sonu_spalva2", "1F938-1F3FC"},
    {"persivertimas_sonu_spalva3", "1F938-1F3FD"},
    {"persivertimas_sonu_spalva4", "1F938-1F3FE"},
    {"persivertimas_sonu_spalva5", "1F938-1F3FF"},
    {"zongliravimas", "1F939"},
    {"zongliravimas_spalva1", "1F939-1F3FB"},
    {"zongliravimas_spalva2", "1F939-1F3FC"},
    {"zongliravimas_spalva3", "1F939-1F3FD"},
    {"zongliravimas_spalva4", "1F939-1F3FE"},
    {"zongliravimas_spalva5", "1F939-1F3FF"},
    {"fechtuotojas", "1F93A"},
    {"imtynininkai", "1F93C"},
    {"imtynininkai_spalva1", "1F93C-1F3FB"},
    {"imtynininkai_spalva2", "1F93C-1F3FC"},
    {"imtynininkai_spalva3", "1F93C-1F3FD"},
    {"imtynininkai_spalva4", "1F93C-1F3FE"},
    {"imtynininkai_spalva5", "1F93C-1F3FF"},
    {"imtynininkai_spalva1-2", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"imtynininkai_spalva1-3", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"imtynininkai_spalva1-4", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"imtynininkai_spalva1-5", "1F9D1-1F3FB-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"imtynininkai_spalva2-1", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"imtynininkai_spalva2-3", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"imtynininkai_spalva2-4", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"imtynininkai_spalva2-5", "1F9D1-1F3FC-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"imtynininkai_spalva3-1", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"imtynininkai_spalva3-2", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"imtynininkai_spalva3-4", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"imtynininkai_spalva3-5", "1F9D1-1F3FD-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"imtynininkai_spalva4-1", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"imtynininkai_spalva4-2", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"imtynininkai_spalva4-3", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"imtynininkai_spalva4-5", "1F9D1-1F3FE-200D-1FAEF-200D-1F9D1-1F3FF"},
    {"imtynininkai_spalva5-1", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FB"},
    {"imtynininkai_spalva5-2", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FC"},
    {"imtynininkai_spalva5-3", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FD"},
    {"imtynininkai_spalva5-4", "1F9D1-1F3FF-200D-1FAEF-200D-1F9D1-1F3FE"},
    {"vandensvydis", "1F93D"},
    {"vandensvydis_spalva1", "1F93D-1F3FB"},
    {"vandensvydis_spalva2", "1F93D-1F3FC"},
    {"vandensvydis_spalva3", "1F93D-1F3FD"},
    {"vandensvydis_spalva4", "1F93D-1F3FE"},
    {"vandensvydis_spalva5", "1F93D-1F3FF"},
    {"rankinis", "1F93E"},
    {"rankinis_spalva1", "1F93E-1F3FB"},
    {"rankinis_spalva2", "1F93E-1F3FC"},
    {"rankinis_spalva3", "1F93E-1F3FD"},
    {"rankinis_spalva4", "1F93E-1F3FE"},
    {"rankinis_spalva5", "1F93E-1F3FF"},
    {"nardymo_kauke", "1F93F"},
    {"glezna_gele", "1F940"},
    {"bugnas", "1F941"},
    {"sudauztos_taures", "1F942"},
    {"stikline", "1F943"},
    {"saukstas", "1F944"},
    {"vartai", "1F945"},
    {"1_osios_vietos_medalis", "1F947"},
    {"2_osios_vietos_medalis", "1F948"},
    {"3_osios_vietos_medalis", "1F949"},
    {"bokso_pirstine", "1F94A"},
    {"kovos_sporto_sakos_uniforma", "1F94B"},
    {"akmenslydis", "1F94C"},
    {"lakrosas", "1F94D"},
    {"softbolas", "1F94E"},
    {"skraidantis_diskas", "1F94F"},
    {"raguolis", "1F950"},
    {"avokadas", "1F951"},
    {"agurkas", "1F952"},
    {"kumpis", "1F953"},
    {"bulve", "1F954"},
    {"morka", "1F955"},
    {"ilgas_batonas", "1F956"},
    {"zalios_salotos", "1F957"},
    {"prikaistuvis_su_maistu", "1F958"},
    {"idarytas_lavasas", "1F959"},
    {"kiausinis", "1F95A"},
    {"pieno_stikline", "1F95B"},
    {"zemes_riesutas", "1F95C"},
    {"kivis", "1F95D"},
    {"blynai", "1F95E"},
    {"kukulis", "1F95F"},
    {"laimes_sausainiukas", "1F960"},
    {"dezute_issinesti", "1F961"},
    {"lazdeles", "1F962"},
    {"dubuo_su_saukstu", "1F963"},
    {"puodelis_su_siaudeliu", "1F964"},
    {"kokosas", "1F965"},
    {"brokolis", "1F966"},
    {"pyragas", "1F967"},
    {"riestainis", "1F968"},
    {"mesos_gabaleliai", "1F969"},
    {"sumustinis", "1F96A"},
    {"konservai", "1F96B"},
    {"lapine_darzove", "1F96C"},
    {"mangas", "1F96D"},
    {"menulio_pyragas", "1F96E"},
    {"apvalus_riestainis", "1F96F"},
    {"besisypsantis_veidas_su_trimis_sirdelemis", "1F970"},
    {"ziovaujantis_veidas", "1F971"},
    {"besisypsantis_veidas_su_asara", "1F972"},
    {"svenciantis_veidas", "1F973"},
    {"apkvaises_veidas", "1F974"},
    {"ikaites_veidas", "1F975"},
    {"susales_veidas", "1F976"},
    {"nindze", "1F977"},
    {"nindze_spalva1", "1F977-1F3FB"},
    {"nindze_spalva2", "1F977-1F3FC"},
    {"nindze_spalva3", "1F977-1F3FD"},
    {"nindze_spalva4", "1F977-1F3FE"},
    {"nindze_spalva5", "1F977-1F3FF"},
    {"pasleptas_veidas", "1F978"},
    {"asaras_tramdantis_veidas", "1F979"},
    {"maldaujantis_veidas", "1F97A"},
    {"saris", "1F97B"},
    {"laboratorinis_chalatas", "1F97C"},
    {"apsauginiai_akiniai", "1F97D"},
    {"zygio_batai", "1F97E"},
    {"lygiapadis_batelis", "1F97F"},
    {"krabas", "1F980"},
    {"liuto_snukis", "1F981"},
    {"skorpionas_gyvunas", "1F982"},
    {"kalakutas", "1F983"},
    {"vienaragio_snukis", "1F984"},
    {"erelis", "1F985"},
    {"antis", "1F986"},
    {"siksnosparnis", "1F987"},
    {"ryklys", "1F988"},
    {"peleda", "1F989"},
    {"lapes_snukis", "1F98A"},
    {"drugelis", "1F98B"},
    {"elnias", "1F98C"},
    {"gorila", "1F98D"},
    {"driezas", "1F98E"},
    {"raganosis", "1F98F"},
    {"krevete", "1F990"},
    {"kalmaras", "1F991"},
    {"zirafa", "1F992"},
    {"zebras", "1F993"},
    {"ezys", "1F994"},
    {"sauropodas", "1F995"},
    {"reksas", "1F996"},
    {"svirplys", "1F997"},
    {"kengura", "1F998"},
    {"lama", "1F999"},
    {"povas", "1F99A"},
    {"begemotas", "1F99B"},
    {"papuga", "1F99C"},
    {"meskenas", "1F99D"},
    {"omaras", "1F99E"},
    {"uodas", "1F99F"},
    {"mikrobas", "1F9A0"},
    {"barsukas", "1F9A1"},
    {"gulbe", "1F9A2"},
    {"mamutas", "1F9A3"},
    {"drontas", "1F9A4"},
    {"tinginys", "1F9A5"},
    {"udra", "1F9A6"},
    {"orangutangas", "1F9A7"},
    {"skunkas", "1F9A8"},
    {"flamingas", "1F9A9"},
    {"austre", "1F9AA"},
    {"bebras", "1F9AB"},
    {"bizonas", "1F9AC"},
    {"ruonis", "1F9AD"},
    {"suo_vedlys", "1F9AE"},
    {"neregio_lazda", "1F9AF"},
    {"raudoni_plaukai", "1F9B0"},
    {"garbanoti_plaukai", "1F9B1"},
    {"plikas", "1F9B2"},
    {"zili_plaukai", "1F9B3"},
    {"kaulas", "1F9B4"},
    {"koja", "1F9B5"},
    {"koja_spalva1", "1F9B5-1F3FB"},
    {"koja_spalva2", "1F9B5-1F3FC"},
    {"koja_spalva3", "1F9B5-1F3FD"},
    {"koja_spalva4", "1F9B5-1F3FE"},
    {"koja_spalva5", "1F9B5-1F3FF"},
    {"peda", "1F9B6"},
    {"peda_spalva1", "1F9B6-1F3FB"},
    {"peda_spalva2", "1F9B6-1F3FC"},
    {"peda_spalva3", "1F9B6-1F3FD"},
    {"peda_spalva4", "1F9B6-1F3FE"},
    {"peda_spalva5", "1F9B6-1F3FF"},
    {"dantis", "1F9B7"},
    {"superherojus", "1F9B8"},
    {"superherojus_spalva1", "1F9B8-1F3FB"},
    {"superherojus_spalva2", "1F9B8-1F3FC"},
    {"superherojus_spalva3", "1F9B8-1F3FD"},
    {"superherojus_spalva4", "1F9B8-1F3FE"},
    {"superherojus_spalva5", "1F9B8-1F3FF"},
    {"superpiktadarys", "1F9B9"},
    {"superpiktadarys_spalva1", "1F9B9-1F3FB"},
    {"superpiktadarys_spalva2", "1F9B9-1F3FC"},
    {"superpiktadarys_spalva3", "1F9B9-1F3FD"},
    {"superpiktadarys_spalva4", "1F9B9-1F3FE"},
    {"superpiktadarys_spalva5", "1F9B9-1F3FF"},
    {"gelbejimosi_liemene", "1F9BA"},
    {"ausis_su_klausos_aparatu", "1F9BB"},
    {"ausis_su_klausos_aparatu_spalva1", "1F9BB-1F3FB"},
    {"ausis_su_klausos_aparatu_spalva2", "1F9BB-1F3FC"},
    {"ausis_su_klausos_aparatu_spalva3", "1F9BB-1F3FD"},
    {"ausis_su_klausos_aparatu_spalva4", "1F9BB-1F3FE"},
    {"ausis_su_klausos_aparatu_spalva5", "1F9BB-1F3FF"},
    {"motorizuotas_vezimelis", "1F9BC"},
    {"rankomis_valdomas_vezimelis", "1F9BD"},
    {"mechanine_ranka", "1F9BE"},
    {"mechanine_koja", "1F9BF"},
    {"surio_gabaliukas", "1F9C0"},
    {"keksiukas", "1F9C1"},
    {"druska", "1F9C2"},
    {"gerimo_pakelis", "1F9C3"},
    {"cesnakas", "1F9C4"},
    {"svogunas", "1F9C5"},
    {"falafelis", "1F9C6"},
    {"vaflis", "1F9C7"},
    {"sviestas", "1F9C8"},
    {"mate", "1F9C9"},
    {"ledo_kubelis", "1F9CA"},
    {"burbuline_arbata", "1F9CB"},
    {"trolis", "1F9CC"},
    {"stovintis_zmogus", "1F9CD"},
    {"stovintis_zmogus_spalva1", "1F9CD-1F3FB"},
    {"stovintis_zmogus_spalva2", "1F9CD-1F3FC"},
    {"stovintis_zmogus_spalva3", "1F9CD-1F3FD"},
    {"stovintis_zmogus_spalva4", "1F9CD-1F3FE"},
    {"stovintis_zmogus_spalva5", "1F9CD-1F3FF"},
    {"klupantis_zmogus", "1F9CE"},
    {"klupantis_zmogus_spalva1", "1F9CE-1F3FB"},
    {"klupantis_zmogus_spalva2", "1F9CE-1F3FC"},
    {"klupantis_zmogus_spalva3", "1F9CE-1F3FD"},
    {"klupantis_zmogus_spalva4", "1F9CE-1F3FE"},
    {"klupantis_zmogus_spalva5", "1F9CE-1F3FF"},
    {"kurcias_zmogus", "1F9CF"},
    {"kurcias_zmogus_spalva1", "1F9CF-1F3FB"},
    {"kurcias_zmogus_spalva2", "1F9CF-1F3FC"},
    {"kurcias_zmogus_spalva3", "1F9CF-1F3FD"},
    {"kurcias_zmogus_spalva4", "1F9CF-1F3FE"},
    {"kurcias_zmogus_spalva5", "1F9CF-1F3FF"},
    {"veidas_su_monokliu", "1F9D0"},
    {"suaugusysis", "1F9D1"},
    {"suaugusysis_spalva1", "1F9D1-1F3FB"},
    {"suaugusysis_spalva2", "1F9D1-1F3FC"},
    {"suaugusysis_spalva3", "1F9D1-1F3FD"},
    {"suaugusysis_spalva4", "1F9D1-1F3FE"},
    {"suaugusysis_spalva5", "1F9D1-1F3FF"},
    {"vaikas", "1F9D2"},
    {"vaikas_spalva1", "1F9D2-1F3FB"},
    {"vaikas_spalva2", "1F9D2-1F3FC"},
    {"vaikas_spalva3", "1F9D2-1F3FD"},
    {"vaikas_spalva4", "1F9D2-1F3FE"},
    {"vaikas_spalva5", "1F9D2-1F3FF"},
    {"kitas_suaugusysis", "1F9D3"},
    {"kitas_suaugusysis_spalva1", "1F9D3-1F3FB"},
    {"kitas_suaugusysis_spalva2", "1F9D3-1F3FC"},
    {"kitas_suaugusysis_spalva3", "1F9D3-1F3FD"},
    {"kitas_suaugusysis_spalva4", "1F9D3-1F3FE"},
    {"kitas_suaugusysis_spalva5", "1F9D3-1F3FF"},
    {"barzdotas_zmogus", "1F9D4"},
    {"barzdotas_zmogus_spalva1", "1F9D4-1F3FB"},
    {"barzdotas_zmogus_spalva2", "1F9D4-1F3FC"},
    {"barzdotas_zmogus_spalva3", "1F9D4-1F3FD"},
    {"barzdotas_zmogus_spalva4", "1F9D4-1F3FE"},
    {"barzdotas_zmogus_spalva5", "1F9D4-1F3FF"},
    {"moteris_su_skarele", "1F9D5"},
    {"moteris_su_skarele_spalva1", "1F9D5-1F3FB"},
    {"moteris_su_skarele_spalva2", "1F9D5-1F3FC"},
    {"moteris_su_skarele_spalva3", "1F9D5-1F3FD"},
    {"moteris_su_skarele_spalva4", "1F9D5-1F3FE"},
    {"moteris_su_skarele_spalva5", "1F9D5-1F3FF"},
    {"asmuo_garu_kambaryje", "1F9D6"},
    {"asmuo_garu_kambaryje_spalva1", "1F9D6-1F3FB"},
    {"asmuo_garu_kambaryje_spalva2", "1F9D6-1F3FC"},
    {"asmuo_garu_kambaryje_spalva3", "1F9D6-1F3FD"},
    {"asmuo_garu_kambaryje_spalva4", "1F9D6-1F3FE"},
    {"asmuo_garu_kambaryje_spalva5", "1F9D6-1F3FF"},
    {"kopiantis_zmogus", "1F9D7"},
    {"kopiantis_zmogus_spalva1", "1F9D7-1F3FB"},
    {"kopiantis_zmogus_spalva2", "1F9D7-1F3FC"},
    {"kopiantis_zmogus_spalva3", "1F9D7-1F3FD"},
    {"kopiantis_zmogus_spalva4", "1F9D7-1F3FE"},
    {"kopiantis_zmogus_spalva5", "1F9D7-1F3FF"},
    {"zmogus_lotoso_pozoje", "1F9D8"},
    {"zmogus_lotoso_pozoje_spalva1", "1F9D8-1F3FB"},
    {"zmogus_lotoso_pozoje_spalva2", "1F9D8-1F3FC"},
    {"zmogus_lotoso_pozoje_spalva3", "1F9D8-1F3FD"},
    {"zmogus_lotoso_pozoje_spalva4", "1F9D8-1F3FE"},
    {"zmogus_lotoso_pozoje_spalva5", "1F9D8-1F3FF"},
    {"magas", "1F9D9"},
    {"magas_spalva1", "1F9D9-1F3FB"},
    {"magas_spalva2", "1F9D9-1F3FC"},
    {"magas_spalva3", "1F9D9-1F3FD"},
    {"magas_spalva4", "1F9D9-1F3FE"},
    {"magas_spalva5", "1F9D9-1F3FF"},
    {"feja", "1F9DA"},
    {"feja_spalva1", "1F9DA-1F3FB"},
    {"feja_spalva2", "1F9DA-1F3FC"},
    {"feja_spalva3", "1F9DA-1F3FD"},
    {"feja_spalva4", "1F9DA-1F3FE"},
    {"feja_spalva5", "1F9DA-1F3FF"},
    {"vampyras", "1F9DB"},
    {"vampyras_spalva1", "1F9DB-1F3FB"},
    {"vampyras_spalva2", "1F9DB-1F3FC"},
    {"vampyras_spalva3", "1F9DB-1F3FD"},
    {"vampyras_spalva4", "1F9DB-1F3FE"},
    {"vampyras_spalva5", "1F9DB-1F3FF"},
    {"undinius", "1F9DC"},
    {"undinius_spalva1", "1F9DC-1F3FB"},
    {"undinius_spalva2", "1F9DC-1F3FC"},
    {"undinius_spalva3", "1F9DC-1F3FD"},
    {"undinius_spalva4", "1F9DC-1F3FE"},
    {"undinius_spalva5", "1F9DC-1F3FF"},
    {"elfas", "1F9DD"},
    {"elfas_spalva1", "1F9DD-1F3FB"},
    {"elfas_spalva2", "1F9DD-1F3FC"},
    {"elfas_spalva3", "1F9DD-1F3FD"},
    {"elfas_spalva4", "1F9DD-1F3FE"},
    {"elfas_spalva5", "1F9DD-1F3FF"},
    {"dvasia", "1F9DE"},
    {"zombis", "1F9DF"},
    {"smegenys", "1F9E0"},
    {"oranzine_sirdele", "1F9E1"},
    {"reklamine_kepure", "1F9E2"},
    {"skara", "1F9E3"},
    {"pirstines", "1F9E4"},
    {"paltas", "1F9E5"},
    {"kojines", "1F9E6"},
    {"raudonas_vokas", "1F9E7"},
    {"fejerverkas", "1F9E8"},
    {"delione", "1F9E9"},
    {"megintuvelis", "1F9EA"},
    {"petri_lekstele", "1F9EB"},
    {"dnr", "1F9EC"},
    {"kompasas", "1F9ED"},
    {"skaitytuvai", "1F9EE"},
    {"gesintuvas", "1F9EF"},
    {"irankiu_rinkinys", "1F9F0"},
    {"plyta", "1F9F1"},
    {"magnetas", "1F9F2"},
    {"bagazas", "1F9F3"},
    {"losjono_buteliukas", "1F9F4"},
    {"siulas", "1F9F5"},
    {"siulu_kamuolys", "1F9F6"},
    {"ziogelis", "1F9F7"},
    {"meskiukas", "1F9F8"},
    {"sluota", "1F9F9"},
    {"pintine", "1F9FA"},
    {"popieriaus_ritinys", "1F9FB"},
    {"muilas", "1F9FC"},
    {"kempine", "1F9FD"},
    {"cekis", "1F9FE"},
    {"amuletas_akis", "1F9FF"},
    {"baleto_bateliai", "1FA70"},
    {"vientisas_maudymosi_kostiumelis", "1FA71"},
    {"trumpikes", "1FA72"},
    {"sortai", "1FA73"},
    {"sandalai_su_dirzeliu", "1FA74"},
    {"sviesiai_melyna_sirdele", "1FA75"},
    {"pilka_sirdele", "1FA76"},
    {"rozine_sirdele", "1FA77"},
    {"kraujo_lasas", "1FA78"},
    {"pleistras", "1FA79"},
    {"stetoskopas", "1FA7A"},
    {"rentgenas", "1FA7B"},
    {"ramentas", "1FA7C"},
    {"jo_jo", "1FA80"},
    {"aitvaras", "1FA81"},
    {"parasiutas", "1FA82"},
    {"bumerangas", "1FA83"},
    {"stebuklinga_lazdele", "1FA84"},
    {"pinjata", "1FA85"},
    {"matrioskos", "1FA86"},
    {"marakasai", "1FA87"},
    {"fleita", "1FA88"},
    {"arfa", "1FA89"},
    {"trombonas", "1FA8A"},
    {"lobiu_skrynia", "1FA8E"},
    {"kastuvas", "1FA8F"},
    {"planeta_su_ziedu", "1FA90"},
    {"kede", "1FA91"},
    {"skustuvas", "1FA92"},
    {"kirvis", "1FA93"},
    {"zibaline_lempa", "1FA94"},
    {"bandza", "1FA95"},
    {"kareiviskas_salmas", "1FA96"},
    {"akordeonas", "1FA97"},
    {"ilgas_bugnas", "1FA98"},
    {"moneta", "1FA99"},
    {"pjuklas", "1FA9A"},
    {"atsuktuvas", "1FA9B"},
    {"kopecios", "1FA9C"},
    {"kablys", "1FA9D"},
    {"veidrodis", "1FA9E"},
    {"langas", "1FA9F"},
    {"santechnine_pompa", "1FAA0"},
    {"siuvimo_adata", "1FAA1"},
    {"mazgas", "1FAA2"},
    {"kibiras", "1FAA3"},
    {"pelekautai", "1FAA4"},
    {"dantu_sepetelis", "1FAA5"},
    {"paminklas", "1FAA6"},
    {"plakatas", "1FAA7"},
    {"uoliena", "1FAA8"},
    {"veidrodinis_rutulys", "1FAA9"},
    {"tapatybes_kortele", "1FAAA"},
    {"iseikvota_baterija", "1FAAB"},
    {"hamsa", "1FAAC"},
    {"sulankstoma_veduokle", "1FAAD"},
    {"plauku_sukutes", "1FAAE"},
    {"khanda", "1FAAF"},
    {"muse", "1FAB0"},
    {"kirmele", "1FAB1"},
    {"vabaliukas", "1FAB2"},
    {"tarakonas", "1FAB3"},
    {"pasodintas_augalas", "1FAB4"},
    {"mediena", "1FAB5"},
    {"plunksna", "1FAB6"},
    {"lotusas", "1FAB7"},
    {"koralas", "1FAB8"},
    {"tuscias_lizdas", "1FAB9"},
    {"lizdas_su_kiausiniais", "1FABA"},
    {"hiacintas", "1FABB"},
    {"meduza", "1FABC"},
    {"sparnas", "1FABD"},
    {"medis_be_lapu", "1FABE"},
    {"zasis", "1FABF"},
    {"sirdis", "1FAC0"},
    {"plauciai", "1FAC1"},
    {"apsikabine_zmones", "1FAC2"},
    {"nescias_vyras", "1FAC3"},
    {"nescias_vyras_spalva1", "1FAC3-1F3FB"},
    {"nescias_vyras_spalva2", "1FAC3-1F3FC"},
    {"nescias_vyras_spalva3", "1FAC3-1F3FD"},
    {"nescias_vyras_spalva4", "1FAC3-1F3FE"},
    {"nescias_vyras_spalva5", "1FAC3-1F3FF"},
    {"nescias_asmuo", "1FAC4"},
    {"nescias_asmuo_spalva1", "1FAC4-1F3FB"},
    {"nescias_asmuo_spalva2", "1FAC4-1F3FC"},
    {"nescias_asmuo_spalva3", "1FAC4-1F3FD"},
    {"nescias_asmuo_spalva4", "1FAC4-1F3FE"},
    {"nescias_asmuo_spalva5", "1FAC4-1F3FF"},
    {"asmuo_su_karuna", "1FAC5"},
    {"asmuo_su_karuna_spalva1", "1FAC5-1F3FB"},
    {"asmuo_su_karuna_spalva2", "1FAC5-1F3FC"},
    {"asmuo_su_karuna_spalva3", "1FAC5-1F3FD"},
    {"asmuo_su_karuna_spalva4", "1FAC5-1F3FE"},
    {"asmuo_su_karuna_spalva5", "1FAC5-1F3FF"},
    {"pirsto_atspaudas", "1FAC6"},
    {"plaukuotas_padaras", "1FAC8"},
    {"orka", "1FACD"},
    {"briedis", "1FACE"},
    {"asilas", "1FACF"},
    {"melynes", "1FAD0"},
    {"paprika", "1FAD1"},
    {"alyvuoge", "1FAD2"},
    {"paplotis", "1FAD3"},
    {"vyniotinis", "1FAD4"},
    {"fondiu", "1FAD5"},
    {"arbatinukas", "1FAD6"},
    {"pilamas_skystis", "1FAD7"},
    {"pupeles", "1FAD8"},
    {"stiklainis", "1FAD9"},
    {"imbiero_saknis", "1FADA"},
    {"zirniu_ankstis", "1FADB"},
    {"saknine_darzove", "1FADC"},
    {"tiskalas", "1FADF"},
    {"tirpstantis_veidas", "1FAE0"},
    {"pagarba_atiduodantis_veidas", "1FAE1"},
    {"veidas_atmerktomis_akimis_ir_ranka_uzdengta_burna", "1FAE2"},
    {"veidas_su_stebincia_akimi", "1FAE3"},
    {"veidas_istriza_burna", "1FAE4"},
    {"punktyrinis_veidas", "1FAE5"},
    {"prikasta_lupa", "1FAE6"},
    {"burbulai", "1FAE7"},
    {"besikratantis_veidukas", "1FAE8"},
    {"veidas_su_maiseliais_po_akimis", "1FAE9"},
    {"istises_veidas", "1FAEA"},
    {"kovos_debesis", "1FAEF"},
    {"ranka_sukryziuotu_smiliumi_ir_nyksciu", "1FAF0"},
    {"ranka_sukryziuotu_smiliumi_ir_nyksciu_spalva1", "1FAF0-1F3FB"},
    {"ranka_sukryziuotu_smiliumi_ir_nyksciu_spalva2", "1FAF0-1F3FC"},
    {"ranka_sukryziuotu_smiliumi_ir_nyksciu_spalva3", "1FAF0-1F3FD"},
    {"ranka_sukryziuotu_smiliumi_ir_nyksciu_spalva4", "1FAF0-1F3FE"},
    {"ranka_sukryziuotu_smiliumi_ir_nyksciu_spalva5", "1FAF0-1F3FF"},
    {"i_desine_rodanti_ranka", "1FAF1"},
    {"i_desine_rodanti_ranka_spalva1", "1FAF1-1F3FB"},
    {"i_desine_rodanti_ranka_spalva2", "1FAF1-1F3FC"},
    {"i_desine_rodanti_ranka_spalva3", "1FAF1-1F3FD"},
    {"i_desine_rodanti_ranka_spalva4", "1FAF1-1F3FE"},
    {"i_desine_rodanti_ranka_spalva5", "1FAF1-1F3FF"},
    {"i_kaire_rodanti_ranka", "1FAF2"},
    {"i_kaire_rodanti_ranka_spalva1", "1FAF2-1F3FB"},
    {"i_kaire_rodanti_ranka_spalva2", "1FAF2-1F3FC"},
    {"i_kaire_rodanti_ranka_spalva3", "1FAF2-1F3FD"},
    {"i_kaire_rodanti_ranka_spalva4", "1FAF2-1F3FE"},
    {"i_kaire_rodanti_ranka_spalva5", "1FAF2-1F3FF"},
    {"ranka_delnu_zemyn", "1FAF3"},
    {"ranka_delnu_zemyn_spalva1", "1FAF3-1F3FB"},
    {"ranka_delnu_zemyn_spalva2", "1FAF3-1F3FC"},
    {"ranka_delnu_zemyn_spalva3", "1FAF3-1F3FD"},
    {"ranka_delnu_zemyn_spalva4", "1FAF3-1F3FE"},
    {"ranka_delnu_zemyn_spalva5", "1FAF3-1F3FF"},
    {"ranka_delnu_i_virsu", "1FAF4"},
    {"ranka_delnu_i_virsu_spalva1", "1FAF4-1F3FB"},
    {"ranka_delnu_i_virsu_spalva2", "1FAF4-1F3FC"},
    {"ranka_delnu_i_virsu_spalva3", "1FAF4-1F3FD"},
    {"ranka_delnu_i_virsu_spalva4", "1FAF4-1F3FE"},
    {"ranka_delnu_i_virsu_spalva5", "1FAF4-1F3FF"},
    {"i_ziurova_rodantis_smilius", "1FAF5"},
    {"i_ziurova_rodantis_smilius_spalva1", "1FAF5-1F3FB"},
    {"i_ziurova_rodantis_smilius_spalva2", "1FAF5-1F3FC"},
    {"i_ziurova_rodantis_smilius_spalva3", "1FAF5-1F3FD"},
    {"i_ziurova_rodantis_smilius_spalva4", "1FAF5-1F3FE"},
    {"i_ziurova_rodantis_smilius_spalva5", "1FAF5-1F3FF"},
    {"sirdele_is_ranku", "1FAF6"},
    {"sirdele_is_ranku_spalva1", "1FAF6-1F3FB"},
    {"sirdele_is_ranku_spalva2", "1FAF6-1F3FC"},
    {"sirdele_is_ranku_spalva3", "1FAF6-1F3FD"},
    {"sirdele_is_ranku_spalva4", "1FAF6-1F3FE"},
    {"sirdele_is_ranku_spalva5", "1FAF6-1F3FF"},
    {"i_kaire_stumianti_ranka", "1FAF7"},
    {"i_kaire_stumianti_ranka_spalva1", "1FAF7-1F3FB"},
    {"i_kaire_stumianti_ranka_spalva2", "1FAF7-1F3FC"},
    {"i_kaire_stumianti_ranka_spalva3", "1FAF7-1F3FD"},
    {"i_kaire_stumianti_ranka_spalva4", "1FAF7-1F3FE"},
    {"i_kaire_stumianti_ranka_spalva5", "1FAF7-1F3FF"},
    {"i_desine_stumianti_ranka", "1FAF8"},
    {"i_desine_stumianti_ranka_spalva1", "1FAF8-1F3FB"},
    {"i_desine_stumianti_ranka_spalva2", "1FAF8-1F3FC"},
    {"i_desine_stumianti_ranka_spalva3", "1FAF8-1F3FD"},
    {"i_desine_stumianti_ranka_spalva4", "1FAF8-1F3FE"},
    {"i_desine_stumianti_ranka_spalva5", "1FAF8-1F3FF"},
    {"mygtukas_numerio_zenklas", "0023-FE0F-20E3"},
    {"mygtukas_zvaigzdute", "002A-FE0F-20E3"},
    {"mygtukas_0", "0030-FE0F-20E3"},
    {"mygtukas_1", "0031-FE0F-20E3"},
    {"mygtukas_2", "0032-FE0F-20E3"},
    {"mygtukas_3", "0033-FE0F-20E3"},
    {"mygtukas_4", "0034-FE0F-20E3"},
    {"mygtukas_5", "0035-FE0F-20E3"},
    {"mygtukas_6", "0036-FE0F-20E3"},
    {"mygtukas_7", "0037-FE0F-20E3"},
    {"mygtukas_8", "0038-FE0F-20E3"},
    {"mygtukas_9", "0039-FE0F-20E3"},
    {"veliava_dangun_zengimo_sala", "1F1E6-1F1E8"},
    {"veliava_andora", "1F1E6-1F1E9"},
    {"veliava_jungtiniai_arabu_emyratai", "1F1E6-1F1EA"},
    {"veliava_afganistanas", "1F1E6-1F1EB"},
    {"veliava_antigva_ir_barbuda", "1F1E6-1F1EC"},
    {"veliava_angilija", "1F1E6-1F1EE"},
    {"veliava_albanija", "1F1E6-1F1F1"},
    {"veliava_armenija", "1F1E6-1F1F2"},
    {"veliava_angola", "1F1E6-1F1F4"},
    {"veliava_antarktida", "1F1E6-1F1F6"},
    {"veliava_argentina", "1F1E6-1F1F7"},
    {"veliava_amerikos_samoa", "1F1E6-1F1F8"},
    {"veliava_austrija", "1F1E6-1F1F9"},
    {"veliava_australija", "1F1E6-1F1FA"},
    {"veliava_aruba", "1F1E6-1F1FC"},
    {"veliava_alandu_salos", "1F1E6-1F1FD"},
    {"veliava_azerbaidzanas", "1F1E6-1F1FF"},
    {"veliava_bosnija_ir_hercegovina", "1F1E7-1F1E6"},
    {"veliava_barbadosas", "1F1E7-1F1E7"},
    {"veliava_bangladesas", "1F1E7-1F1E9"},
    {"veliava_belgija", "1F1E7-1F1EA"},
    {"veliava_burkina_fasas", "1F1E7-1F1EB"},
    {"veliava_bulgarija", "1F1E7-1F1EC"},
    {"veliava_bahreinas", "1F1E7-1F1ED"},
    {"veliava_burundis", "1F1E7-1F1EE"},
    {"veliava_beninas", "1F1E7-1F1EF"},
    {"veliava_sen_bartelemi", "1F1E7-1F1F1"},
    {"veliava_bermuda", "1F1E7-1F1F2"},
    {"veliava_brunejus", "1F1E7-1F1F3"},
    {"veliava_bolivija", "1F1E7-1F1F4"},
    {"veliava_karibu_nyderlandai", "1F1E7-1F1F6"},
    {"veliava_brazilija", "1F1E7-1F1F7"},
    {"veliava_bahamos", "1F1E7-1F1F8"},
    {"veliava_butanas", "1F1E7-1F1F9"},
    {"veliava_buve_sala", "1F1E7-1F1FB"},
    {"veliava_botsvana", "1F1E7-1F1FC"},
    {"veliava_baltarusija", "1F1E7-1F1FE"},
    {"veliava_belizas", "1F1E7-1F1FF"},
    {"veliava_kanada", "1F1E8-1F1E6"},
    {"veliava_kokosu_kilingo_salos", "1F1E8-1F1E8"},
    {"veliava_kongas_kinsasa", "1F1E8-1F1E9"},
    {"veliava_centrines_afrikos_respublika", "1F1E8-1F1EB"},
    {"veliava_kongas_brazavilis", "1F1E8-1F1EC"},
    {"veliava_sveicarija", "1F1E8-1F1ED"},
    {"veliava_dramblio_kaulo_krantas", "1F1E8-1F1EE"},
    {"veliava_kuko_salos", "1F1E8-1F1F0"},
    {"veliava_cile", "1F1E8-1F1F1"},
    {"veliava_kamerunas", "1F1E8-1F1F2"},
    {"veliava_kinija", "1F1E8-1F1F3"},
    {"veliava_kolumbija", "1F1E8-1F1F4"},
    {"veliava_klipertono_sala", "1F1E8-1F1F5"},
    {"veliava_sarkas", "1F1E8-1F1F6"},
    {"veliava_kosta_rika", "1F1E8-1F1F7"},
    {"veliava_kuba", "1F1E8-1F1FA"},
    {"veliava_zaliasis_kysulys", "1F1E8-1F1FB"},
    {"veliava_kiurasao", "1F1E8-1F1FC"},
    {"veliava_kaledu_sala", "1F1E8-1F1FD"},
    {"veliava_kipras", "1F1E8-1F1FE"},
    {"veliava_cekija", "1F1E8-1F1FF"},
    {"veliava_vokietija", "1F1E9-1F1EA"},
    {"veliava_diego_garsija", "1F1E9-1F1EC"},
    {"veliava_dzibutis", "1F1E9-1F1EF"},
    {"veliava_danija", "1F1E9-1F1F0"},
    {"veliava_dominika", "1F1E9-1F1F2"},
    {"veliava_dominikos_respublika", "1F1E9-1F1F4"},
    {"veliava_alzyras", "1F1E9-1F1FF"},
    {"veliava_seuta_ir_melila", "1F1EA-1F1E6"},
    {"veliava_ekvadoras", "1F1EA-1F1E8"},
    {"veliava_estija", "1F1EA-1F1EA"},
    {"veliava_egiptas", "1F1EA-1F1EC"},
    {"veliava_vakaru_sachara", "1F1EA-1F1ED"},
    {"veliava_eritreja", "1F1EA-1F1F7"},
    {"veliava_ispanija", "1F1EA-1F1F8"},
    {"veliava_etiopija", "1F1EA-1F1F9"},
    {"veliava_europos_sajunga", "1F1EA-1F1FA"},
    {"veliava_suomija", "1F1EB-1F1EE"},
    {"veliava_fidzis", "1F1EB-1F1EF"},
    {"veliava_folklando_salos", "1F1EB-1F1F0"},
    {"veliava_mikronezija", "1F1EB-1F1F2"},
    {"veliava_fareru_salos", "1F1EB-1F1F4"},
    {"veliava_prancuzija", "1F1EB-1F1F7"},
    {"veliava_gabonas", "1F1EC-1F1E6"},
    {"veliava_jungtine_karalyste", "1F1EC-1F1E7"},
    {"veliava_grenada", "1F1EC-1F1E9"},
    {"veliava_gruzija", "1F1EC-1F1EA"},
    {"veliava_prancuzijos_gviana", "1F1EC-1F1EB"},
    {"veliava_gernsis", "1F1EC-1F1EC"},
    {"veliava_gana", "1F1EC-1F1ED"},
    {"veliava_gibraltaras", "1F1EC-1F1EE"},
    {"veliava_grenlandija", "1F1EC-1F1F1"},
    {"veliava_gambija", "1F1EC-1F1F2"},
    {"veliava_gvineja", "1F1EC-1F1F3"},
    {"veliava_gvadelupa", "1F1EC-1F1F5"},
    {"veliava_pusiaujo_gvineja", "1F1EC-1F1F6"},
    {"veliava_graikija", "1F1EC-1F1F7"},
    {"veliava_pietu_dzordzija_ir_pietu_sandvico_salos", "1F1EC-1F1F8"},
    {"veliava_gvatemala", "1F1EC-1F1F9"},
    {"veliava_guamas", "1F1EC-1F1FA"},
    {"veliava_bisau_gvineja", "1F1EC-1F1FC"},
    {"veliava_gajana", "1F1EC-1F1FE"},
    {"veliava_ypatingasis_administracinis_kinijos_regionas_honkongas", "1F1ED-1F1F0"},
    {"veliava_herdo_ir_makdonaldo_salos", "1F1ED-1F1F2"},
    {"veliava_honduras", "1F1ED-1F1F3"},
    {"veliava_kroatija", "1F1ED-1F1F7"},
    {"veliava_haitis", "1F1ED-1F1F9"},
    {"veliava_vengrija", "1F1ED-1F1FA"},
    {"veliava_kanaru_salos", "1F1EE-1F1E8"},
    {"veliava_indonezija", "1F1EE-1F1E9"},
    {"veliava_airija", "1F1EE-1F1EA"},
    {"veliava_izraelis", "1F1EE-1F1F1"},
    {"veliava_meno_sala", "1F1EE-1F1F2"},
    {"veliava_indija", "1F1EE-1F1F3"},
    {"veliava_indijos_vandenyno_britu_sritis", "1F1EE-1F1F4"},
    {"veliava_irakas", "1F1EE-1F1F6"},
    {"veliava_iranas", "1F1EE-1F1F7"},
    {"veliava_islandija", "1F1EE-1F1F8"},
    {"veliava_italija", "1F1EE-1F1F9"},
    {"veliava_dzersis", "1F1EF-1F1EA"},
    {"veliava_jamaika", "1F1EF-1F1F2"},
    {"veliava_jordanija", "1F1EF-1F1F4"},
    {"veliava_japonija", "1F1EF-1F1F5"},
    {"veliava_kenija", "1F1F0-1F1EA"},
    {"veliava_kirgizija", "1F1F0-1F1EC"},
    {"veliava_kambodza", "1F1F0-1F1ED"},
    {"veliava_kiribatis", "1F1F0-1F1EE"},
    {"veliava_komorai", "1F1F0-1F1F2"},
    {"veliava_sent_kitsas_ir_nevis", "1F1F0-1F1F3"},
    {"veliava_siaures_koreja", "1F1F0-1F1F5"},
    {"veliava_pietu_koreja", "1F1F0-1F1F7"},
    {"veliava_kuveitas", "1F1F0-1F1FC"},
    {"veliava_kaimanu_salos", "1F1F0-1F1FE"},
    {"veliava_kazachstanas", "1F1F0-1F1FF"},
    {"veliava_laosas", "1F1F1-1F1E6"},
    {"veliava_libanas", "1F1F1-1F1E7"},
    {"veliava_sent_lusija", "1F1F1-1F1E8"},
    {"veliava_lichtensteinas", "1F1F1-1F1EE"},
    {"veliava_sri_lanka", "1F1F1-1F1F0"},
    {"veliava_liberija", "1F1F1-1F1F7"},
    {"veliava_lesotas", "1F1F1-1F1F8"},
    {"veliava_lietuva", "1F1F1-1F1F9"},
    {"veliava_liuksemburgas", "1F1F1-1F1FA"},
    {"veliava_latvija", "1F1F1-1F1FB"},
    {"veliava_libija", "1F1F1-1F1FE"},
    {"veliava_marokas", "1F1F2-1F1E6"},
    {"veliava_monakas", "1F1F2-1F1E8"},
    {"veliava_moldova", "1F1F2-1F1E9"},
    {"veliava_juodkalnija", "1F1F2-1F1EA"},
    {"veliava_sen_martenas", "1F1F2-1F1EB"},
    {"veliava_madagaskaras", "1F1F2-1F1EC"},
    {"veliava_marsalo_salos", "1F1F2-1F1ED"},
    {"veliava_siaures_makedonija", "1F1F2-1F1F0"},
    {"veliava_malis", "1F1F2-1F1F1"},
    {"veliava_mianmaras_birma", "1F1F2-1F1F2"},
    {"veliava_mongolija", "1F1F2-1F1F3"},
    {"veliava_ypatingasis_administracinis_kinijos_regionas_makao", "1F1F2-1F1F4"},
    {"veliava_marianos_siaurines_salos", "1F1F2-1F1F5"},
    {"veliava_martinika", "1F1F2-1F1F6"},
    {"veliava_mauritanija", "1F1F2-1F1F7"},
    {"veliava_montseratas", "1F1F2-1F1F8"},
    {"veliava_malta", "1F1F2-1F1F9"},
    {"veliava_mauricijus", "1F1F2-1F1FA"},
    {"veliava_maldyvai", "1F1F2-1F1FB"},
    {"veliava_malavis", "1F1F2-1F1FC"},
    {"veliava_meksika", "1F1F2-1F1FD"},
    {"veliava_malaizija", "1F1F2-1F1FE"},
    {"veliava_mozambikas", "1F1F2-1F1FF"},
    {"veliava_namibija", "1F1F3-1F1E6"},
    {"veliava_naujoji_kaledonija", "1F1F3-1F1E8"},
    {"veliava_nigeris", "1F1F3-1F1EA"},
    {"veliava_norfolko_sala", "1F1F3-1F1EB"},
    {"veliava_nigerija", "1F1F3-1F1EC"},
    {"veliava_nikaragva", "1F1F3-1F1EE"},
    {"veliava_nyderlandai", "1F1F3-1F1F1"},
    {"veliava_norvegija", "1F1F3-1F1F4"},
    {"veliava_nepalas", "1F1F3-1F1F5"},
    {"veliava_nauru", "1F1F3-1F1F7"},
    {"veliava_niuje", "1F1F3-1F1FA"},
    {"veliava_naujoji_zelandija", "1F1F3-1F1FF"},
    {"veliava_omanas", "1F1F4-1F1F2"},
    {"veliava_panama", "1F1F5-1F1E6"},
    {"veliava_peru", "1F1F5-1F1EA"},
    {"veliava_prancuzijos_polinezija", "1F1F5-1F1EB"},
    {"veliava_papua_naujoji_gvineja", "1F1F5-1F1EC"},
    {"veliava_filipinai", "1F1F5-1F1ED"},
    {"veliava_pakistanas", "1F1F5-1F1F0"},
    {"veliava_lenkija", "1F1F5-1F1F1"},
    {"veliava_sen_pjeras_ir_mikelonas", "1F1F5-1F1F2"},
    {"veliava_pitkerno_salos", "1F1F5-1F1F3"},
    {"veliava_puerto_rikas", "1F1F5-1F1F7"},
    {"veliava_palestinos_teritorija", "1F1F5-1F1F8"},
    {"veliava_portugalija", "1F1F5-1F1F9"},
    {"veliava_palau", "1F1F5-1F1FC"},
    {"veliava_paragvajus", "1F1F5-1F1FE"},
    {"veliava_kataras", "1F1F6-1F1E6"},
    {"veliava_reunjonas", "1F1F7-1F1EA"},
    {"veliava_rumunija", "1F1F7-1F1F4"},
    {"veliava_serbija", "1F1F7-1F1F8"},
    {"veliava_rusija", "1F1F7-1F1FA"},
    {"veliava_ruanda", "1F1F7-1F1FC"},
    {"veliava_saudo_arabija", "1F1F8-1F1E6"},
    {"veliava_saliamono_salos", "1F1F8-1F1E7"},
    {"veliava_seiseliai", "1F1F8-1F1E8"},
    {"veliava_sudanas", "1F1F8-1F1E9"},
    {"veliava_svedija", "1F1F8-1F1EA"},
    {"veliava_singapuras", "1F1F8-1F1EC"},
    {"veliava_sv_elenos_sala", "1F1F8-1F1ED"},
    {"veliava_slovenija", "1F1F8-1F1EE"},
    {"veliava_svalbardas_ir_janas_majenas", "1F1F8-1F1EF"},
    {"veliava_slovakija", "1F1F8-1F1F0"},
    {"veliava_siera_leone", "1F1F8-1F1F1"},
    {"veliava_san_marinas", "1F1F8-1F1F2"},
    {"veliava_senegalas", "1F1F8-1F1F3"},
    {"veliava_somalis", "1F1F8-1F1F4"},
    {"veliava_surinamas", "1F1F8-1F1F7"},
    {"veliava_pietu_sudanas", "1F1F8-1F1F8"},
    {"veliava_san_tome_ir_prinsipe", "1F1F8-1F1F9"},
    {"veliava_salvadoras", "1F1F8-1F1FB"},
    {"veliava_sint_martenas", "1F1F8-1F1FD"},
    {"veliava_sirija", "1F1F8-1F1FE"},
    {"veliava_svazilandas", "1F1F8-1F1FF"},
    {"veliava_tristano_da_kunjos", "1F1F9-1F1E6"},
    {"veliava_terkso_ir_kaikoso_salos", "1F1F9-1F1E8"},
    {"veliava_cadas", "1F1F9-1F1E9"},
    {"veliava_prancuzijos_pietu_sritys", "1F1F9-1F1EB"},
    {"veliava_togas", "1F1F9-1F1EC"},
    {"veliava_tailandas", "1F1F9-1F1ED"},
    {"veliava_tadzikija", "1F1F9-1F1EF"},
    {"veliava_tokelau", "1F1F9-1F1F0"},
    {"veliava_rytu_timoras", "1F1F9-1F1F1"},
    {"veliava_turkmenistanas", "1F1F9-1F1F2"},
    {"veliava_tunisas", "1F1F9-1F1F3"},
    {"veliava_tonga", "1F1F9-1F1F4"},
    {"veliava_turkija", "1F1F9-1F1F7"},
    {"veliava_trinidadas_ir_tobagas", "1F1F9-1F1F9"},
    {"veliava_tuvalu", "1F1F9-1F1FB"},
    {"veliava_taivanas", "1F1F9-1F1FC"},
    {"veliava_tanzanija", "1F1F9-1F1FF"},
    {"veliava_ukraina", "1F1FA-1F1E6"},
    {"veliava_uganda", "1F1FA-1F1EC"},
    {"veliava_jungtiniu_valstiju_mazosios_tolimosios_salos", "1F1FA-1F1F2"},
    {"veliava_jungtines_tautos", "1F1FA-1F1F3"},
    {"veliava_jungtines_valstijos", "1F1FA-1F1F8"},
    {"veliava_urugvajus", "1F1FA-1F1FE"},
    {"veliava_uzbekistanas", "1F1FA-1F1FF"},
    {"veliava_vatikano_miesto_valstybe", "1F1FB-1F1E6"},
    {"veliava_sventasis_vincentas_ir_grenadinai", "1F1FB-1F1E8"},
    {"veliava_venesuela", "1F1FB-1F1EA"},
    {"veliava_didziosios_britanijos_mergeliu_salos", "1F1FB-1F1EC"},
    {"veliava_jungtiniu_valstiju_mergeliu_salos", "1F1FB-1F1EE"},
    {"veliava_vietnamas", "1F1FB-1F1F3"},
    {"veliava_vanuatu", "1F1FB-1F1FA"},
    {"veliava_volisas_ir_futuna", "1F1FC-1F1EB"},
    {"veliava_samoa", "1F1FC-1F1F8"},
    {"veliava_kosovas", "1F1FD-1F1F0"},
    {"veliava_jemenas", "1F1FE-1F1EA"},
    {"veliava_majotas", "1F1FE-1F1F9"},
    {"veliava_pietu_afrika", "1F1FF-1F1E6"},
    {"veliava_zambija", "1F1FF-1F1F2"},
    {"veliava_zimbabve", "1F1FF-1F1FC"},
    {"veliava_anglija", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"veliava_skotija", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"veliava_velsas", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"pora_su_sirdele_vyras_vyras", "1F468-200D-2764-FE0F-200D-1F468"},
    {"pora_su_sirdele_vyras_vyras_spalva1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_sirdele_vyras_vyras_spalva1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_sirdele_vyras_vyras_spalva1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_sirdele_vyras_vyras_spalva1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_sirdele_vyras_vyras_spalva1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_sirdele_vyras_vyras_spalva2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_sirdele_vyras_vyras_spalva2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_sirdele_vyras_vyras_spalva2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_sirdele_vyras_vyras_spalva2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_sirdele_vyras_vyras_spalva2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_sirdele_vyras_vyras_spalva3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_sirdele_vyras_vyras_spalva3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_sirdele_vyras_vyras_spalva3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_sirdele_vyras_vyras_spalva3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_sirdele_vyras_vyras_spalva3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_sirdele_vyras_vyras_spalva4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_sirdele_vyras_vyras_spalva4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_sirdele_vyras_vyras_spalva4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_sirdele_vyras_vyras_spalva4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_sirdele_vyras_vyras_spalva4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_sirdele_vyras_vyras_spalva5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_sirdele_vyras_vyras_spalva5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_sirdele_vyras_vyras_spalva5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_sirdele_vyras_vyras_spalva5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_sirdele_vyras_vyras_spalva5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"bucinys_vyras_vyras", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"bucinys_vyras_vyras_spalva1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bucinys_vyras_vyras_spalva1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bucinys_vyras_vyras_spalva1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bucinys_vyras_vyras_spalva1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bucinys_vyras_vyras_spalva1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bucinys_vyras_vyras_spalva2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bucinys_vyras_vyras_spalva2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bucinys_vyras_vyras_spalva2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bucinys_vyras_vyras_spalva2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bucinys_vyras_vyras_spalva2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bucinys_vyras_vyras_spalva3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bucinys_vyras_vyras_spalva3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bucinys_vyras_vyras_spalva3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bucinys_vyras_vyras_spalva3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bucinys_vyras_vyras_spalva3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bucinys_vyras_vyras_spalva4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bucinys_vyras_vyras_spalva4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bucinys_vyras_vyras_spalva4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bucinys_vyras_vyras_spalva4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bucinys_vyras_vyras_spalva4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bucinys_vyras_vyras_spalva5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bucinys_vyras_vyras_spalva5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bucinys_vyras_vyras_spalva5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bucinys_vyras_vyras_spalva5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bucinys_vyras_vyras_spalva5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"seima_vyras_berniukas", "1F468-200D-1F466"},
    {"seima_vyras_berniukas_berniukas", "1F468-200D-1F466-200D-1F466"},
    {"seima_vyras_mergaite", "1F468-200D-1F467"},
    {"seima_vyras_mergaite_berniukas", "1F468-200D-1F467-200D-1F466"},
    {"seima_vyras_mergaite_mergaite", "1F468-200D-1F467-200D-1F467"},
    {"seima_vyras_vyras_berniukas", "1F468-200D-1F468-200D-1F466"},
    {"seima_vyras_vyras_berniukas_berniukas", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"seima_vyras_vyras_mergaite", "1F468-200D-1F468-200D-1F467"},
    {"seima_vyras_vyras_mergaite_berniukas", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"seima_vyras_vyras_mergaite_mergaite", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"seima_vyras_moteris_berniukas", "1F468-200D-1F469-200D-1F466"},
    {"seima_vyras_moteris_berniukas_berniukas", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"seima_vyras_moteris_mergaite", "1F468-200D-1F469-200D-1F467"},
    {"seima_vyras_moteris_mergaite_berniukas", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"seima_vyras_moteris_mergaite_mergaite", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"pora_su_sirdele_moteris_vyras", "1F469-200D-2764-FE0F-200D-1F468"},
    {"pora_su_sirdele_moteris_vyras_spalva1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_sirdele_moteris_vyras_spalva1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_sirdele_moteris_vyras_spalva1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_sirdele_moteris_vyras_spalva1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_sirdele_moteris_vyras_spalva1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_sirdele_moteris_vyras_spalva2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_sirdele_moteris_vyras_spalva2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_sirdele_moteris_vyras_spalva2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_sirdele_moteris_vyras_spalva2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_sirdele_moteris_vyras_spalva2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_sirdele_moteris_vyras_spalva3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_sirdele_moteris_vyras_spalva3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_sirdele_moteris_vyras_spalva3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_sirdele_moteris_vyras_spalva3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_sirdele_moteris_vyras_spalva3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_sirdele_moteris_vyras_spalva4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_sirdele_moteris_vyras_spalva4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_sirdele_moteris_vyras_spalva4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_sirdele_moteris_vyras_spalva4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_sirdele_moteris_vyras_spalva4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_sirdele_moteris_vyras_spalva5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_sirdele_moteris_vyras_spalva5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_sirdele_moteris_vyras_spalva5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_sirdele_moteris_vyras_spalva5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_sirdele_moteris_vyras_spalva5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_sirdele_moteris_moteris", "1F469-200D-2764-FE0F-200D-1F469"},
    {"pora_su_sirdele_moteris_moteris_spalva1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pora_su_sirdele_moteris_moteris_spalva1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pora_su_sirdele_moteris_moteris_spalva1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pora_su_sirdele_moteris_moteris_spalva1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pora_su_sirdele_moteris_moteris_spalva1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pora_su_sirdele_moteris_moteris_spalva2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pora_su_sirdele_moteris_moteris_spalva2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pora_su_sirdele_moteris_moteris_spalva2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pora_su_sirdele_moteris_moteris_spalva2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pora_su_sirdele_moteris_moteris_spalva2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pora_su_sirdele_moteris_moteris_spalva3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pora_su_sirdele_moteris_moteris_spalva3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pora_su_sirdele_moteris_moteris_spalva3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pora_su_sirdele_moteris_moteris_spalva3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pora_su_sirdele_moteris_moteris_spalva3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pora_su_sirdele_moteris_moteris_spalva4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pora_su_sirdele_moteris_moteris_spalva4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pora_su_sirdele_moteris_moteris_spalva4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pora_su_sirdele_moteris_moteris_spalva4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pora_su_sirdele_moteris_moteris_spalva4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pora_su_sirdele_moteris_moteris_spalva5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pora_su_sirdele_moteris_moteris_spalva5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pora_su_sirdele_moteris_moteris_spalva5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pora_su_sirdele_moteris_moteris_spalva5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pora_su_sirdele_moteris_moteris_spalva5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"bucinys_moteris_vyras", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"bucinys_moteris_vyras_spalva1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bucinys_moteris_vyras_spalva1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bucinys_moteris_vyras_spalva1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bucinys_moteris_vyras_spalva1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bucinys_moteris_vyras_spalva1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bucinys_moteris_vyras_spalva2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bucinys_moteris_vyras_spalva2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bucinys_moteris_vyras_spalva2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bucinys_moteris_vyras_spalva2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bucinys_moteris_vyras_spalva2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bucinys_moteris_vyras_spalva3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bucinys_moteris_vyras_spalva3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bucinys_moteris_vyras_spalva3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bucinys_moteris_vyras_spalva3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bucinys_moteris_vyras_spalva3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bucinys_moteris_vyras_spalva4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bucinys_moteris_vyras_spalva4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bucinys_moteris_vyras_spalva4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bucinys_moteris_vyras_spalva4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bucinys_moteris_vyras_spalva4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bucinys_moteris_vyras_spalva5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bucinys_moteris_vyras_spalva5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bucinys_moteris_vyras_spalva5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bucinys_moteris_vyras_spalva5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bucinys_moteris_vyras_spalva5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bucinys_moteris_moteris", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"bucinys_moteris_moteris_spalva1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bucinys_moteris_moteris_spalva1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bucinys_moteris_moteris_spalva1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bucinys_moteris_moteris_spalva1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bucinys_moteris_moteris_spalva1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bucinys_moteris_moteris_spalva2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bucinys_moteris_moteris_spalva2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bucinys_moteris_moteris_spalva2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bucinys_moteris_moteris_spalva2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bucinys_moteris_moteris_spalva2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bucinys_moteris_moteris_spalva3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bucinys_moteris_moteris_spalva3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bucinys_moteris_moteris_spalva3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bucinys_moteris_moteris_spalva3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bucinys_moteris_moteris_spalva3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bucinys_moteris_moteris_spalva4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bucinys_moteris_moteris_spalva4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bucinys_moteris_moteris_spalva4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bucinys_moteris_moteris_spalva4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bucinys_moteris_moteris_spalva4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bucinys_moteris_moteris_spalva5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bucinys_moteris_moteris_spalva5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bucinys_moteris_moteris_spalva5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bucinys_moteris_moteris_spalva5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bucinys_moteris_moteris_spalva5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"seima_moteris_berniukas", "1F469-200D-1F466"},
    {"seima_moteris_berniukas_berniukas", "1F469-200D-1F466-200D-1F466"},
    {"seima_moteris_mergaite", "1F469-200D-1F467"},
    {"seima_moteris_mergaite_berniukas", "1F469-200D-1F467-200D-1F466"},
    {"seima_moteris_mergaite_mergaite", "1F469-200D-1F467-200D-1F467"},
    {"seima_moteris_moteris_berniukas", "1F469-200D-1F469-200D-1F466"},
    {"seima_moteris_moteris_berniukas_berniukas", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"seima_moteris_moteris_mergaite", "1F469-200D-1F469-200D-1F467"},
    {"seima_moteris_moteris_mergaite_berniukas", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"seima_moteris_moteris_mergaite_mergaite", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"rankomis_susikibe_zmones", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"rankomis_susikibe_zmones_spalva1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"rankomis_susikibe_zmones_spalva1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"rankomis_susikibe_zmones_spalva1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"rankomis_susikibe_zmones_spalva1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"rankomis_susikibe_zmones_spalva1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"rankomis_susikibe_zmones_spalva2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"rankomis_susikibe_zmones_spalva2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"rankomis_susikibe_zmones_spalva2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"rankomis_susikibe_zmones_spalva2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"rankomis_susikibe_zmones_spalva2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"rankomis_susikibe_zmones_spalva3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"rankomis_susikibe_zmones_spalva3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"rankomis_susikibe_zmones_spalva3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"rankomis_susikibe_zmones_spalva3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"rankomis_susikibe_zmones_spalva3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"rankomis_susikibe_zmones_spalva4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"rankomis_susikibe_zmones_spalva4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"rankomis_susikibe_zmones_spalva4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"rankomis_susikibe_zmones_spalva4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"rankomis_susikibe_zmones_spalva4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"rankomis_susikibe_zmones_spalva5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"rankomis_susikibe_zmones_spalva5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"rankomis_susikibe_zmones_spalva5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"rankomis_susikibe_zmones_spalva5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"rankomis_susikibe_zmones_spalva5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"seima_suaugusysis_suaugusysis_vaikas", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"seima_suaugusysis_suaugusysis_vaikas_vaikas", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"seima_suaugusysis_vaikas", "1F9D1-200D-1F9D2"},
    {"seima_suaugusysis_vaikas_vaikas", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"begikas_atsisukes_i_desine", "1F3C3-200D-27A1-FE0F"},
    {"begikas_atsisukes_i_desine_spalva1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"begikas_atsisukes_i_desine_spalva2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"begikas_atsisukes_i_desine_spalva3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"begikas_atsisukes_i_desine_spalva4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"begikas_atsisukes_i_desine_spalva5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"medikas", "1F468-200D-2695-FE0F"},
    {"medikas_spalva1", "1F468-1F3FB-200D-2695-FE0F"},
    {"medikas_spalva2", "1F468-1F3FC-200D-2695-FE0F"},
    {"medikas_spalva3", "1F468-1F3FD-200D-2695-FE0F"},
    {"medikas_spalva4", "1F468-1F3FE-200D-2695-FE0F"},
    {"medikas_spalva5", "1F468-1F3FF-200D-2695-FE0F"},
    {"teisejas", "1F468-200D-2696-FE0F"},
    {"teisejas_spalva1", "1F468-1F3FB-200D-2696-FE0F"},
    {"teisejas_spalva2", "1F468-1F3FC-200D-2696-FE0F"},
    {"teisejas_spalva3", "1F468-1F3FD-200D-2696-FE0F"},
    {"teisejas_spalva4", "1F468-1F3FE-200D-2696-FE0F"},
    {"teisejas_spalva5", "1F468-1F3FF-200D-2696-FE0F"},
    {"pilotas", "1F468-200D-2708-FE0F"},
    {"pilotas_spalva1", "1F468-1F3FB-200D-2708-FE0F"},
    {"pilotas_spalva2", "1F468-1F3FC-200D-2708-FE0F"},
    {"pilotas_spalva3", "1F468-1F3FD-200D-2708-FE0F"},
    {"pilotas_spalva4", "1F468-1F3FE-200D-2708-FE0F"},
    {"pilotas_spalva5", "1F468-1F3FF-200D-2708-FE0F"},
    {"ukininkas", "1F468-200D-1F33E"},
    {"ukininkas_spalva1", "1F468-1F3FB-200D-1F33E"},
    {"ukininkas_spalva2", "1F468-1F3FC-200D-1F33E"},
    {"ukininkas_spalva3", "1F468-1F3FD-200D-1F33E"},
    {"ukininkas_spalva4", "1F468-1F3FE-200D-1F33E"},
    {"ukininkas_spalva5", "1F468-1F3FF-200D-1F33E"},
    {"virejas", "1F468-200D-1F373"},
    {"virejas_spalva1", "1F468-1F3FB-200D-1F373"},
    {"virejas_spalva2", "1F468-1F3FC-200D-1F373"},
    {"virejas_spalva3", "1F468-1F3FD-200D-1F373"},
    {"virejas_spalva4", "1F468-1F3FE-200D-1F373"},
    {"virejas_spalva5", "1F468-1F3FF-200D-1F373"},
    {"kudiki_maitinantis_vyras", "1F468-200D-1F37C"},
    {"kudiki_maitinantis_vyras_spalva1", "1F468-1F3FB-200D-1F37C"},
    {"kudiki_maitinantis_vyras_spalva2", "1F468-1F3FC-200D-1F37C"},
    {"kudiki_maitinantis_vyras_spalva3", "1F468-1F3FD-200D-1F37C"},
    {"kudiki_maitinantis_vyras_spalva4", "1F468-1F3FE-200D-1F37C"},
    {"kudiki_maitinantis_vyras_spalva5", "1F468-1F3FF-200D-1F37C"},
    {"studentas", "1F468-200D-1F393"},
    {"studentas_spalva1", "1F468-1F3FB-200D-1F393"},
    {"studentas_spalva2", "1F468-1F3FC-200D-1F393"},
    {"studentas_spalva3", "1F468-1F3FD-200D-1F393"},
    {"studentas_spalva4", "1F468-1F3FE-200D-1F393"},
    {"studentas_spalva5", "1F468-1F3FF-200D-1F393"},
    {"dainininkas", "1F468-200D-1F3A4"},
    {"dainininkas_spalva1", "1F468-1F3FB-200D-1F3A4"},
    {"dainininkas_spalva2", "1F468-1F3FC-200D-1F3A4"},
    {"dainininkas_spalva3", "1F468-1F3FD-200D-1F3A4"},
    {"dainininkas_spalva4", "1F468-1F3FE-200D-1F3A4"},
    {"dainininkas_spalva5", "1F468-1F3FF-200D-1F3A4"},
    {"dailininkas", "1F468-200D-1F3A8"},
    {"dailininkas_spalva1", "1F468-1F3FB-200D-1F3A8"},
    {"dailininkas_spalva2", "1F468-1F3FC-200D-1F3A8"},
    {"dailininkas_spalva3", "1F468-1F3FD-200D-1F3A8"},
    {"dailininkas_spalva4", "1F468-1F3FE-200D-1F3A8"},
    {"dailininkas_spalva5", "1F468-1F3FF-200D-1F3A8"},
    {"mokytojas", "1F468-200D-1F3EB"},
    {"mokytojas_spalva1", "1F468-1F3FB-200D-1F3EB"},
    {"mokytojas_spalva2", "1F468-1F3FC-200D-1F3EB"},
    {"mokytojas_spalva3", "1F468-1F3FD-200D-1F3EB"},
    {"mokytojas_spalva4", "1F468-1F3FE-200D-1F3EB"},
    {"mokytojas_spalva5", "1F468-1F3FF-200D-1F3EB"},
    {"fabriko_darbininkas", "1F468-200D-1F3ED"},
    {"fabriko_darbininkas_spalva1", "1F468-1F3FB-200D-1F3ED"},
    {"fabriko_darbininkas_spalva2", "1F468-1F3FC-200D-1F3ED"},
    {"fabriko_darbininkas_spalva3", "1F468-1F3FD-200D-1F3ED"},
    {"fabriko_darbininkas_spalva4", "1F468-1F3FE-200D-1F3ED"},
    {"fabriko_darbininkas_spalva5", "1F468-1F3FF-200D-1F3ED"},
    {"technologas", "1F468-200D-1F4BB"},
    {"technologas_spalva1", "1F468-1F3FB-200D-1F4BB"},
    {"technologas_spalva2", "1F468-1F3FC-200D-1F4BB"},
    {"technologas_spalva3", "1F468-1F3FD-200D-1F4BB"},
    {"technologas_spalva4", "1F468-1F3FE-200D-1F4BB"},
    {"technologas_spalva5", "1F468-1F3FF-200D-1F4BB"},
    {"biuro_darbuotojas", "1F468-200D-1F4BC"},
    {"biuro_darbuotojas_spalva1", "1F468-1F3FB-200D-1F4BC"},
    {"biuro_darbuotojas_spalva2", "1F468-1F3FC-200D-1F4BC"},
    {"biuro_darbuotojas_spalva3", "1F468-1F3FD-200D-1F4BC"},
    {"biuro_darbuotojas_spalva4", "1F468-1F3FE-200D-1F4BC"},
    {"biuro_darbuotojas_spalva5", "1F468-1F3FF-200D-1F4BC"},
    {"mechanikas", "1F468-200D-1F527"},
    {"mechanikas_spalva1", "1F468-1F3FB-200D-1F527"},
    {"mechanikas_spalva2", "1F468-1F3FC-200D-1F527"},
    {"mechanikas_spalva3", "1F468-1F3FD-200D-1F527"},
    {"mechanikas_spalva4", "1F468-1F3FE-200D-1F527"},
    {"mechanikas_spalva5", "1F468-1F3FF-200D-1F527"},
    {"mokslininkas", "1F468-200D-1F52C"},
    {"mokslininkas_spalva1", "1F468-1F3FB-200D-1F52C"},
    {"mokslininkas_spalva2", "1F468-1F3FC-200D-1F52C"},
    {"mokslininkas_spalva3", "1F468-1F3FD-200D-1F52C"},
    {"mokslininkas_spalva4", "1F468-1F3FE-200D-1F52C"},
    {"mokslininkas_spalva5", "1F468-1F3FF-200D-1F52C"},
    {"astronautas", "1F468-200D-1F680"},
    {"astronautas_spalva1", "1F468-1F3FB-200D-1F680"},
    {"astronautas_spalva2", "1F468-1F3FC-200D-1F680"},
    {"astronautas_spalva3", "1F468-1F3FD-200D-1F680"},
    {"astronautas_spalva4", "1F468-1F3FE-200D-1F680"},
    {"astronautas_spalva5", "1F468-1F3FF-200D-1F680"},
    {"gaisrininkas", "1F468-200D-1F692"},
    {"gaisrininkas_spalva1", "1F468-1F3FB-200D-1F692"},
    {"gaisrininkas_spalva2", "1F468-1F3FC-200D-1F692"},
    {"gaisrininkas_spalva3", "1F468-1F3FD-200D-1F692"},
    {"gaisrininkas_spalva4", "1F468-1F3FE-200D-1F692"},
    {"gaisrininkas_spalva5", "1F468-1F3FF-200D-1F692"},
    {"vyras_su_neregio_lazdele", "1F468-200D-1F9AF"},
    {"vyras_su_neregio_lazdele_spalva1", "1F468-1F3FB-200D-1F9AF"},
    {"vyras_su_neregio_lazdele_spalva2", "1F468-1F3FC-200D-1F9AF"},
    {"vyras_su_neregio_lazdele_spalva3", "1F468-1F3FD-200D-1F9AF"},
    {"vyras_su_neregio_lazdele_spalva4", "1F468-1F3FE-200D-1F9AF"},
    {"vyras_su_neregio_lazdele_spalva5", "1F468-1F3FF-200D-1F9AF"},
    {"vyras_su_neregio_lazdele_atsisukes_i_desine", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"vyras_su_neregio_lazdele_atsisukes_i_desine_spalva1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"vyras_su_neregio_lazdele_atsisukes_i_desine_spalva2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"vyras_su_neregio_lazdele_atsisukes_i_desine_spalva3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"vyras_su_neregio_lazdele_atsisukes_i_desine_spalva4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"vyras_su_neregio_lazdele_atsisukes_i_desine_spalva5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"vyras_motorizuotame_vezimelyje", "1F468-200D-1F9BC"},
    {"vyras_motorizuotame_vezimelyje_spalva1", "1F468-1F3FB-200D-1F9BC"},
    {"vyras_motorizuotame_vezimelyje_spalva2", "1F468-1F3FC-200D-1F9BC"},
    {"vyras_motorizuotame_vezimelyje_spalva3", "1F468-1F3FD-200D-1F9BC"},
    {"vyras_motorizuotame_vezimelyje_spalva4", "1F468-1F3FE-200D-1F9BC"},
    {"vyras_motorizuotame_vezimelyje_spalva5", "1F468-1F3FF-200D-1F9BC"},
    {"vyras_motorizuotame_vezimelyje_atsisukes_i_desine", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"vyras_motorizuotame_vezimelyje_atsisukes_i_desine_spalva1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"vyras_motorizuotame_vezimelyje_atsisukes_i_desine_spalva2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"vyras_motorizuotame_vezimelyje_atsisukes_i_desine_spalva3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"vyras_motorizuotame_vezimelyje_atsisukes_i_desine_spalva4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"vyras_motorizuotame_vezimelyje_atsisukes_i_desine_spalva5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"vyras_rankomis_valdomame_vezimelyje", "1F468-200D-1F9BD"},
    {"vyras_rankomis_valdomame_vezimelyje_spalva1", "1F468-1F3FB-200D-1F9BD"},
    {"vyras_rankomis_valdomame_vezimelyje_spalva2", "1F468-1F3FC-200D-1F9BD"},
    {"vyras_rankomis_valdomame_vezimelyje_spalva3", "1F468-1F3FD-200D-1F9BD"},
    {"vyras_rankomis_valdomame_vezimelyje_spalva4", "1F468-1F3FE-200D-1F9BD"},
    {"vyras_rankomis_valdomame_vezimelyje_spalva5", "1F468-1F3FF-200D-1F9BD"},
    {"vyras_rankomis_valdomame_vezimelyje_atsisukes_i_desine", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"vyras_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"vyras_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"vyras_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"vyras_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"vyras_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"medike", "1F469-200D-2695-FE0F"},
    {"medike_spalva1", "1F469-1F3FB-200D-2695-FE0F"},
    {"medike_spalva2", "1F469-1F3FC-200D-2695-FE0F"},
    {"medike_spalva3", "1F469-1F3FD-200D-2695-FE0F"},
    {"medike_spalva4", "1F469-1F3FE-200D-2695-FE0F"},
    {"medike_spalva5", "1F469-1F3FF-200D-2695-FE0F"},
    {"teiseja", "1F469-200D-2696-FE0F"},
    {"teiseja_spalva1", "1F469-1F3FB-200D-2696-FE0F"},
    {"teiseja_spalva2", "1F469-1F3FC-200D-2696-FE0F"},
    {"teiseja_spalva3", "1F469-1F3FD-200D-2696-FE0F"},
    {"teiseja_spalva4", "1F469-1F3FE-200D-2696-FE0F"},
    {"teiseja_spalva5", "1F469-1F3FF-200D-2696-FE0F"},
    {"pilote", "1F469-200D-2708-FE0F"},
    {"pilote_spalva1", "1F469-1F3FB-200D-2708-FE0F"},
    {"pilote_spalva2", "1F469-1F3FC-200D-2708-FE0F"},
    {"pilote_spalva3", "1F469-1F3FD-200D-2708-FE0F"},
    {"pilote_spalva4", "1F469-1F3FE-200D-2708-FE0F"},
    {"pilote_spalva5", "1F469-1F3FF-200D-2708-FE0F"},
    {"ukininke", "1F469-200D-1F33E"},
    {"ukininke_spalva1", "1F469-1F3FB-200D-1F33E"},
    {"ukininke_spalva2", "1F469-1F3FC-200D-1F33E"},
    {"ukininke_spalva3", "1F469-1F3FD-200D-1F33E"},
    {"ukininke_spalva4", "1F469-1F3FE-200D-1F33E"},
    {"ukininke_spalva5", "1F469-1F3FF-200D-1F33E"},
    {"vireja", "1F469-200D-1F373"},
    {"vireja_spalva1", "1F469-1F3FB-200D-1F373"},
    {"vireja_spalva2", "1F469-1F3FC-200D-1F373"},
    {"vireja_spalva3", "1F469-1F3FD-200D-1F373"},
    {"vireja_spalva4", "1F469-1F3FE-200D-1F373"},
    {"vireja_spalva5", "1F469-1F3FF-200D-1F373"},
    {"kudiki_maitinanti_moteris", "1F469-200D-1F37C"},
    {"kudiki_maitinanti_moteris_spalva1", "1F469-1F3FB-200D-1F37C"},
    {"kudiki_maitinanti_moteris_spalva2", "1F469-1F3FC-200D-1F37C"},
    {"kudiki_maitinanti_moteris_spalva3", "1F469-1F3FD-200D-1F37C"},
    {"kudiki_maitinanti_moteris_spalva4", "1F469-1F3FE-200D-1F37C"},
    {"kudiki_maitinanti_moteris_spalva5", "1F469-1F3FF-200D-1F37C"},
    {"studente", "1F469-200D-1F393"},
    {"studente_spalva1", "1F469-1F3FB-200D-1F393"},
    {"studente_spalva2", "1F469-1F3FC-200D-1F393"},
    {"studente_spalva3", "1F469-1F3FD-200D-1F393"},
    {"studente_spalva4", "1F469-1F3FE-200D-1F393"},
    {"studente_spalva5", "1F469-1F3FF-200D-1F393"},
    {"dainininke", "1F469-200D-1F3A4"},
    {"dainininke_spalva1", "1F469-1F3FB-200D-1F3A4"},
    {"dainininke_spalva2", "1F469-1F3FC-200D-1F3A4"},
    {"dainininke_spalva3", "1F469-1F3FD-200D-1F3A4"},
    {"dainininke_spalva4", "1F469-1F3FE-200D-1F3A4"},
    {"dainininke_spalva5", "1F469-1F3FF-200D-1F3A4"},
    {"dailininke", "1F469-200D-1F3A8"},
    {"dailininke_spalva1", "1F469-1F3FB-200D-1F3A8"},
    {"dailininke_spalva2", "1F469-1F3FC-200D-1F3A8"},
    {"dailininke_spalva3", "1F469-1F3FD-200D-1F3A8"},
    {"dailininke_spalva4", "1F469-1F3FE-200D-1F3A8"},
    {"dailininke_spalva5", "1F469-1F3FF-200D-1F3A8"},
    {"mokytoja", "1F469-200D-1F3EB"},
    {"mokytoja_spalva1", "1F469-1F3FB-200D-1F3EB"},
    {"mokytoja_spalva2", "1F469-1F3FC-200D-1F3EB"},
    {"mokytoja_spalva3", "1F469-1F3FD-200D-1F3EB"},
    {"mokytoja_spalva4", "1F469-1F3FE-200D-1F3EB"},
    {"mokytoja_spalva5", "1F469-1F3FF-200D-1F3EB"},
    {"fabriko_darbininke", "1F469-200D-1F3ED"},
    {"fabriko_darbininke_spalva1", "1F469-1F3FB-200D-1F3ED"},
    {"fabriko_darbininke_spalva2", "1F469-1F3FC-200D-1F3ED"},
    {"fabriko_darbininke_spalva3", "1F469-1F3FD-200D-1F3ED"},
    {"fabriko_darbininke_spalva4", "1F469-1F3FE-200D-1F3ED"},
    {"fabriko_darbininke_spalva5", "1F469-1F3FF-200D-1F3ED"},
    {"technologe", "1F469-200D-1F4BB"},
    {"technologe_spalva1", "1F469-1F3FB-200D-1F4BB"},
    {"technologe_spalva2", "1F469-1F3FC-200D-1F4BB"},
    {"technologe_spalva3", "1F469-1F3FD-200D-1F4BB"},
    {"technologe_spalva4", "1F469-1F3FE-200D-1F4BB"},
    {"technologe_spalva5", "1F469-1F3FF-200D-1F4BB"},
    {"biuro_darbuotoja", "1F469-200D-1F4BC"},
    {"biuro_darbuotoja_spalva1", "1F469-1F3FB-200D-1F4BC"},
    {"biuro_darbuotoja_spalva2", "1F469-1F3FC-200D-1F4BC"},
    {"biuro_darbuotoja_spalva3", "1F469-1F3FD-200D-1F4BC"},
    {"biuro_darbuotoja_spalva4", "1F469-1F3FE-200D-1F4BC"},
    {"biuro_darbuotoja_spalva5", "1F469-1F3FF-200D-1F4BC"},
    {"mechanike", "1F469-200D-1F527"},
    {"mechanike_spalva1", "1F469-1F3FB-200D-1F527"},
    {"mechanike_spalva2", "1F469-1F3FC-200D-1F527"},
    {"mechanike_spalva3", "1F469-1F3FD-200D-1F527"},
    {"mechanike_spalva4", "1F469-1F3FE-200D-1F527"},
    {"mechanike_spalva5", "1F469-1F3FF-200D-1F527"},
    {"mokslininke", "1F469-200D-1F52C"},
    {"mokslininke_spalva1", "1F469-1F3FB-200D-1F52C"},
    {"mokslininke_spalva2", "1F469-1F3FC-200D-1F52C"},
    {"mokslininke_spalva3", "1F469-1F3FD-200D-1F52C"},
    {"mokslininke_spalva4", "1F469-1F3FE-200D-1F52C"},
    {"mokslininke_spalva5", "1F469-1F3FF-200D-1F52C"},
    {"astronaute", "1F469-200D-1F680"},
    {"astronaute_spalva1", "1F469-1F3FB-200D-1F680"},
    {"astronaute_spalva2", "1F469-1F3FC-200D-1F680"},
    {"astronaute_spalva3", "1F469-1F3FD-200D-1F680"},
    {"astronaute_spalva4", "1F469-1F3FE-200D-1F680"},
    {"astronaute_spalva5", "1F469-1F3FF-200D-1F680"},
    {"gaisrininke", "1F469-200D-1F692"},
    {"gaisrininke_spalva1", "1F469-1F3FB-200D-1F692"},
    {"gaisrininke_spalva2", "1F469-1F3FC-200D-1F692"},
    {"gaisrininke_spalva3", "1F469-1F3FD-200D-1F692"},
    {"gaisrininke_spalva4", "1F469-1F3FE-200D-1F692"},
    {"gaisrininke_spalva5", "1F469-1F3FF-200D-1F692"},
    {"moteris_su_neregio_lazdele", "1F469-200D-1F9AF"},
    {"moteris_su_neregio_lazdele_spalva1", "1F469-1F3FB-200D-1F9AF"},
    {"moteris_su_neregio_lazdele_spalva2", "1F469-1F3FC-200D-1F9AF"},
    {"moteris_su_neregio_lazdele_spalva3", "1F469-1F3FD-200D-1F9AF"},
    {"moteris_su_neregio_lazdele_spalva4", "1F469-1F3FE-200D-1F9AF"},
    {"moteris_su_neregio_lazdele_spalva5", "1F469-1F3FF-200D-1F9AF"},
    {"moteris_su_neregio_lazdele_atsisukes_i_desine", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"moteris_su_neregio_lazdele_atsisukes_i_desine_spalva1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"moteris_su_neregio_lazdele_atsisukes_i_desine_spalva2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"moteris_su_neregio_lazdele_atsisukes_i_desine_spalva3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"moteris_su_neregio_lazdele_atsisukes_i_desine_spalva4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"moteris_su_neregio_lazdele_atsisukes_i_desine_spalva5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"moteris_motorizuotame_vezimelyje", "1F469-200D-1F9BC"},
    {"moteris_motorizuotame_vezimelyje_spalva1", "1F469-1F3FB-200D-1F9BC"},
    {"moteris_motorizuotame_vezimelyje_spalva2", "1F469-1F3FC-200D-1F9BC"},
    {"moteris_motorizuotame_vezimelyje_spalva3", "1F469-1F3FD-200D-1F9BC"},
    {"moteris_motorizuotame_vezimelyje_spalva4", "1F469-1F3FE-200D-1F9BC"},
    {"moteris_motorizuotame_vezimelyje_spalva5", "1F469-1F3FF-200D-1F9BC"},
    {"moteris_motorizuotame_vezimelyje_atsisukes_i_desine", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"moteris_motorizuotame_vezimelyje_atsisukes_i_desine_spalva1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"moteris_motorizuotame_vezimelyje_atsisukes_i_desine_spalva2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"moteris_motorizuotame_vezimelyje_atsisukes_i_desine_spalva3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"moteris_motorizuotame_vezimelyje_atsisukes_i_desine_spalva4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"moteris_motorizuotame_vezimelyje_atsisukes_i_desine_spalva5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"moteris_rankomis_valdomame_vezimelyje", "1F469-200D-1F9BD"},
    {"moteris_rankomis_valdomame_vezimelyje_spalva1", "1F469-1F3FB-200D-1F9BD"},
    {"moteris_rankomis_valdomame_vezimelyje_spalva2", "1F469-1F3FC-200D-1F9BD"},
    {"moteris_rankomis_valdomame_vezimelyje_spalva3", "1F469-1F3FD-200D-1F9BD"},
    {"moteris_rankomis_valdomame_vezimelyje_spalva4", "1F469-1F3FE-200D-1F9BD"},
    {"moteris_rankomis_valdomame_vezimelyje_spalva5", "1F469-1F3FF-200D-1F9BD"},
    {"moteris_rankomis_valdomame_vezimelyje_atsisukes_i_desine", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"moteris_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"moteris_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"moteris_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"moteris_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"moteris_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"pestysis_atsisukes_i_desine", "1F6B6-200D-27A1-FE0F"},
    {"pestysis_atsisukes_i_desine_spalva1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"pestysis_atsisukes_i_desine_spalva2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"pestysis_atsisukes_i_desine_spalva3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"pestysis_atsisukes_i_desine_spalva4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"pestysis_atsisukes_i_desine_spalva5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"klupantis_zmogus_atsisukes_i_desine", "1F9CE-200D-27A1-FE0F"},
    {"klupantis_zmogus_atsisukes_i_desine_spalva1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"klupantis_zmogus_atsisukes_i_desine_spalva2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"klupantis_zmogus_atsisukes_i_desine_spalva3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"klupantis_zmogus_atsisukes_i_desine_spalva4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"klupantis_zmogus_atsisukes_i_desine_spalva5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"sveikatos_darbuotojas", "1F9D1-200D-2695-FE0F"},
    {"sveikatos_darbuotojas_spalva1", "1F9D1-1F3FB-200D-2695-FE0F"},
    {"sveikatos_darbuotojas_spalva2", "1F9D1-1F3FC-200D-2695-FE0F"},
    {"sveikatos_darbuotojas_spalva3", "1F9D1-1F3FD-200D-2695-FE0F"},
    {"sveikatos_darbuotojas_spalva4", "1F9D1-1F3FE-200D-2695-FE0F"},
    {"sveikatos_darbuotojas_spalva5", "1F9D1-1F3FF-200D-2695-FE0F"},
    {"teisiantis_zmogus", "1F9D1-200D-2696-FE0F"},
    {"teisiantis_zmogus_spalva1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"teisiantis_zmogus_spalva2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"teisiantis_zmogus_spalva3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"teisiantis_zmogus_spalva4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"teisiantis_zmogus_spalva5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"pilotuojantis_zmogus", "1F9D1-200D-2708-FE0F"},
    {"pilotuojantis_zmogus_spalva1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"pilotuojantis_zmogus_spalva2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"pilotuojantis_zmogus_spalva3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"pilotuojantis_zmogus_spalva4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"pilotuojantis_zmogus_spalva5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"ukininkaujantis_zmogus", "1F9D1-200D-1F33E"},
    {"ukininkaujantis_zmogus_spalva1", "1F9D1-1F3FB-200D-1F33E"},
    {"ukininkaujantis_zmogus_spalva2", "1F9D1-1F3FC-200D-1F33E"},
    {"ukininkaujantis_zmogus_spalva3", "1F9D1-1F3FD-200D-1F33E"},
    {"ukininkaujantis_zmogus_spalva4", "1F9D1-1F3FE-200D-1F33E"},
    {"ukininkaujantis_zmogus_spalva5", "1F9D1-1F3FF-200D-1F33E"},
    {"gaminantis_zmogus", "1F9D1-200D-1F373"},
    {"gaminantis_zmogus_spalva1", "1F9D1-1F3FB-200D-1F373"},
    {"gaminantis_zmogus_spalva2", "1F9D1-1F3FC-200D-1F373"},
    {"gaminantis_zmogus_spalva3", "1F9D1-1F3FD-200D-1F373"},
    {"gaminantis_zmogus_spalva4", "1F9D1-1F3FE-200D-1F373"},
    {"gaminantis_zmogus_spalva5", "1F9D1-1F3FF-200D-1F373"},
    {"kudiki_maitinantis_asmuo", "1F9D1-200D-1F37C"},
    {"kudiki_maitinantis_asmuo_spalva1", "1F9D1-1F3FB-200D-1F37C"},
    {"kudiki_maitinantis_asmuo_spalva2", "1F9D1-1F3FC-200D-1F37C"},
    {"kudiki_maitinantis_asmuo_spalva3", "1F9D1-1F3FD-200D-1F37C"},
    {"kudiki_maitinantis_asmuo_spalva4", "1F9D1-1F3FE-200D-1F37C"},
    {"kudiki_maitinantis_asmuo_spalva5", "1F9D1-1F3FF-200D-1F37C"},
    {"kaledu_asmuo", "1F9D1-200D-1F384"},
    {"kaledu_asmuo_spalva1", "1F9D1-1F3FB-200D-1F384"},
    {"kaledu_asmuo_spalva2", "1F9D1-1F3FC-200D-1F384"},
    {"kaledu_asmuo_spalva3", "1F9D1-1F3FD-200D-1F384"},
    {"kaledu_asmuo_spalva4", "1F9D1-1F3FE-200D-1F384"},
    {"kaledu_asmuo_spalva5", "1F9D1-1F3FF-200D-1F384"},
    {"paskutinio_kurso_studentas", "1F9D1-200D-1F393"},
    {"paskutinio_kurso_studentas_spalva1", "1F9D1-1F3FB-200D-1F393"},
    {"paskutinio_kurso_studentas_spalva2", "1F9D1-1F3FC-200D-1F393"},
    {"paskutinio_kurso_studentas_spalva3", "1F9D1-1F3FD-200D-1F393"},
    {"paskutinio_kurso_studentas_spalva4", "1F9D1-1F3FE-200D-1F393"},
    {"paskutinio_kurso_studentas_spalva5", "1F9D1-1F3FF-200D-1F393"},
    {"dainuojantis_zmogus", "1F9D1-200D-1F3A4"},
    {"dainuojantis_zmogus_spalva1", "1F9D1-1F3FB-200D-1F3A4"},
    {"dainuojantis_zmogus_spalva2", "1F9D1-1F3FC-200D-1F3A4"},
    {"dainuojantis_zmogus_spalva3", "1F9D1-1F3FD-200D-1F3A4"},
    {"dainuojantis_zmogus_spalva4", "1F9D1-1F3FE-200D-1F3A4"},
    {"dainuojantis_zmogus_spalva5", "1F9D1-1F3FF-200D-1F3A4"},
    {"tapytojas", "1F9D1-200D-1F3A8"},
    {"tapytojas_spalva1", "1F9D1-1F3FB-200D-1F3A8"},
    {"tapytojas_spalva2", "1F9D1-1F3FC-200D-1F3A8"},
    {"tapytojas_spalva3", "1F9D1-1F3FD-200D-1F3A8"},
    {"tapytojas_spalva4", "1F9D1-1F3FE-200D-1F3A8"},
    {"tapytojas_spalva5", "1F9D1-1F3FF-200D-1F3A8"},
    {"mokantis_zmogus", "1F9D1-200D-1F3EB"},
    {"mokantis_zmogus_spalva1", "1F9D1-1F3FB-200D-1F3EB"},
    {"mokantis_zmogus_spalva2", "1F9D1-1F3FC-200D-1F3EB"},
    {"mokantis_zmogus_spalva3", "1F9D1-1F3FD-200D-1F3EB"},
    {"mokantis_zmogus_spalva4", "1F9D1-1F3FE-200D-1F3EB"},
    {"mokantis_zmogus_spalva5", "1F9D1-1F3FF-200D-1F3EB"},
    {"gamyklos_darbuotojas", "1F9D1-200D-1F3ED"},
    {"gamyklos_darbuotojas_spalva1", "1F9D1-1F3FB-200D-1F3ED"},
    {"gamyklos_darbuotojas_spalva2", "1F9D1-1F3FC-200D-1F3ED"},
    {"gamyklos_darbuotojas_spalva3", "1F9D1-1F3FD-200D-1F3ED"},
    {"gamyklos_darbuotojas_spalva4", "1F9D1-1F3FE-200D-1F3ED"},
    {"gamyklos_darbuotojas_spalva5", "1F9D1-1F3FF-200D-1F3ED"},
    {"technikas", "1F9D1-200D-1F4BB"},
    {"technikas_spalva1", "1F9D1-1F3FB-200D-1F4BB"},
    {"technikas_spalva2", "1F9D1-1F3FC-200D-1F4BB"},
    {"technikas_spalva3", "1F9D1-1F3FD-200D-1F4BB"},
    {"technikas_spalva4", "1F9D1-1F3FE-200D-1F4BB"},
    {"technikas_spalva5", "1F9D1-1F3FF-200D-1F4BB"},
    {"dirbantysis_biure", "1F9D1-200D-1F4BC"},
    {"dirbantysis_biure_spalva1", "1F9D1-1F3FB-200D-1F4BC"},
    {"dirbantysis_biure_spalva2", "1F9D1-1F3FC-200D-1F4BC"},
    {"dirbantysis_biure_spalva3", "1F9D1-1F3FD-200D-1F4BC"},
    {"dirbantysis_biure_spalva4", "1F9D1-1F3FE-200D-1F4BC"},
    {"dirbantysis_biure_spalva5", "1F9D1-1F3FF-200D-1F4BC"},
    {"remontuojantis_zmogus", "1F9D1-200D-1F527"},
    {"remontuojantis_zmogus_spalva1", "1F9D1-1F3FB-200D-1F527"},
    {"remontuojantis_zmogus_spalva2", "1F9D1-1F3FC-200D-1F527"},
    {"remontuojantis_zmogus_spalva3", "1F9D1-1F3FD-200D-1F527"},
    {"remontuojantis_zmogus_spalva4", "1F9D1-1F3FE-200D-1F527"},
    {"remontuojantis_zmogus_spalva5", "1F9D1-1F3FF-200D-1F527"},
    {"mokslo_darbuotojas", "1F9D1-200D-1F52C"},
    {"mokslo_darbuotojas_spalva1", "1F9D1-1F3FB-200D-1F52C"},
    {"mokslo_darbuotojas_spalva2", "1F9D1-1F3FC-200D-1F52C"},
    {"mokslo_darbuotojas_spalva3", "1F9D1-1F3FD-200D-1F52C"},
    {"mokslo_darbuotojas_spalva4", "1F9D1-1F3FE-200D-1F52C"},
    {"mokslo_darbuotojas_spalva5", "1F9D1-1F3FF-200D-1F52C"},
    {"kosmonautas", "1F9D1-200D-1F680"},
    {"kosmonautas_spalva1", "1F9D1-1F3FB-200D-1F680"},
    {"kosmonautas_spalva2", "1F9D1-1F3FC-200D-1F680"},
    {"kosmonautas_spalva3", "1F9D1-1F3FD-200D-1F680"},
    {"kosmonautas_spalva4", "1F9D1-1F3FE-200D-1F680"},
    {"kosmonautas_spalva5", "1F9D1-1F3FF-200D-1F680"},
    {"ugniagesys", "1F9D1-200D-1F692"},
    {"ugniagesys_spalva1", "1F9D1-1F3FB-200D-1F692"},
    {"ugniagesys_spalva2", "1F9D1-1F3FC-200D-1F692"},
    {"ugniagesys_spalva3", "1F9D1-1F3FD-200D-1F692"},
    {"ugniagesys_spalva4", "1F9D1-1F3FE-200D-1F692"},
    {"ugniagesys_spalva5", "1F9D1-1F3FF-200D-1F692"},
    {"zmogus_neregio_lazdele", "1F9D1-200D-1F9AF"},
    {"zmogus_neregio_lazdele_spalva1", "1F9D1-1F3FB-200D-1F9AF"},
    {"zmogus_neregio_lazdele_spalva2", "1F9D1-1F3FC-200D-1F9AF"},
    {"zmogus_neregio_lazdele_spalva3", "1F9D1-1F3FD-200D-1F9AF"},
    {"zmogus_neregio_lazdele_spalva4", "1F9D1-1F3FE-200D-1F9AF"},
    {"zmogus_neregio_lazdele_spalva5", "1F9D1-1F3FF-200D-1F9AF"},
    {"zmogus_neregio_lazdele_atsisukes_i_desine", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"zmogus_neregio_lazdele_atsisukes_i_desine_spalva1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"zmogus_neregio_lazdele_atsisukes_i_desine_spalva2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"zmogus_neregio_lazdele_atsisukes_i_desine_spalva3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"zmogus_neregio_lazdele_atsisukes_i_desine_spalva4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"zmogus_neregio_lazdele_atsisukes_i_desine_spalva5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"zmogus_motorizuotame_vezimelyje", "1F9D1-200D-1F9BC"},
    {"zmogus_motorizuotame_vezimelyje_spalva1", "1F9D1-1F3FB-200D-1F9BC"},
    {"zmogus_motorizuotame_vezimelyje_spalva2", "1F9D1-1F3FC-200D-1F9BC"},
    {"zmogus_motorizuotame_vezimelyje_spalva3", "1F9D1-1F3FD-200D-1F9BC"},
    {"zmogus_motorizuotame_vezimelyje_spalva4", "1F9D1-1F3FE-200D-1F9BC"},
    {"zmogus_motorizuotame_vezimelyje_spalva5", "1F9D1-1F3FF-200D-1F9BC"},
    {"zmogus_motorizuotame_vezimelyje_atsisukes_i_desine", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"zmogus_motorizuotame_vezimelyje_atsisukes_i_desine_spalva1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"zmogus_motorizuotame_vezimelyje_atsisukes_i_desine_spalva2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"zmogus_motorizuotame_vezimelyje_atsisukes_i_desine_spalva3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"zmogus_motorizuotame_vezimelyje_atsisukes_i_desine_spalva4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"zmogus_motorizuotame_vezimelyje_atsisukes_i_desine_spalva5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"zmogus_rankomis_valdomame_vezimelyje", "1F9D1-200D-1F9BD"},
    {"zmogus_rankomis_valdomame_vezimelyje_spalva1", "1F9D1-1F3FB-200D-1F9BD"},
    {"zmogus_rankomis_valdomame_vezimelyje_spalva2", "1F9D1-1F3FC-200D-1F9BD"},
    {"zmogus_rankomis_valdomame_vezimelyje_spalva3", "1F9D1-1F3FD-200D-1F9BD"},
    {"zmogus_rankomis_valdomame_vezimelyje_spalva4", "1F9D1-1F3FE-200D-1F9BD"},
    {"zmogus_rankomis_valdomame_vezimelyje_spalva5", "1F9D1-1F3FF-200D-1F9BD"},
    {"zmogus_rankomis_valdomame_vezimelyje_atsisukes_i_desine", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"zmogus_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"zmogus_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"zmogus_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"zmogus_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"zmogus_rankomis_valdomame_vezimelyje_atsisukes_i_desine_spalva5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"moteris_su_kamuoliu", "26F9-FE0F-200D-2640-FE0F"},
    {"moteris_su_kamuoliu_spalva1", "26F9-1F3FB-200D-2640-FE0F"},
    {"moteris_su_kamuoliu_spalva2", "26F9-1F3FC-200D-2640-FE0F"},
    {"moteris_su_kamuoliu_spalva3", "26F9-1F3FD-200D-2640-FE0F"},
    {"moteris_su_kamuoliu_spalva4", "26F9-1F3FE-200D-2640-FE0F"},
    {"moteris_su_kamuoliu_spalva5", "26F9-1F3FF-200D-2640-FE0F"},
    {"vyras_su_kamuoliu", "26F9-FE0F-200D-2642-FE0F"},
    {"vyras_su_kamuoliu_spalva1", "26F9-1F3FB-200D-2642-FE0F"},
    {"vyras_su_kamuoliu_spalva2", "26F9-1F3FC-200D-2642-FE0F"},
    {"vyras_su_kamuoliu_spalva3", "26F9-1F3FD-200D-2642-FE0F"},
    {"vyras_su_kamuoliu_spalva4", "26F9-1F3FE-200D-2642-FE0F"},
    {"vyras_su_kamuoliu_spalva5", "26F9-1F3FF-200D-2642-FE0F"},
    {"beganti_moteris", "1F3C3-200D-2640-FE0F"},
    {"beganti_moteris_spalva1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"beganti_moteris_spalva2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"beganti_moteris_spalva3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"beganti_moteris_spalva4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"beganti_moteris_spalva5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"beganti_moteris_atsisukes_i_desine", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"beganti_moteris_atsisukes_i_desine_spalva1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"beganti_moteris_atsisukes_i_desine_spalva2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"beganti_moteris_atsisukes_i_desine_spalva3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"beganti_moteris_atsisukes_i_desine_spalva4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"beganti_moteris_atsisukes_i_desine_spalva5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"begantis_vyras", "1F3C3-200D-2642-FE0F"},
    {"begantis_vyras_spalva1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"begantis_vyras_spalva2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"begantis_vyras_spalva3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"begantis_vyras_spalva4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"begantis_vyras_spalva5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"begantis_vyras_atsisukes_i_desine", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"begantis_vyras_atsisukes_i_desine_spalva1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"begantis_vyras_atsisukes_i_desine_spalva2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"begantis_vyras_atsisukes_i_desine_spalva3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"begantis_vyras_atsisukes_i_desine_spalva4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"begantis_vyras_atsisukes_i_desine_spalva5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"banglentininke", "1F3C4-200D-2640-FE0F"},
    {"banglentininke_spalva1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"banglentininke_spalva2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"banglentininke_spalva3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"banglentininke_spalva4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"banglentininke_spalva5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"vyras_banglentininkas", "1F3C4-200D-2642-FE0F"},
    {"vyras_banglentininkas_spalva1", "1F3C4-1F3FB-200D-2642-FE0F"},
    {"vyras_banglentininkas_spalva2", "1F3C4-1F3FC-200D-2642-FE0F"},
    {"vyras_banglentininkas_spalva3", "1F3C4-1F3FD-200D-2642-FE0F"},
    {"vyras_banglentininkas_spalva4", "1F3C4-1F3FE-200D-2642-FE0F"},
    {"vyras_banglentininkas_spalva5", "1F3C4-1F3FF-200D-2642-FE0F"},
    {"plaukike", "1F3CA-200D-2640-FE0F"},
    {"plaukike_spalva1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"plaukike_spalva2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"plaukike_spalva3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"plaukike_spalva4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"plaukike_spalva5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"vyras_plaukikas", "1F3CA-200D-2642-FE0F"},
    {"vyras_plaukikas_spalva1", "1F3CA-1F3FB-200D-2642-FE0F"},
    {"vyras_plaukikas_spalva2", "1F3CA-1F3FC-200D-2642-FE0F"},
    {"vyras_plaukikas_spalva3", "1F3CA-1F3FD-200D-2642-FE0F"},
    {"vyras_plaukikas_spalva4", "1F3CA-1F3FE-200D-2642-FE0F"},
    {"vyras_plaukikas_spalva5", "1F3CA-1F3FF-200D-2642-FE0F"},
    {"svorius_kelianti_moteris", "1F3CB-FE0F-200D-2640-FE0F"},
    {"svorius_kelianti_moteris_spalva1", "1F3CB-1F3FB-200D-2640-FE0F"},
    {"svorius_kelianti_moteris_spalva2", "1F3CB-1F3FC-200D-2640-FE0F"},
    {"svorius_kelianti_moteris_spalva3", "1F3CB-1F3FD-200D-2640-FE0F"},
    {"svorius_kelianti_moteris_spalva4", "1F3CB-1F3FE-200D-2640-FE0F"},
    {"svorius_kelianti_moteris_spalva5", "1F3CB-1F3FF-200D-2640-FE0F"},
    {"svorius_keliantis_vyras", "1F3CB-FE0F-200D-2642-FE0F"},
    {"svorius_keliantis_vyras_spalva1", "1F3CB-1F3FB-200D-2642-FE0F"},
    {"svorius_keliantis_vyras_spalva2", "1F3CB-1F3FC-200D-2642-FE0F"},
    {"svorius_keliantis_vyras_spalva3", "1F3CB-1F3FD-200D-2642-FE0F"},
    {"svorius_keliantis_vyras_spalva4", "1F3CB-1F3FE-200D-2642-FE0F"},
    {"svorius_keliantis_vyras_spalva5", "1F3CB-1F3FF-200D-2642-FE0F"},
    {"golfo_zaideja", "1F3CC-FE0F-200D-2640-FE0F"},
    {"golfo_zaideja_spalva1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"golfo_zaideja_spalva2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"golfo_zaideja_spalva3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"golfo_zaideja_spalva4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"golfo_zaideja_spalva5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"golfo_zaidejas_vyras", "1F3CC-FE0F-200D-2642-FE0F"},
    {"golfo_zaidejas_vyras_spalva1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"golfo_zaidejas_vyras_spalva2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"golfo_zaidejas_vyras_spalva3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"golfo_zaidejas_vyras_spalva4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"golfo_zaidejas_vyras_spalva5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"moteris_policininke", "1F46E-200D-2640-FE0F"},
    {"moteris_policininke_spalva1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"moteris_policininke_spalva2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"moteris_policininke_spalva3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"moteris_policininke_spalva4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"moteris_policininke_spalva5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"vyras_policininkas", "1F46E-200D-2642-FE0F"},
    {"vyras_policininkas_spalva1", "1F46E-1F3FB-200D-2642-FE0F"},
    {"vyras_policininkas_spalva2", "1F46E-1F3FC-200D-2642-FE0F"},
    {"vyras_policininkas_spalva3", "1F46E-1F3FD-200D-2642-FE0F"},
    {"vyras_policininkas_spalva4", "1F46E-1F3FE-200D-2642-FE0F"},
    {"vyras_policininkas_spalva5", "1F46E-1F3FF-200D-2642-FE0F"},
    {"vakarelyje_dalyvaujancios_moterys", "1F46F-200D-2640-FE0F"},
    {"vakarelyje_dalyvaujancios_moterys_spalva1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"vakarelyje_dalyvaujancios_moterys_spalva1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"vakarelyje_dalyvaujancios_moterys_spalva1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"vakarelyje_dalyvaujancios_moterys_spalva1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"vakarelyje_dalyvaujancios_moterys_spalva2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"vakarelyje_dalyvaujancios_moterys_spalva2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"vakarelyje_dalyvaujancios_moterys_spalva2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"vakarelyje_dalyvaujancios_moterys_spalva2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"vakarelyje_dalyvaujancios_moterys_spalva3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"vakarelyje_dalyvaujancios_moterys_spalva3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"vakarelyje_dalyvaujancios_moterys_spalva3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"vakarelyje_dalyvaujancios_moterys_spalva3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"vakarelyje_dalyvaujancios_moterys_spalva4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"vakarelyje_dalyvaujancios_moterys_spalva4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"vakarelyje_dalyvaujancios_moterys_spalva4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"vakarelyje_dalyvaujancios_moterys_spalva4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"vakarelyje_dalyvaujancios_moterys_spalva5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"vakarelyje_dalyvaujancios_moterys_spalva5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"vakarelyje_dalyvaujancios_moterys_spalva5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"vakarelyje_dalyvaujancios_moterys_spalva5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"vakarelyje_dalyvaujancios_moterys_spalva1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"vakarelyje_dalyvaujancios_moterys_spalva2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"vakarelyje_dalyvaujancios_moterys_spalva3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"vakarelyje_dalyvaujancios_moterys_spalva4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"vakarelyje_dalyvaujancios_moterys_spalva5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"vakarelyje_dalyvaujantys_vyrai", "1F46F-200D-2642-FE0F"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"vakarelyje_dalyvaujantys_vyrai_spalva5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"moteris_su_veliumu", "1F470-200D-2640-FE0F"},
    {"moteris_su_veliumu_spalva1", "1F470-1F3FB-200D-2640-FE0F"},
    {"moteris_su_veliumu_spalva2", "1F470-1F3FC-200D-2640-FE0F"},
    {"moteris_su_veliumu_spalva3", "1F470-1F3FD-200D-2640-FE0F"},
    {"moteris_su_veliumu_spalva4", "1F470-1F3FE-200D-2640-FE0F"},
    {"moteris_su_veliumu_spalva5", "1F470-1F3FF-200D-2640-FE0F"},
    {"vyras_su_veliumu", "1F470-200D-2642-FE0F"},
    {"vyras_su_veliumu_spalva1", "1F470-1F3FB-200D-2642-FE0F"},
    {"vyras_su_veliumu_spalva2", "1F470-1F3FC-200D-2642-FE0F"},
    {"vyras_su_veliumu_spalva3", "1F470-1F3FD-200D-2642-FE0F"},
    {"vyras_su_veliumu_spalva4", "1F470-1F3FE-200D-2642-FE0F"},
    {"vyras_su_veliumu_spalva5", "1F470-1F3FF-200D-2642-FE0F"},
    {"blondine", "1F471-200D-2640-FE0F"},
    {"blondine_spalva1", "1F471-1F3FB-200D-2640-FE0F"},
    {"blondine_spalva2", "1F471-1F3FC-200D-2640-FE0F"},
    {"blondine_spalva3", "1F471-1F3FD-200D-2640-FE0F"},
    {"blondine_spalva4", "1F471-1F3FE-200D-2640-FE0F"},
    {"blondine_spalva5", "1F471-1F3FF-200D-2640-FE0F"},
    {"blondinas", "1F471-200D-2642-FE0F"},
    {"blondinas_spalva1", "1F471-1F3FB-200D-2642-FE0F"},
    {"blondinas_spalva2", "1F471-1F3FC-200D-2642-FE0F"},
    {"blondinas_spalva3", "1F471-1F3FD-200D-2642-FE0F"},
    {"blondinas_spalva4", "1F471-1F3FE-200D-2642-FE0F"},
    {"blondinas_spalva5", "1F471-1F3FF-200D-2642-FE0F"},
    {"moteris_su_turbanu", "1F473-200D-2640-FE0F"},
    {"moteris_su_turbanu_spalva1", "1F473-1F3FB-200D-2640-FE0F"},
    {"moteris_su_turbanu_spalva2", "1F473-1F3FC-200D-2640-FE0F"},
    {"moteris_su_turbanu_spalva3", "1F473-1F3FD-200D-2640-FE0F"},
    {"moteris_su_turbanu_spalva4", "1F473-1F3FE-200D-2640-FE0F"},
    {"moteris_su_turbanu_spalva5", "1F473-1F3FF-200D-2640-FE0F"},
    {"vyriskis_su_turbanu", "1F473-200D-2642-FE0F"},
    {"vyriskis_su_turbanu_spalva1", "1F473-1F3FB-200D-2642-FE0F"},
    {"vyriskis_su_turbanu_spalva2", "1F473-1F3FC-200D-2642-FE0F"},
    {"vyriskis_su_turbanu_spalva3", "1F473-1F3FD-200D-2642-FE0F"},
    {"vyriskis_su_turbanu_spalva4", "1F473-1F3FE-200D-2642-FE0F"},
    {"vyriskis_su_turbanu_spalva5", "1F473-1F3FF-200D-2642-FE0F"},
    {"statybininke", "1F477-200D-2640-FE0F"},
    {"statybininke_spalva1", "1F477-1F3FB-200D-2640-FE0F"},
    {"statybininke_spalva2", "1F477-1F3FC-200D-2640-FE0F"},
    {"statybininke_spalva3", "1F477-1F3FD-200D-2640-FE0F"},
    {"statybininke_spalva4", "1F477-1F3FE-200D-2640-FE0F"},
    {"statybininke_spalva5", "1F477-1F3FF-200D-2640-FE0F"},
    {"vyras_statybininkas", "1F477-200D-2642-FE0F"},
    {"vyras_statybininkas_spalva1", "1F477-1F3FB-200D-2642-FE0F"},
    {"vyras_statybininkas_spalva2", "1F477-1F3FC-200D-2642-FE0F"},
    {"vyras_statybininkas_spalva3", "1F477-1F3FD-200D-2642-FE0F"},
    {"vyras_statybininkas_spalva4", "1F477-1F3FE-200D-2642-FE0F"},
    {"vyras_statybininkas_spalva5", "1F477-1F3FF-200D-2642-FE0F"},
    {"ranka_pakelusi_moteris", "1F481-200D-2640-FE0F"},
    {"ranka_pakelusi_moteris_spalva1", "1F481-1F3FB-200D-2640-FE0F"},
    {"ranka_pakelusi_moteris_spalva2", "1F481-1F3FC-200D-2640-FE0F"},
    {"ranka_pakelusi_moteris_spalva3", "1F481-1F3FD-200D-2640-FE0F"},
    {"ranka_pakelusi_moteris_spalva4", "1F481-1F3FE-200D-2640-FE0F"},
    {"ranka_pakelusi_moteris_spalva5", "1F481-1F3FF-200D-2640-FE0F"},
    {"lengvai_ranka_pakeles_vyras", "1F481-200D-2642-FE0F"},
    {"lengvai_ranka_pakeles_vyras_spalva1", "1F481-1F3FB-200D-2642-FE0F"},
    {"lengvai_ranka_pakeles_vyras_spalva2", "1F481-1F3FC-200D-2642-FE0F"},
    {"lengvai_ranka_pakeles_vyras_spalva3", "1F481-1F3FD-200D-2642-FE0F"},
    {"lengvai_ranka_pakeles_vyras_spalva4", "1F481-1F3FE-200D-2642-FE0F"},
    {"lengvai_ranka_pakeles_vyras_spalva5", "1F481-1F3FF-200D-2642-FE0F"},
    {"sargybine", "1F482-200D-2640-FE0F"},
    {"sargybine_spalva1", "1F482-1F3FB-200D-2640-FE0F"},
    {"sargybine_spalva2", "1F482-1F3FC-200D-2640-FE0F"},
    {"sargybine_spalva3", "1F482-1F3FD-200D-2640-FE0F"},
    {"sargybine_spalva4", "1F482-1F3FE-200D-2640-FE0F"},
    {"sargybine_spalva5", "1F482-1F3FF-200D-2640-FE0F"},
    {"vyras_sargybinis", "1F482-200D-2642-FE0F"},
    {"vyras_sargybinis_spalva1", "1F482-1F3FB-200D-2642-FE0F"},
    {"vyras_sargybinis_spalva2", "1F482-1F3FC-200D-2642-FE0F"},
    {"vyras_sargybinis_spalva3", "1F482-1F3FD-200D-2642-FE0F"},
    {"vyras_sargybinis_spalva4", "1F482-1F3FE-200D-2642-FE0F"},
    {"vyras_sargybinis_spalva5", "1F482-1F3FF-200D-2642-FE0F"},
    {"masazuojama_moters_galva", "1F486-200D-2640-FE0F"},
    {"masazuojama_moters_galva_spalva1", "1F486-1F3FB-200D-2640-FE0F"},
    {"masazuojama_moters_galva_spalva2", "1F486-1F3FC-200D-2640-FE0F"},
    {"masazuojama_moters_galva_spalva3", "1F486-1F3FD-200D-2640-FE0F"},
    {"masazuojama_moters_galva_spalva4", "1F486-1F3FE-200D-2640-FE0F"},
    {"masazuojama_moters_galva_spalva5", "1F486-1F3FF-200D-2640-FE0F"},
    {"masazuojama_vyro_galva", "1F486-200D-2642-FE0F"},
    {"masazuojama_vyro_galva_spalva1", "1F486-1F3FB-200D-2642-FE0F"},
    {"masazuojama_vyro_galva_spalva2", "1F486-1F3FC-200D-2642-FE0F"},
    {"masazuojama_vyro_galva_spalva3", "1F486-1F3FD-200D-2642-FE0F"},
    {"masazuojama_vyro_galva_spalva4", "1F486-1F3FE-200D-2642-FE0F"},
    {"masazuojama_vyro_galva_spalva5", "1F486-1F3FF-200D-2642-FE0F"},
    {"apkerpama_moteris", "1F487-200D-2640-FE0F"},
    {"apkerpama_moteris_spalva1", "1F487-1F3FB-200D-2640-FE0F"},
    {"apkerpama_moteris_spalva2", "1F487-1F3FC-200D-2640-FE0F"},
    {"apkerpama_moteris_spalva3", "1F487-1F3FD-200D-2640-FE0F"},
    {"apkerpama_moteris_spalva4", "1F487-1F3FE-200D-2640-FE0F"},
    {"apkerpama_moteris_spalva5", "1F487-1F3FF-200D-2640-FE0F"},
    {"apkerpamas_vyras", "1F487-200D-2642-FE0F"},
    {"apkerpamas_vyras_spalva1", "1F487-1F3FB-200D-2642-FE0F"},
    {"apkerpamas_vyras_spalva2", "1F487-1F3FC-200D-2642-FE0F"},
    {"apkerpamas_vyras_spalva3", "1F487-1F3FD-200D-2642-FE0F"},
    {"apkerpamas_vyras_spalva4", "1F487-1F3FE-200D-2642-FE0F"},
    {"apkerpamas_vyras_spalva5", "1F487-1F3FF-200D-2642-FE0F"},
    {"sekle", "1F575-FE0F-200D-2640-FE0F"},
    {"sekle_spalva1", "1F575-1F3FB-200D-2640-FE0F"},
    {"sekle_spalva2", "1F575-1F3FC-200D-2640-FE0F"},
    {"sekle_spalva3", "1F575-1F3FD-200D-2640-FE0F"},
    {"sekle_spalva4", "1F575-1F3FE-200D-2640-FE0F"},
    {"sekle_spalva5", "1F575-1F3FF-200D-2640-FE0F"},
    {"seklys", "1F575-FE0F-200D-2642-FE0F"},
    {"seklys_spalva1", "1F575-1F3FB-200D-2642-FE0F"},
    {"seklys_spalva2", "1F575-1F3FC-200D-2642-FE0F"},
    {"seklys_spalva3", "1F575-1F3FD-200D-2642-FE0F"},
    {"seklys_spalva4", "1F575-1F3FE-200D-2642-FE0F"},
    {"seklys_spalva5", "1F575-1F3FF-200D-2642-FE0F"},
    {"draudzianti_moteris", "1F645-200D-2640-FE0F"},
    {"draudzianti_moteris_spalva1", "1F645-1F3FB-200D-2640-FE0F"},
    {"draudzianti_moteris_spalva2", "1F645-1F3FC-200D-2640-FE0F"},
    {"draudzianti_moteris_spalva3", "1F645-1F3FD-200D-2640-FE0F"},
    {"draudzianti_moteris_spalva4", "1F645-1F3FE-200D-2640-FE0F"},
    {"draudzianti_moteris_spalva5", "1F645-1F3FF-200D-2640-FE0F"},
    {"draudziantis_vyras", "1F645-200D-2642-FE0F"},
    {"draudziantis_vyras_spalva1", "1F645-1F3FB-200D-2642-FE0F"},
    {"draudziantis_vyras_spalva2", "1F645-1F3FC-200D-2642-FE0F"},
    {"draudziantis_vyras_spalva3", "1F645-1F3FD-200D-2642-FE0F"},
    {"draudziantis_vyras_spalva4", "1F645-1F3FE-200D-2642-FE0F"},
    {"draudziantis_vyras_spalva5", "1F645-1F3FF-200D-2642-FE0F"},
    {"leidzianti_moteris", "1F646-200D-2640-FE0F"},
    {"leidzianti_moteris_spalva1", "1F646-1F3FB-200D-2640-FE0F"},
    {"leidzianti_moteris_spalva2", "1F646-1F3FC-200D-2640-FE0F"},
    {"leidzianti_moteris_spalva3", "1F646-1F3FD-200D-2640-FE0F"},
    {"leidzianti_moteris_spalva4", "1F646-1F3FE-200D-2640-FE0F"},
    {"leidzianti_moteris_spalva5", "1F646-1F3FF-200D-2640-FE0F"},
    {"leidziantis_vyras", "1F646-200D-2642-FE0F"},
    {"leidziantis_vyras_spalva1", "1F646-1F3FB-200D-2642-FE0F"},
    {"leidziantis_vyras_spalva2", "1F646-1F3FC-200D-2642-FE0F"},
    {"leidziantis_vyras_spalva3", "1F646-1F3FD-200D-2642-FE0F"},
    {"leidziantis_vyras_spalva4", "1F646-1F3FE-200D-2642-FE0F"},
    {"leidziantis_vyras_spalva5", "1F646-1F3FF-200D-2642-FE0F"},
    {"nusilenkianti_moteris", "1F647-200D-2640-FE0F"},
    {"nusilenkianti_moteris_spalva1", "1F647-1F3FB-200D-2640-FE0F"},
    {"nusilenkianti_moteris_spalva2", "1F647-1F3FC-200D-2640-FE0F"},
    {"nusilenkianti_moteris_spalva3", "1F647-1F3FD-200D-2640-FE0F"},
    {"nusilenkianti_moteris_spalva4", "1F647-1F3FE-200D-2640-FE0F"},
    {"nusilenkianti_moteris_spalva5", "1F647-1F3FF-200D-2640-FE0F"},
    {"nusilenkiantis_vyras", "1F647-200D-2642-FE0F"},
    {"nusilenkiantis_vyras_spalva1", "1F647-1F3FB-200D-2642-FE0F"},
    {"nusilenkiantis_vyras_spalva2", "1F647-1F3FC-200D-2642-FE0F"},
    {"nusilenkiantis_vyras_spalva3", "1F647-1F3FD-200D-2642-FE0F"},
    {"nusilenkiantis_vyras_spalva4", "1F647-1F3FE-200D-2642-FE0F"},
    {"nusilenkiantis_vyras_spalva5", "1F647-1F3FF-200D-2642-FE0F"},
    {"ranka_iskelusi_moteris", "1F64B-200D-2640-FE0F"},
    {"ranka_iskelusi_moteris_spalva1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"ranka_iskelusi_moteris_spalva2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"ranka_iskelusi_moteris_spalva3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"ranka_iskelusi_moteris_spalva4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"ranka_iskelusi_moteris_spalva5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"ranka_pakeles_vyras", "1F64B-200D-2642-FE0F"},
    {"ranka_pakeles_vyras_spalva1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"ranka_pakeles_vyras_spalva2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"ranka_pakeles_vyras_spalva3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"ranka_pakeles_vyras_spalva4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"ranka_pakeles_vyras_spalva5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"paniurusi_moteris", "1F64D-200D-2640-FE0F"},
    {"paniurusi_moteris_spalva1", "1F64D-1F3FB-200D-2640-FE0F"},
    {"paniurusi_moteris_spalva2", "1F64D-1F3FC-200D-2640-FE0F"},
    {"paniurusi_moteris_spalva3", "1F64D-1F3FD-200D-2640-FE0F"},
    {"paniurusi_moteris_spalva4", "1F64D-1F3FE-200D-2640-FE0F"},
    {"paniurusi_moteris_spalva5", "1F64D-1F3FF-200D-2640-FE0F"},
    {"paniures_vyras", "1F64D-200D-2642-FE0F"},
    {"paniures_vyras_spalva1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"paniures_vyras_spalva2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"paniures_vyras_spalva3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"paniures_vyras_spalva4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"paniures_vyras_spalva5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"susiraukusi_moteris", "1F64E-200D-2640-FE0F"},
    {"susiraukusi_moteris_spalva1", "1F64E-1F3FB-200D-2640-FE0F"},
    {"susiraukusi_moteris_spalva2", "1F64E-1F3FC-200D-2640-FE0F"},
    {"susiraukusi_moteris_spalva3", "1F64E-1F3FD-200D-2640-FE0F"},
    {"susiraukusi_moteris_spalva4", "1F64E-1F3FE-200D-2640-FE0F"},
    {"susiraukusi_moteris_spalva5", "1F64E-1F3FF-200D-2640-FE0F"},
    {"susiraukes_vyras", "1F64E-200D-2642-FE0F"},
    {"susiraukes_vyras_spalva1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"susiraukes_vyras_spalva2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"susiraukes_vyras_spalva3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"susiraukes_vyras_spalva4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"susiraukes_vyras_spalva5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"irkluotoja", "1F6A3-200D-2640-FE0F"},
    {"irkluotoja_spalva1", "1F6A3-1F3FB-200D-2640-FE0F"},
    {"irkluotoja_spalva2", "1F6A3-1F3FC-200D-2640-FE0F"},
    {"irkluotoja_spalva3", "1F6A3-1F3FD-200D-2640-FE0F"},
    {"irkluotoja_spalva4", "1F6A3-1F3FE-200D-2640-FE0F"},
    {"irkluotoja_spalva5", "1F6A3-1F3FF-200D-2640-FE0F"},
    {"irkluotojas", "1F6A3-200D-2642-FE0F"},
    {"irkluotojas_spalva1", "1F6A3-1F3FB-200D-2642-FE0F"},
    {"irkluotojas_spalva2", "1F6A3-1F3FC-200D-2642-FE0F"},
    {"irkluotojas_spalva3", "1F6A3-1F3FD-200D-2642-FE0F"},
    {"irkluotojas_spalva4", "1F6A3-1F3FE-200D-2642-FE0F"},
    {"irkluotojas_spalva5", "1F6A3-1F3FF-200D-2642-FE0F"},
    {"dviratininke", "1F6B4-200D-2640-FE0F"},
    {"dviratininke_spalva1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"dviratininke_spalva2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"dviratininke_spalva3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"dviratininke_spalva4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"dviratininke_spalva5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"vyras_dviratininkas", "1F6B4-200D-2642-FE0F"},
    {"vyras_dviratininkas_spalva1", "1F6B4-1F3FB-200D-2642-FE0F"},
    {"vyras_dviratininkas_spalva2", "1F6B4-1F3FC-200D-2642-FE0F"},
    {"vyras_dviratininkas_spalva3", "1F6B4-1F3FD-200D-2642-FE0F"},
    {"vyras_dviratininkas_spalva4", "1F6B4-1F3FE-200D-2642-FE0F"},
    {"vyras_dviratininkas_spalva5", "1F6B4-1F3FF-200D-2642-FE0F"},
    {"kalnu_dviratininke_moteris", "1F6B5-200D-2640-FE0F"},
    {"kalnu_dviratininke_moteris_spalva1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"kalnu_dviratininke_moteris_spalva2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"kalnu_dviratininke_moteris_spalva3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"kalnu_dviratininke_moteris_spalva4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"kalnu_dviratininke_moteris_spalva5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"kalnu_dviratininkas_vyras", "1F6B5-200D-2642-FE0F"},
    {"kalnu_dviratininkas_vyras_spalva1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"kalnu_dviratininkas_vyras_spalva2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"kalnu_dviratininkas_vyras_spalva3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"kalnu_dviratininkas_vyras_spalva4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"kalnu_dviratininkas_vyras_spalva5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"einanti_moteris", "1F6B6-200D-2640-FE0F"},
    {"einanti_moteris_spalva1", "1F6B6-1F3FB-200D-2640-FE0F"},
    {"einanti_moteris_spalva2", "1F6B6-1F3FC-200D-2640-FE0F"},
    {"einanti_moteris_spalva3", "1F6B6-1F3FD-200D-2640-FE0F"},
    {"einanti_moteris_spalva4", "1F6B6-1F3FE-200D-2640-FE0F"},
    {"einanti_moteris_spalva5", "1F6B6-1F3FF-200D-2640-FE0F"},
    {"einanti_moteris_atsisukes_i_desine", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"einanti_moteris_atsisukes_i_desine_spalva1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"einanti_moteris_atsisukes_i_desine_spalva2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"einanti_moteris_atsisukes_i_desine_spalva3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"einanti_moteris_atsisukes_i_desine_spalva4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"einanti_moteris_atsisukes_i_desine_spalva5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"einantis_vyras", "1F6B6-200D-2642-FE0F"},
    {"einantis_vyras_spalva1", "1F6B6-1F3FB-200D-2642-FE0F"},
    {"einantis_vyras_spalva2", "1F6B6-1F3FC-200D-2642-FE0F"},
    {"einantis_vyras_spalva3", "1F6B6-1F3FD-200D-2642-FE0F"},
    {"einantis_vyras_spalva4", "1F6B6-1F3FE-200D-2642-FE0F"},
    {"einantis_vyras_spalva5", "1F6B6-1F3FF-200D-2642-FE0F"},
    {"einantis_vyras_atsisukes_i_desine", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"einantis_vyras_atsisukes_i_desine_spalva1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"einantis_vyras_atsisukes_i_desine_spalva2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"einantis_vyras_atsisukes_i_desine_spalva3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"einantis_vyras_atsisukes_i_desine_spalva4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"einantis_vyras_atsisukes_i_desine_spalva5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"susierzinusi_moteris", "1F926-200D-2640-FE0F"},
    {"susierzinusi_moteris_spalva1", "1F926-1F3FB-200D-2640-FE0F"},
    {"susierzinusi_moteris_spalva2", "1F926-1F3FC-200D-2640-FE0F"},
    {"susierzinusi_moteris_spalva3", "1F926-1F3FD-200D-2640-FE0F"},
    {"susierzinusi_moteris_spalva4", "1F926-1F3FE-200D-2640-FE0F"},
    {"susierzinusi_moteris_spalva5", "1F926-1F3FF-200D-2640-FE0F"},
    {"susierzines_vyras", "1F926-200D-2642-FE0F"},
    {"susierzines_vyras_spalva1", "1F926-1F3FB-200D-2642-FE0F"},
    {"susierzines_vyras_spalva2", "1F926-1F3FC-200D-2642-FE0F"},
    {"susierzines_vyras_spalva3", "1F926-1F3FD-200D-2642-FE0F"},
    {"susierzines_vyras_spalva4", "1F926-1F3FE-200D-2642-FE0F"},
    {"susierzines_vyras_spalva5", "1F926-1F3FF-200D-2642-FE0F"},
    {"moteris_su_smokingu", "1F935-200D-2640-FE0F"},
    {"moteris_su_smokingu_spalva1", "1F935-1F3FB-200D-2640-FE0F"},
    {"moteris_su_smokingu_spalva2", "1F935-1F3FC-200D-2640-FE0F"},
    {"moteris_su_smokingu_spalva3", "1F935-1F3FD-200D-2640-FE0F"},
    {"moteris_su_smokingu_spalva4", "1F935-1F3FE-200D-2640-FE0F"},
    {"moteris_su_smokingu_spalva5", "1F935-1F3FF-200D-2640-FE0F"},
    {"vyras_su_smokingu", "1F935-200D-2642-FE0F"},
    {"vyras_su_smokingu_spalva1", "1F935-1F3FB-200D-2642-FE0F"},
    {"vyras_su_smokingu_spalva2", "1F935-1F3FC-200D-2642-FE0F"},
    {"vyras_su_smokingu_spalva3", "1F935-1F3FD-200D-2642-FE0F"},
    {"vyras_su_smokingu_spalva4", "1F935-1F3FE-200D-2642-FE0F"},
    {"vyras_su_smokingu_spalva5", "1F935-1F3FF-200D-2642-FE0F"},
    {"abejojanti_moteris", "1F937-200D-2640-FE0F"},
    {"abejojanti_moteris_spalva1", "1F937-1F3FB-200D-2640-FE0F"},
    {"abejojanti_moteris_spalva2", "1F937-1F3FC-200D-2640-FE0F"},
    {"abejojanti_moteris_spalva3", "1F937-1F3FD-200D-2640-FE0F"},
    {"abejojanti_moteris_spalva4", "1F937-1F3FE-200D-2640-FE0F"},
    {"abejojanti_moteris_spalva5", "1F937-1F3FF-200D-2640-FE0F"},
    {"abejojantis_vyras", "1F937-200D-2642-FE0F"},
    {"abejojantis_vyras_spalva1", "1F937-1F3FB-200D-2642-FE0F"},
    {"abejojantis_vyras_spalva2", "1F937-1F3FC-200D-2642-FE0F"},
    {"abejojantis_vyras_spalva3", "1F937-1F3FD-200D-2642-FE0F"},
    {"abejojantis_vyras_spalva4", "1F937-1F3FE-200D-2642-FE0F"},
    {"abejojantis_vyras_spalva5", "1F937-1F3FF-200D-2642-FE0F"},
    {"moteris_daro_suktuka", "1F938-200D-2640-FE0F"},
    {"moteris_daro_suktuka_spalva1", "1F938-1F3FB-200D-2640-FE0F"},
    {"moteris_daro_suktuka_spalva2", "1F938-1F3FC-200D-2640-FE0F"},
    {"moteris_daro_suktuka_spalva3", "1F938-1F3FD-200D-2640-FE0F"},
    {"moteris_daro_suktuka_spalva4", "1F938-1F3FE-200D-2640-FE0F"},
    {"moteris_daro_suktuka_spalva5", "1F938-1F3FF-200D-2640-FE0F"},
    {"vyras_daro_suktuka", "1F938-200D-2642-FE0F"},
    {"vyras_daro_suktuka_spalva1", "1F938-1F3FB-200D-2642-FE0F"},
    {"vyras_daro_suktuka_spalva2", "1F938-1F3FC-200D-2642-FE0F"},
    {"vyras_daro_suktuka_spalva3", "1F938-1F3FD-200D-2642-FE0F"},
    {"vyras_daro_suktuka_spalva4", "1F938-1F3FE-200D-2642-FE0F"},
    {"vyras_daro_suktuka_spalva5", "1F938-1F3FF-200D-2642-FE0F"},
    {"zongliruotoja", "1F939-200D-2640-FE0F"},
    {"zongliruotoja_spalva1", "1F939-1F3FB-200D-2640-FE0F"},
    {"zongliruotoja_spalva2", "1F939-1F3FC-200D-2640-FE0F"},
    {"zongliruotoja_spalva3", "1F939-1F3FD-200D-2640-FE0F"},
    {"zongliruotoja_spalva4", "1F939-1F3FE-200D-2640-FE0F"},
    {"zongliruotoja_spalva5", "1F939-1F3FF-200D-2640-FE0F"},
    {"zongliruotojas", "1F939-200D-2642-FE0F"},
    {"zongliruotojas_spalva1", "1F939-1F3FB-200D-2642-FE0F"},
    {"zongliruotojas_spalva2", "1F939-1F3FC-200D-2642-FE0F"},
    {"zongliruotojas_spalva3", "1F939-1F3FD-200D-2642-FE0F"},
    {"zongliruotojas_spalva4", "1F939-1F3FE-200D-2642-FE0F"},
    {"zongliruotojas_spalva5", "1F939-1F3FF-200D-2642-FE0F"},
    {"moteru_imtynes", "1F93C-200D-2640-FE0F"},
    {"moteru_imtynes_spalva1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"moteru_imtynes_spalva1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"moteru_imtynes_spalva1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"moteru_imtynes_spalva1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"moteru_imtynes_spalva2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"moteru_imtynes_spalva2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"moteru_imtynes_spalva2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"moteru_imtynes_spalva2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"moteru_imtynes_spalva3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"moteru_imtynes_spalva3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"moteru_imtynes_spalva3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"moteru_imtynes_spalva3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"moteru_imtynes_spalva4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"moteru_imtynes_spalva4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"moteru_imtynes_spalva4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"moteru_imtynes_spalva4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"moteru_imtynes_spalva5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"moteru_imtynes_spalva5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"moteru_imtynes_spalva5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"moteru_imtynes_spalva5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"moteru_imtynes_spalva1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"moteru_imtynes_spalva2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"moteru_imtynes_spalva3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"moteru_imtynes_spalva4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"moteru_imtynes_spalva5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"vyru_imtynes", "1F93C-200D-2642-FE0F"},
    {"vyru_imtynes_spalva1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"vyru_imtynes_spalva1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"vyru_imtynes_spalva1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"vyru_imtynes_spalva1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"vyru_imtynes_spalva2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"vyru_imtynes_spalva2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"vyru_imtynes_spalva2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"vyru_imtynes_spalva2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"vyru_imtynes_spalva3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"vyru_imtynes_spalva3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"vyru_imtynes_spalva3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"vyru_imtynes_spalva3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"vyru_imtynes_spalva4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"vyru_imtynes_spalva4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"vyru_imtynes_spalva4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"vyru_imtynes_spalva4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"vyru_imtynes_spalva5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"vyru_imtynes_spalva5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"vyru_imtynes_spalva5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"vyru_imtynes_spalva5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"vyru_imtynes_spalva1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"vyru_imtynes_spalva2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"vyru_imtynes_spalva3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"vyru_imtynes_spalva4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"vyru_imtynes_spalva5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"vandensvydzio_zaideja", "1F93D-200D-2640-FE0F"},
    {"vandensvydzio_zaideja_spalva1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"vandensvydzio_zaideja_spalva2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"vandensvydzio_zaideja_spalva3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"vandensvydzio_zaideja_spalva4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"vandensvydzio_zaideja_spalva5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"vandensvydzio_zaidejas", "1F93D-200D-2642-FE0F"},
    {"vandensvydzio_zaidejas_spalva1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"vandensvydzio_zaidejas_spalva2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"vandensvydzio_zaidejas_spalva3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"vandensvydzio_zaidejas_spalva4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"vandensvydzio_zaidejas_spalva5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"rankininke", "1F93E-200D-2640-FE0F"},
    {"rankininke_spalva1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"rankininke_spalva2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"rankininke_spalva3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"rankininke_spalva4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"rankininke_spalva5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"rankininkas", "1F93E-200D-2642-FE0F"},
    {"rankininkas_spalva1", "1F93E-1F3FB-200D-2642-FE0F"},
    {"rankininkas_spalva2", "1F93E-1F3FC-200D-2642-FE0F"},
    {"rankininkas_spalva3", "1F93E-1F3FD-200D-2642-FE0F"},
    {"rankininkas_spalva4", "1F93E-1F3FE-200D-2642-FE0F"},
    {"rankininkas_spalva5", "1F93E-1F3FF-200D-2642-FE0F"},
    {"moteris_superheroje", "1F9B8-200D-2640-FE0F"},
    {"moteris_superheroje_spalva1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"moteris_superheroje_spalva2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"moteris_superheroje_spalva3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"moteris_superheroje_spalva4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"moteris_superheroje_spalva5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"vyras_superherojus", "1F9B8-200D-2642-FE0F"},
    {"vyras_superherojus_spalva1", "1F9B8-1F3FB-200D-2642-FE0F"},
    {"vyras_superherojus_spalva2", "1F9B8-1F3FC-200D-2642-FE0F"},
    {"vyras_superherojus_spalva3", "1F9B8-1F3FD-200D-2642-FE0F"},
    {"vyras_superherojus_spalva4", "1F9B8-1F3FE-200D-2642-FE0F"},
    {"vyras_superherojus_spalva5", "1F9B8-1F3FF-200D-2642-FE0F"},
    {"moteris_piktadare", "1F9B9-200D-2640-FE0F"},
    {"moteris_piktadare_spalva1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"moteris_piktadare_spalva2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"moteris_piktadare_spalva3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"moteris_piktadare_spalva4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"moteris_piktadare_spalva5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"vyras_piktadarys", "1F9B9-200D-2642-FE0F"},
    {"vyras_piktadarys_spalva1", "1F9B9-1F3FB-200D-2642-FE0F"},
    {"vyras_piktadarys_spalva2", "1F9B9-1F3FC-200D-2642-FE0F"},
    {"vyras_piktadarys_spalva3", "1F9B9-1F3FD-200D-2642-FE0F"},
    {"vyras_piktadarys_spalva4", "1F9B9-1F3FE-200D-2642-FE0F"},
    {"vyras_piktadarys_spalva5", "1F9B9-1F3FF-200D-2642-FE0F"},
    {"stovinti_moteris", "1F9CD-200D-2640-FE0F"},
    {"stovinti_moteris_spalva1", "1F9CD-1F3FB-200D-2640-FE0F"},
    {"stovinti_moteris_spalva2", "1F9CD-1F3FC-200D-2640-FE0F"},
    {"stovinti_moteris_spalva3", "1F9CD-1F3FD-200D-2640-FE0F"},
    {"stovinti_moteris_spalva4", "1F9CD-1F3FE-200D-2640-FE0F"},
    {"stovinti_moteris_spalva5", "1F9CD-1F3FF-200D-2640-FE0F"},
    {"stovintis_vyras", "1F9CD-200D-2642-FE0F"},
    {"stovintis_vyras_spalva1", "1F9CD-1F3FB-200D-2642-FE0F"},
    {"stovintis_vyras_spalva2", "1F9CD-1F3FC-200D-2642-FE0F"},
    {"stovintis_vyras_spalva3", "1F9CD-1F3FD-200D-2642-FE0F"},
    {"stovintis_vyras_spalva4", "1F9CD-1F3FE-200D-2642-FE0F"},
    {"stovintis_vyras_spalva5", "1F9CD-1F3FF-200D-2642-FE0F"},
    {"klupanti_moteris", "1F9CE-200D-2640-FE0F"},
    {"klupanti_moteris_spalva1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"klupanti_moteris_spalva2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"klupanti_moteris_spalva3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"klupanti_moteris_spalva4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"klupanti_moteris_spalva5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"klupanti_moteris_atsisukes_i_desine", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klupanti_moteris_atsisukes_i_desine_spalva1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klupanti_moteris_atsisukes_i_desine_spalva2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klupanti_moteris_atsisukes_i_desine_spalva3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klupanti_moteris_atsisukes_i_desine_spalva4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klupanti_moteris_atsisukes_i_desine_spalva5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klupantis_vyras", "1F9CE-200D-2642-FE0F"},
    {"klupantis_vyras_spalva1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"klupantis_vyras_spalva2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"klupantis_vyras_spalva3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"klupantis_vyras_spalva4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"klupantis_vyras_spalva5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"klupantis_vyras_atsisukes_i_desine", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klupantis_vyras_atsisukes_i_desine_spalva1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klupantis_vyras_atsisukes_i_desine_spalva2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klupantis_vyras_atsisukes_i_desine_spalva3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klupantis_vyras_atsisukes_i_desine_spalva4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klupantis_vyras_atsisukes_i_desine_spalva5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kurcia_moteris", "1F9CF-200D-2640-FE0F"},
    {"kurcia_moteris_spalva1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"kurcia_moteris_spalva2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"kurcia_moteris_spalva3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"kurcia_moteris_spalva4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"kurcia_moteris_spalva5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"kurcias_vyras", "1F9CF-200D-2642-FE0F"},
    {"kurcias_vyras_spalva1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"kurcias_vyras_spalva2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"kurcias_vyras_spalva3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"kurcias_vyras_spalva4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"kurcias_vyras_spalva5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"moteris_barzda", "1F9D4-200D-2640-FE0F"},
    {"moteris_barzda_spalva1", "1F9D4-1F3FB-200D-2640-FE0F"},
    {"moteris_barzda_spalva2", "1F9D4-1F3FC-200D-2640-FE0F"},
    {"moteris_barzda_spalva3", "1F9D4-1F3FD-200D-2640-FE0F"},
    {"moteris_barzda_spalva4", "1F9D4-1F3FE-200D-2640-FE0F"},
    {"moteris_barzda_spalva5", "1F9D4-1F3FF-200D-2640-FE0F"},
    {"vyras_barzda", "1F9D4-200D-2642-FE0F"},
    {"vyras_barzda_spalva1", "1F9D4-1F3FB-200D-2642-FE0F"},
    {"vyras_barzda_spalva2", "1F9D4-1F3FC-200D-2642-FE0F"},
    {"vyras_barzda_spalva3", "1F9D4-1F3FD-200D-2642-FE0F"},
    {"vyras_barzda_spalva4", "1F9D4-1F3FE-200D-2642-FE0F"},
    {"vyras_barzda_spalva5", "1F9D4-1F3FF-200D-2642-FE0F"},
    {"moteris_garu_kambaryje", "1F9D6-200D-2640-FE0F"},
    {"moteris_garu_kambaryje_spalva1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"moteris_garu_kambaryje_spalva2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"moteris_garu_kambaryje_spalva3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"moteris_garu_kambaryje_spalva4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"moteris_garu_kambaryje_spalva5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"vyras_garu_kambaryje", "1F9D6-200D-2642-FE0F"},
    {"vyras_garu_kambaryje_spalva1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"vyras_garu_kambaryje_spalva2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"vyras_garu_kambaryje_spalva3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"vyras_garu_kambaryje_spalva4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"vyras_garu_kambaryje_spalva5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"kopianti_moteris", "1F9D7-200D-2640-FE0F"},
    {"kopianti_moteris_spalva1", "1F9D7-1F3FB-200D-2640-FE0F"},
    {"kopianti_moteris_spalva2", "1F9D7-1F3FC-200D-2640-FE0F"},
    {"kopianti_moteris_spalva3", "1F9D7-1F3FD-200D-2640-FE0F"},
    {"kopianti_moteris_spalva4", "1F9D7-1F3FE-200D-2640-FE0F"},
    {"kopianti_moteris_spalva5", "1F9D7-1F3FF-200D-2640-FE0F"},
    {"kopiantis_vyras", "1F9D7-200D-2642-FE0F"},
    {"kopiantis_vyras_spalva1", "1F9D7-1F3FB-200D-2642-FE0F"},
    {"kopiantis_vyras_spalva2", "1F9D7-1F3FC-200D-2642-FE0F"},
    {"kopiantis_vyras_spalva3", "1F9D7-1F3FD-200D-2642-FE0F"},
    {"kopiantis_vyras_spalva4", "1F9D7-1F3FE-200D-2642-FE0F"},
    {"kopiantis_vyras_spalva5", "1F9D7-1F3FF-200D-2642-FE0F"},
    {"moteris_lotoso_pozoje", "1F9D8-200D-2640-FE0F"},
    {"moteris_lotoso_pozoje_spalva1", "1F9D8-1F3FB-200D-2640-FE0F"},
    {"moteris_lotoso_pozoje_spalva2", "1F9D8-1F3FC-200D-2640-FE0F"},
    {"moteris_lotoso_pozoje_spalva3", "1F9D8-1F3FD-200D-2640-FE0F"},
    {"moteris_lotoso_pozoje_spalva4", "1F9D8-1F3FE-200D-2640-FE0F"},
    {"moteris_lotoso_pozoje_spalva5", "1F9D8-1F3FF-200D-2640-FE0F"},
    {"vyras_lotoso_pozoje", "1F9D8-200D-2642-FE0F"},
    {"vyras_lotoso_pozoje_spalva1", "1F9D8-1F3FB-200D-2642-FE0F"},
    {"vyras_lotoso_pozoje_spalva2", "1F9D8-1F3FC-200D-2642-FE0F"},
    {"vyras_lotoso_pozoje_spalva3", "1F9D8-1F3FD-200D-2642-FE0F"},
    {"vyras_lotoso_pozoje_spalva4", "1F9D8-1F3FE-200D-2642-FE0F"},
    {"vyras_lotoso_pozoje_spalva5", "1F9D8-1F3FF-200D-2642-FE0F"},
    {"mage", "1F9D9-200D-2640-FE0F"},
    {"mage_spalva1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"mage_spalva2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"mage_spalva3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"mage_spalva4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"mage_spalva5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"vyras_magas", "1F9D9-200D-2642-FE0F"},
    {"vyras_magas_spalva1", "1F9D9-1F3FB-200D-2642-FE0F"},
    {"vyras_magas_spalva2", "1F9D9-1F3FC-200D-2642-FE0F"},
    {"vyras_magas_spalva3", "1F9D9-1F3FD-200D-2642-FE0F"},
    {"vyras_magas_spalva4", "1F9D9-1F3FE-200D-2642-FE0F"},
    {"vyras_magas_spalva5", "1F9D9-1F3FF-200D-2642-FE0F"},
    {"moteris_feja", "1F9DA-200D-2640-FE0F"},
    {"moteris_feja_spalva1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"moteris_feja_spalva2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"moteris_feja_spalva3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"moteris_feja_spalva4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"moteris_feja_spalva5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"burtininkas", "1F9DA-200D-2642-FE0F"},
    {"burtininkas_spalva1", "1F9DA-1F3FB-200D-2642-FE0F"},
    {"burtininkas_spalva2", "1F9DA-1F3FC-200D-2642-FE0F"},
    {"burtininkas_spalva3", "1F9DA-1F3FD-200D-2642-FE0F"},
    {"burtininkas_spalva4", "1F9DA-1F3FE-200D-2642-FE0F"},
    {"burtininkas_spalva5", "1F9DA-1F3FF-200D-2642-FE0F"},
    {"vampyre", "1F9DB-200D-2640-FE0F"},
    {"vampyre_spalva1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"vampyre_spalva2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"vampyre_spalva3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"vampyre_spalva4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"vampyre_spalva5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"vampyras_vyras", "1F9DB-200D-2642-FE0F"},
    {"vampyras_vyras_spalva1", "1F9DB-1F3FB-200D-2642-FE0F"},
    {"vampyras_vyras_spalva2", "1F9DB-1F3FC-200D-2642-FE0F"},
    {"vampyras_vyras_spalva3", "1F9DB-1F3FD-200D-2642-FE0F"},
    {"vampyras_vyras_spalva4", "1F9DB-1F3FE-200D-2642-FE0F"},
    {"vampyras_vyras_spalva5", "1F9DB-1F3FF-200D-2642-FE0F"},
    {"undinele", "1F9DC-200D-2640-FE0F"},
    {"undinele_spalva1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"undinele_spalva2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"undinele_spalva3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"undinele_spalva4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"undinele_spalva5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"vandenis_undinius", "1F9DC-200D-2642-FE0F"},
    {"vandenis_undinius_spalva1", "1F9DC-1F3FB-200D-2642-FE0F"},
    {"vandenis_undinius_spalva2", "1F9DC-1F3FC-200D-2642-FE0F"},
    {"vandenis_undinius_spalva3", "1F9DC-1F3FD-200D-2642-FE0F"},
    {"vandenis_undinius_spalva4", "1F9DC-1F3FE-200D-2642-FE0F"},
    {"vandenis_undinius_spalva5", "1F9DC-1F3FF-200D-2642-FE0F"},
    {"elfe", "1F9DD-200D-2640-FE0F"},
    {"elfe_spalva1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"elfe_spalva2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"elfe_spalva3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"elfe_spalva4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"elfe_spalva5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"elfas_vyras", "1F9DD-200D-2642-FE0F"},
    {"elfas_vyras_spalva1", "1F9DD-1F3FB-200D-2642-FE0F"},
    {"elfas_vyras_spalva2", "1F9DD-1F3FC-200D-2642-FE0F"},
    {"elfas_vyras_spalva3", "1F9DD-1F3FD-200D-2642-FE0F"},
    {"elfas_vyras_spalva4", "1F9DD-1F3FE-200D-2642-FE0F"},
    {"elfas_vyras_spalva5", "1F9DD-1F3FF-200D-2642-FE0F"},
    {"dzine", "1F9DE-200D-2640-FE0F"},
    {"dzinas", "1F9DE-200D-2642-FE0F"},
    {"zombe", "1F9DF-200D-2640-FE0F"},
    {"zombis_vyras", "1F9DF-200D-2642-FE0F"},
    {"vyras_raudoni_plaukai", "1F468-200D-1F9B0"},
    {"vyras_raudoni_plaukai_spalva1", "1F468-1F3FB-200D-1F9B0"},
    {"vyras_raudoni_plaukai_spalva2", "1F468-1F3FC-200D-1F9B0"},
    {"vyras_raudoni_plaukai_spalva3", "1F468-1F3FD-200D-1F9B0"},
    {"vyras_raudoni_plaukai_spalva4", "1F468-1F3FE-200D-1F9B0"},
    {"vyras_raudoni_plaukai_spalva5", "1F468-1F3FF-200D-1F9B0"},
    {"vyras_garbanoti_plaukai", "1F468-200D-1F9B1"},
    {"vyras_garbanoti_plaukai_spalva1", "1F468-1F3FB-200D-1F9B1"},
    {"vyras_garbanoti_plaukai_spalva2", "1F468-1F3FC-200D-1F9B1"},
    {"vyras_garbanoti_plaukai_spalva3", "1F468-1F3FD-200D-1F9B1"},
    {"vyras_garbanoti_plaukai_spalva4", "1F468-1F3FE-200D-1F9B1"},
    {"vyras_garbanoti_plaukai_spalva5", "1F468-1F3FF-200D-1F9B1"},
    {"vyras_plikas", "1F468-200D-1F9B2"},
    {"vyras_plikas_spalva1", "1F468-1F3FB-200D-1F9B2"},
    {"vyras_plikas_spalva2", "1F468-1F3FC-200D-1F9B2"},
    {"vyras_plikas_spalva3", "1F468-1F3FD-200D-1F9B2"},
    {"vyras_plikas_spalva4", "1F468-1F3FE-200D-1F9B2"},
    {"vyras_plikas_spalva5", "1F468-1F3FF-200D-1F9B2"},
    {"vyras_zili_plaukai", "1F468-200D-1F9B3"},
    {"vyras_zili_plaukai_spalva1", "1F468-1F3FB-200D-1F9B3"},
    {"vyras_zili_plaukai_spalva2", "1F468-1F3FC-200D-1F9B3"},
    {"vyras_zili_plaukai_spalva3", "1F468-1F3FD-200D-1F9B3"},
    {"vyras_zili_plaukai_spalva4", "1F468-1F3FE-200D-1F9B3"},
    {"vyras_zili_plaukai_spalva5", "1F468-1F3FF-200D-1F9B3"},
    {"moteris_raudoni_plaukai", "1F469-200D-1F9B0"},
    {"moteris_raudoni_plaukai_spalva1", "1F469-1F3FB-200D-1F9B0"},
    {"moteris_raudoni_plaukai_spalva2", "1F469-1F3FC-200D-1F9B0"},
    {"moteris_raudoni_plaukai_spalva3", "1F469-1F3FD-200D-1F9B0"},
    {"moteris_raudoni_plaukai_spalva4", "1F469-1F3FE-200D-1F9B0"},
    {"moteris_raudoni_plaukai_spalva5", "1F469-1F3FF-200D-1F9B0"},
    {"moteris_garbanoti_plaukai", "1F469-200D-1F9B1"},
    {"moteris_garbanoti_plaukai_spalva1", "1F469-1F3FB-200D-1F9B1"},
    {"moteris_garbanoti_plaukai_spalva2", "1F469-1F3FC-200D-1F9B1"},
    {"moteris_garbanoti_plaukai_spalva3", "1F469-1F3FD-200D-1F9B1"},
    {"moteris_garbanoti_plaukai_spalva4", "1F469-1F3FE-200D-1F9B1"},
    {"moteris_garbanoti_plaukai_spalva5", "1F469-1F3FF-200D-1F9B1"},
    {"moteris_plikas", "1F469-200D-1F9B2"},
    {"moteris_plikas_spalva1", "1F469-1F3FB-200D-1F9B2"},
    {"moteris_plikas_spalva2", "1F469-1F3FC-200D-1F9B2"},
    {"moteris_plikas_spalva3", "1F469-1F3FD-200D-1F9B2"},
    {"moteris_plikas_spalva4", "1F469-1F3FE-200D-1F9B2"},
    {"moteris_plikas_spalva5", "1F469-1F3FF-200D-1F9B2"},
    {"moteris_zili_plaukai", "1F469-200D-1F9B3"},
    {"moteris_zili_plaukai_spalva1", "1F469-1F3FB-200D-1F9B3"},
    {"moteris_zili_plaukai_spalva2", "1F469-1F3FC-200D-1F9B3"},
    {"moteris_zili_plaukai_spalva3", "1F469-1F3FD-200D-1F9B3"},
    {"moteris_zili_plaukai_spalva4", "1F469-1F3FE-200D-1F9B3"},
    {"moteris_zili_plaukai_spalva5", "1F469-1F3FF-200D-1F9B3"},
    {"suaugusysis_raudoni_plaukai", "1F9D1-200D-1F9B0"},
    {"suaugusysis_raudoni_plaukai_spalva1", "1F9D1-1F3FB-200D-1F9B0"},
    {"suaugusysis_raudoni_plaukai_spalva2", "1F9D1-1F3FC-200D-1F9B0"},
    {"suaugusysis_raudoni_plaukai_spalva3", "1F9D1-1F3FD-200D-1F9B0"},
    {"suaugusysis_raudoni_plaukai_spalva4", "1F9D1-1F3FE-200D-1F9B0"},
    {"suaugusysis_raudoni_plaukai_spalva5", "1F9D1-1F3FF-200D-1F9B0"},
    {"suaugusysis_garbanoti_plaukai", "1F9D1-200D-1F9B1"},
    {"suaugusysis_garbanoti_plaukai_spalva1", "1F9D1-1F3FB-200D-1F9B1"},
    {"suaugusysis_garbanoti_plaukai_spalva2", "1F9D1-1F3FC-200D-1F9B1"},
    {"suaugusysis_garbanoti_plaukai_spalva3", "1F9D1-1F3FD-200D-1F9B1"},
    {"suaugusysis_garbanoti_plaukai_spalva4", "1F9D1-1F3FE-200D-1F9B1"},
    {"suaugusysis_garbanoti_plaukai_spalva5", "1F9D1-1F3FF-200D-1F9B1"},
    {"suaugusysis_plikas", "1F9D1-200D-1F9B2"},
    {"suaugusysis_plikas_spalva1", "1F9D1-1F3FB-200D-1F9B2"},
    {"suaugusysis_plikas_spalva2", "1F9D1-1F3FC-200D-1F9B2"},
    {"suaugusysis_plikas_spalva3", "1F9D1-1F3FD-200D-1F9B2"},
    {"suaugusysis_plikas_spalva4", "1F9D1-1F3FE-200D-1F9B2"},
    {"suaugusysis_plikas_spalva5", "1F9D1-1F3FF-200D-1F9B2"},
    {"suaugusysis_zili_plaukai", "1F9D1-200D-1F9B3"},
    {"suaugusysis_zili_plaukai_spalva1", "1F9D1-1F3FB-200D-1F9B3"},
    {"suaugusysis_zili_plaukai_spalva2", "1F9D1-1F3FC-200D-1F9B3"},
    {"suaugusysis_zili_plaukai_spalva3", "1F9D1-1F3FD-200D-1F9B3"},
    {"suaugusysis_zili_plaukai_spalva4", "1F9D1-1F3FE-200D-1F9B3"},
    {"suaugusysis_zili_plaukai_spalva5", "1F9D1-1F3FF-200D-1F9B3"},
    {"nutraukta_grandine", "26D3-FE0F-200D-1F4A5"},
    {"deganti_sirdis", "2764-FE0F-200D-1F525"},
    {"gydytina_sirdis", "2764-FE0F-200D-1FA79"},
    {"baravykas", "1F344-200D-1F7EB"},
    {"zalioji_citrina", "1F34B-200D-1F7E9"},
    {"zydra_rozine_ir_balta_veliava", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"vaivorykstine_veliava", "1F3F3-FE0F-200D-1F308"},
    {"piratu_veliava", "1F3F4-200D-2620-FE0F"},
    {"juoda_kate", "1F408-200D-2B1B"},
    {"tarnaujantis_suo", "1F415-200D-1F9BA"},
    {"juodas_paukstis", "1F426-200D-2B1B"},
    {"feniksas", "1F426-200D-1F525"},
    {"baltoji_meska", "1F43B-200D-2744-FE0F"},
    {"akis_kalbejimo_burbule", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"veidas_su_garu", "1F62E-200D-1F4A8"},
    {"veidas_su_spiralinemis_akimis", "1F635-200D-1F4AB"},
    {"veidas_debesyse", "1F636-200D-1F32B-FE0F"},
    {"purtantis_galva", "1F642-200D-2194-FE0F"},
    {"linksintis_galva", "1F642-200D-2195-FE0F"},
    {"baleto_sokejas", "1F9D1-200D-1FA70"},
    {"baleto_sokejas_spalva1", "1F9D1-1F3FB-200D-1FA70"},
    {"baleto_sokejas_spalva2", "1F9D1-1F3FC-200D-1FA70"},
    {"baleto_sokejas_spalva3", "1F9D1-1F3FD-200D-1FA70"},
    {"baleto_sokejas_spalva4", "1F9D1-1F3FE-200D-1FA70"},
    {"baleto_sokejas_spalva5", "1F9D1-1F3FF-200D-1FA70"},
    {"šauktukas_ir_klaustukas", "2049"},
    {"prekės_ženklas", "2122"},
    {"rodyklė_į_kairę_ir_į_dešinę", "2194"},
    {"rodyklė_į_viršų_ir_į_apačią", "2195"},
    {"rodyklė_į_viršų_ir_į_kairę", "2196"},
    {"rodyklė_į_viršų_ir_į_dešinę", "2197"},
    {"rodyklė_į_apačią_ir_į_dešinę", "2198"},
    {"rodyklė_į_apačią_ir_į_kairę", "2199"},
    {"klaviatūra", "2328"},
    {"saulė", "2600"},
    {"skėtis", "2602"},
    {"skėtis_su_lietaus_lašais", "2614"},
    {"karštas_gėrimas", "2615"},
    {"kaukolė_ir_sukryžiuoti_kaulai", "2620"},
    {"ortodoksų_kryžius", "2626"},
    {"paniuręs_veidas", "2639"},
    {"moteriškos_lyties_ženklas", "2640"},
    {"vyriškos_lyties_ženklas", "2642"},
    {"šaulys", "2650"},
    {"ožiaragis", "2651"},
    {"vandenio_ženklas", "2652"},
    {"žuvys", "2653"},
    {"kryžiai", "2663"},
    {"širdys", "2665"},
    {"būgnai", "2666"},
    {"karštosios_versmės", "2668"},
    {"sukryžiuoti_kardai", "2694"},
    {"svarstyklės_su_lėkštėmis", "2696"},
    {"žirklės", "2702"},
    {"balta_paryškinta_varnelė", "2705"},
    {"lėktuvas", "2708"},
    {"paryškinta_varnelė", "2714"},
    {"paryškintas_daugybos_ženklas", "2716"},
    {"dovydo_žvaigždė", "2721"},
    {"kibirkštėlės", "2728"},
    {"aštuoniakampė_žvaigždutė", "2733"},
    {"aštuoniakampė_žvaigždė", "2734"},
    {"snaigė", "2744"},
    {"kibirkštėlė", "2747"},
    {"baltas_šauktukas", "2755"},
    {"raudonas_šauktukas", "2757"},
    {"storas_širdelės_formos_šauktukas", "2763"},
    {"raudona_širdelė", "2764"},
    {"paryškintas_pliusas", "2795"},
    {"paryškintas_minusas", "2796"},
    {"paryškintas_dalybos_ženklas", "2797"},
    {"rodyklė_į_dešinę_užlenkta_į_viršų", "2934"},
    {"rodyklė_į_dešinę_užlenkta_į_apačią", "2935"},
    {"vingiuotas_brūkšnys", "3030"},
    {"pasveikinimą_reiškiantis_hieroglifas", "3297"},
    {"paslaptį_reiškiantis_hieroglifas", "3299"},
    {"autorių_teisės", "00A9"},
    {"užregistruota", "00AE"},
    {"du_šauktukai", "203C"},
    {"rodyklė_į_dešinę_užlenkta_į_kairę", "21A9"},
    {"rodyklė_į_kairę_užlenkta_į_dešinę", "21AA"},
    {"smėlio_laikrodis", "231B"},
    {"išmetimo_mygtukas", "23CF"},
    {"greitai_į_viršų", "23EB"},
    {"greitai_į_apačią", "23EC"},
    {"žadintuvas", "23F0"},
    {"smėlio_laikrodis_su_krentančiu_smėliu", "23F3"},
    {"įrašymo_mygtukas", "23FA"},
    {"raidė_m", "24C2"},
    {"juodas_mažas_kvadratas", "25AA"},
    {"baltas_mažas_kvadratas", "25AB"},
    {"atvirkštinė_tvarka", "25C0"},
    {"baltas_vidutiniškai_mažas_kvadratas", "25FD"},
    {"juodas_vidutiniškai_mažas_kvadratas", "25FE"},
    {"delnu_atsukta_aukštyn_pirštu_rodanti_ranka", "261D"},
    {"delnu_atsukta_aukštyn_pirštu_rodanti_ranka_spalva1", "261D-1F3FB"},
    {"delnu_atsukta_aukštyn_pirštu_rodanti_ranka_spalva2", "261D-1F3FC"},
    {"delnu_atsukta_aukštyn_pirštu_rodanti_ranka_spalva3", "261D-1F3FD"},
    {"delnu_atsukta_aukštyn_pirštu_rodanti_ranka_spalva4", "261D-1F3FE"},
    {"delnu_atsukta_aukštyn_pirštu_rodanti_ranka_spalva5", "261D-1F3FF"},
    {"žvaigždė_ir_pusmėnulis", "262A"},
    {"besišypsantis_veidas", "263A"},
    {"vėžys", "264B"},
    {"liūtas", "264C"},
    {"mergelė", "264D"},
    {"svarstyklės", "264E"},
    {"šachmatų_pėstininkas", "265F"},
    {"begalybė", "267E"},
    {"neįgaliojo_vežimėlio_simbolis", "267F"},
    {"įspėjimas", "26A0"},
    {"aukštoji_įtampa", "26A1"},
    {"translyčių_simbolis", "26A7"},
    {"saulė_už_debesų", "26C5"},
    {"debesis_su_žaibu_ir_lietumi", "26C8"},
    {"gyvatnešis", "26CE"},
    {"šalmas_su_baltu_kryžiumi", "26D1"},
    {"grandinės", "26D3"},
    {"įėjimo_nėra", "26D4"},
    {"šintoizmo_šventykla", "26E9"},
    {"bažnyčia", "26EA"},
    {"skėtis_ant_žemės", "26F1"},
    {"vėliava_skylutėje", "26F3"},
    {"pačiūžos", "26F8"},
    {"žmogus_su_kamuoliu", "26F9"},
    {"žmogus_su_kamuoliu_spalva1", "26F9-1F3FB"},
    {"žmogus_su_kamuoliu_spalva2", "26F9-1F3FC"},
    {"žmogus_su_kamuoliu_spalva3", "26F9-1F3FD"},
    {"žmogus_su_kamuoliu_spalva4", "26F9-1F3FE"},
    {"žmogus_su_kamuoliu_spalva5", "26F9-1F3FF"},
    {"palapinė", "26FA"},
    {"degalų_siurblys", "26FD"},
    {"iškeltas_kumštis", "270A"},
    {"iškeltas_kumštis_spalva1", "270A-1F3FB"},
    {"iškeltas_kumštis_spalva2", "270A-1F3FC"},
    {"iškeltas_kumštis_spalva3", "270A-1F3FD"},
    {"iškeltas_kumštis_spalva4", "270A-1F3FE"},
    {"iškeltas_kumštis_spalva5", "270A-1F3FF"},
    {"iškelta_ranka", "270B"},
    {"iškelta_ranka_spalva1", "270B-1F3FB"},
    {"iškelta_ranka_spalva2", "270B-1F3FC"},
    {"iškelta_ranka_spalva3", "270B-1F3FD"},
    {"iškelta_ranka_spalva4", "270B-1F3FE"},
    {"iškelta_ranka_spalva5", "270B-1F3FF"},
    {"pergalės_ženklą_rodanti_ranka", "270C"},
    {"pergalės_ženklą_rodanti_ranka_spalva1", "270C-1F3FB"},
    {"pergalės_ženklą_rodanti_ranka_spalva2", "270C-1F3FC"},
    {"pergalės_ženklą_rodanti_ranka_spalva3", "270C-1F3FD"},
    {"pergalės_ženklą_rodanti_ranka_spalva4", "270C-1F3FE"},
    {"pergalės_ženklą_rodanti_ranka_spalva5", "270C-1F3FF"},
    {"rašanti_ranka", "270D"},
    {"rašanti_ranka_spalva1", "270D-1F3FB"},
    {"rašanti_ranka_spalva2", "270D-1F3FC"},
    {"rašanti_ranka_spalva3", "270D-1F3FD"},
    {"rašanti_ranka_spalva4", "270D-1F3FE"},
    {"rašanti_ranka_spalva5", "270D-1F3FF"},
    {"pieštukas", "270F"},
    {"romos_katalikų_kryžius", "271D"},
    {"kryžiukas", "274C"},
    {"kryžiuko_mygtukas", "274E"},
    {"rodyklė_į_dešinę", "27A1"},
    {"kilpelė", "27B0"},
    {"dviguba_kilpelė", "27BF"},
    {"rodyklė_į_kairę", "2B05"},
    {"rodyklė_į_viršų", "2B06"},
    {"rodyklė_į_apačią", "2B07"},
    {"balta_vidutinė_žvaigždė", "2B50"},
    {"ryškus_didelis_ratas", "2B55"},
    {"dalių_apkeitimo_ženklas", "303D"},
    {"madžongo_raudonas_drakonas", "1F004"},
    {"čia_reiškiantis_hieroglifas", "1F201"},
    {"paslaugos_mokestį_reiškiantis_hieroglifas", "1F202"},
    {"nemokamumą_reiškiantis_hieroglifas", "1F21A"},
    {"rezervavimą_reiškiantis_hieroglifas", "1F22F"},
    {"draudimą_reiškiantis_hieroglifas", "1F232"},
    {"tuštumą_reiškiantis_hieroglifas", "1F233"},
    {"bendrumą_reiškiantis_hieroglifas", "1F234"},
    {"pilnatvę_reiškiantis_hieroglifas", "1F235"},
    {"ne_nemokamą_reiškiantis_hieroglifas", "1F236"},
    {"mėnesio_sumą_reiškiantis_hieroglifas", "1F237"},
    {"taikymą_reiškiantis_hieroglifas", "1F238"},
    {"nuolaidą_reiškiantis_hieroglifas", "1F239"},
    {"veikimą_reiškiantis_hieroglifas", "1F23A"},
    {"susitarimą_reiškiantis_hieroglifas", "1F250"},
    {"sutikimą_reiškiantis_hieroglifas", "1F251"},
    {"rūkas", "1F301"},
    {"suskleistas_skėtis", "1F302"},
    {"žvaigždėta_naktis", "1F303"},
    {"saulėtekis_virš_kalnų", "1F304"},
    {"saulėtekis", "1F305"},
    {"sutemos_virš_miesto", "1F306"},
    {"saulėlydis", "1F307"},
    {"vaivorykštė", "1F308"},
    {"tiltas_naktį", "1F309"},
    {"paukščių_takas", "1F30C"},
    {"žemės_rutulys_su_europos_ir_afrikos_žemynais", "1F30D"},
    {"žemės_rutulys_su_šiaurės_ir_pietų_amerikos_žemynais", "1F30E"},
    {"žemės_rutulys_su_azijos_ir_australijos_žemynais", "1F30F"},
    {"žemės_rutulys_su_dienovidiniais", "1F310"},
    {"jaunas_mėnulis", "1F311"},
    {"priešpilnis_mėnulis", "1F312"},
    {"pirmas_fazės_ketvirtis", "1F313"},
    {"priešpilnis_pabaiga", "1F314"},
    {"delčia", "1F316"},
    {"paskutinis_fazės_ketvirtis", "1F317"},
    {"delčia_pabaiga", "1F318"},
    {"pusmėnulis", "1F319"},
    {"jaunas_mėnulis_su_veidu", "1F31A"},
    {"priešpilnio_mėnulis_su_veidu", "1F31B"},
    {"delčios_mėnulis_su_veidu", "1F31C"},
    {"saulė_su_veidu", "1F31E"},
    {"žėrinti_žvaigždė", "1F31F"},
    {"krintanti_žvaigždė", "1F320"},
    {"saulė_už_nedidelio_debesies", "1F324"},
    {"saulė_už_didelio_debesies", "1F325"},
    {"saulė_už_debesies_iš_kurio_lyja", "1F326"},
    {"debesis_iš_kurio_lyja", "1F327"},
    {"debesis_iš_kurio_sninga", "1F328"},
    {"debesis_su_žaibu", "1F329"},
    {"ūkana", "1F32B"},
    {"vėjo_veidas", "1F32C"},
    {"dešrainis", "1F32D"},
    {"meksikietiškas_paplotėlis", "1F32E"},
    {"lazdyno_riešutas", "1F330"},
    {"amžinai_žaliuojantis_medis", "1F332"},
    {"palmė", "1F334"},
    {"aštrusis_pipiras", "1F336"},
    {"tulpė", "1F337"},
    {"vyšnios_žiedas", "1F338"},
    {"rožė", "1F339"},
    {"kinrožė", "1F33A"},
    {"saulėgrąža", "1F33B"},
    {"gėlės_žiedas", "1F33C"},
    {"kukurūzo_burbuolė", "1F33D"},
    {"ryžių_pėdas", "1F33E"},
    {"žolelė", "1F33F"},
    {"nukritęs_lapas", "1F342"},
    {"lapas_vėjyje", "1F343"},
    {"baklažanas", "1F346"},
    {"vynuogės", "1F347"},
    {"arbūzas", "1F349"},
    {"žalias_obuolys", "1F34F"},
    {"kriaušė", "1F350"},
    {"vyšnios", "1F352"},
    {"braškė", "1F353"},
    {"mėsainis", "1F354"},
    {"mėsa_ant_kaulo", "1F356"},
    {"paukščio_šlaunelė", "1F357"},
    {"ryžių_paplotėlis", "1F358"},
    {"ryžių_rutuliukas", "1F359"},
    {"virti_ryžiai", "1F35A"},
    {"ryžiai_su_kariu", "1F35B"},
    {"makaronų_sriubos_dubenėlis", "1F35C"},
    {"spagečiai", "1F35D"},
    {"gruzdintos_bulvytės", "1F35F"},
    {"kepta_saldžioji_bulvė", "1F360"},
    {"sušis", "1F363"},
    {"kepta_krevetė", "1F364"},
    {"žuvies_pyragėlis_su_išpieštu_sūkurio_simboliu", "1F365"},
    {"minkšti_ledai", "1F366"},
    {"šokolado_batonėlis", "1F36B"},
    {"medaus_puodynė", "1F36F"},
    {"bento_dėžutė", "1F371"},
    {"šakutė_ir_peilis", "1F374"},
    {"sakė", "1F376"},
    {"vyno_taurė", "1F377"},
    {"kokteilio_taurė", "1F378"},
    {"tropinis_gėrimas", "1F379"},
    {"susidaužiantys_alaus_bokalai", "1F37B"},
    {"kūdikio_buteliukas", "1F37C"},
    {"šakutė_ir_peilis_su_lėkšte", "1F37D"},
    {"butelis_su_iššokusiu_kamščiu", "1F37E"},
    {"spragėsiai", "1F37F"},
    {"žibintas_iš_moliūgo", "1F383"},
    {"kalėdų_eglutė", "1F384"},
    {"kalėdų_senelis", "1F385"},
    {"kalėdų_senelis_spalva1", "1F385-1F3FB"},
    {"kalėdų_senelis_spalva2", "1F385-1F3FC"},
    {"kalėdų_senelis_spalva3", "1F385-1F3FD"},
    {"kalėdų_senelis_spalva4", "1F385-1F3FE"},
    {"kalėdų_senelis_spalva5", "1F385-1F3FF"},
    {"šaltoji_ugnelė", "1F387"},
    {"konfeti_šaudyklė", "1F389"},
    {"sukryžiuotos_vėliavos", "1F38C"},
    {"dekoracija_iš_pušies", "1F38D"},
    {"japoniškos_lėlės", "1F38E"},
    {"karpio_vėtrungė", "1F38F"},
    {"vėjo_varpeliai", "1F390"},
    {"mėnesienos_ceremonija", "1F391"},
    {"mokyklinė_kuprinė", "1F392"},
    {"mokyklos_baigimo_kepurėlė", "1F393"},
    {"kariuomenės_medalis", "1F396"},
    {"įėjimo_bilietai", "1F39F"},
    {"karuselės_arkliukas", "1F3A0"},
    {"apžvalgos_ratas", "1F3A1"},
    {"meškerė", "1F3A3"},
    {"ausinės", "1F3A7"},
    {"dailininko_paletė", "1F3A8"},
    {"cirko_palapinė", "1F3AA"},
    {"pliauškė", "1F3AC"},
    {"vaizdo_žaidimas", "1F3AE"},
    {"smūgis_tiesiai_į_taikinį", "1F3AF"},
    {"žaidimų_automatas", "1F3B0"},
    {"kortos_su_gėlėmis", "1F3B4"},
    {"muzikos_klaviatūra", "1F3B9"},
    {"partitūra", "1F3BC"},
    {"bėgimo_marškinėliai", "1F3BD"},
    {"slidės", "1F3BF"},
    {"krepšinio_kamuolys", "1F3C0"},
    {"vėliava_langeliais", "1F3C1"},
    {"snieglenčių_sporto_mėgėjas", "1F3C2"},
    {"snieglenčių_sporto_mėgėjas_spalva1", "1F3C2-1F3FB"},
    {"snieglenčių_sporto_mėgėjas_spalva2", "1F3C2-1F3FC"},
    {"snieglenčių_sporto_mėgėjas_spalva3", "1F3C2-1F3FD"},
    {"snieglenčių_sporto_mėgėjas_spalva4", "1F3C2-1F3FE"},
    {"snieglenčių_sporto_mėgėjas_spalva5", "1F3C2-1F3FF"},
    {"bėgikas", "1F3C3"},
    {"bėgikas_spalva1", "1F3C3-1F3FB"},
    {"bėgikas_spalva2", "1F3C3-1F3FC"},
    {"bėgikas_spalva3", "1F3C3-1F3FD"},
    {"bėgikas_spalva4", "1F3C3-1F3FE"},
    {"bėgikas_spalva5", "1F3C3-1F3FF"},
    {"trofėjus", "1F3C6"},
    {"arklių_lenktynės", "1F3C7"},
    {"arklių_lenktynės_spalva1", "1F3C7-1F3FB"},
    {"arklių_lenktynės_spalva2", "1F3C7-1F3FC"},
    {"arklių_lenktynės_spalva3", "1F3C7-1F3FD"},
    {"arklių_lenktynės_spalva4", "1F3C7-1F3FE"},
    {"arklių_lenktynės_spalva5", "1F3C7-1F3FF"},
    {"amerikietiškojo_futbolo_kamuolys", "1F3C8"},
    {"svorių_kilnotojas", "1F3CB"},
    {"svorių_kilnotojas_spalva1", "1F3CB-1F3FB"},
    {"svorių_kilnotojas_spalva2", "1F3CB-1F3FC"},
    {"svorių_kilnotojas_spalva3", "1F3CB-1F3FD"},
    {"svorių_kilnotojas_spalva4", "1F3CB-1F3FE"},
    {"svorių_kilnotojas_spalva5", "1F3CB-1F3FF"},
    {"golfo_žaidėjas", "1F3CC"},
    {"golfo_žaidėjas_spalva1", "1F3CC-1F3FB"},
    {"golfo_žaidėjas_spalva2", "1F3CC-1F3FC"},
    {"golfo_žaidėjas_spalva3", "1F3CC-1F3FD"},
    {"golfo_žaidėjas_spalva4", "1F3CC-1F3FE"},
    {"golfo_žaidėjas_spalva5", "1F3CC-1F3FF"},
    {"žolės_riedulys", "1F3D1"},
    {"kalnas_su_snieguota_viršūne", "1F3D4"},
    {"paplūdimys_ir_skėtis", "1F3D6"},
    {"biurų_pastatas", "1F3E2"},
    {"japoniškas_paštas", "1F3E3"},
    {"pašto_pastatas", "1F3E4"},
    {"ligoninė", "1F3E5"},
    {"bankomato_ženklas", "1F3E7"},
    {"viešbutis", "1F3E8"},
    {"meilės_viešbutis", "1F3E9"},
    {"visą_parą_dirbanti_parduotuvė", "1F3EA"},
    {"raudonas_popierinis_žibintas", "1F3EE"},
    {"japoniška_pilis", "1F3EF"},
    {"balta_vėliava", "1F3F3"},
    {"juoda_vėliava", "1F3F4"},
    {"rozetė", "1F3F5"},
    {"etiketė", "1F3F7"},
    {"lankas_ir_strėlė", "1F3F9"},
    {"senovinė_vaza", "1F3FA"},
    {"šviesios_odos", "1F3FB"},
    {"žiurkė", "1F400"},
    {"pelė", "1F401"},
    {"karvė", "1F404"},
    {"triušis", "1F407"},
    {"katė", "1F408"},
    {"sraigė", "1F40C"},
    {"gyvatė", "1F40D"},
    {"avinas_gyvūnas", "1F40F"},
    {"ožys", "1F410"},
    {"beždžionė", "1F412"},
    {"višta", "1F414"},
    {"šuo", "1F415"},
    {"kiaulė", "1F416"},
    {"šernas", "1F417"},
    {"aštuonkojis", "1F419"},
    {"susisukusi_kriauklė", "1F41A"},
    {"skruzdėlė", "1F41C"},
    {"bitė", "1F41D"},
    {"boružė", "1F41E"},
    {"žuvis", "1F41F"},
    {"tropinė_žuvis", "1F420"},
    {"vėžlys", "1F422"},
    {"išsiritęs_viščiukas", "1F423"},
    {"viščiukas", "1F424"},
    {"atsisukęs_snapu_viščiukas", "1F425"},
    {"paukštis", "1F426"},
    {"pelės_snukutis", "1F42D"},
    {"karvės_snukis", "1F42E"},
    {"triušio_snukutis", "1F430"},
    {"katės_snukutis", "1F431"},
    {"purškiantis_banginis", "1F433"},
    {"beždžionės_snukutis", "1F435"},
    {"šuns_snukutis", "1F436"},
    {"kiaulės_snukis", "1F437"},
    {"varlės_snukutis", "1F438"},
    {"žiurkėno_snukutis", "1F439"},
    {"meškos_snukis", "1F43B"},
    {"kiaulės_nosis", "1F43D"},
    {"pėdutės", "1F43E"},
    {"voverė", "1F43F"},
    {"liežuvis", "1F445"},
    {"delnu_nusukta_aukštyn_pirštu_rodanti_ranka", "1F446"},
    {"delnu_nusukta_aukštyn_pirštu_rodanti_ranka_spalva1", "1F446-1F3FB"},
    {"delnu_nusukta_aukštyn_pirštu_rodanti_ranka_spalva2", "1F446-1F3FC"},
    {"delnu_nusukta_aukštyn_pirštu_rodanti_ranka_spalva3", "1F446-1F3FD"},
    {"delnu_nusukta_aukštyn_pirštu_rodanti_ranka_spalva4", "1F446-1F3FE"},
    {"delnu_nusukta_aukštyn_pirštu_rodanti_ranka_spalva5", "1F446-1F3FF"},
    {"delnu_nusukta_žemyn_pirštu_rodanti_ranka", "1F447"},
    {"delnu_nusukta_žemyn_pirštu_rodanti_ranka_spalva1", "1F447-1F3FB"},
    {"delnu_nusukta_žemyn_pirštu_rodanti_ranka_spalva2", "1F447-1F3FC"},
    {"delnu_nusukta_žemyn_pirštu_rodanti_ranka_spalva3", "1F447-1F3FD"},
    {"delnu_nusukta_žemyn_pirštu_rodanti_ranka_spalva4", "1F447-1F3FE"},
    {"delnu_nusukta_žemyn_pirštu_rodanti_ranka_spalva5", "1F447-1F3FF"},
    {"į_kairę_pirštu_rodanti_ranka", "1F448"},
    {"į_kairę_pirštu_rodanti_ranka_spalva1", "1F448-1F3FB"},
    {"į_kairę_pirštu_rodanti_ranka_spalva2", "1F448-1F3FC"},
    {"į_kairę_pirštu_rodanti_ranka_spalva3", "1F448-1F3FD"},
    {"į_kairę_pirštu_rodanti_ranka_spalva4", "1F448-1F3FE"},
    {"į_kairę_pirštu_rodanti_ranka_spalva5", "1F448-1F3FF"},
    {"į_dešinę_pirštu_rodanti_ranka", "1F449"},
    {"į_dešinę_pirštu_rodanti_ranka_spalva1", "1F449-1F3FB"},
    {"į_dešinę_pirštu_rodanti_ranka_spalva2", "1F449-1F3FC"},
    {"į_dešinę_pirštu_rodanti_ranka_spalva3", "1F449-1F3FD"},
    {"į_dešinę_pirštu_rodanti_ranka_spalva4", "1F449-1F3FE"},
    {"į_dešinę_pirštu_rodanti_ranka_spalva5", "1F449-1F3FF"},
    {"artėjantis_kumštis", "1F44A"},
    {"artėjantis_kumštis_spalva1", "1F44A-1F3FB"},
    {"artėjantis_kumštis_spalva2", "1F44A-1F3FC"},
    {"artėjantis_kumštis_spalva3", "1F44A-1F3FD"},
    {"artėjantis_kumštis_spalva4", "1F44A-1F3FE"},
    {"artėjantis_kumštis_spalva5", "1F44A-1F3FF"},
    {"gerai_ženklas_rankomis", "1F44C"},
    {"gerai_ženklas_rankomis_spalva1", "1F44C-1F3FB"},
    {"gerai_ženklas_rankomis_spalva2", "1F44C-1F3FC"},
    {"gerai_ženklas_rankomis_spalva3", "1F44C-1F3FD"},
    {"gerai_ženklas_rankomis_spalva4", "1F44C-1F3FE"},
    {"gerai_ženklas_rankomis_spalva5", "1F44C-1F3FF"},
    {"nykštys_į_viršų", "1F44D"},
    {"nykštys_į_viršų_spalva1", "1F44D-1F3FB"},
    {"nykštys_į_viršų_spalva2", "1F44D-1F3FC"},
    {"nykštys_į_viršų_spalva3", "1F44D-1F3FD"},
    {"nykštys_į_viršų_spalva4", "1F44D-1F3FE"},
    {"nykštys_į_viršų_spalva5", "1F44D-1F3FF"},
    {"nykštys_į_apačią", "1F44E"},
    {"nykštys_į_apačią_spalva1", "1F44E-1F3FB"},
    {"nykštys_į_apačią_spalva2", "1F44E-1F3FC"},
    {"nykštys_į_apačią_spalva3", "1F44E-1F3FD"},
    {"nykštys_į_apačią_spalva4", "1F44E-1F3FE"},
    {"nykštys_į_apačią_spalva5", "1F44E-1F3FF"},
    {"plojančios_rankos", "1F44F"},
    {"plojančios_rankos_spalva1", "1F44F-1F3FB"},
    {"plojančios_rankos_spalva2", "1F44F-1F3FC"},
    {"plojančios_rankos_spalva3", "1F44F-1F3FD"},
    {"plojančios_rankos_spalva4", "1F44F-1F3FE"},
    {"plojančios_rankos_spalva5", "1F44F-1F3FF"},
    {"karūna", "1F451"},
    {"moteriška_skrybėlė", "1F452"},
    {"kaklaraištis", "1F454"},
    {"marškinėliai", "1F455"},
    {"džinsai", "1F456"},
    {"suknelė", "1F457"},
    {"moteriški_drabužiai", "1F45A"},
    {"piniginė", "1F45B"},
    {"rankinė", "1F45C"},
    {"maišelis", "1F45D"},
    {"vyriškas_batas", "1F45E"},
    {"bėgimo_batelis", "1F45F"},
    {"aukštakulnis_batelis", "1F460"},
    {"moteriška_basutė", "1F461"},
    {"moteriškas_batas", "1F462"},
    {"pėdsakai", "1F463"},
    {"biustų_siluetai", "1F465"},
    {"mergaitė", "1F467"},
    {"mergaitė_spalva1", "1F467-1F3FB"},
    {"mergaitė_spalva2", "1F467-1F3FC"},
    {"mergaitė_spalva3", "1F467-1F3FD"},
    {"mergaitė_spalva4", "1F467-1F3FE"},
    {"mergaitė_spalva5", "1F467-1F3FF"},
    {"šeima", "1F46A"},
    {"vyras_ir_moteris_susikibę_už_rankų", "1F46B"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva1", "1F46B-1F3FB"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva2", "1F46B-1F3FC"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva3", "1F46B-1F3FD"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva4", "1F46B-1F3FE"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva5", "1F46B-1F3FF"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva1-2", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva1-3", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva1-4", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva1-5", "1F469-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva2-1", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva2-3", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva2-4", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva2-5", "1F469-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva3-1", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva3-2", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva3-4", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva3-5", "1F469-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva4-1", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva4-2", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva4-3", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva4-5", "1F469-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva5-1", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva5-2", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva5-3", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"vyras_ir_moteris_susikibę_už_rankų_spalva5-4", "1F469-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"du_vyrai_susikibę_už_rankų", "1F46C"},
    {"du_vyrai_susikibę_už_rankų_spalva1", "1F46C-1F3FB"},
    {"du_vyrai_susikibę_už_rankų_spalva2", "1F46C-1F3FC"},
    {"du_vyrai_susikibę_už_rankų_spalva3", "1F46C-1F3FD"},
    {"du_vyrai_susikibę_už_rankų_spalva4", "1F46C-1F3FE"},
    {"du_vyrai_susikibę_už_rankų_spalva5", "1F46C-1F3FF"},
    {"du_vyrai_susikibę_už_rankų_spalva1-2", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FC"},
    {"du_vyrai_susikibę_už_rankų_spalva1-3", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FD"},
    {"du_vyrai_susikibę_už_rankų_spalva1-4", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FE"},
    {"du_vyrai_susikibę_už_rankų_spalva1-5", "1F468-1F3FB-200D-1F91D-200D-1F468-1F3FF"},
    {"du_vyrai_susikibę_už_rankų_spalva2-1", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FB"},
    {"du_vyrai_susikibę_už_rankų_spalva2-3", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FD"},
    {"du_vyrai_susikibę_už_rankų_spalva2-4", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FE"},
    {"du_vyrai_susikibę_už_rankų_spalva2-5", "1F468-1F3FC-200D-1F91D-200D-1F468-1F3FF"},
    {"du_vyrai_susikibę_už_rankų_spalva3-1", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FB"},
    {"du_vyrai_susikibę_už_rankų_spalva3-2", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FC"},
    {"du_vyrai_susikibę_už_rankų_spalva3-4", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FE"},
    {"du_vyrai_susikibę_už_rankų_spalva3-5", "1F468-1F3FD-200D-1F91D-200D-1F468-1F3FF"},
    {"du_vyrai_susikibę_už_rankų_spalva4-1", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FB"},
    {"du_vyrai_susikibę_už_rankų_spalva4-2", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FC"},
    {"du_vyrai_susikibę_už_rankų_spalva4-3", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FD"},
    {"du_vyrai_susikibę_už_rankų_spalva4-5", "1F468-1F3FE-200D-1F91D-200D-1F468-1F3FF"},
    {"du_vyrai_susikibę_už_rankų_spalva5-1", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FB"},
    {"du_vyrai_susikibę_už_rankų_spalva5-2", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FC"},
    {"du_vyrai_susikibę_už_rankų_spalva5-3", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FD"},
    {"du_vyrai_susikibę_už_rankų_spalva5-4", "1F468-1F3FF-200D-1F91D-200D-1F468-1F3FE"},
    {"dvi_moterys_susikibusios_už_rankų", "1F46D"},
    {"dvi_moterys_susikibusios_už_rankų_spalva1", "1F46D-1F3FB"},
    {"dvi_moterys_susikibusios_už_rankų_spalva2", "1F46D-1F3FC"},
    {"dvi_moterys_susikibusios_už_rankų_spalva3", "1F46D-1F3FD"},
    {"dvi_moterys_susikibusios_už_rankų_spalva4", "1F46D-1F3FE"},
    {"dvi_moterys_susikibusios_už_rankų_spalva5", "1F46D-1F3FF"},
    {"dvi_moterys_susikibusios_už_rankų_spalva1-2", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FC"},
    {"dvi_moterys_susikibusios_už_rankų_spalva1-3", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FD"},
    {"dvi_moterys_susikibusios_už_rankų_spalva1-4", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FE"},
    {"dvi_moterys_susikibusios_už_rankų_spalva1-5", "1F469-1F3FB-200D-1F91D-200D-1F469-1F3FF"},
    {"dvi_moterys_susikibusios_už_rankų_spalva2-1", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FB"},
    {"dvi_moterys_susikibusios_už_rankų_spalva2-3", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FD"},
    {"dvi_moterys_susikibusios_už_rankų_spalva2-4", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FE"},
    {"dvi_moterys_susikibusios_už_rankų_spalva2-5", "1F469-1F3FC-200D-1F91D-200D-1F469-1F3FF"},
    {"dvi_moterys_susikibusios_už_rankų_spalva3-1", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FB"},
    {"dvi_moterys_susikibusios_už_rankų_spalva3-2", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FC"},
    {"dvi_moterys_susikibusios_už_rankų_spalva3-4", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FE"},
    {"dvi_moterys_susikibusios_už_rankų_spalva3-5", "1F469-1F3FD-200D-1F91D-200D-1F469-1F3FF"},
    {"dvi_moterys_susikibusios_už_rankų_spalva4-1", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FB"},
    {"dvi_moterys_susikibusios_už_rankų_spalva4-2", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FC"},
    {"dvi_moterys_susikibusios_už_rankų_spalva4-3", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FD"},
    {"dvi_moterys_susikibusios_už_rankų_spalva4-5", "1F469-1F3FE-200D-1F91D-200D-1F469-1F3FF"},
    {"dvi_moterys_susikibusios_už_rankų_spalva5-1", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FB"},
    {"dvi_moterys_susikibusios_už_rankų_spalva5-2", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FC"},
    {"dvi_moterys_susikibusios_už_rankų_spalva5-3", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FD"},
    {"dvi_moterys_susikibusios_už_rankų_spalva5-4", "1F469-1F3FF-200D-1F91D-200D-1F469-1F3FE"},
    {"besilinksminančios_moterys", "1F46F"},
    {"besilinksminančios_moterys_spalva1", "1F46F-1F3FB"},
    {"besilinksminančios_moterys_spalva2", "1F46F-1F3FC"},
    {"besilinksminančios_moterys_spalva3", "1F46F-1F3FD"},
    {"besilinksminančios_moterys_spalva4", "1F46F-1F3FE"},
    {"besilinksminančios_moterys_spalva5", "1F46F-1F3FF"},
    {"besilinksminančios_moterys_spalva1-2", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FC"},
    {"besilinksminančios_moterys_spalva1-3", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FD"},
    {"besilinksminančios_moterys_spalva1-4", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FE"},
    {"besilinksminančios_moterys_spalva1-5", "1F9D1-1F3FB-200D-1F430-200D-1F9D1-1F3FF"},
    {"besilinksminančios_moterys_spalva2-1", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FB"},
    {"besilinksminančios_moterys_spalva2-3", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FD"},
    {"besilinksminančios_moterys_spalva2-4", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FE"},
    {"besilinksminančios_moterys_spalva2-5", "1F9D1-1F3FC-200D-1F430-200D-1F9D1-1F3FF"},
    {"besilinksminančios_moterys_spalva3-1", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FB"},
    {"besilinksminančios_moterys_spalva3-2", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FC"},
    {"besilinksminančios_moterys_spalva3-4", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FE"},
    {"besilinksminančios_moterys_spalva3-5", "1F9D1-1F3FD-200D-1F430-200D-1F9D1-1F3FF"},
    {"besilinksminančios_moterys_spalva4-1", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FB"},
    {"besilinksminančios_moterys_spalva4-2", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FC"},
    {"besilinksminančios_moterys_spalva4-3", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FD"},
    {"besilinksminančios_moterys_spalva4-5", "1F9D1-1F3FE-200D-1F430-200D-1F9D1-1F3FF"},
    {"besilinksminančios_moterys_spalva5-1", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FB"},
    {"besilinksminančios_moterys_spalva5-2", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FC"},
    {"besilinksminančios_moterys_spalva5-3", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FD"},
    {"besilinksminančios_moterys_spalva5-4", "1F9D1-1F3FF-200D-1F430-200D-1F9D1-1F3FE"},
    {"šviesiaplaukis_žmogus", "1F471"},
    {"šviesiaplaukis_žmogus_spalva1", "1F471-1F3FB"},
    {"šviesiaplaukis_žmogus_spalva2", "1F471-1F3FC"},
    {"šviesiaplaukis_žmogus_spalva3", "1F471-1F3FD"},
    {"šviesiaplaukis_žmogus_spalva4", "1F471-1F3FE"},
    {"šviesiaplaukis_žmogus_spalva5", "1F471-1F3FF"},
    {"vyras_su_kinietiška_kepurėle", "1F472"},
    {"vyras_su_kinietiška_kepurėle_spalva1", "1F472-1F3FB"},
    {"vyras_su_kinietiška_kepurėle_spalva2", "1F472-1F3FC"},
    {"vyras_su_kinietiška_kepurėle_spalva3", "1F472-1F3FD"},
    {"vyras_su_kinietiška_kepurėle_spalva4", "1F472-1F3FE"},
    {"vyras_su_kinietiška_kepurėle_spalva5", "1F472-1F3FF"},
    {"kūdikis", "1F476"},
    {"kūdikis_spalva1", "1F476-1F3FB"},
    {"kūdikis_spalva2", "1F476-1F3FC"},
    {"kūdikis_spalva3", "1F476-1F3FD"},
    {"kūdikis_spalva4", "1F476-1F3FE"},
    {"kūdikis_spalva5", "1F476-1F3FF"},
    {"princesė", "1F478"},
    {"princesė_spalva1", "1F478-1F3FB"},
    {"princesė_spalva2", "1F478-1F3FC"},
    {"princesė_spalva3", "1F478-1F3FD"},
    {"princesė_spalva4", "1F478-1F3FE"},
    {"princesė_spalva5", "1F478-1F3FF"},
    {"velniūkštis", "1F47F"},
    {"kaukolė", "1F480"},
    {"šokanti_moteris", "1F483"},
    {"šokanti_moteris_spalva1", "1F483-1F3FB"},
    {"šokanti_moteris_spalva2", "1F483-1F3FC"},
    {"šokanti_moteris_spalva3", "1F483-1F3FD"},
    {"šokanti_moteris_spalva4", "1F483-1F3FE"},
    {"šokanti_moteris_spalva5", "1F483-1F3FF"},
    {"lūpų_dažai", "1F484"},
    {"nagų_lakas", "1F485"},
    {"nagų_lakas_spalva1", "1F485-1F3FB"},
    {"nagų_lakas_spalva2", "1F485-1F3FC"},
    {"nagų_lakas_spalva3", "1F485-1F3FD"},
    {"nagų_lakas_spalva4", "1F485-1F3FE"},
    {"nagų_lakas_spalva5", "1F485-1F3FF"},
    {"veido_masažas", "1F486"},
    {"veido_masažas_spalva1", "1F486-1F3FB"},
    {"veido_masažas_spalva2", "1F486-1F3FC"},
    {"veido_masažas_spalva3", "1F486-1F3FD"},
    {"veido_masažas_spalva4", "1F486-1F3FE"},
    {"veido_masažas_spalva5", "1F486-1F3FF"},
    {"apkerpamas_žmogus", "1F487"},
    {"apkerpamas_žmogus_spalva1", "1F487-1F3FB"},
    {"apkerpamas_žmogus_spalva2", "1F487-1F3FC"},
    {"apkerpamas_žmogus_spalva3", "1F487-1F3FD"},
    {"apkerpamas_žmogus_spalva4", "1F487-1F3FE"},
    {"apkerpamas_žmogus_spalva5", "1F487-1F3FF"},
    {"kirpėjo_suktukas", "1F488"},
    {"švirkštas", "1F489"},
    {"piliulė", "1F48A"},
    {"bučinio_žymė", "1F48B"},
    {"meilės_laiškas", "1F48C"},
    {"žiedas", "1F48D"},
    {"bučinys", "1F48F"},
    {"bučinys_spalva1", "1F48F-1F3FB"},
    {"bučinys_spalva2", "1F48F-1F3FC"},
    {"bučinys_spalva3", "1F48F-1F3FD"},
    {"bučinys_spalva4", "1F48F-1F3FE"},
    {"bučinys_spalva5", "1F48F-1F3FF"},
    {"bučinys_spalva1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bučinys_spalva1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bučinys_spalva1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bučinys_spalva1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bučinys_spalva2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bučinys_spalva2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bučinys_spalva2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bučinys_spalva2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bučinys_spalva3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bučinys_spalva3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bučinys_spalva3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"bučinys_spalva3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bučinys_spalva4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bučinys_spalva4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bučinys_spalva4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bučinys_spalva4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FF"},
    {"bučinys_spalva5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FB"},
    {"bučinys_spalva5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FC"},
    {"bučinys_spalva5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FD"},
    {"bučinys_spalva5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F9D1-1F3FE"},
    {"puokštė", "1F490"},
    {"pora_su_širdele", "1F491"},
    {"pora_su_širdele_spalva1", "1F491-1F3FB"},
    {"pora_su_širdele_spalva2", "1F491-1F3FC"},
    {"pora_su_širdele_spalva3", "1F491-1F3FD"},
    {"pora_su_širdele_spalva4", "1F491-1F3FE"},
    {"pora_su_širdele_spalva5", "1F491-1F3FF"},
    {"pora_su_širdele_spalva1-2", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pora_su_širdele_spalva1-3", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pora_su_širdele_spalva1-4", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pora_su_širdele_spalva1-5", "1F9D1-1F3FB-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pora_su_širdele_spalva2-1", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pora_su_širdele_spalva2-3", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pora_su_širdele_spalva2-4", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pora_su_širdele_spalva2-5", "1F9D1-1F3FC-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pora_su_širdele_spalva3-1", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pora_su_širdele_spalva3-2", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pora_su_širdele_spalva3-4", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"pora_su_širdele_spalva3-5", "1F9D1-1F3FD-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pora_su_širdele_spalva4-1", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pora_su_širdele_spalva4-2", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pora_su_širdele_spalva4-3", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pora_su_širdele_spalva4-5", "1F9D1-1F3FE-200D-2764-FE0F-200D-1F9D1-1F3FF"},
    {"pora_su_širdele_spalva5-1", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FB"},
    {"pora_su_širdele_spalva5-2", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FC"},
    {"pora_su_širdele_spalva5-3", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FD"},
    {"pora_su_širdele_spalva5-4", "1F9D1-1F3FF-200D-2764-FE0F-200D-1F9D1-1F3FE"},
    {"vestuvės", "1F492"},
    {"plakanti_širdelė", "1F493"},
    {"sudaužyta_širdelė", "1F494"},
    {"dvi_širdelės", "1F495"},
    {"žėrinti_širdelė", "1F496"},
    {"auganti_širdelė", "1F497"},
    {"širdelė_su_strėle", "1F498"},
    {"mėlyna_širdelė", "1F499"},
    {"žalia_širdelė", "1F49A"},
    {"geltona_širdelė", "1F49B"},
    {"purpurinė_širdelė", "1F49C"},
    {"širdelė_su_kaspinu", "1F49D"},
    {"besisukančios_širdelės", "1F49E"},
    {"širdelės_dekoracija", "1F49F"},
    {"deimantas_su_tašku", "1F4A0"},
    {"lemputė", "1F4A1"},
    {"pykčio_simbolis", "1F4A2"},
    {"susidūrimas", "1F4A5"},
    {"prakaito_lašeliai", "1F4A6"},
    {"lašas", "1F4A7"},
    {"sukeltos_dulkės", "1F4A8"},
    {"išmatų_krūvelė", "1F4A9"},
    {"paryškinti_bicepsai", "1F4AA"},
    {"paryškinti_bicepsai_spalva1", "1F4AA-1F3FB"},
    {"paryškinti_bicepsai_spalva2", "1F4AA-1F3FC"},
    {"paryškinti_bicepsai_spalva3", "1F4AA-1F3FD"},
    {"paryškinti_bicepsai_spalva4", "1F4AA-1F3FE"},
    {"paryškinti_bicepsai_spalva5", "1F4AA-1F3FF"},
    {"apsvaigęs", "1F4AB"},
    {"kalbos_debesėlis", "1F4AC"},
    {"minties_debesėlis", "1F4AD"},
    {"balta_gėlė", "1F4AE"},
    {"šimtas_taškų", "1F4AF"},
    {"pinigų_maišas", "1F4B0"},
    {"paryškintas_dolerio_ženklas", "1F4B2"},
    {"kredito_kortelė", "1F4B3"},
    {"svaro_sterlingų_banknotas", "1F4B7"},
    {"didėjimo_diagrama_su_jenos_simboliu", "1F4B9"},
    {"failų_aplankas", "1F4C1"},
    {"atidarytas_failų_aplankas", "1F4C2"},
    {"užsirietęs_puslapis", "1F4C3"},
    {"puslapis_viršutine_puse", "1F4C4"},
    {"nuplėšiamas_kalendorius", "1F4C6"},
    {"kortelių_dėklas", "1F4C7"},
    {"didėjimo_tendencijos_diagrama", "1F4C8"},
    {"mažėjimo_tendencijos_diagrama", "1F4C9"},
    {"juostinė_diagrama", "1F4CA"},
    {"sąvaržėlė", "1F4CE"},
    {"tiesi_liniuotė", "1F4CF"},
    {"trikampė_liniuotė", "1F4D0"},
    {"žymių_skirtukai", "1F4D1"},
    {"sąskaitų_knyga", "1F4D2"},
    {"užrašinė", "1F4D3"},
    {"užrašų_knygelė_su_dekoratyviniu_viršeliu", "1F4D4"},
    {"užversta_knyga", "1F4D5"},
    {"žalia_knyga", "1F4D7"},
    {"mėlyna_knyga", "1F4D8"},
    {"oranžinė_knyga", "1F4D9"},
    {"vardo_ženklelis", "1F4DB"},
    {"atmintinė", "1F4DD"},
    {"palydovinė_antena", "1F4E1"},
    {"siunčiamų_laiškų_dėklas", "1F4E4"},
    {"gaunamų_laiškų_dėklas", "1F4E5"},
    {"el_laiškas", "1F4E7"},
    {"uždaryta_pašto_dėžutė_su_nuleista_vėliava", "1F4EA"},
    {"uždaryta_pašto_dėžutė_su_pakelta_vėliava", "1F4EB"},
    {"atidaryta_pašto_dėžutė_su_pakelta_vėliava", "1F4EC"},
    {"atidaryta_pašto_dėžutė_su_nuleista_vėliava", "1F4ED"},
    {"pašto_dėžutė", "1F4EE"},
    {"pašto_ragas", "1F4EF"},
    {"laikraštis", "1F4F0"},
    {"vibravimo_režimas", "1F4F3"},
    {"išjungti_mobilųjį_telefoną", "1F4F4"},
    {"jokių_mobiliųjų_telefonų", "1F4F5"},
    {"vaizdo_įrašų_kasetė", "1F4FC"},
    {"filmų_projektorius", "1F4FD"},
    {"rožinis", "1F4FF"},
    {"maišyti_takelius", "1F500"},
    {"kartoti_vieną", "1F502"},
    {"vertikalios_laikrodžio_rodyklės_kryptimis_nukreiptos_rodyklės", "1F503"},
    {"prieš_laikrodžio_rodyklę_nukreiptų_rodyklių_mygtukas", "1F504"},
    {"užtemdymo_mygtukas", "1F505"},
    {"ryškumo_mygtukas", "1F506"},
    {"garsiakalbis_išjungtas", "1F507"},
    {"elektros_kištukas", "1F50C"},
    {"didinimo_stiklas_į_kairę", "1F50D"},
    {"didinimo_stiklas_į_dešinę", "1F50E"},
    {"spyna_ir_rašiklis", "1F50F"},
    {"užrakinta_spyna_su_raktu", "1F510"},
    {"žymė", "1F516"},
    {"akutė", "1F518"},
    {"rodyklė_su_užrašu_back", "1F519"},
    {"rodyklė_su_užrašu_end", "1F51A"},
    {"rodyklė_su_užrašu_on", "1F51B"},
    {"rodyklė_su_užrašu_soon", "1F51C"},
    {"rodyklė_su_užrašu_top", "1F51D"},
    {"tik_pilnamečiams", "1F51E"},
    {"įvestis_didžiosios_lotyniškos_raidės", "1F520"},
    {"įvestis_mažosios_lotyniškos_raidės", "1F521"},
    {"įvesties_skaičiai", "1F522"},
    {"įvesties_simboliai", "1F523"},
    {"įvestis_lotyniškos_raidės", "1F524"},
    {"žibintuvėlis", "1F526"},
    {"veržliaraktis", "1F527"},
    {"veržlė_ir_varžtas", "1F529"},
    {"vandens_šautuvas", "1F52B"},
    {"krištolinis_rutulys", "1F52E"},
    {"šešiakampė_žvaigždė_su_tašku", "1F52F"},
    {"japoniškas_pradedančiojo_simbolis", "1F530"},
    {"trišakė_emblema", "1F531"},
    {"mėlynas_apskritimas", "1F535"},
    {"didelis_oranžinis_deimantas", "1F536"},
    {"didelis_mėlynas_deimantas", "1F537"},
    {"mažas_oranžinis_deimantas", "1F538"},
    {"mažas_mėlynas_deimantas", "1F539"},
    {"raudonas_į_viršų_nukreiptas_trikampis", "1F53A"},
    {"raudonas_į_apačią_nukreiptas_trikampis", "1F53B"},
    {"į_viršų", "1F53C"},
    {"į_apačią", "1F53D"},
    {"mečetė", "1F54C"},
    {"šešios_valandos", "1F555"},
    {"aštuonios_valandos", "1F557"},
    {"dešimt_valandų", "1F559"},
    {"vienuolika_valandų", "1F55A"},
    {"dvylika_valandų", "1F55B"},
    {"pusė_dviejų", "1F55C"},
    {"pusė_trijų", "1F55D"},
    {"pusė_keturių", "1F55E"},
    {"pusė_penkių", "1F55F"},
    {"pusė_šešių", "1F560"},
    {"pusė_septynių", "1F561"},
    {"pusė_aštuonių", "1F562"},
    {"pusė_devynių", "1F563"},
    {"pusė_dešimt", "1F564"},
    {"pusė_vienuolikos", "1F565"},
    {"pusė_dvylikos", "1F566"},
    {"dvylika_trisdešimt", "1F567"},
    {"žvakė", "1F56F"},
    {"virš_židinio_laikomas_laikrodis", "1F570"},
    {"duobė", "1F573"},
    {"į_orą_kylantis_vyras_su_kostiumu", "1F574"},
    {"į_orą_kylantis_vyras_su_kostiumu_spalva1", "1F574-1F3FB"},
    {"į_orą_kylantis_vyras_su_kostiumu_spalva2", "1F574-1F3FC"},
    {"į_orą_kylantis_vyras_su_kostiumu_spalva3", "1F574-1F3FD"},
    {"į_orą_kylantis_vyras_su_kostiumu_spalva4", "1F574-1F3FE"},
    {"į_orą_kylantis_vyras_su_kostiumu_spalva5", "1F574-1F3FF"},
    {"akiniai_nuo_saulės", "1F576"},
    {"šokantis_vyras", "1F57A"},
    {"šokantis_vyras_spalva1", "1F57A-1F3FB"},
    {"šokantis_vyras_spalva2", "1F57A-1F3FC"},
    {"šokantis_vyras_spalva3", "1F57A-1F3FD"},
    {"šokantis_vyras_spalva4", "1F57A-1F3FE"},
    {"šokantis_vyras_spalva5", "1F57A-1F3FF"},
    {"sukabintos_sąvaržėlės", "1F587"},
    {"rašiklis", "1F58A"},
    {"kreidelė", "1F58D"},
    {"iškelta_ranka_su_išskėstais_pirštais", "1F590"},
    {"iškelta_ranka_su_išskėstais_pirštais_spalva1", "1F590-1F3FB"},
    {"iškelta_ranka_su_išskėstais_pirštais_spalva2", "1F590-1F3FC"},
    {"iškelta_ranka_su_išskėstais_pirštais_spalva3", "1F590-1F3FD"},
    {"iškelta_ranka_su_išskėstais_pirštais_spalva4", "1F590-1F3FE"},
    {"iškelta_ranka_su_išskėstais_pirštais_spalva5", "1F590-1F3FF"},
    {"vidurinis_pirštas", "1F595"},
    {"vidurinis_pirštas_spalva1", "1F595-1F3FB"},
    {"vidurinis_pirštas_spalva2", "1F595-1F3FC"},
    {"vidurinis_pirštas_spalva3", "1F595-1F3FD"},
    {"vidurinis_pirštas_spalva4", "1F595-1F3FE"},
    {"vidurinis_pirštas_spalva5", "1F595-1F3FF"},
    {"vulkanų_rasės_pasisveikinimo_ženklas", "1F596"},
    {"vulkanų_rasės_pasisveikinimo_ženklas_spalva1", "1F596-1F3FB"},
    {"vulkanų_rasės_pasisveikinimo_ženklas_spalva2", "1F596-1F3FC"},
    {"vulkanų_rasės_pasisveikinimo_ženklas_spalva3", "1F596-1F3FD"},
    {"vulkanų_rasės_pasisveikinimo_ženklas_spalva4", "1F596-1F3FE"},
    {"vulkanų_rasės_pasisveikinimo_ženklas_spalva5", "1F596-1F3FF"},
    {"juoda_širdis", "1F5A4"},
    {"kompiuterio_pelė", "1F5B1"},
    {"rėmas_su_paveikslu", "1F5BC"},
    {"kortelių_laikiklio_skirtukai", "1F5C2"},
    {"kortelių_saugojimo_dėžė", "1F5C3"},
    {"bylų_spintelė_su_stalčiais", "1F5C4"},
    {"šiukšliadėžė", "1F5D1"},
    {"užrašinė_su_spirale", "1F5D2"},
    {"susuktas_laikraštis", "1F5DE"},
    {"kairysis_kalbos_debesėlis", "1F5E8"},
    {"dešinysis_pykčio_debesėlis", "1F5EF"},
    {"balsavimo_dėžutė_su_balsavimo_biuleteniu", "1F5F3"},
    {"pasaulio_žemėlapis", "1F5FA"},
    {"fudžio_kalnas", "1F5FB"},
    {"tokijo_bokštas", "1F5FC"},
    {"laisvės_statula", "1F5FD"},
    {"japonijos_žemėlapis", "1F5FE"},
    {"išsišiepęs_veidas", "1F600"},
    {"išsišiepęs_veidas_su_besišypsančiomis_akimis", "1F601"},
    {"veidas_su_džiaugsmo_ašaromis", "1F602"},
    {"besišypsantis_išsižiojęs_veidas", "1F603"},
    {"besišypsantis_išsižiojęs_veidas_besišypsančiomis_akimis", "1F604"},
    {"besišypsantis_išsižiojęs_veidas_išpiltas_šalto_prakaito", "1F605"},
    {"besišypsantis_išsižiojęs_veidas_su_smarkiai_užmerktomis_akimis", "1F606"},
    {"besišypsantis_veidas_su_aureole", "1F607"},
    {"besišypsantis_veidas_su_ragais", "1F608"},
    {"besišypsantis_veidas_su_besišypsančiomis_akimis", "1F60A"},
    {"skaniu_maistu_besimėgaujantis_veidas", "1F60B"},
    {"veidas_su_palengvėjimu", "1F60C"},
    {"besišypsantis_veidas_su_širdelės_formos_akimis", "1F60D"},
    {"besišypsantis_veidas_su_akiniais_nuo_saulės", "1F60E"},
    {"iš_pasitenkinimo_besišypsantis_veidas", "1F60F"},
    {"veidas_be_jokios_išraiškos", "1F611"},
    {"veidas_išpiltas_šalto_prakaito", "1F613"},
    {"susimąstęs_veidas", "1F614"},
    {"sutrikęs_veidas", "1F615"},
    {"bučiuojantis_veidas", "1F617"},
    {"bučinį_siunčiantis_veidas", "1F618"},
    {"bučiuojantis_su_besišypsančiomis_akimis", "1F619"},
    {"bučiuojantis_su_užmerktomis_akimis", "1F61A"},
    {"veidas_su_iškištu_liežuviu", "1F61B"},
    {"veidas_su_iškištu_liežuviu_ir_viena_primerkta_akimi", "1F61C"},
    {"veidas_su_iškištu_liežuviu_ir_stipriai_užmerktomis_akimis", "1F61D"},
    {"nusivylęs_veidas", "1F61E"},
    {"susirūpinęs_veidas", "1F61F"},
    {"susiraukęs_veidas", "1F621"},
    {"veidas_su_iš_nosies_einančiais_garais", "1F624"},
    {"nusivylęs_veidas_su_palengvėjimu", "1F625"},
    {"paniuręs_išsižiojęs_veidas", "1F626"},
    {"kenčiantis_veidas", "1F627"},
    {"baimės_pilnas_veidas", "1F628"},
    {"nuvargęs_veidas", "1F629"},
    {"pavargęs_veidas", "1F62B"},
    {"išsižiojęs_veidas", "1F62E"},
    {"nuščiuvęs_veidas", "1F62F"},
    {"išsižiojęs_veidas_išpiltas_šalto_prakaito", "1F630"},
    {"iš_baimės_šaukiantis_veidas", "1F631"},
    {"apstulbęs_veidas", "1F632"},
    {"nuraudęs_veidas", "1F633"},
    {"apsvaigęs_veidas", "1F635"},
    {"išsišiepusios_katės_snukutis_su_besišypsančiomis_akimis", "1F638"},
    {"katės_snukutis_su_džiaugsmo_ašaromis", "1F639"},
    {"besišypsančios_katės_snukutis", "1F63A"},
    {"besišypsančios_katės_snukutis_su_širdelės_formos_akimis", "1F63B"},
    {"nuvargusios_besišypsančios_katės_snukutis", "1F63C"},
    {"bučiuojančios_katės_snukutis", "1F63D"},
    {"susiraukusios_katės_snukutis", "1F63E"},
    {"verkiančios_katės_snukutis", "1F63F"},
    {"nustebusios_katės_snukutis", "1F640"},
    {"šiek_tiek_paniuręs_veidas", "1F641"},
    {"šiek_tiek_besišypsantis_veidas", "1F642"},
    {"nusilenkęs_žmogus", "1F647"},
    {"nusilenkęs_žmogus_spalva1", "1F647-1F3FB"},
    {"nusilenkęs_žmogus_spalva2", "1F647-1F3FC"},
    {"nusilenkęs_žmogus_spalva3", "1F647-1F3FD"},
    {"nusilenkęs_žmogus_spalva4", "1F647-1F3FE"},
    {"nusilenkęs_žmogus_spalva5", "1F647-1F3FF"},
    {"užsidengusi_akis_beždžionė", "1F648"},
    {"užsidengusi_ausis_beždžionė", "1F649"},
    {"užsidengusi_burną_beždžionė", "1F64A"},
    {"žmogus_iškėlęs_ranką", "1F64B"},
    {"žmogus_iškėlęs_ranką_spalva1", "1F64B-1F3FB"},
    {"žmogus_iškėlęs_ranką_spalva2", "1F64B-1F3FC"},
    {"žmogus_iškėlęs_ranką_spalva3", "1F64B-1F3FD"},
    {"žmogus_iškėlęs_ranką_spalva4", "1F64B-1F3FE"},
    {"žmogus_iškėlęs_ranką_spalva5", "1F64B-1F3FF"},
    {"žmogus_iškėlęs_rankas", "1F64C"},
    {"žmogus_iškėlęs_rankas_spalva1", "1F64C-1F3FB"},
    {"žmogus_iškėlęs_rankas_spalva2", "1F64C-1F3FC"},
    {"žmogus_iškėlęs_rankas_spalva3", "1F64C-1F3FD"},
    {"žmogus_iškėlęs_rankas_spalva4", "1F64C-1F3FE"},
    {"žmogus_iškėlęs_rankas_spalva5", "1F64C-1F3FF"},
    {"paniuręs_žmogus", "1F64D"},
    {"paniuręs_žmogus_spalva1", "1F64D-1F3FB"},
    {"paniuręs_žmogus_spalva2", "1F64D-1F3FC"},
    {"paniuręs_žmogus_spalva3", "1F64D-1F3FD"},
    {"paniuręs_žmogus_spalva4", "1F64D-1F3FE"},
    {"paniuręs_žmogus_spalva5", "1F64D-1F3FF"},
    {"susiraukęs_žmogus", "1F64E"},
    {"susiraukęs_žmogus_spalva1", "1F64E-1F3FB"},
    {"susiraukęs_žmogus_spalva2", "1F64E-1F3FC"},
    {"susiraukęs_žmogus_spalva3", "1F64E-1F3FD"},
    {"susiraukęs_žmogus_spalva4", "1F64E-1F3FE"},
    {"susiraukęs_žmogus_spalva5", "1F64E-1F3FF"},
    {"artėjantis_autobusas", "1F68D"},
    {"autobusų_stotelė", "1F68F"},
    {"ugniagesių_automobilis", "1F692"},
    {"artėjantis_policijos_automobilis", "1F694"},
    {"artėjantis_taksi", "1F696"},
    {"artėjantis_automobilis", "1F698"},
    {"krovininis_sunkvežimis", "1F69A"},
    {"sunkvežimis_su_priekaba", "1F69B"},
    {"vienbėgis_geležinkelis", "1F69D"},
    {"kalnų_traukinys", "1F69E"},
    {"kabanti_geležinkelio_linija", "1F69F"},
    {"kalnų_keltuvas", "1F6A0"},
    {"lynų_keltuvas", "1F6A1"},
    {"irkluojantis_žmogus", "1F6A3"},
    {"irkluojantis_žmogus_spalva1", "1F6A3-1F3FB"},
    {"irkluojantis_žmogus_spalva2", "1F6A3-1F3FC"},
    {"irkluojantis_žmogus_spalva3", "1F6A3-1F3FD"},
    {"irkluojantis_žmogus_spalva4", "1F6A3-1F3FE"},
    {"irkluojantis_žmogus_spalva5", "1F6A3-1F3FF"},
    {"horizontalus_šviesoforas", "1F6A5"},
    {"vertikalus_šviesoforas", "1F6A6"},
    {"trikampė_vėliava", "1F6A9"},
    {"draudžiama", "1F6AB"},
    {"rūkymas", "1F6AC"},
    {"rūkyti_draudžiama", "1F6AD"},
    {"šiukšlių_metimo_į_šiukšliadėžes_ženklas", "1F6AE"},
    {"šiukšlinti_draudžiama", "1F6AF"},
    {"draudžiama_gerti_vandenį", "1F6B1"},
    {"dviračiai_draudžiami", "1F6B3"},
    {"kalnų_dviratininkas", "1F6B5"},
    {"kalnų_dviratininkas_spalva1", "1F6B5-1F3FB"},
    {"kalnų_dviratininkas_spalva2", "1F6B5-1F3FC"},
    {"kalnų_dviratininkas_spalva3", "1F6B5-1F3FD"},
    {"kalnų_dviratininkas_spalva4", "1F6B5-1F3FE"},
    {"kalnų_dviratininkas_spalva5", "1F6B5-1F3FF"},
    {"pėstysis", "1F6B6"},
    {"pėstysis_spalva1", "1F6B6-1F3FB"},
    {"pėstysis_spalva2", "1F6B6-1F3FC"},
    {"pėstysis_spalva3", "1F6B6-1F3FD"},
    {"pėstysis_spalva4", "1F6B6-1F3FE"},
    {"pėstysis_spalva5", "1F6B6-1F3FF"},
    {"pėstiesiems_eiti_draudžiama", "1F6B7"},
    {"vaikų_perėja", "1F6B8"},
    {"vyrų_tualetas", "1F6B9"},
    {"moterų_tualetas", "1F6BA"},
    {"kūdikio_simbolis", "1F6BC"},
    {"dušas", "1F6BF"},
    {"žmogus_prausiasi_vonioje", "1F6C0"},
    {"žmogus_prausiasi_vonioje_spalva1", "1F6C0-1F3FB"},
    {"žmogus_prausiasi_vonioje_spalva2", "1F6C0-1F3FC"},
    {"žmogus_prausiasi_vonioje_spalva3", "1F6C0-1F3FD"},
    {"žmogus_prausiasi_vonioje_spalva4", "1F6C0-1F3FE"},
    {"žmogus_prausiasi_vonioje_spalva5", "1F6C0-1F3FF"},
    {"pasų_kontrolė", "1F6C2"},
    {"muitinė", "1F6C3"},
    {"bagažo_skyrius", "1F6C4"},
    {"paliktas_bagažas", "1F6C5"},
    {"žmogus_lovoje", "1F6CC"},
    {"žmogus_lovoje_spalva1", "1F6CC-1F3FB"},
    {"žmogus_lovoje_spalva2", "1F6CC-1F3FC"},
    {"žmogus_lovoje_spalva3", "1F6CC-1F3FD"},
    {"žmogus_lovoje_spalva4", "1F6CC-1F3FE"},
    {"žmogus_lovoje_spalva5", "1F6CC-1F3FF"},
    {"pirkinių_krepšeliai", "1F6CD"},
    {"sustojimo_ženklas", "1F6D1"},
    {"pirkinių_vežimėlis", "1F6D2"},
    {"induistų_šventykla", "1F6D5"},
    {"nuošliauža", "1F6D8"},
    {"belaidis_ryšys", "1F6DC"},
    {"čiuožykla", "1F6DD"},
    {"gelbėjimo_ratas", "1F6DF"},
    {"plaktukas_ir_veržliaraktis", "1F6E0"},
    {"degalų_statinė", "1F6E2"},
    {"geležinkelio_bėgiai", "1F6E4"},
    {"motorinė_valtis", "1F6E5"},
    {"mažas_lėktuvas", "1F6E9"},
    {"lėktuvo_pakilimas", "1F6EB"},
    {"lėktuvo_nusileidimas", "1F6EC"},
    {"rogės", "1F6F7"},
    {"skraidanti_lėkštė", "1F6F8"},
    {"riedlentė", "1F6F9"},
    {"autorikša", "1F6FA"},
    {"riedučiai", "1F6FC"},
    {"oranžinis_apskritimas", "1F7E0"},
    {"žalias_apskritimas", "1F7E2"},
    {"mėlynas_kvadratas", "1F7E6"},
    {"oranžinis_kvadratas", "1F7E7"},
    {"žalias_kvadratas", "1F7E9"},
    {"paryškintas_lygybės_ženklas", "1F7F0"},
    {"suspausti_pirštai", "1F90C"},
    {"suspausti_pirštai_spalva1", "1F90C-1F3FB"},
    {"suspausti_pirštai_spalva2", "1F90C-1F3FC"},
    {"suspausti_pirštai_spalva3", "1F90C-1F3FD"},
    {"suspausti_pirštai_spalva4", "1F90C-1F3FE"},
    {"suspausti_pirštai_spalva5", "1F90C-1F3FF"},
    {"balta_širdelė", "1F90D"},
    {"ruda_širdelė", "1F90E"},
    {"žnybianti_ranka", "1F90F"},
    {"žnybianti_ranka_spalva1", "1F90F-1F3FB"},
    {"žnybianti_ranka_spalva2", "1F90F-1F3FC"},
    {"žnybianti_ranka_spalva3", "1F90F-1F3FD"},
    {"žnybianti_ranka_spalva4", "1F90F-1F3FE"},
    {"žnybianti_ranka_spalva5", "1F90F-1F3FF"},
    {"veidas_su_užtrauktuku_vietoje_burnos", "1F910"},
    {"veidas_su_pinigu_vietoje_liežuvio", "1F911"},
    {"mąstantis_veidas", "1F914"},
    {"veidas_su_galvos_tvarsčiu", "1F915"},
    {"ragų_ženklas", "1F918"},
    {"ragų_ženklas_spalva1", "1F918-1F3FB"},
    {"ragų_ženklas_spalva2", "1F918-1F3FC"},
    {"ragų_ženklas_spalva3", "1F918-1F3FD"},
    {"ragų_ženklas_spalva4", "1F918-1F3FE"},
    {"ragų_ženklas_spalva5", "1F918-1F3FF"},
    {"pakelta_viršutinė_plaštakos_pusė", "1F91A"},
    {"pakelta_viršutinė_plaštakos_pusė_spalva1", "1F91A-1F3FB"},
    {"pakelta_viršutinė_plaštakos_pusė_spalva2", "1F91A-1F3FC"},
    {"pakelta_viršutinė_plaštakos_pusė_spalva3", "1F91A-1F3FD"},
    {"pakelta_viršutinė_plaštakos_pusė_spalva4", "1F91A-1F3FE"},
    {"pakelta_viršutinė_plaštakos_pusė_spalva5", "1F91A-1F3FF"},
    {"į_kairę_nukreiptas_kumštis", "1F91B"},
    {"į_kairę_nukreiptas_kumštis_spalva1", "1F91B-1F3FB"},
    {"į_kairę_nukreiptas_kumštis_spalva2", "1F91B-1F3FC"},
    {"į_kairę_nukreiptas_kumštis_spalva3", "1F91B-1F3FD"},
    {"į_kairę_nukreiptas_kumštis_spalva4", "1F91B-1F3FE"},
    {"į_kairę_nukreiptas_kumštis_spalva5", "1F91B-1F3FF"},
    {"į_dešinę_nukreiptas_kumštis", "1F91C"},
    {"į_dešinę_nukreiptas_kumštis_spalva1", "1F91C-1F3FB"},
    {"į_dešinę_nukreiptas_kumštis_spalva2", "1F91C-1F3FC"},
    {"į_dešinę_nukreiptas_kumštis_spalva3", "1F91C-1F3FD"},
    {"į_dešinę_nukreiptas_kumštis_spalva4", "1F91C-1F3FE"},
    {"į_dešinę_nukreiptas_kumštis_spalva5", "1F91C-1F3FF"},
    {"sukryžiuoti_pirštai", "1F91E"},
    {"sukryžiuoti_pirštai_spalva1", "1F91E-1F3FB"},
    {"sukryžiuoti_pirštai_spalva2", "1F91E-1F3FC"},
    {"sukryžiuoti_pirštai_spalva3", "1F91E-1F3FD"},
    {"sukryžiuoti_pirštai_spalva4", "1F91E-1F3FE"},
    {"sukryžiuoti_pirštai_spalva5", "1F91E-1F3FF"},
    {"kaubojaus_veidas_su_skrybėle", "1F920"},
    {"besiruošiantis_vemti_veidas", "1F922"},
    {"raitausi_iš_juoko", "1F923"},
    {"besiseilėjantis_veidas", "1F924"},
    {"čiaudantis_veidas", "1F927"},
    {"žvaigždžių_liga", "1F929"},
    {"pamišęs_veidas", "1F92A"},
    {"nėščia_moteris", "1F930"},
    {"nėščia_moteris_spalva1", "1F930-1F3FB"},
    {"nėščia_moteris_spalva2", "1F930-1F3FC"},
    {"nėščia_moteris_spalva3", "1F930-1F3FD"},
    {"nėščia_moteris_spalva4", "1F930-1F3FE"},
    {"nėščia_moteris_spalva5", "1F930-1F3FF"},
    {"maitinimas_krūtimi", "1F931"},
    {"maitinimas_krūtimi_spalva1", "1F931-1F3FB"},
    {"maitinimas_krūtimi_spalva2", "1F931-1F3FC"},
    {"maitinimas_krūtimi_spalva3", "1F931-1F3FD"},
    {"maitinimas_krūtimi_spalva4", "1F931-1F3FE"},
    {"maitinimas_krūtimi_spalva5", "1F931-1F3FF"},
    {"asmenukė", "1F933"},
    {"asmenukė_spalva1", "1F933-1F3FB"},
    {"asmenukė_spalva2", "1F933-1F3FC"},
    {"asmenukė_spalva3", "1F933-1F3FD"},
    {"asmenukė_spalva4", "1F933-1F3FE"},
    {"asmenukė_spalva5", "1F933-1F3FF"},
    {"gūžtelėjimas_pečiais", "1F937"},
    {"gūžtelėjimas_pečiais_spalva1", "1F937-1F3FB"},
    {"gūžtelėjimas_pečiais_spalva2", "1F937-1F3FC"},
    {"gūžtelėjimas_pečiais_spalva3", "1F937-1F3FD"},
    {"gūžtelėjimas_pečiais_spalva4", "1F937-1F3FE"},
    {"gūžtelėjimas_pečiais_spalva5", "1F937-1F3FF"},
    {"persivertimas_šonu", "1F938"},
    {"persivertimas_šonu_spalva1", "1F938-1F3FB"},
    {"persivertimas_šonu_spalva2", "1F938-1F3FC"},
    {"persivertimas_šonu_spalva3", "1F938-1F3FD"},
    {"persivertimas_šonu_spalva4", "1F938-1F3FE"},
    {"persivertimas_šonu_spalva5", "1F938-1F3FF"},
    {"žongliravimas", "1F939"},
    {"žongliravimas_spalva1", "1F939-1F3FB"},
    {"žongliravimas_spalva2", "1F939-1F3FC"},
    {"žongliravimas_spalva3", "1F939-1F3FD"},
    {"žongliravimas_spalva4", "1F939-1F3FE"},
    {"žongliravimas_spalva5", "1F939-1F3FF"},
    {"nardymo_kaukė", "1F93F"},
    {"gležna_gėlė", "1F940"},
    {"būgnas", "1F941"},
    {"sudaužtos_taurės", "1F942"},
    {"stiklinė", "1F943"},
    {"šaukštas", "1F944"},
    {"bokso_pirštinė", "1F94A"},
    {"kovos_sporto_šakos_uniforma", "1F94B"},
    {"bulvė", "1F954"},
    {"žalios_salotos", "1F957"},
    {"įdarytas_lavašas", "1F959"},
    {"kiaušinis", "1F95A"},
    {"pieno_stiklinė", "1F95B"},
    {"žemės_riešutas", "1F95C"},
    {"laimės_sausainiukas", "1F960"},
    {"dėžutė_išsinešti", "1F961"},
    {"lazdelės", "1F962"},
    {"dubuo_su_šaukštu", "1F963"},
    {"puodelis_su_šiaudeliu", "1F964"},
    {"mėsos_gabalėliai", "1F969"},
    {"sumuštinis", "1F96A"},
    {"lapinė_daržovė", "1F96C"},
    {"mėnulio_pyragas", "1F96E"},
    {"besišypsantis_veidas_su_trimis_širdelėmis", "1F970"},
    {"žiovaujantis_veidas", "1F971"},
    {"besišypsantis_veidas_su_ašara", "1F972"},
    {"švenčiantis_veidas", "1F973"},
    {"apkvaišęs_veidas", "1F974"},
    {"įkaitęs_veidas", "1F975"},
    {"sušalęs_veidas", "1F976"},
    {"nindzė", "1F977"},
    {"nindzė_spalva1", "1F977-1F3FB"},
    {"nindzė_spalva2", "1F977-1F3FC"},
    {"nindzė_spalva3", "1F977-1F3FD"},
    {"nindzė_spalva4", "1F977-1F3FE"},
    {"nindzė_spalva5", "1F977-1F3FF"},
    {"paslėptas_veidas", "1F978"},
    {"ašaras_tramdantis_veidas", "1F979"},
    {"žygio_batai", "1F97E"},
    {"liūto_snukis", "1F981"},
    {"skorpionas_gyvūnas", "1F982"},
    {"šikšnosparnis", "1F987"},
    {"pelėda", "1F989"},
    {"lapės_snukis", "1F98A"},
    {"driežas", "1F98E"},
    {"krevetė", "1F990"},
    {"žirafa", "1F992"},
    {"ežys", "1F994"},
    {"kengūra", "1F998"},
    {"papūga", "1F99C"},
    {"meškėnas", "1F99D"},
    {"gulbė", "1F9A2"},
    {"ūdra", "1F9A6"},
    {"austrė", "1F9AA"},
    {"šuo_vedlys", "1F9AE"},
    {"žili_plaukai", "1F9B3"},
    {"pėda", "1F9B6"},
    {"pėda_spalva1", "1F9B6-1F3FB"},
    {"pėda_spalva2", "1F9B6-1F3FC"},
    {"pėda_spalva3", "1F9B6-1F3FD"},
    {"pėda_spalva4", "1F9B6-1F3FE"},
    {"pėda_spalva5", "1F9B6-1F3FF"},
    {"gelbėjimosi_liemenė", "1F9BA"},
    {"motorizuotas_vežimėlis", "1F9BC"},
    {"rankomis_valdomas_vežimėlis", "1F9BD"},
    {"mechaninė_ranka", "1F9BE"},
    {"mechaninė_koja", "1F9BF"},
    {"sūrio_gabaliukas", "1F9C0"},
    {"gėrimo_pakelis", "1F9C3"},
    {"česnakas", "1F9C4"},
    {"svogūnas", "1F9C5"},
    {"matė", "1F9C9"},
    {"burbulinė_arbata", "1F9CB"},
    {"stovintis_žmogus", "1F9CD"},
    {"stovintis_žmogus_spalva1", "1F9CD-1F3FB"},
    {"stovintis_žmogus_spalva2", "1F9CD-1F3FC"},
    {"stovintis_žmogus_spalva3", "1F9CD-1F3FD"},
    {"stovintis_žmogus_spalva4", "1F9CD-1F3FE"},
    {"stovintis_žmogus_spalva5", "1F9CD-1F3FF"},
    {"klūpantis_žmogus", "1F9CE"},
    {"klūpantis_žmogus_spalva1", "1F9CE-1F3FB"},
    {"klūpantis_žmogus_spalva2", "1F9CE-1F3FC"},
    {"klūpantis_žmogus_spalva3", "1F9CE-1F3FD"},
    {"klūpantis_žmogus_spalva4", "1F9CE-1F3FE"},
    {"klūpantis_žmogus_spalva5", "1F9CE-1F3FF"},
    {"kurčias_žmogus", "1F9CF"},
    {"kurčias_žmogus_spalva1", "1F9CF-1F3FB"},
    {"kurčias_žmogus_spalva2", "1F9CF-1F3FC"},
    {"kurčias_žmogus_spalva3", "1F9CF-1F3FD"},
    {"kurčias_žmogus_spalva4", "1F9CF-1F3FE"},
    {"kurčias_žmogus_spalva5", "1F9CF-1F3FF"},
    {"barzdotas_žmogus", "1F9D4"},
    {"barzdotas_žmogus_spalva1", "1F9D4-1F3FB"},
    {"barzdotas_žmogus_spalva2", "1F9D4-1F3FC"},
    {"barzdotas_žmogus_spalva3", "1F9D4-1F3FD"},
    {"barzdotas_žmogus_spalva4", "1F9D4-1F3FE"},
    {"barzdotas_žmogus_spalva5", "1F9D4-1F3FF"},
    {"asmuo_garų_kambaryje", "1F9D6"},
    {"asmuo_garų_kambaryje_spalva1", "1F9D6-1F3FB"},
    {"asmuo_garų_kambaryje_spalva2", "1F9D6-1F3FC"},
    {"asmuo_garų_kambaryje_spalva3", "1F9D6-1F3FD"},
    {"asmuo_garų_kambaryje_spalva4", "1F9D6-1F3FE"},
    {"asmuo_garų_kambaryje_spalva5", "1F9D6-1F3FF"},
    {"kopiantis_žmogus", "1F9D7"},
    {"kopiantis_žmogus_spalva1", "1F9D7-1F3FB"},
    {"kopiantis_žmogus_spalva2", "1F9D7-1F3FC"},
    {"kopiantis_žmogus_spalva3", "1F9D7-1F3FD"},
    {"kopiantis_žmogus_spalva4", "1F9D7-1F3FE"},
    {"kopiantis_žmogus_spalva5", "1F9D7-1F3FF"},
    {"žmogus_lotoso_pozoje", "1F9D8"},
    {"žmogus_lotoso_pozoje_spalva1", "1F9D8-1F3FB"},
    {"žmogus_lotoso_pozoje_spalva2", "1F9D8-1F3FC"},
    {"žmogus_lotoso_pozoje_spalva3", "1F9D8-1F3FD"},
    {"žmogus_lotoso_pozoje_spalva4", "1F9D8-1F3FE"},
    {"žmogus_lotoso_pozoje_spalva5", "1F9D8-1F3FF"},
    {"fėja", "1F9DA"},
    {"fėja_spalva1", "1F9DA-1F3FB"},
    {"fėja_spalva2", "1F9DA-1F3FC"},
    {"fėja_spalva3", "1F9DA-1F3FD"},
    {"fėja_spalva4", "1F9DA-1F3FE"},
    {"fėja_spalva5", "1F9DA-1F3FF"},
    {"oranžinė_širdelė", "1F9E1"},
    {"reklaminė_kepurė", "1F9E2"},
    {"pirštinės", "1F9E4"},
    {"kojinės", "1F9E6"},
    {"dėlionė", "1F9E9"},
    {"mėgintuvėlis", "1F9EA"},
    {"petri_lėkštelė", "1F9EB"},
    {"įrankių_rinkinys", "1F9F0"},
    {"bagažas", "1F9F3"},
    {"siūlas", "1F9F5"},
    {"siūlų_kamuolys", "1F9F6"},
    {"žiogelis", "1F9F7"},
    {"meškiukas", "1F9F8"},
    {"šluota", "1F9F9"},
    {"pintinė", "1F9FA"},
    {"kempinė", "1F9FD"},
    {"čekis", "1F9FE"},
    {"vientisas_maudymosi_kostiumėlis", "1FA71"},
    {"trumpikės", "1FA72"},
    {"šortai", "1FA73"},
    {"sandalai_su_dirželiu", "1FA74"},
    {"šviesiai_mėlyna_širdelė", "1FA75"},
    {"pilka_širdelė", "1FA76"},
    {"rožinė_širdelė", "1FA77"},
    {"kraujo_lašas", "1FA78"},
    {"parašiutas", "1FA82"},
    {"stebuklinga_lazdelė", "1FA84"},
    {"matrioškos", "1FA86"},
    {"lobių_skrynia", "1FA8E"},
    {"planeta_su_žiedu", "1FA90"},
    {"kėdė", "1FA91"},
    {"žibalinė_lempa", "1FA94"},
    {"bandža", "1FA95"},
    {"kareiviškas_šalmas", "1FA96"},
    {"ilgas_būgnas", "1FA98"},
    {"pjūklas", "1FA9A"},
    {"kopėčios", "1FA9C"},
    {"santechninė_pompa", "1FAA0"},
    {"pelėkautai", "1FAA4"},
    {"dantų_šepetėlis", "1FAA5"},
    {"tapatybės_kortelė", "1FAAA"},
    {"išeikvota_baterija", "1FAAB"},
    {"sulankstoma_vėduoklė", "1FAAD"},
    {"plaukų_šukutės", "1FAAE"},
    {"musė", "1FAB0"},
    {"kirmėlė", "1FAB1"},
    {"tuščias_lizdas", "1FAB9"},
    {"lizdas_su_kiaušiniais", "1FABA"},
    {"medūza", "1FABC"},
    {"medis_be_lapų", "1FABE"},
    {"žąsis", "1FABF"},
    {"širdis", "1FAC0"},
    {"plaučiai", "1FAC1"},
    {"apsikabinę_žmonės", "1FAC2"},
    {"nėščias_vyras", "1FAC3"},
    {"nėščias_vyras_spalva1", "1FAC3-1F3FB"},
    {"nėščias_vyras_spalva2", "1FAC3-1F3FC"},
    {"nėščias_vyras_spalva3", "1FAC3-1F3FD"},
    {"nėščias_vyras_spalva4", "1FAC3-1F3FE"},
    {"nėščias_vyras_spalva5", "1FAC3-1F3FF"},
    {"nėščias_asmuo", "1FAC4"},
    {"nėščias_asmuo_spalva1", "1FAC4-1F3FB"},
    {"nėščias_asmuo_spalva2", "1FAC4-1F3FC"},
    {"nėščias_asmuo_spalva3", "1FAC4-1F3FD"},
    {"nėščias_asmuo_spalva4", "1FAC4-1F3FE"},
    {"nėščias_asmuo_spalva5", "1FAC4-1F3FF"},
    {"asmuo_su_karūna", "1FAC5"},
    {"asmuo_su_karūna_spalva1", "1FAC5-1F3FB"},
    {"asmuo_su_karūna_spalva2", "1FAC5-1F3FC"},
    {"asmuo_su_karūna_spalva3", "1FAC5-1F3FD"},
    {"asmuo_su_karūna_spalva4", "1FAC5-1F3FE"},
    {"asmuo_su_karūna_spalva5", "1FAC5-1F3FF"},
    {"piršto_atspaudas", "1FAC6"},
    {"mėlynės", "1FAD0"},
    {"alyvuogė", "1FAD2"},
    {"pupelės", "1FAD8"},
    {"imbiero_šaknis", "1FADA"},
    {"žirnių_ankštis", "1FADB"},
    {"šakninė_daržovė", "1FADC"},
    {"tiškalas", "1FADF"},
    {"pagarbą_atiduodantis_veidas", "1FAE1"},
    {"veidas_atmerktomis_akimis_ir_ranka_uždengta_burna", "1FAE2"},
    {"veidas_su_stebinčia_akimi", "1FAE3"},
    {"veidas_įstriža_burna", "1FAE4"},
    {"prikąsta_lūpa", "1FAE6"},
    {"veidas_su_maišeliais_po_akimis", "1FAE9"},
    {"ištįsęs_veidas", "1FAEA"},
    {"ranka_sukryžiuotu_smiliumi_ir_nykščiu", "1FAF0"},
    {"ranka_sukryžiuotu_smiliumi_ir_nykščiu_spalva1", "1FAF0-1F3FB"},
    {"ranka_sukryžiuotu_smiliumi_ir_nykščiu_spalva2", "1FAF0-1F3FC"},
    {"ranka_sukryžiuotu_smiliumi_ir_nykščiu_spalva3", "1FAF0-1F3FD"},
    {"ranka_sukryžiuotu_smiliumi_ir_nykščiu_spalva4", "1FAF0-1F3FE"},
    {"ranka_sukryžiuotu_smiliumi_ir_nykščiu_spalva5", "1FAF0-1F3FF"},
    {"į_dešinę_rodanti_ranka", "1FAF1"},
    {"į_dešinę_rodanti_ranka_spalva1", "1FAF1-1F3FB"},
    {"į_dešinę_rodanti_ranka_spalva2", "1FAF1-1F3FC"},
    {"į_dešinę_rodanti_ranka_spalva3", "1FAF1-1F3FD"},
    {"į_dešinę_rodanti_ranka_spalva4", "1FAF1-1F3FE"},
    {"į_dešinę_rodanti_ranka_spalva5", "1FAF1-1F3FF"},
    {"į_kairę_rodanti_ranka", "1FAF2"},
    {"į_kairę_rodanti_ranka_spalva1", "1FAF2-1F3FB"},
    {"į_kairę_rodanti_ranka_spalva2", "1FAF2-1F3FC"},
    {"į_kairę_rodanti_ranka_spalva3", "1FAF2-1F3FD"},
    {"į_kairę_rodanti_ranka_spalva4", "1FAF2-1F3FE"},
    {"į_kairę_rodanti_ranka_spalva5", "1FAF2-1F3FF"},
    {"ranka_delnu_žemyn", "1FAF3"},
    {"ranka_delnu_žemyn_spalva1", "1FAF3-1F3FB"},
    {"ranka_delnu_žemyn_spalva2", "1FAF3-1F3FC"},
    {"ranka_delnu_žemyn_spalva3", "1FAF3-1F3FD"},
    {"ranka_delnu_žemyn_spalva4", "1FAF3-1F3FE"},
    {"ranka_delnu_žemyn_spalva5", "1FAF3-1F3FF"},
    {"ranka_delnu_į_viršų", "1FAF4"},
    {"ranka_delnu_į_viršų_spalva1", "1FAF4-1F3FB"},
    {"ranka_delnu_į_viršų_spalva2", "1FAF4-1F3FC"},
    {"ranka_delnu_į_viršų_spalva3", "1FAF4-1F3FD"},
    {"ranka_delnu_į_viršų_spalva4", "1FAF4-1F3FE"},
    {"ranka_delnu_į_viršų_spalva5", "1FAF4-1F3FF"},
    {"į_žiūrovą_rodantis_smilius", "1FAF5"},
    {"į_žiūrovą_rodantis_smilius_spalva1", "1FAF5-1F3FB"},
    {"į_žiūrovą_rodantis_smilius_spalva2", "1FAF5-1F3FC"},
    {"į_žiūrovą_rodantis_smilius_spalva3", "1FAF5-1F3FD"},
    {"į_žiūrovą_rodantis_smilius_spalva4", "1FAF5-1F3FE"},
    {"į_žiūrovą_rodantis_smilius_spalva5", "1FAF5-1F3FF"},
    {"širdelė_iš_rankų", "1FAF6"},
    {"širdelė_iš_rankų_spalva1", "1FAF6-1F3FB"},
    {"širdelė_iš_rankų_spalva2", "1FAF6-1F3FC"},
    {"širdelė_iš_rankų_spalva3", "1FAF6-1F3FD"},
    {"širdelė_iš_rankų_spalva4", "1FAF6-1F3FE"},
    {"širdelė_iš_rankų_spalva5", "1FAF6-1F3FF"},
    {"į_kairę_stumianti_ranka", "1FAF7"},
    {"į_kairę_stumianti_ranka_spalva1", "1FAF7-1F3FB"},
    {"į_kairę_stumianti_ranka_spalva2", "1FAF7-1F3FC"},
    {"į_kairę_stumianti_ranka_spalva3", "1FAF7-1F3FD"},
    {"į_kairę_stumianti_ranka_spalva4", "1FAF7-1F3FE"},
    {"į_kairę_stumianti_ranka_spalva5", "1FAF7-1F3FF"},
    {"į_dešinę_stumianti_ranka", "1FAF8"},
    {"į_dešinę_stumianti_ranka_spalva1", "1FAF8-1F3FB"},
    {"į_dešinę_stumianti_ranka_spalva2", "1FAF8-1F3FC"},
    {"į_dešinę_stumianti_ranka_spalva3", "1FAF8-1F3FD"},
    {"į_dešinę_stumianti_ranka_spalva4", "1FAF8-1F3FE"},
    {"į_dešinę_stumianti_ranka_spalva5", "1FAF8-1F3FF"},
    {"mygtukas_numerio_ženklas", "0023-FE0F-20E3"},
    {"mygtukas_žvaigždute", "002A-FE0F-20E3"},
    {"vėliava_dangun_žengimo_sala", "1F1E6-1F1E8"},
    {"vėliava_andora", "1F1E6-1F1E9"},
    {"vėliava_jungtiniai_arabų_emyratai", "1F1E6-1F1EA"},
    {"vėliava_afganistanas", "1F1E6-1F1EB"},
    {"vėliava_antigva_ir_barbuda", "1F1E6-1F1EC"},
    {"vėliava_angilija", "1F1E6-1F1EE"},
    {"vėliava_albanija", "1F1E6-1F1F1"},
    {"vėliava_armėnija", "1F1E6-1F1F2"},
    {"vėliava_angola", "1F1E6-1F1F4"},
    {"vėliava_antarktida", "1F1E6-1F1F6"},
    {"vėliava_argentina", "1F1E6-1F1F7"},
    {"vėliava_amerikos_samoa", "1F1E6-1F1F8"},
    {"vėliava_austrija", "1F1E6-1F1F9"},
    {"vėliava_australija", "1F1E6-1F1FA"},
    {"vėliava_aruba", "1F1E6-1F1FC"},
    {"vėliava_alandų_salos", "1F1E6-1F1FD"},
    {"vėliava_azerbaidžanas", "1F1E6-1F1FF"},
    {"vėliava_bosnija_ir_hercegovina", "1F1E7-1F1E6"},
    {"vėliava_barbadosas", "1F1E7-1F1E7"},
    {"vėliava_bangladešas", "1F1E7-1F1E9"},
    {"vėliava_belgija", "1F1E7-1F1EA"},
    {"vėliava_burkina_fasas", "1F1E7-1F1EB"},
    {"vėliava_bulgarija", "1F1E7-1F1EC"},
    {"vėliava_bahreinas", "1F1E7-1F1ED"},
    {"vėliava_burundis", "1F1E7-1F1EE"},
    {"vėliava_beninas", "1F1E7-1F1EF"},
    {"vėliava_sen_bartelemi", "1F1E7-1F1F1"},
    {"vėliava_bermuda", "1F1E7-1F1F2"},
    {"vėliava_brunėjus", "1F1E7-1F1F3"},
    {"vėliava_bolivija", "1F1E7-1F1F4"},
    {"vėliava_karibų_nyderlandai", "1F1E7-1F1F6"},
    {"vėliava_brazilija", "1F1E7-1F1F7"},
    {"vėliava_bahamos", "1F1E7-1F1F8"},
    {"vėliava_butanas", "1F1E7-1F1F9"},
    {"vėliava_buvė_sala", "1F1E7-1F1FB"},
    {"vėliava_botsvana", "1F1E7-1F1FC"},
    {"vėliava_baltarusija", "1F1E7-1F1FE"},
    {"vėliava_belizas", "1F1E7-1F1FF"},
    {"vėliava_kanada", "1F1E8-1F1E6"},
    {"vėliava_kokosų_kilingo_salos", "1F1E8-1F1E8"},
    {"vėliava_kongas_kinšasa", "1F1E8-1F1E9"},
    {"vėliava_centrinės_afrikos_respublika", "1F1E8-1F1EB"},
    {"vėliava_kongas_brazavilis", "1F1E8-1F1EC"},
    {"vėliava_šveicarija", "1F1E8-1F1ED"},
    {"vėliava_dramblio_kaulo_krantas", "1F1E8-1F1EE"},
    {"vėliava_kuko_salos", "1F1E8-1F1F0"},
    {"vėliava_čilė", "1F1E8-1F1F1"},
    {"vėliava_kamerūnas", "1F1E8-1F1F2"},
    {"vėliava_kinija", "1F1E8-1F1F3"},
    {"vėliava_kolumbija", "1F1E8-1F1F4"},
    {"vėliava_klipertono_sala", "1F1E8-1F1F5"},
    {"vėliava_sarkas", "1F1E8-1F1F6"},
    {"vėliava_kosta_rika", "1F1E8-1F1F7"},
    {"vėliava_kuba", "1F1E8-1F1FA"},
    {"vėliava_žaliasis_kyšulys", "1F1E8-1F1FB"},
    {"vėliava_kiurasao", "1F1E8-1F1FC"},
    {"vėliava_kalėdų_sala", "1F1E8-1F1FD"},
    {"vėliava_kipras", "1F1E8-1F1FE"},
    {"vėliava_čekija", "1F1E8-1F1FF"},
    {"vėliava_vokietija", "1F1E9-1F1EA"},
    {"vėliava_diego_garsija", "1F1E9-1F1EC"},
    {"vėliava_džibutis", "1F1E9-1F1EF"},
    {"vėliava_danija", "1F1E9-1F1F0"},
    {"vėliava_dominika", "1F1E9-1F1F2"},
    {"vėliava_dominikos_respublika", "1F1E9-1F1F4"},
    {"vėliava_alžyras", "1F1E9-1F1FF"},
    {"vėliava_seuta_ir_melila", "1F1EA-1F1E6"},
    {"vėliava_ekvadoras", "1F1EA-1F1E8"},
    {"vėliava_estija", "1F1EA-1F1EA"},
    {"vėliava_egiptas", "1F1EA-1F1EC"},
    {"vėliava_vakarų_sachara", "1F1EA-1F1ED"},
    {"vėliava_eritrėja", "1F1EA-1F1F7"},
    {"vėliava_ispanija", "1F1EA-1F1F8"},
    {"vėliava_etiopija", "1F1EA-1F1F9"},
    {"vėliava_europos_sąjunga", "1F1EA-1F1FA"},
    {"vėliava_suomija", "1F1EB-1F1EE"},
    {"vėliava_fidžis", "1F1EB-1F1EF"},
    {"vėliava_folklando_salos", "1F1EB-1F1F0"},
    {"vėliava_mikronezija", "1F1EB-1F1F2"},
    {"vėliava_farerų_salos", "1F1EB-1F1F4"},
    {"vėliava_prancūzija", "1F1EB-1F1F7"},
    {"vėliava_gabonas", "1F1EC-1F1E6"},
    {"vėliava_jungtinė_karalystė", "1F1EC-1F1E7"},
    {"vėliava_grenada", "1F1EC-1F1E9"},
    {"vėliava_gruzija", "1F1EC-1F1EA"},
    {"vėliava_prancūzijos_gviana", "1F1EC-1F1EB"},
    {"vėliava_gernsis", "1F1EC-1F1EC"},
    {"vėliava_gana", "1F1EC-1F1ED"},
    {"vėliava_gibraltaras", "1F1EC-1F1EE"},
    {"vėliava_grenlandija", "1F1EC-1F1F1"},
    {"vėliava_gambija", "1F1EC-1F1F2"},
    {"vėliava_gvinėja", "1F1EC-1F1F3"},
    {"vėliava_gvadelupa", "1F1EC-1F1F5"},
    {"vėliava_pusiaujo_gvinėja", "1F1EC-1F1F6"},
    {"vėliava_graikija", "1F1EC-1F1F7"},
    {"vėliava_pietų_džordžija_ir_pietų_sandvičo_salos", "1F1EC-1F1F8"},
    {"vėliava_gvatemala", "1F1EC-1F1F9"},
    {"vėliava_guamas", "1F1EC-1F1FA"},
    {"vėliava_bisau_gvinėja", "1F1EC-1F1FC"},
    {"vėliava_gajana", "1F1EC-1F1FE"},
    {"vėliava_ypatingasis_administracinis_kinijos_regionas_honkongas", "1F1ED-1F1F0"},
    {"vėliava_herdo_ir_makdonaldo_salos", "1F1ED-1F1F2"},
    {"vėliava_hondūras", "1F1ED-1F1F3"},
    {"vėliava_kroatija", "1F1ED-1F1F7"},
    {"vėliava_haitis", "1F1ED-1F1F9"},
    {"vėliava_vengrija", "1F1ED-1F1FA"},
    {"vėliava_kanarų_salos", "1F1EE-1F1E8"},
    {"vėliava_indonezija", "1F1EE-1F1E9"},
    {"vėliava_airija", "1F1EE-1F1EA"},
    {"vėliava_izraelis", "1F1EE-1F1F1"},
    {"vėliava_meno_sala", "1F1EE-1F1F2"},
    {"vėliava_indija", "1F1EE-1F1F3"},
    {"vėliava_indijos_vandenyno_britų_sritis", "1F1EE-1F1F4"},
    {"vėliava_irakas", "1F1EE-1F1F6"},
    {"vėliava_iranas", "1F1EE-1F1F7"},
    {"vėliava_islandija", "1F1EE-1F1F8"},
    {"vėliava_italija", "1F1EE-1F1F9"},
    {"vėliava_džersis", "1F1EF-1F1EA"},
    {"vėliava_jamaika", "1F1EF-1F1F2"},
    {"vėliava_jordanija", "1F1EF-1F1F4"},
    {"vėliava_japonija", "1F1EF-1F1F5"},
    {"vėliava_kenija", "1F1F0-1F1EA"},
    {"vėliava_kirgizija", "1F1F0-1F1EC"},
    {"vėliava_kambodža", "1F1F0-1F1ED"},
    {"vėliava_kiribatis", "1F1F0-1F1EE"},
    {"vėliava_komorai", "1F1F0-1F1F2"},
    {"vėliava_sent_kitsas_ir_nevis", "1F1F0-1F1F3"},
    {"vėliava_šiaurės_korėja", "1F1F0-1F1F5"},
    {"vėliava_pietų_korėja", "1F1F0-1F1F7"},
    {"vėliava_kuveitas", "1F1F0-1F1FC"},
    {"vėliava_kaimanų_salos", "1F1F0-1F1FE"},
    {"vėliava_kazachstanas", "1F1F0-1F1FF"},
    {"vėliava_laosas", "1F1F1-1F1E6"},
    {"vėliava_libanas", "1F1F1-1F1E7"},
    {"vėliava_sent_lusija", "1F1F1-1F1E8"},
    {"vėliava_lichtenšteinas", "1F1F1-1F1EE"},
    {"vėliava_šri_lanka", "1F1F1-1F1F0"},
    {"vėliava_liberija", "1F1F1-1F1F7"},
    {"vėliava_lesotas", "1F1F1-1F1F8"},
    {"vėliava_lietuva", "1F1F1-1F1F9"},
    {"vėliava_liuksemburgas", "1F1F1-1F1FA"},
    {"vėliava_latvija", "1F1F1-1F1FB"},
    {"vėliava_libija", "1F1F1-1F1FE"},
    {"vėliava_marokas", "1F1F2-1F1E6"},
    {"vėliava_monakas", "1F1F2-1F1E8"},
    {"vėliava_moldova", "1F1F2-1F1E9"},
    {"vėliava_juodkalnija", "1F1F2-1F1EA"},
    {"vėliava_sen_martenas", "1F1F2-1F1EB"},
    {"vėliava_madagaskaras", "1F1F2-1F1EC"},
    {"vėliava_maršalo_salos", "1F1F2-1F1ED"},
    {"vėliava_šiaurės_makedonija", "1F1F2-1F1F0"},
    {"vėliava_malis", "1F1F2-1F1F1"},
    {"vėliava_mianmaras_birma", "1F1F2-1F1F2"},
    {"vėliava_mongolija", "1F1F2-1F1F3"},
    {"vėliava_ypatingasis_administracinis_kinijos_regionas_makao", "1F1F2-1F1F4"},
    {"vėliava_marianos_šiaurinės_salos", "1F1F2-1F1F5"},
    {"vėliava_martinika", "1F1F2-1F1F6"},
    {"vėliava_mauritanija", "1F1F2-1F1F7"},
    {"vėliava_montseratas", "1F1F2-1F1F8"},
    {"vėliava_malta", "1F1F2-1F1F9"},
    {"vėliava_mauricijus", "1F1F2-1F1FA"},
    {"vėliava_maldyvai", "1F1F2-1F1FB"},
    {"vėliava_malavis", "1F1F2-1F1FC"},
    {"vėliava_meksika", "1F1F2-1F1FD"},
    {"vėliava_malaizija", "1F1F2-1F1FE"},
    {"vėliava_mozambikas", "1F1F2-1F1FF"},
    {"vėliava_namibija", "1F1F3-1F1E6"},
    {"vėliava_naujoji_kaledonija", "1F1F3-1F1E8"},
    {"vėliava_nigeris", "1F1F3-1F1EA"},
    {"vėliava_norfolko_sala", "1F1F3-1F1EB"},
    {"vėliava_nigerija", "1F1F3-1F1EC"},
    {"vėliava_nikaragva", "1F1F3-1F1EE"},
    {"vėliava_nyderlandai", "1F1F3-1F1F1"},
    {"vėliava_norvegija", "1F1F3-1F1F4"},
    {"vėliava_nepalas", "1F1F3-1F1F5"},
    {"vėliava_nauru", "1F1F3-1F1F7"},
    {"vėliava_niujė", "1F1F3-1F1FA"},
    {"vėliava_naujoji_zelandija", "1F1F3-1F1FF"},
    {"vėliava_omanas", "1F1F4-1F1F2"},
    {"vėliava_panama", "1F1F5-1F1E6"},
    {"vėliava_peru", "1F1F5-1F1EA"},
    {"vėliava_prancūzijos_polinezija", "1F1F5-1F1EB"},
    {"vėliava_papua_naujoji_gvinėja", "1F1F5-1F1EC"},
    {"vėliava_filipinai", "1F1F5-1F1ED"},
    {"vėliava_pakistanas", "1F1F5-1F1F0"},
    {"vėliava_lenkija", "1F1F5-1F1F1"},
    {"vėliava_sen_pjeras_ir_mikelonas", "1F1F5-1F1F2"},
    {"vėliava_pitkerno_salos", "1F1F5-1F1F3"},
    {"vėliava_puerto_rikas", "1F1F5-1F1F7"},
    {"vėliava_palestinos_teritorija", "1F1F5-1F1F8"},
    {"vėliava_portugalija", "1F1F5-1F1F9"},
    {"vėliava_palau", "1F1F5-1F1FC"},
    {"vėliava_paragvajus", "1F1F5-1F1FE"},
    {"vėliava_kataras", "1F1F6-1F1E6"},
    {"vėliava_reunjonas", "1F1F7-1F1EA"},
    {"vėliava_rumunija", "1F1F7-1F1F4"},
    {"vėliava_serbija", "1F1F7-1F1F8"},
    {"vėliava_rusija", "1F1F7-1F1FA"},
    {"vėliava_ruanda", "1F1F7-1F1FC"},
    {"vėliava_saudo_arabija", "1F1F8-1F1E6"},
    {"vėliava_saliamono_salos", "1F1F8-1F1E7"},
    {"vėliava_seišeliai", "1F1F8-1F1E8"},
    {"vėliava_sudanas", "1F1F8-1F1E9"},
    {"vėliava_švedija", "1F1F8-1F1EA"},
    {"vėliava_singapūras", "1F1F8-1F1EC"},
    {"vėliava_šv_elenos_sala", "1F1F8-1F1ED"},
    {"vėliava_slovėnija", "1F1F8-1F1EE"},
    {"vėliava_svalbardas_ir_janas_majenas", "1F1F8-1F1EF"},
    {"vėliava_slovakija", "1F1F8-1F1F0"},
    {"vėliava_siera_leonė", "1F1F8-1F1F1"},
    {"vėliava_san_marinas", "1F1F8-1F1F2"},
    {"vėliava_senegalas", "1F1F8-1F1F3"},
    {"vėliava_somalis", "1F1F8-1F1F4"},
    {"vėliava_surinamas", "1F1F8-1F1F7"},
    {"vėliava_pietų_sudanas", "1F1F8-1F1F8"},
    {"vėliava_san_tomė_ir_prinsipė", "1F1F8-1F1F9"},
    {"vėliava_salvadoras", "1F1F8-1F1FB"},
    {"vėliava_sint_martenas", "1F1F8-1F1FD"},
    {"vėliava_sirija", "1F1F8-1F1FE"},
    {"vėliava_svazilandas", "1F1F8-1F1FF"},
    {"vėliava_tristano_da_kunjos", "1F1F9-1F1E6"},
    {"vėliava_terkso_ir_kaikoso_salos", "1F1F9-1F1E8"},
    {"vėliava_čadas", "1F1F9-1F1E9"},
    {"vėliava_prancūzijos_pietų_sritys", "1F1F9-1F1EB"},
    {"vėliava_togas", "1F1F9-1F1EC"},
    {"vėliava_tailandas", "1F1F9-1F1ED"},
    {"vėliava_tadžikija", "1F1F9-1F1EF"},
    {"vėliava_tokelau", "1F1F9-1F1F0"},
    {"vėliava_rytų_timoras", "1F1F9-1F1F1"},
    {"vėliava_turkmėnistanas", "1F1F9-1F1F2"},
    {"vėliava_tunisas", "1F1F9-1F1F3"},
    {"vėliava_tonga", "1F1F9-1F1F4"},
    {"vėliava_turkija", "1F1F9-1F1F7"},
    {"vėliava_trinidadas_ir_tobagas", "1F1F9-1F1F9"},
    {"vėliava_tuvalu", "1F1F9-1F1FB"},
    {"vėliava_taivanas", "1F1F9-1F1FC"},
    {"vėliava_tanzanija", "1F1F9-1F1FF"},
    {"vėliava_ukraina", "1F1FA-1F1E6"},
    {"vėliava_uganda", "1F1FA-1F1EC"},
    {"vėliava_jungtinių_valstijų_mažosios_tolimosios_salos", "1F1FA-1F1F2"},
    {"vėliava_jungtinės_tautos", "1F1FA-1F1F3"},
    {"vėliava_jungtinės_valstijos", "1F1FA-1F1F8"},
    {"vėliava_urugvajus", "1F1FA-1F1FE"},
    {"vėliava_uzbekistanas", "1F1FA-1F1FF"},
    {"vėliava_vatikano_miesto_valstybė", "1F1FB-1F1E6"},
    {"vėliava_šventasis_vincentas_ir_grenadinai", "1F1FB-1F1E8"},
    {"vėliava_venesuela", "1F1FB-1F1EA"},
    {"vėliava_didžiosios_britanijos_mergelių_salos", "1F1FB-1F1EC"},
    {"vėliava_jungtinių_valstijų_mergelių_salos", "1F1FB-1F1EE"},
    {"vėliava_vietnamas", "1F1FB-1F1F3"},
    {"vėliava_vanuatu", "1F1FB-1F1FA"},
    {"vėliava_volisas_ir_futūna", "1F1FC-1F1EB"},
    {"vėliava_samoa", "1F1FC-1F1F8"},
    {"vėliava_kosovas", "1F1FD-1F1F0"},
    {"vėliava_jemenas", "1F1FE-1F1EA"},
    {"vėliava_majotas", "1F1FE-1F1F9"},
    {"vėliava_pietų_afrika", "1F1FF-1F1E6"},
    {"vėliava_zambija", "1F1FF-1F1F2"},
    {"vėliava_zimbabvė", "1F1FF-1F1FC"},
    {"vėliava_anglija", "1F3F4-E0067-E0062-E0065-E006E-E0067-E007F"},
    {"vėliava_škotija", "1F3F4-E0067-E0062-E0073-E0063-E0074-E007F"},
    {"vėliava_velsas", "1F3F4-E0067-E0062-E0077-E006C-E0073-E007F"},
    {"pora_su_širdele_vyras_vyras", "1F468-200D-2764-FE0F-200D-1F468"},
    {"pora_su_širdele_vyras_vyras_spalva1", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_širdele_vyras_vyras_spalva1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_širdele_vyras_vyras_spalva1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_širdele_vyras_vyras_spalva1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_širdele_vyras_vyras_spalva1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_širdele_vyras_vyras_spalva2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_širdele_vyras_vyras_spalva2", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_širdele_vyras_vyras_spalva2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_širdele_vyras_vyras_spalva2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_širdele_vyras_vyras_spalva2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_širdele_vyras_vyras_spalva3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_širdele_vyras_vyras_spalva3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_širdele_vyras_vyras_spalva3", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_širdele_vyras_vyras_spalva3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_širdele_vyras_vyras_spalva3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_širdele_vyras_vyras_spalva4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_širdele_vyras_vyras_spalva4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_širdele_vyras_vyras_spalva4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_širdele_vyras_vyras_spalva4", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_širdele_vyras_vyras_spalva4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_širdele_vyras_vyras_spalva5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_širdele_vyras_vyras_spalva5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_širdele_vyras_vyras_spalva5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_širdele_vyras_vyras_spalva5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_širdele_vyras_vyras_spalva5", "1F468-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"bučinys_vyras_vyras", "1F468-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"bučinys_vyras_vyras_spalva1", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bučinys_vyras_vyras_spalva1-2", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bučinys_vyras_vyras_spalva1-3", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bučinys_vyras_vyras_spalva1-4", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bučinys_vyras_vyras_spalva1-5", "1F468-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bučinys_vyras_vyras_spalva2-1", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bučinys_vyras_vyras_spalva2", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bučinys_vyras_vyras_spalva2-3", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bučinys_vyras_vyras_spalva2-4", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bučinys_vyras_vyras_spalva2-5", "1F468-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bučinys_vyras_vyras_spalva3-1", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bučinys_vyras_vyras_spalva3-2", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bučinys_vyras_vyras_spalva3", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bučinys_vyras_vyras_spalva3-4", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bučinys_vyras_vyras_spalva3-5", "1F468-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bučinys_vyras_vyras_spalva4-1", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bučinys_vyras_vyras_spalva4-2", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bučinys_vyras_vyras_spalva4-3", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bučinys_vyras_vyras_spalva4", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bučinys_vyras_vyras_spalva4-5", "1F468-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bučinys_vyras_vyras_spalva5-1", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bučinys_vyras_vyras_spalva5-2", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bučinys_vyras_vyras_spalva5-3", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bučinys_vyras_vyras_spalva5-4", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bučinys_vyras_vyras_spalva5", "1F468-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"šeima_vyras_berniukas", "1F468-200D-1F466"},
    {"šeima_vyras_berniukas_berniukas", "1F468-200D-1F466-200D-1F466"},
    {"šeima_vyras_mergaitė", "1F468-200D-1F467"},
    {"šeima_vyras_mergaitė_berniukas", "1F468-200D-1F467-200D-1F466"},
    {"šeima_vyras_mergaitė_mergaitė", "1F468-200D-1F467-200D-1F467"},
    {"šeima_vyras_vyras_berniukas", "1F468-200D-1F468-200D-1F466"},
    {"šeima_vyras_vyras_berniukas_berniukas", "1F468-200D-1F468-200D-1F466-200D-1F466"},
    {"šeima_vyras_vyras_mergaitė", "1F468-200D-1F468-200D-1F467"},
    {"šeima_vyras_vyras_mergaitė_berniukas", "1F468-200D-1F468-200D-1F467-200D-1F466"},
    {"šeima_vyras_vyras_mergaitė_mergaitė", "1F468-200D-1F468-200D-1F467-200D-1F467"},
    {"šeima_vyras_moteris_berniukas", "1F468-200D-1F469-200D-1F466"},
    {"šeima_vyras_moteris_berniukas_berniukas", "1F468-200D-1F469-200D-1F466-200D-1F466"},
    {"šeima_vyras_moteris_mergaitė", "1F468-200D-1F469-200D-1F467"},
    {"šeima_vyras_moteris_mergaitė_berniukas", "1F468-200D-1F469-200D-1F467-200D-1F466"},
    {"šeima_vyras_moteris_mergaitė_mergaitė", "1F468-200D-1F469-200D-1F467-200D-1F467"},
    {"pora_su_širdele_moteris_vyras", "1F469-200D-2764-FE0F-200D-1F468"},
    {"pora_su_širdele_moteris_vyras_spalva1", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_širdele_moteris_vyras_spalva1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_širdele_moteris_vyras_spalva1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_širdele_moteris_vyras_spalva1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_širdele_moteris_vyras_spalva1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_širdele_moteris_vyras_spalva2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_širdele_moteris_vyras_spalva2", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_širdele_moteris_vyras_spalva2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_širdele_moteris_vyras_spalva2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_širdele_moteris_vyras_spalva2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_širdele_moteris_vyras_spalva3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_širdele_moteris_vyras_spalva3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_širdele_moteris_vyras_spalva3", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_širdele_moteris_vyras_spalva3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_širdele_moteris_vyras_spalva3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_širdele_moteris_vyras_spalva4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_širdele_moteris_vyras_spalva4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_širdele_moteris_vyras_spalva4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_širdele_moteris_vyras_spalva4", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_širdele_moteris_vyras_spalva4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_širdele_moteris_vyras_spalva5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FB"},
    {"pora_su_širdele_moteris_vyras_spalva5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FC"},
    {"pora_su_širdele_moteris_vyras_spalva5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FD"},
    {"pora_su_širdele_moteris_vyras_spalva5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FE"},
    {"pora_su_širdele_moteris_vyras_spalva5", "1F469-1F3FF-200D-2764-FE0F-200D-1F468-1F3FF"},
    {"pora_su_širdele_moteris_moteris", "1F469-200D-2764-FE0F-200D-1F469"},
    {"pora_su_širdele_moteris_moteris_spalva1", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pora_su_širdele_moteris_moteris_spalva1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pora_su_širdele_moteris_moteris_spalva1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pora_su_širdele_moteris_moteris_spalva1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pora_su_širdele_moteris_moteris_spalva1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pora_su_širdele_moteris_moteris_spalva2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pora_su_širdele_moteris_moteris_spalva2", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pora_su_širdele_moteris_moteris_spalva2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pora_su_širdele_moteris_moteris_spalva2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pora_su_širdele_moteris_moteris_spalva2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pora_su_širdele_moteris_moteris_spalva3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pora_su_širdele_moteris_moteris_spalva3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pora_su_širdele_moteris_moteris_spalva3", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pora_su_širdele_moteris_moteris_spalva3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pora_su_širdele_moteris_moteris_spalva3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pora_su_širdele_moteris_moteris_spalva4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pora_su_širdele_moteris_moteris_spalva4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pora_su_širdele_moteris_moteris_spalva4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pora_su_širdele_moteris_moteris_spalva4", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pora_su_širdele_moteris_moteris_spalva4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"pora_su_širdele_moteris_moteris_spalva5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FB"},
    {"pora_su_širdele_moteris_moteris_spalva5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FC"},
    {"pora_su_širdele_moteris_moteris_spalva5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FD"},
    {"pora_su_širdele_moteris_moteris_spalva5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FE"},
    {"pora_su_širdele_moteris_moteris_spalva5", "1F469-1F3FF-200D-2764-FE0F-200D-1F469-1F3FF"},
    {"bučinys_moteris_vyras", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F468"},
    {"bučinys_moteris_vyras_spalva1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bučinys_moteris_vyras_spalva1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bučinys_moteris_vyras_spalva1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bučinys_moteris_vyras_spalva1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bučinys_moteris_vyras_spalva1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bučinys_moteris_vyras_spalva2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bučinys_moteris_vyras_spalva2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bučinys_moteris_vyras_spalva2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bučinys_moteris_vyras_spalva2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bučinys_moteris_vyras_spalva2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bučinys_moteris_vyras_spalva3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bučinys_moteris_vyras_spalva3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bučinys_moteris_vyras_spalva3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bučinys_moteris_vyras_spalva3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bučinys_moteris_vyras_spalva3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bučinys_moteris_vyras_spalva4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bučinys_moteris_vyras_spalva4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bučinys_moteris_vyras_spalva4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bučinys_moteris_vyras_spalva4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bučinys_moteris_vyras_spalva4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bučinys_moteris_vyras_spalva5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FB"},
    {"bučinys_moteris_vyras_spalva5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FC"},
    {"bučinys_moteris_vyras_spalva5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FD"},
    {"bučinys_moteris_vyras_spalva5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FE"},
    {"bučinys_moteris_vyras_spalva5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F468-1F3FF"},
    {"bučinys_moteris_moteris", "1F469-200D-2764-FE0F-200D-1F48B-200D-1F469"},
    {"bučinys_moteris_moteris_spalva1", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bučinys_moteris_moteris_spalva1-2", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bučinys_moteris_moteris_spalva1-3", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bučinys_moteris_moteris_spalva1-4", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bučinys_moteris_moteris_spalva1-5", "1F469-1F3FB-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bučinys_moteris_moteris_spalva2-1", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bučinys_moteris_moteris_spalva2", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bučinys_moteris_moteris_spalva2-3", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bučinys_moteris_moteris_spalva2-4", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bučinys_moteris_moteris_spalva2-5", "1F469-1F3FC-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bučinys_moteris_moteris_spalva3-1", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bučinys_moteris_moteris_spalva3-2", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bučinys_moteris_moteris_spalva3", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bučinys_moteris_moteris_spalva3-4", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bučinys_moteris_moteris_spalva3-5", "1F469-1F3FD-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bučinys_moteris_moteris_spalva4-1", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bučinys_moteris_moteris_spalva4-2", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bučinys_moteris_moteris_spalva4-3", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bučinys_moteris_moteris_spalva4", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bučinys_moteris_moteris_spalva4-5", "1F469-1F3FE-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"bučinys_moteris_moteris_spalva5-1", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FB"},
    {"bučinys_moteris_moteris_spalva5-2", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FC"},
    {"bučinys_moteris_moteris_spalva5-3", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FD"},
    {"bučinys_moteris_moteris_spalva5-4", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FE"},
    {"bučinys_moteris_moteris_spalva5", "1F469-1F3FF-200D-2764-FE0F-200D-1F48B-200D-1F469-1F3FF"},
    {"šeima_moteris_berniukas", "1F469-200D-1F466"},
    {"šeima_moteris_berniukas_berniukas", "1F469-200D-1F466-200D-1F466"},
    {"šeima_moteris_mergaitė", "1F469-200D-1F467"},
    {"šeima_moteris_mergaitė_berniukas", "1F469-200D-1F467-200D-1F466"},
    {"šeima_moteris_mergaitė_mergaitė", "1F469-200D-1F467-200D-1F467"},
    {"šeima_moteris_moteris_berniukas", "1F469-200D-1F469-200D-1F466"},
    {"šeima_moteris_moteris_berniukas_berniukas", "1F469-200D-1F469-200D-1F466-200D-1F466"},
    {"šeima_moteris_moteris_mergaitė", "1F469-200D-1F469-200D-1F467"},
    {"šeima_moteris_moteris_mergaitė_berniukas", "1F469-200D-1F469-200D-1F467-200D-1F466"},
    {"šeima_moteris_moteris_mergaitė_mergaitė", "1F469-200D-1F469-200D-1F467-200D-1F467"},
    {"rankomis_susikibę_žmonės", "1F9D1-200D-1F91D-200D-1F9D1"},
    {"rankomis_susikibę_žmonės_spalva1", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FB"},
    {"rankomis_susikibę_žmonės_spalva1-2", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FC"},
    {"rankomis_susikibę_žmonės_spalva1-3", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FD"},
    {"rankomis_susikibę_žmonės_spalva1-4", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FE"},
    {"rankomis_susikibę_žmonės_spalva1-5", "1F9D1-1F3FB-200D-1F91D-200D-1F9D1-1F3FF"},
    {"rankomis_susikibę_žmonės_spalva2-1", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FB"},
    {"rankomis_susikibę_žmonės_spalva2", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FC"},
    {"rankomis_susikibę_žmonės_spalva2-3", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FD"},
    {"rankomis_susikibę_žmonės_spalva2-4", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FE"},
    {"rankomis_susikibę_žmonės_spalva2-5", "1F9D1-1F3FC-200D-1F91D-200D-1F9D1-1F3FF"},
    {"rankomis_susikibę_žmonės_spalva3-1", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FB"},
    {"rankomis_susikibę_žmonės_spalva3-2", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FC"},
    {"rankomis_susikibę_žmonės_spalva3", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FD"},
    {"rankomis_susikibę_žmonės_spalva3-4", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FE"},
    {"rankomis_susikibę_žmonės_spalva3-5", "1F9D1-1F3FD-200D-1F91D-200D-1F9D1-1F3FF"},
    {"rankomis_susikibę_žmonės_spalva4-1", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FB"},
    {"rankomis_susikibę_žmonės_spalva4-2", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FC"},
    {"rankomis_susikibę_žmonės_spalva4-3", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FD"},
    {"rankomis_susikibę_žmonės_spalva4", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FE"},
    {"rankomis_susikibę_žmonės_spalva4-5", "1F9D1-1F3FE-200D-1F91D-200D-1F9D1-1F3FF"},
    {"rankomis_susikibę_žmonės_spalva5-1", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FB"},
    {"rankomis_susikibę_žmonės_spalva5-2", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FC"},
    {"rankomis_susikibę_žmonės_spalva5-3", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FD"},
    {"rankomis_susikibę_žmonės_spalva5-4", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FE"},
    {"rankomis_susikibę_žmonės_spalva5", "1F9D1-1F3FF-200D-1F91D-200D-1F9D1-1F3FF"},
    {"šeima_suaugusysis_suaugusysis_vaikas", "1F9D1-200D-1F9D1-200D-1F9D2"},
    {"šeima_suaugusysis_suaugusysis_vaikas_vaikas", "1F9D1-200D-1F9D1-200D-1F9D2-200D-1F9D2"},
    {"šeima_suaugusysis_vaikas", "1F9D1-200D-1F9D2"},
    {"šeima_suaugusysis_vaikas_vaikas", "1F9D1-200D-1F9D2-200D-1F9D2"},
    {"bėgikas_atsisukęs_į_dešinę", "1F3C3-200D-27A1-FE0F"},
    {"bėgikas_atsisukęs_į_dešinę_spalva1", "1F3C3-1F3FB-200D-27A1-FE0F"},
    {"bėgikas_atsisukęs_į_dešinę_spalva2", "1F3C3-1F3FC-200D-27A1-FE0F"},
    {"bėgikas_atsisukęs_į_dešinę_spalva3", "1F3C3-1F3FD-200D-27A1-FE0F"},
    {"bėgikas_atsisukęs_į_dešinę_spalva4", "1F3C3-1F3FE-200D-27A1-FE0F"},
    {"bėgikas_atsisukęs_į_dešinę_spalva5", "1F3C3-1F3FF-200D-27A1-FE0F"},
    {"teisėjas", "1F468-200D-2696-FE0F"},
    {"teisėjas_spalva1", "1F468-1F3FB-200D-2696-FE0F"},
    {"teisėjas_spalva2", "1F468-1F3FC-200D-2696-FE0F"},
    {"teisėjas_spalva3", "1F468-1F3FD-200D-2696-FE0F"},
    {"teisėjas_spalva4", "1F468-1F3FE-200D-2696-FE0F"},
    {"teisėjas_spalva5", "1F468-1F3FF-200D-2696-FE0F"},
    {"ūkininkas", "1F468-200D-1F33E"},
    {"ūkininkas_spalva1", "1F468-1F3FB-200D-1F33E"},
    {"ūkininkas_spalva2", "1F468-1F3FC-200D-1F33E"},
    {"ūkininkas_spalva3", "1F468-1F3FD-200D-1F33E"},
    {"ūkininkas_spalva4", "1F468-1F3FE-200D-1F33E"},
    {"ūkininkas_spalva5", "1F468-1F3FF-200D-1F33E"},
    {"virėjas", "1F468-200D-1F373"},
    {"virėjas_spalva1", "1F468-1F3FB-200D-1F373"},
    {"virėjas_spalva2", "1F468-1F3FC-200D-1F373"},
    {"virėjas_spalva3", "1F468-1F3FD-200D-1F373"},
    {"virėjas_spalva4", "1F468-1F3FE-200D-1F373"},
    {"virėjas_spalva5", "1F468-1F3FF-200D-1F373"},
    {"kūdikį_maitinantis_vyras", "1F468-200D-1F37C"},
    {"kūdikį_maitinantis_vyras_spalva1", "1F468-1F3FB-200D-1F37C"},
    {"kūdikį_maitinantis_vyras_spalva2", "1F468-1F3FC-200D-1F37C"},
    {"kūdikį_maitinantis_vyras_spalva3", "1F468-1F3FD-200D-1F37C"},
    {"kūdikį_maitinantis_vyras_spalva4", "1F468-1F3FE-200D-1F37C"},
    {"kūdikį_maitinantis_vyras_spalva5", "1F468-1F3FF-200D-1F37C"},
    {"vyras_su_neregio_lazdele_atsisukęs_į_dešinę", "1F468-200D-1F9AF-200D-27A1-FE0F"},
    {"vyras_su_neregio_lazdele_atsisukęs_į_dešinę_spalva1", "1F468-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"vyras_su_neregio_lazdele_atsisukęs_į_dešinę_spalva2", "1F468-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"vyras_su_neregio_lazdele_atsisukęs_į_dešinę_spalva3", "1F468-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"vyras_su_neregio_lazdele_atsisukęs_į_dešinę_spalva4", "1F468-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"vyras_su_neregio_lazdele_atsisukęs_į_dešinę_spalva5", "1F468-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"vyras_motorizuotame_vežimėlyje", "1F468-200D-1F9BC"},
    {"vyras_motorizuotame_vežimėlyje_spalva1", "1F468-1F3FB-200D-1F9BC"},
    {"vyras_motorizuotame_vežimėlyje_spalva2", "1F468-1F3FC-200D-1F9BC"},
    {"vyras_motorizuotame_vežimėlyje_spalva3", "1F468-1F3FD-200D-1F9BC"},
    {"vyras_motorizuotame_vežimėlyje_spalva4", "1F468-1F3FE-200D-1F9BC"},
    {"vyras_motorizuotame_vežimėlyje_spalva5", "1F468-1F3FF-200D-1F9BC"},
    {"vyras_motorizuotame_vežimėlyje_atsisukęs_į_dešinę", "1F468-200D-1F9BC-200D-27A1-FE0F"},
    {"vyras_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva1", "1F468-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"vyras_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva2", "1F468-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"vyras_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva3", "1F468-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"vyras_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva4", "1F468-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"vyras_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva5", "1F468-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"vyras_rankomis_valdomame_vežimėlyje", "1F468-200D-1F9BD"},
    {"vyras_rankomis_valdomame_vežimėlyje_spalva1", "1F468-1F3FB-200D-1F9BD"},
    {"vyras_rankomis_valdomame_vežimėlyje_spalva2", "1F468-1F3FC-200D-1F9BD"},
    {"vyras_rankomis_valdomame_vežimėlyje_spalva3", "1F468-1F3FD-200D-1F9BD"},
    {"vyras_rankomis_valdomame_vežimėlyje_spalva4", "1F468-1F3FE-200D-1F9BD"},
    {"vyras_rankomis_valdomame_vežimėlyje_spalva5", "1F468-1F3FF-200D-1F9BD"},
    {"vyras_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę", "1F468-200D-1F9BD-200D-27A1-FE0F"},
    {"vyras_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva1", "1F468-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"vyras_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva2", "1F468-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"vyras_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva3", "1F468-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"vyras_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva4", "1F468-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"vyras_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva5", "1F468-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"medikė", "1F469-200D-2695-FE0F"},
    {"medikė_spalva1", "1F469-1F3FB-200D-2695-FE0F"},
    {"medikė_spalva2", "1F469-1F3FC-200D-2695-FE0F"},
    {"medikė_spalva3", "1F469-1F3FD-200D-2695-FE0F"},
    {"medikė_spalva4", "1F469-1F3FE-200D-2695-FE0F"},
    {"medikė_spalva5", "1F469-1F3FF-200D-2695-FE0F"},
    {"teisėja", "1F469-200D-2696-FE0F"},
    {"teisėja_spalva1", "1F469-1F3FB-200D-2696-FE0F"},
    {"teisėja_spalva2", "1F469-1F3FC-200D-2696-FE0F"},
    {"teisėja_spalva3", "1F469-1F3FD-200D-2696-FE0F"},
    {"teisėja_spalva4", "1F469-1F3FE-200D-2696-FE0F"},
    {"teisėja_spalva5", "1F469-1F3FF-200D-2696-FE0F"},
    {"pilotė", "1F469-200D-2708-FE0F"},
    {"pilotė_spalva1", "1F469-1F3FB-200D-2708-FE0F"},
    {"pilotė_spalva2", "1F469-1F3FC-200D-2708-FE0F"},
    {"pilotė_spalva3", "1F469-1F3FD-200D-2708-FE0F"},
    {"pilotė_spalva4", "1F469-1F3FE-200D-2708-FE0F"},
    {"pilotė_spalva5", "1F469-1F3FF-200D-2708-FE0F"},
    {"ūkininkė", "1F469-200D-1F33E"},
    {"ūkininkė_spalva1", "1F469-1F3FB-200D-1F33E"},
    {"ūkininkė_spalva2", "1F469-1F3FC-200D-1F33E"},
    {"ūkininkė_spalva3", "1F469-1F3FD-200D-1F33E"},
    {"ūkininkė_spalva4", "1F469-1F3FE-200D-1F33E"},
    {"ūkininkė_spalva5", "1F469-1F3FF-200D-1F33E"},
    {"virėja", "1F469-200D-1F373"},
    {"virėja_spalva1", "1F469-1F3FB-200D-1F373"},
    {"virėja_spalva2", "1F469-1F3FC-200D-1F373"},
    {"virėja_spalva3", "1F469-1F3FD-200D-1F373"},
    {"virėja_spalva4", "1F469-1F3FE-200D-1F373"},
    {"virėja_spalva5", "1F469-1F3FF-200D-1F373"},
    {"kūdikį_maitinanti_moteris", "1F469-200D-1F37C"},
    {"kūdikį_maitinanti_moteris_spalva1", "1F469-1F3FB-200D-1F37C"},
    {"kūdikį_maitinanti_moteris_spalva2", "1F469-1F3FC-200D-1F37C"},
    {"kūdikį_maitinanti_moteris_spalva3", "1F469-1F3FD-200D-1F37C"},
    {"kūdikį_maitinanti_moteris_spalva4", "1F469-1F3FE-200D-1F37C"},
    {"kūdikį_maitinanti_moteris_spalva5", "1F469-1F3FF-200D-1F37C"},
    {"studentė", "1F469-200D-1F393"},
    {"studentė_spalva1", "1F469-1F3FB-200D-1F393"},
    {"studentė_spalva2", "1F469-1F3FC-200D-1F393"},
    {"studentė_spalva3", "1F469-1F3FD-200D-1F393"},
    {"studentė_spalva4", "1F469-1F3FE-200D-1F393"},
    {"studentė_spalva5", "1F469-1F3FF-200D-1F393"},
    {"dainininkė", "1F469-200D-1F3A4"},
    {"dainininkė_spalva1", "1F469-1F3FB-200D-1F3A4"},
    {"dainininkė_spalva2", "1F469-1F3FC-200D-1F3A4"},
    {"dainininkė_spalva3", "1F469-1F3FD-200D-1F3A4"},
    {"dainininkė_spalva4", "1F469-1F3FE-200D-1F3A4"},
    {"dainininkė_spalva5", "1F469-1F3FF-200D-1F3A4"},
    {"dailininkė", "1F469-200D-1F3A8"},
    {"dailininkė_spalva1", "1F469-1F3FB-200D-1F3A8"},
    {"dailininkė_spalva2", "1F469-1F3FC-200D-1F3A8"},
    {"dailininkė_spalva3", "1F469-1F3FD-200D-1F3A8"},
    {"dailininkė_spalva4", "1F469-1F3FE-200D-1F3A8"},
    {"dailininkė_spalva5", "1F469-1F3FF-200D-1F3A8"},
    {"fabriko_darbininkė", "1F469-200D-1F3ED"},
    {"fabriko_darbininkė_spalva1", "1F469-1F3FB-200D-1F3ED"},
    {"fabriko_darbininkė_spalva2", "1F469-1F3FC-200D-1F3ED"},
    {"fabriko_darbininkė_spalva3", "1F469-1F3FD-200D-1F3ED"},
    {"fabriko_darbininkė_spalva4", "1F469-1F3FE-200D-1F3ED"},
    {"fabriko_darbininkė_spalva5", "1F469-1F3FF-200D-1F3ED"},
    {"technologė", "1F469-200D-1F4BB"},
    {"technologė_spalva1", "1F469-1F3FB-200D-1F4BB"},
    {"technologė_spalva2", "1F469-1F3FC-200D-1F4BB"},
    {"technologė_spalva3", "1F469-1F3FD-200D-1F4BB"},
    {"technologė_spalva4", "1F469-1F3FE-200D-1F4BB"},
    {"technologė_spalva5", "1F469-1F3FF-200D-1F4BB"},
    {"mechanikė", "1F469-200D-1F527"},
    {"mechanikė_spalva1", "1F469-1F3FB-200D-1F527"},
    {"mechanikė_spalva2", "1F469-1F3FC-200D-1F527"},
    {"mechanikė_spalva3", "1F469-1F3FD-200D-1F527"},
    {"mechanikė_spalva4", "1F469-1F3FE-200D-1F527"},
    {"mechanikė_spalva5", "1F469-1F3FF-200D-1F527"},
    {"mokslininkė", "1F469-200D-1F52C"},
    {"mokslininkė_spalva1", "1F469-1F3FB-200D-1F52C"},
    {"mokslininkė_spalva2", "1F469-1F3FC-200D-1F52C"},
    {"mokslininkė_spalva3", "1F469-1F3FD-200D-1F52C"},
    {"mokslininkė_spalva4", "1F469-1F3FE-200D-1F52C"},
    {"mokslininkė_spalva5", "1F469-1F3FF-200D-1F52C"},
    {"astronautė", "1F469-200D-1F680"},
    {"astronautė_spalva1", "1F469-1F3FB-200D-1F680"},
    {"astronautė_spalva2", "1F469-1F3FC-200D-1F680"},
    {"astronautė_spalva3", "1F469-1F3FD-200D-1F680"},
    {"astronautė_spalva4", "1F469-1F3FE-200D-1F680"},
    {"astronautė_spalva5", "1F469-1F3FF-200D-1F680"},
    {"gaisrininkė", "1F469-200D-1F692"},
    {"gaisrininkė_spalva1", "1F469-1F3FB-200D-1F692"},
    {"gaisrininkė_spalva2", "1F469-1F3FC-200D-1F692"},
    {"gaisrininkė_spalva3", "1F469-1F3FD-200D-1F692"},
    {"gaisrininkė_spalva4", "1F469-1F3FE-200D-1F692"},
    {"gaisrininkė_spalva5", "1F469-1F3FF-200D-1F692"},
    {"moteris_su_neregio_lazdele_atsisukęs_į_dešinę", "1F469-200D-1F9AF-200D-27A1-FE0F"},
    {"moteris_su_neregio_lazdele_atsisukęs_į_dešinę_spalva1", "1F469-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"moteris_su_neregio_lazdele_atsisukęs_į_dešinę_spalva2", "1F469-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"moteris_su_neregio_lazdele_atsisukęs_į_dešinę_spalva3", "1F469-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"moteris_su_neregio_lazdele_atsisukęs_į_dešinę_spalva4", "1F469-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"moteris_su_neregio_lazdele_atsisukęs_į_dešinę_spalva5", "1F469-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"moteris_motorizuotame_vežimėlyje", "1F469-200D-1F9BC"},
    {"moteris_motorizuotame_vežimėlyje_spalva1", "1F469-1F3FB-200D-1F9BC"},
    {"moteris_motorizuotame_vežimėlyje_spalva2", "1F469-1F3FC-200D-1F9BC"},
    {"moteris_motorizuotame_vežimėlyje_spalva3", "1F469-1F3FD-200D-1F9BC"},
    {"moteris_motorizuotame_vežimėlyje_spalva4", "1F469-1F3FE-200D-1F9BC"},
    {"moteris_motorizuotame_vežimėlyje_spalva5", "1F469-1F3FF-200D-1F9BC"},
    {"moteris_motorizuotame_vežimėlyje_atsisukęs_į_dešinę", "1F469-200D-1F9BC-200D-27A1-FE0F"},
    {"moteris_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva1", "1F469-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"moteris_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva2", "1F469-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"moteris_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva3", "1F469-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"moteris_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva4", "1F469-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"moteris_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva5", "1F469-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"moteris_rankomis_valdomame_vežimėlyje", "1F469-200D-1F9BD"},
    {"moteris_rankomis_valdomame_vežimėlyje_spalva1", "1F469-1F3FB-200D-1F9BD"},
    {"moteris_rankomis_valdomame_vežimėlyje_spalva2", "1F469-1F3FC-200D-1F9BD"},
    {"moteris_rankomis_valdomame_vežimėlyje_spalva3", "1F469-1F3FD-200D-1F9BD"},
    {"moteris_rankomis_valdomame_vežimėlyje_spalva4", "1F469-1F3FE-200D-1F9BD"},
    {"moteris_rankomis_valdomame_vežimėlyje_spalva5", "1F469-1F3FF-200D-1F9BD"},
    {"moteris_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę", "1F469-200D-1F9BD-200D-27A1-FE0F"},
    {"moteris_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva1", "1F469-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"moteris_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva2", "1F469-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"moteris_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva3", "1F469-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"moteris_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva4", "1F469-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"moteris_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva5", "1F469-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"pėstysis_atsisukęs_į_dešinę", "1F6B6-200D-27A1-FE0F"},
    {"pėstysis_atsisukęs_į_dešinę_spalva1", "1F6B6-1F3FB-200D-27A1-FE0F"},
    {"pėstysis_atsisukęs_į_dešinę_spalva2", "1F6B6-1F3FC-200D-27A1-FE0F"},
    {"pėstysis_atsisukęs_į_dešinę_spalva3", "1F6B6-1F3FD-200D-27A1-FE0F"},
    {"pėstysis_atsisukęs_į_dešinę_spalva4", "1F6B6-1F3FE-200D-27A1-FE0F"},
    {"pėstysis_atsisukęs_į_dešinę_spalva5", "1F6B6-1F3FF-200D-27A1-FE0F"},
    {"klūpantis_žmogus_atsisukęs_į_dešinę", "1F9CE-200D-27A1-FE0F"},
    {"klūpantis_žmogus_atsisukęs_į_dešinę_spalva1", "1F9CE-1F3FB-200D-27A1-FE0F"},
    {"klūpantis_žmogus_atsisukęs_į_dešinę_spalva2", "1F9CE-1F3FC-200D-27A1-FE0F"},
    {"klūpantis_žmogus_atsisukęs_į_dešinę_spalva3", "1F9CE-1F3FD-200D-27A1-FE0F"},
    {"klūpantis_žmogus_atsisukęs_į_dešinę_spalva4", "1F9CE-1F3FE-200D-27A1-FE0F"},
    {"klūpantis_žmogus_atsisukęs_į_dešinę_spalva5", "1F9CE-1F3FF-200D-27A1-FE0F"},
    {"teisiantis_žmogus", "1F9D1-200D-2696-FE0F"},
    {"teisiantis_žmogus_spalva1", "1F9D1-1F3FB-200D-2696-FE0F"},
    {"teisiantis_žmogus_spalva2", "1F9D1-1F3FC-200D-2696-FE0F"},
    {"teisiantis_žmogus_spalva3", "1F9D1-1F3FD-200D-2696-FE0F"},
    {"teisiantis_žmogus_spalva4", "1F9D1-1F3FE-200D-2696-FE0F"},
    {"teisiantis_žmogus_spalva5", "1F9D1-1F3FF-200D-2696-FE0F"},
    {"pilotuojantis_žmogus", "1F9D1-200D-2708-FE0F"},
    {"pilotuojantis_žmogus_spalva1", "1F9D1-1F3FB-200D-2708-FE0F"},
    {"pilotuojantis_žmogus_spalva2", "1F9D1-1F3FC-200D-2708-FE0F"},
    {"pilotuojantis_žmogus_spalva3", "1F9D1-1F3FD-200D-2708-FE0F"},
    {"pilotuojantis_žmogus_spalva4", "1F9D1-1F3FE-200D-2708-FE0F"},
    {"pilotuojantis_žmogus_spalva5", "1F9D1-1F3FF-200D-2708-FE0F"},
    {"ūkininkaujantis_žmogus", "1F9D1-200D-1F33E"},
    {"ūkininkaujantis_žmogus_spalva1", "1F9D1-1F3FB-200D-1F33E"},
    {"ūkininkaujantis_žmogus_spalva2", "1F9D1-1F3FC-200D-1F33E"},
    {"ūkininkaujantis_žmogus_spalva3", "1F9D1-1F3FD-200D-1F33E"},
    {"ūkininkaujantis_žmogus_spalva4", "1F9D1-1F3FE-200D-1F33E"},
    {"ūkininkaujantis_žmogus_spalva5", "1F9D1-1F3FF-200D-1F33E"},
    {"gaminantis_žmogus", "1F9D1-200D-1F373"},
    {"gaminantis_žmogus_spalva1", "1F9D1-1F3FB-200D-1F373"},
    {"gaminantis_žmogus_spalva2", "1F9D1-1F3FC-200D-1F373"},
    {"gaminantis_žmogus_spalva3", "1F9D1-1F3FD-200D-1F373"},
    {"gaminantis_žmogus_spalva4", "1F9D1-1F3FE-200D-1F373"},
    {"gaminantis_žmogus_spalva5", "1F9D1-1F3FF-200D-1F373"},
    {"kūdikį_maitinantis_asmuo", "1F9D1-200D-1F37C"},
    {"kūdikį_maitinantis_asmuo_spalva1", "1F9D1-1F3FB-200D-1F37C"},
    {"kūdikį_maitinantis_asmuo_spalva2", "1F9D1-1F3FC-200D-1F37C"},
    {"kūdikį_maitinantis_asmuo_spalva3", "1F9D1-1F3FD-200D-1F37C"},
    {"kūdikį_maitinantis_asmuo_spalva4", "1F9D1-1F3FE-200D-1F37C"},
    {"kūdikį_maitinantis_asmuo_spalva5", "1F9D1-1F3FF-200D-1F37C"},
    {"kalėdų_asmuo", "1F9D1-200D-1F384"},
    {"kalėdų_asmuo_spalva1", "1F9D1-1F3FB-200D-1F384"},
    {"kalėdų_asmuo_spalva2", "1F9D1-1F3FC-200D-1F384"},
    {"kalėdų_asmuo_spalva3", "1F9D1-1F3FD-200D-1F384"},
    {"kalėdų_asmuo_spalva4", "1F9D1-1F3FE-200D-1F384"},
    {"kalėdų_asmuo_spalva5", "1F9D1-1F3FF-200D-1F384"},
    {"dainuojantis_žmogus", "1F9D1-200D-1F3A4"},
    {"dainuojantis_žmogus_spalva1", "1F9D1-1F3FB-200D-1F3A4"},
    {"dainuojantis_žmogus_spalva2", "1F9D1-1F3FC-200D-1F3A4"},
    {"dainuojantis_žmogus_spalva3", "1F9D1-1F3FD-200D-1F3A4"},
    {"dainuojantis_žmogus_spalva4", "1F9D1-1F3FE-200D-1F3A4"},
    {"dainuojantis_žmogus_spalva5", "1F9D1-1F3FF-200D-1F3A4"},
    {"mokantis_žmogus", "1F9D1-200D-1F3EB"},
    {"mokantis_žmogus_spalva1", "1F9D1-1F3FB-200D-1F3EB"},
    {"mokantis_žmogus_spalva2", "1F9D1-1F3FC-200D-1F3EB"},
    {"mokantis_žmogus_spalva3", "1F9D1-1F3FD-200D-1F3EB"},
    {"mokantis_žmogus_spalva4", "1F9D1-1F3FE-200D-1F3EB"},
    {"mokantis_žmogus_spalva5", "1F9D1-1F3FF-200D-1F3EB"},
    {"remontuojantis_žmogus", "1F9D1-200D-1F527"},
    {"remontuojantis_žmogus_spalva1", "1F9D1-1F3FB-200D-1F527"},
    {"remontuojantis_žmogus_spalva2", "1F9D1-1F3FC-200D-1F527"},
    {"remontuojantis_žmogus_spalva3", "1F9D1-1F3FD-200D-1F527"},
    {"remontuojantis_žmogus_spalva4", "1F9D1-1F3FE-200D-1F527"},
    {"remontuojantis_žmogus_spalva5", "1F9D1-1F3FF-200D-1F527"},
    {"žmogus_neregio_lazdele", "1F9D1-200D-1F9AF"},
    {"žmogus_neregio_lazdele_spalva1", "1F9D1-1F3FB-200D-1F9AF"},
    {"žmogus_neregio_lazdele_spalva2", "1F9D1-1F3FC-200D-1F9AF"},
    {"žmogus_neregio_lazdele_spalva3", "1F9D1-1F3FD-200D-1F9AF"},
    {"žmogus_neregio_lazdele_spalva4", "1F9D1-1F3FE-200D-1F9AF"},
    {"žmogus_neregio_lazdele_spalva5", "1F9D1-1F3FF-200D-1F9AF"},
    {"žmogus_neregio_lazdele_atsisukęs_į_dešinę", "1F9D1-200D-1F9AF-200D-27A1-FE0F"},
    {"žmogus_neregio_lazdele_atsisukęs_į_dešinę_spalva1", "1F9D1-1F3FB-200D-1F9AF-200D-27A1-FE0F"},
    {"žmogus_neregio_lazdele_atsisukęs_į_dešinę_spalva2", "1F9D1-1F3FC-200D-1F9AF-200D-27A1-FE0F"},
    {"žmogus_neregio_lazdele_atsisukęs_į_dešinę_spalva3", "1F9D1-1F3FD-200D-1F9AF-200D-27A1-FE0F"},
    {"žmogus_neregio_lazdele_atsisukęs_į_dešinę_spalva4", "1F9D1-1F3FE-200D-1F9AF-200D-27A1-FE0F"},
    {"žmogus_neregio_lazdele_atsisukęs_į_dešinę_spalva5", "1F9D1-1F3FF-200D-1F9AF-200D-27A1-FE0F"},
    {"žmogus_motorizuotame_vežimėlyje", "1F9D1-200D-1F9BC"},
    {"žmogus_motorizuotame_vežimėlyje_spalva1", "1F9D1-1F3FB-200D-1F9BC"},
    {"žmogus_motorizuotame_vežimėlyje_spalva2", "1F9D1-1F3FC-200D-1F9BC"},
    {"žmogus_motorizuotame_vežimėlyje_spalva3", "1F9D1-1F3FD-200D-1F9BC"},
    {"žmogus_motorizuotame_vežimėlyje_spalva4", "1F9D1-1F3FE-200D-1F9BC"},
    {"žmogus_motorizuotame_vežimėlyje_spalva5", "1F9D1-1F3FF-200D-1F9BC"},
    {"žmogus_motorizuotame_vežimėlyje_atsisukęs_į_dešinę", "1F9D1-200D-1F9BC-200D-27A1-FE0F"},
    {"žmogus_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva1", "1F9D1-1F3FB-200D-1F9BC-200D-27A1-FE0F"},
    {"žmogus_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva2", "1F9D1-1F3FC-200D-1F9BC-200D-27A1-FE0F"},
    {"žmogus_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva3", "1F9D1-1F3FD-200D-1F9BC-200D-27A1-FE0F"},
    {"žmogus_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva4", "1F9D1-1F3FE-200D-1F9BC-200D-27A1-FE0F"},
    {"žmogus_motorizuotame_vežimėlyje_atsisukęs_į_dešinę_spalva5", "1F9D1-1F3FF-200D-1F9BC-200D-27A1-FE0F"},
    {"žmogus_rankomis_valdomame_vežimėlyje", "1F9D1-200D-1F9BD"},
    {"žmogus_rankomis_valdomame_vežimėlyje_spalva1", "1F9D1-1F3FB-200D-1F9BD"},
    {"žmogus_rankomis_valdomame_vežimėlyje_spalva2", "1F9D1-1F3FC-200D-1F9BD"},
    {"žmogus_rankomis_valdomame_vežimėlyje_spalva3", "1F9D1-1F3FD-200D-1F9BD"},
    {"žmogus_rankomis_valdomame_vežimėlyje_spalva4", "1F9D1-1F3FE-200D-1F9BD"},
    {"žmogus_rankomis_valdomame_vežimėlyje_spalva5", "1F9D1-1F3FF-200D-1F9BD"},
    {"žmogus_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę", "1F9D1-200D-1F9BD-200D-27A1-FE0F"},
    {"žmogus_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva1", "1F9D1-1F3FB-200D-1F9BD-200D-27A1-FE0F"},
    {"žmogus_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva2", "1F9D1-1F3FC-200D-1F9BD-200D-27A1-FE0F"},
    {"žmogus_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva3", "1F9D1-1F3FD-200D-1F9BD-200D-27A1-FE0F"},
    {"žmogus_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva4", "1F9D1-1F3FE-200D-1F9BD-200D-27A1-FE0F"},
    {"žmogus_rankomis_valdomame_vežimėlyje_atsisukęs_į_dešinę_spalva5", "1F9D1-1F3FF-200D-1F9BD-200D-27A1-FE0F"},
    {"bėganti_moteris", "1F3C3-200D-2640-FE0F"},
    {"bėganti_moteris_spalva1", "1F3C3-1F3FB-200D-2640-FE0F"},
    {"bėganti_moteris_spalva2", "1F3C3-1F3FC-200D-2640-FE0F"},
    {"bėganti_moteris_spalva3", "1F3C3-1F3FD-200D-2640-FE0F"},
    {"bėganti_moteris_spalva4", "1F3C3-1F3FE-200D-2640-FE0F"},
    {"bėganti_moteris_spalva5", "1F3C3-1F3FF-200D-2640-FE0F"},
    {"bėganti_moteris_atsisukęs_į_dešinę", "1F3C3-200D-2640-FE0F-200D-27A1-FE0F"},
    {"bėganti_moteris_atsisukęs_į_dešinę_spalva1", "1F3C3-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"bėganti_moteris_atsisukęs_į_dešinę_spalva2", "1F3C3-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"bėganti_moteris_atsisukęs_į_dešinę_spalva3", "1F3C3-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"bėganti_moteris_atsisukęs_į_dešinę_spalva4", "1F3C3-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"bėganti_moteris_atsisukęs_į_dešinę_spalva5", "1F3C3-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"bėgantis_vyras", "1F3C3-200D-2642-FE0F"},
    {"bėgantis_vyras_spalva1", "1F3C3-1F3FB-200D-2642-FE0F"},
    {"bėgantis_vyras_spalva2", "1F3C3-1F3FC-200D-2642-FE0F"},
    {"bėgantis_vyras_spalva3", "1F3C3-1F3FD-200D-2642-FE0F"},
    {"bėgantis_vyras_spalva4", "1F3C3-1F3FE-200D-2642-FE0F"},
    {"bėgantis_vyras_spalva5", "1F3C3-1F3FF-200D-2642-FE0F"},
    {"bėgantis_vyras_atsisukęs_į_dešinę", "1F3C3-200D-2642-FE0F-200D-27A1-FE0F"},
    {"bėgantis_vyras_atsisukęs_į_dešinę_spalva1", "1F3C3-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"bėgantis_vyras_atsisukęs_į_dešinę_spalva2", "1F3C3-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"bėgantis_vyras_atsisukęs_į_dešinę_spalva3", "1F3C3-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"bėgantis_vyras_atsisukęs_į_dešinę_spalva4", "1F3C3-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"bėgantis_vyras_atsisukęs_į_dešinę_spalva5", "1F3C3-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"banglentininkė", "1F3C4-200D-2640-FE0F"},
    {"banglentininkė_spalva1", "1F3C4-1F3FB-200D-2640-FE0F"},
    {"banglentininkė_spalva2", "1F3C4-1F3FC-200D-2640-FE0F"},
    {"banglentininkė_spalva3", "1F3C4-1F3FD-200D-2640-FE0F"},
    {"banglentininkė_spalva4", "1F3C4-1F3FE-200D-2640-FE0F"},
    {"banglentininkė_spalva5", "1F3C4-1F3FF-200D-2640-FE0F"},
    {"plaukikė", "1F3CA-200D-2640-FE0F"},
    {"plaukikė_spalva1", "1F3CA-1F3FB-200D-2640-FE0F"},
    {"plaukikė_spalva2", "1F3CA-1F3FC-200D-2640-FE0F"},
    {"plaukikė_spalva3", "1F3CA-1F3FD-200D-2640-FE0F"},
    {"plaukikė_spalva4", "1F3CA-1F3FE-200D-2640-FE0F"},
    {"plaukikė_spalva5", "1F3CA-1F3FF-200D-2640-FE0F"},
    {"golfo_žaidėja", "1F3CC-FE0F-200D-2640-FE0F"},
    {"golfo_žaidėja_spalva1", "1F3CC-1F3FB-200D-2640-FE0F"},
    {"golfo_žaidėja_spalva2", "1F3CC-1F3FC-200D-2640-FE0F"},
    {"golfo_žaidėja_spalva3", "1F3CC-1F3FD-200D-2640-FE0F"},
    {"golfo_žaidėja_spalva4", "1F3CC-1F3FE-200D-2640-FE0F"},
    {"golfo_žaidėja_spalva5", "1F3CC-1F3FF-200D-2640-FE0F"},
    {"golfo_žaidėjas_vyras", "1F3CC-FE0F-200D-2642-FE0F"},
    {"golfo_žaidėjas_vyras_spalva1", "1F3CC-1F3FB-200D-2642-FE0F"},
    {"golfo_žaidėjas_vyras_spalva2", "1F3CC-1F3FC-200D-2642-FE0F"},
    {"golfo_žaidėjas_vyras_spalva3", "1F3CC-1F3FD-200D-2642-FE0F"},
    {"golfo_žaidėjas_vyras_spalva4", "1F3CC-1F3FE-200D-2642-FE0F"},
    {"golfo_žaidėjas_vyras_spalva5", "1F3CC-1F3FF-200D-2642-FE0F"},
    {"moteris_policininkė", "1F46E-200D-2640-FE0F"},
    {"moteris_policininkė_spalva1", "1F46E-1F3FB-200D-2640-FE0F"},
    {"moteris_policininkė_spalva2", "1F46E-1F3FC-200D-2640-FE0F"},
    {"moteris_policininkė_spalva3", "1F46E-1F3FD-200D-2640-FE0F"},
    {"moteris_policininkė_spalva4", "1F46E-1F3FE-200D-2640-FE0F"},
    {"moteris_policininkė_spalva5", "1F46E-1F3FF-200D-2640-FE0F"},
    {"vakarėlyje_dalyvaujančios_moterys", "1F46F-200D-2640-FE0F"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva1-2", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FC"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva1-3", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FD"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva1-4", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FE"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva1-5", "1F469-1F3FB-200D-1F430-200D-1F469-1F3FF"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva2-1", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FB"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva2-3", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FD"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva2-4", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FE"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva2-5", "1F469-1F3FC-200D-1F430-200D-1F469-1F3FF"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva3-1", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FB"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva3-2", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FC"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva3-4", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FE"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva3-5", "1F469-1F3FD-200D-1F430-200D-1F469-1F3FF"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva4-1", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FB"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva4-2", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FC"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva4-3", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FD"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva4-5", "1F469-1F3FE-200D-1F430-200D-1F469-1F3FF"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva5-1", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FB"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva5-2", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FC"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva5-3", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FD"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva5-4", "1F469-1F3FF-200D-1F430-200D-1F469-1F3FE"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva1", "1F46F-1F3FB-200D-2640-FE0F"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva2", "1F46F-1F3FC-200D-2640-FE0F"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva3", "1F46F-1F3FD-200D-2640-FE0F"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva4", "1F46F-1F3FE-200D-2640-FE0F"},
    {"vakarėlyje_dalyvaujančios_moterys_spalva5", "1F46F-1F3FF-200D-2640-FE0F"},
    {"vakarėlyje_dalyvaujantys_vyrai", "1F46F-200D-2642-FE0F"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva1-2", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FC"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva1-3", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FD"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva1-4", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FE"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva1-5", "1F468-1F3FB-200D-1F430-200D-1F468-1F3FF"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva2-1", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FB"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva2-3", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FD"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva2-4", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FE"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva2-5", "1F468-1F3FC-200D-1F430-200D-1F468-1F3FF"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva3-1", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FB"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva3-2", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FC"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva3-4", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FE"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva3-5", "1F468-1F3FD-200D-1F430-200D-1F468-1F3FF"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva4-1", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FB"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva4-2", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FC"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva4-3", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FD"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva4-5", "1F468-1F3FE-200D-1F430-200D-1F468-1F3FF"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva5-1", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FB"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva5-2", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FC"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva5-3", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FD"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva5-4", "1F468-1F3FF-200D-1F430-200D-1F468-1F3FE"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva1", "1F46F-1F3FB-200D-2642-FE0F"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva2", "1F46F-1F3FC-200D-2642-FE0F"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva3", "1F46F-1F3FD-200D-2642-FE0F"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva4", "1F46F-1F3FE-200D-2642-FE0F"},
    {"vakarėlyje_dalyvaujantys_vyrai_spalva5", "1F46F-1F3FF-200D-2642-FE0F"},
    {"blondinė", "1F471-200D-2640-FE0F"},
    {"blondinė_spalva1", "1F471-1F3FB-200D-2640-FE0F"},
    {"blondinė_spalva2", "1F471-1F3FC-200D-2640-FE0F"},
    {"blondinė_spalva3", "1F471-1F3FD-200D-2640-FE0F"},
    {"blondinė_spalva4", "1F471-1F3FE-200D-2640-FE0F"},
    {"blondinė_spalva5", "1F471-1F3FF-200D-2640-FE0F"},
    {"vyriškis_su_turbanu", "1F473-200D-2642-FE0F"},
    {"vyriškis_su_turbanu_spalva1", "1F473-1F3FB-200D-2642-FE0F"},
    {"vyriškis_su_turbanu_spalva2", "1F473-1F3FC-200D-2642-FE0F"},
    {"vyriškis_su_turbanu_spalva3", "1F473-1F3FD-200D-2642-FE0F"},
    {"vyriškis_su_turbanu_spalva4", "1F473-1F3FE-200D-2642-FE0F"},
    {"vyriškis_su_turbanu_spalva5", "1F473-1F3FF-200D-2642-FE0F"},
    {"statybininkė", "1F477-200D-2640-FE0F"},
    {"statybininkė_spalva1", "1F477-1F3FB-200D-2640-FE0F"},
    {"statybininkė_spalva2", "1F477-1F3FC-200D-2640-FE0F"},
    {"statybininkė_spalva3", "1F477-1F3FD-200D-2640-FE0F"},
    {"statybininkė_spalva4", "1F477-1F3FE-200D-2640-FE0F"},
    {"statybininkė_spalva5", "1F477-1F3FF-200D-2640-FE0F"},
    {"ranką_pakėlusi_moteris", "1F481-200D-2640-FE0F"},
    {"ranką_pakėlusi_moteris_spalva1", "1F481-1F3FB-200D-2640-FE0F"},
    {"ranką_pakėlusi_moteris_spalva2", "1F481-1F3FC-200D-2640-FE0F"},
    {"ranką_pakėlusi_moteris_spalva3", "1F481-1F3FD-200D-2640-FE0F"},
    {"ranką_pakėlusi_moteris_spalva4", "1F481-1F3FE-200D-2640-FE0F"},
    {"ranką_pakėlusi_moteris_spalva5", "1F481-1F3FF-200D-2640-FE0F"},
    {"lengvai_ranką_pakėlęs_vyras", "1F481-200D-2642-FE0F"},
    {"lengvai_ranką_pakėlęs_vyras_spalva1", "1F481-1F3FB-200D-2642-FE0F"},
    {"lengvai_ranką_pakėlęs_vyras_spalva2", "1F481-1F3FC-200D-2642-FE0F"},
    {"lengvai_ranką_pakėlęs_vyras_spalva3", "1F481-1F3FD-200D-2642-FE0F"},
    {"lengvai_ranką_pakėlęs_vyras_spalva4", "1F481-1F3FE-200D-2642-FE0F"},
    {"lengvai_ranką_pakėlęs_vyras_spalva5", "1F481-1F3FF-200D-2642-FE0F"},
    {"sargybinė", "1F482-200D-2640-FE0F"},
    {"sargybinė_spalva1", "1F482-1F3FB-200D-2640-FE0F"},
    {"sargybinė_spalva2", "1F482-1F3FC-200D-2640-FE0F"},
    {"sargybinė_spalva3", "1F482-1F3FD-200D-2640-FE0F"},
    {"sargybinė_spalva4", "1F482-1F3FE-200D-2640-FE0F"},
    {"sargybinė_spalva5", "1F482-1F3FF-200D-2640-FE0F"},
    {"masažuojama_moters_galva", "1F486-200D-2640-FE0F"},
    {"masažuojama_moters_galva_spalva1", "1F486-1F3FB-200D-2640-FE0F"},
    {"masažuojama_moters_galva_spalva2", "1F486-1F3FC-200D-2640-FE0F"},
    {"masažuojama_moters_galva_spalva3", "1F486-1F3FD-200D-2640-FE0F"},
    {"masažuojama_moters_galva_spalva4", "1F486-1F3FE-200D-2640-FE0F"},
    {"masažuojama_moters_galva_spalva5", "1F486-1F3FF-200D-2640-FE0F"},
    {"masažuojama_vyro_galva", "1F486-200D-2642-FE0F"},
    {"masažuojama_vyro_galva_spalva1", "1F486-1F3FB-200D-2642-FE0F"},
    {"masažuojama_vyro_galva_spalva2", "1F486-1F3FC-200D-2642-FE0F"},
    {"masažuojama_vyro_galva_spalva3", "1F486-1F3FD-200D-2642-FE0F"},
    {"masažuojama_vyro_galva_spalva4", "1F486-1F3FE-200D-2642-FE0F"},
    {"masažuojama_vyro_galva_spalva5", "1F486-1F3FF-200D-2642-FE0F"},
    {"seklė", "1F575-FE0F-200D-2640-FE0F"},
    {"seklė_spalva1", "1F575-1F3FB-200D-2640-FE0F"},
    {"seklė_spalva2", "1F575-1F3FC-200D-2640-FE0F"},
    {"seklė_spalva3", "1F575-1F3FD-200D-2640-FE0F"},
    {"seklė_spalva4", "1F575-1F3FE-200D-2640-FE0F"},
    {"seklė_spalva5", "1F575-1F3FF-200D-2640-FE0F"},
    {"draudžianti_moteris", "1F645-200D-2640-FE0F"},
    {"draudžianti_moteris_spalva1", "1F645-1F3FB-200D-2640-FE0F"},
    {"draudžianti_moteris_spalva2", "1F645-1F3FC-200D-2640-FE0F"},
    {"draudžianti_moteris_spalva3", "1F645-1F3FD-200D-2640-FE0F"},
    {"draudžianti_moteris_spalva4", "1F645-1F3FE-200D-2640-FE0F"},
    {"draudžianti_moteris_spalva5", "1F645-1F3FF-200D-2640-FE0F"},
    {"draudžiantis_vyras", "1F645-200D-2642-FE0F"},
    {"draudžiantis_vyras_spalva1", "1F645-1F3FB-200D-2642-FE0F"},
    {"draudžiantis_vyras_spalva2", "1F645-1F3FC-200D-2642-FE0F"},
    {"draudžiantis_vyras_spalva3", "1F645-1F3FD-200D-2642-FE0F"},
    {"draudžiantis_vyras_spalva4", "1F645-1F3FE-200D-2642-FE0F"},
    {"draudžiantis_vyras_spalva5", "1F645-1F3FF-200D-2642-FE0F"},
    {"leidžianti_moteris", "1F646-200D-2640-FE0F"},
    {"leidžianti_moteris_spalva1", "1F646-1F3FB-200D-2640-FE0F"},
    {"leidžianti_moteris_spalva2", "1F646-1F3FC-200D-2640-FE0F"},
    {"leidžianti_moteris_spalva3", "1F646-1F3FD-200D-2640-FE0F"},
    {"leidžianti_moteris_spalva4", "1F646-1F3FE-200D-2640-FE0F"},
    {"leidžianti_moteris_spalva5", "1F646-1F3FF-200D-2640-FE0F"},
    {"leidžiantis_vyras", "1F646-200D-2642-FE0F"},
    {"leidžiantis_vyras_spalva1", "1F646-1F3FB-200D-2642-FE0F"},
    {"leidžiantis_vyras_spalva2", "1F646-1F3FC-200D-2642-FE0F"},
    {"leidžiantis_vyras_spalva3", "1F646-1F3FD-200D-2642-FE0F"},
    {"leidžiantis_vyras_spalva4", "1F646-1F3FE-200D-2642-FE0F"},
    {"leidžiantis_vyras_spalva5", "1F646-1F3FF-200D-2642-FE0F"},
    {"ranką_iškėlusi_moteris", "1F64B-200D-2640-FE0F"},
    {"ranką_iškėlusi_moteris_spalva1", "1F64B-1F3FB-200D-2640-FE0F"},
    {"ranką_iškėlusi_moteris_spalva2", "1F64B-1F3FC-200D-2640-FE0F"},
    {"ranką_iškėlusi_moteris_spalva3", "1F64B-1F3FD-200D-2640-FE0F"},
    {"ranką_iškėlusi_moteris_spalva4", "1F64B-1F3FE-200D-2640-FE0F"},
    {"ranką_iškėlusi_moteris_spalva5", "1F64B-1F3FF-200D-2640-FE0F"},
    {"ranką_pakėlęs_vyras", "1F64B-200D-2642-FE0F"},
    {"ranką_pakėlęs_vyras_spalva1", "1F64B-1F3FB-200D-2642-FE0F"},
    {"ranką_pakėlęs_vyras_spalva2", "1F64B-1F3FC-200D-2642-FE0F"},
    {"ranką_pakėlęs_vyras_spalva3", "1F64B-1F3FD-200D-2642-FE0F"},
    {"ranką_pakėlęs_vyras_spalva4", "1F64B-1F3FE-200D-2642-FE0F"},
    {"ranką_pakėlęs_vyras_spalva5", "1F64B-1F3FF-200D-2642-FE0F"},
    {"paniuręs_vyras", "1F64D-200D-2642-FE0F"},
    {"paniuręs_vyras_spalva1", "1F64D-1F3FB-200D-2642-FE0F"},
    {"paniuręs_vyras_spalva2", "1F64D-1F3FC-200D-2642-FE0F"},
    {"paniuręs_vyras_spalva3", "1F64D-1F3FD-200D-2642-FE0F"},
    {"paniuręs_vyras_spalva4", "1F64D-1F3FE-200D-2642-FE0F"},
    {"paniuręs_vyras_spalva5", "1F64D-1F3FF-200D-2642-FE0F"},
    {"susiraukęs_vyras", "1F64E-200D-2642-FE0F"},
    {"susiraukęs_vyras_spalva1", "1F64E-1F3FB-200D-2642-FE0F"},
    {"susiraukęs_vyras_spalva2", "1F64E-1F3FC-200D-2642-FE0F"},
    {"susiraukęs_vyras_spalva3", "1F64E-1F3FD-200D-2642-FE0F"},
    {"susiraukęs_vyras_spalva4", "1F64E-1F3FE-200D-2642-FE0F"},
    {"susiraukęs_vyras_spalva5", "1F64E-1F3FF-200D-2642-FE0F"},
    {"dviratininkė", "1F6B4-200D-2640-FE0F"},
    {"dviratininkė_spalva1", "1F6B4-1F3FB-200D-2640-FE0F"},
    {"dviratininkė_spalva2", "1F6B4-1F3FC-200D-2640-FE0F"},
    {"dviratininkė_spalva3", "1F6B4-1F3FD-200D-2640-FE0F"},
    {"dviratininkė_spalva4", "1F6B4-1F3FE-200D-2640-FE0F"},
    {"dviratininkė_spalva5", "1F6B4-1F3FF-200D-2640-FE0F"},
    {"kalnų_dviratininkė_moteris", "1F6B5-200D-2640-FE0F"},
    {"kalnų_dviratininkė_moteris_spalva1", "1F6B5-1F3FB-200D-2640-FE0F"},
    {"kalnų_dviratininkė_moteris_spalva2", "1F6B5-1F3FC-200D-2640-FE0F"},
    {"kalnų_dviratininkė_moteris_spalva3", "1F6B5-1F3FD-200D-2640-FE0F"},
    {"kalnų_dviratininkė_moteris_spalva4", "1F6B5-1F3FE-200D-2640-FE0F"},
    {"kalnų_dviratininkė_moteris_spalva5", "1F6B5-1F3FF-200D-2640-FE0F"},
    {"kalnų_dviratininkas_vyras", "1F6B5-200D-2642-FE0F"},
    {"kalnų_dviratininkas_vyras_spalva1", "1F6B5-1F3FB-200D-2642-FE0F"},
    {"kalnų_dviratininkas_vyras_spalva2", "1F6B5-1F3FC-200D-2642-FE0F"},
    {"kalnų_dviratininkas_vyras_spalva3", "1F6B5-1F3FD-200D-2642-FE0F"},
    {"kalnų_dviratininkas_vyras_spalva4", "1F6B5-1F3FE-200D-2642-FE0F"},
    {"kalnų_dviratininkas_vyras_spalva5", "1F6B5-1F3FF-200D-2642-FE0F"},
    {"einanti_moteris_atsisukęs_į_dešinę", "1F6B6-200D-2640-FE0F-200D-27A1-FE0F"},
    {"einanti_moteris_atsisukęs_į_dešinę_spalva1", "1F6B6-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"einanti_moteris_atsisukęs_į_dešinę_spalva2", "1F6B6-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"einanti_moteris_atsisukęs_į_dešinę_spalva3", "1F6B6-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"einanti_moteris_atsisukęs_į_dešinę_spalva4", "1F6B6-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"einanti_moteris_atsisukęs_į_dešinę_spalva5", "1F6B6-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"einantis_vyras_atsisukęs_į_dešinę", "1F6B6-200D-2642-FE0F-200D-27A1-FE0F"},
    {"einantis_vyras_atsisukęs_į_dešinę_spalva1", "1F6B6-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"einantis_vyras_atsisukęs_į_dešinę_spalva2", "1F6B6-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"einantis_vyras_atsisukęs_į_dešinę_spalva3", "1F6B6-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"einantis_vyras_atsisukęs_į_dešinę_spalva4", "1F6B6-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"einantis_vyras_atsisukęs_į_dešinę_spalva5", "1F6B6-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"susierzinęs_vyras", "1F926-200D-2642-FE0F"},
    {"susierzinęs_vyras_spalva1", "1F926-1F3FB-200D-2642-FE0F"},
    {"susierzinęs_vyras_spalva2", "1F926-1F3FC-200D-2642-FE0F"},
    {"susierzinęs_vyras_spalva3", "1F926-1F3FD-200D-2642-FE0F"},
    {"susierzinęs_vyras_spalva4", "1F926-1F3FE-200D-2642-FE0F"},
    {"susierzinęs_vyras_spalva5", "1F926-1F3FF-200D-2642-FE0F"},
    {"moteris_daro_suktuką", "1F938-200D-2640-FE0F"},
    {"moteris_daro_suktuką_spalva1", "1F938-1F3FB-200D-2640-FE0F"},
    {"moteris_daro_suktuką_spalva2", "1F938-1F3FC-200D-2640-FE0F"},
    {"moteris_daro_suktuką_spalva3", "1F938-1F3FD-200D-2640-FE0F"},
    {"moteris_daro_suktuką_spalva4", "1F938-1F3FE-200D-2640-FE0F"},
    {"moteris_daro_suktuką_spalva5", "1F938-1F3FF-200D-2640-FE0F"},
    {"vyras_daro_suktuką", "1F938-200D-2642-FE0F"},
    {"vyras_daro_suktuką_spalva1", "1F938-1F3FB-200D-2642-FE0F"},
    {"vyras_daro_suktuką_spalva2", "1F938-1F3FC-200D-2642-FE0F"},
    {"vyras_daro_suktuką_spalva3", "1F938-1F3FD-200D-2642-FE0F"},
    {"vyras_daro_suktuką_spalva4", "1F938-1F3FE-200D-2642-FE0F"},
    {"vyras_daro_suktuką_spalva5", "1F938-1F3FF-200D-2642-FE0F"},
    {"žongliruotoja", "1F939-200D-2640-FE0F"},
    {"žongliruotoja_spalva1", "1F939-1F3FB-200D-2640-FE0F"},
    {"žongliruotoja_spalva2", "1F939-1F3FC-200D-2640-FE0F"},
    {"žongliruotoja_spalva3", "1F939-1F3FD-200D-2640-FE0F"},
    {"žongliruotoja_spalva4", "1F939-1F3FE-200D-2640-FE0F"},
    {"žongliruotoja_spalva5", "1F939-1F3FF-200D-2640-FE0F"},
    {"žongliruotojas", "1F939-200D-2642-FE0F"},
    {"žongliruotojas_spalva1", "1F939-1F3FB-200D-2642-FE0F"},
    {"žongliruotojas_spalva2", "1F939-1F3FC-200D-2642-FE0F"},
    {"žongliruotojas_spalva3", "1F939-1F3FD-200D-2642-FE0F"},
    {"žongliruotojas_spalva4", "1F939-1F3FE-200D-2642-FE0F"},
    {"žongliruotojas_spalva5", "1F939-1F3FF-200D-2642-FE0F"},
    {"moterų_imtynės", "1F93C-200D-2640-FE0F"},
    {"moterų_imtynės_spalva1-2", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FC"},
    {"moterų_imtynės_spalva1-3", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FD"},
    {"moterų_imtynės_spalva1-4", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FE"},
    {"moterų_imtynės_spalva1-5", "1F469-1F3FB-200D-1FAEF-200D-1F469-1F3FF"},
    {"moterų_imtynės_spalva2-1", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FB"},
    {"moterų_imtynės_spalva2-3", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FD"},
    {"moterų_imtynės_spalva2-4", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FE"},
    {"moterų_imtynės_spalva2-5", "1F469-1F3FC-200D-1FAEF-200D-1F469-1F3FF"},
    {"moterų_imtynės_spalva3-1", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FB"},
    {"moterų_imtynės_spalva3-2", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FC"},
    {"moterų_imtynės_spalva3-4", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FE"},
    {"moterų_imtynės_spalva3-5", "1F469-1F3FD-200D-1FAEF-200D-1F469-1F3FF"},
    {"moterų_imtynės_spalva4-1", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FB"},
    {"moterų_imtynės_spalva4-2", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FC"},
    {"moterų_imtynės_spalva4-3", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FD"},
    {"moterų_imtynės_spalva4-5", "1F469-1F3FE-200D-1FAEF-200D-1F469-1F3FF"},
    {"moterų_imtynės_spalva5-1", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FB"},
    {"moterų_imtynės_spalva5-2", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FC"},
    {"moterų_imtynės_spalva5-3", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FD"},
    {"moterų_imtynės_spalva5-4", "1F469-1F3FF-200D-1FAEF-200D-1F469-1F3FE"},
    {"moterų_imtynės_spalva1", "1F93C-1F3FB-200D-2640-FE0F"},
    {"moterų_imtynės_spalva2", "1F93C-1F3FC-200D-2640-FE0F"},
    {"moterų_imtynės_spalva3", "1F93C-1F3FD-200D-2640-FE0F"},
    {"moterų_imtynės_spalva4", "1F93C-1F3FE-200D-2640-FE0F"},
    {"moterų_imtynės_spalva5", "1F93C-1F3FF-200D-2640-FE0F"},
    {"vyrų_imtynės", "1F93C-200D-2642-FE0F"},
    {"vyrų_imtynės_spalva1-2", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FC"},
    {"vyrų_imtynės_spalva1-3", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FD"},
    {"vyrų_imtynės_spalva1-4", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FE"},
    {"vyrų_imtynės_spalva1-5", "1F468-1F3FB-200D-1FAEF-200D-1F468-1F3FF"},
    {"vyrų_imtynės_spalva2-1", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FB"},
    {"vyrų_imtynės_spalva2-3", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FD"},
    {"vyrų_imtynės_spalva2-4", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FE"},
    {"vyrų_imtynės_spalva2-5", "1F468-1F3FC-200D-1FAEF-200D-1F468-1F3FF"},
    {"vyrų_imtynės_spalva3-1", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FB"},
    {"vyrų_imtynės_spalva3-2", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FC"},
    {"vyrų_imtynės_spalva3-4", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FE"},
    {"vyrų_imtynės_spalva3-5", "1F468-1F3FD-200D-1FAEF-200D-1F468-1F3FF"},
    {"vyrų_imtynės_spalva4-1", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FB"},
    {"vyrų_imtynės_spalva4-2", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FC"},
    {"vyrų_imtynės_spalva4-3", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FD"},
    {"vyrų_imtynės_spalva4-5", "1F468-1F3FE-200D-1FAEF-200D-1F468-1F3FF"},
    {"vyrų_imtynės_spalva5-1", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FB"},
    {"vyrų_imtynės_spalva5-2", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FC"},
    {"vyrų_imtynės_spalva5-3", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FD"},
    {"vyrų_imtynės_spalva5-4", "1F468-1F3FF-200D-1FAEF-200D-1F468-1F3FE"},
    {"vyrų_imtynės_spalva1", "1F93C-1F3FB-200D-2642-FE0F"},
    {"vyrų_imtynės_spalva2", "1F93C-1F3FC-200D-2642-FE0F"},
    {"vyrų_imtynės_spalva3", "1F93C-1F3FD-200D-2642-FE0F"},
    {"vyrų_imtynės_spalva4", "1F93C-1F3FE-200D-2642-FE0F"},
    {"vyrų_imtynės_spalva5", "1F93C-1F3FF-200D-2642-FE0F"},
    {"vandensvydžio_žaidėja", "1F93D-200D-2640-FE0F"},
    {"vandensvydžio_žaidėja_spalva1", "1F93D-1F3FB-200D-2640-FE0F"},
    {"vandensvydžio_žaidėja_spalva2", "1F93D-1F3FC-200D-2640-FE0F"},
    {"vandensvydžio_žaidėja_spalva3", "1F93D-1F3FD-200D-2640-FE0F"},
    {"vandensvydžio_žaidėja_spalva4", "1F93D-1F3FE-200D-2640-FE0F"},
    {"vandensvydžio_žaidėja_spalva5", "1F93D-1F3FF-200D-2640-FE0F"},
    {"vandensvydžio_žaidėjas", "1F93D-200D-2642-FE0F"},
    {"vandensvydžio_žaidėjas_spalva1", "1F93D-1F3FB-200D-2642-FE0F"},
    {"vandensvydžio_žaidėjas_spalva2", "1F93D-1F3FC-200D-2642-FE0F"},
    {"vandensvydžio_žaidėjas_spalva3", "1F93D-1F3FD-200D-2642-FE0F"},
    {"vandensvydžio_žaidėjas_spalva4", "1F93D-1F3FE-200D-2642-FE0F"},
    {"vandensvydžio_žaidėjas_spalva5", "1F93D-1F3FF-200D-2642-FE0F"},
    {"rankininkė", "1F93E-200D-2640-FE0F"},
    {"rankininkė_spalva1", "1F93E-1F3FB-200D-2640-FE0F"},
    {"rankininkė_spalva2", "1F93E-1F3FC-200D-2640-FE0F"},
    {"rankininkė_spalva3", "1F93E-1F3FD-200D-2640-FE0F"},
    {"rankininkė_spalva4", "1F93E-1F3FE-200D-2640-FE0F"},
    {"rankininkė_spalva5", "1F93E-1F3FF-200D-2640-FE0F"},
    {"moteris_superherojė", "1F9B8-200D-2640-FE0F"},
    {"moteris_superherojė_spalva1", "1F9B8-1F3FB-200D-2640-FE0F"},
    {"moteris_superherojė_spalva2", "1F9B8-1F3FC-200D-2640-FE0F"},
    {"moteris_superherojė_spalva3", "1F9B8-1F3FD-200D-2640-FE0F"},
    {"moteris_superherojė_spalva4", "1F9B8-1F3FE-200D-2640-FE0F"},
    {"moteris_superherojė_spalva5", "1F9B8-1F3FF-200D-2640-FE0F"},
    {"moteris_piktadarė", "1F9B9-200D-2640-FE0F"},
    {"moteris_piktadarė_spalva1", "1F9B9-1F3FB-200D-2640-FE0F"},
    {"moteris_piktadarė_spalva2", "1F9B9-1F3FC-200D-2640-FE0F"},
    {"moteris_piktadarė_spalva3", "1F9B9-1F3FD-200D-2640-FE0F"},
    {"moteris_piktadarė_spalva4", "1F9B9-1F3FE-200D-2640-FE0F"},
    {"moteris_piktadarė_spalva5", "1F9B9-1F3FF-200D-2640-FE0F"},
    {"klūpanti_moteris", "1F9CE-200D-2640-FE0F"},
    {"klūpanti_moteris_spalva1", "1F9CE-1F3FB-200D-2640-FE0F"},
    {"klūpanti_moteris_spalva2", "1F9CE-1F3FC-200D-2640-FE0F"},
    {"klūpanti_moteris_spalva3", "1F9CE-1F3FD-200D-2640-FE0F"},
    {"klūpanti_moteris_spalva4", "1F9CE-1F3FE-200D-2640-FE0F"},
    {"klūpanti_moteris_spalva5", "1F9CE-1F3FF-200D-2640-FE0F"},
    {"klūpanti_moteris_atsisukęs_į_dešinę", "1F9CE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klūpanti_moteris_atsisukęs_į_dešinę_spalva1", "1F9CE-1F3FB-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klūpanti_moteris_atsisukęs_į_dešinę_spalva2", "1F9CE-1F3FC-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klūpanti_moteris_atsisukęs_į_dešinę_spalva3", "1F9CE-1F3FD-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klūpanti_moteris_atsisukęs_į_dešinę_spalva4", "1F9CE-1F3FE-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klūpanti_moteris_atsisukęs_į_dešinę_spalva5", "1F9CE-1F3FF-200D-2640-FE0F-200D-27A1-FE0F"},
    {"klūpantis_vyras", "1F9CE-200D-2642-FE0F"},
    {"klūpantis_vyras_spalva1", "1F9CE-1F3FB-200D-2642-FE0F"},
    {"klūpantis_vyras_spalva2", "1F9CE-1F3FC-200D-2642-FE0F"},
    {"klūpantis_vyras_spalva3", "1F9CE-1F3FD-200D-2642-FE0F"},
    {"klūpantis_vyras_spalva4", "1F9CE-1F3FE-200D-2642-FE0F"},
    {"klūpantis_vyras_spalva5", "1F9CE-1F3FF-200D-2642-FE0F"},
    {"klūpantis_vyras_atsisukęs_į_dešinę", "1F9CE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klūpantis_vyras_atsisukęs_į_dešinę_spalva1", "1F9CE-1F3FB-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klūpantis_vyras_atsisukęs_į_dešinę_spalva2", "1F9CE-1F3FC-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klūpantis_vyras_atsisukęs_į_dešinę_spalva3", "1F9CE-1F3FD-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klūpantis_vyras_atsisukęs_į_dešinę_spalva4", "1F9CE-1F3FE-200D-2642-FE0F-200D-27A1-FE0F"},
    {"klūpantis_vyras_atsisukęs_į_dešinę_spalva5", "1F9CE-1F3FF-200D-2642-FE0F-200D-27A1-FE0F"},
    {"kurčia_moteris", "1F9CF-200D-2640-FE0F"},
    {"kurčia_moteris_spalva1", "1F9CF-1F3FB-200D-2640-FE0F"},
    {"kurčia_moteris_spalva2", "1F9CF-1F3FC-200D-2640-FE0F"},
    {"kurčia_moteris_spalva3", "1F9CF-1F3FD-200D-2640-FE0F"},
    {"kurčia_moteris_spalva4", "1F9CF-1F3FE-200D-2640-FE0F"},
    {"kurčia_moteris_spalva5", "1F9CF-1F3FF-200D-2640-FE0F"},
    {"kurčias_vyras", "1F9CF-200D-2642-FE0F"},
    {"kurčias_vyras_spalva1", "1F9CF-1F3FB-200D-2642-FE0F"},
    {"kurčias_vyras_spalva2", "1F9CF-1F3FC-200D-2642-FE0F"},
    {"kurčias_vyras_spalva3", "1F9CF-1F3FD-200D-2642-FE0F"},
    {"kurčias_vyras_spalva4", "1F9CF-1F3FE-200D-2642-FE0F"},
    {"kurčias_vyras_spalva5", "1F9CF-1F3FF-200D-2642-FE0F"},
    {"moteris_garų_kambaryje", "1F9D6-200D-2640-FE0F"},
    {"moteris_garų_kambaryje_spalva1", "1F9D6-1F3FB-200D-2640-FE0F"},
    {"moteris_garų_kambaryje_spalva2", "1F9D6-1F3FC-200D-2640-FE0F"},
    {"moteris_garų_kambaryje_spalva3", "1F9D6-1F3FD-200D-2640-FE0F"},
    {"moteris_garų_kambaryje_spalva4", "1F9D6-1F3FE-200D-2640-FE0F"},
    {"moteris_garų_kambaryje_spalva5", "1F9D6-1F3FF-200D-2640-FE0F"},
    {"vyras_garų_kambaryje", "1F9D6-200D-2642-FE0F"},
    {"vyras_garų_kambaryje_spalva1", "1F9D6-1F3FB-200D-2642-FE0F"},
    {"vyras_garų_kambaryje_spalva2", "1F9D6-1F3FC-200D-2642-FE0F"},
    {"vyras_garų_kambaryje_spalva3", "1F9D6-1F3FD-200D-2642-FE0F"},
    {"vyras_garų_kambaryje_spalva4", "1F9D6-1F3FE-200D-2642-FE0F"},
    {"vyras_garų_kambaryje_spalva5", "1F9D6-1F3FF-200D-2642-FE0F"},
    {"magė", "1F9D9-200D-2640-FE0F"},
    {"magė_spalva1", "1F9D9-1F3FB-200D-2640-FE0F"},
    {"magė_spalva2", "1F9D9-1F3FC-200D-2640-FE0F"},
    {"magė_spalva3", "1F9D9-1F3FD-200D-2640-FE0F"},
    {"magė_spalva4", "1F9D9-1F3FE-200D-2640-FE0F"},
    {"magė_spalva5", "1F9D9-1F3FF-200D-2640-FE0F"},
    {"moteris_fėja", "1F9DA-200D-2640-FE0F"},
    {"moteris_fėja_spalva1", "1F9DA-1F3FB-200D-2640-FE0F"},
    {"moteris_fėja_spalva2", "1F9DA-1F3FC-200D-2640-FE0F"},
    {"moteris_fėja_spalva3", "1F9DA-1F3FD-200D-2640-FE0F"},
    {"moteris_fėja_spalva4", "1F9DA-1F3FE-200D-2640-FE0F"},
    {"moteris_fėja_spalva5", "1F9DA-1F3FF-200D-2640-FE0F"},
    {"vampyrė", "1F9DB-200D-2640-FE0F"},
    {"vampyrė_spalva1", "1F9DB-1F3FB-200D-2640-FE0F"},
    {"vampyrė_spalva2", "1F9DB-1F3FC-200D-2640-FE0F"},
    {"vampyrė_spalva3", "1F9DB-1F3FD-200D-2640-FE0F"},
    {"vampyrė_spalva4", "1F9DB-1F3FE-200D-2640-FE0F"},
    {"vampyrė_spalva5", "1F9DB-1F3FF-200D-2640-FE0F"},
    {"undinėlė", "1F9DC-200D-2640-FE0F"},
    {"undinėlė_spalva1", "1F9DC-1F3FB-200D-2640-FE0F"},
    {"undinėlė_spalva2", "1F9DC-1F3FC-200D-2640-FE0F"},
    {"undinėlė_spalva3", "1F9DC-1F3FD-200D-2640-FE0F"},
    {"undinėlė_spalva4", "1F9DC-1F3FE-200D-2640-FE0F"},
    {"undinėlė_spalva5", "1F9DC-1F3FF-200D-2640-FE0F"},
    {"elfė", "1F9DD-200D-2640-FE0F"},
    {"elfė_spalva1", "1F9DD-1F3FB-200D-2640-FE0F"},
    {"elfė_spalva2", "1F9DD-1F3FC-200D-2640-FE0F"},
    {"elfė_spalva3", "1F9DD-1F3FD-200D-2640-FE0F"},
    {"elfė_spalva4", "1F9DD-1F3FE-200D-2640-FE0F"},
    {"elfė_spalva5", "1F9DD-1F3FF-200D-2640-FE0F"},
    {"džinė", "1F9DE-200D-2640-FE0F"},
    {"džinas", "1F9DE-200D-2642-FE0F"},
    {"zombė", "1F9DF-200D-2640-FE0F"},
    {"vyras_žili_plaukai", "1F468-200D-1F9B3"},
    {"vyras_žili_plaukai_spalva1", "1F468-1F3FB-200D-1F9B3"},
    {"vyras_žili_plaukai_spalva2", "1F468-1F3FC-200D-1F9B3"},
    {"vyras_žili_plaukai_spalva3", "1F468-1F3FD-200D-1F9B3"},
    {"vyras_žili_plaukai_spalva4", "1F468-1F3FE-200D-1F9B3"},
    {"vyras_žili_plaukai_spalva5", "1F468-1F3FF-200D-1F9B3"},
    {"moteris_žili_plaukai", "1F469-200D-1F9B3"},
    {"moteris_žili_plaukai_spalva1", "1F469-1F3FB-200D-1F9B3"},
    {"moteris_žili_plaukai_spalva2", "1F469-1F3FC-200D-1F9B3"},
    {"moteris_žili_plaukai_spalva3", "1F469-1F3FD-200D-1F9B3"},
    {"moteris_žili_plaukai_spalva4", "1F469-1F3FE-200D-1F9B3"},
    {"moteris_žili_plaukai_spalva5", "1F469-1F3FF-200D-1F9B3"},
    {"suaugusysis_žili_plaukai", "1F9D1-200D-1F9B3"},
    {"suaugusysis_žili_plaukai_spalva1", "1F9D1-1F3FB-200D-1F9B3"},
    {"suaugusysis_žili_plaukai_spalva2", "1F9D1-1F3FC-200D-1F9B3"},
    {"suaugusysis_žili_plaukai_spalva3", "1F9D1-1F3FD-200D-1F9B3"},
    {"suaugusysis_žili_plaukai_spalva4", "1F9D1-1F3FE-200D-1F9B3"},
    {"suaugusysis_žili_plaukai_spalva5", "1F9D1-1F3FF-200D-1F9B3"},
    {"nutraukta_grandinė", "26D3-FE0F-200D-1F4A5"},
    {"deganti_širdis", "2764-FE0F-200D-1F525"},
    {"gydytina_širdis", "2764-FE0F-200D-1FA79"},
    {"žalioji_citrina", "1F34B-200D-1F7E9"},
    {"žydra_rožinė_ir_balta_vėliava", "1F3F3-FE0F-200D-26A7-FE0F"},
    {"vaivorykštinė_vėliava", "1F3F3-FE0F-200D-1F308"},
    {"piratų_vėliava", "1F3F4-200D-2620-FE0F"},
    {"juoda_katė", "1F408-200D-2B1B"},
    {"tarnaujantis_šuo", "1F415-200D-1F9BA"},
    {"juodas_paukštis", "1F426-200D-2B1B"},
    {"baltoji_meška", "1F43B-200D-2744-FE0F"},
    {"akis_kalbėjimo_burbule", "1F441-FE0F-200D-1F5E8-FE0F"},
    {"veidas_su_spiralinėmis_akimis", "1F635-200D-1F4AB"},
    {"purtantis_galvą", "1F642-200D-2194-FE0F"},
    {"baleto_šokėjas", "1F9D1-200D-1FA70"},
    {"baleto_šokėjas_spalva1", "1F9D1-1F3FB-200D-1FA70"},
    {"baleto_šokėjas_spalva2", "1F9D1-1F3FC-200D-1FA70"},
    {"baleto_šokėjas_spalva3", "1F9D1-1F3FD-200D-1FA70"},
    {"baleto_šokėjas_spalva4", "1F9D1-1F3FE-200D-1FA70"},
    {"baleto_šokėjas_spalva5", "1F9D1-1F3FF-200D-1FA70"},
};

